// pbrt is Copyright(c) 1998-2020 Matt Pharr, Wenzel Jakob, and Greg Humphreys.
// The pbrt source code is licensed under the Apache License, Version 2.0.
// SPDX: Apache-2.0

#include <pbrt/util/bluenoise.h>

namespace pbrt {

/*
Converted the 128x128 HDR RGB textures from http://momentsingraphics.de/?p=127
using this:


#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <fstream>
#include <string>
#include <vector>
#include "pbrt-v4/src/ext/lodepng/lodepng.cpp"
#include "pbrt-v4/src/ext/lodepng/lodepng.h"

std::string ReadFileContents(const std::string &filename) {
    std::ifstream ifs(filename, std::ios::binary);
    if (!ifs) {
        fprintf(stderr, "%s: %s", filename.c_str(), strerror(errno));
        exit(1);
    }
    return std::string((std::istreambuf_iterator<char>(ifs)),
                       (std::istreambuf_iterator<char>()));
}


std::vector<uint16_t> ReadPNG(const char *name, int *xres, int *yres) {
    std::string contents = ReadFileContents(name);

    unsigned width, height;
    LodePNGState state;
    lodepng_state_init(&state);
    unsigned int error = lodepng_inspect(&width, &height, &state,
                                         (const unsigned char *)contents.data(),
                                         contents.size());
    if (error != 0) {
        fprintf(stderr, "%s: %s", name, lodepng_error_text(error));
        exit(1);
    }

    *xres = width;
    *yres = height;

    assert(state.info_png.color.colortype != LCT_GREY &&
           state.info_png.color.colortype != LCT_GREY_ALPHA);

    int bpp = state.info_png.color.bitdepth == 16 ? 16 : 8;
    std::vector<unsigned char> buf;
    error = lodepng::decode(buf, width, height,
                            (const unsigned char *)contents.data(),
                            contents.size(), LCT_RGB, bpp);
    if (error != 0) {
        fprintf(stderr, "%s: %s", name, lodepng_error_text(error));
        exit(1);
    }

    std::vector<uint16_t> pix;
    if (state.info_png.color.bitdepth == 16) {
        auto bufIter = buf.begin();
        for (unsigned int y = 0; y < height; ++y)
            for (unsigned int x = 0; x < width; ++x, bufIter += 6) {
                assert(bufIter < buf.end());
                // Convert from little endian.
                pix.push_back(((int)bufIter[0] << 8) + (int)bufIter[1]);
                pix.push_back(((int)bufIter[2] << 8) + (int)bufIter[3]);
                pix.push_back(((int)bufIter[4] << 8) + (int)bufIter[5]);
            }
        assert(bufIter == buf.end());
    } else {
        assert(!"todo 8 bit");
    }

    return pix;
}

int main(int argc, char *argv[]) {
    int xres = -1, yres = -1;
    printf("{ ");
    for (++argv; *argv; ++argv) {
        int xr, yr;
        std::vector<uint16_t> png = ReadPNG(*argv, &xr, &yr);
        if (xres == -1) {
            xres = xr;
            yres = yr;
        } else {
            assert(xres == xr);
            assert(yres == yr);
        }
        assert(png.size() == 3 * xres * yres);
        for (int c = 0; c < 3; ++c) {
            int offset = c;
            printf("{ ");
            for (int y = 0; y < yres; ++y) {
                printf("{ ");
                for (int x = 0; x < xres; ++x, offset += 3) {
                    printf("%d, ", png[offset]);
                }
                printf("},\n");
            }
            printf("},\n");
        }
    }
    printf("};\n");
}
 */

// clang-format off
PBRT_CONST uint16_t BlueNoiseTextures
    [NumBlueNoiseTextures][BlueNoiseResolution][BlueNoiseResolution] = {
        {
            {
                54008, 25112, 14312, 45952, 7520,  23708, 17904, 648,   40536,
                9480,  43996, 34844, 60684, 39076, 56660, 512,   59856, 36556,
                20012, 32368, 1088,  42468, 17604, 31028, 48648, 19828, 58504,
                42892, 23268, 32140, 59116, 19220, 61176, 2192,  21464, 64572,
                12068, 39316, 20992, 35752, 57768, 6512,  53076, 37592, 46092,
                55600, 27900, 49112, 58256, 3672,  19424, 45996, 33024, 49084,
                9132,  36572, 65416, 15400, 20704, 10036, 54100, 58484, 48656,
                64836, 52128, 1048,  36536, 58984, 4296,  51056, 65044, 13760,
                5320,  42496, 27712, 21352, 10220, 53292, 65412, 35868, 13640,
                5072,  33288, 54388, 37188, 2360,  16336, 25008, 10576, 34880,
                51036, 7628,  33064, 48864, 56476, 2676,  39672, 9612,  22764,
                5108,  27348, 39312, 14600, 57532, 1140,  31804, 50504, 36600,
                15952, 7568,  42348, 23392, 1444,  17236, 65184, 27484, 45780,
                31464, 62912, 7328,  38640, 26684, 43296, 23408, 53236, 15708,
                58072, 21188,
            },
            {
                6244,  50392, 33536, 55076, 30636, 59832, 49424, 33876, 14968,
                21804, 31240, 12356, 7456,  21064, 48148, 13900, 23848, 44072,
                8292,  46728, 25484, 50356, 63560, 24164, 2396,  55148, 38188,
                13332, 48108, 5508,  27512, 11516, 46760, 52688, 43320, 56200,
                48216, 17204, 45172, 28600, 13928, 42080, 63840, 8836,  21900,
                61840, 5816,  13968, 31916, 43216, 25172, 56456, 38744, 2832,
                18104, 24060, 4356,  46916, 40340, 1412,  44024, 37040, 27440,
                16612, 46076, 29108, 19864, 53708, 40952, 32248, 44716, 37768,
                54984, 17432, 59872, 32016, 40436, 16136, 23788, 42408, 20516,
                28584, 62900, 24100, 47548, 60584, 41316, 46232, 65232, 13736,
                42280, 2992,  58976, 27744, 17580, 6480,  30876, 47432, 16104,
                51764, 2068,  43228, 62820, 35328, 26476, 59444, 13340, 61716,
                56044, 34624, 11572, 28324, 51120, 39776, 47400, 19696, 3572,
                11716, 41232, 52232, 14456, 61104, 9216,  31248, 65224, 11528,
                42500, 29680,
            },
            {
                18252, 64152, 10428, 41068, 19928, 3732,  38232, 57040, 46480,
                65476, 50112, 53064, 25604, 63584, 42620, 9832,  28044, 52612,
                65108, 15000, 56080, 35312, 7348,  40744, 15656, 26880, 8660,
                65472, 18208, 39964, 51268, 36240, 7308,  30828, 9848,  26388,
                34600, 6848,  62752, 50916, 2740,  26088, 34048, 16412, 30316,
                43964, 40652, 20352, 36376, 64888, 11840, 6108,  27704, 61624,
                53148, 44328, 59232, 26664, 51152, 32764, 17980, 7256,  12904,
                22948, 5800,  33208, 61460, 10216, 25708, 18272, 244,   21764,
                34640, 8848,  46088, 4456,  58088, 34000, 3692,  46600, 51832,
                828,   39532, 15016, 7344,  31264, 21772, 5728,  37944, 29200,
                55008, 24540, 9868,  40396, 51500, 64728, 12504, 54716, 60756,
                37448, 30116, 46872, 21664, 17316, 53144, 42828, 22628, 5756,
                29596, 48572, 20024, 60404, 6200,  33148, 13820, 60816, 37924,
                57608, 49488, 29212, 1428,  57104, 17740, 48384, 37948, 27228,
                8100,  47200,
            },
            {
                39208, 23036, 1452,  26760, 62220, 44536, 12756, 25140, 6328,
                28320, 1964,  16960, 37232, 4036,  30368, 35628, 58320, 18372,
                4972,  39588, 29216, 20828, 10696, 53180, 61196, 36012, 50780,
                124,   33364, 55908, 14884, 22528, 63844, 17640, 37744, 3664,
                58608, 23568, 31832, 10644, 55784, 20124, 49912, 59340, 592,
                11588, 52432, 59740, 1704,  47784, 52016, 16808, 41668, 34764,
                13232, 31580, 7448,  11436, 57228, 22520, 63868, 53048, 41296,
                60288, 56224, 38448, 14384, 48168, 7080,  56364, 60436, 48888,
                26352, 63784, 50724, 12916, 25324, 54836, 61000, 11828, 30884,
                57476, 18000, 49920, 10404, 58660, 52852, 96,    49508, 20804,
                16204, 45304, 62148, 19568, 35468, 44116, 26696, 33592, 19772,
                7408,  57080, 10416, 50220, 3852,  8612,  38824, 47564, 18772,
                316,   64076, 38292, 53380, 44008, 9916,  26272, 54268, 23620,
                5620,  18632, 33684, 22496, 41736, 35228, 2556,  19988, 50820,
                56300, 34972,
            },
            {
                13184, 59324, 49080, 35700, 16044, 51852, 8592,  32364, 54952,
                19276, 41208, 61248, 45240, 54556, 15440, 48892, 1724,  41380,
                33724, 12360, 58820, 3616,  47928, 31740, 43800, 12240, 29292,
                46036, 24892, 60792, 4284,  45188, 57452, 42300, 50700, 13612,
                40556, 53208, 15436, 43664, 38172, 4304,  46652, 39428, 54252,
                24292, 34492, 8684,  27204, 22468, 31092, 57816, 9828,  50640,
                21408, 63572, 39256, 16476, 36256, 5700,  29044, 45396, 31828,
                9772,  49868, 2112,  43888, 64292, 30948, 39408, 15448, 11592,
                29640, 2816,  19836, 39104, 44432, 6736,  19488, 37092, 8388,
                25804, 44644, 64208, 36284, 28096, 13416, 33736, 61568, 6788,
                57464, 36416, 30328, 5520,  14432, 23316, 1608,  50116, 41536,
                24864, 14236, 62044, 32444, 27808, 65284, 12156, 31052, 54976,
                44848, 25360, 14768, 2908,  22560, 57840, 48968, 772,   34884,
                44880, 9572,  64288, 47148, 12132, 54248, 62800, 5452,  23900,
                61352, 3432,
            },
            {
                30404, 53868, 42232, 5272,  29264, 57944, 22184, 42584, 60228,
                36060, 10488, 32848, 23188, 7912,  57172, 20372, 62060, 25852,
                50852, 22376, 44692, 36912, 62728, 18168, 22940, 4688,  59844,
                20868, 41432, 10380, 29832, 34388, 1312,  24312, 28552, 62168,
                20388, 292,   59644, 27844, 65376, 23200, 7616,  14460, 28208,
                64116, 18420, 46244, 14888, 39848, 61360, 3916,  24780, 47192,
                1528,  28428, 54020, 42708, 49560, 61420, 14524, 340,   19652,
                36132, 24732, 17192, 27912, 22112, 35768, 3240,  45428, 54156,
                42064, 36432, 56780, 31232, 15064, 28496, 49368, 40872, 62100,
                53452, 33620, 2800,  42536, 24636, 56240, 44392, 39916, 26028,
                11560, 47388, 2584,  52636, 59716, 56356, 10764, 63300, 4476,
                45200, 35968, 18780, 40940, 55460, 36828, 23764, 59156, 7496,
                34200, 10548, 58632, 31304, 35944, 16936, 41180, 30308, 62256,
                15784, 27756, 53284, 6584,  25552, 29920, 45192, 14352, 32520,
                43944, 16852,
            },
            {
                25644, 46328, 19368, 11352, 64812, 37916, 1220,  49824, 14548,
                5340,  46964, 51968, 12940, 27508, 38952, 32208, 11252, 46128,
                8572,  60716, 54704, 15460, 26500, 1844,  57236, 38832, 54628,
                15824, 6388,  53716, 48856, 19660, 12332, 54052, 6140,  33412,
                44340, 48944, 8068,  35128, 17488, 30704, 56564, 33436, 45132,
                5592,  37740, 50172, 55996, 6908,  35248, 44828, 19236, 37864,
                58732, 6304,  18732, 24976, 3132,  33520, 47172, 26224, 55400,
                62844, 5024,  53192, 58280, 11960, 51744, 19152, 24776, 63040,
                7604,  22752, 60548, 9672,  52256, 64680, 22980, 2136,  16668,
                13240, 22380, 6252,  19552, 48224, 16076, 9108,  19012, 52236,
                32428, 63236, 21792, 40980, 27480, 37896, 46604, 17024, 28800,
                59064, 53216, 508,   6188,  46160, 15484, 2412,  49776, 40472,
                16576, 51828, 42008, 47704, 8096,  64528, 20444, 12268, 51204,
                39456, 59524, 36292, 20480, 57896, 38352, 10032, 59388, 40428,
                52460, 9424,
            },
            {
                38068, 6620,  55808, 33732, 24640, 47452, 18412, 30516, 63688,
                25904, 20740, 58540, 0,     64404, 43788, 5428,  53600, 16628,
                28536, 2224,  31208, 7252,  40928, 51648, 33012, 9432,  47608,
                35224, 64280, 26976, 37332, 62568, 39660, 59932, 47700, 16584,
                10448, 26516, 39060, 52384, 12592, 47924, 61436, 10336, 21136,
                58108, 2684,  31724, 63124, 21252, 12944, 54636, 29576, 62336,
                14760, 34372, 45668, 56804, 9520,  21508, 59600, 39876, 11400,
                48324, 43280, 30096, 40184, 8056,  47032, 59356, 4808,  32652,
                50140, 16964, 884,   47260, 34968, 4900,  42636, 55660, 35716,
                46748, 29608, 57144, 60888, 37524, 3660,  64548, 29704, 1504,
                43068, 14988, 9292,  49152, 33984, 7560,  20788, 31856, 39524,
                12220, 22384, 48736, 25964, 60536, 20376, 43512, 28756, 63284,
                21700, 27196, 3756,  61336, 28716, 45444, 4788,  56540, 7788,
                22136, 3588,  13744, 43708, 1104,  17312, 50364, 21624, 27724,
                4,     63440,
            },
            {
                32172, 22328, 61212, 2760,  14096, 53408, 9332,  40100, 3492,
                54452, 37468, 29416, 40856, 18436, 49884, 24768, 35924, 63664,
                38656, 42896, 19292, 48428, 64940, 13904, 44412, 24484, 19056,
                30832, 12948, 44248, 2900,  16184, 8704,  30964, 21828, 36832,
                64108, 57092, 22460, 2580,  43008, 5260,  39948, 51308, 26296,
                42568, 16460, 10892, 28604, 43292, 112,   49188, 7764,  42168,
                10836, 51420, 30592, 63996, 13388, 37276, 51620, 17732, 7424,
                32872, 14252, 20520, 2796,  62136, 34032, 41548, 28404, 10300,
                38548, 43660, 27668, 54504, 18948, 12500, 31732, 26736, 59540,
                9652,  39108, 50384, 11732, 31408, 54192, 23264, 49664, 59756,
                37112, 24108, 54564, 18032, 65028, 780,   55564, 50812, 61096,
                8688,  34676, 56720, 31372, 10992, 35556, 54420, 13572, 5852,
                38032, 56620, 18984, 12740, 23704, 52804, 37052, 26184, 33220,
                42096, 48120, 30996, 52008, 61288, 33384, 6100,  46724, 35668,
                18720, 57472,
            },
            {
                44104, 15648, 51040, 41140, 45352, 27316, 57072, 34136, 44176,
                16468, 48404, 7064,  33884, 14836, 59960, 2836,  21644, 56964,
                12672, 52116, 24036, 35160, 10628, 28908, 58488, 5516,  61652,
                876,   52492, 22124, 50440, 25616, 56344, 45980, 4992,  52644,
                3348,  30396, 18688, 62516, 55140, 24244, 17956, 872,   35292,
                62580, 54200, 40248, 25132, 52160, 60168, 32736, 17676, 26608,
                55556, 22672, 952,   40284, 26896, 44560, 3708,  29220, 56592,
                23388, 64184, 37252, 54612, 25508, 16696, 13252, 53400, 21140,
                56968, 13964, 63964, 24364, 39568, 45832, 62624, 7312,  20208,
                52572, 544,   24856, 41956, 7592,  45384, 34808, 12788, 6032,
                56068, 28192, 4052,  45272, 12888, 25872, 42460, 14776, 3632,
                27156, 43728, 16420, 6596,  64384, 50996, 8804,  33056, 60156,
                1744,  48380, 34900, 54660, 40032, 15140, 1548,  49596, 58716,
                16528, 65432, 24664, 11232, 40140, 26380, 56164, 64732, 8188,
                13112, 49408,
            },
            {
                4692,  10228, 35528, 29836, 7404,  60164, 21640, 12880, 61292,
                23832, 10328, 62484, 45212, 52828, 10856, 30584, 47080, 7832,
                33124, 4352,  55852, 60060, 1336,  20960, 36476, 49652, 39204,
                55964, 41920, 7944,  59112, 32840, 41108, 13680, 27852, 42024,
                12076, 50260, 46120, 14632, 32496, 36968, 59096, 29732, 13668,
                7552,  47212, 19340, 4576,  36788, 14468, 23624, 39808, 65240,
                4968,  36052, 48272, 16860, 6792,  53684, 61300, 34728, 42388,
                1468,  49248, 44900, 9948,  31156, 49764, 1300,  44640, 60292,
                35712, 3832,  30908, 8216,  58104, 1772,  16248, 49000, 43420,
                33224, 65032, 18276, 14844, 61752, 27260, 20016, 39800, 17116,
                47040, 33276, 61552, 39112, 30500, 59000, 36220, 23380, 47412,
                63460, 52704, 37760, 21496, 41320, 26724, 17600, 45556, 24084,
                30064, 43096, 9900,  6576,  32300, 63340, 43212, 19108, 29148,
                9476,  53856, 2408,  19712, 45636, 4404,  14492, 23204, 53212,
                40736, 28484,
            },
            {
                24452, 58452, 20576, 64008, 17136, 37624, 852,   51984, 6344,
                28188, 57416, 19704, 2056,  26796, 37736, 41884, 58992, 18552,
                49680, 27824, 15584, 45328, 40276, 54036, 16632, 26228, 9104,
                14856, 28708, 36260, 18324, 4480,  65204, 20264, 54808, 59608,
                24596, 34240, 40196, 9268,  26980, 6532,  48228, 43812, 52756,
                23432, 33020, 57844, 64420, 9052,  47748, 57384, 2236,  50688,
                44036, 20172, 60220, 57332, 32200, 23740, 14916, 10196, 19776,
                59240, 27532, 5712,  18540, 57316, 39180, 65380, 23864, 6984,
                18576, 52052, 46596, 34064, 21588, 51440, 37116, 23884, 13824,
                5980,  28868, 58036, 36440, 48956, 2060,  57480, 63116, 10432,
                42120, 21284, 8212,  52328, 19444, 9800,  53824, 5244,  32564,
                19612, 2208,  12996, 58640, 48112, 1228,  38764, 57000, 52280,
                15964, 65088, 26116, 59104, 47112, 21852, 11076, 60628, 5540,
                46544, 34336, 38036, 58152, 62388, 29744, 37024, 43368, 31708,
                2268,  62736,
            },
            {
                37776, 46860, 53640, 2388,  25816, 48244, 42736, 30920, 36076,
                49980, 39268, 32596, 54732, 22364, 65272, 5920,  13892, 25288,
                39352, 61124, 9024,  22680, 31244, 47752, 3608,  63476, 32540,
                46412, 23668, 62120, 11772, 48084, 37972, 456,   35260, 16984,
                6872,  61192, 1280,  57616, 53268, 64908, 11452, 20540, 60568,
                38424, 3720,  15496, 30272, 43628, 20552, 34496, 11524, 31288,
                15332, 8400,  28384, 11172, 41632, 50004, 64672, 46344, 38368,
                51548, 12764, 35604, 61064, 21972, 4148,  34596, 29612, 48692,
                40516, 26524, 12396, 60700, 42068, 10664, 30100, 61508, 55788,
                40136, 46236, 10480, 53776, 22236, 5288,  30104, 52088, 24940,
                964,   56992, 14708, 49492, 2876,  44800, 62784, 16884, 39720,
                57376, 29380, 45924, 23828, 5656,  30572, 62308, 20588, 7528,
                11760, 50236, 39348, 17916, 4164,  30508, 54256, 35976, 40628,
                23284, 14540, 27504, 7880,  49296, 17476, 54464, 10564, 48052,
                19376, 55584,
            },
            {
                8488,  13596, 31172, 39548, 11016, 56304, 14912, 64916, 18532,
                3716,  13496, 42600, 8312,  16144, 50968, 35124, 45736, 54372,
                740,   43540, 34556, 64160, 6772,  13020, 59016, 42328, 19344,
                57268, 2212,  54016, 43240, 29492, 24408, 51332, 9840,  44916,
                48904, 28476, 19632, 44084, 23048, 16708, 40832, 2104,  28112,
                9992,  45468, 50592, 25888, 55704, 5476,  61696, 27864, 54652,
                38296, 63276, 45896, 34316, 5588,  21176, 2312,  25824, 6944,
                31976, 16268, 53020, 41436, 46804, 12256, 54948, 15736, 9660,
                63048, 144,   55924, 17528, 4468,  26040, 54004, 3320,  34948,
                21228, 2328,  25656, 32440, 41100, 44580, 14064, 35620, 48376,
                31620, 63936, 26644, 36692, 22392, 34140, 27940, 48868, 11300,
                43024, 8196,  35060, 61464, 54876, 14956, 36120, 3740,  41604,
                33664, 22744, 88,    36296, 57064, 13620, 25304, 50428, 896,
                63160, 56176, 44584, 12304, 33164, 22080, 1392,  59040, 26240,
                15776, 33860,
            },
            {
                3412,  42028, 18188, 49896, 59584, 33696, 24020, 8616,  54940,
                46516, 62996, 25136, 58400, 47864, 28704, 2952,  20836, 63140,
                30084, 11764, 19892, 51924, 37692, 25128, 29348, 51464, 11340,
                35384, 5856,  49820, 15988, 7216,  60784, 57544, 31072, 21952,
                63464, 14204, 37536, 30928, 4176,  35048, 50876, 32356, 55396,
                62956, 18460, 36040, 756,   39052, 14032, 41376, 46848, 22060,
                3344,  24548, 52664, 18432, 55868, 37088, 29080, 57812, 40760,
                62936, 24188, 724,   28924, 8384,  26036, 43104, 58456, 37544,
                21756, 43808, 29032, 49756, 38440, 63896, 44488, 17844, 9504,
                59760, 50920, 63376, 16848, 7484,  55644, 60280, 8980,  18536,
                38192, 43876, 5172,  59676, 41464, 13312, 56472, 6308,  25044,
                65452, 51208, 16308, 27364, 10132, 49964, 44872, 25480, 59392,
                54692, 45804, 62648, 27672, 51880, 44360, 7724,  32512, 17084,
                48360, 20104, 39584, 3868,  51868, 41656, 64168, 6720,  39232,
                51496, 61608,
            },
            {
                44760, 26472, 63444, 6828,  21212, 4228,  44256, 27616, 40152,
                21864, 30456, 1096,  36304, 11872, 61164, 32748, 9664,  37976,
                16868, 58024, 48672, 3172,  56484, 17644, 45196, 7492,  22588,
                40600, 64536, 26848, 33252, 39164, 18752, 13372, 42288, 2280,
                39652, 56312, 10944, 52188, 46924, 60284, 13260, 24860, 6068,
                41496, 12292, 54060, 22856, 63384, 9628,  52068, 17984, 7244,
                59004, 42324, 13444, 332,   60860, 48220, 12176, 53312, 18088,
                9620,  48564, 55688, 62080, 32964, 51044, 19868, 5112,  31968,
                52316, 14280, 35044, 7748,  20936, 13700, 32516, 48296, 27664,
                42688, 37644, 12624, 47828, 19756, 38452, 27772, 22844, 4376,
                54908, 11532, 16988, 53324, 30728, 348,   61012, 37804, 21372,
                32004, 1748,  39200, 20060, 42384, 32296, 63660, 12824, 28852,
                18604, 5820,  15120, 10288, 40828, 19268, 64700, 58124, 10612,
                28592, 6684,  61712, 25116, 56928, 28804, 34996, 14016, 46144,
                29968, 22316,
            },
            {
                57192, 35456, 52112, 29136, 46176, 37072, 53052, 12208, 57908,
                5788,  16664, 52408, 44356, 19328, 40740, 56376, 24600, 52520,
                41676, 4904,  26988, 32900, 40716, 61528, 1544,  36708, 60260,
                30384, 13808, 20404, 45812, 55500, 3112,  47420, 26632, 53624,
                33648, 4608,  25308, 62776, 8340,  19100, 38092, 58472, 49040,
                21668, 33768, 59576, 29484, 48628, 32816, 26504, 60956, 35780,
                31796, 49916, 26940, 39920, 31132, 15860, 44648, 33728, 3024,
                43132, 36564, 20224, 14404, 38840, 2524,  64392, 47744, 24980,
                59736, 2964,  62028, 46252, 58060, 53112, 6236,  23288, 56324,
                14996, 4812,  30760, 57644, 232,   33712, 65360, 45452, 49788,
                61308, 29476, 24116, 47852, 9580,  45628, 18092, 52240, 14616,
                55248, 47964, 59816, 6548,  53392, 752,   21996, 8288,  48264,
                37392, 31528, 59928, 48896, 33976, 1876,  22540, 37856, 42628,
                54996, 36272, 31688, 15596, 46680, 18348, 8680,  24372, 60180,
                5068,  11392,
            },
            {
                48344, 1672,  15152, 10424, 60788, 17472, 220,   62604, 32956,
                50296, 38384, 59972, 7584,  27556, 3940,  14772, 46720, 7052,
                60344, 22368, 46292, 14420, 9864,  21144, 53528, 15732, 47888,
                55896, 736,   51760, 8824,  62032, 23300, 36248, 64724, 9204,
                20324, 49684, 16304, 41848, 28168, 28,    44232, 30240, 15820,
                3032,  46444, 7808,  16516, 5100,  20300, 44992, 1292,  15828,
                55180, 10400, 20728, 65052, 8640,  23932, 5292,  63708, 21940,
                58568, 28020, 4372,  44476, 57488, 17428, 35776, 12872, 8852,
                40376, 18600, 31324, 24320, 10520, 36356, 40956, 816,   64504,
                34344, 20628, 52980, 26792, 41724, 51240, 12108, 15760, 2432,
                40852, 34976, 6128,  64684, 20408, 33104, 26032, 43328, 3456,
                28508, 10732, 23184, 35800, 57504, 17372, 39500, 58548, 52448,
                2188,  42872, 24340, 56112, 7056,  61228, 29928, 46632, 14092,
                2496,  21608, 52628, 10860, 356,   58532, 50656, 43088, 54684,
                37492, 19800,
            },
            {
                32132, 24820, 39276, 54720, 23180, 41544, 30076, 47836, 19816,
                25380, 10164, 34108, 55084, 23344, 64376, 50708, 34672, 28968,
                12656, 36368, 54136, 65020, 29844, 49256, 33780, 27212, 9288,
                43980, 24772, 35032, 41160, 12376, 31536, 5536,  16972, 58380,
                29888, 45416, 59416, 34488, 55468, 23296, 63748, 9120,  52548,
                25672, 65516, 42704, 36652, 53412, 40016, 56716, 12004, 38020,
                47168, 4512,  43820, 35236, 52044, 56604, 37532, 27136, 49672,
                13600, 31788, 47664, 10284, 23640, 30332, 53676, 45544, 28100,
                56516, 48772, 5824,  42944, 27920, 17180, 60256, 30320, 49876,
                8896,  44628, 60892, 10888, 23480, 6796,  58560, 31504, 26316,
                19216, 58076, 13976, 39280, 55800, 49892, 7904,  61828, 39868,
                34036, 62740, 44312, 13152, 29772, 46928, 25192, 34312, 15924,
                64912, 11620, 21068, 38596, 27024, 16344, 53488, 4856,  25828,
                60328, 48668, 43804, 65116, 40416, 36520, 20972, 31068, 2844,
                16508, 62216,
            },
            {
                7696,  43496, 65464, 4632,  34708, 57128, 8248,  14816, 43524,
                3388,  63648, 13772, 45880, 30648, 42368, 17816, 296,   62744,
                19496, 43912, 2164,  24804, 6628,  42648, 62856, 4720,  38780,
                17964, 63260, 4236,  58208, 28460, 52908, 49180, 38720, 43152,
                12224, 6560,  21784, 3232,  39772, 12036, 47872, 35688, 56828,
                38932, 28228, 10800, 58368, 24476, 3556,  64180, 28000, 22828,
                61848, 29940, 59440, 17344, 2044,  46200, 19076, 10460, 39820,
                60772, 7428,  54980, 63884, 40840, 6048,  60864, 708,   22048,
                36784, 15320, 54044, 64876, 1972,  51160, 12864, 46360, 18448,
                25452, 39596, 3280,  35912, 62668, 17924, 43424, 37680, 54244,
                8900,  51452, 43668, 27788, 1692,  36756, 12340, 57108, 4984,
                46792, 17912, 26584, 2972,  64328, 51296, 11084, 3568,  45332,
                27872, 54312, 5012,  47416, 12920, 50912, 40312, 34772, 56628,
                18060, 8332,  27404, 33300, 23904, 6212,  12772, 63448, 26788,
                41236, 52588,
            },
            {
                57716, 29652, 18304, 49192, 13060, 26484, 51540, 36704, 58600,
                28888, 40112, 49316, 20860, 4912,  10872, 58500, 38884, 48788,
                8656,  31904, 59208, 38584, 18900, 56052, 13308, 23124, 59492,
                32252, 49544, 15136, 21904, 45100, 19264, 61152, 684,   23792,
                51776, 37124, 61480, 14932, 53304, 31484, 18052, 5064,  20424,
                13740, 928,   50844, 19068, 14212, 34612, 50028, 42800, 18152,
                52512, 7848,  24992, 12816, 41264, 28372, 61804, 53616, 608,
                24608, 17756, 33888, 20644, 51656, 26452, 15804, 42160, 32700,
                63016, 11396, 21124, 38872, 32136, 55980, 22664, 37940, 6632,
                58752, 55096, 16040, 45984, 28960, 48548, 1488,  21560, 63960,
                47052, 32376, 4264,  22736, 59956, 15840, 29932, 24228, 20840,
                53068, 8380,  56536, 37588, 41864, 7280,  32640, 60976, 40308,
                18936, 35828, 31112, 63020, 44064, 8800,  20676, 63912, 11368,
                30808, 38992, 3584,  14288, 59776, 54376, 44300, 48900, 9364,
                34732, 13692,
            },
            {
                620,   21908, 9824,  32460, 45932, 2780,  64052, 22480, 6144,
                54340, 17504, 1040,  61748, 37008, 53592, 32628, 22524, 26220,
                56552, 51820, 16604, 11192, 46708, 35872, 2864,  51684, 41492,
                11244, 29288, 54560, 38076, 6220,  10044, 33688, 14744, 27444,
                56400, 31920, 46468, 26252, 8036,  50020, 59976, 41508, 33524,
                61396, 44636, 31964, 62424, 47308, 6704,  30784, 9904,  1944,
                33708, 39380, 57140, 50936, 32380, 6000,  14568, 34328, 42676,
                51180, 45712, 38316, 3136,  12492, 35068, 49536, 58588, 7272,
                51328, 3404,  47120, 26112, 9036,  44172, 4020,  62968, 28744,
                11724, 32420, 21916, 51840, 8276,  33576, 56688, 14356, 5952,
                24460, 11996, 62480, 18512, 45800, 51976, 41936, 63608, 35844,
                14732, 32080, 49312, 22280, 15612, 19764, 55872, 23096, 49956,
                8696,  57592, 476,   17212, 59196, 32636, 2324,  24444, 45412,
                49428, 62228, 53352, 19396, 47712, 28844, 1752,  17836, 59212,
                23332, 45592,
            },
            {
                50528, 61736, 53956, 40984, 59616, 20200, 39136, 11500, 33908,
                44924, 23988, 32240, 57164, 27040, 16124, 6436,  44976, 13836,
                40876, 4920,  28340, 62400, 50464, 30940, 26464, 57576, 20244,
                7188,  47100, 2376,  65444, 26984, 55568, 40512, 47876, 63296,
                5436,  10396, 17632, 64648, 43464, 28752, 2648,  24920, 46520,
                53908, 22648, 8604,  40596, 26160, 54932, 21080, 58216, 62676,
                14860, 46024, 3968,  22056, 63628, 48180, 58432, 21336, 30264,
                5016,  15160, 58116, 29396, 61996, 44336, 9716,  19416, 24924,
                29700, 40560, 57924, 33848, 18960, 60664, 14980, 34924, 50460,
                43436, 1656,  64172, 13216, 59152, 25420, 40292, 50888, 29664,
                42336, 35336, 54944, 38536, 9636,  32920, 2480,  48492, 10456,
                58988, 40532, 148,   61316, 52500, 28028, 43440, 4748,  13916,
                25900, 51772, 41964, 22556, 28288, 37412, 55356, 41648, 5792,
                15716, 36080, 23044, 41860, 10224, 35108, 39168, 55628, 31612,
                6404,  38472,
            },
            {
                14920, 35088, 25220, 6892,  16212, 55620, 30800, 49648, 15552,
                59744, 8644,  47660, 12324, 42984, 51368, 65328, 2540,  60376,
                20632, 35460, 43604, 1620,  21820, 8512,  15036, 45336, 37308,
                61616, 25184, 35640, 42464, 17128, 60320, 22128, 2944,  35972,
                20064, 41916, 54644, 984,   22216, 36668, 55916, 16200, 11456,
                5580,  29468, 57740, 3316,  36176, 16672, 44788, 37304, 25472,
                49056, 28808, 19172, 36736, 10728, 25912, 38544, 8172,  55760,
                64652, 26952, 9124,  53936, 23016, 1580,  39128, 55404, 45908,
                13132, 61068, 16444, 5444,  52736, 48088, 24704, 40204, 19308,
                53924, 26868, 36940, 41520, 3892,  19252, 10704, 36500, 61384,
                16644, 404,   49732, 29224, 6568,  55700, 19440, 27004, 4460,
                44440, 24800, 29504, 6204,  34980, 11884, 63848, 47196, 30436,
                38052, 62004, 11304, 48416, 6448,  14260, 51988, 18728, 61100,
                29128, 1108,  58160, 7284,  64012, 25208, 15092, 21088, 51876,
                64508, 28264,
            },
            {
                18968, 3028,  44752, 37556, 28444, 948,   43688, 61988, 3800,
                27576, 52144, 38512, 19336, 6748,  23492, 33920, 47512, 29660,
                54284, 12060, 24428, 58136, 39012, 53096, 64276, 32680, 256,
                17420, 52952, 13984, 8904,  50900, 31184, 12568, 44284, 59300,
                29232, 51024, 38752, 33112, 13344, 60688, 8960,  51492, 63420,
                38276, 17704, 43704, 13392, 50480, 64952, 11344, 4988,  41744,
                53960, 8864,  60448, 44264, 55108, 2284,  18004, 44932, 12552,
                36952, 49176, 19084, 40444, 47220, 31544, 16816, 65320, 34152,
                2076,  22904, 43784, 36512, 28116, 11600, 1128,  57664, 9560,
                5084,  47708, 17016, 30220, 45296, 62428, 53792, 2920,  44840,
                21096, 59396, 25748, 15076, 65196, 23060, 60324, 39336, 53840,
                17820, 64712, 13428, 46612, 57252, 38972, 1556,  33500, 58436,
                16396, 3812,  34324, 56232, 44988, 65404, 25856, 9304,  33608,
                50928, 12836, 44876, 32164, 50100, 5344,  60236, 43224, 3872,
                10504, 47344,
            },
            {
                55024, 60360, 11188, 63644, 52420, 23760, 9444,  18716, 41488,
                35056, 64232, 2032,  30460, 58880, 40384, 10084, 18144, 37980,
                7980,  63348, 49088, 33416, 6016,  18544, 41752, 10736, 49708,
                29728, 58628, 22504, 33440, 46616, 5312,  25676, 53460, 7964,
                15392, 24132, 4384,  58100, 48164, 19404, 40520, 32468, 26348,
                48964, 34860, 61128, 21920, 28120, 1480,  32384, 55928, 22924,
                13256, 160,   34008, 15772, 30556, 41456, 52024, 61156, 23676,
                1400,  32560, 62504, 4504,  13832, 60400, 27892, 5008,  37848,
                53876, 49228, 8416,  62348, 55420, 41912, 30916, 65136, 33472,
                22268, 61072, 57196, 6924,  50060, 23304, 32316, 27360, 57372,
                8544,  46880, 39704, 11280, 43392, 34868, 46284, 12684, 30652,
                37176, 51004, 21260, 42544, 9028,  24376, 18800, 53620, 7400,
                21480, 40920, 25204, 18548, 31556, 1696,  39796, 47256, 57436,
                20336, 26856, 38420, 17520, 56648, 30180, 46476, 12576, 36100,
                24044, 41284,
            },
            {
                7544,  30492, 21524, 48288, 13796, 32324, 47064, 57308, 24996,
                11236, 21556, 55636, 14608, 46420, 62608, 27748, 56480, 50932,
                812,   17340, 45248, 14316, 29084, 60572, 23732, 56088, 4804,
                39752, 44228, 62496, 1984,  54784, 19472, 39332, 34668, 48544,
                64300, 11144, 43768, 26016, 7020,  29684, 45072, 4800,  20752,
                1012,  55464, 9556,  52632, 40024, 47540, 60068, 17400, 29872,
                62980, 40164, 50748, 24072, 65436, 4732,  27964, 34648, 47584,
                16740, 42432, 25416, 52260, 35404, 8316,  51156, 21028, 10956,
                26836, 17960, 31900, 14168, 4620,  20692, 50836, 16196, 44672,
                38532, 25276, 11196, 34960, 14780, 39160, 5776,  17728, 13048,
                33532, 53004, 4668,  31340, 50768, 1348,  16732, 5812,  57340,
                8568,  2176,  60740, 32176, 16596, 50492, 59828, 26692, 44944,
                49196, 63240, 52220, 8608,  60276, 22108, 12308, 36196, 3108,
                42864, 54712, 62588, 10984, 21868, 80,    40028, 27496, 62464,
                57088, 33292,
            },
            {
                51528, 26256, 40172, 4492,  58388, 36064, 6332,  53424, 38152,
                5000,  50080, 44080, 25768, 35956, 2960,  21216, 14208, 42772,
                32012, 26480, 39976, 54548, 3248,  47268, 36352, 13452, 27028,
                20608, 7648,  12272, 37516, 28088, 63392, 57932, 428,   18384,
                32064, 55716, 36932, 61264, 16768, 53108, 65104, 12204, 58964,
                42392, 14944, 24448, 6276,  34068, 19728, 8448,  51172, 35520,
                45524, 20948, 58768, 6972,  46772, 13624, 20420, 9880,  54080,
                6496,  56932, 10880, 21512, 58428, 44012, 40132, 56308, 47284,
                63028, 40972, 56972, 23972, 39176, 47004, 26532, 7380,  13000,
                55136, 76,    51708, 20564, 64452, 56128, 47380, 42260, 63656,
                37300, 23556, 61852, 19916, 58728, 27324, 52620, 63488, 24012,
                34384, 48268, 27148, 55244, 3472,  36460, 41608, 13644, 10384,
                30012, 860,   14756, 36620, 43160, 53864, 29440, 59176, 16300,
                24056, 8348,  3924,  48080, 34872, 54104, 15680, 50944, 20148,
                2348,  16832,
            },
            {
                13104, 59284, 34548, 17328, 42816, 20680, 64708, 16048, 29880,
                60200, 32936, 17544, 8724,  53656, 49392, 5236,  34904, 61976,
                22596, 57940, 10360, 64752, 21116, 8868,  42888, 59500, 51844,
                31696, 48200, 56384, 24396, 43256, 15544, 10620, 42164, 23484,
                45920, 3012,  20604, 50300, 33996, 2508,  23460, 36068, 46908,
                27732, 31844, 63700, 45656, 56792, 12704, 25640, 38732, 5872,
                3128,  27256, 11488, 37820, 32396, 56420, 60132, 39384, 30688,
                64016, 37328, 46124, 29900, 248,   18332, 25056, 14564, 30788,
                6288,  820,   51256, 34776, 63804, 2352,  58192, 35796, 61784,
                29016, 46704, 41112, 27036, 9192,  2012,  29560, 21836, 49660,
                1148,  28400, 15892, 40768, 9040,  36732, 21156, 41340, 44896,
                14060, 18860, 40128, 11496, 62500, 29428, 5420,  64320, 34072,
                39132, 56032, 27372, 47492, 4288,  17380, 49996, 6784,  64252,
                32372, 52212, 29708, 42016, 25788, 64568, 8692,  32556, 6564,
                48836, 43480,
            },
            {
                65188, 796,   54176, 9960,  50808, 2000,  26808, 45892, 13124,
                42244, 48,    63164, 39008, 12644, 30912, 59752, 45728, 7500,
                52380, 4068,  36616, 31228, 50452, 25600, 33628, 17568, 1768,
                64464, 35216, 18196, 4232,  50564, 30932, 6824,  51724, 61872,
                13608, 29552, 40924, 9748,  14344, 39356, 56280, 17920, 8028,
                53572, 38792, 18484, 4432,  29324, 42724, 64400, 53320, 15208,
                57508, 48624, 17724, 53628, 43080, 968,   15956, 49448, 22720,
                2764,  27388, 14964, 61220, 48920, 33632, 3648,  59368, 36392,
                45232, 19900, 12520, 28368, 9600,  17784, 53188, 42956, 20180,
                5968,  17224, 59672, 32552, 43940, 52820, 35424, 11320, 56888,
                7928,  54336, 43740, 48432, 2976,  55780, 32284, 10544, 28988,
                60148, 53468, 6624,  44252, 22232, 48028, 19548, 54840, 23032,
                6600,  59296, 20276, 10744, 61428, 33584, 26192, 40824, 45376,
                11240, 37756, 61232, 13580, 19048, 37016, 57548, 44804, 60512,
                25556, 36548,
            },
            {
                8832,  45784, 28848, 23912, 61132, 31344, 39460, 8052,  23308,
                56772, 20236, 27860, 58264, 23024, 41588, 16504, 25224, 38428,
                13008, 19280, 44056, 16188, 1184,  61324, 53524, 6716,  44548,
                14644, 40912, 9696,  60052, 46764, 20776, 34276, 38268, 26748,
                59252, 7472,  54308, 62908, 27312, 44000, 30844, 49360, 62600,
                3360,  13140, 50856, 59400, 36864, 644,   46788, 23340, 31936,
                42236, 63352, 30008, 9068,  21976, 26508, 35704, 44772, 8652,
                41480, 19624, 55052, 7000,  38208, 12488, 64500, 50436, 9312,
                22812, 61572, 42480, 58936, 45888, 33036, 23152, 10688, 31360,
                49220, 36584, 3476,  13788, 58092, 24708, 16608, 61088, 39028,
                19384, 32732, 12092, 63932, 25700, 14724, 46840, 4572,  50412,
                720,   37464, 24816, 33784, 58040, 15292, 1812,  42716, 50272,
                16088, 45536, 31040, 41792, 22932, 55304, 13804, 520,   21060,
                56556, 17052, 46668, 6152,  49644, 1888,  22996, 12012, 29840,
                39676, 21348,
            },
            {
                32216, 49548, 18976, 37380, 56144, 12096, 48496, 62472, 51892,
                33940, 47364, 6060,  45120, 2428,  64628, 9744,  55332, 28928,
                47144, 62672, 55848, 27828, 41184, 12548, 37876, 23108, 29828,
                57916, 25880, 53012, 28944, 13440, 65152, 56760, 3700,  16896,
                48880, 33032, 22972, 46240, 5904,  60004, 24,    11204, 25896,
                35084, 22820, 41804, 9908,  21456, 16472, 60764, 10708, 6728,
                19648, 36644, 2248,  59364, 50160, 64336, 5948,  53224, 61664,
                12784, 50924, 31892, 43452, 23528, 53780, 28240, 41628, 17036,
                26396, 54800, 37616, 15920, 6520,  50516, 38920, 3192,  64640,
                54524, 23928, 62372, 38788, 19812, 5440,  48600, 2664,  26824,
                45680, 60048, 5044,  34992, 18488, 59140, 38180, 62140, 19780,
                56492, 16236, 65356, 9072,  52224, 38624, 31592, 61484, 26124,
                36852, 2228,  64988, 51184, 7780,  38504, 63208, 47848, 35492,
                27716, 3352,  23452, 59460, 28416, 52692, 42404, 17712, 55520,
                3436,  15060,
            },
            {
                57420, 41176, 6688,  14688, 44324, 5300,  35208, 17892, 3372,
                10264, 54744, 14940, 36396, 51460, 32676, 20140, 50176, 524,
                33844, 23244, 6112,  49384, 58544, 19788, 46204, 63528, 8996,
                48948, 3164,  22096, 36688, 908,   41128, 24628, 53724, 11612,
                43156, 1216,  18468, 35864, 51704, 16128, 21688, 42248, 57060,
                45988, 60592, 28060, 55128, 32400, 50144, 26704, 34876, 55020,
                45928, 12680, 24284, 40592, 33428, 11784, 18744, 29688, 24360,
                34432, 58940, 4536,  63308, 9896,  20052, 46048, 5464,  32824,
                52148, 7260,  30900, 1740,  25428, 55976, 60100, 14112, 27152,
                42000, 7292,  11568, 50776, 45956, 31728, 65076, 42564, 13380,
                30640, 51012, 24040, 39520, 53084, 29984, 7716,  23168, 42804,
                26784, 31880, 48616, 3056,  27932, 12720, 47180, 7892,  11268,
                53744, 18920, 13400, 34712, 3504,  28936, 18292, 9692,  58908,
                51232, 43828, 33680, 39576, 9416,  32096, 63516, 4832,  46188,
                51960, 62620,
            },
            {
                22956, 2732,  54540, 64236, 25992, 21340, 59656, 28680, 41312,
                24812, 38396, 30156, 18456, 56656, 26432, 6524,  43276, 58812,
                10960, 40368, 15084, 35616, 9676,  32104, 4612,  54856, 34664,
                19244, 39400, 62764, 45108, 32192, 8808,  19140, 45532, 30228,
                37044, 57484, 64476, 12164, 25080, 55472, 38100, 32576, 18912,
                6776,  15364, 2608,  47636, 7376,  58468, 40408, 4316,  62188,
                27768, 51096, 55904, 16588, 4860,  47596, 57188, 38348, 1304,
                45644, 17436, 26188, 40332, 16116, 56736, 35648, 1028,  58420,
                47024, 11768, 64808, 48532, 21384, 29972, 44272, 18996, 47816,
                34288, 16180, 30176, 56192, 26204, 8264,  36980, 53500, 22292,
                6544,  16068, 56548, 116,   10864, 44436, 48844, 13628, 35116,
                6056,  11616, 45372, 20812, 41196, 63432, 23672, 35100, 58680,
                29448, 39744, 47948, 25404, 58268, 44736, 52504, 31792, 24296,
                7192,  14580, 65264, 55184, 20940, 15228, 37668, 24624, 34308,
                10088, 27648,
            },
            {
                38360, 11836, 33740, 29820, 50372, 704,   42960, 57216, 12912,
                65468, 50228, 59804, 8156,  11776, 40648, 35432, 61472, 17532,
                30732, 53660, 65276, 2220,  24424, 52200, 28140, 16056, 42580,
                12328, 56712, 5560,  16624, 54776, 60708, 49244, 5252,  62264,
                21448, 7984,  40564, 29372, 4660,  47208, 8940,  63600, 49864,
                29628, 40288, 65520, 36404, 19476, 13508, 44108, 22552, 15676,
                37912, 8464,  31148, 44668, 61280, 27472, 42132, 15032, 54900,
                10248, 49324, 36804, 2088,  48044, 30780, 25240, 62240, 14880,
                39944, 19296, 35072, 41720, 61052, 12112, 4760,  37228, 1636,
                58616, 52268, 21220, 568,   41424, 17280, 10844, 57148, 34296,
                62612, 47648, 41596, 21184, 64920, 25956, 2628,  55544, 63224,
                52900, 38012, 60296, 56776, 17412, 53396, 1016,  19888, 44364,
                4364,  55892, 9468,  61116, 20732, 15252, 40424, 1880,  62688,
                41972, 19724, 11668, 956,   47724, 57788, 7316,  61140, 49224,
                19372, 43428,
            },
            {
                16920, 61764, 46112, 8424,  39220, 15788, 52812, 32696, 7108,
                20500, 1196,  43116, 22792, 63012, 47892, 14488, 24048, 4604,
                49100, 20768, 27052, 42808, 47304, 61936, 38460, 120,   60848,
                26340, 33160, 51432, 10656, 27096, 38380, 14684, 34004, 26092,
                52172, 15684, 49792, 44960, 34428, 61084, 27624, 13364, 4048,
                54120, 33864, 10280, 25456, 52140, 1936,  30224, 53988, 48712,
                3208,  64792, 21628, 392,   36348, 19980, 8256,  63800, 31568,
                21244, 62104, 28824, 57984, 8468,  51520, 12280, 37384, 22536,
                28768, 3536,  55452, 8924,  16844, 33700, 53896, 63248, 23008,
                9700,  44720, 64080, 35544, 59592, 49332, 28284, 19960, 2928,
                38028, 9276,  28516, 33488, 50348, 16368, 31644, 40224, 18136,
                29076, 4144,  24156, 34180, 6852,  29952, 37652, 60724, 51252,
                15660, 22592, 42504, 31492, 6020,  54392, 11404, 37240, 48560,
                28736, 53992, 36608, 30720, 44136, 26912, 40724, 13404, 30352,
                440,   58688,
            },
            {
                48316, 24488, 53256, 20144, 60876, 11020, 23500, 47808, 37280,
                27604, 46636, 33800, 5156,  29336, 52508, 1676,  55068, 38668,
                45156, 8012,  36972, 59384, 18044, 10780, 21880, 45012, 49968,
                7392,  20712, 47092, 30444, 23132, 58556, 1568,  41984, 11032,
                55960, 2604,  23496, 58756, 19360, 1396,  52020, 43020, 24380,
                17140, 57996, 21240, 45612, 62416, 38436, 59544, 11160, 18896,
                33904, 57004, 39972, 13872, 53588, 48740, 24052, 3448,  44100,
                52040, 5056,  13632, 43048, 21652, 65180, 41016, 6988,  53168,
                59352, 43936, 24328, 51272, 27608, 39528, 7680,  50312, 26672,
                32404, 39412, 5424,  13952, 24024, 4092,  63288, 45208, 51400,
                24836, 60916, 14224, 3792,  58080, 36860, 61372, 8600,  22016,
                51032, 44128, 14972, 9932,  49616, 43128, 13576, 5636,  32916,
                27856, 63876, 216,   36780, 49936, 27108, 57260, 22256, 15948,
                4648,  61032, 24824, 50740, 17308, 3168,  51720, 21760, 55120,
                36680, 7136,
            },
            {
                13708, 28628, 5196,  35488, 44520, 31272, 3120,  55708, 17336,
                61720, 14292, 57652, 39556, 17188, 43932, 32344, 27448, 63916,
                12700, 56988, 3124,  14284, 33108, 6208,  55632, 30208, 13912,
                58096, 41460, 64764, 3420,  44168, 52496, 20508, 64032, 46508,
                28872, 38568, 32692, 9720,  14504, 40068, 21520, 35500, 60492,
                47448, 588,   41980, 14300, 27936, 8220,  32924, 24232, 42108,
                46956, 9820,  25844, 60136, 30068, 11276, 34256, 58232, 39512,
                25744, 35436, 18816, 55036, 33172, 2868,  45484, 18500, 47844,
                10352, 31400, 62680, 416,   46700, 58980, 20240, 42980, 12628,
                56296, 18264, 28840, 47652, 54872, 33580, 39904, 12780, 31356,
                17840, 42948, 53288, 45960, 19824, 5624,  43348, 12368, 47556,
                616,   55840, 64192, 40116, 26624, 62696, 57020, 24912, 48608,
                38896, 11792, 46876, 17952, 62192, 33776, 3300,  46620, 59504,
                34120, 9180,  39832, 5836,  63268, 35480, 59692, 9648,  44868,
                64924, 32660,
            },
            {
                42372, 56664, 1440,  17416, 57964, 26216, 64128, 40900, 9256,
                50536, 3512,  25340, 54028, 8480,  21324, 60000, 10256, 18684,
                33744, 23532, 28892, 49520, 53140, 41024, 64284, 35532, 23964,
                37444, 4932,  17760, 34788, 13164, 8364,  31308, 36364, 6240,
                17884, 61860, 43620, 53456, 65012, 31168, 55828, 6316,  11536,
                37964, 30140, 53132, 5388,  49404, 17648, 56264, 63324, 5124,
                28164, 52164, 17460, 45472, 6408,  62692, 50632, 17880, 12468,
                7548,  47176, 60760, 38796, 24724, 15468, 28452, 60968, 34204,
                4268,  16452, 37900, 13292, 22684, 5544,  31032, 15620, 47320,
                2840,  62108, 51288, 9976,  42748, 16500, 7708,  58164, 832,
                55724, 7096,  35024, 11424, 26968, 54368, 23856, 59680, 33616,
                27428, 36444, 20100, 32040, 2132,  17064, 45968, 9736,  18812,
                59408, 6888,  52740, 24064, 13652, 8624,  42988, 30600, 18592,
                52404, 44812, 20848, 56956, 11376, 23592, 29624, 15808, 4520,
                26120, 50568,
            },
            {
                10472, 37528, 63668, 40156, 49240, 13376, 6216,  34448, 20048,
                43304, 30664, 35904, 64460, 13180, 49160, 37360, 5504,  47076,
                51616, 39684, 63068, 44456, 20212, 25920, 1592,  17148, 8964,
                52028, 28632, 55408, 39732, 61644, 48004, 15940, 57668, 24848,
                50388, 12956, 3636,  26364, 7132,  49276, 17608, 45316, 26852,
                64264, 8708,  23348, 60900, 35368, 41044, 1004,  50764, 14596,
                36752, 59052, 2160,  23020, 37636, 41328, 1372,  27916, 54460,
                64592, 30812, 104,   9360,  50244, 58156, 5764,  54596, 26044,
                21552, 49124, 57240, 34704, 41908, 65528, 55156, 36200, 60088,
                23692, 6968,  37264, 20432, 30852, 60780, 23100, 26888, 49136,
                39284, 21712, 29740, 63468, 38560, 48520, 30692, 15664, 41156,
                57704, 7636,  12080, 50636, 54764, 22564, 35764, 3296,  54132,
                41644, 34224, 29600, 44180, 58236, 39436, 24956, 63856, 12556,
                1596,  27476, 14416, 48356, 32644, 42560, 47108, 54304, 39956,
                18100, 22260,
            },
            {
                52276, 30480, 20796, 9056,  54472, 22508, 46332, 28860, 53580,
                60632, 10900, 22152, 45056, 4128,  55444, 24684, 42256, 58748,
                640,   16592, 9624,  4628,  31520, 12432, 48092, 59020, 45492,
                62276, 11656, 48988, 612,   23544, 27776, 53664, 2436,  40616,
                60192, 21044, 35464, 47684, 38220, 22896, 59916, 2660,  33200,
                50168, 15916, 55476, 31676, 20128, 11708, 45236, 22044, 31236,
                7956,  43576, 33528, 54804, 15284, 47516, 20660, 32524, 42632,
                15976, 22296, 52772, 13056, 29980, 44208, 36760, 11596, 40048,
                64324, 43112, 8040,  53736, 18064, 28684, 2184,  9080,  40708,
                33336, 44492, 27140, 57124, 2216,  52648, 44376, 36340, 65364,
                14444, 46536, 57628, 1684,  18020, 9688,  3016,  64624, 4736,
                17772, 24960, 61424, 46812, 5432,  41988, 31204, 61880, 27016,
                21592, 15068, 65332, 1988,  20764, 54796, 5748,  49608, 35948,
                41132, 54624, 64852, 37432, 668,   18760, 6956,  62904, 35996,
                2848,  60452,
            },
            {
                14896, 5900,  43612, 27544, 33400, 2152,  37168, 59244, 15216,
                572,   38748, 51544, 17788, 33192, 28560, 61524, 14000, 30840,
                21888, 54096, 36212, 60824, 57304, 38784, 54288, 33356, 2904,
                20112, 25780, 32276, 59276, 18864, 42952, 33268, 11556, 45448,
                8088,  31012, 56668, 16484, 10928, 29800, 41564, 14036, 57720,
                19640, 43712, 39188, 3972,  47776, 64000, 27436, 38864, 55776,
                62024, 20184, 11664, 65260, 26648, 8760,  56704, 60336, 5256,
                36480, 45584, 59308, 40432, 17516, 63740, 20416, 2688,  52204,
                13980, 29116, 1780,  25092, 11176, 51696, 47908, 26100, 19512,
                52992, 11092, 64316, 14388, 38868, 5664,  11880, 19480, 9236,
                32792, 5248,  25596, 41852, 51144, 56348, 44572, 36996, 49184,
                53376, 43472, 38044, 28136, 15616, 58324, 12580, 38556, 49076,
                6400,  56008, 10660, 37772, 47480, 16560, 28712, 10020, 58792,
                22788, 31328, 6500,  25516, 50520, 59192, 27968, 12800, 31496,
                57460, 45916,
            },
            {
                34656, 62724, 48136, 16228, 61776, 50904, 18380, 8372,  24096,
                47980, 27000, 7076,  62528, 40816, 1632,  19532, 35756, 7952,
                45576, 27888, 41808, 24700, 15164, 6884,  28152, 22332, 42228,
                37840, 15592, 44664, 9656,  38388, 6044,  65396, 22144, 55004,
                27200, 39088, 63652, 308,   53940, 62316, 4560,  52108, 36544,
                25576, 1708,  12540, 58692, 24588, 9100,  53632, 16780, 2528,
                25436, 49164, 40496, 30184, 3736,  35008, 51384, 24696, 10996,
                49768, 3604,  25524, 34300, 7300,  48660, 24104, 31508, 46280,
                59704, 19184, 33232, 62152, 45024, 38520, 13888, 62540, 57800,
                16512, 200,   46564, 22740, 34052, 61800, 47800, 29256, 59248,
                54088, 16836, 61932, 12600, 23064, 34092, 20164, 13436, 29400,
                21808, 10052, 1232,  34356, 64788, 8528,  51304, 18228, 760,
                45008, 32052, 51088, 25732, 4252,  34984, 60692, 43700, 19392,
                3856,  46504, 17008, 60964, 9584,  39084, 52880, 23104, 48972,
                8888,  27008,
            },
            {
                68,    24172, 55624, 11008, 4868,  58224, 39344, 45140, 65100,
                32476, 55984, 12968, 57920, 46556, 10104, 52536, 48784, 65176,
                56360, 12232, 2624,  50772, 19016, 44220, 10760, 60348, 5772,
                50488, 64124, 3960,  55152, 51508, 14628, 48480, 36956, 18012,
                4816,  13860, 46260, 24580, 33636, 44004, 27240, 21596, 7700,
                46888, 61536, 28692, 51904, 36892, 6036,  34268, 60604, 13420,
                45508, 4944,  17824, 52932, 60884, 45904, 13720, 18964, 40124,
                29516, 62068, 19556, 56276, 2016,  41800, 57396, 9684,  35588,
                6488,  39340, 49804, 57688, 4884,  30364, 22612, 3508,  32148,
                49944, 29196, 41368, 55484, 18248, 50884, 25164, 40884, 1192,
                49828, 43388, 37456, 31140, 7600,  59412, 26908, 62356, 6928,
                32604, 60468, 55792, 18872, 23420, 44384, 29812, 25200, 63360,
                36252, 59944, 17652, 42364, 62808, 52916, 31876, 14788, 51732,
                38604, 56340, 12052, 35148, 21132, 44540, 15572, 2028,  43108,
                20072, 40936,
            },
            {
                59168, 18556, 36636, 31348, 41664, 25408, 29388, 11720, 3044,
                41784, 20308, 36560, 30168, 23312, 15580, 39228, 26232, 3408,
                17576, 33964, 61920, 46984, 35380, 64756, 52248, 31076, 13636,
                56284, 34472, 27340, 21476, 30512, 60560, 1904,  29164, 57456,
                51908, 42272, 20196, 50424, 8444,  18220, 58288, 40236, 11260,
                54924, 33788, 41220, 15012, 18796, 44368, 29268, 41096, 51164,
                32256, 36992, 58228, 10532, 21876, 38060, 1516,  63436, 44400,
                7960,  54068, 12300, 28348, 51408, 14640, 61928, 26804, 16692,
                55288, 23472, 9392,  15180, 20612, 35200, 54832, 44144, 39592,
                6808,  35992, 59900, 8948,  4324,  31656, 15416, 7932,  35244,
                20032, 27760, 3244,  53176, 48100, 39388, 384,   54300, 40932,
                50328, 45772, 13724, 39828, 52708, 3656,  57368, 41120, 10012,
                14012, 22864, 28576, 8044,  12796, 22104, 488,   26096, 8492,
                62272, 28008, 42664, 53784, 30700, 4484,  57176, 33452, 65252,
                54904, 12084,
            },
            {
                38608, 4104,  52400, 46692, 21172, 14240, 56432, 34636, 53032,
                16256, 50136, 3936,  43344, 6352,  59920, 54512, 31720, 43908,
                23772, 40380, 6572,  29736, 21732, 328,   39616, 25236, 47460,
                18464, 8164,  41500, 46496, 11140, 35512, 24808, 41216, 9544,
                61380, 31604, 3480,  59088, 37580, 13520, 47820, 32044, 63088,
                17748, 5088,  22732, 65128, 48816, 55948, 3908,  20624, 9588,
                64144, 23216, 28624, 7176,  42820, 31336, 26428, 57344, 32388,
                15852, 35840, 39472, 45000, 22004, 32836, 38228, 47496, 528,
                43984, 63504, 27996, 47244, 41392, 61496, 7976,  11860, 64864,
                24672, 20028, 12896, 26960, 43968, 53912, 63400, 45992, 57016,
                24028, 64516, 10524, 60392, 18916, 15520, 46356, 11208, 24000,
                16932, 2948,  26660, 30936, 48172, 7232,  20884, 33936, 53668,
                47524, 5568,  58032, 49412, 40552, 55284, 45768, 36928, 48700,
                32856, 2156,  23920, 7356,  47616, 61844, 25984, 37688, 6876,
                29748, 50408,
            },
            {
                14328, 27928, 64492, 8596,  1968,  62924, 49016, 6268,  22884,
                62012, 9604,  25892, 64060, 48076, 34360, 20080, 7148,  10816,
                58000, 51756, 14024, 59044, 9740,  54620, 16540, 4420,  36872,
                62756, 23508, 912,   61760, 17144, 58016, 6756,  45840, 15848,
                22816, 35876, 12116, 28952, 64976, 5528,  52964, 1252,  29260,
                50644, 43844, 9836,  336,   31848, 11972, 59908, 26336, 54260,
                840,   48020, 14764, 50760, 55492, 16480, 49260, 6092,  52568,
                23252, 904,   63000, 9020,  59812, 5204,  11412, 19492, 51748,
                31200, 4664,  36768, 52604, 1092,  17856, 49060, 27736, 16060,
                56740, 46952, 51680, 62200, 37160, 2288,  21380, 12392, 39016,
                4684,  14612, 44956, 34468, 29592, 5576,  58340, 35104, 65256,
                37540, 6396,  58780, 63520, 11132, 37396, 62184, 15876, 3096,
                31180, 44020, 37164, 20232, 30216, 5076,  64096, 11912, 58592,
                20504, 16292, 64428, 39368, 13328, 19028, 51224, 10436, 17464,
                24688, 44816,
            },
            {
                60224, 22388, 33360, 43384, 54124, 36236, 18640, 43864, 28272,
                39068, 31924, 55252, 17860, 100,   28148, 13348, 62396, 37100,
                46372, 27380, 18680, 32984, 48940, 41868, 27640, 59724, 43884,
                10632, 52904, 32536, 37708, 50076, 20580, 54112, 32720, 64036,
                944,   48336, 54756, 43056, 21432, 25504, 39264, 15704, 23372,
                36420, 57264, 27020, 59228, 39560, 23712, 46768, 37780, 17196,
                43692, 34508, 61900, 39696, 2256,  64448, 20436, 11492, 41396,
                58708, 47940, 26600, 16748, 49512, 29300, 55920, 64380, 40460,
                21728, 13880, 60796, 10848, 26196, 32664, 58464, 37376, 53184,
                5760,  1784,  33308, 22852, 16900, 41896, 29340, 60152, 51412,
                32068, 55220, 40644, 21740, 51188, 25004, 43448, 31188, 19748,
                51576, 42292, 21580, 34748, 17636, 25568, 43036, 50032, 27044,
                55572, 64520, 1248,  15100, 60528, 34364, 18076, 24384, 41052,
                5780,  45128, 49748, 56728, 28932, 3220,  41820, 46304, 62592,
                34816, 6476,
            },
            {
                47956, 40212, 11548, 17092, 25108, 30712, 10580, 60352, 1076,
                58584, 45324, 12404, 51956, 38128, 41948, 55740, 49800, 21988,
                1524,  64120, 5668,  38148, 23476, 63108, 12748, 33552, 20756,
                48748, 29344, 56984, 13756, 3804,  43652, 27728, 11888, 50868,
                38244, 26000, 17264, 9240,  34584, 56388, 45312, 59572, 7480,
                12692, 46572, 19904, 34608, 52120, 7372,  14536, 30988, 61024,
                6616,  20120, 26716, 12436, 33368, 24304, 37108, 46688, 28696,
                4600,  19860, 33040, 54600, 42224, 3088,  36136, 25496, 8252,
                57604, 34176, 42732, 54488, 23088, 45732, 2996,  21500, 42904,
                30236, 40500, 10552, 57588, 49120, 6864,  34396, 9512,  25720,
                18148, 48388, 7828,  2344,  62408, 56180, 9224,  3580,  12992,
                27412, 56756, 8820,  45004, 54352, 224,   57992, 12152, 39624,
                17512, 23796, 11124, 51672, 26588, 39504, 9324,  53476, 29168,
                55516, 34292, 9852,  21704, 35896, 59648, 31436, 22240, 57692,
                1060,  53604,
            },
            {
                19968, 4436,  56744, 50664, 61492, 6960,  40848, 47692, 14924,
                19636, 4948,  35860, 21848, 8792,  60460, 24928, 4840,  16992,
                34888, 53732, 43180, 57068, 2884,  8064,  45876, 55648, 2020,
                6484,  16276, 42608, 26152, 8756,  62548, 39812, 4452,  19248,
                56816, 44308, 5128,  62616, 50216, 3048,  18776, 32844, 64220,
                40860, 53764, 3680,  16736, 43192, 63720, 3264,  56844, 41356,
                10412, 53196, 46532, 4440,  58296, 44068, 8768,  60800, 34452,
                13924, 65384, 7128,  37460, 23576, 14380, 44604, 17736, 48276,
                2420,  28916, 16384, 6124,  39152, 63760, 13556, 9372,  34724,
                63136, 18824, 45496, 28616, 13948, 64256, 54492, 46808, 580,
                37724, 63316, 28828, 13300, 36084, 16996, 38940, 53512, 60856,
                47360, 1804,  29716, 13996, 49232, 32456, 5628,  21268, 60092,
                7948,  33564, 41960, 45856, 57056, 3236,  49604, 44504, 1720,
                14296, 61296, 26060, 312,   15476, 52924, 7632,  12344, 39516,
                15992, 30520,
            },
            {
                36308, 62412, 27292, 1384,  37920, 21944, 56132, 26616, 32592,
                51100, 24500, 57152, 29692, 47104, 15464, 32360, 45436, 39912,
                29548, 11848, 25460, 15432, 31472, 52528, 17800, 36488, 24236,
                58776, 38964, 64372, 22212, 48196, 31008, 58904, 24112, 34804,
                8496,  59808, 31104, 40344, 14652, 29376, 10668, 49320, 25936,
                5844,  30628, 62040, 11012, 29036, 21128, 35636, 50128, 28196,
                22340, 37208, 56268, 15696, 30020, 51788, 3072,  17076, 55308,
                38988, 44796, 50980, 10752, 57400, 62160, 31060, 53680, 11976,
                60056, 50476, 45148, 19796, 56108, 29808, 50908, 59320, 48208,
                14900, 25252, 60252, 4080,  39144, 24368, 19628, 43252, 58972,
                15740, 23052, 56944, 42144, 46664, 26928, 49284, 22756, 32980,
                16556, 40328, 62848, 23824, 38260, 61260, 27948, 46484, 35824,
                52956, 28980, 61744, 6812,  21924, 31744, 16176, 58996, 19872,
                37632, 30668, 51608, 43140, 38684, 63604, 48484, 27268, 55616,
                44216, 9732,
            },
            {
                50180, 42912, 32716, 15652, 45344, 12616, 52292, 4272,  36904,
                61404, 41716, 7412,  65124, 2168,  52556, 11136, 62716, 56676,
                8160,  50840, 60768, 47600, 21308, 40504, 28972, 62360, 50720,
                32288, 11820, 53496, 1472,  35832, 17440, 52392, 14824, 47188,
                28280, 13456, 21100, 52544, 24332, 60920, 43460, 55588, 37260,
                15380, 22420, 49712, 39032, 55684, 47520, 25356, 12216, 1644,
                64984, 31564, 6080,  42312, 19564, 63064, 27280, 48568, 22500,
                1820,  25364, 30524, 17972, 700,   46776, 6708,  34644, 22224,
                26488, 37056, 62572, 9776,  33460, 4248,  17324, 27048, 7152,
                55212, 856,   52552, 35724, 50256, 5916,  32308, 12024, 27836,
                5384,  50124, 31736, 9788,  20292, 1420,  64244, 6732,  44508,
                10208, 35960, 55192, 3988,  19200, 52652, 9928,  41684, 14272,
                3956,  48640, 19032, 37992, 13488, 63704, 35696, 24988, 41680,
                65500, 11704, 6452,  46336, 18316, 24344, 4408,  34124, 19456,
                64204, 25392,
            },
            {
                13588, 22872, 5616,  48704, 58860, 34464, 18212, 63480, 8976,
                13448, 49048, 17172, 33156, 39600, 19996, 27780, 3828,  22968,
                42184, 19212, 864,   34524, 65308, 10172, 3900,  13868, 43516,
                8336,  28016, 19228, 45660, 56560, 10468, 3284,  42452, 65236,
                54144, 3848,  37684, 45788, 196,   36160, 7784,  20996, 2448,
                42436, 58804, 33792, 1044,  8440,  60868, 15752, 44276, 51944,
                17812, 47904, 59348, 23648, 35656, 10536, 40400, 32112, 59316,
                11832, 53056, 61604, 41336, 27612, 20460, 39064, 64820, 3844,
                41072, 14828, 364,   24884, 47680, 41944, 37560, 22876, 44028,
                39496, 32988, 21672, 10184, 16520, 55512, 62112, 40104, 52868,
                35264, 44552, 3768,  61448, 54076, 34220, 14520, 29024, 58448,
                19976, 25944, 48448, 7988,  43556, 15896, 30824, 23404, 65164,
                56752, 25584, 1464,  55000, 47272, 10332, 51800, 3880,  8016,
                48812, 22992, 54964, 60624, 32616, 9096,  58580, 37800, 2576,
                51824, 7164,
            },
            {
                59720, 53980, 18932, 65504, 28556, 2860,  42764, 23520, 46096,
                28836, 3596,  54264, 25832, 44596, 59132, 37120, 49572, 33372,
                13512, 58604, 37932, 6184,  44808, 54844, 26052, 47996, 20280,
                60656, 40720, 5148,  33312, 61892, 29000, 37364, 21684, 7104,
                40636, 33152, 63076, 11332, 57864, 16944, 32116, 63252, 47768,
                52364, 12976, 27100, 44792, 18852, 37656, 4540,  57708, 33484,
                9004,  39636, 13040, 164,   49772, 54580, 14692, 5264,  43100,
                20736, 33980, 8436,  48820, 58956, 13076, 50580, 9536,  43904,
                55316, 31912, 58920, 52228, 13156, 65532, 56764, 2404,  60948,
                12148, 29332, 64696, 42316, 47184, 28956, 1476,  21892, 8952,
                65304, 19036, 12736, 39444, 24260, 59224, 42512, 37704, 4416,
                51480, 63724, 13068, 33296, 57736, 62244, 1328,  50824, 7360,
                37428, 16400, 32548, 43040, 23056, 30324, 60508, 27552, 45300,
                33856, 17200, 4172,  28316, 12532, 42056, 53788, 14712, 45608,
                29004, 41080,
            },
            {
                492,   38304, 31064, 8208,  40364, 10948, 55228, 31588, 58260,
                37984, 22012, 60356, 9524,  14052, 55272, 7156,  17664, 64436,
                46568, 27076, 30612, 23384, 14872, 59476, 33072, 37620, 396,
                56096, 15980, 50500, 23996, 13472, 43852, 51348, 60524, 24932,
                17624, 49640, 26444, 19700, 48284, 53492, 25512, 11000, 29100,
                18056, 5168,  65524, 23696, 53832, 32312, 27988, 40996, 20976,
                61344, 25168, 28920, 43632, 60680, 7536,  26072, 64308, 47428,
                56896, 4132,  15888, 36472, 22888, 32724, 54108, 29364, 18564,
                47404, 5276,  21368, 36036, 28668, 7732,  20220, 31312, 51248,
                46672, 19120, 7452,  57744, 25444, 37096, 60072, 14752, 46424,
                25264, 57728, 29352, 51796, 17112, 7556,  50068, 11780, 56328,
                31368, 696,   41448, 27804, 21104, 37880, 47352, 34192, 19784,
                44920, 59292, 9708,  62536, 5280,  39736, 18392, 56056, 13144,
                52996, 40220, 57432, 36496, 20008, 47668, 30248, 22432, 60820,
                17536, 33756,
            },
            {
                46976, 11812, 56784, 44968, 21084, 25688, 49736, 14840, 368,
                18808, 41036, 46800, 35352, 1204,  23536, 30856, 43468, 2892,
                52264, 9368,  55736, 41412, 50224, 18476, 7464,  53100, 23512,
                30092, 46512, 64840, 38272, 7772,  19744, 652,   32212, 12088,
                56444, 2320,  8668,  30212, 40888, 4488,  44304, 38724, 60316,
                35092, 40684, 56568, 10512, 50108, 13476, 63588, 7060,  54748,
                2488,  36792, 53480, 16008, 31448, 20320, 38524, 34800, 17660,
                29228, 39908, 62988, 45632, 6076,  2232,  60972, 15600, 24604,
                63204, 11148, 39824, 16784, 45040, 54048, 10852, 35428, 15700,
                5532,  54276, 38620, 13536, 3148,  18428, 43680, 50596, 33284,
                6460,  41728, 36320, 84,    47696, 32488, 20856, 25476, 45720,
                17584, 23128, 46784, 53928, 5976,  10464, 25752, 55864, 12188,
                29120, 40788, 53040, 27216, 15356, 50420, 2712,  37200, 21248,
                30868, 980,   25572, 61952, 50976, 2140,  64956, 4836,  10608,
                55944, 24436,
            },
            {
                63756, 26904, 15420, 36128, 52780, 61984, 6668,  34692, 64468,
                53272, 11804, 29848, 62812, 51712, 40056, 61944, 11696, 21744,
                39192, 16804, 5284,  62532, 11580, 29072, 39880, 63500, 12876,
                35272, 10232, 3040,  26676, 58312, 47228, 55676, 27740, 45288,
                35364, 42556, 58828, 61912, 34404, 15388, 56208, 1288,  22264,
                8140,  46560, 30804, 4004,  39480, 21780, 47732, 16928, 42972,
                12000, 48980, 64104, 4204,  46224, 57520, 52124, 1404,  9996,
                24516, 51244, 12160, 26288, 56136, 42424, 38312, 7040,  35516,
                57076, 30792, 50648, 61612, 1952,  24536, 48776, 62952, 26008,
                43052, 22644, 62008, 48928, 34604, 9428,  56440, 30232, 3460,
                53964, 16112, 63112, 10572, 27464, 60116, 2724,  64596, 39892,
                7884,  61520, 35548, 15324, 60504, 40620, 17360, 63780, 2752,
                51396, 6052,  21020, 35476, 45568, 57516, 9348,  43380, 63172,
                59432, 10588, 45724, 15028, 8192,  38876, 26104, 43572, 36032,
                49524, 7704,
            },
            {
                20020, 51376, 4704,  60416, 1564,  17744, 38336, 44160, 26876,
                8092,  48680, 5216,  16012, 19408, 26972, 47860, 57244, 34132,
                25756, 60932, 35920, 44124, 1832,  22024, 48636, 4028,  43236,
                58916, 21412, 41992, 53568, 34216, 14908, 39888, 9248,  64332,
                15560, 51580, 21612, 12604, 6636,  23968, 64068, 49148, 13432,
                53692, 25400, 15264, 61020, 35448, 444,   59568, 26260, 30992,
                34740, 22800, 18828, 10056, 40864, 23444, 13204, 60992, 44620,
                55580, 2812,  31320, 16956, 49872, 20044, 27956, 46324, 52860,
                1456,  43136, 26780, 6804,  57876, 32544, 40244, 3748,  59260,
                36836, 240,   27636, 31460, 52940, 64040, 23608, 12260, 38500,
                20260, 47084, 23208, 56000, 44444, 37220, 53036, 14952, 34184,
                55208, 11060, 50892, 2416,  30440, 44564, 49752, 32244, 36944,
                23596, 61112, 13196, 52,    63988, 24140, 33432, 26712, 6780,
                16684, 49576, 41592, 23136, 34304, 56168, 18164, 52308, 31932,
                14048, 40752,
            },
            {
                29784, 43456, 33880, 23260, 28376, 47296, 12408, 57904, 22400,
                33044, 56864, 25160, 45096, 32500, 3932,  8392,  15172, 53276,
                208,   47124, 19768, 31632, 54396, 34760, 57032, 16404, 27180,
                51584, 6868,  31396, 17796, 62176, 23176, 4640,  48688, 19388,
                5652,  38468, 27304, 47216, 52748, 37152, 28760, 18988, 32756,
                58888, 42936, 20396, 51884, 29020, 45308, 9548,  56160, 50680,
                3496,  44416, 58840, 27352, 34040, 5092,  29924, 48476, 37296,
                21332, 64996, 41268, 35096, 58292, 10584, 62552, 13920, 22620,
                8992,  19544, 14672, 37472, 22072, 43772, 18140, 13780, 29992,
                56140, 9892,  16272, 44616, 19948, 6680,  41256, 27332, 61980,
                58652, 7992,  34696, 14220, 5640,  18664, 30016, 42796, 4916,
                28132, 20152, 38852, 24412, 56800, 4796,  21532, 8556,  14804,
                46588, 56612, 31364, 49128, 39048, 19504, 12276, 47056, 52288,
                35164, 28940, 5060,  63616, 31276, 12192, 59824, 5960,  21328,
                61824, 2036,
            },
            {
                54364, 10724, 58836, 49156, 8460,  42004, 30560, 4544,  51784,
                15088, 37236, 63744, 41388, 53852, 59940, 37424, 42940, 29636,
                58944, 10272, 50672, 13500, 64768, 24756, 9452,  46152, 19584,
                37700, 62984, 48512, 11544, 1956,  37248, 29896, 54292, 60196,
                24068, 31628, 804,   55596, 17480, 45028, 9592,  41708, 5964,
                36740, 2912,  63084, 7668,  18084, 41200, 23820, 37784, 14936,
                61868, 8152,  38196, 54852, 49600, 62196, 18160, 7676,  32648,
                14864, 5680,  53104, 8072,  24128, 108,   32084, 49116, 40316,
                64968, 34084, 53904, 48008, 11956, 64296, 8324,  52560, 45700,
                20364, 50456, 39836, 58240, 3864,  47316, 15020, 51716, 976,
                43176, 31640, 26064, 51128, 40268, 61456, 9680,  23560, 49624,
                58648, 45084, 32704, 65068, 12608, 27560, 53080, 59516, 39700,
                26324, 42788, 17560, 7900,  28572, 59988, 55524, 1600,  39644,
                22320, 57132, 18940, 53880, 47508, 556,   39928, 28336, 44928,
                57468, 37180,
            },
            {
                6368,  17868, 39116, 13716, 64880, 54992, 19352, 62452, 39792,
                3004,  20744, 10308, 1688,  13116, 22476, 50368, 19004, 6296,
                40256, 23336, 27972, 3308,  38976, 42204, 5612,  61376, 30564,
                1172,  13972, 25100, 44704, 51952, 59420, 42284, 11856, 33916,
                44280, 62440, 10636, 40480, 33320, 3020,  57084, 60896, 50732,
                26720, 11504, 47628, 33084, 57860, 12792, 64720, 4844,  20380,
                52884, 29856, 13016, 680,   15488, 42856, 54000, 25528, 57988,
                43752, 27884, 19508, 47328, 60552, 44608, 36820, 4064,  57444,
                29092, 45652, 59688, 3876,  27848, 55372, 35000, 23588, 5676,
                61056, 33008, 12728, 25348, 36332, 60464, 30724, 35204, 18280,
                49436, 11788, 64972, 1700,  21052, 57296, 47368, 13088, 63156,
                1116,  16156, 6744,  48184, 18992, 42100, 34444, 988,   62644,
                10904, 3552,  34172, 53584, 44352, 4424,  16164, 31760, 64740,
                13532, 3000,  37316, 9116,  42728, 17132, 23948, 49288, 10064,
                16028, 25784,
            },
            {
                63280, 45464, 21428, 32436, 3824,  36408, 24480, 9448,  49960,
                60312, 46740, 30888, 55608, 35984, 28184, 65156, 11108, 32944,
                61660, 44984, 53612, 17708, 59620, 29460, 15328, 53024, 34160,
                58020, 40880, 55652, 33444, 6096,  20808, 26312, 16760, 3784,
                50744, 14624, 57380, 20832, 64928, 25836, 14104, 22632, 30860,
                16536, 55952, 38600, 24456, 2128,  54128, 46904, 27876, 33716,
                41924, 47396, 25300, 65312, 21872, 36124, 10912, 40168, 1120,
                49724, 61392, 37852, 11608, 29908, 15048, 51896, 17900, 25648,
                11212, 5448,  16660, 41144, 31456, 1024,  49584, 38320, 26708,
                41580, 2680,  65208, 7800,  53420, 21696, 10276, 56748, 24744,
                4976,  37816, 45672, 55324, 27300, 35508, 4184,  31840, 37664,
                26564, 40792, 54936, 9500,  36896, 58520, 16432, 47476, 29712,
                20312, 50088, 65316, 22396, 10368, 35792, 49516, 41996, 25320,
                58896, 45264, 51324, 26556, 61172, 30052, 55232, 64588, 3376,
                34196, 48500,
            },
            {
                30816, 400,   57312, 26212, 52312, 15564, 45752, 34012, 27396,
                17688, 42680, 24152, 49500, 16976, 5392,  43860, 25024, 56228,
                15672, 4556,  36988, 8552,  46988, 21112, 49832, 11876, 44240,
                22472, 9008,  18904, 28904, 63304, 9752,  46432, 57748, 39372,
                27832, 7836,  47912, 29904, 5180,  49932, 39292, 46268, 536,
                63944, 43824, 21008, 51092, 30296, 36224, 16872, 9112,  58352,
                2852,  17992, 56248, 32728, 45388, 4260,  28580, 63008, 23000,
                13384, 33944, 2936,  54544, 40800, 21376, 63408, 7624,  54656,
                38900, 21540, 51108, 24716, 61916, 19952, 58372, 9728,  14544,
                54988, 48252, 18668, 28948, 42136, 45824, 3116,  63796, 40692,
                54356, 22492, 16876, 30864, 8936,  15112, 43300, 54064, 18240,
                52168, 22352, 61548, 30684, 2548,  51292, 23272, 7620,  55368,
                37988, 15360, 58300, 40252, 26964, 61700, 21300, 54448, 7432,
                10840, 31080, 20488, 14648, 35608, 6952,  12840, 37796, 53000,
                41484, 22912,
            },
            {
                11896, 51124, 42616, 7572,  60836, 40240, 2332,  59144, 12716,
                6752,  35440, 57784, 8236,  61584, 39260, 47372, 1056,  51504,
                35256, 21600, 64028, 31216, 55876, 628,   38200, 25216, 4656,
                65324, 48068, 2612,  38756, 15492, 49844, 35504, 1132,  52892,
                22360, 37140, 42456, 16856, 54056, 35932, 8676,  19676, 53704,
                33812, 5052,  9492,  14464, 62212, 6840,  42860, 50292, 63176,
                23428, 39952, 10372, 7172,  50624, 59488, 19400, 46684, 8200,
                52656, 17248, 26536, 57640, 5096,  25040, 43260, 34944, 30644,
                60924, 44344, 33316, 8672,  13208, 46780, 43076, 17424, 62772,
                31220, 22412, 37484, 57388, 14392, 33376, 17456, 48312, 29556,
                13876, 6916,  60936, 41560, 49992, 63624, 24280, 60008, 6336,
                10792, 34820, 14008, 44156, 26920, 64020, 11932, 32972, 43716,
                25592, 6072,  31036, 844,   47736, 13768, 5296,  29480, 17556,
                38592, 48744, 62872, 4516,  57584, 46312, 22100, 32028, 19500,
                8228,  60580,
            },
            {
                55712, 35372, 16456, 29236, 11440, 48440, 22288, 30268, 53672,
                63452, 320,   29028, 14268, 21484, 31444, 58396, 18472, 29320,
                12712, 48840, 42524, 25636, 11080, 34912, 62232, 53920, 28596,
                16036, 36024, 52296, 61028, 42932, 22944, 30108, 64816, 18804,
                59180, 4496,  63044, 24432, 12008, 59588, 27500, 61564, 12808,
                41420, 28640, 58524, 46528, 39360, 26248, 21708, 31420, 12136,
                37020, 52444, 61200, 26560, 16800, 38740, 54428, 34908, 30504,
                41748, 64416, 44724, 9640,  32812, 59556, 12400, 1536,  48596,
                14188, 3612,  64044, 53520, 37372, 4636,  33956, 28528, 1736,
                45864, 5140,  10812, 50972, 716,   61688, 27184, 8796,  36656,
                59100, 50872, 34232, 3328,  19364, 38184, 2244,  46052, 29580,
                41952, 50276, 57884, 5136,  20056, 46388, 39896, 3780,  61208,
                52156, 12312, 45224, 55940, 18620, 37732, 51740, 44676, 60676,
                34420, 2560,  43064, 24272, 39768, 52036, 1344,  58948, 44244,
                15204, 27596,
            },
            {
                2748,  20464, 46736, 37416, 63836, 18724, 56524, 43748, 36984,
                20472, 51436, 40640, 45976, 54220, 4056,  10072, 37512, 63024,
                7852,  54916, 3484,  15040, 58912, 45684, 18036, 8084,  43264,
                56488, 31716, 25916, 12984, 5364,  55060, 8124,  12448, 43856,
                31412, 9944,  34056, 46060, 3728,  31284, 44096, 6552,  49204,
                36824, 24916, 53008, 18644, 1552,  60028, 55240, 4096,  46104,
                15300, 1640,  34340, 43972, 31164, 11936, 2240,  15080, 58364,
                4060,  21928, 48724, 19080, 51076, 37956, 46028, 20368, 56512,
                26840, 18492, 40484, 28896, 57024, 22180, 52060, 59996, 39100,
                54032, 35188, 63372, 43684, 23880, 39212, 52816, 20800, 4088,
                44120, 25812, 10748, 46968, 28856, 58348, 12596, 33652, 21036,
                64844, 376,   25096, 38432, 54696, 15224, 28792, 57044, 21620,
                17508, 35012, 63120, 23028, 8472,  33100, 64356, 1520,  22716,
                55816, 27420, 52796, 11296, 32772, 16568, 28072, 62760, 5080,
                48060, 39484,
            },
            {
                65372, 6516,  58864, 24740, 1308,  33128, 4712,  9264,  15108,
                47812, 25532, 11096, 64608, 34544, 26548, 52080, 41528, 22304,
                45476, 27272, 39376, 20068, 51228, 5696,  32412, 21968, 39304,
                3836,  10140, 20476, 46972, 58868, 33720, 40820, 25232, 48188,
                15748, 55224, 50104, 20560, 56868, 40216, 17616, 23004, 57496,
                15844, 3600,  64776, 33480, 11068, 44040, 35172, 19052, 57776,
                28996, 48952, 19680, 6312,  56696, 64048, 24420, 49848, 27192,
                38404, 12288, 35888, 888,   27976, 16120, 65428, 8328,  52176,
                36664, 10556, 47772, 600,   15640, 26108, 7196,  11660, 19604,
                24584, 13844, 28076, 17940, 6844,  59708, 11824, 41872, 55880,
                32220, 17184, 62000, 55016, 22600, 7844,  51112, 55576, 15800,
                8872,  48024, 17696, 32504, 60652, 8360,  35732, 48332, 9860,
                41640, 2632,  50432, 28468, 42812, 57764, 25564, 12228, 40664,
                8720,  15132, 19288, 65048, 7488,  49700, 42252, 10240, 24528,
                54400, 33820,
            },
            {
                23292, 50304, 9876,  40452, 54528, 51068, 26460, 39536, 58824,
                5828,  32808, 18596, 2792,  42924, 16332, 61400, 13516, 32780,
                1920,  59796, 34208, 63508, 29180, 41352, 57912, 12424, 63060,
                50132, 60532, 37828, 132,   28104, 18356, 51476, 2504,  61832,
                36380, 26700, 472,   14124, 37388, 52272, 2084,  62628, 32584,
                8892,  45424, 22168, 50576, 29656, 15972, 52564, 8540,  41524,
                25412, 63564, 53532, 22768, 40668, 47528, 9200,  43324, 61504,
                6664,  56092, 31088, 62436, 54172, 5808,  22728, 30000, 41620,
                60396, 23944, 32348, 59288, 44744, 50196, 64440, 41136, 47960,
                57952, 8120,  49860, 56460, 34380, 30484, 48760, 15248, 64780,
                23400, 168,   40064, 14028, 36612, 44496, 26056, 40456, 59536,
                37348, 27908, 43816, 11520, 50496, 23812, 1256,  63612, 31684,
                26144, 54860, 38712, 15008, 3980,  48464, 17000, 31948, 50396,
                59896, 46540, 30312, 57872, 35412, 20784, 56640, 36876, 30388,
                18364, 12196,
            },
            {
                41760, 31928, 28720, 13848, 44768, 16812, 62876, 29884, 22688,
                52760, 60620, 38728, 55728, 7916,  23516, 48016, 5688,  56288,
                50024, 17268, 12056, 8884,  23212, 1496,  48792, 26620, 44428,
                17240, 23728, 54232, 42652, 63828, 14832, 38408, 57552, 21164,
                6936,  41472, 60020, 29524, 65268, 25500, 11448, 46980, 28068,
                55340, 37812, 13324, 40988, 5468,  63232, 23456, 38488, 60500,
                5372,  10916, 36468, 14244, 304,   28412, 35772, 20496, 16140,
                51996, 23468, 46716, 14556, 42352, 39096, 57804, 33760, 2512,
                16680, 5020,  54444, 20716, 9092,  34952, 30128, 16728, 5336,
                43188, 32204, 38324, 3528,  45776, 21856, 2464,  35168, 7768,
                46056, 52624, 29824, 6412,  63908, 18048, 1648,  31552, 4908,
                22804, 53688, 62940, 6228,  41228, 18560, 53368, 44568, 14136,
                58876, 7160,  19672, 61740, 10392, 36700, 54320, 5884,  20256,
                35728, 4772,  38948, 388,   44908, 26268, 14160, 2052,  51516,
                59268, 45708,
            },
            {
                15628, 62208, 56812, 5400,  21800, 34864, 11372, 46868, 42428,
                1112,  13592, 27584, 50212, 30924, 59120, 36400, 19708, 30148,
                25016, 43272, 53700, 46836, 37792, 55116, 15548, 35660, 2696,
                30344, 34560, 13816, 8664,  32072, 46228, 4552,  11264, 30536,
                45624, 53156, 18408, 8988,  43376, 6380,  35016, 51064, 41300,
                17948, 1276,  61252, 25468, 57204, 45820, 604,   32600, 13940,
                48032, 30608, 44780, 58128, 18124, 60408, 54116, 3180,  33068,
                40036, 8968,  59864, 4156,  25692, 10004, 48524, 13264, 44820,
                50284, 63380, 38008, 42124, 13792, 61444, 3092,  36776, 27220,
                60540, 21000, 65000, 16024, 25764, 53916, 62288, 19064, 27456,
                37520, 59468, 20096, 49416, 33560, 54228, 11740, 61556, 49496,
                13284, 34592, 2252,  30376, 56468, 36696, 27692, 4652,  21032,
                34736, 47456, 30280, 45760, 24472, 60144, 41976, 29360, 63948,
                43532, 55172, 23584, 12964, 60488, 6132,  48364, 63336, 39040,
                8204,  3788,
            },
            {
                52876, 19320, 37148, 48352, 61036, 2456,  57780, 7740,  17888,
                64176, 35880, 45364, 20820, 11868, 44092, 768,   40548, 65480,
                7644,  35524, 4592,  61184, 31196, 19324, 64832, 9816,  52412,
                59652, 6640,  49208, 58424, 22300, 26412, 55888, 35156, 64360,
                23804, 12660, 33204, 48884, 22372, 54960, 14796, 20412, 4940,
                59376, 31476, 48456, 19808, 34572, 10340, 27600, 51596, 20896,
                56424, 3620,  24276, 38820, 50816, 31524, 12480, 45796, 64948,
                27520, 18352, 43892, 35004, 21004, 52660, 28832, 62052, 20000,
                26636, 11408, 29272, 6456,  24504, 56100, 19884, 49044, 53948,
                272,   12732, 51804, 8920,  42032, 11184, 38960, 58460, 51136,
                12972, 9456,  41932, 2820,  23280, 47728, 29160, 41672, 19256,
                45516, 58676, 15936, 21676, 46852, 12668, 61136, 40304, 50860,
                10908, 65016, 1916,  53264, 33344, 21304, 744,   13732, 26004,
                9568,  16820, 49888, 28788, 53296, 40528, 32320, 16388, 21748,
                28216, 34988,
            },
            {
                26068, 72,    9060,  24520, 42092, 28040, 37868, 50448, 31576,
                24492, 9340,  57492, 4108,  63152, 26936, 54588, 10236, 15128,
                58392, 21536, 27144, 13856, 42268, 6800,  24252, 45756, 38508,
                20176, 27492, 40324, 1908,  53252, 19024, 41904, 50152, 16496,
                3516,  39604, 57712, 1712,  61216, 31096, 39424, 63872, 26540,
                43776, 9940,  54204, 7140,  39640, 50064, 17976, 62128, 42752,
                35452, 17056, 64456, 9616,  6464,  42472, 24832, 5200,  57556,
                49292, 1296,  30188, 55080, 64268, 17040, 380,   36588, 7792,
                55884, 34100, 47468, 53136, 43956, 31672, 39668, 10988, 45180,
                24308, 30716, 35856, 47232, 57364, 29176, 6024,  44196, 1388,
                31712, 25036, 55164, 60712, 15404, 38264, 9048,  55756, 24788,
                7320,  39240, 51488, 26528, 64716, 8244,  32868, 16776, 57888,
                25156, 37596, 17668, 13084, 40208, 8304,  51360, 58900, 47992,
                33448, 3140,  62560, 34848, 18956, 10648, 24180, 54956, 42884,
                60520, 47472,
            },
            {
                64556, 43896, 55820, 33380, 17296, 52972, 14348, 21360, 55352,
                40908, 48800, 16052, 33948, 38356, 17524, 50572, 33140, 46216,
                52452, 38816, 48536, 288,   57356, 50164, 28488, 56416, 4336,
                14904, 64072, 45068, 36056, 12264, 62880, 5924,  9552,  28256,
                60480, 47644, 26128, 37028, 16600, 45744, 3952,  57220, 12648,
                22436, 36604, 16260, 29472, 65172, 13100, 55656, 2888,  8180,
                25808, 53648, 46156, 33932, 19936, 62456, 15568, 37696, 21460,
                10772, 36448, 13648, 47560, 6832,  32408, 42900, 58796, 23736,
                40656, 2956,  17852, 65084, 1052,  15508, 58416, 7508,  63428,
                17220, 41048, 59904, 2640,  18072, 33600, 23840, 15052, 48096,
                64112, 17384, 34660, 45152, 27080, 4012,  65424, 35540, 596,
                62180, 32228, 10200, 42768, 4396,  49528, 24080, 284,   43172,
                29184, 5768,  44048, 56392, 27232, 63092, 45252, 19072, 38280,
                57276, 21860, 41696, 7144,  47332, 64772, 2444,  36388, 5632,
                9920,  13560,
            },
            {
                39580, 29500, 50204, 11744, 63552, 3928,  45052, 6292,  60268,
                2292,  29152, 53328, 6656,  25048, 59624, 3444,  22616, 28540,
                2536,  18616, 62576, 32896, 16888, 36796, 11652, 32236, 41796,
                53712, 10204, 23916, 17368, 30772, 47324, 38696, 32992, 55268,
                43168, 19720, 7044,  11248, 51856, 28612, 9280,  49268, 33972,
                52072, 62584, 46272, 2064,  23324, 44288, 36464, 29052, 40704,
                60064, 12612, 1264,  28548, 58668, 48140, 26864, 55732, 41892,
                51752, 62916, 24316, 57868, 39572, 11644, 51148, 46072, 15212,
                61040, 51992, 22160, 12348, 36384, 26404, 50752, 21656, 33900,
                55592, 5744,  27032, 22428, 63244, 49468, 55668, 36168, 20304,
                40488, 52064, 6740,  12016, 57500, 21256, 43500, 17100, 52696,
                28676, 20668, 57324, 35316, 18328, 38392, 59848, 54928, 9308,
                52836, 48588, 61368, 22440, 3400,  35484, 15812, 6164,  30748,
                11680, 52832, 26320, 15096, 57980, 29864, 44184, 50948, 56996,
                31664, 20908,
            },
            {
                3332,  16148, 6712,  22532, 38644, 58788, 32580, 25800, 35444,
                12812, 19536, 61808, 42572, 47944, 13352, 41040, 63556, 36640,
                12380, 43492, 9440,  53484, 22836, 59964, 3184,  62268, 21604,
                48176, 33772, 61304, 50804, 3676,  60176, 24760, 21208, 996,
                13484, 35736, 54412, 64560, 41636, 19188, 24204, 38416, 172,
                28180, 5804,  41084, 57900, 32940, 4568,  21288, 47408, 15816,
                32032, 49728, 22784, 39760, 52600, 11364, 1856,  32800, 7920,
                29316, 16352, 44600, 2452,  22148, 27840, 19284, 4696,  31128,
                10356, 37576, 28304, 56636, 46384, 61268, 4016,  29368, 38212,
                13932, 51168, 44576, 8320,  39900, 12128, 4508,  60140, 8740,
                28520, 3100,  62340, 30304, 39664, 50960, 32048, 10700, 48204,
                14084, 44892, 2472,  53608, 14560, 45748, 30496, 21788, 35020,
                15244, 19880, 32268, 11288, 50096, 29444, 55496, 24552, 43872,
                1368,  60672, 46064, 37256, 4188,  20584, 12868, 25536, 17392,
                48908, 58816,
            },
            {
                37036, 54568, 61412, 47068, 30204, 19752, 10496, 48392, 64564,
                39488, 46116, 22780, 10108, 56616, 31868, 20116, 8500,  54208,
                59012, 30764, 25440, 45596, 7212,  40808, 47136, 18224, 26292,
                7588,  532,   28724, 42908, 8376,  14232, 56936, 44460, 51388,
                62036, 29204, 22960, 32508, 2272,  58800, 62896, 44748, 17164,
                56076, 20688, 14056, 49440, 26376, 60808, 51932, 10560, 64004,
                4980,  57120, 43356, 16428, 5648,  35680, 45104, 61364, 19572,
                3592,  60204, 33516, 9464,  53576, 35536, 63712, 55440, 25540,
                48768, 6940,  42048, 5164,  33048, 18708, 10152, 43248, 48508,
                1560,  58740, 32188, 15868, 53160, 30588, 42644, 25776, 53768,
                46676, 23012, 36748, 49020, 14484, 5152,  58928, 26392, 60752,
                40584, 6172,  63920, 24736, 28292, 62352, 3712,  12064, 41252,
                64140, 2596,  37244, 42320, 60104, 8580,  39252, 65368, 51140,
                35252, 18132, 8776,  31452, 55436, 39996, 61904, 34460, 1732,
                41172, 23956,
            },
            {
                10796, 33832, 25996, 41416, 992,   56116, 42744, 15532, 52800,
                7820,  31124, 236,   36900, 28232, 4852,  45064, 49072, 23976,
                5784,  39712, 64304, 15056, 34416, 28176, 52004, 13704, 37672,
                58720, 54824, 39608, 19760, 52920, 36964, 27208, 16912, 10440,
                39476, 4320,  49900, 15368, 47240, 35212, 13280, 53436, 7340,
                32196, 61600, 9844,  38216, 17716, 7024,  56012, 39004, 24896,
                35040, 20440, 8564,  30416, 65460, 24208, 13752, 53804, 48684,
                38956, 26448, 50012, 41304, 59264, 13840, 42552, 1588,  33660,
                62420, 20288, 59024, 50360, 23488, 39756, 53260, 64612, 24120,
                19920, 10820, 62072, 37324, 20872, 1864,  65200, 14508, 32708,
                11220, 56840, 18128, 8048,  24904, 54280, 18636, 2984,  37436,
                22424, 34060, 48828, 11460, 41968, 7012,  51312, 57256, 47760,
                27488, 7324,  58220, 25296, 14200, 46996, 20160, 4024,  13188,
                27960, 41056, 63456, 23636, 48804, 10160, 27800, 52352, 7968,
                63620, 46296,
            },
            {
                4672,  51300, 13748, 17932, 8712,  51668, 24220, 2968,  27392,
                17564, 57536, 49924, 63860, 15316, 54812, 61468, 17168, 34716,
                14196, 50692, 3912,  20132, 54708, 1316,  9136,  65120, 30476,
                44712, 16172, 11916, 23352, 59456, 33216, 4680,  65492, 30960,
                45580, 18652, 56824, 9064,  26456, 5408,  29616, 22092, 42172,
                47612, 24752, 35300, 53748, 42832, 12760, 31488, 19124, 2100,
                50984, 45256, 59480, 38224, 55604, 18360, 41248, 30836, 9888,
                22696, 55856, 5004,  18404, 29852, 7416,  47788, 17272, 39124,
                12752, 44740, 16132, 30196, 2756,  13564, 59684, 6964,  36096,
                28236, 41700, 46944, 24564, 56332, 48960, 18504, 38400, 44236,
                916,   61240, 40836, 45620, 63168, 35320, 42356, 46624, 7864,
                56520, 29696, 18840, 58968, 36684, 20536, 33556, 17260, 38284,
                22624, 55216, 45160, 18268, 1572,  53800, 32932, 23224, 59436,
                49092, 5524,  14364, 53232, 324,   18768, 59076, 44776, 14852,
                30336, 19420,
            },
            {
                27992, 43640, 58308, 65344, 36812, 33196, 62076, 38412, 59604,
                34652, 43444, 11540, 25544, 39840, 33840, 3008,  27012, 42848,
                57212, 28780, 46796, 61060, 38112, 58004, 43120, 22608, 49364,
                5604,  35496, 62476, 48452, 2588,  46300, 41260, 54360, 7088,
                24900, 59496, 37184, 42964, 52368, 39968, 57428, 10896, 37000,
                59640, 3640,  28900, 63924, 732,   45948, 27632, 61908, 41844,
                14620, 26420, 11928, 20,    47132, 27528, 3204,  63188, 36108,
                15236, 46428, 12048, 64656, 37032, 25280, 52580, 22544, 57580,
                27536, 53556, 9152,  63312, 35592, 46044, 17596, 31852, 56920,
                15272, 52764, 3068,  6444,  34892, 27904, 7228,  58176, 51116,
                21148, 29528, 13052, 32960, 2276,  21636, 28636, 64496, 12140,
                51948, 15148, 1500,  54416, 9244,  47048, 26276, 972,   62884,
                31616, 10192, 50512, 29780, 35788, 61772, 41732, 10060, 55072,
                30596, 21120, 58084, 33272, 43004, 36268, 6040,  22572, 38048,
                55772, 60648,
            },
            {
                35760, 22140, 5936,  28784, 48936, 21092, 12516, 5404,  47204,
                22028, 3520,  53416, 18908, 8024,  22208, 51560, 11200, 38444,
                576,   22120, 10444, 31424, 17232, 25084, 12200, 33572, 18764,
                1928,  56152, 25424, 31280, 10028, 18116, 22408, 13224, 49636,
                34080, 412,   12444, 64024, 21016, 17032, 62292, 1200,  50604,
                15256, 19020, 8272,  50016, 20984, 34096, 58404, 48580, 7444,
                54724, 63388, 32768, 52084, 21232, 8916,  50532, 57752, 5972,
                43340, 60600, 32424, 21504, 3752,  45348, 61236, 10652, 5496,
                36360, 16,    41116, 25188, 56020, 51568, 26860, 1180,  44652,
                9176,  63940, 30132, 13336, 60308, 45116, 10508, 23752, 35812,
                6248,  54912, 25696, 50120, 59852, 10244, 15932, 49692, 32464,
                25628, 61168, 39452, 44532, 31108, 61792, 52720, 13852, 44148,
                4528,  16316, 40088, 64600, 5644,  27336, 15296, 46208, 37204,
                1896,  44864, 38848, 25268, 12020, 47280, 64900, 32108, 11388,
                49328, 748,
            },
            {
                53356, 39872, 10452, 45740, 1776,  57448, 43792, 30896, 54772,
                14376, 29296, 60476, 42072, 48304, 58180, 44424, 65056, 19448,
                60300, 53772, 36648, 44296, 5868,  50444, 39932, 61636, 53300,
                28396, 42112, 7200,  38768, 52252, 63128, 29432, 36324, 61144,
                20344, 47988, 28424, 32060, 3196,  46856, 34252, 26020, 31044,
                56644, 45092, 39036, 55552, 14428, 23768, 3984,  11104, 36372,
                29572, 17872, 4880,  37136, 60844, 44292, 34408, 17468, 25144,
                28740, 584,   38480, 50376, 54972, 15500, 40020, 30696, 49784,
                65040, 32740, 48368, 20964, 6260,  11056, 42720, 22284, 49504,
                36856, 19620, 39748, 51048, 17208, 41152, 61924, 31176, 16032,
                64132, 42712, 17332, 4548,  37836, 56652, 40052, 12,    20672,
                42836, 35392, 4112,  23192, 16524, 5724,  40608, 23632, 60212,
                54152, 34564, 21576, 12236, 52596, 48676, 19232, 6880,  25940,
                64408, 17408, 7692,  61204, 29208, 16320, 54164, 3488,  41900,
                24936, 16656,
            },
            {
                63416, 14172, 55540, 32340, 16828, 26628, 8284,  18696, 40084,
                64824, 6348,  37508, 32568, 1912,  13688, 29008, 5992,  31776,
                47856, 7996,  13784, 63736, 27592, 2364,  56708, 7840,  15072,
                46576, 64136, 20204, 14072, 58012, 43148, 1508,  56608, 9376,
                40120, 16016, 55508, 7868,  23872, 54668, 13800, 43928, 5928,
                65220, 24092, 11844, 30660, 60364, 41476, 52480, 16372, 40008,
                22652, 48056, 42520, 24748, 15344, 29760, 12144, 39740, 54184,
                48072, 19092, 58272, 10040, 26812, 34476, 2008,  19932, 43352,
                23744, 17484, 13108, 60304, 38908, 34332, 58700, 62636, 4472,
                54516, 25180, 57624, 33260, 21724, 252,   53540, 47488, 3304,
                39092, 9808,  46140, 52608, 30548, 23256, 44936, 62224, 6284,
                56024, 10000, 50600, 65132, 28048, 57828, 11552, 35852, 28964,
                7896,  48132, 58764, 8,     23892, 38040, 60984, 56396, 34484,
                51624, 11116, 48424, 56220, 4292,  50788, 20592, 27296, 57632,
                46380, 8224,
            },
            {
                29488, 50084, 24612, 61640, 41772, 52700, 63144, 49452, 23852,
                10180, 50864, 26280, 16492, 62332, 24176, 36844, 55612, 15412,
                25980, 41504, 23616, 58528, 34392, 47580, 21492, 30904, 37496,
                23684, 11028, 50620, 34512, 26580, 6148,  16704, 51264, 26916,
                4076,  43580, 59788, 50696, 41364, 37572, 60928, 9356,  20356,
                40796, 35152, 1848,  47972, 5596,  35784, 26356, 64636, 55456,
                2772,  61388, 9668,  52984, 56252, 1960,  63928, 22356, 8020,
                61756, 13544, 42588, 30452, 6996,  63004, 56320, 12512, 59448,
                7936,  55348, 45676, 29948, 2516,  19260, 14076, 28364, 32928,
                16636, 11468, 5224,  46500, 8404,  37552, 25588, 12980, 56880,
                33476, 27524, 58064, 19804, 12676, 7804,  33912, 53228, 17680,
                29412, 45972, 13072, 19272, 38108, 48040, 2108,  51000, 42516,
                18880, 25960, 45280, 32352, 42152, 8860,  31048, 2896,  13936,
                42308, 20092, 31752, 23084, 40688, 35080, 9596,  39000, 62300,
                18700, 33328,
            },
            {
                44512, 2644,  19380, 35360, 4756,  13460, 36908, 3084,  32120,
                44404, 57960, 20528, 46020, 52376, 9388,  40804, 49696, 1616,
                35408, 51692, 3776,  19104, 53124, 15912, 11484, 45284, 59172,
                5144,  32452, 60032, 2788,  45164, 37960, 23664, 32968, 47012,
                64804, 22116, 35416, 11316, 18320, 4192,  28884, 49236, 52640,
                15996, 54520, 27424, 62468, 17848, 43780, 9220,  20284, 32328,
                44684, 13584, 34856, 19664, 38540, 5796,  46352, 51320, 32180,
                3424,  35340, 23816, 52976, 17252, 41332, 47440, 37340, 28564,
                51192, 3532,  25796, 63992, 52388, 41812, 48992, 8620,  40076,
                47804, 63264, 43232, 27452, 58808, 64884, 29796, 18784, 50548,
                22464, 62824, 1576,  36004, 60720, 49172, 26944, 14180, 37564,
                24240, 60124, 32088, 6976,  55672, 33596, 21424, 15524, 64396,
                3468,  55376, 14148, 62828, 17452, 53432, 44728, 21956, 28620,
                58724, 37752, 892,   63544, 14412, 44972, 59700, 1788,  13504,
                36532, 5456,
            },
            {
                22704, 38672, 59660, 9436,  47780, 21980, 28440, 59548, 16224,
                35140, 1080,  11992, 56452, 3944,  33496, 21024, 59332, 17988,
                63032, 45848, 32620, 9412,  38588, 42412, 64932, 26304, 1492,
                52752, 40356, 17764, 48232, 20952, 55528, 61964, 10540, 38808,
                14324, 30776, 5732,  25792, 63768, 33248, 58196, 22908, 2636,
                31960, 46080, 10292, 21812, 51336, 57540, 1484,  28728, 50652,
                6932,  59036, 27688, 49472, 31468, 58576, 25948, 41780, 16216,
                56104, 45044, 65168, 1380,  49444, 21416, 25248, 4700,  16488,
                40464, 33544, 10212, 36728, 15908, 23140, 56632, 552,   59888,
                21040, 29996, 2048,  51604, 15768, 11088, 41556, 4780,  44188,
                8004,  14740, 40296, 24964, 5028,  41576, 64856, 3252,  57136,
                48248, 1728,  40144, 52428, 25028, 44860, 9044,  59048, 31652,
                38716, 10684, 35076, 4240,  27764, 57284, 11752, 49812, 62972,
                5840,  53848, 46640, 27112, 7288,  54728, 24268, 30580, 52680,
                49064, 56576,
            },
            {
                63968, 12768, 51792, 30464, 57928, 40976, 55320, 46220, 7048,
                62488, 39020, 30028, 42444, 27236, 63900, 12776, 44212, 6696,
                28156, 11712, 60748, 55176, 29408, 6064,  50992, 33644, 19436,
                56372, 28248, 8104,  63632, 11892, 29096, 6856,  52668, 19732,
                1164,  57820, 48756, 54040, 45460, 15116, 42612, 12104, 38636,
                62960, 7532,  59328, 37832, 13412, 32888, 47036, 62804, 15428,
                39172, 23896, 1364,  64944, 8184,  12988, 20844, 36744, 9956,
                28220, 20272, 39024, 11864, 33264, 59968, 9208,  31332, 54440,
                61476, 21280, 44544, 57404, 7240,  32020, 45088, 25664, 35740,
                52844, 13712, 38156, 23148, 34916, 48584, 56028, 36288, 60872,
                31020, 47592, 51652, 55280, 18244, 45912, 21168, 32272, 8356,
                42508, 21932, 63596, 15188, 4444,  61628, 41012, 27128, 52776,
                23120, 49540, 59664, 40696, 20468, 6360,  35804, 39804, 16436,
                24324, 10112, 34792, 18572, 49592, 33236, 16548, 43144, 20928,
                10420, 27684,
            },
            {
                17348, 42756, 6700,  26080, 15856, 352,   10936, 19848, 26384,
                49840, 22712, 53968, 18528, 6192,  48436, 38164, 30752, 52944,
                22576, 39860, 15024, 20932, 480,   59428, 23164, 9140,  38124,
                44420, 14576, 24692, 42296, 31896, 50044, 15576, 41000, 60412,
                44588, 28004, 37476, 8428,  21272, 504,   48240, 26764, 55720,
                17572, 24288, 43060, 29404, 6372,  40680, 23552, 11380, 36044,
                56684, 18336, 46828, 41240, 16792, 44396, 62444, 52616, 4564,
                49036, 60556, 6004,  26732, 57272, 43204, 14532, 64424, 46396,
                692,   12412, 49580, 28276, 3920,  62700, 54140, 10980, 17700,
                6088,  42052, 61668, 54848, 19520, 6992,  24532, 1152,  53240,
                20492, 11628, 34620, 28568, 9644,  58344, 38492, 11952, 52808,
                16840, 28648, 10388, 36772, 30576, 18236, 12296, 712,   46552,
                7184,  16904, 29768, 46108, 51588, 65288, 25624, 1224,  47932,
                31160, 43624, 62392, 12420, 57696, 2824,  65192, 6280,  60188,
                40412, 868,
            },
            {
                31824, 54592, 34568, 45616, 64992, 38836, 31560, 53404, 42844,
                14132, 8628,  59768, 36244, 15156, 58200, 24556, 9076,  56404,
                2620,  49952, 43588, 25728, 48444, 36164, 17176, 62748, 12524,
                48612, 60904, 35564, 212,   58140, 36720, 3368,  25660, 33896,
                55764, 12472, 17496, 62684, 31376, 39980, 60340, 6160,  35624,
                50704, 3440,  34144, 56496, 19304, 53796, 60084, 4776,  42740,
                30488, 53200, 10676, 34044, 55260, 28544, 548,   33176, 57956,
                13424, 31772, 46948, 16916, 37048, 2700,  50584, 35420, 18952,
                38660, 24952, 59256, 40916, 19540, 13552, 38376, 29356, 65352,
                46472, 32128, 9804,  2940,  45964, 32876, 63200, 17120, 27384,
                43164, 5832,  63536, 16364, 500,   30244, 50340, 25520, 62632,
                35400, 59272, 49484, 45564, 57288, 50544, 34616, 54752, 19944,
                37084, 62260, 1980,  24032, 9756,  14664, 32752, 60644, 19668,
                57048, 4168,  38516, 22568, 41344, 27984, 37584, 46264, 25052,
                34936, 47504,
            },
            {
                61340, 24004, 3340,  18944, 50288, 22840, 60332, 5208,  34168,
                64432, 2356,  46416, 31812, 51628, 264,   42076, 16640, 65484,
                36492, 31432, 61836, 7856,  56600, 30424, 45816, 40968, 2484,
                30880, 5608,  54476, 17364, 45844, 21840, 64188, 47676, 9300,
                23236, 2468,  41836, 50784, 24568, 10964, 53316, 20084, 29960,
                45144, 14276, 63576, 48620, 128,   16244, 27284, 49720, 21320,
                3292,  62156, 26680, 5672,  22248, 48348, 38236, 24620, 18040,
                43268, 22668, 54828, 8828,  52584, 29508, 22936, 7116,  27540,
                52936, 5956,  16952, 34700, 51356, 43504, 21948, 48664, 3684,
                24196, 56808, 27320, 15220, 59060, 29172, 40572, 8880,  58536,
                37608, 23580, 56908, 44952, 36336, 61432, 14424, 6364,  44016,
                1188,  19736, 7352,  26136, 2776,  22584, 63752, 29156, 43520,
                56352, 13212, 33764, 58276, 38344, 44372, 55032, 41568, 8736,
                52184, 15472, 29392, 53552, 8984,  19592, 51416, 11624, 15336,
                57200, 8560,
            },
            {
                49628, 14500, 37720, 57096, 8476,  13360, 36424, 48160, 17996,
                28816, 40468, 25396, 11428, 63216, 21884, 34796, 47340, 27084,
                5160,  11048, 18676, 38984, 12932, 3576,  53932, 27064, 57568,
                23936, 20252, 51052, 10804, 26832, 4824,  54892, 18156, 30984,
                51928, 61580, 34552, 5480,  56860, 36764, 16296, 65024, 9344,
                57600, 21544, 11064, 25176, 39320, 45648, 65488, 34236, 9420,
                44528, 14868, 50540, 40300, 63580, 11808, 56580, 7924,  61408,
                39492, 1900,  63812, 41148, 20168, 62084, 45828, 11224, 56436,
                42416, 63636, 30952, 9396,  60244, 1320,  33144, 8856,  58884,
                39508, 20088, 50184, 43032, 36936, 12124, 51284, 46816, 14320,
                32144, 49432, 3544,  19964, 54480, 41940, 22344, 47576, 56256,
                33304, 40544, 54316, 14080, 42784, 37968, 5692,  15452, 8728,
                25000, 41532, 4828,  48912, 18400, 29064, 3624,  12688, 27188,
                36180, 45276, 59532, 424,   47356, 61488, 31404, 3540,  53444,
                28420, 20216,
            },
            {
                41308, 10884, 62860, 30024, 43736, 26756, 1668,  56236, 9792,
                52472, 57676, 20188, 43796, 4200,  28480, 54404, 12476, 20268,
                51372, 59008, 46368, 32976, 64412, 22164, 15604, 7180,  50208,
                37716, 64668, 32892, 39688, 61544, 42792, 12832, 38248, 6384,
                44032, 14792, 20340, 47632, 29012, 3392,  46608, 33004, 1612,
                41212, 37408, 28880, 52872, 8268,  30676, 12924, 38120, 57440,
                24652, 59936, 36280, 19164, 2656,  30796, 15744, 45184, 51016,
                29436, 10920, 25876, 34128, 4428,  13616, 59148, 32804, 37284,
                15196, 2924,  47764, 23424, 54616, 26872, 62204, 16552, 51768,
                12460, 34812, 7440,  64312, 23416, 5472,  20924, 54888, 4340,
                64580, 10832, 40660, 26568, 8260,  32784, 2600,  17612, 28172,
                11284, 24136, 65456, 30820, 58684, 10952, 48328, 59508, 51916,
                31952, 64620, 21516, 53384, 6904,  60120, 23068, 50728, 63984,
                20980, 7068,  25060, 33692, 13776, 42656, 23040, 36028, 64056,
                39072, 2120,
            },
            {
                59512, 45852, 25376, 5036,  51644, 40092, 63684, 30956, 24016,
                15340, 37788, 6536,  60744, 49940, 38904, 8132,  61284, 44452,
                40192, 23660, 1260,  27660, 52132, 42760, 35376, 60420, 18200,
                9608,  43564, 14396, 6760,  29868, 52304, 34348, 60096, 27812,
                58056, 39632, 26012, 64092, 13160, 42840, 23228, 52340, 26424,
                49656, 60960, 4244,  43476, 58516, 20904, 5492,  54552, 18068,
                1868,  32224, 7640,  46012, 59160, 53636, 34320, 20544, 3796,
                36300, 14928, 58744, 49852, 43960, 38576, 25072, 1532,  49280,
                21108, 58440, 44088, 11944, 39448, 5740,  36924, 44696, 30144,
                2204,  54672, 17552, 46348, 436,   61896, 33892, 26284, 38892,
                18756, 29420, 52968, 60812, 13012, 50952, 63492, 37892, 59628,
                51980, 3840,  16376, 46896, 33852, 21196, 26800, 39728, 17864,
                1604,  47060, 27368, 36916, 15168, 31636, 46212, 34680, 1852,
                40420, 48780, 62252, 18284, 54572, 5860,  58168, 16208, 44468,
                6428,  31056,
            },
            {
                53816, 17244, 33992, 21404, 55196, 14948, 19924, 45792, 41756,
                3324,  47796, 34964, 13276, 16940, 32852, 57100, 2092,  30300,
                14748, 34424, 55040, 17288, 4380,  48144, 10968, 31956, 45440,
                28588, 1268,  59596, 47424, 22824, 16192, 932,   20664, 49480,
                10788, 2980,  53720, 8452,  59124, 18832, 39324, 62016, 14704,
                7304,  18452, 34932, 15764, 62792, 33076, 50756, 26652, 47072,
                39960, 53088, 28032, 13120, 23464, 42276, 5396,  27308, 65036,
                54380, 46464, 19040, 31084, 7752,  16572, 54788, 65292, 9212,
                29936, 26168, 35568, 19196, 50264, 14496, 56464, 20656, 25488,
                41444, 60732, 31768, 26820, 53220, 41776, 15796, 44836, 60440,
                1084,  47840, 16092, 34940, 45500, 23724, 29944, 9272,  43200,
                20988, 48852, 38800, 8548,  464,   53644, 63196, 4344,  44112,
                35836, 12456, 9508,  56588, 42684, 63320, 9084,  17540, 57636,
                14480, 31016, 10048, 37172, 40060, 27700, 48556, 10188, 26048,
                50556, 22448,
            },
            {
                36484, 7652,  49052, 2720,  60668, 10692, 34744, 7016,  61680,
                58696, 21444, 29584, 55028, 26552, 46644, 19148, 24728, 52728,
                6816,  62652, 9856,  58480, 40448, 24644, 62964, 5116,  56940,
                22036, 53972, 36140, 25580, 57232, 9148,  65496, 46016, 24144,
                36152, 17216, 33052, 44784, 30672, 5892,  55768, 11416, 31480,
                44964, 54092, 23908, 47500, 11472, 1064,  42380, 15288, 61948,
                10768, 21472, 64676, 49908, 37064, 9912,  47640, 57936, 40260,
                8636,  22880, 40,    61684, 51816, 28664, 46744, 18740, 40148,
                52484, 7028,  60736, 3808,  63412, 32076, 46696, 10024, 63956,
                6120,  48308, 14068, 38176, 10716, 29804, 56980, 7008,  23748,
                36328, 58112, 21692, 6232,  3160,  40272, 57724, 15268, 4956,
                35144, 62524, 25712, 57012, 19044, 36516, 13136, 30340, 22312,
                57772, 61328, 19580, 50048, 204,   25336, 39244, 53844, 28532,
                43536, 52324, 4084,  56796, 21312, 1976,  63148, 34532, 18928,
                56336, 12452,
            },
            {
                27980, 65392, 42824, 30528, 37648, 47336, 28244, 53752, 12528,
                32292, 50712, 8928,  62432, 4784,  42200, 10120, 64748, 37368,
                42880, 21364, 49620, 36568, 13884, 29876, 19128, 38084, 49104,
                12652, 40492, 17072, 4120,  50252, 41876, 38704, 31820, 4964,
                56120, 63080, 50880, 22444, 37600, 50008, 460,   35672, 21236,
                59636, 2368,  27792, 39856, 55824, 22276, 36888, 58184, 4196,
                35132, 43840, 16220, 452,   30252, 61440, 17060, 25012, 13004,
                32160, 43084, 56428, 36676, 12044, 23412, 4072,  31972, 13248,
                56912, 45540, 15832, 41168, 28492, 23888, 920,   52732, 35596,
                18712, 57292, 22860, 8000,  59564, 19412, 49068, 12960, 51640,
                9252,  43184, 31024, 55168, 64960, 19136, 47672, 27224, 54896,
                31780, 12384, 45036, 29384, 41028, 60704, 51084, 45944, 7124,
                52476, 28864, 38856, 33352, 6084,  20912, 48064, 11676, 5324,
                22748, 64964, 26148, 12696, 50484, 31252, 45268, 7524,  41584,
                61676, 176,
            },
            {
                39988, 15884, 9760,  58408, 18232, 24212, 632,   38736, 17548,
                25716, 2392,  44948, 39364, 23364, 58376, 35052, 16020, 47824,
                3268,  26244, 31572, 44940, 7580,  52516, 55844, 1924,  26492,
                61312, 8232,  34028, 62888, 30380, 12028, 18972, 53340, 14088,
                42264, 28500, 1948,  11988, 15516, 27164, 63680, 24668, 48716,
                42116, 9980,  64848, 32124, 6912,  49564, 29452, 9032,  24348,
                31192, 56196, 6180,  41600, 54780, 19844, 51676, 2828,  35276,
                49376, 16360, 27468, 6540,  41272, 58052, 62660, 43548, 36868,
                688,   23116, 32996, 54296, 8420,  57648, 42780, 13128, 29248,
                39940, 3380,  45124, 51940, 34688, 2196,  40072, 32632, 28224,
                62852, 14044, 39464, 26176, 11420, 36596, 51780, 1792,  61356,
                18660, 6224,  53336, 2492,  15512, 24088, 9404,  33240, 16796,
                24712, 3820,  14636, 54632, 62116, 30080, 58784, 35396, 60804,
                18836, 33212, 46308, 42220, 16764, 59884, 23628, 52424, 14716,
                31992, 46648,
            },
            {
                20900, 55336, 25968, 51072, 5896,  63472, 56724, 49372, 43616,
                64796, 36016, 19356, 53364, 12900, 672,   50956, 27564, 56380,
                13044, 60940, 18204, 32,    64064, 21816, 42012, 16084, 32280,
                43720, 52152, 20428, 45380, 55956, 2500,  26956, 61516, 7860,
                46912, 19856, 40004, 61108, 53092, 45900, 40676, 8296,  16648,
                56852, 36816, 14100, 19852, 60232, 17068, 45292, 64156, 51860,
                19132, 46452, 60264, 26164, 34024, 7888,  38888, 44464, 62380,
                5232,  52792, 64340, 19912, 48468, 34016, 10168, 21680, 49212,
                27620, 63972, 10972, 47604, 20888, 37440, 17404, 60380, 50036,
                24544, 32528, 16448, 61272, 42596, 25228, 65096, 4952,  55424,
                19908, 2460,  50404, 46440, 7072,  32232, 22640, 13528, 41360,
                37764, 50332, 22156, 64736, 43592, 4500,  48752, 55796, 40264,
                63952, 43044, 48460, 10824, 45080, 16652, 42020, 2300,  49796,
                38564, 8764,  660,   57792, 35572, 4532,  11432, 28352, 37568,
                6196,  59404,
            },
            {
                3428,  34720, 13168, 45320, 32908, 41452, 14360, 22132, 4388,
                10324, 57328, 15396, 28984, 63548, 31748, 20920, 8032,  39540,
                33332, 53204, 10672, 58848, 27168, 34228, 46756, 9532,  57424,
                23720, 5988,  28200, 14728, 37712, 22580, 48924, 35180, 58572,
                25104, 10116, 57156, 34456, 6324,  19224, 59204, 33340, 3464,
                29252, 50716, 5104,  52848, 41704, 26236, 12588, 39288, 1436,
                14120, 37320, 11180, 22220, 62868, 14588, 27176, 58736, 20684,
                10720, 30040, 39080, 13944, 25868, 2652,  54344, 16240, 5356,
                58492, 18308, 39248, 51552, 3224,  62976, 33392, 4764,  9160,
                55276, 64364, 11356, 28472, 6692,  21420, 47248, 16688, 45016,
                34436, 23952, 59360, 15720, 61992, 43484, 56924, 8748,  46320,
                24632, 59236, 10828, 30976, 35748, 58624, 27172, 12252, 824,
                35332, 58328, 21452, 26576, 36552, 8176,  23984, 13604, 27092,
                55344, 15536, 51812, 28976, 20384, 39236, 64576, 54236, 19432,
                49588, 24256,
            },
            {
                63852, 52576, 38844, 2080,  20348, 9472,  29512, 35344, 52712,
                27132, 40672, 48708, 6768,  37548, 47436, 43988, 59880, 4044,
                23248, 45940, 40964, 36960, 50268, 3964,  13176, 62020, 36020,
                3076,  64616, 47288, 10600, 59912, 5564,  43364, 15864, 36,
                38136, 31692, 48516, 3644,  23756, 30408, 11044, 51536, 44472,
                20696, 25312, 46100, 34500, 1996,  54920, 6272,  33748, 57564,
                27784, 49344, 53472, 4312,  40732, 48720, 1460,  31660, 54188,
                23652, 46820, 1124,  56952, 44732, 30744, 61320, 40440, 52320,
                30172, 35288, 6504,  25608, 15176, 43924, 27408, 47236, 21292,
                41764, 1332,  37660, 49388, 54160, 13356, 57968, 38652, 10808,
                52208, 8116,  37400, 28700, 1168,  18028, 35296, 64216, 29276,
                16280, 34148, 7512,  47884, 17720, 39056, 20968, 61576, 29544,
                18584, 7760,  31996, 1764,  65408, 52856, 57300, 45868, 31292,
                63532, 41324, 24248, 61724, 7276,  46900, 43016, 1448,  57052,
                10492, 44140,
            },
            {
                27328, 7824,  30044, 62172, 54676, 47088, 61244, 7220,  59280,
                31988, 20708, 1884,  60596, 24796, 10924, 17592, 51900, 30260,
                64368, 19896, 4744,  15712, 23604, 55532, 30072, 19940, 48872,
                38916, 17828, 31000, 40960, 52676, 32820, 64212, 29048, 51736,
                21436, 63364, 17048, 44192, 53952, 65228, 14800, 38496, 58008,
                62124, 11908, 38056, 63568, 22808, 30944, 48256, 61796, 20876,
                42916, 8168,  32036, 18024, 45392, 57116, 12484, 37344, 17156,
                41828, 59484, 34852, 7972,  51008, 18096, 37352, 24844, 8844,
                14004, 45244, 61188, 56368, 31136, 11640, 59068, 53516, 35744,
                13992, 26572, 45456, 18792, 31352, 36412, 92,    27160, 60840,
                30624, 42576, 56016, 21388, 48996, 53820, 25632, 4596,  52416,
                188,   42996, 56004, 25432, 13492, 54456, 6320,  44152, 52032,
                46844, 13548, 55388, 49948, 39648, 18884, 34156, 6136,  21012,
                3104,  10568, 34824, 48832, 12560, 32572, 15348, 25388, 30848,
                36208, 16152,
            },
            {
                58304, 18508, 48764, 23360, 15424, 36840, 25152, 43976, 17228,
                12496, 45400, 51572, 33624, 42592, 55488, 2692,  35936, 14656,
                8932,  48652, 28332, 57320, 44260, 7728,  40000, 53812, 26892,
                11984, 58384, 54664, 1244,  24888, 8632,  18580, 11584, 45664,
                55416, 7208,  12636, 27880, 35268, 42304, 7112,  26416, 764,
                32264, 7660,  15724, 58360, 18288, 10260, 44200, 15144, 24676,
                2880,  59056, 63524, 35892, 23940, 28764, 64980, 8588,  50896,
                4100,  26512, 15688, 63424, 22984, 11796, 4448,  47568, 64532,
                42216, 2372,  22272, 49336, 40764, 300,   23448, 7092,  18184,
                62364, 58356, 5572,  56188, 9232,  63356, 50524, 22832, 4716,
                18416, 63904, 3888,  12828, 41032, 30768, 10940, 39404, 48192,
                21224, 60248, 38364, 4152,  62944, 49676, 32668, 10136, 23656,
                37844, 60484, 41928, 22848, 4308,  29284, 12172, 61532, 48152,
                37336, 53128, 18016, 5120,  59032, 21564, 51532, 62832, 3860,
                47792, 42060,
            },
            {
                4580,  34088, 40776, 11324, 57512, 4888,  50792, 1212,  38464,
                55144, 62312, 8816,  22760, 13960, 28388, 63328, 40712, 25864,
                54324, 34516, 61416, 11924, 32092, 65420, 16908, 788,   45408,
                7032,  34284, 21344, 15668, 49568, 36172, 57112, 39156, 3356,
                26372, 41428, 50796, 60588, 1624,  19840, 48876, 55504, 17376,
                42640, 54180, 49264, 4224,  41192, 27288, 56532, 35600, 52440,
                40108, 29632, 15968, 9952,  3256,  55012, 19528, 33868, 44224,
                62144, 10320, 38308, 31888, 42700, 55640, 59868, 20136, 33116,
                27276, 54072, 10304, 18788, 34272, 65248, 45696, 38372, 30056,
                48576, 33180, 22484, 40372, 25548, 16000, 33408, 44268, 39780,
                14452, 47532, 24868, 33228, 61560, 6900,  57832, 15636, 62508,
                12168, 32304, 18300, 45168, 29912, 1828,  41652, 15792, 64584,
                2744,  27588, 17028, 11036, 59580, 44524, 51592, 16284, 42488,
                58212, 26344, 30400, 41856, 55480, 27796, 38484, 7756,  59836,
                22348, 12724,
            },
            {
                53944, 63212, 516,   44912, 28212, 32712, 19608, 65144, 29192,
                23924, 4124,  36452, 18388, 58612, 5368,  49032, 19560, 57756,
                43636, 6592,  38828, 2316,  51380, 21392, 35552, 60208, 24388,
                50832, 62376, 42624, 27868, 60496, 44700, 22076, 30568, 62328,
                14820, 32880, 37080, 22636, 9884,  58644, 29124, 36628, 22000,
                47008, 24404, 28512, 33508, 51936, 62460, 792,   7468,  19332,
                11852, 55412, 47536, 43672, 51456, 39196, 6340,  47832, 22452,
                29648, 54536, 19876, 48320, 6492,  28392, 35632, 900,   50684,
                15960, 59028, 37748, 7596,  27924, 55296, 15780, 52136, 10128,
                3904,  43028, 12316, 51428, 2872,  46992, 57612, 7100,  59840,
                53388, 35708, 9780,  51028, 43648, 19984, 36580, 44832, 23980,
                5584,  27680, 53464, 9460,  22176, 59860, 26360, 36216, 56584,
                5500,  34260, 48592, 31516, 7540,  24468, 36072, 848,   22252,
                8108,  13684, 64936, 45572, 1008,  35472, 10976, 18876, 40044,
                33092, 50800,
            },
            {
                25328, 20456, 37628, 60996, 8352,  53152, 42360, 13908, 9564,
                49420, 41288, 30656, 53860, 46288, 38676, 10348, 31392, 1068,
                13096, 24356, 18892, 42548, 26264, 47740, 10096, 41468, 14960,
                29792, 3772,  38168, 7908,  13268, 2568,  53172, 6612,  48732,
                57808, 18924, 4524,  54468, 44996, 39816, 13220, 62840, 5032,
                10784, 64704, 39416, 9000,  20532, 13812, 37872, 50280, 65508,
                33404, 5552,  25724, 21048, 60636, 13304, 26612, 58672, 1240,
                14408, 40892, 58144, 2476,  52724, 16864, 45808, 39788, 12212,
                23320, 31316, 47484, 62640, 13368, 21200, 4924,  42044, 24840,
                54880, 17012, 64760, 59184, 29892, 37908, 19600, 11100, 28796,
                21160, 1020,  27548, 58476, 16892, 1840,  28344, 55696, 34264,
                50324, 41516, 64868, 35560, 14156, 51404, 18704, 46136, 53044,
                20852, 43332, 62720, 53740, 38700, 56184, 64240, 28084, 32672,
                54432, 38628, 4220,  23356, 16324, 62280, 49704, 44888, 55112,
                28464, 9320,
            },
            {
                46408, 31212, 13540, 49096, 17044, 22772, 58852, 46864, 34536,
                56848, 15384, 63832, 484,   25124, 34020, 61672, 22200, 45356,
                62864, 52948, 31004, 59732, 14436, 4996,  57184, 32952, 53344,
                19428, 11216, 55428, 46712, 65092, 33512, 17096, 42448, 24352,
                10592, 46660, 64472, 26084, 16004, 32100, 2336,  52092, 33872,
                56672, 14984, 2668,  60040, 44764, 23600, 31440, 46316, 27896,
                42376, 59528, 38096, 268,   34580, 30620, 42140, 16980, 51912,
                35684, 7656,  24336, 13272, 65512, 25668, 8700,  56832, 63344,
                5316,  44348, 3188,  41244, 50848, 35964, 58148, 32684, 61648,
                28408, 1540,  35356, 20824, 6528,  13896, 42396, 52012, 32656,
                65348, 46164, 55088, 38088, 23196, 63692, 47140, 4360,  9988,
                58872, 16720, 3080,  47688, 57408, 39564, 6648,  11948, 30468,
                8752,  24984, 13032, 1432,  20036, 14660, 46436, 10376, 49856,
                60012, 19112, 47720, 33464, 52372, 6604,  25928, 14304, 2520,
                65244, 16716,
            },
            {
                59200, 6676,  56544, 26640, 35324, 2172,  39184, 5752,  26308,
                20076, 7168,  43724, 11648, 50400, 16416, 7736,  51972, 35112,
                17020, 47984, 9128,  54688, 36456, 62928, 28024, 2592,  44688,
                63776, 48976, 32024, 25684, 19992, 39936, 28360, 60108, 35192,
                664,   38528, 29776, 7236,  49460, 61276, 23688, 46192, 41088,
                19688, 30124, 36312, 48604, 55264, 3396,  57836, 16924, 9316,
                22244, 14336, 52056, 18256, 62556, 48984, 10156, 64248, 32492,
                44824, 61148, 46932, 37488, 33420, 42208, 21396, 29964, 52456,
                34896, 18172, 55804, 25076, 30160, 1356,  46940, 12032, 19312,
                49688, 39216, 45404, 26132, 48400, 63100, 56572, 24720, 3696,
                15608, 41692, 5736,  13200, 8716,  49880, 31648, 14584, 21984,
                38144, 26300, 8136,  30736, 23540, 1000,  33752, 62992, 41004,
                58844, 50608, 35692, 45520, 26932, 4136,  40336, 17672, 5940,
                43312, 25088, 11576, 56292, 40352, 21152, 30680, 57360, 37196,
                5460,  42420,
            },
            {
                22188, 39716, 3336,  43744, 54508, 62596, 11964, 31512, 60076,
                52532, 37904, 28328, 21440, 59164, 40180, 55936, 26948, 3948,
                38016, 28260, 1664,  22604, 45488, 17944, 39544, 22032, 8524,
                36632, 23564, 16072, 5984,  56448, 49972, 8780,  12640, 54196,
                20640, 51524, 13728, 42148, 57008, 18524, 9284,  27252, 6392,
                60980, 43836, 25196, 17780, 11900, 41372, 26024, 5228,  61632,
                48396, 3652,  29040, 56216, 7872,  23612, 4280,  55104, 25292,
                3312,  20392, 28812, 10100, 50148, 60576, 5880,  14516, 47976,
                26828, 11348, 61704, 7204,  16440, 59892, 23144, 37928, 6424,
                57028, 14592, 8128,  54272, 31764, 624,   34780, 9188,  49612,
                36716, 18848, 30872, 61016, 25612, 39884, 53996, 42528, 61940,
                51920, 46148, 19588, 55292, 43288, 60424, 27516, 49008, 15192,
                2572,  18108, 29644, 56960, 60696, 33540, 53248, 62344, 30472,
                35980, 2148,  63484, 28144, 9196,  60952, 43432, 48116, 19596,
                32608, 50248,
            },
            {
                35820, 15692, 64352, 10252, 18520, 29140, 41616, 48552, 16752,
                1652,  61592, 46032, 33132, 3228,  30032, 14140, 42440, 60184,
                11816, 64444, 41060, 33348, 12364, 6724,  52076, 56260, 14176,
                60372, 228,   43560, 59380, 36948, 1892,  45716, 62920, 26328,
                44332, 58924, 33120, 22192, 4008,  34956, 38912, 55364, 12372,
                50628, 408,   53560, 7564,  34376, 63072, 51484, 32744, 38760,
                54820, 35580, 44164, 11904, 40200, 46404, 36576, 19000, 12184,
                39420, 53428, 15728, 56244, 44,    18608, 38616, 59136, 2440,
                43244, 36884, 20748, 39328, 54212, 44556, 9484,  64388, 30532,
                43760, 23868, 62164, 11328, 40580, 17292, 22308, 44708, 61884,
                26468, 58760, 51600, 44044, 34828, 2260,  19160, 11336, 29304,
                636,   33028, 63184, 12508, 37224, 9972,  20448, 54608, 24264,
                38476, 64480, 5348,  41572, 9796,  23716, 48156, 20720, 12884,
                57336, 51060, 15456, 45360, 4624,  34440, 12440, 136,   24464,
                62092, 11256,
            },
            {
                55056, 29672, 51848, 33640, 24680, 50668, 4400,  21824, 35928,
                25372, 9968,  54884, 17808, 65072, 47312, 5600,  21072, 49308,
                24784, 7384,  55536, 50344, 58932, 25932, 31908, 48012, 29188,
                41276, 33928, 51104, 10680, 29976, 22868, 15104, 31816, 5188,
                16824, 2784,  62492, 10484, 53448, 47572, 65112, 29676, 16772,
                22084, 31980, 64164, 39680, 28204, 20880, 14876, 43920, 11292,
                18736, 24124, 64524, 32156, 15836, 57848, 28656, 60240, 50940,
                31416, 62732, 5484,  22708, 30740, 44612, 54648, 32828, 23700,
                9088,  57572, 49012, 31608, 13172, 27104, 50416, 17492, 53028,
                4216,  33604, 20004, 51344, 28052, 59948, 55132, 14552, 38996,
                7248,  11272, 22040, 4208,  15480, 64800, 57524, 36504, 7120,
                59784, 15260, 40604, 3996,  52896, 16880, 44884, 3688,  32860,
                47620, 11384, 21568, 51392, 16288, 6660,  37824, 468,   27068,
                41104, 22324, 31296, 37728, 54012, 17448, 58960, 39552, 52768,
                7876,  27056,
            },
            {
                1932,  47028, 20548, 7388,  45216, 58044, 14892, 64196, 43000,
                51080, 13480, 39620, 23776, 8408,  53072, 36724, 58172, 17304,
                44592, 30288, 14848, 20400, 3760,  42532, 64904, 10604, 18980,
                5048,  54700, 26496, 17968, 61348, 52672, 41816, 56788, 36048,
                40568, 48808, 28432, 37952, 25068, 14736, 1816,  42344, 58656,
                36976, 46584, 10148, 57208, 4328,  49200, 60388, 808,   29764,
                58512, 7420,  2124,  50040, 20616, 5220,  43316, 1176,  47292,
                8648,  41840, 35120, 48540, 63716, 25332, 11672, 17104, 64604,
                51468, 28732, 560,   62788, 4752,  33492, 40632, 1992,  26180,
                36592, 58284, 46256, 2400,  37192, 5328,  47464, 30372, 2532,
                53728, 33704, 48412, 56316, 29568, 45560, 23376, 26992, 49132,
                44480, 21276, 25260, 47916, 28820, 65400, 36088, 7940,  57700,
                28436, 42868, 55124, 36508, 31584, 63980, 43196, 54500, 59612,
                7776,  47588, 3416,  19972, 62248, 26400, 49744, 29816, 15304,
                57672, 42876,
            },
            {
                63192, 12856, 40784, 61044, 420,   38140, 9784,  54384, 7004,
                32056, 58252, 2564,  34784, 41832, 26668, 10712, 32392, 276,
                53360, 34920, 61968, 38664, 46656, 16580, 1352,  37612, 24972,
                62368, 46548, 13396, 38812, 3992,  47264, 20228, 7036,  64896,
                23220, 8504,  19060, 57852, 45420, 31860, 20792, 8412,  48860,
                3524,  24416, 14020, 19204, 45204, 35884, 8584,  22512, 52332,
                41092, 46752, 27572, 54168, 61780, 25908, 33568, 14108, 23072,
                17876, 26740, 57732, 13192, 40080, 7744,  36104, 46400, 3668,
                40280, 15124, 19168, 42604, 46824, 21572, 60272, 55748, 13656,
                41612, 9380,  16252, 65008, 24880, 42180, 12072, 20568, 64228,
                43220, 27816, 18180, 38580, 9352,  52464, 4892,  17284, 54680,
                38340, 9164,  56904, 34504, 6168,  21720, 50056, 60728, 14256,
                19240, 1032,  59992, 25972, 13676, 49380, 28992, 10076, 17088,
                33676, 65060, 52436, 11120, 41440, 7396,  22404, 3724,  45604,
                37404, 18368,
            },
            {
                6104,  34928, 26172, 55992, 30632, 22916, 34480, 26768, 47160,
                19832, 29312, 62132, 49028, 14440, 60612, 45504, 22964, 63036,
                40476, 4644,  10068, 24224, 29308, 60544, 34768, 51212, 57976,
                8076,  21400, 32480, 64224, 8908,  34576, 27568, 12040, 50320,
                30432, 53892, 43012, 12820, 5908,  52360, 60016, 26440, 54604,
                33804, 60428, 41768, 52888, 31152, 25384, 55972, 37888, 63732,
                16080, 34528, 13464, 37156, 8516,  39784, 52788, 65140, 37504,
                54640, 3976,  43732, 20780, 1760,  53280, 50232, 29668, 21896,
                61160, 35232, 25820, 56596, 11508, 36808, 7504,  18868, 48104,
                61080, 22892, 32152, 49396, 7516,  57036, 33924, 50336, 24184,
                15232, 59080, 1756,  62656, 13568, 41124, 34400, 63256, 12352,
                31300, 2856,  51340, 59472, 13664, 41064, 30448, 24592, 39296,
                52196, 34248, 8840,  46592, 2768,  58244, 19524, 24508, 45764,
                38776, 14340, 25344, 56836, 32184, 47000, 35308, 64744, 10124,
                31540, 23876,
            },
            {
                47952, 53508, 16544, 10868, 49988, 17896, 60160, 40348, 1376,
                16312, 43900, 4936,  21768, 55836, 18692, 3564,  51216, 13288,
                27120, 19484, 57180, 49808, 7268,  54148, 21936, 14680, 44656,
                40040, 3060,  50508, 42976, 23680, 58952, 54816, 39432, 2072,
                15408, 59876, 33656, 180,   63888, 35900, 17620, 39308, 11736,
                16108, 29056, 6508,  62520, 2716,  17108, 12860, 47868, 26844,
                3272,  21264, 60036, 43696, 18612, 11700, 48408, 6440,  29604,
                60564, 10268, 33168, 62064, 16788, 27072, 59800, 5332,  43596,
                55256, 9924,  6356,  49348, 65216, 24168, 52192, 31380, 28108,
                5192,  44660, 54532, 13064, 28660, 18296, 61092, 9496,  39708,
                5512,  36264, 46172, 24944, 31756, 48128, 21468, 1284,  43072,
                60832, 16168, 27400, 19156, 43508, 60,    56060, 11052, 4588,
                44060, 63792, 22988, 17792, 41188, 5876,  35644, 51888, 61256,
                5376,  29620, 43676, 1136,  16744, 60516, 13236, 55300, 20956,
                51236, 59344,
            },
            {
                28312, 2672,  44632, 39612, 65440, 3628,  48280, 12536, 55560,
                63864, 37268, 52864, 27696, 7332,  33280, 38300, 29956, 56172,
                48260, 36276, 44052, 12892, 33456, 41624, 4464,  11072, 31224,
                27088, 16620, 57224, 29088, 1036,  14720, 18888, 45136, 61512,
                25976, 47252, 38708, 22488, 27628, 49976, 43568, 4256,  63496,
                45688, 37356, 21616, 49904, 39140, 58188, 43544, 5308,  33244,
                10176, 49816, 55812, 30060, 360,   24828, 58596, 20916, 45340,
                15588, 51200, 47544, 30284, 55360, 37996, 11004, 19348, 14184,
                32432, 52048, 38552, 30004, 16968, 1408,  40188, 10616, 63788,
                35348, 56,    19820, 40012, 36112, 52468, 1144,  43992, 30968,
                55988, 51364, 8396,  19300, 60608, 6012,  58048, 28652, 50240,
                23644, 36116, 46196, 63824, 9540,  32484, 47044, 61972, 27644,
                15556, 48916, 30892, 38104, 61708, 56156, 31784, 12908, 2296,
                55384, 21204, 36228, 49928, 6556,  38456, 28012, 43208, 836,
                40948, 14144,
            },
            {
                37808, 61876, 22456, 32416, 7712,  28036, 42668, 20772, 32948,
                24616, 8576,  11980, 40748, 47880, 64344, 43360, 8972,  16532,
                5944,  65336, 1272,  59452, 17588, 27652, 63808, 56048, 48236,
                61048, 35848, 11692, 52592, 37500, 48932, 31624, 4332,  35612,
                9768,  18216, 6608,  56408, 14572, 10092, 30544, 21964, 51472,
                776,   56900, 8876,  26900, 11360, 32336, 20512, 61728, 53760,
                41660, 24160, 6264,  45836, 62236, 32588, 41348, 3384,  35304,
                25620, 39300, 2708,  23160, 8508,  42400, 33796, 63892, 47656,
                23076, 2144,  60384, 13244, 43416, 34164, 59108, 45480, 15280,
                50828, 25368, 57392, 62284, 4792,  46132, 25952, 15928, 59520,
                21528, 12544, 29464, 41788, 53332, 37012, 10316, 18080, 39984,
                7476,  55692, 4368,  38688, 22796, 53348, 17684, 36660, 20864,
                54968, 6644,  12100, 52684, 27124, 9336,  44380, 22660, 40576,
                48632, 9632,  63368, 59084, 23572, 52912, 18820, 5548,  62712,
                33952, 9144,
            },
            {
                56884, 5040,  18656, 54496, 14400, 57892, 36192, 4616,  50316,
                59728, 46040, 31864, 58028, 16096, 880,   25284, 58704, 22228,
                41540, 30756, 25680, 21192, 51700, 46524, 38116, 23368, 496,
                19116, 5488,  45600, 20620, 60368, 9408,  63272, 42176, 21736,
                57656, 29752, 52000, 41552, 46276, 62048, 7364,  58336, 25492,
                33016, 18312, 42968, 65388, 46392, 1100,  50660, 28608, 17804,
                38080, 64872, 13028, 36000, 16380, 49464, 55092, 13524, 8112,
                57796, 19460, 63672, 13956, 46008, 57280, 24876, 4160,  40700,
                57880, 27376, 46000, 20572, 54584, 26596, 5848,  55968, 22172,
                38288, 11800, 43488, 8956,  31600, 22204, 63764, 38000, 3764,
                47224, 34832, 65080, 260,   15376, 26368, 56084, 46492, 64692,
                14476, 33080, 11636, 49836, 29132, 58856, 7608,  40780, 1584,
                33668, 58712, 42476, 3052,  20636, 16424, 50676, 62948, 28128,
                15004, 33388, 18192, 26996, 11940, 45588, 31120, 54484, 25148,
                16408, 50192,
            },
            {
                43404, 25884, 35584, 46732, 51020, 23524, 62796, 10476, 29520,
                15312, 2040,  19716, 23328, 51352, 36428, 12584, 52960, 46364,
                34540, 11024, 54424, 37104, 3276,  9328,  15352, 33088, 42332,
                53836, 65160, 25212, 40756, 6580,  28252, 24216, 53564, 13320,
                48528, 64664, 3260,  24512, 32788, 18672, 35576, 40992, 47392,
                12416, 54084, 30276, 14976, 23240, 59220, 36524, 8484,  14372,
                57680, 1680,  28268, 52336, 9528,  22196, 27580, 64084, 43308,
                53120, 29060, 5704,  35664, 52348, 372,   16264, 50092, 30164,
                8432,  17656, 36008, 4960,  62564, 9724,  48728, 18008, 30604,
                6820,  59552, 27952, 17500, 49140, 13660, 54408, 10296, 28124,
                17832, 55044, 23844, 44856, 50588, 33612, 21912, 2380,  30292,
                51872, 26076, 20652, 62056, 2096,  14668, 26208, 51836, 63220,
                46004, 29104, 24560, 64552, 47156, 34840, 280,   37272, 7684,
                57948, 42492, 46892, 2728,  39852, 8148,  35064, 60216, 47756,
                21648, 30152,
            },
            {
                64544, 12804, 59820, 9764,  940,   38928, 17752, 53544, 43952,
                39440, 55236, 35388, 62932, 6256,  29244, 60432, 17928, 2616,
                61856, 49456, 14528, 40904, 62296, 29988, 59952, 6652,  49552,
                10312, 30552, 14368, 33968, 55900, 17276, 46488, 5660,  32612,
                39992, 16700, 36672, 12120, 59984, 1796,  53116, 15644, 4876,
                38064, 60616, 3200,  35196, 55392, 5912,  42156, 25272, 47376,
                31800, 43400, 19516, 59188, 40612, 2308,  37936, 18444, 32008,
                11152, 48124, 40404, 59668, 31548, 20360, 62704, 37216, 54216,
                11728, 64484, 41740, 51132, 23232, 38804, 33256, 2808,  63292,
                41384, 52988, 34280, 2384,  56272, 40728, 33096, 51176, 43092,
                61224, 6176,  11168, 38612, 4040,  59560, 12708, 43756, 6468,
                37288, 57856, 45692, 40176, 34188, 43372, 48504, 12336, 23092,
                17256, 10016, 37640, 14696, 53872, 30036, 59128, 24792, 18844,
                53308, 5556,  30356, 56820, 51556, 63816, 14784, 3452,  11128,
                38880, 1512,
            },
            {
                7296,  20596, 41512, 31256, 27244, 45112, 33836, 6980,  25652,
                62088, 4300,  27356, 47276, 10344, 44316, 39692, 31944, 26728,
                8240,  19616, 57352, 5708,  23860, 43644, 52784, 20724, 26656,
                38936, 57824, 1660,  51664, 43396, 11512, 36344, 57168, 960,
                26436, 8788,  45048, 54868, 28524, 48372, 23396, 64540, 29112,
                20484, 8008,  49556, 40508, 19096, 12564, 52100, 63676, 3884,
                55048, 23172, 7612,  33324, 62780, 46936, 6156,  50472, 60660,
                1156,  22008, 15880, 25352, 8944,  44624, 28160, 6472,  21796,
                34112, 47900, 936,   15240, 29588, 12284, 53652, 44132, 25772,
                13408, 20316, 46580, 65300, 24648, 6420,  19452, 1800,  36092,
                25120, 49476, 31808, 20332, 63332, 28772, 41164, 53984, 61588,
                18376, 9576,  3704,  16348, 55312, 5720,  61004, 30088, 35940,
                3560,  56412, 49984, 7576,  40524, 4708,  11308, 46248, 39148,
                65340, 22052, 13080, 36712, 17444, 22776, 42192, 27416, 44904,
                56064, 52244,
            },
            {
                32448, 48796, 55100, 16676, 63976, 5304,  56976, 21316, 12464,
                49252, 18424, 42104, 14128, 57348, 20556, 49760, 4392,  55860,
                37884, 45020, 28380, 33184, 48472, 17352, 12248, 56768, 34684,
                16232, 47020, 22088, 61788, 4212,  30112, 64100, 21076, 51512,
                61864, 19316, 58620, 5996,  20892, 38860, 9812,  43412, 56680,
                51808, 24572, 44980, 27432, 62708, 33468, 29720, 21056, 38980,
                11464, 50612, 45528, 14808, 25020, 30348, 56892, 14448, 26156,
                37004, 56148, 42240, 50188, 12572, 60776, 39044, 14332, 42992,
                59304, 25032, 31704, 56072, 60908, 47128, 19692, 57596, 36436,
                49632, 4676,  10144, 37676, 44484, 29532, 57760, 63228, 8812,
                14264, 41292, 56624, 16392, 47096, 7084,  24968, 15044, 33988,
                23156, 49024, 64088, 31268, 24392, 20108, 38160, 8144,  53696,
                41880, 19656, 59632, 33188, 21960, 62544, 55744, 14308, 31700,
                3500,  49716, 25860, 61692, 540,   32884, 50308, 59336, 19176,
                35916, 15632,
            },
            {
                27720, 37212, 2544,  23080, 51448, 14116, 47624, 60852, 31384,
                37132, 8520,  52396, 33424, 1324,  24496, 63820, 15308, 42660,
                22724, 63056, 64,    9936,  36204, 64784, 2116,  40388, 4728,
                63404, 8368,  36540, 27060, 19192, 47896, 39120, 14152, 43656,
                29280, 49300, 33960, 42188, 14192, 61500, 32260, 3036,  13764,
                34104, 61620, 10516, 16100, 2004,  46180, 9016,  60024, 17004,
                34756, 58552, 5360,  37060, 53372, 10008, 20700, 44448, 33808,
                7812,  65296, 4412,  32864, 54760, 2340,  46832, 56692, 3896,
                51636, 18856, 7672,  39864, 4000,  27460, 8732,  1236,  15944,
                61816, 31836, 22900, 59780, 15624, 11564, 48292, 21844, 46068,
                30120, 59792, 4724,  51964, 36528, 9872,  58508, 50156, 1340,
                38924, 28296, 42196, 53060, 10932, 57684, 45176, 15276, 65212,
                31872, 46920, 924,   25988, 44500, 17768, 27752, 42692, 52096,
                34632, 9488,  41204, 45432, 55204, 10500, 29788, 4348,  8344,
                24660, 61332,
            },
            {
                5132,  46376, 57972, 9184,  39724, 35808, 28672, 41888, 156,
                56040, 23504, 65148, 29496, 38632, 53596, 8772,  35652, 11444,
                52744, 16756, 50200, 58772, 26200, 21632, 45368, 31596, 51280,
                24148, 42928, 12936, 59772, 50552, 9516,  24400, 6764,  35136,
                3212,  10776, 24852, 53536, 800,   50616, 26520, 46960, 18496,
                41400, 5352,  37312, 58496, 54864, 23780, 50072, 42696, 564,
                25924, 29328, 64632, 18260, 42340, 59696, 4276,  40232, 52824,
                17388, 45884, 27944, 19644, 36156, 24076, 18112, 31116, 26816,
                9984,  62096, 45640, 21584, 35720, 42852, 64828, 24008, 40588,
                28456, 55752, 51276, 432,   33816, 39764, 26544, 52524, 17300,
                38256, 784,   27264, 22656, 32760, 40360, 19180, 30420, 45060,
                56204, 5416,  13672, 35908, 728,   27708, 50380, 4140,  23548,
                10528, 28064, 61888, 12928, 36184, 49168, 8308,  1424,  20328,
                60472, 16160, 28876, 6376,  20040, 39272, 64860, 47116, 57620,
                41280, 11748,
            },
            {
                63052, 42776, 19088, 26332, 60044, 3816,  18648, 10596, 51632,
                15984, 45076, 5184,  59764, 17124, 43868, 27820, 60640, 32832,
                5268,  30256, 41076, 13036, 38572, 55064, 8080,  28092, 59424,
                18568, 54328, 30412, 40096, 2552,  33060, 58412, 52488, 60912,
                16356, 41020, 65280, 30192, 17320, 58316, 36316, 7720,  63640,
                30048, 48848, 21752, 31500, 39844, 6432,  36188, 13696, 55912,
                48488, 40160, 22068, 2424,  13296, 47968, 28688, 62404, 12180,
                23784, 58248, 10624, 51220, 61820, 7460,  48644, 64644, 40868,
                15756, 37128, 54576, 13148, 59072, 17396, 52180, 34412, 48000,
                11920, 7264,  42036, 18516, 54348, 62320, 3064,  6860,  33000,
                54792, 64688, 44516, 12852, 61452, 53244, 3364,  65448, 11756,
                21296, 59372, 17776, 47348, 63180, 32904, 40540, 18396, 58332,
                51564, 38332, 16724, 5240,  52840, 58068, 30972, 63772, 24192,
                38024, 53828, 58636, 48212, 35176, 14040, 25560, 16712, 33136,
                21356, 51260,
            },
            {
                15200, 30392, 34628, 12320, 44204, 53900, 24440, 63592, 34076,
                26924, 40228, 21548, 12620, 48696, 2736,  19468, 47164, 56500,
                24948, 45860, 20600, 53504, 2916,  61120, 15444, 49004, 11632,
                676,   46448, 5932,  15904, 55448, 42088, 17908, 27248, 46084,
                55680, 22548, 4180,  38132, 44844, 21488, 11688, 53884, 23992,
                55932, 140,   12744, 52344, 18340, 65004, 28308, 20248, 62324,
                4768,  10408, 52284, 56700, 31940, 35816, 22516, 192,   49340,
                31260, 39396, 1872,  43528, 14992, 38328, 11684, 33588, 656,
                50352, 29068, 4896,  32796, 49272, 10364, 30136, 5212,  60444,
                20816, 36848, 25740, 45936, 9228,  30616, 58664, 43336, 12428,
                24200, 9168,  19684, 49144, 5888,  15900, 42920, 26744, 35280,
                8532,  51688, 25464, 43552, 21832, 8744,  60080, 25760, 35284,
                6588,  45704, 55432, 41404, 22920, 10876, 39628, 47152, 5716,
                44756, 12388, 2704,  23276, 60988, 51660, 1416,  43948, 54332,
                38240, 2180,
            },
            {
                23960, 55832, 6008,  65064, 49352, 32332, 7796,  58564, 46184,
                3152,  56872, 9156,  36144, 31884, 58120, 7368,  37480, 14164,
                1836,  63728, 34368, 6896,  18480, 32472, 43284, 22676, 37604,
                32912, 61656, 25316, 35220, 64892, 21992, 11604, 344,   31532,
                36624, 13092, 48048, 7436,  62816, 33548, 4740,  43880, 38772,
                15184, 59464, 26688, 44320, 3288,  11312, 54240, 44852, 31100,
                16328, 34116, 43916, 24872, 63396, 8912,  17080, 55160, 37452,
                6672,  63512, 21180, 29240, 55664, 25736, 58444, 20192, 53888,
                22952, 43600, 63880, 26140, 2200,  23112, 55200, 14676, 44076,
                1808,  57660, 64512, 13468, 49532, 23436, 20156, 35988, 50052,
                57412, 41688, 35184, 28356, 58204, 38204, 23440, 55380, 48300,
                61768, 31144, 38004, 3156,  55188, 12952, 48824, 1824,  43832,
                13988, 20944, 29456, 64260, 2264,  34520, 18748, 14512, 55548,
                26772, 32624, 18344, 42484, 8300,  31428, 56508, 11164, 6920,
                28912, 48036,
            },
            {
                9012,  51864, 40896, 21776, 1628,  15540, 38692, 20760, 13316,
                49780, 30428, 62892, 51316, 42212, 25448, 64488, 22020, 51728,
                43408, 9384,  59092, 47716, 28504, 51612, 64348, 3548,  41008,
                56916, 20524, 50736, 8280,  28748, 49108, 44408, 62448, 8456,
                19008, 59712, 52104, 27656, 15504, 56504, 49216, 28644, 19792,
                9260,  42540, 32532, 47936, 60456, 35028, 24524, 7664,  38072,
                47384, 60828, 19464, 6836,  39224, 51424, 45228, 60172, 26604,
                14604, 47016, 53016, 32776, 5684,  41712, 2988,  45548, 61076,
                7224,  11480, 19144, 57160, 41408, 45872, 62512, 38680, 32168,
                50964, 16948, 29424, 34752, 3744,  38968, 14700, 63096, 27116,
                5412,  16616, 53548, 1940,  46628, 31984, 7336,  13024, 184,
                40772, 15372, 6116,  60944, 28448, 17152, 37420, 30200, 56528,
                61008, 33504, 9172,  48420, 25704, 59748, 50560, 29860, 61956,
                9964,  37144, 50988, 64272, 28300, 40320, 21660, 36148, 63540,
                18624, 59216,
            },
            {
                31680, 37068, 17356, 27676, 57248, 47552, 29540, 43124, 5864,
                36880, 18440, 23808, 14812, 5380,  54024, 11040, 40624, 29756,
                16464, 26884, 39392, 22336, 37076, 13828, 9712,  25840, 52928,
                16340, 10740, 45032, 37292, 14248, 5176,  56948, 38648, 50468,
                25256, 42672, 32688, 10080, 23800, 40392, 1860,  60880, 51444,
                35240, 64148, 6948,  22928, 16064, 40944, 51196, 1072,  58832,
                26752, 12244, 2024,  49400, 28080, 13864, 4928,  42084, 19740,
                34352, 57528, 8536,  17628, 62768, 49356, 35676, 16564, 30708,
                38944, 47920, 34212, 14472, 36920, 7688,  18588, 26592, 9296,
                22692, 40844, 6416,  52300, 61540, 43848, 8060,  47300, 1160,
                39468, 30360, 63696, 10640, 20648, 62384, 52216, 18120, 29144,
                45512, 20296, 49668, 33824, 52716, 41824, 64660, 10756, 22976,
                52356, 3176,  18176, 37860, 13228, 43764, 4848,  22064, 42040,
                152,   58132, 47444, 4032,  12664, 17160, 59924, 49740, 25848,
                45552, 12848,
            },
            {
                62304, 43608, 4116,  61180, 36232, 11112, 53164, 63132, 26408,
                59312, 54736, 2304,  46340, 28776, 17936, 33396, 49304, 4584,
                61732, 53756, 11968, 56876, 4872,  60128, 45260, 34588, 6300,
                29564, 62836, 1716,  54224, 41224, 24764, 33828, 15528, 29724,
                4864,  54768, 1208,  61596, 46884, 18628, 31388, 12632, 25064,
                2932,  17692, 38252, 53808, 9704,  29916, 63104, 14216, 21716,
                54740, 42128, 30540, 56124, 35036, 61960, 23836, 30980, 3216,
                11156, 39924, 24908, 44388, 12844, 22416, 28056, 9400,  57560,
                24576, 52540, 1360,  61812, 50000, 29536, 2804,  48340, 56212,
                59980, 45220, 11476, 19576, 27344, 56732, 32000, 53976, 19208,
                60112, 22700, 44680, 14228, 36800, 25244, 43584, 58892, 34836,
                64200, 56036, 11228, 24300, 7816,  46168, 19956, 4820,  40812,
                26776, 46456, 62664, 54436, 31668, 56856, 8784,  35952, 53440,
                15872, 24656, 20452, 34244, 55328, 46460, 6028,  3144,  33672,
                39656, 448,
            },
        },
        {
            {
                48516, 21168, 16008, 34864, 24944, 64268, 43168, 33288, 61452,
                49800, 40016, 31376, 22124, 59192, 6940,  40424, 1672,  43832,
                47872, 4572,  36500, 28588, 19432, 5052,  42324, 11296, 21252,
                55732, 50992, 5572,  15160, 53000, 48428, 60920, 44084, 39964,
                50476, 33924, 2240,  51744, 55284, 8396,  60772, 21272, 10484,
                64156, 46528, 41988, 60472, 5704,  45272, 35260, 19308, 26516,
                59184, 3324,  64712, 52480, 340,   56988, 6640,  38212, 16832,
                59384, 20300, 48756, 15352, 64348, 4456,  34684, 53608, 29292,
                59048, 36084, 31476, 14988, 20288, 60688, 7380,  28488, 43440,
                2292,  12756, 18204, 23292, 59988, 15268, 7868,  49576, 32856,
                17424, 10676, 53804, 63456, 26412, 8900,  20272, 34472, 27028,
                14304, 31460, 52308, 4132,  21476, 38776, 58852, 41796, 50368,
                8804,  18756, 46888, 58684, 1832,  52564, 6748,  26844, 22300,
                6344,  48356, 55088, 8828,  60476, 50884, 5684,  53580, 34764,
                17944, 8248,
            },
            {
                28724, 55464, 62704, 50996, 3576,  19464, 47216, 9616,  58696,
                24140, 6516,  51964, 888,   44724, 48876, 30408, 19668, 11200,
                33272, 15344, 56384, 9912,  61824, 14064, 49488, 29444, 64180,
                8368,  40124, 61512, 45008, 25692, 29936, 7968,  14244, 22472,
                9716,  30368, 59432, 19884, 12396, 29324, 49736, 43736, 53028,
                7416,  25904, 3140,  17428, 28904, 22952, 53572, 30448, 7364,
                16792, 45704, 34376, 28012, 41220, 23712, 12508, 31208, 3640,
                42980, 33160, 612,   11704, 27388, 41768, 23832, 2908,  18468,
                45812, 23008, 52660, 41248, 55648, 37852, 45568, 58232, 15504,
                24980, 63860, 33080, 51212, 280,   35936, 26216, 38576, 2056,
                61416, 40468, 35132, 46568, 31708, 38412, 62588, 45364, 54128,
                620,   59320, 46920, 24728, 54612, 16724, 62740, 32524, 3748,
                53596, 34600, 29752, 43024, 39308, 31884, 62732, 55964, 40980,
                440,   31540, 12124, 17264, 23112, 37016, 27636, 62404, 24996,
                11260, 42008,
            },
            {
                5328,  13316, 37356, 6504,  40936, 30888, 12868, 36980, 2444,
                44120, 16664, 34832, 56668, 24604, 15056, 53904, 58212, 38516,
                63288, 21112, 51868, 44860, 24764, 34172, 57464, 38016, 26496,
                16328, 30868, 1608,  56940, 11872, 39092, 35384, 55476, 18096,
                63896, 44732, 7028,  26248, 41148, 36236, 4124,  15404, 34464,
                30592, 13540, 55556, 37768, 62892, 8864,  42372, 57692, 12444,
                38564, 55060, 5640,  14500, 49220, 35824, 54124, 63664, 46684,
                57156, 24524, 61564, 52216, 45296, 58736, 9400,  50364, 61224,
                12924, 1664,  65172, 8712,  25100, 11804, 1188,  31792, 36440,
                49128, 40336, 7196,  44800, 29480, 55976, 19224, 52536, 46128,
                29064, 7044,  20844, 928,   14636, 52036, 5228,  10888, 16952,
                22324, 39396, 8016,  13156, 44000, 30364, 1060,  46504, 22984,
                15796, 65252, 12948, 4416,  24392, 20100, 36060, 15112, 50148,
                60244, 38136, 46432, 65444, 42552, 4168,  15204, 48708, 20232,
                45328, 57144,
            },
            {
                25992, 32152, 45956, 23296, 60744, 52648, 57092, 27120, 21328,
                53400, 63388, 28016, 11464, 37992, 65228, 8872,  27292, 2660,
                49864, 31056, 7680,  532,   39860, 17268, 5920,  52544, 3400,
                48096, 43392, 22260, 33644, 19288, 65448, 2592,  49384, 28064,
                4044,  36908, 47824, 61208, 16676, 56336, 63336, 23992, 39756,
                59388, 51368, 22208, 47888, 32836, 14348, 51244, 2792,  48540,
                61724, 24324, 43624, 20852, 62388, 8132,  19348, 26596, 14572,
                9132,  37704, 6864,  18996, 36056, 16248, 31784, 39608, 26492,
                33936, 48996, 17436, 28752, 47600, 59280, 22220, 51536, 4820,
                10320, 27576, 60952, 20696, 12228, 40968, 63232, 9372,  14252,
                57324, 25508, 49260, 60144, 43324, 24008, 29184, 57296, 41932,
                61156, 35476, 27624, 64916, 5448,  49596, 25892, 12064, 56252,
                37300, 26304, 49664, 60420, 54768, 8344,  44316, 10936, 28896,
                18104, 25068, 34544, 2572,  56116, 30988, 39916, 59304, 128,
                36576, 61604,
            },
            {
                2332,  50120, 58632, 18184, 8052,  15224, 49016, 5244,  40688,
                32052, 8120,  47028, 41864, 20196, 3768,  34100, 43336, 17736,
                23972, 41596, 58808, 27892, 64972, 46320, 22036, 61012, 12432,
                35612, 58600, 9528,  52260, 47096, 6688,  59008, 24568, 40812,
                57388, 13752, 53288, 22812, 1268,  32092, 46608, 6112,  19052,
                1484,  42480, 10688, 5320,  58296, 39908, 20704, 25868, 33616,
                18432, 9620,  32076, 58480, 29260, 45228, 1576,  51592, 34676,
                21816, 49652, 41032, 29716, 56332, 5828,  63640, 20576, 43664,
                7256,  57456, 37388, 5048,  33260, 40548, 18124, 64380, 42768,
                56068, 17300, 53100, 3860,  58664, 31692, 5624,  43916, 22616,
                37296, 64688, 11396, 17804, 54992, 36532, 19336, 48408, 32796,
                2596,  51040, 19748, 55432, 41000, 36320, 19164, 61424, 40204,
                6960,  44996, 10232, 33752, 17500, 48504, 57840, 3936,  64084,
                53236, 7628,  14200, 49944, 21656, 11004, 54140, 8440,  33052,
                51836, 17132,
            },
            {
                22120, 43544, 11216, 29756, 38880, 25364, 34256, 62184, 18412,
                55868, 14024, 4532,  60644, 31072, 51516, 47632, 13524, 62164,
                55580, 10272, 36012, 13212, 53984, 31680, 9136,  42244, 24480,
                18364, 62696, 27224, 14540, 41856, 29240, 16056, 45260, 10896,
                31128, 19504, 34180, 9936,  43588, 27644, 11900, 54372, 49196,
                28676, 65224, 36268, 18320, 27272, 468,   63760, 46012, 37032,
                4660,  52148, 39780, 2920,  11860, 38000, 59656, 41992, 30420,
                55536, 2216,  65396, 25660, 13172, 47628, 1264,  51400, 11096,
                54608, 24108, 42316, 62728, 14192, 54368, 6332,  29564, 13164,
                23668, 38472, 33620, 46656, 24348, 49872, 16872, 34160, 54336,
                2496,  30908, 41704, 33632, 8568,  3416,  64212, 7500,  25404,
                14972, 45728, 10576, 32252, 15584, 58308, 8672,  51332, 29660,
                2640,  57948, 21528, 28732, 1168,  41516, 25916, 31336, 21360,
                39688, 44892, 58556, 28220, 61408, 17640, 44096, 23924, 29144,
                14136, 40740,
            },
            {
                8636,  34496, 53348, 2956,  65284, 42536, 732,   45620, 10368,
                38148, 50688, 26340, 16480, 57104, 22796, 37020, 6728,  29692,
                46376, 4296,  16640, 49256, 19972, 3088,  37564, 56528, 30316,
                49204, 168,   38756, 4856,  54892, 36192, 20676, 52008, 712,
                61924, 50072, 4956,  64896, 38508, 59012, 20808, 35784, 61104,
                14624, 24504, 45164, 55132, 31248, 43156, 10968, 15684, 55720,
                28852, 65128, 14092, 47004, 53352, 24884, 16424, 5416,  11500,
                61016, 17512, 10444, 44124, 53712, 22816, 35504, 28192, 62140,
                15420, 31000, 20236, 24,    50928, 26180, 36036, 59876, 45768,
                1444,  62664, 8560,  14380, 37184, 1084,  27240, 61480, 47540,
                19936, 5352,  51880, 27516, 58908, 46072, 40020, 12296, 53004,
                62688, 29340, 3900,  60792, 23660, 1852,  45496, 33568, 20648,
                16320, 42176, 63508, 50844, 37808, 62000, 14604, 35564, 47692,
                1884,  10548, 32624, 41364, 6100,  36852, 48076, 62880, 4520,
                55716, 64168,
            },
            {
                26372, 59980, 19832, 47180, 55320, 12308, 21720, 58004, 29212,
                23336, 64480, 36104, 44772, 216,   10564, 59160, 52984, 19352,
                39028, 33144, 63172, 43888, 25004, 60212, 50988, 14840, 6648,
                45152, 55704, 31804, 23288, 63468, 8892,  59844, 33200, 38240,
                23256, 41600, 26880, 47320, 15152, 51876, 6844,  41556, 9368,
                32888, 3472,  48604, 8000,  60800, 52060, 23656, 59032, 7768,
                49500, 21860, 26896, 57044, 19040, 33064, 64208, 48292, 36868,
                27816, 46064, 38848, 33624, 4728,  60132, 41944, 17908, 38372,
                3276,  46152, 60192, 10920, 44952, 21628, 8908,  49412, 32792,
                19212, 28524, 50596, 43360, 65164, 57008, 9800,  42076, 13036,
                38972, 56824, 16476, 48664, 13916, 22864, 30724, 56544, 35704,
                42476, 21516, 38648, 48044, 52196, 27508, 13612, 62988, 54592,
                47688, 35664, 13724, 5608,  23484, 11124, 52612, 7336,  55644,
                62312, 23860, 18908, 53776, 924,   25812, 33992, 12740, 19632,
                46352, 37336,
            },
            {
                30824, 15288, 6032,  36808, 27596, 32644, 17112, 52396, 6924,
                48308, 3684,  19508, 54648, 28720, 40244, 15140, 26260, 2916,
                59904, 22164, 27616, 7320,  39572, 11248, 34152, 27480, 64556,
                21100, 11400, 16828, 44048, 47904, 13040, 26668, 5740,  54344,
                14400, 8240,  56132, 18336, 2704,  24700, 30068, 45784, 17624,
                57544, 38816, 21480, 12960, 16808, 35152, 3740,  39276, 32184,
                43896, 968,   41028, 35548, 4964,  8880,  43804, 22752, 54280,
                19664, 7632,  59348, 21240, 15044, 30460, 8532,  57116, 48580,
                6448,  52372, 35064, 28532, 56616, 38836, 16212, 4004,  57404,
                40280, 54640, 5152,  21952, 30252, 18660, 52624, 23540, 29316,
                7644,  63108, 24776, 4,     37124, 60588, 20004, 1676,  17320,
                5540,  59284, 9016,  34344, 18020, 43000, 37836, 4192,  24328,
                9148,  27304, 53120, 32104, 45924, 59568, 20344, 43184, 16384,
                27440, 38012, 48592, 64648, 15296, 51576, 57624, 7576,  42240,
                50616, 3540,
            },
            {
                57472, 44852, 51764, 23604, 62896, 8700,  43608, 60916, 34872,
                41420, 12660, 32828, 8752,  49624, 64044, 32392, 44440, 48584,
                12092, 55140, 51272, 872,   56912, 46712, 18120, 2348,  40912,
                36488, 50644, 61232, 34420, 2020,  39580, 49692, 17932, 63100,
                45780, 29188, 60296, 36536, 32360, 55244, 63992, 12,    50560,
                26936, 53480, 63056, 29040, 40696, 49724, 26652, 19848, 12732,
                60888, 17292, 10500, 62304, 50976, 29872, 58032, 8,     13772,
                50156, 31184, 2996,  52432, 46936, 63220, 24704, 12656, 32456,
                25820, 40624, 18308, 7412,  64536, 31704, 43036, 61544, 24992,
                10016, 15124, 60860, 35020, 12028, 39460, 6088,  48752, 60668,
                35732, 45424, 32192, 41960, 9512,  52320, 44632, 27216, 50268,
                46772, 24788, 13048, 53960, 64588, 6896,  56592, 31932, 41064,
                59268, 0,     19316, 57108, 39488, 3136,  29524, 33672, 4500,
                58372, 12540, 7992,  29916, 45428, 38928, 22148, 60424, 32120,
                24164, 11776,
            },
            {
                18072, 61364, 41268, 13104, 4316,  50452, 38812, 2092,  15000,
                26048, 58652, 61992, 42564, 24192, 5632,  20988, 61056, 35252,
                8924,  42208, 15660, 36452, 29720, 62048, 23280, 52952, 57916,
                8948,  25356, 4504,  28644, 57064, 21908, 30736, 42840, 2468,
                35188, 21120, 11724, 44208, 7448,  38984, 13716, 22844, 34980,
                12052, 5788,  44044, 1288,  57756, 6964,  64164, 53764, 46840,
                34448, 24344, 54596, 45952, 20896, 15328, 38864, 26204, 34388,
                63516, 41528, 56728, 26988, 35776, 668,   43396, 55280, 20764,
                63848, 58864, 13868, 23508, 48988, 2520,  12344, 52388, 21048,
                36876, 47496, 26568, 1916,  45708, 55172, 32960, 2988,  15148,
                20888, 11080, 55196, 18504, 65520, 4616,  34412, 11412, 63600,
                31384, 56104, 41228, 696,   30248, 22408, 10724, 17564, 50168,
                12772, 64904, 43836, 7960,  16732, 25524, 63788, 47104, 51052,
                41532, 35816, 54756, 20956, 3564,  9796,  28116, 16572, 1712,
                53840, 39360,
            },
            {
                9116,  640,   33244, 29352, 19196, 57412, 24084, 30828, 54284,
                20512, 45900, 1652,  16756, 36768, 13348, 54160, 1304,  18052,
                25460, 31564, 65092, 20672, 8528,  13456, 43688, 5748,  29128,
                15556, 42992, 53972, 19208, 8024,  64408, 51588, 10820, 24816,
                58728, 49136, 3800,  52592, 62368, 19840, 47856, 58968, 42892,
                61236, 19168, 36844, 32276, 22976, 45256, 9880,  29288, 3060,
                58256, 6432,  37948, 27392, 2616,  42380, 61636, 47020, 4268,
                9916,  23812, 16496, 11688, 40248, 19460, 50856, 5012,  37728,
                10144, 1452,  44144, 54044, 37264, 17688, 27700, 46456, 30676,
                6808,  53276, 41612, 58844, 16152, 63228, 25108, 43576, 58772,
                52212, 26768, 3528,  49424, 29264, 24268, 54576, 39500, 7528,
                15132, 37152, 19340, 26712, 50600, 43912, 58560, 46500, 28332,
                38640, 22084, 30012, 48828, 36420, 52104, 13440, 9084,  22928,
                18396, 1416,  59808, 32540, 42680, 62964, 56084, 36204, 48744,
                65012, 28584,
            },
            {
                47296, 22836, 55392, 48728, 37216, 44608, 11636, 64368, 49284,
                9112,  38480, 28304, 51520, 57584, 30140, 43492, 38568, 50868,
                57024, 3692,  47928, 40076, 53512, 32824, 49120, 38172, 63192,
                32220, 48524, 59584, 13284, 41732, 36924, 15208, 56452, 6936,
                39772, 31452, 15416, 26268, 41628, 10024, 28548, 3996,  31052,
                9008,  24956, 47264, 51920, 13760, 18392, 36404, 42180, 15596,
                50428, 31036, 64476, 13512, 32944, 53340, 10924, 18088, 56144,
                37856, 44792, 61436, 48192, 7136,  58612, 14320, 34060, 46448,
                29968, 50440, 26804, 33760, 5172,  55876, 62356, 40644, 512,
                64720, 18080, 31976, 23036, 8492,  50444, 19268, 37792, 30392,
                7084,  40820, 57628, 37924, 13684, 43212, 16692, 60968, 22592,
                2756,  44336, 61492, 4928,  33916, 14928, 36428, 1808,  61532,
                6164,  34788, 55496, 11376, 60348, 1580,  40452, 56804, 31232,
                61808, 26528, 46596, 14508, 24468, 50296, 12608, 41004, 7064,
                20328, 35096,
            },
            {
                42968, 62780, 5480,  15188, 59916, 2720,  27512, 17460, 5976,
                32112, 55588, 22196, 4932,  47456, 10160, 62800, 7272,  28068,
                45136, 12784, 23776, 5164,  60844, 26320, 1756,  19852, 11768,
                22028, 2600,  35348, 24592, 46260, 1152,  33076, 27736, 47256,
                18868, 64760, 55052, 22728, 33676, 50908, 17076, 54168, 38052,
                56312, 15920, 63708, 3592,  59364, 54660, 25720, 61152, 21356,
                39832, 9344,  18500, 48768, 5552,  59952, 22088, 31596, 50464,
                28680, 6076,  21144, 33088, 29700, 64984, 26148, 54316, 16936,
                62320, 21784, 57732, 11344, 42472, 20400, 9028,  35048, 14396,
                57752, 11392, 49456, 3396,  35668, 28660, 11552, 680,   64572,
                47568, 17028, 33892, 21788, 59888, 5680,  32600, 48080, 51644,
                27820, 57804, 47380, 11444, 55252, 63864, 8144,  25612, 19824,
                51500, 15452, 3760,  24012, 42620, 28380, 21020, 35016, 6092,
                44368, 10932, 51980, 5316,  34880, 18760, 4304,  45092, 26016,
                14340, 57072,
            },
            {
                11616, 26932, 39256, 21660, 9440,  33584, 53052, 46828, 41072,
                61052, 12212, 35416, 65512, 25688, 18684, 33516, 22760, 15904,
                61832, 35988, 58208, 18740, 42528, 16140, 55744, 59972, 44904,
                52332, 39492, 9752,  30240, 60964, 54636, 21288, 61868, 43224,
                13016, 648,   37848, 6384,  61340, 1828,  45028, 22336, 7116,
                50016, 41040, 28004, 10960, 39424, 30872, 5028,  47504, 1156,
                56684, 23792, 44148, 57964, 40796, 25468, 36620, 7812,  64616,
                14620, 54812, 1720,  49908, 39240, 9592,  22424, 41904, 2936,
                8288,  40188, 15356, 31084, 47932, 58820, 25964, 50912, 22508,
                44104, 27220, 38800, 62204, 44728, 57304, 41124, 51488, 14072,
                23816, 9220,  62592, 1352,  46868, 10352, 55952, 20176, 36380,
                9124,  32140, 16500, 24128, 39700, 21160, 48160, 31828, 54308,
                40596, 45332, 62736, 33360, 17724, 53388, 64640, 15072, 50136,
                37696, 17412, 64132, 39628, 59136, 54448, 30552, 61124, 33068,
                52740, 2356,
            },
            {
                17548, 31988, 50532, 54396, 42212, 63068, 22956, 36172, 952,
                19516, 44464, 15616, 40228, 2432,  52668, 59308, 41816, 1948,
                53268, 32012, 9176,  29028, 50920, 37208, 7348,  34444, 28020,
                6024,  65508, 18356, 50116, 5668,  16892, 4060,  52428, 9260,
                35284, 50436, 27428, 46616, 12120, 58052, 35472, 65336, 12912,
                33044, 1588,  20372, 34796, 49308, 8112,  65016, 33376, 12192,
                52872, 28648, 35196, 2248,  52064, 16040, 46648, 948,   43056,
                26120, 35512, 60396, 17612, 53060, 4184,  45716, 28696, 59484,
                35288, 48800, 5780,  65384, 1216,  16408, 38896, 3736,  54916,
                33400, 19640, 5068,  54496, 14704, 20752, 6052,  32560, 55236,
                36564, 44328, 28716, 53488, 25740, 40416, 29632, 1836,  13332,
                64928, 42420, 5804,  52412, 29520, 2500,  42716, 60664, 13548,
                9312,  26552, 57688, 48336, 6980,  10116, 46300, 3964,  24924,
                55424, 29056, 7980,  21524, 27460, 184,   47432, 9416,  23048,
                63520, 37540,
            },
            {
                46412, 60564, 7000,  18916, 29512, 3788,  12768, 58568, 26136,
                51396, 56564, 6804,  31420, 46200, 13856, 36832, 49024, 11488,
                20960, 39824, 47008, 320,   63804, 10704, 24096, 47512, 13688,
                54860, 23676, 43708, 58380, 27176, 40996, 38036, 24176, 29864,
                59880, 20584, 57208, 15852, 39952, 29708, 19408, 42624, 26876,
                59948, 46220, 56876, 43372, 14716, 22720, 17980, 44552, 37524,
                15360, 63216, 7964,  21136, 12456, 29732, 62560, 53496, 18784,
                40056, 11032, 43984, 24484, 13924, 36652, 56760, 12276, 51100,
                18116, 25168, 53768, 38048, 22888, 44748, 60732, 29636, 7568,
                63632, 48116, 10236, 30632, 24964, 60604, 46104, 26904, 59588,
                19704, 5016,  50284, 12716, 18360, 63964, 45516, 58244, 39316,
                25592, 54408, 19680, 62724, 36632, 10264, 57372, 18156, 4832,
                37664, 30856, 608,   20016, 38740, 58856, 30548, 41784, 61488,
                1096,  33324, 42776, 48960, 11924, 37104, 15812, 40472, 19080,
                49920, 4440,
            },
            {
                23852, 55968, 13888, 35480, 45128, 49612, 16436, 39088, 30432,
                9908,  22668, 62216, 27600, 58204, 8716,  24440, 29980, 64504,
                6436,  55956, 25412, 14828, 44500, 31344, 58028, 17684, 41360,
                1500,  32880, 14732, 35828, 11308, 48384, 64248, 14168, 45904,
                3304,  41592, 7716,  32292, 53676, 24624, 9040,  3228,  51596,
                16748, 23488, 6672,  61720, 54048, 29740, 58288, 50960, 24628,
                5588,  40356, 48224, 59064, 33764, 38544, 9404,  23624, 59180,
                31952, 2964,  56192, 30044, 47444, 61500, 20164, 416,   31292,
                61872, 45144, 12892, 28480, 33036, 9984,  49328, 13288, 42396,
                16976, 36696, 52176, 40708, 1200,  34552, 17488, 9960,  2712,
                41680, 31312, 57328, 37660, 6576,  34028, 15156, 23172, 7288,
                48448, 34428, 264,   45216, 14388, 49744, 24532, 34652, 46256,
                65440, 22488, 44056, 12288, 51756, 25776, 35904, 22228, 12628,
                19220, 57892, 14968, 53148, 63012, 24144, 58668, 55512, 6608,
                43712, 29332,
            },
            {
                9640,  41172, 404,   62004, 24732, 55368, 7780,  64800, 47408,
                4540,  34384, 49168, 42704, 20528, 54416, 3584,  45076, 17172,
                50312, 34144, 59476, 19584, 52852, 39556, 3288,  62828, 49952,
                29176, 61516, 7800,  56504, 21904, 31328, 6692,  54188, 33492,
                17876, 63448, 23264, 48620, 4568,  45532, 62852, 55600, 38720,
                30744, 10496, 37460, 26344, 268,   40776, 4324,  10068, 60152,
                31912, 53732, 18056, 26660, 44788, 56264, 4800,  49368, 13604,
                46044, 22020, 64060, 8680,  5080,  33264, 25856, 43808, 39076,
                9724,  21140, 3284,  55572, 63024, 18976, 36196, 56800, 25292,
                2028,  58912, 23004, 13148, 65232, 48984, 37880, 53112, 62464,
                48344, 15604, 22480, 60344, 43356, 52888, 3984,  50192, 62028,
                17672, 11516, 30904, 60048, 27112, 40192, 3436,  53692, 28112,
                50496, 16324, 55024, 32984, 63312, 15380, 6280,  53664, 48700,
                38328, 46120, 5744,  30468, 3688,  44960, 33476, 28280, 13072,
                59420, 34808,
            },
            {
                15664, 51664, 27544, 38316, 10988, 32956, 20872, 42856, 53820,
                18248, 37620, 11620, 176,   16460, 33268, 61744, 37976, 27064,
                42452, 2944,  11732, 36460, 5284,  27172, 21988, 35536, 9076,
                20180, 38884, 45064, 50840, 520,   42540, 19296, 59068, 10176,
                26188, 51280, 37224, 11496, 58476, 18148, 36160, 14036, 47124,
                5120,  64112, 49468, 19072, 12688, 46856, 35656, 27396, 20228,
                43448, 892,   11128, 61448, 2940,  16560, 41980, 28416, 35212,
                6720,  51728, 38140, 18516, 41656, 54024, 15716, 64336, 6652,
                57188, 34040, 42068, 46988, 7224,  51080, 4752,  20784, 32664,
                46428, 8744,  28260, 44232, 56100, 7832,  22100, 29704, 12572,
                25328, 35716, 352,   9884,  27300, 19572, 31372, 36848, 28216,
                41328, 57088, 21956, 51248, 8080,  17012, 58432, 20612, 11448,
                7220,  59448, 3732,  40868, 28580, 2384,  43860, 60612, 9256,
                27052, 65120, 23348, 40804, 19764, 8968,  51440, 1824,  39000,
                20776, 65316,
            },
            {
                31640, 57992, 19804, 46932, 5500,  59796, 28824, 1432,  14124,
                60740, 26480, 55220, 63408, 39504, 50712, 12752, 21876, 7928,
                60808, 31132, 23084, 64216, 48000, 56708, 12872, 53788, 46360,
                58992, 4404,  26728, 16772, 63104, 25116, 36548, 49516, 44076,
                1668,  56188, 14668, 28908, 42432, 448,   32820, 27524, 21200,
                57848, 42112, 34036, 52576, 31416, 62596, 56432, 16092, 48844,
                64012, 36784, 22376, 46964, 31200, 24616, 65412, 20044, 54360,
                61820, 16112, 26572, 49040, 57980, 10952, 37436, 47828, 23148,
                51860, 16804, 60204, 25644, 15144, 40848, 27664, 61684, 52652,
                39200, 62908, 18840, 31868, 3260,  15896, 58508, 40152, 44824,
                7128,  64728, 55652, 45928, 38832, 63352, 12204, 54760, 9160,
                3216,  46688, 37868, 4692,  43420, 33448, 61892, 30656, 42948,
                38612, 35116, 24632, 9904,  50068, 20716, 56244, 17260, 31320,
                1912,  34916, 11184, 54956, 36216, 62040, 16964, 47532, 25308,
                53780, 3176,
            },
            {
                48764, 7536,  42648, 64020, 17056, 50944, 36708, 23696, 46088,
                31392, 41464, 7520,  24544, 30496, 46696, 4876,  57300, 53156,
                14148, 45572, 51684, 40952, 17020, 7488,  43368, 30844, 16032,
                23908, 11992, 53392, 33800, 9876,  55488, 4232,  13208, 28428,
                40176, 34612, 6484,  62060, 21640, 60196, 49968, 11176, 53292,
                2692,  24492, 15172, 7456,  44024, 21900, 8432,  39356, 3792,
                13308, 28472, 55608, 39988, 8380,  52324, 11828, 37896, 76,
                10244, 43192, 34192, 3236,  29036, 21312, 2180,  30568, 13296,
                27764, 1436,  36948, 9428,  31468, 58404, 72,    43856, 11228,
                14884, 5220,  55148, 50292, 42568, 26772, 33944, 4328,  19272,
                51504, 32884, 16972, 49980, 23672, 2552,  58928, 44004, 20804,
                61200, 15588, 25704, 64624, 55852, 10492, 47412, 864,   14908,
                63480, 48616, 18608, 57136, 45720, 34372, 37760, 24232, 41488,
                52088, 59292, 16200, 49052, 26584, 42424, 31480, 60236, 11272,
                44384, 36572,
            },
            {
                26532, 11744, 23220, 34380, 2284,  56884, 40304, 11844, 58048,
                4156,  52352, 14920, 19564, 58352, 9508,  28096, 34328, 40024,
                18800, 26080, 824,   9820,  29072, 61040, 34564, 1388,  63548,
                37796, 42020, 29560, 59752, 46464, 39496, 30848, 60228, 21076,
                65144, 17224, 45264, 52328, 39220, 25556, 7080,  44404, 40004,
                18344, 61272, 28484, 59272, 2048,  53928, 25324, 30652, 51356,
                58072, 6500,  33856, 15096, 49080, 59000, 33308, 45160, 58064,
                30820, 22436, 60096, 13996, 63660, 50684, 45036, 59732, 55500,
                40544, 49164, 64872, 21636, 53416, 37944, 17844, 23552, 30696,
                47896, 35324, 21996, 37600, 11404, 63668, 47304, 54144, 61020,
                27908, 11340, 41948, 30004, 7852,  14560, 47740, 25300, 32408,
                52116, 35120, 11968, 29472, 18776, 23352, 37072, 26456, 53308,
                22388, 5576,  31864, 61568, 14108, 8092,  63844, 4064,  12268,
                44712, 21188, 7004,  56880, 804,   13764, 4696,  22348, 57004,
                6424,  18028,
            },
            {
                62980, 38860, 54544, 30092, 14836, 25876, 48348, 19252, 8980,
                34524, 64272, 48036, 3432,  36448, 43788, 16760, 65112, 48128,
                4336,  55676, 32908, 58896, 38200, 20532, 25232, 51264, 6792,
                57160, 48920, 2816,  19008, 6580,  23168, 15888, 52192, 8188,
                47452, 24380, 2576,  30544, 9648,  16096, 56764, 64880, 29804,
                34812, 8852,  46768, 36916, 40340, 17752, 65168, 34608, 45368,
                23532, 42128, 19020, 63364, 5476,  21728, 3572,  17048, 25088,
                50900, 5292,  46924, 40816, 7924,  25160, 35588, 4700,  9932,
                32972, 18264, 5544,  45456, 12616, 48480, 63764, 56320, 6856,
                59396, 26144, 1044,  60528, 17708, 8256,  24044, 14440, 1260,
                38248, 21696, 5208,  58360, 52344, 33848, 40048, 17520, 832,
                42520, 7484,  48520, 53508, 2108,  41224, 50748, 7908,  57880,
                44936, 11940, 42064, 26200, 1256,  30428, 53256, 47416, 28420,
                61388, 33092, 39304, 29900, 46744, 64276, 53184, 34616, 40492,
                29784, 51192,
            },
            {
                14328, 46264, 4412,  60456, 44532, 7036,  53452, 61956, 28248,
                43524, 21752, 26940, 40120, 60524, 51388, 2212,  23564, 10744,
                37292, 63028, 22004, 48860, 13356, 54484, 45304, 18348, 10692,
                32128, 21396, 13876, 36724, 64240, 50336, 42220, 936,   32428,
                57904, 11696, 55008, 34048, 46756, 20208, 36600, 1492,  14256,
                55344, 51068, 12552, 21300, 5460,  48628, 10760, 14764, 2932,
                60216, 9524,  54320, 27072, 37244, 43100, 29516, 48340, 39668,
                12748, 55100, 36672, 18812, 31920, 53224, 15988, 20096, 63008,
                44080, 24088, 56856, 35156, 29256, 8320,  3312,  33844, 41416,
                16960, 53272, 44360, 30032, 40920, 51004, 36812, 31324, 43152,
                55984, 47868, 62192, 35884, 19244, 26760, 60832, 6080,  55628,
                63616, 22944, 39372, 60124, 31872, 13676, 63036, 34132, 16820,
                29140, 2852,  55296, 38236, 50768, 23108, 16348, 57588, 19140,
                5040,  49728, 9948,  23608, 18400, 37024, 25840, 9164,  19960,
                60884, 880,
            },
            {
                32764, 21644, 50188, 10316, 36272, 22268, 32048, 2632,  38676,
                16080, 50728, 10520, 32768, 13152, 21148, 31444, 56392, 27420,
                43012, 7088,  16364, 43980, 5956,  2344,  64956, 36252, 47468,
                26540, 62292, 52764, 44388, 28060, 11104, 35036, 61760, 25808,
                37884, 19248, 41068, 63744, 3752,  52804, 27196, 48072, 23128,
                4884,  42812, 62676, 31956, 57384, 27956, 42436, 55204, 20892,
                37776, 30308, 46468, 788,   61624, 13124, 53620, 60224, 6900,
                62960, 28136, 1428,  56888, 11160, 60908, 43284, 38852, 26788,
                54452, 14372, 2372,  60972, 25456, 43204, 20432, 51484, 28152,
                9720,  64792, 13276, 4084,  57332, 20276, 5708,  59164, 10076,
                16584, 25264, 2200,  12968, 44644, 54312, 10288, 49892, 30144,
                14128, 27836, 17936, 5392,  45400, 55460, 3448,  20552, 39788,
                49844, 64960, 35496, 17600, 60160, 43240, 10476, 39976, 36408,
                26360, 14708, 62764, 41428, 58520, 5456,  44592, 49564, 15432,
                42708, 55364,
            },
            {
                8504,  28168, 18528, 55680, 42044, 63608, 12468, 46728, 59080,
                5848,  56308, 616,   62868, 54436, 7340,  45840, 60068, 14688,
                52512, 30356, 35280, 51032, 27124, 41448, 30204, 14468, 56044,
                39808, 164,   8656,  24416, 57448, 5096,  17856, 53600, 43792,
                4152,  49392, 14980, 28224, 57600, 12324, 41996, 59016, 33156,
                38352, 19228, 26232, 552,   52620, 35568, 7244,  61776, 25972,
                49484, 12408, 17396, 51020, 24104, 32872, 19400, 9764,  35396,
                21044, 15492, 44496, 24052, 49076, 5856,  29244, 356,   51544,
                8064,  31368, 39800, 50864, 16512, 37584, 55376, 14084, 46704,
                39112, 21376, 49208, 25360, 33512, 53536, 45352, 27232, 64320,
                34204, 53064, 40184, 29156, 65516, 3652,  38100, 21588, 35328,
                46176, 57312, 9856,  35836, 22092, 26996, 42584, 10304, 60880,
                24808, 13792, 8724,  21332, 46896, 6380,  32372, 64524, 104,
                51284, 55620, 34424, 2620,  52156, 12648, 31780, 62524, 3232,
                24252, 36732,
            },
            {
                44220, 64932, 39904, 1360,  15628, 29024, 51476, 17948, 24888,
                35820, 30220, 47284, 23872, 42456, 18152, 34928, 38976, 1012,
                20108, 62016, 11152, 23396, 60336, 57220, 9476,  22192, 4704,
                43340, 17680, 59428, 32996, 39116, 48280, 21864, 13032, 29364,
                8776,  61076, 23704, 5908,  37648, 21980, 7728,  16452, 63492,
                10468, 49628, 58524, 14532, 23372, 45644, 18548, 32404, 1644,
                41140, 64492, 34960, 8100,  39836, 55936, 2556,  26960, 45636,
                52968, 31188, 39140, 63936, 33900, 21772, 57632, 13132, 36120,
                46032, 62340, 21320, 6420,  58768, 10480, 62636, 24364, 1632,
                31616, 6232,  36184, 62052, 10904, 15572, 2768,  22448, 49056,
                7040,  20008, 45880, 8740,  16192, 24312, 42972, 12736, 62384,
                3084,  40732, 51188, 65064, 15688, 49216, 59056, 30060, 5356,
                47336, 32696, 53404, 28084, 3428,  55920, 24436, 13816, 46368,
                20620, 8300,  43612, 17008, 28884, 21976, 54788, 38452, 27688,
                58000, 12292,
            },
            {
                3916,  59140, 34092, 25700, 6212,  57252, 34736, 8648,  41312,
                65300, 19816, 14172, 37804, 28196, 4216,  50052, 25416, 9384,
                47676, 40460, 54968, 3112,  18140, 38704, 34052, 52208, 61692,
                28992, 35304, 50736, 14696, 3256,  30816, 56376, 63828, 36208,
                45996, 55880, 31736, 50664, 44436, 62104, 30896, 51784, 2448,
                29100, 44192, 7876,  40788, 63328, 11808, 50576, 59168, 15740,
                54036, 4748,  22216, 44244, 58824, 15264, 47476, 65048, 41572,
                5412,  58576, 9284,  3484,  14736, 40712, 47580, 65176, 23572,
                18040, 11320, 49360, 27620, 45104, 30804, 4976,  35292, 59828,
                52928, 57696, 18132, 46396, 39408, 29160, 60976, 41644, 37252,
                13652, 57440, 32752, 50604, 59132, 31256, 49156, 56596, 8108,
                19356, 25352, 31380, 236,   38368, 7556,  53232, 18252, 37364,
                56648, 676,   42752, 39224, 63084, 35004, 51812, 29204, 41244,
                58920, 30836, 24696, 48740, 65036, 10228, 46440, 17952, 7620,
                51300, 47660,
            },
            {
                16372, 22960, 52552, 48368, 61144, 21284, 44068, 1680,  54196,
                11300, 44844, 52680, 8264,  62080, 56872, 12128, 63892, 53920,
                32536, 4772,  28324, 44352, 12844, 48260, 7204,  16208, 24744,
                47812, 11556, 20728, 65504, 45808, 9680,  41392, 25196, 1472,
                16932, 20508, 10864, 34708, 80,    18572, 46348, 39888, 24580,
                56248, 34660, 17352, 27672, 37456, 3376,  29652, 39164, 9268,
                47160, 27296, 31404, 52152, 3988,  28960, 36988, 11852, 23236,
                17380, 47936, 25676, 51752, 55700, 18704, 7392,  32264, 2928,
                59424, 42292, 34276, 1072,  53364, 19188, 41860, 48404, 15996,
                9172,  41012, 22964, 856,   54556, 8044,  50412, 17860, 30524,
                204,   61936, 22784, 4208,  36612, 18368, 1032,  27400, 33312,
                52860, 43844, 60676, 12936, 46624, 24112, 33880, 12100, 44636,
                21612, 15200, 59228, 10680, 19396, 15764, 8548,  61476, 3884,
                11912, 38096, 60680, 6480,  39884, 33296, 544,   59500, 34716,
                20724, 30768,
            },
            {
                38672, 7208,  13476, 31424, 11136, 37964, 50044, 26892, 32444,
                22860, 60320, 2160,  33684, 47876, 15728, 30404, 43488, 22932,
                17064, 58096, 37132, 64312, 25488, 31812, 62812, 45372, 2320,
                55616, 7504,  40756, 24076, 58188, 18472, 51996, 7148,  49028,
                38872, 59072, 42072, 53336, 14280, 26900, 59704, 11380, 6036,
                20496, 52980, 46720, 60560, 6324,  54952, 20204, 43628, 24412,
                35948, 62716, 18332, 10324, 60496, 20652, 49896, 32452, 624,
                62296, 34416, 12600, 29684, 36472, 44932, 26504, 53540, 38732,
                28984, 55348, 15392, 23824, 37160, 13088, 65072, 21880, 26620,
                44796, 29588, 13640, 64448, 26224, 35256, 57100, 11824, 44988,
                53656, 27456, 42328, 10868, 55036, 63300, 39984, 46460, 59728,
                15956, 5372,  21492, 54948, 28408, 62612, 41408, 4248,  64420,
                27404, 50760, 24212, 31388, 49248, 44504, 37328, 22024, 54940,
                18388, 45772, 1696,  15260, 56464, 26648, 43300, 14212, 63716,
                41120, 55092,
            },
            {
                27212, 62856, 46020, 42700, 18256, 4564,  62412, 13748, 57532,
                6176,  39404, 17880, 25580, 41060, 21256, 2856,  36124, 7168,
                46572, 13904, 20884, 49572, 572,   56704, 19592, 40132, 36168,
                60868, 31508, 52960, 1104,  28468, 35392, 13336, 32436, 62836,
                28336, 3464,  24760, 64344, 8268,  55144, 35928, 48864, 32672,
                65152, 12944, 1204,  22780, 48600, 33836, 64356, 12848, 58228,
                5428,  56092, 13812, 38320, 42612, 25484, 7344,  57264, 38892,
                54292, 43260, 20308, 61296, 1196,  58784, 10604, 16428, 50264,
                5508,  9492,  63880, 47348, 58180, 8276,  54752, 2460,  34016,
                60824, 3876,  51904, 33000, 43584, 20404, 4720,  24512, 64096,
                6376,  38324, 15236, 47312, 25800, 7460,  14152, 23012, 11016,
                38232, 50012, 34640, 9196,  17820, 2564,  56932, 31060, 48008,
                8472,  34856, 61664, 5928,  57764, 1292,  25844, 47796, 33668,
                27552, 52092, 35724, 23300, 49828, 18924, 53312, 5620,  25016,
                10516, 1936,
            },
            {
                18752, 35772, 54164, 152,   25384, 55396, 30192, 46476, 16648,
                36140, 50876, 29500, 58816, 10460, 61132, 50468, 55712, 27092,
                39716, 60200, 5924,  35080, 10072, 52588, 29108, 4340,  12744,
                27012, 17332, 44032, 37632, 5256,  60428, 45052, 21264, 54424,
                10060, 15552, 47908, 30312, 39192, 22156, 3252,  42764, 15580,
                58412, 38160, 42096, 31004, 15784, 8496,  26236, 50904, 156,
                21304, 44808, 49240, 33536, 1840,  64004, 52716, 19000, 14652,
                27912, 5124,  50792, 8424,  41564, 22992, 63152, 34760, 43996,
                21052, 25664, 40136, 4144,  32424, 29044, 39752, 50608, 12216,
                38164, 56656, 48812, 10036, 15788, 59524, 48136, 39992, 32380,
                21400, 51872, 60748, 19236, 33736, 43308, 53204, 30624, 3696,
                64436, 42488, 29552, 58956, 39968, 51668, 22908, 14156, 19864,
                54928, 2024,  40208, 13428, 28500, 41692, 65360, 6340,  14080,
                57968, 10148, 63984, 31892, 7880,  62044, 36360, 30176, 48472,
                44344, 58832,
            },
            {
                49676, 22112, 9564,  58300, 33948, 40292, 20136, 7544,  42028,
                63916, 4016,  45944, 53476, 6600,  37676, 32684, 18788, 64816,
                10768, 51884, 30260, 42504, 22988, 15024, 43768, 59776, 21688,
                48992, 9328,  62752, 14952, 49444, 55416, 26228, 4388,  42652,
                35984, 51640, 33488, 5496,  45792, 17560, 62328, 29208, 23512,
                4784,  26644, 10212, 51648, 57420, 44920, 36952, 17116, 41288,
                32544, 28340, 6908,  54908, 16652, 29844, 44432, 9376,  46628,
                59324, 24184, 37560, 16840, 31852, 48868, 3956,  13000, 56628,
                30976, 61008, 51224, 18588, 44648, 22748, 16396, 61840, 7612,
                20744, 17456, 24912, 6020,  37484, 29860, 1804,  54716, 14528,
                9964,  35084, 1556,  29696, 57268, 4792,  48884, 36096, 56064,
                19004, 24060, 1512,  14524, 46136, 36128, 6868,  60816, 42592,
                37800, 17496, 46384, 53128, 20900, 55796, 16704, 50652, 39648,
                4740,  42264, 20500, 45056, 13052, 40648, 2892,  21512, 56916,
                14892, 32400,
            },
            {
                4092,  41344, 29360, 15744, 47716, 64680, 10340, 52904, 24564,
                28172, 12332, 20756, 15028, 42864, 24000, 60,    13168, 44492,
                3644,  24752, 16516, 55808, 38132, 63428, 7600,  32916, 53976,
                41812, 56732, 23468, 33700, 20072, 11168, 39008, 16728, 61848,
                23736, 56908, 20060, 60012, 12004, 51340, 56608, 9540,  50252,
                45544, 54856, 34356, 19900, 62264, 2588,  29228, 60504, 53356,
                9788,  59548, 24204, 62132, 11652, 40312, 22368, 34680, 3556,
                31472, 11352, 64920, 45536, 57460, 28388, 39644, 19404, 47112,
                1752,  36044, 14224, 10780, 62860, 592,   47764, 27320, 42952,
                31528, 46636, 63732, 41376, 52916, 61968, 19068, 46180, 26744,
                59708, 41468, 49688, 8676,  65000, 21556, 16332, 60872, 27944,
                8596,  48460, 54216, 62392, 26056, 11196, 32484, 49820, 28308,
                9956,  63776, 23464, 33356, 10992, 35840, 8244,  31592, 24056,
                60608, 29544, 2328,  54684, 27188, 59736, 10860, 52572, 37548,
                7776,  65452,
            },
            {
                12472, 52892, 61240, 6612,  23164, 2560,  37428, 60164, 48676,
                900,   35088, 56412, 31760, 63460, 48020, 57736, 29148, 54012,
                34108, 47448, 62188, 1732,  50408, 26752, 46852, 18008, 36872,
                1400,  5592,  29764, 46292, 7356,  64700, 31732, 428,   46740,
                8172,  13912, 2132,  43728, 24452, 35160, 796,   31936, 39448,
                17812, 63884, 3516,  14116, 24864, 40476, 6616,  22320, 47712,
                14612, 38512, 18772, 46820, 36288, 5020,  58396, 48720, 62496,
                41216, 51776, 1700,  21500, 14068, 6712,  53376, 24400, 59516,
                7788,  43208, 27980, 57020, 33480, 52796, 36608, 56340, 4468,
                59744, 35380, 3000,  27784, 22412, 12424, 8148,  35976, 56612,
                3536,  17796, 45120, 24072, 38536, 11324, 41332, 408,   45524,
                12816, 37304, 31532, 5696,  18276, 56372, 45040, 3504,  24840,
                58216, 5024,  30048, 48748, 2388,  62652, 45608, 19060, 53524,
                47176, 15644, 37284, 49496, 17788, 33576, 46620, 24340, 17000,
                28520, 39436,
            },
            {
                25728, 17928, 45020, 35540, 51344, 27572, 13328, 32736, 17348,
                43700, 61996, 40060, 4496,  26748, 8352,  16908, 41252, 21560,
                38492, 8640,  19444, 31900, 13028, 5192,  57672, 10624, 25532,
                64184, 13832, 39284, 58492, 52380, 42384, 27920, 50304, 58324,
                40408, 30104, 37624, 65024, 27848, 41336, 14784, 21280, 59684,
                7756,  28344, 43432, 37376, 49036, 55300, 12164, 34456, 65524,
                4788,  43364, 1356,  51136, 27344, 53604, 13452, 25724, 20056,
                16344, 55040, 26392, 36024, 50132, 33208, 63584, 10448, 37688,
                17136, 54340, 23444, 41232, 6520,  20292, 9448,  13908, 23768,
                51660, 10700, 14880, 55772, 44688, 33464, 50636, 42500, 22832,
                31088, 62968, 13696, 27756, 54848, 32216, 52364, 25008, 34212,
                63704, 21172, 43868, 51208, 39428, 21748, 65196, 16304, 52224,
                36260, 44212, 15104, 60288, 41352, 25452, 58368, 38696, 444,
                10796, 63368, 25112, 7328,  58108, 3896,  64092, 42616, 656,
                60332, 47436,
            },
            {
                56032, 1792,  31288, 11572, 59100, 42444, 56148, 21448, 6780,
                54584, 23152, 10096, 52096, 19608, 36412, 50316, 61528, 5868,
                59712, 27280, 57176, 45468, 35856, 21004, 40748, 51816, 31168,
                45420, 50536, 18652, 25132, 2428,  13260, 21964, 34528, 18448,
                25548, 54224, 48064, 9804,  52424, 6244,  63016, 48624, 36092,
                12684, 52828, 22496, 60092, 9212,  30944, 45392, 18224, 26096,
                55860, 29776, 34004, 21232, 8512,  64652, 32244, 360,   38028,
                7284,  29892, 43960, 9572,  60672, 3628,  41648, 26600, 50720,
                32228, 65528, 4272,  48880, 60316, 26564, 45800, 64668, 41580,
                29432, 19772, 39600, 58932, 332,   17180, 65532, 4420,  11072,
                53092, 37680, 6888,  58532, 2740,  18600, 46900, 6732,  57416,
                16912, 2780,  59692, 9940,  29328, 2084,  34400, 41280, 12232,
                20368, 55288, 8104,  51364, 18164, 4344,  13360, 28912, 22328,
                35112, 56124, 31204, 43892, 21716, 38956, 14376, 31608, 50796,
                20524, 34312,
            },
            {
                41928, 22512, 63852, 38528, 19200, 5168,  46240, 30232, 38220,
                49964, 15372, 29252, 58536, 44744, 12012, 33328, 2528,  14960,
                48976, 11416, 41792, 3784,  65060, 29048, 60580, 15256, 6968,
                21932, 34068, 61880, 9852,  36456, 56088, 61168, 6068,  10828,
                62608, 4584,  16600, 22176, 57860, 18656, 44980, 25632, 56036,
                4920,  47000, 33008, 1100,  16160, 63032, 51820, 1928,  39508,
                49804, 11356, 61412, 57556, 14816, 41584, 44680, 49644, 56948,
                61312, 12908, 39056, 56524, 17072, 20572, 46116, 14644, 272,
                44100, 20996, 12792, 34976, 15976, 30556, 38580, 18292, 1368,
                49092, 54060, 8008,  31104, 48316, 25920, 38600, 29052, 60628,
                20252, 47860, 33776, 50460, 43016, 60268, 14300, 29004, 38760,
                49776, 41616, 26816, 15196, 61652, 47500, 8604,  59336, 30860,
                140,   39532, 22724, 27752, 37236, 32460, 54920, 44276, 50124,
                6464,  40296, 13692, 52400, 10012, 28700, 53964, 5672,  57552,
                11020, 6920,
            },
            {
                15440, 53784, 9036,  48296, 26012, 15952, 63392, 9468,  59756,
                3808,  34436, 47044, 1280,  39288, 65420, 22676, 53248, 31492,
                24536, 36884, 54444, 15880, 23568, 48396, 288,   37908, 54392,
                59108, 3476,  41264, 29096, 47984, 15828, 44604, 38204, 51704,
                42880, 32236, 36316, 3080,  28872, 33460, 11224, 2040,  40720,
                30296, 19016, 58036, 42012, 27104, 21428, 36052, 57820, 8292,
                22996, 16616, 45868, 37448, 3608,  23884, 18180, 10672, 35056,
                21796, 46816, 5272,  24016, 48568, 30532, 35508, 55780, 61640,
                29412, 8932,  53136, 40036, 2232,  55336, 51056, 5992,  33116,
                62136, 24288, 36584, 60948, 21204, 6752,  52360, 14776, 44476,
                25380, 816,   16816, 10152, 22096, 35844, 62680, 9632,  53796,
                23476, 4900,  33180, 52752, 36508, 23332, 54732, 26428, 49316,
                63188, 33732, 57184, 11984, 64852, 48108, 9348,  61616, 16624,
                59004, 19948, 2664,  46952, 60072, 35028, 18416, 25952, 44564,
                36824, 62112,
            },
            {
                50096, 27488, 3552,  57776, 33820, 436,   50624, 40728, 24476,
                12252, 62556, 18004, 25184, 7604,  55096, 28296, 42548, 45912,
                63096, 18736, 6836,  33708, 52240, 9500,  42804, 19144, 27372,
                11980, 44520, 17144, 5232,  53456, 32488, 26184, 1636,  21192,
                14448, 56632, 45824, 60600, 49852, 38652, 54376, 61656, 22940,
                14204, 64900, 7944,  51044, 38660, 4656,  13448, 28856, 42808,
                63840, 32592, 6320,  52608, 28312, 55016, 30648, 63124, 4332,
                27720, 52992, 33032, 64612, 2128,  58628, 11360, 6316,  23020,
                37420, 58068, 46296, 25156, 63500, 11520, 22700, 59488, 13188,
                44092, 16420, 3668,  45884, 12996, 41728, 57492, 32248, 8460,
                55792, 40880, 64568, 30492, 25984, 39928, 3356,  20392, 31724,
                12240, 58800, 45756, 19860, 12980, 43592, 5896,  18720, 14844,
                44584, 7012,  46520, 2648,  41892, 20976, 5512,  24976, 33024,
                42824, 27924, 65108, 23888, 4892,  41444, 63076, 49108, 13292,
                23356, 30180,
            },
            {
                45916, 32920, 40384, 21352, 44156, 54524, 35892, 19556, 27988,
                43444, 55540, 37784, 31212, 60108, 13796, 4112,  17748, 9692,
                808,   56584, 27996, 44888, 59456, 13592, 31652, 63484, 35604,
                49720, 24636, 57476, 64260, 22656, 8372,  59532, 49320, 64776,
                27812, 7916,  24356, 12412, 42248, 15840, 8400,  31748, 43928,
                50100, 26376, 34752, 11336, 56136, 44340, 60756, 18700, 53520,
                3292,  25560, 40864, 19776, 47760, 59784, 7820,  40212, 50852,
                14512, 41744, 18324, 8976,  43124, 25832, 51008, 40260, 18460,
                48240, 3524,  16904, 31984, 42908, 7784,  36688, 47132, 26440,
                40284, 9504,  28828, 55124, 34072, 63540, 1920,  19204, 49708,
                35104, 12932, 46008, 58088, 51616, 6296,  55836, 48424, 42440,
                64836, 37096, 8088,  56272, 1616,  64152, 38980, 29800, 53444,
                10812, 24388, 17912, 31428, 58892, 15192, 52724, 39264, 1124,
                11836, 51724, 37352, 31980, 14992, 55384, 8580,  1340,  39340,
                59216, 4260,
            },
            {
                65052, 17872, 13768, 60360, 10664, 29248, 14260, 58708, 2360,
                48500, 21780, 9188,  50888, 41560, 20592, 49560, 36200, 58448,
                40100, 50140, 21468, 38208, 2540,  25540, 55888, 47012, 8156,
                2064,  39332, 31116, 13060, 37480, 42032, 18616, 11560, 31144,
                40516, 53264, 19700, 63256, 220,   26924, 59196, 17996, 5336,
                38116, 2952,  45680, 17524, 24132, 31456, 47596, 9808,  34584,
                49280, 14104, 58316, 11120, 988,   33656, 16488, 46184, 25228,
                1764,  60728, 54804, 28848, 37912, 15720, 62316, 32812, 54532,
                27900, 13848, 60064, 20580, 49556, 28460, 57288, 19420, 2972,
                56204, 64036, 48672, 18220, 25120, 10248, 37756, 23308, 59252,
                5728,  28272, 20832, 2480,  15488, 43800, 33416, 17528, 27464,
                684,   15836, 29476, 49400, 25744, 32724, 57872, 3708,  41764,
                35584, 59768, 50004, 54624, 26208, 36368, 29396, 63680, 47576,
                57164, 7920,  17628, 49704, 45016, 21344, 27760, 32336, 53252,
                19612, 9580,
            },
            {
                51912, 37136, 6284,  49180, 24172, 64080, 7708,  52140, 32860,
                6552,  64264, 15528, 2904,  34000, 46708, 63780, 29984, 25144,
                14208, 32232, 11068, 64400, 17200, 40532, 5404,  22464, 15992,
                60828, 54704, 19544, 51452, 496,   45652, 55804, 34644, 2976,
                47068, 5264,  38272, 34008, 51960, 21776, 45556, 35560, 53192,
                62832, 20828, 60020, 52656, 364,   64396, 6040,  22600, 38796,
                59992, 29572, 44240, 36136, 62492, 22484, 12720, 37324, 57260,
                20488, 35556, 11460, 48924, 4160,  21700, 44556, 1184,  9732,
                64560, 34604, 5612,  38992, 692,   61856, 14724, 33704, 51952,
                30288, 21804, 38664, 5692,  51428, 43780, 29356, 47224, 16580,
                39616, 61256, 53720, 37700, 9052,  63624, 23068, 57744, 7164,
                51796, 61044, 20984, 40904, 10156, 16992, 48284, 21920, 61952,
                27500, 1760,  38304, 12852, 5004,  43720, 10204, 18688, 22580,
                34484, 25932, 60812, 1780,  10900, 58152, 36072, 47836, 15752,
                43504, 25648,
            },
            {
                776,   42492, 56752, 31148, 2880,  37888, 41916, 17152, 46208,
                36776, 29828, 52720, 58144, 27004, 11384, 4952,  53916, 7516,
                61928, 47520, 4600,  53672, 30116, 50808, 61736, 33524, 44008,
                28632, 10064, 35000, 47396, 7156,  27100, 14860, 23056, 61324,
                57844, 17108, 29944, 10428, 48088, 6784,  29300, 57520, 13160,
                9676,  28536, 32712, 14460, 36940, 41180, 27264, 55240, 16144,
                1660,  21152, 7216,  51652, 26324, 41520, 53900, 65288, 31360,
                6832,  45004, 23420, 63316, 56680, 30940, 13380, 52728, 25284,
                41804, 45628, 56108, 51584, 24456, 10208, 45288, 40832, 7032,
                12272, 35544, 600,   61116, 14976, 56768, 7104,  62300, 52492,
                3804,  10984, 24064, 31620, 49244, 28740, 12836, 46692, 38624,
                24948, 43956, 33960, 54548, 62444, 37312, 7428,  13228, 51084,
                19104, 9408,  63384, 22184, 48028, 60988, 2876,  55928, 45560,
                13588, 40520, 53772, 29492, 38688, 64372, 24332, 3360,  61876,
                57172, 34268,
            },
            {
                22432, 61068, 26976, 18844, 46848, 53644, 22104, 26280, 60892,
                13004, 24120, 40320, 18592, 44268, 22840, 38764, 16764, 43556,
                20456, 35240, 26592, 42576, 8996,  36660, 19844, 11716, 52128,
                41236, 3920,  23796, 60040, 29952, 63932, 52336, 37080, 8652,
                25816, 43772, 55064, 59600, 14752, 65408, 39432, 1508,  24576,
                47592, 42288, 56664, 8208,  48916, 19896, 12108, 50932, 45520,
                63280, 33220, 56268, 17764, 48760, 5712,  3068,  28212, 10332,
                52272, 39748, 16540, 33720, 6368,  46904, 59788, 36992, 20768,
                6572,  17772, 11876, 29508, 36228, 54620, 18480, 64944, 25092,
                59240, 43468, 53684, 26916, 32308, 21260, 35752, 13068, 25880,
                42192, 34128, 45132, 16928, 55456, 40232, 2100,  34824, 10044,
                59592, 13320, 5484,  2396,  46068, 23648, 30336, 43724, 56220,
                33344, 45940, 30672, 40616, 17420, 33808, 51200, 30936, 6744,
                62472, 3600,  20908, 43740, 5528,  18172, 12920, 41044, 6756,
                29880, 11588,
            },
            {
                15308, 8200,  54400, 12428, 33332, 61920, 9828,  908,   56868,
                43172, 4076,  10472, 61752, 460,   55180, 59924, 33124, 52076,
                1688,  57760, 15792, 60400, 23312, 48836, 944,   58356, 25860,
                65400, 17480, 56624, 13836, 39872, 20024, 4164,  42340, 49552,
                12956, 20924, 2192,  35780, 23388, 42796, 18848, 33280, 50788,
                16288, 38556, 3636,  25864, 58720, 30964, 61976, 35176, 4360,
                24508, 13056, 39348, 9548,  31028, 59564, 43760, 17360, 49352,
                60304, 160,   27584, 53448, 40572, 19044, 2832,  28508, 49760,
                57780, 32548, 62720, 22292, 43428, 4484,  31816, 2072,  49480,
                15636, 19768, 46676, 11148, 40412, 50360, 1648,  45980, 30852,
                19672, 63976, 100,   59940, 6628,  20088, 52144, 62576, 21852,
                30536, 50824, 19160, 28124, 14872, 52440, 60080, 940,   39132,
                5432,  15176, 57724, 7176,  53944, 24820, 14576, 41664, 27336,
                36800, 48792, 16064, 57016, 33932, 46724, 51864, 55128, 20156,
                38092, 50416,
            },
            {
                40760, 47980, 35520, 44456, 5656,  15984, 39856, 30668, 49344,
                20212, 54324, 31824, 48272, 35208, 28544, 13376, 8940,  24588,
                40216, 46080, 12464, 55684, 4080,  28708, 45348, 14936, 7408,
                31016, 37180, 44696, 50404, 9420,  33580, 57124, 16660, 31772,
                63580, 39172, 46400, 26828, 4488,  53548, 11648, 62012, 55740,
                5768,  22248, 63948, 44956, 17324, 2272,  43564, 9216,  28796,
                42360, 52880, 47016, 64176, 21756, 13808, 38428, 24620, 34492,
                19940, 13256, 58200, 8176,  24800, 11604, 43304, 62180, 14580,
                39368, 1456,  48168, 7700,  15560, 61252, 47328, 27376, 56356,
                34228, 8324,  61944, 3560,  58168, 17212, 65212, 9264,  57548,
                54480, 14196, 47460, 35460, 26380, 57960, 15100, 41624, 4364,
                48048, 37172, 57320, 65352, 40000, 34912, 10504, 26888, 20864,
                64532, 24136, 49712, 28316, 516,   37612, 65312, 20256, 10424,
                59156, 32368, 23716, 8304,  62772, 26000, 31140, 9204,  44760,
                27244, 63572,
            },
            {
                18444, 4436,  23320, 64744, 28144, 51580, 59312, 35636, 5128,
                27484, 65140, 38336, 16456, 6684,  45408, 19716, 50032, 64844,
                29012, 6884,  31896, 37820, 18272, 63888, 34240, 39328, 55388,
                48544, 1448,  21828, 5348,  26408, 62600, 46660, 1316,  28188,
                6676,  51308, 9888,  61404, 30972, 49432, 8076,  26416, 44548,
                30748, 35676, 51692, 10456, 53632, 39136, 23072, 56548, 59624,
                16632, 5536,  26688, 580,   35364, 45696, 55848, 7984,  63684,
                47196, 43092, 36164, 50628, 64508, 32044, 54864, 35060, 8808,
                23304, 27504, 53756, 40672, 58272, 20736, 37400, 10788, 39456,
                22400, 42160, 30916, 24200, 37332, 28512, 43188, 23184, 38284,
                5036,  28424, 8788,  21668, 43536, 11044, 32588, 27704, 55156,
                16848, 8736,  43076, 22380, 6932,  55820, 17808, 46844, 51448,
                35344, 42332, 11204, 59956, 45032, 8340,  56944, 47248, 1584,
                42860, 53432, 12168, 49992, 38296, 232,   14916, 60704, 34992,
                1996,  56520,
            },
            {
                31992, 59092, 38656, 1412,  20688, 42640, 11432, 18260, 45588,
                13956, 8444,  51236, 25568, 57728, 62996, 41300, 2476,  36188,
                17304, 61000, 22236, 52384, 41840, 10620, 51220, 24196, 20152,
                11972, 62160, 53296, 35796, 43176, 12520, 24028, 54504, 36340,
                58336, 18696, 33860, 55996, 16696, 40288, 36496, 20708, 2808,
                60752, 13280, 18904, 29084, 33528, 6656,  14416, 49548, 20244,
                32476, 61740, 37616, 57644, 18552, 28496, 51328, 2308,  31860,
                22528, 4196,  29616, 15168, 20548, 48140, 4592,  17188, 51324,
                44812, 65372, 13412, 34184, 25684, 51772, 5204,  14284, 53220,
                63652, 5492,  51216, 13840, 54896, 6352,  48252, 33340, 16104,
                49528, 41108, 52312, 62944, 37656, 50176, 6044,  44964, 64072,
                576,   25280, 33104, 12188, 49212, 3172,  31848, 61260, 13532,
                2744,  55268, 18872, 32508, 21896, 13012, 34560, 52628, 17644,
                26132, 64032, 4176,  28832, 18864, 42152, 56072, 22608, 48596,
                6252,  24908,
            },
            {
                45412, 13820, 29664, 55276, 48724, 7616,  24668, 62872, 56020,
                33508, 42132, 20816, 3264,  11608, 31064, 23228, 56168, 47244,
                11788, 54276, 88,    47656, 26388, 5380,  61192, 2416,  42848,
                32952, 27712, 16012, 58164, 30596, 18280, 50832, 9000,  21616,
                41912, 14272, 44348, 740,   22428, 64340, 46948, 14948, 57620,
                41184, 7720,  47852, 59244, 25060, 64732, 46304, 36284, 1540,
                40884, 12248, 23324, 49264, 6596,  11576, 62468, 16148, 40988,
                11112, 54264, 61608, 38168, 1592,  41380, 26264, 60576, 30008,
                5408,  36836, 19116, 3268,  9848,  31192, 45656, 59024, 29068,
                1132,  17592, 47384, 35100, 62700, 19812, 12264, 60912, 2700,
                59104, 24448, 31284, 17720, 1504,  60616, 23192, 18944, 35204,
                39620, 53460, 59316, 28756, 62200, 44152, 24376, 41168, 8576,
                29876, 38256, 4472,  48668, 61556, 39720, 29272, 23136, 6016,
                39020, 14428, 35912, 45240, 61180, 51152, 11748, 37068, 17236,
                53424, 10912,
            },
            {
                62516, 51412, 8500,  17060, 58472, 32124, 37928, 52464, 1724,
                23160, 59052, 46780, 36596, 53008, 39248, 15336, 4688,  32892,
                25928, 43472, 8820,  33772, 14632, 58488, 29840, 17364, 56796,
                7740,  47108, 40916, 3440,  61280, 6444,  38840, 45244, 65096,
                4088,  25396, 60148, 29284, 12448, 6160,  54836, 32852, 28040,
                50272, 23920, 37520, 504,   55308, 41524, 10872, 27144, 54904,
                51268, 8228,  30136, 42508, 53948, 32976, 38908, 26060, 57828,
                50324, 6556,  24188, 45676, 59044, 9868,  55768, 21968, 40088,
                11736, 58960, 46492, 56716, 41896, 63004, 17848, 24304, 43148,
                33452, 56892, 26792, 9332,  40824, 508,   51976, 27612, 20932,
                36008, 11524, 6392,  54080, 13780, 29436, 47940, 56160, 9784,
                14096, 46392, 19736, 5520,  15464, 37512, 20436, 53244, 57996,
                17004, 62824, 44284, 26672, 15968, 2440,  50088, 63148, 45828,
                55812, 30832, 58592, 21584, 7048,  25400, 3420,  30212, 65192,
                42744, 34120,
            },
            {
                3224,  40508, 26628, 35424, 46156, 4596,  14692, 27860, 40448,
                7020,  30128, 16784, 61336, 27680, 9092,  44928, 61836, 51312,
                18856, 39040, 64128, 55844, 20340, 37156, 49504, 45604, 35172,
                64684, 24540, 10256, 21228, 48144, 34332, 56992, 28860, 11960,
                32356, 53188, 48652, 38736, 51988, 24832, 43132, 1800,  10220,
                17140, 63532, 43988, 6488,  20668, 15932, 31308, 3868,  60584,
                17828, 45820, 65056, 15292, 20916, 3724,  60232, 44540, 19240,
                35216, 28356, 12704, 18168, 31012, 34348, 13920, 46976, 632,
                52948, 31944, 22912, 26968, 50208, 1888,  36444, 8184,  49420,
                12140, 20944, 45172, 59760, 23448, 30680, 44280, 38916, 55224,
                46548, 64664, 42656, 34488, 58724, 39168, 3372,  26244, 61136,
                30756, 3952,  50528, 34828, 55068, 10004, 48012, 96,    27452,
                34156, 22804, 52700, 9584,  58276, 42944, 7512,  33248, 11328,
                19996, 2260,  48380, 10112, 34236, 40704, 58020, 47212, 7688,
                27368, 20412,
            },
            {
                48928, 22892, 61472, 12036, 41952, 64244, 21388, 49940, 10800,
                62232, 54776, 12964, 644,   49768, 57128, 21648, 7008,  29136,
                59076, 3040,  23612, 30584, 41372, 6680,  12364, 22500, 4032,
                14516, 51168, 37048, 54972, 13728, 25512, 588,   16524, 60632,
                37036, 19796, 5440,  9628,  34948, 59360, 19576, 62528, 39684,
                54172, 30968, 12524, 34220, 51924, 47276, 62788, 21792, 34688,
                24984, 38500, 2352,  59148, 35804, 47100, 23584, 8588,  664,
                48532, 65156, 56236, 42352, 52160, 5664,  63656, 36492, 18808,
                62008, 16360, 6704,  38620, 14592, 54632, 21832, 60264, 39216,
                65076, 6224,  37496, 15436, 54192, 7300,  63528, 14608, 9152,
                3848,  18836, 27084, 45740, 21196, 8952,  42204, 16668, 52232,
                43964, 23516, 63620, 41576, 25888, 30416, 64784, 6816,  39820,
                50152, 14140, 5280,  36224, 20656, 53852, 27808, 16940, 59872,
                41852, 51888, 26972, 16172, 54020, 62280, 18484, 13504, 39376,
                56772, 15396,
            },
            {
                37228, 6012,  54356, 18876, 92,    30280, 53836, 34868, 45416,
                19728, 36016, 43532, 24036, 32584, 17700, 35268, 42368, 12820,
                36928, 16116, 45476, 10884, 51564, 63052, 27872, 54440, 39160,
                30764, 60252, 2268,  28604, 63628, 40464, 51384, 22568, 47388,
                2896,  41604, 63120, 17464, 46168, 31276, 14120, 48784, 35920,
                4968,  21484, 59848, 27340, 57376, 4776,  38112, 43816, 7308,
                13092, 56060, 27468, 10040, 50896, 29336, 13400, 55636, 31216,
                15516, 39656, 3300,  8708,  26840, 48716, 23500, 43404, 27608,
                9208,  42216, 48228, 64416, 11024, 33988, 28008, 4240,  16220,
                29648, 51408, 2992,  32556, 48932, 18404, 34648, 25208, 49680,
                32332, 56984, 15364, 1940,  50640, 32964, 64204, 36540, 6812,
                11740, 38312, 17328, 2436,  57000, 13268, 45096, 18952, 59716,
                10740, 46380, 64388, 41104, 31720, 12220, 61964, 38032, 25096,
                4864,  36328, 64828, 44160, 848,   28980, 22712, 33012, 51956,
                1220,  59996,
            },
            {
                44324, 9736,  32804, 47344, 25036, 59964, 8252,  17252, 2708,
                26220, 49296, 5488,  39712, 63044, 3892,  47552, 65424, 25028,
                53564, 49448, 4992,  60636, 18312, 1464,  43904, 58936, 8476,
                19560, 42232, 45860, 17616, 32728, 7432,  43616, 10048, 56120,
                30796, 54256, 23600, 28236, 57040, 3492,  8152,  27148, 23496,
                51508, 45896, 9048,  40784, 18612, 13712, 29032, 59028, 52548,
                48264, 31964, 44420, 18580, 4432,  63748, 41540, 37084, 61360,
                22212, 46140, 33148, 20408, 61220, 15980, 2532,  57592, 51180,
                32928, 55544, 3500,  29928, 19732, 44424, 57792, 47664, 53516,
                24836, 42228, 61368, 10816, 27932, 58084, 1980,  41956, 61284,
                22404, 37580, 52736, 61864, 24548, 12336, 54736, 20188, 57432,
                27532, 48980, 33316, 9004,  51464, 22548, 36276, 32416, 54088,
                25044, 29092, 1776,  56740, 24004, 48156, 396,   45156, 54876,
                9060,  22476, 31656, 12568, 37468, 49536, 4392,  45660, 10464,
                24600, 31008,
            },
            {
                63576, 27520, 57908, 51540, 14232, 38876, 43320, 56456, 63808,
                31464, 58016, 9728,  53164, 27284, 13900, 55260, 10344, 956,
                32132, 58160, 27324, 34772, 39864, 24748, 32432, 15908, 49084,
                26436, 56688, 5784,  11600, 53584, 20172, 58976, 34932, 26764,
                13984, 7772,  44400, 12068, 51000, 37840, 42772, 61184, 56476,
                1056,  16100, 65380, 36464, 2844,  50396, 23948, 9740,  16996,
                1300,  21212, 57912, 39696, 24720, 54204, 17540, 6152,  51940,
                10552, 25620, 59552, 36756, 54040, 39012, 30440, 12624, 6440,
                21132, 35952, 25252, 60556, 51380, 464,   40872, 12788, 8868,
                35308, 19364, 56636, 22776, 38376, 44836, 12668, 52180, 6532,
                29536, 9980,  40792, 6584,  28812, 47640, 4816,  31484, 45584,
                1656,  61584, 20796, 59220, 43636, 5156,  61108, 16224, 2864,
                42844, 20116, 34848, 16708, 51788, 5840,  19436, 34044, 14640,
                58460, 47268, 17728, 59720, 8124,  56572, 41036, 64144, 35720,
                54960, 17344,
            },
            {
                13120, 2304,  39956, 21652, 36028, 3596,  28432, 23144, 12404,
                41076, 15564, 22276, 59920, 19472, 44296, 30352, 38340, 18672,
                41676, 8328,  21316, 14240, 47508, 53428, 10392, 36760, 62484,
                812,   34728, 22852, 62092, 48688, 37652, 4644,  64576, 18560,
                39880, 60796, 36232, 252,   64884, 20972, 17044, 33636, 11904,
                30500, 43028, 25544, 47924, 32080, 55756, 42544, 64296, 35672,
                40932, 62640, 5436,  34596, 48492, 11720, 32816, 27060, 42628,
                2156,  49928, 13660, 4676,  44036, 10028, 64796, 46552, 40860,
                61680, 13636, 45460, 9704,  37724, 23684, 18268, 63592, 31352,
                46248, 1088,  14164, 47544, 4200,  64748, 21000, 36344, 17336,
                47952, 58444, 13432, 34852, 43756, 17196, 59436, 23284, 14356,
                41404, 53660, 30028, 15408, 39708, 28264, 48124, 8584,  38524,
                58176, 49580, 61576, 9320,  39596, 30588, 64100, 41400, 50128,
                29600, 3036,  39416, 52508, 25900, 20040, 15668, 27716, 5384,
                20860, 41876,
            },
            {
                50680, 46060, 17972, 5144,  65356, 10964, 48496, 52252, 34216,
                6372,  47840, 37640, 1772,  34440, 51916, 6136,  60788, 22896,
                48412, 62624, 44116, 56172, 6048,  64936, 3744,  22172, 45140,
                13180, 51892, 39476, 31436, 25340, 15040, 29640, 50204, 2016,
                46644, 52280, 16388, 31960, 25024, 47616, 54512, 5596,  39544,
                19492, 58112, 7100,  12200, 61800, 20492, 5964,  30956, 26140,
                53728, 12156, 29200, 15768, 60492, 964,   44976, 59176, 19868,
                64200, 30572, 56816, 18512, 28048, 21940, 34248, 16876, 24560,
                1768,  53108, 17472, 57084, 6192,  33348, 59264, 27056, 5812,
                55152, 62176, 40308, 33440, 9144,  53216, 30960, 25668, 55748,
                1120,  45268, 19376, 65500, 54112, 328,   40040, 35580, 63132,
                8036,  25220, 36748, 11332, 772,   56028, 25552, 63560, 12708,
                30880, 6460,  14384, 44816, 26984, 59816, 12644, 7016,  26276,
                20720, 63440, 42964, 5732,  34636, 61112, 11784, 53300, 48200,
                59632, 7636,
            },
            {
                34820, 29956, 60940, 54720, 32028, 44528, 59096, 16604, 780,
                55012, 62432, 29460, 46000, 11692, 40764, 25576, 57028, 14956,
                33968, 3852,  12016, 30612, 37344, 28108, 18676, 41196, 58012,
                29384, 17192, 9360,  46804, 2424,  57344, 41452, 12236, 24408,
                33424, 20520, 5756,  58304, 40364, 9392,  29216, 45112, 63360,
                49956, 35220, 53172, 23180, 39268, 44,    45788, 14364, 49112,
                7108,  23268, 46332, 51984, 8992,  38264, 22660, 55916, 14808,
                35456, 40148, 6528,  47488, 52524, 40,    58388, 49100, 55940,
                37404, 31728, 27668, 42408, 49064, 14664, 52348, 38820, 43848,
                16544, 24092, 51132, 26848, 59020, 16120, 43108, 5200,  63136,
                39096, 27472, 22788, 31904, 9064,  25760, 48576, 10772, 50756,
                18244, 3272,  46484, 65028, 49932, 32868, 17276, 45500, 21536,
                53464, 24688, 37572, 55116, 3716,  21708, 53196, 36436, 56344,
                10372, 33072, 13892, 23772, 48856, 30172, 43692, 1488,  31604,
                38540, 25424,
            },
            {
                10512, 52932, 22848, 14772, 25848, 7472,  20284, 37256, 27228,
                42872, 18596, 24848, 8140,  63524, 16744, 49988, 2312,  28932,
                53228, 38636, 17716, 51608, 23100, 59748, 49796, 33192, 54244,
                6172,  64424, 43200, 59620, 7120,  21768, 53952, 44428, 8028,
                62900, 56516, 28140, 49172, 13788, 59892, 1872,  15656, 26128,
                10080, 2668,  28668, 59604, 17184, 33504, 54708, 61092, 18912,
                36944, 59572, 41900, 20092, 64848, 31044, 50516, 3832,  28968,
                8608,  45384, 23616, 32660, 60716, 41820, 8632,  29296, 4904,
                10956, 62584, 20356, 3092,  65292, 29672, 10140, 2188,  21092,
                36560, 11428, 30372, 6700,  19652, 37532, 49592, 11564, 33796,
                15272, 60276, 3124,  51076, 37972, 61456, 20020, 29884, 56228,
                33888, 58408, 42320, 19540, 23404, 6972,  52300, 34900, 41432,
                4408,  48780, 65364, 18748, 34176, 42036, 47724, 16532, 1028,
                45504, 50828, 58140, 2052,  54824, 9352,  36680, 22968, 62360,
                14544, 57812,
            },
            {
                456,   42728, 6204,  35464, 49700, 40940, 62228, 31032, 9652,
                58260, 13096, 53080, 33140, 56416, 21552, 36772, 9412,  45344,
                64472, 26164, 57896, 728,   42024, 7668,  15244, 11772, 3208,
                25956, 36348, 19332, 52640, 28036, 35428, 17032, 60000, 30516,
                37368, 10436, 42644, 3676,  34712, 22576, 52048, 37320, 55972,
                22060, 46252, 14504, 42268, 50540, 8796,  27132, 40396, 10584,
                56492, 1812,  33164, 4836,  26692, 17124, 41316, 12528, 48548,
                53856, 17444, 62628, 11372, 15428, 26092, 36820, 19580, 44260,
                51732, 39784, 47392, 8336,  35708, 22716, 45960, 56436, 60440,
                48840, 3512,  64252, 45404, 55872, 2068,  61628, 23364, 53708,
                8464,  49292, 42496, 12620, 57408, 15592, 3588,  43160, 5464,
                27080, 13116, 52732, 30900, 60696, 38356, 10732, 1864,  56964,
                28120, 11304, 32468, 1348,  58496, 10020, 29312, 24396, 62568,
                37556, 18436, 27864, 38604, 16380, 65220, 47464, 18084, 4848,
                45444, 19756,
            },
            {
                27948, 48104, 38772, 63240, 11764, 55884, 4368,  46748, 50780,
                22644, 39912, 2764,  48560, 5212,  30540, 61164, 42312, 13216,
                4716,  20748, 47736, 10540, 31548, 45540, 61984, 37864, 47236,
                22384, 50392, 13552, 32844, 10592, 63676, 39296, 1148,  51148,
                14796, 23120, 54984, 18732, 46124, 64088, 30564, 43476, 4444,
                62372, 38520, 31240, 5568,  64672, 36332, 21452, 3364,  29852,
                44684, 25136, 13824, 54808, 36308, 57748, 6776,  63020, 37052,
                25500, 1208,  57680, 38936, 4140,  50612, 63912, 13344, 59864,
                23276, 15472, 25948, 59040, 55184, 16920, 40668, 26124, 13240,
                33648, 53980, 41844, 22564, 34748, 13672, 28436, 40992, 46204,
                30328, 18508, 35312, 7056,  28608, 45640, 53384, 36792, 64192,
                21820, 40080, 8936,  4040,  14888, 44064, 58964, 20464, 62656,
                16596, 43648, 22704, 46056, 15368, 51480, 5116,  57392, 43424,
                31152, 7840,  61496, 21992, 42004, 6788,  26824, 52244, 40716,
                33368, 55120,
            },
            {
                12672, 58392, 17384, 3056,  29400, 18920, 24572, 15036, 60392,
                6824,  35684, 64708, 43292, 14180, 25872, 47484, 18620, 59296,
                31948, 40620, 35628, 63304, 25244, 55168, 20396, 27548, 57076,
                40376, 61308, 28,    42416, 24244, 46340, 5136,  20048, 26308,
                45852, 6348,  38468, 61276, 26020, 11472, 7444,  20012, 33336,
                17296, 49660, 54384, 11252, 24856, 57524, 47128, 52460, 63144,
                16948, 49508, 61748, 43048, 10944, 47232, 23976, 32744, 60076,
                42760, 20836, 28400, 34656, 55200, 22588, 46652, 30508, 6928,
                34996, 1160,  32412, 11944, 50984, 5524,  31744, 62540, 7560,
                18804, 27732, 9756,  16852, 59908, 48304, 7464,  20104, 3656,
                58264, 24640, 54672, 64052, 21024, 33352, 24180, 11680, 47164,
                16608, 49380, 62508, 36000, 28768, 47996, 26296, 32024, 39960,
                8356,  54664, 60380, 37176, 26084, 63380, 39796, 20332, 13648,
                3620,  52976, 11632, 46076, 31916, 55896, 13520, 59260, 9624,
                24216, 64452,
            },
            {
                7352,  32260, 22200, 53752, 43496, 34112, 37936, 1440,  44288,
                32288, 16472, 28288, 20124, 38104, 52748, 32,    35228, 23940,
                50472, 8180,  14408, 53048, 17308, 5752,  34232, 1892,  9316,
                16076, 29996, 6492,  58788, 54432, 14432, 49616, 57612, 34252,
                65260, 53380, 32072, 16448, 688,   49348, 39940, 56972, 12888,
                59440, 26400, 920,   19304, 44272, 15608, 4896,  12584, 34136,
                38684, 8468,  21544, 28596, 344,   53140, 19920, 15760, 3452,
                10296, 51420, 47672, 7732,  43704, 18068, 1820,  40800, 56920,
                53036, 64292, 41752, 44828, 19692, 37860, 244,   47964, 43276,
                57684, 39420, 51848, 1000,  31244, 38364, 55160, 64996, 33204,
                11312, 38728, 4744,  47400, 40440, 1332,  59416, 8128,  54912,
                31840, 528,   25076, 56580, 18208, 54492, 6124,  13784, 50332,
                35608, 29748, 6000,  12576, 48312, 31628, 8364,  34952, 54600,
                23524, 48660, 35744, 58884, 2724,  20316, 35244, 28772, 2112,
                49608, 36920,
            },
            {
                41276, 45848, 26256, 59540, 9304,  51472, 64412, 56388, 26476,
                53812, 10588, 50276, 57560, 8544,  62936, 11916, 54832, 6072,
                56756, 27648, 59968, 3412,  42976, 39124, 49588, 65472, 44660,
                53116, 34884, 47644, 21436, 37736, 31796, 28328, 9568,  43228,
                12392, 2772,  28440, 41320, 58756, 35320, 52868, 27876, 44820,
                8668,  35880, 47976, 61440, 32632, 28952, 41348, 59300, 23912,
                2224,  31740, 52040, 39740, 59884, 34704, 44356, 30132, 56024,
                39640, 14496, 65468, 30984, 12460, 58988, 33556, 10136, 26448,
                16824, 48052, 4000,  28560, 60692, 24812, 53544, 15772, 22444,
                30688, 5176,  63372, 25320, 43752, 10696, 23724, 50432, 15460,
                44396, 59832, 17036, 26832, 13368, 50588, 18160, 42136, 27496,
                38784, 58688, 45232, 11368, 40836, 3076,  64228, 24224, 45724,
                296,   18680, 52136, 41552, 2236,  17836, 60640, 45088, 27772,
                64604, 40332, 15732, 25464, 5628,  51036, 39848, 62504, 44708,
                21128, 16132,
            },
            {
                52028, 61960, 1108,  14176, 47548, 5948,  21548, 12504, 40976,
                19092, 61796, 1908,  24240, 44700, 33588, 28892, 41880, 16084,
                37552, 20052, 44908, 30216, 23528, 12316, 58240, 29076, 19180,
                24736, 11456, 63960, 17476, 8452,  2164,  61908, 40344, 22684,
                18624, 60248, 47832, 9280,  21576, 14048, 5516,  23480, 64980,
                3780,  41512, 22452, 55776, 38904, 7912,  49824, 20068, 54148,
                45672, 65200, 5636,  18544, 14444, 9108,  64028, 5472,  46420,
                26576, 35800, 19416, 5104,  52504, 27380, 63244, 49936, 20468,
                37392, 7972,  22544, 13700, 56736, 9688,  34284, 64840, 11640,
                50228, 36416, 14768, 47152, 56696, 18076, 3028,  27432, 36220,
                108,   29624, 51972, 8904,  62604, 36080, 30472, 61632, 5312,
                14344, 20032, 7268,  33604, 22052, 51780, 37380, 10132, 59616,
                56424, 27140, 64676, 21944, 56008, 24896, 50500, 4460,  10836,
                18964, 112,   29576, 42868, 63868, 46908, 16700, 11436, 31400,
                54796, 4212,
            },
            {
                28664, 19528, 35876, 39840, 31164, 17024, 28372, 48852, 4912,
                37056, 46472, 31632, 40500, 4708,  17632, 21724, 46148, 64976,
                2484,  48964, 10876, 62428, 52124, 32968, 7696,  14536, 41640,
                5304,  56232, 39612, 26520, 44028, 55560, 16264, 52112, 6764,
                36568, 50692, 24584, 44392, 56176, 62660, 32760, 46996, 15868,
                30884, 53260, 10284, 14276, 4224,  64136, 26292, 36100, 9664,
                15476, 27316, 57256, 47848, 25444, 50776, 38064, 22264, 58880,
                53552, 2568,  61328, 23416, 40392, 36424, 14820, 4476,  43884,
                55672, 30124, 62364, 35748, 49008, 40780, 6184,  45508, 27536,
                2516,  60280, 21460, 7296,  35200, 62476, 41588, 54072, 47572,
                63236, 21840, 42560, 32864, 55604, 3844,  44612, 22240, 52056,
                34888, 65296, 49696, 29780, 61196, 15012, 43500, 32564, 21084,
                40536, 14420, 34532, 9080,  43992, 36240, 14832, 38548, 59492,
                33016, 51376, 55728, 9552,  33540, 23400, 7436,  57120, 25212,
                60464, 9100,
            },
            {
                48644, 11932, 56904, 23788, 63264, 42828, 60300, 33084, 57648,
                8832,  22360, 14112, 54464, 61120, 49396, 58280, 9292,  32144,
                24872, 40944, 34288, 18236, 492,   47528, 21568, 60256, 36896,
                50812, 30740, 3144,  49808, 60100, 35484, 25040, 46668, 63568,
                32668, 15128, 3820,  30512, 37748, 18796, 1740,  51708, 38708,
                20144, 58468, 27960, 35044, 17964, 45280, 56936, 3344,  60896,
                43316, 37212, 11656, 33292, 42632, 3244,  28808, 12832, 17960,
                8520,  32200, 43180, 48464, 57192, 8780,  46536, 24672, 60060,
                11364, 40104, 52012, 18832, 2008,  30528, 17244, 58376, 38808,
                19032, 42048, 53084, 32224, 12500, 28988, 6236,  20384, 9604,
                13596, 57080, 6508,  18768, 24464, 48420, 15676, 9892,  56792,
                2104,  42900, 24496, 54008, 1404,  47272, 27984, 17652, 7212,
                47668, 4120,  49656, 31176, 62380, 5796,  28844, 56960, 21740,
                46212, 7264,  61520, 13960, 19360, 53568, 36704, 764,   43272,
                38444, 34300,
            },
            {
                65180, 43924, 7232,  53436, 3308,  10644, 36148, 424,   25236,
                51256, 65388, 28736, 6912,  34964, 26616, 1192,  51176, 13496,
                60408, 53828, 7932,  57244, 38552, 26076, 55328, 45776, 1596,
                62276, 22416, 14184, 32896, 19988, 4712,  12640, 29304, 476,
                39652, 59204, 52936, 7388,  12320, 42740, 25432, 61320, 11264,
                44112, 2624,  62844, 47352, 52488, 23368, 12824, 30580, 19012,
                51492, 22456, 1336,  61792, 20472, 55352, 60656, 34188, 41272,
                49472, 25256, 11584, 16556, 1880,  21088, 31832, 53440, 1068,
                33664, 15864, 5008,  46516, 26640, 61688, 52416, 23632, 32692,
                55164, 10188, 25968, 45060, 58248, 50696, 38184, 61036, 34088,
                25348, 39900, 46664, 37248, 11888, 59112, 29120, 38432, 25852,
                47052, 12348, 17312, 8684,  35272, 57796, 4552,  63784, 52808,
                36144, 61380, 24284, 17516, 11624, 53700, 47300, 1628,  42224,
                16184, 37168, 26872, 48476, 39344, 60184, 27748, 50240, 18640,
                13576, 22924,
            },
            {
                1716,  17368, 32784, 26956, 50384, 19708, 45972, 53936, 12860,
                42120, 18000, 38584, 47560, 10792, 15680, 39924, 36392, 29424,
                4204,  21960, 15412, 28564, 63536, 4960,  16896, 34080, 27652,
                11036, 43040, 58192, 47080, 9968,  53024, 42448, 21340, 57032,
                10808, 27000, 20792, 64724, 48016, 54988, 29112, 8212,  34208,
                49332, 24404, 7052,  31348, 41736, 304,   39512, 33904, 47728,
                6512,  28880, 53884, 39980, 16492, 6768,  45668, 52532, 596,
                64484, 37744, 55444, 60368, 28368, 38968, 62848, 17676, 41972,
                27136, 64772, 23176, 56004, 10420, 42104, 13324, 3456,  7904,
                47368, 62220, 536,   17584, 4852,  22744, 14700, 43464, 4104,
                52684, 30236, 1528,  64964, 51612, 2872,  41456, 63552, 19064,
                32096, 59928, 37952, 41492, 21268, 11936, 24876, 39400, 29948,
                12856, 44488, 1008,  58904, 40876, 20268, 25696, 63160, 12072,
                30640, 23728, 2412,  44168, 5444,  12104, 30980, 62816, 46752,
                5716,  54016,
            },
            {
                30400, 40772, 46912, 14324, 38072, 59248, 23052, 15916, 62348,
                31928, 2420,  59328, 20484, 55640, 43900, 62512, 18860, 56808,
                42748, 47024, 35576, 50708, 44256, 13244, 41128, 8824,  53872,
                18896, 35956, 6548,  28556, 64912, 37008, 61004, 31668, 49224,
                17896, 45856, 40736, 33212, 15692, 4736,  36480, 59680, 17884,
                55436, 14756, 37424, 20660, 60112, 8792,  63432, 53332, 14824,
                41384, 64308, 8624,  49288, 31836, 26912, 10748, 23644, 15312,
                29568, 20192, 4256,  33896, 44508, 51568, 5824,  12760, 58436,
                49524, 7276,  44200, 36728, 32084, 20936, 49888, 63836, 35448,
                15228, 29540, 40352, 36744, 65476, 46600, 26852, 57512, 18232,
                48896, 59932, 16376, 27088, 21364, 33612, 8224,  50196, 5776,
                55076, 3680,  27348, 62932, 55688, 44164, 50704, 60292, 9228,
                22128, 57140, 27896, 37940, 52052, 32272, 8384,  34892, 49948,
                56048, 65392, 52616, 32472, 58124, 22108, 3004,  15944, 39768,
                26444, 59740,
            },
            {
                20120, 57664, 8692,  62536, 4284,  29824, 7068,  39932, 26536,
                44944, 5600,  50508, 24860, 30620, 3496,  23044, 52636, 6588,
                26252, 64440, 10408, 2288,  20556, 31496, 61504, 49068, 23124,
                63736, 51196, 15804, 40604, 1164,  23740, 14728, 5676,  54136,
                8280,  35792, 1640,  56864, 24068, 51140, 21244, 41048, 760,
                27352, 42988, 57468, 12032, 49544, 17160, 27724, 24844, 4580,
                56304, 35108, 24220, 12896, 58692, 63292, 36396, 43948, 57012,
                7624,  47092, 52792, 14076, 9464,  25448, 47404, 35372, 21416,
                38276, 29656, 14000, 60624, 720,   58100, 38044, 25588, 43668,
                56928, 22340, 50000, 11976, 30732, 54260, 1420,  32680, 11140,
                23360, 7792,  35124, 42888, 56152, 45688, 14292, 23580, 36280,
                44664, 15332, 51800, 30692, 6668,  18692, 33388, 1944,  16028,
                54132, 33812, 6636,  15052, 43436, 2984,  60116, 18228, 40092,
                4320,  9684,  20248, 16688, 41460, 50816, 35944, 56040, 9300,
                33436, 51012,
            },
            {
                3120,  36248, 24828, 54724, 43376, 33412, 55632, 49148, 9836,
                35316, 56788, 12280, 37088, 63872, 8540,  33252, 45548, 12048,
                17376, 32320, 40196, 59660, 25128, 56536, 37148, 5936,  29736,
                2848,  44752, 25680, 55032, 18384, 48776, 44580, 38960, 26064,
                63436, 13644, 60712, 28572, 10240, 44940, 63164, 12952, 52560,
                32776, 65308, 3816,  29468, 54236, 35236, 44332, 58836, 10616,
                20820, 45836, 17660, 38348, 2800,  47424, 4888,  18632, 39816,
                62268, 31700, 22636, 41608, 63980, 18992, 30644, 56676, 10008,
                3132,  54208, 18948, 47192, 8616,  16968, 28616, 4804,  11172,
                18440, 52996, 6632,  59520, 19628, 8508,  40888, 61444, 37920,
                53528, 45356, 13192, 4636,  62024, 18216, 52772, 30412, 61536,
                9780,  21620, 48056, 1052,  14004, 37544, 48816, 26508, 41760,
                46404, 19184, 50348, 64988, 10736, 22808, 54976, 45248, 14412,
                28568, 46972, 38396, 60544, 26004, 7092,  45512, 64528, 23856,
                42784, 12532,
            },
            {
                29172, 15600, 48612, 21372, 11492, 17492, 64464, 1692,  20224,
                61384, 29224, 16712, 46824, 40608, 14876, 49272, 57924, 38424,
                1380,  54784, 48004, 7572,  52472, 11544, 17904, 46700, 58676,
                38844, 12360, 57708, 34800, 31068, 9560,  59652, 2548,  33372,
                22252, 50820, 42308, 18456, 3052,  38416, 31332, 5836,  47696,
                22520, 9600,  46308, 40044, 6336,  22256, 2060,  38088, 50448,
                32328, 1024,  57640, 43244, 30168, 21504, 51604, 33996, 25836,
                12728, 1532,  58996, 35592, 5072,  55048, 180,   61288, 42392,
                25828, 63324, 50752, 24336, 33876, 42756, 53704, 48872, 60412,
                33716, 2096,  27036, 42792, 34516, 13804, 50236, 20992, 3048,
                27356, 63900, 40420, 31136, 10120, 25944, 388,   39236, 57228,
                26424, 42260, 33184, 58484, 24368, 64000, 8644,  56508, 30376,
                62840, 2656,  36712, 26176, 47036, 30324, 36336, 5984,  21684,
                62440, 33592, 1284,  13304, 54844, 10668, 28656, 19452, 49012,
                6196,  60960,
            },
            {
                44856, 64064, 39624, 868,   52236, 27788, 37308, 13952, 41008,
                51968, 23132, 7580,  54228, 248,   60168, 27564, 20780, 29940,
                62068, 23520, 13568, 28816, 35168, 3604,  42524, 26556, 14996,
                32708, 21368, 8312,  4048,  63224, 51760, 27844, 19644, 57616,
                45552, 6312,  30780, 55188, 48956, 59392, 26700, 15960, 57868,
                37112, 18628, 25804, 15512, 62784, 48068, 12664, 60856, 28620,
                15748, 61896, 25708, 53044, 7856,  15164, 55864, 60404, 9128,
                50480, 45448, 27444, 16188, 49228, 38644, 23092, 15280, 46316,
                31820, 11964, 40164, 4012,  55932, 65080, 14520, 39636, 23836,
                46016, 64108, 37752, 48300, 24432, 63064, 44576, 29408, 56256,
                15084, 48120, 19916, 54816, 36580, 59472, 47144, 12312, 5084,
                16736, 64788, 7760,  40108, 52696, 44804, 21176, 4020,  11612,
                38824, 23628, 8220,  58092, 17240, 61644, 51128, 26948, 58608,
                43088, 53468, 23952, 48692, 31280, 61916, 34792, 1016,  14584,
                37384, 54488,
            },
            {
                10184, 26696, 7144,  34988, 60520, 46036, 23760, 58132, 47784,
                4624,  34056, 43104, 19136, 25304, 35644, 10640, 4516,  42200,
                50060, 36628, 19132, 44900, 60780, 21744, 49984, 65216, 256,
                54116, 61304, 47116, 41308, 22792, 13352, 42608, 36048, 11108,
                16216, 39564, 24904, 12160, 34096, 20160, 8592,  54296, 43672,
                1876,  59800, 52068, 34020, 55892, 31272, 18932, 42732, 55072,
                7468,  48360, 36352, 11440, 65264, 40540, 28104, 2404,  42692,
                18016, 37824, 56292, 10572, 29308, 44412, 8416,  34624, 52408,
                17824, 6096,  35900, 20684, 30380, 10656, 2512,  20296, 7332,
                30952, 10052, 14744, 57348, 3480,  6256,  17104, 36764, 9236,
                33660, 5340,  24384, 2368,  49848, 41868, 22296, 32628, 53636,
                49972, 35468, 19952, 11988, 28204, 16256, 35736, 61696, 51360,
                14488, 55924, 32116, 40664, 4632,  13372, 300,   39632, 11668,
                7864,  16236, 36888, 5580,  43796, 17452, 40084, 52584, 58236,
                31356, 21844,
            },
            {
                51852, 18824, 58764, 30224, 40840, 8876,  3024,  32208, 26860,
                10928, 57240, 64232, 31040, 50388, 55440, 44132, 65124, 15540,
                8784,  59200, 2688,  39736, 15776, 53808, 33600, 10056, 40656,
                18584, 28072, 50484, 37060, 16944, 56300, 5248,  48152, 65416,
                53368, 144,   61816, 46100, 4528,  64496, 40496, 23984, 33428,
                28684, 12416, 41188, 3460,  8412,  38888, 26452, 3100,  34588,
                21732, 40168, 4356,  19476, 33320, 23756, 49000, 13572, 31880,
                64160, 22188, 6592,  61080, 19956, 53532, 65404, 26776, 2256,
                59852, 56812, 43976, 61172, 46576, 27008, 37120, 57816, 52340,
                42088, 17744, 54156, 21532, 32448, 51948, 58348, 25368, 62236,
                51124, 39048, 58732, 29580, 15972, 7724,  62920, 18884, 43644,
                29164, 2036,  46336, 61216, 5560,  57576, 32256, 42376, 18852,
                27852, 44304, 49336, 20460, 53908, 34668, 46216, 19792, 56240,
                29992, 50024, 64444, 20692, 55736, 3712,  22980, 45252, 25912,
                4544,  41176,
            },
            {
                33784, 3664,  49976, 12144, 16168, 20952, 54120, 62692, 18408,
                44876, 15316, 2580,  37604, 12764, 6740,  18064, 33496, 52940,
                26316, 31776, 56444, 6028,  63268, 30480, 7312,  24372, 36520,
                45596, 5820,  11592, 25504, 1516,  60784, 30188, 24152, 8536,
                32700, 28228, 57340, 17228, 36324, 51680, 13896, 48236, 6268,
                63876, 49916, 20420, 61140, 23456, 50980, 64868, 10312, 46716,
                13704, 62888, 29688, 50916, 56468, 45712, 6104,  36640, 53588,
                25204, 3340,  51528, 40560, 32832, 4640,  13176, 41484, 50056,
                21948, 28456, 13392, 8040,  52884, 17208, 48148, 33172, 25496,
                61592, 932,   29020, 45580, 38384, 11728, 41436, 20,    45168,
                12340, 17868, 43480, 65236, 11040, 34756, 27020, 3016,  60384,
                14060, 56288, 22680, 38108, 50328, 368,   24920, 47808, 6228,
                60084, 1328,  10596, 64148, 28672, 42516, 23828, 63488, 33196,
                2220,  25080, 41568, 9484,  28240, 59596, 12400, 64016, 9044,
                16844, 61812,
            },
            {
                13628, 46544, 23548, 56836, 65304, 42664, 48940, 5996,  35164,
                39064, 53068, 27972, 22000, 62124, 45932, 24236, 38932, 912,
                47292, 12172, 22916, 45988, 27068, 12940, 42928, 59820, 55316,
                14100, 31108, 63812, 52824, 33740, 45108, 39852, 18724, 51512,
                38252, 20868, 7440,  41788, 22872, 29676, 1276,  58952, 17468,
                37956, 9456,  45832, 30100, 14332, 44596, 17356, 57864, 52920,
                24124, 44108, 58588, 700,   15576, 10092, 61464, 20376, 58504,
                47156, 11644, 35264, 14804, 48112, 23592, 55484, 37500, 9768,
                33004, 48432, 39152, 24664, 540,   41052, 63092, 4948,  12604,
                49568, 35276, 59232, 6820,  26156, 63756, 19280, 30520, 22352,
                54476, 6524,  32500, 20948, 51720, 55656, 46588, 37144, 6084,
                40552, 25656, 9248,  54764, 29620, 13600, 39868, 9872,  54004,
                34940, 22596, 37816, 15964, 6660,  59380, 9588,  48376, 17440,
                38420, 60596, 13624, 32532, 46864, 36644, 50656, 33128, 38944,
                48276, 55232,
            },
            {
                27992, 43004, 38144, 32344, 5184,  36076, 28876, 13484, 24680,
                61268, 8408,  49300, 41472, 3944,  59088, 29596, 56200, 20912,
                61704, 41020, 52000, 35908, 19872, 48512, 1612,  50700, 20772,
                3836,  58876, 42172, 19312, 7548,  55212, 14600, 2676,  58424,
                12388, 44536, 49640, 54520, 9772,  61400, 32616, 45080, 21872,
                26884, 54000, 35600, 5952,  56472, 208,   36960, 28088, 5216,
                32064, 8972,  18036, 26928, 39252, 43044, 29168, 1960,  16352,
                41748, 30120, 60136, 26272, 63184, 840,   30484, 19172, 61552,
                15780, 3772,  64488, 19496, 32040, 58868, 14984, 22612, 44992,
                8760,  19836, 41024, 13732, 55468, 47228, 8204,  49772, 35024,
                60468, 26544, 47372, 1092,  39464, 13720, 23408, 57740, 49372,
                32068, 63728, 15844, 34168, 44176, 65376, 21564, 58848, 16812,
                30072, 61972, 47208, 25796, 52292, 31568, 14760, 4036,  55272,
                44884, 5252,  52844, 58120, 16044, 392,   19532, 6948,  29268,
                21068, 2144,
            },
            {
                63416, 9596,  19724, 192,   25908, 52864, 10108, 59460, 46268,
                960,   20084, 57660, 32740, 16612, 9472,  48968, 13444, 5584,
                34224, 17176, 9336,  3424,  58420, 38668, 16856, 62240, 29116,
                34984, 47804, 23208, 38300, 10852, 28000, 63128, 47376, 26632,
                35248, 63720, 1964,  15500, 27168, 39184, 12024, 56644, 41984,
                3204,  15276, 58792, 24676, 41780, 33380, 49532, 20712, 40768,
                60452, 37716, 49144, 35192, 63176, 21984, 51164, 33976, 38448,
                7112,  55828, 44912, 8888,  17508, 43380, 58516, 46408, 5368,
                42460, 56448, 35076, 54328, 44572, 9996,  28940, 53844, 39204,
                31096, 64952, 50952, 23096, 2244,  33640, 15640, 57096, 3180,
                14012, 37508, 9840,  58056, 28448, 44704, 7060,  17588, 10380,
                20596, 43144, 4860,  48680, 18972, 7588,  52304, 26724, 44692,
                4940,  12680, 41884, 57536, 1932,  39144, 50380, 27040, 35812,
                20280, 28764, 8116,  23196, 42720, 26784, 54096, 60944, 43840,
                57572, 35960,
            },
            {
                16340, 51296, 60648, 56012, 45528, 17556, 40380, 21916, 55508,
                29856, 43528, 11920, 26104, 52692, 34908, 42724, 63944, 27180,
                54084, 44468, 29464, 64864, 54744, 31432, 11012, 24040, 40252,
                8928,  15756, 736,   57564, 49600, 32060, 21124, 41688, 4428,
                17692, 30760, 23752, 58672, 48444, 4264,  19284, 52436, 7956,
                63344, 32160, 47624, 18716, 62352, 12484, 8316,  63464, 15808,
                2228,  55332, 12700, 6976,  52988, 4252,  57272, 11056, 64968,
                23820, 19616, 2760,  50880, 39100, 34592, 11928, 24768, 28704,
                51232, 22740, 11792, 27200, 6772,  50556, 35436, 2124,  56852,
                16204, 3660,  27744, 37232, 60188, 43548, 28504, 39244, 64584,
                42256, 53012, 23016, 61620, 4376,  33852, 63060, 30108, 52604,
                59804, 35864, 27740, 1896,  60416, 37592, 32932, 2608,  49672,
                36676, 55528, 8020,  18048, 34076, 21672, 61228, 42920, 11892,
                62984, 51048, 34304, 39444, 65272, 4548,  48848, 14748, 10728,
                24488, 5932,
            },
            {
                31048, 39920, 27192, 7640,  14188, 63792, 49404, 3960,  34084,
                16124, 37100, 64992, 1992,  39380, 23000, 7024,  19260, 37684,
                2140,  49856, 24740, 14472, 7692,  34740, 46512, 4352,  57052,
                44652, 51636, 65040, 25764, 4840,  36604, 52376, 9120,  60028,
                54980, 6272,  40928, 33744, 43944, 65088, 36692, 24280, 49836,
                28232, 39936, 10432, 1968,  52248, 29504, 25420, 43908, 51372,
                23392, 29988, 46024, 19820, 25164, 14268, 47364, 26720, 43812,
                49752, 13416, 28376, 57832, 21764, 5892,  53688, 64188, 8772,
                36900, 1240,  47332, 16536, 63356, 38288, 18568, 60836, 25316,
                47732, 42852, 11232, 17920, 52184, 9988,  20624, 24960, 5060,
                18476, 29392, 49096, 15712, 19880, 54104, 40652, 25152, 784,
                46132, 12904, 56832, 53344, 24708, 10784, 42348, 14028, 62912,
                19456, 31808, 24528, 64892, 44924, 10440, 6148,  56380, 23964,
                2912,  47620, 18024, 11480, 55992, 21164, 37472, 31688, 41056,
                52760, 47480,
            },
            {
                22152, 12352, 43936, 33928, 23200, 37260, 31120, 8944,  51676,
                62156, 5816,  46680, 18496, 54880, 60460, 45592, 30928, 57292,
                11536, 59560, 39520, 20200, 43280, 26560, 52676, 63252, 18328,
                27096, 32936, 11996, 43032, 17732, 61204, 13184, 24360, 45236,
                37444, 50020, 20184, 13100, 8168,  30056, 16004, 800,   34260,
                13408, 60848, 22920, 44720, 38004, 55764, 3756,  35404, 58464,
                10084, 42284, 64324, 33628, 60776, 41152, 31364, 18108, 224,
                32980, 54388, 36468, 62460, 31612, 14940, 49356, 40112, 17280,
                57396, 30788, 60508, 40964, 3392,  21936, 46356, 12380, 7936,
                34024, 61660, 54728, 32348, 6396,  63200, 47992, 55324, 11952,
                45320, 32904, 7884,  36176, 43632, 11212, 50300, 14964, 38020,
                7684,  22884, 40156, 31340, 16528, 47032, 57836, 29280, 22524,
                53104, 828,   40268, 49440, 28784, 53504, 16880, 30600, 40680,
                15032, 59128, 31268, 1796,  44916, 27884, 7844,  63088, 18532,
                1144,  58712,
            },
            {
                64460, 2828,  49784, 54852, 59172, 1748,  18988, 43060, 24420,
                13664, 27628, 50220, 31752, 10196, 14464, 3332,  51352, 15704,
                22312, 32364, 4972,  48352, 61396, 1020,  22008, 12884, 37528,
                6544,  54628, 20664, 39120, 29920, 46480, 1728,  32876, 15424,
                27916, 10488, 61600, 56352, 25884, 53484, 60512, 45472, 55576,
                20000, 48636, 5364,  57716, 16232, 20504, 47084, 13944, 31676,
                18576, 5376,  27708, 1552,  54540, 9488,  37280, 59868, 8296,
                61676, 40360, 4628,  10252, 46736, 26944, 1976,  32568, 45196,
                20628, 13668, 52848, 25012, 33304, 58172, 28156, 41740, 52256,
                20516, 4480,  23904, 46228, 40752, 716,   30864, 35352, 50860,
                60140, 1312,  56600, 64068, 23800, 2628,  58804, 31756, 64636,
                18200, 49004, 61588, 5688,  20904, 64288, 3764,  38628, 8664,
                46436, 61064, 11540, 15116, 3580,  37408, 46980, 63772, 34700,
                8856,  43344, 25216, 62168, 51304, 35528, 13340, 54932, 46112,
                26040, 34540,
            },
            {
                37712, 19600, 5648,  29556, 11292, 46328, 53200, 60436, 38692,
                56980, 21296, 41192, 58116, 23880, 36296, 63204, 26364, 47240,
                41304, 64124, 9792,  36556, 16548, 56080, 40684, 50008, 30064,
                60056, 2804,  47972, 62244, 6064,  55592, 58888, 39672, 53892,
                64632, 628,   35852, 46580, 4908,  21104, 37872, 10948, 40852,
                6852,  30196, 36512, 26612, 32848, 65456, 7496,  39540, 61784,
                48100, 56700, 39072, 15948, 49388, 23576, 3992,  51992, 45432,
                21608, 16228, 24892, 43136, 19124, 56000, 60840, 23560, 6996,
                62408, 38596, 4680,  44088, 10400, 54868, 15016, 336,   64076,
                38768, 29404, 58944, 12560, 19888, 57428, 15008, 22628, 7400,
                40600, 25988, 13384, 38900, 47188, 27804, 6168,  42956, 55404,
                26420, 34672, 9832,  44380, 51288, 36004, 25716, 55216, 16156,
                43268, 27116, 35728, 56976, 60180, 20336, 25980, 116,   51556,
                21712, 54696, 38040, 6840,  16276, 59860, 23412, 4872,  39676,
                9748,  15068,
            },
            {
                52468, 42364, 61700, 16024, 40220, 26088, 6984,  28948, 11672,
                432,   34780, 8360,  4384,  44848, 28924, 40068, 7676,  34480,
                584,   54300, 27160, 51832, 29680, 6008,  33420, 9532,  44264,
                14720, 35388, 24596, 10124, 16292, 27128, 22372, 7492,  18728,
                42736, 23216, 31264, 17340, 41716, 50964, 2644,  27032, 58616,
                16988, 63996, 53716, 2716,  42084, 11348, 50552, 27332, 524,
                22492, 52392, 12180, 34776, 44600, 19108, 56216, 27840, 11684,
                30424, 48216, 53180, 64660, 3612,  35444, 41396, 12208, 54232,
                34404, 27288, 50184, 19448, 7372,  47956, 36780, 31536, 49060,
                9496,  15524, 35632, 50512, 26028, 38228, 44616, 62020, 29000,
                16884, 52100, 21096, 31224, 54508, 16356, 35764, 20480, 11756,
                2400,  52820, 28964, 15064, 212,   32388, 12176, 49904, 4764,
                33060, 18424, 7160,  23472, 31552, 5420,  39664, 13492, 58024,
                28284, 4072,  48196, 19648, 30272, 41156, 48552, 33300, 60852,
                28452, 56724,
            },
            {
                7404,  25172, 32168, 47880, 21676, 65132, 35488, 16836, 44668,
                53912, 48332, 61264, 17404, 52896, 59612, 19760, 11116, 56744,
                17888, 43872, 21180, 12496, 45920, 58640, 18892, 64764, 23032,
                56588, 41292, 53040, 32148, 50244, 44184, 35532, 51404, 11568,
                47892, 5792,  52568, 58416, 13968, 33544, 63376, 22620, 32036,
                50344, 42984, 9656,  15088, 59332, 23380, 54900, 45012, 16464,
                37076, 26192, 6292,  58924, 29868, 64812, 13948, 39972, 35356,
                57580, 6240,  38076, 14020, 28728, 8704,  51804, 16740, 47756,
                1524,  11076, 58828, 29964, 63768, 23232, 59376, 18100, 25652,
                56620, 44216, 1856,  65276, 7892,  53240, 10612, 2736,  37128,
                47472, 62864, 9952,  3812,  59764, 8388,  49740, 62152, 38792,
                45224, 59840, 22392, 42100, 62212, 57236, 19964, 41164, 59404,
                63988, 47920, 52172, 42196, 55084, 49032, 61828, 43748, 18564,
                10636, 64596, 33216, 12712, 57932, 636,   10764, 20064, 50372,
                3488,  44372,
            },
            {
                12000, 36156, 57284, 3388,  9104,  55452, 50036, 4664,  62748,
                20080, 30244, 25476, 38156, 13112, 1560,  47912, 52044, 25176,
                61580, 31572, 3856,  60356, 38716, 25296, 1984,  48456, 28200,
                7608,  17924, 200,   64364, 38360, 13656, 2728,  63248, 30360,
                25824, 60568, 39036, 9608,  28592, 44484, 7896,  47420, 12544,
                4128,  25224, 21292, 46984, 38120, 29924, 5032,  33804, 57988,
                9232,  63424, 46560, 21680, 2980,  42464, 7260,  24716, 62548,
                1364,  19980, 33404, 58544, 49252, 21884, 59672, 30304, 25312,
                65464, 41696, 16440, 35072, 39828, 12612, 3520,  42272, 6208,
                52556, 22540, 39776, 17544, 28292, 33408, 59356, 18764, 55428,
                5396,  43660, 34324, 41440, 18780, 44880, 29628, 23956, 32716,
                4668,  17148, 55724, 37504, 6708,  46672, 28492, 9612,  22764,
                29768, 13108, 1568,  37636, 11064, 16400, 7988,  30052, 35868,
                50352, 40264, 24228, 45976, 53208, 27236, 43236, 64888, 15876,
                31580, 21592,
            },
            {
                63636, 17408, 51444, 38408, 14248, 30664, 41240, 23504, 33096,
                14548, 42276, 10556, 63556, 33596, 22532, 30444, 42052, 14308,
                5916,  35932, 49876, 15900, 8392,  34816, 54404, 13500, 39212,
                60876, 45284, 21892, 28944, 5180,  59648, 20644, 41016, 56420,
                14684, 34452, 21208, 3728,  61788, 19372, 52288, 35768, 57368,
                38912, 61940, 34200, 52756, 1140,  62744, 19740, 12588, 42832,
                31172, 18880, 39804, 53888, 10776, 32580, 51336, 46084, 15860,
                49668, 41712, 11088, 25616, 43880, 672,   39660, 36624, 6132,
                44656, 21692, 56316, 2488,  53696, 45312, 28900, 62620, 34196,
                13580, 61100, 31020, 46052, 3864,  48632, 41092, 24784, 32280,
                12804, 22356, 57048, 26240, 52664, 65320, 556,   13936, 51828,
                47280, 8896,  31524, 12880, 25124, 17704, 53212, 36256, 3908,
                39060, 57800, 20800, 26352, 65460, 34340, 22648, 46288, 56568,
                1604,  15544, 59644, 8564,  36816, 5400,  56956, 24352, 38216,
                54460, 46928,
            },
            {
                29124, 372,   23688, 44832, 61376, 19388, 1496,  58328, 46764,
                6872,  55660, 2264,  50504, 43828, 57488, 6988,  64040, 38496,
                46312, 23424, 57212, 40924, 63496, 21800, 43120, 31220, 4052,
                51120, 34296, 12480, 57424, 48400, 25272, 33688, 46796, 8660,
                1176,  45336, 54644, 48952, 25000, 40368, 56,    16280, 29440,
                18792, 44624, 7236,  28364, 17580, 48320, 40240, 60088, 52020,
                2080,  49324, 14852, 61884, 27024, 37772, 60104, 21028, 4828,
                29452, 53860, 63856, 7248,  32016, 15212, 62252, 19076, 57704,
                13544, 32316, 48388, 7072,  26116, 20636, 50764, 10180, 47360,
                19256, 5076,  54516, 11800, 62804, 21224, 14296, 51416, 64552,
                28164, 49188, 2120,  15184, 36880, 11280, 39744, 58148, 19676,
                61344, 26152, 40844, 64468, 49492, 3064,  44196, 63168, 15232,
                54588, 43508, 8376,  47048, 29276, 52768, 2796,  60044, 25388,
                7376,  31396, 42896, 22140, 62808, 17372, 33832, 12984, 1956,
                8484,  59628,
            },
            {
                41940, 6360,  58660, 33884, 27016, 53864, 12596, 36292, 26288,
                39528, 60172, 21392, 27540, 4988,  36484, 16252, 26656, 9760,
                54092, 2948,  29836, 11452, 27424, 52296, 6696,  57876, 16780,
                24804, 8916,  53612, 42344, 18708, 10600, 54212, 16416, 28392,
                62760, 37576, 17164, 12040, 30840, 56296, 65184, 10388, 53668,
                3220,  58860, 13312, 55696, 10200, 35652, 25528, 7816,  22652,
                28180, 5876,  33964, 44444, 852,   17816, 55624, 9296,  36656,
                59288, 23064, 17668, 46196, 56164, 51348, 10508, 46968, 3384,
                28476, 51688, 9896,  37692, 63140, 14616, 32136, 57400, 37980,
                27260, 41828, 34936, 24264, 56672, 36912, 6356,  16,    45484,
                8748,  39004, 61468, 32516, 5556,  22640, 27676, 43328, 6108,
                36244, 53832, 1408,  20536, 34012, 59640, 8072,  31412, 48288,
                24520, 33232, 17780, 59340, 4348,  42580, 19684, 12488, 37832,
                54656, 19300, 52164, 3192,  29608, 49408, 45324, 52708, 40700,
                25640, 34696,
            },
            {
                13680, 20312, 48416, 10628, 7240,  42600, 49312, 63800, 9356,
                17784, 48528, 32300, 13844, 54964, 19440, 48736, 59412, 32088,
                20380, 62056, 17756, 48796, 836,   45632, 19688, 61980, 36740,
                47308, 63972, 30628, 37092, 6760,  62144, 39300, 4916,  51028,
                23452, 32212, 58596, 43484, 5388,  36984, 22904, 42780, 26588,
                49376, 32204, 41160, 24020, 64140, 50968, 14492, 45188, 54712,
                65032, 36828, 57280, 23868, 12812, 48732, 30992, 25564, 44136,
                13076, 33972, 38400, 3184,  28148, 21424, 34468, 24660, 54820,
                40052, 60724, 23328, 17916, 59060, 41388, 2044,  7764,  23596,
                60284, 860,   48232, 8840,  16672, 43852, 26868, 58972, 35740,
                16916, 53740, 20112, 58840, 45844, 51240, 56500, 33696, 10216,
                15856, 29320, 44516, 57604, 11028, 38744, 26992, 19596, 12008,
                604,   62616, 50112, 35964, 14904, 39412, 62948, 49132, 27968,
                44568, 64196, 34520, 13976, 39728, 9516,  20824, 28640, 62448,
                18300, 50956,
            },
            {
                56076, 31644, 39384, 64600, 16924, 30228, 4756,  22080, 52448,
                29448, 3196,  37764, 45624, 64736, 8764,  40984, 52,    51736,
                13388, 42960, 34336, 56260, 37812, 14484, 33028, 10536, 27888,
                1320,  20760, 15548, 3156,  26836, 44784, 31588, 59256, 19552,
                41808, 10280, 3408,  27360, 50544, 14016, 33768, 47060, 6120,
                62436, 21624, 36636, 46280, 5760,  30320, 3128,  32620, 38532,
                16720, 20680, 9432,  53016, 61128, 41496, 3888,  64564, 51928,
                1536,  48468, 10104, 60996, 41096, 5044,  63472, 43096, 7940,
                15092, 33920, 36,    46452, 27660, 35620, 52596, 17576, 43972,
                55056, 15216, 51672, 28688, 40236, 61804, 53072, 31196, 22116,
                42556, 11708, 29416, 7596,  40900, 17204, 1704,  47684, 64120,
                22064, 50804, 5288,  14896, 23652, 55480, 42916, 51700, 60984,
                40528, 28056, 6216,  11484, 55664, 24796, 9072,  32948, 16796,
                4868,  10364, 47356, 25608, 61088, 55340, 6060,  58060, 3912,
                10980, 44292,
            },
            {
                61756, 5268,  52788, 24472, 45616, 57364, 34512, 40436, 15248,
                56284, 62196, 11512, 23440, 52228, 29220, 34460, 22072, 45376,
                37476, 5452,  23988, 8216,  65484, 25732, 41920, 54680, 49584,
                38700, 44252, 59504, 55356, 49268, 22232, 13932, 35640, 2316,
                47704, 64360, 52964, 21384, 61428, 8552,  59512, 19856, 39312,
                11832, 15612, 1480,  59664, 19928, 42636, 57132, 60652, 11780,
                3948,  46776, 29488, 38868, 7172,  33364, 20036, 15304, 39336,
                28600, 57944, 20700, 53020, 16240, 36684, 12476, 29712, 50160,
                20476, 64540, 43676, 54536, 4724,  12376, 47988, 65100, 29728,
                11528, 21408, 63956, 32304, 2456,  19752, 11256, 6992,  50212,
                62684, 4056,  47916, 25772, 63504, 12692, 30948, 25020, 58716,
                41832, 37828, 61728, 46916, 30720, 2604,  16716, 35408, 4844,
                22280, 53896, 44716, 20388, 30812, 45816, 57720, 384,   60992,
                40636, 23272, 58364, 1172,  36088, 42660, 15348, 33132, 47524,
                37240, 22584,
            },
            {
                16072, 27824, 35712, 3108,  13064, 19624, 61300, 548,   24968,
                43876, 35128, 6264,  42304, 17096, 3776,  56400, 11180, 63196,
                28128, 59836, 53372, 30716, 47220, 3444,  60536, 5832,  22856,
                13528, 8068,  33384, 40316, 11052, 65116, 52800, 8488,  56896,
                25480, 14552, 34744, 45276, 38828, 16680, 54064, 2788,  30488,
                56496, 51628, 28528, 9556,  54200, 16412, 27044, 22012, 49748,
                41668, 62260, 51104, 14312, 26676, 57768, 47184, 8720,  61768,
                23916, 6604,  31512, 43932, 26196, 48084, 56512, 59608, 2776,
                38460, 26312, 10940, 31260, 58040, 20140, 24652, 33500, 5564,
                39680, 36020, 45192, 7360,  58312, 48788, 34732, 15480, 25072,
                56952, 33168, 37628, 55400, 21444, 36032, 54252, 4308,  18956,
                7952,  12452, 26716, 34784, 52856, 65148, 7664,  45992, 57508,
                32648, 15700, 38436, 64692, 3020,  51064, 21760, 37004, 53076,
                30160, 13200, 50892, 31504, 17712, 22288, 50400, 26432, 65268,
                30348, 2148,
            },
            {
                46164, 9992,  58512, 50420, 38380, 54568, 27656, 47776, 8236,
                50836, 19292, 58008, 26524, 61560, 39392, 47088, 25268, 16036,
                2584,  49020, 10292, 15300, 21412, 39188, 17852, 31252, 57308,
                52200, 62924, 28840, 24248, 17992, 400,   27940, 45664, 30708,
                40012, 18176, 6564,  29808, 1564,  24308, 32688, 49860, 44776,
                25516, 64768, 37644, 33224, 48176, 39480, 452,   35296, 8732,
                24940, 31888, 18144, 5548,  44248, 22556, 54068, 35412, 42428,
                17552, 55840, 13584, 65204, 560,   8956,  23260, 32464, 18044,
                47136, 53096, 7200,  16564, 42188, 38632, 8620,  60904, 50676,
                2888,  56844, 26008, 14288, 23344, 38404, 55612, 45692, 1228,
                40276, 18644, 13972, 2364,  44204, 9140,  49900, 39468, 45732,
                32772, 55692, 484,   17988, 9924,  39588, 20632, 28576, 13300,
                48704, 9184,  58528, 26548, 34576, 7996,  15320, 26212, 43520,
                18520, 56660, 7584,  45148, 63296, 3648,  38588, 12304, 8196,
                54996, 40388,
            },
            {
                60220, 18712, 31300, 22332, 42800, 6620,  11288, 31996, 65488,
                13128, 30284, 2684,  48820, 32572, 14452, 6956,  58500, 31712,
                40144, 19100, 35516, 42696, 59036, 50520, 11956, 45440, 35300,
                1860,  16068, 48188, 4672,  58456, 43140, 37780, 20608, 60660,
                11760, 53880, 62072, 48572, 56692, 41632, 63512, 10916, 6892,
                18092, 42884, 6260,  23224, 61772, 12112, 52876, 63908, 45388,
                56460, 2000,  59124, 35924, 63444, 172,   11812, 30396, 2812,
                50724, 46236, 33692, 39024, 19536, 51936, 42016, 5360,  13708,
                61732, 35968, 22768, 49816, 62572, 28320, 55788, 43312, 16284,
                19620, 47252, 9816,  62224, 42668, 4576,  27604, 61024, 30344,
                9672,  64456, 52024, 28360, 59812, 16656, 62084, 27560, 14436,
                63116, 23060, 48824, 43080, 57248, 25376, 50080, 60308, 37348,
                24024, 1116,  51908, 18212, 42296, 60484, 48488, 11000, 64432,
                5332,  34148, 39180, 26980, 9976,  53816, 59580, 43764, 19924,
                51424, 24916,
            },
            {
                6796,  49176, 64280, 1308,  15532, 57772, 36376, 21072, 41200,
                54268, 38388, 22244, 9812,  53592, 36932, 21012, 50784, 43940,
                55112, 23436, 63596, 996,   28300, 7736,  24260, 64304, 27076,
                19992, 42820, 36504, 54468, 31160, 51260, 14864, 5700,  49464,
                3468,  36400, 26384, 9388,  19720, 13480, 27832, 36516, 22044,
                58780, 13940, 2204,  45648, 19028, 31316, 5296,  15400, 28176,
                19944, 12800, 40580, 53176, 28996, 49340, 19232, 60008, 25636,
                37672, 4768,  11132, 58656, 29532, 62096, 35184, 45612, 55104,
                28540, 2176,  58980, 33728, 4396,  14316, 896,   23040, 32020,
                59856, 28516, 54180, 33940, 17940, 52368, 12696, 20544, 42080,
                49124, 22828, 5884,  38988, 32172, 23780, 6276,  34836, 3044,
                51656, 30000, 37192, 60004, 3840,  32384, 11532, 2380,  41708,
                63952, 31556, 44308, 12244, 4556,  39044, 55292, 29388, 2152,
                47120, 20980, 60620, 14672, 48900, 29960, 23876, 34244, 132,
                14228, 36356,
            },
            {
                29016, 12556, 44676, 33828, 24296, 52188, 45360, 59556, 2088,
                15912, 46800, 63412, 43232, 59696, 1040,  29008, 65004, 9240,
                4028,  13084, 51624, 32296, 56404, 36616, 53988, 3168,  40568,
                10192, 60016, 22364, 12780, 8164,  34692, 63724, 24724, 55136,
                41964, 21912, 44236, 33456, 52072, 4984,  59700, 47172, 54700,
                33864, 40172, 51160, 55596, 26108, 58292, 41976, 50488, 37932,
                33228, 47636, 8404,  24612, 15136, 38940, 43248, 55552, 14628,
                64024, 21336, 27384, 48772, 3012,  14800, 25520, 10396, 21008,
                39196, 44140, 11948, 19176, 46508, 37440, 53420, 48888, 11848,
                40972, 5888,  38128, 376,   30684, 64804, 8456,  36388, 58700,
                15928, 34508, 45936, 57444, 12020, 48172, 53940, 43456, 58580,
                19696, 11236, 6916,  15820, 22180, 52500, 45316, 18488, 55248,
                7424,  20424, 27880, 62420, 32492, 23808, 19656, 36108, 52004,
                41284, 24656, 55004, 1424,  36528, 18452, 6180,  46584, 62976,
                57504, 42116,
            },
            {
                3532,  55472, 39208, 9024,  62796, 28888, 17832, 6140,  25436,
                34536, 28080, 7132,  18112, 24932, 12876, 41888, 17760, 34724,
                27408, 38456, 46096, 6400,  17248, 43784, 14336, 47648, 32520,
                51204, 6188,  45964, 62500, 26624, 18900, 46640, 10128, 32188,
                16800, 62972, 724,   58076, 15484, 39104, 43220, 1232,  9088,
                26812, 63284, 30152, 16180, 7704,  36212, 21632, 10360, 61352,
                3280,  55524, 65480, 20788, 3980,  33908, 9712,  5960,  31672,
                52704, 8272,  43516, 54772, 17792, 40808, 57712, 49924, 63400,
                7828,  24772, 52520, 30800, 64716, 26472, 9380,  34548, 63604,
                25260, 15080, 51436, 22168, 43864, 47584, 24684, 2336,  54220,
                4188,  26604, 61888, 19036, 276,   41296, 20880, 9644,  26036,
                40028, 47608, 64352, 27728, 41212, 35052, 62992, 29792, 14456,
                35420, 46960, 53124, 16020, 57056, 46108, 6456,  13756, 58268,
                16636, 32284, 11240, 44416, 64544, 41548, 55948, 16404, 27308,
                32528, 21324,
            },
            {
                62288, 17016, 26348, 47748, 4172,  40272, 10712, 49832, 61932,
                55784, 11868, 52452, 36116, 31544, 49604, 56156, 5344,  47864,
                61032, 57528, 19932, 25940, 60372, 8912,  22308, 62128, 18012,
                28976, 56368, 38308, 1036,  41476, 59276, 2524,  39584, 57060,
                27684, 13044, 46880, 29972, 23996, 65324, 20928, 31076, 16752,
                49088, 3548,  11008, 38592, 48248, 64620, 4448,  44948, 23900,
                17100, 42688, 30504, 50580, 45908, 58252, 61780, 23428, 47968,
                40504, 34924, 60804, 23968, 36972, 7076,  30268, 992,   33040,
                17128, 56576, 41208, 6540,  15456, 57452, 45024, 20352, 2508,
                55264, 45736, 61668, 11192, 57884, 16316, 38804, 50740, 29152,
                44628, 13508, 8960,  51316, 35832, 29496, 65496, 15736, 55000,
                1272,  33532, 44376, 56184, 13080, 8420,  4812,  26072, 49540,
                58284, 3924,  37900, 9700,  2612,  50572, 63796, 26800, 8884,
                61460, 5644,  51432, 28076, 21848, 8756,  50524, 39084, 4612,
                10336, 53144,
            },
            {
                50164, 36968, 20448, 60328, 14392, 56556, 35368, 30792, 42936,
                21488, 3972,  58760, 44864, 3148,  62544, 38924, 21404, 10848,
                31156, 15108, 2276,  41636, 49764, 29348, 39148, 57784, 4228,
                13204, 24972, 33236, 16620, 53304, 29932, 22972, 48972, 35896,
                8012,  51560, 6428,  37396, 49416, 11188, 34804, 53320, 60708,
                44228, 20216, 23960, 56532, 32376, 13852, 28956, 51552, 58620,
                27328, 6248,  36304, 11144, 25976, 16768, 51740, 28184, 1924,
                19468, 15696, 4300,  12256, 46048, 64236, 53408, 13828, 47708,
                36788, 60572, 1736,  48588, 22284, 39068, 5132,  59236, 28780,
                36864, 8348,  18960, 35376, 27780, 5724,  60176, 33112, 18340,
                63752, 40324, 31716, 56224, 24880, 5112,  50108, 37268, 31488,
                61176, 17604, 5308,  20600, 48212, 60552, 54348, 39876, 22344,
                10976, 61712, 25192, 40892, 22076, 34320, 42932, 31100, 39896,
                45752, 19192, 34844, 57568, 2960,  31408, 12972, 59152, 47700,
                23944, 43824,
            },
            {
                904,   30184, 7368,  32720, 50660, 23536, 19324, 380,   47564,
                15180, 29088, 40256, 23316, 8812,  15508, 26864, 53500, 44480,
                24168, 52220, 35224, 62904, 11864, 33572, 84,    52204, 35940,
                43604, 65248, 20856, 50200, 10824, 44756, 13860, 4492,  64740,
                20568, 41116, 60240, 17984, 2492,  56196, 7324,  25768, 12988,
                35616, 52080, 62424, 42572, 660,   18556, 54352, 34032, 2208,
                39560, 14564, 53924, 62480, 1048,  40896, 36964, 12976, 44672,
                62396, 55256, 29896, 50800, 33392, 19056, 22820, 42996, 26636,
                10168, 20304, 29344, 34876, 51696, 11060, 31664, 16888, 41836,
                50308, 23244, 32100, 1708,  49756, 40692, 13144, 9436,  23252,
                55532, 6716,  21216, 43640, 14352, 59668, 44896, 7524,  12916,
                23744, 52444, 38260, 30276, 24608, 36648, 16628, 420,   43196,
                33136, 17636, 30040, 45200, 60260, 12084, 17256, 55988, 292,
                25180, 48712, 38484, 15004, 61764, 25628, 35888, 20168, 63904,
                34352, 15020,
            },
            {
                61248, 11816, 58648, 45116, 2868,  41848, 64608, 52924, 9276,
                38180, 63824, 50848, 19568, 46784, 60156, 33748, 488,   63564,
                7976,  39732, 4808,  21540, 46792, 54528, 24556, 19512, 10660,
                47064, 7384,  54792, 3568,  35012, 61904, 57676, 17620, 31080,
                53868, 25440, 33788, 45208, 28552, 61860, 41700, 47516, 4100,
                39892, 8428,  28028, 6496,  47056, 60432, 40612, 11504, 20740,
                49192, 60352, 18984, 43596, 32156, 8060,  21420, 57608, 33048,
                10276, 37996, 25672, 59824, 9536,  3380,  39108, 58644, 5092,
                54248, 44984, 13420, 61900, 25188, 43600, 63080, 52956, 13468,
                4148,  65492, 56860, 43164, 54472, 20968, 63040, 46632, 36112,
                916,   48684, 37916, 61084, 2116,  18296, 26952, 57380, 48948,
                41772, 9324,  57976, 1952,  50940, 10544, 31968, 46444, 65208,
                52132, 6568,  56360, 1372,  53648, 28692, 5236,  49632, 21108,
                65368, 12196, 3976,  54428, 42412, 45488, 51856, 1904,  29236,
                8084,  40640,
            },
            {
                47204, 24644, 38920, 53996, 28044, 12328, 36904, 26500, 56780,
                32808, 6304,  12580, 35148, 55900, 29744, 11820, 37196, 18304,
                48804, 56276, 27976, 58636, 16260, 7228,  63832, 42056, 27568,
                60864, 31092, 14912, 39536, 27904, 6624,  24648, 38440, 43820,
                10412, 14944, 4180,  52108, 22876, 14424, 19968, 54944, 30096,
                17956, 59400, 14900, 34396, 22136, 26368, 8696,  63816, 44128,
                29592, 7452,  34664, 23204, 55668, 48328, 63688, 4136,  52816,
                22604, 47288, 1112,  41136, 56440, 28396, 49072, 16244, 31112,
                65020, 38188, 3328,  55492, 7592,  18928, 472,   34124, 26484,
                46992, 39440, 10528, 14588, 25052, 4684,  30464, 56996, 26780,
                52276, 16576, 28464, 10164, 52812, 40432, 34720, 21596, 2472,
                28776, 64104, 14792, 44872, 62488, 19392, 57496, 27152, 13436,
                21064, 36856, 48204, 14368, 19744, 35808, 62452, 10172, 37116,
                32552, 58104, 29420, 22468, 9860,  16956, 6736,  38292, 54052,
                18372, 56180,
            },
            {
                33276, 5196,  21248, 17648, 8160,  58128, 16336, 4980,  45380,
                18464, 24928, 59464, 1572,  42604, 5720,  50232, 25428, 42124,
                13928, 32504, 9928,  43296, 37904, 30384, 13632, 50548, 4924,
                37732, 21824, 58572, 48244, 42876, 19904, 49912, 55412, 284,
                47744, 61432, 56712, 39764, 8332,  32596, 37596, 820,   63260,
                23700, 45396, 53556, 38196, 57972, 50144, 16048, 36716, 24756,
                52688, 13140, 46604, 3160,  28404, 14032, 17800, 30452, 42060,
                6560,  65188, 17156, 21040, 44560, 13008, 60900, 35648, 8556,
                21496, 50248, 17656, 28160, 40140, 48060, 56396, 22420, 59724,
                6848,  18428, 28256, 60388, 37536, 45436, 17092, 7836,  42140,
                12636, 64924, 33984, 24148, 47816, 30932, 11468, 61672, 46612,
                35972, 18980, 25784, 33824, 41424, 5988,  38952, 3320,  59688,
                8844,  41800, 62792, 26740, 39524, 47076, 24100, 52600, 43332,
                15940, 7280,  40512, 50104, 33520, 64284, 57216, 23588, 12652,
                43400, 27048,
            },
            {
                15708, 65332, 52168, 35032, 61548, 43084, 31768, 22056, 62708,
                40956, 47772, 52528, 27556, 17232, 65256, 20812, 54240, 58940,
                4116,  23212, 64644, 19380, 1544,  45760, 59984, 32992, 17504,
                56484, 9068,  25572, 1180,  63420, 12536, 32676, 8768,  27588,
                35332, 21664, 30256, 12372, 64300, 48208, 58704, 42684, 9696,
                50592, 31648, 4824,  11880, 2968,  30920, 45876, 5852,  1244,
                56208, 38060, 64256, 10376, 58428, 51276, 39272, 45892, 12044,
                27412, 35848, 50320, 7796,  34116, 24320, 52780, 148,   46876,
                25604, 33560, 43064, 58984, 9864,  36364, 30148, 12060, 43952,
                35756, 53576, 31560, 48648, 3072,  52632, 33868, 59452, 22068,
                4276,  45184, 57204, 6144,  62776, 3616,  54304, 16196, 6404,
                55568, 49788, 4508,  12056, 54748, 22460, 47788, 29668, 50568,
                24256, 31660, 4108,  11100, 57824, 7804,  30660, 1848,  55564,
                27492, 46892, 60720, 18968, 756,   27696, 46788, 31576, 62376,
                49184, 2820,
            },
            {
                57112, 9660,  40224, 29644, 47340, 1212,  49684, 54100, 10716,
                2924,  14088, 31228, 37416, 9824,  45748, 39516, 2252,  30476,
                47072, 35760, 53056, 50172, 26172, 56016, 22664, 40444, 2032,
                52908, 45220, 34264, 17864, 51896, 36976, 59496, 16300, 62928,
                42156, 18188, 2296,  44640, 24824, 5064,  17040, 26580, 13744,
                36664, 20840, 64860, 43620, 54884, 19912, 62416, 33284, 59144,
                17776, 21888, 31636, 41968, 25712, 35992, 308,   24160, 61392,
                57168, 14932, 54500, 29772, 63212, 4424,  19328, 42164, 62820,
                11752, 53472, 5880,  14856, 23896, 64500, 3104,  50048, 16176,
                62756, 1248,  21924, 8920,  64328, 20320, 11316, 39552, 49792,
                31788, 37452, 19788, 15048, 42676, 21032, 38084, 44040, 32176,
                23104, 40456, 59976, 30016, 51320, 8048,  15384, 35696, 42912,
                17216, 55420, 34632, 51824, 16136, 61244, 41672, 18664, 13136,
                63348, 4292,  24516, 53724, 36860, 14132, 40008, 4008,  10224,
                21604, 36668,
            },
            {
                50564, 45212, 12992, 4288,  23892, 14660, 37988, 27364, 34556,
                57336, 22624, 62088, 6828,  54872, 23980, 33444, 15620, 9200,
                60700, 16980, 6408,  12864, 33756, 4452,  10856, 49276, 26664,
                13264, 64516, 41652, 10752, 29604, 6412,  45764, 23556, 4936,
                51156, 58604, 37516, 55044, 50376, 35400, 29584, 52836, 57036,
                7124,  48912, 25756, 17288, 28744, 39016, 13472, 51292, 27204,
                40676, 6876,  50224, 4560,  19424, 60032, 8612,  53412, 19784,
                32032, 3200,  40116, 10844, 45744, 36892, 57652, 30608, 15816,
                39844, 2464,  61332, 31440, 51496, 44972, 20732, 40564, 8480,
                24712, 57936, 41724, 36372, 15892, 43416, 28760, 62308, 1324,
                25624, 55312, 9096,  50936, 29372, 59212, 25996, 7948,  63272,
                13560, 196,   37012, 17572, 64704, 26708, 56840, 61844, 11284,
                876,   64048, 20444, 44060, 23028, 3116,  33840, 46284, 22204,
                38616, 34308, 11388, 44312, 8004,  59780, 19976, 52744, 42000,
                60120, 29976,
            },
            {
                324,   19216, 26856, 63308, 55944, 20220, 59900, 8448,  17596,
                39568, 44188, 49428, 20076, 60052, 13024, 51024, 62952, 26464,
                38080, 45068, 28624, 41100, 62632, 37360, 18492, 61524, 29888,
                36592, 5900,  23080, 61028, 56348, 20616, 40948, 53956, 31800,
                10880, 26068, 7252,  15824, 20604, 10348, 62956, 45048, 2884,
                33468, 58812, 41536, 260,   60324, 8436,  23024, 43072, 10560,
                47820, 15252, 61292, 54616, 45084, 15800, 33712, 47604, 5276,
                43512, 48904, 22552, 58748, 26300, 14404, 6328,  50632, 23240,
                28348, 56324, 46372, 37608, 18128, 5108,  57636, 27252, 52404,
                33020, 46344, 12840, 50492, 26032, 56820, 5912,  53876, 13620,
                17448, 60340, 40584, 2504,  35452, 12420, 46272, 18420, 51228,
                27768, 56748, 9424,  46324, 43112, 2004,  33056, 19984, 53088,
                46364, 28820, 5968,  48892, 37316, 28412, 56652, 50260, 8964,
                58744, 51572, 16684, 29760, 56560, 47800, 26024, 34620, 16868,
                7292,  24048,
            },
            {
                32788, 58916, 42596, 51716, 30924, 5872,  44968, 50948, 64948,
                5160,  28636, 124,   35336, 41324, 29788, 5860,  43384, 56552,
                19604, 564,   54676, 58384, 21572, 47492, 7180,  43352, 57656,
                16272, 50716, 31844, 3824,  47844, 26468, 1468,  13800, 38192,
                48608, 65244, 33724, 42940, 60592, 1080,  40484, 22696, 19088,
                38752, 15444, 10268, 52972, 35524, 48564, 55816, 3096,  65240,
                30084, 34860, 23640, 12132, 37788, 29232, 64756, 11596, 26044,
                36752, 62532, 17968, 1816,  52496, 38780, 47168, 9272,  60036,
                35596, 19480, 7540,  26168, 10892, 34164, 61072, 13424, 38224,
                18744, 6116,  29180, 59944, 2184,  32756, 47780, 23432, 35552,
                45864, 30604, 48256, 22672, 63696, 53616, 1548,  58156, 34432,
                41924, 48324, 20640, 31940, 23748, 13884, 40096, 4760,  37040,
                24852, 14568, 40428, 59468, 9244,  64832, 14480, 5468,  31304,
                26404, 1460,  42512, 65008, 22692, 2300,  12096, 63700, 44452,
                54780, 39128,
            },
            {
                47768, 7888,  16592, 37140, 10832, 33956, 40744, 13364, 24424,
                32604, 53824, 11712, 16644, 57940, 48132, 2636,  22032, 34920,
                11208, 49812, 24552, 9032,  15060, 30892, 53628, 24208, 2836,
                46488, 19800, 54888, 39176, 15496, 33952, 63820, 57436, 28652,
                3248,  22304, 13224, 47324, 27256, 32108, 56128, 12260, 47900,
                60736, 29368, 23460, 46416, 31448, 5100,  26356, 18288, 37220,
                58184, 768,   43388, 26964, 3624,  57316, 22040, 41660, 55548,
                13776, 8848,  28800, 33652, 64224, 20324, 32008, 54840, 1344,
                44448, 13460, 49304, 62940, 54176, 41936, 792,   30576, 55960,
                3508,  63396, 53748, 21432, 39156, 10632, 18648, 41504, 8192,
                64628, 4588,  11244, 26732, 16568, 31876, 39592, 10308, 24460,
                4608,  15520, 62332, 6220,  53736, 49512, 60768, 45340, 8688,
                58044, 51096, 32340, 12284, 25788, 19484, 52900, 39944, 44868,
                62148, 20588, 35916, 5972,  39472, 33188, 50084, 29132, 4796,
                14360, 61212,
            },
            {
                11548, 53652, 22688, 1600,  48944, 62272, 28132, 2280,  55708,
                19084, 46192, 63544, 25492, 8860,  38344, 27640, 52644, 14680,
                64220, 39232, 32652, 44364, 3348,  65160, 39548, 12440, 34140,
                8728,  59772, 27792, 11420, 43568, 7672,  51016, 19428, 45124,
                16860, 54364, 39452, 58332, 4400,  50772, 36936, 7808,  26328,
                51768, 5980,  63740, 13252, 20364, 62256, 40160, 13988, 51900,
                21016, 9396,  53396, 63156, 49048, 7824,  51600, 1384,  30808,
                60480, 50040, 40576, 5504,  43776, 11856, 25408, 17084, 41368,
                65340, 22560, 30996, 39352, 15848, 21276, 45984, 49116, 23376,
                40908, 34360, 8308,  15376, 45480, 62644, 51548, 58948, 27164,
                38280, 20876, 52264, 57668, 44740, 7184,  49476, 21184, 65348,
                52944, 29380, 39292, 58080, 35232, 10404, 28024, 21380, 30876,
                62400, 18536, 2840,  55520, 43572, 120,   35040, 23764, 17220,
                12152, 48268, 53284, 14556, 55516, 18816, 9056,  57596, 37488,
                21220, 27208,
            },
            {
                35680, 64876, 41620, 25748, 57984, 16108, 21116, 47752, 59636,
                37204, 6880,  42300, 51844, 32940, 18196, 62344, 45576, 7648,
                30164, 4096,  59912, 17416, 36264, 51072, 27952, 20428, 62116,
                42252, 37188, 412,   49436, 58564, 24692, 36552, 10000, 41544,
                62248, 6388,  30036, 9480,  18636, 24156, 15076, 64820, 44172,
                3544,  32912, 42148, 54580, 8836,  44052, 57484, 33152, 7152,
                46232, 40404, 16508, 32180, 19096, 39760, 34364, 16864, 44780,
                19876, 24292, 54380, 16052, 48452, 56924, 60932, 4372,  33548,
                10356, 52840, 5764,  2748,  57360, 28972, 9460,  64752, 16788,
                11164, 43580, 49452, 25200, 36720, 5000,  30076, 1076,  15872,
                55584, 32612, 42388, 3188,  36064, 61356, 28748, 13396, 44224,
                36544, 1128,  12116, 25752, 18604, 2752,  56364, 15324, 41756,
                6288,  35180, 48372, 38724, 62916, 30300, 47680, 60928, 3940,
                57356, 32576, 8572,  27276, 46092, 60376, 41412, 24428, 46812,
                51808, 3212,
            },
            {
                45300, 18936, 32440, 4536,  54604, 38856, 7656,  43264, 10384,
                30112, 14848, 22536, 60532, 5324,  12300, 57224, 24300, 40592,
                53992, 20240, 47944, 26684, 56112, 10532, 1520,  48536, 54420,
                25472, 13964, 64780, 22016, 17432, 30088, 61096, 2324,  32924,
                23384, 49868, 36040, 45464, 61596, 53792, 41340, 30332, 17976,
                55752, 37876, 16312, 25584, 35996, 1136,  28628, 48936, 61708,
                25140, 29428, 59536, 13020, 46884, 25344, 10804, 63644, 4312,
                37984, 7304,  59508, 35092, 1004,  22736, 29612, 38332, 50076,
                26396, 58224, 35144, 43460, 24952, 51532, 6468,  36132, 26512,
                58320, 19520, 60760, 31764, 55832, 13736, 22160, 43920, 49232,
                11896, 6536,  63048, 14264, 19524, 24792, 54936, 5224,  32640,
                17536, 60924, 46004, 50924, 42532, 63692, 37960, 47260, 54332,
                11628, 23188, 27856, 16164, 21508, 7748,  54668, 10684, 42404,
                28864, 38788, 62712, 23844, 3796,  30700, 13516, 744,   17400,
                62284, 30208,
            },
            {
                6300,  14008, 59576, 46388, 12592, 30636, 24080, 64404, 34944,
                52784, 1684,  39484, 28936, 49200, 43288, 35692, 984,   16060,
                61648, 11676, 42468, 5300,  22800, 60980, 44764, 32164, 16000,
                4244,  30712, 45968, 34896, 54108, 4604,  46760, 52456, 12676,
                26704, 56096, 64,    21036, 12776, 33780, 2136,  58440, 10708,
                22144, 49140, 2408,  59372, 51060, 17484, 22732, 11580, 3632,
                36700, 55380, 6416,  1624,  36152, 60272, 53760, 28100, 57700,
                46836, 31696, 11092, 27248, 42236, 9020,  53160, 44768, 14216,
                20452, 46496, 17272, 62520, 11840, 33396, 60488, 44512, 2824,
                52516, 28804, 188,   7096,  65432, 40660, 53032, 34292, 61508,
                25056, 47316, 29104, 39812, 51620, 9192,  41508, 59444, 48092,
                23140, 54056, 7476,  31024, 22224, 5140,  32988, 25148, 708,
                64696, 44472, 57500, 4512,  51392, 13880, 37276, 26100, 19148,
                50340, 980,   16268, 43348, 52268, 36524, 63940, 54288, 34316,
                9972,  40556,
            },
            {
                55908, 50672, 27796, 8600,  35872, 52316, 2680,  49460, 17080,
                26160, 58900, 46956, 19320, 3720,  53800, 21156, 31684, 51144,
                28252, 37272, 58196, 33120, 13980, 40632, 18828, 6904,  56828,
                38476, 52032, 10328, 7140,  39948, 14788, 42356, 19892, 59116,
                38996, 15884, 64548, 42904, 27828, 49044, 6200,  46144, 35360,
                28032, 62876, 12512, 30156, 45888, 64392, 38748, 56140, 15388,
                19344, 41720, 23076, 51840, 43656, 21080, 5424,  41500, 22948,
                13556, 52016, 64824, 20416, 37492, 62108, 18284, 6952,  63476,
                2392,  29196, 8136,  38392, 48180, 19908, 15340, 40064, 55176,
                13564, 38124, 47884, 23664, 17892, 10908, 28092, 2900,  19712,
                36956, 17168, 56604, 844,   33680, 46532, 16016, 2652,  38268,
                9900,  27592, 40128, 15724, 58400, 12632, 52776, 17608, 30020,
                50216, 36804, 9308,  61372, 40300, 45600, 33100, 59188, 5940,
                55360, 34904, 21440, 58624, 11156, 19368, 6004,  43732, 25600,
                58680, 20848,
            },
            {
                37064, 23680, 63968, 17768, 44016, 61148, 20148, 41204, 4996,
                55912, 11908, 33564, 63072, 14044, 26488, 65428, 10032, 46808,
                3404,  23804, 8032,  50064, 54552, 28928, 35700, 64056, 23932,
                42588, 21056, 62456, 27108, 57852, 23492, 64172, 28352, 5188,
                47720, 10720, 32324, 7564,  57180, 25032, 38024, 20028, 60364,
                8056,  43716, 40348, 19780, 5772,  9340,  32352, 52580, 45000,
                63452, 32840, 48024, 61912, 8984,  30616, 15120, 49884, 48,
                35660, 17696, 3032,  45796, 50668, 12808, 31856, 55760, 35980,
                41088, 59896, 54692, 23156, 1224,  56280, 30264, 4524,  22396,
                32480, 9224,  43020, 34956, 51108, 45684, 58548, 8516,  54272,
                42924, 9844,  60128, 22132, 26820, 64520, 55904, 30024, 19156,
                62552, 56720, 1900,  34368, 48364, 44072, 61948, 7752,  40960,
                13740, 19660, 32780, 24868, 29192, 1476,  65328, 22824, 14868,
                45100, 63648, 7848,  48440, 39996, 27776, 50280, 31972, 14740,
                48508, 1972,
            },
            {
                11660, 42168, 33432, 240,   57352, 26808, 14220, 32900, 62648,
                22048, 44180, 8232,  37432, 55408, 41684, 6644,  38504, 59528,
                19112, 44460, 64376, 16984, 68,    46732, 9452,  50424, 12492,
                59676, 2196,  18192, 49648, 32004, 972,   37200, 8988,  34408,
                54800, 22632, 51944, 40200, 17088, 63320, 53132, 14236, 50648,
                23928, 4464,  53324, 34392, 58452, 24316, 42280, 4780,  26680,
                2012,  10524, 14476, 27632, 18316, 56784, 38464, 32800, 61188,
                55856, 43408, 29796, 57540, 25084, 4620,  47556, 27416, 21868,
                10608, 50732, 32000, 14052, 42788, 64428, 25536, 46172, 58776,
                50180, 63920, 15936, 59936, 4068,  25792, 32312, 38964, 64332,
                5260,  30912, 50744, 45308, 4732,  12184, 36384, 23864, 49840,
                13536, 43552, 20920, 64940, 9576,  26756, 37164, 22900, 56480,
                60312, 3152,  47960, 55824, 17316, 52284, 9920,  48640, 40856,
                30436, 12384, 25064, 32636, 2452,  56540, 22272, 8628,  39052,
                61060, 28920,
            },
            {
                54572, 49636, 5800,  21812, 39280, 10300, 48484, 7188,  37892,
                28268, 51112, 480,   24444, 30456, 17068, 48656, 33472, 53152,
                12148, 35340, 29724, 39724, 21808, 62208, 25736, 4648,  33484,
                28712, 47040, 35432, 43680, 11884, 55800, 50872, 43256, 17392,
                60820, 3932,  28836, 46564, 2784,  11268, 30784, 976,   39604,
                32420, 56056, 15536, 26908, 49780, 13196, 61048, 20964, 30772,
                58136, 38008, 54564, 34972, 2544,  65344, 46276, 10972, 26336,
                7872,  21472, 38712, 6472,  15652, 34500, 59208, 704,   43696,
                16444, 25372, 3968,  35492, 52540, 7316,  10652, 34768, 18352,
                5944,  27448, 21520, 39364, 55108, 14144, 20560, 49520, 15648,
                23840, 35140, 13616, 38680, 18820, 52476, 7508,  41132, 33256,
                5864,  52120, 36476, 29484, 3928,  19548, 51460, 2172,  34104,
                27268, 43452, 6476,  12088, 42092, 21060, 4944,  36432, 27580,
                2076,  53680, 38176, 59960, 15468, 45872, 65136, 53360, 3828,
                45176, 17052,
            },
            {
                31180, 61988, 15240, 46592, 53316, 29820, 60024, 43652, 54740,
                13488, 18240, 58340, 46224, 10996, 60936, 22756, 1392,  15220,
                26052, 55304, 5240,  57516, 13404, 37044, 53560, 41260, 16588,
                55192, 8096,  14676, 60956, 5808,  21348, 14160, 30388, 25596,
                45292, 37740, 19588, 59480, 34476, 23116, 44300, 62032, 17740,
                46940, 11084, 65436, 38244, 228,   47140, 35688, 16468, 51184,
                46424, 7480,  24276, 49716, 40328, 6128,  23732, 54032, 16308,
                48696, 63588, 10568, 54184, 61572, 40400, 19152, 52672, 65044,
                37844, 48436, 62336, 19024, 27868, 38948, 49160, 61160, 41824,
                12076, 53168, 1376,  30292, 7900,  63180, 43684, 652,   29456,
                47148, 57232, 2340,  62100, 28276, 43252, 60444, 57068, 136,
                46832, 25392, 16552, 54560, 42144, 60764, 11508, 45700, 15448,
                53492, 21456, 63340, 37968, 58752, 30728, 55020, 62324, 17664,
                58220, 50584, 18940, 43068, 5736,  29908, 12224, 18540, 36068,
                25048, 6944,
            },
            {
                40180, 9288,  56212, 25276, 36300, 4652,  16504, 23936, 2860,
                31296, 40488, 62172, 34572, 5904,  51632, 43128, 28612, 62884,
                40724, 46188, 9744,  48832, 32196, 3352,  19828, 45452, 57928,
                22880, 37720, 52712, 26112, 40628, 47948, 63332, 57276, 1868,
                12564, 64008, 8260,  14648, 49996, 56408, 36588, 8816,  28444,
                59408, 3672,  21856, 29904, 18668, 57152, 6308,  40480, 11424,
                64316, 19500, 42484, 59352, 12368, 20360, 43560, 29912, 4280,
                34568, 42108, 27964, 32732, 22936, 45564, 9364,  29848, 13608,
                5604,  56776, 8800,  45180, 59544, 21736, 16392, 2672,  29060,
                36840, 44620, 57956, 48220, 33980, 11664, 36736, 60500, 53280,
                10452, 41356, 21580, 54152, 32608, 9668,  14656, 21308, 28868,
                63208, 11048, 59368, 7396,  49152, 31908, 39032, 57920, 29832,
                9544,  49236, 34628, 25780, 14924, 46556, 33816, 11796, 43744,
                6860,  22572, 9776,  34840, 61612, 23636, 41112, 32396, 57196,
                63000, 20564,
            },
            {
                35068, 27528, 43412, 2536,  63404, 19808, 50456, 34280, 65104,
                9168,  49364, 20940, 14812, 39228, 56428, 19412, 9180,  57952,
                3240,  20264, 24272, 61420, 43216, 27692, 60208, 10756, 30340,
                752,   65068, 19264, 31236, 2732,  34580, 24032, 7744,  51712,
                41080, 31600, 53624, 26332, 42040, 5148,  20260, 53968, 24988,
                41776, 35440, 45044, 52356, 10088, 43052, 62036, 25240, 29080,
                34064, 1396,  15832, 28384, 32240, 51468, 62076, 37288, 57808,
                19276, 52084, 1296,  14056, 49960, 3700,  37000, 58344, 24116,
                33792, 40828, 30752, 12516, 312,   54412, 32268, 56900, 23708,
                63112, 19128, 14596, 41084, 22316, 51792, 25336, 6364,  18888,
                27156, 63924, 16088, 5844,  48392, 25288, 35788, 50972, 17388,
                38152, 32512, 40372, 22772, 13236, 25104, 18376, 6056,  64656,
                40524, 17532, 3704,  60448, 8284,  884,   23620, 49732, 26460,
                37372, 64512, 46872, 27436, 748,   48556, 52484, 9444,  1744,
                50256, 46860,
            },
            {
                5056,  60548, 18524, 13232, 31584, 41144, 57888, 11700, 45204,
                25960, 57148, 2168,  29376, 24936, 3904,  32656, 45492, 37412,
                30776, 51252, 34504, 17120, 52832, 7860,  15284, 50092, 39576,
                6724,  44020, 49104, 9520,  54924, 16432, 38488, 46540, 18380,
                35572, 21704, 47612, 3336,  29816, 64856, 12724, 48032, 6156,
                51456, 14040, 62580, 7532,  55208, 32496, 21600, 4220,  52912,
                56328, 44544, 61484, 4880,  55228, 8524,  14712, 1988,  46944,
                12796, 24900, 60516, 43968, 18060, 62668, 26608, 51172, 1784,
                47200, 16776, 22440, 51524, 36996, 42184, 13864, 46524, 4380,
                50028, 9012,  26284, 62064, 2696,  16900, 57480, 39704, 45804,
                34272, 7712,  37464, 45072, 59084, 1620,  65280, 44320, 6800,
                53848, 3296,  46244, 56640, 1788,  61852, 35860, 47792, 316,
                23848, 32056, 51088, 42400, 54028, 19636, 40032, 56848, 3008,
                15924, 31924, 12928, 54076, 57788, 16484, 37340, 26736, 42672,
                15632, 12136,
            },
            {
                23720, 51932, 38068, 58796, 48184, 7652,  22504, 38560, 53744,
                5088,  36180, 47428, 42816, 53328, 64116, 12548, 50288, 16368,
                7552,  64808, 12436, 1236,  38608, 22708, 63276, 35136, 25248,
                59120, 12828, 27928, 62564, 42336, 58552, 11408, 28244, 62768,
                5652,  57900, 10840, 61540, 17284, 33240, 40072, 58584, 16520,
                31516, 1844,  23692, 39320, 27312, 16128, 48164, 37708, 13872,
                9156,  23340, 36796, 47652, 39692, 25936, 33552, 21972, 64580,
                40588, 5616,  35500, 55980, 7660,  31624, 11220, 42712, 20440,
                63612, 55080, 27476, 61716, 6452,  25896, 64592, 10100, 39324,
                30704, 35008, 5532,  53932, 28792, 47588, 31520, 3872,  12900,
                59792, 51092, 30200, 23088, 18136, 39792, 30560, 12356, 23784,
                61316, 19488, 27936, 15624, 43540, 51116, 28788, 55504, 13992,
                44588, 56968, 11476, 28052, 36312, 64384, 30080, 9636,  61348,
                51748, 41872, 3880,  39260, 20128, 7420,  63672, 21928, 58740,
                29812, 54688,
            },
            {
                32748, 1252,  44408, 10260, 28208, 54828, 568,   15672, 28916,
                19120, 13220, 62672, 9944,  17568, 35624, 26920, 60684, 22868,
                55028, 25924, 41908, 58872, 47440, 31500, 54652, 2376,  46028,
                17900, 33108, 37028, 23248, 4024,  20348, 32704, 348,   54456,
                25332, 15568, 43116, 37668, 23544, 46040, 500,   27184, 37568,
                56488, 19384, 33872, 50356, 12080, 58540, 1064,  63712, 41480,
                49880, 31124, 20540, 11276, 18032, 60540, 44736, 50620, 27800,
                9708,  53640, 29528, 39080, 21836, 47536, 57200, 14780, 38572,
                6664,  10416, 44012, 3316,  48808, 33912, 17840, 52420, 21236,
                55372, 59224, 45948, 38056, 10580, 42836, 65084, 21464, 55448,
                24500, 43008, 3368,  13464, 56052, 49620, 4236,  57856, 41256,
                48908, 9252,  37108, 63928, 33608, 10648, 7192,  20132, 26796,
                37588, 62120, 5660,  22516, 16296, 48040, 13248, 44840, 34968,
                19048, 59344, 24780, 29548, 44524, 33176, 13272, 46160, 3164,
                39388, 64908,
            },
        },
        {
            {
                1836,  64308, 32508, 51376, 43680, 63380, 47472, 35988, 16016,
                51860, 23164, 18448, 26936, 50936, 64400, 31256, 13584, 17876,
                2444,  41036, 22028, 28764, 192,   48444, 11200, 43212, 20068,
                30444, 8804,  55712, 22304, 61676, 39776, 18268, 54472, 9324,
                5488,  64816, 34068, 53444, 39844, 12492, 26632, 50024, 16332,
                60056, 13216, 39952, 17856, 50884, 58904, 4936,  33948, 40460,
                52176, 61408, 43420, 36008, 55492, 23104, 46880, 4184,  65372,
                20976, 40864, 24376, 36272, 28504, 41688, 17168, 62700, 26208,
                20652, 53464, 65296, 22652, 13844, 28660, 5656,  61472, 53920,
                30912, 64604, 18840, 35124, 12424, 63408, 9196,  18468, 51544,
                28612, 46988, 2920,  33696, 50400, 28196, 40948, 58668, 24964,
                52916, 12024, 35648, 19620, 15548, 4780,  22676, 43724, 18112,
                26080, 38540, 4016,  8940,  50652, 45064, 22800, 48216, 10076,
                24848, 46892, 62772, 27728, 38112, 2780,  33524, 10840, 39896,
                6024,  42976,
            },
            {
                46000, 14072, 25240, 3580,  19540, 11704, 23544, 27184, 5468,
                62876, 9964,  55420, 44460, 35956, 5012,  58652, 46340, 25052,
                49652, 63864, 12296, 44964, 54600, 60840, 38180, 15028, 47644,
                2224,  63208, 27276, 44956, 736,   12056, 46580, 21764, 61356,
                41392, 19432, 16312, 4,     22328, 36492, 62112, 3108,  34700,
                6692,  48404, 25744, 29140, 1808,  21344, 42500, 64880, 18008,
                10252, 49036, 14396, 19108, 1552,  60460, 10768, 42548, 35176,
                30736, 9276,  61144, 52840, 14076, 22556, 59592, 38004, 13264,
                2840,  41452, 32692, 8188,  38080, 52988, 42424, 46448, 472,
                14648, 41652, 23828, 51132, 44160, 21952, 57452, 45872, 1320,
                35892, 58840, 24472, 41928, 21768, 4736,  55236, 1936,  16748,
                39068, 44048, 49880, 8380,  31012, 37356, 52068, 1004,  33584,
                49632, 63980, 29900, 17620, 62336, 27884, 6784,  55204, 65200,
                29464, 53852, 19216, 9624,  49824, 42032, 55576, 63692, 15268,
                24556, 52884,
            },
            {
                9548,  35016, 38320, 57548, 41432, 31184, 58704, 49712, 42672,
                38360, 29588, 6696,  40416, 15552, 20648, 28444, 42024, 34424,
                7592,  55524, 38988, 24680, 16712, 30996, 6388,  26372, 59832,
                34268, 40952, 14332, 53076, 36736, 32092, 51372, 3652,  35336,
                30716, 49128, 44060, 56116, 31936, 58404, 9540,  46308, 21860,
                56084, 31892, 9668,  61608, 36372, 15140, 49340, 24436, 0,
                27220, 32108, 6944,  64248, 26808, 33768, 51740, 18012, 54192,
                2216,  49540, 44248, 19032, 1140,  47992, 8780,  30388, 50792,
                57640, 27608, 15508, 48108, 59332, 19236, 11688, 32276, 21420,
                58320, 47824, 3508,  55952, 6324,  38924, 27124, 32996, 13532,
                39572, 6480,  17076, 62476, 12304, 46044, 19404, 35168, 64556,
                32184, 22260, 60976, 26924, 56604, 65140, 11484, 28688, 60640,
                6904,  12604, 21136, 54828, 37016, 13892, 41276, 18744, 38036,
                15804, 220,   33332, 56992, 13736, 25404, 7672,  21532, 36320,
                57856, 29812,
            },
            {
                17760, 50584, 6456,  61908, 15204, 52672, 9044,  3864,  17872,
                53764, 34140, 21688, 57752, 48260, 2812,  53288, 10524, 60116,
                19612, 30040, 3724,  51428, 8700,  41760, 56696, 52308, 22840,
                10720, 49800, 5668,  19412, 65520, 8520,  26828, 59924, 13368,
                24240, 4704,  62888, 25972, 14452, 48296, 18276, 28056, 38400,
                65524, 43032, 4604,  54448, 44908, 57520, 8500,  38648, 54912,
                46400, 57804, 39696, 52868, 44992, 12792, 29852, 7528,  62384,
                26460, 15572, 38440, 33192, 64600, 54232, 34896, 4120,  45048,
                19060, 61836, 5536,  43592, 24236, 2916,  64148, 36444, 26128,
                39252, 9252,  33476, 29540, 16928, 48872, 3968,  53940, 60452,
                20596, 48476, 55876, 30580, 37784, 59604, 26996, 48508, 10652,
                6728,  55584, 14084, 40500, 3428,  48200, 20368, 45100, 53792,
                24004, 40760, 47544, 32432, 2796,  25276, 61108, 31328, 4408,
                51696, 42568, 23276, 36840, 45120, 30052, 59536, 47696, 632,
                41588, 62540,
            },
            {
                54972, 21224, 48056, 27784, 964,   36228, 21340, 46216, 65348,
                13696, 772,   61400, 12148, 32192, 63436, 38468, 16432, 1116,
                47408, 36784, 14408, 61640, 33836, 20828, 4208,  36044, 45568,
                18296, 57268, 30872, 24740, 42204, 56528, 16128, 39344, 45240,
                54760, 10972, 37312, 8104,  40484, 2300,  53160, 42048, 3916,
                15044, 19552, 51292, 12536, 23580, 32628, 28256, 62688, 12268,
                34952, 16056, 22516, 3744,  37196, 20552, 48648, 39924, 22920,
                45500, 58504, 4468,  29192, 11800, 25724, 42352, 56568, 23508,
                10240, 36472, 54516, 29892, 34660, 55832, 49784, 6572,  16116,
                52184, 62796, 13660, 60148, 36880, 10932, 65468, 23592, 8116,
                42260, 27644, 10144, 12,    53096, 8212,  14664, 42144, 51524,
                28780, 1464,  46312, 17512, 24544, 34076, 39476, 8668,  16320,
                35760, 1812,  58724, 10156, 52316, 44164, 49440, 11292, 58432,
                45932, 8956,  63988, 5584,  61264, 3404,  16796, 51980, 27444,
                12884, 4660,
            },
            {
                33388, 40180, 12220, 44312, 24264, 55100, 39904, 32616, 28308,
                24120, 45088, 50096, 26104, 7804,  43136, 23152, 56428, 27092,
                65228, 22248, 42616, 58032, 27568, 49092, 64684, 13484, 29132,
                1628,  38652, 60168, 46912, 3340,  21448, 49376, 29776, 1132,
                58524, 21052, 52000, 29580, 59488, 23060, 62644, 31040, 10796,
                59340, 25188, 34860, 40296, 3184,  18160, 6064,  51864, 20036,
                4644,  59740, 25900, 10348, 63076, 56576, 560,   59576, 13676,
                35924, 10140, 55304, 22008, 46084, 5764,  18180, 13544, 63024,
                40156, 8,     50188, 12740, 17360, 9360,  27812, 41908, 56800,
                1896,  20540, 45844, 24516, 51640, 19408, 43080, 30524, 16052,
                51916, 63192, 34668, 44452, 22964, 32908, 61564, 24036, 58116,
                19984, 36488, 62344, 53088, 6124,  57684, 13392, 62660, 31068,
                56288, 43092, 27196, 64368, 17156, 5440,  34360, 20424, 24256,
                35408, 27528, 17836, 50588, 20660, 32272, 39060, 8760,  34848,
                45660, 23740,
            },
            {
                59628, 8356,  64628, 30680, 16940, 61364, 11372, 59112, 7308,
                55892, 40796, 36676, 18764, 52148, 29872, 13420, 45072, 32840,
                50592, 5004,  11432, 46396, 852,   17472, 10084, 40136, 61156,
                53980, 8828,  15240, 34516, 11748, 36384, 6540,  63504, 18016,
                33636, 41844, 15348, 46104, 34656, 6788,  13376, 44788, 36924,
                55032, 48828, 29572, 63220, 47100, 60448, 41752, 36600, 45444,
                30712, 40744, 49544, 43616, 31644, 17352, 28244, 5520,  32728,
                52140, 18788, 40772, 62756, 50540, 59148, 38820, 28788, 48700,
                33040, 26324, 20852, 58400, 45096, 38368, 22340, 61584, 31660,
                44076, 28968, 7016,  40728, 2976,  58680, 34308, 47276, 1648,
                37276, 24872, 12904, 50048, 17480, 40488, 6056,  2528,  38536,
                45040, 12208, 31368, 42304, 28080, 50164, 1264,  22408, 47068,
                4440,  19368, 13560, 38416, 22812, 56940, 40468, 1660,  53696,
                60276, 13568, 39888, 56044, 10988, 43264, 54224, 65120, 18648,
                56812, 29276,
            },
            {
                15612, 3256,  52408, 37316, 5772,  49820, 2360,  19820, 48028,
                16088, 2912,  10604, 59848, 4460,  37640, 61944, 5852,  9188,
                53860, 18108, 31324, 38576, 23820, 54396, 44408, 31960, 25028,
                21312, 43800, 28376, 50324, 61688, 53796, 42996, 25688, 9700,
                50376, 27312, 2792,  64964, 18900, 49100, 56744, 26656, 20772,
                7552,  2460,  16528, 8960,  21664, 54020, 14112, 24116, 9308,
                55724, 1364,  61544, 14360, 7704,  54444, 42076, 46972, 64768,
                25164, 2412,  31412, 7412,  14904, 33756, 1860,  53188, 7820,
                16260, 60220, 41284, 7156,  63732, 1416,  47568, 13416, 18684,
                10364, 35752, 55364, 64008, 27500, 8820,  13220, 56424, 61508,
                5248,  19072, 58588, 3276,  64344, 54172, 47536, 30228, 15500,
                64136, 7204,  22512, 9780,  60044, 18924, 36708, 41272, 26068,
                61320, 50472, 34060, 6576,  45760, 28064, 63084, 16012, 30472,
                7096,  47892, 2852,  29196, 46792, 25752, 14176, 22932, 48668,
                2096,  42716,
            },
            {
                35304, 25872, 19388, 57504, 45428, 33784, 26508, 42508, 35904,
                29348, 63072, 21492, 33748, 55084, 16708, 48884, 21192, 40844,
                36184, 25536, 56616, 62860, 7600,  34956, 58852, 2892,  48640,
                5592,  63900, 17988, 468,   23560, 31284, 14364, 47312, 38692,
                60944, 5904,  54356, 11332, 24584, 39144, 784,   52416, 32484,
                64340, 45692, 39116, 57108, 33500, 1128,  28908, 48592, 65132,
                17200, 27600, 21328, 34136, 51576, 24296, 36432, 21004, 8472,
                57140, 44064, 49000, 27484, 37208, 19508, 43528, 64472, 22056,
                45672, 3344,  52204, 30584, 24844, 53716, 33304, 4824,  51936,
                59860, 49556, 22564, 16516, 32340, 50056, 39708, 21316, 29092,
                41428, 46560, 31484, 38788, 27876, 20804, 10804, 57160, 24976,
                52252, 34800, 55848, 47560, 15260, 32424, 64280, 52728, 7992,
                11248, 55040, 30008, 59660, 48712, 9176,  12768, 50424, 43788,
                36396, 62776, 22592, 33628, 59160, 1456,  36748, 5968,  60660,
                10920, 51480,
            },
            {
                62744, 47868, 41336, 10388, 22776, 14628, 64056, 53840, 12552,
                50468, 57912, 43692, 26764, 46740, 240,   24852, 57820, 12384,
                61288, 44016, 2316,  14552, 51408, 27952, 19788, 15152, 53216,
                38028, 33412, 57700, 41224, 7996,  58620, 19952, 2312,  56652,
                22680, 31032, 36648, 43332, 57772, 29268, 16920, 9672,  41708,
                14560, 23140, 51952, 26708, 11216, 43976, 58864, 4816,  34500,
                52504, 11988, 38296, 46524, 2244,  62492, 11472, 15232, 50800,
                38340, 17088, 12380, 60316, 54264, 10344, 24708, 31568, 12348,
                56216, 37868, 9916,  18780, 35996, 15336, 57132, 41116, 23852,
                38852, 3796,  12576, 46852, 392,   54536, 25196, 6476,  15200,
                52752, 9552,  55976, 6936,  14012, 36284, 43224, 33384, 18096,
                49020, 428,   26968, 39440, 3472,  45196, 5444,  29020, 17332,
                44680, 39656, 23984, 3420,  18384, 37072, 32708, 56444, 25556,
                19352, 10276, 52812, 16588, 8044,  62460, 41696, 27168, 31124,
                38728, 20760,
            },
            {
                12996, 28772, 54140, 272,   59868, 29976, 39392, 8900,  3576,
                23576, 31816, 8120,  14148, 39772, 64428, 29512, 34544, 3844,
                16236, 28608, 49400, 21668, 39860, 11064, 64172, 42608, 29948,
                9744,  13312, 52104, 27064, 45764, 37788, 64516, 34156, 12432,
                44272, 15844, 20464, 51160, 5208,  33964, 62276, 47808, 58332,
                35508, 5736,  61556, 18864, 37260, 50600, 15928, 39444, 22384,
                43100, 59224, 7064,  56380, 18164, 39748, 58372, 27348, 2988,
                30436, 61784, 23200, 404,   45912, 57612, 50952, 3912,  61092,
                34320, 27396, 48528, 62076, 44040, 10900, 28720, 64588, 8084,
                27188, 33672, 62176, 42580, 36852, 18332, 58380, 43868, 35980,
                64832, 26248, 21988, 44372, 49596, 59048, 2980,  61988, 5672,
                40840, 13480, 61056, 20220, 58476, 24228, 12328, 56868, 34760,
                21392, 76,    14956, 65368, 42748, 53932, 21992, 5556,  660,
                41380, 58136, 38108, 44388, 50032, 20044, 55496, 15852, 53164,
                44636, 4288,
            },
            {
                33032, 7508,  17092, 36656, 50156, 6304,  20488, 46772, 56312,
                17940, 38264, 4872,  53556, 20012, 9824,  43016, 51020, 54868,
                46624, 8328,  33796, 59556, 4564,  47464, 36148, 6764,  56972,
                24348, 47216, 20448, 3940,  11152, 16960, 51516, 6276,  28072,
                48540, 63204, 8280,  59716, 13968, 40408, 3688,  19776, 25180,
                29788, 1640,  42992, 31864, 6660,  63680, 25460, 8664,  30360,
                2512,  48016, 32288, 25396, 29524, 4592,  45520, 33844, 54144,
                43052, 6724,  35572, 41268, 20628, 5492,  28540, 40340, 18144,
                42908, 14524, 22976, 5784,  50784, 2712,  20888, 45816, 17196,
                48240, 57736, 20404, 9444,  29712, 63584, 11020, 32600, 3632,
                48132, 12644, 34232, 62432, 17084, 23912, 9068,  27440, 54660,
                44728, 30024, 37116, 7728,  53448, 49640, 43108, 38088, 61744,
                48944, 58780, 31736, 51668, 26296, 7788,  60304, 45552, 63860,
                28424, 14216, 6256,  24448, 30100, 11352, 34336, 47528, 8548,
                23412, 59212,
            },
            {
                55828, 46868, 64444, 24412, 32132, 43832, 62332, 26164, 34680,
                60388, 45028, 63492, 48456, 33276, 59056, 6732,  18312, 23128,
                38840, 64656, 18660, 42316, 30604, 55800, 22848, 17548, 61068,
                1816,  40300, 62844, 36068, 55784, 30424, 42348, 24908, 54068,
                39248, 304,   32856, 46728, 27596, 22048, 55404, 44528, 12284,
                53520, 59600, 48800, 13824, 56172, 20304, 45416, 54744, 61152,
                19176, 14768, 63556, 10064, 41880, 49500, 21740, 64128, 10464,
                19200, 48208, 26552, 65392, 32632, 15936, 36892, 49072, 8288,
                54908, 696,   57496, 32832, 39820, 59164, 36588, 55104, 31292,
                1360,  14344, 52528, 24336, 5088,  50856, 45792, 22356, 60520,
                18036, 39964, 5800,  552,   53452, 31876, 38580, 50904, 19460,
                11160, 23364, 63344, 16496, 33648, 27656, 1572,  14676, 6748,
                25808, 10116, 46516, 35792, 12108, 40668, 30132, 16208, 34768,
                48308, 55060, 32336, 64580, 4304,  39536, 57964, 628,   62984,
                17752, 37724,
            },
            {
                2400,  21592, 40116, 11116, 55464, 15492, 2000,  52172, 13116,
                28856, 1000,  16228, 24304, 28360, 37376, 13596, 62208, 31900,
                1708,  11256, 25448, 52596, 13648, 368,   27016, 50608, 44772,
                34448, 15700, 28980, 59632, 22928, 47876, 2832,  60824, 10856,
                18536, 56460, 24076, 41680, 10480, 64288, 36216, 49972, 7936,
                38208, 18308, 9880,  40032, 29064, 636,   35860, 12812, 49364,
                40688, 36344, 23148, 52652, 60244, 13504, 6208,  37476, 24820,
                58732, 51680, 13724, 8744,  53100, 58648, 11620, 63164, 25964,
                20732, 45516, 65056, 29496, 16564, 25804, 12516, 6492,  41484,
                61340, 37896, 44496, 56184, 40384, 16072, 27264, 1328,  54220,
                29596, 49984, 58212, 28120, 41568, 47244, 14536, 65116, 35204,
                56624, 1844,  48472, 41892, 10356, 21772, 64860, 30924, 18956,
                52580, 40980, 5020,  17888, 58292, 2076,  20332, 51312, 11568,
                23028, 3372,  18244, 52152, 46424, 21852, 14516, 28936, 42384,
                25452, 50680,
            },
            {
                14908, 43180, 27724, 3504,  58856, 19276, 37228, 41056, 9620,
                21376, 42068, 51112, 11260, 56816, 2496,  45512, 26300, 41220,
                58052, 50152, 37052, 5484,  62988, 43844, 38388, 10760, 31236,
                54484, 8476,  49736, 5600,  13908, 33244, 8832,  21056, 36436,
                29676, 49512, 4376,  17208, 52776, 30816, 2640,  15676, 32376,
                63776, 27628, 45924, 24128, 61972, 51776, 32224, 5396,  26480,
                57992, 4220,  44660, 140,   33292, 17260, 56208, 31332, 1288,
                16144, 39420, 3328,  29644, 42520, 23836, 46672, 2376,  31004,
                52356, 35468, 12072, 8808,  53996, 48408, 62980, 22708, 51060,
                10128, 26404, 3244,  31512, 8224,  59980, 37456, 13428, 42232,
                8984,  23372, 36120, 19384, 11508, 59640, 2488,  22272, 6384,
                46648, 31764, 59092, 4392,  55336, 50988, 39700, 45400, 56220,
                34084, 63712, 29216, 23204, 54424, 47384, 37500, 61620, 8204,
                40168, 59880, 36112, 42248, 26572, 60624, 53752, 7076,  36204,
                11564, 30452,
            },
            {
                60492, 8736,  53372, 34964, 45484, 29320, 6820,  48616, 59428,
                55312, 31424, 61460, 5788,  40292, 21552, 49508, 53652, 5092,
                15368, 28888, 20092, 56760, 33284, 16152, 60480, 20964, 3532,
                65044, 25844, 19348, 43424, 53544, 38876, 58188, 44628, 65436,
                14484, 42892, 62052, 37348, 58260, 7240,  26136, 60636, 42528,
                22364, 5436,  57816, 16284, 3948,  42872, 21932, 64084, 17644,
                9232,  29116, 55628, 20396, 39020, 26760, 51116, 42312, 46500,
                61428, 33664, 56828, 45380, 17764, 5972,  34272, 60544, 14168,
                39292, 4316,  24016, 42556, 19524, 1792,  43736, 29152, 34724,
                19968, 16852, 58728, 35464, 21000, 47436, 52696, 33660, 62496,
                56780, 15040, 45420, 63612, 7956,  25068, 30304, 43748, 16616,
                39980, 24692, 13804, 28672, 36036, 17600, 3536,  8672,  24040,
                12868, 1384,  44456, 14776, 62716, 32992, 5364,  24748, 56264,
                45016, 27924, 13016, 8788,  2608,  17188, 32556, 48916, 65512,
                45304, 57032,
            },
            {
                33456, 18816, 62576, 13372, 51036, 22452, 65036, 33100, 24828,
                3624,  36360, 18828, 44368, 34168, 65320, 16968, 10236, 35732,
                47620, 61624, 44436, 8592,  23936, 47984, 7292,  51708, 40752,
                12480, 46652, 37112, 556,   63508, 27260, 15980, 50916, 1636,
                34116, 7696,  27224, 12984, 20812, 48380, 40228, 19088, 52124,
                1912,  54564, 34152, 49464, 37588, 11460, 53812, 39120, 47172,
                34512, 50964, 11808, 65208, 47804, 8004,  62512, 11632, 20188,
                7180,  27456, 21948, 63312, 37736, 54152, 21100, 41536, 56092,
                17892, 50532, 61752, 37592, 59768, 33512, 5544,  14688, 54748,
                64928, 45928, 49564, 12760, 63836, 4380,  18644, 25420, 6516,
                38836, 1884,  51536, 33000, 53880, 37716, 56864, 49692, 60436,
                8740,  53244, 20164, 44592, 62096, 25904, 60136, 47888, 37132,
                59296, 21088, 50300, 38524, 9436,  27108, 42948, 17628, 31128,
                1892,  53672, 20992, 63388, 51096, 37968, 10272, 22900, 1900,
                19948, 5860,
            },
            {
                38144, 47744, 25144, 4500,  38696, 10268, 54268, 17404, 13576,
                46416, 8648,  26444, 14308, 54764, 30060, 7224,  60372, 31576,
                22432, 1032,  12444, 40092, 29972, 55000, 34940, 27648, 57692,
                17236, 33344, 56736, 21836, 11196, 31820, 6224,  25492, 54888,
                22904, 59920, 51736, 45800, 1068,  34924, 56956, 11080, 46368,
                36764, 14000, 29672, 8728,  19100, 60632, 28404, 14728, 6472,
                59948, 41340, 25248, 15484, 35448, 2672,  23196, 29364, 53572,
                36872, 49428, 9504,  12788, 672,   50260, 28348, 10372, 7004,
                27036, 47272, 30576, 7520,  52704, 27208, 47492, 58244, 37400,
                28,    6808,  28092, 40704, 23812, 30588, 43600, 11328, 48656,
                31360, 21724, 27344, 5104,  17324, 42168, 4000,  12700, 26848,
                33328, 64360, 37928, 11396, 6876,  32968, 14812, 53232, 28800,
                42868, 6424,  31564, 56936, 3612,  52692, 13552, 49524, 65108,
                38592, 16108, 33864, 45980, 29536, 58280, 43140, 55240, 27560,
                40652, 52324,
            },
            {
                15624, 492,   31348, 42472, 59796, 27256, 5280,  43276, 39456,
                52480, 63868, 58104, 48696, 1580,  24896, 38880, 19204, 42448,
                55632, 26472, 52196, 64824, 18392, 2484,  14036, 43284, 23016,
                6196,  61576, 28828, 51172, 41768, 48768, 18760, 40476, 47064,
                11884, 18340, 39868, 31136, 24676, 64800, 14872, 28924, 7848,
                24380, 59500, 44080, 65072, 25372, 45704, 1656,  57552, 23612,
                31628, 19492, 5640,  43712, 30804, 59004, 40824, 56976, 14572,
                4068,  43088, 55480, 25524, 59436, 15604, 32268, 64744, 45052,
                58600, 956,   15376, 22280, 12652, 40176, 18076, 9204,  24048,
                42744, 32668, 54052, 9584,  56996, 1564,  61076, 54108, 16828,
                65460, 44056, 59504, 13792, 47708, 21200, 62404, 35868, 19068,
                24,    48784, 4904,  57340, 46820, 41152, 20632, 2268,  10160,
                17956, 62156, 25412, 46484, 19824, 36576, 59680, 22368, 10392,
                6840,  58628, 49928, 24032, 5392,  19496, 14756, 34756, 61716,
                12168, 23804,
            },
            {
                56556, 64240, 49732, 20728, 16524, 46700, 30608, 61524, 21040,
                68,    29084, 38188, 22284, 11660, 51924, 45024, 58192, 13364,
                4228,  37660, 33076, 46032, 5720,  58764, 37412, 63700, 49132,
                1788,  39552, 9560,  15088, 3036,  61936, 57808, 36952, 3476,
                32312, 56176, 5608,  9996,  53896, 43336, 4548,  33408, 62152,
                50892, 20280, 39528, 6044,  32308, 51136, 35424, 10204, 44428,
                52236, 1024,  61112, 54904, 49904, 10448, 18168, 45748, 33204,
                64840, 17244, 31488, 40328, 35532, 48036, 3172,  24092, 38040,
                20392, 33956, 63600, 44200, 55592, 2396,  62676, 30772, 51664,
                13180, 20420, 62440, 15744, 36032, 50476, 39256, 26732, 35064,
                3116,  10528, 39892, 55532, 34496, 28776, 9900,  51220, 55808,
                40584, 29176, 23596, 17120, 51800, 30068, 63264, 55764, 34720,
                51040, 39792, 15628, 11176, 63280, 29252, 1148,  33220, 47732,
                26988, 42464, 11716, 204,   40540, 62680, 7764,  47940, 3588,
                45708, 29764,
            },
            {
                43700, 10808, 34600, 7924,  55824, 2240,  36012, 56952, 11100,
                34364, 17860, 6868,  42192, 32152, 63416, 3308,  27844, 49392,
                62284, 16844, 9860,  21336, 50320, 25848, 10944, 19632, 30528,
                53360, 25572, 45544, 54792, 35260, 23720, 29604, 8492,  64092,
                44196, 27808, 62312, 36644, 17100, 49380, 21332, 55316, 41636,
                512,   11580, 56824, 17316, 3076,  55484, 20400, 40368, 16196,
                64356, 36908, 13580, 27768, 20988, 37676, 6776,  25860, 1468,
                50896, 23088, 46896, 4540,  19880, 61484, 43472, 51960, 13168,
                54692, 8884,  37064, 25928, 32284, 49492, 21304, 45448, 60732,
                39348, 26312, 44344, 5448,  46968, 21916, 13192, 7212,  20328,
                56516, 50236, 24408, 7656,  64012, 2424,  23856, 45188, 15668,
                7756,  43444, 61696, 35356, 3268,  25012, 12368, 43944, 22500,
                58756, 316,   33520, 54604, 41172, 6132,  44044, 55968, 37384,
                18832, 62916, 30700, 57284, 36504, 26800, 51676, 31712, 18216,
                59240, 6248,
            },
            {
                21900, 27916, 60984, 40316, 52200, 25776, 14892, 48084, 24044,
                50900, 44888, 55432, 60740, 16324, 36108, 9544,  23176, 34340,
                7548,  53904, 42780, 60620, 30868, 40908, 54548, 46428, 8396,
                34280, 16924, 59068, 20344, 6644,  43364, 13096, 16984, 50732,
                21084, 14392, 48272, 23620, 3168,  61368, 38708, 26004, 16560,
                47488, 35780, 27432, 49284, 42020, 13396, 60332, 30084, 26016,
                48840, 8976,  32852, 46860, 4020,  63252, 52488, 60048, 39140,
                11540, 58240, 8060,  29660, 55996, 11268, 6584,  18120, 30336,
                41288, 2944,  47684, 57980, 10904, 4744,  36296, 15908, 7384,
                3040,  57788, 52708, 18480, 29188, 33096, 59264, 41840, 46180,
                29752, 37156, 18616, 31916, 42936, 53424, 37304, 60928, 31352,
                58688, 21388, 13884, 54824, 39864, 8488,  49384, 37656, 4456,
                26504, 45228, 8172,  49108, 21208, 16540, 51604, 24372, 14596,
                2760,  52744, 22100, 15540, 44944, 55008, 12856, 24124, 39124,
                53620, 36616,
            },
            {
                51008, 2636,  18344, 12344, 32104, 64040, 7232,  40808, 3712,
                63004, 13924, 27176, 4608,  20580, 45916, 57048, 40780, 18520,
                47308, 29220, 24396, 13628, 196,   35076, 15772, 3628,  61300,
                41656, 12664, 37932, 63248, 32384, 47796, 53656, 26184, 58976,
                33780, 80,    42124, 58096, 29384, 12764, 9088,  32168, 5432,
                63804, 30560, 21636, 7304,  62784, 23972, 46832, 3608,  6964,
                54596, 22444, 42864, 57264, 24756, 15784, 30536, 42648, 20712,
                28212, 36496, 62832, 15120, 41780, 22824, 34472, 57536, 62788,
                24720, 50860, 16256, 19292, 41984, 65308, 24616, 56324, 28668,
                34160, 11832, 37324, 944,   64772, 10016, 54968, 3692,  62864,
                14696, 60008, 524,   12920, 49088, 19920, 5384,  11584, 26264,
                2156,  49860, 10396, 32124, 64176, 19580, 57676, 16028, 31996,
                65316, 13508, 29928, 61084, 36020, 27804, 64440, 9180,  59272,
                40064, 46384, 6508,  33184, 9332,  20736, 42496, 1448,  63160,
                9508,  14948,
            },
            {
                41560, 54920, 46064, 37344, 22636, 44212, 19232, 57948, 29376,
                32924, 9292,  37224, 48424, 53392, 29744, 14064, 51432, 1200,
                63948, 38980, 6428,  57588, 44972, 63016, 20868, 28584, 56204,
                22696, 51456, 720,   27684, 11004, 57404, 2040,  41100, 6244,
                37908, 11276, 52992, 19532, 34624, 46004, 52096, 60204, 43816,
                13624, 52896, 58496, 37776, 33176, 9472,  53296, 38600, 34596,
                58560, 17484, 2440,  40152, 11724, 35608, 48060, 8304,  14172,
                53748, 44356, 2048,  48496, 52800, 27588, 38832, 47204, 5352,
                10740, 28516, 61272, 6708,  30116, 52940, 13088, 40020, 50848,
                47340, 23052, 61244, 41352, 24924, 51344, 16604, 27828, 22980,
                9064,  52804, 44848, 26984, 61596, 16084, 41200, 47872, 34092,
                54084, 37552, 46036, 26672, 868,   44844, 28224, 6960,  53540,
                19988, 39288, 56644, 23964, 2660,  12080, 46096, 31364, 4384,
                35088, 28184, 60832, 50348, 64760, 5116,  57016, 28864, 48688,
                33400, 26088,
            },
            {
                65052, 30668, 6440,  59140, 764,   50232, 10300, 36520, 53152,
                21528, 60064, 41360, 24972, 2388,  8052,  61032, 32300, 25896,
                11296, 55728, 36328, 18768, 51320, 9492,  39128, 47748, 7032,
                31476, 44208, 5652,  49320, 23108, 36744, 18560, 30692, 61196,
                46948, 24764, 64728, 7984,  40612, 56964, 1296,  20184, 24024,
                39484, 9940,  2356,  45808, 15080, 28564, 18824, 63620, 12864,
                44900, 28240, 62712, 50736, 19908, 58796, 396,   63964, 34028,
                5220,  18588, 25032, 32776, 8624,  64564, 60,    16476, 32100,
                45308, 55052, 38904, 22544, 34380, 44872, 740,   18572, 59852,
                8772,  16836, 30480, 48908, 6700,  34420, 43972, 38048, 48364,
                32636, 40644, 4788,  35600, 54884, 29420, 58020, 22548, 65076,
                18604, 4624,  60572, 16296, 51964, 34588, 59864, 41616, 48792,
                10176, 46596, 5504,  18024, 50536, 38072, 57248, 42136, 19648,
                54700, 13204, 25104, 17980, 38348, 47556, 35028, 16736, 59724,
                20172, 4080,
            },
            {
                57308, 23472, 48416, 14744, 28868, 62168, 26456, 16024, 46412,
                1232,  18128, 56108, 12452, 64976, 38712, 21880, 43416, 59104,
                16044, 46136, 22504, 3888,  27012, 32008, 58340, 24684, 14448,
                65500, 18208, 59412, 40000, 15064, 64316, 45176, 9208,  21628,
                55572, 16552, 31264, 4400,  26280, 17376, 37444, 28452, 48888,
                56508, 17944, 26132, 50740, 60528, 43400, 64,    50180, 21156,
                31588, 10456, 37020, 6200,  54352, 28680, 44936, 22492, 55924,
                49608, 61956, 40012, 57196, 21268, 13452, 49772, 60020, 20636,
                35772, 13260, 1772,  58712, 48284, 9408,  62160, 27368, 32964,
                4036,  44700, 55960, 13936, 20364, 58888, 12008, 1356,  19320,
                58200, 25384, 64700, 21444, 9832,  2532,  38740, 7604,  14304,
                42116, 30320, 23944, 40788, 9424,  22212, 13300, 2816,  25172,
                36156, 60420, 32824, 42876, 54028, 7768,  15412, 22632, 62388,
                49192, 10012, 43468, 1144,  30812, 23532, 11964, 40272, 6780,
                44508, 12800,
            },
            {
                38768, 17388, 8708,  34228, 40048, 55356, 5328,  43132, 64412,
                8176,  50456, 30660, 44476, 33816, 17212, 49920, 5756,  35328,
                3064,  30324, 53140, 64724, 43184, 12872, 49840, 2256,  37424,
                53580, 33868, 8732,  29272, 52520, 4552,  33544, 51592, 28020,
                2468,  43768, 50684, 59416, 47832, 13808, 63124, 6380,  33152,
                3820,  65312, 35232, 20904, 6168,  40276, 57256, 25824, 5064,
                47604, 60752, 15476, 25212, 32612, 9284,  17336, 41136, 26392,
                31076, 9972,  16040, 2928,  44068, 36388, 25716, 42436, 53588,
                4572,  64020, 43008, 26500, 15112, 20696, 36928, 54648, 41624,
                21556, 64196, 35380, 2220,  39516, 54036, 29944, 63760, 50440,
                6268,  16856, 12496, 46704, 50984, 33060, 24948, 45580, 52028,
                11008, 57768, 48252, 6352,  62816, 38172, 56052, 29780, 63796,
                16316, 22796, 1700,  28848, 63588, 25732, 34368, 444,   30092,
                5888,  36596, 57192, 60084, 51804, 3468,  62292, 26616, 53552,
                50292, 35460,
            },
            {
                27732, 61520, 43792, 52180, 20256, 12128, 33168, 23244, 39204,
                27388, 35640, 3896,  22684, 58344, 10520, 27744, 55016, 20028,
                40964, 48852, 8800,  33536, 17460, 5924,  61348, 19736, 42488,
                11356, 25760, 47036, 19980, 55752, 24284, 11696, 62120, 14856,
                39828, 36308, 12332, 33920, 22472, 42960, 51532, 55068, 11824,
                45208, 41356, 29236, 54016, 11244, 32136, 16212, 36428, 54160,
                39744, 1856,  42708, 48976, 65352, 38764, 53104, 4620,  12472,
                59584, 37644, 46348, 29632, 60948, 55568, 10360, 7144,  30180,
                23884, 18232, 51056, 32320, 55812, 3416,  50276, 12400, 6812,
                25700, 52328, 10416, 27640, 62084, 24176, 46380, 8768,  33904,
                56240, 38420, 30556, 42516, 59580, 15424, 62952, 55772, 452,
                27436, 35912, 20032, 53512, 31676, 46300, 18060, 50000, 5872,
                43504, 55132, 14040, 47660, 11032, 40148, 58984, 52336, 47192,
                26548, 17020, 40736, 21160, 14476, 45796, 55684, 8836,  31148,
                22144, 976,
            },
            {
                11028, 54464, 3320,  24524, 60368, 49024, 2044,  57044, 13692,
                59612, 46984, 15456, 52760, 39692, 416,   63320, 45392, 12944,
                61792, 23644, 14480, 59444, 40444, 54844, 34992, 46280, 29932,
                57320, 1260,  62976, 43448, 3084,  37252, 41540, 48144, 6804,
                58528, 20324, 8504,  62192, 1992,  30172, 8964,  36232, 16612,
                23004, 58816, 14212, 1820,  61616, 46320, 22308, 64540, 8880,
                27556, 18380, 56712, 21748, 3640,  13816, 61096, 35280, 47680,
                20500, 6956,  23692, 51196, 4792,  33416, 17252, 39648, 58520,
                47468, 37892, 11236, 7776,  40216, 64644, 29704, 43720, 59316,
                15724, 38148, 47688, 18156, 42900, 4312,  15464, 22004, 41576,
                26676, 2340,  54592, 7496,  19568, 4352,  36828, 30928, 17544,
                44404, 63956, 3444,  15032, 26836, 1476,  11144, 34100, 39540,
                51832, 8268,  58516, 35564, 20832, 3836,  18252, 44236, 13944,
                65260, 54328, 32400, 7440,  28116, 35840, 19260, 43000, 15684,
                63696, 46376,
            },
            {
                19664, 41324, 30288, 6912,  37428, 45788, 18056, 30032, 52040,
                10184, 20284, 62356, 5368,  29564, 48076, 24056, 36700, 7220,
                28716, 56984, 37380, 712,   20840, 28324, 10168, 23760, 4740,
                51072, 16908, 35080, 13404, 59812, 30988, 17408, 56596, 25324,
                29800, 46532, 52516, 27272, 55616, 19144, 40380, 25092, 60876,
                31632, 7960,  48244, 36868, 24804, 52144, 6884,  13084, 33548,
                59644, 51084, 11776, 34816, 46048, 29448, 24420, 50944, 1920,
                42400, 54732, 64936, 12980, 19356, 27140, 52732, 63540, 14048,
                3520,  28252, 61324, 21672, 46408, 17312, 23960, 34208, 49260,
                2968,  30908, 58484, 7480,  33104, 53128, 37332, 60668, 12852,
                45292, 63156, 22944, 49060, 28208, 40892, 11340, 49520, 21712,
                39704, 9800,  33124, 60484, 43128, 58088, 21904, 62364, 27520,
                19724, 31888, 25348, 41364, 50172, 61892, 31664, 9120,  38792,
                3008,  23480, 11280, 49900, 63392, 39232, 4284,  60416, 33860,
                6160,  56904,
            },
            {
                13864, 32936, 63000, 16576, 10772, 27204, 63572, 41856, 6300,
                33508, 38300, 25988, 56488, 42588, 18656, 14792, 52512, 33068,
                2732,  51024, 44288, 25968, 47716, 52224, 63820, 15280, 41260,
                61816, 21468, 39424, 27212, 7864,  50656, 22120, 1764,  35228,
                65160, 4864,  17016, 38204, 44532, 5620,  64052, 46224, 572,
                50016, 20240, 55836, 42460, 17776, 28528, 38680, 49448, 44140,
                23468, 30944, 5792,  41440, 58868, 18752, 8684,  56844, 32028,
                17000, 27984, 34776, 40172, 48184, 43408, 948,   21956, 44744,
                34220, 56268, 52232, 440,   35604, 59856, 4708,  10092, 19900,
                63244, 44924, 22536, 55384, 11628, 48612, 28536, 1512,  51368,
                18408, 35812, 10232, 33348, 60908, 54008, 24316, 59128, 7068,
                56388, 51492, 23520, 48864, 7524,  40512, 52720, 4132,  13164,
                45836, 916,   64972, 16464, 6120,  27696, 55424, 21704, 34120,
                58064, 46072, 41964, 2056,  53768, 24144, 12928, 48524, 25236,
                51584, 37840,
            },
            {
                22784, 49604, 45220, 53064, 58512, 34900, 4600,  49884, 21868,
                55088, 44904, 7588,  12628, 32200, 60344, 9060,  56088, 41728,
                18988, 11616, 62704, 8148,  31520, 4144,  37884, 56196, 8388,
                31056, 48352, 5980,  54168, 45848, 63904, 10228, 44472, 53928,
                13064, 42188, 23188, 59708, 11252, 33608, 15060, 52644, 28332,
                38968, 5024,  33916, 10256, 62624, 3180,  58068, 19624, 1400,
                62932, 16308, 55216, 25916, 680,   63552, 39472, 14740, 44756,
                62668, 10600, 3824,  58012, 8612,  62200, 37216, 30620, 49708,
                8868,  16656, 25672, 43436, 28904, 14128, 53148, 57084, 37292,
                28292, 13500, 540,   40536, 25432, 65152, 19928, 58624, 31540,
                5152,  57368, 15820, 43744, 3596,  14564, 46476, 34460, 1704,
                15996, 29056, 37328, 12504, 18512, 35480, 24744, 47944, 60164,
                37988, 55992, 10724, 52604, 44664, 37404, 12300, 49336, 6892,
                28984, 19392, 61292, 34524, 16668, 30884, 58596, 9732,  41096,
                28756, 2508,
            },
            {
                58024, 8440,  25640, 592,   21216, 39756, 15068, 24912, 60600,
                892,   16500, 65376, 35940, 50868, 1908,  38136, 25484, 64284,
                46840, 22756, 35244, 16916, 58804, 12784, 19380, 44760, 25856,
                36264, 12256, 58060, 23900, 32604, 14896, 28840, 38616, 18948,
                32916, 50228, 864,   31028, 48824, 57968, 21568, 9412,  56832,
                13244, 64624, 26536, 44876, 15148, 31168, 54708, 41140, 6500,
                37624, 46764, 9736,  35916, 48492, 31584, 22376, 5252,  36856,
                25376, 52524, 21220, 30952, 16136, 24112, 12060, 5552,  57464,
                20260, 41608, 12460, 62736, 7036,  39168, 46956, 25332, 42156,
                8244,  51340, 61480, 36160, 16068, 5884,  43624, 9592,  40220,
                47132, 25400, 53220, 39028, 21172, 8464,  65480, 27624, 42172,
                62308, 45936, 3764,  54996, 64868, 30844, 15516, 9228,  28876,
                17864, 34252, 23208, 30428, 3296,  60584, 24988, 42752, 63812,
                53020, 14320, 25976, 8652,  47208, 42776, 216,   20296, 55416,
                17004, 65256,
            },
            {
                35008, 39056, 12632, 43124, 29616, 55500, 9380,  46028, 31800,
                40680, 28420, 48396, 23436, 20084, 45324, 29016, 4512,  15372,
                30904, 5680,  54880, 43040, 39636, 50244, 33340, 1060,  60364,
                51444, 4084,  43172, 17716, 144,   40920, 57492, 3860,  61184,
                24940, 9688,  63052, 16164, 26292, 2972,  37620, 42968, 32960,
                18432, 51236, 22584, 59772, 7636,  47788, 34872, 24648, 14120,
                27964, 58364, 20680, 52856, 13052, 42576, 54832, 50008, 60100,
                1056,  46528, 41860, 36132, 50632, 54304, 60076, 26356, 38916,
                65484, 2372,  32052, 50212, 57936, 20896, 31704, 1608,  16372,
                54612, 32948, 19592, 47516, 30120, 56672, 34176, 23536, 13608,
                63996, 28804, 200,   61824, 30760, 51788, 37652, 19104, 12152,
                32380, 58228, 20428, 27028, 44672, 152,   57664, 53848, 43912,
                6580,  62604, 40072, 48552, 20264, 15396, 56708, 732,   17596,
                40248, 4712,  38020, 56160, 21908, 64404, 50712, 37048, 31904,
                44544, 5156,
            },
            {
                27392, 54180, 18732, 61872, 48608, 2984,  64456, 51520, 12896,
                19304, 58692, 9864,  5188,  54588, 61772, 11388, 58036, 49560,
                39236, 60920, 27700, 2560,  24640, 65212, 22068, 54064, 16364,
                28824, 20568, 63520, 34708, 60644, 48924, 20968, 51984, 7632,
                47000, 56200, 40696, 35472, 54112, 45432, 63516, 24356, 3992,
                47104, 40236, 1236,  37152, 53092, 21308, 10628, 65128, 45132,
                51424, 32796, 3096,  62252, 17116, 7324,  26900, 11348, 18364,
                29880, 13620, 61964, 6488,  2160,  45296, 33268, 18200, 47704,
                52956, 35420, 23228, 45372, 17440, 10744, 60512, 49852, 64836,
                23464, 5244,  43988, 9960,  60376, 3484,  50184, 54176, 37184,
                6448,  19052, 48636, 11576, 24560, 44608, 2580,  54496, 48236,
                22912, 7360,  10784, 41640, 50840, 6364,  39032, 32580, 21232,
                50012, 2180,  13876, 54632, 7968,  33048, 47292, 35888, 10048,
                31796, 48048, 59172, 29816, 3616,  13444, 27200, 6932,  62016,
                11000, 47820,
            },
            {
                14828, 59064, 3944,  32208, 36124, 16812, 27332, 37516, 7168,
                34432, 52592, 43292, 30948, 39012, 16336, 34312, 42432, 19452,
                8936,  51972, 14056, 47856, 10608, 29984, 7216,  41064, 9772,
                47128, 38456, 53084, 8584,  25656, 11820, 35836, 27708, 14244,
                37368, 29424, 21188, 6332,  12964, 19212, 8048,  30568, 60696,
                11536, 58424, 28084, 17192, 42284, 4156,  29912, 59388, 18260,
                8000,  42336, 24188, 38752, 29372, 45172, 65016, 34988, 40208,
                58508, 23860, 55248, 19500, 27776, 40024, 7980,  14204, 3900,
                29028, 10212, 55620, 5820,  37744, 26976, 4308,  34688, 12216,
                39836, 58444, 26728, 14800, 38428, 21352, 27536, 17080, 44740,
                55696, 32704, 40784, 59080, 35936, 17264, 57580, 6144,  26528,
                40004, 53172, 33960, 59472, 17372, 23324, 62960, 11976, 26152,
                41348, 58556, 28096, 36808, 64408, 41812, 26876, 22188, 62448,
                51412, 20228, 11856, 44712, 35036, 54804, 40112, 15808, 24084,
                52216, 21400,
            },
            {
                30276, 41520, 50700, 23376, 11440, 57344, 20564, 42108, 61628,
                1768,  25512, 14700, 63848, 2592,  26824, 53024, 24088, 104,
                32700, 21728, 37264, 56924, 18632, 45504, 58236, 35712, 61448,
                24724, 13112, 2620,  30156, 46452, 55740, 4612,  64328, 45092,
                17576, 2848,  60820, 52060, 41804, 27980, 55552, 49516, 16304,
                35560, 6004,  55020, 31940, 63276, 13324, 49932, 36212, 264,
                55504, 11996, 60440, 49548, 4852,  19792, 56600, 3004,  48068,
                7116,  32876, 9992,  49216, 64632, 22668, 56416, 42852, 61780,
                21856, 15528, 41032, 63872, 30416, 54432, 41972, 46012, 18376,
                29444, 48632, 2252,  63628, 52796, 42620, 11588, 1156,  60132,
                9084,  22736, 14840, 5256,  50308, 9856,  33632, 63460, 14996,
                60256, 1404,  37704, 13592, 29868, 47284, 35724, 2772,  60924,
                16716, 10028, 46112, 23640, 4040,  11700, 58996, 5348,  43908,
                28104, 1888,  61104, 24900, 6356,  18808, 46780, 59292, 33484,
                42584, 1080,
            },
            {
                36456, 6328,  63984, 8340,  47396, 44204, 5172,  54428, 23144,
                48868, 56212, 36464, 47124, 21092, 59844, 7516,  45668, 64664,
                55640, 43808, 4324,  62504, 34648, 1760,  14832, 49328, 5372,
                31884, 42820, 57296, 15660, 39608, 18844, 32360, 41564, 22880,
                58368, 49748, 11304, 33224, 46572, 62300, 224,   39436, 21476,
                44232, 25024, 49188, 8944,  45716, 23380, 57080, 26668, 40068,
                22184, 31408, 15636, 35324, 53856, 10152, 31276, 21776, 15284,
                52816, 44340, 38392, 16452, 35144, 11148, 30128, 51672, 37040,
                45984, 57696, 25352, 1696,  51388, 14636, 21744, 62560, 7836,
                36904, 55692, 22292, 35100, 6664,  31928, 48052, 63260, 29760,
                35592, 51580, 43200, 64348, 28176, 20980, 42276, 30392, 45664,
                18544, 49004, 24532, 64320, 8424,  56520, 20008, 44864, 51460,
                33724, 55912, 19528, 31852, 52848, 49404, 18508, 53876, 14160,
                37484, 55676, 16724, 41480, 52388, 63140, 30632, 3052,  8916,
                56808, 61464,
            },
            {
                46512, 25680, 16184, 39588, 29012, 60468, 32760, 14884, 10516,
                30200, 18048, 7868,  12000, 39944, 50076, 13832, 35832, 28320,
                11068, 16440, 25912, 30756, 53352, 40056, 27372, 23096, 56036,
                18212, 64784, 50132, 22140, 59492, 7092,  52716, 10444, 1180,
                30664, 39268, 25696, 5120,  23428, 14608, 31840, 9932,  52564,
                64336, 13740, 19184, 34732, 2380,  38892, 16216, 5880,  47912,
                44096, 64116, 2320,  26408, 46540, 39432, 59976, 43084, 25472,
                62408, 28592, 1284,  58268, 46328, 4528,  60264, 18872, 384,
                7536,  32236, 48968, 19332, 9484,  33604, 59044, 26028, 228,
                51644, 15912, 10512, 45908, 18576, 57564, 24688, 39668, 20140,
                4388,  58176, 25852, 2196,  46584, 53756, 13156, 4240,  36704,
                9524,  28976, 43532, 3564,  50304, 40576, 4808,  14652, 29392,
                7432,  38756, 536,   62108, 15652, 39308, 29432, 34452, 8160,
                22724, 48980, 33008, 9756,  36684, 21812, 12124, 49292, 38628,
                18428, 12668,
            },
            {
                20824, 55668, 33516, 52944, 18868, 156,   25148, 38596, 45964,
                62584, 41208, 58428, 27620, 32780, 4276,  57024, 17832, 48796,
                41068, 58880, 50192, 7860,  12500, 20588, 63880, 44240, 11204,
                580,   34288, 26788, 3956,  36416, 44020, 24596, 62804, 48568,
                55252, 15716, 65432, 35848, 58040, 18264, 37492, 59540, 26476,
                2820,  40816, 56500, 61600, 27508, 53576, 59748, 9636,  33752,
                19396, 52352, 8412,  58900, 22628, 13548, 520,   49868, 36548,
                12204, 5624,  21024, 53516, 25932, 41596, 13812, 34080, 26768,
                63316, 12524, 38640, 60472, 44732, 3684,  48248, 11868, 56872,
                43452, 32480, 61028, 27756, 41020, 13672, 2624,  53532, 15504,
                46172, 12076, 31948, 18696, 38240, 61796, 23624, 56480, 51164,
                62144, 21292, 54980, 16468, 32464, 27132, 59884, 54344, 24152,
                65404, 48988, 42984, 26240, 9416,  45384, 2060,  63780, 57440,
                40372, 3752,  65272, 46140, 800,   27856, 43552, 54508, 24568,
                28724, 51336,
            },
            {
                44112, 10280, 4796,  58836, 13188, 49244, 65172, 52552, 3164,
                34300, 5636,  22288, 51304, 62912, 43352, 23488, 30860, 2392,
                6192,  38100, 23216, 46164, 60180, 3828,  51612, 29068, 37960,
                53680, 41280, 8660,  47592, 12156, 60972, 28448, 13932, 34564,
                20080, 8300,  42268, 53608, 3464,  50208, 43492, 7316,  48008,
                33832, 30072, 4480,  43300, 11140, 20724, 30548, 51120, 62368,
                12728, 28596, 36920, 41992, 17556, 63792, 34016, 7800,  18708,
                55588, 41248, 63876, 32140, 8904,  50828, 23084, 47352, 55092,
                43340, 17036, 53332, 27668, 23180, 40932, 36276, 30688, 20288,
                39004, 24272, 3048,  53964, 65288, 8508,  49796, 36916, 27000,
                62184, 40264, 7208,  56856, 34740, 8928,  1124,  40188, 26260,
                6632,  33880, 58584, 11756, 38584, 22132, 9660,  42696, 37088,
                12028, 21600, 5844,  56900, 36056, 60588, 20040, 25008, 44420,
                11336, 30168, 25924, 14412, 56224, 61724, 17456, 5168,  35192,
                64904, 2140,
            },
            {
                30432, 62648, 37888, 27076, 43048, 35108, 7544,  20844, 28380,
                15596, 54724, 44396, 676,   19324, 10640, 37568, 60312, 52116,
                20504, 63064, 14920, 33136, 42552, 36092, 16744, 6984,  48516,
                19608, 14276, 62628, 30284, 51868, 17564, 3288,  38424, 44820,
                5616,  27748, 47460, 21796, 10440, 29456, 56684, 20096, 12840,
                60568, 22852, 17052, 50380, 36356, 46404, 1028,  41444, 24212,
                3304,  57904, 48816, 5164,  54668, 30096, 51464, 27172, 60940,
                46080, 30352, 14468, 37508, 18140, 2464,  62068, 38864, 10096,
                21104, 2956,  35716, 6036,  65124, 15172, 53872, 5356,  63332,
                14060, 50168, 6408,  37340, 30296, 22816, 59196, 32448, 9816,
                55124, 22608, 48940, 13668, 52128, 16696, 30992, 48256, 17972,
                13716, 41396, 412,   51992, 46816, 63636, 17936, 1904,  47496,
                32524, 17232, 51852, 28872, 13044, 48224, 4948,  32252, 15924,
                52616, 59088, 18488, 50004, 31828, 8560,  39660, 59560, 14808,
                7944,  40732,
            },
            {
                17532, 23568, 48112, 2572,  21840, 57236, 40320, 11828, 47760,
                59116, 25020, 36164, 14116, 29684, 54060, 7840,  46228, 26620,
                34880, 9944,  56276, 748,   24880, 57240, 10264, 61660, 32392,
                59156, 25044, 55148, 20916, 33624, 40632, 56256, 22268, 58644,
                51416, 60808, 32120, 13776, 62528, 40592, 24564, 1732,  36612,
                44604, 52948, 7708,  65252, 25576, 14108, 60776, 18256, 37952,
                45472, 15752, 32176, 10584, 24788, 44656, 3720,  39180, 9984,
                23784, 1976,  49136, 59816, 44268, 56948, 28544, 5916,  31404,
                48468, 59744, 29768, 11480, 50336, 18652, 57840, 28328, 46184,
                9644,  60300, 19376, 47864, 16076, 45004, 5376,  19048, 43556,
                352,   29360, 64804, 42592, 24920, 60960, 44156, 58968, 35744,
                65176, 45320, 30232, 25224, 6172,  35428, 52828, 27848, 61648,
                55156, 3312,  59764, 33644, 23024, 40904, 55352, 50704, 37956,
                7100,  21624, 42008, 4680,  37524, 23636, 45460, 51968, 32540,
                47600, 57924,
            },
            {
                35800, 52164, 15124, 61412, 9616,  31188, 16788, 61760, 42688,
                31648, 7356,  64312, 48680, 57484, 40804, 16780, 65232, 13412,
                43928, 53612, 28632, 19188, 49472, 30344, 40664, 22616, 46468,
                5232,  37100, 1516,  45036, 6060,  9752,  47356, 31336, 432,
                11684, 17636, 38960, 2228,  34480, 46284, 15876, 64076, 54704,
                27376, 10292, 39412, 31492, 5380,  54076, 33228, 8040,  55860,
                26820, 64924, 20384, 40492, 57164, 12972, 21212, 58576, 15832,
                52264, 35300, 7104,  22240, 11944, 34396, 52680, 15680, 64224,
                24624, 55964, 39876, 45900, 32900, 8580,  25264, 1112,  41524,
                34304, 55180, 26308, 42088, 11104, 34444, 52272, 61392, 48400,
                16556, 36180, 5604,  20308, 3000,  28392, 11544, 5848,  22200,
                52736, 9936,  20196, 61268, 14944, 43228, 31432, 8072,  13952,
                40636, 25308, 44324, 7732,  63576, 17728, 10284, 27920, 136,
                62824, 47316, 34056, 60648, 53832, 12844, 19556, 3400,  26724,
                21236, 11956,
            },
            {
                54940, 6600,  32472, 44720, 55540, 50972, 24540, 1604,  53732,
                18672, 10296, 38896, 3700,  21360, 34184, 25076, 1368,  31716,
                4248,  39808, 47136, 5732,  63764, 13468, 52632, 2092,  27156,
                12580, 43020, 16444, 58436, 64304, 26644, 14672, 62224, 36976,
                25520, 43460, 56484, 52996, 26832, 8376,  51004, 31260, 5032,
                14804, 48832, 19852, 58848, 42824, 22124, 47836, 11468, 50464,
                5500,  34548, 52556, 880,   62036, 35856, 47332, 31688, 42572,
                65384, 27780, 55344, 40076, 26236, 4448,  20124, 41372, 36168,
                8208,  13980, 19436, 2128,  62072, 43036, 37564, 52400, 21396,
                15416, 31508, 2036,  57480, 62340, 28344, 2936,  25244, 39000,
                11864, 58396, 51560, 33820, 47220, 39096, 53972, 32544, 2656,
                26784, 56720, 37712, 48128, 3336,  59432, 21456, 56804, 36460,
                20024, 10492, 49744, 38056, 1680,  46784, 58640, 35312, 23648,
                56336, 13488, 9648,  27424, 1380,  64420, 29916, 56732, 42036,
                63240, 616,
            },
            {
                45860, 28476, 41156, 19252, 4684,  27356, 38452, 5836,  35652,
                23400, 51384, 45688, 26932, 60836, 6320,  47648, 51144, 56908,
                22992, 16100, 60360, 37032, 8628,  44464, 17572, 34592, 65496,
                54460, 50744, 28368, 35528, 22748, 52860, 42112, 19560, 49828,
                7964,  64708, 22996, 5984,  19316, 58324, 38216, 21604, 42256,
                61512, 34788, 55264, 1324,  15992, 37056, 63112, 28484, 40976,
                23008, 14164, 44108, 29624, 7444,  16980, 54100, 6368,  2860,
                19884, 10672, 45592, 16292, 63512, 50392, 46204, 59052, 1100,
                44192, 49848, 27068, 53776, 22752, 59280, 12228, 48280, 64484,
                6108,  39568, 49720, 22412, 7740,  36776, 14568, 55904, 21648,
                31300, 41828, 26348, 10136, 63352, 18896, 57116, 15460, 49764,
                40452, 7472,  16868, 33360, 11124, 50064, 26000, 824,   46520,
                64568, 29908, 15556, 54336, 21280, 31320, 14376, 43248, 19044,
                39408, 31248, 45252, 17772, 41048, 48304, 35672, 10308, 50052,
                38068, 24728,
            },
            {
                9460,  14196, 64844, 36348, 11708, 48760, 63580, 45180, 56420,
                13360, 62268, 32980, 15236, 53016, 11168, 42660, 18356, 35928,
                62372, 10748, 25984, 33024, 55260, 21960, 59752, 38812, 6524,
                19220, 40252, 7720,  11404, 48644, 3188,  30004, 5196,  55232,
                34772, 15472, 30888, 48704, 44536, 61248, 12012, 356,   47228,
                24280, 6888,  29304, 45684, 26060, 4180,  57372, 19296, 1664,
                61332, 58472, 9532,  38532, 50332, 27680, 23660, 60516, 38352,
                49304, 30208, 61132, 260,   32020, 8676,  28168, 12236, 23460,
                33264, 58312, 5180,  36836, 15892, 30916, 3620,  18032, 29404,
                58248, 45044, 12616, 17724, 52920, 47152, 41308, 64272, 8372,
                1740,  54492, 59940, 14692, 6832,  44880, 23848, 36340, 62536,
                43828, 29380, 64064, 23116, 55320, 38472, 16220, 42196, 51716,
                6316,  57832, 34784, 26340, 61020, 5716,  52380, 64984, 8524,
                49788, 4924,  53124, 22020, 58016, 6372,  23996, 15748, 5128,
                18104, 60072,
            },
            {
                48992, 57204, 2880,  22876, 59360, 33308, 15272, 20708, 29548,
                6880,  41304, 1016,  20076, 30368, 38268, 59188, 28712, 7900,
                41160, 48904, 19684, 1536,  42028, 28336, 4096,  49148, 31420,
                23824, 59408, 33180, 56308, 18148, 39224, 61080, 12660, 46124,
                27316, 41784, 11016, 3456,  36028, 16860, 28460, 33380, 56540,
                18000, 39104, 10884, 63840, 52304, 32672, 13388, 45248, 30492,
                36080, 47540, 25636, 19652, 64296, 45540, 11052, 33320, 56876,
                14124, 24468, 36192, 42240, 18852, 56796, 37668, 61940, 53388,
                17880, 29356, 64752, 42344, 8024,  46904, 34976, 55608, 23948,
                9740,  35664, 25428, 63656, 30896, 3788,  19504, 33436, 49980,
                44512, 17976, 23056, 37852, 50928, 704,   30136, 8312,  12588,
                19584, 1472,  52312, 46056, 4216,  28412, 62892, 9476,  33160,
                23708, 3436,  42560, 12104, 45352, 39716, 24968, 3364,  29680,
                57680, 26352, 63296, 37024, 14316, 32056, 60400, 44828, 53460,
                33976, 30696,
            },
            {
                20416, 37708, 26420, 43764, 53804, 372,   40716, 9704,  59928,
                25192, 48600, 58160, 43644, 63652, 2940,  21784, 13292, 53416,
                4968,  29796, 58684, 51820, 11924, 47400, 15916, 56060, 10220,
                14260, 46544, 63108, 1108,  44336, 24644, 32256, 58224, 22116,
                1560,  51756, 57288, 63428, 24432, 41024, 53888, 9384,  64636,
                51284, 13800, 59352, 21008, 41704, 9500,  38488, 50768, 7344,
                16684, 53548, 3384,  32460, 14592, 53200, 1240,  18756, 41148,
                8076,  51956, 5428,  54656, 48348, 24180, 14880, 4028,  40764,
                10004, 47656, 12736, 20600, 51268, 25604, 62488, 40820, 4896,
                50864, 60736, 1212,  38332, 43732, 59136, 26864, 13152, 60780,
                28972, 4472,  48188, 32848, 27268, 61952, 42456, 58044, 47928,
                25204, 59596, 35248, 13712, 41368, 31880, 18800, 59792, 13252,
                39228, 19284, 62608, 50508, 9072,  18352, 36412, 46420, 12584,
                34540, 16328, 1944,  43488, 10992, 50960, 39332, 27472, 62908,
                2004,  42564,
            },
            {
                6336,  16492, 50912, 8540,  30484, 18624, 46944, 52608, 32520,
                37824, 17552, 11604, 34652, 8336,  55300, 45896, 26568, 64236,
                33892, 43676, 14424, 38344, 64560, 24460, 36176, 62480, 43296,
                26812, 4252,  21296, 29332, 15168, 53344, 7276,  35896, 17340,
                62360, 38160, 20100, 32564, 13972, 50128, 2932,  44024, 25864,
                4696,  31392, 35944, 49160, 5824,  24548, 54728, 62664, 22372,
                43112, 11780, 56648, 39532, 5992,  42808, 35540, 63148, 26496,
                46864, 59532, 21676, 12824, 2668,  34036, 44572, 51348, 21804,
                35768, 1924,  54696, 39300, 57412, 128,   11428, 44556, 14824,
                20356, 32732, 54240, 15956, 10576, 23124, 6232,  53348, 36364,
                9948,  40312, 65340, 12188, 53524, 16848, 21260, 34144, 54204,
                5816,  39072, 9956,  20884, 56960, 6080,  44784, 52688, 27232,
                48764, 56404, 28884, 508,   32344, 59784, 56072, 21848, 60880,
                41344, 48448, 24404, 54856, 29060, 3080,  18884, 8640,  22080,
                12940, 55856,
            },
            {
                46688, 60548, 34172, 63452, 13000, 58416, 23816, 6068,  65532,
                3252,  54864, 28228, 50428, 24136, 15712, 36560, 49484, 476,
                17496, 56632, 22456, 3212,  32072, 7680,  19932, 348,   50368,
                34316, 53700, 41112, 36760, 50116, 65012, 10208, 40960, 49152,
                5528,  29032, 9004,  47028, 6444,  60068, 22024, 35400, 19456,
                40280, 46244, 2612,  27488, 18316, 58216, 14992, 3804,  27616,
                33788, 63884, 20532, 48772, 61212, 28604, 22332, 55412, 3984,
                16096, 32660, 28836, 40008, 64944, 58120, 30056, 7420,  62852,
                59216, 31516, 23988, 15252, 33680, 28568, 18640, 53960, 37752,
                26648, 46960, 7916,  28736, 49612, 56580, 39560, 46444, 16884,
                24612, 55544, 20512, 2764,  46116, 9076,  40060, 3280,  14736,
                31788, 51260, 27516, 61920, 49300, 24388, 1276,  34972, 6948,
                43256, 15776, 35452, 22828, 53204, 14604, 28468, 2348,  51568,
                9920,  19728, 7392,  58828, 33728, 65528, 47728, 57028, 41060,
                51496, 29120,
            },
            {
                24352, 3228,  39172, 21112, 4756,  42420, 35708, 27460, 39816,
                14348, 22104, 45572, 5596,  40360, 60160, 31204, 10488, 39936,
                24856, 47584, 9628,  60956, 45260, 53504, 41460, 29560, 58536,
                17808, 8872,  61736, 12292, 5080,  20744, 45756, 25920, 53864,
                14680, 44052, 54892, 26344, 38928, 56260, 30640, 12276, 62140,
                57828, 16224, 53236, 61560, 44360, 30272, 35156, 46808, 58964,
                40868, 176,   30732, 24496, 9404,  16932, 51048, 12412, 37420,
                43544, 62040, 9928,  46076, 17044, 26124, 10852, 19124, 42296,
                26704, 5696,  45852, 63736, 7268,  59664, 48204, 31144, 65452,
                2652,  57644, 40108, 63116, 19092, 34204, 336,   31104, 62856,
                5420,  43392, 34632, 29832, 59200, 37144, 61216, 26880, 44816,
                64956, 17932, 42492, 7372,  15944, 37920, 63544, 57928, 22316,
                10432, 65300, 4556,  47256, 40692, 7340,  43932, 17384, 37504,
                32476, 61992, 45084, 39040, 21436, 15656, 5460,  25708, 32404,
                36876, 9924,
            },
            {
                57432, 14844, 45272, 26888, 49412, 55244, 15764, 47948, 10216,
                51548, 61924, 32116, 56524, 19096, 3856,  52656, 20876, 61788,
                6652,  54416, 29256, 35388, 16244, 26056, 60268, 13408, 6628,
                39188, 25208, 47616, 57356, 27932, 33764, 59544, 84,    31208,
                23396, 34728, 60800, 18564, 1216,  15792, 45560, 7172,  48724,
                28532, 9124,  22780, 12540, 37856, 1484,  49872, 10968, 18792,
                8324,  51500, 14008, 37636, 44776, 58604, 2308,  30796, 48384,
                7124,  19696, 53008, 36632, 5896,  50272, 38948, 55172, 48576,
                16344, 37468, 50620, 10712, 43516, 22228, 36484, 5804,  10324,
                42740, 13400, 23656, 3920,  45440, 12248, 59636, 21512, 48708,
                13680, 58124, 51620, 15288, 6772,  24012, 48836, 11188, 56144,
                22856, 20,    47640, 33700, 54284, 30044, 12748, 46748, 17736,
                31732, 53740, 37816, 12696, 61896, 25532, 49256, 64572, 4724,
                54308, 26792, 500,   11736, 52960, 36136, 42760, 61548, 1316,
                17288, 64504,
            },
            {
                40572, 32768, 53112, 10996, 61860, 31044, 2168,  60688, 19668,
                43244, 804,   36932, 9304,  64584, 27308, 44292, 14520, 57528,
                37176, 42324, 19604, 4908,  49416, 10592, 37756, 47076, 22152,
                55556, 31572, 15576, 2176,  43232, 17688, 39024, 12968, 56360,
                64140, 3368,  11740, 41720, 52332, 65420, 36820, 24232, 54976,
                664,   33988, 42440, 56400, 7484,  65148, 21980, 39304, 26380,
                55680, 46052, 60616, 4364,  54164, 32880, 40936, 64524, 24332,
                57916, 27380, 1440,  56140, 23684, 61500, 2020,  31924, 12932,
                3696,  57168, 19516, 27548, 40464, 2436,  56024, 17172, 49728,
                21412, 53072, 30764, 36824, 50824, 26044, 42452, 8748,  28032,
                38276, 2008,  25764, 41144, 19240, 55012, 32420, 17128, 4892,
                29716, 36552, 58812, 10548, 21428, 3028,  40924, 26592, 50640,
                1780,  59996, 27824, 18064, 55512, 34164, 10408, 29824, 22888,
                40428, 14848, 31084, 50104, 24656, 8972,  46068, 13352, 53708,
                49084, 21584,
            },
            {
                7752,  28264, 900,   18116, 37916, 8596,  22620, 33824, 57020,
                29288, 25480, 16784, 42308, 12592, 48012, 35180, 2248,  32156,
                26448, 11896, 51888, 63288, 23280, 57176, 2012,  33300, 64812,
                3892,  44488, 52676, 35684, 23168, 63496, 52136, 6968,  20316,
                46352, 37572, 50804, 22600, 33004, 27280, 10648, 43104, 17744,
                38804, 63748, 50516, 25600, 32044, 17144, 52628, 60080, 5212,
                29888, 22656, 35272, 18028, 26172, 7560,  20700, 15356, 10556,
                50416, 41668, 33808, 13320, 43076, 20516, 35404, 45288, 63348,
                23572, 52768, 34408, 61304, 14588, 52100, 24996, 60464, 27988,
                33996, 61492, 17696, 58736, 6640,  15648, 64876, 52188, 18300,
                33072, 61756, 10316, 46788, 63888, 1092,  43684, 51828, 61828,
                40568, 13516, 53056, 25616, 44548, 61444, 55908, 7912,  36620,
                42380, 23356, 45564, 5452,  20872, 1596,  59024, 42884, 12392,
                56692, 46460, 63688, 18084, 59552, 55736, 27664, 19596, 30572,
                4368,  43904,
            },
            {
                54752, 62884, 46916, 57908, 25040, 41612, 52432, 45212, 12404,
                6296,  49460, 58772, 53936, 30488, 21928, 6512,  50280, 63684,
                16768, 46344, 980,   39452, 30596, 42680, 18700, 27476, 50872,
                11512, 19672, 60560, 7688,  10776, 47964, 28784, 32984, 42328,
                9760,  16872, 29664, 7780,  58552, 4004,  49420, 60756, 6184,
                30880, 14200, 19844, 3072,  47740, 41076, 13140, 33452, 43812,
                15440, 64204, 11220, 41980, 50268, 61832, 47160, 35000, 39076,
                3704,  16664, 63568, 30516, 7880,  59496, 15180, 27940, 9092,
                41456, 30376, 744,   8460,  47188, 31832, 11648, 45020, 39584,
                8136,  788,   44616, 10452, 55732, 40440, 31844, 5752,  57124,
                43916, 50112, 21228, 30676, 36696, 13344, 27760, 34684, 8824,
                46952, 19484, 63828, 5160,  38512, 16544, 32500, 20216, 64088,
                14668, 8852,  52112, 39164, 31428, 46980, 36728, 52764, 19248,
                7736,  35432, 5264,  42368, 3204,  33376, 6844,  39940, 58168,
                35352, 11908,
            },
            {
                38256, 14240, 20380, 34412, 5312,  64616, 16548, 3128,  39200,
                63356, 34812, 20684, 3496,  38644, 61540, 56284, 40640, 23588,
                8408,  59828, 33532, 14088, 54680, 8704,  61344, 15448, 36440,
                41488, 24452, 30124, 39580, 56148, 16428, 4524,  58928, 50216,
                26664, 62008, 56788, 47320, 19444, 13512, 34404, 21096, 53400,
                46488, 59244, 10472, 36452, 61804, 27892, 9048,  55192, 1020,
                38176, 48584, 2836,  57508, 28844, 13228, 692,   56916, 29696,
                61008, 21944, 52192, 47140, 26228, 49588, 53976, 4568,  57812,
                47996, 16892, 64496, 39132, 20784, 58960, 35948, 3636,  64268,
                15048, 48652, 24816, 35296, 28860, 22196, 1396,  37060, 24028,
                12636, 3748,  56068, 7596,  52980, 59464, 5932,  20560, 58220,
                24716, 2576,  33088, 28480, 51204, 11852, 47364, 5588,  54004,
                34332, 29592, 58832, 11744, 63624, 24512, 15212, 4244,  61000,
                25768, 49584, 21504, 29096, 38684, 15780, 64332, 47972, 22572,
                61012, 25820,
            },
            {
                5628,  30956, 51600, 43380, 12684, 49316, 27328, 55560, 31316,
                23896, 14820, 8216,  45804, 26180, 15920, 10132, 29292, 19148,
                43120, 55368, 28036, 21496, 48328, 3032,  45464, 52864, 5336,
                59232, 48520, 652,   62392, 26492, 44808, 37372, 13612, 23500,
                1388,  34996, 5096,  38328, 42728, 63564, 29296, 40884, 2804,
                23628, 27088, 43576, 54740, 5400,  23304, 45696, 19172, 62424,
                24768, 31624, 20452, 8812,  38664, 23092, 43144, 53476, 25692,
                45388, 5416,  36888, 10984, 116,   40100, 18284, 38008, 33652,
                21464, 11788, 44884, 25940, 54156, 6284,  16608, 23392, 51596,
                30204, 55220, 41872, 63188, 18880, 48292, 54244, 45772, 62532,
                17012, 35200, 27192, 39340, 17672, 23476, 42532, 50372, 38796,
                15404, 43520, 49468, 8528,  57476, 22908, 60412, 25788, 44092,
                3644,  49200, 17096, 41600, 56544, 8184,  50252, 28624, 39084,
                32740, 57976, 10340, 61528, 46964, 23844, 51920, 9728,  44,
                16632, 50088,
            },
            {
                57292, 23432, 8932,  60228, 29648, 36680, 21080, 9908,  59616,
                47792, 53436, 41764, 60456, 51396, 420,   37104, 47388, 52428,
                3512,  35864, 5988,  64648, 37996, 25840, 33984, 20936, 29004,
                9340,  14708, 32816, 18784, 50940, 2768,  21012, 60676, 40204,
                53500, 45232, 15316, 25140, 55472, 836,   51212, 16168, 56820,
                37936, 7272,  51484, 16368, 31440, 58632, 49916, 36052, 7236,
                51840, 59204, 45264, 54468, 33876, 65088, 6740,  18492, 9156,
                13764, 58916, 19028, 56440, 32572, 65396, 12440, 25184, 61604,
                6464,  55308, 36324, 4048,  29688, 49620, 41700, 56392, 10248,
                37692, 21036, 6088,  12488, 3548,  60532, 14444, 9680,  29840,
                41252, 58788, 48936, 10948, 45160, 65048, 2344,  30160, 10868,
                62632, 55080, 36672, 17784, 41796, 840,   30648, 39524, 10436,
                62580, 21760, 26540, 48,    35052, 19800, 44128, 62800, 1160,
                16988, 43688, 14232, 53596, 1800,  34752, 12464, 29488, 42784,
                33036, 45480,
            },
            {
                65096, 40184, 17796, 1352,  53640, 4348,  45988, 41196, 18020,
                1412,  36532, 11548, 31016, 18812, 33872, 65112, 12648, 59108,
                25588, 15008, 50060, 10188, 17568, 58488, 12528, 63104, 43704,
                56848, 36984, 53468, 41836, 12340, 34864, 54876, 29172, 10552,
                18716, 64940, 8364,  31756, 11488, 23080, 36536, 8908,  62464,
                12732, 33712, 64848, 20604, 39176, 2692,  14584, 29324, 41716,
                12164, 16688, 27144, 3880,  14556, 51528, 31456, 37612, 48948,
                34132, 41052, 28992, 43920, 23728, 46168, 3848,  52276, 43060,
                28340, 48928, 60152, 18584, 13304, 61160, 33856, 27632, 18328,
                2720,  59368, 33144, 50996, 38460, 43216, 33932, 26256, 51836,
                6856,  22236, 276,   61380, 32868, 14912, 35908, 57360, 26636,
                6612,  31224, 21844, 12780, 65216, 34528, 52664, 15020, 18932,
                55272, 38232, 46836, 6796,  51812, 30312, 13108, 22232, 48120,
                55360, 6588,  37028, 27228, 20236, 57144, 40364, 62572, 54956,
                19416, 3852,
            },
            {
                11344, 35500, 48228, 26204, 63120, 14384, 57952, 33492, 7408,
                26276, 64180, 22296, 5540,  55392, 44588, 23112, 7312,  31848,
                39036, 61808, 44752, 30220, 42356, 53808, 4416,  39092, 23904,
                17792, 3380,  65360, 6972,  24160, 47112, 62708, 6012,  43620,
                33600, 27244, 48360, 52440, 61328, 44768, 27860, 48580, 19168,
                41816, 29212, 232,   48860, 11364, 43160, 63632, 22012, 55816,
                4844,  35396, 39880, 60336, 19640, 46436, 2028,  61612, 55228,
                22576, 64024, 2416,  8384,  15564, 58448, 19872, 30972, 9480,
                15116, 1340,  23716, 39928, 44104, 9000,  1052,  47776, 63048,
                43440, 46768, 25476, 16580, 28288, 57352, 20920, 2148,  64252,
                47440, 13880, 30792, 52156, 25608, 5076,  48172, 53648, 18776,
                45340, 1644,  59972, 47532, 4580,  26956, 46188, 59288, 2536,
                33488, 28656, 60308, 15812, 64856, 37576, 54452, 10148, 40472,
                24732, 31036, 65236, 50388, 4932,  45224, 16160, 24736, 7684,
                37084, 27052,
            },
            {
                15328, 55916, 42628, 32764, 20156, 38228, 23632, 51552, 61316,
                44304, 29164, 50568, 40224, 14100, 27712, 57756, 41888, 17300,
                1988,  20752, 56012, 24276, 172,   32680, 47156, 7700,  51228,
                31200, 46268, 21372, 28300, 58308, 9388,  31612, 17032, 49948,
                58004, 4124,  20676, 38920, 17160, 6460,  59152, 32496, 4452,
                46240, 60280, 24360, 57608, 34284, 26776, 51276, 9344,  32972,
                61968, 23048, 49684, 9892,  42064, 24608, 28428, 16512, 11920,
                4192,  26368, 47800, 53820, 38224, 34236, 49908, 62920, 40564,
                56784, 35032, 51436, 31812, 64908, 21276, 57128, 24368, 11916,
                30776, 7400,  53824, 65264, 9104,  5000,  53356, 17380, 35692,
                43884, 38404, 19024, 56608, 39992, 20972, 8432,  41552, 13820,
                33804, 52452, 25268, 37580, 56296, 19804, 7884,  42596, 24528,
                50876, 12724, 8892,  44500, 24300, 4720,  58752, 33928, 3088,
                59992, 12112, 18124, 41904, 9112,  60884, 32324, 2200,  48820,
                59912, 52168,
            },
            {
                22476, 2480,  7164,  59452, 10792, 49704, 5876,  30356, 12252,
                16200, 56472, 2236,  47120, 62436, 9568,  3816,  49164, 53948,
                46008, 34180, 7920,  13432, 60804, 19864, 27832, 15392, 55704,
                11184, 60952, 38556, 14256, 43288, 52228, 39192, 856,   22940,
                14024, 36968, 54988, 2120,  42056, 21984, 13896, 56048, 25548,
                53688, 15016, 8016,  45080, 16904, 5760,  59460, 18928, 47512,
                44576, 2520,  54948, 30788, 63432, 7132,  58568, 39396, 44332,
                51872, 32228, 17364, 61036, 11524, 5664,  26756, 2740,  22404,
                46744, 17468, 11056, 6768,  53032, 36072, 16020, 39388, 54768,
                35412, 14712, 40828, 22596, 47976, 31340, 40044, 58532, 11240,
                23680, 60112, 4356,  9708,  44696, 62260, 29232, 59656, 23172,
                63784, 39728, 16392, 10368, 29820, 50264, 13856, 36100, 64208,
                5768,  39956, 57616, 32368, 18728, 42104, 27048, 45632, 20436,
                52072, 35344, 47484, 22832, 29724, 38024, 53292, 20664, 13448,
                43964, 29408,
            },
            {
                46656, 63548, 30748, 16124, 45364, 40520, 268,   54716, 36196,
                21424, 8600,  38504, 17740, 34556, 20548, 37540, 30464, 26364,
                11208, 63728, 51252, 40336, 36400, 49224, 64492, 41292, 34612,
                26048, 1568,  50072, 33112, 4172,  19660, 26012, 64096, 41516,
                60852, 46120, 11136, 28432, 64392, 35096, 50708, 1304,  39520,
                10656, 21752, 38324, 29756, 55284, 40748, 36512, 1220,  26224,
                13988, 28644, 17868, 37200, 12816, 48288, 34464, 20212, 8688,
                59904, 37308, 21064, 42468, 29500, 55884, 45112, 13296, 36664,
                60896, 29440, 57972, 26200, 46304, 3552,  28832, 50780, 4396,
                20408, 59908, 716,   36740, 12224, 44892, 24824, 49552, 7348,
                32640, 54272, 27820, 35092, 16652, 50948, 940,   37352, 11164,
                4224,  49232, 7388,  44348, 62896, 40196, 22380, 54012, 30708,
                17896, 48624, 21576, 53844, 1972,  51080, 14784, 8200,  62280,
                28248, 6084,  55656, 600,   58952, 11492, 26612, 64232, 33668,
                38964, 9032,
            },
            {
                54388, 34804, 50548, 25160, 56792, 27992, 18996, 63724, 47520,
                43192, 59076, 32048, 25132, 54620, 51476, 61376, 15344, 59348,
                22520, 4464,  28216, 17820, 57008, 10380, 3232,  22040, 58824,
                6176,  44468, 17412, 55112, 59872, 36336, 11424, 53676, 30112,
                7932,  25168, 32676, 57384, 47576, 9216,  30400, 43344, 62612,
                35544, 51704, 64048, 3136,  49360, 23448, 12388, 53036, 65416,
                38508, 57388, 6220,  51724, 22204, 860,   56756, 53280, 30240,
                5812,  13828, 50296, 752,   23888, 64468, 39352, 19036, 53364,
                6544,  2024,  41976, 19688, 14052, 62564, 42692, 8656,  61496,
                45728, 28472, 49724, 57944, 18580, 62764, 3376,  29412, 61732,
                16064, 1308,  48536, 64676, 12796, 24864, 33256, 47184, 54652,
                27592, 57764, 32036, 18320, 380,   58412, 5240,  11120, 43772,
                1372,  61960, 35736, 27572, 10860, 63272, 33044, 49208, 17328,
                38128, 44084, 15228, 33128, 51556, 17756, 45720, 3388,  56280,
                5660,  18272,
            },
            {
                40712, 12560, 20788, 3528,  36408, 60764, 10196, 33684, 23908,
                2948,  13912, 64892, 4880,  11560, 42340, 644,   8320,  40852,
                35700, 55580, 43740, 31592, 6616,  25516, 45152, 52548, 13032,
                37280, 62004, 29956, 22868, 7796,  47900, 44812, 15868, 35060,
                4504,  51352, 18980, 15444, 5780,  23876, 60128, 19828, 16504,
                27752, 6032,  32812, 18748, 9764,  61004, 31768, 43880, 20944,
                8496,  42132, 60252, 33252, 45032, 26840, 14868, 42320, 24928,
                46888, 63424, 33020, 9848,  52008, 16676, 7660,  48912, 25996,
                33368, 63756, 49776, 38940, 31176, 58792, 22948, 33588, 17612,
                25004, 10304, 32240, 6404,  55028, 34216, 14144, 20440, 42952,
                51976, 38912, 22160, 41788, 55164, 5676,  58784, 19860, 15608,
                42940, 21404, 36304, 51748, 24096, 34388, 28456, 46844, 60212,
                25784, 14352, 7428,  46236, 38560, 55388, 23332, 41016, 1496,
                57056, 10636, 64072, 25060, 40212, 7512,  49668, 42600, 23608,
                62236, 26212,
            },
            {
                700,   59736, 43896, 8152,  51784, 14288, 42252, 6744,  53264,
                27084, 39988, 50036, 29568, 45744, 22220, 33336, 47180, 18908,
                49968, 13888, 1876,  47812, 59564, 15736, 38952, 29200, 18628,
                49040, 9160,  42096, 13524, 63088, 27836, 3152,  59268, 21716,
                56104, 43660, 62900, 36268, 41164, 53632, 12672, 45996, 2684,
                48488, 58408, 41404, 45488, 57060, 27100, 16036, 4628,  34916,
                50604, 24552, 15640, 3392,  64264, 38396, 10496, 62172, 18280,
                2208,  40876, 27216, 58284, 44188, 35504, 31500, 59284, 11784,
                43728, 15024, 23924, 7476,  53420, 212,   12512, 49156, 56100,
                41176, 64000, 15728, 42848, 26316, 39312, 9028,  56340, 35952,
                26192, 12040, 8404,  30864, 17816, 37944, 44220, 8128,  31444,
                1804,  64596, 12120, 45680, 61692, 14788, 54840, 19512, 38336,
                32112, 56376, 41676, 17220, 30076, 3408,  12952, 60500, 25944,
                30960, 21540, 47032, 3012,  60968, 28944, 34856, 15856, 9912,
                31060, 47904,
            },
            {
                16964, 28900, 38432, 65504, 24140, 46800, 30016, 16996, 57872,
                48548, 19876, 7856,  55564, 16876, 63184, 25952, 57408, 5224,
                65292, 24208, 38200, 20996, 53340, 34112, 63448, 1344,  57472,
                32976, 25152, 54292, 464,   39972, 32148, 18136, 49228, 9632,
                38984, 12976, 148,   49488, 26684, 4076,  38828, 33612, 54624,
                24792, 11072, 14704, 21536, 624,   38744, 48088, 54552, 61656,
                11376, 30976, 47720, 20592, 53988, 29328, 4992,  50496, 35776,
                22320, 55064, 37848, 15352, 4092,  61976, 22768, 2864,  55072,
                20268, 37336, 56436, 10384, 35044, 44596, 27612, 38272, 5708,
                2448,  36256, 52936, 21828, 1828,  51404, 47224, 63852, 2700,
                45608, 60748, 57880, 47416, 2080,  63212, 26912, 50452, 60700,
                40916, 53408, 6292,  26868, 9316,  41988, 3020,  49444, 8552,
                22732, 51824, 4428,  65508, 58776, 20748, 47260, 35920, 4988,
                53968, 42416, 36312, 19440, 53132, 12244, 21924, 58844, 51140,
                36960, 57600,
            },
            {
                22552, 52872, 11380, 31964, 19600, 55708, 4268,  63044, 37436,
                11132, 32844, 61668, 36220, 2352,  38868, 12988, 52084, 28748,
                32364, 9748,  61844, 27480, 11772, 41864, 8436,  23652, 46464,
                4640,  64900, 19752, 35976, 51472, 57776, 24668, 41748, 65220,
                28700, 23516, 31544, 61060, 20908, 58092, 28912, 64300, 7792,
                37244, 62060, 31048, 53176, 35496, 64212, 7872,  29248, 18460,
                1408,  56532, 36720, 7908,  41120, 17280, 57784, 31436, 60176,
                12324, 49080, 7176,  20232, 47852, 10976, 28548, 39796, 46576,
                5532,  27464, 60828, 47448, 18680, 65136, 15312, 51948, 21148,
                30188, 58360, 12052, 45144, 59960, 30496, 23064, 17268, 32080,
                6344,  19740, 34580, 28164, 52536, 21368, 10700, 34936, 13476,
                23484, 29876, 17284, 48596, 37256, 31156, 57468, 16660, 63232,
                35284, 12556, 48168, 24860, 34148, 7824,  43272, 50888, 18292,
                63180, 8352,  14264, 57628, 4648,  38632, 64960, 1848,  41604,
                13540, 4776,
            },
            {
                45952, 62116, 6568,  49016, 1500,  40308, 34436, 21348, 44948,
                912,   15672, 23308, 44376, 27580, 59512, 9288,  43280, 16248,
                40304, 45960, 51332, 6076,  18092, 60712, 31088, 51052, 14924,
                43220, 11172, 27552, 47008, 15292, 7044,  11952, 33972, 2088,
                54132, 47092, 16896, 7564,  45312, 10564, 15216, 50580, 21936,
                17608, 44004, 5072,  49504, 25216, 13316, 42212, 22640, 46272,
                39508, 26304, 62836, 12704, 49372, 25340, 45328, 8792,  42860,
                3660,  26064, 64788, 30828, 41580, 56572, 52500, 17540, 64576,
                32656, 50924, 1720,  30372, 22352, 57052, 7048,  41960, 62232,
                46220, 7952,  27304, 19076, 37360, 5548,  11192, 40792, 25176,
                53412, 13720, 42272, 4484,  14816, 40516, 57316, 47724, 4104,
                55932, 39108, 3240,  59120, 52404, 22172, 6564,  40028, 27948,
                808,   43012, 18912, 10036, 39884, 52932, 15308, 28556, 11408,
                33424, 48484, 29884, 26328, 43664, 32220, 47280, 25544, 19548,
                55548, 33396,
            },
            {
                27056, 41996, 36096, 15872, 58940, 26244, 12744, 52572, 28520,
                59700, 41584, 50752, 6116,  54072, 20740, 34324, 4264,  55792,
                19972, 1040,  57136, 34928, 44380, 4140,  55516, 20544, 39804,
                35264, 52888, 59952, 4044,  22568, 43804, 61468, 52076, 20176,
                6016,  37800, 56880, 35004, 52508, 42264, 32144, 1192,  40684,
                56128, 28396, 59780, 9100,  19280, 56724, 2868,  33440, 59468,
                14312, 43652, 4536,  31984, 55212, 604,   21940, 14488, 33736,
                53040, 16752, 57448, 34696, 24320, 13940, 436,   36016, 9128,
                42964, 13080, 16808, 40832, 4960,  36792, 32596, 11128, 24588,
                16396, 34876, 50692, 64828, 14616, 54772, 61232, 48972, 57244,
                37740, 65464, 50596, 24216, 61644, 31784, 5832,  25836, 19520,
                45396, 62136, 33312, 19896, 12692, 64988, 25668, 44832, 52824,
                58208, 29980, 61536, 54992, 26804, 1392,  57568, 61124, 24108,
                40232, 168,   59944, 51076, 17140, 9528,  54124, 6112,  29620,
                60932, 9036,
            },
            {
                2728,  18724, 24492, 54044, 44256, 61856, 9116,  47932, 4884,
                24936, 55760, 10420, 30720, 14532, 46324, 49496, 23892, 60868,
                37096, 12548, 30292, 22396, 49648, 37900, 13256, 28600, 61764,
                6140,  17588, 31376, 38720, 55168, 29452, 37232, 26252, 44912,
                14772, 62656, 11264, 24344, 3524,  19268, 59892, 25344, 46732,
                13028, 2548,  34024, 39960, 46924, 27324, 62212, 50436, 9664,
                53300, 21264, 58656, 17996, 35288, 65164, 38944, 61188, 47344,
                28896, 39784, 9684,  45972, 5476,  61308, 49308, 26272, 21076,
                60296, 24072, 54872, 63308, 45588, 25956, 53944, 59900, 48000,
                3780,  55780, 40376, 504,   32696, 42980, 27960, 7784,  15760,
                928,   29352, 9144,  45992, 35492, 17060, 53240, 64452, 36316,
                15436, 10068, 26360, 42408, 47072, 2016,  33744, 11040, 20584,
                14440, 37596, 3648,  45068, 32248, 21256, 37388, 5612,  44824,
                54544, 21632, 7024,  36796, 23440, 61704, 14512, 35340, 44716,
                39400, 50408,
            },
            {
                64592, 57744, 13196, 30380, 4508,  36980, 17664, 32188, 64864,
                38376, 18772, 34912, 63640, 39360, 2676,  62444, 31132, 7812,
                52836, 26628, 64488, 15332, 58700, 25364, 608,   47268, 9676,
                57332, 24776, 49476, 9988,  64220, 2952,  17248, 8980,  59028,
                32504, 41216, 50316, 29784, 64712, 38084, 54528, 6236,  35852,
                62940, 51156, 23316, 15952, 54348, 4812,  36480, 17488, 30552,
                6396,  37836, 28812, 46720, 8276,  51032, 26992, 3200,  19760,
                58672, 23036, 1668,  51856, 19400, 37972, 43856, 31956, 57336,
                6672,  38132, 33900, 9724,  49656, 14180, 952,   18372, 38000,
                31472, 20860, 10668, 25252, 48176, 4404,  35580, 20108, 44860,
                33232, 22484, 12396, 55776, 1688,  43780, 8260,  29112, 41572,
                16,    51148, 54900, 7140,  36572, 16352, 56332, 60860, 47960,
                7188,  23864, 49688, 16596, 12756, 64352, 49044, 9828,  34968,
                15984, 63908, 12832, 45864, 40436, 56076, 1044,  49052, 16888,
                21864, 11892,
            },
            {
                37128, 47548, 34108, 8236,  51028, 22460, 57212, 42712, 14016,
                7888,  46588, 1600,  56944, 17464, 25828, 11768, 41932, 17908,
                48220, 43872, 2644,  40448, 8220,  32032, 63912, 52288, 22088,
                33472, 1980,  42000, 14032, 20492, 46260, 56668, 50688, 4652,
                21680, 27160, 1272,  17492, 44132, 13184, 48136, 9768,  30752,
                20276, 43308, 10936, 64480, 31656, 44896, 12420, 41388, 24312,
                64036, 49116, 2584,  23872, 42476, 16080, 11940, 54300, 6704,
                35984, 13144, 42604, 63560, 28640, 10736, 54456, 15428, 3424,
                45876, 52876, 27636, 2788,  20208, 61712, 29224, 42832, 8568,
                57540, 62928, 44136, 54096, 17072, 22772, 63488, 51660, 58564,
                40160, 47772, 61148, 19084, 38312, 59800, 22820, 11476, 48956,
                60628, 31480, 23336, 62808, 28964, 50644, 39160, 4668,  31552,
                41900, 63404, 33980, 59124, 4980,  41408, 29484, 18892, 51628,
                26696, 31976, 57276, 3144,  19868, 26944, 31304, 62880, 7628,
                52972, 28932,
            },
            {
                43064, 1348,  20204, 62544, 40596, 27672, 612,   49752, 20864,
                29960, 51732, 22960, 28272, 43628, 53080, 36568, 58156, 3756,
                33428, 20956, 10848, 54360, 46556, 19288, 42484, 15864, 37000,
                45148, 63136, 54208, 26860, 34552, 40624, 30832, 12720, 36004,
                48104, 60552, 55108, 8096,  33852, 22664, 27512, 61140, 16508,
                57896, 26416, 3908,  37760, 7692,  20644, 60704, 456,   55604,
                44480, 14976, 10872, 60848, 55880, 33592, 29748, 43824, 48900,
                61880, 31464, 55984, 15880, 33912, 23388, 7584,  63032, 29520,
                18360, 12044, 59392, 44028, 32296, 39468, 52684, 22808, 50284,
                14716, 27840, 6420,  30808, 59516, 39688, 9200,  13012, 25876,
                3600,  7264,  53048, 30728, 25560, 49768, 33776, 56664, 20244,
                38716, 5940,  13288, 18548, 44364, 9844,  22488, 26748, 17276,
                55488, 9080,  27864, 52444, 22968, 47240, 56512, 2260,  62124,
                42724, 8572,  48952, 34520, 52492, 10780, 42920, 37876, 24952,
                56496, 5296,
            },
            {
                16048, 26524, 55440, 44980, 15104, 59624, 11500, 34620, 54924,
                61220, 40800, 11384, 59984, 4976,  8532,  21896, 29144, 14096,
                63176, 37460, 59692, 28044, 35112, 5108,  56016, 11284, 29428,
                4320,  19080, 7640,  48320, 60656, 996,   22448, 63660, 43024,
                18600, 10536, 39364, 45976, 57732, 5016,  52892, 41744, 648,
                38860, 55076, 47304, 52296, 29160, 57172, 48500, 26652, 32888,
                18944, 35588, 52904, 39512, 20160, 5360,  63476, 37696, 17768,
                25120, 8632,  46548, 4728,  49944, 58716, 41544, 47212, 36524,
                50772, 40192, 22700, 15256, 56236, 6280,  11888, 64660, 34608,
                1744,  47168, 37272, 11984, 49528, 2304,  56408, 30396, 18236,
                62812, 35160, 15560, 41384, 3904,  14044, 6392,  45676, 16148,
                26852, 57652, 48064, 32944, 58800, 1064,  64216, 53376, 46336,
                36756, 20152, 332,   39612, 11060, 35796, 25316, 14368, 38900,
                5132,  22720, 17676, 29284, 65328, 4520,  58460, 18856, 13056,
                33108, 60556,
            },
            {
                22156, 51636, 10692, 31760, 3132,  38076, 47116, 25712, 6908,
                3356,  36724, 15196, 48736, 34048, 65064, 45920, 50524, 54636,
                24672, 6008,  49180, 16704, 23220, 60788, 38700, 25648, 50044,
                58720, 39596, 31772, 11556, 16408, 51848, 7072,  28160, 53564,
                4012,  24916, 31396, 14940, 62508, 36896, 18964, 49888, 32748,
                8484,  14068, 34492, 17560, 23920, 40456, 14428, 9468,  51608,
                3872,  62412, 26940, 31252, 1436,  58144, 23132, 10088, 52560,
                164,   40260, 20776, 27296, 38572, 1960,  12912, 20928, 25812,
                812,   65324, 5236,  48432, 25304, 36624, 45316, 4332,  26560,
                40860, 16760, 61864, 19796, 33992, 24268, 45640, 36972, 43068,
                49956, 27956, 57396, 9696,  64652, 51892, 28628, 61432, 2292,
                35372, 42924, 4200,  52952, 40348, 15688, 35224, 11960, 3260,
                60036, 13644, 44416, 57716, 17968, 7452,  60844, 31700, 45712,
                54184, 59020, 37964, 44952, 15004, 23600, 50720, 46264, 2204,
                41188, 48532,
            },
            {
                38552, 6688,  36152, 64500, 23160, 53068, 18984, 63336, 16404,
                45140, 26700, 54368, 31244, 19976, 39912, 16424, 2108,  38932,
                10100, 44568, 30684, 52624, 13024, 2784,  48020, 8064,  62164,
                14296, 23256, 56836, 44144, 36804, 25280, 58820, 38408, 14644,
                34000, 61748, 51712, 20472, 28792, 2596,  12020, 24480, 65492,
                44296, 21964, 62400, 2032,  59824, 5976,  35252, 64920, 39080,
                22560, 7012,  44032, 13760, 50504, 41692, 46664, 14876, 28500,
                58956, 34904, 65020, 54688, 17448, 61276, 31000, 53156, 57800,
                10120, 34064, 30328, 61120, 8680,  17228, 60236, 20376, 53816,
                58876, 23932, 51296, 42220, 5568,  65032, 15384, 54584, 248,
                11360, 23232, 46696, 20056, 43956, 36140, 22164, 40084, 54400,
                10952, 65280, 20780, 8912,  23928, 27688, 49424, 43188, 29964,
                24888, 50832, 32492, 65192, 28808, 42228, 52924, 20764, 10104,
                28388, 13236, 728,   60796, 40836, 6684,  35676, 27704, 63608,
                9172,  29492,
            },
            {
                61568, 57208, 17952, 47848, 29148, 8796,  43312, 32408, 56464,
                21612, 62316, 8164,  480,   58744, 12200, 26440, 32576, 60652,
                19856, 57460, 40,    40856, 65488, 43564, 31868, 20480, 34844,
                768,   52820, 27492, 4856,  65144, 19768, 47040, 9108,  57096,
                44280, 252,   48512, 7260,  40708, 56020, 47424, 30224, 58544,
                4212,  28232, 50384, 11972, 42364, 46392, 19716, 29640, 56244,
                16264, 47700, 59008, 8988,  25260, 18420, 33944, 55528, 4148,
                44540, 49212, 13556, 9776,  44008, 24164, 35584, 6756,  16204,
                43836, 54320, 19808, 41628, 51588, 28008, 33324, 48024, 12836,
                31824, 9376,  2704,  29612, 53004, 26896, 8368,  21180, 32456,
                60936, 4984,  38816, 31580, 1252,  56456, 16840, 8088,  32180,
                25056, 46628, 30260, 37092, 61932, 56468, 5228,  19256, 62672,
                38888, 6260,  21440, 48312, 3480,  15300, 49356, 1540,  63808,
                36236, 50100, 25892, 9212,  53644, 31728, 16804, 57280, 21184,
                54248, 14104,
            },
            {
                36,    25456, 42120, 4116,  14252, 57888, 5332,  39600, 10624,
                30256, 41736, 47348, 24132, 51880, 44284, 55972, 7028,  47844,
                14388, 26952, 34356, 8536,  18904, 58296, 26740, 54756, 45840,
                17584, 42244, 9392,  49576, 32872, 2324,  41776, 29848, 17844,
                23224, 37204, 12956, 26116, 63824, 43864, 9192,  15276, 39684,
                18228, 55536, 37220, 32076, 26076, 54104, 2856,  49876, 11596,
                41328, 28616, 36188, 54440, 62012, 38636, 6676,  63976, 31116,
                18916, 23496, 5124,  32788, 51300, 3332,  48192, 63456, 38996,
                27420, 46824, 13772, 2808,  37812, 57300, 52,    42764, 7112,
                37632, 56180, 44940, 14508, 60768, 38848, 47420, 58608, 41000,
                51256, 17308, 63020, 53484, 26432, 11656, 47652, 63324, 50676,
                18688, 14356, 51616, 3324,  12752, 41664, 31836, 8468,  53720,
                16276, 56884, 10540, 25772, 36996, 55948, 33564, 24184, 40544,
                18472, 57724, 33968, 21488, 47676, 61740, 11504, 39488, 4424,
                34568, 44252,
            },
            {
                31528, 11792, 50560, 33732, 60536, 24892, 35880, 49816, 1432,
                52256, 13436, 37488, 17884, 34476, 4412,  62092, 22744, 36500,
                42736, 62972, 50140, 24080, 37764, 15244, 6000,  10836, 64068,
                36944, 29952, 60096, 21356, 15160, 55652, 12372, 50576, 63468,
                6092,  54216, 59180, 31860, 17180, 34344, 23272, 53328, 35516,
                7196,  45736, 9788,  58288, 15840, 62792, 8316,  33352, 24416,
                61440, 756,   20608, 32012, 3284,  12312, 49644, 26108, 10680,
                41820, 60540, 38196, 29104, 58028, 20048, 42292, 11668, 18736,
                4088,  58908, 24064, 64132, 11180, 22112, 15156, 62500, 28512,
                18424, 64228, 22000, 35552, 11012, 19012, 3432,  29052, 13148,
                25036, 34892, 8516,  14600, 59696, 41740, 34352, 5140,  43372,
                820,   60028, 39916, 55224, 17680, 47624, 59776, 22604, 44644,
                33924, 1752,  41180, 46088, 61504, 7108,  12184, 59176, 47144,
                6504,  3584,  43612, 14132, 29040, 1952,  43164, 49388, 26284,
                52240, 19572,
            },
            {
                55940, 63936, 39244, 20816, 53760, 44916, 16032, 65344, 27544,
                19848, 57424, 3604,  64144, 28312, 41124, 10764, 30824, 17320,
                52808, 5684,  11932, 56492, 46936, 29008, 52056, 40620, 23940,
                5268,  13128, 47392, 38992, 62132, 35616, 26716, 3832,  39712,
                33208, 45528, 21068, 10476, 52344, 6376,  60000, 1196,  61928,
                49296, 24784, 20004, 56,    39740, 22416, 43776, 37780, 57860,
                5944,  45076, 51440, 16912, 43096, 22464, 59668, 36292, 52080,
                1376,  54212, 16584, 62552, 7616,  26376, 56096, 33704, 30216,
                50420, 8252,  35276, 31596, 49344, 44976, 54476, 24712, 52020,
                40400, 48660, 6028,  32280, 57796, 49760, 44180, 36280, 6720,
                55380, 48124, 44412, 29400, 3516,  19224, 23744, 30048, 57996,
                27180, 35624, 22480, 7380,  33620, 25704, 620,   37688, 11368,
                27976, 63924, 52392, 17700, 30460, 20460, 44328, 27496, 16288,
                32304, 54776, 63268, 38748, 56660, 19208, 23752, 64528, 8012,
                15848, 47016,
            },
            {
                23012, 6980,  28048, 9244,  2264,  30856, 11692, 22712, 46440,
                60396, 32828, 9596,  50220, 14964, 20528, 48692, 55056, 1152,
                28128, 39628, 20940, 3960,  32160, 61984, 1244,  33460, 49964,
                58884, 55036, 1796,  24364, 7184,  52780, 43580, 20248, 60564,
                14284, 28024, 1872,  42216, 37616, 48672, 20120, 27352, 31696,
                13992, 42644, 64436, 52788, 27564, 49068, 12808, 18456, 52396,
                15224, 27284, 10020, 65412, 47916, 29508, 40480, 15420, 20692,
                47296, 34400, 12016, 45868, 40888, 14400, 488,   52772, 22760,
                61980, 40332, 55596, 17436, 4256,  39604, 6212,  34920, 10684,
                1492,  16004, 26160, 54332, 2068,  23840, 16416, 63532, 59420,
                19784, 1480,  22388, 37168, 52208, 48748, 62044, 38308, 12564,
                9024,  45532, 62692, 29048, 43496, 64256, 15132, 51356, 58204,
                48684, 23756, 13332, 35456, 3924,  54080, 38140, 64796, 10468,
                51876, 20272, 24876, 8140,  51200, 5192,  36640, 32356, 58500,
                38280, 3208,
            },
            {
                34832, 43696, 16792, 48844, 62732, 41504, 56132, 3988,  38844,
                7088,  43480, 24620, 54196, 39088, 59356, 7852,  43948, 64896,
                13636, 33800, 61100, 43196, 53664, 13384, 22136, 44580, 16188,
                19940, 27972, 32652, 45300, 18040, 30364, 10424, 56700, 24800,
                8196,  49276, 64476, 57180, 29436, 3556,  45344, 40356, 10580,
                56892, 3448,  29700, 35104, 5580,  60792, 30544, 4052,  63616,
                32388, 39564, 57036, 35188, 4828,  55868, 2616,  8644,  57656,
                27816, 5744,  25388, 21656, 49992, 35900, 64776, 9236,  44856,
                14968, 1956,  43716, 26512, 62924, 29476, 20088, 61456, 31388,
                59236, 45584, 36912, 61204, 42512, 30500, 52540, 10332, 26624,
                41940, 32792, 64872, 57868, 10504, 6928,  15644, 55332, 21548,
                53496, 16568, 2800,  49076, 10560, 54128, 21144, 30468, 3996,
                18516, 6872,  39676, 58540, 9576,  47920, 23348, 2420,  42332,
                29316, 37004, 12272, 46492, 31072, 60320, 14796, 45884, 11076,
                28996, 61136,
            },
            {
                13664, 51044, 58664, 36200, 19244, 24476, 34372, 52588, 17992,
                29808, 14336, 35484, 46092, 2144,  31604, 26140, 37172, 23504,
                57512, 46016, 9452,  25328, 17748, 39100, 7556,  63200, 36208,
                9240,  41896, 63832, 11912, 58376, 49988, 596,   37808, 46636,
                53636, 34884, 16732, 23688, 12880, 55376, 16180, 64004, 50404,
                22180, 37604, 47736, 15320, 10828, 45200, 55476, 36448, 21756,
                47020, 7284,  23528, 13572, 25980, 18440, 45156, 61768, 31608,
                42816, 64384, 55720, 3220,  31724, 59248, 17712, 47612, 37496,
                28740, 21072, 53000, 12828, 9312,  57596, 47060, 13928, 43028,
                23452, 50808, 9280,  13104, 20412, 7228,  34104, 38956, 4128,
                50668, 12192, 16484, 40124, 26196, 31668, 44552, 2280,  40984,
                49836, 32512, 57584, 19764, 38688, 5008,  35152, 41296, 46456,
                61164, 33116, 43524, 28652, 62724, 15400, 32756, 56688, 7280,
                49204, 61900, 288,   41712, 17652, 26816, 54376, 1204,  21500,
                42372, 53304,
            },
            {
                25300, 30968, 816,   12240, 46620, 6360,  59840, 10312, 49332,
                63196, 55520, 21788, 5644,  63664, 17812, 12360, 51180, 3460,
                19128, 49572, 29740, 1928,  58992, 50628, 28384, 56764, 2564,
                48964, 53628, 3800,  39644, 34292, 23040, 64888, 15968, 30848,
                4164,  19616, 40972, 6680,  61208, 35308, 25296, 4616,  33132,
                18068, 8056,  61284, 54684, 19536, 40556, 25580, 2052,  50512,
                11760, 42624, 60392, 54024, 38016, 49808, 33740, 23268, 13224,
                38772, 17516, 51452, 10704, 39848, 28180, 4692,  24960, 58108,
                6264,  33356, 60016, 36940, 23328, 51232, 38120, 3316,  56320,
                19004, 5480,  29532, 39780, 65444, 48316, 55980, 14988, 45652,
                28916, 54616, 5840,  46828, 20716, 60172, 35208, 63028, 28508,
                5948,  37180, 25108, 13340, 61488, 27020, 57120, 14328, 9904,
                25644, 52928, 20656, 1088,  50360, 26096, 40580, 18848, 59652,
                22192, 15052, 55896, 34640, 64164, 10056, 39592, 49896, 57224,
                18004, 5560,
            },
            {
                40756, 61812, 21700, 56560, 39724, 29100, 15520, 44432, 25948,
                244,   37904, 11928, 28192, 57220, 42236, 34908, 60916, 6452,
                41192, 15340, 54580, 37464, 34248, 11572, 46776, 24020, 31708,
                14324, 26556, 21620, 16488, 47628, 5340,  28124, 42800, 59096,
                11552, 62240, 44868, 31992, 51264, 9448,  39016, 43980, 59508,
                52476, 26604, 41508, 23264, 33316, 7468,  59936, 16420, 57708,
                28132, 19476, 932,   31152, 5828,  63484, 10816, 53052, 296,
                48460, 7456,  35360, 60608, 44484, 20432, 13284, 41948, 53828,
                11636, 48920, 18396, 45820, 936,   32532, 17132, 8444,  27452,
                35820, 62696, 53736, 16944, 26516, 324,   22924, 60200, 19360,
                62752, 24156, 36248, 57012, 724,   51108, 12908, 24260, 18608,
                10832, 64696, 41848, 46552, 7652,  52424, 44244, 1524,  65448,
                38060, 15904, 56304, 11456, 45368, 5900,  53268, 13020, 35576,
                44708, 25888, 5040,  47980, 20948, 6852,  44316, 24536, 33288,
                65240, 9512,
            },
            {
                36544, 45332, 4436,  26796, 51932, 64620, 2744,  32772, 41312,
                19312, 60988, 48436, 52120, 8864,  23072, 47588, 53900, 28496,
                32688, 62484, 8020,  21060, 64608, 5856,  40812, 18540, 43368,
                60120, 37880, 57392, 8068,  61256, 54852, 13424, 35556, 21140,
                52024, 38608, 26388, 872,   46752, 21696, 54644, 13868, 29836,
                1576,  12088, 46536, 3060,  64916, 51996, 29996, 9652,  44320,
                34960, 64460, 52032, 40268, 21516, 16092, 41376, 26680, 59328,
                19892, 29844, 24428, 15360, 1300,  56156, 50488, 30940, 61580,
                3148,  40616, 65268, 7376,  28220, 42524, 63920, 53440, 47952,
                40956, 2212,  33248, 46708, 58464, 43176, 36668, 8876,  32172,
                2476,  43464, 9868,  30520, 17432, 42100, 7744,  47860, 58592,
                52036, 30192, 184,   22648, 34020, 17164, 31616, 23300, 49480,
                29792, 4372,  60024, 34820, 22660, 37556, 64032, 1712,  30300,
                9248,  40104, 32620, 53060, 28928, 58892, 35728, 13356, 2756,
                28488, 48164,
            },
            {
                15084, 55200, 33580, 17720, 8776,  37008, 20900, 54568, 58300,
                7620,  31112, 16300, 39832, 33216, 1692,  14888, 20572, 10688,
                45128, 24520, 39840, 47784, 26964, 15836, 52348, 61680, 10032,
                5036,  51572, 44688, 30892, 25500, 40996, 9488,  46024, 2504,
                24292, 8512,  57348, 14732, 28664, 58808, 19008, 6920,  48752,
                62088, 38620, 56636, 28732, 13960, 36712, 48268, 22672, 39212,
                5456,  14092, 47564, 9356,  28492, 46252, 57400, 4100,  36660,
                43632, 62248, 54148, 46812, 32644, 63040, 7000,  35444, 22612,
                15692, 29668, 24604, 55944, 14236, 59168, 20792, 11600, 24704,
                57836, 14624, 21800, 10412, 4900,  30212, 12712, 49672, 53136,
                39480, 14656, 48464, 60872, 26736, 37720, 55116, 33172, 3492,
                44000, 14932, 56120, 60292, 48780, 11412, 39872, 58860, 18796,
                7832,  46900, 41080, 31192, 17500, 55688, 27400, 47328, 51652,
                61416, 57232, 18464, 11664, 1520,  50728, 16936, 61672, 38908,
                52792, 19996,
            },
            {
                6792,  11528, 63480, 42480, 49096, 13464, 47052, 23992, 11416,
                35872, 45776, 3176,  24980, 59988, 44836, 62868, 38304, 57088,
                320,   59400, 13200, 4260,  55456, 32736, 1120,  22692, 35688,
                28100, 12676, 17924, 1036,  36780, 19344, 50612, 63360, 55900,
                33812, 48300, 18248, 65248, 34644, 4196,  41756, 36812, 23796,
                16436, 34188, 20292, 6528,  42412, 18240, 1208,  62180, 55920,
                32516, 26020, 17692, 58612, 35476, 54576, 8008,  31344, 50356,
                14028, 9368,  4956,  21508, 38476, 26720, 9976,  18636, 46372,
                38252, 51512, 10320, 35332, 48556, 39196, 4188,  34744, 30852,
                6976,  43876, 51360, 38724, 54784, 18500, 63340, 25088, 6496,
                58304, 21872, 34200, 52092, 5508,  63800, 14420, 21588, 39824,
                26120, 35384, 19464, 4820,  28088, 63420, 3348,  54120, 36084,
                26752, 52108, 13708, 63168, 8112,  42392, 10788, 21248, 16448,
                3776,  23732, 45768, 63672, 37820, 22468, 31024, 45656, 4772,
                23492, 60324,
            },
            {
                50200, 29340, 24868, 1784,  57420, 29924, 61776, 3936,  50880,
                27944, 64168, 20960, 55624, 12484, 29692, 4748,  26332, 50108,
                35816, 17632, 51328, 30244, 43796, 58580, 38760, 46256, 54364,
                65364, 33272, 48392, 62472, 54032, 4492,  31896, 27116, 16900,
                6072,  29708, 42376, 52984, 10260, 49264, 63060, 31196, 55872,
                9328,  43512, 63396, 51288, 25464, 58932, 45940, 12520, 20116,
                50020, 3024,  61384, 43568, 1532,  13176, 22872, 18088, 65188,
                25000, 34736, 41868, 57760, 12448, 49312, 43376, 54936, 58948,
                208,   62320, 44392, 19692, 2492,  23980, 52484, 45284, 61128,
                17800, 63740, 28940, 24252, 60348, 33936, 40988, 46604, 28304,
                17580, 64980, 3976,  11720, 23552, 45452, 29344, 9896,  62396,
                47360, 7436,  53884, 38624, 43384, 21524, 15264, 45104, 10620,
                60900, 21032, 340,   25320, 50120, 4152,  59440, 32892, 43404,
                38436, 7760,  27896, 14684, 42732, 54136, 7876,  11212, 56316,
                41620, 32428,
            },
            {
                44520, 38248, 53568, 22312, 34932, 6992,  39672, 18324, 42704,
                14612, 53384, 5912,  41556, 35236, 51468, 19272, 43044, 7336,
                31740, 23100, 63604, 9440,  19116, 25232, 13852, 7576,  20168,
                3812,  24328, 41424, 8696,  22936, 58128, 14272, 43924, 39680,
                60964, 12428, 37140, 2232,  22532, 25868, 12596, 45556, 532,
                27300, 49568, 3592,  31912, 10752, 54848, 34040, 27888, 8232,
                42044, 53336, 38052, 24200, 33296, 48356, 59820, 37432, 45192,
                1824,  52052, 29024, 17848, 2328,  64716, 23676, 5060,  13632,
                33556, 26384, 6604,  54676, 60708, 29992, 15972, 9852,  49792,
                37828, 780,   12936, 48420, 3784,  7896,  15364, 56896, 1444,
                37980, 44564, 29864, 40608, 18712, 57560, 2884,  50444, 17640,
                58972, 12680, 31524, 50196, 9132,  34792, 58072, 30176, 6096,
                42224, 33016, 38612, 57848, 46200, 29656, 36948, 13788, 64992,
                49804, 55024, 35172, 59788, 4964,  25620, 62620, 48776, 27716,
                17508, 160,
            },
            {
                62848, 19180, 9136,  60184, 16380, 45904, 25796, 33236, 58936,
                9372,  37948, 48732, 17420, 58148, 9152,  64520, 15164, 55004,
                12064, 46724, 41548, 37212, 2332,  62304, 48804, 30476, 42544,
                58252, 50552, 15188, 29552, 46232, 38384, 10400, 21560, 88,
                50312, 56860, 19924, 45008, 60248, 39428, 17124, 52648, 60996,
                21564, 14432, 59704, 38116, 16640, 40352, 4800,  23192, 64044,
                30920, 15100, 10716, 19840, 63592, 6040,  40588, 28076, 10896,
                56028, 15524, 47264, 60104, 36392, 31548, 40560, 28464, 52220,
                21016, 41632, 17064, 32452, 12612, 43056, 56656, 5424,  21544,
                26144, 55864, 35376, 19904, 52784, 43508, 22252, 11496, 32712,
                54408, 8576,  49236, 60040, 35568, 53248, 41884, 32488, 37164,
                1076,  23136, 65028, 16340, 25740, 1420,  51272, 23764, 48044,
                64688, 15664, 53144, 11672, 18176, 22360, 53956, 1248,  25792,
                19192, 30740, 2284,  47480, 18072, 33196, 40128, 20508, 36288,
                59192, 14140,
            },
            {
                34536, 47084, 4732,  41436, 51280, 10912, 65060, 1184,  55188,
                30820, 21968, 26920, 656,   32204, 23736, 48092, 39376, 28140,
                60612, 4576,  56740, 27164, 51896, 34572, 10508, 60604, 17056,
                37452, 11048, 35268, 2164,  60196, 51504, 34012, 64668, 45812,
                23968, 8248,  32352, 27504, 54816, 5808,  33840, 8032,  41072,
                36404, 30280, 46908, 24104, 7624,  52044, 43840, 59524, 35596,
                47096, 3980,  57780, 29732, 51092, 16192, 53984, 4232,  21028,
                61052, 33092, 6204,  22644, 53560, 7892,  15248, 61396, 36864,
                9516,  50080, 63892, 47200, 37408, 27044, 64968, 40412, 32648,
                59456, 42896, 8920,  30800, 64372, 27096, 36800, 61532, 50648,
                24508, 20180, 13744, 496,   26400, 15468, 7572,  24696, 54928,
                43860, 28760, 39504, 46608, 61168, 54628, 38212, 12948, 18620,
                2544,  28352, 44668, 4700,  34532, 60692, 40776, 48212, 9808,
                58452, 12456, 22072, 52284, 9320,  56728, 13092, 3452,  53184,
                7352,  25440,
            },
            {
                57076, 12316, 27320, 31504, 56344, 36752, 20132, 48160, 6308,
                43940, 60352, 12160, 62348, 45356, 4292,  36116, 20672, 1336,
                33064, 16972, 21472, 7040,  15388, 44920, 22276, 55600, 360,
                27032, 53312, 64016, 20704, 26220, 17668, 5260,  55328, 28200,
                35788, 16176, 63384, 4072,  14500, 50240, 28708, 19544, 56716,
                3016,  11392, 55096, 19112, 65388, 29240, 124,   13308, 17780,
                55136, 26464, 39788, 46020, 8840,  25360, 32016, 44260, 49616,
                38604, 26040, 41132, 11464, 44704, 19472, 56612, 47380, 2600,
                58048, 29204, 4280,  22428, 8588,  1452,  18412, 47896, 12096,
                3092,  16636, 47024, 39720, 1632,  57604, 47664, 5748,  16592,
                42080, 62840, 56348, 32232, 47580, 64544, 11084, 61312, 20320,
                4656,  57648, 6668,  11400, 19704, 4972,  32244, 60060, 40244,
                56504, 24440, 37240, 62456, 8292,  27136, 15816, 5952,  33596,
                44792, 39968, 63376, 36732, 42300, 28960, 46504, 65080, 31296,
                42888, 50616,
            },
            {
                3216,  38884, 63648, 21300, 2336,  13728, 29136, 40676, 24592,
                15192, 35048, 52496, 39852, 29000, 56192, 11728, 61636, 52280,
                44168, 49368, 40088, 65000, 54288, 28704, 4848,  39064, 32608,
                47444, 6556,  44400, 55756, 8036,  42788, 30876, 13900, 3120,
                53404, 41464, 47524, 57960, 37792, 43156, 64792, 46684, 24956,
                62328, 50820, 5412,  34616, 45216, 57188, 21196, 37584, 50724,
                7200,  22036, 62728, 828,   36352, 56584, 13004, 64124, 18444,
                9300,  364,   62592, 49028, 27996, 3708,  33896, 25100, 12260,
                44492, 16140, 39336, 53952, 59712, 50796, 35632, 24424, 54340,
                29416, 51656, 62324, 13872, 24752, 17920, 9656,  29528, 34868,
                2776,  28028, 37124, 5292,  21364, 38856, 30636, 45616, 49120,
                13956, 33676, 51688, 42180, 36332, 27872, 44996, 8192,  49536,
                31308, 10040, 54316, 20112, 50340, 42768, 55928, 62964, 29128,
                51308, 3952,  25084, 15108, 60580, 776,   23360, 38372, 16776,
                10172, 22256,
            },
            {
                33364, 54280, 15988, 48984, 44800, 61072, 52748, 8848,  63144,
                49936, 2924,  19780, 8132,  16008, 50844, 42084, 7568,  25596,
                59012, 9820,  35660, 31008, 12376, 42280, 49220, 63444, 19756,
                13704, 23712, 30624, 39156, 12280, 49740, 62272, 39932, 20444,
                60340, 11024, 25408, 19300, 9148,  23076, 1940,  12232, 32860,
                16240, 42700, 38808, 26780, 14248, 9336,  48880, 31804, 60772,
                11096, 44176, 33372, 17304, 60124, 41528, 3056,  23748, 35440,
                58264, 30932, 54276, 13768, 37364, 64160, 51016, 42684, 21780,
                63132, 35020, 25592, 11640, 31164, 15184, 43936, 6596,  60728,
                38220, 19720, 4420,  34276, 54800, 44748, 38032, 52372, 58740,
                12048, 45276, 53120, 9804,  42880, 51780, 17520, 2296,  35876,
                26580, 63224, 17152, 22716, 56452, 63916, 15720, 21808, 3876,
                63012, 14340, 47336, 1556,  31944, 13212, 23808, 37992, 20456,
                16952, 55348, 7580,  32292, 50564, 19308, 6240,  58420, 48564,
                28372, 61424,
            },
            {
                18692, 41808, 7716,  25220, 35148, 5324,  23292, 17400, 37296,
                30080, 46148, 57272, 25992, 65304, 22740, 33692, 18172, 30384,
                14632, 2384,  23976, 58056, 3396,  17708, 25628, 8272,  57456,
                41320, 60004, 16472, 3264,  34828, 58356, 23444, 46932, 6712,
                50980, 29652, 1096,  34468, 52376, 31096, 59672, 36848, 53660,
                8400,  21824, 59336, 1724,  62568, 40604, 25096, 2888,  41772,
                28768, 53744, 13700, 50160, 21120, 29044, 47636, 6996,  50968,
                15948, 43320, 20340, 24060, 57876, 9612,  16620, 30340, 7404,
                53012, 1840,  46132, 57156, 41416, 20848, 62760, 33432, 10164,
                22972, 45136, 27448, 59060, 11608, 21816, 4716,  65356, 19424,
                48728, 23320, 16268, 59396, 25612, 57112, 6620,  60424, 40420,
                53784, 8456,  30404, 47764, 292,   10200, 34576, 52700, 26928,
                42812, 35292, 22988, 39908, 59228, 5184,  46760, 10644, 844,
                61280, 35220, 47108, 11448, 27652, 44736, 54228, 32820, 13472,
                1528,  45648,
            },
            {
                5572,  29856, 51944, 10616, 57556, 31140, 43636, 59364, 54296,
                4836,  11228, 32932, 42956, 2368,  37648, 5864,  46612, 63368,
                38500, 53368, 19488, 45436, 61192, 37532, 51188, 11676, 35808,
                2288,  52360, 48372, 63096, 27852, 18664, 9364,  37036, 32896,
                15324, 44072, 38516, 61912, 13920, 48544, 17704, 5284,  44796,
                27692, 49452, 31468, 17616, 52568, 33656, 55508, 18976, 63536,
                6484,  23616, 38484, 4444,  9716,  54916, 32592, 61252, 39260,
                27736, 2872,  46876, 5464,  32372, 40504, 1292,  55184, 38492,
                19152, 32436, 61948, 7976,  4532,  27720, 52064, 576,   56032,
                14852, 49716, 7904,  42184, 31108, 50224, 40140, 25936, 32664,
                7664,  40532, 63708, 1756,  33716, 13588, 29336, 23724, 11752,
                19636, 43328, 4160,  59184, 25136, 40700, 55408, 46496, 18920,
                59720, 6816,  57040, 16456, 28112, 65284, 34428, 52016, 44036,
                26972, 40928, 21972, 64548, 57544, 39416, 9600,  25676, 63816,
                36240, 55852,
            },
            {
                60672, 14280, 37472, 64820, 19016, 39976, 312,   12364, 27248,
                21168, 39136, 60680, 53316, 13860, 48664, 58388, 54820, 12172,
                42932, 27740, 48812, 7972,  32744, 22508, 55276, 30036, 45536,
                26488, 33012, 21572, 5320,  45164, 53116, 568,   26640, 56000,
                65440, 22440, 54736, 7320,  40992, 55636, 25756, 39460, 63412,
                57572, 12620, 6552,  46716, 22528, 5304,  11860, 44632, 15620,
                35364, 57068, 45624, 65180, 26608, 43500, 14752, 19700, 10756,
                55936, 64912, 35740, 52584, 18224, 61904, 48440, 26320, 59608,
                13328, 50556, 23772, 17824, 48876, 37520, 12776, 46356, 30020,
                39284, 64324, 35668, 1852,  62684, 15768, 56152, 120,   14188,
                54672, 36224, 30268, 20348, 47816, 37864, 44692, 55428, 49812,
                61848, 32588, 15128, 37600, 50068, 13336, 30964, 2996,  38676,
                11420, 30152, 50756, 44764, 9420,  53584, 18528, 30616, 57312,
                13984, 49952, 5564,  16000, 1916,  34796, 17524, 43168, 51152,
                20484, 24500,
            },
            {
                43992, 48452, 2124,  26744, 46160, 14916, 51324, 34240, 47884,
                63844, 16480, 23952, 9396,  29772, 19372, 26288, 108,   21996,
                34628, 4632,  56228, 40552, 15096, 1672,  62204, 5920,  16880,
                64640, 9008,  13036, 38448, 56676, 14984, 41968, 59804, 12036,
                3352,  17964, 46432, 28152, 2604,  21288, 10892, 30504, 344,
                19676, 34488, 41500, 60476, 36832, 29468, 48264, 58276, 27072,
                51760, 1832,  31228, 17916, 58180, 37672, 1012,  51912, 25568,
                41832, 13248, 8156,  28956, 44100, 10864, 22264, 36024, 4136,
                44676, 9720,  40432, 30184, 55152, 64764, 24836, 18940, 58468,
                4876,  17732, 53196, 25652, 19992, 9256,  46108, 29036, 60272,
                43456, 5344,  11532, 51012, 61228, 8896,  18288, 904,   35128,
                5724,  26188, 45412, 65428, 20468, 7084,  58000, 16800, 24456,
                64464, 876,   20336, 37320, 25380, 2276,  39272, 22540, 3808,
                8816,  59384, 38292, 29728, 53488, 23032, 61552, 3840,  8028,
                39764, 11224,
            },
            {
                53604, 32040, 20892, 55128, 7500,  62652, 23000, 56368, 7808,
                42164, 1428,  50788, 45000, 35620, 62296, 41008, 32164, 9272,
                59876, 17444, 64704, 11088, 26156, 47632, 42904, 20576, 39812,
                49248, 57892, 43072, 29260, 33848, 22836, 47880, 30252, 40324,
                49144, 26100, 9296,  32684, 57328, 64380, 42916, 47412, 14972,
                51240, 24600, 55396, 10060, 15960, 64732, 38704, 3760,  20612,
                40392, 10288, 49272, 22084, 7364,  34808, 47300, 63256, 5796,
                33392, 21912, 49892, 59344, 24772, 56628, 14860, 42388, 64080,
                27788, 58140, 35548, 14268, 3360,  43348, 9040,  35072, 41644,
                23540, 11108, 32316, 47428, 57660, 37148, 22436, 34584, 49636,
                17904, 24576, 58076, 41264, 4176,  26468, 64720, 39800, 21736,
                57592, 10612, 52260, 2552,  34044, 28284, 61296, 42052, 51624,
                47452, 33480, 55340, 13120, 59932, 49048, 15220, 42656, 61916,
                25504, 32912, 19480, 46060, 42092, 12656, 49012, 31384, 58696,
                28880, 16532,
            },
            {
                35056, 4344,  60144, 42396, 35612, 28408, 4584,  31636, 19468,
                36652, 57884, 27936, 6412,  54520, 3680,  16360, 52004, 45124,
                49924, 24220, 36300, 30744, 52448, 59132, 34004, 13348, 54780,
                24196, 1084,  19264, 61388, 6824,  10352, 63644, 5936,  20720,
                34384, 52088, 62352, 37528, 15800, 49676, 35196, 8452,  61360,
                37976, 4204,  44276, 27968, 1136,  53228, 24100, 8860,  62056,
                33792, 14580, 60432, 42428, 12640, 54000, 23236, 30108, 44516,
                60404, 37912, 15756, 688,   39324, 4868,  34072, 7676,  51684,
                19936, 180,   47988, 61224, 21884, 32568, 59676, 2100,  53668,
                49172, 60856, 44172, 14004, 5712,  61572, 41936, 3544,  8480,
                62520, 46568, 28560, 15964, 54964, 32000, 46360, 14980, 30644,
                47672, 41256, 17028, 23416, 54712, 44224, 9588,  36368, 4672,
                14436, 27340, 40724, 8080,  30936, 63236, 6432,  54324, 36528,
                47376, 52576, 10820, 62748, 4360,  26520, 55044, 37160, 46480,
                484,   63284,
            },
            {
                41128, 8556,  23704, 12600, 17204, 50176, 39492, 61172, 10044,
                46676, 13124, 32552, 21720, 60108, 12308, 37732, 64100, 28820,
                13656, 2900,  43960, 6104,  18484, 38092, 9400,  28356, 4952,
                31448, 36988, 53224, 46616, 25728, 50836, 17804, 36476, 54540,
                14576, 4236,  42444, 11876, 23312, 5388,  19132, 27604, 54380,
                22032, 31180, 63172, 18368, 49976, 32952, 13272, 42944, 30064,
                46680, 55256, 25124, 28816, 3568,  59520, 16648, 10112, 2404,
                18888, 54256, 46364, 31776, 63292, 53428, 47080, 29480, 16252,
                33164, 25064, 53868, 6136,  17068, 51508, 46692, 26596, 15708,
                7712,  28040, 2524,  40080, 30088, 50956, 12132, 27336, 52660,
                20912, 38260, 1048,  35316, 22060, 12356, 53272, 4912,  59964,
                7988,  28636, 62468, 38236, 12408, 48336, 19164, 31908, 22624,
                57444, 62376, 3772,  43596, 21692, 46156, 33708, 11948, 28796,
                17176, 408,   23664, 56888, 33688, 8636,  19056, 14836, 22224,
                56364, 26148,
            },
            {
                18400, 52368, 29636, 47968, 58132, 1548,  44352, 14720, 54896,
                25356, 64448, 40256, 18152, 48412, 43656, 25072, 7252,  20620,
                57852, 39280, 54556, 62556, 22896, 528,   50248, 63932, 45348,
                59568, 11312, 16412, 2960,  39548, 58112, 43604, 1880,  61436,
                24288, 57932, 29628, 45424, 53728, 31452, 59380, 3160,  44932,
                11652, 57920, 7532,  40648, 21320, 45548, 59144, 54420, 18220,
                6864,  376,   64388, 39112, 45752, 32752, 50748, 40524, 57436,
                28552, 11520, 7052,  26576, 18100, 12688, 21484, 60504, 56372,
                10960, 45700, 37940, 42176, 28728, 10588, 39380, 20104, 62380,
                38044, 34348, 56252, 19064, 24444, 55144, 17240, 64396, 32928,
                14020, 56932, 10080, 49324, 59000, 42360, 36768, 24780, 50852,
                19812, 35784, 56432, 6312,  26948, 1612,  63972, 52544, 7644,
                44968, 18184, 35136, 52976, 16172, 26960, 57148, 19944, 4888,
                64680, 44120, 37832, 15616, 50084, 40288, 65204, 44600, 5516,
                10244, 49600,
            },
            {
                13848, 44448, 61372, 37440, 10500, 25748, 53028, 21384, 30448,
                2588,  51564, 5472,  34664, 8692,  30600, 55988, 1508,  47780,
                33028, 8756,  27292, 14900, 45880, 57364, 25552, 15788, 39384,
                51316, 22392, 34456, 65100, 13600, 28588, 31952, 11612, 46208,
                39264, 8544,  18204, 640,   60992, 38656, 51648, 41412, 17256,
                34052, 48156, 14504, 36380, 60816, 2828,  5960,  26412, 37544,
                51900, 21244, 35368, 15696, 8348,  20072, 62820, 36956, 24504,
                48628, 64532, 41956, 36088, 50124, 43432, 3112,  37512, 41084,
                5056,  65456, 13240, 23408, 62028, 55452, 5204,  31216, 57100,
                12212, 22044, 65168, 48032, 9432,  35992, 1280,  45636, 39900,
                4336,  43268, 30148, 63744, 7396,  18972, 2084,  62936, 11516,
                43968, 424,   14548, 49996, 60508, 39616, 15580, 25684, 37628,
                11816, 49696, 24340, 58572, 10124, 2908,  38192, 41944, 59620,
                49352, 31920, 7020,  29460, 58980, 1624,  28156, 35164, 60012,
                31120, 38156,
            },
            {
                54200, 1228,  6252,  32332, 18992, 65472, 34328, 7060,  37924,
                16628, 43388, 61652, 24148, 59036, 52612, 17428, 36584, 42060,
                60980, 18952, 48960, 35320, 11872, 41028, 32084, 19772, 8296,
                3440,  27152, 56008, 42664, 20252, 7256,  52740, 21792, 27040,
                33140, 50412, 64808, 21240, 26588, 13784, 9872,  24464, 62640,
                924,   26980, 52964, 23776, 10880, 29076, 34672, 15568, 62740,
                41920, 12004, 48324, 58480, 53528, 27004, 5688,  13640, 3668,
                34124, 21284, 56536, 2116,  23668, 62128, 9952,  26908, 49252,
                18504, 30412, 57720, 7816,  34128, 49104, 14472, 43236, 832,
                45784, 52300, 16112, 3672,  43572, 59756, 30768, 22728, 58336,
                25832, 54412, 17024, 23404, 33260, 45408, 27900, 39372, 31212,
                54404, 23120, 33496, 42612, 30512, 21580, 45832, 59308, 54932,
                29080, 72,    39640, 31972, 64060, 47756, 51728, 22844, 9140,
                25256, 13836, 55296, 20192, 43260, 22984, 53380, 12468, 47508,
                24168, 63960,
            },
            {
                34296, 27772, 21732, 55400, 46592, 4060,  40848, 59916, 49624,
                56704, 27532, 11712, 548,   41228, 13528, 27408, 63092, 10924,
                23788, 53616, 2192,  30012, 55744, 6848,  61048, 36128, 43984,
                62228, 30704, 48716, 5084,  37680, 47232, 62452, 56768, 16384,
                4764,  55372, 35720, 41952, 47552, 7248,  36172, 49124, 30196,
                56564, 39148, 6468,  65084, 42540, 51208, 46920, 57488, 23020,
                8716,  32212, 24628, 2540,  30724, 42828, 49408, 55888, 45116,
                16824, 8240,  52132, 14220, 31092, 57824, 34264, 54512, 23068,
                1104,  36104, 50760, 20756, 2364,  25880, 36608, 63856, 23868,
                29184, 6436,  32716, 39220, 26892, 50352, 14496, 6760,  48856,
                11272, 35764, 47012, 3044,  52620, 60428, 8996,  56752, 16348,
                48196, 64948, 8752,  17928, 57620, 10460, 3560,  33120, 6020,
                42836, 60356, 14760, 6836,  19340, 29584, 13232, 1172,  34944,
                53308, 39448, 46856, 10572, 61588, 5100,  17112, 38976, 2876,
                20052, 7540,
            },
            {
                16600, 40096, 62636, 14492, 42640, 28920, 12416, 23456, 8968,
                32196, 20148, 46992, 36376, 50136, 21324, 3884,  45948, 31268,
                15704, 5276,  43536, 65476, 21132, 51420, 2664,  48100, 13780,
                53800, 10596, 17424, 60216, 24632, 9884,  34712, 1164,  40872,
                48180, 11112, 15056, 5740,  32536, 54532, 58768, 15488, 5272,
                20800, 45468, 17828, 31792, 55716, 19748, 12992, 39048, 1964,
                49628, 56232, 44780, 62220, 18196, 10376, 21920, 31560, 60088,
                41184, 29312, 60912, 39008, 5692,  46152, 16376, 6896,  62780,
                43892, 53704, 15532, 41332, 45956, 59312, 17356, 53456, 10072,
                50040, 41852, 61728, 11840, 54560, 20476, 62828, 38104, 18452,
                61040, 6288,  28280, 41108, 15380, 12084, 49660, 21888, 3272,
                36468, 26168, 46316, 5044,  53168, 36064, 51104, 13536, 19836,
                65196, 22344, 46848, 55648, 36244, 44148, 56592, 62216, 28136,
                17852, 63440, 6340,  36000, 26268, 48560, 32444, 57988, 51168,
                42988, 56840,
            },
            {
                46996, 50256, 24572, 9184,  51844, 57740, 17788, 47956, 63080,
                3100,  39736, 54952, 16272, 64952, 32396, 58348, 6940,  51816,
                40240, 60284, 37012, 24832, 9564,  16700, 28068, 58548, 21684,
                25156, 41204, 96,    32784, 51192, 14416, 43820, 19120, 59888,
                30420, 23044, 58100, 28648, 61876, 18404, 23424, 43648, 63768,
                34392, 12092, 49268, 1592,  9164,  25720, 5524,  64108, 30104,
                17624, 36936, 14184, 6052,  40144, 35804, 65332, 2272,  25468,
                11156, 47368, 18612, 27104, 50992, 20352, 41684, 29264, 12624,
                32808, 8948,  27384, 63500, 11796, 32088, 3716,  40284, 20536,
                35212, 57416, 17684, 24052, 5512,  35040, 8952,  42536, 32128,
                51632, 21452, 65336, 56448, 37392, 25584, 34260, 42792, 6128,
                60892, 13160, 40200, 29396, 63036, 24812, 41472, 27880, 48932,
                34852, 8564,  26856, 51448, 3728,  24840, 16764, 39996, 46100,
                4516,  21660, 58184, 31100, 12924, 41244, 64292, 8844,  25424,
                29936, 11324,
            },
            {
                5892,  59404, 2724,  38588, 33888, 300,   26692, 36772, 44228,
                13604, 58920, 6716,  29180, 9840,  43996, 38356, 55208, 26036,
                20016, 49780, 13208, 33084, 46872, 38936, 42756, 31748, 5632,
                35256, 63772, 45476, 20984, 55196, 29804, 64508, 26112, 9012,
                37188, 43360, 51488, 1312,  45336, 38800, 4108,  10336, 40508,
                25292, 54088, 60496, 37192, 41232, 58256, 34008, 44560, 53180,
                26336, 3740,  60744, 28524, 47456, 54384, 14936, 50460, 38480,
                5148,  55140, 35392, 584,   64432, 10676, 36628, 59648, 48080,
                19140, 58384, 37684, 23512, 6752,  48332, 28436, 56476, 7612,
                13492, 27428, 2152,  59208, 46668, 29504, 52832, 256,   25444,
                44652, 14136, 4676,  31380, 848,   62048, 17648, 58316, 30308,
                52048, 18860, 56040, 2432,  20856, 15884, 60192, 1332,  54392,
                17048, 38672, 42208, 12068, 32864, 60904, 7424,  10940, 50288,
                33468, 43708, 2452,  50776, 56004, 19160, 100,   37108, 15512,
                61720, 35828,
            },
            {
                31620, 18676, 44012, 29576, 20688, 64028, 54500, 5404,  29988,
                22168, 50696, 25272, 35436, 2112,  23340, 18188, 12772, 35068,
                8720,  28416, 57380, 684,   53916, 63216, 12196, 55436, 18568,
                51908, 37984, 12888, 59220, 7056,  39652, 2736,  49700, 53908,
                3676,  63128, 19916, 25540, 12336, 53108, 31372, 50364, 57728,
                2904,  29368, 14764, 22096, 28260, 50624, 16120, 21048, 10024,
                58896, 42928, 51772, 20128, 33156, 24172, 8144,  58872, 19732,
                32328, 62992, 13456, 44524, 24808, 55820, 2964,  52012, 25416,
                5136,  45360, 2064,  55956, 51720, 19420, 61592, 37288, 45280,
                64672, 48676, 31640, 43316, 38564, 64276, 16740, 56776, 10484,
                59528, 38668, 49348, 19708, 44116, 51764, 7724,  46976, 23696,
                38284, 9608,  33952, 43240, 48344, 8092,  37284, 44984, 11092,
                30900, 57940, 1968,  63364, 21116, 37664, 53480, 30248, 59040,
                14300, 26688, 38364, 16280, 24068, 44504, 28952, 54056, 46276,
                3500,  22400,
            },
            {
                65008, 53472, 13652, 58352, 49196, 11452, 15860, 40912, 60140,
                33444, 18044, 45496, 62616, 53492, 48788, 61420, 3124,  47048,
                64120, 4488,  44444, 18436, 22524, 7332,  26072, 1684,  61632,
                9224,  29072, 3972,  27240, 47572, 16820, 35696, 22064, 13564,
                33200, 16520, 39752, 8180,  34888, 65380, 27024, 20496, 16728,
                47044, 7648,  43752, 63372, 4928,  46248, 61064, 888,   40628,
                35120, 22704, 7244,  12116, 63300, 112,   45492, 28108, 43396,
                52412, 22884, 7948,  49280, 31416, 39624, 21892, 14928, 34376,
                64932, 28948, 40660, 13136, 34780, 42796, 15296, 25016, 448,
                22300, 53788, 9496,  19888, 3292,  13752, 23284, 47432, 34504,
                28892, 7464,  24224, 55448, 10732, 27796, 39632, 13904, 2172,
                64104, 26884, 50396, 59688, 12608, 31556, 55460, 22128, 61884,
                5928,  52384, 24204, 15072, 49064, 43412, 18596, 1728,  23260,
                65400, 7828,  52900, 61024, 9980,  5704,  59956, 20880, 12176,
                52212, 41912,
            },
            {
                10000, 25908, 4272,  36048, 7448,  45724, 24796, 52248, 9792,
                1176,  55660, 11844, 41800, 14660, 27676, 40528, 30532, 56396,
                23156, 41400, 31356, 59072, 37748, 50328, 34836, 45108, 40396,
                48620, 15592, 56680, 41732, 23880, 10708, 61700, 45060, 57428,
                27524, 48720, 60488, 46288, 55468, 14864, 1748,  37768, 62100,
                32416, 36144, 52880, 18720, 11320, 32260, 24484, 13440, 54720,
                31080, 15408, 48756, 38164, 56328, 41012, 17348, 11848, 36580,
                1948,  15828, 42072, 57792, 17476, 6348,  61684, 42720, 9712,
                54372, 17008, 21432, 60156, 30592, 9456,  4860,  52292, 33464,
                41168, 15896, 36900, 56188, 50492, 27576, 41476, 4768,  18336,
                54040, 63056, 41648, 16400, 35488, 63528, 21272, 33052, 53772,
                44724, 15732, 6532,  23184, 39320, 64756, 4340,  26052, 34508,
                19364, 44088, 29108, 59260, 4496,  27764, 57324, 46928, 40740,
                35636, 20144, 46296, 29904, 34636, 48892, 39760, 32584, 57844,
                27404, 38184,
            },
            {
                15824, 47712, 60888, 22148, 55844, 31744, 38288, 19960, 48116,
                64692, 37044, 4596,  31752, 7844,  20796, 58748, 10632, 16976,
                36692, 13976, 51884, 10052, 15796, 29600, 60576, 13380, 23548,
                32988, 20668, 64260, 34604, 50520, 54640, 31052, 4804,  41512,
                6952,  11232, 29968, 4476,  22420, 41300, 48604, 8764,  55160,
                11968, 22952, 59084, 3540,  39500, 56272, 36540, 48040, 6100,
                62720, 45184, 59896, 25128, 4832,  30164, 49140, 62064, 57064,
                25632, 61088, 29228, 34984, 11732, 50432, 27868, 46756, 972,
                38012, 49912, 6148,  47668, 26092, 62872, 46128, 57304, 11980,
                61176, 6228,  25228, 62516, 11624, 32348, 58440, 61996, 37136,
                13008, 2104,  31872, 45888, 3248,  56852, 48428, 5700,  60384,
                19656, 32060, 57532, 1544,  28172, 17840, 46884, 51392, 41420,
                12860, 47908, 8228,  40016, 35512, 9968,  13520, 31980, 6188,
                54524, 11316, 41916, 1224,  63952, 14080, 25336, 2696,  17504,
                7416,  49960,
            },
            {
                1864,  33528, 40344, 28316, 42856, 2456,  62524, 6180,  28144,
                42676, 21708, 26396, 57516, 50448, 37872, 188,   44300, 52724,
                6548,  48388, 25736, 63720, 4024,  47196, 19428, 5112,  58164,
                51100, 2136,  44308, 7668,  18704, 760,   38496, 20300, 52436,
                64612, 23556, 36060, 57200, 18304, 32956, 60032, 28744, 44152,
                2472,  41004, 26584, 30564, 51244, 8360,  65244, 27120, 19832,
                2428,  28752, 9640,  18532, 34196, 53724, 21976, 9056,  32800,
                5868,  48148, 38736, 3488,  64112, 19744, 33212, 58084, 23368,
                31288, 11288, 55672, 35824, 1620,  16672, 39044, 20388, 27904,
                44264, 30508, 47500, 4168,  35240, 45376, 8256,  20836, 51248,
                26424, 48744, 22108, 60812, 8420,  25312, 12264, 41792, 28276,
                37248, 10728, 47596, 41212, 53992, 35520, 9016,  15208, 388,
                56356, 64416, 32064, 17148, 54312, 63668, 25080, 50544, 62104,
                16572, 27540, 57228, 22764, 18348, 54808, 42912, 62948, 45600,
                35132, 61852,
            },
            {
                21044, 54436, 18192, 8416,  14192, 59304, 17528, 35024, 12716,
                53780, 15588, 47164, 61336, 17660, 33772, 65224, 24992, 32440,
                19628, 61820, 1776,  33572, 39152, 53276, 42840, 28236, 8620,
                38444, 30316, 12140, 25800, 62600, 14544, 59256, 26216, 34212,
                17216, 50092, 920,   43484, 51768, 6536,  12900, 24400, 52244,
                17104, 64184, 47924, 13964, 44684, 22580, 16356, 41496, 52640,
                37772, 56772, 42040, 51364, 13280, 64424, 3068,  39576, 45824,
                18740, 54984, 10404, 22092, 52472, 44640, 4688,  12916, 61260,
                41532, 19020, 63896, 43848, 22792, 53692, 31680, 49844, 2628,
                18052, 58232, 40040, 21596, 17040, 54664, 1072,  30332, 43672,
                9876,  39316, 53260, 14152, 29736, 38412, 52464, 17900, 32,
                56352, 24488, 63328, 13460, 20624, 58912, 30264, 62196, 39052,
                27128, 21020, 2748,  50920, 22348, 45056, 988,   19576, 38152,
                3412,  48376, 32804, 6560,  50664, 36592, 8392,  11592, 51988,
                23584, 29608,
            },
            {
                44424, 11880, 64536, 47088, 52756, 26484, 49940, 44584, 56588,
                31240, 1736,  39984, 6636,  23420, 12544, 46212, 4544,  54812,
                39768, 28012, 43756, 21616, 56384, 11308, 24652, 64996, 54188,
                17912, 60664, 55796, 36508, 45892, 29308, 48152, 8712,  12508,
                44840, 38872, 59100, 14156, 26532, 63944, 45508, 39240, 20924,
                35184, 7368,  56548, 19964, 236,   34484, 59424, 6624,  12476,
                32836, 21820, 3964,  26544, 36816, 44440, 16060, 24636, 51940,
                13496, 28052, 43548, 60208, 25288, 36516, 56164, 16388, 26428,
                52912, 3156,  28400, 14292, 58492, 6860,  10964, 65408, 36788,
                8604,  53432, 12960, 51068, 64512, 25624, 40424, 15036, 56912,
                5576,  65024, 35084, 19680, 58392, 43952, 34032, 64552, 49580,
                8264,  44216, 33420, 3236,  50500, 6608,  45268, 23792, 52968,
                33576, 10908, 37844, 60188, 5408,  41236, 34096, 58988, 29472,
                43428, 60684, 13060, 39664, 58924, 27236, 30980, 20020, 38784,
                56748, 5144,
            },
            {
                59836, 35960, 30836, 960,   37396, 20360, 9168,  4032,  23768,
                60760, 10664, 34704, 52668, 42288, 56080, 28684, 15544, 60592,
                9580,  13172, 58172, 7940,  17296, 31652, 460,   35756, 14224,
                47768, 3932,  42128, 22424, 53284, 2992,  40388, 56112, 61476,
                3736,  28004, 20952, 31932, 9784,  36564, 1504,  56920, 4840,
                61044, 31272, 10328, 37700, 62024, 53320, 29940, 45644, 49832,
                63788, 15584, 61016, 47476, 7160,  59016, 29556, 56412, 35524,
                63676, 2752,  32024, 15480, 400,   9264,  39544, 49032, 7128,
                35140, 46176, 8888,  32884, 38168, 42152, 47404, 26452, 22336,
                60444, 32904, 3192,  29156, 9428,  36860, 47752, 59684, 23384,
                32724, 17108, 792,   47372, 6656,  22892, 3792,  15136, 30628,
                21608, 53252, 16104, 38124, 25664, 42668, 11900, 17292, 3664,
                57956, 46292, 15600, 26024, 30236, 49288, 10812, 14620, 53192,
                21164, 9020,  24884, 45732, 16156, 5648,  47148, 60328, 588,
                14748, 41492,
            },
            {
                25116, 51216, 6828,  22864, 57624, 40880, 29388, 64152, 36964,
                49240, 20556, 26904, 63472, 2896,  9096,  38544, 49056, 22016,
                34824, 51224, 46632, 37080, 61452, 50528, 40900, 45740, 21124,
                33240, 27008, 6924,  15976, 10696, 33404, 18960, 23916, 31532,
                53924, 8124,  63008, 41104, 55268, 19000, 49756, 29208, 15496,
                46804, 25564, 50716, 43304, 27792, 5052,  10844, 19040, 24984,
                1488,  43888, 30348, 54796, 11436, 21204, 992,   42632, 7772,
                20520, 49864, 57152, 40944, 47504, 28268, 18416, 31720, 62996,
                21460, 57688, 50148, 24248, 61800, 19564, 1256,  56168, 14780,
                45204, 38380, 24000, 43540, 60716, 5300,  18968, 51692, 10888,
                45524, 28576, 41092, 60240, 11644, 54944, 27080, 46508, 58008,
                40052, 4296,  28440, 62420, 55700, 31856, 59808, 36556, 48996,
                6164,  41876, 63100, 55292, 8100,  18668, 56980, 26772, 6900,
                36424, 63068, 54608, 34764, 2632,  64852, 53208, 24580, 33908,
                49112, 9352,
            },
            {
                19156, 56248, 15740, 45456, 62428, 12144, 33056, 16460, 41592,
                7608,  45404, 16992, 30612, 50344, 19956, 62288, 31280, 1008,
                41672, 18476, 5496,  29720, 23352, 15012, 6156,  57668, 10224,
                52268, 63400, 38780, 58616, 50480, 65276, 6400,  42804, 14724,
                36716, 46192, 16536, 4760,  47176, 23564, 60232, 33616, 10980,
                40548, 54504, 2708,  14540, 23248, 40968, 55840, 35964, 58152,
                39276, 9536,  34980, 18080, 40132, 50816, 32940, 61840, 12136,
                38528, 24192, 6048,  34244, 65184, 53508, 59032, 43148, 1460,
                12180, 40404, 15940, 4944,  52468, 12568, 29088, 34192, 50812,
                7152,  17224, 57636, 49168, 14404, 33560, 27584, 2408,  36076,
                61664, 55788, 24760, 49856, 31400, 39620, 62596, 9464,  35972,
                12532, 59484, 48140, 18820, 9836,  984,   21644, 64780, 27800,
                19588, 31496, 22956, 13132, 44184, 35884, 65104, 47056, 40720,
                49664, 308,   18552, 28188, 41660, 21380, 12804, 43852, 17452,
                63752, 28620,
            },
            {
                46644, 39856, 34416, 27468, 4300,  51808, 55048, 2556,  57864,
                13616, 53396, 59372, 35932, 13072, 25528, 44124, 7296,  59548,
                53044, 26712, 64748, 3104,  43004, 53836, 26436, 62264, 29920,
                2716,  43640, 18496, 30672, 25436, 35844, 56988, 48848, 2132,
                51540, 22244, 58272, 34716, 52844, 13268, 7288,  42972, 65156,
                22216, 35656, 18804, 58944, 64364, 33188, 16692, 47324, 7300,
                28000, 51700, 23296, 5316,  65040, 26084, 48096, 17136, 28692,
                46640, 60864, 17656, 11300, 21152, 5068,  13732, 23604, 51752,
                36604, 27112, 64736, 30984, 45604, 36420, 60720, 4112,  41240,
                63464, 31064, 92,    20372, 39496, 54860, 62588, 42148, 13796,
                20932, 3732,  7928,  14640, 35680, 17960, 51176, 1188,  19912,
                43252, 23956, 5996,  35116, 40940, 47252, 52456, 13748, 39948,
                9268,  51088, 708,   53324, 39464, 2824,  31600, 4940,  16864,
                23240, 32068, 57984, 10108, 51128, 36992, 31536, 3868,  54236,
                38096, 6272,
            },
            {
                2688,  13040, 60380, 9520,  43204, 18556, 24904, 46736, 21832,
                28696, 564,   39328, 5028,  46660, 57260, 16720, 37300, 23800,
                14372, 33280, 55664, 11936, 48072, 34224, 18936, 37480, 13100,
                23780, 49680, 8448,  46940, 280,   12572, 21128, 27416, 62080,
                30140, 11764, 26008, 668,   30652, 38972, 26916, 18132, 2188,
                57900, 6736,  49396, 8992,  44960, 3928,  52340, 21652, 63152,
                13688, 59968, 46600, 56968, 14228, 35968, 9052,  53712, 4056,
                55768, 30784, 43356, 51400, 26600, 37536, 30540, 45968, 9812,
                56552, 19328, 2680,  42504, 8616,  16816, 53536, 21108, 25048,
                11680, 47004, 27252, 52160, 10660, 23252, 7080,  31460, 47288,
                53892, 37608, 43588, 64200, 22076, 5308,  44620, 25960, 54092,
                31808, 65424, 51744, 15000, 29516, 24700, 7504,  33760, 57252,
                45168, 59476, 34440, 28016, 61868, 20808, 24664, 59728, 11804,
                52636, 43152, 13916, 46144, 61888, 6592,  57104, 26872, 11044,
                58708, 32528,
            },
            {
                62956, 52712, 20064, 49432, 31684, 38464, 7148,  36260, 60408,
                33828, 43760, 24244, 10180, 64884, 32624, 2500,  54488, 48276,
                8856,  44648, 39296, 21252, 59300, 8724,  1268,  40496, 55748,
                60052, 34948, 14960, 55324, 61240, 40896, 45612, 16948, 8784,
                38824, 42140, 64376, 44804, 9556,  61516, 56064, 51184, 44624,
                28148, 31988, 39356, 24324, 30408, 12288, 38316, 1996,  31692,
                41448, 19816, 796,   31220, 43116, 3196,  59376, 24944, 40600,
                14464, 7460,  35748, 1676,  55280, 62256, 41316, 3768,  61116,
                33148, 47608, 54048, 22804, 59324, 27360, 48480, 38520, 8108,
                54480, 59276, 36688, 4664,  65092, 44988, 17416, 57748, 25884,
                9604,  16956, 27928, 52600, 58760, 10824, 30028, 61404, 15932,
                38732, 2648,  10876, 56928, 61236, 43584, 55176, 16624, 4064,
                25392, 12352, 17984, 7008,  49592, 14460, 45828, 54788, 34560,
                38568, 64244, 4328,  30144, 23524, 16772, 48504, 41040, 20312,
                50484, 23288,
            },
            {
                29896, 42012, 25816, 132,   57524, 13840, 65068, 50660, 11036,
                17604, 62768, 54116, 20820, 51380, 27364, 40120, 11812, 30000,
                63304, 1616,  17536, 25508, 31160, 63940, 50028, 45576, 21520,
                5728,  41924, 28364, 20136, 37120, 4920,  31232, 53212, 59320,
                5908,  19712, 54260, 17396, 49184, 21416, 5288,  36040, 15144,
                11836, 52460, 60784, 15632, 48340, 57376, 62148, 26032, 50144,
                34748, 10956, 53912, 27288, 38548, 21480, 45236, 18836, 33568,
                63596, 22788, 59448, 46332, 15076, 8284,  49456, 17948, 28572,
                14528, 7328,  38244, 12032, 33504, 908,   64188, 14456, 43208,
                32920, 18524, 13756, 42652, 33884, 28852, 50572, 884,   40036,
                63228, 49176, 33080, 1424,  40672, 47936, 34840, 56056, 7492,
                49532, 27412, 45596, 22208, 36252, 2184,  20000, 38660, 30656,
                62968, 41724, 47692, 37548, 56620, 9888,  42004, 28580, 1932,
                8428,  26660, 19336, 55120, 39368, 9260,  59588, 34256, 968,
                44612, 16132,
            },
            {
                57000, 4916,  35628, 61708, 22916, 44852, 29300, 4636,  26176,
                48588, 8308,  29828, 15176, 42200, 5956,  18876, 61352, 21640,
                35536, 50672, 57072, 6800,  42160, 13996, 28204, 16756, 10008,
                32264, 64740, 2516,  51904, 24308, 10544, 63928, 34676, 22688,
                47828, 32468, 3224,  36884, 14208, 41468, 32720, 24392, 47248,
                62944, 20640, 516,   42016, 27148, 19196, 8168,  43324, 15900,
                6520,  45620, 63968, 16832, 51000, 11904, 62244, 52532, 284,
                48740, 10192, 27908, 39556, 20060, 25284, 35012, 58636, 22324,
                43560, 62904, 25660, 50908, 56292, 19644, 45012, 4588,  29244,
                62020, 2568,  24384, 56640, 8332,  21176, 60288, 13076, 34692,
                22588, 5776,  57500, 12848, 25496, 18592, 3140,  23344, 13276,
                42404, 33448, 17184, 6988,  50636, 12892, 64156, 48004, 51928,
                21408, 1984,  61180, 31780, 26564, 5176,  63704, 17368, 50932,
                21876, 60596, 48808, 35704, 1584,  43476, 24932, 13948, 65516,
                37076, 8344,
            },
            {
                12320, 39216, 47236, 10428, 16680, 53600, 39732, 20524, 56300,
                41088, 3656,  37416, 58196, 33940, 47840, 56136, 43668, 4560,
                26844, 14952, 46564, 37268, 52364, 4784,  57576, 36636, 54252,
                25368, 46388, 12708, 58676, 42844, 49008, 15452, 1652,  43608,
                13684, 57184, 24008, 59732, 28988, 63716, 1716,  57672, 8924,
                37804, 4752,  33540, 55444, 6216,  36844, 53592, 22860, 60224,
                56124, 37796, 24660, 4432,  58660, 32560, 7136,  29168, 37236,
                43900, 17592, 50976, 32548, 64192, 11992, 52420, 544,   39404,
                55804, 5200,  17392, 30440, 40768, 9692,  35216, 57216, 22052,
                50204, 45780, 39184, 52852, 16232, 48232, 37448, 4996,  54960,
                43784, 30780, 20224, 38064, 44928, 60724, 51792, 37068, 63524,
                20616, 57632, 62416, 39920, 25488, 32216, 9348,  28060, 5964,
                35644, 10928, 23672, 15432, 53256, 20200, 39208, 30840, 57092,
                44384, 15304, 31968, 12100, 52320, 62188, 29544, 5048,  54836,
                27060, 48896,
            },
            {
                22496, 18388, 55612, 28296, 7488,  33720, 1588,  59760, 12876,
                32004, 45856, 61200, 1168,  12436, 23700, 8608,  37860, 52908,
                59252, 32412, 9572,  62000, 19228, 33640, 23832, 62548, 4008,
                51124, 38776, 17272, 33552, 6648,  29280, 57704, 26232, 37728,
                50764, 27912, 10568, 45628, 7192,  51796, 19136, 42772, 29860,
                53684, 23212, 45256, 63992, 11444, 31020, 46796, 13048, 28676,
                2844,  20768, 30456, 9220,  48368, 40164, 15888, 25780, 57712,
                13068, 55644, 6152,  3300,  57004, 45244, 6916,  31672, 48572,
                10916, 35416, 45944, 2072,  60524, 49436, 25112, 16644, 37560,
                10800, 7120,  27660, 59572, 1868,  31172, 41824, 25200, 62032,
                15536, 10532, 53624, 65004, 7748,  15092, 31312, 9796,  46472,
                5216,  29124, 328,   54572, 46712, 60092, 41044, 58080, 14380,
                55288, 46040, 58456, 42636, 34088, 896,   59480, 12820, 35348,
                3572,  40656, 6416,  58328, 22176, 17344, 46196, 51064, 19448,
                32096, 60260,
            },
        },
        {
            {
                10020, 55432, 2660,  21072, 62816, 7560,  15836, 61560, 21852,
                28888, 55828, 14448, 36724, 53768, 58804, 8472,  33904, 65440,
                54320, 30960, 59628, 40468, 63908, 52872, 33828, 3596,  54528,
                38044, 21836, 9720,  17328, 61472, 20236, 33536, 492,   44792,
                37016, 25840, 52604, 2076,  38616, 12012, 17460, 44004, 56584,
                14740, 62756, 26012, 30112, 43040, 37316, 5712,  32584, 44404,
                64800, 34552, 5916,  25588, 56556, 63184, 21204, 26996, 47212,
                37696, 12120, 59240, 53076, 1724,  35020, 26424, 14904, 37292,
                30504, 52480, 9212,  65400, 18412, 57708, 23180, 61648, 53832,
                39896, 14776, 22508, 60992, 9552,  40352, 59508, 11824, 55376,
                172,   35352, 51064, 5820,  41352, 60024, 1240,  36440, 12700,
                53688, 39840, 64448, 80,    58820, 10500, 29944, 55396, 64804,
                18324, 21968, 57100, 11564, 26164, 20540, 43336, 56684, 3644,
                10468, 46612, 19352, 31524, 54004, 41056, 60928, 35948, 1004,
                57404, 6260,
            },
            {
                15652, 44828, 31380, 12604, 47936, 51776, 25132, 56924, 47176,
                33176, 10124, 60760, 23200, 45728, 19524, 40208, 51616, 13100,
                3452,  37604, 27064, 14220, 23400, 46440, 8920,  62512, 26000,
                12516, 51040, 45792, 2004,  38564, 27536, 48120, 15380, 57620,
                50732, 9724,  20052, 34496, 47972, 30572, 64360, 8660,  27980,
                34368, 6636,  11684, 49252, 456,   14392, 28156, 56888, 17512,
                10156, 30056, 59760, 19000, 39528, 8372,  50372, 16980, 2332,
                61856, 5760,  25508, 39652, 65088, 10200, 42820, 60540, 48996,
                76,    43508, 34432, 27000, 4172,  42288, 33084, 5832,  10664,
                46040, 2100,  56828, 42972, 47764, 33292, 17100, 6096,  28864,
                48432, 43052, 61900, 28176, 45488, 21180, 15848, 63232, 28532,
                23456, 45328, 17872, 27344, 51968, 19748, 34596, 50472, 7420,
                27544, 44348, 35568, 41676, 51004, 828,   53456, 38888, 25204,
                64924, 51120, 216,   44024, 27164, 4332,  52028, 45736, 23084,
                39944, 47588,
            },
            {
                25400, 36568, 57924, 27532, 18140, 38292, 34876, 12144, 6964,
                18936, 39996, 612,   27840, 12008, 31676, 6536,  60376, 25248,
                45636, 8248,  49272, 56092, 2188,  30476, 16196, 43416, 18884,
                32628, 59840, 28784, 56896, 53616, 11732, 59940, 41652, 31680,
                3868,  29476, 63636, 14780, 54716, 24612, 3656,  50340, 41976,
                19168, 54352, 39392, 21564, 60572, 53040, 41664, 4020,  49912,
                22176, 54424, 37248, 14548, 52768, 32380, 42792, 55724, 34880,
                29156, 48884, 43288, 21692, 13616, 50308, 20596, 32212, 5684,
                25072, 16800, 54208, 59972, 11476, 46784, 51928, 16908, 28568,
                62984, 32344, 6588,  27396, 13628, 51444, 25264, 36584, 63536,
                21928, 8540,  17436, 58208, 3236,  31228, 51648, 7048,  33748,
                58176, 3940,  32052, 8072,  38784, 43892, 3084,  22936, 12492,
                32300, 54028, 14996, 5264,  29516, 63660, 8764,  13748, 31072,
                16500, 37276, 8304,  59092, 23528, 11568, 32304, 18208, 28564,
                53620, 63208,
            },
            {
                19320, 42312, 8580,  52932, 4928,  59080, 1360,  42480, 62680,
                52964, 48856, 35096, 64152, 50388, 55988, 1660,  21268, 35796,
                17016, 63268, 19676, 6184,  35404, 60696, 39384, 57860, 116,
                36764, 5892,  14720, 22924, 34196, 5120,  24524, 7544,  20936,
                38836, 59012, 43588, 5824,  40988, 57824, 20904, 31988, 59780,
                1832,  47060, 65116, 16356, 8108,  33496, 23488, 63288, 38672,
                45784, 7252,  24840, 11432, 46476, 1056,  23148, 10920, 64424,
                20048, 15452, 2884,  56384, 29468, 3948,  54780, 38832, 62736,
                57512, 45608, 7340,  20960, 37588, 30656, 1124,  55836, 36736,
                21536, 50452, 18132, 38460, 64492, 1192,  57552, 46528, 39824,
                13752, 53208, 26080, 37548, 11852, 55392, 40308, 47228, 19412,
                38356, 11040, 48388, 61064, 54700, 15792, 63296, 57320, 37420,
                48064, 3540,  60272, 37788, 17388, 23064, 34452, 46076, 60488,
                21696, 54324, 34056, 14504, 48948, 38720, 64720, 7484,  14268,
                34300, 4272,
            },
            {
                60948, 532,   23512, 64440, 32328, 45396, 20344, 26732, 30464,
                15528, 24280, 43864, 4324,  16844, 37656, 42528, 48500, 29564,
                57372, 41076, 32224, 43712, 53720, 21380, 10452, 50376, 23744,
                48008, 63560, 43804, 39992, 18408, 49896, 44572, 64400, 54244,
                12452, 47456, 17680, 27692, 10520, 35812, 45940, 13252, 38092,
                9976,  23216, 30964, 35956, 44900, 58788, 11036, 27100, 14976,
                1404,  58940, 42064, 64004, 27368, 60660, 30440, 37992, 7076,
                45088, 52368, 32808, 59848, 36320, 45356, 17624, 9016,  22812,
                13124, 29744, 40124, 50276, 14764, 61908, 24672, 44512, 7868,
                41836, 58568, 11572, 54212, 23188, 8740,  31500, 19804, 2900,
                56332, 33816, 4644,  49228, 23248, 61336, 9240,  26228, 2112,
                63848, 55948, 22180, 13376, 25136, 6420,  29120, 41316, 17604,
                62408, 10208, 24936, 48892, 54920, 58904, 40036, 3272,  49844,
                6396,  28012, 41864, 62316, 19708, 3188,  54632, 42924, 59412,
                10692, 50540,
            },
            {
                30924, 12980, 48608, 40064, 14984, 10812, 50080, 55544, 38916,
                5356,  59368, 9628,  21528, 58068, 26056, 14416, 62324, 9760,
                2720,  50824, 11268, 24628, 47660, 13876, 29168, 7064,  55640,
                27188, 11096, 52612, 2996,  62016, 29352, 36512, 16804, 26332,
                34976, 2268,  23632, 53336, 61688, 232,   16888, 63220, 26532,
                51176, 57688, 4444,  25892, 50628, 2632,  19620, 48516, 35724,
                31780, 51376, 20604, 34216, 6340,  48968, 15044, 58604, 54012,
                26612, 39416, 9852,  23160, 7152,  63592, 27284, 42072, 51992,
                36236, 2824,  64200, 26328, 3732,  57204, 19092, 12524, 64912,
                26208, 2520,  47064, 35488, 28920, 44956, 16012, 61348, 10888,
                42752, 30456, 65380, 44708, 18656, 34932, 14368, 43372, 50852,
                16008, 29452, 42364, 36372, 46500, 33336, 49532, 772,   26180,
                45204, 21052, 31484, 42676, 6904,  10856, 27220, 18892, 43036,
                11776, 57228, 2024,  25376, 45072, 29488, 35544, 22652, 47152,
                26744, 38316,
            },
            {
                43636, 56540, 17584, 27996, 36288, 61724, 22908, 3468,  64708,
                12968, 32816, 54188, 29724, 47080, 7020,  33420, 53160, 23792,
                38876, 15584, 28076, 61676, 1348,  37712, 65180, 42176, 16880,
                34956, 20216, 31560, 8628,  57804, 13196, 816,   52012, 9568,
                61252, 56336, 32724, 39524, 48260, 29284, 52180, 5200,  33936,
                44376, 15064, 41552, 11936, 56520, 29504, 39796, 54280, 61248,
                9644,  16756, 57120, 3600,  38812, 18708, 43328, 22048, 3176,
                13224, 61088, 18348, 51312, 47808, 15132, 31904, 1384,  48440,
                18912, 55400, 10616, 32552, 47912, 40948, 35136, 51044, 31616,
                15688, 39532, 19956, 5984,  62320, 41000, 53096, 35756, 48944,
                24384, 15420, 7692,  39680, 1764,  52632, 30348, 58736, 21436,
                34412, 4884,  53100, 1576,  19248, 58248, 11880, 53856, 8048,
                35196, 52816, 2128,  65300, 14352, 46868, 33080, 52732, 63544,
                30696, 37972, 17500, 50404, 9600,  56816, 6308,  16852, 2088,
                55508, 20652,
            },
            {
                5416,  34200, 59904, 2372,  53964, 7228,  42904, 16952, 35532,
                47844, 42000, 18392, 38712, 63672, 11472, 45480, 18904, 5596,
                54984, 35044, 58624, 5232,  18712, 56756, 31972, 45980, 2404,
                58988, 38960, 47208, 26028, 42368, 21604, 47984, 31304, 40420,
                44940, 6548,  19492, 13432, 8452,  21384, 42408, 56256, 18964,
                7680,  60892, 20620, 37244, 63796, 17404, 7380,  43268, 4548,
                25384, 45060, 29384, 12888, 55380, 51740, 9400,  36008, 47440,
                29300, 42060, 748,   25716, 38548, 56648, 11580, 60512, 34284,
                25232, 59512, 16180, 44044, 22648, 6024,  9664,  55008, 4120,
                48880, 59844, 33480, 56428, 13912, 3368,  22076, 6612,  27716,
                59712, 51268, 20888, 57352, 28232, 62784, 5400,  38656, 8524,
                46152, 61256, 26456, 10052, 65060, 38136, 22324, 60988, 15600,
                40120, 58592, 19200, 28836, 36836, 22108, 57628, 1664,  15304,
                22856, 47016, 61352, 32124, 13640, 63388, 52288, 41372, 33128,
                64224, 8792,
            },
            {
                25868, 51808, 45800, 9800,  24852, 47400, 32064, 28772, 52200,
                8300,  24956, 3804,  50984, 1088,  35728, 60200, 27040, 43196,
                64820, 20748, 46372, 41436, 52300, 26136, 9660,  22348, 50832,
                14060, 6180,  64284, 15960, 55496, 37796, 60448, 5288,  22512,
                14792, 27332, 50692, 65492, 36740, 59672, 25080, 11708, 39208,
                30324, 53388, 47820, 704,   23708, 46232, 14024, 31236, 21356,
                65340, 49744, 40436, 22956, 63216, 31928, 25600, 61916, 5996,
                33108, 56140, 64960, 34872, 4800,  43448, 21560, 53488, 4556,
                46112, 7760,  37416, 53272, 63028, 29116, 60820, 20712, 27792,
                43320, 23336, 10248, 51084, 25808, 48216, 32680, 57060, 39240,
                696,   10116, 36276, 46136, 16856, 11336, 47920, 23580, 54824,
                12624, 17960, 39812, 48692, 29792, 5752,  43732, 32088, 27408,
                47200, 12628, 6100,  50952, 41744, 61912, 5372,  25940, 36200,
                55600, 7864,  4436,  40152, 20836, 37104, 24820, 11756, 28388,
                49812, 15256,
            },
            {
                62024, 21636, 30300, 41400, 19572, 56832, 60772, 376,   20912,
                58168, 61568, 28288, 55860, 22892, 13928, 30304, 49628, 2148,
                31900, 12172, 7712,  29896, 15036, 35912, 3992,  62804, 40596,
                28648, 53376, 23384, 3800,  33068, 10776, 18672, 53248, 63600,
                35528, 57724, 41256, 4940,  45584, 31284, 2400,  49500, 64344,
                3484,  27004, 32884, 9612,  52360, 34304, 55924, 60076, 38128,
                11816, 1720,  35440, 7784,  46704, 156,   16976, 57176, 49900,
                20120, 14928, 45960, 10964, 30664, 17444, 62248, 28832, 14604,
                41060, 30708, 20068, 2296,  13444, 39104, 16608, 45916, 36068,
                13096, 63656, 248,   30076, 42384, 16656, 64836, 11940, 18460,
                44236, 63364, 32468, 54332, 25392, 42592, 33612, 352,   64520,
                30972, 44040, 57392, 23720, 13972, 55160, 18188, 50392, 4096,
                63872, 23804, 33556, 56288, 17344, 9360,  49552, 45388, 40704,
                10716, 51216, 26564, 59456, 43952, 424,   47412, 59160, 19228,
                44748, 39228,
            },
            {
                35680, 12500, 3520,  64984, 37780, 14348, 11016, 39664, 44208,
                14860, 36456, 10292, 17008, 43960, 40664, 57900, 8984,  16332,
                51580, 40076, 23320, 62124, 49540, 43700, 54880, 17628, 32840,
                11408, 59960, 35512, 45180, 50076, 28928, 2456,  25792, 46616,
                11444, 29868, 1064,  24540, 16576, 54064, 35176, 14528, 46192,
                22448, 58084, 16644, 61804, 39988, 26040, 2872,  48772, 28056,
                17848, 54676, 58688, 15216, 42644, 28528, 37896, 11964, 40256,
                24600, 8264,  54416, 22352, 58672, 48224, 6216,  39256, 23656,
                64532, 51400, 57644, 26736, 33836, 49892, 58840, 1644,  52268,
                7584,  55532, 38732, 19020, 60492, 8636,  37356, 46604, 53712,
                29880, 23364, 3556,  13704, 6328,  59028, 19024, 50032, 27360,
                36652, 7016,  3160,  34088, 60544, 40564, 1784,  35976, 10868,
                54412, 37624, 44776, 512,   26764, 31980, 13140, 20452, 64196,
                29236, 18988, 33940, 54680, 16168, 30352, 8312,  34804, 4692,
                56936, 1232,
            },
            {
                54660, 47744, 17160, 53356, 33344, 4976,  50672, 27024, 54728,
                5420,  46420, 31528, 49204, 62420, 4816,  20324, 37760, 25564,
                55444, 34668, 57344, 900,   11048, 20496, 60472, 25348, 47720,
                636,   43068, 19292, 7524,  62128, 39008, 56864, 42476, 8036,
                17580, 49044, 54860, 62460, 10580, 20312, 58828, 28468, 8508,
                42088, 36340, 5536,  43888, 13320, 19712, 6304,  42224, 9028,
                51892, 32992, 26156, 20232, 60944, 53480, 4856,  64100, 44328,
                2976,  51424, 28236, 41380, 1736,  35884, 51784, 10244, 44724,
                380,   12388, 47576, 9364,  43124, 4852,  23888, 32216, 41152,
                26444, 15508, 34268, 45292, 54184, 2800,  27432, 21408, 5896,
                14728, 40836, 48600, 61508, 37504, 52148, 40472, 14708, 60100,
                20964, 52760, 16580, 49636, 8692,  45124, 25632, 62604, 21420,
                29280, 7852,  15208, 60780, 39488, 52440, 59252, 35092, 1160,
                57168, 46248, 12996, 3268,  23712, 49420, 53540, 61740, 14432,
                30908, 23636,
            },
            {
                42740, 59360, 6576,  28164, 45440, 23192, 62980, 18244, 34464,
                22572, 65332, 39352, 7132,  24788, 33380, 52636, 46956, 63924,
                4056,  18472, 47448, 27120, 33572, 39056, 4932,  8704,  37928,
                51656, 15068, 27224, 54312, 12660, 22788, 15480, 31936, 36508,
                59864, 21276, 33672, 38904, 43356, 6580,  51104, 38260, 62336,
                18544, 11840, 56064, 51000, 29100, 64488, 57516, 36468, 22480,
                62636, 44520, 5432,  47756, 9748,  34576, 23184, 18932, 30428,
                34000, 61576, 37632, 16296, 63724, 25516, 19592, 55732, 32944,
                17656, 61360, 35860, 21820, 56012, 65456, 18608, 10796, 57436,
                61720, 21840, 49220, 4952,  23896, 31932, 58724, 50544, 33884,
                60264, 55464, 8728,  28084, 21884, 30616, 9332,  3720,  45512,
                10936, 39328, 63772, 27860, 19876, 31156, 58352, 14744, 51768,
                42456, 57720, 19656, 48624, 22268, 3700,  43736, 24764, 16820,
                6404,  42232, 60812, 36564, 65460, 10948, 38364, 21984, 41480,
                50940, 10056,
            },
            {
                18948, 25800, 37200, 9428,  58424, 41720, 2200,  47552, 30020,
                12400, 2820,  51888, 19112, 12920, 59180, 164,   11384, 28924,
                14596, 44048, 5884,  63516, 13700, 53296, 58264, 30040, 21760,
                64656, 56440, 31788, 41192, 1504,  46896, 65028, 50908, 3740,
                44112, 6324,  14152, 26816, 48140, 31888, 23644, 464,   54900,
                26448, 48824, 31668, 1388,  23360, 46740, 10332, 30716, 15732,
                2408,  38344, 13068, 31084, 50408, 39800, 14208, 55340, 46968,
                58412, 11404, 4652,  45104, 12692, 31768, 6804,  59756, 37900,
                25832, 29512, 3784,  40096, 15412, 30240, 47976, 37812, 5800,
                44060, 29164, 9700,  64220, 14288, 40144, 10552, 42988, 1352,
                25460, 35832, 19240, 43728, 1648,  56480, 63584, 26196, 33340,
                57212, 23428, 42980, 1304,  54308, 11240, 48012, 6672,  39284,
                2612,  46860, 31496, 11212, 64784, 28764, 8784,  54252, 39052,
                31096, 49656, 21260, 7304,  28276, 45460, 17216, 26924, 6728,
                64080, 33704,
            },
            {
                14768, 62716, 52216, 20692, 31584, 15520, 55776, 8376,  60548,
                43300, 58600, 26352, 55320, 29656, 36240, 44964, 22456, 41016,
                61164, 36656, 21196, 31308, 49208, 42504, 16408, 46272, 35052,
                3252,  9772,  23780, 59424, 35740, 20172, 8832,  24528, 57440,
                28316, 63064, 51948, 2256,  61316, 18020, 12616, 44212, 34040,
                4476,  21272, 60212, 38008, 17212, 34636, 53704, 43504, 50128,
                59104, 24604, 55052, 64884, 1928,  26888, 60028, 7976,  1120,
                17232, 26044, 21780, 53528, 57468, 48732, 41964, 14856, 46348,
                5716,  50480, 58980, 45664, 7848,  54092, 25900, 13344, 50504,
                19652, 1108,  53112, 36428, 46880, 56252, 19928, 63408, 16992,
                47232, 12040, 64668, 50892, 15780, 46848, 36156, 18024, 49116,
                4664,  29772, 13056, 35408, 61260, 37892, 23324, 33600, 17996,
                26484, 61944, 35384, 5324,  55304, 37488, 14660, 47464, 63052,
                11424, 26240, 52924, 40744, 32588, 55220, 1776,  58272, 47040,
                28968, 2992,
            },
            {
                44864, 39340, 740,   49468, 12048, 35636, 24624, 38640, 49664,
                16720, 36896, 9216,  41632, 48080, 63004, 17108, 56508, 32416,
                7732,  50492, 58420, 8896,  24160, 2236,  28140, 62048, 13036,
                49716, 38800, 17412, 48828, 4764,  53080, 30604, 40048, 11720,
                18532, 37728, 23068, 9936,  56052, 40404, 59708, 52472, 15748,
                64716, 45892, 9764,  41844, 7280,  61596, 13800, 5212,  27640,
                19260, 8320,  41520, 16512, 20824, 43744, 51976, 36108, 42068,
                29332, 49920, 39908, 34680, 5992,  28524, 21148, 9248,  62732,
                54292, 19740, 12716, 24244, 34136, 1872,  41716, 62916, 35412,
                31436, 59544, 40892, 17456, 25908, 6856,  30160, 38036, 52888,
                4668,  31172, 39300, 6824,  32772, 11228, 23104, 41816, 55056,
                8548,  59696, 46180, 51056, 16268, 5904,  42696, 65148, 52968,
                9168,  13648, 24148, 41344, 17740, 50616, 32612, 4236,  22776,
                57536, 2692,  18136, 14524, 58900, 5004,  42680, 20508, 13148,
                36632, 55548,
            },
            {
                22536, 30080, 5936,  60176, 46056, 64640, 3932,  28356, 20304,
                1168,  32572, 22288, 14992, 3192,  6192,  27248, 9904,  53196,
                2452,  26104, 39544, 17148, 35632, 56204, 7040,  19832, 41556,
                26300, 5488,  63524, 28464, 42540, 13936, 61380, 45712, 176,
                54964, 47380, 42212, 29416, 34260, 3788,  25576, 8148,  30016,
                40028, 13220, 28144, 51980, 58000, 26324, 21572, 40376, 63604,
                47404, 35536, 29028, 57636, 4144,  32232, 11668, 24164, 61996,
                54048, 9032,  65396, 18480, 684,   61328, 52312, 23984, 34768,
                1484,  43216, 38676, 63688, 51436, 20852, 56920, 4064,  16156,
                55144, 24472, 12196, 33152, 61184, 2228,  49612, 13156, 27572,
                56644, 21304, 58520, 25116, 60588, 52332, 2488,  62448, 14468,
                38796, 19076, 26036, 3980,  21588, 57544, 28472, 16,    55992,
                30252, 49964, 58232, 45796, 940,   26628, 60128, 19692, 45212,
                33876, 38208, 61612, 8512,  47476, 24564, 34588, 51392, 60976,
                7888,  48956,
            },
            {
                10864, 57116, 34112, 26636, 17660, 42824, 53864, 10696, 62708,
                51632, 57704, 45452, 61388, 34080, 50724, 39116, 43528, 19528,
                60072, 46104, 12020, 52500, 65372, 45368, 10932, 51512, 32936,
                58028, 53632, 44472, 11204, 33592, 56160, 26652, 16248, 35336,
                32096, 7260,  15116, 64904, 20468, 46368, 37068, 49856, 22028,
                56844, 36128, 19068, 2516,  48308, 33316, 3792,  56516, 31264,
                472,   12060, 53072, 45912, 37720, 62812, 48068, 19300, 5620,
                33548, 14912, 46428, 31164, 44120, 37360, 13264, 40788, 31420,
                16184, 27032, 9752,  30956, 17240, 47192, 10432, 27676, 44692,
                8420,  47328, 3708,  51060, 43764, 22044, 35324, 62072, 7792,
                40780, 45248, 20,    17336, 43176, 38148, 27548, 5960,  31048,
                52796, 34688, 64500, 40768, 32196, 49068, 13904, 45184, 19828,
                40528, 37152, 7012,  21360, 63680, 12268, 42396, 55904, 9740,
                27892, 50924, 43596, 29628, 19268, 63860, 11352, 30472, 40188,
                25416, 18092,
            },
            {
                43468, 52520, 13944, 40164, 22060, 6552,  30872, 46684, 39940,
                13152, 26984, 7548,  54080, 18144, 24172, 64856, 13732, 35260,
                22988, 30628, 4700,  41996, 20636, 29324, 38264, 60388, 360,
                22568, 14436, 36608, 19728, 1700,  22900, 7588,  49376, 63640,
                21404, 59296, 50620, 4768,  54164, 11588, 16944, 62800, 1812,
                6008,  53812, 61976, 23704, 43944, 16000, 36976, 10112, 50596,
                17304, 60484, 22212, 6124,  25652, 8944,  15868, 56024, 39148,
                2524,  23000, 56968, 11264, 25412, 58732, 7892,  47284, 64368,
                55696, 49880, 60368, 5188,  58224, 37496, 32888, 60124, 39808,
                22640, 64988, 28844, 57828, 9352,  54736, 16212, 46452, 24000,
                34144, 10368, 49788, 29684, 13112, 55932, 19780, 44516, 48088,
                22620, 856,   11644, 54128, 9072,  61528, 25244, 35288, 11116,
                3508,  60700, 16028, 34020, 53256, 30140, 36688, 6820,  16452,
                64604, 1180,  12488, 36160, 54500, 39144, 204,   16232, 57484,
                5184,  64556,
            },
            {
                36080, 1708,  62036, 8796,  50508, 59196, 15916, 34028, 23080,
                4820,  42660, 30776, 37708, 11428, 57360, 29548, 932,   49084,
                55080, 15828, 62468, 33552, 1556,  14004, 25120, 48200, 17608,
                30496, 8380,  47392, 62696, 50856, 59744, 37532, 43228, 3100,
                25616, 12600, 36604, 27288, 43752, 58268, 31760, 41052, 26896,
                46760, 33024, 8776,  30152, 12232, 65240, 53312, 45228, 23988,
                38536, 43072, 33296, 14620, 51116, 59156, 30308, 44848, 27300,
                48644, 59924, 41360, 3764,  50132, 17112, 28100, 3244,  20352,
                6160,  35604, 21932, 44560, 32,    24856, 14032, 51864, 18276,
                5148,  36636, 14888, 19280, 38600, 26664, 41584, 4492,  58948,
                18880, 63292, 54456, 36824, 8124,  65388, 33776, 10744, 60980,
                16476, 58120, 29000, 44064, 17748, 38000, 6732,  52340, 63824,
                22592, 48236, 26296, 10000, 44268, 2876,  23496, 49576, 40340,
                53956, 21096, 25324, 59500, 5748,  50068, 22232, 46412, 53328,
                33096, 28148,
            },
            {
                20164, 47172, 24792, 29836, 54784, 37072, 256,   61624, 55716,
                18352, 64480, 49012, 2192,  21244, 41952, 46912, 5508,  60956,
                37836, 8052,  27128, 47244, 53384, 56992, 6060,  35012, 42652,
                61792, 55884, 24712, 5852,  40964, 31344, 10092, 18640, 57184,
                52900, 41672, 31368, 17700, 2752,  23600, 9052,  59936, 13488,
                43188, 16344, 49496, 58952, 40872, 6360,  18760, 27244, 62640,
                7776,  56212, 2756,  64084, 40216, 35128, 784,   21080, 64356,
                12932, 35820, 29560, 20644, 55540, 34220, 61728, 52688, 44916,
                11912, 40172, 14968, 29368, 52824, 42220, 6752,  63244, 30316,
                48780, 54240, 42872, 31652, 548,   64024, 11248, 32352, 51428,
                13768, 2916,  25976, 21104, 46768, 4604,  24536, 51320, 39684,
                7540,  36452, 47124, 23856, 3096,  56500, 47676, 15492, 32560,
                43240, 29672, 56892, 50916, 19060, 62096, 58048, 13268, 34772,
                30772, 4780,  48400, 15572, 32976, 27660, 61244, 9776,  42512,
                3392,  14832,
            },
            {
                38556, 58200, 5364,  18248, 42012, 12792, 26752, 48372, 9452,
                36300, 24972, 14616, 60180, 52324, 8872,  33772, 25332, 19052,
                11520, 43908, 58468, 23448, 17976, 39816, 63696, 10348, 52048,
                3920,  39436, 34308, 16380, 27212, 54508, 13580, 46928, 29424,
                5768,  9356,  48984, 61488, 38396, 47308, 52668, 35344, 20872,
                64156, 24780, 38592, 608,   21236, 34760, 57820, 2036,  13744,
                30336, 19772, 47252, 28424, 23268, 12204, 42472, 6388,  51828,
                16876, 8216,  53144, 6052,  39332, 13728, 23076, 37884, 30196,
                57652, 25856, 48568, 65176, 9480,  55864, 21340, 45828, 11004,
                1604,  25728, 61588, 7232,  50516, 47304, 20976, 55328, 28000,
                38848, 44056, 31636, 58640, 40496, 14908, 59736, 30144, 3276,
                26704, 52084, 14128, 62888, 30612, 20756, 41612, 26820, 58992,
                5812,  13088, 1256,  40940, 37472, 6292,  27784, 47156, 18512,
                60796, 44840, 57020, 40848, 8256,  44196, 18588, 37324, 24152,
                62740, 50432,
            },
            {
                10672, 43848, 31996, 65424, 49120, 21424, 5972,  44052, 30380,
                53192, 4204,  45920, 32896, 27236, 16860, 62600, 53744, 40576,
                31412, 51556, 2856,  36532, 8788,  30844, 44844, 15764, 21936,
                28820, 12292, 45904, 58380, 3480,  21608, 65092, 33956, 23768,
                39592, 63056, 20788, 55020, 14200, 7108,  29132, 1148,  50552,
                10724, 4284,  54856, 28384, 52432, 47612, 31564, 42244, 49364,
                35448, 54316, 9884,  16704, 57984, 49644, 55528, 31924, 38076,
                60836, 26184, 44504, 63792, 47948, 9624,  43056, 1848,  18296,
                8356,  60620, 2556,  32548, 18748, 35880, 26760, 39044, 33484,
                58040, 13424, 22188, 34864, 59452, 14584, 36912, 3572,  61404,
                48712, 6700,  16188, 52540, 1456,  35808, 48868, 17904, 43368,
                64184, 19608, 41172, 6204,  33992, 60092, 1524,  9744,  18852,
                55108, 34968, 63256, 24488, 15172, 32388, 55024, 3760,  9088,
                26420, 37004, 10992, 23596, 65012, 2576,  56196, 13532, 30728,
                7200,  55284,
            },
            {
                27500, 16308, 2464,  35452, 9996,  56116, 39336, 16520, 58896,
                20488, 40560, 10876, 56188, 39032, 6376,  44856, 13480, 57648,
                4892,  21544, 64180, 12964, 49752, 59920, 3292,  25992, 48580,
                54980, 19564, 63464, 9696,  43828, 49036, 38160, 688,   58984,
                16460, 45052, 1860,  24876, 34052, 57572, 18896, 40236, 56156,
                31512, 36464, 18284, 60400, 14156, 9308,  24328, 61284, 5404,
                22444, 39264, 60720, 3960,  45196, 8032,  18364, 24348, 46804,
                10980, 34092, 18636, 272,   31632, 27084, 56484, 63404, 33408,
                51680, 45708, 23616, 38792, 12580, 59468, 2844,  61896, 16680,
                51092, 40336, 44928, 10324, 29520, 24688, 43644, 17620, 9232,
                22196, 35264, 63932, 10588, 28252, 54888, 22328, 12180, 57200,
                32336, 9876,  55244, 49064, 12000, 53204, 36776, 45688, 49932,
                39004, 21924, 46824, 8116,  59444, 45100, 21248, 64060, 42536,
                52164, 1916,  17316, 50568, 29568, 52484, 34868, 48256, 59832,
                40756, 21848,
            },
            {
                47800, 59108, 52608, 23784, 60768, 28460, 33416, 63884, 1540,
                50208, 28320, 63192, 22676, 1696,  50652, 35780, 23296, 28508,
                48636, 16284, 33820, 41940, 27944, 19012, 55832, 34008, 60676,
                37600, 896,   32296, 52716, 29212, 18076, 8004,  53804, 12028,
                51068, 35900, 30636, 11128, 41892, 65512, 45492, 15796, 26132,
                48076, 62676, 43500, 6768,  45932, 39568, 16568, 55456, 11508,
                52224, 43444, 26092, 33116, 37312, 29588, 62836, 1652,  58280,
                5280,  42260, 56164, 23564, 60060, 15752, 49360, 21680, 12344,
                41364, 16748, 4828,  55168, 50556, 43696, 6104,  47864, 20512,
                28884, 5008,  56100, 18572, 52928, 6232,  57884, 31196, 41500,
                56840, 25312, 46084, 19560, 42192, 62456, 8176,  45308, 476,
                35500, 25068, 4632,  38380, 27520, 22880, 14460, 65320, 29060,
                4812,  16516, 53680, 30012, 51640, 10840, 39844, 34476, 14560,
                22696, 62476, 32324, 39076, 12796, 20560, 5100,  26388, 17028,
                1104,  34560,
            },
            {
                12868, 38144, 7608,  44884, 14684, 3904,  46320, 24132, 13304,
                34700, 7116,  15936, 43540, 30800, 19372, 65272, 9516,  64,
                60656, 37988, 54532, 24176, 1400,  43580, 38628, 5236,  13852,
                8500,  42280, 24056, 15136, 36056, 61280, 25272, 42372, 28304,
                19916, 5452,  60108, 48648, 27348, 3308,  22732, 60932, 4944,
                8588,  13340, 22560, 33580, 26836, 63956, 3004,  37084, 28500,
                18272, 136,   14492, 64972, 20916, 52740, 14056, 40288, 27956,
                21396, 50196, 12704, 37164, 45560, 3840,  35692, 6792,  54144,
                26684, 36404, 63804, 28808, 22228, 15160, 30752, 53576, 9296,
                64848, 36132, 2444,  62724, 38780, 46072, 1012,  65172, 50172,
                12080, 1988,  51820, 33492, 5468,  24652, 31184, 38896, 52884,
                21020, 59664, 46264, 16632, 62296, 43572, 7196,  32836, 24312,
                60452, 11648, 41544, 3020,  25824, 17840, 280,   54084, 28124,
                48472, 58088, 5424,  46472, 59536, 42884, 62988, 10428, 45220,
                51664, 63424,
            },
            {
                25336, 30620, 20404, 40696, 49908, 18200, 53728, 8860,  41608,
                58064, 47116, 37340, 60616, 12116, 56952, 47636, 39792, 32408,
                43820, 11060, 7004,  58976, 50828, 10604, 65144, 23124, 30176,
                51712, 47592, 56736, 6084,  41156, 2684,  57696, 32600, 46484,
                64168, 15108, 38568, 54452, 7972,  51852, 12104, 33744, 38816,
                53608, 29176, 57016, 1760,  51800, 20084, 48960, 32636, 62960,
                45676, 58676, 50660, 6872,  42172, 10748, 47424, 35024, 53940,
                16132, 64788, 29608, 8384,  52548, 19508, 39972, 59332, 30944,
                872,   48340, 10360, 42636, 7124,  57388, 34812, 24568, 41216,
                14280, 22824, 49092, 25956, 33132, 13080, 21128, 26572, 37772,
                15784, 30212, 39472, 60036, 13576, 58004, 47516, 16736, 63048,
                14284, 42412, 28908, 56248, 2008,  19420, 57920, 52104, 700,
                44612, 56200, 33224, 64496, 36908, 49508, 61408, 30884, 11944,
                7324,  36260, 19536, 27452, 644,   24092, 36664, 54768, 32260,
                19272, 4372,
            },
            {
                9576,  54020, 64252, 552,   57800, 30980, 37792, 61936, 26528,
                19648, 54936, 2792,  25032, 52736, 4040,  27016, 15228, 55068,
                18580, 25608, 45860, 20388, 32496, 15408, 53500, 46032, 17716,
                62164, 26968, 20448, 64536, 12672, 49988, 21912, 10404, 3944,
                55584, 24508, 44452, 21432, 17000, 36640, 29756, 43224, 58760,
                20184, 46712, 41376, 36704, 12208, 58416, 43152, 6172,  21956,
                8608,  35000, 30048, 23844, 56724, 2504,  25684, 61400, 4404,
                45004, 33072, 2092,  41424, 62340, 27712, 9716,  46356, 24364,
                56772, 14044, 61620, 33784, 18476, 47096, 61144, 528,   45564,
                55252, 31696, 43104, 7812,  16728, 60916, 48092, 34512, 7052,
                53136, 62012, 4456,  18040, 49640, 36100, 2544,  27804, 7640,
                50148, 3888,  11224, 32108, 50960, 35940, 40412, 10780, 18128,
                37976, 26892, 20360, 13712, 43584, 23032, 5692,  57368, 45648,
                16560, 42092, 55088, 10024, 52804, 15544, 40380, 7428,  28644,
                57288, 43364,
            },
            {
                59700, 33952, 14168, 26972, 35776, 11632, 22164, 5708,  51620,
                30100, 10256, 32720, 17844, 40828, 34316, 21324, 6448,  61988,
                51684, 30280, 63532, 4224,  36980, 28536, 7268,  40432, 35192,
                11328, 1892,  38220, 33508, 45656, 30104, 17264, 52344, 40108,
                34444, 9380,  1312,  32368, 63300, 56928, 48280, 100,   24392,
                14900, 4196,  64844, 17640, 9008,  30492, 24980, 14924, 56876,
                40844, 11956, 48184, 38864, 16636, 59752, 31352, 18976, 8968,
                39024, 58892, 25448, 15272, 55744, 33640, 13200, 65044, 17676,
                43824, 20608, 51912, 2500,  40132, 26724, 10060, 37456, 17928,
                5740,  60248, 11748, 52272, 56728, 40344, 9796,  54732, 19116,
                45404, 23136, 27620, 43608, 8616,  21412, 41868, 55752, 23968,
                34076, 37612, 64728, 21948, 8496,  26076, 46840, 30244, 63308,
                49112, 8024,  3900,  47496, 58748, 9236,  35008, 20020, 39200,
                64892, 24420, 33456, 62172, 30680, 47452, 58712, 22316, 13500,
                49292, 38704,
            },
            {
                21688, 5772,  50944, 44092, 7316,  59600, 42404, 47768, 1292,
                15052, 44636, 62856, 50344, 8228,  46212, 57968, 42560, 36460,
                2136,  14116, 40784, 56068, 47792, 61664, 20956, 57152, 3952,
                59244, 43988, 15852, 54848, 8584,  59448, 36784, 62412, 14176,
                27256, 58304, 50792, 42008, 13392, 6196,  19108, 62480, 10076,
                49592, 31920, 25912, 52792, 45504, 61384, 38804, 3504,  53048,
                27316, 61872, 19768, 4660,  51884, 44340, 36368, 49476, 23372,
                12152, 52156, 20180, 6316,  48204, 22484, 50888, 4036,  34944,
                5912,  38764, 31544, 23300, 58728, 49236, 13240, 63616, 50648,
                27460, 38388, 20016, 29920, 1768,  22540, 28492, 4280,  63580,
                40672, 10496, 57336, 31860, 53544, 64376, 11784, 30784, 60476,
                18384, 48020, 53900, 40984, 15864, 61636, 2744,  14196, 22808,
                58368, 34844, 53564, 29316, 15908, 40716, 53212, 26360, 4004,
                51288, 2164,  13540, 44344, 6288,  18696, 3812,  65220, 35856,
                1464,  16524,
            },
            {
                41820, 29700, 46936, 19096, 24324, 54376, 16640, 65140, 34160,
                39716, 59000, 22264, 28184, 13920, 64056, 29852, 11420, 24100,
                49516, 8640,  34504, 17920, 12296, 540,   26248, 49888, 31832,
                24224, 51364, 29104, 4888,  22760, 26264, 308,   43516, 6736,
                46972, 18648, 61016, 29172, 24944, 46200, 40628, 27948, 53320,
                37756, 59976, 6476,  34860, 760,   19552, 50796, 34288, 46220,
                16320, 1368,  55676, 33488, 28180, 12788, 7100,  63816, 55604,
                30340, 46664, 36020, 60276, 43284, 1128,  29524, 40832, 53400,
                59980, 27988, 8812,  54852, 15952, 4432,  30000, 21720, 33452,
                57188, 3372,  47024, 64572, 34848, 44320, 59328, 32412, 13960,
                35992, 2944,  48268, 14784, 25184, 38056, 5296,  51404, 44580,
                1152,  13452, 26780, 5476,  44104, 56720, 33624, 52852, 5344,
                41780, 12340, 61964, 24836, 1220,  63540, 32156, 11356, 48004,
                29804, 60640, 21648, 38156, 50420, 34392, 28108, 52204, 44908,
                25040, 62492,
            },
            {
                11156, 56608, 1856,  62032, 38852, 31756, 3544,  27904, 20012,
                56268, 6724,  36244, 4736,  53888, 38188, 3580,  19056, 59812,
                44600, 27608, 52920, 23368, 59292, 38688, 44888, 15152, 8020,
                41576, 19124, 62000, 46656, 39564, 49432, 19600, 56128, 31608,
                23092, 38116, 3248,  11104, 35904, 55820, 3820,  34280, 16780,
                44444, 12464, 21520, 40368, 55728, 13460, 28324, 10236, 65500,
                22436, 37400, 42948, 9544,  62844, 21972, 40428, 504,   16928,
                42336, 3212,  27576, 11208, 18008, 37856, 57224, 25440, 15640,
                48704, 12288, 46064, 64296, 42420, 35444, 53640, 44524, 8592,
                15556, 25100, 41600, 14396, 6852,  48848, 17496, 51260, 25484,
                55856, 20336, 61240, 34212, 52,    59228, 16064, 20276, 40044,
                9784,  57600, 29960, 60652, 19636, 24376, 48276, 38180, 27780,
                17576, 45304, 31664, 37640, 50400, 18736, 44820, 56356, 15120,
                36412, 7984,  55736, 25836, 10892, 56836, 41452, 12432, 8404,
                32520, 53332,
            },
            {
                27400, 36344, 15532, 52512, 8884,  13004, 45692, 50272, 9896,
                25260, 12896, 49136, 44016, 17564, 24908, 48300, 55300, 31960,
                15696, 65004, 6164,  43096, 29720, 9524,  33980, 55132, 64776,
                12644, 36348, 56656, 10436, 14664, 63948, 35108, 12084, 51940,
                63492, 16080, 48832, 53876, 20672, 65000, 9104,  22396, 61440,
                2080,  56596, 29200, 48900, 63588, 24096, 44124, 59128, 6920,
                31712, 50184, 25572, 58476, 15816, 53756, 47000, 25924, 34348,
                58216, 8576,  64268, 53116, 32136, 61376, 7044,  10132, 63200,
                22064, 316,   33660, 19312, 7284,  24700, 59852, 1408,  39884,
                62040, 54340, 10176, 51772, 26920, 38016, 11540, 928,   41936,
                46196, 28272, 6492,  51760, 43012, 46800, 28716, 35224, 63368,
                23516, 33168, 46964, 36500, 3316,  12496, 8868,  64992, 20860,
                56076, 9888,  4348,  21704, 57584, 8516,  5036,  23864, 59648,
                19580, 41948, 46844, 16300, 63608, 1144,  21308, 60788, 17652,
                47740, 3548,
            },
            {
                40456, 64624, 22344, 33512, 25916, 56872, 60224, 36840, 41960,
                53140, 31428, 61616, 1864,  33352, 60392, 9096,  40712, 716,
                22036, 37280, 55648, 3240,  50356, 63008, 18508, 22564, 47104,
                6440,  27096, 1508,  33228, 53684, 4360,  42348, 28480, 2252,
                8932,  43980, 33240, 6516,  39920, 15356, 48480, 30756, 51224,
                25452, 42240, 9684,  17284, 7288,  32476, 3872,  40136, 18952,
                55028, 13104, 5656,  45548, 2288,  29908, 37816, 60360, 13336,
                50696, 21484, 39124, 24644, 13636, 46380, 20284, 43976, 30976,
                37092, 41304, 55780, 27192, 51416, 38308, 11932, 20328, 49008,
                30372, 36000, 22256, 60824, 19456, 55412, 63920, 23288, 8332,
                62612, 37824, 11892, 18196, 22612, 9000,  56712, 49308, 3152,
                52996, 15288, 7700,  51572, 39948, 55060, 31036, 42856, 228,
                51012, 60320, 40100, 47068, 13384, 28752, 42684, 38376, 33216,
                428,   53372, 28520, 5076,  39560, 32248, 49024, 26812, 37608,
                58856, 20168,
            },
            {
                13680, 45456, 6092,  49056, 40020, 17516, 5164,  22208, 152,
                64028, 18176, 40252, 22820, 56120, 14540, 26840, 35168, 52108,
                46012, 10732, 19496, 33020, 14148, 40764, 2020,  52728, 30424,
                38408, 59532, 21472, 44448, 30072, 24476, 16836, 59036, 37124,
                25668, 57248, 60732, 29956, 26244, 43324, 58580, 37012, 6888,
                14172, 33724, 59064, 37840, 46460, 53580, 15008, 35932, 48000,
                29096, 60876, 41252, 35244, 20260, 51140, 9900,  4248,  18728,
                29532, 44744, 4900,  49348, 2108,  55316, 34536, 52132, 5072,
                58384, 17184, 61652, 3000,  14632, 47132, 28972, 65452, 17204,
                6508,  45952, 2548,  33596, 5368,  42896, 30264, 35160, 52800,
                15900, 30904, 58160, 40460, 65244, 32616, 13400, 6832,  25724,
                61176, 42392, 21252, 63952, 17116, 25896, 58800, 15404, 35764,
                29152, 24020, 16368, 64388, 35304, 52680, 61708, 48584, 9972,
                63896, 12648, 34632, 59176, 23624, 54044, 14508, 6512,  43720,
                54748, 7408,
            },
            {
                50476, 60164, 30536, 2344,  43936, 63276, 29012, 34672, 14692,
                45040, 7744,  28748, 10804, 47072, 42508, 64312, 6908,  13396,
                61852, 28996, 58388, 47908, 24484, 27456, 57456, 5268,  43452,
                15724, 48816, 52124, 18044, 61204, 7212,  46024, 50484, 20300,
                41004, 13572, 18808, 920,   52192, 12248, 3356,  18432, 46280,
                64340, 54264, 20296, 1468,  26832, 62644, 21596, 57776, 852,
                10564, 23960, 17892, 63612, 32200, 57296, 23548, 64940, 40680,
                54884, 33004, 15812, 62904, 28756, 40388, 16784, 26592, 12776,
                49700, 24052, 9636,  44116, 32684, 58080, 4136,  55100, 24296,
                42496, 12800, 57940, 28300, 47356, 14864, 58320, 4536,  19984,
                49260, 3288,  24584, 53784, 1952,  27056, 39184, 44988, 18792,
                37872, 10996, 29180, 1616,  45632, 33900, 50072, 4580,  46536,
                7964,  54504, 32860, 1612,  25788, 20064, 3216,  16136, 27140,
                22800, 50488, 45244, 18240, 9344,  42112, 61756, 2492,  31056,
                23868, 34544,
            },
            {
                28248, 17856, 57500, 11484, 20784, 50968, 8344,  54260, 48252,
                26204, 58552, 51396, 38464, 5448,  20544, 30052, 48680, 23500,
                39188, 4048,  42184, 7772,  61300, 37524, 11068, 34836, 62664,
                25468, 3768,  9732,  35540, 40176, 55224, 10720, 65080, 30868,
                5460,  54476, 45380, 23172, 63412, 34204, 56148, 28416, 39600,
                23492, 5132,  49756, 30064, 41704, 11300, 50584, 26060, 42548,
                53036, 38304, 3360,  49108, 7432,  12448, 43688, 47876, 26604,
                6696,  10900, 57956, 36540, 22660, 9112,  56856, 65184, 1500,
                46204, 29336, 36284, 53016, 22304, 40620, 10104, 36792, 32252,
                52280, 61744, 39088, 17760, 50572, 9616,  40192, 26160, 44788,
                60592, 9392,  42748, 34984, 48244, 16740, 51472, 59304, 30624,
                54620, 48188, 35616, 59636, 6676,  22912, 11368, 19948, 63196,
                38340, 12728, 44420, 49944, 10860, 55920, 31736, 40372, 57716,
                6880,  30768, 38516, 1880,  48164, 28204, 35372, 51516, 15940,
                63568, 10384,
            },
            {
                500,   38384, 24708, 35676, 55176, 13848, 32420, 39028, 60608,
                19860, 2620,  35420, 57076, 16996, 53120, 1492,  58060, 18164,
                54944, 33812, 16208, 21492, 53896, 45008, 17528, 51096, 20752,
                12944, 42204, 64092, 27832, 752,   14236, 33476, 23008, 2700,
                48656, 35576, 9880,  38520, 47964, 8044,  21624, 60564, 10636,
                32084, 44628, 13328, 61564, 8120,  34780, 17788, 5256,  64660,
                30232, 47020, 14940, 55428, 28044, 36856, 17280, 2704,  35040,
                61532, 51916, 19640, 42328, 48288, 3968,  31452, 42840, 21352,
                38940, 7140,  63472, 18704, 5464,  61292, 16504, 50124, 436,
                20996, 8348,  25212, 55172, 1704,  22672, 61464, 12384, 36956,
                32180, 55868, 14340, 20920, 5276,  61980, 10304, 22272, 672,
                5944,  14124, 56364, 27052, 43624, 52960, 60900, 41096, 26796,
                57232, 22420, 5384,  41324, 59996, 37308, 5920,  44720, 51728,
                20528, 60636, 14132, 55896, 64748, 19504, 11896, 46468, 21164,
                56528, 41416,
            },
            {
                53292, 47872, 8068,  61068, 41180, 27312, 3068,  23716, 11020,
                43044, 31784, 13272, 24208, 63376, 33544, 37444, 43480, 9508,
                26004, 50996, 64696, 12316, 160,   31640, 7000,  59968, 28608,
                56760, 32448, 54584, 22104, 47012, 58372, 38204, 43208, 16160,
                56848, 28432, 61504, 16900, 31316, 42252, 14880, 48796, 388,
                52752, 16964, 36964, 55488, 24228, 59272, 45672, 32580, 13804,
                8480,  21388, 62140, 24948, 41560, 52528, 59668, 30508, 14796,
                22440, 45432, 124,   26256, 12572, 62132, 35628, 51200, 15348,
                60148, 54932, 13528, 31388, 48872, 25504, 44736, 27876, 63204,
                15220, 35424, 44292, 31024, 64964, 33976, 46488, 54624, 592,
                17864, 27636, 63968, 46120, 29984, 57180, 36448, 43360, 33436,
                63448, 24516, 18112, 8752,  39000, 14892, 30580, 1096,  48940,
                17060, 31176, 62156, 19144, 28612, 15016, 24580, 65360, 11364,
                35284, 42544, 4356,  26096, 33172, 6712,  58340, 37940, 26212,
                5600,  44688,
            },
            {
                19844, 65160, 31628, 16048, 5700,  46976, 64308, 56236, 16732,
                6168,  62044, 44812, 48928, 4104,  27484, 12096, 60908, 31360,
                2836,  44596, 35876, 28028, 49032, 23236, 40932, 46480, 1584,
                39476, 7696,  16092, 50624, 4988,  19016, 26412, 51560, 62384,
                12328, 24848, 6628,  51380, 2260,  26140, 54788, 35508, 40708,
                62956, 27796, 3200,  19724, 48356, 39220, 2636,  54968, 37528,
                58636, 44012, 33124, 10896, 712,   20864, 6244,  49544, 55956,
                37392, 8392,  31816, 60396, 53596, 18552, 6252,  24884, 10344,
                33524, 26916, 42108, 1808,  59136, 11528, 38828, 55616, 4916,
                47108, 59260, 11768, 6240,  38412, 15352, 28744, 20840, 7624,
                48756, 38328, 3908,  11360, 40292, 24756, 7576,  15324, 52896,
                40440, 45972, 50728, 32824, 65436, 5300,  36744, 55656, 9220,
                34828, 2788,  53484, 46432, 8328,  52464, 48496, 18516, 2432,
                27912, 54688, 17092, 44664, 23012, 52836, 40924, 2860,  61484,
                32444, 14272,
            },
            {
                27756, 3884,  23052, 51748, 43112, 18720, 36396, 49840, 40652,
                27964, 52568, 21188, 8900,  41228, 15996, 50804, 21716, 47312,
                14884, 5924,  59200, 38692, 56548, 62484, 14472, 33916, 10372,
                49600, 24036, 36520, 43408, 30344, 60500, 9064,  5940,  32080,
                46392, 40896, 20368, 58184, 44140, 65516, 5756,  19284, 25128,
                8460,  57300, 42768, 31028, 6556,  15464, 28484, 22844, 50980,
                18592, 6432,  36324, 57816, 46532, 61268, 39636, 25636, 11436,
                63496, 41440, 50744, 15284, 38432, 29948, 43760, 58944, 47988,
                19576, 4208,  51984, 45384, 35280, 20044, 7724,  33560, 23220,
                40500, 29380, 49456, 19428, 53496, 3432,  51464, 40956, 63328,
                57712, 23356, 51660, 34168, 55288, 18452, 49832, 28728, 60168,
                20372, 2956,  11900, 57268, 21804, 48416, 18668, 24336, 45092,
                64420, 14244, 39288, 23764, 36024, 648,   57356, 32984, 39344,
                47536, 8440,  62820, 36592, 48784, 10228, 30168, 17828, 50012,
                9808,  35920,
            },
            {
                40220, 57172, 12264, 34156, 59620, 792,   21980, 9320,  33360,
                58752, 1432,  37764, 30448, 55940, 59916, 7348,  39388, 63236,
                55516, 24220, 19788, 10152, 3696,  17968, 26128, 52844, 65324,
                19324, 58652, 2780,  63416, 12036, 39952, 54976, 21512, 37264,
                524,   53780, 34572, 13684, 29184, 37304, 12024, 59592, 33140,
                50236, 22392, 12460, 64896, 51896, 60380, 41456, 11056, 63736,
                26260, 3564,  53816, 29252, 18860, 13664, 32392, 44176, 1844,
                18028, 27584, 5584,  23604, 46512, 9668,  54720, 1044,  37160,
                40772, 64348, 23540, 15660, 56824, 28600, 63964, 14052, 52452,
                17668, 2304,  57104, 23824, 42320, 60188, 10544, 25500, 32792,
                12960, 42452, 16396, 61456, 1852,  44696, 64916, 12512, 4472,
                25872, 36804, 29796, 42296, 2032,  28332, 60440, 12324, 41880,
                51220, 27172, 57896, 11032, 60860, 29968, 42620, 13676, 61952,
                22252, 31552, 12536, 57524, 120,   15436, 63440, 24660, 45628,
                59856, 54380,
            },
            {
                16864, 49224, 45036, 8940,  26980, 53352, 30548, 13052, 45764,
                24400, 14748, 65420, 18192, 46828, 25344, 35248, 668,   28584,
                33252, 42228, 51612, 30512, 46892, 43264, 6028,  37724, 29768,
                13468, 44784, 27104, 53448, 34492, 15680, 48760, 28704, 64672,
                17812, 10100, 60752, 49240, 4400,  22780, 51880, 46560, 15432,
                4568,  45084, 37996, 34048, 9828,  20780, 46516, 200,   31964,
                48716, 39160, 15956, 50252, 7664,  35360, 65200, 21808, 53980,
                47644, 59060, 33444, 56632, 2608,  64816, 21048, 27888, 13016,
                56072, 7320,  30412, 10088, 39784, 50788, 1420,  41796, 60372,
                9560,  37372, 62968, 27540, 7996,  35140, 45704, 18424, 5032,
                47336, 29688, 8556,  21060, 27328, 6584,  31592, 39576, 47088,
                58172, 8296,  54388, 61704, 16508, 39776, 53284, 33540, 3444,
                19444, 6940,  32508, 43784, 17420, 21440, 6224,  54400, 25780,
                4576,  49996, 40212, 20804, 28296, 43156, 54100, 37220, 7592,
                22020, 1336,
            },
            {
                62276, 24992, 37332, 63828, 15260, 39396, 56804, 63144, 4152,
                50600, 34800, 53996, 11304, 3048,  43180, 19912, 53732, 11184,
                17208, 7704,  64052, 13624, 34692, 54464, 21140, 57448, 2208,
                48228, 33064, 8672,  20412, 4364,  23736, 42168, 3424,  57792,
                43868, 23884, 31556, 39880, 17036, 56416, 9372,  41396, 30460,
                61928, 18304, 54764, 1592,  25920, 57052, 35968, 17104, 55588,
                42952, 12136, 62952, 23416, 44824, 56752, 4516,  9736,  29764,
                14136, 36600, 20432, 12220, 40952, 34660, 15700, 52196, 61076,
                32272, 17948, 46552, 61732, 4752,  22084, 47704, 31280, 25432,
                45240, 33920, 12880, 47616, 16316, 30520, 56664, 37960, 62136,
                1320,  36136, 59588, 53324, 49160, 37876, 22876, 53852, 17340,
                33668, 43776, 13988, 22940, 46424, 10288, 7240,  58236, 29864,
                61556, 22516, 50120, 3776,  47728, 63996, 37968, 10120, 45756,
                58836, 18168, 53760, 7356,  34796, 59556, 4080,  31984, 13300,
                42568, 29444,
            },
            {
                11216, 6276,  31088, 3256,  20532, 47240, 6924,  25812, 42440,
                19184, 7924,  40364, 27252, 32712, 58440, 14304, 61808, 44768,
                50084, 39648, 56308, 1328,  27276, 61512, 9176,  41624, 24772,
                60064, 17004, 40592, 50044, 56464, 62176, 10484, 33636, 14228,
                47956, 7536,  27268, 2936,  44892, 63760, 26740, 36144, 2224,
                58140, 28208, 49052, 14612, 43872, 29876, 5284,  61212, 24304,
                7900,  28636, 59188, 2104,  40996, 26116, 52292, 38584, 60304,
                42272, 3496,  51672, 62312, 28936, 49668, 25368, 42664, 3536,
                36028, 50212, 25524, 54220, 37052, 16776, 58692, 6156,  19488,
                54384, 4084,  21592, 55964, 112,   64508, 22460, 13660, 50156,
                54816, 24632, 43772, 11044, 33308, 14164, 62380, 220,   10600,
                51228, 64300, 5240,  32292, 52032, 36104, 25172, 47540, 15368,
                37300, 40856, 56348, 13916, 34032, 27324, 50976, 15896, 35656,
                29596, 2652,  43916, 64664, 24084, 10848, 50636, 19348, 57740,
                48376, 35068,
            },
            {
                18944, 55872, 41980, 51172, 58716, 33260, 17540, 52004, 37500,
                60632, 22520, 47968, 62780, 5128,  50876, 22308, 37448, 6544,
                26108, 20340, 36112, 23176, 48844, 16120, 32424, 51160, 12156,
                36148, 5804,  63096, 29020, 37132, 45156, 26656, 52652, 20188,
                36384, 55804, 61868, 50712, 19960, 32828, 7008,  53408, 20968,
                11488, 39900, 7936,  23436, 62436, 50932, 40072, 13440, 47428,
                20948, 37580, 33720, 19384, 10476, 31456, 15524, 48412, 23112,
                7136,  25880, 44800, 9152,  17664, 4532,  46140, 8152,  22740,
                11460, 44160, 300,   13536, 32652, 27132, 43544, 11920, 65072,
                49620, 29824, 43088, 39232, 51284, 6364,  44028, 8948,  28404,
                20108, 15500, 39688, 2712,  57064, 18764, 41724, 59208, 29256,
                24344, 19452, 38500, 27448, 1248,  63016, 20572, 43064, 54596,
                260,   10160, 62776, 24968, 8208,  55524, 1520,  41792, 23196,
                61220, 14028, 38172, 32320, 16432, 46540, 40016, 27556, 64328,
                3032,  52524,
            },
            {
                45976, 60292, 14216, 22584, 8716,  44244, 1968,  29264, 10376,
                31776, 212,   56208, 13292, 36580, 29016, 9416,  48272, 31384,
                58128, 4636,  60428, 11600, 44336, 40000, 3808,  63676, 19936,
                46408, 52712, 23060, 14040, 1792,  18036, 6668,  58704, 30156,
                1364,  41740, 15636, 11100, 38212, 59288, 14072, 47736, 24752,
                44424, 64464, 33192, 36816, 4000,  19340, 9564,  34264, 52260,
                57672, 5572,  46044, 54876, 49460, 61640, 1204,  18440, 33984,
                63868, 54672, 31672, 39692, 58376, 53656, 36872, 63704, 56780,
                29392, 60220, 20772, 40724, 63484, 8924,  55800, 34336, 37660,
                15032, 8340,  61120, 18372, 32176, 25952, 53992, 40868, 32672,
                61008, 5648,  64580, 30368, 47724, 26624, 8368,  45392, 36644,
                3824,  48652, 41800, 56456, 17080, 58968, 13092, 5580,  27664,
                49352, 18220, 30920, 44432, 39128, 59576, 18772, 31588, 52812,
                5632,  48096, 26148, 52112, 1236,  55888, 22340, 6088,  15196,
                38504, 23612,
            },
            {
                27080, 372,   39092, 28576, 35708, 62356, 53860, 13948, 58292,
                43220, 16228, 25220, 44860, 19028, 55228, 41368, 64412, 1688,
                15624, 46340, 29940, 53652, 18596, 6956,  56688, 27604, 30940,
                844,   55916, 10872, 42704, 32128, 61052, 49332, 39204, 12468,
                63876, 22616, 34024, 54036, 25428, 328,   42032, 17708, 56172,
                5732,  51624, 16448, 59016, 45808, 54608, 26648, 65496, 30788,
                2852,  17400, 63620, 13872, 28088, 36308, 43576, 57788, 11492,
                46660, 15088, 19556, 560,   23956, 12760, 30764, 19408, 15176,
                39168, 52532, 6496,  57164, 47472, 3136,  51108, 22556, 1272,
                27800, 58312, 24044, 11124, 36040, 14772, 59776, 2804,  48232,
                11692, 35032, 45216, 22472, 4468,  52744, 34564, 21108, 55620,
                15060, 60424, 11796, 7648,  49632, 29748, 39248, 33280, 65056,
                23696, 36336, 52508, 2924,  21176, 46812, 12364, 65152, 9060,
                57412, 21056, 11448, 62392, 8244,  36424, 60864, 31268, 44200,
                9680,  33712,
            },
            {
                54776, 7508,  65280, 50248, 11480, 25008, 40396, 21332, 46756,
                64760, 35660, 52676, 7084,  59580, 3380,  16620, 23860, 34612,
                52176, 42140, 9012,  65124, 33372, 24128, 49916, 14836, 44564,
                38580, 25672, 34528, 57960, 47320, 22152, 9436,  25556, 44168,
                51236, 4744,  46464, 7968,  29680, 49176, 62912, 32072, 37128,
                27136, 1112,  30948, 21976, 12312, 41208, 1828,  15540, 38700,
                44584, 25536, 40492, 8400,  21296, 5204,  24816, 52832, 29204,
                3092,  37680, 60560, 48040, 34272, 61940, 5980,  41536, 48808,
                2736,  26620, 35104, 17144, 24468, 30204, 18724, 41460, 45872,
                53088, 39872, 47752, 2040,  63280, 49804, 20472, 37380, 17372,
                55684, 26176, 51448, 16404, 58460, 13020, 64012, 2028,  50688,
                31392, 26084, 45936, 35100, 22624, 44260, 2628,  55568, 46176,
                7104,  11792, 57496, 15692, 28824, 5092,  36768, 43464, 27116,
                34852, 40532, 45500, 29240, 18984, 42468, 12632, 49276, 53424,
                62860, 17308,
            },
            {
                47840, 31956, 15944, 45332, 5012,  18400, 56600, 6348,  26956,
                2904,  9504,  29704, 39976, 33428, 47260, 27036, 54076, 12668,
                38140, 21868, 26800, 348,   38892, 58924, 35952, 9868,  60228,
                17980, 7876,  65052, 4316,  15620, 53316, 35892, 3396,  19212,
                59496, 27900, 16924, 40660, 57700, 21508, 4504,  9712,  60776,
                13544, 42784, 57332, 8476,  50220, 29632, 60444, 47896, 22816,
                11416, 54096, 33164, 51544, 58660, 39112, 64864, 8744,  41260,
                22072, 56900, 26876, 9676,  51016, 43928, 27632, 55596, 9608,
                33256, 65348, 11120, 44544, 38576, 13952, 59280, 63148, 5000,
                16796, 33356, 6980,  54844, 44932, 29540, 4708,  24440, 43248,
                58996, 7392,  41280, 9952,  38276, 32348, 43392, 17780, 40148,
                9084,  62500, 4480,  18780, 63844, 51836, 9520,  16144, 20256,
                40968, 60332, 32492, 48024, 63084, 54152, 24268, 49884, 384,
                16744, 51264, 3168,  55140, 33500, 58052, 25928, 1936,  21124,
                5056,  40360,
            },
            {
                12424, 57072, 20384, 37636, 59416, 30864, 48520, 33684, 38624,
                51048, 19616, 60852, 12784, 21916, 62964, 10264, 44144, 59312,
                6836,  61628, 17256, 51076, 12892, 46780, 21000, 3296,  52320,
                41296, 48736, 28720, 19888, 39960, 30576, 62116, 55908, 14672,
                31728, 38024, 54840, 65168, 12032, 34896, 44872, 23640, 54140,
                49728, 20144, 38240, 63624, 24676, 35672, 6428,  19612, 58116,
                7072,  62104, 29148, 736,   47340, 16624, 32144, 13736, 50256,
                34664, 5872,  42492, 14192, 21392, 2484,  16816, 59120, 23212,
                46556, 20060, 50808, 61092, 6928,  53828, 9548,  32280, 25696,
                12016, 62056, 21908, 27480, 13428, 40524, 9672,  65404, 31704,
                1460,  21004, 29496, 62928, 48676, 24920, 6068,  28004, 57996,
                23020, 37584, 53644, 32556, 12688, 26436, 37320, 61668, 29456,
                50872, 4428,  25936, 39632, 9864,  33864, 14000, 60704, 20116,
                31212, 63136, 14332, 24060, 4552,  48172, 16104, 29936, 59112,
                35320, 24016,
            },
            {
                29144, 42572, 3924,  26344, 52152, 1000,  62672, 13000, 16892,
                55664, 24012, 41776, 48932, 972,   37184, 30984, 4008,  19476,
                28872, 47120, 55204, 31740, 43132, 6200,  29080, 63936, 32704,
                13372, 22736, 54828, 46000, 11836, 24,    27076, 41188, 46736,
                5888,  10392, 23940, 1984,  19120, 52456, 15148, 39912, 28952,
                6704,  33804, 47680, 3816,  17156, 43080, 55460, 32160, 50884,
                36328, 42520, 18740, 12412, 23900, 44324, 3724,  54560, 19064,
                46068, 61324, 30036, 53764, 63596, 36224, 31468, 52008, 4452,
                37460, 14872, 1052,  31644, 27424, 48156, 36004, 20484, 49536,
                56812, 36796, 42892, 18568, 57508, 34184, 50548, 54516, 15144,
                46244, 52788, 35864, 3912,  18804, 55036, 61304, 11188, 47112,
                14476, 43664, 516,   56784, 41528, 5792,  58072, 23412, 34808,
                13772, 44020, 19176, 1020,  22524, 42048, 6740,  55576, 38372,
                7752,  58540, 39540, 44272, 10252, 64384, 38112, 41920, 11152,
                51148, 61096,
            },
            {
                8852,  53348, 63088, 34556, 10904, 23652, 42488, 7992,  45832,
                63628, 32284, 5808,  58124, 17712, 51744, 56572, 41136, 49828,
                33756, 2592,  10548, 23684, 62192, 16056, 55560, 24588, 37404,
                1376,  61428, 5624,  36536, 59236, 49740, 8620,  23024, 34372,
                62668, 50260, 43968, 36380, 48336, 30848, 58664, 1452,  63912,
                18152, 11080, 26224, 53364, 13856, 61680, 340,   10660, 27672,
                15024, 2984,  49164, 55952, 37492, 59660, 28224, 62552, 25052,
                10960, 1264,  17596, 38808, 7248,  46916, 10824, 40324, 25308,
                60344, 42756, 56392, 21996, 40796, 3632,  58012, 43164, 460,
                28880, 8760,  51484, 2664,  60408, 5380,  22312, 37924, 26788,
                8388,  61132, 13456, 23732, 44836, 824,   36612, 52120, 30524,
                6864,  49572, 20176, 29068, 15808, 48864, 43524, 1356,  53600,
                8648,  64516, 55876, 49424, 58928, 17032, 29232, 44180, 47832,
                11544, 22216, 27880, 53000, 34656, 20740, 7272,  54872, 46344,
                18456, 1140,
            },
            {
                44660, 22404, 14816, 40760, 47444, 17776, 54592, 27240, 36484,
                3336,  11288, 44384, 25712, 29228, 13512, 7220,  24464, 63284,
                14224, 39620, 58360, 35556, 4408,  40976, 48912, 11168, 45256,
                57292, 17132, 31424, 24136, 43788, 18520, 64596, 52600, 12708,
                20588, 57416, 3896,  26860, 61432, 7188,  25300, 46872, 36700,
                55936, 44096, 59764, 40316, 29276, 46724, 23432, 39048, 45624,
                59088, 24860, 64544, 33912, 9940,  20732, 7532,  35476, 40008,
                58008, 32620, 49368, 22952, 26504, 54792, 19236, 62540, 13420,
                7808,  28956, 52648, 17684, 64132, 12772, 24180, 15372, 63504,
                39080, 46708, 14724, 24864, 31232, 47484, 12072, 62516, 19380,
                33196, 39496, 57624, 50380, 32000, 15880, 41628, 20688, 34380,
                58808, 65268, 25000, 36188, 60240, 10740, 31748, 18576, 47376,
                27992, 37864, 31248, 11976, 35700, 52408, 63812, 2368,  25104,
                36232, 49384, 5124,  16552, 59896, 50088, 24928, 3756,  32356,
                26844, 37212,
            },
            {
                49952, 33104, 6656,  58888, 2932,  37964, 61392, 20764, 29812,
                49688, 15800, 53976, 38824, 65376, 45908, 21488, 36084, 53536,
                18000, 26284, 45096, 20848, 52560, 30600, 18960, 6876,  26948,
                50300, 41988, 9620,  52212, 33708, 14972, 37676, 2616,  42416,
                29636, 16416, 33060, 40512, 14296, 10240, 42812, 16544, 4808,
                22764, 31464, 2380,  8848,  20440, 33212, 51688, 63356, 7568,
                20152, 40636, 29556, 4256,  52336, 41856, 48876, 2184,  53028,
                15592, 44400, 64944, 12336, 59372, 43236, 244,   34516, 49956,
                45948, 33036, 10232, 36628, 5500,  45276, 33848, 51844, 18864,
                5308,  56088, 34716, 64320, 40068, 16660, 43292, 284,   48888,
                5652,  42444, 10572, 28152, 64704, 8088,  56472, 26188, 2528,
                17648, 9984,  40860, 3680,  52404, 22428, 63992, 39892, 4216,
                59836, 16244, 24316, 7164,  46404, 26552, 4824,  33244, 18428,
                56636, 64936, 32068, 1172,  41020, 29644, 13600, 62348, 56432,
                15664, 64136,
            },
            {
                12560, 57396, 19396, 27968, 30996, 51336, 13880, 5068,  57012,
                60172, 22976, 35004, 9044,  2740,  33092, 58700, 10984, 4776,
                48152, 8520,  64872, 1076,  13404, 38404, 58884, 62588, 34824,
                21208, 13720, 63856, 1876,  27820, 7144,  57932, 25720, 47556,
                60464, 8100,  53524, 22040, 50564, 56996, 32184, 62424, 52828,
                12428, 50152, 64968, 36932, 58696, 5540,  12564, 17452, 35072,
                53604, 5744,  12972, 46324, 17536, 63180, 14100, 30644, 21896,
                8412,  27808, 3608,  35220, 6460,  30488, 16172, 55788, 27044,
                20196, 3040,  61684, 47640, 26280, 59688, 55236, 9280,  26644,
                31808, 21732, 11260, 53968, 7528,  28120, 53024, 59024, 30008,
                56184, 17084, 22492, 4464,  37192, 47528, 12108, 62228, 45600,
                37936, 54996, 13816, 33272, 45348, 7424,  26756, 13012, 54956,
                21724, 42612, 2068,  62752, 40280, 20716, 14576, 59652, 41272,
                8536,  13316, 45964, 54068, 19232, 8696,  36556, 43568, 22056,
                39876, 4912,
            },
            {
                47924, 25224, 53920, 43120, 65048, 8272,  34376, 45200, 39616,
                680,   42556, 19364, 61420, 48396, 16588, 27444, 43532, 61020,
                31288, 41756, 28796, 49264, 55972, 25708, 9396,  43924, 2588,
                53988, 29712, 38284, 59816, 45496, 54156, 19868, 32168, 5644,
                38620, 11780, 45528, 63336, 1036,  38176, 20796, 27304, 34920,
                41660, 18840, 25108, 15340, 48420, 55180, 42736, 26548, 57472,
                31224, 48544, 61072, 22836, 32592, 25972, 58308, 45896, 37112,
                56080, 41680, 51100, 20088, 47500, 39764, 23576, 64236, 5764,
                41512, 57728, 23128, 39508, 15912, 30560, 1588,  40004, 48752,
                61660, 42596, 1904,  45484, 19940, 37144, 23224, 9968,  35612,
                25464, 52308, 44476, 59316, 19460, 53188, 29996, 22840, 4792,
                51276, 27356, 48196, 62976, 19160, 56468, 36864, 50200, 33972,
                9284,  45876, 51764, 32852, 55652, 9432,  49760, 44880, 52644,
                29388, 21744, 38100, 26020, 61716, 57044, 46876, 1544,  52700,
                10004, 29800,
            },
            {
                41648, 36624, 4,     16372, 11516, 23936, 49300, 18500, 26144,
                12684, 51832, 31192, 5292,  55944, 40476, 51476, 0,     37436,
                20080, 57024, 15756, 34060, 6368,  46080, 16456, 32092, 39788,
                5928,  18548, 48452, 22204, 12612, 40808, 49132, 61572, 17228,
                55344, 28136, 24504, 34744, 17644, 6108,  54864, 49004, 8064,
                3320,  57260, 45264, 28668, 1600,  22080, 38540, 3576,  10128,
                43556, 16100, 36900, 56580, 2688,  39408, 10924, 5780,  61232,
                17696, 24724, 62300, 14668, 57452, 52276, 9148,  44592, 36356,
                15300, 50716, 11788, 54204, 7688,  21064, 44184, 12520, 35888,
                57384, 15716, 29728, 59788, 49588, 4496,  63332, 14688, 46796,
                3672,  63744, 13672, 34148, 1324,  40104, 15112, 43840, 60208,
                32104, 21084, 8240,  2364,  28916, 42796, 4784,  17172, 62008,
                29396, 57460, 14320, 18972, 27852, 61460, 36120, 22996, 556,
                16364, 60348, 5948,  42672, 11660, 32604, 15976, 26540, 34568,
                18824, 60600,
            },
            {
                22744, 9200,  61772, 46088, 39272, 55968, 60712, 32192, 7244,
                64128, 47100, 24212, 10772, 28860, 20808, 14308, 25024, 54008,
                12360, 3260,  23920, 60112, 36984, 21640, 64212, 51244, 23752,
                60800, 56640, 8488,  35064, 3728,  25036, 9448,  36016, 616,
                42936, 50004, 4424,  59148, 29832, 44080, 11672, 15496, 60524,
                30564, 37804, 9788,  53428, 34256, 61332, 30556, 64620, 49648,
                20776, 1392,  27516, 8916,  44876, 54364, 19792, 28816, 49800,
                984,   32748, 9932,  37648, 28112, 2264,  32360, 12664, 59284,
                29420, 1048,  31744, 65064, 34964, 49652, 62528, 18032, 3936,
                23664, 7028,  38200, 25276, 12408, 55360, 32856, 41592, 19604,
                38696, 7980,  31480, 49732, 25016, 63032, 6080,  35720, 10968,
                16616, 42040, 57444, 39348, 14700, 60364, 23996, 47712, 468,
                41212, 25628, 5696,  38988, 3156,  12252, 31336, 6800,  39500,
                55000, 34312, 50292, 2696,  23592, 48860, 6452,  65532, 57772,
                50436, 3104,
            },
            {
                17332, 52380, 31836, 26912, 5496,  21240, 1752,  42124, 54480,
                36332, 15628, 59792, 38232, 44904, 62564, 7180,  34948, 63904,
                44360, 50428, 40572, 10408, 52904, 1516,  28488, 11728, 47288,
                14496, 27272, 44108, 31448, 55268, 62772, 14564, 30356, 23444,
                64272, 19992, 13908, 41112, 48384, 65312, 25644, 40084, 45988,
                23392, 62944, 13888, 42052, 6868,  18316, 46016, 14440, 24520,
                59384, 39848, 63396, 50800, 34508, 15648, 65384, 43172, 35520,
                13436, 53984, 46288, 5028,  22224, 60756, 18380, 54304, 47052,
                25048, 38648, 19008, 42828, 4304,  24064, 57980, 29092, 52912,
                45768, 50468, 65228, 34116, 17800, 43592, 28964, 2396,  61028,
                51208, 27868, 57756, 42104, 10224, 56280, 27564, 49400, 55408,
                196,   62080, 24760, 30804, 50704, 9592,  54488, 31828, 11536,
                20356, 36572, 65528, 46592, 54056, 42836, 57676, 25364, 62656,
                46228, 12748, 27404, 63436, 36952, 53368, 20900, 40852, 13044,
                31472, 45032,
            },
            {
                63352, 35548, 12244, 58516, 47784, 37100, 29544, 17020, 10180,
                27592, 3312,  53156, 34068, 1948,  49488, 57308, 29916, 17508,
                5552,  32908, 27528, 62720, 18928, 42928, 55688, 5180,  33680,
                41332, 400,   65464, 16812, 50384, 38104, 44652, 57272, 52244,
                7224,  37032, 57780, 9836,  21264, 32956, 3856,  56216, 17396,
                188,   50700, 20880, 58296, 26348, 11548, 51360, 35580, 7960,
                53884, 32020, 12680, 21856, 6136,  25316, 52228, 3780,  22356,
                58644, 41276, 30120, 64632, 49248, 42868, 34980, 4244,  21132,
                63392, 7300,  52628, 46688, 27340, 14048, 40888, 8624,  33348,
                20268, 10712, 41240, 820,   58432, 8952,  48212, 24832, 54436,
                11772, 22244, 16220, 3352,  20812, 45652, 17296, 38736, 23608,
                33700, 52352, 12304, 45780, 35456, 21672, 38644, 63152, 44100,
                58396, 50684, 8288,  33616, 21548, 15296, 48108, 17984, 9944,
                30028, 20036, 58104, 8308,  17860, 44192, 29320, 4252,  37572,
                24960, 7492,
            },
            {
                28312, 43304, 3528,  19156, 54040, 13976, 63060, 50868, 59072,
                45924, 21876, 41636, 18632, 13724, 23308, 9912,  39556, 47628,
                58824, 21872, 8136,  46580, 13696, 31276, 38084, 59324, 18252,
                52588, 36968, 10908, 20648, 6144,  27856, 2336,  18900, 11552,
                32760, 46156, 28740, 1396,  53508, 37520, 7880,  51440, 27940,
                36248, 32532, 5376,  46960, 39072, 55332, 832,   41196, 28344,
                4420,  17876, 47180, 42744, 59164, 30840, 38760, 9324,  48036,
                26676, 6796,  19332, 11676, 38936, 15668, 26496, 51308, 10940,
                41900, 57028, 16420, 9948,  60504, 55084, 36,    37384, 63272,
                55768, 14488, 27504, 51804, 22144, 62764, 15744, 6284,  34236,
                43972, 37216, 47692, 65316, 35116, 30912, 7884,  58876, 4412,
                47272, 19660, 6844,  65040, 1632,  17472, 5880,  27700, 14096,
                3304,  16848, 24112, 60584, 28724, 1416,  63940, 37224, 51252,
                4672,  43348, 40032, 32016, 13892, 60880, 56048, 10536, 46384,
                59640, 53836,
            },
            {
                14636, 39428, 50092, 25256, 33796, 7416,  44224, 24140, 33248,
                5396,  56376, 8816,  64928, 32024, 54740, 42916, 26516, 2308,
                52580, 15248, 36196, 56692, 3860,  24332, 48992, 9260,  22500,
                61644, 25648, 46908, 58784, 42344, 22848, 60256, 47804, 40588,
                25828, 55096, 16956, 61160, 24308, 14588, 62364, 19736, 59248,
                9596,  43800, 64736, 15776, 29828, 60744, 22700, 63012, 44992,
                58336, 37088, 9204,  56028, 40,    13756, 62568, 33376, 17320,
                60552, 36480, 56324, 24496, 452,   58916, 7920,  61540, 37256,
                28892, 1804,  33332, 36648, 20584, 30740, 47512, 17048, 25540,
                3428,  31272, 44984, 5840,  36432, 30432, 39372, 56980, 18652,
                904,   59436, 6976,  53504, 12956, 50676, 61880, 41728, 14356,
                28944, 37228, 43488, 26500, 53840, 59168, 47048, 52092, 34788,
                55672, 30992, 48324, 11088, 53148, 41128, 5392,  32624, 56448,
                23876, 2084,  53960, 48124, 25700, 356,   34400, 49604, 16664,
                20552, 1260,
            },
            {
                56300, 22116, 64116, 9708,  60136, 40516, 2316,  19892, 12532,
                39768, 28980, 49960, 25528, 46416, 5040,  62152, 20100, 12200,
                38216, 65252, 29244, 50820, 41176, 62088, 15728, 45136, 2928,
                30596, 7448,  54940, 33440, 13588, 51596, 34440, 4564,  15564,
                62884, 3220,  42844, 35204, 49868, 45020, 39640, 31476, 47892,
                12928, 55064, 24796, 2832,  35080, 19224, 9768,  32916, 15092,
                20556, 49268, 24996, 29136, 40400, 49624, 19516, 55032, 43884,
                2460,  14716, 45312, 51636, 33752, 54648, 43992, 31244, 14652,
                49872, 23748, 62196, 44728, 5020,  51540, 11608, 43016, 59480,
                48976, 38552, 57904, 18356, 54132, 46164, 10788, 50016, 63652,
                32060, 25592, 29344, 40040, 23132, 1288,  26216, 18968, 52908,
                63312, 9268,  57836, 15268, 40908, 32972, 24696, 9020,  41992,
                21028, 39364, 6016,  44492, 35960, 19388, 26588, 44968, 14456,
                61760, 35348, 16444, 64768, 7096,  39292, 22548, 27816, 62808,
                41392, 33528,
            },
            {
                47668, 5848,  30852, 45820, 16128, 28540, 57588, 48576, 63684,
                35624, 15292, 60792, 884,   38032, 17120, 35400, 59408, 31220,
                45192, 24868, 480,   19680, 7516,  34084, 27176, 56260, 35804,
                50144, 15244, 39656, 1788,  29616, 9648,  64628, 37560, 30532,
                49040, 10212, 22136, 6660,  29536, 10944, 1908,  25980, 5096,
                41492, 21944, 38108, 53060, 48240, 6268,  50288, 26976, 53216,
                2168,  64372, 34188, 16596, 61364, 23168, 5156,  27776, 11312,
                50604, 30668, 61904, 8988,  28380, 17692, 22884, 3408,  46184,
                19304, 55264, 12712, 26536, 39856, 64076, 22768, 34328, 6632,
                21320, 9840,  62288, 24240, 13184, 2760,  27280, 21468, 41712,
                13900, 45572, 10380, 18080, 57928, 43628, 36788, 10976, 32732,
                1868,  48696, 20724, 30216, 3180,  11640, 19532, 64124, 924,
                59888, 26336, 63348, 14944, 57376, 8056,  59872, 11388, 21088,
                48904, 28516, 10648, 20396, 45420, 51608, 58196, 3956,  8184,
                51960, 11272,
            },
            {
                58772, 37948, 18264, 1496,  51568, 36936, 10676, 26048, 53052,
                3108,  43276, 20944, 11296, 52876, 27920, 7872,  50104, 4044,
                55368, 10272, 43028, 58220, 12588, 52376, 20924, 64564, 12044,
                43344, 19732, 63320, 24248, 45856, 56604, 20212, 7736,  53748,
                24620, 59624, 39020, 57080, 18344, 64588, 53020, 56476, 15948,
                61180, 28940, 8544,  60088, 14232, 42712, 57736, 39468, 12440,
                42300, 5828,  54752, 11760, 7816,  46364, 36036, 41692, 64452,
                23544, 39464, 20096, 4068,  41088, 48292, 11960, 62744, 39120,
                5312,  59220, 7504,  48744, 17772, 2376,  57144, 28512, 53084,
                16032, 42024, 1380,  33184, 42956, 61452, 35524, 8084,  51848,
                4680,  55312, 61340, 48392, 33284, 5968,  54992, 46128, 60008,
                23904, 39740, 35276, 51588, 61308, 45232, 37576, 49280, 29860,
                12676, 51460, 3648,  32948, 23280, 50056, 30672, 38488, 54572,
                6380,  41808, 59528, 36668, 29652, 12608, 18300, 42900, 36832,
                30516, 24456,
            },
            {
                13692, 27048, 61108, 34320, 55520, 21652, 41772, 5724,  17832,
                30780, 58628, 47504, 32692, 57880, 44232, 13172, 40972, 22088,
                16808, 60840, 35252, 47480, 31112, 40112, 2152,  6012,  38912,
                28200, 59948, 4128,  52460, 16176, 41140, 27092, 43680, 17544,
                12756, 45472, 31548, 51796, 13548, 41708, 23264, 36316, 32372,
                50416, 1208,  18292, 44780, 31068, 25160, 3196,  36032, 60596,
                23096, 31396, 44428, 38416, 58524, 52496, 31532, 1748,  57540,
                6480,  34620, 55124, 13632, 64020, 36772, 57832, 32964, 51164,
                25772, 41696, 35208, 30004, 54328, 9460,  38420, 13408, 45880,
                64796, 36072, 26672, 49312, 56536, 17024, 47204, 58484, 24072,
                38400, 20076, 36116, 3148,  15504, 64160, 21312, 28220, 16388,
                5224,  56320, 13064, 6692,  23204, 54540, 4616,  56908, 17252,
                43832, 36596, 18924, 54696, 41604, 88,    64880, 46944, 3204,
                32988, 24352, 52304, 1824,  55708, 33112, 60960, 48592, 15456,
                64684, 3112,
            },
            {
                45520, 53168, 42340, 7620,  13236, 64828, 32316, 45236, 61672,
                9156,  23468, 39152, 6720,  19188, 64228, 24264, 53916, 33264,
                48296, 27812, 4896,  23164, 17220, 59540, 44716, 24124, 54448,
                32928, 8652,  48316, 35668, 11868, 59396, 28,    50580, 61748,
                36152, 2072,  5836,  20600, 34480, 2888,  46752, 9816,  20156,
                45752, 64036, 34224, 56008, 10532, 21224, 65284, 17432, 8292,
                27760, 48104, 19776, 1136,  26480, 17524, 13332, 21112, 47256,
                16192, 27308, 49568, 43816, 25228, 6688,  20940, 1188,  29248,
                10316, 15876, 21152, 62608, 43400, 32608, 24888, 60716, 5436,
                29932, 51644, 11700, 7512,  20656, 28856, 3688,  31724, 15040,
                64764, 28672, 9188,  25352, 52064, 30436, 8528,  40648, 51124,
                43404, 26400, 64732, 17900, 42436, 28048, 14752, 33768, 25476,
                6592,  58844, 46572, 9124,  28280, 16968, 12504, 25680, 18416,
                62684, 9256,  15872, 44164, 5316,  23048, 9652,  26396, 54288,
                21100, 40392,
            },
            {
                32384, 9860,  19900, 47848, 24988, 4416,  49776, 14852, 36672,
                55720, 27292, 13792, 50836, 3876,  30256, 37196, 1080,  62268,
                8700,  38860, 14344, 64164, 54868, 28760, 10688, 51188, 14428,
                18120, 57692, 25988, 21200, 31356, 38456, 6116,  22284, 32708,
                56296, 26600, 63156, 48132, 58392, 28680, 61700, 6708,  58044,
                12276, 37700, 4964,  27376, 39836, 52696, 46208, 32428, 51372,
                56560, 14736, 62592, 50784, 35088, 63500, 43308, 54180, 37540,
                61000, 10560, 3332,  59612, 31060, 52772, 16492, 55704, 44088,
                65304, 53516, 47036, 652,   14404, 50312, 19696, 41444, 2896,
                17612, 22832, 55388, 39932, 63700, 37440, 10412, 53716, 41320,
                440,   49192, 43296, 59908, 39236, 46696, 13352, 62120, 2892,
                31104, 10704, 46920, 32364, 36728, 9456,  62204, 47572, 40052,
                10828, 22904, 31012, 62260, 37336, 44616, 53104, 35172, 42976,
                56616, 39424, 49968, 27260, 64248, 41008, 46884, 1132,  34996,
                6936,  56868,
            },
            {
                23668, 488,   63228, 30236, 39068, 57476, 28228, 19368, 51792,
                84,    41264, 62972, 34484, 55188, 10632, 46600, 15392, 20148,
                57048, 42132, 50748, 1372,  37024, 7860,  34580, 62624, 46568,
                744,   38048, 44036, 5084,  61236, 54644, 28620, 47696, 7412,
                40544, 15384, 43192, 9488,  23916, 16912, 38768, 52160, 26804,
                43092, 23840, 15440, 49484, 62100, 312,   13764, 6000,  43148,
                37376, 3604,  10336, 29928, 22192, 4992,  9484,  25020, 6772,
                29224, 40536, 17964, 22292, 35704, 11220, 47508, 38728, 7672,
                23420, 37116, 5048,  27952, 58112, 6788,  55212, 47608, 36992,
                58868, 32844, 46360, 15264, 5260,  50664, 44300, 60120, 18940,
                33760, 54772, 12188, 21824, 34628, 1196,  19344, 57148, 37260,
                22200, 53676, 59216, 288,   50644, 57560, 2812,  19644, 53740,
                65120, 50216, 2272,  14108, 57580, 4656,  22184, 59772, 7172,
                29492, 21456, 11904, 58036, 18116, 37664, 14376, 29844, 62264,
                49864, 16972,
            },
            {
                59392, 36088, 50844, 16152, 44364, 2064,  10032, 60508, 33464,
                6236,  21632, 48560, 17072, 42816, 25420, 60012, 52040, 6220,
                29440, 24576, 11256, 19256, 43420, 49188, 25860, 19972, 41288,
                29976, 65208, 10400, 50364, 17768, 41580, 13956, 64460, 10952,
                19220, 54432, 30396, 36852, 50228, 4016,  42284, 14184, 31044,
                2640,  54612, 59604, 8236,  19808, 35644, 29400, 59268, 22724,
                18492, 60336, 41408, 46744, 39164, 56380, 59956, 49100, 32332,
                52000, 64976, 45748, 57132, 1996,  41896, 27392, 60952, 19080,
                33320, 11996, 60312, 40024, 23908, 34820, 10680, 26900, 62400,
                8564,  43280, 600,   58096, 31364, 25760, 21900, 13144, 27508,
                7344,  62292, 16708, 4984,  58400, 53344, 24776, 33148, 49404,
                7780,  18616, 39036, 14500, 25280, 21812, 45028, 30444, 5720,
                26964, 35592, 20576, 40284, 48556, 27124, 10340, 50440, 15156,
                1028,  47164, 35664, 31572, 7840,  53928, 59596, 20032, 43312,
                11392, 38604,
            },
            {
                28992, 5864,  54372, 11624, 61836, 22300, 37476, 42360, 24740,
                45740, 12092, 59472, 28708, 2052,  20932, 38772, 32004, 44276,
                35648, 63040, 53476, 32872, 61044, 15192, 2728,  55452, 5484,
                22968, 13356, 56224, 33588, 2356,  24596, 45684, 34524, 51432,
                25584, 60044, 1060,  13128, 65352, 32660, 56084, 21328, 63452,
                48208, 17880, 32764, 45448, 41888, 24260, 54336, 48924, 9540,
                34152, 25692, 54808, 13032, 2412,  28060, 15684, 41832, 19448,
                292,   12984, 33404, 8680,  50904, 63384, 14188, 4392,  57908,
                49340, 30640, 51956, 18088, 45716, 64504, 15844, 31580, 21556,
                13232, 53340, 24556, 18820, 36360, 61864, 2180,  48672, 39308,
                45148, 24184, 31136, 47468, 41220, 28852, 10352, 45352, 15048,
                62692, 28456, 44540, 4840,  63360, 41108, 35076, 12380, 60668,
                15552, 43428, 56144, 7828,  17916, 34072, 61656, 31372, 37980,
                55072, 63520, 4296,  24912, 49072, 2512,  33532, 51604, 4124,
                25328, 46524,
            },
            {
                14120, 21256, 41644, 26716, 31908, 48476, 56564, 14008, 54120,
                65232, 31292, 7032,  36808, 57216, 8992,  64612, 12936, 2496,
                16872, 47220, 3640,  27560, 22632, 57616, 38248, 32436, 60204,
                52436, 36588, 47084, 28260, 62360, 8468,  20364, 58496, 2808,
                38932, 44388, 22588, 52976, 8012,  25844, 46376, 10644, 36076,
                6148,  39136, 28436, 11844, 57208, 4176,  16348, 39016, 63720,
                5340,  51592, 31688, 20316, 64712, 34352, 45588, 11000, 36800,
                23256, 55192, 26276, 38476, 19968, 30296, 24252, 34748, 45544,
                2448,  15140, 42692, 9136,  3532,  38196, 52936, 1564,  49444,
                39516, 29076, 65428, 6764,  40904, 52384, 9632,  34832, 57864,
                3280,  51456, 37288, 8484,  65128, 17804, 4012,  60280, 41876,
                5592,  35824, 55976, 30812, 10268, 52756, 17224, 49512, 38956,
                964,   31812, 51908, 24572, 64040, 45208, 2508,  41620, 19432,
                26368, 43940, 16528, 61040, 41924, 22140, 12992, 27684, 55536,
                8280,  65448,
            },
            {
                49076, 57316, 2568,  35112, 7820,  18156, 5052,  29328, 2724,
                18716, 39060, 53032, 14936, 44548, 51152, 27180, 48952, 55692,
                21952, 59440, 9388,  40416, 45360, 6840,  12284, 48180, 17124,
                26864, 7368,  20760, 15580, 43144, 53056, 37240, 29292, 16164,
                6372,  32480, 47492, 17348, 38444, 59040, 19004, 484,   52868,
                23304, 61320, 50880, 2352,  64192, 31764, 46628, 12764, 28308,
                44332, 728,   16484, 48660, 7948,  24008, 52892, 4544,  62900,
                58956, 43244, 16204, 48220, 5616,  54928, 43548, 9692,  53776,
                21288, 26556, 62168, 54600, 28976, 19440, 60032, 42028, 5796,
                57660, 34176, 10584, 47884, 14276, 56304, 17616, 28736, 63800,
                14732, 20280, 56036, 26168, 13216, 50668, 35268, 54760, 26616,
                22532, 12456, 48840, 19812, 59912, 26784, 7604,  55552, 23108,
                47268, 58436, 11744, 4060,  29304, 13808, 54420, 23452, 58812,
                12656, 8268,  52488, 37140, 10280, 46100, 63756, 39748, 17504,
                42388, 32112,
            },
            {
                37424, 18868, 45260, 59572, 51872, 64008, 44608, 35816, 50268,
                9184,  26356, 47128, 23252, 32744, 4648,  18620, 40804, 7616,
                28840, 37740, 14092, 50244, 18084, 64832, 29048, 41752, 10084,
                44536, 63664, 1072,  39888, 59308, 11992, 3892,  55960, 49684,
                63124, 10444, 61172, 28244, 4740,  43796, 34004, 29296, 57328,
                41688, 8888,  14844, 21756, 37744, 25768, 7440,  56436, 21024,
                35988, 60740, 57668, 41984, 38080, 56056, 17824, 31152, 49708,
                7656,  28616, 2768,  61984, 12160, 59404, 37432, 16688, 64744,
                39720, 7552,  36436, 22628, 47168, 33780, 11604, 24080, 45176,
                16540, 20192, 51328, 26728, 44644, 30968, 5088,  23424, 41420,
                10816, 33276, 60520, 1436,  44752, 38964, 20024, 31604, 724,
                52252, 64324, 39864, 1816,  46620, 32736, 3512,  64428, 28664,
                9224,  18796, 37684, 42276, 60040, 35836, 9092,  47352, 5112,
                49816, 33932, 28652, 19104, 56000, 30608, 6484,  34908, 58132,
                22872, 3492,
            },
            {
                62352, 9228,  25852, 12912, 21728, 38756, 15892, 23680, 61208,
                41116, 58032, 1008,  62728, 11412, 59048, 36204, 23820, 63324,
                43896, 33576, 56792, 4508,  31404, 54116, 24396, 51860, 3436,
                32876, 55152, 49748, 30408, 25196, 48504, 32312, 22280, 41012,
                26432, 19464, 35396, 55104, 14412, 49504, 60868, 11924, 16412,
                47656, 26988, 34856, 44440, 52504, 17424, 61100, 50588, 40168,
                10300, 24380, 29072, 11972, 3440,  61224, 27232, 13516, 39700,
                20380, 34600, 53308, 40656, 31940, 22248, 1640,  28572, 50460,
                32152, 56492, 12,    13656, 57856, 6528,  50812, 27896, 63212,
                36256, 4716,  60984, 1956,  21344, 59400, 37484, 45640, 54256,
                49256, 5456,  43000, 22432, 29508, 63072, 7296,  58024, 47248,
                17380, 8936,  33948, 23932, 15488, 42728, 37036, 20428, 44356,
                34428, 61828, 26068, 49452, 20928, 52232, 16884, 39180, 30692,
                64644, 40876, 59964, 572,   24200, 15596, 51296, 1964,  47396,
                11496, 53004,
            },
            {
                15364, 33872, 55644, 40912, 1116,  28592, 10220, 54684, 32400,
                13228, 20896, 35332, 17532, 42600, 54168, 30288, 15204, 276,
                51724, 19172, 25380, 62428, 35944, 836,   20820, 61516, 36988,
                18812, 23396, 10728, 35844, 4760,  18204, 64824, 14260, 45516,
                7460,  57528, 42428, 1800,  21364, 40268, 24108, 6816,  37516,
                64600, 1596,  55356, 59748, 4732,  42292, 33644, 1536,  15460,
                54224, 6648,  45140, 32924, 22012, 46496, 35916, 1024,  44252,
                57840, 10556, 46888, 25156, 17836, 51736, 45848, 6352,  12652,
                18980, 48460, 43676, 30880, 64172, 40580, 15516, 55852, 9368,
                30744, 54556, 38924, 42804, 33388, 8428,  62632, 8,     18920,
                26632, 36476, 16424, 53228, 48144, 11132, 15448, 24900, 37156,
                43536, 28360, 61116, 53908, 57684, 11348, 50528, 59232, 13992,
                5248,  53724, 15772, 7360,  412,   32568, 58108, 26944, 2212,
                20580, 14960, 5964,  48056, 43424, 61696, 38272, 20072, 60084,
                26908, 29924,
            },
            {
                50576, 4728,  46748, 31256, 61796, 49660, 58356, 3592,  45776,
                5856,  49060, 55468, 28104, 3712,  49772, 9272,  46816, 60268,
                12308, 6032,  47904, 10524, 40012, 15632, 45108, 8156,  58536,
                40824, 14572, 62092, 56708, 44352, 8688,  39360, 34292, 304,
                52248, 13072, 30124, 51036, 9688,  62704, 31444, 51904, 44920,
                20680, 32540, 12884, 19548, 30180, 10884, 22368, 27320, 45992,
                62376, 18888, 52024, 64112, 14820, 51008, 9340,  54544, 65276,
                23044, 15336, 4232,  63432, 7952,  35460, 57576, 62924, 41828,
                23316, 60096, 10480, 17192, 24768, 3844,  37468, 21828, 788,
                47488, 14140, 24232, 11400, 55352, 16672, 50936, 12592, 29964,
                64396, 57324, 9588,  32396, 3744,  55792, 41484, 60748, 3064,
                13204, 49936, 6132,  18856, 4140,  29696, 25240, 40232, 1716,
                46720, 30676, 39752, 65468, 43876, 23100, 12192, 62488, 45408,
                54568, 25208, 36164, 56956, 9160,  27420, 33180, 13620, 7660,
                44640, 39828,
            },
            {
                20640, 64188, 23460, 17416, 6756,  42604, 34012, 19672, 26928,
                63372, 30792, 8316,  39608, 21988, 65008, 25804, 41736, 32268,
                38040, 53752, 21540, 43136, 59928, 26940, 49824, 12752, 29732,
                47584, 6152,  51692, 27384, 21336, 60708, 53844, 28292, 59724,
                23568, 37464, 64216, 25552, 46436, 3488,  18100, 56328, 4276,
                25288, 40792, 49836, 7928,  46984, 57400, 65364, 49608, 37616,
                31004, 3036,  35584, 40752, 5520,  59224, 25320, 18420, 29592,
                48508, 32640, 42264, 55184, 28068, 39444, 14648, 25688, 33792,
                2840,  37932, 27596, 53200, 45536, 49796, 32544, 61496, 43748,
                52556, 19132, 64316, 29668, 47824, 25968, 39572, 34888, 46692,
                6652,  40736, 24048, 61524, 37888, 20980, 27732, 34228, 52584,
                21700, 31848, 38880, 45424, 35712, 63888, 8600,  54896, 17888,
                24028, 57464, 9920,  27624, 47716, 56244, 36920, 6984,  42084,
                10144, 51136, 31516, 12696, 21668, 53820, 41972, 64852, 23948,
                55364, 184,
            },
            {
                57944, 8724,  38168, 53520, 12128, 24880, 14896, 52944, 37748,
                43032, 16468, 59548, 45284, 13932, 37076, 5568,  20252, 16360,
                58016, 27468, 30748, 2416,  56284, 33016, 5108,  54692, 35016,
                25480, 1428,  42216, 31868, 16264, 2648,  49492, 10508, 18484,
                47344, 4948,  16788, 33568, 39316, 58276, 35924, 28580, 10596,
                60384, 15824, 63160, 28052, 39172, 35180, 4024,  13504, 8260,
                25180, 56652, 11332, 27844, 20828, 31824, 42968, 38900, 7376,
                2480,  37020, 59076, 11888, 20908, 948,   48380, 9604,  55480,
                51600, 5412,  62504, 34936, 8772,  19588, 58612, 12540, 26364,
                39756, 33968, 2716,  6860,  58608, 20612, 3124,  56532, 22124,
                15252, 51532, 2156,  45316, 17360, 50412, 4860,  45996, 9304,
                65328, 56168, 25764, 12064, 52116, 20704, 47600, 32512, 62992,
                36680, 51420, 19684, 35188, 14544, 3972,  18828, 50000, 33324,
                17552, 58632, 38976, 63128, 46520, 5548,  2772,  49792, 31216,
                17040, 35060,
            },
            {
                13940, 28712, 44152, 60736, 35744, 48596, 64680, 8800,  628,
                23272, 11804, 51132, 2160,  33564, 56444, 52572, 61584, 1728,
                45464, 8664,  64752, 13552, 17736, 23244, 38884, 62620, 16948,
                20456, 65520, 38152, 12548, 46728, 36732, 25076, 41852, 31648,
                56488, 43672, 8172,  54296, 11856, 22868, 14696, 42780, 47952,
                53700, 36676, 5988,  52692, 23852, 16896, 55148, 20224, 41224,
                59632, 43912, 17288, 47620, 55756, 1620,  49680, 12988, 61764,
                52616, 25944, 17568, 50116, 31008, 44316, 61712, 18212, 29580,
                43412, 15840, 21168, 57420, 2216,  42268, 29220, 5844,  17448,
                8396,  60152, 45324, 36060, 52096, 14424, 44172, 62240, 9892,
                32688, 59256, 26380, 11616, 30480, 63548, 13984, 58404, 18692,
                40312, 2340,  16224, 61692, 42116, 756,   14840, 27200, 6112,
                12300, 43024, 2420,  59212, 31312, 52992, 24932, 61056, 28408,
                4500,  22896, 1280,  29140, 16260, 35364, 19084, 57128, 10768,
                48084, 42196,
            },
            {
                25164, 50048, 1608,  19500, 6208,  27588, 39780, 46096, 58880,
                54408, 35696, 25456, 62144, 17392, 29552, 10456, 48060, 34928,
                23380, 40196, 50264, 35788, 46168, 52448, 10036, 2784,  43256,
                49448, 56124, 8196,  52596, 57912, 6400,  64064, 14676, 3676,
                62496, 20988, 28168, 60728, 2116,  50456, 65032, 7628,  33052,
                564,   22016, 45580, 12484, 1924,  60068, 44296, 32220, 53044,
                140,   22748, 61296, 8808,  36528, 63068, 16328, 34340, 56776,
                21144, 45660, 8416,  64408, 5868,  53380, 36620, 22604, 58816,
                7580,  39968, 47188, 31400, 14324, 55112, 37048, 65096, 51304,
                56264, 22296, 27412, 10928, 41072, 31968, 4908,  28032, 49124,
                42944, 19416, 38480, 55424, 34712, 7908,  43084, 24104, 33040,
                28192, 48248, 7684,  30484, 22992, 59388, 34608, 40608, 60784,
                45720, 22752, 7956,  62580, 41788, 10912, 38856, 45760, 13776,
                64948, 41204, 44556, 8612,  53068, 59824, 40944, 26608, 37844,
                6408,  61752,
            },
            {
                33384, 11064, 59144, 31116, 56368, 3088,  21400, 32308, 18280,
                29288, 4240,  40688, 47276, 7668,  22260, 39244, 26748, 13784,
                55256, 3344,  19744, 60356, 6472,  41700, 31252, 59276, 27296,
                11344, 30712, 23584, 18916, 33696, 28828, 22128, 54548, 38952,
                34764, 12568, 48484, 40600, 31324, 45144, 19824, 26544, 55980,
                17176, 29572, 57752, 33608, 40584, 26508, 9824,  48364, 28560,
                15560, 38944, 51072, 30452, 4052,  45340, 24360, 10296, 28264,
                4620,  40200, 14704, 34208, 24404, 41120, 4264,  12528, 32868,
                50140, 24448, 11292, 63556, 26808, 48536, 23116, 1332,  41568,
                31536, 15168, 49412, 63460, 18236, 59728, 54916, 23208, 35768,
                912,   65100, 5952,  47672, 21600, 52780, 38748, 444,   55128,
                11164, 43684, 58260, 37716, 49372, 10028, 53948, 3552,  49992,
                16272, 30188, 54832, 26416, 49212, 21416, 1068,  54176, 7940,
                34960, 55824, 19424, 49436, 24784, 30700, 13276, 63488, 21580,
                3584,  54148,
            },
            {
                46172, 22156, 37596, 15076, 41308, 50816, 61548, 13788, 9956,
                48792, 63840, 20000, 31864, 58328, 43964, 51468, 4596,  63224,
                43204, 32752, 10608, 28932, 25612, 63768, 21228, 15308, 46588,
                35928, 4220,  61396, 43860, 604,   59524, 45984, 8448,  50772,
                25796, 1216,  52952, 17376, 24748, 5876,  36264, 61924, 39404,
                11244, 42524, 62560, 19688, 50240, 64264, 4572,  36828, 62068,
                6916,  34108, 12056, 25948, 54196, 19512, 39380, 51628, 44248,
                31612, 61128, 57492, 47960, 29024, 60324, 16600, 46544, 65344,
                588,   54628, 36208, 4540,  44188, 9100,  18564, 34364, 11560,
                46224, 4520,  37908, 568,   24904, 8352,  38224, 12228, 17292,
                53880, 29676, 14608, 58576, 3636,  28624, 60480, 16652, 49976,
                63176, 20320, 33728, 4484,  17632, 44760, 29356, 24552, 19308,
                56704, 39040, 34132, 15396, 4868,  36216, 58020, 30544, 16648,
                48728, 26596, 11232, 62184, 37268, 2204,  7556,  51716, 44604,
                29600, 17480,
            },
            {
                62788, 6932,  53436, 47408, 24116, 8060,  35036, 43200, 56860,
                38332, 6188,  14532, 54812, 11592, 508,   60612, 19196, 30384,
                16604, 58544, 52208, 48028, 13256, 396,   38256, 57236, 7256,
                53668, 40604, 14980, 48836, 37176, 17140, 11952, 42576, 19140,
                30136, 61444, 37348, 9584,  59680, 54652, 13740, 2656,  51408,
                23724, 48572, 3832,  7716,  15644, 31204, 22452, 13116, 19036,
                55292, 46132, 65024, 41640, 14592, 58472, 7632,  64260, 408,
                17912, 36904, 12184, 2120,  20008, 10204, 56408, 26576, 38368,
                19816, 29084, 59884, 17064, 52416, 38684, 61228, 50192, 25064,
                58796, 19796, 53944, 43552, 30592, 51032, 45620, 62876, 40464,
                7476,  50280, 24916, 41508, 18468, 46596, 8828,  25660, 36524,
                5672,  14252, 52364, 26692, 64840, 12952, 57872, 37480, 9048,
                63648, 92,    11764, 46292, 64336, 18312, 43048, 24432, 60460,
                38532, 2828,  32664, 15280, 56744, 42716, 47888, 33028, 14760,
                59560, 40332,
            },
            {
                34988, 27060, 2848,  65224, 17820, 29500, 52684, 3748,  26384,
                22972, 52060, 42552, 27708, 36576, 24892, 33924, 48920, 7520,
                38392, 24592, 4756,  36616, 43756, 54104, 33604, 49860, 24808,
                19856, 64120, 28376, 9980,  25520, 52284, 31596, 62792, 5172,
                55448, 14964, 44464, 21816, 33808, 47760, 42236, 30260, 8840,
                58508, 34992, 27648, 37984, 56696, 46300, 52516, 58756, 42800,
                25188, 2144,  21012, 6072,  48448, 28696, 35480, 22236, 55200,
                26712, 50496, 23524, 54512, 43108, 35200, 50656, 3328,  44764,
                14640, 8560,  42656, 22488, 32116, 3116,  56004, 14712, 29464,
                62868, 7332,  34696, 57312, 15760, 21192, 2324,  27108, 57748,
                32076, 44772, 9992,  36660, 32940, 64144, 12824, 42080, 31140,
                46268, 23260, 39484, 55628, 1212,  21712, 41768, 4848,  31844,
                47436, 25568, 41412, 53492, 28780, 9076,  51156, 12132, 5560,
                21300, 45116, 52444, 6332,  23588, 19800, 10260, 55012, 24004,
                800,   9556,
            },
            {
                51272, 13796, 42852, 32132, 57768, 12224, 45024, 62932, 16084,
                33880, 60528, 3024,  46652, 65392, 15712, 41596, 56700, 12376,
                45844, 64552, 20832, 15472, 57784, 8856,  18052, 3448,  44408,
                12828, 34104, 2284,  60104, 55840, 40128, 3012,  35560, 48264,
                23660, 40992, 6624,  57084, 3624,  27612, 16628, 60532, 46028,
                18536, 14076, 54536, 43340, 10684, 24744, 888,   39832, 8676,
                50740, 31032, 37868, 59820, 32952, 3056,  42700, 12420, 47388,
                9384,  41032, 6336,  30688, 64044, 7740,  22828, 31820, 53008,
                62652, 34164, 49676, 57036, 12356, 26376, 45700, 5660,  37552,
                10072, 42020, 27704, 12900, 64952, 33736, 48320, 5516,  19848,
                13244, 22408, 55564, 61824, 1444,  51204, 21788, 54668, 59204,
                2392,  61892, 9756,  30088, 46808, 35600, 50752, 60556, 14388,
                52660, 20684, 6892,  58920, 22916, 32880, 61812, 46980, 34456,
                57160, 64072, 28804, 39860, 59720, 34924, 65084, 27768, 37628,
                57568, 19128,
            },
            {
                45080, 56240, 20972, 9276,  39368, 332,   20632, 37204, 7452,
                48348, 10836, 31020, 18600, 8432,  52372, 5044,  21736, 27916,
                53172, 2000,  40980, 29716, 61216, 23352, 28132, 62796, 31168,
                58284, 51128, 46116, 18148, 6744,  22708, 13968, 57988, 27884,
                10808, 65264, 31720, 49528, 12140, 62892, 21076, 37284, 5736,
                32700, 64888, 1820,  20516, 61632, 35792, 17356, 33188, 28092,
                62840, 15704, 56040, 10760, 17992, 53252, 62472, 15964, 32028,
                57004, 61920, 17076, 45824, 37832, 14020, 59300, 40920, 11396,
                24732, 2668,  39296, 6776,  61876, 19208, 41104, 33032, 21476,
                51964, 48100, 23276, 4328,  53012, 10196, 37428, 61148, 52072,
                34776, 59892, 6272,  27168, 15788, 38980, 29412, 7088,  17572,
                34784, 50532, 43396, 15820, 59800, 7372,  18180, 27428, 44128,
                34884, 61536, 16832, 36780, 2232,  39964, 4384,  27148, 41496,
                14336, 9532,  17940, 50756, 1740,  46160, 16532, 4092,  41524,
                49144, 30724,
            },
            {
                24548, 5196,  35828, 60856, 47144, 25236, 54360, 58936, 28628,
                41132, 21684, 55580, 38452, 23972, 59476, 31656, 37668, 62084,
                9908,  32980, 50520, 6948,  35492, 47056, 42180, 11324, 39852,
                5160,  22220, 38312, 30268, 43456, 64416, 50464, 20200, 44976,
                848,   17884, 36516, 25884, 52564, 39660, 68,    53872, 22984,
                41348, 25848, 52140, 30760, 48284, 6020,  59784, 45320, 12640,
                4072,  47224, 22980, 43780, 26852, 50020, 23760, 38652, 5136,
                20492, 34616, 1476,  52880, 27440, 19204, 47812, 5492,  17972,
                58288, 27828, 16096, 47416, 30084, 53672, 1992,  64724, 56620,
                16960, 1316,  61820, 38992, 45224, 25088, 29660, 16108, 41404,
                2732,  46940, 18216, 43232, 48788, 4640,  57916, 44972, 11876,
                27364, 19752, 4956,  24544, 32800, 11532, 54756, 23240, 1512,
                10308, 29404, 50008, 43260, 13308, 55004, 16336, 20112, 53264,
                544,   31912, 43604, 25888, 12260, 31064, 52840, 21960, 7364,
                11752, 63764,
            },
            {
                2044,  39868, 52296, 27960, 15184, 33000, 50052, 18288, 2672,
                63984, 14088, 49704, 1156,  44136, 12872, 47868, 3516,  42992,
                17724, 25124, 56180, 12924, 19484, 1344,  52820, 16536, 55248,
                26452, 8972,  15376, 54284, 34892, 26872, 8820,  38468, 30860,
                53276, 60232, 46336, 8336,  15312, 43600, 28848, 10736, 57192,
                49284, 12544, 8324,  39552, 14920, 26952, 53572, 20272, 57136,
                40548, 34840, 7312,  63668, 764,   35896, 8408,  58548, 44132,
                27936, 49692, 12844, 24492, 56628, 4088,  62224, 29752, 54988,
                36272, 45436, 64240, 21292, 35516, 9056,  48664, 24560, 11500,
                44368, 35464, 30888, 18768, 59484, 7636,  43128, 55324, 11084,
                23572, 37904, 29888, 53848, 12212, 24532, 32716, 52544, 40928,
                63832, 37368, 58456, 52948, 41284, 62368, 38244, 47776, 64512,
                40488, 4612,  57812, 26032, 47368, 65248, 30400, 58584, 48972,
                37808, 22384, 55612, 63264, 8212,  38752, 58684, 44228, 61104,
                34096, 16040,
            },
            {
                29736, 57992, 18072, 7352,  64256, 4488,  42208, 9704,  35552,
                45296, 26088, 57348, 33412, 62544, 26880, 19328, 55136, 14300,
                59004, 36416, 44680, 63780, 49184, 32256, 59380, 4180,  36268,
                63892, 48612, 61200, 988,   12320, 47236, 4712,  61956, 15616,
                40444, 5336,  22852, 58408, 20028, 33300, 64204, 46820, 35328,
                4116,  60328, 45012, 22320, 63000, 3080,  37408, 9924,  24216,
                52052, 18488, 29584, 54368, 41388, 14204, 30096, 46856, 3420,
                65236, 40260, 59876, 8572,  43852, 33076, 38968, 22276, 42888,
                9728,  51528, 224,   12916, 43768, 60340, 15768, 39480, 28700,
                59032, 8160,  54636, 14536, 49480, 21880, 104,   64292, 26992,
                50680, 62216, 8568,  34016, 65504, 19896, 60004, 9472,  22716,
                240,   47836, 13608, 28160, 2476,  20668, 5388,  30932, 13980,
                53552, 19336, 32488, 7800,  21660, 35732, 10764, 7128,  25252,
                62416, 5472,  47076, 35388, 20208, 48896, 28660, 13924, 25664,
                54564, 47432,
            },
            {
                43168, 22920, 49288, 11808, 44240, 22000, 56276, 13176, 30828,
                52400, 5144,  16868, 10188, 40520, 6140,  35468, 50304, 29816,
                56,    21524, 5444,  27644, 9196,  38724, 25488, 21044, 45644,
                32832, 18700, 40700, 24656, 57276, 19624, 33008, 55632, 25176,
                11872, 35048, 29624, 54972, 2132,  50444, 7384,  18360, 14516,
                27372, 31804, 17744, 57556, 33788, 42308, 49768, 64208, 32264,
                1440,  60580, 48564, 11832, 21372, 61368, 55712, 18628, 10440,
                22784, 15276, 30420, 36684, 51324, 11736, 1300,  49244, 14568,
                6664,  32516, 24192, 56748, 27204, 3704,  32008, 51080, 5528,
                20800, 41824, 26460, 3476,  32820, 57096, 35736, 17764, 31852,
                4796,  15020, 56976, 960,   42316, 38288, 3500,  47028, 16060,
                31348, 56568, 8424,  34408, 49428, 45268, 16352, 58864, 36716,
                24716, 45064, 12256, 62556, 51816, 1164,  45524, 42460, 33364,
                12556, 17200, 40384, 2564,  15424, 56672, 6320,  60,    37028,
                19852, 9916,
            },
            {
                6504,  62440, 37752, 34244, 53132, 29480, 38544, 62148, 23408,
                59352, 37692, 20568, 30044, 52036, 58596, 22528, 9040,  64908,
                46952, 39260, 53628, 16052, 42688, 62020, 13652, 51340, 7572,
                11800, 28188, 5532,  51708, 36860, 60080, 42160, 1656,  51088,
                47932, 64032, 42448, 13556, 38096, 59376, 24720, 41040, 61356,
                54836, 37644, 1176,  51352, 7676,  13280, 17164, 28452, 6620,
                44256, 15000, 25216, 38744, 5064,  33200, 51344, 25624, 37880,
                53692, 48464, 6436,  20056, 63880, 16496, 58620, 26124, 63444,
                19976, 60056, 38508, 41328, 52980, 18752, 37136, 62616, 13668,
                46308, 52316, 64108, 10612, 40740, 45604, 12836, 52392, 47384,
                39772, 44588, 22100, 26440, 17136, 51248, 28632, 35368, 62252,
                25372, 43812, 17816, 65068, 23520, 57140, 9500,  27864, 51368,
                2312,  56316, 35152, 16592, 39712, 28364, 60536, 18956, 51500,
                56460, 29856, 60000, 53460, 24340, 33800, 42624, 64584, 51720,
                40816, 60048,
            },
            {
                32276, 14464, 976,   25740, 59464, 3472,  19032, 47596, 732,
                8360,  43008, 48532, 64148, 1756,  45928, 16020, 42144, 25360,
                33120, 11968, 60284, 31492, 22928, 2764,  57040, 30312, 41656,
                58872, 53932, 44948, 14520, 30584, 10044, 16824, 28392, 21864,
                7836,  18684, 3848,  26392, 45344, 10492, 30736, 3348,  20952,
                43844, 10184, 25044, 46548, 29604, 55484, 22964, 47580, 39252,
                57604, 35380, 8192,  64868, 45168, 16668, 1980,  42920, 60644,
                32648, 520,   56104, 45864, 23848, 29008, 41168, 35748, 52424,
                29972, 46772, 16572, 4588,  10568, 47560, 22656, 55572, 916,
                33628, 16556, 37056, 23332, 61344, 28072, 7092,  60184, 20696,
                9572,  35120, 60920, 47732, 10136, 56496, 14212, 6992,  54952,
                40568, 3796,  51988, 38872, 12508, 31856, 42332, 63480, 6996,
                21280, 41928, 59808, 4108,  23556, 54656, 14064, 37364, 2864,
                23144, 9244,  43856, 27512, 11024, 60804, 21532, 31120, 17324,
                3988,  26720,
            },
            {
                47880, 55772, 19988, 45812, 9140,  15568, 40960, 26960, 32892,
                55296, 13844, 24984, 34180, 12068, 28196, 37512, 55760, 6988,
                51812, 19944, 3996,  48456, 54708, 36880, 17784, 47632, 620,
                23028, 34548, 65524, 2328,  23344, 46504, 63456, 38060, 44436,
                58324, 52240, 32536, 61412, 21460, 54216, 36492, 48604, 52348,
                6004,  65408, 40272, 20040, 61800, 36884, 1912,  60052, 11716,
                20876, 54192, 30684, 50368, 27580, 58616, 36012, 7652,  13160,
                28172, 18060, 41468, 34676, 4624,  53952, 8964,  2980,  44084,
                11440, 6488,  34324, 58244, 65416, 29112, 7436,  42352, 25424,
                59932, 29268, 4720,  50332, 18996, 1672,  38028, 25404, 56136,
                3052,  28900, 53412, 5544,  31520, 63344, 44912, 21764, 49848,
                11192, 20220, 29760, 6044,  54664, 1032,  35688, 17944, 48016,
                11576, 29848, 46680, 9036,  49148, 31992, 6600,  26708, 63036,
                48552, 34424, 3976,  50612, 37188, 45612, 8456,  49096, 12848,
                35652, 53472,
            },
            {
                10516, 39760, 63788, 30988, 51144, 56984, 65296, 11176, 50348,
                17924, 61848, 39460, 4168,  53696, 18836, 61448, 3140,  13964,
                62936, 40160, 28504, 10312, 45132, 7236,  27088, 62996, 38324,
                16304, 8880,  20204, 39628, 50328, 6900,  54016, 3060,  34652,
                10652, 14876, 39676, 47348, 6944,  16792, 63552, 13192, 27720,
                32864, 15612, 58464, 12076, 4804,  42632, 33048, 51452, 26776,
                4344,  42136, 17908, 3144,  23140, 11372, 49016, 21964, 64000,
                44632, 57808, 10108, 61152, 13612, 47292, 57608, 18784, 23312,
                62064, 54796, 26016, 21120, 44480, 14556, 34904, 49356, 11656,
                39668, 8844,  57744, 46636, 14952, 55280, 30916, 42076, 16760,
                63732, 13364, 41356, 18788, 36292, 24640, 39580, 1488,  32576,
                58332, 36868, 61500, 26312, 44204, 21940, 50760, 25512, 39440,
                61024, 33840, 26192, 18172, 64104, 40840, 57480, 45852, 16116,
                39268, 20500, 65108, 14628, 18680, 1960,  55416, 25396, 58768,
                43616, 21860,
            },
            {
                28656, 17152, 3772,  42760, 23776, 35056, 5304,  44624, 36720,
                22160, 6572,  29472, 57796, 44068, 23808, 49152, 30388, 35132,
                44532, 17384, 58300, 34396, 64780, 20400, 32932, 12280, 55504,
                49980, 59988, 43472, 26768, 56504, 31840, 13780, 25752, 61136,
                29912, 24088, 55680, 996,   28868, 34448, 41904, 2240,  56820,
                23464, 47708, 35292, 53420, 25748, 18016, 8892,  64608, 15708,
                46716, 59344, 9848,  62112, 40880, 56340, 29620, 39400, 52748,
                2940,  24408, 37652, 27072, 20624, 30292, 39504, 33468, 14360,
                48768, 37820, 868,   50060, 31684, 2964,  56312, 63512, 17368,
                53152, 21612, 31600, 42484, 34648, 65212, 51548, 11144, 49000,
                32472, 23892, 59184, 50500, 3660,  12372, 60244, 17932, 27616,
                45900, 15608, 8960,  48776, 59044, 13716, 62200, 4192,  55796,
                15080, 496,   54348, 36760, 12808, 21428, 2096,  10356, 30732,
                53224, 7068,  57244, 31896, 40684, 62232, 29372, 38636, 6780,
                63116, 1628,
            },
            {
                57548, 49720, 36848, 7708,  12908, 20504, 28284, 54108, 3164,
                59568, 45668, 51388, 15736, 8204,  40800, 10624, 59336, 21644,
                26560, 50720, 880,   24196, 14316, 52088, 43656, 5152,  24824,
                29260, 3460,  33232, 11200, 17520, 59552, 41084, 19952, 49416,
                4836,  43512, 65112, 19632, 50900, 58532, 9132,  45076, 18676,
                38844, 8232,  296,   30660, 49196, 57380, 45300, 29428, 22644,
                34172, 52388, 37568, 32032, 46328, 14948, 4924,  19376, 32172,
                16312, 49408, 7080,  52236, 64860, 992,   50536, 59692, 4396,
                28444, 42580, 17756, 11908, 61032, 38440, 26772, 20140, 5328,
                44924, 61736, 2424,  12352, 24828, 8276,  4388,  21772, 44484,
                584,   37776, 8444,  45888, 27976, 55472, 42708, 8164,  53788,
                64352, 2972,  34248, 40864, 17096, 32696, 8168,  28904, 43384,
                20700, 49596, 7148,  59420, 43792, 52056, 34592, 61060, 24428,
                42248, 12404, 46400, 22960, 4724,  51996, 11636, 20264, 15832,
                46456, 34532,
            },
            {
                13464, 61784, 26308, 53064, 60160, 48528, 40140, 15984, 31700,
                25340, 11456, 34704, 20660, 65016, 32504, 1884,  46832, 54228,
                5320,  9412,  42156, 55900, 37956, 3028,  60940, 40348, 46900,
                14656, 37396, 62824, 47420, 4592,  36064, 9128,  45416, 53288,
                16548, 35596, 8532,  13060, 39064, 26064, 22496, 62576, 50592,
                29064, 61012, 44220, 22024, 63420, 14016, 37180, 6640,  40748,
                2196,  13412, 24692, 19756, 48,    53800, 59828, 9300,  46640,
                61968, 12240, 34044, 43920, 16988, 41684, 10852, 25388, 36176,
                63784, 8224,  57532, 52136, 23700, 41516, 9376,  47360, 32656,
                37296, 27764, 48964, 54232, 19664, 40056, 29004, 60972, 35300,
                57156, 52552, 15672, 20844, 64740, 33944, 49020, 22504, 37672,
                12864, 24512, 56972, 20892, 1668,  46108, 53304, 36408, 10820,
                65368, 38572, 23816, 30108, 3984,  27264, 48332, 18908, 55660,
                420,   28336, 35908, 60300, 26152, 47996, 33716, 42256, 54524,
                30832, 24236,
            },
            {
                41092, 4704,  44396, 18224, 33236, 392,   62760, 8904,  56660,
                41764, 62180, 1184,  38496, 27680, 56800, 16700, 36548, 13048,
                39012, 62244, 31952, 16428, 48708, 29808, 10988, 21568, 58152,
                7596,  54088, 19544, 23912, 51536, 27908, 64456, 44,    22576,
                38632, 57504, 32208, 46576, 54496, 5080,  15444, 33448, 11848,
                4032,  52724, 16920, 10080, 39924, 2868,  20460, 55196, 61816,
                48488, 57280, 6356,  63564, 27144, 34360, 43160, 36948, 25596,
                55116, 40304, 3620,  58708, 23152, 31876, 6256,  55372, 16400,
                21740, 46632, 30200, 35216, 6392,  15992, 53592, 59100, 264,
                23532, 14624, 6912,  62828, 36496, 58560, 47460, 14380, 18260,
                26580, 5680,  30588, 39744, 10592, 1092,  16036, 31080, 4300,
                52144, 42164, 50396, 30720, 63852, 39612, 23036, 58504, 17728,
                31708, 45428, 9856,  53180, 16072, 39928, 7764,  14112, 37620,
                62284, 50948, 17364, 9468,  56292, 14956, 812,   64364, 5428,
                50608, 9080,
            },
            {
                55984, 22052, 31320, 10972, 57952, 46092, 24272, 37120, 19148,
                49696, 14804, 47184, 54820, 5208,  50160, 25296, 63080, 19920,
                28676, 46352, 22636, 58964, 7768,  26100, 53584, 34736, 17352,
                31880, 44312, 1796,  58444, 38908, 14812, 55764, 30936, 12052,
                62220, 26488, 3184,  21376, 63240, 30568, 49140, 37328, 59428,
                41300, 26268, 36280, 56272, 32768, 27628, 51752, 11680, 31508,
                17672, 28080, 43660, 39584, 50204, 11688, 22380, 65488, 1836,
                30220, 20420, 27848, 54072, 13380, 48304, 61776, 44488, 52656,
                12444, 40328, 1676,  19472, 63024, 43632, 28984, 13348, 50864,
                40424, 58180, 43724, 17044, 26200, 2776,  42984, 53452, 9492,
                63188, 48468, 43496, 60724, 53240, 25604, 57432, 45120, 61272,
                27928, 18340, 10512, 6952,  26428, 12348, 4676,  47688, 27388,
                1780,  55556, 60648, 19924, 33520, 64468, 58188, 44576, 31208,
                4832,  21496, 43212, 6212,  38360, 44656, 21892, 27748, 37388,
                60832, 19480,
            },
            {
                35148, 48328, 64932, 38948, 6884,  29160, 13416, 52420, 2748,
                27020, 7404,  30652, 23016, 12584, 44732, 41448, 7612,  58364,
                51972, 1568,  11820, 36312, 19668, 45164, 61972, 364,   50708,
                64980, 26272, 12940, 42096, 7824,  21040, 34124, 48548, 42772,
                6344,  50040, 15740, 41504, 11052, 44156, 780,   55812, 7264,
                20728, 13860, 62280, 48640, 5860,  46312, 60416, 25012, 44508,
                36560, 10016, 53220, 15388, 58228, 7216,  17988, 49200, 6464,
                15128, 60828, 45804, 7488,  35240, 19596, 27228, 37704, 3388,
                32848, 58720, 27112, 49168, 55048, 3880,  36220, 21500, 64692,
                9872,  34128, 30032, 52016, 11092, 32812, 6300,  23648, 38984,
                33868, 2012,  22668, 13168, 35852, 7056,  19872, 39096, 6048,
                34500, 59364, 46676, 55384, 35840, 60816, 51508, 14756, 41488,
                6520,  35144, 13604, 42004, 49180, 2300,  22364, 25876, 9832,
                47700, 54604, 33312, 63712, 29528, 52328, 58680, 10388, 16768,
                44936, 3376,
            },
            {
                14372, 27984, 1932,  16200, 54492, 42668, 21752, 64244, 44468,
                34252, 58348, 39956, 63708, 35356, 18556, 2960,  30808, 34416,
                15656, 42584, 54892, 64540, 3752,  41464, 14036, 38192, 22796,
                10096, 48436, 35416, 30132, 60296, 45128, 2580,  24896, 18256,
                59656, 29272, 36752, 52704, 60468, 18104, 28020, 23536, 47216,
                32240, 43460, 1896,  23880, 18776, 41588, 15232, 8000,  776,
                64392, 21656, 4368,  33220, 23928, 45696, 31460, 41428, 57256,
                33732, 50928, 10172, 38612, 64280, 1528,  56512, 9844,  23976,
                50352, 65104, 14600, 10420, 32500, 24500, 59992, 7112,  45160,
                18448, 55784, 1840,  22120, 48160, 56344, 64648, 29448, 12620,
                59356, 19100, 56016, 28800, 50560, 42608, 62584, 48344, 13760,
                22092, 108,   38348, 16112, 43932, 19904, 33100, 56524, 62688,
                22664, 51112, 26304, 6560,  29360, 11864, 53708, 35376, 63108,
                16140, 40276, 11584, 24320, 18664, 3232,  34460, 48404, 24812,
                30392, 53548,
            },
            {
                62524, 40728, 57252, 25704, 35872, 49984, 4112,  31108, 11112,
                54160, 20548, 16236, 9440,  51480, 60404, 56176, 10700, 49104,
                23692, 6416,  26492, 33268, 50108, 30416, 56676, 6076,  28596,
                40248, 4340,  56020, 17056, 50284, 11160, 63728, 53432, 40092,
                9528,  56396, 23156, 2124,  33856, 8464,  40296, 64088, 16124,
                53736, 58156, 9464,  30824, 65204, 53468, 38120, 33824, 49128,
                54704, 30068, 60352, 37800, 1412,  62452, 52776, 13388, 37412,
                23476, 43060, 17464, 25152, 51780, 41232, 30060, 17752, 43140,
                5524,  20792, 37952, 45680, 40612, 16916, 48176, 31076, 38608,
                25532, 4904,  63168, 36960, 40732, 16076, 20228, 45112, 51348,
                41540, 8188,  46732, 4200,  17236, 32188, 9964,  26856, 55092,
                63476, 29408, 49928, 4184,  23832, 8876,  29056, 3668,  37944,
                18992, 47004, 63572, 37000, 56732, 45372, 17484, 39420, 3828,
                57876, 27688, 1224,  50168, 57284, 13868, 40900, 7204,  59684,
                38524, 11308,
            },
            {
                23340, 6412,  46624, 20480, 9476,  59132, 17720, 38788, 60996,
                5788,  42188, 48804, 128,   24924, 27972, 38268, 21348, 63976,
                45724, 39820, 60684, 16772, 9004,  24388, 18320, 45744, 59520,
                53184, 20132, 63172, 24120, 6540,  28240, 36820, 15100, 4584,
                32756, 45272, 13584, 47648, 25776, 56852, 50848, 12124, 35428,
                5332,  27728, 51052, 35980, 12768, 3324,  22596, 59084, 19296,
                41124, 12596, 46924, 51300, 19568, 28692, 9640,  26372, 4448,
                63692, 720,   59340, 32464, 5816,  14552, 47944, 59868, 34952,
                53936, 25960, 2948,  61436, 8096,  56412, 1276,  12368, 54444,
                50036, 28776, 46284, 13568, 7832,  59984, 404,   35684, 4748,
                25200, 31716, 37172, 65508, 23788, 59124, 3284,  36844, 17792,
                41236, 11028, 32432, 57340, 64920, 40264, 49172, 45068, 11740,
                31092, 8436,  16712, 252,   24024, 60884, 7496,  29952, 51924,
                20816, 45732, 61600, 36304, 43900, 31440, 65480, 54272, 20616,
                1040,  50360,
            },
            {
                18268, 31620, 52068, 13028, 63092, 33676, 26472, 48048, 14644,
                24204, 28948, 59264, 32900, 46304, 5900,  43652, 14248, 53232,
                2348,  29376, 12744, 47652, 54024, 36552, 63380, 11512, 33692,
                15416, 43388, 1016,  32376, 39804, 55156, 19356, 47772, 26568,
                61580, 20392, 7396,  64876, 38428, 19436, 31200, 2908,  45884,
                20004, 39708, 60688, 17244, 43100, 56388, 29784, 10576, 27436,
                5664,  16932, 25784, 8632,  40448, 59192, 44456, 54800, 21136,
                47900, 29648, 12480, 46332, 57428, 21796, 62940, 11376, 7728,
                46508, 57488, 30856, 18604, 51920, 27872, 62272, 20720, 42432,
                15924, 10320, 61036, 23964, 31732, 49380, 27492, 53892, 62396,
                14292, 57972, 10756, 53532, 40812, 14868, 46448, 51496, 7600,
                24940, 52984, 45468, 13296, 27668, 18068, 980,   60308, 24872,
                53512, 58148, 44032, 52020, 33304, 20160, 48588, 42616, 12476,
                32960, 8104,  15240, 22388, 5640,  10040, 26052, 16088, 45616,
                33620, 42876,
            },
            {
                57868, 36184, 3628,  44216, 40156, 676,   8128,  56232, 44808,
                2604,  37544, 52396, 13120, 62464, 17248, 55044, 32056, 8712,
                19164, 37536, 57948, 21888, 1284,  41932, 4628,  51524, 25356,
                8180,  37060, 47136, 13284, 60672, 43948, 3652,  58452, 35312,
                51564, 41036, 29776, 54268, 4972,  14512, 60156, 42424, 62864,
                24668, 11504, 432,   49392, 26120, 7388,  46864, 51520, 62908,
                44288, 57732, 35752, 63304, 15104, 2680,  34816, 17128, 39624,
                7292,  55668, 19180, 36876, 27724, 4076,  33664, 39108, 28792,
                16016, 41760, 11704, 35392, 23472, 43668, 36972, 32632, 3664,
                58144, 34624, 41432, 17852, 55348, 37912, 11252, 22728, 17408,
                42880, 49156, 20376, 27152, 664,   33688, 57068, 29980, 43620,
                2708,  59676, 20664, 6176,  35608, 51700, 55272, 15476, 34756,
                5016,  39980, 28552, 10752, 38424, 14364, 65164, 1732,  24436,
                60432, 38336, 53792, 29188, 47568, 39448, 51576, 4260,  28788,
                61960, 8604,
            },
            {
                15164, 64688, 28440, 55040, 23504, 30364, 51212, 21212, 34644,
                65476, 18464, 7788,  22400, 40632, 35504, 3916,  24144, 61884,
                34488, 51232, 5688,  43904, 32484, 60140, 20332, 48128, 30528,
                57848, 62372, 27380, 52128, 21480, 8928,  29988, 12720, 22772,
                640,   10528, 17300, 43492, 34332, 48740, 22720, 8076,  29364,
                54356, 47544, 33144, 21800, 63752, 39936, 34752, 15344, 1624,
                38668, 32140, 4148,  48852, 23376, 30900, 56612, 11140, 50336,
                32728, 62700, 41860, 9116,  54032, 44076, 18108, 52492, 23120,
                64332, 1560,  49556, 58656, 4336,  13812, 9336,  50640, 65260,
                22464, 6604,  52916, 2676,  43564, 5228,  58792, 47300, 33896,
                1944,  30128, 6680,  44712, 60964, 12148, 22376, 18876, 64404,
                34540, 15548, 38012, 62520, 42304, 9928,  31884, 22112, 43004,
                64576, 13836, 21068, 59716, 4528,  53868, 26660, 35564, 56424,
                49304, 18444, 3044,  62920, 56552, 19700, 60904, 36924, 12816,
                55216, 25084,
            },
            {
                46948, 38820, 10048, 17492, 58852, 13896, 62212, 41184, 11320,
                53972, 31792, 47852, 56948, 10148, 28476, 58908, 48700, 41292,
                26932, 15804, 65288, 25144, 11984, 28400, 16044, 39984, 2428,
                22372, 11196, 4920,  17180, 34740, 49320, 65472, 39320, 53904,
                46388, 63260, 24844, 57108, 1972,  27184, 53164, 36228, 15888,
                58500, 5676,  37736, 57196, 14080, 3928,  19836, 60888, 24424,
                55308, 21464, 11928, 53444, 42036, 6716,  64524, 45836, 27488,
                14384, 23228, 2952,  49396, 24636, 61048, 148,   48360, 6468,
                55232, 32788, 20104, 39176, 63428, 47296, 55848, 18056, 27352,
                38252, 48428, 30224, 25408, 64232, 19784, 28896, 7944,  39696,
                64016, 52720, 37556, 15860, 50448, 39724, 5220,  54172, 10540,
                27476, 48980, 23508, 2056,  46644, 25864, 61288, 7308,  48492,
                27196, 2816,  49852, 31540, 46240, 41340, 17704, 6064,  44392,
                9812,  31016, 41804, 25756, 12100, 192,   33044, 22684, 48748,
                40820, 1684,
            },
            {
                52640, 21676, 5244,  49472, 35084, 46060, 5636,  28024, 16564,
                4160,  26696, 43316, 1532,  63632, 19908, 45596, 12860, 692,
                54428, 10276, 46492, 39304, 56940, 7748,  54552, 64176, 35472,
                43076, 54236, 38492, 57124, 42500, 2552,  26008, 18048, 5360,
                31300, 37236, 49924, 13008, 59564, 39644, 11180, 45152, 2532,
                41080, 18744, 28064, 8144,  45016, 54060, 31892, 43432, 8364,
                49616, 17600, 45556, 28212, 58744, 18004, 36996, 21776, 1084,
                53128, 35232, 59008, 16292, 31340, 12876, 40676, 26904, 36352,
                14312, 44980, 25028, 7456,  16276, 30328, 24480, 336,   44952,
                8684,  14916, 60116, 11696, 35296, 51180, 44832, 13508, 56060,
                25620, 10068, 58488, 23088, 28768, 62768, 36052, 47524, 41024,
                58764, 8040,  54908, 30552, 58076, 19392, 12976, 38776, 56668,
                18404, 36044, 62508, 24156, 9180,  57760, 29216, 62304, 13472,
                23004, 52860, 6972,  35784, 45552, 16828, 43560, 59096, 7176,
                18624, 30816,
            },
            {
                12164, 61140, 27068, 41732, 2172,  20092, 38184, 57304, 50224,
                61492, 39452, 14068, 24460, 36748, 52080, 7400,  38320, 22336,
                60196, 31052, 3612,  17896, 50024, 34120, 44460, 13564, 6056,
                18644, 48628, 31488, 23924, 14452, 59492, 33012, 44552, 56220,
                15028, 7168,  20564, 33160, 6128,  18540, 62160, 21504, 32288,
                63972, 51676, 11980, 61112, 23348, 50320, 27012, 11596, 36940,
                29904, 65132, 236,   34192, 9192,  25736, 51756, 5504,  61188,
                40452, 10460, 45592, 38472, 65196, 7988,  56944, 19468, 59732,
                10364, 61780, 28420, 50896, 42564, 34232, 60576, 40720, 52256,
                32120, 55624, 21184, 39588, 1480,  16252, 61124, 31944, 21452,
                3340,  18096, 34640, 46608, 2540,  8976,  17260, 25056, 1268,
                32148, 13596, 39412, 16716, 5608,  50188, 33400, 1308,  45540,
                10448, 52664, 42532, 16376, 892,   36916, 21368, 47044, 33460,
                39212, 58776, 15004, 64068, 51332, 30320, 54472, 10488, 27696,
                63508, 34708,
            },
            {
                57964, 44700, 32036, 51584, 64304, 10668, 31328, 23676, 8708,
                46296, 19316, 55076, 48908, 32796, 16004, 30024, 56112, 18376,
                42912, 49088, 36472, 23072, 62052, 908,   21116, 25820, 51412,
                61192, 28040, 168,   63916, 7932,  51868, 10464, 21748, 62108,
                40616, 28548, 61084, 42128, 46792, 30248, 50736, 56452, 9312,
                25148, 46852, 30820, 41916, 35996, 16440, 1940,  56764, 59704,
                5776,  41148, 14932, 60568, 39432, 48112, 12804, 42724, 30276,
                57264, 18660, 28368, 4212,  21584, 34356, 50780, 43692, 31040,
                4460,  37920, 54300, 2048,  57364, 10800, 5784,  20476, 12840,
                63776, 4100,  42776, 47008, 57424, 26848, 5956,  41668, 47684,
                51696, 61932, 42720, 13324, 56400, 52848, 30540, 43440, 63316,
                20244, 52356, 44740, 65292, 27744, 41548, 53636, 63044, 23812,
                29576, 6036,  32920, 55440, 60936, 12088, 50912, 4872,  55208,
                1200,  27932, 48512, 21016, 3560,  24608, 5704,  37452, 50028,
                42764, 2968,
            },
            {
                24168, 7904,  18800, 14408, 25112, 53648, 60216, 43184, 808,
                34420, 29740, 10620, 5628,  58240, 2880,  44308, 64432, 33996,
                4960,  14400, 27336, 8912,  53392, 29820, 38840, 58832, 32456,
                8736,  12436, 45280, 20436, 40180, 29340, 36388, 48720, 1228,
                25192, 52856, 10064, 55476, 23796, 72,    14264, 38228, 5176,
                35572, 15200, 956,   55880, 6248,  63248, 38996, 46672, 18560,
                25492, 50860, 22552, 55016, 4376,  19880, 56132, 33432, 15332,
                24964, 6896,  51292, 55608, 47316, 25444, 2560,  15604, 53244,
                22600, 47796, 17868, 13448, 22008, 36944, 53416, 46396, 29124,
                36544, 25676, 17468, 33204, 9444,  22712, 54396, 35640, 11076,
                29044, 38680, 4512,  26220, 33088, 20708, 60412, 11860, 49440,
                4156,  35028, 22688, 10640, 36092, 3692,  21036, 15316, 37040,
                57596, 13492, 20628, 48044, 39140, 25268, 30896, 15968, 64812,
                19192, 43380, 10832, 32904, 56932, 40668, 62328, 13884, 21172,
                55912, 16340,
            },
            {
                47928, 36124, 54616, 39728, 6608,  47564, 17012, 36756, 13592,
                56588, 64636, 22680, 38132, 61196, 26316, 11612, 23620, 8284,
                51028, 62832, 57936, 40936, 45288, 11276, 4600,  46992, 16216,
                41748, 56916, 35164, 53588, 47148, 4876,  17636, 58588, 13520,
                34472, 47624, 3864,  16676, 35228, 64380, 26468, 43740, 49296,
                60436, 22416, 52784, 19244, 27740, 10164, 22168, 32804, 13524,
                44072, 35156, 10416, 31124, 46444, 28496, 62256, 2276,  48988,
                64096, 43612, 32236, 14104, 61608, 10880, 39156, 28604, 63252,
                8980,  41200, 33516, 65020, 44500, 26024, 62596, 15988, 3172,
                57892, 50324, 6424,  62332, 49028, 38436, 64528, 708,   15428,
                59900, 19996, 8504,  65136, 48800, 40228, 6264,  37852, 16288,
                56796, 26700, 6812,  55492, 47916, 58668, 30892, 8656,  49324,
                40224, 64444, 3072,  28096, 7720,  44284, 53612, 41048, 9108,
                36180, 25472, 61416, 38280, 8140,  18012, 46144, 34100, 26236,
                5168,  39324,
            },
            {
                11280, 59804, 320,   62660, 34140, 29032, 3132,  21216, 52808,
                26476, 7060,  42100, 51492, 18756, 46776, 40356, 53464, 29040,
                37688, 20984, 2280,  32968, 15512, 19400, 64676, 54640, 22704,
                37592, 5252,  25060, 14808, 62404, 26680, 55164, 42588, 7464,
                64772, 19764, 38448, 45056, 57852, 8768,  53300, 17956, 28876,
                11380, 39132, 65412, 33844, 44684, 59068, 49808, 54924, 4560,
                60848, 52864, 2468,  63988, 16752, 36504, 8016,  22864, 38560,
                10168, 20524, 1448,  36812, 41968, 18496, 58740, 46148, 35760,
                796,   26520, 58912, 5216,  29884, 49736, 8748,  32740, 40116,
                23328, 10472, 53808, 28328, 13248, 30704, 18688, 52168, 25168,
                43436, 49712, 31260, 14848, 44804, 180,   24412, 58448, 29348,
                46036, 41872, 61476, 18228, 13288, 25092, 43956, 60764, 4292,
                25984, 44868, 51652, 35308, 18388, 63104, 3412,  23484, 49876,
                59172, 46256, 2440,  14240, 28988, 49584, 59504, 936,   53360,
                65256, 29872,
            },
            {
                50296, 27160, 21920, 42832, 12732, 58252, 44944, 63076, 38920,
                49548, 16584, 31872, 1340,  13996, 34584, 4688,  60664, 16148,
                46236, 12392, 54392, 24680, 60496, 50316, 34900, 27472, 2140,
                49784, 63644, 19152, 31976, 2920,  11468, 38664, 23480, 30952,
                50988, 27524, 11652, 31568, 21664, 40080, 4312,  33472, 58344,
                3404,  42648, 8112,  48524, 13108, 2596,  30116, 40508, 8836,
                27772, 20136, 38716, 43116, 25140, 11884, 53280, 45336, 59020,
                34972, 54440, 60016, 26828, 7796,  52264, 5480,  20856, 13360,
                54588, 50176, 11664, 20128, 38928, 2292,  59320, 18816, 47940,
                61552, 44008, 20288, 41028, 2436,  45444, 8220,  58316, 33396,
                5728,  56624, 22860, 54052, 35272, 18184, 50956, 9292,  33740,
                14160, 2176,  50768, 32596, 39356, 944,   52188, 19264, 33908,
                16612, 10956, 22888, 58556, 14580, 55892, 11464, 34384, 29108,
                5440,  17428, 52100, 55260, 42628, 23552, 11556, 31408, 44380,
                9024,  19276,
            },
            {
                33584, 45476, 4440,  17548, 50964, 25732, 9348,  31504, 4268,
                11524, 43984, 57680, 62712, 28116, 48668, 21156, 56808, 208,
                42056, 31240, 48168, 5668,  37272, 42788, 8200,  13312, 30876,
                44568, 10328, 57888, 42908, 51196, 33764, 61156, 1976,  45944,
                15932, 54404, 60288, 1712,  50064, 61372, 12636, 46124, 20768,
                54468, 25560, 31624, 17196, 37916, 20520, 24952, 64436, 16392,
                48368, 32440, 58212, 6444,  50232, 57000, 40484, 18368, 5408,
                28912, 16240, 47856, 22468, 63716, 30208, 33652, 56768, 42864,
                23980, 31332, 45508, 56044, 15180, 42732, 24736, 55420, 13496,
                656,   33832, 15400, 56988, 36140, 60912, 22004, 39704, 47092,
                12820, 37168, 3016,  61312, 11236, 27416, 62532, 54712, 21092,
                64484, 24032, 8824,  28812, 46988, 64140, 10284, 41784, 28352,
                54248, 61888, 96,    42120, 31752, 38164, 47748, 20292, 56716,
                12740, 31660, 21628, 34724, 64476, 4880,  19964, 37044, 15720,
                41044, 57240,
            },
            {
                14480, 61276, 30836, 55592, 7328,  36400, 19252, 48424, 59944,
                23688, 35496, 20424, 8908,  54112, 37564, 10784, 32676, 25640,
                19136, 65156, 9780,  28428, 56420, 3208,  21220, 61520, 55928,
                16696, 26340, 39456, 7856,  22172, 47664, 18524, 57032, 9264,
                40204, 4864,  24704, 42964, 15468, 29432, 24372, 36892, 62948,
                15588, 50424, 1548,  61788, 57220, 51384, 43648, 3736,  36888,
                61860, 11628, 23628, 15124, 29840, 580,   32668, 26172, 13560,
                51952, 3384,  40240, 11620, 44796, 15188, 49232, 3964,  64616,
                16464, 6828,  61948, 34732, 9512,  52708, 35984, 4968,  30360,
                38296, 27028, 65356, 5348,  24368, 49580, 16024, 3568,  27836,
                63928, 17796, 41164, 29788, 7120,  42400, 46668, 36708, 5588,
                40644, 53396, 37508, 59952, 21768, 15884, 36488, 56576, 5612,
                47160, 37732, 29308, 49948, 6280,  26464, 2244,  62028, 40504,
                44496, 63164, 39100, 7472,  26668, 51184, 58428, 47372, 62852,
                24300, 2572,
            },
            {
                6456,  53176, 38512, 23560, 63576, 41336, 56912, 14824, 27656,
                53268, 2320,  46048, 40552, 24292, 6228,  63528, 43836, 50100,
                7208,  39216, 58960, 22692, 17188, 31948, 46788, 40320, 51552,
                36048, 624,   60260, 29900, 54724, 14144, 36696, 28396, 21444,
                63112, 33328, 37232, 56680, 7276,  47196, 52624, 6500,  41268,
                10192, 28340, 45412, 22944, 5576,  34388, 12168, 53556, 19540,
                45000, 1636,  54580, 47364, 63468, 21316, 59768, 48192, 65484,
                43996, 31416, 57640, 35620, 368,   25292, 38740, 10628, 28216,
                36712, 40540, 1888,  22424, 27736, 63960, 46052, 21008, 60132,
                49940, 8596,  46316, 52536, 29640, 9960,  43020, 31916, 54520,
                9316,  25548, 51840, 48052, 59116, 19840, 13188, 1552,  30092,
                17068, 45568, 12176, 3076,  54960, 7156,  31772, 24076, 59740,
                20416, 14328, 9172,  65076, 21616, 45532, 52476, 15856, 8492,
                24188, 448,   48312, 11172, 16488, 40184, 2060,  28008, 7804,
                52220, 36296,
            },
            {
                26524, 42380, 9496,  13212, 46932, 1252,  32644, 6296,  39376,
                65308, 30404, 13076, 60924, 17272, 51256, 29460, 14444, 58164,
                35316, 52940, 13260, 43352, 51024, 63808, 10012, 24616, 6120,
                19704, 48540, 12724, 45188, 3852,  65192, 5512,  52412, 44416,
                13132, 48820, 19216, 10216, 32044, 63944, 18160, 804,   32404,
                59052, 35432, 55336, 39736, 9420,  27652, 60020, 30648, 7468,
                25932, 34276, 39916, 8780,  36392, 42808, 4696,  9792,  34604,
                23772, 6960,  19760, 49904, 61768, 53236, 21448, 60456, 47032,
                19288, 51704, 59644, 47860, 17440, 32204, 7192,  11712, 52184,
                41840, 22932, 18332, 12576, 56032, 34436, 62880, 50776, 19584,
                60144, 34296, 1424,  15096, 37784, 24664, 32780, 57632, 49564,
                26072, 58564, 19520, 34468, 27008, 44648, 49672, 12852, 3524,
                43880, 51932, 35212, 56964, 17560, 11108, 59432, 32996, 36692,
                54276, 28372, 57008, 60628, 31180, 45400, 55996, 12904, 32228,
                20868, 48352,
            },
            {
                16724, 64756, 19932, 34916, 28588, 53912, 22148, 43716, 9820,
                18396, 47980, 5272,  33288, 57112, 1680,  39904, 21992, 3300,
                16904, 27600, 5140,  33892, 860,   38484, 14828, 29052, 58480,
                33648, 63140, 23388, 38352, 26884, 17276, 41384, 31148, 58572,
                344,   26824, 61592, 53660, 22132, 39196, 27568, 55808, 44264,
                20124, 4936,  13484, 18436, 63120, 47548, 15972, 41532, 56788,
                49388, 59584, 16940, 27924, 19360, 12736, 55500, 38588, 16764,
                53824, 62572, 28016, 9328,  17488, 32012, 42356, 2472,  54136,
                7896,  30148, 12004, 42200, 4132,  56372, 39492, 26112, 16692,
                1772,  62432, 37096, 3836,  40408, 20592, 416,   13840, 6968,
                42016, 45816, 21904, 55276, 4772,  64792, 52576, 42932, 10792,
                63340, 7444,  41912, 51788, 62536, 38708, 18528, 61604, 40244,
                26288, 32244, 1920,  24452, 39276, 42996, 27496, 5104,  50096,
                18328, 13208, 35124, 20408, 3456,  23952, 35800, 17936, 61468,
                43520, 58136,
            },
            {
                30172, 3120,  56740, 49520, 4788,  16984, 62308, 50840, 58100,
                36212, 22580, 55240, 26688, 44372, 36364, 11452, 48116, 62388,
                44668, 23672, 61264, 47532, 18832, 59616, 54804, 44000, 2624,
                52988, 15904, 9288,  56960, 34728, 50524, 10844, 24276, 8092,
                35588, 16280, 41160, 3224,  45376, 13820, 50632, 11008, 23828,
                64652, 48620, 52172, 30928, 42960, 24040, 132,   38004, 21708,
                13708, 4028,  51668, 62444, 46020, 31692, 50920, 22096, 1744,
                41956, 12856, 37008, 45576, 55844, 5932,  13688, 34684, 26232,
                15084, 57844, 33988, 24648, 49724, 13824, 61424, 44704, 54484,
                33504, 28960, 58056, 44528, 26408, 59488, 47816, 24804, 38660,
                28544, 57564, 10424, 31000, 40300, 8668,  16472, 22068, 3400,
                36376, 31160, 22948, 4844,  14484, 596,   57408, 30164, 8552,
                53772, 16384, 60624, 46564, 7184,  31188, 63020, 22608, 58256,
                41248, 4352,  46648, 52592, 42464, 63836, 9424,  54000, 38596,
                576,   11072,
            },
            {
                51168, 39604, 44676, 24444, 60680, 38124, 11948, 25816, 3264,
                15360, 40776, 50164, 8644,  19720, 64472, 25496, 54744, 32048,
                8996,  37860, 56192, 30332, 8008,  26404, 35340, 12396, 21516,
                41884, 30632, 46904, 2988,  62344, 20736, 43484, 55404, 63864,
                46188, 51824, 29780, 6312,  59152, 33888, 6784,  60872, 38528,
                15224, 26320, 2248,  36444, 56592, 6684,  52956, 65188, 10140,
                32784, 43272, 23872, 6532,  2220,  57656, 25904, 60808, 29484,
                48684, 58528, 3684,  24288, 39732, 29436, 58932, 44620, 63400,
                39536, 21032, 768,   64568, 37064, 19980, 29708, 3008,  36420,
                8864,  14084, 49216, 6524,  31376, 16436, 35936, 54200, 65036,
                4308,  17168, 50180, 61840, 26584, 48936, 34940, 28684, 54344,
                46996, 15980, 55740, 48616, 33424, 24684, 11012, 47280, 22032,
                63820, 37252, 11916, 49780, 56108, 13832, 952,   47788, 9804,
                30272, 65216, 25656, 7756,  14800, 49764, 29088, 5116,  46836,
                25544, 34520,
            },
            {
                21576, 6808,  14420, 32460, 8756,  42152, 31128, 46252, 34064,
                62872, 28640, 876,   60236, 15676, 30796, 4288,  41908, 18612,
                51944, 2796,  11724, 41616, 21832, 49464, 4524,  46276, 64956,
                7844,  25964, 39672, 53796, 14012, 32528, 1580,  37848, 18844,
                12236, 22544, 57092, 36928, 25436, 17688, 43376, 29612, 3616,
                33368, 46260, 8804,  59796, 12472, 19884, 33928, 26512, 48688,
                61480, 29312, 55120, 33964, 40916, 15012, 7480,  44756, 10708,
                33136, 20676, 52672, 15484, 64700, 47992, 8956,  23232, 4164,
                48632, 55700, 43708, 9656,  52764, 5556,  58192, 22360, 47520,
                64808, 24976, 19864, 53568, 62060, 10916, 7912,  44996, 21620,
                33156, 13024, 36096, 43252, 19088, 632,   44896, 59056, 12216,
                39084, 60968, 9252,  28256, 65432, 42848, 51316, 35484, 2536,
                44816, 6040,  29128, 21232, 34404, 18584, 53092, 37080, 15536,
                45044, 21160, 37828, 59140, 33208, 19048, 40884, 22756, 58648,
                14988, 62628,
            },
            {
                54904, 28036, 58972, 52468, 19072, 144,   57056, 20536, 6748,
                52620, 43332, 12812, 38052, 46004, 52428, 7564,  57976, 13864,
                28268, 45968, 64288, 15320, 53924, 62004, 31432, 17732, 37344,
                51240, 60604, 19040, 5564,  28688, 49080, 60844, 7036,  26640,
                33656, 2644,  49316, 10008, 65336, 47324, 53404, 21552, 57764,
                42044, 54124, 22412, 28732, 49820, 41064, 45364, 15072, 2584,
                18308, 7160,  38236, 20348, 64560, 49344, 37208, 18336, 63748,
                39312, 5024,  27216, 35236, 968,   19716, 51504, 37768, 17052,
                11340, 31100, 27156, 18124, 46008, 32564, 41144, 11316, 17312,
                56544, 42148, 38072, 1296,  23284, 41068, 58204, 29208, 2016,
                52076, 59880, 7336,  23056, 56152, 14180, 63132, 6848,  19820,
                25284, 1692,  44304, 18732, 38068, 6760,  16068, 56096, 27752,
                18872, 58848, 42324, 2912,  64592, 40692, 24728, 60692, 32776,
                2384,  56228, 11148, 27788, 1244,  62076, 12112, 51856, 31800,
                8028,  41848,
            },
            {
                1900,  11812, 35848, 46700, 64276, 27464, 49328, 13828, 59348,
                23992, 9404,  32040, 56884, 20992, 34912, 23756, 39224, 61004,
                35868, 20308, 32912, 25304, 1472,  39548, 9536,  56360, 23800,
                840,   34240, 11828, 44672, 58492, 22332, 15928, 46764, 53124,
                39596, 60516, 42640, 20464, 15056, 660,   35256, 9164,  12832,
                18504, 61168, 39520, 16324, 63900, 3416,  30468, 58860, 36252,
                56352, 46216, 11988, 53108, 9208,  23040, 28448, 268,   54912,
                14680, 45956, 61544, 42940, 57088, 12272, 28348, 33056, 53624,
                61992, 5976,  35716, 60420, 15212, 25096, 63100, 51280, 28412,
                4684,  32100, 12416, 45772, 51020, 33960, 14256, 49048, 18232,
                41472, 26252, 46584, 3364,  29892, 38088, 50512, 32164, 41476,
                51876, 34344, 57664, 12552, 53560, 21284, 32340, 60192, 40480,
                13180, 52044, 23836, 54460, 9988,  45788, 28128, 7500,  51356,
                42376, 17268, 52972, 48444, 39192, 45172, 55816, 3588,  37108,
                17476, 48408,
            },
            {
                24068, 38868, 16936, 5060,  23404, 10656, 54576, 39280, 35972,
                17808, 64900, 25996, 48148, 2600,  62848, 11284, 50136, 324,
                48032, 6596,  57520, 50972, 36172, 44852, 13368, 59460, 29196,
                15576, 47780, 55748, 25004, 36168, 8720,  40624, 30284, 56404,
                4996,  13644, 28556, 32452, 55436, 40640, 27208, 62236, 48764,
                31296, 1572,  6384,  35184, 10504, 24800, 53664, 8252,  40060,
                23292, 27552, 60896, 16480, 31144, 44148, 59516, 51900, 31796,
                23944, 9120,  50112, 7024,  22804, 40556, 62896, 3128,  45252,
                24416, 41572, 49972, 2360,  54948, 7916,  38020, 536,   44280,
                60252, 8132,  55512, 17556, 64048, 26936, 4980,  37148, 62748,
                9068,  56904, 38972, 64548, 53440, 9580,  17588, 26868, 4228,
                61832, 21844, 30192, 5604,  47140, 62188, 3716,  25580, 9408,
                48916, 34036, 30376, 15920, 38064, 4320,  58512, 12780, 22240,
                62548, 5352,  24356, 34720, 6564,  21792, 30500, 26292, 59860,
                43704, 65444,
            },
            {
                29192, 50992, 60316, 41100, 33860, 43640, 30228, 2388,  44960,
                4380,  50764, 41244, 5908,  17088, 29692, 43808, 26792, 16256,
                22476, 42860, 10396, 17952, 5192,  27824, 20444, 48812, 41312,
                6568,  63980, 31132, 42516, 4188,  51732, 64996, 11136, 23728,
                18064, 62648, 45840, 7644,  59608, 22792, 51488, 4608,  37352,
                23440, 45048, 51936, 57592, 18848, 47604, 61844, 20884, 13084,
                51192, 864,   41944, 48072, 3464,  35772, 13136, 6616,  41564,
                57612, 19044, 33852, 30184, 16840, 47264, 55636, 8080,  19404,
                58780, 12948, 21784, 39512, 29664, 48136, 13476, 34792, 19628,
                25744, 49560, 36192, 29992, 3228,  20744, 53260, 44412, 23740,
                31576, 15328, 20248, 12332, 33632, 24256, 47828, 56880, 43476,
                8732,  15236, 49336, 35964, 24284, 41812, 14788, 45868, 38300,
                1100,  61080, 6644,  47332, 62208, 32524, 17592, 48724, 35436,
                29036, 40088, 58092, 13164, 63740, 16684, 9144,  49524, 13040,
                20240, 33392,
            },
        },
        {
            {
                16556, 30276, 8172,  32892, 57684, 44568, 53960, 39948, 9900,
                30048, 17064, 22208, 7256,  34332, 42644, 65304, 52952, 11624,
                40852, 26588, 62428, 42444, 5224,  10732, 40544, 33648, 52824,
                23028, 18388, 4652,  61572, 52348, 512,   45856, 62492, 11648,
                44716, 4764,  61160, 41800, 9856,  32804, 45564, 21868, 54604,
                26760, 7068,  24100, 52168, 60424, 28580, 204,   9636,  52036,
                41120, 58360, 15896, 12032, 56568, 22140, 6516,  44500, 59288,
                41284, 55860, 6764,  49780, 32940, 58972, 21296, 50896, 6000,
                43724, 61852, 27808, 4996,  44912, 29740, 8224,  59160, 50508,
                11652, 54864, 22388, 36320, 28472, 8852,  43916, 15736, 23844,
                7644,  29328, 38392, 14824, 44484, 62288, 40804, 5284,  59396,
                19504, 61972, 25692, 7132,  49436, 27576, 62936, 55160, 12180,
                16220, 47512, 1060,  56812, 18120, 12016, 54372, 41864, 13840,
                45268, 30040, 63064, 47576, 52876, 59616, 2988,  49840, 19256,
                53060, 58968,
            },
            {
                48048, 55052, 42992, 23500, 16044, 28340, 18960, 2312,  33540,
                61628, 48512, 1832,  29292, 58540, 15876, 448,   24676, 35864,
                56052, 2456,  31008, 14636, 57596, 47244, 26012, 4252,  8444,
                45524, 11060, 48816, 43372, 29752, 57488, 19364, 31708, 54324,
                21432, 29316, 52248, 2156,  23508, 13816, 51404, 5492,  11716,
                18500, 58208, 37360, 11224, 46868, 15720, 64080, 31344, 56008,
                46408, 3044,  34324, 26728, 42924, 49224, 30776, 25804, 336,
                11300, 35132, 26752, 14028, 41816, 46752, 28780, 2964,  25000,
                54336, 13796, 41152, 56884, 49948, 37868, 14832, 42792, 24192,
                38608, 61076, 47032, 6068,  14200, 62412, 37756, 960,   33164,
                54904, 10800, 59536, 25176, 52668, 8100,  36112, 12648, 33720,
                29484, 4120,  55384, 34788, 15308, 10564, 42144, 2472,  35436,
                26108, 42580, 59992, 31500, 24012, 46308, 37248, 3464,  59328,
                18792, 128,   24812, 11692, 17916, 43376, 22728, 10256, 31100,
                7524,  24832,
            },
            {
                35872, 5404,  61676, 51256, 3388,  48396, 37436, 59096, 24492,
                14608, 40496, 44540, 53544, 20100, 49568, 39656, 46268, 7916,
                20984, 48448, 18236, 53692, 37280, 21968, 60872, 50412, 29496,
                63796, 55504, 32632, 21332, 6992,  38672, 14164, 3700,  41248,
                8512,  59244, 34888, 19564, 55520, 64028, 40088, 27968, 43068,
                33552, 49096, 2728,  31920, 20420, 36196, 44088, 25236, 18888,
                6644,  23412, 64908, 53620, 8456,  38200, 18776, 13668, 62956,
                46500, 22392, 52112, 61368, 18396, 7992,  12908, 57504, 33480,
                48556, 21884, 8672,  32532, 19760, 1624,  62560, 27008, 33620,
                2696,  15968, 30132, 52924, 25712, 56072, 48276, 58880, 20768,
                51564, 43568, 35012, 19828, 124,   48568, 21980, 57348, 44248,
                50392, 17240, 39592, 22136, 58560, 31976, 24276, 46040, 60760,
                51952, 21144, 14124, 4060,  50056, 9312,  21012, 28752, 51812,
                34628, 39508, 55680, 7272,  37200, 28672, 61764, 46580, 39320,
                65044, 1972,
            },
            {
                27408, 14292, 19872, 34624, 8960,  63440, 11960, 43712, 7396,
                51908, 26600, 11616, 37480, 9056,  30688, 61272, 13288, 28716,
                63216, 33940, 44584, 8488,  27944, 928,   16744, 42676, 38428,
                14900, 1824,  36364, 16368, 59500, 49772, 27512, 47248, 56588,
                24800, 38412, 15488, 48872, 30660, 3688,  35324, 59864, 572,
                61800, 14772, 41260, 57060, 8064,  54368, 3820,  59416, 10836,
                37728, 50520, 29888, 17560, 1780,  60032, 55448, 33580, 49988,
                8632,  29552, 38328, 1908,  55344, 31692, 62828, 40812, 16772,
                38108, 180,   60636, 25240, 52972, 11820, 47812, 5664,  57104,
                44884, 8992,  63224, 40960, 19168, 34108, 4832,  12876, 41244,
                26916, 16500, 5580,  63108, 39416, 28276, 61104, 16164, 2204,
                9260,  63724, 46932, 1264,  53224, 5936,  51096, 16800, 8996,
                32736, 5712,  38276, 54708, 34296, 63136, 57492, 6504,  47460,
                10476, 61856, 31372, 49984, 58088, 33864, 1056,  15752, 56936,
                21444, 42240,
            },
            {
                60676, 45996, 39248, 57300, 25760, 31436, 55512, 21140, 35780,
                62580, 3812,  56316, 64296, 24096, 5140,  55300, 17540, 38296,
                58876, 4868,  12052, 40080, 64576, 32708, 54680, 12160, 58280,
                23776, 27852, 62404, 41620, 9716,  23268, 63424, 35284, 17480,
                1052,  64520, 11332, 45304, 26244, 8940,  16544, 46856, 20608,
                53172, 29456, 23264, 62756, 27372, 13220, 48792, 41772, 33380,
                61220, 14004, 40404, 47544, 11496, 44860, 23204, 4824,  39732,
                17016, 57436, 5296,  44656, 24740, 36960, 20408, 51836, 7212,
                64040, 30344, 45876, 15936, 58236, 34764, 39892, 17188, 53632,
                20108, 49348, 35456, 1776,  10864, 44772, 24156, 31460, 61704,
                2440,  49168, 57028, 13344, 45584, 10008, 31948, 53628, 42380,
                27020, 36396, 11788, 28848, 43136, 65020, 20348, 39820, 28488,
                56476, 64132, 43972, 25620, 17396, 39848, 14700, 43404, 26232,
                16336, 22588, 43060, 13764, 20464, 6152,  53696, 26540, 12336,
                33436, 52212,
            },
            {
                10184, 29912, 1132,  13020, 41568, 47164, 17164, 100,   29816,
                46424, 18252, 33152, 15624, 47632, 34900, 43900, 22564, 2168,
                49776, 25184, 52076, 57896, 19160, 49100, 6116,  35260, 19988,
                7332,  44448, 50660, 3244,  54080, 33224, 5192,  12128, 51264,
                43280, 31836, 6004,  41420, 58432, 50360, 56200, 24776, 38752,
                10420, 6860,  45016, 17456, 51700, 38648, 22688, 16676, 28144,
                52812, 4488,  21164, 32496, 63280, 36080, 27692, 52288, 61128,
                43440, 20688, 64144, 14860, 48820, 10876, 4468,  43592, 26684,
                12120, 55516, 36224, 42588, 6640,  29036, 22844, 64596, 31560,
                12620, 28180, 22548, 58136, 50948, 64920, 16888, 54280, 9428,
                38148, 29652, 22936, 33000, 54856, 24912, 4796,  37660, 19600,
                56416, 23328, 60568, 18516, 37860, 13300, 34228, 47712, 232,
                23380, 11608, 48840, 7824,  30072, 932,   52516, 32600, 64076,
                4236,  54556, 1992,  27288, 46996, 64172, 38704, 44084, 50364,
                3476,  17812,
            },
            {
                23852, 54512, 49428, 62312, 22284, 5040,  37812, 50704, 58740,
                23340, 8056,  42816, 28220, 1736,  60476, 52624, 10376, 32884,
                42632, 16080, 30392, 22836, 3744,  44252, 25668, 62124, 47444,
                53276, 31884, 11604, 25560, 19556, 45488, 40060, 58900, 20584,
                26828, 53396, 60772, 18704, 22296, 36716, 13096, 4884,  32580,
                64900, 50592, 35904, 1468,  33088, 6380,  64616, 2048,  56620,
                8764,  43892, 26404, 57772, 7376,  19496, 15016, 1184,  31336,
                12328, 35896, 27012, 33276, 53076, 60280, 29864, 47108, 59348,
                23604, 19356, 4136,  10572, 61904, 50868, 2196,  42104, 7520,
                60164, 43760, 39224, 14868, 26968, 7620,  36344, 47628, 19344,
                44292, 64200, 7924,  42700, 17668, 50192, 65376, 13044, 48028,
                5812,  33420, 49868, 7968,  54860, 26124, 4108,  61212, 53640,
                15656, 37032, 19428, 58196, 46000, 61084, 11908, 21628, 37932,
                48768, 35016, 60776, 40792, 10224, 16884, 24160, 30976, 8340,
                59716, 37580,
            },
            {
                5800,  33156, 18680, 7928,  28420, 53796, 65080, 10012, 13868,
                40788, 54292, 59808, 50872, 12512, 19660, 40240, 26872, 64940,
                55980, 9268,  36916, 62568, 13952, 39240, 10592, 29820, 1284,
                39712, 17860, 56952, 37636, 60724, 14964, 30104, 8780,  48356,
                37376, 13824, 2604,  34052, 7624,  63600, 29800, 43868, 17960,
                57744, 26104, 12544, 60344, 42424, 54968, 30856, 45288, 20116,
                36904, 62376, 49832, 2444,  53256, 41652, 57012, 47664, 24712,
                6500,  54964, 8968,  41436, 640,   22460, 16092, 35332, 2508,
                40276, 48432, 52432, 27248, 21424, 46900, 14168, 36472, 25372,
                51500, 3104,  5836,  55280, 31988, 42168, 420,   60480, 25464,
                12416, 52732, 3368,  36848, 60012, 1348,  21544, 40356, 29616,
                59336, 15020, 3448,  45108, 31080, 57856, 10436, 44168, 29764,
                41336, 62612, 2580,  35500, 22732, 27796, 42136, 5508,  56616,
                18652, 9096,  29404, 52360, 36624, 59132, 4100,  55968, 20552,
                46328, 63484,
            },
            {
                40676, 44864, 58584, 36548, 42984, 15468, 32744, 44968, 26768,
                34748, 4704,  20820, 37740, 31800, 7012,  46708, 14724, 5576,
                20692, 45884, 216,   47904, 32284, 56924, 51112, 21860, 59316,
                14356, 63984, 4304,  47072, 28384, 156,   65040, 55120, 3876,
                22892, 56712, 29044, 47704, 52068, 39552, 1748,  54724, 48752,
                40476, 3900,  21800, 47596, 18892, 9700,  24864, 48364, 12040,
                33680, 16276, 24092, 13304, 38076, 29128, 9400,  64508, 37284,
                50668, 60560, 46880, 18624, 56900, 39124, 62440, 9456,  54236,
                31540, 13732, 64924, 33044, 38484, 17768, 55752, 58952, 10832,
                33728, 63512, 18496, 46720, 59668, 20492, 50156, 28908, 5244,
                34672, 57852, 21176, 15264, 28548, 46724, 34636, 8864,  51796,
                43296, 25720, 64224, 40144, 16632, 22660, 49388, 18752, 7352,
                25168, 51356, 32524, 55340, 10132, 49808, 16996, 59832, 30460,
                43904, 13236, 23368, 5004,  19572, 49292, 14144, 41836, 35244,
                27668, 12540,
            },
            {
                3048,  25976, 14224, 51984, 1868,  24480, 56844, 19860, 2384,
                49504, 64052, 9764,  25376, 57400, 62092, 35708, 54256, 29464,
                59036, 40444, 26176, 53436, 7476,  16408, 35720, 42016, 9192,
                27416, 34084, 22636, 8248,  50100, 42148, 24412, 34304, 17028,
                44832, 63248, 40900, 10180, 15824, 26648, 61584, 20924, 8732,
                15284, 31340, 37600, 56516, 28880, 14620, 62084, 39532, 58724,
                4000,  42608, 31208, 46300, 60208, 22340, 33744, 43876, 17272,
                21408, 3768,  30700, 13676, 25436, 6228,  49760, 28048, 18460,
                57672, 7504,  44092, 1176,  60232, 8624,  30236, 44360, 20544,
                48108, 29192, 37808, 24508, 10016, 13556, 39016, 56396, 15888,
                46224, 30612, 40196, 51656, 11384, 56184, 25980, 62220, 2864,
                18796, 11088, 35592, 52660, 1604,  62988, 34604, 38400, 58728,
                12964, 4324,  43560, 14976, 6632,  64464, 36512, 2200,  25684,
                51572, 46684, 65232, 57336, 33572, 26304, 63232, 428,   48476,
                16584, 56892,
            },
            {
                22176, 64516, 31308, 10412, 61316, 48344, 7036,  39148, 60268,
                30456, 16504, 39692, 45272, 612,   17400, 22940, 49476, 2616,
                33912, 17832, 11576, 61172, 28480, 20024, 65284, 3364,  46584,
                52144, 43504, 55188, 36456, 12608, 18880, 51936, 39116, 10840,
                31412, 6828,  19908, 58816, 23544, 46132, 12204, 35404, 28520,
                59420, 45812, 64048, 7448,  50892, 35444, 1204,  53004, 23092,
                7684,  54576, 63588, 10216, 18336, 5844,  51444, 2704,  12776,
                40452, 27388, 44880, 65220, 52096, 32036, 42848, 12592, 46068,
                23172, 36484, 16200, 24860, 53648, 41180, 5292,  23880, 748,
                54068, 16144, 7728,  52820, 35364, 62532, 3320,  22456, 41844,
                60968, 9576,  24648, 62600, 6656,  42276, 31116, 16036, 54312,
                38464, 58116, 21308, 28708, 46652, 13884, 5072,  54516, 32000,
                47504, 22000, 60396, 26956, 46840, 31484, 40912, 54780, 7700,
                15868, 38460, 1344,  41496, 8572,  45092, 29828, 53776, 10520,
                32404, 51048,
            },
            {
                8520,  47340, 19780, 40084, 34508, 17888, 28892, 11484, 52436,
                22920, 46920, 12872, 54040, 28012, 43104, 9444,  39064, 13196,
                63912, 51648, 23464, 36616, 44124, 5960,  54396, 25092, 32108,
                18716, 5132,  15820, 60416, 31268, 57712, 6204,  62536, 47144,
                26988, 50640, 36640, 1120,  32268, 53712, 3760,  41812, 51120,
                23272, 108,   11664, 41712, 25688, 5464,  44012, 29952, 17732,
                50064, 27848, 39352, 260,   48992, 27120, 55572, 62184, 31824,
                53492, 57984, 10472, 37676, 2636,  20296, 35848, 63756, 1600,
                39612, 59404, 51484, 29324, 12188, 35152, 49276, 64428, 38884,
                12812, 60788, 41544, 1864,  30384, 44192, 54552, 33052, 7052,
                50536, 18420, 780,   48172, 37628, 19624, 4812,  45452, 23728,
                47892, 32208, 6028,  9436,  56552, 24520, 42064, 27276, 9136,
                65336, 39876, 18004, 37136, 53320, 20128, 23740, 10884, 33768,
                62516, 20576, 31792, 54488, 12568, 17564, 39168, 6688,  24668,
                62156, 38384,
            },
            {
                42132, 54028, 27500, 3788,  45400, 55128, 63340, 42780, 35632,
                3720,  57148, 33344, 6132,  50312, 63392, 31196, 58012, 25836,
                44892, 8212,  42320, 2376,  59748, 48748, 13384, 38888, 58128,
                11276, 62008, 40520, 26172, 1732,  44312, 22228, 14904, 2184,
                60948, 13092, 54988, 43748, 65332, 38008, 18552, 60808, 6244,
                55692, 33996, 19904, 53636, 16660, 60464, 21228, 37712, 64532,
                12420, 35160, 20392, 57196, 43016, 36296, 15572, 24288, 47012,
                7904,  34592, 23212, 16780, 47524, 60308, 8956,  54876, 26652,
                5488,  20656, 47308, 3640,  62296, 20156, 15500, 56816, 31584,
                25740, 45620, 57624, 21148, 49652, 17256, 25400, 10992, 63672,
                27308, 36176, 54284, 32336, 13420, 64468, 57188, 35228, 12832,
                284,   61724, 51040, 43688, 36696, 61252, 20164, 51636, 16240,
                2824,  29460, 55920, 5924,  424,   13396, 58788, 43472, 48332,
                56836, 28124, 5880,  24780, 50472, 61824, 22368, 58464, 35980,
                18656, 1792,
            },
            {
                15364, 60628, 6256,  58276, 13212, 22236, 884,   15856, 25864,
                8668,  62844, 18724, 22480, 36308, 15252, 4960,  20748, 47948,
                3580,  31788, 54888, 15728, 30060, 22748, 34436, 680,   50060,
                29748, 23224, 47692, 53800, 10408, 33828, 28040, 53468, 36016,
                41872, 24772, 17792, 5840,  28052, 9476,  49004, 31156, 14064,
                44340, 27488, 62260, 37336, 31740, 47148, 56956, 8964,  45588,
                2764,  59844, 25340, 14112, 32040, 60700, 10960, 39464, 19732,
                1068,  63036, 41752, 5832,  53732, 24752, 14504, 30324, 50616,
                61376, 34160, 10120, 43128, 55676, 26520, 45972, 9656,  4308,
                19012, 34032, 10784, 27908, 65176, 4608,  47956, 38440, 14236,
                45260, 3956,  59592, 21740, 44372, 25056, 50600, 9296,  59868,
                41672, 26908, 14928, 18324, 31624, 11564, 1148,  59148, 44852,
                35688, 49864, 10580, 42824, 63640, 50816, 29712, 17752, 3340,
                36420, 13992, 46184, 60128, 34864, 2504,  48076, 13820, 44452,
                56260, 29960,
            },
            {
                49884, 33256, 36864, 24200, 51216, 32072, 37988, 50024, 59744,
                44224, 29896, 40916, 55556, 11208, 60136, 42068, 53472, 17100,
                36020, 61988, 21192, 39884, 10388, 57280, 45760, 16952, 63516,
                8028,  37400, 3572,  19832, 64772, 38140, 48952, 59780, 20300,
                9304,  57808, 34732, 47928, 14528, 22424, 56136, 25476, 39188,
                17684, 2936,  48500, 8628,  13108, 3500,  27224, 15292, 32784,
                41220, 47568, 4780,  53104, 8200,  45908, 3336,  30088, 56412,
                50480, 13616, 29196, 59080, 32704, 39412, 44756, 18144, 7652,
                41740, 13500, 31244, 17744, 37084, 6472,  33232, 52488, 42660,
                62972, 50568, 5820,  40028, 14668, 34796, 56576, 20096, 30124,
                57724, 40500, 16464, 10384, 28996, 1920,  39436, 17264, 29876,
                20564, 37444, 64784, 3616,  54348, 47432, 28900, 39244, 23652,
                12660, 58048, 25584, 20796, 34260, 38680, 7020,  60932, 26156,
                9300,  53088, 42428, 18992, 10236, 30772, 40616, 4132,  52508,
                26692, 11924,
            },
            {
                21696, 44060, 16904, 9032,  65340, 41172, 6552,  54384, 20568,
                13628, 49076, 1536,  26772, 46024, 2748,  33684, 28648, 9112,
                57084, 13736, 27720, 50548, 64460, 7076,  41240, 26808, 20816,
                43192, 55712, 33036, 14096, 45320, 7692,  17092, 3144,  32012,
                46016, 4672,  29848, 52916, 61384, 42176, 6692,  63960, 10764,
                52652, 58224, 40944, 24064, 55376, 42532, 51672, 62332, 23980,
                55824, 17328, 29384, 62720, 23168, 18840, 41344, 64612, 26300,
                36400, 43996, 21032, 49244, 10856, 552,   57604, 34824, 48300,
                23828, 57136, 52784, 65416, 356,   40468, 61032, 21832, 28264,
                1656,  36784, 55228, 23648, 59384, 8168,  41968, 2536,  52552,
                23200, 5816,  34476, 51876, 61332, 55380, 33204, 4096,  48472,
                53332, 7136,  45784, 24952, 40352, 8052,  63696, 33308, 19060,
                6988,  61832, 32308, 48244, 15492, 24320, 45780, 54176, 40416,
                21964, 64356, 584,   38576, 57264, 27028, 64956, 20048, 33548,
                5592,  63732,
            },
            {
                40140, 172,   55284, 46656, 29668, 18972, 11072, 28072, 33860,
                4740,  37352, 58636, 16300, 51588, 38268, 65012, 24300, 49688,
                38848, 5476,  46796, 1252,  18588, 32764, 53416, 4196,  60016,
                11928, 50644, 27872, 59088, 24272, 51432, 41408, 12840, 55396,
                23148, 63844, 39768, 416,   20212, 36368, 32180, 1808,  46136,
                35020, 21600, 4772,  29440, 64872, 35128, 19792, 984,   11168,
                36820, 6940,  50840, 38712, 34452, 49560, 54424, 6036,  16268,
                9184,  61464, 4452,  17368, 26796, 62268, 22060, 4912,  63880,
                28312, 3440,  21300, 48600, 24768, 14612, 49736, 11404, 57320,
                15368, 47328, 18208, 30664, 44816, 50884, 26744, 61612, 9772,
                46872, 64668, 25448, 42316, 7284,  19368, 45976, 62788, 23928,
                56920, 11888, 34500, 57864, 21652, 13976, 49324, 4924,  53580,
                46048, 40036, 2160,  54844, 8804,  62572, 1688,  12392, 33120,
                49848, 30176, 16416, 23616, 51896, 14600, 8156,  42860, 59408,
                46888, 9648,
            },
            {
                52128, 31368, 61524, 26224, 4052,  57612, 43272, 61780, 46512,
                24108, 64432, 10052, 31832, 23184, 19280, 7164,  12708, 59008,
                44008, 22532, 34896, 55916, 25004, 37616, 14440, 48256, 31056,
                35856, 16476, 6128,  39384, 548,   30704, 62420, 26888, 37216,
                49800, 16008, 11184, 26152, 58204, 44792, 13308, 50440, 28284,
                62472, 14596, 32992, 49908, 17252, 5780,  39380, 48884, 31312,
                59524, 21524, 43684, 10312, 808,   59236, 12984, 32756, 48004,
                23580, 52836, 33884, 56092, 43056, 36680, 51652, 12256, 40388,
                16088, 38048, 8484,  44296, 28864, 59540, 7512,  31132, 38784,
                24500, 8816,  61448, 12300, 112,   21708, 33264, 13552, 37348,
                18728, 30880, 12900, 49176, 37688, 14488, 27504, 10160, 43172,
                16104, 30992, 1424,  50320, 28032, 59940, 17236, 37256, 57308,
                27312, 14412, 29872, 18456, 43708, 36772, 28248, 57556, 19440,
                5308,  59304, 45172, 6248,  47572, 36032, 55108, 22932, 16816,
                37172, 24784,
            },
            {
                15004, 5124,  20484, 12676, 37612, 48668, 15328, 1324,  55944,
                17720, 40568, 53064, 5436,  43116, 56120, 47008, 32424, 1768,
                17488, 63124, 30572, 12268, 60596, 44528, 8400,  63048, 23424,
                2044,  58032, 46860, 63680, 18576, 53724, 21768, 10040, 60532,
                6588,  43176, 34060, 51552, 17184, 8440,  59876, 23696, 18920,
                38312, 9164,  60144, 43932, 11812, 23208, 57616, 45180, 27916,
                54056, 13844, 65384, 25984, 16696, 28532, 21380, 44612, 38036,
                58844, 1844,  39652, 29880, 8236,  15280, 45244, 31564, 55452,
                46852, 60088, 33500, 12096, 54128, 35328, 20308, 45592, 64608,
                54628, 33092, 42296, 52928, 38340, 64044, 16928, 49012, 55176,
                1312,  44136, 56656, 3652,  22556, 58692, 52724, 35660, 5448,
                39996, 60548, 18684, 43816, 10020, 32568, 42032, 604,   21008,
                9496,  51900, 64952, 4072,  59012, 21632, 51312, 15164, 42692,
                37512, 11116, 34312, 28304, 62920, 2924,  31392, 49680, 1704,
                28688, 57208,
            },
            {
                48492, 35604, 41500, 59352, 53488, 32492, 22592, 36428, 7912,
                27068, 12844, 34676, 61016, 28116, 14604, 62544, 40856, 26624,
                53964, 9928,  42288, 52324, 3660,  21064, 28868, 39032, 55204,
                19320, 41832, 25592, 11520, 35280, 42928, 4760,  33504, 45392,
                19796, 28496, 56660, 3564,  47636, 30192, 40000, 53644, 5956,
                56296, 41680, 568,   53860, 27356, 61516, 8548,  15980, 4520,
                41564, 2284,  33104, 46436, 57176, 40624, 63508, 3508,  10500,
                27784, 19284, 12624, 47352, 65116, 25300, 2484,  20508, 9712,
                25888, 1268,  18604, 40172, 57880, 2108,  16444, 41472, 5428,
                2560,  19400, 26512, 6136,  56832, 29176, 3852,  41124, 60756,
                27880, 35960, 9092,  62300, 31728, 41264, 1104,  65108, 21040,
                26072, 52256, 36228, 64116, 3832,  54572, 25748, 63068, 44280,
                35368, 23316, 41528, 33248, 49156, 10088, 31648, 4392,  46928,
                63820, 25432, 55952, 13456, 40556, 18572, 10432, 61728, 39200,
                12304, 63000,
            },
            {
                18788, 8260,  45292, 23712, 2408,  9820,  63488, 29976, 52152,
                42744, 48024, 20640, 236,   49124, 9684,  36240, 4380,  50428,
                20736, 37460, 6288,  27064, 16820, 49656, 59168, 11052, 45472,
                6664,  52268, 31664, 7940,  56036, 14616, 48008, 58688, 1256,
                53016, 13996, 64636, 37504, 20632, 62064, 2316,  33984, 15704,
                48072, 25920, 20764, 31004, 47224, 37584, 52344, 34460, 24976,
                60508, 19592, 52636, 8368,  36248, 5068,  49184, 31120, 53608,
                62452, 41940, 51228, 22720, 5144,  57996, 52688, 61040, 34408,
                49180, 63060, 30476, 50788, 6312,  23660, 61808, 47996, 27896,
                36376, 51400, 45420, 14844, 34984, 9960,  45932, 24884, 7108,
                20792, 51800, 15100, 26312, 17716, 50544, 12136, 29932, 56252,
                48624, 8280,  13524, 23440, 39440, 47212, 12352, 31028, 15684,
                48584, 57868, 6144,  13356, 38652, 26268, 54064, 61168, 22764,
                1168,  52704, 8388,  21004, 59040, 44184, 26140, 53360, 45956,
                5872,  32840,
            },
            {
                27256, 52644, 64696, 29040, 17144, 40628, 50352, 19232, 60740,
                3148,  58384, 30908, 38800, 54624, 17688, 22784, 29916, 57416,
                14424, 59712, 47452, 65496, 35544, 41400, 148,   32860, 15548,
                27588, 36720, 61324, 22432, 49936, 27708, 38696, 17464, 30108,
                40304, 23720, 9412,  42464, 25548, 12400, 44156, 27788, 65160,
                11048, 36052, 58680, 3776,  14328, 6544,  20384, 63528, 11680,
                38504, 49060, 30168, 23136, 54952, 12184, 18884, 24748, 15524,
                35552, 7364,  56648, 16788, 37268, 30860, 41060, 17592, 7028,
                42496, 21796, 14376, 46032, 26700, 38204, 52008, 13264, 21876,
                57192, 11000, 63584, 23236, 48760, 58940, 19484, 53484, 11932,
                32896, 59300, 47888, 37240, 45396, 6996,  34484, 44120, 15512,
                2664,  45664, 28964, 56828, 5868,  19924, 51344, 7660,  60908,
                2780,  28760, 19520, 46736, 63272, 17076, 7304,  34948, 39896,
                17628, 31068, 48380, 36504, 51044, 4532,  33976, 15556, 20772,
                55812, 43216,
            },
            {
                58784, 11748, 4640,  34216, 56536, 46168, 6064,  12240, 38136,
                24324, 16452, 6980,  64036, 25472, 44228, 60340, 7688,  45984,
                33700, 1200,  23584, 31192, 13232, 57468, 25108, 53564, 64152,
                48812, 2952,  13600, 44344, 1612,  65188, 24144, 10900, 61476,
                49616, 4936,  33200, 54200, 6628,  56916, 49964, 22196, 4972,
                52012, 45712, 16964, 63116, 33524, 40612, 50264, 29100, 56228,
                17524, 44236, 5556,  15052, 61228, 42920, 38292, 57692, 45388,
                832,   26000, 32688, 44044, 10092, 48348, 27532, 13408, 37672,
                57256, 4756,  54356, 10532, 63948, 31388, 9340,  33576, 60428,
                6960,  30576, 39020, 17612, 4492,  42388, 30372, 37724, 65492,
                40596, 4768,  23344, 2392,  63904, 54376, 20232, 24132, 61216,
                38776, 32716, 62508, 16600, 33800, 59472, 36888, 23904, 54868,
                39012, 11100, 52680, 24548, 104,   56512, 45164, 12952, 28592,
                59676, 42188, 14956, 2724,  23080, 29504, 60716, 7956,  36788,
                23800, 988,
            },
            {
                16560, 50484, 21060, 38760, 13780, 59984, 26496, 32980, 44636,
                9212,  56496, 46116, 12124, 33296, 2624,  51416, 38300, 11712,
                42084, 18472, 51960, 8208,  45176, 5452,  18128, 38244, 8828,
                20444, 58380, 39816, 17928, 57356, 32832, 6464,  54744, 35360,
                21104, 57968, 46440, 16704, 30968, 36520, 18264, 58980, 38084,
                31828, 24176, 8616,  42736, 55068, 23804, 1072,  45968, 9140,
                3232,  35644, 64216, 27740, 33936, 2164,  51756, 8640,  60736,
                20844, 50112, 63952, 3544,  59424, 21604, 196,   64344, 50516,
                24344, 28872, 36432, 43764, 17012, 3864,  55200, 44408, 18372,
                49968, 41896, 1180,  55312, 27048, 61180, 13716, 700,   47100,
                16608, 29360, 56876, 11488, 39432, 27820, 57876, 9776,  51320,
                5320,  19620, 10692, 52884, 43124, 2372,  28000, 45380, 18036,
                33484, 42520, 59100, 36528, 30760, 41644, 20712, 50388, 9276,
                55460, 6176,  45464, 65224, 56440, 11952, 41276, 49112, 64108,
                30580, 40128,
            },
            {
                62160, 32172, 44368, 24304, 49204, 508,   20148, 55328, 65048,
                29060, 35440, 21260, 52760, 41636, 15084, 20032, 63780, 24692,
                28508, 60844, 56336, 39688, 21340, 50972, 62068, 29636, 43604,
                24060, 33788, 53880, 30260, 9860,  42008, 47020, 15428, 2496,
                43856, 27676, 12864, 63316, 48596, 488,   9980,  41656, 14256,
                1848,  55832, 29392, 48908, 12116, 18268, 59320, 32044, 41976,
                26084, 57328, 50688, 9940,  46688, 21516, 26680, 32260, 14000,
                40212, 11004, 28564, 53740, 14856, 34600, 55044, 44496, 11080,
                32648, 2584,  61132, 20216, 58216, 41320, 22740, 1812,  37464,
                14708, 24920, 62380, 33328, 8924,  50992, 34700, 24000, 55000,
                9204,  43452, 50268, 19252, 32188, 14496, 52,    42668, 36340,
                26748, 55400, 40672, 24788, 49396, 14148, 65508, 10148, 5020,
                62520, 22328, 14516, 7016,  60668, 26896, 3692,  64208, 37232,
                24120, 19652, 33692, 26016, 38996, 18060, 53872, 5200,  13592,
                47112, 9168,
            },
            {
                35784, 2248,  54400, 7604,  62820, 36300, 41328, 3884,  14040,
                51192, 39908, 1232,  26764, 59516, 30240, 5016,  47264, 54164,
                3128,  15828, 36452, 11244, 27628, 33360, 3064,  14024, 47900,
                4836,  11360, 61624, 3816,  21336, 37100, 51592, 26088, 63920,
                38544, 19076, 3300,  39804, 24044, 60812, 28972, 53452, 63532,
                44748, 19720, 61008, 5256,  38248, 27200, 65016, 15104, 54004,
                22396, 12588, 19304, 40264, 58452, 16640, 63020, 4736,  55644,
                47944, 36332, 18544, 41572, 24408, 38836, 8092,  19560, 59720,
                46808, 15396, 52192, 7832,  34784, 26168, 47616, 65180, 28320,
                58472, 46108, 12384, 52848, 21404, 44548, 6600,  18996, 62880,
                26516, 34096, 61596, 5756,  52856, 46164, 60312, 30920, 17020,
                64640, 46980, 1508,  31228, 8108,  21392, 40176, 47732, 30188,
                51056, 1772,  44216, 49484, 10232, 52988, 15776, 32112, 47532,
                1432,  61480, 52228, 9544,  31812, 300,   44908, 27560, 20184,
                56524, 25488,
            },
            {
                18484, 57800, 15028, 27360, 10912, 30728, 52788, 23040, 48228,
                17276, 5952,  62636, 47856, 10596, 37576, 57236, 35216, 9512,
                32084, 49252, 6420,  64348, 46568, 54508, 42072, 59892, 35504,
                52564, 26440, 44988, 49844, 28576, 62588, 18400, 11996, 31700,
                8308,  50556, 59768, 34580, 7152,  45188, 20836, 33376, 25628,
                7712,  39964, 34472, 22644, 51580, 9520,  35744, 3920,  48576,
                38756, 62660, 48,    30988, 7104,  49564, 37652, 44464, 29692,
                22756, 6808,  61392, 5288,  49416, 62644, 31472, 27024, 5656,
                40532, 22804, 38104, 48784, 30296, 13152, 5740,  52576, 10036,
                20248, 6088,  36236, 29444, 15528, 39784, 57948, 48808, 37196,
                2480,  13568, 21900, 41072, 35932, 24448, 4412,  49792, 21864,
                7292,  12828, 58780, 37296, 61560, 54484, 34144, 57560, 25636,
                16424, 55428, 32396, 18932, 35636, 23164, 39252, 54588, 11740,
                27776, 40920, 13896, 48640, 58056, 22252, 60188, 35008, 51912,
                3384,  42592,
            },
            {
                50036, 29500, 40984, 47708, 58644, 17708, 43896, 8496,  27956,
                60528, 33876, 19104, 54532, 7204,  23404, 16828, 42456, 21680,
                62248, 44208, 19476, 25192, 896,   16488, 23408, 7920,  19868,
                64864, 16984, 37844, 14436, 7536,  54980, 364,   58404, 45820,
                53780, 22796, 30028, 55600, 11644, 51272, 15212, 4576,  47316,
                12644, 57956, 16180, 3004,  46576, 57512, 44024, 20524, 7648,
                33416, 28356, 44980, 55072, 23988, 34292, 12308, 1272,  59872,
                15604, 51788, 32464, 45860, 1512,  12704, 43292, 52992, 17444,
                56248, 63356, 11972, 1012,  62264, 55900, 16612, 35480, 31544,
                43020, 56420, 3416,  48208, 64688, 1960,  28148, 11056, 30936,
                53240, 45904, 58608, 8448,  17316, 63056, 10852, 39564, 53576,
                34200, 43588, 23060, 15408, 27616, 18232, 456,   12628, 37664,
                7988,  40400, 27924, 64712, 47116, 5240,  61188, 43836, 18332,
                58808, 35868, 4408,  17180, 43160, 6972,  38060, 15636, 31376,
                61636, 12272,
            },
            {
                37288, 65424, 4012,  22472, 34924, 5440,  63652, 56140, 37704,
                11468, 42812, 24948, 31444, 44928, 64556, 51884, 388,   27036,
                12976, 40188, 52908, 30024, 58028, 39176, 11628, 56432, 31936,
                40408, 2568,  57156, 22608, 42876, 34072, 24112, 40952, 36128,
                4024,  14664, 42852, 17844, 26824, 40988, 56984, 62324, 36768,
                50444, 28452, 64372, 54480, 31588, 14348, 29300, 61244, 52376,
                16812, 59212, 5084,  15448, 41160, 65228, 20136, 53376, 25524,
                42712, 9888,  56100, 26584, 16796, 57820, 22408, 36844, 2276,
                33732, 28504, 43168, 25188, 19128, 44752, 40072, 60804, 49512,
                23948, 62000, 39112, 18736, 54840, 23480, 42796, 60412, 17196,
                39000, 4184,  25280, 29884, 55884, 48204, 27732, 15172, 59860,
                29400, 56044, 4952,  50708, 46252, 6636,  52496, 44404, 63184,
                23244, 58932, 11592, 2836,  56196, 14212, 29600, 8244,  2544,
                22684, 45696, 63768, 29684, 54268, 25384, 64552, 9984,  46200,
                22800, 6536,
            },
            {
                44760, 9420,  19412, 53908, 39264, 12768, 25380, 32276, 1744,
                46288, 58708, 3540,  14736, 39748, 11916, 29212, 48392, 59268,
                55196, 4748,  33856, 9748,  61420, 35968, 48636, 28128, 51304,
                6384,  46988, 29588, 10328, 60068, 47760, 5796,  16344, 27768,
                61148, 9824,  48044, 65428, 1932,  35608, 29860, 18580, 996,
                23056, 9808,  43288, 19272, 40748, 688,   23736, 39404, 10780,
                25596, 47564, 36600, 51920, 10628, 27604, 57316, 6012,  39388,
                34588, 64196, 20744, 37912, 30512, 50748, 10068, 65360, 48692,
                14500, 8688,  54308, 50152, 32328, 4980,  22244, 8436,  492,
                15288, 11796, 26668, 32388, 9432,  35532, 14344, 5332,  56176,
                20884, 49872, 64628, 42492, 1300,  33400, 20412, 44992, 3092,
                9816,  18980, 41040, 64252, 32092, 38952, 60124, 20172, 32996,
                4708,  51976, 45704, 21480, 41972, 25408, 34584, 49328, 57064,
                31672, 51316, 8884,  20244, 34264, 12604, 49636, 2632,  39992,
                54924, 26508,
            },
            {
                13772, 51844, 31272, 59936, 2592,  47080, 51000, 20436, 16280,
                52456, 22012, 36048, 49708, 60184, 5328,  33636, 18592, 8228,
                36612, 15616, 22376, 50624, 3944,  18628, 44844, 1728,  14876,
                24628, 62740, 35264, 53092, 18660, 30980, 64656, 50144, 20440,
                56300, 31320, 37344, 5264,  22400, 59560, 8008,  43768, 53336,
                38904, 32776, 5848,  26364, 11660, 62916, 49508, 5324,  56180,
                42612, 2052,  22652, 61096, 7748,  43260, 31200, 46552, 18240,
                13132, 1940,  44876, 7756,  60260, 40956, 4984,  29276, 45100,
                20260, 59552, 3380,  36276, 11008, 58832, 27072, 47004, 29728,
                51640, 42268, 53816, 6944,  46940, 61472, 51380, 25884, 44708,
                33988, 10360, 15564, 36880, 12736, 52196, 6924,  37312, 63740,
                49172, 25812, 35892, 2492,  12088, 25016, 9196,  28804, 14464,
                48292, 35912, 17048, 31364, 9256,  52312, 63464, 19980, 40656,
                12220, 26180, 38824, 60484, 1384,  41664, 56988, 28928, 19080,
                59048, 34396,
            },
            {
                760,   62616, 24440, 43164, 15300, 28904, 61176, 40368, 6280,
                64820, 29064, 8812,  55252, 26216, 21348, 61912, 39260, 45708,
                24908, 64212, 46984, 42476, 13400, 25912, 63100, 34036, 58348,
                20904, 43844, 13112, 1356,  26024, 8676,  38044, 12396, 1684,
                41916, 24988, 51720, 45276, 54520, 11240, 48924, 26048, 13460,
                56700, 61660, 48068, 52244, 36732, 58840, 32616, 18064, 35180,
                64500, 12888, 19196, 33028, 48848, 16784, 3624,  51176, 61932,
                28784, 49860, 24460, 53496, 14360, 18964, 35204, 25344, 56624,
                38700, 24016, 41748, 16084, 64708, 52720, 38964, 56908, 18440,
                64072, 34632, 21752, 58532, 16824, 40208, 916,   31380, 62776,
                7004,  28212, 55016, 47176, 22908, 61412, 56708, 41588, 23376,
                31916, 14288, 52620, 57200, 21240, 43224, 49896, 56824, 40892,
                1852,  26656, 62316, 38632, 59448, 404,   16124, 36760, 3896,
                62056, 15356, 5652,  52676, 46904, 16228, 21660, 33128, 4904,
                48236, 16980,
            },
            {
                28192, 46364, 38088, 7404,  33928, 57688, 9832,  44304, 34300,
                12560, 56860, 42124, 1452,  16028, 43672, 53540, 3196,  30480,
                58160, 1596,  28404, 56756, 31880, 53388, 8984,  41104, 54352,
                6844,  38232, 60972, 50572, 41132, 57824, 45168, 53700, 34556,
                63128, 7564,  12760, 19544, 32920, 39912, 16340, 64240, 30716,
                20592, 3488,  16524, 22220, 44436, 8060,  14888, 46392, 27060,
                53072, 30204, 40168, 56452, 25724, 38144, 60292, 23064, 9012,
                37036, 56600, 4076,  33516, 62908, 47972, 400,   52828, 11556,
                6576,  61380, 30680, 47624, 20740, 1856,  14180, 32816, 10560,
                37748, 5216,  2348,  44784, 28468, 10868, 22124, 48616, 17900,
                41212, 59344, 2216,  19292, 30864, 9548,  26992, 16720, 480,
                58888, 45992, 7480,  29700, 62704, 16432, 34360, 5628,  64972,
                18860, 53588, 6916,  13172, 44188, 27840, 46680, 23864, 54316,
                48360, 33900, 43572, 22996, 27696, 37116, 63008, 53284, 11504,
                60900, 41788,
            },
            {
                56888, 20732, 11340, 54820, 18564, 22812, 32,    27056, 49008,
                24340, 38172, 19148, 32488, 48000, 35748, 9360,  13956, 51164,
                17920, 11764, 39620, 19672, 5700,  37928, 22104, 16920, 30436,
                11036, 27936, 16580, 32324, 20704, 3512,  15444, 29340, 22192,
                47216, 17148, 58212, 28860, 62116, 24572, 2872,  35380, 45528,
                7200,  41632, 34512, 56212, 2140,  29840, 54560, 21120, 3012,
                9384,  45240, 6084,  62500, 764,   11428, 53280, 33848, 15140,
                41256, 19756, 43344, 11756, 28152, 22572, 58924, 42180, 32432,
                17804, 50848, 4472,  27644, 34608, 43880, 24188, 6448,  46036,
                60200, 25808, 49136, 31912, 63276, 38064, 54464, 57884, 13248,
                35736, 24328, 51064, 39844, 44588, 4256,  50028, 34812, 53828,
                11212, 42776, 18524, 37144, 47324, 1420,  55292, 23540, 45220,
                30288, 58080, 24368, 49464, 33060, 56592, 6060,  10988, 42228,
                28536, 18664, 56032, 10244, 59760, 4164,  8460,  44264, 38492,
                23796, 6236,
            },
            {
                35884, 51100, 3752,  64704, 48508, 37052, 52984, 63168, 17300,
                3856,  51664, 62360, 11516, 58504, 28024, 65004, 23688, 42360,
                33084, 54660, 8556,  63376, 48844, 59724, 3068,  50888, 65344,
                45004, 48704, 5048,  55320, 36104, 62524, 25140, 10004, 60252,
                3972,  40364, 36444, 264,   43152, 6460,  52296, 58756, 10364,
                50208, 59840, 28388, 12944, 65268, 25032, 40256, 60912, 36152,
                58272, 23400, 51136, 14764, 42748, 20336, 27460, 45456, 2288,
                64904, 30848, 58296, 5772,  46204, 36736, 8620,  15260, 64316,
                45500, 37068, 12804, 57788, 8768,  49452, 61752, 55588, 41324,
                20980, 15756, 55056, 12820, 19416, 5988,  25588, 3192,  43804,
                9020,  63428, 5504,  33048, 57828, 65072, 13744, 38820, 21496,
                63088, 27320, 4036,  59480, 10200, 26428, 39308, 13612, 8396,
                37640, 11020, 41840, 3292,  21368, 18132, 65348, 31112, 59632,
                8044,  872,   63868, 31784, 13980, 50712, 30676, 25916, 14952,
                49996, 31580,
            },
            {
                61964, 16152, 25788, 41380, 29648, 5112,  14520, 39588, 58188,
                29972, 7724,  44916, 23088, 5932,  54608, 19700, 38316, 4188,
                61200, 45056, 35888, 24580, 15412, 27752, 43240, 34820, 24868,
                244,   58860, 23024, 42628, 7844,  47880, 52188, 43636, 32440,
                54816, 25716, 50676, 14728, 56024, 47860, 21644, 38596, 27108,
                17852, 23460, 37888, 8708,  42724, 50088, 5472,  47836, 13680,
                39040, 17360, 34104, 28940, 48548, 36380, 59068, 54824, 7324,
                48240, 26004, 16236, 39236, 54692, 18304, 49852, 27404, 3008,
                23596, 53380, 19664, 63536, 39860, 15472, 30196, 2524,  27368,
                51440, 8292,  35680, 59640, 42800, 51748, 33764, 47416, 30320,
                20928, 53476, 16364, 11588, 25304, 18448, 46804, 29420, 4828,
                48456, 32872, 40224, 22184, 53308, 31072, 51152, 61616, 21084,
                33840, 51584, 16032, 60540, 35796, 40308, 50836, 14332, 38208,
                21788, 50304, 35584, 40560, 45508, 19960, 58620, 1000,  64412,
                19372, 9320,
            },
            {
                1720,  44848, 33488, 12856, 57232, 46176, 32364, 10284, 21592,
                47280, 35988, 15256, 41000, 31024, 620,   45576, 49364, 26464,
                6848,  21096, 52332, 848,   32560, 46600, 9508,  13876, 56740,
                39632, 19324, 12216, 63996, 30000, 13700, 18328, 39088, 6540,
                20140, 11120, 64716, 33892, 9336,  18492, 31736, 12524, 55060,
                800,   63308, 46876, 53372, 15884, 33284, 19004, 10608, 31432,
                63452, 1556,  55776, 8304,  64320, 4092,  17596, 32064, 13016,
                22120, 51372, 9828,  63244, 31652, 1636,  61140, 40164, 56328,
                29732, 7608,  42832, 624,   22792, 52364, 37184, 18260, 12144,
                64676, 29596, 39676, 176,   23496, 9840,  65432, 14836, 60192,
                39524, 48972, 28712, 37980, 43196, 1752,  55508, 59972, 9176,
                17308, 56400, 12928, 62392, 44444, 15504, 3708,  48040, 56748,
                956,   63348, 46444, 29368, 8600,  53864, 1840,  24568, 44660,
                55004, 16624, 26396, 3400,  24056, 54592, 39364, 34280, 46640,
                55744, 40784,
            },
            {
                29200, 54196, 60108, 21188, 8180,  24756, 62048, 43412, 1896,
                54800, 60264, 26332, 52604, 63320, 17224, 34648, 12668, 58804,
                15740, 29432, 41308, 11956, 57756, 62224, 20872, 53992, 6976,
                31796, 36908, 51848, 26848, 2240,  34284, 61308, 3284,  58356,
                49340, 30220, 45384, 22468, 60896, 28108, 41892, 62336, 44664,
                33804, 6148,  30368, 20916, 1620,  61760, 55528, 28056, 52160,
                43752, 25764, 46128, 22056, 39272, 10588, 24924, 44128, 40412,
                60056, 35852, 3532,  43820, 23372, 13064, 34364, 20628, 10788,
                48416, 35468, 55208, 26756, 32548, 10632, 57480, 47604, 33968,
                43692, 4180,  48148, 16764, 56352, 28600, 37260, 18468, 7160,
                26212, 648,   56284, 60512, 8016,  33808, 22604, 41288, 36648,
                51384, 24428, 936,   34988, 6804,  19644, 36172, 42184, 28232,
                17748, 25172, 5024,  55672, 12428, 26812, 47384, 33652, 4296,
                61788, 11988, 47620, 57804, 7416,  11252, 16900, 28088, 5512,
                13124, 23180,
            },
            {
                49392, 10548, 39044, 3188,  51292, 18368, 53836, 27468, 34576,
                13128, 19508, 3548,  10080, 38728, 56960, 8152,  51596, 31928,
                63828, 47060, 55624, 18464, 38252, 5316,  42232, 29332, 17952,
                47376, 60580, 9180,  45536, 40104, 54344, 22416, 27984, 36608,
                16108, 41504, 7100,  2024,  37776, 52792, 3316,  7852,  16460,
                51520, 40180, 11632, 58684, 35788, 24644, 39536, 3732,  21344,
                7288,  60328, 12364, 53080, 30696, 57620, 50576, 61900, 96,
                28604, 55372, 18760, 26920, 47456, 52392, 57912, 4500,  44648,
                62996, 13740, 16732, 60516, 46156, 3968,  62804, 6964,  22128,
                54096, 25012, 61440, 32348, 44872, 49912, 2552,  41988, 55232,
                46104, 35220, 14132, 19996, 45796, 50612, 27508, 12548, 5716,
                64436, 42952, 47220, 27724, 54872, 58820, 24652, 11216, 7596,
                44640, 32820, 39396, 21996, 43608, 61296, 17624, 58156, 9676,
                36896, 30304, 41808, 64504, 32796, 49408, 61668, 43200, 52444,
                59672, 37108,
            },
            {
                4800,  17112, 26944, 63704, 42292, 36588, 15552, 6680,  49248,
                65168, 41960, 32740, 46728, 21640, 28352, 42728, 23868, 1916,
                37308, 9932,  3736,  27412, 50984, 23792, 35756, 50004, 64256,
                4580,  14924, 24244, 58284, 16308, 48876, 10480, 44608, 62808,
                12504, 55804, 26580, 59116, 46748, 15196, 57500, 35824, 25124,
                19824, 56508, 26936, 49240, 43476, 9292,  46368, 57548, 15340,
                37320, 33364, 19108, 2356,  42248, 14884, 6324,  20428, 33460,
                8744,  14476, 38716, 59292, 6776,  41444, 17060, 25608, 31420,
                38892, 22356, 6528,  50248, 40780, 19144, 28664, 38644, 15228,
                58440, 10136, 19884, 13372, 5596,  21312, 62256, 12236, 31732,
                22856, 9952,  64448, 30628, 2992,  62620, 15844, 54492, 32080,
                20060, 29900, 14740, 9672,  49824, 39928, 31468, 62872, 53032,
                59620, 50104, 14404, 64620, 30952, 6428,  38588, 20788, 27640,
                52816, 5676,  19536, 14692, 37976, 21908, 2804,  25104, 8728,
                32096, 19836,
            },
            {
                45212, 57448, 34640, 47864, 30904, 1304,  58288, 39924, 23140,
                29576, 5688,  59028, 50916, 12360, 2768,  60816, 18872, 53772,
                44052, 21848, 34504, 62796, 13504, 58864, 1476,  10816, 26196,
                41364, 35464, 1108,  31428, 20364, 6592,  57260, 32828, 1448,
                23700, 52100, 34956, 17604, 10428, 23432, 48688, 30112, 64880,
                46076, 2840,  14628, 63012, 4792,  17872, 30824, 64852, 50940,
                41424, 28800, 48648, 62744, 34920, 27128, 47752, 37496, 52944,
                45732, 63548, 49084, 30556, 11440, 36656, 65124, 8948,  51556,
                3224,  59364, 29140, 34684, 12252, 24472, 44740, 52048, 1128,
                42060, 36120, 50664, 39180, 57404, 34848, 25368, 52252, 59156,
                5804,  49500, 39904, 24704, 53132, 37272, 10064, 45000, 60616,
                3444,  58492, 38280, 23188, 63956, 2428,  17384, 13960, 37080,
                4292,  19820, 9472,  36028, 140,   51776, 14984, 48920, 63264,
                42996, 23680, 56312, 1464,  28984, 53784, 46396, 36352, 14872,
                41932, 65396,
            },
            {
                51904, 23920, 13852, 5788,  22188, 11860, 61576, 45236, 9448,
                56076, 17040, 37432, 24972, 62868, 35288, 48124, 40116, 14072,
                25948, 57120, 48948, 7508,  44684, 31508, 16480, 45964, 57092,
                20964, 55096, 51204, 61716, 37796, 41796, 26120, 51408, 19532,
                39628, 4272,  43680, 61948, 32728, 39944, 4476,  12752, 42264,
                9368,  33020, 38336, 22600, 52712, 34252, 12508, 23504, 460,
                10892, 58976, 16652, 5180,  54188, 22736, 65328, 11668, 17304,
                24688, 5412,  21212, 1316,  53852, 22580, 28512, 46644, 56480,
                19040, 42364, 54104, 1804,  62240, 56640, 7812,  32692, 63620,
                26492, 30440, 4544,  64368, 27596, 8088,  47608, 16172, 37916,
                19332, 43468, 57960, 16916, 5136,  42096, 21512, 25676, 35448,
                49296, 7632,  52860, 18536, 33820, 6208,  42768, 48832, 22668,
                30044, 41372, 54804, 47192, 57888, 23352, 40940, 32120, 2828,
                13376, 35068, 50776, 44596, 59548, 10440, 18212, 62892, 56404,
                27428, 2012,
            },
            {
                30608, 9352,  60384, 44040, 55020, 50332, 25956, 33220, 20092,
                52572, 384,   44020, 14808, 7060,  30332, 55148, 9480,  5616,
                65272, 31012, 17036, 40640, 20652, 54252, 38880, 61404, 33116,
                8112,  28684, 43976, 12176, 5352,  63636, 14784, 8808,  46968,
                65076, 30808, 8264,  20668, 54568, 27868, 51092, 59664, 21976,
                53656, 61396, 28796, 7592,  41848, 59728, 48196, 27448, 44204,
                6816,  52292, 25148, 38972, 9796,  44620, 2612,  55728, 31752,
                42504, 56736, 34268, 44516, 60420, 15040, 5632,  33796, 12748,
                36928, 24900, 9844,  46540, 20892, 37492, 14100, 48580, 18308,
                9116,  47048, 16892, 53552, 43720, 11296, 41016, 1008,  29348,
                61696, 3324,  27592, 11880, 32932, 48428, 57396, 56,    17660,
                40588, 11780, 44712, 28448, 60856, 46596, 55880, 26688, 58120,
                1592,  62252, 25404, 16756, 28360, 11068, 60704, 7800,  45656,
                57648, 17544, 9404,  25320, 40292, 33740, 4712,  50336, 21400,
                7600,  39824,
            },
            {
                16564, 48632, 38288, 19596, 28912, 36204, 3712,  14264, 39024,
                47496, 27084, 32156, 57124, 20784, 44900, 17408, 27168, 33836,
                50724, 64,    11436, 59644, 28924, 6468,  2968,  23568, 12732,
                49280, 3344,  17336, 25228, 48288, 29656, 54760, 36792, 16656,
                27516, 57736, 14104, 45504, 1116,  63824, 7360,  18816, 36884,
                60,    16936, 47232, 55668, 19608, 2576,  37556, 16372, 56880,
                35676, 21792, 46196, 61556, 32292, 19404, 40764, 28168, 4284,
                61036, 9220,  26256, 17944, 38212, 49992, 57740, 43228, 304,
                49456, 61024, 32772, 17204, 52684, 30364, 4668,  60092, 22348,
                55240, 59252, 1956,  23144, 32420, 19512, 63016, 54504, 33632,
                13720, 51608, 36392, 55584, 61068, 8076,  29372, 65148, 53572,
                31164, 24332, 57220, 1140,  13608, 21168, 10468, 35112, 8192,
                45024, 12780, 33228, 4860,  44272, 34716, 55012, 19676, 26360,
                38168, 65164, 31640, 6700,  61196, 13672, 28336, 43676, 34964,
                12520, 54440,
            },
            {
                25860, 33688, 828,   64420, 7888,  16940, 62732, 56672, 7232,
                60764, 11560, 63544, 41028, 4280,  52264, 58988, 39228, 61336,
                21248, 42312, 36108, 24984, 47916, 64128, 35116, 51752, 19408,
                39664, 65192, 36056, 59916, 21888, 34124, 268,   59308, 42244,
                5304,  35668, 49812, 24736, 37836, 16160, 42900, 33512, 26676,
                49672, 40536, 11696, 32252, 25992, 64140, 8644,  53896, 31868,
                63324, 11940, 29520, 1416,  57304, 14016, 60028, 50404, 15648,
                39360, 51676, 13032, 64388, 2648,  30944, 24424, 18376, 63760,
                27912, 14776, 4368,  39980, 64748, 26040, 43620, 34904, 39776,
                12164, 42968, 37548, 14260, 60444, 49604, 4436,  24552, 44836,
                21372, 9332,  46944, 22760, 18088, 38956, 13104, 45940, 5648,
                15188, 61604, 34832, 41668, 50988, 31040, 39312, 65512, 18928,
                51448, 37964, 53844, 21364, 64312, 2228,  50296, 30316, 12044,
                53392, 964,   46660, 20624, 48628, 54148, 23968, 224,   58264,
                45600, 62040,
            },
            {
                21940, 58568, 42896, 11524, 52308, 40540, 45772, 30356, 22512,
                34992, 17472, 50092, 9124,  23300, 36008, 11856, 3140,  15384,
                46428, 55444, 4440,  53384, 14660, 9280,  43328, 27400, 59024,
                45608, 30200, 10504, 53216, 4176,  45480, 12892, 24136, 47980,
                21172, 55332, 11356, 60876, 29812, 47528, 9804,  58340, 55192,
                5776,  24004, 58700, 4064,  45144, 14172, 49944, 40600, 19020,
                5372,  42196, 15680, 37176, 47840, 7804,  35572, 23468, 46588,
                20268, 36216, 29532, 47684, 41948, 10488, 53404, 7676,  39084,
                21920, 55032, 44956, 8004,  49056, 11400, 656,   53448, 28456,
                5912,  33440, 25512, 46112, 7488,  28380, 36208, 56560, 7024,
                40252, 64744, 31400, 1408,  44108, 52132, 26500, 20880, 37876,
                48780, 9044,  19948, 5268,  54448, 25120, 2976,  49116, 15192,
                28584, 60460, 6912,  48080, 40440, 14480, 37220, 6172,  61540,
                22568, 42140, 27876, 56896, 16356, 38764, 63592, 31820, 17520,
                37324, 3696,
            },
            {
                6676,  14540, 55840, 31536, 20888, 24796, 49420, 1456,  43040,
                53548, 3332,  38112, 26336, 46772, 64568, 29284, 49608, 24260,
                8476,  28196, 62164, 19808, 39448, 31316, 17736, 55596, 632,
                15388, 7000,  41340, 57332, 19516, 38616, 51712, 61536, 9972,
                33268, 2468,  41136, 18320, 4128,  52556, 22656, 13492, 44932,
                30632, 15352, 62028, 51528, 34932, 22100, 29272, 1208,  25312,
                59572, 49372, 54932, 20516, 25480, 52692, 30648, 10984, 616,
                62984, 7236,  54848, 4676,  21728, 35212, 61796, 45832, 32316,
                11172, 58772, 34980, 29772, 22928, 57584, 15872, 61828, 20040,
                51052, 65244, 17968, 57020, 52520, 39724, 12988, 17704, 48696,
                58508, 14852, 25308, 6416,  62848, 34404, 2676,  59960, 32360,
                55336, 27764, 64032, 36976, 16288, 59956, 44256, 6508,  32540,
                42192, 23612, 10928, 17820, 27580, 56088, 24576, 43832, 48496,
                15392, 33664, 10948, 3408,  35524, 5548,  11512, 52004, 8492,
                29248, 49132,
            },
            {
                41520, 27856, 47068, 36852, 5916,  60168, 13576, 10044, 28044,
                58148, 20256, 31328, 55720, 14280, 944,   18944, 43492, 57228,
                37824, 32968, 11964, 45280, 50460, 5520,  62992, 11320, 37572,
                23076, 50752, 26416, 32184, 15848, 7568,  27268, 30788, 17672,
                63976, 51296, 26900, 59000, 32196, 62448, 36168, 1572,  65388,
                39332, 20380, 42956, 9640,  38100, 55872, 61428, 12284, 46676,
                34616, 9688,  3680,  64824, 43884, 4568,  61328, 56964, 43064,
                32900, 24476, 59280, 16880, 27380, 58220, 14284, 3636,  50228,
                42480, 17536, 1984,  51476, 19236, 37832, 45920, 24960, 8800,
                47808, 3348,  30740, 10744, 540,   22152, 63752, 31600, 2124,
                27888, 37072, 55104, 50108, 40752, 15708, 56852, 10824, 41468,
                4200,  23004, 47204, 43408, 11620, 29188, 56280, 21484, 53244,
                588,   58244, 36516, 62944, 32136, 8900,  59856, 18800, 3844,
                39968, 58072, 52412, 62320, 44376, 22204, 41700, 46972, 19952,
                64868, 53596,
            },
            {
                9988,  18280, 63200, 2256,  26592, 54808, 34132, 65132, 39800,
                15184, 45572, 6056,  59636, 41488, 34204, 54248, 31172, 5080,
                60080, 17068, 1552,  23332, 58096, 35836, 25772, 48168, 33160,
                54500, 61588, 3180,  46836, 63412, 43484, 57936, 40480, 6160,
                44888, 22988, 38080, 13856, 7008,  42048, 19300, 28560, 50952,
                7732,  34024, 2888,  27728, 16760, 5096,  43812, 31220, 52800,
                22524, 39340, 28368, 32700, 17244, 40316, 13560, 27432, 18160,
                53560, 39920, 12192, 44152, 49052, 38356, 19316, 29608, 25200,
                5972,  60796, 26972, 41080, 63668, 5380,  32152, 41876, 58836,
                13348, 38660, 44100, 61276, 35156, 42372, 47312, 9460,  53236,
                19932, 43308, 10736, 18452, 28652, 8360,  24248, 45376, 18200,
                50804, 13984, 1520,  32852, 19420, 8020,  37744, 63476, 12720,
                46732, 26612, 4056,  50540, 45060, 1372,  52136, 35248, 64220,
                30800, 7412,  25388, 18340, 29980, 55568, 26468, 59948, 14076,
                39468, 24252,
            },
            {
                61020, 34960, 50700, 43704, 15900, 38908, 19424, 5152,  24028,
                50528, 62928, 12108, 22464, 48800, 7780,  62548, 10396, 20852,
                40636, 52044, 64968, 41984, 29516, 3608,  21116, 59660, 8096,
                43024, 20016, 13548, 36460, 22848, 11344, 53760, 1796,  35860,
                15156, 54896, 9228,  49268, 45868, 24264, 55536, 11176, 47716,
                25212, 57376, 53752, 63800, 48996, 20876, 57848, 7708,  17532,
                62128, 13256, 58132, 51516, 7472,  22820, 49588, 37420, 9156,
                48676, 3404,  30420, 63708, 8612,  1112,  56304, 51964, 65356,
                36940, 46312, 12440, 56520, 9596,  14492, 54836, 2300,  35416,
                29184, 23192, 55392, 19896, 27132, 15168, 59324, 25048, 34352,
                62372, 4632,  59140, 33136, 47016, 52700, 36808, 64308, 29968,
                60688, 39192, 57916, 52284, 62200, 48184, 23960, 34704, 17416,
                39900, 54920, 20548, 15348, 41440, 22448, 13488, 28328, 10096,
                20856, 50620, 37864, 13352, 49644, 2344,  33272, 6484,  35728,
                1480,  31176,
            },
            {
                4208,  12904, 22828, 8324,  58812, 46336, 52728, 30684, 43992,
                35840, 1880,  33192, 27756, 17364, 39092, 24660, 51368, 47024,
                13312, 26324, 34348, 8748,  14352, 52656, 46260, 16352, 39504,
                1668,  29468, 49064, 56048, 5184,  34020, 19228, 25468, 48140,
                58748, 29416, 544,   34568, 57432, 15608, 5012,  38488, 60848,
                17552, 13320, 45208, 31840, 10944, 26280, 39840, 36060, 3020,
                42828, 26076, 40,    45800, 62672, 34772, 55972, 1820,  64588,
                22040, 15436, 36312, 52464, 23128, 33960, 41640, 10188, 22260,
                15712, 53988, 32880, 24352, 36272, 49916, 27384, 21080, 63368,
                51512, 16212, 4336,  49312, 7964,  52020, 3524,  38472, 46060,
                12680, 23456, 39444, 392,   61860, 21356, 3328,  12728, 6712,
                25544, 35316, 9724,  26792, 5540,  40548, 2152,  51188, 9744,
                30564, 5736,  34128, 59508, 29428, 56564, 38448, 47180, 54912,
                43300, 436,   63396, 46072, 9200,  61680, 17116, 57572, 51284,
                44672, 55156,
            },
            {
                47720, 40576, 56760, 32812, 28700, 11868, 36,    61400, 9028,
                18180, 54916, 42128, 52104, 60496, 4148,  14880, 28620, 35952,
                2876,  55852, 48572, 18844, 57096, 37076, 62556, 11944, 31504,
                24556, 64112, 9868,  39796, 59812, 29988, 50376, 64964, 11832,
                21292, 42508, 61688, 19752, 27864, 64384, 33432, 43528, 30472,
                2336,  37152, 22700, 41020, 1472,  52148, 14644, 64548, 48680,
                55388, 20288, 37204, 15948, 11032, 29744, 19212, 44080, 31996,
                57792, 46476, 26484, 5360,  59800, 13452, 47408, 32104, 2912,
                40392, 7576,  20328, 1036,  46976, 59828, 17644, 44560, 10460,
                6732,  58248, 40552, 33660, 64540, 30208, 18708, 57388, 6276,
                50452, 29308, 44800, 14276, 26460, 56256, 31696, 43488, 48720,
                54320, 16120, 21528, 45324, 31152, 15372, 60860, 27824, 58848,
                44096, 64736, 47912, 7716,  11492, 62284, 5564,  17376, 24880,
                59264, 15660, 31856, 21584, 40840, 36936, 23984, 11372, 27948,
                21284, 16392,
            },
            {
                59820, 26284, 19248, 3472,  63944, 37024, 21460, 48656, 14216,
                59440, 25656, 6852,  10724, 36544, 45840, 57476, 43416, 63928,
                7336,  22132, 31016, 43236, 672,   27184, 6140,  44300, 55040,
                50936, 18072, 45412, 26800, 15664, 3024,  44536, 7880,  38444,
                32584, 4848,  52352, 10276, 40816, 3164,  53568, 21688, 9088,
                52016, 59240, 5620,  18820, 61552, 46352, 33428, 23668, 28832,
                9152,  32204, 50496, 57016, 41744, 4216,  52960, 25208, 6524,
                40828, 11144, 61992, 20360, 43188, 28068, 17412, 62668, 57312,
                28896, 49316, 62212, 42356, 30052, 4700,  39276, 53708, 31636,
                37536, 46884, 24684, 11104, 45192, 13540, 41724, 22648, 32452,
                17080, 65392, 54764, 7872,  49440, 16864, 39008, 58828, 19656,
                788,   42024, 64812, 56792, 11828, 54672, 42936, 21092, 4516,
                13748, 24540, 18284, 37148, 25664, 49572, 33024, 2772,  40236,
                35568, 6728,  26776, 56240, 5312,  53368, 47940, 42256, 62980,
                7500,  37168,
            },
            {
                30280, 6120,  53716, 43004, 51392, 24716, 41784, 56584, 29152,
                39292, 21816, 47052, 63196, 30444, 20396, 372,   32332, 17956,
                59248, 39716, 11272, 63460, 50652, 23160, 33456, 19540, 58164,
                4116,  35268, 7116,  54088, 22160, 41348, 53116, 17484, 60768,
                26784, 46632, 16588, 36704, 49632, 25576, 12992, 46064, 62940,
                27664, 48700, 34224, 55988, 29540, 8712,  58600, 4724,  12460,
                45484, 60372, 5752,  23232, 27136, 48096, 61528, 13156, 17088,
                34420, 54664, 2436,  38424, 49140, 55540, 6932,  24616, 45120,
                12880, 34976, 58060, 16128, 9240,  64768, 23892, 13004, 84,
                62512, 19260, 28932, 2896,  56308, 35472, 62152, 1004,  53672,
                10124, 36968, 20588, 34440, 42208, 4920,  10084, 63120, 28104,
                33352, 8320,  49888, 2928,  36796, 18584, 33780, 49516, 38056,
                57180, 52420, 41052, 1084,  54072, 43508, 20236, 65316, 51940,
                11572, 48776, 60840, 34192, 12956, 29424, 19488, 696,   32448,
                50308, 10280,
            },
            {
                65400, 46092, 34008, 15224, 9948,  17468, 6456,  34372, 2776,
                51280, 32436, 3724,  56368, 12924, 26560, 53604, 9452,  23872,
                52304, 46420, 4588,  16684, 38236, 60256, 9936,  41032, 27772,
                14908, 38468, 60360, 32676, 62976, 13056, 35104, 23944, 1664,
                57152, 13436, 62840, 22816, 31288, 58324, 7308,  18712, 38944,
                14996, 23996, 11256, 44104, 16296, 38808, 20968, 53952, 35624,
                18408, 39288, 14156, 65440, 35772, 9628,  38900, 58908, 46008,
                28376, 50224, 18692, 31088, 9752,  34516, 288,   51768, 37684,
                18600, 3796,  25924, 51668, 21776, 34172, 55756, 41920, 26572,
                50784, 14340, 60284, 53316, 21576, 7432,  25828, 48164, 43396,
                59484, 25084, 1968,  58044, 30016, 51624, 23484, 36264, 13964,
                47268, 22692, 38456, 29236, 24760, 62468, 8928,  4,     26276,
                32592, 10192, 22308, 61720, 30152, 13144, 8564,  27972, 58528,
                22724, 44520, 17856, 42500, 2980,  64652, 58328, 15180, 39072,
                56288, 23600,
            },
            {
                13060, 39472, 1444,  60828, 47800, 27832, 62168, 45112, 12260,
                64948, 15808, 44064, 18764, 39868, 49524, 60956, 41992, 37244,
                13636, 33844, 25264, 29072, 54636, 13932, 46148, 2844,  65380,
                48436, 11636, 20372, 200,   47844, 9356,  56272, 30464, 50148,
                43284, 39336, 6232,  55404, 2076,  42052, 61432, 34664, 56324,
                412,   41460, 65064, 6716,  25908, 49384, 864,   42392, 63180,
                51428, 2380,  30148, 44580, 19768, 3312,  31464, 22428, 712,
                64020, 7808,  24184, 59372, 15588, 64724, 41144, 21384, 60592,
                8580,  55256, 44232, 11768, 37996, 48444, 7056,  17260, 45360,
                33396, 8416,  36840, 41272, 16728, 50580, 38540, 28644, 14792,
                6668,  40328, 47304, 12528, 62144, 18696, 45676, 53820, 4340,
                57172, 61388, 17004, 53456, 4928,  43968, 52056, 14864, 46508,
                63176, 6660,  45140, 16056, 34860, 57564, 39408, 46452, 15268,
                4896,  29784, 9524,  50464, 24720, 36348, 45948, 26608, 4976,
                43728, 18204,
            },
            {
                51548, 25780, 21428, 55468, 31608, 37952, 53192, 20324, 26228,
                8508,  36416, 53916, 24600, 8036,  34816, 5076,  16192, 48384,
                2608,  56628, 62148, 8240,  48916, 21132, 35540, 52964, 24212,
                30736, 43796, 52108, 28656, 25348, 44936, 37520, 4032,  20404,
                10024, 28348, 34240, 19028, 47924, 15484, 28980, 50980, 5188,
                31904, 20480, 51576, 36572, 54976, 61924, 31952, 14896, 27568,
                24372, 59052, 8328,  54232, 49148, 16312, 56332, 51248, 36412,
                43096, 13848, 40344, 52756, 45644, 26604, 11376, 47884, 32712,
                27800, 40248, 63896, 30948, 1404,  58556, 28628, 61268, 3200,
                56784, 22992, 48772, 31096, 1912,  58704, 11500, 19584, 64068,
                31592, 52380, 16628, 27480, 8716,  38260, 2716,  26112, 42976,
                10796, 31356, 40796, 12884, 59152, 20600, 30096, 40220, 55912,
                17756, 28588, 50960, 59192, 4004,  48804, 23896, 2144,  32356,
                54044, 37332, 61656, 55552, 20572, 7244,  52568, 10756, 62232,
                34368, 2660,
            },
            {
                29544, 59428, 8724,  41464, 5252,  13536, 824,   59276, 40332,
                47688, 58372, 29296, 1248,  63644, 57044, 21720, 27804, 65136,
                30552, 20476, 44432, 40376, 1532,  31660, 6740,  59792, 17608,
                8480,  55820, 39872, 5340,  57908, 18780, 63864, 15000, 60072,
                54140, 24296, 65296, 44388, 8664,  37736, 22324, 10512, 43400,
                48052, 59924, 28692, 2556,  12576, 23000, 46160, 9696,  5212,
                47736, 40696, 21748, 33536, 62476, 41552, 12148, 5496,  26380,
                20168, 58180, 33556, 3552,  6356,  36816, 17120, 57496, 2084,
                14296, 22964, 6260,  46744, 19188, 14048, 51924, 21396, 39544,
                12156, 65200, 6156,  26852, 61916, 44196, 33528, 55580, 4540,
                36180, 22088, 60712, 43584, 33608, 55888, 63504, 15596, 34804,
                21224, 51180, 6052,  27080, 47588, 34524, 65252, 11364, 22944,
                35984, 1904,  39672, 11076, 26820, 18068, 55424, 35696, 63364,
                19784, 41044, 1360,  14068, 33080, 40100, 16712, 30984, 22064,
                47488, 54900,
            },
            {
                15456, 44676, 35804, 18636, 64488, 23108, 50328, 30596, 17156,
                5572,  22032, 42340, 14300, 47380, 31528, 11284, 45264, 54224,
                6196,  12276, 36156, 18012, 61048, 56012, 27444, 41856, 37404,
                1676,  22676, 61064, 16232, 34532, 7484,  49576, 40708, 32168,
                47096, 576,   12388, 52388, 58752, 26552, 54428, 64204, 13596,
                25324, 8284,  16020, 45012, 33992, 18556, 58444, 38432, 55764,
                34736, 16060, 10996, 968,   28256, 24588, 60212, 45428, 55164,
                9680,  48464, 28916, 61864, 22404, 55876, 30076, 63296, 43148,
                53736, 50232, 33628, 61436, 25456, 43668, 35620, 8648,  30652,
                53904, 42656, 19036, 47124, 14084, 8792,  24256, 51460, 45652,
                10416, 56680, 2464,  49740, 5928,  20680, 30928, 48032, 58592,
                7764,  64396, 45216, 1224,  56500, 15952, 7444,  3108,  49304,
                61240, 43624, 20228, 31480, 64572, 6092,  42040, 11676, 50904,
                8132,  25428, 45688, 28460, 63492, 48156, 57048, 42600, 60096,
                6800,  37656,
            },
            {
                63572, 3872,  48412, 27364, 57276, 43264, 34952, 10880, 55100,
                61340, 33132, 10048, 52476, 35808, 18112, 40992, 1872,  38028,
                59460, 26548, 50244, 9484,  23640, 45416, 15344, 11536, 49228,
                63632, 33016, 12616, 50588, 42664, 29664, 11236, 21992, 6040,
                36660, 17740, 30328, 40008, 4788,  32944, 1660,  19392, 35432,
                57652, 38792, 52596, 63552, 40528, 3964,  50864, 29408, 20192,
                63612, 52372, 43964, 57660, 37608, 7184,  18668, 34276, 30488,
                2808,  39048, 17760, 12448, 51196, 41448, 8752,  19972, 4692,
                35996, 16672, 10528, 54948, 38736, 4880,  64064, 47440, 24992,
                15744, 34908, 4140,  56264, 36564, 39788, 17528, 328,   41628,
                26436, 30140, 14372, 39056, 24820, 51972, 12000, 820,   40360,
                28960, 37904, 18436, 32512, 23576, 42536, 37596, 54124, 25972,
                33212, 13228, 56572, 53160, 37028, 46140, 21872, 28856, 58076,
                16456, 49200, 59968, 10252, 23320, 5668,  2396,  12912, 28208,
                19180, 10100,
            },
            {
                24640, 31968, 11884, 52532, 15928, 8012,  46692, 25028, 37752,
                2116,  49520, 26928, 62436, 4172,  25624, 60800, 49712, 23116,
                17084, 42912, 53508, 63260, 34680, 3452,  52064, 58648, 25492,
                46524, 19472, 36800, 3860,  23676, 59092, 53412, 27172, 44552,
                55868, 61888, 50416, 21156, 14688, 60632, 49732, 40740, 45616,
                3428,  30932, 20068, 10708, 26616, 60892, 7760,  43380, 12368,
                2792,  25872, 31276, 48272, 14188, 53040, 40136, 62764, 15532,
                50684, 65140, 24980, 44764, 1216,  32460, 49264, 25600, 46316,
                29076, 59544, 21804, 2244,  29796, 17776, 11728, 57524, 1156,
                51084, 59788, 29280, 21828, 52716, 63884, 30768, 59580, 20504,
                62388, 48420, 18316, 65068, 9188,  42036, 60388, 17492, 54972,
                24632, 13836, 53668, 8916,  60936, 51464, 28184, 59908, 17140,
                8392,  47140, 4244,  24392, 9048,  60052, 14108, 272,   43436,
                31216, 4448,  34968, 39104, 18184, 50856, 37044, 25088, 53528,
                39936, 50272,
            },
            {
                55956, 21100, 58928, 1884,  38960, 29572, 3704,  63236, 19552,
                44704, 15464, 39492, 20188, 43944, 55524, 8860,  14656, 33980,
                7560,  31304, 240,   14304, 29268, 39456, 21504, 32240, 4564,
                9624,  28280, 54832, 62824, 45960, 1260,  38624, 64380, 13468,
                3656,  25420, 9784,  35372, 43716, 23924, 29716, 7980,  17044,
                62608, 23492, 5692,  55632, 47908, 31620, 16664, 24220, 37356,
                60020, 6620,  18392, 64832, 22576, 4260,  47056, 10804, 23280,
                43600, 6772,  35972, 54120, 60272, 16000, 38216, 13088, 57972,
                39580, 7836,  42856, 49036, 60400, 52840, 40932, 32628, 19856,
                38188, 7456,  45224, 11444, 2732,  26192, 6948,  47448, 12664,
                37940, 4968,  34668, 54772, 45096, 27212, 36628, 22148, 45668,
                3600,  49196, 62528, 35296, 20252, 11540, 4364,  45460, 21572,
                41332, 63692, 29676, 15692, 40448, 33600, 52340, 61936, 37456,
                22772, 55664, 13200, 53204, 43132, 59120, 32244, 65032, 44940,
                12,    33832,
            },
            {
                43340, 7264,  40872, 34212, 61868, 21732, 51840, 56772, 12064,
                28432, 54008, 6900,  58020, 13036, 37328, 30056, 51988, 64416,
                45716, 58480, 20900, 41376, 48536, 7296,  64840, 17436, 56540,
                38984, 43392, 7228,  15132, 31852, 19928, 8856,  17312, 32844,
                48728, 41396, 57708, 6968,  63416, 47160, 11312, 57144, 53252,
                33868, 50468, 41692, 14820, 35828, 192,   53516, 57704, 46760,
                33348, 50692, 40620, 9592,  35732, 29940, 58856, 27232, 92,
                55996, 31060, 20512, 9872,  27920, 5536,  62396, 53264, 2640,
                24036, 65484, 13804, 34176, 26372, 7044,  23256, 44920, 62552,
                27324, 14184, 61484, 33424, 50048, 41128, 15320, 53664, 33040,
                8584,  57656, 22440, 1936,  31764, 15116, 4684,  63656, 33712,
                10908, 42716, 26456, 1836,  47776, 40632, 14584, 31396, 36252,
                608,   58100, 51360, 19548, 49068, 3096,  25632, 18528, 48012,
                7348,  64084, 20376, 1088,  29168, 8332,  14892, 20920, 5892,
                61116, 16832,
            },
            {
                13508, 62812, 26044, 18196, 48016, 13388, 32652, 41548, 4988,
                35564, 65276, 31612, 24048, 668,   47796, 21184, 4852,  27352,
                38852, 11752, 55696, 25220, 60044, 10660, 44364, 35832, 50384,
                13432, 60924, 52172, 26524, 41088, 56608, 50944, 35880, 60324,
                21808, 29604, 16472, 52900, 2308,  18632, 36828, 26804, 908,
                12716, 28288, 59004, 44856, 21612, 65104, 39420, 9364,  20824,
                14576, 28100, 56728, 1440,  53912, 43044, 20000, 51236, 38404,
                61184, 13188, 42224, 58388, 47208, 23016, 35076, 18812, 10772,
                32720, 51956, 19892, 46544, 37380, 16072, 56344, 3780,  9792,
                49128, 41888, 24124, 57052, 18188, 36084, 58260, 23708, 43864,
                28528, 50800, 40896, 11844, 59488, 52932, 48612, 8160,  56372,
                30412, 59064, 16552, 38580, 29792, 58256, 64280, 50240, 55144,
                10260, 26636, 34248, 6748,  44352, 56732, 31496, 9916,  40040,
                26840, 44824, 32964, 60656, 25704, 46340, 56108, 35900, 11264,
                47976, 28888,
            },
            {
                51148, 4376,  45596, 54108, 9224,  58572, 26788, 16868, 50124,
                22068, 46616, 9556,  51116, 34496, 63072, 41280, 57352, 18024,
                2388,  47472, 34232, 4112,  19024, 54412, 27104, 1028,  23564,
                30228, 3592,  21912, 35420, 11304, 47348, 24400, 5644,  43028,
                840,   11948, 38128, 27208, 33144, 55848, 40428, 60216, 20940,
                48152, 38068, 8692,  4204,  25424, 11936, 28828, 50200, 2424,
                63096, 45036, 23356, 17284, 61772, 13000, 33672, 8348,  16436,
                48336, 34468, 3456,  17656, 38876, 51268, 30336, 44576, 56544,
                41736, 27812, 6304,  228,   63080, 12200, 50400, 29208, 35428,
                17108, 54452, 644,   29624, 8880,  46456, 4384,  64764, 1228,
                17372, 61508, 25244, 47000, 19816, 29080, 39144, 24172, 19152,
                50756, 5824,  21764, 55492, 8268,  24824, 6292,  18984, 23608,
                46212, 13712, 38532, 62596, 22508, 12496, 64800, 53968, 4048,
                15216, 51172, 10924, 40860, 16912, 49768, 2700,  30524, 58484,
                23832, 38020,
            },
            {
                55168, 31084, 22320, 36876, 676,   43500, 6624,  38552, 60660,
                1564,  42468, 18364, 60084, 27952, 15760, 10964, 44472, 23552,
                60940, 29804, 15860, 51560, 37788, 31808, 62292, 15976, 42204,
                58396, 47784, 17836, 65240, 2360,  29312, 61784, 14400, 53972,
                58408, 46152, 64928, 49468, 22752, 4044,  14508, 44336, 5884,
                63932, 16388, 31848, 61504, 55928, 46080, 17192, 32392, 42420,
                5552,  34880, 11388, 38572, 46504, 25564, 3204,  64476, 29120,
                22168, 53684, 25540, 63936, 8164,  14160, 816,   63328, 5060,
                48504, 16736, 59060, 53788, 43564, 32028, 39744, 58716, 22144,
                65028, 5516,  39128, 20616, 62860, 12472, 31488, 21036, 39100,
                55420, 14140, 5584,  36948, 9424,  62904, 220,   43648, 13328,
                36508, 64780, 41540, 12808, 52320, 34756, 43940, 2400,  39568,
                61664, 17892, 53052, 1504,  28632, 42352, 17024, 35640, 60296,
                29376, 57628, 36576, 4892,  62728, 22312, 39296, 52844, 42756,
                18512, 8896,
            },
            {
                34320, 64752, 11188, 16140, 28640, 50880, 64148, 24104, 11416,
                30516, 54544, 14092, 39972, 3160,  55064, 32684, 6488,  53268,
                36480, 9016,  64088, 43496, 6300,  13160, 46820, 52976, 9396,
                33908, 6880,  38000, 44412, 57644, 7896,  18784, 38936, 26008,
                31124, 8272,  19728, 42284, 10612, 62204, 30972, 51308, 34972,
                23104, 42576, 52664, 19460, 40436, 35120, 6580,  58544, 53084,
                26204, 60316, 55748, 30428, 5260,  52060, 56340, 40592, 45368,
                5728,  11132, 43740, 32344, 55488, 41092, 20700, 26188, 36116,
                12868, 38520, 30224, 20896, 24708, 7572,  18720, 2884,  46492,
                12712, 44048, 33904, 48828, 53504, 42396, 25672, 51688, 10920,
                34928, 49016, 32164, 42788, 56716, 16148, 34068, 60236, 47300,
                27344, 3808,  32876, 46712, 60916, 15732, 27700, 57132, 32624,
                8048,  29936, 58516, 47680, 36524, 8184,  49856, 24008, 45848,
                1576,  18648, 23628, 53592, 9284,  34028, 13040, 6744,  26392,
                60524, 1488,
            },
            {
                20056, 47368, 42156, 59432, 55532, 33616, 19340, 45916, 35200,
                57924, 26424, 5172,  36584, 45512, 19888, 49412, 26308, 39648,
                13920, 48372, 26640, 19992, 58320, 23324, 2628,  39300, 20604,
                63044, 25952, 54644, 14968, 32516, 42092, 49620, 34164, 2736,
                51536, 15668, 34564, 5484,  54540, 45864, 17772, 8116,  26408,
                55296, 1764,  11156, 27620, 2816,  48672, 63688, 22300, 12920,
                19328, 41068, 8420,  49160, 20912, 36316, 14848, 19164, 59564,
                36748, 49356, 58036, 2260,  28428, 47584, 59776, 52864, 8560,
                23476, 61620, 49952, 10496, 35528, 54852, 60648, 26432, 52548,
                30820, 57892, 24996, 7220,  15724, 36668, 60320, 5944,  46012,
                27196, 63380, 2292,  21568, 51200, 26128, 6180,  54940, 17556,
                9244,  53624, 24436, 1100,  20160, 37700, 48892, 11872, 51080,
                42160, 4732,  24512, 11472, 20596, 59340, 5348,  33032, 13296,
                41116, 48788, 31676, 43364, 58912, 28344, 45432, 64188, 15864,
                48856, 39960,
            },
            {
                14052, 27284, 3052,  23656, 39668, 10156, 2432,  15112, 51412,
                7468,  20332, 48424, 62696, 24180, 8536,  58960, 63788, 1044,
                21552, 55244, 3076,  40652, 30252, 50212, 35384, 56084, 28756,
                12468, 49336, 136,   23876, 52280, 21288, 12280, 63816, 44556,
                22436, 60792, 57116, 37500, 29260, 24128, 59016, 39036, 12488,
                46704, 58696, 36580, 50672, 14624, 24524, 9620,  38516, 30964,
                47132, 556,   16376, 27680, 62460, 43388, 10072, 31924, 1632,
                26696, 13924, 18028, 23252, 37112, 10728, 16316, 33320, 45844,
                57344, 1608,  41176, 4660,  64492, 47792, 13728, 41596, 8140,
                37588, 19360, 10844, 61192, 28092, 2500,  18852, 57576, 40836,
                8216,  18344, 58304, 29720, 45756, 12152, 41428, 22680, 31108,
                39476, 59656, 44468, 29512, 10108, 62604, 4268,  21852, 65216,
                15240, 54784, 33736, 44644, 51928, 39644, 26876, 53180, 61892,
                21124, 7640,  65444, 14572, 432,   18296, 55412, 36184, 3992,
                29892, 57460,
            },
            {
                7636,  52956, 62132, 6396,  30852, 48100, 61496, 27792, 39096,
                63596, 43616, 32076, 56144, 12600, 30212, 35672, 16592, 42696,
                31684, 59436, 34760, 11456, 61984, 8040,  44668, 17388, 4584,
                59492, 41416, 31076, 10300, 62480, 3568,  27988, 56292, 6792,
                11044, 40880, 25680, 3420,  13604, 52336, 804,   33056, 65364,
                20144, 29856, 7792,  60784, 33332, 42988, 57076, 4068,  51868,
                35724, 64176, 54408, 33388, 2860,  58400, 24672, 50500, 63420,
                54244, 40300, 61424, 6404,  51780, 64804, 3936,  39512, 28920,
                19388, 14948, 32788, 45064, 17172, 28544, 2036,  33340, 16420,
                63832, 1516,  50356, 44952, 55608, 32988, 48128, 13800, 30644,
                24072, 53012, 15056, 3976,  37956, 65448, 52300, 1996,  62112,
                48824, 12056, 16748, 57636, 51760, 40564, 31260, 45476, 26032,
                35920, 19112, 61264, 3260,  14640, 63204, 18220, 2296,  56192,
                10584, 37820, 51300, 25076, 40020, 47764, 21508, 9756,  51808,
                22952, 44068,
            },
            {
                33496, 37716, 18016, 50204, 14396, 21500, 42416, 56320, 3840,
                23276, 10508, 16244, 344,   41604, 52156, 5468,  46208, 10652,
                51364, 6784,  17724, 46636, 53984, 15232, 25756, 64692, 32792,
                46124, 19220, 37128, 57928, 46664, 39984, 16988, 36652, 47648,
                30452, 18568, 49752, 44284, 62876, 20972, 41560, 48964, 4456,
                16136, 40924, 44984, 21840, 54496, 18148, 29244, 61144, 15380,
                24888, 10356, 22036, 41952, 13380, 46400, 6024,  35304, 21256,
                8376,  29780, 34544, 45980, 20008, 41924, 24528, 55304, 6200,
                62768, 51332, 26632, 56124, 21564, 37936, 50732, 43664, 23176,
                54060, 28976, 40340, 21492, 4872,  38264, 22980, 63916, 440,
                35776, 44076, 60408, 33240, 8976,  19208, 28764, 35096, 7696,
                20828, 37228, 5420,  34112, 23856, 7436,  17996, 59204, 0,
                9388,  48468, 27492, 38116, 31548, 9568,  42720, 29996, 35512,
                44564, 27780, 3484,  57160, 30416, 6220,  61896, 32288, 38812,
                63388, 11760,
            },
            {
                28516, 54956, 25224, 44724, 35556, 59144, 8428,  32732, 17692,
                53744, 37180, 50040, 26904, 61812, 18688, 22956, 58008, 28772,
                24932, 65528, 38872, 22332, 27976, 724,   42436, 10552, 22632,
                50912, 7816,  15904, 27148, 5852,  33304, 59904, 23196, 54476,
                560,   64680, 32596, 8468,  16492, 35712, 10544, 26984, 53292,
                57360, 25268, 3040,  63448, 11804, 940,   46608, 40508, 6336,
                44004, 49628, 59180, 38016, 18644, 53480, 28500, 16528, 47228,
                43076, 11896, 25196, 56692, 2672,  30828, 14204, 48544, 11476,
                43456, 36664, 8432,  60000, 11840, 6388,  62912, 57288, 10264,
                47536, 34712, 14816, 62400, 11772, 52500, 8588,  41660, 50720,
                56380, 11148, 39556, 49260, 25064, 46556, 57668, 15400, 43952,
                56236, 27156, 64600, 42412, 48368, 55740, 13324, 37396, 52804,
                41516, 57964, 6496,  22280, 55288, 49332, 24432, 47136, 13584,
                22776, 59688, 17616, 34696, 11976, 52612, 43640, 25832, 16956,
                1024,  46780,
            },
            {
                60664, 3504,  10672, 65532, 476,   52616, 25844, 12968, 45788,
                29364, 60708, 6272,  33952, 46764, 39280, 2712,  54208, 36932,
                14812, 44400, 4688,  32220, 59896, 36336, 52416, 57552, 38664,
                2060,  61364, 53728, 43204, 20684, 50300, 9064,  14536, 42948,
                26704, 13052, 38152, 55616, 59692, 47772, 30868, 61976, 6920,
                35172, 13252, 38584, 31232, 49232, 36980, 26368, 20760, 55908,
                32564, 2004,  26712, 7180,  30832, 65452, 39696, 57508, 740,
                60440, 52272, 6904,  16872, 50052, 61876, 35100, 59596, 21020,
                31424, 3644,  23228, 40268, 53220, 32016, 25516, 18560, 3376,
                60404, 6864,  26064, 43244, 30996, 58104, 27292, 15800, 20316,
                4716,  28060, 21948, 6124,  61648, 13316, 3576,  39080, 23536,
                50824, 592,   14368, 19604, 3792,  27736, 33008, 63336, 22876,
                29760, 13752, 45684, 64544, 1080,  16844, 57816, 4312,  64168,
                51732, 6604,  48220, 42440, 20456, 60228, 2516,  13724, 49880,
                58588, 20432,
            },
            {
                42740, 15592, 40004, 32060, 19764, 5532,  37924, 49548, 64336,
                2120,  41048, 20488, 58916, 8920,  13656, 63628, 32576, 20220,
                8788,  50380, 56360, 12816, 47540, 18248, 6936,  28596, 14380,
                34652, 24232, 30352, 12824, 63188, 1436,  53464, 31236, 61880,
                4944,  51144, 21700, 2264,  24488, 5612,  43000, 17876, 22884,
                45880, 51504, 59104, 5860,  16404, 53660, 8084,  64936, 12564,
                17512, 62344, 45912, 12212, 50340, 3632,  10240, 23900, 36972,
                14804, 31656, 64284, 38560, 43988, 8296,  26816, 836,   39076,
                54748, 65152, 17800, 48896, 376,   46388, 14196, 36596, 30248,
                39604, 20240, 56776, 876,   18152, 46896, 35340, 61232, 44492,
                32128, 64376, 16932, 54908, 41676, 31204, 53352, 63408, 9924,
                33180, 30032, 53056, 38372, 58896, 46548, 10700, 44028, 3124,
                17380, 34492, 52240, 11108, 40580, 33096, 36996, 8696,  20084,
                32248, 39304, 15076, 63580, 8576,  37416, 27892, 56596, 40608,
                35168, 6100,
            },
            {
                52000, 22552, 57024, 47668, 29156, 55544, 41484, 22072, 34540,
                11128, 25136, 15136, 51804, 30844, 25736, 43924, 48860, 3888,
                61312, 41388, 26868, 2280,  62504, 40288, 21448, 48620, 63840,
                45196, 55796, 4840,  48252, 35400, 25696, 38864, 45724, 19480,
                35768, 58376, 40644, 45936, 29508, 53888, 13496, 38976, 56220,
                9464,  27636, 19696, 41820, 24068, 59884, 33596, 51036, 39428,
                28676, 36644, 57216, 20520, 34344, 42672, 54768, 19308, 49284,
                6332,  41224, 20996, 28308, 12536, 22904, 52396, 46344, 16224,
                28140, 44720, 12796, 34416, 28812, 62036, 42888, 55316, 49684,
                12464, 45148, 51696, 37012, 64980, 6348,  24420, 2708,  12772,
                47672, 37552, 51212, 1320,  35300, 20536, 26184, 47696, 17964,
                60100, 45556, 8276,  62104, 23364, 16208, 54640, 6672,  49552,
                61732, 38708, 24968, 19588, 28680, 50656, 61608, 26720, 41704,
                55076, 768,   28992, 53688, 23296, 33100, 46332, 19016, 9104,
                24380, 31300,
            },
            {
                63164, 7868,  36632, 13340, 62464, 9852,  16188, 60024, 6756,
                57580, 48180, 55660, 37484, 1376,  17764, 56244, 12024, 38524,
                29808, 16304, 22024, 35188, 9736,  55028, 31136, 3620,  11380,
                19492, 40384, 9232,  58996, 41904, 17460, 11192, 57392, 6892,
                28164, 16064, 8904,  33772, 19068, 64268, 3208,  49380, 32416,
                152,   64664, 34156, 48284, 10696, 29524, 4344,  43276, 23420,
                3296,  9344,  52452, 24724, 59772, 14248, 27904, 63160, 33544,
                25776, 58232, 47548, 1828,  54388, 37384, 18504, 58660, 10204,
                6452,  50908, 25272, 58344, 9160,  20080, 4816,  24052, 8404,
                63684, 32960, 23284, 9780,  14384, 50288, 39880, 54264, 57676,
                7048,  23716, 9896,  58552, 45444, 11448, 6568,  36700, 2656,
                41216, 21892, 12632, 35460, 1640,  30924, 39792, 20720, 27236,
                56796, 7976,  2528,  60152, 43360, 5888,  12476, 45340, 16332,
                60576, 10684, 25288, 45084, 3304,  50176, 5568,  65000, 54144,
                12800, 48340,
            },
            {
                27816, 17640, 44276, 25648, 4396,  50608, 45548, 28224, 19056,
                31576, 44332, 4156,  22352, 42404, 60652, 34644, 7316,  23744,
                52920, 58564, 46144, 51420, 15024, 24828, 43980, 58652, 51816,
                27016, 32948, 15764, 22272, 29252, 65372, 2892,  23780, 49072,
                42484, 63744, 54436, 11744, 50408, 25800, 36824, 60620, 21352,
                44356, 14548, 4636,  55140, 62736, 45080, 14936, 19352, 55352,
                48960, 41188, 15988, 44,    44812, 38600, 5408,  46044, 2304,
                52628, 17808, 10524, 60040, 33148, 63304, 5396,  41580, 32352,
                61280, 36160, 2448,  41964, 54036, 38620, 16112, 57764, 27648,
                2364,  16840, 54360, 29644, 42624, 32556, 21128, 28252, 34656,
                19296, 42376, 30216, 26980, 15940, 61348, 55932, 50284, 28572,
                15748, 58004, 48652, 26644, 43596, 51472, 64000, 36012, 12332,
                42212, 32508, 46700, 55592, 15440, 21988, 54000, 2680,  30744,
                49344, 34520, 38220, 57088, 12412, 59588, 16012, 30172, 43792,
                38360, 4316,
            },
            {
                34828, 54536, 1460,  59532, 33244, 22716, 37016, 1220,  52560,
                39708, 12784, 26996, 65092, 9976,  49756, 27564, 46964, 64096,
                188,   32176, 6440,  37948, 64404, 5176,  34236, 16648, 37364,
                692,   62628, 49900, 3892,  54112, 46488, 33208, 52544, 13904,
                31816, 2028,  22452, 59444, 5984,  41252, 16284, 10324, 28792,
                58292, 40460, 18256, 37560, 22372, 1240,  36124, 57932, 11180,
                60744, 33716, 29696, 64104, 8256,  31804, 22360, 60352, 12452,
                43144, 29680, 36404, 4680,  44420, 14588, 29548, 49288, 23748,
                55628, 19156, 14912, 64004, 22612, 32552, 51108, 44260, 34852,
                40744, 58800, 4556,  46520, 61492, 1948,  59232, 10484, 16096,
                48708, 60036, 53112, 3784,  40284, 32412, 23120, 43180, 64192,
                7368,  38840, 4752,  56056, 18476, 9996,  5744,  58144, 15700,
                22664, 52736, 10368, 30624, 37468, 63556, 34876, 24308, 57976,
                20840, 4484,  62424, 17788, 21844, 41112, 36468, 26164, 8,
                21136, 60160,
            },
            {
                9956,  46672, 40504, 19628, 52888, 42596, 14020, 63908, 9264,
                61072, 35032, 17228, 53212, 32752, 5052,  20468, 41056, 13424,
                18640, 44200, 10972, 20304, 27520, 47360, 60552, 8844,  42760,
                23932, 57004, 44116, 38304, 13292, 7680,  37140, 20132, 60944,
                39424, 26100, 44308, 35060, 47392, 31092, 55800, 6956,  46528,
                24228, 53520, 31408, 8704,  27140, 52216, 46844, 30588, 25364,
                6872,  21200, 47252, 54160, 18444, 56840, 50280, 16248, 34836,
                7860,  56456, 23336, 51060, 19684, 25960, 57240, 1528,  11852,
                39284, 47196, 30712, 7340,  48388, 11292, 1276,  62964, 13068,
                47960, 21252, 38120, 26356, 18988, 7948,  52140, 45404, 63140,
                5368,  38420, 13792, 64728, 46864, 19120, 844,   13276, 33852,
                25020, 52400, 29948, 62748, 34356, 24164, 47724, 28484, 45440,
                3996,  65352, 18896, 26504, 340,   47320, 9392,  51616, 39108,
                13448, 47560, 8136,  43520, 32340, 52084, 10248, 61748, 55940,
                49460, 14532,
            },
            {
                22924, 64776, 29908, 11816, 7356,  57796, 31148, 24744, 48904,
                21180, 6372,  46792, 57844, 39520, 15788, 59624, 54676, 36752,
                28808, 62180, 55612, 41756, 53400, 1544,  29776, 21856, 54180,
                13784, 7196,  28004, 18864, 61456, 25816, 58520, 44868, 9468,
                4776,  51452, 17548, 13332, 912,   62772, 19736, 51784, 35240,
                2136,  11992, 62004, 42564, 59976, 17008, 6532,  64496, 38828,
                14556, 43420, 4356,  25968, 36780, 10616, 40716, 25024, 53892,
                61708, 39660, 13940, 65120, 40964, 8740,  45776, 35484, 21276,
                62692, 4008,  43784, 26288, 37252, 59980, 29240, 18744, 25116,
                52180, 6752,  63968, 11684, 56652, 40396, 24020, 37208, 30672,
                116,   25728, 22028, 9108,  35752, 50996, 58764, 9560,  54564,
                20180, 44288, 10980, 3212,  14420, 41156, 53940, 1188,  60692,
                34464, 39560, 49368, 13704, 56704, 40928, 28416, 17332, 6556,
                64888, 25868, 29380, 55636, 2176,  24356, 6708,  45872, 18360,
                39624, 32520,
            },
            {
                51328, 5056,  26348, 38124, 62140, 17476, 46280, 2900,  54984,
                42076, 29536, 732,   23684, 12100, 30536, 2064,  25156, 9036,
                49192, 3288,  23472, 33288, 13048, 17784, 50096, 40132, 64960,
                31876, 47968, 35140, 10776, 50532, 30888, 164,   16156, 56224,
                29472, 62368, 37528, 57528, 28372, 23644, 43268, 13768, 63664,
                39196, 49804, 5672,  47728, 13012, 32904, 40968, 2188,  49828,
                61908, 35648, 51680, 13368, 61248, 1380,  47876, 30064, 3252,
                20644, 27484, 47744, 324,   30940, 53272, 60520, 15792, 50512,
                28736, 9732,  53100, 58168, 20496, 5196,  41776, 54216, 9380,
                30876, 35956, 15160, 29000, 49088, 34016, 16804, 12636, 55008,
                43860, 50180, 32148, 55440, 5900,  26292, 29824, 37780, 47556,
                60836, 16272, 36684, 49820, 59020, 20400, 31960, 38024, 17296,
                25252, 6760,  43852, 59952, 4744,  20556, 61632, 44624, 32132,
                42056, 53556, 37156, 15044, 48560, 64260, 35148, 12684, 29396,
                3432,  58460,
            },
            {
                43984, 16320, 55704, 48136, 3772,  35548, 27544, 10748, 37960,
                14564, 59732, 50316, 36072, 44732, 63568, 51568, 42648, 33604,
                58152, 15544, 39680, 61088, 7128,  57768, 36500, 10636, 4212,
                19900, 58920, 1876,  41612, 54688, 21420, 40488, 34780, 23936,
                11640, 48176, 21068, 8424,  52852, 39584, 4016,  32724, 27076,
                16792, 21208, 29992, 25216, 57324, 19916, 54192, 23636, 28200,
                18080, 9144,  58720, 29232, 42120, 19844, 64580, 6340,  38180,
                45020, 17432, 7040,  34092, 12208, 37892, 23964, 6616,  41912,
                55308, 17620, 34792, 12584, 50084, 15672, 45944, 33896, 2916,
                60864, 44628, 55464, 924,   42732, 5720,  65212, 27452, 7148,
                20264, 61640, 40052, 17568, 44972, 62676, 14512, 41296, 2832,
                6376,  32160, 22696, 45648, 27280, 64564, 8636,  12960, 56492,
                51160, 9944,  31052, 23444, 33880, 53184, 11980, 3032,  59228,
                19800, 708,   10144, 60004, 19204, 28204, 57860, 41944, 53748,
                25260, 8188,
            },
            {
                20472, 33336, 42012, 13860, 22304, 51772, 56940, 19788, 33184,
                64648, 26136, 18040, 9372,  55724, 20752, 7084,  17428, 46496,
                21812, 5528,  50820, 30484, 46100, 24896, 43736, 27328, 51852,
                15796, 44840, 24236, 62760, 14828, 4160,  63876, 49536, 53840,
                42544, 33076, 3112,  45728, 60780, 11164, 48740, 56856, 8596,
                59504, 54328, 35876, 720,   44680, 7556,  35080, 11452, 45040,
                56060, 32296, 2756,  22872, 46276, 34196, 15176, 50828, 58496,
                10956, 62900, 50164, 55364, 59256, 18172, 2692,  64184, 33464,
                25552, 2132,  38980, 65480, 31348, 24808, 62340, 56996, 22500,
                40216, 17992, 50852, 23572, 60572, 20804, 46172, 51068, 58368,
                34776, 15036, 10624, 57752, 1696,  22516, 52940, 18612, 65288,
                27940, 58200, 39888, 184,   52536, 5100,  43352, 48296, 29556,
                19944, 63172, 54432, 16004, 38772, 27152, 49928, 36476, 15304,
                46816, 23032, 32952, 40184, 44700, 3676,  22316, 15908, 47984,
                62576, 37104,
            },
            {
                59528, 2040,  63300, 8988,  31872, 61120, 41648, 5208,  44820,
                7388,  53008, 39752, 4028,  32888, 27088, 60640, 37800, 11724,
                65280, 26532, 35940, 9892,  20088, 63924, 2752,  34288, 59988,
                38796, 56096, 28568, 6188,  37056, 46256, 27240, 7840,  18900,
                5500,  65060, 15324, 26236, 31516, 18416, 36604, 22476, 44180,
                2820,  41624, 65052, 14460, 38692, 62108, 51016, 59764, 5064,
                21616, 48724, 38636, 54456, 7720,  26568, 56168, 31716, 22620,
                36088, 29112, 41936, 21232, 26092, 43776, 30144, 48460, 13120,
                57424, 46264, 22156, 6876,  41268, 160,   10424, 37696, 14324,
                4864,  27164, 10664, 32864, 38352, 3496,  31492, 10116, 40908,
                2088,  28852, 47396, 36620, 30884, 42908, 11196, 34536, 24168,
                50216, 9000,  55236, 12692, 17980, 35336, 23520, 60928, 2984,
                40732, 36200, 1308,  47288, 5792,  64264, 8876,  24636, 30376,
                57412, 51660, 62784, 13260, 6360,  50716, 38496, 9616,  952,
                30592, 12316,
            },
            {
                50068, 23912, 53704, 27004, 39324, 276,   15944, 49924, 58268,
                22560, 13176, 46536, 61776, 42020, 13888, 48324, 29304, 2540,
                52492, 44144, 57072, 900,   54100, 16204, 48888, 13440, 5640,
                21760, 8772,  33472, 52904, 18076, 31688, 57444, 13240, 38416,
                28540, 56992, 36188, 41452, 55268, 6512,  64092, 14940, 51396,
                28444, 18828, 10620, 48400, 22048, 26844, 16380, 30448, 41404,
                14756, 65436, 11892, 17152, 60600, 3988,  42904, 12836, 8104,
                54788, 2180,  14648, 4528,  46936, 9812,  39160, 52744, 19976,
                8372,  29736, 60064, 48796, 56152, 19288, 52224, 28392, 49612,
                64560, 47552, 58488, 7664,  53140, 14128, 56208, 22868, 17816,
                62856, 53200, 25060, 4844,  49544, 59756, 7656,  56556, 46412,
                15272, 43036, 20800, 61592, 30840, 38856, 57472, 15480, 26400,
                45640, 12132, 22384, 58672, 42044, 19200, 45764, 54792, 40844,
                4320,  8204,  36004, 26260, 54580, 31520, 63436, 34480, 57008,
                18192, 45752,
            },
            {
                5424,  35424, 17420, 44508, 48852, 11776, 30120, 25144, 36672,
                31964, 2684,  28436, 19444, 51072, 23860, 6480,  58536, 41108,
                31844, 18616, 13216, 38768, 28624, 41616, 23260, 61792, 30668,
                47120, 42216, 12532, 48864, 60172, 10060, 43872, 22268, 60684,
                47600, 2148,  23124, 12112, 50648, 360,   25068, 46320, 37852,
                5088,  33748, 52600, 30748, 56004, 3412,  9572,  47852, 37192,
                380,   43800, 25248, 35576, 51288, 39120, 19216, 63712, 48664,
                25336, 40092, 61092, 32068, 56820, 63312, 15460, 3836,  61700,
                36708, 42460, 15816, 11792, 26732, 35812, 44416, 7208,  32116,
                20708, 42336, 35596, 16572, 63604, 44396, 26272, 49120, 37372,
                43744, 8148,  21456, 61284, 13828, 26448, 38868, 2488,  29476,
                36868, 5388,  33792, 25752, 47436, 10344, 6192,  53676, 34044,
                8144,  56116, 50120, 28324, 14472, 32004, 2460,  60132, 12744,
                17976, 28840, 49208, 20756, 59600, 17052, 24196, 43516, 52632,
                27396, 40120,
            },
            {
                11232, 29708, 56428, 6736,  20992, 64008, 53432, 18384, 9536,
                62816, 54776, 38184, 10568, 56980, 368,   36536, 53532, 15420,
                7772,  24620, 63252, 49720, 32680, 58332, 8464,  37592, 52232,
                18620, 63456, 3036,  23068, 39760, 25932, 1628,  51888, 32320,
                16972, 53288, 45316, 19852, 59216, 34612, 29584, 61500, 9584,
                57944, 24504, 60468, 6424,  43088, 34392, 63992, 24080, 56972,
                27932, 53408, 62272, 30784, 10288, 46952, 28396, 852,   34488,
                44896, 17288, 51532, 37048, 7168,  22616, 34808, 27216, 44776,
                23888, 1352,  53756, 32572, 3536,  61836, 16256, 55432, 60348,
                12484, 2324,  24444, 30300, 19936, 40012, 6308,  59852, 2944,
                32480, 12612, 55264, 40472, 17884, 33280, 52164, 20500, 62584,
                12068, 58984, 51324, 3236,  64292, 41556, 49528, 19612, 63400,
                30656, 17500, 61736, 35024, 10220, 52832, 38348, 21836, 34596,
                65172, 45336, 41828, 1340,  10352, 46628, 4220,  13572, 7548,
                21824, 64440,
            },
            {
                58824, 42408, 14580, 60472, 34056, 37632, 4168,  59072, 47400,
                42884, 15600, 48736, 34040, 64248, 17220, 45540, 21360, 34256,
                60060, 46740, 5164,  21972, 11288, 4144,  45680, 26208, 316,
                56464, 27748, 36192, 15892, 53856, 64392, 5768,  35192, 42116,
                10760, 7080,  63352, 27528, 8660,  43524, 13184, 53120, 20176,
                40864, 16440, 45804, 12792, 19864, 39500, 51480, 18052, 7140,
                33708, 19576, 6344,  2532,  21472, 59392, 15452, 53164, 60300,
                21588, 5148,  11568, 27624, 18740, 54184, 49000, 11968, 58424,
                51436, 18288, 63900, 38612, 47740, 21716, 40680, 25072, 4512,
                46380, 38228, 57608, 51740, 444,   11552, 28952, 34944, 15956,
                64720, 50080, 30012, 45312, 408,   64120, 47188, 15984, 41476,
                54228, 23304, 45156, 17160, 29160, 14312, 22832, 792,   43512,
                38396, 3268,  24680, 6392,  44348, 63240, 26320, 48936, 6836,
                55844, 24024, 15032, 57520, 37392, 29480, 40760, 61372, 48540,
                32620, 2812,
            },
            {
                37972, 22544, 51496, 2236,  25792, 45988, 13916, 39940, 27828,
                1540,  20808, 25328, 7788,  30232, 40272, 26420, 62216, 10128,
                28492, 55456, 35760, 43628, 61056, 17924, 54880, 34188, 15060,
                9904,  44132, 59044, 7240,  30340, 45332, 20460, 14720, 61848,
                24544, 39836, 31292, 37368, 48984, 17712, 39172, 3952,  31940,
                49744, 1096,  36692, 62236, 28824, 2092,  44476, 13692, 61288,
                10848, 49960, 40044, 45128, 56376, 33168, 24376, 40996, 9652,
                30348, 57292, 65248, 42220, 45768, 472,   40424, 31044, 6476,
                33972, 10668, 28268, 5232,  57700, 13412, 8652,  34916, 29048,
                53808, 15120, 9008,  48328, 60980, 45356, 56696, 52740, 20908,
                25440, 5120,  37880, 9488,  57100, 24032, 5940,  31452, 9328,
                27660, 1680,  39140, 8784,  56580, 36244, 59900, 26864, 52200,
                13224, 48264, 41192, 54304, 19716, 144,   16132, 30164, 11112,
                39600, 3220,  32672, 50628, 63972, 18948, 53324, 26080, 36756,
                15688, 54364,
            },
            {
                8312,  27836, 47652, 10492, 31256, 55708, 7960,  22232, 33624,
                6520,  56104, 60288, 44268, 4464,  54240, 12852, 3556,  50920,
                41584, 1716,  14780, 26476, 51824, 39952, 21324, 62456, 50456,
                40768, 20388, 32544, 49580, 11656, 37760, 57812, 47816, 28744,
                55960, 3372,  58124, 14320, 1648,  60696, 55112, 22976, 64472,
                26952, 56364, 8356,  48732, 25508, 58796, 53956, 31456, 37884,
                46828, 58412, 24804, 13476, 37092, 7312,  49676, 4264,  14116,
                38320, 47500, 33760, 23312, 9100,  62308, 56232, 17056, 21316,
                60356, 41536, 50368, 24700, 44960, 31160, 64944, 50136, 43052,
                19396, 62792, 32808, 26576, 36280, 22504, 13944, 8316,  47592,
                42640, 58336, 19048, 27548, 35580, 12288, 43100, 59452, 35040,
                48604, 19136, 61108, 33292, 53108, 4300,  45736, 31888, 9308,
                57832, 21756, 29264, 59784, 36856, 47292, 57080, 42784, 51644,
                19636, 60888, 27096, 12500, 6096,  34180, 9408,  320,   58052,
                20272, 46272,
            },
            {
                62032, 17000, 40464, 64584, 19192, 42636, 61044, 48980, 52936,
                65368, 36360, 13564, 50696, 22912, 58872, 32484, 47920, 16752,
                37668, 19776, 59512, 7588,  29632, 2548,  12228, 30892, 5364,
                24664, 65156, 2340,  54928, 17404, 26940, 752,   8936,  50432,
                18700, 44424, 21744, 51864, 25572, 32780, 6812,  46768, 10820,
                14592, 43008, 21656, 33124, 16960, 10176, 5444,  23704, 20676,
                1192,  16048, 29688, 52748, 60996, 18092, 43480, 64016, 55212,
                26144, 2072,  15580, 51992, 29612, 13756, 35924, 26316, 2688,
                46812, 37224, 7224,  20028, 54740, 652,   17508, 58776, 3060,
                23448, 6048,  40516, 17292, 4596,  41996, 32124, 62228, 39004,
                1388,  51872, 15012, 61684, 48320, 51468, 21052, 55184, 4496,
                63360, 13520, 42940, 25644, 21376, 12248, 40244, 16680, 65524,
                35612, 4572,  15496, 10936, 32228, 8024,  22860, 62024, 4784,
                36384, 54212, 46848, 21464, 44320, 56216, 23524, 49600, 43184,
                11460, 31032,
            },
            {
                24844, 1048,  54016, 36132, 3940,  24240, 15520, 29852, 11548,
                18244, 41076, 29224, 9316,  38344, 19244, 43072, 7984,  64988,
                23084, 52880, 33072, 48744, 64276, 36712, 44592, 48408, 59680,
                35516, 13484, 46460, 39832, 23952, 62348, 41492, 31596, 36552,
                13116, 5896,  35064, 65476, 40848, 19968, 44160, 28740, 35256,
                52584, 38932, 4992,  63676, 50764, 36000, 40320, 63028, 55684,
                42368, 35236, 64916, 3516,  10676, 27376, 31416, 20944, 34892,
                50632, 18836, 6884,  60076, 38732, 3928,  50008, 44112, 64792,
                53424, 12596, 16184, 62196, 35916, 42252, 27260, 11124, 37648,
                55356, 45068, 51492, 10400, 65516, 54332, 2788,  18352, 28112,
                11092, 31744, 22960, 7124,  39216, 2252,  29620, 16944, 24792,
                37764, 50832, 29944, 7156,  47064, 62624, 50372, 6268,  24916,
                55224, 44904, 49892, 63748, 26932, 40572, 13516, 33568, 25708,
                15716, 8684,  29788, 2008,  38548, 60148, 14408, 28660, 65520,
                35072, 5160,
            },
            {
                59076, 49764, 32956, 13072, 57588, 51184, 38376, 524,   45124,
                26068, 3360,  47284, 16052, 63468, 972,   35588, 27464, 57600,
                30500, 10604, 42688, 13364, 24084, 17032, 57272, 9172,  27688,
                18228, 52480, 6928,  29724, 58768, 4508,  51944, 22052, 63576,
                53936, 27296, 46912, 8452,  11732, 55984, 16168, 62564, 1964,
                59388, 18488, 29436, 57140, 45136, 14388, 29836, 47992, 4400,
                27524, 8756,  44692, 22712, 40336, 48120, 88,    58428, 10076,
                41608, 62888, 44240, 54936, 25412, 20276, 58316, 9912,  31956,
                22520, 28968, 57296, 32748, 9720,  52580, 22420, 47412, 33564,
                13660, 30584, 59128, 21024, 28788, 49020, 24076, 45624, 59208,
                36832, 63716, 46348, 56000, 33216, 14468, 65208, 40684, 46188,
                11012, 628,   54720, 58420, 18020, 2908,  37476, 30092, 20036,
                41804, 33476, 1160,  18876, 53144, 58456, 2652,  50564, 45612,
                56460, 41524, 63624, 49024, 17132, 31644, 41236, 3256,  17940,
                51820, 39052,
            },
            {
                42820, 7344,  21928, 44532, 28136, 6168,  63228, 32848, 54996,
                59312, 21904, 52208, 32100, 55828, 25008, 50140, 14484, 45892,
                5704,  55936, 452,   38912, 53932, 4964,  22292, 41768, 992,
                38272, 56436, 21924, 10372, 35036, 15516, 45568, 11308, 3056,
                42584, 17168, 59740, 30068, 50740, 3668,  36992, 49592, 9416,
                24608, 47356, 12048, 312,   22880, 7428,  53036, 18404, 12324,
                59412, 50736, 19528, 56188, 33508, 52204, 16576, 45328, 4916,
                28616, 22444, 31324, 11464, 48192, 33560, 41780, 15248, 5600,
                39684, 1164,  43576, 47872, 5764,  14744, 39932, 60500, 7328,
                63892, 25352, 904,   37000, 12740, 57748, 33872, 15404, 6316,
                53824, 20532, 3848,  41360, 25500, 10196, 53500, 6952,  57592,
                32264, 22916, 15376, 40824, 34076, 26976, 54032, 61568, 12764,
                8776,  60492, 23724, 38604, 6896,  44488, 21272, 64844, 10812,
                19240, 4332,  34548, 24208, 10348, 54020, 7172,  47824, 22484,
                55700, 13892,
            },
            {
                18940, 30160, 61544, 9668,  17588, 47868, 20724, 10464, 42300,
                7672,  35164, 62012, 6104,  44744, 11228, 60964, 2904,  39704,
                18952, 25732, 44944, 60832, 28120, 34380, 62948, 51632, 15540,
                45448, 31896, 60304, 43532, 49472, 19380, 55688, 38156, 24836,
                56864, 33704, 168,   23428, 39232, 58436, 26780, 21236, 42324,
                31532, 55276, 34840, 41576, 27436, 61052, 43464, 34080, 24956,
                38988, 31892, 14796, 5604,  63332, 12552, 25496, 61532, 39060,
                13480, 57204, 36488, 1064,  16992, 4804,  63472, 24452, 56016,
                51168, 61416, 25848, 20988, 64328, 30396, 56204, 2096,  19964,
                49724, 16668, 42804, 47476, 5092,  39528, 9216,  51352, 26220,
                42280, 13140, 29356, 17700, 60992, 43928, 35352, 27124, 18804,
                60852, 36328, 45252, 64684, 9876,  22108, 43248, 1712,  47364,
                51892, 28608, 14044, 48056, 30600, 16856, 36260, 27704, 31864,
                39156, 58640, 13644, 51140, 60952, 26664, 36388, 62708, 32476,
                9208,  27420,
            },
            {
                63776, 47092, 35488, 55260, 40228, 58476, 36492, 24596, 49704,
                17356, 13008, 38740, 27744, 20368, 41208, 31264, 22380, 51708,
                63024, 35704, 15768, 8176,  20012, 46232, 11420, 30496, 25536,
                61804, 12404, 2848,  26724, 5280,  64796, 32824, 6796,  29052,
                49044, 14568, 63104, 43540, 18924, 13640, 33064, 6584,  60332,
                15296, 3528,  65036, 51792, 16040, 57836, 3100,  9708,  64244,
                54472, 1692,  47424, 28944, 38176, 7628,  35496, 54048, 49048,
                18048, 7752,  51684, 40904, 60456, 52772, 28776, 45284, 18956,
                34148, 10768, 16716, 37524, 3824,  50260, 24384, 45468, 29220,
                35716, 9052,  54204, 31908, 63404, 19176, 43920, 61112, 120,
                35292, 48868, 8068,  58364, 50160, 1136,  21712, 47788, 3248,
                52120, 7936,  28172, 3672,  48988, 13972, 57112, 32140, 17200,
                36440, 4348,  56808, 62716, 10556, 55172, 5416,  48592, 564,
                53804, 22888, 29104, 42528, 1368,  20104, 43888, 12028, 58240,
                45520, 1560,
            },
            {
                37428, 15416, 2784,  25700, 4644,  31384, 14252, 2332,  60448,
                30492, 57056, 1428,  49300, 15332, 54404, 8760,  58676, 33612,
                12292, 48188, 57728, 31512, 50424, 2224,  39616, 55896, 7668,
                49256, 20528, 36568, 53356, 40200, 23396, 13652, 60820, 40976,
                20112, 9788,  36212, 4948,  54668, 47516, 64288, 44728, 53232,
                37532, 19568, 8832,  39328, 21196, 31184, 37488, 49532, 21952,
                16468, 42088, 60752, 23772, 57760, 44036, 20812, 1616,  30136,
                24348, 64984, 45928, 26884, 21636, 35452, 7996,  12672, 38332,
                48644, 6696,  54700, 59356, 41868, 34552, 17840, 12104, 39068,
                57284, 61136, 22076, 14652, 27340, 55216, 23112, 30796, 16768,
                64408, 54884, 32224, 23808, 37188, 15424, 62432, 31140, 42328,
                13028, 39636, 20120, 59848, 52968, 38480, 7320,  24536, 59220,
                40868, 21648, 43612, 34308, 25096, 40296, 61004, 43156, 15932,
                62832, 8336,  46304, 6212,  37908, 55548, 16176, 4404,  24484,
                40108, 50844,
            },
            {
                54172, 23436, 59944, 12480, 51724, 45408, 64340, 53600, 27336,
                46296, 40664, 23760, 64848, 4552,  35820, 46696, 17572, 6768,
                28008, 3280,  23012, 37896, 65504, 14632, 59224, 18856, 4240,
                33920, 41600, 63052, 15920, 30308, 9084,  46292, 51252, 1652,
                58500, 44948, 52440, 27144, 30912, 10268, 2104,  23620, 12172,
                28936, 50012, 25824, 46960, 5660,  13024, 55368, 26860, 45924,
                7112,  36040, 11432, 3684,  49728, 17232, 32088, 10976, 58884,
                42680, 2940,  32400, 10032, 14136, 47296, 61820, 3460,  58092,
                27244, 31476, 44976, 13392, 27928, 8872,  63212, 53188, 6008,
                25996, 3272,  40584, 52356, 1700,  7616,  38032, 11708, 46532,
                21536, 38920, 3352,  45720, 11612, 55604, 5204,  50976, 17176,
                63940, 55808, 24840, 33196, 16636, 29528, 46404, 63764, 11040,
                27496, 54152, 7932,  19184, 2192,  51424, 13168, 21580, 26536,
                35476, 51704, 18140, 33068, 64884, 47492, 30872, 52088, 34688,
                20448, 6832,
            },
            {
                10452, 33176, 49212, 41164, 28876, 18608, 8912,  37840, 5116,
                20152, 10716, 33356, 55964, 43548, 29344, 24560, 63860, 39164,
                43848, 54732, 10576, 42400, 5748,  26412, 35348, 43312, 54420,
                23812, 9608,  47200, 468,   59056, 54736, 35664, 26296, 16740,
                32032, 22364, 7464,  61520, 40876, 16616, 57040, 34424, 41232,
                4908,  56668, 61236, 32928, 43208, 63292, 18908, 772,   61952,
                29904, 52648, 20416, 27040, 40832, 54648, 61944, 46564, 37772,
                14916, 55648, 19436, 59468, 39316, 54132, 30268, 42652, 21328,
                65488, 1892,  23044, 52032, 19804, 208,   57780, 42960, 15676,
                47168, 33948, 11836, 45372, 35936, 48488, 60204, 56968, 4424,
                28240, 9740,  59920, 19140, 27712, 40420, 33812, 24312, 9600,
                29216, 46004, 5608,  10828, 42868, 20,    35344, 19456, 50744,
                3480,  15128, 65096, 46084, 29096, 58604, 32768, 7088,  56932,
                10896, 40016, 60048, 14208, 21984, 9920,  27228, 62076, 14060,
                59556, 29492,
            },
            {
                64996, 43956, 20780, 6492,  62408, 34428, 22628, 43332, 15632,
                61940, 51604, 7544,  18168, 12020, 51008, 744,   56504, 14552,
                49488, 19688, 60624, 29832, 52428, 21960, 47644, 29180, 63836,
                13280, 51336, 27980, 18772, 44324, 21520, 4360,  11544, 64180,
                38860, 56020, 12788, 49308, 24364, 37828, 51240, 20292, 62632,
                45344, 17320, 1196,  23152, 10904, 48552, 34528, 40156, 14008,
                43772, 56612, 33372, 65312, 14452, 8944,  4524,  22412, 26488,
                7384,  49648, 35184, 5228,  24116, 528,   17908, 50596, 14788,
                9288,  40728, 35176, 61920, 48836, 32236, 36892, 21056, 30544,
                51276, 65112, 18732, 28696, 62488, 15568, 25256, 32408, 40756,
                49796, 14840, 43220, 52524, 64976, 7776,  46948, 57632, 38160,
                1292,  34912, 50016, 62960, 58140, 22788, 54468, 5980,  44696,
                39028, 30520, 49496, 37132, 9760,  17392, 38564, 50076, 45492,
                30764, 1584,  24940, 52996, 3596,  56528, 40648, 16,    48684,
                42452, 17648,
            },
            {
                55892, 2068,  37984, 16324, 53136, 396,   56752, 49448, 31944,
                58300, 36464, 42164, 26132, 61260, 37708, 21304, 6364,  32216,
                25616, 36372, 1816,  16264, 56676, 12380, 8796,  1032,  17096,
                39208, 32472, 56148, 7216,  38512, 33140, 61996, 42304, 48060,
                28244, 3264,  35056, 19312, 1016,  59260, 6252,  26708, 9060,
                31036, 13648, 36636, 53148, 28768, 58000, 4624,  51600, 24564,
                9148,  17868, 2232,  46096, 36920, 29560, 51132, 34120, 64024,
                53000, 40776, 28296, 63032, 44164, 56872, 34048, 25944, 37064,
                53512, 57068, 16836, 11200, 6240,  45896, 25052, 60748, 10316,
                4932,  23672, 57920, 8604,  42616, 20864, 5524,  53260, 18116,
                62968, 34100, 24532, 888,   30636, 22016, 13812, 60380, 18356,
                53900, 21548, 14544, 39184, 27192, 47896, 13444, 60988, 33408,
                23836, 60196, 980,   22224, 55864, 62080, 23956, 2920,  64524,
                19692, 48452, 42808, 28260, 36812, 45248, 18768, 8128,  37276,
                26252, 4604,
            },
            {
                31568, 24360, 58228, 47484, 26960, 11336, 39356, 25036, 12972,
                3908,  22092, 1760,  30616, 48516, 16216, 59608, 40604, 52052,
                8972,  62384, 45660, 33384, 39344, 49556, 61564, 36740, 45852,
                58760, 3240,  25332, 61000, 13948, 50680, 17348, 23824, 6044,
                51932, 15432, 43304, 63524, 46020, 29452, 14696, 47640, 54696,
                39608, 49640, 59084, 41760, 7460,  15696, 22112, 31704, 60432,
                38688, 48976, 59172, 6572,  24204, 60536, 40192, 12688, 17904,
                3468,  21416, 10740, 15964, 48956, 7784,  11596, 62208, 46324,
                3912,  24464, 43912, 29008, 39204, 54992, 14036, 3116,  41168,
                55836, 38240, 32192, 2420,  50584, 35028, 10656, 44788, 1980,
                55088, 7300,  58112, 37060, 16596, 51388, 43840, 3584,  26948,
                41688, 61408, 31404, 3728,  18532, 8736,  41300, 28816, 16548,
                57248, 11824, 42552, 33916, 14432, 4820,  43780, 28424, 35976,
                15236, 5432,  57716, 11268, 63812, 33452, 23916, 58580, 53196,
                11528, 46240,
            },
            {
                51948, 13192, 8252,  30360, 60392, 45152, 7144,  65300, 29136,
                46668, 54600, 63616, 45256, 9376,  33696, 4388,  44176, 28948,
                13164, 57776, 7120,  24852, 19172, 4808,  31756, 23036, 53208,
                20072, 11016, 42944, 48756, 29928, 1784,  57696, 37120, 9968,
                60112, 31224, 25296, 54092, 11220, 32972, 41980, 65292, 2852,
                21556, 25160, 3740,  19132, 64672, 35948, 45628, 55048, 3000,
                12356, 28844, 21596, 42512, 16348, 48216, 1800,  57980, 44572,
                32008, 47156, 57452, 37900, 30956, 21916, 41384, 28668, 6436,
                20020, 32444, 59296, 2268,  64536, 18176, 49976, 27656, 45048,
                16808, 48880, 13336, 25896, 54756, 59804, 29320, 37680, 22116,
                27116, 13148, 47104, 41412, 62120, 5876,  35992, 32212, 49352,
                10720, 7192,  44524, 52640, 56604, 37212, 64136, 2600,  51260,
                8384,  20052, 53536, 27112, 48520, 39808, 52116, 12140, 54632,
                22896, 61468, 31984, 17136, 6328,  50396, 13632, 30380, 63192,
                21664, 35408,
            },
            {
                60884, 19032, 42260, 36408, 14672, 21072, 53812, 17972, 35252,
                9864,  39700, 19468, 14308, 58024, 53980, 23556, 65088, 18108,
                47656, 22212, 41508, 53792, 64332, 44148, 57368, 13696, 28412,
                5784,  64760, 36064, 22200, 8472,  54012, 27304, 46468, 19708,
                40160, 56764, 7892,  37568, 4192,  22584, 57440, 18056, 35088,
                11600, 62072, 33316, 46908, 27100, 9908,  40704, 18156, 26020,
                64100, 35700, 53768, 32656, 56080, 10212, 20224, 28612, 54340,
                8608,  61548, 25660, 2452,  53444, 63980, 14232, 51024, 60156,
                38832, 48316, 13404, 52328, 22540, 8352,  31932, 62356, 35224,
                21412, 7248,  63856, 40076, 17652, 46372, 14228, 64876, 48716,
                40024, 32048, 20648, 10228, 28400, 54652, 12408, 63720, 22528,
                55992, 16976, 35356, 24944, 12072, 30720, 21620, 43700, 25640,
                35628, 47260, 5336,  63784, 9076,  31724, 18548, 59796, 7972,
                41148, 50728, 37804, 46788, 21160, 55348, 3828,  43232, 15996,
                6548,  40700,
            },
            {
                27992, 48532, 63964, 5400,  50768, 32908, 1672,  41368, 57380,
                50552, 25360, 6560,  28724, 37388, 2476,  27272, 10644, 55436,
                35944, 348,   30424, 14800, 9880,  26832, 2016,  40980, 49904,
                44996, 33448, 15784, 39976, 62680, 43556, 12196, 32504, 64892,
                536,   22004, 47828, 17600, 60920, 51508, 7532,  38924, 28188,
                51880, 42864, 6296,  56468, 13284, 52472, 61580, 5708,  50000,
                43048, 8544,  728,   13544, 27220, 62896, 34868, 38844, 24396,
                14992, 42172, 34388, 19092, 5856,  44828, 35000, 1172,  17424,
                54444, 10104, 27160, 34376, 37624, 42880, 56904, 11580, 1332,
                59576, 29448, 52776, 33584, 6076,  332,   24468, 56160, 9252,
                2948,  61124, 56976, 4432,  48484, 18508, 25392, 39756, 1724,
                46416, 28704, 65456, 50220, 1124,  59108, 48564, 6284,  55416,
                61512, 29660, 37856, 15620, 58468, 25044, 352,   45792, 33668,
                27044, 3136,  9528,  25416, 40724, 62352, 34384, 27188, 49668,
                56688, 1212,
            },
            {
                10916, 23156, 2972,  56268, 25612, 44072, 61028, 23664, 5044,
                15812, 62192, 34220, 52408, 42236, 20424, 50128, 39256, 31768,
                5808,  62752, 51540, 37944, 48304, 34412, 16908, 60984, 8568,
                24284, 54944, 57940, 4728,  18904, 25040, 3168,  35652, 14960,
                50876, 29092, 12936, 33956, 42772, 25820, 13776, 45836, 24,
                55472, 15832, 30528, 20832, 38452, 1040,  29204, 33592, 15152,
                20352, 57988, 46356, 39812, 50324, 3948,  43732, 7176,  63888,
                212,   51716, 11904, 60644, 40148, 26564, 56384, 23632, 30812,
                42980, 7252,  61600, 16068, 47520, 6032,  19640, 53344, 24892,
                47036, 42100, 10932, 22744, 61304, 43368, 31572, 19348, 35508,
                51996, 16968, 43908, 23840, 34872, 59824, 45028, 8700,  58016,
                37424, 13532, 5384,  41004, 20124, 34140, 15780, 39212, 12860,
                18032, 1500,  45532, 41880, 21152, 55620, 36964, 62864, 14336,
                20204, 57244, 53364, 29768, 13080, 856,   47700, 9272,  38676,
                18164, 32380,
            },
            {
                54272, 35044, 39460, 16700, 29564, 9248,  37644, 13204, 48280,
                31440, 44328, 516,   58992, 11920, 45816, 7580,  60960, 14056,
                43108, 26052, 17824, 3588,  59752, 21264, 54460, 37264, 30292,
                19268, 28,    12940, 31552, 52540, 47028, 59888, 54812, 41432,
                5220,  45076, 62684, 53152, 1976,  30312, 49404, 59332, 23548,
                37340, 9348,  60588, 50792, 44544, 24040, 57168, 47964, 38052,
                62136, 30836, 24872, 17912, 59912, 23008, 52808, 16644, 47660,
                36292, 21216, 29148, 49108, 15640, 8380,  47372, 11424, 64740,
                36100, 21560, 55760, 464,   25852, 63004, 30264, 39916, 15336,
                36220, 4480,  19116, 48524, 36924, 57532, 7740,  50044, 42272,
                13880, 29580, 37768, 53228, 484,   31448, 14944, 51088, 20340,
                30960, 53928, 23972, 60180, 46800, 10332, 28080, 63480, 49788,
                32588, 23248, 54392, 12008, 3492,  50276, 28988, 5976,  48912,
                42844, 64896, 16260, 44504, 60544, 37008, 19680, 59612, 24404,
                64836, 44780,
            },
            {
                7584,  58904, 46572, 12036, 53044, 63540, 19276, 58628, 28132,
                55640, 10292, 18124, 23360, 64232, 32664, 16876, 53128, 22840,
                58176, 46592, 11584, 56064, 29016, 42964, 5108,  12012, 50860,
                62328, 40524, 45604, 27556, 37408, 9440,  16852, 30100, 23452,
                58632, 26148, 9664,  39136, 20636, 63728, 10392, 32800, 18868,
                64456, 48160, 25840, 4416,  17340, 59928, 7856,  22080, 10888,
                3084,  53124, 5248,  37124, 8820,  29388, 11736, 58668, 30752,
                56724, 45516, 4940,  54076, 32456, 58576, 20280, 39376, 2744,
                49188, 29068, 44316, 39540, 51692, 12948, 3880,  49624, 58252,
                8952,  62088, 55408, 26340, 16016, 12424, 28216, 63560, 4664,
                25528, 62020, 7492,  12232, 64624, 42348, 5724,  26964, 63112,
                3172,  43660, 17496, 7900,  38040, 56156, 2760,  22216, 41184,
                9516,  59696, 26528, 65236, 34560, 17212, 40260, 22596, 11368,
                30896, 1944,  34940, 5276,  23292, 54380, 30804, 12060, 4620,
                51764, 14560,
            },
            {
                30432, 19648, 62664, 22344, 33676, 6612,  45952, 2272,  34996,
                21268, 39516, 49584, 36724, 26352, 3392,  48248, 29172, 2080,
                35084, 20612, 39740, 32384, 8196,  23784, 65260, 46624, 26200,
                35048, 6720,  59112, 49216, 20848, 63220, 6432,  49716, 11280,
                18424, 36592, 55560, 5828,  16384, 56484, 40800, 3764,  44056,
                6072,  34340, 12492, 41480, 36148, 31712, 14032, 42200, 55500,
                27600, 44244, 14760, 64360, 47044, 41928, 33964, 19464, 3308,
                26344, 9580,  64932, 23348, 37236, 3628,  27612, 52316, 59648,
                13808, 5344,  18096, 32304, 58868, 23384, 45636, 33944, 20952,
                28272, 44392, 32272, 2740,  51488, 45308, 39956, 21780, 33656,
                55904, 46216, 19712, 49480, 22672, 57540, 17828, 35928, 47676,
                10968, 33300, 62096, 26556, 52512, 31188, 44460, 58548, 4460,
                52024, 36744, 6596,  30504, 47508, 9660,  62044, 52220, 59164,
                38012, 24764, 47344, 18348, 50604, 7400,  42560, 57544, 33752,
                40948, 26160,
            },
            {
                37872, 5008,  42568, 252,   49700, 40324, 25928, 52236, 42540,
                4040,  63040, 7496,  54596, 14392, 41304, 57408, 37540, 10168,
                64632, 6648,  50760, 60436, 45232, 14364, 38364, 2100,  16360,
                53848, 10340, 23388, 14680, 3520,  42308, 32976, 38500, 60732,
                1288,  43824, 31900, 48112, 24696, 35964, 27392, 14448, 52868,
                22492, 29336, 55220, 63152, 2564,  52948, 46284, 64808, 35276,
                18216, 50812, 31604, 55948, 21044, 1336,  62648, 50072, 37848,
                13968, 40660, 18044, 43320, 12656, 62836, 45116, 16688, 33252,
                25180, 56632, 63608, 10860, 7452,  36860, 16776, 65056, 880,
                53720, 13912, 37992, 64424, 9564,  59196, 1280,  17324, 52752,
                10952, 2128,  40668, 27860, 33776, 9492,  39488, 52672, 59964,
                22852, 40456, 49432, 248,   13708, 18520, 24612, 34936, 16328,
                46924, 19668, 43808, 14456, 53612, 25276, 4232,  44796, 15144,
                8072,  56532, 40484, 11084, 63092, 27052, 15644, 46384, 22020,
                1524,  49360,
            },
            {
                10600, 55656, 28300, 15208, 57268, 30708, 17248, 11704, 59604,
                15096, 27252, 45296, 30620, 4888,  61712, 19088, 44220, 25128,
                54684, 16116, 27536, 860,   18848, 52608, 33236, 57420, 21028,
                42004, 32280, 64236, 35740, 52912, 56928, 25396, 45552, 21280,
                51728, 28524, 13908, 61928, 52276, 8124,  46560, 61080, 57720,
                38324, 16448, 45300, 8892,  27476, 19052, 24516, 6016,  308,
                40808, 22824, 6724,  35684, 13136, 26736, 53948, 7908,  44428,
                61644, 52176, 33724, 2220,  55564, 30272, 6780,  41732, 9504,
                38096, 46120, 21016, 50344, 28408, 55036, 42916, 9884,  25568,
                40712, 7064,  23132, 18000, 29640, 35004, 24496, 48144, 60608,
                30388, 36324, 58524, 15048, 54136, 46756, 1392,  13360, 29568,
                4560,  16024, 57164, 36136, 46228, 64828, 6412,  53764, 12264,
                29028, 63084, 920,   39220, 57784, 18968, 36024, 27652, 33172,
                20976, 53048, 28636, 58892, 32528, 38528, 3436,  52892, 60224,
                17988, 62176,
            },
            {
                34432, 51856, 24624, 60880, 36900, 8848,  64512, 23528, 47748,
                32612, 56684, 10872, 51612, 21704, 34168, 8232,  51224, 13664,
                31680, 48712, 41956, 36448, 62800, 25232, 9728,  48104, 61488,
                27760, 2668,  44616, 18380, 29116, 12224, 2328,  15008, 65464,
                8532,  57732, 4420,  22084, 42840, 2932,  19384, 31296, 10808,
                812,   49032, 20672, 58064, 39764, 50184, 61204, 29956, 12996,
                56644, 60584, 10444, 49028, 59476, 39852, 16400, 24216, 31972,
                21608, 6020,  59376, 24936, 48260, 19524, 51104, 23052, 60904,
                53300, 1548,  34856, 41036, 4600,  61460, 19940, 52384, 31212,
                60504, 50032, 57372, 47172, 42112, 55136, 5904,  38476, 13684,
                7072,  44736, 24656, 3904,  62172, 31104, 21000, 25316, 45560,
                54584, 8656,  20620, 28292, 10704, 32536, 42000, 61012, 38284,
                25452, 56388, 33444, 22264, 7820,  42080, 60336, 1496,  49092,
                64400, 5624,  13832, 132,   48940, 20560, 10000, 35800, 29592,
                13416, 43356,
            },
            {
                7848,  47256, 12980, 19224, 45424, 2588,  54548, 38916, 6108,
                36356, 18008, 24388, 40068, 58792, 47404, 28440, 1076,  39368,
                61164, 4352,  22708, 12084, 55736, 5356,  40112, 30244, 7280,
                13472, 50924, 56128, 5732,  39988, 62100, 48588, 31000, 35560,
                41064, 17576, 33520, 38684, 12092, 64592, 34884, 50780, 42344,
                25504, 62060, 5460,  32608, 15200, 10688, 37692, 48232, 33492,
                45228, 25936, 38744, 3716,  29108, 45700, 5480,  58172, 11316,
                49080, 29056, 14988, 39152, 10336, 35272, 57872, 14120, 4020,
                30560, 17344, 12516, 25652, 47464, 14524, 2320,  35816, 46056,
                16252, 3028,  32916, 12296, 4080,  20196, 63076, 28720, 42752,
                21532, 64444, 51232, 18808, 8344,  42836, 56744, 65128, 34456,
                38508, 62688, 43252, 50900, 59284, 4124,  21936, 48968, 8552,
                3612,  50636, 11160, 61840, 48020, 31128, 12896, 55716, 17440,
                41592, 45824, 36560, 23764, 43580, 54728, 65468, 25532, 5920,
                56112, 23100,
            },
            {
                2032,  31352, 41100, 4696,  33324, 49272, 29032, 20064, 61692,
                716,   44016, 65472, 3756,  16396, 12244, 63500, 23488, 56848,
                35320, 19772, 58612, 47128, 28236, 15836, 51524, 44384, 19704,
                59188, 37816, 24848, 47468, 9004,  23072, 19084, 53680, 5028,
                26376, 50492, 47336, 59684, 28096, 55132, 23788, 15092, 7380,
                29844, 53944, 35908, 43828, 1568,  55084, 21896, 7420,  16532,
                63648, 19912, 52404, 15312, 33188, 19264, 65308, 41684, 35492,
                1756,  56800, 45744, 64304, 27472, 636,   40152, 46480, 26672,
                65324, 43032, 55496, 59836, 32020, 38640, 23820, 58040, 11688,
                28552, 21668, 39484, 61960, 26388, 52424, 10304, 49836, 16896,
                56132, 32312, 12340, 38816, 35376, 50188, 3156,  11248, 17984,
                6264,  31616, 24292, 2416,  38092, 54892, 15316, 30004, 19000,
                40936, 45052, 15992, 28020, 3228,  52872, 23692, 38668, 10320,
                26056, 31668, 19100, 60556, 7828,  16568, 41084, 50524, 45672,
                38748, 63564,
            },
            {
                27032, 58948, 53460, 62780, 22172, 10140, 43448, 13760, 53020,
                27996, 49816, 9540,  31780, 37440, 54416, 42488, 17936, 45808,
                10792, 6908,  30400, 43256, 3132,  34708, 63736, 23308, 660,
                34136, 17128, 11324, 31556, 60248, 37020, 42556, 59136, 10940,
                56448, 23624, 892,   9080,  16520, 45692, 1988,  37564, 58936,
                47240, 12652, 23216, 65320, 26480, 60140, 41316, 53156, 28036,
                2172,  8932,  42548, 61768, 54828, 664,   48312, 27204, 53248,
                17636, 37300, 20868, 8000,  54220, 17452, 62280, 32924, 20076,
                36952, 6320,  50796, 22044, 8908,  63872, 49696, 6184,  42516,
                62884, 8032,  54620, 36532, 44440, 15080, 34244, 58744, 704,
                26856, 4372,  47428, 61300, 25940, 15576, 28732, 40492, 52460,
                46832, 14152, 57664, 18832, 12320, 26384, 45740, 60120, 35844,
                64480, 24088, 54296, 37620, 18312, 43756, 62952, 6608,  50488,
                61356, 2620,  48084, 57000, 29488, 34336, 11900, 1244,  32052,
                14748, 20004,
            },
            {
                36288, 11136, 17072, 37968, 25880, 56912, 60728, 34740, 7440,
                40820, 21388, 56028, 46180, 26116, 7096,  30116, 3356,  53176,
                26544, 64484, 37412, 53832, 18292, 59584, 10516, 38408, 55272,
                49164, 64788, 43080, 4248,  52124, 15772, 1900,  28332, 45900,
                14316, 37316, 63148, 20716, 41512, 32300, 52768, 62640, 19956,
                40372, 17216, 9040,  50964, 30724, 17932, 4104,  35208, 58852,
                49932, 36804, 30916, 24336, 11984, 38164, 22180, 13928, 8824,
                61452, 42472, 4088,  32468, 49400, 43424, 12312, 4876,  56424,
                48092, 11508, 29084, 256,   16484, 41532, 27440, 18912, 34400,
                52028, 47804, 17676, 1144,  22640, 30408, 7424,  37720, 46052,
                40884, 9552,  54616, 21944, 5376,  59032, 44604, 20284, 60116,
                1588,  27900, 48404, 41792, 64012, 33004, 7268,  52796, 580,
                13620, 32644, 6080,  59816, 9632,  35388, 29256, 20932, 34272,
                15924, 53876, 12444, 38992, 4228,  61816, 24148, 18672, 59652,
                52780, 48116,
            },
            {
                57464, 43960, 5996,  50348, 14980, 3980,  39720, 17580, 24928,
                59124, 12372, 33888, 2404,  19724, 62416, 49980, 34620, 15244,
                41392, 49444, 14080, 8980,  22276, 47936, 31748, 14272, 26740,
                8300,  29144, 21488, 57032, 26060, 33312, 64160, 21684, 6216,
                31628, 43444, 53296, 29628, 57840, 5964,  11328, 26924, 3924,
                33468, 56392, 2880,  36956, 44964, 11808, 47088, 24140, 14192,
                43696, 21784, 5236,  57384, 50956, 7516,  34728, 56068, 30540,
                24728, 51208, 11848, 23532, 59708, 30008, 25356, 51832, 8524,
                23768, 39856, 57640, 45364, 33012, 53924, 13208, 56472, 4428,
                25204, 31772, 10112, 60976, 55792, 48088, 65404, 19500, 23732,
                60368, 33644, 17632, 42572, 30532, 53328, 9120,  32936, 23588,
                37164, 54260, 9800,  35392, 5156,  50196, 23020, 43336, 10272,
                28728, 56276, 21320, 46220, 51076, 14712, 684,   45104, 58616,
                40772, 8412,  27316, 21452, 51828, 44212, 55092, 28464, 42108,
                3748,  8408,
            },
            {
                30284, 21672, 64604, 28064, 46620, 31240, 52072, 2212,  48268,
                30404, 63256, 15804, 51744, 58308, 39736, 10028, 22336, 56348,
                32024, 68,    24584, 61256, 40204, 2208,  56944, 41824, 46248,
                3648,  53440, 39780, 13864, 45352, 7876,  48764, 40056, 51860,
                61672, 18916, 3984,  13076, 36144, 49940, 22780, 44172, 48376,
                28556, 63384, 41696, 21244, 57364, 7736,  62924, 29672, 55248,
                10404, 64644, 17896, 44924, 28160, 41008, 63284, 46716, 2520,
                44512, 34004, 63660, 40512, 15476, 2572,  38224, 61100, 34744,
                15064, 62932, 19008, 36036, 60564, 3176,  46472, 37472, 64912,
                14752, 44808, 39828, 28228, 13272, 41764, 2956,  52092, 11800,
                29228, 50116, 63804, 13688, 37304, 292,   48660, 64156, 15276,
                7372,  61844, 21772, 30156, 59520, 16100, 39372, 19848, 58512,
                49220, 41884, 2368,  26096, 39132, 65196, 56488, 25152, 4612,
                30732, 63848, 35692, 47084, 15148, 6820,  37096, 9768,  64124,
                34660, 24732,
            },
            {
                51620, 39452, 596,   34012, 9068,  62052, 23048, 55732, 10712,
                44688, 5128,  38132, 43092, 24416, 13512, 47152, 60604, 4956,
                19124, 58184, 45544, 34116, 52080, 26028, 5680,  20452, 62496,
                16540, 34800, 60680, 1396,  19748, 36068, 11720, 57536, 16604,
                34844, 8540,  47832, 25904, 65084, 39576, 17696, 60672, 14240,
                6888,  50420, 25112, 15068, 52504, 33932, 39728, 19616, 504,
                37920, 31760, 48064, 2352,  60220, 16076, 19920, 5300,  59496,
                15088, 19096, 53868, 5864,  47272, 21736, 55116, 18136, 44804,
                27552, 4616,  52696, 7540,  25744, 20936, 9692,  30468, 22808,
                1788,  58596, 20580, 51020, 5104,  25784, 32636, 58108, 15960,
                36688, 1952,  7276,  46892, 24316, 55784, 12376, 39640, 26596,
                51456, 41312, 17208, 44480, 1236,  55180, 27844, 62652, 36912,
                14972, 34448, 61756, 11876, 31284, 19452, 7864,  53168, 48200,
                17848, 23288, 42932, 868,   62656, 31992, 49320, 22248, 16848,
                46824, 13900,
            },
            {
                60868, 17664, 55476, 43120, 13084, 37292, 19432, 41860, 35312,
                27000, 53524, 18756, 8504,  29164, 1412,  36424, 27180, 43460,
                38536, 12168, 28884, 6684,  16292, 65500, 30080, 51124, 36676,
                24268, 11140, 30792, 50448, 63208, 54276, 29964, 24280, 72,
                27092, 58712, 42152, 21540, 2412,  54528, 8840,  34720, 55576,
                38568, 11024, 32056, 1484,  60488, 26892, 5456,  49104, 59184,
                52368, 25856, 13588, 35124, 9588,  25164, 32912, 52764, 28956,
                39348, 9644,  27348, 36256, 58656, 10640, 50132, 31252, 1020,
                41228, 49040, 30692, 12344, 43084, 51028, 61872, 40232, 49144,
                53616, 35616, 7260,  33640, 56664, 17864, 38896, 8888,  45044,
                55152, 20696, 39864, 27684, 59704, 18412, 34444, 3424,  46448,
                31168, 4224,  57992, 25604, 13244, 46376, 9072,  3560,  24816,
                6652,  18748, 52260, 4288,  47580, 41456, 33588, 11028, 37508,
                13528, 55856, 59464, 10536, 19448, 26328, 57516, 1888,  38928,
                56164, 5636,
            },
            {
                11396, 48608, 20660, 25292, 51220, 59368, 5544,  15612, 64860,
                296,   22256, 61652, 49376, 32668, 55484, 64992, 15916, 8080,
                50236, 63156, 54712, 21076, 37792, 43936, 12756, 8516,  59400,
                43432, 47756, 6704,  27176, 41720, 15560, 5392,  44000, 38072,
                50560, 11064, 53652, 33784, 14716, 30128, 42684, 496,   23848,
                19744, 58964, 43652, 47480, 18428, 12432, 42892, 22984, 16196,
                7952,  41716, 61980, 54716, 38720, 50292, 43324, 11784, 22768,
                48480, 56788, 76,    31776, 43012, 25988, 7188,  62540, 13580,
                58352, 21436, 64816, 38556, 17104, 58084, 28084, 5036,  18276,
                11532, 61320, 15584, 43656, 63988, 46432, 21932, 61152, 3932,
                26264, 62304, 51736, 32232, 9612,  43212, 22536, 57212, 62852,
                19740, 11096, 49540, 38196, 65460, 34208, 53312, 30900, 47932,
                55772, 44600, 29012, 57484, 15880, 22704, 60176, 27608, 43536,
                51004, 3604,  28748, 33816, 53920, 41140, 12640, 61208, 44456,
                27456, 32696,
            },
            {
                41908, 36076, 7768,  63432, 2720,  28476, 48928, 32604, 57428,
                46360, 14176, 36164, 59176, 5696,  45132, 20776, 52040, 30756,
                23096, 35308, 2056,  10448, 47332, 56548, 33368, 18936, 500,
                54752, 21108, 58416, 38308, 9236,  22864, 60104, 47184, 64364,
                19812, 31020, 61360, 6112,  46776, 58068, 26628, 63132, 45580,
                52708, 29704, 4548,  64732, 35412, 55480, 30020, 63520, 45888,
                33532, 3184,  19064, 22496, 6352,  58956, 2020,  64452, 35600,
                7092,  61060, 45348, 20356, 64756, 16508, 40048, 35520, 46612,
                9604,  24856, 3800,  55080, 34692, 1400,  14012, 33108, 42448,
                26620, 47276, 29756, 23416, 80,    11912, 50504, 29984, 35052,
                42872, 11204, 15072, 4592,  65100, 50020, 6788,  29024, 14088,
                36232, 54656, 32428, 7032,  21204, 18104, 42604, 12348, 38628,
                64352, 9500,  23816, 37452, 62484, 5760,  54704, 1364,  20860,
                64528, 7704,  40096, 49956, 15840, 46192, 6224,  30180, 20312,
                53348, 3080,
            },
            {
                25964, 57224, 30508, 15972, 39572, 45032, 11700, 24456, 7612,
                29352, 41292, 10208, 23884, 16860, 40688, 11412, 2868,  58736,
                47612, 17796, 42524, 60008, 23908, 3664,  50168, 27716, 38940,
                13988, 32144, 3868,  17880, 56040, 49784, 32856, 3016,  14416,
                7408,  40736, 16412, 23940, 38368, 51128, 18540, 12700, 32836,
                6408,  16496, 38004, 22488, 8720,  51012, 2512,  36944, 10296,
                27572, 57252, 49664, 30084, 46648, 15508, 31280, 54300, 17516,
                41356, 25284, 14684, 11392, 52448, 3152,  57128, 28176, 18824,
                53996, 32500, 44380, 8592,  48212, 23560, 45160, 56408, 64300,
                6444,  38804, 55360, 9132,  36872, 27540, 54024, 6824,  17504,
                48528, 23516, 56804, 45496, 35764, 16516, 40380, 53024, 45200,
                976,   23512, 43632, 61224, 29288, 50928, 1860,  59736, 21676,
                16428, 32760, 280,   43316, 12724, 49596, 36140, 45828, 32368,
                14704, 25444, 36304, 22008, 4328,  63700, 24592, 35092, 8528,
                15204, 65256,
            },
            {
                22096, 46244, 4720,  53584, 33412, 58192, 19876, 54168, 38436,
                51156, 4152,  55780, 47952, 27964, 53428, 34088, 25100, 38948,
                5168,  13936, 26836, 52592, 31064, 15536, 63444, 41728, 60720,
                52468, 24904, 62244, 46028, 35536, 12080, 26472, 20948, 36268,
                55024, 27584, 45184, 3276,  64056, 10164, 4656,  36436, 48308,
                60452, 10680, 57680, 28028, 41196, 15220, 60240, 19984, 53340,
                39772, 12932, 65184, 42764, 10752, 26912, 37516, 21048, 4628,
                51032, 29868, 55324, 37604, 48352, 33260, 22972, 5000,  51244,
                36728, 15852, 60364, 29124, 19580, 62444, 37544, 9992,  21476,
                51628, 2856,  19632, 58944, 42384, 61744, 14444, 40348, 57900,
                31980, 796,   38256, 19336, 29920, 2960,  59200, 25484, 9836,
                60276, 39392, 15652, 4900,  10648, 57184, 35792, 26444, 6164,
                54052, 40124, 51340, 19072, 30816, 26240, 8364,  18084, 61344,
                53028, 42332, 58624, 9964,  31248, 55652, 37784, 52184, 59456,
                47988, 38192,
            },
            {
                9532,  61292, 13268, 43644, 22900, 9128,  2000,  62592, 16536,
                33924, 20956, 64164, 31632, 1092,  61156, 14732, 43112, 63852,
                54116, 32984, 62016, 40032, 7392,  36540, 10056, 21880, 5948,
                35108, 11480, 42972, 28836, 1924,  63496, 41352, 51916, 57904,
                10456, 48900, 59932, 35144, 20664, 29020, 56356, 40720, 22240,
                26036, 42432, 50256, 1152,  46464, 24876, 32200, 4856,  48224,
                23392, 5968,  35656, 17584, 756,   61884, 57340, 45008, 12572,
                34000, 62700, 1928,  21820, 7944,  41200, 61444, 12076, 42620,
                63288, 1644,  39596, 13100, 50476, 5908,  30640, 53420, 25556,
                14920, 34752, 49236, 31144, 16724, 47768, 2596,  21440, 63344,
                9324,  52484, 60824, 25900, 54524, 12580, 21112, 33112, 48932,
                18596, 27884, 55924, 46604, 24604, 40312, 48440, 14244, 62188,
                28364, 46088, 59272, 4648,  63772, 55816, 40972, 11352, 29412,
                2644,  46916, 18432, 61956, 43788, 736,   16948, 11672, 42708,
                1708,  18224,
            },
            {
                55788, 34724, 28016, 17948, 60244, 36496, 47780, 41900, 27332,
                59628, 12004, 43544, 18272, 8836,  37040, 56780, 6868,  29004,
                10152, 21692, 776,   46516, 18480, 58312, 44140, 26452, 46992,
                16924, 49692, 7796,  20540, 54156, 15912, 5032,  30036, 43428,
                22968, 948,   31860, 12556, 53068, 43140, 49660, 15360, 61968,
                3120,  53884, 20044, 34064, 62712, 11864, 54796, 43348, 16708,
                59380, 31720, 54960, 25080, 51968, 40692, 8120,  49492, 23992,
                58448, 9704,  43552, 27672, 59880, 17728, 30256, 48944, 25892,
                7744,  22164, 57456, 26248, 42028, 58664, 16692, 600,   41024,
                45748, 60612, 12436, 24152, 5560,  34184, 25796, 45072, 36096,
                28156, 13788, 41852, 6400,  46484, 63268, 37448, 42704, 7528,
                64700, 3396,  51544, 33404, 63808, 17280, 8288,  44032, 33824,
                2996,  11152, 24964, 14676, 34572, 21724, 48036, 58392, 38380,
                23752, 5992,  12916, 27632, 50968, 20728, 47040, 29088, 23220,
                32376, 50932,
            },
            {
                3804,  41096, 49424, 520,   51980, 25520, 31360, 14220, 6564,
                45632, 3216,  25460, 39480, 51376, 46784, 22624, 19044, 49152,
                44652, 36836, 56948, 12696, 29132, 49876, 2796,  55124, 30548,
                1296,  65024, 56768, 33392, 39496, 48132, 59360, 17680, 6856,
                65204, 39268, 18676, 57036, 6984,  25132, 1328,  31048, 8868,
                35232, 13428, 30184, 17124, 56636, 7552,  38388, 26660, 63144,
                2112,  45088, 9024,  47848, 19880, 32640, 14428, 28316, 3088,
                38788, 47820, 15628, 36044, 53976, 45204, 784,   15124, 55976,
                31524, 47076, 52588, 3072,  33756, 11348, 46956, 36092, 63372,
                27960, 7884,  56868, 39548, 65144, 53096, 11260, 55768, 4536,
                18300, 49972, 22288, 34152, 10308, 50772, 1580,  56172, 14768,
                31180, 36988, 21468, 13180, 532,   30568, 52980, 22948, 57952,
                49920, 20320, 36764, 52896, 44768, 6840,  1740,  16512, 50252,
                35136, 54084, 57568, 32868, 38592, 8220,  64272, 56444, 5588,
                62276, 26716,
            },
            {
                11408, 21088, 63792, 38656, 12456, 5684,  65008, 21624, 50436,
                57108, 36284, 54612, 62364, 12848, 27300, 4276,  32372, 59996,
                16620, 6368,  51348, 23592, 34768, 64228, 20200, 14268, 59700,
                37088, 41012, 27264, 13464, 23976, 9848,  36984, 25580, 34316,
                50808, 14932, 26880, 47388, 37004, 62724, 17448, 58732, 38968,
                47420, 65412, 43948, 4444,  40280, 21956, 49748, 35196, 13624,
                20960, 28820, 38780, 61352, 3916,  36556, 64060, 53304, 17268,
                31332, 20208, 65264, 3960,  24224, 10540, 38724, 64324, 35396,
                18976, 10380, 37160, 17352, 65420, 23240, 54288, 20344, 4504,
                32256, 18100, 43384, 1492,  50172, 19840, 30604, 60376, 40064,
                47236, 64660, 2800,  56720, 30780, 17780, 27424, 23756, 46236,
                52528, 10172, 41708, 45436, 59568, 37732, 19376, 5272,  39400,
                29924, 64856, 8500,  40432, 28076, 61740, 32660, 25768, 65408,
                8680,  21220, 41204, 4084,  15108, 25876, 34328, 40888, 13872,
                36776, 44632,
            },
        },
        {
            {
                50672, 4884,  34576, 46304, 63860, 11160, 55128, 42372, 1360,
                30776, 56292, 45724, 17060, 54584, 62500, 33824, 9172,  37076,
                3920,  50080, 14332, 38564, 60408, 18540, 46208, 7476,  40532,
                61756, 37364, 22872, 55412, 41876, 59472, 39024, 8084,  15500,
                42980, 56840, 21184, 63772, 45980, 5232,  56744, 33464, 6732,
                53004, 11560, 61344, 272,   19396, 49708, 59528, 8096,  55352,
                35052, 27584, 13236, 19600, 55176, 15360, 27228, 7776,  37016,
                1652,  39832, 18816, 49684, 15592, 568,   8860,  30020, 56524,
                26708, 40284, 34636, 4260,  50892, 15748, 34248, 10624, 54120,
                31724, 36340, 12096, 40932, 26312, 63404, 17520, 53444, 9400,
                57508, 15440, 50796, 61424, 41396, 16188, 56760, 28888, 39756,
                3816,  56372, 45004, 61728, 7696,  17640, 47140, 30880, 49372,
                23172, 564,   52748, 17956, 63868, 40904, 15612, 26956, 3504,
                18456, 36188, 50652, 60880, 8728,  37012, 62020, 17788, 56080,
                47868, 33156,
            },
            {
                65428, 12972, 54620, 17412, 22204, 38144, 51256, 6448,  18228,
                49120, 22568, 34988, 59888, 20844, 6744,  44868, 16068, 64644,
                26848, 43576, 55164, 1592,  10120, 33224, 13248, 26476, 15700,
                2196,  50620, 17460, 10468, 14328, 51472, 18972, 46012, 64876,
                3108,  33416, 50484, 25616, 12164, 37964, 16912, 49592, 27188,
                37104, 21684, 47324, 55784, 5528,  28168, 45332, 32620, 16888,
                1716,  57492, 9584,  45892, 59916, 34848, 41268, 63740, 16664,
                32684, 62148, 11688, 6820,  35856, 47516, 22264, 43856, 14300,
                6716,  62112, 54568, 24516, 57568, 28992, 38528, 61376, 18028,
                43420, 62896, 22304, 1196,  56352, 37416, 3760,  44196, 35112,
                24292, 5044,  20252, 29720, 9692,  35056, 44612, 6724,  52648,
                36656, 9392,  50632, 21992, 41924, 37592, 3264,  10288, 54400,
                65352, 27128, 46356, 31980, 10484, 57396, 37344, 53588, 47176,
                64100, 42192, 776,   15720, 57228, 24220, 11556, 44940, 7440,
                21240, 28764,
            },
            {
                3328,  24744, 40204, 30100, 2128,  61176, 26656, 34260, 13252,
                62024, 40780, 10364, 2952,  38132, 26396, 49300, 41864, 30780,
                21028, 6528,  47760, 25208, 58400, 41944, 65184, 48800, 57676,
                20744, 45676, 28320, 64224, 33680, 24676, 5676,  36252, 26936,
                52328, 17588, 4708,  30300, 60068, 42072, 20908, 9556,  65116,
                2760,  41296, 16364, 32252, 37800, 14288, 41740, 62428, 5932,
                50608, 40600, 22716, 49268, 5672,  24240, 10752, 56248, 47964,
                23200, 51532, 42772, 27380, 57412, 40716, 53520, 63520, 19540,
                50176, 31420, 17216, 44740, 12084, 21844, 7352,  49864, 26524,
                6396,  52536, 29232, 45632, 10156, 32148, 58848, 12572, 27844,
                65172, 38548, 55360, 48020, 588,   59700, 22260, 32132, 13908,
                24696, 18824, 31308, 1944,  27536, 63280, 24556, 34928, 42564,
                5100,  39432, 58992, 6412,  22240, 50856, 30256, 7744,  34016,
                22148, 11376, 27700, 33024, 46180, 3892,  31216, 40848, 59988,
                14680, 37912,
            },
            {
                43300, 9952,  52012, 59312, 7884,  43500, 20000, 46712, 58352,
                25092, 7232,  53412, 30008, 51564, 12380, 58376, 828,   10688,
                59780, 34944, 16996, 29748, 36848, 52776, 22960, 5184,  35928,
                32352, 6712,  53448, 38316, 128,   48120, 61648, 30812, 12688,
                58684, 40056, 47144, 36104, 7468,  53768, 47720, 31260, 14044,
                57416, 51128, 8452,  25928, 52568, 58228, 10504, 20428, 25488,
                36784, 60960, 29336, 3088,  32104, 52128, 44220, 2492,  29764,
                13636, 3828,  59228, 20860, 31700, 5140,  11872, 34036, 3432,
                42508, 10320, 37500, 1352,  65432, 41348, 46984, 13876, 58940,
                2384,  39536, 15228, 49256, 19632, 23816, 40376, 16520, 52300,
                2088,  45204, 32500, 13472, 26588, 53624, 18384, 47584, 62104,
                57712, 42600, 54784, 15352, 48412, 52436, 13260, 56688, 16832,
                30104, 20416, 13656, 42992, 26092, 1176,  62712, 14388, 25056,
                58300, 48500, 55384, 39628, 21328, 52468, 64432, 26836, 1728,
                49052, 55088,
            },
            {
                62460, 20120, 47124, 35832, 15268, 54020, 10072, 37232, 2636,
                32096, 44064, 15368, 64284, 46628, 17808, 33000, 24684, 54180,
                50432, 41100, 63156, 12312, 7924,  19292, 676,   28868, 43792,
                62888, 12700, 23712, 58900, 19560, 41388, 9304,  54196, 20820,
                1228,  24004, 10428, 56304, 15196, 23512, 772,   60580, 43988,
                19200, 34168, 45964, 63576, 22648, 3888,  47724, 30696, 54000,
                44456, 14600, 18668, 64072, 37764, 17024, 60460, 21176, 39016,
                54476, 35136, 9360,  45396, 62864, 17680, 25268, 38712, 57616,
                23920, 59652, 27648, 53460, 33244, 5348,  30700, 55576, 35764,
                21468, 32528, 60280, 6900,  64416, 51604, 4572,  47764, 60016,
                22308, 7760,  17424, 61876, 39548, 5856,  36388, 10864, 1404,
                27160, 5636,  64980, 38492, 33364, 4296,  21228, 45096, 60864,
                9244,  49648, 55744, 61624, 35044, 48084, 19412, 44652, 4932,
                38228, 9116,  17648, 62412, 6936,  14092, 49836, 18572, 35944,
                23372, 32068,
            },
            {
                16808, 27476, 592,   32608, 23680, 48944, 29556, 65024, 16844,
                49952, 57292, 27472, 21800, 34696, 4960,  61564, 19764, 38212,
                14648, 2632,  23240, 56492, 44332, 50012, 39044, 55056, 17332,
                9032,  40404, 49388, 4836,  29448, 57116, 16696, 44116, 37708,
                48600, 62880, 42384, 19800, 64140, 28884, 39484, 35252, 24936,
                4580,  28584, 12340, 1564,  35660, 18108, 39508, 65436, 8392,
                840,   34196, 10188, 54688, 42376, 12544, 27972, 8236,  65048,
                46364, 25476, 16260, 38368, 2612,  55132, 50540, 29624, 15424,
                46256, 7688,  20736, 48420, 16540, 60532, 24232, 18784, 8936,
                42856, 54260, 11692, 36732, 42256, 27104, 34528, 8844,  30452,
                42784, 35800, 50388, 57404, 23536, 43476, 63832, 51404, 40788,
                34188, 46732, 11504, 19984, 8312,  57976, 40208, 28164, 1284,
                37400, 32944, 3360,  16480, 38996, 11900, 56092, 41160, 60428,
                51868, 30620, 1604,  43680, 25908, 37612, 9708,  42796, 58132,
                11788, 6684,
            },
            {
                45708, 38892, 56764, 63548, 5612,  40832, 55644, 4524,  22776,
                39388, 11532, 1212,  42632, 8796,  55968, 40216, 45684, 8228,
                28372, 32772, 46912, 4856,  61088, 31096, 14580, 59400, 47136,
                27012, 60996, 15356, 36116, 45196, 11652, 26212, 4180,  60576,
                31812, 27756, 6072,  33896, 44724, 4280,  52292, 10880, 55624,
                62676, 40260, 54240, 59256, 43092, 27720, 57196, 13320, 24192,
                51512, 59144, 46260, 25624, 6768,  57612, 35820, 50252, 18276,
                5460,  32420, 61096, 48648, 28580, 8040,  43980, 64848, 1036,
                52744, 35608, 62680, 12696, 39356, 2608,  51940, 45476, 64056,
                27824, 48136, 24964, 17784, 3292,  54664, 13960, 63004, 19272,
                55680, 12060, 28428, 3020,  9880,  31372, 15916, 28932, 21588,
                14484, 60040, 23976, 43452, 29716, 62048, 12068, 50596, 25148,
                53504, 45972, 22868, 28544, 54124, 8264,  32168, 2676,  27508,
                15512, 20472, 35292, 54028, 59532, 32536, 55836, 4400,  29812,
                51572, 60260,
            },
            {
                3884,  14104, 49572, 11224, 18780, 26988, 13744, 45140, 33964,
                60312, 19496, 54144, 36788, 49060, 14264, 26640, 3900,  57552,
                65016, 52980, 18084, 26388, 34752, 21144, 10740, 5988,  24476,
                34084, 2500,  55956, 31628, 21388, 65364, 51900, 34284, 8304,
                13632, 17124, 54860, 50968, 12580, 59224, 18136, 48384, 7148,
                16040, 20960, 31300, 9704,  15052, 48748, 6828,  32456, 42776,
                19856, 28864, 39368, 21836, 48616, 31504, 1160,  24012, 40572,
                58100, 11176, 51644, 22488, 14132, 36312, 19716, 10664, 22780,
                40492, 30624, 4160,  43948, 25952, 57332, 36396, 11528, 5968,
                16400, 612,   57720, 30716, 61012, 21104, 46368, 38336, 25280,
                992,   40044, 64236, 20740, 45640, 49620, 4016,  56348, 7872,
                52728, 36756, 4676,  49760, 54340, 17052, 35964, 6624,  64652,
                18772, 10780, 62520, 5624,  43720, 24940, 63500, 21820, 36436,
                45824, 65288, 48904, 12676, 3584,  22636, 15856, 47784, 20604,
                40576, 34312,
            },
            {
                54592, 22312, 29268, 42664, 59136, 36176, 62360, 9052,  52180,
                28856, 6392,  63632, 30348, 23768, 62456, 51948, 31464, 21972,
                11448, 42552, 37888, 9212,  54484, 41620, 64476, 37252, 52352,
                42832, 18776, 51284, 8140,  47888, 2004,  40248, 23108, 56388,
                49520, 39448, 2548,  25340, 37452, 32160, 22548, 27984, 42780,
                36504, 50424, 46376, 23924, 62384, 52112, 2116,  37676, 56144,
                15912, 5320,  60792, 3388,  13968, 63460, 44532, 53044, 14668,
                30084, 43364, 292,   56844, 41616, 61680, 31748, 58652, 49352,
                13940, 56228, 19044, 50940, 9420,  21280, 29404, 41004, 32988,
                62072, 38016, 44100, 8220,  49692, 33356, 10512, 57652, 6304,
                52636, 47500, 15280, 33804, 54248, 37848, 24468, 60680, 42464,
                18032, 31244, 57248, 26560, 152,   47152, 23104, 32304, 43036,
                15012, 39924, 57580, 33876, 51828, 17828, 48700, 13080, 57548,
                9948,  5996,  24388, 28912, 40408, 45228, 63356, 27712, 61260,
                8808,  25064,
            },
            {
                64972, 37480, 7820,  52496, 2880,  21704, 48332, 868,   24764,
                14712, 41124, 45468, 17544, 11120, 1788,  43776, 16588, 35772,
                48088, 1416,  61644, 15696, 4104,  48436, 28624, 1124,  57080,
                21948, 11272, 63448, 27448, 37316, 14248, 61364, 29272, 18552,
                45464, 59460, 21408, 62432, 8620,  46460, 2140,  64868, 13040,
                60060, 368,   5844,  38764, 18460, 33744, 26084, 61572, 46924,
                9440,  49512, 35580, 53804, 17732, 38428, 9560,  20504, 61776,
                4336,  37188, 19480, 27208, 6776,  47552, 2320,  26168, 34080,
                5596,  61332, 28236, 34472, 64640, 47548, 14708, 59192, 49896,
                19864, 52452, 13412, 22900, 40228, 2180,  28500, 43616, 17076,
                31424, 58608, 26500, 5376,  59492, 18532, 11536, 33188, 47008,
                2420,  63432, 9760,  39196, 14744, 41684, 58392, 3480,  55336,
                48124, 30492, 2332,  14128, 37160, 59828, 5052,  39764, 52712,
                31048, 41912, 60556, 18184, 52176, 7748,  36288, 356,   13296,
                44308, 17612,
            },
            {
                2080,  57716, 44960, 33080, 15820, 39680, 31028, 43876, 57960,
                35740, 49700, 3460,  59116, 33420, 38396, 56928, 6708,  60088,
                25468, 20188, 29912, 51608, 58256, 23612, 18216, 46308, 14168,
                30520, 39504, 3620,  43936, 17300, 54076, 6496,  42984, 488,
                35224, 10228, 30852, 42684, 15568, 57896, 39640, 52976, 33116,
                25264, 56692, 29412, 53844, 8328,  41872, 20876, 12168, 30252,
                23428, 64520, 32080, 43084, 26348, 56400, 28924, 34484, 46160,
                24904, 55540, 63872, 33452, 12080, 52812, 17264, 38776, 45804,
                8888,  42472, 15992, 40,    38544, 7032,  53728, 2068,  24044,
                4388,  28900, 35468, 56720, 65272, 15128, 51972, 62200, 35600,
                22504, 8536,  44708, 12872, 41720, 444,   65380, 26376, 38820,
                13284, 22820, 45448, 19484, 33764, 64004, 10268, 20248, 27460,
                7968,  24080, 50704, 20804, 9468,  27924, 45176, 48,    26188,
                15120, 55664, 1968,  33632, 11408, 56616, 19180, 50512, 38944,
                53264, 31368,
            },
            {
                48100, 10836, 19732, 25192, 55376, 60836, 11624, 17860, 53920,
                20660, 9980,  26924, 55092, 21644, 47384, 28136, 9868,  40632,
                50108, 13056, 45188, 33568, 39360, 11848, 60876, 32924, 8532,
                62308, 49040, 59328, 22896, 33128, 58104, 25576, 50700, 12848,
                64080, 26668, 5208,  50376, 54244, 29180, 6192,  20272, 10048,
                17480, 44848, 34932, 13856, 64720, 57544, 45216, 3732,  53200,
                41176, 44,    19368, 11716, 7596,  49928, 1816,  58980, 12672,
                49324, 8260,  15552, 45252, 40472, 58056, 23888, 64220, 54936,
                20672, 51952, 24772, 59968, 45280, 32152, 18392, 27240, 42964,
                46844, 60548, 10652, 5204,  45784, 18884, 24880, 4828,  11908,
                39372, 50688, 62672, 36960, 23004, 30284, 48476, 7376,  55420,
                50444, 28228, 60824, 52152, 7008,  29572, 50260, 38268, 61520,
                35160, 58924, 45512, 63136, 42308, 31832, 22700, 64108, 34596,
                20060, 49604, 37164, 46560, 64156, 30432, 23744, 33068, 58864,
                5392,  26996,
            },
            {
                15180, 34876, 62876, 3912,  49912, 5540,  27872, 64316, 6892,
                32336, 61144, 40116, 14180, 5196,  51660, 18988, 63272, 31852,
                55168, 5112,  63700, 7828,  27108, 2888,  44072, 36560, 54324,
                25708, 5032,  35344, 12176, 46116, 9288,  31008, 38796, 21988,
                47288, 57040, 36952, 19448, 11968, 24248, 35804, 49176, 41468,
                63096, 3664,  21556, 48356, 27944, 5452,  16956, 36860, 59864,
                15040, 27596, 57820, 45788, 62440, 23256, 41472, 6516,  18212,
                39396, 30956, 51764, 22692, 3416,  29796, 9644,  14740, 4468,
                31172, 36000, 11876, 55372, 22340, 10760, 62288, 56172, 8164,
                33916, 16764, 53596, 26156, 31960, 37328, 55960, 47688, 60108,
                19544, 1752,  28620, 15968, 53320, 57636, 43400, 20376, 16100,
                4420,  35240, 42368, 1724,  25360, 54764, 13172, 43984, 17148,
                764,   12316, 38880, 6568,  53376, 15640, 56232, 47944, 7196,
                61220, 9640,  23224, 4860,  16112, 43104, 2916,  46088, 10344,
                21460, 42180,
            },
            {
                59832, 51296, 29000, 41188, 14208, 36408, 22680, 42960, 38272,
                1848,  46860, 24208, 65260, 35256, 42532, 12632, 23592, 0,
                37548, 17620, 22768, 42248, 56800, 20580, 50516, 6404,  16704,
                41356, 19680, 28860, 55592, 2672,  64632, 19160, 52948, 3648,
                32676, 16352, 41248, 2956,  62960, 44472, 60272, 1348,  27100,
                51524, 37756, 58440, 10748, 39788, 51132, 22744, 31480, 48240,
                7388,  38940, 50944, 33480, 16396, 37044, 30548, 53736, 65244,
                26692, 2268,  36184, 59884, 18800, 49812, 37728, 43316, 27224,
                61924, 47372, 39996, 3268,  29576, 49284, 36100, 40300, 12740,
                64116, 38988, 21488, 49440, 41636, 236,   9072,  29604, 42428,
                32916, 54968, 46980, 7640,  33860, 3276,  10848, 36024, 62848,
                31436, 57060, 12172, 17336, 59208, 35876, 4188,  23184, 49132,
                54128, 29240, 18520, 25692, 2020,  36048, 10956, 18232, 38932,
                43880, 28496, 53684, 58276, 27216, 38304, 60508, 14008, 54344,
                63908, 36872,
            },
            {
                1192,  23264, 8068,  46028, 32008, 59940, 47804, 10404, 51328,
                16416, 29120, 53304, 8108,  30192, 3180,  59352, 53556, 46384,
                27484, 58948, 34908, 52740, 14640, 31616, 64956, 24184, 58364,
                46500, 52252, 14696, 37924, 49488, 41504, 15244, 59512, 7248,
                61460, 24568, 55196, 48712, 33668, 8320,  16828, 55392, 13308,
                31444, 6728,  16172, 24580, 33168, 1204,  55712, 63396, 10192,
                24832, 54908, 4728,  21928, 2800,  60240, 13844, 46796, 20808,
                43672, 55000, 11208, 63164, 6120,  34480, 54024, 59568, 1128,
                17508, 7848,  57488, 19308, 43976, 5472,  15580, 51476, 23448,
                30092, 2836,  7364,  59404, 13776, 62604, 22244, 51556, 14360,
                4000,  64772, 25240, 21080, 61852, 49820, 27740, 40344, 52644,
                22060, 9056,  48300, 38628, 20620, 46448, 62352, 31768, 9492,
                41012, 65316, 33448, 57140, 47016, 61884, 29516, 51304, 2992,
                14216, 32532, 41064, 12284, 21156, 8172,  50232, 29044, 24968,
                6604,  18592,
            },
            {
                44500, 53144, 12120, 57068, 19364, 188,   54804, 25992, 34120,
                57260, 12000, 19120, 44080, 49576, 16184, 39124, 32952, 8552,
                15136, 48908, 3392,  10224, 46068, 37668, 1776,  12988, 30124,
                10164, 252,   63236, 21860, 7840,  24956, 35980, 45436, 29600,
                43148, 9504,  13696, 28104, 21608, 40080, 30576, 46476, 23324,
                42828, 61256, 52868, 45984, 60356, 42624, 13192, 35228, 20216,
                44048, 29996, 63808, 11344, 42892, 52148, 24736, 9848,  4428,
                32968, 15004, 41224, 28196, 45636, 25224, 12384, 48220, 32748,
                23328, 50960, 13936, 33952, 62944, 25800, 59232, 760,   19728,
                47684, 43312, 55220, 27664, 18268, 45236, 35340, 58596, 26976,
                17440, 40288, 10432, 37348, 44640, 13452, 18308, 60400, 1240,
                43800, 26176, 64696, 30456, 7716,  41464, 15276, 55660, 26044,
                6084,  21572, 51636, 14456, 8672,  41212, 20528, 58384, 25532,
                55272, 65008, 740,   47520, 52560, 62724, 17540, 34228, 41040,
                48636, 31788,
            },
            {
                14756, 38244, 25596, 65188, 40036, 15880, 8348,  20356, 63564,
                41256, 5876,  60544, 37260, 21856, 62512, 25640, 44372, 20768,
                64112, 40432, 29872, 61864, 18556, 26256, 54628, 48932, 39912,
                33988, 59548, 43668, 31948, 56488, 28280, 1236,  11920, 20444,
                50884, 39340, 63672, 684,   53536, 59092, 4460,  65388, 9024,
                35408, 20128, 3104,  29444, 8500,  18348, 27052, 47044, 2056,
                60892, 17556, 36064, 48016, 27760, 56960, 34388, 38708, 61880,
                58452, 48108, 21980, 464,   56956, 16632, 7412,  20956, 41104,
                65488, 44844, 27864, 38376, 53548, 9436,  41600, 32552, 56812,
                61972, 35892, 10036, 33216, 52604, 4672,  11476, 38792, 7096,
                45584, 57320, 52420, 29788, 4348,  58448, 24464, 6952,  47388,
                14932, 34616, 4740,  50024, 53224, 27788, 2688,  34444, 60484,
                47728, 36540, 3764,  43944, 23424, 32300, 4872,  37424, 46044,
                5920,  22676, 18680, 36120, 31092, 5300,  44384, 1544,  55716,
                9804,  58024,
            },
            {
                61480, 5728,  33468, 3368,  28552, 51784, 44180, 30572, 2740,
                46352, 23860, 32696, 932,   55332, 10908, 5552,  57724, 2024,
                51856, 12524, 23988, 6144,  58016, 42724, 8716,  60764, 17936,
                21212, 26564, 4424,  51072, 17136, 62524, 48204, 54300, 58152,
                33192, 6056,  18376, 46944, 36096, 14504, 26032, 18808, 49872,
                56716, 12304, 40352, 64368, 36456, 54192, 5820,  58564, 38568,
                50572, 13720, 6584,  40500, 18636, 8284,  968,   50368, 17608,
                25856, 7120,  52316, 32112, 39792, 61368, 30372, 55344, 36932,
                10516, 5076,  58456, 1660,  21912, 17692, 46228, 27500, 11720,
                4440,  24828, 15608, 64940, 40392, 30476, 24612, 63192, 49408,
                20716, 34216, 700,   15664, 41536, 54040, 33088, 38148, 29128,
                55580, 19452, 58084, 13520, 22848, 63416, 10860, 19760, 44728,
                16660, 11972, 59160, 27684, 63588, 50268, 12756, 60848, 16368,
                34432, 50084, 59732, 10136, 39700, 23500, 58960, 13124, 37280,
                22404, 27464,
            },
            {
                17352, 41908, 54984, 48152, 22892, 61932, 35144, 56308, 13116,
                38780, 58560, 51036, 14832, 27044, 47544, 36136, 17948, 28468,
                33540, 56432, 37968, 48164, 32280, 21712, 35604, 5440,  45024,
                52992, 36984, 8992,  13160, 38784, 42260, 5424,  23400, 16024,
                37572, 25872, 60320, 31024, 11012, 51892, 44352, 38392, 1876,
                32368, 46968, 26408, 21944, 14592, 49048, 30436, 23188, 9148,
                32672, 26200, 54268, 59744, 31880, 64412, 21696, 44428, 30012,
                11424, 37576, 64260, 19236, 9816,  50556, 43068, 2480,  15320,
                51840, 24660, 32224, 12716, 49888, 64488, 7020,  37672, 53364,
                18604, 50876, 45740, 1308,  21172, 47748, 56252, 2460,  32052,
                12452, 59972, 24136, 48176, 64172, 20324, 8932,  51244, 62388,
                11128, 39824, 44944, 660,   36508, 43136, 56864, 38992, 50920,
                1808,  30532, 53712, 38760, 17616, 816,   54328, 28600, 41480,
                7804,  26296, 44084, 14736, 55116, 27600, 47128, 19568, 64776,
                3988,  50780,
            },
            {
                45952, 21136, 8832,  14076, 37168, 10924, 4696,  49856, 22184,
                17028, 28508, 9280,  64676, 41728, 31484, 54372, 61512, 42584,
                9568,  45444, 16892, 584,   53700, 14096, 63080, 29312, 2592,
                14856, 64760, 46956, 57904, 20320, 34232, 10524, 29160, 65104,
                3336,  44752, 55628, 22936, 42144, 5656,  57868, 29172, 61764,
                16720, 5520,  51480, 57788, 748,   44300, 62664, 41076, 15904,
                56560, 43592, 3692,  23812, 12596, 46280, 55152, 14508, 41444,
                57172, 3960,  45992, 13680, 35808, 23120, 5588,  62768, 28504,
                18984, 60328, 47596, 42712, 28920, 55692, 34352, 14564, 60768,
                41988, 29936, 38140, 58008, 13276, 8632,  42520, 18968, 54004,
                27948, 43320, 9256,  36468, 5908,  27200, 45552, 16804, 2300,
                23480, 32196, 60496, 25948, 17196, 31688, 6188,  28780, 23944,
                63132, 41812, 20312, 9844,  48004, 35392, 43412, 21020, 10656,
                63084, 53080, 30272, 6256,  63756, 2504,  32852, 52848, 42956,
                29736, 35184,
            },
            {
                800,   63456, 31168, 59056, 43612, 18092, 27248, 60128, 7432,
                47660, 35752, 3976,  45076, 19564, 7092,  13072, 49884, 24524,
                4756,  21292, 65444, 27320, 41352, 10368, 49900, 24768, 56980,
                40924, 22672, 31204, 27576, 2784,  61416, 53260, 47496, 40864,
                13928, 9380,  49704, 2136,  16300, 34144, 20796, 9872,  24456,
                54276, 42572, 9316,  37896, 32840, 11232, 19700, 51292, 2708,
                65304, 20104, 36980, 49336, 5852,  35476, 26872, 3144,  51812,
                33616, 22728, 28404, 54084, 59072, 26644, 48724, 33588, 56676,
                39332, 8176,  35456, 16732, 3408,  40400, 23980, 1420,  48112,
                8836,  22008, 5900,  52912, 27120, 60512, 36224, 15324, 6176,
                38164, 62052, 17220, 55544, 31556, 13052, 57216, 34756, 42588,
                49232, 5328,  53764, 9364,  47936, 52488, 59584, 14476, 8744,
                55120, 33928, 4736,  26264, 56612, 7136,  24284, 57624, 32836,
                47228, 1612,  37816, 17192, 49468, 40936, 10508, 16128, 7296,
                59428, 11988,
            },
            {
                56428, 39424, 26280, 50152, 2532,  54440, 39876, 32396, 42740,
                63260, 53800, 25144, 56992, 38324, 22996, 2792,  39704, 15504,
                34772, 59416, 51172, 7644,  36496, 58764, 17184, 46672, 32900,
                11288, 55072, 6040,  49296, 43396, 24836, 17916, 32208, 56948,
                21436, 35428, 28332, 64128, 39068, 53288, 59964, 45752, 13528,
                35192, 63948, 28272, 17636, 59824, 25628, 4992,  35076, 27292,
                45936, 10152, 29452, 53368, 16196, 62032, 39092, 9588,  60372,
                18484, 62872, 42592, 1688,  8116,  16880, 44468, 11256, 21272,
                45548, 4156,  63788, 52456, 10896, 19816, 61848, 31232, 58552,
                26124, 35048, 63680, 17452, 33456, 50096, 23164, 64392, 46128,
                51836, 3348,  22104, 50120, 40144, 60748, 4564,  21868, 63656,
                27696, 15168, 37404, 65152, 21796, 3524,  45616, 18828, 37060,
                48808, 12724, 45180, 64764, 31528, 14156, 61984, 3936,  39336,
                13156, 20016, 58896, 24168, 34552, 21416, 56892, 26616, 38804,
                48492, 24216,
            },
            {
                15188, 53000, 19012, 6836,  34392, 62800, 24372, 14576, 1684,
                20032, 11492, 32876, 15784, 60948, 51392, 29612, 63336, 55620,
                47248, 30528, 13148, 43716, 23576, 31408, 1664,  39052, 7520,
                62212, 18904, 36152, 15616, 59004, 8764,  38096, 392,   6420,
                51448, 60952, 18692, 47200, 25176, 7932,  30112, 200,   40188,
                48608, 22492, 2472,  55564, 46064, 39672, 61720, 54880, 13024,
                38444, 60276, 1852,  41208, 57620, 19528, 31180, 23608, 46828,
                5580,  15192, 49948, 32324, 39664, 64680, 37088, 848,   53348,
                13956, 26368, 30744, 22512, 57188, 44328, 50428, 5304,  16552,
                45520, 55416, 12144, 43736, 4652,  29724, 376,   40816, 25680,
                11944, 33600, 29132, 44496, 2204,  25292, 52640, 38464, 9976,
                55036, 19104, 41672, 29660, 12608, 34640, 40724, 62256, 25508,
                288,   59952, 21888, 16820, 40468, 50396, 45768, 18144, 51640,
                25732, 55720, 42104, 8324,  61068, 45556, 4416,  63044, 20524,
                32192, 5652,
            },
            {
                35796, 29352, 45336, 12604, 21720, 47164, 9224,  51336, 57688,
                30664, 40328, 49072, 284,   10040, 34512, 45544, 7788,  25896,
                1712,  37832, 18792, 3700,  54656, 63688, 20560, 52708, 26796,
                44172, 50664, 1332,  40136, 52304, 12348, 46176, 63364, 26672,
                43568, 30872, 11684, 4684,  57052, 14604, 49832, 63048, 18924,
                6660,  31492, 12448, 49788, 21320, 15064, 8128,  30816, 48676,
                17436, 21804, 32956, 25516, 11512, 44156, 28,    55808, 40660,
                27136, 36428, 10728, 56788, 20004, 51600, 24604, 29504, 61412,
                42060, 58632, 48612, 38468, 6880,  33376, 27276, 37408, 10356,
                40244, 2280,  20728, 48256, 39192, 10012, 59040, 19672, 55948,
                7648,  58224, 14676, 65412, 9536,  18428, 47232, 14224, 31384,
                59624, 7332,  46156, 2108,  50532, 26760, 56588, 5276,  32376,
                43020, 29340, 51844, 37560, 2316,  27708, 9184,  36476, 29740,
                65408, 4780,  32064, 11764, 28128, 50352, 36800, 13212, 44360,
                52120, 61032,
            },
            {
                2156,  10204, 64120, 41856, 57076, 30080, 17168, 37644, 45908,
                4752,  23688, 59128, 42628, 26624, 53908, 17288, 20684, 42340,
                11096, 62640, 57348, 28984, 46200, 14892, 4908,  59932, 35376,
                13852, 29824, 23528, 64036, 19648, 28796, 34792, 22360, 15100,
                8440,  54668, 41180, 36568, 21952, 32488, 42996, 26628, 37148,
                56412, 61204, 41840, 34560, 3636,  53024, 43352, 22920, 58928,
                5216,  52516, 63276, 47676, 6944,  51612, 64248, 34356, 12992,
                48628, 59344, 22116, 29992, 3028,  12944, 56000, 6292,  33220,
                18256, 10008, 1880,  15952, 62620, 13176, 54800, 18720, 63032,
                51552, 32516, 57500, 25452, 61468, 54288, 13988, 35492, 31084,
                48296, 41980, 37684, 20780, 54408, 35172, 28596, 43344, 20,
                48720, 25072, 33432, 62056, 54320, 16840, 10500, 21160, 52900,
                14984, 57308, 10724, 5576,  60812, 54572, 22884, 58624, 1016,
                16064, 48392, 43852, 54204, 15644, 16,    30804, 55032, 9220,
                17820, 40892,
            },
            {
                47572, 55728, 25048, 36824, 448,   60308, 6132,  26716, 65460,
                13280, 51992, 18016, 6360,  36936, 64552, 12760, 58500, 32480,
                52616, 22852, 48736, 8576,  39604, 33084, 42524, 24496, 9604,
                57736, 4224,  45728, 33332, 6840,  55796, 60304, 48252, 39436,
                58716, 17672, 1868,  45688, 60228, 51708, 3520,  10692, 16104,
                52512, 24928, 8684,  29468, 64888, 37064, 27928, 508,   33984,
                40304, 8800,  43180, 14448, 36664, 28760, 21688, 16984, 7984,
                54252, 4276,  44036, 61504, 47624, 40972, 16212, 45832, 50292,
                23468, 36240, 53952, 46432, 25152, 42220, 24,    48488, 23304,
                28288, 13900, 7708,  36832, 16456, 22652, 42888, 51148, 3752,
                17524, 60476, 1260,  26480, 49312, 57792, 7040,  63028, 21312,
                36488, 56852, 13268, 22576, 8008,  37580, 44892, 64420, 39048,
                7704,  24064, 35336, 48224, 19548, 32024, 44476, 11992, 40728,
                34280, 21016, 7264,  37664, 64024, 18796, 24952, 59156, 3216,
                27952, 22508,
            },
            {
                32504, 16284, 4912,  51964, 14012, 33324, 54116, 43684, 20912,
                35436, 28852, 61180, 47604, 21700, 30616, 4628,  49364, 39932,
                6276,  35488, 16652, 26056, 61044, 11888, 51580, 48180, 17656,
                38732, 53292, 60852, 10540, 25608, 42088, 2584,  5364,  31508,
                50192, 25080, 63828, 28648, 12832, 19720, 35536, 58844, 46316,
                4288,  43844, 19840, 58700, 16948, 10840, 48104, 62140, 12520,
                57044, 24484, 30480, 19064, 59500, 3652,  49400, 60820, 30288,
                37784, 25000, 17900, 9452,  35816, 27368, 63124, 8848,  37980,
                4840,  65140, 28484, 32484, 20616, 60332, 9096,  35000, 58264,
                3564,  44260, 65236, 30376, 1588,  45932, 6352,  63988, 28180,
                23880, 10592, 46172, 31836, 5000,  16232, 39652, 11516, 51800,
                16680, 5428,  40004, 43552, 31348, 59856, 28284, 2868,  49804,
                18124, 45792, 62644, 26772, 42252, 15416, 6288,  49316, 61584,
                54980, 27312, 59716, 23356, 51928, 41344, 47392, 33284, 43268,
                65176, 50952,
            },
            {
                38876, 61328, 19964, 28264, 48836, 22796, 38596, 11028, 3332,
                55096, 8272,  39272, 11264, 56584, 2404,  44648, 27064, 60912,
                14364, 644,   44132, 53696, 19736, 6064,  28516, 64212, 120,
                32128, 21048, 15096, 48952, 37384, 18056, 13864, 54056, 20388,
                11100, 34548, 6280,  53036, 39128, 8568,  65220, 29244, 22572,
                32612, 13676, 38296, 50764, 5696,  25384, 54624, 20008, 45244,
                16144, 49828, 1560,  55756, 39232, 25996, 9892,  41548, 45828,
                2324,  65464, 32948, 53316, 5128,  22740, 31956, 59132, 20436,
                41492, 56396, 11064, 4172,  51272, 39288, 29200, 46232, 16004,
                40900, 54384, 19304, 48728, 52732, 26532, 33148, 12236, 39984,
                53780, 36192, 63452, 13420, 42516, 61524, 23556, 33004, 45164,
                26832, 64324, 50236, 18624, 1116,  47900, 23364, 12660, 33500,
                58584, 30204, 1072,  13004, 55508, 64256, 37768, 25156, 18372,
                9628,  45864, 13480, 2428,  34900, 6148,  10800, 21336, 37048,
                14116, 7340,
            },
            {
                11564, 44152, 56820, 40548, 7500,  63736, 18404, 50032, 61660,
                32036, 16564, 44176, 25392, 33280, 51204, 37680, 16432, 23844,
                56316, 31760, 64916, 38320, 2948,  58380, 36764, 22432, 41048,
                55896, 5412,  43516, 29116, 52004, 65468, 45356, 27780, 37840,
                61708, 42720, 22620, 15732, 48156, 25688, 1656,  40744, 54480,
                48768, 63228, 976,   56516, 45960, 40620, 32712, 2912,  38184,
                27112, 35708, 64404, 11976, 45360, 52692, 33828, 14812, 57180,
                20372, 49920, 14204, 42904, 58244, 48716, 324,   12732, 51192,
                26404, 14760, 43648, 18068, 57920, 6476,  12720, 63392, 21204,
                5380,  34148, 11748, 38728, 9428,  62172, 20516, 58108, 49816,
                7952,  18956, 56192, 22056, 52232, 29068, 1628,  55012, 59868,
                3204,  29732, 10824, 58220, 54872, 15508, 36208, 61452, 42672,
                5152,  53252, 40436, 22352, 8772,  33904, 2580,  52788, 30876,
                5192,  39364, 50064, 29840, 56204, 15104, 62764, 57876, 4508,
                53860, 25808,
            },
            {
                30836, 1424,  35108, 10052, 45944, 30964, 1800,  41204, 23960,
                46440, 58720, 1500,  63936, 13944, 19404, 62452, 6816,  47400,
                10580, 50736, 27532, 22012, 47000, 33620, 15988, 45868, 12460,
                27072, 63068, 35744, 8416,  1108,  22684, 33728, 56644, 8604,
                46916, 560,   57776, 30648, 61212, 43952, 56240, 17880, 6636,
                10944, 26812, 35280, 30680, 22436, 14192, 63952, 7836,  51356,
                60644, 4804,  32172, 22452, 7112,  17812, 63536, 23656, 6052,
                28348, 40132, 11200, 26224, 37584, 16872, 55248, 44592, 30316,
                3132,  61464, 33968, 48272, 36808, 24532, 54888, 31940, 51944,
                26840, 60944, 24088, 59020, 3872,  35868, 42056, 16356, 784,
                30600, 44980, 3272,  33812, 7472,  47912, 37908, 19420, 9136,
                41916, 35272, 20760, 38592, 26184, 4520,  51260, 9580,  20228,
                25852, 15796, 36308, 50592, 58860, 28540, 46848, 21520, 57672,
                63824, 33528, 17568, 61288, 44316, 26268, 49096, 29176, 17700,
                46108, 59448,
            },
            {
                50208, 23616, 63108, 17536, 26036, 59276, 54844, 14940, 27908,
                7072,  36736, 21960, 53152, 40256, 9204,  54868, 29852, 41804,
                35940, 17992, 7452,  13140, 55468, 9656,  52580, 6676,  59636,
                50604, 17920, 24144, 54824, 60528, 12032, 41032, 3940,  16780,
                26488, 51288, 11644, 36816, 4248,  33048, 14284, 50724, 37604,
                21316, 60132, 42548, 9548,  18152, 59052, 28556, 43968, 23524,
                10564, 19316, 47828, 53756, 42752, 30036, 860,   48504, 36532,
                53644, 59852, 1512,  62940, 21224, 7852,  34688, 63964, 18512,
                39468, 53088, 22532, 1092,  64164, 15444, 44604, 2228,  38012,
                8628,  42544, 17036, 45212, 28576, 13524, 54160, 47420, 24668,
                38416, 59812, 26696, 41260, 17464, 58160, 12976, 25840, 49348,
                15036, 53276, 46300, 7380,  33044, 63568, 40180, 29392, 47292,
                56924, 65252, 11040, 44792, 4264,  17064, 40076, 13708, 43484,
                512,   24100, 11312, 40684, 7888,  22448, 38092, 808,   41020,
                34308, 20824,
            },
            {
                15216, 4292,  47848, 52944, 13232, 37304, 4640,  34364, 52140,
                12004, 48516, 30840, 6116,  27252, 43928, 34532, 21604, 4200,
                57916, 61960, 43308, 39456, 60608, 29220, 25076, 42280, 31520,
                2448,  10928, 39720, 46772, 30912, 19748, 50228, 59360, 32380,
                63644, 39772, 18164, 54092, 20892, 9396,  24108, 58472, 29896,
                45452, 15740, 3404,  55136, 49224, 4648,  34524, 12920, 56180,
                41420, 58524, 36868, 15284, 62016, 38364, 58920, 12564, 44516,
                9128,  18788, 31080, 51988, 46656, 28604, 41992, 10808, 24084,
                6832,  46572, 9528,  30984, 27784, 41136, 20012, 11324, 48980,
                56100, 30672, 516,   50816, 56680, 22440, 4864,  31920, 64988,
                11136, 14652, 50104, 63144, 10248, 35132, 43632, 65052, 31936,
                5088,  61552, 24260, 59120, 13536, 44416, 17320, 54012, 6848,
                32808, 2620,  19252, 31236, 24188, 62744, 49960, 9960,  36524,
                27820, 54876, 51396, 20340, 2908,  32828, 55408, 51668, 10576,
                6572,  61828,
            },
            {
                54616, 38488, 28836, 41568, 6692,  21828, 43768, 57760, 19780,
                63516, 42292, 17348, 57168, 50116, 708,   59324, 12472, 52336,
                25876, 2252,  30924, 49892, 19376, 1024,  64492, 37704, 20632,
                48568, 34152, 57664, 14964, 3232,  37020, 25404, 44000, 14024,
                22320, 6988,  29388, 45232, 64244, 40440, 47612, 35028, 1244,
                64620, 51568, 38888, 24692, 62832, 46532, 39692, 52824, 17116,
                30740, 1896,  27968, 5308,  9780,  26664, 19788, 32348, 55888,
                25600, 34776, 43756, 6500,  14588, 57328, 3744,  49508, 54500,
                36708, 59604, 13264, 55588, 50144, 7752,  58180, 34904, 62196,
                22668, 13880, 64440, 33052, 18928, 40084, 58832, 8776,  43384,
                19876, 54972, 29828, 4220,  24272, 52932, 6252,  21448, 56404,
                39416, 17960, 2184,  48788, 28352, 21728, 276,   60552, 23728,
                37292, 42420, 52484, 59904, 38072, 6680,  55208, 19488, 60728,
                47244, 15420, 34628, 58292, 64732, 45316, 13824, 19228, 57408,
                26440, 42708,
            },
            {
                32984, 9188,  19144, 60200, 32320, 64360, 47508, 29524, 9016,
                25828, 2972,  60832, 33872, 14752, 23796, 65120, 18288, 39020,
                46032, 15764, 23632, 5980,  35584, 44304, 17132, 13624, 54164,
                62588, 27912, 6980,  42764, 64032, 53076, 5480,  9940,  54696,
                35496, 49228, 57456, 3308,  13724, 28032, 60360, 5568,  12396,
                26072, 19456, 7916,  32256, 11732, 20840, 144,   25736, 7140,
                49504, 65080, 44912, 23412, 55456, 51428, 40452, 3044,  61540,
                15108, 4064,  39304, 60316, 24404, 32892, 61888, 16020, 27616,
                32468, 20328, 42808, 17020, 60808, 4044,  46320, 25716, 16572,
                4820,  47112, 37008, 6648,  10796, 49108, 35004, 27212, 51628,
                36500, 2232,  21932, 45920, 39688, 60076, 30512, 476,   45456,
                27176, 11932, 34712, 41776, 9200,  57024, 35324, 11708, 46040,
                14848, 8548,  27492, 47808, 12728, 33584, 26100, 3192,  30580,
                7420,  41896, 4396,  27180, 48188, 9120,  30660, 62980, 36672,
                47368, 12588,
            },
            {
                65056, 24408, 56672, 2628,  50492, 11188, 16316, 332,   36928,
                55656, 39556, 21400, 10032, 47028, 37112, 29472, 48620, 9500,
                33200, 63816, 57252, 53600, 11212, 47380, 56520, 4768,  24380,
                9048,  45276, 19232, 23260, 29672, 48352, 17292, 28200, 62120,
                1496,  42080, 25132, 33940, 51188, 7972,  17800, 54380, 41972,
                57256, 34276, 44680, 53856, 28316, 37284, 56804, 61392, 33400,
                20148, 38884, 13352, 34320, 47032, 16576, 7164,  22988, 49656,
                45692, 64912, 21584, 48564, 11784, 37952, 19892, 41200, 912,
                64556, 5880,  26128, 37388, 33328, 22188, 39384, 30268, 52340,
                40640, 27400, 54596, 43152, 25272, 63112, 1328,  17500, 61548,
                6912,  47904, 57372, 33592, 12636, 18976, 49680, 16060, 36460,
                62648, 51040, 55484, 30656, 64716, 51652, 39224, 26336, 49932,
                63752, 56416, 22004, 788,   16404, 45136, 40812, 64924, 52016,
                23144, 59432, 10832, 38908, 16724, 24680, 41772, 3492,  22764,
                29488, 1292,
            },
            {
                43932, 35956, 15628, 45588, 27428, 23452, 40588, 61064, 51340,
                13424, 45840, 30296, 53012, 5720,  41488, 2856,  55996, 6696,
                27736, 13012, 41700, 20552, 32328, 26916, 61684, 30384, 40800,
                35148, 52272, 736,   60968, 11608, 33516, 58020, 38556, 45368,
                19928, 10608, 16028, 62556, 21332, 43432, 38224, 23048, 30604,
                48584, 14724, 3980,  59260, 16616, 6184,  42488, 14444, 45652,
                9112,  53216, 4168,  60208, 29540, 57468, 42948, 35284, 27852,
                9912,  17728, 29772, 54660, 1648,  52172, 8872,  56472, 47428,
                11896, 51508, 45000, 2488,  53688, 9792,  65420, 12712, 1772,
                59308, 9156,  20668, 60292, 14880, 30232, 45504, 12800, 24040,
                31288, 41184, 15840, 62108, 7920,  28300, 42300, 54436, 10144,
                6968,  22836, 19040, 3624,  15780, 6268,  19784, 31652, 2848,
                18220, 39712, 30464, 62492, 53584, 57940, 21268, 12024, 35424,
                44600, 18656, 31924, 50304, 56636, 35208, 6172,  53352, 58372,
                16508, 50052,
            },
            {
                59284, 5756,  52784, 39268, 8336,  34564, 53716, 18492, 32804,
                23996, 4748,  63252, 16596, 58588, 26316, 20772, 62040, 44012,
                51672, 37744, 1436,  60036, 8524,  38644, 3072,  50216, 12392,
                58356, 16584, 37752, 55632, 41280, 7600,  21548, 3964,  50628,
                31336, 60160, 37772, 46816, 228,   31588, 52276, 63488, 9884,
                2076,  61252, 40964, 23464, 48076, 63992, 30352, 51268, 23076,
                58040, 26352, 41952, 18844, 10648, 492,   63664, 13220, 53184,
                37740, 56916, 7292,  42500, 35532, 27220, 44656, 22228, 30584,
                39212, 57828, 23228, 62580, 14716, 48868, 19432, 55940, 44784,
                34624, 17632, 32032, 51516, 3608,  41392, 56408, 53056, 38328,
                59424, 10644, 26340, 52500, 37824, 4732,  63924, 24860, 32928,
                59476, 43096, 47620, 37660, 25320, 45720, 61020, 42716, 55212,
                5068,  34720, 11072, 42172, 5712,  28240, 8676,  49248, 1104,
                14432, 54044, 63704, 2292,  20984, 61172, 14956, 44856, 38028,
                8048,  21064,
            },
            {
                26684, 12276, 30728, 19932, 62336, 57692, 4052,  43040, 7604,
                49200, 27992, 35884, 43424, 12124, 49664, 31820, 15412, 35388,
                22856, 18440, 48008, 25248, 52388, 15024, 22484, 43640, 19808,
                5904,  47448, 31620, 25328, 14816, 46864, 64960, 25972, 12996,
                55824, 5604,  27636, 53256, 12812, 58792, 4832,  15668, 36716,
                20816, 27288, 51932, 8896,  35084, 19664, 3596,  11076, 36760,
                1740,  31696, 63240, 37720, 22524, 50408, 31476, 19948, 59296,
                2412,  32716, 25036, 13932, 58580, 17156, 63304, 5040,  14400,
                34464, 18280, 8696,  29656, 41640, 26800, 32636, 6324,  24132,
                49536, 62128, 5864,  46528, 23508, 36344, 8504,  21056, 4656,
                50200, 18648, 952,   44760, 21188, 56136, 47252, 13896, 39892,
                1516,  29072, 12908, 62080, 33180, 53632, 14036, 10476, 28656,
                59436, 48116, 24048, 50848, 36516, 18116, 32816, 39308, 60516,
                29332, 25436, 37144, 8588,  43416, 26964, 48484, 11580, 31996,
                62160, 40888,
            },
            {
                34096, 54812, 49304, 1200,  14548, 46436, 30128, 12152, 65320,
                20520, 59392, 1720,  55892, 19260, 39096, 60268, 10244, 54588,
                3748,  63224, 5736,  34360, 44704, 65456, 55704, 32784, 60116,
                28724, 63660, 10324, 51216, 2368,  36280, 53960, 42900, 35064,
                16988, 40656, 22364, 8436,  35848, 25704, 49088, 29012, 46372,
                55848, 39088, 31828, 13008, 44252, 55288, 39420, 27836, 62328,
                17760, 47668, 14684, 54924, 7668,  44964, 39260, 26104, 5884,
                41140, 46136, 50528, 61796, 3672,  31320, 48244, 25212, 60168,
                49756, 3052,  54520, 36016, 46412, 372,   58648, 38064, 42648,
                13984, 28784, 39180, 12052, 58388, 16976, 28204, 64548, 33104,
                42560, 29080, 63528, 35196, 12136, 31868, 3208,  19900, 52160,
                16684, 49376, 56776, 9384,  41060, 900,   22164, 36268, 51360,
                7624,  20200, 15068, 60388, 1836,  46708, 62988, 23868, 55988,
                41956, 4148,  47316, 13324, 30796, 55020, 12,    23476, 51100,
                18000, 2528,
            },
            {
                44320, 23196, 63768, 41820, 36492, 21612, 25900, 56004, 38008,
                15340, 40960, 9368,  24856, 33636, 7312,  8,     46756, 25580,
                42424, 30552, 57156, 12012, 28392, 18100, 388,   9812,  39600,
                23736, 3736,  44364, 20856, 59572, 18284, 30184, 9832,  3140,
                63060, 48752, 57840, 44768, 65348, 18764, 41236, 11336, 61872,
                7400,  17396, 64864, 4936,  25696, 60736, 15848, 49180, 56112,
                43304, 6260,  51920, 25120, 33604, 61296, 11628, 48080, 15468,
                64388, 18252, 10480, 21824, 39632, 8248,  53628, 37732, 10636,
                27676, 40368, 58976, 20624, 12160, 52836, 17100, 63820, 10280,
                21040, 55636, 1040,  63424, 33688, 49328, 2276,  46360, 15076,
                57220, 9008,  47708, 23828, 59668, 41836, 27496, 58504, 34832,
                65476, 23984, 4620,  20452, 27688, 58808, 47212, 63864, 25084,
                40644, 44932, 31932, 55780, 26604, 13404, 52872, 6216,  16292,
                10436, 21740, 52056, 62152, 39816, 18936, 34732, 64792, 27816,
                56336, 10736,
            },
            {
                59696, 15932, 4876,  28220, 10536, 60520, 51912, 2192,  33920,
                47644, 53796, 30668, 51180, 45372, 64744, 52780, 28572, 13240,
                58984, 16468, 38208, 51236, 6764,  41312, 35576, 49244, 14664,
                53084, 34956, 56712, 27528, 39252, 6356,  61080, 23876, 52020,
                20204, 29224, 1792,  14732, 32584, 54524, 3092,  23652, 33252,
                42908, 1148,  48692, 21808, 53016, 32880, 652,   8072,  21376,
                34864, 12104, 28964, 4036,  17200, 55952, 1928,  30168, 52060,
                23640, 35664, 29428, 55740, 43860, 33580, 19828, 724,   44340,
                64796, 15492, 5772,  24932, 61456, 7908,  30444, 50488, 3288,
                36660, 52364, 25044, 43428, 19688, 53812, 39976, 11596, 23252,
                36632, 2840,  55008, 15404, 6400,  51176, 10304, 44896, 7824,
                30136, 38920, 45324, 35668, 50716, 7100,  31056, 17308, 2008,
                12776, 62164, 4492,  38284, 9864,  43940, 20644, 30968, 36304,
                44800, 58096, 33808, 16792, 4812,  59580, 9612,  41612, 4092,
                37184, 47424,
            },
            {
                7768,  38668, 32724, 57312, 48452, 7320,  17780, 43780, 5496,
                23152, 11436, 62476, 4500,  16836, 22616, 37100, 19636, 40384,
                49260, 8652,  20904, 46132, 23504, 54448, 61532, 26220, 46468,
                7504,  17768, 41748, 12156, 49432, 33288, 46420, 13808, 44164,
                38684, 34176, 11080, 24776, 39312, 6444,  50552, 59152, 20048,
                53832, 26748, 57584, 36944, 10588, 45760, 41128, 24824, 31052,
                60424, 39740, 46520, 65484, 41760, 36588, 20484, 58064, 43216,
                8344,  60364, 1288,  48844, 15272, 63668, 12500, 57144, 22828,
                31044, 47080, 51444, 33028, 37596, 44040, 21880, 26204, 57388,
                31716, 45776, 15376, 9684,  30980, 5416,  61704, 26356, 60056,
                52480, 20084, 39036, 30004, 62704, 18236, 36968, 22808, 54940,
                2440,  12072, 62540, 17688, 55608, 14660, 43688, 38724, 54556,
                34268, 52044, 29260, 18612, 65368, 40836, 59204, 3592,  48528,
                64352, 26412, 7492,  28444, 49460, 22356, 45184, 53104, 15328,
                30336, 21544,
            },
            {
                26344, 53472, 18996, 45152, 24140, 39580, 31916, 62916, 26868,
                58348, 19112, 36292, 27604, 42644, 12280, 56660, 31560, 5432,
                62596, 33976, 1252,  63968, 31636, 13904, 4380,  20400, 57472,
                64528, 31340, 1912,  62380, 16204, 54852, 472,   26932, 58336,
                7612,  54992, 61264, 47440, 57268, 16876, 30428, 44520, 35512,
                14472, 8756,  40340, 29632, 18940, 58520, 13684, 64272, 52552,
                18396, 2284,  58164, 22736, 9036,  48892, 26844, 5116,  33056,
                13608, 53384, 37944, 6812,  24460, 28256, 41068, 50812, 35548,
                8076,  18872, 42112, 3948,  56124, 14980, 47872, 40944, 5508,
                18132, 60124, 7052,  27796, 56484, 37436, 16636, 43712, 6540,
                32248, 13068, 45272, 25116, 40760, 452,   49080, 60664, 15480,
                42728, 52756, 25944, 32312, 10196, 60684, 3412,  20920, 27436,
                57832, 8396,  22704, 49808, 2660,  33428, 24748, 51528, 12868,
                19336, 728,   41088, 56284, 36220, 12204, 25560, 58320, 6532,
                50320, 61916,
            },
            {
                42152, 3352,  65004, 13020, 544,   55548, 14904, 49980, 8608,
                40448, 46696, 1076,  59836, 50420, 8520,  60784, 2516,  44924,
                23948, 52676, 26896, 56028, 9356,  38868, 44636, 29304, 11396,
                38044, 22860, 51536, 25396, 8840,  37380, 21420, 64704, 31388,
                16384, 4184,  19084, 42236, 27980, 9444,  63984, 12416, 2372,
                61056, 47328, 16152, 62444, 2736,  50544, 35480, 4344,  47204,
                9984,  27116, 35780, 13872, 31140, 52896, 16012, 62600, 46872,
                40388, 27168, 20024, 57696, 45388, 4228,  61136, 16608, 2796,
                55112, 61928, 13064, 27504, 64152, 1492,  35352, 11004, 62788,
                49192, 34332, 40464, 64768, 47168, 22108, 108,   50868, 28728,
                48756, 64936, 3680,  53092, 57380, 32004, 12656, 27124, 33396,
                19344, 58736, 5684,  47052, 37516, 23808, 48900, 64780, 11112,
                41656, 16612, 47096, 36180, 54772, 15496, 28560, 8224,  38248,
                54636, 31744, 14776, 46616, 62820, 2060,  31248, 39060, 19960,
                34336, 13504,
            },
            {
                36544, 9596,  49716, 30416, 35232, 60992, 21068, 36996, 29512,
                13500, 56312, 31732, 15456, 39488, 24448, 34800, 54536, 18424,
                11228, 37432, 15252, 42232, 19008, 50276, 60456, 1576,  33524,
                42792, 5372,  46216, 59332, 30404, 43392, 48860, 10988, 41092,
                50804, 36368, 23008, 52804, 616,   37484, 21552, 48956, 38896,
                28360, 23780, 52296, 6688,  32184, 23212, 27588, 42952, 15788,
                57012, 20572, 54048, 44700, 240,   59796, 38360, 10396, 22052,
                2680,  63896, 16800, 34180, 51544, 11196, 32144, 47988, 25820,
                38068, 29860, 9752,  23032, 52432, 19296, 59212, 28696, 14280,
                24176, 1296,  20300, 12644, 51804, 33884, 10788, 58484, 41232,
                8636,  17224, 34556, 10104, 20648, 5212,  46296, 64424, 6800,
                40192, 48740, 22172, 672,   42140, 28708, 53300, 32688, 4612,
                45208, 25824, 61380, 6336,  11620, 57524, 45780, 60368, 22496,
                43172, 61240, 9496,  20832, 52208, 43492, 16648, 64264, 46192,
                916,   57452,
            },
            {
                52348, 21916, 43520, 26700, 53840, 10900, 4536,  44544, 64684,
                2968,  51424, 22120, 45120, 5892,  29668, 47980, 14692, 40756,
                61348, 30000, 47452, 5600,  57756, 34844, 24584, 17256, 52400,
                56072, 14852, 19440, 35552, 2756,  57072, 18316, 6264,  24852,
                45940, 63292, 13028, 33240, 61744, 43628, 25744, 56392, 53128,
                4720,  34404, 41968, 45304, 56084, 9416,  63036, 38672, 6972,
                33532, 49712, 40672, 62284, 7368,  19352, 42880, 25460, 56628,
                49356, 31036, 8540,  42064, 59872, 21600, 39240, 7016,  19976,
                43536, 59576, 49124, 39852, 45296, 31984, 8000,  50976, 43920,
                54612, 38000, 59024, 42844, 4984,  25348, 63220, 18932, 35792,
                22832, 56460, 27040, 37792, 59768, 41588, 23668, 52460, 36676,
                9916,  30344, 63636, 16568, 56832, 7392,  18496, 13996, 36792,
                59588, 880,   31600, 39808, 21208, 42044, 1448,  17368, 34996,
                3916,  50676, 24444, 34380, 4316,  27620, 8104,  54840, 11000,
                24236, 29228,
            },
            {
                63244, 14636, 5780,  40112, 16864, 46964, 23716, 52548, 18096,
                25776, 34680, 9488,  62796, 53968, 20192, 64508, 26416, 7168,
                50440, 58576, 3024,  22168, 28732, 12428, 7944,  62736, 47692,
                27256, 9672,  63384, 40648, 12864, 27860, 61816, 34748, 54516,
                1584,  29440, 9160,  48624, 5316,  15348, 32088, 7672,  18004,
                11236, 64976, 20160, 13336, 37080, 17668, 48096, 21576, 59420,
                30160, 2824,  11880, 23912, 29140, 33972, 51184, 4144,  36036,
                12664, 44812, 54416, 628,   14584, 29288, 56276, 65144, 51884,
                14212, 72,    34568, 5952,  15408, 55532, 37448, 21996, 4340,
                30792, 9808,  26584, 16772, 31952, 53776, 39296, 14440, 46484,
                2764,  62292, 48056, 13760, 50760, 29092, 16272, 1456,  56512,
                14164, 26744, 51280, 34888, 12456, 46016, 62296, 40840, 51908,
                23360, 49000, 14372, 63880, 27056, 52916, 30900, 62572, 48360,
                11444, 29084, 40052, 58644, 18480, 37808, 60788, 32920, 40908,
                48656, 17312,
            },
            {
                45108, 32356, 56280, 62348, 1964,  59272, 33212, 40668, 6460,
                60232, 42512, 16620, 37476, 3420,  11636, 43016, 804,   35852,
                19684, 10088, 33012, 65160, 53864, 45620, 36748, 40992, 4060,
                21656, 32644, 49004, 54352, 23644, 4404,  51468, 39280, 14120,
                20152, 59688, 40524, 26764, 57844, 20344, 45460, 60380, 40880,
                30140, 57808, 104,   25940, 60928, 54284, 2016,  12692, 44188,
                25296, 64532, 17268, 38048, 57572, 45712, 14380, 64900, 18620,
                28652, 61536, 23024, 37132, 26112, 46992, 3476,  9668,  33100,
                23904, 57192, 18200, 63360, 25552, 2328,  42284, 65292, 17276,
                46576, 62240, 55912, 48968, 2408,  45408, 7384,  28108, 55296,
                30936, 6328,  43460, 19208, 7700,  62920, 34396, 43024, 21256,
                60572, 44804, 3160,  39160, 60044, 25724, 30724, 6140,  20144,
                9996,  56052, 35288, 18476, 50284, 9252,  37640, 5660,  26076,
                55844, 16124, 65148, 7236,  48580, 53208, 14112, 21232, 2712,
                60000, 4712,
            },
            {
                38536, 8404,  24992, 19904, 37240, 27900, 8912,  56972, 30096,
                12216, 55080, 49524, 27372, 32996, 58868, 52220, 30484, 56700,
                45912, 24512, 43440, 39300, 16096, 204,   59684, 30196, 13672,
                57124, 38748, 6920,  29496, 44016, 16516, 46740, 30808, 8600,
                56260, 44584, 16244, 37200, 53664, 11700, 50872, 2984,  36056,
                22816, 48324, 51436, 31316, 5004,  40252, 28132, 50684, 35100,
                55652, 5072,  46996, 52204, 9788,  3084,  26888, 53212, 7480,
                39616, 4600,  50576, 10668, 60704, 40680, 17912, 36084, 45432,
                27892, 53416, 11268, 46904, 30500, 60568, 12148, 27568, 52768,
                34056, 13700, 6948,  35992, 20536, 57980, 23840, 60700, 12108,
                51412, 39980, 24820, 32728, 2240,  55264, 11016, 48596, 31456,
                4704,  53828, 18736, 8232,  21832, 54976, 1640,  34580, 58460,
                27640, 43208, 7436,  2832,  44716, 23740, 13868, 58404, 20380,
                44112, 33256, 2596,  42876, 11832, 25520, 44608, 29972, 51324,
                36004, 27068,
            },
            {
                59036, 52704, 47840, 11808, 50832, 44160, 14808, 48828, 20132,
                38600, 352,   23676, 60924, 46808, 14324, 22236, 38480, 16464,
                62028, 13316, 52584, 6808,  27132, 49780, 19384, 23784, 44580,
                64668, 2104,  18360, 61192, 10448, 57640, 21748, 64320, 26292,
                33956, 3972,  23384, 65224, 1940,  34300, 24792, 63248, 14032,
                43264, 8712,  16180, 44324, 34264, 10632, 18896, 61920, 7456,
                15060, 42136, 20112, 31728, 60652, 22464, 40580, 47208, 32580,
                58748, 15264, 43660, 19844, 32308, 54808, 49028, 12844, 62504,
                6296,  41808, 37860, 21636, 50740, 35540, 19804, 8928,  39440,
                3456,  22468, 41024, 28644, 63980, 10112, 37372, 42392, 792,
                21368, 16252, 54304, 60872, 45124, 38844, 26332, 17964, 64168,
                36940, 24712, 41832, 32980, 49668, 10884, 43796, 48000, 63540,
                16676, 38812, 30088, 59976, 55268, 33360, 64452, 40356, 47148,
                8476,  53680, 22944, 30568, 57592, 35316, 5948,  63916, 9660,
                18760, 12924,
            },
            {
                22428, 68,    34112, 29932, 4324,  64296, 22640, 2728,  61968,
                32372, 45492, 8256,  18852, 5160,  40844, 9080,  48972, 4688,
                28140, 2220,  35220, 20800, 62612, 10744, 55444, 34184, 8812,
                52632, 25796, 35812, 50824, 33312, 336,   37532, 5836,  43108,
                49252, 52264, 31568, 8036,  47012, 39920, 29552, 6156,  54452,
                27520, 38524, 63692, 22048, 52792, 58964, 46124, 30856, 23012,
                37228, 53676, 27880, 1376,  35776, 16308, 62884, 10964, 20748,
                54996, 25220, 34952, 64456, 2092,  7156,  23552, 57536, 20364,
                1904,  31576, 59996, 8380,  4236,  43336, 57448, 48884, 59236,
                25188, 60900, 50956, 44044, 15072, 31664, 17748, 49696, 33264,
                65264, 36692, 9512,  28532, 13208, 21628, 58000, 5812,  9152,
                46692, 12744, 58112, 28776, 65300, 14960, 38040, 23572, 4692,
                12960, 51596, 47332, 20888, 11756, 3820,  28312, 17712, 1168,
                36264, 12856, 62992, 38836, 19468, 704,   54736, 23236, 47576,
                57284, 43060,
            },
            {
                65268, 15964, 41084, 60432, 18292, 53332, 35156, 42840, 26460,
                52652, 15652, 64800, 28844, 35676, 55424, 25464, 63852, 32404,
                53612, 41376, 47880, 57376, 31292, 37712, 46472, 5248,  42040,
                15996, 48012, 12064, 41712, 24616, 53516, 13648, 59320, 17704,
                11180, 62192, 19248, 13552, 56420, 21640, 17084, 49568, 59100,
                19516, 2072,  57232, 7128,  13836, 24900, 41576, 768,   56936,
                10064, 63344, 12904, 48708, 43064, 55460, 5156,  30328, 36812,
                948,   48036, 8964,  28028, 52096, 16088, 30560, 43132, 38420,
                51108, 15000, 24352, 55144, 17044, 28980, 13948, 1820,  31344,
                45052, 17968, 11172, 1272,  56880, 52832, 3924,  45704, 26900,
                5084,  58208, 47404, 3568,  52716, 34852, 49736, 40456, 29980,
                55140, 16784, 360,   45148, 5284,  52132, 18240, 29592, 57100,
                33640, 484,   24924, 61736, 36712, 42264, 49020, 52572, 31352,
                57096, 26768, 50600, 16492, 46680, 61752, 41608, 14764, 39400,
                31584, 6152,
            },
            {
                35620, 45844, 54964, 9716,  24276, 39220, 12912, 5240,  58772,
                10620, 36744, 41752, 57420, 50852, 1280,  44736, 20064, 11840,
                58304, 17528, 8916,  25124, 14068, 2920,  17988, 59196, 27416,
                31580, 61840, 21108, 58420, 7212,  28700, 48552, 40152, 25032,
                1508,  41400, 36624, 27364, 61000, 42620, 9988,  45172, 35404,
                12240, 32832, 46668, 29196, 36144, 5500,  65392, 16344, 47664,
                32700, 39608, 25540, 58288, 8276,  23864, 50636, 45284, 17584,
                62272, 41340, 13464, 58136, 44872, 35624, 63508, 26544, 9196,
                61688, 48064, 34924, 45116, 64444, 40156, 53344, 21008, 37396,
                5680,  54420, 34804, 26144, 39172, 8512,  62696, 22940, 56056,
                14560, 19360, 41788, 24128, 63572, 15852, 1596,  61320, 23636,
                51348, 34028, 62060, 21096, 26132, 35564, 59440, 42432, 9648,
                53980, 40544, 17864, 8240,  53428, 15712, 23092, 10056, 61352,
                20988, 44956, 3488,  7620,  28824, 10548, 33300, 26028, 3856,
                53872, 28116,
            },
            {
                20180, 12616, 26720, 6700,  32776, 56772, 47456, 30164, 19276,
                49636, 6208,  21760, 12828, 31256, 16640, 60048, 34920, 6524,
                37856, 22632, 61788, 43356, 51820, 40264, 64192, 22180, 53656,
                1000,  39320, 4588,  46204, 16708, 62608, 19940, 34788, 54488,
                29780, 57224, 47844, 5592,  50980, 428,   31396, 61912, 4268,
                25760, 51976, 42052, 17832, 55464, 49860, 38220, 27560, 19752,
                5912,  51208, 3812,  18224, 34124, 13752, 38916, 59180, 3712,
                28880, 51316, 23204, 18380, 39072, 5344,  11792, 53528, 3356,
                18324, 28772, 10756, 844,   26384, 7360,  33308, 47340, 62484,
                15896, 29688, 64700, 21476, 48236, 18980, 29944, 12432, 39732,
                34316, 52064, 30440, 7568,  38152, 32044, 11456, 42940, 19532,
                3800,  38380, 9928,  40396, 53772, 8004,  2380,  48780, 20260,
                26724, 64816, 44432, 32120, 27352, 4548,  58804, 38252, 6060,
                14828, 40104, 34132, 58672, 52796, 20764, 49560, 60596, 17764,
                9076,  49168,
            },
            {
                2476,  58884, 50504, 43512, 62276, 1916,  16312, 63328, 40200,
                25196, 54684, 3120,  47568, 24228, 9520,  39804, 27028, 46652,
                51144, 29808, 712,   33716, 6224,  28408, 48396, 11568, 36636,
                14780, 56324, 10000, 30684, 36896, 43212, 4048,  9020,  60656,
                14860, 21812, 9800,  33788, 24116, 38124, 14824, 55668, 40220,
                22316, 62732, 9192,  60248, 21616, 3536,  11576, 53008, 59608,
                42348, 22224, 61112, 45036, 30024, 65084, 27204, 21432, 12252,
                57112, 7540,  33172, 3000,  48772, 59632, 21252, 46892, 32576,
                55772, 42496, 59600, 20792, 51880, 60740, 11828, 23732, 56272,
                9824,  41844, 46292, 6284,  57864, 36256, 43372, 50692, 6616,
                61476, 10612, 44184, 59508, 18052, 47892, 57204, 28076, 46056,
                14348, 59252, 29548, 47772, 12892, 63184, 32212, 14624, 60536,
                35952, 6512,  13444, 56728, 46940, 63268, 33560, 43900, 25424,
                48288, 62624, 11552, 24204, 43296, 36700, 1924,  56888, 44212,
                37632, 61844,
            },
            {
                23896, 39728, 17004, 36416, 21120, 28368, 52036, 8544,  44644,
                32060, 61052, 34648, 58496, 42528, 62528, 53984, 13764, 3876,
                65100, 15400, 45308, 54492, 20288, 57728, 8420,  32932, 44908,
                50616, 25660, 65404, 22292, 54212, 12764, 50880, 32588, 46600,
                2780,  43916, 64712, 16920, 59760, 53240, 20240, 28364, 7464,
                49188, 15624, 928,   39140, 32264, 25420, 44424, 34784, 14256,
                29460, 9328,  36376, 54600, 424,   51860, 5688,  42468, 35036,
                46120, 38424, 54652, 62560, 29984, 25008, 15056, 40948, 36988,
                23056, 5524,  14312, 38980, 35908, 17816, 4088,  43808, 27632,
                216,   51384, 13780, 32888, 2936,  15544, 59876, 24312, 2396,
                47216, 27812, 22044, 116,   54952, 25440, 4376,  36332, 64564,
                7408,  25276, 55928, 18444, 43764, 23752, 28216, 39324, 45872,
                3100,  50188, 23664, 37988, 1408,  11836, 17280, 51020, 29500,
                124,   53944, 30844, 17080, 63544, 6248,  13828, 30108, 22212,
                11736, 32732,
            },
            {
                5792,  55768, 30888, 3496,  46776, 11904, 38084, 23376, 680,
                13356, 17212, 28148, 7980,  19004, 1976,  33132, 21816, 56668,
                40884, 25792, 9904,  61060, 36300, 16628, 24848, 62748, 5872,
                19300, 41512, 34192, 47636, 1400,  26592, 63712, 23268, 38236,
                27172, 52408, 40012, 30608, 3444,  43376, 11584, 64040, 45820,
                36076, 30392, 53672, 13036, 47976, 62952, 8368,  56632, 1632,
                64068, 48540, 16208, 11760, 40480, 19268, 47832, 9632,  63352,
                24224, 14792, 20028, 10812, 42816, 52268, 524,   57356, 8012,
                65524, 50996, 31324, 57772, 46612, 29296, 49996, 58316, 35560,
                19340, 38408, 23352, 61216, 54732, 26820, 10176, 32520, 53912,
                16600, 36020, 64740, 14236, 39656, 50452, 9696,  21560, 53172,
                32788, 50256, 1680,  36044, 5804,  57496, 51076, 10292, 22208,
                16644, 58048, 30896, 19224, 51792, 41196, 22016, 7576,  56540,
                19288, 37172, 4128,  47048, 40536, 27348, 55184, 46424, 64628,
                41496, 52100,
            },
            {
                47820, 14228, 64180, 8804,  54356, 60724, 34000, 56360, 48336,
                65440, 36884, 52940, 46060, 38356, 51592, 29408, 48916, 7528,
                18696, 32572, 49368, 12600, 39624, 1524,  42612, 55388, 29692,
                2860,  60352, 17140, 6508,  58516, 39596, 15300, 56792, 6164,
                18528, 12264, 55972, 7608,  26444, 48924, 34292, 1692,  17736,
                57336, 24436, 43272, 19500, 58044, 28680, 17120, 39392, 24020,
                33268, 43604, 26448, 57740, 23288, 61428, 32812, 16452, 52928,
                30508, 1392,  47488, 26968, 7184,  34244, 61140, 17504, 27304,
                11712, 44216, 3200,  24420, 8704,  64184, 13592, 40604, 6480,
                63052, 53268, 29028, 7880,  40856, 49104, 38004, 63628, 19592,
                41416, 56752, 6180,  45796, 33608, 29436, 61100, 13044, 40504,
                16908, 43188, 22408, 61300, 15564, 33424, 796,   63888, 55060,
                34024, 42732, 62304, 9084,  28688, 54944, 36228, 64844, 12772,
                42016, 60500, 23216, 9600,  57924, 19880, 34760, 8080,  16372,
                640,   28456,
            },
            {
                59704, 26160, 44492, 23028, 18012, 41880, 26052, 6592,  20224,
                41476, 10444, 4512,  26020, 15112, 64176, 11360, 43256, 60404,
                37256, 2900,  58476, 21624, 30732, 51996, 47852, 13568, 22792,
                38648, 52936, 11956, 28476, 44248, 20220, 10060, 30928, 49616,
                61668, 36140, 21364, 46884, 14244, 58824, 23136, 40956, 51540,
                4892,  10892, 64756, 3048,  35432, 5784,  46380, 50828, 20404,
                6332,  53576, 3008,  31808, 7676,  37072, 55860, 25780, 5148,
                59984, 40692, 56068, 64784, 37444, 12884, 45420, 30636, 49500,
                35248, 19852, 54856, 16268, 42276, 2312,  21900, 32056, 25912,
                16200, 47792, 11648, 44412, 17756, 21776, 744,   44852, 7712,
                30892, 11452, 24756, 52384, 3032,  19644, 43728, 58344, 5272,
                30356, 63368, 11296, 39204, 53884, 46676, 19428, 41800, 7424,
                25676, 12044, 3792,  46104, 60264, 24588, 2436,  45264, 32628,
                26712, 49152, 15576, 52040, 31524, 2624,  50136, 25372, 53816,
                36132, 19032,
            },
            {
                6932,  10628, 38276, 32864, 49384, 4232,  14412, 59084, 31328,
                51052, 22724, 60216, 56860, 20776, 5984,  35420, 24576, 16120,
                28632, 52532, 41948, 5544,  64584, 26912, 9680,  61484, 35260,
                45656, 8148,  49492, 32492, 62316, 35988, 53492, 41452, 608,
                45068, 32824, 4100,  63596, 37852, 54900, 9616,  31708, 60708,
                26632, 38696, 32904, 50384, 23336, 41560, 11696, 55688, 61980,
                37540, 18416, 59336, 50156, 45988, 14144, 1864,  43448, 49968,
                21632, 35516, 17892, 3464,  50744, 20284, 23544, 55260, 1972,
                39968, 60504, 48048, 28188, 34076, 51588, 55172, 46020, 10128,
                59176, 33776, 3224,  65360, 31672, 58876, 13596, 51648, 27408,
                61804, 48696, 37636, 17600, 62716, 8332,  23540, 47492, 37192,
                2576,  49340, 27672, 8288,  30788, 24972, 12408, 37368, 29152,
                49360, 53692, 39412, 15520, 34468, 6088,  48796, 16348, 58428,
                8752,  4988,  35204, 64332, 12532, 44088, 61108, 39880, 11240,
                62772, 42492,
            },
            {
                33888, 52972, 57948, 1300,  63056, 28208, 52380, 45528, 16424,
                3040,  28848, 44392, 40160, 32296, 48032, 55648, 196,   46792,
                63532, 9168,  23972, 45200, 17360, 34116, 3804,  19980, 58072,
                15704, 26164, 55992, 23320, 2692,  48248, 7024,  17380, 28336,
                58116, 10256, 25232, 42132, 18920, 28528, 5788,  15528, 44296,
                20720, 47512, 8244,  55148, 14608, 61084, 248,   26192, 30592,
                8816,  13716, 40168, 27772, 21084, 63624, 29308, 38800, 11948,
                8412,  44992, 13832, 31712, 58800, 41692, 5356,  62044, 9348,
                25632, 13436, 6212,  62576, 10984, 38352, 18244, 62076, 1428,
                42852, 20696, 54368, 24800, 37152, 4456,  56020, 35040, 23068,
                15032, 1600,  59372, 31988, 40920, 55760, 34368, 15164, 26400,
                54208, 18524, 45668, 59820, 3300,  50640, 58088, 61568, 5048,
                17928, 65332, 21404, 27088, 57048, 11548, 38688, 29816, 21568,
                40120, 54228, 45732, 25020, 38448, 17400, 22380, 4116,  30688,
                47060, 21756,
            },
            {
                50212, 29324, 14996, 20116, 40540, 11604, 34692, 8700,  38744,
                61996, 35824, 13604, 1608,  9836,  27856, 59880, 38904, 13060,
                20092, 55276, 35916, 14176, 56984, 47004, 53572, 40968, 30992,
                1900,  64500, 42164, 18732, 13732, 57516, 25592, 64996, 21784,
                51916, 15944, 49036, 60136, 1908,  50948, 35996, 63152, 53916,
                12936, 58888, 17560, 27932, 45348, 31792, 36612, 15676, 44068,
                47984, 65028, 34912, 5012,  10360, 52856, 18104, 57964, 33380,
                62868, 28068, 54224, 24864, 10180, 48464, 33460, 15768, 43828,
                53168, 31656, 36900, 21088, 56848, 25480, 5716,  29920, 36060,
                50728, 39148, 7336,  15160, 49740, 29492, 41708, 6600,  46584,
                54820, 20680, 44008, 5132,  25980, 11332, 50656, 65472, 7356,
                57428, 12296, 36412, 21168, 40488, 16376, 43580, 22592, 32640,
                47088, 36348, 364,   43956, 50924, 18836, 62692, 52520, 3864,
                61284, 18388, 28996, 720,   7728,  54604, 33692, 59080, 14632,
                56384, 2976,
            },
            {
                38924, 65168, 5180,  24816, 54932, 44052, 61208, 23460, 53808,
                48228, 18628, 54912, 63148, 50060, 17444, 22472, 44660, 4332,
                33060, 50008, 2124,  29164, 38472, 7152,  24656, 12036, 50464,
                37288, 5764,  33508, 9828,  38572, 29784, 43704, 34892, 12824,
                39636, 5136,  29868, 34328, 12476, 46072, 22400, 40092, 4072,
                29804, 1100,  37032, 62180, 6796,  20244, 53196, 60032, 22084,
                2600,  52504, 24180, 44620, 60632, 40940, 3436,  46684, 23724,
                52144, 132,   61292, 39820, 4252,  28548, 64328, 37948, 21592,
                58360, 17660, 42028, 436,   44588, 49144, 15248, 57988, 23520,
                12492, 27768, 56876, 46144, 62356, 10332, 19056, 60152, 32260,
                39588, 9972,  28820, 53032, 16052, 46188, 548,   29712, 40088,
                22800, 32896, 62660, 5768,  55512, 34424, 9688,  2376,  56108,
                14520, 10200, 30120, 60908, 7304,  32016, 23340, 42580, 10400,
                34104, 13492, 57340, 41780, 62584, 49404, 27420, 44888, 8820,
                25892, 17724,
            },
            {
                11964, 45592, 35568, 48776, 30524, 6424,  19240, 460,   27152,
                11372, 5060,  24700, 33440, 41884, 6804,  53660, 30304, 61120,
                25720, 41252, 62684, 10704, 59860, 18916, 63408, 28536, 44564,
                22176, 60984, 53124, 46080, 51004, 60020, 3688,  8940,  55432,
                61500, 44764, 23296, 54068, 7268,  57272, 17260, 25740, 49600,
                33848, 42864, 22040, 51116, 40664, 10316, 48560, 4216,  38964,
                28872, 11404, 56196, 15980, 31500, 25644, 36216, 14568, 6272,
                20044, 37700, 16072, 46832, 19320, 51848, 12116, 26492, 7224,
                47432, 4408,  51712, 64904, 30764, 8296,  40292, 52764, 4448,
                64000, 17888, 32800, 1044,  22028, 43748, 26468, 12640, 64624,
                4008,  50184, 36088, 63552, 22132, 58200, 37552, 19520, 44832,
                52192, 1832,  43124, 26048, 13132, 64096, 28568, 49904, 39936,
                59776, 23956, 53280, 41172, 13800, 47796, 1388,  55832, 26300,
                46524, 50784, 22824, 31784, 19744, 11540, 4916,  36780, 52236,
                41540, 59816,
            },
            {
                28044, 22024, 9388,  57600, 16560, 37788, 58732, 33076, 40776,
                64688, 30652, 46004, 20464, 59292, 14424, 36448, 10604, 51456,
                7652,  16248, 22068, 48468, 32288, 43072, 960,   55600, 8724,
                17876, 13340, 27244, 340,   20664, 15484, 24200, 47444, 31552,
                1324,  37028, 19220, 64484, 41456, 31268, 61156, 8376,  11488,
                65328, 55856, 6428,  13848, 26424, 64292, 34208, 17904, 58416,
                42688, 33792, 19812, 656,   49024, 8976,  55084, 64832, 49476,
                32344, 43836, 8064,  57900, 35268, 55908, 2872,  44776, 60092,
                34428, 28756, 10420, 24196, 13312, 60140, 33548, 20468, 43408,
                47908, 8900,  40700, 59376, 35704, 4956,  53248, 37864, 17304,
                25104, 56528, 13244, 7004,  41704, 32392, 4452,  13740, 59300,
                10016, 16716, 49116, 31020, 52624, 45900, 17716, 25136, 7260,
                44616, 19436, 4744,  37564, 25752, 63720, 35096, 15172, 38056,
                65092, 3304,  8956,  37220, 58712, 46836, 16428, 64400, 20236,
                32176, 64,
            },
            {
                61444, 54140, 42100, 2096,  62860, 12260, 50844, 46840, 14872,
                56756, 8060,  51732, 38052, 3848,  27396, 65156, 43276, 19152,
                34372, 58604, 45072, 26360, 5956,  51368, 14988, 34496, 59612,
                47672, 40612, 57164, 31804, 64084, 36108, 41484, 53424, 16944,
                27488, 49976, 9888,  14396, 26880, 148,   44144, 53040, 37940,
                18728, 24500, 45012, 58252, 30048, 5336,  56064, 12624, 25556,
                7844,  50288, 63300, 38104, 59088, 21648, 42400, 27904, 10976,
                59544, 24376, 29508, 13076, 22660, 41000, 31276, 18060, 53580,
                13816, 40068, 56096, 36420, 18608, 46620, 26636, 2164,  61396,
                31088, 53820, 24896, 13628, 50356, 28712, 57804, 47896, 1264,
                44780, 30708, 19708, 48804, 26864, 55052, 62448, 48348, 25100,
                28808, 61628, 38832, 8584,  20056, 580,   38404, 54444, 31596,
                63436, 34588, 49484, 58988, 18040, 8920,  44348, 20692, 6220,
                28464, 17128, 43960, 52372, 1700,  29372, 40348, 23628, 55616,
                7448,  49444,
            },
            {
                14864, 18616, 32592, 26520, 45260, 23148, 29004, 4136,  21456,
                43044, 35412, 16760, 11772, 55516, 47504, 24212, 5668,  56048,
                38176, 1480,  13180, 54704, 36576, 20884, 39952, 23916, 30016,
                6436,  36432, 24648, 4668,  44748, 11460, 6824,  62372, 21116,
                56816, 6096,  59068, 39120, 52092, 21180, 35068, 15208, 28084,
                48548, 1796,  32940, 16656, 39236, 46936, 22564, 44244, 31040,
                60816, 14420, 6168,  28304, 45292, 4632,  34408, 18688, 1572,
                38660, 54088, 64044, 5964,  49824, 61772, 9732,  24760, 63444,
                1216,  22372, 49008, 61904, 3236,  54016, 38504, 11480, 16328,
                36924, 5944,  44952, 20232, 64188, 16500, 8464,  34092, 22932,
                59772, 40008, 61712, 1960,  10856, 17448, 35888, 7616,  42076,
                33908, 55284, 4624,  23776, 57276, 35180, 61160, 11048, 3772,
                15760, 27840, 11936, 2012,  54528, 29628, 51724, 60492, 57476,
                48732, 33304, 55524, 25700, 14720, 60860, 34256, 3768,  13000,
                43664, 35872,
            },
            {
                46736, 3552,  39008, 52216, 6924,  35236, 54748, 10076, 60236,
                26152, 2152,  62264, 29280, 40556, 576,   32616, 15660, 48792,
                27936, 63976, 31160, 3952,  57908, 9920,  65400, 53484, 3156,
                50112, 62840, 10108, 54948, 18756, 51720, 29376, 38676, 13272,
                34140, 42804, 24416, 32204, 47972, 5620,  56364, 3676,  58740,
                41316, 9900,  53988, 61908, 3852,  52224, 35332, 1056,  54692,
                39972, 46748, 24008, 54136, 17072, 12332, 62092, 50904, 46332,
                15816, 3788,  34584, 45092, 16916, 1984,  38412, 47968, 42736,
                32360, 16332, 7964,  28012, 41564, 6896,  58708, 50776, 22708,
                28420, 56500, 10572, 38452, 2556,  31252, 42208, 11804, 51200,
                6456,  14672, 33512, 52592, 46452, 29844, 22992, 53312, 2336,
                20588, 12896, 45288, 65040, 42312, 14536, 47336, 22248, 51352,
                42872, 56920, 46392, 22696, 33484, 41604, 12512, 2820,  39404,
                23388, 12088, 61780, 20392, 42744, 10024, 47648, 51312, 62216,
                29900, 24740,
            },
            {
                5992,  64604, 56548, 13516, 19740, 41528, 64136, 17624, 38636,
                47960, 53888, 23276, 44916, 58268, 21508, 52076, 62396, 11800,
                21004, 40412, 50564, 18500, 42616, 27660, 46164, 12748, 17376,
                43508, 21300, 14920, 47116, 33900, 25748, 59592, 3784,  49056,
                45744, 2172,  61544, 17340, 11952, 64060, 45628, 23492, 31676,
                62852, 21344, 36584, 12484, 25236, 19380, 9748,  63612, 16440,
                21076, 3152,  36160, 32316, 41368, 57780, 22928, 30488, 56212,
                41132, 21092, 52072, 25444, 32048, 59008, 28096, 20592, 6368,
                50820, 59552, 45104, 34824, 21288, 14464, 31876, 43080, 65196,
                224,   48604, 60220, 26364, 54428, 46244, 60920, 19392, 55064,
                28384, 43608, 21652, 8432,  38108, 64376, 44508, 15724, 61132,
                50448, 36864, 26984, 17472, 32084, 5632,  29248, 58828, 36320,
                19712, 8580,  39328, 61820, 6488,  48460, 27084, 16528, 31448,
                8508,  41096, 4124,  36592, 6608,  54292, 26688, 17896, 38340,
                9012,  58856,
            },
            {
                50308, 23040, 30704, 9700,  59640, 49280, 896,   33164, 29776,
                5296,  13392, 33780, 6580,  18048, 9248,  36888, 42188, 4636,
                45800, 8472,  24504, 15312, 59928, 32116, 7228,  37472, 56596,
                33016, 27832, 39292, 61432, 1636,  42228, 15800, 23060, 65276,
                10416, 28064, 54508, 7284,  37196, 28944, 18508, 39352, 13304,
                7080,  45956, 50000, 29236, 42700, 60296, 48268, 37992, 28904,
                50748, 11248, 65520, 8280,  48312, 1956,  37248, 9876,  6644,
                27260, 12400, 60976, 8868,  42476, 54376, 14800, 11152, 57028,
                36920, 25864, 11724, 64160, 55244, 47608, 25068, 8592,  35416,
                19572, 41520, 15532, 33576, 22252, 5760,  39528, 25432, 35880,
                65256, 4112,  56868, 24976, 58724, 13092, 5896,  39612, 31188,
                9664,  58628, 3616,  53848, 49944, 9124,  26008, 40508, 2588,
                64664, 30364, 25228, 14868, 56140, 21060, 64504, 35860, 54848,
                46444, 63428, 51016, 30220, 59316, 22192, 32648, 1020,  56188,
                20424, 41044,
            },
            {
                33732, 16972, 37308, 46052, 28120, 24640, 15372, 44828, 57392,
                20276, 50536, 63776, 39208, 49292, 60676, 30504, 26064, 57768,
                34004, 61148, 53540, 35640, 47296, 232,   20280, 25112, 60336,
                2340,  51196, 7028,  30736, 57856, 9140,  37492, 52960, 31228,
                18832, 40496, 22152, 51008, 42440, 55304, 9544,  60560, 51744,
                27236, 17720, 2364,  55048, 7488,  32436, 14596, 25964, 6628,
                59216, 42304, 56296, 27032, 14804, 52884, 25772, 64144, 33772,
                58464, 48376, 36628, 18860, 5224,  49392, 34700, 65528, 30044,
                3112,  52864, 19388, 5664,  29880, 2648,  17364, 60760, 53308,
                12440, 30072, 4012,  51500, 63332, 13204, 49580, 820,   10020,
                16756, 39112, 47928, 18560, 34384, 212,   26780, 55904, 18120,
                46648, 22608, 40096, 12020, 34676, 62300, 55672, 17868, 12888,
                49044, 4924,  52688, 36916, 140,   45128, 10132, 50568, 19080,
                1140,  26516, 13612, 17356, 44224, 38872, 65336, 15344, 45916,
                28376, 11484,
            },
            {
                54220, 43116, 1672,  63348, 5096,  39668, 53456, 7988,  61596,
                41292, 10348, 28628, 14976, 24888, 2564,  12736, 55040, 16952,
                1256,  29696, 11144, 5572,  39284, 63188, 52952, 48208, 40876,
                10980, 63956, 23788, 13380, 49164, 20756, 44268, 56572, 6596,
                36336, 58660, 13660, 33644, 4176,  24704, 32752, 1048,  43232,
                57436, 34304, 64952, 39760, 21964, 57108, 2684,  53336, 45524,
                33756, 17980, 22328, 31272, 60588, 19888, 39868, 45472, 17392,
                876,   43284, 29480, 63168, 23096, 40124, 468,   46084, 22412,
                41304, 15084, 43992, 37920, 50580, 57400, 33748, 44664, 4680,
                38240, 57860, 45880, 8424,  36640, 17976, 30496, 57892, 46988,
                32220, 53272, 11680, 29416, 51460, 41532, 49160, 62204, 35464,
                1708,  29484, 60412, 48196, 20636, 1088,  44452, 52164, 23560,
                33912, 45852, 59560, 18408, 32240, 40560, 24600, 6564,  60744,
                43128, 21924, 34992, 56328, 2544,  11616, 50076, 7656,  35628,
                62632, 3256,
            },
            {
                26088, 14460, 51064, 21392, 55900, 32140, 12540, 23000, 36404,
                25860, 1536,  56024, 43732, 35528, 53116, 46148, 20184, 38808,
                48520, 43184, 21752, 56480, 26860, 14016, 8640,  30148, 15884,
                35920, 19504, 45080, 54464, 34828, 27392, 3312,  12788, 25588,
                48308, 456,   63476, 46952, 16496, 62408, 49548, 20916, 36392,
                5516,  24344, 13772, 10212, 44568, 35960, 18420, 61508, 39248,
                13428, 168,   47364, 5816,  43812, 3396,  12008, 50172, 22156,
                53904, 14612, 4328,  55504, 11796, 27148, 58572, 18356, 8560,
                55640, 32704, 62792, 10276, 22980, 40272, 13476, 26992, 50140,
                23624, 62064, 20704, 27612, 43904, 55572, 23584, 62168, 41816,
                26648, 2552,  63296, 7416,  59788, 15516, 21352, 10560, 24708,
                54156, 7960,  42660, 15844, 25028, 38624, 30996, 10424, 61616,
                41676, 27264, 9300,  13088, 62436, 28892, 57120, 14992, 38204,
                30556, 58468, 9336,  47696, 28816, 53244, 25456, 41688, 22584,
                48216, 58232,
            },
            {
                30420, 61048, 7532,  35728, 17552, 59244, 48260, 3096,  51576,
                16512, 46404, 60024, 18812, 5020,  64896, 32992, 7280,  61976,
                24300, 13572, 64372, 50088, 18192, 34492, 43784, 22604, 55568,
                4384,  58728, 41552, 884,   17432, 64312, 59048, 32384, 51584,
                16960, 43444, 29596, 8648,  56976, 26952, 12056, 46184, 15736,
                53748, 31112, 47816, 58944, 26944, 51408, 5028,  23568, 10296,
                29904, 55280, 37312, 51252, 34236, 63176, 29112, 7188,  60180,
                31660, 24908, 38736, 46568, 33020, 16048, 50908, 37588, 61576,
                25252, 49076, 4700,  27960, 60176, 1064,  19968, 63844, 10084,
                32268, 16768, 1208,  53176, 11060, 40332, 3868,  7732,  20448,
                14480, 45064, 23272, 37056, 43912, 3896,  32408, 45884, 38540,
                14160, 63484, 33136, 4664,  57208, 64232, 7088,  36212, 16168,
                1828,  21356, 55916, 43692, 51056, 5360,  48028, 34100, 11052,
                52968, 4368,  39956, 63912, 20508, 5200,  60964, 31032, 13416,
                5808,  18864,
            },
            {
                39164, 10916, 47236, 41660, 25764, 9756,  43504, 34212, 64592,
                30076, 38308, 8156,  31540, 22804, 40820, 10552, 28356, 51452,
                4556,  36828, 32000, 2224,  45664, 59032, 3344,  61672, 49988,
                32228, 26040, 9340,  29760, 38116, 5408,  46284, 40988, 9264,
                61092, 23692, 38912, 20420, 35460, 2256,  40428, 58328, 8428,
                63492, 42008, 20264, 552,   16140, 62488, 31376, 41228, 49728,
                64356, 25316, 59844, 9472,  16900, 24716, 57032, 36080, 41732,
                10496, 64880, 51772, 8400,  60720, 42196, 5860,  31012, 13104,
                1748,  36156, 17236, 46544, 53900, 34936, 42868, 47736, 5936,
                56224, 41936, 48284, 34724, 64908, 15620, 33248, 51876, 37512,
                60804, 54404, 33816, 19328, 55404, 27308, 65036, 6556,  58068,
                50800, 19696, 27564, 52808, 46720, 13496, 28052, 50028, 58972,
                53500, 30768, 6748,  38348, 23404, 2508,  20108, 64216, 26804,
                17592, 49764, 24060, 14836, 45604, 33856, 17984, 55748, 37356,
                52700, 44504,
            },
            {
                33980, 23708, 54816, 29320, 220,   62068, 20124, 14552, 4540,
                55236, 21772, 12840, 52084, 48276, 58012, 14644, 44756, 59108,
                16536, 54468, 8768,  40028, 29216, 10940, 25184, 37600, 7496,
                13748, 46400, 53160, 61016, 50436, 24384, 14368, 20336, 35724,
                54216, 4948,  12356, 51140, 44948, 53992, 30116, 23248, 38168,
                28308, 4444,  50476, 34796, 38772, 8892,  46820, 56736, 19356,
                7104,  15200, 41796, 21372, 54796, 39076, 14336, 47564, 19076,
                3556,  35024, 16688, 20932, 2260,  56932, 23408, 45696, 53096,
                43220, 59444, 21512, 12668, 30544, 7912,  16132, 58424, 28952,
                35948, 13736, 25088, 6640,  29856, 57264, 26136, 47732, 11780,
                28876, 5468,  49456, 9260,  13468, 39748, 52656, 17372, 30040,
                3188,  44632, 37392, 9924,  18452, 41500, 22088, 3468,  39552,
                19156, 47856, 65424, 35072, 16788, 54792, 41580, 45536, 59396,
                1540,  43708, 31928, 7564,  37868, 58908, 396,   42896, 9508,
                64544, 2272,
            },
            {
                57836, 4788,  62956, 15144, 52556, 31428, 39084, 58444, 27404,
                47744, 42360, 62936, 864,   37264, 25584, 3324,  21000, 34768,
                41828, 26452, 60324, 19756, 51248, 55232, 16592, 42124, 20596,
                65076, 39492, 16228, 21484, 11412, 56036, 30612, 62400, 2208,
                27896, 58140, 32200, 64984, 6044,  17932, 61948, 3380,  51728,
                18704, 12268, 60856, 55980, 29156, 22392, 13224, 1936,  34008,
                44864, 28524, 2524,  32624, 46272, 6348,  1136,  52852, 27340,
                57608, 49380, 44272, 26228, 36948, 29328, 10376, 64496, 19552,
                27336, 9064,  55192, 41016, 65088, 25312, 52440, 22460, 38956,
                2960,  53720, 61152, 21744, 50312, 2216,  18892, 59660, 43032,
                532,   17456, 57796, 30536, 61340, 1224,  23848, 36008, 41628,
                11940, 22416, 61664, 344,   34500, 54904, 60660, 45392, 33196,
                11132, 25712, 13964, 60080, 27540, 32440, 12336, 6668,  22332,
                36360, 56160, 62516, 27468, 52312, 12196, 49264, 24652, 16160,
                28016, 21012,
            },
            {
                12376, 49212, 18180, 37140, 6860,  44904, 50468, 23416, 7404,
                10868, 18412, 34020, 28948, 16848, 54188, 62224, 30180, 49516,
                520,   11864, 47156, 23368, 5080,  63380, 34984, 48432, 54148,
                2864,  28896, 5748,  36380, 44484, 7736,  40100, 49272, 17872,
                43176, 47700, 14944, 25968, 41572, 10768, 33476, 14316, 59504,
                46388, 32600, 24152, 7076,  44240, 63972, 52508, 26556, 37052,
                61232, 53668, 49196, 12132, 61716, 29608, 42576, 63648, 22272,
                7664,  30564, 12704, 62088, 54236, 48212, 14688, 34672, 3016,
                39524, 32560, 50224, 6136,  37084, 45160, 3808,  11868, 62912,
                19148, 45924, 9332,  37456, 44524, 40676, 8360,  35524, 22840,
                64524, 38652, 25376, 42216, 46268, 20052, 49732, 8680,  60376,
                47812, 25868, 56780, 49688, 29032, 7680,  24564, 15212, 5420,
                56896, 42748, 1276,  46344, 9776,  57996, 52052, 38692, 29916,
                9272,  13788, 18992, 4576,  40916, 21236, 30060, 61868, 35900,
                51168, 45716,
            },
            {
                40336, 27164, 43144, 22588, 56724, 12900, 3424,  35520, 54280,
                40064, 59764, 6028,  56536, 43652, 9240,  39960, 6992,  56944,
                18584, 65308, 32960, 43248, 14928, 30920, 996,   26816, 11244,
                33176, 59840, 48912, 25380, 63592, 33496, 3660,  53120, 23192,
                9932,  34288, 832,   21852, 56552, 48456, 37464, 44076, 26328,
                9088,  40072, 48632, 14968, 3572,  40584, 17840, 59112, 11328,
                22752, 5608,  18300, 35756, 57732, 23220, 17088, 10876, 33340,
                40636, 59564, 296,   39560, 18168, 4040,  41592, 51420, 58556,
                16256, 62976, 23992, 36,    18856, 60640, 33036, 49720, 42344,
                26304, 31212, 58592, 17492, 12684, 62544, 54548, 14276, 31632,
                52600, 48316, 7012,  35308, 11500, 55828, 28212, 15156, 54008,
                31072, 6312,  15920, 42936, 12220, 64812, 38280, 50500, 29708,
                62892, 52752, 22160, 40268, 4140,  48664, 24544, 16236, 63204,
                46536, 51012, 34296, 60164, 55124, 8644,  46908, 3364,  56152,
                7220,  31644,
            },
            {
                1032,  58760, 9268,  32544, 65452, 28112, 19124, 60772, 31108,
                45748, 25812, 48992, 23160, 12288, 35596, 46688, 23764, 13892,
                37776, 27848, 7548,  55820, 38584, 9840,  58688, 45056, 22644,
                18140, 56576, 42944, 136,   18944, 57888, 26972, 12612, 37208,
                63920, 55068, 39520, 60616, 29088, 19416, 6864,  53432, 21716,
                1096,  65448, 54332, 36244, 20864, 49868, 32472, 8112,  42452,
                47600, 38588, 65204, 26180, 8448,  51028, 37336, 56236, 45168,
                15008, 52368, 24016, 6536,  33064, 63556, 21940, 25788, 6784,
                46252, 12140, 29744, 56440, 48496, 15152, 28260, 8460,  55676,
                15548, 600,   51152, 33676, 4920,  29064, 25040, 46596, 3196,
                10716, 21132, 15672, 62344, 2980,  32908, 64064, 44192, 2344,
                34704, 63196, 39800, 21284, 32136, 46928, 2828,  20532, 35320,
                8972,  17532, 36772, 30940, 63960, 19332, 35296, 316,   55380,
                40420, 25528, 2656,  43192, 23604, 15584, 33152, 39864, 13820,
                19692, 63696,
            },
            {
                16504, 35356, 50732, 2892,  47076, 41028, 8484,  49448, 16044,
                2112,  13712, 64928, 3836,  51768, 19820, 63600, 31404, 53236,
                42364, 50660, 3240,  60896, 47924, 20136, 51904, 36520, 61892,
                40032, 7956,  13368, 30460, 51548, 15472, 46608, 60172, 29988,
                6092,  16700, 45672, 8664,  50836, 2804,  63684, 30784, 57360,
                35060, 17208, 29728, 10412, 57748, 27512, 1452,  54928, 15828,
                30340, 3184,  55852, 13972, 44256, 31648, 5692,  48876, 3372,
                19832, 35784, 46948, 27480, 56424, 44692, 9276,  30692, 54096,
                37892, 20828, 43588, 35788, 10380, 40772, 21896, 59408, 34716,
                6316,  64020, 41220, 22348, 56656, 48848, 19584, 39132, 57000,
                44288, 59464, 29752, 53192, 24068, 40568, 5740,  22020, 38484,
                18332, 10328, 55340, 3996,  59380, 17248, 53324, 58092, 41868,
                27296, 48168, 5708,  55252, 14296, 28344, 43624, 59672, 6012,
                20072, 29052, 11752, 48380, 37332, 65136, 26756, 59268, 44140,
                24124, 52528,
            },
            {
                11304, 25648, 61336, 20848, 15016, 55292, 24720, 62828, 36976,
                21200, 29188, 38560, 32740, 59168, 27624, 6388,  2700,  58060,
                10096, 21496, 35572, 16744, 24624, 29192, 6112,  14232, 3644,
                50324, 24520, 35700, 55100, 38968, 22732, 8488,  41328, 19860,
                48688, 25204, 31452, 13188, 36032, 24608, 15540, 42408, 11924,
                50372, 4616,  45532, 24724, 61652, 38292, 44672, 62816, 24960,
                51488, 19868, 33276, 40808, 1488,  20812, 59748, 24980, 28740,
                64964, 9564,  60904, 16220, 50480, 12436, 35896, 60012, 48840,
                1552,  57700, 7524,  61812, 25848, 54036, 2200,  44836, 38188,
                20080, 47348, 27556, 9964,  36480, 2036,  64240, 7724,  34040,
                26456, 4132,  37204, 49908, 18568, 57852, 13640, 48996, 59724,
                24892, 46276, 50756, 27976, 36284, 8792,  25920, 13840, 636,
                61384, 11744, 24304, 45016, 58836, 7444,  11308, 49496, 32244,
                15140, 53732, 61588, 7460,  18128, 1120,  53148, 10300, 4164,
                47916, 38088,
            },
            {
                55792, 4592,  44928, 39144, 29928, 34740, 668,   11384, 42096,
                57744, 53068, 9712,  44860, 14772, 41668, 49084, 17848, 34220,
                25916, 63212, 45596, 11768, 41308, 54728, 64288, 43752, 31496,
                16728, 46492, 64660, 10440, 4868,  62252, 34880, 1884,  57092,
                52628, 4528,  62536, 43804, 54364, 59948, 38716, 47284, 27284,
                20176, 59140, 41432, 6904,  13440, 18948, 4996,  35304, 12212,
                60144, 9572,  46100, 58904, 52200, 16296, 62668, 39156, 13128,
                53636, 42608, 6876,  30308, 20700, 40372, 4476,  18472, 14048,
                24308, 33472, 16668, 51520, 4772,  32180, 64736, 11124, 52772,
                24784, 54988, 13388, 61404, 43164, 52892, 15132, 30216, 60592,
                17384, 42012, 13084, 9028,  45876, 27764, 36200, 8192,  52524,
                29580, 924,   13488, 23124, 43620, 63000, 39444, 45580, 31900,
                56300, 50328, 38840, 20500, 33144, 52240, 41168, 23080, 64804,
                38160, 45480, 33892, 58148, 30412, 50132, 21128, 36092, 28160,
                62012, 31156,
            },
            {
                42168, 17772, 53408, 12404, 5484,  59104, 52184, 45404, 32092,
                4516,  48184, 19060, 56012, 172,   22560, 37972, 61356, 43468,
                15428, 1172,  30276, 59064, 4080,  34156, 9180,  26572, 58124,
                53940, 1812,  27384, 20584, 44840, 50056, 28940, 43572, 11508,
                37820, 33624, 21668, 1320,  18676, 10216, 32720, 3544,  7976,
                63284, 36768, 32164, 55700, 47484, 52576, 31572, 21464, 48668,
                39676, 29008, 5220,  23488, 27732, 10680, 34608, 46700, 904,
                32040, 22748, 37976, 58680, 2040,  53404, 62740, 28020, 39108,
                61072, 45424, 28972, 39796, 19348, 47256, 14308, 29564, 17564,
                3012,  33368, 7256,  30800, 18076, 23820, 40460, 11388, 51136,
                22968, 55536, 65200, 31968, 112,   61592, 43436, 16924, 32548,
                64656, 40804, 57800, 33784, 6520,  51704, 19580, 4940,  22124,
                34780, 15872, 4432,  62480, 1856,  36616, 17644, 27456, 2296,
                9784,  21864, 3532,  25052, 39724, 12300, 43528, 57648, 15452,
                8088,  21908,
            },
            {
                59740, 26740, 33120, 64196, 50036, 19508, 27376, 16740, 23444,
                60284, 7272,  26324, 34836, 62156, 30388, 11912, 52612, 7892,
                47580, 56356, 38984, 51740, 18564, 49564, 22912, 38112, 19792,
                11852, 34224, 41380, 59516, 31860, 14148, 24392, 17908, 65340,
                27048, 14408, 58872, 40928, 27868, 49992, 23128, 56940, 52260,
                14088, 22720, 888,   16524, 28388, 8748,  64860, 42224, 280,
                55364, 17096, 64048, 37520, 48524, 54760, 7580,  42036, 18436,
                56464, 50124, 11368, 44200, 25652, 47412, 32760, 43348, 7900,
                54892, 3168,  12556, 63372, 23748, 57464, 36564, 42200, 50696,
                62752, 39944, 45040, 59596, 48508, 5512,  35168, 44676, 1476,
                47180, 6080,  39512, 25216, 54104, 21264, 3880,  57280, 12076,
                5400,  20156, 44984, 16460, 60800, 11340, 29168, 47656, 65124,
                9464,  42812, 29956, 53752, 25888, 13664, 61184, 46456, 55556,
                51300, 42652, 14500, 54496, 46540, 63760, 4972,  32292, 40872,
                50392, 1988,
            },
            {
                46092, 10172, 6780,  22908, 42988, 37828, 8960,  63512, 36072,
                13144, 39228, 43848, 16380, 51120, 6576,  57668, 27424, 19668,
                32692, 23300, 5492,  26884, 13540, 61388, 504,   47824, 42596,
                63072, 7324,  51264, 17284, 3228,  55984, 60716, 6844,  51228,
                45764, 8788,  55528, 47836, 6964,  64340, 44772, 17752, 39648,
                29400, 43636, 49612, 60288, 38900, 24112, 57384, 15080, 26536,
                8024,  34272, 43100, 12928, 2120,  31000, 21884, 58176, 26824,
                5164,  15864, 34476, 64052, 6004,  15316, 22456, 10948, 51240,
                19944, 36444, 49672, 9104,  43404, 696,   6236,  26736, 8720,
                58036, 22256, 16016, 812,   56164, 26892, 53740, 63584, 20712,
                28592, 35768, 15536, 48280, 10792, 34812, 40424, 24540, 47120,
                37136, 26960, 53704, 3004,  31076, 37624, 55872, 15092, 52660,
                27092, 58168, 18900, 47464, 10308, 44004, 57344, 4852,  30368,
                35452, 18804, 62376, 28460, 8856,  17016, 24160, 55316, 19204,
                63200, 34884,
            },
            {
                38400, 57932, 48364, 29108, 14544, 3600,  47224, 55108, 2212,
                28520, 53640, 64608, 10544, 24592, 46096, 36472, 3116,  39992,
                54460, 10220, 64088, 35760, 44336, 30972, 56444, 16056, 4968,
                29640, 23280, 38456, 46972, 9044,  36728, 40516, 32868, 412,
                23036, 39736, 16776, 35080, 30756, 13112, 1860,  34060, 61304,
                10472, 54916, 4120,  34644, 11440, 46036, 3704,  36164, 50720,
                62220, 22296, 52564, 19280, 60776, 44732, 14524, 64436, 37272,
                47952, 61248, 29060, 19512, 54756, 36880, 57784, 384,   65532,
                31548, 26080, 59520, 15684, 33564, 54552, 61608, 20332, 48816,
                35444, 11776, 28908, 38136, 19896, 10148, 32460, 13860, 57240,
                8596,  61492, 19588, 59188, 29676, 50916, 63440, 19052, 55788,
                50296, 9484,  59656, 22064, 48928, 41932, 1468,  24728, 36112,
                3588,  40108, 7552,  63640, 31608, 21492, 39576, 8100,  24664,
                12536, 57708, 38976, 5700,  33544, 37652, 48060, 76,    11400,
                29368, 14428,
            },
            {
                2876,  19884, 54256, 35688, 61280, 57528, 31704, 21308, 41900,
                49556, 20036, 31360, 1732,  41544, 20928, 63724, 13096, 49956,
                60420, 15124, 46348, 20608, 7084,  40624, 10684, 36600, 52696,
                59960, 13560, 57520, 26236, 52996, 12488, 20992, 48388, 54956,
                29908, 63288, 3908,  24916, 60464, 53608, 37760, 25960, 5368,
                47036, 19324, 26512, 63748, 21220, 51676, 17940, 29768, 44400,
                11672, 31756, 4360,  57296, 25536, 35840, 51212, 3452,  10236,
                40996, 24412, 1744,  51096, 8988,  41976, 27412, 48160, 16136,
                40320, 5744,  46644, 21244, 29820, 51104, 38476, 14788, 31844,
                4020,  46076, 65356, 50888, 42272, 61696, 4096,  24596, 38196,
                42128, 52188, 2468,  43596, 5176,  14268, 8408,  31624, 1696,
                15524, 34544, 39908, 12860, 63464, 18640, 8760,  62000, 43884,
                13032, 50912, 23720, 36908, 500,   49964, 34204, 16692, 63040,
                47592, 1004,  49644, 22664, 58852, 51780, 27036, 61116, 44128,
                53060, 24868,
            },
            {
                64580, 41148, 11984, 5172,  17836, 39888, 25388, 11068, 61800,
                15688, 5532,  58280, 47932, 33672, 55240, 17112, 30824, 24948,
                42320, 4552,  29532, 57956, 53836, 23804, 65280, 27644, 20868,
                43584, 33108, 980,   19088, 31132, 63840, 27704, 43196, 10492,
                19212, 36356, 51924, 11168, 20896, 42604, 15936, 50932, 22220,
                57884, 41300, 8052,  31736, 42640, 5848,  61528, 40416, 2348,
                59452, 46856, 38156, 9856,  41244, 6364,  29292, 18880, 32872,
                53892, 13384, 45644, 38288, 31312, 12568, 60448, 34516, 23548,
                55696, 11860, 41860, 56768, 6652,  24492, 10776, 42824, 56952,
                25472, 53360, 7144,  23208, 14888, 35616, 46556, 50044, 17296,
                31104, 11728, 33936, 23052, 54960, 37736, 46248, 26060, 60732,
                44124, 29184, 6884,  24348, 45376, 28088, 33628, 57160, 20924,
                32020, 61040, 17160, 54644, 14416, 59992, 27080, 52908, 42204,
                20140, 31680, 10640, 44292, 13508, 3604,  20564, 40048, 34200,
                7116,  49288,
            },
            {
                32596, 27316, 44448, 23684, 51080, 1368,  45836, 52680, 7636,
                34868, 38436, 25936, 12876, 4300,  61164, 8480,  45048, 732,
                33796, 18312, 37796, 48644, 1952,  17092, 33700, 50164, 3612,
                8924,  49016, 61496, 45320, 5648,  56592, 2732,  15792, 60224,
                5352,  58196, 41288, 47436, 2536,  57128, 8212,  30212, 65216,
                12932, 36740, 15744, 53468, 58260, 13532, 25784, 54560, 20096,
                24324, 14704, 28492, 50240, 16672, 62656, 55880, 47416, 22344,
                60120, 7372,  57532, 20944, 63012, 2644,  18368, 45008, 4480,
                28748, 61560, 1364,  35484, 64576, 17388, 47524, 2356,  62636,
                17996, 37628, 32820, 2084,  58216, 28448, 7860,  59340, 908,
                55348, 26272, 47108, 64692, 28072, 18112, 58004, 41240, 5564,
                20980, 65480, 51060, 57604, 5024,  53400, 15876, 49584, 6024,
                46752, 2720,  29528, 45084, 41364, 6452,  11092, 56856, 3080,
                35976, 26576, 60064, 53972, 29256, 65492, 15680, 9608,  58640,
                22544, 17056,
            },
            {
                59912, 8784,  56200, 30712, 62984, 34324, 13924, 29456, 43548,
                23308, 60440, 54132, 40712, 28720, 51832, 22788, 38752, 58312,
                53480, 63120, 12272, 26472, 9720,  60612, 45904, 13152, 55884,
                25604, 39780, 15336, 35200, 41324, 24288, 49748, 38508, 33932,
                45144, 26208, 15048, 28976, 33556, 61832, 45248, 39716, 56,
                32540, 50148, 3164,  28144, 46716, 38020, 9908,  34420, 48640,
                6916,  56696, 65344, 1616,  33844, 23672, 11104, 42800, 60,
                35592, 30056, 15772, 43340, 25496, 55356, 49276, 8136,  53340,
                37928, 18768, 49880, 13796, 27744, 39836, 53100, 30640, 21780,
                8952,  59924, 12836, 44464, 54720, 18156, 40164, 22032, 43924,
                62420, 7344,  15776, 40608, 3508,  10952, 51808, 33184, 13372,
                54168, 37904, 17676, 31564, 856,   36452, 40596, 11036, 26484,
                38532, 58820, 10460, 25804, 65020, 20612, 30828, 45412, 15292,
                64028, 40140, 7396,  17576, 41964, 36260, 47304, 31144, 54768,
                3944,  37884,
            },
            {
                46824, 596,   16116, 47940, 6656,  19492, 54712, 65372, 4532,
                17428, 46196, 9040,  19796, 15088, 35656, 48960, 11572, 27808,
                7508,  21976, 51416, 43328, 31944, 38608, 6016,  41632, 18752,
                62964, 30068, 22276, 10996, 52320, 17844, 8556,  62392, 21892,
                12320, 50668, 7584,  55448, 18296, 23244, 10844, 27268, 19556,
                54780, 43228, 63100, 18336, 22772, 4304,  30408, 63604, 16796,
                39100, 32340, 43388, 20852, 46240, 52904, 38960, 15236, 61792,
                26108, 48052, 52620, 6484,  33704, 39828, 22072, 14196, 63892,
                42884, 31800, 22876, 44816, 8496,  59808, 5256,  36328, 45508,
                51372, 40736, 29420, 49220, 25904, 10464, 64516, 30312, 13108,
                35020, 20384, 50860, 57512, 36204, 21876, 62468, 440,   24092,
                46704, 8200,  11472, 42668, 47800, 60084, 22144, 54580, 64396,
                19116, 35300, 52024, 4824,  33664, 48764, 38460, 23564, 50180,
                4980,  52404, 24328, 33768, 1268,  56824, 6124,  25368, 42760,
                51224, 13484,
            },
            {
                21396, 42404, 35972, 58840, 39256, 10068, 26172, 37000, 57812,
                32452, 50048, 572,   62836, 57016, 42480, 5584,  65132, 16164,
                46920, 35188, 4192,  61732, 19596, 52360, 28172, 58512, 35636,
                2248,  50928, 6984,  58796, 64944, 32108, 46336, 28440, 53868,
                824,   31888, 64824, 39056, 5236,  35932, 52800, 47240, 59720,
                7316,  23964, 11156, 35472, 59484, 51112, 56208, 42004, 480,
                53232, 11184, 25836, 8144,  59384, 3984,  27792, 56664, 9068,
                37872, 18576, 65064, 3284,  11140, 59364, 30244, 36424, 26024,
                10520, 2752,  60752, 51936, 33096, 20408, 56268, 12192, 27440,
                496,   16388, 63412, 20476, 3056,  36956, 47408, 4784,  53620,
                38756, 2844,  29476, 9100,  48660, 44376, 30240, 16936, 59692,
                35140, 27572, 61580, 20208, 25500, 14240, 6960,  29924, 1804,
                43280, 13576, 23100, 55976, 17952, 2304,  59648, 8944,  29208,
                19244, 12948, 46780, 61988, 21764, 50520, 12048, 62968, 18588,
                29096, 61740,
            },
            {
                25832, 53592, 11676, 28396, 22600, 49528, 41928, 1948,  21348,
                11432, 26876, 37412, 30516, 24036, 2944,  32912, 20480, 55308,
                30360, 40752, 14556, 24268, 56624, 164,   15476, 23180, 10100,
                46488, 54724, 37844, 27548, 1248,  13812, 36324, 4244,  42212,
                58752, 19536, 24368, 43892, 49544, 14060, 62780, 3724,  16264,
                38120, 29396, 49148, 1316,  40276, 7856,  20656, 13952, 27884,
                61992, 18664, 51044, 36852, 13364, 30952, 19096, 44976, 4764,
                50404, 12424, 40708, 28684, 46236, 17328, 51320, 5628,  58528,
                48072, 16336, 55452, 4808,  25728, 42240, 15972, 63996, 23484,
                58340, 34340, 6008,  43028, 57152, 33352, 51896, 15940, 24172,
                59124, 45572, 61724, 25172, 14136, 5092,  39744, 49796, 55428,
                42176, 2444,  53552, 49532, 34440, 63312, 44720, 33208, 58128,
                48192, 8160,  39376, 62268, 44148, 14900, 54052, 34940, 61168,
                43260, 55816, 37688, 9944,  28340, 16288, 45156, 36856, 1932,
                10184, 34348,
            },
            {
                4904,  64808, 18748, 3340,  61104, 55812, 15240, 46804, 51980,
                60668, 15752, 44356, 53328, 12480, 47220, 60416, 25344, 43464,
                1356,  58756, 49628, 8444,  35968, 47268, 31100, 63616, 42568,
                32464, 14340, 18160, 43480, 48044, 20640, 55944, 25012, 16392,
                37440, 48292, 8968,  56476, 1872,  30632, 41744, 25844, 33488,
                56288, 44572, 61768, 15288, 32660, 46212, 25284, 44096, 36236,
                47264, 3484,  55920, 40980, 64132, 48760, 34856, 62808, 22128,
                32236, 57880, 23600, 35176, 53792, 62280, 1180,  44284, 19848,
                34452, 28024, 39472, 12628, 36912, 48472, 3212,  31220, 50100,
                37932, 10708, 54424, 24796, 14004, 8352,  61360, 27876, 41404,
                11108, 32448, 17572, 52828, 34632, 64300, 26928, 10240, 6560,
                21524, 31176, 13120, 4108,  39200, 9448,  18364, 52280, 23952,
                16276, 28412, 50072, 31264, 7216,  27272, 41216, 11996, 244,
                32156, 26116, 4004,  58600, 41156, 32780, 53648, 22948, 58236,
                49112, 41056,
            },
            {
                8372,  30772, 51664, 43760, 33388, 5824,  31116, 8612,  35312,
                40796, 3656,  64408, 7692,  39564, 17708, 51232, 6704,  37568,
                10372, 19184, 27008, 64588, 39940, 11856, 53876, 6432,  19972,
                60096, 4028,  24356, 57148, 9372,  39684, 60868, 51752, 6788,
                62004, 13396, 34656, 27752, 17580, 58956, 21536, 8668,  51624,
                12468, 5168,  20100, 26732, 52424, 65012, 11612, 59892, 6308,
                22880, 33552, 29464, 16176, 988,   24396, 7204,  53512, 16340,
                42484, 1460,  61024, 20068, 7740,  13804, 24840, 40976, 56264,
                8300,  65164, 45576, 19020, 62548, 9652,  54744, 43700, 7036,
                19524, 47020, 30248, 60444, 39964, 22288, 44380, 19100, 180,
                54752, 6888,  22000, 42504, 2484,  56176, 19992, 14972, 45496,
                38512, 63728, 16748, 56872, 29036, 50584, 5252,  37340, 11428,
                41384, 60540, 944,   20196, 36776, 58308, 47172, 22112, 51968,
                64856, 15860, 21192, 48556, 7536,  64336, 3128,  39660, 31380,
                15384, 55104,
            },
            {
                37360, 24104, 47376, 12680, 38060, 18020, 63780, 24252, 28480,
                56340, 18952, 33536, 22444, 58936, 28960, 34968, 14436, 62188,
                52860, 45416, 33720, 3036,  17628, 43908, 26012, 50244, 38664,
                27888, 49184, 34980, 63064, 29616, 5008,  33696, 11280, 30932,
                45948, 22284, 41144, 63504, 11464, 53932, 36620, 46800, 64672,
                23072, 41360, 58396, 9424,  37620, 18080, 4464,  30292, 57088,
                50248, 9460,  61228, 21692, 44108, 58192, 11364, 37156, 29136,
                55496, 10208, 48172, 43888, 27776, 38520, 32476, 50196, 11824,
                22424, 31128, 32,    52412, 23596, 28616, 33960, 59620, 14100,
                40868, 52816, 16928, 1624,  48372, 31640, 5560,  50204, 64364,
                35364, 48200, 57984, 12292, 30676, 37648, 50984, 60828, 33492,
                57848, 25140, 47084, 41848, 22612, 58880, 26568, 62416, 53948,
                4544,  34872, 55328, 12516, 63524, 24400, 4284,  18148, 38128,
                7792,  46008, 35276, 54672, 14020, 19188, 26432, 6504,  60756,
                20308, 44536,
            },
            {
                57176, 16444, 62176, 2308,  26908, 58412, 44880, 52924, 13228,
                5536,  47628, 26068, 49676, 1084,  9676,  45756, 56156, 31536,
                22904, 12952, 59956, 29680, 55752, 21936, 61488, 2400,  13016,
                55400, 8132,  40652, 12528, 22528, 50340, 44092, 18820, 26856,
                55324, 1412,  5976,  50900, 32564, 44420, 3244,  18548, 29620,
                1568,  35104, 53568, 31184, 2788,  48572, 54544, 39152, 15716,
                41596, 12816, 46428, 6036,  38788, 51484, 26620, 47320, 2996,
                39344, 25288, 15648, 5324,  56436, 63708, 18304, 3172,  60156,
                37528, 48684, 6868,  57872, 40520, 16868, 1996,  21660, 26428,
                64752, 3780,  28184, 35912, 63264, 12360, 56908, 37916, 29020,
                14512, 25636, 38928, 63016, 6108,  47192, 23836, 1008,  28636,
                8884,  4604,  35264, 11276, 540,   45816, 14064, 31004, 21036,
                47272, 25976, 17420, 45240, 52196, 9968,  30720, 48948, 57484,
                28768, 40444, 1980,  60348, 29704, 42916, 51824, 46744, 12548,
                28056, 1528,
            },
            {
                10272, 34460, 21152, 41584, 54676, 9736,  20600, 160,   42704,
                62340, 37216, 10676, 53928, 42148, 63608, 20040, 26596, 2416,
                41284, 6048,  38192, 51380, 4796,  10044, 32364, 36528, 16420,
                65240, 45224, 1580,  53436, 16940, 58656, 2932,  64448, 38848,
                49320, 16092, 59664, 25244, 38616, 7348,  14952, 60624, 39480,
                50160, 14220, 45192, 61308, 22420, 42920, 34088, 24788, 20412,
                63900, 27608, 35216, 55004, 18504, 32428, 14496, 59936, 19824,
                64636, 52168, 30916, 36384, 51032, 9796,  46564, 29344, 53744,
                15924, 26464, 43140, 13884, 35556, 61276, 51308, 44936, 36820,
                9992,  45968, 58816, 21148, 9132,  52156, 17144, 23156, 3252,
                46000, 8736,  19372, 51656, 27724, 16980, 11116, 43112, 52844,
                19024, 48940, 54920, 20488, 64568, 33112, 40016, 2724,  43056,
                9108,  65396, 32708, 39812, 2604,  42336, 34376, 61636, 13556,
                23620, 10452, 51048, 33072, 24244, 11008, 36688, 56032, 33712,
                64076, 50380,
            },
            {
                25356, 59848, 6672,  49136, 14540, 35712, 50460, 39244, 23016,
                29588, 58248, 16964, 32012, 13912, 37956, 4880,  50272, 17176,
                63104, 47884, 24984, 15600, 46220, 41736, 58784, 48344, 23900,
                30260, 20440, 33320, 26320, 46324, 36440, 13992, 23892, 8044,
                35348, 43008, 29792, 19408, 57004, 23168, 48920, 27096, 57660,
                10124, 24876, 17240, 7160,  28276, 13544, 62184, 664,   8124,
                52392, 3860,  60148, 24164, 1892,  62720, 8996,  43252, 6492,
                33832, 12784, 58532, 21620, 892,   42048, 23436, 35008, 4944,
                62248, 20732, 55224, 31908, 8824,  47864, 5800,  12652, 56532,
                32028, 23968, 5436,  39460, 43456, 26548, 34252, 59728, 42324,
                55300, 61128, 33092, 1464,  44552, 59792, 36272, 65232, 40176,
                14200, 61676, 29544, 38172, 52376, 8092,  18196, 55028, 59708,
                15332, 50560, 5752,  21920, 28092, 56556, 16548, 6032,  53112,
                43564, 62496, 17048, 5796,  45440, 63092, 8268,  17344, 22092,
                5016,  42156,
            },
            {
                53108, 38860, 31416, 28192, 65192, 3776,  31848, 60648, 15832,
                7756,  45100, 4256,  21788, 55472, 28248, 60600, 43600, 36400,
                9956,  58144, 20304, 34820, 63928, 27552, 18652, 6752,  52820,
                42912, 10532, 56380, 62424, 7544,  31304, 41120, 52668, 56708,
                10932, 62972, 3448,  47132, 9852,  64252, 31684, 42032, 5916,
                33868, 63944, 38640, 56608, 50840, 10312, 55596, 46896, 36696,
                43772, 30860, 16968, 49208, 42316, 29560, 36128, 56912, 23800,
                45516, 4056,  41072, 61656, 27024, 16436, 57440, 13448, 44232,
                39500, 10672, 2540,  64204, 18716, 24280, 29648, 39380, 62228,
                16544, 53964, 49772, 14572, 55560, 1068,  47260, 7064,  11496,
                31068, 16108, 41080, 22956, 57020, 7484,  21564, 31296, 3840,
                24732, 44204, 6472,  16080, 26096, 60692, 48144, 35924, 28400,
                24032, 37984, 58544, 13332, 62708, 45808, 25412, 37268, 1188,
                20316, 27388, 55200, 39032, 21052, 57364, 1144,  40316, 48832,
                29892, 14172,
            },
            {
                18964, 872,   56076, 45044, 17484, 24804, 46548, 11292, 55764,
                33740, 51440, 64884, 40528, 48416, 8196,  24028, 12552, 32424,
                54788, 29348, 7556,  53136, 2028,  12504, 56008, 39856, 836,
                59556, 14784, 38500, 3824,  49972, 19284, 60112, 84,    28840,
                21472, 33316, 13704, 54716, 35720, 620,   52288, 12752, 19312,
                54456, 47184, 1152,  32668, 20268, 39904, 29964, 18176, 26252,
                59000, 10168, 39276, 13328, 6856,  53976, 20872, 50864, 15888,
                28672, 49412, 18536, 11044, 33228, 48444, 64948, 8120,  52448,
                32656, 50416, 28060, 45344, 41264, 59480, 52608, 716,   20460,
                7588,  34896, 28988, 61760, 19092, 32216, 65512, 39028, 21044,
                52444, 4084,  48440, 12916, 34664, 53616, 15304, 48128, 56168,
                10732, 59148, 32568, 2264,  42296, 12808, 22864, 4240,  10968,
                46660, 692,   30176, 53188, 7288,  19704, 10828, 60004, 31752,
                47556, 36172, 3060,  49240, 14376, 31208, 52428, 24988, 60884,
                35128, 46480,
            },
            {
                62728, 12620, 22540, 9228,  40792, 5724,  52244, 19256, 27156,
                1888,  36668, 24920, 18516, 1372,  35396, 57936, 51760, 21196,
                404,   44712, 14532, 39216, 30988, 22580, 35836, 9592,  26120,
                33852, 47956, 28660, 23064, 43292, 11884, 25564, 47992, 16580,
                45364, 51276, 26244, 40852, 17152, 44876, 21516, 37488, 3576,
                43332, 22280, 26680, 6300,  58692, 44404, 3248,  64480, 14260,
                50524, 22376, 57036, 46760, 65376, 25612, 11592, 40148, 184,
                62932, 37036, 53396, 6772,  55724, 38852, 3472,  28936, 24572,
                17408, 58668, 22232, 36648, 4032,  15308, 34044, 26540, 43696,
                48672, 40824, 2360,  10568, 37212, 24440, 13408, 49464, 28244,
                61964, 25504, 37324, 64148, 29056, 42416, 2776,  26904, 38612,
                34840, 21124, 50220, 46396, 57632, 30208, 51716, 62136, 40764,
                56344, 18740, 43864, 35152, 48784, 38740, 55604, 50412, 41272,
                8216,  65128, 12112, 58540, 34600, 26940, 43972, 11588, 3796,
                57752, 8056,
            },
            {
                43524, 36804, 50964, 61244, 33032, 57560, 37880, 62856, 43560,
                48864, 9324,  13288, 60980, 31136, 45700, 16216, 6468,  40696,
                65384, 26240, 49848, 58952, 42848, 62208, 46552, 51092, 64380,
                19952, 5644,  61188, 16472, 54176, 64852, 34592, 5396,  37748,
                61900, 6760,  58636, 4356,  30588, 60888, 26784, 56808, 63216,
                30448, 13692, 60564, 49472, 15960, 35092, 23908, 53464, 6068,
                34412, 1312,  32508, 19196, 37320, 2924,  33504, 46264, 60072,
                8292,  24844, 43360, 30152, 22476, 45564, 19652, 37108, 60436,
                1384,  46632, 7512,  12508, 57540, 49236, 10160, 65068, 54632,
                12984, 22964, 57628, 44968, 51748, 58776, 3580,  17404, 44056,
                9376,  57316, 18248, 5188,  10264, 50092, 19616, 62928, 7720,
                52212, 17164, 63764, 9624,  36644, 19140, 7328,  34164, 16008,
                31796, 63256, 10004, 26308, 15232, 2052,  28452, 22556, 13652,
                17804, 29800, 24636, 42268, 4212,  18708, 64228, 37716, 16484,
                20964, 27940,
            },
            {
                32124, 2884,  25884, 47356, 14108, 29076, 2572,  23332, 14768,
                59388, 29204, 52920, 42556, 56088, 10804, 62332, 28676, 47716,
                17856, 36684, 10912, 5444,  19400, 3400,  16036, 29664, 13256,
                44528, 52324, 35716, 9216,  39844, 2032,  29864, 57300, 41960,
                9636,  18912, 36572, 23832, 49216, 10972, 7208,  48140, 16624,
                9552,  53028, 37128, 41764, 8780,  61372, 11820, 48040, 39620,
                42680, 60932, 8572,  28588, 51864, 59044, 17792, 54740, 31400,
                20432, 14628, 58080, 1920,  13184, 61224, 9772,  54708, 42820,
                14252, 34572, 62628, 54316, 30472, 23440, 38620, 17740, 3516,
                31764, 62784, 16992, 27332, 8168,  41652, 30064, 55044, 34764,
                432,   40128, 46340, 54396, 24340, 32936, 59536, 45328, 12820,
                41888, 88,    28008, 24432, 40360, 3512,  55396, 44512, 25548,
                51024, 6416,  21792, 59304, 42456, 64464, 33336, 5704,  60972,
                56508, 45384, 52760, 7132,  60100, 47704, 8864,  55520, 49744,
                41108, 53604,
            },
            {
                64708, 15900, 59172, 7048,  21072, 53524, 10340, 34744, 40984,
                5104,  20360, 38032, 2896,  26504, 22368, 39104, 34048, 3684,
                55204, 23344, 61420, 33436, 54608, 25352, 57564, 7172,  41116,
                23856, 2748,  56648, 31364, 46960, 19676, 51888, 14056, 22076,
                54264, 31612, 44024, 65416, 14936, 55312, 34728, 39540, 23420,
                45500, 33292, 2696,  18660, 28640, 54832, 31824, 20708, 27444,
                17596, 55160, 24472, 44668, 14796, 41524, 4724,  26420, 10600,
                35732, 47876, 64280, 39988, 34448, 49012, 26196, 31440, 5928,
                50004, 40324, 27020, 20164, 42972, 5840,  46416, 60344, 28512,
                37024, 46868, 5124,  33944, 20880, 63676, 5832,  22984, 50808,
                14912, 31604, 21736, 60452, 36232, 1620,  16032, 29976, 22976,
                58612, 31668, 53924, 14188, 61268, 48424, 22216, 64540, 13376,
                2772,  47056, 39752, 54700, 3928,  18468, 52256, 43556, 35360,
                420,   21276, 32680, 15824, 36752, 22760, 30396, 632,   33404,
                23792, 5640,
            },
            {
                45380, 19608, 39572, 35212, 42024, 63848, 49588, 18008, 56600,
                45568, 63472, 33160, 16860, 50300, 5504,  53372, 13920, 59900,
                8948,  41920, 15448, 28916, 45112, 37096, 48544, 32412, 60196,
                38080, 18044, 26792, 12880, 59264, 24752, 8020,  63208, 26752,
                48320, 12768, 3548,  52104, 20688, 42352, 1676,  28792, 61748,
                4896,  57684, 25568, 65096, 46604, 96,    38100, 7428,  62812,
                4024,  48680, 11416, 63812, 6232,  23088, 49480, 63400, 43656,
                3064,  51816, 27324, 18036, 5264,  52876, 16000, 63180, 19108,
                24056, 10528, 52248, 16076, 1028,  32964, 56116, 7800,  21164,
                52048, 11216, 55736, 39040, 48856, 12228, 46152, 37004, 58076,
                62700, 27516, 7176,  12464, 43236, 51960, 39784, 55964, 4320,
                37296, 49428, 5868,  44236, 33124, 16856, 29360, 9412,  37224,
                57304, 28004, 34400, 11928, 31240, 23872, 47640, 10284, 26528,
                39696, 49308, 10960, 63140, 40476, 51492, 12576, 45860, 61856,
                14488, 10260,
            },
            {
                50264, 28224, 54512, 11664, 320,   44624, 26276, 3696,  31488,
                24428, 11960, 58508, 8660,  44276, 64384, 19072, 25400, 45032,
                30748, 49128, 1784,  52344, 7896,  12248, 21664, 268,   53048,
                11164, 63560, 49840, 42396, 3992,  37524, 45020, 33656, 40512,
                972,   60712, 28472, 38700, 6228,  32332, 59540, 13136, 49396,
                19924, 40784, 12352, 51124, 22100, 14352, 57928, 51632, 45060,
                14948, 36296, 30468, 39004, 56748, 34976, 29684, 12980, 37696,
                56244, 7252,  32736, 23432, 57444, 41696, 408,   44688, 58488,
                33572, 4392,  64596, 37244, 59016, 49596, 25768, 40860, 14844,
                43048, 92,    61124, 25524, 15596, 59920, 32072, 18400, 2808,
                11024, 41112, 49724, 17104, 64600, 6376,  25620, 10816, 46288,
                18352, 65108, 21424, 11468, 59060, 1484,  39408, 53156, 43088,
                20088, 16360, 61692, 50016, 7940,  38372, 56884, 15948, 63804,
                53852, 5288,  25656, 57084, 2560,  27996, 19768, 58324, 26508,
                35648, 57064,
            },
            {
                31120, 4204,  24072, 58436, 16812, 30400, 60672, 38552, 54296,
                7240,  51560, 27964, 48132, 35544, 30424, 41164, 752,   57656,
                21100, 35012, 63884, 18908, 58536, 41516, 62564, 16904, 28100,
                45608, 7868,  32764, 20968, 61956, 16148, 50612, 4952,  18320,
                55732, 35696, 16604, 57816, 25672, 46888, 17468, 52840, 8616,
                36796, 31192, 55440, 6976,  35844, 43544, 25200, 10232, 33444,
                59200, 21480, 53880, 984,   19472, 8880,  46724, 59348, 21672,
                16752, 40912, 12208, 47188, 9092,  36680, 29520, 12100, 38332,
                55156, 47740, 28328, 44120, 9768,  19036, 12308, 35448, 64008,
                24076, 30960, 19604, 53488, 2464,  28156, 9176,  43824, 25816,
                53724, 46876, 33924, 56828, 28788, 20556, 48408, 62124, 34520,
                27432, 8208,  40564, 35500, 55180, 26676, 50128, 6588,  31896,
                59456, 300,   24808, 44488, 14320, 62616, 29432, 6552,  20212,
                31284, 13728, 44020, 18188, 34860, 54576, 43012, 7300,  38656,
                2640,  42356,
            },
            {
                62320, 8564,  47560, 33648, 51496, 22324, 9232,  13916, 46900,
                16556, 40232, 1156,  21428, 13616, 60524, 10116, 51432, 37276,
                12420, 4412,  27140, 38632, 24688, 50508, 34240, 5924,  56148,
                22688, 35936, 1824,  54388, 28804, 9820,  57460, 30820, 11284,
                24096, 45736, 7928,  50400, 10772, 62776, 22916, 43472, 27356,
                63856, 3220,  44820, 17652, 30052, 63160, 40308, 19216, 28828,
                2964,  41680, 25364, 49940, 62132, 15908, 53064, 2388,  25512,
                62532, 30132, 54100, 60204, 20544, 65208, 25096, 50648, 7592,
                21248, 14908, 2392,  23176, 30760, 61952, 51688, 3076,  57944,
                48304, 8904,  44744, 36068, 41436, 49632, 56568, 65032, 38496,
                20076, 4776,  23648, 1052,  38312, 32556, 14072, 2812,  53388,
                15256, 57132, 24320, 46580, 4596,  19464, 62908, 23700, 11656,
                47460, 35984, 5732,  52664, 19908, 42116, 1344,  36484, 46140,
                58480, 38580, 61236, 47616, 4644,  10388, 65496, 15220, 48264,
                22300, 18088,
            },
            {
                55228, 37040, 14656, 64308, 39316, 5888,  57212, 36168, 65312,
                22844, 34136, 61620, 43052, 54864, 4848,  27544, 17188, 47540,
                61784, 53532, 44168, 9744,  14292, 3260,  30380, 43488, 38952,
                48888, 15020, 58744, 39948, 47476, 23392, 36556, 64932, 52540,
                42000, 62096, 21032, 33408, 40184, 348,   36248, 5556,  58172,
                15364, 23580, 50336, 10596, 59240, 1992,  5776,  56060, 47680,
                64748, 7784,  13100, 32416, 43204, 27652, 40312, 33996, 6548,
                49852, 43868, 1396,  34964, 14820, 45976, 3296,  18204, 62232,
                31856, 41664, 60584, 53052, 7000,  39644, 45680, 27956, 6204,
                33296, 17172, 62900, 12708, 6020,  21532, 33372, 6664,  13200,
                30320, 58768, 62948, 10456, 43740, 55492, 59896, 41460, 30332,
                44576, 956,   61008, 30864, 13584, 41892, 36536, 15560, 54532,
                28704, 65212, 39896, 56104, 32524, 26920, 60300, 50988, 23084,
                9312,  1844,  28668, 21968, 32432, 50360, 24644, 30264, 59644,
                52116, 11892,
            },
            {
                25300, 45088, 20168, 1556,  27728, 48976, 18580, 43724, 2064,
                29148, 56968, 6160,  18488, 24780, 38824, 56836, 33596, 23496,
                7056,  15892, 31840, 55924, 47352, 65072, 19936, 59468, 12184,
                25304, 64344, 19168, 5120,  13548, 43872, 1340,  19988, 27592,
                14140, 2424,  30032, 54060, 14896, 27988, 55800, 19424, 33660,
                47780, 38604, 61472, 27004, 33840, 53180, 23772, 15804, 37900,
                52108, 18024, 46112, 60252, 3632,  11088, 20576, 45772, 58156,
                10336, 19048, 27804, 5108,  51684, 30884, 42636, 56468, 26852,
                46784, 10712, 35504, 57244, 17664, 33736, 22268, 15464, 53784,
                38076, 56456, 29384, 50936, 59076, 24480, 40196, 17204, 51984,
                44920, 35400, 15556, 50068, 18848, 6720,  24424, 9208,  20952,
                37836, 49752, 17796, 52736, 9456,  48704, 59784, 2568,  44808,
                8204,  21444, 17000, 10720, 3500,  47752, 12852, 17108, 33836,
                41852, 53228, 15572, 37176, 62404, 41412, 17744, 1504,  34052,
                6736,  40720,
            },
            {
                3136,  32100, 60192, 41768, 11348, 54184, 32744, 25924, 12444,
                52492, 48368, 10384, 31776, 51068, 45132, 2664,  64920, 43200,
                29104, 39928, 60688, 22656, 35828, 7628,  27344, 53452, 2432,
                9292,  31972, 51696, 27328, 34536, 60940, 49540, 6956,  39188,
                47756, 59496, 5476,  44444, 64428, 48824, 41984, 12188, 52308,
                9480,  1304,  20512, 41940, 13048, 48880, 45268, 31992, 11668,
                26372, 34540, 56964, 22596, 36844, 55016, 65112, 31064, 14184,
                36372, 39584, 63792, 24256, 57824, 37616, 6464,  13564, 52596,
                39136, 4844,  25484, 48404, 12592, 1132,  65248, 41904, 10408,
                20940, 1432,  26248, 4196,  15296, 46168, 54828, 604,   61896,
                25180, 7996,  40828, 54272, 27916, 46768, 35652, 51332, 64208,
                12224, 5312,  34032, 27748, 64468, 21848, 32768, 27000, 40740,
                58332, 34160, 49776, 25428, 58780, 37780, 7772,  54232, 64872,
                25164, 49608, 5616,  56044, 12364, 7864,  54152, 37812, 46504,
                63800, 28692,
            },
            {
                16736, 51692, 7812,  47064, 23940, 63008, 4308,  59248, 20784,
                36972, 41152, 15488, 63376, 35684, 7992,  14728, 20292, 11084,
                50280, 4928,  18672, 1112,  51956, 41320, 15176, 46664, 37236,
                57424, 44796, 40952, 8388,  53420, 17496, 31280, 56620, 10252,
                34068, 25168, 37460, 18960, 8356,  24508, 3540,  60844, 29212,
                44396, 31780, 56784, 7276,  36652, 19576, 58676, 8732,  61324,
                648,   40688, 4760,  30228, 8180,  17516, 48328, 264,   23884,
                52684, 55840, 15984, 48092, 11148, 21384, 60916, 34012, 756,
                16852, 64124, 20296, 43788, 29284, 50588, 58912, 25324, 47068,
                60636, 43676, 39532, 34808, 64012, 31196, 9764,  36604, 28424,
                49092, 3384,  21708, 31512, 12388, 61440, 4488,  17040, 27184,
                58492, 23284, 43324, 38856, 3320,  56544, 6940,  18208, 52032,
                13512, 1220,  30156, 42932, 63308, 22516, 44972, 28956, 2852,
                11704, 19620, 59628, 26380, 45560, 21260, 58548, 26704, 13456,
                20628, 56320,
            },
            {
                34604, 58296, 13580, 29848, 36012, 15836, 39768, 45888, 7660,
                62244, 3720,  26600, 46496, 22080, 53508, 30280, 58368, 36720,
                54884, 26004, 45600, 57480, 11220, 32756, 23824, 60628, 18064,
                28736, 21680, 528,   62652, 24944, 12016, 42388, 22888, 63784,
                15812, 51000, 57184, 11524, 32284, 59012, 38232, 22140, 14700,
                65180, 25408, 53440, 16408, 63468, 2668,  28176, 43244, 21956,
                49652, 63496, 14924, 53140, 44208, 38440, 26828, 60796, 42656,
                8624,  3124,  32844, 44228, 7068,  41276, 28832, 49660, 23472,
                58664, 31124, 7948,  37876, 55936, 4352,  35380, 7780,  31148,
                13348, 52284, 18072, 48812, 7180,  20652, 41996, 60384, 19192,
                11192, 56220, 64536, 37536, 1520,  56652, 39496, 32496, 46024,
                53476, 8016,  61556, 15604, 45028, 12804, 47652, 38256, 23316,
                63904, 46408, 54032, 15148, 4504,  34660, 18644, 56796, 31864,
                39476, 44436, 35372, 30188, 328,   48936, 32232, 4472,  43240,
                9896,  49032,
            },
            {
                6244,  38432, 21676, 61400, 2652,  51156, 10140, 28436, 55500,
                16932, 31200, 57556, 256,   12180, 60396, 41332, 47196, 1548,
                32972, 62760, 13600, 37692, 29636, 63324, 3628,  42928, 6076,
                51084, 35736, 16224, 45928, 58616, 37508, 5548,  54336, 29040,
                45612, 2988,  21840, 41428, 53380, 16896, 47024, 34948, 7572,
                39916, 4568,  46516, 23112, 50624, 40484, 5064,  35164, 54112,
                18632, 28608, 46812, 24672, 58968, 12256, 6000,  51088, 21216,
                29652, 59908, 19612, 26728, 62260, 54200, 18328, 9540,  45484,
                40768, 54412, 14528, 61196, 23020, 16712, 40552, 19164, 55368,
                5464,  23852, 61600, 11252, 56132, 25880, 51736, 14152, 46588,
                34064, 42836, 16240, 52464, 25984, 43820, 22552, 14356, 2904,
                41036, 30236, 19956, 55488, 24912, 34816, 29424, 61448, 10352,
                4900,  36276, 20396, 9404,  39860, 52000, 13360, 61436, 7044,
                48592, 62996, 9728,  17324, 39000, 64572, 15116, 52356, 60480,
                40040, 24264,
            },
            {
                556,   44136, 53560, 18344, 42448, 56604, 22924, 33724, 49140,
                43076, 23588, 50712, 39064, 18260, 27452, 6128,  24156, 17228,
                9060,  21440, 42160, 6740,  48148, 19660, 50316, 34108, 13688,
                64016, 10872, 55320, 4372,  32884, 48896, 19920, 2160,  40212,
                8708,  62008, 27192, 49156, 4792,  29940, 1184,  55480, 50972,
                18888, 58916, 33800, 10784, 29948, 60188, 25128, 56684, 13164,
                38036, 6196,  10392, 33864, 20368, 62508, 31904, 15380, 36040,
                46512, 11632, 50496, 35120, 1232,  14040, 36724, 63088, 3832,
                27196, 34652, 1756,  51388, 9936,  46312, 28488, 49984, 63620,
                37116, 33344, 2520,  29756, 44368, 37936, 1644,  5972,  57764,
                23312, 29644, 5488,  47104, 19772, 9724,  63732, 49172, 57704,
                11420, 36876, 176,   50992, 9296,  59412, 2132,  54680, 42184,
                31460, 26284, 57136, 60488, 28324, 47948, 780,   42120, 26808,
                15432, 22812, 4272,  51700, 42380, 24024, 8364,  35748, 19016,
                30628, 63232,
            },
            {
                27656, 47920, 9144,  26232, 31592, 6408,  64968, 19476, 12792,
                4964,  36028, 9408,  64612, 44264, 33948, 49924, 63652, 38936,
                52228, 59980, 27800, 54312, 15692, 58932, 9576,  27076, 56452,
                47588, 30832, 39900, 23116, 28048, 14876, 60696, 51400, 35680,
                17852, 33008, 13620, 36512, 63932, 43964, 26140, 62472, 11844,
                28380, 42768, 260,   37724, 13980, 44988, 17512, 9432,  47360,
                31392, 58204, 41716, 2000,  48964, 39776, 3528,  56128, 41556,
                64092, 5384,  39872, 57968, 23660, 47092, 30596, 56732, 21732,
                11600, 48820, 42108, 25668, 32848, 62368, 12344, 308,   26436,
                42756, 15260, 47396, 59028, 22196, 16084, 65516, 31692, 39840,
                8340,  62568, 36364, 13344, 60052, 33232, 6620,  29048, 18744,
                25260, 62756, 47160, 31720, 41624, 17012, 49332, 19872, 14492,
                51344, 44556, 16532, 6440,  32604, 24364, 55552, 20936, 37376,
                53020, 57588, 32796, 60244, 12244, 55188, 28564, 45340, 3440,
                54360, 12940,
            },
            {
                59356, 32792, 63836, 12200, 49552, 37120, 45400, 1012,  39428,
                61604, 54172, 15028, 29968, 4660,  55864, 10820, 14916, 29568,
                44480, 3932,  35384, 24548, 400,   39592, 21904, 36904, 2144,
                24336, 19000, 52080, 9000,  65500, 42924, 11356, 25756, 57912,
                44280, 54836, 59184, 6996,  20492, 15396, 9740,  38816, 21412,
                47860, 15976, 55256, 63940, 6344,  52956, 33236, 63024, 1380,
                51616, 21324, 64892, 54308, 17232, 27692, 52472, 9524,  25004,
                18172, 28416, 14884, 53164, 9004,  42020, 4888,  50896, 39080,
                15756, 64616, 18724, 6544,  56496, 37544, 43816, 58028, 9320,
                53996, 19628, 40000, 8456,  53356, 34960, 48448, 27016, 52888,
                17244, 49844, 380,   27300, 50768, 38768, 54080, 42540, 35692,
                52332, 15224, 5448,  22388, 65508, 28040, 36704, 7796,  24628,
                62552, 1444,  37656, 49916, 63172, 10936, 44684, 33820, 8828,
                2456,  29100, 45856, 19916, 36836, 2100,  61824, 21580, 50020,
                16192, 37300,
            },
            {
                22500, 17776, 40380, 2044,  58052, 15204, 25108, 59616, 51872,
                27580, 21340, 46764, 58996, 25336, 20348, 36580, 2188,  57596,
                19724, 12128, 47300, 65228, 31412, 45256, 52416, 61944, 41508,
                7200,  44388, 58032, 1164,  34736, 47344, 6872,  31472, 21140,
                80,    10764, 24528, 40060, 46728, 53896, 34344, 58184, 5340,
                32388, 61272, 22520, 26788, 48744, 20256, 39516, 23348, 42536,
                15636, 25956, 35672, 12496, 7764,  44344, 21768, 34128, 58620,
                1532,  48872, 32400, 20568, 65504, 26496, 17824, 33348, 7632,
                58120, 45300, 30644, 53132, 20976, 3560,  16156, 31740, 22096,
                48988, 4068,  63320, 28252, 12328, 5244,  19444, 43644, 10852,
                59800, 21504, 44628, 57104, 4364,  23704, 16320, 2244,  59096,
                8692,  44460, 39452, 53708, 12092, 4208,  45848, 39848, 56184,
                33392, 11916, 22396, 41408, 19128, 3844,  15708, 58212, 65324,
                50040, 40172, 14344, 6792,  26776, 48400, 31976, 11352, 40628,
                56996, 7684,
            },
            {
                52136, 4436,  35612, 54108, 20752, 29584, 41792, 10556, 17488,
                32076, 7560,  35116, 1760,  41644, 53284, 47996, 61860, 40704,
                32512, 51364, 8316,  18600, 56988, 5828,  11660, 17272, 29960,
                14272, 63076, 37556, 16884, 21724, 55876, 39184, 62464, 15656,
                48232, 28800, 51164, 31964, 2496,  61200, 23756, 43288, 50472,
                2716,  36464, 8656,  41336, 31332, 2616,  58812, 7876,  28928,
                61036, 3904,  45648, 30848, 56904, 37504, 62804, 47280, 12956,
                43540, 61700, 38388, 3176,  44548, 12372, 60472, 53760, 28744,
                23760, 2744,  36608, 13292, 60340, 46640, 24552, 65044, 35644,
                13668, 57368, 32664, 23936, 42244, 61808, 57200, 2236,  24996,
                37608, 33520, 15184, 41448, 30908, 61936, 11204, 46788, 27668,
                32272, 20352, 60392, 25932, 34456, 57504, 21360, 60936, 28812,
                5056,  46592, 54564, 30028, 59164, 36052, 47472, 26216, 18448,
                11740, 23928, 56216, 63388, 42788, 17548, 58272, 5456,  34072,
                25572, 46636,
            },
            {
                30172, 42676, 14084, 46048, 62084, 8032,  34708, 47632, 3068,
                57352, 43532, 13196, 62824, 17180, 8740,  28752, 23044, 5280,
                26148, 55776, 37468, 14384, 42092, 28152, 34916, 59752, 47100,
                54432, 32652, 26392, 49792, 12324, 29024, 4152,  52984, 36552,
                41784, 5260,  65000, 18544, 12968, 28000, 7936,  17696, 29700,
                14404, 52588, 44824, 12232, 54640, 16448, 35588, 50548, 46508,
                11056, 38948, 18876, 59756, 23516, 920,   15588, 6200,  31224,
                54776, 22784, 7808,  57288, 34508, 49344, 37804, 156,   43156,
                61612, 10660, 41052, 49800, 28292, 8876,  39264, 52028, 6240,
                27144, 38260, 45488, 852,   50772, 36596, 31016, 14392, 54348,
                47528, 6928,  64788, 9308,  19176, 36124, 55612, 21872, 64472,
                13168, 1440,  48428, 8188,  16488, 50168, 940,   13976, 52476,
                18340, 64840, 8252,  14080, 27060, 51788, 7192,  39168, 31468,
                43832, 5388,  34504, 51376, 9860,  38344, 23456, 65060, 44104,
                1080,  61312,
            },
            {
                8980,  57324, 24316, 27628, 4976,  50680, 55684, 22384, 64200,
                37428, 49452, 24332, 52124, 38864, 32860, 44884, 13300, 16816,
                59680, 43368, 1780,  23696, 62312, 50344, 2704,  25060, 20724,
                3716,  10700, 5292,  42288, 59280, 45896, 18464, 24872, 8468,
                60184, 22036, 44440, 38024, 57056, 52396, 41440, 47276, 64560,
                56640, 25024, 18840, 60104, 4608,  65284, 25332, 13400, 56016,
                31884, 53496, 5228,  49768, 41192, 29356, 50752, 19996, 36840,
                26652, 11020, 52068, 29536, 15728, 24488, 9352,  21380, 47312,
                17416, 56120, 25988, 5144,  34488, 55216, 18272, 1472,  42332,
                61640, 10904, 17972, 56040, 15392, 21304, 8184,  40896, 63020,
                3640,  29016, 24148, 53296, 49064, 44996, 6384,  40280, 50348,
                34972, 56332, 38384, 28612, 62100, 41824, 31772, 8852,  42968,
                35088, 24296, 38576, 44060, 1768,  63580, 21984, 54896, 52,
                58892, 48024, 20788, 30200, 1336,  53788, 14468, 50364, 28716,
                12796, 19136,
            },
            {
                49424, 33752, 64648, 11320, 38972, 17032, 30948, 13432, 26612,
                5404,  19172, 30224, 11032, 56564, 3756,  63796, 53824, 49436,
                10028, 29832, 33708, 47768, 20020, 38720, 9284,  55584, 43840,
                35632, 51620, 61316, 23532, 31164, 1736,  64304, 50644, 13756,
                33384, 54064, 11392, 3956,  34172, 22536, 624,   37124, 11260,
                5960,  39708, 33760, 46976, 29252, 38216, 43000, 19656, 688,
                22480, 63316, 26980, 14516, 10616, 64460, 55932, 42976, 59944,
                2048,  40592, 18432, 46852, 4816,  62924, 55476, 31516, 14028,
                35508, 51504, 20540, 62844, 15436, 43224, 60956, 29300, 47532,
                21112, 52724, 6632,  30144, 48340, 60008, 27524, 45812, 32856,
                19624, 56900, 39012, 12964, 1060,  26016, 29876, 17476, 3968,
                24360, 43168, 18700, 4496,  54648, 23132, 37352, 26608, 47712,
                59736, 3148,  50332, 57576, 33204, 10696, 42412, 13784, 62324,
                28080, 37420, 15928, 60604, 26000, 45220, 7904,  35904, 20948,
                54504, 38180,
            },
            {
                16324, 2168,  21452, 53220, 43380, 208,   61516, 45312, 40616,
                54392, 60212, 964,   46880, 21596, 26552, 7244,  34668, 38300,
                20496, 64348, 4584,  53072, 12368, 58788, 31912, 15460, 64728,
                28036, 40024, 13792, 53544, 37996, 9236,  34684, 41008, 27716,
                57992, 17316, 30540, 49204, 63128, 15632, 59488, 32276, 20972,
                27680, 62592, 1668,  22200, 10080, 49100, 6756,  58408, 33684,
                40732, 8688,  36352, 47788, 33140, 3740,  25544, 8384,  17068,
                48576, 63876, 35440, 59220, 42068, 27828, 50792, 39884, 3668,
                64512, 8028,  32632, 2148,  48512, 23140, 11544, 32976, 14616,
                58432, 25308, 35032, 64820, 38704, 11520, 4800,  52088, 16412,
                10584, 49876, 43896, 34612, 61028, 57680, 14452, 62236, 53652,
                10488, 58696, 30752, 14620, 45624, 10920, 64276, 55804, 16824,
                11380, 21500, 31152, 15388, 19264, 47040, 25664, 35124, 17604,
                52008, 3316,  10888, 41648, 56448, 18596, 30904, 57952, 4076,
                41756, 62688,
            },
            {
                6372,  47480, 36892, 29380, 59524, 19068, 35864, 6908,  10092,
                33272, 15868, 43004, 36196, 58704, 17924, 42444, 60468, 4,
                24884, 45996, 56376, 16476, 26948, 6100,  41724, 49420, 536,
                18264, 47308, 6380,  20996, 16304, 56704, 44560, 19912, 5940,
                936,   46624, 24452, 7308,  43120, 26660, 9164,  48480, 55436,
                42436, 50788, 14748, 57644, 53936, 17252, 27232, 61940, 44900,
                15808, 51796, 57972, 17944, 44696, 22136, 39568, 34416, 52672,
                14000, 31892, 22336, 2768,  12780, 19776, 6340,  58188, 24000,
                42460, 28124, 45428, 38680, 58284, 7124,  51756, 37180, 3708,
                40240, 9476,  44540, 2512,  23292, 42860, 55708, 25492, 36148,
                59288, 22624, 2928,  17684, 31872, 8528,  42328, 37496, 33040,
                46464, 7108,  63744, 49640, 35368, 20456, 6320,  2288,  51160,
                28232, 41424, 5268,  61004, 54072, 38264, 4716,  49784, 8492,
                22756, 45540, 64724, 33412, 6104,  38828, 63480, 46328, 10992,
                26580, 31544,
            },
            {
                43744, 56368, 14304, 8516,  49936, 24120, 57432, 47536, 20676,
                51852, 25016, 65296, 8308,  50456, 31532, 13768, 47468, 28968,
                11816, 40748, 7832,  35244, 44408, 63116, 21528, 36548, 11640,
                24560, 58568, 33992, 63340, 48536, 25684, 12028, 62116, 49416,
                37068, 60988, 40296, 55076, 19028, 51776, 39116, 13588, 3280,
                17884, 35328, 7816,  44312, 31432, 36316, 2940,  52544, 11316,
                29952, 3428,  24620, 6852,  62036, 53564, 12040, 61076, 28664,
                5332,  44032, 9620,  53204, 38200, 45352, 33652, 17008, 11300,
                52964, 60028, 13460, 18684, 30976, 26288, 56256, 19836, 64104,
                48852, 28432, 55024, 19132, 33612, 14052, 61408, 192,   40020,
                7060,  63716, 27280, 54540, 47072, 21540, 51220, 1840,  19640,
                27360, 22972, 40364, 304,   52880, 25448, 43496, 39176, 33260,
                62364, 56504, 44788, 24536, 8984,  29144, 61836, 56740, 31816,
                39464, 54472, 25160, 13644, 49068, 2816,  22628, 15044, 51680,
                59676, 23396,
            },
            {
                18712, 60840, 26260, 41372, 3728,  32444, 14840, 27920, 62904,
                38516, 4560,  12584, 28268, 40224, 2452,  23232, 52720, 57008,
                18868, 61632, 31060, 22712, 51464, 3376,  29276, 56580, 52988,
                30868, 8908,  44028, 4312,  29316, 2352,  32188, 53956, 22952,
                29124, 9532,  13888, 35016, 1764,  29364, 62144, 23380, 44596,
                59804, 29836, 24632, 40236, 64268, 12648, 23932, 39348, 21024,
                48068, 64828, 42692, 37960, 30944, 100,   49624, 20548, 46224,
                58240, 24812, 55868, 30324, 64992, 25416, 60780, 48984, 36964,
                1704,  21296, 47776, 9344,  63420, 42084, 312,   45660, 24052,
                5036,  15956, 59712, 50708, 7516,  46932, 29888, 20836, 48532,
                31356, 42696, 11812, 36992, 4484,  25256, 64836, 11980, 48588,
                60620, 16280, 33880, 12412, 29264, 57236, 60256, 14124, 19460,
                8152,  37092, 13216, 34436, 48652, 2176,  20900, 43996, 12560,
                59368, 6612,  29884, 20172, 43160, 53392, 28296, 34620, 416,
                39544, 9516,
            },
        },
        {
            {
                31656, 10316, 56960, 45840, 53444, 33376, 40484, 164,   20084,
                59728, 25476, 5332,  21400, 37760, 824,   42240, 62504, 15816,
                2292,  47624, 44204, 24988, 55708, 556,   28468, 7752,  51108,
                5220,  29792, 2232,  48472, 19140, 28112, 432,   49840, 42208,
                54628, 28772, 14344, 60144, 31608, 12748, 40392, 54600, 47844,
                7220,  61540, 15444, 9800,  31736, 51712, 27484, 4672,  35680,
                53492, 29212, 38500, 34100, 51360, 60800, 19000, 7936,  57204,
                16044, 44604, 22068, 38404, 25672, 51252, 6392,  43340, 16480,
                35516, 60896, 3880,  45652, 55612, 25168, 13692, 8784,  50868,
                46568, 22560, 6972,  14960, 48188, 12000, 57836, 51296, 34156,
                30064, 38128, 63124, 27504, 51140, 4880,  61572, 26296, 52648,
                35052, 3512,  60644, 26840, 64872, 996,   52108, 15704, 62092,
                49568, 17292, 4848,  52048, 23672, 63992, 46240, 36776, 13304,
                328,   38448, 54112, 29040, 22164, 1668,  33464, 54256, 29480,
                17700, 60864,
            },
            {
                24748, 19792, 35128, 3580,  12704, 16636, 63040, 55872, 46884,
                15356, 42504, 52708, 29288, 61156, 46484, 9532,  54488, 29628,
                35864, 20564, 12452, 6336,  62284, 36248, 14824, 38440, 24080,
                58204, 12740, 37800, 59576, 14816, 44460, 63292, 12880, 17072,
                3832,  65312, 22432, 52656, 42732, 27724, 64148, 19412, 10548,
                42928, 26480, 3240,  54120, 45800, 60684, 12848, 49524, 21320,
                14680, 9048,  55676, 5672,  22176, 44848, 30092, 48068, 23668,
                11972, 28868, 1660,  64088, 14988, 57776, 30988, 54180, 26160,
                47800, 21472, 29876, 11324, 50508, 19828, 62580, 27700, 38124,
                2076,  62180, 26492, 35756, 43484, 24800, 61848, 2392,  21164,
                44288, 48852, 16680, 57192, 36096, 19356, 43852, 15392, 39388,
                21056, 47532, 31172, 44956, 12056, 23908, 57092, 32056, 3396,
                8788,  35736, 29328, 12728, 42412, 7348,  56844, 30160, 62576,
                44236, 32560, 18956, 65260, 40872, 9944,  26116, 47380, 12064,
                7868,  36700,
            },
            {
                728,   42032, 64996, 38540, 21696, 27828, 50320, 7180,  31052,
                11112, 34988, 3032,  17448, 55344, 26128, 14480, 22440, 5700,
                65508, 52396, 32308, 39524, 49728, 10596, 54400, 60284, 44528,
                33328, 21060, 42228, 25516, 34584, 7956,  55240, 24840, 33280,
                48276, 45296, 38896, 7656,  2760,  50596, 15224, 33488, 58700,
                37796, 49872, 64800, 35472, 5144,  22944, 41044, 30888, 63096,
                43144, 47240, 16992, 64640, 11832, 36480, 1204,  63640, 35084,
                51940, 60696, 32260, 41556, 8276,  46432, 23112, 12408, 64560,
                8556,  52240, 40912, 17588, 5552,  36160, 43036, 4196,  58844,
                18472, 10456, 49656, 55072, 20180, 8656,  37624, 17800, 56680,
                10356, 5124,  24232, 41892, 8924,  30584, 49940, 56076, 6480,
                59436, 9860,  37220, 53432, 20216, 5340,  43024, 27308, 46228,
                21424, 56020, 61476, 48252, 38144, 15772, 25668, 9828,  21416,
                5548,  47544, 11036, 49804, 35236, 57592, 51748, 19252, 39468,
                63732, 55104,
            },
            {
                50828, 14420, 29176, 47732, 57688, 9316,  43132, 37104, 22812,
                64676, 57128, 48752, 40072, 7340,  31916, 36836, 48908, 40680,
                56780, 9232,  17076, 58576, 27228, 21240, 31384, 18208, 2672,
                9868,  52392, 61976, 6380,  50580, 21908, 40772, 36772, 58252,
                6172,  26456, 11464, 59204, 34468, 21136, 56036, 3592,  30244,
                23364, 12392, 20256, 29156, 17676, 58092, 37364, 7564,  1900,
                58476, 31832, 25496, 49852, 41856, 26872, 53768, 6460,  17724,
                40168, 4380,  48628, 55952, 18860, 36420, 1264,  40244, 33336,
                4604,  44404, 24596, 61488, 57232, 48340, 21884, 32072, 53052,
                34608, 41760, 30076, 59736, 1360,  31864, 50440, 41468, 26548,
                64752, 54476, 32284, 13528, 59624, 3600,  23444, 11292, 28664,
                42784, 24968, 1812,  16464, 39884, 60052, 35332, 13464, 63752,
                39508, 11152, 26592, 18784, 1684,  32888, 53512, 49972, 39712,
                59076, 55180, 25064, 3256,  16620, 6096,  43876, 31088, 2900,
                22136, 45520,
            },
            {
                9780,  59720, 5892,  53000, 1536,  34260, 61140, 17916, 2360,
                45228, 26588, 20396, 13136, 63156, 43532, 58920, 18300, 1592,
                28076, 24312, 42888, 2964,  45156, 4696,  48004, 40728, 64956,
                27336, 46092, 16724, 1368,  57560, 29664, 10724, 3160,  61680,
                20212, 30908, 53668, 16848, 24612, 44896, 39848, 6376,  46632,
                52652, 1428,  57044, 42316, 48564, 10288, 25652, 50956, 19056,
                12152, 39936, 2820,  20436, 8164,  57292, 13940, 31752, 46280,
                25952, 10404, 21188, 13016, 27008, 61888, 50180, 58972, 19596,
                56088, 14896, 34668, 236,   31024, 9704,  63856, 15220, 45872,
                24192, 13048, 5080,  16540, 45368, 63520, 14052, 6012,  46488,
                15312, 35424, 880,   45336, 62924, 37680, 47232, 34236, 64512,
                18228, 57108, 48428, 62848, 29456, 50428, 7720,  18268, 54000,
                48,    33732, 52528, 6308,  57932, 62184, 43456, 3576,  17172,
                34616, 14212, 30692, 61044, 38264, 23488, 62472, 13696, 58996,
                34128, 27516,
            },
            {
                17144, 32624, 23800, 18952, 44924, 25512, 13852, 29336, 53172,
                39224, 8648,  33420, 51516, 23876, 4500,  10776, 51832, 34412,
                46216, 61056, 14140, 37324, 62908, 53352, 25308, 7412,  35664,
                13788, 55760, 31724, 37244, 47056, 64412, 17944, 44784, 51464,
                14828, 41880, 36132, 63632, 48812, 9708,  61656, 26916, 59632,
                15824, 33036, 38960, 6668,  14376, 62676, 44044, 55052, 28492,
                36276, 52948, 59636, 33756, 61884, 38756, 43820, 22504, 62652,
                57708, 36716, 54024, 44324, 33728, 6108,  16796, 29544, 10752,
                27460, 62688, 39296, 53388, 13000, 26236, 40160, 2968,  7852,
                57608, 64740, 37988, 53636, 23044, 27604, 36332, 52812, 30284,
                22604, 39780, 51928, 27776, 21004, 17128, 54396, 1236,  40672,
                51768, 5472,  13632, 33484, 10496, 56100, 25416, 47724, 30220,
                43600, 23076, 46828, 41088, 20672, 29592, 12308, 23532, 64348,
                28056, 7104,  42656, 46208, 52880, 28368, 8652,  48744, 53584,
                5468,  40320,
            },
            {
                56624, 43780, 36884, 63444, 10956, 40004, 58960, 47972, 5064,
                62132, 16108, 58464, 1940,  45736, 30196, 21276, 64000, 12996,
                7024,  31220, 50860, 19528, 33940, 12016, 16328, 57056, 49284,
                22200, 40012, 8960,  19808, 26632, 13896, 53764, 32300, 23756,
                8736,  57172, 660,   5776,  29264, 13544, 36612, 18684, 8272,
                43776, 62400, 21692, 53884, 31216, 720,   33656, 15800, 64240,
                22532, 4240,  45856, 15196, 29536, 18804, 4680,  49456, 8812,
                15856, 168,   30268, 64712, 3300,  52636, 42212, 46072, 37396,
                48840, 3096,  22492, 47184, 18996, 44768, 55556, 29552, 50960,
                20308, 26936, 47496, 9268,  40760, 58612, 11548, 2828,  60764,
                8124,  18648, 57712, 10756, 50248, 7392,  25984, 14688, 31020,
                21584, 36372, 27388, 45924, 2660,  20820, 36488, 4676,  58356,
                16196, 8216,  65000, 14476, 36308, 49108, 8488,  37520, 47096,
                52160, 19388, 57696, 2060,  12276, 18604, 41736, 36076, 25508,
                20420, 47392,
            },
            {
                2256,  60564, 7016,  30168, 51448, 3844,  54896, 19672, 32020,
                12160, 42848, 28060, 36284, 61208, 39408, 56260, 26372, 41964,
                55008, 22516, 80,    56664, 8800,  29780, 42408, 61792, 1560,
                28808, 5528,  62500, 50016, 41688, 5164,  35588, 2100,  39472,
                49588, 26960, 45904, 19556, 55660, 42580, 52088, 31432, 24160,
                49376, 11024, 27144, 45864, 60224, 23708, 39800, 5012,  47900,
                8780,  41952, 24428, 10916, 51588, 1704,  55452, 35552, 28616,
                42432, 50556, 23736, 39284, 15412, 60296, 24872, 6908,  21488,
                54500, 8908,  32456, 59264, 6624,  37232, 60408, 16280, 35096,
                42628, 636,   32592, 18620, 3940,  33928, 20508, 56072, 41844,
                48072, 62312, 3380,  31460, 43148, 39184, 57416, 60904, 44944,
                8592,  61864, 54580, 23420, 43112, 64628, 40828, 11988, 61388,
                37888, 27924, 54788, 32088, 2600,  60056, 26988, 56532, 744,
                41152, 10460, 35492, 22572, 64688, 32332, 56112, 12,    15452,
                64116, 11848,
            },
            {
                28296, 21268, 13552, 48392, 16576, 33616, 22928, 8380,  37616,
                24740, 52380, 48684, 18024, 6440,  14452, 47940, 2648,  37024,
                17396, 49020, 40060, 64572, 23548, 47000, 38164, 20132, 32664,
                59488, 43692, 52984, 12008, 23204, 59800, 55848, 28548, 63176,
                12188, 58956, 33424, 38672, 23144, 2556,  65088, 5076,  57200,
                37660, 2688,  35376, 17860, 9420,  50116, 19468, 59108, 13184,
                30960, 61568, 56312, 37640, 47576, 65320, 25172, 12828, 60736,
                19260, 58472, 7444,  47132, 20384, 10088, 32092, 58132, 14092,
                65248, 43596, 16460, 51460, 28864, 1920,  23184, 10380, 49036,
                62536, 12132, 56396, 51176, 62100, 44996, 49444, 16088, 24416,
                33268, 13096, 36696, 23792, 65364, 4348,  33724, 11740, 49220,
                3236,  16800, 38688, 58952, 7140,  15252, 52172, 32584, 21964,
                49648, 3780,  11416, 24336, 51036, 17620, 44340, 13612, 21296,
                31680, 61608, 26104, 48880, 37876, 5148,  46524, 59952, 30200,
                38588, 50980,
            },
            {
                35012, 42196, 54060, 38704, 26816, 61720, 41924, 45396, 63868,
                56028, 336,   21716, 10200, 53712, 32976, 19964, 8636,  29008,
                58856, 10908, 34928, 27692, 6116,  54416, 3688,  50808, 10040,
                15268, 24420, 34704, 292,   30996, 45560, 10012, 18792, 43488,
                22064, 15948, 7568,  60592, 50064, 17376, 34356, 12112, 40940,
                20464, 63596, 52196, 4736,  56192, 28164, 36928, 44612, 53664,
                35056, 17244, 27856, 6252,  20628, 32324, 40692, 45464, 5172,
                33332, 38012, 12028, 27896, 56384, 35392, 41176, 49640, 4316,
                30040, 35820, 24440, 11184, 41368, 64308, 32792, 53520, 4564,
                25100, 37452, 29984, 14780, 24016, 6752,  29104, 37772, 172,
                27356, 44200, 54656, 15568, 47656, 28416, 18020, 22528, 41384,
                29260, 51476, 10992, 31364, 19072, 48332, 26048, 1172,  55776,
                17964, 46008, 42008, 56896, 39268, 5376,  33644, 63148, 53692,
                4836,  16080, 44196, 7620,  54648, 16764, 21668, 10544, 43268,
                7584,  55404,
            },
            {
                4244,  25192, 65296, 652,   9348,  56368, 2864,  14864, 28568,
                5612,  34436, 40524, 65196, 25136, 57900, 43208, 51344, 62608,
                45604, 4092,  14884, 44248, 18000, 60000, 13272, 26180, 44868,
                37272, 57880, 18320, 65132, 38740, 15396, 51708, 6332,  36976,
                48228, 3464,  53888, 30892, 9960,  44428, 27656, 47760, 54528,
                14820, 29360, 43116, 12940, 40504, 65172, 7176,  24964, 2800,
                21440, 49816, 400,   43460, 58564, 16276, 9836,  51916, 22744,
                54304, 2572,  62940, 44008, 51404, 1580,  62540, 26924, 18688,
                39988, 1388,  61272, 55912, 48328, 13684, 19696, 45780, 39836,
                17768, 7380,  59192, 43408, 39256, 10684, 64484, 53856, 59064,
                9164,  51132, 5800,  20160, 59224, 9508,  52216, 56300, 35836,
                63320, 25632, 46920, 1716,  35288, 62492, 29676, 44512, 6484,
                34688, 63012, 29968, 20100, 61284, 10212, 23732, 48240, 28488,
                39676, 58344, 51260, 13508, 29752, 62864, 27188, 50328, 33588,
                23584, 16952,
            },
            {
                58768, 11052, 19080, 32612, 46500, 21564, 35940, 50728, 18548,
                47388, 58560, 13668, 45124, 29284, 4964,  35948, 15928, 21912,
                32112, 25380, 61512, 51848, 41136, 31332, 35184, 63476, 55412,
                2700,  47740, 7828,  27756, 48832, 21648, 61644, 25784, 56672,
                29380, 64600, 40964, 25360, 14216, 62140, 58080, 1620,  25032,
                7616,  59936, 32828, 21868, 47084, 16188, 32192, 11804, 57452,
                38760, 63932, 14032, 53960, 36312, 6984,  29856, 63340, 17168,
                26844, 48248, 14924, 31344, 17780, 6292,  22640, 11468, 57288,
                53040, 45876, 20888, 5492,  37908, 26284, 7896,  58496, 28288,
                54704, 47716, 20756, 1588,  52236, 32024, 19340, 14268, 46172,
                21688, 63144, 30044, 41012, 2716,  37756, 45548, 552,   13608,
                6152,  20024, 58004, 40460, 53420, 13248, 9584,  59628, 40032,
                25072, 8740,  13956, 544,   35544, 53024, 42596, 7228,  18564,
                35156, 1440,  23936, 33432, 42632, 2520,  39308, 58128, 3540,
                62160, 45996,
            },
            {
                49684, 29620, 43816, 52376, 59288, 12588, 30436, 60208, 10500,
                23788, 31664, 7876,  37828, 50256, 61328, 12164, 776,   53968,
                39588, 7428,  55780, 20544, 1064,  48972, 8128,  16580, 22092,
                29648, 41412, 12876, 54680, 3980,  31868, 42060, 1696,  33804,
                11196, 17976, 45616, 36020, 5288,  21468, 37960, 31804, 18160,
                45480, 50268, 52,    25868, 58772, 4180,  52736, 48376, 28676,
                8576,  45312, 33200, 22548, 26196, 61536, 46936, 4144,  37060,
                42224, 59928, 7840,  24116, 54804, 37472, 43428, 47836, 33072,
                15300, 9480,  27952, 33580, 17012, 44084, 62664, 35324, 2772,
                13128, 65432, 34540, 26768, 60368, 3672,  42456, 34724, 4572,
                39320, 12360, 34004, 55884, 26152, 63884, 31544, 24580, 59996,
                44120, 32652, 15024, 4620,  23868, 57156, 37580, 20724, 15564,
                53532, 50004, 57792, 26732, 47500, 15660, 30876, 55948, 11872,
                64364, 46236, 9984,  61468, 19280, 52232, 12580, 45020, 20076,
                14232, 36752,
            },
            {
                8032,  39400, 5604,  15456, 24456, 41120, 6924,  38328, 54376,
                43092, 63064, 20220, 2280,  17236, 22996, 41780, 27588, 63956,
                48184, 13068, 37292, 28624, 10512, 23776, 39108, 59196, 4968,
                51972, 33404, 20936, 60104, 44088, 9524,  57724, 14148, 39920,
                53036, 23440, 404,   51196, 56328, 42400, 8280,  52680, 63332,
                39212, 35140, 10800, 54732, 38496, 18716, 35660, 62080, 41204,
                19328, 59304, 4024,  10692, 50244, 39608, 13636, 56980, 32140,
                10504, 20296, 34264, 40096, 64924, 13472, 30276, 61148, 7128,
                38224, 63772, 50176, 59200, 53744, 560,   49160, 22060, 41836,
                31796, 9512,  44496, 15536, 56000, 22776, 48756, 28324, 56604,
                25372, 49380, 17880, 43736, 10960, 15120, 20900, 49780, 8188,
                54464, 37672, 64424, 45300, 50136, 28136, 2652,  47604, 65348,
                31536, 4700,  43940, 38848, 64064, 21768, 2236,  60672, 27124,
                49744, 20904, 38972, 55128, 6040,  35632, 25752, 9016,  31124,
                53364, 26556,
            },
            {
                22276, 55248, 61248, 34300, 3320,  49748, 64460, 26580, 1308,
                15608, 52184, 27396, 57284, 46336, 54700, 31280, 9732,  44728,
                17416, 3416,  33684, 57800, 65284, 46680, 53336, 27904, 43180,
                10668, 63960, 37692, 25404, 16972, 35688, 49704, 19788, 62316,
                46992, 8456,  60176, 28464, 33128, 16304, 46812, 11808, 23264,
                3920,  15620, 61072, 28756, 8348,  44464, 23168, 1116,  15076,
                25520, 49012, 30724, 42132, 55568, 19116, 960,   21932, 44328,
                52756, 2108,  57308, 49332, 3912,  26624, 51828, 800,   21444,
                25120, 41932, 4540,  12612, 23568, 30712, 10732, 15992, 52164,
                58028, 24240, 50636, 6352,  36644, 11560, 63812, 7756,  15920,
                61696, 1304,  59740, 6776,  47312, 53524, 35328, 39960, 17308,
                26572, 10764, 29460, 21528, 8448,  17532, 33372, 42924, 11632,
                22956, 36740, 18764, 10748, 32804, 5036,  40516, 44776, 36996,
                14748, 3808,  31396, 27876, 15696, 48664, 64964, 56488, 41612,
                1944,  63776,
            },
            {
                13108, 18368, 47136, 27344, 56684, 20516, 17160, 45880, 32924,
                4804,  35816, 40916, 9312,  33704, 3284,  58512, 38468, 20640,
                59768, 24408, 50568, 43516, 18476, 13948, 2004,  35600, 19408,
                49624, 15096, 1136,  52832, 6404,  63180, 28936, 24536, 5668,
                31156, 37576, 13512, 20184, 64156, 3632,  27268, 59580, 30128,
                43608, 51780, 20324, 47852, 63888, 13252, 30340, 56964, 6676,
                53752, 36788, 17112, 60708, 7688,  34720, 29276, 50852, 25200,
                63688, 13212, 28640, 45392, 9612,  19440, 59024, 16384, 45596,
                55980, 31188, 18796, 36656, 42900, 56284, 39372, 61508, 27488,
                4716,  18776, 40424, 29364, 46628, 17904, 39720, 53500, 44060,
                36784, 31676, 22328, 38536, 29128, 3088,  58444, 5104,  43076,
                61624, 1300,  52764, 34872, 41272, 62028, 52084, 6656,  56168,
                28840, 1656,  62128, 52548, 24680, 58880, 50260, 18036, 8536,
                25128, 52420, 57768, 43620, 60560, 448,   21184, 37844, 16748,
                48128, 34644,
            },
            {
                42724, 30884, 956,   10876, 37248, 43500, 8176,  58192, 22648,
                60928, 47828, 12992, 62496, 25084, 14860, 49228, 6260,  35204,
                52228, 30644, 8628,  39916, 6120,  32540, 25484, 62600, 56388,
                7136,  31616, 46144, 40396, 56048, 11428, 45308, 2612,  55324,
                16688, 58460, 43844, 49512, 10084, 54032, 37284, 19576, 56840,
                5908,  40624, 24752, 33512, 2568,  40164, 51384, 46068, 33052,
                64212, 12496, 3064,  23012, 45144, 65040, 15840, 59008, 5364,
                39052, 17000, 36636, 22312, 61876, 33308, 40856, 36268, 49820,
                10536, 60508, 2336,  47680, 64848, 7276,  20492, 3224,  45724,
                37740, 63536, 53660, 428,   61276, 33084, 24480, 2476,  20064,
                11880, 52004, 8744,  55280, 62000, 24036, 12924, 51392, 30408,
                19680, 48296, 57048, 5632,  12492, 58400, 26352, 19688, 39484,
                60268, 45920, 16344, 48212, 6964,  13744, 28896, 54300, 33500,
                59716, 40252, 7500,  11380, 33956, 24568, 46476, 7056,  28412,
                59960, 4640,
            },
            {
                50632, 62308, 40568, 53456, 63228, 30300, 13736, 52800, 39840,
                28892, 6804,  21120, 51268, 43928, 19092, 64876, 26452, 12672,
                1312,  62280, 15408, 22564, 60148, 54672, 45588, 11716, 40768,
                23384, 59532, 27172, 18224, 30608, 22452, 39012, 34748, 51124,
                41536, 26016, 2216,  35308, 22924, 41116, 48260, 968,   34408,
                64896, 13040, 7440,  56464, 17260, 59400, 26696, 10156, 19708,
                42464, 28404, 51692, 38732, 26496, 11312, 48488, 41148, 8972,
                31008, 47028, 54448, 6848,  43060, 12508, 53852, 5464,  28268,
                14388, 34528, 52560, 26132, 14856, 29244, 34952, 50148, 12752,
                32740, 9024,  22348, 14016, 56468, 9804,  49352, 29964, 58828,
                46556, 25844, 42488, 19012, 33380, 16400, 45356, 64260, 34252,
                11996, 39316, 24492, 16136, 46824, 36936, 188,   14624, 49340,
                8524,  34612, 26628, 40808, 56292, 36572, 21480, 11224, 65204,
                2904,  47712, 23048, 18328, 50732, 41020, 13556, 32500, 52408,
                11088, 24112,
            },
            {
                58316, 21160, 6868,  16096, 25068, 4040,  35364, 48388, 10424,
                18016, 56704, 31452, 36824, 4568,  39788, 30116, 53544, 46540,
                42252, 55516, 27888, 48688, 3724,  36932, 29240, 16436, 3280,
                48076, 36732, 9272,  51468, 3444,  59832, 14648, 65372, 7820,
                21396, 11776, 62728, 30272, 7004,  60828, 13968, 25620, 17432,
                49216, 54312, 30944, 44052, 36168, 4536,  21936, 38192, 60812,
                1844,  47648, 58288, 5680,  54148, 32492, 2248,  35572, 56484,
                20172, 62376, 452,   25660, 58396, 30632, 2592,  24200, 64020,
                39564, 6828,  21996, 44816, 9636,  53936, 60072, 23956, 57588,
                17092, 48844, 43824, 35136, 26612, 42328, 4376,  62352, 35264,
                6256,  14736, 64664, 48120, 284,   41484, 26876, 8460,  22056,
                4308,  60544, 44744, 32052, 64784, 29292, 22540, 44192, 32148,
                53484, 12416, 20452, 2776,  31012, 61348, 42652, 5580,  45348,
                30264, 15544, 36240, 63416, 54044, 4080,  58832, 62784, 19508,
                44648, 36008,
            },
            {
                14248, 27912, 33388, 49912, 45376, 21544, 60436, 348,   25636,
                64660, 42124, 2208,  54100, 11604, 59252, 8416,  22220, 17104,
                37340, 10636, 34020, 44420, 20336, 9572,  50380, 60612, 33440,
                20656, 64692, 13732, 57264, 42020, 48588, 19248, 27524, 47160,
                33236, 56940, 52076, 46012, 18352, 56144, 31992, 44680, 9276,
                38040, 27212, 20948, 11272, 62520, 48612, 14180, 55176, 7952,
                24472, 18008, 34888, 14544, 20572, 61588, 43640, 23376, 12680,
                27608, 50976, 33936, 15288, 49968, 18488, 47784, 56648, 20684,
                42704, 58012, 30364, 62592, 17932, 39768, 1164,  42760, 6564,
                28932, 55020, 5556,  59168, 19160, 51492, 15500, 22076, 40812,
                54204, 28284, 4484,  36552, 11308, 57584, 52352, 37512, 48848,
                55384, 28072, 2000,  19068, 8808,  50044, 55580, 62840, 3608,
                24028, 57888, 64672, 44692, 9332,  15980, 49204, 24816, 38280,
                19776, 55752, 1380,  27288, 30796, 9344,  38608, 26432, 1224,
                56040, 8244,
            },
            {
                47652, 2296,  38572, 56476, 11912, 41764, 55372, 32700, 37904,
                50292, 13964, 24308, 61828, 27408, 45184, 33624, 57116, 4840,
                63160, 25384, 2252,  57636, 64524, 41496, 24280, 6572,  55608,
                43748, 140,   25140, 34896, 5896,  32120, 9900,  54460, 860,
                39620, 16200, 4496,  24664, 11008, 39116, 4780,  53128, 62104,
                3108,  58816, 46448, 484,   52752, 28580, 42804, 34476, 50376,
                30540, 40508, 63336, 46192, 8672,  28992, 16704, 52816, 60084,
                3620,  41976, 10688, 39736, 64668, 7776,  35092, 44732, 10072,
                16560, 280,   50076, 36044, 5000,  47880, 32012, 19668, 64408,
                36560, 24928, 39636, 1820,  30440, 65140, 37332, 11128, 57176,
                18140, 32396, 50220, 60620, 21116, 30780, 6408,  59664, 18012,
                40696, 14584, 35860, 54072, 42004, 4544,  11404, 38388, 17852,
                41632, 6156,  36872, 27968, 53176, 34980, 340,   57864, 51812,
                12328, 61004, 41436, 45692, 14512, 21500, 49080, 43244, 16432,
                29992, 39952,
            },
            {
                53612, 59880, 18124, 64248, 8468,  27620, 18788, 5992,  16300,
                44644, 8584,  34840, 47404, 18744, 15332, 704,   49092, 41076,
                31356, 19632, 50804, 13764, 17360, 31192, 52912, 12868, 27572,
                38220, 50120, 16976, 45428, 63356, 22916, 37172, 60624, 13388,
                29668, 61728, 42972, 36952, 64344, 50772, 28764, 19844, 42044,
                23344, 15200, 33980, 39488, 18448, 23872, 5752,  65304, 16148,
                57032, 10696, 3744,  22692, 55668, 39876, 49008, 6944,  37656,
                45784, 18936, 55920, 22708, 28356, 37912, 13300, 26720, 62260,
                33244, 54996, 24704, 11656, 59036, 27012, 13596, 51788, 10328,
                46744, 15440, 61376, 12456, 44548, 7204,  47504, 27412, 884,
                45028, 9004,  23812, 39696, 54696, 15260, 25432, 2316,  32580,
                9716,  63516, 47428, 25568, 59312, 21308, 33564, 26532, 60576,
                30072, 47068, 50460, 13356, 23092, 62444, 18220, 32532, 27880,
                7708,  23504, 50040, 5640,  58148, 36668, 55288, 6780,  34084,
                65368, 20416,
            },
            {
                10120, 24576, 31000, 4936,  35912, 47064, 62240, 53032, 29980,
                60068, 56092, 21760, 4068,  52120, 36632, 61204, 23704, 13020,
                52864, 60096, 7908,  35768, 39376, 5116,  45884, 62752, 18932,
                8760,  30316, 54224, 11544, 28336, 52524, 3232,  44440, 23968,
                50288, 7592,  20368, 27784, 1096,  14976, 59640, 35580, 12072,
                30640, 51912, 63372, 7764,  57784, 31464, 12208, 46288, 1104,
                26804, 43364, 51480, 36804, 31744, 680,   64684, 26192, 14764,
                32108, 62956, 5244,  46772, 1436,  60688, 51096, 3448,  52872,
                38748, 7540,  44380, 20328, 41620, 55876, 37532, 61752, 22400,
                3076,  32588, 49944, 21324, 53924, 33392, 24060, 60776, 51932,
                34832, 62984, 13168, 2724,  46424, 35296, 43160, 62200, 45752,
                23628, 51648, 6328,  12540, 39060, 16388, 52504, 48448, 9648,
                15172, 2412,  19600, 57168, 6592,  41264, 46096, 4204,  64124,
                43704, 35228, 10628, 32692, 63880, 18028, 25356, 11512, 50800,
                3908,  45180,
            },
            {
                37360, 13620, 43236, 51864, 22404, 14516, 39132, 23720, 11440,
                1548,  27920, 40372, 63940, 30548, 43012, 10064, 28968, 38372,
                5972,  43612, 47476, 29060, 55804, 22576, 2016,  35216, 40880,
                59444, 4364,  61340, 39192, 20108, 15088, 41372, 58540, 18092,
                35384, 45324, 54140, 33032, 48928, 22656, 46256, 7648,  56380,
                48008, 4044,  25768, 10704, 41772, 54652, 59980, 38776, 21604,
                33340, 60968, 14000, 25488, 58684, 11964, 19912, 34556, 57480,
                9368,  24436, 30240, 53384, 16360, 41776, 19832, 30920, 23028,
                47152, 13884, 28652, 65352, 2204,  16708, 7972,  30148, 43868,
                56904, 40840, 8368,  28276, 57580, 16936, 39492, 5568,  14576,
                20176, 41512, 58156, 29084, 18344, 8392,  51152, 11764, 28704,
                57516, 20164, 34380, 29804, 62144, 44556, 1880,  57596, 36060,
                43496, 63504, 33232, 39196, 10904, 26112, 55076, 13716, 38684,
                16932, 54408, 20432, 26748, 40608, 1824,  47172, 60900, 22892,
                57160, 27304,
            },
            {
                5844,  61292, 48400, 716,   59132, 32124, 3292,  49428, 43400,
                33696, 47992, 17064, 12460, 7256,  54624, 20340, 56336, 65500,
                16256, 26360, 21592, 11096, 63528, 49664, 25760, 57316, 15632,
                46644, 22428, 33320, 1740,  48284, 62440, 6612,  26336, 10576,
                55824, 3572,  12636, 58696, 9192,  41560, 26692, 2376,  65120,
                17820, 37488, 44292, 50104, 20860, 35952, 4588,  17460, 48936,
                53240, 6756,  19344, 48024, 5832,  41244, 46688, 52248, 2628,
                43324, 49760, 36208, 12024, 58592, 33104, 9484,  57164, 5716,
                17988, 60188, 32232, 48888, 34996, 53476, 25556, 4748,  50464,
                17528, 26024, 63736, 36128, 3812,  10152, 43412, 48992, 31080,
                53616, 25480, 6584,  49172, 65032, 56668, 22104, 39444, 16272,
                4796,  42460, 480,   55236, 8408,  23336, 31284, 5592,  20676,
                54888, 24944, 52280, 29228, 60720, 48696, 21456, 30848, 59236,
                1612,  46960, 60312, 4464,  51728, 13576, 30108, 35740, 41840,
                15180, 32604,
            },
            {
                52964, 40292, 19420, 28876, 56208, 41624, 9080,  65192, 54324,
                20196, 5540,  57936, 38008, 45916, 25748, 2576,  34904, 46748,
                3616,  32996, 59336, 424,   42600, 14432, 8172,  31596, 51808,
                11060, 26760, 56912, 42940, 9288,  30020, 51576, 33168, 40132,
                64024, 31072, 16820, 38556, 62808, 18900, 55028, 31776, 40108,
                22088, 29012, 59568, 15740, 2120,  27164, 45072, 29840, 62156,
                9928,  37980, 28532, 54332, 16956, 60404, 29596, 22620, 38516,
                17636, 59724, 7504,  20892, 44792, 25788, 48948, 63876, 35344,
                42660, 55320, 4340,  9600,  22216, 45412, 40196, 60376, 33568,
                12856, 92,    47204, 19868, 51280, 59112, 22704, 64204, 2920,
                36608, 10784, 43724, 32196, 37720, 26780, 1340,  33836, 53880,
                63980, 37240, 49956, 26356, 46880, 13268, 65396, 41108, 27788,
                12292, 7596,  17136, 1524,  44272, 4652,  36524, 9352,  50156,
                23996, 29472, 14832, 37188, 57464, 44876, 7600,  53688, 18988,
                1504,  63540,
            },
            {
                22112, 7468,  34792, 11108, 15692, 25528, 18980, 36432, 13800,
                26004, 61556, 51320, 23260, 32716, 60124, 49788, 18108, 8932,
                40440, 50940, 54076, 37476, 19028, 33824, 61780, 44344, 20856,
                6488,  37164, 53148, 17428, 24892, 36596, 57568, 20776, 372,
                46392, 22792, 27148, 52256, 5584,  34460, 13624, 44832, 9812,
                52428, 5496,  12476, 33120, 53892, 64152, 7968,  13460, 24124,
                41968, 2332,  64464, 44472, 33452, 3548,  10004, 63652, 13372,
                55152, 27816, 40800, 65188, 4644,  38492, 1936,  15060, 24204,
                11768, 39716, 26616, 51408, 15256, 63388, 11460, 19620, 57764,
                38352, 55172, 42544, 14796, 31444, 37852, 12536, 28524, 18236,
                56440, 61360, 21720, 16392, 5424,  13432, 59872, 48280, 7904,
                30772, 13880, 18144, 60452, 39852, 34972, 16960, 51400, 59792,
                38036, 46588, 57804, 35244, 14416, 65068, 18572, 52920, 40336,
                6216,  63352, 42184, 8728,  31972, 21956, 64476, 27708, 10532,
                49692, 46348,
            },
            {
                16412, 26420, 54288, 62896, 45280, 51128, 60184, 6728,  30508,
                42472, 35220, 10324, 36,    15876, 40920, 12852, 28512, 61764,
                20744, 12268, 29788, 5516,  48724, 27428, 54496, 2816,  39632,
                64952, 49400, 13640, 62928, 4944,  44968, 12300, 16172, 50108,
                6540,  60772, 42852, 47568, 1500,  24376, 50368, 61268, 16416,
                57088, 35484, 62384, 47396, 23300, 40208, 50532, 58140, 35668,
                55260, 31852, 12304, 23108, 39016, 56508, 25828, 43984, 5444,
                32884, 24,    23396, 14132, 52128, 56240, 28724, 46164, 53088,
                61616, 656,   19332, 58188, 37748, 29436, 1556,  48264, 8752,
                22688, 28612, 6000,  62324, 25104, 1852,  54536, 45580, 7888,
                40384, 47464, 1808,  51488, 55364, 45232, 41064, 18748, 24552,
                43576, 58116, 10112, 22072, 3084,  56160, 48736, 9424,  896,
                32620, 22588, 49980, 41172, 24464, 31800, 55996, 27136, 12712,
                34292, 19372, 54036, 25152, 49064, 17044, 2696,  39684, 59028,
                36676, 30492,
            },
            {
                57220, 44140, 37684, 3100,  32776, 4764,  38928, 55292, 47216,
                3436,  17908, 56148, 29632, 64168, 4912,  52672, 37152, 43476,
                57140, 25016, 63824, 16448, 41248, 58440, 12120, 24128, 17864,
                28668, 624,   31056, 34852, 22084, 59536, 38832, 53816, 29024,
                35784, 13916, 9884,  20468, 36728, 57544, 28364, 6132,  38696,
                23980, 20,    42312, 18152, 9896,  31604, 19716, 920,   15916,
                46776, 20560, 51504, 7652,  15144, 49984, 35532, 20332, 51760,
                46296, 60756, 48512, 31028, 34880, 18136, 10616, 21344, 37016,
                30012, 48064, 33828, 43464, 6628,  54664, 24672, 31792, 44164,
                65008, 34956, 52556, 9520,  48636, 41380, 60332, 34780, 26316,
                14928, 29784, 33788, 63672, 23612, 28920, 9624,  62828, 4188,
                51032, 27720, 45688, 33176, 6452,  29020, 20204, 25492, 44948,
                63840, 4172,  19108, 10104, 59480, 7292,  46132, 2636,  43288,
                58744, 46544, 572,   11336, 61176, 34760, 43664, 54920, 24120,
                5176,  12732,
            },
            {
                924,   60356, 9796,  20636, 48584, 23820, 28508, 11944, 21712,
                62616, 50212, 38520, 43860, 47580, 21220, 58064, 23964, 1140,
                6860,  35824, 45276, 9536,  22664, 3860,  46904, 35408, 60260,
                42144, 45628, 9888,  55220, 47744, 2040,  8388,  65256, 24168,
                41568, 58300, 54444, 29936, 64548, 12088, 43300, 19664, 49188,
                30936, 45912, 26820, 55360, 3452,  37840, 60516, 44000, 26332,
                5868,  57300, 27744, 61244, 45756, 1032,  30216, 62012, 16156,
                37068, 10192, 19164, 42992, 7912,  62868, 40668, 59388, 5188,
                8708,  16504, 63048, 23404, 13204, 61100, 35692, 51064, 14968,
                3428,  17704, 45488, 21700, 33080, 16756, 5196,  20500, 52260,
                57728, 4212,  12220, 39520, 17744, 35904, 53112, 32380, 20972,
                36580, 1396,  64472, 54124, 42092, 62512, 36680, 15056, 53008,
                29868, 12864, 61768, 28116, 51876, 38508, 16656, 62264, 22872,
                30932, 15416, 37812, 28256, 52400, 4292,  14144, 19860, 32720,
                61972, 41668,
            },
            {
                22848, 49464, 31480, 14072, 64380, 42692, 17032, 57968, 33304,
                8804,  24700, 13424, 27060, 7352,  34480, 10644, 30952, 49048,
                14676, 55208, 32132, 50640, 61424, 38368, 30172, 8640,  52792,
                15380, 57996, 25228, 18836, 40488, 27892, 32508, 18420, 48692,
                2692,  33108, 17152, 7252,  40344, 53020, 3676,  33896, 9388,
                63700, 13752, 7008,  59144, 48940, 28712, 12244, 52664, 34132,
                62976, 10572, 36328, 40172, 18912, 59068, 11756, 41720, 6788,
                28588, 24820, 53876, 58312, 4108,  50052, 25944, 32812, 44172,
                57604, 50764, 28140, 3308,  46796, 18304, 40984, 5016,  59348,
                27000, 40176, 60600, 13076, 56988, 29392, 50352, 11124, 65436,
                23952, 44488, 49552, 58824, 7404,  204,   47372, 12004, 56984,
                40732, 16728, 13100, 24180, 47792, 11668, 57424, 4876,  39496,
                42876, 55228, 34064, 47608, 100,   21104, 35528, 11168, 49652,
                7980,  55284, 64920, 20584, 40112, 58360, 26704, 48556, 7668,
                51872, 17912,
            },
            {
                35896, 27592, 55688, 39276, 53644, 8316,  616,   51732, 45648,
                40492, 2356,  59540, 18828, 54596, 61452, 16336, 45824, 64488,
                19376, 40100, 27332, 1752,  19988, 55924, 14164, 49732, 5412,
                21596, 37944, 3516,  50668, 61576, 14916, 44232, 11396, 61052,
                5744,  21900, 45208, 49528, 25112, 15480, 61564, 22952, 57868,
                51044, 40660, 20576, 34920, 15624, 64812, 22392, 3952,  41160,
                17828, 49356, 2484,  31416, 24532, 5728,  52612, 22720, 56952,
                64468, 1964,  39436, 12800, 29760, 16252, 47048, 2868,  14284,
                20244, 41220, 10940, 38084, 53224, 8552,  21464, 56296, 11528,
                30728, 53952, 1336,  36508, 7336,  62040, 38840, 816,   41956,
                36340, 9056,  19448, 27020, 43224, 62060, 15852, 25284, 60440,
                6196,  52660, 30704, 38316, 8028,  18664, 31964, 61260, 21348,
                8432,  24140, 16244, 5788,  44144, 57296, 29304, 53708, 39324,
                26144, 4036,  32748, 47840, 6088,  45332, 10236, 64160, 38860,
                29152, 46084,
            },
            {
                63604, 7328,  2588,  18740, 26052, 34420, 61688, 30180, 14380,
                64864, 35844, 52368, 32376, 4264,  41140, 37696, 25996, 8480,
                52168, 4660,  59396, 11624, 34188, 42800, 25576, 62048, 33124,
                27732, 64288, 12380, 34076, 6772,  56736, 52176, 25916, 38432,
                55584, 27772, 63196, 36004, 644,   31704, 46940, 38156, 17340,
                1692,  29708, 53400, 24784, 42080, 45600, 8020,  56196, 30392,
                23816, 44984, 13784, 65444, 55040, 47584, 33992, 38376, 14056,
                49224, 33028, 45476, 21944, 36164, 59956, 23208, 55136, 65020,
                35876, 24984, 56576, 30532, 64284, 33240, 26288, 45304, 37296,
                48768, 19652, 24460, 47628, 43520, 26772, 22644, 47024, 17584,
                30680, 61660, 52940, 32796, 22160, 55620, 38620, 29412, 44524,
                34400, 48592, 20020, 58860, 2352,  51836, 43696, 27696, 2956,
                50612, 36256, 58676, 40256, 25764, 13496, 63768, 3332,  19524,
                59788, 42356, 17372, 12488, 23380, 30320, 36604, 16608, 56288,
                3828,  11288,
            },
            {
                15788, 40980, 59816, 46404, 12192, 43960, 37436, 23372, 5928,
                19968, 28020, 11328, 45080, 23304, 50964, 2212,  56968, 33888,
                42880, 23368, 16004, 47560, 65036, 6560,  17564, 44884, 1020,
                40572, 53416, 46860, 29896, 22900, 36356, 4228,  20608, 47360,
                34140, 12648, 9036,  59152, 19956, 55864, 5072,  43972, 27684,
                12336, 37204, 61120, 10376, 4472,  32480, 50324, 14956, 38812,
                59472, 53628, 6596,  42568, 16484, 27528, 2712,  44700, 8144,
                18520, 55192, 4984,  63328, 51264, 736,   39692, 11452, 27908,
                52308, 5660,  1712,  48304, 14160, 43248, 632,   62024, 16308,
                8988,  64352, 5300,  32100, 15932, 54436, 12624, 33560, 56132,
                5808,  15192, 2584,  37420, 12768, 4752,  50540, 9176,  2668,
                22980, 10348, 61712, 41636, 25532, 35280, 14468, 55656, 46276,
                18164, 64624, 10872, 52384, 31708, 7816,  48092, 33772, 45040,
                10304, 51444, 35356, 55908, 61812, 50656, 1836,  21664, 43444,
                25544, 52896,
            },
            {
                20412, 30568, 50564, 22520, 57240, 3336,  17608, 49736, 56236,
                42168, 48104, 58124, 7660,  63664, 14156, 29356, 21000, 12720,
                62732, 30304, 38868, 52868, 28848, 37404, 54796, 10924, 58792,
                19512, 8844,  16248, 43892, 63496, 10452, 42372, 58580, 1028,
                16060, 43680, 52480, 38836, 14112, 26300, 51472, 8048,  65280,
                56600, 47212, 2888,  18540, 57952, 26884, 62192, 20400, 368,
                9256,  27984, 21216, 33344, 9892,  62108, 20240, 58804, 25564,
                30056, 42360, 15672, 27244, 9488,  19088, 43172, 30916, 7528,
                17280, 45804, 60116, 22268, 17968, 58152, 6968,  51248, 23332,
                34524, 42260, 58076, 52316, 9740,  63344, 3868,  59584, 51300,
                28216, 45216, 57532, 48324, 63168, 31424, 41888, 18960, 65200,
                56560, 15136, 28796, 5020,  47032, 64268, 22396, 10368, 38580,
                32864, 28736, 1928,  21724, 61404, 18404, 40976, 24148, 14848,
                28372, 63016, 932,   25604, 8436,  40896, 59136, 32080, 13220,
                35168, 57828,
            },
            {
                45044, 9336,  33968, 4988,  28484, 52512, 60424, 9568,  26524,
                1208,  15768, 37864, 31256, 18052, 43632, 60308, 49628, 6212,
                46760, 276,   58340, 8852,  20376, 2560,  23932, 31500, 48412,
                35760, 56320, 26292, 1584,  49996, 18120, 28432, 62116, 31320,
                50264, 23880, 6204,  30140, 42384, 60444, 11116, 35684, 19492,
                24044, 31104, 42744, 50876, 35964, 39864, 11352, 47684, 35468,
                63936, 50692, 37596, 57680, 48884, 39880, 12556, 35608, 51944,
                60652, 11056, 37564, 56416, 46520, 33668, 53268, 60768, 48540,
                38236, 32752, 42028, 11868, 35648, 27544, 38864, 30208, 54860,
                2532,  12968, 38384, 28804, 21200, 40368, 35512, 19228, 8252,
                25248, 39592, 20148, 9832,  24468, 16236, 54512, 35272, 45680,
                25948, 37884, 53780, 32964, 17436, 58020, 1088,  49904, 5576,
                56788, 13664, 48728, 44500, 36816, 55100, 1272,  57736, 49416,
                5520,  20140, 38720, 44384, 13856, 18876, 53604, 5588,  63000,
                49148, 1192,
            },
            {
                24444, 61440, 38104, 65152, 15428, 32400, 39888, 46888, 35068,
                62276, 22240, 54020, 3408,  24896, 36048, 9952,  39476, 54816,
                17412, 35852, 25300, 14088, 45668, 51352, 41960, 63056, 13396,
                22796, 4100,  60676, 32252, 38048, 53776, 13548, 8240,  40832,
                19308, 64940, 57392, 2320,  48572, 22204, 32216, 40140, 54540,
                5772,  14260, 63220, 22316, 16068, 6412,  54956, 24736, 31488,
                42192, 15872, 4320,  24328, 1156,  30688, 54056, 5968,  46872,
                3460,  21612, 31872, 6800,  61960, 24228, 13312, 3372,  21040,
                63800, 9052,  24484, 52712, 62884, 49104, 10244, 20032, 46456,
                59916, 26020, 17788, 50152, 2152,  46412, 24164, 42920, 48792,
                13280, 64760, 34976, 43344, 1232,  60060, 27768, 6980,  12176,
                51208, 548,   43348, 8472,  12444, 39828, 30504, 26124, 62072,
                42344, 24508, 59600, 4288,  15576, 26908, 9376,  64792, 37180,
                32048, 54620, 58604, 29844, 48408, 34496, 27068, 45748, 9844,
                29420, 39924,
            },
            {
                6068,  17876, 47644, 10948, 43080, 21244, 6916,  13520, 30592,
                51924, 10476, 40348, 59596, 48632, 55812, 1416,  27320, 32596,
                22256, 56528, 41456, 62244, 32712, 59332, 16472, 4828,  29124,
                39008, 51852, 41820, 21016, 5792,  47220, 34824, 25552, 55480,
                37212, 10984, 27976, 34984, 17856, 62992, 3180,  15664, 48956,
                45212, 34368, 8888,  53468, 28752, 46016, 60596, 3520,  18428,
                57100, 12204, 46196, 60320, 18272, 43332, 63864, 23120, 16776,
                39384, 65116, 44348, 49992, 17524, 4772,  36988, 55628, 26552,
                14656, 57816, 29872, 5980,  2752,  42664, 56924, 14604, 4284,
                32520, 44416, 62464, 7556,  56828, 30852, 11148, 60840, 392,
                31828, 54780, 5156,  29812, 51552, 47004, 21392, 40324, 61020,
                31788, 21872, 62476, 49144, 19944, 54800, 44516, 36304, 15744,
                20028, 8160,  39364, 30404, 51084, 34776, 43184, 12972, 21824,
                45928, 16240, 7420,  22784, 4200,  65400, 38300, 17088, 22260,
                60340, 14668,
            },
            {
                50820, 56212, 27132, 564,   54160, 25132, 63304, 57096, 19104,
                4324,  45776, 28344, 13004, 33272, 20224, 64724, 15552, 44564,
                10824, 50500, 7356,  3640,  27368, 36224, 9544,  55892, 46616,
                7976,  65404, 15152, 11160, 57104, 60100, 22168, 2088,  45968,
                5268,  15376, 47176, 53648, 7784,  44316, 55728, 28908, 61880,
                592,   26488, 59308, 40860, 2228,  32832, 13444, 43656, 52584,
                7740,  26964, 34648, 53180, 28564, 14068, 8996,  33312, 27176,
                50916, 252,   14012, 25904, 40740, 58588, 29680, 45424, 42364,
                852,   49608, 40152, 16596, 21292, 33444, 25008, 65376, 40104,
                52900, 10556, 22232, 37196, 14708, 64216, 52044, 17020, 36864,
                58296, 22596, 17948, 11728, 58656, 36764, 14300, 3048,  48452,
                17568, 5732,  58364, 25212, 34624, 63616, 7048,  11412, 51340,
                54136, 33784, 46604, 11232, 61912, 19196, 53632, 29772, 4076,
                51540, 10712, 42120, 60388, 52320, 11720, 1992,  57012, 42620,
                54104, 36512,
            },
            {
                32028, 41928, 19976, 59012, 35828, 45452, 2944,  38056, 42612,
                24024, 64252, 16884, 5720,  42104, 8428,  46496, 52452, 4852,
                62932, 30428, 18896, 48600, 12320, 22376, 40268, 50232, 25364,
                19188, 34440, 44820, 27052, 30480, 16908, 42016, 63764, 52132,
                33040, 59704, 21132, 40972, 13032, 25440, 37040, 10108, 52040,
                20644, 38452, 12232, 49580, 19128, 64908, 37448, 21864, 29808,
                39928, 63404, 22252, 5704,  38212, 49564, 55968, 41792, 59680,
                11600, 29904, 56932, 34968, 8828,  21880, 51600, 10520, 34312,
                19424, 53680, 36396, 61064, 55740, 47180, 8212,  36688, 17232,
                29072, 5532,  48596, 33948, 43792, 25696, 4952,  28956, 41800,
                7684,  46272, 52988, 41032, 25292, 8376,  63928, 34080, 57036,
                29560, 42112, 37288, 14692, 28996, 4160,  23212, 46200, 29416,
                2244,  65236, 22024, 56956, 6520,  25088, 2448,  58748, 39940,
                62640, 27168, 36028, 18388, 31180, 25052, 50284, 32732, 7760,
                26140, 3112,
            },
            {
                63828, 8308,  29960, 12936, 50192, 17156, 29028, 51364, 8372,
                34012, 56360, 36780, 52952, 61872, 30152, 23724, 35320, 40556,
                26248, 57564, 38568, 43964, 53552, 64208, 18056, 3244,  58412,
                30832, 312,   54532, 48484, 3468,  38480, 7920,  12284, 29208,
                24020, 38752, 72,    64080, 31576, 58284, 4236,  23268, 42560,
                16996, 30476, 56836, 6844,  24504, 55268, 9540,  50480, 58864,
                784,   48164, 10360, 44536, 61308, 1776,  21076, 4632,  45816,
                37052, 53076, 19748, 62396, 47660, 1848,  64620, 16816, 59760,
                6716,  27628, 10056, 44584, 13192, 30820, 76,    58840, 49836,
                23684, 61108, 56136, 19156, 1432,  39568, 55064, 20460, 50000,
                13592, 27220, 62528, 2084,  32360, 45092, 19132, 23656, 52328,
                8864,  11864, 46864, 1700,  51772, 41392, 57136, 18668, 60892,
                38064, 5100,  16720, 28124, 41240, 49000, 45400, 15720, 33740,
                24000, 47988, 240,   55332, 39356, 46652, 15092, 62044, 19904,
                46128, 12048,
            },
            {
                52728, 22844, 38772, 61796, 5736,  33532, 10604, 58220, 15104,
                47824, 40,    20496, 26036, 49620, 3168,  18348, 60648, 13564,
                2144,  21572, 15404, 59752, 844,   29600, 33852, 43448, 13688,
                37768, 61928, 9820,  23712, 63044, 51348, 32164, 56728, 18292,
                44400, 9580,  48872, 27248, 50928, 18888, 46460, 60256, 33636,
                5232,  63648, 44132, 35208, 47400, 28016, 17108, 5240,  33544,
                14788, 19284, 54692, 31308, 16836, 35988, 26044, 63272, 18584,
                7364,  24096, 4252,  43452, 31524, 12988, 37428, 24588, 47040,
                32068, 62572, 22736, 3848,  25832, 51304, 19044, 43512, 11652,
                3500,  41704, 31176, 12716, 59976, 47448, 8840,  33204, 63552,
                3144,  34636, 38308, 15700, 49636, 54380, 6124,  38464, 44128,
                27184, 64804, 55868, 19272, 60504, 10100, 48764, 33160, 13408,
                24300, 43976, 49888, 35920, 13092, 63456, 37444, 8384,  20044,
                56992, 5416,  14360, 63984, 8288,  21236, 4888,  40428, 35728,
                58876, 16736,
            },
            {
                34336, 44284, 2456,  47440, 24348, 41572, 62620, 21508, 26952,
                60816, 31484, 44268, 9592,  14740, 38780, 57084, 10224, 43580,
                54280, 31848, 35032, 8464,  24176, 46508, 6020,  61172, 53208,
                21708, 41224, 17752, 35832, 13992, 42956, 20688, 48048, 4516,
                61832, 55164, 16792, 3652,  35936, 11448, 39536, 14632, 53564,
                48180, 9640,  22188, 15128, 1276,  61332, 41592, 45760, 25792,
                36440, 60008, 41876, 24368, 12356, 52064, 40748, 32200, 14556,
                58948, 48864, 39068, 15940, 27128, 54152, 41812, 4480,  56264,
                13904, 39156, 49308, 34944, 63976, 39512, 55120, 27840, 63312,
                35808, 15596, 53756, 7088,  28108, 23392, 15936, 59084, 44900,
                24584, 56496, 9876,  21124, 59268, 27996, 12652, 60680, 15572,
                2848,  22224, 35212, 30844, 38784, 16000, 27284, 44,    39956,
                58908, 31252, 9652,  55840, 836,   22780, 29508, 59660, 11664,
                43044, 53016, 28328, 34116, 44124, 57840, 30068, 53348, 780,
                28456, 49128,
            },
            {
                6200,  59984, 15020, 56628, 18992, 53204, 1796,  46224, 39860,
                4768,  12428, 54468, 41000, 59368, 32656, 47224, 28608, 50384,
                6740,  64936, 47788, 11828, 37044, 56108, 15956, 26320, 10632,
                6808,  49196, 57752, 5112,  28384, 58644, 1092,  25468, 37612,
                14176, 34392, 42624, 23744, 56928, 7432,  62912, 27940, 2896,
                25244, 40676, 59552, 53012, 31984, 38888, 12376, 63944, 52268,
                8616,  28304, 2508,  65272, 7572,  57852, 47884, 9916,  54516,
                28596, 34160, 10676, 56056, 61252, 7924,  49060, 28944, 20380,
                43232, 1528,  18064, 56860, 5308,  15496, 9452,  33352, 20968,
                47776, 25272, 38424, 45164, 65028, 52276, 37552, 11616, 29612,
                18168, 40788, 47696, 4428,  31064, 42648, 300,   55616, 32188,
                50096, 41188, 4712,  53656, 24144, 7672,  43672, 64296, 54220,
                6648,  51604, 20632, 60476, 33024, 17492, 53928, 4452,  47356,
                31780, 38708, 17224, 49344, 10884, 26268, 63268, 13260, 43276,
                9824,  21088,
            },
            {
                41072, 26088, 32268, 9116,  28380, 36820, 13264, 32524, 55540,
                18392, 50712, 23124, 65492, 5512,  21672, 928,   36876, 24636,
                19984, 40068, 27472, 52644, 62176, 20112, 39944, 50884, 29120,
                45320, 32484, 25736, 52532, 46792, 8880,  33540, 64912, 53212,
                27748, 6056,  60660, 30260, 52492, 44716, 21800, 32276, 55528,
                36968, 12896, 29144, 4208,  19316, 55764, 23676, 2856,  21012,
                56720, 38392, 50644, 45844, 34752, 29272, 3480,  23188, 42932,
                1280,  64340, 45272, 3528,  18812, 22932, 35304, 63560, 9356,
                52028, 59292, 11704, 28744, 42272, 23156, 45980, 6472,  57704,
                1372,  61584, 10020, 20392, 34288, 5272,  42264, 620,   50972,
                6892,  60480, 52628, 23920, 65488, 17556, 35676, 46808, 24828,
                62568, 18116, 13380, 45444, 62944, 50824, 34924, 21780, 17680,
                11672, 26920, 45972, 3948,  39176, 44452, 50696, 26348, 65112,
                21556, 7060,  61408, 23324, 2184,  36852, 17884, 48396, 24108,
                56456, 64788,
            },
            {
                12180, 54812, 37948, 50652, 64356, 44148, 7184,  25280, 63424,
                37628, 29888, 7396,  35480, 27380, 17208, 52404, 63204, 14548,
                55852, 2680,  17332, 4628,  42764, 31392, 1452,  59320, 35932,
                63432, 12944, 2364,  19392, 60460, 39604, 16600, 11552, 41112,
                21548, 49772, 10124, 19772, 40432, 1856,  16064, 49468, 8720,
                19952, 65424, 51164, 46384, 36200, 9988,  48712, 30536, 43788,
                13132, 17792, 5440,  22436, 14292, 18984, 62748, 36360, 16488,
                51216, 20716, 25224, 32364, 50832, 40796, 408,   15340, 31312,
                36760, 22272, 46516, 32644, 53976, 60992, 36324, 52840, 14288,
                28788, 42676, 50944, 2924,  16744, 58228, 31508, 55576, 21828,
                36300, 14580, 32724, 11788, 39412, 7800,  51596, 20712, 10164,
                37556, 7072,  59376, 28572, 11164, 3116,  57876, 29916, 47144,
                37148, 41712, 15492, 64004, 25180, 9980,  14392, 41648, 1344,
                35848, 13180, 58376, 40288, 51436, 55972, 7192,  32892, 38648,
                3664,  30880,
            },
            {
                46260, 5324,  17844, 416,   22228, 57440, 16544, 48300, 3212,
                10472, 43856, 15676, 48660, 55424, 44424, 11500, 41584, 7700,
                32432, 45492, 59140, 23588, 50240, 9684,  14560, 22556, 5352,
                17320, 55984, 37252, 44100, 30376, 23140, 55448, 45516, 3796,
                31260, 57872, 46732, 13660, 64584, 33816, 58084, 38564, 60628,
                384,   43380, 16832, 7116,  58492, 27084, 60728, 5848,  35160,
                62532, 48108, 32316, 59048, 40328, 52464, 44920, 56592, 5856,
                39724, 59620, 13680, 37636, 58256, 11792, 53540, 60316, 45404,
                26384, 3660,  65184, 8012,  19444, 1916,  26376, 49168, 40184,
                18336, 31860, 55392, 23060, 49028, 26848, 13652, 47344, 64056,
                44108, 28244, 2024,  46356, 57648, 26956, 61860, 4580,  29972,
                57348, 43272, 48272, 33492, 20128, 38356, 14620, 5904,  61692,
                2516,  52780, 30740, 7748,  55400, 36196, 61104, 30420, 18372,
                54952, 44308, 27668, 3932,  30812, 20876, 45720, 61200, 15528,
                51824, 19244,
            },
            {
                24884, 49540, 61980, 41944, 11176, 30488, 35416, 59684, 20168,
                51868, 61448, 24648, 39288, 4552,  33608, 58836, 29672, 48740,
                61816, 36352, 13104, 28660, 34432, 64392, 47704, 43652, 53528,
                24776, 40928, 9596,  64336, 14636, 5940,  35764, 50780, 18484,
                62988, 692,   37972, 25924, 6416,  28452, 11796, 23948, 45236,
                30736, 26620, 34272, 22652, 42188, 14060, 52564, 16588, 39992,
                25684, 10188, 54296, 26764, 308,   9136,  24640, 12252, 31948,
                27180, 8860,  47668, 5572,  28144, 44024, 24424, 18244, 6092,
                55044, 41004, 16356, 50396, 38616, 12904, 31116, 9816,  64552,
                4808,  59244, 12324, 37208, 63368, 40128, 6464,  19368, 25348,
                9320,  61484, 54292, 20072, 34192, 15176, 44672, 40816, 53844,
                14660, 26428, 2304,  22948, 56552, 41476, 49432, 25964, 55188,
                23512, 34244, 58712, 19456, 47600, 21988, 5756,  57652, 48788,
                24084, 9688,  47196, 16660, 62584, 12684, 41288, 1152,  27240,
                59464, 35992,
            },
            {
                57740, 13868, 33188, 27156, 47112, 4596,  53916, 40536, 28000,
                33196, 688,   57340, 13360, 62256, 20848, 1872,  25780, 17900,
                22412, 10068, 40968, 55036, 5696,  18500, 37620, 57508, 33008,
                3036,  48800, 21376, 51440, 1332,  26932, 61664, 8256,  24248,
                43304, 34912, 15844, 51840, 42976, 54368, 47528, 18492, 4816,
                53496, 57380, 10772, 61964, 38360, 1016,  32708, 45168, 57924,
                2080,  20552, 42128, 64324, 37028, 30372, 60920, 49132, 19752,
                62412, 42684, 54744, 17312, 64976, 3312,  33324, 39464, 48320,
                13008, 57504, 34744, 24848, 62296, 43104, 58528, 21848, 45064,
                24792, 34868, 7552,  45964, 29848, 10436, 59644, 35180, 3644,
                38724, 16264, 41376, 5092,  50388, 10492, 23400, 1376,  32868,
                19324, 64852, 36404, 52472, 9028,  60860, 18456, 32800, 43840,
                13376, 10096, 49724, 1212,  28720, 40948, 12196, 33752, 3220,
                38932, 63508, 32948, 52592, 8712,  35036, 49460, 22600, 10656,
                44372, 8004,
            },
            {
                1888,  20864, 38828, 52704, 60880, 15004, 23836, 8036,  12572,
                46724, 41992, 18592, 31588, 9644,  45672, 54064, 39680, 6356,
                57436, 51996, 220,   46836, 60820, 26564, 7788,  29588, 11532,
                62300, 27792, 35108, 58572, 31968, 40716, 47976, 57072, 13176,
                28008, 54980, 8896,  22380, 61984, 2460,  35800, 63892, 9216,
                40844, 15016, 47128, 6344,  50648, 20288, 64720, 23224, 8080,
                29712, 51332, 15656, 4892,  46968, 17408, 43188, 3140,  34784,
                52716, 940,   30576, 36100, 21552, 50452, 9620,  61804, 29204,
                20284, 8504,  30448, 2788,  47248, 5404,  15524, 55916, 364,
                39168, 16092, 53600, 19616, 1268,  42712, 50756, 53908, 31840,
                57332, 49324, 22512, 31016, 63276, 38052, 55396, 60732, 47008,
                11896, 50336, 5836,  16144, 30904, 45828, 1760,  7984,  65392,
                20884, 39976, 62272, 16364, 44992, 53312, 64492, 26508, 51328,
                14804, 20592, 4820,  42572, 25020, 58168, 29312, 53304, 63216,
                32244, 40720,
            },
            {
                29388, 65380, 9632,  3172,  19300, 44264, 36848, 64244, 56332,
                22460, 5824,  52436, 26576, 49260, 36748, 14760, 64836, 34212,
                44216, 30792, 16780, 21636, 39100, 13836, 52776, 20668, 42296,
                16112, 7080,  46328, 17928, 54576, 10392, 20428, 38268, 33180,
                5620,  59732, 48492, 30984, 19432, 39420, 14696, 32420, 51144,
                21300, 36516, 24956, 31472, 54964, 28740, 11268, 47876, 55352,
                38200, 60304, 33224, 10972, 56732, 21744, 53904, 7964,  38956,
                15352, 23128, 11256, 57024, 46720, 14752, 58504, 36544, 1296,
                51292, 43528, 59764, 22488, 52116, 28280, 36584, 33044, 50536,
                62844, 28104, 48256, 61640, 25860, 14208, 22804, 17720, 45372,
                12036, 792,   26468, 58924, 7320,  17756, 28312, 35888, 8400,
                24208, 39328, 42952, 61840, 26972, 12620, 53260, 36992, 29064,
                46788, 5476,  35520, 25448, 4296,  37504, 8816,  17960, 44576,
                60392, 29052, 36868, 55060, 18416, 2580,  39136, 14044, 4656,
                17304, 55272,
            },
            {
                50200, 43124, 35268, 25728, 55828, 31468, 50848, 1904,  17252,
                35008, 59428, 38340, 63164, 2808,  29252, 23452, 50676, 3588,
                13604, 27256, 63676, 33396, 4576,  58304, 45820, 1644,  49496,
                59508, 39808, 12772, 4776,  25124, 44628, 15712, 3324,  65232,
                45260, 17716, 40752, 4456,  57920, 10564, 25436, 55956, 28272,
                3700,  59432, 63468, 2608,  16348, 43420, 4300,  35444, 18376,
                13448, 43712, 23984, 50072, 35700, 28940, 13148, 58332, 26136,
                46080, 63296, 40724, 26724, 6896,  31376, 42616, 22984, 26984,
                63140, 37892, 14520, 10524, 40620, 18084, 54200, 11144, 20476,
                8180,  41640, 4192,  32608, 55932, 36808, 60412, 5488,  27948,
                65044, 42516, 34640, 13656, 44004, 47980, 21272, 4104,  52180,
                59820, 29468, 684,   55672, 34864, 19676, 40764, 59128, 15636,
                56948, 50512, 31448, 55256, 60632, 13996, 30660, 56324, 40236,
                208,   11620, 58732, 48172, 6948,  64748, 44932, 20404, 37092,
                46764, 22964,
            },
            {
                5208,  15140, 57940, 48032, 6432,  10804, 41180, 26424, 48520,
                30228, 10016, 15372, 43552, 8116,  56868, 19048, 10624, 41616,
                59648, 49152, 9064,  54712, 42808, 24940, 31236, 65388, 34168,
                23292, 55348, 30716, 63968, 35564, 60272, 51072, 30188, 52772,
                23096, 11816, 36528, 26976, 50888, 46076, 60940, 7040,  48560,
                43032, 11908, 19400, 39516, 49040, 58448, 25572, 41340, 62996,
                2980,  27636, 7456,  63480, 1604,  40596, 61496, 33100, 50472,
                4120,  18752, 59572, 51856, 2148,  17972, 53832, 11992, 4408,
                17200, 46292, 54832, 33736, 64656, 6956,  60492, 26500, 46304,
                14448, 58608, 22100, 10220, 44600, 3000,  30468, 48760, 9576,
                37724, 18624, 51560, 56172, 3016,  39756, 64368, 15488, 32280,
                45532, 17484, 21576, 10136, 47164, 63620, 6220,  22580, 26504,
                472,   11476, 18512, 22876, 42772, 48016, 20832, 6504,  24476,
                49824, 31884, 22752, 14336, 27384, 32668, 9372,  56492, 60488,
                27644, 11640,
            },
            {
                61548, 32464, 18308, 39248, 63240, 14356, 20740, 60464, 4444,
                44724, 54612, 25156, 21024, 33292, 60432, 46332, 54592, 37664,
                24540, 20040, 2036,  35924, 50716, 11364, 17712, 8660,  37992,
                3656,  19648, 52348, 43068, 160,   21952, 7900,  41440, 26520,
                1896,  62360, 54008, 15324, 34200, 488,   41448, 22920, 17048,
                34568, 53956, 45380, 8064,  32932, 12808, 61256, 6748,  30668,
                48544, 54208, 19948, 45508, 16700, 24768, 6180,  20116, 10464,
                29924, 36484, 8420,  32876, 45188, 38908, 64040, 48640, 56504,
                31908, 6536,  25592, 984,   48772, 21652, 43708, 2676,  37652,
                30212, 53256, 35116, 18060, 51908, 63608, 40776, 15364, 54340,
                21660, 61856, 8140,  24868, 30052, 11172, 54440, 25624, 41728,
                12884, 57860, 38248, 49688, 25000, 3484,  43548, 51704, 33820,
                38964, 44504, 63708, 8208,  2536,  33916, 59504, 52572, 35452,
                63780, 9112,  46100, 38400, 52052, 41972, 23664, 49312, 476,
                34592, 52100,
            },
            {
                25240, 45524, 2740,  23200, 29068, 52140, 33692, 56856, 37120,
                13236, 64968, 912,   51200, 40540, 4856,  15984, 30096, 7376,
                32768, 45940, 62456, 15652, 22624, 60924, 41236, 27848, 51240,
                14724, 26656, 9180,  38656, 28516, 48676, 13984, 58788, 37268,
                19360, 47424, 8052,  59908, 29776, 19876, 12820, 38116, 62632,
                2192,  29816, 26464, 57020, 22628, 36664, 17552, 52884, 21852,
                34248, 59468, 11976, 37056, 57320, 52568, 43040, 48080, 64480,
                55800, 44036, 13980, 24496, 61224, 20988, 28204, 8024,  41360,
                36124, 19580, 58408, 29564, 38612, 14992, 31712, 56944, 49952,
                5524,  65452, 25108, 39112, 12908, 7108,  23972, 34340, 59092,
                2596,  32236, 46204, 16176, 59932, 36944, 49124, 6888,  61936,
                3384,  52844, 7248,  32928, 15052, 58248, 30164, 8940,  13728,
                59876, 53480, 28184, 36652, 51528, 10736, 27468, 16048, 3864,
                42856, 19692, 61480, 1652,  17120, 59840, 12124, 30256, 16420,
                41396, 8572,
            },
            {
                37508, 53720, 12140, 59284, 42896, 124,   46492, 9148,  23728,
                18768, 39380, 28520, 46844, 11884, 35956, 26472, 63808, 2972,
                58100, 12084, 38872, 29704, 6580,  47304, 664,   56376, 45112,
                62552, 58104, 47564, 12040, 61900, 18440, 32872, 54720, 10444,
                31132, 42244, 25036, 3740,  43624, 64828, 53252, 31688, 57676,
                9660,  50584, 15424, 65072, 144,   49828, 44376, 9924,  39448,
                15580, 1144,  41832, 28680, 9184,  31164, 2464,  37976, 15884,
                22304, 192,   49240, 54716, 10744, 3188,  34652, 14312, 23352,
                60744, 50144, 12976, 42420, 53296, 9092,  62648, 23152, 11484,
                17028, 42868, 1960,  47796, 57052, 28748, 19052, 46908, 42304,
                26148, 5596,  40360, 50184, 22352, 128,   33384, 19796, 29140,
                36364, 23408, 64608, 27808, 54052, 39232, 18216, 63020, 48924,
                21108, 6424,  16232, 41404, 24708, 61736, 46264, 38824, 57572,
                13036, 28844, 55456, 34448, 5004,  54244, 36504, 5864,  64188,
                55408, 19900,
            },
            {
                62852, 30564, 49180, 7992,  35576, 16644, 26892, 61716, 5852,
                53464, 32256, 57972, 17040, 62404, 53288, 44176, 20648, 48464,
                52020, 17836, 43252, 56920, 53056, 32468, 36188, 12932, 18884,
                31404, 5960,  34656, 23796, 56176, 3776,  45432, 6076,  64176,
                49800, 16712, 34848, 55444, 22448, 9228,  48124, 5932,  25012,
                42248, 20964, 39128, 6224,  31152, 55300, 27348, 3944,  57664,
                46244, 25160, 65448, 49572, 22096, 62328, 13308, 33912, 27004,
                40272, 58796, 29032, 17276, 38184, 46784, 57892, 52760, 44572,
                3996,  9788,  63908, 24032, 4624,  18360, 47432, 41124, 33996,
                59748, 28012, 9248,  61668, 32084, 50796, 456,   10816, 52104,
                13756, 55544, 35540, 9244,  62740, 42840, 13724, 56064, 44676,
                10716, 46756, 18964, 42424, 1120,  11236, 46048, 35744, 25504,
                2200,  31904, 47636, 57912, 13256, 19208, 936,   32340, 22500,
                49068, 6708,  40864, 25700, 21144, 43880, 27092, 48200, 22444,
                44736, 4028,
            },
            {
                27204, 15532, 39912, 20540, 64576, 54352, 31436, 40312, 50372,
                43256, 3976,  9772,  22852, 2260,  31196, 8756,  13340, 39708,
                23808, 28320, 9224,  3736,  25776, 20568, 63716, 7704,  24412,
                43564, 2480,  40900, 16296, 51624, 36052, 25692, 39428, 22140,
                812,   57632, 12600, 38712, 51068, 15812, 27812, 36908, 45740,
                13124, 61724, 34948, 46668, 11492, 41580, 19480, 63324, 30080,
                51156, 7996,  33164, 4556,  16216, 45812, 54412, 60088, 8092,
                51744, 5388,  35232, 42780, 65276, 6388,  31096, 26308, 16592,
                39092, 28800, 45568, 34360, 56420, 36900, 26340, 508,   54172,
                20088, 46052, 37132, 15112, 21204, 43716, 60528, 38528, 64092,
                30940, 19640, 58532, 28784, 18200, 53236, 26672, 39576, 59016,
                2272,  51484, 13920, 35412, 61744, 22660, 56752, 6720,  41756,
                55340, 38204, 65128, 3824,  30324, 55572, 44096, 64100, 9564,
                53452, 31328, 14996, 65240, 50164, 8904,  61116, 13488, 39504,
                33064, 10828,
            },
            {
                59040, 1512,  56068, 24908, 47816, 3820,  10652, 19940, 15232,
                25732, 34728, 61084, 47996, 37716, 25460, 56340, 59900, 33664,
                772,   64420, 35200, 59220, 15044, 40148, 48344, 54912, 38488,
                60240, 50736, 21100, 64496, 8848,  29824, 60972, 14964, 52824,
                28600, 44812, 61288, 5296,  30956, 59088, 1148,  18680, 55216,
                4336,  58780, 17148, 53308, 23768, 59796, 14436, 35892, 23328,
                12592, 38700, 19192, 55856, 36436, 25960, 3472,  19040, 43432,
                12108, 61808, 20484, 25548, 13428, 50048, 19660, 696,   62876,
                55296, 20768, 1864,  15556, 51120, 60936, 7808,  29820, 13536,
                50252, 6140,  24256, 54760, 3668,  35712, 26604, 16964, 7384,
                23428, 45012, 1624,  12372, 46388, 5380,  65484, 8108,  16928,
                24384, 29756, 59988, 5096,  49388, 31556, 27496, 15508, 61060,
                11504, 18432, 23508, 9856,  35188, 50020, 5628,  26172, 36616,
                17888, 44756, 57820, 2932,  38644, 18464, 31044, 1188,  57640,
                17536, 50932,
            },
            {
                35772, 42816, 6324,  33136, 13084, 44652, 37168, 56712, 64028,
                45640, 12216, 55264, 40996, 19112, 50060, 4256,  42756, 15796,
                54884, 47036, 20000, 44656, 51256, 10516, 3080,  29488, 15952,
                11304, 26744, 32568, 46692, 1412,  43368, 19936, 11280, 41732,
                33612, 8724,  26040, 20252, 46536, 40616, 63872, 33988, 51356,
                22172, 32152, 1480,  27928, 8600,  37788, 2068,  47848, 5372,
                61472, 53504, 43904, 59836, 11136, 41212, 50844, 30016, 24064,
                47512, 31740, 2048,  56832, 8912,  33528, 59940, 42108, 11696,
                35500, 49272, 59188, 32176, 11016, 21452, 43920, 64400, 39732,
                57428, 31528, 63544, 41464, 12128, 58416, 5060,  53864, 48208,
                36812, 60780, 40876, 51936, 32856, 38004, 22824, 31428, 47896,
                34600, 55420, 39964, 9760,  19892, 43384, 2656,  51424, 33296,
                46552, 28824, 43796, 52416, 39804, 21580, 14568, 41308, 60352,
                4528,  24360, 33864, 10792, 47236, 55716, 35616, 52072, 24976,
                7736,  46188,
            },
            {
                23104, 19076, 52284, 60788, 28088, 58292, 23492, 7216,  30176,
                1884,  21428, 27820, 6784,  14664, 65496, 35292, 21732, 7084,
                30756, 11480, 37592, 4800,  28004, 60532, 21904, 34344, 45504,
                57660, 6952,  53868, 13348, 38152, 58708, 55432, 48608, 3368,
                63576, 17996, 49396, 36260, 10888, 23116, 14076, 43372, 10248,
                26280, 39892, 49116, 44020, 64200, 51428, 56220, 32744, 42284,
                17036, 27260, 512,   22908, 32172, 7288,  57980, 38216, 64568,
                15276, 54080, 38884, 44364, 52024, 22472, 37480, 47632, 7880,
                30032, 24284, 6160,  41544, 27280, 48432, 3968,  16804, 34932,
                10312, 1564,  18400, 49368, 29616, 45900, 32848, 20836, 10180,
                28068, 4056,  15708, 25600, 56804, 2988,  50100, 12764, 61324,
                6268,  44028, 16076, 26240, 53716, 37320, 64532, 8876,  21804,
                54452, 32,    59864, 6188,  16676, 62348, 29188, 54040, 47408,
                12240, 51228, 62952, 21492, 28768, 14536, 5976,  63804, 40988,
                29512, 61416,
            },
            {
                13760, 31144, 9140,  39036, 16516, 1068,  42204, 48876, 52876,
                36180, 59164, 51052, 32628, 44748, 10600, 27464, 52608, 60888,
                25940, 49932, 62388, 17384, 32328, 42256, 53276, 65096, 908,
                19500, 36216, 62692, 28872, 18284, 24920, 6232,  30976, 22616,
                37500, 54320, 4476,  62268, 53200, 3552,  57324, 29608, 6604,
                62788, 56060, 12604, 19852, 30308, 15848, 26056, 21256, 58596,
                9628,  30860, 49052, 14200, 63408, 47148, 17264, 1720,  9680,
                35872, 6048,  26864, 16648, 63120, 2728,  27764, 17824, 53784,
                44436, 13772, 65004, 18636, 54660, 37732, 58640, 24560, 52600,
                22032, 44704, 37996, 25796, 7928,  15764, 65104, 39440, 56620,
                50364, 33812, 63844, 20956, 9556,  60044, 18916, 41432, 52944,
                21208, 528,   63236, 33496, 57896, 12788, 47736, 17772, 40276,
                36088, 13816, 25892, 48476, 56580, 33596, 8056,  1732,  19740,
                30544, 39540, 396,   42428, 59912, 24072, 44492, 20372, 11692,
                48944, 2388,
            },
            {
                55304, 65308, 45868, 49708, 21920, 62564, 33516, 18384, 14116,
                9492,  39028, 17400, 61852, 272,   57336, 38484, 16944, 45160,
                40264, 1664,  22904, 54328, 8016,  13492, 25324, 5480,  40388,
                49256, 23648, 42588, 4088,  49856, 10320, 35020, 45728, 59460,
                13420, 27508, 43468, 15316, 32528, 25336, 48536, 37464, 46120,
                16220, 35556, 5200,  60004, 3360,  41200, 7112,  11952, 45016,
                64588, 38072, 52496, 5032,  35336, 28120, 21248, 44520, 56584,
                22356, 49680, 60792, 10868, 30524, 41008, 14352, 58120, 4360,
                61516, 38816, 51548, 2404,  33712, 14328, 9200,  46548, 28780,
                61024, 14756, 55988, 62436, 52500, 22732, 1072,  42908, 13116,
                18652, 6552,  42276, 47692, 29548, 44296, 35868, 27080, 10440,
                37432, 28656, 46036, 7532,  23308, 3424,  30060, 24788, 58384,
                4392,  63628, 31316, 37920, 11048, 24068, 43128, 58160, 36092,
                25840, 55896, 16760, 7368,  36744, 49844, 3216,  32676, 56744,
                16808, 37336,
            },
            {
                42300, 24196, 3376,  11836, 36388, 5368,  53948, 28904, 63400,
                24728, 47320, 5312,  41564, 21048, 29944, 48716, 3276,  9132,
                55844, 14460, 34960, 43636, 58320, 38136, 50344, 16988, 31100,
                55940, 8692,  15148, 60636, 39552, 53424, 65328, 16140, 40664,
                1988,  57432, 30528, 7744,  41332, 59852, 18296, 2640,  54088,
                21384, 42164, 24876, 33192, 46596, 37100, 50140, 54752, 34632,
                2604,  24304, 19824, 41016, 54632, 60664, 11920, 51988, 29108,
                32728, 42200, 19428, 46444, 5052,  56244, 49112, 23696, 32960,
                19864, 26684, 10416, 57228, 43320, 30612, 63004, 2836,  41388,
                7280,  32968, 4140,  11420, 34700, 47728, 59812, 30596, 25880,
                61924, 54068, 37744, 13456, 840,   54604, 15520, 3896,  64036,
                56652, 13560, 49976, 18844, 52252, 41980, 61280, 44852, 7304,
                49440, 17068, 42448, 20480, 2984,  46268, 64960, 13900, 49140,
                9300,  61152, 45448, 52224, 31496, 10528, 53964, 38948, 62168,
                27760, 4908,
            },
            {
                53320, 34056, 28700, 59448, 55756, 26092, 8444,  40356, 45068,
                3340,  31760, 54876, 26188, 51660, 12796, 62904, 23784, 33148,
                19856, 64732, 47904, 26228, 2432,  20444, 46032, 61772, 12440,
                26992, 44444, 33716, 21568, 30336, 832,   26668, 20788, 51676,
                9744,  23760, 47252, 51008, 21008, 34936, 9956,  64452, 28152,
                8268,  61672, 53140, 10944, 57376, 22668, 62112, 17872, 27536,
                6468,  57412, 46308, 16352, 8920,  25344, 38976, 4132,  59380,
                13872, 440,   55460, 24780, 33872, 36796, 64520, 10028, 42728,
                848,   36176, 45984, 23036, 6796,  20524, 50208, 17628, 36344,
                58024, 50920, 27232, 40332, 19536, 5984,  37304, 9072,  45684,
                3164,  22128, 31372, 58144, 24572, 62424, 33768, 47272, 23560,
                42788, 32040, 59512, 39180, 10740, 35048, 15132, 55016, 33000,
                27800, 53132, 8356,  61648, 55132, 28396, 17188, 38968, 32136,
                3584,  23072, 13412, 27160, 19380, 64856, 15724, 25524, 8976,
                44892, 20692,
            },
            {
                6856,  17648, 40852, 14808, 44412, 20208, 48440, 12676, 56808,
                22080, 60244, 10708, 15752, 34676, 7312,  42884, 36832, 58964,
                29280, 11296, 5328,  30652, 60712, 10160, 33808, 6660,  37236,
                58436, 2524,  52036, 46472, 57700, 12724, 44072, 7268,  33892,
                62556, 38660, 17496, 63680, 344,   55512, 13156, 39220, 50300,
                31592, 14616, 48152, 18632, 28792, 360,   14276, 31672, 40472,
                48856, 13328, 62340, 32988, 1348,  43096, 48624, 18640, 36984,
                63852, 8716,  39500, 51740, 12744, 20852, 7264,  29200, 52624,
                59692, 49484, 16008, 28460, 59216, 38100, 55032, 25956, 12272,
                21064, 46952, 16684, 43700, 55436, 24364, 57616, 17576, 51680,
                35420, 11376, 49248, 8044,  17672, 39332, 6176,  51060, 8684,
                17476, 2188,  25720, 5276,  62936, 27372, 992,   20352, 11712,
                39584, 22868, 45248, 13296, 34304, 49868, 4512,  21212, 53196,
                63232, 40924, 34148, 59228, 4960,  46180, 41768, 712,   35284,
                48456, 58388,
            },
            {
                11012, 63556, 50856, 1980,  31812, 60568, 37384, 580,   34352,
                18148, 49864, 37528, 64320, 45772, 55592, 18332, 4412,  52936,
                45584, 38996, 51536, 17728, 41664, 55160, 28480, 51668, 22676,
                18516, 63252, 11216, 5652,  17652, 36712, 61192, 49016, 28928,
                54808, 12404, 5452,  36376, 29100, 45472, 26304, 22536, 44360,
                1472,  36660, 4728,  39452, 65332, 9012,  43308, 53560, 59520,
                9912,  36452, 29320, 22748, 56032, 65216, 30464, 6384,  23216,
                45132, 27492, 17212, 61820, 3392,  43480, 47192, 16912, 39600,
                12568, 6512,  63720, 34280, 47916, 10760, 96,    44808, 65300,
                30384, 3124,  62008, 31924, 1840,  49404, 28528, 14640, 60804,
                41300, 27264, 64884, 44800, 52732, 28408, 21512, 60752, 30280,
                54768, 36416, 47968, 21608, 44432, 56180, 38108, 50660, 64868,
                3568,  60040, 36016, 1424,  25092, 41036, 60472, 7796,  27532,
                44448, 17940, 2276,  54776, 37328, 29192, 56280, 22788, 60348,
                13504, 30624,
            },
            {
                26168, 47288, 35596, 23192, 9968,  52152, 16012, 64980, 29880,
                43336, 7536,  27932, 1636,  22772, 30588, 61164, 25616, 15008,
                1132,  21924, 57528, 35360, 12844, 23848, 64508, 648,   47872,
                42076, 31684, 38624, 27988, 54188, 23900, 41532, 4276,  19036,
                46000, 25896, 42524, 58992, 15040, 52364, 5912,  61220, 56692,
                19436, 59300, 25236, 51948, 45340, 34800, 23860, 5360,  20144,
                25920, 44212, 51800, 4692,  12348, 34828, 51148, 14672, 57512,
                53396, 33648, 48268, 30912, 58896, 27044, 54504, 62228, 24556,
                31632, 21260, 53896, 3856,  14488, 61336, 32292, 40052, 53568,
                8796,  24676, 52092, 13908, 10148, 64504, 39652, 33012, 296,
                55092, 5768,  19728, 33428, 2308,  42140, 11588, 45456, 14412,
                40512, 65208, 11916, 53328, 16104, 31076, 9020,  24448, 46716,
                16284, 30000, 56772, 19200, 52488, 31652, 11284, 55492, 36292,
                12644, 51432, 47748, 9616,  20952, 12096, 50080, 5796,  18204,
                52136, 39728,
            },
            {
                21148, 60908, 4824,  57368, 27576, 42100, 5820,  25408, 54144,
                58716, 14372, 41280, 53120, 5392,  38348, 9604,  48112, 41344,
                28080, 8700,  62612, 49244, 6400,  44916, 39424, 15400, 9724,
                54772, 24864, 14768, 49500, 64404, 7988,  31440, 15328, 59616,
                1572,  32636, 20732, 48816, 9384,  33584, 40952, 16880, 10836,
                34216, 42548, 7864,  30700, 16520, 56972, 49764, 37824, 63360,
                1648,  17140, 60876, 41740, 20248, 46160, 9252,  26204, 41056,
                2180,  11820, 5624,  22332, 38320, 14716, 1568,  34416, 4896,
                57780, 44484, 38396, 26064, 42376, 19604, 23252, 6364,  15728,
                35968, 41916, 59968, 38068, 22116, 45108, 6932,  20304, 48424,
                24004, 13740, 38228, 58892, 15640, 55788, 35672, 4332,  58052,
                27200, 19932, 6664,  34308, 61040, 4488,  41260, 58868, 33880,
                6736,  42176, 48220, 9428,  64112, 15592, 46676, 23472, 58500,
                5708,  30620, 25464, 63644, 40532, 31980, 61940, 26868, 44064,
                32956, 1616,
            },
            {
                9152,  42668, 13932, 38640, 18096, 61988, 46364, 20764, 3748,
                11344, 47276, 33260, 19700, 59420, 50036, 12500, 34328, 64084,
                54156, 32688, 16212, 25392, 2840,  31764, 18976, 58680, 35476,
                5140,  60364, 3040,  40904, 34576, 21928, 55964, 50416, 35960,
                10264, 62004, 55244, 2880,  23864, 65060, 30484, 3904,  49236,
                27704, 63848, 54960, 12260, 21312, 2124,  28172, 11360, 46956,
                33920, 55744, 8284,  38336, 27736, 59780, 17580, 62392, 36444,
                20736, 63448, 43008, 56520, 9748,  51160, 19840, 41804, 10996,
                50216, 17248, 29700, 9236,  51224, 56724, 46800, 27728, 58424,
                49584, 19312, 5284,  28100, 34256, 58392, 53064, 11564, 43028,
                62696, 29440, 47324, 9380,  25820, 63920, 48916, 24076, 33020,
                184,   51172, 42492, 29056, 23004, 48648, 13648, 19148, 27552,
                54844, 12576, 22292, 38120, 5216,  28592, 42608, 264,   38792,
                19812, 34860, 15080, 57268, 1256,  46728, 16428, 8568,  37868,
                64704, 55376,
            },
            {
                46416, 29500, 53028, 33088, 49592, 7836,  30696, 35504, 51180,
                38796, 23132, 61896, 26712, 16452, 43196, 56356, 22364, 17892,
                3960,  46532, 42404, 37644, 61048, 53216, 47200, 22000, 29716,
                43416, 51000, 20156, 10540, 46932, 4,     12424, 43944, 25004,
                39872, 51420, 28376, 38208, 18172, 46816, 56080, 37412, 53344,
                23236, 15068, 4432,  47548, 62216, 41596, 59004, 14952, 52552,
                30328, 22480, 14040, 49876, 108,   53548, 32404, 4304,  47316,
                29496, 50724, 16312, 25500, 35536, 45256, 65512, 28160, 59888,
                35976, 1060,  55004, 64592, 3024,  35152, 12044, 63112, 1552,
                32844, 11384, 47256, 55784, 3560,  17296, 26032, 31204, 36624,
                56864, 4012,  51308, 41096, 31108, 20708, 7460,  53124, 18048,
                62460, 37392, 9284,  45848, 56888, 2288,  63572, 51524, 36568,
                628,   62684, 26544, 59100, 50312, 33932, 18176, 63068, 48580,
                61124, 53100, 43604, 7512,  23624, 35352, 53700, 58552, 3936,
                24092, 15108,
            },
            {
                19168, 58980, 3344,  22264, 11980, 56128, 40588, 15072, 63712,
                28976, 2172,  55520, 8476,  36156, 948,   28540, 5872,  39236,
                60280, 11660, 20596, 56820, 14064, 27660, 7492,  62980, 12248,
                56768, 26896, 32184, 61800, 17328, 58176, 29684, 63624, 20388,
                6820,  16564, 13228, 44552, 58524, 7628,  12656, 21068, 596,
                45292, 40200, 31640, 37728, 24604, 33412, 6532,  19568, 39644,
                4020,  45832, 64540, 24812, 35704, 7544,  44580, 23688, 13024,
                54748, 8364,  38788, 59324, 2884,  31560, 7932,  23316, 47456,
                14168, 20772, 46324, 24264, 16224, 39776, 30840, 18356, 43524,
                54684, 23468, 64192, 14472, 42480, 50412, 63392, 2552,  16116,
                8324,  22284, 17748, 61932, 1316,  13164, 38112, 43168, 10820,
                47336, 15228, 59560, 25824, 16888, 39340, 31728, 10172, 43088,
                46428, 17632, 32116, 3508,  44560, 56500, 8688,  13324, 26948,
                3316,  10860, 28552, 39120, 50360, 19572, 13112, 41356, 28836,
                49860, 35856,
            },
            {
                40040, 7316,  25836, 64388, 45084, 892,   24332, 59452, 10048,
                18568, 49560, 44220, 13468, 65064, 47116, 32180, 58480, 50436,
                26220, 30824, 196,   49712, 9404,  34660, 41912, 3816,  38436,
                16168, 8104,  45128, 36684, 51640, 26012, 38512, 3984,  53624,
                47536, 31612, 63208, 5648,  34500, 26060, 42320, 29344, 62780,
                9808,  58244, 18608, 51636, 3348,  54228, 44660, 63816, 26164,
                57904, 10240, 41348, 30872, 56556, 15780, 58616, 39688, 61128,
                1324,  32780, 19488, 46560, 13540, 53048, 17368, 56232, 5336,
                38992, 62332, 32760, 42012, 7064,  60048, 50700, 4468,  37712,
                26756, 7300,  39596, 30924, 21328, 9280,  38592, 46648, 59776,
                54028, 32952, 45116, 36212, 54916, 46892, 61196, 26596, 56012,
                29988, 21980, 3488,  52388, 35172, 6444,  20944, 57948, 24056,
                7856,  52424, 40580, 14404, 20912, 24960, 52296, 41328, 31900,
                22524, 46312, 17444, 65324, 59032, 3412,  30896, 45768, 21408,
                11436, 61368,
            },
            {
                31700, 55024, 42988, 34548, 16604, 37200, 52508, 4668,  42152,
                34384, 6632,  30288, 39700, 24184, 52700, 19552, 14400, 8204,
                44468, 53488, 36072, 65336, 23752, 18232, 51792, 55508, 24244,
                48524, 64840, 2112,  23068, 6556,  42636, 15504, 9676,  34232,
                56800, 41852, 2240,  23136, 53860, 50672, 61096, 15996, 33672,
                48804, 6760,  27328, 60012, 11000, 16624, 29748, 8112,  34716,
                50608, 17980, 2504,  20680, 43492, 11252, 29000, 19144, 49192,
                26996, 42332, 63916, 4900,  28960, 61320, 40780, 33720, 51784,
                26120, 9792,  56564, 12780, 28360, 54108, 21304, 10252, 61036,
                15420, 45808, 51572, 496,   59360, 34916, 24188, 12628, 28628,
                40700, 25352, 11084, 5168,  28096, 15888, 33796, 5948,  1996,
                40116, 32756, 63904, 12668, 42072, 55604, 48160, 29464, 13952,
                60584, 35072, 55548, 10232, 65076, 37400, 1984,  60020, 35708,
                57628, 54360, 4992,  33116, 25708, 9964,  54984, 62764, 6840,
                52540, 116,
            },
            {
                17544, 13120, 50840, 9432,  61240, 28672, 19396, 48000, 26408,
                58184, 53824, 21288, 61016, 3788,  10428, 42084, 37064, 63580,
                22728, 15828, 6500,  28852, 44136, 60496, 1576,  30580, 33744,
                19628, 40028, 53412, 13588, 55596, 31548, 59556, 45936, 23844,
                18468, 27432, 60264, 37348, 19816, 10720, 4792,  39456, 56404,
                24356, 13936, 35428, 42052, 22288, 49448, 36736, 55388, 23248,
                12900, 60580, 36888, 52856, 62776, 6168,  51112, 37260, 9448,
                15384, 55600, 23572, 50172, 37632, 20348, 232,   11956, 43356,
                30084, 2468,  49676, 19268, 35652, 44012, 24836, 47480, 56816,
                34396, 19756, 61992, 27972, 54248, 4928,  43688, 19136, 49964,
                1672,  65440, 52544, 21028, 60180, 41864, 22816, 51236, 19008,
                58452, 49076, 7860,  44972, 18508, 27076, 1728,  64328, 38736,
                19712, 2620,  27956, 47268, 30684, 5784,  49360, 19972, 15732,
                6872,  42340, 37216, 14228, 48404, 43136, 37608, 16616, 33976,
                24632, 47488,
            },
            {
                38160, 27376, 21180, 2408,  46468, 57344, 12480, 32224, 62672,
                15560, 11076, 46064, 35568, 17132, 55992, 25852, 48824, 2096,
                33212, 55704, 39772, 48100, 11424, 37932, 15244, 46640, 6084,
                58420, 9932,  29332, 35224, 61132, 20980, 49900, 1076,  65412,
                12168, 7792,  48668, 14564, 43992, 32368, 47768, 18772, 2780,
                44224, 52996, 65160, 1708,  46584, 61428, 14240, 524,   43220,
                47340, 32240, 5320,  26680, 47708, 33688, 22696, 65148, 4532,
                45060, 34796, 11572, 7604,  58544, 25656, 48232, 63692, 21832,
                16572, 60512, 45440, 5252,  63200, 808,   14036, 39064, 30156,
                2488,  9516,  36964, 12948, 17740, 48012, 56616, 62416, 6988,
                34544, 14316, 37952, 48784, 8644,  56796, 11592, 64644, 43932,
                14272, 27804, 37036, 23764, 60832, 10448, 53104, 33576, 5428,
                43804, 49632, 22988, 61356, 16084, 43200, 26412, 11784, 63612,
                29168, 50908, 20408, 1100,  62032, 21948, 27624, 2704,  57388,
                43784, 63524,
            },
            {
                4112,  59372, 55928, 30968, 40088, 23084, 5384,  44332, 2824,
                38092, 24972, 320,   51192, 32440, 59672, 6296,  29352, 61532,
                12448, 18712, 59744, 4224,  21044, 57536, 27292, 63184, 12756,
                50788, 25320, 44664, 17776, 3636,  11568, 40364, 28632, 35788,
                52676, 39532, 30616, 55212, 764,   58212, 27192, 63488, 30432,
                36960, 20804, 10372, 32356, 28648, 39088, 19204, 58324, 6816,
                27964, 40248, 56084, 13792, 17420, 2032,  41528, 56716, 31084,
                21680, 60016, 40156, 17024, 32472, 44624, 9104,  35592, 57396,
                52588, 37372, 23080, 40640, 58264, 33364, 50404, 6692,  65224,
                22636, 49316, 44208, 25184, 41256, 32648, 10336, 30100, 21736,
                55012, 42864, 18364, 29476, 412,   31768, 39148, 25912, 35260,
                9408,  53472, 5192,  56740, 31976, 40296, 14836, 46856, 24948,
                59232, 12228, 6824,  39744, 34532, 58308, 53372, 45788, 38412,
                9412,  24524, 60420, 31120, 55720, 7324,  50996, 40632, 13672,
                29744, 8944,
            },
            {
                34348, 41900, 48380, 14504, 6996,  65456, 35060, 51452, 55748,
                20056, 41472, 64432, 27584, 13960, 43956, 20272, 39204, 9364,
                46480, 42396, 25264, 31644, 50228, 7824,  42716, 22680, 36384,
                41276, 464,   63836, 48096, 37848, 26536, 6496,  43676, 15864,
                58704, 4184,  21784, 64096, 24656, 36144, 8404,  13028, 54284,
                6288,  49616, 17288, 56936, 4616,  9044,  25220, 52340, 64756,
                20792, 9904,  61520, 24132, 38444, 59256, 27548, 12352, 53184,
                700,   47932, 29316, 53516, 3204,  55808, 14908, 28228, 3840,
                7132,  32212, 14600, 26396, 8764,  17316, 55156, 27596, 41444,
                16040, 53812, 58380, 6852,  64316, 1488,  15628, 39096, 25976,
                46700, 3888,  58888, 24316, 45624, 53912, 17452, 2908,  47516,
                21036, 61952, 16528, 788,   49600, 21600, 7484,  36924, 17052,
                56372, 31668, 51580, 18720, 9096,  1184,  22420, 32444, 3268,
                56004, 47296, 12264, 44408, 17688, 35612, 10812, 64796, 19452,
                53760, 22612,
            },
            {
                62020, 11320, 18676, 25680, 54004, 37572, 16408, 27680, 8620,
                31224, 48656, 57112, 8992,  36708, 4272,  50576, 54392, 23540,
                35144, 52828, 980,   63500, 16692, 34240, 54308, 2732,  18560,
                59868, 31732, 15308, 22496, 53672, 62084, 57500, 51204, 19884,
                32536, 46040, 9776,  42116, 16784, 51952, 40656, 46148, 23636,
                60328, 39668, 26276, 43100, 62768, 50316, 34516, 41608, 31128,
                16372, 49296, 3924,  45000, 52192, 8836,  46636, 15944, 36368,
                18732, 62744, 6016,  24616, 64824, 41316, 50548, 18972, 39528,
                47284, 11092, 64744, 53408, 30332, 46156, 20456, 10612, 59160,
                33908, 3456,  31068, 20556, 52292, 36152, 60220, 50504, 57312,
                8900,  32352, 63896, 11984, 35780, 62096, 7520,  50604, 58904,
                31228, 38504, 46136, 28536, 65316, 43000, 54948, 29732, 62920,
                28,    41920, 26072, 62356, 55368, 29520, 14244, 41492, 59848,
                16752, 34596, 6420,  39172, 25560, 47920, 58848, 31572, 45364,
                5400,  49476,
            },
            {
                36892, 1364,  32640, 60548, 43812, 10344, 47664, 22044, 62136,
                13500, 5764,  17832, 22408, 47140, 63284, 31348, 16160, 58352,
                6992,  14592, 28340, 37588, 56308, 10788, 45676, 29932, 51880,
                9360,  55724, 5212,  41816, 8236,  33708, 13812, 24212, 2348,
                62736, 26208, 13364, 49756, 29696, 3184,  21196, 58736, 16204,
                2160,  34044, 12368, 53828, 21796, 15448, 45712, 11628, 2960,
                37544, 53944, 34180, 29044, 18904, 32836, 61944, 6812,  50272,
                26312, 43176, 14124, 37468, 20992, 11212, 30828, 61836, 23828,
                58556, 42820, 19644, 49088, 3128,  38172, 62960, 43800, 5008,
                25740, 46868, 39480, 11752, 28348, 23312, 43108, 3152,  13196,
                17500, 41504, 51556, 4868,  15284, 40648, 22372, 28696, 12692,
                4000,  42048, 24292, 11080, 34884, 18908, 3800,  50744, 10664,
                21876, 35380, 14628, 45620, 37896, 48644, 64616, 5656,  51388,
                27480, 21752, 64104, 53920, 4032,  14464, 22884, 904,   38844,
                26440, 15736,
            },
            {
                28888, 46168, 51276, 4404,  20200, 29724, 708,   58520, 42916,
                38808, 53640, 33944, 60160, 40868, 11812, 1352,  26836, 43264,
                64820, 21364, 47244, 41028, 4872,  23608, 61236, 14080, 39648,
                26184, 35120, 49504, 28212, 18940, 46396, 30236, 10396, 48616,
                38524, 56612, 35256, 59484, 6052,  62232, 32736, 10076, 42640,
                29148, 64376, 46924, 5684,  37316, 1544,  29372, 55224, 62448,
                23164, 43752, 12984, 63948, 1040,  42552, 23556, 55488, 39984,
                33356, 9396,  52324, 46996, 34268, 952,   44280, 8040,  52908,
                1912,  27556, 36040, 5920,  57824, 13052, 22960, 35548, 51244,
                13976, 18524, 63364, 55736, 8500,  48568, 19020, 33792, 61432,
                29112, 37456, 21616, 27404, 49320, 57716, 33416, 44076, 63760,
                19024, 56524, 51716, 6236,  58800, 13616, 60656, 26900, 39216,
                48420, 54316, 6516,  3496,  20932, 10700, 25388, 18944, 36828,
                43728, 288,   49880, 29404, 41988, 61492, 34120, 51860, 9440,
                58268, 54728,
            },
            {
                64528, 23552, 8560,  57988, 40792, 63420, 52144, 33256, 4268,
                26156, 45608, 1804,  29400, 24520, 55684, 19416, 38652, 33556,
                3156,  49960, 11924, 59524, 19172, 27456, 48912, 3768,  64764,
                21368, 44980, 11760, 62836, 58852, 1284,  36192, 55468, 41652,
                5600,  18076, 7948,  44668, 19484, 54572, 37900, 47924, 25208,
                52032, 8000,  19768, 31200, 60848, 24544, 58932, 18628, 6492,
                26776, 58280, 7200,  21788, 56776, 11876, 30024, 3708,  20472,
                59808, 2284,  57244, 17812, 60172, 26652, 56120, 36756, 16632,
                32944, 13864, 61552, 24824, 41692, 32144, 352,   54272, 29572,
                60884, 36460, 1228,  41908, 15208, 59516, 5636,  45716, 24272,
                53676, 1956,  56644, 44772, 18528, 9920,  1288,  54364, 25688,
                8768,  32320, 15776, 37224, 47420, 22608, 33156, 45220, 7848,
                18132, 64300, 29036, 58652, 33520, 44312, 54480, 31268, 8320,
                57252, 15236, 32704, 19684, 10340, 56448, 28036, 17220, 41488,
                20916, 12524,
            },
            {
                43240, 39300, 16844, 27252, 34856, 11948, 24344, 14792, 19060,
                56544, 10848, 51048, 15860, 7692,  49644, 62368, 46152, 10032,
                54616, 30104, 35752, 8068,  52916, 32920, 43312, 37084, 7524,
                17364, 56872, 4220,  23988, 38304, 52440, 16532, 63724, 21856,
                28400, 53608, 31880, 23692, 40280, 27208, 13780, 216,   56588,
                36392, 14800, 57524, 41848, 50816, 9756,  47292, 38988, 32852,
                48496, 15820, 40400, 35348, 50784, 46300, 37312, 64372, 13368,
                44888, 31232, 23160, 39160, 6576,  13628, 48148, 21972, 63472,
                49848, 44684, 8928,  56164, 16864, 47260, 59544, 19232, 9940,
                6316,  49384, 24388, 32880, 51920, 30144, 37776, 65124, 8304,
                47912, 11348, 62816, 6912,  30636, 61392, 37076, 14220, 40124,
                48364, 61000, 2540,  54012, 26212, 40216, 1408,  56976, 12832,
                31808, 42324, 23604, 50348, 16664, 60944, 1532,  41092, 63036,
                11340, 46912, 60500, 6680,  36956, 45636, 3504,  63172, 47780,
                31836, 6080,
            },
            {
                52596, 33476, 13452, 56272, 49808, 5964,  38380, 47092, 61012,
                35584, 21756, 63128, 38016, 44080, 34560, 4956,  14784, 23176,
                61580, 16640, 25144, 44636, 62756, 604,   16024, 58488, 30560,
                53428, 40468, 33216, 14524, 43388, 6732,  26728, 45076, 3404,
                14364, 61344, 46988, 11240, 65100, 4460,  50496, 18408, 63132,
                22300, 45500, 2564,  27340, 16824, 34900, 13344, 42996, 468,
                52212, 10332, 60824, 27836, 17592, 5128,  25928, 49424, 17100,
                54192, 27300, 11028, 50056, 64172, 29308, 41676, 3792,  11612,
                29920, 39416, 20984, 51764, 28816, 7452,  38456, 26108, 45460,
                40308, 57352, 21316, 45004, 4548,  25864, 12784, 40892, 20784,
                32156, 16440, 35044, 39548, 24980, 46900, 20700, 51020, 5884,
                34820, 21448, 29348, 43164, 9444,  63396, 16732, 51568, 36912,
                61136, 2064,  11132, 39932, 7676,  36228, 13776, 52300, 22764,
                26716, 39348, 54556, 24652, 13480, 50936, 21896, 8084,  35812,
                59404, 2168,
            },
            {
                47016, 22144, 62624, 1456,  45120, 18536, 55080, 8892,  27936,
                41424, 6608,  30656, 53988, 12332, 27416, 57208, 31844, 51904,
                40444, 6148,  57944, 38936, 13072, 22424, 50544, 10080, 24880,
                47080, 1628,  60204, 50132, 31060, 20828, 11316, 58508, 34428,
                50880, 37560, 1048,  16132, 57460, 35628, 44188, 30804, 6744,
                33436, 11736, 39080, 62320, 54740, 5256,  65468, 22860, 56876,
                20504, 30672, 45468, 2496,  53316, 58636, 10092, 41100, 34324,
                7120,  62680, 42388, 2992,  33076, 53652, 18864, 25332, 59828,
                5564,  54456, 1448,  35016, 12060, 62288, 50432, 14384, 65016,
                31816, 16552, 11472, 63052, 58628, 17924, 55056, 68,    57684,
                27100, 51896, 43540, 13708, 53356, 4152,  59116, 28220, 17004,
                64360, 45556, 12528, 58336, 19504, 48776, 7068,  28732, 24852,
                20488, 47460, 53160, 27048, 56392, 21684, 29556, 45892, 5304,
                35300, 16292, 2876,  43140, 65084, 27116, 40192, 14944, 55484,
                25048, 18180,
            },
            {
                60716, 7944,  36792, 30500, 25312, 42268, 65048, 31876, 1952,
                48516, 57600, 17764, 3196,  20620, 41716, 60668, 16,    19736,
                28428, 48468, 2644,  18736, 31516, 55096, 41680, 35460, 63660,
                12916, 28716, 18212, 9156,  65164, 55904, 39360, 48416, 30764,
                9588,  25376, 42644, 51900, 29080, 22004, 10176, 54236, 41048,
                59944, 49156, 23896, 8792,  20268, 31716, 49924, 28388, 8520,
                37940, 62812, 13700, 23516, 39208, 31336, 21152, 61352, 64,
                47124, 37032, 21340, 16288, 45660, 8856,  38000, 57060, 34092,
                46664, 17784, 64536, 23916, 43572, 2912,  33604, 22832, 3760,
                52668, 28424, 2176,  38324, 8964,  33884, 49292, 44388, 36316,
                4684,  60216, 22800, 1508,  64816, 33144, 10928, 42024, 54904,
                568,   24764, 38768, 5028,  35776, 31540, 55084, 43424, 3964,
                58112, 14840, 44456, 32616, 4724,  65464, 49004, 18452, 58936,
                30928, 49336, 21072, 57540, 33660, 724,   53580, 30836, 43916,
                10648, 29384,
            },
            {
                4048,  40836, 54164, 10364, 59652, 3892,  22584, 16568, 53248,
                13584, 25368, 44872, 64436, 48920, 24236, 9712,  36692, 44964,
                12660, 56348, 33900, 65472, 46840, 5436,  27992, 2804,  20232,
                51804, 38168, 23024, 46176, 35524, 0,     17656, 23912, 4440,
                64016, 20344, 59412, 33140, 7096,  48136, 60948, 26028, 15648,
                3956,  29268, 52724, 36084, 41284, 45204, 3680,  60336, 16268,
                34296, 54336, 5484,  43644, 64972, 7360,  14900, 55000, 24220,
                29568, 12892, 52536, 58044, 23884, 61504, 1196,  50948, 43052,
                14844, 27140, 37548, 48900, 57916, 20120, 41068, 55772, 8156,
                43832, 34892, 54404, 47468, 42416, 22672, 28980, 15348, 63440,
                19624, 9948,  49892, 29604, 18240, 45388, 23904, 36500, 8148,
                31264, 57212, 51816, 15432, 60080, 23196, 11032, 64892, 39264,
                34332, 8704,  62560, 17604, 38292, 11888, 40644, 2684,  10024,
                62516, 52904, 7872,  29196, 11188, 18504, 48604, 4916,  62772,
                37956, 50400,
            },
            {
                26264, 15900, 20364, 48708, 14348, 34456, 50708, 37156, 62248,
                9992,  39980, 32264, 36140, 6008,  54564, 16868, 50304, 63796,
                7780,  42584, 26416, 10660, 23484, 59548, 16904, 44960, 61088,
                6764,  43016, 55200, 5136,  27120, 51980, 60136, 44152, 15028,
                40684, 55112, 3056,  18032, 39820, 14020, 1420,  36532, 20796,
                45952, 56432, 17596, 988,   58668, 14540, 24380, 39368, 47540,
                2136,  25804, 50024, 19064, 36000, 48204, 33548, 43328, 50124,
                18780, 60588, 4064,  35088, 28300, 14572, 31600, 10508, 22740,
                6688,  60868, 9324,  31628, 5828,  16164, 29652, 61544, 47928,
                18340, 12688, 25216, 19820, 62088, 2872,  11732, 53168, 7124,
                31400, 37688, 41628, 55640, 7576,  57908, 14868, 48700, 61604,
                18700, 11408, 47264, 28024, 40528, 1084,  46656, 14108, 18924,
                54372, 30028, 23020, 356,   59340, 26068, 53724, 23340, 44032,
                37440, 13860, 41416, 46580, 62220, 39084, 59180, 23804, 20016,
                13416, 57772,
            },
            {
                33092, 46108, 63728, 27560, 39760, 56824, 7372,  46220, 29424,
                20760, 58584, 1220,  15344, 60132, 29892, 39904, 25612, 32436,
                21808, 53848, 15668, 35900, 52188, 8612,  38800, 57080, 33316,
                10880, 31272, 62708, 15748, 41660, 12484, 29236, 7172,  53152,
                34844, 11576, 45852, 27632, 62948, 56480, 42824, 52208, 64716,
                8008,  33016, 12436, 63348, 26784, 51796, 10284, 55560, 19364,
                42220, 59184, 9196,  28952, 56400, 11780, 26224, 5688,  57408,
                10204, 39020, 44540, 7716,  48116, 40820, 65516, 18444, 55232,
                29736, 40144, 52112, 13160, 45224, 53840, 36104, 10608, 26856,
                37524, 63704, 56436, 6920,  31244, 50984, 59240, 39748, 24772,
                56880, 47772, 12564, 25988, 35456, 40076, 4708,  52888, 21676,
                43440, 33472, 3884,  63264, 20580, 53044, 32816, 25712, 49948,
                6284,  41524, 47720, 51644, 34588, 45664, 15204, 57236, 33152,
                27016, 19180, 5160,  24500, 35560, 16032, 8916,  45436, 34992,
                1640,  42872,
            },
            {
                22856, 52060, 6300,  132,   31324, 19544, 12296, 26328, 43292,
                5088,  51592, 47808, 22820, 8484,  45744, 13200, 5280,  58812,
                3260,  38600, 60360, 1248,  41268, 30444, 50028, 21560, 14704,
                48720, 25164, 2344,  58240, 21460, 33408, 49304, 37484, 61892,
                22148, 30348, 49916, 8832,  23640, 34040, 18920, 10892, 28200,
                39272, 23008, 43216, 48028, 30816, 7020,  35080, 64236, 29428,
                12236, 32784, 15476, 39908, 972,   61868, 17668, 37416, 2472,
                64280, 31276, 25444, 56408, 19764, 2424,  49988, 45100, 34772,
                58872, 224,   20996, 63564, 26200, 2052,  58620, 21812, 4744,
                50296, 504,   40704, 14700, 36720, 26688, 17464, 34224, 45352,
                1004,  16340, 61916, 3440,  20300, 63136, 30412, 27108, 1932,
                38020, 50616, 25424, 9664,  42776, 6880,  62236, 37752, 2268,
                60112, 13044, 27796, 10144, 19872, 5840,  30124, 8232,  63964,
                868,   55768, 59992, 50768, 2444,  54344, 31820, 26940, 52688,
                64636, 8360,
            },
            {
                56688, 11968, 35624, 58784, 42172, 62900, 53972, 1828,  60024,
                34068, 17600, 38252, 27944, 56316, 33524, 62916, 52244, 18708,
                46768, 29224, 49024, 20360, 62468, 13240, 26568, 3420,  37124,
                54092, 18460, 35908, 47452, 6340,  64332, 9864,  18288, 26364,
                5504,  57248, 15896, 37984, 54128, 5264,  30732, 47676, 3192,
                50992, 57328, 5860,  19848, 38088, 16716, 43908, 21968, 5616,
                52956, 45792, 63152, 51720, 23244, 44040, 53284, 28148, 41644,
                21632, 15484, 50864, 12280, 59688, 38284, 26752, 13064, 4232,
                24628, 48052, 42452, 33284, 18080, 39000, 47052, 14920, 42352,
                59676, 32220, 23432, 46124, 60556, 3720,  48088, 10412, 64708,
                22124, 28644, 53940, 44104, 51372, 11332, 47020, 58944, 13768,
                65480, 16380, 55712, 35604, 58456, 17392, 28924, 45244, 55880,
                21504, 35984, 64456, 56892, 39152, 61788, 50168, 36288, 17440,
                48176, 38636, 30776, 21420, 43020, 12980, 57484, 6072,  40220,
                15600, 29852,
            },
            {
                38944, 18248, 44856, 24104, 9128,  16056, 47588, 23356, 40564,
                13292, 64860, 53684, 11204, 42564, 20532, 864,   36184, 43556,
                9308,  14264, 24688, 7076,  44068, 56512, 6312,  64312, 41460,
                60524, 9204,  27852, 39572, 53300, 24260, 45172, 59020, 888,
                47364, 41696, 65340, 2012,  44740, 13392, 62364, 59060, 25648,
                14684, 34536, 61188, 2616,  54864, 59772, 49392, 248,   60748,
                36280, 24548, 3804,  20260, 31304, 6700,  58536, 13384, 48808,
                34620, 54756, 42944, 4740,  23280, 32808, 8664,  62304, 53800,
                36904, 15756, 10796, 56548, 50704, 7496,  61920, 34196, 52748,
                29136, 10116, 17180, 53740, 8292,  41480, 57384, 19124, 5396,
                42520, 36844, 8516,  31960, 24324, 17468, 34444, 9084,  40612,
                23000, 6100,  45632, 30112, 12840, 49408, 23480, 10432, 15616,
                31888, 43968, 3988,  16320, 24396, 1792,  41896, 12824, 22324,
                44352, 6240,  11368, 64888, 28444, 49452, 18800, 61076, 46660,
                21112, 49288,
            },
            {
                62292, 2492,  53588, 28880, 50424, 37860, 33360, 5880,  56536,
                30980, 7588,  24696, 2936,  61264, 15680, 49668, 26796, 57472,
                31408, 64072, 54868, 34520, 18112, 31940, 46572, 23228, 29828,
                500,   44396, 57152, 13188, 17164, 2744,  30132, 39024, 51892,
                12420, 31928, 19304, 25040, 51496, 35620, 21816, 39896, 17508,
                45704, 9340,  28912, 41940, 24888, 11388, 32384, 26600, 40228,
                17808, 8624,  57124, 37916, 47120, 10352, 33184, 19352, 61180,
                8596,  3148,  29504, 36412, 63280, 51700, 16900, 43684, 20616,
                6144,  61212, 28128, 3612,  22716, 30600, 11676, 24904, 19184,
                6024,  48356, 65476, 34484, 28044, 13832, 22808, 52788, 32824,
                50088, 60288, 14932, 58236, 38980, 2372,  56660, 44508, 54196,
                32448, 60300, 136,   19724, 40092, 2852,  34404, 61436, 51336,
                7728,  25932, 53376, 47012, 32680, 55144, 28156, 60836, 52576,
                26000, 54924, 16500, 34804, 8336,  37784, 25188, 612,   11244,
                36296, 5132,
            },
            {
                24684, 32296, 13848, 65460, 21332, 2832,  61632, 27676, 19992,
                50068, 43996, 36564, 47228, 30344, 40284, 7724,  23180, 11904,
                37704, 3876,  40708, 50968, 10988, 38096, 53060, 16668, 12384,
                51564, 20872, 32208, 61676, 42748, 34680, 56364, 14892, 21232,
                35840, 60740, 6652,  28728, 58404, 10300, 43048, 420,   52892,
                37008, 64416, 21336, 52200, 15272, 7400,  46528, 56228, 14368,
                49908, 43004, 27064, 13644, 50552, 64108, 40044, 1948,  45700,
                26852, 57812, 18424, 47368, 6768,  41144, 28476, 57760, 31368,
                39624, 52468, 44712, 35656, 64068, 41196, 57280, 1168,  44300,
                54976, 38632, 21264, 1516,  44592, 61456, 30664, 39140, 25888,
                12912, 1876,  21084, 48312, 6184,  64256, 28556, 20152, 7152,
                26008, 48140, 36880, 52272, 62860, 54820, 42968, 4524,  38472,
                59276, 18756, 40560, 12256, 8540,  20896, 45316, 2996,  9500,
                32412, 40456, 58180, 47308, 3628,  62336, 41400, 32896, 55700,
                28180, 58912,
            },
            {
                9416,  47752, 40404, 7092,  43912, 12136, 52992, 46112, 9292,
                15320, 63060, 4508,  18004, 59208, 54836, 34580, 63244, 53220,
                17348, 21748, 58516, 756,   28132, 61824, 4372,  59528, 35000,
                48040, 7296,  25340, 4072,  49660, 10896, 63188, 27712, 44016,
                8868,  54644, 39996, 46360, 16852, 32516, 7480,  56268, 30204,
                4648,  12696, 48132, 31580, 39336, 63092, 35996, 20660, 6028,
                65244, 33832, 59052, 1384,  29076, 16896, 25304, 52344, 22152,
                37856, 64696, 10844, 53324, 21520, 14104, 876,   47960, 11508,
                25536, 18692, 8980,  48964, 13748, 17388, 46608, 36672, 62968,
                13924, 26460, 58468, 11860, 37256, 49200, 3120,  7408,  62704,
                56152, 45512, 35164, 26708, 41232, 50516, 13208, 37300, 51756,
                16984, 12012, 28236, 8776,  24040, 16128, 27424, 21080, 48044,
                29656, 752,   63748, 50032, 37096, 62488, 17992, 35196, 58928,
                14728, 4920,  23652, 19744, 53436, 13532, 45140, 22296, 51752,
                14876, 42960,
            },
            {
                34836, 54764, 16740, 57040, 31136, 35400, 17920, 58224, 39072,
                23032, 34184, 52016, 27564, 21628, 13400, 2632,  45576, 5608,
                42348, 48168, 25716, 15164, 44836, 20536, 9108,  42444, 27296,
                39292, 65168, 52684, 37836, 19588, 23520, 6280,  53820, 3364,
                49208, 23416, 14024, 2528,  63592, 50008, 27196, 61652, 19100,
                24224, 59424, 43504, 1108,  18252, 58208, 28776, 2340,  53136,
                30352, 11444, 19496, 53572, 44688, 36476, 4844,  55792, 12412,
                31564, 15836, 44640, 33776, 26176, 56124, 60380, 35124, 5120,
                64928, 55820, 32416, 2416,  26244, 53448, 5344,  28544, 9144,
                32840, 41748, 4600,  51844, 24716, 18588, 54588, 16028, 41904,
                10680, 29720, 17952, 61232, 10036, 22712, 31292, 62548, 4388,
                41364, 61600, 57644, 43936, 33132, 6244,  64500, 11300, 56764,
                14428, 34788, 22836, 27616, 56632, 5448,  41128, 48836, 29532,
                51316, 63376, 43828, 27392, 31552, 58764, 17256, 7388,  64516,
                3020,  19732,
            },
            {
                60952, 27036, 964,   22244, 60140, 25628, 3752,  41996, 29940,
                1496,  55960, 11524, 41604, 8344,  49184, 39252, 28232, 32424,
                61184, 8676,  30232, 64832, 34144, 49280, 24156, 57612, 1868,
                18660, 14744, 10008, 30312, 58196, 47088, 40604, 32600, 17816,
                64448, 29764, 34696, 55588, 20844, 37408, 11208, 47188, 41024,
                33640, 55472, 5956,  26476, 54116, 9784,  45268, 41312, 24268,
                47352, 38596, 4280,  22724, 62720, 9472,  59860, 42592, 48508,
                6272,  40712, 1800,  61760, 38668, 9752,  45052, 22896, 50560,
                15116, 42500, 22108, 58260, 39076, 60416, 20124, 51520, 23100,
                56412, 16424, 45960, 30972, 8608,  64140, 34032, 46512, 22012,
                36704, 51608, 3712,  54016, 43952, 16020, 740,   55832, 45552,
                34764, 21976, 3028,  13988, 39228, 50528, 46060, 31660, 41744,
                8396,  54216, 43668, 3900,  31476, 15248, 25176, 11680, 21404,
                38552, 156,   36056, 9060,  50684, 2544,  37020, 29576, 46832,
                38476, 50092,
            },
            {
                30752, 6228,  45992, 36916, 48372, 8220,  64008, 50628, 13568,
                60552, 25456, 44632, 62212, 31212, 57120, 24008, 18820, 51092,
                14100, 36556, 56696, 5236,  39968, 54724, 13288, 30992, 45612,
                54048, 33368, 60956, 44480, 1240,  15976, 60200, 25420, 38232,
                11744, 58036, 41808, 5152,  25968, 44252, 14596, 1680,  51272,
                15924, 9120,  38272, 48976, 34692, 23056, 13056, 62204, 16100,
                8100,  60872, 49100, 41576, 31112, 14972, 34052, 19996, 28224,
                56884, 23596, 51212, 18872, 4448,  32060, 16696, 40600, 29528,
                1676,  36856, 46248, 7000,  29996, 10272, 34284, 40736, 2764,
                48216, 62076, 20512, 59592, 40080, 12960, 28500, 57304, 536,
                25096, 65180, 7884,  32660, 57844, 38332, 48020, 25024, 18576,
                8224,  30452, 48620, 55500, 19460, 59756, 1328,  17956, 24856,
                37368, 61968, 17284, 11068, 45976, 59964, 53292, 65356, 6884,
                46316, 17488, 56568, 12584, 40592, 61756, 25596, 10468, 54428,
                23580, 11928,
            },
            {
                18276, 64220, 39752, 10228, 52444, 15460, 20808, 36600, 7012,
                47804, 17520, 5508,  35696, 180,   15988, 64032, 9880,  59328,
                2140,  21252, 46852, 11800, 18556, 2664,  37492, 63380, 6864,
                22864, 41040, 5228,  26404, 36112, 12116, 51056, 8088,  45896,
                332,   19464, 47688, 9088,  52484, 60784, 23448, 35372, 63912,
                28472, 22120, 60156, 14120, 64604, 3388,  50924, 32548, 57068,
                36116, 26392, 17560, 54824, 7044,  24932, 50988, 228,   62148,
                36120, 9304,  58728, 30036, 47484, 55088, 63740, 7644,  52580,
                61300, 12032, 18100, 63100, 49920, 15292, 44180, 64768, 12700,
                29800, 7332,  35720, 1772,  53084, 43720, 5456,  19656, 59892,
                48960, 39796, 13012, 19896, 27520, 5780,  33572, 59708, 11520,
                53156, 65056, 37584, 25908, 10268, 29048, 36064, 52696, 58200,
                3288,  48384, 21228, 51960, 38900, 33600, 2452,  42904, 27960,
                54608, 32912, 60164, 22476, 48444, 19152, 44168, 57672, 33348,
                5260,  42036,
            },
            {
                58372, 25060, 13972, 56096, 29160, 33468, 44244, 27056, 53576,
                32228, 38692, 54972, 20612, 51508, 46504, 33480, 43660, 26512,
                54264, 40756, 24924, 32684, 52336, 60996, 26740, 16492, 51012,
                11596, 57028, 20312, 50012, 55048, 29624, 21372, 62452, 34104,
                53264, 28048, 61996, 32344, 17216, 39664, 4052,  57496, 6320,
                44840, 53080, 31944, 19784, 42708, 28040, 39640, 5204,  20880,
                43756, 944,   12148, 35312, 57668, 47104, 39240, 11536, 17540,
                45252, 26388, 14308, 42064, 22016, 12596, 36424, 25472, 19520,
                48244, 34024, 27096, 54212, 23680, 1000,  57556, 25676, 18260,
                54676, 39304, 26944, 15036, 22308, 49672, 37816, 9468,  30720,
                15124, 34492, 45420, 23616, 50332, 63532, 14492, 40020, 28584,
                42752, 1180,  16052, 45176, 4832,  62604, 22684, 12364, 44156,
                7208,  30628, 27032, 57744, 8580,  23088, 18380, 35748, 9528,
                14324, 24916, 3916,  29324, 5692,  34708, 13876, 760,   62820,
                15832, 51168,
            },
            {
                1968,  32428, 43436, 19224, 5644,  62668, 576,   58000, 10840,
                22368, 65220, 14096, 28900, 40188, 11684, 3716,  38024, 6360,
                49328, 16316, 62856, 43560, 7560,  21860, 42288, 34016, 47168,
                29220, 37700, 63952, 17080, 3060,  42336, 57448, 4060,  43616,
                15212, 23256, 37108, 12872, 58664, 27568, 50468, 31140, 18656,
                12156, 40464, 2164,  7580,  47108, 16916, 58808, 10952, 54508,
                29728, 50520, 64776, 27864, 3072,  21412, 63548, 52860, 32288,
                4248,  48996, 65360, 7164,  52008, 668,   28440, 59356, 3296,
                57360, 8820,  41408, 4148,  38244, 32544, 47640, 36252, 4732,
                51080, 44860, 10280, 57180, 32916, 62832, 25640, 53796, 42440,
                3928,  55184, 61732, 1832,  42280, 9124,  54560, 3732,  20868,
                50892, 23992, 58940, 34552, 54232, 40932, 49612, 15880, 33276,
                64988, 40352, 13928, 460,   62964, 30004, 50408, 58088, 47612,
                61592, 41656, 51532, 45048, 64632, 53192, 39460, 21532, 28052,
                45696, 35880,
            },
            {
                53696, 7960,  48984, 59948, 23660, 39344, 47156, 18280, 41548,
                4004,  45284, 8632,  58632, 25992, 61784, 22456, 56116, 29580,
                34756, 10488, 676,   28692, 47948, 58432, 10196, 4608,  60232,
                176,   24352, 8732,  48056, 33680, 14084, 39164, 26968, 10256,
                56444, 50188, 6904,  2400,  42964, 20752, 8312,  46464, 37160,
                25848, 62524, 57256, 35944, 52520, 61748, 25276, 37356, 46624,
                14732, 59712, 19704, 41052, 45360, 14408, 29692, 8060,  43056,
                54880, 20264, 37600, 33860, 61008, 39832, 46352, 15548, 43280,
                23016, 30520, 45496, 16584, 60852, 11892, 21352, 8492,  61708,
                31620, 23524, 64076, 47412, 4932,  17424, 11960, 60984, 47280,
                18412, 28084, 10768, 58600, 30388, 35396, 17804, 45948, 61948,
                32408, 7212,  12640, 19880, 31048, 8948,  1908,  60344, 25452,
                19276, 54828, 34452, 43064, 46696, 15784, 4436,  39544, 21892,
                1216,  18848, 37496, 10900, 16512, 8136,  31240, 49516, 55796,
                9328,  20276,
            },
            {
                63072, 28884, 12104, 34876, 3200,  54424, 13336, 30948, 61068,
                25044, 34572, 48988, 2128,  52976, 17896, 47892, 14772, 64944,
                20136, 58740, 38416, 55632, 13844, 31512, 36980, 19564, 52808,
                43568, 13140, 31692, 45024, 61216, 7244,  23780, 65292, 17840,
                31532, 40636, 63464, 47100, 54176, 33700, 64780, 15160, 55648,
                48480, 9908,  29740, 23832, 12516, 88,    31696, 8300,  22592,
                4344,  38984, 9552,  32460, 5916,  59072, 37112, 22700, 60168,
                12472, 28832, 2428,  17096, 24744, 10832, 53932, 32788, 38140,
                12952, 54872, 65176, 6836,  51992, 40496, 28420, 56044, 42540,
                14048, 444,   19548, 41184, 27980, 36540, 1392,  32000, 21740,
                6936,  38576, 52456, 15912, 21820, 48368, 26084, 37072, 10296,
                57984, 48868, 41872, 63972, 47060, 27216, 37820, 45484, 51688,
                10044, 4860,  49372, 24012, 11156, 54472, 26800, 64232, 12396,
                30400, 57004, 33552, 26484, 54968, 23360, 60960, 3536,  41428,
                25808, 38296,
            },
            {
                4424,  44620, 21728, 41724, 51116, 26216, 36272, 8332,  50476,
                55624, 16612, 30248, 42160, 36576, 4996,  32632, 41080, 7812,
                45328, 51004, 23632, 18316, 4136,  50448, 25252, 65108, 15716,
                39948, 59000, 55308, 27444, 19720, 53404, 49520, 36172, 46320,
                4520,  21020, 28504, 14652, 24900, 10808, 532,   41552, 22880,
                3092,  17612, 50792, 44368, 20280, 40956, 56344, 49832, 63832,
                34088, 53440, 24136, 48360, 55564, 18188, 50224, 1400,  25588,
                39628, 56848, 44844, 50456, 57788, 20920, 8072,  62484, 4400,
                50420, 20696, 35316, 26436, 19264, 58760, 2440,  49604, 17192,
                38196, 58900, 34364, 8200,  56184, 52604, 45152, 40448, 57732,
                64120, 25076, 34096, 40992, 5068,  64736, 56756, 2008,  29644,
                15604, 38532, 25288, 6004,  17196, 57364, 21516, 4128,  29376,
                36588, 59272, 17016, 61844, 37080, 41540, 7712,  34136, 52360,
                44588, 6192,  62508, 3248,  46340, 42392, 35644, 18104, 12184,
                58776, 15184,
            },
            {
                50128, 57372, 17176, 60932, 6644,  15792, 64060, 21496, 1780,
                38176, 12432, 63008, 20036, 10560, 60808, 24644, 52080, 12552,
                27476, 3272,  33592, 63696, 40960, 44796, 54708, 7236,  28604,
                34964, 22040, 6112,  1596,  40688, 11276, 29912, 2300,  59496,
                53788, 9436,  35248, 57260, 38604, 51100, 59700, 27824, 34740,
                61112, 38904, 4788,  65080, 34464, 15240, 45416, 27440, 18196,
                43228, 1292,  16072, 63104, 26736, 11072, 42308, 34072, 64128,
                16208, 9076,  32496, 5356,  41784, 31148, 48796, 18184, 26812,
                60108, 10420, 60,    48336, 14196, 31360, 45032, 24252, 10728,
                53732, 29540, 48748, 12956, 60572, 24320, 14904, 8952,  29116,
                49548, 12632, 56,    44908, 51088, 8264,  13436, 43260, 52784,
                22280, 4016,  55312, 35148, 50160, 11100, 63212, 14644, 42476,
                56304, 22340, 28252, 1976,  31648, 20440, 56676, 17512, 24528,
                38804, 15468, 49768, 20424, 13332, 57964, 28860, 7032,  63996,
                46736, 30292,
            },
            {
                23716, 35724, 9068,  27648, 32556, 56452, 43732, 48156, 27868,
                59804, 5560,  23496, 54636, 43988, 28448, 57552, 1476,  37352,
                60236, 42468, 15512, 8696,  26828, 12100, 21280, 2380,  47384,
                10588, 51324, 17868, 64196, 34036, 57748, 15960, 22388, 38820,
                13572, 61528, 45288, 17664, 6508,  30856, 18948, 44760, 6792,
                53272, 31988, 11496, 25756, 54988, 2952,  59924, 5664,  11104,
                30184, 57520, 36036, 44112, 3176,  31208, 54168, 7168,  45888,
                51632, 22028, 61400, 14320, 27364, 63684, 1748,  36068, 45264,
                40408, 29884, 58164, 42980, 37708, 63512, 7472,  35432, 62188,
                26400, 4116,  21092, 43156, 2384,  18968, 48288, 35100, 4164,
                16968, 55652, 61612, 19800, 32160, 23856, 39616, 18480, 63080,
                33264, 45344, 60400, 324,   29832, 39764, 52924, 32016, 47936,
                7772,  12560, 44940, 52124, 6136,  48820, 60092, 2708,  46876,
                9260,  28192, 59416, 40136, 32096, 1540,  48892, 52632, 21140,
                33504, 120,
            },
            {
                55328, 40436, 65144, 46848, 1124,  38584, 11200, 18824, 32076,
                40908, 46020, 50952, 33628, 7344,  14456, 46380, 17692, 22020,
                54900, 30676, 47572, 56916, 35252, 60448, 39104, 58060, 31920,
                62432, 44336, 37832, 24600, 48196, 42768, 5804,  51776, 32972,
                25800, 41796, 1460,  23436, 62892, 47944, 12836, 56700, 21436,
                14532, 42672, 59148, 47764, 8668,  30552, 37872, 21620, 61704,
                39784, 50492, 12760, 20928, 61460, 38916, 14236, 19980, 29172,
                4008,  35792, 54484, 46896, 38548, 11772, 23424, 56156, 6696,
                15156, 23960, 53096, 5432,  22484, 11608, 51696, 18808, 5944,
                45908, 39660, 64700, 51040, 30424, 38664, 63316, 53360, 21224,
                42040, 36896, 29952, 10060, 54524, 60724, 27844, 47436, 6928,
                11540, 26888, 19348, 14004, 44056, 7160,  24592, 18728, 1160,
                60028, 33508, 39352, 64808, 14776, 43396, 29524, 12080, 63084,
                36220, 53380, 22632, 7508,  65264, 37540, 16036, 26528, 43284,
                38856, 10912,
            },
            {
                51976, 5740,  13352, 19584, 53144, 23924, 59104, 4168,  54940,
                9296,  15540, 26980, 584,   64564, 39580, 49928, 35340, 62792,
                5348,  11228, 19920, 51396, 976,   29956, 17572, 49720, 13888,
                25744, 2928,  54132, 9240,  13484, 28028, 60916, 44828, 18596,
                7624,  48952, 30008, 52968, 4156,  40784, 36468, 9264,  64224,
                49276, 28316, 18264, 22196, 40476, 50904, 17084, 53556, 46440,
                24400, 7832,  28308, 4756,  47596, 23700, 59884, 49252, 57856,
                40316, 10920, 24844, 3208,  17660, 59056, 43544, 51612, 31892,
                64276, 46948, 17380, 33840, 55196, 27452, 40048, 59564, 32772,
                55944, 15032, 9476,  25116, 33848, 6960,  11392, 27088, 59408,
                46708, 3012,  25664, 48460, 15388, 37380, 1176,  34564, 57848,
                50592, 41132, 53596, 65268, 22008, 58048, 36232, 61636, 41292,
                26640, 19908, 53984, 24692, 9188,  35636, 21644, 40260, 55536,
                18156, 376,   43864, 11900, 54388, 24712, 9736,  56104, 4972,
                60212, 18724,
            },
            {
                31456, 25876, 44304, 34508, 29584, 49084, 15012, 42232, 35488,
                63792, 21792, 37048, 57928, 18544, 30788, 9672,  3728,  26560,
                44988, 40024, 24724, 61908, 6128,  45956, 23644, 8548,  36492,
                41788, 16628, 30748, 58548, 20316, 36948, 1036,  54936, 11120,
                64648, 56428, 34768, 15644, 58988, 26324, 32936, 1860,  24796,
                38044, 5936,  1012,  35508, 63460, 12928, 26788, 33652, 1688,
                14812, 65420, 52720, 34276, 56052, 9100,  36348, 304,   26252,
                16940, 43768, 64992, 30396, 53188, 34228, 8748,  20728, 12920,
                2624,  37128, 9208,  60796, 1492,  44320, 15964, 48548, 22936,
                796,   36860, 52848, 17268, 61364, 46004, 55380, 5056,  32392,
                14204, 64916, 7476,  57492, 43744, 5460,  53164, 21388, 16152,
                30296, 4812,  37140, 8956,  33056, 48348, 3104,  15908, 54892,
                9872,  49268, 4720,  17184, 57276, 27272, 51628, 5408,  31584,
                25316, 49364, 34388, 28964, 47348, 19288, 45196, 62644, 29296,
                14608, 47520,
            },
            {
                63224, 3252,  56852, 60604, 9936,  6900,  62588, 30604, 25328,
                2916,  48316, 53228, 11724, 42532, 23928, 52740, 57064, 33252,
                16332, 53772, 8076,  37184, 14888, 42512, 63032, 52640, 56276,
                5224,  60844, 45732, 7052,  49696, 63588, 31568, 23464, 40120,
                27236, 5044,  21380, 10480, 43948, 19516, 50484, 55416, 45240,
                30456, 52476, 57272, 45860, 6684,  55476, 43360, 6208,  58720,
                36400, 41208, 18612, 44608, 15972, 29948, 42792, 12172, 61684,
                32984, 55860, 7488,  20292, 48528, 640,   28560, 39612, 61412,
                26448, 49784, 41420, 29396, 21176, 64900, 4664,  30416, 12736,
                42936, 27900, 59260, 4492,  41228, 13704, 19804, 43592, 23600,
                39844, 52156, 18816, 34008, 22828, 12340, 64304, 40084, 9728,
                60036, 23320, 46408, 17336, 26272, 51620, 12400, 29088, 45944,
                34488, 62972, 30784, 37736, 47524, 1008,  63900, 45796, 14252,
                62016, 4356,  57756, 15336, 61304, 6164,  34060, 39972, 2368,
                36496, 22972,
            },
            {
                42056, 16184, 39188, 21432, 37192, 17796, 45540, 51672, 12664,
                19540, 60976, 28828, 5812,  47072, 59896, 20320, 38420, 13144,
                65504, 47984, 32248, 27640, 55204, 21096, 32696, 1764,  28168,
                19928, 34628, 22940, 39560, 26644, 4096,  16396, 47672, 35884,
                14192, 45592, 51512, 38028, 63988, 7260,  13828, 60456, 17324,
                11400, 62760, 14980, 32452, 23620, 10184, 61032, 20652, 47812,
                30964, 10400, 57808, 22840, 2792,  64048, 53392, 19320, 47300,
                4980,  50620, 14588, 36620, 41352, 63116, 16192, 45572, 53704,
                32980, 19016, 57148, 12544, 47856, 37964, 9840,  57468, 50776,
                62628, 20068, 31912, 49096, 22760, 57184, 29492, 50280, 768,
                35732, 10388, 27664, 41500, 59344, 31492, 45544, 26096, 49480,
                35276, 1600,  61776, 56224, 4176,  42436, 63636, 38364, 5996,
                23276, 14280, 3044,  41192, 59172, 11372, 19032, 33868, 8824,
                42556, 20824, 37264, 41452, 1972,  52288, 22132, 12464, 51188,
                57796, 8508,
            },
            {
                33632, 11932, 50760, 1756,  54668, 27352, 4784,  56516, 38180,
                44476, 8412,  40652, 34316, 15064, 2056,  28196, 7548,  43152,
                84,    22384, 59280, 3008,  49532, 9764,  40412, 13088, 48084,
                64984, 10216, 51380, 55696, 12212, 42508, 52332, 8772,  57956,
                62172, 30760, 104,   57188, 27652, 31748, 42216, 23500, 4260,
                40212, 20720, 42984, 27580, 49436, 38764, 29688, 15760, 54260,
                3772,  26648, 49576, 6528,  33048, 39432, 24608, 7732,  29300,
                39044, 22192, 27512, 59904, 10552, 24276, 55636, 6396,  22568,
                3556,  60032, 7196,  24720, 52768, 32572, 19472, 25884, 35980,
                7632,  46672, 11456, 2072,  38188, 34672, 9172,  62888, 16456,
                60536, 55252, 47700, 2224,  16556, 51376, 3532,  18552, 55276,
                13320, 43352, 28684, 39144, 10856, 31296, 21656, 18324, 57620,
                42828, 60276, 51136, 20664, 28820, 43884, 23888, 53460, 59476,
                27152, 50896, 11044, 31896, 25816, 59352, 29860, 42912, 17228,
                26808, 49176,
            },
            {
                60128, 28352, 46284, 24340, 64544, 41168, 33964, 22508, 1608,
                58568, 32036, 17696, 55504, 64164, 36448, 49264, 61464, 51684,
                30192, 18672, 11688, 44276, 64012, 17472, 57832, 35796, 25204,
                43472, 15588, 388,   32552, 18644, 59176, 33952, 22180, 3228,
                19096, 41824, 24100, 16368, 47820, 2756,  54432, 37012, 48932,
                33984, 8132,  60064, 3136,  18436, 64184, 492,   41884, 23412,
                60248, 37792, 13912, 61420, 45564, 51724, 14712, 58624, 35404,
                62804, 2220,  44976, 51964, 4584,  31504, 47952, 37808, 11488,
                43296, 15188, 35104, 45708, 2420,  61296, 41708, 54348, 3304,
                16856, 40304, 58368, 27104, 65428, 6636,  53068, 25412, 45644,
                30472, 21360, 6548,  62800, 38680, 29092, 8528,  62344, 32128,
                6672,  48144, 20356, 15304, 53728, 59392, 46620, 516,   52412,
                27688, 10964, 35496, 7804,  55692, 14984, 36920, 5876,  39312,
                16444, 56460, 6724,  64292, 17848, 48308, 8872,  55316, 64772,
                6032,  20060,
            },
            {
                3704,  56424, 7608,  32168, 14396, 10584, 49348, 61628, 15968,
                26928, 50812, 22968, 4612,  25644, 44708, 10672, 16772, 24804,
                40520, 56608, 34204, 38288, 24216, 29452, 3872,  51888, 7416,
                38892, 57456, 29180, 46452, 62712, 4924,  28292, 44116, 53332,
                12092, 49212, 6976,  61384, 34940, 21624, 10932, 62052, 28812,
                55900, 25400, 51024, 37280, 13580, 52744, 46140, 34732, 8604,
                51104, 19256, 42236, 28176, 11220, 20600, 1128,  54568, 43376,
                17640, 12608, 56640, 33972, 19636, 14188, 58072, 27780, 65416,
                51416, 40240, 55108, 28064, 17272, 6060,  13892, 44904, 64144,
                33228, 24624, 54848, 14304, 44928, 18192, 42096, 12316, 3264,
                37804, 13824, 43316, 24992, 11700, 54420, 36080, 22248, 41752,
                58040, 24808, 63108, 34376, 6448,  25256, 36456, 9212,  32672,
                16496, 48552, 25028, 65384, 32272, 1816,  61620, 49164, 30572,
                2844,  46804, 23172, 44356, 39040, 14008, 36012, 828,   32204,
                39404, 44764,
            },
            {
                36236, 17504, 61396, 42720, 52000, 19924, 29484, 7100,  42832,
                10208, 62428, 46576, 12776, 53004, 31040, 21156, 58428, 35240,
                4864,  8680,  53244, 15264, 6368,  46684, 62196, 31520, 19296,
                60468, 23476, 6036,  36428, 21356, 14528, 38544, 65344, 25232,
                37176, 32372, 55440, 39868, 13080, 44912, 51584, 7516,  18600,
                732,   46820, 11192, 56748, 33096, 5816,  25900, 57404, 12312,
                31952, 64428, 2264,  55532, 36592, 48060, 31784, 26664, 10140,
                49300, 23940, 40000, 8424,  62064, 43764, 35916, 2748,  17644,
                30800, 23592, 9504,  63412, 39392, 49556, 23040, 29408, 10568,
                51652, 20812, 5180,  49792, 31004, 22156, 59920, 32820, 49508,
                58484, 52304, 34208, 56572, 19084, 48352, 44160, 14936, 260,
                37936, 10384, 51968, 2196,  41252, 49488, 13060, 64228, 55068,
                40008, 2512,  44780, 18396, 52796, 46372, 26540, 21536, 12224,
                63256, 35388, 28636, 54688, 3400,  57692, 20528, 46252, 23528,
                10936, 54268,
            },
            {
                30744, 12804, 22416, 38428, 820,   58216, 36408, 46184, 54856,
                32940, 37924, 608,   58136, 39396, 7036,  42148, 2892,  47620,
                62656, 44804, 26444, 60912, 41860, 21640, 54784, 14496, 45104,
                11580, 48980, 54384, 41984, 8184,  51076, 47860, 1044,  16788,
                60252, 9560,  28760, 19612, 3756,  63788, 24564, 31352, 14484,
                40936, 65288, 22468, 29516, 43836, 20048, 62636, 16860, 40340,
                4976,  46704, 24860, 17356, 8120,  63436, 41508, 60292, 6064,
                32900, 64844, 29444, 47552, 856,   25872, 52852, 21484, 7768,
                49072, 60372, 380,   20448, 31756, 57960, 35436, 59856, 1200,
                38132, 43588, 60396, 35804, 152,   55836, 39244, 5292,  27224,
                17548, 8440,  29448, 1484,  65156, 5108,  26584, 60964, 50624,
                28436, 16768, 44544, 30868, 60692, 19404, 29256, 43980, 21960,
                7028,  61700, 38076, 13172, 4560,  40500, 9548,  58108, 42696,
                18852, 52460, 8588,  12812, 32992, 25540, 60856, 52932, 15368,
                62372, 25704,
            },
            {
                2328,  48224, 53108, 26660, 5724,  63484, 17300, 25080, 3132,
                20748, 14256, 29132, 18868, 33856, 63248, 15612, 54908, 28948,
                14028, 20588, 32044, 2156,  49752, 10276, 37136, 1244,  26636,
                33760, 3352,  17624, 61160, 27112, 57576, 10980, 30512, 54584,
                6456,  46028, 50748, 58820, 26880, 41936, 56140, 37388, 59096,
                44616, 34604, 16536, 3544,  59612, 8248,  37880, 49740, 28620,
                21844, 52928, 34164, 59296, 30136, 3696,  22912, 15584, 51284,
                37604, 4396,  20960, 54792, 16652, 38560, 10308, 59156, 44880,
                34000, 13804, 37344, 47044, 11748, 53340, 7892,  15892, 47964,
                27872, 12964, 8936,  29232, 16476, 47328, 9692,  63452, 20924,
                41296, 61956, 23232, 46376, 40376, 31380, 9608,  56792, 33676,
                20104, 64440, 7612,  23064, 54276, 15688, 5748,  57420, 26080,
                50340, 30900, 20800, 59120, 28972, 35132, 16948, 54240, 32488,
                520,   40036, 21940, 62036, 49536, 6544,  41320, 29368, 4416,
                50276, 41684,
            },
            {
                19384, 65408, 9392,  43760, 34108, 47492, 12860, 40452, 57720,
                50196, 65092, 41828, 48780, 9460,  26860, 50664, 23272, 38080,
                10484, 51368, 58660, 18068, 35112, 28260, 58172, 41216, 50872,
                64580, 38276, 30648, 13244, 34908, 23840, 40544, 20008, 34220,
                42860, 22464, 14444, 35368, 5988,  16876, 1632,  49596, 21128,
                4760,  9456,  53992, 39792, 48292, 23544, 55168, 1768,  44596,
                58348, 9696,  14184, 43536, 50084, 39032, 11936, 56760, 19212,
                45088, 57976, 13720, 42700, 31824, 49884, 63656, 28708, 40552,
                4904,  56996, 27028, 43084, 4636,  25428, 41164, 21476, 56200,
                62120, 18760, 52148, 64052, 42576, 24756, 13440, 35004, 53900,
                2736,  48576, 11856, 35176, 16376, 21776, 53280, 13404, 3328,
                45988, 39740, 56252, 11840, 37568, 47376, 32908, 2812,  36724,
                12468, 8968,  55552, 47592, 23772, 62480, 49936, 6940,  25608,
                45536, 59044, 15296, 37276, 43924, 18496, 11356, 34512, 38148,
                8340,  58016,
            },
            {
                32476, 37116, 16016, 56016, 30048, 20548, 51656, 7436,  27716,
                35464, 5900,  24048, 55356, 3604,  45836, 60540, 1252,  33400,
                65012, 40300, 4420,  46104, 12512, 63820, 7148,  16180, 20192,
                55644, 9700,  46972, 52448, 1924,  63924, 45656, 4312,  61904,
                26380, 2044,  64396, 38940, 53804, 47616, 33780, 12288, 28092,
                51456, 25148, 62124, 32008, 14296, 27832, 11004, 35440, 15684,
                62380, 37424, 112,   20780, 54736, 25980, 46564, 34736, 28916,
                1260,  25724, 9464,  60988, 23148, 6248,  13232, 18704, 54356,
                24052, 16468, 64556, 52068, 19120, 61524, 31036, 45408, 6276,
                33068, 40224, 23388, 3564,  37004, 58068, 50572, 28240, 44184,
                15280, 38060, 59604, 6876,  57656, 49544, 36800, 42484, 25196,
                6044,  49032, 35024, 26700, 3784,  62880, 40416, 52616, 59588,
                17240, 42088, 33800, 1520,  6428,  43508, 14500, 37928, 64932,
                4592,  50720, 31032, 26824, 2976,  64596, 51664, 57196, 17352,
                22236, 45384,
            },
            {
                28248, 6476,  59972, 23852, 3836,  62152, 10260, 31996, 61316,
                16548, 44240, 11648, 59364, 21524, 36548, 12388, 18532, 43740,
                7284,  24952, 30088, 56256, 21984, 52012, 24832, 32904, 44048,
                5952,  25580, 21836, 43072, 7240,  18040, 55936, 15216, 37460,
                48732, 56656, 18880, 10640, 30460, 23564, 62700, 8192,  60324,
                38260, 46212, 18892, 2312,  42892, 64444, 51184, 31184, 6832,
                25268, 41600, 29432, 65024, 6104,  17060, 61168, 8296,  52836,
                63668, 40824, 48860, 36336, 3432,  56812, 34320, 46964, 1788,
                32004, 49716, 8544,  34684, 1444,  55524, 13276, 36464, 2720,
                26260, 53980, 7448,  48896, 31168, 19836, 1736,  7308,  56540,
                24088, 31248, 51820, 26076, 43648, 1056,  18832, 62716, 58648,
                29908, 14440, 60228, 18696, 43888, 9996,  23976, 19760, 28208,
                64612, 46044, 25296, 52692, 18928, 57132, 10592, 22212, 29640,
                17732, 11584, 55336, 7844,  46740, 22976, 27916, 8,     47556,
                63192, 12068,
            },
            {
                1724,  40944, 49120, 13216, 38852, 54520, 42680, 22768, 46780,
                1892,  53508, 39048, 32348, 16124, 41324, 28188, 57488, 47332,
                53792, 15464, 36032, 42836, 316,   38876, 47416, 60520, 2796,
                29928, 62224, 14880, 59456, 38920, 32304, 50600, 29004, 8532,
                12520, 32104, 41304, 7224,  44568, 57548, 20092, 42536, 15868,
                30368, 56900, 12020, 36148, 58008, 5420,  21192, 46592, 56248,
                19216, 51736, 47956, 11708, 36024, 31720, 2940,  42136, 21840,
                14720, 33300, 18256, 28332, 52264, 43872, 25812, 37968, 62420,
                11852, 41516, 21772, 29164, 39124, 47472, 17684, 50236, 64948,
                12052, 44532, 59316, 15436, 10580, 61500, 46024, 33460, 64384,
                10128, 19336, 4124,  14424, 63860, 29096, 7680,  32860, 10864,
                51312, 22036, 872,   31772, 52056, 58056, 14136, 49056, 200,
                6712,  37648, 14948, 63076, 31300, 39900, 34048, 54640, 48532,
                42156, 35192, 61560, 19592, 39856, 59608, 13820, 33220, 42528,
                25936, 53748,
            },
            {
                35040, 51984, 19780, 45036, 26516, 588,   34664, 13796, 56636,
                19532, 30224, 25584, 7940,  64272, 53092, 4688,  31340, 9720,
                21168, 59668, 6372,  62056, 17008, 29216, 9852,  13840, 57356,
                40628, 49136, 34152, 53836, 26912, 10852, 23052, 59844, 44228,
                52804, 24512, 60152, 50524, 27672, 13712, 3492,  35028, 53592,
                900,   7144,  23284, 52980, 26368, 16920, 40056, 61228, 12144,
                33456, 3648,  60076, 23220, 44936, 57000, 50588, 11040, 38240,
                55464, 6616,  58976, 11432, 64728, 15904, 9008,  20604, 58032,
                6704,  45056, 59500, 14872, 63024, 9768,  27316, 58752, 22756,
                38716, 18044, 34504, 27740, 41156, 52820, 22544, 17056, 39280,
                42812, 60704, 47076, 33288, 53872, 23460, 39004, 46928, 17124,
                41700, 54992, 38256, 65136, 5184,  28984, 35972, 44752, 32032,
                55976, 21888, 10052, 49412, 5084,  26100, 61444, 1024,  8496,
                57992, 24296, 2436,  32336, 53072, 9848,  37088, 56008, 5676,
                15472, 61312,
            },
            {
                23240, 58272, 9976,  31624, 64264, 17456, 48236, 60192, 37668,
                4948,  62596, 50392, 44696, 436,   48672, 23536, 61144, 38312,
                50740, 27612, 12076, 48828, 54932, 34372, 20840, 52972, 36204,
                17936, 11260, 672,   20080, 4940,  47832, 40180, 1404,  20624,
                5712,  36472, 16508, 244,   63600, 37780, 46600, 24432, 48968,
                64904, 40584, 44392, 33172, 49812, 9668,  36628, 1112,  43900,
                27324, 16120, 8884,  40548, 18856, 27884, 62544, 24668, 48036,
                29796, 2396,  42948, 20908, 38880, 148,   47868, 30380, 51956,
                27500, 36320, 51016, 5544,  24152, 42844, 4084,  30516, 8152,
                56596, 1784,  47888, 63784, 5760,  37676, 12548, 29660, 54944,
                600,   27360, 36520, 8512,  40884, 12344, 59784, 55732, 2324,
                24372, 8260,  47444, 16260, 42068, 21172, 7464,  62208, 12200,
                51500, 40848, 29184, 59248, 44260, 12816, 47208, 19960, 28264,
                38408, 13516, 49700, 43120, 15804, 29248, 3972,  50488, 20520,
                38924, 7664,
            },
            {
                29768, 16404, 4504,  36536, 51028, 6600,  28988, 9040,  24488,
                41104, 11140, 15100, 36244, 20012, 34424, 13524, 42740, 17204,
                1464,  33208, 40804, 24452, 3356,  44256, 63300, 5536,  23576,
                46056, 27860, 65212, 37308, 58276, 14552, 62724, 33248, 54852,
                64680, 29864, 47908, 22052, 32764, 6620,  55496, 17344, 27400,
                9220,  20940, 15048, 63028, 3476,  59384, 29976, 22344, 53116,
                63288, 38676, 57624, 53996, 2104,  14152, 6348,  33960, 16324,
                60616, 45932, 23748, 50680, 32564, 54252, 61092, 41060, 17708,
                13676, 2092,  19476, 31092, 55148, 34472, 53368, 46436, 15000,
                41948, 31956, 20816, 55124, 24392, 2860,  60120, 50900, 6640,
                48436, 21540, 57476, 16812, 49896, 20976, 4352,  27040, 36264,
                63260, 33536, 27420, 11940, 50204, 56352, 38912, 16840, 26256,
                60384, 3852,  35076, 17760, 23288, 3068,  36136, 52204, 16228,
                45136, 62824, 7156,  21704, 64132, 46116, 24404, 60484, 31420,
                48500, 43772,
            },
            {
                54492, 46980, 62660, 24620, 56204, 41588, 21284, 52516, 33060,
                46232, 55664, 28032, 60700, 57216, 8984,  26608, 63568, 5888,
                55888, 46420, 64652, 19560, 9160,  58884, 26792, 39824, 50444,
                8328,  55428, 43212, 30864, 51220, 8196,  42736, 27448, 18072,
                13600, 39672, 10168, 58916, 52372, 41868, 11180, 60760, 39200,
                32504, 57444, 46944, 28576, 19004, 42180, 55140, 14508, 47508,
                4384,  35064, 24760, 30356, 49044, 36940, 44092, 52432, 20188,
                9032,  35716, 56472, 27540, 7272,  12888, 24516, 4328,  34812,
                64044, 57016, 46752, 39556, 60608, 12532, 18580, 37536, 63308,
                51616, 26904, 9400,  13808, 44952, 33876, 42380, 25856, 15516,
                32064, 10968, 65528, 2948,  44864, 62252, 31856, 52220, 14224,
                45148, 19608, 61080, 2548,  31288, 24288, 46400, 2116,  33904,
                20096, 46976, 8168,  50752, 56908, 42000, 63584, 30808, 59080,
                3624,  25972, 54840, 35928, 1052,  57008, 40576, 16980, 10408,
                64880, 12792,
            },
            {
                21032, 33924, 1356,  39260, 11248, 15084, 59124, 3524,  65252,
                18128, 2132,  22992, 6524,  39812, 30252, 54096, 45200, 36380,
                22360, 29900, 13944, 37228, 51884, 30768, 15808, 2020,  32312,
                61028, 13316, 3684,  21992, 16524, 24936, 35640, 46984, 3004,
                51412, 43808, 4884,  26232, 15360, 20236, 30120, 4704,  50308,
                13152, 2500,  36768, 6436,  11824, 48680, 25396, 7188,  31932,
                20004, 50688, 12504, 7636,  63756, 21588, 58096, 540,   65228,
                40480, 13476, 3596,  63088, 16892, 36972, 45764, 55680, 22208,
                8096,  42604, 23456, 10000, 3052,  49492, 21916, 1080,  24732,
                5316,  60640, 36648, 49776, 58724, 17516, 9000,  63492, 37000,
                56216, 43628, 25056, 34712, 29016, 7232,  18088, 42796, 9168,
                57400, 5040,  51232, 36840, 58756, 9496,  63428, 53232, 43204,
                55116, 14172, 65532, 38344, 27276, 15168, 6568,  22320, 10976,
                33748, 41336, 18312, 31160, 11844, 27312, 7424,  52356, 34808,
                2028,  27436,
            },
            {
                6264,  59824, 43192, 18616, 30556, 49472, 44720, 36108, 13224,
                30708, 50912, 43404, 47864, 16828, 51288, 3992,  19240, 11264,
                58984, 7916,  53620, 4216,  42496, 48192, 11516, 57076, 20228,
                35448, 25772, 39656, 45528, 57144, 5500,  61372, 11556, 57884,
                23892, 62068, 34028, 46088, 56708, 36640, 62872, 44824, 22552,
                59492, 52096, 24172, 55816, 62164, 34176, 38952, 65520, 10620,
                41672, 61380, 17616, 46368, 40888, 32388, 10780, 28392, 25260,
                49796, 31412, 19292, 47756, 42368, 59212, 28688, 11020, 49420,
                31636, 52620, 27072, 16924, 65524, 29340, 44788, 59440, 34112,
                11636, 43896, 19220, 268,   28856, 53736, 22048, 46712, 3764,
                19236, 52312, 40380, 14332, 54548, 58672, 38628, 23348, 48704,
                29636, 40064, 22888, 43584, 14612, 19176, 28496, 5924,  11064,
                30496, 24912, 804,   31936, 10292, 54084, 48504, 40420, 56024,
                50836, 8352,  46916, 61596, 50356, 37376, 20052, 45096, 23740,
                56784, 38460,
            },
            {
                14852, 51544, 26676, 7696,  61740, 23296, 6304,  26832, 54184,
                38744, 9656,  63384, 33620, 12708, 58688, 24660, 33004, 50112,
                40016, 27024, 60980, 17480, 33764, 22184, 64180, 41520, 46612,
                17116, 53808, 63744, 10132, 32576, 52960, 38728, 21052, 31388,
                8452,  37764, 18840, 1744,  49232, 8564,  24996, 748,   34816,
                17736, 29836, 39816, 43848, 20704, 256,   16872, 57224, 45192,
                23824, 1320,  58692, 26344, 3692,  14940, 54544, 42688, 37516,
                5048,  59696, 53536, 9972,  33844, 2768,  19916, 62408, 39056,
                916,   14128, 58328, 38032, 33448, 6588,  40744, 54552, 16672,
                30360, 56188, 39704, 65052, 32512, 5248,  40204, 27752, 13284,
                60196, 8228,  1468,  21764, 46344, 11644, 33112, 212,   64136,
                12616, 17404, 54928, 7640,  34124, 48652, 41084, 37144, 22336,
                58232, 40232, 45008, 59656, 19888, 34172, 2784,  17984, 27600,
                60428, 23944, 14912, 5024,  43392, 58968, 14340, 62796, 4368,
                42292, 48904,
            },
        },
        {
            {
                62408, 48216, 27928, 16728, 33952, 19944, 11316, 53132, 15588,
                41320, 33908, 55336, 24424, 63992, 9436,  21616, 38812, 12096,
                48560, 52388, 14184, 3204,  34172, 9688,  50412, 20200, 63976,
                45400, 12248, 41732, 25036, 1124,  37680, 20032, 53652, 41908,
                47036, 38236, 51212, 24224, 48060, 356,   55660, 44940, 3572,
                41400, 15348, 31760, 820,   64216, 13612, 57020, 36740, 18172,
                26672, 45152, 13788, 51484, 30784, 46912, 41788, 21340, 33816,
                52456, 57276, 27060, 61488, 51728, 6444,  29392, 49372, 53860,
                35868, 46548, 61016, 7192,  34620, 13256, 19212, 30824, 26944,
                56488, 12116, 54072, 33084, 27640, 50584, 6304,  25976, 59176,
                19768, 14844, 28704, 47088, 36236, 62252, 30040, 42828, 18816,
                4124,  15212, 41712, 57540, 36076, 14192, 6756,  60568, 49952,
                39032, 30168, 48212, 15780, 25700, 10780, 1712,  17212, 50736,
                9648,  31304, 15776, 54464, 18632, 5740,  63456, 23384, 55084,
                19776, 34588,
            },
            {
                24760, 52224, 9976,  38748, 2584,  47044, 58856, 36404, 45072,
                65088, 108,   19196, 43960, 5512,  40856, 29296, 34116, 2512,
                24124, 31864, 44608, 63624, 54972, 29316, 57436, 25360, 36292,
                52864, 5292,  22088, 54680, 45752, 10648, 50812, 32052, 23236,
                12344, 2372,  61428, 16268, 10384, 33544, 22828, 35860, 49164,
                62748, 25076, 51524, 47252, 34672, 43540, 22372, 9144,  1856,
                53648, 20660, 59860, 37200, 10632, 26156, 4,     60996, 38060,
                9856,  47976, 39948, 16440, 2092,  19664, 38680, 12176, 32796,
                1092,  10768, 41628, 21876, 53744, 58592, 47468, 37272, 8040,
                40660, 20092, 48576, 864,   38416, 45744, 63344, 30268, 3284,
                56072, 33940, 7832,  58672, 25100, 16880, 48192, 956,   64060,
                37676, 52124, 22052, 8948,  64632, 24412, 54988, 42960, 4348,
                19132, 13252, 1172,  61444, 40616, 31984, 53036, 38204, 44972,
                23332, 6468,  48640, 26500, 60716, 32004, 36768, 14956, 42780,
                29008, 7056,
            },
            {
                36576, 14588, 60732, 54456, 30548, 63400, 23920, 28264, 9204,
                22408, 48888, 30204, 58380, 17004, 51888, 60476, 46676, 62976,
                6292,  58372, 10560, 22272, 17152, 42928, 2228,  15744, 8284,
                30376, 18496, 32652, 63196, 14516, 28740, 60320, 3352,  58368,
                18344, 34296, 55028, 41112, 28476, 51972, 59644, 17032, 30076,
                9092,  21272, 55972, 11652, 3992,  26040, 52620, 48084, 38820,
                65228, 28796, 3016,  7464,  49324, 15796, 54264, 6620,  24368,
                30028, 3140,  59808, 32436, 25192, 64208, 46180, 58360, 23252,
                63096, 56176, 31208, 16560, 4232,  28940, 1256,  52244, 16192,
                63208, 24420, 43616, 16980, 8308,  21316, 14156, 40696, 10964,
                22924, 51476, 43376, 12440, 4708,  40568, 57288, 27740, 23308,
                32264, 11836, 48108, 28900, 39796, 1016,  20644, 31700, 26100,
                63536, 54324, 22688, 36012, 55888, 19988, 64904, 13432, 3180,
                62440, 57916, 37848, 392,   41924, 21360, 49052, 10816, 3576,
                47572, 56596,
            },
            {
                1948,  41504, 22220, 8080,  44296, 5868,  17568, 39856, 51468,
                14140, 6880,  38576, 11544, 35700, 3552,  25628, 19332, 41880,
                15700, 27688, 39456, 49772, 7340,  37288, 47416, 62520, 41152,
                59340, 48976, 38636, 4148,  47696, 35052, 7316,  38836, 49296,
                26304, 45324, 7104,  20500, 63496, 13272, 4596,  38264, 54788,
                5468,  45424, 37076, 17664, 58232, 30368, 60824, 15680, 32512,
                12004, 41976, 56000, 34892, 63740, 39836, 32720, 57792, 43112,
                14824, 20652, 44664, 8804,  49040, 35216, 7324,  42700, 18100,
                5996,  26632, 50220, 39324, 64636, 45704, 23028, 60904, 32488,
                49924, 3332,  57864, 31288, 61376, 52848, 34204, 54684, 48044,
                37580, 63704, 18696, 31500, 53664, 20844, 9524,  50344, 6784,
                54196, 60388, 2772,  18360, 33248, 46532, 59212, 51824, 10332,
                41588, 33356, 46332, 8196,  28408, 4652,  48372, 25236, 35040,
                28260, 20520, 51688, 13592, 9008,  29252, 59544, 52652, 25996,
                64012, 17936,
            },
            {
                46316, 32240, 50316, 26704, 35360, 13340, 55216, 1492,  61616,
                32872, 53500, 62756, 27196, 47648, 56792, 12968, 8060,  54616,
                51024, 35108, 708,   56696, 32756, 23792, 53512, 27920, 13304,
                272,   55304, 9544,  26572, 42248, 20724, 64604, 15904, 43000,
                10124, 60104, 30908, 37396, 2100,  47080, 43708, 25880, 64584,
                14724, 33276, 60520, 42624, 7644,  40152, 20136, 6032,  44472,
                24912, 50640, 17296, 23428, 44968, 19960, 28316, 11660, 48460,
                64652, 54584, 37392, 13068, 21820, 54976, 15428, 28400, 51976,
                37240, 45272, 13672, 8388,  34976, 18388, 10976, 39996, 5920,
                13748, 35864, 27912, 10676, 41324, 25116, 1904,  60408, 17068,
                5904,  27220, 168,   46132, 60492, 38460, 33592, 44836, 16028,
                40148, 26112, 43768, 56052, 62072, 12148, 16812, 7156,  37568,
                58176, 3028,  17248, 51152, 12112, 59956, 42020, 17976, 54516,
                10436, 43072, 32896, 63040, 45896, 57024, 17132, 7304,  33664,
                40060, 12020,
            },
            {
                58712, 4536,  65236, 19224, 57744, 48436, 42448, 29900, 19920,
                45392, 24712, 41808, 2064,  21172, 32540, 44768, 37864, 30772,
                21324, 65024, 46372, 18720, 61932, 11156, 4616,  20760, 34312,
                44984, 23276, 16452, 60816, 50424, 12048, 24624, 55780, 29780,
                620,   52232, 14296, 24332, 56424, 32424, 18708, 10616, 41052,
                52660, 1292,  27888, 23452, 50496, 13224, 36072, 62740, 440,
                57948, 8252,  31064, 12756, 4944,  59192, 2016,  51696, 17916,
                34168, 5728,  27696, 50724, 61712, 644,   39896, 11084, 62268,
                3268,  20352, 60328, 24084, 52988, 57408, 43404, 26464, 54700,
                46008, 21784, 64368, 50984, 4912,  46944, 19576, 28568, 9608,
                32660, 41920, 56656, 14576, 26428, 2852,  62948, 12872, 59048,
                19704, 35560, 9732,  5000,  50680, 36224, 28012, 45396, 23932,
                14020, 29344, 62388, 44088, 23664, 37596, 31136, 7364,  47024,
                61272, 15356, 2604,  24436, 35564, 4692,  38828, 44188, 61352,
                19992, 29628,
            },
            {
                54844, 10380, 15684, 38968, 2364,  25080, 59604, 8352,  37224,
                4584,  60172, 17520, 9528,  49960, 64448, 15316, 59396, 1936,
                11260, 25836, 5568,  14064, 29960, 43364, 58420, 39612, 51412,
                65248, 29640, 37140, 57324, 1548,  31320, 52600, 4808,  35964,
                22184, 62380, 48180, 42192, 8792,  59136, 50232, 22304, 29372,
                48308, 19352, 10948, 64016, 2804,  54924, 47332, 28376, 53272,
                21148, 40076, 47592, 61260, 53848, 25508, 37256, 8528,  40788,
                23676, 2608,  57664, 41760, 31120, 25524, 58716, 34252, 48208,
                30260, 55220, 42556, 332,   31852, 5412,  15240, 30348, 59656,
                9072,  39040, 18196, 55912, 12844, 36652, 58580, 39476, 45200,
                61864, 50588, 23260, 36364, 7712,  52168, 22696, 29860, 5344,
                47320, 65260, 31044, 14812, 21496, 42392, 1844,  64256, 49272,
                53504, 21032, 39596, 57100, 16,    15556, 50364, 58200, 3944,
                29708, 39936, 53456, 18488, 50140, 27732, 22544, 14012, 812,
                49448, 23908,
            },
            {
                42628, 34940, 45368, 53308, 31356, 11100, 21924, 50716, 15480,
                57040, 12236, 35404, 55460, 29428, 39500, 4400,  24304, 49280,
                44032, 60136, 41292, 36888, 55256, 49456, 25592, 8500,  17948,
                3792,  11444, 43500, 6908,  34148, 18140, 40820, 59524, 46700,
                19056, 39132, 5248,  17532, 27496, 35236, 6084,  61904, 3528,
                35672, 59812, 39004, 31924, 43680, 25552, 17324, 34144, 10776,
                37524, 64388, 2500,  34396, 42440, 15284, 46628, 62508, 29736,
                59728, 46156, 11180, 16592, 6436,  19128, 44584, 4916,  22912,
                16780, 9756,  36160, 27232, 48500, 63180, 37068, 20436, 4000,
                49212, 32856, 588,   42824, 23596, 31992, 7064,  52624, 13892,
                3308,  19012, 11236, 64568, 48344, 17172, 43168, 37356, 56280,
                540,   23216, 52380, 39104, 25820, 54968, 32588, 18804, 4676,
                31480, 9612,  5760,  26240, 34344, 64180, 10136, 22468, 36348,
                19844, 45308, 7900,  65452, 11496, 56040, 47284, 32164, 53720,
                37432, 7720,
            },
            {
                60216, 21396, 28528, 6988,  61592, 40888, 33984, 64540, 27676,
                47784, 31860, 22656, 43748, 7152,  52564, 19656, 34760, 57192,
                17224, 31728, 52676, 8952,  20204, 2276,  15616, 61580, 32508,
                47740, 54176, 20544, 26212, 62896, 45472, 8328,  13924, 27872,
                10856, 33596, 54376, 65352, 12252, 53072, 39276, 16004, 56032,
                8112,  45944, 14260, 57072, 21052, 8520,  59012, 4040,  45492,
                14816, 26636, 18956, 29016, 5964,  21492, 32304, 55152, 13936,
                19828, 36356, 52844, 33280, 47844, 64112, 53216, 13724, 56988,
                40904, 65532, 51504, 13124, 18980, 44628, 10276, 51060, 62004,
                25632, 14804, 58244, 29264, 48568, 16456, 65528, 21620, 25892,
                35312, 54896, 30304, 40444, 32860, 58432, 9652,  26904, 49744,
                33916, 12212, 45556, 61164, 7356,  59000, 10500, 40264, 60668,
                43592, 36816, 47984, 54368, 19092, 46064, 40956, 26828, 55060,
                12552, 59300, 25928, 31636, 42372, 3212,  59088, 9812,  64300,
                16676, 27128,
            },
            {
                13500, 1568,  56800, 17852, 46780, 14452, 5656,  43508, 3356,
                39572, 52200, 872,   63360, 26296, 11796, 61800, 28684, 7572,
                39788, 3076,  22672, 28244, 64168, 34628, 46268, 38240, 27348,
                60000, 40220, 14944, 49568, 55492, 22700, 38004, 64184, 49748,
                58264, 2420,  22928, 30480, 44748, 908,   25228, 43416, 31036,
                23168, 53524, 26732, 5628,  48532, 37816, 62112, 30488, 23148,
                56404, 52448, 9600,  48572, 58268, 51376, 10920, 508,   43720,
                7208,  26268, 62844, 22280, 38572, 9148,  27636, 36756, 31800,
                1864,  25348, 6776,  59576, 39412, 2980,  56044, 23320, 40912,
                35516, 45508, 6192,  61232, 9868,  55212, 1556,  43560, 50028,
                58964, 7296,  46720, 1360,  20428, 4296,  54304, 14744, 41072,
                18156, 57264, 28428, 2316,  16720, 46728, 50148, 13540, 25376,
                56264, 16076, 63200, 11776, 32472, 6912,  52396, 2556,  62364,
                33524, 48876, 652,   39112, 16196, 20744, 36408, 29744, 41344,
                4080,  51308,
            },
            {
                63376, 38660, 49120, 24356, 35620, 52332, 20408, 55840, 25444,
                9176,  19256, 59252, 36424, 16844, 47068, 42476, 54796, 14172,
                46652, 62892, 11944, 48400, 42304, 5696,  56832, 12708, 1132,
                22832, 5308,  35952, 10056, 2860,  30116, 16580, 3748,  24720,
                43904, 40964, 15436, 50756, 59896, 20668, 47816, 63288, 10748,
                18104, 42008, 716,   34268, 16616, 52236, 12044, 42680, 49620,
                6656,  41300, 60268, 35764, 17168, 39168, 65328, 27448, 49376,
                39964, 56048, 1952,  14480, 51252, 3736,  42336, 20456, 61236,
                49352, 45532, 34292, 21808, 54232, 28480, 33164, 17052, 1940,
                11232, 53852, 19032, 37424, 26888, 40212, 34360, 11928, 28768,
                38296, 16228, 24212, 60336, 28340, 45444, 63616, 22028, 6068,
                61980, 8884,  43088, 20656, 36784, 29608, 22972, 34912, 796,
                28772, 20356, 3200,  40056, 59764, 14072, 29380, 43940, 17276,
                5316,  37320, 22320, 54476, 61436, 51776, 6904,  24948, 18420,
                46540, 33796,
            },
            {
                43492, 5788,  30820, 9344,  63016, 232,   29312, 59792, 16444,
                45872, 30012, 13576, 49180, 4964,  31388, 1696,  22216, 36552,
                26452, 56364, 32976, 16380, 53956, 24256, 30900, 18976, 52572,
                44440, 57084, 28388, 61960, 41656, 47160, 33320, 53584, 7288,
                29580, 56352, 9200,  34668, 37856, 7696,  13996, 33404, 58508,
                37488, 49472, 65140, 60080, 40836, 28756, 2648,  19544, 63780,
                32616, 1156,  13104, 24792, 44888, 3752,  23020, 34864, 16940,
                60808, 10200, 30000, 45564, 24940, 60100, 12184, 54280, 5212,
                17644, 10824, 29556, 15112, 47664, 8688,  64316, 43468, 57740,
                31140, 63600, 22376, 51980, 5008,  47440, 61920, 19748, 3672,
                62924, 10088, 42520, 52940, 13456, 38784, 35264, 30804, 51240,
                25372, 38372, 32728, 63448, 53128, 3968,  57524, 65004, 8496,
                46168, 51328, 30764, 24184, 49384, 21288, 38044, 57612, 24400,
                51012, 14852, 9956,  45952, 27492, 13352, 43664, 62240, 54912,
                10732, 23172,
            },
            {
                19460, 53572, 58840, 12916, 42736, 39252, 48416, 11548, 36980,
                62784, 53844, 41624, 21424, 56868, 38856, 51784, 10108, 61072,
                4740,  19324, 38708, 788,   60332, 10376, 40624, 62912, 48328,
                8988,  32168, 17456, 51572, 12776, 57620, 19448, 61396, 36360,
                13556, 63636, 19136, 26216, 4284,  55548, 27936, 51172, 2900,
                6168,  29268, 12808, 21744, 9564,  24576, 54632, 36228, 15752,
                39244, 21416, 29492, 62224, 8084,  31172, 57392, 52672, 4444,
                21336, 33024, 41616, 18456, 57412, 34488, 30500, 44700, 24264,
                39156, 52344, 62884, 42232, 4304,  36920, 25280, 13388, 48708,
                27860, 8028,  42176, 32968, 13180, 17316, 24732, 45672, 31896,
                55960, 48524, 33884, 18552, 6724,  49260, 10828, 1652,  44868,
                55016, 3468,  48932, 10188, 14568, 44108, 39940, 18148, 53808,
                36388, 60240, 6400,  44520, 54048, 944,   65044, 10908, 31944,
                40604, 62904, 56348, 30620, 3948,  34548, 48512, 988,   38212,
                57988, 29112,
            },
            {
                2176,  16044, 33476, 26284, 21816, 54764, 7768,  32716, 22768,
                2832,  6332,  34008, 8844,  25216, 64688, 18504, 33452, 48984,
                42980, 8684,  52020, 45288, 26964, 36380, 6812,  21944, 15244,
                35208, 64908, 24700, 38712, 360,   26584, 9396,  40068, 21600,
                52116, 1484,  45096, 48916, 60884, 42468, 17588, 39912, 24312,
                46204, 19472, 55652, 31988, 44392, 58300, 47512, 7592,  26924,
                57552, 46396, 51088, 54792, 41992, 15080, 46188, 12132, 38304,
                47836, 53876, 64176, 7944,  49712, 104,   15872, 64504, 8368,
                58812, 35980, 912,   57136, 18684, 60140, 52076, 5364,  20836,
                39288, 15812, 50076, 3288,  60404, 53364, 57336, 8456,  40936,
                22504, 5180,  27192, 65040, 56716, 24804, 60740, 19940, 58620,
                13100, 17216, 22784, 59976, 26308, 6100,  31764, 11900, 22176,
                42228, 14932, 10656, 33532, 16756, 27420, 8172,  47668, 20164,
                6328,  26392, 42756, 19356, 8292,  58664, 22496, 31604, 14536,
                7852,  48144,
            },
            {
                40580, 61644, 51036, 46312, 3644,  17548, 65520, 26860, 44644,
                57856, 50804, 27700, 59900, 12496, 45320, 28908, 15416, 58648,
                23636, 30660, 65524, 20848, 14188, 50180, 43944, 59100, 29564,
                2352,  43064, 6388,  21168, 60828, 44072, 50552, 4180,  46340,
                27296, 32648, 39092, 23272, 11832, 30148, 64296, 9548,  54076,
                60556, 35776, 7764,  51548, 1588,  34608, 14232, 61268, 43372,
                10588, 4892,  17884, 37228, 2396,  20348, 63348, 27960, 6876,
                24616, 13740, 2836,  27432, 37860, 22440, 40712, 47996, 28116,
                20060, 13288, 26324, 32228, 23268, 10932, 44788, 34468, 61688,
                208,   56456, 45380, 26360, 36124, 30288, 964,   37584, 15728,
                52036, 12864, 36692, 396,   41576, 15580, 33216, 42668, 27528,
                36944, 30280, 41496, 55828, 34412, 51528, 47364, 27788, 61828,
                4052,  26004, 56884, 62684, 36956, 41948, 55296, 34768, 52292,
                59688, 1284,  35944, 50416, 64356, 39760, 17536, 51280, 26844,
                64944, 35840,
            },
            {
                24492, 10888, 7272,  37780, 60096, 34888, 13872, 49708, 40668,
                19668, 14828, 37948, 43480, 384,   54092, 6628,  40372, 2256,
                55204, 12120, 35556, 3424,  56552, 32628, 4208,  54288, 38464,
                50760, 11352, 55724, 47724, 14380, 31104, 34576, 16352, 58120,
                11040, 62260, 6448,  15864, 52852, 2536,  34244, 22040, 14252,
                42704, 26364, 16400, 40072, 63836, 18676, 5356,  30980, 52876,
                23904, 64828, 33852, 11680, 25860, 50452, 32768, 59972, 43504,
                56240, 36320, 17136, 44304, 60416, 11172, 56844, 4492,  32640,
                50632, 6492,  46252, 55224, 49428, 39708, 29228, 17408, 54096,
                31516, 24108, 9924,  64692, 18800, 43440, 11472, 49060, 64004,
                29700, 59080, 46928, 20992, 30392, 51828, 8780,  47812, 4432,
                64668, 7588,  46096, 124,   18960, 63884, 15956, 2108,  49880,
                32452, 39044, 19772, 47296, 5596,  23140, 13772, 17876, 28708,
                44364, 12448, 16064, 23964, 46624, 11148, 55352, 4544,  42012,
                20812, 54544,
            },
            {
                32092, 45008, 28504, 19932, 56464, 30016, 5856,  53420, 1452,
                10468, 31816, 63232, 17724, 23512, 35440, 61588, 47508, 26764,
                37704, 17692, 46956, 28764, 61872, 24024, 17924, 12588, 26120,
                19868, 60088, 28128, 36624, 53564, 7532,  64340, 24440, 54104,
                41840, 18564, 49840, 58876, 36596, 45680, 56976, 48364, 300,
                31380, 62256, 4016,  47028, 23516, 27976, 49932, 38568, 19812,
                100,   48632, 28924, 59408, 55564, 38996, 9416,  1128,  19164,
                29548, 62484, 50208, 31312, 52628, 19368, 35364, 14332, 54944,
                41916, 63512, 37012, 9016,  15228, 1860,  65204, 7652,  47152,
                37700, 59592, 14392, 40492, 5608,  22864, 58736, 26852, 20028,
                2504,  42912, 8176,  54620, 37900, 3740,  62808, 18248, 23588,
                51016, 54308, 11816, 38772, 24796, 8904,  37540, 59132, 44648,
                55480, 8104,  51096, 29328, 2440,  61364, 58284, 46440, 3692,
                63812, 38844, 57472, 33208, 2280,  29272, 36716, 60056, 45600,
                12556, 3108,
            },
            {
                58340, 63608, 14964, 936,   48000, 42952, 22128, 38336, 58780,
                24800, 48112, 55240, 3908,  51184, 30848, 20540, 13940, 8232,
                63304, 51116, 6256,  39984, 10084, 42104, 48052, 64244, 8020,
                46176, 33556, 16740, 4688,  23092, 39748, 12052, 29176, 992,
                35780, 8744,  31072, 25208, 5244,  19300, 27572, 7888,  38584,
                50748, 11264, 56188, 36952, 8840,  58808, 42328, 12092, 56372,
                35380, 40940, 14384, 6768,  45612, 23232, 15664, 49096, 40588,
                11392, 3904,  22948, 9504,  5592,  25908, 43056, 61660, 23424,
                3312,  17028, 21176, 29880, 57508, 42860, 22292, 27164, 12768,
                3896,  20332, 34712, 55300, 51692, 47188, 33408, 7360,  53240,
                35284, 24448, 16512, 60636, 26136, 13720, 56220, 39528, 34792,
                14544, 31812, 21236, 61476, 56856, 42608, 29976, 20640, 11528,
                23984, 16992, 64748, 12764, 43048, 32260, 37784, 7568,  24872,
                31432, 21700, 53872, 6844,  61764, 25852, 15440, 8920,  33704,
                49304, 17680,
            },
            {
                38308, 51624, 41476, 24048, 11996, 61108, 9052,  16852, 64064,
                29412, 36268, 7932,  40812, 46248, 10040, 58228, 53004, 43288,
                31684, 21260, 60352, 15948, 53252, 252,   36844, 29288, 56912,
                40112, 1228,  63284, 42688, 58468, 19608, 44976, 60548, 48392,
                21124, 56236, 44164, 64984, 12828, 41464, 61788, 15652, 58836,
                24552, 44000, 20728, 29820, 53732, 15832, 2748,  63368, 25992,
                8392,  61648, 21672, 42592, 3496,  64284, 36064, 58676, 54032,
                34044, 46300, 57884, 38428, 65272, 47520, 1792,  30092, 9864,
                39524, 53008, 45140, 61244, 4184,  51008, 35212, 59108, 53124,
                43764, 49208, 8316,  28600, 2720,  16116, 62128, 13532, 39472,
                46032, 63248, 11112, 33612, 49032, 44120, 29196, 10116, 58004,
                43496, 2948,  48612, 27880, 4168,  13848, 53376, 5504,  36028,
                41228, 1068,  34724, 25316, 53064, 20784, 49692, 11516, 41032,
                51724, 9496,  47504, 18680, 41428, 44540, 52784, 23112, 63128,
                29488, 5464,
            },
            {
                21200, 9320,  35056, 54720, 31632, 50520, 26180, 33492, 44248,
                4508,  13552, 22480, 62008, 16120, 25964, 39140, 3160,  34744,
                24360, 1732,  45080, 26228, 33348, 59224, 22316, 6116,  15008,
                52384, 24584, 10212, 30564, 49076, 2796,  52496, 6128,  38232,
                13972, 27756, 4360,  38672, 53712, 21572, 30676, 52052, 34740,
                18208, 5936,  65048, 13108, 33396, 44844, 22760, 31792, 46880,
                16956, 51224, 32464, 53540, 18448, 31004, 27400, 7508,  21264,
                26032, 16704, 42164, 13656, 32912, 53632, 16180, 58732, 49624,
                27308, 34324, 12072, 24480, 32900, 19172, 10324, 39176, 16708,
                30580, 63924, 24880, 60296, 38548, 31352, 42288, 54960, 4804,
                18412, 30892, 51028, 6124,  19676, 1616,  22456, 47292, 6028,
                25856, 60152, 16244, 35936, 45156, 32292, 49532, 62660, 27408,
                48184, 60064, 45648, 57096, 6852,  15896, 27968, 55356, 60348,
                15120, 3048,  35372, 30036, 11876, 58496, 19948, 468,   40232,
                56680, 26204,
            },
            {
                44204, 2696,  16660, 62540, 5648,  39408, 2388,  47008, 55584,
                20000, 51936, 57452, 28336, 33740, 4932,  64940, 19156, 48448,
                11128, 57604, 38256, 13028, 49132, 18268, 44260, 61632, 35416,
                20688, 45148, 55192, 14056, 26416, 35096, 17116, 32088, 23748,
                62864, 51312, 17040, 32960, 48164, 1728,  10068, 45068, 3928,
                28928, 41144, 47868, 1320,  51780, 61136, 6508,  36712, 54816,
                3272,  27652, 37544, 10312, 56956, 46808, 12856, 43736, 4396,
                60812, 51588, 560,   28276, 20756, 6968,  36088, 44384, 19720,
                64232, 408,   55528, 7608,  41272, 48484, 63244, 5564,  22940,
                1072,  41492, 11968, 18240, 45968, 21204, 9884,  23776, 28064,
                57224, 2928,  41764, 58388, 38972, 61928, 53856, 32576, 64164,
                19416, 39868, 52084, 7976,  65416, 18216, 22596, 9472,  56288,
                13616, 18912, 30296, 10104, 39336, 63928, 12,    33300, 19396,
                43428, 27268, 56140, 65060, 4340,  49896, 37908, 7376,  47236,
                13912, 53436,
            },
            {
                65156, 30428, 46148, 27404, 58720, 19580, 14924, 61888, 11216,
                37684, 41820, 804,   47780, 12060, 43996, 55636, 29752, 15712,
                62644, 53604, 28656, 7936,  56256, 3380,  30532, 11324, 50008,
                4264,  32564, 61064, 38080, 8424,  65320, 56480, 41124, 9440,
                45804, 2676,  58644, 11420, 23840, 56564, 37148, 63872, 13888,
                54604, 59484, 35612, 26932, 17860, 39428, 10692, 19380, 59096,
                13584, 43872, 5604,  60616, 25052, 924,   50808, 63028, 37416,
                31192, 8912,  48388, 62708, 55644, 40756, 24788, 12296, 5156,
                38324, 15612, 46724, 28364, 58376, 14136, 26184, 54672, 46192,
                57180, 33520, 50904, 6340,  55856, 1372,  49320, 65220, 36232,
                47688, 15176, 22980, 27508, 35060, 8240,  16692, 13052, 36800,
                732,   30200, 11192, 23856, 55092, 1468,  39272, 44144, 4504,
                33304, 50864, 3164,  22348, 35324, 44316, 47636, 6188,  37032,
                62820, 50708, 39536, 23592, 14776, 34020, 26832, 31648, 61384,
                10288, 35704,
            },
            {
                6336,  49732, 22096, 12784, 41796, 35668, 51660, 28916, 6616,
                24408, 31292, 17836, 60508, 35872, 20772, 50668, 8812,  41060,
                36344, 4212,  18852, 42484, 65100, 24316, 39440, 55380, 27236,
                41672, 17844, 1944,  50868, 22528, 43524, 12632, 28664, 59512,
                20176, 34836, 41928, 29476, 61604, 7476,  42544, 25576, 49452,
                22020, 10912, 7396,  23316, 57520, 49756, 29524, 41488, 24200,
                50080, 65436, 21164, 40692, 16752, 33496, 39444, 19692, 15072,
                54940, 23344, 35100, 17712, 10548, 45936, 60552, 32152, 57420,
                51944, 22108, 61988, 35760, 18004, 2128,  31628, 37024, 9308,
                27896, 14632, 62024, 36420, 26024, 59444, 33748, 12920, 43432,
                8672,  61652, 54388, 11732, 45792, 51288, 24288, 42636, 49632,
                55488, 45620, 58684, 41708, 33936, 26872, 60460, 16548, 37296,
                24740, 63204, 40768, 54268, 60776, 17988, 25684, 58804, 22616,
                12576, 1504,  17684, 7896,  46512, 54504, 59768, 16504, 42548,
                24916, 18660,
            },
            {
                39692, 33176, 53716, 192,   56608, 8464,  22796, 43116, 58272,
                49608, 64580, 9348,  53476, 5948,  24236, 59560, 1232,  26784,
                46668, 23032, 50260, 31656, 35228, 15576, 46732, 8624,  62880,
                13168, 58356, 28456, 46584, 19152, 5064,  54036, 80,    48688,
                26060, 6820,  55064, 18764, 46496, 15472, 33724, 19232, 756,
                32256, 39196, 62436, 43808, 14520, 4240,  63472, 46348, 632,
                33644, 29904, 11860, 48276, 52780, 8332,  58216, 29332, 10848,
                45128, 41336, 58972, 2708,  30604, 22400, 3684,  48256, 26652,
                8148,  30440, 42812, 4448,  53772, 44612, 59968, 20600, 51916,
                40200, 4112,  21804, 44556, 29988, 15512, 51752, 3940,  20140,
                31448, 37980, 76,    18528, 63612, 29876, 3720,  60936, 27080,
                7552,  21868, 14976, 4788,  50168, 12656, 47752, 53560, 29468,
                57456, 8476,  15308, 28320, 11600, 3840,  52540, 30552, 9376,
                48796, 32724, 26140, 58156, 41016, 10796, 1932,  21964, 51712,
                3676,  56216,
            },
            {
                14484, 61008, 10488, 25624, 47248, 64136, 32496, 17100, 1920,
                35092, 13708, 26884, 45628, 38768, 30616, 42416, 13172, 33512,
                61472, 6604,  58624, 11728, 1160,  52584, 20932, 4880,  37112,
                23448, 53316, 7636,  34732, 62324, 31748, 39352, 16220, 36280,
                63716, 13904, 50560, 39880, 4664,  27444, 52424, 61124, 57080,
                46992, 16656, 53172, 31048, 37604, 9032,  26496, 54136, 15880,
                8076,  57352, 36132, 2428,  27812, 62028, 22988, 47420, 2112,
                64880, 6544,  25924, 38508, 52348, 15200, 64552, 36532, 17816,
                40572, 13444, 50620, 10520, 24152, 39036, 12620, 6892,  64888,
                17776, 57908, 48824, 10392, 41860, 7744,  38900, 25028, 60276,
                53088, 26524, 49412, 33436, 6608,  39392, 56748, 10596, 17892,
                32924, 63736, 37928, 28992, 62220, 20704, 5992,  10564, 41900,
                668,   20460, 45160, 49268, 32928, 42724, 14600, 56876, 40852,
                45220, 60684, 53208, 35956, 19436, 28776, 44616, 64088, 36476,
                48080, 28092,
            },
            {
                20796, 43676, 37512, 17904, 29788, 38988, 4764,  54512, 45208,
                21100, 40652, 56484, 2780,  15452, 63508, 18252, 55276, 52028,
                20628, 15324, 38984, 27120, 62424, 43564, 58052, 33428, 49036,
                15840, 43804, 38884, 11564, 25492, 57348, 51260, 23076, 44080,
                10356, 31456, 21988, 1168,  64696, 36752, 12484, 9168,  40596,
                6236,  28004, 2704,  48636, 20252, 56428, 34524, 21740, 38920,
                62272, 18896, 45988, 55496, 13732, 42732, 5460,  35996, 53468,
                21116, 33400, 12688, 47136, 57588, 42428, 9324,  53760, 1472,
                56420, 63620, 33768, 19764, 61100, 48900, 29184, 34900, 47172,
                25048, 31996, 460,   54540, 18716, 63892, 56560, 46868, 16848,
                5560,  40880, 21956, 59184, 15064, 44436, 20776, 35300, 41536,
                47128, 2920,  51332, 17592, 44432, 36568, 31436, 23684, 64260,
                50336, 35532, 59748, 5264,  23568, 65292, 37564, 7100,  20308,
                28020, 5700,  13976, 3336,  61804, 50564, 6752,  13192, 30680,
                9084,  59284,
            },
            {
                52320, 5104,  63008, 7264,  50468, 13984, 60228, 10960, 27964,
                62764, 7536,  32972, 23144, 49348, 10428, 46236, 3700,  37144,
                9820,  44696, 55764, 48628, 17992, 9172,  29684, 25620, 64264,
                528,   30340, 60788, 48280, 3324,  14228, 8220,  29056, 60632,
                53404, 38648, 57544, 47808, 24160, 55768, 43984, 30072, 21476,
                35112, 63816, 24472, 11804, 60396, 44740, 2052,  12480, 52212,
                43224, 4772,  25792, 21904, 38312, 31892, 17812, 57012, 27088,
                15688, 50276, 61048, 19116, 4724,  28040, 34128, 20528, 44904,
                24508, 5272,  46840, 27736, 748,   55364, 16600, 4712,  43396,
                11164, 38168, 62528, 27628, 34440, 22228, 2856,  28856, 43976,
                9932,  64304, 12804, 46984, 54804, 25564, 1564,  52060, 59868,
                13428, 24920, 54468, 9800,  1768,  56304, 59432, 46136, 17792,
                12948, 25916, 38908, 55896, 18920, 1788,  50844, 34228, 16820,
                63580, 38496, 22448, 43472, 31904, 23992, 39024, 56572, 17960,
                41892, 1512,
            },
            {
                32064, 23972, 35000, 55172, 44744, 20464, 24896, 52548, 37456,
                47628, 18540, 59492, 52272, 36396, 27300, 58428, 31960, 24980,
                64656, 29032, 5148,  23724, 36392, 2632,  41012, 11992, 19024,
                54628, 6352,  21608, 17452, 42280, 64624, 45772, 20096, 2308,
                5752,  17756, 27712, 8560,  32832, 16128, 5176,  51464, 59936,
                14420, 42508, 55104, 7092,  32156, 17108, 64364, 47916, 28056,
                31568, 10584, 61032, 51732, 3712,  64116, 49540, 11984, 39752,
                43380, 29464, 8616,  36108, 24028, 50772, 12892, 62212, 30140,
                38544, 15964, 58212, 36720, 8432,  41308, 62992, 22124, 59064,
                50404, 16080, 6648,  45560, 12384, 49876, 37212, 13704, 32272,
                55248, 35748, 2660,  30832, 8420,  37252, 64752, 28728, 5132,
                31300, 57844, 40392, 34004, 26404, 14328, 39312, 3816,  33820,
                54244, 6480,  30960, 10456, 47544, 28636, 59960, 24632, 54180,
                836,   49480, 57516, 9244,  47612, 15432, 52796, 34000, 62648,
                26188, 46648,
            },
            {
                16008, 58144, 12640, 27772, 1624,  33560, 41524, 3920,  15668,
                31068, 488,   12560, 41752, 4588,  19996, 7428,  40908, 50000,
                52,    42256, 31588, 13396, 61028, 51228, 57052, 46964, 37588,
                59832, 34352, 51760, 27984, 55236, 36048, 32188, 49928, 40960,
                34156, 59320, 12936, 43080, 60524, 19588, 48496, 25948, 3248,
                46128, 18620, 38020, 50912, 27084, 41200, 36004, 23716, 6288,
                58740, 40272, 33804, 16072, 44860, 9160,  25008, 59480, 1516,
                5928,  62700, 54736, 45296, 296,   59468, 41424, 6816,  48104,
                9996,  51352, 22024, 31552, 12984, 52188, 26224, 33288, 1780,
                29688, 56436, 23708, 40704, 53548, 6528,  61636, 57776, 19640,
                23504, 50096, 27568, 41800, 19084, 48864, 11572, 16872, 45132,
                22668, 7484,  19560, 48068, 65068, 22008, 51104, 9140,  28044,
                61504, 44024, 16584, 62468, 40292, 13692, 43588, 9560,  46376,
                12444, 29672, 34800, 18296, 64948, 27428, 340,   6424,  22224,
                11212, 38316,
            },
            {
                3464,  48404, 19392, 39776, 65340, 57132, 48664, 9060,  61716,
                55532, 25464, 44496, 65000, 29796, 54220, 62328, 13664, 22092,
                17252, 60016, 53660, 45596, 20284, 33920, 15260, 22956, 7520,
                44596, 10836, 41076, 2124,  23836, 5972,  11220, 15860, 62796,
                24524, 46872, 51604, 35880, 2028,  40188, 63192, 37120, 9936,
                58084, 33244, 172,   22356, 59288, 9872,  3360,  56092, 14076,
                18560, 50020, 1656,  56816, 29972, 20180, 35020, 46068, 52684,
                23124, 32624, 11364, 39424, 17560, 31308, 55664, 15368, 26556,
                60004, 3044,  42312, 65092, 45900, 18584, 38616, 9500,  54272,
                13568, 36464, 60452, 17660, 31368, 26440, 1248,  48064, 40044,
                4100,  62352, 15988, 56440, 60624, 23164, 33392, 53892, 39260,
                50104, 61808, 11784, 43032, 6512,  30568, 55148, 19220, 41216,
                48516, 1336,  21760, 51884, 34752, 3032,  21072, 31624, 58276,
                41376, 25760, 53052, 5924,  40424, 11936, 58952, 42560, 49684,
                54652, 61172,
            },
            {
                30272, 43264, 51948, 7820,  11328, 16912, 30060, 22148, 34748,
                50816, 38684, 9448,  22244, 34332, 16644, 44852, 38592, 57368,
                34944, 11748, 7496,  38276, 4368,  63804, 28588, 3604,  50660,
                26408, 14784, 31276, 61956, 58456, 38716, 53076, 26928, 56248,
                9424,  20792, 4164,  29180, 55008, 11480, 22288, 31464, 53340,
                27692, 13120, 65064, 43712, 15220, 49124, 53204, 30696, 61884,
                37360, 46572, 24296, 12412, 41260, 61568, 7008,  28328, 16712,
                38444, 19956, 48120, 25608, 65504, 5420,  22412, 37448, 33716,
                54256, 19404, 25664, 6232,  56144, 2524,  48656, 61704, 41648,
                21056, 47700, 3240,  9668,  65508, 42776, 15444, 33992, 9336,
                53412, 44716, 6172,  38396, 784,   43384, 7240,  62816, 2060,
                14800, 36068, 27924, 58988, 37776, 316,   45044, 60356, 14760,
                24092, 37508, 57944, 7972,  25332, 55404, 64188, 37988, 7380,
                15764, 62840, 3188,  20816, 55688, 36632, 45984, 29576, 19788,
                35244, 8372,
            },
            {
                26560, 62288, 22792, 36436, 45932, 25936, 59028, 43180, 6308,
                19180, 46552, 14912, 57816, 1612,  48396, 10620, 4828,  28216,
                51496, 46744, 23212, 27252, 55056, 10348, 42760, 31948, 54080,
                65216, 20516, 48968, 8380,  16736, 21088, 47412, 768,   30840,
                44456, 39372, 14860, 65500, 25436, 45052, 6672,  15760, 41364,
                49416, 7840,  54296, 29756, 4496,  19200, 38580, 44528, 21348,
                8008,  27576, 65124, 6080,  52476, 37624, 48764, 13472, 63652,
                55968, 3080,  58288, 14504, 52408, 43172, 49584, 1908,  63352,
                12428, 44344, 39208, 15104, 34764, 29352, 23604, 15884, 7132,
                44320, 27092, 33684, 51956, 20172, 46564, 23012, 58944, 29088,
                18200, 25432, 34584, 13360, 52480, 27272, 57616, 18788, 25764,
                31856, 55864, 3656,  17440, 49224, 13368, 25800, 35720, 7088,
                32076, 64800, 12100, 30048, 45796, 50236, 18132, 4044,  48608,
                22952, 36192, 43936, 50600, 32020, 23720, 16524, 63752, 4924,
                13956, 40656,
            },
            {
                17668, 1828,  55704, 5804,  31912, 53460, 1224,  13332, 63604,
                28524, 2996,  60660, 26592, 37548, 53180, 23848, 60836, 32800,
                2240,  18688, 61948, 41388, 16248, 48924, 59092, 18384, 39368,
                1404,  35972, 56780, 45956, 34592, 42412, 64320, 13700, 36444,
                5440,  61128, 53268, 32376, 18032, 49812, 56588, 61796, 2716,
                20396, 36152, 24496, 40000, 61524, 33056, 25956, 11592, 1032,
                54916, 16288, 34660, 43660, 22396, 17572, 456,   31400, 42272,
                9304,  34036, 27368, 7436,  35320, 10788, 19756, 28112, 46480,
                7392,  31220, 52248, 62064, 10192, 59596, 37192, 51616, 31644,
                64020, 5396,  59164, 39120, 12028, 55952, 6592,  38016, 61328,
                11612, 48528, 64968, 20720, 30984, 10036, 36612, 47452, 41660,
                9588,  44656, 51416, 23624, 33012, 63476, 9704,  57116, 50040,
                2844,  53544, 19504, 42148, 5152,  14592, 33456, 28008, 60704,
                54948, 30448, 10388, 14404, 61248, 8100,  2172,  51796, 25164,
                47016, 59452,
            },
            {
                50448, 33772, 15020, 63784, 39052, 20216, 49392, 35652, 40388,
                56132, 32752, 52072, 7836,  41872, 31392, 19476, 55440, 42516,
                9632,  37116, 56728, 31160, 776,   24884, 35032, 7848,  13260,
                43536, 24972, 5032,  12504, 28160, 3804,  24352, 50160, 57748,
                19192, 23376, 47972, 8248,  868,   38540, 33788, 23388, 44004,
                58868, 16984, 47892, 10704, 45568, 6860,  57848, 63948, 41664,
                51176, 29340, 58832, 3096,  55744, 26668, 59336, 53624, 24156,
                50484, 45092, 62316, 41352, 29884, 56308, 60888, 38736, 16544,
                58188, 22728, 1000,  27504, 47212, 20576, 3732,  57932, 11896,
                18284, 49504, 24476, 196,   30136, 35168, 50512, 2032,  42500,
                32024, 7524,  40440, 58400, 45524, 15424, 50948, 4248,  60448,
                21192, 64456, 7312,  39968, 54028, 20228, 41772, 29216, 17716,
                39056, 46908, 27320, 36376, 61388, 57560, 39624, 11756, 45024,
                19228, 1140,  57704, 47200, 27672, 41824, 58032, 38176, 32400,
                56380, 9752,
            },
            {
                20868, 41956, 28212, 47516, 12288, 4216,  60324, 24344, 16568,
                10660, 22896, 43836, 17696, 12308, 63856, 6464,  15068, 25680,
                65308, 48200, 13960, 6376,  52364, 44764, 21768, 60976, 55232,
                29200, 62536, 51048, 19428, 59648, 54580, 32764, 7920,  41356,
                28672, 11304, 35408, 42376, 57608, 13572, 28224, 10336, 5660,
                30416, 62804, 1176,  56624, 21576, 50244, 17084, 35548, 23160,
                47484, 9432,  39376, 14500, 32500, 46352, 8564,  36144, 4188,
                12316, 21400, 16176, 1580,  47168, 23672, 3688,  52688, 9672,
                35196, 49464, 17652, 54900, 41564, 13784, 44996, 25808, 39780,
                35504, 54204, 14284, 43920, 62752, 17304, 26812, 13828, 21556,
                56892, 51868, 2768,  23900, 5188,  63468, 32836, 24516, 13012,
                29924, 35376, 15800, 27580, 2212,  45824, 5472,  62196, 23152,
                10852, 56196, 15592, 8480,  24280, 8,     21780, 53280, 7724,
                41512, 25460, 38752, 5556,  34716, 19000, 10952, 44184, 21836,
                2972,  37008,
            },
            {
                45116, 60896, 8260,  23380, 54460, 29496, 44104, 7432,  51360,
                46940, 5332,  62780, 35848, 49572, 28624, 46072, 39608, 50712,
                3636,  21320, 27808, 33956, 63164, 11240, 38220, 2764,  47112,
                16608, 9152,  40180, 31056, 44220, 10236, 39116, 17676, 61860,
                51772, 3168,  59908, 26380, 20964, 64052, 51608, 47184, 55280,
                39620, 33988, 27048, 14428, 37128, 28996, 2432,  13376, 31976,
                5352,  62060, 20588, 49988, 11176, 61144, 40348, 18988, 64224,
                30624, 39096, 60028, 51456, 18608, 13344, 33044, 43704, 25256,
                64132, 40108, 4976,  32680, 8312,  64728, 30056, 55860, 1528,
                8776,  28696, 20880, 57168, 7332,  40500, 53920, 64268, 44960,
                27664, 15648, 36976, 29484, 54892, 18120, 39604, 56996, 520,
                52884, 47968, 58864, 11308, 61152, 36564, 14796, 52416, 33336,
                43020, 4500,  63628, 50696, 43740, 31224, 48904, 63444, 34908,
                15140, 52100, 65448, 22600, 49500, 62560, 53736, 15232, 28872,
                64852, 13076,
            },
            {
                30948, 51988, 980,   58792, 41028, 18364, 33380, 64876, 26868,
                38208, 30456, 57076, 3744,  21756, 54116, 636,   59148, 34916,
                12076, 44016, 59520, 40032, 19176, 56136, 26720, 50576, 33328,
                20980, 36628, 53668, 116,   15412, 63688, 22388, 1880,  44820,
                14460, 31956, 54768, 16324, 45480, 3952,  35796, 18444, 25040,
                12108, 52360, 7544,  42512, 61208, 54588, 39844, 52444, 59904,
                17784, 36728, 25648, 44848, 4860,  22100, 28604, 15344, 48464,
                55388, 25548, 6588,  36528, 57184, 40612, 62568, 5820,  29620,
                14176, 45212, 56840, 24964, 38012, 50684, 5744,  19304, 48780,
                62264, 41148, 46208, 33068, 10752, 23728, 47980, 5496,  33776,
                9724,  19856, 59564, 42752, 11724, 49188, 8572,  44280, 37352,
                19388, 5612,  41984, 22332, 31256, 55624, 25944, 48672, 1136,
                59420, 30112, 19912, 34188, 54724, 10628, 17628, 27140, 2656,
                58492, 32608, 11560, 17016, 30976, 1620,  25672, 6676,  47896,
                54416, 5304,
            },
            {
                18468, 38364, 26940, 15628, 36184, 9700,  57244, 13868, 112,
                53640, 19784, 15024, 41160, 26232, 9900,  32364, 16988, 23292,
                29728, 51744, 7964,  15944, 47480, 5136,  30344, 14664, 64644,
                5768,  24416, 60608, 47788, 27020, 35656, 49364, 56908, 25408,
                37276, 47448, 6276,  39640, 30708, 9088,  61056, 41596, 1708,
                16032, 64384, 49652, 18692, 4744,  24072, 10024, 46264, 27284,
                43092, 692,   52932, 64600, 33944, 56544, 51564, 7252,  42152,
                2376,  45604, 11344, 31780, 27028, 8620,  48148, 21208, 59020,
                2096,  19900, 11424, 60084, 21752, 15980, 34640, 43600, 23584,
                58280, 16476, 2548,  52436, 60864, 36372, 19020, 2936,  38728,
                50288, 61820, 46244, 1300,  35172, 21980, 27844, 59268, 14292,
                26272, 56160, 33608, 46680, 18192, 8152,  40544, 12320, 21220,
                37812, 46356, 26108, 13516, 40124, 6052,  60024, 38036, 47584,
                20752, 43348, 3472,  55416, 45644, 39892, 36164, 60120, 41244,
                34180, 24764,
            },
            {
                57280, 11464, 48552, 62144, 3132,  45552, 49912, 21992, 42472,
                61836, 11636, 33484, 59360, 45120, 62400, 38412, 55940, 5668,
                63044, 36592, 54732, 2252,  23828, 42904, 58240, 9644,  41700,
                46060, 57068, 12732, 7736,  20084, 52828, 5328,  30152, 10720,
                64980, 19512, 23768, 62620, 12716, 49152, 22068, 32212, 56768,
                44276, 36872, 22716, 31200, 44724, 34356, 65212, 19852, 7112,
                57600, 12016, 30792, 15656, 41304, 1704,  37388, 62604, 23392,
                33828, 58936, 20016, 65028, 164,   54560, 16652, 37868, 53676,
                50440, 31264, 35912, 47884, 388,   53888, 63024, 9920,  31488,
                12820, 37220, 26356, 6092,  29732, 13276, 57820, 25304, 55524,
                30088, 7016,  13524, 25804, 62872, 52608, 6112,  31592, 65404,
                50492, 10364, 1928,  63432, 51196, 3440,  58092, 28748, 65172,
                16800, 7108,  57144, 2072,  62876, 22236, 45832, 13008, 29572,
                8888,  51120, 26312, 61200, 8296,  13660, 51492, 20300, 10100,
                16020, 62856,
            },
            {
                4144,  42168, 33000, 19636, 52792, 30844, 25596, 4468,  35820,
                28404, 48772, 2140,  51436, 7248,  20128, 13348, 43096, 47832,
                19400, 10864, 26276, 31784, 64080, 36008, 52172, 18520, 37764,
                3232,  28464, 32044, 42488, 38472, 59168, 13980, 34196, 42016,
                7648,  53628, 412,   43464, 56360, 27500, 52776, 7400,  59928,
                29108, 3756,  9372,  59260, 12900, 56408, 3544,  30432, 49900,
                38740, 23456, 55456, 8492,  19280, 26352, 13084, 29712, 9576,
                53536, 15016, 38652, 49752, 22876, 44884, 33728, 10684, 25832,
                41000, 7548,  62188, 27656, 42072, 13232, 26000, 38992, 52140,
                4816,  47392, 65016, 21828, 49660, 42160, 45576, 62656, 16164,
                41208, 23016, 54276, 32112, 39792, 16648, 46804, 2628,  42456,
                23052, 40036, 29164, 15496, 37612, 24404, 43668, 35156, 53836,
                9844,  41804, 48380, 35680, 52180, 25320, 32344, 4884,  55808,
                64000, 39404, 35540, 18436, 28804, 57580, 23564, 31680, 244,
                50320, 45836,
            },
            {
                21536, 8068,  55844, 23612, 6440,  38656, 63108, 17300, 54564,
                8640,  39496, 24692, 16688, 36812, 30728, 52756, 25484, 1520,
                40532, 61076, 45284, 50144, 20732, 12964, 960,   26088, 61976,
                22568, 50752, 17092, 63292, 3932,  24128, 46196, 18076, 60316,
                48748, 28584, 32980, 36252, 17708, 3208,  38376, 14264, 19836,
                46752, 26028, 53588, 39960, 47960, 16764, 25572, 41940, 14768,
                63320, 34464, 48004, 43512, 58452, 49168, 60560, 44356, 18644,
                47604, 4272,  28520, 41868, 12852, 61288, 29240, 3608,  64392,
                15332, 46360, 18288, 4732,  56276, 30388, 45764, 2700,  60736,
                18640, 33488, 54712, 39308, 17484, 7952,  32276, 10412, 500,
                35808, 47796, 18700, 3976,  57556, 10144, 20892, 60952, 35464,
                17416, 7260,  58392, 45252, 11580, 61520, 18924, 4564,  50408,
                23132, 62076, 30936, 19060, 15468, 44176, 58976, 49688, 17828,
                23780, 13860, 4276,  48920, 42316, 5640,  64556, 43888, 55200,
                27452, 35448,
            },
            {
                13916, 29080, 65032, 44028, 14684, 57868, 11272, 46536, 60076,
                21092, 43956, 56828, 61356, 47040, 2976,  64240, 9236,  56708,
                33344, 14276, 4604,  37888, 8736,  59620, 33696, 48240, 54600,
                10892, 35428, 55868, 44796, 9808,  29456, 54852, 2188,  38944,
                22192, 14940, 58544, 10840, 61392, 25072, 47576, 34056, 40920,
                11500, 61324, 35200, 21132, 448,   51636, 37452, 61156, 10640,
                20900, 5548,  28536, 2848,  22248, 38560, 5864,  32280, 56008,
                36836, 63340, 8056,  52820, 18040, 5288,  55932, 42832, 35176,
                23040, 58752, 10452, 38404, 51092, 20320, 8512,  57204, 22752,
                28288, 43068, 11064, 1608,  59684, 53212, 27440, 21612, 51084,
                59312, 11532, 64832, 43276, 49940, 36804, 29516, 54824, 12088,
                49024, 53012, 25824, 34572, 55088, 30492, 47716, 14320, 32732,
                424,   26540, 11484, 55520, 3224,  9224,  39184, 35124, 552,
                41848, 59348, 33032, 21516, 53196, 37260, 12172, 17788, 8696,
                39248, 60860,
            },
            {
                52280, 47596, 37136, 1412,  27592, 41092, 33700, 6800,  29704,
                14248, 32196, 4948,  11152, 27684, 21908, 35480, 49432, 29320,
                22852, 58600, 17364, 28132, 55752, 44680, 16136, 30100, 6108,
                41044, 14748, 1348,  21704, 33880, 50252, 62524, 12024, 26528,
                52428, 4372,  44660, 50156, 40396, 5796,  54328, 65072, 1120,
                50860, 17204, 6532,  64076, 27992, 8632,  32740, 54448, 1984,
                46588, 52184, 16860, 62404, 35128, 11604, 50992, 16460, 816,
                21716, 25400, 32784, 57824, 35960, 24840, 47692, 19516, 51592,
                1264,  54692, 32888, 24460, 65360, 35292, 16968, 40980, 49904,
                6792,  62636, 15568, 25196, 35436, 4152,  63764, 37904, 43716,
                6060,  29140, 34368, 26620, 8004,  14904, 45740, 25148, 5204,
                32140, 64036, 20088, 1312,  6196,  22268, 8448,  40008, 59324,
                45076, 38024, 60420, 41180, 28348, 64712, 20452, 11824, 27012,
                52336, 45312, 9540,  63104, 1672,  26196, 47988, 33712, 58484,
                24244, 2740,
            },
            {
                30872, 17176, 10592, 54408, 50056, 18668, 22604, 52404, 984,
                49596, 65376, 19028, 38724, 53312, 42788, 14668, 18056, 39564,
                6668,  46408, 51860, 42284, 23608, 4108,  39152, 60676, 19976,
                45712, 65380, 27612, 59536, 40416, 18832, 6152,  42716, 36700,
                64412, 9184,  20268, 30860, 22932, 15528, 28668, 9788,  21832,
                57536, 30212, 43344, 13620, 46404, 58796, 18376, 44116, 24088,
                36480, 59952, 31612, 8880,  54832, 24976, 64308, 28168, 42260,
                60260, 13604, 45660, 2040,  11072, 39580, 59344, 14148, 6280,
                28384, 40144, 44772, 14548, 1804,  48580, 59532, 31968, 13624,
                37056, 45224, 55740, 31076, 41268, 48048, 19088, 14424, 55320,
                24636, 16904, 53828, 1800,  22264, 63440, 58500, 136,   40892,
                44684, 14016, 38612, 60428, 42632, 52004, 64396, 27240, 56340,
                16320, 6704,  21552, 47076, 5692,  50648, 31132, 54612, 62192,
                6576,  29848, 16188, 56296, 31396, 14764, 61816, 4824,  49888,
                19540, 41692,
            },
            {
                6056,  58776, 25328, 34824, 4260,  63900, 45016, 59032, 36468,
                40504, 23988, 55076, 33812, 7908,  58436, 4548,  60900, 53596,
                12032, 65152, 48,    31112, 62580, 12192, 52808, 25180, 9024,
                32712, 51984, 5168,  48288, 12944, 25736, 57696, 31652, 15856,
                47540, 34600, 55984, 1460,  62952, 42892, 58932, 32664, 45264,
                37692, 24136, 54984, 33460, 39000, 22736, 4324,  29792, 57528,
                13144, 26420, 42180, 45520, 14648, 40256, 3868,  46860, 10496,
                52196, 39012, 49340, 20700, 61912, 29676, 8472,  32520, 63056,
                49300, 20616, 7660,  60432, 27204, 11872, 4904,  23756, 53464,
                720,   20184, 8436,  50932, 22484, 11848, 57148, 33756, 2528,
                46608, 38840, 60748, 41552, 52012, 31512, 38184, 18600, 56088,
                22720, 8996,  28836, 48624, 16888, 32012, 36620, 11096, 20044,
                52896, 34568, 13480, 53972, 36696, 23640, 42900, 15088, 47196,
                19360, 37828, 24676, 40512, 46704, 20612, 38484, 28896, 44424,
                11368, 63568,
            },
            {
                37744, 45436, 20484, 61044, 12860, 29024, 7704,  15804, 26316,
                9952,  3560,  45972, 13776, 48456, 30868, 24868, 44676, 36636,
                26608, 33932, 20496, 8184,  36216, 18236, 47312, 2288,  57668,
                38292, 22856, 17368, 35920, 8156,  53368, 45248, 664,   23412,
                59624, 27868, 39684, 13392, 52192, 35968, 18880, 7136,  49408,
                15040, 5280,  1892,  48756, 10552, 63364, 15524, 50624, 7048,
                39304, 19140, 308,   52976, 21076, 58908, 33780, 18928, 56628,
                30400, 6952,  17144, 27100, 42212, 52588, 45328, 23580, 37412,
                11432, 57480, 31052, 52892, 37844, 42748, 46612, 28820, 62044,
                34304, 57904, 27564, 64372, 4972,  44452, 28800, 7624,  59744,
                30796, 12380, 9012,  19672, 47856, 12928, 6996,  27776, 50924,
                62020, 35732, 3512,  54644, 12564, 24744, 5532,  43900, 2292,
                47936, 25124, 63212, 32144, 17336, 1280,  58364, 8320,  34328,
                3984,  57888, 50500, 11552, 7556,  54200, 2416,  57376, 15816,
                53080, 32952,
            },
            {
                548,   56200, 8444,  42112, 48348, 38388, 32028, 56636, 43220,
                62812, 17804, 60132, 26916, 568,   63664, 19276, 10112, 2568,
                57256, 15972, 40252, 50396, 56524, 27588, 41780, 64008, 29208,
                11648, 55400, 43484, 30828, 61024, 38128, 20572, 10928, 54144,
                5004,  18332, 7728,  45468, 25724, 11124, 3276,  53792, 27008,
                60040, 42196, 62420, 19520, 25940, 53356, 41728, 35004, 61368,
                10272, 48940, 65432, 5724,  30232, 48596, 7876,  37956, 23548,
                2544,  35016, 65116, 56964, 5108,  15392, 452,   55608, 17468,
                43292, 2408,  15952, 22612, 63556, 18276, 56112, 9984,  40312,
                16300, 47256, 12364, 38352, 17864, 60844, 52712, 40052, 20972,
                50044, 63160, 24040, 35604, 3640,  44180, 60252, 34288, 11060,
                16372, 46916, 59120, 19716, 39676, 58056, 49956, 62348, 29368,
                41280, 9356,  3632,  56880, 45780, 11268, 61184, 25972, 41288,
                21872, 64576, 904,   35432, 61292, 42924, 23364, 34604, 7808,
                26744, 21892,
            },
            {
                13932, 50284, 27524, 16364, 23736, 52764, 1584,  11736, 21468,
                50280, 29560, 35640, 42068, 22276, 50656, 40556, 32220, 51512,
                47048, 23496, 61192, 44252, 14396, 4844,  22036, 34564, 15248,
                49560, 884,   62888, 4428,  14612, 28760, 48584, 63684, 33368,
                43608, 51348, 61204, 31820, 58160, 47964, 38832, 63972, 20996,
                12160, 35812, 29096, 56444, 7388,  31824, 3388,  47000, 27328,
                55640, 32772, 23184, 37172, 12468, 26336, 63000, 15280, 61012,
                44048, 54060, 12456, 32380, 37788, 22200, 63696, 34596, 27356,
                60148, 47356, 35552, 50360, 3304,  33468, 6896,  21312, 51540,
                3060,  24208, 42424, 32324, 1740,  36040, 10532, 25876, 15160,
                43280, 4848,  55004, 58180, 29720, 15380, 54004, 24196, 2456,
                42432, 30888, 25424, 7988,  45580, 740,   21628, 33508, 17596,
                60060, 51148, 27624, 40352, 20196, 29504, 38088, 49404, 55648,
                13096, 28560, 44580, 18340, 26864, 14160, 50800, 64760, 40600,
                46932, 61484,
            },
            {
                29948, 39560, 34092, 3400,  57444, 61624, 19252, 47624, 33660,
                54364, 5644,  8824,  58016, 11864, 37560, 56316, 15044, 62928,
                5192,  29396, 10524, 2184,  32388, 58564, 51628, 7212,  45176,
                19584, 26576, 39548, 51668, 23924, 57964, 3172,  40716, 25092,
                14028, 36584, 21240, 2460,  16420, 23128, 29912, 43784, 33980,
                8120,  51844, 16132, 45100, 37472, 59352, 18064, 13876, 21376,
                2304,  40832, 16408, 57488, 43388, 55020, 1408,  40720, 28944,
                20116, 47708, 8608,  24512, 50204, 46412, 10580, 51448, 40328,
                4608,  25416, 9740,  41676, 13092, 26512, 58960, 37468, 30632,
                63276, 5888,  55068, 59044, 50508, 23336, 47804, 64964, 344,
                37196, 33076, 17508, 26716, 40864, 64792, 20368, 49636, 39216,
                63420, 5116,  52840, 33832, 65168, 28080, 14572, 53616, 37528,
                12256, 22416, 35924, 6920,  65408, 52680, 2488,  16900, 31732,
                5380,  47800, 53520, 32884, 59016, 5216,  30816, 10600, 17772,
                4252,  54780,
            },
            {
                19344, 9832,  65240, 44416, 7024,  35904, 41348, 27824, 4048,
                38804, 15148, 45184, 52956, 17384, 3476,  28556, 8144,  21500,
                42136, 34876, 54980, 19492, 48116, 39380, 11092, 61868, 37108,
                56500, 32600, 10140, 46504, 18936, 34868, 7880,  17284, 56724,
                29324, 9300,  49012, 65496, 41832, 5400,  56632, 15304, 128,
                57784, 40296, 22664, 2596,  11140, 50120, 24652, 63828, 44624,
                53108, 28972, 62052, 8784,  34460, 18544, 50312, 10744, 52544,
                4380,  26680, 39636, 58784, 2712,  19160, 31420, 6660,  13636,
                20984, 64400, 56356, 29768, 61432, 46336, 53256, 14636, 44444,
                49104, 26980, 19696, 14004, 29764, 6780,  16944, 56292, 28228,
                53388, 46296, 7420,  48588, 10636, 1052,  36928, 8720,  28644,
                56640, 21952, 15000, 37840, 10544, 42100, 56592, 7604,  4072,
                44332, 57572, 16048, 48832, 13000, 24548, 44428, 9728,  62972,
                37096, 23872, 15552, 8960,  39832, 20936, 48220, 37672, 58116,
                24380, 43176,
            },
            {
                2168,  59264, 51928, 22564, 30864, 14720, 9364,  64144, 57108,
                23104, 61196, 31580, 25504, 64884, 33840, 47732, 44156, 26152,
                59988, 13668, 37716, 64772, 26952, 23816, 17308, 29696, 2728,
                22732, 60232, 6584,  42240, 64324, 12436, 51052, 44348, 62416,
                1644,  39420, 55080, 26820, 34540, 53148, 10004, 50112, 61944,
                25156, 31364, 47684, 64996, 28472, 41408, 33676, 8360,  37892,
                12268, 5096,  50728, 46120, 3372,  24232, 31360, 45692, 35568,
                59568, 13640, 17632, 42996, 62544, 27940, 55108, 59416, 43828,
                33688, 52400, 16620, 7600,  36308, 23404, 380,   18856, 11276,
                35028, 8644,  39956, 46112, 62760, 33968, 44236, 40772, 9604,
                20608, 13476, 62148, 22640, 52144, 32316, 45388, 59380, 18124,
                12432, 44196, 50792, 59948, 18704, 48880, 31424, 25364, 47436,
                63996, 30496, 40,    41952, 32984, 56468, 35396, 21096, 51808,
                42692, 57648, 2760,  63564, 43780, 55252, 1100,  27784, 51816,
                12960, 36052,
            },
            {
                31940, 25960, 12084, 17868, 46712, 55124, 25088, 43852, 12476,
                18224, 48284, 2884,  39928, 20916, 6408,  58768, 53608, 18136,
                1296,  51876, 8716,  44896, 5772,  59600, 54120, 42988, 48700,
                12956, 52788, 16564, 29432, 55024, 26016, 37020, 31348, 22000,
                12080, 46776, 18128, 7236,  13460, 38000, 20040, 27900, 46084,
                17888, 4576,  54340, 13736, 20360, 52568, 688,   56584, 30588,
                60792, 25560, 20400, 14256, 39488, 64328, 58040, 7036,  21996,
                63132, 32656, 54696, 5748,  35348, 15128, 38696, 24060, 48648,
                28732, 1688,  38924, 19800, 49360, 5268,  43024, 65176, 55632,
                22592, 60424, 53516, 4452,  11468, 21120, 58532, 3136,  31556,
                60308, 39256, 4280,  36016, 57384, 14036, 25404, 4820,  47848,
                35180, 1992,  27380, 6320,  23440, 3376,  61908, 40040, 20144,
                9028,  26684, 53788, 61560, 18408, 3660,  60340, 26800, 6144,
                12164, 19336, 29360, 34688, 25160, 12548, 61360, 33572, 7128,
                63776, 46040,
            },
            {
                6284,  49220, 40968, 62216, 5184,  37968, 50476, 1184,  34796,
                52368, 28824, 10568, 43184, 49984, 14244, 36520, 11056, 39804,
                32848, 48760, 22384, 15908, 31544, 580,   34784, 8940,  63528,
                25752, 38448, 34428, 3892,  20384, 184,   47712, 5880,  59372,
                52296, 33692, 24260, 60304, 45056, 63216, 32100, 3348,  42084,
                36588, 9760,  58688, 38852, 5664,  62152, 15420, 48236, 18588,
                42572, 35728, 58824, 32880, 48956, 28188, 12204, 16920, 38132,
                68,    48152, 29100, 51072, 11256, 45668, 968,   9080,  17984,
                60640, 11828, 45636, 55244, 30880, 59664, 27132, 32828, 40916,
                2816,  28832, 17524, 37952, 25184, 52580, 36504, 14448, 49128,
                24620, 55572, 29996, 42964, 18876, 63100, 40608, 30736, 54496,
                62480, 41444, 32060, 55468, 45364, 35624, 12796, 52760, 16472,
                34376, 39076, 11536, 23044, 46240, 8676,  39740, 48324, 31244,
                62040, 39008, 49368, 52800, 4488,  46760, 18752, 42244, 22460,
                15784, 54384,
            },
            {
                20232, 56516, 36640, 2692,  27584, 58172, 20132, 31440, 59612,
                6924,  37480, 62640, 56984, 744,   23544, 61096, 30236, 7080,
                63800, 56852, 28144, 41204, 52972, 47268, 14052, 40340, 18872,
                5408,  57252, 43928, 49856, 61460, 39468, 56068, 16216, 27616,
                41560, 4020,  57364, 29644, 1268,  50976, 23348, 58096, 12788,
                49256, 21852, 33188, 44148, 25356, 35412, 45428, 22960, 7020,
                51440, 1420,  10224, 54532, 22692, 5492,  41792, 55264, 44928,
                24572, 9716,  40332, 19452, 22868, 58168, 52696, 65424, 35928,
                5892,  41864, 24992, 62868, 14196, 10160, 38172, 16424, 7116,
                50556, 43968, 32200, 57196, 772,   47376, 27276, 63384, 6156,
                17352, 45280, 11292, 27144, 2896,  8032,  50884, 16636, 10808,
                24568, 20392, 8524,  64048, 17036, 28744, 57440, 1764,  50052,
                60196, 45408, 5392,  32036, 51432, 29040, 13628, 55132, 16684,
                532,   45300, 9836,  21676, 14696, 37444, 59104, 9640,  50436,
                39048, 29104,
            },
            {
                43584, 14604, 23796, 33384, 48360, 16368, 8856,  42184, 24228,
                45908, 15032, 19316, 33360, 28204, 54128, 17000, 45528, 25132,
                20068, 3516,  35784, 11224, 63084, 20828, 56328, 27344, 32852,
                46788, 22156, 10000, 14752, 31840, 23812, 11708, 64684, 8848,
                19468, 37184, 14436, 42532, 10516, 17060, 39816, 6808,  62572,
                53228, 28948, 61340, 17208, 8852,  29980, 59836, 11640, 28000,
                39192, 64924, 17436, 44060, 2588,  61780, 19364, 31008, 51836,
                15092, 56568, 4076,  63484, 34012, 26552, 41544, 31248, 21548,
                57044, 50444, 32748, 3196,  21984, 44560, 51708, 57872, 25368,
                61812, 9516,  14728, 64536, 41196, 18612, 9284,  42400, 32996,
                51700, 932,   65428, 53556, 47220, 34472, 21636, 60948, 260,
                37460, 53328, 14132, 49288, 38932, 9888,  42984, 24888, 7076,
                21580, 56244, 14692, 63720, 37492, 59056, 20488, 42200, 24328,
                36300, 58524, 27256, 55936, 65284, 31736, 28552, 5296,  25388,
                62716, 120,
            },
            {
                58888, 8244,  64564, 11120, 44360, 60980, 54044, 13492, 65108,
                3864,  55516, 25872, 8280,  46952, 39320, 4888,  51336, 41896,
                13364, 46484, 59736, 6856,  24892, 37244, 3716,  61416, 51340,
                1576,  65348, 28680, 58748, 7700,  42028, 35484, 45240, 30876,
                48544, 62300, 53916, 22212, 35500, 55680, 47864, 25932, 34516,
                15940, 1208,  5088,  55228, 50688, 40016, 2912,  52964, 57656,
                32312, 24240, 47004, 29944, 37328, 50872, 34636, 7780,  64028,
                27176, 36336, 59676, 46960, 7504,  16208, 4440,  13588, 47348,
                27312, 15396, 37760, 8460,  53984, 34676, 1376,  20444, 12912,
                46856, 36176, 21332, 49048, 6700,  30572, 60068, 22660, 56968,
                38228, 21016, 35384, 14344, 24080, 56664, 38800, 43844, 28840,
                58516, 46160, 33868, 2300,  59816, 20800, 31756, 64916, 37124,
                28192, 41852, 2908,  17648, 25480, 1532,  7348,  64680, 52648,
                4832,  33156, 17956, 7564,  40736, 1464,  54160, 44668, 57156,
                11740, 33976,
            },
            {
                49848, 38796, 52596, 31016, 21932, 5792,  35628, 29520, 39592,
                21304, 50340, 41828, 59740, 12736, 64596, 35088, 10060, 61684,
                54480, 32080, 16748, 43744, 50764, 30404, 9720,  16388, 42128,
                12728, 36860, 52528, 18652, 47528, 54188, 4812,  21532, 51584,
                1600,  25816, 7096,  46232, 63868, 4236,  30760, 19860, 9112,
                45460, 41156, 24324, 46988, 13032, 64160, 21036, 43104, 16280,
                4156,  12396, 55824, 6736,  14732, 26036, 57344, 11540, 3244,
                43752, 20768, 12792, 29512, 42884, 50516, 37164, 55372, 9824,
                3696,  63772, 18656, 59464, 47260, 28508, 64332, 39888, 30744,
                56104, 2264,  28208, 53152, 12008, 35012, 54152, 3876,  12924,
                27956, 8488,  40928, 60360, 31404, 10448, 5240,  48800, 13148,
                6432,  18324, 42276, 26536, 5060,  54636, 45888, 13676, 48592,
                10416, 33388, 53908, 47500, 43196, 50304, 32528, 10940, 46672,
                14464, 61464, 43352, 50368, 23928, 48408, 16052, 20532, 36168,
                47564, 17608,
            },
            {
                5720,  28084, 19248, 1872,  40800, 51276, 26072, 536,   48020,
                10672, 31596, 36156, 2368,  18212, 22296, 31116, 3020,  27552,
                37936, 22976, 1116,  55580, 19112, 39632, 58980, 45736, 24116,
                31020, 6580,  40184, 25528, 2120,  63236, 28028, 16864, 60924,
                13404, 39292, 32348, 18724, 28076, 12684, 60036, 51268, 37576,
                65164, 56980, 31504, 36616, 18888, 27072, 33536, 8132,  36776,
                49588, 62692, 40576, 21256, 60116, 42784, 47300, 23444, 39736,
                49836, 33108, 1080,  53612, 18428, 62308, 24044, 60184, 44808,
                40516, 51920, 30420, 43052, 24392, 6368,  17340, 49400, 4484,
                42452, 23808, 60856, 39072, 44620, 24664, 16348, 49972, 43412,
                62552, 46636, 16832, 50324, 3524,  19124, 64404, 33072, 25516,
                51508, 63272, 30768, 23036, 50980, 35984, 17840, 728,   61576,
                19204, 58756, 23524, 30216, 12420, 57316, 35592, 27160, 40216,
                22104, 30032, 2484,  12592, 35308, 62964, 10708, 40080, 2824,
                30352, 24268,
            },
            {
                41968, 60412, 55800, 15036, 62156, 45760, 18024, 59076, 55040,
                16312, 63116, 6552,  52472, 56644, 44100, 49792, 58920, 18940,
                7672,  48660, 64436, 28576, 12608, 5552,  34452, 54148, 20056,
                60876, 56024, 15516, 44832, 33240, 12352, 38456, 57240, 34284,
                42908, 9988,  59248, 52308, 336,   43632, 40408, 5912,  14268,
                27836, 20604, 10996, 6772,  59196, 432,   56396, 47388, 60728,
                25652, 18180, 34480, 53112, 9520,  736,   17780, 29828, 61220,
                16360, 55804, 25068, 38532, 10476, 32372, 2204,  28616, 20260,
                34372, 23024, 160,   10320, 14660, 55484, 36656, 10792, 62452,
                34076, 15740, 5784,  19036, 58344, 1400,  63988, 36660, 19624,
                31000, 2216,  25604, 58596, 44308, 28180, 40084, 54956, 16100,
                36688, 9392,  56416, 12260, 62184, 8024,  29748, 40744, 52928,
                6212,  39348, 4028,  63268, 8568,  21232, 16776, 62372, 6004,
                55760, 51596, 38268, 58728, 19568, 26756, 33040, 59276, 51068,
                62368, 13320,
            },
            {
                46384, 4008,  33888, 37532, 24560, 12040, 32944, 8396,  37916,
                26848, 45228, 22848, 28712, 38720, 15216, 25340, 12000, 41172,
                53104, 14712, 33016, 40808, 47400, 62596, 26808, 312,   8700,
                47180, 35144, 4096,  61752, 20212, 50092, 8552,  23200, 3040,
                55072, 49728, 24148, 36524, 16404, 57176, 33028, 23996, 54744,
                1784,  47912, 61112, 42408, 50216, 38688, 14236, 22740, 30508,
                10280, 1808,  45828, 27884, 32632, 64676, 38384, 52464, 4992,
                8984,  46016, 64416, 6612,  57760, 48604, 14884, 53056, 5824,
                12168, 58656, 48444, 62104, 33144, 41484, 58800, 26780, 19904,
                53656, 46224, 50596, 9876,  32232, 26992, 47460, 6600,  10936,
                57484, 53924, 6016,  33552, 13416, 52816, 22820, 2888,  59856,
                21252, 5324,  39480, 46884, 15932, 43572, 56992, 24704, 34432,
                27396, 51292, 16108, 36196, 41100, 54360, 44312, 2932,  48132,
                18532, 25600, 8532,  45060, 53156, 3600,  7224,  43908, 22404,
                8664,  53864,
            },
            {
                20380, 26248, 10096, 48768, 6764,  53480, 64148, 20736, 42708,
                4648,  12868, 60288, 34164, 8724,  156,   63768, 46852, 34700,
                57500, 2652,  25020, 9264,  56400, 21420, 15224, 43360, 50256,
                28544, 11288, 22780, 51960, 30104, 59004, 41004, 48032, 29248,
                19804, 14492, 4868,  64724, 47536, 21060, 10328, 62944, 44336,
                17552, 53068, 34096, 23700, 16488, 28724, 53276, 5488,  44420,
                54752, 39128, 59084, 4756,  14008, 49228, 21540, 12336, 58128,
                36704, 19968, 28140, 41328, 22420, 44380, 35616, 64900, 39720,
                56252, 16624, 38032, 27244, 52632, 2452,  22512, 45236, 7488,
                13060, 29724, 65280, 36996, 54820, 14124, 41704, 33752, 23616,
                39656, 48676, 21356, 37960, 61628, 7640,  47524, 11356, 45356,
                42004, 34152, 50184, 28232, 3664,  22232, 60220, 10740, 14400,
                46772, 20948, 44784, 60480, 328,   24488, 28592, 58064, 37304,
                11812, 34384, 64432, 15824, 29408, 47828, 55900, 14780, 28352,
                38512, 32552,
            },
            {
                16508, 63732, 43516, 58548, 28980, 41316, 2448,  49736, 30776,
                56864, 51424, 19148, 48732, 42048, 53992, 31848, 4520,  28976,
                20336, 43884, 60572, 35800, 4160,  52312, 38432, 31832, 63984,
                17796, 58044, 42660, 37104, 860,   14868, 26080, 5988,  63488,
                45784, 31408, 41380, 27108, 8216,  30164, 50228, 35740, 4308,
                39332, 29832, 9064,  3292,  45652, 62096, 11508, 41540, 65368,
                20108, 15264, 51144, 23732, 42324, 56268, 34580, 26332, 43568,
                31028, 14580, 51552, 1676,  12568, 30380, 19240, 9056,  47408,
                24780, 31528, 6948,  44732, 18868, 5716,  49816, 31908, 60504,
                40536, 484,   25060, 43340, 3432,  21488, 59752, 51452, 62164,
                14716, 29128, 9768,  43044, 17200, 35708, 30248, 63700, 26736,
                57356, 1896,  18404, 65312, 32936, 37436, 48992, 2160,  54576,
                64516, 9004,  29460, 13020, 33020, 52164, 7120,  14304, 30956,
                60224, 42816, 1164,  22772, 41396, 61280, 36912, 18452, 65056,
                1912,  57232,
            },
            {
                36312, 49936, 724,   21660, 17196, 35132, 23744, 14584, 61496,
                35520, 1760,  25168, 11024, 62132, 21560, 17156, 59652, 9912,
                39284, 51704, 17480, 12180, 45336, 24064, 59924, 2992,  13316,
                40436, 25240, 7344,  64956, 10176, 44272, 53952, 17464, 35688,
                56732, 9508,  59588, 37992, 2636,  55920, 15052, 25584, 59964,
                21936, 13268, 64464, 56232, 36256, 20876, 32204, 2192,  35248,
                26960, 7276,  31412, 63020, 18952, 7612,  2224,  46920, 63436,
                4120,  54808, 34068, 62232, 49656, 59476, 5084,  26988, 43076,
                1744,  63088, 21212, 55100, 13372, 64572, 38280, 16144, 55600,
                35736, 21000, 57968, 17288, 49776, 28868, 9248,  4376,  18168,
                44588, 1364,  64788, 54848, 24384, 50196, 584,   19732, 14556,
                31468, 52040, 25064, 13420, 53780, 7460,  19412, 42848, 31608,
                36868, 5536,  58384, 50108, 17964, 46000, 65128, 40104, 20072,
                49976, 5684,  56692, 32248, 9296,  12360, 25144, 5540,  40900,
                51952, 11924,
            },
            {
                5076,  40164, 30932, 12976, 55208, 60204, 47316, 10232, 27856,
                17388, 40508, 46488, 30276, 5968,  37052, 50264, 45144, 23472,
                6488,  65184, 27152, 49968, 30516, 8108,  18944, 35240, 53184,
                5884,  55096, 33960, 48724, 19604, 32480, 61004, 39328, 12512,
                1328,  21724, 52516, 17940, 45352, 62032, 42000, 6964,  46792,
                32776, 49672, 40728, 25728, 6392,  51132, 58312, 17564, 49332,
                56156, 38028, 47288, 11452, 36428, 29544, 60188, 10240, 17376,
                23488, 39976, 7796,  25840, 17064, 37688, 55544, 51864, 15536,
                36580, 50700, 9696,  34556, 40932, 29964, 24140, 10080, 3936,
                27288, 48412, 11396, 7168,  61784, 38732, 56124, 31616, 37332,
                25768, 58048, 32360, 12712, 5828,  59200, 41108, 53800, 38320,
                7052,  60444, 9828,  45012, 58556, 40160, 26220, 61776, 16876,
                22676, 41632, 25740, 38136, 10256, 22324, 35116, 53988, 9584,
                24096, 28156, 46888, 52048, 20324, 34840, 54936, 49088, 30008,
                44752, 24008,
            },
            {
                27828, 62068, 51648, 45716, 5832,  38340, 3852,  43036, 52492,
                7328,  65460, 55032, 15488, 57684, 27216, 12388, 35080, 54424,
                15276, 32356, 680,   38040, 62732, 57128, 41340, 49064, 29376,
                20944, 45732, 15756, 27980, 56228, 22620, 4512,  50780, 28368,
                43368, 33760, 48816, 24456, 12664, 32184, 19708, 52316, 11136,
                57708, 2080,  19440, 43756, 14908, 28152, 46492, 9104,  24292,
                12748, 60764, 228,   43896, 54520, 25096, 40724, 52908, 37284,
                49160, 61088, 11488, 42712, 46400, 3236,  22996, 32992, 61536,
                12348, 29224, 60020, 47940, 852,   58108, 51472, 45688, 61532,
                42204, 52748, 30608, 33948, 45608, 24276, 12680, 46736, 53472,
                7984,  50608, 40192, 19444, 27748, 44812, 33252, 10220, 23468,
                48680, 42580, 36600, 21440, 288,   30512, 11696, 51684, 3856,
                57268, 48804, 1692,  63324, 55784, 4668,  27388, 2312,  44044,
                62776, 17244, 37628, 4192,  63576, 46004, 280,   60700, 20940,
                9068,  59436,
            },
            {
                18400, 14772, 8136,  34200, 26596, 64440, 31788, 20124, 59040,
                37820, 21948, 32920, 4024,  39436, 43520, 1636,  61304, 26076,
                47216, 58304, 42820, 21792, 14340, 5252,  26724, 10284, 63124,
                1028,  38912, 59680, 3492,  12908, 37340, 47020, 8504,  24836,
                64344, 15464, 60516, 5484,  36328, 54524, 596,   29064, 37656,
                16412, 27520, 52740, 10400, 60892, 38620, 4068,  63228, 40460,
                30668, 21344, 5528,  58536, 16840, 49644, 14060, 32300, 20476,
                828,   29068, 58412, 20920, 31668, 63956, 10228, 40992, 20220,
                3704,  44468, 23312, 16316, 26628, 8300,  19840, 33124, 13712,
                18616, 5336,  63724, 15336, 41392, 19372, 792,   64276, 21268,
                15960, 3420,  35460, 48160, 61296, 4140,  16000, 56108, 62092,
                3384,  17800, 28096, 63908, 47956, 16056, 56452, 44240, 28272,
                34680, 14092, 19076, 31844, 42744, 15492, 47472, 58572, 13204,
                33876, 7500,  59660, 14972, 39840, 27560, 16928, 32684, 13528,
                55928, 35084,
            },
            {
                42340, 54436, 23464, 57940, 19068, 11184, 49176, 24952, 608,
                13548, 44212, 9964,  24192, 53084, 63544, 20664, 8364,  40116,
                4292,  19376, 10724, 34856, 48008, 54860, 16836, 43260, 57788,
                24220, 31708, 9592,  51716, 42480, 63316, 30560, 57428, 19420,
                53752, 3252,  39544, 9208,  26728, 65332, 40868, 23300, 60284,
                44760, 63860, 5576,  35136, 55756, 22140, 33160, 53804, 43400,
                15600, 52804, 34088, 28392, 22540, 3088,  64476, 5956,  57164,
                43656, 15188, 35608, 6040,  52512, 14208, 28460, 48176, 58336,
                53600, 37756, 5976,  55448, 43392, 36908, 63396, 2520,  39124,
                56788, 26020, 36676, 3012,  50568, 60368, 27424, 34980, 42956,
                30156, 62736, 23436, 9220,  52912, 37852, 21520, 26976, 45816,
                34952, 13968, 52488, 4632,  33140, 23760, 36212, 6384,  10148,
                62356, 45548, 7904,  51444, 28648, 61264, 38596, 18760, 51232,
                41456, 54336, 30332, 24816, 10492, 53320, 42840, 6680,  38936,
                47052, 2672,
            },
            {
                30412, 48644, 1252,  39648, 44592, 15184, 53932, 57332, 34928,
                28952, 50996, 60592, 48124, 18392, 31716, 14288, 51736, 33576,
                56172, 29800, 61848, 52136, 2600,  23296, 32904, 36748, 13212,
                46976, 18336, 62100, 35036, 25616, 17188, 216,   11608, 41116,
                32556, 46500, 28888, 57880, 43964, 15912, 48188, 4480,  8816,
                33416, 21180, 39764, 47932, 30300, 13048, 18280, 1660,  26504,
                7044,  64220, 46524, 39108, 9580,  42032, 35140, 46320, 26896,
                9276,  53508, 48340, 24824, 38892, 56540, 628,   34860, 8188,
                25176, 18188, 65488, 32804, 12888, 49184, 28812, 54016, 23048,
                47760, 8828,  59024, 22260, 54556, 10572, 6252,  48812, 58820,
                10972, 54344, 45904, 17928, 30664, 13220, 65112, 1112,  50880,
                30264, 58124, 39884, 8836,  54488, 41980, 63092, 49380, 21464,
                39584, 25004, 54452, 36828, 11584, 23432, 6788,  32048, 25512,
                1104,  21064, 47992, 44192, 2732,  57652, 22472, 61736, 50968,
                25688, 64736,
            },
            {
                21152, 36732, 10156, 63264, 28240, 5120,  40752, 8704,  46420,
                63756, 17020, 6476,  35896, 2580,  28628, 58512, 48488, 22880,
                12124, 44352, 7028,  39068, 28300, 44956, 64872, 4796,  51164,
                7540,  40972, 49668, 21772, 6184,  53032, 45188, 60072, 36180,
                6500,  13512, 18628, 51044, 21972, 11412, 34720, 56508, 50796,
                13452, 54640, 952,   24528, 7408,  62360, 45720, 59160, 49716,
                36984, 11768, 18860, 51344, 61896, 55052, 12580, 23912, 61480,
                31156, 18440, 65180, 3004,  43912, 19488, 62616, 16908, 46028,
                30648, 49820, 40048, 3036,  21380, 59804, 6324,  15976, 42664,
                31228, 12424, 43992, 28532, 38120, 32148, 18000, 39800, 2436,
                14336, 26624, 41416, 6296,  58608, 43688, 32780, 40548, 11684,
                6224,  20764, 24944, 60756, 12616, 19480, 1436,  15312, 57692,
                30808, 3344,  60032, 20552, 492,   56900, 48704, 64380, 10316,
                37092, 57060, 12500, 65372, 31796, 18500, 36644, 29060, 11348,
                16432, 6312,
            },
            {
                56712, 13992, 52264, 32124, 59424, 35852, 22608, 18184, 31252,
                3116,  26508, 39232, 57120, 42504, 10440, 44920, 36992, 996,
                63144, 15716, 24988, 55500, 18096, 9444,  59116, 21080, 30176,
                56368, 27124, 2916,  14080, 55852, 38940, 27540, 21292, 49804,
                24112, 55776, 61668, 37556, 1524,  30460, 62744, 18840, 25476,
                29472, 42800, 17348, 57504, 50972, 40644, 34804, 9664,  22944,
                56848, 31524, 1204,  25220, 6748,  30720, 19736, 50536, 2344,
                37800, 41608, 12376, 33464, 8964,  50060, 26340, 42080, 4780,
                60612, 13988, 9412,  56952, 25380, 45864, 10464, 35692, 62332,
                248,   52176, 19740, 64920, 14528, 46436, 57328, 52452, 21888,
                33480, 38408, 56412, 1888,  50072, 24556, 8412,  54888, 18592,
                63240, 44372, 49276, 31204, 37536, 46696, 29400, 52220, 34160,
                12292, 47124, 16532, 43452, 34444, 40228, 14688, 45028, 28984,
                60436, 17636, 5620,  35224, 50616, 8652,  46664, 4352,  53704,
                33764, 45196,
            },
            {
                40780, 3812,  25324, 19896, 46904, 1996,  51100, 62448, 43152,
                11332, 54552, 20824, 13128, 25016, 64952, 6956,  17056, 26748,
                41212, 50704, 34052, 60236, 12832, 48860, 40288, 1236,  15456,
                61516, 37516, 45448, 30896, 65244, 33672, 8748,  15520, 62988,
                2036,  42384, 31920, 7812,  47304, 53292, 6244,  38520, 45960,
                3316,  63332, 36924, 11012, 27864, 20264, 4456,  29776, 16308,
                40096, 61092, 43800, 15048, 36808, 59188, 44484, 16396, 56224,
                7184,  59872, 23176, 54840, 29612, 58860, 37156, 10984, 55604,
                22572, 35816, 43160, 52120, 30240, 38328, 18396, 50548, 26772,
                58024, 34132, 5800,  49496, 2020,  8228,  24388, 29292, 63584,
                7216,  60984, 20020, 28920, 35076, 16572, 60008, 47824, 26480,
                36544, 56760, 1924,  16804, 6888,  58692, 25420, 4560,  41240,
                64200, 7336,  50768, 29188, 62960, 5128,  52668, 21588, 3536,
                43008, 52044, 23304, 27632, 41080, 59580, 14620, 63336, 38556,
                23680, 59128,
            },
            {
                28048, 49648, 62320, 16612, 7912,  55576, 13312, 24728, 37080,
                49576, 59796, 32084, 47240, 52432, 34400, 21976, 54068, 59144,
                31060, 4836,  20856, 1852,  37060, 31472, 25636, 52372, 34896,
                23396, 8596,  17700, 59280, 11964, 43060, 3780,  47064, 29584,
                52880, 10756, 25884, 16732, 59552, 22984, 41784, 14676, 60472,
                9736,  49444, 32128, 59668, 14968, 43860, 65252, 55620, 47496,
                3588,  20820, 53968, 49252, 28052, 4408,  40024, 10008, 34064,
                47640, 28268, 15484, 45036, 3916,  21632, 14468, 31692, 51192,
                2208,  28068, 61740, 15636, 1352,  64120, 54992, 4720,  21680,
                39864, 15144, 25264, 41556, 36172, 59412, 44548, 12244, 42156,
                48244, 15596, 10152, 46368, 64100, 4220,  22060, 38504, 5124,
                14240, 33652, 10264, 42880, 64764, 21640, 54140, 44712, 18232,
                27044, 38072, 55344, 23740, 9460,  18580, 26712, 55836, 33448,
                8544,  39228, 15076, 55168, 2156,  20748, 25868, 43144, 1324,
                18812, 8976,
            },
            {
                31696, 12036, 44232, 38640, 33472, 42064, 29300, 57532, 4704,
                16024, 7416,  40648, 56,    17872, 60656, 4012,  39084, 49056,
                8760,  46796, 64492, 53728, 43724, 7452,  57512, 46472, 11744,
                42568, 54112, 48508, 1008,  24724, 51380, 18916, 58844, 39916,
                34988, 20276, 64824, 44704, 33912, 12208, 27472, 57340, 35356,
                19688, 23088, 5404,  46520, 1748,  52708, 25044, 12280, 35648,
                27168, 8636,  33856, 11208, 64860, 21728, 53360, 26124, 63712,
                20896, 60,    51912, 38700, 61612, 47424, 6688,  64708, 38792,
                18848, 48548, 6240,  34140, 20416, 41856, 12744, 32572, 44840,
                9532,  55392, 60696, 29784, 53304, 20928, 16896, 35152, 3000,
                51568, 25744, 37236, 52612, 41904, 13716, 53700, 43424, 29716,
                51080, 23116, 53384, 27260, 39204, 34448, 11644, 8556,  60312,
                22112, 2904,  13816, 33312, 45724, 58472, 37876, 12144, 49616,
                63880, 30004, 47324, 62084, 10804, 33104, 50036, 29836, 54608,
                47616, 60932,
            },
            {
                37000, 22364, 496,   53392, 26344, 61168, 9880,  47556, 21524,
                34336, 27608, 62768, 9256,  30324, 45912, 14756, 29308, 11440,
                19048, 36104, 14048, 28564, 22756, 16296, 62472, 19268, 3460,
                63968, 32252, 20956, 40244, 29012, 36412, 55332, 23368, 13408,
                6972,  49200, 2988,  39364, 54872, 4680,  49724, 188,   30724,
                52980, 41008, 26548, 55288, 33824, 38344, 7204,  42644, 50896,
                63220, 57772, 41284, 18312, 46848, 1820,  32308, 49784, 13216,
                43016, 36080, 57732, 11108, 18480, 34320, 53484, 24768, 43696,
                57640, 12068, 23936, 46716, 58256, 7576,  48908, 29028, 62696,
                3072,  46828, 18604, 6804,  11032, 62284, 27208, 55988, 5580,
                30476, 59640, 22816, 212,   31928, 27156, 10664, 62652, 1428,
                58060, 17848, 61596, 47232, 3184,  14864, 48884, 30312, 36036,
                57104, 42652, 48424, 61284, 1392,  30520, 42208, 16856, 1976,
                24852, 19752, 4336,  35900, 44388, 58576, 7032,  12488, 35352,
                15868, 5052,
            },
            {
                51112, 65288, 57152, 6564,  20412, 15364, 3364,  39516, 64856,
                45112, 52812, 23552, 50832, 36288, 42036, 25952, 55144, 61856,
                43324, 24856, 57028, 38888, 10508, 50308, 5500,  29440, 38492,
                26116, 6632,  14788, 57236, 61876, 4556,  9456,  32408, 45164,
                60564, 28164, 56796, 14856, 30024, 20960, 63640, 43856, 16204,
                7800,  64312, 12516, 17808, 61424, 22160, 15996, 31340, 304,
                23560, 14644, 4792,  30224, 56056, 38244, 7916,  60256, 17900,
                5448,  31376, 23952, 7872,  27480, 41236, 3152,  16236, 9164,
                29348, 62980, 39908, 54168, 27184, 37216, 23136, 59308, 16948,
                35296, 24068, 38076, 48980, 33100, 780,   39532, 46656, 65420,
                18732, 40552, 7772,  55432, 17744, 58660, 34424, 19824, 45852,
                8932,  37384, 5636,  30852, 20236, 55452, 62680, 32,    51820,
                16148, 28440, 10344, 20680, 51392, 7060,  63156, 34612, 56672,
                44864, 59376, 13236, 52992, 24204, 17920, 39744, 64156, 22884,
                56332, 42056,
            },
            {
                19016, 13844, 29592, 46220, 35948, 50720, 54920, 30856, 18048,
                1604,  13560, 57016, 19816, 11908, 58568, 1900,  7412,  50388,
                32584, 5860,  2876,  46048, 60652, 41232, 34300, 55472, 44464,
                59772, 49556, 34776, 10612, 43776, 16640, 50088, 63832, 612,
                18328, 36324, 23876, 8088,  51556, 36880, 10480, 24640, 48036,
                56164, 37664, 45180, 29868, 9180,  48844, 56940, 62204, 19684,
                46104, 36936, 44340, 25084, 61984, 15504, 23084, 45588, 40520,
                55892, 62172, 48996, 44524, 63912, 30752, 59776, 50128, 36560,
                416,   32956, 16932, 3884,  10044, 14872, 51652, 896,   42496,
                11408, 53496, 64588, 14200, 43772, 51848, 23220, 15084, 9772,
                32940, 45168, 12452, 63312, 47900, 6460,  39728, 50432, 25412,
                32620, 41584, 13264, 58892, 43988, 26412, 40664, 23492, 45536,
                6024,  65132, 34956, 39988, 25520, 14496, 46576, 22076, 9692,
                38524, 31744, 7584,  41572, 28640, 36,    52128, 45540, 3776,
                27136, 10268,
            },
            {
                47204, 24752, 40472, 9272,  62792, 42940, 23188, 11556, 58072,
                37728, 43332, 32700, 3444,  48784, 64620, 22904, 37872, 15448,
                21044, 63632, 52008, 18220, 30716, 25128, 72,    21388, 12128,
                17104, 53000, 2340,  27064, 47404, 21928, 38604, 26064, 42404,
                53296, 11456, 47156, 62664, 43272, 18228, 58260, 39644, 32668,
                2348,  21184, 4552,  51188, 41972, 2492,  27840, 39972, 10644,
                53260, 60164, 7616,  50672, 11948, 35008, 54108, 29532, 2640,
                26520, 11916, 16796, 1444,  54348, 12644, 20328, 45788, 56600,
                22056, 52996, 44948, 60528, 30592, 64468, 40356, 20248, 56180,
                26396, 31268, 3620,  21456, 28424, 57032, 6396,  37048, 54648,
                49860, 21352, 35584, 28716, 43236, 23648, 2644,  54444, 15476,
                65256, 49044, 24036, 51768, 10208, 6836,  17580, 37368, 12520,
                32428, 53748, 3872,  17612, 55044, 60644, 28892, 5024,  53576,
                26796, 50676, 15984, 62676, 57576, 9980,  37168, 19808, 31108,
                62124, 34492,
            },
            {
                1536,  58948, 32644, 17500, 2392,  27280, 33668, 48504, 7860,
                26776, 61676, 6360,  40064, 28688, 17220, 34216, 46292, 57584,
                40632, 27248, 35204, 13152, 7664,  53440, 65384, 47748, 8516,
                36964, 24020, 41368, 64660, 30836, 54548, 5780,  13856, 30124,
                58628, 5388,  40464, 31704, 1304,  27332, 5900,  61852, 13952,
                28120, 60620, 34696, 24444, 58424, 36556, 13896, 47632, 34024,
                3880,  28220, 32268, 19464, 916,   42188, 59356, 10292, 20340,
                36972, 52872, 39668, 21848, 35368, 25912, 4952,  39400, 14456,
                26920, 7172,  41460, 19348, 35744, 46280, 6640,  33064, 48300,
                5016,  44908, 59060, 9136,  40584, 18316, 60496, 29680, 24932,
                1440,  59072, 4196,  51880, 14472, 37016, 60988, 11620, 28932,
                4624,  18984, 948,   35724, 63668, 33132, 56536, 50292, 61404,
                21216, 26612, 58916, 43684, 36500, 11204, 49944, 40676, 18380,
                65020, 1664,  34948, 21364, 46560, 32676, 14204, 59400, 48944,
                7816,  53884,
            },
            {
                38896, 5808,  49244, 55476, 59980, 14388, 4864,  63916, 20148,
                53452, 15724, 24172, 55872, 44600, 8468,  52744, 12332, 30044,
                800,   9852,  47952, 59584, 37836, 42972, 14656, 28496, 58248,
                32000, 61224, 19644, 13072, 8140,  35792, 60264, 20224, 50952,
                34520, 21824, 16552, 57208, 13208, 54132, 22136, 50488, 9116,
                46164, 52700, 16516, 11244, 64732, 19104, 5340,  55820, 22908,
                16924, 57272, 39388, 64292, 47920, 27228, 6260,  51372, 65388,
                46748, 33052, 6980,  59244, 43304, 48600, 9940,  60708, 33924,
                63648, 11504, 50352, 25284, 2000,  57112, 23520, 13756, 63328,
                38692, 15968, 50024, 35488, 63188, 2892,  47532, 13188, 41684,
                62396, 16816, 26600, 57360, 8752,  20272, 31484, 42604, 59304,
                38928, 56688, 43140, 27876, 15096, 47680, 2024,  29364, 9280,
                41932, 14920, 49072, 8612,  23628, 524,   32840, 58772, 13608,
                43548, 23868, 48692, 5572,  55788, 26164, 3052,  41040, 23896,
                16716, 43640,
            },
            {
                21020, 28148, 11976, 22464, 44816, 38632, 52024, 41580, 35908,
                30396, 46816, 10196, 36796, 20472, 58940, 4436,  25204, 61148,
                54012, 43952, 20296, 55588, 2412,  23500, 18624, 51420, 40128,
                1572,  6228,  44900, 50224, 56528, 3056,  40380, 46288, 9680,
                2200,  64524, 25748, 49136, 35268, 45452, 30220, 41636, 36548,
                19260, 57124, 39860, 6936,  44512, 32592, 50372, 26568, 63572,
                43336, 9192,  13324, 52536, 21916, 16344, 38112, 31640, 13840,
                4332,  60920, 24864, 14892, 55792, 31540, 18108, 52252, 2404,
                47104, 30020, 59008, 15736, 53688, 10844, 28864, 59784, 8800,
                19188, 29660, 55268, 11692, 25984, 33308, 51264, 7864,  31776,
                44576, 11228, 39616, 33584, 45820, 64352, 48776, 7176,  21864,
                33636, 9476,  52984, 60280, 22196, 39824, 19564, 58612, 4628,
                46692, 38068, 2960,  30700, 63676, 45416, 20904, 37312, 3980,
                30084, 60192, 11476, 37792, 17272, 43732, 64740, 52504, 11596,
                29844, 60604,
            },
            {
                15056, 64528, 52532, 35468, 30128, 19288, 9904,  24580, 1084,
                59504, 4060,  50604, 64072, 13820, 31236, 49344, 42388, 18044,
                6568,  33648, 15340, 26472, 31672, 62120, 4636,  34704, 46308,
                25888, 54024, 15732, 28808, 33324, 23652, 17740, 62304, 27548,
                55260, 44536, 38848, 8808,  60584, 3008,  16828, 65144, 5376,
                25296, 556,   31280, 27052, 54216, 21800, 8928,  40796, 1108,
                35992, 46380, 25336, 3024,  34656, 44936, 58208, 24076, 42792,
                18052, 29092, 50240, 9316,  2116,  64612, 28248, 42308, 23668,
                37944, 20584, 5384,  34248, 39432, 49140, 44216, 37316, 52412,
                24660, 41448, 24,    21860, 43164, 15352, 38252, 22636, 54252,
                19728, 49192, 6216,  54928, 17584, 564,   26160, 52156, 16464,
                46580, 25568, 13652, 6356,  31216, 44508, 11856, 52484, 34848,
                22804, 62176, 52948, 18808, 56652, 27800, 15696, 55612, 47144,
                8336,  54312, 41276, 27904, 50956, 8944,  20780, 34276, 4736,
                56476, 36440,
            },
            {
                45776, 9096,  42040, 264,   6796,  61972, 48024, 56192, 13248,
                43420, 18020, 33148, 40816, 26264, 1552,  36020, 63068, 38552,
                23620, 50032, 64288, 40384, 8820,  44504, 57372, 10540, 13780,
                64044, 22256, 37724, 59496, 10700, 43652, 49328, 14992, 37600,
                31168, 12800, 3784,  20004, 51904, 23968, 11716, 54688, 33736,
                48988, 61320, 42696, 13356, 62432, 46784, 15768, 59392, 52328,
                19876, 29276, 60916, 54656, 5452,  61964, 8956,  48740, 1316,
                57092, 35512, 45316, 20160, 40896, 36540, 6208,  57296, 15376,
                8628,  55160, 43120, 65492, 3788,  21276, 17044, 1560,  33900,
                5676,  57976, 46812, 62016, 52916, 4532,  57140, 64808, 2004,
                28036, 37204, 63076, 23800, 30180, 41020, 13296, 35804, 55444,
                2540,  64096, 36724, 49992, 3216,  55668, 26092, 64648, 16744,
                28548, 10432, 33260, 40452, 12404, 50852, 6000,  61756, 25012,
                34236, 19820, 14896, 62936, 832,   31240, 58480, 38744, 48016,
                26252, 2076,
            },
            {
                31152, 18776, 25056, 59572, 54052, 26816, 16576, 32484, 62852,
                28136, 22824, 53192, 6160,  60544, 47060, 22016, 14680, 10576,
                29120, 58460, 4968,  12584, 48792, 53092, 37292, 20120, 30468,
                48912, 9044,  41912, 4908,  63080, 26484, 268,   57596, 6380,
                22916, 48452, 61548, 28608, 33112, 43208, 38064, 58652, 21408,
                9464,  15400, 52240, 4572,  35528, 2272,  38160, 30688, 12368,
                6840,  49172, 15548, 41360, 31032, 12104, 28016, 20288, 39808,
                52728, 11360, 62496, 27116, 54084, 22424, 11932, 45676, 31964,
                62592, 48356, 12392, 27436, 31740, 60724, 54740, 26752, 62532,
                49536, 13544, 31936, 8428,  27892, 19244, 45432, 10528, 35044,
                59384, 13768, 3396,  50944, 10180, 60588, 57056, 5044,  43312,
                30484, 19924, 41164, 17328, 61600, 33836, 7200,  37644, 42896,
                50428, 5844,  58012, 25540, 2056,  35940, 42656, 10252, 39520,
                52092, 3532,  32492, 56668, 23228, 46764, 12544, 17824, 7596,
                62788, 50824,
            },
            {
                55536, 37964, 48232, 33088, 12780, 46056, 37336, 5444,  39812,
                8864,  57900, 45496, 11988, 17268, 54568, 7792,  51388, 56736,
                3124,  45684, 35588, 21748, 28692, 16184, 24688, 60464, 2968,
                55324, 33656, 17640, 52936, 20808, 36188, 51852, 32548, 42144,
                18524, 53832, 40432, 15640, 56084, 6720,  26876, 2284,  44964,
                29148, 38488, 23072, 57840, 18744, 28284, 55992, 22588, 63120,
                33996, 58036, 23688, 37248, 18784, 50268, 56260, 33620, 63388,
                7676,  16260, 32072, 5624,  48272, 16768, 61564, 50928, 25848,
                900,   35768, 17624, 23244, 46632, 7372,  14096, 42868, 10072,
                20912, 39200, 17096, 36316, 60600, 40480, 30472, 24348, 17128,
                47108, 42764, 32448, 20952, 45456, 37736, 27716, 22436, 62392,
                11688, 48540, 58528, 27512, 10052, 46392, 23480, 14444, 372,
                60868, 21108, 44636, 17256, 65264, 47904, 21564, 29600, 17396,
                64084, 44040, 49240, 7000,  36824, 61512, 28752, 54728, 40344,
                21432, 13116,
            },
            {
                3296,  63516, 16172, 8344,  40748, 21144, 3084,  54668, 50348,
                20364, 35772, 2792,  29652, 39088, 34240, 27820, 43824, 32476,
                39672, 18844, 53840, 61728, 1188,  33060, 6064,  40224, 43788,
                26692, 58900, 1480,  29404, 46364, 14312, 8440,  65316, 11372,
                59368, 34260, 4748,  10120, 64252, 46968, 13920, 51356, 17616,
                63852, 47396, 32948, 7884,  49796, 43916, 10368, 47944, 40168,
                3912,  44284, 10092, 224,   64928, 42464, 14848, 3808,  26144,
                22744, 37428, 42600, 480,   58616, 38980, 34060, 4356,  19612,
                40364, 53796, 58220, 9764,  51720, 41120, 35316, 57380, 30780,
                45848, 64988, 25344, 2068,  50920, 12760, 6044,  49740, 53672,
                8536,  26176, 57804, 15560, 53824, 7716,  18116, 49516, 33352,
                8192,  24168, 856,   54380, 39552, 20404, 51832, 56472, 31496,
                48136, 13136, 34652, 55180, 8540,  31712, 53764, 59528, 1196,
                12840, 27360, 21764, 10956, 42492, 15828, 2384,  45048, 24608,
                33228, 43228,
            },
            {
                29204, 5616,  23484, 51124, 57548, 64756, 30320, 24336, 44292,
                13680, 64124, 48716, 24648, 56388, 62344, 19848, 276,   65148,
                15620, 25844, 8288,  42292, 56212, 46844, 63380, 51004, 8096,
                14564, 38704, 47876, 11780, 56944, 39652, 24644, 45000, 29540,
                1824,  25828, 45812, 36708, 22624, 31180, 59632, 40560, 35836,
                5256,  11912, 60212, 41068, 24520, 65400, 5432,  17556, 26048,
                14212, 55188, 27456, 51908, 21692, 6036,  35988, 44572, 47644,
                59720, 50820, 56080, 13684, 24272, 29000, 8304,  43604, 59912,
                13804, 28420, 4716,  38008, 63148, 3368,  24564, 18596, 2380,
                51460, 6164,  56076, 43436, 22556, 58640, 32816, 63224, 37592,
                696,   62140, 39704, 4876,  34504, 64912, 1368,  40236, 14116,
                53232, 44924, 34624, 15404, 30656, 4476,  62848, 41084, 8340,
                24904, 38392, 4176,  28432, 41412, 14128, 4620,  24320, 45728,
                35336, 56096, 39148, 59844, 25708, 32096, 52304, 5896,  60968,
                10260, 58020,
            },
            {
                19972, 53188, 44480, 28412, 1684,  34484, 11312, 17476, 58476,
                6944,  32032, 16392, 42024, 9792,  5196,  46600, 24032, 11320,
                58968, 48896, 31092, 13484, 38500, 18424, 10896, 22628, 35120,
                19328, 64780, 23288, 32180, 61952, 3428,  18272, 55812, 49544,
                21448, 52868, 62312, 17164, 49780, 472,   19500, 8972,  25788,
                53288, 20860, 1216,  54772, 13520, 30988, 35220, 59760, 51000,
                31688, 62208, 38536, 46284, 32580, 59232, 53552, 28652, 12604,
                2624,  19556, 29984, 46172, 65192, 51896, 11380, 55676, 31088,
                22748, 49580, 44400, 32916, 20256, 29692, 48140, 60440, 37344,
                13036, 28452, 34124, 9948,  46684, 15916, 3796,  41996, 19052,
                29172, 22368, 11628, 47568, 30424, 23888, 44012, 57996, 29420,
                61440, 38140, 6536,  65196, 47924, 12264, 35664, 18432, 27972,
                58724, 51520, 63032, 46528, 19600, 61300, 49852, 38468, 57836,
                8160,  50504, 18204, 4956,  47880, 65296, 19660, 37476, 49548,
                15564, 35632,
            },
            {
                41264, 59716, 13864, 37040, 19184, 48960, 42132, 61996, 37940,
                27340, 53448, 1020,  59704, 50960, 14164, 37772, 53696, 42524,
                35252, 3436,  51804, 23536, 4388,  27412, 58448, 30252, 54492,
                44856, 5200,  51676, 7708,  43040, 27816, 34992, 6760,  15820,
                38424, 12896, 42968, 7868,  28308, 54000, 34388, 62712, 57312,
                42864, 32176, 27724, 17080, 39188, 52152, 21248, 892,   42640,
                8208,  19528, 2724,  16496, 11624, 24592, 8584,  18308, 64172,
                41184, 9992,  34420, 6740,  17236, 21104, 37004, 46936, 3416,
                63896, 7492,  16240, 59404, 11888, 55512, 15204, 7824,  53532,
                22860, 40400, 59540, 19888, 53880, 36488, 26948, 57088, 13944,
                44992, 55924, 50532, 16972, 59848, 9288,  51236, 20684, 3836,
                17180, 26676, 21652, 56684, 42888, 23892, 59916, 2952,  45304,
                16492, 10900, 22840, 1628,  36788, 26348, 11252, 33048, 20536,
                15192, 31128, 2132,  54884, 34920, 7992,  12200, 56928, 27532,
                572,   47620,
            },
            {
                31336, 25272, 10484, 61492, 6644,  55876, 25612, 9124,  4132,
                47140, 40012, 22892, 34728, 18932, 26956, 31836, 61212, 6548,
                28468, 20712, 62668, 57448, 35824, 42920, 48720, 348,   13004,
                59672, 25660, 36892, 16284, 21160, 53352, 59880, 40976, 63848,
                30436, 4312,  32844, 57752, 39064, 11588, 44532, 23096, 2868,
                13792, 49492, 45192, 62860, 4104,  9748,  45924, 55696, 36760,
                23508, 48384, 29244, 60964, 55832, 39316, 48564, 30916, 36840,
                55036, 25392, 60648, 39344, 43148, 1964,  62168, 25252, 18300,
                41640, 34256, 53020, 26008, 684,   39768, 43792, 26476, 63560,
                45440, 16936, 512,   30628, 7256,  64196, 23528, 48848, 9188,
                32336, 35340, 3484,  42172, 26984, 37632, 12720, 35332, 55284,
                47368, 10504, 51108, 2008,  28876, 8900,  49488, 32288, 53580,
                6560,  41988, 33528, 55944, 16068, 52692, 44200, 5944,  64472,
                42216, 27752, 62384, 44736, 21228, 41296, 30372, 43876, 23008,
                61732, 8668,
            },
            {
                54932, 3668,  49680, 39872, 32460, 45500, 14412, 52000, 30952,
                19760, 56904, 11772, 45064, 62672, 3844,  49600, 22164, 17148,
                47820, 40140, 10296, 45204, 15100, 7804,  64228, 17232, 41220,
                32908, 2664,  49092, 57720, 10012, 46892, 920,   23804, 10536,
                47244, 54536, 18780, 24100, 61176, 4900,  48248, 16520, 30256,
                38052, 7232,  57896, 34016, 25656, 61000, 28612, 16224, 12628,
                64544, 53936, 6404,  34756, 42120, 4424,  62296, 1012,  15624,
                51612, 21668, 4092,  49388, 57320, 28100, 33272, 13044, 54880,
                38260, 10444, 46076, 22180, 64972, 50892, 34788, 3996,  31660,
                11004, 49512, 61344, 42808, 51272, 11956, 39932, 1648,  60168,
                52208, 6884,  63048, 19524, 56756, 2196,  63500, 24924, 6684,
                42588, 32352, 59448, 19040, 40316, 15672, 55140, 37036, 21840,
                64592, 29260, 48368, 62000, 8924,  29864, 60400, 220,   47652,
                22808, 52016, 9636,  14068, 24828, 53408, 16700, 4660,  51056,
                38952, 18008,
            },
            {
                34736, 65076, 22336, 16952, 27804, 428,   21328, 58872, 35272,
                15408, 65464, 7444,  29416, 54800, 41452, 8936,  57972, 12228,
                55196, 2048,  33424, 19552, 54064, 31620, 24540, 37804, 52664,
                9400,  28516, 63356, 39568, 30184, 33680, 14476, 51284, 26972,
                35260, 61968, 1340,  50916, 14628, 26664, 56548, 40680, 52284,
                64500, 18728, 11036, 22044, 48316, 41652, 6708,  57632, 33284,
                40404, 10460, 44564, 25904, 13948, 19312, 22968, 57284, 42616,
                7692,  46820, 31584, 15152, 10904, 53248, 7788,  48616, 60244,
                1488,  27648, 57300, 5840,  30732, 17192, 9620,  21436, 57956,
                36060, 5960,  25496, 14508, 34476, 28816, 46088, 20648, 16632,
                27916, 24536, 39236, 13644, 31772, 49020, 44824, 15708, 30108,
                61844, 39080, 13696, 35400, 46024, 62728, 25988, 12652, 656,
                39220, 18636, 13536, 4996,  38776, 23372, 18476, 40304, 12312,
                36352, 56776, 32332, 38668, 60664, 1260,  58088, 33316, 63728,
                13080, 45876,
            },
            {
                29020, 7656,  43728, 56936, 52592, 62824, 37652, 48072, 3068,
                43456, 26044, 49312, 2356,  38300, 16428, 33972, 44096, 29892,
                37380, 64480, 26068, 5260,  61372, 50480, 3988,  20872, 60956,
                46152, 18576, 22844, 12772, 4592,  61380, 20292, 43316, 58316,
                7268,  21712, 41740, 45464, 36084, 31868, 20432, 9860,  5436,
                24252, 55308, 36492, 2564,  14108, 37620, 19096, 49908, 25000,
                3092,  21300, 58164, 50068, 31808, 52508, 45844, 27488, 33600,
                12932, 63708, 26688, 36668, 61540, 23356, 40496, 16012, 30228,
                21296, 50016, 14180, 42852, 37648, 47736, 60596, 51892, 41724,
                18900, 54708, 38348, 21788, 57780, 5100,  62108, 54596, 37808,
                43528, 11028, 46568, 53812, 22392, 8508,  40428, 58204, 53492,
                140,   22012, 7940,  52440, 3772,  31100, 6864,  60468, 43848,
                52096, 57876, 26544, 44804, 50692, 31752, 54676, 58192, 26460,
                17160, 6220,  3520,  49460, 28444, 46304, 37028, 10404, 26516,
                20676, 1972,
            },
            {
                51580, 15168, 36136, 11132, 4872,  42332, 12508, 24248, 60672,
                9848,  52888, 32708, 21708, 12836, 25268, 63644, 1200,  23256,
                52064, 13812, 41756, 47092, 28988, 11676, 43980, 34708, 14216,
                56344, 6096,  42224, 53912, 44912, 55692, 37372, 2688,  16996,
                39492, 13280, 29116, 9108,  65232, 2612,  58816, 44264, 34684,
                27516, 42096, 50664, 59992, 29168, 53776, 63960, 4896,  30712,
                47192, 36432, 17736, 1988,  63844, 9048,  36208, 5600,  59068,
                20372, 38824, 55796, 64,    19408, 45592, 34880, 4032,  43812,
                64212, 35892, 18948, 61824, 2872,  23820, 28792, 14044, 1332,
                27200, 47380, 65336, 2508,  44228, 18292, 9908,  31548, 4056,
                64608, 56320, 624,   60772, 28184, 5020,  34084, 18904, 10712,
                27704, 47720, 64024, 24900, 57248, 21028, 48652, 17392, 33172,
                9536,  23704, 35752, 1956,  59800, 14540, 7244,  34528, 48948,
                61916, 44064, 20244, 64664, 11808, 18772, 7300,  48268, 55464,
                42684, 58852,
            },
            {
                24656, 47852, 60272, 31572, 25796, 18416, 33632, 6832,  29636,
                38880, 16868, 41720, 59892, 56028, 46996, 50876, 6088,  60484,
                18348, 7688,  59292, 22584, 16784, 39512, 58236, 26280, 2144,
                30080, 51156, 35524, 25308, 16140, 8404,  27952, 64348, 47672,
                31676, 56872, 49420, 18084, 53244, 38860, 12952, 48868, 61452,
                368,   14948, 31536, 8164,  44756, 23324, 11404, 43488, 61228,
                53372, 13752, 60112, 29036, 39164, 12356, 54352, 17344, 49476,
                3280,  44544, 9384,  50472, 29424, 6344,  59332, 51220, 24432,
                9840,  6732,  53220, 31900, 11752, 55736, 7740,  45700, 62916,
                31176, 12284, 8324,  32440, 53164, 39296, 26244, 47660, 36000,
                15156, 21480, 33720, 17072, 37268, 52216, 65468, 24180, 49868,
                37768, 17088, 42976, 33844, 10980, 41612, 37572, 28124, 56100,
                3624,  47476, 63592, 11792, 41688, 21224, 46272, 3144,  10420,
                29632, 39924, 54400, 24164, 35148, 41884, 60180, 22644, 31880,
                5372,  38156,
            },
            {
                9480,  19936, 3156,  40028, 46864, 64424, 57388, 49640, 55136,
                20620, 62548, 5732,  36096, 9292,  19272, 28492, 40308, 35600,
                44952, 32328, 49704, 36672, 760,   54748, 7980,  63552, 48536,
                38788, 11104, 64796, 516,   32812, 49788, 22032, 10820, 53120,
                24596, 3584,  60380, 26168, 6364,  23208, 29448, 17008, 54236,
                21428, 46216, 63168, 18060, 57160, 1716,  35496, 16040, 27264,
                8648,  41568, 23280, 5228,  44092, 25188, 61240, 41252, 29840,
                62628, 25032, 16480, 41888, 64776, 12848, 54500, 18112, 32504,
                58224, 46980, 39356, 25396, 58744, 40828, 33116, 53040, 17540,
                39460, 56896, 24532, 15876, 49116, 60796, 13300, 23224, 51536,
                8036,  30356, 49892, 42836, 9944,  46100, 14324, 2616,  43932,
                59888, 4288,  29152, 54224, 15108, 1212,  65104, 51384, 13688,
                40924, 30652, 19980, 54056, 27680, 37400, 65472, 25152, 53592,
                14928, 33340, 8272,  57768, 15632, 3100,  29336, 50464, 13964,
                17424, 62500,
            },
            {
                41500, 55904, 29808, 53684, 21856, 8916,  15788, 44324, 2400,
                13648, 46604, 27468, 204,   43204, 32056, 14792, 57220, 11144,
                26432, 2800,  56152, 10204, 30528, 50936, 19284, 33152, 23264,
                16680, 57432, 20052, 46012, 61664, 40420, 58760, 34932, 6020,
                44516, 36848, 11376, 34104, 46276, 62920, 42676, 7684,  33232,
                10680, 38216, 4244,  26488, 40204, 32696, 52552, 20696, 38644,
                144,   55328, 50420, 34100, 47560, 15500, 888,   32892, 22548,
                11492, 35676, 53724, 32016, 22048, 38380, 27076, 2296,  42044,
                13760, 28304, 576,   16332, 49028, 20524, 4656,  35976, 23080,
                3580,  50732, 42436, 36648, 28256, 6076,  55948, 2788,  59208,
                41816, 61900, 4684,  25588, 57912, 19652, 30536, 55428, 35328,
                13156, 51480, 7280,  61420, 19952, 45488, 25704, 8400,  22516,
                60156, 16156, 5836,  33616, 9020,  51840, 17460, 43100, 59228,
                22064, 752,   47012, 27180, 51664, 44776, 63588, 37616, 176,
                52924, 33548,
            },
            {
                12532, 6516,  61508, 14272, 1144,  38100, 28356, 36056, 23940,
                31428, 51160, 58068, 23352, 64836, 52656, 3848,  21844, 47580,
                61992, 14360, 20580, 65444, 24604, 41056, 13504, 45860, 4268,
                41844, 27780, 7760,  30600, 12692, 4460,  17908, 29856, 14532,
                63308, 19044, 40952, 55420, 14820, 1432,  51396, 36904, 58080,
                24012, 50296, 55712, 13380, 48856, 9388,  59272, 45800, 63260,
                31876, 18740, 10884, 65440, 20556, 58296, 51064, 8484,  56744,
                48304, 6048,  58520, 1668,  45916, 8384,  48168, 63060, 36832,
                55656, 21644, 62584, 43544, 9156,  64864, 14916, 44692, 61556,
                10868, 59616, 19700, 20,    63424, 21112, 33808, 45332, 17412,
                26892, 38564, 13596, 53708, 1884,  39900, 61256, 26616, 9036,
                21084, 32040, 23880, 39724, 49068, 57828, 34960, 4800,  44136,
                36960, 49628, 57396, 45656, 60744, 1356,  31272, 6120,  35544,
                49808, 63876, 12624, 38196, 5848,  19984, 10916, 25352, 57676,
                45476, 27484,
            },
            {
                49552, 23660, 35344, 45104, 50116, 58924, 53136, 5312,  63412,
                10360, 34772, 18176, 12492, 48440, 37408, 60108, 7960,  54212,
                29668, 37984, 42872, 46468, 3564,  58680, 35536, 62460, 10304,
                60500, 50100, 54904, 37180, 23060, 52824, 47072, 42092, 57400,
                23340, 2220,  50544, 30132, 21308, 59124, 26656, 18892, 5164,
                62516, 28104, 43004, 20012, 65096, 23784, 5172,  14040, 48312,
                24668, 57212, 36792, 29568, 6848,  27112, 37668, 18712, 45084,
                39944, 14624, 26424, 19964, 60720, 34112, 16200, 23476, 10604,
                50152, 6420,  34832, 30612, 52108, 37712, 26588, 54812, 29444,
                7004,  34176, 46924, 12752, 43868, 9492,  40368, 29508, 54528,
                10168, 20424, 47056, 64032, 22776, 33504, 6652,  48788, 41936,
                63036, 56612, 36148, 2424,  9968,  30812, 17492, 53168, 62456,
                28500, 3176,  25312, 38864, 14736, 23788, 48264, 11668, 40848,
                28620, 18368, 42344, 59840, 31080, 54008, 34308, 41136, 8692,
                18964, 64068,
            },
            {
                1836,  43108, 17312, 26644, 32208, 11384, 19008, 43308, 40088,
                56740, 44944, 7292,  39628, 29156, 16528, 24696, 33964, 41528,
                17760, 5208,  52732, 32792, 16304, 28176, 6692,  53144, 25696,
                31828, 1960,  14832, 43300, 59996, 26456, 1272,  38108, 8116,
                47928, 26968, 60940, 5476,  39240, 9196,  32216, 48092, 41372,
                14708, 1088,  35164, 6696,  30972, 37748, 42264, 29232, 3480,
                8012,  43672, 14952, 2560,  54164, 42300, 12536, 61544, 2808,
                28780, 63932, 50908, 40684, 12240, 52556, 56720, 4700,  31184,
                45020, 17184, 59216, 3960,  22524, 12300, 1916,  48472, 18648,
                40996, 25468, 54192, 30640, 58104, 51168, 15604, 65036, 1024,
                49696, 35716, 7180,  28844, 44132, 11868, 16336, 53380, 3128,
                18028, 46876, 14836, 43612, 26532, 63524, 13112, 39300, 20440,
                9676,  32416, 12400, 19384, 43628, 55312, 61940, 20788, 57920,
                4600,  52836, 24848, 2476,  16788, 62080, 22500, 48520, 4540,
                31232, 36968,
            },
            {
                59240, 54716, 8576,  64840, 4320,  22712, 60780, 26256, 16376,
                1848,  21960, 61456, 55012, 2828,  10184, 45276, 50964, 1064,
                63980, 25668, 8660,  61312, 11700, 55412, 20832, 44020, 17240,
                39452, 21656, 47756, 5952,  33440, 10048, 65364, 20512, 54284,
                32560, 10688, 45136, 17292, 53100, 43760, 64892, 11672, 56492,
                29812, 47096, 53816, 11388, 57952, 50848, 17832, 54964, 34500,
                60840, 49828, 39484, 59472, 46592, 32296, 24056, 52720, 34212,
                21604, 36900, 4084,  30928, 6900,  43188, 28396, 39060, 61020,
                20168, 40620, 54184, 27476, 46328, 60832, 42368, 32320, 57764,
                13852, 62488, 4940,  22252, 35844, 26084, 6828,  37752, 23960,
                60384, 32532, 57048, 17764, 51672, 36448, 58884, 24284, 38600,
                28660, 10924, 60712, 54952, 6132,  50856, 33444, 1832,  47548,
                59036, 54396, 50528, 63656, 6932,  34192, 27304, 37608, 15808,
                9040,  32964, 56752, 36452, 46556, 7084,  13564, 56060, 60784,
                15540, 21596,
            },
            {
                11764, 29132, 38272, 51940, 40844, 47432, 34380, 7752,  48936,
                29908, 52088, 33004, 25980, 42380, 62968, 31096, 20468, 58000,
                12472, 36120, 49760, 22836, 39852, 48296, 36896, 3260,  50900,
                56808, 35048, 63904, 18516, 56012, 14032, 50392, 28720, 16500,
                62136, 37464, 57000, 33904, 25276, 14112, 2880,  23064, 38332,
                17600, 61708, 21696, 44476, 15360, 1752,  26908, 62556, 21368,
                12216, 26436, 17972, 22476, 9596,  62828, 16104, 6204,  55956,
                10472, 17356, 57492, 47344, 24680, 19120, 65276, 880,   51600,
                25900, 12980, 8604,  36276, 15848, 56208, 5736,  23864, 9252,
                51216, 38248, 17228, 49768, 2572,  61744, 19216, 46660, 12528,
                43256, 15272, 3616,  41468, 62412, 1180,  45512, 31024, 64820,
                5816,  22900, 40136, 30068, 18536, 22188, 56824, 41436, 24452,
                16604, 35888, 44,    41024, 29936, 10164, 2940,  51032, 46416,
                62956, 44124, 21392, 10248, 50788, 29124, 43192, 38476, 26300,
                51788, 47228,
            },
            {
                33896, 5652,  20420, 14984, 57756, 364,   13024, 54292, 63944,
                38368, 14416, 5584,  18464, 50048, 56504, 6716,  15124, 38956,
                28436, 44268, 19004, 57008, 700,   30996, 63744, 13908, 29212,
                7384,  11512, 25140, 41140, 30172, 45372, 36400, 3300,  43212,
                5984,  13632, 22300, 148,   61640, 36416, 49920, 54260, 4640,
                33640, 8680,  25896, 39660, 60160, 36260, 9712,  44988, 40484,
                52276, 31372, 5080,  35456, 51208, 1148,  40636, 48808, 25812,
                46144, 42536, 61040, 14120, 54228, 36044, 9452,  15256, 32688,
                47280, 3452,  64360, 49564, 30940, 19916, 39180, 63760, 34664,
                1416,  28172, 45708, 10800, 42268, 56204, 33264, 52920, 4920,
                30308, 53940, 25104, 10728, 27376, 21024, 8712,  13808, 48156,
                34632, 52716, 464,   48728, 37264, 45004, 11088, 60200, 7448,
                44488, 26840, 21444, 46900, 59364, 17604, 54020, 40412, 25212,
                1056,  30120, 14144, 39876, 63540, 24612, 876,   32672, 9572,
                3680,  40324,
            },
            {
                62476, 56924, 49008, 24960, 30364, 36508, 45292, 27848, 20560,
                9512,  58788, 47120, 36200, 12156, 22204, 34884, 46596, 53976,
                3708,  60768, 33412, 6520,  26824, 42932, 8876,  24052, 58408,
                45260, 60580, 444,   49520, 4852,  21812, 58196, 24716, 52560,
                30672, 49016, 41668, 8788,  46476, 30540, 20064, 27792, 45376,
                57628, 49308, 2736,  32340, 51632, 22888, 48128, 6484,  16664,
                840,   56020, 64700, 44652, 27316, 20008, 29968, 37440, 64520,
                2244,  29220, 8276,  33372, 3120,  49664, 44408, 22532, 59788,
                37188, 55732, 23956, 41964, 324,   10652, 52856, 14512, 47176,
                21284, 55156, 64784, 31316, 14988, 8072,  28280, 40824, 58928,
                21776, 64152, 47384, 38872, 50188, 33184, 54412, 57464, 19072,
                42460, 59428, 16256, 9132,  63936, 4064,  28252, 15136, 31416,
                65052, 5416,  56616, 14100, 37232, 23460, 64616, 12136, 19292,
                35452, 60376, 51968, 5056,  17320, 57808, 20080, 53644, 65268,
                44300, 23944,
            },
            {
                13464, 2248,  42528, 9896,  63136, 17748, 59724, 4696,  33580,
                43880, 24016, 844,   64640, 40764, 27660, 2496,  62056, 24300,
                9368,  52204, 14024, 47272, 17124, 59852, 54240, 18828, 40360,
                32536, 16160, 37884, 54040, 63072, 8356,  40156, 11892, 59820,
                19572, 64548, 26648, 55560, 17932, 59052, 6984,  63416, 10832,
                41048, 14220, 64672, 18244, 4856,  28852, 63796, 32932, 58348,
                37348, 10968, 41600, 14084, 60048, 55120, 11200, 58252, 14560,
                19424, 51932, 23196, 39280, 63296, 27024, 57796, 40840, 11576,
                6824,  29052, 17512, 33588, 61336, 45088, 26096, 29888, 58896,
                42772, 6744,  24808, 36284, 59732, 48172, 23004, 11052, 17496,
                1700,  36764, 7528,  18672, 61060, 5480,  40628, 25756, 2924,
                11448, 27036, 32444, 56064, 24908, 34812, 53896, 49156, 38676,
                52752, 11340, 29236, 51364, 3612,  44780, 31508, 5876,  48556,
                56184, 8168,  26580, 47444, 34536, 45232, 11840, 36248, 16272,
                28572, 18864,
            },
            {
                46228, 36684, 32596, 55368, 6136,  22352, 41432, 50384, 16084,
                55568, 39508, 31560, 52636, 8656,  59172, 49440, 17420, 42320,
                31460, 21472, 40172, 64716, 36536, 11884, 33792, 49108, 2480,
                52324, 21452, 26788, 12824, 34232, 28372, 16772, 46516, 1152,
                35256, 39144, 5036,  51320, 12304, 38048, 42804, 16060, 35572,
                23420, 30192, 53096, 43796, 11196, 55340, 13928, 20024, 25532,
                49248, 29388, 22992, 7480,  47328, 3828,  33080, 44076, 7352,
                35188, 41420, 56272, 11296, 20672, 16252, 5672,  30360, 53200,
                45880, 62800, 13832, 51656, 22132, 57860, 7668,  37044, 3328,
                18088, 12696, 40540, 19880, 704,   52268, 39212, 62908, 44688,
                51020, 33848, 14000, 58416, 29616, 44880, 16092, 36340, 51296,
                61696, 46388, 20976, 41104, 13088, 60992, 19680, 1036,  22708,
                34208, 17912, 40448, 60852, 8860,  34968, 58076, 28296, 42740,
                15608, 38092, 22492, 54660, 3568,  29760, 60540, 48828, 6504,
                58696, 53060,
            },
            {
                8044,  60692, 15508, 27416, 46644, 53348, 11524, 25776, 2620,
                60972, 12648, 19780, 25580, 15004, 44460, 30244, 11460, 37300,
                58444, 5112,  27944, 50356, 1868,  22560, 46116, 5756,  28024,
                65120, 9360,  44052, 47948, 19340, 61500, 51300, 32368, 54440,
                9624,  15292, 29256, 23712, 33296, 3728,  53680, 26564, 1276,
                60488, 5776,  20596, 36456, 59296, 41532, 34820, 46124, 2964,
                61036, 18908, 52736, 34456, 38868, 17012, 21896, 53116, 25172,
                61332, 48952, 240,   32104, 60012, 47164, 35068, 1396,  18160,
                21504, 42796, 2332,  36472, 5012,  40468, 16916, 50524, 62940,
                33136, 48872, 60944, 45216, 5092,  26900, 16016, 6200,  31284,
                26052, 56432, 43328, 23884, 132,   12188, 63920, 22680, 30704,
                7892,  37484, 1776,  51756, 5872,  47032, 42564, 8592,  62632,
                44008, 2444,  48332, 25112, 19796, 14980, 50540, 21484, 62200,
                2864,  31664, 12372, 64280, 40672, 8980,  23540, 42052, 1980,
                38628, 30944,
            },
            {
                25716, 21624, 50888, 39716, 3148,  31332, 38124, 65476, 45032,
                29076, 7068,  48028, 57200, 3972,  34348, 20548, 64236, 600,
                54996, 44828, 18460, 10064, 57064, 26148, 38964, 61748, 14364,
                36744, 55116, 4364,  58308, 2104,  42444, 6524,  25544, 22152,
                44128, 63064, 48480, 57832, 44708, 62564, 20888, 50400, 46184,
                55980, 38876, 47908, 26836, 236,   23852, 7040,  53488, 39172,
                9216,  43644, 4580,  62228, 24812, 50652, 63480, 2516,  30408,
                12676, 17404, 27948, 44916, 7928,  52056, 39556, 55884, 60960,
                26604, 49084, 10016, 55000, 27852, 47772, 12276, 23752, 55816,
                27352, 9240,  53424, 29992, 34392, 57564, 37364, 54624, 48892,
                3768,  19832, 9708,  52148, 38116, 55112, 47456, 4204,  57980,
                43012, 15180, 65480, 28060, 17968, 37880, 30788, 25720, 55552,
                13240, 58148, 32224, 63392, 37972, 54316, 676,   10668, 39540,
                52900, 58396, 44376, 19548, 33168, 15852, 55748, 27384, 20692,
                50328, 11436,
            },
            {
                42768, 616,   64192, 19144, 57644, 14168, 8092,  34340, 17856,
                51900, 35932, 42360, 61792, 38144, 53756, 7472,  51040, 26388,
                13468, 34972, 61584, 32392, 42348, 15836, 51856, 8416,  19872,
                31916, 41248, 23980, 30336, 36112, 14440, 56512, 38200, 60512,
                4416,  18820, 40528, 1756,  9892,  14672, 30644, 7732,  33564,
                17488, 12876, 8588,  62180, 16596, 50012, 30740, 63140, 13176,
                56300, 32120, 15888, 57724, 28912, 10020, 41192, 36092, 46768,
                54908, 4728,  65080, 36460, 14640, 25784, 22724, 12340, 33236,
                7228,  38516, 31296, 59500, 19100, 65484, 32456, 1544,  43620,
                38440, 4316,  22648, 16824, 11940, 64512, 8768,  21136, 12940,
                60176, 40376, 65188, 32544, 27556, 17372, 34072, 9972,  19484,
                48820, 24504, 54876, 33204, 57716, 10408, 59780, 16212, 50192,
                28344, 21544, 8064,  4464,  43248, 27148, 47224, 33224, 24776,
                17448, 27996, 5764,  48336, 96,    51992, 62248, 13728, 34904,
                63660, 56532,
            },
            {
                33332, 45544, 7320,  35392, 24364, 47768, 54484, 21196, 58584,
                10308, 23192, 1968,  17048, 28360, 22704, 45940, 39416, 16892,
                48468, 23156, 6928,  53400, 3488,  59636, 30160, 44552, 56336,
                49396, 11820, 16964, 63404, 50580, 20632, 10340, 47776, 28880,
                13228, 56036, 32160, 22552, 36244, 60344, 40248, 24708, 64336,
                42584, 28292, 52392, 32524, 57036, 10624, 44068, 18068, 28108,
                22172, 36572, 48012, 544,   13440, 45768, 6960,  19884, 59316,
                38780, 23532, 42620, 19320, 58028, 3508,  63184, 43552, 50784,
                15928, 63888, 23360, 14652, 44892, 35576, 8896,  52724, 20076,
                14348, 61672, 56820, 40208, 50124, 46428, 24500, 42948, 29280,
                35492, 16152, 6572,  45992, 3412,  61936, 41836, 52468, 29004,
                35280, 12152, 3504,  44368, 23068, 48292, 2784,  34544, 4960,
                41620, 36988, 46836, 16124, 56496, 12464, 59460, 6148,  65356,
                49824, 9268,  35884, 61084, 24936, 37796, 43132, 7012,  46508,
                4036,  17444,
            },
            {
                13424, 29656, 59608, 10992, 41548, 62280, 28512, 824,   40284,
                48972, 63748, 32876, 55164, 13140, 9928,  59440, 32760, 3596,
                62996, 40984, 29072, 47084, 12700, 21040, 35184, 180,   25440,
                5524,  60052, 45948, 8048,  26740, 40120, 62116, 2956,  34032,
                50176, 8000,  26492, 51680, 17112, 54404, 47744, 11720, 4768,
                57308, 22428, 2328,  45504, 39820, 21008, 36948, 2012,  51204,
                5048,  65324, 26368, 42236, 53636, 31384, 56704, 15792, 27172,
                10872, 7580,  49916, 54088, 31164, 48100, 9628,  29552, 4384,
                57188, 41168, 2776,  52228, 5704,  25224, 41516, 59152, 30284,
                46544, 35828, 26260, 6992,  19596, 2752,  33092, 62088, 1076,
                47764, 57680, 23240, 50136, 20448, 13496, 24968, 58880, 6452,
                63280, 40020, 50828, 7628,  14104, 62772, 39680, 20304, 54172,
                11000, 64848, 52500, 23644, 30144, 40320, 18756, 36368, 44800,
                14352, 41256, 21068, 56604, 29452, 10396, 18748, 31888, 59140,
                24216, 39268,
            },
            {
                48056, 55184, 26292, 51544, 3764,  16448, 43972, 6264,  31040,
                14704, 26640, 5144,  46972, 40944, 62612, 5812,  29536, 56448,
                10784, 19712, 57672, 37552, 24772, 62340, 40760, 17472, 64248,
                38164, 21460, 34136, 1476,  53784, 31532, 45420, 23408, 16628,
                37160, 43232, 59628, 45928, 6456,  28448, 592,   19792, 51400,
                34512, 38056, 15288, 63864, 5284,  25864, 60912, 55292, 46324,
                40336, 11048, 59700, 17752, 21508, 34984, 64128, 1508,  52132,
                33568, 60820, 28540, 2136,  12724, 40592, 37152, 18796, 46688,
                21048, 35304, 28632, 11760, 48852, 56284, 15644, 3628,  63520,
                11248, 28,    51488, 44224, 30384, 58956, 38152, 17768, 52032,
                26804, 11568, 38816, 30748, 55556, 8128,  45348, 38960, 764,
                15920, 56460, 21140, 29872, 36332, 52352, 27372, 45808, 60804,
                31012, 18356, 1004,  35024, 9228,  62012, 3408,  22632, 55592,
                31444, 1720,  51532, 12740, 46256, 4984,  54868, 49864, 28200,
                9656,  53224,
            },
            {
                21884, 4412,  18492, 38224, 32420, 22452, 59328, 52524, 36516,
                55996, 44728, 20376, 51748, 35756, 18548, 24832, 42992, 52256,
                36100, 1096,  15296, 51136, 9232,  4392,  46092, 53528, 10216,
                27932, 52520, 42936, 57688, 12600, 18372, 6008,  58560, 52112,
                65512, 20312, 3548,  12672, 63532, 37896, 58404, 31260, 45108,
                60144, 9660,  48964, 29920, 53336, 13580, 8268,  33708, 14740,
                24340, 32808, 7844,  49316, 3724,  39992, 24372, 48492, 44404,
                20708, 40872, 16384, 35160, 22080, 59556, 52644, 26696, 60800,
                8016,  54428, 43408, 62292, 31804, 20480, 39136, 27908, 49592,
                23180, 32744, 15900, 64056, 10424, 53740, 14368, 5276,  56016,
                8256,  44056, 62900, 2320,  36512, 60848, 33292, 21940, 27596,
                46452, 32404, 61468, 42844, 17264, 1244,  9796,  24000, 7140,
                14372, 49436, 26912, 58848, 44412, 48668, 52704, 28964, 7756,
                59932, 24376, 63176, 33268, 16276, 39848, 65012, 20924, 36852,
                1192,  62984,
            },
            {
                33860, 8180,  43200, 64936, 12988, 50064, 9352,  25024, 18968,
                11920, 60248, 8548,  30464, 152,   57924, 48352, 13796, 21684,
                45664, 27032, 65516, 42720, 33732, 28488, 56660, 14152, 32412,
                48260, 4760,  15132, 24600, 64452, 36272, 42296, 27708, 292,
                10988, 30208, 41480, 25288, 34120, 21732, 42364, 14224, 26444,
                17944, 3924,  24860, 19080, 35232, 41096, 48224, 28860, 58352,
                19456, 44172, 55616, 37176, 61408, 11972, 6268,  29956, 13880,
                63004, 4840,  56784, 44876, 63792, 6556,  15384, 404,   33928,
                12880, 24984, 17428, 1344,  36856, 58632, 7516,  44140, 17996,
                54756, 41188, 57416, 36932, 23696, 27604, 42140, 46948, 22340,
                34644, 19532, 14900, 51800, 26384, 17120, 11168, 54332, 49980,
                19108, 10696, 3956,  25536, 55376, 63824, 32244, 57984, 43820,
                38360, 55964, 40884, 5236,  20316, 32988, 13056, 39396, 16808,
                47100, 36916, 4404,  43296, 58488, 26288, 2984,  12272, 45124,
                16792, 40988,
            },
            {
                14088, 61104, 28332, 56148, 1240,  46080, 34364, 63596, 42728,
                2044,  38096, 24004, 64896, 15460, 27644, 4172,  38420, 8124,
                54784, 31932, 6316,  59112, 18352, 49612, 22536, 7164,  36604,
                59508, 19536, 39160, 29304, 47600, 8772,  56168, 14596, 48376,
                33420, 54592, 15924, 56920, 49524, 9744,  53620, 7144,  64844,
                39904, 55596, 43268, 61720, 57636, 660,   22576, 64420, 5680,
                52576, 1540,  29824, 15172, 27000, 57228, 42876, 53904, 8796,
                36664, 25772, 47728, 10128, 24464, 32824, 49528, 56384, 44632,
                64720, 38764, 48196, 9556,  23572, 52616, 13284, 34408, 60680,
                9340,  5068,  20564, 2944,  48684, 7776,  60872, 31476, 64804,
                40268, 29804, 59204, 6372,  41744, 48096, 4568,  65008, 41176,
                7368,  59884, 37420, 49216, 12460, 40564, 47336, 19264, 34108,
                2820,  22432, 11416, 62588, 15720, 25384, 1388,  64040, 54156,
                11008, 27720, 18556, 8892,  22120, 48204, 34560, 60292, 52104,
                30452, 57736,
            },
            {
                49964, 45616, 35916, 24144, 17260, 40092, 29500, 6868,  57304,
                49508, 33120, 53964, 40196, 44160, 50644, 33364, 63408, 59920,
                17280, 23860, 47840, 13160, 1812,  39020, 63372, 44152, 25244,
                1044,  62244, 10860, 51076, 2684,  32468, 19648, 39952, 62576,
                22168, 38288, 6072,  61160, 2812,  17720, 47208, 35876, 2152,
                50612, 13164, 30884, 7424,  11616, 51140, 16328, 42596, 10556,
                35424, 62836, 21900, 51516, 46424, 18836, 33192, 22508, 59256,
                17880, 31576, 51408, 1592,  38624, 18508, 42116, 29916, 21512,
                3456,  51792, 57712, 29160, 45756, 64376, 5632,  26192, 47676,
                37912, 29480, 62600, 43700, 13384, 35072, 19168, 4224,  11336,
                50212, 352,   46464, 23948, 31884, 58140, 37976, 29648, 14700,
                34808, 53016, 44872, 22812, 6664,  28960, 15372, 4980,  51620,
                64428, 29192, 46448, 36780, 50744, 57476, 42576, 34508, 20840,
                44244, 57728, 50164, 38356, 53428, 30924, 7456,  42088, 23284,
                5424,  25556,
            },
            {
                1736,  19616, 11020, 6248,  54100, 60536, 20864, 14280, 26792,
                17732, 3820,  13412, 21384, 9804,  55708, 19592, 11788, 42108,
                3064,  35636, 40260, 55628, 20804, 30684, 10716, 58292, 17140,
                41128, 30964, 55540, 43448, 22872, 61188, 53332, 25448, 4256,
                9468,  46832, 27292, 44396, 31952, 24396, 62376, 28196, 20740,
                33200, 46616, 22208, 37696, 27324, 45964, 31724, 38400, 26376,
                47352, 13308, 39504, 4328,  9484,  64868, 0,     49000, 38528,
                3320,  55728, 13196, 29048, 59180, 53236, 11704, 8580,  37376,
                14888, 32108, 5140,  19632, 15208, 41748, 30912, 2576,  53568,
                14376, 24628, 50200, 32116, 56620, 53264, 38976, 58332, 15936,
                25428, 54472, 9408,  63820, 13184, 19724, 1308,  23108, 56676,
                26132, 2148,  31148, 19864, 57700, 53344, 37732, 59756, 26236,
                8832,  16884, 54248, 31452, 6348,  9780,  27364, 49604, 5916,
                2464,  30328, 61528, 13132, 476,   63504, 15320, 18824, 55684,
                9816,  38904,
            },
            {
                53044, 58700, 31196, 48752, 37636, 3392,  47872, 53324, 40792,
                61080, 29896, 47656, 62048, 6416,  37100, 25232, 30636, 49336,
                53176, 28700, 10028, 61180, 25732, 52340, 34080, 3556,  54320,
                49144, 21720, 5908,  15772, 37496, 7196,  12416, 44208, 35712,
                58996, 53024, 19308, 13800, 40456, 55396, 11904, 38948, 58464,
                9328,  61120, 53820, 2468,  63300, 20100, 56556, 3192,  60688,
                18304, 57928, 32132, 24820, 42252, 36116, 28380, 15544, 61680,
                45360, 41404, 19892, 65136, 43580, 4004,  27188, 57496, 47492,
                62492, 26056, 40804, 60532, 35644, 50460, 21880, 61772, 18884,
                45404, 59488, 11076, 17528, 26660, 1632,  23120, 45632, 29044,
                37404, 43124, 17432, 35104, 53048, 44492, 50572, 9076,  43136,
                61068, 11428, 15892, 64508, 33872, 88,    23772, 11852, 48428,
                35580, 41592, 848,   21128, 59548, 38480, 17952, 55700, 23248,
                40640, 15572, 34924, 24956, 45340, 56836, 26936, 37064, 47308,
                64532, 33212,
            },
            {
                16484, 22788, 41312, 63452, 27056, 12056, 32788, 8908,  35684,
                5588,  43556, 24844, 34436, 58112, 1448,  46344, 15236, 5932,
                21528, 64204, 16416, 45412, 7620,  41944, 14808, 46756, 27760,
                12012, 35444, 64092, 26328, 46708, 58636, 28904, 50248, 16588,
                30920, 1048,  63672, 36220, 7996,  50740, 1424,  45516, 16468,
                5028,  25640, 14408, 40776, 34028, 5320,  52288, 23832, 8868,
                50332, 43840, 7160,  55384, 60044, 12660, 53416, 6140,  24308,
                10772, 33744, 7308,  22312, 14960, 36128, 61308, 23692, 17400,
                43128, 7512,  12324, 54508, 640,   10372, 57216, 39448, 7968,
                35388, 972,   40240, 6596,  64488, 41768, 9616,  51500, 62428,
                3648,  56576, 21796, 39664, 6940,  25712, 62160, 32568, 18092,
                47264, 38588, 51640, 41440, 9120,  45624, 62336, 43156, 20568,
                56376, 61552, 12572, 24876, 44640, 63492, 13436, 46108, 32704,
                64768, 52768, 7072,  20240, 39700, 10764, 50940, 2268,  29384,
                12972, 43532,
            },
            {
                60928, 3592,  8236,  15328, 56812, 44604, 24756, 64496, 22284,
                51316, 56932, 11068, 15992, 28956, 52300, 62624, 39464, 58984,
                43636, 808,   33008, 50456, 4088,  36240, 64932, 19620, 6872,
                39696, 57592, 1680,  51964, 33800, 18664, 2180,  64960, 23556,
                41960, 11632, 49232, 21912, 29940, 59456, 23204, 34416, 52832,
                30064, 43596, 49236, 17896, 58604, 12232, 43444, 15196, 29928,
                34552, 976,   27988, 17360, 47488, 20592, 39920, 31084, 49996,
                57468, 26700, 47464, 54704, 30444, 49948, 6636,  34048, 1496,
                55436, 49264, 33628, 23056, 28032, 44288, 16292, 32284, 27068,
                55916, 22580, 52460, 33864, 46620, 14552, 30576, 20048, 7188,
                13900, 32396, 61116, 2236,  28936, 55848, 11980, 36384, 4228,
                54572, 28236, 3256,  24672, 49080, 17504, 30188, 4612,  32604,
                6916,  27832, 49148, 53160, 33892, 2088,  28884, 7784,  58764,
                11300, 26320, 47372, 59828, 4128,  32384, 43028, 61700, 21244,
                4988,  26448,
            },
            {
                37308, 46740, 51740, 34872, 19908, 284,   54372, 16672, 46212,
                2084,  19508, 37740, 49844, 40948, 22684, 8732,  32692, 26104,
                11400, 56312, 37996, 22936, 58588, 29084, 53868, 24132, 61348,
                32516, 44792, 20508, 13448, 42220, 10132, 39224, 54208, 6104,
                34648, 26208, 56004, 43356, 5508,  15300, 64408, 41472, 19296,
                57404, 36264, 65396, 7632,  28580, 47428, 36608, 62720, 40488,
                21280, 64484, 52772, 10300, 34040, 2840,  44720, 63788, 17728,
                3900,  37708, 62688, 672,   40004, 45856, 12812, 51872, 39028,
                29596, 20112, 65344, 46200, 37504, 4528,  63692, 48040, 12884,
                42396, 63112, 28596, 19432, 58132, 37088, 60364, 48420, 34816,
                44568, 26880, 10944, 49284, 45996, 15704, 41332, 22444, 64108,
                7956,  20884, 58504, 36140, 12996, 55504, 38808, 58324, 15012,
                45868, 18512, 37720, 4344,  16036, 40476, 51996, 20636, 37520,
                3264,  42420, 17332, 36296, 54432, 23400, 14316, 57960, 34780,
                49720, 55048,
            },
            {
                30524, 12196, 23916, 29436, 61448, 41876, 36772, 30196, 12704,
                61656, 31972, 7948,  65208, 4300,  18320, 13508, 54776, 36304,
                19744, 47132, 27544, 13600, 18152, 9684,  43624, 504,   50592,
                16724, 8992,  29588, 63012, 55772, 31328, 48320, 20988, 13744,
                46456, 61216, 3228,  18260, 38192, 46896, 27040, 8708,  3540,
                11712, 23656, 92,    32736, 22144, 55408, 25500, 6180,  53900,
                45884, 13764, 37640, 23328, 61412, 56520, 8728,  22652, 34936,
                12904, 51244, 16668, 9260,  25300, 18736, 64140, 21664, 58828,
                9776,  14308, 3340,  57992, 17688, 53948, 25120, 6300,  51404,
                3652,  15584, 8740,  49484, 2164,  25452, 5040,  53980, 23632,
                65160, 52068, 18264, 59516, 33432, 5224,  58668, 49700, 30692,
                14488, 45840, 32864, 62608, 5348,  27460, 22308, 1596,  51560,
                40280, 63808, 9000,  31124, 56120, 23976, 61840, 48840, 14936,
                30756, 50628, 62276, 28736, 8968,  48432, 18572, 6728,  40708,
                9428,  17820,
            },
            {
                1060,  64444, 39576, 5852,  48928, 10736, 7220,  57460, 39340,
                25920, 42540, 54124, 27224, 45748, 59388, 43084, 50084, 2360,
                63680, 7924,  52376, 60752, 39688, 48476, 31564, 13064, 37132,
                54828, 25472, 48836, 3824,  23100, 7468,  61892, 27796, 57936,
                38608, 9568,  52420, 33256, 57172, 12612, 51200, 32172, 60092,
                47888, 55128, 39712, 51924, 9552,  60908, 2336,  19236, 10880,
                58912, 4776,  49196, 41716, 30504, 15268, 27212, 53768, 42060,
                60628, 29284, 43924, 58440, 35420, 55880, 3104,  32432, 44672,
                25968, 42352, 51428, 31188, 11284, 41036, 34692, 21188, 60456,
                37824, 45696, 31768, 55424, 12596, 43692, 17704, 10608, 40700,
                712,   37208, 7560,  42076, 24484, 20104, 38756, 420,   25200,
                52260, 41696, 10760, 18996, 50132, 42944, 64992, 34220, 11080,
                25260, 53928, 21688, 59944, 42916, 11160, 5428,  34616, 44448,
                55176, 22240, 1040,  13200, 40176, 65456, 31492, 52860, 24588,
                45244, 59236,
            },
            {
                20624, 43244, 53692, 14608, 59156, 27668, 19208, 52080, 4784,
                49204, 15060, 21372, 648,   35476, 30800, 24188, 5072,  29356,
                16976, 40860, 32136, 5520,  2668,  63256, 20280, 58184, 41508,
                4928,  60208, 35660, 40688, 15252, 45256, 36680, 17672, 4936,
                31520, 16168, 22488, 28416, 63052, 928,   20716, 36736, 42356,
                28848, 13824, 19928, 45172, 16340, 42672, 33604, 39384, 27600,
                31980, 16960, 26172, 1816,  51324, 6412,  37920, 47860, 1380,
                20036, 5940,  32868, 22072, 11664, 41604, 28324, 8052,  16536,
                62724, 35696, 6540,  22344, 61880, 50004, 9128,  30316, 18080,
                57292, 24784, 35856, 21568, 39732, 63952, 33500, 29604, 56860,
                15844, 22004, 30992, 54864, 12540, 63252, 47148, 10076, 56804,
                37500, 61252, 2744,  29816, 54420, 8452,  15660, 47792, 59712,
                29136, 2324,  13884, 35472, 47340, 19496, 27392, 58100, 9332,
                25136, 6496,  34272, 55976, 45892, 26012, 1796,  38104, 62236,
                15116, 33180,
            },
            {
                56916, 27004, 3800,  34224, 22520, 43948, 63840, 33096, 23600,
                59984, 34404, 9916,  56700, 48076, 11656, 62932, 38180, 56128,
                45976, 21920, 57424, 24928, 44328, 34964, 27096, 7748,  22964,
                46444, 11520, 18568, 52640, 57660, 25692, 84,    51812, 42612,
                63964, 49800, 40732, 7284,  44084, 24748, 54856, 16096, 6272,
                63940, 25456, 5232,  62504, 29772, 50272, 12816, 56772, 51644,
                43252, 61768, 35788, 57800, 19064, 64272, 32236, 11016, 56948,
                25676, 46460, 62036, 52960, 4516,  49004, 60576, 53960, 47276,
                376,   56324, 39980, 27744, 14880, 1772,  56116, 46800, 41812,
                200,   10812, 62444, 6176,  50988, 27104, 46020, 59692, 4420,
                48088, 43576, 62464, 2592,  36032, 52968, 29528, 18144, 34184,
                6472,  16264, 23416, 44980, 36496, 25780, 40524, 20188, 7148,
                36940, 44256, 49884, 4672,  29932, 65200, 256,   38452, 18016,
                63152, 41776, 59696, 16760, 21012, 5368,  57260, 11952, 28676,
                4524,  48736,
            },
            {
                36484, 11116, 17980, 46432, 55316, 1456,  16088, 47360, 11844,
                2472,  43476, 63548, 39588, 16540, 6712,  20152, 51256, 14300,
                10244, 35596, 49424, 15532, 10876, 55348, 51248, 14876, 30096,
                64704, 33516, 2260,  28484, 61132, 9404,  33036, 55508, 12224,
                24428, 1876,  60128, 11880, 34264, 48744, 61400, 10032, 45980,
                53028, 33128, 38084, 58008, 3112,  36204, 21648, 64744, 604,
                7440,  20968, 9784,  46044, 12992, 43816, 22084, 59824, 16112,
                40132, 8764,  14616, 36864, 26760, 17516, 37924, 13016, 34280,
                24120, 18484, 48620, 59708, 45344, 36884, 26128, 64628, 13488,
                28784, 53300, 18792, 58540, 2756,  14996, 8212,  20208, 38664,
                13040, 27728, 50300, 8872,  26468, 15388, 3500,  43892, 65412,
                27768, 48228, 55720, 60124, 13292, 4472,  51180, 31600, 55272,
                17656, 61572, 23844, 56960, 16436, 40740, 53996, 45640, 32068,
                51576, 11800, 37660, 30052, 49292, 42648, 35508, 51764, 22116,
                41224, 8204,
            },
            {
                24176, 60300, 50776, 29740, 36876, 8756,  40876, 28788, 37932,
                54836, 18164, 29664, 22920, 52904, 60760, 41680, 33624, 25292,
                59220, 320,   65304, 30584, 18768, 37832, 61620, 1220,  43240,
                55908, 21404, 39100, 47608, 13328, 21736, 39756, 48712, 18992,
                29852, 45572, 36820, 53396, 18472, 4116,  30596, 40276, 22328,
                1840,  17544, 11188, 48252, 26768, 8224,  15748, 46140, 24544,
                38188, 48696, 29624, 54356, 25488, 39360, 2552,  49468, 36024,
                30556, 55360, 50172, 940,   63428, 46036, 22380, 5220,  30292,
                52160, 8600,  12064, 32192, 20852, 5456,  17076, 33220, 7124,
                49100, 38284, 43320, 30904, 47552, 35276, 52604, 24468, 54392,
                61052, 34496, 17380, 56972, 45484, 60372, 40300, 50840, 21156,
                12220, 39600, 1384,  33540, 21584, 63172, 57892, 436,   45920,
                12524, 41384, 33196, 8264,  51368, 12140, 24684, 6976,  14356,
                47704, 27620, 2232,  52952, 7828,  14524, 61276, 18072, 45584,
                64104, 53284,
            },
            {
                31872, 40100, 6428,  64812, 13336, 25108, 61724, 57568, 20908,
                50376, 8200,  46260, 3760,  32636, 27336, 1724,  45384, 7856,
                28828, 42552, 6012,  53944, 46824, 6624,  23824, 32820, 49112,
                17024, 9100,  51304, 5980,  31144, 44168, 65300, 3860,  35288,
                56648, 9212,  21004, 58152, 14432, 38760, 26708, 58904, 50836,
                35192, 56580, 44112, 23000, 53444, 59876, 41644, 31212, 11280,
                55672, 16232, 63464, 3404,  34532, 8348,  53432, 28088, 5708,
                65392, 23764, 42124, 20344, 31324, 9380,  58992, 40784, 64976,
                43460, 57852, 38148, 2680,  63460, 54664, 39772, 51128, 22360,
                59864, 15676, 25248, 9312,  21968, 41520, 61924, 1288,  31344,
                10352, 5360,  23576, 39016, 20504, 32612, 5712,  24736, 58320,
                8408,  30968, 50380, 18012, 43648, 10512, 38436, 27016, 22800,
                64460, 3448,  27764, 20388, 37324, 62704, 34316, 21592, 60880,
                4136,  20156, 64560, 24104, 58136, 33376, 26856, 9488,  30544,
                3220,  16356,
            },
            {
                44500, 400,   21412, 56764, 42856, 49764, 3888,  32008, 5688,
                26372, 60392, 12668, 37280, 49356, 14840, 57624, 54300, 18036,
                61608, 37072, 20492, 26400, 41088, 59940, 13400, 39828, 3964,
                27536, 58708, 34844, 61832, 16696, 57004, 26996, 7680,  15164,
                62832, 27464, 5300,  31720, 43284, 65084, 8376,  19628, 12636,
                28312, 61140, 7404,  39264, 18972, 29144, 4644,  49832, 61316,
                33784, 5516,  41064, 22764, 58328, 17620, 62512, 13244, 45268,
                18256, 4200,  11960, 34852, 43832, 55716, 15028, 25644, 3832,
                20192, 15692, 28072, 49872, 24996, 43216, 9960,  61692, 2532,
                45040, 34996, 4752,  65224, 56888, 12408, 17576, 37084, 44932,
                49676, 41736, 64816, 1640,  52356, 10172, 13836, 54760, 35064,
                46052, 61760, 26200, 5160,  53140, 29952, 47588, 16556, 34828,
                9420,  52944, 58704, 43864, 5544,  47116, 57812, 28968, 41132,
                56392, 35832, 44036, 11044, 39784, 46640, 1500,  50636, 38916,
                58552, 12328,
            },
        },
        {
            {
                47820, 13128, 52152, 5444,  39012, 42332, 26208, 36280, 44640,
                27552, 49844, 620,   38608, 52720, 3380,  46784, 65180, 7008,
                60788, 27728, 48784, 41120, 24280, 11912, 35388, 15588, 41592,
                28456, 11560, 2952,  48864, 42704, 58748, 26644, 39964, 30224,
                13308, 49592, 60340, 19912, 4452,  58000, 50308, 7616,  29220,
                57308, 50976, 38676, 3784,  55344, 33652, 42504, 396,   31616,
                4528,  27584, 50472, 63524, 42548, 59448, 52276, 22932, 63988,
                29720, 38720, 21120, 62224, 44224, 56396, 28672, 6656,  17824,
                54076, 8780,  47612, 59068, 16816, 52236, 41440, 22524, 45328,
                30892, 18276, 8248,  24164, 12560, 6456,  40236, 25612, 9888,
                49304, 58208, 28532, 52692, 39192, 8096,  14424, 29980, 6256,
                20672, 13032, 45164, 9392,  57736, 5440,  35832, 45964, 17840,
                12328, 43876, 22660, 32860, 51552, 20100, 26596, 45228, 52464,
                40180, 2948,  43392, 12620, 56232, 60184, 47080, 31040, 43036,
                23540, 1808,
            },
            {
                32956, 57216, 25104, 20444, 30416, 62684, 8180,  19820, 3524,
                64236, 34392, 20796, 59568, 43856, 13820, 19260, 36880, 11652,
                52384, 32584, 32,    59420, 29708, 39260, 64108, 49416, 32528,
                21340, 36464, 56808, 24104, 16960, 1988,  33940, 47656, 64096,
                53240, 17300, 9596,  38920, 29976, 13708, 25900, 53728, 16496,
                2340,  64864, 26668, 48560, 29840, 6528,  21048, 52504, 14232,
                23452, 55116, 6632,  21820, 3612,  8972,  35116, 45912, 2200,
                15804, 11536, 50416, 25028, 8904,  19188, 64384, 40916, 25240,
                43484, 34052, 3188,  23504, 31684, 37680, 9096,  54364, 35332,
                11988, 48828, 41280, 51296, 20480, 32644, 52320, 14596, 56688,
                37596, 2460,  5680,  13084, 24876, 50052, 61468, 34792, 53820,
                436,   64128, 32804, 19320, 39272, 48900, 14036, 61404, 51644,
                37600, 5316,  48580, 7776,  42408, 800,   61520, 34224, 4652,
                57944, 17280, 29252, 7888,  39592, 25772, 2620,  11372, 63032,
                54644, 27152,
            },
            {
                37476, 16228, 44304, 232,   48800, 58580, 16360, 51492, 56732,
                12396, 41580, 16940, 5332,  32720, 26560, 50800, 29616, 45368,
                23300, 17784, 54380, 14728, 7476,  20500, 56204, 10000, 916,
                60300, 51872, 14312, 39624, 62092, 51160, 11808, 18528, 6048,
                42888, 32756, 3108,  47144, 64432, 41948, 10732, 59860, 44876,
                20632, 36196, 14060, 18496, 60868, 45476, 58820, 37732, 65532,
                41312, 33144, 17600, 45172, 38260, 55912, 12416, 26000, 41648,
                57040, 32496, 46444, 3296,  52432, 31216, 11464, 47024, 4324,
                60152, 13772, 40020, 55512, 45960, 912,   63416, 25596, 6368,
                60316, 27344, 480,   63652, 44212, 4548,  61152, 28288, 20904,
                41824, 17724, 34976, 59976, 42708, 4236,  17880, 38168, 23336,
                48524, 40720, 27564, 51432, 3420,  25120, 31752, 41156, 2820,
                29600, 58964, 62352, 36392, 54692, 16336, 56712, 9020,  24072,
                37028, 61008, 49196, 53736, 33044, 15372, 51284, 34044, 39352,
                50432, 6624,
            },
            {
                59540, 9736,  64804, 35348, 11876, 22648, 46612, 38176, 30764,
                23760, 7732,  47320, 62912, 55028, 8944,  61544, 1784,  56476,
                5416,  42228, 37504, 63000, 44764, 4268,  27320, 46276, 17792,
                42876, 26180, 4792,  31728, 8644,  28640, 44776, 60092, 25244,
                37320, 57388, 27348, 51440, 22612, 35364, 592,   37784, 31600,
                6328,  42928, 53076, 9048,  40292, 12800, 26864, 17008, 8860,
                46776, 11756, 60884, 52804, 27944, 31760, 18448, 60780, 51424,
                8268,  37204, 17964, 59544, 40288, 35336, 22376, 57576, 16276,
                49944, 29296, 19108, 61580, 11896, 28440, 15712, 47836, 19212,
                38512, 55640, 14832, 29880, 58020, 17396, 36100, 45644, 7784,
                65200, 47484, 55192, 21964, 31148, 45612, 57580, 28176, 8928,
                15564, 56228, 6996,  61096, 16644, 58800, 10336, 21928, 56508,
                25760, 15584, 20452, 10804, 25168, 46684, 29384, 50216, 13928,
                44356, 26936, 19856, 72,    64584, 22236, 58276, 18404, 3844,
                14296, 21112,
            },
            {
                42176, 49816, 28316, 53188, 40912, 5212,  26948, 1604,  43560,
                60524, 53344, 28652, 36180, 21812, 42588, 16456, 39144, 33628,
                60244, 26456, 10848, 50444, 31236, 34872, 53060, 23660, 62380,
                7672,  58500, 48200, 20188, 54576, 35776, 22064, 55608, 1176,
                10580, 20576, 61216, 7092,  15504, 55376, 18556, 62540, 49332,
                22980, 61608, 33208, 24652, 1344,  35180, 49740, 3520,  29124,
                57952, 36048, 25400, 760,   15160, 63036, 47928, 5784,  21460,
                28648, 64796, 5456,  27132, 14444, 54664, 788,   38144, 32484,
                9508,  53572, 35864, 7156,  42920, 51088, 33612, 58664, 43556,
                3148,  22172, 34688, 8784,  24756, 39292, 2672,  50352, 11696,
                30748, 26268, 15208, 9952,  63840, 2128,  12212, 52788, 63260,
                44160, 30868, 11848, 46680, 34828, 43032, 52916, 7464,  47468,
                34316, 44496, 3864,  31556, 40736, 2676,  19444, 39068, 63464,
                6016,  10712, 35872, 47336, 42100, 9168,  44440, 26752, 61804,
                46132, 30248,
            },
            {
                2352,  23144, 6244,  18400, 32112, 62004, 55556, 14204, 33804,
                18784, 10636, 39744, 1200,  12868, 31252, 47716, 6844,  20640,
                13756, 48052, 2844,  22812, 58080, 16396, 40660, 13320, 37608,
                29200, 34588, 11064, 40956, 65244, 3624,  14716, 38932, 50500,
                29900, 45796, 13060, 33884, 43416, 25636, 46636, 9152,  28020,
                12432, 2908,  57916, 46876, 54156, 63992, 20696, 43772, 52168,
                5672,  19288, 49620, 8060,  43312, 39604, 3032,  36232, 14320,
                44648, 53476, 23220, 43184, 47500, 7996,  63736, 26420, 44616,
                62156, 22948, 1612,  26140, 57116, 21468, 3956,  10584, 30348,
                64504, 49980, 41976, 53568, 47364, 12968, 55684, 22944, 34240,
                53184, 416,   39540, 49680, 36176, 25376, 40964, 21280, 33588,
                2588,  18720, 24124, 37812, 936,   27304, 65264, 18932, 39024,
                12920, 53336, 57600, 49608, 64496, 12676, 59692, 32600, 22732,
                52304, 56000, 31488, 14792, 4848,  29776, 37224, 56788, 7324,
                35716, 53860,
            },
            {
                63136, 39568, 13176, 60160, 44904, 9344,  36936, 50128, 65220,
                6448,  48748, 25836, 57396, 52100, 64264, 24720, 58436, 51176,
                28800, 62360, 35968, 18940, 8200,  64664, 1732,  49808, 55416,
                3808,  45276, 15672, 24852, 50112, 43264, 7868,  32004, 63552,
                16776, 40428, 54468, 1764,  63424, 30276, 3732,  40628, 56224,
                51840, 38748, 15100, 19660, 30112, 10660, 56620, 32652, 23164,
                61488, 39328, 31244, 57392, 22160, 53840, 24664, 58940, 49924,
                32908, 11252, 2700,  61584, 29988, 17328, 50288, 20184, 3884,
                13276, 38912, 46320, 65132, 17320, 37180, 40848, 52944, 25144,
                16972, 11748, 5004,  62672, 19488, 31736, 59920, 6596,  62044,
                18808, 44124, 58780, 7580,  54456, 17256, 47780, 5960,  50652,
                58624, 40084, 62336, 53700, 20860, 13376, 48660, 30996, 1952,
                62736, 8732,  27224, 21980, 35224, 43332, 7880,  47224, 4076,
                16984, 41236, 59924, 24920, 62112, 54980, 12836, 49276, 24108,
                11176, 17000,
            },
            {
                26900, 48136, 34664, 55072, 24960, 16136, 21364, 3020,  29528,
                23116, 59100, 15400, 44412, 19508, 3172,  11228, 35076, 44532,
                8796,  40228, 55860, 43552, 29488, 45948, 11184, 32368, 18668,
                22288, 63152, 57348, 320,   18116, 27388, 56836, 48084, 24272,
                4188,  58620, 22204, 37116, 48724, 11944, 58536, 21024, 16316,
                31108, 45084, 7520,  26368, 41748, 4896,  37856, 13980, 1524,
                42472, 15708, 10616, 64700, 34040, 12108, 29872, 8896,  41276,
                19028, 56036, 38956, 35684, 12484, 58292, 34200, 41224, 57068,
                28888, 52240, 8656,  31764, 49516, 13712, 60644, 7992,  56312,
                45824, 32724, 27548, 37296, 1412,  43884, 26112, 41052, 14724,
                37212, 5052,  28588, 20712, 32808, 59880, 29572, 14188, 36576,
                25736, 4284,  49120, 8792,  57096, 33484, 5380,  59136, 23440,
                42420, 17864, 37424, 6076,  15912, 54256, 25936, 57260, 37656,
                28912, 1692,  21432, 48488, 38908, 17520, 1532,  20584, 41288,
                32436, 57400,
            },
            {
                8332,  20064, 804,   29112, 4604,  52036, 40132, 56948, 46292,
                41700, 35608, 4912,  31904, 38240, 27600, 41260, 54188, 18156,
                636,   15732, 24316, 5088,  53276, 36756, 25344, 61256, 42244,
                9116,  52128, 30404, 38044, 33968, 61496, 12444, 19876, 35476,
                9356,  52008, 28328, 7528,  19008, 53180, 33620, 5948,  65252,
                23684, 34992, 59012, 62628, 51008, 17384, 45996, 63228, 26580,
                53980, 47604, 28256, 3312,  44392, 19672, 47152, 63716, 1436,
                26588, 60048, 7304,  20968, 52344, 25056, 5096,  10348, 48708,
                16752, 59816, 20472, 41872, 4756,  28616, 23284, 34908, 1204,
                39108, 60164, 21588, 58124, 16088, 52212, 10072, 29376, 48128,
                57004, 24376, 62784, 46624, 10940, 1628,  43732, 62096, 55716,
                11352, 32124, 15424, 29188, 41144, 45096, 16448, 36480, 50744,
                29268, 46544, 60512, 51484, 324,   30372, 18596, 10036, 50676,
                65376, 45184, 12168, 6816,  52664, 33932, 27464, 64400, 51508,
                3348,  44728,
            },
            {
                52676, 38368, 58852, 42312, 46924, 63020, 31392, 10112, 17680,
                12612, 54608, 63536, 9164,  50012, 61264, 5956,  22856, 31164,
                65120, 51980, 33368, 60284, 13676, 20476, 57464, 5468,  47640,
                26700, 39728, 13752, 6772,  51352, 46076, 4964,  41820, 59932,
                44684, 14472, 32636, 42688, 60368, 24460, 38008, 50212, 42432,
                4112,  10876, 48160, 336,   32188, 23796, 8720,  58808, 34700,
                7104,  21688, 36660, 50804, 6240,  55668, 14624, 35240, 52144,
                16416, 31316, 48824, 42008, 1052,  45296, 62524, 31292, 37160,
                24200, 2264,  34628, 54896, 10700, 63584, 43640, 48416, 20028,
                14412, 51868, 9796,  48576, 6356,  35108, 63444, 21264, 1720,
                51388, 12616, 35376, 15868, 40608, 51992, 22908, 8320,  19988,
                46788, 42952, 64392, 23328, 6576,  61420, 26104, 55880, 11024,
                3564,  14056, 32624, 40396, 58396, 45356, 62208, 33784, 23712,
                14372, 35720, 58644, 32080, 9792,  42604, 46580, 5284,  36504,
                60212, 14664,
            },
            {
                30928, 65008, 10608, 22668, 14376, 7564,  34784, 26552, 60464,
                896,   28352, 21148, 46716, 17372, 55472, 14148, 45816, 58904,
                38412, 11764, 27876, 47368, 1428,  41096, 50864, 17020, 35544,
                1912,  54968, 20292, 58956, 10148, 22560, 29276, 53548, 2280,
                25880, 64940, 47868, 3196,  16592, 9504,  45944, 13808, 26944,
                55748, 19264, 37508, 14540, 55128, 40356, 28840, 48872, 18692,
                12688, 56928, 62032, 17080, 41364, 60636, 27648, 39176, 4436,
                45432, 23644, 13452, 65524, 27972, 18640, 54476, 14228, 43740,
                6540,  63112, 45732, 25796, 16032, 52072, 3156,  30972, 61984,
                25828, 5336,  44492, 30032, 40328, 55312, 18080, 44924, 32296,
                8400,  42404, 55224, 3648,  26436, 64256, 33828, 39104, 27792,
                1212,  52820, 18300, 37256, 12188, 49400, 1336,  39712, 21456,
                53712, 63008, 25124, 9440,  22676, 12888, 39112, 2564,  42860,
                5600,  53124, 18280, 25572, 63024, 22480, 57832, 13432, 29548,
                19168, 24700,
            },
            {
                5716,  33820, 17576, 49300, 37132, 57816, 19220, 44360, 49444,
                37692, 52532, 33796, 3576,  25100, 36320, 29380, 2332,  48764,
                19768, 6728,  42744, 21984, 63892, 8868,  31724, 28380, 11716,
                63496, 44952, 32512, 25248, 64380, 40176, 15520, 36848, 31560,
                17892, 10920, 38724, 55984, 27620, 63224, 31300, 992,   61288,
                51956, 29556, 43804, 22136, 60428, 11936, 53056, 2060,  44184,
                30956, 39936, 25664, 132,   32356, 23892, 7680,  20916, 54956,
                61948, 10192, 37164, 56284, 33048, 8916,  39316, 22232, 51156,
                56220, 30712, 13024, 40008, 57512, 37076, 18488, 9032,  54712,
                41360, 35920, 65340, 22700, 13144, 25332, 3424,  57812, 38848,
                64836, 22380, 30268, 48500, 18560, 56912, 6192,  15092, 50044,
                58932, 34876, 5168,  60112, 54604, 20096, 30620, 64056, 34788,
                6920,  43848, 18260, 48796, 4048,  53432, 27264, 48008, 20404,
                56332, 28432, 40108, 49424, 792,   16028, 53236, 39608, 9332,
                43840, 49776,
            },
            {
                40420, 55132, 27088, 2644,  53080, 25296, 3912,  64408, 6408,
                23728, 15084, 43752, 62132, 11676, 57996, 42984, 9304,  34572,
                25800, 56052, 50096, 35196, 15420, 38936, 54060, 60136, 22632,
                50116, 16112, 5160,  42840, 48548, 1104,  55596, 62304, 7036,
                50620, 57544, 21292, 34900, 6288,  54068, 41104, 21972, 35996,
                16688, 8368,  64280, 3696,  34324, 6732,  20360, 37020, 64860,
                4788,  49688, 9776,  53556, 46324, 11348, 58720, 48516, 32752,
                42344, 6000,  51480, 16676, 4212,  48056, 60488, 2836,  27376,
                19272, 8476,  49840, 488,   21408, 32692, 47220, 58972, 12496,
                28468, 17136, 112,   53620, 32868, 61212, 50284, 10784, 27252,
                17392, 5184,  60308, 38060, 9900,  45132, 31100, 60896, 41864,
                24344, 9984,  29620, 46248, 25076, 43460, 8600,  15512, 47540,
                27008, 57132, 37152, 29992, 61064, 35400, 16824, 63360, 31924,
                8804,  60408, 3768,  44668, 36372, 30452, 6412,  26100, 63352,
                56444, 1588,
            },
            {
                60744, 13040, 45496, 62552, 30480, 39588, 13576, 54308, 41428,
                9512,  56876, 30720, 40000, 7312,  51196, 21008, 63116, 15824,
                60772, 3440,  10612, 30264, 57712, 24536, 2756,  43276, 7248,
                37880, 56612, 29072, 18248, 34852, 12964, 26976, 20432, 45760,
                41248, 24396, 1320,  44472, 49832, 19388, 11604, 47636, 4796,
                58516, 39332, 25316, 46072, 50412, 27160, 42508, 55856, 24644,
                16476, 59536, 35628, 20492, 64328, 38108, 29248, 13996, 1016,
                18852, 26056, 30120, 40532, 58032, 24672, 34940, 10936, 64224,
                41708, 35824, 59576, 28956, 62408, 5900,  24964, 38684, 4416,
                50452, 46232, 57588, 19576, 7952,  43000, 36556, 15096, 47240,
                53852, 34424, 13336, 51080, 236,   21568, 36336, 12140, 3796,
                17312, 57160, 40576, 14116, 2884,  33356, 52092, 58988, 41716,
                12076, 976,   52416, 14456, 41504, 6164,  57924, 11392, 51368,
                37272, 15644, 21968, 12040, 55728, 61232, 20484, 48404, 34668,
                16808, 23140,
            },
            {
                36644, 20300, 6948,  42580, 10228, 21772, 47104, 32592, 27680,
                20256, 60068, 1644,  49000, 18016, 27340, 38812, 52996, 31536,
                40520, 45060, 17908, 62852, 6036,  46156, 19312, 48884, 14736,
                33556, 9780,  61628, 3652,  58140, 38768, 49576, 9200,  59104,
                34064, 13648, 63764, 30212, 14964, 38500, 59520, 24812, 33504,
                49632, 12780, 31140, 56760, 17588, 62704, 14712, 32492, 8288,
                45460, 29440, 13536, 43084, 5700,  17664, 50884, 44016, 56888,
                35884, 63756, 47272, 21416, 12712, 43216, 52560, 17108, 31772,
                47548, 14828, 23604, 44844, 10328, 51500, 14612, 43476, 63932,
                22996, 33712, 10236, 38312, 48412, 4820,  29648, 21128, 2324,
                59196, 43612, 24328, 28848, 63668, 54788, 25876, 52696, 47572,
                65420, 32500, 21520, 49764, 62248, 38508, 18740, 4680,  23552,
                32968, 20024, 64344, 8676,  21692, 46296, 24560, 716,   43600,
                25892, 47532, 64052, 33300, 8340,  50956, 42748, 3908,  31248,
                11456, 46764,
            },
            {
                28620, 51620, 32424, 56148, 16908, 58788, 352,   61360, 16536,
                35368, 46424, 12956, 23156, 63852, 33512, 6004,  240,   12524,
                23368, 54000, 37128, 51536, 27260, 11888, 36032, 65480, 26452,
                41004, 51044, 24212, 45580, 52880, 21756, 30964, 2792,  17072,
                28372, 4748,  52940, 8580,  25848, 62512, 2736,  43048, 54828,
                7084,  20700, 41308, 1188,  9908,  37972, 48276, 2536,  61512,
                22320, 52172, 1976,  25924, 55408, 34144, 4116,  61072, 23360,
                10832, 52908, 2988,  8176,  62228, 29028, 948,   38372, 58236,
                6292,  3392,  54800, 17700, 41240, 33976, 55892, 19952, 30228,
                2392,  15308, 61748, 28116, 23696, 58528, 63220, 52336, 31968,
                39596, 7376,  19324, 46408, 40764, 15840, 5308,  43464, 19656,
                37936, 744,   55240, 7220,  26576, 56588, 10640, 29076, 48692,
                60608, 39392, 28192, 50120, 33880, 54672, 31356, 39752, 19284,
                6644,  53560, 29256, 40972, 24624, 17912, 13332, 38468, 58704,
                54052, 8184,
            },
            {
                63068, 15176, 1888,  23804, 35676, 50148, 8068,  38124, 51276,
                4656,  53876, 29540, 37376, 55664, 14796, 47276, 43628, 57488,
                28692, 7596,  21140, 1152,  42148, 32020, 54796, 5104,  58376,
                108,   19916, 13636, 32340, 6672,  11356, 64672, 43472, 54924,
                39520, 60836, 46420, 35552, 56544, 31668, 18144, 10564, 28420,
                63944, 15780, 52096, 60036, 23936, 29964, 53416, 12380, 40592,
                36380, 57284, 47476, 31588, 62496, 22128, 9268,  27772, 39572,
                15808, 32096, 59464, 37560, 45712, 15196, 55464, 20176, 43924,
                25808, 51772, 37184, 65012, 1624,  26328, 61020, 8396,  36456,
                53916, 40940, 51704, 44368, 12984, 17336, 40692, 9516,  25592,
                12236, 62100, 55552, 3704,  10840, 33548, 61696, 30356, 8848,
                27448, 51504, 12628, 43900, 16652, 35860, 64780, 43044, 52924,
                2980,  16232, 44784, 4780,  59084, 17332, 12632, 65048, 56892,
                35100, 14092, 1496,  59004, 5028,  46656, 27556, 65240, 104,
                25180, 43076,
            },
            {
                4376,  59500, 47768, 40560, 64112, 28964, 44520, 25544, 11384,
                65504, 42204, 8240,  44900, 3476,  59836, 25472, 18912, 65004,
                48836, 33848, 60228, 14200, 48024, 61732, 16772, 21884, 30036,
                44172, 54360, 63596, 26680, 41928, 59820, 36020, 25448, 51024,
                22564, 10472, 16436, 21436, 43820, 6556,  52288, 36216, 46352,
                23272, 32504, 44156, 35260, 5412,  45668, 21284, 58440, 27032,
                17924, 10528, 6916,  38964, 15292, 42012, 46896, 54552, 2216,
                49820, 43040, 18584, 25656, 34172, 5372,  23404, 48676, 9540,
                61832, 29428, 20628, 11928, 31056, 46628, 5172,  49236, 12180,
                59192, 26032, 7296,  3536,  35124, 54844, 844,   45464, 35784,
                49356, 16384, 31120, 37084, 58616, 50092, 22816, 55908, 14348,
                58836, 34544, 23544, 47288, 31208, 1548,  22104, 14336, 8204,
                36684, 56068, 11208, 26080, 42164, 2316,  28916, 48116, 9852,
                45244, 61068, 20920, 37748, 51876, 56784, 35204, 21528, 52388,
                17644, 39212,
            },
            {
                20748, 34484, 26720, 11828, 18688, 4004,  14696, 55380, 22724,
                32028, 17524, 26776, 21208, 11096, 50988, 30392, 36956, 9700,
                2496,  16600, 39252, 56580, 24152, 7948,  40088, 52216, 11140,
                37604, 8328,  35044, 16664, 48224, 1008,  19112, 14208, 7256,
                32408, 48820, 37888, 376,   50520, 13344, 40432, 57824, 1468,
                60668, 4616,  11804, 56072, 18552, 65396, 7724,  33032, 4216,
                44468, 64188, 19964, 50644, 59848, 12020, 36488, 19504, 64956,
                29596, 5720,  12316, 54232, 50584, 64624, 30664, 59772, 13684,
                33624, 6792,  40400, 50008, 58156, 16640, 39320, 23412, 43812,
                18132, 32344, 20928, 64360, 49836, 29272, 19932, 57184, 65068,
                22892, 1852,  52120, 20376, 26684, 7644,  39972, 2500,  46188,
                42000, 4808,  63364, 9620,  60292, 40124, 54340, 46268, 24856,
                30760, 63216, 20984, 34936, 61624, 52476, 37420, 22896, 3588,
                27296, 32264, 49552, 10980, 15720, 30048, 8880,  45420, 12300,
                32952, 49860,
            },
            {
                9860,  55808, 6704,  53424, 43140, 57920, 34016, 47960, 40272,
                2572,  57312, 49216, 62916, 34248, 40604, 7068,  22396, 42276,
                55280, 26168, 44748, 10388, 29372, 35068, 2132,  45632, 62500,
                23496, 49544, 2960,  57692, 38488, 29560, 55976, 46204, 62956,
                3448,  57752, 29264, 64996, 24292, 61176, 26896, 20496, 14732,
                38408, 49160, 26016, 39992, 28676, 50568, 42260, 15964, 49284,
                55104, 34696, 28772, 24160, 460,   30652, 57936, 7132,  24896,
                45332, 38696, 57480, 21124, 8940,  40440, 2892,  36784, 42356,
                18240, 56356, 45424, 24636, 9064,  35520, 55108, 28968, 62176,
                596,   47892, 57772, 38892, 24684, 8292,  32736, 13960, 5612,
                28336, 40968, 60520, 44504, 13132, 47444, 17932, 36500, 62432,
                21316, 17176, 38692, 28776, 49492, 19608, 6052,  33664, 59740,
                17752, 49784, 6896,  46916, 14760, 8264,  19980, 59988, 55396,
                41040, 17124, 23744, 43992, 63808, 1992,  41936, 61088, 5608,
                57984, 29712,
            },
            {
                64540, 44812, 38296, 22004, 30932, 9320,  63140, 19900, 6900,
                60424, 13592, 38492, 764,   54508, 16280, 58328, 61788, 13424,
                51648, 31104, 63672, 4020,  50436, 19100, 59064, 13780, 32836,
                18220, 28716, 60660, 12784, 22020, 52376, 9376,  40900, 34732,
                26604, 12348, 41712, 18812, 4264,  34516, 47732, 7760,  29764,
                55424, 18020, 8808,  62528, 220,   13048, 36264, 24368, 61312,
                1704,  13616, 42752, 53912, 5236,  48976, 16768, 33952, 51912,
                13944, 60532, 684,   27956, 47156, 17640, 12000, 26404, 53232,
                40,    63004, 15280, 4304,  60544, 21480, 3008,  13964, 51904,
                9612,  36740, 15612, 11548, 46496, 61228, 53220, 42896, 37528,
                48520, 9188,  34164, 4892,  63920, 29452, 53812, 24968, 11308,
                31352, 52604, 57360, 13824, 3456,  56188, 27244, 11360, 41556,
                876,   38268, 55048, 23960, 30164, 39852, 49996, 33140, 13508,
                5144,  62604, 7432,  53628, 33632, 19828, 26392, 37556, 23364,
                47248, 14504,
            },
            {
                25308, 2204,  16548, 61000, 51380, 1084,  37260, 26888, 52232,
                44760, 33352, 19044, 29032, 46476, 24420, 4732,  48172, 34964,
                18568, 6348,  21736, 37304, 61136, 42944, 27168, 48480, 4104,
                55200, 42384, 6524,  45832, 33236, 4500,  24360, 16308, 59384,
                20784, 52832, 44828, 55788, 31096, 10992, 53864, 42392, 64464,
                33096, 44992, 53004, 21920, 32068, 58976, 53744, 10044, 39636,
                31196, 22220, 9192,  62308, 38276, 26216, 44068, 63064, 9468,
                22764, 31360, 36052, 42664, 63244, 32844, 58952, 45116, 49524,
                22548, 28548, 38348, 31980, 52396, 42960, 64556, 33568, 40816,
                26780, 54940, 30288, 4336,  41092, 16896, 2776,  22228, 59456,
                18460, 55204, 14968, 23764, 56900, 484,   33420, 60280, 6476,
                48040, 1708,  33964, 25320, 43444, 36528, 63688, 47944, 53260,
                28492, 13604, 43504, 4168,  57856, 63096, 228,   25616, 43096,
                52328, 36496, 27912, 39372, 3880,  59468, 48972, 16796, 54660,
                3200,  40504,
            },
            {
                52768, 35940, 28404, 46196, 13740, 24048, 42264, 15956, 30052,
                10096, 23860, 56460, 7700,  12480, 43588, 32440, 27276, 2860,
                40248, 46760, 53800, 15628, 11924, 55660, 22652, 38564, 9544,
                63856, 24752, 39396, 19744, 56504, 65192, 44168, 49684, 1916,
                38712, 5304,  15592, 8468,  36840, 59624, 17260, 2380,  23872,
                12268, 3316,  36748, 41856, 16356, 45376, 5080,  28148, 17708,
                47232, 59716, 35820, 45916, 14560, 56560, 20980, 41000, 2440,
                47644, 53512, 16104, 6416,  23948, 54448, 4008,  20268, 8540,
                35420, 11476, 55196, 47496, 26556, 10912, 17276, 46532, 6112,
                19708, 63436, 43964, 22672, 56692, 27612, 35664, 50984, 11444,
                26488, 43836, 30736, 38084, 41656, 9680,  51004, 15056, 43196,
                27292, 64476, 9104,  51076, 18212, 7472,  23120, 15576, 4524,
                21672, 65352, 32416, 10896, 18752, 45216, 16220, 9604,  47648,
                19040, 57992, 10492, 46768, 56092, 13668, 31432, 9692,  62972,
                33824, 19348,
            },
            {
                61756, 11132, 57196, 4816,  33176, 49140, 55100, 59080, 3364,
                61736, 39748, 50536, 64616, 36744, 52380, 19804, 55952, 64192,
                11280, 58676, 24936, 33728, 1544,  31520, 6988,  17368, 52756,
                34748, 14692, 50820, 10672, 27528, 37140, 13164, 31888, 28296,
                62200, 47332, 24940, 63468, 49536, 22364, 45656, 39264, 51264,
                59912, 27492, 57056, 6924,  25112, 48496, 20548, 64436, 56364,
                6992,  51968, 3752,  19192, 32196, 10544, 5824,  29156, 37468,
                56908, 19204, 61960, 10524, 49948, 13720, 38904, 30548, 56904,
                65388, 41264, 2336,  19256, 7392,  36980, 29728, 58840, 24736,
                49724, 2504,  34300, 9180,  62016, 47740, 7176,  31340, 63844,
                1520,  6588,  61376, 49376, 17464, 21944, 45576, 37624, 19772,
                55572, 40016, 22324, 45516, 58776, 32444, 39232, 60764, 45128,
                35652, 56532, 50368, 25968, 37196, 53768, 34480, 28744, 64828,
                2772,  31044, 15244, 21220, 24980, 6420,  44740, 51516, 27500,
                42552, 7768,
            },
            {
                30448, 49604, 20384, 41476, 63504, 6628,  19408, 12088, 47196,
                34924, 5140,  16560, 21464, 1896,  59788, 8836,  14900, 38068,
                29472, 43956, 8436,  49208, 41672, 63292, 46084, 57452, 29580,
                512,   47176, 31404, 61988, 2160,  53288, 7744,  18800, 57136,
                11056, 35288, 29836, 40688, 1132,  13736, 28444, 6308,  35452,
                20220, 15080, 49696, 10808, 60928, 34644, 2780,  37568, 43152,
                25268, 15524, 41076, 27488, 65208, 53376, 49508, 61320, 12544,
                25272, 3604,  39856, 44804, 34624, 27052, 61148, 16544, 6032,
                25508, 44652, 14704, 61936, 57180, 49044, 1652,  54228, 12664,
                37820, 59864, 14556, 52164, 38520, 19812, 13448, 55516, 39484,
                46356, 19936, 52624, 11992, 4276,  65232, 28684, 59272, 2904,
                12160, 35648, 15660, 61688, 68,    11900, 52316, 30272, 9752,
                18396, 7124,  40664, 2344,  59940, 22252, 6788,  56896, 38828,
                23316, 35308, 61316, 51016, 40872, 64644, 36692, 1684,  21912,
                13140, 46616,
            },
            {
                17156, 612,   24456, 36836, 9624,  27576, 39308, 31716, 22356,
                54344, 26648, 48456, 41300, 30568, 25528, 42656, 49412, 22940,
                536,   17012, 60564, 27796, 19396, 51592, 24424, 11124, 40920,
                20656, 59268, 4900,  17976, 42724, 23472, 61124, 40740, 51632,
                22224, 6940,  55220, 19152, 52652, 33552, 57236, 61744, 9588,
                46812, 31620, 64116, 39684, 29048, 54944, 13572, 50252, 30512,
                11120, 34128, 58312, 47884, 2688,  23320, 35120, 17808, 45976,
                33384, 51172, 28880, 21856, 58556, 1452,  42484, 47236, 52052,
                33000, 21732, 50504, 34660, 24312, 39828, 21336, 43364, 32648,
                7796,  45748, 18204, 25896, 32312, 192,   44332, 24868, 15548,
                34060, 28908, 58628, 36236, 26164, 32976, 53964, 8220,  24464,
                31068, 49708, 4852,  28932, 41344, 25516, 57072, 3344,  49280,
                24052, 62068, 15812, 29512, 47464, 13652, 41604, 51612, 12152,
                49108, 43704, 8380,  580,   32936, 28324, 16140, 58452, 55448,
                37788, 59592,
            },
            {
                32828, 54484, 44196, 60260, 15212, 56252, 45200, 64924, 168,
                43088, 14104, 62412, 10696, 35328, 55096, 3772,  61556, 33068,
                52572, 36144, 55288, 4800,  39096, 14144, 3340,  36624, 64876,
                44584, 25956, 55440, 35804, 48464, 14392, 33396, 46044, 12,
                16412, 42828, 59524, 12148, 4080,  43712, 24184, 17944, 53412,
                42016, 5536,  22152, 952,   44296, 18728, 8312,  22988, 59896,
                728,   54084, 21608, 8484,  37808, 13968, 43164, 1108,  58756,
                8428,  63616, 14820, 5624,  52724, 9640,  19524, 36412, 12136,
                3780,  58524, 8832,  30256, 12292, 4632,  60628, 16236, 63904,
                23308, 29940, 57472, 4864,  48596, 65144, 58096, 9476,  49864,
                61872, 23032, 8404,  42712, 47956, 1444,  14420, 41232, 46852,
                63460, 57964, 19336, 54240, 47872, 14552, 20316, 42852, 38080,
                27664, 54144, 44276, 10116, 64144, 32568, 1324,  19824, 27112,
                4320,  59344, 18040, 54780, 46148, 11712, 20068, 43764, 7120,
                24904, 4556,
            },
            {
                65344, 12356, 28196, 50448, 22096, 3228,  52184, 17096, 36360,
                8448,  29120, 56708, 5760,  45740, 17648, 12652, 39660, 19512,
                7568,  46192, 12284, 23252, 61844, 34100, 58856, 28100, 5908,
                16212, 12192, 39008, 8020,  28668, 58844, 4412,  25768, 37884,
                64352, 49780, 32376, 27468, 38340, 65432, 48196, 30572, 1828,
                26312, 56176, 36208, 16008, 52104, 33076, 62760, 39312, 45720,
                17084, 63456, 28660, 44252, 51672, 60404, 30688, 54284, 27288,
                21100, 41760, 37112, 46632, 30380, 64248, 24540, 55696, 61644,
                27980, 38972, 17716, 63644, 46036, 52840, 28136, 9480,  50236,
                1004,  42184, 53192, 11628, 40048, 28612, 21824, 36340, 2548,
                41036, 4784,  13812, 54740, 18980, 61224, 36824, 51236, 17340,
                6352,  43548, 10420, 33008, 37396, 7676,  64588, 33424, 58736,
                12700, 5024,  35232, 20972, 52488, 25348, 58076, 45784, 63252,
                15008, 30296, 37232, 22472, 62428, 3988,  52264, 34252, 29336,
                48384, 41140,
            },
            {
                19908, 8572,  39480, 5968,  34780, 30464, 10932, 25384, 60688,
                49628, 18952, 32396, 24008, 51028, 63912, 28448, 47880, 57304,
                25884, 65428, 41136, 31028, 9184,  48644, 17884, 45756, 54288,
                33172, 49316, 62488, 21348, 51812, 10380, 19836, 56832, 30188,
                5988,  23996, 9036,  45676, 21384, 7396,  15536, 11060, 37584,
                63128, 13208, 46048, 59340, 24264, 3800,  56524, 27040, 5520,
                35524, 40528, 12240, 4380,  18892, 25196, 9988,  39580, 15788,
                4484,  57420, 11296, 55264, 17052, 34076, 40808, 2704,  15684,
                48880, 43240, 54112, 1180,  36444, 41744, 19352, 33692, 56108,
                38716, 20520, 62800, 35416, 7896,  16624, 42992, 53828, 30724,
                17728, 57364, 45520, 31880, 24148, 10200, 57564, 21624, 33912,
                27516, 38872, 23400, 62436, 3040,  27356, 50528, 16932, 1556,
                47048, 31612, 56428, 39780, 3860,  17208, 37416, 9148,  34168,
                52980, 41268, 10464, 26592, 49384, 39400, 59252, 9216,  62880,
                14028, 51452,
            },
            {
                35792, 46236, 57876, 17744, 63048, 48140, 38632, 57444, 4940,
                44540, 39736, 58912, 1616,  38040, 21324, 8976,  4572,  31776,
                14648, 2136,  20552, 52948, 43316, 820,   56180, 21668, 9788,
                41612, 24168, 5324,  31264, 43736, 63712, 40200, 47908, 12912,
                35752, 53660, 14952, 60436, 56004, 34432, 40992, 58576, 49656,
                19764, 32300, 8752,  28220, 41084, 11700, 47776, 14932, 53140,
                20212, 49496, 31952, 61484, 56144, 34024, 6188,  64428, 50792,
                45648, 32476, 23792, 296,   49548, 26880, 7648,  45340, 22900,
                31428, 4924,  20880, 26316, 15264, 57792, 5836,  47700, 12848,
                26804, 3824,  15184, 24596, 50420, 60344, 5648,  12748, 63532,
                51120, 27504, 37832, 63164, 5620,  40240, 29236, 3940,  12824,
                52688, 1220,  16080, 56844, 44960, 21876, 53784, 9396,  60824,
                23052, 18492, 49772, 8016,  60540, 43220, 50456, 28976, 5388,
                20332, 45272, 57720, 6784,  14524, 30644, 24340, 18008, 38444,
                22540, 1516,
            },
            {
                60876, 31280, 23824, 53148, 1248,  42188, 20560, 14440, 33760,
                23036, 11508, 53216, 15700, 43716, 34080, 54092, 60028, 44376,
                35780, 49896, 28504, 58084, 15496, 24976, 37816, 30488, 61100,
                2052,  52864, 57784, 14980, 1272,  34944, 16620, 54584, 2600,
                62060, 41600, 18788, 49048, 748,   26012, 52472, 3760,  23516,
                43144, 54684, 2508,  50932, 65080, 20760, 37248, 30992, 9464,
                57240, 2512,  23092, 46368, 14904, 41540, 48640, 22464, 36428,
                28724, 61976, 18956, 43512, 60652, 13148, 58016, 51408, 10268,
                64872, 34968, 58860, 8452,  50788, 30636, 24140, 64628, 36240,
                44396, 61180, 47172, 32964, 56552, 19784, 45744, 33508, 25148,
                9728,  20488, 504,   52256, 16400, 49436, 44820, 64376, 55528,
                46384, 60336, 30884, 49928, 11640, 41644, 29824, 35992, 40096,
                43988, 26640, 63316, 14532, 30220, 22844, 11948, 62076, 56656,
                24804, 1172,  64960, 34612, 43068, 1824,  55060, 45304, 6248,
                56608, 27068,
            },
            {
                15460, 4400,  43432, 13348, 26760, 9368,  61576, 28984, 50960,
                2436,  64372, 27780, 7712,  60872, 13180, 25004, 40924, 17476,
                10452, 62324, 38616, 5676,  33108, 64244, 7588,  50848, 13476,
                36072, 19328, 27976, 38528, 47244, 24716, 7488,  27484, 21144,
                45052, 29136, 4772,  37428, 13196, 31128, 61508, 16604, 29104,
                6332,  60480, 38280, 17820, 33840, 6964,  44108, 61164, 24924,
                42740, 64680, 7924,  37444, 26548, 584,   58148, 18044, 12744,
                2428,  8864,  52024, 35888, 5048,  39036, 20336, 29448, 37288,
                54984, 13492, 47124, 40192, 62660, 11596, 2416,  53684, 16588,
                7288,  29752, 10180, 41616, 1568,  28124, 38048, 3600,  59280,
                47472, 43344, 34916, 12172, 60020, 26116, 35432, 19364, 9424,
                24480, 40744, 7144,  35008, 18084, 4640,  62116, 14436, 6344,
                56972, 11028, 292,   38332, 34320, 47252, 2412,  16480, 39536,
                47992, 31948, 17772, 51152, 21268, 61612, 48540, 12008, 31836,
                42380, 52704,
            },
            {
                47668, 10724, 63700, 37464, 33160, 54744, 45532, 7040,  55700,
                16832, 47268, 37100, 19756, 49136, 29552, 328,   50768, 22680,
                56028, 3632,  18504, 47964, 12060, 45416, 40772, 27064, 17200,
                48148, 44316, 64736, 11568, 55420, 61348, 32152, 49936, 58936,
                8788,  33928, 23384, 64028, 43300, 20276, 45928, 9816,  35564,
                47564, 13984, 26048, 10508, 49088, 58144, 148,   51624, 17544,
                33636, 12724, 29524, 54028, 50384, 10952, 31380, 43932, 59956,
                54392, 40080, 25952, 15360, 31080, 47556, 63144, 1820,  41784,
                18420, 5744,  27816, 22596, 44996, 38136, 32516, 42072, 21424,
                50948, 57572, 18828, 54432, 13892, 64252, 22488, 49624, 40116,
                15064, 7216,  56568, 31324, 22068, 8168,  1908,  42684, 29908,
                37024, 14048, 21044, 65296, 26252, 46604, 55492, 24180, 32628,
                48236, 29168, 52308, 20172, 55412, 59608, 27216, 53868, 35932,
                13168, 8216,  55228, 11380, 37876, 28068, 16648, 36272, 64500,
                3280,  19076,
            },
            {
                34472, 28604, 56868, 21180, 48932, 3500,  18060, 40064, 35016,
                25716, 41972, 31828, 3744,  56924, 39836, 64772, 6840,  31192,
                46316, 27436, 53484, 23980, 59828, 20012, 2516,  62220, 56780,
                3968,  31932, 8076,  22916, 3416,  18196, 41812, 12128, 38844,
                15928, 52716, 57368, 11092, 51064, 6652,  55708, 39256, 62624,
                52836, 21764, 42132, 56032, 30080, 23172, 14300, 28428, 4560,
                39612, 59604, 44724, 16488, 20428, 63452, 38256, 4316,  24772,
                33400, 48316, 65316, 56596, 22740, 9568,  53524, 15960, 25012,
                59752, 33792, 52800, 156,   16128, 19888, 61004, 4272,  59112,
                25132, 35084, 39344, 6168,  31660, 52048, 8284,  17460, 55124,
                29636, 24216, 65096, 48256, 39544, 53496, 58700, 15836, 62636,
                3024,  47744, 53352, 58244, 588,   38020, 51184, 19528, 2760,
                64340, 16576, 41456, 4688,  44576, 10400, 6944,  42156, 21868,
                62980, 29672, 40516, 23780, 58896, 4704,  8728,  50648, 25192,
                58716, 40172,
            },
            {
                23428, 51192, 6084,  15908, 60468, 24916, 31204, 63272, 12924,
                58364, 9724,  62184, 22504, 45812, 10728, 16612, 36316, 58304,
                13860, 34108, 41728, 8228,  36736, 29576, 52420, 34600, 10484,
                25720, 40156, 59588, 50728, 37048, 45528, 53388, 520,   65104,
                25588, 46584, 2112,  40208, 27964, 33680, 24816, 1660,  18320,
                31780, 5364,  64008, 2148,  46060, 40244, 62448, 36304, 48564,
                55972, 21960, 2032,  35396, 5224,  55796, 28264, 46824, 14400,
                19892, 6688,  11680, 2708,  43028, 34952, 27256, 7080,  50084,
                43888, 11240, 61780, 30140, 56664, 49916, 9580,  47340, 28988,
                12016, 656,   49164, 60204, 24244, 43580, 36128, 11780, 60656,
                1716,  37324, 17984, 4700,  28216, 13108, 46780, 32880, 50484,
                56400, 27896, 10856, 43384, 31440, 8872,  12860, 60420, 44892,
                36868, 22208, 58324, 31912, 24952, 65392, 18780, 32784, 49468,
                5532,  59744, 44796, 2300,  47064, 33316, 41724, 20764, 29860,
                13496, 7668,
            },
            {
                54708, 38456, 412,   45036, 41324, 10592, 52020, 20864, 828,
                49472, 5320,  52884, 14236, 33916, 27012, 52248, 20644, 43528,
                9528,  63276, 2236,  57412, 14884, 48892, 6284,  22852, 43456,
                53164, 20536, 15640, 33980, 28700, 6324,  21832, 30292, 35768,
                5668,  20388, 32056, 17812, 62892, 14908, 59612, 48396, 12264,
                44256, 27164, 34896, 15904, 19948, 8508,  53596, 11496, 19228,
                7496,  27108, 52272, 60924, 41900, 22864, 9060,  52736, 62696,
                42052, 29456, 37752, 51048, 18108, 61564, 45768, 57676, 30968,
                20620, 4108,  47772, 37000, 7152,  25860, 34692, 15036, 39940,
                44884, 65336, 15920, 20780, 46652, 2424,  62668, 26372, 33120,
                42612, 51692, 10860, 44860, 61800, 20344, 37544, 6212,  23244,
                18832, 4876,  33984, 16628, 22756, 41048, 28656, 34604, 7100,
                53708, 12260, 8472,  50260, 15216, 36612, 46508, 52760, 708,
                15676, 27056, 19360, 51852, 14848, 62900, 53716, 57208, 2420,
                44628, 61356,
            },
            {
                10296, 18360, 64916, 29504, 35412, 7404,  57376, 37200, 46152,
                24336, 30180, 18416, 44072, 55324, 2308,  61820, 38572, 5348,
                25992, 50396, 21488, 31436, 44236, 63864, 18148, 38180, 13004,
                58596, 5544,  47412, 63268, 13316, 56640, 60740, 15020, 43700,
                55548, 61304, 49112, 8296,  44888, 4096,  36924, 20856, 54536,
                8652,  58612, 50588, 38184, 60388, 32800, 25748, 42804, 64200,
                31452, 46416, 10536, 14108, 32108, 48432, 17352, 35056, 1456,
                58648, 45624, 21712, 59644, 32208, 12884, 928,   38248, 14592,
                64620, 40480, 23260, 13556, 42848, 64068, 53112, 22268, 55732,
                7800,  32824, 27380, 38192, 10708, 29804, 53268, 18844, 47040,
                6876,  21388, 55784, 33616, 924,   57520, 26276, 9868,  41496,
                60416, 38452, 45672, 63760, 52872, 59452, 49152, 15972, 25044,
                42284, 27812, 61352, 39496, 2144,  28780, 12548, 57540, 24492,
                38648, 54716, 34836, 7224,  30792, 25996, 11172, 17672, 36940,
                48356, 32116,
            },
            {
                42212, 26284, 49456, 22168, 59180, 14780, 27396, 43008, 16500,
                65508, 39584, 36108, 60588, 8272,  23648, 30808, 47812, 59552,
                15428, 54560, 39704, 11264, 25464, 1392,  54988, 28232, 61908,
                31160, 692,   26928, 42352, 8740,  25096, 39440, 51232, 9608,
                27248, 13288, 37360, 22860, 58240, 51532, 42600, 28892, 63640,
                40092, 23672, 3492,  12972, 55276, 4988,  47324, 1144,  15048,
                58024, 37088, 40596, 25556, 64884, 6268,  57596, 39356, 24308,
                10224, 16304, 54912, 26636, 5232,  41016, 23444, 53848, 9240,
                35088, 55080, 27192, 58416, 17936, 1676,  30696, 5016,  59528,
                18484, 51072, 3488,  54160, 57968, 15040, 40788, 4128,  58340,
                13580, 63580, 25184, 40748, 14772, 51168, 43952, 64744, 31348,
                12400, 51324, 24792, 7972,  2444,  20544, 4716,  55592, 62864,
                1012,  46332, 34120, 19140, 54244, 62340, 40948, 8748,  31024,
                64064, 43188, 10384, 60568, 21704, 44328, 39504, 5892,  63448,
                22308, 15276,
            },
            {
                56480, 6880,  33524, 12112, 47072, 54356, 2232,  32812, 6276,
                55844, 10356, 27568, 4460,  41500, 50280, 12224, 18712, 836,
                29052, 35212, 17872, 47400, 60556, 33460, 41336, 8956,  49788,
                44624, 35712, 19240, 54024, 49076, 2012,  18292, 46804, 33592,
                3216,  41668, 53808, 30000, 11908, 25916, 7408,  32948, 340,
                16880, 47092, 30468, 43676, 21864, 29020, 52016, 35040, 23900,
                54192, 5796,  17896, 50708, 780,   44548, 27656, 12500, 31664,
                51960, 4180,  36300, 8392,  49884, 63372, 47120, 28500, 19136,
                48744, 2260,  8036,  32972, 51104, 39080, 46012, 12368, 36448,
                41448, 23296, 62532, 43272, 7384,  33732, 50004, 22884, 37840,
                28556, 31844, 48772, 8604,  36416, 29956, 3896,  16780, 48144,
                1148,  54932, 15116, 30316, 36024, 46796, 39404, 10448, 31876,
                17956, 51676, 5764,  23636, 48260, 4472,  21508, 45480, 17988,
                3356,  13596, 48916, 36752, 56208, 160,   49520, 34188, 52972,
                28224, 1712,
            },
            {
                36476, 50968, 61752, 3184,  38888, 19832, 62544, 49992, 22412,
                13500, 51712, 47392, 21092, 15284, 63624, 34440, 45336, 56268,
                42400, 65224, 4032,  7436,  52068, 14448, 21636, 3504,  16464,
                23616, 57528, 10444, 37964, 32332, 64460, 29308, 57104, 21080,
                59436, 16924, 64040, 1364,  35252, 18604, 57112, 14244, 53204,
                35896, 10344, 56452, 65328, 6932,  17032, 61392, 41080, 20136,
                3400,  29760, 59400, 22100, 34096, 54868, 20156, 61428, 41304,
                46704, 64592, 29924, 43572, 20104, 15668, 33416, 3932,  61380,
                43100, 16364, 60384, 45236, 10436, 20240, 62788, 25032, 49092,
                28856, 9416,  13904, 35692, 19728, 25252, 64572, 8756,  55004,
                43080, 2156,  17224, 61516, 19928, 54036, 23904, 59060, 34776,
                20768, 27420, 42360, 58092, 62256, 13368, 26824, 44024, 21580,
                38088, 59152, 13900, 56600, 27272, 33328, 58732, 35952, 50944,
                61236, 22640, 28600, 4744,  18508, 65168, 14388, 24084, 8768,
                46144, 59888,
            },
            {
                30340, 21252, 16760, 43788, 25824, 31408, 8980,  40668, 60132,
                34752, 1120,  59204, 32232, 57044, 38024, 26176, 6676,  22348,
                9804,  49568, 24604, 37704, 29496, 56992, 46524, 64636, 39680,
                51636, 6892,  61640, 15492, 22972, 5504,  12364, 36564, 7696,
                24500, 45968, 5800,  48508, 40332, 60980, 45688, 22728, 49020,
                60208, 20040, 26416, 33992, 39444, 49132, 9276,  12536, 45300,
                63408, 47988, 42372, 9932,  38736, 15416, 7828,  48988, 2592,
                17756, 22592, 13340, 60156, 56296, 37316, 10548, 57556, 24552,
                30408, 52192, 22240, 37756, 28628, 4300,  53532, 16004, 60852,
                1360,  57024, 46988, 30128, 60016, 456,   45180, 16616, 11500,
                60500, 52648, 5384,  45444, 56956, 11448, 7412,  38900, 45104,
                62780, 6300,  37716, 9628,  18632, 33436, 50808, 56872, 64536,
                8280,  29744, 41188, 9980,  44120, 16632, 11860, 6756,  25972,
                40196, 32576, 53552, 46756, 41776, 27084, 31864, 57476, 43204,
                19448, 12572,
            },
            {
                54208, 40572, 9644,  58048, 5664,  53024, 17444, 24592, 45196,
                29100, 18928, 43352, 25024, 2764,  10588, 51976, 59952, 40384,
                33024, 13464, 59020, 19972, 42680, 10772, 35140, 26132, 32276,
                12448, 29676, 46648, 41164, 55452, 44972, 50524, 61220, 40044,
                53032, 32220, 26716, 20312, 52412, 9960,  2916,  28152, 38324,
                6180,  42224, 2104,  13848, 53740, 24864, 58544, 37684, 26860,
                33088, 7048,  13788, 62048, 51568, 30456, 58468, 36712, 26332,
                55996, 34372, 40476, 260,   25628, 6060,  51208, 40264, 13612,
                36492, 6184,  11984, 64320, 55848, 41664, 34840, 7368,  32016,
                43644, 18036, 38688, 5936,  52956, 48776, 27336, 36704, 32904,
                20652, 24276, 38392, 27096, 32696, 41212, 50328, 28664, 13692,
                2852,  56384, 48536, 23012, 53384, 268,   6028,  24132, 15332,
                2796,  35160, 49156, 62824, 1480,  38600, 64856, 47304, 54916,
                1140,  9744,  16124, 59532, 7320,  11820, 50560, 3328,  60900,
                39180, 4648,
            },
            {
                48424, 62700, 28740, 46176, 34452, 64608, 48248, 11452, 3836,
                37240, 64044, 8324,  53492, 17676, 47020, 29508, 3828,  16804,
                54304, 27704, 45024, 53300, 188,   62856, 6232,  18756, 54884,
                59696, 4384,  21036, 26572, 2468,  33736, 17776, 27444, 8,
                14856, 11204, 37796, 62196, 15384, 33572, 43424, 64316, 12512,
                31304, 51288, 61944, 45936, 18804, 4456,  31512, 1592,  56192,
                16440, 53564, 24332, 28128, 3208,  46100, 23396, 11556, 63084,
                6684,  52808, 9836,  48176, 31840, 44400, 21852, 65196, 1928,
                54596, 47672, 32452, 17636, 964,   26464, 47260, 21380, 11688,
                54960, 26336, 63540, 22156, 12084, 39956, 3964,  59208, 50924,
                6648,  47588, 65404, 14460, 164,   22056, 63752, 18364, 52576,
                25092, 32640, 16444, 43268, 28412, 59308, 37120, 48280, 42020,
                54328, 25692, 17732, 21936, 31568, 52280, 28864, 19380, 14064,
                58040, 44660, 37516, 24848, 34356, 54840, 37984, 21888, 16856,
                33956, 25440,
            },
            {
                15168, 508,   23512, 12820, 38156, 2480,  27880, 58548, 55184,
                15224, 49240, 39840, 12584, 34228, 41660, 64424, 20896, 36932,
                62088, 2912,  8232,  32024, 16340, 23084, 48188, 37372, 1700,
                43160, 48920, 35456, 63684, 13504, 57896, 9676,  42568, 48956,
                65400, 54928, 44536, 6868,  30428, 24188, 54248, 4984,  17612,
                57516, 23372, 9592,  29360, 36388, 62756, 47216, 22176, 50264,
                10212, 39888, 35208, 57148, 19480, 5896,  42672, 32764, 15844,
                43828, 29144, 19644, 63568, 53988, 16732, 28876, 46308, 18948,
                26220, 42216, 58136, 23884, 50508, 13784, 62144, 58388, 36920,
                49640, 3056,  33936, 15604, 56540, 31544, 18868, 42320, 14012,
                29480, 35384, 9944,  42592, 58652, 47028, 5180,  34460, 9292,
                61308, 39976, 7632,  64984, 12272, 45436, 31396, 17492, 10872,
                61104, 44864, 6336,  59900, 55560, 4144,  24020, 42180, 34864,
                29996, 21368, 64048, 2636,  19788, 62320, 28476, 9956,  46600,
                64552, 7736,
            },
            {
                43348, 36276, 57144, 50152, 20668, 15652, 42880, 32472, 21532,
                6884,  25928, 30676, 60352, 55484, 23468, 7772,  50016, 11376,
                47420, 24176, 35796, 60224, 39804, 28928, 51848, 61364, 14304,
                24828, 17584, 53108, 7516,  38560, 51832, 24080, 30828, 19712,
                34296, 22704, 3592,  58072, 48788, 39688, 59264, 21304, 47816,
                34448, 40444, 15616, 59428, 42836, 8136,  13200, 41216, 64020,
                18608, 44572, 84,    48296, 65288, 37932, 53296, 59720, 1432,
                50720, 38424, 24496, 3112,  36028, 11972, 59760, 7848,  34648,
                61760, 10056, 5036,  38756, 44076, 8876,  30252, 4740,  40556,
                23732, 9664,  51548, 44264, 61848, 7652,  24444, 55544, 62944,
                1572,  54080, 19296, 49876, 30308, 16424, 37368, 57744, 44516,
                29876, 20252, 49964, 35236, 4520,  20824, 62120, 7728,  27920,
                33060, 51316, 13204, 40004, 36096, 11248, 49796, 8196,  62124,
                4804,  48588, 52012, 41044, 14744, 47760, 42916, 1340,  56644,
                30848, 51280,
            },
            {
                55212, 18368, 32596, 6320,  60604, 53480, 8712,  39224, 61940,
                50920, 45764, 4712,  20228, 304,   45136, 15396, 32388, 56344,
                42104, 18664, 14032, 46088, 55824, 11632, 44188, 8004,  31072,
                40424, 9884,  59052, 28200, 20368, 44228, 63044, 3264,  59360,
                8680,  41412, 28836, 13224, 18972, 9524,  824,   36728, 27676,
                7244,  55356, 1228,  50480, 20744, 27524, 56988, 34540, 29432,
                6056,  60712, 21916, 30924, 9672,  13236, 18180, 27748, 21872,
                46388, 61972, 14184, 58052, 42608, 5044,  39188, 55900, 15452,
                49480, 29960, 21072, 63564, 35312, 56420, 19724, 45716, 16976,
                64520, 29652, 41436, 5072,  26800, 47728, 37872, 10624, 44780,
                23264, 40152, 60672, 7792,  25260, 3132,  51560, 23188, 11960,
                55144, 1288,  15072, 56128, 26028, 52440, 40584, 57228, 22484,
                38160, 884,   20164, 47016, 26892, 16164, 44800, 56372, 39280,
                17696, 11744, 26448, 8708,  30564, 4956,  52784, 23924, 39632,
                12064, 27172,
            },
            {
                22352, 10104, 63708, 40824, 30044, 47296, 23956, 1064,  35612,
                11856, 17112, 63200, 36852, 28360, 58816, 38944, 25436, 1664,
                30204, 51756, 64820, 4596,  21228, 2284,  26400, 57456, 20244,
                65172, 34000, 45512, 1160,  32348, 15304, 5820,  36256, 46800,
                17004, 56720, 36896, 51880, 60732, 26224, 43072, 53104, 14252,
                62792, 44688, 25072, 32932, 4920,  52448, 16100, 2272,  24780,
                54212, 37380, 14880, 51988, 26376, 41132, 56100, 4544,  36252,
                11196, 7344,  33248, 18440, 27104, 51032, 23656, 31156, 776,
                37920, 12756, 53324, 2528,  15288, 27352, 52352, 33704, 88,
                53944, 14216, 20440, 60348, 35596, 17268, 52080, 3568,  33600,
                16116, 27472, 12812, 56008, 32876, 63028, 40892, 6484,  26932,
                47764, 41992, 60840, 30832, 37844, 1792,  13872, 49068, 4972,
                54964, 65092, 30152, 8900,  57972, 62940, 21640, 420,   28056,
                59372, 33532, 55140, 36984, 60808, 57508, 16016, 35700, 19020,
                60012, 3700,
            },
            {
                37700, 45780, 1804,  25976, 11624, 17972, 56576, 44620, 27544,
                54296, 32040, 42456, 51716, 5980,  13360, 53128, 62420, 6516,
                58128, 10092, 27948, 38232, 50072, 33220, 16044, 36508, 53844,
                3304,  50220, 12732, 56468, 41128, 48484, 54056, 27308, 12384,
                50684, 24708, 1880,  45140, 33064, 16068, 64808, 31388, 10788,
                38092, 18576, 48408, 11964, 64988, 36796, 45380, 59948, 48780,
                11512, 32260, 57460, 6872,  46024, 33724, 63476, 48876, 31032,
                53092, 40928, 55612, 48616, 9244,  45556, 64036, 20004, 52568,
                44600, 24528, 60140, 47460, 41564, 6936,  60800, 12096, 25844,
                59348, 38532, 48372, 1748,  12588, 30796, 64712, 21260, 59560,
                49084, 5576,  45884, 38072, 20756, 44080, 14688, 60084, 18216,
                33488, 4904,  22960, 10372, 46996, 63608, 19436, 34288, 43852,
                16252, 24248, 41544, 52748, 3244,  34176, 30952, 53616, 13456,
                23380, 46568, 1932,  43308, 21660, 11276, 32788, 64000, 5704,
                41944, 48664,
            },
            {
                61852, 15992, 51884, 57756, 36908, 4228,  64928, 13940, 20052,
                59128, 3284,  10376, 22692, 47212, 33904, 18032, 43996, 21564,
                40544, 34576, 23268, 7372,  58684, 63432, 41068, 47044, 6500,
                29228, 23016, 17184, 25700, 60600, 10752, 22400, 62000, 39156,
                31460, 6260,  63544, 21168, 7884,  46948, 3936,  50080, 22780,
                59840, 2840,  28832, 39900, 55760, 22712, 9420,  19268, 39228,
                3960,  43420, 17768, 62140, 2004,  23800, 8624,  16168, 60528,
                25564, 20688, 608,   29664, 59236, 14516, 3252,  41444, 61280,
                27864, 8132,  33960, 18228, 31744, 23392, 48244, 36808, 42976,
                7456,  32216, 57592, 23160, 44940, 55764, 8636,  42004, 29160,
                53656, 35184, 63976, 2208,  52852, 10040, 28552, 36188, 53200,
                64004, 38968, 50912, 17324, 42988, 24656, 9212,  29704, 61768,
                11492, 48096, 35600, 14256, 19060, 43468, 6752,  37864, 50712,
                41512, 5992,  65488, 18424, 25568, 49428, 44868, 27828, 51740,
                13988, 32008,
            },
            {
                7556,  28936, 43260, 21176, 33276, 49484, 41356, 30788, 7264,
                47984, 25380, 40188, 55932, 65492, 8736,  27072, 36584, 12184,
                49232, 15828, 55160, 42824, 18236, 9712,  24384, 13304, 59768,
                43664, 38948, 62776, 7004,  36996, 30508, 17968, 652,   43708,
                58360, 15448, 42208, 27824, 55460, 35460, 19516, 40336, 5972,
                56628, 34276, 51916, 7508,  14584, 31036, 42760, 26428, 52900,
                64272, 23064, 27924, 38916, 51128, 20232, 44000, 37800, 3360,
                13680, 44716, 65076, 38636, 22820, 35912, 32544, 11128, 16904,
                5156,  57884, 39508, 55800, 10664, 65460, 3092,  21412, 51020,
                18572, 10828, 28024, 51764, 6380,  39132, 25724, 14488, 404,
                18884, 11084, 23828, 30944, 17528, 59048, 46208, 560,   8432,
                21204, 13124, 28808, 59332, 6548,  36268, 58132, 51660, 26524,
                2188,  58532, 5568,  61464, 49364, 25604, 64300, 17236, 9400,
                60448, 31896, 15140, 51596, 7192,  40100, 244,   9288,  20924,
                59132, 24628,
            },
            {
                35300, 54176, 12840, 5740,  61328, 25084, 9976,  52580, 37792,
                62572, 33676, 18776, 14660, 30320, 1876,  51060, 59412, 5176,
                63356, 45548, 440,   31132, 48016, 28568, 52508, 1372,  19592,
                34740, 9352,  53392, 46452, 3388,  51224, 56076, 8852,  34260,
                19904, 52780, 10600, 38484, 62108, 12004, 53720, 29688, 25672,
                44136, 16256, 21288, 60768, 46900, 428,   62252, 6440,  34116,
                13552, 46460, 8440,  35144, 58752, 12440, 54236, 29096, 57908,
                47980, 34844, 4856,  17272, 53404, 6780,  57100, 50048, 36696,
                46096, 22188, 14180, 1280,  44312, 29304, 54864, 14544, 62584,
                46288, 3540,  36148, 63788, 19432, 33716, 50316, 58692, 47112,
                37664, 61620, 42648, 50532, 5924,  39412, 23280, 49264, 31464,
                57380, 45388, 2084,  54544, 32700, 48452, 3584,  40840, 18316,
                37900, 31856, 21584, 28636, 39028, 11432, 56952, 45360, 35828,
                20624, 26940, 56136, 36244, 29796, 54496, 63012, 37308, 46344,
                55468, 3068,
            },
            {
                18428, 64016, 47896, 39552, 17432, 400,   55768, 21800, 15952,
                44180, 1352,  50320, 60736, 37992, 45316, 23508, 54472, 32732,
                20080, 26068, 60860, 14000, 36356, 5540,  62344, 32320, 55400,
                49504, 14680, 28008, 20540, 33268, 40468, 23680, 45776, 64824,
                26384, 48512, 3028,  30804, 23432, 308,   58812, 14604, 47972,
                64092, 10164, 41624, 27000, 35916, 57728, 50168, 16836, 29108,
                56636, 1168,  49324, 15624, 30640, 5732,  24912, 41384, 18792,
                9772,  61412, 27836, 11812, 43288, 63232, 25608, 21060, 54764,
                29460, 63776, 49004, 25756, 51900, 37676, 5244,  33864, 26232,
                40344, 54300, 16224, 43200, 13172, 60944, 4392,  22792, 32324,
                56816, 8056,  26836, 14136, 34656, 55680, 11824, 62576, 42424,
                16108, 35892, 24004, 65356, 12320, 21352, 15108, 56328, 7708,
                44192, 53208, 46428, 8544,  54428, 1424,  22768, 29392, 2740,
                48792, 4840,  44664, 10432, 61080, 14224, 23436, 17516, 30700,
                11100, 40484,
            },
            {
                27872, 22572, 9384,  31540, 58868, 45544, 35060, 28784, 4752,
                58060, 12520, 28092, 6824,  20728, 41088, 11088, 16472, 29084,
                3168,  39464, 10716, 50468, 56728, 20976, 44768, 38236, 25416,
                60864, 4496,  42768, 64072, 11708, 58984, 15024, 29420, 4172,
                12872, 36152, 56520, 45468, 17556, 50552, 42336, 8244,  36664,
                1788,  32132, 54588, 5452,  18352, 23768, 11692, 38448, 41904,
                60816, 19652, 54976, 40908, 61060, 45708, 64660, 1316,  50356,
                31824, 21332, 52148, 39912, 47000, 30376, 15356, 2660,  42624,
                9648,  4296,  35172, 59244, 19776, 16204, 42140, 57644, 8692,
                30812, 60460, 24056, 1100,  29852, 45984, 10556, 40708, 16992,
                3628,  44444, 54260, 19688, 64560, 28000, 3724,  18824, 25780,
                5832,  52644, 9324,  40512, 27832, 44880, 60976, 30772, 23624,
                64568, 12892, 16708, 60324, 33596, 41796, 15268, 52556, 62080,
                13088, 58432, 39064, 22088, 3740,  47596, 34616, 5560,  57440,
                43412, 50208,
            },
            {
                60236, 36568, 2692,  51688, 26660, 13468, 63516, 48936, 41816,
                24524, 53644, 36172, 46516, 57248, 3892,  63884, 48476, 43208,
                57484, 52224, 33988, 23984, 3708,  40804, 15756, 11192, 7336,
                18064, 31212, 57084, 1968,  25888, 48928, 6820,  54316, 41572,
                59724, 22108, 5696,  61160, 39512, 26960, 33496, 20272, 60412,
                24196, 50964, 13540, 63072, 45808, 31472, 53836, 2560,  21752,
                9228,  32508, 26188, 4124,  21616, 10504, 34048, 16292, 37008,
                56624, 5632,  24572, 58740, 1056,  9052,  48904, 37952, 62260,
                18544, 32428, 40980, 11620, 7720,  61452, 23204, 49564, 12792,
                20260, 5616,  47856, 57424, 37192, 53928, 27312, 65276, 51600,
                30084, 12036, 36688, 1040,  47404, 41228, 32760, 53668, 38196,
                60176, 29868, 49596, 19224, 2768,  53952, 37060, 10460, 47676,
                868,   34932, 26196, 4636,  19560, 63412, 46832, 32456, 40216,
                25156, 33972, 16712, 50668, 27316, 41208, 59040, 51540, 25632,
                1420,  15000,
            },
            {
                7236,  45232, 57176, 42136, 20284, 38620, 7460,  10864, 19244,
                32612, 64468, 8992,  16800, 31928, 25532, 34632, 8160,  36916,
                14576, 7128,  64220, 18644, 46740, 58520, 26924, 64488, 48716,
                41520, 22752, 47356, 38436, 17472, 35448, 62284, 19600, 37696,
                50908, 16324, 33204, 8948,  13104, 65272, 4464,  45980, 55920,
                17308, 38976, 28416, 3408,  37588, 8316,  43296, 65424, 48628,
                35852, 44976, 12600, 52180, 37564, 57108, 27200, 53176, 7988,
                42128, 45592, 14268, 35740, 19208, 33404, 60144, 27720, 12916,
                51108, 23756, 54396, 47528, 27632, 31800, 45220, 816,   64288,
                38228, 52356, 32872, 9328,  21312, 15692, 5864,  35580, 20396,
                60220, 24544, 49408, 59224, 21956, 9572,  60996, 14636, 47840,
                1736,  43592, 13868, 62388, 34764, 5328,  25708, 51092, 18820,
                39576, 55404, 43236, 50704, 37640, 27080, 5652,  10100, 18944,
                7468,  53224, 64632, 1028,  32044, 18920, 8416,  12776, 38264,
                65100, 33412,
            },
            {
                55064, 12080, 24448, 16064, 4884,  54436, 30520, 59168, 51488,
                3128,  39564, 22032, 59904, 51248, 13512, 55352, 19440, 61388,
                22544, 28072, 43916, 30852, 8596,  35024, 51732, 29896, 532,
                54548, 34136, 13460, 52284, 9136,  44696, 31832, 2972,  10160,
                25000, 44456, 29612, 53892, 48212, 22836, 52764, 29948, 11616,
                6804,  43680, 48736, 58908, 20456, 56736, 27712, 13800, 25064,
                58068, 6152,  62844, 29344, 17608, 2712,  47008, 22808, 12024,
                63952, 29892, 3472,  62676, 53028, 23168, 55616, 6156,  45344,
                58408, 224,   15476, 64648, 38980, 4208,  55432, 35508, 28460,
                43092, 17056, 26704, 41072, 63196, 49676, 42904, 1772,  46404,
                14468, 39468, 4976,  31288, 15892, 45348, 7164,  23596, 34528,
                11160, 21696, 58120, 30916, 46856, 16724, 41296, 63548, 6520,
                29348, 59868, 9808,  23228, 57088, 13548, 49748, 59700, 55568,
                44248, 28796, 11412, 42676, 55876, 62936, 45188, 30096, 21740,
                48032, 19640,
            },
            {
                40756, 30824, 63728, 34552, 60444, 47720, 23024, 36532, 44936,
                15012, 27360, 48332, 128,   42112, 28900, 45124, 2784,  50056,
                40684, 848,   53972, 12424, 61244, 2944,  14052, 21700, 37484,
                10064, 60004, 5584,  29044, 58164, 22316, 53460, 27372, 46976,
                64204, 2068,  57872, 19276, 37336, 1508,  15944, 41340, 35808,
                63344, 22052, 33040, 10016, 16096, 51140, 34420, 5056,  18696,
                52700, 15924, 39600, 49800, 43156, 61636, 31260, 39872, 58604,
                18628, 26236, 50040, 38784, 11156, 43172, 16368, 39628, 20708,
                35728, 30148, 44012, 6376,  21500, 51996, 18512, 14344, 9696,
                56404, 2252,  60128, 13372, 6860,  25484, 58796, 32100, 54600,
                8236,  62960, 55936, 26320, 35280, 57652, 50612, 29500, 63384,
                55644, 26616, 38704, 8012,  54904, 22936, 12372, 57372, 33292,
                21724, 15516, 3484,  32052, 45224, 180,   29844, 36668, 23788,
                2288,  47508, 20588, 35092, 14496, 24436, 4844,  53984, 59804,
                3872,  27136,
            },
            {
                52752, 47132, 680,   9812,  43656, 28156, 1984,  17552, 6280,
                61036, 55232, 12116, 37300, 6040,  62692, 23928, 11292, 31648,
                59120, 15872, 47504, 37044, 25088, 40204, 55564, 45820, 62492,
                18816, 44092, 24840, 65416, 1284,  39052, 15472, 60456, 12624,
                34220, 39868, 7268,  24348, 43544, 60040, 9896,  57492, 25392,
                51892, 552,   60972, 26092, 41064, 2864,  62020, 46888, 38200,
                31176, 696,   23592, 9824,  20836, 7332,  14484, 49116, 288,
                34892, 54376, 15784, 7624,  47712, 26768, 31416, 2372,  60812,
                8812,  25944, 57320, 37080, 10680, 60948, 25668, 47228, 62896,
                22924, 50064, 45788, 34816, 55272, 38628, 17348, 11672, 22200,
                28480, 41940, 18516, 10412, 38740, 1600,  42652, 18096, 40388,
                3900,  16376, 50872, 1256,  43052, 60704, 28280, 2368,  44232,
                48320, 53448, 61868, 38780, 17828, 65204, 21320, 42308, 15772,
                62396, 37912, 58992, 6832,  49056, 39380, 17240, 34416, 43020,
                10568, 16452,
            },
            {
                6144,  22140, 38868, 50996, 19092, 12740, 65304, 52964, 33052,
                24040, 42924, 30136, 20304, 56880, 17028, 39776, 53504, 35904,
                26204, 5148,  56200, 21152, 50336, 17624, 32280, 5120,  26344,
                49932, 14632, 32912, 41100, 20084, 50496, 5904,  41892, 56776,
                18724, 52124, 14548, 62588, 34716, 28108, 47004, 32404, 5116,
                19544, 14652, 46348, 36768, 54620, 23212, 29800, 10988, 59144,
                41696, 55052, 64508, 33304, 58252, 35960, 55336, 25412, 6024,
                43864, 60268, 32036, 21392, 58332, 4508,  64124, 46160, 50324,
                12336, 53292, 17852, 48760, 32144, 43256, 2492,  40648, 33320,
                7388,  30580, 19588, 3656,  23356, 29784, 48368, 61880, 34328,
                52680, 2640,  45836, 51336, 65020, 21108, 5432,  53348, 10048,
                45840, 35856, 64848, 20532, 33028, 9248,  36620, 50144, 13832,
                7764,  35484, 27520, 11532, 51340, 7576,  34132, 52728, 12068,
                31608, 5264,  26516, 52108, 9616,  61676, 27852, 880,   50616,
                37400, 62712,
            },
            {
                28992, 14044, 55864, 25712, 32400, 57860, 37860, 41468, 11108,
                47256, 8100,  64012, 52064, 33476, 9112,  47692, 4244,  64100,
                18588, 42192, 32772, 7184,  65060, 10948, 58400, 42024, 8304,
                36056, 2868,  56248, 46512, 11520, 30208, 35640, 23564, 3996,
                28632, 48568, 31552, 11104, 55316, 3552,  18140, 64456, 49500,
                39988, 56684, 30632, 7716,  12864, 49360, 43128, 20612, 7808,
                26520, 13632, 47348, 27640, 3224,  45028, 17228, 63556, 38416,
                22496, 10816, 2044,  41416, 36040, 54652, 14096, 22872, 33152,
                41840, 63104, 1728,  23576, 13532, 59000, 28760, 54120, 16152,
                37364, 11788, 40076, 64788, 51952, 9636,  740,   44704, 5528,
                37124, 15412, 23772, 32848, 12692, 28584, 61112, 24872, 32380,
                59368, 28884, 12752, 47952, 25652, 53068, 18704, 62884, 39904,
                25060, 19624, 41808, 57732, 46984, 25620, 60620, 3320,  48680,
                56792, 18192, 41196, 22216, 30988, 45608, 57124, 12508, 23196,
                32000, 58288,
            },
            {
                44560, 35500, 3820,  62268, 7824,  48768, 4836,  26440, 56436,
                18348, 35620, 2872,  14676, 26360, 60496, 21844, 30196, 13392,
                46108, 9992,  60052, 27856, 44292, 1076,  29568, 53016, 20908,
                59656, 28496, 51820, 17104, 7584,  62976, 54696, 43928, 9720,
                63648, 21452, 20,    41632, 50088, 22092, 38800, 13520, 8908,
                27060, 43340, 3636,  62616, 18372, 57800, 1328,  63972, 36228,
                51240, 3972,  18324, 37776, 11892, 50760, 29444, 8668,  52296,
                46792, 27892, 65280, 48992, 17904, 29300, 9340,  37852, 18896,
                4232,  28300, 40036, 34768, 51580, 8684,  20236, 5428,  61912,
                51328, 58832, 26996, 56852, 15444, 42396, 20516, 55024, 26248,
                63812, 48688, 58672, 6716,  54460, 44104, 37156, 14416, 49756,
                19248, 7328,  55176, 4372,  40612, 15368, 56884, 5816,  31296,
                54876, 64216, 1484,  5480,  30976, 15104, 40164, 9848,  27668,
                44084, 64412, 12876, 54532, 2828,  36472, 19760, 64784, 46736,
                8372,  18452,
            },
            {
                49188, 60232, 40312, 16684, 43064, 21244, 14960, 30984, 784,
                61596, 22264, 49464, 40868, 44968, 36864, 1232,  50764, 57212,
                24532, 38480, 49372, 14340, 36548, 23920, 48696, 16084, 38988,
                63816, 10692, 23240, 38288, 59336, 26528, 48268, 16680, 33184,
                45972, 38428, 59392, 16052, 26388, 6148,  45076, 54572, 36212,
                59824, 21356, 53020, 34512, 38960, 27204, 32268, 16696, 45956,
                22720, 60516, 43396, 54124, 62040, 21596, 41524, 32728, 13732,
                19300, 34292, 53600, 12708, 24744, 44448, 61816, 52436, 57416,
                47488, 15088, 55364, 5984,  65332, 38504, 47888, 24784, 44544,
                272,   18176, 43500, 4776,  36140, 31584, 60200, 39268, 13120,
                19392, 9784,  30312, 39880, 17216, 48100, 8620,  57524, 120,
                42756, 38164, 23100, 60180, 44916, 29656, 668,   22624, 45876,
                10060, 16296, 44308, 23464, 36884, 55740, 20600, 53872, 35972,
                23128, 4252,  33740, 60700, 15996, 42772, 5688,  26076, 40148,
                53672, 1744,
            },
            {
                26988, 11232, 30524, 23520, 52536, 35244, 62988, 45292, 50756,
                38860, 28204, 59416, 4708,  11436, 55992, 17736, 40624, 6508,
                62316, 20844, 4612,  53732, 18436, 62820, 33444, 12456, 46940,
                4724,  34008, 43868, 432,   32192, 21256, 13324, 2168,  56156,
                25200, 5548,  52732, 35800, 56796, 62028, 31976, 23880, 1036,
                30116, 16300, 10240, 24516, 47424, 5124,  52140, 11932, 56064,
                34036, 6316,  30864, 9336,  24820, 1476,  55968, 4532,  61524,
                57428, 3048,  8008,  38992, 59736, 5728,  476,   31008, 25660,
                7660,  61292, 21808, 45440, 17580, 30444, 57848, 14172, 35296,
                29832, 22408, 8492,  48012, 11460, 23808, 50672, 7428,  33536,
                56820, 42812, 960,   61368, 26040, 4084,  33668, 21720, 27460,
                62328, 51344, 17152, 34020, 9936,  49728, 37916, 61384, 34848,
                49060, 28052, 59316, 50556, 12452, 63872, 1940,  45868, 16404,
                7836,  39368, 46836, 28344, 49792, 11040, 51752, 33608, 56336,
                13760, 21656,
            },
            {
                63704, 6928,  54624, 46340, 3240,  9740,  57316, 24404, 7424,
                12296, 16120, 54072, 31332, 24260, 65164, 33084, 27428, 44144,
                34640, 2072,  30840, 58224, 40656, 6800,  55852, 2328,  25540,
                54612, 19032, 57448, 49224, 6104,  53152, 40112, 36424, 61784,
                18392, 12716, 30008, 8612,  19216, 42640, 10396, 52112, 63208,
                5708,  48912, 44424, 65184, 14368, 41368, 61272, 8064,  38652,
                26020, 14920, 50312, 40636, 35096, 15528, 48228, 25964, 36700,
                42784, 23536, 45652, 16508, 32996, 49952, 21300, 39760, 12048,
                35936, 42700, 32012, 11072, 26044, 3336,  41996, 9820,  56056,
                49664, 64104, 33392, 53940, 61296, 28544, 17484, 2956,  46864,
                27700, 22588, 51728, 35748, 20468, 56084, 63888, 40224, 46556,
                11564, 3044,  30692, 64228, 14156, 26480, 58184, 19976, 12248,
                3468,  38852, 32924, 8384,  18128, 42516, 33232, 29040, 61884,
                51116, 58592, 19156, 284,   24576, 57700, 29936, 16968, 3136,
                38376, 43320,
            },
            {
                50740, 19620, 34360, 58204, 13644, 29388, 37500, 19064, 55076,
                43292, 34720, 46748, 20092, 6668,  48020, 13764, 8212,  55244,
                16432, 51300, 26120, 45680, 9452,  22492, 43356, 61092, 31052,
                41256, 14004, 27404, 64532, 15768, 42764, 60116, 7956,  28120,
                50596, 43660, 65228, 47792, 2376,  28364, 14408, 46676, 17844,
                37236, 55904, 32832, 1540,  57532, 22512, 29920, 18468, 44176,
                63488, 80,    57844, 19840, 65056, 44808, 59968, 18092, 11004,
                30344, 51012, 62568, 28752, 56632, 10304, 43192, 64668, 17408,
                50780, 58668, 1048,  54292, 49176, 34012, 62948, 21232, 4492,
                39388, 12768, 25868, 1648,  38120, 44280, 56012, 65364, 41056,
                15500, 62680, 5828,  45636, 13916, 10156, 30368, 52372, 15408,
                36112, 24832, 56320, 41792, 3840,  52792, 7844,  43004, 25336,
                52252, 56244, 21032, 61656, 26696, 48120, 6092,  56740, 13220,
                25580, 31632, 10188, 37628, 63796, 40680, 8028,  61552, 23692,
                58848, 31672,
            },
            {
                37036, 15968, 348,   26096, 64752, 49644, 41704, 1832,  32460,
                63740, 3692,  9564,  41628, 61648, 38356, 52932, 22984, 60716,
                10968, 38004, 64652, 13428, 35424, 28452, 51520, 16392, 36952,
                5752,  45844, 8960,  34948, 23868, 29828, 11408, 46928, 22528,
                3616,  39528, 33764, 21504, 60088, 39920, 34744, 58116, 21768,
                41528, 12736, 27756, 19968, 9144,  35696, 50028, 2968,  53428,
                21492, 47876, 28788, 12472, 5208,  27616, 8084,  39732, 54816,
                6252,  14640, 20832, 1872,  36772, 26264, 54336, 46548, 2896,
                24116, 28972, 19740, 38152, 60912, 15664, 51784, 27908, 46260,
                17076, 58512, 41852, 19956, 14940, 5552,  21992, 9364,  36348,
                49812, 11668, 32084, 59688, 38284, 49528, 1672,  18564, 5472,
                58888, 44524, 8420,  21572, 35436, 46664, 17704, 32184, 65516,
                14872, 6296,  41484, 360,   53368, 37672, 10512, 21428, 40820,
                3924,  43748, 55260, 48440, 14752, 21116, 45120, 35948, 48000,
                5460,  10080,
            },
            {
                46032, 59852, 42720, 38744, 20692, 5996,  16728, 61076, 27588,
                22028, 51244, 25432, 55704, 29060, 516,   19048, 45540, 31484,
                41916, 6564,  18056, 48392, 184,   62164, 20144, 10144, 58568,
                49900, 21188, 60820, 38592, 55672, 1680,  19860, 52600, 31988,
                58004, 10052, 15124, 55584, 24836, 6968,  50300, 3560,  26072,
                7816,  50828, 59652, 38104, 53880, 43388, 24988, 59504, 31964,
                10560, 41252, 33748, 55520, 38588, 52984, 32136, 22500, 49228,
                63960, 34444, 41492, 59072, 13044, 18860, 6392,  33540, 14476,
                62688, 9560,  47580, 5296,  12704, 23912, 1768,  37392, 54704,
                7280,  31084, 50900, 62872, 47372, 34532, 51860, 30880, 25304,
                2176,  54564, 17860, 28376, 23852, 42328, 55012, 26964, 61828,
                31972, 48924, 18876, 54312, 28736, 62764, 39324, 1948,  48080,
                36960, 29904, 46336, 16564, 31796, 23748, 59640, 49396, 34712,
                65128, 16820, 22608, 5508,  32308, 2800,  54872, 12100, 19096,
                52896, 28260,
            },
            {
                4340,  23200, 51476, 11440, 32776, 44988, 53088, 10408, 48164,
                13888, 58192, 36732, 16988, 11836, 58996, 35592, 4888,  49476,
                28624, 56920, 24612, 53284, 40160, 31944, 47012, 3580,  24568,
                29132, 53000, 12796, 4996,  48036, 62480, 44204, 35704, 16460,
                63184, 26764, 660,   51768, 44592, 12556, 61592, 31636, 43824,
                64196, 30544, 2788,  17048, 47520, 4348,  11760, 15888, 37532,
                5556,  62612, 16748, 23256, 2396,  47608, 14264, 58308, 544,
                26820, 46184, 4576,  52892, 48604, 39420, 60880, 30072, 37592,
                52404, 42084, 34592, 56484, 40536, 45016, 32328, 59036, 11736,
                24748, 35836, 3080,  10084, 29148, 58920, 12696, 60832, 19236,
                38672, 57948, 44260, 4024,  8080,  64992, 34404, 22084, 41452,
                10844, 37848, 1156,  60124, 12808, 5888,  23588, 54820, 10500,
                59676, 23072, 57340, 12304, 63612, 4812,  44788, 14616, 1128,
                52824, 28212, 59212, 36260, 51572, 62580, 29024, 42956, 25680,
                65348, 14508,
            },
            {
                30836, 61692, 8188,  56140, 28384, 59484, 24096, 35156, 39700,
                7608,  45160, 2388,  32284, 49920, 43516, 26364, 15044, 63772,
                1840,  20572, 33996, 59216, 5192,  14876, 56272, 38380, 65292,
                34388, 860,   42368, 31320, 17780, 26632, 7272,  54880, 4444,
                42056, 49036, 36764, 17684, 29396, 38336, 20056, 55236, 16192,
                10996, 23232, 40144, 61968, 26916, 33580, 64604, 55916, 28104,
                45872, 52056, 7984,  43284, 61144, 36156, 20200, 43684, 10032,
                38172, 17816, 24296, 31676, 8480,  22800, 55480, 3688,  20528,
                57888, 7012,  16892, 27148, 20848, 64932, 8532,  18700, 42220,
                63744, 21136, 45856, 55576, 24372, 40168, 568,   43960, 48552,
                7444,  33284, 21784, 52460, 47900, 16288, 12564, 46172, 6984,
                51652, 15120, 26692, 40828, 50492, 31444, 15884, 44048, 27268,
                19088, 8156,  34072, 50516, 42808, 27904, 55632, 18344, 30856,
                39204, 8560,  12276, 46228, 24764, 7524,  16484, 34412, 908,
                56432, 40932,
            },
            {
                20464, 35352, 16888, 47056, 1504,  14808, 3984,  55820, 30716,
                18388, 64868, 41160, 21056, 62740, 5844,  52484, 22248, 55772,
                37056, 12404, 44348, 9100,  27456, 22848, 43228, 11656, 18244,
                45080, 15344, 57020, 22584, 51420, 33892, 14068, 38896, 24324,
                11704, 30940, 59488, 5788,  64480, 9204,  47920, 1068,  34124,
                52772, 45248, 6068,  56412, 13896, 21988, 39556, 48288, 2244,
                20436, 25452, 58216, 30784, 11488, 26624, 4408,  62168, 29592,
                54264, 59936, 11864, 65040, 42564, 15744, 46992, 27004, 44680,
                12576, 24796, 49580, 60272, 30648, 4428,  50572, 47380, 28596,
                52540, 5240,  38404, 15436, 18520, 7864,  53464, 27424, 15648,
                63724, 41148, 10796, 61424, 36716, 30948, 56368, 3276,  58484,
                29532, 64472, 23008, 45640, 9448,  57504, 61660, 34708, 49340,
                3748,  62364, 38352, 2384,  20308, 9004,  36352, 62280, 24984,
                47864, 61132, 42496, 18524, 57632, 39968, 49344, 60476, 9272,
                37632, 48964,
            },
            {
                2476,  54580, 25152, 40220, 64304, 36536, 42788, 21404, 60296,
                4860,  51856, 28180, 14720, 8884,  39376, 33256, 10440, 46564,
                40880, 61716, 16876, 51800, 64060, 35944, 49720, 6552,  53924,
                26512, 62264, 8104,  40500, 59424, 10276, 64944, 47344, 56748,
                18888, 45792, 22000, 53256, 33372, 24088, 42144, 27400, 58696,
                36816, 18984, 49336, 32076, 8932,  44008, 6812,  18024, 60376,
                35272, 13864, 39092, 49768, 18480, 55780, 51436, 40072, 15916,
                33380, 7108,  50488, 36324, 28580, 988,   62024, 10088, 40140,
                64292, 32228, 92,    43452, 10604, 55296, 15980, 33888, 1060,
                14128, 60724, 31548, 58012, 43060, 33660, 61928, 36820, 23388,
                3548,  29632, 14088, 26160, 300,   20424, 40136, 24508, 35468,
                18336, 53764, 33432, 5376,  37064, 19848, 524,   39908, 13416,
                52868, 29468, 45696, 24740, 54148, 40620, 12720, 50752, 6696,
                20772, 3460,  32940, 27440, 1620,  11244, 21524, 31480, 45288,
                23964, 12660,
            },
            {
                63332, 43480, 5084,  31412, 19344, 50876, 9088,  46456, 26476,
                11660, 34584, 48388, 24476, 54920, 29356, 59668, 18172, 3140,
                25404, 7308,  31700, 47184, 1300,  19180, 29424, 60680, 32716,
                2880,  48472, 35816, 28512, 3236,  43696, 20788, 30060, 2716,
                61168, 8344,  40184, 2180,  57200, 15792, 50980, 5008,  13728,
                63492, 2356,  25212, 60288, 36296, 54488, 50832, 29864, 10880,
                55152, 45100, 856,   6236,  64756, 34272, 8648,  24680, 48720,
                21880, 45064, 2984,  19852, 57192, 53196, 34728, 18456, 50624,
                5404,  36068, 53956, 14084, 39276, 23376, 37188, 62484, 26124,
                40716, 10688, 48944, 26676, 2664,  50184, 20736, 10256, 56548,
                51220, 45552, 59284, 54724, 42844, 50732, 63340, 10196, 49352,
                43620, 2064,  13268, 62212, 48376, 25740, 30100, 56152, 22144,
                41636, 17424, 56916, 14928, 64160, 32572, 1576,  58296, 44036,
                35660, 56120, 14132, 53968, 36944, 63868, 52684, 4120,  57956,
                17760, 52228,
            },
            {
                28948, 10208, 48152, 58560, 12540, 27684, 61432, 16420, 53788,
                38220, 57064, 1472,  60580, 44452, 4628,  47948, 36084, 54004,
                28728, 57748, 21684, 38460, 12668, 41828, 56860, 9856,  39916,
                23332, 19884, 11884, 53576, 16868, 49968, 5596,  36900, 26084,
                51148, 34488, 13388, 27800, 44928, 10656, 61932, 39336, 22340,
                29808, 42560, 10632, 16248, 28268, 344,   62128, 24220, 42524,
                32292, 62984, 22748, 28436, 41988, 14912, 46020, 1488,  63792,
                13092, 57940, 25816, 38940, 13844, 7532,  23640, 30616, 56516,
                16184, 22392, 61240, 28712, 45752, 58260, 3120,  20020, 44372,
                55084, 35516, 22148, 5756,  64368, 13384, 46372, 32172, 6272,
                39160, 18996, 35164, 8592,  17356, 5228,  27848, 14580, 59960,
                7900,  21708, 56764, 42292, 16556, 52632, 11424, 7600,  64896,
                4516,  33200, 10644, 6176,  28720, 47208, 23276, 15596, 29624,
                10404, 64656, 49488, 22444, 44208, 29732, 14520, 41272, 26876,
                6544,  33464,
            },
            {
                59260, 16176, 37068, 22276, 55332, 6116,  33640, 2652,  41576,
                22708, 8116,  18908, 32432, 12252, 16372, 23292, 65464, 13724,
                43540, 49272, 3848,  60916, 53796, 25176, 4936,  16656, 51908,
                44388, 59856, 39044, 64024, 25204, 33164, 58572, 14948, 53896,
                42872, 17024, 63804, 49388, 36432, 20460, 31696, 55496, 8496,
                46040, 57296, 51808, 39716, 46592, 19792, 14284, 38304, 3676,
                8588,  17592, 48360, 37708, 57696, 21064, 31592, 54640, 37540,
                5352,  42816, 30284, 63168, 46064, 49392, 41488, 60168, 2172,
                38116, 48264, 8764,  18580, 5580,  32668, 52708, 9688,  59440,
                7032,  17888, 45392, 53904, 29492, 37948, 60196, 16736, 28036,
                64908, 2000,  24224, 32364, 47616, 57684, 37412, 32816, 46860,
                39476, 31060, 27208, 35812, 3100,  59872, 44200, 37536, 46820,
                26816, 59156, 48968, 42980, 37284, 18684, 60492, 53608, 41380,
                25744, 17848, 39084, 8252,  4832,  48124, 19492, 35536, 62036,
                46872, 39408,
            },
            {
                7832,  26144, 51604, 196,   39236, 44816, 49368, 30352, 64180,
                13876, 47292, 42864, 63396, 37924, 51664, 41400, 276,   31984,
                19356, 10652, 35372, 15324, 30504, 45560, 34760, 62968, 27804,
                14024, 6724,  30904, 1904,  46068, 22212, 41204, 62920, 10020,
                16,    31144, 23348, 3876,  59792, 6224,  25784, 47660, 17212,
                33852, 3680,  21224, 65468, 5476,  56300, 34032, 49288, 53804,
                26840, 59276, 12880, 52300, 9940,  3496,  60304, 27820, 18224,
                34636, 52992, 9756,  17060, 32988, 4040,  11148, 27580, 45092,
                12900, 25352, 42152, 51228, 64596, 26884, 47800, 13252, 31136,
                25228, 63392, 14804, 40028, 9600,  24000, 1368,  42440, 55020,
                11468, 49732, 41560, 62592, 12772, 22696, 53228, 1164,  25388,
                17036, 62832, 51464, 9660,  20372, 32588, 24208, 15608, 51304,
                19400, 36044, 22620, 61476, 3672,  50344, 8108,  33812, 4432,
                46668, 96,    31792, 60828, 26064, 58760, 9684,  54616, 1024,
                13364, 21192,
            },
            {
                41896, 64140, 34264, 14080, 62728, 24472, 10648, 19800, 36080,
                58160, 26292, 29660, 3372,  21516, 58488, 27604, 8704,  60248,
                39200, 52452, 63572, 23088, 8140,  59200, 20296, 48004, 380,
                37208, 57780, 49556, 9316,  56304, 12684, 7060,  29116, 19996,
                48292, 37524, 55268, 43608, 18312, 53756, 41596, 648,   63056,
                37460, 26628, 13380, 32684, 23876, 41848, 11368, 63660, 20912,
                46124, 36192, 4592,  32556, 24800, 44508, 49868, 11224, 41176,
                22684, 59376, 616,   51112, 21540, 61436, 36440, 20060, 52364,
                63576, 33856, 55752, 1308,  35764, 16092, 22336, 42444, 38548,
                50700, 712,   34236, 59140, 51736, 19464, 57140, 35584, 47432,
                21476, 15380, 53888, 4136,  29944, 44708, 19552, 65256, 6208,
                55360, 12144, 4132,  48088, 40060, 63376, 54224, 1404,  31284,
                9760,  2552,  55252, 14288, 26244, 11920, 39216, 21000, 57796,
                62964, 51136, 12516, 55156, 34148, 16936, 42792, 23080, 30920,
                50032, 55724,
            },
            {
                18164, 46448, 5220,  29984, 42632, 17188, 59916, 51944, 6512,
                808,   49960, 54276, 10904, 34088, 5976,  48556, 55120, 24608,
                46028, 5804,  27300, 42232, 50464, 3532,  12092, 40792, 54760,
                32980, 19428, 25932, 16408, 34812, 51332, 38140, 44984, 57432,
                25624, 61500, 7316,  11216, 28592, 33240, 14140, 50392, 10824,
                59620, 52920, 7284,  49032, 59016, 28792, 2276,  16428, 31020,
                7112,  43104, 55328, 19484, 61400, 39124, 16320, 7072,  62836,
                47516, 14428, 28524, 43912, 25172, 54952, 15560, 58220, 8260,
                3816,  29536, 20988, 11552, 40268, 7960,  56768, 62848, 6372,
                20604, 46980, 27076, 4036,  31720, 44568, 5256,  26280, 8024,
                31124, 37728, 25552, 60756, 7420,  39960, 9996,  49652, 41688,
                34256, 43984, 23304, 58056, 28704, 6796,  13112, 41836, 57408,
                63748, 45032, 40460, 30388, 52268, 64792, 45268, 27652, 14432,
                36632, 24128, 19632, 45584, 40300, 2304,  51404, 64544, 37292,
                25364, 3300,
            },
            {
                32244, 10368, 53360, 20676, 57092, 7752,  28040, 38472, 45596,
                23700, 17364, 39816, 61700, 45252, 18660, 36904, 11852, 15976,
                34004, 1760,  56704, 17488, 37252, 32204, 26956, 64336, 23580,
                10244, 45852, 62516, 42540, 27888, 60484, 18152, 5112,  33332,
                13008, 40652, 16156, 51392, 65036, 38524, 58392, 23664, 19696,
                29340, 43972, 38880, 18296, 9840,  45008, 52408, 39668, 60504,
                23720, 14568, 65088, 29368, 2268,  34464, 57204, 26192, 31308,
                55476, 5748,  37740, 65472, 40276, 6480,  47068, 31328, 42900,
                39384, 16912, 61712, 44832, 59480, 49848, 28088, 2580,  36572,
                54108, 58152, 12568, 41964, 17120, 60988, 50160, 14160, 63236,
                58180, 2432,  45660, 34548, 17180, 58336, 36168, 14020, 29080,
                20132, 60964, 37960, 14860, 45684, 18764, 36376, 49448, 21748,
                28188, 16980, 7208,  34508, 19196, 864,   56604, 32160, 5408,
                42448, 9488,  30132, 6096,  62332, 13952, 28516, 7044,  11340,
                44380, 61084,
            },
            {
                39016, 27384, 59320, 35672, 48076, 2964,  54812, 33224, 11956,
                60848, 31420, 8276,  15152, 25340, 57008, 29316, 43012, 61056,
                20592, 30672, 48060, 13420, 58476, 52904, 43756, 6856,  16048,
                51036, 3104,  39516, 6488,  54492, 1072,  23848, 63656, 53156,
                3096,  22432, 35228, 46256, 20792, 2748,  8352,  45524, 34904,
                2192,  15484, 55392, 61208, 35756, 4644,  25904, 56856, 35148,
                812,   48636, 40488, 9284,  47052, 13020, 51212, 20872, 3644,
                44980, 33696, 19072, 8984,  12960, 34436, 1580,  26380, 49908,
                22976, 53400, 5660,  31624, 23976, 19120, 33684, 45472, 15320,
                30184, 8388,  23476, 65476, 37268, 10928, 22964, 33644, 39796,
                52188, 10320, 20248, 50628, 27900, 55068, 24076, 60044, 2164,
                52796, 8044,  31892, 0,     54908, 25684, 61172, 4068,  34028,
                12360, 60008, 53440, 47188, 23456, 41720, 9124,  17452, 54044,
                47328, 59764, 52548, 37456, 22196, 56116, 32884, 47096, 20128,
                57764, 15240,
            },
            {
                23000, 49752, 1844,  12832, 25220, 40580, 15752, 64920, 21096,
                42388, 56280, 35928, 52468, 4368,  64708, 1116,  50292, 7380,
                53272, 64212, 39764, 8500,  22292, 2524,  18680, 60264, 30056,
                36328, 58448, 21680, 14272, 31344, 47160, 11540, 49192, 29928,
                43368, 56752, 60000, 5312,  25852, 49292, 30596, 56376, 40632,
                63860, 25052, 6444,  30472, 22024, 48308, 13568, 19700, 8356,
                51164, 27240, 17628, 53832, 22920, 42908, 63404, 37940, 10468,
                52400, 23532, 57152, 48816, 59776, 52084, 18304, 64148, 10012,
                13584, 57272, 47284, 37496, 14192, 4056,  55436, 10744, 61108,
                18624, 43872, 49052, 28292, 6160,  53488, 46128, 4,     28472,
                16812, 42892, 64684, 12460, 3212,  47032, 5868,  32616, 44608,
                26212, 17456, 49168, 63680, 10764, 30420, 51924, 8556,  47624,
                25216, 38476, 2932,  11144, 62724, 37072, 48848, 60992, 22036,
                34216, 1312,  15984, 26728, 49248, 3952,  17656, 39756, 54096,
                34796, 6384,
            },
            {
                65484, 41676, 19052, 61920, 44912, 30576, 9552,  48940, 3556,
                27496, 5420,  48092, 22300, 40800, 32820, 14324, 39140, 23344,
                35848, 11020, 45040, 25476, 61952, 34928, 46904, 39340, 55256,
                12608, 25372, 48960, 65320, 35868, 20152, 40552, 7964,  37172,
                17500, 27432, 10136, 32360, 41732, 12464, 61408, 17148, 4536,
                51780, 47036, 12308, 42832, 64728, 53676, 32524, 41932, 63188,
                11592, 37760, 58636, 33124, 6364,  28408, 372,   17404, 59980,
                40640, 14824, 1996,  30896, 27624, 21804, 44612, 38552, 60120,
                35668, 872,   25776, 9224,  52816, 64980, 41024, 25988, 51588,
                32680, 39304, 2228,  55756, 20036, 30888, 41352, 62272, 21196,
                5164,  55836, 30496, 39060, 22304, 42288, 15876, 51456, 64208,
                11608, 57168, 41420, 21848, 37356, 43880, 16024, 40508, 56528,
                18936, 44152, 29152, 55816, 15220, 31188, 26460, 4072,  13076,
                28820, 65216, 40984, 11600, 58344, 43568, 63692, 8856,  26856,
                528,   52200,
            },
            {
                45628, 29244, 8952,  37548, 52936, 22260, 58688, 35360, 53264,
                14016, 19084, 62456, 28732, 11664, 46536, 19748, 55508, 27408,
                2724,  17876, 29604, 5360,  50108, 14360, 26504, 8800,  1596,
                43252, 33252, 4728,  9408,  44044, 59784, 26540, 57584, 13936,
                50632, 768,   54420, 63320, 15704, 52912, 36876, 22280, 9920,
                27124, 33212, 20180, 56,    37980, 16492, 3764,  59628, 29208,
                45320, 20816, 3012,  62232, 15392, 56800, 35908, 47904, 30300,
                26620, 63848, 42968, 36964, 5040,  55692, 14996, 3664,  28508,
                32236, 18616, 62804, 43448, 29232, 48352, 35744, 364,   22076,
                5644,  63508, 12932, 34620, 60148, 15552, 8968,  57616, 48284,
                36332, 24512, 7548,  58600, 33564, 62312, 9456,  20608, 37040,
                5020,  28528, 34556, 3204,  58708, 6204,  23484, 65512, 900,
                32032, 62180, 5424,  50852, 20352, 6720,  57244, 40128, 50580,
                44476, 19000, 54668, 7980,  30800, 35708, 24144, 48436, 59680,
                31004, 12232,
            },
            {
                17264, 4100,  56648, 32888, 5488,  17688, 492,   24880, 45492,
                38764, 58104, 43844, 2088,  51460, 59232, 9280,  62288, 43920,
                49128, 59616, 56592, 41332, 54204, 31940, 64776, 21420, 52292,
                16828, 61604, 56472, 29140, 15728, 52088, 2752,  33168, 64232,
                21676, 45072, 38840, 19716, 47512, 2292,  28400, 44632, 55172,
                38576, 60072, 50680, 57852, 27988, 10284, 46312, 24064, 6200,
                34676, 52640, 25236, 43944, 39448, 50180, 24320, 8568,  54444,
                5356,  20224, 11420, 46272, 61980, 9668,  41536, 24012, 54128,
                45952, 50928, 39652, 21444, 6592,  17144, 12324, 59388, 46848,
                56724, 16864, 50656, 25704, 47300, 3920,  38400, 26712, 12804,
                51724, 44324, 14572, 48868, 672,   53692, 29856, 40392, 48108,
                23352, 54184, 14356, 47264, 19172, 53472, 28096, 35324, 10140,
                49008, 13436, 24412, 36016, 42488, 46052, 64076, 23784, 10232,
                36092, 3232,  24956, 46436, 20392, 2016,  12948, 15860, 42480,
                21376, 36652,
            },
            {
                54832, 26156, 47704, 13696, 63996, 50548, 40904, 62752, 31628,
                7096,  10288, 34568, 17192, 25640, 36640, 30584, 4980,  15776,
                33148, 12828, 36672, 20000, 52,    10756, 44460, 58100, 28080,
                37012, 19940, 46376, 24060, 39072, 7056,  18840, 46056, 25128,
                5656,  34856, 29280, 7704,  24548, 58944, 34160, 6108,  62652,
                13564, 17788, 7976,  34884, 23208, 55148, 39812, 49616, 17252,
                56308, 14168, 8164,  31064, 11116, 19684, 64936, 13748, 45168,
                33196, 58456, 50256, 17448, 25328, 32468, 48608, 58248, 7512,
                12032, 4292,  14776, 60552, 56964, 32336, 23688, 38708, 8640,
                28576, 42172, 37052, 10132, 22536, 43596, 54400, 18356, 1696,
                32740, 63428, 19500, 37644, 26772, 17428, 57724, 13356, 1640,
                60188, 43168, 8700,  61892, 32540, 12216, 45904, 59248, 21248,
                54628, 39792, 17164, 60076, 8912,  33456, 1564,  18052, 53724,
                58924, 32256, 62952, 38568, 60356, 51844, 56440, 38876, 6768,
                50200, 61472,
            },
            {
                10124, 39864, 59596, 23568, 43124, 28644, 10676, 15204, 54772,
                20996, 29724, 52712, 64176, 5848,  42716, 21744, 53580, 38728,
                25116, 7932,  61532, 23560, 47584, 16012, 35344, 6360,  41028,
                3756,  50812, 12164, 496,   54768, 30744, 61052, 41464, 10532,
                52636, 61720, 13664, 55776, 40724, 11048, 50100, 20556, 41924,
                1816,  31376, 48832, 43744, 3220,  61776, 12844, 31496, 1080,
                64640, 38224, 48648, 60892, 54700, 4552,  34204, 41640, 1440,
                22796, 39116, 7656,  35176, 52584, 60,    19844, 35956, 63964,
                22372, 42240, 27660, 34400, 1740,  53792, 44116, 63248, 14800,
                31768, 20116, 1536,  61964, 52592, 31532, 64576, 35392, 58828,
                9892,  29164, 4720,  60932, 46120, 7968,  24844, 34428, 50992,
                31016, 16584, 38792, 24488, 50348, 1552,  37944, 15300, 4396,
                43376, 29756, 2056,  56172, 27860, 52260, 14708, 30588, 39364,
                6980,  13672, 49700, 16468, 4968,  33288, 29092, 62768, 25356,
                3088,  32660,
            },
            {
                14992, 20636, 1196,  35136, 7740,  20016, 37344, 48704, 2920,
                61196, 47388, 23820, 39720, 14892, 47860, 57912, 1380,  65360,
                46224, 50896, 4388,  28896, 39220, 56196, 24928, 48908, 62644,
                9832,  32252, 42040, 63924, 35200, 48208, 13296, 57120, 37436,
                17088, 48840, 3596,  44060, 32688, 65436, 15440, 53776, 26544,
                46304, 24016, 64396, 15144, 51984, 19420, 36600, 58356, 26060,
                41680, 19056, 28044, 1864,  22384, 46712, 27508, 51640, 60440,
                15848, 56060, 27696, 63088, 13488, 59556, 40364, 15696, 30384,
                52004, 55888, 37352, 9432,  49420, 19628, 27036, 3268,  51040,
                54992, 44792, 58724, 5840,  27868, 13544, 7592,  21004, 46944,
                42512, 55620, 22952, 11996, 52976, 41548, 65148, 45364, 6432,
                19732, 63420, 3888,  56088, 29436, 42116, 62472, 26200, 51744,
                33872, 63800, 46724, 11832, 38536, 21040, 48572, 61016, 26688,
                45932, 22520, 28236, 9388,  43404, 22164, 11044, 45940, 18744,
                53012, 44464,
            },
            {
                64312, 49268, 30424, 45508, 53136, 60888, 57060, 33816, 26912,
                42576, 12476, 996,   56096, 8564,  33012, 28032, 19292, 10792,
                30144, 17380, 42300, 53340, 63732, 3036,  18968, 30556, 14500,
                22880, 59660, 26396, 16076, 20828, 8996,  27724, 22628, 1276,
                31256, 26304, 59572, 18328, 22736, 424,   29972, 8536,  35644,
                59476, 5736,  39152, 29260, 9236,  42416, 5064,  21652, 51200,
                6952,  45000, 12220, 36288, 59240, 40360, 17652, 9904,  30592,
                48364, 3684,  42660, 21592, 5912,  28828, 44852, 10120, 2312,
                47128, 17044, 24588, 64448, 45828, 11416, 40860, 35576, 6396,
                24972, 11516, 33800, 39772, 17244, 49692, 41284, 3260,  25424,
                15724, 39284, 49856, 35636, 31804, 2116,  21344, 11344, 54404,
                36872, 26564, 44672, 11000, 20568, 6776,  17496, 56940, 9260,
                23028, 7088,  18916, 25324, 61996, 3452,  43400, 11284, 55388,
                408,   41800, 52988, 57384, 36524, 48852, 1236,  55112, 36060,
                8348,  28172,
            },
            {
                37144, 5724,  55428, 17504, 13036, 3620,  24192, 6580,  17376,
                38100, 58824, 19944, 35284, 50304, 60908, 13116, 52552, 40768,
                35528, 59364, 21648, 9256,  33340, 12340, 59024, 52660, 39620,
                46828, 53752, 6120,  57648, 45020, 51364, 4332,  64948, 43720,
                54032, 40052, 10176, 36404, 46468, 57716, 39416, 49028, 18464,
                55292, 11880, 20876, 56492, 34112, 26732, 63500, 46112, 10540,
                33516, 62460, 53408, 31596, 14852, 7352,  56512, 62928, 24724,
                36776, 12528, 32092, 46640, 54592, 37472, 50404, 26052, 62376,
                33260, 6264,  58176, 3948,  30540, 15272, 61916, 57552, 17992,
                48732, 65136, 21776, 46880, 56256, 36560, 61540, 30160, 51836,
                60548, 8112,  2996,  27636, 14936, 59296, 38396, 29176, 57468,
                14528, 49260, 32780, 59324, 53120, 36076, 47736, 31492, 39532,
                14328, 49636, 54152, 41884, 34808, 57668, 7904,  36628, 18548,
                64676, 34244, 5768,  61684, 17740, 26484, 64912, 13660, 41348,
                23136, 57612,
            },
            {
                42856, 11504, 25948, 62904, 41152, 32164, 51472, 46520, 65044,
                9484,  31384, 45284, 25688, 17604, 4140,  44512, 24092, 6572,
                55164, 1812,  14756, 45484, 26228, 37340, 43672, 7908,  980,
                34156, 18112, 37712, 2452,  31152, 39824, 55676, 35464, 15068,
                7812,  20616, 62608, 4608,  52340, 25288, 12788, 63348, 2628,
                41404, 32768, 47664, 60536, 664,   53612, 13976, 30092, 56016,
                16520, 23612, 4696,  49956, 25312, 44564, 37868, 2184,  19520,
                53636, 60956, 8672,  18536, 64800, 3016,  22832, 57560, 18992,
                41008, 13264, 43896, 21212, 39100, 51948, 23104, 8308,  29224,
                38308, 2744,  14076, 26664, 312,   10316, 23192, 54996, 12312,
                33104, 19304, 63916, 56352, 44428, 18208, 50576, 4476,  42596,
                23068, 956,   8172,  39984, 13956, 23708, 63620, 4016,  44836,
                60172, 36408, 30236, 5588,  16188, 22580, 32140, 25644, 50296,
                14868, 20804, 31116, 12120, 40496, 4364,  30020, 58728, 33440,
                16948, 2488,
            },
            {
                31580, 50892, 22272, 38112, 8828,  28244, 43692, 14308, 22424,
                54200, 4668,  52136, 63768, 41220, 30460, 37724, 62708, 47308,
                26796, 31688, 61836, 49064, 5436,  51216, 29212, 21068, 65176,
                11612, 28536, 43360, 62064, 13948, 24392, 18412, 29640, 50036,
                58384, 33216, 48324, 28852, 15864, 7260,  31748, 21728, 45068,
                27284, 16072, 4280,  24484, 44436, 17836, 49320, 40340, 2484,
                37440, 58864, 40852, 20048, 63528, 11180, 33092, 47552, 5964,
                43380, 26496, 49736, 40760, 15488, 34408, 12176, 7756,  53132,
                36608, 28464, 49904, 61192, 34056, 500,   54792, 46472, 32856,
                60508, 42476, 53044, 31168, 63180, 19636, 45400, 6404,  42964,
                37616, 47280, 22184, 40120, 5932,  24668, 63124, 9764,  33324,
                60936, 17832, 64580, 46912, 27940, 1884,  54676, 11588, 20820,
                26532, 48,    65112, 10812, 47968, 53096, 63288, 45428, 4248,
                58008, 39148, 47440, 24860, 52204, 44824, 20964, 50140, 6140,
                46560, 62012,
            },
            {
                19564, 56340, 1900,  47452, 59188, 19416, 152,   57356, 34680,
                40408, 28748, 15248, 7232,  11112, 57252, 468,   20752, 16240,
                9880,  57436, 39460, 23460, 18184, 55688, 15352, 59964, 25488,
                49588, 56700, 22600, 9576,  47916, 59228, 5636,  11236, 45896,
                2008,  24236, 12196, 40752, 55868, 43324, 60748, 37552, 53396,
                9704,  62148, 50952, 38584, 10892, 36120, 7440,  22776, 61440,
                28304, 9084,  48104, 172,   55092, 27392, 52028, 22120, 15716,
                58348, 34756, 796,   23888, 29916, 58872, 42316, 47592, 31564,
                1192,  55300, 9844,  17712, 7200,  27176, 42932, 19868, 10900,
                5100,  16964, 58424, 7892,  40352, 34960, 57640, 27212, 17420,
                1292,  29064, 10168, 52132, 13072, 48192, 36064, 27452, 46136,
                53008, 37968, 30536, 51788, 19496, 43016, 34820, 29700, 58232,
                51928, 18308, 44220, 56408, 27608, 2612,  13292, 37804, 9964,
                28332, 54132, 1964,  8408,  60952, 15180, 37744, 10204, 53652,
                27220, 13136,
            },
            {
                39832, 7780,  33772, 14924, 52160, 36008, 63284, 11268, 49512,
                2848,  61680, 21512, 36224, 49212, 23148, 54272, 33688, 51272,
                42464, 36364, 12012, 2812,  64032, 34608, 40056, 4344,  44756,
                36396, 16148, 6680,  53144, 33112, 36720, 42468, 63108, 26848,
                38132, 53304, 18200, 64688, 34332, 1460,  50248, 17416, 4944,
                35000, 57128, 30432, 20448, 65212, 58368, 31448, 54524, 19332,
                44344, 14100, 34772, 30680, 17360, 6836,  41320, 65380, 30336,
                38952, 11396, 62596, 55500, 4880,  51124, 26832, 61248, 16144,
                22256, 65156, 25188, 46972, 59672, 37228, 13932, 63940, 50020,
                27788, 36220, 47708, 24136, 51372, 14988, 3916,  48592, 65052,
                53780, 58200, 34888, 60776, 31312, 44,    55928, 15236, 20960,
                3144,  12672, 25008, 5516,  10272, 60728, 16060, 48220, 41012,
                6468,  13856, 33136, 38984, 19896, 41396, 30436, 60328, 21612,
                43564, 62840, 18252, 35472, 56316, 31916, 556,   62908, 24556,
                35320, 59492,
            },
            {
                29408, 45568, 64748, 26736, 4480,  23408, 39692, 30528, 25444,
                18748, 47108, 42820, 58268, 27280, 45604, 14124, 7484,  64900,
                3716,  19532, 53664, 27744, 42936, 8152,  31524, 10868, 54012,
                1856,  30476, 40372, 63908, 19680, 136,   52196, 21536, 15740,
                59548, 3660,  44700, 8776,  26744, 20720, 11484, 29820, 25068,
                46220, 13792, 6700,  43136, 2720,  25792, 12640, 47148, 5808,
                51936, 25048, 57628, 42736, 60804, 36132, 13588, 56980, 3084,
                50776, 21020, 14628, 44528, 37736, 10004, 20684, 3544,  45192,
                38816, 5344,  41692, 11772, 30280, 56448, 3000,  24732, 39808,
                55988, 21440, 3640,  43816, 11052, 60844, 32840, 38928, 13012,
                24024, 7000,  16180, 26152, 43624, 18924, 39360, 59996, 7620,
                41584, 58504, 34396, 44088, 55628, 38656, 4620,  24240, 9208,
                36844, 61672, 22688, 4764,  59404, 52000, 17220, 47620, 6904,
                33180, 14220, 41920, 26868, 22456, 48624, 43892, 18904, 41532,
                3396,  49432,
            },
            {
                23324, 10740, 18072, 41684, 54900, 9656,  16852, 55828, 6972,
                59876, 12936, 32224, 2024,  18004, 62056, 39644, 29192, 24996,
                48812, 30396, 60252, 46264, 13912, 57656, 47996, 23988, 63280,
                19164, 58584, 46244, 26088, 11840, 28764, 56660, 8964,  40996,
                30668, 35380, 22656, 57620, 47684, 39172, 59808, 54252, 64132,
                39740, 22360, 48468, 16944, 52608, 33788, 42060, 62276, 1240,
                38856, 64492, 11300, 21360, 4092,  49012, 24352, 9460,  45536,
                27960, 7480,  48204, 31808, 17720, 64080, 35020, 13156, 53760,
                27932, 58108, 35404, 51252, 21788, 43796, 16840, 52428, 8816,
                33472, 13300, 64268, 30192, 18384, 25980, 52952, 21172, 30484,
                44752, 50024, 41316, 4624,  63400, 9348,  50748, 28708, 23932,
                54368, 47796, 16788, 63092, 21556, 49532, 32672, 64240, 56696,
                28144, 50244, 46696, 25524, 8892,  35028, 1296,  56020, 23944,
                52828, 3944,  50228, 11288, 59600, 7212,  28804, 12288, 55600,
                16172, 6504,
            },
            {
                51656, 58372, 36580, 30816, 48952, 61156, 46200, 43116, 34340,
                51920, 38804, 9308,  53508, 5604,  34984, 10684, 59684, 44216,
                16928, 9072,  33924, 22804, 264,   20504, 51000, 28240, 41844,
                13272, 34368, 4760,  51052, 15296, 44636, 32480, 47448, 4916,
                61704, 50640, 13908, 6196,  31884, 15480, 4488,  43688, 9016,
                200,   59076, 27732, 36788, 56424, 8088,  15132, 28756, 35736,
                17512, 32412, 7292,  45352, 53168, 29400, 38052, 19184, 54100,
                34104, 60380, 24712, 53040, 6136,  41388, 56996, 49144, 32552,
                8124,  19252, 15256, 1448,  33080, 63052, 5204,  29036, 45772,
                60364, 736,   50564, 38544, 56124, 6100,  42436, 2464,  9120,
                61840, 19604, 36552, 54748, 21816, 46400, 33656, 3728,  65444,
                31228, 9496,  676,   26184, 29588, 13068, 2124,  20072, 44932,
                16764, 2556,  54972, 12644, 62888, 44896, 26812, 11584, 61444,
                36780, 29608, 63880, 38292, 2936,  53856, 34456, 65084, 47084,
                37896, 61856,
            },
            {
                772,   42868, 13352, 5776,  20940, 1632,  27156, 14564, 3352,
                21792, 28168, 62816, 24776, 41292, 50880, 20120, 2248,  36588,
                56976, 4768,  51760, 39184, 62392, 35840, 16608, 6428,  37908,
                52888, 8840,  23724, 60596, 38440, 3412,  64904, 24620, 17920,
                11312, 27776, 54016, 42160, 62520, 49668, 23556, 34804, 19796,
                32148, 50696, 11068, 63472, 23676, 46380, 20900, 50972, 57328,
                23076, 49744, 55792, 26908, 15932, 61924, 904,   42636, 63148,
                16272, 40024, 2348,  58280, 28940, 22132, 368,   25036, 63312,
                42732, 47544, 61340, 54332, 9500,  48460, 38360, 57680, 20088,
                41772, 15936, 23584, 45208, 9800,  34832, 63204, 47204, 56680,
                33364, 12156, 2924,  28076, 14384, 57704, 37772, 17344, 44404,
                13440, 40212, 36616, 52348, 7340,  59832, 42340, 53372, 35444,
                10516, 31872, 41956, 37688, 15680, 50824, 32304, 39996, 18652,
                44336, 8072,  20340, 46572, 16916, 40728, 21076, 24932, 8208,
                31272, 19992,
            },
            {
                33908, 56488, 25256, 63212, 52424, 33272, 37892, 57788, 65408,
                49244, 16244, 45108, 56264, 30216, 13744, 47428, 54808, 22244,
                40856, 63956, 15388, 26612, 10976, 43936, 59124, 3248,  61560,
                21544, 48532, 30064, 42428, 55088, 20732, 36164, 49572, 57276,
                39648, 46000, 20124, 944,   36828, 28608, 12532, 57256, 52668,
                15896, 41508, 5780,  30552, 2140,  39944, 60792, 4312,  9912,
                41608, 13516, 2456,  40232, 34672, 10800, 58112, 32240, 5068,
                22344, 11080, 36124, 43496, 13776, 46688, 38032, 16716, 10780,
                3832,  23628, 29680, 39844, 26352, 18540, 12412, 34972, 6848,
                26984, 54636, 32088, 4960,  59508, 28164, 14240, 17572, 24992,
                40032, 51260, 60096, 47940, 32072, 6400,  11016, 52960, 22388,
                49988, 62152, 19116, 57868, 46284, 38252, 15328, 27228, 5572,
                59460, 21900, 64704, 29088, 6584,  20280, 58916, 3332,  49648,
                14956, 56556, 24704, 31784, 57672, 13804, 51932, 2096,  60372,
                44408, 27048,
            },
            {
                48888, 8820,  17196, 45384, 40260, 11872, 23524, 9008,  30108,
                5280,  36968, 11684, 1464,  22532, 64484, 32992, 6496,  27480,
                12468, 31740, 46772, 55648, 7276,  29768, 54180, 32792, 45144,
                15404, 57188, 1224,  17616, 11076, 6888,  28248, 13796, 2816,
                33188, 7504,  64420, 25576, 17304, 59432, 2900,  46104, 38076,
                25696, 60984, 44676, 19372, 53604, 11980, 32104, 26412, 45588,
                62748, 29516, 59636, 20160, 47456, 23236, 50164, 14380, 27416,
                51720, 47560, 64964, 18760, 9132,  61396, 55636, 31224, 51696,
                60080, 36292, 13524, 45880, 4468,  59408, 23908, 49716, 65452,
                10488, 36592, 61372, 19664, 48348, 22436, 37520, 54040, 752,
                29312, 7688,  23060, 42996, 64084, 25504, 40836, 60968, 2320,
                27572, 5152,  33336, 23040, 10324, 31692, 63604, 23528, 49096,
                39300, 52516, 17868, 688,   56772, 42728, 23448, 10108, 65448,
                27784, 216,   35732, 61260, 4600,  43176, 30156, 36832, 11200,
                54852, 14700,
            },
            {
                64404, 38760, 28680, 2624,  60032, 19144, 53772, 47628, 41172,
                55348, 20216, 33780, 59096, 8120,  42668, 16660, 58496, 50512,
                44644, 19160, 2404,  23836, 49220, 40712, 18736, 24888, 10456,
                27412, 36484, 63780, 33308, 46596, 62204, 51496, 43428, 61276,
                23572, 37764, 55712, 10248, 51068, 40936, 7840,  30124, 64364,
                3976,  13624, 34344, 48672, 59444, 16332, 37580, 54568, 18500,
                7612,  36992, 44284, 5392,  64832, 8224,  39472, 55956, 43792,
                7820,  30612, 54480, 25908, 33100, 50388, 6740,  27140, 40884,
                19692, 56668, 7452,  63936, 32704, 41880, 54824, 30624, 2876,
                46432, 52656, 12976, 1636,  40548, 52076, 11256, 44272, 64880,
                58352, 14972, 37956, 17764, 1184,  55040, 19460, 30328, 36036,
                55944, 15832, 48544, 43536, 3164,  18268, 54512, 1376,  12352,
                45700, 7416,  25360, 48072, 36184, 53320, 30168, 46004, 33448,
                38680, 54324, 12104, 50428, 9012,  26748, 47936, 58228, 18120,
                40452, 3512,
            },
            {
                20704, 53648, 35256, 48380, 7140,  31704, 62084, 540,   17284,
                25284, 61184, 39492, 47116, 52496, 26408, 38540, 840,   35032,
                9252,  59812, 37512, 63016, 34336, 12856, 1416,  65016, 47748,
                41480, 7492,  52044, 25492, 40412, 22080, 31576, 9360,  18976,
                53084, 15688, 29952, 44096, 33016, 22568, 54384, 18848, 10068,
                22044, 56360, 28540, 8516,  24520, 43024, 65064, 472,   48064,
                33744, 24468, 15340, 53468, 31604, 17660, 25428, 2120,  34524,
                59380, 16744, 968,   41740, 4152,  20884, 44840, 12420, 34500,
                1800,  48808, 22476, 16996, 50888, 140,   15076, 21552, 39048,
                17692, 24900, 42948, 29324, 57660, 7916,  31504, 4872,  35104,
                20580, 46360, 52520, 10280, 34684, 48896, 13640, 45692, 8192,
                42028, 11388, 60104, 25788, 57172, 50476, 36160, 29716, 60760,
                33920, 14784, 31424, 61792, 9380,  14008, 4736,  60100, 16552,
                7168,  22124, 45148, 39848, 18872, 64156, 22604, 6828,  33228,
                50816, 24304,
            },
            {
                5464,  12592, 57288, 22968, 15316, 43336, 26992, 35844, 44848,
                10028, 50784, 14768, 3256,  31528, 18964, 11316, 61572, 24256,
                54048, 29544, 5012,  16380, 57012, 21780, 51204, 37668, 4448,
                54632, 20364, 13256, 3792,  59312, 15164, 452,   58196, 40888,
                48068, 2240,  60056, 5252,  13184, 61632, 48328, 34920, 43440,
                49892, 39424, 1384,  52324, 35656, 5500,  29692, 10720, 22116,
                58472, 51668, 2028,  27968, 57344, 36200, 45900, 63308, 20108,
                11400, 38668, 23288, 57604, 35568, 62648, 16040, 59028, 24204,
                53992, 10260, 44320, 26872, 36648, 11168, 62996, 43584, 57028,
                4660,  62296, 33708, 15760, 63980, 24268, 18472, 49852, 26568,
                41408, 3436,  31076, 62600, 27512, 4676,  59044, 52060, 23992,
                63304, 29520, 38420, 6692,  14512, 40348, 8364,  44128, 19752,
                56080, 42096, 58564, 21448, 39428, 27116, 50192, 19468, 41328,
                48304, 62440, 29288, 15028, 33868, 53172, 1096,  13620, 61708,
                28960, 44956,
            },
            {
                60192, 31184, 42092, 9828,  58784, 51700, 4356,  13312, 63632,
                32464, 6600,  28688, 23492, 63160, 55384, 43520, 48712, 14620,
                40104, 20868, 51544, 42296, 8772,  44964, 28868, 58492, 15004,
                31852, 60856, 29772, 49404, 37904, 55248, 35080, 24580, 28488,
                11968, 35540, 21104, 46720, 38836, 27184, 564,   15232, 58272,
                31276, 63040, 17440, 46168, 21308, 14404, 57264, 50836, 40540,
                12944, 63636, 38556, 9732,  48860, 12488, 4256,  52848, 29216,
                49328, 61740, 46808, 13600, 52596, 27716, 48444, 5876,  43180,
                64692, 31088, 39208, 58680, 53592, 29284, 7356,  34496, 48948,
                28348, 9536,  50196, 6012,  45924, 39248, 56132, 60064, 13700,
                9296,  54440, 24300, 56984, 44148, 38604, 21272, 32656, 1492,
                17800, 53996, 20680, 34348, 64892, 27984, 22048, 62348, 3736,
                26272, 5396,  11012, 46672, 2256,  63836, 34364, 57036, 10972,
                25560, 1868,  56416, 5032,  59176, 37220, 46644, 41764, 55948,
                9056,  37108,
            },
            {
                332,   18272, 50380, 25520, 33896, 20044, 39244, 56496, 22192,
                53884, 41968, 58380, 45176, 9748,  4420,  33376, 28228, 6956,
                65300, 46932, 12124, 26756, 32120, 61896, 6604,  17948, 35168,
                43768, 23056, 9160,  44944, 18676, 5216,  46164, 65124, 7364,
                56804, 51376, 25960, 63696, 17960, 55964, 6080,  37836, 24356,
                6736,  12388, 26288, 55588, 61652, 38056, 45012, 25840, 32384,
                4176,  18000, 43492, 20936, 61480, 40832, 22616, 14816, 42532,
                5108,  26424, 7172,  31336, 10152, 39696, 1348,  37312, 28980,
                18532, 14120, 2520,  5524,  19580, 45264, 60432, 25300, 13188,
                19132, 38128, 54556, 21160, 35788, 12256, 3720,  32944, 47316,
                61904, 37032, 19080, 12344, 6908,  16344, 65072, 10496, 36724,
                43728, 4308,  49828, 46216, 64,    52860, 11164, 47688, 15764,
                38664, 52176, 35488, 17400, 54500, 42572, 22840, 4012,  36760,
                52856, 31848, 20508, 43212, 24100, 10428, 30708, 25832, 20140,
                15796, 47848,
            },
            {
                27364, 61888, 38644, 6492,  64888, 46528, 8552,  29628, 48612,
                18232, 1608,  35132, 15852, 50132, 37488, 21940, 57232, 18188,
                3516,  34180, 58640, 644,   53364, 39000, 24288, 49824, 10776,
                56536, 3124,  62808, 33808, 27044, 11220, 50868, 16284, 43224,
                32448, 39676, 4164,  8936,  31572, 51888, 44692, 65440, 20324,
                53248, 42616, 4580,  32488, 9948,  3368,  18836, 62812, 8048,
                46884, 30076, 56104, 34196, 6960,  26808, 59708, 32532, 56260,
                37096, 17116, 54104, 44480, 19316, 63720, 22784, 55960, 8888,
                51512, 35480, 62244, 47844, 23488, 16036, 40040, 52512, 1956,
                58300, 31268, 60684, 1136,  27188, 53316, 42780, 22556, 17068,
                29404, 144,   50904, 33752, 48504, 40944, 25140, 47060, 57080,
                26852, 61664, 12436, 31400, 17248, 41952, 58420, 33072, 55304,
                29180, 49200, 62560, 24788, 31000, 7872,  15432, 29848, 44744,
                64524, 12952, 50540, 7804,  62072, 17508, 51444, 3608,  65260,
                34376, 53976,
            },
            {
                21996, 44268, 14176, 2296,  28272, 16672, 60400, 3480,  36800,
                11916, 26600, 64816, 20148, 40672, 61012, 13244, 52616, 45308,
                30516, 38316, 22956, 48336, 19176, 13972, 2616,  64332, 27328,
                39892, 51308, 14748, 53936, 41804, 59648, 30628, 22772, 1656,
                19556, 13836, 58212, 42364, 22876, 11776, 33604, 28816, 9472,
                47976, 35980, 60624, 39924, 49104, 28920, 54268, 35072, 15636,
                59148, 50188, 24824, 1032,  16532, 45056, 51256, 3052,  10364,
                64760, 21760, 35004, 59908, 4288,  50688, 33128, 16860, 46608,
                60696, 25136, 41768, 33244, 55832, 9108,  31936, 5260,  46840,
                41472, 23776, 43800, 14536, 48156, 30412, 8988,  63120, 38388,
                6088,  42236, 58712, 22452, 61504, 3064,  30732, 51576, 6568,
                15568, 39788, 23112, 56676, 7912,  25468, 37332, 2408,  23176,
                9176,  19232, 576,   13028, 45324, 57820, 51564, 60572, 8724,
                17940, 40588, 33520, 47192, 27840, 38752, 57840, 44732, 7348,
                40600, 11008,
            },
            {
                3980,  32272, 56288, 52456, 35572, 43132, 53452, 24676, 41372,
                57292, 46240, 51748, 7204,  30040, 248,   26008, 9080,  41376,
                62452, 15336, 10332, 61140, 35496, 55604, 42048, 46396, 20648,
                7536,  36812, 24660, 2036,  21088, 7944,  38580, 62932, 47492,
                53544, 61876, 36384, 48756, 16388, 60904, 2668,  40304, 59748,
                14812, 27884, 18592, 22636, 13160, 43372, 24028, 2020,  42080,
                21796, 37720, 10352, 53624, 64296, 36604, 29668, 19536, 39560,
                46392, 1848,  28952, 41568, 14672, 25480, 58064, 11572, 40524,
                3464,  21164, 7936,  12760, 49872, 64972, 37092, 21484, 61808,
                16528, 6660,  10024, 64840, 18104, 58892, 2680,  49660, 55136,
                25940, 46092, 15136, 8092,  27692, 55896, 13000, 19808, 59704,
                33388, 9404,  52220, 35760, 63176, 19596, 45664, 14152, 64260,
                40376, 43784, 59164, 33776, 37824, 20196, 2940,  38808, 27672,
                55804, 23548, 4424,  15456, 54168, 1944,  12604, 31860, 23152,
                49912, 58880,
            },
            {
                17620, 63440, 24172, 47804, 21012, 10008, 31820, 13920, 62356,
                5812,  30960, 10480, 24068, 55216, 48240, 36104, 58540, 2604,
                24228, 55008, 7252,  44032, 25820, 4504,  29792, 33192, 60256,
                16872, 48112, 30936, 64172, 45908, 55368, 13400, 35356, 28012,
                10220, 24440, 29740, 1044,  54860, 26172, 50000, 18256, 45736,
                3528,  54892, 50276, 208,   56756, 64548, 9028,  52360, 60784,
                5628,  13768, 28860, 40456, 47312, 11976, 7748,  58464, 25500,
                50224, 12992, 55652, 8924,  47824, 36948, 44064, 6124,  31812,
                54936, 28396, 57928, 43808, 628,   26740, 14464, 53632, 29204,
                35276, 56500, 51684, 36860, 40280, 24696, 34312, 14112, 19920,
                10572, 64612, 31956, 39056, 53328, 35112, 45044, 932,   42584,
                28312, 48252, 3160,  43968, 29780, 5592,  50232, 59984, 30660,
                6132,  52740, 27628, 4868,  65188, 47360, 25160, 12244, 48980,
                892,   35900, 59032, 63592, 42196, 21016, 35408, 61332, 15060,
                28844, 37280,
            },
            {
                8608,  41424, 12132, 6664,  39548, 57828, 624,   49976, 21860,
                17316, 39784, 34268, 61860, 14072, 42628, 17480, 31908, 50332,
                19424, 28872, 51828, 40284, 16572, 57804, 8520,  12728, 51896,
                316,   58256, 11728, 40564, 17548, 26136, 5684,  57708, 3572,
                43976, 17748, 6340,  45404, 38996, 8424,  35248, 56944, 23896,
                31992, 8524,  62424, 33768, 16524, 30244, 46540, 38772, 27128,
                33132, 57048, 62744, 4664,  18376, 23752, 54164, 43576, 15880,
                62384, 33500, 23632, 61492, 20348, 212,   27764, 65152, 19668,
                50240, 15128, 38212, 30364, 18332, 59352, 40700, 10852, 2728,
                49204, 26584, 31712, 21372, 5292,  54424, 44240, 28308, 58088,
                36136, 52156, 3868,  20776, 10960, 17412, 64152, 24036, 37452,
                62172, 21052, 16784, 58632, 13216, 54776, 26904, 10596, 35268,
                47600, 15572, 22644, 54412, 9652,  16884, 33036, 42036, 62992,
                20812, 45616, 11336, 25456, 29968, 9092,  47752, 56044, 5492,
                53540, 46908,
            },
            {
                34280, 50600, 29888, 60920, 16132, 26788, 64388, 44140, 36004,
                54504, 1972,  58588, 44300, 4568,  22060, 64088, 11332, 46692,
                35220, 5300,  64764, 32748, 22012, 49172, 62732, 38244, 23864,
                42776, 27592, 35624, 4184,  52244, 32796, 49040, 20416, 40252,
                60216, 50076, 34152, 62656, 14452, 20412, 63984, 5368,  13056,
                37928, 44056, 25232, 40796, 6312,  36452, 20400, 11752, 17796,
                48652, 44288, 20724, 51524, 32048, 61116, 35152, 604,   30600,
                6020,  38096, 3812,  51348, 30860, 58764, 53536, 13232, 35560,
                45572, 2100,  62444, 52968, 5944,  47436, 23940, 44100, 63512,
                19036, 13096, 46140, 388,   62188, 12056, 47536, 6976,  41200,
                1408,  23740, 44052, 48232, 58928, 40368, 7024,  50772, 14040,
                4584,  53064, 40568, 24728, 1244,  38208, 42644, 21952, 18068,
                57536, 38732, 11816, 42796, 29464, 50360, 57052, 6216,  28924,
                53948, 17160, 50796, 5792,  37480, 52116, 18284, 24632, 43148,
                2152,  20800,
            },
            {
                59304, 25856, 1388,  53312, 45620, 33844, 19572, 8300,  28768,
                47632, 12492, 27028, 18960, 50604, 38148, 27844, 7160,  60360,
                39488, 13408, 45804, 984,   11136, 28320, 44720, 18880, 55320,
                6764,  62240, 19960, 57032, 8628,  42940, 61456, 15364, 30876,
                53048, 11904, 21828, 56212, 31220, 46936, 27808, 42304, 51468,
                58876, 20808, 11304, 47384, 59884, 50692, 4028,  62008, 55536,
                720,   9232,  36308, 26468, 2832,  14248, 40776, 49124, 21276,
                54736, 45408, 17952, 40952, 10184, 16160, 38884, 23704, 8412,
                59728, 25752, 10736, 21396, 35876, 32420, 55656, 4824,  38028,
                58044, 8336,  42348, 59732, 16900, 32896, 22428, 63588, 17668,
                30604, 60576, 16056, 26260, 4928,  28904, 32708, 46844, 57548,
                31092, 10836, 34184, 65308, 49100, 32156, 62504, 5000,  51180,
                2472,  25912, 62924, 34980, 1356,  21604, 61824, 14836, 34596,
                9024,  39820, 32060, 43304, 57016, 76,    62564, 39896, 27476,
                64564, 13816,
            },
            {
                54688, 43636, 18636, 36912, 5484,  12816, 55744, 39020, 3712,
                59664, 23020, 52744, 8488,  30844, 57336, 1396,  53704, 16512,
                26240, 55940, 20740, 59160, 37648, 53688, 3076,  34912, 14944,
                31468, 46252, 49940, 13924, 29128, 22904, 36680, 2224,  25276,
                8128,  37572, 26920, 3376,  41124, 10908, 53680, 832,   17292,
                29884, 64976, 2608,  53640, 14364, 28140, 23224, 40436, 31652,
                25264, 41736, 15900, 59780, 45152, 56292, 9532,  65268, 26128,
                11480, 58960, 28208, 64120, 34736, 46180, 3116,  49252, 42120,
                32920, 17436, 40472, 48804, 64184, 8464,  16352, 29736, 21948,
                33860, 53052, 25020, 38640, 29008, 50592, 9520,  52928, 38064,
                55504, 12408, 50156, 35616, 62720, 54540, 21644, 8920,  18088,
                26672, 55308, 45448, 7196,  19308, 15348, 9128,  53908, 44712,
                33020, 59516, 7788,  46116, 55532, 40404, 4052,  44736, 23620,
                58264, 2976,  65024, 13260, 22760, 16200, 33836, 11636, 8040,
                32392, 38188,
            },
            {
                6008,  10668, 23096, 63928, 31172, 47924, 24692, 61772, 15656,
                32900, 41552, 65500, 35724, 15312, 47480, 41868, 33312, 23216,
                43488, 4404,  30740, 50460, 15736, 24616, 6064,  64416, 40308,
                10340, 22332, 3324,  38516, 65496, 53516, 11152, 46620, 56464,
                43524, 64308, 16700, 50940, 7052,  61128, 23124, 33360, 48340,
                6616,  35964, 26784, 18732, 33004, 57076, 44432, 7856,  51864,
                13228, 64516, 49348, 7136,  33560, 28284, 19024, 37216, 6744,
                42520, 14840, 49704, 6532,  25164, 54408, 20664, 63132, 5340,
                52628, 56584, 4328,  28064, 42696, 13528, 51824, 45456, 61568,
                2108,  14916, 48312, 6612,  56672, 35680, 1924,  43328, 26652,
                5884,  33452, 9040,  41756, 632,   12908, 43908, 36516, 63820,
                41876, 1688,  22448, 36792, 28484, 59892, 40320, 23416, 29352,
                13472, 20288, 49312, 17168, 31364, 12552, 27024, 52332, 38272,
                48684, 19864, 26444, 46576, 60632, 29184, 44552, 49972, 57568,
                46364, 17092,
            },
            {
                61668, 48620, 40416, 56564, 2808,  51964, 41912, 10392, 50136,
                20564, 5564,  45704, 2684,  24452, 62136, 12996, 8660,  51144,
                63628, 36400, 9556,  61616, 42108, 33480, 57960, 48528, 26792,
                52672, 60720, 33900, 25548, 44920, 18160, 5512,  62368, 19720,
                33056, 124,   47576, 58188, 34860, 18708, 38596, 57776, 14844,
                40492, 56216, 45728, 8824,  38036, 1304,  63480, 17016, 34800,
                57740, 21716, 38016, 54516, 11524, 23132, 51680, 60472, 47416,
                31476, 35688, 22516, 1216,  57300, 32288, 10916, 27020, 29932,
                14316, 37328, 23004, 60060, 972,   20328, 57624, 25460, 10576,
                41116, 30752, 65032, 20168, 12680, 24112, 61600, 14196, 20524,
                47140, 65412, 22468, 56744, 18988, 51816, 25496, 59172, 6464,
                49712, 14292, 61324, 51396, 11796, 46732, 56160, 256,   37448,
                64324, 41652, 4220,  24416, 37148, 58968, 19368, 7300,  61200,
                11768, 31012, 54220, 7500,  36544, 4360,  55180, 21084, 3272,
                23840, 30240,
            },
            {
                1780,  33700, 13688, 28028, 21240, 17388, 7028,  29812, 36976,
                56348, 27752, 11732, 39500, 54784, 18380, 29292, 57900, 19984,
                2824,  48180, 27368, 18648, 7116,  45504, 12428, 19872, 1584,
                43244, 16740, 8144,  58804, 940,   40644, 34956, 50636, 28520,
                12904, 39672, 23480, 29644, 13404, 45312, 26256, 9000,  63256,
                21904, 12392, 52528, 60612, 42200, 21184, 10956, 47828, 28564,
                3176,  6220,  30360, 17564, 46708, 42168, 5136,  1500,  16596,
                56392, 8576,  61812, 43604, 13560, 39932, 59288, 44364, 18796,
                61300, 47200, 11952, 30768, 49760, 34140, 38700, 4440,  47724,
                17536, 36512, 54520, 3180,  45500, 39164, 49016, 30232, 58792,
                2532,  39768, 28356, 45848, 31240, 38012, 16196, 3004,  33832,
                20076, 39436, 30560, 57808, 3404,  25812, 33408, 17232, 50660,
                10076, 27324, 56932, 53100, 9436,  64768, 47180, 29584, 16216,
                35504, 1088,  41112, 17532, 51608, 10216, 39120, 15016, 59928,
                41620, 51356,
            },
            {
                26536, 57976, 8052,  45088, 62372, 35048, 59256, 46464, 28,
                63328, 17172, 52208, 60648, 33672, 5856,  37576, 45860, 40380,
                12072, 53520, 14600, 39664, 56388, 30176, 22824, 61336, 36420,
                29004, 56276, 47788, 12280, 31868, 55340, 15188, 24648, 6808,
                59472, 54348, 9916,  42064, 62828, 1796,  54856, 31788, 44352,
                3928,  29364, 24156, 4992,  30820, 50104, 25732, 55000, 39456,
                45888, 61528, 24892, 2040,  63896, 57688, 34580, 29484, 39876,
                24032, 51100, 19568, 37016, 52444, 17132, 7228,  35428, 50860,
                2992,  8696,  43632, 54720, 16568, 7400,  63076, 27532, 52492,
                60616, 23184, 9492,  28060, 58284, 17140, 4352,  53356, 34824,
                17996, 51400, 14984, 3444,  58316, 10172, 62508, 47832, 53748,
                27736, 9372,  44132, 16704, 5952,  42376, 21028, 61416, 6852,
                46328, 35064, 15252, 43508, 852,   32852, 41460, 3288,  56168,
                43860, 63336, 21548, 59092, 32316, 25040, 63900, 28112, 35188,
                12376, 19452,
            },
            {
                43948, 36676, 16264, 54064, 38820, 11528, 25864, 53696, 13628,
                23608, 43436, 31180, 9428,  20948, 50272, 724,   63080, 24948,
                31500, 34724, 21620, 65236, 24,    49888, 9172,  53920, 13992,
                4948,  39452, 19584, 51412, 28048, 21976, 60276, 42256, 48428,
                36520, 17288, 4512,  52312, 20356, 48656, 6388,  17540, 51804,
                35264, 49080, 16636, 62400, 13240, 58172, 36888, 7064,  14740,
                19816, 42800, 52500, 32248, 39004, 14788, 20840, 53308, 63368,
                12228, 45600, 4224,  29328, 25676, 47784, 2080,  63784, 24408,
                40784, 33156, 25536, 65384, 36708, 22404, 44580, 13280, 32664,
                640,   43232, 50736, 34068, 41780, 63824, 26108, 8548,  42280,
                11188, 62052, 24584, 7852,  49880, 41432, 21932, 30088, 12648,
                56968, 23916, 63560, 35316, 54136, 49072, 38364, 12896, 57980,
                22888, 30900, 62468, 19780, 50400, 13192, 22460, 51792, 25080,
                9928,  49672, 27996, 12596, 47912, 42248, 2092,  46744, 6912,
                53160, 65372,
            },
            {
                4240,  23972, 50268, 1020,  31368, 5076,  42500, 19004, 33428,
                38432, 6560,  48996, 25920, 41516, 59116, 27928, 16328, 7544,
                55372, 60332, 46588, 5940,  36972, 25292, 33116, 41060, 47164,
                64168, 24044, 35304, 61956, 4200,  46016, 9444,  2576,  30908,
                21200, 64724, 32892, 25512, 37136, 58980, 28424, 12028, 41908,
                59580, 1112,  39524, 44908, 34492, 18612, 1776,  65528, 32744,
                59972, 8844,  12980, 49184, 6708,  26300, 10416, 47696, 5872,
                27536, 33544, 58884, 65000, 9264,  55884, 31456, 21560, 54280,
                15228, 57904, 19148, 1668,  50916, 10884, 40012, 55924, 19616,
                7940,  58948, 14280, 21328, 5272,  11800, 32676, 22928, 46280,
                54804, 37768, 31920, 60024, 34880, 26308, 4088,  44772, 36312,
                444,   47100, 6636,  19384, 31644, 62536, 1892,  29564, 52564,
                40704, 2584,  6536,  38776, 25984, 58028, 36436, 17468, 60452,
                33280, 6712,  37652, 4692,  56048, 15820, 20264, 54752, 30492,
                37996, 9972,
            },
            {
                32536, 62664, 20208, 58480, 47352, 22284, 64812, 49560, 9140,
                61028, 57836, 1980,  15464, 46484, 12504, 35052, 51940, 44588,
                10620, 2212,  28572, 42556, 52544, 15544, 57352, 3788,  17900,
                30656, 10748, 7016,  43532, 14164, 37780, 63060, 53036, 11844,
                56024, 46440, 43776, 8664,  14588, 40676, 33492, 65116, 24908,
                19704, 10312, 55524, 26656, 8032,  53900, 41788, 23812, 48740,
                27232, 35976, 56184, 22040, 59632, 44604, 36988, 61044, 18012,
                42692, 252,   15112, 39432, 18768, 42068, 12988, 45452, 9716,
                38328, 5916,  48272, 27992, 59008, 5268,  29748, 62160, 24768,
                46952, 38864, 30016, 55720, 48400, 37432, 56812, 60396, 116,
                29016, 5448,  20888, 45280, 12200, 55456, 18856, 64732, 15508,
                59512, 40696, 51320, 26972, 14212, 9156,  23668, 44028, 17928,
                10768, 54888, 48584, 59944, 29416, 7692,  46504, 4564,  39656,
                14480, 53824, 44484, 23856, 62216, 34304, 9312,  61344, 23248,
                17296, 48856,
            },
            {
                14260, 28944, 39800, 8360,  27180, 14888, 55016, 2540,  29172,
                45260, 21840, 36000, 54416, 64968, 3432,  22868, 57324, 19472,
                40976, 24252, 17632, 58552, 11472, 20512, 63324, 26980, 44872,
                59396, 49256, 53528, 26356, 57932, 33296, 17804, 25648, 40296,
                15600, 1268,  27916, 61588, 3856,  56572, 22072, 2364,  53436,
                37244, 31048, 63828, 22016, 47408, 30004, 11580, 51528, 4192,
                17040, 40464, 448,   29068, 18408, 54656, 2568,  30728, 50608,
                57500, 21908, 53960, 48420, 5188,  36012, 60560, 27740, 49984,
                62548, 30440, 42536, 35512, 14684, 45988, 53584, 2596,  16000,
                35588, 64696, 25872, 1936,  18656, 44488, 15148, 50408, 19736,
                13840, 64512, 40812, 16848, 51312, 5772,  38660, 49380, 8444,
                32580, 21236, 11428, 37404, 56456, 47524, 59584, 35924, 27120,
                65248, 32712, 21496, 16312, 43904, 10964, 62868, 55580, 27236,
                20932, 65160, 30324, 18672, 572,   39288, 49600, 44384, 5060,
                41168, 57164,
            },
            {
                45920, 53380, 11724, 43408, 60392, 36460, 32180, 40256, 16844,
                52612, 12204, 28340, 19104, 32200, 39724, 6128,  29912, 37384,
                64164, 50364, 33948, 7188,  30756, 48752, 39032, 8376,  35036,
                2076,  22180, 16208, 41184, 356,   20992, 50424, 29788, 5712,
                35604, 58320, 22440, 50376, 18900, 47808, 9876,  43108, 46492,
                7076,  13884, 50176, 5400,  15632, 37612, 62772, 20892, 44596,
                58660, 11032, 47076, 63676, 34232, 41192, 8512,  24232, 12940,
                34652, 37976, 9864,  30400, 24284, 57220, 1000,  33876, 17204,
                3852,  21132, 11784, 61252, 23600, 18600, 33348, 41984, 50372,
                11644, 5640,  53252, 10360, 61536, 30776, 7360,  27100, 39980,
                35192, 47396, 57156, 1512,  27596, 61284, 31112, 23340, 43112,
                27952, 55036, 3060,  64348, 42032, 4260,  15620, 51096, 7240,
                45800, 13880, 37348, 3192,  54196, 34352, 19128, 31232, 42088,
                2656,  46968, 11272, 51360, 58744, 25764, 31708, 12852, 27540,
                60320, 1332,
            },
            {
                24380, 34284, 3072,  18048, 51292, 6044,  10888, 62876, 25016,
                4540,  41860, 62292, 7448,  49460, 43724, 61048, 14656, 47756,
                4044,  13328, 54352, 61268, 44340, 2696,  23500, 51796, 13444,
                56616, 38204, 31816, 65108, 46956, 8460,  56824, 43280, 63968,
                47568, 10628, 31680, 38000, 54216, 29320, 35128, 15816, 61204,
                27760, 57496, 34092, 43248, 60676, 3152,  55736, 28016, 7664,
                31960, 53456, 24504, 4932,  13608, 51628, 16540, 64528, 45256,
                6620,  26608, 63100, 51708, 43652, 20596, 8000,  52844, 25728,
                44112, 57000, 51556, 7180,  55284, 38216, 9584,  27092, 60664,
                20408, 45412, 40732, 34476, 51084, 22416, 42884, 62640, 3508,
                52588, 10128, 22328, 33528, 42452, 14668, 10552, 56652, 1836,
                61864, 16668, 46008, 24400, 33576, 30068, 21600, 39616, 700,
                57268, 25600, 61188, 40896, 23716, 51132, 280,   12928, 49540,
                59356, 8148,  35440, 40780, 14764, 7604,  55444, 21628, 52032,
                35492, 19068,
            },
            {
                7628,  55980, 64444, 30024, 22576, 56864, 44948, 20008, 34520,
                47376, 57404, 37988, 24428, 10704, 17100, 52480, 26324, 8716,
                32352, 21576, 39860, 25804, 36344, 15052, 62104, 42044, 28696,
                18716, 60796, 4588,  11940, 28392, 36248, 13480, 2856,  23736,
                52524, 17596, 42124, 6452,  13100, 63664, 24364, 4204,  51188,
                18136, 40448, 100,   25584, 19612, 32564, 40988, 13852, 36204,
                61724, 19340, 38496, 44004, 60312, 26828, 32984, 56236, 39088,
                19016, 58768, 1920,  16580, 13484, 61728, 47652, 39928, 64740,
                14332, 36928, 32548, 26508, 888,   64452, 47932, 4156,  31640,
                57224, 23816, 16756, 28084, 58412, 2544,  36856, 11576, 56240,
                25280, 30608, 7148,  63520, 53212, 37004, 46752, 19924, 51776,
                35340, 39196, 5692,  53332, 18264, 8632,  63172, 54452, 34468,
                18620, 9196,  48048, 5092,  28388, 64440, 45372, 22552, 36892,
                25368, 17128, 56848, 28252, 45864, 62556, 37276, 3308,  64136,
                10760, 39128,
            },
            {
                48668, 41588, 13716, 46920, 38344, 26432, 464,   50340, 59496,
                8528,  15260, 1752,  55624, 29412, 59300, 392,   36116, 55812,
                62464, 45204, 1208,  10308, 19280, 56960, 5288,  32620, 55056,
                9768,  43980, 50664, 23108, 54320, 39884, 59844, 20192, 33648,
                7920,  61900, 26348, 59220, 756,   44856, 40068, 58036, 32168,
                10424, 23044, 54756, 48448, 58460, 9872,  50544, 24120, 1260,
                48184, 15192, 30028, 9412,  48700, 1756,  20944, 4196,  49612,
                11212, 46660, 41020, 33344, 28276, 37176, 11324, 30684, 22992,
                2888,  9968,  49180, 40844, 16260, 44020, 21664, 54372, 14352,
                38624, 384,   63484, 7684,  13656, 46868, 18028, 32608, 45212,
                15748, 59292, 49148, 18432, 24388, 3428,  59992, 26340, 6640,
                13284, 29684, 49804, 11868, 60960, 43780, 48492, 12636, 28560,
                42076, 52040, 31732, 12208, 57864, 15988, 38796, 9300,  61764,
                52392, 32560, 5196,  22828, 2400,  19676, 42620, 16920, 46816,
                28812, 59184,
            },
            {
                26036, 20956, 32604, 5200,  61548, 16348, 41180, 13080, 27708,
                31372, 22296, 51268, 33720, 46212, 39136, 23312, 19012, 41888,
                28184, 16516, 59796, 48028, 52776, 27688, 46080, 21296, 732,
                48168, 25408, 34564, 6760,  62236, 16792, 48728, 27560, 55488,
                39348, 14644, 45992, 34308, 18444, 52368, 7928,  20852, 37388,
                64600, 45892, 13248, 7560,  38300, 16720, 45240, 64844, 54388,
                42272, 5852,  57664, 22312, 35632, 54528, 42412, 28132, 61120,
                30912, 22744, 3904,  55188, 50172, 6072,  58900, 18476, 54464,
                47136, 60240, 19456, 62300, 30200, 10984, 59328, 34504, 6424,
                51972, 29816, 48344, 42268, 54948, 24944, 64952, 51584, 21400,
                4908,  36368, 39708, 12580, 44420, 8584,  32520, 40616, 65324,
                44656, 21896, 58656, 27016, 1264,  37104, 25108, 3776,  58444,
                22420, 63876, 36804, 20420, 43120, 6864,  32064, 55208, 4000,
                13828, 42912, 64276, 48600, 54140, 34084, 9548,  51056, 32212,
                5928,  15468,
            },
            {
                62620, 1092,  50716, 10300, 54648, 35548, 7636,  65312, 54008,
                37408, 44040, 63388, 19648, 4160,  13396, 63948, 48132, 11364,
                5248,  51448, 30304, 34536, 6460,  65028, 38788, 12764, 35984,
                63156, 14864, 41392, 19404, 31200, 1400,  10252, 43648, 4672,
                30500, 50844, 3292,  22716, 56380, 30704, 26844, 14276, 49452,
                2928,  29000, 34560, 62192, 26724, 30980, 4952,  21632, 12044,
                33756, 26004, 51384, 64284, 17568, 12656, 37620, 6436,  15376,
                52620, 36540, 65520, 19412, 24432, 45564, 2220,  34212, 41980,
                7640,  29012, 23968, 4828,  52876, 37168, 24760, 18100, 42972,
                61460, 11740, 35988, 19376, 31756, 39076, 4060,  8688,  41032,
                61796, 26500, 548,   54836, 29196, 57892, 15156, 48844, 17980,
                55168, 9044,  34380, 40864, 20204, 56828, 32176, 17064, 45004,
                6228,  15032, 1724,  53932, 26024, 50068, 60584, 18340, 28996,
                46700, 20112, 38320, 11720, 30012, 60940, 25208, 57988, 22368,
                43940, 53500,
            },
            {
                11544, 37828, 58428, 23652, 43616, 29332, 20660, 46480, 2448,
                17856, 11260, 6608,  26952, 42544, 54172, 7876,  33264, 58772,
                24600, 38384, 13152, 22664, 42460, 16956, 8744,  50840, 58508,
                30104, 3804,  53116, 58168, 45112, 51416, 24564, 34868, 60640,
                19200, 11328, 65368, 37660, 48300, 10520, 42528, 61024, 55044,
                5920,  43832, 19880, 53588, 1960,  51428, 60108, 39296, 29056,
                59088, 2632,  40316, 7572,  31508, 45488, 62404, 57332, 25224,
                43760, 7540,  13064, 57880, 9708,  39240, 63264, 26708, 15948,
                57636, 45048, 35292, 14976, 56716, 1860,  46964, 8504,  50060,
                26492, 22008, 56324, 2196,  10264, 49584, 58608, 29476, 13212,
                53816, 47852, 34208, 60708, 21472, 50856, 36636, 1528,  23832,
                31656, 4684,  62476, 47168, 14492, 50724, 7552,  60692, 38832,
                49440, 30332, 59756, 46552, 34996, 10944, 23424, 41752, 36088,
                1124,  26472, 58404, 6472,  15940, 41244, 704,   13412, 38612,
                4148,  34704,
            },
            {
                19548, 45724, 27936, 17560, 2804,  49308, 60752, 32632, 23952,
                39948, 55840, 47680, 61448, 35772, 16692, 30172, 50440, 2048,
                44164, 55032, 63296, 3668,  57760, 31996, 26296, 44244, 23048,
                18124, 38464, 9068,  27144, 13016, 37492, 63240, 15580, 46412,
                56936, 41832, 28368, 7020,  15800, 62860, 1560,  35216, 17916,
                24808, 59712, 41108, 15200, 46488, 36284, 18288, 8456,  44500,
                15856, 20032, 46892, 56164, 24688, 920,   49296, 10476, 33584,
                18076, 48044, 27332, 42252, 31516, 53244, 21892, 12332, 50936,
                36,    38824, 11404, 50204, 42492, 32916, 62632, 28824, 38924,
                15532, 5132,  44556, 59564, 27644, 16268, 22112, 35532, 43668,
                18864, 6748,  16020, 10292, 41960, 3992,  26968, 62796, 11320,
                39040, 51616, 16952, 2648,  30260, 42324, 11036, 53420, 33944,
                24640, 9076,  40960, 18168, 3384,  57280, 5860,  48632, 63600,
                54728, 9632,  50124, 34988, 45396, 52812, 28004, 65284, 49024,
                55872, 29696,
            },
            {
                40876, 7860,  63832, 34192, 56112, 14396, 10820, 53072, 6172,
                59056, 14608, 30532, 22912, 9924,  57768, 25420, 40324, 21216,
                15540, 35556, 9220,  19124, 47088, 53444, 2360,  61040, 6304,
                54848, 46500, 64716, 32960, 20716, 5496,  54116, 8256,  22508,
                204,   35880, 24884, 54020, 44416, 32864, 21836, 39640, 46960,
                11792, 32372, 8616,  22788, 63380, 11648, 25396, 56840, 48984,
                61992, 37264, 10924, 34384, 14896, 29240, 19476, 39952, 5128,
                54088, 59800, 600,   35012, 15172, 4064,  48216, 36656, 61032,
                30872, 21156, 64852, 27196, 5880,  20380, 13052, 57608, 2732,
                65456, 53280, 33468, 37636, 62416, 46300, 54732, 1252,  63300,
                31900, 23420, 45156, 64356, 32928, 19540, 56040, 46728, 43056,
                59108, 26148, 36468, 54680, 23180, 64720, 27768, 20952, 176,
                63192, 55540, 28160, 13704, 65140, 39168, 30780, 12052, 15556,
                21924, 32128, 62716, 23844, 18772, 5276,  36596, 20320, 8760,
                16504, 60592,
            },
        },
        {
            {
                19560, 49120, 8868,  15836, 31204, 58608, 8504,  15492, 62688,
                26200, 12180, 36628, 16700, 24424, 63732, 32520, 4180,  47004,
                6464,  50000, 31056, 64500, 47404, 9756,  17764, 33884, 2932,
                16420, 54680, 63284, 34360, 57032, 46696, 37536, 42528, 10916,
                20276, 5736,  26684, 37516, 18612, 32796, 55300, 16096, 21880,
                3384,  49728, 23628, 60632, 11792, 40776, 8932,  15080, 21492,
                45032, 31124, 35936, 48284, 59028, 37736, 50220, 2244,  16156,
                6380,  33728, 46624, 39836, 30428, 53124, 37888, 57208, 13896,
                5724,  60096, 23248, 29980, 45392, 61968, 200,   38648, 44048,
                11384, 61456, 26604, 14420, 20588, 51764, 2064,  11340, 33756,
                53944, 57996, 43056, 35968, 49048, 11588, 46608, 62164, 44028,
                10796, 60132, 38404, 27328, 17192, 48616, 9604,  53044, 32856,
                21848, 47152, 8884,  27504, 42932, 1944,  56380, 21176, 47100,
                59872, 2444,  56900, 4276,  28572, 20740, 48208, 59332, 10444,
                45232, 62796,
            },
            {
                13232, 42536, 27608, 61480, 5168,  24344, 40900, 33672, 1636,
                19900, 60260, 31848, 55312, 1172,  48168, 18524, 40112, 58668,
                16300, 26040, 18936, 37080, 1260,  39960, 56688, 50396, 60864,
                31268, 9960,  20508, 43992, 13136, 22584, 460,   63916, 34116,
                57336, 40140, 53860, 14924, 9260,  41828, 5012,  48424, 36656,
                8744,  14544, 32468, 45976, 17380, 51824, 59204, 36792, 5640,
                49720, 16904, 64248, 6420,  19276, 31944, 62404, 28920, 57176,
                53200, 25772, 65208, 22988, 4512,  16920, 7640,  23872, 42176,
                33964, 54872, 1684,  40200, 17076, 33464, 24600, 60052, 29480,
                18744, 47064, 1512,  42764, 58996, 34816, 61076, 22788, 48176,
                8008,  12704, 29004, 16304, 54912, 22932, 34312, 25784, 20120,
                53856, 3820,  23456, 51372, 35364, 43068, 58672, 18940, 37916,
                12292, 52024, 35480, 22948, 5636,  52588, 36036, 41212, 7096,
                27060, 12784, 37284, 44908, 14948, 8484,  25428, 2856,  15920,
                51020, 30556,
            },
            {
                2380,  56728, 32972, 46628, 37068, 51040, 13008, 57344, 44724,
                53548, 4148,  50312, 42476, 28460, 59572, 11724, 22688, 28988,
                55564, 44116, 3844,  52144, 59340, 30036, 25084, 12212, 22012,
                41500, 27276, 49260, 4248,  60208, 31488, 27528, 15008, 48308,
                24688, 2440,  61564, 30408, 51188, 57628, 23948, 60048, 28180,
                54132, 39576, 65212, 7028,  35172, 24440, 28292, 724,   42920,
                57792, 11260, 28560, 1764,  45384, 54272, 8272,  22120, 43480,
                10840, 37276, 14392, 50052, 43192, 60460, 28444, 46356, 64504,
                19812, 37504, 48700, 11404, 57472, 7104,  50760, 56200, 15756,
                4092,  36944, 32752, 55792, 6268,  16460, 43596, 27180, 64280,
                37060, 51428, 21024, 60004, 5644,  3012,  51116, 58968, 7128,
                31396, 40592, 45568, 8100,  62568, 5520,  14920, 28004, 63596,
                6536,  58280, 39632, 62356, 48204, 12096, 28984, 16636, 64464,
                54352, 31260, 21948, 62644, 51124, 33844, 61464, 43368, 37664,
                22084, 54192,
            },
            {
                5688,  38740, 18052, 11120, 712,   64476, 21476, 48572, 29796,
                9720,  39144, 22148, 14388, 7980,  37800, 51112, 2232,  35292,
                8208,  62592, 32832, 11912, 20852, 7596,  44400, 36160, 57980,
                780,   62080, 38864, 17956, 53612, 41136, 51136, 6520,  18772,
                43644, 12516, 35244, 7032,  45448, 20040, 11540, 33388, 1960,
                43448, 20252, 26864, 3160,  55392, 44160, 19320, 63860, 33276,
                23804, 38064, 51944, 41796, 25176, 12016, 40344, 17240, 47924,
                31264, 760,   58324, 8800,  55220, 35264, 12988, 51584, 10380,
                3752,  25836, 15132, 62700, 27372, 36228, 21824, 10172, 40612,
                54124, 63808, 21536, 12112, 28868, 39832, 53500, 4312,  14716,
                30676, 124,   40936, 44804, 31752, 38596, 18236, 13468, 36872,
                49524, 64188, 12064, 19168, 30292, 56904, 1932,  46112, 23916,
                42352, 1184,  14324, 18312, 32072, 60620, 24228, 45428, 524,
                39000, 50040, 17576, 6472,  41380, 56192, 12588, 31552, 6808,
                64356, 26320,
            },
            {
                48188, 59004, 23972, 52064, 42052, 27188, 34872, 4672,  17788,
                24948, 35356, 65428, 56664, 19448, 33488, 63008, 45544, 53756,
                41724, 14692, 24132, 48712, 41224, 64032, 15700, 47740, 6120,
                52604, 14456, 35280, 7316,  24872, 10448, 36028, 59384, 55532,
                29644, 49716, 22032, 59144, 26340, 37992, 64160, 47216, 16892,
                61520, 51024, 12952, 58228, 38672, 9724,  50416, 13460, 54180,
                7584,  19932, 62180, 14992, 56180, 34288, 60328, 3428,  51640,
                63356, 20076, 41392, 32764, 26272, 20880, 1504,  39504, 30060,
                58940, 45920, 31864, 53244, 42988, 2360,  46084, 31132, 48456,
                25480, 8396,  51256, 60340, 24248, 49276, 9240,  19992, 45464,
                58316, 24836, 61728, 10500, 26788, 65172, 56420, 42292, 616,
                26700, 15884, 33576, 54252, 25072, 39108, 49556, 34968, 55844,
                30632, 50688, 26692, 44348, 3388,  55368, 8656,  50896, 19788,
                33700, 9860,  59072, 29056, 23640, 1272,  19616, 52908, 46588,
                17536, 34976,
            },
            {
                14460, 62584, 7740,  20428, 55856, 15300, 8256,  58432, 43656,
                61040, 12880, 46900, 5896,  43940, 23952, 15760, 26572, 10836,
                20604, 59732, 38148, 54988, 28400, 2408,  53736, 31640, 18992,
                25872, 45416, 30236, 57532, 47432, 64332, 21180, 1596,  38924,
                9032,  65476, 16780, 41260, 900,   14432, 5372,  29656, 35524,
                10096, 23288, 46568, 29864, 16548, 62268, 31304, 3840,  40524,
                46296, 29784, 3016,  48756, 20984, 6636,  27624, 38288, 23488,
                7308,  28164, 45740, 17704, 4932,  48444, 63468, 56116, 22652,
                17312, 41020, 8900,  4548,  20136, 64676, 13524, 58548, 4936,
                34524, 14232, 38808, 45588, 1020,  35856, 62864, 32616, 54724,
                38436, 17800, 6892,  50684, 15044, 46008, 8036,  29840, 53080,
                22132, 60668, 47320, 2616,  42604, 21520, 11508, 7532,  16168,
                20396, 9108,  57420, 34284, 21564, 37404, 41664, 14272, 59992,
                25356, 44476, 3212,  47172, 35692, 49308, 39620, 28044, 59904,
                9744,  41588,
            },
            {
                32236, 27944, 44300, 36460, 30464, 61360, 49768, 39968, 31792,
                54400, 148,   26972, 30672, 52764, 2876,  57492, 5228,  31336,
                47128, 380,   6340,  17608, 35468, 11032, 23572, 60692, 39488,
                9084,  63572, 22112, 12684, 3288,  43496, 15916, 32108, 25664,
                46172, 4304,  52980, 32300, 55964, 49292, 42612, 52732, 7260,
                55816, 41276, 184,   33848, 5568,  22204, 47624, 26432, 16036,
                61124, 10244, 58184, 36272, 30792, 64976, 46920, 53772, 14676,
                35540, 60028, 12108, 52992, 61428, 37336, 14576, 44020, 6948,
                35092, 61552, 56788, 24152, 51280, 38728, 18124, 28176, 61676,
                44404, 19332, 30448, 5428,  17748, 57616, 13160, 23172, 2936,
                10968, 47976, 33260, 55428, 35704, 23860, 19592, 59876, 10616,
                43788, 5448,  36192, 9676,  58412, 65248, 32444, 53284, 61840,
                36596, 47960, 64692, 11332, 52936, 5852,  62520, 30728, 4236,
                56268, 40456, 16048, 64684, 11232, 57648, 15376, 4028,  23280,
                55284, 1012,
            },
            {
                50552, 11528, 53848, 2488,  5384,  45600, 11744, 23124, 3932,
                16040, 51220, 41008, 10788, 61760, 38308, 48956, 40784, 64832,
                34664, 51064, 25656, 63032, 46028, 58320, 4096,  34256, 43636,
                50884, 1720,  55076, 33212, 40060, 28388, 54656, 50180, 61828,
                14348, 23624, 37108, 10716, 27916, 60480, 17828, 25040, 58924,
                19484, 37332, 64084, 49708, 53900, 41904, 60364, 37236, 55980,
                34052, 24740, 44052, 13184, 41164, 292,   18284, 11000, 42448,
                55596, 48752, 2152,  39412, 29636, 9236,  24984, 32420, 50984,
                2472,  48332, 14208, 29324, 34260, 6448,  42348, 52368, 568,
                23200, 56936, 65480, 48780, 26504, 42584, 30016, 51772, 40800,
                64628, 28608, 21856, 1416,  41496, 5268,  48764, 32920, 37840,
                17308, 55472, 27792, 14660, 50996, 17864, 26244, 44712, 4340,
                40180, 24828, 2452,  28928, 46848, 17124, 27164, 48120, 36308,
                12160, 21436, 54120, 32572, 26672, 8192,  62376, 30924, 36388,
                45328, 19740,
            },
            {
                58380, 40240, 16880, 63992, 24896, 38172, 18836, 28464, 64180,
                37400, 21716, 58188, 18204, 35124, 13796, 21364, 28268, 17248,
                12536, 55960, 30136, 9136,  42308, 20212, 49532, 12396, 16008,
                27860, 37392, 17572, 48672, 62232, 19596, 7696,  11580, 34960,
                40960, 56932, 19016, 47564, 3424,  21860, 39728, 11776, 32132,
                4152,  28284, 14464, 25484, 18600, 8056,  12252, 1072,  20188,
                5968,  50284, 17648, 7760,  51588, 25944, 58576, 31480, 5124,
                25368, 33240, 16368, 22700, 57412, 47372, 19040, 54628, 12272,
                26984, 20940, 37588, 10308, 47248, 57560, 25264, 12192, 36324,
                7776,  41180, 11136, 53036, 37140, 8324,  60828, 5624,  34920,
                15612, 43712, 57168, 62196, 13080, 51956, 63280, 2160,  25160,
                47932, 63720, 31360, 41032, 6408,  38176, 8,     29096, 13188,
                60036, 18652, 43112, 14588, 39348, 57932, 22868, 9272,  52504,
                63228, 29428, 6560,  37740, 18872, 44900, 41728, 51540, 12088,
                65240, 25544,
            },
            {
                7436,  34528, 21764, 48252, 32092, 59520, 55108, 50568, 9332,
                33572, 44480, 7012,  48084, 25604, 8564,  54084, 59348, 6824,
                43600, 22332, 39852, 14888, 52768, 30976, 25252, 65292, 54000,
                5588,  57876, 10372, 24732, 5096,  36864, 45144, 58472, 2492,
                29832, 6988,  43464, 62436, 34248, 8084,  63864, 45996, 50512,
                43224, 60820, 10764, 45488, 59248, 35820, 28932, 44276, 52780,
                64552, 39072, 28512, 55572, 63012, 22088, 37204, 45096, 62188,
                20484, 8532,  64372, 44384, 6232,  34860, 16,    59596, 39116,
                65260, 43544, 53684, 62676, 1856,  16688, 60704, 31740, 49100,
                54520, 16332, 33344, 2060,  21744, 15340, 44916, 19540, 49032,
                26260, 8616,  18864, 37132, 30424, 26960, 15800, 40000, 58348,
                12756, 3644,  23152, 46260, 60940, 21060, 48396, 58128, 33164,
                51568, 5712,  56368, 31636, 63412, 864,   34548, 45128, 18728,
                2456,  42940, 50260, 61288, 12,    53520, 21792, 5820,  16676,
                38688, 3552,
            },
            {
                46348, 29280, 57356, 9716,  14212, 6324,  41364, 1328,  46748,
                13476, 60984, 29728, 1648,  63820, 32168, 45204, 2460,  36772,
                49884, 61452, 4500,  57556, 1128,  37028, 7392,  40152, 21568,
                46444, 30416, 41964, 59896, 14960, 53444, 26776, 22552, 17468,
                52176, 59780, 25864, 13836, 54324, 30108, 36512, 1276,  15168,
                55280, 22764, 39020, 2432,  31576, 56708, 23348, 48076, 14608,
                31916, 21752, 1984,  46476, 32948, 15032, 8996,  50024, 2996,
                56692, 41004, 51972, 26388, 11584, 61928, 41736, 16068, 30124,
                8164,  3916,  18248, 31076, 22676, 44640, 8544,  39192, 20392,
                63100, 29076, 24592, 47252, 62560, 54288, 28016, 56544, 1696,
                60200, 51508, 4192,  46456, 10116, 54040, 44324, 21288, 8612,
                51212, 35532, 19084, 53180, 11752, 34332, 54812, 15752, 8984,
                22964, 46400, 35612, 20536, 7844,  49156, 13216, 60548, 26392,
                39204, 15200, 23324, 47616, 13620, 25632, 57312, 34096, 28660,
                49584, 61212,
            },
            {
                14872, 51964, 608,   43340, 52916, 35428, 17092, 26224, 57012,
                19700, 24268, 52616, 38520, 55776, 41872, 18928, 51664, 25184,
                10224, 33180, 19248, 27008, 44988, 17512, 59676, 33264, 2048,
                61364, 18776, 34684, 51564, 248,   31756, 64948, 41836, 48304,
                10132, 33048, 624,   39352, 49116, 17200, 57760, 26832, 20452,
                8492,  34888, 53032, 6060,  50320, 16496, 63192, 37544, 4016,
                9948,  42344, 61528, 12216, 4948,  40348, 54036, 28440, 17412,
                36108, 29960, 14052, 37128, 49948, 28184, 21692, 52672, 46504,
                24120, 35488, 58620, 40868, 51120, 28352, 55268, 14936, 3208,
                45836, 6160,  59844, 39932, 3888,  31380, 10028, 38624, 22556,
                12840, 32724, 39580, 59100, 23240, 34424, 6444,  56536, 30832,
                27220, 42236, 62100, 2032,  29496, 7620,  25096, 42272, 38632,
                64384, 27968, 10956, 53440, 25108, 41368, 55240, 30948, 7004,
                56564, 34952, 10068, 58832, 31164, 36932, 3024,  60452, 43516,
                9308,  23180,
            },
            {
                41676, 18712, 37420, 26884, 61932, 22664, 30496, 60496, 36888,
                5344,  42728, 11512, 16296, 4812,  22864, 12044, 62512, 15992,
                28936, 47644, 38760, 64088, 12824, 54580, 48088, 10056, 43128,
                26128, 12624, 7504,  47080, 21344, 39012, 13096, 3708,  35720,
                62860, 20220, 45388, 22980, 10484, 5056,  51816, 41268, 62044,
                47916, 29240, 65304, 42104, 21328, 11132, 7008,  27136, 59740,
                56240, 24360, 49644, 36652, 19952, 60600, 43756, 24128, 11344,
                47620, 59772, 1408,  19428, 5316,  56804, 33008, 3372,  10564,
                56092, 49656, 13912, 5320,  11628, 36832, 63712, 43044, 26628,
                35324, 12980, 51504, 18444, 43888, 13960, 36040, 63536, 47632,
                42200, 28964, 54900, 14988, 2532,  61592, 18404, 37368, 64420,
                46000, 5176,  16276, 38984, 58692, 44932, 62720, 3268,  49444,
                17892, 1292,  43684, 60908, 37676, 16520, 3760,  21952, 43652,
                51552, 64848, 27732, 5452,  40880, 17964, 46944, 13032, 19912,
                55824, 32208,
            },
            {
                64076, 11152, 58676, 5580,  47488, 11828, 44256, 8560,  53588,
                32416, 64364, 48612, 28216, 35496, 59968, 30688, 40248, 55260,
                1464,  58816, 7632,  50748, 34964, 22048, 28224, 15532, 55768,
                50264, 38048, 63780, 28704, 56944, 9472,  45752, 55588, 28340,
                15184, 50560, 56376, 29444, 64880, 43768, 24468, 12560, 32940,
                3220,  17896, 13756, 25820, 33840, 46540, 54472, 43284, 18920,
                33972, 15716, 53228, 30324, 26600, 57056, 1592,  34564, 65384,
                7184,  22008, 53276, 45460, 63892, 38784, 13240, 43724, 62608,
                20100, 31936, 27296, 46068, 60040, 1148,  21452, 7396,  49956,
                57392, 22976, 37632, 8516,  58820, 25908, 52240, 6344,  17060,
                61320, 7156,  20172, 25832, 49608, 41152, 47340, 920,   14564,
                10452, 55248, 32556, 49272, 23468, 18608, 35876, 12508, 30752,
                55272, 58856, 14368, 5196,  29168, 52124, 62292, 33864, 11372,
                17680, 1564,  20948, 44772, 54524, 63548, 8372,  51648, 27036,
                36476, 2588,
            },
            {
                53292, 24824, 30880, 40016, 20264, 64864, 2388,  49484, 14984,
                39936, 3476,  21028, 58080, 44040, 9132,  47912, 6088,  36128,
                21212, 42616, 11488, 24392, 3276,  41776, 6224,  62944, 31600,
                23204, 5284,  43740, 16344, 24524, 61256, 19692, 6176,  25008,
                39820, 8156,  3892,  37692, 18532, 34472, 59224, 7416,  38708,
                56100, 44236, 59996, 916,   57764, 39244, 30140, 1800,  49028,
                39840, 7732,  2768,  44940, 17188, 5948,  50844, 13748, 39508,
                55072, 32284, 42016, 25140, 8636,  17760, 48556, 25860, 36296,
                6888,  39320, 64312, 16848, 24992, 34268, 52828, 17952, 32068,
                61060, 844,   29736, 54848, 33720, 20016, 45768, 228,   24968,
                34728, 10892, 45084, 65316, 31432, 12532, 28052, 53472, 24900,
                20760, 60068, 26428, 9000,  13820, 56724, 51724, 5028,  26276,
                21360, 36952, 33300, 48024, 19532, 9380,  46024, 25376, 58456,
                48340, 37668, 32476, 50464, 11636, 23668, 29904, 39732, 62256,
                6508,  45048,
            },
            {
                14520, 34984, 3532,  50656, 15664, 33332, 55880, 24664, 19204,
                29140, 10480, 52000, 25168, 580,   50940, 15104, 26564, 65068,
                45892, 31496, 17752, 56332, 29836, 60348, 52412, 36688, 2536,
                18400, 59056, 52256, 1700,  36708, 48852, 33480, 52420, 43152,
                58796, 32232, 60868, 14152, 54840, 1452,  47548, 16200, 28024,
                52292, 23080, 36700, 9096,  51244, 19784, 14860, 62076, 11936,
                25904, 64892, 58304, 11092, 38644, 62316, 30960, 20688, 46300,
                26976, 3652,  15324, 35248, 60572, 30876, 840,   58492, 15444,
                50628, 2056,  42672, 9104,  56808, 48040, 29192, 39808, 10064,
                14640, 41584, 47760, 11472, 5216,  64916, 41040, 30744, 54068,
                49168, 57212, 37432, 5044,  52180, 8896,  59792, 33552, 43540,
                39372, 50608, 35316, 44232, 572,   40096, 32256, 46432, 63484,
                41756, 8136,  51364, 23664, 65512, 39656, 536,   14732, 41184,
                29488, 8796,  59860, 15952, 3588,  35260, 57576, 932,   16408,
                49912, 21380,
            },
            {
                8456,  61588, 42588, 54980, 7936,  28236, 36632, 41824, 58808,
                61804, 45520, 37160, 17432, 63300, 33184, 19960, 53816, 3180,
                13564, 51740, 61636, 38300, 44664, 9580,  20096, 12944, 40268,
                47664, 34168, 11476, 30472, 41876, 14172, 3096,  64416, 17184,
                11780, 47672, 24124, 42376, 26992, 50924, 21408, 31180, 63688,
                4468,  11664, 49052, 31976, 63512, 4984,  23920, 36304, 52132,
                32272, 22020, 47428, 35212, 54984, 23120, 42544, 8768,  59168,
                18272, 62264, 10624, 49576, 55372, 22632, 41472, 53356, 28752,
                61244, 23424, 54560, 19824, 37940, 12816, 3864,  64780, 55460,
                45272, 20888, 62652, 27468, 49832, 15408, 22308, 59268, 12512,
                19020, 2680,  43200, 22612, 16212, 36276, 19356, 3440,  62884,
                7124,  17372, 4388,  65164, 28684, 60292, 20008, 9824,  53960,
                16324, 2552,  44636, 11612, 31404, 56968, 35164, 54188, 5932,
                62640, 23212, 55448, 39440, 26856, 42956, 19624, 46376, 33612,
                25660, 57020,
            },
            {
                29440, 48072, 18076, 22844, 59660, 46544, 5692,  13144, 1080,
                34428, 7576,  54820, 30040, 12584, 42524, 38008, 60240, 23716,
                34440, 7120,  26796, 108,   15744, 33000, 46312, 57748, 27432,
                65024, 7636,  25536, 56588, 60312, 8752,  22960, 38180, 30336,
                924,   21088, 36448, 6844,  10984, 61724, 39796, 9428,  45812,
                19128, 41480, 27100, 16860, 43668, 28516, 55568, 8684,  44576,
                4372,  41848, 19236, 28456, 7248,  15268, 49128, 34016, 260,
                51404, 38144, 44304, 28300, 4108,  12540, 46660, 7716,  18832,
                11228, 34756, 47396, 31096, 6480,  59728, 43772, 22732, 26300,
                2428,  34208, 17296, 38492, 2860,  55900, 35004, 6664,  39380,
                27300, 33044, 63108, 29384, 58596, 40464, 55464, 23372, 47716,
                12796, 30076, 54564, 22220, 47368, 15220, 6644,  24304, 38328,
                29580, 61860, 58564, 27020, 4836,  16776, 49588, 27828, 19940,
                43960, 2104,  12948, 46904, 64556, 52004, 7608,  60964, 11268,
                41956, 37792,
            },
            {
                13088, 304,   26412, 38252, 11432, 62912, 21064, 52608, 48100,
                16092, 21864, 41204, 2832,  56992, 7920,  27748, 44644, 10208,
                58144, 40368, 47804, 21960, 64344, 50564, 3976,  22752, 42864,
                14560, 54504, 21160, 45088, 18036, 50528, 27440, 54684, 44652,
                57668, 50196, 63912, 53996, 33708, 44124, 3044,  56508, 35536,
                60224, 53668, 6804,  58840, 12836, 47816, 40124, 61168, 17700,
                56852, 13904, 60084, 1332,  52680, 64192, 25788, 12356, 56640,
                29660, 23500, 6724,  57684, 20364, 35948, 65096, 32328, 39952,
                44688, 4644,  14448, 63440, 27576, 51164, 15976, 35696, 49676,
                6832,  53552, 58236, 24288, 44436, 29072, 10136, 47256, 60896,
                51400, 14784, 9456,  48044, 700,   45288, 7736,  31896, 51544,
                37764, 59012, 41616, 10632, 52876, 36760, 43216, 57888, 50080,
                12940, 35276, 18816, 53604, 42240, 22596, 60764, 12072, 37324,
                52424, 33524, 18088, 30152, 4820,  22264, 14700, 31308, 55204,
                4232,  65368,
            },
            {
                32892, 59020, 45412, 53392, 2660,  32324, 43704, 25420, 56296,
                27896, 65220, 50144, 24216, 47460, 62300, 3984,  16444, 48980,
                19032, 29512, 55772, 10900, 37044, 26064, 54308, 8664,  30968,
                816,   49648, 36032, 3416,  39740, 32148, 61416, 4700,  10336,
                25976, 14720, 5188,  18956, 28664, 15956, 23312, 13004, 25636,
                84,    32924, 22372, 38236, 3020,  34252, 20708, 772,   30988,
                26816, 49408, 9940,  39768, 31376, 43660, 5300,  36456, 40980,
                16032, 64016, 33408, 14184, 52484, 38988, 25052, 3396,  55152,
                58984, 26120, 52252, 41056, 208,   20548, 32220, 9584,  61956,
                39088, 30520, 9072,  13176, 51228, 61656, 42424, 16964, 23908,
                4332,  42004, 54996, 26012, 18412, 11988, 64284, 27580, 15580,
                1232,  25580, 18736, 33872, 2988,  27084, 63944, 32696, 1396,
                22152, 40584, 6548,  48368, 32968, 2396,  44948, 8512,  64288,
                24588, 48832, 60252, 36608, 10192, 57920, 38756, 49012, 23720,
                17564, 51448,
            },
            {
                20516, 6868,  35484, 15328, 28672, 40340, 17640, 9140,  37756,
                31788, 5404,  14160, 33868, 39092, 21012, 32020, 52232, 36256,
                2020,  63508, 14176, 43352, 5220,  17152, 59352, 34940, 61700,
                38764, 16532, 28872, 63464, 6992,  13120, 46932, 20228, 35136,
                40628, 31648, 46244, 37952, 58908, 48348, 52196, 64748, 38812,
                49688, 14876, 45180, 54716, 64092, 25576, 50592, 11452, 53844,
                37520, 62244, 34432, 23896, 46536, 19948, 59376, 54028, 21624,
                45896, 3316,  48648, 41764, 61940, 8672,  15608, 50244, 21680,
                30296, 17264, 36120, 8320,  58528, 46088, 56036, 42284, 13900,
                18948, 47328, 63716, 4396,  36744, 20560, 996,   31772, 56868,
                35800, 21164, 61584, 38472, 52888, 35160, 21560, 42568, 60856,
                46200, 56136, 7864,  62580, 49020, 12496, 18388, 6116,  46328,
                56220, 26216, 9684,  63124, 14260, 38616, 58392, 30620, 16012,
                3728,  7648,  42204, 54480, 25492, 44596, 33948, 2976,  28596,
                9512,  43732,
            },
            {
                25012, 49220, 62456, 9576,  57696, 50288, 63904, 3500,  60316,
                11600, 44700, 58252, 18472, 9704,  54492, 13424, 59704, 41592,
                25144, 6136,  33600, 53648, 30628, 39988, 48116, 12276, 19620,
                45872, 57092, 11024, 52944, 22480, 58468, 42156, 16376, 63364,
                53468, 2320,  61220, 24876, 8764,  4420,  30500, 18260, 7544,
                28060, 62340, 10248, 29744, 17472, 7904,  57948, 45484, 22756,
                5684,  16208, 3684,  56308, 11860, 16648, 1948,  28736, 8508,
                60536, 11116, 19424, 30604, 968,   27640, 47220, 60944, 10384,
                2308,  62412, 48772, 23492, 11896, 37824, 24780, 3420,  51708,
                27844, 22572, 41092, 33272, 15516, 25804, 52700, 64304, 8644,
                45196, 12776, 3128,  32376, 6516,  49924, 56824, 4592,  10164,
                20368, 36692, 30560, 44800, 23564, 39436, 54620, 29220, 61560,
                15644, 43492, 51644, 29452, 20664, 54776, 25708, 47336, 40400,
                57028, 28648, 20960, 15144, 740,   62788, 18924, 53640, 61304,
                39468, 57112,
            },
            {
                13800, 30652, 41528, 19520, 23600, 6212,  33908, 47720, 20616,
                40088, 52392, 268,   63096, 26248, 45776, 29128, 7072,  22160,
                57400, 51176, 46420, 19776, 62804, 23604, 2920,  28116, 52384,
                24644, 5976,  41432, 33516, 26384, 36612, 128,   50868, 29104,
                7384,  22428, 43052, 12352, 55876, 41768, 33336, 46736, 55520,
                42468, 20628, 4684,  51748, 36424, 42248, 15048, 33416, 65436,
                40772, 44264, 29996, 51260, 62828, 37572, 48180, 32748, 50824,
                26028, 37988, 55068, 24176, 59124, 43988, 18572, 33064, 42264,
                38340, 13396, 55620, 33448, 18220, 65432, 29888, 6192,  60580,
                57116, 1200,  45728, 54544, 59620, 47860, 11832, 38568, 27808,
                50016, 30328, 17216, 58852, 23732, 14532, 28860, 39712, 33256,
                53256, 13364, 2484,  59380, 15912, 51368, 8688,  21100, 34200,
                4344,  37320, 57500, 80,    35768, 10812, 4680,  18372, 34780,
                51328, 11900, 63832, 32672, 47768, 40600, 12640, 7220,  46640,
                34652, 1728,
            },
            {
                53896, 4900,  37168, 55688, 44140, 13180, 26696, 53888, 16488,
                24456, 29268, 35868, 41864, 50460, 2972,  35288, 62104, 11356,
                38640, 16728, 27324, 908,   10292, 57644, 44248, 7516,  64116,
                32436, 1744,  59500, 17656, 48504, 9156,  56112, 24604, 11932,
                58172, 18148, 51308, 35568, 15260, 60608, 21656, 3148,  11904,
                34352, 58284, 39600, 23836, 61204, 49068, 2180,  29092, 7484,
                19100, 48584, 25724, 8704,  20980, 41656, 6432,  65008, 14096,
                43300, 34456, 7100,  51132, 12308, 35700, 56144, 6020,  52784,
                25304, 45784, 6496,  43008, 2028,  52552, 47604, 16484, 36904,
                32656, 11176, 17496, 8176,  29816, 5748,  41752, 18968, 4724,
                54456, 62824, 40128, 46956, 43524, 62092, 1480,  48684, 24568,
                64896, 50132, 27416, 42320, 4628,  35884, 60392, 41324, 47864,
                11212, 23944, 17388, 45748, 64740, 41412, 49852, 61504, 22412,
                1712,  45344, 36964, 53084, 5972,  27988, 58880, 21728, 15636,
                26676, 63312,
            },
            {
                23024, 45804, 16528, 1252,  29336, 59260, 36532, 2340,  45616,
                57292, 8760,  15424, 22356, 6484,  56576, 16260, 43996, 48796,
                31296, 8216,  42428, 60852, 37784, 50084, 15788, 35840, 42140,
                13628, 50484, 21420, 45008, 65440, 14764, 31224, 44220, 37200,
                47012, 32608, 64900, 27776, 652,   39228, 52668, 25244, 61628,
                16592, 47972, 1472,  32120, 12400, 20308, 26212, 59832, 52408,
                12916, 57844, 472,   60532, 34740, 27268, 55696, 23532, 18384,
                57768, 2344,  62176, 16808, 39992, 3944,  22588, 14408, 28784,
                63896, 19984, 31428, 60712, 27684, 40564, 8888,  21572, 43872,
                50148, 25080, 64680, 37948, 21456, 57064, 34136, 60280, 14020,
                22420, 300,   26336, 7884,  11148, 36404, 18580, 58096, 7684,
                16840, 38608, 21640, 55884, 10944, 31452, 25768, 1160,  64196,
                50672, 31800, 59436, 6752,  22928, 28392, 15056, 8104,  31692,
                58828, 25900, 9100,  17532, 23388, 50700, 31544, 37808, 52108,
                42724, 10076,
            },
            {
                59400, 49836, 32560, 64960, 52056, 8568,  41172, 61548, 31368,
                5172,  64668, 48760, 32612, 60228, 38368, 19444, 25448, 1860,
                65324, 21304, 52728, 34196, 12892, 29164, 22180, 54208, 18700,
                61048, 37272, 27184, 6756,  39344, 2740,  61316, 20656, 3696,
                40120, 9852,  6128,  49452, 17492, 29692, 9176,  44024, 37164,
                5680,  26956, 53404, 44964, 6788,  56600, 40160, 45880, 35824,
                24192, 38440, 31880, 53240, 14548, 45764, 11008, 3904,  40272,
                30224, 47900, 21472, 45628, 31456, 64876, 49420, 58844, 36776,
                892,   9628,  49536, 22384, 15076, 58248, 34760, 62476, 13164,
                6684,  40680, 52776, 27336, 44344, 1972,  50544, 25156, 43400,
                37112, 48812, 34572, 52080, 20736, 54556, 31208, 41560, 12288,
                44372, 6012,  32784, 63056, 17588, 45928, 56792, 19480, 14600,
                27264, 54128, 39560, 13012, 33792, 51920, 44268, 55436, 38292,
                13924, 42900, 54736, 61684, 3508,  43880, 10672, 64700, 2516,
                29940, 18492,
            },
            {
                40040, 7244,  12008, 25700, 46552, 20092, 14652, 22920, 11164,
                37436, 42808, 54024, 12416, 28260, 9224,  51052, 33436, 54440,
                13692, 45992, 4004,  24700, 55652, 6068,  46332, 31448, 9688,
                3432,  55812, 30356, 11392, 51852, 35040, 53784, 16980, 49000,
                60056, 26048, 21936, 54644, 45524, 62452, 58036, 13716, 51016,
                31052, 65048, 19392, 14708, 62768, 34656, 9972,  3804,  16448,
                63532, 10496, 43256, 17912, 5656,  50328, 36492, 61340, 52960,
                9008,  13260, 27812, 53644, 8184,  26096, 10960, 41540, 46412,
                16768, 56668, 39024, 53884, 11308, 3648,  25284, 54772, 30864,
                59836, 19656, 3988,  14360, 61868, 10724, 16908, 31340, 65444,
                8988,  56412, 16204, 29224, 64080, 5416,  25248, 47728, 62368,
                28744, 54152, 272,   48272, 23968, 52760, 6396,  38264, 44672,
                8824,  2756,  20236, 47496, 60540, 2076,  18912, 62488, 4212,
                48448, 20376, 30344, 39828, 35312, 56404, 19820, 47996, 5528,
                57824, 35232,
            },
            {
                27176, 61960, 21748, 38848, 3524,  35072, 50864, 56048, 48160,
                18192, 25712, 3584,  20964, 46520, 62668, 42304, 4728,  58200,
                28056, 36436, 59128, 16100, 40724, 64528, 1288,  58192, 48972,
                25028, 40988, 47240, 16180, 58872, 23012, 27404, 5812,  30148,
                55668, 14328, 42596, 36848, 4416,  20348, 34012, 1664,  22656,
                57052, 10476, 42884, 37760, 29312, 49700, 22644, 54952, 30828,
                47212, 2884,  57068, 23228, 64276, 28896, 20244, 43984, 24788,
                33936, 63692, 38372, 312,   35016, 55332, 19372, 3184,  32704,
                23832, 61936, 5272,  33780, 44680, 48436, 38060, 18328, 396,
                45924, 35596, 56632, 32464, 48004, 36736, 54792, 6800,  46440,
                20088, 12648, 58780, 2960,  45072, 38280, 14724, 2100,  34908,
                19268, 59632, 36724, 9112,  40012, 13204, 29944, 58884, 35076,
                62876, 42180, 29284, 56364, 9868,  26360, 36432, 23352, 28900,
                10756, 64448, 404,   15772, 7820,  26416, 13444, 33312, 23876,
                45312, 14436,
            },
            {
                48,    43536, 52644, 57264, 17552, 63260, 30308, 664,   27936,
                62320, 33856, 58480, 39904, 964,   30804, 14828, 22784, 10536,
                40044, 18788, 49564, 8816,  32992, 20164, 26648, 38100, 14704,
                63660, 5208,  19696, 33220, 43860, 8352,  41596, 64236, 11248,
                34768, 1132,  63164, 31072, 11968, 25444, 41304, 47360, 17416,
                39896, 7304,  24416, 55432, 340,   17684, 60584, 42332, 5992,
                20820, 28296, 49200, 40068, 33056, 1164,  59548, 16240, 3164,
                49132, 56612, 18740, 44352, 15528, 61576, 47964, 29124, 53132,
                12264, 42820, 26152, 19104, 29684, 64456, 6056,  51676, 28160,
                10576, 49620, 22368, 8556,  41096, 23708, 59200, 28812, 4116,
                40812, 25504, 33116, 41900, 22880, 59824, 51248, 55920, 9480,
                23008, 50256, 15252, 26804, 43856, 65124, 1876,  22692, 16844,
                49088, 24204, 5108,  37604, 16340, 43332, 54100, 6584,  40572,
                34068, 51788, 24892, 46596, 59456, 49928, 61232, 41256, 52864,
                8856,  55424,
            },
            {
                19736, 30412, 5740,  33472, 10652, 42516, 7900,  44856, 39356,
                5772,  14088, 50556, 16988, 24784, 57164, 53160, 35236, 63740,
                44936, 188,   30388, 62516, 42756, 51608, 11236, 44472, 34004,
                22648, 52276, 57268, 540,   62148, 13864, 50668, 37624, 46220,
                23868, 52200, 16656, 48248, 56344, 7148,  52928, 63676, 27140,
                50188, 35580, 59804, 4896,  46288, 26752, 13492, 33940, 52708,
                58736, 36672, 14128, 7984,  56000, 12456, 47660, 39532, 30584,
                6604,  22464, 10860, 60104, 40856, 24264, 5600,  38800, 63400,
                7876,  36116, 51048, 1628,  57424, 16056, 23016, 42892, 62928,
                39800, 16288, 29724, 64212, 1016,  18672, 12348, 52624, 38380,
                61020, 54092, 47536, 6528,  10888, 17824, 31892, 27560, 41052,
                46164, 3156,  30780, 61104, 20108, 33876, 50920, 10864, 54852,
                31884, 13676, 61128, 51172, 32840, 65516, 12432, 49736, 59940,
                15020, 56876, 42364, 32032, 19024, 37048, 1812,  28580, 17028,
                63428, 37264,
            },
            {
                51108, 13064, 60892, 48652, 27396, 54148, 24256, 60244, 20580,
                52816, 9912,  43184, 64412, 37004, 7560,  48196, 17792, 6092,
                25668, 54172, 12608, 23132, 5064,  55120, 17292, 59428, 7328,
                29300, 10488, 36588, 26660, 48380, 31408, 20404, 2548,  18004,
                58532, 39984, 8628,  27592, 60160, 37984, 15204, 32216, 3628,
                12672, 45380, 20416, 32564, 52260, 64480, 7800,  39256, 25004,
                11052, 63128, 19416, 42856, 25728, 52220, 35296, 9656,  58312,
                42660, 51148, 26760, 4408,  30200, 51776, 13504, 21604, 58152,
                18072, 47168, 14072, 61268, 40684, 8836,  31940, 13292, 56080,
                25408, 3068,  57992, 42752, 51180, 26948, 45604, 34220, 22124,
                16752, 1616,  29984, 63396, 49920, 36620, 7256,  64828, 13388,
                57780, 38428, 52400, 7044,  56604, 4608,  46796, 28080, 40500,
                3352,  57620, 44880, 8124,  22080, 1532,  30364, 18080, 45316,
                2696,  21552, 9568,  4828,  55384, 11644, 44832, 6320,  22028,
                31532, 46396,
            },
            {
                25100, 40292, 16252, 22272, 4000,  37892, 12448, 16140, 57552,
                36060, 31716, 22704, 4492,  45508, 11924, 27444, 41680, 50020,
                31988, 60956, 38344, 47184, 28484, 36000, 2780,  48432, 41352,
                60992, 45452, 17476, 40056, 6880,  24596, 60008, 54468, 28912,
                5024,  32976, 21208, 43220, 2072,  19256, 44860, 9460,  58772,
                29232, 62916, 16144, 40992, 11868, 30712, 48892, 18764, 44516,
                1620,  50740, 31328, 4112,  62184, 17036, 22716, 27620, 65380,
                15140, 36976, 33216, 47068, 55888, 9612,  34484, 44604, 732,
                31280, 55744, 28168, 21284, 34720, 54016, 47732, 36768, 7444,
                20716, 52436, 36204, 13876, 6568,  31272, 63152, 3324,  10128,
                50376, 36008, 13624, 19468, 26236, 54428, 43952, 21616, 4072,
                33596, 18044, 11420, 24448, 41196, 15012, 36820, 60444, 7748,
                21184, 35504, 18172, 27032, 40752, 47612, 57172, 24444, 37560,
                27920, 63072, 38992, 50716, 29436, 24168, 65204, 39512, 58328,
                11180, 3636,
            },
            {
                62120, 8288,  35744, 58100, 46772, 65460, 30900, 49112, 1672,
                26256, 47260, 55172, 29508, 60552, 34104, 20632, 59448, 3436,
                14412, 21056, 8220,  57452, 15052, 61892, 25380, 31504, 20732,
                14056, 2272,  54592, 63836, 12468, 52560, 34580, 9508,  45176,
                62116, 12868, 51140, 35716, 64992, 24376, 54540, 35200, 22076,
                51620, 2316,  54836, 8916,  23588, 37556, 3492,  55892, 61176,
                16088, 27064, 46508, 54608, 38052, 6980,  45348, 2748,  54140,
                20544, 12324, 62572, 1508,  17116, 37976, 64316, 49908, 24852,
                40440, 4972,  10180, 45000, 3168,  25912, 59212, 4296,  44328,
                60920, 33144, 9552,  47076, 60304, 39096, 15696, 56896, 42488,
                61980, 23652, 44556, 58056, 39520, 1192,  15780, 48708, 60020,
                25520, 44812, 62500, 48376, 32164, 53920, 18604, 24904, 43384,
                63244, 48484, 10936, 54264, 61624, 4704,  34900, 10024, 52352,
                7108,  33104, 13324, 61344, 16584, 35272, 53196, 14648, 49304,
                34144, 56224,
            },
            {
                28852, 44584, 53384, 1236,  41692, 19076, 34500, 6140,  41264,
                61356, 8368,  13616, 18752, 1888,  53540, 39516, 9368,  55804,
                36808, 52288, 43916, 1908,  19316, 39652, 51104, 5916,  56052,
                34480, 50208, 28272, 21896, 43268, 4216,  38332, 14820, 41340,
                26076, 55828, 47060, 5796,  10688, 30080, 48948, 5040,  42716,
                38748, 25760, 33684, 43932, 61744, 57660, 21116, 29404, 5556,
                35168, 40700, 9916,  21156, 33676, 59900, 50008, 40664, 31932,
                5792,  44776, 52976, 23528, 59112, 19972, 6768,  27696, 15416,
                60756, 52632, 37372, 65296, 50572, 11876, 19588, 30048, 15488,
                48996, 27256, 17580, 24052, 54956, 20180, 25812, 47820, 7312,
                27984, 54616, 5104,  32532, 9304,  62312, 30384, 35384, 9908,
                53128, 29068, 636,   21832, 8708,  63768, 2296,  12152, 51916,
                29228, 244,   39120, 31440, 14812, 20388, 42948, 64020, 16192,
                55140, 48532, 43584, 22680, 1188,  46984, 7932,  27216, 2132,
                42112, 17384,
            },
            {
                4652,  20636, 32016, 25564, 14496, 9164,  56248, 23900, 52028,
                17736, 38712, 63404, 42008, 49356, 25452, 15872, 65156, 28620,
                46768, 24284, 33624, 64052, 30680, 9528,  44896, 12364, 65300,
                24496, 38944, 8284,  32740, 58624, 46792, 18980, 64788, 22708,
                1212,  31148, 16352, 59460, 40424, 14180, 57524, 17584, 61420,
                13092, 7496,  49460, 17960, 972,   47400, 14156, 42812, 54104,
                23184, 64968, 58108, 356,   14852, 28712, 11192, 18516, 57276,
                25552, 39112, 8600,  28892, 42224, 46080, 31912, 56672, 11172,
                43552, 18808, 32904, 24116, 16628, 42216, 62812, 39424, 53388,
                660,   64868, 38204, 4816,  43852, 1488,  35664, 11112, 31832,
                18484, 38128, 12312, 46980, 51836, 22288, 42132, 55996, 5200,
                37780, 16268, 42708, 57480, 39364, 26888, 45684, 59336, 33860,
                15336, 56524, 24236, 5832,  51348, 59000, 28940, 952,   39928,
                19716, 25776, 3824,  57332, 40712, 59980, 32172, 19864, 62696,
                23284, 47592,
            },
            {
                38936, 63812, 12128, 60044, 50540, 28804, 62240, 44876, 11812,
                28188, 3292,  32980, 21588, 58068, 6680,  31624, 43104, 2324,
                18184, 11800, 5304,  41572, 53308, 22808, 58296, 36912, 18456,
                3616,  47608, 61672, 15728, 832,   26492, 30112, 56416, 48568,
                7892,  37876, 19808, 25340, 53212, 33920, 72,    28220, 46988,
                20884, 55640, 65264, 28632, 35852, 26136, 9300,  33016, 49252,
                7388,  13060, 30316, 47968, 43432, 24316, 63940, 36064, 1652,
                60776, 48564, 15596, 34388, 4688,  13168, 53236, 2688,  35128,
                22956, 7224,  58476, 1656,  29388, 56196, 8092,  34108, 22924,
                11408, 41716, 30276, 58588, 12896, 50152, 64164, 53416, 41140,
                58932, 140,   65128, 25648, 17524, 3688,  27692, 13528, 19744,
                64492, 49976, 6524,  35080, 14568, 50888, 30644, 38216, 5364,
                20216, 42192, 64452, 46100, 37148, 12304, 23408, 46532, 32144,
                60528, 11124, 36368, 30096, 18376, 10712, 44240, 55928, 37228,
                52136, 10300,
            },
            {
                57252, 49256, 34812, 5860,  43196, 38588, 2168,  20372, 35980,
                59528, 46208, 50948, 10592, 36548, 45396, 12812, 22488, 35560,
                61336, 50280, 56760, 26284, 16476, 48736, 432,   28740, 42124,
                53740, 10508, 20332, 56864, 36788, 49936, 6184,  11252, 33968,
                52084, 43512, 63756, 3108,  45688, 21836, 63316, 41464, 10460,
                32280, 37860, 14788, 5388,  53064, 59108, 40020, 60792, 17692,
                37096, 52284, 19232, 38872, 56492, 8260,  51520, 45980, 13976,
                29648, 20932, 55756, 64724, 50772, 25392, 61824, 38456, 47484,
                63816, 50164, 13272, 45644, 38004, 48560, 3704,  26680, 46048,
                56848, 7568,  51612, 21528, 33592, 28208, 6096,  23336, 14316,
                49232, 21332, 34060, 43048, 60232, 36980, 48264, 58428, 45308,
                23700, 31604, 11548, 60504, 20592, 3596,  10188, 23232, 57836,
                50112, 9124,  27736, 17832, 33036, 8472,  55580, 50064, 14244,
                5504,  44952, 63704, 52744, 6700,  50120, 25272, 13532, 6440,
                30896, 15496,
            },
            {
                504,   27080, 18064, 55232, 22916, 16152, 52696, 31964, 7628,
                54720, 23416, 15604, 29748, 992,   62016, 52496, 48372, 58260,
                8436,  30116, 37956, 13408, 60640, 6772,  32660, 62732, 15212,
                26316, 45020, 30696, 40560, 12788, 62588, 42736, 17740, 61120,
                27596, 12224, 56780, 32060, 8580,  37340, 51012, 6696,  24464,
                60284, 1864,  41992, 22984, 44528, 19944, 12256, 4452,  24844,
                44452, 2804,  62764, 26524, 4176,  34212, 22336, 4996,  53808,
                43032, 10392, 37364, 64,    17796, 40972, 21132, 9548,  16308,
                30252, 39760, 26444, 54844, 9780,  20840, 59476, 13700, 62112,
                18688, 36048, 15120, 62760, 46360, 39460, 17804, 60884, 44696,
                6732,  28820, 56204, 14804, 7848,  53712, 10964, 32956, 1804,
                40028, 54424, 27144, 46652, 53300, 43472, 65268, 16744, 46896,
                35816, 13172, 54376, 2404,  40444, 61888, 3956,  26916, 37672,
                57072, 21584, 27492, 15016, 39388, 34724, 64460, 2608,  41492,
                60804, 24624,
            },
            {
                45112, 40420, 9424,  46864, 30480, 4300,  64484, 48000, 13892,
                41628, 4732,  63852, 56756, 26140, 38396, 17172, 4132,  25280,
                41072, 20844, 1448,  46344, 35140, 19816, 44148, 55420, 7588,
                35512, 60264, 5608,  52524, 25060, 21460, 54160, 2544,  39276,
                22108, 4524,  15584, 50216, 27120, 18288, 13484, 56068, 35404,
                52712, 48848, 30348, 57244, 7968,  31688, 50756, 64300, 28768,
                57136, 32508, 10596, 49628, 16692, 60652, 40932, 17928, 32588,
                63256, 6476,  27108, 46764, 31596, 59976, 5224,  27824, 57504,
                1052,  19832, 4664,  62996, 31320, 16104, 35424, 50636, 29144,
                2172,  44100, 25352, 4240,  9636,  58028, 2424,  30964, 36260,
                10220, 39336, 50692, 2412,  30808, 40836, 24808, 61740, 17932,
                8972,  62936, 2872,  17316, 33128, 24800, 55236, 40236, 1568,
                31668, 62252, 22448, 48604, 57652, 20780, 43908, 17140, 52016,
                9148,  34008, 48992, 88,    55028, 20280, 28848, 47284, 53764,
                19324, 33476,
            },
            {
                64996, 4752,  58336, 36464, 60700, 11276, 24744, 56888, 27408,
                37704, 18896, 33824, 43428, 8012,  20572, 55636, 32568, 10808,
                44544, 64444, 54372, 28064, 10328, 40084, 23864, 50780, 12696,
                21788, 49096, 2284,  16716, 33244, 7940,  35912, 46092, 30392,
                59616, 47580, 35340, 40984, 61532, 58160, 43024, 29292, 3812,
                15632, 19340, 10856, 63248, 47236, 16516, 37700, 632,   46680,
                14492, 20676, 42232, 35748, 54880, 13048, 29028, 57936, 24640,
                39324, 50240, 58760, 23084, 12172, 44792, 54600, 33892, 42316,
                52192, 60164, 36172, 41884, 23168, 53264, 43452, 5596,  40436,
                55128, 31684, 48148, 53136, 20052, 42172, 55380, 25616, 51884,
                62656, 19044, 24356, 46488, 63828, 20876, 5348,  36284, 50472,
                30064, 21988, 42020, 37440, 5784,  13016, 28844, 8312,  59452,
                25852, 5048,  44660, 29180, 14164, 35152, 30936, 64068, 24188,
                41396, 4476,  62260, 43320, 24068, 9400,  58420, 16044, 36092,
                8548,  50788,
            },
            {
                29408, 21044, 14092, 51732, 19400, 40696, 34228, 44180, 68,
                60432, 9644,  47312, 53352, 13288, 40872, 28452, 63000, 14956,
                34308, 3656,  16740, 51336, 62032, 56880, 3872,  30776, 38540,
                64728, 28384, 42012, 56104, 47832, 64060, 26396, 14900, 9572,
                54640, 18932, 24920, 6052,  11036, 1304,  48164, 21140, 64580,
                45500, 39856, 25076, 2600,  34868, 27228, 55132, 22192, 40748,
                53208, 5276,  61312, 23548, 912,   43736, 9068,  48464, 1904,
                11576, 16264, 34616, 53008, 7712,  37052, 15004, 48856, 22508,
                10668, 45376, 14336, 7492,  47780, 428,   64988, 24492, 10080,
                21296, 61088, 11940, 37944, 27616, 33268, 12680, 16132, 46184,
                5816,  57536, 11980, 35144, 16640, 52036, 56484, 12876, 44548,
                58132, 14628, 48136, 56252, 61264, 51396, 35132, 21252, 42760,
                52312, 18856, 38860, 7348,  53176, 10280, 752,   47688, 12832,
                58448, 30192, 17568, 12092, 60384, 32136, 44420, 5164,  23188,
                56596, 12480,
            },
            {
                38076, 43240, 32496, 26332, 1772,  49764, 8432,  17016, 21708,
                50508, 31500, 24008, 2572,  61388, 50272, 6452,  46524, 51872,
                57496, 38816, 24336, 31828, 5540,  14808, 47444, 18060, 1476,
                57868, 9828,  19012, 37312, 11768, 3332,  57600, 50348, 42044,
                684,   65472, 44628, 53108, 31144, 23764, 38920, 33360, 9004,
                27428, 58892, 54052, 13712, 41312, 59716, 6996,  11224, 33784,
                58636, 8952,  27448, 47268, 30852, 65144, 52620, 36280, 20420,
                61720, 42768, 3216,  19260, 62744, 26084, 2812,  65180, 6784,
                31780, 25364, 18276, 61436, 27772, 12328, 33040, 17520, 37648,
                58440, 6904,  16736, 64664, 500,   50108, 59984, 22340, 3228,
                32888, 27412, 43228, 60476, 8620,  29328, 42116, 26424, 872,
                33760, 6836,  28140, 10020, 19856, 596,   47476, 63376, 15276,
                11312, 36236, 58512, 64884, 23632, 42288, 59604, 36516, 6284,
                53608, 20992, 40312, 47984, 36480, 3568,  51936, 38880, 27668,
                46236, 2140,
            },
            {
                24064, 53664, 62124, 6596,  44980, 63556, 29616, 58680, 53980,
                12268, 64964, 39528, 17600, 30576, 35732, 22908, 480,   19556,
                27272, 7048,  49056, 20324, 42392, 36908, 59760, 25652, 53424,
                45284, 32512, 24172, 61404, 44044, 31128, 20200, 38296, 23520,
                33536, 28520, 13548, 37144, 16940, 63028, 54784, 12872, 50392,
                4632,  36984, 32248, 5572,  51456, 20292, 44208, 62816, 30020,
                17980, 37024, 51792, 15088, 38976, 18352, 5752,  26172, 46096,
                54500, 27876, 30984, 47504, 40224, 56228, 30028, 17444, 39220,
                58540, 55008, 34820, 50840, 38892, 57940, 54548, 45564, 49780,
                28628, 42592, 34712, 23756, 54712, 8140,  43884, 36740, 64264,
                40636, 54312, 1576,  22540, 38392, 4436,  48896, 19464, 60092,
                53372, 39272, 64172, 25124, 45140, 31136, 39212, 4100,  56620,
                32628, 50184, 2016,  16504, 26704, 49516, 19888, 15548, 28204,
                46132, 33628, 1968,  56844, 26104, 19240, 63680, 13880, 59368,
                18084, 61188,
            },
            {
                48676, 9632,  17408, 56072, 11984, 38420, 23748, 36340, 6364,
                42512, 27800, 4864,  57584, 45708, 10032, 59848, 37680, 42844,
                11304, 60616, 45136, 9520,  63968, 27852, 11108, 34244, 40940,
                7792,  14396, 52072, 6360,  27460, 16232, 60136, 4472,  12664,
                56120, 8032,  49596, 59008, 2968,  46264, 7040,  19200, 61024,
                43612, 17256, 22260, 62308, 48228, 15748, 26408, 1784,  50172,
                23100, 44976, 2772,  62288, 11060, 22564, 59068, 32352, 14516,
                38556, 4940,  57372, 13500, 9208,  21412, 43568, 51624, 12592,
                46576, 1956,  5160,  29900, 9268,  20072, 5940,  26500, 1608,
                13828, 52060, 3788,  46836, 39128, 19136, 29396, 11012, 49464,
                18096, 13916, 30872, 48092, 57240, 15352, 65336, 37180, 11064,
                23136, 16292, 49788, 35516, 13728, 59164, 17436, 24112, 27612,
                44244, 8776,  29876, 46712, 34156, 3472,  54744, 39052, 61680,
                24680, 10524, 65364, 15680, 7756,  49288, 30856, 9936,  41712,
                33832, 6412,
            },
            {
                14968, 35608, 41120, 28256, 20668, 52920, 14836, 3052,  46352,
                15948, 34932, 49312, 21312, 14116, 26588, 54360, 16164, 65388,
                30092, 14216, 55800, 33076, 860,   50916, 22440, 4612,  62448,
                20856, 48748, 39260, 132,   55212, 46952, 34448, 51300, 63336,
                43404, 18296, 39556, 21964, 27092, 34916, 41068, 29416, 24328,
                52956, 512,   56568, 29884, 7780,  38200, 34452, 56464, 12860,
                39812, 59412, 7292,  34164, 55932, 41328, 50324, 2384,  64040,
                10004, 22292, 61044, 35760, 50804, 1216,  34044, 60016, 27724,
                20792, 37176, 64228, 22640, 44668, 41636, 15216, 63444, 36508,
                60308, 20460, 57476, 30764, 14296, 61784, 4696,  57788, 26092,
                7080,  51516, 63084, 10568, 44440, 33328, 24208, 3480,  30756,
                46604, 5616,  57612, 3340,  41844, 8444,  54244, 48392, 6168,
                62172, 22216, 55044, 40092, 12060, 62940, 31412, 8080,  42828,
                3896,  51056, 37856, 28796, 54248, 40596, 22792, 2620,  55020,
                50712, 29872,
            },
            {
                620,   58216, 47556, 4912,  33504, 59796, 48776, 31768, 63136,
                24792, 55584, 9088,  62364, 43572, 2040,  31852, 48840, 4916,
                35056, 52756, 22000, 39672, 17612, 57772, 43252, 54796, 16612,
                29340, 58744, 35156, 64440, 22744, 10040, 40896, 6640,  25716,
                30504, 1724,  62392, 10312, 55536, 14524, 49300, 57960, 9952,
                34304, 13276, 40264, 45664, 10660, 23508, 65392, 4056,  46828,
                16452, 31764, 25808, 20328, 48008, 28816, 8448,  35032, 43692,
                17728, 49808, 25460, 45056, 16724, 63620, 24432, 10072, 6204,
                42388, 15960, 48800, 13076, 59536, 51720, 31804, 23000, 47576,
                7836,  33284, 25704, 9364,  41800, 48576, 23260, 32480, 45332,
                39588, 21580, 35640, 26580, 18756, 7428,  54384, 50936, 60988,
                43132, 18716, 32368, 52912, 21340, 28524, 65028, 33932, 12384,
                37636, 14896, 60372, 19120, 5924,  51656, 22728, 13436, 56532,
                18512, 59416, 22208, 44076, 12956, 34732, 58264, 45816, 25672,
                20836, 63952,
            },
            {
                39376, 19304, 24908, 65456, 43372, 1120,  10364, 41344, 57408,
                19720, 38912, 3412,  33520, 52940, 40308, 58684, 24472, 20148,
                41816, 2528,  25992, 6144,  48056, 30660, 13400, 37472, 46740,
                2912,  25496, 12452, 43036, 18656, 29000, 53924, 14028, 20788,
                48240, 35904, 52372, 45736, 32188, 5240,  64920, 3312,  38160,
                47800, 63528, 26928, 18964, 60332, 52564, 41460, 27964, 54332,
                9816,  63856, 42824, 52468, 4712,  16000, 24504, 57800, 52752,
                30164, 40964, 7212,  32044, 4588,  54004, 38084, 47560, 55984,
                62444, 33156, 54364, 26988, 2332,  35676, 4224,  56232, 40212,
                12236, 43420, 62992, 17860, 55528, 35176, 1732,  53804, 15456,
                61152, 4884,  55604, 680,   59656, 38940, 28244, 14036, 40452,
                9116,  26468, 63180, 11956, 38024, 44748, 2256,  19980, 41168,
                52692, 76,    25756, 43288, 58112, 37012, 45536, 27240, 48664,
                35400, 30548, 5780,  47596, 880,   62416, 6960,  16336, 37576,
                11352, 44288,
            },
            {
                52236, 32304, 8772,  13508, 51320, 22500, 17660, 27928, 7216,
                11764, 50680, 29352, 23608, 18164, 6656,  12520, 46212, 8840,
                57704, 49796, 63092, 36176, 59928, 8280,  26480, 65080, 10324,
                53004, 31536, 7200,  50192, 57828, 2500,  37540, 45324, 61008,
                57436, 15832, 7164,  23552, 18668, 43144, 26196, 21244, 16440,
                30972, 6840,  51092, 2124,  33132, 14680, 6100,  18504, 35964,
                22424, 24,    57076, 11852, 36524, 45228, 63188, 12524, 20652,
                488,   55516, 11964, 58164, 42228, 28800, 14288, 18884, 31088,
                220,   24004, 8072,  39792, 19072, 61900, 10348, 16556, 29136,
                54020, 5004,  50420, 2824,  27804, 12572, 64928, 19844, 37300,
                9732,  29460, 46644, 42032, 12344, 47776, 63348, 2188,  21648,
                35864, 55512, 716,   47132, 24292, 60624, 15892, 50304, 57448,
                31048, 46064, 35028, 10152, 29036, 2964,  16216, 63924, 1580,
                41036, 11412, 16792, 56188, 26880, 19848, 51688, 33088, 60196,
                3608,  27436,
            },
            {
                62036, 4440,  55712, 36372, 29964, 39680, 61736, 53332, 36856,
                44448, 60768, 14488, 47712, 64240, 36428, 28564, 61468, 38592,
                16616, 29060, 10824, 14776, 44416, 20412, 1928,  33580, 41624,
                18496, 62140, 38504, 15476, 33400, 63504, 23788, 31472, 10748,
                3284,  41688, 28880, 61500, 38852, 59284, 11300, 53660, 44820,
                57108, 22968, 42436, 37124, 57632, 48388, 30536, 44620, 61540,
                50732, 38284, 29860, 19092, 60968, 2612,  31228, 39148, 47300,
                27540, 37412, 62724, 19228, 22796, 49392, 3076,  58904, 40796,
                11348, 51044, 45940, 58724, 30220, 48460, 24436, 44572, 64392,
                19496, 37972, 22820, 59708, 44996, 39964, 7796,  50576, 43108,
                58372, 23848, 15996, 51752, 31012, 23048, 34784, 17368, 45152,
                58876, 29588, 15244, 51224, 34468, 5156,  29936, 10580, 23044,
                3972,  17460, 64428, 53948, 21076, 49428, 33092, 9440,  24400,
                60344, 53288, 33528, 63540, 39408, 42848, 29948, 9256,  48220,
                56088, 14068,
            },
            {
                22936, 49212, 16784, 45548, 57944, 6256,  46760, 25400, 4516,
                31436, 54908, 344,   42984, 10320, 56040, 21440, 50252, 3344,
                33900, 45608, 53840, 24032, 40644, 51500, 56560, 48416, 22620,
                5700,  46036, 55952, 21172, 9384,  48508, 6016,  17636, 55088,
                26476, 34184, 12372, 47748, 236,   35556, 50072, 28248, 2420,
                62012, 9404,  13996, 25676, 4384,  21700, 59564, 8964,  25152,
                5080,  13964, 48296, 26176, 40884, 54388, 17456, 7888,  60524,
                4660,  15708, 33888, 45936, 8908,  64636, 35368, 26372, 44824,
                21812, 36540, 16984, 4968,  12656, 37512, 52824, 33124, 464,
                26664, 47796, 35476, 15736, 31712, 52972, 24816, 30596, 13596,
                3796,  34036, 64396, 19996, 8452,  56972, 5092,  53068, 11492,
                49376, 6216,  39628, 19440, 8924,  58544, 42160, 36268, 62872,
                27788, 48596, 7708,  14132, 41580, 61952, 55660, 38664, 44492,
                20724, 49952, 8016,  23496, 3980,  14540, 61100, 24720, 18316,
                40928, 35520,
            },
            {
                9668,  42208, 25960, 3260,  20344, 11908, 34340, 15040, 63844,
                18556, 22092, 34688, 26344, 39764, 5536,  32652, 14076, 25128,
                64588, 19404, 416,   31652, 61440, 5116,  28428, 15348, 60512,
                36396, 25016, 1424,  43748, 26740, 40580, 60148, 50988, 38412,
                44128, 64764, 52832, 17244, 56184, 8304,  14312, 32776, 40492,
                19528, 35860, 51968, 64520, 43564, 11804, 38656, 53812, 32604,
                17356, 42084, 59088, 6556,  10104, 33348, 49724, 23960, 56700,
                42076, 50640, 26000, 1604,  39688, 52140, 12716, 5708,  61328,
                53048, 28692, 65416, 55852, 43608, 21384, 6744,  57260, 9408,
                60336, 13612, 6512,  56580, 10260, 20752, 812,   62156, 47384,
                26900, 54816, 40388, 2508,  36804, 61776, 41744, 25844, 32872,
                64972, 22884, 43688, 62460, 53172, 25952, 46464, 55156, 5840,
                39444, 59488, 32412, 37732, 25260, 6428,  18336, 30312, 4504,
                14268, 28680, 37620, 46196, 10988, 54296, 50168, 1312,  45408,
                5400,  30636,
            },
            {
                54280, 59784, 37928, 63104, 28332, 52096, 59372, 1912,  48500,
                41272, 8976,  58732, 48968, 16580, 59764, 51992, 43848, 55360,
                6456,  42056, 57376, 8980,  35044, 17712, 38792, 10760, 53992,
                30128, 13772, 59364, 52548, 29704, 12620, 35252, 836,   22436,
                8820,  19916, 4020,  25412, 30708, 22104, 42840, 60732, 4928,
                54876, 46600, 31736, 17904, 55872, 28088, 15900, 1380,  47104,
                64824, 54924, 34628, 23076, 52184, 64168, 28504, 13132, 35308,
                21168, 9976,  54184, 60904, 29364, 17920, 32100, 55656, 15660,
                3404,  9252,  34140, 25624, 1468,  62344, 41760, 15360, 50524,
                39924, 29972, 42480, 63332, 49208, 34296, 59140, 16660, 36136,
                7408,  49612, 11400, 45580, 28124, 14144, 48356, 19244, 8652,
                37240, 2828,  27520, 12728, 32004, 1204,  16924, 13360, 20336,
                44184, 11496, 22416, 50900, 976,   46948, 12700, 51996, 57840,
                65076, 284,   18108, 59228, 31528, 35600, 20296, 38500, 57692,
                63288, 21104,
            },
            {
                46324, 44,    13040, 33568, 8332,  43580, 38732, 24232, 56328,
                28856, 12968, 52544, 4164,  30996, 19804, 1544,  27552, 11456,
                36800, 49164, 22824, 13312, 47144, 25344, 64232, 44848, 4032,
                50336, 8412,  34364, 18040, 3940,  56952, 45064, 15136, 62360,
                29844, 50140, 58536, 36356, 46392, 63748, 52100, 16708, 24692,
                11220, 27480, 564,   7808,  40036, 48904, 61716, 35740, 21196,
                27352, 3028,  12688, 46108, 19896, 1004,  43532, 6416,  46500,
                63564, 31288, 13972, 23360, 6504,  47652, 43208, 20892, 37352,
                48280, 41372, 20004, 14668, 50076, 31660, 27752, 18568, 34640,
                23976, 52664, 2080,  18320, 26488, 4944,  41128, 44692, 56288,
                22184, 18396, 31744, 59592, 23452, 55328, 280,   30844, 60356,
                54672, 16480, 57308, 48808, 38692, 22072, 61508, 33068, 52332,
                26852, 3356,  54968, 58404, 28556, 60980, 35972, 42780, 26024,
                33960, 47888, 55164, 42404, 25748, 64368, 7784,  28552, 12048,
                33980, 15808,
            },
            {
                50292, 29584, 18768, 56820, 47264, 21776, 16608, 32276, 7448,
                36184, 45576, 65252, 24812, 37820, 62612, 46668, 40380, 60636,
                17052, 30168, 62248, 39480, 52872, 58032, 1268,  31908, 21576,
                42244, 62728, 39748, 46852, 65332, 20312, 25324, 53800, 33192,
                5340,  40832, 13984, 9988,  39420, 2092,  6972,  34504, 57588,
                38476, 50032, 62204, 23144, 33804, 3676,  52516, 11156, 44200,
                8404,  38080, 31108, 61772, 39724, 15620, 36900, 59296, 18488,
                2584,  38972, 43936, 56452, 36076, 58572, 9876,  63700, 24512,
                29716, 59700, 54368, 45844, 36756, 10432, 58556, 47308, 5204,
                64784, 11328, 21840, 46228, 38124, 54320, 28864, 10608, 3192,
                39208, 62432, 52584, 4488,  16876, 38704, 44360, 51356, 13308,
                40768, 45852, 34224, 9692,  4392,  58084, 41060, 7860,  47848,
                65452, 17996, 34744, 15028, 45036, 20104, 9896,  22828, 7296,
                40320, 11756, 21820, 6040,  2632,  48420, 15504, 43796, 53248,
                25572, 6308,
            },
            {
                65104, 23276, 40192, 53024, 2788,  64208, 10752, 50276, 61780,
                20160, 1376,  15968, 43124, 10980, 6672,  23612, 33680, 8408,
                21484, 54832, 2264,  27456, 6984,  20056, 12552, 37580, 55632,
                27124, 15568, 23924, 11048, 32184, 6592,  41944, 9760,  47364,
                57324, 23984, 44428, 61748, 27000, 19188, 48844, 29612, 44888,
                20816, 14944, 58980, 10532, 45668, 19376, 24632, 30800, 58452,
                49712, 18544, 55836, 5776,  27112, 57404, 48744, 30616, 24612,
                52280, 8228,  49828, 17064, 3912,  27244, 40472, 668,   50808,
                7572,  12912, 2776,  63584, 7208,  22696, 53748, 2864,  38400,
                44496, 55664, 32156, 61400, 7056,  13372, 23432, 65448, 32592,
                14300, 25032, 42648, 9316,  33912, 64576, 5728,  22096, 29108,
                7024,  24884, 20568, 64200, 28568, 50964, 24764, 35572, 2260,
                29828, 42380, 8300,  39848, 4792,  32240, 53904, 62536, 15824,
                51268, 31168, 61884, 16852, 39604, 57000, 23104, 61380, 1880,
                41284, 36348,
            },
            {
                10784, 44500, 4808,  15304, 31032, 27128, 42296, 5532,  53984,
                39696, 26868, 33228, 56164, 49248, 54256, 14252, 58356, 48384,
                3948,  35772, 15208, 45828, 33764, 43500, 50880, 60300, 18208,
                5956,  48920, 324,   54136, 36884, 49844, 61616, 27924, 18616,
                35736, 2604,  16976, 32124, 54532, 59884, 13328, 9160,  65312,
                2692,  41428, 35576, 28824, 56732, 64328, 6292,  41188, 15284,
                61080, 24076, 43408, 51480, 10556, 22116, 4404,  54356, 11620,
                61036, 27716, 33280, 64924, 22312, 12012, 52948, 34316, 18468,
                44336, 38152, 32268, 26296, 17220, 40324, 61572, 13888, 19852,
                28720, 8476,  15876, 36180, 58088, 48880, 17908, 51760, 45948,
                58720, 1348,  29604, 47244, 58240, 26688, 11320, 36188, 62684,
                48080, 1692,  53616, 43004, 18116, 13872, 45572, 10728, 53508,
                21500, 60296, 49988, 24000, 63884, 48792, 2200,  38196, 57384,
                3380,  45264, 27380, 53360, 34624, 29776, 9648,  50848, 31120,
                17868, 58352,
            },
            {
                54116, 26324, 35120, 61140, 49456, 37480, 58752, 23148, 13404,
                46992, 58460, 8860,  21148, 29820, 36536, 18424, 27940, 43264,
                64940, 40948, 51432, 10872, 63212, 24016, 29380, 9324,  35500,
                45108, 58016, 28600, 60788, 21368, 16224, 1896,  39280, 12932,
                64544, 49380, 52808, 5848,  21540, 35084, 42412, 24300, 55452,
                31632, 5788,  53312, 17084, 42680, 13256, 37072, 54624, 4244,
                29500, 420,   13440, 35892, 32584, 65520, 40844, 14752, 34692,
                41988, 20440, 1064,  38072, 44536, 60112, 31004, 14472, 61732,
                56812, 22228, 51768, 57872, 43892, 29968, 50776, 24864, 33916,
                60168, 49744, 26148, 43620, 168,   30704, 42144, 8388,  35620,
                20024, 50160, 37192, 13480, 20804, 41104, 53724, 18256, 57004,
                32580, 15432, 59104, 36096, 5392,  30796, 56456, 62984, 16624,
                38156, 12936, 56884, 27424, 11160, 17720, 29492, 43344, 24768,
                19956, 36860, 9220,  58952, 6580,  46156, 13148, 36936, 21548,
                47456, 7340,
            },
            {
                32700, 20288, 57180, 7996,  12000, 19336, 776,   34384, 29916,
                17816, 3088,  61284, 41880, 4708,  63708, 1096,  52852, 12376,
                25048, 30836, 18860, 57100, 4324,  16176, 41668, 2336,  64808,
                13224, 33028, 38752, 7828,  43900, 30716, 52676, 59152, 22456,
                7672,  29024, 38532, 11788, 46104, 1444,  51592, 37416, 18168,
                48152, 26496, 8960,  49864, 1996,  25608, 32828, 50752, 21876,
                47852, 39972, 62832, 17484, 47276, 2724,  26072, 45424, 6816,
                57060, 47084, 10236, 15344, 55148, 48468, 24860, 4456,  47140,
                27956, 10196, 15468, 4924,  34928, 508,   11504, 46168, 6280,
                41048, 2540,  53192, 12140, 20504, 63036, 25548, 3560,  55000,
                27664, 6708,  63872, 56012, 4988,  49320, 1180,  45244, 9040,
                39160, 27132, 11820, 49972, 61192, 39700, 212,   23584, 43440,
                3968,  32708, 5768,  36484, 44704, 59084, 34636, 13196, 6964,
                48096, 64560, 14308, 23616, 42668, 19552, 64028, 55672, 4624,
                60320, 14428,
            },
            {
                38908, 3736,  47000, 40820, 24332, 45720, 55944, 63112, 43236,
                53016, 37852, 50444, 15036, 24136, 46032, 39536, 32624, 6004,
                56016, 9016,  22996, 48132, 59680, 38544, 53560, 47792, 25736,
                22276, 51660, 19068, 11920, 56076, 25764, 4432,  45856, 32636,
                55484, 43484, 24488, 63568, 56388, 28012, 15544, 4272,  58308,
                12184, 62712, 38876, 22224, 59912, 46336, 7724,  63276, 10156,
                34344, 26436, 57952, 8308,  53596, 20240, 60156, 29808, 18016,
                62072, 24108, 51380, 29176, 5352,  19668, 8420,  40756, 36384,
                2176,  42360, 62616, 48608, 20184, 65092, 54916, 37208, 57736,
                17744, 64112, 23064, 33748, 56432, 39016, 15320, 60872, 40764,
                12024, 17228, 43760, 24088, 32116, 15888, 28748, 60060, 23820,
                51636, 3456,  42196, 22544, 8828,  19988, 33716, 49416, 26708,
                55400, 46616, 19380, 52492, 604,   22320, 50224, 55252, 60212,
                28372, 33176, 52040, 648,   50048, 26940, 2888,  40032, 24648,
                43376, 28280,
            },
            {
                61512, 52376, 17196, 64632, 28616, 51616, 15648, 9804,  4868,
                25300, 11484, 28264, 34736, 56964, 10232, 50980, 20808, 60188,
                45516, 38108, 152,   32860, 26964, 7536,  19876, 31064, 56548,
                5192,  42888, 3176,  63932, 47436, 14772, 36156, 10200, 40552,
                19652, 424,   14500, 31248, 9264,  41176, 62028, 33324, 45296,
                20456, 43468, 34280, 55264, 29260, 18628, 40364, 15856, 56392,
                19108, 45168, 5380,  30912, 42276, 37908, 10352, 49440, 36288,
                4740,  39720, 32312, 63160, 42644, 34856, 58360, 64404, 21336,
                55352, 33080, 25212, 39036, 9488,  27768, 15108, 21940, 31316,
                9984,  47008, 37924, 3996,  48600, 9340,  45172, 32384, 21984,
                59180, 34632, 51932, 10216, 39056, 63232, 35616, 13044, 43168,
                19348, 65060, 34420, 29504, 45200, 53040, 15092, 59568, 9740,
                30492, 64744, 14304, 60924, 41896, 8700,  16124, 39368, 2576,
                10740, 41236, 17776, 38092, 61172, 31372, 15792, 52636, 33704,
                11336, 49932,
            },
            {
                23040, 9624,  31244, 2364,  35788, 6248,  39596, 32844, 48740,
                21644, 64820, 44656, 6848,  17992, 62144, 26720, 13932, 35908,
                16576, 62680, 52396, 13584, 44612, 35208, 11608, 61800, 15272,
                36376, 59664, 27588, 39980, 32096, 22888, 61160, 50496, 16632,
                62372, 46844, 58496, 36960, 18588, 49476, 22156, 6852,  53864,
                29652, 320,   15192, 5476,  11056, 61488, 31464, 1256,  52440,
                37696, 12472, 50300, 24508, 14200, 56768, 768,   52652, 22636,
                13296, 54196, 2512,  17544, 11672, 26208, 50968, 15684, 11040,
                49604, 18200, 6872,  53516, 45276, 58960, 41792, 3444,  51988,
                26568, 14256, 29624, 59404, 18068, 28308, 5144,  53280, 48988,
                888,   29520, 46728, 3084,  58592, 21216, 52952, 4124,  31644,
                7852,  54488, 16244, 57968, 5952,  63736, 36948, 2880,  41604,
                22136, 6692,  38856, 24936, 29244, 33548, 64136, 26192, 19500,
                44984, 54612, 4888,  57288, 11684, 35688, 47304, 8592,  62888,
                18868, 456,
            },
            {
                34844, 42400, 14004, 57572, 44156, 22236, 59540, 18708, 54112,
                36200, 58716, 392,   31580, 53504, 41452, 2504,  30460, 49372,
                3780,  28152, 41404, 20912, 64516, 55476, 1848,  40656, 45480,
                9872,  49488, 16836, 53088, 936,   8496,  56720, 28716, 5288,
                26380, 34896, 7168,  52540, 3504,  59640, 25948, 39648, 10048,
                64472, 24560, 58964, 51512, 36956, 48688, 43628, 23340, 28136,
                59080, 2836,  61324, 21512, 63408, 27648, 34272, 16560, 63868,
                43868, 26784, 59748, 37460, 56716, 45744, 40,    31992, 28380,
                44172, 61016, 13488, 30992, 23112, 5252,  49876, 33584, 63236,
                44332, 55728, 6820,  42324, 51844, 24012, 64776, 13604, 37872,
                8920,  61996, 19152, 26592, 42676, 7524,  47844, 25680, 61228,
                46192, 37564, 984,   48912, 24348, 11520, 28108, 47572, 18308,
                53776, 34936, 50876, 10552, 57728, 47348, 3856,  52704, 31592,
                62648, 22624, 30144, 25360, 44392, 6460,  21976, 58628, 29532,
                45156, 56448,
            },
            {
                26936, 63304, 20680, 53412, 26168, 12444, 62468, 30288, 3256,
                14016, 40132, 19928, 48524, 23244, 37036, 8244,  57488, 43412,
                23980, 55056, 10168, 4788,  49508, 16928, 24996, 51900, 21200,
                29668, 6572,  24160, 35008, 62848, 20576, 37836, 42916, 53988,
                11748, 48944, 20612, 27728, 44380, 12080, 32380, 47772, 16732,
                36012, 46496, 41216, 19284, 25744, 3692,  13632, 65004, 7116,
                41648, 32424, 35992, 44008, 4080,  46996, 40792, 7400,  30720,
                48020, 9536,  20936, 49328, 6896,  23068, 39644, 53460, 4852,
                37600, 56860, 1176,  35956, 62200, 17420, 38424, 9120,  19512,
                1680,  34948, 21260, 62552, 10768, 36616, 31212, 19756, 43456,
                24856, 53872, 14928, 36752, 55644, 11872, 17808, 40116, 56260,
                21808, 13724, 26836, 40412, 18888, 43232, 32456, 57340, 13320,
                60612, 1420,  45040, 16816, 20620, 6132,  43064, 12924, 36316,
                47676, 8204,  14536, 49324, 65228, 18244, 54928, 41388, 13824,
                5764,  38224,
            },
            {
                48876, 7432,  46060, 37500, 4720,  47540, 8172,  42152, 26932,
                46884, 9168,  55940, 12120, 60072, 15732, 46404, 65496, 11556,
                18460, 34552, 60440, 31584, 37720, 28972, 59036, 4528,  33836,
                64296, 54972, 41912, 11464, 44960, 48644, 13660, 3008,  32460,
                23332, 65188, 39300, 56608, 15152, 63048, 1280,  51896, 56828,
                4692,  12904, 32264, 8588,  54048, 58224, 34436, 46928, 20140,
                49296, 17236, 8756,  54948, 18528, 11360, 58860, 24976, 56264,
                3540,  41784, 33608, 14972, 29824, 62324, 18852, 59624, 9616,
                21756, 25936, 41356, 47544, 11364, 28960, 54804, 24972, 60468,
                12504, 48244, 39152, 16116, 1036,  45848, 55048, 3496,  57864,
                33052, 40620, 5800,  65232, 30264, 33444, 51240, 1556,  29552,
                10396, 63116, 33736, 53600, 60216, 3336,  51284, 7268,  23784,
                39892, 27508, 31312, 62880, 54808, 38000, 23740, 60996, 16912,
                1832,  58444, 34608, 40360, 3620,  27280, 37244, 1816,  24324,
                51820, 16508,
            },
            {
                10924, 33200, 1756,  18132, 58568, 32408, 50796, 56352, 17180,
                63296, 34040, 29424, 43312, 26520, 5036,  32744, 21348, 50708,
                40232, 5804,  46556, 14712, 22760, 43844, 8864,  39416, 46676,
                13696, 18828, 57460, 3848,  26512, 30544, 58136, 18944, 60152,
                45252, 1572,  9372,  33668, 41968, 23940, 37476, 19028, 27888,
                60948, 21780, 49820, 63592, 28652, 16388, 39864, 10276, 26560,
                55972, 13348, 29600, 51580, 38188, 32152, 50504, 20144, 12992,
                36836, 52748, 65016, 1824,  54688, 42968, 12720, 34072, 48668,
                64888, 16072, 51464, 20064, 57484, 2716,  46224, 42696, 30444,
                52772, 23460, 58040, 32252, 50352, 26328, 60024, 7720,  16644,
                47512, 11196, 49692, 2480,  21672, 45292, 59092, 15400, 36580,
                43812, 50596, 4268,  9184,  30176, 15556, 37868, 45780, 64468,
                9888,  49804, 4120,  13608, 34576, 49036, 9504,  56156, 27996,
                39168, 51880, 20256, 60720, 9900,  51160, 32792, 47764, 63640,
                30748, 59560,
            },
            {
                21796, 54552, 61184, 29212, 40568, 15224, 22684, 1108,  38336,
                23892, 6368,  49868, 1884,  62564, 52556, 38948, 28656, 704,
                61448, 25068, 52188, 56176, 1372,  63672, 12476, 53916, 25984,
                1092,  37384, 32432, 60976, 16272, 51980, 6932,  41248, 36544,
                14332, 51472, 17872, 29912, 4760,  53496, 45552, 7468,  31332,
                42860, 1852,  37864, 5888,  44756, 22672, 2128,  53012, 36712,
                62692, 28,    59888, 25640, 6008,  64140, 1520,  44316, 61852,
                17856, 28028, 23760, 46840, 8720,  36056, 25528, 2728,  45352,
                30204, 5512,  32824, 8240,  63476, 34972, 14340, 6776,  17024,
                40828, 4264,  28144, 8068,  41640, 13392, 22268, 39384, 29332,
                63120, 23296, 28156, 60124, 37788, 6912,  24668, 64008, 5564,
                19568, 58488, 23636, 47016, 62304, 21504, 55896, 26756, 19492,
                36100, 59132, 21844, 41780, 25588, 1136,  30400, 19172, 44708,
                6376,  11892, 24528, 30916, 42952, 15828, 59148, 12056, 19584,
                4600,  39916,
            },
            {
                26544, 13708, 50228, 24196, 9596,  63984, 36568, 11568, 44228,
                52216, 59816, 14404, 20924, 35108, 16944, 9768,  55508, 13780,
                36504, 30024, 8028,  41732, 19564, 48804, 32536, 17628, 61972,
                50648, 22200, 47656, 9024,  39316, 23524, 63328, 10368, 28276,
                55040, 25592, 61376, 47180, 59116, 20864, 10744, 64796, 14592,
                51060, 25120, 59948, 14044, 33744, 57044, 61848, 30156, 5648,
                43280, 21676, 39772, 46124, 15820, 22720, 29020, 39640, 33112,
                6540,  10848, 38904, 19688, 59356, 51416, 17012, 40648, 56084,
                13832, 53252, 39612, 44144, 22724, 27192, 50432, 56348, 65420,
                36640, 11200, 61880, 54412, 18804, 64324, 34748, 52592, 44368,
                372,   54892, 15720, 42960, 18636, 52868, 9808,  27900, 49196,
                39908, 31524, 17008, 35928, 41384, 11072, 1600,  33384, 5324,
                15124, 44092, 52452, 8224,  58024, 65140, 40280, 50952, 33644,
                63144, 57008, 46316, 144,   55340, 23140, 6968,  45404, 36144,
                56984, 43876,
            },
            {
                64812, 35284, 42500, 3104,  45504, 53304, 27392, 58896, 4288,
                18680, 31516, 39500, 54992, 42504, 24272, 47352, 63792, 43792,
                17768, 58764, 11440, 34176, 27704, 57320, 36728, 7076,  29992,
                42256, 5120,  27496, 54224, 44224, 468,   33988, 49732, 20424,
                3328,  43020, 7652,  35944, 13580, 27068, 57088, 34700, 39592,
                55492, 9392,  47408, 19508, 41704, 10940, 46436, 17464, 24684,
                48592, 33236, 10628, 57188, 34828, 53484, 9880,  48940, 59244,
                53912, 45584, 57300, 32064, 4044,  28356, 63772, 7324,  23176,
                58976, 27472, 1560,  60684, 12436, 38256, 18900, 32544, 232,
                48580, 20796, 34192, 45212, 2204,  48124, 4860,  11652, 19548,
                37136, 8384,  34444, 57360, 12616, 40548, 46548, 34808, 57048,
                12428, 576,   54964, 7192,  27820, 52068, 59484, 48628, 40616,
                54884, 28328, 32224, 18432, 46144, 15868, 11168, 3468,  22280,
                14868, 26824, 36044, 18100, 64144, 38620, 28908, 53976, 24932,
                9752,  692,
            },
            {
                47204, 7688,  19144, 57508, 33776, 5676,  20584, 48836, 34832,
                64712, 25796, 10316, 48192, 3116,  59220, 7404,  27116, 4616,
                49592, 21740, 45876, 61904, 15160, 3000,  23512, 45420, 10684,
                58660, 15588, 65356, 12544, 35988, 17988, 56152, 45456, 31540,
                64348, 38564, 22812, 50580, 216,   40944, 48732, 2868,  22660,
                17688, 29344, 35420, 52472, 26844, 4380,  32372, 51436, 8468,
                14364, 64400, 19432, 2656,  41508, 60560, 14136, 3700,  24804,
                16364, 436,   42384, 14424, 48620, 11376, 44016, 32716, 19924,
                37376, 9696,  17924, 49040, 54536, 3668,  59644, 9728,  25792,
                43776, 59160, 14616, 24676, 38584, 27764, 31356, 58148, 61964,
                25972, 51236, 47028, 3776,  31300, 61664, 2192,  16400, 22064,
                61348, 26188, 45104, 65424, 13944, 43708, 24084, 17540, 63292,
                12336, 376,   61692, 4784,  36196, 27524, 53624, 60012, 42700,
                48936, 4584,  41064, 9144,  48360, 13640, 2684,  62660, 16896,
                32212, 52896,
            },
            {
                15164, 29560, 38636, 11856, 61632, 16396, 42036, 29772, 13856,
                46012, 7728,  57364, 28760, 13024, 31996, 19860, 41220, 33616,
                53852, 1980,  38676, 6404,  51868, 40276, 60376, 54568, 19456,
                38628, 52348, 31068, 20772, 57848, 4636,  25220, 13056, 6180,
                15984, 56772, 10996, 32448, 63220, 17128, 30184, 5828,  43948,
                62220, 6600,  57664, 528,   65288, 21908, 55676, 40508, 59192,
                37548, 54260, 30848, 50344, 7764,  27344, 46784, 21308, 36320,
                30668, 63616, 26472, 21784, 61484, 37752, 55496, 1144,  49384,
                62400, 46512, 34644, 24500, 30376, 41696, 46704, 22052, 52988,
                29780, 5988,  51556, 9064,  56908, 17132, 50104, 7016,  41436,
                14280, 30068, 21544, 64376, 25256, 19964, 54704, 29392, 42912,
                4756,  52416, 37428, 20352, 32868, 6000,  38028, 30612, 8576,
                46560, 24820, 39100, 49900, 56276, 23208, 32576, 37768, 7960,
                29576, 54108, 61156, 32244, 52532, 20048, 34540, 42148, 49640,
                39308, 21144,
            },
            {
                3576,  60436, 54276, 22892, 31112, 51156, 9012,  56504, 23464,
                52,    41748, 18156, 36352, 62748, 51080, 38140, 15448, 58048,
                23752, 30840, 64984, 26204, 20720, 48140, 13140, 4580,  25568,
                34696, 1496,  7840,  46484, 40536, 28480, 62064, 51804, 41080,
                46752, 27744, 19408, 52904, 44552, 24616, 60456, 54232, 32880,
                50124, 12928, 39164, 16316, 43664, 36812, 12188, 19080, 1264,
                28532, 5368,  44488, 25112, 17620, 37796, 65524, 43336, 57896,
                49704, 8956,  52272, 34604, 5456,  18344, 25116, 52380, 12920,
                29196, 42620, 5072,  64644, 15484, 7440,  35984, 62472, 12320,
                39844, 18380, 32772, 42368, 60836, 12752, 35456, 22600, 45772,
                1716,  56132, 10720, 38952, 6628,  50116, 36600, 8344,  48268,
                32340, 10544, 15940, 49400, 60080, 2112,  57096, 22060, 50744,
                34996, 57528, 20356, 9484,  13656, 44752, 1688,  18844, 64024,
                11740, 16760, 21768, 3592,  26100, 45024, 59956, 4876,  12576,
                27332, 57272,
            },
            {
                34172, 48476, 26008, 988,   46636, 39404, 4060,  62212, 53144,
                38012, 60472, 49284, 5360,  22316, 44864, 904,   61648, 8364,
                12564, 43080, 16672, 10044, 35188, 28876, 42908, 31700, 63604,
                49412, 43632, 23620, 61004, 50004, 10492, 37656, 21324, 58740,
                1428,  35460, 60088, 3964,  8648,  37616, 14756, 10588, 19572,
                26420, 46016, 23724, 30664, 58784, 6956,  50044, 25388, 45304,
                60696, 22068, 11836, 62056, 56020, 31876, 5984,  11660, 18732,
                4572,  38572, 13116, 41556, 56960, 30100, 9196,  40824, 16572,
                3556,  56516, 21612, 11316, 51840, 58140, 27660, 16800, 4400,
                55592, 64488, 1156,  46816, 20696, 3224,  64104, 54652, 37896,
                17784, 60380, 42220, 33808, 15388, 44532, 59988, 13340, 64056,
                39584, 56272, 24388, 7880,  28996, 45716, 53420, 13592, 3304,
                42628, 16280, 29852, 64252, 40892, 6864,  58436, 51560, 25064,
                46708, 34188, 58300, 38208, 10772, 30272, 54788, 23412, 8604,
                62084, 44564,
            },
            {
                17984, 10436, 41652, 14484, 65148, 19672, 26516, 35636, 11020,
                20952, 32356, 14556, 26656, 54728, 10412, 29788, 52344, 46040,
                36240, 56480, 48884, 59820, 53112, 56,    61476, 16404, 9036,
                56752, 12232, 17560, 33752, 3884,  15812, 30732, 7668,  33308,
                24412, 49044, 14320, 40544, 29048, 48144, 57892, 35352, 42060,
                64100, 3120,  54444, 10144, 47032, 28072, 62508, 34556, 16024,
                51672, 41672, 35508, 15148, 47412, 820,   52212, 34408, 28788,
                55388, 23808, 60752, 3080,  47992, 44616, 65152, 33484, 59300,
                26556, 36796, 49940, 32028, 39268, 1644,  44732, 33352, 48868,
                36664, 22736, 27104, 54204, 30232, 40976, 25440, 9608,  32392,
                27312, 47520, 4024,  53204, 58288, 26584, 852,   23096, 18584,
                27740, 3172,  42088, 62420, 35784, 18392, 40396, 64604, 27536,
                59720, 5660,  52156, 33712, 26636, 48724, 15404, 30860, 39996,
                56324, 6076,  43700, 50232, 65412, 800,   18448, 37156, 51184,
                31564, 6148,
            },
            {
                63880, 52120, 36112, 7240,  55488, 32808, 43840, 16596, 47704,
                28604, 51904, 2560,  43028, 65100, 34904, 17040, 25540, 21020,
                5524,  27644, 3488,  39872, 19056, 7856,  38132, 47136, 21924,
                28344, 39236, 59420, 26368, 55100, 64656, 42580, 53576, 17840,
                62396, 10160, 55320, 21444, 65136, 5632,  23224, 1040,  52052,
                8340,  37216, 60680, 18264, 33440, 14196, 2372,  56676, 9348,
                31348, 2984,  53760, 8268,  39464, 23936, 20600, 59608, 40608,
                46280, 17100, 32688, 27872, 20660, 14892, 6300,  22560, 53788,
                45820, 8356,  61032, 18340, 25476, 55012, 20340, 8716,  60780,
                42944, 14864, 10804, 37980, 7332,  51352, 15624, 48548, 5480,
                63172, 13108, 24100, 19132, 9788,  32000, 40604, 53536, 35064,
                58064, 46688, 20896, 51196, 12220, 26020, 9416,  33148, 19312,
                37492, 11264, 45120, 2280,  22044, 60268, 35932, 9864,  20640,
                1196,  28240, 13744, 23744, 16312, 41332, 47156, 58824, 13464,
                42688, 24724,
            },
            {
                38416, 21464, 58332, 28304, 50212, 12104, 2156,  59856, 5936,
                63432, 8892,  57120, 19264, 39296, 6676,  59932, 40660, 49892,
                63928, 31572, 14284, 23252, 33108, 57248, 25024, 54460, 35900,
                2352,  52716, 5908,  45620, 20260, 792,   48520, 12712, 45184,
                3528,  38096, 43676, 31492, 17972, 53400, 44972, 28396, 15964,
                31820, 21400, 40780, 5544,  52984, 42520, 23480, 47840, 39664,
                64872, 19524, 26744, 57656, 30188, 63352, 44132, 9212,  14604,
                1484,  64292, 49368, 10820, 58196, 35880, 50612, 39360, 588,
                19612, 14220, 41812, 5468,  47808, 63624, 13192, 24096, 29464,
                2764,  50036, 57432, 62740, 18876, 33740, 59964, 43148, 21004,
                55708, 35780, 51572, 29448, 65116, 46976, 4540,  49752, 11076,
                6796,  14452, 33560, 0,     59172, 48200, 4920,  44212, 54936,
                48900, 24584, 61768, 55304, 17852, 43188, 3832,  63176, 45476,
                53148, 36232, 61820, 8052,  55480, 33732, 5628,  27948, 20468,
                56424, 1324,
            },
            {
                29696, 45080, 4768,  17376, 23828, 45952, 57284, 38232, 24848,
                40876, 34492, 45632, 23904, 30952, 13124, 53752, 2368,  11096,
                18136, 54740, 41708, 46248, 62772, 11884, 43436, 4496,  14744,
                64436, 42656, 30588, 13928, 35604, 38720, 27780, 22484, 56336,
                29928, 25996, 51208, 1704,  34336, 12200, 39172, 62484, 49268,
                58664, 13052, 27096, 48788, 63724, 20156, 37496, 5088,  28904,
                11500, 49072, 43416, 4428,  17080, 12736, 48480, 27288, 53120,
                36520, 7580,  42560, 53732, 2268,  24912, 61752, 11844, 31256,
                63888, 35452, 57640, 30004, 10100, 33976, 51452, 40168, 58612,
                18048, 31708, 5312,  23556, 44852, 12204, 53708, 4,     29956,
                40256, 8692,  2300,  43292, 37224, 21732, 16524, 61792, 29688,
                38120, 63636, 55356, 30368, 38788, 16720, 57016, 23004, 1244,
                14952, 30904, 8144,  38528, 12228, 29400, 50604, 24312, 12744,
                59736, 17348, 40588, 31104, 48908, 25856, 11184, 63020, 35068,
                49092, 15396,
            },
            {
                60888, 11624, 33396, 63456, 40496, 8736,  29368, 20204, 54432,
                12888, 17260, 50644, 4308,  58704, 46940, 28424, 43784, 34264,
                37832, 59344, 9320,  1460,  51324, 29528, 18092, 49332, 32756,
                19884, 10952, 50764, 23684, 61640, 9444,  58500, 5664,  34840,
                64000, 7500,  13968, 61660, 56680, 8520,  25828, 19872, 3520,
                43392, 55208, 34852, 796,   25236, 8804,  60360, 54696, 17696,
                58600, 22836, 37356, 60972, 51392, 35344, 6348,  32360, 60416,
                22492, 26448, 19220, 38260, 29564, 45960, 17592, 42832, 27548,
                51276, 23056, 2796,  44280, 16348, 37288, 368,   45964, 7472,
                53524, 35360, 47020, 40460, 3036,  27796, 36920, 24384, 14240,
                62160, 46240, 16952, 57368, 6072,  13448, 54664, 42452, 24484,
                45672, 17968, 23264, 43836, 7476,  63060, 31612, 51488, 36088,
                63848, 40996, 20860, 47124, 34648, 57548, 7644,  38868, 32548,
                26576, 4168,  51684, 21120, 2208,  59840, 44364, 39496, 3756,
                8992,  53096,
            },
            {
                26160, 48060, 1988,  53728, 35796, 61788, 15392, 32640, 49800,
                1248,  27980, 60656, 10148, 36928, 16148, 21468, 62636, 24220,
                3952,  47812, 26632, 21432, 36364, 6624,  58788, 27072, 61260,
                40404, 57512, 7152,  46804, 54636, 16384, 41868, 49480, 19544,
                40624, 16972, 46656, 22392, 41244, 47980, 59952, 36972, 6544,
                30180, 10212, 45932, 57708, 15500, 32632, 44396, 13472, 34944,
                45792, 288,   32516, 9848,  25404, 42080, 56940, 18364, 3612,
                44844, 55736, 62976, 13512, 4976,  60176, 8680,  56320, 4204,
                47532, 10828, 54164, 26528, 62236, 56128, 21904, 27572, 65280,
                11720, 25688, 61196, 16052, 55764, 64548, 7036,  49860, 57880,
                19732, 32080, 26532, 49568, 60412, 33412, 27476, 1548,  59320,
                5248,  9764,  52476, 27600, 13536, 21356, 41852, 11016, 28068,
                6356,  58052, 53000, 27340, 276,   64704, 15688, 54732, 19300,
                48336, 9216,  43040, 64340, 37744, 14480, 22848, 54012, 17336,
                32484, 40956,
            },
            {
                6400,  18880, 43260, 13496, 22212, 3136,  47176, 5604,  43672,
                64308, 22404, 42492, 32728, 55396, 744,   51856, 8316,  57352,
                13352, 30228, 53060, 16256, 64760, 44632, 38928, 9844,  672,
                22528, 36704, 28416, 2916,  33340, 25472, 3960,  31140, 59388,
                10780, 53188, 36624, 28148, 3296,  31028, 15100, 54144, 24244,
                64856, 17812, 22504, 38724, 50100, 29472, 4316,  52044, 26268,
                7564,  62464, 15452, 54340, 20080, 2108,  64816, 39180, 49916,
                15656, 9524,  31060, 41516, 34324, 48872, 21136, 37452, 33232,
                16684, 59524, 38228, 20032, 49520, 4188,  32196, 14356, 43316,
                19724, 38780, 8748,  30476, 21320, 47920, 32492, 10400, 38804,
                3600,  54284, 11700, 40676, 23856, 9328,  39328, 51264, 19800,
                31728, 48408, 36052, 1820,  54176, 47092, 3232,  61296, 18780,
                45592, 13432, 4328,  17400, 49348, 23428, 41612, 5084,  44600,
                58524, 34568, 12032, 28772, 18540, 7252,  56816, 29892, 46964,
                64612, 21272,
            },
            {
                36416, 55740, 60256, 31016, 27048, 51812, 59076, 36500, 10464,
                56572, 38460, 7180,  18796, 63184, 26356, 38696, 49124, 33224,
                19280, 62024, 42440, 4844,  33656, 12972, 23884, 51796, 46972,
                55344, 15512, 63908, 18792, 43968, 62820, 52404, 13776, 45760,
                172,   24796, 60760, 6080,  50832, 19252, 10572, 45596, 33660,
                41660, 51340, 3152,  62000, 11616, 41520, 64388, 21828, 39888,
                56652, 28496, 49792, 38660, 46620, 30736, 23400, 11668, 28544,
                35656, 51596, 120,   57852, 23376, 52820, 14512, 62900, 25192,
                7452,  41320, 29568, 13304, 8460,  41916, 59916, 51800, 5412,
                56872, 49496, 928,   52796, 13560, 42024, 18000, 60928, 44568,
                22896, 35552, 63492, 720,   18440, 48104, 64244, 35196, 12780,
                55692, 40968, 65284, 17204, 59668, 33368, 25348, 38112, 49840,
                34084, 24028, 60848, 44036, 31888, 10992, 35996, 28132, 61272,
                1008,  23256, 52884, 57604, 45648, 35592, 50488, 112,   25196,
                12424, 58176,
            },
            {
                29044, 9800,  39008, 49364, 7412,  42092, 20920, 17504, 25696,
                30084, 14760, 52932, 48316, 12300, 30528, 41996, 14996, 5508,
                45888, 39544, 10584, 57988, 48976, 55748, 17004, 30120, 5020,
                43076, 31720, 12500, 49772, 39332, 9924,  21352, 27880, 38360,
                55992, 20540, 32668, 43380, 57328, 63956, 40024, 676,   59308,
                13948, 8200,  36648, 27200, 19088, 54316, 1352,  48156, 10620,
                18684, 42852, 4560,  12176, 59868, 8040,  54896, 43212, 4576,
                61292, 25208, 46732, 18188, 5920,  28196, 40172, 1788,  45712,
                52308, 64836, 1068,  46808, 35832, 24952, 17772, 37828, 28808,
                34088, 23320, 44780, 58220, 35048, 5132,  25732, 51736, 29112,
                15864, 5868,  52148, 45444, 30700, 56520, 7176,  15924, 44388,
                26004, 4208,  11128, 29800, 39780, 7992,  57380, 15572, 9352,
                55288, 29348, 37328, 8528,  56912, 62388, 21016, 53744, 13672,
                31036, 16696, 39956, 3392,  24888, 62272, 10284, 38684, 43120,
                5232,  51028,
            },
            {
                14848, 1220,  24252, 16832, 65328, 12116, 32180, 53880, 61352,
                45916, 2956,  34788, 24748, 44720, 3300,  20648, 65500, 56300,
                27972, 2008,  22580, 31324, 25876, 3072,  61412, 35544, 19664,
                59852, 8088,  53408, 23156, 1116,  36004, 60428, 7084,  34032,
                65528, 8416,  48328, 12484, 16392, 35644, 26652, 52788, 21068,
                28924, 56684, 47552, 59936, 5944,  34560, 24520, 15764, 36560,
                31424, 63840, 22304, 35220, 26920, 16828, 33636, 47836, 58168,
                19296, 39448, 12464, 65340, 43272, 9812,  55976, 31416, 12208,
                18536, 34216, 22248, 58044, 53260, 62624, 10612, 48300, 2704,
                63016, 16932, 10896, 27232, 63544, 37904, 1836,  56740, 8640,
                41232, 58948, 27360, 13540, 37360, 22364, 3548,  29184, 62424,
                21248, 57724, 50400, 23696, 45372, 19764, 52128, 4668,  43244,
                64852, 2436,  19708, 51576, 39396, 1868,  42796, 7052,  50632,
                37688, 63308, 47416, 6276,  33160, 14032, 20976, 58712, 16540,
                33952, 45400,
            },
            {
                54768, 63068, 46672, 35012, 3808,  56436, 38068, 360,   8360,
                41156, 21372, 57964, 5904,  61216, 54660, 9180,  36020, 24460,
                51524, 60724, 37212, 18640, 44012, 8784,  40720, 52964, 11708,
                38136, 26144, 45556, 58560, 28916, 55684, 17452, 44412, 51068,
                15024, 26800, 39636, 1828,  30260, 23472, 7908,  49064, 4256,
                32692, 43904, 23568, 16108, 31240, 45868, 57808, 61492, 7020,
                53156, 2212,  58008, 44540, 51200, 62716, 956,   14376, 22432,
                32056, 7480,  53692, 36636, 33084, 59600, 19628, 50060, 61396,
                27368, 44032, 4748,  14792, 31024, 5960,  21204, 55336, 13804,
                40864, 59472, 32932, 6304,  43696, 22460, 47196, 13128, 65064,
                32908, 48720, 20512, 9776,  43444, 60204, 53492, 40576, 47648,
                8540,  37912, 34236, 14696, 2628,  63496, 27960, 31512, 22524,
                40244, 11524, 47692, 25216, 14504, 30280, 17232, 25884, 45268,
                9376,  21516, 27212, 55084, 41600, 49404, 30820, 53480, 27388,
                61620, 21804,
            },
            {
                31824, 40336, 19272, 52304, 28756, 44872, 23660, 48440, 13760,
                50696, 64660, 28676, 40104, 17596, 49472, 32812, 44108, 11716,
                16788, 6704,  54800, 13644, 63760, 47332, 28048, 21884, 65216,
                1760,  32944, 16424, 6228,  41200, 11416, 48544, 23644, 2520,
                42096, 18648, 51940, 62800, 54976, 46492, 60516, 38016, 18228,
                63384, 12052, 1540,  53316, 38844, 9464,  42972, 27720, 49344,
                40328, 25436, 14204, 6036,  19660, 40512, 36844, 53164, 5432,
                63240, 44444, 26904, 16004, 2720,  47044, 23648, 4280,  38488,
                7956,  56656, 39976, 50428, 27832, 42904, 34884, 45256, 25956,
                8000,  47588, 20644, 51252, 15704, 55216, 30656, 19308, 36244,
                24408, 2448,  55780, 62908, 32288, 17048, 24880, 11880, 32780,
                1708,  18500, 54064, 60076, 42280, 10360, 35844, 46516, 58748,
                16472, 53672, 32428, 5996,  59756, 54780, 48428, 65348, 33532,
                58956, 2164,  12900, 60628, 18020, 7376,  65036, 2096,  9092,
                41280, 4360,
            },
            {
                10948, 26716, 5716,  58776, 9500,  15440, 62604, 27840, 35652,
                19192, 32364, 9640,  13104, 36988, 22856, 58848, 176,   29188,
                47784, 41444, 32712, 50380, 944,   34300, 15112, 5488,  50204,
                42000, 48260, 61696, 20476, 34600, 64256, 31156, 5128,  54604,
                29752, 59496, 33404, 10228, 21316, 4832,  14140, 55936, 9836,
                41000, 35300, 49980, 20672, 65372, 14188, 3236,  21392, 11564,
                17820, 46744, 33004, 55408, 30196, 10880, 24832, 42420, 28764,
                48640, 10528, 56892, 21280, 51100, 29804, 14012, 35000, 54336,
                16712, 24364, 63788, 9884,  19000, 59312, 516,   64936, 31724,
                53652, 38464, 29640, 1344,  61984, 10264, 39884, 50296, 6740,
                45340, 16436, 39080, 5436,  47420, 1076,  36140, 58344, 52388,
                61280, 28548, 46720, 5928,  25720, 49580, 55704, 13020, 1400,
                26624, 36332, 63132, 44840, 22188, 35148, 11604, 3992,  19180,
                41024, 52640, 37000, 29308, 44308, 24056, 35020, 45972, 18800,
                48632, 36732,
            },
            {
                60900, 14008, 49964, 42432, 33692, 21276, 40008, 4520,  60424,
                55444, 1840,  47052, 52572, 27040, 4856,  15460, 53456, 38536,
                62892, 21424, 9748,  59556, 24144, 38956, 58696, 54200, 30592,
                23852, 13452, 55136, 3360,  52300, 25628, 14552, 37644, 61856,
                12124, 6660,  24404, 44740, 36676, 41928, 32024, 28100, 45236,
                24476, 61012, 27484, 7276,  56356, 29932, 51536, 34064, 54960,
                59440, 37708, 8904,  65072, 49872, 3280,  60860, 56148, 17624,
                35112, 1660,  40908, 61164, 6668,  64120, 41400, 44784, 10976,
                49016, 32160, 2184,  46528, 37996, 51876, 15844, 23220, 5236,
                18324, 11688, 57192, 42048, 34668, 25384, 3320,  57636, 61580,
                11596, 53768, 29484, 25888, 51704, 42212, 20728, 6720,  15472,
                43096, 22616, 12668, 32104, 38580, 17944, 21600, 7272,  61704,
                50412, 19148, 9532,  41424, 2708,  27500, 38320, 57700, 24308,
                14596, 31252, 48016, 10052, 4048,  57732, 39224, 12380, 52528,
                24752, 57084,
            },
            {
                44424, 22852, 30684, 63580, 1612,  57464, 53056, 11280, 44176,
                16588, 23540, 42640, 57232, 63588, 41108, 31556, 60484, 19472,
                3092,  26280, 45696, 29952, 6028,  19516, 43328, 10272, 18160,
                35976, 7824,  28508, 38376, 9652,  42712, 57544, 45692, 20000,
                35096, 47744, 58076, 3716,  17424, 50452, 64952, 19976, 6384,
                52968, 3732,  17288, 46860, 41488, 23840, 37120, 63148, 6272,
                29456, 1060,  22860, 43072, 15308, 21224, 46136, 7772,  13200,
                51700, 23344, 31020, 12748, 37596, 25892, 55840, 440,   28172,
                59652, 20124, 35708, 13112, 26080, 7132,  41856, 57756, 37040,
                50456, 61180, 23988, 7204,  46320, 52660, 17136, 27912, 21636,
                41456, 34864, 60388, 8788,  14000, 63644, 30420, 49424, 26780,
                39040, 9448,  64336, 56488, 96,    60664, 41088, 33424, 43640,
                29200, 5052,  57140, 14108, 51948, 62540, 8596,  43832, 50816,
                6764,  56496, 22056, 63652, 15776, 53940, 21036, 30268, 62704,
                15936, 548,
            },
            {
                54216, 8152,  38388, 17344, 25508, 46180, 7188,  31520, 26164,
                38272, 7928,  33992, 3564,  20380, 10928, 45068, 7952,  49024,
                13344, 36556, 56628, 16328, 51680, 62492, 26508, 2944,  56460,
                63156, 44376, 59060, 22472, 47668, 18212, 448,   8024,  50676,
                27708, 14816, 38768, 53436, 26460, 12196, 332,   59136, 48068,
                14664, 37608, 33420, 59588, 12244, 1320,  18292, 45360, 15552,
                41208, 48412, 26892, 60172, 34880, 39136, 27544, 32936, 64508,
                38524, 59280, 45800, 18692, 49240, 9248,  16936, 21920, 62556,
                5460,  42496, 57596, 53396, 61092, 32900, 10644, 29376, 46664,
                3484,  15068, 28476, 19364, 64756, 13316, 37628, 44560, 32200,
                196,   49848, 18552, 22876, 37524, 56056, 10680, 45280, 62168,
                2416,  50600, 35920, 16128, 29536, 52088, 24652, 14636, 54396,
                37884, 23504, 48312, 30980, 17068, 20824, 47036, 32768, 28252,
                62216, 308,   42128, 34056, 26348, 49264, 3140,  42976, 6728,
                27884, 34792,
            },
            {
                18972, 47696, 3784,  55292, 13000, 35376, 19772, 49144, 58644,
                51984, 63044, 29672, 14748, 50616, 24608, 55796, 27932, 33924,
                52328, 64532, 4444,  40176, 11784, 35192, 48860, 32388, 46632,
                40052, 1124,  16704, 51384, 30568, 63268, 33696, 23192, 40532,
                64720, 1920,  21852, 61384, 30332, 56956, 34400, 39476, 22252,
                29252, 55244, 44216, 8780,  31444, 49104, 58012, 25312, 9996,
                61524, 19292, 53836, 11656, 5496,  52488, 58416, 336,   20444,
                26452, 6064,  55112, 2908,  34112, 58656, 52856, 31704, 47188,
                38884, 15188, 29676, 3308,  22144, 49352, 63936, 19892, 55624,
                34804, 44312, 39736, 49504, 31000, 4480,  58920, 9244,  63040,
                15060, 6252,  57204, 46408, 3188,  33540, 17516, 4980,  24164,
                33120, 54508, 20800, 44768, 4284,  47492, 9060,  65196, 3672,
                45860, 11140, 63656, 34488, 40184, 54056, 2000,  12144, 16184,
                36216, 18696, 46284, 7832,  59272, 37848, 10908, 33020, 58552,
                51232, 40484,
            },
            {
                64624, 32620, 60064, 28408, 51344, 39456, 65020, 15232, 644,
                21416, 12076, 48216, 58276, 39004, 35808, 5876,  16620, 42748,
                23292, 9228,  31400, 44320, 21944, 57676, 7088,  14672, 20988,
                11632, 25524, 34396, 5836,  12136, 59724, 26644, 53092, 11044,
                56024, 32008, 42624, 9468,  46128, 6944,  16416, 43460, 9360,
                63964, 2068,  25500, 62664, 21968, 53584, 39564, 5280,  32916,
                50704, 36212, 3132,  31620, 18112, 47872, 13884, 41348, 9944,
                50268, 16120, 42600, 28448, 23060, 40384, 4824,  36868, 11712,
                51756, 25044, 9080,  44432, 17272, 39704, 1412,  13792, 24552,
                8572,  63208, 656,   10816, 56440, 34356, 22972, 48012, 54420,
                26640, 43276, 30724, 65272, 39784, 27052, 52152, 41292, 57812,
                18660, 7896,  27632, 40376, 57580, 12296, 36892, 31568, 17440,
                20996, 58792, 764,   25532, 6620,  61116, 23128, 59208, 55556,
                39524, 25560, 52460, 13544, 29080, 19388, 64132, 47468, 14400,
                21912, 9920,
            },
            {
                26052, 11952, 43424, 22232, 5100,  9396,  42300, 30012, 5612,
                34796, 43720, 26536, 18416, 2464,  65160, 46916, 61052, 1440,
                58640, 18984, 47628, 54228, 556,   28588, 38268, 64496, 53364,
                30044, 60952, 55548, 46304, 41360, 15196, 37100, 4364,  44624,
                16668, 5492,  49244, 19352, 36968, 51484, 25020, 54156, 31904,
                49784, 13428, 18824, 38212, 4136,  14228, 27532, 64564, 43296,
                13456, 22344, 56124, 44608, 64036, 24212, 29120, 54932, 44912,
                34836, 63004, 11444, 60116, 46448, 14380, 63876, 26876, 19096,
                6648,  56108, 63560, 47896, 35332, 54944, 27864, 43504, 52076,
                32052, 58972, 21556, 53432, 25964, 42464, 18144, 2024,  39428,
                20284, 36380, 12632, 53104, 8292,  21228, 60464, 12280, 36696,
                47392, 13808, 60784, 6240,  23036, 61948, 26860, 51004, 55468,
                41336, 28468, 50308, 43976, 13336, 36584, 29216, 42460, 4716,
                49340, 10424, 62924, 4352,  40640, 55404, 1524,  24532, 36576,
                45044, 2712,
            },
            {
                57316, 50356, 16160, 34148, 56988, 60824, 23824, 47160, 54496,
                59576, 40316, 6828,  53272, 9700,  30452, 21664, 11888, 39616,
                29064, 37084, 14120, 25840, 61132, 16864, 42720, 24072, 3740,
                49672, 8608,  38652, 20712, 2304,  54404, 48624, 19712, 29360,
                59504, 35372, 25940, 63824, 14224, 1740,  62296, 19748, 4568,
                35888, 41940, 57024, 51600, 45864, 35036, 20084, 55412, 552,
                29088, 59120, 6616,  40260, 9076,  36916, 2644,  61908, 22284,
                4772,  30132, 37280, 20320, 8048,  50056, 1748,  44056, 60420,
                33724, 40916, 20756, 31616, 5548,  10088, 60184, 37900, 5212,
                17716, 27224, 36816, 15436, 45832, 7604,  63976, 29084, 50728,
                10176, 59424, 24564, 2328,  44252, 15640, 48864, 544,   28584,
                64792, 42632, 31776, 49816, 34320, 19036, 43140, 7488,  2592,
                35380, 15240, 8728,  56444, 19208, 46388, 51548, 8276,  33812,
                20956, 31100, 57388, 22452, 45224, 31972, 16492, 52324, 5500,
                61796, 30508,
            },
            {
                39668, 6856,  37252, 1432,  45700, 18452, 27512, 37076, 10732,
                16652, 23092, 62136, 45912, 33392, 41804, 54060, 26308, 50720,
                56312, 3876,  63632, 7616,  34020, 51952, 10420, 59464, 36336,
                43820, 18360, 28200, 64844, 24428, 31044, 8808,  62808, 39400,
                22776, 52576, 12040, 57236, 29768, 40468, 45188, 58916, 11592,
                26612, 61308, 6876,  28968, 10904, 60876, 48320, 8676,  17876,
                38384, 46756, 26252, 15852, 59960, 19760, 50800, 33152, 16872,
                57228, 48064, 2228,  54032, 32472, 24772, 56836, 29632, 15928,
                48828, 708,   12800, 26724, 61808, 16236, 22604, 46232, 57280,
                12004, 41564, 48184, 5880,  60604, 38408, 13028, 32528, 55956,
                4068,  47200, 17836, 28840, 62380, 35672, 31484, 55308, 9796,
                21916, 3464,  16856, 53972, 1024,  38512, 14276, 46152, 59508,
                24452, 62224, 38352, 31284, 64944, 2940,  26036, 17644, 63900,
                44728, 2240,  15296, 36104, 50956, 7512,  59920, 28412, 42336,
                54596, 18008,
            },
            {
                46908, 20832, 63520, 53688, 31196, 14024, 3100,  63800, 50908,
                32076, 1992,  28436, 12848, 20132, 59044, 15380, 5076,  34772,
                17392, 43648, 49616, 20848, 40632, 46716, 2564,  19968, 31920,
                14344, 5184,  57516, 10932, 45116, 35548, 57976, 13556, 1528,
                46188, 7692,  41608, 3680,  33432, 22388, 8148,  28688, 38560,
                48256, 22568, 17108, 33828, 1892,  23592, 41252, 30940, 52600,
                63424, 10636, 34092, 52924, 30608, 43164, 12248, 7556,  40760,
                25204, 13572, 39452, 65468, 17332, 41788, 35668, 9556,  54348,
                23812, 38116, 57820, 52568, 41920, 49076, 29896, 1916,  33428,
                50516, 61612, 2496,  30788, 20748, 51924, 57856, 16564, 23712,
                41124, 61368, 33292, 50200, 56800, 5584,  40392, 24736, 45432,
                52792, 38840, 59392, 25180, 10604, 64152, 56736, 21816, 30624,
                11696, 47880, 4228,  22324, 53376, 11080, 40816, 59696, 13852,
                54676, 27868, 41980, 61084, 25848, 12164, 38968, 20360, 9232,
                33956, 13284,
            },
            {
                60912, 28956, 24940, 11204, 39084, 48660, 56140, 8044,  19904,
                44468, 38612, 55864, 49984, 36488, 532,   48784, 63380, 9356,
                23776, 32048, 11272, 59016, 29868, 13736, 57156, 27172, 52900,
                63360, 46936, 32884, 51216, 16604, 4008,  41932, 49636, 25432,
                32296, 16080, 60276, 47856, 54392, 17508, 50404, 55860, 15804,
                612,   53556, 44764, 65000, 39196, 55988, 15692, 5464,  44004,
                24636, 20524, 2044,  48036, 5136,  55968, 27252, 46116, 53448,
                61056, 8936,  27836, 51152, 5424,  12404, 61652, 21268, 3272,
                46252, 64716, 8552,  18216, 3768,  36444, 13636, 64360, 19704,
                23928, 8968,  55192, 34956, 25132, 10468, 2900,  35660, 45560,
                5732,  14416, 38044, 9544,  22396, 13244, 18724, 63368, 4796,
                34800, 12760, 29856, 44096, 48400, 28360, 4564,  51492, 40304,
                17652, 54240, 33800, 42880, 16172, 29720, 35728, 47116, 22944,
                38032, 6736,  48960, 18560, 1056,  34680, 47208, 65464, 49944,
                23692, 728,
            },
            {
                51860, 8832,  44068, 58464, 4656,  22468, 41440, 34536, 25372,
                59924, 5704,  15944, 63948, 8212,  24576, 29792, 45100, 40540,
                61408, 55160, 1224,  37772, 24536, 5856,  49316, 38836, 7656,
                23444, 40904, 1792,  37484, 22296, 60816, 27816, 18596, 56644,
                65532, 37968, 20020, 27320, 10440, 37104, 4160,  64184, 34372,
                42484, 9020,  31292, 13684, 50532, 26840, 60108, 36024, 12708,
                49756, 58504, 37660, 62004, 14684, 22740, 64572, 37308, 1100,
                20564, 35216, 43764, 22780, 57672, 45824, 38468, 52320, 31968,
                14644, 28324, 34892, 44808, 24296, 59256, 53796, 7336,  44136,
                38548, 28700, 17176, 65488, 44520, 40352, 48964, 27636, 63524,
                21000, 51848, 26156, 1104,  42732, 47024, 59752, 27656, 16372,
                50332, 56832, 19908, 7140,  36564, 15988, 32836, 8632,  63080,
                520,   27384, 61392, 6928,  49600, 58340, 1152,  6220,  51892,
                11680, 33172, 63168, 9664,  55024, 58520, 14768, 3400,  27284,
                56696, 41552,
            },
            {
                32344, 15784, 35896, 19052, 50792, 27904, 61896, 15236, 53512,
                12132, 47096, 27044, 32504, 42744, 57428, 37652, 19344, 13576,
                5552,  27516, 46120, 18268, 64380, 43364, 33620, 60448, 16960,
                11480, 56096, 26220, 62496, 7752,  52684, 11288, 34508, 4904,
                9820,  51288, 256,   44060, 61912, 24760, 41144, 13156, 20908,
                60688, 25332, 57520, 19780, 4604,  8480,  21124, 53476, 2784,
                32112, 17224, 7916,  25512, 41684, 34672, 3836,  17884, 29924,
                57972, 49432, 15292, 31476, 192,   27076, 18644, 6748,  42260,
                56744, 4336,  20624, 51412, 32140, 10388, 40432, 26352, 56556,
                14528, 46648, 51632, 12360, 5152,  58864, 19216, 53832, 8944,
                30816, 43920, 55552, 63052, 29740, 53328, 36292, 8188,  41960,
                32488, 1588,  63664, 22940, 60180, 54868, 42408, 24716, 36852,
                44580, 20784, 10404, 39124, 25336, 20128, 62620, 31508, 26312,
                57904, 20500, 45336, 29540, 21996, 41028, 31236, 44464, 17280,
                37532, 5560,
            },
            {
                59516, 48048, 1964,  64708, 33252, 7380,  46072, 164,   30352,
                40220, 21704, 2636,  52356, 18056, 11144, 3660,  60248, 52092,
                21956, 35924, 53336, 14964, 55832, 9932,  21928, 160,   45320,
                30648, 48536, 18840, 13920, 43476, 30304, 47164, 55036, 40148,
                21696, 29316, 35336, 13860, 57128, 31816, 46780, 6924,  30432,
                49180, 3408,  36592, 47452, 41536, 33204, 46308, 62736, 28076,
                42252, 54692, 45788, 29700, 57440, 9496,  52228, 47316, 11976,
                42040, 4536,  62336, 54748, 10600, 34620, 63744, 48128, 24704,
                60144, 37396, 12020, 62708, 1300,  47528, 17480, 34708, 4144,
                62784, 824,   36472, 23116, 29596, 32732, 14620, 452,   37508,
                58424, 12388, 7236,  19476, 33496, 11084, 3860,  58232, 23792,
                47680, 10108, 40208, 45732, 3924,  12460, 19504, 49888, 56004,
                14100, 59892, 51692, 45908, 13720, 55560, 9476,  40428, 44112,
                3196,  15576, 39140, 4532,  52508, 25000, 7196,  53676, 10876,
                62532, 21800,
            },
            {
                12996, 26240, 55124, 41924, 12316, 16956, 56616, 36896, 9660,
                65120, 49448, 35328, 60960, 23392, 54576, 47192, 33772, 29032,
                48112, 8376,  41576, 32552, 4780,  50960, 28728, 36452, 62152,
                53876, 5000,  35304, 57396, 39252, 784,   24040, 15480, 44836,
                60564, 48716, 53532, 7464,  22536, 2664,  52008, 58508, 27420,
                55032, 17676, 11536, 63420, 28884, 58364, 16948, 38828, 9892,
                22548, 65308, 240,   12492, 49544, 21072, 38680, 63024, 24516,
                33632, 8128,  19048, 46460, 40332, 58616, 14064, 30056, 9736,
                17672, 49468, 40188, 55732, 29372, 22300, 61836, 51976, 31216,
                21188, 42552, 60032, 8196,  49824, 61568, 42184, 46960, 25308,
                17604, 34612, 39816, 48212, 24396, 15932, 39048, 51192, 18976,
                62280, 14508, 26060, 50904, 34000, 27908, 65352, 2524,  31008,
                6048,  35104, 28828, 2800,  32648, 36720, 48032, 17320, 53824,
                65044, 34496, 50620, 59396, 12844, 36668, 63576, 20224, 34380,
                46424, 29524,
            },
            {
                53076, 6712,  37724, 23440, 30000, 52208, 20900, 43116, 58124,
                17756, 6940,  44676, 14060, 5008,  40788, 26124, 15652, 1500,
                65492, 12340, 59196, 20208, 26116, 40136, 58700, 14688, 19640,
                39708, 9128,  28020, 50384, 20768, 63216, 58804, 6200,  31960,
                2892,  17088, 25932, 41908, 62856, 15904, 19224, 36224, 10092,
                44088, 39900, 23680, 52340, 1228,  13280, 25104, 3744,  51168,
                14932, 34128, 39944, 18848, 60592, 32040, 6332,  15848, 58992,
                53928, 28888, 37468, 25780, 3764,  22036, 50520, 2216,  44524,
                33320, 5892,  26044, 15592, 7976,  43604, 13252, 5672,  48924,
                10664, 25752, 53220, 18024, 39076, 3936,  21688, 55296, 64512,
                5516,  50736, 60288, 2676,  56748, 65200, 45248, 29236, 2904,
                55324, 31116, 37612, 17708, 53488, 9052,  38432, 47232, 17160,
                23772, 41972, 57296, 17880, 64424, 5332,  23912, 29304, 12084,
                22348, 8464,  26616, 47324, 17780, 42740, 116,   48516, 57860,
                3928,  39484,
            },
            {
                43800, 20436, 62968, 9840,  47280, 60808, 5668,  27364, 23816,
                33100, 54292, 28500, 38484, 31608, 63796, 8696,  44408, 56648,
                39188, 24240, 50096, 2116,  62948, 45884, 11648, 3724,  25240,
                44104, 65184, 12856, 4076,  32820, 17276, 37064, 27028, 12820,
                63776, 38316, 10376, 55416, 32788, 45528, 39624, 65108, 828,
                15264, 61372, 31656, 7944,  43324, 55576, 35448, 61236, 45012,
                57144, 7000,  26728, 53380, 3624,  42664, 28096, 45220, 1392,
                21628, 50068, 12556, 64912, 53324, 17144, 36072, 55680, 61496,
                22628, 53908, 65256, 45904, 36552, 58688, 39200, 28212, 57220,
                40840, 14908, 33492, 56316, 27260, 13084, 35180, 9540,  28952,
                44968, 22816, 14780, 27348, 42072, 6884,  21724, 12828, 34988,
                43092, 6500,  59480, 36,    57160, 22516, 43060, 59032, 11388,
                62352, 48512, 8812,  21872, 52836, 43508, 60588, 496,   49560,
                37728, 61108, 2120,  31956, 10016, 60492, 28432, 14384, 25316,
                9312,  17072,
            },
            {
                31784, 49436, 3800,  58020, 34048, 1384,  39028, 50364, 13212,
                2312,  59328, 9968,  19124, 50652, 57720, 21632, 36420, 18408,
                31184, 6424,  43816, 35584, 16468, 31548, 56400, 47956, 52224,
                33652, 21972, 58268, 45016, 54168, 8252,  48292, 52268, 42836,
                56360, 19792, 47424, 28832, 4348,  11960, 23448, 26608, 51376,
                34240, 5260,  47948, 18356, 37880, 21736, 49192, 6164,  19488,
                29848, 36996, 61844, 46052, 23164, 13516, 56408, 35792, 10428,
                40740, 60408, 43824, 5864,  30908, 42396, 7816,  26952, 12392,
                41504, 560,   30300, 20116, 4260,  53232, 23964, 18176, 35804,
                64608, 2672,  47568, 6608,  43084, 62548, 52536, 16360, 40804,
                1936,  32684, 54572, 10120, 30740, 36684, 53224, 60644, 26712,
                46968, 20812, 11948, 41484, 29760, 15256, 5140,  32088, 26596,
                53952, 1240,  37684, 27292, 11104, 39548, 15064, 56976, 32952,
                43000, 55628, 19676, 40356, 54452, 22992, 51036, 32680, 40668,
                55904, 61688,
            },
            {
                27604, 35952, 14656, 18784, 25516, 43980, 15980, 63372, 35764,
                47120, 41100, 24916, 61596, 640,   12604, 48452, 4196,  53868,
                61812, 14124, 28668, 54756, 9436,  22840, 37748, 7364,  30032,
                2648,  15668, 40888, 24656, 29160, 11380, 40372, 21900, 7172,
                34160, 24260, 1048,  60708, 35408, 50092, 59768, 8108,  46204,
                20764, 53716, 58180, 28288, 64620, 10344, 32228, 26400, 42120,
                11944, 48704, 16772, 9956,  33372, 50768, 64696, 25188, 48052,
                18368, 3240,  34376, 15420, 23536, 47148, 58944, 39240, 18904,
                47904, 35632, 10416, 50584, 33168, 11732, 61276, 980,   9048,
                45160, 20488, 30208, 58584, 22748, 3004,  31384, 48492, 60716,
                20036, 37592, 63436, 46368, 18428, 49152, 896,   16536, 9516,
                52160, 64596, 24628, 49136, 60812, 46176, 35872, 52012, 20272,
                39920, 15908, 33556, 63252, 50368, 30768, 7660,  26108, 18120,
                4064,  13380, 29476, 45984, 6652,  35916, 2980,  64352, 45612,
                22016, 1316,
            },
            {
                11848, 55180, 46256, 41148, 53700, 11068, 29296, 55608, 8232,
                20472, 51908, 15628, 45216, 35564, 30244, 42064, 27004, 10204,
                50892, 21236, 46832, 60544, 4172,  42872, 64064, 18632, 60120,
                54860, 36876, 49904, 1456,  59744, 35116, 64540, 3264,  30104,
                58936, 44716, 16112, 51928, 6532,  41016, 14444, 56008, 30160,
                38964, 11676, 25396, 2224,  45240, 14856, 59468, 52520, 876,
                64176, 24060, 4412,  55760, 38356, 2276,  17212, 7612,  58156,
                31860, 26812, 55720, 50852, 61832, 11368, 2596,  31924, 62896,
                15176, 57744, 25296, 60400, 16804, 46824, 41860, 31420, 50408,
                25292, 54212, 11552, 36784, 17324, 44684, 25684, 12068, 57592,
                8336,  51832, 13332, 4084,  25464, 62088, 32760, 58368, 40476,
                4736,  37016, 33060, 8076,  18676, 2236,  62868, 10000, 44828,
                6632,  58768, 46872, 4620,  18624, 45132, 58988, 35756, 47224,
                64124, 24200, 51912, 62052, 16064, 57456, 19460, 12612, 7624,
                37344, 51468,
            },
            {
                23956, 8380,  65176, 30468, 6920,  61644, 22712, 45512, 31856,
                4200,  64536, 27628, 6312,  56172, 23160, 16228, 59040, 32876,
                38244, 808,   40708, 17528, 26440, 51096, 13764, 28112, 41476,
                11240, 7988,  62108, 19728, 46572, 14904, 55116, 18280, 49664,
                13704, 9280,  37220, 64260, 21532, 27712, 18620, 33640, 2640,
                63696, 16456, 41948, 34516, 49992, 5032,  40488, 17404, 33996,
                54864, 43560, 31192, 59176, 20596, 41984, 29764, 52364, 39752,
                13752, 63608, 20704, 8940,  37020, 28092, 54464, 21592, 4776,
                51440, 7664,  44084, 38820, 3112,  27304, 55912, 21404, 62752,
                15524, 40076, 60596, 8236,  49492, 65056, 39568, 6104,  35444,
                28000, 23796, 42804, 57080, 39776, 7592,  44064, 23476, 29812,
                18140, 56712, 14048, 27756, 39312, 50480, 25600, 14468, 56032,
                28724, 22496, 12732, 55016, 24944, 2580,  21480, 53152, 9680,
                38736, 5576,  34752, 10844, 26552, 39132, 43528, 49668, 30284,
                16500, 59828,
            },
            {
                32864, 39304, 1952,  20944, 50500, 37256, 2752,  17396, 58204,
                38104, 12864, 32848, 40288, 49660, 2760,  62844, 45796, 7600,
                24712, 64268, 11448, 33588, 57608, 36300, 6196,  49204, 23656,
                45368, 17112, 27556, 32192, 6156,  23560, 43912, 26456, 61424,
                39948, 53932, 25644, 30944, 43176, 57784, 49224, 5652,  43928,
                23268, 56704, 7804,  60744, 20060, 54436, 29608, 8912,  21892,
                38744, 14112, 8112,  25928, 49896, 12408, 62068, 22240, 1084,
                36360, 46340, 4992,  40296, 44796, 17940, 49572, 33896, 42772,
                23996, 29432, 54760, 20868, 64220, 7064,  13236, 37260, 6112,
                33880, 4128,  27584, 52164, 33208, 92,    20904, 55256, 46868,
                17020, 1308,  49996, 34124, 21128, 15116, 55544, 11824, 50236,
                45808, 1924,  62504, 43616, 55064, 21152, 34416, 4320,  64524,
                36400, 42800, 31812, 61112, 37380, 41076, 62840, 13868, 29012,
                56512, 20384, 44508, 49140, 31796, 4640,  23704, 54708, 62428,
                5060,  42868,
            },
            {
                57776, 17900, 26924, 56996, 33304, 13732, 48288, 27088, 42328,
                23844, 54516, 8872,  60272, 20744, 11208, 52048, 37296, 19384,
                55144, 48696, 29628, 52860, 2624,  19836, 59292, 32600, 484,
                63480, 34704, 56916, 39876, 53116, 10140, 38056, 100,   32596,
                4956,  47892, 18032, 2376,  8328,  12628, 63064, 36016, 10696,
                47636, 51272, 31628, 13368, 37408, 23992, 62776, 48236, 58104,
                2476,  51388, 63344, 36252, 46292, 6296,  34328, 43808, 55224,
                10972, 29208, 53072, 24840, 316,   59636, 13384, 8536,  64568,
                37640, 2356,  13956, 34460, 49160, 30600, 52720, 43520, 58812,
                48488, 19420, 42652, 13152, 24024, 58004, 15364, 29712, 41808,
                63196, 58632, 30516, 10472, 64768, 28492, 37488, 4156,  61608,
                34824, 22256, 51864, 6352,  11572, 59712, 30760, 41376, 17668,
                49176, 388,   8624,  50944, 16512, 6936,  33380, 49812, 856,
                41976, 59788, 15072, 1776,  65408, 58272, 9784,  34464, 27248,
                20532, 47700,
            },
            {
                52628, 10332, 44760, 4464,  40852, 60396, 53368, 10008, 62192,
                1140,  50476, 43780, 17120, 36528, 26056, 31352, 13908, 4040,
                42268, 15768, 6916,  23368, 44000, 47112, 9904,  39044, 53680,
                21040, 50424, 13208, 2852,  47828, 63204, 29420, 57200, 20928,
                11544, 34876, 59584, 55200, 45988, 38036, 24572, 53464, 28836,
                19608, 38348, 352,   27700, 43804, 3712,  11396, 36148, 26548,
                44744, 19752, 29204, 17328, 3040,  60000, 15536, 24988, 48352,
                19160, 58396, 16320, 62964, 30868, 35412, 55752, 26132, 46788,
                53052, 18996, 58212, 9432,  41160, 24184, 18436, 10456, 26768,
                1668,  55104, 62208, 30580, 46416, 37920, 10304, 52336, 4764,
                12852, 38516, 19004, 45704, 6372,  52848, 47344, 19580, 27148,
                9672,  40156, 31392, 16900, 37584, 2844,  46056, 53592, 10868,
                25224, 60140, 20492, 27976, 44072, 11692, 26376, 45956, 17548,
                32440, 27056, 53572, 22168, 40860, 17936, 46692, 14612, 38240,
                628,   13248,
            },
            {
                29516, 35648, 62048, 15724, 24020, 7768,  19412, 35392, 31188,
                15288, 21496, 28624, 47952, 5256,  65192, 44592, 56372, 28376,
                59812, 35240, 62284, 39912, 12976, 65132, 25880, 29548, 14840,
                42532, 5592,  25612, 59236, 21888, 35388, 13952, 51032, 44460,
                64804, 23364, 41132, 14824, 32404, 20828, 1088,  15520, 61604,
                4892,  58652, 17156, 65376, 51888, 56856, 46480, 15564, 5296,
                61200, 32960, 10704, 53564, 39156, 56384, 28404, 8844,  64072,
                5824,  33544, 42572, 9856,  48588, 21192, 6336,  40704, 16428,
                10776, 32500, 45164, 62020, 736,   56428, 35724, 63628, 46044,
                22800, 38324, 16552, 7288,  3604,  63804, 43588, 26540, 33664,
                23108, 53580, 3460,  25968, 35432, 60100, 408,   41568, 54752,
                14624, 59288, 25424, 47516, 64108, 28540, 23420, 7548,  57900,
                39184, 14916, 54584, 35436, 61916, 52444, 57832, 22576, 63752,
                6436,  11244, 37528, 8120,  29320, 52656, 3452,  61096, 51144,
                41524, 63920,
            },
            {
                6208,  21684, 49684, 55376, 46612, 29116, 39264, 4964,  46276,
                56028, 63612, 7264,  33316, 53720, 104,   39660, 8660,  22360,
                46372, 1768,  20176, 54344, 31980, 17044, 56396, 3364,  61208,
                36604, 10736, 31176, 43396, 18240, 7136,  41532, 17168, 8440,
                27652, 52432, 6468,  28696, 61136, 51604, 57216, 40064, 32800,
                46380, 24960, 41620, 9980,  22172, 33288, 18664, 30772, 53828,
                41012, 7284,  48540, 23672, 42816, 20968, 32400, 52020, 40716,
                37444, 22768, 50536, 3880,  14728, 44120, 61976, 1808,  30072,
                60576, 23404, 39232, 27400, 15880, 47088, 5264,  14572, 30052,
                8160,  33468, 50012, 59252, 21388, 35052, 17632, 49236, 60916,
                8132,  47824, 62060, 40692, 16756, 12660, 23736, 32896, 63460,
                7144,  43972, 1168,  20496, 56060, 13588, 49632, 19360, 32984,
                44868, 29708, 47272, 6288,  2292,  18960, 38996, 3580,  36572,
                51316, 60844, 47040, 56216, 34028, 24924, 44292, 21376, 31200,
                8712,  25232,
            },
            {
                43864, 38576, 2568,  32316, 11972, 64840, 57416, 50860, 25056,
                11292, 37388, 41116, 13788, 58000, 24224, 17848, 61556, 33512,
                11088, 51332, 25556, 8348,  36880, 5180,  41408, 47940, 19116,
                55612, 46140, 64672, 51444, 27784, 54764, 2144,  60500, 46876,
                37248, 2924,  19576, 48276, 10504, 5356,  26304, 43136, 8500,
                12908, 54080, 35100, 48888, 6552,  39880, 1364,  64044, 24148,
                12884, 36780, 59444, 180,   65244, 11840, 4460,  46020, 13940,
                2668,  61516, 27680, 56924, 38668, 24080, 52296, 36940, 56304,
                49216, 7112,  3852,  51808, 21760, 31900, 61068, 42164, 51408,
                57132, 12412, 41228, 28036, 53696, 9044,  56244, 1000,  20304,
                37092, 14912, 31732, 56796, 50128, 44356, 29732, 51204, 17916,
                36328, 48928, 53340, 32676, 8928,  41820, 35776, 62480, 1552,
                5244,  65344, 12636, 23880, 42068, 30436, 10288, 54888, 15280,
                24044, 42980, 19164, 13740, 63488, 6084,  11816, 36924, 55616,
                17284, 59232,
            },
            {
                54072, 13844, 19176, 27152, 42792, 444,   22352, 17148, 44296,
                3816,  26748, 59544, 20168, 43100, 30088, 48656, 52804, 15428,
                38716, 58756, 30456, 44284, 61472, 50624, 22912, 8948,  33604,
                24140, 1296,  15740, 8400,  38480, 62524, 30884, 24540, 32996,
                12528, 56292, 62596, 38916, 44340, 35752, 17844, 50316, 21292,
                63272, 2012,  29100, 14740, 61148, 26996, 58256, 42992, 49548,
                20464, 56064, 16136, 26968, 34532, 50340, 18520, 60648, 25620,
                54300, 17268, 31844, 11424, 65052, 33364, 18104, 27156, 13036,
                20520, 42684, 64908, 36312, 55004, 11004, 38552, 2808,  25276,
                20300, 65484, 1336,  47736, 24156, 39472, 13268, 30928, 58928,
                44512, 24964, 10340, 1752,  21396, 7680,  58208, 10920, 4596,
                22952, 28336, 12100, 39340, 4184,  60556, 16464, 51828, 27308,
                37184, 21660, 50548, 58708, 34132, 62780, 48472, 44944, 28536,
                34924, 4484,  30892, 1112,  39552, 50448, 58400, 28120, 2840,
                47944, 33816,
            },
            {
                4292,  63392, 50784, 59024, 7368,  37316, 13680, 33356, 60352,
                53268, 31868, 1736,  50388, 7704,  11728, 36124, 5760,  27196,
                3248,  48028, 17664, 12676, 756,   28312, 59580, 13416, 52244,
                29052, 40528, 57916, 34512, 22608, 4376,  48680, 20028, 58604,
                42168, 16220, 24036, 30640, 13664, 65032, 3048,  59276, 30932,
                37088, 56284, 44260, 19840, 52892, 36992, 11772, 8060,  28644,
                4012,  46592, 31092, 44504, 6264,  38192, 57256, 29256, 36084,
                8584,  41468, 45076, 20268, 6860,  47108, 3572,  9564,  45900,
                58516, 33508, 14248, 8524,  18748, 48824, 28104, 58648, 16680,
                44788, 36764, 5720,  15384, 32912, 45624, 62628, 27236, 41192,
                3920,  54700, 65236, 28348, 39392, 33928, 62852, 46452, 39064,
                56776, 65040, 15896, 58292, 45440, 30216, 24092, 11296, 42876,
                56528, 9336,  39860, 15712, 26848, 7160,  13848, 21032, 64956,
                8732,  49748, 59316, 26764, 45208, 15816, 19828, 42372, 64860,
                10800, 23196,
            },
            {
                30360, 40556, 9412,  34392, 47472, 54236, 62960, 41316, 9028,
                18892, 47524, 15172, 38444, 64204, 21712, 44892, 62792, 56584,
                42100, 23316, 64592, 55276, 40164, 19644, 35256, 44992, 63960,
                5912,  18128, 49080, 11736, 45468, 14440, 39804, 10036, 53632,
                1044,  35228, 49540, 7924,  53020, 22128, 28084, 47056, 15000,
                5900,  23436, 9284,  32540, 3536,  45652, 17616, 51360, 33796,
                38776, 63668, 10060, 22244, 54448, 15128, 42456, 1340,  21080,
                48820, 59432, 2036,  55540, 51000, 29152, 60404, 54824, 40504,
                476,   24776, 28976, 45532, 59800, 23676, 6488,  34080, 53792,
                9712,  30956, 55784, 60192, 19064, 2992,  51696, 7372,  16868,
                47388, 33452, 19396, 42896, 52116, 16016, 2816,  26732, 20068,
                42576, 2220,  34776, 25740, 21300, 53956, 6236,  47724, 59512,
                18304, 31952, 53820, 1872,  43956, 56236, 37844, 136,   57348,
                40688, 17096, 53528, 21448, 62128, 32396, 7972,  35204, 52596,
                15156, 46076,
            },
            {
                17976, 57184, 25816, 21052, 16196, 30008, 24372, 6044,  28320,
                36152, 62096, 23800, 56476, 27856, 54076, 2052,  31984, 19196,
                9592,  35396, 6244,  32988, 49760, 10012, 57804, 26068, 2248,
                37936, 55060, 61992, 25916, 32348, 59096, 51628, 26896, 7068,
                45028, 28488, 60740, 4252,  55948, 33788, 39572, 10708, 52448,
                42556, 60660, 50156, 64320, 25868, 55788, 22668, 62328, 14352,
                57444, 18592, 2736,  49680, 61460, 24660, 10560, 62920, 52812,
                30532, 13356, 24620, 34716, 39868, 14084, 22592, 36220, 30540,
                17360, 62952, 51424, 38896, 2252,  43204, 64128, 13520, 40448,
                48528, 22520, 26872, 50492, 42996, 9992,  35440, 55524, 23328,
                11516, 50912, 5396,  13768, 60672, 24280, 36080, 54088, 13228,
                31220, 52264, 7300,  49500, 61872, 412,   40300, 34412, 14264,
                25824, 3868,  46880, 19880, 61072, 23384, 32084, 51476, 25692,
                12548, 32804, 37932, 10124, 3516,  54940, 48916, 24320, 296,
                27376, 37716,
            },
            {
                6124,  48364, 1360,  60676, 43308, 3772,  51712, 46888, 57104,
                12596, 42776, 4424,  9772,  33852, 16748, 40920, 25288, 13812,
                51716, 45660, 27452, 15332, 22444, 46468, 6760,  17164, 43356,
                31160, 20972, 7456,  41952, 348,   18952, 65396, 36068, 16812,
                62972, 11188, 20196, 41084, 17000, 45124, 264,   57716, 19328,
                26808, 38600, 16820, 12568, 40284, 6952,  35472, 804,   25328,
                42188, 52612, 27672, 40080, 31808, 7552,  46384, 34076, 16544,
                6188,  38276, 63264, 9188,  17104, 43728, 63972, 5068,  52840,
                47908, 10792, 5440,  57040, 16888, 31232, 52112, 21084, 224,
                61920, 7520,  39068, 12772, 64048, 29680, 21264, 38164, 63472,
                30404, 58580, 36644, 45636, 9420,  56168, 6780,  48496, 60128,
                9600,  44928, 18548, 38220, 10912, 32128, 19796, 63988, 7508,
                50176, 62544, 36164, 29272, 8392,  49004, 17732, 6152,  61764,
                44456, 4880,  47356, 23768, 29248, 38772, 12960, 60220, 40924,
                55808, 62528,
            },
            {
                33296, 52844, 12808, 36132, 55188, 10268, 19684, 38496, 948,
                21992, 52456, 30256, 49552, 45540, 5420,  60796, 48224, 57908,
                38088, 59972, 4356,  61944, 54268, 29572, 39432, 62988, 13840,
                51084, 10520, 36248, 53544, 47584, 6172,  29920, 43680, 22140,
                55096, 32644, 47708, 25988, 59628, 8876,  24352, 63728, 31748,
                4676,  34520, 1492,  47928, 30212, 59532, 43360, 48636, 31696,
                11256, 5444,  35628, 13688, 57924, 19936, 37552, 27168, 56500,
                43944, 22004, 47508, 28032, 58072, 1368,  26264, 12156, 20316,
                34120, 42312, 22328, 35224, 26736, 11532, 37172, 46776, 28420,
                57680, 17948, 34912, 1976,  25320, 57884, 46160, 14236, 5964,
                40480, 26288, 204,   22512, 29276, 40672, 32260, 17252, 38604,
                23300, 62632, 29356, 14584, 57196, 42704, 27848, 55052, 44736,
                38888, 23020, 12804, 42540, 55500, 11284, 34596, 41936, 28708,
                19768, 55348, 64148, 15316, 57928, 43896, 18704, 5328,  31152,
                20684, 10112,
            },
            {
                42340, 23032, 28780, 40004, 64772, 32308, 14884, 59776, 34180,
                64096, 17032, 40072, 58408, 20192, 37188, 12420, 7964,  29620,
                400,   20448, 42928, 11996, 36660, 2732,  56784, 33376, 47788,
                24580, 60488, 28364, 16084, 23236, 56468, 12028, 3348,  39248,
                50136, 2088,  37568, 13220, 30248, 51312, 35416, 48172, 12284,
                55716, 45756, 58728, 21772, 53100, 14800, 19436, 8848,  54856,
                61600, 20700, 59908, 43624, 600,   48344, 64584, 11916, 3908,
                50932, 60772, 2952,  33196, 53892, 49696, 37964, 45676, 59052,
                7788,  61988, 14796, 49188, 60880, 8020,  56160, 5016,  14844,
                32524, 44588, 54096, 41448, 6716,  16568, 52740, 3244,  61000,
                49388, 18196, 53964, 62132, 47600, 19604, 64688, 1208,  27688,
                5872,  53620, 35836, 46428, 4908,  50872, 16188, 2928,  10512,
                30320, 58060, 16996, 4648,  63416, 38700, 53216, 14880, 58900,
                2004,  36168, 7744,  51296, 33500, 1796,  62932, 47072, 51088,
                14148, 45356,
            },
            {
                2392,  61756, 15796, 7460,  24928, 45968, 50436, 26908, 44188,
                8424,  25136, 2848,  14040, 26404, 65360, 51072, 22900, 34348,
                16916, 53348, 31344, 24756, 50660, 18480, 9276,  21932, 3828,
                55724, 5744,  45052, 63648, 40196, 34764, 50992, 59864, 14832,
                24544, 57152, 6388,  64648, 43488, 3512,  15228, 21112, 40848,
                8096,  18152, 28232, 9708,  37448, 3124,  63088, 28992, 37960,
                23544, 46564, 50856, 16764, 29264, 5980,  52360, 23272, 40520,
                31872, 18576, 12724, 42136, 20608, 6908,  31084, 16380, 55908,
                28576, 24368, 1284,  54528, 44920, 18464, 42028, 62440, 24672,
                50972, 10516, 21508, 60748, 49184, 32320, 42608, 28008, 34276,
                10648, 43556, 15248, 33136, 4140,  12240, 44196, 50248, 58384,
                41700, 20480, 2136,  26112, 64652, 23516, 59188, 37292, 21008,
                53428, 748,   33456, 47380, 27204, 24048, 3032,  46364, 22040,
                50088, 31040, 25488, 41416, 10656, 22196, 26944, 35088, 7540,
                25036, 57816,
            },
            {
                30512, 19452, 49528, 58484, 2820,  21620, 4800,  11760, 55440,
                31080, 47292, 60936, 53936, 32452, 1536,  41632, 55820, 46800,
                63340, 39692, 7360,  47640, 65012, 40952, 26828, 59880, 42356,
                37804, 19184, 32664, 13264, 8792,  26620, 20248, 31364, 46812,
                9560,  28776, 18508, 52464, 22532, 38796, 60520, 54220, 25924,
                61536, 51744, 43548, 64932, 33188, 25468, 50360, 45260, 15616,
                1844,  33648, 8428,  25896, 55924, 33072, 42692, 15312, 59552,
                26484, 8248,  37268, 65276, 24868, 10540, 62348, 40808, 3200,
                51292, 37456, 31948, 39744, 4368,  30172, 21668, 34204, 39288,
                2652,  64980, 29988, 13420, 36840, 19112, 9172,  55316, 20556,
                24556, 57564, 7356,  39032, 52168, 57124, 25172, 15096, 34660,
                10832, 47500, 55648, 12964, 39940, 8004,  33904, 48952, 26772,
                61300, 40108, 51668, 20408, 59672, 7232,  31664, 64432, 9588,
                39104, 13068, 17888, 46684, 59944, 53184, 15676, 56208, 39608,
                65112, 36468,
            },
            {
                51780, 9620,  34368, 43744, 54044, 37008, 62836, 41300, 18812,
                5620,  39060, 11216, 35940, 44484, 9204,  28948, 14632, 3368,
                27016, 10408, 56920, 16028, 1632,  34024, 12652, 52500, 15540,
                30396, 1436,  50028, 57912, 3748,  62332, 43012, 7420,  61432,
                35588, 41288, 58868, 33688, 11028, 46724, 7060,  29544, 1028,
                36264, 13816, 23580, 4104,  55916, 41832, 11928, 6576,  58244,
                41112, 65400, 12332, 39292, 61876, 19060, 9388,  36344, 1780,
                49280, 57540, 52512, 15084, 46148, 55364, 35348, 21980, 13600,
                44164, 18820, 64736, 12260, 25596, 53296, 48932, 9492,  58800,
                17340, 46912, 5844,  24180, 58376, 592,   64408, 39216, 47448,
                2556,  36208, 63996, 27316, 21048, 9152,  37152, 3252,  30240,
                63600, 16884, 33024, 60832, 30552, 18300, 45492, 12488, 5696,
                15372, 43752, 8724,  13784, 37056, 44272, 18028, 55228, 27488,
                43160, 62576, 56660, 3144,  37424, 6328,  43388, 29592, 11460,
                17556, 4952,
            },
            {
                40652, 46580, 26292, 12368, 30824, 16248, 28368, 57984, 34848,
                49228, 21720, 51728, 17448, 23780, 59304, 19008, 36680, 61252,
                21256, 44924, 35680, 23396, 59048, 43576, 29008, 46820, 7812,
                64272, 54484, 21488, 41544, 36408, 52724, 17428, 696,   54408,
                21524, 45364, 15672, 1356,  26792, 56624, 19212, 42508, 50372,
                32720, 5808,  39492, 47296, 16412, 20012, 53704, 35684, 27464,
                18332, 52428, 30572, 2468,  22380, 45724, 53636, 63784, 29908,
                21092, 44152, 33820, 4804,  29172, 20,    50440, 26912, 59492,
                9288,  48232, 6564,  57796, 16020, 63224, 788,   13648, 43348,
                27356, 35960, 56212, 40416, 45148, 27012, 50724, 12644, 59688,
                30372, 45004, 12984, 49880, 31460, 42548, 61712, 54632, 22176,
                51496, 7700,  43924, 21868, 3204,  52736, 41420, 63320, 55868,
                32176, 24696, 65088, 29756, 56764, 50836, 11796, 35464, 492,
                52580, 5884,  23356, 28472, 32336, 19140, 63500, 884,   48552,
                22872, 54512,
            },
            {
                14292, 60508, 1388,  63516, 39284, 8296,  51960, 384,   14372,
                62276, 29288, 3640,  63552, 6688,  54588, 49512, 42784, 5292,
                52316, 13412, 30484, 6260,  53452, 19680, 4036,  61668, 35492,
                25268, 11560, 47884, 27676, 14976, 23932, 48404, 38304, 27024,
                12168, 4556,  62672, 49108, 37196, 31676, 64224, 9832,  16284,
                62600, 20916, 57688, 29132, 60932, 32,    31588, 62756, 22024,
                3900,  56540, 43016, 49776, 35184, 13300, 4544,  25088, 47440,
                11704, 6792,  17488, 39788, 58912, 19368, 42788, 6492,  33096,
                53140, 23576, 34588, 41740, 28732, 45640, 36748, 31468, 55092,
                20232, 3448,  51652, 10256, 15560, 31560, 7092,  23304, 17056,
                5148,  54648, 18564, 848,   59240, 16076, 6316,  46272, 39684,
                26228, 328,   37712, 49624, 57740, 10676, 27952, 22772, 1676,
                36496, 18180, 48692, 2288,  5308,  23028, 40408, 60736, 20776,
                33032, 16060, 47868, 61064, 41044, 12768, 50664, 34292, 44956,
                59408, 28316,
            },
            {
                37348, 6976,  23484, 18476, 57148, 47464, 20872, 24728, 45680,
                8852,  56636, 25980, 40736, 46924, 30784, 11428, 25092, 33140,
                57752, 40216, 48324, 63684, 38168, 10852, 50976, 22904, 16796,
                44884, 38960, 6392,  60324, 30812, 9792,  56552, 33012, 65084,
                50828, 30340, 24708, 53628, 5376,  13988, 23688, 38448, 55080,
                44080, 48728, 11436, 7872,  36828, 24276, 49172, 10296, 46216,
                37232, 14580, 24480, 6108,  60840, 57468, 41512, 16432, 38184,
                61816, 54476, 26820, 63388, 47988, 12440, 36824, 64216, 17208,
                29664, 61224, 1584,  20044, 51420, 7320,  23216, 61028, 49960,
                8488,  62956, 33460, 22408, 61544, 53780, 37812, 43248, 34100,
                62408, 26668, 41548, 23728, 47228, 35268, 28528, 18916, 13100,
                59612, 55168, 28592, 14476, 35712, 5112,  60236, 38508, 51460,
                46584, 10252, 58116, 41892, 33248, 16356, 47684, 8180,  26464,
                45192, 38452, 10884, 51528, 7228,  21428, 58092, 25200, 9192,
                4088,  20112,
            },
            {
                32036, 55700, 42964, 51076, 33768, 5336,  41888, 59360, 32012,
                37776, 43592, 12692, 34584, 15412, 584,   38980, 64800, 16600,
                9296,  27992, 940,   16992, 26088, 32292, 58120, 40516, 60,
                56044, 32928, 18760, 2196,  63764, 45472, 5408,  13936, 19600,
                43716, 8880,  39716, 17364, 43304, 57712, 47480, 4220,  27760,
                1940,  24956, 32204, 41240, 52648, 13668, 57224, 42636, 7352,
                59692, 29040, 47288, 10640, 18232, 27564, 31276, 51304, 688,
                32816, 42924, 22100, 3664,  31680, 24340, 56840, 3056,  45436,
                11992, 38312, 47376, 56928, 10692, 39676, 18252, 4552,  26032,
                41308, 12792, 28980, 47936, 1640,  19868, 58888, 48768, 10356,
                52480, 37464, 8264,  56472, 11100, 64904, 3720,  50820, 32736,
                10084, 41644, 18420, 65508, 24548, 47876, 19632, 15972, 7676,
                28640, 61708, 21096, 26232, 53320, 63200, 29156, 57304, 13552,
                54328, 65504, 1516,  35024, 26980, 55184, 15356, 38248, 30692,
                42416, 64012,
            },
            {
                16664, 47048, 10700, 29148, 13980, 26016, 61924, 11324, 17352,
                53968, 2400,  19288, 50812, 60728, 22164, 55848, 45840, 3064,
                51008, 60368, 20552, 54668, 45944, 7344,  13376, 49084, 29880,
                9200,  59264, 52172, 42108, 35320, 21220, 40228, 58836, 1900,
                36440, 55504, 59684, 34152, 21240, 7948,  60800, 35060, 18908,
                51532, 59324, 15600, 63076, 45300, 4744,  27160, 17724, 33564,
                51128, 20152, 63324, 32964, 38932, 54904, 8116,  20528, 58388,
                9344,  14080, 37116, 50588, 8064,  15860, 51784, 40252, 21652,
                55176, 4848,  15464, 25456, 32332, 59156, 54416, 14940, 46472,
                37304, 57956, 16968, 43828, 35528, 5756,  13992, 28228, 2792,
                21608, 14736, 50468, 32852, 39176, 21232, 53344, 43180, 24380,
                62904, 4508,  46004, 7912,  31172, 56256, 44168, 33944, 54380,
                13652, 39756, 34980, 6588,  11468, 44648, 2948,  36716, 4960,
                18988, 30380, 22804, 59064, 42936, 46268, 2348,  62228, 49360,
                12580, 52204,
            },
            {
                25800, 35624, 3792,  65224, 38364, 2084,  49740, 35812, 6612,
                22832, 64616, 30440, 57624, 27568, 7424,  35848, 19920, 29976,
                23888, 43088, 34476, 4448,  41840, 61444, 36412, 20692, 65404,
                26184, 14168, 23596, 49652, 11808, 54008, 25584, 29016, 47756,
                15840, 23308, 3376,  11864, 28944, 52032, 40204, 12764, 30920,
                10184, 37816, 6032,  19536, 34592, 22376, 64600, 38900, 3060,
                12740, 40912, 960,   52996, 43964, 4052,  64640, 45496, 24104,
                48816, 28636, 55600, 59648, 44204, 34676, 26364, 9928,  61864,
                27892, 43156, 35828, 65320, 3632,  42444, 28040, 34404, 64156,
                2744,  23380, 6900,  55196, 65168, 25148, 41720, 56340, 63452,
                31388, 45656, 60568, 2148,  27208, 15180, 57568, 8168,  37032,
                48984, 21596, 34232, 52800, 40144, 12148, 2896,  64004, 25164,
                45060, 252,   55456, 49336, 19636, 38824, 24420, 50928, 62040,
                42564, 49060, 8740,  17500, 12036, 31760, 6024,  18348, 23072,
                57036, 156,
            },
            {
                39824, 59184, 22400, 53028, 44192, 19648, 54920, 28792, 46700,
                40100, 49148, 5484,  44816, 10240, 41772, 52880, 13568, 62008,
                38020, 10548, 56980, 14192, 23052, 28128, 53656, 5472,  43172,
                46856, 38040, 6812,  29468, 3544,  17304, 61240, 7868,  52104,
                31840, 62384, 44904, 49008, 64752, 364,   25416, 45092, 63448,
                56948, 42472, 28612, 54828, 1624,  48108, 30524, 53568, 61144,
                25920, 56280, 9292,  23088, 15368, 26180, 12600, 34992, 40732,
                61248, 18720, 2700,  11640, 20400, 62980, 868,   47752, 31928,
                18224, 49968, 9056,  52688, 20432, 13072, 49396, 8668,  21108,
                53168, 31836, 49856, 38832, 9452,  30564, 46892, 18012, 39540,
                4840,  25228, 19156, 7280,  63140, 40744, 30488, 1404,  16824,
                29412, 56592, 14104, 1032,  61332, 22476, 29556, 41296, 4872,
                59808, 17300, 30888, 64732, 14980, 59216, 31672, 9964,  15508,
                33984, 24980, 40048, 52248, 63980, 36392, 54304, 40728, 28192,
                34544, 7528,
            },
        },
        {
            {
                29676, 12364, 36020, 17424, 47756, 34636, 41840, 60064, 8676,
                16364, 34364, 50956, 12568, 46812, 21628, 43684, 59324, 18864,
                2920,  26548, 62932, 16656, 57628, 52324, 10024, 55656, 61900,
                1372,  53516, 22120, 36252, 11956, 54832, 6564,  34436, 50836,
                41748, 18308, 8940,  50336, 24892, 560,   35224, 11032, 21884,
                51672, 39800, 55220, 19756, 12456, 3128,  65188, 38536, 220,
                54576, 49740, 60836, 24832, 10596, 34032, 61840, 18392, 52796,
                30856, 21872, 42476, 2448,  35724, 14208, 27016, 53392, 36940,
                12624, 42764, 34888, 59988, 47000, 36436, 9548,  61924, 38344,
                15212, 51016, 1432,  55952, 12268, 52204, 2676,  40732, 13740,
                54516, 896,   17444, 59100, 39016, 31532, 14968, 57428, 29556,
                36632, 3652,  46040, 21848, 13744, 36012, 26888, 3348,  37936,
                43848, 50820, 58092, 35976, 40716, 27172, 21276, 46544, 5296,
                29272, 60316, 20440, 25772, 64536, 17948, 11940, 2572,  40976,
                7248,  65352,
            },
            {
                39360, 49800, 44088, 6076,  24960, 14772, 57036, 22688, 31640,
                43000, 20000, 61740, 39292, 58260, 28760, 6140,  35292, 12692,
                56316, 37340, 22828, 48572, 28460, 20936, 45208, 39300, 12976,
                27964, 47136, 9492,  25140, 51500, 40312, 20444, 57892, 26008,
                1356,  47608, 33224, 12396, 41444, 54212, 15572, 56876, 6136,
                59784, 31356, 4492,  24388, 57328, 27360, 51360, 30724, 9064,
                34700, 20896, 16012, 43892, 57032, 14180, 54888, 37664, 3200,
                65024, 57936, 28288, 55160, 47732, 31360, 61012, 852,   49372,
                25372, 3876,  18364, 14292, 5432,  27840, 23148, 48868, 20496,
                59540, 26428, 35108, 21152, 32036, 62316, 59252, 48264, 35408,
                20680, 44220, 28824, 64336, 23012, 2628,  45908, 61960, 20324,
                43028, 60592, 10136, 57560, 29864, 7884,  56060, 48076, 65028,
                21316, 5404,  28872, 63404, 6652,  10252, 34172, 65460, 23620,
                40000, 53596, 11160, 42528, 55500, 29788, 50764, 21096, 58288,
                15968, 23984,
            },
            {
                60580, 228,   32716, 59600, 51940, 29532, 4612,  12896, 49512,
                64048, 968,   30164, 23564, 4044,  16448, 52916, 25552, 40580,
                46680, 7668,  31696, 11404, 156,   64272, 6072,  17832, 33600,
                42024, 4528,  62496, 58348, 32460, 2168,  46932, 10328, 31392,
                60628, 37880, 27736, 61568, 6900,  20916, 63456, 28548, 42332,
                47808, 14484, 37900, 44948, 34040, 7256,  41576, 18728, 57724,
                12888, 46736, 4996,  37004, 1080,  28188, 22056, 47200, 9488,
                35184, 13988, 4736,  38588, 11172, 17996, 6124,  43912, 20320,
                57500, 61908, 40224, 30900, 51572, 64348, 43728, 3696,  29764,
                12748, 41940, 46280, 5316,  17152, 37316, 23464, 6216,  29960,
                58120, 9196,  49212, 12320, 42352, 54880, 7856,  50400, 24372,
                13328, 33652, 48876, 38804, 1316,  44856, 18612, 32628, 14812,
                53704, 944,   46888, 15812, 55296, 49528, 44064, 13480, 36844,
                2040,  47688, 33276, 3868,  23180, 6840,  60404, 43216, 35332,
                47380, 9352,
            },
            {
                18908, 55664, 22204, 8908,  38288, 19676, 53904, 40516, 35824,
                7096,  56264, 45092, 10060, 37364, 64664, 49156, 8836,  62104,
                15648, 51956, 60356, 43600, 54332, 36444, 26072, 60040, 50940,
                56508, 19660, 37516, 14664, 6244,  28340, 63972, 17728, 43236,
                14920, 4664,  56996, 17204, 52272, 46448, 38884, 33584, 3148,
                23612, 63988, 10532, 52508, 60616, 14892, 47916, 61824, 24556,
                40392, 64076, 32240, 51300, 59228, 40160, 6544,  42820, 26420,
                50044, 18920, 45740, 24652, 51368, 58540, 40752, 32928, 14988,
                29036, 10144, 45940, 54652, 19848, 11288, 55904, 34536, 7192,
                52704, 65212, 9800,  57284, 50504, 27668, 11024, 42460, 16572,
                61764, 37988, 4040,  33556, 16428, 26712, 34660, 38348, 456,
                52444, 6904,  17356, 25932, 63420, 51520, 23904, 10464, 41536,
                26512, 34792, 39276, 23060, 31492, 3884,  59308, 17340, 55820,
                8436,  27624, 58236, 16312, 49412, 38424, 14560, 26504, 4036,
                54024, 28256,
            },
            {
                45488, 14416, 42004, 26904, 64728, 46112, 1976,  59024, 24400,
                28056, 18172, 14564, 53792, 33608, 42436, 19636, 30988, 22580,
                1904,  34008, 19432, 40368, 13984, 30740, 47008, 3212,  23104,
                11420, 30500, 49564, 41464, 45388, 23456, 35928, 49804, 7600,
                55320, 21824, 44876, 30712, 2096,  26080, 12956, 8128,  50884,
                19516, 35560, 2228,  17820, 28956, 22600, 4812,  36108, 2460,
                52636, 27540, 10976, 23220, 17864, 30540, 63204, 16116, 56396,
                32348, 59496, 7796,  63232, 29712, 21512, 8924,  64232, 52228,
                38416, 2776,  34024, 7584,  25896, 38948, 16244, 58676, 47860,
                18724, 24584, 31036, 39476, 3244,  44928, 63848, 53572, 1816,
                25912, 51872, 22168, 56600, 47708, 62780, 11220, 18064, 28764,
                65524, 55756, 31344, 42100, 14412, 34924, 4708,  61548, 56500,
                7236,  60140, 12480, 62236, 18868, 42148, 24720, 29652, 50620,
                61644, 19656, 44468, 36348, 62352, 9524,  46084, 32876, 64052,
                12704, 37448,
            },
            {
                31000, 61936, 4396,  50408, 11920, 30832, 16556, 44180, 9620,
                51756, 39988, 47936, 60936, 26364, 1296,  12220, 55388, 44128,
                57448, 28068, 4112,  24876, 9068,  58144, 16184, 38548, 8104,
                64868, 26708, 716,   17136, 56128, 60108, 12292, 53272, 39136,
                25036, 65068, 11348, 40804, 36476, 62612, 18508, 60088, 45236,
                57752, 26604, 55552, 48784, 39024, 9476,  54440, 44604, 30096,
                16660, 7412,  60300, 45020, 3416,  54064, 48688, 11276, 1340,
                22504, 36764, 41672, 2092,  15716, 44764, 36560, 4848,  23596,
                47468, 56328, 21880, 63312, 42892, 1868,  32108, 22060, 40708,
                320,   36696, 15888, 60000, 22260, 14012, 34468, 19960, 46988,
                31444, 14444, 40816, 5820,  30232, 2080,  43888, 59652, 40964,
                46324, 22804, 3368,  59068, 20872, 54520, 47052, 37512, 29804,
                19460, 50396, 45516, 9156,  52036, 37572, 260,   11256, 32780,
                41192, 14156, 2880,  26056, 52408, 30728, 832,   57128, 19916,
                6584,  52500,
            },
            {
                2188,  34576, 18100, 57672, 36432, 6308,  61068, 33736, 20668,
                62732, 2652,  31948, 21508, 7576,  46732, 59928, 35400, 5676,
                38668, 65384, 45512, 50448, 62308, 21660, 49168, 32728, 44612,
                53688, 35624, 61076, 21840, 9248,  33364, 3216,  19592, 30132,
                268,   34780, 51328, 8524,  49048, 23312, 53288, 29728, 11084,
                39912, 31928, 6984,  42812, 64788, 32532, 58824, 11660, 21080,
                49416, 55572, 36496, 41256, 13240, 33844, 20412, 38016, 61316,
                51004, 27808, 12216, 53520, 34132, 56148, 49088, 27448, 60196,
                11636, 17088, 27972, 49700, 13156, 52908, 61236, 9360,  45244,
                62656, 53852, 6688,  48184, 55104, 29232, 8352,  38604, 57144,
                10520, 65012, 45420, 18972, 58548, 24656, 50700, 21232, 8516,
                15284, 37120, 11332, 49816, 8232,  27920, 12912, 2624,  43604,
                15912, 25104, 32724, 4524,  27728, 54864, 63836, 45440, 6180,
                22384, 54288, 64388, 11856, 21304, 41012, 16720, 24612, 49756,
                39612, 22972,
            },
            {
                56816, 47668, 8304,  40468, 22348, 48844, 25960, 55632, 12652,
                36600, 5904,  58276, 13564, 38360, 29544, 17160, 51012, 26012,
                14176, 10740, 17708, 35632, 6480,  41788, 1560,  56140, 19092,
                13752, 42248, 6600,  52092, 38772, 48564, 27760, 63048, 42276,
                47472, 16856, 27288, 59036, 14604, 33316, 6520,  42908, 476,
                16188, 62548, 13528, 20528, 1192,  25796, 15428, 41828, 63192,
                33300, 696,   19244, 26144, 65236, 46172, 7968,  25428, 44060,
                5880,  17492, 46728, 60444, 25612, 3732,  13184, 18844, 32144,
                404,   41548, 36056, 58400, 4360,  29448, 37228, 25040, 13764,
                28328, 33708, 11596, 25988, 40928, 1092,  61016, 50260, 4132,
                23492, 34092, 7268,  37112, 53136, 13664, 35552, 5204,  55016,
                33184, 26940, 62480, 39240, 44396, 33304, 64404, 22584, 52360,
                59292, 1668,  39984, 56780, 15472, 33892, 20540, 48472, 60304,
                38572, 29176, 34940, 7632,  46892, 55776, 60932, 35856, 43940,
                62556, 10968,
            },
            {
                42592, 29288, 25056, 63496, 14960, 53108, 588,   39652, 29388,
                44752, 50532, 25244, 42652, 64132, 54700, 3380,  41468, 21076,
                48160, 53036, 31812, 55148, 28648, 12736, 25264, 63392, 29236,
                3908,  47376, 31888, 25848, 14336, 44008, 5888,  56836, 13868,
                60296, 5232,  54928, 20784, 2620,  46652, 56720, 37548, 21808,
                53760, 47192, 24772, 51964, 36400, 45944, 50352, 3524,  23956,
                38444, 47436, 9868,  56736, 4892,  29372, 58904, 52868, 14312,
                30784, 63944, 38552, 9144,  20208, 42728, 57864, 39168, 53196,
                44960, 61624, 7016,  15384, 46400, 19420, 48976, 56896, 5380,
                50744, 20460, 43336, 64032, 18112, 31988, 45704, 15944, 27212,
                41564, 54784, 60504, 28132, 9644,  40216, 64208, 30656, 48000,
                58224, 1780,  51816, 19060, 4980,  16892, 56968, 40624, 9968,
                35472, 48684, 64892, 22272, 43868, 6796,  12516, 26716, 16924,
                1568,  49992, 43680, 58852, 4784,  28788, 13136, 3196,  8616,
                27076, 15916,
            },
            {
                51824, 19888, 54560, 3608,  32692, 43316, 19120, 8452,  65260,
                16272, 54240, 10688, 18776, 48700, 23020, 9788,  63044, 33692,
                7728,  61428, 23156, 3592,  60320, 36756, 51100, 9348,  40148,
                23980, 57816, 10544, 63820, 54844, 17944, 24332, 36064, 9028,
                22664, 31772, 38044, 43696, 29116, 64616, 17440, 26956, 61020,
                34408, 4756,  30348, 59180, 8504,  17640, 28620, 56960, 7880,
                13980, 61232, 30704, 51640, 15776, 22588, 39692, 2324,  35404,
                55064, 21560, 1096,  33204, 51972, 30452, 6436,  64692, 9496,
                24912, 20516, 55176, 33136, 23764, 65020, 8252,  41048, 17564,
                60648, 37860, 4052,  56164, 9312,  52548, 21708, 36508, 63132,
                12388, 17680, 244,   22000, 49324, 16472, 2872,  25312, 18368,
                12756, 42864, 24072, 30388, 60684, 48604, 25296, 6572,  31292,
                20424, 13748, 29136, 9648,  59528, 51512, 41608, 36224, 57432,
                10096, 24048, 18620, 15084, 33464, 39200, 51200, 19136, 47424,
                31560, 59360,
            },
            {
                1388,  12948, 35924, 46480, 10348, 27784, 60124, 48324, 23452,
                3776,  34160, 28156, 1020,  36156, 31756, 15184, 57604, 28412,
                484,   43132, 39792, 14956, 47460, 43784, 18044, 33732, 61168,
                16328, 50372, 19904, 35352, 1824,  59424, 31424, 50816, 40544,
                45556, 52624, 11832, 61344, 8028,  40968, 12796, 3432,  49540,
                10052, 43972, 38484, 12468, 55068, 61576, 33948, 40192, 52256,
                26732, 43920, 21196, 35612, 42724, 49080, 62620, 18452, 10504,
                41540, 57664, 49160, 16060, 62492, 23336, 47388, 14840, 35328,
                49608, 30252, 39884, 10384, 43704, 992,   35452, 31712, 54336,
                1948,  30320, 46296, 23640, 35120, 12840, 59300, 6096,  43468,
                30184, 50900, 46776, 32592, 42704, 61044, 54400, 45532, 38924,
                63268, 6388,  35860, 55524, 14048, 88,    37408, 45668, 62280,
                55612, 5688,  38396, 47160, 17924, 30564, 63276, 5560,  53876,
                31728, 40424, 52676, 61332, 26380, 21624, 54612, 63892, 34328,
                5544,  37840,
            },
            {
                65088, 41584, 7232,  61948, 56320, 38512, 13796, 31708, 56772,
                41376, 46424, 57796, 61888, 7240,  44504, 53188, 38840, 46444,
                18484, 58672, 10172, 27272, 20616, 57872, 6784,  53360, 328,
                37592, 43188, 5312,  28476, 41020, 46524, 12420, 3064,  19764,
                62552, 4180,  25636, 18312, 35772, 48320, 23932, 32012, 55872,
                18780, 63860, 22668, 2484,  41244, 20200, 5236,  11352, 63644,
                17116, 4460,  59380, 1980,  12452, 8380,  32936, 27532, 46528,
                24264, 5120,  28116, 40240, 11244, 2856,  37532, 27820, 17904,
                1800,  6048,  63264, 52344, 27352, 57576, 12336, 22220, 47392,
                26748, 14536, 58452, 6580,  49600, 40500, 26480, 2944,  54052,
                9148,  58360, 37768, 6452,  25832, 11628, 20828, 34456, 9212,
                52808, 22144, 47072, 9768,  41784, 28112, 53340, 10940, 17508,
                42808, 26916, 53448, 940,   35096, 24904, 3120,  21160, 45676,
                13524, 65144, 3292,  47612, 9628,  416,   44580, 11180, 24924,
                50184, 21912,
            },
            {
                45280, 30672, 25728, 16728, 23072, 50568, 2348,  35984, 6744,
                18576, 12140, 21976, 39492, 50128, 26236, 3844,  12432, 23752,
                34824, 51932, 30692, 64380, 2068,  32120, 11908, 22736, 46252,
                30316, 13188, 59936, 52648, 21992, 8428,  65336, 53968, 29304,
                15808, 34292, 49652, 56232, 756,   59684, 53004, 39068, 5800,
                28948, 15192, 51136, 27044, 45256, 31224, 48940, 22900, 36744,
                46292, 32640, 53828, 38712, 24524, 55304, 58364, 3632,  52004,
                64908, 13844, 45348, 34892, 54204, 60072, 43300, 50968, 60980,
                57148, 45428, 13768, 18552, 37964, 49296, 16680, 62052, 42456,
                10600, 63228, 39256, 16796, 28964, 65412, 19220, 48148, 33856,
                24636, 20032, 14020, 63716, 52020, 3928,  29936, 59352, 788,
                28376, 16288, 31956, 62176, 20644, 58636, 34056, 22820, 50772,
                3756,  58448, 23276, 61736, 40672, 14792, 49024, 58560, 27196,
                37668, 7504,  29480, 35788, 42196, 57480, 30336, 17020, 40760,
                56156, 14848,
            },
            {
                9276,  57440, 48896, 33956, 5116,  20276, 44864, 63792, 25892,
                51372, 59872, 31396, 9384,  17112, 55760, 20556, 62560, 5876,
                49092, 16168, 7004,  38204, 54128, 48948, 41904, 27028, 64816,
                55136, 8736,  25256, 47868, 33804, 17208, 37260, 26312, 39340,
                57496, 10148, 41816, 31028, 21916, 15456, 8904,  45764, 60572,
                41976, 35852, 57592, 7768,  62728, 13704, 54100, 20,    58036,
                28512, 7496,  19132, 64400, 29680, 44308, 16564, 36676, 20552,
                31404, 7756,  59112, 18596, 25664, 8168,  21088, 12060, 32484,
                22676, 36544, 25872, 3480,  59812, 29868, 4544,  52168, 7068,
                36032, 20992, 50920, 33408, 524,   43852, 57264, 14908, 38596,
                60756, 1632,  28440, 44660, 35420, 56708, 41884, 14404, 49252,
                43872, 57712, 37200, 2868,  49824, 7472,  15264, 64096, 29928,
                36800, 14240, 32512, 11152, 51948, 7612,  43304, 33260, 17316,
                51208, 20396, 56236, 23260, 14588, 62112, 37540, 6404,  60536,
                2568,  28232,
            },
            {
                18748, 38056, 96,    61072, 52900, 40280, 11524, 54956, 29816,
                15532, 37132, 4464,  43388, 64228, 29596, 35644, 42304, 59676,
                26516, 56584, 45100, 13352, 24416, 35148, 59052, 15680, 2984,
                20500, 36424, 62296, 14964, 1024,  58376, 44032, 6720,  48292,
                1996,  24512, 64440, 6944,  44624, 28260, 34676, 19828, 13324,
                25012, 924,   47272, 33320, 17516, 37808, 26048, 42848, 15480,
                51344, 10860, 41400, 50136, 14376, 5424,  48192, 61564, 11164,
                42976, 38144, 50824, 16,    30788, 46944, 55316, 3828,  40836,
                7520,  53960, 48348, 41780, 8848,  23460, 45344, 33172, 25420,
                56904, 3376,  45080, 55004, 11712, 22412, 7168,  31060, 10676,
                46532, 40844, 55476, 17332, 7912,  23572, 38140, 18820, 65208,
                25692, 5128,  54036, 13424, 24980, 39848, 44380, 1628,  48028,
                8472,  41436, 46188, 18700, 56916, 28952, 62892, 9964,  700,
                60624, 44888, 11644, 49684, 4488,  26000, 52528, 20800, 46376,
                35756, 53092,
            },
            {
                62848, 11800, 42372, 14492, 28708, 58528, 24228, 8536,  42264,
                912,   47036, 54348, 22996, 13548, 2824,  48080, 9696,  31704,
                1492,  40364, 21668, 62816, 4012,  18676, 9588,  38824, 49784,
                32972, 44428, 4732,  39600, 54680, 31256, 23016, 13596, 59968,
                20660, 46064, 14256, 52068, 38752, 62764, 54340, 3636,  49908,
                64864, 10624, 20820, 52472, 4144,  56624, 9656,  65492, 34096,
                21772, 60976, 25748, 1276,  34592, 21920, 40416, 25472, 2008,
                54660, 16200, 22932, 63504, 41752, 13644, 34512, 65488, 29212,
                15580, 62536, 20052, 31260, 54988, 64328, 14904, 39992, 60916,
                17324, 30572, 8200,  24836, 60576, 36828, 53540, 50116, 62344,
                22884, 5704,  51140, 32368, 12620, 61760, 46620, 6824,  32708,
                11396, 40548, 19408, 46312, 63296, 31456, 56440, 19104, 26172,
                60176, 21228, 65452, 6432,  36296, 21720, 38796, 54760, 24248,
                41000, 5452,  31588, 63580, 18564, 33996, 48468, 12980, 8688,
                31720, 24380,
            },
            {
                49596, 21700, 46896, 32044, 7452,  17928, 34864, 48120, 62648,
                20124, 60776, 26532, 34480, 51764, 58712, 38624, 15228, 53332,
                19316, 11076, 32812, 50832, 28228, 46432, 61584, 5856,  56984,
                28432, 11784, 51256, 19108, 27672, 10220, 62020, 51692, 35024,
                30280, 55608, 36416, 19052, 2396,  11424, 25764, 58696, 31352,
                38264, 44460, 29356, 60396, 23948, 40320, 30524, 5624,  48364,
                2900,  37912, 55644, 44896, 62724, 9380,  50668, 29256, 57016,
                33876, 47248, 27888, 5572,  36944, 57928, 17496, 25160, 51896,
                44316, 10416, 34836, 356,   12688, 36344, 20768, 2408,  11224,
                48856, 42120, 63584, 14120, 28252, 40952, 18152, 2272,  27068,
                15668, 36312, 64764, 21396, 49040, 26920, 1488,  54720, 51048,
                22452, 60716, 34432, 27904, 9932,  4388,  37940, 51340, 12164,
                54184, 30964, 1972,  49332, 26864, 3320,  16212, 46692, 12780,
                35164, 27616, 52952, 38412, 43544, 1200,  29072, 58340, 64580,
                43876, 3456,
            },
            {
                5636,  58944, 36376, 25208, 64844, 50328, 2144,  38644, 13288,
                32900, 5600,  39900, 8360,  45508, 18616, 24360, 28060, 64704,
                43664, 36172, 60416, 7980,  42356, 54476, 31128, 23024, 41428,
                17512, 63964, 24132, 57812, 42012, 49440, 5896,  18520, 40724,
                11496, 4684,  27224, 61060, 33044, 48812, 40888, 22340, 17248,
                7280,  55764, 15024, 42284, 11696, 50376, 19452, 45056, 58796,
                16908, 29992, 12472, 32048, 17520, 57828, 36092, 13196, 18772,
                4616,  61088, 10436, 53228, 20868, 8704,  49456, 1436,  37616,
                5308,  59688, 27380, 50848, 43428, 58368, 47552, 52752, 27252,
                55880, 37224, 19724, 52276, 47448, 4148,  34120, 45196, 58780,
                42364, 29896, 2828,  9888,  40084, 59716, 30796, 15836, 36888,
                8260,  43420, 1008,  52620, 59884, 22140, 16196, 34216, 6012,
                43608, 38964, 15332, 56028, 44400, 61512, 51416, 30400, 64532,
                19184, 58916, 8460,  15768, 57132, 10736, 40920, 22880, 16340,
                39464, 55696,
            },
            {
                29620, 17040, 9468,  53880, 4416,  42988, 56200, 22488, 27852,
                49748, 17060, 57636, 11980, 31268, 160,   61600, 7392,  46708,
                3184,  55532, 25144, 16404, 532,   37036, 14320, 10672, 52656,
                1232,  46072, 7800,  32452, 2488,  35708, 45848, 25820, 236,
                62924, 43068, 50384, 8764,  56932, 15664, 5036,  46500, 35988,
                51388, 3252,  25916, 34508, 1428,  61608, 36564, 13340, 24532,
                41888, 52948, 7972,  47368, 23736, 5872,  52828, 64720, 45496,
                24540, 39624, 14660, 44164, 62128, 32768, 42732, 56368, 23128,
                19156, 46964, 55856, 16508, 23976, 4680,  29084, 8312,  34396,
                23056, 1208,  6092,  32632, 10468, 56504, 64184, 8552,  20548,
                11936, 53204, 57736, 44304, 35060, 6016,  19556, 45136, 63768,
                28776, 56376, 17656, 13972, 48228, 41764, 57956, 28380, 63592,
                23600, 9400,  34816, 24564, 12236, 33056, 8844,  4376,  42328,
                49384, 1856,  22280, 46120, 25076, 61920, 53620, 6852,  35264,
                26652, 13760,
            },
            {
                33972, 61580, 45052, 39172, 20168, 11996, 30440, 59296, 9416,
                53320, 43096, 65340, 21260, 54708, 41744, 50248, 33060, 22080,
                14188, 29560, 48996, 58332, 20684, 65096, 26700, 48032, 60084,
                34084, 37992, 13864, 61048, 21100, 15792, 64352, 56628, 33460,
                52980, 16976, 23652, 39560, 21136, 43936, 29724, 63500, 9912,
                58048, 20284, 62592, 47532, 53812, 22428, 6548,  56020, 27576,
                63308, 4516,  20012, 60676, 42888, 27644, 604,   41412, 8640,
                31088, 59452, 51104, 29192, 2668,  26064, 11960, 60568, 30380,
                13832, 40420, 32924, 7484,  39268, 60688, 18208, 41424, 65060,
                15460, 44836, 59900, 40004, 21752, 25660, 14768, 30880, 39048,
                47564, 25340, 18592, 13912, 52200, 23964, 57164, 41340, 13120,
                4444,  49392, 23860, 36160, 31020, 7644,  45968, 1212,  55380,
                48496, 17972, 52236, 59384, 40600, 19524, 57708, 37128, 25900,
                14700, 54456, 29944, 36332, 3916,  31980, 49980, 19800, 2504,
                47432, 52464,
            },
            {
                41476, 22956, 856,   27372, 62508, 33820, 16612, 45792, 2976,
                36556, 24200, 3956,  28772, 37320, 16016, 57272, 10708, 40492,
                52784, 5648,  38864, 11664, 45352, 32524, 40524, 3600,  19832,
                25684, 56344, 29564, 48452, 53396, 39028, 12004, 22208, 7888,
                28812, 47720, 12588, 31828, 1656,  59256, 53248, 13404, 27304,
                42644, 31856, 38736, 9136,  29732, 17700, 46060, 32324, 10064,
                39260, 48680, 34488, 56472, 38528, 14844, 33176, 21320, 49780,
                37180, 3604,  19364, 35324, 48360, 16676, 38748, 6620,  49028,
                62072, 2180,  25600, 63692, 11472, 31164, 53456, 46192, 12332,
                57652, 29608, 50320, 16964, 61676, 43500, 54608, 51020, 128,
                63056, 4816,  32872, 61664, 28484, 2328,  47748, 9096,  26188,
                33236, 61544, 39696, 2876,  64292, 26328, 12696, 20656, 39976,
                4248,  29968, 62972, 7072,  176,   28352, 47536, 22636, 60780,
                10592, 39544, 63900, 48552, 12488, 17164, 37684, 42816, 62512,
                57348, 7564,
            },
            {
                18976, 51260, 58284, 14764, 47568, 52240, 5944,  39540, 63224,
                14548, 32436, 46196, 7892,  48172, 25220, 5320,  35752, 60712,
                17636, 63808, 34180, 24028, 4592,  51592, 8476,  55376, 15872,
                42588, 5112,  10388, 43764, 26840, 3296,  31064, 44532, 37372,
                59448, 5096,  55124, 64820, 36592, 7764,  24740, 34928, 344,
                48612, 16772, 4408,  14056, 58604, 40088, 64144, 51740, 500,
                16192, 22916, 28944, 2364,  10324, 62136, 46596, 58912, 16764,
                11544, 54164, 23788, 7736,  63548, 55404, 21532, 44372, 34472,
                9988,  53764, 45572, 20408, 49716, 35980, 904,   21552, 6440,
                38320, 25176, 9436,  35256, 1956,  28632, 6592,  35712, 22952,
                56788, 37820, 8016,  49292, 39456, 16672, 63084, 36300, 53800,
                20160, 45732, 10892, 55028, 18992, 49912, 36988, 59064, 32608,
                14628, 45492, 21828, 37504, 44024, 55008, 16468, 52024, 6716,
                45060, 33724, 5224,  20452, 56352, 60724, 27516, 9824,  30708,
                24484, 11888,
            },
            {
                37596, 5476,  31860, 10496, 37028, 23892, 60184, 20696, 27024,
                55048, 18896, 51652, 59480, 13592, 63428, 19996, 30188, 44904,
                27012, 9080,  47672, 56860, 18196, 62776, 27988, 35348, 61640,
                49904, 22404, 63700, 17804, 6808,  60012, 54924, 49344, 14436,
                19032, 41208, 25688, 45648, 16004, 50736, 19212, 39948, 55372,
                22984, 65064, 51620, 43804, 24244, 2852,  12300, 35188, 43064,
                59840, 54952, 13164, 51156, 19064, 25736, 53720, 7000,  28828,
                64056, 43792, 58032, 13668, 41108, 29872, 468,   52744, 27592,
                17776, 36832, 57948, 3816,  15152, 57064, 26124, 62628, 32740,
                51856, 2928,  55072, 45728, 13092, 49284, 19372, 42052, 10792,
                17100, 27556, 43232, 21308, 55428, 11248, 31656, 22608, 364,
                58952, 15788, 29536, 5812,  60244, 42176, 9484,  52124, 25260,
                61524, 11008, 50604, 25840, 14028, 64984, 4092,  32844, 29308,
                18488, 59104, 26852, 41116, 8160,  44172, 284,   54920, 15032,
                45876, 65392,
            },
            {
                28304, 48776, 42296, 56436, 2368,  29512, 8776,  42000, 48780,
                640,   10948, 40072, 33900, 2044,  42500, 55660, 3684,  49844,
                22648, 1220,  41984, 30688, 13824, 43212, 21952, 11824, 38944,
                1776,  31512, 52000, 36536, 40588, 20176, 10080, 25336, 1952,
                62276, 34140, 10380, 4116,  28564, 61928, 47336, 10752, 59964,
                6176,  34004, 28044, 56108, 36488, 49552, 26520, 21052, 30408,
                7372,  45252, 36044, 65080, 31336, 43288, 2904,  23480, 39928,
                33836, 1392,  27236, 37488, 46520, 10240, 58892, 15080, 62956,
                4476,  22496, 40924, 28748, 44016, 8744,  39672, 47844, 17328,
                43060, 20048, 63384, 23196, 39496, 57280, 65276, 32336, 46912,
                60540, 53788, 12868, 34760, 6356,  59808, 44200, 51528, 38292,
                7404,  43176, 52864, 35172, 24336, 31308, 16768, 4968,  43036,
                35648, 1784,  57224, 31680, 9192,  35428, 40980, 11804, 62340,
                50264, 1324,  15496, 52596, 24164, 34596, 50484, 21836, 40020,
                33576, 2192,
            },
            {
                60548, 21016, 25396, 63348, 18460, 44796, 55672, 13028, 34560,
                58196, 30128, 62448, 22484, 27452, 52544, 37976, 12360, 58772,
                15464, 54468, 60092, 37012, 2452,  53748, 48616, 6748,  45936,
                58864, 25224, 12972, 57324, 47232, 32700, 65400, 42340, 52384,
                22792, 46900, 57764, 38608, 21652, 42404, 2360,  31176, 15364,
                45132, 20492, 11292, 7744,  19176, 59472, 5124,  53436, 62452,
                18532, 25360, 4016,  40944, 8264,  48892, 36808, 56596, 14108,
                51812, 20792, 49036, 6064,  18524, 33120, 50036, 24040, 42440,
                31372, 52104, 12040, 64880, 33580, 23300, 53868, 13304, 28516,
                10440, 57900, 33896, 30776, 5044,  8668,  25060, 14648, 4216,
                29920, 1460,  45804, 64604, 25648, 3464,  29000, 18160, 12288,
                65528, 26572, 20788, 47560, 13456, 748,   54016, 64840, 20272,
                48016, 17432, 40112, 53668, 20716, 47016, 57616, 24844, 43372,
                21676, 38404, 57004, 46220, 31072, 13556, 63112, 5652,  59164,
                17660, 52008,
            },
            {
                12864, 6996,  15576, 34192, 50124, 4448,  37736, 64976, 24692,
                5760,  43828, 15596, 8972,  45408, 17176, 31564, 8044,  40868,
                34992, 28612, 19616, 10500, 25152, 60964, 33148, 29328, 19504,
                15400, 34720, 8448,  3060,  27892, 15896, 5396,  36028, 30024,
                7164,  16724, 53900, 32096, 13064, 51648, 58640, 26768, 37184,
                53524, 40896, 61368, 47208, 32912, 41908, 14600, 46364, 10644,
                39616, 50500, 58052, 22084, 16072, 61028, 10944, 18604, 46260,
                4624,  30936, 60704, 55740, 25400, 65256, 3644,  38872, 8556,
                46868, 60348, 6908,  16836, 50716, 2464,  61500, 5512,  59636,
                37452, 3864,  14308, 47152, 17892, 51176, 37584, 58884, 52292,
                40188, 23364, 18804, 50924, 15588, 40696, 47172, 57848, 33984,
                49568, 40412, 3116,  57668, 62148, 44956, 38860, 27816, 8196,
                58768, 29032, 6328,  61292, 3168,  27932, 17808, 2248,  54860,
                35868, 6936,  11300, 64960, 3268,  18828, 36860, 47464, 25532,
                8648,  43156,
            },
            {
                36176, 47064, 39896, 53836, 11452, 22744, 31604, 16492, 52968,
                19824, 48024, 36952, 56056, 4372,  64848, 20904, 47744, 24356,
                62060, 5772,  51364, 46540, 40444, 16864, 4128,  55916, 64424,
                40220, 54196, 43448, 62900, 23192, 50808, 55744, 13468, 59888,
                39384, 27436, 520,   63164, 8292,  34800, 18156, 5164,  63396,
                24052, 2884,  30028, 16620, 1068,  64516, 27652, 37508, 57168,
                1576,  33268, 12632, 27216, 52380, 34392, 29580, 58592, 41860,
                62908, 9712,  16324, 42884, 12252, 36448, 20372, 57156, 13532,
                34716, 19648, 25756, 37796, 45972, 30124, 19972, 35312, 45108,
                49856, 26256, 41956, 54092, 61636, 27968, 43964, 20756, 7476,
                33640, 62088, 9568,  37336, 31416, 54732, 8756,  23416, 4776,
                14868, 30852, 9864,  37252, 18416, 33152, 22968, 49576, 13168,
                33672, 23900, 44228, 15372, 38088, 50032, 63436, 8864,  14360,
                48048, 31916, 27468, 22388, 40292, 54684, 28348, 11592, 53560,
                32124, 57068,
            },
            {
                63740, 30376, 424,   58980, 27620, 61468, 46388, 7692,  40800,
                28668, 1636,  61272, 25500, 49484, 34604, 59232, 368,   53124,
                43744, 13544, 31800, 65360, 8236,  50192, 37852, 24120, 10300,
                112,   48236, 30180, 17364, 38048, 45008, 1140,  20964, 47692,
                10868, 50428, 20300, 44036, 25348, 56388, 39468, 46828, 9404,
                13712, 49868, 57208, 38788, 54672, 21988, 8192,  31484, 23960,
                17172, 47992, 63508, 5080,  44092, 472,   20996, 6676,  26668,
                35804, 23632, 39348, 52448, 1108,  29672, 44696, 53240, 27384,
                49100, 876,   54968, 58568, 10392, 41388, 15376, 56184, 23920,
                8344,  64632, 21220, 1152,  10904, 34900, 3000,  48832, 12088,
                54852, 27184, 42412, 58632, 1912,  20892, 63632, 27740, 42292,
                61284, 55824, 24996, 51120, 6940,  11316, 56800, 3044,  54896,
                41524, 63864, 10116, 52404, 22672, 33772, 44912, 30160, 39588,
                61828, 17028, 58200, 49828, 9052,  60132, 44552, 1244,  39372,
                21440, 4244,
            },
            {
                18240, 23624, 43432, 9016,  19948, 39012, 3036,  49640, 59800,
                12304, 54248, 32088, 10568, 40036, 14472, 29496, 11440, 27124,
                18388, 3784,  35796, 22284, 27608, 58588, 13892, 44084, 51808,
                26884, 21124, 60224, 5936,  10516, 58344, 34112, 25540, 42756,
                64100, 32920, 58104, 15536, 48632, 3500,  22156, 52280, 28584,
                42984, 34176, 6284,  25968, 12156, 51348, 43980, 62076, 52872,
                5996,  42708, 29240, 38688, 55636, 62124, 47652, 51056, 12728,
                54872, 2936,  32280, 47592, 59780, 8156,  62012, 16956, 5472,
                63612, 43408, 29156, 4692,  21704, 63476, 51908, 140,   32284,
                18188, 52560, 39868, 29656, 59832, 23628, 15352, 63960, 36848,
                17008, 47212, 6240,  13808, 49180, 35020, 11964, 51604, 36736,
                1236,  16920, 44212, 63412, 28128, 42240, 60912, 16172, 36660,
                5332,  47604, 30792, 56604, 12664, 4352,  58612, 20080, 25368,
                51732, 668,   35192, 42972, 4232,  32672, 17268, 62584, 14712,
                50224, 27340,
            },
            {
                55440, 48336, 13880, 51584, 32392, 55540, 15136, 26588, 22112,
                34896, 43160, 17752, 23268, 6684,  51796, 44484, 36812, 63176,
                39784, 57916, 54252, 44940, 1044,  19844, 34204, 7108,  61692,
                36636, 13348, 33380, 51468, 41092, 28816, 62360, 8320,  17624,
                2804,  36576, 5664,  41156, 30172, 11928, 65232, 32748, 16232,
                60120, 21192, 63008, 18516, 48224, 34564, 3804,  19628, 13688,
                36220, 59500, 20432, 8136,  14420, 24568, 31296, 37568, 17296,
                44852, 64584, 19952, 14244, 26536, 22164, 33616, 40684, 11012,
                23340, 38612, 14328, 33376, 48708, 36456, 7024,  27120, 43908,
                11552, 34308, 6492,  16420, 45664, 56224, 40904, 30420, 25192,
                972,   56716, 21996, 29128, 61132, 44356, 5804,  18696, 46336,
                54200, 22116, 32732, 5264,  48344, 20344, 30548, 45832, 26268,
                21556, 18076, 376,   35664, 26844, 42312, 7700,  54108, 12196,
                5824,  45452, 20400, 14192, 25940, 52160, 23048, 35040, 57580,
                42512, 10524,
            },
            {
                61096, 6288,  35276, 64460, 5048,  44664, 36272, 63140, 9412,
                56640, 3256,  47312, 62192, 58212, 20216, 2512,  55908, 21864,
                7364,  48628, 15696, 10844, 62944, 41496, 56792, 30932, 17960,
                46920, 3516,  56632, 24784, 19472, 14168, 48556, 53104, 31144,
                55560, 13896, 24092, 52852, 61108, 37560, 44968, 7216,  55564,
                620,   37096, 45552, 9840,  59060, 40140, 28896, 56844, 46560,
                9624,  25672, 53980, 34936, 18332, 41572, 10208, 2404,  59932,
                29016, 8432,  50080, 57468, 37944, 4768,  46784, 51036, 30844,
                56732, 18280, 53508, 60744, 25248, 13104, 57012, 39632, 60192,
                47932, 62784, 54856, 36976, 49380, 4192,  9040,  51860, 43668,
                62540, 33212, 39096, 52632, 16376, 25480, 39956, 56928, 29840,
                8336,  12484, 36152, 58516, 14204, 38336, 1168,  9460,  53056,
                62868, 39472, 50432, 60020, 15904, 65328, 48744, 32396, 41328,
                59316, 28544, 64312, 38660, 56784, 10884, 46624, 7060,  29836,
                2496,  37500,
            },
            {
                31748, 16692, 41316, 28508, 24596, 18512, 1396,  40176, 52064,
                30228, 14896, 37728, 27684, 41812, 32604, 16032, 46860, 10108,
                33680, 30276, 24796, 38600, 29096, 49352, 3024,  22708, 53412,
                39620, 9428,  42800, 65008, 2200,  36916, 4972,  22568, 39980,
                45204, 59628, 28676, 9692,  18272, 2388,  26228, 20180, 40728,
                12436, 53544, 27764, 32140, 5448,  15852, 24444, 52,    41160,
                32032, 65408, 48536, 1876,  58684, 45780, 63992, 52932, 22616,
                40772, 34740, 3988,  42260, 10808, 54728, 15396, 59364, 3052,
                35740, 7456,  44528, 2240,  9432,  46600, 17720, 29548, 22004,
                15000, 2196,  23000, 26800, 12676, 32888, 21460, 59128, 5460,
                18760, 11096, 46076, 2616,  8868,  32556, 63092, 4196,  24140,
                60812, 41624, 52604, 3324,  24664, 65032, 50588, 32588, 57540,
                13020, 28296, 8620,  45436, 20920, 38284, 2352,  18204, 36404,
                23076, 9560,  53452, 31148, 5408,  60960, 40756, 18892, 65176,
                24204, 45624,
            },
            {
                20316, 52880, 11232, 59776, 48792, 57380, 12660, 46212, 20480,
                6508,  64264, 50844, 9000,  4700,  54096, 64684, 24116, 41248,
                60584, 1680,  52308, 59244, 6616,  16520, 44472, 11900, 63760,
                28084, 15972, 49480, 30544, 54208, 45560, 57960, 27636, 10956,
                6756,  49356, 20740, 42664, 34360, 56656, 50732, 59304, 30684,
                47872, 23836, 3924,  42144, 64740, 54276, 36640, 61700, 50676,
                16280, 5328,  22940, 11668, 51376, 26224, 32796, 5032,  14996,
                55616, 25424, 18440, 31328, 62680, 28096, 43824, 20092, 25836,
                64544, 39920, 50512, 21004, 32104, 41916, 65324, 3056,  51560,
                9124,  30972, 43184, 57568, 18544, 64928, 38476, 27924, 13920,
                50256, 24192, 59644, 36168, 55224, 21176, 47488, 14580, 34168,
                49312, 17876, 27144, 44508, 55332, 17416, 41144, 23496, 5768,
                43796, 34516, 3796,  53684, 24456, 10580, 29604, 56392, 62760,
                13876, 47252, 2004,  49592, 15688, 36140, 27404, 48512, 13432,
                54556, 4940,
            },
            {
                26396, 38248, 836,   22252, 7516,  34260, 29588, 60556, 25628,
                33764, 43724, 19300, 24824, 35736, 13044, 29708, 3728,  51196,
                18684, 45372, 13372, 21164, 33888, 55252, 37276, 25300, 5016,
                57776, 35124, 6660,  22028, 11608, 16736, 61720, 34856, 19152,
                64772, 37904, 720,   62744, 46916, 5208,  13620, 36100, 9020,
                17464, 61416, 50960, 13472, 22192, 48764, 7272,  12100, 20312,
                27680, 44780, 39888, 30088, 36872, 7368,  19976, 39088, 48664,
                11836, 46048, 59176, 51508, 23080, 232,   34948, 8936,  12884,
                47648, 28888, 16240, 62260, 26796, 54324, 6516,  37928, 33872,
                59272, 46980, 39284, 5780,  51052, 452,   44996, 54688, 35364,
                42396, 30556, 6692,  65120, 27248, 42156, 10756, 58900, 37760,
                124,   63984, 9904,  31248, 7328,  35252, 11476, 59764, 19868,
                48448, 15208, 62268, 32136, 57992, 43144, 51536, 25992, 4588,
                42608, 33444, 19272, 24872, 44028, 21288, 120,   51924, 9632,
                34400, 58328,
            },
            {
                50004, 62472, 45984, 55232, 43032, 15828, 53200, 9884,  49572,
                2284,  58572, 11616, 56120, 45072, 48508, 38420, 58408, 27440,
                36380, 62504, 4860,  43004, 27008, 9364,  60076, 50912, 32552,
                20376, 45852, 61040, 26756, 39188, 32264, 1252,  43240, 51784,
                15232, 26344, 32360, 53700, 16132, 29880, 24736, 64156, 43712,
                2216,  38236, 57160, 34608, 17880, 30528, 43460, 58068, 33488,
                53140, 59288, 14344, 63124, 54820, 16460, 44048, 57408, 27496,
                63472, 1828,  7780,  37420, 16532, 48872, 40340, 60808, 55732,
                22476, 1224,  10824, 57100, 35204, 14280, 48804, 19520, 25048,
                55512, 17168, 11444, 35064, 59944, 24732, 10164, 17080, 3228,
                58148, 20640, 39516, 13056, 17812, 50640, 3468,  53640, 28244,
                20692, 46016, 39396, 22016, 62048, 46968, 28108, 4364,  37244,
                55864, 25528, 40860, 18888, 1536,  14128, 46612, 7908,  17252,
                55044, 38128, 62212, 58976, 9012,  55288, 63188, 30272, 39076,
                43108, 15292,
            },
            {
                28976, 8540,  13228, 31132, 36708, 65124, 4716,  22936, 37300,
                16684, 40572, 31052, 61300, 1088,  17052, 21104, 6992,  11280,
                15508, 31184, 54464, 39216, 64996, 18876, 46228, 1136,  41052,
                13804, 52928, 2576,  42348, 50564, 56540, 7752,  24328, 46948,
                3900,  55808, 11700, 22768, 8164,  41508, 49188, 21244, 54548,
                32788, 7420,  26744, 10076, 45952, 1580,  26336, 38980, 9300,
                2524,  35376, 4576,  21520, 47332, 652,   60488, 9708,  30888,
                36200, 21684, 43504, 13312, 65200, 6320,  53112, 30468, 45000,
                33440, 51980, 37856, 43516, 3896,  23052, 60848, 11884, 41232,
                5156,  27836, 62404, 20588, 29644, 41556, 47876, 32112, 63144,
                8124,  46820, 53432, 768,   44628, 31740, 23608, 40576, 16392,
                6424,  56492, 13428, 49920, 1964,  15448, 51884, 43164, 63220,
                30360, 9224,  51220, 6252,  37624, 64412, 34728, 59788, 30828,
                50244, 3412,  12508, 29444, 41180, 33208, 4436,  17580, 59416,
                22372, 2376,
            },
            {
                33748, 40656, 17528, 25592, 2844,  19572, 41204, 45464, 62164,
                54664, 27136, 5492,  22740, 34196, 53404, 62768, 43836, 49624,
                57216, 22500, 8512,  50420, 2672,  30644, 15240, 23324, 63168,
                8288,  36960, 18020, 9952,  64112, 20620, 13724, 30020, 60420,
                40744, 36016, 58992, 45296, 38160, 61776, 3260,  10880, 28280,
                46756, 19600, 63484, 39740, 55548, 60880, 51700, 23236, 64332,
                18344, 48912, 41348, 10632, 28780, 34080, 24196, 51636, 3832,
                17064, 53712, 32648, 56416, 29172, 24308, 18736, 3264,  14916,
                6932,  58888, 18624, 30640, 8592,  53164, 44488, 29196, 63936,
                45864, 1724,  52944, 49232, 6880,  13336, 56580, 21948, 51540,
                25524, 15880, 28720, 57180, 34736, 62856, 7812,  61080, 30596,
                51072, 33388, 59848, 36256, 25844, 58468, 32932, 18136, 672,
                12424, 21632, 61032, 47224, 28844, 23732, 19924, 40428, 10816,
                21928, 26868, 46156, 6664,  51432, 14608, 47576, 25520, 45216,
                7196,  64448,
            },
            {
                47296, 56752, 52420, 60840, 48188, 57656, 28236, 13292, 31920,
                8100,  48072, 14932, 51128, 41720, 8832,  26148, 32116, 40256,
                296,   46460, 34988, 24988, 12276, 57504, 35492, 48692, 28464,
                55956, 31016, 58324, 25492, 34244, 47784, 38432, 53680, 10200,
                21496, 6168,  28444, 18632, 51248, 31912, 57660, 16372, 40584,
                59672, 14656, 50012, 4072,  21172, 15824, 5520,  13316, 46232,
                31208, 53860, 24552, 62080, 56288, 42804, 13100, 38200, 61596,
                41832, 25708, 48100, 10068, 39644, 46864, 61668, 37216, 27316,
                41640, 63316, 24268, 49628, 59616, 39152, 388,   16148, 35480,
                9480,  22532, 37404, 15392, 33496, 64036, 1908,  36284, 40568,
                11072, 60492, 38036, 5808,  19588, 47144, 13840, 55784, 43856,
                11100, 24156, 5192,  19048, 41528, 54080, 8528,  24296, 39736,
                54916, 44520, 33092, 16480, 53916, 11828, 56880, 5568,  44636,
                63600, 36776, 57964, 18444, 23520, 38696, 56952, 10268, 36712,
                53708, 18932,
            },
            {
                4992,  11536, 37676, 21820, 6528,  10652, 34652, 52580, 152,
                21608, 57512, 38632, 64988, 28904, 13500, 59440, 5056,  19356,
                64320, 28540, 17228, 60392, 44216, 39828, 53948, 10400, 5500,
                43628, 22124, 46300, 15432, 6264,  3084,  27116, 62700, 33484,
                16516, 48580, 64484, 1732,  14096, 7020,  25556, 36656, 52680,
                1264,  24032, 35540, 44264, 28204, 32680, 37000, 41708, 57332,
                27292, 7436,  15548, 37264, 5884,  18016, 49364, 21184, 7548,
                56936, 14440, 5220,  60172, 1608,  34128, 11356, 57552, 20976,
                48036, 12176, 4648,  15268, 26680, 33052, 21056, 56676, 47880,
                31632, 55244, 43120, 58732, 26004, 45448, 18060, 27876, 4536,
                44336, 31600, 22896, 52032, 10352, 26500, 36724, 21408, 1704,
                38340, 65532, 45096, 29080, 3660,  12732, 45736, 64708, 50332,
                35836, 26088, 2752,  7976,  42444, 49580, 292,   32488, 51996,
                15568, 2236,  53328, 34280, 64964, 1452,  28756, 61540, 13148,
                31196, 24008,
            },
            {
                43584, 27244, 2028,  32228, 42288, 50600, 24404, 63880, 17844,
                35792, 44368, 10996, 19732, 2232,  47244, 36264, 24292, 55936,
                37648, 13604, 53180, 6740,  21216, 4312,  25924, 61480, 19336,
                38856, 332,   51708, 60220, 41416, 55444, 18584, 44512, 648,
                42008, 52956, 24956, 34908, 42712, 55280, 20044, 45040, 65520,
                6304,  30976, 58412, 11576, 62532, 48836, 54996, 10188, 20120,
                1896,  60112, 45172, 31976, 52088, 2952,  63668, 28400, 33188,
                45272, 19716, 51324, 27860, 22324, 16960, 43264, 51144, 8308,
                54984, 34828, 39844, 46216, 64912, 11364, 51168, 5828,  25324,
                60200, 18756, 11728, 2924,  38896, 8800,  48592, 53080, 61844,
                14488, 47800, 1588,  64680, 42536, 59548, 4328,  49644, 27788,
                17628, 52804, 8944,  48584, 61356, 37208, 31188, 20744, 6144,
                14888, 60264, 56216, 38576, 63304, 26928, 36468, 60400, 24620,
                29612, 41936, 13384, 9160,  43692, 20248, 49400, 5792,  42028,
                50852, 59620,
            },
            {
                35104, 63616, 16744, 45680, 62084, 14332, 39180, 3716,  47012,
                59188, 25724, 4320,  33504, 56740, 42944, 16456, 51276, 10660,
                3672,  48756, 41176, 32364, 63756, 47572, 16068, 33604, 50048,
                13600, 65516, 10788, 35748, 29132, 49428, 11932, 23668, 59168,
                8712,  30808, 57124, 11092, 47712, 61200, 28920, 9208,  33776,
                13332, 51624, 42368, 8064,  18400, 860,   24800, 63448, 34352,
                50300, 39312, 12264, 21400, 58192, 25408, 46284, 10692, 39220,
                1040,  64452, 36588, 41300, 55400, 62860, 4160,  25092, 31628,
                680,   28420, 57988, 20132, 2612,  37284, 42544, 61768, 13836,
                40304, 4452,  30288, 51448, 23208, 59836, 31280, 20860, 7112,
                57472, 34288, 18472, 39752, 30260, 15988, 54796, 33592, 62988,
                41772, 14348, 32644, 56268, 22644, 16536, 57904, 2076,  28700,
                48104, 11148, 19476, 30928, 22172, 13268, 17644, 47772, 8396,
                19816, 61684, 49004, 25676, 31476, 55156, 15644, 34732, 21516,
                596,   15060,
            },
            {
                8424,  48884, 58008, 29316, 7324,  18856, 55924, 30604, 8996,
                40764, 53656, 29844, 61464, 49240, 22148, 7332,  62272, 33248,
                44620, 26368, 58576, 9860,  29060, 36732, 1884,  42040, 58860,
                27700, 32712, 24184, 17572, 8052,  63340, 32068, 36528, 51484,
                14268, 39364, 19240, 4600,  22552, 38972, 2556,  49472, 18252,
                39576, 21796, 26592, 54176, 38328, 45800, 30396, 6896,  43816,
                17384, 29364, 65240, 9152,  35044, 41124, 14316, 54944, 59892,
                23428, 31544, 12272, 7052,  30668, 48636, 14504, 38892, 64496,
                45376, 17128, 52412, 9260,  30532, 54344, 17744, 28500, 8404,
                49988, 36112, 65244, 44192, 16108, 55460, 12228, 42344, 37892,
                24776, 54220, 11708, 49936, 7896,  22196, 44800, 12012, 6872,
                58380, 20944, 732,   26540, 40180, 9976,  52220, 42700, 62156,
                33720, 40768, 51616, 4260,  59236, 45956, 5976,  41100, 56044,
                33568, 5152,  57484, 39436, 3280,  59696, 46436, 26872, 63216,
                39812, 54432,
            },
            {
                19884, 38500, 12148, 23232, 53028, 36120, 26212, 60208, 12752,
                22720, 15560, 6912,  38060, 11776, 27192, 40432, 30100, 53636,
                15488, 22692, 1216,  19576, 55716, 12512, 52700, 22872, 7560,
                55180, 3272,  44640, 40052, 57260, 4108,  21344, 46788, 6060,
                61516, 27848, 45788, 63728, 32248, 15200, 53292, 24816, 57876,
                62228, 46332, 4712,  15692, 60344, 21292, 52772, 14668, 59000,
                23100, 4852,  54592, 48484, 300,   29856, 19340, 49888, 4136,
                16284, 47228, 58456, 18708, 44500, 9340,  35388, 19840, 59320,
                13012, 23092, 41320, 62032, 25008, 47260, 3668,  33928, 45548,
                23812, 54532, 20368, 28032, 6068,  35016, 380,   64252, 16780,
                29420, 2912,  62524, 26856, 56660, 2596,  37756, 25228, 47900,
                29996, 36204, 44268, 62388, 5844,  34520, 47108, 24756, 7464,
                17156, 23164, 44724, 9368,  35004, 52452, 28628, 65280, 11516,
                37920, 22856, 45880, 17544, 12116, 36928, 7596,  52376, 11064,
                30256, 24828,
            },
            {
                3436,  32612, 55116, 1204,  41064, 48280, 5892,  43044, 34220,
                51460, 63156, 45076, 18732, 55284, 608,   59172, 19464, 5564,
                64040, 35692, 50720, 45140, 38704, 62332, 18260, 30752, 45592,
                37980, 20096, 53064, 48220, 26704, 15020, 53928, 41240, 17320,
                24436, 2268,  54716, 37076, 8116,  59824, 42060, 11640, 30240,
                192,   36208, 56428, 28968, 34644, 3312,  40440, 11200, 55892,
                36684, 41876, 26276, 16716, 44232, 63012, 37356, 8220,  33964,
                42836, 27156, 54172, 1944,  25996, 61052, 52172, 27828, 6364,
                49548, 32976, 5356,  35660, 14452, 59708, 21896, 56100, 62836,
                15624, 1148,  10140, 47196, 61184, 40808, 26580, 50480, 46848,
                9716,  43636, 20388, 34944, 46648, 31524, 62144, 51612, 19076,
                3920,  53268, 11312, 49560, 55236, 19804, 13856, 38216, 54328,
                58680, 600,   63752, 25544, 15608, 39584, 1364,  23916, 54140,
                15248, 2140,  51564, 28664, 64084, 24088, 19012, 43228, 4392,
                57608, 45836,
            },
            {
                64316, 26384, 44240, 61756, 10256, 21272, 65512, 3248,  17596,
                47632, 1712,  25200, 31892, 50904, 14592, 46836, 37016, 43360,
                11892, 28560, 59876, 7704,  25148, 5200,  49012, 60660, 10340,
                15416, 63656, 6132,  11820, 60476, 37604, 1864,  29048, 57924,
                33696, 42672, 12284, 50120, 17772, 26616, 35464, 6028,  43916,
                16872, 48828, 8772,  12772, 64896, 50160, 25920, 32244, 47660,
                1768,  60620, 13660, 52608, 22160, 5588,  58816, 24572, 51900,
                62444, 10248, 38040, 50272, 22748, 40596, 3488,  54764, 42676,
                10316, 56296, 44784, 2032,  50868, 38904, 7380,  12112, 37748,
                30968, 42624, 57852, 32752, 13584, 53280, 22100, 4060,  33072,
                58580, 52072, 38832, 13800, 59080, 17372, 9424,  40940, 13976,
                60292, 39124, 23244, 16008, 29160, 2592,  63356, 31044, 11508,
                27880, 36440, 48768, 55032, 31300, 57740, 20380, 49764, 31932,
                42768, 62528, 34904, 6348,  41776, 49964, 31244, 61304, 34208,
                17244, 49812,
            },
            {
                6488,  14508, 18528, 34500, 28784, 15600, 51928, 31548, 27268,
                58616, 36492, 10228, 42080, 64792, 34324, 9108,  24712, 56576,
                2428,  40136, 17280, 33028, 14184, 42860, 34448, 2588,  27112,
                56452, 35152, 29860, 22780, 33324, 43928, 64648, 11184, 47828,
                7396,  62420, 21432, 30060, 2908,  48088, 55620, 64500, 21020,
                52456, 24136, 32132, 41724, 19180, 44744, 5752,  63128, 18096,
                28384, 8280,  33700, 38992, 31040, 56104, 12940, 28640, 40456,
                20664, 5260,  14776, 65428, 34044, 11912, 17224, 32164, 24348,
                37952, 16144, 63804, 27232, 18960, 31672, 43084, 48544, 26488,
                5728,  51108, 18356, 24624, 37416, 8664,  18880, 62752, 37048,
                14816, 23320, 7172,  3708,  42400, 24128, 180,   55980, 34532,
                27704, 7872,  64640, 33536, 42492, 57780, 45332, 21784, 50276,
                5284,  43556, 19992, 12776, 3944,  62376, 10204, 45120, 7180,
                26136, 19096, 58804, 9900,  54528, 14276, 1524,  55876, 22448,
                12600, 41344,
            },
            {
                36672, 52056, 59248, 6968,  39060, 57236, 44596, 11464, 54936,
                39420, 20252, 4900,  56356, 22312, 3588,  28908, 60636, 48384,
                21504, 52616, 46320, 55272, 65000, 21008, 51920, 40640, 23992,
                47848, 43024, 708,   51568, 18132, 8684,  24916, 19720, 51668,
                36280, 15940, 52936, 58784, 44252, 23116, 13720, 33036, 9984,
                38560, 62596, 58220, 2056,  54444, 22652, 9552,  35800, 51968,
                21108, 46144, 62384, 3512,  49084, 9744,  45652, 16980, 1444,
                55420, 32852, 29348, 45460, 6812,  57052, 47716, 63432, 1448,
                58736, 30048, 22048, 53224, 9664,  57696, 64828, 2600,  20608,
                59568, 40040, 63416, 2888,  48920, 56568, 44916, 28248, 5428,
                54912, 31112, 65292, 49260, 53236, 28980, 63524, 47304, 22104,
                44988, 56832, 1484,  48352, 25720, 9328,  4520,  40352, 60728,
                16036, 32996, 7988,  37960, 42376, 26752, 17200, 36964, 59912,
                13048, 47816, 30248, 22184, 38176, 27176, 47540, 39676, 8724,
                28868, 60256,
            },
            {
                488,   30916, 46416, 22236, 50148, 2292,  35776, 23780, 7528,
                14520, 61788, 30168, 45844, 16388, 39104, 53840, 17748, 13212,
                31192, 6324,  10304, 27276, 172,   29916, 12128, 59184, 16816,
                8296,  61892, 13732, 39832, 58856, 48660, 55996, 30328, 40184,
                316,   27020, 9880,  37888, 5248,  40076, 60696, 1596,  47132,
                27552, 4876,  14800, 36880, 28140, 59896, 39448, 15276, 58300,
                43088, 11796, 54736, 25120, 19580, 41924, 64492, 35488, 48560,
                61128, 43272, 18228, 59576, 21692, 37464, 28372, 43900, 20196,
                12580, 48268, 7032,  39688, 46408, 13656, 28928, 16784, 53924,
                34344, 12464, 28800, 7264,  42900, 30648, 59492, 11680, 41440,
                17992, 46136, 26300, 12136, 19496, 33228, 37384, 11168, 4820,
                17036, 32016, 20088, 12368, 36780, 53116, 18704, 34832, 25044,
                52340, 57624, 64824, 22812, 51440, 47340, 33336, 55516, 3016,
                39996, 53352, 116,   45044, 57840, 5144,  16704, 65252, 35212,
                53624, 24152,
            },
            {
                43368, 9852,  56068, 12988, 25876, 60700, 18940, 63372, 49176,
                42824, 52312, 25572, 11972, 49760, 62364, 7480,  42392, 35340,
                63120, 38568, 23728, 60816, 48588, 37800, 54352, 4308,  32688,
                37296, 21348, 54540, 27220, 5456,  36040, 3132,  13960, 60520,
                22524, 46476, 65272, 32332, 18716, 51336, 28972, 16336, 54260,
                42432, 18860, 45868, 51552, 11540, 47780, 31552, 900,   24520,
                4556,  30244, 36948, 15584, 59648, 2424,  31984, 23108, 7860,
                25304, 12160, 53024, 56,    49148, 9540,  15172, 5348,  51960,
                35732, 41740, 59976, 796,   33624, 22992, 36836, 50316, 8644,
                45284, 22656, 55448, 17260, 35080, 14544, 952,   24448, 49116,
                60692, 8784,  35528, 40104, 61244, 6332,  15700, 49940, 54020,
                39424, 61904, 50672, 41644, 58932, 14980, 64176, 47956, 1312,
                10192, 45920, 28200, 2204,  14196, 61204, 5488,  29312, 23140,
                64672, 33884, 17816, 11480, 61188, 32616, 20968, 44592, 3552,
                48656, 19700,
            },
            {
                62912, 27688, 40396, 3892,  32576, 42108, 9572,  29260, 272,
                33384, 6036,  59088, 35864, 1376,  32152, 26524, 51232, 4120,
                58044, 44792, 15360, 34580, 18412, 9172,  43528, 19784, 64044,
                50200, 3472,  45776, 31684, 16256, 63732, 45308, 33540, 53156,
                42268, 4564,  55576, 12936, 25352, 7352,  57188, 35372, 22592,
                61312, 8420,  30816, 24632, 5352,  17452, 62312, 55712, 49432,
                40380, 65072, 7284,  50872, 28220, 38364, 52684, 14512, 58020,
                39212, 4288,  36504, 26936, 41188, 54148, 62140, 33860, 24688,
                55940, 10044, 26556, 18244, 61748, 3968,  56908, 40668, 25696,
                212,   61000, 38080, 50556, 64924, 21656, 53068, 38472, 32356,
                4508,  21356, 57364, 2156,  45168, 55416, 27088, 42912, 24408,
                29812, 8992,  3712,  27400, 22508, 6188,  31808, 28480, 13252,
                39120, 17916, 31700, 55840, 39864, 9756,  20504, 52436, 15112,
                9100,  43520, 25172, 50876, 36900, 8004,  52728, 25740, 12108,
                57756, 15616,
            },
            {
                37788, 51288, 17696, 64852, 47140, 15144, 53616, 38332, 46204,
                16700, 22548, 40704, 19020, 56940, 44156, 10432, 23420, 19216,
                28136, 1920,  55816, 41560, 5496,  57748, 26084, 46904, 29380,
                12740, 25068, 60792, 10584, 42092, 23216, 7132,  19172, 10992,
                28176, 17716, 35216, 49264, 61156, 45528, 41596, 11460, 3424,
                49712, 37824, 57396, 64592, 34724, 41964, 8152,  26968, 13180,
                18668, 34188, 22860, 46764, 10508, 56556, 5292,  21028, 46356,
                50992, 29784, 64204, 14072, 23272, 31504, 18848, 46028, 2336,
                61296, 15724, 31596, 54356, 48908, 42960, 11672, 62704, 31832,
                15780, 47896, 10420, 4920,  26292, 46008, 9464,  61996, 15856,
                51192, 42868, 29888, 16792, 23240, 10048, 59240, 1280,  64968,
                14216, 55668, 34796, 60956, 46636, 38448, 54072, 43180, 56840,
                62456, 21236, 6448,  44516, 24928, 35260, 58748, 42496, 37688,
                48356, 4744,  56768, 28912, 14808, 42168, 63000, 30340, 40680,
                6224,  33008,
            },
            {
                2104,  21968, 7636,  34748, 58152, 5960,  20924, 59328, 26476,
                55724, 64656, 9204,  28856, 48136, 15088, 38252, 65308, 54364,
                12628, 47912, 30200, 21456, 51164, 61912, 14856, 35720, 7148,
                57212, 39036, 18660, 51984, 56664, 29252, 38652, 54800, 47156,
                62712, 8340,  39528, 23380, 1016,  30660, 20148, 64288, 26440,
                32784, 13076, 460,   20360, 53084, 45416, 21984, 37424, 53532,
                44452, 57252, 1604,  61656, 18032, 43996, 26260, 63528, 33800,
                9772,  19360, 45124, 55780, 2780,  59004, 7524,  40028, 29200,
                49444, 21188, 45116, 36512, 5552,  24840, 19288, 7084,  55024,
                43356, 28020, 33196, 19668, 41692, 57376, 3460,  34424, 27464,
                55752, 12996, 64340, 48392, 38900, 51660, 32544, 20416, 36480,
                47928, 18428, 43412, 11528, 616,   19396, 8600,  23568, 3164,
                49420, 36472, 53324, 61672, 16512, 50156, 2724,  26848, 61812,
                31472, 20028, 63596, 3236,  22776, 49052, 1256,  18628, 55212,
                46668, 61208,
            },
            {
                14288, 45584, 53316, 28528, 24104, 50516, 31220, 11860, 3752,
                36392, 13448, 51084, 4404,  59728, 21584, 6108,  46004, 33308,
                40356, 7620,  63620, 11088, 32856, 2832,  22924, 40872, 53728,
                980,   43992, 33292, 8056,  1804,  48992, 61420, 25196, 2280,
                31768, 57352, 44072, 15204, 53444, 10132, 38620, 52388, 15656,
                44740, 55328, 40120, 28268, 15048, 2696,  58184, 32492, 4948,
                9720,  29264, 14676, 40720, 35780, 31464, 13088, 948,   41716,
                16400, 60004, 5848,  32964, 42736, 11272, 51212, 65048, 12964,
                37876, 4156,  8388,  63380, 14132, 30476, 52156, 35504, 21280,
                4228,  59340, 52252, 61804, 13512, 29972, 39308, 18408, 46908,
                1716,  24932, 7772,  33816, 4872,  28160, 12716, 41824, 7936,
                4268,  52584, 24024, 32480, 57424, 51380, 63068, 35568, 16224,
                26464, 9512,  41492, 384,   30044, 11600, 45088, 18180, 6708,
                12992, 53564, 39296, 46236, 34460, 58740, 13068, 38304, 8816,
                23352, 27032,
            },
            {
                10036, 57528, 39072, 12280, 44288, 1124,  41104, 61024, 48380,
                43948, 32440, 24964, 42280, 34844, 52768, 30812, 724,   58428,
                16436, 24804, 52484, 37716, 44684, 28488, 49960, 10112, 59768,
                16624, 21812, 64244, 36840, 15072, 21116, 12048, 34784, 16384,
                50892, 21252, 3808,  63940, 34036, 58624, 4208,  48044, 6860,
                60352, 23816, 9816,  48404, 61364, 29976, 11380, 63604, 47504,
                20632, 60036, 49076, 24748, 6608,  55544, 49648, 58708, 27976,
                54012, 36024, 24968, 50092, 20704, 36936, 27356, 17096, 23712,
                56748, 42076, 53072, 27632, 58480, 40248, 46128, 65404, 13636,
                39132, 23828, 1764,  36548, 8072,  55036, 22944, 63388, 11044,
                41732, 37468, 60204, 54084, 18656, 63152, 47256, 56172, 60448,
                26736, 38352, 63488, 15512, 28588, 40268, 12324, 30600, 59908,
                46504, 32908, 13788, 58472, 22296, 65456, 33524, 57040, 51356,
                36080, 26060, 10344, 16928, 6952,  54420, 27500, 32696, 60152,
                50944, 35880,
            },
            {
                30580, 64000, 3140,  19252, 62572, 35320, 17300, 23500, 7920,
                19932, 54460, 1860,  62232, 17736, 11736, 61092, 27572, 50252,
                36124, 19664, 5012,  59120, 13680, 18140, 65108, 33936, 26308,
                47520, 30368, 51152, 26892, 59532, 40448, 44300, 6220,  58660,
                37628, 12616, 29984, 26096, 47088, 18220, 28452, 22108, 36068,
                30068, 17840, 43016, 4500,  36568, 50524, 41296, 16588, 25824,
                38812, 33116, 4336,  53048, 62840, 19836, 8792,  39428, 22560,
                3700,  47704, 8488,  63344, 15052, 58128, 44900, 780,   34236,
                60516, 10920, 32432, 22620, 17420, 1188,  9700,  26972, 49924,
                57096, 10988, 46768, 17032, 49676, 43012, 5508,  51804, 58124,
                20960, 31272, 14884, 43104, 22396, 1924,  35336, 16020, 30804,
                21360, 44936, 9704,  5240,  48444, 22024, 44832, 6724,  55340,
                3996,  19672, 51068, 27168, 38496, 47656, 8496,  40636, 23688,
                776,   60428, 41856, 30296, 62288, 43596, 20232, 47812, 5256,
                16984, 43080,
            },
            {
                20612, 48548, 25620, 32472, 54568, 9736,  56704, 37984, 63696,
                30064, 15704, 39092, 47300, 7440,  24160, 41004, 14500, 9168,
                43140, 61796, 27132, 47180, 1404,  55300, 42780, 3736,  38564,
                13296, 5644,  11056, 46468, 4432,  31324, 52512, 64752, 27488,
                46396, 41920, 55796, 6624,  37052, 11652, 43624, 56132, 62380,
                1520,  51852, 57968, 26324, 19560, 7160,  22976, 52468, 576,
                56116, 43456, 17368, 11968, 45444, 29440, 36956, 15544, 61980,
                44416, 12604, 28852, 39756, 3360,  30860, 52848, 6948,  46752,
                19436, 50776, 2860,  37324, 47768, 54692, 34072, 5148,  18168,
                44364, 32236, 29044, 64224, 25476, 33428, 15176, 28592, 35496,
                48248, 4296,  50436, 9856,  58256, 39908, 25128, 10852, 53744,
                1028,  50536, 56480, 33620, 60288, 2756,  18384, 50208, 38744,
                24108, 63672, 43348, 6576,  54884, 3528,  20260, 15492, 29628,
                46704, 55348, 21388, 50540, 2052,  24708, 11052, 40488, 63440,
                280,   55804,
            },
            {
                7400,  15452, 41620, 6128,  45812, 14152, 51444, 26612, 5732,
                50100, 58608, 10196, 28028, 55556, 37460, 45356, 56696, 22456,
                2548,  54152, 33840, 40692, 22320, 30652, 8468,  20288, 52964,
                61112, 56616, 42308, 23136, 54648, 15980, 8976,  23544, 1692,
                17868, 9580,  60864, 50416, 23684, 63252, 39960, 8760,  14172,
                33452, 38692, 12380, 65432, 54768, 34184, 45524, 60524, 13536,
                8132,  64856, 27568, 34684, 1384,  23396, 48308, 5104,  52180,
                32064, 18884, 60804, 54424, 23388, 10572, 41016, 63712, 25448,
                29688, 15312, 44348, 64212, 12036, 60164, 42380, 23448, 61180,
                37160, 7340,  53460, 20128, 2632,  59916, 45396, 9576,  612,
                17796, 64008, 26016, 29636, 52628, 45640, 5744,  61508, 42696,
                36624, 13200, 17856, 41112, 25240, 36236, 65036, 27992, 14584,
                33940, 57288, 10936, 17192, 35888, 31160, 49612, 59056, 63244,
                4956,  14296, 32952, 8184,  37100, 52496, 15620, 57416, 29352,
                22844, 38092,
            },
            {
                60908, 34712, 52520, 58788, 22764, 29188, 1792,  42680, 33960,
                12944, 44448, 21524, 64488, 31852, 19444, 3704,  63368, 32308,
                48732, 12724, 7820,  16156, 58164, 36188, 62108, 49204, 24280,
                31936, 18404, 36104, 12,    62936, 19760, 38776, 49744, 34444,
                53732, 32076, 20468, 884,   15760, 52492, 3404,  25732, 49820,
                20560, 46952, 8024,  31400, 15468, 2816,  39708, 28308, 31796,
                37144, 22012, 50468, 58556, 42508, 54368, 63980, 11104, 26552,
                57452, 436,   43364, 33780, 49336, 18276, 35516, 55504, 9232,
                39004, 56468, 5992,  26356, 20804, 30996, 15732, 50864, 28196,
                496,   14596, 58040, 41212, 12224, 38300, 22216, 62400, 53752,
                39640, 44712, 6864,  34952, 13572, 18056, 32060, 49436, 19944,
                27580, 64196, 30332, 7836,  52736, 43572, 11132, 53936, 8188,
                41284, 1548,  29108, 45744, 62668, 13060, 25364, 43244, 10684,
                38112, 19024, 44648, 64408, 27696, 59544, 4096,  45924, 33520,
                13140, 53612,
            },
            {
                10980, 27420, 3772,  17988, 39512, 61408, 47864, 19712, 57596,
                3612,  25416, 36288, 340,   50464, 6392,  52128, 26320, 17388,
                39160, 29332, 64540, 50552, 25804, 11228, 45992, 15160, 5100,
                41352, 9088,  27432, 58720, 33232, 43552, 28556, 57228, 13812,
                62748, 25156, 38868, 44632, 33864, 29120, 59752, 17548, 54948,
                44184, 4856,  23656, 41516, 47836, 62652, 10312, 18812, 55276,
                46132, 3076,  9804,  38956, 14424, 31056, 18376, 34404, 41324,
                21416, 38072, 15924, 5972,  27060, 60272, 13632, 21588, 3880,
                48240, 61520, 34452, 41140, 52120, 2300,  8224,  56004, 39324,
                63624, 22304, 45612, 34572, 4456,  55604, 49256, 26908, 31868,
                11748, 23588, 54756, 59076, 3032,  65504, 38032, 8692,  58080,
                2848,  47628, 22628, 58752, 14940, 24,    21364, 31520, 58444,
                48144, 20848, 52328, 23144, 40252, 56260, 1332,  53032, 35116,
                27052, 57984, 49236, 11744, 40116, 17936, 35548, 8632,  25856,
                49680, 43652,
            },
            {
                31236, 47044, 64140, 12356, 36096, 8636,  31840, 16220, 52664,
                39820, 60896, 48388, 16788, 42532, 13904, 59712, 35832, 10456,
                44280, 55912, 20648, 5516,  43536, 53832, 808,   28936, 35168,
                64364, 45160, 52316, 14144, 49164, 11408, 3388,  45948, 5468,
                41260, 7468,  48728, 11992, 56496, 5964,  41896, 35604, 31024,
                63948, 28012, 58384, 52284, 21120, 27084, 57256, 5596,  51316,
                15848, 61252, 25468, 20184, 5932,  59864, 2420,  47004, 7672,
                49976, 65424, 9892,  56796, 46700, 1640,  44148, 51748, 32300,
                17824, 23636, 13996, 10040, 58812, 35896, 47216, 19540, 33144,
                10776, 48736, 25948, 51952, 29436, 16408, 7448,  19296, 42204,
                60732, 15740, 37556, 20336, 47324, 28300, 23384, 55056, 12636,
                34556, 42180, 4532,  37656, 48888, 27200, 62184, 45644, 3788,
                15796, 36664, 60764, 4844,  8888,  33688, 17588, 21956, 7508,
                41392, 2212,  24016, 5612,  31076, 47924, 21540, 54584, 65248,
                18916, 2580,
            },
            {
                40628, 21300, 50360, 24944, 44176, 56300, 4896,  65508, 23708,
                9452,  30384, 7012,  54780, 34108, 28992, 22132, 46304, 6636,
                61144, 24480, 2176,  37792, 32000, 18832, 39232, 58500, 55216,
                17216, 2604,  21792, 39416, 7376,  25564, 37248, 60328, 22520,
                16808, 58232, 27444, 65220, 19208, 47120, 22604, 12680, 100,
                10404, 16740, 36192, 2152,  13116, 37672, 43292, 24312, 34528,
                41768, 29344, 47996, 57200, 51492, 27048, 37612, 61396, 14104,
                25024, 52888, 30780, 36572, 22980, 63240, 28504, 37832, 58476,
                42720, 256,   53864, 29296, 45472, 24588, 63108, 13452, 43268,
                4960,  59564, 18316, 9120,  65164, 43716, 58652, 35696, 5540,
                2532,  48620, 32840, 9724,  41008, 51704, 5024,  44260, 17012,
                25788, 62328, 10636, 32508, 55112, 17428, 39788, 34772, 24848,
                63400, 30464, 12132, 49104, 27408, 59508, 46804, 64972, 50712,
                31572, 62064, 53300, 14568, 56256, 61552, 888,   42244, 36820,
                6164,  56868,
            },
            {
                16440, 8216,  33096, 1300,  51936, 20356, 27376, 41792, 34960,
                14368, 46012, 20332, 62872, 11416, 57080, 40096, 1344,  53672,
                15216, 34524, 47752, 59512, 14080, 62716, 22800, 7308,  26248,
                12348, 48168, 61784, 30512, 54868, 64952, 18808, 51696, 29772,
                35468, 52660, 2992,  31384, 8840,  37744, 61808, 51412, 57400,
                39368, 48540, 61476, 32584, 8520,  60236, 49500, 1528,  64476,
                11532, 6680,  33472, 40512, 12080, 45328, 22068, 55580, 29504,
                3960,  44204, 19980, 13024, 41164, 8580,  16632, 4800,  11756,
                26560, 64564, 49584, 18928, 4000,  38580, 6496,  27512, 52920,
                30744, 40616, 3088,  37868, 32344, 12552, 24300, 46816, 52356,
                29744, 56280, 63576, 26140, 396,   14732, 61820, 31284, 39252,
                48520, 53608, 20152, 44524, 61148, 5376,  12448, 50784, 9600,
                42616, 18648, 56012, 43840, 14796, 37660, 3848,  28804, 11984,
                15932, 45784, 20308, 34964, 43496, 26724, 10656, 30152, 14000,
                24472, 61716,
            },
            {
                28212, 54536, 60440, 38508, 15156, 62068, 11116, 49220, 984,
                59280, 53312, 27712, 38932, 24684, 8068,  65076, 49452, 19920,
                30436, 41892, 8412,  27528, 51832, 10308, 46792, 42464, 50856,
                32828, 36996, 24792, 4172,  15628, 44568, 32504, 1888,  9732,
                46744, 13272, 42776, 23972, 54412, 15300, 4056,  27140, 43464,
                24260, 6416,  19860, 44768, 54416, 17540, 30608, 14692, 22328,
                54008, 18680, 62424, 204,   16476, 32204, 9128,  41808, 17956,
                35596, 62248, 2296,  54980, 49664, 59008, 31288, 52824, 47084,
                35272, 7956,  39924, 32660, 60796, 16528, 56276, 49328, 20932,
                62028, 15256, 54992, 23084, 57152, 712,   39772, 62808, 14720,
                22408, 7356,  18664, 43008, 59388, 35872, 21044, 56572, 6512,
                1120,  29424, 14460, 8088,  24504, 30300, 57720, 19968, 53736,
                6696,  38640, 804,   32656, 23676, 53756, 19412, 44296, 39408,
                57268, 26020, 9084,  38392, 3596,  16936, 50296, 59468, 52640,
                39176, 45900,
            },
            {
                34672, 13236, 42928, 5940,  30016, 46264, 32760, 55624, 38020,
                18116, 43280, 6500,  2060,  47820, 17784, 32388, 26900, 37444,
                11340, 63636, 17600, 56464, 3204,  34868, 29956, 4384,  60800,
                19688, 9444,  59580, 41604, 20824, 50024, 12644, 40592, 55568,
                63568, 20700, 38524, 60740, 49408, 34640, 45856, 20244, 33192,
                13952, 55680, 29320, 40200, 24908, 5160,  35652, 46996, 59092,
                37956, 44872, 24428, 56312, 36216, 64808, 52116, 5484,  59096,
                11068, 47944, 27460, 33468, 6280,  18872, 39248, 23840, 62256,
                20256, 14480, 55488, 23032, 11924, 42472, 34368, 9916,  956,
                36036, 26508, 44272, 48340, 20104, 50496, 27912, 10008, 33904,
                44920, 39108, 12096, 53388, 29164, 46740, 10492, 50176, 24580,
                63424, 35956, 59192, 51532, 41372, 36968, 1304,  47680, 28996,
                58840, 25548, 64548, 51768, 5464,  61340, 10280, 55464, 68,
                34000, 6088,  59772, 51280, 64572, 23248, 46592, 33760, 20108,
                9392,  1968,
            },
            {
                23004, 58460, 19620, 26196, 53740, 7960,  22008, 4300,  25444,
                12020, 31312, 61212, 35936, 52184, 43832, 13216, 55356, 5556,
                58084, 50064, 24020, 44312, 39804, 21208, 65376, 16208, 38324,
                53548, 660,   46976, 56728, 34600, 5392,  61896, 23488, 27164,
                6596,  32816, 592,   17344, 7048,  29484, 63640, 9408,  59524,
                52440, 2956,  65016, 10900, 50980, 62056, 27724, 41332, 9316,
                3664,  31156, 8080,  48596, 20472, 3160,  25964, 46372, 31568,
                53796, 39744, 16056, 24548, 64512, 45872, 10444, 56900, 2516,
                43632, 28284, 59736, 1480,  48488, 30104, 64872, 24056, 45988,
                57460, 11816, 8040,  33784, 5360,  60028, 17532, 53892, 2116,
                58116, 25252, 49672, 4164,  16884, 33372, 7040,  40876, 18236,
                12124, 45808, 21500, 3568,  16576, 65264, 44012, 22808, 34348,
                15304, 46160, 11556, 20624, 35684, 41796, 30756, 24864, 63332,
                21256, 48084, 28660, 13144, 31228, 40496, 7512,  4200,  28768,
                63540, 49140,
            },
            {
                37268, 4628,  47292, 64784, 17560, 35580, 59852, 44680, 64012,
                50952, 21128, 56064, 15388, 23044, 62044, 3748,  41068, 45696,
                21760, 864,   33012, 7036,  14620, 48332, 55640, 11284, 23536,
                44772, 31212, 14252, 26444, 10552, 29144, 58112, 36500, 17652,
                48964, 59336, 44732, 26108, 57044, 12520, 41612, 1796,  38816,
                18380, 35920, 46464, 15180, 21148, 656,   56848, 12608, 53088,
                26352, 51228, 15100, 41168, 28820, 60232, 37812, 14340, 23584,
                20732, 7964,  56544, 43400, 528,   35932, 29152, 15008, 33264,
                6428,  51224, 36292, 45604, 7708,  19384, 53492, 5616,  39748,
                17980, 51080, 64192, 40936, 13944, 31008, 36876, 42940, 20524,
                64624, 30484, 36268, 61136, 23332, 54696, 64932, 26764, 58532,
                31648, 38764, 49388, 27676, 33340, 54504, 13640, 8896,  61988,
                2840,  41080, 30092, 48528, 16444, 7492,  50076, 13408, 36692,
                16852, 42084, 54816, 1600,  18956, 58204, 53952, 15324, 44116,
                56384, 11844,
            },
            {
                53000, 30560, 40520, 304,   50804, 12352, 39452, 15752, 28624,
                8808,  34164, 3308,  40008, 9728,  30308, 58960, 25536, 16160,
                35828, 61456, 29104, 52984, 58940, 25780, 1472,  42020, 27856,
                7544,  63064, 18788, 39328, 52708, 43172, 1408,  46168, 54380,
                13928, 39916, 10232, 51456, 36796, 21756, 53816, 24992, 49864,
                30120, 6712,  26240, 58176, 38148, 44068, 33284, 18536, 64108,
                21024, 34668, 61728, 54972, 12720, 9504,  44124, 50324, 63888,
                4724,  37084, 60340, 13696, 51504, 21856, 60788, 48312, 53604,
                40784, 10000, 25484, 16128, 61712, 37780, 27916, 14124, 59692,
                32216, 28796, 1928,  24432, 45468, 62484, 8456,  26200, 47396,
                5304,  15900, 9176,  45768, 1400,  38136, 13820, 44980, 52720,
                2972,  7748,  57020, 10908, 60068, 5092,  25864, 52396, 18052,
                36516, 54960, 4428,  62580, 26984, 58424, 45716, 5000,  52224,
                60608, 9456,  23800, 37344, 44880, 10960, 34804, 60988, 24644,
                41504, 17272,
            },
            {
                26996, 61436, 9948,  33280, 43508, 57772, 23844, 2356,  48004,
                41408, 58372, 46488, 27204, 49404, 18608, 34584, 7624,  54304,
                10480, 47356, 38728, 12152, 18548, 36792, 32600, 51608, 59484,
                35268, 55432, 49496, 6080,  64524, 16104, 21324, 9304,  30848,
                4584,  22824, 28736, 61352, 3288,  47092, 32372, 15744, 60952,
                10396, 43048, 49016, 31676, 8672,  24064, 49952, 5748,  38996,
                43612, 1228,  46688, 6112,  33040, 58060, 18824, 812,   29324,
                42072, 49184, 32428, 28076, 9280,  40316, 4088,  25064, 17688,
                65216, 21472, 55092, 31668, 57056, 3144,  49884, 43924, 21868,
                3980,  54676, 37332, 58356, 16732, 52320, 3340,  56412, 12768,
                40632, 51320, 55852, 19736, 42104, 28688, 5108,  21736, 34968,
                16332, 43308, 24100, 18792, 46572, 39304, 31540, 42172, 49044,
                59560, 23592, 8700,  38192, 52756, 21900, 32884, 39852, 26332,
                3316,  46604, 32720, 63816, 49516, 27364, 21644, 38452, 816,
                32316, 7080,
            },
            {
                21340, 55344, 14356, 25084, 20076, 6380,  54116, 32084, 62284,
                19268, 5868,  12968, 64676, 53380, 136,   38800, 63336, 50648,
                19484, 26820, 4944,  64276, 43640, 8948,  62396, 5440,  17536,
                12028, 22300, 3284,  37176, 24660, 59792, 33636, 52148, 42324,
                64116, 55988, 33768, 14612, 19652, 64888, 5736,  44444, 56324,
                22376, 63408, 19248, 3564,  53488, 63032, 13992, 29792, 55964,
                10856, 17072, 27596, 21724, 39628, 24868, 52788, 34656, 55932,
                12256, 17800, 3548,  20064, 63676, 44824, 55684, 11896, 38124,
                29468, 2012,  44644, 12684, 41276, 24608, 10796, 35412, 62896,
                47024, 13032, 19892, 49128, 10336, 28388, 22268, 38460, 32208,
                60768, 23700, 34284, 11620, 62460, 57600, 50976, 9640,  60464,
                29184, 64248, 53576, 35432, 2224,  63324, 20940, 6612,  28192,
                12844, 33016, 19148, 44004, 11872, 1100,  15140, 65312, 19224,
                29756, 57536, 13848, 6732,  16688, 3888,  52136, 9048,  47852,
                64296, 50740,
            },
            {
                2456,  45148, 38756, 49208, 63256, 29488, 37080, 10152, 52304,
                26164, 37916, 23392, 32676, 42636, 21672, 12208, 44236, 31688,
                2112,  42164, 56196, 22964, 49636, 30620, 20988, 46240, 39944,
                48232, 29696, 43800, 32180, 11500, 47028, 6792,  26288, 38000,
                18768, 1128,  49900, 43276, 38916, 9288,  28532, 36212, 508,
                13708, 34384, 41184, 59356, 16696, 35764, 2208,  48132, 25456,
                59664, 36788, 53856, 64600, 4020,  43192, 15708, 7348,  61620,
                26720, 46024, 58108, 53060, 34152, 15736, 31240, 5720,  46748,
                59048, 50488, 34116, 5340,  20476, 64080, 52960, 17188, 8464,
                26408, 40016, 6876,  32548, 43148, 35572, 59268, 49656, 18372,
                276,   27832, 6156,  48916, 31320, 24592, 18088, 41364, 47664,
                1348,  13132, 40360, 30352, 15272, 50908, 10424, 57384, 45068,
                1380,  64072, 47792, 55884, 28988, 60144, 43252, 54112, 10452,
                35484, 41308, 20816, 55992, 43020, 62416, 31048, 58488, 19144,
                12528, 35784,
            },
            {
                59756, 16664, 7720,  56512, 2896,  46876, 17084, 59132, 14556,
                1688,  45484, 56856, 16296, 6704,  55896, 28040, 58240, 23928,
                14304, 60668, 33256, 16368, 2492,  55208, 13616, 25328, 1848,
                57372, 15340, 60484, 51264, 19728, 40476, 57488, 14552, 61876,
                11780, 45916, 7664,  25680, 54792, 58896, 17524, 52812, 40204,
                27080, 5180,  51188, 11504, 28000, 45404, 57588, 40652, 22064,
                7304,  50860, 30364, 13684, 48852, 60888, 31140, 47556, 21876,
                40092, 9680,  36336, 24208, 1748,  50040, 26448, 57436, 19876,
                8608,  14836, 37536, 60048, 48376, 28604, 144,   33412, 42648,
                57688, 50452, 61240, 23604, 1104,  65268, 14652, 7648,  42316,
                53996, 64168, 37492, 44760, 14944, 3448,  36552, 32304, 55452,
                25784, 20652, 49736, 6152,  58416, 23528, 36184, 17016, 54268,
                39000, 26040, 15500, 34776, 4100,  24744, 37092, 6208,  48140,
                59404, 568,   51684, 28568, 36132, 24112, 40376, 14756, 45588,
                29244, 42448,
            },
            {
                23284, 32572, 28052, 35136, 12192, 40436, 22788, 42948, 34264,
                49832, 29064, 61952, 10376, 35704, 48180, 17460, 4188,  46720,
                37360, 52084, 7840,  40608, 27492, 59748, 38372, 52884, 34088,
                64992, 27092, 9272,  8,     54552, 28636, 4932,  49112, 23256,
                30136, 36072, 60968, 21832, 31660, 12476, 48152, 23880, 60508,
                45840, 55772, 31096, 23356, 38760, 19908, 9676,  31608, 15444,
                61856, 3156,  44868, 9188,  35536, 20036, 11176, 37704, 2688,
                51296, 14744, 62336, 43540, 7904,  39824, 63104, 36452, 43224,
                23756, 63628, 26628, 17500, 7948,  45476, 39244, 54808, 22836,
                4788,  30032, 15356, 55652, 46644, 5528,  20864, 30456, 45708,
                10668, 16292, 21592, 58948, 8544,  54272, 63956, 12244, 5400,
                38516, 61412, 9844,  45268, 27160, 41404, 61964, 32156, 4264,
                22020, 50476, 6780,  61392, 41152, 18480, 50984, 31408, 22416,
                15840, 25504, 8788,  47700, 12052, 2084,  54448, 5852,  25644,
                55944, 4332,
            },
            {
                52972, 65356, 50196, 19320, 61280, 26640, 4504,  64256, 8076,
                21296, 39872, 3860,  51864, 30432, 60856, 40660, 9596,  64464,
                29068, 20780, 11676, 48928, 35088, 5816,  44168, 10264, 18436,
                6920,  41676, 36620, 23616, 63512, 17132, 44932, 34380, 2520,
                51776, 57220, 16580, 4964,  41088, 2344,  33952, 7076,  19772,
                9872,  15976, 65436, 1156,  47760, 60308, 4076,  52644, 42564,
                34012, 19168, 39208, 26624, 59108, 1624,  56404, 25584, 64980,
                32272, 5632,  28312, 55188, 22200, 12876, 18148, 3992,  33020,
                53692, 1012,  40840, 51408, 32536, 58264, 15236, 11808, 62180,
                19332, 36052, 10696, 38260, 26636, 52428, 40308, 57620, 25320,
                35140, 51600, 2308,  39876, 26468, 42936, 49852, 22336, 46124,
                57476, 17352, 34304, 54144, 28,    12564, 47384, 8208,  28552,
                42796, 58688, 10888, 31012, 46960, 9356,  57084, 12820, 61320,
                45112, 38100, 63284, 32128, 60032, 19552, 50132, 32860, 61660,
                37632, 10872,
            },
            {
                40900, 13852, 1196,  43484, 8588,  52552, 48176, 30404, 53956,
                58016, 12500, 19164, 43436, 24936, 1320,  22724, 53408, 34124,
                5172,  43820, 57548, 63744, 18764, 23680, 62036, 31636, 50548,
                21892, 55264, 45364, 12952, 32940, 38716, 10284, 59284, 19536,
                40044, 9116,  27948, 53660, 64188, 44884, 51424, 62016, 29912,
                36520, 49340, 42208, 7868,  54668, 34876, 26460, 64428, 12400,
                48436, 24168, 55396, 16628, 50456, 29568, 40748, 52904, 17212,
                42784, 59016, 19000, 48244, 30584, 60216, 52476, 10244, 48720,
                13612, 29964, 56524, 11204, 21620, 3476,  25888, 31380, 46412,
                1808,  51904, 44408, 63072, 17604, 32868, 12560, 62468, 3912,
                55704, 29500, 47412, 62116, 19028, 30372, 676,   16124, 28468,
                8092,  43708, 31316, 22180, 60324, 38256, 19380, 51792, 64620,
                14076, 35728, 20544, 53368, 23556, 63968, 1516,  28272, 34428,
                3964,  53964, 17668, 4828,  39508, 22732, 44320, 8916,  16944,
                48200, 20228,
            },
            {
                7056,  46676, 25188, 57492, 37752, 32756, 13400, 18324, 448,
                37148, 27036, 47264, 65148, 14616, 38184, 49788, 12800, 18500,
                55060, 25792, 736,   30536, 46780, 15168, 3504,  56692, 28324,
                47508, 16092, 5584,  61492, 50284, 3648,  53256, 26432, 65368,
                44292, 13692, 48012, 37040, 18980, 26568, 14672, 39264, 3224,
                57856, 25632, 20844, 32448, 13420, 17780, 44536, 5980,  29276,
                58188, 432,   63660, 6964,  42640, 22364, 5324,  13380, 46384,
                23648, 35952, 11320, 60,    34648, 46052, 41660, 27472, 61532,
                21112, 45064, 5920,  61196, 37056, 43076, 65128, 49848, 38456,
                59856, 27884, 7260,  22088, 3344,  48788, 9032,  36992, 18176,
                22576, 7288,  13576, 33828, 10764, 57088, 35436, 53148, 61972,
                37388, 2968,  64760, 14432, 50304, 5684,  25656, 33916, 55392,
                1056,  46484, 39724, 3452,  16500, 38480, 43880, 49696, 19864,
                42256, 11048, 27872, 46288, 57820, 13412, 64360, 35584, 240,
                56988, 28428,
            },
            {
                31584, 36144, 60736, 16344, 21716, 55412, 41368, 24272, 45032,
                61528, 33508, 8952,  54256, 5836,  58544, 31612, 45500, 61884,
                39592, 14148, 36904, 9324,  41804, 54000, 36260, 12892, 40552,
                2648,  34768, 57920, 29524, 25108, 20752, 42576, 15368, 30552,
                6368,  23852, 33312, 324,   58496, 10548, 22912, 55088, 43524,
                12656, 6444,  52840, 39684, 62412, 24392, 50624, 38208, 20908,
                10296, 36980, 46020, 14392, 32020, 62600, 34932, 59956, 8956,
                3240,  50068, 39656, 57196, 15308, 6248,  23872, 2312,  55628,
                39380, 35200, 18104, 47964, 24212, 55096, 9320,  5640,  16644,
                23660, 13392, 34340, 58464, 42056, 54360, 28404, 43932, 60248,
                49508, 38680, 58404, 24704, 45104, 5908,  40832, 24084, 13172,
                50936, 26216, 19116, 42088, 29796, 57120, 44728, 10156, 23716,
                18256, 29660, 62960, 56680, 27348, 33416, 14572, 7120,  55888,
                64832, 23776, 51292, 35144, 7388,  25884, 29704, 51516, 40068,
                24004, 63208,
            },
            {
                3028,  53932, 10472, 30040, 3392,  6272,  59516, 9780,  50988,
                15800, 5040,  22264, 41224, 34972, 28152, 20352, 2552,  7952,
                23264, 49108, 60532, 51716, 21368, 26932, 7608,  59504, 24672,
                63828, 10916, 39044, 18080, 47948, 7848,  56348, 35948, 1744,
                54904, 50724, 62208, 42656, 30760, 7592,  49728, 33244, 17184,
                63920, 45520, 28740, 58564, 9936,  2000,  56812, 16024, 41868,
                53652, 33100, 27692, 52392, 9972,  47728, 18936, 27908, 55648,
                31152, 63464, 26252, 52844, 20520, 65316, 37680, 32192, 16304,
                8008,  63260, 28680, 2740,  13964, 34064, 19424, 29376, 42124,
                53172, 56340, 47080, 30984, 15504, 64732, 23888, 1836,  14716,
                30624, 42424, 2680,  50760, 16860, 65500, 47512, 9612,  32520,
                56360, 6296,  48840, 10760, 35912, 2468,  16360, 59556, 41456,
                49532, 7832,  12540, 44404, 5956,  52592, 60376, 25596, 39816,
                31624, 1072,  15328, 56152, 41836, 2480,  18288, 59552, 10648,
                15092, 44840,
            },
            {
                18068, 22460, 42540, 50692, 46924, 64612, 35076, 28672, 38888,
                55920, 31332, 49928, 60016, 16668, 10180, 52652, 42272, 56620,
                33552, 28408, 17232, 2940,  33108, 62664, 48416, 44376, 18984,
                32404, 53372, 43864, 844,   60436, 32056, 11584, 62792, 45964,
                38052, 21040, 11468, 16504, 53264, 38196, 62824, 27980, 1084,
                22036, 34100, 3972,  18800, 47440, 35476, 31180, 61556, 3624,
                23672, 60412, 18008, 4176,  39320, 54840, 936,   43416, 38064,
                21168, 16164, 7732,  42140, 29024, 9508,  50544, 58280, 46660,
                25812, 11788, 53416, 41472, 59028, 51252, 45912, 60644, 36340,
                4420,  10012, 39704, 976,   20760, 7688,  35072, 46808, 53176,
                9944,  20004, 62564, 27708, 37124, 21844, 2160,  59844, 20172,
                44844, 39596, 59156, 23796, 62888, 52588, 27936, 37708, 3976,
                61104, 35008, 25004, 50880, 37212, 21564, 11436, 47484, 18108,
                9396,  37524, 60560, 20912, 47616, 32200, 54104, 43748, 4548,
                33796, 49172,
            },
            {
                8012,  62676, 38368, 12900, 26388, 19508, 43844, 14456, 3172,
                63328, 19040, 25828, 1004,  46268, 63552, 37204, 26596, 12200,
                64904, 5672,  43956, 57912, 11772, 38792, 16180, 1292,  50240,
                8612,  21612, 26688, 14088, 51820, 40196, 22908, 27896, 17276,
                8564,  59624, 25976, 3496,  46724, 19744, 4904,  41628, 47524,
                55788, 37804, 50932, 14380, 42596, 26924, 7488,  45300, 13516,
                49288, 7844,  43756, 22044, 58728, 25276, 15012, 50704, 11400,
                61124, 45152, 4284,  33512, 60460, 44152, 12640, 19388, 4752,
                42748, 49896, 20676, 31744, 7212,  26264, 492,   12460, 21332,
                63748, 25576, 18048, 60860, 51548, 40932, 12760, 61248, 26116,
                56528, 33088, 5364,  52688, 12832, 31448, 54776, 28884, 16464,
                34860, 1164,  14100, 31776, 7992,  46176, 21404, 13496, 53776,
                31092, 20236, 58024, 16888, 2148,  62320, 30768, 4032,  59116,
                44992, 50168, 28520, 12536, 62196, 8508,  22232, 37712, 26272,
                55588, 60008,
            },
            {
                27944, 32500, 644,   58492, 33808, 8480,  53296, 23040, 48056,
                10964, 36680, 44436, 13284, 55436, 23112, 6336,  50492, 15948,
                46992, 40328, 20484, 54224, 29948, 4928,  23696, 55688, 28572,
                37484, 57444, 5388,  64948, 34620, 45276, 3176,  58000, 48500,
                34156, 41136, 29208, 56220, 35112, 58880, 24728, 14032, 60904,
                11308, 8228,  59744, 23440, 64372, 55184, 20068, 38968, 29684,
                56400, 35280, 65284, 11880, 30876, 36740, 63912, 6160,  32668,
                24788, 48304, 57868, 13924, 23328, 1272,  35672, 53696, 30356,
                59940, 1708,  37156, 64088, 16568, 39332, 56052, 32476, 50180,
                43592, 30392, 48944, 37140, 27428, 57296, 31960, 4984,  37884,
                16352, 41272, 46472, 8892,  60924, 43380, 39116, 7676,  52040,
                64216, 26600, 47580, 55352, 17672, 40988, 33356, 65092, 6848,
                42980, 47840, 9832,  40132, 27956, 55520, 42320, 35656, 54280,
                22920, 6400,  33532, 3520,  39344, 15748, 49996, 65116, 11904,
                20436, 40532,
            },
            {
                14248, 52572, 24464, 16988, 55120, 39532, 1772,  60096, 27308,
                41864, 57980, 4676,  29852, 39660, 33352, 19344, 58984, 30872,
                420,   35396, 8716,  25168, 47776, 64456, 42792, 34956, 13308,
                61572, 41056, 47176, 30324, 16320, 7188,  19940, 53132, 13376,
                4880,  64756, 50088, 14528, 43952, 10072, 32100, 50424, 18320,
                30116, 26092, 41096, 32736, 200,   11720, 51800, 62672, 17392,
                1788,  26564, 39892, 48648, 2544,  46352, 20536, 41356, 53920,
                1916,  36304, 19232, 39280, 55536, 48712, 26944, 63492, 39860,
                14496, 24452, 55200, 10412, 45048, 22848, 61460, 8328,  15960,
                3020,  58920, 13244, 8604,  4004,  43488, 18948, 50380, 22292,
                104,   64384, 23720, 35508, 18840, 3764,  49216, 24880, 11688,
                42716, 4512,  21092, 37440, 61612, 1684,  11020, 49192, 25908,
                15636, 632,   63444, 33004, 45996, 8392,  19400, 12848, 26980,
                15876, 63780, 42996, 53168, 24468, 58252, 34540, 6276,  30496,
                47112, 3800,
            },
            {
                57584, 43440, 48208, 5416,  63172, 29760, 50028, 16152, 32540,
                6884,  21688, 51060, 62804, 9044,  48640, 2664,  43568, 54628,
                22784, 62160, 51480, 14068, 33596, 18816, 58808, 7200,  51876,
                17584, 3544,  11324, 55040, 24304, 59592, 38672, 42924, 24752,
                31736, 19084, 7616,  23372, 1248,  39700, 64396, 6856,  36716,
                44560, 52976, 6524,  16748, 49424, 37008, 5136,  24676, 47100,
                10564, 53848, 19856, 15720, 57508, 27756, 9760,  59020, 17396,
                29648, 51580, 9092,  64504, 31120, 16992, 5680,  22072, 8680,
                33944, 18448, 48600, 29664, 5276,  34820, 47284, 28064, 40384,
                54656, 22752, 33908, 64468, 24384, 54740, 10720, 62348, 44704,
                29412, 48824, 14140, 55336, 26656, 58076, 15188, 62216, 36116,
                57008, 30472, 50680, 6672,  27720, 51848, 23304, 57844, 39228,
                29248, 54524, 22728, 13884, 50112, 24288, 65464, 51788, 64,
                40508, 56548, 10032, 18004, 46684, 27392, 760,   44616, 17380,
                51544, 35440,
            },
            {
                63800, 19292, 10408, 41072, 21468, 12312, 35900, 44828, 65476,
                53976, 14092, 34548, 17964, 27588, 60156, 14364, 26128, 10628,
                38648, 17768, 59732, 44540, 2220,  40128, 10620, 22436, 45752,
                32764, 26024, 20952, 36984, 51028, 9672,  28344, 63020, 624,
                54640, 37696, 45576, 61724, 52740, 20584, 27716, 54620, 1892,
                58096, 20508, 62240, 45692, 29168, 21636, 59196, 33752, 41520,
                31276, 60168, 5028,  33164, 52564, 6772,  35220, 23408, 63024,
                12440, 43332, 26576, 2932,  45684, 11240, 41680, 50796, 57972,
                46036, 61968, 3396,  42212, 58320, 13648, 19124, 52288, 35940,
                6360,  45960, 16832, 39112, 48008, 30952, 15296, 35680, 7792,
                57896, 11260, 38836, 31084, 5984,  45288, 33064, 772,   22700,
                17860, 9344,  45012, 58524, 15288, 43392, 35520, 3352,  18496,
                45580, 59572, 36972, 5700,  60496, 2916,  38308, 33344, 47676,
                29452, 20736, 36180, 30924, 63076, 13520, 41432, 56204, 61708,
                23444, 8708,
            },
            {
                31428, 26896, 37152, 60832, 46380, 25652, 4316,  19624, 9520,
                40144, 25308, 46180, 3192,  52732, 41592, 36388, 63648, 47920,
                4104,  27732, 31264, 5900,  56336, 26740, 53216, 29460, 108,
                64020, 49300, 42660, 60672, 1760,  46348, 15128, 35416, 49616,
                11412, 58964, 15756, 30344, 34744, 11760, 47020, 40960, 23820,
                14724, 34840, 2980,  38880, 54480, 13700, 43344, 7712,  15220,
                50584, 22704, 44812, 62220, 41972, 13436, 49668, 45192, 39040,
                4888,  56872, 21528, 34476, 54060, 61780, 37196, 29360, 308,
                12932, 27752, 36648, 53232, 25712, 64716, 1360,  10556, 62608,
                26404, 57136, 664,   52716, 20856, 2712,  58988, 27260, 40408,
                53428, 20024, 3232,  51428, 60104, 21284, 40152, 48060, 53208,
                38224, 63536, 33644, 11848, 19984, 29988, 53896, 63876, 8284,
                33976, 11588, 41532, 18968, 31216, 43620, 16560, 58396, 11252,
                5004,  60884, 49348, 2500,  7588,  52216, 21268, 10288, 28336,
                39236, 1556,
            },
            {
                13096, 50708, 2864,  33332, 7916,  59160, 51752, 56536, 30628,
                184,   61444, 37436, 57112, 11388, 23412, 6628,  32072, 20580,
                56700, 41800, 12852, 37400, 49068, 15964, 62408, 35904, 39568,
                14864, 55800, 6000,  33628, 18652, 30508, 55456, 6476,  20364,
                27056, 41304, 2528,  25280, 55976, 4644,  17076, 62828, 32956,
                11036, 48396, 25376, 9336,  65100, 4080,  26456, 56884, 63664,
                1000,  38104, 8920,  24948, 17664, 29876, 56032, 252,   31972,
                16276, 47904, 60388, 13756, 6976,  24852, 19352, 52724, 43676,
                21376, 56764, 7136,  16052, 21908, 40912, 48900, 31908, 20596,
                12792, 42132, 29528, 9928,  62756, 42548, 46712, 5740,  17556,
                24724, 34252, 61836, 42840, 16760, 10612, 29280, 7116,  13652,
                27336, 2796,  24420, 40856, 61224, 5660,  47148, 13608, 21980,
                50616, 4272,  26184, 52712, 56852, 27768, 7044,  21936, 44848,
                53500, 14804, 23944, 43324, 38120, 57884, 32836, 36608, 15524,
                54392, 46256,
            },
            {
                21732, 56808, 15928, 54156, 28732, 14044, 38012, 42152, 22544,
                48276, 16596, 6204,  31432, 19612, 45180, 54284, 16028, 50528,
                9024,  45892, 65364, 23924, 8140,  20832, 46452, 4552,  59204,
                19704, 9056,  27648, 13280, 40564, 64688, 23228, 44052, 60472,
                33560, 47788, 9184,  50632, 43040, 38436, 58700, 8176,  51476,
                42360, 60824, 57172, 18672, 31436, 47600, 52532, 35228, 20392,
                28124, 12784, 55592, 35808, 2708,  63852, 19956, 26304, 61216,
                8300,  36892, 28536, 42136, 49632, 32276, 15064, 4620,  65104,
                32808, 39272, 10700, 50964, 30840, 4220,  56124, 38084, 44432,
                59700, 50728, 18644, 36768, 25388, 13320, 32804, 51148, 64796,
                1464,  47360, 28144, 8204,  37220, 49476, 65156, 56432, 42584,
                60268, 46576, 54384, 50016, 224,   38068, 24900, 42192, 31780,
                56112, 62636, 46428, 15040, 9592,  48968, 35356, 62132, 25640,
                38960, 32560, 9252,  59816, 18944, 26112, 3112,  50368, 4780,
                59488, 41544,
            },
            {
                6700,  65044, 43648, 23552, 48696, 1736,  17912, 63916, 11580,
                27864, 59040, 43052, 49836, 64748, 29004, 2316,  39196, 61472,
                34296, 964,   17404, 51000, 59596, 32904, 41488, 11128, 24884,
                31724, 44188, 62040, 53556, 3180,  50404, 10160, 38912, 3932,
                17312, 62876, 22684, 57716, 19596, 13716, 26392, 30908, 21744,
                4344,  28436, 15420, 36532, 40852, 22852, 10488, 17476, 39840,
                48400, 59348, 31068, 46640, 51396, 38316, 10952, 41228, 52820,
                22840, 55020, 18784, 1076,  38556, 56972, 59740, 8968,  26424,
                49060, 17468, 62788, 45212, 34664, 60332, 8852,  16496, 24276,
                3616,  7656,  32220, 57744, 4640,  54168, 38384, 21964, 11692,
                41500, 56756, 14024, 22760, 54124, 25236, 3080,  19880, 31940,
                16484, 8660,  21452, 31004, 17704, 56648, 9980,  59868, 16452,
                27960, 1184,  36232, 23644, 64028, 40212, 13008, 54940, 2792,
                17792, 63896, 51916, 29028, 46140, 14400, 64700, 44360, 24820,
                18328, 33992,
            },
            {
                29540, 10804, 38808, 19788, 62188, 35304, 45636, 32320, 4068,
                54596, 34052, 13624, 24316, 9180,  36008, 59444, 13040, 25868,
                22152, 29828, 54836, 35768, 27388, 1844,  14236, 57368, 48256,
                52260, 37496, 22360, 46872, 35640, 26472, 14708, 56456, 31516,
                52152, 13364, 36908, 3336,  32624, 65344, 48052, 556,   37872,
                53884, 44600, 6656,  50972, 840,   61648, 45184, 3040,  54296,
                6004,  42628, 16636, 4380,  22040, 58072, 6928,  48740, 14728,
                33424, 45340, 5384,  63100, 10924, 23124, 46548, 36316, 41380,
                2764,  54004, 23876, 5504,  14040, 26648, 46880, 29140, 54488,
                35392, 65416, 40484, 48216, 16252, 60708, 29100, 8348,  59144,
                36004, 30612, 4804,  63036, 33124, 45888, 12372, 35444, 50780,
                4212,  39776, 36364, 13936, 62616, 27412, 48748, 35424, 6828,
                44284, 39460, 19684, 6384,  33132, 1840,  20100, 30680, 42408,
                47316, 5668,  35636, 692,   11520, 55480, 39084, 30520, 8624,
                61956, 47832,
            },
            {
                25496, 3856,  53252, 30764, 5604,  9748,  55584, 23988, 50444,
                8096,  20296, 39880, 1268,  56180, 17288, 47428, 5756,  44108,
                57704, 40648, 11212, 6504,  43220, 62940, 53664, 28652, 17712,
                6312,  2124,  16120, 58168, 7444,  20728, 59992, 47328, 28164,
                7788,  40504, 45980, 28808, 6668,  52800, 44076, 17884, 61944,
                12708, 33912, 20808, 55900, 27560, 13072, 33432, 29700, 62568,
                24648, 10216, 64916, 27300, 13264, 32968, 43660, 28924, 3152,
                65332, 12180, 26192, 51664, 43576, 29752, 16868, 55204, 13248,
                20796, 30144, 59212, 37240, 41872, 19928, 62976, 1720,  49792,
                14260, 21568, 10864, 23808, 43200, 2380,  19160, 44924, 25852,
                16708, 49316, 43752, 19276, 412,   60596, 41268, 26960, 58848,
                23504, 64200, 55928, 6484,  45224, 32944, 3304,  20204, 54512,
                12804, 61036, 47940, 53356, 59464, 44564, 50752, 57836, 10124,
                27536, 21680, 56560, 41704, 23360, 49120, 5796,  21012, 52692,
                12668, 37576,
            },
            {
                50084, 60276, 41688, 13476, 58596, 26976, 40916, 15044, 60892,
                37172, 46580, 62696, 51488, 26664, 41568, 32468, 52400, 19228,
                3620,  63376, 15436, 47408, 19068, 23168, 38024, 8732,  35028,
                65040, 41992, 29924, 11948, 32848, 42828, 37652, 336,   23804,
                64300, 19080, 54564, 61220, 11548, 23160, 35032, 9688,  57028,
                25180, 47184, 64180, 8548,  41960, 59612, 48924, 15244, 37392,
                21200, 34504, 49308, 40300, 53676, 61744, 24148, 18296, 58876,
                36352, 40168, 56476, 31816, 20220, 7296,  2252,  63908, 33544,
                44020, 10028, 47308, 572,   56588, 52044, 11484, 39480, 31620,
                61256, 45424, 28264, 56016, 7220,  34812, 50212, 62820, 53472,
                6532,  55832, 10848, 39128, 52080, 16228, 6916,  54632, 10020,
                47584, 28984, 42788, 11684, 52832, 22380, 41580, 65288, 51032,
                24440, 30176, 10820, 16640, 29056, 25112, 4636,  15984, 37312,
                61288, 13932, 50580, 17456, 31992, 59344, 35240, 16604, 41120,
                57108, 348,
            },
            {
                15196, 33400, 18572, 47056, 36460, 51392, 20568, 740,   43740,
                29268, 17004, 4580,  31124, 11564, 20980, 63812, 8484,  28320,
                49536, 36852, 52860, 30940, 58692, 3428,  50460, 45536, 20776,
                59332, 26032, 49704, 55312, 63688, 5132,  16848, 54132, 12064,
                43760, 2808,  25124, 34200, 16424, 41756, 59412, 29392, 40012,
                2720,  16648, 31204, 38164, 23548, 18164, 4976,  57632, 7580,
                43984, 56240, 504,   19348, 8324,  37116, 1672,  51992, 46360,
                21576, 8444,  16080, 48820, 35512, 60456, 52232, 27280, 49708,
                6100,  62300, 25608, 17148, 32880, 7500,  23008, 43656, 18120,
                6372,  2732,  37376, 51240, 63904, 31364, 39376, 14136, 1060,
                33368, 21032, 27564, 58584, 23996, 30212, 37088, 43976, 18336,
                1176,  14880, 34320, 25980, 2120,  58308, 15224, 8416,  37720,
                4368,  34212, 57388, 42096, 8144,  38468, 65056, 22464, 48752,
                33448, 7312,  38720, 64268, 9528,  26816, 2020,  63272, 45128,
                28048, 22392,
            },
            {
                54312, 44140, 6376,  24716, 3068,  62980, 31804, 57412, 11144,
                25452, 53768, 35592, 60060, 43988, 3008,  37932, 55240, 13484,
                33328, 22816, 9264,  26176, 39728, 12648, 32664, 56552, 636,
                13780, 39536, 9164,  23368, 45456, 28396, 49064, 61848, 31864,
                35916, 50472, 8964,  56488, 48420, 1984,  51236, 6148,  19852,
                49752, 54572, 4472,  11768, 46512, 53016, 26204, 40232, 51728,
                12344, 28456, 59948, 31340, 45360, 15528, 56036, 10784, 30588,
                4028,  25028, 62476, 1328,  12816, 40828, 23664, 37412, 18504,
                58100, 39836, 12572, 50268, 28712, 63976, 35992, 57292, 26412,
                52612, 58996, 15992, 20240, 12700, 25432, 9220,  22556, 47644,
                41844, 37480, 64900, 8244,  48204, 13508, 63708, 22212, 32652,
                61120, 51844, 20020, 60116, 48424, 39056, 30800, 26780, 46200,
                18540, 62436, 440,   21064, 51632, 13540, 31576, 54752, 1564,
                44860, 58644, 29228, 3768,  43168, 47736, 54748, 11328, 32580,
                7416,  61400,
            },
            {
                10224, 29052, 64024, 56188, 11916, 17120, 38928, 7532,  49732,
                64940, 6228,  14468, 23132, 57336, 49196, 16064, 24896, 59368,
                43480, 264,   61448, 44808, 4924,  64124, 16972, 24600, 30304,
                44044, 53336, 17900, 2400,  35296, 19988, 9776,  40740, 21392,
                58620, 15036, 29848, 38664, 21060, 26832, 13112, 62920, 36884,
                43560, 27188, 61588, 57860, 34696, 1260,  64508, 20464, 31872,
                47204, 17220, 5576,  50768, 25588, 64284, 34416, 41084, 49692,
                60136, 42956, 53100, 28728, 47164, 57344, 8804,  14684, 3576,
                31108, 21960, 35564, 54624, 45324, 4720,  14876, 48428, 9808,
                34316, 40604, 30156, 46936, 1700,  58432, 44256, 61772, 29404,
                57060, 15348, 3940,  45232, 34852, 2692,  50204, 5620,  56168,
                38732, 45820, 4484,  36700, 9956,  18224, 55828, 60528, 13492,
                49724, 40452, 54616, 27152, 47060, 60368, 43112, 6768,  26068,
                19100, 12212, 23884, 52504, 20136, 15120, 37844, 24536, 51216,
                19312, 36136,
            },
            {
                1652,  39288, 21488, 49464, 42044, 28276, 46772, 22368, 33564,
                41652, 20008, 47372, 40064, 7824,  28900, 34680, 46492, 6212,
                39968, 17920, 56252, 20900, 53400, 34688, 47740, 10728, 61160,
                6640,  33296, 62904, 38408, 59760, 14228, 53040, 1660,  26076,
                6120,  47128, 60252, 4796,  64652, 45596, 32292, 55720, 23348,
                10332, 33612, 14696, 20948, 43248, 28744, 10712, 36276, 3444,
                62880, 23172, 37776, 42032, 12032, 21484, 4908,  26728, 17608,
                13588, 33476, 38028, 18836, 5280,  32528, 44276, 61680, 48092,
                42236, 56136, 2332,  8780,  20156, 60424, 39156, 3220,  21140,
                60844, 24240, 6832,  54636, 41760, 35884, 52108, 18124, 6104,
                11344, 51644, 31692, 25956, 17896, 54932, 40284, 28184, 11512,
                25132, 8316,  29740, 64556, 23784, 44572, 6192,  1456,  35500,
                22128, 10004, 32196, 6008,  36360, 18144, 11384, 35012, 63160,
                41420, 55860, 46628, 33048, 62092, 6924,  30220, 59680, 4124,
                40612, 48124,
            },
            {
                58140, 16652, 31900, 8368,  34984, 54236, 2320,  61652, 13220,
                55484, 1336,  27312, 52112, 11028, 62008, 1512,  20328, 65172,
                11704, 48568, 31452, 14232, 28104, 7152,  41700, 55152, 37328,
                21712, 47168, 27256, 4688,  50696, 30268, 57464, 46340, 38076,
                17848, 53632, 12412, 41076, 18424, 36148, 7628,  15612, 47068,
                248,   52192, 40460, 7204,  49492, 16800, 60544, 44892, 14924,
                56924, 9060,  54580, 61384, 2164,  47796, 57824, 38768, 63060,
                6184,  55172, 9812,  22712, 64308, 16356, 25972, 53440, 11120,
                27584, 65160, 16000, 24920, 42556, 31200, 27072, 54076, 44676,
                13360, 49556, 62864, 10664, 26876, 15540, 4324,  32992, 24488,
                60512, 20056, 42516, 62432, 9836,  58316, 20628, 46760, 62968,
                16876, 42188, 56944, 13872, 50052, 33180, 41736, 51780, 63280,
                29416, 43884, 15764, 63840, 24324, 2288,  53772, 29768, 50628,
                15632, 36816, 196,   10372, 40344, 57316, 22076, 46248, 13900,
                64744, 25604,
            },
            {
                12056, 45400, 60876, 3872,  15476, 59220, 25816, 18464, 44248,
                29396, 37280, 63236, 32792, 18740, 54436, 42480, 30568, 51400,
                27148, 35600, 2640,  38488, 50140, 60612, 19280, 26808, 4064,
                51772, 15344, 56912, 11868, 41396, 24080, 7228,  33788, 63180,
                43092, 27628, 32948, 23580, 1116,  57800, 52836, 29076, 59880,
                19008, 65084, 30632, 24284, 55076, 38592, 5764,  53304, 25292,
                40972, 33832, 27520, 19748, 32684, 53512, 29780, 9228,  44804,
                20720, 27928, 59276, 45932, 51524, 39504, 216,   34912, 18964,
                6568,  37732, 46308, 52780, 58724, 11196, 17108, 64724, 32864,
                892,   18420, 37636, 32380, 21412, 65136, 48476, 55308, 40264,
                36396, 49360, 2016,  53468, 38168, 30864, 14952, 540,   36000,
                53244, 32408, 2340,  21204, 54188, 27424, 16840, 8532,  24940,
                55260, 4276,  59400, 49020, 38740, 57808, 45760, 20404, 9296,
                5784,  60588, 28120, 17568, 49688, 34788, 2184,  42452, 17952,
                52424, 34232,
            },
            {
                22572, 53528, 27296, 50572, 43384, 10528, 38240, 51044, 4696,
                9556,  58764, 15860, 3484,  24700, 38540, 13676, 57524, 8272,
                23068, 58600, 62992, 10088, 23896, 44332, 1032,  13444, 64596,
                40736, 8748,  32148, 20708, 44556, 65468, 16616, 11296, 22660,
                3372,  9268,  61816, 51588, 44584, 11156, 25272, 41848, 37476,
                5228,  44100, 9740,  58208, 2100,  33252, 22492, 30412, 49932,
                1532,  13192, 46672, 7104,  16584, 42876, 14212, 23432, 51760,
                1728,  48860, 35556, 3812,  12184, 29336, 56668, 21740, 60360,
                50748, 29736, 13176, 33396, 1476,  49136, 37236, 7808,  41216,
                28216, 58244, 5436,  43312, 56892, 8248,  28836, 132,   44964,
                8932,  27800, 13940, 22316, 5536,  25000, 43788, 59436, 6844,
                23400, 48760, 61228, 39184, 4748,  11568, 58160, 38628, 19632,
                46088, 12168, 34276, 20688, 7680,  14464, 26672, 40688, 58800,
                23096, 32080, 53012, 64944, 25436, 12584, 54300, 27780, 9072,
                30748, 4884,
            },
            {
                39768, 6836,  37020, 20040, 65440, 23868, 31080, 56864, 34632,
                47988, 23560, 41200, 45928, 50292, 60640, 5052,  36324, 17104,
                45036, 4400,  41480, 52052, 16100, 32960, 57648, 35876, 29800,
                45600, 24972, 62004, 49000, 1992,  36464, 28840, 47764, 54320,
                58388, 36868, 48460, 15132, 29616, 63704, 3840,  21380, 12712,
                50356, 26232, 35608, 13956, 63364, 45828, 11448, 58760, 17412,
                65472, 36784, 52364, 59372, 38440, 64016, 3532,  36368, 59952,
                31644, 41336, 15552, 24528, 43352, 62488, 7724,  47320, 41632,
                2704,  22928, 63136, 40228, 5968,  21572, 57140, 24424, 51096,
                46044, 22288, 52556, 14300, 47268, 39224, 19792, 13108, 59140,
                16940, 64472, 34764, 46032, 60868, 49876, 33404, 51868, 12960,
                28616, 18852, 10084, 46000, 34848, 63796, 47952, 31172, 60432,
                2820,  41128, 28208, 52892, 43204, 31412, 64628, 4604,  48108,
                38116, 44652, 3820,  21312, 43732, 5064,  60280, 38188, 62644,
                49272, 56888,
            },
            {
                17196, 61704, 13160, 33436, 1144,  47528, 6260,  14004, 62692,
                20384, 53600, 12068, 6960,  33968, 28240, 21328, 48128, 53992,
                32516, 12872, 20188, 55368, 28932, 8172,  48644, 21212, 53972,
                16904, 3536,  34880, 18348, 55248, 5952,  51892, 40336, 14036,
                30956, 20264, 6764,  39352, 19044, 34020, 49460, 56652, 31348,
                61932, 53944, 17836, 48272, 27664, 19568, 39500, 7572,  43260,
                21596, 5272,  29520, 24340, 10560, 49892, 26152, 55676, 18248,
                11492, 65204, 8148,  57804, 33128, 17504, 38232, 14284, 31168,
                10236, 56272, 18216, 26544, 61336, 45144, 34628, 15260, 4240,
                11632, 62252, 30664, 1940,  25184, 34104, 62024, 50916, 22960,
                31996, 4480,  54492, 40464, 11864, 17620, 8256,  40824, 65496,
                37192, 43136, 57788, 16084, 29624, 22772, 44,    14372, 35668,
                51556, 17236, 23912, 57304, 704,   11188, 50364, 34076, 17292,
                12828, 55472, 9668,  39648, 15804, 48880, 31952, 19456, 23772,
                792,   43644,
            },
            {
                25860, 47888, 54824, 28360, 41220, 52896, 17872, 39564, 26804,
                392,   30836, 64280, 18024, 56160, 10184, 43396, 1696,  64104,
                39412, 26168, 46984, 34884, 5168,  63320, 40332, 11368, 59904,
                7156,  50188, 58352, 39572, 10640, 26104, 21352, 61064, 40,
                44820, 26452, 56484, 2636,  60632, 42932, 7316,  16812, 39716,
                1420,  22696, 7140,  41928, 3384,  51352, 61276, 34756, 27004,
                48724, 57116, 41696, 48,    20772, 44420, 34136, 7360,  46440,
                39204, 26484, 53592, 20212, 50388, 2172,  55324, 25020, 52460,
                35588, 43860, 48316, 8652,  51828, 12404, 29908, 63520, 55848,
                39392, 35616, 17336, 59972, 10840, 54136, 5924,  37060, 42560,
                57072, 47980, 19308, 29224, 1412,  56144, 26372, 20884, 4236,
                54452, 1592,  26220, 7336,  50308, 55844, 43512, 25904, 6892,
                65224, 47076, 9500,  61868, 36704, 19752, 54508, 24508, 1616,
                62772, 26316, 51160, 29776, 35288, 55984, 7252,  46584, 13736,
                35700, 10732,
            },
            {
                31752, 19304, 3584,  9584,  58928, 22540, 61140, 46100, 8408,
                43056, 36612, 51088, 25356, 38988, 61496, 15640, 24144, 29716,
                9796,  59428, 2716,  61432, 18040, 45164, 24320, 1676,  37764,
                42964, 27344, 13000, 30592, 43812, 56964, 33084, 8036,  18072,
                64128, 35128, 52012, 24220, 10276, 54120, 26824, 47492, 10724,
                34332, 45540, 57564, 37600, 30196, 56408, 15592, 4280,  54316,
                11004, 14384, 33684, 62152, 54908, 30504, 13080, 62844, 22480,
                828,   30236, 44440, 4772,  37168, 28292, 45544, 64568, 5336,
                60056, 15920, 4152,  32496, 38492, 18924, 288,   42428, 20428,
                8796,  25760, 49588, 40812, 44708, 18360, 29720, 2748,  26912,
                11428, 7864,  24616, 62736, 37396, 44244, 61376, 30284, 47480,
                14576, 32252, 63016, 40024, 20292, 36748, 9828,  53584, 40664,
                21384, 33068, 4728,  29952, 15892, 45336, 40244, 60228, 30448,
                36420, 20072, 58668, 2072,  63196, 11124, 26988, 40712, 58296,
                52188, 64560,
            },
            {
                41552, 57768, 38212, 45316, 16088, 35576, 11792, 32464, 54804,
                57976, 14516, 5344,  45016, 2644,  31844, 52540, 57644, 7028,
                49520, 15320, 22256, 37544, 50608, 13672, 27796, 52144, 31556,
                19404, 61504, 22624, 1160,  64376, 15652, 46840, 38244, 50596,
                10984, 41384, 13552, 46928, 31788, 37256, 21084, 58420, 64660,
                28724, 14984, 60748, 12016, 21224, 8880,  24808, 46852, 31480,
                63248, 19368, 46164, 8492,  16952, 39032, 4540,  51124, 56920,
                16432, 49276, 59124, 14052, 61992, 9332,  21600, 12024, 19284,
                27416, 39668, 57996, 23176, 54544, 60180, 27484, 48672, 53536,
                32400, 3004,  56956, 7176,  23184, 63212, 50072, 58832, 16136,
                38828, 52988, 32800, 4048,  51132, 15336, 34440, 10168, 58436,
                24252, 44144, 51436, 12392, 4084,  61832, 18092, 57944, 28860,
                12240, 58844, 38676, 49524, 55948, 27320, 5444,  8824,  15028,
                42632, 47104, 6268,  17448, 45176, 21776, 60972, 2684,  16552,
                28716, 4760,
            },
            {
                8112,  50636, 24188, 62800, 30216, 5860,  50228, 2700,  19128,
                23200, 28876, 59996, 20972, 48808, 12928, 36408, 19392, 40908,
                33712, 55960, 42856, 30892, 8856,  57544, 34300, 65168, 15124,
                54892, 9216,  47444, 53384, 37064, 5212,  24696, 58716, 29220,
                22796, 4252,  59668, 17308, 63288, 684,   14220, 4936,  42336,
                24236, 3124,  51944, 32984, 49096, 64800, 36248, 41484, 2436,
                23516, 39952, 27952, 51840, 60364, 48252, 19808, 28224, 41852,
                36652, 10292, 33988, 23748, 40472, 31460, 48800, 42048, 34028,
                50592, 1644,  30052, 13860, 43256, 6728,  10716, 36924, 15936,
                46244, 64636, 28696, 14976, 37896, 33080, 8984,  21788, 45884,
                63932, 43444, 13460, 57784, 22516, 7088,  49152, 18900, 39356,
                5140,  35460, 21816, 59216, 30720, 45156, 33492, 2244,  48772,
                16488, 44480, 25164, 2384,  12744, 64260, 21780, 52776, 49124,
                57204, 11612, 33168, 41280, 25016, 53628, 37348, 49916, 33572,
                44672, 22188,
            },
            {
                15404, 34256, 12444, 1820,  56048, 43700, 27744, 64812, 38004,
                48492, 10476, 34228, 40164, 8560,  65296, 26964, 46420, 4,
                62684, 11656, 26760, 53580, 2128,  40780, 21448, 6172,  44136,
                3356,  40056, 33676, 27940, 19796, 51304, 13356, 2656,  43152,
                54964, 33660, 27792, 7540,  39548, 44668, 29456, 50688, 35380,
                55192, 19780, 40536, 26608, 400,   44492, 18468, 52996, 9848,
                57908, 36644, 6540,  2024,  24640, 35628, 9784,  61260, 2836,
                24952, 63788, 6560,  18492, 52132, 58172, 628,   56176, 7536,
                62096, 45380, 20448, 65300, 47476, 34312, 24576, 62520, 5548,
                21800, 11852, 43208, 52248, 552,   48020, 13204, 55508, 30920,
                1132,  20340, 35968, 46956, 27640, 41460, 55492, 352,   64356,
                53152, 17068, 7816,  47588, 27328, 14748, 52060, 23512, 37164,
                5580,  62948, 51736, 19264, 35068, 42916, 31784, 37864, 2768,
                23296, 28168, 65372, 50812, 13948, 31468, 5184,  19640, 10320,
                55408, 61308,
            },
            {
                53364, 27332, 48116, 37072, 20572, 52372, 24856, 13568, 42016,
                4424,  62812, 52760, 15952, 55596, 30204, 5216,  51628, 23188,
                17436, 45024, 5088,  19324, 46940, 60720, 10832, 49268, 36308,
                24216, 57832, 17056, 6800,  63004, 41616, 31592, 61688, 9372,
                46344, 19500, 50172, 57104, 21860, 53160, 61860, 18268, 7224,
                48096, 9752,  62916, 16912, 6316,  56080, 28580, 13728, 60928,
                49504, 16268, 44340, 56228, 32008, 14992, 43772, 53128, 31496,
                13504, 54588, 42896, 46564, 4008,  26244, 15520, 36320, 23936,
                12596, 53348, 9308,  37272, 3012,  17592, 51204, 56444, 30292,
                38780, 59432, 34624, 19004, 61008, 27368, 41948, 35052, 5288,
                25332, 59632, 9292,  17484, 63572, 11944, 31876, 25316, 45608,
                29012, 11016, 56448, 41288, 916,   64780, 8568,  42744, 60468,
                10768, 30944, 40884, 7876,  57700, 47968, 10536, 18012, 62368,
                34708, 7660,  38952, 824,   59924, 9036,  42684, 64136, 25288,
                39100, 84,
            },
            {
                18636, 42228, 64436, 7124,  17180, 39932, 9376,  58968, 33024,
                17612, 44424, 25944, 1052,  22240, 42568, 60604, 10428, 38228,
                58272, 31580, 36584, 64148, 23908, 30012, 17124, 55728, 26952,
                62708, 12204, 49968, 44576, 10540, 35176, 56608, 21640, 37528,
                14780, 64420, 2304,  36076, 10120, 25440, 33288, 12340, 57392,
                38220, 31420, 45312, 58056, 34612, 39404, 22888, 6980,  33756,
                29892, 21424, 64432, 12764, 40788, 58744, 22136, 6044,  47116,
                38156, 20564, 29088, 59656, 12860, 38852, 64220, 29292, 43780,
                17764, 32772, 25512, 55768, 28848, 59136, 13052, 41292, 1664,
                49872, 25568, 4792,  54976, 23768, 6752,  65184, 18084, 52016,
                48624, 40948, 29632, 53308, 2536,  38544, 60336, 20488, 13816,
                37232, 61616, 32160, 24096, 53808, 38388, 20192, 26788, 55144,
                34372, 21180, 53844, 15252, 23864, 464,   60080, 26052, 53552,
                45320, 15604, 21480, 54772, 18712, 28416, 48676, 57232, 12916,
                46696, 29668,
            },
            {
                59152, 9820,  23224, 56992, 30960, 61864, 46972, 1288,  53644,
                29384, 7064,  37188, 59460, 45976, 18128, 34060, 14688, 25668,
                48716, 7940,  55292, 13036, 51312, 38844, 4596,  32820, 46116,
                20116, 1872,  31032, 59264, 25488, 880,   16396, 49144, 27096,
                5072,  40676, 30192, 43580, 16044, 48960, 1936,  41312, 23252,
                27748, 14388, 3904,  21764, 50440, 10364, 63776, 47288, 43196,
                872,   52264, 26400, 8860,  47804, 3300,  27524, 64956, 16896,
                57300, 988,   50660, 9140,  33348, 19544, 52856, 47984, 4292,
                61152, 40080, 48932, 6196,  42420, 22248, 32456, 8060,  19696,
                44748, 16548, 9660,  40032, 30824, 45304, 11216, 57932, 38464,
                14680, 7276,  61752, 33848, 44160, 6084,  52212, 48516, 2960,
                42968, 50348, 6396,  17940, 35196, 12612, 50000, 5076,  17240,
                47048, 3108,  28332, 63680, 44776, 29820, 39964, 13396, 4836,
                41032, 49376, 58936, 32296, 44324, 36688, 2608,  22308, 32564,
                5712,  36088,
            },
            {
                50232, 2784,  32980, 46208, 3692,  14972, 35384, 23476, 20348,
                57728, 48956, 11740, 32168, 50840, 3492,  39388, 53276, 63516,
                2508,  41820, 16316, 28316, 728,   43492, 58484, 14164, 7804,
                52912, 42504, 37552, 22356, 54644, 47352, 29492, 7696,  54040,
                59208, 23540, 12428, 54712, 61328, 5840,  58628, 45724, 63996,
                6456,  54192, 60996, 42612, 30148, 15816, 53496, 25700, 19192,
                39164, 5864,  34808, 54408, 19528, 37044, 51420, 33548, 10608,
                41264, 26684, 35816, 62984, 23524, 42620, 6632,  10780, 21428,
                57612, 2108,  15684, 63088, 11060, 46800, 37700, 60564, 53912,
                28368, 63872, 57640, 50888, 14112, 36240, 21536, 1648,  28492,
                23468, 55700, 19768, 13004, 26360, 22948, 16204, 35036, 9004,
                27000, 22096, 58552, 3824,  44544, 57532, 29808, 62632, 39144,
                59396, 13124, 37428, 49956, 5368,  34224, 50792, 21036, 56212,
                27900, 9996,  3628,  25380, 11572, 62928, 16236, 40792, 52672,
                62264, 14540,
            },
            {
                24760, 39664, 53780, 12316, 42772, 27844, 51180, 7684,  63884,
                39336, 15148, 24368, 62324, 8752,  27504, 57276, 6024,  29832,
                21372, 33852, 62224, 46516, 9896,  35236, 21768, 64588, 39764,
                28692, 16112, 60872, 4656,  14060, 39580, 65348, 42872, 18212,
                51284, 35084, 46844, 19036, 26880, 38684, 30676, 20112, 16380,
                49984, 33648, 25052, 12260, 1416,  60024, 35908, 3720,  57092,
                13368, 59584, 45412, 30416, 62604, 15676, 7716,  44736, 23088,
                60948, 5372,  14860, 45896, 2760,  55364, 60408, 32004, 37972,
                51452, 28524, 35620, 20224, 52940, 752,   26284, 48464, 13968,
                35716, 2440,  33104, 47188, 3332,  60384, 53568, 47620, 33456,
                43720, 3640,  49488, 37772, 46392, 56420, 59312, 30616, 62688,
                55196, 40124, 14900, 64060, 47972, 24888, 2472,  32892, 9008,
                44328, 22864, 57192, 10260, 19512, 58312, 8384,  64644, 31500,
                17576, 61164, 34496, 52164, 47400, 6536,  55228, 26860, 8828,
                20600, 43548,
            },
            {
                28752, 63460, 17724, 21492, 60260, 38268, 56088, 11372, 43328,
                33868, 2264,  53044, 20084, 41588, 13300, 47908, 18752, 44352,
                11080, 51976, 25088, 19604, 60128, 53984, 25752, 49056, 3724,
                56592, 9608,  33932, 50996, 19564, 6420,  36060, 11812, 32636,
                3740,  9684,  63148, 764,   33704, 8500,  51880, 11392, 36244,
                2772,  40404, 47364, 18588, 38152, 45688, 8628,  31896, 48980,
                63360, 17048, 10176, 23344, 1308,  40236, 59376, 29552, 54812,
                18300, 48936, 53476, 30108, 39712, 17140, 26436, 46884, 14476,
                24680, 8332,  45672, 59860, 30904, 17400, 64576, 4632,  23824,
                42852, 11268, 22432, 18744, 29180, 42484, 16712, 9112,  62372,
                58664, 10932, 30480, 64172, 1284,  9992,  41060, 6552,  18304,
                45756, 11432, 28940, 33728, 19548, 10588, 43284, 15564, 52480,
                26280, 1424,  31924, 41968, 27040, 15660, 38428, 46272, 1368,
                36932, 43072, 22564, 15016, 38784, 30080, 60160, 34976, 49132,
                1440,  58180,
            },
            {
                10928, 6200,  35048, 49396, 4652,  25136, 16932, 47884, 28576,
                5244,  46456, 56636, 36328, 30716, 64696, 23952, 35688, 59608,
                37996, 56364, 4468,  40348, 6956,  30208, 17692, 11988, 36356,
                47420, 23436, 63764, 28088, 46068, 57516, 24232, 60920, 21420,
                45240, 28880, 39524, 56760, 48112, 22400, 42036, 61604, 55692,
                29284, 10092, 64836, 51680, 27984, 56244, 21068, 41880, 24044,
                28684, 37472, 42668, 50788, 56564, 25624, 48296, 12376, 1960,
                37520, 32260, 9516,  20888, 58012, 12232, 35996, 164,   64416,
                41980, 56084, 12504, 23152, 39904, 44056, 34376, 9284,  56372,
                61404, 38700, 53144, 7740,  63352, 24180, 39020, 4808,  26156,
                17760, 35244, 53096, 15516, 27996, 20268, 50020, 36912, 25072,
                1812,  52268, 37908, 7852,  54308, 40480, 61696, 21616, 48288,
                65388, 17788, 55100, 47236, 62576, 3420,  54180, 24560, 12544,
                48668, 53212, 7996,  64392, 444,   23692, 17976, 12532, 45028,
                31440, 37724,
            },
            {
                15832, 46536, 57356, 30576, 9420,  65196, 312,   32340, 19708,
                61372, 26296, 17348, 6468,  44688, 688,   53908, 8120,  15996,
                1756,  48212, 13256, 32596, 50828, 44588, 1504,  61852, 31820,
                18640, 43296, 148,   12328, 41712, 30896, 2412,  48372, 55108,
                14872, 59720, 24516, 17024, 12920, 53824, 4568,  26100, 44816,
                21664, 57792, 7320,  23376, 5084,  62516, 16592, 11876, 4704,
                52696, 2660,  20456, 32028, 14016, 4824,  34272, 20812, 43532,
                62172, 25384, 42116, 65444, 7008,  49228, 29592, 52488, 19872,
                4608,  33656, 49768, 3096,  7292,  58132, 14936, 50612, 30460,
                17828, 46080, 27208, 58336, 35284, 12076, 56292, 32188, 50216,
                41040, 21940, 5948,  42752, 61056, 33420, 54292, 13560, 63924,
                32412, 20420, 60752, 49280, 800,   27324, 56092, 37108, 7144,
                34144, 40108, 5776,  13580, 34980, 21544, 42692, 29600, 61732,
                5912,  27104, 19492, 56820, 46328, 41648, 51272, 3952,  63844,
                19480, 55384,
            },
            {
                52352, 2564,  26528, 40172, 44608, 52196, 36372, 58504, 54216,
                12856, 39972, 51244, 10704, 58028, 21436, 39732, 32420, 26620,
                64064, 29092, 22176, 61484, 36896, 15104, 57880, 40956, 52076,
                5692,  26696, 59012, 37032, 53872, 16820, 10128, 26776, 40528,
                7344,  36824, 3100,  43672, 65004, 29884, 35360, 14324, 1584,
                17408, 37692, 32916, 14640, 43944, 35092, 49796, 54828, 33140,
                46496, 58924, 7984,  61452, 46148, 38280, 64068, 52332, 6352,
                56824, 13888, 2584,  35000, 24000, 44000, 61424, 9940,  38376,
                59148, 16608, 29428, 63468, 37024, 53484, 27804, 21156, 41044,
                6648,  76,    15068, 49544, 3688,  44984, 52140, 14788, 64884,
                744,   46824, 57680, 38636, 7760,  23760, 43768, 4668,  47348,
                56248, 6040,  43128, 23792, 16996, 31968, 13628, 3676,  28916,
                11716, 59032, 24176, 30264, 52176, 59476, 9908,  17000, 33716,
                56076, 39760, 13464, 32232, 35964, 9448,  58872, 26376, 39488,
                7784,  24364,
            },
            {
                42068, 20840, 62952, 12008, 18456, 23316, 14396, 42128, 8696,
                21972, 34996, 63052, 29204, 15096, 49200, 60984, 12188, 51720,
                40796, 44972, 17480, 8576,  55132, 23704, 27824, 10056, 21852,
                55812, 14008, 49008, 8180,  20672, 61880, 51184, 33664, 63724,
                19188, 53020, 32184, 50096, 6776,  19532, 59044, 46856, 63480,
                50280, 42416, 53724, 59796, 30008, 72,    25776, 38312, 65304,
                14760, 26836, 35944, 54044, 18232, 28172, 8980,  23968, 16248,
                29040, 46276, 51076, 17852, 54900, 3948,  15844, 22528, 46108,
                25928, 53988, 42992, 19644, 24628, 10972, 2372,  47416, 54744,
                36428, 65448, 32704, 41448, 19112, 30000, 6972,  20956, 36804,
                29300, 12908, 25284, 49072, 18568, 11304, 58820, 29476, 16540,
                39484, 26828, 12880, 35820, 57340, 45504, 60380, 50800, 19260,
                53784, 42832, 48904, 208,   37564, 7208,  45720, 50948, 2892,
                22904, 44548, 59588, 4672,  28496, 21048, 54264, 15076, 33980,
                48480, 61172,
            },
            {
                31664, 36668, 7384,  50508, 34016, 60900, 5832,  29584, 50060,
                2416,  45656, 4764,  42952, 25392, 34412, 3104,  46608, 19812,
                5420,  58216, 34920, 2088,  42300, 48440, 6472,  65228, 38876,
                44700, 34492, 63080, 24376, 32384, 4168,  38096, 44040, 928,
                12812, 46316, 26180, 10744, 41252, 37948, 23484, 8960,  31488,
                27064, 11976, 3836,  18988, 47548, 11040, 60680, 6460,  19680,
                9636,  40700, 22276, 920,   11752, 42920, 48816, 59640, 40644,
                32832, 10812, 60652, 38724, 28008, 33240, 41028, 56672, 31368,
                5480,  9200,  13416, 48504, 61380, 44952, 33740, 60052, 12548,
                25268, 57240, 22596, 10100, 62740, 25744, 59408, 42844, 9536,
                60480, 55012, 2996,  30912, 64480, 52100, 80,    35248, 21476,
                61976, 9792,  51656, 64520, 4572,  8596,  41360, 24496, 38504,
                63856, 8268,  20836, 16752, 63300, 25100, 40208, 19820, 65052,
                36720, 10576, 49944, 16880, 61940, 47624, 6032,  43904, 29472,
                1036,  12808,
            },
            {
                17360, 45824, 58392, 28448, 1852,  55868, 47032, 38296, 64304,
                24476, 31760, 59960, 19140, 38356, 7432,  55424, 29900, 24500,
                10104, 53120, 26028, 63556, 11952, 29932, 19896, 33160, 3656,
                16544, 28704, 1932,  40100, 47636, 57952, 15056, 22472, 55792,
                30056, 58304, 21748, 61872, 54484, 372,   51596, 56776, 40560,
                6344,  60852, 24060, 36196, 56644, 40820, 21904, 43320, 30492,
                57572, 50344, 44692, 63588, 30772, 57024, 3408,  19912, 36540,
                428,   56000, 21924, 8364,  47892, 63952, 12084, 1468,  50412,
                60452, 34544, 40260, 996,   30992, 16900, 39440, 4840,  19488,
                30084, 8084,  51308, 46568, 38276, 54376, 2788,  48040, 23940,
                18284, 44384, 34616, 15108, 37368, 45712, 25716, 41196, 49948,
                2540,  31508, 46656, 18720, 29508, 21696, 34356, 47596, 2276,
                14200, 32052, 44976, 55708, 28016, 12104, 57520, 31732, 14736,
                54088, 30072, 1112,  42388, 24812, 37836, 11208, 65480, 22420,
                51472, 57000,
            },
            {
                25800, 4024,  39316, 15412, 43376, 20144, 26036, 11264, 17616,
                56516, 14532, 53820, 9876,  51496, 65484, 16804, 42064, 62000,
                37608, 14948, 31528, 49972, 38400, 60100, 45616, 54500, 49772,
                11236, 57320, 51724, 18192, 12092, 29112, 5628,  25616, 49368,
                7944,  36384, 4256,  15116, 33588, 28072, 12308, 16948, 34692,
                20712, 46224, 51332, 28600, 8020,  14272, 53424, 48576, 2256,
                16076, 33812, 6116,  24976, 51464, 34704, 14408, 26772, 64776,
                49720, 43404, 25516, 4912,  14740, 52244, 20512, 27180, 37580,
                18692, 65112, 23424, 56424, 50896, 8720,  26784, 63532, 49304,
                43808, 37304, 17256, 1572,  13276, 34388, 15956, 31536, 40372,
                53076, 7604,  62720, 20852, 5788,  55736, 13792, 8728,  59604,
                54068, 15164, 37640, 42200, 55128, 11560, 61800, 16412, 57676,
                26948, 36228, 60992, 4988,  50312, 34420, 1900,  47696, 5268,
                26632, 60784, 20636, 52816, 33200, 2964,  55496, 18556, 40880,
                36084, 9564,
            },
            {
                61440, 21548, 54496, 63772, 8872,  32256, 52416, 4356,  41684,
                35456, 48064, 868,   28472, 23028, 44456, 12556, 33528, 0,
                56724, 47124, 3984,  22632, 17888, 5524,  13908, 25508, 61628,
                37456, 22988, 6888,  45564, 60664, 42184, 53480, 65140, 39856,
                17264, 60772, 42520, 47856, 18996, 64164, 44120, 48864, 2444,
                62884, 15440, 38380, 1496,  64552, 32824, 25116, 35844, 63200,
                27776, 55084, 13224, 38976, 17756, 7928,  41932, 54160, 6988,
                12788, 18292, 31616, 62500, 35840, 41988, 58536, 7556,  44388,
                15424, 46632, 28392, 6820,  35892, 58756, 22052, 53716, 15316,
                3744,  62292, 55164, 28424, 58956, 22032, 49808, 61916, 5020,
                12296, 26132, 50236, 42252, 28356, 61268, 33156, 18952, 44464,
                28080, 23288, 3780,  58656, 25580, 536,   51836, 30076, 6464,
                52792, 10896, 22680, 39556, 15484, 43564, 61536, 23576, 41132,
                8812,  46404, 38848, 12496, 63116, 15908, 48984, 27128, 59072,
                5060,  47220,
            },
            {
                29008, 33480, 12384, 49224, 24012, 37104, 58064, 61488, 27812,
                7300,  21464, 39808, 57940, 36728, 3508,  59260, 48532, 21944,
                18396, 41600, 8440,  58704, 52576, 43424, 35812, 516,   19256,
                41656, 31652, 63868, 26792, 35960, 9256,  34068, 19692, 11376,
                45620, 32444, 26660, 8820,  56208, 3760,  36596, 24984, 58004,
                29572, 9960,  55168, 44208, 17732, 57732, 5412,  11648, 45796,
                18796, 61004, 2988,  47640, 58512, 62464, 23204, 46508, 29576,
                58088, 39444, 53260, 45480, 28644, 3364,  23508, 33924, 54704,
                10512, 3276,  52516, 20016, 14524, 42580, 46552, 11108, 34568,
                41148, 23724, 32312, 45292, 5724,  39680, 10604, 27476, 56980,
                33792, 38180, 58792, 3792,  10800, 48196, 24076, 38936, 5188,
                63832, 34752, 8000,  49448, 14264, 46092, 36752, 43588, 19448,
                40848, 48068, 3048,  54604, 30948, 9920,  18904, 55528, 50652,
                34872, 17552, 57760, 7460,  28656, 44344, 35308, 8276,  31848,
                14644, 53008,
            },
            {
                38532, 1544,  42384, 6056,  16844, 46556, 1180,  13776, 44496,
                51092, 63608, 12000, 46368, 18264, 31884, 26584, 7184,  53464,
                29020, 35344, 62660, 27264, 32776, 10448, 56948, 29432, 47452,
                9076,  3136,  53888, 20880, 15728, 1240,  50928, 30820, 4832,
                57048, 1752,  23036, 52208, 39520, 31100, 21648, 7828,  53192,
                42740, 33500, 22468, 26416, 49840, 40324, 20984, 52368, 38656,
                8656,  42224, 21728, 28608, 10484, 32212, 37924, 5300,  21072,
                34916, 9652,  1612,  59724, 11140, 17044, 47544, 62168, 29980,
                40620, 59420, 33000, 39064, 62588, 5708,  30488, 544,   27604,
                58156, 7552,  12120, 19196, 51888, 64876, 44104, 908,   19776,
                46616, 15820, 22512, 31824, 17648, 52600, 1500,  56332, 12172,
                47456, 20304, 60940, 39936, 32176, 65152, 22424, 9532,  62996,
                33076, 60044, 17472, 26348, 64804, 6556,  37740, 28960, 13208,
                63560, 22536, 31904, 3680,  50104, 58508, 22228, 168,   62244,
                43688, 19236,
            },
            {
                64668, 51064, 25404, 60828, 29940, 40400, 19740, 34336, 23120,
                30696, 16216, 33632, 5608,  62392, 50056, 14784, 39140, 64120,
                10876, 45632, 13772, 2036,  40556, 24068, 64528, 51008, 14852,
                58908, 43776, 12524, 39080, 59536, 56308, 43116, 22352, 62832,
                38132, 48704, 14084, 62356, 11764, 45816, 59704, 14824, 39940,
                4660,  13232, 61632, 3556,  8912,  29640, 62204, 564,   31836,
                24172, 50576, 35132, 54372, 45248, 784,   16096, 52432, 60820,
                48456, 15672, 24668, 20280, 50152, 56688, 38616, 13388, 584,
                22440, 49248, 26340, 11488, 54032, 24124, 37352, 64936, 51404,
                20724, 47684, 61360, 35368, 25936, 16416, 36772, 30736, 50684,
                63452, 2736,  54472, 45432, 40540, 65420, 36128, 30312, 42552,
                16300, 51688, 28892, 10748, 18000, 5592,  27772, 54028, 1552,
                24460, 13084, 44392, 35676, 49032, 22092, 46056, 60240, 580,
                5916,  44756, 56072, 25460, 41024, 19072, 12924, 39636, 54876,
                23740, 10272,
            },
            {
                45660, 14160, 35316, 57312, 52668, 10828, 65180, 55468, 9236,
                60312, 2948,  52748, 40984, 24412, 8876,  57404, 43100, 4340,
                24860, 51712, 19936, 55600, 48652, 16824, 4224,  21336, 36920,
                25808, 33032, 49860, 28832, 4140,  24764, 47724, 12672, 28036,
                16776, 54428, 35300, 29408, 5988,  18028, 49880, 27284, 65380,
                20164, 51040, 45392, 37288, 59084, 15784, 42600, 12984, 56380,
                60212, 4304,  14624, 65132, 19584, 25984, 57176, 42604, 3092,
                27108, 62796, 43968, 36164, 32024, 6052,  26208, 51912, 18912,
                64160, 7100,  16968, 44788, 3852,  48608, 18600, 56284, 16048,
                38908, 2476,  42572, 56460, 3936,  9240,  54600, 23292, 13828,
                41664, 8768,  28864, 59984, 6300,  14352, 20764, 8212,  58248,
                25096, 1988,  54724, 36048, 56976, 41900, 49776, 15280, 38820,
                56744, 29824, 7776,  58292, 2048,  52876, 33220, 16756, 25880,
                39148, 52336, 15408, 36580, 9604,  64444, 52028, 47096, 26812,
                36524, 3440,
            },
            {
                56192, 18340, 8376,  21264, 3572,  27456, 48796, 5532,  39008,
                47496, 26644, 56684, 20532, 29692, 54048, 35760, 17684, 31116,
                59660, 38008, 6604,  30732, 61176, 34484, 45264, 7640,  55360,
                62544, 17632, 6340,  45860, 64344, 18184, 36760, 6816,  60188,
                44476, 8048,  20060, 42160, 57684, 37308, 820,   32092, 10616,
                35744, 56712, 17488, 31376, 48848, 23064, 34268, 46588, 26496,
                17144, 37588, 30516, 6236,  40388, 49660, 34240, 12904, 31104,
                8372,  40272, 55000, 4496,  64920, 42488, 9104,  45260, 32568,
                55968, 42216, 35448, 57244, 61084, 32040, 12824, 43476, 8400,
                29748, 24780, 14752, 31232, 48432, 40864, 60760, 6868,  58440,
                25464, 37620, 11840, 35100, 24352, 50164, 27612, 46152, 63028,
                33516, 40892, 7128,  45748, 20928, 3208,  59820, 35160, 47276,
                4440,  51112, 41512, 20140, 14704, 40288, 10356, 55052, 42880,
                30224, 11604, 48284, 61100, 2136,  29464, 33112, 4740,  16176,
                59828, 31388,
            },
            {
                28100, 40996, 47960, 32224, 43616, 36616, 15380, 24768, 42904,
                17984, 35532, 10016, 44132, 1508,  13152, 46184, 21804, 2392,
                49320, 15884, 43472, 22444, 9764,  38984, 13260, 30112, 41036,
                1064,  23472, 53220, 35180, 10212, 31880, 52300, 40992, 36,
                32268, 23832, 64768, 2800,  25232, 51576, 61324, 43452, 47824,
                24256, 2560,  27868, 6588,  11360, 54788, 2744,  64240, 8108,
                48952, 53588, 43960, 12044, 60500, 8900,  22832, 63684, 46096,
                19328, 51024, 12144, 29216, 14636, 22244, 59520, 36604, 4204,
                24604, 14428, 29400, 8988,  21580, 26676, 1832,  34464, 50144,
                60372, 53284, 10772, 64092, 21248, 28004, 17932, 45772, 31964,
                53052, 48972, 18352, 57184, 408,   61192, 38272, 10912, 4864,
                50664, 14632, 23532, 62304, 12492, 31304, 25032, 9924,  18200,
                21888, 62624, 11136, 32416, 63720, 28092, 4184,  59200, 19416,
                64712, 7244,  33744, 18476, 24344, 45568, 56828, 11336, 21000,
                49620, 6408,
            },
            {
                23280, 63184, 960,   58648, 12592, 61116, 54388, 32744, 360,
                58136, 13916, 50288, 64860, 37816, 58972, 6412,  62120, 33880,
                56520, 27656, 65192, 848,   53344, 58520, 27396, 52096, 46832,
                11192, 60600, 14448, 42468, 21604, 56024, 14928, 25768, 57248,
                49468, 11112, 53184, 47040, 34048, 9388,  22224, 16264, 7092,
                58868, 41236, 52764, 63652, 38708, 44112, 19200, 28792, 40276,
                21240, 1620,  23856, 55972, 27660, 36628, 4412,  16828, 54404,
                37848, 23212, 32,    57360, 47280, 17740, 28364, 54232, 11456,
                48260, 60552, 2432,  37828, 47516, 52924, 39608, 62852, 21132,
                4348,  37068, 44944, 33920, 92,    51268, 12576, 35524, 2132,
                5252,  64608, 21444, 44720, 32424, 41952, 16916, 55256, 30980,
                19440, 37292, 57692, 28148, 51988, 44096, 64152, 55268, 29124,
                45228, 36864, 26120, 54228, 45628, 22876, 50392, 35232, 47344,
                2728,  21932, 54244, 41668, 59392, 6644,  37432, 43340, 63564,
                39052, 53648,
            },
            {
                9472,  15712, 37644, 50656, 23848, 18560, 7408,  51384, 63040,
                29368, 22612, 5176,  31764, 19376, 27508, 48368, 24224, 41912,
                7900,  11624, 36484, 47404, 25204, 18688, 3072,  64004, 19964,
                33824, 26492, 39432, 58836, 1880,  48524, 8740,  62964, 19468,
                35544, 39780, 16040, 28180, 13488, 55748, 40296, 29340, 53804,
                12836, 33216, 21144, 14832, 25212, 59456, 5068,  51676, 35848,
                10836, 58776, 33272, 46796, 14828, 52296, 41636, 58268, 28024,
                6292,  34588, 61792, 32328, 38920, 50560, 2064,  62100, 20576,
                39400, 52536, 44224, 19016, 63928, 14820, 6804,  45700, 12072,
                27480, 18432, 58828, 6256,  55140, 38108, 63096, 22332, 43124,
                15864, 39796, 30036, 7652,  13440, 53376, 3188,  22756, 59804,
                43736, 1172,  47976, 8884,  16348, 34248, 188,   40060, 6736,
                14224, 57076, 1740,  5716,  16600, 37968, 7932,  13784, 25080,
                57556, 38520, 27544, 13016, 50220, 16524, 22716, 28596, 1352,
                13344, 34148,
            },
            {
                57456, 42524, 27100, 4952,  30868, 45384, 39192, 20604, 11724,
                37380, 45200, 60656, 41172, 15868, 52348, 10368, 38584, 14912,
                54396, 20592, 32288, 60148, 14116, 42760, 49604, 5656,  37784,
                57308, 8032,  50108, 4916,  27600, 30700, 38172, 45368, 3580,
                29516, 5928,  58228, 44196, 4868,  19088, 64236, 3540,  36856,
                62428, 45904, 932,   50008, 35156, 9440,  31944, 56532, 14064,
                62640, 41996, 17968, 3328,  64368, 20748, 31792, 1540,  48328,
                60284, 15556, 45084, 25412, 10460, 6788,  41728, 33668, 16260,
                30444, 5696,  27240, 11000, 32620, 57888, 24544, 30428, 56804,
                42220, 48300, 23744, 16140, 30140, 9544,  47240, 57420, 27548,
                61264, 10680, 56096, 46716, 26044, 63292, 35208, 48412, 26992,
                12248, 64736, 32376, 39552, 5196,  19900, 49244, 23404, 52568,
                61592, 33360, 42232, 48408, 59920, 30424, 62200, 41944, 32352,
                9244,  44908, 62440, 480,   31252, 53420, 8928,  61348, 48164,
                18580, 45220,
            },
            {
                3560,  20876, 61560, 54848, 10360, 64324, 2260,  27228, 48156,
                55080, 3400,  25704, 7924,  56008, 33392, 548,   63824, 30004,
                50644, 45004, 4560,  40156, 7428,  35056, 30884, 24036, 15772,
                44412, 31716, 18036, 65320, 43896, 16308, 60944, 23308, 13128,
                54056, 63736, 22344, 37136, 59980, 31136, 48988, 44716, 26160,
                17376, 8572,  29148, 61388, 42112, 47500, 17092, 22892, 45188,
                26692, 6760,  29904, 38940, 50520, 7568,  44656, 25216, 12416,
                40048, 8240,  51744, 19204, 63784, 56612, 22868, 46104, 8356,
                65396, 55836, 22640, 50756, 512,   41452, 17304, 53504, 2812,
                32896, 8584,  63544, 39720, 44080, 25952, 19056, 7424,  32988,
                50340, 24396, 36412, 5008,  51116, 19608, 9616,  40776, 6232,
                52992, 17676, 25344, 53940, 61984, 42688, 59224, 11732, 30636,
                17284, 9132,  24492, 19428, 12408, 52524, 1048,  56304, 17908,
                51172, 15004, 35972, 6020,  46664, 39604, 34552, 56040, 25560,
                32172, 52048,
            },
        },
        {
            {
                32784, 3024,  35652, 44564, 60444, 27320, 32940, 9212,  58696,
                29640, 5764,  48632, 33516, 23500, 1020,  31556, 39752, 57536,
                24184, 5516,  32048, 22136, 55712, 40576, 24468, 32668, 3820,
                36776, 7472,  33976, 61796, 40224, 5132,  56276, 34804, 9056,
                59024, 52760, 5012,  27020, 43200, 60392, 33908, 47000, 37660,
                62692, 12128, 57872, 24772, 52544, 21544, 348,   47916, 23460,
                29784, 50464, 63548, 7136,  37888, 47304, 56032, 20020, 5240,
                15872, 28656, 22072, 18644, 4244,  40068, 46440, 7304,  31164,
                60024, 19404, 34516, 55200, 24840, 8044,  42468, 62868, 10748,
                5164,  17888, 48732, 1372,  43872, 35688, 3688,  22716, 57412,
                28124, 12,    13640, 36384, 9252,  32232, 14632, 21132, 28380,
                8308,  24788, 34580, 51368, 47180, 61068, 16,    36752, 47808,
                5452,  62732, 49472, 37640, 29100, 18528, 55048, 3824,  49580,
                27860, 492,   31600, 21188, 63392, 14112, 39656, 54236, 6680,
                50776, 15124,
            },
            {
                28972, 63524, 18200, 7652,  40616, 2452,  16832, 37272, 46032,
                25048, 52896, 15812, 42936, 58308, 11980, 47260, 7856,  19484,
                44160, 64828, 52568, 15212, 60632, 912,   47384, 9964,  20016,
                41352, 52904, 13252, 18352, 44724, 11116, 22500, 16368, 42580,
                24896, 18492, 40360, 36320, 20524, 8056,  57696, 24964, 5772,
                54072, 33236, 16220, 39084, 9984,  34512, 28044, 40916, 58108,
                17904, 13764, 45060, 54676, 16904, 29176, 63012, 2600,  36120,
                44840, 59968, 56736, 32024, 53944, 10384, 51048, 20572, 13428,
                3436,  48324, 15092, 2072,  30840, 47456, 33132, 50772, 28160,
                59164, 36504, 31960, 57120, 11920, 29448, 65340, 8616,  51532,
                16396, 41596, 30368, 49720, 23648, 46240, 53508, 40932, 37024,
                65204, 15412, 43380, 20212, 6984,  27552, 40732, 17424, 11740,
                20856, 32832, 14516, 2808,  52644, 21960, 10860, 42660, 15608,
                36692, 52752, 60096, 10904, 36280, 49772, 23424, 908,   59144,
                37484, 43532,
            },
            {
                22052, 46392, 13116, 54420, 29964, 63148, 55828, 42136, 172,
                63964, 32156, 10704, 7224,  37460, 21092, 54944, 62264, 34504,
                12736, 38992, 8728,  45252, 18860, 37060, 29780, 14568, 65224,
                26500, 46508, 59132, 24340, 30672, 64604, 49404, 28548, 37732,
                50700, 804,   12704, 64104, 50056, 14760, 30508, 51332, 10312,
                22456, 45152, 1660,  63716, 46428, 55204, 15668, 5896,  52136,
                36396, 60476, 4712,  40668, 21616, 8880,  42952, 32688, 23196,
                51584, 12208, 2200,  43252, 14572, 23900, 34300, 61784, 41876,
                36620, 63664, 43560, 22192, 61976, 5420,  19716, 14740, 2348,
                41120, 21272, 7768,  24396, 50228, 19748, 46548, 40028, 33816,
                62116, 20828, 54064, 59256, 18032, 63204, 1048,  10976, 56200,
                46856, 31424, 1516,  58572, 13800, 33748, 54600, 24556, 59388,
                42940, 55932, 9500,  40996, 61024, 33740, 58452, 25628, 61776,
                22748, 13072, 47728, 18348, 3428,  44132, 62012, 33916, 27664,
                12164, 56000,
            },
            {
                1488,  59664, 25540, 38064, 47880, 5324,  20624, 49824, 12228,
                18300, 39888, 55704, 61648, 25472, 45136, 16348, 4428,  26392,
                48812, 28920, 56656, 4928,  25864, 49616, 54464, 58420, 43840,
                5656,  16508, 1944,  50492, 8104,  36208, 3200,  58212, 7228,
                61968, 31932, 55952, 23416, 45356, 3476,  38844, 18692, 65376,
                41668, 29816, 59208, 19676, 6868,  31384, 61900, 43972, 20708,
                2656,  31244, 24548, 34552, 49084, 57300, 12936, 53592, 60848,
                8036,  39736, 47644, 26396, 63212, 38884, 1428,  55420, 28360,
                24256, 8468,  51516, 39316, 12668, 57348, 44864, 38224, 64944,
                54820, 47128, 15768, 62376, 38296, 54628, 2084,  14116, 26624,
                4280,  44896, 7460,  38756, 5020,  29252, 34424, 26272, 17128,
                6188,  52692, 38520, 23788, 63332, 44916, 8920,  50968, 3516,
                29248, 36112, 51276, 27128, 16744, 44340, 1520,  39144, 7468,
                55644, 33280, 40056, 25788, 57476, 30676, 8160,  20508, 17212,
                48860, 9020,
            },
            {
                40820, 51000, 33572, 10140, 22868, 15132, 34648, 28532, 59112,
                23712, 4716,  48056, 28812, 2152,  52056, 30356, 41408, 59372,
                408,   20924, 33800, 41968, 63684, 12008, 2644,  34688, 21180,
                29132, 55916, 33524, 39472, 19388, 54748, 12872, 44456, 21128,
                15216, 46976, 29000, 6548,  33488, 53084, 61320, 27428, 13728,
                36136, 4656,  50512, 26772, 37096, 13200, 49444, 25884, 9564,
                56344, 47204, 11396, 64012, 1212,  27508, 37244, 18292, 25844,
                30580, 16800, 35036, 6424,  58364, 19180, 49460, 16380, 4876,
                57948, 32112, 17464, 27452, 54200, 35308, 24516, 8400,  30524,
                11664, 26068, 43044, 4796,  32956, 9724,  59588, 23376, 48648,
                58096, 11272, 35192, 25308, 13276, 57208, 44056, 50532, 22432,
                62016, 28068, 11528, 49228, 4896,  16792, 30732, 38388, 65028,
                7668,  18720, 23428, 63168, 6260,  12828, 48608, 28652, 19608,
                45820, 2240,  63968, 6016,  51800, 15288, 38972, 53316, 42444,
                65216, 35396,
            },
            {
                16052, 19752, 6312,  61176, 53352, 44680, 62332, 8508,  52164,
                36300, 44248, 20184, 34012, 13888, 64176, 35700, 9792,  52928,
                14488, 61840, 50824, 16636, 31356, 22916, 40432, 8440,  52372,
                38360, 10848, 63488, 43504, 27132, 60844, 32468, 40048, 25836,
                53780, 10620, 41608, 58860, 17104, 9804,  42864, 104,   46808,
                55260, 8968,  17364, 43348, 56864, 22904, 1252,  38656, 62916,
                28584, 42284, 18960, 51520, 15716, 59296, 6304,  46628, 3568,
                49872, 62184, 21060, 52288, 29700, 8956,  32828, 44212, 11788,
                53268, 46688, 1592,  10644, 49088, 3020,  60000, 17360, 51564,
                34420, 552,   60340, 52492, 27512, 17328, 42524, 36992, 53128,
                31428, 16848, 64724, 48000, 52652, 40008, 8576,  59924, 3540,
                41988, 19360, 55592, 35412, 41132, 56980, 21704, 48244, 14884,
                45296, 58332, 632,   46876, 38212, 53756, 32256, 65528, 51052,
                10400, 29868, 42876, 21828, 28052, 46148, 60584, 3684,  24732,
                5740,  30252,
            },
            {
                26852, 57320, 43040, 30908, 948,   39788, 25188, 3624,  31488,
                14896, 57108, 9608,  50732, 40788, 6068,  23340, 19184, 44372,
                38056, 24756, 10420, 3744,  46112, 59800, 27944, 18088, 61992,
                47556, 3544,  22288, 14384, 52264, 264,   17116, 48468, 4180,
                63136, 35924, 1744,  21576, 37572, 25292, 56972, 31576, 21260,
                59680, 25116, 34248, 52980, 10560, 40984, 58676, 17092, 34812,
                14220, 54040, 6896,  32768, 39252, 44348, 22436, 65408, 40548,
                10916, 55688, 744,   37408, 13240, 46396, 65048, 22876, 40544,
                19692, 60656, 37180, 64524, 20988, 29392, 41432, 47872, 21984,
                56920, 39764, 19408, 13304, 45384, 64004, 29900, 7220,  19996,
                936,   43264, 28480, 22212, 2512,  19656, 15612, 30940, 36548,
                13912, 45864, 31904, 2340,  26532, 60440, 11892, 1896,  27464,
                33624, 41240, 11572, 30396, 57152, 17756, 22600, 4468,  15160,
                57308, 37468, 16936, 54424, 12384, 35264, 10112, 32308, 57860,
                45304, 54836,
            },
            {
                2468,  62948, 13548, 49688, 16960, 56800, 11612, 48224, 41824,
                65148, 1312,  27628, 60164, 17912, 46208, 58292, 55212, 31704,
                7084,  47364, 58068, 35784, 55052, 7256,  51088, 44356, 13436,
                25388, 31776, 58952, 6604,  46648, 23708, 9248,  57220, 34044,
                19544, 50972, 27344, 44292, 64296, 48440, 12812, 6748,  40460,
                49340, 14524, 2728,  64580, 28508, 5136,  32524, 51212, 46024,
                4116,  21864, 60332, 24904, 9384,  29496, 52784, 34664, 14640,
                32220, 44944, 24176, 42008, 56496, 27076, 5712,  59244, 35436,
                29804, 7536,  24736, 43064, 33812, 15440, 7000,  62408, 4044,
                27836, 9460,  48720, 36420, 22504, 2576,  56108, 12720, 61948,
                40160, 50160, 8964,  60872, 32424, 46492, 63848, 23812, 49424,
                58688, 6792,  64756, 9552,  18304, 43312, 52884, 36936, 61760,
                54880, 21036, 50636, 25160, 35724, 8456,  60032, 40708, 34360,
                24408, 61112, 8316,  48160, 62708, 164,   50304, 22736, 14288,
                37644, 11376,
            },
            {
                47320, 38860, 23928, 35516, 28236, 20380, 37012, 59964, 17664,
                22368, 38928, 53928, 24444, 3808,  37092, 12820, 27728, 3060,
                65368, 17484, 29076, 39408, 14000, 20764, 32720, 572,   36672,
                57224, 17292, 42040, 35664, 30124, 38268, 65044, 28676, 42544,
                7020,  12004, 60980, 15472, 3600,  54444, 34940, 19072, 63360,
                37872, 30156, 44948, 21584, 47648, 60944, 20276, 8240,  25284,
                64384, 40724, 36104, 49584, 55604, 12496, 1644,  20176, 57800,
                27764, 8180,  18372, 62904, 4072,  16008, 38572, 51868, 28,
                14844, 45592, 50432, 4304,  54764, 58372, 12816, 36084, 43364,
                16140, 63676, 30620, 58876, 5544,  50480, 34472, 45972, 23848,
                55168, 14136, 36500, 56364, 5572,  37956, 53536, 312,   10728,
                25900, 39912, 53204, 22508, 49800, 33272, 5624,  23996, 16596,
                8588,  4392,  63800, 14652, 2876,  43592, 27876, 48008, 12124,
                45176, 3800,  32168, 20092, 38316, 25416, 43700, 18248, 63564,
                52232, 21388,
            },
            {
                32812, 53920, 4860,  9320,  64360, 45932, 7308,  53600, 29800,
                6244,  45440, 10908, 32980, 49116, 63328, 8732,  50308, 34680,
                42112, 22216, 51904, 1572,  26484, 61020, 41380, 64860, 6012,
                53484, 49504, 9568,  62388, 55140, 12348, 50564, 21340, 14520,
                55416, 39432, 46736, 32608, 8544,  29268, 23328, 51256, 1832,
                10688, 60408, 7296,  35568, 15976, 12044, 42536, 55780, 29576,
                12992, 32,    45156, 5732,  17504, 62956, 41460, 47140, 5192,
                61260, 53136, 35256, 49356, 31308, 21512, 48036, 9876,  26060,
                62556, 56984, 11276, 17772, 39832, 26312, 51816, 31844, 46600,
                23844, 53472, 7352,  42020, 25744, 15700, 39232, 10004, 27448,
                33104, 3408,  18276, 26440, 42432, 12840, 28148, 43960, 33416,
                56848, 16548, 29256, 37820, 14304, 63052, 30012, 46192, 40312,
                49120, 31784, 44584, 38548, 52676, 62464, 20540, 55800, 776,
                52456, 27204, 41620, 56192, 4960,  30488, 59432, 40328, 3332,
                28964, 8448,
            },
            {
                44252, 18660, 58792, 41992, 51524, 2092,  33596, 26304, 13208,
                50876, 35424, 62504, 14756, 28932, 21084, 43948, 24860, 15656,
                55896, 11796, 62600, 43308, 9532,  48800, 16412, 24092, 11588,
                29048, 21624, 1968,  26344, 19208, 4764,  44692, 1496,  59552,
                30680, 22980, 52576, 18244, 59288, 41268, 46164, 58240, 27372,
                17680, 54088, 24064, 57400, 38976, 52224, 3084,  36984, 48964,
                58268, 32104, 19976, 27888, 57516, 31532, 36792, 26104, 50756,
                16700, 38936, 2316,  11308, 58916, 43572, 33936, 55020, 18912,
                41972, 33240, 22452, 30956, 61456, 880,   20488, 9688,  56360,
                1904,  38504, 12404, 32844, 52736, 18980, 57092, 65128, 4736,
                47852, 60312, 51852, 21244, 9656,  58900, 17176, 62588, 5296,
                20552, 47572, 3712,  44976, 55660, 524,   58360, 10968, 19904,
                60152, 26564, 56096, 18904, 9988,  32744, 6164,  16016, 36596,
                59668, 17728, 64504, 13032, 50980, 15896, 54568, 6936,  48992,
                36160, 61424,
            },
            {
                568,   27112, 12060, 31444, 21812, 15192, 61244, 43160, 58168,
                4076,  20236, 55792, 42556, 296,   39260, 54068, 61284, 2100,
                37520, 5532,  33384, 18872, 57488, 30788, 35560, 54844, 46376,
                38740, 33556, 44144, 15452, 40156, 61040, 34736, 25772, 47748,
                36712, 6120,  2180,  26184, 37700, 11472, 4224,  14816, 43892,
                32992, 41072, 48252, 560,   30976, 26740, 61896, 23040, 16864,
                9348,  53092, 62400, 14564, 48060, 3464,  22928, 8596,  12800,
                42808, 22408, 28688, 54440, 24988, 7216,  63368, 13016, 28904,
                2632,  37920, 53116, 7776,  47768, 44440, 37204, 65460, 18136,
                29472, 61076, 21268, 47372, 62816, 228,   30212, 42612, 21724,
                16212, 38428, 44208, 64260, 34824, 48588, 23012, 40600, 50928,
                35892, 60800, 11772, 25132, 8408,  41540, 27196, 51864, 6236,
                35800, 12560, 956,   29452, 47568, 24200, 39976, 50788, 30376,
                21876, 7252,  34240, 23980, 44404, 9308,  27520, 33432, 12416,
                24356, 15784,
            },
            {
                56196, 39684, 63400, 48516, 37432, 54816, 10028, 38752, 23024,
                47828, 31088, 8168,  17612, 58992, 6664,  31544, 19280, 46500,
                29772, 59720, 49360, 23580, 45824, 2924,  7492,  59572, 18700,
                3644,  64028, 57732, 52344, 47324, 31724, 8152,  17172, 63408,
                10660, 57552, 42740, 64700, 49672, 20404, 62160, 36288, 52748,
                5380,  20684, 13204, 63116, 9580,  18704, 46692, 5504,  34500,
                43952, 40152, 23888, 37800, 10484, 43192, 54672, 64680, 58532,
                32560, 46236, 63756, 14928, 41056, 19804, 3672,  45936, 59980,
                49804, 5932,  64400, 15268, 28448, 12288, 24148, 6116,  41216,
                49380, 14680, 4248,  44204, 9260,  37324, 13552, 49508, 54012,
                8720,  28912, 892,   31180, 6760,  54992, 2484,  30296, 8984,
                14804, 53848, 31804, 63624, 35156, 21172, 15320, 38480, 54552,
                23516, 65184, 16468, 41836, 61004, 57388, 13124, 63972, 44096,
                10204, 39200, 46944, 2324,  61524, 37248, 21040, 65092, 41604,
                58560, 50400,
            },
            {
                30240, 22768, 5260,  17204, 3056,  25496, 30720, 18356, 1260,
                61984, 40768, 25672, 51648, 45924, 35676, 13840, 10428, 51412,
                40564, 26332, 7772,  12964, 38276, 52532, 14836, 27384, 40640,
                50664, 13668, 6908,  28264, 10972, 23232, 54516, 39068, 50796,
                20528, 28636, 16108, 34072, 7568,  55224, 31496, 25096, 8948,
                64968, 28492, 55540, 34652, 44892, 40020, 53776, 13808, 65188,
                27072, 7128,  2192,  60420, 33292, 51592, 17984, 27656, 472,
                36072, 5008,  9436,  48980, 30924, 35944, 52828, 39036, 16236,
                21420, 25296, 42492, 58616, 33736, 50060, 54916, 59220, 32240,
                26408, 53856, 34748, 60028, 28336, 24360, 58612, 6308,  35360,
                24884, 56900, 12960, 46940, 15780, 25168, 37528, 58196, 46184,
                22440, 1204,  42848, 18328, 50032, 59512, 47104, 32260, 3496,
                43752, 48048, 33092, 7704,  21404, 4668,  35032, 1900,  26140,
                49760, 58220, 53636, 31284, 17216, 56896, 48360, 1528,  45424,
                19572, 9808,
            },
            {
                53028, 46768, 34792, 43632, 56580, 49700, 64612, 45196, 53112,
                14868, 36704, 11776, 63604, 21776, 27304, 56348, 64588, 4704,
                21524, 15988, 53816, 64188, 28848, 42316, 62744, 21932, 9068,
                31500, 24800, 37156, 20396, 788,   59020, 43488, 13896, 5356,
                32996, 45072, 53340, 12908, 23112, 47272, 2472,  16796, 45720,
                38072, 49728, 4140,  15544, 24868, 58812, 1532,  29672, 57248,
                19552, 46948, 50216, 29340, 21176, 5992,  39124, 14164, 48636,
                20876, 56976, 17716, 60940, 1196,  58048, 26244, 7844,  30244,
                55804, 35284, 10212, 19376, 3704,  39712, 16532, 2828,  10900,
                43436, 7656,  22724, 16400, 50368, 41040, 19760, 31796, 45380,
                63576, 39560, 19908, 60168, 41316, 52208, 11552, 18592, 64500,
                27948, 38604, 57716, 5568,  29108, 2176,  11908, 62808, 18948,
                9788,  58132, 51064, 37756, 53820, 46144, 28776, 55392, 19296,
                14808, 4380,  28308, 11940, 42228, 6064,  25112, 14064, 31604,
                4188,  37084,
            },
            {
                6692,  14420, 62080, 9236,  29040, 13336, 6212,  34592, 8972,
                28200, 56888, 5060,  32776, 2652,  49320, 41396, 24488, 33848,
                58192, 43884, 35848, 2004,  20168, 5972,  34284, 55772, 47724,
                60460, 42980, 55032, 62652, 49272, 29728, 35852, 3040,  55984,
                24936, 62748, 388,   38132, 59784, 28808, 40268, 60796, 57052,
                11224, 22692, 42832, 60368, 7672,  32068, 20844, 51100, 38044,
                10220, 54968, 16600, 12036, 56376, 45828, 62140, 30776, 41884,
                53588, 25588, 38596, 44168, 22252, 13892, 43152, 64024, 11416,
                48256, 764,   45204, 51624, 26264, 63512, 21328, 46400, 35648,
                63040, 57416, 38888, 1604,  55716, 11548, 3028,  60768, 17348,
                4008,  10572, 49928, 33168, 27792, 62448, 4100,  33924, 42160,
                6636,  52048, 24208, 16200, 45420, 37052, 24916, 56408, 30028,
                40508, 27056, 1784,  13884, 25340, 17428, 9292,  42124, 61856,
                37540, 23432, 63768, 20324, 50752, 35348, 60808, 39180, 54520,
                62884, 26648,
            },
            {
                41488, 57644, 19308, 24616, 40256, 58512, 21488, 42076, 59700,
                23624, 48872, 44080, 16740, 60596, 8688,  37568, 17380, 724,
                48088, 9648,  31032, 60252, 49656, 25700, 44924, 48,    17300,
                11804, 3268,  34160, 15204, 41024, 18000, 65428, 22184, 47280,
                40660, 9600,  19204, 49296, 4968,  43568, 13960, 19940, 33588,
                916,   30660, 18116, 51656, 36064, 47988, 11596, 41960, 3816,
                34056, 24664, 40884, 64100, 35548, 3792,  24436, 9900,  59952,
                7240,  12352, 28820, 51780, 34456, 5428,  50220, 18488, 32928,
                40848, 62548, 14504, 57500, 37280, 8976,  29756, 52604, 24928,
                13112, 18560, 30808, 45164, 64844, 33420, 47968, 52120, 26784,
                42224, 55528, 23124, 1640,  8112,  45044, 21872, 55972, 13792,
                48376, 9940,  33484, 54384, 64872, 7504,  50572, 42748, 4444,
                15920, 20960, 60360, 30896, 64300, 39364, 49200, 31696, 6036,
                46352, 34032, 40524, 584,   55232, 8656,  28884, 17596, 49744,
                11080, 21560,
            },
            {
                30784, 48352, 3380,  32596, 51828, 848,   17344, 48120, 32020,
                2428,  19620, 38984, 54732, 29884, 12940, 45360, 54228, 61124,
                26848, 14224, 55956, 38920, 17860, 12500, 59004, 37592, 29520,
                52788, 23176, 46712, 4952,  9936,  52976, 7140,  27588, 60200,
                15424, 30796, 57312, 26452, 34964, 52704, 7996,  48540, 25624,
                54276, 62640, 39804, 26940, 2664,  61716, 23940, 16988, 62696,
                59056, 30636, 668,   43780, 27252, 19188, 52588, 37240, 17064,
                33528, 46868, 3272,  65316, 9736,  56424, 37708, 24068, 59120,
                5032,  27584, 22012, 31280, 5592,  44304, 60328, 372,   40840,
                48816, 4536,  51944, 25604, 7052,  21140, 37792, 14024, 7624,
                30856, 36352, 14900, 57608, 38712, 17208, 50408, 26496, 61640,
                30652, 20268, 60532, 12336, 39952, 21720, 14600, 34584, 61492,
                54208, 36912, 45744, 11052, 4160,  55964, 22068, 59500, 12804,
                53108, 9072,  60060, 15596, 43352, 22628, 46408, 2604,  34064,
                43880, 59916,
            },
            {
                38076, 11996, 54408, 45396, 36192, 63844, 27016, 38284, 12324,
                52320, 65136, 6864,  34276, 23832, 50996, 20352, 6524,  31664,
                42208, 21020, 3628,  45788, 8780,  52252, 32476, 6820,  64880,
                40140, 18768, 61936, 30716, 25892, 45408, 33708, 11860, 51496,
                36744, 6340,  42568, 61732, 11484, 22116, 64444, 37328, 5688,
                45276, 12984, 8560,  57468, 14732, 54796, 44276, 27896, 46320,
                6088,  15044, 53320, 8584,  59656, 13536, 49556, 1408,  45068,
                62728, 55124, 19052, 31800, 15772, 27484, 45636, 2000,  52364,
                16612, 38820, 53964, 11872, 49412, 18240, 55992, 15336, 33356,
                20668, 59932, 9760,  36476, 15800, 28660, 43720, 57728, 63076,
                20300, 53236, 47612, 65512, 29164, 10264, 35216, 2112,  40424,
                4664,  36768, 44328, 2748,  27148, 32148, 52964, 9284,  23320,
                48304, 6876,  33288, 52160, 42640, 15684, 35476, 1448,  25776,
                18384, 30000, 48080, 26544, 32448, 58368, 12676, 64320, 24668,
                6360,  16692,
            },
            {
                27624, 63220, 7436,  22596, 14660, 9824,  55936, 4404,  60772,
                15924, 27748, 10316, 58380, 43124, 1768,  63696, 35456, 10672,
                52044, 65212, 29724, 24272, 62672, 41512, 22328, 14388, 50004,
                25444, 8312,  58088, 38116, 55524, 20088, 58472, 39524, 1236,
                21316, 46272, 54172, 17220, 1628,  31752, 56296, 28416, 17452,
                50908, 34740, 43764, 20076, 37860, 5264,  33204, 8432,  50728,
                21236, 35004, 48752, 22968, 39312, 32892, 56648, 29860, 22224,
                5760,  39700, 24708, 48552, 42260, 61108, 20704, 10996, 29932,
                47252, 7716,  61408, 33872, 41592, 23364, 36440, 10440, 63900,
                29044, 55324, 42296, 47340, 61224, 53836, 576,   23960, 11364,
                45532, 3052,  6364,  24684, 43244, 59604, 54584, 46884, 15680,
                24900, 51076, 58800, 17040, 48128, 62176, 216,   44608, 28896,
                12760, 59188, 18440, 24132, 62960, 8252,  28404, 51220, 44524,
                65532, 3252,  56236, 38376, 6480,  51456, 19604, 40308, 56332,
                53168, 328,
            },
            {
                49904, 19000, 39616, 60288, 30128, 50124, 42676, 24192, 44808,
                35804, 46880, 21644, 40244, 14392, 28992, 56064, 25560, 39724,
                16416, 49300, 5880,  33904, 54688, 2388,  28288, 57580, 43664,
                1700,  34744, 48400, 12268, 43112, 2488,  16204, 48748, 62320,
                28940, 13156, 33980, 24672, 40176, 46732, 14500, 42000, 60896,
                1856,  24032, 65424, 29160, 49292, 22624, 63452, 40216, 12700,
                58016, 38420, 63380, 17508, 46268, 6768,  64692, 11120, 42976,
                27848, 13588, 58884, 40,    53304, 6772,  35732, 63556, 41940,
                56956, 19960, 25220, 1308,  64780, 4708,  27840, 48308, 6620,
                38780, 2380,  23132, 12752, 31476, 4972,  40792, 49852, 35296,
                27832, 40196, 33404, 18744, 13100, 31944, 7016,  19784, 64356,
                56436, 8836,  28620, 5944,  41524, 19228, 57268, 38692, 63980,
                3936,  40980, 26860, 812,   37880, 48116, 19684, 57816, 41108,
                36836, 14124, 21308, 11156, 62352, 3572,  36080, 29212, 9432,
                45228, 32364,
            },
            {
                56948, 25476, 43216, 1808,  17796, 33948, 6476,  20056, 29432,
                57604, 60,    51396, 62976, 5628,  48820, 18196, 45884, 2916,
                57764, 36864, 13088, 19028, 38380, 48472, 16844, 35956, 10852,
                54316, 28980, 15728, 4516,  22804, 64208, 31416, 9092,  24320,
                56004, 3972,  64140, 8888,  51968, 59408, 3400,  10208, 21456,
                55064, 32124, 6644,  11028, 52876, 36,    17396, 30136, 55408,
                26260, 2400,  10736, 29196, 4020,  25612, 40948, 19772, 53860,
                60516, 35148, 9256,  38096, 22932, 14548, 49724, 32388, 3012,
                13412, 36644, 45464, 50556, 14772, 58224, 43468, 53480, 25300,
                45816, 16284, 34952, 58136, 19844, 64312, 9488,  18420, 60544,
                15676, 56624, 61516, 48896, 268,   52832, 23020, 37940, 11372,
                42880, 34144, 20832, 54952, 36228, 11072, 25916, 15484, 20576,
                31748, 50276, 57436, 44688, 13560, 61200, 32692, 10824, 5768,
                24164, 31992, 52672, 45352, 25320, 49024, 15040, 61096, 22888,
                37076, 13364,
            },
            {
                5948,  34684, 10492, 47564, 58692, 55084, 38944, 63312, 11888,
                8200,  31952, 17700, 24792, 37600, 32532, 11332, 62248, 7840,
                22892, 27596, 60380, 44016, 10616, 61416, 4644,  21344, 63780,
                46128, 19476, 61144, 51204, 36580, 26920, 53292, 41732, 34852,
                18624, 43896, 49880, 30264, 36996, 19528, 26348, 35576, 49176,
                39256, 15832, 58720, 40880, 46100, 35708, 59888, 43620, 4772,
                19264, 45224, 32488, 54460, 59092, 49268, 15060, 33764, 2256,
                50680, 16760, 47668, 30040, 57040, 41020, 25688, 18180, 54392,
                27880, 60220, 9968,  30372, 39484, 20968, 32612, 12944, 18824,
                62204, 51640, 7760,  27708, 48692, 38232, 26116, 32436, 51992,
                7348,  22412, 9944,  35968, 41180, 63004, 27612, 45480, 30748,
                1068,  49124, 13636, 61052, 30232, 51792, 7184,  46988, 55372,
                36428, 8096,  16688, 53440, 29476, 22652, 3080,  54548, 16996,
                49552, 61572, 7824,  40664, 18228, 33592, 42780, 7008,  52084,
                17520, 65256,
            },
            {
                21408, 53732, 62384, 28184, 23092, 13676, 3232,  52616, 47704,
                37048, 55312, 43392, 3376,  60236, 53888, 21436, 42292, 34432,
                52940, 47840, 608,   56252, 31340, 26384, 52100, 41900, 31884,
                5940,  24924, 40712, 32572, 58932, 7036,  13524, 47156, 5308,
                38300, 58772, 15208, 22556, 5652,  55668, 43600, 63932, 29604,
                3828,  47144, 26012, 18676, 56760, 14248, 24476, 9604,  36472,
                52316, 61412, 41312, 13812, 21156, 36356, 61560, 8148,  24464,
                44460, 4172,  19500, 64308, 11924, 4816,  59632, 8672,  43556,
                22396, 5968,  52632, 17440, 3532,  55236, 8644,  59644, 1664,
                41648, 32072, 55776, 3500,  44836, 12212, 54888, 43408, 1500,
                47072, 30480, 54320, 26372, 16012, 50616, 4136,  58436, 17812,
                62360, 38900, 23836, 44884, 2520,  65320, 33892, 42688, 1872,
                23704, 62128, 11104, 34408, 6472,  40356, 46216, 59620, 35144,
                43788, 64,    56964, 29288, 64184, 1844,  57876, 27028, 46684,
                2432,  41188,
            },
            {
                48592, 3696,  15908, 37308, 7792,  45808, 31644, 26340, 15488,
                22352, 61680, 27296, 13360, 46536, 6848,  28328, 58336, 14492,
                30420, 9768,  40860, 21068, 15560, 46800, 8624,  59420, 37840,
                13052, 55432, 400,   10180, 44972, 17408, 56928, 21104, 62004,
                11128, 27000, 196,   47760, 60608, 11600, 16720, 7196,  53648,
                13024, 60448, 34212, 2440,  28384, 7144,  32412, 64784, 48200,
                15232, 23568, 7700,  50840, 1136,  43404, 30080, 56016, 38332,
                62964, 31676, 53512, 26632, 45900, 34040, 51884, 37168, 65120,
                48452, 33716, 41036, 61344, 26292, 35244, 48956, 38148, 29792,
                21732, 10524, 36664, 60352, 17008, 22156, 6096,  58412, 14708,
                37688, 64684, 3856,  44612, 20852, 8428,  33268, 14120, 54204,
                6832,  26716, 53160, 4984,  16696, 22364, 49816, 13464, 58896,
                29844, 48024, 43136, 20284, 64920, 49924, 25288, 12788, 30024,
                20480, 26696, 15712, 37480, 12524, 22200, 54512, 9948,  31956,
                59468, 29536,
            },
            {
                11808, 44600, 32484, 51552, 41904, 64752, 18916, 40304, 58300,
                6300,  41372, 9972,  50012, 35288, 16096, 39368, 1852,  44780,
                18536, 64452, 24864, 6796,  62392, 35252, 1524,  23560, 17688,
                50540, 30068, 63172, 21904, 49576, 38788, 29352, 1812,  51512,
                31288, 54616, 42348, 33728, 39412, 28464, 51268, 33336, 23596,
                42252, 8820,  21256, 63776, 44200, 49732, 53900, 20416, 40536,
                1716,  28720, 34560, 58064, 26576, 10396, 18104, 47536, 12656,
                21976, 6716,  36240, 42704, 1836,  21600, 17020, 29284, 688,
                14852, 57284, 19848, 11844, 44084, 63760, 23300, 5076,  46672,
                65464, 14960, 50280, 24796, 40388, 63424, 35820, 29232, 24308,
                41936, 19248, 12988, 59136, 39072, 56512, 47600, 36800, 21924,
                42196, 10056, 32900, 57592, 41148, 37752, 27260, 9224,  18484,
                39556, 4356,  26280, 56144, 2096,  17544, 37332, 6932,  63840,
                39796, 9796,  54092, 46028, 4728,  49856, 35204, 19728, 14432,
                37900, 24552,
            },
            {
                57296, 61552, 19868, 25800, 476,   54240, 10556, 49248, 1696,
                32884, 51972, 20592, 30492, 56816, 23776, 63352, 51636, 6076,
                37508, 54412, 32324, 50116, 38640, 54984, 29204, 11392, 57936,
                47316, 42868, 15280, 35464, 5080,  25464, 65016, 34376, 44628,
                23668, 17620, 7152,  64608, 20136, 4096,  45388, 62736, 18876,
                37852, 55624, 51016, 36412, 12092, 39064, 3392,  26908, 11348,
                55676, 62460, 17096, 46912, 39404, 65208, 4612,  52348, 28008,
                41112, 57464, 9848,  14692, 55412, 62084, 47700, 11460, 39848,
                24944, 4432,  31460, 47264, 1064,  15180, 53044, 17744, 57480,
                26928, 43084, 140,   30656, 9360,  51708, 2860,  48576, 61288,
                10516, 33480, 50960, 28320, 1364,  24520, 11244, 64992, 29468,
                50236, 59712, 19340, 47428, 12068, 60136, 53724, 32396, 63580,
                51464, 14980, 36988, 60376, 31296, 10128, 51832, 57524, 44624,
                4332,  51144, 61208, 23716, 30700, 59696, 39856, 44400, 63600,
                50376, 8260,
            },
            {
                34260, 39740, 6752,  13080, 60480, 35076, 22816, 28880, 62540,
                36400, 17488, 65448, 2500,  44192, 8340,  19536, 48356, 26708,
                59688, 11692, 3044,  43128, 13372, 19588, 45436, 65452, 33492,
                4168,  7928,  27600, 60636, 52516, 11756, 41456, 14944, 8852,
                60020, 36128, 13456, 50316, 24620, 57884, 14308, 1152,  48728,
                27740, 5492,  15628, 30200, 23128, 62180, 17284, 59084, 44532,
                31072, 37864, 3876,  12340, 54564, 23344, 33568, 58748, 520,
                16024, 50240, 60996, 24312, 39120, 7544,  32528, 59032, 44920,
                53624, 62740, 37716, 51660, 28976, 9476,  41300, 32392, 12172,
                7344,  54100, 19088, 46260, 57044, 33196, 20196, 16276, 7660,
                53260, 45492, 5140,  63152, 31700, 41796, 18324, 6420,  45108,
                3992,  15552, 36048, 28120, 7664,  20608, 636,   44004, 6264,
                22812, 54960, 12196, 46564, 22124, 42216, 27968, 15076, 23148,
                31492, 18404, 34400, 42096, 8404,  16984, 26232, 6232,  696,
                54964, 18008,
            },
            {
                3452,  27812, 47896, 56204, 30596, 16924, 43688, 8048,  13856,
                45288, 4684,  54400, 37808, 26096, 59028, 32800, 13128, 35716,
                22356, 46696, 17400, 57440, 27288, 5212,  51796, 40592, 25952,
                21028, 56796, 39356, 18612, 46708, 2872,  54948, 20028, 49992,
                4724,  47076, 56320, 29956, 9860,  36572, 41080, 31276, 57104,
                11236, 61812, 45888, 656,   56400, 8444,  33312, 47544, 6052,
                21708, 50172, 42752, 19912, 31396, 7476,  44256, 36844, 20364,
                46080, 29912, 33160, 18600, 49152, 28304, 3092,  22532, 18900,
                9328,  27312, 6960,  21936, 56636, 34892, 60540, 24428, 49692,
                39192, 34596, 63016, 4832,  13716, 26672, 44424, 55692, 39228,
                26220, 21832, 36448, 16520, 48412, 53896, 60432, 34404, 25808,
                39496, 63356, 2308,  52008, 31264, 61932, 25224, 48460, 35200,
                41560, 29072, 1092,  33452, 5440,  54168, 63100, 34820, 2264,
                48716, 58752, 13756, 1248,  65104, 52844, 12120, 48032, 28840,
                22756, 42480,
            },
            {
                52620, 63880, 21496, 44880, 4440,  39208, 59488, 51080, 56356,
                26904, 23324, 47512, 15148, 10836, 40988, 872,   53852, 42828,
                4948,  29300, 62928, 34252, 23276, 61384, 9136,  16168, 2352,
                36180, 53208, 10336, 30920, 23896, 58312, 32696, 28240, 62952,
                39264, 26580, 42984, 1980,  62152, 53228, 21012, 6656,  43808,
                24292, 32852, 40184, 18452, 52444, 42328, 24804, 36748, 14656,
                63792, 9712,  25640, 60092, 51844, 15480, 61156, 11812, 26224,
                64096, 6380,  3676,  43932, 11748, 64352, 52772, 41580, 56232,
                34052, 49632, 16356, 42964, 63372, 18884, 4800,  44980, 2640,
                58784, 28268, 22548, 37624, 60956, 41092, 11040, 63960, 31268,
                720,   60624, 57212, 8556,  13400, 22404, 2852,  50900, 57012,
                12784, 21536, 55072, 42336, 45756, 14292, 55840, 16972, 10708,
                57820, 18736, 64496, 49636, 38564, 19056, 7576,  45696, 56660,
                10352, 38536, 28132, 46460, 20748, 33068, 56784, 36820, 62532,
                32056, 13624,
            },
            {
                37604, 16564, 10684, 36056, 53232, 25432, 2464,  20360, 31908,
                41476, 7064,  58256, 33772, 51240, 61536, 30020, 16500, 64868,
                9696,  52200, 40940, 88,    48584, 37696, 32092, 44712, 60112,
                48908, 14096, 64264, 44376, 6284,  16572, 37472, 43988, 864,
                22392, 11256, 19068, 33140, 15808, 46020, 26236, 60316, 17432,
                53736, 4540,  50324, 27120, 59844, 13140, 4212,  57804, 52952,
                28476, 412,   56316, 35228, 5528,  28300, 49516, 41440, 8620,
                54144, 38584, 51384, 57748, 35408, 25336, 15420, 36840, 4840,
                13232, 59608, 39012, 3152,  12356, 26160, 37232, 54344, 20676,
                14424, 9228,  52240, 17024, 48480, 2232,  23080, 36116, 6240,
                47528, 18500, 40608, 27436, 44324, 37988, 29140, 9592,  17260,
                32444, 48232, 24652, 10504, 5540,  34164, 39140, 3228,  62584,
                26124, 8008,  43528, 15672, 58592, 26620, 12528, 40380, 20548,
                25516, 62800, 5680,  50836, 24344, 40516, 2768,  18812, 43692,
                9720,  59324,
            },
            {
                2016,  50724, 23984, 7232,  62168, 15100, 48220, 10188, 38192,
                61064, 12368, 29276, 18568, 4060,  21088, 44908, 24380, 56492,
                38408, 20444, 15036, 7784,  55968, 12888, 18816, 54404, 23752,
                29112, 41944, 652,   34288, 50712, 61272, 7888,  13168, 57140,
                53176, 35316, 58744, 48688, 4308,  38908, 8904,  35140, 13212,
                63976, 37028, 21164, 9596,  34968, 30760, 48912, 19164, 40288,
                32924, 45716, 17392, 40844, 46540, 1824,  22168, 56068, 17836,
                34564, 23448, 13832, 20892, 1140,  46404, 8684,  61396, 29484,
                47764, 23816, 32032, 55180, 46416, 50748, 8680,  30528, 64284,
                48028, 42488, 32224, 6028,  54876, 29304, 58600, 50184, 15284,
                54328, 11340, 33984, 51672, 5448,  58796, 62644, 46380, 41324,
                136,   61608, 37176, 18044, 64412, 22048, 51032, 30236, 46632,
                37836, 52276, 23312, 30648, 2992,  47424, 61724, 32028, 51928,
                15300, 42920, 35592, 55272, 10000, 61036, 15568, 51764, 7156,
                24992, 46568,
            },
            {
                29896, 41616, 33076, 57432, 28472, 42560, 33928, 63736, 17228,
                736,   53188, 43564, 64216, 39544, 49260, 7924,  34868, 2988,
                49944, 31816, 25552, 60052, 43196, 29908, 4576,  63404, 11496,
                7200,  20224, 58968, 27068, 21856, 40648, 48736, 25920, 31204,
                15980, 5552,  65404, 28096, 22964, 56180, 51344, 28864, 47300,
                1756,  42644, 15032, 62312, 46412, 1912,  65324, 22604, 6560,
                10936, 61664, 23172, 13440, 64532, 38080, 31900, 62148, 3420,
                47548, 60472, 29144, 41832, 59332, 31320, 51748, 18412, 44032,
                176,   63952, 9896,  18100, 29128, 61368, 41780, 16136, 34244,
                1032,  60692, 25732, 45076, 12712, 34096, 8380,  19352, 25664,
                42796, 30324, 65508, 1932,  24352, 15340, 19564, 35744, 26684,
                52560, 8064,  29420, 56904, 40888, 27208, 59568, 12792, 20308,
                4936,  33720, 56776, 11012, 51408, 35276, 23644, 1456,  55640,
                6596,  29848, 17928, 3312,  44756, 30888, 27060, 58604, 34152,
                20312, 54884,
            },
            {
                5096,  64824, 18264, 49500, 12244, 20880, 5860,  54700, 25928,
                49952, 22108, 35596, 9164,  27376, 55060, 13768, 59520, 28228,
                11560, 62260, 46136, 35580, 52528, 21136, 40340, 47256, 35168,
                38996, 52312, 15052, 5340,  54324, 11212, 19232, 63856, 45732,
                50864, 41176, 8360,  44684, 13744, 62576, 3244,  20108, 58128,
                31376, 23636, 56728, 6804,  39304, 25332, 43356, 15444, 55548,
                37292, 50856, 30564, 7904,  53144, 19084, 10172, 27308, 14684,
                43480, 6904,  10080, 53632, 17316, 5512,  39492, 26976, 57112,
                20796, 41060, 52564, 36232, 1748,  22256, 5660,  56184, 23860,
                11524, 39456, 18652, 57704, 21504, 64696, 42036, 38164, 61916,
                3288,  45916, 20980, 55884, 48724, 32964, 54692, 6044,  12392,
                58304, 44076, 14752, 4588,  47472, 1624,  8960,  42992, 54288,
                61092, 14700, 41156, 6248,  59896, 16816, 44156, 9352,  37140,
                59280, 49312, 22936, 63536, 13700, 37456, 5348,  47836, 39624,
                61564, 14336,
            },
            {
                36552, 9504,  45488, 1024,  38736, 58708, 44700, 36816, 29988,
                46068, 5288,  15492, 56968, 23944, 1256,  37764, 47188, 18808,
                42300, 6320,  17048, 3364,  9816,  27676, 58828, 1228,  25120,
                57660, 32196, 47772, 62764, 35640, 30044, 2128,  36764, 4164,
                20564, 24560, 38040, 53912, 17964, 32576, 42420, 11296, 39244,
                6124,  52924, 28284, 49540, 17868, 10368, 52268, 60560, 26960,
                41996, 4528,  57376, 26024, 34296, 58576, 48952, 40128, 52768,
                32708, 25392, 63180, 36140, 48348, 23628, 64776, 10344, 35084,
                14360, 6372,  25348, 45216, 57304, 13196, 37768, 48996, 44312,
                29008, 53052, 4352,  36740, 47480, 416,   27752, 51736, 13544,
                56680, 7968,  37360, 13108, 40076, 9280,  42896, 64328, 23480,
                31808, 19096, 50356, 35792, 53768, 32652, 17600, 24524, 35980,
                364,   27736, 46340, 21892, 38208, 28968, 65060, 20008, 27048,
                45220, 11240, 33464, 40872, 56616, 49972, 22240, 11672, 276,
                44604, 27400,
            },
            {
                51072, 22380, 26204, 53424, 31196, 3796,  23520, 8816,  12844,
                60228, 39212, 31620, 62048, 45240, 17668, 31124, 63748, 22528,
                54340, 39428, 57260, 23592, 64720, 13928, 50072, 16956, 8756,
                43000, 18168, 2540,  44852, 24240, 56936, 42716, 59476, 9888,
                33776, 61392, 12056, 30220, 5268,  36608, 24188, 49840, 64840,
                16264, 44300, 12144, 32764, 59436, 36688, 30048, 2716,  34380,
                12696, 21052, 47888, 2572,  16192, 44528, 5640,  22972, 344,
                56328, 19764, 39292, 2360,  12932, 33448, 45644, 3640,  54768,
                48840, 30540, 60080, 15932, 32788, 65276, 27444, 19240, 59228,
                7756,  63720, 15272, 31140, 10008, 53908, 16928, 5304,  33224,
                22952, 28948, 17144, 62812, 25880, 30168, 1340,  49868, 38084,
                3300,  60688, 25600, 21288, 11876, 58200, 45540, 65296, 49808,
                30884, 18844, 62924, 50212, 13092, 3520,  53788, 41640, 14596,
                62132, 604,   53296, 26212, 8212,  19276, 64668, 53868, 31916,
                17272, 56352,
            },
            {
                12628, 40748, 60272, 14508, 62432, 47244, 16660, 65144, 56100,
                19432, 2460,  51508, 11320, 41260, 5564,  52696, 9572,  33968,
                1828,  26996, 50976, 32296, 38460, 44804, 33996, 55396, 30432,
                64068, 11828, 28388, 38320, 8488,  13596, 17264, 52508, 28256,
                55344, 48044, 988,   57960, 51084, 60660, 45844, 1328,  26724,
                34540, 61048, 21752, 272,   55104, 20244, 46132, 8232,  48984,
                17684, 63672, 43500, 38512, 62396, 11648, 29396, 35912, 65236,
                46048, 15880, 50104, 57316, 28408, 60264, 16484, 41660, 22140,
                62936, 38312, 9180,  50452, 4408,  40752, 10148, 52476, 2996,
                35448, 22944, 50296, 40972, 62336, 24232, 44412, 60824, 49132,
                41140, 58488, 51416, 6200,  47516, 59548, 21380, 53804, 16948,
                46928, 40292, 9404,  63504, 29836, 38696, 6412,  15224, 40136,
                10612, 55220, 7960,  25576, 44752, 57780, 32604, 5596,  48428,
                35184, 21648, 16704, 46640, 2744,  28836, 34920, 42592, 24400,
                7688,  62968,
            },
            {
                2756,  33304, 5464,  20012, 36016, 27720, 52004, 33192, 42072,
                24880, 48176, 34720, 21428, 27984, 58644, 48744, 15292, 43452,
                59996, 12896, 7900,  47796, 18436, 2320,  61988, 6024,  22832,
                48932, 53348, 21680, 60648, 51116, 33128, 47116, 23336, 39468,
                14436, 18688, 41500, 26936, 21432, 8092,  14172, 55552, 19196,
                9144,  38036, 47972, 40644, 5792,  13936, 64276, 24096, 58768,
                54108, 28756, 8812,  31980, 54800, 19852, 51572, 59232, 8084,
                12184, 30520, 5996,  43900, 20964, 8244,  52328, 31452, 12048,
                26804, 2008,  43260, 19892, 23444, 54544, 46008, 16456, 31592,
                42804, 12360, 56524, 26864, 2896,  35816, 12232, 30484, 19800,
                10288, 35060, 2548,  43824, 18656, 36424, 10576, 28528, 13512,
                33668, 6812,  54932, 42844, 2300,  52152, 26764, 21696, 3780,
                60388, 42860, 36900, 2032,  34772, 18108, 10844, 24452, 39152,
                8652,  58396, 31224, 62620, 38880, 57624, 15240, 4252,  49040,
                38720, 30016,
            },
            {
                58780, 49776, 43916, 57020, 7868,  11300, 40320, 148,   7032,
                28944, 14208, 54152, 7908,  64652, 35836, 24744, 4484,  29648,
                46580, 37044, 21664, 60776, 29092, 11384, 25976, 41044, 36812,
                14668, 4048,  40352, 6688,  18928, 65108, 280,   5524,  62088,
                7524,  32416, 63920, 44244, 16424, 34808, 39896, 28996, 51984,
                58376, 3612,  25236, 62664, 51324, 27844, 42372, 32336, 39564,
                4200,  36536, 14372, 23764, 1244,  40580, 25736, 17108, 42628,
                22348, 38308, 62712, 25832, 54212, 35760, 712,   49476, 58324,
                18332, 47212, 56544, 28400, 62364, 34536, 432,   25228, 63008,
                48444, 20072, 5360,  45768, 18444, 58940, 39548, 55736, 1336,
                47196, 24432, 61440, 12580, 32588, 55700, 3912,  44560, 61832,
                57336, 27440, 48892, 14400, 18280, 61568, 34816, 56300, 46804,
                32704, 23332, 13944, 53436, 63560, 30216, 51232, 61472, 56136,
                27660, 50736, 4648,  12740, 44368, 51292, 10276, 61264, 55364,
                18848, 46512,
            },
            {
                8952,  28316, 16344, 39308, 22672, 48764, 55296, 17908, 45880,
                57572, 60752, 43304, 3096,  18832, 12256, 40280, 57680, 20144,
                63296, 16288, 55300, 4672,  51468, 43576, 54480, 19704, 46668,
                59972, 31456, 56084, 45840, 26108, 36944, 30628, 43604, 50132,
                25508, 37524, 4268,  10600, 56752, 47864, 63340, 4944,  32656,
                44968, 14904, 30544, 10772, 34692, 45248, 16536, 1112,  11624,
                22444, 51168, 57616, 44904, 61016, 49172, 5016,  33112, 47520,
                54996, 34672, 4036,  11140, 17604, 61888, 39340, 23872, 6132,
                37120, 33184, 14644, 6788,  39644, 12192, 59428, 36824, 8352,
                39024, 28724, 60932, 34328, 8896,  50020, 22376, 6952,  64936,
                15964, 54856, 28088, 40376, 22572, 64632, 50188, 38852, 23892,
                456,   20596, 36856, 32328, 24060, 44172, 9012,  12576, 51420,
                17704, 29024, 59396, 47656, 20600, 4748,  42148, 15532, 1796,
                18920, 42608, 36888, 25012, 18016, 32380, 22524, 27356, 36004,
                13780, 21852,
            },
            {
                37392, 24680, 53708, 1684,  64064, 31508, 25668, 62076, 37448,
                10124, 22856, 30948, 38008, 46972, 55608, 32540, 51040, 448,
                10380, 41840, 25596, 34176, 38980, 15564, 58596, 9464,  1380,
                27360, 11048, 34668, 15536, 58176, 10184, 54356, 13012, 20392,
                57528, 46252, 54024, 22744, 31176, 2336,  24692, 12388, 20836,
                41124, 49608, 17940, 55732, 22332, 7488,  57736, 53080, 61804,
                46544, 18636, 6180,  27980, 10784, 37300, 14848, 63740, 1728,
                27548, 58164, 50596, 41356, 46848, 29412, 14012, 43508, 65032,
                53184, 10680, 60704, 51808, 30588, 17720, 49140, 21076, 57072,
                3448,  51712, 16152, 54076, 29928, 13916, 43228, 26120, 33636,
                44960, 37816, 8668,  52464, 5412,  14764, 30496, 8292,  17660,
                42428, 63860, 11076, 59392, 5900,  49544, 28728, 37516, 64088,
                1044,  6168,  39760, 9220,  27064, 37980, 46060, 23104, 33768,
                47348, 64948, 9800,  54920, 59808, 800,   41444, 7360,  45320,
                1924,  65440,
            },
            {
                51716, 11880, 60108, 34928, 45052, 4320,  12624, 20408, 34368,
                5092,  16504, 51248, 25440, 61544, 15592, 6496,  22684, 36372,
                53564, 31292, 7420,  65444, 2636,  23472, 31744, 35672, 50568,
                64564, 23732, 42108, 2068,  49076, 22608, 39836, 60148, 34644,
                9156,  15600, 28780, 50644, 36708, 61436, 42668, 52684, 59328,
                6736,  63936, 37348, 2076,  60840, 38624, 19980, 26548, 35068,
                30516, 42032, 65036, 34396, 21372, 56692, 30108, 52448, 9812,
                18748, 13784, 21500, 32096, 2672,  58836, 9216,  19312, 28544,
                3076,  25740, 45364, 22696, 42500, 5336,  53700, 27188, 14160,
                44588, 11148, 23948, 40448, 1608,  63304, 36968, 52948, 11112,
                30228, 3588,  17972, 58528, 42856, 25940, 46572, 56812, 35620,
                52076, 4448,  46012, 53288, 39248, 1764,  57160, 19888, 25656,
                44616, 34228, 55820, 16480, 52580, 12644, 58956, 8116,  53864,
                13444, 30308, 21792, 6712,  35328, 48484, 63064, 53524, 30724,
                57084, 41980,
            },
            {
                32352, 19496, 7048,  29012, 15316, 51320, 58832, 41880, 53416,
                48368, 63984, 40908, 1284,  9424,  29056, 44868, 59740, 26944,
                45952, 18256, 49680, 57352, 12420, 47620, 62228, 6488,  17468,
                44272, 54280, 20040, 62944, 30116, 5072,  16856, 27816, 3356,
                42288, 64708, 1272,  19580, 13740, 7172,  17852, 28072, 34932,
                10284, 23880, 27472, 51860, 31656, 47524, 13692, 4804,  49736,
                9076,  15116, 3368,  53572, 46968, 7412,  41480, 24152, 45144,
                61684, 39040, 7616,  64060, 24376, 52092, 33468, 48336, 56288,
                41104, 16568, 35024, 1368,  64432, 9908,  33388, 40624, 63764,
                30876, 35884, 61800, 47500, 21144, 57292, 6048,  19152, 60192,
                49244, 62564, 21692, 47696, 34960, 1192,  20164, 60836, 12424,
                31668, 25232, 16616, 29684, 22204, 63096, 16144, 41632, 7708,
                60580, 49096, 22612, 43056, 64596, 236,   35940, 29124, 62348,
                2884,  38804, 50300, 44844, 27788, 14328, 19596, 10800, 24204,
                16872, 4732,
            },
            {
                61604, 48096, 43188, 56140, 21248, 38260, 8184,  29628, 3208,
                27088, 13468, 33548, 54636, 21216, 49184, 41572, 12012, 2784,
                62864, 13852, 22236, 42744, 28396, 20340, 53104, 40404, 29480,
                13664, 3580,  38368, 9080,  52108, 43920, 61960, 47216, 51372,
                23496, 31640, 39528, 58492, 44744, 55044, 38396, 48464, 3148,
                56440, 46204, 14240, 6392,  41860, 10132, 64484, 43860, 55904,
                24300, 39928, 58252, 26052, 12808, 19368, 60384, 33680, 3552,
                28860, 49524, 54448, 44000, 15644, 37676, 4868,  22300, 61196,
                7644,  51024, 59272, 19584, 47884, 56476, 23804, 46220, 1820,
                19456, 7584,  55356, 4584,  32372, 27224, 46152, 15164, 100,
                24136, 39880, 13136, 32008, 63584, 9976,  53800, 28108, 6572,
                48976, 41164, 56072, 8192,  34728, 11884, 46960, 31236, 52228,
                14696, 10768, 29936, 3812,  32716, 24764, 50764, 17528, 41392,
                26112, 60488, 16060, 57128, 3768,  40492, 33644, 59076, 38660,
                50524, 27220,
            },
            {
                13900, 420,   39420, 10332, 26176, 63032, 47008, 17288, 56340,
                22064, 45508, 6328,  58548, 36532, 64812, 17516, 33392, 39320,
                55872, 5892,  37960, 34544, 9240,  45172, 52,    25280, 57648,
                48704, 32712, 59496, 26404, 36248, 12108, 33324, 19436, 7424,
                55496, 11972, 48960, 25352, 9644,  32584, 23032, 65284, 16460,
                31152, 39984, 54304, 18836, 59756, 23136, 28220, 36976, 1444,
                62472, 18060, 32616, 44152, 640,   50716, 38400, 15692, 57380,
                22740, 36060, 248,   11408, 27216, 55724, 45784, 13048, 36444,
                31608, 11176, 39216, 29532, 13104, 37260, 4492,  16552, 58092,
                49284, 25924, 42604, 17560, 12248, 39480, 51188, 35356, 42248,
                56448, 27336, 4480,  50892, 16516, 40852, 36920, 44484, 15368,
                65500, 1588,  20260, 43956, 61088, 27008, 54972, 2720,  36436,
                20900, 56600, 38792, 61696, 19488, 47736, 57932, 10696, 45264,
                5188,  20096, 31588, 11520, 63700, 23256, 52436, 2292,  46852,
                8864,  35604,
            },
            {
                58060, 23088, 64568, 32948, 50360, 2148,  35376, 11508, 32432,
                62632, 38808, 19216, 11064, 26516, 3528,  53360, 8136,  24372,
                50608, 29680, 61836, 3976,  54504, 60484, 15372, 36648, 10228,
                22496, 6740,  45640, 15304, 56132, 24156, 628,   58392, 37712,
                16268, 62428, 34760, 4844,  61148, 51012, 56,    12016, 43876,
                20912, 62860, 35904, 1040,  49252, 33408, 16224, 53520, 20820,
                29828, 11192, 51872, 36200, 63828, 27652, 5048,  47684, 11800,
                42620, 17496, 31360, 60140, 19812, 40344, 62836, 1104,  18472,
                47356, 23368, 3844,  55308, 26360, 62704, 32332, 52956, 10924,
                38256, 59872, 33932, 52408, 65496, 7896,  59300, 20556, 9820,
                32920, 53396, 7752,  57532, 29152, 23952, 3308,  58984, 22700,
                33552, 38336, 57924, 14020, 50476, 5088,  40324, 24244, 65068,
                5756,  27456, 45580, 8356,  13760, 37396, 6348,  22244, 34324,
                56308, 52740, 37492, 42820, 47816, 8176,  29524, 15688, 64236,
                20732, 44116,
            },
            {
                30812, 16512, 53384, 6184,  18724, 44384, 24160, 60292, 42232,
                460,   48972, 30608, 52204, 44188, 14728, 31000, 60756, 19948,
                43268, 15776, 48436, 25868, 18716, 39132, 31108, 64220, 51588,
                41916, 62276, 18756, 49652, 5536,  64548, 41892, 29512, 45036,
                26868, 2420,  41096, 21096, 29208, 15228, 41428, 26756, 57620,
                5372,  9128,  29264, 25428, 45056, 4012,  58776, 7796,  41484,
                46480, 59816, 6728,  22096, 16724, 55120, 30932, 61376, 7828,
                53216, 65352, 5604,  48856, 34548, 7096,  30224, 25184, 54180,
                64544, 41612, 51924, 16112, 44936, 8368,  20432, 43288, 28488,
                22836, 14416, 704,   46920, 29564, 24696, 2384,  30592, 62064,
                44452, 17324, 37552, 45796, 12096, 61824, 49392, 18784, 9032,
                53132, 26080, 3852,  30352, 37284, 18828, 9928,  59276, 32580,
                41852, 50460, 17240, 59404, 54348, 42736, 27672, 63024, 14876,
                49092, 8996,  996,   25716, 18544, 55316, 36948, 42236, 26144,
                5776,  52016,
            },
            {
                40012, 3460,  28064, 37356, 59452, 14152, 54148, 5300,  50660,
                15000, 25020, 7816,  61460, 39628, 22708, 47336, 57132, 36304,
                708,   10448, 32780, 58440, 12904, 47184, 7484,  19944, 3348,
                34908, 12652, 28832, 40060, 32044, 20872, 52388, 14260, 8800,
                54016, 48140, 17500, 56212, 45920, 35748, 60072, 53492, 33632,
                47436, 51192, 13476, 61912, 55068, 39112, 12276, 51312, 34572,
                14540, 2328,  38776, 47956, 9332,  41912, 24044, 34888, 20928,
                39516, 26656, 45340, 23184, 57788, 10192, 49984, 43256, 14484,
                28060, 6540,  34080, 61400, 144,   40332, 59616, 50192, 6080,
                64108, 41680, 9380,  21816, 15576, 44008, 54264, 48112, 13540,
                3424,  23208, 64820, 20460, 42212, 5584,  31760, 55768, 40204,
                45336, 11704, 63072, 47664, 23392, 53828, 45168, 15904, 48068,
                11952, 904,   35044, 25824, 3456,  31228, 51784, 2088,  39380,
                29592, 23664, 58464, 34780, 61428, 4512,  12604, 59852, 33156,
                56460, 11544,
            },
            {
                47080, 61304, 43076, 10724, 48456, 31052, 40104, 20612, 28356,
                57640, 35740, 18048, 55152, 1584,  34448, 4892,  12140, 27668,
                52860, 63276, 23116, 40836, 1884,  50956, 28224, 43964, 55856,
                25032, 58424, 1148,  54312, 10424, 47676, 3752,  35312, 59612,
                22492, 31160, 11216, 64556, 5832,  9016,  18740, 3868,  23780,
                38348, 17028, 42312, 21656, 6924,  18272, 30664, 22920, 61292,
                27392, 54696, 24704, 31996, 58964, 12956, 45604, 1292,  50580,
                15064, 56916, 2168,  13632, 41896, 17080, 60464, 2496,  37868,
                56940, 9856,  21384, 48924, 30736, 24636, 13864, 35352, 2980,
                31708, 54824, 36588, 61336, 57032, 38688, 18512, 36024, 26256,
                57808, 49960, 34388, 1552,  52624, 26604, 35484, 13940, 292,
                29424, 50868, 16876, 7168,  32792, 62240, 1600,  28560, 20344,
                57652, 52688, 22576, 40688, 64240, 20256, 12308, 57356, 46676,
                18212, 65312, 44520, 14052, 30828, 49884, 22172, 45536, 1556,
                17892, 24040,
            },
            {
                8296,  33864, 55832, 21868, 25936, 7112,  61996, 9684,  37932,
                4388,  65160, 42908, 13244, 28456, 50708, 64040, 40560, 46076,
                17792, 5316,  45080, 55696, 35388, 21980, 59560, 11656, 32876,
                16408, 44788, 37264, 23352, 61612, 17244, 38628, 25216, 63280,
                5172,  43396, 39160, 52188, 24628, 37832, 49316, 30400, 11456,
                60832, 2164,  57920, 32080, 36348, 65096, 43784, 48076, 4580,
                19160, 42712, 64636, 52292, 3664,  19632, 56312, 63324, 29408,
                9744,  37252, 32916, 51876, 28136, 35600, 53380, 22664, 32676,
                46456, 17724, 58636, 36892, 11136, 53604, 18380, 56404, 47468,
                25164, 19256, 50932, 6520,  32084, 3940,  10868, 63036, 5424,
                40716, 15120, 30320, 10604, 60588, 16352, 47488, 64368, 24592,
                60008, 20120, 35704, 41692, 58500, 13328, 39584, 36020, 7380,
                63268, 30092, 14744, 47296, 9372,  34224, 43936, 24912, 7288,
                32872, 10964, 3604,  53156, 39416, 7444,  27828, 53996, 38892,
                63020, 50100,
            },
            {
                27116, 19400, 13392, 1028,  51772, 34588, 16812, 55664, 46660,
                31768, 21808, 47936, 9392,  59672, 20316, 16196, 7280,  24648,
                59416, 38252, 28852, 8912,  16040, 63712, 4292,  38612, 53020,
                8676,  64804, 50392, 6176,  27768, 56952, 45948, 11628, 50096,
                19260, 33656, 1656,  27976, 13300, 57460, 42956, 64008, 20412,
                45468, 27704, 52712, 10564, 49484, 680,   26252, 9296,  56504,
                37212, 11852, 7848,  16032, 28204, 36684, 40284, 5376,  17780,
                47920, 24368, 61520, 8376,  63832, 20384, 5848,  12608, 63308,
                3504,  26300, 43516, 5248,  65244, 46092, 39504, 7340,  62668,
                15464, 44476, 11948, 27328, 48492, 24028, 42944, 51612, 28592,
                8736,  55280, 47060, 24504, 39532, 56988, 8348,  38124, 5924,
                43108, 54684, 10328, 27032, 4620,  22016, 49684, 55424, 24972,
                44124, 4260,  37736, 60436, 5556,  53584, 16128, 36556, 60988,
                51196, 42088, 22384, 16952, 46896, 64084, 19480, 9904,  30100,
                14840, 36468,
            },
            {
                4780,  65288, 46368, 57944, 40924, 63572, 44104, 24816, 12084,
                53096, 2936,  26488, 38620, 32860, 44432, 55996, 30552, 49180,
                34104, 12612, 19576, 51492, 31536, 42320, 26800, 48868, 22880,
                2216,  30360, 19744, 14348, 42448, 33788, 1384,  30568, 55440,
                14796, 58700, 48568, 61928, 21228, 32464, 828,   15496, 55244,
                35392, 6108,  40604, 23876, 14648, 58936, 39780, 16492, 62856,
                29464, 50084, 34148, 45148, 60880, 48844, 25512, 33376, 54428,
                59780, 43996, 19272, 4004,  47608, 39916, 44768, 30444, 49324,
                39004, 55092, 13376, 29820, 23000, 1864,  27380, 33868, 10268,
                29952, 59088, 1168,  40132, 63704, 14364, 58116, 33212, 21712,
                60244, 36784, 19396, 6296,  44180, 31200, 22268, 17976, 49060,
                33208, 3372,  63388, 45728, 51404, 30972, 9512,  42360, 16772,
                33316, 10712, 56112, 19020, 27892, 49388, 58388, 21336, 852,
                28324, 56088, 37632, 26352, 58180, 540,   35416, 57156, 43552,
                3064,  53444,
            },
            {
                42012, 30456, 38228, 15852, 28232, 6252,  2268,  19668, 61084,
                41152, 15652, 63048, 54028, 5984,  24012, 37512, 10248, 1096,
                53420, 65268, 3320,  57852, 46924, 6640,  13996, 18524, 61256,
                41088, 47416, 35280, 53680, 9440,  59724, 21224, 40740, 7416,
                26744, 36980, 9528,  16684, 44848, 54220, 7804,  39784, 24956,
                9668,  62992, 17996, 46344, 29292, 53264, 21100, 32976, 45764,
                2708,  23528, 40864, 132,   21400, 10864, 7088,  14480, 41424,
                2984,  12320, 38728, 31920, 15740, 56292, 26036, 59072, 9108,
                19700, 51568, 33612, 59956, 16628, 49704, 57556, 21304, 42512,
                37316, 54484, 22372, 34620, 52912, 19832, 2404,  46356, 16916,
                42024, 620,   50904, 63252, 13504, 3236,  53344, 59320, 28296,
                13272, 22688, 37496, 15096, 56396, 18936, 64952, 2956,  59364,
                46904, 51460, 24080, 41532, 32408, 2624,  11288, 40408, 47892,
                13532, 5256,  61748, 9104,  32108, 41676, 13152, 24892, 48652,
                61728, 21528,
            },
            {
                59156, 7180,  50496, 10224, 22284, 32724, 49344, 36904, 29708,
                7160,  35104, 49644, 18796, 1976,  58456, 14188, 62020, 41272,
                22132, 44012, 26092, 39696, 21592, 62304, 55252, 33704, 28680,
                57216, 11436, 25876, 63208, 4572,  49000, 16604, 65220, 45128,
                53224, 22772, 41828, 4040,  35056, 60172, 28844, 51836, 48512,
                31136, 56260, 4400,  34556, 61768, 7728,  3508,  55460, 13568,
                51608, 18068, 62192, 53948, 30384, 57276, 50832, 64900, 22228,
                30880, 27560, 58808, 54232, 23160, 11084, 4836,  36432, 15172,
                24076, 816,   41784, 8336,  37532, 43888, 12968, 61324, 2676,
                48172, 6660,  16808, 45556, 8792,  30632, 38464, 7428,  65140,
                12544, 25240, 32544, 56052, 27636, 35272, 45988, 10700, 41248,
                51604, 57720, 30180, 756,   25176, 38800, 35020, 13644, 28080,
                21048, 112,   61788, 13044, 45332, 64644, 25548, 30864, 62760,
                19420, 34876, 45752, 15468, 49328, 21252, 52296, 6196,  17180,
                33008, 11764,
            },
            {
                24440, 18520, 35224, 54656, 45272, 59880, 13616, 52396, 58204,
                45904, 23164, 11696, 42424, 29344, 45812, 50088, 26984, 17192,
                32016, 7944,  15108, 35120, 11036, 580,   37576, 43868, 7736,
                4152,  17184, 38784, 44492, 32140, 24252, 38108, 28768, 13060,
                2108,  62368, 30868, 50472, 25480, 11620, 18672, 3124,  36568,
                13324, 21672, 42104, 51180, 12472, 37760, 43584, 27324, 36132,
                57928, 6004,  26316, 13224, 38060, 43180, 18188, 35768, 45596,
                52796, 7720,  49056, 368,   34312, 65124, 50292, 42652, 62520,
                46716, 57172, 27960, 62172, 52992, 3948,  31540, 18572, 51664,
                25840, 64836, 29136, 59920, 12764, 56036, 49464, 26884, 35468,
                54084, 47944, 9204,  16436, 40636, 61056, 19720, 1200,  63728,
                7116,  17252, 47680, 43320, 7812,  60924, 48564, 5472,  53308,
                39800, 30512, 36340, 6408,  17376, 54476, 38244, 8628,  43432,
                53744, 23672, 29624, 55040, 2848,  62484, 37216, 59792, 27988,
                39332, 45856,
            },
            {
                56388, 116,   26512, 63864, 3156,  39920, 25056, 17776, 9992,
                508,   27432, 55516, 61452, 8908,  20940, 34356, 4640,  54804,
                47328, 56872, 60920, 50044, 30112, 52500, 24888, 12952, 49452,
                53196, 60628, 22036, 200,   58056, 10840, 56124, 5684,  34828,
                47592, 17916, 56860, 6464,  63612, 46212, 40944, 23076, 65384,
                44544, 60364, 2208,  25708, 19364, 48660, 64232, 23060, 10292,
                39908, 31872, 47928, 8396,  59352, 4532,  27192, 1440,  10280,
                62216, 16104, 20420, 44676, 41004, 27940, 18268, 3188,  30008,
                34732, 5980,  17936, 21744, 10744, 24544, 55536, 39820, 9064,
                14056, 35668, 40936, 4700,  24716, 42132, 61352, 22484, 4216,
                30132, 59472, 43240, 23048, 5216,  50168, 25008, 32128, 38532,
                26376, 34696, 62328, 11732, 53960, 21916, 32052, 44776, 18192,
                63160, 8828,  49996, 58804, 21452, 34484, 4316,  51936, 17072,
                3216,  59804, 7376,  39300, 25872, 10436, 30944, 46740, 1816,
                8916,  51548,
            },
            {
                36624, 43376, 13316, 31064, 20472, 8220,  55164, 42960, 33088,
                64372, 37896, 16372, 32264, 39440, 52756, 63184, 11928, 36460,
                2868,  28012, 19032, 5404,  45772, 58672, 16920, 64888, 20716,
                35888, 46016, 30500, 14784, 50988, 18496, 43400, 52020, 59360,
                25812, 39884, 10468, 20604, 37236, 14824, 53124, 57724, 5436,
                16488, 28616, 33296, 9520,  59212, 30056, 15956, 4788,  60304,
                19104, 44108, 63196, 16332, 34008, 20884, 47404, 55468, 32152,
                39992, 25956, 36308, 61388, 13292, 8696,  53464, 21348, 55492,
                11388, 39948, 50528, 32752, 47904, 36036, 64132, 28468, 46228,
                21780, 57192, 49164, 19468, 33564, 1892,  15392, 51848, 10664,
                39284, 20124, 2396,  63912, 37380, 11960, 54740, 15720, 44464,
                56152, 3884,  19792, 40392, 27632, 15844, 58024, 10796, 25644,
                55680, 14988, 43164, 27412, 47400, 56868, 13828, 24364, 48612,
                36520, 12220, 42436, 64476, 18476, 44344, 14496, 54576, 23316,
                64660, 15820,
            },
            {
                4964,  61656, 48828, 52708, 37788, 60912, 27932, 3900,  48676,
                20860, 51424, 5520,  59536, 25128, 6672,  18208, 42168, 23588,
                64340, 38576, 13384, 41808, 22424, 9152,  39864, 32276, 1928,
                26276, 8840,  40928, 62756, 25316, 36540, 8164,  21476, 3316,
                15136, 33456, 43992, 54336, 29156, 2492,  32796, 26652, 10856,
                48188, 38816, 52128, 56520, 45392, 424,   40800, 54112, 49740,
                28828, 1476,  52936, 23988, 41716, 51300, 63596, 14972, 23260,
                57828, 2588,  50888, 6396,  30460, 59248, 45836, 38104, 25276,
                60208, 14352, 63608, 4376,  44580, 15828, 7384,  3160,  59528,
                31940, 532,   11044, 54332, 63188, 46520, 36908, 18588, 44536,
                56640, 14608, 34468, 46180, 28628, 58188, 7676,  48020, 22340,
                10164, 49660, 59356, 32896, 5868,  51296, 1724,  41688, 37100,
                4276,  34116, 23540, 2280,  10720, 30104, 41116, 63744, 28340,
                58208, 32348, 21376, 50260, 33880, 4940,  58004, 20004, 34364,
                42272, 29416,
            },
            {
                22176, 32836, 17412, 5672,  11428, 47408, 15324, 35880, 62108,
                12716, 41652, 46872, 2424,  14332, 44816, 31316, 59108, 50352,
                9884,  45212, 54008, 33372, 62440, 3560,  27772, 55144, 42400,
                59876, 12456, 54664, 5840,  46576, 32952, 64148, 41336, 31036,
                61080, 48948, 964,   23924, 58944, 49628, 8436,  43272, 35232,
                55116, 18732, 6580,  13528, 35908, 24448, 31972, 8196,  35368,
                22120, 11684, 37452, 3764,  29748, 9456,  36696, 5816,  43008,
                11468, 46116, 18092, 55940, 23740, 33860, 15380, 1420,  48416,
                7884,  43012, 26596, 54468, 29760, 57908, 19768, 41244, 52636,
                37420, 17044, 43748, 26600, 7964,  28792, 58996, 5720,  64292,
                27160, 50272, 8268,  53056, 17012, 33064, 41420, 2048,  65504,
                30288, 14280, 23840, 45944, 36452, 64520, 30052, 48192, 60792,
                20272, 51880, 65116, 38680, 59628, 7328,  44992, 19616, 2036,
                9660,  46764, 15824, 340,   52972, 27684, 40472, 49256, 3440,
                12160, 59344,
            },
            {
                9836,  40088, 57364, 24712, 44280, 30204, 58620, 23016, 7132,
                30900, 26416, 19116, 35040, 54652, 48808, 26816, 1752,  15884,
                29944, 21312, 944,   25712, 50812, 35756, 47168, 14628, 6436,
                48508, 34444, 20128, 28868, 2044,  13572, 48260, 27264, 55444,
                19460, 8824,  64892, 13404, 39100, 17556, 62528, 21220, 60748,
                1016,  23760, 64404, 42528, 20504, 62508, 47540, 57332, 14832,
                64572, 45576, 58732, 55216, 13564, 60968, 18640, 28364, 52668,
                33368, 64316, 27248, 39108, 3952,  43368, 52220, 64436, 32280,
                17644, 36312, 20632, 380,   38292, 13040, 49604, 25980, 10084,
                23748, 60552, 50560, 39156, 22040, 13424, 32004, 41276, 23868,
                96,    31256, 62060, 21124, 3492,  60892, 26840, 19016, 35300,
                52888, 41984, 1036,  55580, 9376,  18788, 23144, 13964, 7876,
                27920, 46064, 12476, 18540, 49112, 33328, 15972, 60676, 39220,
                53488, 25752, 56768, 37664, 61672, 13320, 8540,  63436, 26048,
                53904, 46956,
            },
            {
                51244, 28024, 2972,  35052, 64740, 19356, 940,   40044, 56392,
                52324, 10320, 57684, 65492, 22128, 8480,  38120, 57124, 35536,
                6900,  61280, 56572, 17140, 11400, 57632, 19384, 63632, 23248,
                51952, 16776, 57372, 39640, 60664, 52932, 17124, 4064,  11380,
                37064, 52496, 27756, 35000, 46992, 3924,  30604, 40364, 13924,
                46504, 32036, 27536, 50140, 4884,  16840, 10648, 27184, 42128,
                6268,  17636, 32628, 25820, 48488, 44596, 440,   39392, 59824,
                21112, 7988,  14028, 48272, 62100, 9652,  19956, 28608, 5276,
                57004, 50948, 59540, 9324,  23044, 61488, 33536, 43220, 63272,
                4472,  14584, 34156, 5416,  57760, 52660, 48180, 9640,  55024,
                36512, 12976, 40032, 25356, 43100, 48500, 12744, 56568, 39000,
                8060,  62536, 17200, 26196, 38440, 59692, 44120, 33652, 56780,
                40656, 624,   32188, 54908, 26056, 4232,  51068, 35608, 22840,
                5252,  43496, 30248, 6704,  23784, 44952, 31056, 35720, 16860,
                38264, 19448,
            },
            {
                7564,  63140, 14984, 41564, 53120, 8024,  50076, 45180, 17308,
                37276, 4872,  43580, 28484, 40240, 12300, 62608, 19820, 52248,
                47208, 41252, 31860, 38216, 4752,  42632, 29904, 9496,  38616,
                31580, 4460,  44632, 24496, 9148,  21796, 38340, 58428, 24172,
                45956, 42056, 6584,  21652, 56224, 11096, 50720, 25500, 53004,
                7580,  58072, 11564, 39668, 60124, 53684, 37672, 2932,  52548,
                23436, 49988, 40112, 7188,  20456, 31300, 56672, 24784, 49240,
                3224,  37296, 31364, 54540, 21908, 35696, 12548, 40816, 23484,
                45032, 11744, 30572, 40672, 46172, 53408, 2776,  17460, 30964,
                55328, 47780, 29224, 65156, 18072, 1596,  34848, 20476, 16384,
                49364, 45572, 57888, 10596, 6624,  36656, 51340, 23524, 4240,
                28084, 46980, 32204, 51888, 3088,  11992, 49748, 5364,  53012,
                16820, 61952, 8928,  41788, 22196, 63444, 11720, 55816, 31184,
                13656, 65520, 18616, 11168, 48432, 59524, 20744, 5788,  56368,
                860,   43656,
            },
            {
                23676, 55616, 47952, 21492, 11060, 27568, 32760, 13084, 25100,
                60128, 33264, 15256, 92,    50620, 32508, 3832,  44136, 24048,
                10412, 27104, 13868, 49156, 62996, 24284, 45680, 1216,  61000,
                26980, 11512, 65392, 36028, 49796, 30792, 42788, 63336, 33884,
                5208,  14356, 60196, 31868, 44352, 63904, 19920, 4976,  36456,
                43976, 17356, 34488, 1692,  30196, 21472, 44592, 33284, 61348,
                29320, 1076,  62316, 10960, 35728, 64816, 16272, 9040,  13068,
                45028, 57612, 17304, 41340, 556,   58448, 47084, 53704, 62560,
                2508,  34724, 16188, 65396, 6552,  28168, 11580, 36860, 45692,
                8416,  20532, 40804, 11264, 45284, 25760, 42692, 59736, 62892,
                3204,  28496, 18236, 54128, 31880, 63520, 16068, 44640, 11124,
                60600, 20712, 13192, 57976, 42460, 29544, 62788, 26628, 21168,
                37848, 30176, 58536, 14724, 36376, 43296, 27776, 844,   47860,
                40744, 50648, 58788, 34172, 39592, 15508, 51996, 41856, 62420,
                29184, 33500,
            },
            {
                11700, 37352, 30928, 1704,  58384, 38416, 54812, 62284, 2700,
                47708, 21396, 53468, 61844, 17584, 25636, 55880, 14880, 34100,
                64764, 3108,  53336, 20812, 7972,  54956, 33844, 17564, 52744,
                47812, 41672, 1740,  15436, 54376, 6780,  12892, 1580,  51200,
                29036, 55012, 17084, 25704, 68,    37916, 15384, 59832, 28888,
                55004, 22644, 62852, 47688, 8804,  25212, 59052, 12344, 18964,
                38708, 56536, 15760, 42988, 54244, 4988,  51696, 42368, 34060,
                61420, 28632, 6676,  51252, 25608, 30336, 16620, 7976,  38136,
                27176, 55860, 48928, 25264, 18976, 56720, 50176, 22104, 58660,
                836,   26900, 61756, 36660, 3916,  56324, 29492, 7440,  38580,
                22948, 5828,  35212, 60076, 21748, 2856,  29796, 58344, 33724,
                40148, 50440, 5576,  34776, 22580, 16148, 39776, 3784,  10740,
                48780, 24612, 45260, 2724,  53580, 6356,  20320, 58352, 17536,
                8524,  26876, 21888, 4888,  54924, 1540,  25488, 10780, 47344,
                14592, 60224,
            },
            {
                45444, 5852,  16664, 61484, 43812, 23796, 18320, 6452,  41736,
                29240, 9208,  38220, 45300, 7512,  48240, 60788, 40892, 5588,
                18956, 58504, 43116, 36124, 28144, 12216, 40144, 57836, 14204,
                21520, 32888, 59544, 19444, 45460, 27912, 57048, 17932, 47576,
                20916, 62096, 40164, 48388, 8520,  54184, 33052, 41636, 9924,
                2976,  49532, 13284, 42268, 56208, 15708, 50336, 5960,  46636,
                9336,  34384, 24852, 47932, 28684, 21860, 38032, 27036, 19348,
                53676, 23200, 11768, 65012, 44336, 4956,  60576, 32932, 19628,
                13796, 5976,  43484, 1708,  34140, 39328, 14636, 63996, 41792,
                33048, 52192, 13908, 54488, 23212, 49864, 15416, 12284, 32912,
                53500, 43800, 50048, 14216, 41344, 47228, 8924,  18840, 54556,
                288,   25556, 43804, 63944, 8640,  54976, 47240, 57240, 35500,
                64492, 6808,  19556, 50688, 29052, 60396, 46000, 38352, 33520,
                62208, 3016,  37004, 46652, 28804, 64032, 32740, 37544, 22044,
                3720,  53032,
            },
            {
                19092, 25436, 50884, 34196, 4328,  48628, 14284, 36196, 51160,
                63668, 12408, 58160, 23612, 30696, 36484, 10940, 28956, 50912,
                37972, 30872, 16124, 336,   46592, 60644, 22884, 3396,  36616,
                7556,  56120, 25424, 5384,  35080, 61432, 23728, 32452, 38140,
                10116, 3100,  35404, 12400, 59060, 22056, 26820, 46336, 61632,
                19324, 31788, 37904, 26956, 4348,  36188, 31272, 40500, 65400,
                22152, 51644, 4236,  60504, 12636, 1324,  47012, 62624, 6008,
                2196,  38724, 47784, 34704, 14736, 39648, 23008, 48284, 51840,
                41644, 58608, 21508, 52552, 61236, 7680,  29632, 3472,  25052,
                10044, 48364, 18800, 6376,  31552, 39828, 64116, 46720, 57472,
                25528, 10300, 64480, 1144,  27172, 52812, 24412, 36852, 64916,
                14512, 30688, 17856, 53332, 37172, 1676,  19392, 31680, 14576,
                42064, 54296, 11932, 39396, 32864, 15732, 10476, 24212, 54160,
                44100, 16044, 56924, 10156, 42520, 19048, 7244,  50120, 58416,
                27688, 40412,
            },
            {
                64288, 56836, 41288, 28504, 10236, 65168, 31420, 56932, 20032,
                25944, 34480, 3716,  42504, 19472, 1548,  54192, 21468, 46828,
                9028,  54712, 24872, 63660, 51376, 5220,  30804, 49980, 64380,
                44028, 29328, 51036, 39964, 10920, 49016, 41816, 7832,  64212,
                45012, 24280, 53152, 30312, 42900, 51092, 1992,  13576, 56732,
                24624, 6404,  64376, 52836, 20264, 60952, 300,   57228, 27800,
                14312, 44444, 31208, 18888, 40436, 58960, 32320, 14720, 56908,
                49620, 31684, 8776,  20160, 57880, 54632, 10456, 2772,  64180,
                28664, 9388,  36736, 30844, 11032, 44036, 47588, 54140, 16912,
                38508, 60760, 44260, 35292, 59840, 9412,  21300, 760,   41296,
                16652, 30576, 19828, 39476, 33580, 57176, 4384,  42152, 46468,
                7276,  49372, 59812, 11444, 24472, 28640, 44576, 59584, 23156,
                1128,  33744, 60936, 26240, 7732,  64912, 49920, 2172,  29692,
                12380, 51392, 31148, 24056, 60712, 13980, 53652, 44044, 17444,
                9124,  35172,
            },
            {
                212,   8000,  13484, 54256, 19428, 39272, 44820, 8704,  748,
                46664, 15528, 55960, 50344, 64924, 13904, 59176, 26760, 2556,
                61340, 12780, 44812, 33256, 19600, 15024, 42264, 26528, 9784,
                17896, 13076, 2660,  63724, 21992, 16120, 564,   55556, 13708,
                27368, 57368, 5036,  19024, 64620, 6144,  17436, 37588, 33832,
                51668, 40252, 16632, 10308, 46316, 43292, 24264, 17740, 53180,
                3000,  37628, 62784, 7272,  54588, 35444, 10072, 25524, 41404,
                17608, 63320, 43456, 26028, 1348,  29336, 45200, 35028, 24656,
                17404, 46560, 62900, 15156, 24004, 59196, 20368, 35380, 57596,
                4992,  22464, 28732, 1780,  51760, 17388, 27992, 36796, 5056,
                51176, 61328, 7212,  48316, 12464, 16968, 62044, 10876, 21148,
                27040, 38456, 4204,  32904, 47868, 61860, 6600,  9852,  51224,
                28348, 47068, 16656, 55900, 22648, 40896, 36144, 57668, 6544,
                20768, 64192, 41064, 5728,  48600, 34864, 26712, 2344,  61192,
                30852, 48832,
            },
            {
                38324, 22296, 46488, 60520, 24968, 3384,  52900, 22520, 30152,
                60336, 39596, 27808, 5200,  32816, 38364, 43728, 16964, 35008,
                41304, 22476, 6444,  39148, 10540, 56996, 34984, 59776, 47532,
                39056, 54452, 33460, 46324, 57912, 31004, 52536, 35996, 19836,
                40828, 49916, 15660, 34208, 39360, 28176, 47992, 62716, 8500,
                43172, 2524,  58760, 29764, 34108, 7496,  12724, 48380, 34904,
                8748,  55944, 25816, 49144, 16364, 22864, 52468, 45600, 7532,
                29120, 13144, 59704, 52432, 37488, 16756, 61116, 12948, 57016,
                5632,  32748, 252,   41068, 49940, 4592,  27868, 12972, 42332,
                31740, 63544, 12100, 56452, 25648, 43132, 48164, 58668, 54788,
                22336, 34980, 44468, 59172, 23272, 29068, 49792, 35268, 31652,
                52984, 57328, 20192, 40016, 13688, 52340, 41584, 17880, 38024,
                63480, 5964,  43708, 2552,  48168, 19012, 13580, 42800, 26508,
                47444, 35420, 17900, 972,   56412, 21448, 39844, 12264, 55560,
                23936, 15012,
            },
            {
                62880, 29716, 33424, 5692,  49752, 36780, 16392, 62308, 42388,
                10808, 53644, 18296, 8512,  47580, 22984, 52472, 5836,  63892,
                29960, 56028, 48152, 60176, 29016, 53696, 7104,  20984, 728,
                24604, 61132, 19884, 37656, 8892,  26592, 44408, 5228,  59856,
                29872, 2212,  61468, 46084, 10668, 23556, 55028, 30472, 21160,
                12240, 26660, 47720, 22584, 54260, 61828, 38704, 28892, 63468,
                21364, 41236, 11540, 29388, 43716, 2712,  65072, 20100, 37340,
                55380, 23720, 4680,  33364, 8052,  49028, 21948, 40676, 50584,
                38172, 53068, 26468, 55796, 18456, 37724, 65004, 1100,  51124,
                8332,  46388, 16300, 40396, 7076,  62676, 11324, 33232, 14088,
                8632,  26668, 3360,  37924, 53884, 1560,  43856, 5784,  60904,
                15512, 9452,  45552, 63200, 2820,  22160, 34844, 56700, 25248,
                12836, 20648, 35788, 31020, 58872, 5128,  51756, 32180, 61708,
                8700,  38988, 58848, 11632, 28696, 45672, 64964, 4660,  33100,
                42872, 52104,
            },
            {
                2888,  57636, 15792, 43140, 11536, 27732, 58140, 33228, 6336,
                49468, 24268, 36700, 61904, 12272, 29376, 57100, 9840,  49168,
                19840, 3708,  15788, 26180, 1636,  37108, 44320, 16540, 31736,
                50536, 6608,  28432, 14908, 3480,  62656, 17820, 23684, 47624,
                11516, 37784, 22144, 7932,  52596, 59448, 14864, 4340,  44828,
                60764, 55508, 36760, 14184, 5284,  19412, 50672, 1776,  45232,
                15816, 59748, 5884,  33972, 57656, 38940, 5484,  31192, 59236,
                80,    50256, 40372, 19492, 62500, 42892, 3176,  31132, 8768,
                20468, 59820, 12440, 44672, 7236,  33260, 53664, 45656, 25196,
                19524, 54608, 33616, 49612, 36480, 20328, 2796,  29720, 45244,
                65356, 49968, 15836, 31008, 10076, 63888, 19192, 55808, 24112,
                41700, 824,   29968, 26172, 54940, 16232, 31100, 4924,  48092,
                58508, 40520, 53100, 10392, 27280, 63316, 23360, 128,   54784,
                14408, 24876, 50008, 32680, 52852, 8012,  16716, 37908, 47800,
                19300, 10552,
            },
            {
                45628, 40180, 19964, 55388, 64044, 1240,  46120, 13356, 20640,
                35088, 2060,  45428, 57988, 20372, 396,   40096, 24832, 13748,
                37336, 32428, 43072, 50848, 65292, 22924, 12684, 52584, 63432,
                41864, 11784, 57168, 43676, 53988, 39348, 50340, 34392, 7452,
                65112, 53448, 26536, 43316, 32472, 1392,  41320, 35780, 50984,
                18260, 32360, 876,   65000, 41744, 25860, 57676, 10888, 32236,
                24740, 51288, 46604, 19756, 13976, 50420, 27424, 47164, 11968,
                34616, 16080, 46448, 28020, 12512, 53984, 25060, 65100, 15556,
                47792, 4496,  23224, 30424, 61008, 10216, 22540, 14140, 30188,
                39240, 59376, 2288,  23396, 14404, 53812, 60216, 24304, 38636,
                18680, 41728, 56608, 21356, 47024, 40348, 26836, 12592, 37536,
                51360, 18668, 59216, 36368, 49440, 10536, 65436, 44316, 468,
                28796, 7852,  62052, 15104, 45256, 39176, 34840, 17456, 43908,
                30160, 3352,  61136, 19860, 42724, 25360, 58272, 29548, 62256,
                6824,  27864,
            },
            {
                60280, 25420, 8156,  35364, 30904, 23532, 39812, 52068, 26964,
                64676, 54640, 14536, 30772, 43028, 51120, 34336, 59752, 45484,
                53400, 61528, 11268, 18772, 8128,  46680, 57832, 25568, 38028,
                3728,  34716, 48264, 24216, 32304, 10776, 58540, 14192, 43060,
                31596, 16592, 57892, 13160, 49008, 19992, 26988, 64128, 7068,
                25068, 39892, 9728,  49280, 30704, 16324, 35624, 43208, 55288,
                3968,  36828, 64256, 840,   24108, 62056, 9200,  18064, 53528,
                43588, 64072, 9832,  58216, 32312, 6368,  36176, 56044, 27928,
                43340, 34856, 63804, 39944, 16336, 48848, 58148, 41600, 63060,
                6456,  10804, 27420, 64760, 31112, 42344, 47236, 6316,  52088,
                512,   11832, 61660, 36184, 7124,  15168, 32756, 48680, 3904,
                64600, 34192, 12200, 44020, 7448,  23772, 38444, 52808, 14268,
                33440, 50448, 24536, 18780, 55572, 6964,  11956, 58076, 51104,
                21660, 46360, 6040,  15184, 37192, 1480,  49756, 13516, 22852,
                53656, 36244,
            },
            {
                12280, 52728, 47288, 4312,  50508, 17784, 8796,  56480, 3276,
                17232, 41504, 6856,  25728, 63428, 4596,  15664, 27824, 7860,
                1964,  23476, 56420, 35964, 40796, 33084, 4880,  29168, 9544,
                60204, 21612, 17164, 64416, 1000,  20252, 28772, 4456,  55836,
                21324, 320,   35860, 5708,  62820, 38868, 56304, 11144, 47200,
                15328, 58156, 21728, 45092, 53300, 3324,  59596, 8208,  20560,
                28276, 12648, 40504, 30476, 53940, 35812, 41708, 60036, 22808,
                26472, 2964,  20944, 37964, 51472, 17320, 45776, 752,   10820,
                58100, 18556, 1952,  51960, 28140, 4720,  36008, 2560,  17332,
                51896, 36916, 44308, 57028, 4272,  9560,  16932, 35504, 57900,
                28196, 33152, 23968, 4068,  51500, 60716, 58172, 43120, 8988,
                28524, 47088, 4808,  20824, 58184, 42176, 27232, 18376, 60732,
                21884, 45960, 37596, 1508,  32660, 49276, 28412, 41292, 9368,
                65240, 33604, 40172, 53608, 63240, 10152, 34900, 55928, 3564,
                41412, 17100,
            },
            {
                664,   21996, 38828, 14664, 59368, 42700, 62912, 29644, 37388,
                48216, 32208, 11220, 38796, 18696, 55264, 47508, 21444, 64648,
                41908, 30416, 5168,  27004, 14800, 61908, 19936, 49648, 15460,
                45588, 30728, 53476, 7176,  41360, 46484, 61872, 38476, 25452,
                51208, 40264, 47032, 29308, 24316, 16788, 2920,  33696, 29696,
                54080, 3848,  62136, 27692, 11868, 38848, 23492, 47960, 62768,
                52648, 18184, 58624, 7204,  44792, 15900, 2252,  33148, 6564,
                39732, 30992, 55628, 4416,  24144, 60536, 39824, 21552, 50924,
                31612, 38068, 9620,  46472, 57056, 20740, 44092, 25660, 32268,
                47112, 21768, 13056, 18760, 48708, 39608, 26636, 21580, 63248,
                8284,  48248, 54284, 44416, 29752, 25072, 2024,  19680, 54716,
                22824, 14672, 53576, 63080, 32504, 1920,  55812, 6724,  35972,
                10196, 57344, 5312,  43088, 64428, 21072, 60120, 2928,  36676,
                26132, 12364, 57164, 27820, 23064, 45084, 31336, 19988, 46796,
                61364, 32520,
            },
            {
                28540, 65260, 56612, 26504, 33164, 20864, 12540, 4904,  44696,
                22080, 56960, 60104, 50144, 8876,  36388, 32040, 11780, 39008,
                17944, 51528, 45120, 59252, 48064, 2136,  53076, 42476, 56592,
                1436,  39772, 13188, 50820, 25104, 35384, 16036, 48756, 8708,
                63616, 18424, 59200, 9828,  54352, 50456, 44636, 60004, 22552,
                42516, 37000, 32732, 6776,  17632, 61104, 33944, 14396, 41368,
                5476,  33000, 48624, 10460, 26700, 21240, 49068, 56560, 13752,
                51740, 61296, 47476, 14464, 43080, 8108,  29704, 13340, 63000,
                25448, 54472, 14952, 24332, 33428, 12532, 60424, 9188,  55588,
                62032, 29216, 192,   53000, 33436, 61184, 54868, 12860, 43276,
                16076, 40200, 19808, 13736, 9492,  39620, 16296, 36524, 30916,
                61268, 40920, 26044, 37428, 16732, 12884, 50092, 46812, 29668,
                62824, 16560, 26844, 54136, 13228, 25036, 46984, 15888, 55748,
                18880, 7336,  48788, 3980,  17384, 59456, 6448,  39164, 25152,
                9000,  50164,
            },
            {
                34936, 6060,  44288, 10512, 2188,  53840, 49080, 25108, 61252,
                15140, 5664,  27996, 45852, 24168, 62120, 2968,  52360, 57964,
                35116, 13804, 9524,  20736, 38176, 31348, 11976, 35124, 23028,
                62468, 26704, 36592, 59772, 10036, 55544, 2780,  33096, 12756,
                28076, 3192,  43640, 14560, 31648, 6504,  35632, 18576, 8612,
                49716, 13620, 20148, 51140, 40832, 55448, 29864, 204,   25804,
                45800, 37824, 23108, 55404, 60828, 38744, 64768, 28692, 44064,
                8764,  17920, 35132, 27644, 64464, 49932, 34352, 47176, 4028,
                41168, 6152,  61720, 42656, 7044,  52656, 38424, 49848, 14556,
                5204,  42116, 58496, 38020, 23296, 5612,  29824, 3220,  50744,
                31328, 2184,  62524, 55892, 33828, 64200, 49396, 57404, 6708,
                44888, 1052,  9664,  51136, 29192, 40260, 60260, 20084, 3652,
                39104, 51476, 31728, 40428, 9060,  36000, 6020,  31216, 45008,
                39296, 60876, 30388, 43328, 36260, 52392, 11824, 64456, 15584,
                43624, 55196,
            },
            {
                13256, 18684, 48524, 36716, 61540, 41048, 31324, 36012, 10060,
                51452, 33804, 20060, 1616,  14212, 42624, 28952, 6744,  25572,
                49304, 1108,  63384, 53612, 24296, 7608,  64488, 28028, 18004,
                8420,  54904, 4080,  19712, 29368, 42404, 22092, 57840, 44760,
                53540, 34320, 37876, 21004, 61628, 41160, 12832, 65380, 26872,
                596,   56724, 63464, 45760, 2880,  21920, 49160, 11180, 57288,
                16828, 63640, 1732,  14788, 32064, 3620,  11604, 19724, 36868,
                25344, 644,   40952, 10356, 19696, 1936,  55864, 16892, 59012,
                20568, 35112, 49128, 19100, 30640, 64536, 980,   28004, 18984,
                35188, 24324, 8692,  16092, 45040, 10764, 64508, 46532, 35872,
                57456, 22992, 37228, 26748, 6216,  45964, 22000, 10944, 52504,
                33532, 18112, 47596, 56880, 4024,  23692, 8788,  33144, 43300,
                13660, 23056, 2272,  59104, 19968, 49624, 53504, 62456, 1184,
                23756, 51184, 21080, 14452, 352,   47660, 27124, 56912, 30436,
                2680,  21264,
            },
            {
                40552, 62568, 29892, 23640, 16180, 7108,  18312, 55348, 152,
                39768, 43520, 65488, 53172, 38472, 55876, 17592, 60412, 44512,
                22316, 29740, 34036, 42664, 16624, 58648, 44772, 4364,  40904,
                49368, 32144, 47124, 15176, 65472, 49480, 6860,  16884, 24100,
                60232, 5000,  49224, 26288, 55684, 1576,  23452, 52680, 46832,
                39096, 29460, 10068, 25400, 35372, 8144,  64800, 42384, 36168,
                53212, 8940,  28936, 43224, 51912, 46700, 6276,  53164, 59040,
                48292, 62488, 54432, 31716, 58404, 23464, 38568, 27156, 9356,
                45124, 28644, 1668,  58544, 36652, 16292, 22780, 40596, 45324,
                63192, 48876, 30892, 60372, 51056, 20104, 40760, 25148, 17980,
                7080,  10588, 47388, 52628, 18336, 4,     42392, 27964, 14072,
                24236, 64808, 35488, 20932, 61808, 45496, 52776, 25988, 64988,
                48644, 56228, 7260,  43816, 63752, 29236, 17572, 41848, 13608,
                34700, 10296, 55208, 64000, 32624, 40976, 18852, 4776,  48768,
                37376, 59504,
            },
            {
                51724, 1760,  8740,  52700, 58276, 46348, 27592, 63344, 22632,
                58892, 16712, 29608, 11616, 34712, 21804, 47192, 9892,  15448,
                40652, 5236,  61316, 26828, 2804,  48328, 14048, 36364, 57744,
                11440, 21564, 43596, 38392, 772,   34412, 61512, 39688, 31016,
                10752, 19452, 64336, 8224,  45712, 29028, 37116, 57384, 5704,
                33600, 17060, 43428, 54004, 15312, 58284, 27300, 18632, 4424,
                31504, 20800, 59284, 39680, 18548, 35336, 23576, 30184, 15548,
                4688,  22208, 13296, 51044, 43448, 6684,  12484, 52968, 32648,
                65328, 13972, 54196, 11008, 25912, 47028, 57776, 10304, 54896,
                2692,  12708, 53728, 26480, 2124,  32960, 14092, 58816, 52176,
                28760, 60728, 40484, 13036, 57984, 30768, 61864, 38664, 3576,
                58980, 40496, 11896, 5800,  31144, 14472, 36528, 256,   11196,
                18360, 28332, 37440, 14792, 33400, 11000, 3584,  26568, 58264,
                46824, 5144,  38156, 25896, 8300,  59948, 23356, 34332, 54056,
                10956, 24420,
            },
            {
                16472, 45624, 33608, 39188, 13592, 4156,  42648, 50200, 12912,
                26640, 3660,  48616, 7744,  59796, 4616,  31128, 64344, 36732,
                52724, 56676, 12332, 19320, 39508, 55612, 29104, 52352, 24660,
                63656, 5808,  58824, 25680, 12292, 53244, 27140, 46844, 2528,
                52116, 42584, 13460, 32936, 17036, 51112, 10632, 20024, 13860,
                61848, 22088, 59760, 5100,  31832, 38608, 52332, 47172, 13388,
                61736, 50268, 25460, 6968,  12516, 58052, 62196, 42048, 33940,
                45376, 39092, 28740, 3132,  35900, 46864, 61060, 41768, 2504,
                50412, 23004, 39716, 43852, 4092,  51704, 5916,  30072, 33352,
                21412, 39172, 6632,  35832, 42732, 55924, 8188,  38272, 832,
                43648, 15500, 32552, 4768,  23976, 36044, 49764, 19952, 55248,
                30268, 50628, 26588, 43416, 53832, 17840, 59400, 41328, 54760,
                34884, 59944, 45372, 24940, 47996, 57508, 38748, 51812, 20216,
                28856, 61360, 42720, 18148, 53328, 12876, 45544, 63456, 7480,
                42680, 28252,
            },
            {
                4372,  57144, 25764, 64228, 21116, 30952, 34608, 9036,  38128,
                46756, 32664, 56788, 24388, 42100, 50944, 26680, 504,   20048,
                32132, 8076,  46036, 50952, 34764, 9752,  21016, 220,   16992,
                33760, 51060, 30088, 18140, 56336, 7952,  20388, 14340, 36108,
                56832, 25564, 39076, 58484, 3036,  60500, 42120, 31232, 44964,
                50424, 2408,  40572, 48392, 12260, 20840, 1164,  33896, 23864,
                44284, 37112, 3164,  48332, 54708, 26952, 1232,  8528,  56188,
                11004, 64908, 18908, 60100, 16064, 25484, 30344, 21232, 17788,
                36880, 7404,  61172, 17120, 33912, 63648, 14044, 42784, 60804,
                15704, 47096, 65280, 17708, 22312, 60964, 48544, 31636, 23468,
                62616, 20584, 50320, 44876, 65300, 10136, 15640, 6568,  45700,
                9008,  16260, 1908,  63228, 38876, 7892,  49352, 22188, 30064,
                5116,  9408,  21024, 1876,  54032, 6352,  22956, 63120, 8388,
                33496, 15260, 2480,  48052, 30276, 4196,  38488, 15912, 20448,
                32076, 61128,
            },
            {
                36964, 19440, 11356, 48344, 924,   56244, 60256, 18804, 5500,
                54044, 20752, 36956, 62544, 17952, 13492, 39288, 55576, 48968,
                14128, 59744, 23420, 3636,  65064, 31584, 61100, 46780, 43068,
                9024,  40124, 3524,  61868, 44652, 32284, 41756, 50804, 63732,
                6172,  30032, 48496, 21284, 35544, 24812, 4692,  63396, 26464,
                8324,  36100, 23972, 28708, 63232, 45100, 60972, 7520,  55184,
                10108, 15940, 65180, 32736, 44824, 15200, 38644, 21008, 50792,
                26168, 32516, 7400,  48740, 53876, 9636,  63476, 5700,  57568,
                47504, 26556, 31172, 55436, 20044, 38668, 27492, 50196, 7628,
                25192, 58152, 29088, 10880, 45348, 3748,  27640, 11688, 53544,
                5396,  35612, 56664, 2568,  27240, 54132, 43360, 60496, 34676,
                22728, 48124, 56280, 32700, 23616, 28036, 3536,  13000, 46188,
                62932, 39500, 50520, 61500, 18036, 41516, 31412, 12616, 44048,
                50436, 55980, 24456, 35976, 58568, 50920, 27724, 55848, 1424,
                44732, 50372,
            },
            {
                8236,  41280, 54860, 28928, 43540, 15744, 51560, 24484, 64468,
                41812, 14912, 10012, 1504,  45192, 33756, 58924, 7332,  24808,
                41924, 28452, 37080, 44112, 15864, 25520, 6156,  37500, 56852,
                27956, 14776, 47788, 36576, 23932, 1680,  60456, 10088, 22908,
                17848, 72,    54560, 7780,  46724, 15112, 53460, 37928, 17992,
                58928, 55100, 16004, 51676, 9272,  17472, 40296, 26324, 58476,
                30192, 19816, 40464, 22588, 9160,  29324, 63680, 47636, 17132,
                2456,  57692, 41656, 22844, 34256, 39708, 14616, 44216, 33056,
                53324, 11708, 592,   46040, 8744,  23488, 2792,  56604, 19244,
                34612, 40824, 452,   52300, 37608, 15068, 63820, 18508, 46372,
                39652, 9708,  29504, 17148, 38560, 21200, 31988, 1396,  25828,
                62188, 37040, 19464, 10788, 44568, 60816, 57060, 33648, 52012,
                16116, 26380, 32248, 13312, 35928, 28104, 47040, 428,   37008,
                19080, 7072,  65516, 11092, 21292, 41332, 8860,  62036, 35196,
                22800, 13952,
            },
            {
                30584, 59580, 4912,  22988, 36488, 9868,  38932, 28192, 2752,
                57584, 30316, 51728, 27292, 54272, 22060, 10756, 46432, 34996,
                2372,  63420, 18504, 11280, 58120, 41144, 52840, 12900, 19132,
                62684, 53620, 21440, 11152, 52028, 16404, 38468, 29332, 45404,
                34268, 61592, 12504, 43424, 64896, 28872, 10052, 48916, 796,
                41748, 11856, 33956, 4368,  56596, 30980, 36344, 49812, 3896,
                42756, 52880, 392,   59128, 51632, 35828, 4932,  42968, 54852,
                36404, 14068, 45780, 27680, 76,    56628, 51328, 3336,  23544,
                62236, 15860, 40220, 58012, 49568, 61792, 35712, 45748, 11248,
                53844, 5748,  48900, 23800, 31024, 54744, 41696, 34204, 59336,
                25592, 13684, 63484, 47044, 7548,  58868, 11668, 40316, 53552,
                13264, 41712, 6072,  52336, 15236, 36032, 19928, 42180, 24072,
                984,   58036, 43236, 7300,  55484, 64736, 16228, 52764, 60348,
                26968, 40248, 31688, 45140, 868,   16648, 24120, 46752, 11464,
                52996, 65056,
            },
            {
                47268, 16804, 33888, 53284, 63508, 6784,  45476, 48920, 11556,
                34348, 44516, 6800,  40004, 64796, 3760,  28816, 62040, 16248,
                52440, 5956,  49592, 54496, 29568, 1484,  48408, 23052, 32212,
                2948,  35160, 6956,  64940, 27276, 56080, 48404, 6556,  57244,
                40528, 50284, 24228, 32216, 19128, 40272, 56668, 33180, 21464,
                29736, 45912, 65132, 25028, 42816, 2020,  21880, 14920, 11640,
                34636, 61476, 27500, 12980, 17988, 56620, 24508, 12080, 30600,
                22032, 62512, 5160,  59464, 12252, 20292, 37072, 28752, 8592,
                42440, 35452, 21516, 28424, 12564, 30644, 16428, 41528, 64056,
                31572, 14168, 61588, 19732, 57688, 5120,  8636,  21972, 2144,
                51436, 54848, 42728, 33472, 24748, 48572, 51692, 28784, 17968,
                3880,  57188, 30468, 64864, 1316,  47232, 9676,  6616,  64272,
                36896, 10896, 47824, 20652, 3548,  25136, 8868,  57856, 22232,
                4652,  48668, 13824, 60780, 55076, 33444, 59304, 29456, 3304,
                39464, 26540,
            },
            {
                20720, 1332,  44360, 13500, 26768, 19036, 31836, 61964, 22468,
                17844, 60724, 23904, 15804, 47948, 32100, 18896, 42772, 56156,
                21736, 31372, 39388, 8784,  20776, 35772, 60680, 7956,  43864,
                58652, 24776, 46044, 40720, 4208,  34024, 12680, 20636, 26388,
                15220, 9340,  37636, 58588, 1792,  6388,  23792, 13344, 61228,
                50816, 6980,  20052, 38180, 47448, 60056, 53824, 64252, 45996,
                23632, 5620,  47020, 37936, 49072, 2160,  33248, 59976, 7724,
                51788, 39032, 18664, 50428, 30708, 64460, 47284, 16940, 59340,
                48836, 5320,  64956, 3756,  43212, 54300, 7148,  1208,  22220,
                27228, 38840, 43724, 10040, 35332, 27012, 49948, 44988, 15952,
                31852, 5780,  19040, 444,   61704, 14888, 4636,  44488, 63176,
                33692, 46312, 20956, 27024, 39128, 51008, 31432, 54928, 28260,
                18176, 53720, 30272, 60248, 38160, 45704, 33016, 11676, 42672,
                35164, 17280, 52260, 28512, 7456,  37268, 50016, 42452, 18604,
                57576, 6256,
            },
            {
                36600, 62200, 50696, 39720, 58880, 3908,  56716, 14320, 41032,
                536,   49780, 36292, 58032, 12168, 38500, 50640, 8564,  36884,
                12796, 44832, 24348, 62416, 46644, 14476, 26824, 38864, 49820,
                12040, 16072, 55304, 30860, 18448, 42912, 58908, 2248,  64172,
                54020, 4336,  28352, 16576, 52184, 48084, 62908, 43768, 36680,
                3280,  27268, 58516, 13956, 10032, 18220, 6324,  28988, 39280,
                55908, 19168, 31472, 8020,  43336, 62380, 19924, 40992, 46476,
                26664, 10580, 33752, 43668, 8016,  40628, 25204, 55720, 11352,
                31484, 25792, 51616, 18144, 36808, 60040, 26008, 47276, 50860,
                59064, 17340, 2244,  46612, 64332, 13836, 38516, 62796, 28520,
                36932, 60784, 22784, 41008, 56168, 36256, 20680, 38168, 10240,
                24088, 50208, 8088,  59460, 17224, 24828, 62144, 14036, 45548,
                40780, 2620,  23084, 14996, 51988, 40968, 18968, 49900, 29572,
                62636, 2312,  23604, 43476, 20064, 12732, 4256,  62628, 14956,
                32968, 49596,
            },
            {
                23912, 7516,  31188, 10652, 21944, 47380, 35136, 51820, 6280,
                55652, 26792, 9344,  20400, 2140,  61464, 25384, 4412,  58988,
                27496, 356,   57772, 33952, 3864,  56268, 18052, 64624, 2116,
                28612, 51440, 37304, 9088,  61216, 23508, 50828, 31352, 35988,
                43608, 47652, 60564, 41348, 21044, 35752, 26676, 9468,  54524,
                16544, 40212, 53560, 30536, 35340, 51484, 25748, 33200, 9116,
                16156, 51348, 63292, 24908, 14856, 28552, 53276, 5444,  16312,
                65264, 1352,  54492, 23820, 15400, 4360,  33348, 1156,  19636,
                38496, 54752, 14148, 45328, 23140, 10376, 33020, 19144, 37312,
                8760,  55156, 32880, 24752, 51152, 4176,  18368, 55500, 10608,
                47984, 7800,  50624, 12596, 30076, 8472,  26444, 60300, 55240,
                808,   14440, 35844, 43472, 11592, 2732,  34416, 21788, 4604,
                61248, 50036, 33700, 8580,  63812, 28428, 1320,  55292, 6460,
                38356, 56464, 9584,  63872, 39268, 53496, 26460, 21964, 54724,
                9576,  42972,
            },
            {
                52376, 15396, 27884, 54156, 2012,  42572, 25044, 29312, 38412,
                64248, 32244, 46552, 54456, 29180, 44436, 53548, 15088, 33244,
                65228, 49188, 17032, 11752, 43168, 31096, 51768, 10372, 33820,
                42140, 19708, 63568, 384,   14468, 46932, 38144, 7696,  17112,
                22028, 12032, 32972, 7820,  55256, 14264, 4032,  31660, 19660,
                60188, 8492,  48604, 160,   22712, 41544, 62000, 44872, 1564,
                59492, 41256, 4464,  34916, 50548, 10760, 38288, 23620, 57280,
                29972, 48696, 37224, 60868, 57940, 45868, 50780, 63144, 43388,
                6720,  60668, 34124, 1468,  57096, 39932, 62844, 52592, 12996,
                3996,  28460, 41308, 59660, 21280, 30612, 43004, 968,   24180,
                57392, 44052, 27052, 34796, 65052, 46444, 52556, 16320, 31012,
                39840, 63884, 28164, 56468, 52908, 40488, 49148, 57904, 37404,
                11988, 25580, 56944, 36324, 5224,  13584, 24480, 60524, 15376,
                45416, 26064, 32384, 48132, 5352,  30740, 45832, 35508, 1008,
                28800, 60928,
            },
            {
                18856, 46056, 64664, 37648, 59960, 16316, 12028, 7464,  19200,
                43696, 4476,  13472, 40228, 34232, 6492,  22388, 41400, 47108,
                19520, 37996, 7872,  52920, 40108, 24920, 4812,  45520, 22620,
                60144, 5864,  26020, 44236, 29656, 53716, 11188, 27544, 62480,
                51720, 25724, 3120,  64640, 30164, 44656, 57756, 39636, 46900,
                24728, 33412, 43944, 64164, 56176, 5280,  13064, 49560, 20688,
                36332, 12396, 56804, 21416, 45116, 59576, 3340,  34340, 42904,
                13216, 20628, 5068,  11420, 28368, 21756, 36296, 13480, 27572,
                22820, 46776, 9264,  29172, 49768, 6148,  15636, 29876, 43984,
                65164, 49336, 15428, 6576,  11652, 53748, 60992, 33780, 40368,
                20336, 15616, 3296,  53796, 18288, 2132,  41624, 6136,  49004,
                19688, 45584, 4452,  22860, 6844,  29712, 18628, 8808,  43420,
                31060, 16944, 46280, 20068, 54000, 48100, 43792, 33900, 20896,
                51908, 12412, 18944, 59508, 15144, 56992, 10816, 64852, 48288,
                38552, 12556,
            },
            {
                2592,  40756, 5028,  33044, 19916, 49420, 62724, 57672, 53312,
                23204, 16496, 62356, 24644, 59224, 18076, 63244, 9268,  30560,
                2912,  55192, 22796, 28824, 63636, 20140, 58556, 36268, 55480,
                13120, 31812, 49052, 57784, 21120, 5496,  59868, 41684, 960,
                56548, 39324, 46004, 18792, 50252, 23100, 11476, 63440, 51128,
                2276,  14676, 20728, 10828, 27084, 17016, 37584, 54756, 24424,
                30304, 46760, 27144, 612,   64016, 30752, 18388, 54604, 7268,
                61556, 45448, 32636, 41872, 52212, 17568, 9396,  53924, 58728,
                40736, 16256, 63528, 21056, 42204, 25244, 3068,  55672, 20620,
                34460, 23552, 56712, 39088, 35532, 46124, 26152, 6056,  13164,
                59380, 31248, 61972, 38872, 11432, 22292, 32728, 59348, 25252,
                10992, 52072, 32480, 16244, 60556, 46936, 65432, 27136, 55376,
                6192,  62680, 108,   41628, 59484, 31344, 10232, 37620, 64972,
                7552,  41012, 36204, 304,   42948, 24416, 17540, 41264, 7024,
                25200, 58712,
            },
            {
                35048, 22488, 56500, 9316,  26076, 44736, 35952, 31404, 10200,
                47432, 37196, 50784, 1220,  10928, 48228, 36316, 51680, 12660,
                25848, 44140, 61220, 35520, 1116,  14040, 48664, 7620,  16748,
                40612, 52608, 9772,  35236, 39704, 15960, 33072, 48936, 19932,
                13732, 34800, 58920, 9844,  37160, 1132,  28924, 17336, 7356,
                37856, 61688, 29356, 35824, 46308, 60616, 31780, 2696,  65332,
                9692,  52304, 17268, 39980, 8724,  13920, 48072, 39488, 27528,
                51892, 16896, 25260, 63552, 1888,  39344, 31896, 3344,  48672,
                5232,  30712, 51308, 11644, 36076, 60508, 46596, 38304, 7596,
                11164, 47152, 528,   19060, 62444, 8536,  16608, 48700, 52168,
                36328, 47048, 7004,  24576, 49832, 57864, 44644, 14108, 36724,
                2800,  61852, 42352, 38196, 12152, 33628, 1060,  15028, 21352,
                52132, 38768, 27696, 9232,  22636, 15764, 2220,  57796, 27212,
                4112,  48620, 61924, 28180, 51504, 34200, 3072,  59884, 20304,
                32184, 55284,
            },
            {
                11108, 27532, 47908, 14236, 52052, 188,   6160,  40684, 27284,
                2944,  58248, 29616, 42380, 55844, 27180, 3840,  57064, 39756,
                59140, 6140,  14992, 50500, 10160, 43740, 29832, 26356, 63364,
                1464,  24600, 18224, 2892,  62660, 55340, 23252, 4508,  45048,
                28516, 6732,  24336, 15604, 43016, 59928, 54060, 34568, 26812,
                56892, 41772, 52824, 3128,  50328, 8032,  41016, 19136, 43440,
                4696,  61232, 32868, 55532, 43280, 25584, 58356, 22008, 10340,
                1080,  35796, 56652, 8364,  47848, 55268, 61652, 26128, 35016,
                18988, 37844, 54620, 6944,  57844, 18164, 32200, 62268, 27316,
                41764, 59984, 31848, 52484, 28152, 42532, 22164, 64932, 2296,
                27388, 10244, 42240, 56024, 28744, 35012, 8132,  64408, 27716,
                55564, 21608, 8712,  58040, 25696, 54176, 39872, 45456, 35092,
                59148, 12632, 32944, 51132, 63236, 39632, 49384, 18460, 53784,
                31040, 23036, 55456, 9100,  20952, 46608, 37804, 52864, 14588,
                49876, 44472,
            },
            {
                17748, 63628, 42376, 60620, 30144, 23384, 55188, 15056, 65476,
                21628, 33852, 7740,  19508, 13844, 32136, 44912, 16160, 21392,
                34112, 47776, 18532, 41000, 32440, 60284, 54500, 38436, 46140,
                33620, 60700, 42324, 47092, 27080, 11840, 36832, 65468, 31392,
                54704, 61780, 48208, 52404, 32160, 5004,  40964, 21212, 45684,
                12584, 5600,  18468, 59048, 22428, 13820, 57408, 25972, 47804,
                15404, 38344, 22480, 5820,  50108, 36264, 2740,  33360, 64656,
                48792, 40876, 13704, 22760, 29808, 19972, 15244, 45088, 12148,
                65420, 308,   25620, 44940, 13992, 23572, 1300,  50244, 16784,
                54268, 6432,  25372, 14176, 3416,  58232, 37984, 30328, 55728,
                18012, 21320, 63784, 4120,  15332, 19592, 0,     48204, 39572,
                17152, 50444, 29384, 3608,  18992, 49528, 23608, 10256, 4520,
                48384, 24676, 18152, 45104, 6756,  29652, 25092, 42416, 12088,
                45860, 16476, 33560, 13680, 57792, 4560,  26424, 62372, 8772,
                29636, 5388,
            },
            {
                53712, 1432,  36092, 6920,  38556, 58408, 18596, 42996, 49048,
                11912, 52856, 45432, 61764, 39184, 23152, 64576, 7984,  29768,
                62496, 2052,  27972, 52792, 23264, 3172,  19380, 5108,  11492,
                20756, 53764, 14132, 30440, 50348, 7592,  43536, 51916, 17552,
                9444,  38328, 1948,  19624, 26420, 62436, 14544, 8900,  65196,
                49512, 32228, 25376, 44420, 38960, 34028, 29540, 51700, 10948,
                34528, 58564, 28556, 11984, 19284, 60720, 16056, 44708, 55620,
                19008, 31220, 58972, 3836,  36928, 42616, 6332,  56840, 23292,
                49836, 42508, 59828, 33732, 28764, 53072, 8848,  43464, 12824,
                22076, 37128, 64160, 45452, 50040, 33300, 11312, 4856,  44720,
                39988, 32632, 52816, 46160, 37748, 60356, 53956, 23584, 31936,
                12452, 46072, 34436, 43232, 64092, 5468,  60180, 31732, 42484,
                63772, 1972,  54532, 35764, 56808, 3860,  60856, 34464, 8572,
                63376, 1460,  40052, 44068, 64424, 31632, 12024, 42588, 21912,
                36632, 59308,
            },
            {
                24700, 32272, 20948, 16440, 10592, 46288, 32500, 4000,  35572,
                59988, 16680, 26000, 5180,  54936, 35692, 496,   50380, 53760,
                11184, 37612, 56516, 7324,  63908, 36788, 15572, 58244, 49332,
                28576, 8068,  37684, 3920,  58444, 18952, 25368, 2612,  13180,
                41452, 22516, 34316, 57036, 11636, 46264, 50684, 24500, 36628,
                3412,  55596, 15296, 9140,  63940, 548,   5744,  62300, 21032,
                54528, 1268,  45412, 64420, 31104, 53036, 7808,  29020, 12328,
                5244,  26336, 51540, 45872, 64048, 10984, 51096, 28216, 38956,
                8944,  14948, 20588, 3484,  48016, 39904, 64748, 35584, 29660,
                58976, 2224,  40400, 9300,  20464, 15992, 51752, 25156, 60744,
                14300, 992,   24220, 12508, 30692, 41384, 9920,  43460, 4980,
                62828, 1788,  56528, 14324, 37412, 28212, 16888, 51556, 20080,
                29296, 15432, 40692, 10656, 20512, 14252, 51380, 21940, 55988,
                37384, 50404, 25616, 19112, 6880,  47924, 16900, 56456, 1984,
                46908, 39876,
            },
            {
                11760, 51428, 48476, 62220, 53048, 27620, 63108, 8100,  24848,
                39540, 1356,  30348, 50024, 10508, 46556, 17736, 40168, 25084,
                43048, 20656, 46168, 12916, 26148, 45024, 51536, 41196, 23884,
                65008, 43672, 55584, 22180, 63540, 40232, 34120, 56824, 60684,
                27540, 47352, 64348, 6100,  29984, 38648, 716,   58664, 29488,
                19252, 40680, 60468, 27804, 53064, 47016, 16976, 42764, 39576,
                7912,  49064, 24384, 41556, 3932,  37912, 47392, 24036, 42164,
                37220, 61140, 8600,  16524, 24532, 34292, 60640, 2580,  33116,
                62008, 30428, 56384, 36952, 11132, 55368, 18464, 4920,  46248,
                51620, 17832, 33992, 28716, 56444, 41928, 63044, 7684,  47832,
                34632, 59160, 49664, 6112,  62980, 26456, 16528, 51260, 59532,
                20132, 25984, 52400, 22720, 8120,  46704, 12064, 55336, 38092,
                8752,  57708, 46860, 31560, 65192, 37892, 44552, 2356,  17480,
                28436, 5460,  61404, 53364, 29364, 38484, 52096, 33964, 28172,
                64788, 15308,
            },
            {
                3988,  44232, 26328, 4828,  40632, 21896, 13508, 50872, 55656,
                19236, 43848, 64120, 34092, 21540, 60812, 28648, 13368, 59848,
                4148,  31436, 16908, 58820, 34016, 9932,  30284, 6696,  34756,
                1124,  17056, 32376, 11068, 45848, 14932, 29220, 49032, 20672,
                4760,  53692, 14872, 43940, 18396, 55352, 33684, 16000, 43660,
                7540,  47732, 33320, 11568, 20696, 35616, 24640, 56380, 31864,
                27916, 17628, 13772, 51744, 10120, 20992, 57024, 63084, 84,
                53672, 21196, 32556, 39816, 1360,  47052, 17732, 21668, 53240,
                5176,  45616, 17156, 62940, 6400,  22308, 25756, 32772, 56684,
                10364, 24572, 61596, 47396, 5888,  23408, 35932, 29584, 17548,
                54052, 9700,  42068, 36584, 20356, 55636, 7060,  29500, 35240,
                38588, 10676, 40912, 32496, 62024, 57428, 34576, 2940,  25172,
                61300, 5328,  22940, 27396, 48884, 7748,  25996, 32592, 59648,
                48004, 13352, 35220, 9748,  21484, 2764,  60908, 23696, 7604,
                19304, 54988,
            },
            {
                29924, 60400, 9400,  34136, 57564, 856,   37504, 45504, 31260,
                11328, 57448, 8320,  15276, 42396, 3008,  56688, 32984, 47552,
                9132,  65308, 50028, 39748, 232,   21548, 62112, 56588, 13332,
                46892, 59788, 26084, 36220, 6628,  52916, 1388,  10096, 44648,
                36068, 31712, 8496,  40420, 60344, 21928, 10408, 52960, 61876,
                22896, 51284, 1620,  38676, 58084, 7292,  50264, 12072, 2368,
                59096, 63112, 36052, 32356, 60184, 26932, 14256, 33712, 17576,
                48340, 44176, 13932, 57972, 54892, 29940, 41952, 58684, 13652,
                40568, 24724, 52148, 27904, 41800, 50612, 60696, 14976, 39336,
                3668,  42888, 13648, 53404, 224,   58856, 12588, 39352, 4128,
                22420, 27712, 15408, 57068, 1568,  33576, 48532, 13408, 45268,
                57076, 6272,  48424, 16100, 404,   43744, 21900, 40476, 49708,
                33012, 43924, 52716, 484,   16824, 58236, 53088, 39236, 10416,
                42760, 23380, 56248, 40960, 59008, 14232, 43444, 10628, 49220,
                41448, 34948,
            },
            {
                22264, 38700, 14444, 19416, 43636, 29508, 16868, 59684, 2824,
                22668, 47756, 27352, 38048, 51356, 24824, 6988,  19612, 36940,
                55708, 22996, 28444, 5844,  53968, 47632, 17368, 27504, 39384,
                20428, 52256, 3116,  49600, 61012, 23808, 38524, 63224, 16780,
                58764, 23120, 51216, 25908, 3984,  48904, 27604, 2960,  37828,
                12928, 26296, 63416, 17824, 29888, 45724, 65084, 15648, 37668,
                44024, 20332, 5184,  55332, 2120,  44800, 40192, 5904,  29880,
                10500, 65176, 27608, 6084,  49844, 12204, 8276,  26924, 48684,
                36156, 10020, 2064,  33840, 13132, 44764, 768,   28624, 63264,
                21480, 49376, 27408, 37548, 32172, 19172, 46656, 55504, 64856,
                44364, 31692, 62296, 46296, 25016, 39924, 65388, 22148, 2332,
                17512, 63592, 27576, 53880, 30464, 18972, 6872,  64584, 14704,
                11160, 18416, 36284, 62580, 42540, 13096, 4056,  19560, 64136,
                30624, 16280, 792,   46176, 31888, 50064, 26912, 36772, 62700,
                324,   57952,
            },
            {
                7028,  52356, 63876, 47220, 55384, 7364,  64876, 25932, 35344,
                53596, 40764, 3680,  58844, 12112, 63692, 44716, 52732, 15456,
                2228,  42280, 35480, 14060, 60864, 43184, 37372, 4052,  54720,
                8532,  41464, 30820, 15796, 43024, 19664, 54872, 30668, 41548,
                12224, 284,   62920, 37036, 13556, 45132, 64036, 42044, 31368,
                56552, 44928, 5580,  54360, 42156, 3888,  26720, 22536, 53432,
                9276,  47120, 25000, 39060, 49236, 18776, 61700, 50704, 56216,
                35628, 3788,  23852, 37944, 18712, 34660, 64396, 3112,  55428,
                19372, 63492, 46524, 21000, 57868, 37704, 8464,  47628, 35440,
                53972, 6948,  17004, 63852, 9096,  43412, 25852, 2584,  50588,
                19740, 7120,  51932, 3928,  18080, 11200, 54364, 28188, 51580,
                33080, 24008, 42840, 12924, 60212, 50656, 26400, 46384, 54808,
                28712, 58460, 21676, 9776,  33544, 29580, 23072, 46516, 54772,
                4848,  51280, 62164, 25364, 18408, 5804,  55568, 16464, 31048,
                13028, 46224,
            },
            {
                17692, 27852, 1916,  31928, 23992, 10548, 39224, 49348, 5952,
                15856, 63128, 20436, 31984, 17532, 35248, 29776, 40556, 61744,
                26776, 11252, 52172, 19344, 31720, 24780, 11016, 64440, 33108,
                24024, 62688, 11900, 57848, 27700, 8228,  34788, 3632,  25408,
                56172, 47056, 17524, 29436, 58080, 34624, 6292,  16420, 19672,
                8552,  35876, 21784, 14620, 32808, 10520, 40456, 60888, 34188,
                30340, 57544, 12180, 16240, 29596, 8256,  23228, 12620, 41720,
                20492, 60276, 52180, 43524, 59168, 22448, 45280, 38824, 15348,
                30960, 59124, 6816,  53024, 30364, 16132, 59864, 19568, 11172,
                31524, 57712, 40024, 22848, 51172, 60708, 15352, 33660, 11404,
                37740, 41052, 13452, 35400, 60984, 42924, 8040,  36640, 5272,
                61152, 38188, 9196,  4524,  35492, 39204, 9732,  32088, 1648,
                37796, 4420,  48312, 40040, 56508, 50180, 61692, 35660, 8604,
                27580, 37968, 33792, 9120,  39664, 65364, 3184,  44508, 20868,
                53248, 25684,
            },
            {
                42356, 37124, 50312, 12428, 41284, 52868, 20240, 32404, 12768,
                44448, 29280, 7788,  55464, 49456, 1376,  9756,  21844, 5936,
                48852, 58740, 45208, 8172,  56764, 1988,  51316, 14964, 48560,
                44660, 17804, 37616, 5052,  47292, 64616, 13416, 51444, 45236,
                7056,  33120, 39692, 9048,  20452, 54592, 24052, 51940, 61188,
                49212, 29244, 58348, 46624, 62104, 52124, 19220, 48824, 3212,
                6828,  41472, 51028, 64832, 34452, 58288, 37476, 26524, 1472,
                46996, 14860, 29372, 10092, 932,   31448, 51776, 5736,  23388,
                43372, 34992, 11916, 26320, 40700, 3136,  54624, 25304, 41724,
                1848,  45512, 12372, 4608,  29744, 36216, 6528,  56696, 28500,
                60452, 23400, 54648, 26992, 30848, 20972, 46784, 59016, 14104,
                44664, 20200, 55520, 47604, 22752, 57520, 17448, 61120, 23304,
                44268, 63652, 15892, 6912,  24932, 2908,  17648, 41576, 14200,
                45096, 20724, 58640, 12488, 52380, 22776, 34860, 59936, 38968,
                9616,  61940,
            },
            {
                33856, 4824,  56272, 17088, 62280, 3004,  45736, 56164, 61600,
                24288, 51148, 36728, 42928, 23348, 60116, 46096, 55056, 33808,
                17248, 29400, 23660, 63088, 40784, 27856, 35916, 21956, 6996,
                29404, 976,   56220, 32804, 21764, 39028, 18020, 60240, 28580,
                21108, 61444, 52612, 48372, 2056,  41184, 11204, 32988, 884,
                39116, 11820, 4288,  25140, 36924, 208,   28900, 13268, 58948,
                17956, 22084, 27616, 660,   45648, 4548,  52284, 63448, 32368,
                55160, 7508,  40384, 62984, 16644, 55852, 12640, 61504, 28376,
                50844, 168,   18056, 48528, 61164, 22544, 33396, 49788, 65252,
                17656, 28280, 59384, 54572, 47100, 20204, 41760, 49104, 17352,
                1012,  47368, 9168,  57540, 49400, 1956,  16556, 25784, 31400,
                50152, 900,   29920, 64976, 15248, 2444,  52800, 41192, 12776,
                51236, 26728, 31044, 54120, 45564, 11680, 58888, 26200, 1276,
                56540, 49308, 6104,  43176, 29444, 47640, 7432,  27564, 50232,
                2364,  57272,
            },
            {
                48280, 8872,  22020, 29380, 35736, 26432, 5368,  37436, 18124,
                1056,  58296, 11716, 4740,  15072, 38760, 26228, 12692, 63956,
                39448, 1224,  36508, 4852,  16580, 45980, 59676, 55036, 39968,
                60976, 49892, 25504, 53040, 43968, 1880,  48880, 9420,  36212,
                15048, 4132,  24980, 13280, 65372, 27900, 46300, 59600, 26492,
                44220, 64688, 17652, 50484, 8204,  56876, 23168, 38856, 46156,
                35864, 62244, 53976, 10568, 39568, 20692, 15748, 9624,  43928,
                17884, 24540, 34076, 48580, 26572, 37364, 19880, 46788, 9176,
                39996, 55744, 63992, 37164, 9704,  43248, 13776, 4792,  36804,
                52216, 23768, 34492, 14768, 1536,  64528, 10584, 24584, 62840,
                43832, 33276, 15724, 5724,  37464, 64304, 52804, 40208, 10016,
                57200, 17196, 41536, 26308, 34128, 44740, 29032, 5824,  35776,
                19076, 8664,  60064, 35176, 20188, 38964, 32684, 52524, 64552,
                30968, 17416, 24608, 62832, 15128, 55136, 18924, 12000, 32548,
                15984, 23412,
            },
            {
                13380, 63532, 43652, 58020, 47712, 14428, 60016, 30260, 9548,
                40336, 27164, 34020, 47064, 65088, 28960, 52412, 3956,  20580,
                43284, 56116, 13788, 50396, 32512, 8832,  18996, 3512,  34004,
                10532, 20500, 14016, 6500,  59312, 30208, 24760, 54164, 42340,
                57256, 38432, 44164, 31564, 55752, 16768, 37720, 7208,  14344,
                22344, 54104, 41200, 31820, 14924, 43512, 64364, 53016, 9112,
                31876, 43020, 14460, 24768, 32644, 56264, 47744, 30148, 59912,
                36720, 51544, 2760,  57812, 6208,  42364, 32908, 3648,  57956,
                25468, 14080, 21276, 28748, 3892,  46420, 57340, 30004, 7372,
                61752, 9472,  43980, 39048, 26896, 32460, 51976, 36960, 4864,
                29976, 58704, 19872, 41868, 23068, 12436, 33508, 4084,  62424,
                23440, 36088, 11836, 53792, 8072,  49204, 20976, 64076, 57896,
                47844, 732,   42596, 13848, 62752, 6416,  48268, 21532, 9996,
                36544, 4284,  40480, 32640, 516,   36636, 58864, 41964, 64280,
                45368, 30120,
            },
            {
                51956, 19336, 38384, 1672,  10324, 51108, 20700, 42412, 54644,
                48504, 62252, 21184, 56048, 8392,  18480, 36148, 49668, 9288,
                30996, 60592, 46916, 21800, 53752, 64224, 42824, 13008, 26644,
                46364, 65336, 41664, 35556, 16728, 11500, 63808, 32536, 2564,
                18708, 62972, 7640,  22100, 35260, 4744,  51336, 19856, 56856,
                34880, 2736,  27460, 60672, 20172, 34628, 5124,  26164, 16640,
                1996,  56744, 5292,  48864, 63028, 42092, 6612,  22912, 3592,
                11848, 64196, 20792, 44984, 11304, 23236, 53892, 65348, 17588,
                33960, 45620, 7264,  53272, 32288, 62720, 19984, 39604, 15876,
                47740, 21192, 3388,  60820, 18216, 57396, 7632,  21596, 55016,
                12848, 38948, 52308, 61372, 45632, 28916, 55784, 19512, 43616,
                6928,  47360, 61620, 3104,  59480, 14180, 38528, 10636, 25324,
                32824, 56076, 22656, 29200, 50880, 2156,  27780, 41136, 14916,
                44500, 60548, 53556, 11024, 46200, 26368, 4504,  21424, 53008,
                6128,  39936,
            },
            {
                60160, 27480, 6892,  32492, 24492, 65480, 34440, 7164,  23192,
                13348, 3240,  16328, 30764, 41220, 20,    57752, 44796, 25088,
                15928, 7100,  27096, 11284, 38372, 24580, 30332, 58280, 51920,
                37208, 7612,  57008, 27236, 50332, 40236, 5548,  22472, 47560,
                12312, 26608, 50080, 59260, 10716, 42256, 61676, 47132, 30408,
                10168, 49436, 6032,  46256, 55764, 10952, 48536, 30532, 61532,
                50468, 20280, 28704, 36416, 18400, 12480, 26832, 57588, 49936,
                39808, 28048, 14532, 31168, 61956, 50552, 14968, 29148, 38088,
                1960,  50364, 41976, 58736, 17000, 24960, 10812, 55760, 27524,
                41932, 31332, 53452, 49208, 12176, 45708, 41208, 15996, 48144,
                26808, 3700,  10496, 25632, 8,     7992,  48420, 37188, 27100,
                52040, 32564, 19064, 39744, 24528, 31828, 55096, 43628, 3468,
                15360, 39552, 52640, 10348, 36612, 17068, 61332, 54832, 7092,
                25180, 50388, 18764, 22400, 56820, 39424, 49588, 10272, 34768,
                25380, 928,
            },
            {
                35948, 46304, 16172, 54728, 40036, 44504, 16752, 58680, 52572,
                32600, 37680, 43776, 53392, 24260, 61028, 13968, 32292, 62888,
                54388, 41376, 34132, 57420, 740,   6384,  49196, 16028, 2392,
                23240, 18096, 31252, 24,    61980, 20036, 45612, 56740, 34872,
                52512, 30756, 480,   45568, 18128, 28600, 2416,  24836, 39956,
                63164, 37032, 13248, 23964, 38652, 1280,  41804, 21820, 12856,
                45004, 39728, 60404, 8992,  3032,  52420, 38200, 31516, 19212,
                43332, 7712,  56284, 35840, 500,   39512, 7600,  47492, 10432,
                60428, 22872, 12468, 36272, 2448,  49496, 34896, 51596, 692,
                58400, 13420, 35936, 6592,  25652, 29664, 588,   59768, 34532,
                44496, 65232, 32120, 49860, 35680, 17580, 63500, 13604, 58580,
                1304,  15364, 28876, 45316, 51628, 4908,  17672, 28156, 62572,
                46772, 18620, 5332,  65272, 43772, 24084, 46276, 33876, 58468,
                29852, 1860,  35152, 64980, 7500,  16084, 59900, 29260, 62476,
                18132, 55320,
            },
            {
                11792, 22828, 62340, 49044, 12536, 5040,  28568, 896,   45892,
                26136, 63260, 5156,  10472, 47964, 34640, 6512,  39080, 21568,
                2028,  50992, 18308, 61772, 45020, 19876, 40452, 62288, 33028,
                55452, 42496, 47912, 9956,  53668, 37772, 13496, 28392, 8384,
                60948, 40416, 15188, 37056, 54252, 33308, 57484, 8504,  15504,
                53140, 18820, 58840, 29440, 63796, 17296, 54612, 59180, 37136,
                6204,  24276, 53640, 33060, 58260, 45160, 65200, 1072,  15840,
                61480, 52848, 23908, 47856, 17676, 59564, 26032, 20848, 43144,
                52424, 31084, 26520, 64512, 40512, 8140,  44540, 22660, 5716,
                64732, 24844, 19640, 62092, 55648, 38240, 63836, 23680, 8716,
                18364, 56148, 21772, 15080, 60492, 41228, 30832, 22788, 10528,
                42408, 62780, 57000, 9364,  21824, 60960, 34832, 50488, 8476,
                36876, 59292, 26040, 31792, 56644, 12864, 3404,  19864, 10444,
                39020, 47312, 13600, 27952, 43820, 31912, 2704,  47484, 13816,
                44228, 7920,
            },
            {
                50544, 41956, 2304,  30300, 57236, 19796, 61204, 39196, 11576,
                18728, 57080, 50504, 20780, 28016, 55176, 19004, 11480, 46616,
                29360, 5148,  37020, 13712, 31304, 53220, 27556, 9628,  45344,
                5400,  63460, 14376, 25004, 33124, 4628,  59444, 17276, 43092,
                4192,  20616, 64144, 23736, 47940, 12920, 62604, 21740, 44996,
                600,   32620, 43104, 7940,  51004, 34084, 26732, 4108,  32252,
                64080, 11260, 16452, 42456, 21604, 13984, 24688, 34924, 46744,
                5044,  33004, 10452, 41776, 5668,  30256, 55080, 34956, 63104,
                4344,  18392, 6344,  54912, 14828, 28588, 61644, 18024, 38544,
                33188, 46244, 9780,  43324, 4264,  33540, 14196, 49432, 52892,
                2844,  41948, 37288, 5152,  53872, 46588, 2628,  55128, 50052,
                38716, 25256, 5480,  36408, 13172, 41820, 260,   57492, 12552,
                20788, 42472, 1344,  49100, 8080,  40120, 28700, 49896, 64020,
                16836, 55920, 5432,  51732, 37400, 24224, 54308, 20000, 38000,
                57204, 31824,
            },
            {
                59268, 20232, 34508, 9184,  25964, 36224, 51600, 48236, 30556,
                8288,  35540, 14380, 39960, 2596,  64744, 43036, 51432, 59192,
                23652, 64244, 48804, 25536, 7948,  59516, 36276, 22112, 12728,
                38540, 29060, 21152, 60324, 44040, 49448, 23564, 36564, 54900,
                25780, 50940, 10048, 6288,  27364, 3616,  41204, 35896, 50156,
                27152, 56704, 22324, 3444,  15016, 45660, 9508,  52452, 19044,
                47412, 27416, 332,   49696, 29600, 7396,  51576, 10884, 55472,
                38832, 21636, 27200, 64560, 51480, 44704, 12296, 1512,  15916,
                39372, 57184, 44860, 48040, 20348, 31892, 57600, 12156, 49012,
                54840, 15752, 40064, 51228, 22460, 11020, 19156, 39792, 31120,
                27516, 13308, 59412, 29428, 9004,  19644, 26780, 34976, 7316,
                16980, 33136, 54396, 49536, 63688, 26888, 47616, 23504, 32568,
                54416, 29552, 14812, 35128, 21840, 59732, 53456, 36848, 23536,
                43148, 32300, 61312, 19332, 11864, 63288, 9044,  42144, 23280,
                4820,  27908,
            },
            {
                39600, 3804,  53280, 63988, 43912, 14244, 22360, 3740,  64540,
                42792, 24000, 60568, 45528, 32060, 16304, 25960, 3732,  35644,
                15004, 40704, 10388, 56584, 42884, 2668,  15476, 48012, 57740,
                54036, 1652,  51364, 40540, 7916,  30036, 2616,  65040, 11736,
                32012, 44728, 58432, 38896, 56012, 52064, 19292, 30984, 4996,
                64984, 11532, 48184, 40440, 62224, 24820, 38204, 57768, 14156,
                43612, 35588, 55132, 63132, 37088, 40696, 61740, 19736, 28908,
                60268, 1940,  57664, 14272, 19776, 37424, 23744, 58228, 49108,
                27468, 33584, 11208, 37444, 1188,  52820, 42080, 4228,  27256,
                7880,  30452, 2412,  59116, 28112, 54096, 45516, 57992, 6532,
                63216, 44196, 50740, 24020, 40084, 12188, 59036, 45016, 65024,
                21684, 46820, 1804,  19268, 31116, 15756, 7408,  37992, 44396,
                64124, 6516,  51804, 61668, 17420, 44240, 5872,  14412, 156,
                8608,  26208, 40620, 3488,  45188, 57824, 33988, 464,   52460,
                64728, 15588,
            },
            {
                25040, 11056, 47496, 17864, 38100, 7248,  55868, 33672, 16208,
                54580, 1544,  29612, 7284,  58628, 37728, 9516,  56792, 31480,
                6940,  45668, 20920, 33344, 17764, 50852, 65076, 6668,  30936,
                25768, 16584, 36152, 11488, 18932, 58008, 15388, 47148, 39860,
                1632,  18252, 34264, 29316, 15968, 43828, 9484,  60320, 14316,
                38772, 17960, 34784, 52944, 31068, 20424, 1416,  49520, 29516,
                7836,  60604, 20644, 10360, 4624,  47692, 26560, 5928,  43732,
                16588, 48320, 40584, 34216, 3776,  61496, 9244,  31924, 42052,
                7864,  21632, 62516, 25268, 60308, 9880,  23216, 37068, 63472,
                20936, 60736, 47332, 17256, 37132, 62792, 1840,  24568, 35428,
                10176, 17948, 780,   34068, 63916, 52156, 31512, 4088,  14076,
                56844, 29188, 11724, 44264, 39972, 55788, 59764, 18156, 3256,
                11292, 24564, 45976, 38632, 9916,  26616, 33176, 62452, 46332,
                54780, 59624, 21952, 50000, 29788, 17052, 26016, 48796, 12748,
                35868, 46052,
            },
            {
                62232, 31624, 56556, 28420, 184,   60156, 45608, 25676, 40588,
                20152, 52000, 48760, 12460, 22024, 52664, 47752, 19780, 61356,
                27244, 54508, 436,   62068, 28208, 39400, 24104, 34912, 41888,
                61820, 4500,  46728, 63824, 55476, 34428, 52036, 27796, 20888,
                60652, 49288, 13184, 63620, 888,   22640, 57360, 26072, 47160,
                54368, 24392, 8844,  5392,  59836, 12132, 41920, 65412, 23244,
                2836,  40276, 25492, 32176, 18172, 57180, 14456, 54224, 36604,
                31616, 11936, 7588,  50516, 29096, 53516, 46328, 18892, 65020,
                54680, 14008, 50632, 5616,  43384, 34476, 16360, 51684, 45112,
                13220, 35100, 25272, 5408,  31568, 8568,  42220, 15344, 48988,
                54536, 22304, 47136, 56488, 5988,  16668, 43032, 24460, 36464,
                40772, 50964, 60528, 24996, 4124,  10104, 52280, 28672, 48548,
                40728, 19540, 57916, 31156, 1772,  55948, 50604, 18648, 29996,
                35564, 15632, 10912, 36492, 53532, 6764,  39436, 60612, 30448,
                19108, 6220,
            },
            {
                50692, 21640, 14528, 42188, 52520, 32000, 12688, 9740,  61880,
                5636,  36496, 26752, 62772, 34168, 920,   42192, 13288, 38916,
                49964, 16448, 36560, 52236, 5024,  12020, 46232, 19176, 9764,
                49572, 20516, 31976, 27168, 6440,  23180, 42600, 9612,  5488,
                54292, 23828, 7528,  46452, 36756, 50668, 33220, 6836,  42184,
                2236,  29812, 57700, 43544, 27648, 35496, 54668, 16852, 33036,
                47876, 53376, 15356, 59440, 45308, 34272, 952,   23096, 49888,
                62404, 25080, 56256, 22248, 43052, 15624, 26476, 4916,  38280,
                2864,  30412, 45688, 18120, 28040, 55912, 3328,  30912, 58144,
                120,   42852, 56056, 49232, 13488, 51980, 20760, 26948, 59592,
                38732, 31756, 14368, 26552, 37952, 20996, 49216, 9980,  62124,
                2900,  8516,  20908, 36972, 64628, 32840, 22564, 35512, 62804,
                55488, 27108, 4784,  13672, 64716, 41364, 23856, 11904, 39868,
                4436,  48104, 27476, 1412,  62648, 14004, 21572, 43756, 9960,
                54380, 40776,
            },
            {
                34236, 8216,  61212, 4600,  23688, 19328, 50384, 28128, 56532,
                46840, 14936, 8856,  43548, 16880, 55008, 30096, 23288, 3432,
                63868, 11100, 43492, 22320, 30684, 55756, 60916, 1296,  52988,
                14100, 58632, 39276, 44184, 616,   13948, 37416, 63156, 31212,
                35856, 41508, 26584, 59080, 17828, 40532, 12316, 64152, 20296,
                37104, 61892, 49712, 15152, 19120, 46464, 4552,  9680,  59044,
                37344, 12492, 42768, 3772,  50916, 29064, 64904, 41212, 9536,
                3048,  45928, 17800, 64052, 124,   36096, 57136, 12664, 51856,
                23920, 35324, 58756, 39672, 11944, 64772, 48520, 25904, 19148,
                38672, 10640, 22616, 65456, 39676, 58524, 34308, 45940, 5856,
                3144,  62412, 42836, 8412,  61072, 1452,  58480, 28032, 53368,
                18232, 32400, 55360, 14604, 49908, 46284, 6648,  13520, 1084,
                16048, 33920, 50068, 36872, 21204, 47460, 3260,  59240, 53256,
                20288, 65524, 57252, 41416, 45908, 33340, 55400, 3168,  27760,
                58904, 1288,
            },
            {
                44956, 26880, 36516, 46952, 39660, 64268, 34944, 1688,  38764,
                22976, 32672, 57996, 39588, 5196,  25208, 59640, 8280,  45184,
                32116, 25412, 57232, 8460,  48136, 14940, 33676, 43680, 29272,
                37184, 24196, 7764,  54124, 48300, 59556, 19424, 45792, 57148,
                15848, 3960,  52244, 10144, 31076, 2788,  28272, 53252, 16176,
                45664, 10988, 22704, 240,   63056, 38912, 51488, 26856, 20112,
                5908,  63496, 27924, 21556, 8344,  38600, 12304, 20180, 59100,
                35432, 28100, 39168, 31240, 10464, 59940, 40808, 20440, 47464,
                60884, 16020, 676,   53688, 21988, 37368, 8372,  14612, 62596,
                51156, 7012,  33380, 18204, 28572, 1264,  10832, 63708, 29840,
                19652, 12236, 49492, 35524, 29588, 45896, 39580, 15196, 34520,
                47224, 26192, 43712, 488,   28248, 18864, 42172, 59152, 38836,
                45676, 8272,  60860, 53628, 28344, 16444, 34304, 8884,  25796,
                44072, 6840,  31092, 23308, 8932,  18428, 61576, 50204, 37144,
                23700, 17372,
            },
            {
                12520, 53148, 57968, 10872, 16184, 7572,  58724, 42564, 18028,
                4568,  64996, 53388, 19140, 45804, 63300, 35960, 49408, 17924,
                58852, 34708, 1800,  41496, 64388, 26692, 6484,  21688, 56432,
                3656,  64704, 12076, 17816, 30380, 25544, 51020, 2816,  11584,
                28596, 61816, 44332, 21360, 55000, 61032, 48712, 23824, 35180,
                5112,  32192, 40956, 56160, 33512, 11728, 23656, 56564, 44480,
                30800, 49640, 35108, 55740, 61276, 24588, 53660, 44380, 16308,
                51652, 4544,  13236, 54596, 48556, 5876,  32848, 28736, 6976,
                42200, 27044, 9632,  49784, 31388, 5064,  46748, 33836, 41552,
                29528, 54736, 46584, 4016,  43796, 24128, 54980, 17136, 41520,
                52144, 57496, 23372, 53936, 18084, 11412, 56092, 4756,  22260,
                64156, 7040,  58656, 38452, 11232, 56708, 25456, 51400, 29948,
                20484, 24140, 42684, 10980, 5812,  45000, 62848, 50288, 38152,
                32820, 17476, 12568, 52428, 38052, 25692, 5084,  30988, 15084,
                48276, 65172,
            },
            {
                32340, 20228, 2204,  29732, 55172, 26188, 47900, 14084, 30404,
                50128, 11360, 28292, 2476,  31380, 10024, 14820, 40100, 5920,
                52780, 13432, 20536, 37564, 18160, 51352, 40444, 60012, 16388,
                45992, 50224, 34988, 41588, 61168, 9084,  33796, 39444, 23404,
                50148, 18564, 38404, 34220, 14780, 5648,  43204, 9540,  60044,
                55228, 50768, 26248, 7648,  47976, 29676, 3292,  61636, 40092,
                17712, 544,   10624, 15868, 47004, 2040,  30744, 6652,  33216,
                57264, 21620, 61712, 43760, 18608, 23548, 50652, 63092, 11316,
                56060, 33476, 63928, 44224, 19316, 61240, 57324, 22960, 1720,
                12572, 25396, 61920, 15264, 57444, 35920, 48776, 6852,  37812,
                26088, 33504, 2260,  6468,  65248, 25144, 31672, 44060, 51948,
                12432, 41224, 17160, 50808, 62872, 34752, 4296,  9192,  61580,
                54216, 2284,  32344, 18344, 40080, 55112, 30172, 14580, 684,
                61448, 55632, 49020, 2904,  63588, 46792, 42028, 11504, 56828,
                6972,  42636,
            },
            {
                25064, 61616, 49192, 41232, 33688, 21296, 3556,  52720, 60852,
                44556, 24720, 38016, 48108, 21716, 51388, 56472, 27092, 22416,
                29556, 46964, 61624, 54792, 3964,  11608, 35656, 25124, 10252,
                32456, 27272, 20904, 2104,  55976, 15516, 5752,  65344, 42696,
                56484, 180,   7936,  63948, 46816, 25948, 37692, 18516, 1120,
                29084, 13612, 19532, 64884, 16676, 42916, 36336, 14276, 8452,
                53808, 58316, 25532, 41752, 37016, 18868, 49264, 63816, 25968,
                39940, 47440, 8264,  26364, 35064, 1712,  14712, 44392, 37560,
                17872, 22676, 3944,  13396, 41100, 28564, 16252, 52140, 45312,
                59184, 20784, 37556, 8248,  51264, 20208, 31312, 14228, 60512,
                9480,  44668, 16088, 41128, 47376, 37200, 8660,  59652, 2608,
                35984, 30084, 24124, 3284,  31440, 21848, 47420, 17624, 36172,
                12672, 48888, 58320, 64112, 25404, 3596,  21588, 58044, 10488,
                24116, 40868, 35528, 21064, 28536, 16376, 59892, 34656, 20804,
                39044, 3692,
            },
            {
                9716,  37652, 17876, 6228,  44748, 63284, 37152, 9416,  20220,
                34604, 6536,  57504, 13628, 61884, 33824, 316,   44148, 65304,
                3248,  42464, 9312,  27784, 31964, 45524, 62896, 44,    42084,
                54660, 6916,  62156, 38448, 44784, 27936, 53200, 20496, 31840,
                13696, 48196, 30292, 22900, 11712, 57452, 31968, 63124, 42060,
                47248, 58328, 38184, 4324,  52080, 58028, 20544, 49136, 33784,
                22792, 45652, 64712, 32316, 5508,  59000, 13876, 42304, 10404,
                2516,  15580, 30548, 38248, 65080, 54116, 58340, 25720, 6428,
                52032, 47036, 35684, 54776, 24948, 2832,  38236, 10480, 35320,
                5912,  49976, 40856, 30060, 11816, 64516, 42552, 4488,  46888,
                21368, 63348, 27744, 58552, 20808, 50112, 14144, 19504, 27340,
                47980, 61508, 9672,  44388, 53740, 14892, 40468, 65360, 43844,
                26216, 6700,  37976, 46620, 13004, 36040, 49488, 42812, 28056,
                46528, 15696, 5608,  57512, 10260, 53992, 244,   50936, 27488,
                62776, 52024,
            },
            {
                45560, 56020, 13872, 60084, 23724, 12136, 29080, 56756, 41028,
                820,   63644, 17236, 42776, 6992,  38812, 18340, 12052, 54436,
                36380, 16432, 50136, 23188, 57548, 14688, 20772, 48784, 59264,
                18940, 14032, 52384, 23456, 11452, 35304, 59068, 46496, 9872,
                36392, 26284, 59636, 40812, 53412, 4184,  15524, 51824, 21760,
                11336, 24632, 34180, 45292, 10100, 27348, 1400,  63256, 28788,
                6224,  37948, 12868, 51304, 22004, 54864, 28604, 36236, 23268,
                56104, 60900, 51272, 4676,  11228, 40188, 19824, 31464, 2684,
                62492, 29620, 8980,  59904, 50800, 7692,  55888, 64592, 26612,
                17752, 32420, 3264,  59708, 26916, 784,   53428, 24404, 56128,
                35636, 50760, 12448, 30616, 1160,  33664, 62344, 42244, 53192,
                15520, 56576, 20156, 37256, 59860, 5796,  28372, 1172,  33024,
                56428, 15936, 29688, 20664, 8648,  61180, 31764, 18040, 7388,
                53060, 64392, 30504, 44900, 40116, 32164, 24016, 7980,  43644,
                15944, 30280,
            },
            {
                1004,  34372, 28244, 50416, 1736,  53980, 46436, 15948, 25888,
                31528, 50896, 22764, 53932, 29980, 24984, 60088, 48640, 31080,
                7192,  60416, 33640, 40000, 5696,  52488, 37884, 8548,  28092,
                34344, 46836, 4300,  30824, 49676, 17696, 1612,  25024, 4900,
                62988, 17076, 50592, 2544,  19516, 44932, 28000, 35460, 6508,
                39900, 2080,  60572, 30392, 22512, 39520, 55600, 11368, 42004,
                52540, 18432, 3736,  27404, 9448,  44620, 3180,  62552, 49828,
                19788, 33032, 44088, 17492, 29228, 45968, 8484,  49416, 42016,
                57424, 15008, 39612, 18584, 32856, 44548, 30140, 20376, 42276,
                54048, 62272, 44856, 16340, 47820, 34496, 39376, 17312, 29560,
                2532,  8004,  39852, 53772, 6000,  56884, 24328, 7368,  38924,
                700,   33252, 6588,  49428, 12104, 24512, 58112, 48448, 10064,
                22680, 50576, 4144,  52872, 43736, 55824, 1404,  39136, 60048,
                34088, 2376,  22560, 13720, 61044, 19224, 37320, 48148, 12772,
                59316, 22592,
            },
            {
                18752, 64324, 8328,  42548, 31856, 38592, 19288, 7392,  60296,
                47452, 12444, 36252, 9556,  46292, 57088, 4556,  21332, 41172,
                25856, 13176, 19648, 1492,  64204, 26268, 31028, 56772, 2648,
                39536, 64448, 42932, 57560, 8124,  63516, 41844, 51900, 38904,
                55276, 43704, 7312,  34600, 38684, 64848, 9052,  48396, 61548,
                56416, 17760, 53372, 14552, 62592, 5676,  46108, 24976, 16164,
                60132, 31408, 48156, 62072, 40072, 34524, 17768, 7320,  45984,
                12852, 1088,  24856, 59204, 52480, 22732, 61944, 34000, 11964,
                21700, 26796, 46196, 1180,  23220, 61584, 14448, 48596, 376,
                13148, 8936,  23916, 56008, 21208, 9428,  58716, 13260, 62612,
                43156, 19124, 59908, 15540, 37744, 45876, 11088, 29856, 63924,
                22568, 46088, 26688, 63412, 41704, 34972, 52416, 19548, 38492,
                59716, 41568, 62028, 34048, 26972, 15252, 23600, 51164, 12116,
                26448, 38004, 48772, 8424,  51448, 2812,  64928, 56632, 4104,
                35992, 53616,
            },
            {
                47716, 39460, 25992, 15620, 57196, 4564,  65484, 35096, 43096,
                2840,  56040, 27332, 3724,  41388, 14716, 35552, 9912,  62876,
                45740, 51688, 58520, 48212, 43344, 17640, 12600, 44428, 22280,
                16216, 10892, 25312, 20660, 37148, 14296, 31520, 19896, 12376,
                28984, 22276, 15020, 56748, 29992, 23284, 54828, 13448, 26428,
                31772, 43904, 8304,  36360, 48940, 18552, 31948, 35808, 54372,
                2536,  43684, 20520, 57116, 14748, 52196, 59728, 30836, 26156,
                38012, 55148, 41740, 6884,  13880, 36668, 360,   16708, 55512,
                37776, 5104,  65416, 49956, 11660, 36164, 4396,  39016, 58124,
                34184, 29116, 38112, 6092,  41436, 52112, 31548, 5456,  49548,
                24952, 33040, 47308, 21968, 26892, 52060, 18284, 35208, 43836,
                13020, 54188, 17616, 30816, 3140,  15736, 44296, 7636,  13724,
                31628, 1868,  17808, 11424, 40300, 64960, 5560,  44572, 20116,
                56240, 16764, 58912, 25076, 34280, 42708, 16672, 26436, 31468,
                41492, 10792,
            },
            {
                5644,  58000, 51964, 21460, 11448, 48656, 23512, 52272, 13988,
                20816, 39052, 61160, 18580, 64672, 50600, 27872, 53356, 16736,
                2436,  37580, 28312, 10732, 35072, 6888,  62212, 50248, 54248,
                60068, 29348, 48944, 672,   53952, 26788, 47584, 61308, 2688,
                46104, 33172, 62292, 10932, 49036, 1176,  17188, 41468, 3196,
                20248, 49912, 28440, 3872,  42308, 59424, 12880, 65152, 8028,
                38952, 10692, 26100, 33332, 648,   24248, 11344, 41084, 50676,
                64792, 9304,  34836, 48296, 63788, 28116, 58104, 25328, 44128,
                52368, 9864,  28668, 40900, 58584, 17168, 53568, 26004, 21836,
                45500, 52600, 63896, 14624, 26412, 60740, 18316, 45472, 37780,
                10544, 55088, 64472, 4632,  42576, 61380, 2952,  57980, 49564,
                4220,  59992, 39452, 9952,  56372, 22272, 29916, 64168, 53916,
                26736, 45064, 23956, 57628, 7560,  47672, 35472, 30780, 63068,
                9172,  41076, 45400, 5344,  55108, 29004, 9588,  46424, 19900,
                62324, 24596,
            },
        },
        {
            {
                42820, 9368,  47668, 15768, 41972, 49228, 57496, 44192, 38468,
                23988, 45484, 34280, 6136,  56328, 20000, 59768, 34880, 41392,
                62008, 31624, 5560,  55820, 33408, 10264, 58400, 20364, 26988,
                32916, 38488, 44504, 51728, 27732, 36608, 40440, 61728, 30324,
                52148, 34816, 18060, 43948, 12320, 23512, 33956, 20356, 2624,
                45564, 28708, 49192, 37496, 31816, 22408, 58364, 48308, 55848,
                35828, 14776, 53364, 37344, 24080, 45264, 40264, 27396, 37744,
                23456, 2788,  30548, 25840, 46476, 13220, 1788,  62252, 25308,
                57020, 39440, 32148, 54564, 9920,  30156, 6820,  50244, 10500,
                45664, 31328, 57668, 24980, 484,   59800, 41404, 20476, 31032,
                36156, 27080, 49560, 5100,  35268, 47152, 26864, 50836, 64188,
                17904, 35156, 53008, 3048,  26664, 7388,  21964, 31660, 144,
                23756, 16564, 58948, 2144,  64744, 7132,  14148, 35672, 4408,
                44408, 48024, 7376,  42212, 21240, 46412, 30956, 11984, 41116,
                22376, 50884,
            },
            {
                30100, 57192, 4136,  62660, 11676, 34728, 20812, 14836, 10080,
                62624, 16616, 41072, 28868, 64616, 15676, 4996,  44968, 27428,
                2444,  53192, 47468, 14764, 22552, 52128, 39376, 15412, 3636,
                48532, 9064,  59076, 17352, 5944,  22848, 1924,  45364, 24208,
                38008, 10016, 54660, 7136,  39188, 61316, 51072, 42620, 59128,
                10132, 62488, 15496, 3284,  53804, 43632, 7644,  27400, 18880,
                62428, 41288, 8260,  48608, 17696, 31224, 54712, 15656, 61352,
                55908, 44288, 19360, 52476, 39216, 59648, 17176, 41948, 4352,
                19280, 12340, 44260, 4844,  23960, 46604, 19796, 40028, 63404,
                26660, 14336, 61908, 36672, 12372, 22684, 34332, 47912, 58456,
                45168, 1200,  14832, 65116, 53428, 58944, 284,   41272, 21804,
                5924,  59764, 45384, 32932, 65336, 47188, 41132, 11420, 58012,
                44792, 5328,  37688, 25868, 46596, 39104, 24340, 29700, 62972,
                20492, 26236, 32740, 61820, 14704, 60128, 24744, 55668, 8328,
                63104, 3268,
            },
            {
                19180, 53864, 22100, 28200, 52648, 1544,  61608, 51208, 29852,
                47848, 1756,  21908, 10816, 49260, 24576, 32680, 51640, 22092,
                8652,  36936, 19848, 43336, 62400, 7432,  30188, 45656, 55980,
                63956, 21212, 13368, 54768, 42420, 62936, 11584, 57576, 16580,
                4000,  59864, 21028, 26512, 48836, 1216,  15636, 38176, 54760,
                31240, 21692, 41488, 26072, 63972, 35048, 13684, 45432, 33368,
                1892,  29792, 21096, 63968, 2440,  60096, 9756,  20176, 5236,
                48160, 11488, 64700, 33660, 7428,  24104, 55084, 30924, 37260,
                52500, 28432, 62048, 58172, 37968, 13792, 59376, 32604, 1488,
                54916, 42036, 7516,  51428, 28496, 63948, 9132,  3560,  17596,
                10996, 56056, 42096, 30032, 24056, 11156, 32596, 15832, 54904,
                48296, 24572, 11556, 19732, 51124, 15040, 35700, 53956, 18848,
                48668, 33188, 54388, 11888, 51152, 17364, 60456, 2492,  41064,
                12476, 55728, 108,   39404, 9764,  50180, 2216,  44020, 27464,
                15368, 46228,
            },
            {
                12464, 40132, 36448, 6064,  44680, 31516, 39800, 26384, 7548,
                55440, 59124, 36760, 52640, 42800, 832,   39012, 11812, 63768,
                17208, 59328, 29172, 12064, 35388, 25680, 60060, 152,   23984,
                36508, 29540, 4312,  35248, 30920, 49220, 19808, 33516, 51012,
                28624, 41076, 47044, 32780, 63648, 19220, 29088, 7760,  24536,
                4180,  46708, 56940, 6484,  18200, 49688, 60108, 4036,  24020,
                50224, 57868, 43976, 33800, 13860, 26564, 51472, 42384, 33184,
                28632, 39992, 16484, 228,   50372, 29160, 10356, 48516, 65244,
                8672,  35072, 16304, 1164,  49272, 26100, 52884, 5368,  23196,
                17104, 33872, 47320, 4040,  18260, 44436, 53340, 40144, 62728,
                25636, 20980, 33980, 6960,  18304, 43292, 62928, 36380, 4376,
                29724, 43048, 38684, 8584,  29272, 59244, 5712,  25696, 38816,
                8564,  14716, 63020, 20948, 31892, 43600, 9916,  53572, 49592,
                37116, 58972, 18352, 52120, 28004, 36808, 17380, 58096, 38852,
                34792, 60404,
            },
            {
                32384, 49796, 64412, 17124, 24160, 59488, 4648,  18020, 43132,
                12456, 31800, 25736, 17712, 8292,  60508, 29980, 56112, 46452,
                40188, 50088, 1660,  55336, 4748,  47876, 16668, 41944, 10928,
                52384, 40216, 47612, 60632, 2656,  26348, 55868, 7236,  43672,
                64460, 14036, 5540,  56904, 11216, 36716, 45400, 60260, 13916,
                50084, 34272, 11200, 38460, 29760, 21472, 10300, 39452, 55072,
                15984, 11580, 6060,  52516, 38660, 46868, 35936, 3124,  63224,
                22224, 8732,  57540, 36468, 60792, 21008, 44216, 15100, 2756,
                22936, 47124, 41592, 20872, 31160, 8264,  43452, 61344, 37008,
                49540, 9608,  21416, 38732, 55828, 32512, 24116, 13708, 30348,
                51088, 38164, 46760, 58196, 52544, 3476,  45892, 26060, 57160,
                13496, 62440, 2544,  56716, 23184, 43596, 1568,  63576, 30240,
                60032, 43012, 28316, 6548,  972,   57792, 34364, 21644, 15704,
                31172, 5868,  24120, 44904, 63584, 4532,  32068, 48584, 6744,
                23300, 628,
            },
            {
                26028, 10216, 7152,  57872, 46988, 13892, 54912, 37696, 64828,
                20692, 3320,  46728, 62380, 33812, 53656, 20204, 15140, 3400,
                26052, 33600, 23628, 65328, 38740, 51344, 31716, 19712, 62084,
                6696,  14820, 24848, 18552, 9440,  46352, 14924, 39212, 22396,
                800,   35920, 24972, 16480, 52364, 3116,  56064, 32496, 40536,
                65164, 17156, 52600, 59028, 320,   42332, 53396, 31336, 65024,
                37080, 28252, 23148, 61612, 18484, 7372,  23720, 59068, 13384,
                49224, 53320, 25584, 46036, 12796, 5092,  40476, 33988, 57072,
                60012, 27104, 11184, 54092, 64384, 34784, 18660, 11596, 27516,
                57120, 64800, 29672, 59508, 15444, 1376,  48780, 36232, 5908,
                59964, 2916,  9708,  27796, 13788, 38916, 21308, 7568,  50548,
                19068, 32408, 52896, 16488, 48556, 34072, 13132, 20420, 51496,
                3520,  23364, 50368, 39868, 61900, 25316, 47988, 7740,  27624,
                65396, 47000, 10940, 34324, 21448, 54648, 13052, 19912, 65016,
                51424, 56048,
            },
            {
                44428, 20084, 29716, 41732, 35076, 10580, 22712, 48284, 33720,
                51528, 40420, 56580, 13928, 5332,  44724, 27036, 36360, 7224,
                57672, 10184, 45232, 14556, 21228, 8432,  58076, 34788, 44416,
                28504, 56760, 64796, 41768, 52788, 36684, 59956, 31300, 10676,
                54212, 58696, 44728, 30420, 41672, 22568, 26756, 9804,  20280,
                1804,  27940, 23464, 44784, 15004, 61476, 26580, 7412,  19696,
                47692, 904,   42836, 56500, 30520, 10984, 55132, 44072, 17564,
                30752, 3628,  42112, 18440, 32440, 63804, 53580, 25508, 18640,
                7024,  50920, 37552, 5124,  56824, 15164, 48064, 40740, 3608,
                44552, 13628, 5644,  42824, 26104, 62144, 8420,  57048, 18976,
                42468, 15800, 64008, 49692, 34972, 61716, 30636, 59344, 10160,
                40992, 37100, 26288, 6232,  60624, 40368, 55296, 46568, 36176,
                16604, 11560, 56180, 35384, 18184, 13604, 55056, 38728, 42544,
                4200,  54056, 15052, 41524, 8132,  59228, 43192, 26544, 9656,
                41320, 14840,
            },
            {
                60892, 48480, 54240, 4064,  27192, 63252, 20,    29240, 5828,
                15988, 9364,  28652, 37540, 23312, 50376, 10652, 63344, 48624,
                41884, 18616, 31188, 54124, 42844, 27060, 12660, 3664,  48472,
                22340, 2108,  33268, 12308, 29048, 20588, 5436,  62700, 49604,
                27980, 19332, 8872,  62240, 49124, 6160,  63312, 47672, 53568,
                43348, 37548, 7612,  32360, 48756, 36180, 4464,  45860, 12744,
                59660, 32820, 49536, 4664,  40788, 63876, 27476, 712,   39024,
                35220, 61852, 6708,  55884, 27956, 49660, 1896,  38276, 45460,
                31828, 14176, 43704, 28976, 24128, 2100,  52188, 30144, 20284,
                53444, 23948, 35824, 50876, 11148, 40496, 21760, 45648, 28340,
                54140, 32896, 24708, 20240, 1156,  54612, 16320, 47316, 23400,
                688,   64768, 45304, 20984, 30832, 10060, 27536, 7352,  42204,
                58352, 32544, 47364, 9380,  44340, 30552, 3192,  60660, 19236,
                32488, 25980, 61404, 51468, 29188, 1172,  47736, 35884, 30692,
                5168,  37460,
            },
            {
                2588,  33300, 16204, 39200, 50596, 19416, 43936, 53776, 60716,
                25272, 44272, 63924, 1464,  18284, 59308, 39828, 30276, 22120,
                61532, 52412, 5036,  59792, 1232,  37488, 61272, 54640, 15780,
                38456, 50864, 8256,  45008, 58900, 1308,  24624, 41476, 17092,
                45760, 38112, 2728,  33936, 13620, 39224, 17524, 35136, 4764,
                59688, 13240, 62524, 55544, 10796, 22492, 17708, 52212, 40228,
                24504, 9272,  17204, 21660, 13552, 34000, 47824, 52924, 20928,
                57996, 11444, 23772, 37380, 15716, 9088,  22772, 11800, 55076,
                61340, 428,   63380, 19296, 46144, 39184, 60328, 7876,  62724,
                33324, 308,   46468, 19200, 30652, 52688, 33964, 65300, 2032,
                10620, 48184, 40344, 7264,  44248, 11820, 26880, 42460, 34236,
                55472, 15212, 11640, 52108, 2028,  62356, 17952, 23856, 65072,
                492,   26520, 21232, 4696,  64296, 50020, 23660, 10448, 44992,
                56544, 1940,  36776, 18220, 39656, 24212, 62372, 15508, 52380,
                57840, 22336,
            },
            {
                12512, 62996, 24644, 9240,  57464, 14388, 31976, 37272, 11828,
                58044, 21392, 48404, 31176, 42408, 55204, 4476,  16136, 312,
                34460, 13252, 28208, 35868, 17448, 46124, 24480, 30600, 9528,
                62452, 26448, 55536, 17636, 37768, 48496, 32900, 53636, 12572,
                4700,  24028, 51104, 59272, 21492, 54716, 29324, 11672, 23792,
                30296, 49764, 19932, 3356,  27580, 63784, 57152, 29180, 34560,
                62828, 53916, 37908, 58852, 51176, 24680, 5636,  15940, 8552,
                46296, 28772, 51724, 43448, 62444, 47272, 58432, 29332, 17316,
                41172, 22296, 35564, 9732,  58396, 33052, 12868, 26120, 38004,
                16844, 54972, 58420, 7780,  60928, 4712,  16612, 12940, 37404,
                22736, 57836, 29296, 60760, 36740, 51136, 4340,  63556, 6532,
                48800, 28044, 58084, 35768, 39424, 47684, 33492, 53824, 13964,
                39016, 52428, 59384, 40972, 27960, 37628, 16888, 34660, 50832,
                13516, 22760, 48644, 5472,  12132, 33464, 56480, 3164,  18788,
                46160, 28244,
            },
            {
                43412, 36928, 55484, 1800,  45472, 61700, 6900,  41832, 17812,
                2308,  34672, 54380, 7624,  12756, 34076, 25140, 47288, 56668,
                39168, 8116,  23132, 48396, 63916, 7324,  51928, 40664, 19240,
                43432, 35352, 21672, 6192,  63464, 14472, 57808, 8088,  35640,
                65272, 56248, 28596, 10128, 43728, 1652,  61088, 45412, 57380,
                40076, 16680, 34052, 46676, 41808, 37676, 1408,  44368, 6200,
                15024, 3304,  28132, 43908, 1820,  36420, 61356, 42344, 32340,
                65392, 14536, 1532,  20124, 5348,  40860, 35464, 3848,  51964,
                8160,  26848, 49468, 54552, 16540, 5904,  21848, 44304, 49432,
                10008, 40892, 14232, 27780, 38152, 48304, 55672, 27140, 44492,
                51680, 3668,  14364, 17872, 55912, 31992, 21032, 39284, 18032,
                31076, 8240,  43160, 24740, 16016, 4536,  57340, 44660, 5888,
                30484, 18896, 8188,  14848, 54048, 1304,  57552, 62576, 7144,
                40148, 59524, 31044, 64624, 53384, 45320, 22016, 10360, 38588,
                64132, 8232,
            },
            {
                17640, 49616, 30008, 20764, 35212, 26612, 23024, 52848, 28456,
                50400, 38464, 14688, 27008, 60600, 51484, 9244,  65012, 19348,
                44468, 51132, 58716, 41412, 11068, 21420, 33664, 4632,  57848,
                464,   13356, 47484, 31096, 40792, 27128, 22048, 44268, 30332,
                19960, 47284, 40888, 16328, 32852, 25976, 50448, 36752, 8816,
                652,   53088, 25352, 6620,  54204, 16004, 12188, 50192, 20820,
                57944, 46484, 31708, 64640, 9856,  18256, 27820, 54352, 22260,
                40092, 49812, 34396, 60132, 30740, 25016, 13484, 64912, 32200,
                47040, 59804, 3444,  30700, 42336, 51296, 65236, 56116, 4400,
                30952, 24800, 64252, 43836, 20832, 1420,  23916, 40004, 60212,
                6628,  35088, 63844, 46196, 9284,  24288, 58732, 13012, 52348,
                61412, 22164, 3388,  60356, 50668, 29152, 10536, 21512, 36804,
                50204, 45980, 63152, 34132, 22012, 12288, 46312, 25392, 29504,
                19896, 9644,  43080, 16236, 27772, 7560,  41720, 50296, 25236,
                31472, 53360,
            },
            {
                23532, 5792,  41432, 59748, 12256, 48672, 3132,  65524, 9908,
                46032, 4904,  63032, 18964, 45604, 21724, 37856, 28880, 11668,
                32012, 25708, 15920, 3348,  29884, 56804, 14476, 48944, 65120,
                28856, 53520, 60280, 3204,  50720, 10788, 4256,  52116, 61236,
                204,   13756, 7276,  53516, 63980, 19408, 6476,  15208, 21844,
                64680, 43200, 59548, 10156, 29924, 58380, 24012, 32464, 41340,
                26540, 11296, 19420, 53092, 39668, 48244, 57200, 12124, 7028,
                3460,  25568, 56620, 10952, 52876, 18052, 55640, 44064, 20624,
                38872, 15836, 12368, 37232, 24888, 952,   28636, 35440, 19644,
                59332, 2376,  34916, 50640, 11868, 33236, 63136, 8840,  18460,
                31064, 21364, 42548, 27392, 1844,  40868, 47220, 2920,  35892,
                44768, 54676, 33076, 12516, 19764, 63304, 42476, 25388, 61244,
                11664, 2796,  28884, 42700, 49112, 6616,  32024, 42120, 3024,
                55272, 35656, 47492, 700,   57568, 19444, 61464, 34188, 13740,
                59284, 660,
            },
            {
                34796, 61932, 10164, 51804, 17328, 39760, 56652, 33748, 20292,
                58932, 24412, 40684, 32112, 1132,  5812,  42652, 53912, 2868,
                60000, 6284,  54500, 34584, 61588, 44108, 27212, 36724, 23528,
                10440, 39132, 16440, 24788, 45080, 56084, 37036, 15884, 39484,
                25056, 34472, 59736, 37684, 3396,  47968, 41536, 56184, 27948,
                32036, 12860, 38388, 21260, 35360, 47840, 65404, 8620,  52672,
                61292, 35612, 7004,  23768, 15204, 4484,  31156, 44672, 60864,
                37224, 16856, 43944, 7660,  39000, 48056, 1036,  9960,  25988,
                5988,  56224, 45228, 63072, 18728, 47928, 11292, 15092, 41756,
                46824, 52708, 17892, 6184,  57696, 29500, 53072, 46812, 13984,
                56648, 50584, 11736, 53940, 33448, 15888, 62980, 29376, 25460,
                10260, 16636, 48032, 41312, 36592, 7772,  53096, 1300,  33036,
                56980, 16220, 24192, 38384, 56168, 61628, 18196, 52636, 64056,
                23156, 13976, 51812, 24792, 37924, 30204, 3776,  55360, 6396,
                40604, 47812,
            },
            {
                56920, 14948, 27300, 4468,  44632, 30788, 7656,  15240, 43008,
                29788, 11944, 55384, 49776, 58408, 35144, 62716, 14888, 46328,
                36616, 21100, 40072, 47516, 19040, 1648,  52916, 6816,  17788,
                45728, 32300, 8112,  35132, 64204, 18652, 28768, 58808, 9100,
                43616, 50552, 27484, 21788, 30528, 12536, 62028, 34340, 52096,
                46840, 3724,  49588, 61648, 1968,  18908, 5340,  39228, 16780,
                192,   47656, 56636, 42968, 62300, 25864, 35504, 19676, 51916,
                29856, 48416, 63952, 32868, 28248, 22696, 62680, 36500, 50236,
                61492, 33280, 22800, 8596,  58512, 39804, 53344, 60596, 7404,
                27004, 10656, 23180, 39652, 45356, 15692, 4488,  41692, 36696,
                68,    25064, 38436, 61496, 7364,  49456, 54996, 20072, 5652,
                38836, 65452, 448,   25928, 59020, 31028, 14660, 46784, 39912,
                20412, 51624, 60116, 3840,  9884,  27064, 36472, 15308, 39072,
                4688,  60468, 33112, 8440,  63092, 12764, 48928, 44720, 16932,
                28628, 20156,
            },
            {
                45524, 38480, 33144, 64124, 55032, 22320, 37156, 61204, 51656,
                2564,  36092, 7480,  16356, 28196, 9876,  24532, 18400, 50316,
                27676, 64388, 13424, 9152,  25080, 38788, 11880, 55916, 41920,
                61844, 51016, 57012, 21312, 12648, 1868,  33056, 48920, 22844,
                63732, 3860,  17624, 46180, 57968, 39044, 23632, 9432,  5200,
                18820, 24728, 15588, 52904, 42424, 26324, 45456, 56100, 29348,
                22068, 38248, 13044, 32752, 49732, 1348,  58284, 8604,  14048,
                24004, 316,   20436, 13196, 4264,  58792, 14996, 41932, 29104,
                17648, 2236,  51720, 26976, 34308, 6276,  31392, 20724, 36908,
                63900, 32140, 55208, 62328, 36036, 25632, 19956, 58608, 28152,
                64772, 47520, 4796,  19048, 22944, 36304, 11060, 42032, 58028,
                50564, 30300, 21676, 55264, 4888,  49576, 18116, 64280, 27344,
                8768,  43776, 31496, 13980, 20956, 44920, 84,    49828, 11232,
                45480, 27524, 42280, 17776, 54152, 39932, 22504, 26272, 36452,
                64964, 11284,
            },
            {
                54168, 2256,  24656, 18892, 48212, 432,   12784, 26436, 47420,
                18344, 64028, 44168, 22184, 46904, 39888, 55800, 33216, 7820,
                41588, 576,   51852, 58136, 32432, 63720, 48552, 30632, 20580,
                5140,  1236,  27912, 43240, 39876, 53924, 46096, 5788,  13408,
                31268, 54176, 11568, 7632,  52724, 876,   17084, 44280, 59264,
                36532, 63592, 28520, 7832,  31952, 13160, 60840, 34464, 9676,
                51000, 64216, 27352, 6440,  20828, 53616, 38392, 46764, 62548,
                41800, 53528, 59680, 39920, 46552, 51520, 34084, 5520,  54356,
                11520, 39380, 43264, 14156, 55560, 2700,  44808, 24024, 49052,
                4944,  43480, 13900, 1540,  9032,  60708, 50008, 11304, 34216,
                8100,  16584, 31748, 43472, 59444, 27636, 1064,  32676, 45556,
                13652, 8296,  35292, 44624, 11408, 38440, 23380, 3468,  54728,
                35772, 5576,  48004, 64684, 34020, 55172, 59712, 24524, 30968,
                57112, 6724,  21252, 46740, 1684,  31964, 9144,  60700, 3252,
                50220, 7440,
            },
            {
                30044, 41104, 59280, 8908,  42612, 29032, 57768, 39152, 5212,
                23600, 53536, 31380, 13412, 60748, 51960, 2204,  12496, 61284,
                22776, 30468, 17420, 44988, 4824,  22388, 15996, 58840, 35392,
                25448, 15080, 48232, 59976, 7212,  26160, 62568, 20572, 41388,
                56892, 36132, 40164, 60952, 33776, 29132, 49960, 26304, 54692,
                11904, 40388, 45752, 57728, 37984, 54028, 2940,  24512, 14812,
                43904, 2548,  17936, 59188, 41688, 10848, 16664, 28136, 33816,
                5860,  11040, 35580, 26172, 56996, 8600,  19724, 24420, 45608,
                60208, 20908, 30280, 64736, 46944, 17968, 62044, 12604, 56700,
                16872, 51292, 28744, 21836, 47464, 31476, 3128,  43812, 20988,
                54588, 40276, 57228, 51808, 14020, 46548, 64288, 17716, 23820,
                60528, 53716, 18900, 62288, 28604, 56752, 33552, 41896, 13292,
                59208, 18512, 25600, 52856, 40600, 28788, 8300,  18884, 35032,
                51288, 15680, 65364, 36152, 58688, 50644, 15400, 56684, 41528,
                32768, 21428,
            },
            {
                16132, 51092, 13096, 35864, 52816, 16376, 63068, 9336,  33652,
                41100, 59600, 924,   38228, 6332,  20088, 25996, 43516, 57376,
                48580, 35444, 55140, 26572, 37152, 42932, 2800,  9024,  40304,
                52444, 64588, 37136, 31296, 10680, 17164, 35600, 51140, 28256,
                1120,  19032, 25444, 44928, 20904, 14468, 62808, 42088, 3296,
                20464, 33248, 548,   10632, 17632, 21620, 49088, 40708, 58300,
                31556, 55160, 46520, 35368, 29816, 63860, 22884, 2324,  55412,
                18464, 49940, 30460, 2472,  16548, 44276, 31316, 63812, 1316,
                35604, 48776, 4416,  9836,  37292, 25780, 40984, 30448, 1052,
                39160, 34664, 59624, 41348, 54504, 17592, 37512, 52792, 62232,
                23732, 12088, 28968, 3392,  9176,  39180, 55816, 4868,  37132,
                27172, 2312,  40424, 15104, 7052,  47416, 60772, 9636,  50576,
                30000, 38580, 1380,  11508, 16752, 5252,  48616, 42912, 61504,
                2208,  40352, 29424, 11956, 5640,  27068, 19836, 45764, 12440,
                25112, 62148,
            },
            {
                4396,  46592, 23020, 61620, 6280,  32044, 20360, 50156, 55732,
                15548, 10624, 27168, 50436, 34608, 65108, 29896, 37392, 3564,
                15688, 10316, 6164,  60852, 12768, 50212, 62480, 28924, 46220,
                12960, 6016,  22796, 19276, 55996, 44392, 2732,  60256, 10104,
                48132, 14980, 64380, 5408,  9844,  56104, 35312, 6852,  30556,
                47896, 60744, 23488, 50692, 64392, 30672, 6968,  62572, 19736,
                37176, 7812,  12356, 24380, 3372,  48784, 43372, 59856, 39076,
                24804, 64996, 42796, 21696, 61944, 38180, 10884, 55948, 14436,
                26412, 58276, 16916, 53640, 22284, 7104,  52248, 58920, 9948,
                26396, 19512, 4108,  11644, 65516, 27764, 14424, 5912,  26808,
                1312,  48676, 60996, 35376, 25488, 20892, 30532, 52684, 11756,
                48512, 43220, 32224, 51688, 25348, 20756, 912,   16776, 44764,
                22204, 61848, 55656, 46828, 23640, 36836, 63348, 13832, 22332,
                9524,  53308, 47688, 24136, 43824, 62764, 35108, 52872, 232,
                37424, 57744,
            },
            {
                9964,  34804, 27708, 1436,  39980, 44236, 25192, 3260,  45372,
                30236, 21524, 42884, 57232, 17016, 9112,  46284, 54044, 19124,
                63600, 39708, 47132, 20416, 33728, 56152, 18296, 23596, 54136,
                33360, 42432, 57640, 3656,  51996, 39320, 14296, 24296, 32944,
                58404, 42588, 52076, 31728, 49000, 39584, 23056, 12984, 52404,
                16116, 55972, 27720, 5404,  42012, 36436, 47164, 11684, 4084,
                26740, 53060, 63084, 39400, 51792, 8516,  15096, 32208, 12168,
                46268, 8152,  14568, 54076, 6392,  50136, 27816, 40872, 52028,
                7872,  32576, 42560, 61136, 28688, 47884, 15184, 33212, 43920,
                63444, 53724, 46664, 24604, 7460,  39768, 58292, 33316, 45912,
                38236, 15756, 42788, 18428, 50824, 61976, 42464, 16060, 7408,
                63328, 22348, 56460, 4052,  64840, 36732, 31100, 53176, 26792,
                6028,  34496, 8548,  41480, 59324, 32444, 3780,  54276, 38688,
                26328, 33256, 18348, 56320, 2828,  38996, 29784, 7748,  17784,
                49076, 43260,
            },
            {
                30996, 64784, 55900, 17628, 48596, 57204, 11264, 37172, 64628,
                6884,  49136, 62324, 4392,  24636, 40564, 13868, 5524,  32888,
                28312, 24672, 53264, 1296,  29836, 7464,  39172, 11376, 188,
                61696, 16348, 26876, 47524, 33868, 28828, 65212, 46628, 6936,
                37820, 22076, 3464,  26832, 59304, 1448,  46152, 65284, 37916,
                8556,  43512, 35208, 13856, 54744, 25264, 16036, 29312, 50292,
                43060, 22044, 15512, 31168, 19616, 57124, 36052, 52756, 4948,
                29000, 58244, 36820, 47844, 23808, 34544, 3436,  17480, 46444,
                23272, 38648, 12944, 828,   35188, 64112, 3948,  20924, 49344,
                6640,  14380, 36640, 32172, 50428, 44560, 19092, 10608, 51404,
                64272, 7716,  32164, 58088, 6272,  2096,  46864, 33788, 59428,
                29656, 18048, 9696,  39240, 13108, 46116, 58204, 39772, 63564,
                14768, 49844, 19628, 27664, 13004, 50784, 20252, 29880, 46048,
                58448, 6348,  60908, 14552, 49820, 10568, 22948, 55444, 63432,
                26616, 20140,
            },
            {
                52156, 14608, 38432, 8040,  21592, 61032, 14260, 28336, 52668,
                18228, 36068, 12760, 31444, 55148, 48376, 59556, 23036, 50860,
                44608, 58748, 14072, 42104, 65448, 45784, 25972, 58660, 49332,
                30892, 7988,  38080, 63036, 12084, 8416,  21144, 53492, 17260,
                29280, 55212, 12532, 36276, 16340, 19460, 28836, 57140, 25768,
                18416, 2888,  62220, 20244, 45504, 1396,  59920, 56484, 32840,
                61552, 488,   48268, 59392, 4424,  44648, 25900, 21296, 61444,
                40904, 19916, 1876,  31908, 12916, 56796, 59824, 20960, 65204,
                4908,  55572, 50272, 19020, 45248, 10792, 37668, 54968, 23748,
                29692, 42376, 56780, 496,   21624, 61884, 2996,  55688, 23228,
                30020, 20616, 54792, 11176, 37024, 22972, 13332, 26164, 38596,
                216,   45528, 34500, 50600, 27640, 7500,  19072, 10904, 2692,
                42756, 57460, 31264, 65096, 1672,  43504, 56784, 10476, 17080,
                532,   43252, 36600, 21628, 31492, 41716, 59596, 14964, 33232,
                40244, 6076,
            },
            {
                2708,  45688, 24344, 54100, 31252, 34640, 1928,  41844, 58216,
                23932, 2488,  44980, 38768, 19756, 120,   34284, 62692, 11612,
                2316,  8428,  36124, 22208, 16432, 52492, 4540,  36308, 19088,
                44024, 22000, 51036, 2224,  25020, 57992, 43328, 4724,  40436,
                60644, 8932,  45376, 63588, 41668, 53252, 10604, 4988,  33768,
                40800, 48844, 58436, 32020, 10340, 39812, 22444, 7904,  18208,
                38200, 10004, 34688, 41124, 28564, 11412, 64012, 788,   49532,
                16436, 55128, 27260, 63288, 40512, 9168,  30004, 43912, 36248,
                10352, 27096, 60104, 30820, 24360, 57560, 17236, 40400, 1824,
                62460, 18488, 10524, 59884, 15916, 29524, 35056, 41428, 13692,
                4412,  48168, 44848, 27340, 40976, 65188, 48948, 55380, 19928,
                51908, 57532, 15232, 61008, 23068, 54680, 33716, 48888, 25088,
                36596, 4188,  46140, 17484, 37852, 7768,  25428, 34904, 64184,
                51880, 28160, 8400,  63632, 54412, 5784,  47392, 1900,  44640,
                11972, 60476,
            },
            {
                35200, 28852, 10740, 42860, 4708,  51148, 46680, 16528, 8956,
                33016, 60820, 26744, 52352, 10380, 29020, 43296, 16184, 38088,
                55892, 31352, 49632, 60188, 9860,  32212, 40988, 13712, 28348,
                60300, 55716, 14728, 41316, 18608, 36252, 31320, 49336, 372,
                20212, 34376, 51004, 23968, 6692,  32196, 59888, 44404, 51236,
                22148, 12216, 29056, 6624,  52596, 64712, 34160, 43772, 53896,
                13660, 25416, 55500, 6896,  17956, 47628, 38372, 30476, 9628,
                34684, 45260, 5680,  51760, 16092, 47084, 180,   53440, 13816,
                32272, 47260, 2892,  40940, 7420,  51988, 27804, 60812, 12716,
                35328, 47244, 26984, 52068, 38500, 48488, 8632,  26016, 58776,
                36764, 62448, 580,   14720, 53144, 30860, 9028,  4768,  41540,
                11424, 24932, 3484,  43320, 5648,  41032, 16192, 60392, 29940,
                20316, 53692, 9632,  24428, 52576, 60344, 48116, 39540, 14240,
                23288, 48924, 40456, 12436, 25920, 17392, 35652, 28408, 50820,
                23728, 56596,
            },
            {
                20796, 49340, 58260, 63376, 18888, 26216, 62464, 22608, 39300,
                48540, 55860, 7628,  15784, 64084, 56900, 6992,  25752, 20912,
                41700, 18080, 26752, 3316,  47036, 24152, 54820, 63216, 1944,
                10368, 34812, 5768,  46164, 52308, 62004, 13748, 56656, 27156,
                62956, 15216, 30052, 2356,  48692, 21120, 37200, 15416, 124,
                63620, 54108, 38724, 46888, 26652, 15360, 49728, 3104,  28032,
                46252, 64152, 21204, 52056, 61124, 14056, 55804, 22728, 42540,
                59440, 11776, 25188, 38024, 22344, 60964, 18924, 25792, 39792,
                57484, 16620, 21576, 63256, 14308, 32800, 4360,  44344, 50704,
                31552, 7796,  41284, 4552,  23592, 12528, 63500, 53332, 43720,
                16912, 33524, 24724, 60376, 19552, 34864, 17020, 61392, 29004,
                36344, 64268, 31960, 49236, 28720, 63012, 732,   44328, 56052,
                12264, 62532, 40616, 33088, 14580, 21220, 3432,  31568, 5708,
                59632, 19056, 34048, 1600,  45288, 52040, 20604, 64608, 8744,
                17004, 38936,
            },
            {
                6848,  15648, 944,   40032, 13500, 36340, 6196,  53388, 30536,
                13308, 20308, 42068, 47304, 22392, 37040, 50568, 45836, 60580,
                52396, 5572,  64068, 38752, 57036, 19364, 6040,  45160, 48816,
                39276, 25748, 31788, 58796, 28436, 9592,  6524,  23864, 47344,
                37276, 10712, 58980, 39428, 56068, 13116, 61904, 25176, 30572,
                9252,  17140, 24260, 2264,  19324, 37600, 11856, 58320, 5688,
                39960, 31820, 1776,  43084, 24736, 35712, 3424,  51600, 7184,
                18636, 64448, 33148, 3040,  54896, 42356, 34044, 5940,  62236,
                51108, 8492,  43352, 54448, 37868, 48208, 20248, 9616,  25028,
                58744, 21116, 64812, 55068, 32696, 45592, 2180,  20100, 5864,
                50924, 10436, 39884, 7960,  50216, 2480,  46008, 56832, 21784,
                44324, 8180,  18412, 38192, 20852, 10256, 35996, 22616, 6868,
                50800, 26916, 5276,  58040, 46936, 29196, 62292, 53504, 44888,
                11000, 55320, 30072, 57900, 62120, 37896, 3568,  55016, 41324,
                30872, 61856,
            },
            {
                52704, 27044, 33308, 47940, 29536, 55356, 11188, 44464, 596,
                65076, 4620,  35124, 29824, 2140,  13612, 32052, 3760,  9924,
                28700, 14868, 34104, 43532, 12204, 28036, 33480, 15424, 23064,
                52868, 64604, 16716, 1092,  21496, 39700, 34064, 17088, 42268,
                53948, 5556,  44188, 18308, 27868, 7996,  47412, 41952, 56984,
                45120, 35520, 60436, 55808, 42228, 62264, 30608, 24032, 20012,
                60004, 50360, 16216, 29628, 9400,  45492, 32540, 58304, 27600,
                39576, 46672, 14312, 49652, 8044,  28676, 11532, 48892, 23292,
                3584,  29420, 34820, 25556, 536,   61668, 56008, 35952, 16100,
                53612, 2580,  14500, 36520, 18112, 57436, 28448, 39256, 31596,
                22660, 55584, 29452, 57828, 42712, 26676, 38512, 13076, 50948,
                1256,  53988, 59788, 12696, 57896, 53028, 47620, 14216, 32672,
                38644, 16860, 43864, 35508, 1456,  11384, 41352, 17792, 26500,
                37412, 3880,  42516, 15612, 23616, 10460, 32712, 14100, 25668,
                4420,  45128,
            },
            {
                36896, 65384, 22144, 8396,  60992, 41504, 19700, 59896, 27312,
                37808, 51324, 24292, 61920, 58468, 40176, 18824, 63684, 35756,
                47776, 59192, 21928, 444,   49304, 62100, 37844, 58120, 8324,
                19784, 42596, 12164, 49392, 54800, 45020, 63960, 51052, 2680,
                21208, 32608, 25312, 65044, 35324, 52340, 3764,  33488, 18736,
                6380,  48588, 14032, 4752,  33240, 8976,  52932, 47592, 35092,
                8080,  12244, 37284, 57408, 5156,  64908, 20220, 15480, 48904,
                1176,  30232, 56444, 20312, 62580, 36384, 58164, 44228, 17864,
                37364, 64576, 10760, 18408, 45632, 12840, 28984, 42028, 5584,
                38756, 46052, 30056, 9984,  42900, 6908,  48936, 59912, 13168,
                64916, 46488, 3452,  14024, 20932, 63676, 5964,  32724, 24040,
                15968, 34764, 26608, 45204, 2260,  30736, 24976, 63832, 58720,
                3100,  48520, 23200, 64040, 19740, 55508, 36324, 8364,  50492,
                65268, 21280, 47976, 7552,  28920, 49976, 43520, 60348, 48088,
                56244, 12724,
            },
            {
                9776,  43280, 57580, 17036, 2128,  24544, 49840, 33840, 15220,
                57652, 18072, 11308, 44156, 5660,  52780, 24844, 55020, 12672,
                40688, 7744,  25328, 53196, 17436, 9312,  42252, 2640,  30384,
                35432, 4668,  61428, 37400, 26640, 4168,  11280, 29916, 60240,
                13448, 57256, 46440, 824,   15860, 40140, 11960, 22636, 64432,
                26748, 39924, 29600, 50464, 22304, 44936, 992,   15624, 55676,
                42628, 63052, 19148, 47908, 52864, 38612, 26112, 11952, 62396,
                35988, 23712, 10236, 43540, 26444, 15364, 1280,  31092, 55700,
                13360, 46880, 52400, 58992, 31864, 7640,  22276, 64080, 48748,
                26736, 60876, 22836, 50560, 62992, 26432, 15576, 35800, 1208,
                41976, 18476, 33848, 48832, 36476, 53592, 10920, 59164, 47772,
                62192, 40712, 6176,  50456, 17384, 39592, 8488,  41880, 19176,
                54528, 30500, 8852,  13804, 49440, 27996, 58784, 24332, 664,
                31260, 13476, 61148, 39312, 53420, 5452,  17576, 21888, 336,
                34388, 19716,
            },
            {
                24956, 5536,  38680, 51556, 46004, 31072, 4344,  9000,  40640,
                46184, 6904,  32204, 48736, 16176, 28112, 43256, 1000,  21088,
                30972, 56820, 44564, 36568, 31592, 55332, 24856, 51020, 60712,
                47248, 56664, 23544, 32412, 14960, 59064, 19208, 36300, 26232,
                40408, 9208,  50536, 23140, 54884, 30164, 59432, 49408, 53676,
                1204,  11096, 61368, 57372, 17476, 28092, 65220, 39364, 25332,
                3996,  27684, 33072, 23396, 11028, 41616, 2304,  54096, 43888,
                6580,  60196, 51860, 3932,  38584, 53048, 21688, 61512, 7068,
                26980, 41368, 2436,  23940, 39292, 50988, 57904, 2064,  18928,
                33220, 11908, 55432, 148,   39780, 20656, 52176, 9140,  24520,
                53844, 27848, 6480,  61656, 25072, 17284, 30136, 43152, 3708,
                28008, 10108, 21180, 65456, 33856, 56876, 4020,  46420, 28068,
                11720, 59760, 37356, 53212, 32524, 6072,  45692, 15936, 42972,
                54144, 33944, 26892, 19228, 56884, 35592, 62976, 38808, 29544,
                59172, 50524,
            },
            {
                32072, 54720, 28416, 13884, 34872, 64020, 57032, 22868, 53204,
                25812, 61084, 55524, 21832, 38532, 9744,  33176, 60868, 49572,
                15464, 3292,  61676, 11932, 5836,  64940, 20784, 13316, 27012,
                17268, 10176, 44240, 7400,  41472, 48180, 53044, 6964,  44112,
                63332, 18068, 37712, 5928,  62492, 43688, 8544,  16592, 37300,
                32600, 20868, 43316, 7148,  36648, 10732, 58572, 20488, 13008,
                52204, 45680, 92,    59612, 16648, 29340, 57132, 33880, 21268,
                16288, 40928, 32216, 18040, 65340, 5692,  46264, 39664, 50348,
                33272, 20148, 60824, 5464,  16232, 44056, 34228, 14080, 53016,
                7984,  44484, 17232, 35052, 5552,  59456, 32584, 45280, 62168,
                37360, 11716, 58596, 44732, 9044,  728,   39176, 51268, 18948,
                37000, 54452, 43620, 13568, 29092, 22168, 52752, 15428, 34720,
                248,   44820, 24924, 42140, 2296,  22132, 63596, 11740, 38060,
                60044, 4728,  9664,  45184, 1512,  24548, 11328, 46320, 7992,
                41068, 16084,
            },
            {
                44788, 62900, 3944,  20500, 10964, 42980, 17780, 37044, 13376,
                3280,  29096, 36332, 1576,  62628, 56688, 6596,  45732, 37704,
                26560, 52112, 19036, 28992, 47068, 38920, 3928,  45696, 40432,
                1748,  53260, 28616, 20408, 65344, 588,   33352, 22860, 55796,
                2008,  31208, 27220, 13968, 33724, 20188, 2584,  28756, 46072,
                62592, 14784, 25304, 52740, 47444, 41500, 4508,  34136, 30116,
                60728, 38336, 7692,  50188, 35832, 62056, 8836,  47500, 50968,
                28172, 12596, 57800, 47916, 29380, 34348, 11864, 25076, 16764,
                9604,  54020, 36200, 28424, 55832, 10032, 25220, 37976, 62712,
                41144, 30516, 58184, 25340, 48204, 12872, 29336, 56768, 4760,
                16624, 50196, 21304, 40332, 32040, 56800, 64636, 22456, 14008,
                57916, 31228, 2536,  48380, 60504, 10472, 37776, 62268, 50320,
                20424, 64956, 7088,  17604, 60968, 39480, 50816, 29216, 18628,
                47324, 22964, 51772, 41188, 64556, 32316, 15236, 52580, 27272,
                61732, 2348,
            },
            {
                22996, 35840, 49132, 59580, 52512, 27360, 6128,  61680, 44364,
                50308, 10540, 19300, 42284, 14132, 50708, 23108, 17308, 10900,
                64488, 34304, 41960, 23388, 50764, 16280, 34668, 58464, 32144,
                63612, 37072, 49544, 56120, 12100, 25608, 39220, 15816, 49756,
                11860, 46924, 53600, 60008, 39064, 48148, 25528, 58568, 41448,
                4320,  55516, 35084, 2408,  31104, 23324, 54228, 49100, 43536,
                9556,  15760, 55308, 21636, 43248, 4780,  18328, 24516, 3312,
                63448, 36844, 420,   22256, 8448,  41988, 59876, 56812, 1732,
                63656, 43040, 13724, 49108, 65056, 20612, 47664, 29220, 4940,
                23536, 49504, 3220,  65512, 42236, 8248,  17920, 39272, 23048,
                43484, 30360, 3044,  53220, 14700, 48128, 26912, 4956,  46612,
                8352,  63016, 25296, 35928, 4820,  42148, 26868, 6416,  23972,
                40136, 55888, 29464, 47832, 14408, 33680, 55660, 8736,  3052,
                57688, 35412, 16692, 12332, 28568, 48568, 58736, 4916,  19372,
                55552, 12936,
            },
            {
                9228,  17896, 41360, 25432, 2860,  39164, 56088, 30416, 20772,
                33672, 64352, 53876, 46572, 25208, 35280, 29316, 59092, 40560,
                4496,  8616,  57100, 1104,  59796, 9736,  54748, 22872, 11544,
                19256, 6248,  15388, 35100, 30944, 45852, 57752, 8904,  61736,
                34860, 19936, 41836, 3740,  10552, 56808, 51652, 13216, 22924,
                10064, 49564, 18960, 64136, 12480, 16588, 62204, 6408,  18504,
                24204, 63788, 31756, 26820, 13272, 46240, 32688, 38820, 59380,
                9568,  45532, 25832, 55744, 50072, 14992, 20380, 28816, 37504,
                47080, 23268, 3672,  32536, 7384,  39928, 656,   59908, 56364,
                19892, 9276,  15468, 36812, 21936, 54872, 46928, 2052,  64160,
                9660,  35104, 60180, 24432, 19264, 36016, 11500, 53848, 34012,
                39904, 15924, 51800, 19944, 55280, 17196, 59100, 47696, 32160,
                13588, 9224,  36008, 58112, 26672, 5432,  20288, 44864, 24892,
                30440, 61992, 6124,  55792, 37012, 3328,  22128, 43612, 33172,
                37536, 47864,
            },
            {
                30264, 57096, 6844,  32924, 47012, 16000, 10196, 48220, 12,
                40852, 16268, 7600,  31256, 5004,  63452, 2516,  20076, 52616,
                30560, 48328, 14416, 37588, 27208, 43916, 6912,  29952, 51704,
                42172, 61488, 24016, 2908,  60144, 5392,  17908, 29820, 42876,
                25280, 6388,  63996, 23752, 30060, 17616, 36112, 6080,  31676,
                60416, 38552, 44536, 26468, 58876, 37964, 1292,  27968, 57516,
                36536, 47356, 3624,  40220, 53244, 65132, 56348, 29128, 14540,
                42940, 53968, 16740, 35448, 62760, 3380,  44800, 51708, 6216,
                30992, 18136, 59132, 26108, 45068, 53800, 17868, 11496, 34748,
                51212, 45336, 31768, 53352, 28224, 61396, 33884, 26076, 51440,
                55844, 13164, 46216, 7700,  62736, 1796,  43880, 29400, 61600,
                23296, 396,   45964, 8868,  32832, 44396, 12040, 1480,  54252,
                18568, 51392, 3256,  43668, 11364, 53584, 36972, 62988, 40628,
                14088, 50168, 42892, 26092, 18796, 61040, 40452, 10444, 50732,
                24376, 64308,
            },
            {
                39028, 53068, 14328, 60704, 22576, 64984, 35496, 54080, 24596,
                59780, 27612, 57572, 38748, 12920, 47996, 43780, 55452, 12020,
                25320, 43284, 21648, 63056, 32484, 18580, 61048, 48640, 16,
                26620, 45312, 38856, 53816, 48524, 41056, 22280, 52328, 3512,
                58804, 37096, 15628, 50276, 45104, 244,   40528, 65388, 53104,
                16108, 28524, 8536,  4884,  50752, 33092, 45896, 40952, 52860,
                6972,  11876, 58996, 19800, 1628,  10232, 22916, 5428,  52136,
                20528, 30948, 6528,  39648, 10832, 32108, 26176, 55040, 12884,
                61432, 40416, 51080, 11020, 35844, 62364, 30744, 42808, 14288,
                26628, 62016, 39392, 11164, 916,   14440, 6932,  19976, 37244,
                16164, 27544, 41860, 32732, 50480, 38540, 58452, 6720,  18288,
                42056, 56964, 27748, 37956, 64664, 23444, 29744, 37180, 63520,
                41676, 25728, 62012, 21712, 31196, 60324, 2380,  49104, 10168,
                32964, 8,     21236, 46156, 8924,  52900, 29208, 57280, 16496,
                6436,  572,
            },
            {
                20548, 27496, 10564, 42720, 1864,  29492, 19824, 8096,  13732,
                45224, 3916,  21992, 51992, 18920, 58524, 8144,  37104, 33160,
                60736, 17028, 54548, 2852,  46872, 13212, 39988, 35484, 15980,
                58288, 8788,  13388, 19816, 27304, 11048, 62708, 34204, 14120,
                47600, 28716, 56136, 8272,  33108, 59072, 26948, 20020, 43988,
                1972,  34068, 48364, 22560, 54952, 19572, 10076, 14108, 21980,
                34592, 42764, 29456, 50700, 37448, 17500, 42276, 49152, 34984,
                2272,  64292, 23604, 47608, 60552, 18324, 41176, 8124,  22484,
                34456, 1144,  56924, 21456, 15436, 4860,  23688, 52216, 37780,
                2284,  6068,  58312, 20800, 49860, 44960, 41020, 59160, 48968,
                2652,  62196, 5224,  55024, 16644, 25984, 21736, 12820, 50028,
                31048, 10504, 59940, 14248, 2952,  51112, 56376, 20836, 45572,
                4580,  34168, 16344, 38892, 50624, 14856, 23648, 17980, 27752,
                57080, 53768, 38536, 62820, 34180, 14824, 1704,  35764, 26404,
                59828, 46060,
            },
            {
                33616, 63048, 50200, 36916, 45440, 56972, 51216, 39608, 63044,
                36744, 49664, 10600, 34468, 41856, 26688, 15472, 22488, 1444,
                50344, 5992,  35792, 10636, 51936, 25576, 3704,  65100, 20864,
                29396, 55588, 33564, 64104, 6796,  37372, 50064, 764,   54632,
                19520, 10980, 39604, 22352, 62304, 13780, 47432, 10768, 24348,
                57328, 39764, 63472, 14616, 42292, 29408, 58128, 64844, 26152,
                49384, 16284, 62188, 24600, 33432, 59576, 27856, 61636, 11536,
                40384, 56196, 13184, 28528, 996,   57056, 36376, 63880, 48256,
                16568, 43972, 8892,  29564, 39136, 47508, 58884, 8316,  64688,
                48332, 16984, 43184, 24948, 35568, 55608, 29484, 11836, 32056,
                23572, 39500, 20560, 29960, 8832,  65232, 47788, 55940, 37016,
                5052,  52664, 21064, 47972, 41136, 5748,  16012, 8288,  27832,
                11512, 57060, 47100, 608,   8208,  45240, 33828, 41888, 65224,
                7216,  16512, 11772, 30132, 5388,  48360, 64920, 45004, 12060,
                53996, 41816,
            },
            {
                3308,  15820, 24664, 4968,  17752, 12268, 25468, 6036,  31604,
                18264, 28972, 54780, 65376, 2000,  32048, 62068, 40012, 45944,
                28296, 64676, 41444, 30916, 57220, 22264, 43564, 53448, 5060,
                38028, 47556, 2612,  43456, 30272, 15060, 44412, 24064, 31852,
                64980, 5096,  43660, 52480, 18192, 6540,  54756, 35044, 50656,
                7204,  18664, 11692, 59652, 36408, 3700,  7584,  32380, 54344,
                908,   5176,  56436, 8960,  46808, 6492,  15316, 45284, 25284,
                18656, 32836, 50256, 44704, 21368, 51352, 14640, 4672,  29956,
                53248, 26684, 64544, 49428, 54992, 2944,  27756, 18980, 33428,
                22516, 30308, 54420, 8404,  62312, 15796, 3412,  64876, 8756,
                47212, 58544, 52984, 12276, 42936, 34884, 3064,  15504, 26724,
                63712, 44032, 34828, 25360, 30564, 62412, 35488, 48872, 60860,
                31764, 52376, 24368, 64356, 28980, 58912, 53168, 3912,  47648,
                36572, 21984, 58576, 50472, 24468, 18128, 38908, 22440, 30828,
                18836, 8980,
            },
            {
                46972, 56252, 31520, 52440, 59368, 34140, 41736, 47652, 57724,
                2936,  43640, 14648, 24352, 6244,  48628, 53324, 9808,  56108,
                13992, 23996, 19248, 6856,  38444, 60360, 9124,  33100, 12788,
                50724, 23032, 17052, 59496, 52460, 20524, 60688, 8016,  38412,
                46200, 26024, 34800, 1524,  30768, 42364, 38056, 3340,  62516,
                31280, 27040, 45136, 636,   24840, 51868, 47576, 40268, 17804,
                37796, 44576, 21572, 13296, 39624, 54908, 324,   36604, 52732,
                7196,  58496, 3892,  37884, 9412,  33560, 24756, 46112, 58240,
                11356, 36612, 5448,  17960, 32500, 12148, 41704, 45744, 53276,
                10868, 60880, 37420, 4224,  32928, 19312, 26840, 52612, 43104,
                17600, 34008, 736,   49204, 60420, 22892, 45800, 40224, 32588,
                1816,  17344, 7168,  11588, 58080, 19000, 43276, 23836, 38692,
                14116, 6612,  19472, 41168, 10692, 37492, 20760, 13032, 30676,
                25156, 44956, 40156, 3212,  60564, 42780, 56352, 6652,  50840,
                61772, 37220,
            },
            {
                12564, 22096, 40068, 7664,  28372, 62096, 1152,  15728, 22720,
                61448, 9148,  39796, 46400, 60484, 17520, 29212, 20552, 4804,
                34996, 58444, 47064, 50004, 15264, 29592, 17964, 46528, 26380,
                62656, 9992,  41244, 25640, 35424, 4236,  55704, 28116, 16992,
                58308, 13020, 61308, 48856, 23900, 57548, 28012, 20936, 15276,
                40592, 54264, 49056, 32984, 15892, 62688, 21188, 12600, 59924,
                30444, 63188, 27616, 52304, 31504, 64620, 21104, 29948, 9956,
                62172, 41916, 26440, 16956, 54396, 59732, 42484, 1780,  20164,
                39936, 22984, 42948, 57664, 24608, 63112, 36108, 15012, 1404,
                39944, 27120, 13744, 51516, 45868, 40308, 57188, 21944, 37048,
                4568,  25356, 15044, 37680, 27452, 6296,  11240, 61792, 19812,
                49412, 56660, 61164, 39252, 52996, 1140,  10228, 54196, 2816,
                59060, 43968, 34712, 55984, 16632, 26644, 5336,  55376, 61556,
                1260,  52276, 9520,  32448, 27692, 13716, 10280, 33416, 3772,
                28232, 58148,
            },
            {
                49680, 780,   64044, 44336, 20180, 10116, 54628, 36396, 51756,
                27316, 33292, 55968, 20964, 36060, 11636, 44540, 38196, 62796,
                42772, 680,   11548, 26844, 62020, 2288,  55220, 39632, 58460,
                6008,  31616, 56968, 1472,  12492, 48344, 40520, 10468, 51312,
                2460,  21000, 53888, 8864,  16068, 64224, 11784, 46632, 60308,
                9516,  22040, 5628,  56948, 38528, 28724, 44128, 9200,  23796,
                5880,  48808, 35608, 2596,  18092, 42368, 49936, 57116, 23344,
                46952, 14360, 31448, 65104, 5160,  27884, 12704, 63080, 31344,
                7816,  61024, 14752, 300,   46544, 7696,  20880, 59860, 55776,
                6340,  47796, 19864, 63944, 10272, 64,    48172, 13304, 60244,
                30900, 45672, 64108, 56188, 18856, 31700, 51532, 54864, 8276,
                41744, 23160, 28192, 15016, 45848, 33536, 26452, 65468, 17428,
                21876, 49888, 30112, 2076,  51508, 60740, 42696, 49288, 18316,
                34732, 15224, 64096, 20172, 54736, 37108, 46000, 63572, 15928,
                40812, 25464,
            },
            {
                33916, 18272, 36104, 13844, 48820, 24096, 30932, 44860, 12080,
                19432, 5144,  49916, 368,   30760, 58848, 7336,  50680, 25484,
                17152, 31608, 53476, 36940, 43340, 5508,  34644, 21108, 14396,
                44756, 36976, 19588, 46016, 64652, 32372, 22688, 62476, 43128,
                33584, 29356, 40764, 36512, 45148, 4472,  32776, 51660, 1228,
                43356, 35372, 64952, 11192, 19384, 4184,  54084, 34296, 57368,
                41256, 15288, 9564,  57980, 25684, 11844, 4012,  16696, 39008,
                34876, 1740,  20512, 44316, 48688, 39268, 18692, 35720, 50356,
                47224, 54576, 33792, 51956, 38516, 26920, 50340, 29928, 43324,
                24176, 57852, 31360, 42152, 23436, 29612, 34536, 6156,  50268,
                10704, 54024, 7888,  41248, 2188,  58812, 38380, 25024, 29748,
                12736, 46932, 36792, 4612,  20568, 50696, 40816, 29676, 47496,
                36840, 11336, 63316, 6460,  46416, 31868, 22536, 8380,  39116,
                28080, 58108, 41660, 6860,  49140, 480,   23264, 52080, 20264,
                44144, 8228,
            },
            {
                60896, 28916, 52796, 57748, 3416,  38268, 64968, 7292,  58392,
                41452, 63392, 38020, 25880, 13736, 52556, 23308, 2616,  33744,
                55316, 8844,  64172, 19684, 23440, 52632, 48868, 63904, 620,
                28888, 52020, 15640, 54404, 27276, 7288,  17996, 36620, 6032,
                47808, 15176, 3532,  56124, 26592, 18468, 39496, 55112, 24196,
                17320, 30080, 51192, 26296, 41696, 61216, 46848, 1492,  26388,
                51636, 20048, 45424, 61128, 37560, 47424, 32876, 63148, 5416,
                51820, 60192, 55520, 11360, 23044, 52824, 6228,  56092, 16364,
                25476, 3920,  28904, 19428, 10424, 61564, 4152,  34276, 17848,
                9320,  35740, 2856,  16168, 53992, 59048, 18120, 63364, 39844,
                27924, 20428, 23840, 34404, 13124, 47176, 16508, 5244,  35576,
                64820, 60,    53840, 32016, 62832, 7800,  13480, 3828,  57472,
                8220,  27460, 39824, 24564, 14456, 36044, 11700, 63764, 45612,
                4456,  12108, 46996, 26360, 16476, 57440, 29144, 35796, 58916,
                2328,  54476,
            },
            {
                46724, 10376, 4852,  41156, 26772, 16560, 50888, 2636,  28840,
                47388, 16748, 9484,  57644, 42508, 64072, 39372, 15332, 60752,
                45832, 28776, 40312, 14448, 7008,  31332, 11996, 25012, 41940,
                8660,  61580, 23952, 4856,  39808, 59756, 14128, 53588, 25480,
                57788, 64580, 23188, 10584, 60960, 49168, 6800,  28420, 36968,
                57612, 3716,  14420, 47708, 7380,  31440, 13772, 17620, 38220,
                65472, 3784,  32088, 23088, 52936, 7860,  27844, 44708, 12752,
                24796, 29068, 8332,  37268, 32692, 2632,  61200, 29552, 9444,
                43676, 12924, 65492, 41652, 56176, 16272, 44644, 12460, 64464,
                52524, 39356, 60572, 49020, 7848,  37572, 25572, 44512, 14732,
                1620,  61480, 42916, 50520, 28748, 63000, 21596, 44688, 59476,
                50412, 18500, 10320, 43096, 55488, 24964, 60164, 34416, 42608,
                16152, 54544, 61332, 18604, 58712, 41028, 2960,  56396, 24156,
                31084, 53784, 21740, 33340, 38840, 61984, 5764,  11472, 31736,
                14144, 23784,
            },
            {
                42500, 16160, 32416, 22564, 60160, 43872, 33568, 21508, 13936,
                35228, 23976, 54368, 32324, 4520,  18720, 28104, 48724, 11332,
                21652, 3184,  49848, 57028, 44448, 60248, 38416, 17172, 57776,
                32784, 48560, 11244, 34192, 43684, 49836, 30148, 45444, 224,
                19400, 31836, 51980, 43700, 34636, 21316, 58584, 13256, 9004,
                45724, 61652, 38864, 55612, 23876, 36312, 59296, 49704, 29412,
                11552, 55756, 43004, 13624, 772,   19028, 59708, 54520, 41080,
                17656, 48288, 43108, 63640, 15724, 26356, 45332, 40060, 20780,
                58004, 37748, 22512, 2072,  30844, 48156, 23540, 55156, 768,
                26532, 21140, 13236, 27784, 43584, 11832, 4244,  52508, 31720,
                57648, 36884, 16876, 4208,  53684, 8804,  39508, 3156,  14528,
                31420, 27088, 58228, 22664, 16028, 37656, 46696, 19840, 51572,
                23176, 32456, 288,   47676, 52928, 29176, 20136, 49980, 15572,
                37724, 59144, 2200,  51564, 9192,  44444, 19112, 40396, 50172,
                65416, 37716,
            },
            {
                20556, 62064, 54980, 47540, 12144, 8120,  55920, 39984, 52224,
                61888, 1852,  44332, 20688, 47016, 35676, 8356,  56520, 43028,
                36212, 5900,  24812, 34360, 10248, 28076, 2836,  54512, 45392,
                4252,  20716, 38720, 58760, 2752,  21244, 9220,  61860, 11920,
                41600, 38428, 7972,  14392, 29264, 1712,  41684, 63912, 50512,
                21872, 33244, 18384, 128,   63984, 20396, 8996,  44584, 22700,
                6728,  35420, 27188, 62584, 39996, 48864, 35164, 21664, 9348,
                31932, 57432, 468,   19792, 51420, 59260, 12412, 34328, 63292,
                6604,  46188, 51588, 35320, 59464, 5720,  37304, 41016, 31532,
                49636, 45296, 4812,  34032, 65064, 56076, 22460, 48436, 19496,
                7164,  46720, 55624, 11916, 32864, 26048, 48796, 56380, 23612,
                42016, 7240,  39108, 48604, 5936,  30396, 11124, 1884,  64220,
                6840,  44296, 12876, 35012, 5412,  9932,  44092, 33756, 62060,
                7536,  18804, 42984, 64692, 14504, 25324, 54336, 47236, 4176,
                27376, 7244,
            },
            {
                50748, 1392,  35404, 5872,  30096, 63156, 18676, 25908, 10532,
                59080, 6864,  29344, 13104, 51620, 59960, 968,   31432, 17680,
                63692, 52316, 59224, 15852, 47028, 65444, 21868, 35112, 26476,
                14652, 63672, 29428, 17100, 54844, 25184, 37812, 33732, 56732,
                27724, 48912, 4584,  62952, 54444, 46424, 16976, 32376, 26124,
                5916,  43928, 10628, 29148, 42440, 52088, 4308,  55176, 40680,
                59996, 50336, 16936, 9188,  30908, 57352, 14884, 2832,  64816,
                38312, 6292,  27268, 34956, 41276, 7932,  23812, 48768, 864,
                31956, 26456, 17800, 8964,  13840, 27620, 20052, 58268, 7392,
                15460, 61516, 54440, 19368, 30856, 1108,  39264, 62788, 35244,
                10220, 29804, 22236, 65332, 44252, 19168, 60936, 34928, 10844,
                52980, 61768, 33604, 2780,  63608, 52620, 25876, 40116, 56916,
                28184, 38012, 50528, 21248, 26776, 65168, 54340, 1040,  25652,
                47720, 11252, 35396, 28668, 5196,  32244, 60272, 22920, 16896,
                56956, 30720,
            },
            {
                59044, 25060, 40232, 14612, 49724, 37160, 456,   45348, 31540,
                48456, 37020, 41108, 65216, 10304, 24860, 39968, 50164, 26552,
                12964, 38956, 30312, 176,   41416, 18808, 49248, 9104,  40084,
                50612, 53500, 7016,  46392, 12780, 65000, 50672, 6488,  15960,
                22748, 59436, 18212, 36900, 25092, 10896, 38672, 3616,  53268,
                14140, 59520, 54624, 48696, 34424, 12488, 26088, 32816, 14920,
                1168,  20296, 44836, 54188, 23832, 6056,  26708, 43580, 50284,
                23724, 53648, 61400, 13848, 46600, 3980,  55936, 18780, 52776,
                15304, 55284, 61416, 49464, 42456, 62884, 52124, 11172, 35060,
                23144, 41984, 10148, 37216, 51084, 16772, 27248, 13284, 42576,
                60544, 25380, 40448, 752,   37432, 5664,  29472, 16792, 1356,
                46884, 20304, 12708, 17880, 44628, 59220, 14372, 31796, 47360,
                9532,  15720, 59752, 56020, 41468, 17228, 38144, 13824, 29920,
                40576, 60612, 23280, 49748, 56452, 37328, 10804, 2424,  34452,
                43868, 12400,
            },
            {
                18844, 45244, 64516, 27672, 21300, 58328, 53232, 5184,  22856,
                14576, 17972, 26336, 55952, 33472, 16020, 22088, 61720, 6980,
                46208, 19988, 54708, 23216, 32912, 5548,  56232, 13224, 61012,
                900,   24220, 36140, 41624, 32312, 1384,  19144, 42472, 53156,
                3228,  45700, 31276, 51368, 20684, 57240, 61436, 48104, 30180,
                40856, 35976, 24444, 5232,  16456, 58816, 38408, 63232, 48016,
                30244, 37428, 64492, 3612,  34040, 46776, 62272, 36404, 12416,
                17120, 44652, 10024, 21152, 58248, 31112, 37212, 27512, 43136,
                11112, 40256, 5012,  23684, 33208, 3652,  16812, 46464, 64088,
                1736,  29236, 57276, 24696, 8036,  46204, 57948, 5684,  53368,
                3272,  48124, 15036, 52052, 59004, 13400, 54532, 43168, 64472,
                25756, 36632, 57196, 28608, 41088, 21372, 35276, 5400,  18560,
                61664, 24312, 2772,  30960, 7488,  47140, 23172, 58892, 50944,
                3800,  54808, 16208, 1212,  46300, 20268, 41964, 52976, 63580,
                38368, 8692,
            },
            {
                33388, 4976,  52548, 9980,  43032, 16900, 33940, 41512, 57268,
                62916, 50788, 4756,  46104, 2552,  53064, 43208, 57620, 3108,
                34832, 10640, 61116, 7980,  51320, 62564, 37332, 28376, 31844,
                43960, 19484, 10508, 56492, 27124, 60492, 47756, 29616, 35408,
                62352, 13028, 8972,  42784, 600,   34292, 7268,  23052, 18592,
                64568, 2360,  20676, 62088, 28180, 45876, 2248,  18648, 10204,
                24704, 56304, 12896, 41220, 51576, 10864, 19548, 29604, 56284,
                2024,  33608, 28680, 49060, 39524, 16520, 64988, 8780,  59952,
                35572, 29684, 20200, 45512, 38100, 56868, 30152, 25232, 38700,
                52652, 18236, 47900, 3692,  40288, 60872, 32968, 23352, 17560,
                34092, 56976, 20372, 31544, 9164,  49684, 24508, 39308, 8348,
                31180, 51892, 4192,  49256, 9704,  544,   55708, 49944, 42144,
                53696, 35596, 45268, 11968, 62160, 33424, 4528,  10516, 43832,
                20916, 32632, 38548, 26900, 8880,  61536, 30012, 14004, 25860,
                48996, 54732,
            },
            {
                15328, 23492, 37864, 2820,  60804, 29704, 8336,  12720, 27420,
                34908, 9332,  21564, 38860, 28684, 7956,  36576, 14196, 29508,
                41200, 48652, 27328, 43984, 14932, 25000, 42648, 4384,  16848,
                54688, 47380, 63180, 3952,  21896, 39244, 14464, 10364, 24092,
                55120, 40376, 26040, 65380, 15516, 28620, 44264, 12588, 55736,
                8988,  45436, 51584, 40292, 7416,  56412, 22780, 41828, 52964,
                61060, 7332,  28344, 22372, 16628, 59024, 39332, 4232,  60724,
                41024, 51984, 63424, 4952,  24928, 1272,  50972, 21720, 47488,
                2476,  56504, 64440, 12740, 140,   50900, 9408,  43020, 5228,
                60016, 12504, 34448, 63568, 15600, 10944, 43400, 49968, 28464,
                38924, 7072,  62156, 26956, 45516, 35836, 21348, 3596,  59616,
                45064, 15260, 23224, 61408, 38044, 25544, 65080, 12420, 22172,
                4080,  28672, 39816, 19680, 51460, 26032, 55104, 35908, 64052,
                14656, 6204,  62840, 44780, 57820, 17552, 50444, 6828,  21476,
                412,   28820,
            },
            {
                62216, 56532, 46064, 13560, 48976, 24324, 64228, 47816, 54224,
                1148,  44016, 60844, 56960, 18876, 63192, 47072, 20600, 55088,
                64884, 17160, 1696,  58344, 36040, 11036, 48040, 57988, 22808,
                8200,  34744, 28912, 15976, 52336, 7828,  57884, 44740, 2084,
                17992, 32628, 4984,  47992, 56404, 36480, 60448, 50128, 26764,
                33780, 15300, 30804, 11388, 36244, 50756, 13952, 32100, 3804,
                34532, 43768, 49320, 2048,  31876, 53752, 8052,  48528, 21884,
                25872, 8760,  18564, 35204, 55496, 11976, 44352, 33060, 14860,
                24476, 6512,  48712, 26888, 18672, 60568, 22136, 14452, 49216,
                32028, 21132, 44616, 26768, 55312, 30428, 20632, 252,   64760,
                46500, 11468, 42388, 2080,  16596, 63428, 32772, 55964, 18684,
                11072, 41552, 6112,  33436, 53076, 15956, 29632, 45844, 33128,
                8784,  63276, 48812, 676,   58500, 15452, 42600, 18752, 28052,
                47528, 51740, 22672, 12608, 35296, 2716,  32744, 40184, 47104,
                59564, 42708,
            },
            {
                7764,  31152, 20664, 34540, 53000, 39752, 3576,  20028, 37720,
                16608, 30580, 24676, 13048, 49852, 32276, 10756, 25588, 4072,
                32808, 22476, 38676, 53512, 30868, 18456, 2192,  64740, 51204,
                38620, 12880, 60136, 42904, 48788, 36432, 26368, 64376, 49524,
                38156, 60692, 52244, 22268, 9588,  19500, 4348,  41128, 1388,
                38092, 58940, 54304, 25472, 19452, 65196, 29184, 47340, 57824,
                15664, 19844, 37124, 63160, 45324, 24316, 35304, 15244, 44220,
                32260, 13444, 46456, 59176, 42084, 29756, 62368, 7120,  40480,
                52228, 36984, 31052, 41208, 53884, 32796, 36728, 57456, 28048,
                8408,  53544, 2344,  38352, 5480,  51328, 36748, 8500,  54940,
                14696, 22692, 54004, 37700, 51552, 6188,  12728, 48348, 27540,
                53540, 63888, 30108, 47128, 18992, 42724, 7232,  60084, 37824,
                17536, 54608, 14040, 23008, 37084, 30616, 8496,  2592,  39156,
                59880, 9500,  31140, 41328, 53236, 24280, 64372, 55460, 11052,
                17748, 35232,
            },
            {
                50388, 11648, 1936,  59928, 17824, 6432,  32332, 45476, 59672,
                11220, 52196, 35992, 6520,  41792, 1324,  59728, 51784, 44244,
                8920,  49668, 13084, 4840,  60616, 21568, 40340, 26804, 33168,
                5952,  56736, 24832, 1536,  32548, 18988, 5380,  30660, 21376,
                8224,  14760, 27520, 44676, 39408, 30980, 53728, 23860, 63536,
                18172, 47604, 5016,  42232, 840,   44528, 9700,  5488,  39680,
                26496, 51676, 9612,  13760, 59776, 5632,  27480, 55880, 65316,
                76,    53972, 38568, 6260,  15528, 20384, 53124, 26516, 60800,
                18168, 57732, 10100, 15740, 7208,  62588, 4452,  45360, 18028,
                65248, 41332, 59604, 17008, 23028, 62784, 13144, 58356, 25672,
                35140, 31348, 59544, 19596, 29664, 57588, 41308, 376,   38124,
                34780, 21640, 2576,  58360, 10224, 56012, 1636,  51196, 24820,
                57500, 41840, 27364, 6672,  64792, 44600, 49804, 56260, 24568,
                34568, 260,   57224, 19268, 5564,  43860, 15284, 28000, 37028,
                4684,  25160,
            },
            {
                39032, 65208, 28168, 41580, 57336, 25888, 14604, 55568, 22424,
                40912, 2748,  61804, 45756, 17304, 28108, 39316, 15476, 35688,
                57292, 61940, 26156, 45772, 34724, 50228, 9488,  44292, 14488,
                20196, 46668, 40580, 10668, 54516, 61936, 12952, 56276, 42740,
                34736, 58872, 2600,  12032, 62816, 16660, 7084,  46756, 13608,
                29304, 8520,  22084, 61864, 57052, 34256, 17700, 60516, 23548,
                63964, 512,   55420, 29072, 41796, 18632, 50768, 10820, 36960,
                19660, 60156, 27876, 23664, 49904, 2116,  37784, 10544, 45464,
                3276,  22416, 63760, 43468, 47264, 25108, 12076, 40240, 1592,
                35512, 25420, 10836, 49008, 33712, 45948, 41112, 18716, 43940,
                3332,  48752, 4920,  10432, 25844, 44300, 23084, 15596, 60228,
                6940,  49396, 13492, 39248, 27864, 35556, 21560, 14524, 31428,
                3096,  10688, 47632, 33804, 52956, 11764, 21332, 62432, 13536,
                45776, 16736, 27288, 48952, 38260, 59008, 8764,  22324, 62512,
                45012, 53952,
            },
            {
                14512, 47552, 22784, 8648,  44840, 36320, 62896, 28964, 7180,
                48476, 27028, 55060, 32848, 21780, 64316, 54616, 23588, 5420,
                18996, 29204, 40796, 16504, 7316,  55784, 29528, 61880, 53112,
                4048,  30712, 63740, 16576, 22628, 45904, 39020, 792,   50928,
                17300, 46820, 29752, 55644, 35316, 50508, 57756, 33580, 42524,
                59868, 51776, 32640, 14796, 38812, 27152, 53300, 49376, 12328,
                35880, 46280, 32620, 22192, 38496, 2664,  62112, 42880, 30196,
                4328,  48000, 9912,  34004, 64304, 30764, 58632, 14244, 34948,
                28584, 50232, 33400, 1352,  20452, 29904, 49712, 55724, 22668,
                51932, 14584, 30728, 57044, 7936,  27792, 1908,  30092, 52436,
                63140, 16404, 40504, 61192, 50556, 7856,  65180, 30912, 46260,
                26224, 17568, 62872, 24036, 45328, 61788, 48964, 40744, 62904,
                45580, 19992, 61056, 38940, 17820, 5396,  28792, 32920, 3684,
                42080, 53532, 65360, 11928, 30120, 3112,  47764, 51348, 31912,
                1416,  19188,
            },
            {
                56344, 5104,  32704, 51244, 12848, 584,   49600, 10096, 34656,
                65088, 19084, 14012, 9020,  51056, 4284,  12196, 31408, 42852,
                47868, 364,   54332, 11788, 64364, 24308, 1664,  17332, 36080,
                58956, 49360, 27280, 3248,  35272, 52084, 28864, 25292, 10400,
                63724, 23348, 6140,  42244, 25172, 1976,  20324, 27052, 11008,
                3972,  37120, 25180, 48852, 11768, 3896,  21464, 31612, 6984,
                43204, 17060, 4616,  57604, 47936, 8772,  33556, 14256, 21528,
                52252, 40832, 16784, 45292, 55300, 4880,  42868, 24584, 48352,
                6364,  55512, 13140, 38704, 52940, 59116, 16248, 33808, 62208,
                6756,  47504, 42416, 3792,  53604, 21500, 61484, 11132, 37348,
                7776,  24224, 46224, 32504, 13948, 35528, 19192, 52260, 10556,
                57304, 43036, 53944, 8084,  31920, 3732,  11620, 26968, 5456,
                36024, 52016, 30016, 1184,  24464, 54920, 40180, 47704, 58704,
                20032, 36228, 6944,  25068, 61264, 34516, 20320, 12904, 39848,
                60076, 29648,
            },
            {
                42552, 35776, 17108, 61968, 54316, 20432, 40588, 17264, 44048,
                3836,  23760, 39688, 59300, 43460, 35216, 48848, 58588, 9720,
                63628, 36876, 21960, 31848, 43044, 38308, 48372, 11140, 41928,
                22292, 8024,  38132, 44496, 60500, 5716,  14816, 58536, 40632,
                32424, 53288, 37376, 13416, 48340, 9372,  64612, 38912, 45812,
                56164, 18936, 64092, 6564,  55504, 45792, 41268, 58824, 28452,
                54900, 20752, 61776, 12712, 52736, 23844, 58756, 45900, 25912,
                56548, 6976,  61260, 22020, 12628, 39092, 19252, 54132, 63356,
                17076, 40724, 61176, 27216, 7924,  36164, 3956,  10136, 28996,
                19632, 60472, 36460, 18224, 59112, 39456, 15160, 49900, 56612,
                20596, 28388, 54288, 1452,  38804, 56140, 2844,  40404, 5204,
                32760, 36400, 1032,  50896, 16256, 20184, 56704, 53284, 17012,
                23472, 8392,  14184, 57324, 43952, 61704, 15652, 8140,  23892,
                10724, 31468, 50904, 44932, 15828, 55224, 42168, 57960, 6560,
                24372, 9428,
            },
            {
                62856, 26308, 2932,  38632, 30216, 24052, 61240, 31488, 52448,
                56860, 36444, 46940, 29300, 756,   25124, 19900, 38624, 15156,
                25924, 6732,  51332, 57912, 3820,  20112, 59424, 32572, 25960,
                55408, 13336, 18940, 56932, 9816,  31576, 20636, 48236, 7340,
                19340, 3232,  62132, 21484, 59232, 31780, 15840, 53524, 22820,
                268,   30756, 43748, 16640, 34756, 60756, 1412,  15324, 37996,
                9304,  50024, 40200, 27084, 31148, 16372, 37800, 1008,  63128,
                11964, 35912, 31024, 1428,  27412, 49556, 8820,  36372, 528,
                32060, 9832,  23884, 18588, 46428, 63988, 43068, 24264, 39088,
                44748, 360,   27308, 12068, 24112, 47116, 34440, 6168,  31460,
                42580, 12236, 64164, 17768, 22752, 45588, 28552, 62544, 24456,
                20092, 12544, 28876, 41280, 60408, 37736, 33532, 43828, 65464,
                39840, 59420, 48460, 26556, 35168, 3300,  50628, 37064, 63040,
                54524, 4316,  40020, 21488, 1012,  9640,  27076, 36496, 16724,
                45816, 52288,
            },
            {
                21292, 49144, 57128, 11032, 43696, 6180,  58032, 2232,  13668,
                27572, 11088, 6892,  63644, 15972, 56496, 5804,  28860, 52972,
                60640, 33976, 17224, 46744, 13820, 28300, 52660, 5976,  46340,
                476,   65476, 33900, 49992, 24436, 42888, 64328, 54248, 36328,
                45044, 15616, 50124, 27408, 39600, 44360, 5460,  35732, 60940,
                13800, 50112, 40548, 9404,  28948, 24620, 19436, 51752, 64972,
                25116, 2740,  33928, 6804,  64168, 44372, 6424,  49312, 28780,
                18008, 39852, 51340, 43792, 57608, 15484, 60588, 29232, 21356,
                43500, 47532, 56748, 2088,  31088, 12576, 54644, 49352, 57772,
                13880, 53036, 64776, 32292, 8856,  43552, 63652, 26200, 616,
                60412, 48136, 34264, 5592,  58540, 11104, 49856, 14896, 59412,
                47296, 55592, 64524, 22592, 9888,  48432, 6464,  29168, 1440,
                11344, 32552, 42224, 6104,  21580, 12612, 31244, 18232, 26784,
                43664, 13664, 56632, 63384, 32860, 46856, 53452, 3456,  64764,
                33736, 13380,
            },
            {
                40716, 8004,  19396, 46700, 15228, 26524, 36816, 47888, 21796,
                60676, 50148, 18644, 31860, 51376, 40948, 62416, 46376, 2252,
                43384, 11004, 40492, 24716, 61632, 9236,  35400, 62640, 15196,
                51856, 41120, 27992, 2648,  16252, 38848, 4444,  12560, 25952,
                61020, 30252, 8072,  56840, 1252,  18388, 51912, 28648, 8712,
                26096, 58180, 20648, 53872, 62792, 48636, 5600,  32420, 11704,
                43964, 59836, 47168, 19008, 10840, 55928, 20640, 34572, 53688,
                8860,  23332, 64928, 5760,  34504, 24300, 45960, 5372,  51920,
                59572, 14772, 35068, 50856, 41780, 21748, 17040, 3148,  31524,
                7476,  22188, 48660, 37752, 51828, 4604,  17828, 53732, 38076,
                15844, 9048,  39832, 52656, 26332, 31684, 8056,  42156, 34392,
                3844,  38320, 6748,  45124, 26576, 58024, 14952, 24776, 47172,
                20896, 55768, 17408, 53116, 64300, 46128, 59904, 41564, 1624,
                48908, 29644, 17464, 24612, 38340, 59668, 18544, 22840, 49656,
                28440, 196,
            },
            {
                55248, 31384, 60232, 34596, 63280, 53832, 9212,  40160, 33120,
                5120,  41924, 55348, 37692, 22316, 9280,  12664, 32660, 23120,
                19140, 49908, 29848, 1076,  54752, 44476, 18144, 23240, 38696,
                30488, 21076, 11448, 60956, 53256, 47328, 29440, 34208, 40,
                55852, 42488, 11092, 34708, 47092, 12652, 62376, 42132, 47944,
                33476, 5984,  36892, 1932,  13204, 39536, 45180, 36160, 56692,
                21828, 15064, 36756, 51448, 41640, 24552, 61120, 4088,  42636,
                58332, 48144, 13764, 20064, 54776, 40964, 63800, 12136, 38284,
                25808, 7592,  65496, 27704, 4676,  60816, 38068, 26484, 62888,
                35252, 41728, 16472, 2760,  57508, 29964, 12500, 22140, 44844,
                58236, 24640, 19708, 43420, 61324, 36988, 20900, 54060, 17492,
                27632, 51884, 18760, 13836, 35180, 2416,  62520, 51544, 38504,
                60780, 4160,  30340, 38108, 9792,  28056, 52152, 6788,  22272,
                57956, 35588, 7908,  50184, 12176, 4992,  30684, 43388, 10860,
                59236, 37632,
            },
            {
                15748, 42928, 4744,  28696, 1328,  49880, 19940, 16180, 64348,
                45560, 24712, 14228, 1692,  45216, 27892, 54296, 36912, 59640,
                4500,  56856, 64128, 7620,  38048, 31404, 49516, 4808,  43636,
                8704,  56292, 45176, 36168, 7572,  22464, 62540, 18420, 49596,
                23392, 38572, 20484, 65288, 25148, 31292, 22124, 2644,  56024,
                16264, 64892, 44824, 31772, 22428, 58556, 16940, 7968,  26712,
                340,   29800, 55196, 5304,  27828, 13504, 38760, 31900, 16120,
                26376, 1808,  33296, 29384, 10428, 2364,  18044, 31136, 55748,
                2880,  19464, 39432, 11272, 53552, 33412, 9008,  45396, 18756,
                55428, 11108, 60252, 25776, 20228, 40532, 61112, 49364, 33420,
                3688,  30260, 50952, 2036,  14548, 4736,  64672, 45316, 4,
                11404, 62276, 31804, 50068, 55184, 40252, 31284, 17684, 7668,
                34492, 13616, 49480, 23828, 0,     16040, 39060, 34200, 11680,
                64948, 46716, 3176,  61376, 27448, 52424, 15020, 39628, 54488,
                5424,  25168,
            },
            {
                63616, 21952, 51876, 13524, 23692, 41648, 30840, 55752, 3136,
                28660, 10072, 34320, 62860, 59156, 6504,  17852, 49148, 14756,
                39472, 27016, 34444, 20652, 15124, 59932, 12036, 53660, 26992,
                63340, 3992,  17468, 32236, 57332, 13908, 40620, 9472,  59180,
                6208,  16500, 51416, 4296,  54172, 36920, 58612, 10212, 19924,
                39004, 24668, 11152, 52092, 27740, 4008,  54112, 61748, 49740,
                42240, 63260, 9652,  32824, 58548, 1268,  52100, 46388, 11324,
                62696, 38332, 45488, 60288, 51024, 36000, 48548, 22300, 43064,
                34124, 57952, 44812, 23356, 47924, 15296, 59292, 51948, 1096,
                29120, 47400, 5736,  44684, 34144, 54924, 9260,  27488, 6700,
                65488, 55364, 11824, 33972, 57088, 48280, 24828, 30288, 38980,
                58388, 24148, 42512, 4964,  20776, 9116,  43956, 22904, 57300,
                45628, 26068, 61956, 54508, 43212, 58936, 20152, 55664, 25700,
                15356, 39580, 19388, 42320, 34100, 45652, 63932, 8048,  33204,
                19044, 48444,
            },
            {
                35332, 9060,  46308, 57312, 35932, 7360,  59656, 12252, 37960,
                51952, 57972, 48600, 20400, 30816, 42324, 25344, 65032, 8372,
                45708, 11608, 52344, 47740, 42556, 24940, 3120,  58116, 19468,
                36956, 48260, 25248, 51160, 1688,  27416, 44424, 54484, 35956,
                46564, 32756, 60512, 43808, 14592, 6948,  41224, 27648, 45828,
                53700, 3528,  61252, 48356, 41164, 35096, 20232, 30864, 12900,
                38204, 17652, 48112, 21148, 44440, 65280, 18680, 29308, 56216,
                4836,  21200, 53892, 6876,  14968, 26828, 62436, 8280,  14076,
                49780, 29320, 16968, 62776, 6324,  29988, 41048, 24500, 13128,
                39368, 23192, 31192, 63820, 14160, 872,   47192, 36948, 15144,
                20740, 39056, 46396, 17720, 28396, 40348, 7176,  13176, 55400,
                47792, 8740,  36188, 59992, 29780, 63668, 12396, 53484, 1004,
                41508, 10612, 19132, 36072, 7900,  32124, 48140, 4212,  43996,
                52692, 31968, 9548,  23220, 57148, 812,   21156, 26204, 60584,
                44472, 12220,
            },
            {
                1572,  27048, 39148, 17248, 65408, 44548, 26144, 47712, 18148,
                22768, 7260,  39636, 16072, 3360,  52024, 35064, 132,   30160,
                56044, 23448, 17336, 2172,  63212, 35852, 29368, 40644, 14532,
                33468, 60400, 12992, 42004, 64860, 19540, 31632, 2988,  24920,
                12452, 29016, 1596,  26824, 49200, 17756, 63748, 50956, 34580,
                13752, 30316, 18932, 7556,  15152, 59816, 10384, 46660, 23372,
                53128, 4288,  25764, 39716, 15420, 34852, 6704,  23408, 49608,
                35728, 17376, 41196, 24488, 43628, 57476, 38524, 3824,  53228,
                60540, 9864,  276,   37076, 55144, 20824, 4132,  35704, 64704,
                49156, 58440, 9600,  38140, 51248, 18276, 23708, 61824, 42260,
                50328, 25648, 7840,  63196, 22500, 51360, 60172, 19100, 33688,
                21756, 15340, 1604,  46524, 16720, 25716, 49012, 36584, 28428,
                64756, 32872, 3796,  47312, 13944, 63560, 29192, 10648, 62080,
                1240,  28320, 60100, 54272, 16316, 11708, 37476, 51028, 4172,
                29636, 56200,
            },
            {
                52844, 61672, 30976, 3016,  11100, 21168, 52700, 500,   34944,
                60884, 43884, 26856, 54788, 11992, 46496, 57480, 21320, 41568,
                36484, 4788,  60904, 32528, 9976,  56148, 7064,  46192, 51796,
                1220,  9552,  22940, 38420, 6916,  49004, 61092, 15900, 52044,
                63940, 41752, 57396, 10052, 38828, 23484, 32256, 460,   5756,
                59360, 42592, 37904, 55992, 25008, 44036, 2412,  64808, 6680,
                33528, 56296, 61560, 2320,  50496, 54892, 9900,  43140, 61460,
                27000, 9080,  64540, 32128, 696,   19508, 45720, 33000, 23656,
                40272, 26732, 46860, 32556, 12448, 50488, 44208, 56268, 7976,
                20104, 15848, 53400, 4204,  27788, 43288, 57208, 30464, 2520,
                10972, 58140, 35644, 716,   43148, 10528, 36564, 2500,  43724,
                50772, 64240, 27696, 53020, 38376, 3364,  58832, 5896,  15868,
                21552, 50636, 59336, 27056, 40524, 22956, 51388, 17860, 37264,
                21192, 49196, 6336,  35924, 47836, 30964, 41656, 57708, 16884,
                22900, 40248,
            },
            {
                14692, 19316, 48020, 54988, 33508, 39880, 5920,  62408, 29860,
                14092, 3924,  32668, 63868, 37472, 24216, 9108,  15764, 62140,
                53032, 27492, 44580, 39940, 21136, 49064, 16824, 23508, 64452,
                30848, 45276, 57616, 29024, 54816, 10708, 36624, 22196, 39672,
                7140,  18336, 21556, 33740, 61028, 44900, 12772, 55252, 25892,
                47880, 21924, 64192, 1340,  32652, 49488, 39340, 27164, 36516,
                14644, 45116, 28592, 11760, 22864, 30524, 59684, 39232, 14136,
                2928,  47920, 58168, 12928, 52456, 9788,  28728, 16832, 64864,
                5596,  20272, 57000, 41628, 24884, 60296, 17340, 28064, 32956,
                2244,  41968, 35184, 21616, 60064, 11840, 6048,  34704, 52912,
                17348, 45144, 31628, 53672, 15088, 56568, 31116, 61960, 25604,
                6108,  40820, 32336, 10948, 56332, 42300, 20256, 31400, 46732,
                38960, 7160,  55036, 17460, 1952,  57108, 5956,  42200, 33620,
                56128, 14104, 44204, 25436, 3524,  62596, 8480,  46540, 33876,
                64728, 7784,
            },
            {
                25720, 36084, 4864,  23260, 59268, 27656, 15824, 42536, 56236,
                46960, 9936,  50868, 18356, 5608,  60336, 32080, 48764, 12156,
                19156, 7524,  50660, 13632, 54284, 27916, 59352, 11484, 37568,
                19832, 52852, 5268,  18012, 33084, 46736, 26284, 4004,  59636,
                30732, 48164, 55392, 4600,  53152, 8244,  19872, 62740, 36992,
                16996, 9388,  28260, 12432, 53012, 16760, 21588, 57832, 50848,
                18724, 8808,  58700, 42064, 37208, 5568,  46948, 20016, 33540,
                53336, 29520, 38040, 21548, 35160, 61948, 55004, 47964, 12224,
                35760, 51284, 14736, 63908, 8532,  1556,  38640, 10856, 46620,
                57592, 26140, 61964, 45052, 31656, 49448, 39864, 21048, 64264,
                28644, 4576,  60672, 19636, 27284, 4068,  47352, 17072, 53404,
                12684, 58160, 18472, 23316, 7752,  34088, 13488, 61304, 53928,
                11260, 24252, 43364, 30028, 37860, 46088, 12776, 26036, 60432,
                8940,  40080, 64064, 18584, 51580, 28060, 20444, 13520, 2392,
                49528, 43708,
            },
            {
                58036, 51280, 41556, 12892, 44940, 50144, 9232,  36652, 19884,
                24088, 40372, 58152, 21920, 44880, 28476, 39328, 2068,  43896,
                34176, 58692, 24248, 31372, 856,   6100,  34268, 43188, 3004,
                26416, 41044, 14204, 60056, 400,   43180, 14780, 56264, 50468,
                11464, 43712, 14320, 37352, 28556, 50264, 41804, 30704, 6792,
                57928, 51716, 45188, 36216, 61212, 30184, 3908,  11572, 61996,
                40648, 32116, 20536, 52628, 62632, 17276, 26252, 80,    56728,
                42180, 16868, 4624,  46332, 26196, 6456,  40760, 1728,  58888,
                44148, 29708, 4504,  19016, 48592, 31012, 53908, 62912, 22888,
                51172, 13348, 7396,  17456, 1244,  56040, 14972, 8612,  46892,
                24184, 13328, 37308, 41012, 49672, 23284, 39360, 8580,  29560,
                37816, 3448,  49280, 44892, 59664, 50912, 24968, 44180, 1792,
                63240, 34892, 14636, 60764, 9624,  65112, 31424, 53792, 47256,
                4628,  23368, 32480, 10852, 38396, 60560, 54888, 24588, 36800,
                30576, 11024,
            },
            {
                356,   29288, 61104, 7108,  18364, 63468, 31212, 54064, 2968,
                64904, 28952, 34760, 868,   13700, 50040, 54324, 25804, 63728,
                4368,  38072, 16124, 65436, 45236, 38832, 61528, 48544, 55932,
                8664,  62920, 35684, 49708, 21072, 64156, 38128, 8628,  28024,
                34768, 804,   25692, 64780, 17096, 3556,  24628, 47180, 14600,
                40088, 33496, 4636,  19624, 8212,  55764, 42672, 47024, 5968,
                25252, 1068,  48368, 4432,  13940, 33012, 50260, 64408, 12016,
                24236, 8528,  63060, 51408, 43416, 15684, 24124, 31988, 20992,
                9216,  53668, 39396, 34484, 43576, 26020, 13988, 35120, 3420,
                39504, 30208, 47960, 37772, 23848, 62484, 26896, 35972, 59568,
                42124, 51712, 8108,  62968, 11564, 33024, 59108, 64856, 45820,
                22380, 34556, 62636, 28908, 344,   17504, 40112, 8848,  27964,
                19412, 48096, 4680,  52012, 21700, 35480, 18700, 3140,  15604,
                29808, 52948, 58280, 648,   15752, 44752, 4800,  40956, 59644,
                53172, 20660,
            },
            {
                39236, 16332, 33068, 55540, 37564, 1468,  24896, 13192, 49096,
                7504,  16396, 52804, 42348, 62612, 8576,  16904, 20696, 55876,
                29268, 48084, 10144, 56740, 19564, 24916, 13064, 17924, 32064,
                22308, 46968, 28096, 11228, 24752, 52696, 31220, 19380, 41408,
                62704, 22360, 47280, 56604, 10188, 34940, 61796, 52808, 1160,
                22632, 26720, 63324, 48292, 24320, 37836, 14852, 34384, 29244,
                53828, 64560, 35628, 23704, 56228, 43604, 7688,  29036, 36700,
                48916, 58868, 19292, 30604, 10716, 56508, 49472, 38652, 61288,
                27148, 57296, 23716, 2672,  61780, 57860, 21044, 45500, 18124,
                5820,  65052, 55052, 9456,  52572, 41492, 5492,  18712, 54568,
                988,   33380, 21776, 57064, 5624,  43844, 14348, 844,   19608,
                55872, 42492, 6768,  13080, 36772, 64436, 30472, 52592, 58052,
                37620, 31856, 56272, 26420, 388,   50240, 43492, 24392, 62932,
                36680, 20344, 42832, 50060, 34696, 29404, 9324,  47588, 14976,
                5852,  63248,
            },
            {
                23404, 47744, 9724,  26692, 20940, 47184, 58564, 34252, 43656,
                59984, 38168, 11592, 25084, 57284, 33020, 36964, 5932,  41204,
                13040, 22468, 42248, 32980, 3288,  51256, 29740, 7124,  53304,
                15524, 58864, 2136,  39836, 56908, 7220,  2864,  59944, 15908,
                52008, 6148,  39756, 31376, 45140, 21172, 38776, 11380, 59148,
                44000, 54948, 13548, 41396, 58520, 240,   51788, 22532, 57356,
                17024, 12640, 45748, 28144, 9840,  38424, 59184, 21400, 45100,
                3832,  34488, 39896, 1044,  59936, 36264, 3480,  18320, 13312,
                6712,  46956, 16368, 11948, 49832, 6172,  9344,  51748, 59512,
                42992, 27244, 19776, 33924, 12688, 46384, 32252, 49176, 11632,
                29136, 15932, 45308, 26728, 18084, 54364, 36284, 28268, 50580,
                10088, 15544, 26760, 53056, 21396, 46976, 4024,  14912, 22524,
                6240,  12584, 42048, 16796, 39048, 59032, 8136,  56764, 40380,
                11256, 48468, 6956,  26064, 65440, 21864, 57392, 18904, 32716,
                27336, 44232,
            },
            {
                56384, 3440,  65084, 42664, 53292, 5048,  39676, 17136, 27348,
                21288, 4428,  45676, 29992, 2848,  22828, 46768, 59140, 51900,
                2604,  61424, 53748, 6784,  46920, 59000, 40768, 62388, 36668,
                42668, 5732,  33404, 45540, 13808, 35192, 48316, 44136, 25212,
                57600, 12200, 18744, 60176, 2168,  54672, 16024, 29140, 32728,
                19024, 9772,  3544,  31040, 17496, 27936, 64016, 9264,  43556,
                38116, 7532,  60200, 41152, 18432, 51228, 1920,  16296, 54852,
                13720, 26648, 53744, 22528, 14432, 28236, 62984, 42316, 50676,
                35260, 63752, 41900, 32348, 27628, 37144, 41040, 30688, 24692,
                14940, 36464, 1956,  60836, 28804, 21480, 63736, 3244,  57512,
                39084, 65228, 48572, 2804,  31164, 51336, 23676, 4896,  58600,
                33276, 48192, 60776, 39460, 56676, 11080, 33996, 42752, 49496,
                59900, 45668, 64368, 10404, 28796, 32880, 13960, 45840, 27136,
                1724,  60488, 17736, 12976, 53436, 2228,  39572, 51224, 62076,
                36004, 12404,
            },
            {
                18156, 34212, 51144, 14788, 29640, 11892, 62256, 8728,  51500,
                32288, 55528, 63300, 17876, 49400, 39612, 9748,  15200, 30648,
                34920, 25936, 17280, 37196, 27836, 21764, 10520, 592,   26260,
                50416, 19788, 64960, 29728, 17836, 61540, 21800, 9316,  32948,
                3508,  36488, 50092, 27180, 8472,  42072, 48860, 6312,  65324,
                37648, 46964, 61712, 35744, 49964, 11852, 45660, 19456, 2464,
                30904, 50160, 21340, 2900,  63512, 30568, 25540, 61988, 40848,
                32096, 9452,  65432, 43124, 47616, 32720, 8168,  54868, 22908,
                29876, 1528,  20132, 52496, 56420, 17412, 64360, 136,   53736,
                10784, 48664, 57076, 43992, 4404,  15668, 37672, 25500, 43476,
                22980, 6684,  35816, 60140, 40056, 8916,  62320, 42044, 37952,
                24876, 2196,  19260, 31580, 5312,  24632, 61188, 54208, 27144,
                18292, 2160,  24688, 53964, 48980, 61912, 5216,  20768, 34260,
                54796, 30372, 41852, 35888, 46248, 31620, 11432, 24868, 3872,
                8268,  50032,
            },
            {
                30368, 40316, 7596,  24492, 35876, 45640, 56448, 23376, 56,
                47892, 14428, 34936, 7092,  53412, 60216, 26860, 64852, 19720,
                44796, 49956, 9424,  64140, 14404, 55152, 34600, 46120, 12824,
                56720, 23496, 9940,  54696, 41496, 50744, 27576, 38796, 54928,
                46652, 29468, 64336, 14212, 22952, 33684, 57536, 24996, 2972,
                51548, 27660, 22060, 54320, 5352,  39344, 32812, 61468, 53356,
                26372, 57984, 33904, 14180, 54148, 35284, 11712, 46608, 6556,
                58480, 49784, 17744, 5892,  52472, 20044, 11980, 45200, 5444,
                59016, 38064, 9780,  44200, 3816,  22364, 13072, 46040, 33592,
                62180, 22680, 7708,  40040, 50332, 59808, 53936, 8284,  14096,
                52532, 19288, 10728, 55352, 15032, 20876, 47056, 12472, 17128,
                63848, 44152, 8644,  51068, 45552, 16704, 1276,  38136, 9420,
                32468, 36100, 40460, 19212, 3488,  22912, 37544, 48052, 16468,
                63204, 9084,  23476, 58644, 5696,  62604, 16800, 55696, 41548,
                45976, 60148,
            },
            {
                888,   20748, 57180, 61208, 2176,  18572, 31732, 41908, 64100,
                38600, 10244, 26240, 41520, 20720, 12932, 36088, 42996, 7844,
                296,   57736, 40008, 31132, 4276,  43740, 18968, 63528, 30824,
                39476, 4116,  47640, 25760, 6044,  2448,  12364, 63808, 15392,
                20108, 7328,  43796, 38232, 52064, 62600, 12112, 45548, 16768,
                41180, 8064,  14800, 43196, 25552, 59532, 7156,  23340, 41424,
                15776, 5260,  46832, 24824, 43172, 4784,  49416, 37484, 19604,
                29284, 24048, 36832, 2744,  39052, 24780, 57716, 34428, 17212,
                26116, 53496, 14876, 61644, 48384, 35552, 59256, 26704, 4652,
                38824, 17032, 30888, 25916, 10464, 18312, 31500, 45940, 34752,
                62164, 30432, 42688, 25440, 33700, 1284,  28764, 54740, 6448,
                30364, 56952, 14188, 36120, 65460, 28896, 41140, 20736, 63296,
                14164, 52256, 7252,  60972, 30836, 42848, 56192, 11792, 52360,
                3680,  38636, 50716, 19860, 26936, 43380, 37056, 29968, 22032,
                13956, 26340,
            },
            {
                62868, 43784, 10876, 46916, 38288, 54432, 6836,  12656, 28532,
                19152, 60720, 44384, 56628, 1248,  32220, 4608,  51436, 56172,
                24764, 12700, 21016, 48392, 59460, 24336, 51744, 7732,  16864,
                59848, 34028, 15352, 37500, 58476, 32368, 53052, 42856, 36,
                59036, 24760, 56424, 4936,  18176, 820,   28824, 36144, 20860,
                56572, 60832, 34080, 1640,  18508, 55548, 13396, 36528, 48884,
                10872, 65240, 38948, 8680,  58532, 17368, 22988, 56340, 564,
                44120, 12792, 61096, 55856, 27680, 63272, 41260, 408,   48700,
                65008, 40700, 31744, 23904, 29228, 6780,  40100, 50516, 20480,
                56564, 45108, 52272, 62768, 35860, 22064, 2720,  57888, 27108,
                304,   50036, 6400,  47680, 63352, 57412, 43236, 35172, 22804,
                51816, 40000, 27184, 21912, 58424, 11604, 48900, 55896, 4524,
                47336, 58928, 27468, 46092, 15732, 9384,  64200, 28364, 24328,
                44776, 32580, 13696, 168,   54384, 8592,  49324, 2812,  57892,
                52072, 37944,
            },
            {
                16744, 53708, 32624, 15456, 27224, 22180, 49548, 58656, 35416,
                53040, 5440,  30876, 23012, 48324, 62892, 17432, 28412, 47020,
                38096, 62616, 33772, 6452,  41596, 1496,  37980, 28932, 49252,
                2004,  53184, 44256, 62812, 21468, 17112, 36184, 23424, 30344,
                49080, 35020, 11236, 31792, 47452, 40836, 59988, 7272,  53180,
                31144, 11476, 48492, 64144, 29116, 50940, 46244, 740,   28148,
                56028, 31484, 19320, 52388, 28560, 62496, 32828, 9800,  64076,
                53160, 33544, 7920,  46512, 15380, 9752,  50572, 29516, 13440,
                21052, 7896,  2628,  58296, 11848, 18816, 54772, 9416,  14356,
                28380, 1700,  12444, 6132,  55740, 43112, 49676, 38608, 12376,
                17192, 59840, 36856, 22252, 11744, 17900, 50140, 3964,  61000,
                10124, 47404, 472,   5580,  42604, 33440, 7508,  25644, 30540,
                43024, 21972, 12008, 33668, 51316, 1024,  35380, 40696, 6508,
                17976, 57936, 47348, 64668, 34172, 15372, 60684, 19336, 10420,
                33376, 6644,
            },
            {
                23460, 29124, 4592,  50728, 64632, 2992,  43332, 16556, 25004,
                1832,  45988, 15644, 9296,  37320, 58348, 40908, 22080, 10152,
                15580, 29652, 52488, 17608, 27368, 54496, 11516, 61152, 22448,
                42092, 27200, 12620, 7096,  49484, 10268, 46436, 60608, 8468,
                40236, 16816, 62228, 54300, 21444, 25744, 14384, 49264, 39100,
                2496,  23468, 44804, 36864, 9396,  21264, 31832, 16836, 62260,
                22160, 3884,  35916, 41868, 1424,  13864, 45920, 39296, 26584,
                16112, 41456, 21360, 30452, 35668, 19344, 4720,  61380, 37204,
                46080, 33960, 51592, 42496, 47136, 64024, 32268, 43652, 59740,
                36880, 65320, 33032, 47780, 14984, 28280, 5072,  64656, 24232,
                41336, 54464, 4300,  29372, 40360, 8376,  26504, 38328, 15336,
                20128, 34224, 62504, 54684, 18548, 52408, 14892, 61068, 36872,
                17168, 1744,  54836, 39388, 58144, 25944, 20368, 60028, 54032,
                30064, 10396, 41212, 21060, 29248, 39684, 46516, 25532, 64248,
                42944, 48200,
            },
            {
                11816, 37168, 59536, 9292,  40472, 33896, 60496, 8388,  48248,
                57144, 39420, 65504, 50632, 25828, 13428, 6716,  34716, 54292,
                59588, 3712,  44884, 9128,  63796, 35348, 14708, 45716, 33164,
                9172,  19184, 55180, 31340, 39696, 28384, 56096, 4440,  14052,
                52624, 27356, 3864,  42896, 9036,  57448, 33932, 64564, 18764,
                27264, 58904, 16260, 4516,  40572, 60924, 57244, 42812, 8464,
                44816, 49476, 12072, 60768, 24188, 50620, 6776,  20608, 51452,
                4364,  58048, 49048, 1836,  53740, 58680, 43344, 23620, 54180,
                10908, 57092, 26536, 16008, 37948, 976,   25224, 6000,  48792,
                22112, 42196, 18076, 24384, 40540, 60696, 19692, 9572,  46684,
                32168, 20592, 14480, 52392, 59312, 46448, 65020, 31536, 53664,
                44012, 25032, 12408, 30596, 46136, 23764, 3000,  40104, 50760,
                9812,  64932, 24168, 5972,  17816, 43760, 12828, 49072, 3092,
                37192, 62040, 25664, 4096,  51896, 12248, 6024,  36336, 30336,
                1772,  56560,
            },
            {
                63388, 18600, 45352, 25300, 20208, 13812, 29944, 37804, 21536,
                11492, 27872, 19392, 34152, 3404,  52584, 46380, 61160, 1588,
                42188, 20376, 37068, 56988, 23432, 50096, 20544, 57364, 5044,
                65500, 36860, 58828, 23568, 2132,  63696, 18524, 33832, 44664,
                65152, 20620, 36368, 50908, 29660, 3032,  45368, 10968, 5700,
                42568, 50108, 32364, 54232, 26192, 12324, 3368,  38252, 34248,
                53880, 26548, 17588, 56584, 29888, 43800, 35428, 59700, 30940,
                11600, 37624, 25516, 64888, 12908, 39144, 32240, 16516, 28140,
                5752,  19544, 63004, 4388,  21824, 34912, 52528, 16788, 30584,
                3068,  53576, 8992,  58220, 1124,  30928, 52208, 35452, 56712,
                7456,  63548, 44004, 33484, 1616,  23852, 13596, 3084,  57164,
                7112,  49244, 59560, 37228, 9072,  63164, 34612, 19888, 58368,
                28308, 45036, 36192, 49612, 62336, 31412, 7756,  45788, 23244,
                14844, 49948, 7952,  44460, 56848, 23580, 59692, 54664, 15880,
                21436, 39900,
            },
            {
                52168, 34844, 416,   57692, 47332, 55136, 4016,  63088, 52420,
                32936, 42360, 7020,  55192, 43736, 29392, 18340, 24496, 31464,
                50012, 26136, 12300, 47460, 6664,  31004, 2976,  39856, 25788,
                47564, 16196, 44736, 6316,  41864, 51396, 25964, 38296, 22896,
                1368,  47076, 12812, 60352, 16292, 38448, 23680, 55468, 30172,
                61692, 13464, 21812, 7348,  45468, 51976, 19996, 24784, 14300,
                5808,  63416, 40048, 2948,  9996,  15700, 55028, 2300,  47096,
                61928, 18024, 6952,  44656, 22072, 8560,  48400, 2384,  60424,
                44084, 38560, 30456, 48680, 12808, 56160, 61952, 39744, 11528,
                61076, 26800, 34564, 50796, 37828, 11936, 44164, 16200, 27716,
                2220,  49068, 25712, 38744, 55596, 19748, 36208, 42372, 27236,
                17796, 39748, 1812,  15440, 27472, 41636, 53836, 12592, 47160,
                7080,  32188, 14340, 3472,  27644, 56552, 38668, 64520, 33372,
                55840, 19052, 34772, 60976, 16144, 32248, 3180,  44916, 50500,
                8776,  26996,
            },
            {
                42528, 14484, 7544,  30628, 51768, 10696, 26952, 44620, 17688,
                2468,  62284, 14572, 59968, 21852, 10764, 63540, 39512, 14368,
                5672,  64924, 33136, 40824, 16352, 61752, 43804, 10548, 53460,
                30036, 436,   52200, 34836, 14252, 59844, 6880,  10912, 54964,
                31884, 58060, 7040,  25096, 41380, 63008, 19648, 47120, 35804,
                504,   39464, 56516, 65160, 33752, 29200, 62608, 47544, 59592,
                30668, 20732, 46168, 33064, 37988, 64416, 19304, 27972, 23116,
                43076, 33636, 56744, 29892, 51968, 26588, 63400, 34632, 50916,
                14496, 54876, 8984,  59820, 44964, 27908, 8384,  19576, 46648,
                56600, 14284, 45152, 20844, 62880, 6308,  55260, 22832, 61568,
                37596, 12680, 18132, 10120, 6096,  48684, 61184, 9136,  51076,
                32976, 64344, 22676, 56156, 48612, 6012,  31356, 624,   25152,
                56080, 18396, 53024, 41604, 9952,  21516, 16420, 508,   26600,
                11372, 40596, 2112,  27760, 37932, 20060, 41264, 11656, 33784,
                61456, 4512,
            },
            {
                32000, 23788, 62456, 38104, 16448, 41608, 35116, 5724,  57784,
                25732, 46992, 30424, 38216, 2124,  36012, 56992, 8104,  45040,
                53812, 17928, 58176, 1060,  28484, 55124, 37388, 18668, 59480,
                22540, 12260, 62648, 27252, 21184, 46344, 16688, 50076, 40516,
                27768, 17916, 43548, 34432, 1656,  53480, 7436,  14516, 25368,
                51872, 9988,  18380, 37296, 2164,  15896, 9536,  39548, 960,
                51464, 8948,  57684, 23304, 52952, 48012, 7672,  40508, 14084,
                9016,  53608, 184,   15344, 40784, 4832,  18520, 10324, 41240,
                21224, 552,   33224, 23252, 40840, 5240,  50384, 31692, 23504,
                36656, 4216,  7512,  29688, 17360, 26180, 47376, 4480,  42340,
                30268, 51432, 57676, 62200, 28120, 44388, 30748, 21940, 58616,
                12028, 4248,  29844, 43788, 18244, 59088, 21892, 64256, 42748,
                38120, 61584, 22740, 59516, 48068, 36540, 53904, 42816, 50484,
                30776, 63116, 47300, 53976, 9620,  51184, 65296, 28492, 48408,
                18104, 55772,
            },
            {
                46980, 58968, 44104, 21024, 2404,  64976, 23256, 14172, 50440,
                40044, 8700,  23608, 53632, 49116, 16392, 32792, 50936, 22508,
                28892, 36580, 48632, 9968,  24284, 51308, 13544, 33708, 6996,
                49460, 41008, 32404, 9580,  57428, 36704, 29620, 62960, 4692,
                60828, 9436,  48100, 56880, 30880, 11400, 49160, 33004, 58412,
                43764, 28704, 48060, 24272, 42624, 50056, 55096, 18872, 35584,
                27176, 42436, 16544, 5088,  12548, 25820, 31904, 57252, 62940,
                37240, 47760, 20792, 61616, 36296, 58256, 45880, 55600, 30076,
                25660, 64340, 52892, 17760, 36524, 15188, 64748, 1508,  43376,
                52192, 63516, 40808, 59356, 49972, 39140, 33288, 13932, 59472,
                8668,  21808, 41560, 32664, 14064, 52960, 100,   16240, 40936,
                45908, 34808, 52520, 8792,  38780, 13532, 51304, 35456, 16324,
                5280,  11268, 33860, 1904,  29840, 13228, 61372, 5032,  20332,
                57404, 6632,  22928, 13852, 43436, 6344,  25036, 784,   37456,
                22152, 12556,
            },
            {
                3028,  27932, 10240, 50016, 56428, 29164, 47572, 60388, 36588,
                20192, 56316, 12524, 44040, 5208,  26568, 42736, 612,   62560,
                11340, 4240,  42960, 19968, 63208, 45156, 2368,  27320, 64428,
                36392, 17240, 54556, 5300,  49024, 2508,  42480, 12988, 21968,
                35660, 52296, 14112, 20168, 61828, 23124, 37576, 64468, 6412,
                20972, 61300, 3936,  13244, 58336, 6116,  31600, 45192, 58724,
                13364, 64900, 49988, 36064, 60652, 42976, 1048,  51116, 4564,
                29040, 25052, 10924, 49760, 31972, 13276, 23564, 37972, 6568,
                47276, 12104, 42656, 3352,  58664, 26184, 54104, 34108, 10408,
                28220, 15488, 22228, 32460, 9496,  404,   64444, 19120, 53296,
                34952, 1520,  46616, 4960,  19376, 39192, 63488, 26188, 5512,
                55188, 19284, 24912, 63024, 3676,  26700, 46028, 10068, 28580,
                49824, 44908, 26400, 51888, 7312,  46176, 24580, 34356, 9716,
                45048, 17296, 35540, 59216, 31680, 61328, 16524, 54400, 58508,
                7788,  39640,
            },
            {
                52568, 15804, 36784, 6144,  33648, 18096, 7704,  11212, 31560,
                200,   61684, 34508, 18944, 64732, 57680, 20352, 13528, 38160,
                54784, 25408, 60380, 34848, 7308,  39556, 21792, 55904, 46792,
                3728,  24992, 61072, 44376, 19080, 24180, 53456, 32884, 45972,
                292,   26280, 38348, 6092,  28588, 45404, 17544, 41784, 1828,
                15592, 32008, 53424, 34900, 63552, 26488, 21544, 11196, 3868,
                24164, 32568, 2340,  28848, 55492, 21432, 17472, 35148, 45056,
                15912, 59852, 42448, 3324,  27668, 52356, 1400,  62756, 15736,
                59052, 35708, 28028, 49552, 9376,  46896, 21504, 40280, 18248,
                58100, 2532,  46796, 54052, 13060, 56816, 44008, 24356, 48176,
                27560, 16104, 65484, 25128, 56280, 34412, 10960, 49368, 37052,
                60264, 14444, 48072, 36148, 57212, 32320, 60452, 2240,  55424,
                62652, 19852, 39516, 14900, 65480, 18556, 40780, 58560, 28576,
                38712, 52812, 25992, 1476,  39724, 21036, 46900, 34244, 44160,
                29796, 63120,
            },
            {
                33040, 20116, 61220, 26008, 40552, 52104, 43300, 54572, 27456,
                45596, 6352,  51568, 28544, 9828,  39596, 30612, 59252, 46024,
                32428, 15792, 50460, 29488, 12424, 59152, 31588, 15696, 9096,
                42400, 30544, 13704, 38992, 28472, 64116, 7484,  58684, 18688,
                56000, 64752, 42052, 49768, 2712,  52564, 9288,  55168, 27072,
                57444, 40096, 45024, 8304,  17840, 38304, 41992, 62184, 47860,
                52880, 40652, 19480, 45624, 9584,  38628, 63704, 11480, 22356,
                55340, 6376,  34548, 64708, 17668, 9308,  44420, 20340, 32476,
                50736, 4560,  19560, 61364, 31436, 13232, 56456, 6656,  61688,
                49296, 24836, 35624, 42732, 20160, 28736, 36768, 3384,  10936,
                40412, 55064, 36636, 50476, 8452,  45420, 58428, 23000, 31640,
                7948,  28292, 1128,  11452, 20576, 43016, 17132, 22968, 40996,
                31704, 4032,  57780, 35964, 54120, 31812, 1372,  50960, 15056,
                3536,  64644, 11540, 49492, 56528, 27568, 4332,  10588, 14292,
                23908, 49032,
            },
            {
                55164, 604,   45520, 57172, 13648, 2044,  21840, 37664, 64032,
                17116, 23920, 41612, 14944, 47784, 4144,  52332, 7836,  23824,
                2884,  40300, 5884,  55344, 18372, 47872, 52992, 37556, 61784,
                20504, 57804, 984,   52320, 10812, 36220, 15992, 39956, 10288,
                30512, 23560, 16156, 33824, 60668, 13148, 31368, 23880, 35724,
                49816, 11076, 22732, 29900, 51504, 212,   56864, 28600, 16988,
                37184, 7964,  61808, 14632, 49164, 6020,  27436, 58880, 48744,
                30504, 38888, 19880, 46584, 56240, 39872, 60440, 25244, 41572,
                10112, 56872, 23896, 39412, 44176, 956,   37004, 29588, 45032,
                11940, 31056, 8000,  63744, 5468,  51032, 15540, 58232, 62212,
                6288,  30392, 12232, 22220, 2696,  29292, 17548, 3748,  42128,
                64804, 44944, 54492, 39720, 50044, 5824,  53488, 36980, 8308,
                13136, 25040, 47048, 9392,  22384, 43892, 12004, 62556, 21196,
                46556, 30620, 42024, 18828, 8696,  36904, 53216, 63936, 40828,
                18528, 5856,
            },
            {
                38280, 42192, 11396, 23524, 63456, 31308, 58376, 14664, 4140,
                48500, 32748, 60224, 37280, 55580, 25268, 35560, 17292, 61924,
                48252, 21284, 65156, 44068, 26428, 116,   5284,  24108, 11884,
                50544, 34676, 47724, 22008, 59916, 31584, 2056,  50808, 43648,
                48276, 3588,  8512,  54580, 21520, 39352, 58960, 43496, 6920,
                63176, 19556, 4280,  55988, 47060, 15068, 9712,  33764, 4776,
                54804, 22548, 30320, 57468, 24616, 52728, 32520, 40776, 2148,
                9092,  51400, 12284, 24200, 4164,  29008, 35016, 53556, 2824,
                46108, 30200, 15272, 7172,  63460, 17572, 52452, 23128, 4056,
                38240, 55456, 16808, 58908, 25596, 39564, 31840, 21276, 44828,
                49356, 18864, 57452, 43752, 61144, 39972, 54216, 12972, 52236,
                21012, 16080, 31000, 24240, 62508, 14544, 29668, 64696, 48216,
                52184, 59696, 17312, 2528,  29012, 56912, 6004,  26872, 34980,
                54880, 7036,  23556, 61896, 45452, 15248, 30284, 49924, 3168,
                58896, 27228,
            },
            {
                65124, 7684,  30068, 47804, 35736, 4924,  49584, 25616, 40204,
                10744, 57004, 8172,  1580,  21916, 63200, 44572, 11064, 28712,
                56356, 13904, 33332, 9896,  36196, 41632, 63780, 28612, 44872,
                7056,  17144, 26924, 6404,  45028, 56336, 24936, 62128, 27728,
                13372, 37148, 62672, 44400, 27548, 47716, 4972,  18300, 14460,
                46360, 38592, 61232, 25724, 35632, 65036, 24068, 49932, 59316,
                12852, 42208, 748,   34312, 43820, 3756,  19216, 13996, 44284,
                25848, 62136, 58316, 32344, 15560, 48988, 7192,  13512, 18704,
                65060, 36356, 54860, 51200, 33228, 25836, 58592, 14400, 65424,
                19868, 50364, 41084, 1964,  47548, 12048, 53080, 8636,  33912,
                26148, 632,   38476, 32844, 15132, 25536, 63028, 34116, 47440,
                38224, 4900,  57964, 8928,  33984, 46304, 26212, 256,   19204,
                34624, 27944, 41764, 63816, 50300, 36848, 40064, 48048, 16852,
                60316, 37892, 13680, 33132, 5772,  57596, 19768, 25196, 34840,
                46804, 13288,
            },
            {
                21408, 51220, 16048, 53760, 18860, 10040, 44692, 62248, 20216,
                52604, 29812, 18268, 46172, 12816, 31916, 3188,  51096, 42304,
                37652, 1548,  52220, 22620, 57632, 16400, 51456, 32532, 59540,
                39208, 54724, 65276, 41228, 13268, 19004, 38472, 4556,  21336,
                58264, 52680, 32356, 18912, 10888, 644,   65200, 34648, 54328,
                28516, 1668,  32804, 12520, 42716, 5384,  38976, 44696, 19116,
                26908, 47032, 64284, 17664, 11724, 60052, 36720, 65308, 54068,
                17288, 36288, 720,   42680, 54268, 63336, 22432, 39112, 57740,
                26668, 11896, 1912,  20944, 40612, 10044, 45956, 34700, 42380,
                27592, 9476,  33656, 22108, 30088, 62772, 4572,  41876, 59584,
                16492, 63856, 52172, 9468,  5080,  49184, 19476, 884,   9848,
                27032, 61812, 41372, 18740, 3768,  55328, 10560, 39120, 56592,
                44532, 4872,  12184, 33632, 19780, 14588, 61388, 23912, 10084,
                112,   44488, 53432, 26364, 50996, 40260, 1768,  61108, 42692,
                9464,  31080,
            },
            {
                36456, 60532, 26128, 6300,  39704, 55648, 28400, 33312, 860,
                36268, 43392, 65312, 27204, 54192, 40636, 19744, 59320, 26000,
                8568,  18620, 30228, 61384, 46508, 7452,  13180, 19164, 2092,
                22816, 10328, 30800, 3876,  34288, 48896, 54392, 8252,  34992,
                17148, 41420, 6444,  24560, 56836, 38188, 30436, 51736, 24748,
                9012,  58132, 50208, 17508, 53720, 20968, 30508, 2020,  61640,
                32648, 10012, 50452, 39436, 54932, 28404, 47472, 23092, 7060,
                29568, 46336, 21056, 5832,  37644, 10776, 45584, 31304, 50252,
                6368,  43360, 48196, 60808, 28784, 3580,  49624, 7492,  53764,
                172,   57856, 13432, 60576, 36260, 17764, 56016, 23624, 37592,
                12864, 46368, 20788, 28548, 59196, 44924, 36708, 29736, 56220,
                22724, 12384, 48656, 51840, 36560, 60680, 42828, 22612, 15788,
                7588,  61756, 23672, 46056, 55836, 8476,  3644,  52268, 32788,
                29548, 58528, 20508, 3988,  63236, 35984, 12272, 22572, 54244,
                17220, 57264,
            },
            {
                45172, 4436,  33444, 43424, 61572, 22212, 15448, 7916,  59084,
                13120, 22744, 50000, 6256,  35308, 61248, 14808, 5780,  34652,
                57348, 45340, 39712, 3904,  25384, 35128, 40884, 55464, 47368,
                36556, 43876, 15112, 51696, 61764, 28988, 11168, 45864, 64660,
                29864, 1564,  50352, 60320, 15320, 49276, 20804, 12120, 60984,
                40900, 21904, 44452, 6576,  60464, 10496, 56640, 15904, 52280,
                6988,  36924, 25492, 5376,  21160, 8720,  41956, 2912,  34352,
                57584, 13188, 50852, 28276, 59400, 25512, 16412, 1612,  61592,
                34164, 23016, 37936, 16928, 55684, 12956, 62804, 18100, 31752,
                24720, 46480, 52036, 42644, 7808,  45256, 28088, 2620,  50964,
                30048, 6600,  54976, 40704, 23276, 11796, 51644, 6924,  59948,
                44612, 33392, 1752,  28164, 20976, 13592, 32184, 50664, 58580,
                30192, 37384, 53856, 920,   30772, 26248, 44184, 57420, 41148,
                18332, 49292, 11628, 42532, 28928, 16732, 48464, 32284, 6888,
                50304, 1056,
            },
            {
                40036, 19356, 13656, 57564, 3020,  37452, 51604, 47292, 40676,
                55092, 3500,  16244, 39260, 9852,  46836, 29996, 22004, 49328,
                64260, 15536, 54160, 12000, 50052, 20920, 62036, 27440, 4588,
                63168, 25580, 57156, 20672, 540,   23644, 16308, 42728, 56524,
                25624, 12192, 46316, 35648, 28368, 42520, 7580,  45448, 2568,
                36868, 15172, 30896, 26624, 39948, 34692, 47752, 28240, 41384,
                22216, 55616, 16428, 58764, 31456, 51780, 15028, 61004, 49808,
                9576,  39568, 62668, 18164, 33336, 8216,  52888, 42216, 19524,
                54480, 14620, 8524,  31944, 45072, 25404, 36544, 21932, 39820,
                62000, 16388, 29256, 3496,  19984, 49308, 14492, 65040, 10308,
                43408, 35224, 62468, 3548,  33576, 17324, 65148, 41620, 14668,
                37888, 53980, 16820, 64528, 45616, 6824,  25592, 1984,  46876,
                9972,  20540, 40756, 48940, 17540, 63532, 35364, 20840, 13472,
                65136, 7608,  34520, 23320, 55396, 9300,  44060, 64596, 37900,
                28540, 24684,
            },
            {
                10752, 64180, 49188, 27812, 31664, 12024, 64004, 18448, 25424,
                29480, 61044, 32952, 24452, 57248, 2388,  53272, 43588, 11276,
                24304, 5116,  28204, 32308, 58472, 928,   10492, 31184, 52048,
                17808, 7200,  35176, 46348, 40284, 59340, 32616, 5960,  51120,
                38880, 22176, 63508, 9504,  3172,  54184, 62824, 17216, 33612,
                48536, 55864, 64552, 852,   51380, 23500, 63412, 3736,  13572,
                46020, 62528, 1856,  43624, 35900, 63220, 26004, 19652, 31776,
                44100, 23652, 3888,  55436, 47596, 39952, 64312, 29444, 5296,
                27424, 58492, 49372, 63896, 5844,  41912, 59404, 2572,  54828,
                10720, 5272,  38520, 63484, 25956, 55100, 39644, 32232, 59120,
                19108, 25204, 15048, 47948, 58064, 1888,  26300, 31644, 20024,
                5068,  25044, 9180,  30644, 56724, 37616, 62024, 17728, 34408,
                65172, 26812, 14304, 60620, 11136, 39304, 4716,  47252, 28188,
                2268,  38800, 60948, 46640, 980,   58200, 25884, 4148,  14740,
                59812, 55480,
            },
            {
                34528, 23416, 53084, 8076,  45916, 24396, 1760,  34316, 6648,
                45388, 10828, 52820, 42768, 18748, 26836, 63096, 33152, 568,
                41160, 36932, 19584, 62844, 43644, 38000, 48716, 14872, 41980,
                58820, 12056, 49752, 27852, 8752,  53164, 36996, 20336, 2736,
                15120, 58488, 33348, 19448, 39784, 23428, 31928, 26480, 58856,
                5132,  20120, 9684,  46472, 14192, 18708, 7928,  37336, 59396,
                33572, 10596, 29460, 48760, 4448,  12616, 40608, 220,   54260,
                15948, 27240, 36128, 12360, 1080,  21272, 13644, 35960, 47108,
                11144, 39560, 24364, 848,   19920, 29476, 50712, 14904, 47932,
                43524, 35692, 22404, 57084, 11808, 34628, 1484,  22604, 46576,
                5476,  53004, 9160,  27984, 37788, 53712, 45968, 10028, 50588,
                62864, 47636, 59096, 40192, 3720,  10916, 52744, 41292, 55008,
                3984,  44852, 6588,  52712, 32472, 22992, 50780, 9340,  55256,
                24700, 51904, 14928, 30352, 36692, 18424, 52560, 41304, 21040,
                7864,  47728,
            },
            {
                42008, 2040,  15856, 38656, 20236, 55216, 42272, 60444, 50648,
                37872, 20496, 64120, 4848,  48880, 12568, 38148, 16676, 59872,
                56004, 47820, 52552, 7852,  17056, 23924, 56488, 5532,  22056,
                33104, 38964, 2280,  18596, 64496, 13688, 26460, 62332, 47580,
                31008, 7884,  53280, 44556, 14028, 57184, 4076,  51444, 41484,
                12732, 29064, 53780, 37992, 33124, 57656, 44312, 54088, 24732,
                19656, 40124, 53208, 17516, 23232, 56372, 47004, 37128, 59744,
                5152,  64788, 51972, 44912, 58628, 24808, 50312, 56672, 2660,
                60656, 43900, 35264, 52836, 57360, 11780, 33892, 8204,  27160,
                19196, 60220, 50116, 30492, 41712, 8900,  52284, 16840, 56368,
                36352, 40488, 60916, 20392, 43156, 13200, 22520, 57008, 34924,
                524,   42964, 21388, 34128, 15744, 23868, 48320, 28512, 12844,
                22764, 57660, 36224, 28264, 1680,  58924, 15864, 43608, 60268,
                33096, 42308, 21084, 6384,  49792, 12800, 62108, 34112, 45804,
                30808, 17528,
            },
            {
                6212,  28872, 58772, 62308, 33548, 48420, 9868,  16700, 57520,
                13468, 28,    28072, 35028, 59240, 7468,  51596, 20856, 29360,
                9508,  14152, 34400, 3196,  27232, 46408, 35344, 64716, 28692,
                45132, 61312, 55760, 31068, 42160, 4828,  45000, 10572, 55404,
                41296, 27196, 32,    49028, 61660, 36788, 10412, 21412, 63636,
                35524, 24460, 42908, 62360, 5528,  27296, 11504, 30640, 452,
                50324, 7296,  26344, 63992, 34016, 8156,  28356, 10800, 48824,
                22200, 41052, 8796,  29080, 32492, 6752,  42640, 17888, 31200,
                21772, 13876, 7344,  17612, 46212, 37732, 23096, 64832, 56464,
                32592, 6772,  14016, 3080,  18016, 48336, 61876, 27736, 12668,
                30388, 684,   49424, 32296, 4492,  63828, 6660,  29084, 39448,
                17844, 27020, 12304, 51008, 55288, 44500, 19728, 7304,  32856,
                39532, 16384, 47232, 19076, 41464, 54636, 36688, 30104, 5164,
                17388, 10392, 56360, 64060, 40328, 22648, 28156, 2904,  11248,
                51672, 63244,
            },
            {
                36676, 50132, 44116, 5308,  13208, 29984, 4220,  27384, 22496,
                31712, 41516, 47856, 15660, 22960, 30812, 41776, 5364,  46288,
                25740, 65144, 21856, 42504, 60788, 12996, 8888,  53788, 3660,
                19640, 9492,  14520, 24388, 51048, 58372, 33948, 18064, 22704,
                60556, 35788, 16944, 24852, 6516,  28288, 47624, 15952, 45112,
                7680,  50432, 2060,  16228, 21976, 41060, 48564, 65528, 16096,
                43224, 56556, 13260, 38356, 45928, 20704, 58800, 16964, 30356,
                34368, 13456, 19828, 60920, 16600, 38944, 62876, 9544,  26352,
                54040, 64572, 32564, 27888, 61972, 3408,  52160, 40924, 1360,
                46580, 25164, 53868, 64332, 44744, 24268, 37092, 4268,  43924,
                64500, 23740, 10932, 57528, 17672, 36048, 48732, 15492, 52504,
                58224, 8148,  61604, 29976, 2332,  64176, 36076, 59772, 1072,
                63320, 51540, 9120,  61336, 24448, 7188,  12468, 20460, 62348,
                38032, 45600, 26392, 3752,  32560, 8800,  54956, 60152, 38884,
                25256, 56756,
            },
            {
                22412, 9184,  18624, 25904, 56416, 40568, 65176, 36952, 44520,
                54472, 61836, 8944,  38968, 56312, 45212, 61256, 35436, 53820,
                2668,  31636, 38764, 56928, 51252, 31016, 40356, 16140, 49232,
                26208, 52676, 36136, 47216, 21384, 2924,  38364, 29764, 5796,
                49864, 9760,  65532, 43164, 32508, 55532, 40484, 1288,  31036,
                58516, 18832, 56032, 31760, 60628, 52164, 3900,  20532, 35476,
                61052, 32180, 5704,  59980, 1344,  42684, 52432, 5996,  62176,
                43848, 56888, 1960,  46404, 54584, 4104,  34436, 51560, 45220,
                37468, 5000,  49884, 40428, 9820,  15520, 44516, 30328, 17396,
                12096, 39468, 34964, 20848, 57760, 31936, 7648,  59388, 20584,
                50980, 15556, 45196, 26656, 54348, 41812, 23872, 60364, 2872,
                32084, 45796, 38016, 5744,  41848, 25948, 9692,  49568, 43088,
                25136, 30212, 3812,  34156, 44132, 64880, 48224, 26932, 50120,
                392,   52772, 14000, 59348, 47644, 43544, 16464, 48876, 19820,
                164,   14208,
            },
            {
                41192, 32304, 53680, 35780, 892,   49644, 19952, 52368, 7616,
                3152,  25336, 50504, 19704, 4128,  11128, 24772, 14684, 18916,
                58788, 49420, 11624, 5588,  19516, 1188,  23520, 60368, 37516,
                42956, 272,   63976, 7448,  40556, 12172, 62908, 52908, 45932,
                14316, 39520, 20700, 51272, 11900, 18000, 60072, 52372, 25260,
                38904, 10684, 46844, 36416, 13152, 8424,  39040, 25656, 46688,
                9944,  22716, 27896, 49436, 15292, 31236, 25104, 39528, 3060,
                50984, 23112, 37396, 49648, 25612, 13172, 22104, 59168, 15404,
                104,   19568, 58636, 22640, 55604, 26528, 5728,  21452, 58208,
                62964, 49720, 28212, 10056, 328,   40208, 14828, 54308, 41356,
                34196, 7012,  37876, 61508, 2556,  8608,  33696, 11084, 43308,
                19904, 13784, 23236, 56588, 18180, 14988, 31824, 57176, 13580,
                19328, 55236, 11320, 50420, 14680, 31324, 3036,  40464, 57556,
                23168, 34988, 29580, 18612, 37580, 24960, 4980,  35500, 29908,
                44984, 60732,
            },
            {
                11872, 64476, 46232, 15568, 61180, 23928, 10312, 14792, 59704,
                17984, 33820, 12836, 64592, 28808, 52800, 816,   63440, 40720,
                8068,  44212, 16444, 29112, 63872, 48188, 34024, 56324, 11348,
                30220, 57400, 33252, 17676, 29108, 55240, 16128, 26228, 1192,
                59276, 30496, 5256,  57424, 2504,  23076, 33920, 6832,  13564,
                64936, 27564, 3008,  22932, 44668, 28332, 59136, 55368, 14224,
                4640,  40944, 53620, 19244, 35152, 65004, 11300, 54984, 18576,
                27332, 7824,  31128, 10824, 64536, 43216, 28488, 7540,  40736,
                48240, 29680, 12380, 34148, 46772, 60784, 36824, 53380, 33384,
                8436,  4124,  43852, 16652, 52060, 47732, 29448, 10700, 26292,
                2152,  56772, 30936, 19688, 52032, 28800, 47396, 62536, 26316,
                54764, 49092, 63108, 33844, 47456, 52840, 38208, 22368, 5108,
                45808, 35748, 59012, 39128, 21988, 54360, 18376, 33452, 11752,
                7048,  43228, 62684, 2776,  50776, 12344, 65356, 56132, 8092,
                52540, 27024,
            },
            {
                49316, 3144,  21112, 7320,  42988, 31880, 46912, 35004, 40128,
                29768, 46132, 57764, 37060, 42840, 32436, 48504, 37660, 27808,
                33460, 23164, 55244, 37256, 25524, 44976, 7912,  18516, 4644,
                22232, 13576, 44592, 25132, 60080, 48620, 8936,  36852, 43440,
                24044, 54824, 35008, 27588, 45872, 37740, 62316, 48228, 20520,
                43488, 34480, 49920, 61840, 54072, 17692, 1504,  32960, 50532,
                63688, 37112, 45016, 3852,  56204, 7496,  47660, 36780, 13920,
                63492, 59204, 41376, 17452, 35616, 2956,  56792, 32032, 62552,
                24472, 54376, 43464, 7944,  2016,  18776, 49180, 13540, 42676,
                24424, 37832, 55228, 61500, 23412, 35516, 64036, 18496, 45780,
                62392, 22812, 48120, 12428, 40172, 15136, 21424, 38708, 6224,
                36440, 3632,  10048, 28040, 668,   58976, 7720,  29276, 40752,
                62424, 26780, 16924, 692,   28536, 60312, 9204,  46908, 63892,
                16172, 54536, 21020, 9460,  58484, 31924, 22116, 40932, 14908,
                33836, 5148,
            },
            {
                55304, 30680, 58964, 38876, 28272, 57628, 2808,  63132, 21656,
                55116, 6572,  2420,  22556, 15964, 7100,  21164, 56540, 12092,
                51372, 1676,  60124, 9672,  41872, 14220, 54596, 62924, 40980,
                47476, 61168, 5544,  51156, 2212,  42392, 21632, 31888, 64396,
                17556, 12128, 48152, 63396, 15148, 42184, 10036, 29872, 54812,
                4928,  57524, 15564, 6544,  30304, 38084, 11440, 42760, 20068,
                29696, 17044, 58156, 12540, 23800, 41440, 29328, 21816, 45060,
                33908, 48076, 1116,  53624, 21128, 50616, 45984, 18840, 10748,
                35680, 60040, 16572, 38264, 65428, 31216, 10348, 28284, 936,
                59056, 19760, 31652, 12692, 6536,  58324, 3572,  38932, 51612,
                8312,  16656, 43072, 4156,  65260, 58272, 96,    55960, 17724,
                30852, 51684, 40920, 20680, 44952, 12552, 65352, 50176, 2784,
                53148, 8540,  48264, 42404, 6500,  44772, 35940, 51836, 24984,
                30588, 37920, 46624, 26816, 42020, 53596, 1336,  46272, 62280,
                18792, 39976,
            },
            {
                36204, 16728, 25216, 10880, 51924, 13972, 6468,  26508, 50080,
                11456, 41996, 27596, 49016, 53772, 60888, 46356, 17256, 5656,
                62836, 45344, 19964, 34824, 52140, 3200,  31980, 24172, 28508,
                52764, 35240, 20276, 38784, 11204, 34776, 58072, 3960,  50892,
                8128,  39196, 3336,  21880, 7228,  52608, 26132, 560,   17400,
                39692, 24768, 10172, 41820, 21216, 64600, 48412, 24244, 60592,
                8828,  744,   26264, 33180, 50100, 61224, 1608,  58340, 5192,
                9680,  24408, 15072, 28124, 61140, 5504,  38184, 14352, 52300,
                2484,  5816,  27688, 51128, 24900, 41644, 54832, 62072, 45568,
                15384, 50992, 26960, 46644, 41892, 20712, 27800, 13584, 32736,
                58728, 36712, 54944, 25816, 35024, 31688, 44524, 49580, 12348,
                64048, 24060, 15552, 61156, 35848, 25372, 42952, 20040, 34036,
                14236, 24144, 32656, 64456, 55620, 23700, 13888, 4324,  40876,
                1196,  60092, 13036, 5264,  35944, 16056, 29156, 10140, 25896,
                58384, 22976,
            },
            {
                808,   45408, 63700, 48272, 19136, 36388, 53920, 44896, 16452,
                37708, 63864, 58640, 13768, 35536, 9880,  26632, 31120, 42176,
                24540, 13640, 30024, 5084,  61872, 17068, 49208, 38272, 1516,
                9784,  16224, 55448, 30724, 63268, 14956, 27604, 46852, 20004,
                54016, 32892, 43732, 60024, 18480, 32280, 59040, 36032, 50844,
                60636, 31564, 47308, 52832, 34056, 3620,  56788, 5980,  36096,
                52484, 47144, 63064, 39616, 9796,  19804, 15876, 53120, 30984,
                51364, 39068, 57384, 44380, 33028, 9248,  26408, 63604, 23384,
                40320, 44868, 20300, 57216, 13796, 4228,  21728, 38868, 34604,
                5612,  64724, 9740,  2560,  33520, 49868, 56208, 44096, 24000,
                940,   29252, 11160, 50880, 7044,  23100, 9512,  27372, 42116,
                57504, 7880,  46504, 55012, 6304,  31948, 16424, 9728,  55324,
                39544, 59720, 18696, 11988, 37636, 20056, 61744, 27880, 57796,
                18444, 50592, 33364, 61816, 49300, 19908, 57104, 38052, 6420,
                47904, 12856,
            },
            {
                53508, 8968,  32388, 5516,  41300, 61916, 22756, 33320, 1364,
                20456, 8588,  32636, 24416, 1100,  40336, 65368, 3236,  59200,
                38172, 49512, 57712, 40120, 27444, 21732, 58952, 12292, 44044,
                63920, 26884, 7736,  46292, 23956, 52588, 5876,  40660, 13092,
                62152, 24908, 28960, 49876, 38292, 55780, 12228, 45092, 22420,
                13672, 2336,  63360, 17940, 26424, 13088, 44544, 16212, 31808,
                41708, 21708, 14564, 28392, 54856, 44596, 37612, 26044, 42428,
                62124, 18160, 4028,  11804, 55268, 40968, 49128, 30592, 56256,
                33596, 61800, 9768,  47008, 32764, 59724, 8032,  17740, 48440,
                23208, 40732, 55628, 37408, 62500, 16952, 6764,  10208, 60848,
                48972, 20440, 63076, 41788, 18368, 59468, 53852, 35620, 20404,
                1860,  33304, 28936, 39280, 2432,  57816, 47568, 61548, 27988,
                45272, 1552,  50740, 29712, 3552,  52768, 32228, 48424, 11056,
                42660, 22024, 8368,  24484, 44308, 3056,  64320, 51536, 32988,
                41400, 61624,
            },
            {
                26596, 43304, 21680, 56388, 1988,  28732, 9668,  56896, 61080,
                29432, 51944, 45768, 17644, 56400, 44588, 20328, 52324, 33852,
                8320,  17772, 54012, 11316, 46560, 8012,  33500, 55924, 18812,
                50288, 40024, 59248, 896,   43052, 17988, 61064, 30400, 37324,
                332,   57636, 9540,  14044, 2724,  24072, 5112,  62944, 8484,
                43232, 27824, 37316, 7520,  59104, 38716, 54236, 28100, 10092,
                58984, 3160,  56608, 6472,  34868, 2764,  64548, 7940,  12676,
                21176, 35460, 30124, 65256, 22876, 19232, 948,   16536, 7280,
                12624, 50248, 29224, 520,   36272, 43120, 52000, 27528, 11436,
                53560, 29624, 19404, 14060, 25288, 52536, 30140, 41000, 34512,
                15180, 45952, 5316,  38736, 13636, 47428, 3224,  16076, 38324,
                60276, 48508, 13676, 19128, 52264, 23204, 37608, 4356,  21256,
                35948, 7424,  57024, 41664, 46532, 8684,  16416, 38984, 6268,
                53960, 29416, 56536, 14560, 39776, 31396, 12052, 22312, 17488,
                4292,  29772,
            },
            {
                37584, 60524, 16360, 34888, 50404, 14916, 43680, 38664, 48036,
                13112, 40880, 4932,  60664, 28444, 37464, 12180, 15312, 47384,
                22480, 280,   25940, 36548, 64944, 42100, 2372,  29576, 36428,
                5356,  21540, 32700, 12968, 35980, 57068, 10020, 49640, 22792,
                45824, 16920, 35820, 65128, 46460, 41092, 30716, 49084, 19060,
                34576, 55976, 51648, 45884, 21860, 160,   62732, 19160, 48828,
                24872, 37764, 45544, 18108, 49628, 22708, 32076, 48044, 59552,
                724,   50048, 46276, 7444,  37252, 52760, 59372, 46752, 43144,
                21996, 37928, 17912, 55000, 15348, 23816, 64196, 1980,  57864,
                33192, 4544,  60796, 44832, 556,   47112, 22052, 65028, 2964,
                54000, 26940, 32156, 60536, 24440, 29512, 64484, 45096, 5484,
                51300, 25620, 10512, 43840, 64424, 8572,  30248, 51040, 12212,
                63664, 25932, 16064, 22328, 62848, 35080, 58552, 25200, 65184,
                35904, 45936, 1948,  63100, 52784, 6592,  27532, 43100, 55788,
                49444, 14272,
            },
            {
                2452,  51264, 11116, 40108, 65292, 25628, 6688,  18280, 23780,
                3492,  55048, 35040, 21820, 8164,  50424, 4196,  55232, 29076,
                63372, 44972, 31896, 4772,  15256, 23776, 48300, 62420, 14344,
                25240, 53408, 47408, 65412, 27292, 20996, 3516,  33796, 56288,
                7284,  41772, 52180, 27744, 20708, 54128, 15608, 37520, 58016,
                24400, 5344,  15500, 11012, 30480, 50380, 41036, 33624, 4876,
                64420, 12280, 30412, 61868, 40392, 11368, 56944, 17444, 27712,
                40896, 54840, 15808, 25856, 42572, 13324, 34616, 27556, 64236,
                5076,  57976, 25560, 62664, 8884,  49044, 30168, 38360, 46256,
                16276, 42872, 8716,  35784, 57016, 11180, 39204, 57920, 19104,
                37604, 9256,  56680, 1644,  52232, 34552, 10992, 55720, 31364,
                22244, 62472, 35288, 55388, 26964, 15128, 42256, 58836, 18056,
                40668, 32516, 48956, 10664, 27500, 88,    44320, 20728, 3808,
                12832, 17532, 33992, 25800, 19352, 47448, 60284, 36552, 8748,
                63624, 20260,
            },
            {
                57040, 31572, 23576, 4044,  30256, 45856, 53312, 58668, 31724,
                64000, 26464, 15672, 47228, 63264, 32120, 25100, 58604, 35468,
                10332, 40772, 56576, 60396, 51232, 19972, 54280, 10292, 44948,
                58620, 3216,  16972, 9040,  40916, 51692, 45164, 15076, 26788,
                63144, 19592, 2012,  33512, 8192,  61348, 26636, 10644, 1020,
                64584, 32192, 41904, 61832, 36236, 23992, 8196,  14628, 53240,
                43396, 20744, 51848, 7804,  25548, 53464, 3968,  45644, 33952,
                23804, 10416, 58188, 2292,  61172, 31452, 3604,  10576, 39660,
                51524, 31288, 3344,  44568, 39916, 19440, 6252,  13420, 21608,
                63476, 26168, 50540, 20564, 28016, 32396, 4816,  13728, 24528,
                51180, 44856, 17184, 21684, 42920, 6808,  19528, 26696, 41740,
                8340,  17356, 1112,  39964, 4912,  48708, 33888, 1848,  54436,
                45996, 3208,  60332, 53644, 39860, 13392, 55292, 31528, 51064,
                40444, 58988, 49212, 9328,  38344, 57260, 15944, 264,   33644,
                25144, 46656,
            },
            {
                6608,  44076, 59676, 54668, 20008, 12508, 35548, 24,    11016,
                37508, 50684, 5948,  42396, 11312, 39444, 17372, 43272, 20952,
                3428,  49716, 18800, 11912, 30128, 34968, 39080, 6360,  28328,
                41576, 31108, 37880, 55692, 5740,  30040, 61708, 38844, 11524,
                53756, 29932, 48312, 13280, 56644, 3696,  50792, 44996, 40212,
                53100, 20628, 48704, 2688,  18240, 56624, 45416, 60480, 26904,
                36796, 2512,  34120, 16148, 59408, 29720, 37340, 14376, 63792,
                4740,  38256, 29584, 51060, 18204, 47984, 55576, 24648, 19536,
                14804, 47512, 34932, 12152, 53652, 32612, 61296, 55956, 35532,
                2684,  39324, 58768, 14596, 54600, 63184, 43856, 49240, 7032,
                29972, 63836, 12388, 36280, 49700, 58068, 39288, 61276, 13344,
                53140, 46084, 58124, 30988, 19668, 61196, 22600, 10344, 28752,
                24256, 14672, 36628, 5604,  30380, 18140, 47560, 7528,  61760,
                27652, 22248, 42864, 4704,  32104, 11460, 23328, 45636, 54408,
                39732, 10488,
            },
            {
                28216, 35340, 17584, 47952, 8360,  62104, 41532, 49404, 21460,
                44508, 57932, 19612, 29596, 59528, 760,   52468, 7116,  61268,
                14280, 37728, 24592, 7256,  43932, 1708,  57876, 17580, 22288,
                61420, 11696, 49284, 24652, 59492, 18852, 1460,  23584, 42924,
                4596,  36504, 60236, 39348, 43528, 22656, 35336, 18152, 29364,
                13896, 8008,  25796, 54456, 28940, 12392, 4372,  31996, 17404,
                9560,  55680, 47536, 42040, 440,   21124, 43572, 8876,  52004,
                59972, 20096, 44456, 35300, 22544, 8456,  62404, 36828, 53348,
                60384, 8344,  21068, 27432, 58780, 1320,  42704, 25376, 47268,
                10976, 30624, 6928,  48080, 3240,  17704, 36364, 26604, 58740,
                40584, 33456, 4460,  60988, 28084, 15252, 704,   48804, 33676,
                3592,  37440, 24076, 50392, 12632, 44088, 56776, 38576, 65140,
                7564,  52240, 43560, 21604, 57704, 64148, 24100, 37444, 15772,
                10372, 964,   55108, 61036, 51844, 28304, 65068, 5040,  30408,
                18088, 62344,
            },
            {
                41824, 14508, 1028,  38544, 27504, 33080, 15812, 59608, 28760,
                8676,  14268, 34300, 54428, 22400, 48388, 27608, 33284, 46636,
                53808, 28460, 63708, 55712, 47748, 26244, 64480, 33196, 45736,
                51164, 156,   20516, 33588, 14068, 46588, 35000, 50812, 58020,
                17244, 21532, 10000, 25496, 15872, 31312, 63496, 6052,  59832,
                47204, 34184, 60932, 43716, 37760, 65400, 51488, 39788, 57844,
                22588, 62780, 27976, 13452, 65048, 49120, 57316, 18004, 31232,
                47480, 12116, 5840,  63368, 14276, 40692, 4656,  28832, 44196,
                796,   41236, 65192, 49872, 16312, 23136, 9156,  51100, 17048,
                54312, 61524, 23980, 33968, 41584, 22436, 8688,  53468, 16088,
                2540,  22596, 54200, 46816, 8640,  31872, 23080, 44224, 18732,
                63756, 29028, 9688,  62676, 6636,  35856, 16552, 47156, 31784,
                19308, 61132, 33396, 9648,  49736, 1840,  41724, 33628, 52920,
                45208, 30224, 36316, 14744, 20668, 40196, 43312, 50408, 12748,
                21900, 48932,
            },
            {
                53188, 25400, 64512, 51168, 21948, 55824, 5620,  24988, 39620,
                53200, 1880,  64848, 5028,  38036, 13320, 62948, 9928,  23420,
                40468, 1332,  32684, 16672, 41004, 13872, 4336,  53224, 8176,
                15620, 35896, 63228, 43592, 54116, 10388, 63716, 7792,  31124,
                45740, 64720, 51824, 5136,  57940, 48992, 11728, 54372, 21768,
                1996,  38928, 10484, 16336, 6120,  20468, 25396, 836,   49036,
                44172, 5056,  38616, 32400, 23668, 5676,  35196, 27092, 2524,
                23072, 39384, 53316, 27920, 57136, 45992, 33264, 17180, 12352,
                23936, 30708, 36256, 5776,  45508, 38508, 29608, 34336, 5292,
                40864, 19532, 45084, 12012, 51700, 60036, 30884, 46372, 62296,
                10988, 42632, 26084, 18188, 37756, 65372, 56432, 10336, 25824,
                55276, 15632, 42444, 20808, 53328, 27860, 352,   6220,  50804,
                12316, 39492, 25968, 16908, 45076, 28020, 12140, 4120,  59448,
                18536, 64404, 24864, 47088, 7892,  2156,  17188, 56468, 35812,
                60048, 2896,
            },
            {
                33008, 57492, 11652, 7128,  42804, 46324, 2704,  63660, 18252,
                47436, 31508, 41760, 26220, 45088, 18772, 35752, 57272, 16044,
                6328,  60068, 21352, 36116, 8952,  49912, 23516, 38556, 27840,
                56308, 40844, 6736,  26268, 3744,  28572, 38316, 15708, 25852,
                2352,  33640, 41096, 28844, 45300, 380,   37792, 42000, 27508,
                57308, 51732, 23360, 31020, 58504, 46800, 34380, 14748, 29572,
                11616, 19424, 51628, 8444,  54704, 39908, 11748, 62224, 42220,
                56408, 33156, 16756, 208,   24816, 10672, 49952, 59504, 54592,
                48108, 18436, 56300, 11224, 52748, 60428, 13016, 64208, 57344,
                27952, 424,   36888, 64836, 28500, 2184,  13436, 38212, 20472,
                34344, 50828, 59892, 13300, 6264,  51664, 40728, 4304,  36348,
                49892, 5800,  47584, 39096, 32996, 60460, 55632, 24904, 42060,
                58844, 2120,  54936, 5008,  36020, 61440, 56060, 23004, 48028,
                6320,  41216, 12312, 33864, 58008, 62800, 32392, 26312, 6084,
                38972, 9352,
            },
            {
                45576, 19876, 30792, 36056, 60184, 13340, 30404, 35472, 10660,
                57488, 23104, 16828, 51384, 8184,  55476, 2104,  42288, 30292,
                50652, 45380, 11660, 52644, 58652, 29832, 61740, 19600, 11120,
                60120, 30664, 17944, 47700, 58624, 21744, 42164, 56844, 49268,
                12644, 55416, 8460,  19492, 61724, 24276, 17660, 8724,  32328,
                14324, 44432, 62340, 3504,  50152, 42136, 8708,  54156, 60980,
                35716, 58860, 26080, 16532, 60088, 45620, 19948, 49996, 15168,
                6872,  60944, 48984, 43176, 37640, 64868, 21324, 2768,  38484,
                6676,  62620, 26492, 1872,  32452, 19096, 3076,  43980, 21344,
                49348, 15000, 53392, 8060,  18492, 47828, 57236, 5172,  44028,
                28360, 72,    56036, 30656, 45428, 20080, 27552, 60648, 12980,
                32264, 59484, 2396,  24556, 13924, 8812,  18216, 37188, 28656,
                21404, 46068, 63840, 29868, 51632, 14936, 20224, 32004, 38492,
                27132, 54540, 2980,  51276, 38604, 23964, 10252, 47956, 53932,
                16032, 23248,
            },
            {
                50104, 5360,  40804, 17272, 26472, 53900, 19980, 51668, 44140,
                6760,  37140, 60020, 11732, 62052, 31984, 49172, 20388, 65092,
                37480, 25564, 5180,  43116, 17948, 708,   34220, 41936, 48840,
                2428,  24084, 53376, 13024, 32904, 51412, 932,   19272, 35872,
                60604, 22852, 37436, 14200, 53984, 34680, 51044, 64992, 46536,
                19580, 7000,  26276, 35696, 12636, 22644, 63852, 17856, 40748,
                2608,  47200, 42792, 3540,  34232, 28900, 1136,  53132, 36424,
                26056, 9480,  29828, 55372, 18972, 7676,  31648, 27584, 42312,
                34420, 13912, 44636, 22156, 41436, 48720, 55080, 24944, 7300,
                39124, 32152, 58092, 25520, 41184, 33116, 23212, 53796, 64244,
                16960, 9872,  38988, 23552, 3376,  33760, 16052, 53416, 22472,
                44876, 18956, 29776, 64896, 51864, 43508, 48960, 63420, 3648,
                14168, 34740, 10296, 23480, 38792, 6668,  43816, 50280, 9196,
                16460, 61596, 21752, 29052, 15376, 44760, 19772, 61544, 29436,
                42328, 63680,
            },
            {
                27112, 61016, 55636, 2276,  48576, 8824,  38896, 61892, 25456,
                15008, 49520, 3264,  28176, 39416, 23636, 4732,  27388, 8624,
                13508, 56072, 32176, 63308, 27100, 46984, 56616, 6428,  15116,
                44348, 37032, 63772, 8412,  39488, 61228, 11352, 44716, 6088,
                27904, 46788, 2984,  42776, 30780, 6372,  12484, 58672, 3088,
                36644, 54652, 41232, 17200, 52988, 1716,  32132, 27380, 6812,
                23744, 30796, 53564, 13264, 63408, 22788, 10480, 31940, 64324,
                46236, 21080, 13828, 4548,  34976, 51260, 58212, 15584, 61360,
                9596,  50608, 57700, 37416, 63776, 28228, 14624, 36492, 10456,
                62032, 45888, 4272,  11416, 62852, 36412, 15408, 7652,  25676,
                48452, 35356, 61320, 49744, 41544, 62720, 47372, 880,   38400,
                8912,  56936, 40960, 7248,  34524, 22584, 11392, 30696, 57924,
                43092, 53628, 18784, 48092, 58056, 11576, 65304, 384,   57320,
                35636, 42564, 10892, 48772, 55556, 640,   36240, 4660,  13000,
                34372, 1632,
            },
            {
                10736, 37248, 14332, 43368, 64232, 23452, 4792,  32644, 1084,
                29352, 55044, 19620, 35236, 46704, 14864, 44052, 59416, 53136,
                18360, 47800, 22880, 3640,  39740, 12804, 22028, 65252, 32352,
                51240, 20816, 4112,  46148, 16880, 29732, 24228, 64212, 31512,
                52828, 17440, 63284, 58252, 48484, 26852, 40364, 22624, 29532,
                49500, 10772, 63480, 28812, 58104, 38452, 45704, 56828, 49928,
                62748, 10200, 21004, 37164, 48728, 56708, 44604, 17064, 6152,
                38564, 57908, 41252, 61980, 47240, 12296, 44700, 24008, 53108,
                20036, 29912, 5020,  16684, 8896,  4092,  31672, 58708, 52132,
                26972, 16892, 22652, 49896, 43404, 1264,  51188, 31248, 40296,
                58752, 4892,  21092, 14712, 8236,  56696, 11288, 28944, 63928,
                50688, 26256, 16300, 54700, 1088,  46692, 39736, 5500,  16712,
                49696, 7868,  32092, 1584,  41460, 25704, 34588, 29496, 23696,
                52716, 3940,  31696, 63524, 7712,  40624, 59500, 51940, 46748,
                57720, 18532,
            },
            {
                31740, 52416, 20448, 29060, 34060, 56392, 16708, 46432, 58676,
                39728, 43244, 63172, 7368,  57880, 51764, 10780, 33140, 40672,
                1432,  61472, 35036, 15432, 54560, 8508,  37368, 52968, 25688,
                9824,  57416, 34856, 26796, 54604, 5496,  37532, 49788, 14724,
                9068,  39036, 25228, 10956, 20644, 1560,  55812, 43756, 15532,
                59784, 32940, 21328, 5188,  47980, 9052,  19856, 13100, 36660,
                16188, 55652, 40152, 7604,  26680, 4060,  41344, 59620, 24952,
                1916,  51832, 32908, 23040, 2400,  26944, 39552, 6236,  776,
                47852, 33344, 25120, 54220, 60912, 39588, 47208, 18540, 43000,
                1720,  34960, 55564, 30084, 20348, 59364, 10616, 56512, 19672,
                12580, 46712, 28640, 53860, 25412, 36664, 18392, 43692, 33356,
                5220,  12336, 36172, 60112, 19752, 27700, 62244, 56212, 25276,
                36736, 62644, 27256, 60292, 15280, 54256, 17884, 45252, 13248,
                59716, 19664, 46492, 26012, 17832, 32992, 27928, 21600, 25048,
                8028,  39892,
            },
            {
                48428, 23060, 6496,  45712, 10276, 40500, 12948, 50872, 21716,
                8504,  12044, 32972, 22580, 236,   17732, 63824, 37088, 21380,
                29260, 7472,  42616, 50932, 59628, 31060, 46044, 16948, 1992,
                29556, 40656, 14124, 60168, 42076, 21584, 58192, 2876,  43056,
                55944, 33048, 5324,  51512, 36028, 62092, 17932, 37840, 6740,
                25364, 516,   44712, 14880, 34620, 60520, 25772, 4100,  44080,
                32708, 48,    61280, 18404, 51608, 14676, 35808, 29936, 54424,
                19064, 10180, 28324, 16380, 54036, 64276, 18452, 56144, 36944,
                63504, 41680, 11688, 45496, 20884, 51476, 12240, 23736, 6356,
                65420, 40324, 13556, 61520, 6552,  26320, 46100, 37236, 2676,
                42584, 33200, 63436, 1764,  31548, 51356, 3856,  24132, 58416,
                20888, 42108, 48648, 31204, 9904,  52292, 13776, 33504, 20144,
                3072,  40168, 21704, 45536, 5128,  37524, 62116, 2792,  49040,
                40016, 5616,  37912, 13600, 57168, 43568, 9892,  15192, 65264,
                44356, 55200,
            },
            {
                13156, 63544, 58648, 348,   53372, 61100, 27900, 36292, 65520,
                18036, 48740, 26716, 52944, 42296, 30376, 25452, 3788,  49800,
                57812, 45924, 11208, 27776, 18768, 24660, 5288,  60856, 43428,
                62752, 19012, 49508, 672,   10452, 47196, 33704, 19504, 26928,
                60340, 21956, 41364, 15108, 46012, 31388, 9360,  48008, 64400,
                52720, 40052, 56440, 24404, 51492, 42264, 30176, 64824, 52464,
                22036, 47052, 29044, 43268, 33044, 65348, 21440, 12160, 46780,
                63124, 42412, 49452, 37288, 8656,  43444, 31416, 10808, 28128,
                15268, 59452, 3012,  35968, 27116, 52,    34096, 60204, 28740,
                53472, 48448, 9268,  33260, 38952, 16408, 29388, 64648, 24140,
                51616, 17252, 10284, 44480, 57544, 40544, 61576, 15084, 46364,
                54416, 1784,  62744, 23336, 43744, 4260,  37528, 45684, 9056,
                54960, 13056, 52312, 10192, 49388, 31480, 22240, 9356,  27404,
                33328, 64532, 23152, 51480, 2456,  62384, 49380, 38244, 1224,
                28664, 5024,
            },
            {
                35544, 25784, 38404, 32136, 18952, 24880, 7076,  2840,  31272,
                54468, 3976,  60304, 13404, 37848, 47148, 9076,  54656, 13068,
                16804, 23944, 38300, 64508, 1180,  41748, 49772, 35492, 12208,
                54460, 6584,  23888, 31668, 64872, 15396, 53704, 7728,  12888,
                47768, 44,    65508, 28480, 57624, 4380,  54164, 27324, 13352,
                35256, 18868, 8020,  61452, 11924, 1500,  17000, 38900, 10592,
                5320,  25504, 59212, 11428, 5668,  57388, 1016,  50604, 7724,
                34240, 14412, 60372, 5064,  57288, 21612, 34896, 50976, 46076,
                7356,  22452, 49984, 17416, 64504, 42452, 56472, 15712, 37312,
                21668, 3600,  18984, 44124, 54008, 49620, 5248,  13460, 54620,
                7576,  60548, 38772, 22824, 19488, 12912, 7636,  30784, 37940,
                9448,  28352, 14264, 39336, 56852, 17512, 60900, 50396, 28468,
                63884, 42352, 34476, 28956, 19172, 56476, 42744, 52144, 58000,
                16980, 11044, 54848, 29100, 35664, 5848,  19224, 53548, 33692,
                59288, 17424,
            },
        },
        {
            {
                8332,  63024, 5524,  47152, 19072, 65460, 5928,  55916, 14308,
                41636, 52524, 32544, 54656, 2260,  19028, 24216, 61744, 1052,
                56936, 8596,  39020, 26468, 59964, 32972, 39820, 23388, 28448,
                35252, 39180, 51092, 26732, 22344, 47892, 5844,  41448, 54212,
                4636,  63560, 29344, 39612, 61552, 26976, 31720, 59432, 37504,
                49580, 17668, 33420, 42304, 21464, 59148, 46032, 23268, 8916,
                15924, 1660,  59960, 32480, 21240, 58808, 50840, 61724, 23412,
                712,   57328, 18160, 7560,  3928,  28196, 15636, 41744, 31188,
                55112, 35016, 12668, 43092, 2516,  5636,  19284, 13068, 42528,
                62652, 20212, 27812, 6572,  58356, 24932, 4388,  45076, 55248,
                21728, 38708, 47340, 42456, 31488, 57180, 7512,  38880, 14768,
                62432, 25952, 32148, 19920, 57628, 23616, 50076, 13816, 2448,
                20328, 8928,  46660, 22468, 61868, 31004, 7600,  35476, 28352,
                49656, 11920, 61764, 57648, 48188, 17848, 10240, 21764, 64968,
                31992, 58092,
            },
            {
                51088, 39888, 13332, 29048, 2124,  37800, 12176, 31436, 23684,
                49508, 9184,  16392, 40160, 5752,  48456, 37760, 52388, 15160,
                28680, 34884, 53436, 1908,  55452, 9508,  64556, 52116, 56080,
                11140, 60560, 31260, 65144, 9544,  36396, 17900, 60980, 20536,
                25580, 37704, 7576,  56284, 160,   19960, 51460, 4656,  22504,
                30052, 7040,  24608, 54948, 3832,  11120, 64084, 32692, 43684,
                55552, 19200, 25596, 46612, 54308, 2100,  37784, 18980, 48384,
                12964, 36892, 65152, 50376, 33008, 47444, 63360, 36548, 10436,
                17452, 60812, 24864, 53056, 32756, 63136, 58160, 50820, 6968,
                36404, 52280, 46704, 33196, 18104, 50384, 35756, 15960, 27256,
                64436, 8040,  34956, 60372, 22552, 10548, 65056, 18864, 29356,
                55884, 45860, 11496, 41196, 7028,  16024, 43584, 33660, 30192,
                41692, 57160, 39316, 27544, 54828, 43180, 13632, 40852, 23968,
                55660, 43980, 37680, 21168, 64,    36056, 52016, 56552, 5308,
                24724, 15772,
            },
            {
                832,   55184, 32992, 59884, 42440, 57300, 21112, 44280, 62804,
                28120, 4184,  60832, 22148, 30060, 64928, 43772, 11528, 21536,
                47164, 64176, 13508, 18012, 42080, 21084, 15136, 4252,  44276,
                19184, 5112,  15896, 45752, 1416,  57600, 27396, 51152, 10332,
                32980, 48116, 52692, 16656, 42044, 35876, 47208, 15116, 40804,
                62780, 44844, 60312, 38168, 15728, 28044, 39812, 5868,  49280,
                36180, 28960, 41420, 7956,  35148, 16732, 10600, 31320, 44260,
                5444,  28756, 45180, 21328, 38844, 12688, 1356,  26096, 51468,
                3148,  45204, 7108,  39856, 10764, 47032, 30256, 39256, 17380,
                10980, 24624, 2868,  63532, 38920, 10124, 61504, 42052, 5364,
                52496, 14416, 50036, 17608, 4300,  46428, 25472, 49236, 5128,
                36452, 3124,  52180, 61436, 28652, 46624, 64408, 10776, 59912,
                51968, 16508, 5872,  34376, 412,   20116, 50532, 63340, 2500,
                9528,  30400, 5584,  26552, 54496, 45560, 28080, 13656, 43488,
                48536, 37864,
            },
            {
                27800, 17576, 21640, 49080, 25124, 7360,  34188, 53856, 1692,
                39240, 35308, 45848, 58128, 13924, 26748, 6756,  56136, 30840,
                40672, 5512,  50836, 28000, 31840, 46492, 37580, 25792, 30280,
                57796, 41120, 24360, 54080, 33524, 42640, 14104, 38976, 1884,
                58040, 13728, 23200, 30976, 11272, 65276, 24472, 8360,  56524,
                10952, 19644, 1184,  50844, 31152, 57420, 52604, 20144, 13488,
                61748, 4092,  53132, 14360, 63900, 27752, 42348, 55896, 62536,
                26088, 54592, 15976, 10020, 59796, 55444, 44012, 21692, 57220,
                38236, 19588, 28404, 50292, 16216, 21668, 25768, 1044,  61432,
                32324, 55080, 42204, 13576, 56448, 22836, 52,    48248, 33368,
                23896, 40748, 1880,  29872, 55356, 41156, 33216, 59160, 12696,
                42908, 17704, 24064, 34980, 56,    18856, 38628, 5104,  23080,
                36476, 48400, 12268, 62776, 45924, 10420, 26636, 56940, 33788,
                46340, 18604, 64200, 41572, 12208, 7412,  39136, 30772, 60428,
                9280,  34356,
            },
            {
                64372, 45896, 3568,  10792, 61988, 15712, 50920, 9988,  17548,
                55664, 20012, 11288, 33448, 51032, 41104, 17688, 34248, 3480,
                59140, 24548, 10444, 61192, 57252, 3284,  50148, 61980, 8564,
                48148, 35864, 12280, 50264, 6500,  21596, 62616, 46700, 29060,
                64392, 43704, 5228,  46312, 59136, 3320,  53920, 28424, 33540,
                48936, 36208, 26824, 12828, 45364, 8100,  35096, 188,   24844,
                56560, 10672, 44788, 23140, 49920, 58000, 3436,  20432, 8400,
                40076, 49696, 3116,  31536, 24604, 5484,  15268, 62064, 31880,
                47732, 13908, 65376, 57872, 36488, 6064,  53556, 44312, 57096,
                28596, 48376, 21500, 7828,  31052, 45476, 28264, 19336, 60488,
                9328,  56252, 63180, 26368, 36768, 15100, 692,   21448, 51352,
                31292, 63748, 7908,  48344, 58508, 31576, 50432, 55580, 26792,
                1984,  19128, 30904, 24428, 59172, 36968, 16168, 6272,  22116,
                60416, 14412, 51036, 32596, 59084, 17368, 62708, 4060,  19112,
                22596, 57580,
            },
            {
                7060,  41968, 53140, 37012, 28364, 39788, 46596, 30492, 26312,
                61384, 48368, 5400,  24988, 968,   54452, 9992,  61416, 49912,
                19428, 37948, 44100, 35200, 7328,  23904, 12684, 33568, 17224,
                796,   63652, 28020, 19600, 59980, 30604, 8464,  24160, 4188,
                35128, 18020, 55724, 27156, 34136, 19120, 39216, 44020, 16712,
                58992, 5684,  41688, 63468, 23088, 17304, 62248, 41220, 47764,
                30928, 38400, 33752, 18492, 6416,  38772, 47120, 33284, 13940,
                22456, 35092, 63584, 42280, 53428, 35648, 28928, 40332, 8396,
                4372,  23852, 33612, 1536,  41812, 62380, 13288, 35108, 8744,
                15232, 4024,  37508, 51792, 16428, 65328, 54092, 12664, 37048,
                30640, 45116, 20740, 10904, 51960, 60944, 44232, 8852,  39024,
                26616, 57088, 14600, 40056, 11844, 21604, 8268,  15296, 45460,
                63752, 57508, 42956, 52996, 3592,  40392, 48596, 29184, 52052,
                35672, 39500, 3728,  21664, 47104, 25064, 34912, 50804, 54100,
                40440, 12532,
            },
            {
                24552, 30424, 14076, 58388, 4632,  19916, 64104, 6292,  42704,
                13440, 37584, 57204, 43844, 62728, 20764, 36712, 44792, 27248,
                14320, 124,   48420, 15612, 20408, 52988, 43572, 58444, 39352,
                22168, 55324, 44716, 3056,  37368, 56128, 49248, 16212, 54408,
                40144, 21412, 8816,  37880, 49408, 13172, 7132,  62016, 2336,
                22284, 53376, 32308, 55840, 2888,  49652, 26388, 58548, 15308,
                21404, 63480, 2172,  59496, 30172, 11816, 24412, 52440, 60144,
                1580,  57032, 18376, 7712,  46600, 19528, 49012, 11380, 52448,
                60212, 43044, 54240, 9664,  27008, 18804, 48104, 23664, 40456,
                19712, 62880, 59552, 26512, 34820, 3200,  39744, 7020,  58524,
                16572, 4012,  38992, 47556, 6896,  18412, 24488, 62636, 54228,
                2496,  20180, 44704, 27896, 54676, 63092, 42412, 35576, 29648,
                10168, 38960, 7176,  14456, 21148, 32292, 65012, 43240, 916,
                11280, 58044, 27612, 8924,  55908, 1156,  42712, 14928, 27392,
                2532,  47788,
            },
            {
                56260, 18276, 33952, 44168, 23900, 54964, 2280,  36116, 52616,
                22804, 8120,  28456, 15776, 31648, 48132, 24008, 7524,  57640,
                31472, 53804, 62620, 28744, 41028, 65368, 2020,  27504, 51880,
                14100, 32100, 9848,  40560, 17540, 11812, 34036, 42296, 61484,
                12300, 51340, 59868, 964,   63808, 23456, 51856, 31652, 25992,
                47320, 14556, 9112,  19172, 39124, 33824, 11160, 44240, 4740,
                8840,  51608, 45944, 26156, 55340, 42604, 64992, 16980, 37488,
                44824, 25256, 29688, 13300, 59392, 156,   64512, 26464, 17464,
                36240, 29864, 20812, 49292, 56376, 31396, 3232,  58540, 52208,
                33036, 45012, 6208,  42680, 10900, 50788, 24656, 44136, 49520,
                27076, 53368, 13188, 32036, 56548, 35412, 29272, 46796, 11684,
                33348, 50528, 5576,  37228, 1712,  24964, 52172, 3772,  59424,
                22388, 49960, 33900, 27644, 56120, 8712,  12796, 18328, 54876,
                24076, 44848, 16444, 49692, 37100, 65308, 10884, 46036, 31864,
                61932, 36260,
            },
            {
                50136, 284,   62864, 9012,  49224, 31596, 12692, 60048, 17144,
                32648, 65052, 53564, 4064,  39192, 12844, 2976,  64592, 40064,
                10948, 22688, 6020,  56200, 32460, 8932,  36704, 19124, 4980,
                47168, 61496, 25368, 53220, 64904, 45612, 432,   26960, 6620,
                31464, 24888, 42796, 15532, 29404, 45156, 57044, 41480, 11352,
                37764, 64724, 28720, 46436, 61040, 6632,  30260, 53448, 36388,
                28392, 40664, 17104, 13380, 35436, 624,   9240,  31680, 5056,
                48560, 11016, 40780, 52252, 37788, 23600, 33156, 56700, 2948,
                46012, 12604, 5876,  15808, 37288, 45680, 11508, 64808, 27704,
                500,   12912, 23316, 57020, 18612, 32084, 62192, 21400, 1168,
                34464, 59444, 23204, 64860, 2388,  42420, 14424, 4464,  36924,
                17176, 65300, 30272, 59024, 15988, 46132, 32556, 18364, 13320,
                54480, 928,   17616, 46860, 59976, 50868, 25372, 38072, 60716,
                7292,  32112, 62308, 40492, 29616, 19252, 23884, 5552,  57892,
                20600, 9636,
            },
            {
                28656, 40096, 22052, 59724, 15544, 41152, 45420, 25544, 50316,
                616,   46876, 41660, 18868, 59500, 50712, 29016, 16876, 52072,
                34108, 42428, 46884, 18044, 12320, 25056, 49588, 60096, 30348,
                42596, 7096,  35208, 3936,  29324, 22824, 58512, 19756, 53416,
                37180, 3352,  48492, 33472, 9828,  20056, 4420,  17152, 59672,
                51136, 520,   43564, 24372, 51588, 13880, 22096, 65292, 18772,
                55960, 60680, 4180,  48624, 22112, 61104, 50692, 19864, 27184,
                54028, 61524, 33708, 3892,  16288, 44560, 7996,  41944, 22124,
                62928, 50996, 58448, 40108, 61652, 25164, 5136,  17504, 38516,
                47428, 55440, 29608, 48616, 39044, 7652,  14444, 55012, 41804,
                19040, 8236,  40300, 27956, 50940, 21040, 57744, 48956, 60340,
                22616, 42036, 10264, 47660, 20804, 7348,  62260, 37852, 41432,
                26108, 44128, 64264, 36188, 2828,  41592, 5060,  46144, 28472,
                20340, 53364, 2728,  13028, 6648,  54472, 33684, 51400, 38996,
                16464, 44656,
            },
            {
                13252, 52112, 5504,  37420, 27696, 6924,  57028, 9596,  39520,
                21120, 11892, 26448, 34068, 9284,  56148, 36032, 45568, 20164,
                1956,  26540, 59588, 3896,  39148, 45956, 54340, 15548, 10492,
                23580, 57572, 19948, 48396, 14852, 38388, 50112, 10256, 64076,
                46368, 17700, 57368, 62640, 40224, 54520, 36428, 28116, 8016,
                21288, 34612, 16296, 4264,  36760, 58320, 40420, 47188, 2432,
                24660, 10584, 32020, 38028, 54356, 28776, 44268, 39804, 58412,
                14624, 6812,  21300, 63048, 27340, 50344, 54604, 14404, 38804,
                27884, 32728, 696,   22556, 8336,  34432, 42756, 50620, 21380,
                8108,  35416, 15580, 64348, 2352,  58416, 36628, 28632, 11332,
                63520, 48632, 15936, 45484, 10660, 6556,  31696, 26504, 39832,
                8088,  56084, 25644, 34164, 53652, 12092, 28660, 49304, 4700,
                61372, 31440, 11668, 23396, 16036, 30620, 62912, 34252, 14256,
                48212, 42008, 35356, 25632, 59616, 46684, 14176, 61024, 26480,
                4216,  63736,
            },
            {
                31636, 25212, 46928, 17940, 53736, 34688, 20036, 29992, 63104,
                35748, 57808, 5036,  61540, 22208, 42516, 25392, 6448,  55252,
                61952, 14720, 50156, 35236, 63884, 21016, 3180,  33744, 63364,
                37872, 50892, 11664, 41412, 56432, 7912,  31088, 43780, 14192,
                28540, 22440, 7480,  12820, 25924, 1792,  61408, 49584, 32868,
                56276, 41164, 60556, 53968, 9924,  26968, 5516,  16056, 34416,
                50016, 43448, 20216, 64088, 5680,  15648, 3348,  10704, 35992,
                23796, 42396, 46364, 56532, 10508, 31284, 20176, 5188,  60744,
                9352,  17836, 43884, 55556, 29092, 52980, 14436, 57168, 31056,
                60232, 41912, 4568,  20836, 44724, 25072, 51076, 46332, 5448,
                30844, 36076, 1752,  60628, 38060, 53908, 62916, 18144, 628,
                52404, 14976, 3224,  60064, 39460, 43216, 23308, 57704, 9480,
                20064, 52140, 6036,  39492, 55180, 9668,  22032, 56856, 1388,
                10288, 64476, 17680, 50924, 43492, 21192, 1996,  41392, 8716,
                35300, 54848,
            },
            {
                2468,  58028, 11024, 65244, 1444,  49540, 60356, 3332,  15084,
                52216, 44004, 30956, 48728, 14740, 1036,  63860, 11480, 30464,
                37456, 9036,  22316, 29136, 7324,  56996, 41552, 26356, 44832,
                1172,  16976, 32388, 62884, 26044, 2068,  59652, 34548, 4596,
                40628, 54164, 36148, 30932, 51208, 46568, 23332, 13740, 44556,
                4992,  12468, 30120, 18572, 48644, 32236, 42928, 62452, 12644,
                29836, 58184, 7436,  46464, 25488, 34184, 57148, 47128, 64444,
                1460,  29828, 17976, 36756, 3060,  40484, 57992, 46204, 34632,
                48976, 25356, 64632, 11780, 47748, 19548, 63460, 1756,  24588,
                10856, 49804, 26944, 52972, 31532, 9536,  17316, 60984, 22956,
                56212, 51884, 25860, 19556, 33808, 24232, 12924, 46252, 35056,
                29916, 44360, 49860, 18260, 6532,  64108, 376,   33512, 16144,
                36596, 47844, 28348, 58464, 45224, 49764, 17808, 51696, 36980,
                58592, 22848, 4880,  31244, 9100,  37536, 56340, 29924, 48780,
                23064, 42544,
            },
            {
                20076, 38856, 33616, 43472, 22480, 12852, 42064, 47356, 6124,
                23520, 17624, 8636,  53896, 38116, 46344, 32848, 16452, 47960,
                41720, 58148, 53104, 43940, 17140, 30908, 13568, 8688,  52808,
                59200, 28892, 4824,  46128, 20840, 51376, 16848, 24096, 62352,
                49324, 792,   60464, 43416, 16520, 5800,  38784, 19116, 63424,
                26304, 47656, 23112, 64332, 1376,  57332, 20676, 51888, 38432,
                360,   53916, 17448, 40320, 12184, 51820, 22288, 18280, 32168,
                51404, 60672, 8860,  49108, 25228, 65272, 12584, 29336, 1804,
                53756, 14748, 37060, 3428,  33224, 6788,  40340, 36604, 45892,
                17208, 61904, 37976, 12612, 62888, 35184, 904,   39496, 14040,
                42656, 4452,  12508, 57112, 43752, 3864,  40876, 59104, 9468,
                64400, 23688, 37160, 32012, 27468, 14312, 51084, 46404, 55304,
                25060, 60656, 14464, 1160,  32996, 25272, 6936,  43080, 30028,
                26100, 39836, 54944, 12516, 62168, 27380, 18324, 64764, 11424,
                16932, 60304,
            },
            {
                50428, 15020, 8060,  28084, 56584, 32032, 25180, 37120, 55708,
                28732, 40916, 62716, 3916,  28100, 20440, 59952, 51896, 23552,
                4984,  27520, 12752, 468,   49104, 60964, 36900, 47760, 19304,
                22764, 40080, 54632, 35924, 9424,  42788, 37656, 55564, 11180,
                19080, 27028, 9676,  21068, 65388, 34116, 57756, 9996,  31340,
                52160, 2620,  39860, 34828, 45312, 11600, 25084, 7856,  59736,
                27476, 22696, 36252, 62664, 30308, 59892, 42732, 7144,  25908,
                13012, 39084, 55384, 15256, 34236, 52228, 18564, 23592, 38280,
                58584, 21056, 42120, 52164, 26428, 60996, 13284, 28536, 56188,
                2944,  32688, 7184,  19060, 54920, 43756, 28164, 57472, 20372,
                32940, 46820, 29368, 63412, 7740,  17036, 50308, 27528, 20968,
                56832, 5292,  11868, 61536, 54760, 21900, 38600, 29548, 3240,
                40728, 7980,  43540, 19576, 65236, 38296, 12324, 60872, 3356,
                15552, 45056, 33844, 47692, 320,   51992, 33492, 3472,  53704,
                45792, 6144,
            },
            {
                26288, 63204, 46316, 52688, 5244,  61548, 18388, 10076, 64204,
                49820, 12592, 35052, 25008, 57156, 9436,  40312, 2288,  35076,
                63220, 18688, 38740, 33120, 54556, 23860, 2612,  65068, 33248,
                6044,  12560, 61224, 15460, 27928, 64828, 5340,  30000, 46864,
                33488, 58632, 39220, 52760, 2804,  25320, 54844, 42720, 7008,
                36324, 58276, 9208,  15288, 28184, 55088, 36852, 17196, 46824,
                42104, 14328, 4204,  49684, 9152,  1020,  37672, 55064, 48152,
                4120,  44064, 27784, 21620, 584,   42900, 6016,  61144, 44752,
                8632,  4592,  31180, 46664, 57388, 22912, 44164, 8772,  52596,
                21852, 58112, 46696, 40116, 3524,  23548, 48012, 6540,  64568,
                8888,  54320, 38416, 22312, 48436, 31232, 53464, 1676,  39048,
                16176, 48032, 41620, 2340,  44948, 9748,  59608, 18840, 12700,
                63212, 22692, 34996, 52356, 4872,  27992, 47536, 20708, 52640,
                63848, 7204,  19012, 58328, 22156, 14528, 44584, 39604, 23936,
                36020, 30724,
            },
            {
                41288, 556,   21432, 37588, 15916, 40724, 2028,  45936, 32924,
                21252, 60,    59304, 44372, 17996, 49660, 13276, 29332, 54952,
                8300,  44808, 59240, 20904, 10416, 40308, 27652, 16008, 42316,
                56256, 49640, 24220, 28,    47820, 53380, 13468, 22028, 42116,
                7192,  15472, 45140, 29772, 12172, 48112, 14936, 28724, 62044,
                16964, 21416, 49244, 62624, 3632,  41328, 50236, 5080,  64560,
                10412, 31484, 56032, 44964, 26920, 19944, 14724, 63564, 34948,
                17400, 59056, 7508,  62612, 47348, 56724, 31928, 11328, 50332,
                28168, 61960, 18380, 10056, 15660, 1380,  49916, 18176, 34408,
                42032, 14216, 24808, 29852, 59180, 16088, 11680, 52416, 36392,
                25052, 15112, 104,   58856, 10576, 36540, 61276, 13636, 34048,
                54364, 29124, 19900, 51596, 25384, 34080, 7136,  43328, 56920,
                31908, 50456, 11144, 59932, 16116, 42016, 56744, 35524, 31980,
                11216, 49488, 28144, 38088, 42192, 25936, 59476, 8272,  62028,
                13528, 57240,
            },
            {
                9888,  34504, 55164, 11704, 30152, 51308, 58208, 27016, 7288,
                38780, 16304, 52684, 31916, 6352,  36872, 64328, 43052, 21292,
                15216, 50092, 4140,  62568, 46776, 5500,  58420, 52128, 9048,
                29248, 37512, 44032, 31996, 18904, 34336, 59480, 2392,  51876,
                56460, 24004, 4160,  59944, 37008, 41032, 21820, 164,   45952,
                53848, 25156, 42424, 31776, 19368, 60524, 22988, 28992, 33908,
                52484, 19020, 60652, 23484, 33476, 52944, 40768, 28548, 10180,
                22548, 52744, 37144, 30148, 13900, 38680, 26004, 17292, 35892,
                55496, 23208, 40708, 63960, 34928, 39248, 54916, 30608, 62532,
                5148,  48876, 65192, 9956,  51012, 33932, 61512, 30508, 40996,
                18064, 50588, 42368, 27116, 18976, 45044, 24952, 6716,  43056,
                63680, 8536,  32768, 58084, 15596, 64956, 49504, 36356, 4396,
                26672, 46080, 2000,  29600, 54616, 23444, 8492,  680,   41184,
                24500, 55692, 3608,  62440, 10116, 5212,  50208, 17924, 28736,
                4500,  52288,
            },
            {
                18812, 61252, 25048, 48020, 64612, 20448, 35740, 13864, 54804,
                61068, 42352, 10860, 46984, 23864, 53656, 3628,  26172, 57700,
                32272, 37164, 24420, 30040, 17320, 35812, 13052, 45028, 20396,
                4228,  63952, 10876, 57564, 7644,  41112, 24836, 37916, 17952,
                64280, 35780, 49572, 19516, 8436,  63896, 50952, 32884, 10840,
                39016, 4688,  12676, 51584, 7448,  35676, 13840, 1588,  57624,
                43976, 5904,  39236, 2724,  11952, 58024, 45868, 2232,  60864,
                31448, 41828, 3260,  19380, 51160, 9004,  64052, 41700, 3456,
                13640, 33372, 49552, 6196,  27564, 59364, 4152,  25120, 10976,
                38720, 27872, 396,   36776, 42912, 21224, 4888,  45636, 2312,
                62332, 56300, 6324,  33252, 65488, 3476,  56980, 21720, 46992,
                26012, 3344,  37840, 12168, 40756, 1268,  28192, 21648, 54124,
                16824, 39844, 20272, 37076, 48512, 13592, 63260, 18788, 58892,
                14764, 43732, 36016, 16544, 30288, 56632, 32564, 47124, 37912,
                22072, 44324,
            },
            {
                31772, 15196, 2980,  42180, 8684,  4668,  24028, 43432, 50612,
                30344, 22668, 4400,  28856, 62152, 15032, 34212, 10664, 46560,
                860,   53348, 12188, 42740, 56576, 49560, 31684, 25248, 59468,
                34736, 17272, 22212, 50556, 27220, 63156, 11640, 48612, 31324,
                10200, 27536, 13856, 32396, 55276, 3692,  26940, 58792, 18792,
                61048, 34484, 57288, 26752, 44904, 56068, 47976, 40296, 21104,
                25780, 15336, 35224, 48944, 65484, 17516, 6328,  24676, 49320,
                56220, 12440, 46712, 25000, 58308, 4524,  22260, 53784, 47600,
                58760, 244,   44380, 53580, 12228, 19996, 47476, 42960, 16032,
                56516, 20908, 53904, 18236, 7872,  57072, 26980, 13328, 23128,
                34428, 28972, 12416, 47900, 38676, 15792, 30936, 51096, 11320,
                17524, 60204, 49000, 22904, 53260, 18120, 47192, 61228, 13268,
                8096,  64416, 57932, 5704,  33024, 44652, 26888, 38496, 29756,
                51204, 8996,  22564, 46676, 53268, 20648, 11756, 63280, 1680,
                58536, 49536,
            },
            {
                5820,  37064, 56884, 27712, 33764, 60000, 47144, 10476, 2440,
                18052, 65516, 37736, 56044, 19824, 39408, 48996, 61300, 18228,
                41068, 64984, 27188, 8732,  2076,  22624, 63476, 39600, 1464,
                46920, 54696, 39128, 2528,  45192, 16336, 55216, 4704,  44404,
                604,   40864, 61984, 45716, 22968, 16552, 43952, 36508, 5832,
                48240, 29552, 1120,  65212, 16600, 10804, 30552, 62992, 7892,
                50768, 61396, 28332, 9040,  41608, 30176, 37020, 20492, 34012,
                7928,  16332, 64880, 35688, 40968, 32588, 45288, 27860, 7880,
                20848, 30460, 16832, 24452, 37516, 32220, 64572, 8168,  35556,
                60748, 44540, 13448, 31712, 60352, 47156, 38376, 54444, 49792,
                9116,  44624, 60620, 22056, 52420, 9420,  41528, 63004, 35724,
                39880, 55468, 28340, 6072,  62816, 31164, 10316, 38512, 33740,
                51564, 42708, 24996, 10552, 61680, 2684,  52880, 6156,  47236,
                61308, 2196,  33280, 65520, 39728, 3024,  42848, 35212, 15820,
                26592, 40188,
            },
            {
                12628, 53204, 45552, 19392, 50420, 16844, 38024, 55760, 25888,
                34588, 7092,  49300, 12728, 1164,  44180, 7760,  22888, 30248,
                6224,  14516, 58972, 37940, 51908, 18964, 6628,  10780, 52652,
                28068, 13704, 30872, 62132, 5788,  36360, 29664, 60916, 21612,
                51568, 57616, 6604,  38232, 52904, 30816, 12336, 56180, 23984,
                15140, 43604, 20652, 39768, 23716, 3172,  52956, 18404, 37576,
                12368, 45452, 55656, 21504, 53276, 47292, 13564, 62768, 52000,
                44412, 27676, 880,   54200, 10220, 17768, 61364, 14828, 37792,
                63440, 40072, 60472, 8612,  56972, 50520, 1076,  22852, 51804,
                29900, 6488,  49440, 41312, 24436, 1564,  16440, 63656, 19836,
                39980, 3984,  17440, 26032, 1304,  55604, 27500, 5748,  20452,
                448,   45052, 14056, 34936, 42388, 3708,  45236, 23844, 57268,
                68,    30532, 17672, 49980, 21184, 40896, 15980, 34540, 12204,
                19728, 25500, 57452, 5668,  13444, 27044, 51432, 23516, 54108,
                9276,  64464,
            },
            {
                29824, 23612, 61948, 10888, 1868,  31288, 63496, 14168, 41272,
                58268, 45492, 32092, 59764, 27416, 54752, 33092, 58640, 50896,
                36088, 21108, 47940, 30972, 43716, 60468, 34500, 41832, 16260,
                60796, 7320,  42728, 23748, 52428, 19760, 12916, 39368, 25428,
                34232, 18396, 11484, 26348, 1440,  63044, 8144,  49872, 40892,
                63512, 9300,  54576, 50104, 37132, 33292, 43500, 27596, 58704,
                1256,  32600, 5408,  16468, 25324, 196,   57408, 39920, 4444,
                18900, 38192, 58920, 21904, 29724, 49940, 2072,  34728, 51844,
                11184, 26640, 45996, 4332,  14604, 41244, 18300, 45608, 12500,
                25900, 2396,  34200, 64284, 10516, 28944, 35772, 5236,  32176,
                57440, 51452, 37040, 43196, 58724, 34824, 13888, 48652, 60952,
                30444, 8404,  58872, 25236, 19860, 54936, 60296, 6664,  27524,
                14644, 47460, 35868, 56048, 29220, 59096, 23776, 64504, 54344,
                41436, 37324, 49200, 29232, 18936, 45660, 61520, 6656,  31236,
                46320, 20420,
            },
            {
                43008, 4136,  34528, 39928, 57888, 22992, 5944,  28780, 52984,
                9448,  20692, 15680, 24080, 41872, 10432, 16852, 2128,  25948,
                44480, 55096, 3064,  16504, 24256, 12812, 54888, 26112, 48532,
                21520, 37596, 56676, 10028, 33864, 58948, 48296, 8412,  54600,
                15152, 46552, 60448, 20864, 42056, 47244, 34052, 19796, 3072,
                27140, 32420, 13164, 5084,  61736, 56780, 6712,  14588, 47720,
                24072, 42196, 63640, 38812, 60400, 8284,  32320, 26764, 11236,
                60512, 31116, 47768, 6276,  13428, 43428, 57492, 24180, 5384,
                55680, 18860, 49240, 35872, 63216, 26936, 30752, 55616, 62000,
                40124, 58664, 16840, 22040, 51680, 55980, 43692, 48064, 11296,
                27900, 14552, 64080, 7528,  31400, 18536, 45644, 24168, 53080,
                33836, 16372, 51756, 48176, 11420, 37016, 16484, 49700, 20560,
                39380, 61084, 3744,  12548, 6736,  46116, 9108,  31276, 1232,
                45068, 10244, 15476, 53552, 59300, 33988, 10812, 41452, 57208,
                92,    60044,
            },
            {
                50012, 7852,  16152, 26812, 52336, 43228, 46608, 19264, 460,
                62792, 39544, 51652, 3564,  36444, 64268, 52788, 38884, 13208,
                62236, 9572,  33956, 64004, 6080,  46348, 29540, 4036,  65416,
                32656, 788,   49828, 17820, 45988, 26932, 1820,  61788, 42924,
                5672,  29800, 50592, 36804, 57092, 14280, 28796, 59248, 37892,
                51488, 60236, 46244, 18764, 26000, 10440, 22036, 64460, 35632,
                55052, 19676, 30044, 11652, 35068, 44952, 50536, 22200, 43456,
                54780, 14964, 24556, 51708, 63556, 38788, 20104, 46872, 31316,
                42216, 1652,  33044, 21072, 52644, 10832, 2924,  37776, 7420,
                20248, 54224, 46616, 5044,  39572, 14136, 7556,  24056, 61560,
                21216, 2268,  46588, 23284, 11544, 61940, 39052, 4640,  10460,
                43112, 22676, 38456, 65232, 1632,  28948, 32200, 41560, 64248,
                9800,  53808, 44044, 26444, 39040, 51372, 36588, 18252, 27408,
                60440, 22500, 32204, 7344,  38888, 2596,  21956, 17300, 35808,
                13784, 38244,
            },
            {
                18320, 55912, 47040, 65004, 13256, 8968,  33104, 49736, 36224,
                25020, 30628, 6032,  61392, 45880, 20168, 29204, 47268, 4644,
                19452, 41344, 28132, 49452, 39976, 57960, 36236, 9364,  19188,
                44924, 12304, 25296, 63376, 4792,  40980, 31824, 22576, 37280,
                65176, 2356,  32988, 9632,  4072,  24880, 53392, 6832,  16492,
                22252, 1848,  36192, 30720, 49164, 40640, 44860, 29364, 4208,
                9612,  49380, 2332,  52312, 56568, 18336, 14160, 64664, 1776,
                8048,  34864, 41048, 2852,  33576, 26556, 7460,  11864, 64812,
                16124, 58224, 9496,  24828, 44016, 59272, 33916, 48236, 15056,
                28496, 9796,  36212, 30832, 61100, 19244, 58480, 33416, 52456,
                42324, 30580, 55036, 35612, 50572, 3048,  54456, 28860, 57672,
                62680, 2920,  27376, 6396,  44400, 57432, 12876, 52236, 2672,
                34496, 22900, 31432, 19056, 63456, 14332, 57828, 4940,  49676,
                55976, 3580,  64160, 42936, 24708, 49832, 62472, 46848, 27772,
                51860, 24264,
            },
            {
                32364, 28600, 36072, 1056,  21196, 38536, 61220, 15860, 57592,
                12020, 43496, 56104, 14080, 33712, 8200,  23696, 59512, 32080,
                51256, 57036, 23260, 11972, 576,   21684, 15780, 50548, 56224,
                39348, 59708, 29868, 36848, 14084, 57232, 16584, 52176, 11260,
                19156, 26524, 54068, 44496, 17604, 64024, 40428, 11336, 44828,
                55952, 41768, 8588,  15212, 58552, 620,   53676, 17568, 60516,
                39432, 15280, 43176, 23092, 27488, 4000,  40816, 29464, 37244,
                47096, 61756, 19776, 9980,  53612, 16988, 60320, 50216, 36896,
                27776, 53160, 38912, 62208, 5656,  17388, 41624, 23788, 65472,
                51236, 42876, 24956, 2832,  49620, 26580, 37340, 704,   17040,
                38984, 9680,  59848, 15392, 26460, 41336, 20860, 16804, 36312,
                13260, 47372, 18852, 32976, 61268, 21816, 8156,  25184, 46324,
                16900, 5964,  57136, 48356, 760,   33528, 22024, 43384, 39772,
                13956, 35468, 47456, 16796, 56904, 12704, 30436, 54264, 4560,
                9648,  63464,
            },
            {
                2752,  11512, 59092, 42248, 55148, 25348, 4968,  53696, 3040,
                21984, 47800, 27068, 18004, 49884, 2700,  55464, 15424, 43444,
                7224,  36764, 17312, 61248, 52536, 32312, 60248, 43084, 2296,
                23164, 7588,  51248, 54384, 21296, 9008,  44300, 28916, 49008,
                57908, 40620, 12904, 60904, 21760, 30888, 48564, 34304, 61516,
                29384, 24332, 64868, 52240, 20796, 38084, 12776, 33992, 50904,
                26456, 58060, 32764, 62724, 7120,  48508, 58580, 52736, 24292,
                16660, 56876, 28148, 45544, 58456, 42480, 29932, 1088,  21628,
                45432, 13464, 3068,  30324, 50464, 12656, 57192, 4848,  31348,
                140,   57776, 13408, 63344, 44292, 10800, 54420, 45980, 65140,
                4808,  48696, 19540, 6136,  44668, 64720, 32372, 49720, 6652,
                41836, 56316, 52344, 40336, 15592, 48552, 54716, 38216, 59372,
                28528, 62492, 40200, 10296, 25176, 52648, 61052, 11224, 25532,
                30804, 8252,  20368, 28176, 816,   36368, 8044,  40700, 20588,
                58384, 43788,
            },
            {
                38752, 22364, 51172, 7368,  17488, 31196, 45524, 28464, 40920,
                64684, 32664, 9784,  38924, 60636, 42272, 35848, 10628, 26584,
                62956, 2880,  30412, 44520, 7680,  37844, 26736, 13420, 28836,
                34488, 17032, 42160, 2668,  47472, 35464, 63856, 716,   6408,
                33800, 23492, 47572, 7284,  38840, 51368, 1500,  5440,  19488,
                13596, 48200, 4712,  32792, 27100, 62976, 7756,  23668, 46396,
                6320,  20872, 12132, 45708, 35976, 10140, 20344, 33320, 12000,
                6176,  50380, 32164, 13220, 4800,  23240, 35652, 55392, 6244,
                41172, 59560, 18512, 46652, 35984, 27228, 21244, 53452, 45212,
                18808, 38668, 22592, 33212, 16752, 6724,  29032, 22100, 12860,
                25080, 34640, 28800, 56588, 37252, 592,   9560,  58636, 25616,
                21508, 30320, 10992, 7296,  26056, 35196, 540,   19468, 43036,
                13432, 50680, 36096, 15812, 45640, 29964, 5696,  17288, 54252,
                62900, 50884, 38248, 60552, 52892, 44852, 63796, 25672, 34768,
                15912, 48164,
            },
            {
                60408, 26216, 13684, 34032, 48088, 63332, 10360, 37308, 18916,
                7608,  58824, 1368,  52636, 24736, 29972, 65208, 21456, 49060,
                39584, 14028, 56292, 25040, 47592, 19364, 5132,  62116, 55176,
                45828, 64224, 11284, 27020, 32332, 58748, 24892, 39108, 55220,
                15428, 59660, 3132,  35628, 28172, 14732, 58352, 26280, 37256,
                53600, 10128, 39144, 45964, 2784,  43948, 54860, 31044, 62068,
                3120,  37620, 54260, 29672, 16916, 61044, 39076, 2484,  63096,
                43816, 21880, 524,   65404, 38556, 48732, 15096, 62588, 10928,
                26200, 32424, 23016, 54772, 7012,  63968, 39792, 9944,  34472,
                61380, 7312,  47628, 55676, 41076, 60120, 50564, 32604, 57692,
                43364, 61732, 51052, 11392, 23436, 52804, 14704, 46516, 34220,
                63692, 1488,  37564, 60252, 43864, 63924, 29912, 10728, 33016,
                3876,  22292, 54272, 2412,  65344, 41456, 37028, 48056, 1604,
                44484, 23008, 4428,  10592, 32624, 18716, 13964, 3156,  55864,
                29120, 5900,
            },
            {
                18132, 45784, 61860, 39800, 3228,  58036, 22896, 51916, 13732,
                49388, 35760, 43872, 20508, 12436, 5936,  53572, 236,   45400,
                19212, 51812, 34668, 10512, 64864, 53672, 40792, 9660,  49036,
                4272,  20300, 37716, 60852, 5348,  17536, 11904, 20228, 46060,
                27692, 43020, 18424, 64640, 55108, 9772,  45516, 62428, 42716,
                31736, 57496, 16420, 60860, 19308, 36328, 14992, 10560, 41384,
                17904, 49568, 64616, 476,   42152, 25196, 55240, 46200, 26956,
                53844, 40048, 35064, 25068, 52780, 8584,  20088, 33304, 47048,
                51436, 63492, 8944,  42580, 15164, 48516, 2164,  29476, 15892,
                50184, 11900, 27952, 4088,  20636, 36484, 3016,  15656, 39852,
                8052,  1988,  17360, 40184, 60412, 30812, 19732, 42836, 4408,
                50708, 17720, 53776, 23348, 13936, 4664,  50196, 55852, 61920,
                47396, 8228,  27924, 18908, 56244, 7520,  21228, 59828, 27720,
                33840, 13324, 55836, 42448, 48772, 23556, 59688, 50116, 41924,
                10368, 52960,
            },
            {
                32536, 1736,  54476, 20460, 27856, 15568, 6360,  41732, 60912,
                25400, 28904, 15940, 57016, 47084, 37636, 16880, 33656, 28440,
                59176, 5028,  22080, 42828, 1200,  15640, 35744, 24644, 30576,
                14924, 52088, 23784, 55804, 43688, 49416, 53152, 30792, 62700,
                7764,  11000, 52408, 31612, 24356, 41020, 17428, 21324, 7784,
                44,    22620, 50436, 25912, 6388,  56408, 28788, 51456, 58316,
                34388, 26848, 8480,  22452, 51740, 14284, 5072,  31208, 18968,
                14752, 9684,  60360, 16924, 29436, 44248, 61000, 40276, 3652,
                16680, 37264, 744,   57416, 33432, 24324, 60808, 55752, 42172,
                25432, 37552, 64428, 52272, 31016, 10184, 63080, 47564, 27436,
                53144, 21636, 32180, 45304, 3788,  26788, 62336, 38108, 11776,
                59316, 28160, 45084, 31920, 57988, 20984, 40516, 17284, 24012,
                38764, 60184, 34772, 43812, 24748, 13788, 32428, 10536, 57540,
                18844, 40024, 64856, 16500, 27384, 5960,  38808, 31256, 21220,
                65360, 36772,
            },
            {
                43064, 8504,  29736, 11076, 44352, 55848, 35156, 32056, 1800,
                55352, 5256,  63792, 32268, 8520,  61096, 41640, 55856, 12948,
                8896,  62220, 38824, 27876, 32140, 59528, 21004, 44356, 58220,
                33920, 40068, 24,    29596, 14020, 8008,  40704, 3860,  34880,
                48720, 38304, 21584, 2040,  49512, 5784,  33960, 56656, 47864,
                28032, 63948, 35480, 12148, 41908, 48080, 22916, 1040,  20524,
                45148, 12660, 39952, 57484, 47624, 34808, 59260, 8152,  37336,
                57060, 47560, 42392, 6728,  57380, 1688,  12404, 23996, 58256,
                30740, 20720, 45264, 28088, 11056, 18248, 36512, 6404,  20364,
                58372, 1224,  45488, 14264, 42748, 24968, 54116, 19912, 11948,
                35408, 64212, 48184, 14044, 8796,  53900, 49256, 7260,  24744,
                15468, 40424, 8288,  3384,  47836, 36700, 27372, 6228,  44960,
                14816, 31084, 11308, 51848, 61796, 38172, 50140, 43076, 4908,
                52124, 7920,  30396, 1476,  35588, 57668, 9196,  46408, 384,
                12972, 24576,
            },
            {
                16104, 48968, 62892, 38112, 50120, 24716, 64312, 47500, 20664,
                44504, 11156, 40384, 22396, 51188, 26916, 3884,  23652, 50020,
                31156, 44244, 16684, 54700, 48452, 12340, 50980, 2788,  7572,
                63384, 10308, 47924, 18816, 65248, 36716, 22048, 59356, 14880,
                24784, 56788, 61140, 12928, 36964, 63016, 52048, 11676, 39428,
                14680, 43828, 4844,  30428, 58776, 8908,  65312, 37492, 4912,
                60336, 16100, 32392, 4176,  28668, 11152, 21152, 50124, 63628,
                28428, 4096,  31836, 21008, 49480, 34524, 53444, 28260, 6928,
                54568, 13104, 49928, 59628, 40720, 53016, 46224, 13616, 51392,
                32816, 9268,  23500, 17860, 59484, 5564,  38744, 392,   44340,
                57340, 6440,  25820, 55992, 35852, 41648, 17000, 32824, 57000,
                47416, 35424, 65092, 19324, 12264, 51872, 9308,  64396, 53184,
                884,   58136, 48160, 3096,  6024,  29620, 16992, 62608, 26364,
                36592, 47632, 24164, 61132, 51640, 42988, 15620, 62172, 28368,
                50664, 55636,
            },
            {
                59524, 21976, 33632, 5840,  18596, 2756,  13120, 9172,  30016,
                16800, 48812, 36104, 940,   14392, 45648, 18420, 64036, 37352,
                2664,  57568, 25696, 7820,  3660,  62868, 36840, 28492, 17648,
                53032, 26756, 42904, 58476, 32228, 25600, 54984, 44568, 896,
                29952, 6580,  41576, 46288, 16192, 29284, 3576,  25172, 32496,
                53768, 59756, 20252, 51840, 17080, 33300, 40352, 25576, 47332,
                30208, 52840, 63684, 43244, 18212, 61580, 39564, 43632, 1540,
                23540, 13044, 52168, 64336, 26248, 37972, 15412, 47840, 43412,
                64708, 38476, 5756,  22724, 4068,  31000, 64996, 2636,  44088,
                28304, 62460, 35768, 49400, 56404, 33380, 28628, 61232, 16188,
                30568, 39104, 18632, 59592, 22572, 29292, 3276,  63232, 21672,
                16,    53156, 26220, 56012, 33056, 59744, 41948, 29816, 35044,
                21436, 26268, 18344, 40580, 23276, 55572, 46380, 20,    21928,
                14548, 54736, 44176, 19352, 12196, 22300, 53996, 34224, 18920,
                37996, 3912,
            },
            {
                27148, 41060, 13724, 57868, 52196, 42668, 59636, 39568, 53772,
                58752, 24156, 61924, 53440, 30140, 59072, 34896, 10836, 47384,
                20496, 13752, 34064, 45816, 18956, 24044, 41208, 56092, 46384,
                21844, 38080, 13016, 6768,  3076,  49864, 9872,  17204, 63816,
                51956, 18780, 33436, 26008, 59420, 20604, 44272, 58212, 18516,
                2420,  9568,  37232, 46412, 1656,  53644, 13808, 18352, 55880,
                10448, 6980,  24236, 37460, 2088,  53944, 25564, 32724, 16652,
                55328, 35536, 39752, 17844, 10632, 3272,  61712, 19132, 9372,
                32256, 25852, 17620, 61968, 35444, 9744,  26668, 21548, 39384,
                16156, 53196, 3800,  40520, 7792,  13064, 46296, 50472, 23460,
                4432,  52028, 10816, 1328,  43920, 12588, 55300, 37248, 44660,
                10756, 30088, 14540, 43560, 1928,  24484, 16744, 5196,  13312,
                55788, 43012, 63632, 33636, 13156, 60264, 9320,  41324, 34084,
                63872, 3516,  9708,  32504, 38188, 2540,  26072, 5204,  56916,
                45024, 7840,
            },
            {
                35428, 47368, 1028,  25876, 30736, 35900, 21644, 26664, 4212,
                34040, 6984,  19108, 43060, 4604,  39696, 7220,  54172, 28792,
                41516, 52864, 64744, 39396, 58984, 31556, 9644,  14608, 60040,
                1452,  33012, 62040, 53940, 20540, 41428, 34396, 27912, 39152,
                47344, 11452, 55068, 1484,  9360,  48824, 38396, 7212,  64908,
                49532, 41224, 26740, 22496, 60284, 29200, 5392,  63064, 41736,
                35604, 50640, 20616, 56760, 46636, 14956, 6432,  47956, 10144,
                61288, 44900, 7864,  58908, 46672, 42512, 56348, 22600, 36216,
                1608,  52628, 41668, 47388, 14892, 54892, 49800, 57612, 7280,
                60692, 11516, 30668, 19464, 26948, 65156, 21144, 9072,  34712,
                42076, 62240, 46440, 32400, 64800, 50448, 26420, 5464,  18036,
                59972, 39560, 6096,  49672, 21556, 38144, 62600, 51016, 47468,
                37660, 9700,  6680,  50632, 27616, 36376, 20084, 53048, 30164,
                56900, 39336, 27732, 60012, 50468, 64100, 47712, 40256, 11696,
                30700, 63740,
            },
            {
                19752, 10132, 54540, 64636, 16612, 7300,  46184, 63072, 14884,
                50544, 38548, 27660, 12580, 55984, 21272, 24972, 15432, 59656,
                1456,  6100,  22868, 10944, 50288, 4380,  43776, 34916, 5772,
                25264, 50616, 16320, 28784, 46760, 57712, 13388, 61324, 4716,
                22860, 36132, 43552, 64216, 31496, 52528, 13960, 35124, 23340,
                30236, 15444, 62124, 34536, 12444, 44104, 49016, 22204, 27144,
                2368,  44608, 12456, 28112, 8872,  35192, 65204, 29740, 51056,
                20140, 27240, 456,   30600, 24036, 33256, 5376,  29288, 50404,
                60828, 11476, 56712, 28820, 296,   43096, 18644, 37112, 33312,
                46916, 22816, 55812, 48520, 43460, 2824,  38044, 58016, 54392,
                14140, 26676, 20204, 36648, 15804, 8312,  40060, 48060, 32260,
                51980, 23572, 63040, 33996, 58228, 10528, 44468, 27108, 1616,
                60804, 17108, 30380, 54484, 44500, 4660,  15556, 48736, 6744,
                12332, 18156, 46192, 5852,  15360, 29612, 8512,  59020, 23772,
                16784, 50968,
            },
            {
                60884, 44640, 22944, 38712, 48788, 11500, 56308, 112,   31416,
                44856, 10248, 65512, 46964, 31144, 49796, 62860, 43648, 32040,
                50936, 27180, 37200, 16240, 29156, 55048, 20800, 48340, 64180,
                40736, 10580, 44440, 35456, 8780,  24068, 1808,  50500, 31844,
                56908, 7036,  15588, 22004, 39992, 4628,  26612, 54636, 10732,
                46840, 56592, 4084,  7444,  55424, 38156, 9216,  32584, 58004,
                15732, 62400, 33388, 60172, 40608, 21712, 57788, 2772,  37780,
                41704, 57244, 14796, 49072, 62476, 12800, 53568, 40248, 14524,
                45668, 21180, 7796,  33868, 23848, 63296, 11796, 4280,  25640,
                41932, 980,   62744, 36300, 15124, 52580, 31520, 17800, 1528,
                29460, 48636, 6792,  53360, 24456, 58292, 19660, 61664, 13392,
                27904, 8736,  46828, 16140, 3988,  30420, 55292, 19140, 32900,
                23132, 57984, 39972, 2376,  21736, 64936, 58716, 26224, 43356,
                23320, 51288, 58216, 24692, 35420, 44384, 20356, 33720, 52912,
                42568, 2240,
            },
            {
                33220, 14800, 29056, 4340,  32568, 60136, 19736, 40612, 52676,
                22484, 57964, 1844,  16184, 34600, 8900,  3256,  36676, 10172,
                17932, 61584, 56060, 46512, 648,   61996, 26384, 17160, 30652,
                54196, 22648, 59376, 4424,  64732, 38204, 16000, 43868, 19632,
                26528, 62408, 48460, 29000, 57920, 19088, 60920, 43224, 512,
                38732, 19496, 51556, 42184, 25200, 16556, 61188, 52036, 4320,
                39712, 19256, 47548, 136,   52932, 17240, 43424, 24464, 13304,
                5744,  34100, 52820, 20912, 38352, 7496,  18292, 64060, 2884,
                25352, 37980, 16740, 59232, 39440, 48324, 31616, 59612, 14744,
                51164, 18268, 9292,  26060, 5140,  59916, 23928, 45240, 64016,
                10484, 39644, 60888, 3588,  42636, 30304, 944,   34972, 54352,
                2732,  41892, 36464, 20240, 52284, 41136, 7768,  14236, 36424,
                46276, 11576, 25444, 49096, 12720, 38700, 32304, 1436,  37208,
                61612, 33408, 3128,  41540, 11276, 56028, 560,   62848, 13660,
                6496,  26476,
            },
            {
                40132, 56184, 8324,  52100, 43760, 24788, 36824, 28508, 8032,
                17692, 35488, 25804, 40764, 60288, 19816, 57116, 46304, 22476,
                40172, 7064,  33000, 42444, 13944, 35668, 51832, 7116,  12448,
                37440, 2180,  18620, 31592, 51808, 27152, 54792, 60332, 9692,
                40932, 13144, 53052, 2872,  10328, 46688, 32264, 16128, 63120,
                28048, 33052, 13236, 64508, 30408, 836,   20152, 46176, 11636,
                30124, 54960, 5864,  25808, 31296, 49332, 10844, 61488, 54648,
                18400, 63992, 25844, 3548,  43668, 58604, 27672, 34352, 48028,
                57200, 31392, 44472, 6060,  53880, 20308, 8356,  52952, 38656,
                29420, 57928, 45464, 34140, 40084, 12044, 49564, 7628,  35276,
                56000, 22236, 33068, 13280, 56372, 46856, 10848, 22428, 44316,
                17268, 59508, 55772, 11540, 64192, 25584, 48868, 61528, 3452,
                52732, 64000, 8196,  34672, 56572, 17928, 47068, 10644, 54460,
                15868, 8604,  20900, 65072, 52620, 17572, 38832, 28136, 47000,
                36728, 58696,
            },
            {
                3508,  47620, 21332, 62136, 16244, 2452,  55120, 13384, 43296,
                61576, 54724, 48580, 6704,  52444, 29192, 13976, 26708, 64448,
                48676, 12352, 20584, 58052, 24396, 9256,  40192, 58572, 45396,
                27572, 56536, 48072, 13920, 42504, 20784, 5572,  34152, 46720,
                404,   37548, 32952, 42616, 24700, 36420, 7068,  50944, 22560,
                58564, 8768,  44768, 21792, 48700, 35832, 54596, 26704, 42812,
                36864, 23416, 64584, 14348, 38492, 7276,  34740, 27828, 45160,
                30788, 9368,  47072, 36276, 11344, 50964, 23248, 42212, 10040,
                4672,  51532, 65440, 13644, 2292,  27816, 45308, 23148, 64700,
                3208,  12652, 21708, 54504, 63592, 29788, 20268, 41568, 25880,
                16028, 44600, 51524, 18372, 26208, 62784, 38140, 50644, 65436,
                25404, 7052,  28900, 32380, 660,   38948, 22376, 57304, 28828,
                41368, 20200, 43912, 27980, 52012, 6996,  22756, 62580, 28072,
                50044, 39956, 30112, 47524, 26816, 4736,  49756, 21908, 9344,
                54060, 18544,
            },
            {
                65280, 12772, 35584, 41016, 10388, 49180, 63812, 5276,  47136,
                30748, 3556,  11848, 23216, 44264, 38312, 424,   54332, 33792,
                3972,  53468, 30084, 2588,  49716, 65352, 31240, 3584,  20224,
                63000, 41500, 6536,  36264, 61776, 11096, 49196, 29580, 23832,
                56112, 19000, 59440, 16360, 65216, 54972, 12712, 40488, 3784,
                47652, 36692, 2304,  59556, 40288, 5924,  63300, 14072, 7808,
                59864, 50160, 9852,  44452, 58692, 54076, 3504,  20416, 50452,
                1640,  40824, 60068, 17044, 55504, 31460, 1424,  15968, 62212,
                19808, 29008, 24084, 36744, 41784, 61296, 16092, 35380, 6400,
                43172, 32616, 50080, 7188,  16908, 192,   58568, 53620, 4720,
                62672, 2080,  36880, 8804,  41304, 5932,  15500, 31160, 4200,
                39840, 48416, 21588, 43608, 15332, 46628, 34276, 11228, 6508,
                16636, 31984, 336,   14896, 60584, 40820, 30884, 44708, 4852,
                19024, 56960, 1128,  36472, 14452, 57688, 32548, 61592, 43708,
                30364, 24704,
            },
            {
                7004,  32044, 25772, 59076, 29592, 18984, 33948, 26604, 21552,
                38728, 15348, 64320, 33452, 18368, 62316, 6420,  41936, 15996,
                25108, 44872, 60736, 38872, 17584, 44116, 23024, 15108, 48844,
                33168, 9964,  23160, 26084, 55928, 2036,  39632, 15264, 64068,
                7332,  50752, 27464, 4692,  49208, 21036, 29660, 57400, 25932,
                18096, 53708, 14972, 27632, 10912, 17764, 32872, 51020, 20884,
                1408,  34332, 17908, 28912, 22108, 41052, 16072, 63380, 37004,
                57464, 14012, 23028, 29224, 6120,  45276, 60528, 39028, 53180,
                35328, 11908, 49548, 18240, 57604, 31124, 10648, 49260, 55932,
                19232, 59768, 37896, 27312, 47664, 43992, 32352, 38260, 13076,
                28444, 49156, 31268, 60280, 54564, 28216, 20400, 57376, 9720,
                34008, 13124, 51476, 62252, 58668, 5424,  18700, 50776, 37640,
                45440, 54868, 62696, 36912, 24652, 3012,  57924, 13768, 35928,
                42804, 11968, 60484, 45152, 23920, 6816,  41188, 11800, 2696,
                15724, 49376,
            },
            {
                42220, 52836, 532,   45992, 6252,  39172, 57844, 52900, 9552,
                58932, 42244, 27968, 50808, 9312,  55584, 30564, 49412, 58924,
                19456, 7940,  35836, 27356, 10608, 5328,  55228, 36888, 53004,
                132,   59576, 16620, 50792, 31224, 44084, 18348, 52204, 36172,
                11976, 31388, 44712, 9388,  39056, 1012,  45412, 33456, 9760,
                42404, 62276, 30940, 49996, 56752, 24832, 43524, 57840, 29508,
                40464, 46356, 55816, 2524,  32780, 60544, 47256, 12052, 25460,
                32644, 7604,  52516, 65048, 35176, 20532, 13248, 26452, 6948,
                43396, 56140, 2928,  8836,  45856, 25232, 780,   38980, 28868,
                14508, 24820, 2680,  62080, 11716, 23720, 8540,  18432, 45872,
                59464, 20700, 11624, 23596, 3336,  47860, 36012, 44940, 60896,
                54176, 18088, 2632,  36740, 9900,  27756, 54348, 64960, 25872,
                59128, 22536, 5068,  49824, 46212, 9984,  20668, 50660, 63588,
                26392, 22176, 31892, 8876,  53232, 63068, 18768, 35036, 59028,
                55576, 37632,
            },
            {
                20124, 61212, 17256, 55888, 23168, 43588, 14696, 2140,  49724,
                18128, 35164, 1364,  45592, 24908, 13308, 20824, 35120, 10968,
                39848, 63320, 14580, 51684, 57384, 33496, 61444, 26440, 11856,
                39720, 29692, 46768, 64304, 8260,  21724, 60668, 3448,  58088,
                41676, 20528, 62576, 23948, 34960, 60116, 15484, 63776, 51276,
                4780,  22880, 34800, 7452,  45932, 2716,  37084, 4732,  61816,
                15708, 25496, 63624, 12884, 52024, 8148,  23792, 4292,  44416,
                39376, 18776, 47968, 42380, 3388,  49492, 56888, 33028, 47100,
                21796, 60128, 39912, 33964, 62904, 51656, 54816, 21848, 63928,
                46976, 9052,  51268, 41492, 35828, 55500, 65296, 51936, 34320,
                6192,  56668, 38488, 43720, 64276, 13812, 51704, 484,   22712,
                26408, 41760, 30368, 56152, 24020, 41216, 32172, 1372,  14184,
                9020,  30268, 12608, 33680, 17260, 38988, 53912, 33164, 8072,
                2580,  54376, 48608, 16488, 38504, 29448, 50368, 25916, 46480,
                22788, 8820,
            },
            {
                28580, 13664, 36520, 50724, 11896, 64524, 31800, 37448, 25152,
                61028, 54112, 7708,  57312, 37124, 60500, 47228, 2476,  52612,
                28980, 46648, 1636,  21276, 47688, 41856, 18200, 4368,  45176,
                19672, 54516, 5620,  13352, 38464, 34476, 46268, 28356, 24960,
                5232,  47928, 14344, 55948, 51964, 27936, 6132,  20244, 37276,
                13372, 59732, 39524, 19820, 65452, 54056, 22408, 12376, 47952,
                9080,  37728, 6008,  42484, 19440, 35568, 28468, 50900, 55008,
                59696, 636,   28056, 11232, 24884, 16228, 63272, 9564,  88,
                16968, 30632, 26896, 14760, 20016, 5980,  12968, 41264, 3792,
                34628, 57644, 30392, 16272, 21088, 1888,  29636, 14848, 24696,
                42000, 892,   16208, 53324, 33644, 19048, 30232, 11384, 38032,
                7612,  64456, 46844, 4836,  16856, 49748, 44576, 20484, 39364,
                48140, 43072, 53116, 57216, 26628, 65256, 29244, 15696, 46800,
                40496, 61480, 35136, 3764,  20876, 44124, 5676,  14116, 1068,
                31424, 62984,
            },
            {
                3964,  47908, 32932, 3036,  27892, 8484,  20592, 45144, 4856,
                12756, 31100, 22416, 16548, 3888,  40984, 27424, 65084, 23536,
                5716,  56156, 32444, 25336, 7016,  13424, 29376, 50084, 63696,
                23752, 35504, 42620, 27608, 1576,  57084, 14632, 9484,  32904,
                54756, 36948, 2740,  40600, 18264, 11456, 43476, 48620, 55404,
                29104, 46452, 72,    11728, 26996, 41096, 16884, 31272, 51628,
                27640, 54396, 23144, 48848, 57656, 39100, 64756, 10384, 30592,
                15328, 21716, 62292, 53624, 36956, 40948, 28852, 54312, 37436,
                51252, 64356, 4260,  48504, 43880, 37304, 32684, 60992, 17936,
                43484, 12384, 53876, 5508,  60676, 49444, 39588, 7076,  48284,
                63100, 27588, 47184, 7900,  26104, 5264,  61696, 42724, 57740,
                49848, 14432, 20160, 34568, 59852, 12308, 7364,  57488, 35000,
                62232, 2168,  19160, 7884,  42028, 5560,  1048,  58876, 24040,
                18436, 28552, 13336, 57292, 64688, 10668, 54720, 59816, 39916,
                52224, 44488,
            },
            {
                38288, 56476, 23944, 60564, 41236, 55224, 48800, 62004, 28752,
                40208, 47508, 63912, 43372, 51472, 32244, 8432,  15176, 38240,
                18312, 43440, 59956, 40584, 62544, 37984, 58200, 9576,  31952,
                2956,  58804, 17592, 62076, 52568, 24240, 49448, 64892, 43104,
                17328, 60384, 29668, 8000,  33240, 57732, 23136, 32072, 3496,
                25424, 17168, 58020, 52504, 33804, 8624,  45272, 60052, 35060,
                18224, 62324, 3376,  30352, 14000, 1004,  17356, 42232, 6544,
                36380, 45468, 33192, 8220,  58472, 5860,  22240, 44916, 12956,
                24756, 42096, 8004,  55592, 59472, 29316, 24572, 50256, 7536,
                27428, 22972, 37628, 45132, 33316, 26140, 57504, 19480, 10960,
                37044, 31524, 21352, 59704, 41024, 36480, 53884, 16388, 27840,
                32840, 1920,  52188, 39688, 28812, 63208, 23232, 51776, 16476,
                28060, 24544, 59492, 35904, 22216, 49360, 44208, 12248, 37484,
                51332, 6188,  44980, 24416, 40884, 32896, 27172, 35952, 21284,
                17148, 11244,
            },
            {
                26776, 19348, 5532,  45360, 17804, 34624, 1396,  15092, 53212,
                19164, 10148, 34776, 26076, 12256, 19780, 58240, 44764, 54424,
                30680, 12492, 8792,  16648, 52360, 844,   20632, 43260, 55460,
                14832, 8528,  48100, 32468, 10908, 40676, 19472, 6260,  52872,
                22540, 12544, 50200, 63192, 46388, 1360,  61320, 40260, 8388,
                64184, 41848, 49168, 30064, 14912, 61684, 55040, 1100,  6584,
                42932, 11656, 33468, 45556, 60844, 26240, 47804, 22660, 61508,
                56696, 12732, 50492, 18204, 43620, 31096, 2608,  60696, 19544,
                57896, 32288, 11400, 22876, 17472, 1760,  10112, 54328, 46576,
                58900, 496,   63648, 18680, 8936,  13272, 43048, 61740, 52552,
                2984,  55740, 13100, 50108, 10504, 23264, 46572, 3616,  21540,
                44528, 58980, 25144, 10568, 45808, 2776,  37068, 30964, 5032,
                10752, 50992, 39672, 14364, 61844, 31112, 56304, 34292, 62736,
                9908,  55868, 31408, 168,   50748, 16220, 3444,  48328, 63668,
                6780,  57996,
            },
            {
                41928, 65008, 9940,  53068, 13072, 38776, 22568, 57588, 33340,
                5848,  58904, 84,    53860, 61556, 4896,  48316, 34460, 960,
                62824, 50096, 36528, 23100, 28336, 34244, 48820, 24900, 36320,
                51284, 39468, 25796, 21188, 45436, 3668,  36068, 29228, 38968,
                276,   27112, 43944, 19844, 24984, 38036, 16308, 53668, 12824,
                34892, 21960, 6384,  37688, 3920,  23680, 38636, 26024, 20656,
                56824, 39508, 50816, 21444, 7100,  55956, 34000, 52560, 3824,
                26692, 40220, 2060,  24024, 62796, 12212, 51768, 34596, 47224,
                3752,  39300, 50360, 35348, 45824, 65080, 38632, 15584, 20660,
                31664, 40204, 51688, 28692, 47392, 4040,  35372, 30300, 23676,
                45016, 17656, 34168, 64604, 2456,  56756, 31020, 63756, 8664,
                12908, 35968, 62584, 6616,  53728, 15276, 42416, 55732, 64660,
                47196, 32828, 3624,  45572, 9616,  16920, 25648, 20108, 2856,
                42648, 21832, 60640, 36584, 19332, 61728, 9060,  43120, 24116,
                33796, 50164,
            },
            {
                15236, 35560, 49076, 30948, 26340, 63428, 7868,  44148, 24620,
                41672, 49708, 21376, 36816, 29896, 40008, 24504, 10468, 21480,
                26900, 5164,  46508, 55712, 3968,  63904, 11608, 17212, 6092,
                29076, 64752, 1116,  53660, 59784, 16044, 61772, 56380, 46996,
                10052, 58300, 34716, 5456,  52248, 9920,  30244, 49816, 19596,
                56644, 45504, 10796, 62940, 19236, 43392, 50128, 12980, 47516,
                29112, 64628, 8992,  16768, 37344, 40992, 11788, 19572, 31792,
                46908, 16560, 29468, 55776, 48000, 38468, 26400, 16132, 7456,
                29452, 62344, 14380, 27084, 52432, 5432,  42752, 61604, 35596,
                4564,  11128, 14804, 60276, 55072, 21788, 50740, 15920, 5728,
                59208, 40252, 7000,  29084, 43288, 14052, 18712, 39212, 49356,
                55540, 41520, 18068, 31660, 21872, 48888, 26472, 19396, 13348,
                41004, 21600, 55172, 28676, 51620, 38200, 6576,  53548, 47796,
                27000, 15760, 48748, 11940, 54120, 45416, 30332, 56008, 13624,
                1856,  29680,
            },
            {
                8076,  21980, 368,   40120, 58392, 3848,  29976, 51720, 11620,
                65508, 28296, 15492, 7236,  44368, 17392, 63824, 51816, 57632,
                41724, 19148, 32892, 59428, 15188, 40860, 46048, 60508, 56976,
                19940, 43508, 13844, 34684, 29988, 7472,  25300, 13184, 33344,
                63552, 16384, 41548, 55800, 14124, 65100, 26632, 44108, 2056,
                60940, 25760, 31756, 51508, 57100, 28204, 8244,  59244, 36416,
                15508, 1964,  24600, 53928, 28092, 2676,  63256, 43820, 8440,
                58532, 65336, 5012,  35228, 20596, 8956,  56932, 64092, 43344,
                54104, 18832, 1208,  57448, 20928, 30708, 13008, 25332, 57144,
                49636, 44444, 24140, 36668, 1900,  41564, 64848, 10164, 38064,
                27048, 53604, 20892, 47404, 37220, 24824, 51716, 33832, 5076,
                23424, 28344, 264,   57308, 38564, 61812, 8416,  1192,  35296,
                60328, 6864,  17956, 63568, 436,   59648, 41504, 13828, 64300,
                32948, 39760, 7580,  29176, 25304, 4876,  37412, 20236, 39692,
                60880, 54004,
            },
            {
                44052, 61704, 55560, 16632, 45916, 20292, 47932, 36248, 18140,
                3084,  38524, 46472, 60732, 55596, 13092, 2892,  29096, 38648,
                14372, 53432, 10940, 43984, 7924,  25868, 30892, 2264,  37732,
                10188, 47480, 5468,  56168, 41396, 50348, 44160, 20816, 2964,
                49768, 23040, 30476, 3368,  36720, 47680, 6740,  33760, 38716,
                5172,  15480, 40760, 868,   35248, 16568, 64020, 3636,  32752,
                52512, 44820, 60196, 32068, 48924, 57904, 15004, 51320, 24684,
                38356, 11408, 53076, 41940, 15040, 812,   31924, 40408, 21460,
                10760, 36532, 48348, 41364, 9432,  34452, 53484, 2424,  28416,
                19216, 64228, 6048,  31024, 17684, 25788, 56508, 32812, 49032,
                312,   62676, 14968, 9444,  61036, 1272,  58708, 11064, 61376,
                46896, 14820, 51008, 43808, 11444, 33912, 58840, 45480, 52060,
                29876, 25044, 43192, 12388, 33860, 22472, 46424, 29496, 9200,
                55188, 4100,  58332, 42312, 65408, 15024, 51448, 59336, 10712,
                46532, 25456,
            },
            {
                37856, 31988, 12252, 24868, 33944, 9756,  62356, 14440, 57172,
                32240, 52588, 10616, 23620, 31724, 49976, 36008, 8468,  47284,
                3828,  61256, 30032, 21156, 38796, 54684, 50240, 22132, 33480,
                52940, 27060, 62204, 23228, 18704, 10472, 37088, 55208, 28608,
                40040, 8368,  45968, 59164, 21092, 17500, 60192, 22460, 55620,
                29208, 52776, 47832, 23084, 12220, 45948, 30688, 22340, 41372,
                11116, 19872, 5608,  42156, 10216, 22232, 35616, 29700, 228,
                18636, 32820, 22736, 49984, 61172, 45684, 24276, 5492,  50696,
                28004, 60224, 24564, 6548,  63540, 45456, 17792, 47756, 8640,
                41444, 33732, 53616, 58424, 39320, 7992,  45540, 19748, 12432,
                43792, 23956, 50812, 32608, 54744, 27448, 44868, 17100, 29920,
                37692, 8344,  62996, 24492, 4884,  17184, 27780, 21128, 39032,
                9768,  48816, 57852, 37056, 26572, 50268, 18740, 2548,  35800,
                23708, 17180, 49924, 35008, 1512,  22800, 32368, 6492,  27280,
                17416, 4744,
            },
            {
                19716, 50424, 3560,  59900, 41360, 5808,  54552, 27236, 42524,
                7592,  25664, 58072, 1708,  19568, 41972, 26092, 58728, 17640,
                34004, 24244, 48648, 612,   64544, 18080, 4496,  13668, 63128,
                41868, 16516, 31820, 38956, 3760,  58484, 65444, 6472,  17612,
                53060, 61156, 26160, 11356, 32776, 54184, 40552, 9540,  13060,
                62320, 18784, 8800,  59880, 55028, 39388, 6308,  53800, 48336,
                58396, 27224, 38208, 62372, 17644, 4240,  46152, 40028, 60600,
                55104, 44628, 5996,  27604, 10036, 58244, 34852, 55332, 13608,
                2536,  44636, 33172, 15364, 58732, 22628, 36992, 56324, 60764,
                15416, 22348, 10424, 47176, 13600, 51728, 2820,  61672, 28256,
                57548, 36176, 3976,  18848, 39636, 6280,  21688, 41792, 3940,
                54032, 20184, 34780, 55204, 30672, 47604, 53336, 4132,  14408,
                63076, 2012,  16068, 53412, 4344,  10784, 57132, 60848, 39068,
                44988, 62256, 20504, 9696,  46248, 56872, 40648, 48276, 35152,
                52680, 64044,
            },
            {
                41976, 8704,  47172, 28848, 51516, 37388, 23152, 1152,  49868,
                20608, 62552, 43644, 34556, 6256,  64780, 54040, 21696, 44112,
                63304, 6760,  36820, 56648, 27728, 34920, 44648, 58820, 25112,
                7988,  348,   50604, 12596, 46148, 26568, 31248, 47812, 34944,
                13496, 972,   37652, 51428, 43136, 2252,  28432, 50372, 46208,
                35884, 42972, 32464, 27364, 3440,  19604, 61888, 25680, 17228,
                1244,  34360, 13232, 47816, 56016, 25884, 64360, 12764, 7816,
                48656, 14356, 62844, 37796, 19312, 29352, 16532, 47288, 37932,
                64924, 17868, 51520, 29780, 39892, 488,   12216, 31976, 5252,
                38904, 50976, 856,   27244, 60140, 30284, 23056, 34796, 15668,
                7104,  41712, 30108, 65480, 13176, 48916, 57476, 33108, 64296,
                25904, 49232, 2836,  41348, 13716, 64920, 37184, 42888, 55516,
                32968, 23432, 46100, 30384, 65188, 42472, 31960, 14780, 52084,
                26760, 12200, 31300, 52564, 28280, 18184, 12636, 61184, 772,
                30728, 13832,
            },
            {
                58440, 35684, 22184, 64876, 13212, 18584, 45316, 61404, 33572,
                39592, 12008, 16820, 48392, 37812, 11568, 15192, 1292,  30516,
                50688, 12748, 42100, 15664, 8500,  52332, 11416, 39960, 48840,
                19656, 56720, 60756, 34436, 53852, 15756, 2084,  22752, 42496,
                56848, 20032, 64420, 15704, 23940, 5816,  63536, 16292, 25476,
                1348,  6424,  63388, 37936, 50400, 44204, 13692, 36724, 8472,
                65196, 54692, 22840, 30104, 7488,  33128, 52096, 20192, 28420,
                34952, 25312, 1684,  41296, 53404, 3244,  62228, 8320,  30960,
                23176, 57164, 5368,  10120, 54368, 26860, 50024, 43624, 24408,
                29632, 61872, 44888, 35788, 16996, 42824, 55492, 38916, 48320,
                53752, 9884,  59192, 45340, 23296, 35496, 8868,  15292, 45756,
                11984, 39536, 18216, 59908, 22872, 6912,  10640, 25920, 18056,
                59840, 41252, 6372,  20316, 38520, 8012,  47180, 22064, 6528,
                1828,  41388, 59448, 3432,  38284, 62932, 7948,  21744, 44572,
                55688, 24348,
            },
            {
                28156, 54144, 16380, 1704,  31700, 56268, 10340, 28360, 15524,
                5296,  55132, 30144, 51676, 27460, 61108, 46792, 40360, 9456,
                26548, 53504, 20156, 60492, 46744, 23576, 32188, 5536,  29748,
                37052, 44456, 28516, 6300,  21116, 40316, 59460, 51324, 10208,
                27484, 45296, 30024, 7976,  47092, 55524, 37360, 31740, 59220,
                21632, 56504, 14612, 24252, 9824,  58812, 33272, 51828, 29116,
                45448, 39872, 50716, 3248,  44192, 37600, 57596, 2160,  41820,
                59812, 17332, 55860, 32224, 12356, 43724, 51072, 20580, 40952,
                11924, 42896, 35404, 47028, 20476, 59924, 7960,  65476, 19404,
                54728, 12900, 4448,  20992, 64596, 6412,  11252, 3852,  63608,
                21704, 26324, 17188, 50280, 2736,  28400, 52812, 60420, 688,
                31344, 56236, 9260,  28984, 50132, 45112, 57876, 31192, 3280,
                48108, 35472, 12276, 56400, 25440, 17628, 54992, 28832, 63372,
                49124, 33484, 16040, 23528, 55372, 43148, 26300, 50928, 37380,
                10232, 3104,
            },
            {
                46072, 39176, 6860,  44364, 59228, 40404, 4144,  52480, 36600,
                46400, 64364, 21896, 8892,  3212,  23328, 32640, 56468, 35620,
                59408, 3704,  39140, 29524, 2024,  62516, 16720, 55056, 64948,
                14088, 9880,  23868, 42288, 63908, 11548, 36780, 7336,  62960,
                33544, 4728,  39624, 58996, 34516, 12288, 19868, 43768, 9136,
                39784, 51420, 47140, 17880, 30824, 172,   22180, 42564, 4032,
                20376, 14960, 9804,  61180, 19384, 15856, 11032, 45596, 22644,
                51600, 9736,  47160, 63876, 23908, 36160, 26588, 5144,  58884,
                53536, 1544,  25764, 63184, 13676, 33508, 40692, 16164, 2556,
                48288, 33088, 56692, 38364, 25364, 50560, 31780, 19388, 44840,
                33428, 1872,  37540, 11928, 55872, 42916, 18092, 24536, 36868,
                21280, 62140, 43452, 34272, 1520,  16888, 35988, 51304, 21264,
                8748,  28128, 62096, 49992, 1180,  60260, 34868, 11988, 37572,
                19180, 56264, 45820, 6684,  35532, 14640, 4516,  32008, 15884,
                59832, 18960,
            },
            {
                63108, 11916, 51120, 26744, 34804, 23912, 48544, 19980, 57676,
                25376, 656,   41756, 59596, 45100, 53208, 19076, 5460,  16984,
                48016, 22640, 33200, 10408, 42952, 49556, 36052, 21044, 1524,
                46632, 53488, 17484, 2564,  49472, 30228, 18472, 46956, 24528,
                14212, 53924, 49664, 18272, 26716, 61012, 48464, 4484,  53960,
                27548, 11964, 34888, 41740, 64316, 48304, 56344, 12060, 61544,
                57884, 32316, 26892, 55668, 40880, 25204, 61864, 31832, 6640,
                37024, 29856, 4296,  21212, 7700,  60056, 47808, 15932, 33904,
                28308, 49052, 18444, 38384, 3288,  45336, 28388, 52704, 37392,
                9960,  26560, 41984, 8216,  14488, 46716, 59644, 27324, 13532,
                51380, 41268, 61240, 31168, 5100,  63916, 39296, 6464,  51384,
                46952, 4476,  14132, 53200, 24912, 62564, 40460, 13048, 64208,
                43336, 53692, 15344, 32404, 40396, 44896, 5016,  53120, 43324,
                9176,  25828, 64172, 11692, 49728, 29020, 53632, 64760, 41080,
                48360, 33776,
            },
            {
                30204, 23212, 61204, 17960, 8828,  14620, 63676, 30504, 7812,
                12896, 32960, 18180, 35908, 14004, 39308, 28592, 62840, 43592,
                13556, 65428, 54784, 17916, 57864, 5088,  27456, 41404, 59700,
                25740, 32500, 62012, 35752, 57608, 25984, 54380, 652,   41044,
                58080, 22188, 3108,  10540, 41696, 356,   30216, 15156, 64980,
                33100, 2316,  60368, 4956,  26624, 7688,  16608, 39132, 24380,
                6284,  37148, 48496, 568,   34832, 53636, 4124,  49644, 65120,
                15304, 58888, 44448, 40268, 14120, 52548, 352,   38660, 62704,
                9288,  14712, 59340, 52004, 23740, 56740, 5436,  22280, 58576,
                46000, 62748, 17972, 52068, 58152, 34752, 1216,  39268, 62108,
                6732,  57320, 23840, 47016, 19932, 33592, 13088, 59212, 29624,
                16640, 26724, 38320, 58712, 20256, 5116,  28620, 55092, 36,
                24584, 38124, 5700,  22512, 10024, 27700, 16368, 23964, 61328,
                332,   30404, 39344, 21856, 59720, 44428, 1096,  20352, 24928,
                8348,  5008,
            },
            {
                43272, 340,   37156, 47212, 53388, 42308, 2132,  38008, 44228,
                60908, 49352, 54432, 26372, 6344,  58096, 10744, 50888, 440,
                25716, 7616,  45244, 36688, 24852, 14500, 52660, 11708, 8212,
                38704, 51148, 5260,  43660, 8960,  13796, 32912, 61848, 44884,
                16860, 37396, 32120, 62684, 56088, 21396, 51904, 38184, 23640,
                18244, 45348, 55316, 20464, 38056, 53688, 33640, 28372, 46732,
                53168, 18032, 64432, 13056, 21344, 47312, 28896, 12552, 39528,
                26720, 19580, 53780, 34660, 27560, 31404, 19064, 45092, 54748,
                22320, 32520, 43536, 7388,  30612, 11264, 62384, 35088, 13852,
                31304, 1672,  23356, 29796, 4784,  43748, 24060, 54836, 9624,
                18608, 29148, 15008, 53816, 8728,  26804, 49680, 44196, 10324,
                54232, 65116, 8264,  31580, 47780, 10396, 45496, 16264, 34072,
                49516, 18988, 58180, 46728, 64496, 51144, 57284, 33040, 14016,
                47948, 51996, 5528,  17600, 33848, 9764,  38672, 57972, 12572,
                52452, 56792,
            },
            {
                20744, 13948, 55792, 32360, 5200,  60604, 27040, 55472, 17064,
                22980, 40104, 10176, 63800, 31668, 48268, 20948, 37312, 32344,
                41316, 29100, 51764, 2464,  61200, 30784, 45920, 33940, 57528,
                19856, 15608, 28936, 22140, 47232, 38604, 20564, 5940,  28908,
                8508,  51560, 25088, 46108, 14068, 35160, 44700, 6312,  57980,
                49308, 8580,  29728, 14940, 43436, 11072, 50072, 62648, 1584,
                41544, 9244,  29832, 43740, 59860, 8380,  17560, 43168, 54980,
                848,   10612, 48548, 3520,  57348, 64784, 11360, 25028, 6160,
                40616, 2592,  64528, 36832, 47496, 17760, 41248, 49856, 19680,
                7440,  39576, 48796, 61088, 11536, 20480, 15676, 30900, 47576,
                36564, 44328, 120,   35232, 40812, 57104, 22360, 1748,  35644,
                41148, 23104, 3316,  42556, 51724, 36976, 61412, 6828,  30512,
                60540, 11564, 2348,  35972, 30876, 19964, 2816,  41588, 36696,
                20424, 58968, 42452, 54644, 26884, 62264, 16616, 46788, 35660,
                27272, 39552,
            },
            {
                49608, 65432, 27984, 10676, 22372, 19296, 33520, 11932, 51648,
                5004,  29396, 2408,  43256, 16536, 3780,  24316, 55100, 62112,
                18720, 58492, 11372, 21680, 40020, 6972,  63636, 23032, 43308,
                4044,  61124, 12036, 56528, 64696, 3720,  51988, 60152, 35304,
                12484, 65220, 39392, 5320,  28804, 9204,  61428, 25976, 12520,
                40828, 35624, 22352, 61852, 58380, 23764, 3712,  19612, 14576,
                34724, 22948, 57108, 5092,  38592, 33504, 24312, 57832, 35572,
                22264, 32340, 61076, 17088, 8740,  42684, 49596, 36304, 57708,
                29512, 48640, 20288, 13168, 26972, 54968, 116,   25016, 43832,
                64620, 55892, 16352, 36244, 53824, 40444, 63472, 50244, 3404,
                22672, 65076, 11488, 51068, 61708, 4532,  16196, 63164, 31968,
                19168, 49264, 56556, 12372, 17740, 26872, 22720, 56928, 39608,
                44692, 26296, 41844, 54404, 13180, 7596,  48960, 63236, 4816,
                28408, 11324, 32152, 2208,  47676, 7428,  23812, 31132, 2328,
                61080, 17068,
            },
            {
                8696,  3136,  45328, 40044, 58648, 50596, 46352, 7424,  64656,
                35100, 57792, 20548, 52712, 46092, 61020, 40592, 8488,  14904,
                5064,  47148, 34288, 16084, 50272, 54628, 18296, 256,   48052,
                30356, 49988, 41012, 34392, 25512, 17236, 42276, 48832, 26264,
                55004, 1768,  19360, 59032, 50636, 17544, 54188, 31092, 4108,
                63060, 52316, 740,   46504, 6368,  36200, 31080, 55212, 44476,
                59536, 49844, 26136, 15404, 52308, 63852, 2104,  50004, 7208,
                63088, 41140, 45444, 24532, 38440, 21392, 2152,  13872, 61564,
                16912, 9968,  52520, 58176, 4536,  31896, 60496, 9620,  53284,
                27796, 3980,  32736, 9160,  47080, 28236, 5812,  33812, 55936,
                41352, 58936, 25744, 19272, 30296, 38068, 47608, 55308, 6944,
                13952, 60388, 29132, 34784, 63976, 2112,  53008, 13588, 20332,
                8676,  51388, 16472, 62128, 23608, 39512, 26508, 17076, 53812,
                46044, 24424, 65240, 15060, 35932, 40568, 55704, 50852, 6484,
                44908, 33264,
            },
            {
                25720, 35956, 52708, 15692, 29732, 1728,  37376, 41816, 25148,
                47596, 15224, 37708, 25724, 33704, 12640, 29644, 49592, 35376,
                52968, 23076, 64040, 44144, 9396,  37592, 27868, 13972, 55736,
                36204, 24208, 7748,  1352,  54840, 31356, 9192,  14668, 21968,
                40556, 31656, 47980, 23736, 42632, 1212,  38860, 20768, 43900,
                16692, 27352, 32720, 13476, 51200, 41128, 65448, 27532, 6836,
                10864, 37332, 2792,  46300, 31176, 10688, 20148, 45780, 15788,
                29704, 12780, 5024,  51184, 56452, 30676, 53320, 33820, 46272,
                26432, 42532, 34692, 23312, 39328, 45796, 15852, 37468, 33876,
                12540, 21476, 44984, 24612, 668,   58468, 14060, 26380, 10652,
                16896, 6964,  32652, 45356, 13688, 8408,  24268, 27940, 45080,
                37292, 724,   24512, 46336, 5920,  41916, 32280, 48192, 3640,
                65168, 29012, 5412,  32936, 45256, 58928, 10740, 34836, 56820,
                8476,  38424, 18888, 51736, 59324, 12272, 19724, 64128, 14900,
                22744, 57748,
            },
            {
                42024, 18952, 62712, 5948,  24300, 61596, 13748, 18124, 30820,
                200,   62468, 11192, 7552,  57128, 18484, 1144,  59184, 27284,
                42624, 1972,  31212, 26184, 4324,  60548, 33328, 42264, 10556,
                64288, 16252, 53264, 44880, 19684, 37696, 59216, 2508,  44256,
                57976, 7264,  15204, 36608, 11440, 64012, 33748, 47880, 56812,
                10268, 37496, 54924, 19104, 25588, 9816,  16724, 47452, 21308,
                32212, 63352, 17884, 56336, 42128, 27724, 61136, 39756, 25572,
                52396, 59344, 36552, 27888, 14840, 6872,  62772, 23692, 5108,
                55716, 952,   60768, 7744,  49964, 63504, 21100, 2744,  59016,
                51108, 41488, 56804, 63944, 19220, 37888, 43140, 61260, 21096,
                51932, 37644, 49148, 2720,  59988, 53596, 42012, 20472, 58768,
                52696, 9868,  39896, 20856, 55632, 15736, 37668, 59664, 24200,
                35332, 56272, 38360, 20944, 544,   52768, 30200, 43316, 21660,
                1428,  49944, 43728, 5284,  25704, 29628, 3380,  42432, 37676,
                10204, 53864,
            },
            {
                4932,  13216, 48472, 31688, 43388, 56216, 8600,  52976, 58260,
                21204, 42964, 54584, 49972, 39472, 65064, 44304, 21368, 10008,
                55876, 13604, 39448, 56816, 19552, 48920, 23804, 57760, 6436,
                20972, 39420, 27600, 62312, 11036, 46740, 28484, 64028, 33608,
                10604, 51044, 61880, 29604, 55828, 25668, 7772,  14616, 60704,
                23916, 3656,  63888, 42984, 57424, 2176,  38624, 60568, 54132,
                13896, 40348, 51196, 21992, 6592,  13364, 36812, 4588,  56968,
                9472,  20852, 1260,  64168, 18800, 47528, 40536, 11700, 19560,
                38664, 31584, 14636, 18448, 28824, 11520, 43040, 26212, 47324,
                17588, 5632,  29216, 14860, 49100, 52848, 8140,  35180, 46468,
                1240,  27648, 57728, 22244, 34608, 64580, 3600,  11732, 33464,
                17520, 61472, 48852, 30660, 62388, 9460,  28008, 43876, 11872,
                17812, 46416, 10344, 60816, 14984, 48252, 5888,  62540, 13580,
                33424, 61448, 30440, 10000, 54956, 45852, 34148, 49144, 27668,
                61700, 30808,
            },
            {
                21676, 58452, 38892, 10480, 19788, 34520, 27740, 49136, 39204,
                5916,  32836, 27996, 3620,  22768, 30996, 6708,  48144, 37904,
                17636, 62456, 7568,  51900, 11784, 45008, 2308,  29784, 52192,
                46968, 3540,  32484, 14288, 57040, 4548,  23368, 52572, 16748,
                27288, 21064, 45096, 3164,  18188, 46284, 52368, 2384,  29340,
                50408, 40004, 7876,  28504, 48588, 33552, 22964, 6040,  43348,
                528,   25412, 8980,  34696, 59836, 53720, 48688, 18560, 33700,
                41748, 44760, 31220, 49328, 9932,  34700, 43928, 60156, 29188,
                51512, 64872, 45184, 54764, 36144, 5764,  55968, 30980, 8644,
                39072, 61056, 35936, 10964, 31860, 3184,  23004, 29500, 54976,
                64196, 14572, 42736, 9896,  17652, 39540, 29280, 51048, 6316,
                42776, 27124, 14248, 6844,  52008, 19208, 948,   50728, 55268,
                31148, 2404,  50252, 41992, 28036, 36984, 19096, 25128, 40652,
                58100, 16204, 22996, 37452, 63312, 21256, 7732,  56564, 17632,
                1648,  51180,
            },
            {
                45776, 27316, 552,   54192, 63292, 46668, 2656,  23000, 12112,
                59684, 17248, 46944, 14232, 36228, 53592, 15684, 61500, 32856,
                24540, 46708, 28588, 35600, 65184, 15644, 40784, 61064, 18008,
                34924, 59100, 51064, 22160, 41116, 35288, 49340, 6624,  39112,
                56100, 35784, 6380,  40228, 59752, 31760, 22248, 42356, 34792,
                17276, 46024, 21232, 12224, 61824, 15180, 52500, 30004, 35484,
                49364, 56844, 28924, 44816, 1552,  23924, 30528, 64488, 2896,
                14920, 55368, 23280, 38196, 58460, 25756, 1932,  16172, 48380,
                9412,  22008, 3648,  40836, 24616, 60108, 51700, 14512, 65340,
                23544, 1516,  54152, 25592, 40284, 62692, 44616, 11840, 18192,
                39400, 7464,  30796, 53084, 25552, 48256, 15340, 36616, 62548,
                22356, 3220,  57344, 38616, 45248, 33124, 22984, 63772, 7200,
                39596, 26152, 57768, 22812, 7500,  64716, 54996, 2648,  51580,
                27656, 4232,  48768, 41380, 220,   14672, 60444, 24248, 12156,
                40388, 35256,
            },
            {
                14728, 61060, 33324, 23480, 6692,  37284, 16112, 64884, 44408,
                35704, 25024, 61836, 56732, 10276, 42060, 26840, 2400,  51128,
                4944,  41632, 20456, 864,   31804, 22580, 37444, 9672,  26648,
                12932, 43276, 8532,  840,   30092, 17496, 59604, 12940, 42676,
                144,   62952, 24440, 54000, 9156,  13004, 37884, 65148, 5272,
                58232, 53272, 31572, 36680, 5352,  44976, 58852, 18416, 11808,
                61616, 19972, 14736, 63728, 38748, 16456, 9788,  46620, 27300,
                50416, 7308,  61292, 13112, 4752,  53124, 32452, 57056, 6220,
                35896, 26912, 12832, 47512, 33236, 1104,  19800, 41612, 48964,
                33076, 19300, 46236, 6636,  58676, 16664, 56116, 4820,  33140,
                50064, 20276, 61588, 45832, 5420,  57260, 400,   44132, 55508,
                30992, 47052, 35020, 24924, 11304, 59928, 42488, 35728, 13480,
                20080, 62164, 16052, 34812, 12240, 32136, 44620, 9936,  37900,
                46084, 12408, 54220, 18476, 35116, 51920, 43128, 32808, 47420,
                64712, 7468,
            },
            {
                55940, 42508, 17096, 51028, 12744, 41824, 53328, 31140, 3796,
                51292, 8764,  39944, 1024,  48900, 19268, 57824, 35264, 12476,
                59876, 54284, 14700, 58720, 50488, 6128,  54064, 45692, 4488,
                63036, 24328, 37816, 54808, 65040, 44056, 9032,  25636, 31628,
                19896, 48760, 14824, 34088, 49968, 27336, 55360, 19592, 10696,
                26148, 14240, 260,   56108, 25116, 40112, 9092,  27012, 38128,
                3664,  46444, 31540, 5160,  52756, 42384, 58192, 20512, 60300,
                39228, 35072, 17752, 29088, 46056, 20092, 39700, 24176, 62572,
                43068, 53240, 59080, 17024, 62972, 10716, 37968, 27412, 4696,
                12260, 59824, 42284, 50624, 21020, 27636, 37212, 48180, 24368,
                58916, 2204,  36336, 12868, 32772, 63664, 19608, 24592, 13000,
                9348,  18480, 65396, 1744,  53420, 16096, 4304,  27096, 47988,
                54324, 44200, 6520,  52576, 47204, 59368, 17012, 29984, 20832,
                60016, 7664,  31064, 57780, 26168, 10700, 29472, 2840,  53680,
                19412, 25280,
            },
            {
                9740,  3300,  48120, 29300, 59996, 26272, 19344, 10068, 55900,
                28604, 20612, 45064, 30732, 24228, 64032, 7936,  44784, 22332,
                30472, 8984,  38316, 43600, 24856, 61660, 18576, 33588, 56176,
                49272, 30856, 15888, 47088, 27128, 20760, 53296, 61640, 46816,
                57024, 11028, 30220, 43908, 5776,  61016, 1108,  47744, 44332,
                60228, 39480, 49028, 64252, 19664, 51280, 1852,  63444, 55192,
                23448, 41108, 58544, 11068, 26068, 35700, 6840,  32964, 12428,
                24812, 720,   54148, 42140, 65400, 8524,  50312, 14032, 18508,
                272,   31828, 38484, 6796,  50460, 23196, 44536, 58048, 52816,
                36996, 29752, 15376, 9512,  33596, 240,   13512, 63308, 9316,
                41580, 27036, 56228, 22976, 41072, 8648,  28716, 52884, 39080,
                60752, 49948, 41232, 28840, 21316, 49092, 38224, 57948, 9604,
                32788, 3544,  29428, 38136, 2148,  24432, 41132, 5476,  63544,
                34340, 23704, 43408, 64932, 5984,  46836, 17128, 59116, 5316,
                38560, 31328,
            },
            {
                62480, 35944, 20792, 39464, 56708, 2224,  34160, 60928, 38836,
                15516, 5372,  62812, 34424, 12944, 51780, 29196, 40180, 16344,
                62184, 49192, 27024, 3696,  11468, 47364, 28920, 14340, 41440,
                2552,  19804, 60608, 11436, 5216,  33976, 2200,  36220, 16076,
                4936,  38532, 64500, 18028, 22000, 40576, 15536, 32748, 29052,
                6988,  22864, 34028, 10284, 30312, 15900, 43576, 32076, 48312,
                6960,  16760, 34384, 50032, 21700, 45512, 55796, 3844,  51788,
                44684, 62224, 10160, 22700, 33628, 2940,  55512, 30116, 46780,
                57784, 9808,  20468, 28228, 55808, 31280, 2660,  16236, 8756,
                63452, 24920, 56608, 38472, 61780, 43672, 51984, 29840, 45632,
                15496, 52592, 4492,  17280, 50656, 60240, 35712, 46188, 4312,
                32928, 7084,  14996, 56024, 5416,  62496, 30388, 18316, 24152,
                64940, 40660, 21884, 61092, 18524, 49348, 56992, 13160, 53300,
                48084, 15840, 2480,  37188, 20644, 40136, 62148, 36136, 22508,
                50600, 43888,
            },
            {
                28024, 54388, 5248,  11524, 45548, 7720,  49616, 43204, 23980,
                48208, 58132, 53384, 17708, 3088,  38180, 5604,  55136, 47132,
                1332,  36272, 17468, 56036, 32560, 39332, 8240,  64484, 22400,
                36568, 7380,  52136, 39208, 58124, 14520, 50584, 40776, 23440,
                28624, 51592, 2284,  58400, 35500, 53100, 24392, 62688, 37348,
                55032, 17732, 42552, 4512,  47024, 58168, 35368, 20896, 13720,
                53148, 27792, 61804, 3080,  18040, 65028, 14208, 28708, 40796,
                19136, 30852, 47992, 15352, 59488, 27136, 36800, 11396, 40752,
                25464, 49392, 64668, 42260, 13672, 46372, 61876, 35280, 21624,
                43556, 1824,  6332,  47252, 23564, 54872, 18000, 6992,  21452,
                34316, 64564, 38684, 30496, 43840, 11688, 1940,  15908, 21812,
                54896, 26596, 37432, 45624, 32384, 12708, 43964, 40,    50540,
                14564, 46640, 11136, 53888, 8364,  28500, 36128, 640,   26484,
                39628, 10136, 51408, 56204, 13340, 49484, 27232, 9296,  15248,
                58104, 12624,
            },
            {
                18164, 23876, 65356, 32916, 16700, 61972, 21828, 13736, 416,
                32668, 11084, 41000, 22544, 47724, 60432, 26196, 20296, 10624,
                33332, 23352, 63576, 51552, 20808, 58432, 108,   50392, 26864,
                57860, 44464, 32576, 23992, 43116, 29792, 63020, 7484,  44744,
                60684, 9504,  26016, 46220, 11960, 4276,  48952, 8880,  13356,
                2032,  51632, 62296, 27820, 54016, 23988, 3740,  10532, 60208,
                36640, 44972, 8952,  54740, 30984, 38096, 48352, 23392, 57296,
                7844,  36060, 52764, 4976,  39360, 44212, 21172, 63600, 6976,
                34120, 15828, 3292,  35764, 4924,  25692, 18996, 54580, 48568,
                32248, 51540, 18408, 28816, 12028, 3724,  32124, 39704, 60892,
                1228,  11196, 47792, 6776,  20040, 53980, 26176, 48600, 58520,
                42124, 10956, 63508, 23372, 8424,  58940, 40240, 53312, 7672,
                36364, 56160, 26192, 34124, 43512, 15012, 62244, 45196, 32336,
                19512, 61440, 29328, 25012, 33444, 1560,  53096, 41900, 30224,
                46160, 800,
            },
            {
                48500, 37928, 51224, 42340, 25660, 29760, 37720, 52364, 64552,
                26328, 36792, 6952,  44172, 56704, 31888, 14092, 64836, 43004,
                57952, 7476,  40740, 4960,  15088, 44748, 35816, 12620, 54680,
                16936, 10808, 63956, 1016,  17944, 54508, 22172, 12108, 32740,
                55400, 20112, 41600, 32160, 56004, 28524, 43124, 20304, 57652,
                46536, 35172, 21572, 14460, 7548,  38412, 64776, 41508, 29456,
                1472,  24740, 40036, 13128, 43400, 180,   9840,  61464, 16400,
                2624,  64188, 26332, 12072, 56172, 17116, 51168, 2360,  59920,
                53532, 45388, 22932, 60836, 52056, 38896, 7196,  11112, 27680,
                14220, 39984, 60084, 36232, 63964, 41988, 58344, 26436, 49020,
                54660, 23972, 28124, 57816, 62072, 33152, 39904, 64788, 30596,
                17660, 1140,  51116, 19152, 48672, 35440, 25484, 20132, 28576,
                59712, 17372, 1492,  63252, 20568, 51104, 6764,  22652, 55568,
                3688,  42560, 7268,  46216, 62828, 17748, 4868,  56396, 34764,
                6932,  60788,
            },
            {
                31228, 8548,  13916, 1784,  53820, 57536, 9832,  4612,  45772,
                18148, 59296, 29308, 15872, 9404,  35564, 2044,  49760, 28568,
                52856, 13080, 45972, 30760, 27404, 60480, 23628, 42600, 31136,
                5480,  47916, 28328, 49952, 8692,  37424, 47200, 3948,  17092,
                48540, 36108, 14316, 7140,  61900, 16944, 37964, 64376, 26040,
                30500, 40504, 11008, 59284, 31308, 50784, 45712, 15184, 56192,
                51604, 18876, 63392, 22060, 59504, 26188, 53584, 36808, 31856,
                50284, 43300, 20052, 30188, 46880, 8188,  24520, 32000, 13928,
                19224, 29424, 56664, 11980, 32132, 47584, 58516, 41888, 65016,
                4460,  55728, 9776,  20880, 49632, 15824, 8832,  35432, 13376,
                18892, 44696, 36784, 14664, 2968,  22516, 8372,  13412, 6088,
                44728, 34020, 27844, 56716, 3032,  15396, 61456, 4584,  45408,
                32096, 42492, 6180,  38480, 30648, 58700, 40468, 11316, 48836,
                17192, 58008, 35352, 21616, 11580, 43504, 23728, 64340, 19508,
                25728, 40448,
            },
            {
                55644, 21524, 59820, 46076, 6084,  35140, 19828, 41228, 31184,
                54572, 2600,  50480, 63408, 25316, 53476, 41680, 17772, 4652,
                24648, 37384, 19016, 55000, 49252, 8788,  3492,  19176, 62668,
                39732, 21780, 35592, 13264, 60948, 25848, 56672, 40088, 65384,
                24860, 516,   59416, 49900, 23524, 1780,  33668, 51412, 7304,
                3536,  17216, 55380, 44048, 752,   18696, 26652, 22336, 5648,
                48448, 31516, 4404,  47240, 34132, 6236,  18640, 45976, 12648,
                24100, 38652, 59992, 1032,  62948, 35540, 58324, 48388, 43016,
                37224, 1112,  40032, 8276,  17444, 21924, 29884, 708,   23420,
                34512, 44992, 25688, 31364, 912,   53524, 23068, 43220, 5540,
                59548, 31108, 9140,  52108, 42576, 46852, 56492, 37824, 24320,
                52916, 59380, 39676, 12144, 43032, 29892, 54824, 10364, 64056,
                13296, 22772, 52328, 47580, 12856, 3400,  25344, 33772, 65420,
                27264, 14036, 38876, 54292, 58944, 31444, 36988, 47752, 10452,
                51612, 15688,
            },
            {
                4624,  36400, 28512, 17744, 24376, 63620, 48428, 15144, 60840,
                23116, 12836, 39740, 19976, 6140,  46764, 22392, 38336, 59640,
                32732, 62036, 1084,  10828, 65324, 38692, 34268, 52276, 46172,
                2008,  58288, 54256, 42068, 20000, 2960,  34372, 29296, 6428,
                52796, 44060, 30180, 18836, 39996, 54208, 10656, 15168, 45620,
                60664, 49836, 23184, 36456, 63008, 33384, 8280,  61468, 34616,
                11300, 37700, 56840, 15600, 52492, 29360, 41772, 57956, 3952,
                55768, 9168,  15072, 33536, 22432, 41092, 4048,  28180, 9084,
                62628, 26048, 50340, 44296, 64008, 55160, 14536, 37528, 60932,
                53128, 16812, 7316,  57940, 38444, 46556, 28248, 65424, 50356,
                1892,  40156, 63124, 17164, 34444, 26964, 18748, 49956, 61492,
                3736,  21076, 8084,  62832, 37192, 46784, 21260, 34364, 39504,
                48980, 57392, 9248,  27880, 18652, 54788, 46420, 37372, 8456,
                52260, 30432, 5824,  364,   50388, 8124,  14128, 28968, 1968,
                61572, 44596,
            },
            {
                12236, 63132, 49428, 41584, 32704, 11992, 38452, 27204, 7800,
                43764, 48708, 36044, 57736, 30684, 61284, 12012, 55336, 8460,
                14868, 48096, 21768, 43156, 25136, 16224, 57184, 28644, 10032,
                25032, 15540, 6560,  31708, 51440, 45960, 15240, 59156, 21544,
                12524, 38100, 9028,  63580, 47316, 27576, 58832, 41536, 24340,
                32804, 9476,  28012, 5324,  48244, 12472, 53408, 39908, 58528,
                17528, 44008, 27328, 8420,  39640, 11552, 64380, 21528, 34856,
                28284, 48172, 53520, 45200, 11560, 54912, 16324, 20500, 52752,
                12496, 57848, 4988,  34992, 27492, 3572,  42864, 49888, 19500,
                11572, 48008, 42268, 62376, 13700, 4412,  17852, 33048, 56364,
                20776, 25776, 48364, 5912,  59320, 11852, 252,   29576, 35028,
                15312, 47708, 31852, 17220, 24188, 6672,  50860, 2108,  17984,
                25980, 4008,  36844, 64452, 41952, 60396, 16280, 1220,  21140,
                43636, 60800, 47108, 19812, 26836, 40376, 60032, 53952, 38132,
                22820, 33392,
            },
            {
                26600, 128,   53044, 8776,  58736, 3216,  50864, 56636, 876,
                33260, 10372, 26828, 4020,  15284, 34044, 300,   44776, 30056,
                40356, 52132, 35284, 59088, 31600, 4164,  47704, 13824, 60784,
                33756, 38460, 64820, 27132, 11208, 62512, 7896,  42672, 50524,
                27260, 55612, 34908, 3140,  13524, 20916, 4952,  36524, 53480,
                19192, 65036, 39232, 58236, 16348, 42328, 29640, 46584, 1196,
                24308, 65372, 3044,  60688, 23632, 49068, 1532,  16576, 61624,
                7072,  40236, 19328, 4556,  26880, 49528, 60532, 38848, 45736,
                32528, 23468, 15388, 20388, 52220, 10312, 33500, 6200,  26608,
                32108, 1936,  28772, 22708, 35032, 54492, 41088, 11752, 7408,
                37868, 13800, 55312, 22960, 30636, 53828, 63920, 44236, 9492,
                41340, 27168, 56052, 932,   53088, 60624, 27960, 41556, 58764,
                31380, 15628, 53724, 33856, 23836, 7272,  32028, 49908, 57404,
                11944, 24172, 34468, 63324, 16012, 45300, 21964, 6480,  17552,
                42088, 57456,
            },
            {
                47264, 19952, 39264, 15832, 23012, 45324, 30292, 20696, 61944,
                52920, 17716, 65284, 55544, 43208, 49064, 27320, 64308, 19700,
                25604, 3612,  12804, 7676,  54008, 41200, 20920, 5828,  42844,
                53356, 18616, 45000, 548,   23036, 39088, 19068, 32676, 1896,
                62980, 17264, 24092, 57516, 32104, 44664, 61128, 29960, 292,
                12616, 43404, 3264,  21360, 54512, 25188, 5892,  20324, 14276,
                52728, 30544, 49932, 19248, 36500, 55348, 31640, 44308, 50744,
                24628, 58280, 31412, 64124, 36316, 7392,  30584, 1468,  64648,
                6612,  42092, 48896, 60176, 38332, 62144, 46932, 55832, 59068,
                40148, 64440, 50732, 18668, 9132,  60424, 26080, 47376, 61692,
                29504, 45564, 35244, 2492,  41644, 38300, 7248,  21916, 57944,
                51464, 12424, 65260, 43548, 33112, 10868, 14468, 55272, 8192,
                62644, 46028, 20964, 1700,  11148, 44780, 28532, 63436, 39716,
                17492, 2692,  41708, 9376,  53216, 32880, 3188,  65112, 48692,
                30276, 8056,
            },
            {
                61808, 35268, 30912, 64888, 55264, 6688,  17072, 42476, 37172,
                24688, 46112, 40292, 21932, 8628,  36908, 16928, 50876, 6052,
                56392, 62444, 45696, 29160, 17784, 63720, 36440, 51100, 23324,
                30484, 8452,  49004, 56388, 35460, 53028, 57836, 48332, 10016,
                40544, 45532, 6800,  49712, 39036, 9716,  50908, 16268, 46904,
                56864, 34112, 51732, 30860, 10588, 35316, 62524, 56076, 38696,
                9584,  42652, 34760, 12360, 46256, 5760,  26236, 10272, 38220,
                13148, 428,   15744, 43152, 23380, 56984, 14292, 25416, 34900,
                17824, 55420, 29164, 4,     12680, 24680, 16956, 22076, 8884,
                15420, 36448, 5344,  56912, 43836, 30744, 1264,  21868, 52876,
                16836, 9844,  64292, 19784, 51528, 12936, 47532, 25812, 17888,
                36504, 4148,  22464, 38620, 18684, 49120, 35692, 44680, 23256,
                38688, 12420, 50220, 40868, 59224, 51752, 14008, 4684,  26052,
                35960, 55448, 49468, 28208, 12576, 57660, 35920, 25856, 11464,
                54812, 15316,
            },
            {
                24016, 4832,  13116, 43352, 25832, 35888, 59948, 9416,  13680,
                4572,  30456, 1764,  13020, 52212, 24296, 58056, 10524, 42252,
                33736, 15944, 37960, 23800, 49840, 1952,  26856, 11532, 62732,
                3204,  59264, 13516, 28964, 16276, 6056,  25220, 29968, 13868,
                21012, 53832, 28940, 15228, 64792, 25548, 6068,  22584, 62280,
                26832, 7860,  14788, 45652, 59436, 1976,  48488, 27512, 32876,
                60220, 4764,  21512, 56660, 15272, 41276, 62604, 18672, 53256,
                28884, 47248, 60164, 51852, 9856,  40904, 46940, 51424, 11368,
                58964, 37152, 9264,  45380, 31768, 41468, 2688,  30376, 44720,
                52964, 24284, 12740, 46748, 37748, 15604, 51040, 34216, 3676,
                42464, 57800, 4544,  27832, 59940, 16200, 32796, 62056, 5968,
                30968, 54160, 46524, 8920,  25560, 3308,  63868, 19692, 324,
                27440, 6340,  61628, 29880, 18972, 37648, 56600, 21608, 47280,
                7964,  31028, 59256, 4016,  23272, 42808, 18532, 50912, 39412,
                2320,  44836,
            },
            {
                58500, 48856, 52304, 10456, 1668,  47696, 51444, 27964, 63264,
                54932, 49268, 59288, 32592, 61896, 5380,  39616, 31076, 22412,
                48300, 1276,  54668, 9972,  58188, 33020, 46724, 56828, 16868,
                35048, 40432, 21236, 43236, 60988, 46832, 2652,  37260, 59000,
                33396, 61908, 808,   43464, 19436, 35448, 42112, 54768, 37664,
                20060, 40928, 63328, 23892, 38380, 18732, 43932, 7696,  16792,
                50988, 25752, 63152, 1724,  29412, 59348, 33516, 2932,  56444,
                34984, 7564,  20736, 33180, 3992,  18568, 62216, 21560, 43380,
                3512,  26684, 19988, 53764, 65392, 58076, 36156, 50224, 62908,
                4080,  33440, 60820, 26952, 21048, 6904,  63244, 58772, 25096,
                38756, 31632, 48912, 24052, 44524, 36928, 1344,  50068, 39884,
                59012, 15052, 28276, 61244, 56880, 40960, 29380, 52936, 32700,
                47848, 57264, 35040, 24896, 4168,  9188,  33360, 43360, 53352,
                61992, 20008, 15148, 37944, 63932, 47492, 8496,  31676, 59328,
                20712, 37240,
            },
            {
                32552, 18384, 28544, 61368, 39876, 19432, 33308, 5628,  40908,
                20956, 16060, 38372, 25864, 18552, 46988, 2568,  63284, 13708,
                60036, 27252, 43896, 20716, 40668, 14660, 7888,  38760, 44564,
                25944, 54708, 4580,  33144, 11648, 39808, 63276, 17724, 50192,
                5220,  23780, 37860, 11820, 51480, 57724, 3812,  32488, 11124,
                1644,  49500, 28760, 4480,  50872, 13460, 64352, 22920, 40976,
                12084, 44800, 37772, 53108, 24124, 48684, 8164,  39276, 22952,
                65180, 44604, 39708, 27420, 55972, 37832, 28996, 6356,  31932,
                63448, 40244, 48432, 7048,  14492, 18600, 6076,  26496, 11764,
                20196, 41920, 54704, 2996,  49132, 40264, 12100, 44944, 8308,
                19408, 14300, 53576, 11172, 7432,  56612, 28864, 10496, 45108,
                19744, 2300,  34904, 50984, 5304,  16716, 13192, 59792, 10048,
                42884, 17508, 14388, 54084, 45584, 65000, 27592, 16528, 588,
                11616, 40416, 45888, 6512,  30136, 52392, 728,   13984, 62920,
                27768, 6752,
            },
            {
                42292, 56296, 3872,  22448, 31560, 15000, 57556, 23648, 45500,
                34744, 10212, 7056,  44344, 56320, 34648, 28764, 53224, 19084,
                36308, 6608,  31512, 64644, 3932,  61360, 29708, 20028, 804,
                50176, 9712,  64600, 51972, 27444, 7932,  22728, 53472, 42176,
                9400,  47076, 56488, 31428, 7916,  27568, 14164, 60008, 44432,
                53700, 17060, 34720, 57356, 9304,  31796, 36660, 57968, 100,
                54832, 30264, 7180,  17156, 10788, 43664, 20336, 14268, 49812,
                4456,  16892, 10572, 61196, 13848, 2156,  49404, 58368, 15904,
                52632, 12120, 22908, 34532, 28384, 51644, 44040, 56776, 38828,
                48308, 17232, 30128, 10348, 57364, 32184, 18112, 28288, 55168,
                47488, 35708, 61912, 40888, 18928, 65024, 22104, 53792, 24752,
                63416, 42944, 11912, 21428, 31956, 45812, 37236, 49648, 3908,
                63420, 22532, 38076, 1924,  49084, 12760, 41084, 58436, 23976,
                50440, 33676, 26404, 55076, 21468, 16604, 40712, 24664, 44156,
                53544, 9812,
            },
            {
                63780, 13892, 36560, 45760, 54096, 64468, 11884, 852,   52724,
                58504, 29408, 64404, 51000, 11628, 15324, 41212, 8680,  45220,
                57212, 12016, 49296, 17124, 35112, 52320, 24508, 55744, 60572,
                31128, 15572, 24000, 44184, 18656, 56772, 31748, 12768, 28052,
                35384, 16580, 60728, 20628, 45268, 63548, 17736, 48232, 30184,
                23304, 6908,  60612, 42148, 20868, 53280, 28076, 5168,  49024,
                34840, 19852, 58796, 47644, 32048, 63700, 52268, 28224, 58348,
                31808, 25104, 47616, 52772, 22716, 44336, 35452, 9332,  24144,
                45252, 1832,  56288, 61532, 38016, 1296,  21776, 32436, 8296,
                61344, 820,   35696, 65104, 23336, 51940, 36960, 148,   64096,
                22384, 5988,  2604,  29888, 34328, 42660, 4828,  13472, 33728,
                7624,  48932, 38936, 55128, 64344, 24404, 6892,  20520, 26644,
                40100, 55920, 8068,  31552, 59632, 20724, 5192,  30716, 37320,
                64120, 2988,  59740, 10072, 36004, 61752, 56624, 33564, 4336,
                48136, 16736,
            },
            {
                39356, 26316, 49876, 9076,  2860,  41476, 49116, 38268, 26992,
                18744, 3880,  41964, 22152, 600,   60700, 23604, 50248, 3596,
                25988, 39488, 53748, 23052, 9124,  42820, 48040, 5660,  12312,
                34620, 47412, 38584, 3500,  35548, 60380, 96,    44996, 65160,
                3836,  25800, 52508, 2120,  40548, 34756, 24760, 2952,  39452,
                65492, 12952, 45768, 25516, 2380,  46936, 17512, 61832, 24848,
                14656, 64748, 26904, 40372, 6204,  35516, 676,   41800, 12004,
                37080, 55436, 1280,  41008, 30372, 64520, 18136, 54528, 38908,
                33204, 27368, 17456, 41996, 10392, 47824, 63404, 13224, 24516,
                52556, 28096, 46240, 14836, 43160, 4576,  59772, 13776, 30924,
                43368, 50152, 26276, 55484, 15564, 51712, 47716, 38004, 57524,
                29656, 16408, 26292, 888,   9588,  41808, 58684, 54020, 34380,
                11768, 29432, 51616, 43988, 25528, 34564, 52664, 47648, 8556,
                14476, 43232, 18648, 49140, 3748,  45368, 28268, 11212, 20128,
                35388, 23472,
            },
            {
                52860, 6004,  60780, 19276, 29676, 24776, 17296, 7972,  61264,
                13804, 47868, 36340, 32052, 54468, 28040, 37500, 65128, 32744,
                20988, 60936, 204,   45724, 27708, 13612, 37216, 18256, 41256,
                63056, 20940, 7340,  55044, 49152, 15064, 40324, 48264, 19476,
                55700, 38864, 14064, 29264, 6304,  55236, 50296, 9212,  19624,
                36092, 55396, 32196, 15632, 39816, 56232, 11660, 38544, 43480,
                9252,  46016, 2344,  13316, 54372, 18024, 26180, 61668, 46124,
                5952,  63484, 19416, 34060, 8544,  5240,  26876, 12880, 62764,
                4416,  50700, 58196, 30756, 5724,  53664, 16080, 36412, 43292,
                57696, 6216,  39748, 20772, 8620,  26700, 47828, 41376, 9792,
                16904, 60092, 39168, 11748, 58960, 1000,  27064, 18116, 61644,
                4352,  52468, 59532, 36048, 48068, 17780, 31036, 14484, 44756,
                1248,  60436, 19460, 15120, 62364, 10236, 17912, 56952, 22380,
                28728, 53932, 32284, 23452, 13552, 38292, 7112,  50232, 61216,
                58012, 956,
            },
            {
                12296, 31368, 34572, 58556, 44512, 56580, 36164, 32572, 42968,
                55060, 24136, 59804, 6012,  19688, 43968, 6720,  17244, 10684,
                43164, 14568, 30520, 56360, 63500, 2908,  59108, 29180, 50648,
                2048,  27072, 59600, 12064, 30196, 25620, 21940, 5652,  33356,
                10936, 43568, 49612, 62660, 22128, 12032, 59360, 42608, 52324,
                27292, 492,   10196, 50772, 63936, 30160, 7128,  33188, 22144,
                57008, 30768, 51192, 37400, 60344, 23244, 56240, 8856,  16248,
                22120, 28684, 51572, 14908, 59936, 56332, 48880, 42500, 20620,
                46912, 8232,  13772, 22304, 44936, 26116, 59808, 29904, 4384,
                18944, 12396, 50180, 60460, 53956, 33244, 19340, 56604, 25076,
                53072, 32680, 6940,  20932, 24088, 44672, 31912, 10012, 41180,
                22664, 43736, 13860, 20428, 61760, 27620, 3700,  50800, 65332,
                23744, 47540, 5544,  36544, 3340,  42688, 38944, 1324,  45280,
                61272, 5616,  40196, 57880, 26808, 65524, 54664, 15512, 40964,
                30036, 45136,
            },
            {
                64728, 42692, 21772, 15372, 10880, 5096,  63488, 50828, 2560,
                30696, 9876,  16328, 38952, 49336, 13096, 55640, 47400, 58312,
                51344, 35776, 6476,  38768, 20404, 33116, 52828, 8180,  22608,
                57048, 45804, 17424, 42132, 64232, 9220,  54204, 61568, 28740,
                58680, 23712, 8128,  32628, 47328, 37624, 30828, 14888, 5300,
                46868, 62156, 22524, 36940, 4268,  20312, 49368, 60968, 52584,
                5596,  16672, 42168, 21160, 3416,  48908, 29808, 39868, 33148,
                48576, 42772, 11168, 38800, 45216, 22368, 36856, 596,   30076,
                59572, 35636, 39900, 64388, 33692, 19484, 2780,  40800, 47360,
                64132, 34208, 24192, 30536, 1496,  63844, 38972, 5176,  35844,
                1732,  46136, 64624, 48744, 36348, 62756, 5736,  55600, 50188,
                35240, 2644,  31672, 45344, 5856,  39444, 56424, 10096, 36936,
                16940, 40944, 57372, 50052, 28872, 54268, 63788, 26656, 16316,
                34964, 12140, 51784, 152,   42700, 18340, 32784, 3392,  25568,
                9180,  18580,
            },
            {
                27208, 3996,  51416, 37768, 48124, 28312, 22444, 13196, 19892,
                62084, 46656, 53000, 1904,  63604, 24768, 34708, 30008, 2064,
                26928, 62740, 17864, 10004, 48792, 25292, 15952, 44492, 39292,
                10404, 31376, 36836, 51660, 3020,  34344, 38308, 50508, 16480,
                1912,  35980, 18820, 57280, 996,   17056, 61332, 25244, 34092,
                57576, 40412, 17344, 44288, 58304, 25960, 14480, 40912, 1796,
                36168, 25520, 62464, 10984, 33980, 44676, 13904, 4316,  59312,
                53732, 2576,  62592, 25784, 4196,  31812, 16496, 53744, 11312,
                25508, 18540, 54624, 1916,  49220, 11832, 56628, 51672, 22268,
                9532,  55344, 37956, 16808, 45208, 11248, 15128, 49048, 61840,
                20348, 13548, 28636, 9024,  53500, 14988, 39312, 20232, 12600,
                64608, 27400, 57248, 10932, 52800, 33696, 22020, 43252, 29980,
                7252,  26520, 33208, 20552, 11672, 22776, 32440, 6824,  51080,
                38252, 20756, 62856, 29940, 8812,  48764, 21876, 46004, 37364,
                56544, 48984,
            },
            {
                35544, 54776, 8132,  61964, 216,   55476, 39668, 45508, 6588,
                26352, 40684, 33888, 28748, 57444, 9440,  41532, 21496, 5732,
                40476, 23856, 46260, 54852, 41464, 60188, 4076,  65364, 34172,
                54052, 14648, 6240,  24304, 18760, 45088, 13500, 7032,  42040,
                46808, 63432, 51944, 40656, 27160, 45060, 4128,  53292, 20704,
                11744, 7372,  28016, 52740, 33972, 8964,  46096, 28688, 18940,
                59060, 47920, 53684, 28272, 6600,  64912, 51760, 36468, 19984,
                24476, 8204,  35312, 18752, 50568, 58204, 9096,  40572, 62180,
                51220, 5280,  43824, 27664, 60364, 38820, 7168,  35336, 15520,
                27088, 3252,  42832, 7540,  58656, 27824, 51692, 23672, 30252,
                57468, 38392, 42200, 18520, 3312,  29944, 60452, 25224, 46772,
                8064,  17840, 49312, 23512, 63368, 16232, 48,    48572, 61168,
                53496, 13452, 63396, 7984,  46280, 58264, 13876, 48216, 60076,
                9640,  25492, 47060, 16592, 36788, 59676, 11736, 53188, 63112,
                1720,  14188,
            },
            {
                59144, 23404, 17580, 41656, 25652, 33600, 16356, 52300, 35916,
                56436, 12056, 21652, 7516,  18100, 45740, 62416, 15220, 52924,
                59780, 32580, 13200, 1620,  29532, 35880, 12480, 19888, 26124,
                464,   58756, 63200, 47856, 56064, 27580, 58172, 21248, 31072,
                25168, 4760,  14676, 11020, 21384, 63864, 9056,  38176, 43956,
                49576, 31752, 59904, 13240, 1300,  38736, 55608, 65172, 12068,
                32516, 7944,  15048, 39340, 57900, 17396, 25816, 41980, 12180,
                61348, 31332, 56020, 47296, 12452, 65348, 27296, 34144, 45788,
                22636, 15384, 32144, 9752,  16948, 24128, 29392, 45372, 59204,
                32476, 62720, 48444, 53540, 21996, 35204, 41628, 6376,  10092,
                44612, 4680,  25452, 56040, 34876, 51264, 43200, 644,   33532,
                53020, 40152, 36516, 6876,  42256, 26028, 12400, 57176, 19616,
                35104, 2016,  42536, 51576, 37196, 2520,  40988, 18528, 29028,
                1992,  56784, 43640, 4364,  54296, 34128, 5768,  24288, 31312,
                20100, 39988,
            },
            {
                11100, 32920, 29492, 46088, 12364, 64156, 8844,  60244, 30012,
                1204,  64840, 44348, 51124, 38020, 31620, 2972,  48712, 37356,
                19316, 7532,  50352, 64116, 21988, 5708,  57460, 46488, 50040,
                41716, 29128, 21784, 39796, 1596,  32416, 62436, 11176, 52868,
                59844, 39156, 28380, 55996, 49432, 34680, 54928, 29304, 17756,
                64832, 5040,  23932, 47432, 62488, 16788, 23464, 4052,  50856,
                44516, 24184, 920,   46756, 20732, 31568, 9356,  56836, 212,
                45392, 16180, 41124, 29252, 1384,  42976, 20360, 7160,  3168,
                38040, 56132, 63084, 36332, 47304, 53316, 64988, 12564, 1072,
                39968, 19936, 13540, 29568, 4004,  17856, 63288, 55144, 32860,
                16756, 59456, 47440, 62284, 11876, 22680, 7396,  58644, 28520,
                14756, 60792, 2256,  29588, 59716, 46752, 38928, 28696, 4860,
                45172, 23344, 59152, 17564, 30616, 24872, 65108, 44284, 53252,
                33688, 39652, 14152, 23384, 64048, 27932, 41192, 14952, 43960,
                7492,  47056,
            },
            {
                60960, 2228,  50304, 57856, 20672, 3192,  43280, 23732, 18824,
                48228, 15076, 25216, 4220,  61316, 12716, 56796, 25684, 10544,
                55196, 44024, 27196, 15488, 42852, 52144, 37924, 31608, 7632,
                18288, 12040, 35732, 8752,  15848, 43520, 5208,  49160, 17920,
                34860, 8708,  45984, 31788, 2796,  41764, 23584, 13780, 748,
                56464, 36112, 41284, 19628, 29844, 53740, 35132, 42320, 20976,
                38048, 56456, 63704, 35344, 42644, 3144,  62160, 47996, 28140,
                37752, 52292, 5268,  22936, 36624, 14368, 54900, 48044, 59388,
                29144, 12816, 49772, 0,     20952, 4620,  41960, 18500, 50324,
                25132, 57584, 10104, 36876, 59668, 46692, 380,   12632, 48524,
                36576, 21516, 1316,  31940, 16004, 45352, 37684, 18628, 55084,
                22328, 45628, 11432, 19904, 54416, 34788, 8256,  64680, 50972,
                14808, 38540, 27556, 10044, 56328, 4900,  35400, 10988, 21576,
                5356,  61828, 30952, 50668, 19696, 1064,  57124, 49788, 62188,
                28616, 52948,
            },
            {
                37724, 16048, 6808,  35720, 53868, 31336, 49880, 37416, 5156,
                41484, 58956, 35324, 54180, 27972, 20788, 43184, 35084, 30624,
                1060,  61148, 38596, 33892, 58864, 9228,  23872, 15796, 62656,
                53396, 45320, 61120, 50672, 54904, 25940, 36460, 40500, 23768,
                508,   65288, 20188, 58164, 16668, 6460,  60856, 46484, 39724,
                26308, 9928,  51244, 3052,  43700, 11044, 6164,  60724, 30944,
                9732,  17796, 28956, 11824, 54400, 50736, 23188, 33784, 13696,
                21440, 63516, 10380, 57620, 61160, 50880, 24816, 32300, 17408,
                41168, 8028,  25824, 43028, 57936, 33460, 9104,  55652, 36124,
                6564,  45616, 51836, 42372, 25928, 31868, 39484, 24772, 61460,
                28108, 51356, 9384,  40716, 27624, 49780, 65380, 10692, 41776,
                4892,  63784, 32156, 51228, 14204, 3420,  18356, 24596, 32632,
                55416, 6820,  47668, 62328, 40168, 15132, 50008, 27684, 58780,
                16872, 47640, 11792, 8116,  37520, 45844, 10300, 17828, 34404,
                4284,  22308,
            },
            {
                26772, 65252, 40940, 24780, 14772, 59760, 10972, 27348, 55944,
                8112,  31172, 10304, 40140, 16596, 50204, 6232,  65224, 23096,
                16972, 47972, 11428, 19792, 2472,  47568, 28320, 56196, 3028,
                33604, 5804,  23624, 30864, 20004, 64424, 10080, 57152, 14932,
                54276, 42344, 12808, 37532, 26132, 52436, 10820, 33404, 58340,
                21304, 53984, 15300, 33580, 58672, 48480, 27388, 14544, 52540,
                2236,  59196, 49456, 6000,  26488, 15748, 5048,  40452, 55388,
                7232,  48828, 26820, 18172, 30588, 39060, 2184,  11412, 63980,
                51864, 19640, 61116, 29948, 14688, 39304, 22760, 27472, 60324,
                30916, 16864, 22588, 4788,  64472, 14864, 19992, 52464, 6916,
                43108, 14172, 63168, 53840, 5592,  57336, 2844,  31032, 35792,
                49088, 25260, 38348, 42872, 22704, 58428, 48628, 40856, 10896,
                60404, 21532, 31564, 824,   52624, 19848, 45376, 7256,  54672,
                36280, 24224, 42940, 59132, 52888, 32296, 26284, 38932, 58660,
                13400, 43100,
            },
            {
                18168, 55820, 9580,  45040, 4504,  38868, 46528, 16160, 61936,
                51976, 19620, 23504, 47112, 672,   60028, 9340,  46196, 40280,
                56472, 5312,  25276, 53936, 31012, 63732, 12996, 39620, 43532,
                26680, 58976, 38724, 3468,  13560, 46520, 2428,  29908, 48372,
                26988, 32492, 7216,  47352, 61976, 30072, 19004, 49112, 5740,
                28768, 45228, 63596, 7692,  24804, 20284, 63116, 39224, 45260,
                24944, 34656, 41056, 20024, 62800, 38012, 45728, 60132, 18880,
                32024, 43628, 35220, 4056,  46496, 8576,  43996, 56248, 36036,
                23476, 2640,  45404, 53588, 6336,  63708, 48664, 43656, 14352,
                2712,  62104, 34280, 8700,  54500, 47784, 9836,  58140, 35360,
                3768,  18692, 38428, 23888, 34256, 20020, 26144, 14400, 52412,
                8516,  16696, 1404,  55924, 9520,  30988, 62556, 44552, 1716,
                36580, 17980, 43580, 12232, 34240, 23760, 60772, 30448, 41100,
                564,   63764, 27120, 3552,  15964, 21840, 64540, 2628,  54612,
                8976,  51240,
            },
            {
                924,   33184, 49040, 19876, 29080, 63996, 572,   21936, 33176,
                44076, 3820,  64240, 55376, 32508, 37316, 29400, 18896, 13404,
                50608, 36636, 61928, 40680, 44864, 6028,  18592, 51212, 21132,
                11340, 49284, 16460, 42784, 52076, 33296, 60104, 21824, 43804,
                4348,  58848, 50780, 21340, 1288,  43924, 38272, 64244, 14244,
                2188,  31876, 18304, 40512, 54856, 36664, 3868,  17404, 55720,
                7780,  47272, 12976, 31420, 55988, 9780,  29488, 52460, 11880,
                1556,  56868, 14944, 64548, 54012, 21972, 16448, 28324, 5052,
                47260, 33672, 10088, 37556, 18084, 32004, 1600,  10724, 53456,
                47448, 40632, 56616, 28476, 37476, 2244,  44152, 29536, 22928,
                55588, 30780, 44812, 59292, 11612, 47896, 42816, 61800, 21492,
                58788, 28808, 61476, 46960, 36152, 26768, 5884,  16312, 28988,
                54024, 50496, 63224, 26532, 55532, 3412,  38256, 9892,  14324,
                51260, 18728, 33364, 39964, 55696, 6804,  48424, 41864, 24444,
                31104, 46376,
            },
            {
                28200, 62288, 12392, 58616, 34976, 53308, 25608, 50764, 12344,
                36860, 28188, 14784, 42696, 11996, 25408, 52896, 58248, 2212,
                32852, 28664, 8572,  14844, 22732, 35948, 60024, 32612, 1480,
                64236, 35340, 8428,  57480, 27212, 39828, 7156,  18284, 62752,
                38324, 9652,  24524, 35512, 15380, 56440, 8720,  23428, 42228,
                59400, 37092, 10852, 50056, 1412,  30452, 43340, 11860, 32716,
                64736, 22092, 60004, 784,   43936, 36028, 3364,  21888, 26204,
                61420, 38164, 23660, 41320, 12412, 33276, 58404, 61884, 40000,
                13628, 55284, 27268, 58624, 24212, 51336, 60924, 21268, 37176,
                25540, 19144, 12528, 50060, 21472, 16136, 60348, 40524, 11956,
                64952, 49056, 8340,  3324,  17200, 36656, 55152, 248,   39864,
                44740, 34308, 6468,  13620, 19288, 51824, 39476, 56416, 12864,
                24132, 3924,  8864,  41416, 47012, 17308, 64652, 48872, 21756,
                57224, 4996,  46040, 60876, 13228, 29716, 35804, 19420, 14876,
                60596, 37036,
            },
            {
                7400,  40216, 23656, 5556,  42744, 17696, 9336,  39764, 57352,
                6456,  48024, 59236, 2876,  21164, 62484, 4648,  42300, 23724,
                63336, 20096, 52472, 328,   56764, 48660, 9556,  54588, 24948,
                46228, 29712, 19008, 62200, 22684, 11104, 55156, 35260, 13344,
                52672, 17412, 63820, 40636, 53112, 4708,  27552, 34264, 51060,
                55280, 25396, 46164, 61856, 13988, 22808, 60180, 50796, 27944,
                4796,  38228, 15984, 52032, 24580, 17776, 65060, 47220, 34024,
                49892, 8808,  28240, 52080, 452,   48864, 25628, 7088,  50628,
                20640, 64916, 16120, 42628, 3960,  12192, 40128, 29572, 7124,
                65464, 32540, 3268,  42792, 63012, 33620, 26516, 50960, 764,
                15448, 33896, 26360, 52120, 29312, 64220, 7172,  32232, 10520,
                18460, 53640, 23952, 41300, 64796, 2760,  22016, 60020, 35012,
                48204, 38792, 30880, 14448, 59452, 6784,  32984, 25252, 43680,
                35216, 29064, 9128,  22924, 50516, 43612, 10748, 62788, 52928,
                4840,  21060,
            },
            {
                57820, 51796, 15740, 31452, 55244, 3372,  45908, 30776, 62964,
                19052, 24280, 34156, 51664, 39664, 45864, 15880, 35528, 49128,
                11040, 45580, 59308, 42072, 25956, 29820, 16960, 4376,  40644,
                13792, 56280, 4904,  37204, 232,   45048, 49544, 26020, 30800,
                1864,  46020, 28976, 12080, 31972, 48404, 62420, 11404, 16688,
                4224,  20380, 6748,  28292, 34940, 56384, 46604, 9068,  19196,
                53988, 42136, 29584, 48940, 7384,  57140, 13204, 41616, 5488,
                16524, 44920, 19652, 6348,  60504, 36492, 18588, 42768, 30068,
                1148,  38344, 7716,  49752, 35168, 45876, 57664, 15208, 55412,
                44392, 9656,  58360, 24940, 5180,  53560, 7904,  18948, 46980,
                39064, 21364, 61152, 41260, 46168, 13980, 23532, 51024, 27744,
                47672, 3776,  56988, 30096, 49732, 10996, 33072, 45720, 8172,
                64256, 19648, 56944, 22632, 36952, 28648, 52832, 2220,  12076,
                62120, 54248, 16764, 38104, 3000,  58800, 25308, 1448,  33560,
                49384, 44072,
            },
            {
                26128, 10100, 47044, 63616, 37804, 21580, 60272, 15200, 1664,
                42084, 54036, 7832,  17364, 30336, 10084, 57316, 6852,  27164,
                39164, 4192,  16396, 33928, 12152, 65020, 44188, 36736, 60644,
                21800, 50364, 43420, 53516, 32128, 15368, 58364, 5796,  61356,
                42468, 20828, 56748, 3152,  59856, 18332, 22084, 45284, 40092,
                30644, 65500, 52384, 42548, 17020, 5288,  39824, 25708, 33924,
                58068, 2364,  11164, 61208, 33268, 27688, 38644, 53992, 59064,
                29764, 62368, 55824, 39656, 30704, 45744, 14692, 10708, 54168,
                33032, 60160, 22612, 29068, 63228, 18828, 26344, 33872, 372,
                51948, 17252, 36496, 47964, 14108, 38264, 57080, 31848, 62412,
                6148,  54440, 10680, 1568,  56896, 19668, 39244, 59580, 12784,
                63348, 38000, 15784, 7640,  44028, 25972, 53344, 18076, 288,
                27360, 42764, 4916,  51176, 11080, 45188, 19444, 57752, 38640,
                7024,  47772, 26544, 65528, 32636, 18232, 46180, 39932, 56680,
                12792, 17336,
            },
            {
                61636, 34204, 4472,  27864, 12960, 50504, 26320, 35364, 49212,
                28844, 11492, 37612, 61312, 25748, 50168, 33228, 64672, 18468,
                54940, 31548, 21944, 51548, 38568, 6956,  20264, 52748, 27104,
                33556, 7752,  12316, 28232, 64536, 19832, 40540, 23508, 37000,
                48716, 8376,  34604, 25892, 41904, 6108,  35912, 56948, 76,
                48196, 13140, 38212, 9392,  61112, 21356, 664,   64136, 14872,
                44912, 23300, 36684, 18992, 46644, 4356,  20680, 10064, 23560,
                1860,  13884, 34848, 9548,  22488, 3328,  56768, 62504, 24400,
                48036, 44580, 13244, 2704,  53372, 9724,  5548,  42048, 49288,
                22780, 27736, 61236, 30692, 20780, 45292, 11268, 23824, 28252,
                43852, 36256, 17480, 25192, 33400, 5152,  43784, 35656, 2192,
                25268, 31744, 60148, 20412, 36268, 62020, 14504, 40328, 31468,
                54140, 15876, 61164, 33288, 1080,  63528, 15764, 42224, 31008,
                20888, 14496, 41664, 51772, 5976,  54800, 28572, 9224,  22424,
                29928, 38576,
            },
            {
                472,   54536, 20572, 44772, 57188, 7584,  41160, 10376, 55628,
                22224, 65504, 44388, 80,    56096, 5472,  22420, 13584, 43856,
                60316, 8184,  47884, 61820, 3160,  57560, 47380, 10920, 908,
                63176, 18060, 39424, 24344, 47736, 9380,  3092,  54044, 11556,
                16540, 65124, 51744, 14684, 47004, 53424, 28612, 8304,  60392,
                23236, 33816, 57636, 24992, 31732, 54236, 47872, 36284, 50444,
                6504,  62520, 53136, 14144, 40508, 63140, 50948, 31716, 37128,
                43544, 53228, 25448, 47776, 65304, 51496, 27760, 37820, 4244,
                9016,  19280, 56484, 39556, 32328, 47544, 60476, 20068, 63840,
                38816, 12128, 6676,  53036, 1092,  64412, 41652, 2812,  58784,
                13912, 50476, 59776, 30360, 48756, 63612, 9408,  54288, 17384,
                45676, 52340, 10292, 48808, 1836,  56640, 5836,  23828, 59188,
                47216, 12024, 24364, 48724, 39776, 54796, 25968, 4248,  50000,
                59896, 34636, 280,   11472, 21488, 36828, 14776, 47740, 64260,
                6268,  42996,
            },
            {
                24496, 49184, 36672, 16816, 1960,  31948, 62396, 18496, 5664,
                46544, 32708, 14180, 20048, 41524, 47684, 36916, 53460, 29168,
                936,   24796, 36536, 14560, 28376, 23816, 31936, 15452, 42436,
                48860, 56964, 2764,  59800, 34480, 52044, 29812, 60592, 33136,
                27332, 992,   38340, 23820, 10596, 63248, 39280, 15616, 26696,
                43616, 18924, 2248,  50100, 44096, 12536, 29276, 18016, 10488,
                31456, 27344, 42948, 8668,  25240, 224,   58496, 17048, 48604,
                63768, 4256,  40564, 17324, 12380, 34096, 20516, 41684, 16256,
                52424, 35080, 26984, 59520, 16780, 24480, 36612, 13492, 31252,
                3424,  46580, 41144, 56652, 15836, 35144, 51112, 17712, 33060,
                4436,  22656, 7704,  40744, 53012, 15044, 28316, 21208, 58628,
                41596, 4748,  22884, 39924, 33936, 28300, 43312, 51328, 9524,
                34492, 38404, 6848,  28064, 20260, 8224,  35716, 62340, 10108,
                24048, 44804, 57120, 30048, 62392, 43264, 59268, 3304,  51396,
                32348, 58120,
            },
            {
                14532, 8104,  29720, 64944, 42592, 53332, 24260, 37404, 59872,
                2416,  26576, 52784, 58816, 29544, 16708, 3484,  40052, 11200,
                49688, 41880, 56540, 10192, 45528, 40304, 54072, 59396, 35824,
                29744, 22276, 45764, 15948, 6444,  42208, 14420, 45940, 18660,
                55624, 42992, 58608, 31480, 4948,  19636, 32760, 55140, 11740,
                52156, 62348, 6392,  15972, 37272, 6884,  59044, 41424, 56496,
                21200, 3856,  59568, 34932, 55024, 29444, 44688, 12920, 6856,
                27584, 19968, 32408, 57360, 1132,  44424, 7044,  59888, 31504,
                64064, 43132, 5880,  11220, 45104, 1420,  52372, 56164, 8904,
                58912, 18488, 33096, 23172, 28952, 8292,  26036, 54316, 47424,
                37848, 64576, 45384, 12284, 3236,  23756, 47300, 34508, 7776,
                29684, 14628, 65132, 54884, 17788, 12840, 63760, 21052, 2936,
                18624, 55256, 65268, 43796, 58116, 12788, 46008, 29440, 53508,
                17084, 4776,  37992, 48904, 24976, 7080,  17532, 26908, 35820,
                11504, 19240,
            },
            {
                52716, 40436, 59120, 11240, 22196, 48128, 13996, 29240, 50932,
                16564, 39948, 6696,  35552, 9500,  63892, 25208, 59124, 20384,
                62300, 17476, 31068, 21028, 64140, 2092,  18736, 7836,  25380,
                4328,  10636, 55016, 36920, 26868, 63316, 22436, 39012, 4772,
                47904, 7728,  21336, 49776, 60956, 45332, 1504,  48972, 36652,
                4028,  40936, 28220, 55784, 63160, 22528, 26260, 2616,  52040,
                38580, 46924, 16020, 49420, 19884, 5992,  39200, 22892, 35492,
                60060, 50684, 8652,  61452, 29556, 49740, 54908, 25420, 13504,
                176,   23120, 50824, 30416, 62872, 40588, 21920, 28556, 44220,
                25328, 50848, 62944, 10768, 48992, 38964, 61716, 21484, 10620,
                56924, 29140, 19740, 35320, 61632, 38508, 57396, 1124,  62448,
                50720, 37116, 44892, 26536, 8024,  47944, 31120, 37072, 60080,
                45700, 29804, 1876,  16512, 32412, 52020, 22404, 2584,  40380,
                33376, 63988, 19228, 13456, 53164, 33548, 40900, 55844, 21176,
                45036, 62560,
            },
            {
                4676,  47116, 25480, 34644, 6644,  39196, 3112,  57512, 33880,
                42880, 63196, 12556, 23220, 44592, 51536, 33004, 7660,  45928,
                34664, 4116,  52656, 6264,  34988, 49744, 12512, 62092, 38588,
                51364, 65412, 31704, 19372, 50512, 2144,  58064, 10252, 25556,
                62936, 30316, 12244, 35796, 17016, 42164, 24904, 64816, 21864,
                30920, 47008, 20556, 34592, 48892, 9948,  45800, 60776, 33664,
                13084, 63804, 24112, 10924, 32448, 64972, 51952, 56596, 10320,
                42240, 15436, 46140, 23408, 36184, 18912, 10060, 39008, 48156,
                57196, 36796, 19772, 54732, 15036, 4600,  49372, 7164,  35608,
                15464, 37480, 1840,  43024, 58156, 3528,  44508, 5972,  15356,
                41852, 316,   25736, 55780, 16436, 31060, 10464, 40872, 25712,
                18220, 11760, 32696, 504,   58688, 41876, 4528,  50336, 26336,
                13712, 40956, 23156, 49704, 5600,  37544, 59412, 15104, 47888,
                55648, 27988, 8568,  45704, 2504,  60976, 10152, 49436, 1816,
                39324, 28412,
            },
            {
                32956, 18428, 1336,  55320, 61008, 19524, 44644, 11604, 7656,
                20444, 54712, 30664, 49628, 1308,  57064, 14792, 28480, 53972,
                12848, 26376, 38900, 58896, 42980, 27032, 32620, 46592, 20688,
                44120, 13932, 41040, 8092,  44736, 13036, 33768, 52824, 36372,
                41292, 15576, 54608, 2572,  57428, 29372, 6900,  14252, 56412,
                10280, 59692, 7736,  13520, 40212, 31816, 16364, 5712,  19704,
                29004, 976,   40788, 58336, 45664, 2488,  17892, 26416, 31904,
                1432,  54136, 38276, 5404,  14024, 42520, 63356, 3360,  28564,
                17120, 46052, 9088,  38572, 26688, 34448, 60588, 18108, 64384,
                53760, 5692,  21568, 30656, 14228, 18872, 27628, 36000, 59680,
                31784, 48804, 9272,  51216, 5624,  46064, 20120, 54088, 48092,
                3944,  55756, 61388, 19928, 52476, 24196, 16772, 10720, 56520,
                7352,  62176, 35856, 11012, 60868, 18932, 42400, 26120, 11388,
                6104,  21124, 60124, 39736, 22792, 28932, 14428, 64896, 24272,
                8760,  57520,
            },
            {
                12124, 51232, 44068, 15408, 30524, 52152, 64112, 27764, 48584,
                24672, 4616,  60536, 37168, 19044, 42236, 21836, 39436, 1980,
                65200, 48260, 15752, 9608,  22828, 55296, 16404, 5332,  56736,
                28436, 1136,  24468, 60740, 56480, 29528, 48884, 19880, 608,
                59544, 23252, 51140, 26852, 38148, 9600,  51544, 40012, 34968,
                17556, 43212, 24640, 51296, 536,   64740, 54464, 43000, 50212,
                36884, 55528, 7848,  27788, 36080, 14200, 42920, 62060, 47912,
                20752, 29040, 64480, 25840, 56892, 51360, 22012, 34560, 58952,
                6116,  32252, 64532, 1948,  58560, 46888, 10356, 31900, 41496,
                12328, 47276, 61424, 55548, 34420, 52532, 65136, 46972, 23048,
                53512, 17992, 39092, 60580, 42892, 27192, 63836, 13436, 33336,
                22748, 39580, 29036, 9952,  38368, 34296, 62968, 44140, 21808,
                33352, 52376, 46564, 27432, 54640, 30848, 1176,  64588, 50956,
                37032, 43188, 31624, 51300, 57272, 36140, 47336, 31872, 54436,
                16300, 36732,
            },
            {
                22604, 63144, 27308, 37920, 9164,  23288, 35736, 17376, 59584,
                38612, 46448, 15560, 27480, 10892, 62808, 5360,  35392, 57812,
                23400, 44000, 29996, 61600, 50704, 344,   37808, 60268, 9452,
                52700, 17896, 33232, 37472, 5428,  23044, 16676, 64516, 8660,
                32432, 43652, 4520,  46476, 62500, 19840, 58868, 47408, 26424,
                2460,  53796, 62100, 29480, 19100, 37616, 27052, 22796, 9144,
                62196, 44396, 18072, 53092, 22324, 50276, 9232,  37744, 5000,
                59404, 11936, 41332, 2848,  32944, 7636,  45688, 11376, 52908,
                40832, 24936, 49904, 12504, 41884, 21032, 24560, 57012, 420,
                29652, 26232, 39516, 7724,  24352, 11256, 40368, 1788,  12984,
                7404,  63716, 33780, 13992, 21948, 1592,  36120, 8208,  44092,
                60112, 6668,  49496, 45672, 14048, 3176,  53872, 39456, 29388,
                1312,  18452, 15016, 3808,  39404, 9120,  45836, 34260, 23360,
                56684, 15816, 684,   11108, 17352, 3716,  20072, 6212,  41796,
                46156, 60520,
            },
            {
                7356,  41240, 2992,  58620, 49624, 5120,  41860, 408,   53716,
                10260, 33132, 2808,  56420, 48752, 31372, 45576, 51272, 18308,
                8608,  32864, 3672,  19720, 13744, 41308, 30540, 24792, 34584,
                42336, 62896, 48592, 54428, 11204, 42664, 38852, 46232, 26620,
                55200, 11060, 18756, 34400, 13652, 31728, 23072, 5720,  63572,
                32116, 37952, 6112,  45604, 59004, 4308,  12164, 57720, 31352,
                14856, 25468, 4360,  60200, 31500, 3532,  55288, 24720, 15864,
                34580, 52352, 18664, 48928, 60632, 30156, 17676, 27080, 61032,
                14296, 20684, 56512, 29152, 16704, 53040, 5388,  44876, 50760,
                19424, 59564, 15768, 45520, 50048, 5184,  32532, 57068, 29516,
                42540, 24732, 4288,  28704, 54652, 49424, 58252, 17596, 52256,
                27848, 15244, 64368, 21892, 31476, 60648, 26800, 5908,  48468,
                64148, 58272, 43320, 24764, 61784, 20924, 57552, 13968, 5020,
                29936, 48272, 62052, 26796, 44588, 63028, 37876, 52400, 26396,
                184,   30240,
            },
            {
                48224, 34300, 20996, 13760, 45884, 32512, 57052, 14260, 29484,
                43904, 20960, 64804, 40232, 7544,  24388, 14376, 61072, 11592,
                40532, 54560, 36556, 57276, 45900, 7504,  64324, 47612, 12104,
                4392,  22256, 15080, 2904,  27716, 61648, 57324, 3904,  14156,
                37296, 61400, 49188, 7152,  42584, 53648, 304,   44532, 15720,
                50320, 11092, 22492, 14304, 34104, 52992, 40604, 47464, 1612,
                34704, 49228, 38436, 11460, 46308, 62852, 28396, 41064, 57680,
                45424, 23060, 9000,  36932, 14916, 54300, 39160, 1236,  43596,
                36432, 4112,  44732, 7668,  32832, 61352, 37136, 14472, 34652,
                9324,  53244, 2708,  36196, 20748, 62032, 16952, 51348, 20136,
                45588, 58828, 48076, 37712, 10224, 40624, 31200, 25284, 3684,
                37428, 42612, 2116,  56428, 8352,  50484, 16108, 20208, 12736,
                36748, 8656,  31384, 51156, 12568, 48988, 27976, 40808, 62988,
                19376, 8316,  39260, 34348, 55428, 23636, 9628,  59384, 13484,
                19260, 56352,
            },
            {
                10916, 17136, 51912, 65164, 26252, 10824, 19400, 62760, 50228,
                5612,  25896, 52064, 17420, 35004, 54412, 480,   29520, 26256,
                49044, 6516,  63660, 24384, 28244, 53172, 21080, 17348, 59048,
                50756, 39288, 30168, 46500, 20392, 35024, 7684,  31644, 52348,
                21408, 29564, 56956, 24636, 64676, 38940, 11828, 29260, 55488,
                19536, 58108, 42108, 49396, 25676, 17460, 8020,  61656, 19492,
                54280, 64852, 21564, 41780, 16340, 34176, 19504, 12488, 6168,
                31048, 308,   62360, 42860, 24568, 5688,  47020, 65032, 23180,
                9728,  51312, 63240, 39284, 47700, 2436,  27276, 65468, 40688,
                23224, 31528, 63724, 27496, 43516, 58296, 25656, 38328, 8392,
                35396, 2216,  16424, 56368, 19564, 4920,  65320, 12464, 46948,
                58836, 18796, 34748, 25004, 41356, 36408, 58880, 44928, 55904,
                40772, 22272, 54128, 34544, 6288,  37300, 17664, 2768,  53064,
                43848, 24980, 51928, 13368, 5124,  49204, 28452, 43332, 33084,
                63972, 39252,
            },
            {
                54244, 29236, 42588, 4104,  37108, 54524, 40164, 23568, 34872,
                60720, 37756, 12348, 45120, 22228, 62404, 42360, 37608, 59052,
                20980, 44420, 16728, 1740,  39272, 10564, 32524, 2800,  36644,
                25836, 57500, 8560,  64824, 51668, 15992, 25160, 48556, 43248,
                1340,  40848, 15528, 3004,  27920, 18048, 59516, 34196, 40400,
                7648,  27508, 35964, 1252,  60456, 30548, 45164, 36632, 28212,
                12992, 5780,  30340, 8552,  58408, 1320,  53964, 49316, 65316,
                39120, 47984, 56072, 28672, 11720, 58860, 19448, 31692, 55260,
                16284, 34016, 25360, 18724, 11596, 55684, 21952, 6296,  58600,
                46540, 4508,  39004, 13764, 10228, 580,   47728, 14336, 54532,
                63052, 11888, 30836, 61892, 23880, 45168, 34552, 53788, 21592,
                9236,  29768, 54156, 12160, 47436, 700,   30020, 9916,  25524,
                4812,  28712, 15956, 8,     65096, 45072, 55480, 32216, 11188,
                35580, 60972, 2096,  31764, 21024, 40844, 16064, 7148,  50300,
                4864,  24668,
            },
            {
                2324,  61304, 8724,  22520, 60100, 30572, 6452,  47484, 8948,
                16412, 1508,  57228, 28880, 4236,  10352, 47588, 15716, 3408,
                52296, 31492, 13040, 35292, 50260, 60168, 42020, 54548, 45232,
                6152,  13292, 32208, 43860, 32,    40364, 11584, 58652, 18196,
                63940, 9904,  35508, 46264, 52200, 8824,  49344, 24104, 62632,
                3464,  46812, 65496, 9516,  54688, 21752, 4756,  51924, 24148,
                43676, 57436, 47036, 52148, 26728, 37524, 44080, 22068, 9820,
                26228, 16628, 20436, 33584, 2276,  51004, 35996, 7968,  49172,
                28584, 58488, 732,   60292, 30480, 43284, 49852, 17008, 12136,
                54864, 18504, 49604, 56056, 34192, 29320, 41400, 4172,  32088,
                22296, 26660, 52668, 42144, 7804,  50652, 16016, 12,    39188,
                63032, 48740, 5224,  61768, 17004, 22740, 64900, 33652, 50916,
                62088, 47936, 59168, 42460, 21424, 25092, 8176,  59984, 48004,
                22164, 15260, 56208, 45472, 58596, 65532, 35680, 53836, 22044,
                45656, 36904,
            },
            {
                57912, 32456, 45004, 15440, 49216, 1008,  17816, 28104, 43528,
                54880, 31544, 40596, 59728, 50328, 32672, 19584, 56144, 34412,
                9464,  25340, 62424, 56312, 19204, 6920,  23496, 14384, 27852,
                62272, 18884, 55672, 23376, 36288, 60824, 53288, 29268, 6596,
                24120, 54336, 32376, 60900, 21748, 36972, 4468,  45128, 13820,
                21136, 52600, 17112, 32660, 13544, 41728, 63880, 15700, 9976,
                39660, 2052,  17876, 22856, 63744, 4724,  15028, 29696, 60616,
                35840, 3816,  52852, 44224, 63712, 40472, 26500, 13108, 2912,
                41956, 38052, 13836, 52608, 23588, 8384,  38552, 33412, 28700,
                42840, 25036, 8036,  61620, 21320, 52184, 64072, 18676, 59748,
                49896, 39416, 5588,  36064, 13536, 28560, 61340, 43304, 26164,
                32016, 14652, 44544, 27540, 38160, 52488, 5792,  43088, 14592,
                2900,  18708, 38152, 11724, 57236, 30488, 14708, 39548, 27056,
                5640,  41788, 29044, 8912,  18152, 3296,  11632, 30328, 61720,
                14096, 18556,
            },
            {
                48440, 11772, 25504, 35188, 52720, 41204, 58588, 64492, 11448,
                49460, 19924, 23960, 8080,  14196, 26564, 65088, 5644,  40016,
                60376, 46456, 41176, 4964,  47504, 30132, 65264, 49476, 1284,
                34440, 41036, 48784, 10336, 27516, 4540,  20172, 33968, 44796,
                39000, 48168, 12872, 5336,  43144, 56860, 16776, 32064, 54544,
                37836, 29348, 43468, 24916, 38528, 2372,  56688, 33648, 49808,
                60256, 31264, 36344, 13416, 40972, 32908, 51500, 7376,  55764,
                41840, 13024, 58284, 8784,  15400, 22136, 45536, 61176, 56124,
                21740, 47308, 6364,  44956, 35640, 63400, 4156,  51892, 62024,
                1572,  36572, 30896, 15320, 44460, 5956,  24336, 36384, 9428,
                43892, 1624,  20576, 47064, 59252, 33064, 19292, 6772,  57804,
                51636, 2544,  20820, 57004, 32356, 11004, 19520, 55536, 36040,
                24108, 45600, 32472, 7244,  49464, 36084, 51748, 2272,  19032,
                63828, 50580, 37408, 24204, 48668, 39116, 25612, 56800, 900,
                41408, 27748,
            },
            {
                5580,  39940, 64164, 7240,  19768, 12888, 32712, 37988, 26064,
                4972,  62836, 34228, 46292, 53948, 36752, 44216, 23124, 29072,
                17988, 768,   21860, 27004, 12116, 34056, 16588, 38448, 59280,
                21420, 52264, 6660,  16432, 57684, 46736, 13396, 63148, 2748,
                16376, 58988, 19764, 26412, 30556, 11052, 61676, 26844, 776,
                58612, 11364, 4928,  51132, 59624, 47552, 28460, 18456, 6868,
                25624, 54988, 61916, 9064,  48776, 59112, 24728, 45904, 17968,
                32276, 50028, 23488, 38408, 31204, 5396,  52104, 17436, 33160,
                9864,  62820, 17964, 27836, 57772, 19908, 14112, 46120, 21372,
                10400, 59620, 47520, 3644,  39372, 57764, 12048, 46892, 29024,
                57256, 17432, 64768, 53528, 25100, 3100,  55408, 37828, 23364,
                9912,  36292, 41604, 7952,  59968, 49600, 39684, 26924, 61004,
                8592,  52792, 63672, 27804, 3956,  19956, 62528, 45428, 54448,
                33724, 10872, 756,   62048, 53192, 32800, 43696, 47636, 9688,
                34312, 52380,
            },
            {
                16644, 21348, 55964, 28888, 46068, 55020, 2864,  21732, 51800,
                15172, 42376, 3196,  17728, 61608, 1400,  11712, 53024, 48476,
                13756, 55232, 37740, 52092, 42760, 57544, 8672,  44632, 4768,
                11644, 29108, 37328, 64152, 42572, 31360, 50832, 37604, 55748,
                27808, 7824,  51624, 64964, 1696,  39096, 50412, 41896, 8444,
                48220, 62268, 35272, 19532, 7620,  22844, 11804, 37268, 46360,
                20844, 3756,  43376, 27948, 444,   20220, 11588, 38500, 2444,
                64272, 6552,  27200, 62008, 48292, 57076, 28876, 3732,  37096,
                25732, 53492, 31604, 3460,  11088, 42188, 32192, 54216, 26712,
                40480, 55432, 19036, 51316, 27304, 33080, 17172, 54048, 6888,
                14224, 34576, 30560, 11292, 8248,  41280, 48948, 12292, 46460,
                15928, 64704, 54620, 24876, 4608,  15456, 45912, 1188,  31040,
                12672, 40176, 21912, 16148, 55116, 41472, 8940,  13132, 29560,
                43268, 17028, 58844, 21312, 12892, 5568,  16416, 20320, 59040,
                23808, 62596,
            },
            {
                1628,  49524, 37560, 3680,  61688, 24800, 43800, 8328,  57916,
                29256, 48408, 56620, 39184, 21804, 30712, 41460, 7228,  63172,
                35664, 31216, 10156, 63832, 2916,  20936, 53340, 25700, 31884,
                55124, 46328, 24460, 2004,  18440, 8136,  25388, 21388, 10772,
                42332, 31944, 36620, 45748, 15252, 53892, 22088, 18548, 33984,
                23644, 15844, 56356, 44436, 31588, 61792, 40696, 54304, 63188,
                14396, 34368, 52844, 17132, 56852, 35520, 62876, 53608, 28488,
                21000, 46900, 36436, 1548,  19092, 12096, 39680, 64772, 44036,
                14596, 59036, 40272, 48280, 51484, 24632, 268,   64692, 16624,
                5452,  34456, 11836, 23020, 65044, 984,   42868, 62508, 25436,
                40528, 48680, 22784, 44320, 36944, 16828, 63268, 30868, 59008,
                27916, 33828, 18360, 43972, 29860, 63688, 21656, 56500, 17832,
                48704, 58032, 1772,  44376, 60576, 25536, 32844, 22832, 57412,
                6700,  25964, 35860, 46392, 30080, 55364, 63524, 37908, 7416,
                29776, 44252,
            },
            {
                11348, 59352, 31508, 15068, 9860,  39648, 17052, 60712, 33624,
                37104, 11132, 6880,  27908, 13360, 50552, 57532, 17340, 25996,
                4440,  59332, 49936, 15672, 29416, 36352, 61280, 17872, 41752,
                62924, 14716, 58740, 33716, 53176, 39532, 62304, 736,   49668,
                60072, 22940, 3604,  57716, 24840, 34844, 6184,  63644, 45840,
                28640, 39936, 3008,  26784, 14272, 52232, 828,   5496,  30468,
                42004, 10500, 23700, 48048, 30100, 44932, 4804,  15800, 42780,
                59276, 10428, 14812, 54820, 43712, 34284, 8848,  24032, 50172,
                872,   20044, 8160,  36296, 15624, 60216, 38420, 9044,  49784,
                29848, 45020, 60760, 37140, 8988,  48412, 20544, 37464, 3396,
                51504, 58744, 632,   61956, 54488, 26492, 20728, 1812,  5896,
                40732, 50144, 988,   13152, 47852, 34816, 9704,  37568, 42856,
                6524,  34076, 29212, 36708, 11048, 48484, 38608, 1392,  46804,
                64976, 39780, 52244, 8448,  42408, 3488,  27092, 50088, 14148,
                54360, 36100,
            },
            {
                26780, 42800, 23108, 53712, 34676, 50676, 27176, 46548, 208,
                19708, 64144, 24712, 44968, 60088, 35940, 9488,  32888, 46104,
                42364, 19320, 23192, 45032, 6568,  47876, 13032, 388,   49276,
                7788,  20488, 3900,  43916, 12460, 56728, 29956, 15504, 34324,
                45124, 12988, 18464, 49176, 9592,  43744, 13572, 55456, 2136,
                12088, 53304, 64844, 9148,  48528, 36024, 17948, 27216, 58144,
                50396, 38120, 65228, 6172,  12724, 40344, 26244, 8616,  34180,
                51868, 30764, 40924, 25140, 60708, 52544, 20624, 58376, 6248,
                46680, 29172, 63984, 22192, 55520, 28028, 45732, 33852, 23292,
                56808, 1944,  14980, 41512, 53628, 30212, 13136, 59788, 31964,
                10736, 18392, 28152, 5520,  13648, 50576, 34732, 44548, 53236,
                22220, 8972,  60308, 38092, 53976, 3840,  27452, 61336, 53248,
                25288, 65456, 14584, 51492, 4552,  18208, 59540, 53400, 15652,
                20280, 3804,  32060, 14948, 24448, 60660, 33884, 40840, 2512,
                46432, 19356,
            },
        },
        {
            {
                21788, 57412, 38504, 2064,  29792, 20584, 32764, 1204,  10212,
                34952, 59192, 31200, 21016, 57096, 13164, 4324,  63904, 54436,
                15256, 40440, 27956, 8892,  65348, 4116,  15912, 11192, 50840,
                1548,  56500, 40460, 33852, 5556,  50872, 37300, 40628, 3524,
                54220, 11100, 20856, 7216,  30088, 48612, 23244, 44292, 26984,
                47204, 63544, 24268, 29876, 15232, 61484, 18028, 63560, 32240,
                46688, 60352, 13776, 33456, 62300, 55548, 2932,  61192, 45284,
                59220, 27248, 2024,  40224, 11076, 60184, 6420,  53284, 57956,
                24936, 13504, 63016, 10868, 29024, 43452, 14900, 60576, 46348,
                56300, 32412, 7012,  23328, 65036, 44512, 29424, 59008, 35180,
                62700, 9580,  43092, 15660, 45840, 31772, 52620, 34924, 43588,
                16412, 55188, 32908, 62232, 18024, 10440, 64936, 976,   55536,
                12384, 19388, 57416, 1368,  17228, 62072, 48080, 55720, 10176,
                39632, 32112, 35548, 9380,  45252, 39856, 48768, 59356, 55176,
                9920,  28628,
            },
            {
                14852, 6104,  43592, 25060, 49728, 7516,  45024, 51548, 18064,
                41268, 14892, 50160, 27640, 42852, 52832, 45964, 23528, 10988,
                33112, 51780, 21100, 56132, 36232, 49324, 41016, 20572, 32572,
                36880, 28160, 15204, 21252, 62068, 512,   18940, 63364, 27872,
                60084, 34900, 43312, 58168, 13956, 37388, 4348,  34868, 12388,
                6848,  16924, 42912, 8836,  34000, 940,   41748, 51564, 3884,
                22200, 56400, 496,   51464, 9948,  31148, 20272, 15704, 10956,
                41276, 18112, 47640, 62256, 20752, 45076, 30008, 18036, 43272,
                3812,  59700, 38964, 16940, 49180, 5580,  37472, 25656, 11696,
                28680, 16440, 36980, 60160, 13804, 33896, 2468,  11080, 39192,
                26248, 53600, 23240, 51232, 38624, 11988, 64152, 6344,  27844,
                8864,  39296, 1676,  51572, 7188,  34916, 43192, 22772, 33284,
                6808,  42908, 63524, 35120, 51212, 11932, 41684, 24356, 27948,
                64160, 6568,  49136, 17264, 29312, 63052, 20864, 32220, 960,
                45916, 37508,
            },
            {
                31128, 60704, 34592, 10696, 63492, 40240, 57688, 27108, 62172,
                24348, 3660,  11452, 37160, 8144,  16740, 35916, 29280, 48692,
                62180, 6560,  17120, 42592, 2392,  24780, 30052, 53488, 63972,
                5044,  23628, 60464, 45684, 29920, 55860, 25228, 47084, 7668,
                15380, 48456, 996,   26164, 51476, 9128,  60412, 19332, 50576,
                54272, 37128, 56872, 59956, 50372, 27240, 55316, 13420, 38184,
                29212, 44420, 17136, 39548, 22800, 43836, 53288, 37280, 25124,
                56484, 30400, 13212, 34724, 55800, 944,   26076, 39884, 10044,
                20964, 51104, 27552, 1536,  23008, 65296, 32656, 53720, 100,
                63512, 42892, 49500, 3772,  18572, 56760, 50312, 16196, 46236,
                19220, 3260,  33676, 59896, 20936, 992,   41340, 23700, 60700,
                47860, 57352, 23260, 28432, 59084, 48124, 15584, 53252, 60516,
                50188, 27500, 14464, 22484, 44504, 30072, 6136,  36204, 46916,
                14808, 56508, 23464, 52756, 2296,  36772, 15384, 11480, 25928,
                64868, 51300,
            },
            {
                23768, 53236, 19608, 55552, 16668, 35920, 13336, 4496,  37712,
                54024, 44612, 64548, 47312, 58620, 3092,  60896, 20492, 668,
                38556, 26188, 47260, 31364, 60860, 14064, 58596, 8516,  46288,
                42580, 10712, 49632, 7116,  13444, 35928, 10108, 42484, 52596,
                20092, 30812, 64536, 17464, 33176, 54788, 42824, 28652, 62056,
                31456, 22288, 3640,  19728, 11840, 35932, 23108, 47336, 8652,
                19788, 63132, 34920, 27548, 7248,  64892, 1816,  49200, 6844,
                63800, 3604,  38492, 8452,  49012, 15480, 36000, 64876, 54772,
                33180, 7392,  46384, 55696, 41464, 58140, 8272,  19704, 47800,
                24036, 6424,  54780, 31196, 27112, 40700, 22580, 63356, 7460,
                57684, 41620, 13516, 5568,  29244, 57028, 17528, 50008, 36648,
                12748, 19440, 44584, 14324, 38236, 3936,  26624, 9036,  30380,
                2744,  38008, 9648,  56184, 3444,  59140, 19040, 53856, 720,
                43328, 20048, 11128, 41888, 57396, 6396,  44492, 54164, 40540,
                18216, 7972,
            },
            {
                12804, 41516, 76,    46992, 5524,  22748, 31624, 47856, 8912,
                30176, 21368, 33496, 18348, 25236, 31888, 54148, 41524, 8816,
                58296, 13032, 54352, 5640,  45616, 19156, 38336, 144,   26584,
                17848, 57420, 31332, 39016, 51892, 17432, 59028, 32836, 38108,
                3912,  56204, 39924, 45380, 22400, 3048,  39072, 15776, 1232,
                10112, 40876, 26716, 47980, 44096, 5932,  65072, 31304, 58972,
                53048, 5268,  11788, 46956, 57300, 13904, 34080, 28900, 42728,
                21656, 50776, 54084, 23868, 28408, 58588, 4792,  46840, 13148,
                24392, 61328, 35060, 18976, 30112, 12984, 45340, 35320, 15548,
                40388, 58668, 21260, 10172, 46720, 52684, 5028,  36064, 27888,
                31340, 49188, 65308, 36472, 47412, 10180, 32984, 54576, 2668,
                30948, 6564,  64400, 53444, 32016, 21700, 62768, 40168, 45688,
                16600, 54480, 47300, 32420, 25748, 65124, 39060, 12856, 33004,
                61376, 30260, 38016, 63688, 33184, 27092, 22316, 61364, 34432,
                4596,  49348,
            },
            {
                59612, 33680, 29488, 61176, 26628, 42572, 64348, 52840, 16148,
                60348, 440,   55784, 6164,  39808, 10732, 49692, 14660, 44848,
                34248, 23860, 63012, 35856, 10216, 28196, 50952, 55648, 12760,
                35452, 63224, 20352, 1656,  43896, 27060, 3028,  22264, 60820,
                12924, 24608, 10496, 5952,  62340, 13040, 57260, 25132, 45740,
                49424, 64164, 14772, 32524, 60708, 17240, 39828, 2288,  26052,
                15736, 41900, 60028, 50236, 24624, 40840, 18268, 60408, 10048,
                15960, 32504, 44056, 18492, 63344, 42120, 22144, 50516, 30752,
                2832,  40496, 15008, 9412,  50444, 3620,  26424, 52384, 61676,
                28992, 12764, 34484, 38140, 1224,  60852, 13232, 43856, 54464,
                10864, 23984, 17928, 53128, 26404, 62640, 43860, 15108, 25728,
                61724, 41848, 34780, 620,   11260, 49824, 56328, 19196, 34520,
                24600, 63116, 20332, 41360, 7776,  15424, 49472, 22692, 58300,
                5652,  48356, 25188, 3716,  13868, 50904, 47284, 9228,  28472,
                57224, 44172,
            },
            {
                25180, 15244, 8876,  37828, 50216, 11420, 1964,  19964, 39232,
                26316, 42236, 14080, 51648, 61488, 22216, 27388, 65112, 18460,
                29956, 49204, 1884,  20712, 41752, 64492, 33604, 22368, 48516,
                40512, 4396,  52908, 24196, 56600, 65292, 48168, 6792,  45096,
                28764, 49516, 34344, 53028, 27976, 47804, 32276, 20900, 59344,
                34492, 7324,  53564, 1760,  55700, 21560, 10304, 44832, 49528,
                32768, 37024, 1272,  20644, 31856, 4128,  55656, 46596, 26124,
                36500, 57608, 6096,  2200,  31544, 12024, 8908,  38024, 17428,
                57868, 44132, 53072, 63360, 37568, 22128, 59172, 38876, 5756,
                2412,  50688, 44892, 64552, 17080, 24588, 32560, 19836, 59448,
                800,   38916, 8048,  42388, 2952,  22296, 7604,  39564, 46920,
                21300, 51036, 27232, 17896, 58712, 42368, 6980,  13324, 4624,
                59512, 11068, 268,   35892, 52432, 29020, 45368, 9900,  27268,
                51856, 17108, 8648,  45844, 19528, 58780, 1692,  16384, 37784,
                20132, 2640,
            },
            {
                39472, 64596, 21444, 54728, 17792, 57068, 45464, 33100, 58096,
                10860, 48896, 34672, 28620, 44016, 1660,  36892, 4540,  56664,
                6920,  38960, 52868, 14916, 57648, 3156,  16548, 7068,  59476,
                29204, 15760, 33132, 8032,  36548, 14228, 30600, 54300, 16432,
                62828, 41664, 58372, 18260, 36956, 8196,  52212, 4648,  11484,
                38028, 28784, 23752, 41796, 36632, 29896, 51532, 57804, 12992,
                22628, 55032, 28852, 10168, 62592, 38124, 7992,  51832, 304,
                62784, 12920, 47168, 60040, 40048, 51860, 56520, 26292, 63056,
                6688,  20580, 25340, 584,   31644, 43224, 17708, 10768, 33048,
                55576, 19368, 26072, 7152,  56900, 48604, 39572, 6660,  50612,
                15904, 46732, 60920, 33276, 14320, 56592, 30196, 59668, 4308,
                11552, 56724, 8680,  39212, 23516, 29168, 36360, 47012, 52660,
                43008, 28476, 49132, 56940, 18164, 5408,  62636, 37028, 2760,
                40968, 34208, 54684, 60480, 39096, 30700, 42352, 52068, 62096,
                31912, 47992,
            },
            {
                5968,  51904, 44988, 3016,  34776, 28892, 24308, 6476,  50940,
                4232,  23136, 64088, 8028,  16712, 58360, 47344, 32880, 51756,
                11972, 22816, 43812, 26780, 31916, 47504, 38756, 25420, 43760,
                9940,  62052, 46856, 42268, 11284, 19360, 40832, 35128, 9192,
                20836, 5056,  0,     23504, 14456, 43596, 65244, 40604, 16444,
                56000, 18936, 50616, 12648, 8024,  62644, 25552, 3840,  38572,
                7552,  64504, 17048, 44888, 52564, 15048, 23088, 43580, 29752,
                39728, 20920, 25276, 35100, 15268, 20212, 45028, 1140,  35564,
                49228, 29228, 11700, 46676, 56624, 7228,  64612, 48664, 23656,
                41228, 62408, 14384, 31000, 42664, 11276, 29176, 64048, 34832,
                26312, 56052, 29048, 19960, 45172, 50692, 35512, 18196, 53216,
                37092, 15828, 44444, 60760, 48484, 3348,  65452, 26084, 21104,
                32084, 38256, 14432, 23600, 60036, 33252, 42340, 19596, 57544,
                14844, 64256, 22176, 26680, 10972, 35304, 5284,  24408, 13536,
                10424, 55836,
            },
            {
                17632, 30748, 12608, 41640, 7808,  63112, 13856, 36784, 61400,
                30076, 46340, 19432, 37692, 53440, 12972, 20120, 25120, 40428,
                61600, 17532, 59164, 8400,  54920, 11512, 61508, 51420, 3836,
                54092, 21236, 26196, 58292, 51068, 61768, 1840,  25308, 50440,
                37888, 31680, 45656, 54864, 59492, 29400, 2184,  25868, 45280,
                63036, 6064,  43376, 58544, 47444, 15504, 34576, 18652, 61048,
                48180, 42576, 2976,  33332, 25716, 58660, 35368, 19068, 11156,
                56304, 49592, 8360,  54968, 28104, 7136,  61420, 32400, 13984,
                54812, 38740, 60876, 34380, 15648, 27384, 53552, 13624, 29828,
                45564, 8900,  36816, 53932, 2752,  22700, 52936, 4108,  43276,
                21744, 11828, 37564, 5956,  63740, 10908, 312,   27404, 64692,
                24296, 31324, 2092,  33468, 14008, 54816, 16480, 9844,  57876,
                2504,  63684, 8244,  44232, 4008,  12124, 54936, 24148, 47212,
                31096, 6932,  44340, 244,   53092, 17988, 65448, 54832, 43916,
                27504, 35940,
            },
            {
                22912, 58112, 26184, 49332, 59276, 21952, 42840, 53628, 17132,
                40436, 12492, 56396, 3120,  41824, 29904, 63536, 9240,  45476,
                332,   28792, 35728, 4668,  45784, 21732, 30064, 18488, 34420,
                13432, 37768, 864,   29708, 5912,  22644, 44652, 60000, 56756,
                13284, 64200, 27212, 10268, 39408, 19276, 34756, 50064, 22280,
                30852, 592,   33416, 26992, 20540, 5200,  41024, 53820, 31104,
                27448, 20328, 57472, 49304, 12368, 6224,  47280, 65440, 5004,
                33624, 16572, 42428, 64360, 3088,  37316, 48728, 22768, 9704,
                43428, 18168, 5852,  50828, 21636, 41876, 35976, 5188,  60756,
                896,   21716, 49856, 59740, 34360, 18680, 58000, 14928, 8572,
                62160, 49256, 2176,  53716, 39456, 22924, 47740, 41336, 8164,
                45760, 50240, 62944, 21676, 7780,  43748, 30420, 40520, 50104,
                18468, 53660, 27288, 47732, 39788, 30656, 49892, 1512,  10544,
                36328, 50596, 12880, 40140, 29620, 48660, 7860,  21376, 39768,
                868,   63228,
            },
            {
                45708, 3800,  37176, 16208, 31952, 532,   47316, 9252,  26668,
                2224,  59828, 33216, 25968, 50328, 6140,  35576, 54328, 15684,
                32196, 48968, 64784, 41260, 14556, 37188, 1172,  57328, 41492,
                64664, 49868, 17468, 55416, 39352, 32916, 15708, 28668, 6436,
                43004, 17592, 48028, 51372, 6588,  61556, 12808, 57692, 9684,
                53644, 15084, 61336, 38392, 54940, 64364, 44480, 10940, 152,
                14152, 35924, 9396,  38868, 61796, 30820, 40740, 53704, 27196,
                60312, 1952,  30300, 22424, 45704, 53476, 16324, 41392, 27508,
                65168, 2460,  24388, 57992, 10360, 1956,  47228, 18780, 38580,
                57580, 27772, 17348, 5728,  46320, 27048, 40852, 47892, 37060,
                32104, 18172, 59712, 25536, 31424, 16536, 57444, 33692, 13396,
                19724, 5876,  27856, 53796, 37644, 59728, 22864, 35124, 5992,
                12572, 33460, 61104, 15856, 20700, 64912, 26264, 17216, 62272,
                27728, 58848, 20084, 56264, 61152, 14984, 45932, 32668, 59572,
                50112, 9088,
            },
            {
                41140, 52504, 61860, 10492, 39912, 55596, 18832, 65160, 31588,
                45112, 21740, 48772, 10484, 61452, 22736, 43392, 3568,  60048,
                23964, 10788, 20164, 53468, 25012, 63060, 48184, 8624,  22804,
                31248, 6552,  45544, 12788, 63876, 48920, 3200,  53020, 11136,
                34124, 24252, 1724,  35848, 22040, 32192, 46940, 3596,  39644,
                35900, 47916, 23448, 11372, 2572,  29000, 22976, 50876, 58708,
                46432, 24316, 54704, 4616,  21900, 17576, 1028,  14076, 22896,
                46008, 38852, 52140, 13132, 9016,  31444, 59424, 4288,  56888,
                51972, 30344, 45504, 40064, 61828, 32108, 55764, 25160, 52016,
                33848, 10568, 65444, 39740, 13060, 62492, 188,   29836, 23412,
                55016, 45652, 7568,  13964, 43464, 51544, 4588,  60792, 54204,
                38300, 58216, 41704, 11344, 18376, 50808, 472,   62240, 46648,
                42680, 24516, 37212, 1040,  56728, 35292, 7988,  53912, 38484,
                43084, 6200,  33660, 24676, 2820,  37440, 26480, 4724,  18644,
                29448, 14204,
            },
            {
                20592, 33152, 6512,  24640, 50892, 28580, 4964,  35296, 51520,
                14444, 5764,  36668, 16836, 39680, 57600, 18504, 27680, 50944,
                38240, 57016, 2480,  33444, 6088,  28228, 55256, 16060, 44540,
                27180, 59496, 35720, 24828, 8268,  18684, 36684, 46160, 20960,
                40376, 55652, 60316, 14964, 53496, 42372, 17056, 28072, 62532,
                19384, 7476,  52276, 45520, 17516, 36056, 8496,  32384, 38136,
                16824, 60696, 41976, 28948, 50604, 44344, 57136, 34364, 51192,
                10032, 19692, 57856, 34616, 61684, 19148, 25108, 11796, 34164,
                20248, 8328,  35708, 13280, 28020, 16820, 63660, 7524,  42396,
                15260, 49100, 43832, 31284, 24712, 9292,  50748, 59300, 12260,
                3184,  41076, 28188, 35080, 62844, 9368,  21432, 29472, 25328,
                1404,  15948, 34712, 47572, 5008,  26400, 13736, 56100, 28216,
                20344, 59100, 10336, 50480, 5700,  42100, 13528, 46000, 21748,
                3216,  15700, 47388, 52076, 9776,  42740, 63520, 12204, 57292,
                35152, 54408,
            },
            {
                62892, 28220, 17716, 46228, 60376, 13316, 38440, 23340, 58796,
                42256, 62320, 54872, 29032, 908,   13792, 47928, 34192, 6832,
                13204, 42424, 46924, 17784, 50816, 38972, 11956, 33900, 61180,
                4728,  11124, 20756, 53612, 41416, 61096, 26288, 57268, 63576,
                30532, 5744,  44916, 9376,  64776, 24480, 6276,  55320, 43524,
                25724, 58328, 29960, 41656, 60512, 49416, 56628, 20032, 63704,
                3372,  7764,  32688, 13292, 63528, 25796, 37484, 7588,  29720,
                63812, 5596,  25916, 43928, 3628,  40600, 50720, 37964, 46860,
                15276, 60212, 49344, 54364, 3764,  44700, 22576, 12240, 30312,
                4448,  23216, 1752,  53116, 37524, 55980, 20680, 35252, 16968,
                64900, 49716, 20220, 58524, 1124,  46908, 36896, 12032, 49020,
                44100, 61332, 23020, 30688, 64440, 44596, 32820, 38328, 8596,
                3212,  53172, 30996, 45256, 23304, 29128, 58352, 32732, 51500,
                60340, 30728, 64740, 18444, 35556, 22312, 53432, 49096, 38892,
                23980, 2396,
            },
            {
                11380, 56488, 36024, 1820,  43496, 19984, 56740, 10952, 1500,
                18060, 25636, 8044,  45792, 64812, 31572, 9512,  54700, 61004,
                21268, 30040, 62284, 9120,  58784, 42992, 20320, 2580,  52044,
                38584, 56712, 48416, 32316, 3964,  29952, 13728, 9552,  2276,
                49028, 18996, 26928, 38212, 30080, 2840,  50564, 33984, 10720,
                14120, 1296,  33204, 4784,  14724, 25528, 5760,  13128, 43408,
                26620, 51732, 47484, 19516, 3720,  10896, 59396, 48568, 16776,
                41968, 36240, 15240, 48976, 28980, 55120, 7076,  63984, 408,
                42328, 23712, 5340,  18948, 38232, 33576, 48204, 53424, 59884,
                36400, 56820, 61732, 19480, 15644, 5112,  45288, 42244, 7300,
                26012, 38276, 10676, 53376, 24048, 41792, 17360, 65076, 32168,
                8460,  51616, 3340,  54904, 9324,  20884, 52056, 16556, 63428,
                48144, 14316, 39608, 18256, 54660, 62064, 2228,  18664, 9020,
                25628, 12152, 39916, 4628,  58276, 30140, 1108,  16560, 8236,
                44200, 47964,
            },
            {
                41924, 14580, 50128, 63424, 8068,  32972, 52640, 47124, 30632,
                39956, 49444, 33612, 23580, 51876, 20380, 40724, 24960, 44348,
                3824,  35760, 26116, 14336, 31656, 23280, 62692, 46980, 25480,
                18436, 42500, 208,   15844, 59152, 44152, 51452, 23380, 38724,
                15888, 33472, 58624, 12252, 47812, 40568, 59336, 21508, 63860,
                48648, 38576, 53784, 21988, 65336, 40056, 47240, 54492, 30340,
                22468, 39076, 58464, 35528, 54176, 41108, 20676, 2124,  24168,
                56360, 52924, 1208,  63044, 10684, 17344, 32552, 21796, 27836,
                56240, 31872, 63336, 26384, 58320, 9544,  596,   39580, 19804,
                46044, 28052, 8640,  34144, 47632, 63320, 27708, 32216, 51688,
                57280, 4876,  33380, 15364, 30712, 56840, 3232,  39444, 55620,
                19356, 28028, 13208, 40260, 36944, 59316, 1768,  41676, 30276,
                25864, 60472, 34524, 6668,  11656, 37428, 27348, 40596, 48724,
                36184, 56828, 44508, 27428, 13876, 46352, 61804, 33696, 27988,
                60492, 5276,
            },
            {
                26048, 38324, 21372, 30320, 24336, 41272, 5440,  26728, 63924,
                13828, 57784, 3588,  11644, 43616, 36864, 2456,  63172, 16672,
                52884, 49056, 39864, 536,   54808, 5160,  36468, 8520,  30668,
                13260, 65368, 27768, 37788, 21356, 5716,  35200, 55048, 42332,
                60904, 51716, 388,   54768, 19940, 15624, 8708,  36804, 29116,
                17684, 45148, 59468, 9304,  28284, 34024, 936,   36312, 10716,
                61244, 1728,  9588,  16140, 45136, 27820, 33344, 61448, 45648,
                12080, 31884, 20812, 39276, 24012, 59136, 44208, 13016, 48584,
                8740,  39464, 11676, 43584, 50780, 14888, 64788, 25404, 6204,
                16268, 50676, 12772, 41520, 25196, 2660,  11108, 59748, 22460,
                14056, 44600, 61320, 48212, 7144,  27204, 51388, 22548, 5224,
                34400, 42760, 63216, 17100, 49144, 24948, 11992, 55716, 18956,
                4392,  43532, 22492, 64676, 46368, 16224, 52312, 5072,  63624,
                22980, 428,   53848, 7880,  50984, 20564, 41088, 10692, 57788,
                18388, 32156,
            },
            {
                65240, 2748,  54572, 10224, 57900, 16064, 61368, 36800, 9280,
                21592, 35756, 53724, 18880, 59392, 6208,  56528, 29728, 12500,
                7212,  57888, 19176, 64916, 44156, 50020, 16652, 54072, 58444,
                34764, 51124, 9716,  45480, 49980, 63944, 17976, 28140, 4380,
                11216, 22284, 36244, 25416, 62812, 32100, 45432, 1896,  54472,
                5928,  24404, 12340, 49992, 19084, 57636, 16052, 63084, 45692,
                18128, 53188, 31388, 24660, 65104, 6960,  14252, 52164, 38268,
                8016,  27324, 58356, 47360, 34224, 2980,  53684, 37332, 60764,
                16736, 52340, 2436,  21408, 36188, 29236, 55364, 32436, 60744,
                43536, 2240,  31068, 58484, 54428, 39284, 48716, 18312, 37152,
                704,   28968, 39972, 19116, 63836, 43764, 9976,  59844, 15168,
                45908, 57108, 23644, 5732,  29548, 34976, 7196,  46592, 36704,
                51440, 9536,  57084, 632,   25284, 33200, 59516, 10724, 42972,
                15160, 19536, 32368, 38272, 63268, 25048, 6068,  37180, 50368,
                22624, 53008,
            },
            {
                7292,  17068, 44664, 33916, 48760, 696,   19320, 43244, 50880,
                4516,  46604, 27664, 62032, 15056, 25956, 33388, 46132, 38060,
                22664, 28336, 9884,  33868, 25208, 11536, 28928, 41432, 1556,
                22560, 6348,  55732, 24764, 12616, 32940, 8476,  47720, 62564,
                31044, 46448, 7548,  43060, 4572,  52080, 57384, 26788, 42312,
                61280, 31940, 36036, 2432,  43636, 52628, 7968,  21836, 41508,
                27944, 49448, 5280,  40044, 55236, 48132, 3244,  30364, 18340,
                64368, 4788,  43384, 14108, 8412,  50424, 25940, 6236,  19824,
                34852, 29784, 46372, 61360, 6652,  18248, 41564, 11292, 22072,
                52912, 37812, 62968, 21000, 6840,  15052, 30388, 64480, 8064,
                56228, 52976, 23512, 2368,  35240, 13448, 31736, 47988, 26216,
                37136, 10976, 640,   60072, 52336, 44032, 65256, 22244, 58316,
                32296, 12956, 28604, 53464, 42128, 48284, 20696, 30348, 55504,
                35068, 60132, 47876, 11296, 16864, 30860, 55448, 43380, 1572,
                12776, 45952,
            },
            {
                40392, 28608, 59204, 13552, 39156, 25776, 53504, 29508, 60020,
                15664, 40408, 31328, 132,   42152, 48668, 10000, 17916, 50888,
                60528, 42568, 47132, 15676, 60216, 37884, 19944, 63232, 45064,
                15352, 37184, 31132, 60544, 2088,  39776, 57940, 25732, 13812,
                38156, 56408, 17072, 28368, 39252, 13376, 22720, 10340, 16152,
                50768, 20776, 40960, 62480, 30168, 24680, 48444, 32448, 3564,
                13832, 60016, 34528, 11848, 21180, 36796, 57932, 23004, 41716,
                49388, 35256, 55616, 19032, 61984, 41192, 29476, 65352, 1228,
                42656, 58160, 13600, 54488, 24804, 49832, 2860,  47480, 34800,
                26360, 9468,  17960, 44976, 35644, 24076, 52232, 43116, 34020,
                46272, 16448, 11016, 50044, 58812, 38200, 20536, 61936, 2588,
                53332, 30456, 47036, 33620, 18784, 13864, 27860, 2212,  15872,
                40824, 48832, 19780, 38196, 14396, 8356,  2692,  24236, 45540,
                6924,  26320, 2836,  42520, 58636, 4260,  48372, 15000, 27512,
                61516, 35648,
            },
            {
                20304, 51296, 23256, 3892,  63724, 46660, 7512,  12128, 34112,
                24128, 65456, 8688,  51424, 37620, 21932, 55184, 64232, 1416,
                31796, 5592,  56924, 2500,  52236, 6976,  47792, 4096,  26916,
                59820, 48308, 19680, 42056, 16880, 54064, 21844, 43668, 1312,
                19776, 50272, 65116, 9740,  60108, 49184, 34784, 64424, 37848,
                3344,  8536,  55260, 14780, 5156,  38904, 12600, 59260, 36872,
                55488, 23288, 43120, 62520, 16684, 8716,  44044, 60604, 10560,
                15780, 25352, 40,    31308, 22868, 10880, 16044, 45912, 55544,
                22344, 26732, 3996,  33192, 37980, 8940,  62348, 58264, 14344,
                5664,  56364, 29360, 49704, 1004,  61432, 10356, 4520,  21096,
                26228, 62804, 32468, 42320, 25176, 4416,  55248, 8208,  40804,
                17016, 62920, 21528, 41784, 9300,  54636, 38644, 60388, 7976,
                24520, 63588, 5964,  60236, 35432, 56296, 64308, 50320, 13512,
                38004, 62456, 29412, 52116, 23232, 36028, 64084, 19168, 33452,
                56672, 9540,
            },
            {
                31348, 62268, 6316,  36456, 55808, 31944, 22032, 57436, 2424,
                44524, 55556, 20224, 12660, 58748, 28996, 7468,  40544, 14492,
                24808, 36196, 20628, 40304, 31184, 23440, 55028, 32952, 39696,
                11188, 53100, 3776,  62776, 29068, 6112,  35692, 51944, 59520,
                33744, 3792,  24332, 44928, 31272, 1216,  18748, 46712, 29532,
                58996, 44472, 27228, 47784, 18568, 63868, 53584, 26784, 7348,
                19512, 46644, 1268,  29320, 51108, 26008, 33864, 1704,  28836,
                53532, 62860, 37492, 44764, 52952, 59892, 38992, 32304, 7544,
                51240, 10444, 40680, 64448, 15784, 43704, 30240, 20100, 39672,
                45900, 65188, 41652, 12320, 55156, 27452, 40036, 58716, 13144,
                38568, 3540,  57440, 7492,  15432, 51024, 29496, 44012, 23932,
                12420, 49940, 6856,  57364, 25768, 3940,  48280, 30356, 52692,
                34212, 44808, 3172,  30012, 17648, 26900, 40356, 31960, 53576,
                17996, 10164, 56380, 19904, 13296, 40092, 8756,  52804, 23776,
                3644,  47788,
            },
            {
                55136, 12636, 43248, 27124, 18600, 15088, 41988, 37924, 49536,
                17688, 6024,  36320, 25380, 45484, 3856,  34820, 19644, 45000,
                61960, 51512, 11740, 27144, 64156, 9584,  14244, 57980, 17820,
                7584,  25684, 34140, 13220, 45816, 49208, 10068, 15580, 27296,
                41120, 12176, 54252, 35620, 21344, 55772, 5544,  25220, 52280,
                12424, 34132, 23132, 57696, 10504, 35088, 44,    42668, 50084,
                64748, 9980,  32684, 57736, 4684,  47352, 19112, 56208, 39488,
                7168,  20552, 12680, 48156, 5832,  3176,  24852, 48916, 13872,
                36848, 59264, 19164, 48536, 22792, 52536, 27024, 54652, 1632,
                32088, 22536, 3820,  16632, 33124, 46816, 18528, 50324, 29856,
                54144, 47776, 20176, 28244, 45592, 65500, 18532, 33516, 58132,
                36436, 27568, 39524, 15532, 61464, 35672, 20516, 42496, 17484,
                11448, 55268, 21720, 50540, 43608, 11816, 5752,  20424, 1652,
                41476, 47496, 4680,  34072, 46072, 920,   28116, 44912, 59784,
                42028, 16164,
            },
            {
                232,   38772, 49008, 10160, 52604, 1364,  61752, 9424,  59624,
                27536, 32648, 52572, 63312, 16388, 49152, 59908, 53380, 27960,
                8556,  47540, 16796, 55704, 43024, 45988, 36596, 50624, 29180,
                64880, 43960, 21596, 38560, 57676, 24508, 64288, 31852, 5588,
                61532, 48288, 16528, 7260,  41928, 14236, 58416, 39116, 6984,
                17008, 62196, 1844,  39940, 31228, 46232, 21328, 14940, 30316,
                38228, 17380, 40980, 13976, 53668, 37528, 64588, 14576, 46328,
                50912, 33008, 58656, 27780, 17816, 35780, 57500, 20368, 63236,
                28260, 44468, 5648,  31232, 236,   12376, 7032,  36576, 17508,
                60384, 51140, 25648, 38172, 57984, 5180,  23220, 64192, 1444,
                34612, 9392,  40780, 60460, 35660, 36,    10272, 53732, 4912,
                63780, 1376,  32200, 44828, 50852, 10392, 63208, 296,   47224,
                26804, 59592, 38752, 9564,  62904, 47056, 58428, 36488, 61000,
                22952, 30556, 65496, 25096, 51260, 61864, 32124, 12328, 5460,
                37376, 25572,
            },
            {
                58948, 20012, 32576, 64336, 44744, 29744, 35172, 24872, 46436,
                13352, 39640, 2864,  42736, 10312, 31832, 23164, 12416, 4452,
                38512, 58508, 480,   34244, 6492,  19380, 2808,  22328, 41376,
                1744,  49636, 55592, 8748,  684,   18624, 37036, 55304, 45124,
                20208, 29552, 37536, 63028, 50948, 27624, 45224, 63556, 31088,
                42616, 20336, 54508, 50196, 7640,  61076, 25524, 58224, 5400,
                54620, 23832, 61928, 27000, 7700,  22496, 10812, 31060, 24072,
                2516,  42816, 9340,  40220, 64272, 30436, 12004, 41412, 2136,
                53572, 15344, 35332, 60928, 56652, 46120, 63592, 42712, 10292,
                48252, 13732, 7848,  62560, 44008, 9720,  36868, 15092, 44932,
                24784, 16988, 55116, 12684, 22948, 39144, 48172, 14516, 21216,
                46536, 52168, 19200, 5668,  22836, 28504, 13740, 57508, 37348,
                6620,  15476, 32568, 1144,  23592, 33872, 15812, 26032, 44520,
                55144, 15096, 7360,  38176, 16680, 57188, 21628, 49800, 18456,
                64832, 29972,
            },
            {
                51120, 14872, 4856,  22324, 56768, 7232,  50460, 20116, 4072,
                54632, 61312, 21896, 29640, 57660, 1612,  36996, 64992, 43424,
                32588, 21632, 29300, 62908, 25000, 52716, 61668, 12484, 58900,
                35348, 15896, 30660, 61212, 27120, 41920, 13744, 3452,  23612,
                9604,  56908, 8,     24848, 11092, 32872, 2696,  22028, 9972,
                48232, 37336, 4700,  28304, 16376, 36592, 52188, 11504, 33968,
                45696, 2620,  48548, 35188, 42252, 61056, 45092, 4432,  58020,
                26920, 61648, 15196, 22396, 50072, 54032, 45736, 6944,  33656,
                23960, 9160,  50644, 25356, 38872, 19744, 33024, 24184, 58068,
                28208, 34960, 53940, 29672, 20804, 51728, 32024, 60304, 7104,
                49736, 63292, 31040, 5404,  51396, 26940, 62088, 30584, 41904,
                25280, 11648, 60864, 56288, 34588, 40888, 53512, 31764, 23896,
                64496, 41936, 53920, 28276, 56816, 51800, 3872,  8856,  48964,
                11960, 35148, 58872, 43492, 10708, 2912,  41304, 54668, 34536,
                9388,  43968,
            },
            {
                7428,  54848, 35460, 39612, 26112, 17144, 63076, 11668, 31672,
                43316, 15672, 7868,  47664, 19208, 40708, 55748, 26000, 17864,
                52248, 13072, 41420, 48812, 15044, 39388, 27760, 32336, 46968,
                6008,  20256, 11368, 46300, 33716, 53196, 59652, 48020, 39128,
                52444, 33584, 42968, 59948, 19180, 54720, 36332, 15516, 52900,
                60976, 25016, 13192, 64920, 41252, 19464, 2964,  43604, 62680,
                28544, 9284,  20376, 51844, 448,   15924, 33448, 38952, 53312,
                17640, 36140, 55480, 6116,  1456,  25880, 16428, 62144, 56124,
                47116, 18316, 42552, 2880,  28896, 14272, 51360, 4864,  39556,
                2376,  45168, 18480, 40932, 316,   12536, 56692, 27928, 42140,
                19544, 37468, 2040,  44556, 59660, 19328, 6968,  34716, 55076,
                8472,  38496, 29916, 14972, 48432, 4120,  8340,  45236, 18564,
                2908,  49044, 20440, 12824, 39852, 18696, 42356, 63952, 28496,
                60,    52476, 19636, 27660, 48256, 30780, 6820,  26300, 14132,
                47688, 22996,
            },
            {
                27824, 41624, 60848, 3140,  13320, 47576, 40996, 36380, 52088,
                58164, 24344, 38052, 62808, 51724, 14364, 6872,  49672, 9360,
                60688, 3352,  54876, 8312,  35752, 3920,  56864, 9140,  24124,
                54584, 63048, 39268, 50712, 4968,  21880, 8368,  29040, 17548,
                63656, 13972, 6432,  49024, 39496, 4720,  46404, 57516, 28068,
                564,   33752, 45640, 57092, 9612,  48652, 56188, 22148, 38720,
                16948, 59292, 13076, 55744, 30136, 58864, 19656, 49032, 12164,
                7996,  47404, 31392, 44960, 34688, 38472, 10388, 21568, 29196,
                37512, 59436, 64100, 11340, 54440, 7768,  61632, 16808, 55292,
                21940, 63788, 6032,  59304, 47704, 24940, 39600, 3144,  53264,
                10308, 57920, 26088, 33376, 15404, 41384, 57284, 1640,  45360,
                17488, 58800, 2316,  42880, 65144, 21004, 26904, 61204, 51636,
                34328, 7804,  44696, 62004, 4984,  31016, 57928, 37276, 20828,
                32332, 40572, 5528,  63408, 23152, 36440, 61184, 58156, 39504,
                2428,  62896,
            },
            {
                17292, 10460, 45832, 30116, 53156, 58548, 1808,  22596, 6656,
                28728, 340,   34684, 11476, 27592, 33652, 22632, 46360, 30824,
                36164, 45128, 23724, 18992, 59744, 44656, 50284, 17176, 42724,
                36700, 28644, 3168,  25136, 16912, 65100, 44080, 32728, 1748,
                25980, 46112, 21852, 28740, 16028, 65424, 23472, 8296,  40276,
                18848, 51560, 6572,  23156, 29584, 32956, 13632, 26508, 7208,
                51052, 32424, 36972, 24384, 46764, 6740,  27700, 62972, 23024,
                41616, 65148, 19416, 13484, 60736, 57344, 43300, 51660, 812,
                13640, 4716,  30960, 21124, 41244, 48292, 36060, 31264, 47104,
                12360, 26236, 32564, 15152, 35372, 65032, 17244, 22364, 34272,
                13664, 47520, 21668, 53752, 9440,  48640, 12296, 22832, 64024,
                27616, 49684, 36628, 24576, 9952,  55168, 39588, 15252, 11024,
                29436, 58632, 25976, 35796, 9956,  47372, 23348, 13056, 49920,
                61608, 16460, 45660, 54956, 13332, 51672, 17556, 45204, 20452,
                53364, 33136,
            },
            {
                57184, 24892, 50296, 18724, 8828,  33428, 27184, 44568, 65060,
                18156, 49288, 44948, 56332, 5312,  42920, 63396, 1212,  57208,
                15876, 27604, 64560, 33052, 11636, 30284, 21200, 64136, 136,
                14296, 52304, 59044, 41296, 13136, 36156, 54680, 11964, 49308,
                60452, 36840, 9892,  58504, 31928, 50524, 12828, 43136, 31448,
                59764, 14856, 36388, 43924, 60116, 1044,  63316, 40524, 47320,
                3304,  65420, 43168, 4184,  39904, 10252, 52100, 34856, 1380,
                29516, 4092,  50992, 23468, 28364, 3376,  18740, 32772, 40712,
                26376, 49864, 45500, 34292, 57972, 23180, 768,   43412, 8868,
                58816, 37368, 50976, 8488,  55560, 44616, 5252,  49172, 62572,
                28460, 43184, 6488,  64904, 38528, 28920, 36192, 52096, 31900,
                5608,  13300, 53268, 18820, 46020, 32928, 1732,  36092, 56032,
                41236, 22092, 14548, 50336, 55400, 17208, 2096,  43984, 6836,
                56060, 24572, 3504,  38712, 8288,  33296, 1024,  11200, 29864,
                5140,  37612,
            },
            {
                13648, 1096,  64684, 21888, 38356, 61264, 15284, 10648, 55672,
                40248, 8684,  60532, 21288, 17412, 52512, 38612, 11204, 20244,
                40500, 6280,  47900, 2648,  53244, 40820, 6680,  27056, 47624,
                33088, 10060, 19648, 45532, 57116, 27576, 6108,  40552, 19752,
                55844, 5248,  41584, 53736, 1968,  35676, 26068, 62404, 3804,
                47580, 10640, 54952, 20196, 5520,  52600, 35628, 18612, 54236,
                29664, 11440, 21176, 57372, 17812, 60724, 14652, 43788, 56156,
                37052, 58284, 9868,  40288, 53508, 7708,  47864, 64808, 58832,
                8552,  55396, 17368, 6144,  14792, 27356, 65320, 19428, 29284,
                53204, 3008,  42408, 21188, 28700, 11424, 31564, 54536, 8216,
                40088, 59020, 17472, 3000,  24372, 60768, 4252,  15892, 40792,
                20832, 34168, 62360, 3480,  29600, 59504, 50632, 17972, 47052,
                5360,  62924, 1240,  38656, 29108, 65172, 33360, 58908, 27216,
                10680, 35788, 30248, 59832, 21320, 42212, 64952, 25216, 49936,
                61808, 43676,
            },
            {
                47912, 30916, 35028, 42984, 4080,  54776, 46388, 19988, 37004,
                30100, 14140, 25676, 32388, 2348,  59148, 29408, 25056, 53868,
                61824, 13452, 37704, 57476, 25260, 15988, 60504, 37588, 56632,
                22960, 62624, 7220,  30236, 1104,  50264, 23996, 62780, 30020,
                15292, 33884, 24556, 18412, 44064, 6756,  20128, 54104, 37948,
                22600, 28548, 64436, 39428, 25988, 15968, 44964, 9168,  24080,
                59856, 15300, 46108, 27380, 50028, 32236, 25140, 6228,  20880,
                16772, 45604, 26828, 34308, 62516, 12812, 30360, 22564, 15748,
                36712, 24132, 63140, 39664, 52272, 10572, 49620, 39168, 13500,
                24288, 63244, 17552, 48840, 1488,  61384, 37720, 25168, 15772,
                588,   35768, 30644, 49896, 45568, 19004, 55340, 43992, 59144,
                47424, 8968,  56608, 41696, 11944, 22988, 7488,  61572, 24612,
                31956, 52796, 19420, 46048, 7452,  24788, 51988, 38000, 19260,
                41408, 53064, 14572, 49564, 27492, 46848, 56168, 36620, 15360,
                22332, 8416,
            },
            {
                52876, 59944, 16008, 6312,  51588, 24248, 31800, 2732,  50836,
                5624,  63180, 53764, 47436, 37144, 14684, 48636, 4844,  44212,
                32496, 51168, 21968, 43104, 9484,  33776, 51016, 12284, 4300,
                43456, 49600, 38764, 34544, 61876, 17768, 10576, 37504, 3752,
                44792, 51612, 64096, 11444, 59804, 47832, 57148, 9756,  16716,
                51056, 1944,  32452, 42948, 12092, 50004, 31240, 61976, 1628,
                33672, 41320, 7448,  35912, 840,   64120, 38632, 48400, 53392,
                62048, 13820, 5176,  49240, 17452, 37132, 46100, 5540,  52956,
                11580, 44076, 28632, 3040,  59752, 32064, 4212,  55932, 45336,
                5228,  31864, 39964, 57388, 23976, 43052, 19248, 59692, 46504,
                52160, 21068, 56464, 13188, 32960, 7540,  26468, 11176, 628,
                23972, 37756, 26348, 49160, 15612, 37288, 43684, 28412, 13700,
                10056, 43264, 34488, 60076, 11728, 41896, 3980,  15080, 62412,
                47676, 884,   63992, 18272, 10416, 3224,  31404, 6892,  54228,
                41288, 28032,
            },
            {
                20292, 11324, 39880, 57644, 28688, 12472, 62244, 41308, 58220,
                23324, 35316, 11952, 41980, 7664,  23040, 65376, 35000, 17220,
                8916,  708,   30048, 18020, 63552, 1824,  45856, 20356, 31648,
                17332, 26028, 14812, 54496, 21920, 46608, 42444, 52948, 59156,
                21468, 820,   27436, 38412, 14484, 30432, 40984, 27644, 34116,
                44312, 56748, 8096,  18124, 59180, 3456,  56024, 21012, 37960,
                48620, 57852, 52456, 22648, 55160, 13000, 8804,  29892, 2688,
                33104, 41332, 21944, 59432, 148,   25608, 56720, 41644, 33396,
                1600,  49112, 20112, 35336, 42600, 18300, 24976, 61492, 34548,
                15952, 60324, 10220, 35876, 14224, 52472, 4064,  33480, 12324,
                27220, 63852, 9876,  41256, 53060, 62388, 39188, 29992, 64244,
                52608, 17656, 6548,  30984, 63772, 53972, 224,   57256, 40204,
                64628, 3688,  26572, 15592, 53800, 21792, 49004, 9144,  30844,
                22984, 6352,  33528, 43172, 57916, 39960, 19716, 12592, 59460,
                33876, 2780,
            },
            {
                63452, 46076, 25848, 48888, 18408, 36300, 7824,  48036, 27460,
                17324, 44404, 3988,  57620, 27736, 51076, 10468, 41148, 60836,
                55092, 27008, 58952, 48892, 36624, 54192, 28144, 41532, 65264,
                55640, 1960,  60596, 9264,  4364,  32128, 28180, 7100,  13428,
                32628, 46740, 8396,  49924, 34872, 22936, 1292,  63988, 5680,
                13652, 61696, 24916, 36048, 46692, 28344, 40760, 13916, 26740,
                5976,  17372, 10832, 29368, 43656, 19868, 40920, 59776, 23652,
                11132, 51604, 28964, 55108, 44216, 9752,  63448, 19772, 27684,
                61396, 57704, 8336,  54532, 13240, 47296, 37572, 7844,  21812,
                52712, 47672, 27420, 6764,  30488, 64444, 9216,  49352, 39508,
                6152,  44220, 23648, 1680,  16568, 35464, 20608, 49316, 14176,
                33080, 44936, 60220, 3904,  21436, 46220, 18552, 32612, 50220,
                21660, 47948, 56696, 30504, 36716, 61844, 27584, 56440, 35416,
                45004, 54796, 26464, 7840,  52040, 23892, 62884, 48140, 43996,
                17160, 38748,
            },
            {
                23664, 7336,  32432, 384,   64076, 43800, 22132, 14876, 1332,
                52904, 31980, 20824, 61208, 38548, 18320, 30816, 3208,  21360,
                46820, 39040, 12228, 5656,  24028, 14740, 58452, 5080,  10384,
                35484, 29668, 44772, 40368, 51340, 58644, 16308, 64948, 25040,
                41040, 56604, 61940, 19540, 4240,  60572, 51852, 46192, 19256,
                38456, 48464, 21488, 54432, 6788,  64080, 9780,  51948, 44416,
                63204, 31908, 60812, 3948,  47712, 58060, 33812, 17084, 46988,
                64764, 37560, 8172,  19008, 31828, 39336, 14240, 4732,  46680,
                16908, 38848, 31180, 22820, 62856, 1000,  51228, 29492, 11888,
                43332, 1852,  20372, 55520, 40784, 45048, 22084, 57492, 30060,
                18232, 36396, 59296, 28400, 47236, 56860, 6372,  42952, 3292,
                57628, 9576,  39692, 51776, 34512, 10556, 26808, 4804,  36228,
                7908,  17448, 39248, 968,   6452,  43400, 17704, 2896,  39860,
                12780, 19864, 61292, 38048, 15320, 35524, 29392, 680,   26144,
                5772,  55408,
            },
            {
                58324, 37108, 14676, 52672, 9880,  55740, 33324, 59388, 37312,
                64824, 9048,  46616, 15224, 416,   55952, 44856, 52636, 25476,
                15100, 33532, 62760, 44624, 31288, 8544,  22140, 38880, 51908,
                47884, 13116, 23672, 18852, 36824, 11800, 48200, 34836, 54288,
                17540, 10840, 29484, 39796, 53952, 16104, 11824, 25560, 58980,
                30836, 10196, 41568, 492,   31868, 17044, 23744, 34948, 4384,
                19792, 40212, 24120, 36644, 15468, 26812, 1828,  54384, 5304,
                25884, 43156, 3388,  61500, 48764, 23160, 54100, 35908, 51480,
                24696, 12528, 4344,  45560, 27132, 16636, 57088, 41176, 64796,
                25984, 37416, 59548, 49668, 17040, 804,   26104, 15004, 62228,
                3392,  54180, 49760, 32144, 4620,  12512, 25424, 60980, 36860,
                23364, 19600, 28640, 13364, 42040, 57892, 62192, 14624, 54372,
                59992, 25540, 12108, 51144, 58480, 33212, 10992, 64584, 51644,
                58724, 2028,  30104, 50048, 10892, 4444,  53804, 60344, 32744,
                51032, 10656,
            },
            {
                29848, 42936, 61324, 20952, 40748, 29628, 5512,  50648, 11744,
                40112, 29112, 54460, 24904, 33156, 63484, 12816, 36284, 7628,
                57664, 3888,  19576, 52944, 40672, 61216, 46576, 18424, 25604,
                60096, 6868,  53480, 63480, 26964, 692,   22196, 5712,  44168,
                2496,  36516, 23788, 6028,  45008, 32164, 42316, 35984, 2680,
                55540, 27788, 15272, 49680, 60136, 39000, 46056, 58240, 54200,
                11588, 49840, 56152, 8192,  63736, 51236, 11780, 44356, 36216,
                20360, 56780, 15796, 34596, 11784, 2252,  29356, 59780, 9004,
                42224, 64500, 53648, 34240, 59588, 10616, 31712, 5100,  18608,
                53872, 4076,  32540, 12520, 61672, 34700, 37900, 51964, 46212,
                41488, 13460, 8792,  22012, 64744, 39900, 51736, 29292, 15540,
                54184, 48572, 65488, 24732, 2216,  47564, 38424, 22904, 44332,
                29884, 41948, 62740, 45764, 23920, 19616, 46772, 29308, 25072,
                16488, 47828, 22572, 43892, 65492, 18120, 41204, 21604, 13948,
                45588, 19284,
            },
            {
                1912,  49860, 26704, 3600,  47596, 16788, 24844, 45524, 19184,
                22788, 4248,  49872, 42640, 6036,  20172, 48684, 27996, 60508,
                42392, 50140, 9856,  26344, 3012,  34704, 56292, 980,   32856,
                15728, 42096, 2972,  30892, 46016, 55864, 39348, 61240, 30408,
                50588, 58272, 47256, 64624, 27176, 56416, 8964,  22120, 50920,
                6004,  65280, 35224, 52920, 12448, 20732, 29468, 1892,  15588,
                28024, 33724, 2592,  42988, 30152, 38816, 22064, 62512, 31008,
                13160, 47628, 27488, 52172, 41004, 63872, 44532, 20780, 32472,
                580,   49252, 19484, 7124,  43148, 38040, 21404, 48988, 34828,
                14788, 45864, 23488, 8604,  28480, 54396, 5336,  10624, 20308,
                33592, 25492, 61344, 17172, 45608, 34092, 19212, 7704,  44284,
                1156,  32392, 6928,  56220, 16352, 29188, 5944,  51496, 9924,
                1980,  20324, 32444, 14880, 7384,  55228, 38516, 4916,  34156,
                8996,  41576, 6000,  32528, 55464, 27572, 47148, 6720,  38364,
                64260, 35392,
            },
            {
                16340, 54392, 11364, 34216, 57100, 38308, 61540, 2104,  54964,
                62652, 35384, 13088, 59788, 37740, 9520,  40688, 2052,  16724,
                22156, 30372, 37840, 64680, 16468, 49040, 11772, 27544, 64140,
                37632, 21048, 57308, 34424, 8464,  14476, 51880, 18536, 26660,
                8092,  14864, 20972, 12076, 12,    18640, 62608, 47692, 39160,
                17536, 44636, 24192, 4492,  43152, 8344,  62156, 48580, 41868,
                65068, 21688, 46752, 13644, 18916, 58808, 6308,  49284, 8988,
                60660, 39512, 6628,  58184, 18188, 25460, 5812,  56612, 16212,
                27792, 36928, 14100, 25092, 51528, 1992,  63280, 44392, 7644,
                60680, 56824, 39536, 19720, 42404, 47476, 24216, 65316, 29452,
                2048,  55676, 36920, 42304, 28,    58392, 10508, 55580, 62888,
                41080, 11928, 45428, 37012, 60652, 33068, 19336, 63888, 35052,
                49336, 56432, 3952,  37296, 27312, 60600, 49828, 14052, 63160,
                56348, 36888, 60960, 13392, 1256,  36224, 57740, 9608,  52148,
                24684, 5260,
            },
            {
                46296, 39892, 23092, 65212, 6888,  13608, 28012, 32616, 42004,
                7980,  26856, 17888, 56352, 29568, 22524, 53692, 62080, 34904,
                55852, 46532, 13464, 57332, 21440, 29588, 42716, 7616,  51244,
                45020, 10276, 49328, 17440, 59720, 24656, 42836, 11004, 37896,
                62964, 33440, 41600, 52772, 38304, 34356, 29060, 7040,  13692,
                60616, 32848, 19768, 56912, 26736, 37268, 33320, 6252,  25252,
                37760, 9444,  60296, 52156, 25924, 772,   35308, 24328, 17280,
                53056, 2020,  22384, 32256, 9492,  50512, 13172, 38808, 47368,
                54336, 61688, 41200, 57484, 29388, 15756, 54996, 24088, 27828,
                10936, 30108, 51184, 63828, 2420,  57604, 13752, 39684, 59056,
                48224, 6444,  51172, 27764, 14584, 30720, 49356, 21384, 35324,
                26356, 18076, 53184, 21564, 9172,  43200, 54568, 12444, 26380,
                16896, 39568, 65180, 52408, 10536, 42292, 32,    21916, 44024,
                17756, 49292, 26844, 19900, 51416, 23948, 16240, 31504, 43040,
                28812, 60944,
            },
            {
                8936,  58792, 31412, 18720, 43768, 50252, 9652,  58108, 15456,
                52372, 48056, 44308, 1360,  64032, 15028, 45088, 31612, 11120,
                6644,  25156, 344,   43728, 6480,  37232, 54976, 14276, 58768,
                4776,  23444, 27900, 40620, 4156,  63916, 32372, 1644,  48096,
                54892, 3500,  25028, 60148, 48780, 15416, 58528, 43540, 25172,
                53608, 10004, 1192,  62820, 46952, 17168, 55924, 13200, 50900,
                18016, 55424, 5684,  31316, 40100, 46224, 56596, 42168, 33188,
                28560, 43908, 65220, 36908, 46400, 29776, 62648, 34068, 2924,
                10980, 22740, 4000,  9132,  45936, 33688, 12020, 40272, 50308,
                160,   36508, 15716, 6496,  34016, 17680, 31360, 7888,  44036,
                18520, 12312, 22540, 60368, 46804, 3972,  24972, 39148, 6056,
                59208, 30296, 49740, 3740,  27280, 48488, 604,   40976, 58956,
                46628, 7828,  29224, 23188, 18308, 35156, 30376, 54028, 25680,
                3056,  31020, 11160, 40148, 45412, 59252, 3624,  63068, 12676,
                56036, 20764,
            },
            {
                37444, 14216, 48820, 4388,  53472, 35400, 20820, 39596, 23808,
                3664,  30888, 11252, 34128, 49404, 24756, 5024,  51020, 19424,
                39272, 52852, 59888, 32036, 50036, 61640, 24428, 19844, 31004,
                35364, 62484, 54612, 12960, 45424, 36208, 20088, 58104, 22856,
                44760, 17104, 30696, 9372,  5132,  22712, 51864, 1976,  30764,
                36652, 48104, 41180, 29632, 10872, 52244, 22552, 60968, 128,
                29124, 43776, 35792, 16504, 64344, 10368, 14976, 51072, 4204,
                59196, 11064, 14636, 54948, 788,   42440, 23144, 58920, 19060,
                44124, 31472, 35736, 65432, 20480, 59684, 4256,  18924, 62304,
                42996, 21992, 55508, 26616, 44672, 49588, 22388, 52916, 35764,
                27104, 62656, 39264, 9072,  34888, 52828, 64212, 16236, 45056,
                2084,  14072, 57588, 39792, 61944, 15124, 23544, 30988, 4464,
                21808, 13472, 44268, 59552, 48468, 5896,  62568, 12120, 39532,
                60032, 6900,  53396, 63844, 5108,  33780, 38680, 48084, 18324,
                184,   33684,
            },
            {
                27260, 2784,  42360, 29240, 24596, 62428, 456,   45888, 64296,
                36556, 59456, 21332, 41084, 8276,  37256, 58408, 27360, 42604,
                62952, 15656, 35860, 4656,  18048, 10364, 40928, 1868,  48048,
                16256, 39084, 220,   30228, 50364, 6044,  53456, 29156, 13308,
                6684,  36416, 56108, 42564, 27340, 63168, 40080, 11140, 55872,
                20916, 5660,  58652, 14776, 39424, 2812,  31208, 45012, 40284,
                58584, 23532, 3712,  53340, 21248, 28112, 6964,  62792, 19304,
                38072, 25400, 48608, 21156, 17396, 5480,  52348, 8456,  26792,
                49812, 56148, 17236, 48136, 26452, 51952, 37780, 31396, 57424,
                5536,  46880, 12392, 60828, 37732, 10540, 63496, 648,   15328,
                56560, 32584, 3072,  54548, 19932, 41804, 11764, 32636, 50964,
                62448, 36368, 23904, 11112, 33636, 55892, 37496, 63404, 52324,
                34412, 55040, 1340,  32784, 15596, 56320, 45100, 20348, 51132,
                33020, 46564, 22240, 15568, 29332, 8888,  21412, 26440, 53740,
                41660, 50652,
            },
            {
                63252, 54900, 60224, 8332,  16032, 56540, 31684, 11684, 18072,
                54224, 5364,  26260, 61760, 18840, 55148, 3116,  13952, 47796,
                8932,  29100, 22620, 47064, 27004, 52000, 34048, 63380, 43508,
                52732, 8840,  22068, 57640, 18276, 25580, 9548,  61656, 40280,
                52252, 64300, 19916, 46768, 13416, 35584, 19396, 46240, 16160,
                64284, 27028, 51352, 23688, 35176, 64752, 7592,  19364, 34604,
                9728,  47836, 12936, 61068, 49584, 37196, 31840, 23228, 46656,
                55796, 3024,  34840, 62400, 28064, 57240, 37372, 14936, 40096,
                60496, 6576,  13276, 39124, 2140,  8220,  44904, 14208, 25708,
                34540, 17776, 30868, 52120, 3648,  24952, 57896, 40000, 5796,
                46664, 23836, 43176, 16604, 28976, 4888,  56392, 22844, 9400,
                19676, 47408, 7404,  43824, 18576, 5196,  45264, 8508,  17436,
                26880, 61736, 38148, 9764,  40836, 26392, 2716,  36832, 16360,
                9632,  57832, 35948, 42384, 56616, 50428, 44692, 11488, 59628,
                7528,  23856,
            },
            {
                17652, 11088, 21620, 36248, 46932, 41036, 5860,  49452, 27924,
                42480, 14452, 50448, 45956, 12768, 32324, 64572, 21680, 34108,
                1532,  56084, 40200, 65092, 14020, 58200, 7420,  28656, 12596,
                25824, 60476, 33768, 42896, 65436, 38068, 47592, 16424, 34340,
                26532, 10884, 892,   60500, 50572, 3136,  32148, 58172, 4404,
                42380, 33788, 8380,  17832, 46080, 53892, 25612, 49488, 15180,
                62984, 26676, 33412, 41368, 1568,  44516, 58912, 12584, 40300,
                8768,  29868, 51924, 41468, 10420, 32520, 47840, 63996, 30132,
                1480,  24212, 42776, 54688, 30468, 22232, 62880, 53976, 9664,
                49768, 64980, 41444, 7952,  20448, 33256, 42976, 29120, 19584,
                51608, 10804, 64928, 49496, 59372, 37956, 26640, 61080, 42632,
                28540, 54420, 31820, 65056, 51956, 28080, 21076, 49916, 11532,
                42124, 49088, 19936, 24364, 51696, 64568, 7684,  29256, 60972,
                24256, 1160,  27696, 12288, 18628, 61584, 2132,  31704, 36012,
                14796, 47248,
            },
            {
                38020, 45444, 32692, 51764, 12904, 26232, 19504, 61248, 33904,
                8632,  57756, 38524, 816,   53096, 28300, 44316, 38264, 52428,
                59636, 19884, 11432, 43292, 2344,  38220, 20532, 56680, 3756,
                18772, 46800, 5244,  10848, 14420, 31376, 1324,  44060, 4356,
                50016, 32052, 23780, 38664, 28768, 8004,  53948, 25764, 37860,
                48956, 12848, 61896, 43180, 4592,  59656, 12400, 37648, 56572,
                5792,  20068, 55104, 8540,  18560, 25644, 5372,  53452, 33920,
                17064, 61200, 13936, 6440,  45384, 23884, 3784,  11660, 21284,
                53248, 33840, 63432, 10248, 58476, 35248, 17336, 4556,  40108,
                23300, 1280,  28200, 58824, 47492, 16292, 54208, 12988, 61304,
                37248, 31072, 1624,  13616, 34160, 7656,  48112, 3180,  15212,
                39224, 336,   25484, 14496, 40936, 2408,  60940, 35560, 57764,
                6160,  30056, 3360,  58148, 13196, 32040, 47264, 54744, 40296,
                49648, 44084, 65024, 4132,  37696, 24100, 55272, 40416, 64524,
                4840,  29732,
            },
            {
                1608,  40692, 6168,  64696, 2600,  58520, 37892, 53300, 1876,
                24092, 30548, 17480, 34908, 23384, 6196,  9984,  15928, 25432,
                7128,  49460, 32992, 54324, 24204, 30464, 45212, 49300, 36100,
                55000, 39804, 27392, 49400, 56372, 23560, 54256, 60624, 21736,
                58420, 42116, 14860, 56856, 17504, 44120, 63652, 10188, 22500,
                29748, 600,   54928, 31732, 21424, 28524, 41092, 3632,  30744,
                51676, 45776, 29016, 38936, 57776, 14232, 50292, 63744, 468,
                43432, 22168, 56676, 38460, 19324, 58648, 51100, 35044, 44496,
                17752, 46812, 28036, 15308, 48908, 41544, 27476, 46344, 57768,
                37140, 15220, 54672, 11884, 38908, 62772, 2196,  48804, 26412,
                6908,  57160, 40492, 25304, 21144, 44128, 18264, 51796, 35232,
                57004, 6500,  59360, 48220, 9960,  56456, 30708, 23376, 39088,
                16124, 63712, 46060, 36176, 17588, 42684, 21972, 11624, 18968,
                5472,  15132, 54388, 30736, 47396, 7304,  28212, 16696, 20980,
                51316, 56904,
            },
            {
                61816, 19560, 27916, 55492, 30876, 22764, 42980, 15148, 47560,
                65472, 44112, 10844, 62664, 58860, 49036, 41472, 61108, 46216,
                30776, 37364, 3928,  17180, 61472, 8776,  15636, 64188, 23016,
                31660, 14988, 62148, 2332,  35636, 41292, 7344,  28596, 18496,
                9196,  46496, 5148,  52524, 34060, 21064, 40868, 14408, 60252,
                52584, 19240, 38776, 57712, 9812,  48236, 16960, 63816, 21892,
                42524, 2576,  16048, 65476, 47220, 30848, 36148, 19684, 28272,
                47920, 25872, 49876, 2800,  64972, 27612, 15608, 41020, 60192,
                8976,  4368,  38092, 20716, 6356,  612,   51748, 12460, 29924,
                61968, 20816, 45436, 32440, 4924,  23520, 36168, 9268,  44324,
                22256, 17804, 46208, 52664, 62316, 55712, 12232, 29820, 63968,
                45512, 18876, 22416, 37592, 34032, 16868, 46556, 12752, 712,
                44116, 26148, 9332,  53032, 5060,  59988, 516,   34668, 62912,
                26444, 33580, 39292, 20468, 51928, 13584, 43448, 49396, 10208,
                34348, 24884,
            },
            {
                53352, 13656, 48300, 9656,  17204, 50412, 7112,  11224, 36068,
                20988, 54972, 4868,  40012, 19732, 29780, 3532,  54096, 18396,
                12964, 63328, 27704, 45820, 51028, 34324, 41248, 776,   11316,
                7192,  43716, 52024, 21204, 17320, 63200, 12112, 51900, 39440,
                35352, 64960, 26140, 11776, 61564, 6484,  49260, 2676,  35272,
                44944, 6704,  25384, 13960, 35852, 1508,  52932, 34040, 10824,
                60620, 36020, 24540, 11460, 4208,  22372, 42272, 10404, 60112,
                6076,  33328, 11180, 36280, 31012, 8408,  54476, 104,   26180,
                56316, 62152, 50636, 31568, 59412, 25064, 64460, 19120, 33556,
                2628,  6468,  41800, 26200, 50476, 19012, 30800, 60268, 52064,
                34004, 63760, 4320,  9680,  32072, 1060,  41116, 23796, 9084,
                32760, 12644, 50824, 63288, 26656, 4228,  54412, 62488, 51276,
                33272, 55608, 21548, 31028, 49580, 27528, 38312, 56336, 51820,
                8372,  46620, 57456, 9828,  59936, 34896, 62524, 964,   57884,
                44580, 7792,
            },
            {
                32080, 41844, 23624, 35668, 44824, 62328, 32632, 59996, 28832,
                3132,  50792, 27168, 46632, 14592, 56688, 33784, 22348, 124,
                40360, 56404, 10064, 21296, 5708,  57732, 25772, 52988, 60580,
                37864, 58496, 29652, 9336,  45620, 25936, 47984, 32832, 15408,
                54960, 3468,  30424, 37820, 45676, 23632, 31512, 58036, 27752,
                16580, 65268, 41232, 50164, 62424, 43600, 24796, 58368, 27564,
                7960,  48748, 56428, 32352, 51356, 59060, 7504,  54356, 39104,
                15164, 55308, 61912, 17612, 42764, 46996, 22008, 38788, 32232,
                13256, 23536, 42956, 11492, 53828, 38688, 43208, 8844,  56196,
                47612, 52416, 59528, 9916,  63608, 55456, 41168, 15492, 3460,
                11984, 29464, 38604, 15192, 26836, 37356, 58500, 4880,  53936,
                27620, 41688, 3424,  43688, 7904,  20684, 40456, 28696, 18296,
                7080,  59472, 13636, 40180, 64172, 10620, 24096, 43436, 14280,
                29984, 22776, 2552,  17356, 25640, 4608,  31336, 22944, 39008,
                27648, 16068,
            },
            {
                3308,  58076, 63188, 520,   39216, 4992,  25800, 18368, 56112,
                41216, 16220, 32968, 7696,  37636, 11768, 26128, 64908, 35836,
                51708, 24400, 42628, 60004, 38384, 13092, 19532, 28868, 47160,
                17892, 24740, 4328,  33956, 57304, 38332, 5872,  1552,  61352,
                24108, 48732, 20028, 59188, 320,   16216, 52108, 39004, 8664,
                47544, 32712, 4016,  22136, 30028, 7120,  19548, 45576, 14340,
                40188, 18292, 624,   43884, 37992, 26540, 16752, 46324, 30536,
                21256, 44708, 1736,  24052, 52380, 12736, 63184, 6340,  48992,
                18868, 36692, 2044,  16872, 34504, 6776,  22520, 15836, 36744,
                27172, 23064, 17584, 34744, 13888, 1576,  46964, 27192, 58680,
                42960, 55832, 47852, 60888, 50756, 17444, 46420, 20620, 48912,
                61388, 16312, 57560, 30928, 52388, 59696, 35716, 10020, 42812,
                24760, 37008, 1696,  18760, 44884, 6860,  16612, 48712, 4420,
                60292, 41400, 63696, 44816, 37384, 55888, 42160, 19028, 12800,
                60872, 46888,
            },
            {
                20844, 51460, 11948, 28488, 56504, 13968, 52848, 46696, 37532,
                8532,  63488, 24504, 53000, 61800, 49836, 43804, 8920,  47452,
                5456,  14996, 30044, 1520,  48672, 31688, 44836, 3060,  34752,
                12656, 49952, 41580, 64860, 13384, 53304, 30592, 21060, 44576,
                40384, 7924,  14264, 42536, 55752, 26988, 63936, 12440, 20600,
                56996, 53980, 11272, 60584, 16076, 38508, 55220, 2868,  50500,
                30560, 54036, 63284, 20436, 13328, 61772, 2108,  34768, 64512,
                4936,  50988, 40516, 29480, 56920, 3828,  34296, 58924, 43956,
                29544, 55056, 64144, 47008, 28376, 61284, 45072, 50176, 60120,
                11576, 39908, 6728,  44052, 29656, 37972, 5960,  21840, 36572,
                18500, 24584, 7472,  21364, 34644, 6100,  65512, 14136, 33728,
                1788,  36640, 24440, 11164, 47696, 14612, 23568, 4872,  50100,
                63472, 46480, 53620, 33928, 57312, 28972, 62188, 32924, 20616,
                36356, 12184, 50620, 28060, 14380, 47972, 6784,  65340, 53900,
                4892,  36480,
            },
            {
                39840, 7964,  18116, 33760, 48852, 21152, 42412, 1492,  23168,
                12428, 57288, 42808, 1712,  21504, 4244,  17060, 59320, 31760,
                19968, 62832, 54696, 35212, 16860, 65276, 9480,  56308, 61784,
                6556,  55068, 20104, 292,   23924, 16780, 49700, 59404, 11400,
                28328, 63628, 50680, 31780, 9668,  35436, 4708,  43900, 24748,
                1800,  36664, 27276, 45860, 52328, 33288, 11820, 64724, 36412,
                5092,  23864, 34660, 9208,  29596, 52752, 48904, 23952, 11756,
                27412, 58836, 14292, 8896,  37656, 20464, 27100, 15032, 10132,
                51468, 4028,  8904,  25616, 56896, 12820, 4816,  30620, 3108,
                32720, 65236, 54856, 49064, 24240, 57128, 50788, 64456, 10192,
                53164, 568,   40348, 12732, 54556, 28720, 42528, 25008, 40008,
                8752,  53004, 19956, 39436, 876,   65008, 45052, 57236, 31668,
                16184, 11304, 26720, 3748,  22004, 52092, 37616, 1392,  58364,
                53344, 7832,  32132, 204,   21160, 52460, 11280, 29012, 33936,
                49124, 24468,
            },
            {
                30508, 59268, 45548, 25200, 64416, 9936,  30192, 61900, 34384,
                49560, 29096, 18816, 35592, 30636, 55460, 40484, 27932, 38120,
                11560, 49964, 26224, 8124,  22440, 51596, 41808, 20904, 39068,
                26408, 44372, 30096, 36916, 60732, 43288, 8760,  35196, 56144,
                18044, 33988, 3052,  22728, 46724, 18328, 40268, 50784, 60172,
                31260, 17728, 42052, 8136,  84,    58192, 28320, 21480, 43028,
                60868, 15732, 47136, 57120, 39344, 5848,  18200, 40908, 56552,
                36900, 19592, 33588, 63908, 49532, 44900, 61168, 41316, 24272,
                57840, 39132, 21164, 32608, 50972, 18732, 40584, 55360, 21308,
                46256, 15116, 196,   20276, 61880, 8168,  16972, 33208, 28564,
                44300, 62876, 31352, 59200, 45404, 2548,  11000, 56268, 30244,
                59312, 44820, 62324, 27292, 34188, 17992, 29444, 38160, 2384,
                21420, 40732, 60156, 48320, 14616, 41548, 11688, 46728, 27408,
                18088, 23908, 40340, 56700, 60552, 35380, 25004, 45348, 16884,
                9460,  56160,
            },
            {
                14024, 42652, 2244,  53460, 37908, 6324,  17636, 58280, 4276,
                15640, 45376, 6172,  60428, 48492, 13524, 7240,  23196, 56868,
                43236, 4604,  45788, 39704, 57536, 27920, 5052,  14124, 32212,
                52544, 15868, 10580, 56532, 5212,  27952, 39452, 2528,  24880,
                52412, 6084,  38708, 58188, 53772, 61908, 28744, 14508, 7180,
                46520, 64356, 13604, 49372, 24544, 40560, 17196, 47932, 7864,
                26304, 11348, 41512, 3520,  22104, 32280, 60208, 9620,  44336,
                328,   47936, 6604,  25692, 17184, 1164,  53024, 5388,  18140,
                33608, 46144, 14268, 42492, 748,   37020, 63676, 24728, 8620,
                52548, 37520, 28088, 41324, 32004, 12712, 45796, 39260, 2900,
                14528, 49464, 26204, 17052, 37500, 22428, 62040, 51256, 5468,
                17496, 13020, 4012,  50232, 56236, 7264,  52784, 12692, 61992,
                55484, 7660,  36096, 30212, 65416, 8444,  25036, 55996, 6092,
                44400, 64652, 49016, 15752, 8992,  39308, 3100,  58412, 41484,
                1052,  63584,
            },
            {
                27200, 35508, 5444,  16156, 59980, 32548, 47924, 40116, 27084,
                51840, 64984, 11052, 38996, 24912, 44992, 64132, 52488, 2004,
                16368, 33260, 60832, 12836, 2700,  36808, 62600, 48088, 23708,
                2220,  63880, 49084, 33280, 46376, 19152, 64388, 14112, 47568,
                42104, 62352, 15860, 26276, 12084, 1284,  21760, 33828, 53108,
                38084, 23028, 54836, 19876, 35396, 62960, 3584,  56772, 32740,
                52856, 59256, 28556, 50856, 64924, 46172, 26896, 53836, 16016,
                31140, 61616, 42436, 54348, 10592, 32268, 29132, 37224, 65388,
                7576,  27992, 59280, 62416, 10332, 48424, 28856, 14048, 34464,
                61428, 18132, 10612, 58988, 4412,  53652, 25020, 60668, 55984,
                21044, 35276, 4472,  52480, 8012,  32824, 48012, 19452, 35864,
                46700, 24020, 31992, 43128, 10240, 41060, 22708, 26328, 46252,
                33140, 51128, 17384, 4612,  44548, 19412, 32248, 61408, 39100,
                13228, 34584, 4636,  30208, 45924, 18888, 62956, 31508, 22352,
                50376, 19612,
            },
            {
                46708, 61512, 51220, 21640, 11620, 44364, 24112, 54804, 8812,
                36856, 21868, 32696, 54012, 672,   18240, 32180, 10280, 36552,
                62452, 24800, 19132, 53984, 30768, 16180, 44536, 10124, 35496,
                57864, 8072,  40176, 22636, 12436, 54292, 50432, 31528, 21904,
                10428, 29380, 45160, 36260, 8140,  43220, 49128, 58572, 10448,
                2708,  29696, 5344,  59772, 31064, 9804,  44432, 13940, 38056,
                19756, 1016,  36984, 16768, 7936,  13676, 34968, 3356,  23360,
                50684, 12996, 20748, 38956, 59760, 46468, 13592, 22124, 48312,
                54560, 2404,  17456, 52616, 22880, 6156,  58688, 44328, 2704,
                42000, 50332, 23728, 47600, 35412, 43164, 19124, 6120,  47356,
                9592,  40808, 60256, 11968, 57464, 39652, 14436, 308,   27880,
                54484, 38340, 64408, 21184, 15536, 60360, 48792, 6060,  19188,
                43388, 10744, 23936, 58064, 53548, 37056, 48924, 3080,  16992,
                28800, 52404, 21824, 61868, 26176, 55152, 5308,  14480, 53524,
                37752, 11184,
            },
            {
                55192, 24024, 31592, 41636, 28428, 56988, 280,   13576, 19444,
                43576, 3448,  57540, 14708, 28416, 59420, 41992, 21020, 47376,
                29136, 51720, 8796,  42208, 59508, 25372, 50696, 55452, 18648,
                42748, 27400, 17188, 59616, 2968,  25892, 7368,  36452, 60124,
                508,   56744, 19192, 50224, 65216, 31620, 38888, 16092, 26612,
                62612, 43476, 12264, 39460, 50804, 26472, 54116, 23200, 6188,
                49708, 62684, 24264, 44676, 55632, 20860, 41908, 63272, 37320,
                57548, 28704, 5784,  35084, 23540, 56668, 3408,  60740, 42916,
                12352, 35952, 25292, 40120, 31420, 35268, 16944, 53744, 26648,
                6692,  56852, 29888, 1988,  64000, 15484, 27756, 37168, 30804,
                65084, 23916, 18236, 46260, 29276, 25148, 64112, 43772, 60300,
                9912,  6524,  57336, 1616,  36904, 30184, 34428, 57964, 88,
                29164, 63620, 39976, 27656, 1200,  12840, 22408, 54820, 43044,
                59444, 10328, 2248,  42180, 37000, 11568, 48596, 43504, 28044,
                33264, 6624,
            },
            {
                40128, 17744, 8848,  65136, 4484,  50208, 35228, 63220, 31052,
                59904, 26420, 40864, 49360, 7564,  35956, 51084, 3280,  58556,
                13784, 37976, 476,   47772, 20724, 5940,  33164, 1352,  29348,
                61956, 4812,  45728, 35844, 30004, 62204, 44408, 16280, 40536,
                52984, 33372, 4104,  13672, 24304, 55200, 3552,  20108, 45700,
                35784, 56584, 18452, 47208, 1936,  16100, 59124, 41724, 30144,
                46336, 12916, 34448, 30884, 2152,  58776, 48276, 5488,  18524,
                9272,  45716, 64852, 2000,  15852, 50996, 40576, 18768, 9560,
                30788, 50520, 45304, 4744,  56472, 12068, 47340, 20760, 64340,
                33036, 13580, 39724, 21496, 8580,  50708, 58204, 11760, 52972,
                1008,  50096, 33544, 42460, 3256,  55532, 6772,  31788, 20968,
                50860, 16544, 25812, 47736, 53160, 4632,  63096, 13140, 37724,
                54784, 15188, 7056,  33964, 50340, 62472, 30524, 9092,  35624,
                24812, 47292, 57448, 15236, 51156, 33664, 23696, 8628,  64092,
                59488, 2672,
            },
            {
                13612, 58084, 47516, 34276, 15076, 20192, 38976, 7044,  47748,
                52860, 10644, 17900, 45508, 23148, 61628, 12280, 26488, 6376,
                44648, 22584, 63532, 35012, 11604, 40616, 64760, 13808, 38620,
                22036, 49392, 13244, 51912, 39200, 10184, 56516, 20548, 6264,
                27076, 46736, 22452, 57988, 41380, 28552, 6416,  61116, 51824,
                23584, 8156,  32716, 63388, 21948, 7508,  34256, 65356, 18184,
                4160,  58340, 8800,  53200, 39752, 11008, 27972, 32512, 52176,
                39448, 25840, 49412, 30900, 44256, 8100,  27588, 33924, 24116,
                57316, 61392, 14896, 20280, 62872, 42324, 1436,  38660, 9640,
                49108, 17644, 60952, 54580, 32348, 40720, 4940,  22152, 43888,
                15768, 27072, 7276,  63032, 21400, 16336, 48800, 12356, 41708,
                33856, 39716, 29044, 44092, 10792, 24368, 17424, 42196, 51568,
                22188, 48008, 58940, 20076, 41916, 16464, 46196, 5116,  64208,
                18672, 38648, 31628, 7000,  19908, 58976, 828,   39756, 16492,
                21112, 44748,
            },
            {
                29624, 37172, 1300,  53816, 26688, 60420, 45836, 23636, 15720,
                2320,  33824, 64020, 4504,  30336, 55224, 16420, 33948, 41008,
                55804, 31112, 15788, 52708, 28004, 56732, 23564, 46248, 53988,
                9768,  32020, 57380, 19628, 1072,  23992, 32920, 49148, 64196,
                11912, 38208, 61020, 7224,  35344, 17676, 47808, 12672, 40728,
                616,   14992, 27804, 53276, 36964, 45200, 28192, 10532, 38704,
                52204, 27088, 42248, 22320, 17544, 62260, 24524, 14220, 60716,
                3152,  12580, 56020, 19448, 60416, 37772, 53776, 63720, 5900,
                39244, 48,    28912, 9068,  49380, 25688, 60092, 30128, 23408,
                35656, 3400,  44476, 11044, 25852, 46416, 62996, 34416, 55824,
                39356, 58968, 10888, 51580, 35036, 38188, 53760, 24220, 58840,
                2508,  61052, 13480, 63632, 20160, 56272, 46544, 27140, 8860,
                31692, 3192,  44980, 26156, 10152, 55628, 39184, 27892, 50820,
                396,   12516, 53412, 27300, 62720, 46180, 29832, 56256, 52192,
                26456, 49928,
            },
            {
                63332, 19400, 24704, 43080, 8076,  52104, 11328, 29592, 57772,
                40024, 25068, 37124, 53400, 9624,  39152, 47620, 64940, 18428,
                50000, 5128,  60612, 3440,  43528, 7560,  16728, 35772, 4280,
                60068, 26064, 6744,  65304, 41832, 53780, 14488, 3380,  29796,
                43468, 16760, 51292, 10652, 44608, 63824, 54196, 33860, 29344,
                59724, 48928, 42832, 3656,  57816, 13716, 50396, 412,   55820,
                20956, 14732, 63508, 48984, 6052,  37556, 46652, 55312, 43756,
                22056, 34556, 41172, 10140, 24900, 14036, 3896,  48044, 16916,
                43904, 22080, 54232, 37260, 32620, 16120, 7424,  55388, 41504,
                58944, 51328, 28176, 37580, 488,   18180, 13476, 29688, 2796,
                19920, 47192, 31252, 14084, 58028, 1560,  28092, 8660,  45800,
                19252, 5272,  52240, 35888, 7756,  32752, 39220, 4180,  62060,
                35808, 12408, 64848, 37488, 2340,  60564, 23312, 14500, 33312,
                43912, 60908, 22052, 40956, 3968,  17304, 36696, 12724, 5620,
                35312, 9732,
            },
            {
                4032,  55644, 12532, 61920, 31496, 36348, 1620,  42624, 18972,
                49748, 60676, 13760, 20472, 43808, 1412,  22304, 7256,  28760,
                10668, 24924, 39540, 19556, 49080, 32260, 61612, 50832, 20524,
                39932, 44360, 34708, 16584, 28324, 46244, 36580, 58236, 22828,
                54404, 1588,  26096, 30972, 20688, 2172,  25348, 8612,  21272,
                38044, 55252, 11048, 24232, 19268, 40184, 61972, 25044, 32328,
                43840, 36004, 2524,  29336, 54240, 33092, 956,   8256,  29648,
                17192, 62852, 53080, 548,   33340, 57460, 21148, 29976, 52180,
                34892, 12700, 46824, 65016, 4056,  51784, 45664, 12456, 19224,
                5320,  14944, 65396, 21032, 56636, 48260, 60100, 25192, 9060,
                36220, 62296, 5676,  25532, 45440, 42184, 18212, 64968, 36568,
                55172, 31164, 22876, 42788, 740,   49724, 59920, 15304, 19748,
                57252, 23720, 52896, 17628, 31436, 6408,  52048, 58604, 20136,
                8252,  56088, 35968, 49168, 10072, 43324, 65232, 25184, 47244,
                60880, 42232,
            },
            {
                32780, 38864, 48248, 5124,  21996, 16764, 55524, 64716, 4904,
                31844, 7932,  46668, 28536, 62756, 32892, 59580, 52364, 36560,
                57408, 45644, 34096, 12832, 58612, 25288, 10772, 29764, 276,
                15016, 52464, 11308, 48476, 4428,  60568, 8852,  40332, 18484,
                33880, 63376, 37328, 56580, 49092, 39628, 15576, 43544, 62660,
                5144,  17032, 35164, 64532, 31808, 4852,  47872, 16856, 58804,
                5628,  47060, 12244, 59640, 40912, 15816, 20728, 58432, 36788,
                49164, 6392,  45344, 27272, 64220, 46636, 42284, 11220, 62588,
                7328,  59224, 26500, 17944, 40324, 22528, 34572, 63440, 27188,
                47464, 31968, 8316,  53308, 34876, 6232,  41772, 51408, 44552,
                53728, 16980, 40532, 22192, 60948, 4824,  30476, 50672, 11916,
                26592, 48428, 16232, 57848, 25576, 12548, 29456, 54140, 43556,
                47324, 5548,  28464, 42696, 48932, 35220, 11720, 41164, 47656,
                30092, 3412,  25440, 14412, 33168, 55300, 20568, 1836,  31652,
                15156, 22488,
            },
            {
                51540, 17276, 27808, 44844, 57968, 40480, 48588, 27152, 12364,
                52576, 22656, 3488,  55912, 16036, 11404, 26056, 42800, 14588,
                792,   63864, 21872, 54640, 2056,  37940, 47216, 42012, 56040,
                62212, 23352, 30760, 55684, 20636, 25388, 12976, 50848, 5380,
                47668, 9628,  15020, 6040,  60520, 11416, 52124, 57228, 30724,
                46488, 26672, 51976, 8548,  45852, 28936, 11852, 36724, 8952,
                23000, 51536, 19064, 25808, 9352,  65108, 51324, 41996, 11640,
                24376, 31920, 15376, 39400, 7852,  17692, 36272, 1584,  24200,
                41144, 31816, 2208,  55972, 10752, 29152, 53840, 796,   37996,
                57596, 43552, 24172, 39708, 12332, 30520, 16112, 22824, 64756,
                3876,  28280, 49720, 54716, 10028, 34008, 57080, 15560, 6388,
                40856, 62788, 9696,  38464, 45180, 65344, 21652, 2260,  33792,
                10376, 39760, 60932, 14128, 21672, 63464, 27044, 1636,  37308,
                64660, 17772, 52228, 61440, 29028, 7580,  50244, 40508, 54332,
                8088,  58752,
            },
            {
                432,   10476, 64452, 34388, 14040, 8700,  24228, 33540, 38348,
                44352, 59324, 35236, 41352, 50796, 37920, 6368,  20024, 31212,
                48520, 8128,  28236, 41592, 16040, 6612,  64040, 17788, 27336,
                8980,  36968, 2532,  63308, 35536, 43988, 31676, 62168, 27424,
                59004, 42820, 23900, 45896, 32988, 28124, 35832, 2740,  19580,
                13268, 60796, 1064,  42300, 56892, 21220, 61016, 53876, 41612,
                64240, 30908, 56492, 35404, 45324, 23184, 28572, 4736,  61112,
                56212, 2828,  51160, 58884, 23140, 54764, 50108, 28356, 57032,
                15808, 48828, 20840, 44448, 60964, 7004,  42944, 15288, 21524,
                10924, 3020,  49996, 18632, 62184, 58336, 1688,  38204, 10736,
                31984, 13216, 35572, 108,   19072, 47020, 39176, 23804, 44752,
                1188,  33508, 20784, 3676,  32096, 52532, 8364,  37352, 26536,
                17116, 51008, 500,   55280, 8052,  45184, 56736, 16192, 24064,
                9872,  44940, 39412, 4948,  46304, 23596, 59708, 37600, 18304,
                28500, 45152,
            },
            {
                36432, 53896, 20656, 3436,  50076, 60648, 5500,  19856, 62076,
                15500, 10260, 25584, 18416, 2192,  58592, 45216, 54824, 62008,
                38540, 17808, 53052, 61012, 31024, 51040, 22808, 34444, 5232,
                45528, 49616, 41300, 17956, 8020,  53068, 852,   15764, 38284,
                20476, 30304, 53560, 16,    17352, 21848, 64608, 7820,  50280,
                41064, 33064, 23296, 37016, 14960, 50556, 2160,  33364, 26756,
                3300,  15372, 38940, 6940,  2188,  48600, 13680, 34064, 18372,
                43844, 37680, 20596, 10828, 31172, 4100,  13596, 61064, 38316,
                9428,  53140, 34332, 13440, 37448, 24528, 48680, 62252, 32672,
                52440, 60456, 29200, 4760,  45880, 26644, 33908, 49192, 56952,
                20740, 59284, 43516, 61980, 26980, 12668, 63752, 52880, 28772,
                60448, 55604, 51148, 14608, 58560, 18096, 40704, 48340, 56096,
                63976, 31108, 24956, 37988, 33700, 18512, 4972,  53272, 49712,
                59376, 33980, 21460, 12900, 56976, 16700, 2916,  11836, 62280,
                49000, 24000,
            },
            {
                31236, 60044, 41588, 25392, 30120, 36820, 54316, 46500, 168,
                55332, 30648, 48564, 65468, 29316, 22860, 33764, 4528,  11904,
                23928, 44248, 3832,  10464, 36408, 45132, 13004, 59016, 53416,
                20908, 60052, 14180, 28864, 58024, 47528, 23432, 41956, 55908,
                3880,  12564, 62540, 39140, 56044, 42260, 47584, 24152, 37672,
                54852, 9816,  18620, 59352, 6288,  25596, 39824, 17856, 49596,
                46116, 11144, 60332, 54040, 31756, 62536, 39372, 52844, 26544,
                8808,  64528, 28616, 48092, 62224, 44264, 33476, 19104, 46380,
                4536,  26956, 64432, 3464,  30624, 57948, 18756, 5924,  40916,
                25696, 16648, 36768, 42464, 9408,  54712, 19460, 7160,  40420,
                46584, 24652, 8112,  38616, 51684, 32092, 2764,  21024, 9032,
                18008, 37068, 25896, 46484, 28828, 4820,  23872, 61644, 13368,
                7172,  20152, 46152, 11436, 59596, 28952, 42088, 32044, 12476,
                27848, 2452,  62676, 30828, 41960, 35904, 26608, 33500, 43240,
                5032,  13848,
            },
            {
                39368, 7600,  15908, 57164, 43932, 18000, 9908,  28148, 42200,
                21764, 4704,  38716, 7748,  14032, 46876, 51936, 16380, 27732,
                56708, 34648, 49480, 26516, 19740, 56524, 2604,  29564, 39380,
                10852, 32796, 25516, 4544,  37716, 12156, 64252, 33240, 9052,
                51792, 34972, 25668, 49908, 10348, 5064,  13840, 59216, 27432,
                4268,  63604, 47824, 28492, 44620, 55816, 13012, 7676,  62208,
                20392, 29088, 42516, 22116, 17212, 10740, 57952, 724,   47068,
                14868, 54168, 35388, 1252,  40796, 25564, 6016,  54264, 22000,
                42048, 58268, 16996, 50316, 46164, 9864,  54656, 35680, 2068,
                46784, 55664, 12944, 63540, 23192, 51740, 14352, 63108, 28648,
                16692, 4436,  55568, 14920, 22604, 5436,  58312, 35420, 43048,
                48576, 12192, 6824,  41972, 60808, 10756, 32844, 44628, 2768,
                35944, 42804, 53404, 4084,  48240, 62120, 21960, 38832, 57656,
                6952,  43628, 54076, 9136,  50568, 63708, 46936, 8308,  56460,
                19392, 63884,
            },
            {
                2632,  46892, 52724, 11808, 1864,  61704, 51556, 13544, 63900,
                35456, 50608, 57192, 43320, 60916, 10092, 37236, 63580, 42204,
                2016,  59680, 14748, 40292, 65096, 8284,  24752, 43068, 16552,
                916,   64704, 54708, 43304, 50744, 21600, 3124,  26976, 46948,
                18336, 43828, 7132,  19652, 31320, 61236, 34640, 16928, 44812,
                31716, 12396, 52392, 1684,  34312, 64648, 30492, 43652, 36504,
                57452, 5916,  51064, 26296, 36052, 44260, 20560, 30440, 23792,
                41500, 5808,  19272, 12728, 56812, 16372, 36112, 63612, 12060,
                31488, 36844, 8240,  21136, 39208, 27868, 14400, 22716, 64644,
                30512, 7648,  20708, 34436, 380,   30992, 43796, 36604, 2204,
                52820, 33248, 64304, 29572, 45192, 49280, 40664, 14332, 56056,
                30808, 2388,  64556, 21580, 35204, 49176, 54268, 16284, 27740,
                59816, 22992, 57432, 16812, 25992, 8960,  1032,  15184, 47608,
                19796, 37048, 17200, 24532, 1432,  20500, 14904, 52496, 30444,
                24840, 50408,
            },
            {
                35192, 22096, 27560, 33640, 38636, 23176, 31276, 39988, 6992,
                24864, 15940, 32476, 19568, 26272, 716,   31804, 21264, 47976,
                9344,  30772, 22108, 5484,  52256, 32344, 47820, 61596, 51404,
                34848, 46744, 18476, 9708,  32028, 59500, 16272, 37072, 60992,
                56876, 14068, 65464, 37748, 46476, 26712, 52668, 1248,  57668,
                21380, 42044, 16096, 38368, 19232, 10040, 21956, 51692, 24616,
                180,   33632, 13988, 61744, 3512,  50024, 7716,  63644, 34568,
                58580, 50400, 60640, 22968, 46204, 51960, 9224,  28132, 48024,
                1928,  52624, 25444, 60712, 260,   52152, 59184, 42400, 11236,
                50728, 39324, 59868, 48000, 40884, 56848, 11152, 25500, 61132,
                48304, 42064, 9796,  36084, 17908, 60844, 7416,  27332, 62864,
                23460, 51456, 39288, 13780, 57132, 216,   19924, 38796, 51308,
                9836,  32244, 13892, 39384, 34980, 52656, 43936, 64932, 34280,
                26496, 61060, 48808, 40592, 28624, 55900, 39136, 3916,  61156,
                41812, 10096,
            },
            {
                55444, 59556, 18812, 65140, 48688, 5612,  58700, 20056, 46852,
                59768, 11548, 3544,  52588, 40432, 55024, 58928, 6884,  18056,
                38428, 54284, 62688, 45876, 17284, 36988, 13540, 4748,  21616,
                26876, 57528, 6896,  24164, 40236, 54080, 44644, 6184,  30528,
                23764, 1344,  28676, 58400, 3420,  22608, 40452, 9000,  48776,
                36252, 5836,  24432, 61276, 54188, 48872, 3096,  58764, 15916,
                40216, 54368, 47092, 9488,  57180, 40640, 16020, 53632, 11996,
                3320,  27468, 32604, 39560, 29736, 2844,  59116, 43336, 23912,
                61712, 14092, 45388, 41436, 31124, 5636,  33756, 45752, 18508,
                27556, 3528,  15616, 24964, 5384,  18144, 50352, 6364,  22048,
                12932, 19308, 26556, 836,   54924, 11360, 33572, 19896, 4224,
                44704, 16664, 32428, 26884, 8492,  43204, 30576, 64392, 5736,
                47348, 41712, 1716,  63672, 49772, 29964, 11920, 23740, 54888,
                4908,  10156, 13920, 58936, 5572,  32736, 11628, 22592, 36520,
                16244, 28756,
            },
            {
                13436, 42924, 3692,  9356,  44732, 15400, 34740, 1336,  53880,
                43712, 28360, 63400, 36496, 23400, 14832, 44768, 28956, 51336,
                25600, 12888, 34304, 68,    28252, 60244, 55052, 9516,  42008,
                37932, 14312, 30324, 62584, 1668,  12980, 20576, 49628, 11264,
                39552, 51196, 42448, 15964, 54060, 12104, 63072, 18884, 29576,
                60240, 55324, 46600, 30912, 7280,  26948, 41840, 32272, 11704,
                63384, 19700, 29900, 37624, 23552, 32340, 26908, 46028, 38488,
                18004, 44160, 10296, 6724,  65152, 20796, 38260, 17876, 7408,
                56012, 34772, 18952, 10528, 64204, 16704, 24396, 8160,  61856,
                36924, 57752, 52220, 32964, 65200, 28884, 58984, 45948, 31600,
                37936, 57012, 60080, 50388, 38732, 24472, 46864, 54016, 37392,
                9896,  59248, 53168, 46612, 61416, 24260, 14644, 56048, 25428,
                18912, 60788, 28340, 21276, 5584,  17952, 59160, 3188,  40020,
                51348, 30684, 45848, 35168, 51884, 65380, 44736, 48936, 58472,
                932,   45972,
            },
            {
                5776,  31616, 51996, 24284, 28872, 56800, 50124, 26968, 9220,
                38128, 17400, 47508, 7928,  49796, 11208, 34864, 64592, 2728,
                57912, 42708, 50032, 10664, 23452, 40668, 19000, 31384, 63668,
                3252,  52360, 44196, 36600, 48452, 28752, 34628, 64016, 55004,
                5016,  33912, 21476, 8588,  36080, 31976, 44028, 51320, 13996,
                26120, 2612,  11428, 40004, 14392, 36960, 62344, 47468, 8120,
                26108, 44908, 4052,  17596, 65480, 1872,  55668, 6260,  22376,
                49576, 62708, 55916, 15528, 48460, 34180, 13360, 50968, 32120,
                39688, 4216,  28860, 54520, 47472, 37412, 55968, 48980, 1276,
                21588, 42868, 11860, 45260, 8872,  36040, 14628, 40172, 53916,
                2624,  8300,  44716, 21728, 5864,  65508, 15972, 29816, 61604,
                41284, 1136,  19216, 5880,  37120, 50152, 3704,  34516, 12072,
                37380, 53996, 8584,  43352, 55296, 37104, 46408, 32932, 15572,
                21224, 63300, 64,    19288, 23608, 15488, 6608,  27128, 18980,
                52980, 63420,
            },
            {
                39732, 16656, 36940, 61988, 40528, 11732, 18716, 64316, 22916,
                32708, 55708, 2008,  30896, 60396, 18988, 4284,  41160, 21884,
                15680, 7372,  30308, 55940, 44720, 6300,  50584, 25464, 47460,
                59232, 11056, 20052, 55736, 16564, 60012, 8728,  41356, 25964,
                18704, 46776, 60180, 55880, 49244, 5120,  24040, 38684, 7376,
                34928, 42888, 65196, 22464, 57232, 4360,  17700, 21448, 56128,
                34748, 50456, 59880, 11412, 43260, 48160, 14388, 61044, 30180,
                35540, 404,   24888, 41540, 54068, 26476, 58044, 1076,  64044,
                21208, 49932, 59704, 23204, 2884,  12256, 40612, 29824, 14552,
                54304, 31312, 4468,  23436, 54692, 20180, 3560,  26220, 63940,
                17224, 34720, 28404, 14168, 31452, 42648, 2724,  51048, 13084,
                25504, 35264, 29220, 11880, 55124, 21396, 40828, 62616, 45804,
                2444,  30904, 48324, 12968, 24824, 61748, 9500,  27156, 57624,
                43500, 36384, 28040, 8820,  42784, 57860, 31084, 40964, 9772,
                33800, 25264,
            },
            {
                57220, 48188, 20420, 460,   54380, 6852,  36444, 42792, 4848,
                13388, 61260, 20672, 42084, 26744, 56956, 52792, 32032, 46528,
                60560, 36292, 20412, 62368, 14968, 58672, 34624, 1648,  15996,
                22516, 33704, 5884,  27224, 4024,  23276, 46268, 272,   15200,
                62392, 31136, 12560, 27064, 17904, 64412, 844,   56496, 61040,
                16708, 53208, 19468, 50204, 33944, 44376, 52136, 28512, 1316,
                39080, 5856,  22164, 53328, 27832, 34460, 20268, 9904,  42540,
                57512, 12792, 20508, 37096, 5288,  9504,  42376, 29536, 10928,
                44176, 15564, 7920,  41912, 33016, 18604, 58288, 26004, 63500,
                35424, 17764, 61252, 38588, 47844, 62128, 43256, 10796, 23012,
                46792, 49820, 62716, 40320, 53356, 58436, 34472, 8264,  21116,
                48900, 57552, 45396, 64772, 31784, 17124, 9236,  26428, 51704,
                15884, 22744, 58384, 34656, 764,   40636, 20096, 49440, 6700,
                53944, 12316, 48068, 61892, 54340, 37404, 2720,  51432, 61536,
                21756, 7620,
            },
            {
                29328, 11300, 60284, 33220, 14680, 47180, 30832, 58176, 53036,
                48864, 39620, 10480, 46184, 5332,  36132, 13288, 24340, 9600,
                27784, 48560, 39676, 2512,  26336, 38036, 12048, 54120, 45220,
                29540, 40032, 49792, 64656, 42508, 52748, 32884, 57244, 38408,
                51680, 2904,  44632, 6732,  41096, 29812, 45772, 20996, 32484,
                47944, 9644,  29364, 444,   25244, 9180,  59800, 12540, 43032,
                64420, 14172, 32724, 40164, 7736,  58880, 37844, 50704, 4352,
                26684, 52520, 47512, 30956, 17140, 61952, 22532, 55572, 47440,
                25828, 35568, 53516, 27520, 61624, 50472, 9328,  3844,  43564,
                6544,  50380, 27368, 13844, 784,   30032, 50928, 57652, 33012,
                5376,  12268, 20744, 1328,  10320, 18596, 26632, 43976, 55204,
                4780,  15396, 23048, 42068, 1880,  48696, 60168, 32804, 57680,
                6904,  38248, 65404, 17308, 51524, 31708, 14568, 64124, 2300,
                24044, 17152, 40160, 4292,  25816, 11036, 17940, 46944, 13748,
                35988, 44244,
            },
            {
                50348, 5192,  27036, 44604, 22448, 63080, 25632, 2440,  16332,
                28820, 35008, 24700, 65228, 16744, 48708, 39056, 63100, 912,
                55060, 18148, 5816,  52676, 32152, 21428, 65400, 41852, 8040,
                61520, 56836, 14744, 9676,  37548, 18092, 12208, 28152, 22260,
                9800,  35288, 24276, 61580, 37464, 14836, 52816, 11356, 26636,
                4752,  39300, 57944, 45920, 63212, 16128, 31476, 36152, 54748,
                18764, 25436, 57320, 49052, 16640, 1196,  24160, 62500, 17672,
                33504, 64372, 2684,  58960, 44876, 50356, 3492,  36660, 17512,
                5152,  63124, 13112, 928,   44776, 21336, 36828, 53260, 22956,
                46540, 10244, 41132, 58440, 34468, 16592, 7892,  37796, 27876,
                60752, 42308, 55180, 37220, 24180, 47536, 62164, 38344, 28824,
                63256, 39712, 7432,  52216, 27532, 37824, 4832,  44296, 20004,
                41460, 10820, 45044, 27812, 6012,  59928, 42620, 28924, 38360,
                33892, 58844, 29944, 50416, 20400, 33268, 60248, 28256, 55692,
                2100,  64844,
            },
            {
                15444, 55196, 41344, 2848,  52468, 9508,  39364, 19976, 45312,
                7320,  58732, 324,   54424, 28532, 8448,  20892, 57820, 43000,
                33668, 12180, 63764, 45720, 9076,  49340, 4040,  28240, 18692,
                36268, 1148,  25752, 46868, 30068, 58696, 6828,  63516, 48556,
                42608, 53788, 57000, 20252, 48192, 8764,  59024, 35632, 42468,
                63956, 23416, 13568, 36672, 6216,  40776, 49640, 23116, 6696,
                46640, 29948, 3220,  61100, 44460, 31520, 55072, 40716, 46356,
                14768, 8108,  38792, 11724, 27256, 33236, 14156, 40252, 60228,
                30880, 49276, 39048, 56452, 16320, 30160, 64380, 14196, 32356,
                60264, 19820, 52788, 6996,  25088, 56192, 41560, 19052, 1720,
                51628, 15864, 30612, 59664, 4476,  52528, 16400, 11868, 1516,
                33520, 18792, 58964, 10784, 14764, 55516, 23844, 12896, 29872,
                53768, 24928, 3104,  55624, 21392, 47112, 7948,  52872, 11312,
                45732, 62932, 7636,  13912, 57564, 44436, 38928, 6180,  24476,
                42220, 19488,
            },
            {
                38444, 23428, 35284, 58692, 17408, 49540, 33808, 12308, 63948,
                50660, 22160, 13668, 43732, 33432, 51400, 3384,  30084, 15368,
                50656, 23736, 41348, 29232, 57744, 16832, 55768, 23880, 48100,
                12868, 53176, 21484, 62104, 2656,  35116, 45356, 19604, 3760,
                16416, 29324, 13704, 1448,  32852, 25336, 5504,  18824, 49784,
                2372,  53640, 31188, 18580, 55012, 27264, 1888,  62200, 10944,
                52720, 37804, 9232,  20948, 13348, 26944, 11020, 5496,  22044,
                29372, 43740, 23840, 53348, 19852, 63648, 8324,  24816, 52112,
                20460, 9636,  23272, 34284, 7272,  48388, 3044,  39848, 26868,
                4996,  37852, 29432, 63444, 21980, 47488, 12136, 65004, 45756,
                22360, 35520, 8788,  26212, 44920, 32500, 7308,  57700, 50864,
                23988, 46316, 30484, 36072, 62000, 19172, 47096, 63804, 348,
                35504, 60780, 48756, 33060, 12640, 36088, 25408, 57060, 18380,
                21820, 3580,  42092, 35500, 23336, 744,   64128, 16392, 48740,
                10408, 31996,
            },
            {
                8132,  62976, 12952, 30412, 7108,  61072, 27440, 54676, 41596,
                36536, 31640, 18728, 61788, 40380, 12016, 60900, 45928, 37164,
                5264,  60336, 19456, 35776, 13496, 43268, 33336, 38840, 60672,
                6148,  32816, 39420, 16508, 50300, 23772, 54216, 40372, 33316,
                60996, 6240,  39528, 64732, 50936, 41788, 55688, 60436, 28796,
                16408, 44680, 61404, 10636, 47588, 59040, 20184, 44272, 33300,
                16964, 63164, 41940, 34728, 52084, 64236, 47420, 35960, 51436,
                55936, 60912, 35132, 6020,  41608, 55348, 46312, 56,    57196,
                42756, 3864,  46188, 58676, 25720, 41768, 11320, 54540, 57592,
                17232, 48656, 12588, 44000, 2560,  36756, 30920, 53104, 25952,
                5768,  57020, 48816, 13120, 63928, 41280, 22292, 36612, 43296,
                13560, 54376, 4152,  49512, 42596, 33404, 6580,  38628, 50724,
                18288, 14688, 42416, 4552,  62704, 39928, 15632, 984,   48404,
                31524, 55384, 26748, 52352, 46560, 9188,  30332, 52032, 34692,
                59272, 53616,
            },
            {
                44104, 4164,  47152, 20720, 37304, 43672, 5328,  23248, 1580,
                9316,  55904, 47188, 6176,  25756, 22392, 53436, 17720, 26696,
                10396, 54644, 1996,  25908, 6988,  62988, 376,   10704, 51828,
                29684, 44184, 56136, 5076,  41880, 10672, 14200, 26340, 57496,
                49880, 23396, 45748, 10520, 27720, 15556, 22380, 37684, 12304,
                34012, 7788,  39832, 22588, 34480, 5420,  14560, 38468, 57124,
                27712, 5068,  23876, 56280, 2272,  39020, 18944, 60144, 552,
                9808,  17964, 48736, 2012,  15552, 28724, 36116, 10996, 32596,
                15176, 65428, 28984, 17600, 52680, 60800, 20668, 45392, 35020,
                248,   61884, 33116, 16176, 55344, 4884,  58876, 9056,  14952,
                40040, 61740, 19496, 38192, 880,   17852, 29704, 60152, 5368,
                27208, 65088, 9432,  21864, 1700,  28712, 58308, 10144, 26596,
                57264, 7760,  30472, 19344, 51920, 28592, 59076, 43460, 65020,
                37516, 10344, 15508, 62140, 18932, 56512, 13108, 41212, 20488,
                1476,  26308,
            },
            {
                18204, 57568, 51652, 25652, 55760, 14924, 48540, 30640, 59072,
                15932, 26892, 38544, 2664,  57404, 35816, 7712,  41872, 63980,
                32812, 47880, 39340, 58896, 51268, 22100, 46808, 26268, 15688,
                19992, 64324, 9436,  27484, 59848, 31740, 65184, 608,   8576,
                36812, 31256, 18744, 58136, 34984, 4692,  46068, 560,   63504,
                48784, 26368, 3700,  51004, 29272, 43012, 51968, 24604, 2888,
                46976, 50200, 14692, 45156, 29608, 7440,  24944, 31988, 14116,
                39920, 26600, 65012, 31144, 58016, 50132, 21576, 62668, 26936,
                38012, 49984, 11876, 36532, 1388,  31972, 14828, 6248,  22840,
                51280, 25588, 8428,  39032, 50256, 27964, 17920, 34956, 44592,
                32060, 2956,  28888, 51448, 46124, 55440, 9692,  48528, 20168,
                34056, 39968, 17416, 59972, 25032, 52728, 16136, 40872, 46052,
                22616, 64700, 38076, 56424, 23284, 9932,  6268,  34100, 24708,
                4768,  49944, 40588, 3068,  32788, 37968, 27472, 4740,  45996,
                61196, 29772,
            },
            {
                11736, 33120, 40924, 200,   63748, 10904, 39980, 19508, 62796,
                45248, 51368, 11496, 64332, 48504, 14532, 28844, 1308,  49884,
                21056, 14752, 44756, 29912, 17156, 36780, 55140, 40800, 58492,
                3608,  36992, 22972, 46140, 18548, 52008, 38096, 44868, 21312,
                12236, 55380, 2996,  42732, 62508, 52196, 30516, 54160, 40580,
                19316, 58720, 55564, 17024, 65000, 9312,  61820, 31556, 12096,
                59584, 19632, 32700, 10432, 58332, 16276, 49456, 43640, 56796,
                34076, 21492, 45828, 11900, 38432, 7364,  17624, 43548, 53696,
                6336,  19552, 41396, 61716, 8420,  49544, 38564, 64804, 30452,
                41056, 59128, 46568, 21196, 63964, 41816, 23824, 62476, 48344,
                11680, 54504, 23076, 8056,  33976, 24980, 15068, 63024, 53112,
                1120,  45372, 55584, 37540, 47684, 12132, 62980, 31552, 2380,
                34352, 11844, 44148, 1420,  49556, 45824, 17420, 52696, 12508,
                19664, 29064, 60104, 22060, 7500,  48552, 63460, 24224, 53016,
                15040, 39376,
            },
            {
                65384, 23676, 9156,  16576, 28484, 34996, 4372,  52964, 33076,
                3476,  21908, 29428, 17800, 32660, 43940, 54860, 59032, 38292,
                4828,  24360, 7784,  64600, 12008, 4316,  30936, 8116,  33816,
                49808, 13796, 54360, 1484,  34676, 6916,  15628, 29140, 62420,
                46928, 51700, 16748, 25256, 8784,  21040, 14304, 6648,  24692,
                9964,  32284, 45572, 13248, 37208, 176,   21192, 54604, 40472,
                7816,  36948, 64792, 53536, 41152, 21608, 63340, 3128,  52632,
                7624,  41984, 3728,  53528, 24984, 61224, 2400,  47448, 23940,
                59236, 3364,  55328, 25104, 21532, 44040, 27416, 55876, 10148,
                18800, 13720, 2144,  31860, 11212, 5980,  51916, 544,   20236,
                60060, 41448, 16512, 64856, 57376, 3672,  37360, 41720, 31268,
                11608, 25960, 14504, 30716, 7680,  43660, 20408, 55920, 6472,
                48616, 53904, 27688, 14912, 35704, 63416, 30944, 41632, 61084,
                56080, 45316, 35216, 53492, 42904, 17004, 58448, 10584, 35812,
                6816,  49732,
            },
            {
                2536,  36484, 44380, 59824, 50812, 46156, 58116, 25152, 13008,
                43036, 36340, 59976, 40896, 8928,  24004, 19280, 12716, 31536,
                60988, 55976, 34652, 41404, 48524, 57584, 18668, 44952, 61412,
                24820, 42488, 62628, 30148, 48032, 58388, 24536, 55964, 39872,
                5240,  27580, 59964, 32912, 38288, 49212, 57672, 36104, 43196,
                60984, 2268,  22208, 27864, 41864, 48844, 34880, 15356, 28684,
                43872, 23104, 832,   26324, 4688,  37816, 30568, 11572, 27636,
                62216, 15388, 59608, 18984, 35588, 41372, 32416, 13900, 10052,
                34932, 29504, 45884, 32900, 12524, 58212, 16984, 4672,  52516,
                36324, 43212, 54736, 26960, 35600, 56704, 14348, 30024, 37480,
                26752, 4456,  35740, 43340, 12884, 27676, 47272, 6936,  18252,
                58664, 50664, 4640,  64072, 35300, 3164,  51080, 24496, 37148,
                17760, 60280, 21324, 8204,  58740, 25780, 2644,  21804, 37836,
                8388,  14520, 164,   24868, 12012, 30652, 1908,  44564, 31924,
                22020, 56344,
            },
            {
                46836, 19524, 54908, 31300, 6328,  21784, 17664, 38352, 8736,
                56016, 6080,  52648, 756,   49520, 62936, 4236,  46440, 26372,
                10104, 51752, 19860, 2712,  27524, 23816, 52836, 12876, 2484,
                28780, 10400, 17092, 39592, 21512, 11896, 43108, 2164,  19476,
                35360, 14536, 41224, 1012,  44688, 11516, 65460, 27016, 16000,
                48332, 38116, 52268, 7024,  58008, 24896, 60260, 5724,  50360,
                61444, 17112, 47428, 51344, 13836, 59900, 45352, 19572, 36308,
                23556, 47088, 29844, 50488, 9496,  56324, 27344, 62356, 51176,
                39952, 15460, 64248, 6640,  52184, 35684, 1940,  47636, 24436,
                63456, 7036,  49568, 61256, 18344, 47080, 43416, 8184,  57828,
                53180, 10436, 50068, 30796, 20852, 59932, 52132, 22752, 61904,
                34824, 42884, 23344, 57572, 18804, 27748, 40072, 61272, 13044,
                29860, 3416,  41496, 32224, 39012, 11592, 55020, 5552,  48052,
                27776, 32556, 57976, 64708, 39396, 50504, 54980, 19372, 62020,
                42188, 27376,
            },
            {
                60392, 7464,  13488, 25948, 42112, 2356,  61120, 49060, 28156,
                64720, 19024, 25624, 15452, 58120, 37628, 34044, 53012, 40744,
                17268, 43876, 29636, 61692, 9464,  39024, 63664, 35712, 20768,
                48364, 59672, 7288,  53152, 3724,  63776, 49656, 32864, 10024,
                50468, 54276, 7900,  61812, 23732, 18900, 31728, 2788,  55008,
                20484, 12576, 64004, 33512, 17948, 11196, 46096, 20064, 2636,
                55988, 35840, 9184,  29084, 34252, 55080, 8672,  48700, 58180,
                972,   38088, 12928, 6412,  22472, 44872, 16624, 4932,  55064,
                21832, 528,   48632, 18616, 41964, 29172, 62436, 40132, 33292,
                21084, 29556, 15428, 3616,  38296, 25100, 64484, 33616, 17288,
                46452, 24244, 62132, 6332,  45460, 112,   39888, 10136, 28708,
                2236,  48868, 8484,  39036, 53088, 15280, 46512, 9364,  57428,
                43488, 51632, 64376, 47128, 18828, 50508, 43720, 16144, 62744,
                51712, 18420, 5828,  47268, 20788, 28084, 8436,  37676, 12744,
                3684,  15604,
            },
            {
                30384, 51816, 38900, 64268, 34264, 11244, 54248, 31880, 14356,
                45068, 34944, 47308, 31192, 11040, 27308, 20932, 7436,  65224,
                288,   36256, 14148, 50092, 45668, 15740, 31768, 6296,  41780,
                56248, 37708, 33084, 45244, 27148, 36476, 15208, 59308, 25320,
                64712, 21556, 30352, 48072, 57052, 6272,  52060, 46032, 34456,
                4980,  29288, 56388, 44180, 1664,  31048, 53812, 37792, 25860,
                31696, 12864, 42676, 62620, 18556, 24056, 5632,  32792, 16608,
                52300, 43344, 63820, 33436, 58736, 1304,  48856, 30964, 36336,
                44224, 26332, 57036, 10780, 23668, 59536, 14468, 8724,  45492,
                11892, 58088, 40752, 22684, 9416,  51092, 2736,  21612, 40448,
                1544,  14104, 38860, 19128, 33708, 56064, 16756, 36420, 54800,
                14948, 20588, 30328, 11248, 62036, 31792, 284,   22300, 33804,
                26080, 15980, 5492,  24084, 656,   57792, 28452, 36364, 23068,
                40904, 11148, 43440, 35640, 14908, 4480,  60088, 24104, 48120,
                53604, 35076,
            },
            {
                10316, 23484, 1056,  49504, 15984, 45632, 23828, 6676,  41312,
                1784,  22308, 61840, 39936, 3368,  43280, 56276, 13532, 48420,
                23100, 59716, 6748,  55780, 21776, 1348,  58892, 51516, 25712,
                15060, 116,   23308, 18920, 56884, 7944,  30968, 41884, 5920,
                45456, 37808, 12088, 16524, 35612, 28636, 40244, 9788,  62764,
                42336, 23320, 8608,  39660, 50056, 61544, 14256, 41104, 64488,
                6672,  21648, 57844, 38532, 1564,  44204, 65484, 40152, 10964,
                26036, 5040,  20316, 54048, 25232, 38376, 65044, 19892, 12272,
                7812,  62748, 33960, 38696, 5600,  49468, 19688, 26776, 56556,
                452,   48368, 34476, 53420, 59228, 31244, 12412, 54880, 61136,
                29340, 58404, 52396, 26972, 63716, 12248, 25364, 50144, 65504,
                40660, 45308, 56312, 25548, 5824,  42512, 49652, 63680, 6616,
                53448, 37916, 11028, 34736, 61496, 13944, 31584, 8780,  3112,
                55636, 26604, 61436, 52940, 31092, 57272, 45488, 33196, 64520,
                17932, 40736,
            },
            {
                45808, 62248, 20200, 55372, 29264, 36676, 62736, 18392, 52020,
                58640, 12196, 7340,  55472, 51204, 16688, 61232, 28804, 38980,
                32996, 53680, 25876, 31160, 37340, 43064, 18220, 46308, 11500,
                65324, 30288, 49848, 61340, 13104, 47432, 22508, 52996, 17248,
                27984, 3668,  55788, 42132, 1804,  60892, 14144, 25468, 17300,
                58080, 51188, 14848, 26020, 19812, 5948,  28312, 9456,  45472,
                52452, 48396, 3932,  25268, 54128, 14512, 49968, 28660, 56116,
                60628, 31608, 49104, 41128, 14668, 9988,  28444, 42584, 59796,
                52704, 16876, 30216, 45556, 54884, 3228,  32208, 52612, 37952,
                17796, 62100, 5348,  27480, 45232, 16732, 42156, 26460, 7148,
                35280, 44188, 9568,  4148,  48216, 42280, 7740,  32056, 3316,
                6460,  60404, 33368, 48016, 16396, 58196, 36108, 12708, 20148,
                45940, 29520, 56468, 49224, 40564, 21140, 53676, 65328, 46904,
                33740, 20232, 1356,  9176,  22756, 41732, 11908, 576,   26484,
                6596,  58704,
            },
            {
                4140,  33044, 12504, 43396, 8432,  3432,  56660, 27600, 38640,
                33228, 49548, 29560, 19672, 35448, 23944, 46024, 2080,  9736,
                18864, 42216, 3196,  12212, 63040, 8616,  28716, 34200, 3472,
                53292, 40104, 9448,  42704, 4144,  38244, 63352, 760,   57520,
                34176, 48996, 62380, 25052, 45120, 21292, 54308, 38216, 47952,
                1080,  32508, 37292, 63144, 46456, 35040, 59168, 23236, 17616,
                32976, 15112, 60784, 10352, 30864, 35896, 19828, 3396,  22172,
                37156, 8720,  17712, 3696,  34992, 46444, 56804, 6220,  24512,
                39812, 1848,  20984, 13356, 61636, 36372, 43816, 65300, 6516,
                30604, 43072, 20456, 13660, 63596, 4188,  36540, 47532, 19708,
                50764, 22760, 16436, 37544, 30368, 57524, 18284, 46084, 23080,
                27516, 12632, 19044, 1904,  38380, 23704, 28508, 54724, 40756,
                2488,  63008, 17736, 7200,  26760, 45536, 4340,  38180, 12908,
                59864, 15920, 38924, 48952, 63468, 17524, 52004, 39736, 21348,
                50248, 28288,
            },
            {
                37456, 57556, 25544, 39780, 60424, 21860, 46884, 10688, 4576,
                24488, 15848, 44464, 62712, 10088, 5296,  32012, 58048, 50956,
                63572, 15464, 46684, 56928, 49116, 24320, 54468, 59812, 22228,
                36288, 16304, 26100, 55276, 34392, 17872, 29148, 10632, 39500,
                14012, 19840, 7752,  31836, 10984, 49428, 4312,  59132, 30396,
                21664, 6664,  55724, 11096, 3908,  54628, 42240, 1932,  57040,
                36912, 27852, 40316, 44956, 51216, 6600,  61916, 41828, 45416,
                13924, 54792, 62836, 29692, 59068, 51392, 2584,  15880, 31904,
                50532, 47456, 58380, 27980, 9204,  25024, 15944, 22432, 12620,
                50844, 25592, 9008,  39316, 33028, 49676, 57076, 10800, 65048,
                4808,  32644, 62012, 53336, 1860,  21644, 61160, 52036, 38520,
                54136, 35696, 43480, 50592, 64228, 4584,  18404, 8524,  59604,
                25080, 13380, 43664, 32376, 60548, 10456, 19312, 25332, 50452,
                29968, 44728, 56948, 27444, 4220,  36304, 29352, 56340, 9024,
                43612, 14736,
            },
            {
                18904, 47416, 5748,  17148, 50884, 34760, 14664, 52892, 65252,
                42288, 59440, 1180,  26912, 38800, 54608, 41728, 21540, 36728,
                26772, 7252,  35024, 20612, 39656, 4332,  14260, 41100, 7608,
                47384, 62464, 5236,  20924, 51664, 60664, 25144, 43472, 51332,
                59836, 23476, 53756, 37244, 63892, 18108, 34288, 8832,  13176,
                44840, 60372, 40940, 28840, 16228, 21456, 31432, 50552, 11832,
                63176, 5868,  20144, 55612, 16784, 26864, 11556, 58608, 33392,
                20,    47164, 24452, 43188, 11812, 22920, 19204, 37476, 63568,
                10300, 35112, 4976,  42136, 53360, 49156, 1400,  40624, 55404,
                35724, 46396, 60632, 55856, 1116,  18844, 24068, 30564, 15248,
                41552, 55812, 11544, 24836, 40404, 13852, 34404, 9672,  15820,
                62548, 7572,  57168, 29740, 10588, 52344, 44864, 31464, 48376,
                34496, 51504, 22704, 1152,  55208, 36296, 52556, 42176, 7556,
                2284,  23496, 10808, 33032, 43680, 7356,  60728, 13508, 34796,
                63640, 52400,
            },
            {
                1372,  54152, 31560, 63436, 28440, 424,   43964, 31100, 18776,
                8528,  35828, 53792, 13768, 46788, 64540, 16172, 12000, 3808,
                45104, 53136, 61456, 28424, 17252, 33356, 64264, 50544, 19144,
                28016, 57172, 32580, 45596, 2060,  12452, 48004, 6380,  31372,
                2492,  45992, 40684, 4928,  28944, 51848, 41740, 26432, 64672,
                35472, 18400, 24464, 53144, 48196, 65208, 38784, 8188,  43696,
                24424, 47028, 728,   34228, 64028, 39092, 48164, 23956, 18228,
                51744, 28056, 38404, 7316,  61028, 33096, 53928, 44280, 26240,
                55948, 14700, 23268, 64352, 18472, 31516, 60884, 10488, 59104,
                16892, 2652,  22812, 52224, 28732, 44144, 7652,  59532, 37436,
                1396,  44852, 28388, 47700, 58304, 5936,  43620, 29524, 504,
                47004, 21388, 25112, 14192, 40444, 33836, 61228, 15544, 5428,
                38700, 9712,  58124, 39984, 15436, 30232, 63920, 16904, 58252,
                35064, 62632, 51208, 14804, 54868, 20652, 24768, 48208, 3288,
                30200, 22660,
            },
            {
                59756, 35544, 8956,  42476, 11864, 59012, 24856, 37668, 57368,
                22888, 48972, 31720, 20544, 7140,  24564, 29500, 49140, 60432,
                31576, 23252, 10324, 824,   57872, 45744, 11716, 31120, 1408,
                43284, 10236, 14640, 38820, 30016, 58540, 21828, 36792, 65064,
                18012, 27672, 12160, 16072, 57908, 140,   22784, 56120, 46984,
                2776,  50716, 9260,  32980, 464,   12912, 26848, 19092, 54004,
                30224, 13860, 57724, 22796, 8732,  2988,  31380, 54444, 5392,
                10036, 63792, 20416, 16056, 1504,  39876, 48448, 8060,  4088,
                20124, 45976, 29440, 38856, 7244,  35480, 44896, 26396, 29660,
                5616,  32076, 37396, 14704, 10288, 62732, 41052, 53280, 25900,
                50136, 20204, 8228,  16888, 36344, 62900, 49476, 26152, 56480,
                39544, 32292, 3508,  59108, 20240, 1092,  26280, 56076, 21092,
                65120, 28808, 18896, 46428, 6796,  49312, 3708,  28008, 21552,
                47552, 40052, 19036, 2816,  64884, 46092, 38652, 17036, 58032,
                42144, 11268,
            },
            {
                38988, 15976, 23760, 49048, 20296, 53408, 6360,  47500, 2476,
                12488, 40476, 4060,  60816, 52316, 34844, 524,   55828, 19016,
                42900, 14288, 37576, 51116, 41692, 6536,  25920, 53860, 38320,
                61024, 24580, 52968, 64384, 7876,  40652, 15440, 53992, 9852,
                34808, 56356, 61372, 47780, 33172, 44276, 14308, 37608, 5908,
                28128, 15120, 43144, 61792, 36732, 44972, 59544, 4712,  35144,
                61300, 49264, 41756, 28528, 52144, 43420, 15600, 61504, 36736,
                41240, 57296, 34632, 49904, 55840, 27628, 13184, 30732, 61728,
                41044, 59600, 888,   48480, 12568, 56880, 3852,  19872, 47940,
                53964, 41932, 64736, 47400, 21080, 34940, 5096,  17388, 12064,
                34172, 56992, 63728, 31280, 3036,  23388, 19408, 53132, 12372,
                17668, 64956, 48860, 42692, 54416, 36760, 47364, 12056, 41944,
                2168,  53824, 35872, 61148, 23660, 11596, 34300, 45108, 13272,
                53844, 9124,  30172, 37088, 26588, 11540, 53320, 6072,  33148,
                26168, 45868,
            },
            {
                65128, 28000, 56476, 4124,  40232, 33628, 63756, 16404, 29724,
                62016, 55588, 27372, 16848, 43404, 38100, 58756, 9108,  39820,
                5688,  65164, 55392, 30460, 21352, 59540, 35260, 16520, 22016,
                49232, 5804,  35428, 17520, 50012, 27160, 44240, 3404,  48824,
                24412, 42296, 7912,  21212, 25856, 63348, 9348,  20140, 61524,
                54296, 39764, 57632, 20396, 7744,  23052, 56224, 15744, 39896,
                2608,  10100, 17316, 37736, 59732, 12696, 25508, 45188, 21516,
                29936, 13588, 3788,  44780, 24668, 65272, 17936, 54664, 36200,
                11692, 32680, 53148, 17260, 62556, 22672, 51152, 38596, 62136,
                13028, 18636, 7096,  26824, 50868, 57612, 31724, 48228, 61708,
                22932, 4400,  38932, 52012, 42344, 10948, 33484, 60468, 4020,
                44996, 35376, 9528,  28372, 16816, 6736,  63324, 30540, 24688,
                49956, 13724, 4512,  26664, 41820, 57360, 62432, 38836, 1540,
                25996, 61568, 5352,  58628, 41680, 31540, 1900,  49780, 62220,
                20036, 7268,
            },
            {
                51252, 2556,  44320, 61164, 13628, 27080, 9288,  39492, 51584,
                19696, 45140, 33712, 10204, 48624, 13124, 21692, 27032, 33348,
                47120, 25448, 3992,  18032, 48704, 9112,  63156, 2960,  56788,
                12404, 29528, 45300, 948,   22884, 55956, 32136, 19340, 62596,
                29712, 1524,  51412, 38984, 5168,  53228, 30544, 49912, 33720,
                11032, 23812, 30220, 4424,  51136, 31776, 47108, 28184, 52508,
                21816, 32860, 64964, 6124,  20260, 35468, 56436, 1220,  50932,
                7496,  47760, 60484, 31460, 10452, 5672,  42744, 22340, 2304,
                51492, 24032, 8668,  27328, 42876, 33772, 15104, 8200,  24140,
                34396, 57880, 45176, 3240,  38884, 13468, 172,   28164, 40016,
                8924,  46088, 26252, 13684, 55352, 46900, 7612,  41032, 29940,
                22568, 5432,  61128, 12948, 51112, 23096, 44396, 9096,  34136,
                59912, 43852, 17328, 32516, 52992, 20044, 8292,  31296, 56104,
                16516, 42772, 48944, 21120, 15496, 56648, 23112, 35992, 9792,
                55496, 14284,
            },
            {
                34148, 21472, 37044, 31036, 18516, 57960, 45400, 24008, 35688,
                4500,  7836,  23572, 63896, 2232,  31156, 62044, 52800, 15900,
                50548, 11392, 35744, 44412, 27496, 14712, 39360, 47044, 42548,
                33384, 19808, 58744, 37192, 61832, 10512, 4956,  58852, 38608,
                11936, 55368, 17560, 35884, 13252, 58576, 40364, 16588, 45144,
                1100,  48752, 17376, 64312, 14160, 41604, 11012, 63136, 7312,
                43952, 26044, 54740, 45904, 3816,  48836, 32308, 10596, 64636,
                26708, 16720, 22696, 39648, 52560, 37040, 59052, 46828, 28264,
                15992, 44560, 57748, 37408, 2920,  54312, 29260, 41048, 636,
                51760, 10764, 30932, 23208, 55128, 60380, 19500, 43056, 53708,
                15264, 59632, 18736, 900,   28692, 64516, 16248, 36932, 57924,
                52200, 26564, 55356, 38168, 32068, 59400, 3848,  53388, 18708,
                38252, 7176,  64052, 47764, 92,    15036, 51224, 22908, 47156,
                36044, 10380, 33492, 52052, 7884,  63248, 43948, 17492, 40548,
                29788, 47724,
            },
            {
                5720,  54992, 11172, 52208, 6864,  48348, 2148,  55100, 12940,
                58992, 42348, 50980, 56776, 18208, 40396, 6912,  44528, 2792,
                62948, 22780, 57200, 60840, 1428,  52356, 31468, 23852, 4600,
                63856, 8744,  53540, 26136, 13712, 41648, 34196, 46716, 15072,
                23212, 45452, 31216, 60536, 43752, 3572,  23500, 7204,  55928,
                26524, 59036, 38692, 34372, 53864, 25700, 1672,  37228, 18660,
                58376, 50560, 12300, 30664, 23848, 62444, 40412, 18908, 53884,
                42928, 34236, 56916, 572,   19588, 14756, 32536, 8944,  62236,
                39948, 5780,  64728, 19292, 47652, 11060, 58516, 63832, 44236,
                27816, 60776, 36884, 15956, 8424,  46264, 24772, 6520,  65392,
                30672, 35056, 49804, 37856, 58488, 20800, 25312, 1832,  49296,
                13824, 18084, 41860, 664,   19472, 48268, 40656, 14816, 57248,
                22556, 29580, 55848, 9996,  36636, 28904, 41156, 4860,  63960,
                19160, 59876, 24548, 436,   29144, 47024, 12664, 27040, 60608,
                1088,  24500,
            },
            {
                61780, 17568, 41220, 25076, 64476, 35160, 21316, 31948, 62868,
                28224, 15312, 36752, 29416, 46832, 25376, 54944, 36172, 19912,
                29604, 39108, 7772,  32676, 41388, 10816, 55512, 18332, 27800,
                50908, 15348, 40084, 3264,  47752, 51888, 20976, 27936, 53044,
                6132,  64544, 9760,  26888, 19140, 48148, 29192, 65312, 36748,
                13064, 43228, 3336,  9784,  45724, 20624, 60588, 49072, 14404,
                34788, 240,   39312, 16476, 57576, 7916,  14956, 29076, 38032,
                4756,  11976, 62604, 44192, 27896, 48384, 3064,  55992, 20364,
                49764, 34204, 13264, 25904, 31500, 6532,  21304, 16976, 49484,
                5184,  20020, 54172, 41764, 63192, 32360, 36032, 50732, 2156,
                21240, 10600, 44140, 7164,  32532, 51552, 45420, 10232, 63264,
                33832, 7732,  46760, 62544, 25164, 10968, 35532, 26872, 1472,
                46292, 12276, 40328, 25456, 61268, 45240, 58772, 12552, 27632,
                2928,  44740, 13980, 55896, 38804, 34408, 3612,  57824, 50148,
                37460, 43560,
            },
            {
                8504,  57176, 46392, 120,   29268, 15840, 43360, 9596,  40156,
                48876, 860,   20876, 11528, 5104,  60524, 14696, 9572,  58820,
                43076, 13168, 53656, 16952, 21708, 62288, 37032, 7008,  57796,
                34860, 44088, 22184, 31636, 65412, 17272, 8320,  61476, 540,
                35328, 40764, 49688, 2032,  56564, 34500, 10700, 51380, 20928,
                31700, 18352, 62800, 28588, 56984, 6464,  32404, 41028, 28848,
                23332, 9080,  61764, 42688, 27136, 52812, 44660, 1856,  60064,
                20772, 50088, 30496, 7956,  54896, 64328, 25660, 38668, 11856,
                30268, 52,    55436, 51304, 60644, 45292, 37764, 33128, 25412,
                39516, 13928, 1604,  47996, 28520, 11748, 17604, 57780, 38476,
                26952, 55084, 62264, 24060, 12624, 4296,  39700, 30588, 42700,
                21108, 28736, 36128, 57140, 5904,  30264, 64780, 49432, 60288,
                32252, 5316,  50628, 21132, 17840, 7064,  33796, 54452, 38132,
                49624, 30572, 40788, 65288, 18384, 22420, 53688, 7072,  20504,
                15340, 32380,
            },
            {
                22872, 28380, 14060, 39120, 50116, 60200, 53212, 5164,  18244,
                25832, 53672, 65408, 38968, 33992, 51428, 22236, 32408, 48796,
                212,   26552, 46912, 5172,  50184, 25744, 45612, 13400, 48352,
                484,   60656, 11228, 55420, 5576,  29092, 36876, 43356, 56448,
                25452, 16080, 32704, 22076, 62332, 14440, 42108, 2756,  60164,
                46748, 5972,  50156, 22544, 37688, 16204, 51248, 2948,  64176,
                54552, 46796, 20660, 36160, 3484,  33736, 22404, 48076, 56384,
                35616, 15692, 24176, 40768, 11256, 34580, 17020, 45468, 59916,
                23292, 43648, 17580, 40400, 22848, 1776,  55132, 12380, 8352,
                57276, 64864, 34316, 21984, 3736,  52764, 7184,  43624, 14088,
                47048, 5424,  40632, 16084, 54112, 61296, 18928, 56176, 5604,
                59560, 53120, 12172, 16288, 44304, 54600, 20872, 8280,  42228,
                16344, 62916, 34560, 54760, 3272,  49220, 23524, 15832, 9104,
                21452, 52540, 5788,  26520, 8884,  49120, 42148, 31152, 11240,
                64632, 52736,
            },
            {
                4264,  35608, 63196, 20444, 7456,  12188, 37604, 23072, 61576,
                44428, 31580, 8696,  57716, 45208, 3032,  27904, 41012, 64396,
                17724, 56784, 34104, 63412, 29988, 2984,  58856, 20792, 39816,
                30156, 17860, 25296, 38552, 45496, 59380, 23420, 12720, 48508,
                19520, 59080, 52492, 8008,  46036, 38436, 24280, 55244, 9044,
                39604, 25520, 54212, 12348, 42672, 59484, 24988, 11632, 43820,
                4772,  31532, 59792, 11388, 49408, 64292, 13788, 9532,  26256,
                6384,  45944, 61468, 3592,  51288, 22248, 4460,  53568, 7380,
                35936, 63148, 4548,  9820,  35580, 15420, 62384, 51868, 46168,
                29932, 42612, 50528, 26092, 59240, 39320, 61528, 23616, 33424,
                63564, 18360, 29248, 34084, 47604, 36392, 27464, 14656, 46004,
                24776, 2116,  49644, 22732, 39112, 3148,  14188, 37776, 24324,
                52776, 28204, 43944, 13800, 38728, 64888, 30976, 42872, 62364,
                56656, 35140, 12200, 45960, 61032, 14524, 36276, 59112, 26800,
                38500, 47644,
            },
            {
                58468, 10552, 42432, 54448, 33600, 26160, 47708, 56944, 34620,
                4004,  14248, 41556, 24492, 16644, 62028, 12044, 54344, 7388,
                37204, 23680, 9256,  38828, 14544, 43252, 33108, 10116, 64320,
                53712, 7536,  50392, 34368, 1984,  15620, 51200, 32116, 6584,
                38824, 10912, 43112, 4532,  29612, 53372, 17564, 27968, 33732,
                15472, 61620, 35516, 1084,  30124, 8176,  53520, 20060, 35444,
                16840, 26388, 51896, 6880,  19076, 55412, 41136, 31632, 63092,
                42532, 53580, 32748, 19108, 37640, 60364, 42072, 28576, 14800,
                50760, 26504, 47392, 57152, 29420, 42940, 26852, 4664,  23368,
                17980, 6048,  10080, 15332, 45060, 19348, 29804, 49384, 9040,
                2596,  56844, 51640, 3556,  9476,  22676, 228,   64616, 37432,
                10728, 40648, 63104, 31596, 60128, 51656, 33940, 58568, 4048,
                19492, 9616,  872,   59332, 26820, 10548, 46516, 5136,  25760,
                2120,  17984, 58348, 32488, 1264,  54732, 23716, 5564,  44924,
                1708,  18688,
            },
            {
                29840, 50740, 24380, 3328,  59348, 30692, 2312,  16792, 10660,
                50772, 55660, 20284, 6540,  49268, 37584, 19668, 46276, 31176,
                3924,  51592, 44500, 59384, 19228, 52888, 6428,  36316, 26832,
                14672, 41668, 62940, 20384, 57056, 9064,  40352, 64180, 2944,
                30000, 61720, 26352, 35104, 12612, 63796, 264,   48592, 58536,
                45228, 4524,  19376, 48720, 64996, 33308, 46960, 38676, 62376,
                57216, 13372, 40060, 45036, 29376, 24460, 2520,  37868, 18592,
                12496, 1128,  28348, 57760, 13572, 47908, 31604, 9928,  58040,
                39240, 18872, 32204, 13696, 65156, 49816, 19832, 39044, 59952,
                36424, 61776, 31752, 53500, 35804, 5452,  12216, 54516, 37452,
                21704, 42768, 25360, 60008, 44452, 55680, 50260, 30952, 17844,
                48496, 34260, 7332,  19020, 26576, 9832,  43364, 28908, 46472,
                55776, 39868, 48108, 22444, 36264, 56960, 18964, 51484, 40424,
                29252, 47968, 37904, 20640, 43644, 28932, 17312, 63372, 50228,
                13556, 40504,
            },
            {
                61836, 6320,  16024, 46624, 19080, 43692, 64820, 39944, 29676,
                46332, 26768, 62724, 33548, 29928, 1244,  57024, 25128, 14840,
                61424, 20896, 11712, 28420, 1020,  24672, 47016, 56548, 22212,
                45080, 4272,  32320, 12100, 47816, 27668, 18676, 53240, 45268,
                21976, 55600, 15652, 50924, 59576, 20264, 37080, 7444,  12028,
                22272, 29184, 40256, 56216, 13992, 17620, 3276,  27840, 5740,
                48176, 22892, 904,   36588, 58052, 15228, 46552, 52424, 58456,
                23264, 49376, 39772, 8636,  25820, 2112,  63808, 20888, 46200,
                1452,  6400,  52420, 22268, 7800,  3004,  33552, 11408, 53968,
                1424,  47288, 20848, 57104, 2336,  64224, 41856, 26436, 58904,
                14932, 31480, 11612, 38420, 19764, 13564, 40992, 8036,  53592,
                58260, 28108, 14540, 54320, 47196, 420,   17028, 62108, 11616,
                33304, 64056, 30784, 17000, 52592, 2472,  32840, 12604, 61008,
                15128, 8692,  64012, 4564,  52028, 39484, 10120, 34152, 56232,
                21696, 32756,
            },
            {
                43848, 27164, 56536, 36492, 13080, 8560,  52744, 21912, 60540,
                556,   38452, 11872, 52960, 59736, 44020, 9100,  34336, 53316,
                41328, 49068, 35880, 55212, 63932, 41184, 16116, 59984, 1812,
                49236, 28436, 55036, 23748, 43124, 60592, 35092, 25224, 13932,
                36952, 47548, 1464,  40308, 23604, 44796, 31292, 56544, 42020,
                63476, 52780, 9860,  26580, 43920, 23492, 60400, 41480, 10256,
                32460, 53124, 64620, 9700,  33160, 61036, 4676,  27940, 8260,
                35096, 15412, 65360, 55216, 44668, 17696, 34220, 54280, 24932,
                62024, 43520, 34804, 60140, 40068, 46464, 56376, 28940, 16088,
                41452, 27396, 13100, 40124, 23968, 48296, 16844, 32936, 660,
                46424, 50344, 64944, 6760,  28788, 61592, 33644, 24416, 4036,
                21572, 44800, 5396,  41744, 65248, 36464, 23120, 49988, 4920,
                25672, 14632, 5892,  42452, 8512,  62552, 45040, 21928, 35964,
                53852, 42752, 23044, 26924, 13180, 61380, 48264, 3296,  25472,
                8180,  53664,
            },
            {
                1036,  11336, 39584, 63120, 49248, 34036, 24716, 5508,  14980,
                35340, 18544, 42844, 4408,  21800, 14000, 39280, 64988, 2328,
                27068, 7728,  17012, 4440,  32184, 49900, 12052, 30256, 38480,
                9840,  18080, 61548, 7060,  37240, 156,   5644,  57996, 10228,
                63020, 7720,  18860, 32544, 9888,  3516,  49948, 25804, 18040,
                33056, 1972,  36976, 58916, 6448,  35032, 55284, 50600, 20556,
                15668, 43020, 27116, 18176, 49076, 21924, 11792, 43784, 62396,
                47076, 5292,  30280, 21340, 36564, 50536, 6972,  12704, 37424,
                16500, 29808, 54752, 12140, 25932, 17404, 59288, 43868, 8948,
                51180, 63296, 7052,  34532, 29460, 52292, 7672,  20496, 62092,
                39616, 4960,  17824, 35604, 48392, 2360,  56980, 43444, 12292,
                62816, 35356, 59620, 24664, 12464, 32188, 57144, 40948, 21036,
                37832, 54020, 59924, 50212, 24636, 28664, 39468, 6780,  59364,
                1068,  30840, 49776, 57488, 7352,  31224, 19996, 42504, 60232,
                37400, 16828,
            },
            {
                48412, 23316, 31168, 20632, 2036,  28584, 40972, 58396, 49612,
                56420, 8500,  64104, 25740, 51488, 28600, 47768, 18052, 22852,
                58424, 30740, 46756, 39416, 21752, 8148,  35408, 19760, 65176,
                51836, 33888, 40076, 15012, 52368, 30164, 17460, 44388, 33352,
                50484, 29036, 42544, 57812, 53084, 64768, 14564, 38780, 4764,
                46896, 13620, 50960, 21072, 47904, 368,   12756, 29768, 61964,
                37596, 2940,  55884, 6708,  40644, 54456, 37084, 31344, 19948,
                25620, 41364, 52260, 11508, 3976,  61552, 27596, 41428, 49272,
                3796,  10128, 20312, 48440, 392,   36120, 22652, 3732,  32664,
                24568, 18432, 59000, 46128, 10916, 60188, 43484, 55728, 28296,
                9824,  23784, 57936, 54056, 22204, 15792, 26692, 38064, 51772,
                17392, 1384,  29908, 49696, 19980, 3652,  53256, 8224,  60856,
                44784, 1460,  34912, 20432, 10876, 55476, 16260, 48512, 25664,
                19236, 11232, 35016, 16532, 44984, 55264, 36180, 15064, 28172,
                45712, 64508,
            },
            {
                34088, 52264, 58932, 5844,  54524, 61220, 10084, 44136, 27364,
                21008, 30792, 46704, 16168, 35800, 6956,  59176, 11168, 50404,
                43512, 13340, 62372, 56644, 25844, 60684, 54156, 5220,  43216,
                23060, 3744,  57400, 46176, 21500, 64668, 55500, 40880, 20912,
                2324,  59960, 24792, 35700, 16800, 27608, 7872,  60276, 54624,
                23392, 61924, 31116, 16676, 63848, 39744, 25316, 46064, 8304,
                57800, 24628, 45296, 29680, 62840, 1916,  16620, 51508, 56244,
                24,    59968, 17296, 32800, 57212, 43744, 22436, 59064, 31440,
                56072, 63700, 38844, 28392, 61848, 51804, 14016, 64472, 54596,
                37944, 48960, 780,   22512, 16296, 3204,  36936, 12784, 34320,
                51872, 44108, 30416, 41424, 10604, 45636, 64280, 9164,  31400,
                47252, 40144, 56028, 10012, 38760, 45672, 15712, 30188, 12852,
                28048, 18192, 47172, 30392, 41068, 65372, 4508,  32048, 57064,
                45892, 62292, 39392, 3780,  65040, 24292, 256,   52436, 5692,
                12144, 19564,
            },
            {
                4336,  9556,  42864, 15296, 45628, 17732, 37100, 13492, 2876,
                53596, 38948, 60772, 1764,  56092, 40900, 32160, 3248,  37928,
                33996, 20000, 1596,  10776, 42264, 15392, 45680, 26860, 58128,
                14184, 29760, 11116, 26416, 50060, 8376,  12480, 48532, 27716,
                15144, 54052, 6304,  12220, 44004, 47728, 30608, 20156, 34792,
                43700, 8600,  26932, 42720, 6632,  32300, 53956, 18656, 4168,
                33244, 49492, 14416, 10412, 34936, 23588, 42964, 7092,  13896,
                38144, 9968,  48644, 24016, 38912, 14716, 9244,  988,   18588,
                45364, 23480, 5324,  42780, 8644,  31220, 40312, 45780, 11520,
                5800,  30204, 42860, 55096, 39332, 65284, 25736, 48064, 17096,
                3768,  59940, 13908, 1592,  32904, 49972, 5216,  20428, 57808,
                23224, 14884, 6768,  27320, 63732, 58012, 24188, 34636, 48628,
                64428, 7400,  56444, 13884, 2824,  35824, 52296, 12860, 37664,
                8152,  53232, 27912, 21172, 41528, 10628, 47332, 59428, 31936,
                56792, 40944,
            },
            {
                61308, 29700, 37984, 26192, 51272, 32624, 23424, 63548, 47888,
                6456,  33568, 11384, 23684, 44640, 19736, 63152, 26512, 55112,
                6292,  52448, 29056, 37112, 51444, 6636,  32600, 652,   47756,
                36608, 62824, 42560, 2280,  36008, 31932, 24592, 4796,  37324,
                63692, 46012, 39404, 21936, 61664, 644,   40892, 52308, 11752,
                58244, 2544,  37652, 55960, 10932, 60176, 15024, 36616, 65528,
                21464, 39636, 59452, 19264, 47200, 56968, 27284, 64836, 33748,
                45272, 28448, 63240, 3076,  53624, 29756, 64688, 34564, 52700,
                13224, 36512, 50752, 15324, 57720, 18712, 4800,  27252, 21088,
                56972, 35488, 13548, 27540, 50420, 31896, 5464,  21584, 62672,
                37912, 26996, 19624, 63392, 39196, 54932, 29080, 36428, 3428,
                61680, 42828, 50464, 36656, 17884, 952,   41700, 51768, 4176,
                39228, 26060, 43088, 58888, 49368, 22356, 27020, 18440, 43348,
                23640, 5208,  14372, 51012, 33648, 29384, 17828, 38736, 21624,
                49552, 25116,
            },
            {
                54244, 18448, 11940, 65028, 7688,  72,    55792, 29996, 19056,
                41836, 15316, 50052, 28988, 8712,  52072, 13520, 46364, 16484,
                59860, 24552, 64800, 48244, 18356, 22680, 63260, 39180, 17968,
                8248,  51044, 20016, 60272, 16660, 39800, 62276, 56568, 18160,
                10472, 30448, 2936,  49744, 33144, 57532, 24560, 5300,  17076,
                28748, 46548, 19096, 51984, 22276, 48500, 28100, 44252, 51376,
                12148, 732,   30980, 52376, 3860,  39204, 11472, 49420, 22112,
                58100, 19088, 35664, 7724,  45584, 19784, 48316, 39844, 26492,
                7896,  60308, 32944, 25512, 46972, 53484, 34816, 60936, 50276,
                16932, 63088, 7984,  59688, 18960, 9848,  53808, 41952, 56620,
                8772,  46492, 50736, 6404,  24832, 17340, 59872, 13424, 52484,
                32228, 25704, 11600, 54844, 32868, 6508,  59564, 20288, 11288,
                54528, 22940, 16452, 32492, 9420,  45328, 60356, 688,   63768,
                33972, 58092, 48328, 60720, 2264,  53888, 62460, 13304, 8468,
                35820, 1780,
            },
            {
                43880, 47100, 56640, 35244, 21028, 48788, 39624, 9012,  58684,
                26016, 54400, 65520, 36740, 59464, 4988,  22724, 30872, 9744,
                39992, 44164, 14428, 4196,  35048, 53368, 28248, 10524, 56284,
                24720, 32948, 55044, 5448,  45516, 52808, 848,   43780, 33224,
                51812, 23800, 56164, 19136, 8880,  13880, 36852, 62528, 48544,
                39520, 64896, 13764, 34600, 1924,  40696, 4952,  9116,  24156,
                54756, 42796, 63304, 25912, 8676,  61088, 15936, 30596, 6256,
                2540,  52568, 42036, 13052, 25212, 56004, 5000,  16252, 44368,
                57112, 3944,  20692, 65532, 1236,  10748, 22964, 43632, 2992,
                38592, 25496, 45872, 2180,  34052, 44684, 15140, 28776, 80,
                32364, 22688, 57504, 35492, 11332, 45116, 41572, 8348,  46572,
                18808, 1792,  63128, 21684, 47660, 28316, 14448, 43972, 31824,
                61188, 1132,  37264, 62752, 5560,  39164, 29616, 54912, 10284,
                40952, 30616, 17164, 9452,  25272, 43132, 6128,  45448, 27744,
                64148, 15804,
            },
            {
                6244,  23692, 2616,  28332, 42024, 60636, 12796, 34268, 44484,
                4112,  21536, 808,   40264, 17364, 34452, 42556, 57356, 49664,
                192,   21168, 32172, 9404,  58616, 41208, 13404, 60196, 49604,
                2892,  40468, 28096, 11584, 23032, 29072, 14360, 21712, 61140,
                7020,  41736, 35980, 65524, 27304, 54592, 43308, 21520, 9320,
                32264, 4136,  24312, 57044, 30552, 62992, 18100, 58208, 34028,
                28880, 7048,  16456, 37660, 33400, 20664, 44228, 55088, 40336,
                26040, 16316, 61480, 32288, 59328, 37284, 10760, 61288, 22252,
                30924, 49196, 12116, 40860, 37216, 29004, 59120, 14648, 32464,
                9724,  52864, 20604, 41072, 56252, 24420, 61356, 36144, 52128,
                12336, 40196, 15696, 28456, 62124, 1176,  22088, 26568, 38152,
                57392, 34696, 44572, 8232,  38944, 53636, 62728, 36352, 7796,
                49660, 28468, 12652, 52552, 20940, 17128, 50180, 13772, 47304,
                20704, 2872,  45196, 38372, 19888, 35108, 57708, 23124, 51668,
                40608, 31356,
            },
            {
                50800, 38896, 59096, 14784, 53296, 5088,  24632, 16628, 62576,
                48472, 32176, 13096, 46040, 24968, 54648, 2856,  64036, 27052,
                35652, 61700, 54616, 46844, 25368, 1948,  44068, 30580, 21724,
                46148, 14472, 64976, 34964, 48880, 59340, 9524,  38768, 48116,
                26528, 12740, 3984,  16012, 46412, 30856, 1756,  50268, 26448,
                53192, 60024, 44384, 8384,  50192, 25888, 45968, 38400, 14300,
                60556, 48436, 22900, 45768, 58532, 50492, 1112,  36236, 63616,
                46280, 10960, 49844, 20648, 676,   28168, 50436, 33408, 2076,
                42080, 27884, 53076, 16616, 45624, 55432, 6948,  40192, 64168,
                47896, 30676, 61888, 4896,  49752, 11672, 6876,  47144, 17868,
                64064, 44072, 3268,  53384, 48380, 32008, 56068, 65516, 49524,
                5356,  16264, 59808, 30480, 12628, 4376,  22476, 25792, 46460,
                18856, 40772, 56180, 44424, 26208, 35072, 58344, 6812,  25344,
                37064, 53428, 28232, 64464, 55944, 14044, 31748, 1184,  59676,
                11396, 20340,
            },
            {
                34440, 63000, 9688,  32396, 18892, 45332, 51088, 28916, 37880,
                7480,  58056, 28120, 62112, 9992,  48128, 20216, 37876, 12688,
                17660, 6592,  39576, 29396, 19244, 64108, 7692,  16900, 36448,
                5696,  58600, 19100, 7284,  54008, 41892, 31664, 3324,  16920,
                57156, 44588, 60692, 52652, 39064, 5820,  59368, 16496, 35732,
                12468, 19816, 41112, 16200, 36776, 11652, 54132, 1320,  20808,
                40844, 3332,  55428, 12344, 5036,  28384, 14164, 23508, 8568,
                57048, 29304, 38388, 5996,  47072, 43096, 14212, 54260, 63600,
                18364, 9384,  35440, 62504, 3548,  20404, 50040, 24744, 18152,
                400,   12544, 36764, 16328, 28308, 39052, 21052, 57728, 25488,
                5204,  30680, 59648, 9028,  18700, 36544, 4560,  14720, 29880,
                10564, 24352, 51472, 19640, 40812, 50080, 56588, 16108, 2804,
                59244, 9248,  33488, 4652,  64184, 1740,  41672, 31876, 63004,
                15824, 59092, 12168, 7412,  49852, 3956,  39668, 18540, 47276,
                7520,  55528,
            },
            {
                13456, 3496,  48360, 26132, 36680, 64116, 1168,  55164, 10500,
                19800, 42636, 51940, 5840,  33232, 15216, 30036, 8212,  53220,
                44968, 23620, 57836, 11468, 51384, 37800, 55376, 33040, 62116,
                51600, 42660, 25084, 37700, 1496,  18136, 63064, 50384, 36836,
                29404, 20868, 33524, 24648, 11456, 19972, 41440, 64060, 6752,
                55868, 28672, 356,   61996, 21780, 6212,  32456, 64068, 50640,
                10372, 35748, 31416, 18796, 64404, 39836, 54088, 32888, 19436,
                4192,  34664, 22668, 54988, 64468, 17608, 23756, 7596,  36708,
                46672, 59088, 5984,  24996, 31548, 12844, 34376, 56932, 44860,
                27456, 54564, 22928, 58788, 32828, 64872, 1468,  41760, 34552,
                50304, 20388, 38112, 24456, 42192, 12040, 52644, 20992, 40028,
                43864, 55288, 36136, 63912, 752,   27236, 33820, 42848, 65192,
                30500, 50896, 21856, 15512, 48272, 11524, 53924, 22456, 43792,
                5012,  47960, 32592, 21768, 42420, 26700, 53040, 63368, 36584,
                25204, 28876,
            },
            {
                41456, 21964, 43724, 58152, 7028,  12036, 22220, 41628, 34884,
                49756, 14760, 23576, 38744, 56716, 61124, 41536, 49976, 59416,
                33000, 3312,  48408, 15448, 4304,  22504, 45552, 26244, 364,
                11084, 29296, 56140, 47232, 13688, 27352, 23404, 55232, 5048,
                64276, 9748,  4,     48948, 58004, 28352, 32876, 47184, 22640,
                38224, 45580, 33932, 52500, 47328, 57468, 29008, 16956, 44456,
                26840, 61588, 51808, 42856, 24920, 9944,  47000, 60824, 41776,
                51264, 62440, 14820, 41196, 9276,  31428, 58460, 40344, 25972,
                13344, 21280, 54916, 43708, 51576, 61460, 8824,  37872, 5412,
                60328, 40488, 51364, 7632,  43572, 14596, 54000, 29644, 8564,
                61872, 13412, 47648, 56964, 27692, 60748, 45320, 32808, 62468,
                2128,  28136, 6800,  14368, 46588, 59048, 10836, 5476,  38080,
                13024, 24908, 39720, 60440, 27908, 37200, 17924, 8972,  51284,
                27316, 372,   35996, 61560, 16852, 46104, 29980, 4944,  15524,
                44440, 60924,
            },
            {
                18068, 53556, 29852, 15336, 39304, 56368, 31368, 60956, 3960,
                26652, 63276, 1920,  18752, 44804, 4660,  25568, 1288,  21228,
                28292, 64580, 42032, 36124, 31056, 59852, 9200,  48744, 40208,
                15800, 22180, 32652, 9296,  61172, 44288, 34232, 11352, 46284,
                15520, 51980, 42644, 37728, 7940,  54512, 13756, 2308,  51312,
                10608, 58144, 5020,  14028, 24876, 42164, 37744, 4644,  23548,
                56752, 7236,  15172, 1440,  57348, 36524, 5704,  16804, 26284,
                11708, 45084, 2852,  25556, 50668, 35004, 1772,  52928, 4900,
                49788, 32896, 39256, 2564,  17912, 28516, 48040, 22024, 15724,
                33072, 10076, 19660, 3160,  25240, 48884, 10516, 46136, 16936,
                55336, 2292,  33564, 6804,  16356, 736,   49960, 7856,  17740,
                57480, 47868, 22412, 42588, 31812, 17256, 53224, 21244, 48060,
                54588, 96,    56808, 7396,  45408, 30940, 62312, 57324, 39028,
                19712, 56200, 40848, 24724, 8984,  55468, 11924, 22336, 57904,
                50120, 352,
            },
            {
                36016, 65332, 4568,  50288, 20008, 47040, 25396, 17088, 45984,
                57616, 32000, 53832, 11804, 29104, 35476, 54840, 16540, 39236,
                13408, 9308,  53748, 26024, 62696, 42932, 13708, 20080, 58228,
                52688, 64008, 4836,  50168, 39432, 56936, 6504,  20300, 41000,
                25788, 30628, 59524, 22480, 18104, 61316, 43736, 24992, 62308,
                17780, 26724, 31492, 64828, 18836, 9212,  61144, 13312, 47716,
                39480, 20072, 34812, 29052, 49320, 20736, 31744, 52760, 252,
                30428, 58564, 19048, 28960, 60804, 12224, 19952, 45276, 61092,
                29716, 10200, 64840, 14728, 36404, 58072, 1260,  43140, 64576,
                49608, 29512, 45812, 56832, 38104, 60444, 21772, 36376, 27096,
                39996, 23820, 44712, 64816, 30404, 54044, 23128, 38280, 26172,
                13068, 33952, 53700, 9152,  38672, 24484, 62928, 35596, 27796,
                17748, 43372, 34988, 19928, 51620, 2352,  24208, 13236, 33420,
                6716,  65260, 14604, 50580, 2256,  60320, 33560, 38812, 8000,
                31408, 10856,
            },
            {
                40228, 23888, 8480,  59840, 34184, 2772,  52320, 6712,  13036,
                39784, 8592,  36688, 47744, 65080, 22224, 10264, 46872, 61756,
                50496, 34508, 19620, 924,   17500, 5532,  54984, 34732, 30292,
                3284,  44048, 36076, 19848, 16300, 2072,  28568, 52768, 35972,
                62084, 2448,  13156, 33844, 46580, 3576,  36212, 30756, 40556,
                7484,  49216, 43568, 36460, 2364,  49572, 33464, 53324, 1796,
                30272, 65204, 9472,  44788, 59408, 12892, 63840, 43232, 56764,
                39172, 48848, 36720, 7016,  46780, 56172, 38164, 27724, 16472,
                42172, 57008, 23456, 47380, 26364, 40816, 11980, 53544, 24620,
                4580,  14004, 62580, 34608, 17572, 31032, 4068,  63636, 6192,
                51992, 59480, 11376, 21128, 41188, 35552, 10016, 61348, 42016,
                5256,  64604, 19296, 60056, 49888, 2464,  6528,  44488, 8656,
                58760, 32140, 10920, 64564, 14824, 40988, 5888,  44724, 49436,
                28816, 10900, 46508, 31484, 37552, 20188, 43368, 64216, 47140,
                26560, 56684,
            },
            {
                13152, 32640, 45164, 27012, 11072, 42076, 37252, 64640, 29208,
                19880, 43980, 24860, 16004, 51060, 7156,  58256, 43100, 30252,
                6284,  24136, 57340, 47556, 38536, 51624, 27652, 7532,  38920,
                24644, 12872, 59644, 25944, 31080, 65052, 47524, 58512, 8104,
                17836, 49508, 55756, 5516,  27244, 52284, 10008, 20228, 53588,
                14600, 59568, 21544, 11564, 56412, 28076, 20944, 58232, 43016,
                16596, 50172, 54500, 23172, 4044,  38396, 8084,  24144, 14376,
                21416, 9660,  65364, 16348, 42472, 3636,  23036, 63784, 8704,
                360,   19404, 52580, 4488,  8080,  62176, 31204, 18464, 35184,
                58728, 42364, 26764, 1048,  12432, 55240, 51164, 42276, 14096,
                31892, 18224, 37116, 49364, 3900,  56576, 14460, 47176, 29800,
                51096, 37076, 3680,  29216, 13816, 57072, 30704, 51788, 15136,
                61660, 4172,  23372, 46524, 54124, 26816, 58552, 35868, 60488,
                22612, 42456, 18584, 54544, 4696,  28268, 51932, 16132, 3084,
                19300, 52332,
            },
            {
                3868,  48676, 17476, 62496, 54108, 14620, 58828, 23356, 48152,
                54828, 5416,  61852, 856,   40136, 33596, 26796, 18788, 37420,
                2568,  60204, 44544, 12556, 32312, 23056, 59212, 45016, 62336,
                18280, 47956, 8392,  53908, 43160, 11664, 23084, 14328, 31964,
                44880, 24444, 38448, 62848, 41264, 16188, 65132, 57204, 700,
                32776, 38600, 4200,  52824, 45980, 39328, 7088,  25300, 11104,
                36528, 5988,  26548, 41124, 17880, 47616, 28396, 35208, 50916,
                5084,  33536, 53960, 24448, 31848, 51000, 14424, 34120, 48336,
                37344, 31076, 45032, 34680, 56008, 21232, 50700, 3236,  39476,
                9148,  20520, 52740, 47848, 39748, 8404,  24736, 20040, 47680,
                300,   56260, 28612, 7296,  61948, 25784, 19352, 2416,  58868,
                21876, 11464, 45600, 25324, 40676, 47032, 20512, 37272, 25836,
                40464, 48940, 29160, 38416, 8440,  32480, 20528, 3536,  16364,
                52288, 1528,  61652, 25248, 58248, 13048, 7084,  24300, 35136,
                42060, 61932,
            },
        },
        {
            {
                49548, 57744, 10588, 45488, 2744,  30216, 26732, 21592, 62884,
                31564, 41508, 4544,  22396, 15468, 42008, 47784, 20904, 7068,
                44000, 39064, 49400, 4668,  35704, 22812, 212,   9936,  46008,
                25192, 58576, 53068, 20704, 62040, 1292,  27632, 39724, 4448,
                50832, 13436, 57760, 16496, 49132, 53092, 18060, 59768, 36284,
                51688, 41320, 58764, 26364, 100,   29172, 13964, 41412, 2568,
                30964, 9468,  34808, 26964, 6240,  42460, 56144, 18688, 64208,
                11248, 44960, 16380, 28200, 4864,  54688, 1420,  61136, 41940,
                11828, 27864, 58012, 20864, 51516, 33084, 45720, 59396, 14780,
                42456, 928,   21252, 49316, 35144, 19280, 24700, 16336, 2856,
                30072, 14116, 53020, 48064, 19404, 12200, 50392, 21184, 60984,
                30616, 10760, 39372, 17528, 58512, 24360, 3032,  20720, 13200,
                49580, 16844, 54496, 9800,  33148, 44156, 63288, 7280,  16112,
                45156, 40924, 5052,  59528, 37188, 2228,  58016, 22444, 3888,
                36364, 31588,
            },
            {
                1088,  27968, 41124, 53220, 18996, 39032, 56312, 35824, 1380,
                51856, 13064, 46888, 62060, 29264, 58360, 3752,  27608, 35588,
                64044, 19000, 8948,  58084, 26116, 42660, 50472, 30220, 18476,
                13632, 40408, 2652,  37988, 13156, 56204, 24264, 15484, 54436,
                30856, 37416, 41684, 27560, 34076, 11944, 38724, 6004,  1040,
                24728, 62412, 16316, 21656, 56304, 43860, 18764, 64428, 25080,
                48640, 17220, 50988, 13920, 21672, 51952, 9160,  39772, 29656,
                53684, 34528, 61400, 39392, 43988, 13188, 36336, 24280, 46272,
                7664,  53720, 39552, 13100, 252,   41344, 28860, 3732,  53872,
                34456, 9216,  57488, 30140, 55152, 60672, 10292, 63432, 42340,
                46892, 34464, 240,   41004, 8580,  32660, 4404,  37076, 14512,
                51904, 27708, 22972, 61380, 13684, 34144, 37924, 46668, 43056,
                25672, 30328, 65380, 13888, 22120, 36472, 53996, 39476, 56776,
                112,   35552, 21988, 54788, 30944, 46156, 13196, 33868, 43708,
                63244, 17792,
            },
            {
                54276, 21848, 35288, 5448,  62056, 49976, 11244, 44300, 5916,
                28344, 59572, 18220, 34048, 6620,  52724, 17428, 40040, 60108,
                14056, 53052, 29160, 47096, 12280, 16424, 60440, 37204, 56888,
                48844, 8072,  64808, 34416, 6128,  42508, 59896, 8268,  46672,
                21084, 180,   62096, 5372,  20668, 63560, 45872, 55996, 28876,
                33844, 7552,  46044, 37856, 11336, 35284, 8396,  52628, 39036,
                5328,  56444, 60904, 1336,  30460, 62444, 15632, 23928, 49148,
                7140,  21100, 104,   52492, 18804, 57880, 32524, 15636, 50340,
                19616, 64384, 35392, 26124, 61596, 22800, 10928, 18460, 48452,
                23320, 62248, 44280, 14120, 2252,  41020, 5976,  50848, 27124,
                56936, 6396,  23672, 55024, 28048, 58672, 65004, 44632, 2704,
                42468, 35880, 4960,  45268, 9068,  49940, 64344, 6476,  60780,
                52572, 7232,  37372, 46916, 51528, 5900,  24900, 11584, 19880,
                28844, 64556, 47532, 7496,  16868, 24464, 62028, 6932,  51912,
                14548, 39108,
            },
            {
                11732, 64240, 48116, 14100, 32112, 24580, 16440, 65284, 19948,
                39640, 24296, 54888, 37560, 11460, 44388, 31284, 50668, 2504,
                24888, 33044, 41652, 21308, 65408, 33816, 7324,  3028,  21912,
                32028, 43856, 26808, 22768, 50520, 29376, 19068, 36504, 63768,
                26112, 11372, 55044, 47232, 8544,  31312, 23752, 14872, 49388,
                19056, 54316, 30420, 4172,  63256, 49524, 31872, 57860, 12620,
                28716, 22848, 36476, 40864, 54956, 45680, 35952, 3112,  60164,
                42996, 14644, 47348, 31180, 10228, 26692, 5632,  62648, 37724,
                1888,  31308, 16884, 43692, 47436, 52420, 37636, 56852, 31488,
                4832,  27820, 38112, 17828, 25688, 52712, 36100, 20292, 12752,
                38020, 17968, 63144, 43532, 16080, 39060, 22356, 25704, 17648,
                57548, 63696, 20040, 54728, 30832, 27248, 17004, 2248,  32252,
                11296, 19956, 41536, 1432,  17544, 29788, 42492, 59056, 32988,
                52448, 14448, 10572, 27088, 50416, 39692, 54152, 20076, 29512,
                25052, 46364,
            },
            {
                31096, 7708,  26524, 55864, 208,   42000, 7256,  57780, 33212,
                49176, 9016,  45440, 3448,  25932, 62608, 22644, 8348,  46256,
                11252, 57024, 6112,  1696,  38960, 54544, 45640, 28388, 63232,
                52304, 11152, 16932, 60600, 47088, 9696,  52764, 2520,  32832,
                44332, 51708, 15180, 35516, 40012, 57104, 2928,  42632, 10284,
                64888, 40164, 13044, 52352, 23736, 27076, 1672,  20536, 62552,
                46496, 43160, 10432, 19796, 7056,  27636, 11512, 57456, 32184,
                26208, 63488, 36748, 23164, 59984, 48604, 40400, 53152, 10944,
                23548, 55968, 4116,  9584,  58932, 6708,  15504, 64224, 40308,
                12724, 51532, 7244,  64648, 33232, 45944, 59884, 30356, 49084,
                9024,  59408, 31676, 11472, 51308, 6628,  46692, 54196, 8760,
                32808, 12000, 48204, 1036,  41212, 52032, 21692, 56792, 40060,
                23720, 58056, 62104, 26872, 55836, 63908, 3240,  46108, 8424,
                23564, 38460, 56340, 42696, 60632, 1500,  35852, 10168, 42296,
                3208,  59412,
            },
            {
                37232, 18236, 44484, 21016, 59760, 36728, 46776, 29744, 52808,
                1936,  15272, 63596, 20476, 56084, 41192, 14588, 34412, 54408,
                37628, 18116, 61792, 51536, 24304, 10364, 18964, 41884, 14544,
                4308,  35624, 56068, 564,   31612, 14240, 40996, 58352, 16820,
                6924,  23500, 29440, 59652, 18168, 26444, 52312, 60544, 37132,
                22404, 1872,  57492, 42856, 15348, 45704, 38132, 6308,  16404,
                32704, 2908,  49644, 64100, 33932, 16924, 48344, 41244, 18472,
                4312,  12132, 56268, 7808,  42200, 3524,  20428, 28936, 45564,
                60316, 40836, 49212, 21928, 29560, 33584, 1908,  25144, 45600,
                20488, 58896, 47540, 22072, 11064, 4028,  15264, 23344, 1268,
                53740, 45036, 20976, 36416, 2012,  30472, 13144, 35240, 49488,
                28596, 39844, 24712, 15612, 36780, 59532, 6884,  44416, 35432,
                15192, 48192, 4708,  34172, 12832, 48992, 35916, 15696, 61232,
                49404, 2600,  33880, 19480, 5384,  31656, 16020, 65384, 49568,
                56728, 5952,
            },
            {
                52296, 61152, 40508, 10016, 51696, 27324, 13384, 22204, 10504,
                41328, 36280, 28020, 32348, 47384, 468,   58192, 19464, 64508,
                26312, 44064, 29992, 47680, 13900, 32344, 59936, 49688, 23668,
                58972, 40136, 20272, 44796, 37172, 62956, 22036, 27500, 49032,
                39184, 61776, 42988, 1452,  50068, 12864, 5008,  34396, 17104,
                47996, 27536, 32940, 19432, 7704,  59424, 54868, 41656, 51584,
                26420, 57380, 14488, 24992, 52584, 59004, 776,   22516, 51036,
                54764, 39612, 28684, 49960, 16224, 64816, 35436, 13524, 7436,
                33728, 26852, 14404, 63788, 36852, 18956, 43108, 54684, 9156,
                35356, 496,   41580, 55580, 28348, 57352, 39504, 61636, 32776,
                40948, 25036, 4320,  47864, 61460, 56732, 20388, 62924, 1484,
                59180, 6320,  55448, 61864, 9856,  25852, 12964, 49160, 28064,
                54692, 8344,  30868, 44528, 22908, 9444,  20408, 26664, 41404,
                6124,  29444, 63500, 45644, 23284, 57716, 46988, 27364, 21876,
                33464, 15396,
            },
            {
                24084, 28828, 2296,  33552, 17248, 64068, 3528,  54600, 61200,
                17956, 58316, 49876, 7212,  12344, 38668, 29464, 5620,  48928,
                2140,  15388, 8912,  36196, 57668, 27068, 1120,  37656, 9320,
                47272, 27556, 7572,  54420, 25440, 3424,  56552, 11740, 34552,
                3920,  55908, 10120, 21920, 32644, 64368, 44992, 29904, 8888,
                58732, 5584,  50884, 61960, 35660, 29396, 10480, 21820, 34704,
                60640, 8020,  39660, 31276, 5800,  43420, 36692, 65428, 9560,
                34744, 45376, 19712, 1844,  32828, 25392, 54080, 58040, 17868,
                51504, 5120,  44404, 53228, 12152, 58156, 49756, 28132, 62972,
                17132, 30268, 13984, 34228, 50192, 8200,  43688, 17580, 6828,
                65328, 14480, 28360, 53072, 16540, 26176, 40504, 43388, 17016,
                22572, 44736, 31324, 19164, 46772, 34208, 63952, 18308, 552,
                60864, 37308, 19080, 51640, 59676, 38144, 53276, 31820, 56920,
                18820, 52044, 15312, 11228, 37080, 7680,  40956, 12568, 524,
                39776, 45660,
            },
            {
                63924, 13176, 55428, 49672, 6388,  43484, 39000, 34196, 25152,
                44220, 4412,  23068, 53644, 17668, 60952, 51400, 23204, 42132,
                31996, 60576, 20948, 4216,  41384, 62908, 6264,  52816, 33964,
                16024, 30792, 61408, 12560, 18272, 51188, 8356,  46168, 65352,
                19844, 14284, 37640, 47692, 7116,  40852, 19968, 55604, 25336,
                42168, 37956, 12216, 23472, 2336,  48988, 65184, 13456, 624,
                47580, 18184, 54404, 45076, 20732, 13164, 26404, 29944, 15712,
                5404,  24524, 62468, 57320, 8952,  46812, 43376, 500,   37968,
                62048, 20636, 31572, 2328,  39524, 8024,  22856, 3692,  38660,
                51928, 61128, 24620, 3212,  19536, 36916, 25980, 48120, 55048,
                35632, 9716,  58756, 38076, 7328,  33688, 10388, 4388,  51272,
                35776, 13564, 52544, 2400,  42404, 4296,  55876, 31460, 10492,
                42744, 24840, 14176, 2188,  41140, 6372,  61688, 616,   12100,
                44672, 39264, 25612, 59680, 53800, 29836, 51136, 61684, 54472,
                18980, 8940,
            },
            {
                3944,  47164, 36900, 22556, 30840, 57728, 20060, 8252,  48356,
                30540, 13680, 64776, 33784, 43704, 26476, 9140,  35676, 12572,
                53272, 38324, 56244, 50348, 22964, 16800, 44456, 20360, 64960,
                56912, 2276,  41064, 47980, 33468, 38492, 29244, 41636, 24140,
                31764, 53732, 28192, 58132, 24892, 61056, 15036, 51248, 320,
                63912, 16212, 54088, 46360, 40432, 17320, 25508, 43972, 30044,
                38368, 23708, 11448, 63324, 3196,  60076, 38928, 53500, 48112,
                58784, 31360, 12812, 52112, 38844, 21504, 11484, 30584, 24456,
                47900, 9960,  56612, 24832, 62372, 45916, 15280, 32920, 11528,
                46924, 6852,  54252, 44820, 59036, 63792, 11980, 3832,  29628,
                19636, 50712, 42152, 21760, 45264, 64472, 55716, 23864, 60456,
                27528, 8412,  65224, 37824, 28920, 23476, 50768, 39572, 21508,
                52916, 47412, 64692, 56108, 29176, 17020, 24340, 47028, 65144,
                34844, 8676,  1976,  48380, 20868, 3896,  17240, 25272, 35972,
                59016, 31584,
            },
            {
                42884, 16436, 58544, 10796, 26224, 46100, 15404, 62480, 760,
                51800, 40108, 9784,  37680, 1680,  55940, 45852, 63476, 16584,
                6844,  45308, 28168, 11196, 34364, 54928, 29952, 12988, 25212,
                43076, 5840,  22976, 53528, 64036, 4796,  14856, 59336, 692,
                49708, 6104,  43980, 17028, 34892, 3696,  38620, 11000, 47424,
                31908, 20820, 28664, 9392,  56800, 32036, 6544,  58240, 52716,
                61676, 4452,  36032, 28172, 51132, 46592, 7724,  19192, 1568,
                40568, 43876, 17684, 35068, 27472, 63708, 4968,  59384, 40664,
                14860, 34836, 42360, 17148, 28588, 35212, 55304, 60496, 26660,
                42524, 21668, 35868, 9428,  16156, 31708, 52120, 22688, 57948,
                45748, 540,   12668, 31920, 3072,  49792, 14564, 30844, 46836,
                39376, 57360, 21120, 47988, 16332, 58336, 13992, 5728,  62776,
                33840, 3504,  27148, 11044, 45516, 33204, 50360, 14820, 28152,
                22296, 54980, 31244, 62300, 42740, 34216, 9980,  44564, 5292,
                48412, 26504,
            },
            {
                34680, 20696, 40940, 62044, 1788,  51332, 11852, 36828, 27508,
                59788, 20884, 57096, 29236, 14900, 20068, 4196,  30524, 57424,
                25608, 18976, 64728, 644,   47336, 7528,  39024, 48996, 10172,
                51456, 36460, 16744, 9688,  26596, 20800, 43900, 54696, 18020,
                36892, 12352, 63148, 51788, 9348,  54656, 27932, 22080, 36024,
                6760,  61196, 44460, 4468,  62836, 37248, 50324, 19832, 9504,
                14868, 42832, 56872, 32124, 16804, 23332, 33524, 56096, 11596,
                61716, 22564, 6348,  2376,  49020, 15744, 55360, 19288, 50772,
                2876,  65392, 52404, 6592,  48932, 668,   20200, 50852, 5024,
                17728, 65140, 40532, 29276, 47480, 1836,  41820, 38692, 14816,
                33560, 63536, 24692, 59672, 18440, 27172, 37568, 6992,  18848,
                108,   11808, 33256, 5208,  61468, 9040,  36164, 45800, 28692,
                16516, 8108,  39144, 19416, 36332, 58608, 9112,  38888, 4024,
                58184, 41352, 16452, 12780, 26572, 56668, 38872, 64688, 22340,
                11704, 52096,
            },
            {
                63196, 8088,  53252, 5628,  33916, 39888, 56092, 22716, 42992,
                17008, 6768,  47068, 24824, 61724, 49144, 38824, 22180, 49984,
                3400,  41256, 32384, 15176, 24604, 61844, 58864, 1780,  33140,
                27400, 62472, 45080, 31532, 57896, 50076, 35080, 9012,  61584,
                26696, 40240, 2848,  20672, 30864, 46684, 42628, 60252, 56684,
                13404, 49952, 34668, 26228, 15064, 22524, 2736,  41268, 33720,
                27112, 21300, 49232, 1016,  9876,  41716, 63724, 25716, 36264,
                28632, 50268, 54936, 60244, 41720, 32316, 45520, 36604, 26880,
                8664,  29584, 21448, 12772, 59588, 43640, 10204, 37316, 31260,
                57720, 1240,  12428, 24940, 56324, 20784, 62632, 5528,  27404,
                52960, 8364,  39272, 47320, 55112, 10920, 43560, 53480, 63356,
                41628, 50372, 24552, 43844, 53608, 26556, 41228, 20080, 54716,
                59708, 42288, 49348, 62088, 22476, 2664,  53992, 43504, 20064,
                51192, 7272,  36664, 46828, 19468, 1212,  50032, 15488, 30640,
                55572, 1596,
            },
            {
                14252, 29320, 25004, 47908, 19212, 28228, 65076, 3836,  32824,
                54360, 35452, 2992,  41616, 52568, 11520, 33236, 43192, 62416,
                10576, 36528, 58028, 53456, 43232, 35480, 17204, 21856, 55872,
                14244, 59600, 19608, 1172,  39712, 12764, 3984,  30224, 22188,
                47960, 32880, 59460, 24300, 13872, 63972, 2288,  16764, 24780,
                41160, 1456,  18760, 39720, 54756, 47808, 12460, 64440, 56076,
                47004, 5496,  62640, 37684, 58592, 14148, 51924, 3812,  46036,
                15104, 8476,  37220, 25128, 10644, 6612,  23044, 12908, 60588,
                44248, 57280, 38124, 32780, 26004, 40068, 63948, 23812, 14344,
                48424, 53304, 33360, 60836, 6564,  50276, 35052, 11184, 60388,
                43600, 17324, 29480, 5036,  35808, 61592, 2212,  21432, 34380,
                15300, 56468, 30520, 18200, 35124, 1020,  48676, 11960, 3940,
                23336, 30788, 13020, 52520, 6776,  31524, 64084, 11992, 29712,
                60928, 24976, 2912,  63516, 52908, 32640, 60220, 6208,  41092,
                18584, 37940,
            },
            {
                49720, 32716, 61188, 12580, 44364, 16004, 7904,  49224, 10484,
                45016, 13328, 64180, 31120, 7936,  18180, 5300,  55296, 14392,
                29052, 46488, 7868,  20352, 11788, 28520, 4916,  46328, 41816,
                3480,  38504, 8068,  47744, 54168, 23932, 63468, 46220, 15216,
                57012, 7104,  43488, 50604, 39136, 5696,  33336, 51468, 8596,
                29920, 52996, 59052, 10704, 30580, 60460, 36088, 29064, 17492,
                8304,  52468, 25000, 18844, 44508, 29456, 6144,  39336, 20472,
                32772, 64716, 18572, 47000, 30048, 57956, 52144, 860,   33228,
                17672, 4528,  47652, 53792, 5484,  18340, 56524, 7896,  45108,
                27116, 38516, 19488, 43896, 15076, 28184, 45560, 18580, 23844,
                2020,  56548, 50940, 22260, 15668, 31632, 51328, 25280, 59348,
                28104, 4188,  9544,  64332, 13688, 58952, 32108, 62216, 50628,
                37468, 57656, 228,   26424, 46296, 15580, 25404, 37380, 17460,
                47624, 34080, 44596, 11480, 28436, 8380,  23684, 47628, 27580,
                58480, 45124,
            },
            {
                22748, 56012, 2800,  38488, 58744, 53852, 30480, 23900, 61168,
                19960, 27284, 48752, 22468, 58496, 36624, 60548, 25944, 1564,
                52108, 23432, 60840, 3136,  49296, 39792, 52268, 64576, 30780,
                24756, 50508, 34460, 28316, 15952, 58944, 36540, 41304, 1824,
                53188, 11268, 17632, 35940, 56008, 26920, 19684, 44928, 36984,
                65476, 21360, 46480, 3372,  43328, 7748,  24704, 224,   45408,
                38312, 30500, 12028, 34716, 53860, 22724, 47288, 60364, 56924,
                416,   52840, 40448, 13268, 62700, 20096, 38512, 48156, 64064,
                41480, 24160, 10376, 62748, 15452, 30264, 51980, 34400, 4420,
                62428, 10392, 3604,  51676, 36856, 9256,  55080, 31880, 49324,
                37000, 13132, 41456, 62852, 8744,  46180, 40096, 12644, 7728,
                47832, 37448, 45220, 51496, 39880, 21744, 6292,  25748, 15996,
                9192,  44180, 18224, 34256, 59908, 41740, 49204, 56520, 5236,
                9488,  54292, 21960, 58768, 38424, 43252, 13480, 36320, 63820,
                10144, 4928,
            },
            {
                42260, 17312, 9132,  21832, 34740, 192,   42316, 36224, 57888,
                39292, 1228,  55376, 15984, 40688, 28284, 45984, 20644, 40204,
                34180, 17360, 38188, 31304, 62916, 25636, 6604,  19020, 9088,
                53932, 12016, 63756, 21636, 43100, 6324,  10508, 25820, 31960,
                20432, 65024, 29288, 428,   9700,  48468, 59852, 11644, 55084,
                5880,  14060, 25936, 34024, 16576, 50688, 20100, 53788, 59228,
                14664, 61340, 40832, 1912,  65212, 8672,  17416, 12600, 30384,
                42624, 23536, 5524,  34600, 2656,  43728, 25984, 9324,  15204,
                55216, 28740, 20568, 45216, 36720, 2836,  49136, 21932, 40412,
                16732, 59080, 31000, 22208, 64304, 440,   40556, 59252, 7000,
                65172, 26716, 34036, 3192,  57816, 28452, 18056, 55760, 42220,
                19928, 61048, 23252, 2904,  28824, 56072, 46460, 42408, 34964,
                65312, 28080, 55384, 39328, 8512,  20712, 2204,  32868, 62824,
                39672, 14756, 30880, 17688, 3444,  56944, 20740, 52168, 640,
                25548, 35156,
            },
            {
                60580, 51476, 46492, 64108, 26748, 50364, 14072, 4896,  17456,
                51752, 32260, 9644,  43964, 3660,  12104, 54040, 7376,  65240,
                48108, 56568, 13056, 44372, 54616, 14992, 33920, 58588, 37164,
                16304, 45392, 4836,  56752, 32744, 51892, 18488, 60216, 49120,
                38548, 45088, 23648, 61680, 42144, 15748, 22984, 40372, 27732,
                32368, 49004, 60908, 37844, 56592, 63620, 40320, 5304,  33244,
                22608, 43808, 4764,  56784, 27304, 33096, 50316, 36804, 26188,
                9840,  48228, 59632, 28332, 56176, 16528, 53264, 31240, 3680,
                35900, 50148, 1360,  60768, 41708, 24860, 58548, 11232, 29328,
                53916, 47324, 25188, 42368, 57608, 17276, 25856, 13848, 21240,
                46540, 9816,  19300, 48764, 23400, 38016, 5764,  64892, 1392,
                30996, 53412, 11424, 16704, 36628, 8340,  18832, 53640, 2524,
                21220, 48100, 5152,  14008, 62284, 53008, 11556, 23132, 27804,
                50872, 424,   42092, 65332, 49656, 33112, 45256, 16244, 31060,
                54540, 12336,
            },
            {
                29156, 39420, 5712,  31476, 11120, 19816, 62068, 47120, 25976,
                7108,  63576, 35800, 24952, 62616, 32900, 49660, 15772, 30080,
                9740,  5468,  27512, 388,   21196, 9880,  47952, 43284, 23228,
                60416, 26848, 40428, 160,   13668, 46548, 29536, 3036,  55072,
                5500,  13512, 51804, 34160, 57972, 31140, 6752,  63028, 980,
                18032, 43836, 4416,  22884, 9244,  28240, 13284, 48456, 26200,
                9968,  51072, 18400, 48140, 13756, 43240, 61996, 2292,  54124,
                63600, 14268, 18128, 50532, 7976,  36696, 61744, 21516, 59780,
                40292, 12368, 56904, 31072, 13908, 8468,  65056, 19036, 44072,
                2240,  35344, 5436,  12356, 33648, 48072, 53392, 4148,  39520,
                30068, 55552, 42348, 61008, 11872, 54064, 33580, 49480, 14916,
                35652, 26604, 43656, 58384, 48368, 63412, 31084, 14080, 57848,
                11348, 32404, 51120, 24632, 30160, 35928, 44116, 58904, 18600,
                45808, 55368, 24508, 7520,  26656, 10964, 5624,  62120, 38100,
                48252, 19448,
            },
            {
                3264,  24252, 15268, 57368, 43352, 54664, 39868, 29852, 55924,
                12808, 46052, 18268, 50992, 56740, 21840, 1080,  36968, 43852,
                19516, 60340, 41252, 50984, 36260, 57324, 29824, 2124,  5824,
                49820, 31112, 19728, 61752, 37632, 22424, 64392, 42024, 16012,
                35788, 27496, 8036,  19436, 3428,  53624, 45596, 34860, 51348,
                57256, 39040, 11912, 54340, 31904, 2256,  44972, 36940, 57612,
                64272, 29128, 36016, 21828, 7152,  39764, 24068, 5812,  45476,
                21160, 31604, 38108, 44536, 23904, 11412, 42188, 6280,  46388,
                26460, 18216, 6860,  34512, 53076, 27872, 46744, 37616, 56344,
                14660, 63584, 50664, 19936, 7316,  28468, 44396, 62208, 34980,
                52116, 6196,  15124, 31928, 348,   43572, 25872, 21464, 57524,
                8896,  62976, 4760,  32560, 24596, 1576,  41148, 26432, 37984,
                45764, 60468, 40768, 1636,  16944, 47528, 4092,  38440, 7832,
                34748, 12616, 60960, 36404, 40384, 53036, 59484, 23312, 41712,
                7548,  56812,
            },
            {
                44520, 33940, 61284, 36860, 7576,  23280, 2680,  9772,  37716,
                21632, 41472, 29076, 4848,  14068, 39480, 61292, 25196, 58400,
                53236, 22832, 32392, 64280, 17224, 25016, 61352, 40916, 14136,
                55184, 35456, 10736, 48896, 54356, 7072,  34296, 9384,  24760,
                62816, 56576, 47596, 39304, 12564, 26380, 20828, 10004, 14736,
                24688, 64432, 19840, 49772, 41768, 62228, 17620, 21312, 7720,
                1048,  15824, 55108, 61116, 31428, 52040, 19252, 58816, 34168,
                40968, 4748,  52600, 908,   65092, 29784, 49724, 14624, 33440,
                51796, 64464, 43888, 22804, 48768, 16776, 164,   31992, 7228,
                23696, 27328, 39152, 60964, 54948, 37904, 10748, 23528, 17840,
                1932,  63684, 24908, 36752, 58716, 17152, 9976,  45472, 38508,
                52056, 18312, 40000, 13464, 50732, 10008, 59748, 52308, 5588,
                22400, 7960,  19296, 56296, 64540, 26876, 54632, 14880, 63580,
                30456, 4664,  19908, 47920, 14520, 30192, 18840, 2072,  27440,
                14792, 64996,
            },
            {
                20916, 48956, 992,   52780, 28616, 16100, 49556, 65504, 32004,
                58208, 1884,  60428, 47572, 8796,  30728, 42564, 11180, 4364,
                34936, 13704, 47104, 8408,  4488,  45648, 11748, 51864, 22056,
                65500, 8004,  44288, 25260, 15656, 28036, 57676, 50892, 20648,
                43800, 1264,  30476, 60924, 50072, 41548, 61948, 37168, 47552,
                2588,  30560, 7176,  26568, 14248, 34712, 55440, 50172, 30932,
                42364, 38936, 46216, 3220,  10724, 57140, 14712, 29020, 8868,
                61432, 12260, 26812, 57732, 33796, 19392, 4184,  58668, 23324,
                2460,  9812,  38428, 59020, 4120,  55748, 41284, 21048, 60036,
                43020, 10108, 45836, 16372, 31220, 3148,  15320, 59472, 49776,
                27900, 44640, 20676, 51352, 47896, 28572, 63856, 5172,  30188,
                1992,  22624, 46164, 55684, 35092, 20728, 17144, 44012, 29648,
                63024, 35448, 12588, 43116, 31700, 9940,  21528, 41224, 25364,
                48792, 57048, 43652, 22760, 3652,  57364, 35248, 46332, 51548,
                32416, 9928,
            },
            {
                40744, 26704, 12696, 18740, 46160, 59376, 34472, 43936, 13980,
                6288,  53204, 24412, 33956, 20296, 53908, 48888, 27976, 17744,
                50596, 1832,  39592, 26864, 56164, 37920, 19328, 33744, 28092,
                39132, 18108, 3100,  59836, 40736, 1524,  45244, 12432, 31720,
                38736, 17700, 10288, 23420, 15908, 54904, 5316,  28456, 58832,
                43036, 33944, 52748, 46140, 60000, 5920,  24288, 11220, 60832,
                53156, 12736, 23764, 34280, 26484, 44104, 37368, 48960, 17300,
                46340, 55800, 22144, 15044, 47040, 38992, 54028, 27748, 36944,
                55516, 30416, 20088, 12972, 25808, 35416, 63180, 12084, 48372,
                33876, 53840, 1616,  22360, 57808, 48628, 42736, 32668, 6868,
                37360, 57980, 12836, 8052,  3840,  40420, 53056, 14152, 60284,
                48952, 27128, 61732, 6836,  28988, 64964, 37156, 3708,  14776,
                49236, 25040, 53380, 37564, 6224,  50060, 61040, 972,   52348,
                17612, 10132, 32364, 53752, 64292, 8572,  42428, 11652, 62580,
                37140, 53428,
            },
            {
                4280,  55664, 63528, 31660, 41044, 5000,  25316, 19568, 51580,
                27760, 16980, 44584, 38696, 64484, 15848, 6404,  63072, 45028,
                55808, 29980, 62460, 15608, 49392, 30708, 1176,  62076, 4940,
                46852, 57100, 29504, 33548, 52152, 19344, 36644, 62124, 4112,
                58492, 52792, 45672, 36144, 2508,  33072, 8748,  17732, 21792,
                54444, 10868, 16936, 38068, 712,   29532, 39796, 46940, 4044,
                27080, 19760, 62604, 50608, 6236,  64896, 144,   54380, 24544,
                3720,  36052, 42580, 6664,  62496, 9472,  43564, 12536, 63464,
                16468, 41988, 50424, 61656, 45336, 6152,  29636, 52688, 17964,
                5140,  29168, 62340, 35920, 8496,  26164, 64656, 20092, 55744,
                10020, 41216, 31136, 62180, 35216, 18912, 23540, 32512, 42276,
                36308, 11440, 16048, 43084, 188,   24076, 48220, 54432, 33656,
                58484, 888,   28208, 59840, 15968, 44684, 33608, 12760, 28668,
                36824, 60160, 2208,  38752, 28108, 17316, 49928, 25252, 21836,
                6244,  16996,
            },
            {
                47196, 23188, 38456, 8584,  50976, 11984, 62544, 724,   39828,
                35580, 61892, 7024,  11332, 56584, 268,   35440, 21052, 38320,
                8724,  24236, 43332, 20892, 10564, 58656, 44216, 54156, 16188,
                11080, 50492, 23000, 13060, 64160, 5360,  24024, 48616, 15276,
                26464, 7252,  65236, 28852, 57180, 44600, 63720, 49308, 40528,
                3808,  63264, 23724, 57560, 20156, 43768, 64804, 16348, 32856,
                37184, 8428,  57788, 16668, 40684, 21124, 13040, 30760, 40100,
                63508, 28176, 49504, 31680, 18204, 25684, 1348,  48936, 20956,
                45996, 8032,  912,   32980, 16088, 39804, 9292,  24064, 58560,
                39404, 13324, 19560, 41692, 52164, 12272, 38628, 520,   24500,
                47600, 16724, 53596, 26436, 45804, 59872, 10656, 55064, 7536,
                20220, 57892, 33224, 52672, 39216, 57408, 12212, 8080,  41324,
                18284, 10464, 46700, 20500, 4192,  24056, 39224, 57704, 46624,
                6684,  42516, 24988, 12424, 47208, 33640, 60504, 864,   56648,
                30916, 59556,
            },
            {
                14364, 34556, 2232,  57436, 27984, 21548, 53928, 47260, 9944,
                57628, 22916, 49624, 31420, 27208, 41916, 24928, 59992, 31936,
                13004, 57952, 4996,  53328, 35716, 6824,  22920, 41232, 25996,
                37340, 62820, 42804, 7260,  46304, 30816, 53904, 9116,  41860,
                34632, 21872, 49436, 13256, 19732, 24784, 37712, 30388, 13744,
                26264, 35840, 8000,  48664, 13088, 51920, 9064,  22660, 59312,
                49044, 43220, 1436,  31964, 47828, 35748, 59828, 7648,  51224,
                19656, 10832, 1900,  60776, 51868, 35700, 59716, 30076, 5596,
                34420, 60408, 27488, 54848, 22096, 51616, 47044, 37068, 1064,
                64424, 50160, 25076, 56352, 4164,  29772, 45360, 14724, 61492,
                34792, 5432,  22380, 1124,  14432, 38408, 2668,  44764, 25420,
                64028, 3016,  49668, 9344,  19096, 31556, 45832, 27184, 22020,
                63904, 38268, 52224, 30512, 62532, 54104, 8564,  18836, 64612,
                22292, 15924, 51556, 62364, 20580, 4628,  40808, 13544, 38256,
                44256, 28140,
            },
            {
                61092, 10860, 43268, 17188, 45504, 32936, 37252, 15464, 29452,
                43012, 2356,  13604, 19024, 46224, 52400, 10528, 47964, 3364,
                51424, 16988, 32960, 40732, 64752, 27896, 13840, 60172, 8668,
                31716, 2084,  20384, 35572, 56484, 16616, 38148, 28304, 60488,
                55252, 892,   39324, 42796, 4980,  52412, 11308, 128,   60320,
                51068, 43596, 61268, 31968, 27480, 56532, 34804, 2880,  54580,
                13976, 29392, 53744, 11568, 27588, 55536, 23440, 43140, 15232,
                34284, 57188, 38788, 13000, 21884, 41496, 15344, 56848, 39512,
                52772, 24572, 10444, 41372, 64956, 3084,  57448, 27712, 15660,
                44320, 30808, 6720,  46840, 17112, 59148, 33192, 50288, 54532,
                29104, 43456, 65376, 48620, 56328, 33508, 50760, 29828, 16952,
                47284, 37624, 28180, 23088, 61376, 15120, 59548, 5424,  55832,
                32540, 3324,  42464, 13748, 35760, 48652, 27396, 31956, 3764,
                54932, 34828, 30260, 44836, 7948,  55372, 26184, 53140, 19136,
                8816,  50676,
            },
            {
                20552, 54172, 25880, 64560, 6328,  59868, 3664,  55784, 20132,
                64736, 33532, 54312, 60480, 37020, 4108,  57204, 18620, 40072,
                28164, 61896, 47280, 580,   18484, 51744, 48348, 34832, 52992,
                17512, 55520, 48948, 27188, 10604, 59660, 21324, 3712,  45848,
                18648, 10960, 31104, 62244, 58800, 16740, 47752, 55560, 33484,
                19312, 15628, 4852,  40928, 1928,  37780, 18592, 26276, 41640,
                5232,  20812, 64324, 45492, 18612, 9340,  2324,  62312, 46456,
                4128,  53232, 25960, 44704, 7216,  54564, 3168,  23492, 11532,
                17860, 47820, 37528, 14400, 31160, 19696, 12480, 33644, 21276,
                55524, 10580, 34800, 61880, 23024, 40632, 9100,  21004, 3508,
                11700, 38964, 17952, 8372,  27940, 19652, 62452, 6200,  55804,
                12756, 41604, 5228,  54096, 43944, 2808,  35752, 50728, 16408,
                44592, 24364, 8360,  57784, 17708, 1180,  11500, 44188, 40604,
                49536, 10404, 116,   58248, 36092, 15408, 48308, 32576, 64868,
                3436,  35912,
            },
            {
                5604,  31592, 39444, 49540, 12968, 23772, 41936, 8416,  26056,
                48132, 4636,  40488, 23652, 7544,  15520, 33672, 65296, 23220,
                43956, 7956,  37092, 25520, 11436, 31016, 2784,  21088, 4820,
                42076, 60808, 14092, 40008, 308,   43428, 50512, 13652, 64404,
                33492, 51860, 25780, 8456,  35152, 28028, 41380, 22820, 7016,
                45948, 29256, 54636, 21336, 47636, 10800, 63436, 45388, 60276,
                38896, 24520, 34140, 3512,  59220, 50384, 39356, 32280, 27228,
                21224, 64416, 16880, 48312, 33912, 28528, 65460, 45452, 32248,
                62864, 2236,  58908, 6816,  49448, 43732, 4788,  60396, 41500,
                48056, 2920,  38484, 14028, 28016, 1896,  64196, 36596, 26496,
                58344, 51644, 31812, 59140, 42356, 11188, 23032, 39840, 35096,
                59656, 20952, 32172, 64696, 11552, 24816, 40296, 29492, 61820,
                12688, 49012, 27948, 65432, 39736, 23348, 56264, 61696, 14312,
                25240, 60856, 18756, 39808, 23744, 63040, 29484, 10600, 46212,
                41584, 24568,
            },
            {
                62320, 47020, 236,   19012, 36124, 51492, 30620, 61244, 38096,
                12136, 16656, 50868, 28368, 63068, 44848, 30000, 12112, 2264,
                54060, 14416, 21544, 60044, 55132, 44356, 38356, 63760, 46748,
                29616, 23168, 6120,  33220, 63692, 24440, 31376, 7756,  40456,
                23460, 16184, 55952, 47148, 21236, 2448,  64316, 10088, 57908,
                39208, 12024, 65464, 24864, 58600, 14652, 50556, 6416,  30928,
                9912,  52192, 15224, 42376, 36788, 25220, 16480, 58096, 9920,
                37460, 5732,  31088, 59076, 10348, 19044, 37436, 49900, 8292,
                41832, 21264, 28556, 56252, 25468, 62100, 36576, 52124, 8008,
                26048, 18692, 58888, 53340, 43072, 47840, 55848, 18496, 44924,
                7268,  23512, 13832, 36448, 3868,  47420, 52924, 15416, 612,
                26752, 8624,  51260, 17408, 37128, 48020, 7772,  20172, 660,
                34908, 54484, 20896, 5212,  45864, 51700, 34316, 20016, 37692,
                6556,  28960, 52384, 12612, 43404, 6472,  21716, 1288,  58692,
                16916, 56132,
            },
            {
                8388,  15196, 29424, 58728, 9284,  45116, 15880, 1136,  53172,
                44540, 31616, 58684, 9724,  20972, 39564, 55408, 49496, 36268,
                59064, 31688, 50168, 42056, 6560,  15820, 57968, 9028,  26244,
                11900, 37516, 51668, 56780, 19032, 46468, 53736, 36464, 57584,
                1924,  43300, 4744,  37888, 13988, 44864, 31692, 36760, 26036,
                1488,  51484, 35476, 8300,  42688, 28656, 33732, 22816, 16864,
                57396, 46652, 7504,  62952, 30272, 5532,  53976, 48756, 13368,
                42012, 50184, 23356, 1004,  43432, 61144, 4504,  26412, 14788,
                53900, 35816, 44952, 16940, 32700, 9892,  22824, 14840, 29860,
                62744, 50612, 32216, 21536, 7492,  11052, 31144, 14932, 40336,
                48964, 63220, 1448,  54576, 25900, 63916, 31404, 44304, 61540,
                48836, 42808, 58136, 1808,  28540, 55348, 63276, 52060, 41732,
                10060, 59116, 37744, 31828, 15136, 7804,  29900, 2420,  48144,
                57568, 45048, 33076, 4316,  56968, 49760, 34536, 52700, 13172,
                27744, 38080,
            },
            {
                51016, 42240, 53352, 22324, 63852, 27192, 57300, 35204, 21892,
                62688, 25056, 5792,  36424, 47612, 1112,  25616, 6060,  20108,
                27156, 10300, 3856,  64372, 28320, 23620, 33716, 18824, 53548,
                62516, 43720, 16960, 9376,  28724, 3024,  15548, 11496, 27212,
                60972, 49860, 29976, 61856, 57288, 50368, 17652, 53216, 60472,
                20508, 48236, 30980, 17936, 4564,  38444, 55768, 300,   62628,
                36076, 27432, 19932, 54732, 12076, 22480, 44212, 1772,  29008,
                60368, 55128, 15160, 39440, 52496, 12924, 21584, 56052, 30736,
                61576, 5064,  12348, 52844, 1760,  40652, 46588, 57988, 304,
                39604, 12172, 5540,  36172, 63140, 25208, 51436, 60540, 4524,
                28324, 33984, 43804, 21040, 39092, 17644, 4956,  10260, 29072,
                18992, 24120, 14436, 34344, 45288, 22092, 13796, 32788, 26500,
                46452, 17236, 2156,  53064, 41280, 58468, 26272, 63764, 9720,
                16340, 21208, 65128, 26956, 37508, 17980, 61088, 39536, 44548,
                20168, 33080,
            },
            {
                23948, 12196, 34940, 3008,  40288, 5820,  19540, 49424, 10764,
                3164,  40944, 18776, 56972, 13868, 61428, 17156, 42956, 62892,
                46628, 38684, 19064, 34644, 47780, 12652, 50564, 40392, 1072,
                31836, 3864,  48388, 59380, 34376, 41696, 62828, 48832, 21580,
                7076,  34164, 11924, 19572, 7696,  25084, 3380,  12872, 43700,
                5924,  15056, 62020, 44620, 52076, 61076, 19348, 48480, 12548,
                43248, 50960, 1608,  40028, 60676, 33588, 18540, 63396, 35224,
                19780, 8228,  33208, 27832, 57844, 35272, 46320, 40180, 404,
                18628, 24880, 49048, 37848, 63816, 19456, 54412, 34444, 17504,
                44164, 23868, 56768, 46068, 17892, 39068, 684,   34924, 22436,
                56820, 10280, 15964, 60360, 7980,  50092, 34596, 53884, 37972,
                57072, 6616,  40080, 61024, 9532,  3908,  58416, 18376, 5868,
                61276, 30116, 24512, 63240, 11416, 18772, 49856, 42776, 35220,
                54332, 41108, 1644,  46808, 14952, 8972,  25092, 30332, 6100,
                63272, 1984,
            },
            {
                55476, 60624, 17780, 48276, 54920, 33392, 13484, 42792, 28488,
                54840, 32796, 45320, 52260, 29684, 34360, 51176, 8492,  32552,
                53512, 16092, 58372, 41176, 2024,  56152, 6248,  61504, 45972,
                14676, 22004, 36272, 24924, 5348,  20512, 55608, 30392, 44604,
                16568, 40200, 53476, 28460, 42148, 34776, 62760, 29364, 39644,
                33396, 57452, 23428, 2096,  13420, 26256, 9564,  40348, 24080,
                5004,  31492, 14752, 25868, 49408, 8984,  38780, 47716, 6204,
                25088, 46728, 64012, 5320,  17368, 24668, 8688,  64144, 50944,
                43092, 33576, 59964, 9136,  28968, 6140,  25836, 10940, 49836,
                65404, 27880, 2412,  15208, 49396, 28728, 54396, 13108, 45484,
                18904, 52104, 30872, 46956, 27388, 56436, 22428, 14004, 64836,
                2832,  32380, 47880, 52732, 26136, 38088, 43068, 49728, 35540,
                56116, 13972, 43744, 47824, 35936, 3316,  22836, 13216, 30976,
                5796,  24284, 11392, 51128, 31516, 54536, 2456,  47892, 57432,
                11168, 46428,
            },
            {
                26508, 4096,  43556, 28136, 9788,  24112, 63156, 37584, 59812,
                8332,  15244, 65044, 4944,  11032, 23544, 58848, 40824, 13152,
                40,    24724, 7792,  51500, 30732, 20460, 36184, 25180, 28800,
                58276, 54652, 11256, 65340, 51460, 38648, 9576,  756,   37084,
                58968, 24268, 65064, 3968,  48708, 10452, 54924, 21756, 46792,
                9452,  27468, 41068, 53912, 36660, 32724, 45844, 54804, 59776,
                34604, 64052, 56452, 45568, 3104,  58440, 27372, 14044, 56964,
                40608, 11764, 43216, 51736, 2132,  48940, 32284, 11884, 28124,
                7168,  57304, 13800, 21608, 47848, 42140, 61436, 31628, 4144,
                36524, 53984, 41516, 33444, 59096, 6432,  41896, 64580, 8568,
                37648, 62220, 6000,  40656, 12496, 876,   41756, 45756, 25452,
                50888, 12240, 20688, 16728, 30800, 63940, 11832, 23004, 3180,
                39168, 9172,  21000, 6132,  28232, 61984, 56188, 45548, 59664,
                38664, 28872, 61500, 57336, 19964, 42556, 64360, 22452, 16696,
                40952, 36560,
            },
            {
                20400, 65400, 31696, 15532, 58580, 45588, 1800,  52436, 17448,
                46996, 22696, 38716, 27136, 43368, 2500,  37284, 21228, 28444,
                56916, 44840, 63448, 26832, 14376, 59468, 43672, 9548,  17256,
                39380, 7100,  42572, 27936, 45332, 13780, 61520, 26260, 52272,
                5736,  46720, 14308, 21440, 38500, 59536, 16076, 51296, 400,
                64592, 18552, 48980, 7700,  62968, 21372, 2840,  29180, 8016,
                17372, 20980, 10084, 36960, 23532, 17284, 51828, 31988, 364,
                53524, 21936, 30672, 37216, 61888, 20708, 38556, 59216, 16608,
                23216, 45628, 39928, 30952, 3224,  56212, 14472, 45324, 20700,
                8452,  13504, 22280, 62108, 10380, 19716, 24220, 31952, 50256,
                26788, 2104,  23660, 35600, 58228, 62704, 29576, 18212, 8576,
                36256, 43460, 62240, 52,    54304, 7204,  47076, 28832, 52424,
                64976, 31944, 50548, 54760, 40472, 33032, 8556,  17252, 1084,
                50496, 18568, 44444, 33516, 6840,  38832, 12984, 35164, 53420,
                29468, 7824,
            },
            {
                48596, 38804, 6036,  50504, 36932, 19812, 30568, 5276,  26000,
                35060, 460,   56024, 49340, 19204, 54372, 46832, 64488, 5420,
                49264, 35488, 22040, 10112, 38008, 47064, 4660,  64072, 52300,
                33340, 49800, 19084, 3356,  32096, 17500, 47472, 33724, 19316,
                10596, 31896, 55456, 1260,  45316, 31412, 6680,  36924, 24732,
                55624, 35088, 12188, 30248, 16560, 57144, 50576, 13944, 37928,
                48184, 52984, 42840, 30496, 61312, 4756,  43496, 65088, 36544,
                16760, 60492, 8528,  27008, 13636, 55352, 42504, 4076,  53424,
                36548, 1692,  52244, 64732, 17616, 35496, 23816, 59728, 38784,
                51376, 57932, 30632, 4612,  37812, 48032, 56560, 3532,  14208,
                59276, 43548, 55056, 15688, 20312, 48260, 33300, 4784,  59752,
                56080, 28712, 23268, 40676, 35320, 14680, 57516, 19664, 41436,
                16200, 26060, 508,   60664, 14972, 24984, 52608, 36964, 21732,
                63512, 9332,  14584, 2940,  25480, 58880, 27980, 4632,  50144,
                59956, 14172,
            },
            {
                56172, 22776, 53536, 12628, 61924, 7892,  57080, 39916, 48696,
                63556, 31152, 9396,  13276, 62304, 32140, 15732, 9952,  30828,
                17996, 42244, 3492,  60692, 54084, 18304, 31728, 22980, 12976,
                384,   62696, 24144, 56440, 60148, 7420,  23260, 40600, 56908,
                62540, 42756, 36176, 25300, 63352, 18700, 27620, 42604, 13952,
                3916,  44812, 59436, 5572,  38572, 43400, 24764, 65152, 41748,
                772,   27564, 5972,  13220, 54004, 39456, 10664, 20876, 29208,
                48352, 3284,  58020, 45760, 18208, 6296,  25676, 47816, 29800,
                62268, 20072, 11208, 26684, 49620, 7916,  53192, 1388,  27348,
                18248, 47524, 43780, 25756, 52516, 16280, 44608, 35376, 21436,
                39996, 10632, 31768, 50968, 3384,  9848,  53464, 39648, 14964,
                47052, 3576,  10332, 59364, 45396, 25236, 32720, 1972,  10500,
                59172, 45956, 37908, 19172, 10628, 43940, 4244,  47392, 27740,
                55660, 42100, 48984, 36392, 52160, 46416, 10104, 43384, 19408,
                380,   33528,
            },
            {
                10340, 41552, 2008,  25580, 32848, 42860, 22136, 11432, 14552,
                43812, 20008, 58196, 41408, 35524, 6788,  25796, 59816, 38420,
                55844, 13788, 32812, 50048, 28888, 2152,  41592, 55972, 47772,
                37872, 29528, 15080, 40912, 35896, 53292, 2816,  49892, 13304,
                4808,  27956, 15888, 8056,  52648, 11908, 57556, 50040, 61556,
                32008, 52156, 19688, 26772, 47872, 3580,  10900, 18664, 31984,
                58940, 22548, 62752, 34916, 18768, 49332, 26268, 7644,  44580,
                13048, 23728, 39084, 32040, 50592, 35076, 65216, 10152, 14672,
                33808, 55860, 6568,  37964, 43920, 32168, 12116, 41196, 64268,
                5856,  34492, 11324, 2164,  64000, 27772, 7300,  61836, 29252,
                48740, 17468, 65180, 25596, 37300, 44496, 22264, 63680, 26928,
                19888, 34068, 51956, 17912, 5428,  50432, 62388, 36872, 55180,
                23464, 6676,  34548, 49036, 57480, 30768, 64564, 12720, 33352,
                5352,  23912, 30596, 62928, 21628, 17180, 61004, 32212, 24168,
                63004, 44816,
            },
            {
                16592, 30200, 59008, 46580, 16248, 49704, 28648, 53716, 59452,
                6452,  51416, 23820, 28840, 3404,  52848, 44260, 50648, 1384,
                20840, 62716, 6632,  44132, 12220, 25100, 58960, 35592, 5464,
                19920, 44560, 8616,  48644, 12052, 26904, 64616, 29812, 37592,
                22304, 51704, 47296, 59128, 39824, 33800, 5460,  20588, 38364,
                9792,  23496, 40552, 63712, 33168, 52656, 61108, 35732, 6800,
                51180, 15072, 44544, 56956, 3776,  32820, 58176, 61528, 51276,
                34448, 63840, 54424, 10712, 840,   57240, 22568, 39688, 2820,
                46660, 41868, 23944, 61644, 15528, 58912, 21404, 48728, 29984,
                16044, 60752, 55964, 21036, 33000, 40992, 12456, 53756, 76,
                57128, 7652,  46232, 5544,  28472, 60648, 12396, 31560, 6500,
                55172, 42612, 65256, 30124, 39180, 21176, 8312,  42664, 48704,
                30276, 13036, 63152, 27828, 2964,  22420, 38604, 54324, 16364,
                40572, 58760, 11744, 7588,  55956, 1496,  40196, 54676, 8180,
                37792, 26924,
            },
            {
                3484,  63668, 20288, 35584, 5112,  64708, 1192,  33996, 25292,
                37276, 2272,  16356, 46472, 60788, 18492, 22772, 11976, 40748,
                27892, 52976, 23584, 16860, 39600, 64792, 7996,  15756, 27300,
                61832, 54844, 59264, 1724,  21452, 43752, 16524, 9144,  45512,
                58024, 18264, 3644,  30352, 22728, 44408, 1440,  48012, 29356,
                16196, 58236, 2444,  14692, 9096,  21812, 28812, 45032, 55680,
                25812, 39976, 9380,  28448, 47080, 23884, 15764, 40908, 5980,
                17852, 4140,  42208, 20944, 28768, 43916, 16780, 51816, 60980,
                27584, 17960, 50980, 80,    28576, 5028,  55088, 36104, 9048,
                24936, 42108, 38232, 14768, 47212, 59764, 19196, 37444, 25136,
                32664, 20692, 39120, 54712, 15084, 52208, 41968, 2648,  49948,
                37996, 13412, 24636, 1216,  12156, 58664, 27272, 14276, 18640,
                3564,  44472, 53876, 16900, 41928, 51612, 7040,  20284, 61708,
                176,   46092, 18872, 39284, 34240, 14936, 28868, 48212, 13072,
                51792, 57132,
            },
            {
                45788, 39212, 50784, 8944,  54828, 38160, 18968, 9900,  41808,
                62840, 31748, 55140, 39816, 8844,  37016, 57768, 33288, 48080,
                8552,  35984, 46200, 58224, 34432, 51776, 21284, 49508, 42380,
                11588, 33936, 24964, 31128, 38864, 54256, 34696, 60868, 88,
                32156, 41360, 64444, 10024, 14744, 61876, 55268, 25356, 65480,
                7220,  44084, 49768, 36812, 54320, 41276, 16672, 1372,  12264,
                20556, 49072, 2548,  36584, 64624, 12020, 1784,  37356, 55540,
                31092, 25584, 49280, 15352, 60436, 37828, 7240,  31620, 58112,
                8440,  12144, 32536, 56688, 46908, 39760, 18800, 62900, 46028,
                54100, 736,   50448, 4920,  23908, 9148,  50816, 5644,  43236,
                63296, 13228, 34220, 58380, 1272,  18604, 35200, 25340, 58916,
                17108, 7684,  48248, 55980, 45772, 31932, 52596, 64244, 35000,
                60116, 39220, 8784,  25064, 60568, 34088, 10248, 48528, 26340,
                35872, 52560, 29380, 49812, 25020, 45304, 63888, 22412, 5084,
                18828, 34028,
            },
            {
                7196,  12004, 27996, 14604, 23108, 44936, 60344, 47712, 15144,
                21212, 50012, 11784, 26492, 63392, 14272, 29932, 5876,  65372,
                15640, 60120, 4396,  29808, 676,   10412, 45240, 31336, 2712,
                52784, 6192,  17832, 50752, 63424, 4052,  20000, 49312, 25656,
                12404, 54668, 35336, 26984, 50804, 36864, 17856, 12528, 41048,
                34700, 55720, 30452, 24716, 4500,  60132, 47304, 57800, 37756,
                63480, 30336, 17696, 54596, 21384, 43224, 52592, 28244, 20364,
                45368, 62780, 8328,  33436, 52620, 12524, 45912, 24844, 20392,
                37104, 44808, 64520, 22168, 34912, 11088, 26232, 3388,  13428,
                22288, 31192, 58868, 28328, 36292, 55416, 30676, 45988, 16144,
                52340, 9508,  42136, 23860, 30128, 47544, 8704,  62872, 44324,
                22000, 33292, 61736, 36212, 16096, 40452, 2788,  6412,  24408,
                51200, 20600, 32268, 1184,  13916, 45380, 28908, 58432, 42948,
                13472, 6172,  64884, 2748,  59256, 10416, 3668,  36072, 42672,
                62188, 24488,
            },
            {
                55016, 61012, 43052, 32300, 57708, 26592, 3784,  30428, 55780,
                7368,  43200, 4548,  34508, 48636, 936,   43464, 55004, 19996,
                26456, 42280, 11800, 55312, 19168, 26080, 57540, 36920, 63608,
                22184, 40300, 46484, 14296, 10244, 28744, 56724, 14888, 39544,
                7112,  20616, 47876, 2068,  43144, 6668,  32468, 53136, 46696,
                21364, 592,   18360, 63036, 11020, 31804, 26564, 7908,  33556,
                52920, 5400,  60584, 40512, 31900, 7816,  47844, 60300, 9672,
                13520, 57372, 36500, 3520,  26960, 64188, 1408,  49640, 4972,
                54144, 40616, 3588,  14848, 7560,  60100, 51412, 44108, 33240,
                61192, 8156,  17404, 44308, 65084, 13844, 1588,  61228, 21860,
                28236, 60292, 4436,  50176, 64260, 21400, 40344, 55420, 11008,
                29556, 53468, 3852,  26736, 9932,  22704, 56680, 46976, 29080,
                11212, 57604, 47608, 65320, 37620, 55692, 17720, 2284,  23468,
                32464, 56224, 22068, 16472, 42420, 53284, 31392, 58168, 50016,
                15724, 29936,
            },
            {
                48296, 21572, 828,   52768, 5816,  40312, 12928, 51152, 35924,
                27520, 59260, 17976, 57152, 20772, 52644, 24912, 39020, 3260,
                50928, 31816, 22132, 47292, 39252, 61344, 5396,  17120, 12952,
                29360, 56544, 35636, 61484, 42164, 5872,  45976, 33020, 59304,
                43868, 61672, 15920, 23592, 56828, 60400, 28688, 3420,  10344,
                59516, 27312, 13572, 45164, 39104, 50540, 19576, 42772, 14808,
                23200, 44344, 10524, 14128, 26216, 57216, 17364, 34200, 24312,
                172,   40216, 47568, 22320, 55020, 18924, 41336, 34760, 62408,
                16400, 25896, 30324, 47948, 54452, 41952, 29164, 16416, 37544,
                52632, 40120, 48564, 10916, 20416, 41672, 26348, 35196, 39568,
                2696,  46796, 17768, 36896, 12068, 6008,  32244, 15604, 376,
                49516, 39788, 19592, 43904, 59560, 50100, 17760, 37384, 43380,
                15508, 6804,  41368, 27004, 21956, 5668,  50972, 39924, 63224,
                9804,  45008, 38476, 27952, 34972, 13928, 19836, 26156, 8532,
                38204, 2552,
            },
            {
                41028, 35040, 16036, 64476, 36792, 18424, 61848, 22532, 324,
                65120, 38280, 24416, 45232, 32288, 8480,  16796, 61064, 45728,
                9360,  37604, 63088, 52740, 14460, 33280, 43552, 54112, 25232,
                48532, 8128,  600,   26280, 21676, 37524, 52376, 23308, 2344,
                28100, 51364, 31064, 38728, 9264,  19892, 40248, 24396, 63532,
                37796, 48868, 32928, 57164, 5808,  22840, 61300, 1956,  48428,
                27812, 58776, 35252, 51056, 1204,  63836, 5860,  38544, 51592,
                61620, 28580, 17060, 43612, 10264, 31280, 57504, 13396, 29332,
                9668,  50652, 59640, 36680, 19112, 23852, 1060,  64756, 6368,
                19940, 27036, 2956,  34116, 53324, 57680, 7640,  49276, 11420,
                54672, 32340, 25488, 56580, 43676, 60844, 52016, 37440, 27428,
                58404, 6724,  14124, 64652, 30168, 7796,  33384, 63076, 256,
                53832, 35368, 18292, 49528, 11864, 30748, 59712, 35112, 15424,
                49964, 19596, 7248,  48076, 62436, 5904,  39548, 46392, 54032,
                12532, 59204,
            },
            {
                19128, 9660,  45960, 24920, 48860, 10672, 44336, 32444, 16412,
                47084, 10124, 13852, 2392,  40612, 64040, 28476, 35280, 13280,
                59092, 17904, 2260,  6284,  28428, 9108,  50128, 1660,  41116,
                59512, 19160, 32572, 53552, 58808, 12176, 17680, 64640, 8660,
                36836, 11696, 4072,  64112, 48912, 13752, 44756, 51652, 15992,
                5096,  41980, 8540,  52956, 16300, 35664, 54792, 12032, 65104,
                38416, 4328,  20652, 47016, 42016, 22764, 29448, 44296, 15308,
                54528, 32656, 4620,  50600, 60144, 6024,  38052, 22988, 46528,
                55636, 20932, 2100,  12632, 61976, 32784, 10384, 56704, 47156,
                12288, 58348, 63080, 22904, 5156,  31856, 15796, 63896, 19184,
                58876, 41104, 8240,  14864, 28980, 3612,  24436, 46476, 17712,
                62024, 35420, 47536, 24616, 2424,  42172, 13080, 21644, 26072,
                59068, 31216, 61932, 3288,  54604, 43392, 8872,  25176, 4644,
                29768, 61000, 52972, 11680, 24124, 56392, 588,   64740, 22268,
                32932, 27044,
            },
            {
                62936, 56308, 31108, 6796,  28856, 59072, 53776, 8096,  42088,
                51692, 29832, 54160, 60004, 49476, 12060, 4904,  47976, 22872,
                30172, 55740, 34772, 41780, 23636, 64076, 20028, 30484, 10884,
                37944, 64904, 15240, 43304, 48284, 3232,  31484, 41136, 54296,
                19376, 57748, 45260, 26296, 33160, 812,   55652, 35184, 30724,
                57756, 20308, 23848, 64212, 28396, 3556,  41132, 30108, 18192,
                51968, 8804,  32544, 16352, 55176, 35988, 10696, 49612, 2688,
                21624, 12108, 63228, 35712, 25284, 15048, 48876, 2828,  41836,
                6956,  33820, 44208, 39560, 5176,  51060, 45328, 35256, 24748,
                30448, 42388, 15332, 36496, 46120, 51344, 24008, 38040, 29804,
                4964,  22600, 53132, 35568, 48328, 20264, 57736, 8592,  43008,
                22160, 11580, 32120, 52136, 55320, 38580, 57960, 51264, 5688,
                44872, 9776,  23272, 36736, 15836, 63604, 20248, 46084, 57752,
                41632, 36536, 1552,  32964, 43444, 50792, 29868, 17184, 44224,
                4444,  50672,
            },
            {
                14356, 3108,  42824, 52068, 20680, 1804,  35408, 23996, 58088,
                20052, 4716,  35016, 26408, 22116, 37292, 57924, 53972, 36,
                43952, 25476, 48672, 15840, 58536, 45464, 35904, 55576, 23008,
                46324, 3816,  29004, 9460,  35264, 25320, 50232, 14204, 46300,
                24664, 34480, 16360, 53704, 21620, 61476, 7584,  18560, 11612,
                50460, 2916,  36992, 43644, 47708, 10032, 58388, 33980, 45636,
                23956, 56368, 62728, 26916, 3616,  61156, 19120, 58364, 40848,
                26756, 46412, 8524,  20440, 42324, 53580, 65416, 27552, 58852,
                16808, 63776, 24476, 57124, 28196, 17160, 21388, 40268, 4780,
                54960, 50300, 272,   28384, 61700, 10612, 43448, 1332,  47632,
                12940, 44464, 60060, 1000,  65456, 40064, 13496, 33604, 2076,
                55868, 40424, 5184,  20684, 16596, 10488, 28040, 46516, 18680,
                39636, 13820, 52812, 46740, 27724, 39156, 33452, 2480,  53696,
                13208, 17084, 26360, 58572, 21060, 14824, 9612,  35348, 60068,
                7836,  37420,
            },
            {
                46896, 22940, 34112, 12312, 63864, 40964, 14236, 27776, 2780,
                39540, 63732, 15596, 42636, 6364,  17588, 31740, 41248, 19512,
                64628, 7088,  11056, 51960, 38748, 3432,  13292, 60912, 16708,
                6652,  51380, 57252, 20480, 60556, 5956,  62756, 28780, 1128,
                59900, 39684, 5072,  10436, 41944, 37676, 29432, 59576, 46408,
                26560, 61908, 31732, 12676, 18728, 52428, 25544, 7128,  14304,
                872,   42396, 11792, 39836, 52804, 13736, 31344, 7768,  64860,
                33752, 55336, 38436, 58116, 960,   30380, 18524, 11164, 36140,
                52460, 30948, 14316, 48736, 7864,  53760, 63928, 14620, 60272,
                9168,  18932, 38744, 56608, 6896,  17988, 55316, 26012, 60892,
                51628, 33348, 16876, 26680, 10308, 30744, 54012, 62664, 25972,
                50684, 28772, 63828, 44380, 36512, 61184, 32760, 2148,  64912,
                34520, 29112, 60040, 1404,  7432,  57004, 11468, 49112, 28752,
                23072, 65124, 46704, 3848,  38152, 61980, 41208, 49172, 24496,
                55212, 29552,
            },
            {
                17816, 61604, 57264, 38808, 17064, 47484, 54808, 62352, 49796,
                33216, 8852,  55916, 46432, 60900, 50708, 27164, 9492,  14516,
                37876, 33008, 61508, 27780, 18960, 8148,  32224, 52476, 27320,
                42184, 33592, 24036, 39704, 44652, 11140, 38140, 56424, 21720,
                7912,  48548, 62356, 28292, 51228, 14636, 23608, 42812, 6072,
                39812, 15552, 55424, 168,   61084, 37832, 20808, 50304, 63400,
                36724, 47916, 30924, 6436,  23120, 44984, 37852, 48332, 24012,
                5296,  17836, 29124, 13656, 47788, 34288, 7296,  40144, 22076,
                45884, 312,   9984,  36956, 41728, 32052, 1444,  27416, 48852,
                32748, 23692, 45180, 13380, 21652, 41084, 31212, 36040, 8636,
                20612, 39260, 5768,  50056, 43348, 18736, 36784, 4384,  47812,
                9556,  14632, 58724, 1052,  25592, 7396,  49600, 15000, 23888,
                55136, 8484,  40860, 21352, 51116, 24480, 18028, 60652, 6596,
                39904, 9476,  55616, 30316, 7372,  54052, 26912, 2332,  19976,
                11716, 40316,
            },
            {
                984,   26672, 9268,  4556,  24740, 31468, 6228,  10892, 17916,
                45168, 25296, 31040, 20960, 11408, 1584,  62660, 45300, 52504,
                57552, 21768, 4652,  43120, 56380, 24708, 48164, 2128,  36976,
                62336, 12512, 47928, 1684,  15900, 53448, 18516, 33428, 42912,
                12704, 31228, 17872, 55380, 3928,  47072, 64168, 1572,  52884,
                22212, 9104,  34608, 46276, 27624, 5656,  59584, 43528, 28804,
                17200, 60528, 20032, 58232, 50228, 27784, 556,   56480, 14896,
                43260, 52372, 3272,  60748, 23136, 44020, 56708, 49692, 4332,
                61176, 27012, 20356, 62712, 58120, 23504, 11840, 44312, 37764,
                3792,  53312, 65280, 34308, 59028, 49016, 3472,  64136, 15380,
                56892, 28612, 62092, 23448, 55596, 6856,  45688, 22712, 17048,
                38092, 31772, 19256, 46932, 53612, 40724, 21484, 56928, 43164,
                3952,  48824, 17304, 32792, 64252, 42680, 36484, 30784, 44880,
                20136, 34436, 50132, 42044, 18984, 12608, 32632, 58720, 43792,
                63688, 51568,
            },
            {
                31580, 54176, 44744, 49880, 59888, 35848, 43636, 22192, 37784,
                61324, 13588, 5108,  52624, 36012, 39752, 23700, 34372, 3336,
                47312, 29840, 13136, 50088, 34304, 65276, 41072, 21248, 58804,
                8900,  18636, 55032, 64460, 32068, 27000, 48924, 3060,  52332,
                65028, 25500, 44552, 35140, 20332, 8840,  32276, 36372, 58212,
                28932, 50084, 65292, 23384, 40772, 13776, 32676, 2388,  10700,
                53116, 25288, 3368,  35860, 16040, 9180,  61512, 20912, 34952,
                10272, 62680, 39748, 50740, 9832,  16072, 62992, 25536, 13344,
                55496, 33696, 51312, 44980, 3656,  18428, 35604, 56564, 61720,
                16768, 10880, 26084, 2056,  29096, 52616, 11892, 23572, 45664,
                42236, 2384,  11752, 34976, 58836, 15028, 29216, 53000, 60896,
                41856, 56624, 5828,  34660, 12552, 30404, 62536, 10096, 36420,
                26632, 62176, 11344, 45420, 4596,  13896, 55100, 1012,  51672,
                62736, 4560,  15576, 24744, 64024, 37124, 47508, 4860,  15760,
                34708, 7096,
            },
            {
                21340, 37196, 14812, 28680, 19788, 444,   56788, 29500, 53268,
                3120,  49116, 40936, 58356, 15336, 28984, 56160, 6704,  17140,
                25788, 40696, 59892, 904,   16756, 10164, 29152, 14220, 44052,
                26052, 4512,  29644, 37228, 7488,  58752, 23560, 9648,  36200,
                15456, 56996, 284,   38972, 61304, 27020, 56316, 17280, 11968,
                19784, 44068, 4324,  17072, 53844, 49248, 56756, 22632, 39972,
                34224, 7844,  55688, 41644, 64284, 32860, 40496, 53432, 30032,
                47056, 25216, 19272, 32652, 27492, 4924,  36904, 31388, 18952,
                42608, 38576, 16032, 6548,  29620, 47604, 51780, 7356,  20984,
                30232, 40052, 48324, 43760, 8500,  17032, 39664, 32972, 4816,
                53364, 19148, 31396, 48796, 40904, 124,   63744, 33892, 11948,
                3076,  26512, 51552, 23340, 59916, 17208, 3140,  46316, 31972,
                19624, 53444, 38412, 29872, 58460, 26532, 9632,  16376, 37932,
                27240, 11988, 58828, 44276, 120,   52664, 22336, 28976, 56208,
                25496, 48748,
            },
            {
                57664, 60876, 41976, 11676, 64720, 51020, 15200, 8256,  58928,
                34084, 28052, 22928, 65336, 8236,  47696, 19852, 43064, 64120,
                10520, 53856, 20344, 37672, 46088, 54464, 5992,  57064, 49648,
                33128, 52676, 45964, 21888, 13448, 41932, 46404, 61140, 29260,
                40520, 6880,  22932, 11092, 50112, 13996, 40728, 48684, 6888,
                31512, 59692, 38764, 10852, 30020, 7304,  18444, 46128, 64676,
                15140, 44656, 29736, 12784, 46816, 18888, 4268,  12184, 59044,
                1664,  37328, 6944,  54800, 42156, 58596, 46868, 53668, 8700,
                2584,  60208, 11368, 24556, 40540, 59368, 15004, 25572, 42652,
                55396, 5804,  60684, 19864, 36668, 63332, 55484, 27436, 59976,
                47700, 37400, 64796, 7688,  21072, 26892, 38760, 8488,  48268,
                20396, 44120, 65192, 9020,  38216, 42892, 27688, 51440, 5272,
                13700, 59132, 340,   22652, 50560, 35300, 47380, 61016, 23232,
                56000, 48280, 33152, 29428, 9844,  17100, 60288, 8184,  41824,
                38712, 10924,
            },
            {
                18380, 4212,  33776, 7380,  46208, 38332, 24380, 41452, 20084,
                11144, 44328, 17568, 3768,  33016, 51624, 12392, 58000, 31168,
                49616, 35780, 7944,  63200, 23936, 31224, 19364, 38792, 1320,
                63796, 15428, 40680, 60016, 576,   56064, 17336, 5660,  20632,
                51144, 47204, 59820, 54240, 33600, 2716,  24272, 44964, 62880,
                37116, 2896,  55076, 25048, 42972, 62488, 35756, 26760, 4248,
                58396, 50020, 21192, 1512,  54248, 24172, 50808, 26992, 43908,
                16856, 50156, 12992, 65016, 21468, 620,   12232, 23576, 64276,
                27992, 48820, 32180, 52652, 63404, 9412,  33768, 2160,  64320,
                12884, 33368, 51576, 14440, 24788, 46368, 6180,  20880, 13640,
                9408,  25772, 16192, 44844, 54608, 13720, 50480, 23940, 59308,
                31012, 35892, 14408, 48884, 648,   54972, 20964, 64336, 37696,
                47796, 25264, 42484, 16700, 6756,  41100, 19736, 32312, 7676,
                40636, 2608,  20624, 54480, 40112, 50292, 34016, 45612, 13932,
                65300, 1868,
            },
            {
                29588, 51996, 22924, 26516, 55736, 2960,  32016, 62960, 46968,
                1396,  61852, 38656, 54856, 25244, 41568, 572,   37428, 22276,
                2768,  14940, 27676, 43820, 3904,  51712, 61956, 35032, 11124,
                24540, 20336, 9176,  34648, 50140, 28536, 38480, 33900, 63208,
                12364, 31912, 17444, 28096, 43048, 64764, 5204,  53604, 21588,
                26776, 13552, 47992, 33848, 58064, 1068,  13096, 52464, 31480,
                9512,  36988, 62008, 27544, 38520, 60444, 6312,  35948, 63704,
                22720, 56652, 29220, 45184, 34064, 17724, 39128, 51768, 34876,
                44152, 21412, 56868, 1008,  19240, 37096, 45860, 28352, 49428,
                38272, 23192, 544,   58312, 32160, 3172,  49752, 41840, 35136,
                56396, 1656,  51540, 29352, 4232,  57772, 42916, 17480, 1740,
                53888, 6624,  18464, 29292, 58216, 33268, 11108, 15944, 57016,
                7952,  34252, 61396, 10536, 56404, 65412, 1876,  53572, 44028,
                14452, 59944, 36640, 6116,  63416, 26996, 2968,  20328, 53248,
                24248, 47176,
            },
            {
                43256, 62016, 16228, 49184, 35236, 18536, 12656, 52756, 27264,
                35424, 51160, 13860, 30084, 46184, 62768, 16232, 27236, 61308,
                46536, 55752, 33540, 58748, 11964, 40228, 15816, 47704, 28204,
                43152, 57520, 5016,  62792, 25204, 10692, 54036, 3740,  44136,
                26308, 1860,  37804, 8404,  20464, 15600, 29756, 34532, 9744,
                50788, 61444, 16260, 20968, 8764,  39580, 47464, 20144, 41700,
                24472, 16508, 5068,  43228, 10252, 31408, 15088, 41168, 8876,
                32372, 3312,  39968, 9388,  49440, 61316, 30244, 4016,  14256,
                6984,  16848, 37884, 26780, 13440, 55756, 22224, 4672,  54260,
                18508, 9684,  43408, 27920, 39100, 11272, 61472, 30120, 18000,
                62636, 38940, 22512, 60848, 34816, 9500,  31712, 61812, 27412,
                46204, 38876, 62596, 41376, 25096, 4696,  45056, 23488, 41000,
                29196, 19004, 52048, 45692, 31196, 25444, 12852, 28516, 63212,
                18164, 26044, 10948, 45224, 23148, 12488, 37948, 61412, 5148,
                32424, 36296,
            },
            {
                6964,  56448, 39892, 9868,  63496, 44476, 58268, 6948,  15868,
                22536, 5760,  56832, 9568,  20208, 5164,  34020, 52900, 8716,
                40584, 4880,  17676, 49164, 21564, 26724, 8144,  60176, 3548,
                54524, 48720, 30904, 39256, 19116, 45468, 15172, 23964, 58436,
                41292, 52284, 61036, 45140, 56540, 49152, 39384, 57804, 17876,
                41880, 1560,  45100, 29040, 51080, 23920, 59400, 3344,  63336,
                55984, 46372, 33488, 57176, 18172, 48780, 58220, 20520, 52580,
                47172, 60764, 26248, 15768, 5988,  24932, 55900, 46376, 62128,
                40592, 59012, 46764, 5168,  50464, 43044, 61952, 16084, 31444,
                59104, 48036, 62708, 53460, 16736, 57108, 22900, 52636, 7124,
                44148, 32456, 11956, 41484, 18256, 47456, 37336, 5548,  12096,
                56476, 22636, 9888,  51256, 13424, 48060, 35520, 54200, 6012,
                63320, 13116, 2620,  36888, 20776, 49080, 39452, 4976,  35072,
                46524, 51168, 31528, 57328, 16000, 48848, 55820, 29336, 17308,
                58488, 11616,
            },
            {
                27600, 14216, 732,   30700, 21500, 4156,  29184, 39344, 48240,
                59972, 43028, 32476, 24228, 49356, 58688, 11600, 44612, 18928,
                24944, 60248, 30308, 37008, 216,   53728, 45584, 32136, 22496,
                36204, 17088, 13616, 52344, 2604,  32924, 65316, 48648, 9328,
                19640, 14188, 5724,  24640, 36028, 744,   12700, 62136, 7120,
                31924, 38276, 57060, 4872,  64924, 32584, 14748, 37608, 29404,
                6780,  12416, 51108, 23028, 65468, 68,    45448, 28904, 4496,
                13940, 19356, 53968, 37004, 58292, 41796, 11100, 19776, 28640,
                24016, 10448, 32740, 65324, 29716, 7752,  34488, 11316, 41600,
                25432, 6304,  35168, 20436, 7880,  45064, 36432, 924,   27296,
                15220, 48624, 3648,  56016, 25200, 64524, 21560, 52612, 41908,
                16176, 33928, 2200,  59200, 19984, 61388, 27064, 1364,  48828,
                32328, 43680, 27540, 54988, 4088,  15460, 58660, 52548, 9728,
                21348, 3496,  61572, 41236, 964,   32952, 43436, 9608,  40272,
                48196, 21752,
            },
            {
                54368, 37520, 46784, 59684, 51336, 42512, 24528, 54704, 9260,
                18280, 38004, 61568, 2732,  36388, 41144, 29304, 64828, 38252,
                50640, 13760, 10256, 64128, 42196, 56940, 18920, 13140, 64748,
                40900, 6516,  26428, 61132, 42548, 55328, 7060,  36868, 30576,
                56044, 34128, 64088, 29232, 10092, 52028, 23408, 46284, 27456,
                53960, 24428, 12468, 35864, 18296, 54120, 10356, 44876, 17788,
                53368, 27180, 40280, 3860,  35740, 12960, 25540, 38292, 63284,
                34312, 42904, 976,   50716, 21808, 31976, 2304,  36588, 53748,
                50200, 1864,  54644, 20260, 40192, 24768, 60204, 52004, 2684,
                37548, 14040, 29612, 3800,  41316, 26020, 13308, 47308, 64468,
                21028, 59552, 51112, 8192,  28480, 1056,  14184, 48712, 29564,
                63552, 25792, 44648, 37040, 31252, 8136,  39896, 18012, 57420,
                21604, 60136, 9204,  40356, 62484, 33624, 23080, 43104, 29948,
                56292, 37728, 8028,  28000, 19340, 63968, 5848,  25588, 51452,
                1620,  64380,
            },
            {
                18004, 24852, 33252, 5368,  17212, 11692, 36128, 64584, 31596,
                64,    27852, 50580, 12844, 17024, 55620, 22376, 1232,  6148,
                32628, 54288, 45128, 23144, 6920,  29324, 38676, 2032,  51052,
                10128, 58280, 47124, 21108, 11728, 29028, 22852, 17640, 2984,
                40016, 49932, 21512, 46928, 58964, 32728, 19156, 42868, 3468,
                63676, 20280, 50044, 7744,  43360, 26488, 456,   49592, 34572,
                61392, 21032, 9008,  30320, 53688, 42268, 55912, 7032,  22448,
                57688, 9628,  27720, 62360, 12576, 47940, 64300, 44760, 8116,
                15856, 35648, 43512, 12224, 16956, 60,    47356, 21156, 44732,
                64684, 56976, 46604, 51044, 63304, 58772, 33012, 54860, 40128,
                10200, 36240, 30884, 16660, 45400, 39656, 58980, 32620, 3064,
                8308,  50096, 54992, 5748,  14924, 53692, 64992, 10636, 37864,
                14560, 50124, 24588, 17764, 46024, 11312, 7156,  60720, 16976,
                48004, 13912, 24332, 53880, 50208, 36436, 22368, 59040, 15112,
                33780, 44436,
            },
            {
                10208, 60636, 53176, 40324, 28188, 56632, 3160,  45908, 14492,
                53256, 21292, 44716, 63728, 26088, 7888,  47112, 15376, 57156,
                27464, 20176, 3332,  35208, 49804, 15060, 59540, 33684, 25132,
                44488, 29896, 34868, 836,   38136, 49828, 59696, 46072, 62140,
                26868, 10952, 2040,  16644, 41340, 4360,  60812, 36816, 15740,
                9596,  46844, 58736, 30008, 61588, 39728, 56556, 23480, 4984,
                42372, 58740, 47268, 15932, 61964, 19092, 32816, 11136, 49064,
                39852, 30936, 44912, 16712, 34900, 4824,  26448, 18260, 59420,
                31256, 63184, 26552, 48556, 57920, 36820, 55920, 27848, 17552,
                9616,  22860, 33596, 16396, 11076, 22088, 2676,  18320, 29136,
                5380,  24156, 42536, 54140, 61912, 34576, 10472, 19692, 57116,
                40516, 17396, 21852, 28004, 42160, 46920, 22936, 29960, 45536,
                6572,  35844, 1756,  56240, 29696, 51236, 37212, 26740, 296,
                40368, 64340, 34052, 44036, 10196, 13476, 46420, 53588, 38184,
                28564, 4032,
            },
            {
                49764, 20584, 7472,  13716, 61900, 49092, 19152, 23328, 34564,
                58180, 10820, 5376,  39432, 33788, 53360, 42716, 62112, 35728,
                48544, 40700, 61580, 57764, 25940, 9184,  46944, 20888, 61840,
                4868,  18148, 56364, 63388, 8460,  15940, 41080, 6040,  13444,
                52752, 44160, 37740, 54724, 26076, 13600, 55992, 48808, 28784,
                33904, 40716, 14168, 2888,  21128, 47800, 16104, 64396, 31176,
                13532, 2592,  37868, 49956, 24196, 5364,  46116, 60560, 15392,
                2408,  54832, 23896, 7132,  59864, 55584, 39052, 13332, 41464,
                3328,  21984, 5564,  60824, 8904,  32968, 13768, 3636,  31172,
                39232, 53836, 816,   27380, 39596, 48584, 8708,  51756, 61100,
                46192, 56748, 1964,  12820, 22148, 5508,  53212, 25120, 46352,
                35708, 11648, 62052, 57496, 56,    34184, 51404, 3268,  25824,
                53376, 63164, 33104, 42124, 5136,  64876, 19276, 54748, 32500,
                5784,  57976, 17932, 2488,  61332, 30804, 3624,  18852, 7840,
                62620, 41444,
            },
            {
                30564, 35724, 46060, 23856, 32684, 9080,  38564, 60348, 6832,
                40640, 47376, 30776, 59440, 3236,  19400, 30036, 11264, 24192,
                8432,  16828, 12292, 31352, 18452, 53452, 3976,  41564, 52196,
                12412, 39708, 23788, 52988, 27860, 33736, 57172, 24876, 31600,
                19304, 58188, 33276, 8420,  63044, 30876, 6464,  22500, 52540,
                1028,  55388, 25664, 37960, 33264, 11428, 6584,  36672, 52276,
                19668, 26452, 33056, 57076, 1308,  29224, 37376, 52368, 26764,
                18532, 65040, 50280, 37612, 20224, 29764, 52248, 24376, 49844,
                56736, 46136, 38524, 29928, 23416, 53308, 40576, 62252, 50312,
                59596, 7604,  43568, 61640, 55988, 31116, 36580, 43196, 15092,
                34232, 19756, 64104, 38340, 27244, 50164, 43292, 15448, 65488,
                29192, 4000,  47912, 38448, 19332, 12920, 58564, 17076, 40464,
                59688, 11836, 15860, 21952, 48364, 25524, 13508, 44368, 50024,
                11540, 28996, 21784, 38452, 55896, 26608, 41972, 60032, 23036,
                12684, 55852,
            },
            {
                15692, 1816,  64920, 41948, 54908, 948,   44176, 29524, 51992,
                25832, 17196, 54896, 23556, 13732, 51092, 38172, 2440,  58892,
                44512, 51464, 784,   39236, 43608, 63740, 36120, 27072, 16388,
                32000, 48484, 6636,  14420, 43172, 20048, 51320, 1244,  36328,
                64768, 4472,  23128, 50908, 18252, 44888, 39308, 11628, 65484,
                19052, 45208, 62152, 6156,  50996, 60296, 28420, 44232, 9188,
                46712, 62932, 11352, 43712, 14772, 64408, 8848,  21056, 43912,
                35596, 12912, 3704,  47168, 11012, 43716, 920,   9044,  33660,
                19352, 11240, 15248, 51300, 42620, 6576,  18084, 45740, 11796,
                24884, 36252, 14280, 19008, 5092,  24468, 65272, 4084,  26376,
                10772, 49416, 7616,  32484, 17776, 59320, 672,   37496, 7208,
                22520, 51964, 32432, 9288,  25072, 63960, 7336,  43500, 31780,
                20128, 46860, 28524, 57464, 9708,  39316, 2708,  61096, 23296,
                41920, 52892, 45732, 6904,  49124, 16488, 35772, 50656, 32596,
                45188, 25628,
            },
            {
                52452, 58472, 5912,  17476, 27108, 21328, 63096, 16008, 12448,
                4008,  65492, 36732, 8280,  44168, 63204, 26468, 54416, 33124,
                21800, 65168, 28280, 55600, 6580,  22620, 11276, 59912, 2612,
                54836, 64968, 35400, 45980, 4040,  11236, 60804, 47440, 9680,
                43024, 14592, 29316, 47144, 152,   35428, 59124, 26948, 42412,
                8772,  31452, 16612, 22664, 54208, 41984, 18732, 58108, 24388,
                55228, 39864, 7008,  22392, 51164, 40752, 55712, 31100, 6520,
                58884, 41428, 28012, 62000, 32232, 16284, 58504, 64700, 28364,
                37200, 62392, 55404, 3924,  20204, 64908, 26600, 35128, 4464,
                20844, 48392, 28764, 51988, 46112, 12648, 20756, 53540, 38800,
                60264, 28900, 52932, 41272, 47344, 9764,  30300, 55028, 60080,
                42336, 14104, 61060, 44700, 29708, 49560, 36136, 55292, 9988,
                4532,  38608, 632,   52896, 34628, 59156, 30528, 16720, 35232,
                8392,  63100, 14500, 33060, 24104, 65496, 9276,  2340,  57388,
                5416,  39048,
            },
            {
                19520, 28412, 37768, 47868, 51280, 10728, 36236, 57196, 49444,
                32844, 42936, 20232, 29048, 49300, 16368, 6996,  41188, 18328,
                4480,  36304, 46848, 14336, 33960, 48972, 30552, 45676, 38220,
                9404,  28568, 21736, 59432, 55148, 30344, 38344, 22128, 27256,
                55300, 39140, 60412, 11148, 53804, 21460, 14892, 5132,  50412,
                36636, 58408, 3892,  39200, 26676, 13740, 1708,  34440, 5772,
                16600, 29692, 60724, 34768, 18140, 25532, 12516, 47720, 2016,
                54244, 33416, 21244, 8092,  53576, 40036, 22676, 48040, 6356,
                44800, 1960,  27356, 32336, 46752, 10648, 58428, 30092, 56348,
                42192, 64216, 2192,  59136, 34044, 41300, 57876, 17068, 92,
                43872, 21480, 14596, 3308,  62988, 23172, 34072, 12088, 20044,
                27332, 1652,  54428, 16604, 40868, 2452,  21804, 27232, 60728,
                51544, 23780, 63884, 42820, 6820,  20368, 45904, 55264, 3964,
                27308, 19772, 58792, 844,   39984, 12504, 46760, 29580, 21044,
                63612, 10872,
            },
            {
                43960, 8188,  33476, 13288, 59564, 31684, 5492,  41296, 23160,
                2172,  58500, 10616, 61404, 716,   35176, 46348, 57624, 13180,
                60512, 25352, 8104,  20300, 62520, 1700,  17820, 57088, 24088,
                42748, 18520, 248,   40236, 25436, 16896, 63860, 3244,  50332,
                17420, 6552,  34332, 25528, 40984, 63344, 48632, 33200, 56508,
                24516, 12544, 46964, 64420, 9424,  49216, 30860, 63440, 50616,
                37344, 484,   53560, 48300, 3144,  59248, 36400, 62592, 17136,
                24644, 46096, 15032, 50572, 25732, 4856,  35064, 13232, 54220,
                16992, 24096, 41416, 61320, 36612, 49896, 1344,  15040, 52524,
                8296,  31788, 17340, 38880, 9956,  6672,  29844, 48432, 33036,
                56504, 5928,  36556, 25736, 57400, 45024, 16788, 48692, 39556,
                50644, 34848, 23804, 5580,  56836, 62160, 13708, 47856, 16236,
                35048, 30412, 12508, 17736, 26372, 50632, 10980, 62476, 39900,
                47884, 37032, 30212, 51064, 57020, 18712, 54648, 42816, 14360,
                34504, 48580,
            },
            {
                24548, 55644, 62904, 22584, 2972,  45248, 54624, 18232, 27696,
                46788, 51720, 38908, 25104, 53484, 31364, 22472, 9820,  29420,
                48216, 42048, 53496, 38376, 58444, 26152, 50904, 12880, 7064,
                60880, 49336, 13764, 52216, 33404, 7732,  44352, 12744, 31916,
                62192, 45228, 19972, 57284, 3912,  9092,  28736, 17168, 2976,
                20872, 52256, 29144, 35024, 43344, 56472, 21356, 40776, 11560,
                45556, 20560, 26968, 10276, 31980, 44748, 7424,  28916, 38968,
                10552, 63988, 360,   38064, 61120, 57092, 19484, 42300, 31156,
                59480, 51088, 12256, 7388,  16536, 22540, 39632, 44504, 24052,
                37412, 12208, 54376, 26472, 22388, 50876, 63588, 25516, 11860,
                46708, 62200, 31380, 53816, 7900,  38680, 2432,  64184, 4932,
                58464, 9004,  47012, 37492, 19800, 33120, 6408,  39004, 44240,
                8048,  54900, 48460, 61224, 32732, 37900, 1144,  24628, 13836,
                53712, 5836,  10828, 43340, 25700, 4516,  37208, 27100, 51760,
                148,   60268,
            },
            {
                30492, 4100,  41852, 16136, 38296, 25164, 8444,  63936, 13884,
                34888, 7416,  17628, 13252, 41660, 4688,  64532, 37224, 51596,
                1536,  31864, 16572, 10732, 44252, 4876,  41060, 32608, 53808,
                36380, 30180, 63056, 10304, 47160, 57940, 35744, 53892, 41504,
                24048, 1300,  49068, 13452, 31744, 44112, 38196, 61632, 45656,
                40484, 60608, 472,   19260, 6360,  15808, 60996, 4048,  23952,
                59060, 14292, 63976, 40284, 55948, 15960, 21900, 53096, 49572,
                19804, 55656, 43416, 30376, 11952, 45708, 2360,  63016, 9372,
                38128, 20824, 34580, 57412, 53812, 30972, 5312,  59196, 18788,
                61208, 4276,  45920, 62512, 42452, 1540,  15812, 37312, 8548,
                23016, 17496, 42312, 12316, 20256, 51268, 28992, 24320, 43312,
                18372, 30592, 63008, 11820, 50120, 28884, 53564, 24848, 64980,
                3012,  21492, 40076, 4352,  15132, 56844, 43128, 20920, 31496,
                59960, 17600, 64236, 21904, 34652, 62420, 7780,  58936, 22784,
                40396, 16984,
            },
            {
                36456, 10428, 47244, 52364, 61384, 29688, 48760, 39624, 59388,
                21296, 30208, 56100, 60064, 47860, 19108, 55196, 14884, 24256,
                62064, 5884,  56120, 23116, 29296, 35460, 61600, 21540, 16288,
                2776,  44616, 24564, 19508, 4408,  27452, 21568, 15788, 7224,
                59244, 37556, 28288, 65032, 51876, 22892, 55120, 11288, 25948,
                7624,  14424, 32264, 54780, 25560, 36876, 47416, 27972, 33616,
                52180, 43080, 7776,  35352, 24404, 4180,  61256, 41960, 2864,
                34388, 27028, 5568,  36060, 17556, 23508, 29384, 49464, 25964,
                4584,  46544, 292,   43992, 26172, 63128, 12728, 47620, 28008,
                32944, 49992, 20340, 14232, 35372, 55816, 44628, 60464, 52536,
                40152, 1840,  49376, 59580, 35292, 15164, 61248, 10976, 32984,
                55828, 14852, 26316, 44572, 58956, 1916,  42084, 11404, 18432,
                58856, 28644, 45856, 9456,  52392, 28032, 63312, 46756, 8632,
                41156, 26840, 49604, 2532,  46312, 15976, 49292, 11384, 32712,
                6096,  53768,
            },
            {
                65112, 20940, 27572, 1628,  19444, 10888, 33136, 824,   53184,
                5264,  36996, 44088, 1988,  33448, 28300, 7144,  40404, 44412,
                19876, 35004, 49432, 64132, 13936, 53024, 9128,  48024, 27548,
                58204, 6496,  38852, 55928, 42528, 60304, 1848,  51024, 29988,
                11072, 55592, 16324, 35528, 5676,  18900, 1632,  34132, 50764,
                57404, 37404, 48508, 63132, 44268, 12268, 53600, 8832,  18044,
                2540,  29912, 57308, 19216, 51012, 46264, 11156, 31664, 14576,
                57984, 9416,  60092, 48600, 53040, 7512,  56216, 34356, 15500,
                52872, 64852, 29100, 18436, 8968,  41260, 34040, 51820, 9864,
                664,   39528, 57912, 28492, 5752,  30956, 19144, 3980,  33976,
                28636, 64784, 26252, 5356,  30608, 44480, 54488, 7020,  37100,
                51508, 336,   41096, 7352,  16752, 22044, 56148, 36044, 51104,
                33328, 13468, 57392, 23396, 34784, 18672, 5616,  35884, 51984,
                3452,  33680, 56320, 13312, 39584, 31608, 20456, 41904, 61368,
                45880, 13336,
            },
            {
                50188, 39288, 59088, 34864, 56248, 42844, 15624, 23756, 44976,
                12160, 64452, 25632, 9752,  22016, 62832, 50336, 11656, 59592,
                27928, 9232,  39332, 3040,  45744, 19372, 752,   37668, 65288,
                11636, 33700, 51560, 14112, 31080, 37160, 48264, 64256, 39616,
                19708, 46872, 8160,  26180, 41448, 47452, 62404, 15516, 43136,
                28600, 17944, 10116, 22888, 4648,  31068, 40168, 58148, 64832,
                49060, 38192, 5740,  12804, 59808, 27840, 36928, 65516, 25228,
                46820, 40032, 22300, 13360, 64328, 41992, 38600, 11444, 60620,
                22756, 40276, 13008, 49076, 60200, 3824,  21260, 15872, 65220,
                25116, 43060, 8140,  52856, 11572, 49192, 24592, 58632, 13608,
                20532, 54768, 10476, 47252, 22592, 1476,  40784, 26796, 20744,
                45716, 65100, 23100, 34948, 60920, 31348, 8788,  47332, 26344,
                740,   43492, 37484, 64228, 2512,  49304, 12484, 58304, 15652,
                22956, 44316, 9312,  28864, 58256, 52928, 1236,  27596, 54820,
                18384, 25784,
            },
            {
                5308,  8740,  44688, 13804, 5996,  50820, 63492, 58128, 28252,
                49748, 40476, 15916, 52692, 46000, 38300, 16812, 72,    32324,
                47224, 57500, 17772, 25916, 31672, 58976, 41492, 24020, 54504,
                18512, 46724, 22416, 63420, 8384,  17664, 12384, 24996, 33864,
                3772,  43724, 52864, 60772, 12520, 56432, 30820, 23976, 6028,
                64004, 1980,  41420, 59704, 50544, 19808, 932,   34872, 14740,
                21728, 26016, 45608, 33332, 41180, 832,   54512, 17484, 6940,
                51748, 29776, 18624, 3476,  32604, 26416, 1556,  20012, 43544,
                6608,  32708, 56884, 36452, 24276, 53416, 38236, 56600, 46148,
                35980, 17548, 61560, 22732, 38304, 64492, 41204, 7964,  51488,
                43204, 36316, 16068, 39096, 57472, 63592, 17824, 49140, 60224,
                13560, 5160,  28392, 53164, 49240, 39160, 4400,  63472, 15912,
                60356, 6532,  19700, 30536, 55472, 41392, 24896, 29908, 38812,
                54700, 19124, 61032, 36880, 5060,  23836, 64512, 9768,  37480,
                3176,  33704,
            },
            {
                61788, 23524, 53488, 32024, 26292, 37576, 20376, 7148,  35768,
                18156, 2872,  60816, 30756, 4456,  57184, 35244, 25724, 53224,
                4260,  12932, 42616, 54752, 7136,  50380, 28440, 14608, 43772,
                3560,  28940, 40972, 1116,  26648, 54108, 44940, 6220,  56900,
                61868, 23052, 32080, 708,   20492, 39188, 9604,  36232, 54204,
                21288, 46800, 34468, 26588, 13880, 55936, 24348, 46356, 42128,
                10744, 55532, 63064, 16344, 23084, 9280,  48416, 20832, 44004,
                1712,  62260, 35444, 56716, 46076, 16552, 51636, 58680, 47504,
                27692, 9808,  16904, 2396,  31132, 43816, 7736,  29704, 2660,
                12376, 54996, 31860, 45104, 1796,  33272, 16632, 46664, 30204,
                2944,  61936, 7392,  32692, 51368, 12824, 4284,  34684, 9220,
                31076, 57712, 38048, 10512, 19132, 13664, 45132, 29740, 41164,
                23716, 54164, 48320, 10956, 16180, 45348, 60140, 7340,  65208,
                47364, 1604,  25644, 50388, 17488, 43088, 46948, 15304, 30764,
                57644, 42712,
            },
            {
                14796, 29792, 17848, 62644, 47060, 2368,  41476, 30656, 9908,
                55284, 22372, 34476, 42872, 13408, 20024, 61768, 42352, 22740,
                65012, 36772, 21456, 61220, 34336, 11380, 62440, 5044,  35576,
                57068, 9640,  59456, 49780, 32612, 60712, 21780, 41200, 29116,
                16748, 13148, 37840, 57792, 28408, 49000, 2868,  44832, 60008,
                12956, 30288, 52868, 8100,  37720, 62576, 7092,  28652, 60500,
                4540,  30968, 2308,  49852, 61328, 30360, 57008, 33856, 38468,
                12228, 54044, 42284, 24800, 10040, 61828, 29668, 6064,  34676,
                53796, 62528, 45428, 50452, 63876, 13628, 18564, 59160, 22408,
                49816, 6528,  27524, 14908, 59236, 5256,  26636, 56028, 12128,
                23388, 57996, 45616, 25328, 20152, 29412, 37980, 55504, 24004,
                42096, 16392, 46876, 2812,  62612, 25884, 58072, 20640, 9780,
                50552, 32104, 39532, 4068,  27316, 34008, 548,   20444, 10556,
                32956, 14468, 40812, 62844, 11768, 34956, 7264,  59904, 51656,
                20140, 48168,
            },
            {
                40548, 56596, 4460,  10056, 22232, 57640, 13316, 52292, 46244,
                59476, 26900, 48464, 8436,  54500, 29068, 10396, 49700, 7412,
                15620, 30236, 51932, 1224,  16664, 38732, 21976, 47488, 31892,
                52000, 16160, 37892, 19916, 5536,  36000, 10336, 52356, 1996,
                47664, 51124, 8812,  42944, 17328, 64620, 52184, 26140, 18696,
                38976, 3296,  57636, 17056, 44724, 31876, 49156, 17516, 51564,
                39240, 53764, 20348, 43476, 37332, 14088, 4476,  26720, 59568,
                23412, 7988,  15108, 48968, 4520,  37264, 40888, 23436, 14228,
                968,   19388, 38860, 22648, 5260,  27276, 48068, 41736, 34192,
                62672, 38904, 19720, 47924, 52360, 21748, 62164, 37732, 50352,
                34392, 17384, 656,   53868, 9636,  42800, 61544, 46048, 1092,
                52380, 33872, 21776, 54588, 42704, 33004, 1132,  55224, 35116,
                6648,  14536, 60968, 22484, 63020, 51944, 37976, 49832, 43316,
                28056, 52556, 6488,  31544, 22048, 56492, 29060, 39080, 24344,
                11132, 488,
            },
            {
                34752, 51292, 37992, 43628, 49824, 34120, 17376, 61692, 396,
                38392, 14372, 5032,  64724, 24680, 47100, 3056,  39016, 33048,
                58920, 45632, 10064, 48408, 24812, 44144, 57776, 8520,  19660,
                64204, 25028, 12692, 43364, 56456, 47248, 15024, 65072, 38836,
                25564, 35028, 62424, 5336,  23712, 34212, 14568, 7048,  32908,
                49500, 10440, 65364, 22312, 40876, 2560,  12436, 36396, 22792,
                8560,  34152, 11916, 27368, 7160,  52484, 40788, 64032, 16852,
                50080, 32492, 29056, 64864, 21020, 54284, 11688, 49252, 60936,
                42432, 25860, 56196, 11492, 35672, 60628, 53936, 9600,  25332,
                3936,  57820, 42828, 9904,  35044, 40708, 8320,  18908, 4132,
                42596, 27504, 39848, 65528, 49352, 5896,  15440, 27220, 19528,
                64348, 7632,  59700, 27796, 6540,  51384, 16564, 38592, 65532,
                46012, 28116, 56500, 42780, 8216,  17940, 56048, 13248, 23980,
                57440, 36160, 59340, 45460, 2052,  49372, 19060, 3632,  44568,
                63808, 27444,
            },
            {
                21796, 60660, 12856, 25952, 64984, 3672,  28504, 24328, 43272,
                20596, 32480, 51356, 40740, 36344, 15324, 60380, 56184, 18348,
                26612, 4952,  40760, 29036, 63716, 3756,  53660, 27260, 40020,
                352,   49196, 29876, 61272, 2536,  23596, 30716, 19028, 7320,
                58328, 20664, 30396, 45576, 55232, 10816, 41388, 61416, 56236,
                43096, 24576, 28912, 51196, 5692,  59164, 26952, 63364, 56168,
                15284, 45196, 65036, 57696, 18752, 24352, 46596, 10768, 2740,
                45412, 58332, 39412, 48,    44668, 33052, 17608, 57036, 36632,
                30572, 4060,  52008, 32532, 16168, 39484, 1820,  31456, 16684,
                51432, 13028, 30240, 44,    54776, 13592, 29268, 45172, 63920,
                10532, 59168, 13340, 30772, 35856, 23256, 56700, 10824, 50500,
                30144, 14064, 39956, 10996, 36468, 23124, 47984, 11592, 25448,
                18744, 1736,  36740, 12284, 47516, 25512, 30464, 5100,  63236,
                2860,  18364, 9240,  15700, 26784, 53940, 61532, 13536, 36048,
                55240, 8212,
            },
            {
                45816, 2628,  19600, 32396, 7760,  39952, 55412, 36112, 6160,
                63176, 10776, 57832, 17756, 1316,  31264, 21700, 43000, 12340,
                37488, 54268, 60188, 19824, 32872, 11876, 36056, 14380, 59740,
                6512,  45508, 35172, 8612,  27408, 41576, 55124, 33460, 43832,
                11516, 53784, 15340, 3376,  37348, 47792, 28088, 21868, 4728,
                16204, 36716, 47092, 14368, 33636, 54224, 18720, 46464, 1252,
                25184, 40124, 29596, 47676, 604,   34340, 55364, 31052, 20484,
                36360, 6352,  18368, 25816, 60184, 8316,  28340, 5252,  21660,
                9884,  65472, 43664, 7400,  57672, 44896, 21152, 64376, 46736,
                36440, 22028, 63260, 26064, 60520, 49412, 23784, 56816, 31800,
                51340, 22332, 47592, 19236, 3352,  60612, 32472, 40932, 37280,
                5124,  44440, 49472, 18632, 63300, 30444, 60028, 5240,  43124,
                58696, 53128, 21140, 32420, 3412,  59048, 35620, 45768, 40388,
                21664, 48608, 42232, 64436, 38384, 33116, 41724, 5408,  48540,
                31520, 16116,
            },
            {
                53436, 29092, 57120, 47648, 15292, 51448, 11292, 45892, 16172,
                49364, 29720, 22692, 44948, 53168, 62692, 6184,  46504, 52020,
                2224,  23376, 14132, 7312,  56400, 50900, 18504, 30104, 42392,
                22952, 56036, 18136, 52936, 63188, 12400, 4004,  60476, 48360,
                84,    40620, 26768, 59176, 19368, 63012, 1168,  31624, 52532,
                59448, 8776,  412,   62368, 20988, 37572, 9924,  41572, 32296,
                6460,  60740, 4336,  16460, 37788, 60128, 8272,  43424, 62948,
                53084, 12328, 56488, 50952, 14000, 41664, 63328, 52456, 45592,
                15116, 48724, 18596, 28544, 23968, 49732, 14052, 5560,  40976,
                56464, 6960,  44624, 33296, 15956, 3844,  36384, 6784,  16492,
                2168,  38316, 7200,  55236, 43668, 16628, 46780, 2416,  62772,
                22576, 58788, 25624, 55792, 3620,  45012, 14956, 51916, 33824,
                29652, 8932,  49344, 40092, 64312, 16532, 53408, 9712,  14912,
                33504, 55488, 30252, 24188, 11536, 7500,  22864, 18016, 58008,
                25248, 40884,
            },
            {
                62036, 6300,  37052, 42080, 23600, 60384, 20184, 58452, 33188,
                54192, 3348,  38740, 8732,  27384, 35012, 10896, 25576, 30436,
                62292, 35296, 49684, 43468, 39300, 1424,  46768, 61664, 51848,
                11060, 32892, 14828, 38644, 21092, 50444, 36276, 17128, 28336,
                22488, 63868, 49636, 32388, 8692,  51284, 13824, 35820, 45876,
                19952, 40564, 31184, 56140, 43840, 6772,  28720, 50408, 58052,
                19980, 49284, 53568, 23012, 11216, 50296, 28848, 14988, 23276,
                27652, 41172, 33420, 22172, 47192, 35792, 2088,  24316, 39756,
                33388, 59772, 37176, 356,   61988, 10460, 30016, 59356, 26844,
                18404, 10848, 53196, 39608, 20104, 47036, 59508, 41024, 54216,
                25776, 33812, 61972, 28508, 9272,  52236, 26196, 13240, 53920,
                34428, 16432, 220,   32568, 38892, 9152,  21476, 40560, 2528,
                17116, 62204, 13864, 44448, 23632, 6792,  28268, 49608, 60192,
                26240, 6272,  51724, 288,   58448, 46856, 52704, 28924, 65440,
                1368,  12124,
            },
            {
                33668, 18124, 10184, 63812, 4908,  30740, 1744,  27048, 8076,
                41424, 25376, 59416, 13648, 56088, 49736, 40960, 19188, 58032,
                8880,  16652, 27808, 65436, 21708, 25708, 8928,  38044, 4288,
                26828, 64596, 1904,  48104, 5960,  25172, 45200, 9084,  53028,
                37736, 6524,  12732, 43616, 24232, 39408, 57632, 25804, 11204,
                64288, 27340, 49260, 12148, 24980, 53112, 15716, 64536, 11708,
                35944, 8060,  31148, 42448, 63800, 19524, 40176, 5788,  48572,
                1196,  59288, 9652,  3760,  30852, 55884, 11040, 19320, 57508,
                3540,  26540, 12996, 41676, 52912, 33968, 38348, 54236, 1200,
                34880, 49056, 24176, 2244,  65200, 9252,  27736, 12476, 20372,
                44376, 57564, 14904, 40160, 23740, 36840, 59756, 19704, 7716,
                28956, 41996, 47836, 12800, 52736, 27876, 56948, 64164, 24392,
                37552, 55332, 26792, 368,   57468, 38816, 20572, 42640, 2136,
                36952, 12936, 44192, 20268, 61764, 34672, 14728, 43796, 8808,
                38900, 49552,
            },
            {
                52132, 44708, 26576, 54068, 35508, 49228, 38984, 44392, 14260,
                64856, 18988, 47432, 32060, 4380,  15936, 64552, 316,   48376,
                38560, 44868, 3088,  31808, 12664, 58644, 33500, 15616, 20160,
                56988, 44752, 40896, 29348, 59272, 32208, 56636, 14144, 61712,
                31024, 18748, 56744, 34812, 4424,  16912, 44516, 6600,  54968,
                2596,  16680, 35036, 4888,  61464, 39700, 2824,  22384, 44792,
                27024, 38988, 13948, 56376, 26168, 2080,  57992, 35956, 61808,
                18064, 44988, 37152, 64660, 17464, 26652, 49460, 62508, 29884,
                51232, 8980,  55588, 20404, 46656, 7080,  16792, 22504, 43684,
                62380, 29408, 14656, 56984, 31384, 43180, 52604, 34796, 62808,
                48816, 10804, 372,   50436, 64604, 4812,  31568, 44636, 57228,
                51004, 10072, 64900, 20212, 60404, 35492, 6864,  48872, 11776,
                46900, 5664,  34664, 51808, 30680, 12064, 61552, 32352, 18072,
                63460, 56860, 39920, 16556, 27360, 38000, 3020,  56020, 32076,
                19752, 23984,
            },
            {
                13612, 59724, 140,   21688, 12680, 17596, 62308, 53088, 22284,
                34328, 956,   61240, 42784, 20848, 36600, 23732, 32852, 13076,
                55528, 20628, 60644, 5708,  48128, 54348, 42952, 62812, 50180,
                35564, 8284,  23456, 54132, 10912, 18936, 42444, 1416,  40132,
                25492, 45888, 2544,  51896, 64760, 29544, 60796, 21488, 37392,
                50536, 43148, 22744, 58376, 18344, 29972, 47476, 34176, 55144,
                17344, 62684, 46632, 3956,  34060, 45312, 52100, 12840, 31796,
                24856, 50376, 14668, 52564, 7332,  43004, 37664, 14224, 6696,
                43788, 22808, 63772, 31368, 3200,  25928, 64800, 50912, 12944,
                4020,  58648, 46152, 7180,  38168, 17844, 23236, 6276,  2900,
                30004, 18100, 32356, 42976, 20924, 12324, 48524, 1328,  39228,
                21968, 36156, 5720,  45724, 24656, 15400, 43040, 18876, 30040,
                61336, 22256, 41612, 19308, 8600,  46236, 54680, 4264,  50588,
                23680, 9872,  48028, 31648, 4948,  50484, 11048, 22344, 47724,
                60572, 4992,
            },
            {
                28144, 38636, 32556, 57692, 46612, 28792, 9224,  3608,  37024,
                50744, 11816, 29140, 7480,  51444, 58620, 44384, 53400, 7052,
                26108, 34352, 50796, 39820, 17412, 29108, 6656,  22788, 804,
                30468, 13728, 17472, 62144, 2948,  34960, 65360, 21532, 7860,
                49328, 59616, 15184, 22868, 41036, 9368,  47560, 32292, 14704,
                28112, 62236, 8588,  45480, 37044, 51632, 9692,  60448, 6052,
                196,   23916, 50824, 10220, 21496, 16060, 29472, 8168,  43660,
                56300, 4752,  28660, 40536, 23096, 60196, 636,   46280, 34688,
                17096, 38536, 48396, 15412, 58036, 40304, 45432, 9208,  32220,
                40872, 20860, 35736, 25828, 50404, 13236, 60944, 47300, 39948,
                25124, 53508, 58984, 6716,  35556, 55724, 27680, 17660, 63048,
                13580, 26888, 57884, 31328, 4236,  50736, 940,   33408, 53864,
                9736,  44932, 59120, 15852, 64952, 35692, 25956, 14760, 44996,
                29476, 34904, 7084,  64548, 54708, 25008, 40988, 57856, 15560,
                36020, 42900,
            },
            {
                55188, 7872,  16292, 50624, 5612,  42500, 59668, 24660, 57032,
                45668, 17268, 54456, 38588, 26584, 10148, 3572,  29388, 41804,
                62912, 15236, 9852,  57576, 24608, 36768, 10936, 45940, 39508,
                60756, 47512, 52084, 38072, 26708, 46984, 51244, 28512, 54916,
                33316, 11016, 37012, 27960, 55460, 19672, 1812,  53348, 41752,
                4604,  55772, 31548, 13260, 1076,  25844, 14836, 42940, 32204,
                40816, 56980, 28464, 59796, 37536, 54552, 65512, 22544, 39488,
                10864, 20304, 62848, 34236, 12296, 54300, 31636, 20816, 58324,
                53472, 27460, 5648,  35828, 11640, 54476, 18676, 36700, 27704,
                55164, 17092, 10316, 63752, 54572, 4712,  28500, 56260, 9796,
                63992, 37476, 22616, 46732, 15316, 61608, 9036,  42588, 33144,
                3276,  53520, 41220, 17216, 37916, 55556, 62724, 40056, 26128,
                14720, 2796,  32256, 4608,  48804, 22456, 6176,  38084, 56272,
                1280,  59524, 21924, 13364, 45572, 18816, 62156, 7596,  29892,
                2556,  63452,
            },
            {
                19224, 45700, 23568, 65108, 39912, 33740, 19496, 13960, 30532,
                5248,  41760, 23156, 62764, 14484, 47688, 60536, 17656, 22244,
                46584, 1612,  30292, 42344, 3628,  64080, 53280, 18812, 55452,
                25620, 6440,  33172, 43540, 4828,  15720, 11848, 38700, 3804,
                17432, 43324, 62448, 6736,  49128, 13092, 34524, 59240, 10516,
                24092, 38980, 20120, 48496, 57208, 63700, 21916, 53320, 19380,
                12408, 35472, 7384,  18448, 41460, 5048,  47520, 2316,  60236,
                35720, 48404, 1460,  57052, 18024, 47328, 4840,  25504, 10680,
                61800, 2364,  42676, 60688, 29760, 23792, 1580,  59612, 5104,
                61456, 49744, 42204, 448,   33356, 44784, 19924, 36004, 16264,
                43968, 13376, 3416,  29200, 50936, 24752, 37808, 52252, 58844,
                23828, 48668, 8244,  61372, 10808, 28432, 22032, 7740,  58624,
                51288, 38264, 56004, 27664, 40156, 53492, 60280, 11488, 42176,
                19424, 52788, 40648, 28836, 36288, 796,   32592, 44664, 53440,
                25568, 10904,
            },
            {
                34540, 51740, 30060, 3044,  11920, 26132, 55204, 48288, 63120,
                10676, 33068, 57652, 6088,  19628, 31296, 34968, 55944, 11192,
                37644, 52480, 60104, 21272, 47180, 14476, 33100, 28256, 4592,
                12740, 57200, 21880, 9896,  58988, 54384, 23876, 45072, 61204,
                57028, 24804, 780,   31792, 39176, 63380, 26300, 46056, 18288,
                65264, 51812, 6188,  34096, 28796, 39464, 7540,  48136, 27392,
                64780, 44532, 49272, 31416, 62456, 13244, 27252, 32912, 17524,
                53108, 25660, 30556, 44360, 9124,  27644, 64096, 41772, 50328,
                32752, 13516, 20180, 49696, 7788,  34384, 52280, 47184, 14496,
                24828, 6740,  22220, 30088, 15356, 40264, 59844, 1944,  31704,
                50008, 7920,  61124, 41788, 32600, 2048,  20004, 5512,  45352,
                15684, 30228, 20424, 44272, 34108, 48224, 13124, 43524, 17808,
                35132, 23352, 63216, 20188, 10028, 30148, 17012, 33156, 62600,
                26836, 15420, 49520, 3716,  60152, 51684, 12236, 38156, 17520,
                49244, 40780,
            },
            {
                1188,  56604, 14716, 37408, 61180, 7516,  44128, 704,   39028,
                27224, 50212, 1792,  36132, 43880, 51156, 2320,  39936, 63956,
                5288,  27040, 18276, 35328, 56664, 7812,  50284, 44048, 37192,
                64916, 41056, 18456, 29240, 63616, 32048, 19724, 35992, 8752,
                29748, 20656, 46520, 53048, 16832, 22640, 3788,  36428, 30064,
                1340,  44008, 15644, 60704, 11036, 44804, 2844,  36148, 59032,
                4372,  23064, 15536, 1508,  52444, 24000, 44340, 50700, 7288,
                41848, 15256, 5948,  60736, 39732, 51888, 35968, 15884, 7404,
                39276, 24872, 55392, 45004, 15844, 63408, 21096, 39268, 30544,
                44492, 37064, 53180, 47888, 57428, 11260, 24624, 52140, 21368,
                55796, 27096, 19040, 57596, 10100, 54736, 65448, 28312, 11400,
                36348, 63656, 1024,  57112, 24792, 3596,  65160, 31440, 1544,
                45928, 6644,  12748, 47656, 884,   44060, 50680, 3396,  46440,
                7456,  37396, 10180, 56772, 16836, 22828, 27060, 64948, 4344,
                21940, 60732,
            },
            {
                9516,  26388, 43588, 20468, 49100, 53592, 16028, 35688, 21596,
                58604, 18396, 46248, 25312, 65244, 8792,  27824, 13904, 24100,
                48576, 32564, 43884, 12168, 536,   26616, 21992, 59784, 16580,
                2028,  51008, 35544, 48336, 244,   40024, 6332,  48840, 14444,
                52036, 41264, 12036, 58100, 8648,  44236, 55880, 50860, 12304,
                58076, 27104, 37748, 54008, 24536, 18668, 55976, 16208, 30836,
                10812, 54796, 39428, 57740, 9480,  36552, 19748, 58528, 11996,
                64232, 54740, 37704, 23644, 12816, 19648, 1880,  56388, 22140,
                47956, 65268, 36608, 996,   28120, 41288, 10716, 2724,  55668,
                65008, 12444, 18856, 62168, 27152, 8040,  45456, 64744, 5892,
                38380, 34272, 45812, 14456, 38912, 47448, 16588, 41528, 49904,
                7408,  54336, 38944, 14192, 51472, 40668, 19944, 54188, 49968,
                60088, 29372, 42304, 54784, 61212, 35960, 23704, 58252, 21012,
                31776, 63996, 24108, 44184, 33948, 42068, 8908,  47672, 58284,
                13892, 31724,
            },
            {
                46936, 64048, 5076,  58712, 32308, 23452, 29084, 64500, 8988,
                52264, 15020, 40352, 11904, 54944, 21520, 42320, 53828, 58320,
                16312, 8712,  61164, 54136, 39244, 63340, 41688, 10156, 31852,
                24600, 45276, 8164,  14200, 53044, 25680, 56620, 62224, 2616,
                33088, 64016, 4440,  35684, 29044, 61104, 14944, 7444,  40252,
                48560, 21740, 8892,  4256,  49916, 32916, 62132, 42292, 51420,
                26040, 46572, 18796, 28700, 33472, 61660, 5452,  38796, 28276,
                22108, 32436, 3184,  47728, 57828, 33432, 45528, 61160, 29660,
                4176,  17692, 9356,  59344, 51664, 32648, 58124, 17888, 26860,
                8756,  33108, 5216,  39088, 2752,  35804, 18112, 28672, 42028,
                12012, 1156,  63384, 22104, 4900,  34996, 23664, 31736, 59392,
                21420, 26148, 32064, 18240, 60376, 9236,  27352, 36516, 15444,
                10400, 25360, 33828, 18544, 14648, 27204, 8864,  40920, 13016,
                55288, 48476, 4800,  30152, 61740, 5908,  55676, 35188, 28776,
                39192, 54072,
            },
            {
                23176, 34852, 17436, 41032, 9996,  2624,  47264, 42228, 4304,
                31508, 61624, 7192,  29416, 59980, 17052, 33896, 6744,  45404,
                36704, 2792,  50692, 23484, 30992, 15148, 48176, 3676,  58140,
                52588, 28560, 62804, 22896, 42980, 10836, 16544, 43924, 27420,
                38820, 18176, 25696, 50004, 21268, 42224, 32332, 24676, 62856,
                34260, 17332, 31004, 64388, 40088, 13392, 22844, 9052,  800,
                37540, 63308, 6892,  42960, 48744, 14556, 55468, 436,   45868,
                49564, 8960,  18576, 62384, 28964, 7828,  25140, 10988, 53944,
                43604, 32196, 26480, 12900, 22064, 6044,  47944, 43212, 35504,
                50456, 59192, 24040, 42928, 54864, 50856, 60956, 14684, 48292,
                58948, 25368, 49716, 30100, 53296, 61916, 136,   55776, 13576,
                3204,  43176, 47644, 4568,  34944, 45604, 6164,  57248, 22364,
                39768, 64148, 2760,  51148, 38772, 4676,  65304, 52092, 28296,
                232,   39628, 18860, 11504, 49208, 20796, 15492, 1768,  45356,
                18940, 6916,
            },
            {
                51324, 27988, 13012, 50468, 36712, 57160, 18780, 13300, 56112,
                22700, 35380, 48420, 3340,  37352, 49996, 200,   30504, 61780,
                20620, 28820, 41008, 17908, 4700,  36572, 55036, 20192, 34928,
                12420, 17948, 5268,  38104, 58368, 30136, 34820, 20908, 55256,
                9824,  47228, 60240, 13696, 5640,  37760, 132,   19552, 54824,
                5116,  42644, 58616, 45684, 2044,  28148, 47240, 59732, 20276,
                33952, 13724, 58568, 2692,  21424, 25864, 41512, 30988, 16548,
                59920, 35148, 43440, 52744, 15540, 42180, 50636, 35560, 14340,
                40212, 57356, 50400, 44660, 38624, 61748, 24400, 14164, 4104,
                20524, 46400, 15948, 29820, 13052, 21896, 33756, 4580,  31272,
                54184, 17576, 7672,  41112, 12660, 26988, 9572,  45624, 37304,
                61452, 51972, 10756, 56200, 29592, 12848, 62504, 42104, 4376,
                48688, 55568, 8204,  31288, 57868, 47360, 19868, 34004, 16504,
                45176, 35532, 59100, 54180, 27716, 40232, 52012, 25060, 62720,
                11756, 42440,
            },
            {
                61288, 55812, 344,   63000, 30112, 25464, 60680, 38720, 49664,
                26800, 43864, 19460, 57292, 23688, 45992, 12980, 52240, 25384,
                10216, 55856, 64988, 46040, 11172, 27868, 62348, 7464,  46620,
                39468, 60860, 54884, 32800, 2436,  47760, 60112, 4200,  50924,
                23316, 1140,  31356, 57236, 45272, 64712, 51940, 46640, 28356,
                11068, 51216, 14396, 25400, 55272, 35304, 6108,  50440, 40800,
                53988, 24184, 31668, 51728, 10176, 64880, 53260, 8012,  24336,
                56332, 11508, 26528, 1412,  38472, 5444,  21428, 63824, 628,
                19200, 7308,  62944, 3688,  16512, 54440, 30752, 52220, 63848,
                40260, 56696, 2108,  62560, 48812, 8800,  57900, 40332, 23604,
                10268, 37136, 44400, 60508, 18716, 51140, 39744, 20164, 29516,
                16772, 23580, 40436, 64296, 20752, 49468, 24164, 31028, 17540,
                34692, 13224, 20992, 43932, 24448, 11284, 41828, 56384, 6380,
                61296, 25156, 14504, 2476,  37244, 64264, 9484,  32680, 57660,
                36808, 4432,
            },
            {
                15728, 39056, 20228, 43480, 7452,  45712, 4804,  33480, 10540,
                868,   64140, 14096, 32440, 9552,  41040, 63456, 18392, 39172,
                43824, 14428, 5780,  33988, 58048, 51976, 24028, 42004, 30096,
                944,   26192, 44284, 14800, 20020, 8608,  39784, 13032, 65096,
                36192, 42864, 53656, 17532, 10708, 26744, 8220,  16164, 59520,
                36248, 2892,  38768, 20792, 9756,  17752, 62676, 30024, 11176,
                3724,  44524, 17036, 38540, 45552, 34264, 18684, 36972, 61944,
                3932,  39876, 21080, 65136, 55632, 32132, 59428, 28308, 45840,
                24424, 36508, 29460, 20768, 34276, 9964,  184,   37452, 11632,
                28264, 7476,  32408, 37588, 26320, 44244, 392,   19244, 46424,
                63784, 3996,  32548, 56408, 35628, 5984,  48000, 64704, 5020,
                34248, 8176,  27964, 720,   16148, 38120, 53004, 2116,  59604,
                46448, 28696, 61648, 37108, 15988, 62964, 2572,  30628, 22508,
                10360, 50756, 32580, 45900, 23224, 5196,  43784, 17584, 48160,
                22196, 30912,
            },
            {
                46132, 33308, 24584, 11104, 54016, 15212, 21772, 52088, 17380,
                55200, 40116, 6508,  52860, 60604, 27176, 4224,  35324, 58840,
                1464,  31784, 49360, 21024, 38828, 1720,  16420, 13084, 57460,
                50620, 10408, 22588, 49320, 63568, 52760, 25148, 29568, 45776,
                15804, 7556,  28424, 39576, 34616, 22560, 55548, 33164, 43632,
                23628, 62276, 29916, 52488, 59232, 43168, 37776, 14960, 57544,
                27052, 64056, 7656,  60336, 28404, 13024, 1764,  48316, 43848,
                14804, 51908, 30164, 45020, 12876, 18052, 48256, 9536,  52640,
                58412, 11684, 48900, 55892, 42520, 46552, 27120, 60884, 17992,
                44096, 22316, 51716, 10660, 17624, 65308, 35668, 55492, 27532,
                50896, 13924, 21712, 2216,  28952, 15436, 24308, 56712, 42888,
                58540, 46716, 54116, 36216, 44860, 58068, 8504,  43584, 11112,
                25892, 53952, 476,   51880, 7028,  27484, 49652, 37880, 53632,
                42788, 63660, 8064,  18096, 57084, 12868, 60552, 28928, 1152,
                54872, 8656,
            },
            {
                58264, 3544,  49384, 59292, 35412, 65348, 41844, 29572, 61784,
                36532, 24776, 29024, 44420, 16064, 21376, 55732, 8464,  47352,
                23364, 53700, 26668, 60024, 8604,  43264, 48772, 33312, 21200,
                65260, 34588, 6700,  37060, 28072, 42488, 18040, 5608,  32976,
                55788, 20448, 62548, 2936,  49628, 60976, 40764, 1852,  13160,
                48920, 18528, 6988,  46508, 33040, 1164,  23808, 46124, 19604,
                48856, 36412, 22084, 55728, 4912,  49920, 58860, 21648, 27576,
                33380, 5852,  47932, 7924,  36368, 25876, 4080,  38036, 15736,
                32672, 41520, 2772,  14524, 64940, 23040, 57044, 5456,  49028,
                34720, 55544, 60328, 41532, 5012,  53012, 30184, 11724, 7188,
                42256, 59208, 26008, 46864, 63104, 54592, 11224, 31832, 1480,
                14036, 19284, 61804, 11888, 4936,  26856, 32768, 19452, 63548,
                36244, 5776,  41016, 19076, 45416, 34584, 59744, 12592, 17292,
                1256,  28400, 40492, 48048, 30644, 51208, 35404, 41588, 14580,
                50216, 27032,
            },
            {
                62464, 40624, 17452, 30624, 1600,  26288, 48200, 8516,  3392,
                45896, 12120, 49456, 1520,  38212, 33572, 50396, 29864, 15364,
                64020, 37500, 12332, 18188, 30424, 63116, 25484, 53676, 4536,
                17636, 41052, 54344, 59084, 11936, 512,   57232, 61992, 48892,
                40844, 11620, 25032, 44676, 14628, 4768,  20196, 30508, 64200,
                9364,  39872, 56864, 26392, 12604, 65424, 5488,  55052, 8768,
                31232, 32,    42848, 15568, 24720, 39980, 30720, 8936,  55208,
                63280, 19176, 59856, 23656, 57700, 53628, 42764, 62296, 22348,
                5636,  60484, 25668, 38528, 7848,  18916, 31848, 40044, 15100,
                25224, 1220,  13556, 20316, 48016, 24212, 15372, 61280, 38024,
                33304, 17040, 53104, 39588, 8224,  42060, 36844, 21280, 52396,
                38916, 25348, 30364, 50812, 22024, 60700, 14108, 40416, 50240,
                22248, 15564, 59024, 31948, 56056, 9000,  20852, 47132, 25396,
                56804, 36776, 20548, 60124, 3128,  25728, 7764,  20856, 65060,
                37600, 11608,
            },
            {
                43892, 6688,  22880, 52832, 38532, 12828, 19476, 58424, 32756,
                22272, 59948, 18656, 56536, 63880, 6080,  12040, 42760, 19884,
                3068,  41128, 6480,  45540, 56032, 3280,  36656, 9540,  59504,
                45828, 26816, 3488,  16220, 31048, 46960, 34992, 10052, 23208,
                1952,  58160, 51172, 31472, 36324, 57168, 46288, 51032, 24796,
                54964, 34148, 3552,  16108, 41524, 51000, 28948, 34756, 40580,
                62012, 51520, 11564, 33324, 53356, 62332, 16920, 42692, 11880,
                37820, 568,   41012, 16272, 10780, 33568, 1284,  29368, 50036,
                44728, 17796, 53516, 28708, 49864, 2120,  52820, 59792, 8916,
                45364, 63748, 28548, 38672, 33888, 58272, 43288, 3736,  20748,
                49256, 916,   10544, 30612, 19264, 4356,  50136, 28024, 63564,
                9400,  48788, 6456,  41704, 34728, 54628, 45932, 3092,  56796,
                30908, 9732,  47852, 24612, 13784, 39396, 3516,  64352, 43736,
                33400, 4624,  11604, 52948, 15512, 39164, 54548, 45084, 3972,
                24148, 18808,
            },
            {
                28748, 56416, 9704,  63248, 44892, 5716,  55904, 40588, 14264,
                51784, 7036,  41964, 30692, 23760, 46016, 53332, 26356, 57340,
                32272, 60564, 51096, 35008, 22656, 14984, 47684, 29188, 39148,
                12716, 32152, 48472, 62984, 23992, 39352, 19620, 53344, 15368,
                38032, 29272, 6340,  64600, 17972, 9828,  27916, 6588,  37296,
                44772, 21068, 59860, 31268, 47768, 21816, 17352, 60096, 13460,
                25388, 18356, 58456, 46444, 8376,  35764, 2464,  47136, 23892,
                52680, 26712, 50116, 30408, 46080, 63636, 19904, 13184, 56528,
                8920,  35608, 62784, 10624, 33992, 43576, 12540, 36116, 21216,
                51360, 31340, 7004,  50244, 2632,  9420,  46336, 27092, 54892,
                64504, 24324, 36180, 61728, 58104, 44788, 16016, 60052, 3048,
                43764, 32520, 15480, 64936, 1940,  18048, 29120, 7564,  24960,
                38388, 64812, 42648, 1716,  61772, 26804, 52200, 30156, 6536,
                14832, 61668, 45212, 24116, 34496, 63428, 10788, 31844, 57528,
                51900, 34012,
            },
            {
                13808, 36652, 48044, 16268, 33884, 29016, 49972, 24260, 35908,
                64568, 27700, 10136, 35108, 14732, 2280,  62072, 36080, 9620,
                48516, 16888, 27628, 10548, 61884, 41680, 19396, 64152, 23304,
                7440,  57056, 20760, 42876, 8400,  51428, 4640,  44040, 26520,
                61112, 47408, 13192, 42216, 22736, 53120, 60592, 13812, 16968,
                480,   52876, 11812, 63540, 8336,  38652, 53672, 2728,  45060,
                6444,  38200, 28880, 3460,  20496, 27288, 56692, 64448, 6928,
                34560, 13772, 61524, 4792,  38632, 7468,  51660, 40720, 24736,
                31540, 1728,  21104, 46380, 56284, 23776, 61872, 27016, 4576,
                57220, 42496, 18412, 54468, 63052, 22628, 32200, 40188, 13120,
                5412,  42600, 47276, 13624, 22216, 33652, 25760, 12308, 35100,
                18884, 59144, 22968, 56572, 10968, 39368, 48400, 62564, 12712,
                52024, 5040,  20252, 54232, 33692, 16964, 41620, 58172, 23048,
                50584, 55060, 27944, 41356, 896,   48800, 18352, 27144, 6336,
                47008, 560,
            },
            {
                54048, 61216, 25968, 2576,  21392, 59804, 10312, 8,     17000,
                44196, 2952,  48096, 54612, 58996, 17984, 39076, 21908, 4368,
                44080, 24356, 1304,  54556, 31576, 6076,  53160, 12,    44228,
                50696, 35696, 1776,  53980, 14600, 28816, 64928, 57616, 32128,
                8276,  21112, 54308, 34136, 5144,  38948, 43296, 32764, 63160,
                29344, 41800, 26032, 36168, 3988,  57512, 10972, 27884, 49848,
                22604, 55436, 64944, 41912, 50420, 13212, 38868, 18616, 31552,
                59000, 21172, 44016, 54520, 22752, 26980, 58044, 16124, 36884,
                48680, 59360, 40480, 15168, 4064,  30812, 17172, 48148, 38336,
                15664, 10468, 35468, 26352, 12440, 16972, 59800, 52432, 18784,
                30888, 56876, 27640, 6252,  54328, 1376,  46532, 55640, 39676,
                52904, 4736,  45252, 36756, 26120, 53060, 21600, 34000, 43976,
                17704, 35832, 29436, 11004, 49680, 7636,  36068, 11868, 19384,
                38584, 9440,  17400, 6692,  58640, 22432, 61424, 42732, 16056,
                59876, 40380,
            },
            {
                19220, 5244,  31756, 42932, 52408, 39280, 46576, 61612, 31164,
                56840, 38028, 20380, 25752, 40644, 8208,  31300, 49908, 14288,
                65420, 39960, 59268, 37028, 12888, 48908, 25652, 33664, 14352,
                60516, 18152, 25992, 61816, 37816, 33852, 12140, 2372,  40444,
                18092, 36796, 680,   58820, 26328, 48660, 2640,  19768, 57376,
                46912, 6976,  50204, 18408, 45532, 24224, 33712, 42436, 62940,
                36152, 15008, 10188, 32508, 60828, 23380, 45384, 4552,  49380,
                41764, 9972,  2060,  17592, 35268, 11200, 46884, 3880,  65116,
                6980,  12180, 27728, 51524, 63980, 7524,  41812, 54000, 1676,
                65132, 23612, 60228, 44292, 39424, 47564, 7628,  2352,  37180,
                60744, 8992,  16276, 51832, 38228, 29856, 62980, 6748,  23640,
                28496, 13528, 50704, 30176, 8640,  61516, 15752, 1472,  59152,
                26644, 55700, 46260, 60596, 21680, 44904, 63292, 0,     48208,
                31888, 65520, 35680, 46292, 29624, 8964,  33804, 37752, 12204,
                29996, 22672,
            },
            {
                45044, 50720, 11360, 64872, 14332, 7568,  19764, 26548, 52708,
                13296, 5440,  62740, 11824, 51844, 44452, 60168, 28060, 53100,
                33924, 6968,  21076, 45752, 18336, 57936, 38432, 10000, 54880,
                29888, 40104, 11024, 5516,  47428, 21684, 49576, 24684, 55708,
                46396, 62868, 50272, 16132, 30636, 12472, 55156, 24672, 10240,
                37912, 22308, 13204, 55508, 30704, 61824, 14180, 19324, 900,
                30432, 46032, 5180,  17424, 53032, 688,   30028, 54584, 15904,
                27752, 37504, 47396, 64124, 32020, 59928, 41876, 29148, 20608,
                52776, 45152, 23392, 38056, 19412, 35308, 58900, 11476, 29228,
                33180, 49200, 6020,  260,   30280, 57296, 34780, 23840, 50516,
                45144, 21256, 34368, 62172, 41744, 11328, 17564, 48656, 9524,
                61052, 42476, 4,     20056, 46880, 5192,  42072, 30960, 49220,
                8828,  4036,  14948, 39044, 2924,  28372, 56040, 24200, 43308,
                5672,  25604, 50028, 56760, 14640, 52336, 2468,  55192, 49596,
                64356, 7928,
            },
            {
                34544, 56932, 23800, 37800, 28220, 55068, 35104, 4300,  41792,
                22780, 49512, 29284, 34056, 6212,  23092, 1704,  12164, 18660,
                56496, 10224, 30588, 52316, 3004,  28380, 62652, 21944, 42584,
                2644,  49168, 58260, 27856, 17176, 42032, 60020, 6136,  14464,
                28552, 10372, 23832, 44100, 7968,  63644, 35484, 51240, 3796,
                61928, 33836, 59736, 1624,  40712, 6256,  51852, 59188, 9248,
                53780, 21144, 48880, 26620, 40224, 35932, 11720, 57864, 7712,
                61940, 24180, 56420, 6268,  50836, 14300, 96,    55648, 34320,
                17508, 31036, 56660, 848,   10076, 44680, 25408, 21576, 46436,
                40460, 19472, 56228, 51840, 21132, 14784, 64636, 28584, 11576,
                41120, 1044,  48384, 24060, 3748,  20112, 43580, 35976, 56676,
                32176, 16500, 63984, 33764, 58872, 38224, 55932, 11896, 23180,
                40524, 62656, 32144, 25168, 51936, 13372, 32896, 16152, 52968,
                60656, 12892, 1748,  19900, 38688, 43996, 24808, 20240, 4428,
                26324, 39340,
            },
            {
                16320, 1276,  47492, 17900, 3256,  58652, 45284, 10840, 64092,
                32448, 18120, 43592, 58524, 16308, 47740, 63632, 37472, 42212,
                46636, 26160, 63172, 15328, 43396, 5504,  46904, 16520, 7592,
                63944, 23152, 32696, 45192, 55480, 8856,  30920, 52204, 35644,
                42920, 4348,  33320, 60212, 39364, 21188, 45204, 15152, 41308,
                28376, 52500, 16476, 26580, 48244, 21396, 36488, 44824, 25712,
                39116, 56992, 34516, 59644, 8264,  63652, 42720, 21996, 33364,
                51764, 12672, 19292, 29496, 39448, 25380, 22052, 43948, 10568,
                61968, 5388,  41556, 60708, 32624, 54852, 50344, 5520,  14076,
                62288, 8536,  27604, 37432, 10328, 43372, 3464,  53824, 17388,
                63084, 27168, 14020, 55564, 32452, 50800, 65232, 25428, 2852,
                21612, 38884, 54212, 11300, 24136, 17536, 27800, 65344, 36340,
                51372, 19548, 58060, 42896, 6764,  37036, 59924, 8976,  39196,
                29000, 21844, 42264, 31536, 59332, 10876, 63524, 47316, 31940,
                57840, 10420,
            },
            {
                62920, 30548, 42052, 61020, 33176, 21704, 49912, 15864, 38496,
                59636, 8652,  788,   55280, 36312, 26944, 53652, 32504, 5888,
                22440, 1032,  35640, 40256, 23424, 60916, 31868, 35316, 51836,
                19532, 37324, 4720,  15012, 63316, 408,   39248, 17784, 63964,
                21444, 57384, 53124, 18588, 27844, 1592,  56644, 31824, 5864,
                19072, 44428, 7932,  38464, 65176, 10844, 29308, 16120, 4776,
                64172, 12452, 2036,  44140, 14572, 19584, 25848, 46608, 1548,
                36708, 41432, 2996,  45068, 9436,  54076, 63368, 7360,  38840,
                49788, 26024, 13416, 47372, 16128, 28068, 2764,  59968, 36444,
                53620, 17264, 44944, 32876, 61860, 25840, 48716, 38360, 31752,
                6168,  58164, 36356, 8152,  60072, 28156, 5944,  13388, 52072,
                46252, 7344,  26976, 49180, 2988,  52528, 45444, 6428,  14324,
                596,   47108, 10320, 16816, 64116, 48124, 20564, 45148, 4208,
                34592, 55324, 62996, 3876,  17192, 27764, 7284,  36096, 14032,
                43188, 51680,
            },
            {
                27424, 7348,  53384, 13128, 9196,  40892, 30132, 25920, 2472,
                53240, 46688, 24032, 39992, 10620, 20244, 3228,  14928, 61704,
                51316, 58092, 48536, 11928, 53896, 9120,  56516, 13168, 26688,
                57592, 44032, 11856, 52824, 35160, 20604, 26884, 10668, 48612,
                2144,  38244, 13712, 6504,  50948, 47468, 11028, 26028, 64220,
                48732, 58144, 12248, 32516, 54392, 2672,  58520, 51048, 41956,
                32056, 18080, 24452, 29796, 54228, 49868, 5480,  59296, 16688,
                64528, 30688, 50308, 60992, 16240, 35536, 48700, 28212, 15228,
                58420, 36084, 3152,  21204, 64848, 39908, 18708, 42852, 30604,
                24836, 1892,  12916, 57812, 4392,  18324, 55344, 9056,  22464,
                46564, 52212, 18864, 40544, 45040, 16052, 38372, 58584, 30340,
                41312, 60392, 14696, 35020, 42568, 9228,  32588, 21332, 54092,
                26332, 34348, 29520, 23056, 54272, 2196,  27160, 57536, 18104,
                51040, 10192, 46004, 36948, 49676, 53504, 41364, 61068, 18480,
                3124,  22328,
            },
            {
                59624, 19048, 36064, 25372, 51076, 63664, 6084,  56672, 19500,
                35312, 28224, 14412, 65000, 31500, 59848, 45344, 40980, 29088,
                17716, 7820,  31204, 20216, 27616, 37596, 852,   41540, 48448,
                3248,  30256, 60260, 24216, 49840, 41440, 60784, 55000, 32616,
                25648, 46020, 30196, 65248, 41900, 15972, 59824, 38288, 21980,
                34932, 332,   24244, 42592, 17800, 46824, 34424, 22352, 8084,
                48172, 52472, 38396, 60940, 3948,  32688, 39012, 28532, 53208,
                10068, 21408, 26096, 58004, 4252,  19992, 31884, 1948,  45424,
                22960, 53636, 30312, 51252, 8324,  35084, 12080, 56588, 7852,
                48488, 63376, 38920, 47216, 23292, 42112, 35340, 61236, 13400,
                42984, 2980,  30056, 11116, 22996, 528,   47748, 34300, 10212,
                18316, 1492,  56256, 20592, 63108, 29280, 58300, 37708, 61920,
                43520, 56880, 4508,  40148, 35364, 12624, 31464, 41596, 64788,
                14688, 26212, 6900,  23368, 13620, 33344, 584,   24152, 55672,
                46188, 38180,
            },
            {
                49040, 3744,  44644, 56124, 768,   37144, 13660, 45924, 60948,
                11280, 50964, 42552, 3900,  49988, 7172,  23244, 57260, 9948,
                38248, 44712, 64480, 4732,  59404, 50248, 18076, 23972, 65196,
                10036, 39416, 18608, 7532,  28896, 3408,  13356, 44576, 6876,
                16456, 61172, 9448,  36228, 24292, 34124, 3500,  52880, 8680,
                14508, 55432, 30052, 60888, 5080,  26144, 13692, 62796, 28260,
                3640,  57852, 10560, 21788, 46172, 9300,  56616, 13352, 43508,
                6808,  47152, 34092, 13068, 42684, 38404, 56512, 11972, 62524,
                18068, 6848,  42752, 57724, 24652, 45620, 61548, 22460, 51884,
                33620, 20148, 28272, 52576, 6392,  15288, 29032, 808,   58708,
                33632, 25024, 64516, 49584, 54952, 62568, 20516, 53544, 24492,
                64176, 28592, 37120, 47368, 4152,  15784, 49980, 2484,  17812,
                12360, 49024, 8448,  61628, 45780, 51408, 6492,  22684, 728,
                48492, 40048, 30660, 61356, 56628, 20804, 48272, 29632, 8644,
                33860, 11544,
            },
            {
                31208, 65204, 15792, 29340, 20736, 48180, 24204, 41864, 31400,
                4704,  21640, 57580, 17748, 37660, 13320, 47904, 33972, 25768,
                55512, 13644, 34568, 42020, 15556, 45436, 30940, 53404, 34104,
                15260, 47256, 54560, 33536, 43696, 58152, 36568, 23264, 51480,
                39988, 55500, 22008, 608,   58340, 19564, 29700, 43156, 62232,
                45936, 39388, 49888, 20544, 52148, 37048, 40628, 54660, 18880,
                36376, 42700, 15496, 34244, 65368, 17288, 23060, 35616, 62860,
                18548, 39696, 54984, 5756,  23444, 65228, 51620, 25904, 40792,
                34032, 60232, 11304, 38352, 15524, 5680,  29248, 1100,  14976,
                40756, 3728,  10688, 59372, 32012, 65396, 50568, 40468, 19820,
                53664, 15704, 38640, 5088,  26908, 32164, 8620,  42656, 4572,
                50488, 44884, 11660, 23516, 54640, 39620, 10752, 25424, 41400,
                31020, 22060, 59300, 19016, 25744, 15928, 56060, 36912, 60616,
                33760, 19232, 52320, 43244, 4680,  11084, 64680, 39856, 58552,
                15296, 54388,
            },
            {
                5188,  23624, 40500, 10584, 60716, 32996, 17392, 55040, 8368,
                39400, 62264, 32864, 26204, 55116, 29956, 62732, 420,   19100,
                6092,  50932, 22100, 2636,  25456, 62196, 8120,  4124,  20660,
                60800, 26100, 1468,  63572, 16252, 9296,  19588, 64456, 3992,
                28704, 12192, 49188, 44124, 54128, 12584, 50152, 5684,  17896,
                27568, 10720, 6260,  15584, 63520, 9676,  1400,  45784, 11548,
                24916, 61756, 28,    50864, 27056, 40692, 2404,  48512, 25308,
                32320, 492,   61364, 49268, 29940, 16892, 3584,  8780,  47584,
                29212, 764,   20124, 49288, 32488, 54912, 47576, 37268, 64412,
                44768, 56276, 16872, 36616, 21232, 45824, 11760, 26440, 6912,
                47616, 9352,  57244, 43984, 13112, 36800, 17260, 56856, 39492,
                14320, 31644, 7692,  61252, 33376, 19088, 45792, 64588, 6032,
                52828, 36980, 1296,  32948, 10140, 63624, 42968, 28044, 8260,
                11668, 58636, 2804,  15876, 37952, 26824, 44956, 17740, 2004,
                25692, 42272,
            },
            {
                19676, 47548, 34988, 57532, 7164,  52668, 3304,  63136, 27768,
                15680, 47968, 1208,  44908, 9336,  21416, 51824, 42728, 60056,
                46600, 28248, 58780, 54816, 39652, 12252, 36036, 57192, 40804,
                44348, 6640,  37288, 21964, 48864, 52728, 30696, 46644, 34736,
                57648, 18724, 32228, 4988,  38552, 61448, 25268, 33412, 57224,
                22596, 35888, 59112, 32736, 44720, 22124, 29728, 60452, 33284,
                56136, 47556, 30848, 6448,  55096, 44780, 60048, 5968,  52288,
                57588, 15676, 27684, 44076, 10232, 36208, 45096, 21616, 55612,
                14700, 52188, 64496, 27336, 3440,  58824, 19180, 26104, 8232,
                31044, 49588, 25720, 42544, 8720,  3216,  56656, 37860, 63348,
                31316, 22152, 34840, 18556, 61564, 47236, 2268,  60532, 27376,
                21180, 58200, 52852, 42040, 5332,  28128, 59108, 34764, 55696,
                15432, 27132, 44092, 50196, 39296, 3052,  20528, 46952, 54448,
                24460, 44856, 29312, 62888, 34624, 53772, 5600,  50320, 36592,
                60932, 51572,
            },
            {
                12904, 62500, 1688,  26896, 43320, 37896, 12464, 45092, 22876,
                36108, 53532, 12244, 60352, 40704, 16428, 5472,  35996, 11804,
                38956, 16216, 32376, 9304,  47496, 29752, 49632, 23076, 13792,
                51772, 31200, 56220, 12500, 27200, 39008, 748,   14764, 25160,
                8736,  42328, 63252, 27268, 15252, 8288,  46500, 40440, 1648,
                63832, 48444, 2932,  41624, 26640, 57148, 48944, 4136,  16444,
                7276,  20724, 39800, 12708, 19440, 36684, 10160, 29732, 21064,
                11932, 37112, 7508,  19848, 53584, 59212, 31760, 61904, 38240,
                5392,  35388, 44468, 22680, 42064, 10324, 39964, 52796, 12372,
                23372, 60432, 5220,  54352, 62396, 34612, 23404, 14140, 43740,
                1828,  59952, 51588, 4056,  29608, 53244, 23676, 11456, 49104,
                35384, 1148,  16928, 25800, 13488, 50920, 276,   20836, 8996,
                47756, 62116, 12796, 57348, 29676, 53288, 13672, 32236, 17228,
                38932, 50220, 7292,  20320, 23796, 59280, 14016, 32304, 21524,
                28944, 7448,
            },
            {
                37532, 31840, 53388, 14980, 50264, 19208, 30648, 58700, 51392,
                6068,  19612, 24820, 34620, 2884,  63628, 26820, 31424, 23552,
                65164, 1920,  43776, 19912, 63112, 17440, 156,   59496, 27660,
                9528,  18160, 62588, 41872, 4884,  59284, 54940, 43280, 61796,
                50956, 1856,  35812, 48088, 21136, 31320, 52508, 16640, 11772,
                42836, 30348, 53724, 18132, 7884,  12640, 52552, 38616, 43276,
                27792, 53144, 63640, 24560, 58704, 32192, 14968, 62376, 42120,
                46228, 50916, 63444, 40360, 25012, 13816, 2112,  50356, 17348,
                24904, 56720, 12556, 7460,  61360, 33832, 16328, 62256, 35612,
                2564,  19544, 39124, 12788, 47444, 29964, 52940, 18036, 50052,
                27924, 11364, 40172, 25460, 8132,  43032, 38704, 32496, 6908,
                44264, 65048, 39072, 48008, 62084, 36908, 44200, 30668, 24504,
                38856, 4740,  18300, 22912, 7484,  35192, 60256, 5128,  63268,
                1752,  57316, 12948, 41332, 47972, 8684,  42768, 63872, 10608,
                46064, 57268,
            },
            {
                16824, 44776, 8824,  22156, 63892, 25600, 16,    40184, 9992,
                65508, 43184, 57272, 30400, 49420, 45952, 54020, 57416, 7620,
                48232, 53016, 5280,  37700, 24648, 53848, 6048,  38612, 46196,
                34856, 2220,  47732, 24128, 32888, 10932, 20576, 6344,  37424,
                22528, 17356, 59328, 10740, 56412, 64932, 4292,  37260, 55464,
                19856, 25068, 13968, 38308, 65068, 35332, 19360, 23960, 62400,
                14024, 34100, 9316,  42480, 3684,  49492, 54024, 25924, 1356,
                33512, 23196, 3116,  56460, 34500, 6420,  42332, 28160, 10592,
                40660, 30824, 48588, 18088, 53648, 29724, 24,    50064, 43624,
                57916, 46344, 33024, 27448, 16384, 516,   41488, 58676, 5932,
                37088, 45340, 15360, 56104, 64364, 19988, 13876, 62316, 55356,
                19140, 10640, 30296, 22164, 8696,  54772, 17560, 12072, 57944,
                51732, 33196, 42472, 65080, 45736, 24956, 40208, 48436, 21724,
                36520, 26220, 32032, 52512, 204,   30304, 18948, 54508, 2492,
                40084, 24772,
            },
            {
                55248, 27668, 59444, 40820, 5224,  35836, 49008, 15384, 33092,
                28808, 3780,  38924, 14388, 22492, 10352, 18388, 39680, 13584,
                20936, 35056, 26932, 60524, 14160, 33612, 42416, 64824, 19792,
                54364, 60324, 14540, 36352, 45120, 65356, 29300, 47936, 32100,
                12636, 53556, 25556, 39944, 6660,  43412, 24044, 28416, 59620,
                8472,  50724, 4616,  57872, 46804, 652,   31332, 6216,  58244,
                46308, 1312,  50224, 28788, 16636, 45524, 5704,  39884, 17804,
                59352, 9308,  30488, 16676, 47388, 64644, 19504, 57684, 46144,
                63060, 1732,  59720, 37580, 24696, 46556, 6484,  21344, 26728,
                14708, 7184,  64840, 52080, 21764, 61440, 9656,  32084, 24432,
                63032, 20420, 32804, 48188, 20,    35784, 50108, 3252,  28972,
                25112, 51396, 4240,  57000, 32904, 3188,  41888, 64156, 46512,
                2092,  28604, 10048, 54148, 532,   16908, 11128, 52416, 29508,
                9060,  44432, 65524, 16648, 56280, 38164, 26972, 47400, 34292,
                62080, 5936,
            },
            {
                19872, 3536,  33564, 47032, 11664, 56764, 61760, 21556, 46268,
                55628, 17880, 51084, 7668,  59184, 36492, 4656,  42880, 62212,
                29672, 56232, 10784, 45236, 50880, 7784,  30900, 11320, 25304,
                4588,  29680, 50528, 8044,  18896, 52228, 15800, 25888, 57344,
                41076, 45696, 3096,  29816, 33748, 18332, 47660, 264,   34824,
                45280, 62432, 32636, 27816, 21472, 55160, 41468, 51388, 10992,
                36648, 22228, 55220, 38284, 61480, 22552, 35228, 65020, 12776,
                48084, 52980, 42724, 26940, 11376, 37236, 53080, 23588, 7856,
                34408, 20996, 4600,  43536, 13500, 57332, 64008, 41396, 31448,
                55840, 37240, 10368, 42252, 4724,  49096, 35876, 55168, 13272,
                2580,  53820, 60332, 9860,  30516, 23288, 58992, 16464, 46560,
                40856, 59608, 36220, 15472, 46992, 27908, 23616, 6384,  34920,
                15128, 60852, 21316, 36480, 49740, 27216, 57904, 42308, 15156,
                59500, 19336, 3828,  34896, 11452, 61496, 4340,  15592, 23140,
                12492, 49936,
            },
            {
                39204, 65052, 14084, 52388, 29848, 18008, 8196,  42036, 2428,
                25740, 12048, 62556, 44424, 27672, 52684, 32884, 25276, 696,
                49544, 16624, 40244, 2720,  58476, 21948, 48504, 57476, 16140,
                43648, 39780, 22708, 57964, 4228,  38400, 2064,  60792, 9464,
                5200,  64248, 15052, 50660, 55092, 12860, 61616, 52064, 14996,
                21972, 39348, 11396, 44044, 15780, 8956,  61652, 29012, 17124,
                26396, 64672, 18892, 2700,  11736, 30984, 8352,  55764, 27656,
                37652, 20788, 62280, 3872,  50748, 32044, 328,   14612, 29664,
                54460, 16256, 51872, 27904, 9032,  19428, 34884, 11712, 3600,
                51212, 18228, 29548, 59324, 25256, 38952, 18704, 28676, 46680,
                42116, 6936,  26236, 17928, 41184, 44696, 53948, 8836,  33772,
                5700,  12596, 43756, 63804, 20116, 39436, 59940, 52836, 18868,
                25764, 40828, 55960, 13828, 31432, 62584, 3592,  33792, 5964,
                54744, 46972, 40004, 24132, 49484, 21008, 40796, 51604, 57836,
                43472, 30440,
            },
            {
                9212,  44172, 24072, 1504,  38060, 26544, 53680, 34724, 50428,
                60760, 37272, 32116, 19420, 1668,  64668, 15776, 45544, 59544,
                8860,  32188, 23748, 64308, 18196, 27756, 37512, 1248,  52952,
                34324, 62272, 10080, 27516, 54812, 42248, 33676, 49712, 18644,
                35120, 27888, 20716, 60084, 36688, 22944, 9592,  41240, 31436,
                6316,  49452, 2180,  60424, 34984, 24968, 47404, 3292,  39832,
                44556, 5284,  32428, 47500, 57796, 51948, 42964, 19268, 2312,
                32400, 6732,  14212, 58796, 22236, 44920, 39668, 60180, 49108,
                42576, 36564, 65388, 32092, 49884, 38596, 54492, 25412, 61028,
                47764, 23520, 1108,  44852, 15096, 57312, 7608,  64572, 22112,
                50780, 34156, 37936, 52328, 4844,  12768, 27344, 38996, 61488,
                18144, 23360, 52720, 820,   10044, 48916, 13568, 37148, 8728,
                49532, 44900, 4168,  7800,  39320, 19228, 44588, 22668, 38116,
                25908, 13104, 62004, 28892, 5476,  58628, 33132, 7580,  28216,
                1104,  36008,
            },
            {
                16296, 50840, 57444, 20540, 63124, 45112, 5576,  14348, 23112,
                9164,  47640, 4892,  41648, 23924, 48500, 11388, 54720, 20036,
                38012, 52440, 43336, 6468,  35276, 12896, 44692, 63372, 20928,
                5844,  13596, 48904, 31640, 20504, 63484, 12300, 23104, 44024,
                56336, 38568, 48568, 8112,  2300,  45136, 27788, 57620, 17864,
                63900, 26092, 56896, 19104, 38176, 50776, 13264, 33260, 57212,
                53200, 14528, 59832, 40840, 27140, 14844, 24372, 49052, 58392,
                45652, 40340, 54568, 35500, 7992,  17244, 62896, 10456, 4160,
                24368, 12044, 6952,  22612, 59488, 1516,  45496, 17232, 8628,
                40220, 35180, 62876, 53372, 11464, 33628, 3156,  40364, 16716,
                10984, 62572, 14384, 58080, 47140, 65452, 1528,  21696, 55104,
                48592, 31504, 26700, 34636, 55400, 29880, 5324,  57276, 32148,
                63360, 29204, 58532, 23880, 47588, 53924, 10684, 60420, 50000,
                31652, 1532,  52052, 9836,  42908, 45968, 13540, 64060, 18792,
                53708, 61260,
            },
            {
                26624, 6376,  32240, 41784, 9760,  59996, 30652, 56356, 40672,
                28620, 15896, 58508, 53964, 30012, 39112, 6424,  34484, 28076,
                4484,  14196, 57136, 29540, 49872, 55012, 9076,  40376, 30600,
                25912, 45500, 58924, 504,   15188, 46568, 7600,  29968, 53316,
                856,   11356, 24972, 42708, 31032, 62624, 5344,  37364, 53148,
                12056, 42540, 29924, 7660,  53956, 4772,  62668, 19740, 23248,
                7972,  35396, 10424, 21116, 6712,  36188, 3884,  63736, 10792,
                16840, 23872, 29488, 48520, 26368, 56288, 31056, 20324, 27476,
                55444, 58516, 40880, 47116, 15588, 5832,  29400, 33456, 57600,
                13956, 6232,  20764, 30896, 48776, 26628, 52128, 59648, 44916,
                31292, 952,   23988, 28608, 19580, 32528, 36620, 15016, 43112,
                3132,  62492, 7984,  41608, 16784, 61148, 25108, 43928, 21148,
                2756,  11940, 17924, 34452, 64632, 1160,  29600, 16620, 7876,
                64192, 20236, 35796, 55704, 16948, 30684, 24948, 37464, 10908,
                46676, 22240,
            },
            {
                59316, 47668, 12380, 36300, 16692, 49088, 19680, 792,   64844,
                52232, 20780, 35360, 8248,  13548, 57572, 22012, 60668, 41892,
                63780, 47200, 21168, 2184,  61536, 24868, 17272, 3080,  51600,
                56428, 18416, 37456, 41560, 34732, 51648, 26336, 59464, 15708,
                32460, 63168, 17572, 58288, 13676, 19896, 50260, 24484, 1324,
                34268, 47340, 22580, 15068, 45296, 31248, 42384, 1352,  48304,
                63544, 29640, 51304, 46176, 62032, 54264, 39460, 28484, 34204,
                52660, 880,   63192, 12092, 43132, 2496,  36764, 48152, 44056,
                32836, 988,   18944, 27084, 62328, 43016, 53300, 21824, 63648,
                27612, 43516, 55308, 8820,  41668, 19828, 37056, 23240, 5592,
                56180, 48976, 39516, 7612,  54280, 10236, 49944, 58556, 29604,
                11340, 38208, 57824, 50524, 22176, 46104, 1640,  39932, 15476,
                53616, 37772, 51028, 42668, 14532, 25472, 36676, 56192, 46240,
                42532, 14920, 39312, 22992, 60696, 700,   50252, 56128, 41348,
                3568,  34656,
            },
            {
                52888, 2000,  28624, 55276, 4272,  25416, 39500, 32840, 12276,
                45796, 3360,  43748, 63672, 18468, 49924, 45000, 3000,  17080,
                9520,  26284, 36936, 11160, 41088, 32992, 47460, 60012, 35464,
                11672, 7012,  64400, 23824, 5056,  60988, 37836, 3320,  41152,
                47188, 6872,  51512, 39360, 54620, 33520, 46616, 15832, 39940,
                61044, 3700,  54976, 65444, 10856, 24240, 58308, 27648, 37320,
                15828, 41924, 25324, 432,   31372, 16484, 44204, 7940,  20900,
                60372, 41544, 6400,  19248, 59224, 14876, 53336, 5940,  63932,
                14048, 38260, 52944, 9200,  35512, 12388, 39472, 2380,  50104,
                46648, 3960,  36408, 58612, 15672, 1428,  62344, 13348, 27592,
                35260, 18244, 60156, 45580, 42400, 22796, 63092, 5744,  25044,
                46980, 20452, 14328, 4204,  36744, 11780, 63844, 33248, 47736,
                26936, 62148, 8924,  31236, 5552,  52696, 40596, 21552, 3300,
                27836, 57928, 4220,  47220, 8508,  34404, 20412, 6056,  62184,
                30368, 13856,
            },
            {
                42560, 38328, 64608, 21380, 51100, 43356, 58812, 6728,  37072,
                24444, 59932, 27292, 33064, 280,   25640, 36832, 30712, 51184,
                33424, 54056, 59628, 44968, 19492, 6812,  14156, 27912, 21320,
                42924, 50236, 28628, 9588,  55008, 17884, 11524, 22200, 56808,
                19632, 28760, 35656, 4692,  26068, 10296, 64664, 7364,  56360,
                28084, 19644, 32088, 35964, 39496, 17604, 7044,  52964, 11624,
                55340, 4684,  18972, 57040, 12008, 23184, 59492, 50476, 14012,
                47024, 31008, 37688, 50828, 33372, 25344, 40140, 9624,  17920,
                49784, 23300, 60820, 30892, 48552, 24532, 56156, 19556, 10496,
                32072, 17044, 24420, 65252, 28756, 48052, 54400, 8124,  43452,
                51204, 10652, 4492,  30796, 15648, 2176,  34640, 40328, 17164,
                53396, 33544, 64772, 26304, 31124, 52172, 18500, 9496,  56372,
                6016,  23212, 45292, 19860, 60872, 48092, 12320, 59880, 34788,
                9916,  51608, 32412, 26400, 52800, 64972, 39860, 15892, 25988,
                45372, 19744,
            },
            {
                7236,  24384, 15544, 33496, 61080, 8884,  14616, 47804, 54516,
                17296, 9664,  50844, 38848, 55260, 10012, 62800, 12792, 56640,
                23768, 1096,  15572, 30372, 63504, 52324, 55824, 38776, 62788,
                4012,  33184, 15980, 46384, 31188, 43828, 49368, 33708, 65156,
                9432,  44828, 23020, 61072, 1968,  42424, 30284, 22252, 44740,
                13492, 9580,  48340, 5556,  51220, 61264, 47048, 33908, 21864,
                60308, 45052, 38708, 65148, 47776, 33028, 2348,  36848, 26384,
                3820,  56824, 22948, 9072,  65188, 47128, 21164, 29780, 56960,
                34188, 3456,  44500, 5652,  16448, 64820, 7428,  37256, 26536,
                61348, 52176, 39716, 11844, 5340,  45160, 32360, 38756, 58296,
                21304, 64544, 25552, 37388, 61420, 55888, 27196, 45820, 8172,
                56952, 464,   43620, 48440, 6972,  55244, 41776, 29132, 59988,
                35548, 452,   39740, 57484, 27280, 2516,  18420, 30664, 49808,
                24208, 62324, 13432, 18652, 43208, 11096, 28732, 48784, 54876,
                9748,  60312,
            },
        },
        {
            {
                32352, 5312,  37212, 12392, 3044,  51792, 17284, 48016, 904,
                56364, 34560, 18404, 42860, 58492, 21400, 3360,  30912, 46644,
                24100, 57316, 1300,  21140, 40428, 64076, 16672, 24800, 29300,
                57712, 35320, 10576, 1732,  19740, 42504, 63888, 9240,  38612,
                55464, 1408,  50696, 8732,  24988, 18264, 55140, 11632, 33080,
                52760, 27680, 80,    25080, 18020, 46124, 1764,  9916,  24780,
                56344, 2968,  12756, 65152, 51220, 28644, 62176, 11812, 1652,
                55484, 8784,  52352, 21000, 43764, 49956, 29284, 32948, 45320,
                54888, 27952, 39980, 31708, 36928, 59536, 27728, 40160, 35504,
                55756, 45420, 14524, 48400, 65080, 9872,  56480, 31164, 24460,
                43088, 60724, 27640, 52436, 41848, 60044, 9528,  52056, 63168,
                18540, 5424,  28712, 41636, 16720, 45836, 9840,  58064, 15272,
                5112,  28328, 11036, 31888, 25140, 5600,  18616, 29112, 57596,
                23296, 10660, 37660, 45144, 62728, 32680, 47428, 42768, 3312,
                55172, 41364,
            },
            {
                27504, 45788, 60760, 49100, 31976, 8164,  25884, 10716, 29592,
                23028, 50380, 61672, 30132, 10172, 55184, 44472, 52380, 39908,
                4844,  35436, 42764, 13588, 26336, 33440, 9072,  50728, 43448,
                6276,  18108, 52232, 37864, 30240, 50920, 3696,  28724, 47788,
                19124, 36332, 30744, 14936, 44876, 5748,  63784, 29572, 57776,
                20872, 38384, 50484, 8584,  34068, 13244, 30808, 64008, 34988,
                52468, 40216, 47604, 23820, 16692, 39448, 32700, 46128, 60004,
                37528, 29172, 47068, 2932,  13996, 60344, 9900,  52656, 21808,
                5784,  48868, 18400, 9548,  52860, 12604, 20648, 47976, 8032,
                29440, 22104, 37872, 59332, 27128, 23312, 39956, 3552,  48648,
                17632, 54940, 13808, 38812, 5064,  47596, 21144, 35028, 40920,
                25664, 50368, 54948, 65040, 696,   37292, 24068, 62828, 39836,
                52676, 48908, 1660,  38244, 60488, 50392, 34980, 13300, 41180,
                32776, 17248, 26588, 2336,  29880, 21536, 5092,  57940, 10840,
                16956, 65480,
            },
            {
                1080,  56556, 25032, 18352, 40956, 56884, 35884, 62556, 53160,
                38276, 12484, 4412,  46964, 16268, 35608, 64460, 8452,  18612,
                27628, 63180, 9756,  51400, 55696, 3956,  37520, 61836, 13400,
                32332, 62884, 48076, 25660, 59212, 11876, 17440, 57876, 24392,
                44312, 11668, 58600, 27584, 53500, 39092, 49560, 23464, 43572,
                3352,  13620, 58580, 42596, 60788, 54168, 21768, 44232, 15036,
                5828,  18312, 58936, 36396, 6580,  57724, 19520, 25948, 4884,
                43268, 16140, 64096, 33580, 40376, 24280, 58100, 1164,  38180,
                13320, 63236, 24000, 46872, 61840, 42408, 1816,  57512, 63652,
                4368,  52152, 11372, 33056, 6128,  47012, 15308, 35888, 58184,
                6292,  29784, 35472, 65532, 16776, 32016, 316,   29180, 57592,
                3408,  14100, 33588, 8644,  26788, 49908, 13604, 30440, 2992,
                35232, 22664, 57188, 16296, 20356, 46976, 916,   61800, 52184,
                7420,  64184, 49448, 40164, 59872, 15028, 36768, 50732, 34044,
                23372, 49740,
            },
            {
                20468, 9892,  35212, 4784,  64220, 21644, 14520, 43480, 3452,
                19944, 41520, 57756, 24872, 39440, 620,   23404, 14236, 34144,
                59492, 47832, 16300, 31396, 45096, 58912, 19016, 47644, 22264,
                55816, 2624,  15264, 7312,  21980, 45368, 40240, 33372, 7072,
                63284, 3048,  41080, 62104, 20116, 2104,  35616, 16284, 7964,
                47456, 35944, 30216, 22748, 3716,  40144, 7140,  49108, 26688,
                61344, 31588, 10684, 27612, 53496, 9532,  49148, 14212, 52068,
                23480, 10872, 54256, 19108, 7516,  36128, 46628, 19436, 42620,
                30952, 56984, 3392,  34440, 6852,  29740, 16984, 33272, 43780,
                25656, 16500, 56880, 41988, 1988,  54272, 20776, 63884, 11280,
                41360, 46160, 2640,  23692, 57244, 49764, 13092, 54008, 10340,
                45196, 61308, 20876, 39180, 58620, 4828,  56544, 44884, 19284,
                55296, 12964, 40952, 63136, 8028,  30392, 56268, 26208, 21348,
                44032, 4112,  56456, 19656, 8576,  53456, 24968, 63152, 7128,
                44484, 39200,
            },
            {
                31348, 15468, 52936, 45064, 29276, 1844,  47668, 33164, 59316,
                28084, 6300,  32472, 62876, 51144, 28244, 48260, 54216, 43788,
                2008,  20820, 38640, 6452,  22684, 28728, 1000,  10792, 35032,
                40620, 28376, 44240, 54120, 36008, 61016, 404,   54908, 14020,
                52204, 34852, 22416, 8196,  32732, 13232, 59928, 26744, 62276,
                54332, 17884, 65140, 10784, 51992, 28824, 19088, 57600, 38412,
                1324,  22724, 45840, 63324, 2696,  41104, 30168, 64472, 35768,
                58468, 31540, 1868,  49112, 56136, 29692, 12020, 64824, 26356,
                8628,  51700, 15588, 44688, 25868, 38984, 54156, 22628, 13156,
                38460, 49404, 30724, 18764, 61444, 44168, 28472, 50724, 32488,
                25468, 20200, 53292, 8844,  43504, 26420, 38104, 63940, 23456,
                36092, 30336, 48636, 17588, 43508, 22172, 35880, 9092,  27512,
                46868, 6500,  25900, 34208, 44616, 14792, 39732, 10196, 36560,
                15160, 28696, 34944, 12196, 46520, 41828, 488,   18860, 29044,
                12696, 61460,
            },
            {
                51076, 41632, 59944, 12552, 38088, 55144, 24748, 7632,  17112,
                46548, 54288, 14856, 9312,  18852, 5656,  60300, 31840, 11700,
                56728, 25600, 52556, 62524, 12496, 42396, 54060, 60324, 25344,
                4616,  20192, 64904, 31972, 10080, 27292, 49304, 20632, 30324,
                26380, 46884, 16152, 56380, 48672, 42680, 52344, 10552, 32048,
                40496, 5348,  25556, 45068, 15776, 36608, 63452, 11564, 33252,
                55388, 42184, 50656, 15756, 33368, 22052, 45084, 720,   7660,
                39884, 20532, 42144, 26740, 61788, 16596, 4288,  53672, 48188,
                35468, 40604, 21552, 58436, 11584, 65468, 50420, 9112,  60924,
                872,   58968, 7204,  36320, 24824, 13404, 38944, 8440,  536,
                60576, 48948, 12664, 59576, 34028, 5476,  18672, 42320, 15992,
                7468,  52848, 1612,  11728, 62924, 32532, 52124, 16492, 64724,
                59024, 31736, 50136, 2312,  53052, 22120, 65332, 3060,  55216,
                47292, 61040, 23548, 51708, 32276, 27448, 59180, 38404, 47828,
                56808, 2572,
            },
            {
                8076,  22324, 27316, 6364,  19280, 61200, 10404, 51304, 65240,
                36964, 22112, 44400, 56416, 35008, 42256, 21720, 38068, 7800,
                29588, 41344, 3828,  35760, 49396, 33100, 16476, 38256, 51544,
                46556, 57808, 12472, 42060, 4036,  16536, 63672, 38852, 9448,
                42232, 60756, 4000,  38328, 29016, 4932,  22640, 46288, 1372,
                20640, 49184, 33656, 57124, 932,   48220, 24112, 4164,  46708,
                13876, 7724,  20016, 37676, 54516, 12028, 56828, 17732, 27576,
                47368, 63020, 13336, 5152,  44864, 38588, 33748, 23144, 2404,
                17848, 60588, 28460, 480,   48564, 19072, 3844,  35652, 46072,
                27652, 20564, 45152, 10368, 52484, 4300,  62668, 18176, 55752,
                35220, 15648, 40152, 30304, 21512, 50812, 61152, 2408,  47932,
                59636, 41156, 25500, 54784, 28632, 5944,  40860, 1236,  37472,
                11088, 42488, 18444, 60308, 11524, 28412, 42752, 33016, 50456,
                18284, 38976, 1488,  6312,  64880, 14204, 9648,  54724, 16076,
                33500, 25596,
            },
            {
                36704, 46672, 54548, 34276, 49876, 42384, 30108, 39520, 12800,
                26340, 2420,  49432, 30012, 12260, 63512, 3672,  17216, 50816,
                65144, 14808, 19256, 59624, 9168,  24376, 64348, 5876,  30684,
                8708,  17840, 36728, 52500, 23360, 46812, 34596, 5560,  58412,
                18232, 50600, 11140, 24932, 63232, 17936, 58624, 36624, 29412,
                55788, 12336, 61056, 8696,  41876, 31456, 53840, 17896, 61908,
                29372, 25076, 60392, 3596,  43784, 25692, 61488, 51024, 33952,
                10204, 53432, 36724, 24028, 51312, 9160,  57944, 43136, 62572,
                12536, 7328,  54360, 32300, 36672, 42876, 24452, 31028, 55116,
                14656, 39576, 63740, 56124, 29332, 34520, 47648, 22328, 44680,
                27108, 6768,  64256, 3168,  45700, 11324, 55192, 32240, 27500,
                19632, 34672, 4592,  45588, 15164, 60420, 49240, 20432, 25008,
                53772, 4176,  23420, 36156, 46692, 57400, 6252,  25028, 13016,
                8236,  30800, 57856, 43424, 21168, 35268, 45644, 23172, 4084,
                43040, 64456,
            },
            {
                59060, 17384, 3504,  63344, 15108, 396,   20936, 57484, 5704,
                34348, 60460, 8056,  38880, 24536, 52696, 46484, 58932, 23920,
                44488, 32424, 55560, 27024, 46348, 2392,  20464, 44640, 14116,
                62192, 49248, 1048,  29724, 59820, 55964, 12836, 25236, 53760,
                1840,  36220, 32188, 44756, 54228, 8828,  40096, 14776, 63840,
                43272, 17088, 37640, 28004, 21204, 59628, 9856,  43880, 38932,
                52216, 35068, 56484, 10380, 30972, 5540,  36948, 14388, 23272,
                3544,  57032, 17404, 30412, 60820, 21020, 14812, 27452, 31376,
                50824, 38012, 45212, 14324, 5336,  59320, 16280, 62708, 6644,
                51940, 32868, 2816,  23032, 41000, 15176, 58668, 30888, 10936,
                53536, 37128, 17708, 56688, 28012, 35980, 14884, 39724, 9296,
                56664, 13352, 63568, 38260, 23348, 9460,  30552, 56892, 45960,
                14456, 29128, 63500, 7888,  16260, 38604, 19892, 59652, 44784,
                63748, 26988, 53632, 16748, 49204, 2984,  62396, 30452, 50644,
                20148, 11848,
            },
            {
                28804, 40504, 10236, 31480, 25328, 36372, 48036, 44076, 17556,
                52572, 41808, 19568, 57632, 1400,  15168, 27320, 33836, 10448,
                188,   48460, 6792,  13012, 39016, 53376, 56856, 27964, 40964,
                34288, 21592, 26284, 40024, 8092,  19628, 43196, 31276, 45824,
                21796, 64732, 14224, 20280, 964,   34004, 50120, 25944, 6156,
                2820,  53140, 23756, 51148, 4692,  14844, 34736, 26532, 6572,
                440,   16600, 40532, 46824, 65368, 19288, 48352, 40440, 59832,
                43436, 32484, 7260,  45808, 112,   40352, 55204, 5868,  46468,
                19536, 23608, 64188, 26568, 56252, 51160, 10648, 41112, 19640,
                26020, 11928, 48432, 17724, 8156,  50916, 1432,  42864, 5328,
                61376, 48196, 23332, 42560, 51980, 7068,  25084, 64920, 44788,
                22460, 30020, 48784, 18564, 58092, 35576, 42992, 6568,  62044,
                33636, 51100, 40168, 55028, 31304, 1364,  51592, 34908, 4468,
                21900, 41308, 10220, 36996, 25492, 56164, 40724, 8248,  37800,
                53552, 5036,
            },
            {
                49372, 21876, 57336, 44872, 52428, 60520, 4484,  23600, 62940,
                31844, 14184, 28424, 62064, 44224, 36748, 7156,  55992, 40308,
                21116, 37044, 61528, 42300, 30340, 15600, 36200, 10556, 60140,
                5248,  55612, 45464, 63928, 15256, 50508, 2484,  61120, 39696,
                7704,  28564, 52568, 40704, 60204, 12492, 57332, 21256, 48328,
                35240, 61956, 7748,  45280, 40248, 64788, 48660, 57088, 20968,
                63044, 49356, 23084, 13504, 27892, 53076, 8724,  1204,  29064,
                20376, 49584, 63516, 26320, 12828, 32904, 48904, 36540, 10676,
                59768, 1692,  41688, 8808,  21312, 29116, 34396, 47260, 1280,
                57468, 43164, 36016, 60168, 27308, 65180, 20808, 38064, 25392,
                13940, 31836, 1864,  10136, 62068, 20504, 48980, 604,   53352,
                4024,  41868, 7608,  52652, 844,   26852, 12512, 18008, 38924,
                1776,  10260, 20100, 48416, 25616, 62004, 14340, 47824, 56196,
                32056, 15224, 61696, 6624,  32980, 12080, 18048, 27664, 58060,
                14848, 32412,
            },
            {
                43456, 2464,  37360, 18812, 7904,  13700, 29268, 10756, 54868,
                1908,  46592, 9812,  22588, 50092, 30844, 19116, 63948, 49796,
                29252, 16516, 52016, 3724,  22336, 59164, 50584, 1424,  24192,
                52236, 16892, 11676, 3868,  33360, 37464, 27604, 11004, 16168,
                57980, 48744, 4584,  25452, 46516, 30908, 7208,  28464, 41904,
                18336, 31808, 13752, 29788, 55016, 1556,  18292, 11388, 31040,
                41760, 8956,  32808, 2332,  57992, 34340, 24144, 62676, 55524,
                11000, 15292, 38092, 53044, 58368, 18816, 65200, 3460,  25232,
                52528, 32060, 16968, 49568, 39168, 3280,  60700, 22892, 37920,
                64204, 29528, 4104,  52772, 44896, 12476, 33652, 56720, 50104,
                19008, 41204, 58832, 34076, 16096, 38576, 30900, 58416, 17608,
                36884, 61600, 11544, 32596, 46424, 65288, 54116, 49888, 22604,
                27188, 59860, 43616, 4596,  11916, 42108, 28264, 8912,  39244,
                116,   52620, 46996, 20352, 51044, 60280, 44584, 948,   47052,
                24788, 61996,
            },
            {
                54072, 12700, 64988, 27380, 55836, 41052, 49520, 33860, 38748,
                25612, 51128, 35200, 4824,  55160, 11496, 42648, 2916,  13684,
                58228, 8432,  25744, 64524, 34100, 7612,  47040, 19600, 32708,
                42684, 38216, 30768, 57852, 53464, 20644, 62956, 47588, 23792,
                33548, 37628, 9920,  62796, 17108, 36364, 51600, 65524, 10396,
                56396, 47248, 59260, 20024, 25440, 32920, 38116, 44560, 53704,
                27248, 59452, 51784, 18580, 39268, 43000, 16040, 45688, 36044,
                4912,  41824, 22444, 2536,  8748,  43664, 28152, 15492, 42424,
                55796, 12388, 35560, 61628, 46012, 13848, 54032, 7524,  16068,
                49968, 9592,  14504, 24396, 38800, 6256,  29800, 3192,  45988,
                63340, 7592,  28532, 54488, 47100, 4956,  43324, 12544, 33876,
                24628, 50996, 28240, 20528, 40224, 16376, 3208,  31428, 45236,
                56084, 35904, 16628, 64816, 33484, 56768, 21528, 17560, 60692,
                24052, 28752, 42520, 2708,  38232, 29928, 22732, 63936, 10568,
                34876, 7188,
            },
            {
                16508, 24064, 47208, 33220, 1620,  22364, 63804, 15380, 59488,
                7232,  18428, 64976, 39484, 16404, 59384, 26056, 46924, 23416,
                35340, 54416, 45356, 18476, 11772, 43236, 26608, 61168, 13660,
                65520, 7760,  47800, 24760, 43908, 8788,  4940,  41768, 51824,
                216,   19032, 53952, 42316, 22884, 2280,  39652, 15484, 24168,
                368,   38704, 4500,  11648, 50364, 8544,  61236, 23676, 2688,
                15316, 36880, 45100, 6244,  63296, 3996,  12140, 31056, 51392,
                25652, 59008, 47572, 29816, 35664, 51636, 21964, 61328, 38352,
                7540,  29540, 57904, 6324,  19428, 32752, 44120, 25280, 31504,
                55448, 21420, 34644, 58012, 18808, 54428, 60684, 16528, 10408,
                53136, 21584, 36684, 13048, 24292, 57280, 27456, 63196, 6620,
                45292, 15504, 59216, 55240, 5076,  25308, 37332, 62664, 8272,
                13960, 5836,  29812, 23024, 52032, 2212,  36636, 44556, 54568,
                11368, 35216, 63488, 14136, 58572, 5504,  16052, 41420, 51684,
                19684, 39432,
            },
            {
                30824, 59264, 6076,  51008, 17408, 37108, 45400, 4764,  20856,
                41952, 56264, 29576, 45028, 1008,  33120, 37624, 62360, 5000,
                31620, 40788, 1112,  49388, 29744, 56576, 39776, 3056,  49032,
                28636, 21764, 772,   18160, 60172, 28920, 35276, 56204, 12316,
                26888, 59796, 30200, 13080, 56096, 6036,  59360, 44836, 52700,
                33908, 26780, 43892, 63604, 35300, 57508, 16548, 47288, 5772,
                64512, 11784, 25108, 29932, 53964, 22428, 48896, 61332, 18516,
                7840,  33608, 64296, 16512, 56176, 11460, 6192,  33196, 46852,
                20512, 50536, 24184, 408,   27732, 65088, 10980, 59136, 41756,
                5692,  45660, 61412, 1012,  41152, 48096, 22832, 35700, 40108,
                26548, 288,   44684, 64548, 3268,  19352, 9796,  51844, 21180,
                39700, 2672,  9080,  34660, 43584, 60752, 10940, 20880, 41820,
                52584, 47716, 58852, 38444, 45916, 7916,  62496, 31384, 4976,
                50156, 18740, 8276,  26760, 53420, 47804, 36240, 55680, 4080,
                28024, 61004,
            },
            {
                232,   35908, 11364, 41788, 57752, 9032,  30976, 53232, 27656,
                47704, 10288, 13396, 24096, 53648, 20364, 9100,  52132, 17972,
                60744, 10304, 21364, 36568, 62680, 5232,  17084, 35104, 9848,
                53060, 58904, 36108, 50856, 12832, 40516, 16464, 21908, 65060,
                44200, 7172,  46152, 34832, 48396, 27712, 32348, 20804, 7588,
                62200, 14796, 48992, 22492, 41488, 3256,  28428, 40076, 56540,
                34260, 50084, 20720, 8532,  57084, 37768, 27224, 1684,  40068,
                53712, 12592, 4072,  44736, 24772, 39940, 48708, 57684, 2636,
                14296, 63252, 37036, 42524, 48208, 52448, 38312, 1940,  18256,
                36512, 27160, 49260, 32008, 11060, 28088, 7860,  43932, 62380,
                32144, 17832, 51540, 39824, 48500, 32724, 42016, 55808, 30516,
                49676, 62412, 47120, 23468, 14276, 50864, 29100, 56696, 33676,
                18576, 664,   25188, 10692, 19292, 13784, 49256, 26072, 15788,
                39832, 57680, 46004, 32460, 20868, 9896,  25472, 31944, 13608,
                48652, 43028,
            },
            {
                52868, 45724, 20204, 28780, 62100, 24412, 12992, 61184, 476,
                36572, 58920, 32572, 5636,  63024, 49096, 42152, 29148, 13076,
                44764, 55720, 27164, 51484, 13444, 23584, 46144, 54952, 25312,
                41024, 14912, 45200, 6280,  32736, 61872, 48536, 4096,  31560,
                38416, 15400, 21164, 3964,  64196, 17664, 49984, 12244, 37692,
                55100, 18824, 30404, 6136,  53424, 13348, 20932, 52020, 31212,
                17820, 42244, 60104, 46216, 33512, 17024, 9960,  43700, 57424,
                21652, 28840, 37828, 19724, 61616, 668,   30584, 17644, 26328,
                44704, 10580, 54356, 17304, 8868,  22136, 14788, 29840, 51176,
                63596, 12860, 4352,  17400, 62880, 52276, 15136, 4216,  54824,
                12500, 58536, 5660,  29704, 15516, 61452, 36020, 1200,  13740,
                25648, 37072, 18932, 31200, 58136, 6824,  47984, 3808,  26500,
                39624, 60468, 31872, 55080, 41176, 28936, 58356, 3492,  43036,
                65216, 23184, 6092,  38652, 62768, 43620, 1328,  58028, 64660,
                22388, 9620,
            },
            {
                57168, 25720, 14508, 54560, 3072,  47900, 42968, 34152, 16384,
                51348, 22100, 40840, 46456, 27336, 14984, 6816,  57228, 22880,
                37188, 3228,  16128, 59012, 8324,  38716, 32072, 63620, 19128,
                3848,  30364, 63048, 26344, 54776, 1668,  24256, 53724, 9668,
                50576, 61196, 57272, 40228, 24348, 9076,  42552, 61084, 29080,
                2660,  40612, 9772,  58792, 32456, 46300, 63004, 10008, 25788,
                7176,  184,   38856, 13816, 4376,  52376, 65068, 34932, 14732,
                46364, 59960, 50356, 8660,  52900, 13772, 35000, 64124, 53356,
                40200, 28884, 33060, 3720,  62488, 34584, 55912, 46532, 8068,
                20772, 43256, 56520, 25508, 37592, 45364, 59296, 30384, 24432,
                48956, 34956, 22632, 11044, 53808, 7308,  23728, 18096, 65096,
                5288,  54572, 11452, 72,    41084, 36104, 17212, 44468, 64436,
                9352,  15356, 50564, 3116,  64012, 35508, 22680, 53184, 9708,
                34104, 29448, 2296,  56404, 13032, 50568, 18532, 39948, 6872,
                34388, 17520,
            },
            {
                5520,  63664, 40444, 32644, 37580, 6812,  19076, 56736, 25832,
                8816,  64208, 2580,  17856, 58172, 34904, 39552, 1804,  50956,
                65232, 33576, 43356, 30052, 47912, 60644, 376,   11532, 44024,
                57472, 8700,  39132, 20160, 10860, 37604, 43652, 18620, 34936,
                25708, 1356,  29292, 11320, 54148, 36184, 944,   56956, 22344,
                46864, 51096, 64852, 25040, 17272, 38372, 2200,  35612, 61176,
                44920, 55312, 28364, 62540, 24704, 47868, 19524, 30224, 6344,
                24952, 2504,  32668, 41576, 27544, 58824, 43076, 22696, 8424,
                4392,  59856, 23316, 58036, 41016, 26864, 5164,  61128, 24736,
                32848, 52992, 39912, 6972,  22008, 34180, 1552,  20188, 41928,
                8316,  45468, 60952, 38020, 26692, 43884, 58704, 46480, 38928,
                28580, 42784, 60652, 51984, 27056, 62784, 22548, 12352, 53188,
                36912, 46120, 21992, 43428, 6064,  16884, 47496, 37960, 20264,
                13428, 54220, 48212, 17008, 35256, 24436, 29992, 53288, 46492,
                27232, 38696,
            },
            {
                30708, 2132,  49340, 9844,  22212, 65440, 52776, 39184, 4280,
                44904, 30152, 37468, 54704, 11248, 24584, 61680, 31216, 18840,
                47088, 7636,  24668, 4420,  17900, 52480, 21972, 28536, 49884,
                33824, 23244, 52180, 46656, 16012, 57140, 28656, 59352, 13580,
                41328, 46908, 16920, 32748, 50788, 45044, 26288, 15856, 5732,
                34084, 13824, 36592, 3796,  43608, 54512, 23108, 49048, 12264,
                19992, 51268, 16256, 8060,  31948, 41832, 1040,  60584, 49508,
                36668, 11172, 63356, 15696, 21332, 5960,  49056, 12116, 36640,
                19852, 49860, 15472, 46940, 11952, 18912, 44300, 38620, 10760,
                58844, 44,    14488, 64228, 50780, 10344, 55856, 38656, 65388,
                17136, 3520,  55480, 19556, 1680,  50288, 12320, 31828, 9408,
                49052, 15680, 33792, 20744, 8824,  55300, 32600, 2328,  24512,
                29980, 7288,  61576, 27200, 33416, 11864, 60852, 428,   44956,
                61716, 26912, 40436, 9220,  59168, 5544,  61652, 11188, 15408,
                60112, 51328,
            },
            {
                12728, 24172, 60748, 44724, 15652, 28160, 31616, 10948, 49700,
                61776, 13944, 20620, 50076, 4012,  43724, 48512, 9456,  27916,
                12256, 53532, 39124, 56412, 34920, 42100, 14560, 37328, 5364,
                16752, 60424, 2236,  31700, 64652, 4476,  48104, 8008,  63752,
                5716,  55916, 22868, 61980, 3536,  19716, 63108, 31744, 41356,
                59608, 27268, 20128, 56732, 29488, 8000,  15488, 58952, 41320,
                30764, 36300, 23408, 58344, 38140, 11956, 56360, 22304, 39828,
                54104, 18460, 55940, 47656, 38524, 3032,  29628, 56460, 45792,
                62056, 31416, 1544,  37568, 52084, 30488, 2632,  49288, 15880,
                35960, 28236, 46900, 31184, 18452, 27352, 48152, 13192, 25992,
                51848, 32688, 28616, 41300, 63852, 34268, 21636, 56752, 2136,
                59708, 24320, 5892,  45496, 48504, 14608, 40408, 59800, 49336,
                56952, 19932, 13288, 39380, 57416, 51536, 25856, 30884, 55716,
                6896,  50212, 19096, 31936, 45920, 41780, 21816, 36808, 2548,
                43344, 21016,
            },
            {
                47764, 33700, 18388, 57296, 50800, 1388,  59244, 23524, 17656,
                34744, 41564, 27828, 60536, 33116, 16532, 21396, 55884, 41568,
                60144, 15364, 20992, 63360, 10096, 25860, 58288, 64984, 45536,
                54816, 40696, 13144, 27096, 42296, 22616, 36252, 32512, 20276,
                52648, 36708, 43152, 8332,  39300, 12892, 55352, 48004, 10284,
                52908, 6996,  45952, 11976, 49920, 63624, 33904, 27236, 6196,
                1836,  64776, 48192, 3396,  53644, 45036, 27112, 8668,  14160,
                28444, 43140, 7484,  25624, 31856, 65320, 51752, 24080, 14820,
                6552,  42712, 54976, 25320, 7720,  57108, 64768, 22672, 55104,
                19380, 42260, 9124,  59956, 3020,  40916, 61552, 5940,  36308,
                57868, 10768, 14860, 48108, 4776,  16736, 40172, 27784, 51772,
                19328, 36160, 64284, 30208, 3172,  37484, 18900, 6464,  42224,
                34892, 1468,  54064, 48084, 8488,  18960, 5072,  41100, 15348,
                23760, 35972, 1528,  63536, 25688, 3688,  49216, 56536, 33144,
                8596,  55400,
            },
            {
                37500, 7440,  42576, 3968,  35632, 12896, 42104, 46332, 7160,
                55592, 1052,  10088, 53336, 7296,  38548, 63532, 56,    35688,
                4868,  31696, 44028, 1884,  47232, 32500, 3424,  8608,  20368,
                25248, 35448, 6940,  51388, 56184, 10212, 15008, 50808, 26456,
                2260,  30192, 14472, 58592, 27368, 35168, 4708,  23504, 18376,
                37336, 61740, 24036, 42036, 740,   39040, 19180, 46400, 56328,
                14748, 43172, 10636, 34324, 19232, 15684, 64080, 51104, 3980,
                62552, 34184, 1616,  57864, 44536, 10228, 17888, 40684, 33680,
                27280, 10852, 63484, 21124, 35540, 14040, 41584, 32536, 5664,
                62092, 50348, 24132, 36916, 54056, 15724, 32248, 45324, 19320,
                472,   43144, 62604, 24820, 9004,  59112, 54192, 7116,  14384,
                45016, 40688, 11872, 53472, 25520, 57212, 63376, 28624, 11448,
                16820, 65412, 30748, 36520, 23036, 44320, 58868, 33104, 64684,
                10244, 43812, 57324, 12920, 52108, 16184, 28948, 65160, 18296,
                26712, 62744,
            },
            {
                15800, 53620, 29012, 25260, 39476, 20712, 62976, 33152, 27000,
                58496, 48012, 25096, 45512, 30072, 13780, 51576, 26292, 46620,
                23308, 50660, 58524, 29056, 13696, 51004, 38892, 53056, 30388,
                11328, 47752, 61420, 19612, 37884, 452,   62136, 45676, 39936,
                11236, 63056, 49480, 21716, 46100, 51664, 29984, 64464, 43852,
                1876,  28904, 16172, 58312, 31728, 52488, 9688,  24644, 37648,
                53440, 28112, 21852, 61464, 29904, 5228,  36060, 41492, 31388,
                46688, 23328, 52312, 19712, 13804, 35740, 58724, 808,   60984,
                52932, 38616, 17184, 46248, 59592, 28776, 10024, 47160, 26488,
                2176,  13096, 57748, 6216,  44416, 22776, 9496,  58948, 29676,
                49496, 22236, 37752, 52804, 30596, 36496, 43672, 23528, 32828,
                61012, 3864,  58024, 16472, 43116, 9720,  21672, 52168, 47388,
                26232, 43904, 5712,  15056, 62820, 28528, 2376,  47816, 18092,
                38492, 53968, 29832, 21112, 34116, 39436, 6932,  44648, 12188,
                40212, 1004,
            },
            {
                22184, 59764, 10876, 64500, 51944, 8508,  54620, 4656,  14644,
                36992, 19324, 39920, 65076, 22860, 58836, 34156, 6472,  17720,
                62732, 37308, 7336,  19392, 55580, 23196, 17180, 43312, 62400,
                1564,  56700, 32684, 16392, 29004, 43476, 24700, 53756, 17276,
                57504, 34364, 6200,  18156, 60,    60256, 8920,  39840, 13512,
                50428, 55088, 35624, 5468,  21936, 13972, 60416, 3700,  62724,
                17636, 50300, 7696,  39688, 46928, 59432, 24848, 20552, 58512,
                16704, 12124, 37516, 62016, 49580, 28568, 8080,  47216, 20096,
                4688,  49636, 30288, 6356,  524,   51152, 17916, 53604, 39708,
                34992, 45608, 29752, 20588, 34012, 64648, 50840, 25228, 4200,
                55348, 33564, 6564,  46388, 19980, 11980, 64908, 2864,  47852,
                29484, 8524,  23056, 34032, 49544, 572,   31572, 36192, 4060,
                56384, 21032, 51000, 41296, 10712, 53328, 35536, 12612, 51200,
                27068, 3928,  7892,  47524, 61448, 10460, 54476, 23864, 51116,
                30848, 46084,
            },
            {
                35088, 49200, 4880,  32372, 45248, 17480, 29612, 48828, 22564,
                61936, 11856, 5500,  16980, 2224,  44268, 10996, 41216, 54988,
                13360, 27144, 10572, 40488, 34212, 60812, 5104,  27464, 36316,
                15112, 22928, 45020, 4188,  49208, 60128, 7712,  31160, 3640,
                23392, 44336, 28372, 54452, 42128, 25128, 16364, 57536, 21376,
                32452, 8408,  46760, 65308, 40848, 48884, 29620, 42580, 34776,
                11664, 32212, 57516, 2720,  52788, 13160, 9748,  49784, 272,
                54880, 8468,  40884, 24984, 5320,  42172, 22524, 54576, 36988,
                25156, 14232, 56796, 40304, 44292, 24508, 60544, 6660,  21412,
                63720, 16792, 51912, 40552, 12060, 1872,  38176, 14416, 41316,
                61708, 12660, 17224, 60524, 1120,  26668, 50308, 17920, 38968,
                55732, 19848, 51164, 27276, 39260, 61884, 44720, 14208, 60100,
                8932,  33496, 61252, 796,   25636, 56940, 20616, 6508,  59844,
                22500, 62700, 14880, 41980, 58396, 64,    36620, 20072, 60464,
                6440,  56992,
            },
            {
                13216, 18904, 43124, 23476, 13932, 61244, 576,   39064, 44004,
                51556, 31496, 54364, 35404, 50036, 57008, 19680, 31300, 60220,
                3220,  43092, 49824, 64448, 592,   44820, 12108, 58188, 49604,
                41424, 8908,  63064, 38900, 11072, 21340, 34940, 13508, 65356,
                48492, 9068,  38796, 61388, 11752, 36900, 33436, 48812, 3008,
                62548, 24880, 18716, 11516, 26664, 57080, 5052,  20576, 47640,
                456,   23440, 44532, 26508, 18468, 42540, 32560, 38084, 27928,
                44316, 60948, 30752, 2444,  56564, 33336, 15280, 62612, 43796,
                9908,  32576, 65008, 22176, 11992, 33292, 37832, 14464, 56896,
                28016, 9652,  4052,  61096, 47392, 27244, 54388, 18164, 45928,
                8636,  27704, 39504, 56832, 42516, 54696, 34404, 9972,  61784,
                13616, 36336, 64092, 5744,  12600, 18656, 55220, 24708, 40572,
                17976, 38132, 29356, 48472, 16908, 39292, 46136, 30136, 40508,
                44576, 32812, 37228, 24060, 17924, 27312, 46668, 33000, 3648,
                41908, 26108,
            },
            {
                63924, 38628, 2128,  55724, 36356, 26604, 58132, 34036, 9268,
                3820,  27904, 59728, 41956, 25852, 7476,  28400, 38792, 48408,
                22560, 53888, 29888, 15204, 24732, 48168, 31012, 20716, 6672,
                25808, 53576, 30736, 51496, 26612, 58044, 54688, 41264, 36812,
                19984, 52668, 14988, 31408, 3464,  55640, 44848, 6088,  27836,
                42436, 38380, 53344, 1608,  33584, 37340, 15460, 54016, 59748,
                39388, 64560, 14396, 36832, 61944, 4576,  55608, 65164, 6900,
                19340, 34800, 48172, 17392, 64568, 52116, 11704, 30140, 2020,
                59372, 46796, 3884,  18648, 54540, 62492, 29848, 48180, 1444,
                41844, 54780, 32620, 19272, 58408, 7968,  30732, 63036, 34996,
                21460, 51948, 31532, 4416,  14652, 21548, 28892, 5456,  45340,
                25460, 1380,  41948, 46960, 58652, 8180,  29240, 2148,  52856,
                47060, 12400, 22972, 58768, 8224,  32612, 64420, 3036,  15796,
                9224,  52876, 2560,  50064, 31536, 55928, 13976, 63384, 16932,
                53192, 9320,
            },
            {
                50652, 27760, 59204, 47568, 5764,  50256, 11596, 19576, 63260,
                15896, 47344, 21440, 10108, 14500, 46176, 62008, 888,   15864,
                35720, 6184,  20004, 56648, 36632, 9108,  39944, 55164, 64764,
                34492, 17988, 864,   14380, 43024, 5812,  18500, 1592,  26980,
                59064, 5284,  24792, 47616, 19456, 64288, 23060, 52036, 10384,
                58992, 14992, 44552, 61416, 50820, 8988,  45488, 25560, 6948,
                28708, 51280, 9012,  48376, 30772, 22412, 15540, 47196, 24152,
                53308, 13952, 26968, 9644,  21096, 45760, 36312, 26628, 17960,
                39352, 53048, 28388, 36132, 49896, 2948,  8500,  43968, 23636,
                35956, 49528, 25608, 14832, 37264, 22476, 43068, 5056,  57172,
                1664,  48732, 64440, 24992, 35848, 48040, 59228, 41004, 51852,
                32204, 54420, 16132, 30316, 34872, 22536, 49272, 64812, 32308,
                6736,  63116, 3400,  44104, 52316, 13476, 24188, 54920, 47956,
                19596, 57616, 12128, 64800, 5208,  43108, 7576,  48728, 39340,
                29900, 21132,
            },
            {
                33628, 7052,  16412, 30328, 21516, 40204, 54260, 25168, 41592,
                53108, 37784, 2544,  64356, 32932, 54740, 23388, 52292, 10316,
                63964, 32544, 46116, 3684,  62348, 51796, 17260, 2080,  13280,
                46316, 59380, 22092, 36696, 64192, 32856, 46384, 50372, 11404,
                43688, 33156, 62868, 41644, 8300,  29192, 13728, 17876, 36244,
                31080, 20888, 6588,  29260, 23424, 18208, 64024, 32720, 12304,
                17380, 34540, 20736, 53668, 2324,  59676, 35184, 10628, 40232,
                4792,  57568, 43292, 50924, 39812, 2996,  6120,  55676, 49104,
                22740, 7832,  15872, 41672, 10720, 26400, 58196, 17320, 64676,
                11680, 7004,  61868, 44776, 856,   53116, 49424, 13484, 24580,
                40880, 11552, 18856, 44808, 9088,  62280, 1992,  17348, 23100,
                11024, 59668, 21172, 3324,  53560, 43416, 37844, 15552, 20560,
                42288, 26352, 56104, 34636, 19820, 4400,  37928, 27900, 61604,
                35196, 25004, 29452, 45652, 21324, 37576, 10732, 23656, 57652,
                1332,  45564,
            },
            {
                54668, 12232, 41372, 61684, 8948,  33048, 3212,  45428, 30620,
                6992,  24156, 58244, 18136, 40136, 5740,  36716, 18644, 44868,
                26224, 58280, 39172, 11508, 27888, 22772, 33232, 43180, 28816,
                38272, 5132,  56044, 48336, 9580,  24312, 15080, 62216, 29796,
                55892, 17156, 820,   35968, 57628, 50304, 39320, 60872, 53988,
                1128,  48544, 56496, 12668, 41680, 59128, 2684,  40004, 56320,
                43300, 58736, 5864,  41476, 12656, 27016, 43720, 51440, 30156,
                62960, 1456,  36860, 60072, 31420, 24224, 58872, 42332, 63572,
                13136, 34868, 61928, 56644, 46292, 21176, 39136, 33640, 52708,
                20428, 30896, 39852, 55540, 10856, 33468, 17444, 38252, 29008,
                61304, 33072, 5896,  54096, 38684, 13496, 30788, 55944, 37064,
                7148,  48224, 38728, 62516, 9436,  26592, 4212,  57976, 10640,
                36420, 49656, 14280, 29916, 41408, 62804, 50116, 11432, 1852,
                42652, 7228,  39620, 58760, 17188, 54084, 62428, 28052, 35284,
                14684, 60368,
            },
            {
                24596, 36800, 19468, 44952, 56408, 14552, 65284, 17648, 60244,
                13040, 34532, 43748, 29316, 48696, 12440, 57552, 30832, 41060,
                7768,  13928, 48944, 18412, 44192, 58596, 4508,  61232, 52400,
                24424, 11444, 40948, 28188, 2808,  53144, 39000, 20832, 7664,
                37892, 22900, 49492, 12960, 26364, 2780,  32800, 5552,  46020,
                26092, 63328, 33940, 37812, 4548,  48000, 30244, 52160, 21940,
                1264,  46680, 24888, 63656, 38660, 56756, 7488,  20412, 16696,
                33596, 22340, 12280, 16304, 7108,  53468, 19000, 9488,  31792,
                36,    45216, 24756, 5448,  31928, 51952, 13500, 5916,  59972,
                45888, 16176, 3472,  23320, 28256, 58980, 65324, 9276,  52364,
                46776, 15564, 58924, 28104, 20972, 50928, 6424,  26196, 43800,
                65424, 14344, 28468, 45040, 18124, 13228, 61620, 45904, 28844,
                54896, 148,   18276, 59416, 9912,  45300, 21800, 33788, 18000,
                56168, 51332, 14612, 1168,  26132, 33432, 3336,  51632, 18936,
                42404, 4980,
            },
            {
                48160, 64380, 300,   51352, 24072, 28332, 49140, 36492, 1092,
                47712, 55428, 8648,  51504, 1460,  26820, 63072, 3344,  50388,
                60640, 23832, 1260,  55588, 34640, 8388,  14904, 48804, 18572,
                7512,  62860, 33732, 19388, 60740, 31272, 58332, 45184, 3832,
                52768, 10400, 59752, 45548, 20900, 54900, 43916, 23724, 12120,
                40260, 8284,  16380, 24900, 54852, 20244, 14600, 36256, 10000,
                61920, 29972, 14740, 32860, 18032, 47812, 936,   61828, 45964,
                55432, 26448, 49668, 64244, 28900, 35108, 47276, 14616, 38304,
                51760, 28120, 59868, 19224, 2452,  62944, 28832, 42740, 448,
                26044, 50216, 63052, 35348, 47772, 19484, 5964,  44052, 3272,
                20256, 36324, 264,   41816, 63272, 34292, 46640, 58284, 18496,
                33136, 464,   24160, 50160, 56508, 40656, 33316, 51748, 7688,
                22280, 61316, 39516, 5844,  25716, 54500, 7460,  60176, 47188,
                31116, 22728, 36328, 60904, 49512, 41284, 13072, 45136, 61480,
                9824,  31800,
            },
            {
                53364, 27148, 10884, 33872, 4232,  39076, 7692,  57832, 22736,
                27592, 19688, 62368, 15392, 22160, 42820, 35040, 16740, 20756,
                28808, 37548, 51640, 30492, 65312, 40796, 26684, 37204, 31668,
                54904, 44992, 49952, 15972, 43548, 6096,  12468, 34504, 27412,
                64572, 40804, 30352, 34372, 62512, 9500,  15916, 64444, 30264,
                19212, 59404, 51916, 43064, 10464, 62544, 45308, 6732,  27492,
                49468, 37420, 55472, 3800,  22824, 52816, 28848, 37176, 9300,
                42336, 5588,  38632, 46336, 10700, 41428, 60548, 26164, 21308,
                56316, 7164,  40388, 12688, 37140, 48452, 22836, 55360, 36732,
                10356, 41124, 57224, 4948,  14176, 41696, 25844, 31656, 57544,
                23580, 55564, 50172, 25400, 10540, 16336, 2532,  39992, 9388,
                51712, 55508, 36188, 5196,  31144, 20924, 1648,  24660, 16020,
                43320, 30856, 47740, 52628, 36848, 15612, 28988, 40280, 13200,
                3860,  64148, 10440, 46420, 7836,  30676, 22072, 6160,  37788,
                56092, 21352,
            },
            {
                39728, 16632, 58296, 41916, 62776, 18768, 44056, 10788, 52100,
                41352, 4780,  38580, 32840, 60568, 53452, 9616,  47284, 55200,
                4752,  43296, 16016, 6484,  20120, 45940, 11128, 56888, 3412,
                21496, 13656, 1148,  26600, 56772, 23208, 51372, 17340, 48612,
                24444, 15260, 2344,  18632, 4988,  42188, 37172, 49152, 55988,
                3732,  47004, 31576, 84,    28500, 34468, 56876, 23672, 53568,
                18772, 11640, 42432, 8188,  59572, 34836, 11832, 24796, 58548,
                14548, 52472, 19816, 1828,  22964, 54856, 6388,  3636,  65456,
                48244, 16540, 32960, 54800, 43816, 9204,  17532, 58800, 14664,
                32148, 21192, 29204, 8756,  54280, 37112, 50472, 62464, 11056,
                40244, 13624, 7792,  32040, 43860, 53788, 23964, 61580, 29648,
                20672, 42212, 12132, 63916, 8372,  59368, 37216, 48356, 64392,
                34760, 12748, 3152,  23356, 32648, 65192, 1276,  57312, 25176,
                44184, 53132, 27988, 20112, 15828, 57440, 65444, 50408, 25496,
                15116, 2652,
            },
            {
                30728, 7480,  46156, 21924, 13820, 55308, 31252, 34808, 15656,
                64144, 30276, 11540, 45276, 6100,  25356, 39220, 13516, 64000,
                33560, 11340, 61356, 25284, 53064, 1832,  23268, 62472, 42392,
                28628, 38540, 63896, 35444, 10128, 39788, 61400, 104,   42352,
                8568,  58464, 55536, 47676, 27984, 52516, 22040, 6720,  27048,
                35640, 14092, 21804, 64884, 50096, 17452, 3340,  39280, 43736,
                4432,  65448, 26396, 50968, 44428, 15620, 63860, 40460, 4620,
                31140, 61320, 27744, 57184, 32836, 16796, 50376, 30500, 36032,
                42936, 10296, 23780, 50248, 64200, 27076, 34732, 6716,  47548,
                64936, 53024, 18168, 44900, 60592, 22180, 1116,  16448, 34692,
                28516, 45656, 64644, 18552, 59176, 4732,  37432, 48072, 14976,
                3592,  58672, 47236, 16972, 28028, 42588, 54048, 10744, 4888,
                55900, 20144, 62204, 41720, 10808, 48436, 20860, 50492, 37972,
                17524, 6084,  41744, 35072, 54708, 38512, 732,   28664, 34240,
                59104, 47184,
            },
            {
                63428, 36176, 52692, 29664, 6176,  50488, 25712, 60632, 3364,
                47124, 24416, 59028, 50044, 19144, 57496, 2264,  31652, 22472,
                45572, 57276, 40264, 35964, 48428, 59252, 33208, 16360, 50960,
                8976,  59072, 52460, 4896,  47948, 19936, 28836, 37032, 53908,
                30968, 21784, 36260, 39804, 13260, 61364, 31904, 11360, 60228,
                44564, 54144, 38364, 7008,  40992, 13188, 61192, 31884, 59020,
                15440, 34236, 20960, 39052, 30448, 3104,  49744, 21568, 46912,
                17728, 35584, 8744,  44340, 63100, 37372, 11440, 45472, 19492,
                1132,  57452, 29844, 4852,  20732, 1516,  52452, 42056, 23900,
                4608,  38564, 2004,  33844, 12036, 30536, 46948, 7168,  53000,
                60276, 5012,  37780, 48552, 29144, 21380, 11968, 54928, 32540,
                26528, 38884, 22716, 34556, 52388, 13564, 19140, 26680, 39248,
                45156, 28180, 8204,  57880, 17668, 44628, 5188,  30688, 9288,
                62592, 32556, 58828, 2940,  24456, 11304, 44384, 17940, 8244,
                41340, 12052,
            },
            {
                19304, 3912,  24672, 61608, 37968, 984,   45776, 20748, 9308,
                54700, 35620, 196,   14532, 36876, 29468, 41736, 51732, 8040,
                27064, 492,   18996, 8872,  13868, 27720, 39488, 6264,  47108,
                34376, 18088, 23696, 31344, 42676, 55952, 7044,  13424, 62756,
                46320, 11868, 6540,  50224, 25152, 940,   57260, 41412, 17316,
                2424,  24556, 9940,  58128, 26676, 46416, 21336, 9600,  28200,
                48508, 56260, 632,   60476, 10312, 18896, 56988, 33528, 55032,
                248,   50196, 39816, 13296, 2724,  26504, 59400, 23508, 62332,
                13572, 34444, 60936, 46052, 15244, 39548, 31004, 62024, 12568,
                27832, 46096, 58628, 25276, 40768, 64400, 56376, 18984, 43056,
                21484, 26168, 1788,  14564, 52588, 35292, 63960, 42612, 7424,
                62584, 50400, 6512,  1076,  44844, 62304, 3316,  32468, 60472,
                16564, 51060, 37396, 53956, 25892, 34412, 60452, 14940, 54788,
                22464, 47720, 12976, 51084, 63548, 32064, 48128, 53816, 62208,
                23212, 49868,
            },
            {
                28212, 56432, 10300, 43244, 17776, 12332, 57216, 40468, 29096,
                16952, 42592, 21976, 56304, 8820,  64912, 16996, 60316, 48124,
                15420, 54276, 30060, 44772, 63552, 3760,  19828, 54236, 12724,
                25908, 2476,  45624, 12008, 15836, 64836, 33684, 25052, 19040,
                2796,  59640, 32824, 65012, 16544, 45460, 35272, 20572, 51168,
                63792, 32552, 48148, 18708, 35840, 55268, 1500,  51284, 37760,
                7124,  24532, 13760, 53072, 45752, 27520, 7384,  42840, 11312,
                25628, 65056, 22360, 54152, 18792, 41700, 52128, 8004,  39856,
                53248, 25512, 41648, 7756,  55548, 59632, 9576,  18796, 50284,
                56748, 16160, 9700,  51808, 6140,  14576, 27912, 38060, 9544,
                49328, 33216, 58140, 41380, 9152,  24916, 792,   17852, 45936,
                10928, 19240, 59784, 30812, 24480, 36072, 56608, 49776, 7224,
                23944, 708,   31704, 13832, 2896,  39892, 51820, 27684, 36064,
                340,   39408, 26756, 17040, 42816, 4248,  21148, 14140, 36664,
                1772,  44020,
            },
            {
                51548, 14588, 34472, 59268, 49128, 32160, 65024, 36648, 5156,
                52596, 62388, 31320, 47848, 26652, 44392, 21304, 5016,  34312,
                37984, 62184, 23512, 51336, 34696, 57676, 31208, 43840, 64672,
                56240, 40756, 60036, 53412, 27644, 1824,  50200, 45160, 40092,
                52264, 28192, 43632, 19664, 9372,  53600, 4536,  29852, 8476,
                39556, 15060, 28308, 61772, 5760,  42964, 23940, 64160, 17996,
                45116, 41468, 31452, 35628, 64324, 23080, 38408, 60076, 29976,
                15672, 45332, 6208,  28896, 48548, 32104, 4124,  15816, 29288,
                48420, 2824,  18288, 36952, 27948, 22148, 35516, 44396, 2872,
                32952, 36980, 63212, 20028, 48780, 35488, 23232, 2852,  62376,
                12884, 54448, 17456, 46524, 60708, 31748, 50872, 57456, 28552,
                36440, 41528, 55460, 15424, 48100, 8888,  21736, 41396, 12952,
                47036, 64612, 43452, 59500, 21944, 7368,  64068, 11196, 43728,
                57732, 19976, 7508,  30344, 60604, 9628,  39644, 57788, 26316,
                6788,  32716,
            },
            {
                64616, 38772, 2084,  21160, 27772, 8228,  23160, 14376, 49624,
                25996, 11148, 6740,  39508, 2892,  54172, 10824, 24856, 58220,
                12524, 3040,  42340, 6752,  16724, 11200, 49704, 22420, 29084,
                5200,  36284, 20344, 8184,  38948, 22768, 58860, 10264, 16460,
                35860, 4812,  23588, 56040, 38624, 26572, 62656, 48792, 23128,
                55496, 44152, 4204,  52212, 11436, 31052, 14784, 34780, 53628,
                11268, 62040, 5492,  17264, 2500,  48608, 14448, 3708,  51516,
                37096, 56152, 33760, 62096, 10148, 58232, 45884, 35792, 56712,
                21540, 10444, 63904, 51416, 12856, 47960, 53864, 25972, 7444,
                41504, 22756, 29272, 43100, 136,   55440, 45528, 58380, 40140,
                29072, 24216, 6404,  36776, 20336, 5324,  39636, 13880, 22948,
                2340,  52864, 25776, 12016, 38720, 63876, 17600, 29908, 53872,
                34548, 19400, 9608,  28888, 49992, 46428, 18324, 24500, 4020,
                49344, 62224, 45516, 53400, 35876, 23872, 50580, 29604, 61220,
                54384, 17144,
            },
            {
                22580, 47620, 6040,  41572, 53892, 45396, 4132,  55980, 43848,
                18596, 33352, 60360, 13964, 50436, 35304, 30528, 40184, 51112,
                46684, 32232, 20724, 55700, 26452, 41116, 1368,  37956, 9740,
                15448, 50740, 32176, 62448, 48588, 35188, 5596,  30540, 62228,
                57092, 13840, 48484, 2276,  34132, 14736, 42872, 11844, 36956,
                520,   59508, 34456, 21868, 40292, 56552, 47540, 4348,  29608,
                21208, 56724, 26516, 50880, 42116, 57588, 33172, 62836, 21284,
                8460,  18408, 3196,  39088, 14708, 24696, 20320, 64784, 6032,
                32916, 44612, 57580, 31244, 6144,  852,   62952, 15664, 55672,
                61000, 13168, 4356,  59552, 32448, 11104, 17300, 31192, 50324,
                4172,  43576, 65084, 11204, 55976, 26900, 62328, 49160, 34192,
                64864, 7812,  32124, 61172, 5024,  44308, 1504,  57804, 26220,
                3840,  40316, 56436, 15960, 36968, 32820, 55840, 41616, 31776,
                16200, 34352, 12072, 2000,  56276, 18364, 5872,  44496, 12532,
                40932, 9208,
            },
            {
                30740, 58324, 26112, 62076, 15376, 34948, 59744, 30628, 2228,
                63688, 37876, 46056, 20460, 57708, 16164, 63540, 1184,  18920,
                27968, 9504,  65052, 47528, 36024, 61832, 58648, 18624, 47268,
                61492, 25132, 540,   11492, 18028, 55768, 43804, 20272, 26120,
                7656,  41516, 63768, 29644, 51248, 60836, 6420,  57812, 18868,
                26800, 13464, 49696, 17000, 63416, 26436, 8556,  60456, 39916,
                1256,  46876, 37056, 8876,  20236, 29024, 10920, 24844, 47328,
                40852, 58680, 26880, 43936, 52984, 884,   37896, 12092, 54180,
                25960, 38992, 14916, 23732, 42556, 34048, 39860, 21040, 30644,
                47312, 50784, 18188, 53696, 39224, 26644, 64212, 7996,  14768,
                20904, 34652, 51192, 30608, 15560, 45132, 3176,  10176, 43192,
                16620, 46752, 21128, 50040, 27924, 33696, 51316, 11292, 37276,
                61644, 48680, 23644, 5212,  62964, 1968,  13116, 59740, 8520,
                52040, 22620, 27876, 41108, 14720, 47220, 33344, 692,   20380,
                35424, 45848,
            },
            {
                55324, 13056, 18788, 50764, 10472, 39208, 19540, 25264, 40988,
                9008,  55096, 23412, 5632,  27572, 42348, 7816,  55384, 44644,
                60108, 15956, 39152, 4424,  13792, 23948, 7316,  52604, 33900,
                41836, 57252, 45232, 29424, 41092, 14148, 2584,  50708, 54400,
                46596, 32152, 18144, 10900, 21452, 46064, 24840, 32604, 41668,
                65384, 30112, 7564,  45336, 2024,  36688, 19532, 50164, 24344,
                15748, 32936, 59524, 13460, 54020, 39144, 44464, 5888,  54404,
                31248, 12004, 49836, 22096, 60156, 30472, 48724, 42848, 17512,
                2380,  61572, 8404,  49964, 60480, 17984, 46244, 10772, 2096,
                38280, 8944,  24448, 45004, 6676,  21572, 51884, 37132, 47324,
                60388, 56852, 608,   41548, 22192, 37900, 54300, 19616, 29596,
                58340, 37416, 3528,  40624, 18456, 59340, 23252, 41888, 16808,
                8172,  31120, 12384, 53256, 26036, 44972, 20604, 29408, 38888,
                65256, 4552,  58156, 37148, 63408, 26016, 58820, 52636, 63912,
                25476, 2964,
            },
            {
                8120,  37944, 33928, 29256, 348,   64048, 52148, 12560, 48080,
                15908, 28976, 756,   59544, 32672, 48772, 24124, 13292, 36224,
                5912,  22916, 53904, 29228, 50828, 32508, 43924, 27496, 3656,
                12812, 21076, 6696,  63944, 24032, 59584, 33244, 38676, 11536,
                22708, 36712, 5244,  58876, 39428, 1348,  16652, 53960, 9468,
                46920, 38516, 55036, 23552, 32320, 58568, 43776, 12648, 54660,
                64272, 42792, 6224,  22988, 63348, 952,   17968, 59420, 35724,
                2468,  64656, 16552, 5040,  34360, 8612,  62948, 27908, 51324,
                35084, 47096, 20232, 28876, 3932,  52228, 26768, 65380, 57164,
                33796, 28156, 63392, 35324, 15512, 61212, 41900, 1952,  25364,
                10468, 17596, 27740, 7552,  48676, 59704, 32296, 6332,  51572,
                24876, 12912, 56512, 9192,  54196, 13692, 6764,  65344, 45732,
                21272, 59596, 42952, 33892, 39500, 50648, 6612,  54536, 25292,
                46588, 10652, 43444, 20184, 7744,  30780, 10532, 38828, 14300,
                49564, 60116,
            },
            {
                41416, 52928, 4528,  43600, 46984, 22888, 6368,  58208, 34256,
                53212, 44156, 36556, 51296, 11176, 38392, 19832, 61952, 51620,
                33176, 41800, 58564, 100,   19360, 9252,  64300, 16348, 55956,
                30904, 39228, 54036, 36172, 47072, 8984,  27528, 65276, 16028,
                296,   60996, 53380, 43560, 27816, 56680, 36048, 4908,  51068,
                20684, 3588,  60232, 10776, 52812, 15220, 5096,  30464, 35588,
                9784,  27340, 52392, 31324, 46044, 34176, 50168, 15024, 27764,
                20092, 45764, 36920, 56824, 42000, 14056, 19548, 4152,  58376,
                10972, 30956, 41088, 55744, 13112, 37092, 7844,  22576, 14360,
                42904, 3816,  54596, 11548, 48700, 4640,  28688, 54828, 33492,
                44540, 38504, 53684, 62764, 12012, 25428, 14528, 61812, 40876,
                96,    62788, 28828, 45480, 35764, 25848, 47876, 32020, 3384,
                27736, 54992, 244,   18568, 10068, 61068, 15632, 35856, 988,
                18112, 31360, 50768, 54744, 3016,  48880, 42272, 22392, 6376,
                28684, 16924,
            },
            {
                23668, 65376, 20708, 56980, 16328, 31548, 37412, 27384, 3580,
                20288, 61032, 13556, 18204, 65260, 3404,  43808, 29892, 2144,
                25976, 10416, 48528, 35160, 45628, 54732, 39612, 22636, 48240,
                60768, 9820,  17228, 3084,  49972, 21848, 4244,  56228, 45176,
                29336, 49004, 25464, 14112, 7796,  48596, 30656, 62180, 12788,
                25780, 34840, 17760, 42480, 27484, 39164, 62172, 46172, 3184,
                19800, 49020, 17104, 4388,  57964, 25676, 8664,  62480, 41540,
                53240, 24116, 10328, 28812, 47508, 25420, 55492, 40392, 23496,
                15548, 63472, 388,   24616, 59304, 44480, 32832, 53416, 48340,
                17544, 59232, 20492, 40648, 31364, 57692, 18504, 12636, 22504,
                64004, 4292,  31356, 19220, 34844, 2272,  46544, 36528, 22572,
                48540, 33452, 16124, 21620, 60672, 1140,  39044, 56792, 51836,
                35568, 13828, 49608, 64104, 30076, 22924, 57448, 42140, 49172,
                60208, 13284, 35124, 24560, 15644, 61296, 18248, 57352, 36536,
                45108, 32168,
            },
            {
                11484, 48008, 26964, 8692,  61584, 50140, 42812, 10112, 63092,
                24740, 7436,  31132, 41560, 25532, 54028, 8796,  57648, 46488,
                17780, 64088, 14436, 21788, 61268, 12268, 5388,  34632, 1108,
                26064, 51376, 34020, 28612, 14900, 43228, 52280, 19004, 34984,
                40632, 9868,  20360, 33984, 64696, 18372, 22904, 38964, 43412,
                56840, 29480, 63856, 976,   50460, 21704, 7776,  25772, 58108,
                40668, 60716, 37572, 10836, 42200, 20864, 38756, 30720, 12576,
                6984,  61256, 332,   51872, 62288, 5652,  32584, 37236, 46076,
                7684,  53264, 36348, 48848, 16824, 4864,  61664, 29688, 39036,
                6748,  26192, 50976, 688,   23796, 45868, 36196, 60160, 50128,
                8364,  14944, 46200, 51452, 42168, 58804, 55256, 18104, 11092,
                7772,  53588, 42380, 4636,  50832, 30300, 15692, 8680,  18956,
                41184, 24904, 6956,  37868, 47352, 3320,  33108, 9368,  27752,
                21708, 5532,  62908, 38972, 28880, 45772, 33932, 4664,  54092,
                62272, 1068,
            },
            {
                58052, 14868, 35808, 39984, 2168,  13456, 18680, 55412, 45636,
                38824, 49088, 56788, 4904,  47204, 34272, 15812, 22240, 36908,
                55280, 31280, 6592,  38188, 26812, 30280, 42880, 57548, 14312,
                45972, 20068, 61780, 57844, 39792, 63040, 31500, 12980, 7084,
                62352, 51064, 4136,  58180, 41064, 11300, 54736, 2692,  15892,
                48820, 6928,  37852, 13540, 47172, 33572, 55476, 16004, 51228,
                22568, 108,   29140, 65316, 14444, 54772, 1856,  57128, 48264,
                44080, 33624, 38572, 20928, 17364, 11964, 50704, 1508,  60704,
                28392, 43352, 19704, 32244, 10060, 40660, 20584, 1288,  12276,
                62596, 44832, 34428, 15152, 64932, 9988,  6248,  42996, 30124,
                39720, 58528, 26672, 23104, 9768,  5028,  27672, 30560, 44448,
                64408, 26096, 38056, 12096, 63776, 44092, 24276, 59952, 11416,
                62992, 45092, 58016, 16744, 26972, 43740, 14192, 64516, 52272,
                40432, 45148, 56076, 9540,  168,   51384, 12944, 24784, 10040,
                19808, 51088,
            },
            {
                33236, 5524,  46088, 55156, 24548, 33956, 60020, 28864, 1176,
                16116, 35096, 22512, 10624, 59392, 39452, 28116, 60992, 12504,
                3804,  43220, 53068, 58884, 2976,  49292, 18908, 62780, 37552,
                7672,  41312, 2292,  24540, 10484, 5120,  37120, 26268, 59308,
                23124, 16436, 37584, 46784, 29600, 6164,  45680, 33192, 60012,
                9716,  23932, 53592, 58268, 19368, 10132, 64728, 37028, 30860,
                13176, 45088, 34052, 47700, 24336, 51448, 35828, 5252,  18260,
                26540, 14924, 55380, 30548, 40976, 45024, 65272, 22032, 17032,
                56764, 12292, 5672,  64632, 27560, 56668, 50360, 25016, 55148,
                36472, 9096,  29092, 55932, 39128, 52924, 26492, 16936, 1320,
                20452, 54608, 2752,  37168, 65348, 16008, 40056, 49428, 1736,
                57848, 19860, 31568, 56308, 6460,  20800, 49300, 33964, 28760,
                4600,  20652, 32524, 10668, 53220, 56612, 6220,  19496, 36524,
                2732,  16656, 32744, 22844, 58552, 40676, 64820, 31156, 43460,
                38428, 27624,
            },
            {
                41640, 63276, 20060, 30164, 51908, 4116,  40872, 21956, 51420,
                12216, 63700, 52972, 29952, 19404, 528,   51340, 7000,  48628,
                40192, 19104, 24244, 45328, 16180, 8656,  52224, 23844, 28496,
                55044, 31860, 12640, 45348, 54464, 21240, 57012, 47112, 1084,
                43120, 32100, 55444, 2208,  24384, 52288, 62672, 26708, 19720,
                36432, 44272, 31528, 5240,  41012, 28764, 2304,  42960, 6148,
                54312, 9188,  56964, 19208, 7280,  43536, 28604, 63848, 22700,
                60384, 49420, 3296,  58708, 7356,  27104, 35668, 8940,  31712,
                49500, 24352, 39544, 52644, 45992, 13528, 34140, 42368, 18268,
                46844, 22228, 60772, 2772,  19764, 32376, 47484, 56188, 62420,
                34220, 48392, 12844, 43940, 31692, 52192, 21244, 61312, 34172,
                14348, 3916,  52088, 17236, 35368, 41028, 2268,  53564, 46508,
                55516, 36932, 2904,  61968, 21916, 38740, 48776, 25588, 59888,
                29720, 53544, 12444, 48592, 18776, 27240, 3904,  55660, 48424,
                16624, 2740,
            },
            {
                23500, 9596,  58888, 17128, 11560, 64716, 47920, 6012,  31488,
                43160, 26104, 3180,  42196, 14836, 64264, 44328, 25136, 33404,
                63104, 29156, 10184, 34568, 64688, 40536, 32928, 11316, 4924,
                48580, 65100, 17788, 35440, 49412, 29236, 16836, 39480, 10276,
                53304, 27956, 64044, 12284, 20680, 36764, 14840, 41972, 516,
                51856, 63184, 16576, 22232, 60380, 45668, 24864, 49880, 18780,
                63160, 39400, 27060, 3484,  61820, 11792, 15920, 33248, 41040,
                8320,  36836, 11184, 46396, 22996, 52476, 14288, 55124, 41920,
                3480,  60028, 35164, 1748,  22612, 7732,  58688, 3236,  64388,
                31584, 5088,  14000, 42008, 50016, 12300, 3960,  37732, 24272,
                10592, 28548, 60268, 17696, 5980,  56448, 11880, 8152,  23952,
                43372, 39524, 47780, 9036,  27980, 62316, 14260, 7952,  39344,
                16120, 26576, 49708, 41960, 29824, 1220,  34060, 11220, 46748,
                8020,  24076, 63864, 43208, 6480,  37444, 14756, 21928, 7924,
                61632, 53900,
            },
            {
                49820, 36484, 44132, 7104,  38768, 26648, 35356, 57780, 17768,
                59716, 6964,  48032, 35816, 55344, 32260, 11168, 20540, 58764,
                15352, 1180,  56008, 51028, 20764, 1792,  60192, 44276, 15372,
                21668, 58260, 27152, 42608, 3944,  7972,  64328, 50748, 34368,
                19784, 14728, 7908,  44412, 59808, 49192, 8616,  31044, 58120,
                11944, 28020, 3776,  49444, 34484, 15016, 11720, 61240, 35600,
                23808, 16480, 52680, 31340, 37940, 49948, 58756, 46212, 3788,
                52076, 29780, 19092, 62152, 33224, 2520,  59568, 38344, 20152,
                47468, 29716, 14780, 18688, 62372, 37440, 30220, 15764, 10452,
                48644, 53912, 37088, 27404, 57736, 22812, 64112, 15344, 45360,
                7456,  40752, 51132, 24776, 33660, 45612, 27156, 37596, 54164,
                60064, 28980, 22116, 57220, 44816, 25380, 58240, 31624, 22932,
                64028, 12200, 59188, 8192,  14932, 51768, 63480, 17492, 42472,
                56928, 38584, 1448,  34676, 59700, 52980, 46832, 57820, 35484,
                29184, 13180,
            },
            {
                4320,  25936, 31764, 54296, 21024, 45596, 14760, 9640,  54528,
                38172, 20908, 62252, 8840,  23896, 49532, 3992,  37924, 52688,
                8148,  46756, 37152, 13224, 30936, 25300, 56400, 35708, 53728,
                39496, 268,   9440,  52220, 59724, 32208, 13372, 24140, 3124,
                61596, 48284, 40568, 25504, 33068, 17684, 54604, 5472,  23596,
                47560, 40256, 56280, 8412,  38480, 53208, 57584, 4968,  32164,
                48268, 1484,  59528, 42136, 21200, 25620, 9428,  19840, 54980,
                13720, 25204, 56200, 39188, 16032, 43216, 5584,  25372, 10656,
                63244, 7120,  53920, 41464, 49816, 25724, 44652, 52176, 40064,
                24288, 58508, 18732, 45552, 7932,  35372, 30252, 42252, 58088,
                53572, 19868, 3812,  63508, 38732, 392,   62468, 48360, 18244,
                6536,  12376, 63240, 840,   37512, 11272, 50412, 19476, 43668,
                548,   47192, 34712, 19776, 57144, 44988, 23112, 3976,  32000,
                14484, 50612, 20348, 28484, 9120,  31796, 25340, 2028,  40588,
                18940, 45784,
            },
            {
                56660, 15592, 60976, 5268,  50108, 62444, 156,   24252, 44804,
                33420, 12736, 27688, 40060, 17124, 60732, 29344, 56820, 41804,
                26728, 22408, 62300, 42756, 6556,  48228, 17844, 4064,  29380,
                46696, 34112, 14104, 23812, 38184, 20328, 41772, 45404, 57360,
                30148, 35864, 4572,  57760, 1744,  39256, 63764, 43768, 34756,
                14412, 20212, 32632, 64532, 26012, 716,   29660, 21224, 40300,
                9728,  44856, 13704, 6456,  34900, 56300, 500,   38864, 32088,
                64968, 44664, 1124,  49724, 27488, 63756, 31236, 51308, 16860,
                44524, 27364, 57320, 32660, 9680,  4488,  60660, 21068, 28740,
                7332,  33716, 308,   61816, 11120, 52328, 5144,  18316, 924,
                31604, 36068, 13560, 47552, 10052, 20816, 14828, 30984, 3436,
                36012, 50868, 32796, 15700, 53996, 4340,  34332, 61272, 6260,
                28868, 52840, 24404, 40384, 5412,  27668, 37620, 55060, 61072,
                26252, 6060,  62324, 44948, 12864, 17160, 42664, 60888, 11208,
                33380, 64176,
            },
            {
                38212, 22608, 41704, 12708, 33616, 28720, 41196, 52560, 29756,
                2908,  50752, 56640, 46700, 1628,  43540, 14004, 6104,  18180,
                35476, 50480, 3428,  57920, 27896, 39028, 63584, 8540,  24284,
                60400, 19620, 63820, 47692, 55692, 1892,  61164, 26632, 6240,
                17048, 54508, 22652, 51412, 29224, 10904, 21656, 27356, 59976,
                50852, 1916,  44708, 10888, 17784, 47152, 42828, 51568, 15444,
                55876, 28692, 64476, 51716, 17352, 47860, 62856, 27944, 42420,
                22276, 6496,  35464, 9260,  12456, 21356, 46860, 57896, 34612,
                37728, 24,    20788, 12712, 46648, 17164, 55272, 1580,  13164,
                63844, 44116, 15988, 39716, 24980, 47728, 60056, 27792, 49400,
                61512, 23380, 55636, 29656, 57664, 40904, 53120, 58752, 42924,
                25568, 45984, 20084, 41552, 24364, 47580, 17372, 40084, 55224,
                36552, 9780,  17252, 64804, 33004, 12764, 48888, 9524,  41304,
                18536, 36292, 47916, 55304, 23300, 64600, 50684, 7056,  21404,
                52332, 700,
            },
            {
                9228,  30104, 47792, 2756,  55972, 16584, 7928,  60404, 19200,
                65340, 15784, 5792,  31000, 20180, 54108, 33816, 64492, 45748,
                11528, 31460, 15760, 19916, 10280, 52612, 14304, 43596, 51056,
                11688, 40828, 5700,  30572, 15928, 35592, 11612, 52524, 38424,
                9860,  43568, 13212, 62140, 45648, 16104, 48600, 4496,  9396,
                37380, 62416, 29308, 53700, 22912, 36228, 59040, 7948,  62408,
                25144, 36408, 22144, 11152, 30520, 44016, 5172,  15428, 10420,
                48344, 17680, 60120, 56996, 41148, 53716, 1996,  8216,  13480,
                22848, 59788, 51964, 39648, 65112, 35492, 30944, 42448, 36976,
                53348, 49804, 21896, 31024, 55020, 14540, 38036, 21664, 11896,
                40016, 8368,  16432, 43260, 26068, 6920,  35052, 23116, 10968,
                64084, 55276, 8240,  59660, 30432, 65296, 8764,  26264, 13640,
                49220, 59852, 44100, 3216,  53624, 58984, 21088, 320,   30592,
                53312, 11756, 2356,  32880, 40132, 4316,  37104, 29920, 47240,
                27180, 43832,
            },
            {
                53596, 59200, 19480, 63980, 37404, 22960, 46800, 39072, 11112,
                36112, 42988, 23768, 61852, 38544, 10376, 26300, 23064, 292,
                60264, 55288, 38368, 44912, 33288, 59312, 21776, 36784, 31912,
                3204,  54396, 25824, 44580, 8592,  50252, 29568, 19176, 47980,
                65188, 33604, 20012, 7452,  36600, 32236, 53296, 41676, 55924,
                25192, 17044, 6632,  41444, 61044, 4040,  13044, 19988, 33632,
                2300,  49980, 40972, 3748,  60560, 23800, 34244, 53660, 61396,
                37452, 52520, 29884, 23680, 33704, 18060, 29340, 39376, 64376,
                48692, 31096, 5608,  26564, 8044,  22376, 49024, 58944, 18916,
                10664, 26952, 4624,  63084, 2460,  33988, 43760, 6832,  64872,
                45800, 33284, 51560, 2196,  64276, 18024, 49680, 1436,  29164,
                16944, 4952,  37836, 13252, 1904,  43492, 20840, 57060, 32788,
                2036,  21296, 31196, 26024, 46820, 16496, 35916, 62108, 44332,
                22692, 59240, 28000, 8420,  57924, 19924, 14712, 54652, 60152,
                12608, 17692,
            },
            {
                6000,  35744, 26204, 10188, 50900, 27480, 58000, 4840,  21880,
                49072, 58656, 8964,  34888, 50980, 47244, 58972, 40008, 52012,
                28476, 8444,  23784, 65400, 5604,  47340, 776,   27092, 62984,
                18300, 48448, 37716, 57720, 22224, 63288, 4604,  56424, 23652,
                1196,  28348, 40180, 58732, 24472, 76,    64636, 19508, 12760,
                33888, 58328, 48060, 14624, 32132, 49800, 28208, 39272, 46192,
                6204,  58664, 18588, 54860, 37980, 8736,  57916, 19584, 1724,
                26156, 13324, 5416,  46220, 3924,  61904, 44112, 24972, 54956,
                3712,  42428, 18252, 45296, 56616, 14632, 3516,  24964, 6692,
                32900, 56332, 41596, 46308, 19676, 10120, 57052, 51196, 17336,
                25444, 4324,  59940, 37564, 12820, 31520, 45312, 60972, 52296,
                40032, 32228, 48120, 27472, 52684, 35820, 50560, 5176,  41288,
                63256, 15340, 38220, 11356, 7624,  42096, 28576, 5084,  14264,
                49728, 38396, 16832, 42884, 25424, 48740, 34500, 2416,  24020,
                39952, 32288,
            },
            {
                49012, 3332,  45640, 14316, 43148, 1536,  34328, 13800, 55168,
                31524, 780,   27120, 18012, 13432, 2680,  7360,  19564, 14496,
                47940, 42508, 2868,  53448, 30068, 16424, 41532, 55712, 6976,
                58660, 13724, 33556, 892,   17508, 42836, 39364, 32384, 15100,
                59828, 46296, 4848,  49592, 55544, 14692, 38356, 29684, 3096,
                45476, 21368, 38464, 2676,  24600, 56776, 9536,  63908, 54176,
                15884, 43360, 32280, 26776, 14440, 46328, 29152, 40592, 49840,
                32844, 42888, 64424, 38292, 15120, 50716, 10960, 20104, 34252,
                15752, 9956,  62620, 37304, 52800, 29036, 63176, 44420, 39084,
                61136, 14248, 6516,  36576, 59816, 26388, 30028, 3144,  36772,
                54912, 29244, 20664, 48572, 54444, 22016, 8992,  36444, 14044,
                21152, 57396, 62248, 22788, 16396, 58908, 11228, 28600, 45980,
                23772, 52396, 48144, 58160, 62660, 23248, 51648, 55844, 34096,
                65396, 3792,  46720, 61116, 52496, 10240, 63208, 44424, 8048,
                56844, 65212,
            },
            {
                41896, 23592, 55376, 31092, 62124, 18072, 52072, 63780, 25336,
                40356, 46224, 54304, 64100, 30008, 56120, 43996, 32404, 36580,
                63016, 18348, 34400, 13220, 25784, 50332, 34856, 12240, 45416,
                21072, 28912, 9560,  51436, 59644, 26808, 12624, 44908, 7256,
                36840, 54368, 12144, 17808, 27532, 8588,  47332, 59276, 52336,
                26772, 9132,  54984, 64948, 35252, 17504, 42532, 21836, 30680,
                25916, 11412, 48256, 1352,  65512, 52044, 6392,  12364, 22796,
                58576, 8936,  20896, 53980, 26920, 57700, 6952,  47460, 60596,
                28092, 50516, 24088, 32944, 1232,  11456, 34960, 16680, 51244,
                46956, 29492, 23640, 52548, 16588, 48836, 41076, 22944, 58780,
                14144, 41912, 11116, 5684,  27284, 40584, 58256, 4044,  26248,
                42548, 2924,  9520,  44964, 6760,  31960, 39100, 18832, 60496,
                9060,  4380,  28064, 33780, 1360,  39680, 18436, 10232, 25548,
                7464,  20824, 31852, 13468, 660,   30268, 17220, 26940, 37476,
                21060, 15480,
            },
            {
                28792, 60556, 8380,  20660, 40220, 6656,  28768, 44072, 9624,
                19464, 15336, 5484,  42040, 37816, 21288, 60980, 26856, 4756,
                10672, 55812, 39704, 59712, 20316, 62236, 8572,  23860, 52752,
                38432, 42584, 65032, 23972, 36180, 3508,  48840, 62160, 52624,
                25536, 21560, 42004, 31892, 63080, 43712, 22592, 34680, 6052,
                60792, 42948, 30056, 12212, 51812, 6836,  47444, 416,   37192,
                59476, 7432,  61912, 34664, 20884, 42572, 17752, 36212, 62340,
                3612,  47104, 17068, 35180, 1784,  41460, 30784, 36416, 752,
                40368, 58772, 12988, 20424, 48156, 41188, 58032, 21604, 9116,
                1780,  19376, 65020, 32492, 672,   12816, 63528, 32976, 9148,
                47988, 61676, 34604, 44740, 63096, 15284, 33356, 46712, 64860,
                50636, 19068, 34316, 56064, 25432, 63636, 124,   54580, 14128,
                36720, 43328, 17200, 53844, 13024, 45620, 30512, 48024, 57900,
                37664, 41840, 55048, 23612, 36340, 40984, 56000, 46540, 3604,
                51040, 10868,
            },
            {
                328,   38264, 52340, 33168, 58892, 11996, 37232, 48216, 2832,
                60372, 35452, 49664, 24372, 8600,  52764, 15936, 49324, 58068,
                22656, 28108, 7076,  48716, 2364,  37084, 56744, 31032, 1712,
                61292, 4964,  47356, 15744, 54324, 29876, 20792, 9360,  17312,
                30408, 2112,  60356, 35844, 51840, 3448,  11760, 40692, 18272,
                15004, 50144, 1336,  19700, 39800, 26836, 60296, 14252, 50676,
                19300, 53460, 23576, 39472, 5136,  57608, 31088, 55236, 44608,
                28132, 51584, 31660, 61368, 12180, 23016, 63892, 17620, 53608,
                8360,  44160, 4540,  55408, 61752, 7504,  26640, 31684, 54628,
                37400, 57308, 11260, 44164, 39368, 55604, 5460,  45272, 19804,
                1708,  24440, 52136, 18344, 324,   56292, 24044, 7080,  11576,
                54636, 28504, 38496, 12508, 41684, 46992, 21688, 49440, 33300,
                25304, 65004, 56904, 20400, 6652,  36268, 63608, 2352,  15868,
                29400, 10932, 50804, 64020, 5680,  19460, 60320, 12404, 31768,
                62440, 34816,
            },
            {
                18992, 46372, 15888, 4696,  49164, 24420, 56604, 21468, 32368,
                55620, 29280, 12208, 61796, 3004,  33768, 11592, 40808, 1056,
                45268, 51920, 32044, 42772, 11244, 46276, 17912, 41044, 14284,
                26276, 19204, 31964, 6944,  11572, 41268, 57264, 34320, 39108,
                63952, 48088, 10600, 6664,  20848, 58084, 48936, 25576, 62772,
                32388, 37344, 23720, 61888, 45220, 31720, 55452, 4820,  34064,
                40940, 44940, 13612, 28800, 50080, 9808,  25268, 164,   15832,
                11256, 23968, 39528, 7284,  55532, 43960, 50008, 13852, 25740,
                21624, 32340, 28704, 38780, 17836, 44760, 3668,  64120, 13924,
                42568, 27540, 48524, 7604,  25792, 21892, 50448, 15804, 38024,
                54184, 31868, 8036,  39008, 29624, 49132, 19948, 37196, 31060,
                16728, 1272,  49276, 60780, 15716, 3888,  30088, 58224, 7212,
                40716, 2528,  31332, 49736, 27020, 59664, 22272, 52616, 43404,
                61280, 18212, 3288,  45012, 28252, 49044, 7020,  52968, 22332,
                43052, 56284,
            },
            {
                26368, 12488, 63352, 27716, 42856, 1032,  61844, 14332, 7536,
                39784, 51692, 22084, 44368, 28300, 47020, 64944, 24804, 37652,
                17256, 63404, 14196, 21744, 26520, 64364, 6116,  51124, 59172,
                35136, 49348, 56504, 38108, 45744, 60448, 2480,  50452, 5956,
                43680, 14952, 55112, 27132, 16888, 38008, 30600, 1808,  56028,
                10028, 46904, 4660,  52964, 16000, 10620, 22516, 62684, 29200,
                10156, 2436,  55860, 17140, 61440, 46848, 38288, 63188, 53480,
                41620, 4408,  57956, 48560, 19660, 27588, 3080,  39112, 57072,
                47016, 65420, 51580, 14816, 24768, 36040, 52456, 47176, 23856,
                5116,  33320, 62232, 18036, 59376, 29792, 35116, 61144, 27212,
                41452, 65132, 14804, 59604, 10560, 43232, 53492, 60092, 40820,
                62648, 45580, 22312, 6188,  36404, 53316, 18224, 44148, 12752,
                51956, 22660, 10632, 45056, 41664, 14640, 9000,  32908, 5192,
                47488, 26308, 34408, 58488, 14920, 33052, 37952, 25696, 16524,
                5380,  48576,
            },
            {
                39420, 57464, 21780, 36488, 54472, 17132, 34496, 46284, 26472,
                64428, 4532,  16780, 36844, 59016, 19316, 6596,  30980, 53852,
                9272,  35580, 3632,  58544, 54100, 33920, 28860, 22528, 44636,
                3376,  10520, 62928, 21556, 27516, 17824, 24592, 13256, 28492,
                22080, 58744, 32564, 40968, 46432, 65416, 14096, 53252, 42656,
                20020, 28296, 58440, 34588, 7528,  42240, 37916, 17772, 46324,
                25220, 64308, 32120, 36384, 3560,  12692, 21840, 32516, 19112,
                34848, 65016, 29444, 15412, 37504, 9888,  34120, 61720, 5824,
                11428, 35728, 2052,  9516,  60804, 30212, 12024, 19672, 39196,
                50572, 15524, 2172,  36868, 52784, 9264,  3308,  57024, 11816,
                6152,  21500, 46652, 25828, 35288, 3132,  13732, 5292,  26736,
                8620,  33664, 55704, 25896, 31848, 64488, 9328,  27636, 37312,
                61736, 16468, 34896, 62888, 552,   54380, 25184, 40416, 20124,
                12776, 56016, 39660, 8792,  21428, 63008, 10608, 40664, 59116,
                30196, 9024,
            },
            {
                33032, 3052,  44780, 7824,  30876, 10256, 41192, 52940, 18976,
                11156, 42624, 56800, 20,    50276, 14556, 55948, 43532, 20952,
                59396, 29500, 50316, 38592, 16428, 628,   39848, 12628, 54520,
                17116, 30036, 42308, 16,    52780, 32956, 64604, 54648, 47376,
                37456, 1020,  24400, 50796, 4224,  9236,  23384, 35156, 7132,
                38848, 63704, 13064, 25296, 56492, 49316, 1476,  53156, 58056,
                5884,  48752, 20520, 42192, 26272, 58020, 43516, 5728,  50328,
                8716,  45716, 22088, 912,   59980, 45140, 52048, 29580, 16592,
                41924, 19752, 57644, 48664, 40776, 54756, 1096,  59036, 8672,
                61216, 55896, 21504, 41432, 45592, 13896, 48044, 19164, 43976,
                30932, 2252,  57928, 50944, 19012, 63456, 23556, 47408, 51988,
                20760, 14536, 59096, 10892, 50700, 41020, 23604, 59256, 4788,
                46260, 55336, 5796,  29132, 18968, 37348, 57728, 50432, 64896,
                35920, 908,   23292, 51924, 44108, 2788,  54352, 45968, 1640,
                64712, 51736,
            },
            {
                24268, 60648, 14928, 50860, 65168, 23344, 58252, 2512,  37636,
                30176, 48516, 32964, 25552, 10308, 39656, 35012, 3328,  12876,
                46896, 5848,  24576, 44376, 9968,  49224, 60164, 7304,  63692,
                37184, 23956, 50668, 14604, 8624,  36280, 4892,  41600, 9472,
                18712, 63156, 12360, 34592, 57620, 29296, 44692, 59460, 16092,
                51828, 284,   43912, 18556, 30792, 65136, 27388, 35100, 14968,
                38832, 11580, 59684, 51292, 7900,  53012, 28648, 60492, 14544,
                25836, 55396, 12792, 32268, 53940, 24464, 4516,  21316, 55616,
                31508, 44944, 27296, 5140,  16288, 22140, 32892, 42780, 26976,
                35556, 30776, 5952,  25092, 63280, 32112, 39464, 23472, 62636,
                53792, 37268, 42328, 7248,  32428, 40124, 56872, 30180, 35836,
                44632, 3264,  42228, 37840, 19728, 836,   15012, 48844, 32624,
                20108, 26148, 39844, 59100, 47744, 11776, 6432,  30096, 16420,
                27768, 48864, 61636, 31112, 17360, 57344, 27884, 23460, 35016,
                19904, 11672,
            },
            {
                47452, 28280, 40816, 4148,  19792, 47048, 27976, 6020,  62916,
                22532, 59532, 7408,  20228, 62824, 29040, 52156, 61260, 27004,
                41036, 62120, 18872, 56440, 30852, 21268, 43224, 25876, 32200,
                46604, 4140,  58480, 39460, 61360, 44812, 15904, 59052, 30116,
                51892, 45244, 7200,  17172, 53880, 20208, 2368,  49008, 26240,
                32068, 22268, 48064, 36852, 3512,  12056, 21028, 8512,  44060,
                23428, 30064, 816,   15296, 34000, 18240, 39580, 1924,  47736,
                36080, 61484, 39004, 42460, 17416, 62704, 12068, 40112, 50352,
                8220,  59996, 23912, 34716, 64172, 50148, 6232,  46188, 17396,
                12464, 47768, 53388, 10864, 17152, 12,    56340, 28668, 8704,
                49124, 17344, 12168, 28072, 54676, 15772, 9816,  712,   17928,
                65248, 53396, 28540, 5568,  62792, 45264, 29636, 56216, 2472,
                42496, 9572,  50548, 15184, 22968, 33924, 45576, 2768,  59936,
                43820, 7032,  12172, 42268, 5436,  36740, 13052, 50208, 15712,
                42920, 55904,
            },
            {
                1072,  17932, 34304, 59516, 37996, 12716, 33460, 50512, 15984,
                44828, 12596, 53584, 41292, 46180, 5264,  23148, 17324, 48972,
                896,   33992, 13644, 7060,  65072, 35500, 15276, 52580, 19604,
                9248,  55404, 28436, 18892, 48312, 23096, 26872, 2092,  21248,
                35396, 60564, 26784, 42708, 38488, 61060, 13120, 41136, 62212,
                10848, 55556, 5972,  60148, 51608, 40864, 46780, 61624, 32764,
                54112, 63304, 40296, 46408, 23072, 64608, 10372, 54728, 31064,
                20440, 3112,  6328,  49320, 8492,  35080, 47144, 25700, 63648,
                8,     13264, 47808, 38448, 10964, 28512, 61508, 55316, 24236,
                3100,  65228, 43288, 27808, 58504, 49904, 35776, 5044,  15124,
                25396, 34524, 64404, 22496, 3896,  44188, 49464, 58608, 39192,
                11788, 23884, 47164, 16452, 34536, 53752, 8900,  61348, 36152,
                13132, 64344, 31440, 3108,  53708, 63556, 40944, 21640, 9828,
                54884, 38268, 19196, 26460, 64520, 47252, 32548, 4184,  61288,
                7352,  37532,
            },
            {
                62260, 49308, 10160, 52360, 25672, 56620, 43020, 8960,  55284,
                35048, 1956,  26960, 36412, 15312, 56464, 33472, 9464,  57800,
                21884, 37544, 54760, 27192, 46312, 4728,  57076, 1532,  40628,
                62312, 13488, 34136, 5860,  11644, 31672, 56488, 49376, 40424,
                14180, 4972,  31896, 49616, 10104, 24712, 30368, 5332,  34204,
                17796, 39768, 28420, 15180, 23368, 56932, 17268, 6124,  49472,
                3380,  18608, 27288, 56912, 4460,  48964, 37024, 24524, 16060,
                44572, 56708, 27564, 23376, 30696, 58476, 3540,  15360, 37296,
                33256, 18544, 54136, 3276,  44144, 20372, 14124, 36680, 40332,
                57288, 19588, 38052, 33856, 8084,  22108, 42064, 52744, 60800,
                45112, 1044,  58540, 47064, 36596, 61100, 25072, 34008, 27804,
                7344,  60500, 32076, 57048, 12420, 21044, 39356, 24228, 17096,
                47228, 54624, 24940, 38136, 8140,  27040, 13904, 52308, 31876,
                24304, 58444, 34248, 53676, 9480,  20812, 58940, 40236, 53272,
                30632, 21724,
            },
            {
                13140, 42180, 31492, 6712,  16640, 400,   61992, 19344, 24120,
                40336, 61076, 18512, 49976, 65372, 1464,  43240, 39116, 29768,
                63708, 10748, 42640, 50948, 23236, 38664, 12000, 28056, 49692,
                36076, 22128, 43524, 65300, 53916, 38156, 7396,  63640, 10248,
                52796, 57120, 22288, 532,   64704, 36548, 46528, 56868, 52200,
                8064,  64240, 45852, 34564, 9588,  31516, 38348, 25384, 58392,
                13636, 37256, 10124, 43388, 32004, 13364, 58928, 42068, 7428,
                62436, 12176, 50972, 64224, 18952, 43500, 52920, 27748, 57148,
                42444, 22908, 61984, 30380, 57908, 8160,  52508, 616,   29948,
                9684,  50616, 4928,  14852, 46840, 63968, 18864, 11520, 32768,
                39820, 20516, 51724, 10152, 29864, 6468,  20164, 13356, 51260,
                41008, 21492, 1880,  49196, 26360, 42956, 6584,  51520, 28700,
                5640,  21188, 60200, 44012, 18520, 49416, 61024, 36352, 17292,
                46676, 2240,  14872, 49268, 38860, 68,    24892, 17860, 10016,
                45532, 26128,
            },
            {
                4632,  54484, 64216, 23040, 46080, 39588, 28340, 32292, 4740,
                47688, 29512, 6724,  10844, 31336, 25088, 13552, 51360, 6680,
                45876, 15996, 32304, 2372,  18308, 61972, 31152, 59056, 16840,
                45740, 3284,  50544, 26136, 968,   20076, 46168, 33756, 17872,
                25244, 38760, 45856, 15332, 55040, 18972, 3368,  14320, 42916,
                20984, 25064, 50260, 4700,  60864, 53168, 1548,  29020, 41864,
                21508, 52092, 60628, 6668,  55136, 20224, 1100,  27840, 51928,
                32636, 40052, 35496, 14724, 1144,  38648, 10728, 20676, 6776,
                45996, 9904,  51172, 16072, 25904, 41628, 33088, 48388, 16332,
                62244, 23168, 31512, 59932, 25912, 38788, 2888,  30044, 55584,
                7752,  27524, 14268, 41388, 17616, 54132, 42928, 62284, 4048,
                55252, 44360, 9928,  36752, 59612, 3136,  64796, 32740, 58116,
                41692, 224,   33448, 11424, 56472, 30356, 1524,  5928,  43188,
                63076, 28872, 5148,  60436, 30444, 44492, 51644, 28756, 63828,
                35408, 56836,
            },
            {
                39276, 19420, 28716, 36100, 59000, 53504, 11684, 50112, 64504,
                14420, 58204, 44084, 54232, 20836, 47168, 60216, 18184, 55500,
                3856,  24920, 59348, 53392, 9232,  47624, 42080, 5624,  24936,
                8352,  59916, 30320, 16112, 42204, 60396, 13544, 28408, 43048,
                2656,  62740, 29548, 7956,  33392, 50836, 23572, 28732, 59004,
                37508, 848,   13524, 41232, 19884, 44008, 12368, 63192, 8604,
                45944, 34616, 24724, 16608, 29364, 35696, 63544, 46032, 17564,
                4056,  21612, 8836,  46340, 54680, 60712, 32988, 50032, 64664,
                28952, 1604,  35208, 39512, 5432,  64484, 22076, 59284, 27196,
                42324, 45860, 54944, 1676,  51688, 13304, 43900, 62148, 22624,
                50124, 3780,  56784, 33476, 63644, 1396,  47516, 30840, 16732,
                35260, 64128, 18844, 52492, 30292, 15384, 46536, 19100, 9952,
                49792, 36864, 16024, 46768, 6936,  40128, 20916, 57624, 12340,
                50896, 20132, 41612, 22856, 11652, 56600, 6700,  14048, 48300,
                1976,  16456,
            },
            {
                44916, 50344, 8760,  2348,  14800, 21212, 3496,  43496, 37724,
                9348,  22260, 33532, 38668, 57420, 3128,  36792, 27264, 34416,
                20544, 49352, 40048, 27856, 14400, 34608, 54996, 19956, 51936,
                33324, 39692, 10916, 52608, 35672, 24388, 55248, 5360,  58456,
                47968, 11768, 19736, 60040, 44036, 40252, 63320, 9040,  47844,
                32692, 61540, 30032, 55736, 26468, 48200, 35364, 16224, 54872,
                30880, 252,   65516, 47556, 50992, 40936, 9364,  53800, 23848,
                60364, 48532, 58248, 25316, 28772, 5516,  23020, 13912, 40580,
                17700, 59556, 47952, 12916, 54804, 45048, 10588, 36052, 3872,
                13100, 7548,  34924, 17868, 29136, 57256, 5904,  47904, 16204,
                36216, 59656, 45756, 23740, 37668, 11948, 22124, 38744, 7912,
                24692, 28996, 13536, 40320, 7404,  23704, 55232, 37936, 13680,
                25804, 63580, 52844, 28168, 62424, 23828, 48236, 33036, 26076,
                38452, 7780,  55332, 35308, 16856, 63172, 37068, 42484, 23288,
                59892, 32972,
            },
            {
                25812, 12136, 60828, 42600, 48808, 34432, 56444, 25880, 18220,
                52888, 2204,  26312, 15676, 6212,  41852, 10012, 64180, 44624,
                12620, 61160, 35812, 6916,  63148, 22816, 304,   38128, 64372,
                14568, 56740, 21252, 63068, 4156,  9712,  49652, 37708, 21680,
                32496, 53768, 36388, 26304, 1896,  12252, 17072, 53896, 3972,
                18728, 52372, 7216,  16912, 64888, 3600,  22384, 59588, 49732,
                19592, 5800,  39096, 11588, 22152, 3304,  15020, 33540, 38776,
                11836, 30400, 2232,  41228, 17016, 36828, 44596, 56516, 4168,
                32092, 53032, 24180, 20044, 31100, 2248,  18068, 50264, 53948,
                20844, 39604, 64620, 10364, 37632, 21104, 34228, 25288, 41224,
                11076, 29640, 18800, 8472,  52324, 27008, 56048, 59408, 50396,
                45832, 356,   57300, 47924, 62188, 34772, 43332, 1672,  60728,
                31228, 22296, 3468,  42672, 13420, 35392, 54768, 9400,  64352,
                15660, 30796, 61404, 47776, 26892, 4272,  31716, 19500, 9200,
                52672, 6416,
            },
            {
                21156, 57064, 38340, 30640, 24332, 63796, 6844,  30016, 59484,
                35144, 62920, 48364, 52000, 61372, 29060, 23532, 53236, 7500,
                30588, 1156,  43008, 18784, 56636, 30080, 44456, 11804, 27080,
                48184, 2160,  29264, 47024, 40748, 17628, 30716, 64680, 15532,
                8280,  41580, 4648,  49872, 56580, 30648, 35380, 25408, 42216,
                11500, 45728, 35092, 39880, 9852,  32012, 37904, 6884,  26144,
                43276, 14648, 58976, 27072, 57304, 43656, 60956, 26256, 5768,
                55624, 44216, 19872, 65176, 52304, 9552,  62576, 47564, 26552,
                11232, 38500, 7700,  62292, 57376, 37696, 25112, 62844, 32336,
                47416, 56760, 24200, 44852, 49720, 60248, 8896,  54752, 65036,
                2776,  43660, 61156, 4916,  48832, 32588, 2952,  15596, 10524,
                61352, 33264, 21600, 5128,  27176, 10740, 20384, 51880, 5304,
                45816, 57104, 9144,  19756, 59512, 4504,  17100, 45124, 1104,
                42896, 52660, 3068,  9948,  44220, 54208, 50028, 58500, 40556,
                29776, 65352,
            },
            {
                504,   15616, 52052, 4328,  18484, 9944,  39864, 15948, 46268,
                5184,  40176, 10992, 19708, 32128, 13088, 49588, 17168, 39600,
                57528, 22356, 52532, 11032, 46880, 3940,  50664, 60844, 7640,
                18488, 43396, 35056, 7264,  26428, 58740, 43868, 212,   51780,
                27796, 61900, 17660, 23120, 46660, 7152,  58292, 38784, 62696,
                21448, 27428, 57836, 23736, 51344, 44752, 57036, 11796, 40344,
                62652, 32712, 52740, 36656, 7680,  31308, 19084, 50072, 63144,
                36208, 15584, 49476, 33848, 12888, 30704, 468,   19744, 34964,
                61208, 43580, 3616,  27992, 46256, 14424, 40832, 5948,  29220,
                15848, 1304,  27972, 5220,  15172, 40596, 580,   19152, 32116,
                14596, 53080, 25736, 39328, 13184, 64312, 44260, 36236, 40740,
                18332, 54440, 42892, 38048, 16800, 49364, 59220, 29068, 40400,
                17036, 34572, 38908, 49176, 30920, 51968, 37748, 28932, 58720,
                22252, 36148, 25180, 18668, 33688, 14620, 2216,  25060, 12784,
                47140, 36480,
            },
            {
                43200, 27252, 33348, 46932, 61432, 44176, 54564, 51188, 12564,
                23132, 28356, 45168, 192,   42852, 35204, 58584, 2568,  47424,
                26152, 15064, 65208, 33260, 38876, 24564, 15704, 35940, 31756,
                59160, 23176, 56036, 12540, 53028, 20828, 34420, 11160, 23928,
                45352, 54960, 13000, 39372, 65336, 20172, 14628, 50540, 6352,
                55092, 1700,  13688, 60572, 2792,  15304, 28448, 53928, 1820,
                22808, 45452, 4312,  17056, 47092, 54436, 1420,  40276, 9636,
                22596, 3752,  26928, 7244,  56236, 39048, 24956, 51508, 14764,
                54496, 21860, 49628, 16868, 34196, 51868, 9280,  58840, 43264,
                11040, 59772, 33600, 52912, 63112, 31168, 27216, 51488, 37240,
                47448, 21968, 34488, 57192, 16992, 29416, 20692, 6068,  25404,
                28396, 3532,  12672, 65476, 55740, 32864, 7976,  24528, 64168,
                12044, 55128, 26984, 764,   65172, 24308, 11732, 19332, 49932,
                5592,  13128, 59836, 40492, 64036, 21844, 38840, 62052, 5776,
                17548, 55568,
            },
            {
                10820, 62580, 7808,  21660, 13408, 34748, 25120, 1088,  32036,
                60428, 56232, 16716, 63560, 54932, 22024, 5308,  62624, 31644,
                37248, 8564,  49156, 27696, 6132,  58212, 20404, 41448, 53688,
                4800,  39240, 49576, 16044, 62688, 3608,  47836, 39748, 57636,
                33180, 2164,  29696, 8904,  33592, 44324, 484,   28456, 32568,
                46792, 37376, 42688, 30460, 48788, 21184, 63880, 18228, 35680,
                51236, 9752,  29328, 21084, 64116, 24764, 13520, 45252, 28928,
                58076, 52576, 41712, 61704, 21760, 43964, 59964, 6476,  41128,
                9084,  31224, 56628, 11616, 65472, 544,   22820, 54692, 20136,
                38100, 49380, 18368, 42276, 12324, 22320, 45568, 59088, 10500,
                7180,  63248, 2068,  41792, 9244,  46576, 58360, 53216, 62760,
                48068, 51232, 30948, 23992, 1252,  44432, 14292, 36676, 47420,
                6384,  21628, 44792, 14980, 41400, 7268,  46440, 62168, 39808,
                31880, 55632, 45908, 51140, 7300,  30144, 56144, 45556, 28368,
                34088, 49600,
            },
            {
                24024, 40036, 52988, 58420, 28268, 3892,  57404, 38484, 20392,
                49064, 7820,  37392, 25164, 9016,  39020, 45540, 18744, 11380,
                43624, 54504, 3300,  17552, 42456, 52268, 63440, 9136,  13412,
                28404, 64940, 1404,  32444, 38248, 28984, 61036, 14364, 5404,
                19844, 37804, 59824, 50908, 24860, 53548, 60304, 10708, 23916,
                16192, 64584, 19416, 5376,  34056, 40844, 8124,  47116, 31048,
                13888, 61472, 55456, 41880, 11164, 33008, 59920, 35420, 16580,
                47536, 31824, 11308, 36532, 15104, 2736,  49120, 17436, 27824,
                63660, 1704,  36456, 42292, 25796, 30372, 44860, 35192, 4296,
                64268, 23748, 8260,  36460, 3064,  56128, 5612,  16872, 42808,
                28596, 55084, 19652, 31368, 51464, 24356, 956,   34392, 14068,
                7880,  37040, 59904, 10044, 40636, 52140, 19444, 57524, 2828,
                51204, 32052, 58728, 9672,  53284, 33124, 57136, 2400,  25940,
                9212,  16828, 28292, 920,   11888, 48824, 35512, 10072, 20580,
                60440, 4712,
            },
            {
                31076, 19160, 1480,  36820, 42284, 49572, 17624, 65184, 10436,
                43112, 33996, 4404,  30360, 52564, 14216, 28216, 56192, 24632,
                60340, 21064, 34812, 61560, 12580, 30672, 1244,  25924, 47888,
                43928, 17748, 10424, 24928, 45584, 7580,  18380, 51424, 26816,
                63476, 48316, 16056, 42032, 3372,  17540, 36756, 41376, 49312,
                58916, 8916,  52196, 12452, 55364, 61148, 24176, 3824,  59448,
                38224, 25752, 788,   48928, 38836, 3564,  51080, 21392, 6112,
                63676, 824,   18684, 54632, 46744, 29524, 33240, 57488, 37712,
                46452, 24208, 18804, 48404, 6904,  60656, 17368, 49996, 13440,
                27700, 46404, 53804, 60348, 26384, 48756, 33740, 61808, 23520,
                39056, 12904, 49548, 5548,  59720, 37388, 11488, 42220, 22864,
                44700, 16916, 20408, 46828, 28100, 5224,  61180, 29672, 42364,
                25920, 63128, 18424, 36736, 28224, 20980, 16408, 35460, 53812,
                44404, 64752, 37704, 20252, 58316, 24428, 16572, 1752,  41776,
                51432, 14328,
            },
            {
                45164, 64496, 47756, 15968, 9564,  30624, 6756,  45780, 27032,
                14876, 61648, 50640, 20052, 47308, 64844, 33916, 7184,  50984,
                596,   40288, 29556, 56088, 22984, 46512, 38016, 33504, 57240,
                21436, 51972, 35896, 60284, 55392, 22316, 41120, 56532, 9788,
                43732, 22792, 6576,  12228, 57492, 30612, 63032, 20536, 5840,
                34912, 29584, 22600, 45388, 26496, 37012, 11080, 44208, 50176,
                20284, 7492,  34668, 16308, 57208, 27256, 8780,  53992, 43676,
                23876, 38708, 60060, 26184, 4856,  64536, 10292, 20596, 12588,
                52464, 4672,  58300, 14240, 54080, 38868, 10020, 57540, 32024,
                40360, 2284,  15536, 31640, 19836, 39572, 13648, 50760, 3704,
                57292, 34952, 44620, 15240, 27596, 64748, 17764, 56012, 32396,
                58124, 5724,  54088, 34804, 62968, 23052, 16400, 34160, 8464,
                13124, 39104, 1972,  49460, 60288, 43648, 4396,  48920, 14076,
                22764, 6228,  33304, 61080, 43644, 39568, 53176, 63028, 25872,
                57824, 38228,
            },
            {
                54052, 11636, 26636, 60824, 55628, 21932, 59780, 35800, 53360,
                23004, 2288,  39564, 58900, 11016, 1796,  41944, 16504, 36660,
                47880, 14028, 5124,  44548, 9980,  51380, 6520,  16108, 62144,
                3188,  30100, 42020, 5688,  49368, 12296, 34824, 31068, 2648,
                36584, 29088, 55372, 34576, 26660, 47280, 4304,  13600, 53280,
                43400, 3024,  40020, 57936, 144,   17388, 52748, 28572, 15396,
                57660, 42804, 64956, 31556, 45696, 19308, 62476, 36564, 12448,
                28272, 50024, 8676,  43304, 22132, 35752, 50292, 5556,  43132,
                32252, 61336, 40452, 28940, 33672, 21100, 2668,  24568, 61864,
                7364,  55664, 43128, 10200, 63732, 44568, 7668,  18600, 30836,
                25956, 240,   62112, 22216, 40380, 7984,  47464, 29508, 3900,
                63788, 26896, 39888, 2308,  12824, 56156, 38124, 49844, 54556,
                45796, 23708, 55872, 5168,  12348, 24496, 62832, 29936, 59236,
                40908, 50688, 26616, 3728,  14688, 30960, 5080,  47316, 32464,
                8768,  3000,
            },
            {
                29028, 35248, 23092, 5852,  33112, 50772, 12288, 40892, 6236,
                29464, 55852, 17488, 43564, 27220, 23628, 57480, 30480, 61324,
                19972, 53324, 26000, 64556, 35824, 19364, 60444, 40348, 24104,
                8504,  54588, 14036, 19856, 27844, 656,   64756, 46764, 15088,
                60764, 52712, 18320, 64528, 44136, 9180,  39212, 24328, 56624,
                27088, 63336, 15436, 49752, 31220, 64416, 6016,  32984, 40116,
                2600,  11892, 52064, 23008, 4760,  13980, 41656, 30760, 3292,
                56100, 15624, 34092, 52884, 13344, 55764, 40040, 62356, 26868,
                15912, 22068, 9284,  3476,  64140, 46740, 52384, 42632, 36248,
                18196, 22208, 50204, 28520, 1316,  24476, 58676, 36652, 65432,
                52260, 9052,  47896, 32580, 54876, 2608,  52736, 20156, 38168,
                10548, 50892, 15040, 31036, 48304, 43828, 10528, 336,   20444,
                60580, 15792, 30508, 34344, 40732, 47396, 8752,  38000, 364,
                11504, 17716, 46360, 54492, 9384,  57232, 19148, 12556, 36616,
                22156, 17812,
            },
            {
                56572, 41608, 3664,  44352, 38904, 18760, 25888, 3092,  63504,
                48584, 32608, 12768, 7864,  35244, 52916, 4612,  45828, 11624,
                8256,  33212, 58276, 16644, 2612,  28228, 55008, 12704, 49960,
                44984, 37408, 62980, 33012, 58008, 44304, 17476, 24684, 50220,
                5572,  10976, 39616, 724,   21472, 52080, 60912, 32408, 48292,
                18356, 7320,  33804, 10004, 20668, 46476, 56336, 22188, 62632,
                25376, 46788, 29124, 8296,  60908, 50320, 24624, 58640, 46704,
                65428, 20416, 40456, 2040,  29076, 17652, 24640, 344,   46024,
                53820, 35528, 50592, 44508, 26172, 12372, 15944, 29520, 48616,
                11384, 62904, 37976, 57672, 35060, 52996, 47136, 11276, 40700,
                16036, 42564, 19408, 11884, 36500, 26332, 43980, 13452, 49076,
                24608, 43340, 59336, 21228, 6828,  25864, 64972, 32312, 27940,
                41456, 7112,  51960, 64072, 17904, 27692, 54720, 20488, 32316,
                56716, 63224, 35024, 21544, 37448, 49236, 28076, 65156, 44504,
                59900, 48764,
            },
            {
                7292,  62452, 52416, 14512, 63984, 47028, 54600, 15924, 43956,
                20696, 38080, 62692, 46580, 60516, 18836, 38388, 22196, 63372,
                28036, 43888, 38160, 22468, 49408, 42264, 31580, 128,   34464,
                26992, 16976, 4472,  51564, 40000, 8208,  36940, 59328, 21360,
                41860, 33028, 25352, 48976, 14016, 35936, 16844, 1600,  11284,
                37424, 44928, 58812, 24976, 38440, 13576, 41728, 8968,  35572,
                53764, 17816, 37616, 55824, 33620, 39444, 444,   17892, 10084,
                32996, 7040,  45032, 62020, 58004, 47884, 8516,  59192, 30468,
                6616,  11696, 57772, 19168, 37556, 54936, 6284,  60376, 180,
                33268, 25516, 4128,  13240, 17376, 5756,  20892, 29052, 4236,
                23192, 55800, 60960, 5008,  58400, 16272, 62816, 33944, 57784,
                612,   18148, 35932, 61724, 40680, 52208, 17528, 58144, 4440,
                35924, 48288, 10704, 21812, 57576, 5752,  14468, 51428, 43032,
                24664, 7728,  29324, 1568,  62036, 41276, 24048, 5900,  132,
                39900, 15144,
            },
            {
                10828, 32156, 20636, 27852, 624,   9172,  31268, 35344, 58216,
                10224, 1152,  24716, 28856, 5780,  50004, 13796, 54924, 40764,
                2884,  50632, 5908,  12308, 56372, 8980,  47600, 21300, 65220,
                56672, 10764, 47320, 22540, 25928, 11628, 53976, 3740,  28440,
                56052, 45864, 62384, 31128, 7448,  59208, 28556, 41404, 62500,
                22348, 54456, 2116,  51256, 61548, 3572,  27800, 48800, 4876,
                14088, 60184, 1576,  21264, 11376, 44936, 27632, 52724, 38028,
                23396, 49612, 26696, 4448,  32096, 37488, 12328, 20036, 38872,
                65124, 41984, 24084, 1284,  62264, 31312, 40780, 21872, 57096,
                44672, 53508, 47584, 41588, 31924, 60848, 43364, 56652, 33276,
                49040, 37824, 28048, 31084, 45448, 23852, 9492,  41332, 5512,
                31772, 53332, 8804,  28184, 2980,  34040, 23188, 46040, 12680,
                62088, 24816, 43544, 1064,  38608, 45288, 35132, 65392, 3984,
                48248, 15628, 52728, 44976, 11344, 16900, 51904, 33776, 55152,
                30184, 25644,
            },
            {
                45520, 38436, 49504, 57004, 37160, 59580, 22980, 6988,  27536,
                49556, 54336, 14744, 57016, 42528, 31664, 728,   34380, 25216,
                59388, 15456, 30128, 63760, 39756, 25524, 59248, 15320, 7024,
                38712, 19252, 60944, 1920,  30496, 63900, 48372, 33840, 13532,
                8712,  16568, 2840,  56960, 23280, 53528, 44668, 4900,  50468,
                31472, 14516, 28172, 42788, 34764, 17604, 54316, 59364, 20064,
                39764, 30248, 43552, 49456, 16208, 62736, 57640, 5936,  14308,
                61140, 55368, 11600, 16484, 22480, 43484, 51052, 33736, 55188,
                17076, 28304, 47348, 34424, 8776,  17464, 49688, 27440, 9344,
                14948, 19608, 7196,  64340, 26936, 49940, 9704,  15140, 63364,
                2712,  13332, 8136,  50988, 39592, 1636,  54468, 20948, 28784,
                64780, 46108, 12408, 56256, 49228, 14752, 7988,  54140, 39012,
                19156, 56060, 32924, 29000, 61248, 11840, 23564, 30348, 18724,
                40148, 36588, 60528, 25764, 31780, 7144,  59468, 13568, 20436,
                50440, 61284,
            },
            {
                1768,  23216, 6268,  17568, 11620, 42376, 51108, 62080, 39032,
                17676, 33148, 40472, 21728, 9164,  63956, 17356, 48132, 10168,
                19424, 53228, 45932, 33488, 17908, 4804,  36204, 44064, 52432,
                28652, 41784, 32664, 55260, 43472, 16368, 20300, 41072, 61092,
                38192, 51616, 19880, 42716, 37524, 10336, 19048, 25988, 57532,
                7992,  20040, 65404, 5644,  11900, 23224, 44348, 31900, 10492,
                51624, 26432, 64412, 6380,  23888, 36000, 31864, 19636, 43012,
                28796, 39924, 35520, 63420, 53556, 6108,  60512, 25964, 2596,
                49136, 5384,  14120, 52852, 59356, 44180, 4720,  64924, 36628,
                40028, 29700, 56112, 21772, 38376, 804,   24212, 35928, 46140,
                26480, 54024, 21596, 64332, 18384, 47504, 61224, 36056, 50520,
                15208, 23616, 42084, 20248, 37720, 63292, 43016, 30048, 1632,
                49988, 15196, 8548,  52904, 16960, 49488, 2592,  55068, 9376,
                58364, 13028, 4544,  19696, 55504, 38472, 46804, 42112, 9256,
                36164, 16768,
            },
            {
                42660, 53276, 64808, 34480, 30084, 46208, 19912, 13548, 2860,
                44388, 65000, 4872,  51764, 36796, 26948, 55936, 43628, 61744,
                37208, 7872,  23228, 992,   61300, 54412, 12840, 31176, 23324,
                2928,  50180, 6008,  14404, 35736, 7412,  52640, 1192,  23688,
                30376, 65224, 27228, 5880,  33376, 47724, 64064, 13376, 35376,
                46728, 40544, 33024, 48720, 58520, 38120, 1512,  63736, 47472,
                34976, 3248,  56452, 41144, 9484,  53832, 2516,  47384, 51668,
                3584,  8088,  18636, 45844, 1624,  29304, 14908, 40640, 10564,
                22644, 61520, 30308, 37992, 21120, 12032, 33428, 23996, 52300,
                60048, 3028,  34284, 10924, 16244, 54376, 59444, 6180,  40312,
                17424, 58684, 43520, 33724, 4684,  14460, 27084, 10908, 6744,
                38332, 58636, 1756,  32032, 4860,  24944, 57564, 10604, 34828,
                27360, 64540, 4676,  40576, 59044, 26324, 37260, 61732, 46008,
                27648, 33728, 49856, 42940, 64580, 2124,  22584, 27872, 62812,
                3952,  58484,
            },
            {
                28748, 12908, 47128, 25416, 2088,  56068, 5788,  26244, 53436,
                29924, 10952, 19296, 59924, 46980, 12772, 22876, 4580,  31288,
                27372, 57984, 49672, 42912, 28480, 20612, 45600, 63220, 9760,
                60332, 57028, 25000, 46148, 62508, 27012, 57460, 47076, 5444,
                44128, 11420, 49452, 14716, 60272, 868,   29376, 39252, 3388,
                24196, 53444, 10064, 18524, 55576, 29872, 15528, 24828, 6908,
                17176, 21888, 13596, 31284, 46436, 26804, 13172, 64744, 21920,
                33516, 59776, 48704, 24220, 41716, 57356, 19624, 63716, 45192,
                35872, 57100, 42944, 2836,  26100, 55828, 47760, 1596,  18416,
                13004, 46264, 50904, 61556, 42044, 45392, 29428, 20600, 52664,
                30872, 1412,  11480, 25068, 38688, 53104, 32264, 57364, 44840,
                18720, 26560, 48232, 60680, 53200, 16188, 46636, 21576, 60132,
                42120, 23340, 47084, 19952, 34024, 6892,  42512, 14012, 20704,
                1308,  54000, 23664, 10388, 29168, 35352, 17792, 53612, 48272,
                24612, 33044,
            },
            {
                19020, 8884,  59696, 39536, 15476, 62996, 41200, 36392, 60776,
                47748, 24264, 34796, 1760,  30544, 7272,  39668, 52060, 14964,
                2784,  40500, 12224, 35140, 9104,  50776, 1948,  39492, 16216,
                34768, 18660, 11860, 38032, 21960, 9976,  39332, 32780, 18056,
                58588, 35128, 22064, 54832, 40044, 17500, 55760, 51404, 62000,
                16616, 59968, 676,   26732, 45912, 8176,  52112, 40812, 60312,
                43876, 53016, 38144, 61892, 18640, 58776, 40512, 37020, 10480,
                25932, 56500, 13892, 31392, 8880,  35148, 49748, 6856,  31292,
                53404, 18492, 7936,  49892, 63588, 15980, 41536, 58632, 31444,
                43212, 5564,  23744, 27996, 7804,  33204, 13068, 62460, 10092,
                47576, 35360, 61588, 49924, 8384,  59688, 19864, 41796, 4088,
                63712, 33412, 8132,  11780, 28820, 36096, 6772,  51612, 3432,
                12900, 36504, 57328, 30700, 10736, 47872, 22668, 52356, 32108,
                41248, 62720, 6604,  47180, 14956, 57372, 5344,  11984, 37964,
                7324,  56024,
            },
            {
                51240, 35644, 4652,  21408, 49916, 9416,  32592, 21732, 16144,
                7716,  56936, 50888, 41244, 16612, 62600, 58644, 34516, 44824,
                65360, 21444, 55212, 16700, 58696, 37536, 26408, 53860, 47256,
                28008, 41048, 49060, 644,   59412, 29432, 50088, 13872, 25748,
                53428, 8308,  2064,  45704, 31468, 24960, 7496,  21740, 11512,
                44228, 30296, 37492, 41940, 21564, 61436, 12808, 36476, 28136,
                57196, 10956, 25348, 50500, 1716,  5708,  29748, 55004, 16648,
                44500, 40,    38764, 64236, 51860, 12164, 26716, 39140, 652,
                15268, 25048, 11656, 39796, 29516, 6924,  35992, 9696,  26824,
                63424, 20332, 37080, 57704, 2488,  19044, 50736, 37540, 5108,
                23256, 55356, 41172, 16708, 28852, 46060, 4,     24204, 55076,
                13864, 52024, 39740, 44436, 19760, 65328, 40836, 32416, 18440,
                54292, 44512, 0,     14972, 51476, 60068, 3420,  64640, 8456,
                16156, 37136, 18948, 59988, 38896, 50268, 32196, 63992, 45172,
                15300, 40708,
            },
            {
                22752, 63592, 31316, 55420, 44520, 27508, 52248, 58448, 584,
                38680, 13148, 27864, 45180, 53932, 20628, 25488, 10512, 18392,
                47532, 6864,  25800, 31732, 4160,  63872, 13652, 22448, 7648,
                3416,  62900, 31260, 52892, 6684,  17412, 63836, 2744,  41220,
                60840, 37848, 28524, 62800, 10476, 43420, 49036, 37008, 33312,
                5528,  50152, 13788, 63668, 5920,  33960, 49640, 4564,  19188,
                312,   33456, 7884,  44676, 35280, 23140, 48308, 7708,  62392,
                50528, 28956, 21216, 6524,  18016, 60412, 22396, 55656, 47304,
                59760, 34476, 62296, 45880, 19560, 51512, 22688, 54812, 15228,
                38912, 49644, 54040, 11808, 65252, 46624, 25256, 56976, 43104,
                28060, 13668, 3568,  21416, 65092, 35716, 12868, 30660, 48816,
                36644, 17196, 22760, 57828, 800,   49484, 13768, 26180, 61756,
                29212, 8924,  63800, 25632, 39676, 18172, 29108, 36084, 24808,
                57740, 45432, 30828, 25952, 460,   42176, 23936, 20476, 29944,
                57972, 2664,
            },
            {
                49028, 43636, 13492, 37560, 18560, 2936,  11824, 46344, 30204,
                43440, 63332, 22440, 9556,  3548,  36904, 49648, 152,   28352,
                53796, 38044, 60928, 43988, 47840, 19184, 33200, 42868, 59812,
                55176, 20304, 14592, 44280, 34280, 42088, 54916, 23136, 48204,
                12356, 19264, 50720, 15576, 57180, 4364,  64876, 27100, 56348,
                19516, 58816, 25172, 53736, 17460, 30820, 56080, 23260, 65108,
                48480, 29352, 59948, 16520, 64016, 13744, 42452, 19448, 34356,
                4308,  41508, 54712, 36296, 46068, 2644,  43708, 29772, 9692,
                20608, 51072, 27548, 1428,  57520, 32420, 4384,  46972, 60988,
                384,   8288,  16812, 30868, 35784, 40728, 15464, 640,   63868,
                18100, 49000, 32616, 56812, 6820,  51460, 40404, 62164, 6004,
                27820, 2812,  62528, 31404, 25212, 9864,  56180, 43772, 5280,
                38692, 49180, 20484, 53300, 34460, 5296,  43384, 55776, 48852,
                11132, 4256,  52004, 13036, 56220, 9404,  61132, 3456,  51864,
                11008, 26884,
            },
            {
                17028, 260,   29368, 7628,  59436, 65268, 35236, 24604, 19260,
                54864, 4260,  33364, 58112, 14580, 31688, 56020, 42312, 62972,
                33332, 13388, 2524,  23284, 10752, 56972, 5576,  50056, 29476,
                35704, 9304,  25584, 57248, 21260, 11296, 36700, 5276,  29808,
                33744, 6448,  42236, 23836, 35684, 20480, 13272, 41168, 2048,
                46132, 9044,  35312, 3624,  43640, 11068, 38988, 45720, 15048,
                41496, 53664, 20472, 39468, 55596, 31992, 52120, 57952, 24620,
                12780, 60896, 15544, 27028, 57860, 33400, 14080, 65464, 4276,
                37880, 42304, 5720,  14352, 36872, 64832, 11468, 42148, 28972,
                33896, 44088, 25980, 48252, 22204, 6528,  53260, 30332, 34436,
                8416,  59196, 43992, 38148, 26556, 15852, 9196,  21588, 56312,
                46496, 10812, 41368, 54004, 4560,  37672, 33948, 17012, 59080,
                23716, 12184, 31352, 6728,  61468, 46356, 13380, 960,   20448,
                28284, 42748, 61592, 21580, 33424, 47664, 16356, 36924, 43084,
                34336, 61496,
            },
            {
                38508, 53868, 47696, 25272, 41348, 15092, 48456, 6888,  61264,
                36544, 17204, 48872, 40272, 64592, 46856, 5508,  23044, 16488,
                8688,  50952, 29828, 40420, 52404, 27568, 38996, 16988, 12112,
                46112, 39712, 65204, 2184,  47684, 27184, 61880, 16136, 45484,
                64624, 53836, 59120, 3252,  47264, 31832, 52960, 16848, 61536,
                29912, 22720, 39932, 48916, 28044, 60964, 2180,  26228, 8892,
                35432, 12220, 2848,  24924, 5060,  9832,  27436, 1172,  39080,
                46968, 32224, 5236,  49516, 10272, 24688, 40744, 52632, 17496,
                31544, 58432, 23492, 55108, 44380, 17432, 24492, 53196, 20568,
                13416, 57892, 62336, 3260,  56072, 10504, 60892, 21036, 39336,
                51696, 23880, 11716, 1340,  47080, 53524, 32136, 43696, 17964,
                60224, 34080, 50596, 15940, 44996, 63808, 21912, 53004, 1848,
                47044, 35712, 56704, 41856, 16556, 22936, 59224, 32992, 38816,
                63728, 15668, 35548, 5372,  40600, 54372, 28320, 6288,  59288,
                19572, 8536,
            },
            {
                23752, 62852, 11332, 57000, 33480, 21220, 54320, 40120, 10596,
                51368, 27140, 7400,  24248, 18888, 28968, 12272, 39772, 57204,
                44744, 20168, 59648, 15824, 64740, 35400, 424,   62588, 23700,
                52188, 4480,  17864, 50744, 31932, 8024,  58152, 51676, 32,
                20796, 26392, 9776,  39424, 27812, 60292, 8392,  37612, 50496,
                11828, 54716, 64280, 15156, 20728, 57348, 51396, 18420, 62456,
                31436, 59184, 46560, 50060, 36264, 61340, 45456, 17064, 64996,
                9056,  22452, 53124, 42720, 63088, 20056, 7556,  48984, 26160,
                61640, 10612, 48468, 7980,  29896, 39416, 2016,  59680, 36360,
                6272,  51444, 18508, 33140, 42248, 27556, 48860, 12852, 44544,
                5536,  61748, 29652, 19268, 63600, 23304, 58104, 1964,  38072,
                25456, 7600,  21108, 29196, 12076, 47908, 27392, 8436,  40652,
                15044, 63300, 3076,  27168, 10268, 49944, 26216, 52164, 9028,
                54796, 2604,  50188, 26596, 18052, 65436, 12100, 24676, 46464,
                55648, 32172,
            },
            {
                4092,  45344, 19780, 5396,  51492, 3652,  27880, 31612, 1248,
                42400, 12848, 60124, 44344, 1472,  53152, 61020, 34968, 26440,
                1688,  36140, 46472, 5048,  9020,  21696, 56244, 44768, 7064,
                32504, 60096, 28176, 37888, 13988, 43692, 24316, 40268, 32284,
                37740, 14188, 49552, 17944, 55960, 928,   45072, 24516, 4520,
                18432, 32360, 432,   45436, 8104,  33096, 37124, 6072,  54264,
                43072, 22508, 7620,  28736, 41056, 12528, 21384, 30532, 49712,
                36004, 58712, 18396, 29680, 736,   37300, 56588, 34200, 2360,
                44732, 15688, 35456, 20000, 62128, 50012, 9632,  26764, 47608,
                40912, 23840, 9392,  46204, 14636, 37004, 57408, 3836,  26620,
                17004, 36516, 55996, 41992, 4464,  34700, 12084, 49392, 30040,
                14024, 54532, 39456, 61856, 6024,  19028, 60260, 32816, 51272,
                29536, 18928, 45204, 54640, 37680, 65148, 6172,  41096, 18884,
                31016, 23868, 46280, 59732, 8116,  31812, 38456, 50692, 1160,
                13716, 40856,
            },
            {
                27660, 15840, 36116, 29736, 44068, 63400, 13676, 58272, 23088,
                64136, 34016, 55652, 30456, 38152, 9924,  21488, 48688, 7236,
                62048, 54268, 24136, 32220, 49296, 42736, 12948, 26484, 54044,
                19896, 42028, 10516, 55468, 61392, 3348,  18980, 11084, 56596,
                7388,  44460, 62404, 34108, 12416, 21996, 64156, 34740, 42972,
                57932, 38296, 25732, 59804, 41976, 12932, 23976, 47432, 27848,
                828,   13968, 52348, 63684, 16236, 57392, 53384, 3200,  42012,
                25816, 4228,  11924, 39392, 48136, 15192, 60932, 12064, 22424,
                55772, 41440, 28384, 53520, 4772,  32272, 55832, 15736, 63388,
                3764,  29544, 38208, 64320, 980,   31648, 19064, 62988, 46836,
                53128, 32432, 8852,  49912, 15252, 28040, 61112, 41472, 4180,
                64828, 44604, 420,   56692, 35988, 42416, 3876,  55688, 11108,
                24004, 60860, 7828,  33188, 20592, 30228, 2256,  44960, 14172,
                58516, 37480, 11660, 42052, 52952, 2956,  44288, 18820, 30232,
                63988, 52504,
            },
            {
                48048, 57996, 60816, 39752, 9512,  18076, 35596, 44924, 47928,
                16320, 20388, 4144,  50336, 14996, 57668, 43308, 17288, 31552,
                11336, 41032, 13844, 58604, 28560, 18132, 33972, 60808, 37252,
                14668, 48620, 1028,  22048, 46568, 35948, 29404, 49284, 60636,
                22244, 28592, 54480, 5340,  41968, 30664, 51832, 13812, 27960,
                48052, 9964,  53244, 5480,  29384, 50588, 64720, 16264, 39996,
                57056, 19812, 38112, 32728, 4748,  24128, 34384, 8252,  56352,
                14892, 61516, 45076, 54284, 23400, 31988, 27600, 46164, 38844,
                7136,  63932, 88,    25044, 46196, 13060, 42800, 34864, 19348,
                52408, 11988, 59908, 54348, 25116, 50964, 8560,  23444, 40736,
                2244,  13628, 21684, 59108, 37324, 45872, 8168,  20852, 51224,
                35036, 17060, 23488, 31996, 49168, 25360, 15556, 37932, 44048,
                636,   49760, 39304, 12632, 48668, 15728, 56140, 34620, 62156,
                48320, 5932,  21012, 33720, 15188, 23264, 60868, 57380, 36344,
                22200, 7848,
            },
            {
                33388, 2912,  12996, 24544, 48856, 56212, 2556,  26444, 6412,
                53084, 37460, 9048,  46632, 26260, 32888, 5856,  65128, 52144,
                28196, 19336, 4008,  37156, 53516, 2188,  50620, 9804,  4984,
                29868, 57988, 25224, 33880, 16080, 64292, 53488, 4724,  41652,
                16676, 1944,  36788, 25036, 47944, 9128,  58700, 20260, 6840,
                61688, 16084, 22004, 35604, 18752, 56584, 10496, 4192,  34300,
                61656, 45684, 25728, 10352, 48520, 60688, 44588, 38568, 19488,
                51092, 28360, 34648, 8728,  64564, 6544,  3676,  52832, 17744,
                30652, 13632, 49684, 37164, 18480, 59540, 23156, 2628,  58164,
                31600, 42024, 21280, 5676,  16948, 43380, 58784, 34164, 54840,
                28640, 65196, 43948, 25580, 5696,  55180, 18152, 25020, 57156,
                10508, 28232, 52544, 8356,  12584, 58612, 30312, 64960, 20988,
                34448, 58348, 25528, 4436,  42344, 60236, 22800, 10832, 25480,
                29120, 204,   56656, 63120, 27408, 9764,  48368, 4832,  16316,
                10880, 42984,
            },
            {
                26040, 20220, 51740, 6796,  32344, 21700, 38700, 60668, 11740,
                41324, 29564, 61960, 22484, 59144, 40824, 2988,  23776, 37792,
                56356, 47632, 63744, 44372, 8016,  61940, 40540, 24380, 46352,
                63920, 39412, 51804, 6704,  42724, 9156,  26140, 13436, 33712,
                45924, 65504, 51552, 19384, 60080, 16240, 39628, 1740,  55228,
                31564, 40448, 46552, 63000, 1800,  38200, 43684, 22256, 30568,
                8812,  3500,  54808, 42704, 19056, 2076,  28080, 11408, 63616,
                32676, 2448,  20976, 58236, 16560, 43756, 50272, 60196, 35152,
                57556, 21388, 54592, 10076, 63216, 6372,  38980, 27608, 46616,
                7616,  15096, 35332, 49332, 28432, 39308, 11608, 4568,  15732,
                37776, 10180, 51032, 1268,  30616, 39532, 63824, 32192, 46736,
                5352,  40996, 59840, 45768, 22036, 2588,  53204, 46988, 6880,
                13900, 53512, 17756, 63976, 28204, 52320, 38164, 7240,  53680,
                40412, 17052, 50936, 43844, 39264, 54748, 34528, 28992, 40340,
                61876, 55264,
            },
            {
                37808, 64992, 45060, 40772, 62320, 15216, 54128, 49928, 32940,
                24016, 56288, 704,   17828, 35832, 12312, 50848, 44888, 15072,
                360,   9584,  22308, 26748, 15636, 20940, 30712, 56900, 16852,
                3156,  19192, 12396, 59616, 30924, 47992, 20656, 38736, 58428,
                23980, 8312,  12148, 29504, 3012,  33340, 45492, 36368, 50192,
                24012, 4264,  13108, 49828, 32140, 26088, 53744, 48164, 59736,
                50932, 16864, 36276, 65508, 31104, 51976, 15740, 47364, 23764,
                6056,  42696, 48768, 36804, 26464, 40088, 19796, 9568,  24912,
                3232,  45508, 40464, 33308, 29396, 44244, 55208, 11052, 50756,
                65500, 25572, 45104, 57132, 2316,  63832, 46304, 22168, 61524,
                47996, 18964, 33508, 60616, 16340, 48952, 12872, 2972,  36272,
                61764, 15000, 19512, 36960, 63308, 40156, 18080, 9984,  26844,
                41140, 31188, 45956, 8996,  35296, 1216,  47200, 18528, 44248,
                64736, 12804, 30428, 7340,  19372, 1572,  13956, 45260, 24504,
                50100, 1416,
            },
            {
                11912, 16804, 28144, 57152, 588,   30160, 8232,  19276, 3920,
                64952, 14392, 43604, 51672, 7096,  63204, 19888, 30004, 60136,
                35004, 42208, 32380, 59672, 48796, 35480, 12104, 43488, 53268,
                33552, 27708, 45008, 36984, 3948,  54244, 62932, 784,   50524,
                31148, 55684, 39976, 43752, 62640, 56476, 22292, 27124, 11712,
                65064, 43204, 28672, 8652,  60620, 14888, 6636,  18216, 11960,
                40868, 29216, 23220, 12436, 7220,  58452, 35428, 55120, 39904,
                60008, 14052, 53036, 11020, 1228,  56908, 32876, 12956, 42844,
                65120, 7944,  26544, 14476, 1584,  49116, 15900, 20920, 36944,
                496,   53856, 10048, 32784, 13884, 19732, 30864, 52096, 26176,
                5808,  57604, 41160, 23536, 53224, 8800,  43588, 22368, 53692,
                26536, 50000, 1928,  32968, 6336,  28508, 50680, 35532, 57572,
                61380, 2044,  23204, 56428, 12048, 21196, 31920, 59124, 4456,
                33384, 22056, 36424, 61108, 25756, 50384, 64596, 56420, 6348,
                19120, 31804,
            },
            {
                53748, 4004,  35416, 10116, 23568, 46936, 36380, 42900, 27304,
                46236, 9356,  39068, 30816, 26876, 47436, 56924, 8448,  25564,
                49812, 17612, 55320, 5968,  39216, 1312,  65292, 6808,  22952,
                10252, 55552, 62568, 23432, 17484, 40712, 29032, 11028, 19052,
                5924,  15432, 48684, 25968, 4796,  14272, 53100, 7416,  17468,
                57268, 34296, 20008, 54892, 23516, 41260, 57840, 36760, 26792,
                62864, 1912,  56160, 49596, 41660, 26296, 21320, 436,   9604,
                29860, 18472, 64060, 31180, 22456, 47500, 62748, 29048, 38096,
                48968, 16664, 58988, 51728, 61896, 24232, 34168, 60612, 29964,
                18236, 40196, 23544, 62308, 37908, 7532,  55920, 35656, 140,
                44660, 29960, 12156, 6648,  36824, 28276, 58996, 17516, 40208,
                11708, 30260, 44124, 57816, 54200, 13368, 44600, 22648, 3936,
                48996, 15808, 37436, 43704, 62896, 50296, 14680, 27476, 9992,
                49620, 55908, 3576,  45444, 11392, 32392, 21988, 36936, 9444,
                60000, 47360,
            },
            {
                21864, 58472, 42616, 50608, 13748, 55000, 63380, 16928, 52716,
                34232, 20292, 60188, 54836, 2800,  33644, 14008, 40328, 4992,
                62028, 11604, 28676, 52444, 14228, 58384, 26188, 47512, 36692,
                41212, 236,   49280, 7740,  14064, 57176, 25104, 36088, 44932,
                61824, 33884, 21236, 37244, 10544, 32080, 41732, 60600, 47400,
                160,   38956, 50876, 3620,  45304, 33764, 900,   47000, 20784,
                5428,  33692, 46376, 15328, 38420, 4284,  62712, 43592, 50444,
                24744, 45524, 38752, 4344,  54844, 15236, 6688,  52284, 1888,
                22712, 31980, 36436, 20268, 4108,  41740, 8480,  56056, 4996,
                43972, 58808, 3488,  50604, 26932, 42976, 59792, 17428, 39736,
                14572, 54584, 63996, 20396, 47224, 62536, 33808, 648,   56032,
                7028,  65304, 23200, 9452,  25484, 16756, 62344, 32184, 39060,
                19940, 52592, 28964, 5628,  26116, 38820, 54644, 41752, 61504,
                24552, 38532, 15844, 52792, 59300, 41764, 2920,  16876, 43316,
                27932, 39284,
            },
            {
                5988,  30636, 62032, 19748, 26832, 39160, 4936,  11136, 58080,
                1936,  48712, 23540, 11476, 17092, 38468, 22976, 53828, 43944,
                20688, 37496, 2540,  23788, 45120, 19248, 31916, 51300, 15860,
                60920, 21048, 30668, 34784, 51652, 46612, 3440,  59880, 41272,
                53148, 2100,  57340, 63444, 51408, 46412, 19824, 37608, 30556,
                10428, 25684, 13920, 62564, 17704, 11396, 64692, 31072, 52104,
                43920, 59152, 19396, 8972,  60484, 32792, 17688, 13328, 57116,
                33800, 7392,  59472, 26908, 42164, 36028, 44968, 18944, 60432,
                56116, 5368,  47336, 54764, 12152, 37756, 46000, 26372, 13756,
                48924, 31816, 21344, 16228, 46892, 4268,  11252, 23988, 50240,
                8328,  34704, 26996, 43080, 6044,  14704, 24948, 51500, 45900,
                35544, 18604, 48276, 38672, 42744, 176,   56108, 5832,  12204,
                64360, 8632,  33868, 60016, 17880, 2564,  7868,  34880, 19440,
                860,   47544, 28788, 8212,  18756, 27332, 48640, 61228, 33568,
                51888, 14480,
            },
            {
                25012, 45892, 8100,  34148, 2220,  59344, 31984, 24468, 40520,
                29820, 6600,  36468, 62532, 50532, 45708, 64848, 1024,  31412,
                57688, 47148, 64056, 33536, 40800, 7788,  54972, 3660,  28584,
                12744, 57500, 43432, 26080, 64980, 9664,  32640, 16088, 23048,
                12548, 27616, 8684,  17232, 24260, 1496,  28336, 5816,  63696,
                53944, 44172, 58788, 36304, 29632, 48932, 24572, 55072, 9612,
                14132, 37368, 25324, 52412, 29732, 45256, 53372, 27376, 37856,
                3164,  16324, 51480, 20500, 9316,  58716, 24588, 10896, 27920,
                14060, 41208, 9424,  27344, 30988, 64544, 16940, 52028, 62616,
                7380,  36120, 11920, 61028, 52756, 34128, 29472, 64964, 45804,
                21828, 58848, 51264, 1560,  38724, 56560, 31476, 10208, 21372,
                60084, 13276, 3524,  60968, 34072, 51016, 29728, 45440, 25148,
                47520, 41872, 13948, 53924, 48496, 23960, 57872, 46504, 30396,
                12676, 63436, 57444, 34972, 39868, 4644,  53984, 23364, 12424,
                276,   64248,
            },
            {
                17952, 39972, 55848, 15780, 49068, 44140, 21456, 51208, 62848,
                15876, 55804, 42604, 26412, 4336,  9420,  18924, 35336, 26752,
                13472, 8928,  16904, 50552, 60408, 11124, 22400, 62716, 38916,
                46368, 5256,  18040, 2384,  39120, 20548, 56004, 6316,  50184,
                30416, 47372, 39584, 43856, 35524, 59864, 56248, 15076, 23632,
                18692, 32400, 5216,  21752, 7736,  38476, 4068,  16276, 40188,
                28680, 7012,  63448, 996,   21520, 11348, 6080,  64900, 22300,
                46808, 61700, 12516, 34820, 49188, 280,   31784, 39672, 50228,
                34512, 63564, 17948, 43716, 58200, 21424, 876,   33444, 22992,
                42388, 28312, 54964, 38936, 8848,  20456, 41892, 55512, 2428,
                32480, 16248, 11096, 61924, 19216, 49244, 4512,  42732, 28164,
                39876, 31968, 53732, 26916, 20944, 11064, 18448, 59496, 35388,
                55024, 21532, 4816,  31232, 44408, 11724, 65484, 16252, 52252,
                42468, 6488,  20912, 44452, 14108, 64108, 31172, 7596,  44800,
                56948, 35748,
            },
            {
                4196,  52896, 10816, 28324, 65488, 7048,  12924, 35996, 8856,
                44880, 19452, 12684, 32652, 57884, 29312, 59912, 49084, 6608,
                53164, 42536, 30000, 3356,  26404, 35912, 44040, 17328, 33396,
                9184,  53580, 35636, 59520, 13152, 29176, 42668, 62072, 37016,
                19472, 64452, 4744,  54612, 13196, 7764,  48900, 39148, 51800,
                2456,  48116, 40792, 56148, 51704, 61668, 43184, 59876, 22872,
                56916, 50068, 33132, 43044, 56392, 48176, 34884, 40476, 9876,
                30892, 54684, 41256, 28288, 64316, 16880, 55488, 61564, 3744,
                46584, 23240, 2060,  51356, 6848,  48384, 40608, 10324, 56592,
                18064, 2944,  45424, 25124, 760,   62504, 14084, 37736, 5736,
                26476, 47972, 41068, 30092, 23452, 36376, 64508, 16784, 57432,
                6400,  49716, 15640, 8052,  37280, 63368, 40888, 7520,  15720,
                1812,  27552, 61660, 36780, 19692, 40324, 28220, 4076,  37820,
                23484, 32696, 50556, 9880,  24996, 56296, 37320, 16344, 40372,
                26724, 48940,
            },
            {
                32084, 61476, 22840, 36452, 41132, 18676, 57388, 47380, 27936,
                120,   60504, 39560, 52540, 21432, 44356, 15068, 40528, 24092,
                63228, 36888, 21616, 57384, 14660, 48444, 972,   56324, 50628,
                27044, 63520, 22852, 44980, 48988, 52808, 24488, 1540,  45380,
                10804, 58040, 32804, 21948, 26512, 31448, 42492, 11288, 34688,
                62256, 9656,  16100, 28096, 13208, 26212, 19024, 34264, 2056,
                47132, 12480, 18140, 39396, 15148, 24832, 58424, 1696,  50132,
                18280, 24520, 2412,  6532,  46048, 22404, 43348, 7896,  19876,
                30436, 59324, 38600, 25412, 35660, 13104, 29208, 60540, 37200,
                50236, 64792, 15416, 58304, 32896, 48624, 28032, 18652, 51932,
                61104, 35780, 7652,  53936, 2900,  45208, 12616, 52872, 24324,
                1036,  62628, 43952, 58532, 46664, 2764,  24008, 52420, 31724,
                38644, 57612, 49768, 8952,  372,   55780, 33648, 9676,  60552,
                54204, 2320,  62484, 28452, 47636, 1376,  20420, 51604, 62560,
                10032, 21092,
            },
            {
                43464, 13672, 46724, 1240,  51720, 31124, 25024, 3852,  54340,
                34308, 48488, 25240, 6308,  2496,  37384, 11100, 54820, 2072,
                17800, 46380, 5408,  51756, 39384, 64760, 30940, 24656, 6168,
                14428, 40900, 31400, 10724, 5668,  16760, 33708, 8348,  53972,
                25640, 15388, 2748,  52240, 60784, 18004, 65280, 21056, 29436,
                25200, 43376, 59560, 35552, 600,   46572, 9324,  53484, 30916,
                64252, 5300,  27460, 61948, 4032,  31592, 19544, 60876, 14200,
                44212, 57368, 37600, 33040, 53884, 13664, 26584, 36448, 12248,
                53776, 15496, 10056, 55864, 61428, 16388, 53476, 3756,  25680,
                6548,  31464, 10944, 21832, 43408, 6860,  56528, 10216, 43864,
                21136, 13008, 57960, 17712, 60508, 27724, 8864,  33544, 47652,
                36136, 19136, 23068, 30484, 12928, 55644, 28588, 48476, 64772,
                10696, 44712, 17296, 26060, 62912, 51120, 22220, 43176, 18748,
                26524, 13776, 40396, 17584, 60032, 33184, 11692, 46600, 29616,
                2440,  59480,
            },
            {
                38636, 7572,  26048, 56548, 6028,  61052, 16668, 38960, 64152,
                22432, 10432, 16816, 63412, 56684, 47672, 30964, 61848, 33936,
                28316, 58556, 12380, 33064, 18696, 9336,  42700, 20624, 61424,
                47796, 1520,  19356, 55244, 37688, 64164, 40564, 18736, 60024,
                35172, 42072, 49212, 38316, 45240, 6360,  680,   46888, 56860,
                4028,  53112, 19968, 31372, 64856, 50340, 37284, 14696, 40980,
                20700, 44000, 35984, 51212, 8528,  54968, 42644, 28744, 35788,
                7940,  63496, 20324, 10320, 60720, 39232, 4808,  51468, 65460,
                41604, 27676, 45128, 4444,  31908, 22556, 46460, 43096, 19920,
                47732, 40104, 54068, 35564, 51588, 16440, 39312, 30692, 63264,
                4240,  46448, 25332, 32912, 49900, 39348, 21004, 61500, 41708,
                29232, 10644, 39608, 51896, 34656, 6048,  42500, 14256, 19996,
                4452,  23352, 41280, 13340, 30420, 46184, 14896, 5580,  49144,
                59092, 45228, 35852, 4372,  53040, 42832, 6320,  24408, 54408,
                35076, 17308,
            },
            {
                49632, 30024, 63140, 20216, 34728, 45372, 9508,  42760, 13936,
                32028, 51532, 43612, 35384, 27116, 18328, 22784, 7276,  50244,
                9744,  41252, 23624, 55196, 27172, 4252,  59084, 11972, 38004,
                33812, 51876, 59756, 25544, 2960,  57412, 27580, 11744, 50280,
                29956, 6780,  20308, 28488, 10360, 35900, 53720, 39760, 14492,
                8072,  37076, 45812, 11472, 5068,  22044, 57768, 6800,  25592,
                55784, 10680, 59992, 22940, 46104, 38076, 11736, 52976, 4944,
                26084, 47856, 52172, 15952, 29360, 48632, 58388, 32328, 21904,
                744,   33852, 50020, 19552, 40924, 64232, 8344,  33296, 59548,
                14244, 28916, 1492,  60696, 26348, 3784,  59292, 23648, 34224,
                53368, 38400, 352,   42824, 15248, 5804,  57716, 3240,  13836,
                54392, 7016,  65028, 1588,  17332, 21984, 61048, 36896, 56676,
                33408, 51656, 60240, 35756, 3088,  58168, 38360, 64384, 34752,
                11212, 30604, 7960,  57696, 21328, 37700, 65104, 14336, 41624,
                5356,  57560,
            },
            {
                10688, 53172, 15964, 39964, 12428, 50464, 28380, 57912, 20048,
                5620,  59896, 29160, 8096,  12984, 42156, 52880, 39204, 15932,
                60608, 45000, 228,   62268, 48092, 35020, 52828, 45408, 28344,
                17148, 6980,  43528, 29836, 14584, 47440, 20956, 43824, 28,
                62240, 13448, 55668, 63316, 16232, 24148, 59140, 26924, 32436,
                61760, 17240, 23984, 55528, 40012, 44296, 29388, 62872, 17020,
                49524, 2844,  33520, 14164, 26956, 52,    21648, 62608, 17280,
                39744, 31752, 3556,  41964, 24040, 1720,  18700, 9432,  46916,
                56780, 6340,  62520, 12644, 26860, 1188,  37000, 11852, 55748,
                23908, 62892, 9332,  46016, 20196, 13084, 42476, 48912, 8336,
                14700, 28908, 65528, 10796, 55572, 31632, 44440, 26700, 18116,
                49540, 56968, 24752, 43248, 58096, 47708, 26580, 8740,  748,
                46028, 28140, 6428,  54792, 20312, 8496,  24896, 29188, 1392,
                55820, 22956, 48020, 15568, 26008, 30172, 50624, 18848, 60676,
                27396, 21952,
            },
            {
                812,   44044, 32656, 5100,  55436, 22060, 1440,  36508, 52824,
                45896, 3244,  38596, 49264, 58960, 1208,  64052, 4524,  26052,
                19768, 36124, 30284, 6436,  39960, 14072, 23164, 2876,  57744,
                9800,  65452, 21692, 34916, 50912, 7568,  32528, 54248, 37060,
                22828, 46716, 39872, 2012,  33128, 48140, 12412, 19432, 50940,
                41164, 48348, 1932,  60336, 26828, 13392, 33752, 1656,  46392,
                38520, 30120, 52536, 18592, 65364, 56848, 49360, 33668, 44476,
                58148, 22380, 12936, 55984, 35772, 61932, 43872, 14676, 38536,
                24720, 16712, 37356, 55064, 47492, 57968, 50712, 18828, 2880,
                51996, 36604, 17472, 40960, 32256, 64576, 27868, 1984,  18304,
                57068, 22920, 51020, 19504, 36464, 23904, 51680, 63972, 37316,
                33084, 45712, 15500, 31820, 38300, 11556, 50792, 31008, 40644,
                15708, 62432, 10912, 48556, 32440, 44284, 52944, 16212, 41996,
                19036, 51744, 38240, 63460, 10488, 380,   45412, 8640,  31680,
                47056, 37220,
            },
            {
                64132, 24360, 59692, 48296, 26704, 64708, 31264, 7856,  61692,
                26200, 16148, 23180, 54188, 20240, 31596, 45376, 34708, 56388,
                48760, 13712, 53640, 22000, 17420, 63468, 31628, 41480, 54848,
                25160, 46608, 39596, 12040, 60624, 41812, 63396, 5004,  16604,
                58864, 9340,  25368, 52552, 57200, 43156, 3444,  64260, 5648,
                10144, 28960, 34544, 51472, 7376,  19236, 54308, 60856, 21632,
                8376,  59148, 42544, 6396,  40072, 30580, 9836,  15084, 2704,
                51364, 9412,  64840, 45544, 7348,  52948, 27776, 30996, 51036,
                59132, 43168, 29996, 9292,  23152, 15452, 30476, 39928, 27272,
                43984, 5180,  49232, 57656, 6628,  51156, 55132, 37912, 45284,
                61008, 40480, 6868,  47212, 61988, 8720,  1224,  12088, 22520,
                3988,  9660,  27756, 5316,  61792, 3224,  18548, 63772, 53876,
                21676, 37180, 25540, 17736, 39968, 63524, 11748, 47592, 60900,
                6912,  32884, 3628,  43284, 54160, 34624, 58560, 39288, 3680,
                55012, 13908,
            },
            {
                17804, 41384, 3768,  9724,  37272, 17448, 42092, 47964, 11940,
                33784, 40928, 65244, 9932,  36428, 15324, 23924, 11264, 29392,
                7676,  42360, 58796, 10780, 51136, 46732, 8112,  19080, 36400,
                15132, 568,   53780, 4716,  18360, 24296, 10392, 26796, 30804,
                44592, 34724, 18128, 29496, 8428,  21856, 36816, 30992, 44656,
                58072, 21276, 63628, 15976, 37764, 47412, 10332, 41724, 36144,
                27424, 12192, 23712, 54656, 3644,  47272, 36856, 24908, 61384,
                29804, 37796, 27036, 17592, 33612, 21232, 11400, 64040, 2856,
                7644,  20080, 53340, 2156,  35228, 42372, 59984, 7560,  65236,
                34124, 21820, 29552, 12572, 24636, 35316, 10100, 21748, 31380,
                11800, 34720, 2576,  30256, 16292, 27620, 42124, 48380, 59884,
                40284, 63060, 52256, 20340, 54524, 35044, 44204, 24812, 12880,
                59048, 4588,  43336, 57888, 1016,  23436, 4836,  35892, 27444,
                57436, 24728, 13584, 59848, 28124, 17120, 23336, 12520, 62012,
                25820, 51012,
            },
            {
                35796, 28924, 20964, 58048, 45504, 14372, 54240, 24164, 19172,
                56864, 508,   30576, 47032, 5820,  57236, 50504, 61276, 40100,
                2120,  64700, 33076, 27420, 37772, 1344,  59504, 29668, 49832,
                61088, 32760, 27432, 58404, 37744, 45976, 56224, 39324, 53088,
                2728,  49788, 64892, 4920,  41240, 60796, 51596, 15520, 23892,
                38552, 12740, 4332,  43060, 57112, 23824, 31424, 5032,  51776,
                15212, 63268, 44892, 34552, 16352, 20524, 58964, 43004, 5984,
                19412, 47660, 1452,  40524, 57476, 5392,  47476, 41524, 35868,
                13248, 60736, 25984, 45692, 63464, 5400,  20780, 48112, 14824,
                10412, 53636, 61860, 39236, 1384,  47404, 15768, 62752, 5204,
                49852, 25388, 44364, 59456, 52956, 38308, 57796, 19900, 29760,
                16688, 33968, 47820, 13204, 41336, 28608, 7852,  47296, 2108,
                33832, 49252, 29248, 52440, 14220, 31136, 55056, 20740, 39360,
                10036, 46500, 40672, 20296, 7192,  48440, 52720, 42628, 20556,
                33524, 6804,
            },
            {
                48892, 61768, 12720, 52424, 33976, 1960,  59564, 5496,  37948,
                50312, 44096, 14032, 60088, 26904, 42280, 3140,  21292, 52820,
                17980, 24648, 45672, 4680,  57020, 23816, 43792, 12732, 5440,
                22164, 42728, 48736, 13992, 31108, 2396,  20496, 15200, 60884,
                12344, 21524, 38236, 14408, 47480, 11012, 27136, 832,   55968,
                46564, 53320, 33092, 28068, 560,   49848, 65048, 17740, 58336,
                1212,  48960, 29104, 7876,  62364, 51456, 28248, 12652, 55888,
                32756, 54076, 14452, 50052, 60352, 25840, 16064, 55416, 22352,
                32772, 49080, 38952, 17580, 32324, 12160, 52836, 25100, 56568,
                45500, 3772,  16788, 59424, 19908, 42980, 57284, 27328, 52732,
                17536, 56632, 13308, 21008, 33280, 4628,  14152, 54664, 2700,
                44528, 24108, 6708,  57428, 22552, 60328, 16416, 52008, 39664,
                19580, 9936,  62936, 6876,  35412, 45632, 61456, 50424, 15052,
                65264, 1292,  51048, 31740, 63124, 36892, 1860,  29532, 46272,
                9996,  57044,
            },
            {
                2492,  44264, 31760, 7100,  25448, 39988, 29420, 10800, 63492,
                27208, 8304,  22284, 51628, 18628, 38320, 31956, 46816, 14356,
                37048, 9736,  49772, 20088, 15508, 34628, 64336, 52600, 39316,
                10348, 63012, 19312, 7472,  52368, 64480, 33912, 47612, 6304,
                42440, 56804, 26376, 32476, 58748, 18988, 35264, 62840, 7544,
                18084, 25252, 9476,  59272, 20140, 13736, 39296, 29560, 43468,
                25208, 19648, 39156, 57040, 10988, 23276, 1060,  38940, 46036,
                8400,  62220, 24340, 10584, 31432, 38336, 220,   61712, 27400,
                44256, 3736,  10140, 56816, 50844, 27860, 38040, 768,   35976,
                30756, 41500, 26848, 33772, 52244, 30524, 8268,  39640, 556,
                36232, 41556, 7372,  64304, 10192, 46444, 25668, 35732, 8340,
                50404, 59280, 36744, 172,   32216, 10536, 37588, 65408, 26416,
                56468, 42356, 22780, 17244, 38808, 25768, 8772,  3120,  29456,
                34580, 17956, 56208, 26280, 11224, 15368, 57792, 5488,  64868,
                38528, 16048,
            },
            {
                40616, 22744, 55792, 18584, 50232, 62060, 21464, 42796, 32628,
                16312, 55520, 35176, 3692,  61964, 12460, 8552,  63632, 28148,
                55328, 59620, 30504, 39540, 54672, 8832,  28544, 18044, 25704,
                56524, 2032,  36036, 24852, 41512, 9732,  28128, 51180, 23684,
                30188, 8264,  51900, 1728,  44252, 54544, 40484, 31636, 48748,
                42132, 61544, 37288, 51216, 45664, 34452, 8108,  55708, 11144,
                35692, 53824, 3160,  31676, 41884, 48464, 34216, 64668, 17208,
                21608, 36168, 42076, 3880,  19928, 45188, 51184, 9176,  18120,
                54012, 65044, 14960, 23560, 2716,  58956, 43512, 61532, 17992,
                22804, 63812, 8580,  46252, 3148,  13860, 64432, 22436, 32180,
                61568, 47784, 29320, 24056, 39404, 51288, 60940, 31204, 64916,
                11568, 27468, 15572, 42776, 46228, 55728, 3908,  30564, 6296,
                14864, 32704, 58856, 47008, 11820, 55868, 19228, 41392, 44748,
                54432, 22896, 37988, 4556,  45820, 41436, 32872, 24652, 18704,
                54212, 27416,
            },
            {
                5996,  11352, 64396, 37644, 15232, 45948, 4208,  53020, 48568,
                2432,  59440, 40548, 29968, 44796, 54552, 24884, 35328, 412,
                43280, 5260,  13236, 62084, 2192,  41932, 47892, 4016,  33980,
                44236, 30424, 54460, 47300, 60212, 17192, 37224, 512,   62808,
                18340, 35648, 61728, 16636, 24300, 4492,  9216,  22028, 14168,
                2552,  29348, 4736,  16772, 62644, 26028, 2620,  47628, 22488,
                63816, 6784,  45604, 13856, 18464, 59600, 4960,  52704, 26840,
                2508,  58372, 28660, 53096, 64552, 34188, 12968, 29940, 39780,
                5272,  36612, 31328, 47156, 40560, 19772, 9140,  13592, 47680,
                5616,  50416, 11908, 56172, 37144, 25436, 49016, 44516, 10456,
                19644, 4100,  55292, 16684, 43392, 880,   22020, 17640, 42412,
                39176, 20032, 53092, 63576, 25196, 18192, 49808, 21824, 44696,
                53784, 564,   27300, 5020,  64588, 48876, 33228, 58692, 6492,
                12432, 49384, 8860,  58616, 61804, 21052, 53408, 49092, 13268,
                31484, 61064,
            },
            {
                34928, 26124, 48056, 92,    9380,  34508, 57948, 13020, 24836,
                37116, 19668, 11192, 23660, 6640,  16896, 48280, 57764, 19060,
                22544, 33328, 45224, 26032, 51816, 22676, 61204, 13984, 59156,
                50048, 16220, 12236, 4120,  21712, 56368, 45080, 12940, 40720,
                53848, 46240, 11216, 38920, 49436, 29856, 64468, 51276, 58320,
                34792, 56132, 23012, 43836, 11764, 53008, 40784, 60880, 32520,
                16444, 50476, 27204, 61612, 37352, 25604, 9884,  30928, 40364,
                13316, 44444, 48332, 16660, 7252,  23076, 57160, 47864, 59076,
                20768, 51788, 26720, 62116, 7036,  34788, 29712, 54424, 33160,
                58460, 38804, 16196, 21792, 60916, 5420,  16964, 58352, 53540,
                26624, 14368, 59068, 34600, 6444,  54344, 12796, 48656, 4428,
                55600, 2140,  30272, 7784,  13384, 40080, 61976, 35804, 11904,
                60532, 37428, 51252, 40760, 29988, 1980,  23620, 15604, 27324,
                63680, 31296, 43252, 16764, 28620, 2616,  35676, 8144,  43556,
                1296,  46772,
            },
            {
                19528, 59428, 42160, 30236, 55052, 27348, 20176, 6504,  31256,
                64928, 45736, 53740, 49780, 63312, 33020, 41236, 5992,  50672,
                38204, 65492, 7920,  17672, 35952, 11240, 31240, 38324, 20508,
                8200,  65116, 27780, 39632, 32364, 6960,  26004, 59032, 4668,
                31940, 26944, 5452,  21080, 60052, 13312, 37364, 26236, 19244,
                45316, 10164, 39684, 31788, 5976,  20752, 28260, 14432, 4768,
                38284, 58264, 9064,  21476, 54616, 46952, 15820, 56944, 49756,
                63200, 5464,  32356, 11464, 55340, 37052, 41748, 2388,  25056,
                8484,  43368, 11932, 17080, 49864, 44728, 64628, 24368, 2804,
                42692, 26424, 31608, 53180, 43744, 28944, 40896, 2152,  33828,
                38196, 45560, 49660, 31020, 63432, 37100, 28416, 58192, 32852,
                23804, 60832, 48028, 35120, 51660, 2760,  27788, 6968,  48412,
                24484, 17572, 8292,  20996, 14156, 36288, 62132, 52512, 38556,
                19964, 208,   35064, 24240, 50884, 39896, 11936, 63612, 58880,
                22012, 51528,
            },
            {
                8396,  53656, 16372, 23000, 63132, 44196, 38560, 60252, 41936,
                15128, 8012,  27260, 1900,  37656, 13764, 26656, 61124, 11180,
                15404, 53652, 29708, 48604, 58308, 5160,  46092, 54328, 200,
                41484, 23448, 46232, 61188, 53616, 14672, 34684, 49536, 22248,
                15608, 65496, 42048, 56920, 33820, 46944, 1136,  41544, 6560,
                61916, 15580, 48324, 64368, 55424, 36972, 58896, 49936, 42908,
                19788, 29764, 43896, 33864, 256,   6408,  42464, 22372, 34860,
                18664, 24868, 39024, 61408, 27052, 4704,  17576, 63724, 14772,
                35224, 60664, 684,   56272, 22704, 4220,  15288, 51524, 10616,
                18880, 62196, 48,    7584,  35112, 19132, 51320, 12972, 63164,
                8128,  21756, 1644,  10984, 18876, 24680, 45052, 7088,  15032,
                37860, 44716, 11048, 21480, 59464, 31952, 54224, 19960, 42636,
                33464, 64032, 45384, 58176, 54252, 42932, 9792,  45616, 4360,
                47936, 60180, 54124, 13708, 7092,  55880, 18200, 25760, 29460,
                37684, 13916,
            },
        },
        {
            {
                56472, 14208, 7616,  50808, 20568, 60104, 54956, 12424, 45104,
                7372,  34332, 61960, 12136, 22948, 39584, 14516, 10300, 24668,
                50208, 13304, 65528, 46468, 54288, 14652, 1708,  64052, 55580,
                41620, 58540, 28436, 48652, 61924, 1744,  46108, 17168, 54684,
                47748, 29348, 35932, 60900, 42492, 15436, 39388, 4276,  28552,
                14140, 41788, 18484, 23512, 55328, 3408,  26572, 56728, 17680,
                13492, 46972, 6492,  57692, 18632, 9092,  56228, 37732, 63936,
                24528, 7564,  28100, 45808, 31752, 8600,  42800, 21796, 52376,
                61808, 14720, 45012, 4400,  21108, 52276, 6312,  45516, 11824,
                26372, 40704, 5296,  49744, 27928, 42072, 7704,  25512, 65272,
                49528, 29112, 58504, 39328, 16340, 10972, 21752, 5552,  38940,
                45872, 18784, 58964, 40960, 53724, 62024, 26692, 36076, 40020,
                53140, 16468, 38284, 48480, 5792,  19348, 29020, 2180,  46796,
                39880, 30200, 42336, 33504, 44124, 13464, 19228, 51248, 8328,
                27960, 46904,
            },
            {
                31092, 53352, 61540, 34832, 45924, 31388, 25064, 1792,  37832,
                21004, 15960, 29920, 51816, 59216, 2552,  46880, 37404, 7260,
                43472, 18856, 25744, 40600, 5760,  58896, 44428, 26032, 6468,
                18876, 33556, 14548, 43252, 19652, 30696, 35248, 60500, 9760,
                4596,  63116, 2168,  55460, 21680, 6120,  49384, 33220, 61136,
                51964, 1996,  58124, 49584, 38576, 8348,  21192, 59572, 6696,
                51040, 25288, 2252,  43448, 33332, 27056, 41092, 5772,  46672,
                32428, 52420, 14320, 62988, 17140, 56256, 34348, 13468, 36940,
                7428,  24940, 56956, 28740, 65296, 40804, 32752, 18332, 56448,
                15012, 60932, 31288, 18676, 58860, 10928, 37452, 20304, 54824,
                14344, 45780, 23420, 7692,  52096, 42192, 35552, 65064, 14536,
                28616, 61500, 35320, 5708,  30156, 45112, 18340, 57520, 6436,
                31940, 27340, 62384, 13300, 25696, 52548, 37668, 14456, 62964,
                51376, 8972,  25232, 5620,  60472, 29140, 2224,  65288, 44448,
                11536, 21032,
            },
            {
                41908, 5196,  23200, 16228, 3368,  9780,  40700, 51716, 60904,
                48284, 57912, 41532, 6872,  19176, 34808, 64404, 21272, 30644,
                56676, 4076,  32800, 52468, 16556, 20804, 49588, 31292, 53156,
                47052, 556,   65156, 8332,  57852, 23296, 13000, 50244, 21248,
                44012, 25484, 17736, 50652, 45884, 30476, 64776, 19560, 9952,
                25268, 44668, 32092, 11120, 29616, 15200, 33868, 45324, 39780,
                31424, 62704, 53664, 21932, 12172, 51400, 80,    20868, 50092,
                10756, 36832, 1200,  40088, 24716, 4172,  64924, 27148, 46280,
                58708, 31552, 38992, 348,   16424, 47904, 9568,  29936, 50760,
                23104, 36456, 47484, 2044,  24004, 45476, 51300, 3556,  40236,
                10164, 36040, 32208, 55648, 26620, 1052,  48404, 56968, 23716,
                43320, 7912,  51888, 12324, 38484, 23960, 1716,  11460, 47260,
                20436, 43236, 3576,  35508, 59792, 45316, 9608,  34404, 22144,
                17732, 59536, 54680, 16004, 49544, 10148, 40656, 23024, 36848,
                58300, 1612,
            },
            {
                49268, 39128, 29308, 57608, 48764, 64948, 18648, 32784, 14120,
                26976, 2804,  36512, 24576, 9852,  28868, 54688, 16296, 48840,
                61156, 39060, 9268,  62808, 28768, 35072, 9964,  37864, 13188,
                22156, 39364, 50616, 26788, 36944, 53648, 6248,  27436, 38592,
                57992, 13864, 33440, 8456,  37260, 12540, 300,   41116, 54840,
                37644, 7636,  16812, 62804, 46692, 53012, 65348, 448,   11388,
                19796, 35528, 15764, 39072, 58764, 64944, 15296, 29584, 57928,
                17656, 60920, 21468, 54392, 49116, 44596, 10272, 53452, 2612,
                18292, 11720, 43648, 54128, 36304, 23704, 61664, 2240,  39152,
                6628,  43952, 9160,  56852, 34976, 15868, 31480, 59396, 27632,
                61512, 17984, 4700,  63888, 13636, 19772, 31020, 4148,  11372,
                49920, 32652, 20872, 56120, 15228, 63232, 50920, 34052, 61260,
                54812, 8736,  56704, 18416, 31692, 400,   57924, 49592, 6612,
                31900, 44552, 1312,  37800, 21264, 55504, 32108, 6220,  52124,
                33724, 18024,
            },
            {
                26348, 63616, 11296, 36892, 43032, 27800, 53944, 8100,  43732,
                5056,  55372, 46348, 62264, 49944, 44200, 5428,  41220, 1568,
                27488, 12372, 47696, 23136, 924,   43016, 61056, 3240,  62944,
                29760, 55160, 11348, 17628, 3544,  45644, 40836, 64508, 652,
                30144, 53816, 41796, 59596, 27524, 23528, 57272, 14864, 47252,
                21472, 60248, 27424, 40356, 5700,  22920, 42716, 27256, 49668,
                55852, 8260,  45596, 28096, 3496,  47248, 34544, 43856, 8268,
                26600, 41956, 33716, 5460,  28684, 37700, 15920, 22928, 40608,
                49772, 63376, 20736, 8608,  50992, 13760, 44544, 58620, 20292,
                53484, 64720, 13860, 28840, 52716, 63592, 6484,  21812, 48036,
                344,   42728, 50180, 38040, 44704, 59020, 53388, 40312, 18048,
                62676, 1576,  25904, 42816, 48120, 8316,  28348, 41616, 22912,
                13804, 30028, 49788, 40220, 11676, 23760, 42104, 27560, 55472,
                39244, 13128, 26492, 34716, 63124, 45740, 17384, 61656, 25028,
                13196, 59852,
            },
            {
                2960,  45756, 7068,  20876, 13564, 192,   23596, 35836, 62868,
                19964, 30936, 12320, 16684, 32660, 58844, 13428, 23888, 52840,
                34556, 20180, 44588, 58028, 50976, 13952, 56180, 24184, 45892,
                7356,  34628, 42952, 60668, 31716, 56888, 15852, 33836, 18716,
                10708, 48596, 20828, 3144,  63560, 52328, 43948, 34200, 29396,
                3596,  51056, 35864, 1372,  55440, 12968, 17816, 37524, 60292,
                24276, 4428,  61432, 32460, 10444, 24540, 53880, 37872, 63312,
                3008,  45920, 56560, 11928, 19660, 58816, 32940, 61212, 29820,
                4928,  34460, 26444, 58120, 31096, 5648,  27752, 34848, 11152,
                26176, 33060, 18164, 41972, 3848,  39432, 12468, 44184, 34072,
                57392, 25224, 11416, 21340, 28872, 9096,  34616, 25088, 46532,
                37768, 58468, 4988,  35696, 30980, 59112, 3964,  17208, 38348,
                2608,  25356, 60744, 6900,  47152, 64756, 16064, 19876, 3032,
                63704, 48416, 52952, 5100,  11736, 27860, 3464,  42320, 47876,
                8640,  54320,
            },
            {
                34996, 16900, 51596, 56168, 33128, 60432, 49580, 16288, 57696,
                39104, 23300, 53336, 40512, 472,   26824, 37560, 63472, 8468,
                59188, 15568, 36908, 5896,  31756, 19084, 40972, 27532, 17132,
                58680, 20592, 5108,  25152, 48888, 22036, 8928,  51072, 44372,
                62628, 35652, 6428,  15760, 38684, 9372,  18236, 5804,  63056,
                11964, 56296, 19148, 48296, 25716, 33368, 57752, 3048,  30764,
                14436, 48104, 41716, 17632, 56484, 6444,  19428, 12588, 22700,
                50776, 15040, 30712, 62600, 7544,  50624, 1420,  43388, 9328,
                55952, 46628, 15024, 2780,  41932, 64204, 54872, 16676, 40620,
                49692, 512,   59300, 22292, 48808, 26680, 18860, 55812, 8312,
                15120, 30684, 52680, 62128, 2492,  16112, 61100, 6508,  13268,
                29644, 52988, 16352, 64856, 10960, 19864, 53712, 46668, 64264,
                51736, 44680, 36020, 21372, 28756, 52904, 37852, 59964, 10624,
                29896, 22692, 18204, 58612, 39812, 50872, 56944, 15360, 38116,
                28988, 21580,
            },
            {
                40352, 30852, 62576, 24868, 38388, 4580,  44936, 28944, 10084,
                51228, 5848,  65264, 8632,  21540, 56916, 18724, 45480, 30856,
                50444, 3088,  64488, 25520, 38964, 8112,  52540, 4100,  37096,
                50220, 14240, 63844, 39740, 2096,  12900, 61452, 26020, 4520,
                22988, 56772, 28092, 46792, 31824, 49976, 25968, 59248, 40488,
                23852, 32616, 9080,  41892, 61028, 7076,  50896, 44656, 9960,
                39480, 54800, 22016, 36608, 49728, 62404, 42424, 31852, 59524,
                5876,  35980, 24160, 47740, 38788, 14116, 54876, 24864, 17780,
                38456, 22176, 61884, 35960, 18904, 47536, 22492, 3112,  46196,
                62692, 6544,  37624, 54340, 10156, 62064, 36184, 51076, 23700,
                64792, 40844, 5672,  35636, 47532, 42428, 55432, 50544, 20260,
                43672, 9460,  22608, 40120, 45208, 24932, 36508, 7224,  31440,
                10108, 15708, 54520, 1664,  14236, 32996, 4952,  44084, 34900,
                46484, 7232,  42156, 32768, 24156, 8292,  20412, 31544, 1140,
                64288, 44220,
            },
            {
                14180, 4344,  9688,  47756, 18828, 55080, 12208, 40984, 1712,
                33768, 27408, 45696, 35888, 43436, 51064, 10976, 4708,  39340,
                22024, 28844, 48472, 11448, 54968, 60620, 29628, 47420, 10592,
                32016, 43976, 53356, 28680, 35540, 54432, 41572, 30520, 37184,
                53096, 13040, 42764, 60880, 21936, 55180, 1160,  36488, 48720,
                16232, 45256, 64024, 14532, 29504, 21792, 36072, 64376, 20372,
                27876, 62840, 800,   12260, 30304, 26960, 1796,  47068, 39408,
                17876, 53116, 548,   41560, 21596, 27688, 34788, 63712, 12196,
                51928, 6364,  29292, 50204, 11992, 38344, 8956,  31536, 56300,
                24064, 29572, 44524, 15160, 31664, 46512, 1288,  29748, 38784,
                3096,  45848, 18288, 57664, 26660, 22484, 31368, 928,   63668,
                33796, 56716, 28064, 51364, 96,    57740, 13324, 27640, 56476,
                20168, 33624, 41100, 61604, 57412, 9048,  50680, 24564, 13536,
                53920, 61780, 15100, 608,   64960, 44976, 36408, 60856, 52232,
                11044, 57408,
            },
            {
                50164, 59348, 28000, 42108, 6800,  31236, 64380, 21460, 47348,
                61184, 12888, 18296, 59092, 3512,  28456, 34408, 54216, 14368,
                57196, 42384, 18216, 33448, 44300, 15792, 21844, 64896, 56520,
                2476,  23048, 8412,  16592, 45776, 19852, 57628, 1136,  47580,
                8236,  17660, 33276, 2336,  10476, 41076, 14204, 30456, 7824,
                53044, 27664, 2904,  38364, 53732, 10816, 47432, 15756, 5668,
                52068, 33824, 8116,  45348, 60428, 16160, 52028, 9396,  25440,
                57524, 28824, 63872, 10560, 60052, 45128, 3388,  48324, 31568,
                59956, 44332, 1492,  57136, 24656, 52876, 60064, 15384, 35896,
                12120, 19424, 8256,  57508, 25052, 5932,  17116, 60760, 12160,
                20980, 54572, 32596, 13972, 7184,  39788, 11040, 37232, 26072,
                4248,  47812, 7144,  14596, 32720, 61348, 48860, 42540, 2200,
                62592, 24392, 4552,  48256, 26344, 39456, 20628, 58568, 2324,
                26964, 37200, 49704, 54892, 28744, 12576, 48656, 26152, 5544,
                19596, 24440,
            },
            {
                2244,  37024, 22256, 61476, 14888, 52576, 26044, 37228, 16732,
                55464, 24264, 49296, 31376, 15288, 62596, 24452, 47268, 60988,
                1280,  9708,  62440, 5276,  24252, 140,   35976, 13448, 41424,
                26192, 61760, 37396, 59204, 6308,  32852, 10772, 15648, 62100,
                39596, 58836, 25272, 51160, 65312, 19524, 46008, 61552, 22368,
                57372, 5420,  20444, 58532, 44164, 1632,  25080, 31596, 59244,
                43044, 18336, 25760, 53744, 38324, 4184,  34356, 65484, 13776,
                44384, 8016,  33340, 16184, 51680, 6452,  18912, 40668, 8248,
                26696, 14380, 41064, 33316, 7456,  28412, 3876,  43860, 51592,
                61112, 41724, 50560, 65416, 34952, 40304, 53268, 42896, 58328,
                28136, 49464, 9620,  60416, 51212, 63024, 18708, 48792, 58132,
                16868, 40872, 62120, 19344, 38492, 4348,  22884, 17584, 39148,
                50736, 44292, 12204, 18368, 30932, 45360, 15752, 63896, 41072,
                31648, 18548, 10036, 21244, 34456, 4680,  17548, 55656, 42924,
                33912, 46812,
            },
            {
                53524, 17844, 32508, 45468, 840,   35316, 8696,  58500, 2908,
                42324, 5132,  39164, 9836,  56040, 41404, 6324,  17460, 32476,
                27240, 36244, 49796, 40168, 58888, 51340, 46012, 7212,  33532,
                18308, 51192, 11828, 48356, 26940, 64804, 49840, 24640, 29072,
                20504, 45080, 6208,  36948, 29276, 56524, 4808,  33844, 11716,
                42636, 37140, 50028, 33916, 17652, 62332, 41068, 56360, 3824,
                37300, 49604, 14168, 57444, 23484, 19588, 41160, 54996, 21904,
                37100, 48512, 20128, 55776, 30500, 37448, 57580, 23444, 53136,
                36436, 20792, 65208, 17480, 46284, 62920, 39872, 20384, 25516,
                1688,  32640, 4840,  21804, 13280, 49180, 20156, 8452,  33988,
                5080,  37720, 24792, 43476, 29240, 2928,  54240, 42824, 13568,
                23892, 35420, 54044, 44440, 29908, 55012, 10724, 34416, 59280,
                8128,  29076, 37028, 65356, 52432, 640,   35580, 7700,  48752,
                56800, 4080,  45748, 61164, 41764, 58016, 38812, 8844,  14892,
                28256, 39568,
            },
            {
                12732, 65360, 10284, 51384, 57708, 20024, 43536, 49752, 32412,
                28668, 53108, 64124, 22396, 880,   36704, 52856, 11280, 44080,
                22872, 56084, 20228, 12776, 30900, 26648, 62812, 20500, 53996,
                44612, 30060, 1276,  40740, 22264, 3808,  42848, 52768, 34980,
                2820,  54500, 12348, 48980, 15680, 23844, 38856, 51800, 17000,
                64472, 29912, 24512, 12188, 6336,  28416, 51700, 12988, 22888,
                9748,  63884, 30756, 6016,  43452, 10384, 49068, 28396, 6400,
                2472,  61608, 42212, 4188,  25812, 11732, 64408, 15448, 648,
                58464, 49036, 10172, 54788, 23000, 13896, 34300, 57296, 10004,
                47932, 16892, 37364, 55436, 28960, 3456,  62940, 25996, 14528,
                45344, 62536, 500,   17316, 36220, 21980, 32880, 6200,  65420,
                30516, 1340,  11408, 21780, 8396,  64624, 46580, 25732, 53288,
                15112, 21704, 55700, 5652,  10540, 23224, 55164, 29512, 12192,
                22596, 39184, 26100, 51860, 2112,  23792, 30248, 50580, 63716,
                58792, 6660,
            },
            {
                56064, 26220, 4156,  39928, 28888, 23920, 11512, 63300, 14640,
                7624,  19832, 33880, 45408, 26532, 48904, 19604, 63460, 38840,
                4420,  46292, 7916,  53408, 16588, 2756,  38504, 10232, 59360,
                5456,  15468, 57228, 34080, 54808, 17392, 37588, 8724,  14040,
                63468, 41480, 31608, 60312, 8752,  44228, 3068,  26504, 47444,
                9404,  788,   55632, 60376, 47996, 38920, 19552, 34852, 46680,
                27624, 40676, 2100,  47192, 59364, 33084, 62148, 17088, 31864,
                56996, 24704, 13380, 35064, 50236, 46744, 42564, 28828, 33688,
                45432, 4616,  30560, 37324, 2052,  50036, 6512,  30124, 53700,
                63536, 27012, 46332, 59212, 10720, 44728, 31804, 55092, 50476,
                22660, 56156, 11820, 48464, 58880, 14816, 46132, 27100, 9292,
                51296, 46832, 58576, 26616, 50500, 16384, 37160, 4996,  31636,
                1072,  49204, 59972, 32656, 42692, 47492, 61216, 16996, 44248,
                62960, 6216,  15592, 32080, 13212, 62308, 19028, 45376, 248,
                20440, 31600,
            },
            {
                48592, 43828, 36364, 15424, 48176, 61304, 3620,  38316, 25752,
                47084, 59452, 11708, 16440, 61248, 29420, 7764,  58572, 51128,
                14844, 28584, 65532, 33600, 43256, 57128, 47992, 29332, 24532,
                39844, 62180, 20968, 46852, 13100, 61284, 30996, 58272, 46992,
                22716, 4904,  26916, 18832, 34896, 63684, 53820, 32244, 58872,
                19776, 40028, 45048, 15872, 32420, 8812,  65252, 1060,  55068,
                17276, 60928, 21072, 35956, 26048, 14668, 324,   51732, 46044,
                39840, 8996,  53552, 59224, 21508, 2572,  8532,  54540, 19568,
                12676, 25204, 59652, 43420, 27572, 61612, 41344, 17864, 36592,
                5560,  14060, 40968, 148,   23944, 39396, 17360, 36904, 2344,
                41040, 18908, 31400, 39172, 52728, 4028,  60632, 40112, 56216,
                37424, 18928, 41992, 33472, 2380,  59628, 42932, 12336, 63180,
                44868, 40328, 17164, 25940, 13352, 38552, 27248, 3164,  33656,
                49908, 58868, 36120, 56044, 48328, 7488,  34704, 11420, 36952,
                41864, 23240,
            },
            {
                8568,  60644, 19008, 55340, 5832,  33732, 53852, 18564, 56356,
                41012, 35904, 50752, 2828,  43080, 13548, 34768, 23752, 30976,
                1572,  57588, 39464, 23924, 5960,  11936, 35368, 17684, 3344,
                49096, 36144, 8552,  27428, 4604,  44756, 24192, 1064,  18076,
                56280, 38224, 45624, 57680, 180,   21404, 12128, 41776, 7064,
                23672, 34864, 51480, 3984,  22320, 26592, 57536, 43636, 7476,
                32324, 53040, 11368, 50460, 8028,  55384, 37744, 22824, 11508,
                64096, 18936, 29328, 38648, 16520, 32004, 60260, 37816, 50768,
                62768, 40188, 16204, 52496, 9308,  20076, 58248, 11692, 44196,
                23288, 32076, 52280, 19548, 63852, 48560, 6752,  60940, 10088,
                27908, 64988, 5812,  26380, 9068,  34596, 19712, 12340, 23308,
                3188,  15156, 62740, 6776,  39552, 29168, 23056, 57188, 19436,
                27420, 9532,  35884, 3504,  57224, 20012, 53376, 9288,  41292,
                19088, 28604, 10336, 20472, 42948, 27376, 53756, 59828, 25768,
                52396, 14312,
            },
            {
                34188, 1396,  30328, 22160, 41772, 13420, 45068, 31260, 992,
                22304, 5532,  30336, 57768, 53956, 39256, 3720,  47672, 17236,
                36396, 10060, 18580, 49188, 60976, 21776, 53060, 64092, 43756,
                13588, 52060, 31444, 63784, 38716, 10532, 51584, 28132, 34256,
                50228, 7520,  14696, 51500, 40684, 28992, 48056, 15492, 55252,
                62660, 13240, 28672, 61672, 53420, 41324, 13888, 49156, 24428,
                38444, 15256, 42884, 29376, 64788, 18608, 44772, 27676, 6672,
                34048, 48244, 1176,  44632, 63404, 11020, 23304, 27072, 3356,
                7652,  34816, 22072, 5212,  33172, 46612, 25084, 1424,  48988,
                55324, 61352, 9112,  35612, 29496, 56876, 13044, 21528, 52268,
                35340, 47404, 43352, 57212, 22952, 63812, 43772, 49132, 28108,
                61144, 31140, 53444, 13096, 48760, 18232, 7892,  51052, 33760,
                54640, 61844, 22788, 48628, 60540, 6328,  35024, 64608, 24396,
                1040,  54032, 46224, 4464,  65096, 39696, 16388, 6352,  46708,
                3736,  63316,
            },
            {
                27212, 50972, 58356, 9868,  64212, 27700, 7856,  50008, 60032,
                11024, 65192, 17952, 24948, 9524,  21028, 64260, 41740, 54636,
                61700, 43616, 52292, 27596, 768,   41496, 30772, 7116,  26376,
                56508, 22904, 352,   16964, 54384, 20492, 60204, 40204, 65004,
                11812, 31296, 25548, 62852, 9840,  36288, 60676, 5148,  27104,
                38288, 2136,  43224, 10484, 33680, 5712,  18552, 29980, 63252,
                2524,  56688, 5172,  23412, 39776, 3716,  58644, 53132, 42808,
                60564, 15060, 26164, 4968,  52520, 41336, 56032, 48076, 17804,
                43808, 56904, 48848, 62104, 13628, 54192, 37984, 64872, 28360,
                20904, 2872,  43140, 15796, 4316,  26148, 33708, 44304, 58376,
                15980, 3200,  13172, 51088, 17004, 30212, 2192,  54932, 7400,
                35720, 44852, 21588, 25432, 55544, 63944, 36264, 46836, 3216,
                14420, 6708,  52308, 43936, 30432, 15724, 45996, 51112, 31276,
                14588, 60944, 38212, 23996, 31468, 1488,  22360, 56808, 32472,
                18320, 39796,
            },
            {
                45572, 16560, 38600, 47344, 35668, 52880, 24140, 16616, 42784,
                28084, 47892, 37088, 45136, 33260, 50032, 27776, 6480,  11876,
                25764, 5156,  32268, 13716, 36932, 46732, 15416, 60228, 33788,
                10608, 41392, 58976, 47384, 33284, 42776, 6404,  15612, 3152,
                44016, 20288, 54916, 3692,  17348, 24028, 52656, 31576, 45876,
                18112, 50548, 57736, 20788, 47584, 38032, 59060, 9912,  35596,
                45592, 20052, 60224, 47464, 31404, 10152, 34924, 13476, 20312,
                30988, 56196, 50668, 35476, 20188, 6912,  33456, 13968, 64972,
                30768, 10824, 27840, 488,   40788, 30392, 16656, 7880,  34660,
                13328, 39156, 58528, 51232, 45708, 54136, 40400, 996,   30428,
                24636, 61880, 33352, 40772, 6792,  37376, 58364, 15600, 41372,
                52180, 9984,  4176,  34384, 42652, 476,   11468, 30588, 25180,
                42172, 38164, 28052, 12432, 144,   40556, 21848, 10932, 57592,
                42712, 7924,  17308, 52324, 12064, 44644, 49536, 28492, 62172,
                11064, 54756,
            },
            {
                5328,  12812, 25104, 3956,  20032, 1700,  62016, 38076, 34008,
                54768, 14300, 7188,  61800, 36,    15376, 57260, 19280, 34044,
                45820, 21488, 58236, 64388, 9104,  56252, 20328, 49448, 2968,
                18812, 36660, 28896, 5492,  13872, 25984, 49172, 36376, 23312,
                57804, 47228, 37756, 32812, 49380, 43764, 1508,  20424, 63892,
                8968,  35716, 25016, 16164, 64352, 276,   54244, 22192, 51144,
                27704, 12216, 33980, 52344, 15688, 62888, 48756, 25580, 2412,
                6392,  40508, 23180, 12720, 58156, 28544, 46076, 2364,  39288,
                21104, 53872, 36984, 24268, 44928, 57316, 4392,  51824, 59672,
                47072, 30100, 18732, 24816, 11260, 64752, 8224,  19188, 49760,
                38396, 9436,  20992, 54160, 25456, 48204, 11156, 32592, 24572,
                18072, 64512, 47400, 57552, 28176, 16196, 39044, 60584, 56600,
                17432, 65068, 20636, 33420, 62608, 56316, 26728, 37008, 3996,
                48600, 33800, 26328, 58288, 35544, 61372, 14648, 8252,  37344,
                43428, 22808,
            },
            {
                64984, 56572, 32928, 61128, 43396, 14188, 56776, 9540,  4900,
                20684, 51256, 39984, 22548, 55780, 42852, 30904, 60684, 38544,
                50708, 2716,  16252, 40624, 24548, 3968,  28644, 39260, 54152,
                45792, 64880, 50916, 21784, 62196, 8888,  58496, 30324, 52492,
                9508,  27284, 6064,  60272, 13656, 57384, 39280, 11756, 53844,
                41944, 30128, 4656,  45188, 31808, 27000, 42640, 14964, 4780,
                61628, 40460, 7108,  26336, 776,   22068, 41700, 57616, 37544,
                64660, 46508, 9724,  61864, 38404, 17068, 60612, 24920, 50928,
                5548,  59272, 15388, 8144,  63932, 19512, 23036, 42588, 26052,
                10048, 63292, 6060,  37640, 31788, 22784, 14760, 60604, 55444,
                5240,  46096, 29588, 59036, 228,   63324, 20240, 46164, 60352,
                1412,  37920, 12300, 20176, 5796,  51684, 44664, 21168, 8612,
                49508, 2196,  47264, 9168,  53180, 18136, 7008,  61536, 29312,
                20944, 63644, 2936,  46864, 19900, 5264,  40948, 51820, 20244,
                844,   30576,
            },
            {
                49888, 7956,  41008, 52316, 28772, 48788, 32088, 45620, 26268,
                63696, 2220,  31916, 26996, 10636, 48152, 3864,  13392, 8244,
                63280, 29560, 47848, 54368, 33116, 44648, 62620, 11516, 24144,
                8240,  15664, 1936,  43812, 38340, 54856, 18352, 2172,  40904,
                16604, 63448, 42200, 19244, 29620, 7492,  34112, 26264, 59192,
                2668,  14196, 61308, 55620, 12024, 8408,  49876, 36764, 30600,
                55552, 17280, 43480, 59148, 37092, 55320, 7784,  17896, 51576,
                28624, 15772, 32756, 48936, 204,   43340, 54480, 9468,  35300,
                42780, 18500, 46500, 52824, 33856, 11144, 49836, 36888, 15244,
                3360,  41332, 50124, 57192, 1616,  48072, 35988, 27660, 42004,
                12632, 64412, 17724, 42832, 14556, 34432, 39792, 4776,  27520,
                50148, 30436, 40964, 59592, 32804, 62476, 24024, 3912,  53532,
                31684, 35752, 58512, 43372, 23768, 38060, 50264, 45088, 16420,
                11976, 54440, 39304, 9260,  29724, 56056, 33112, 25300, 59756,
                47388, 35852,
            },
            {
                26860, 17848, 21572, 11012, 5524,  16924, 22800, 39324, 59164,
                13004, 42208, 58008, 16456, 52112, 36656, 24380, 41108, 53208,
                22764, 36064, 11228, 18996, 6272,  51336, 14692, 35564, 58316,
                31604, 26524, 57040, 34164, 11644, 27868, 45436, 33092, 59736,
                12612, 34812, 584,   51976, 23148, 65352, 47336, 16400, 22152,
                48704, 51796, 37676, 22984, 40244, 59996, 18980, 63120, 1872,
                23712, 48408, 10496, 19388, 45912, 29100, 32888, 11584, 44872,
                21288, 3988,  54888, 26424, 8076,  31264, 21664, 13580, 28120,
                61740, 31896, 3772,  39804, 28380, 2000,  61036, 31520, 56248,
                52928, 22228, 33736, 16488, 61840, 42996, 52724, 4196,  32396,
                23248, 35600, 50448, 7080,  52484, 28820, 56844, 8788,  54600,
                21912, 14712, 53292, 26012, 9244,  48308, 13868, 28924, 40616,
                11816, 25600, 15460, 4476,  30312, 13432, 2528,  55392, 32008,
                42028, 23748, 51304, 15892, 43608, 63136, 2288,  17232, 10472,
                55756, 14036,
            },
            {
                60568, 45096, 38052, 58516, 35224, 64604, 55140, 2964,  30032,
                18316, 47632, 35380, 8120,  63032, 19520, 58948, 28244, 17296,
                44584, 964,   57652, 27160, 37836, 59440, 22132, 644,   43344,
                52612, 40752, 19820, 48452, 4208,  23896, 64564, 6876,  22008,
                50184, 25076, 56124, 45252, 10416, 36368, 54468, 5484,  40576,
                33252, 27512, 19320, 6132,  47900, 33096, 25332, 44992, 13164,
                38888, 32148, 64848, 3232,  53048, 13988, 62276, 1904,  59640,
                35536, 42520, 62792, 18840, 36252, 51868, 64004, 47948, 1620,
                55640, 12284, 25648, 62648, 48164, 14332, 44128, 7308,  18584,
                28560, 45404, 12592, 27068, 7848,  20144, 10836, 59724, 16408,
                47560, 3116,  26364, 38740, 11624, 23500, 44412, 17364, 61816,
                36824, 6668,  45396, 2948,  18852, 37788, 34496, 57252, 63272,
                45184, 19540, 61060, 41168, 49016, 64120, 34868, 25544, 8576,
                59708, 568,   34724, 27208, 11724, 22336, 53488, 39404, 28576,
                42244, 3384,
            },
            {
                6680,  53056, 29776, 940,   25504, 41928, 7740,  50412, 10940,
                53196, 22280, 4496,  44456, 29392, 1400,  46736, 5364,  55292,
                32048, 14144, 62200, 42584, 9376,  30920, 48136, 17924, 5396,
                63444, 13228, 7328,  60388, 52072, 41808, 14616, 53928, 47040,
                37980, 30776, 4584,  39368, 14828, 27672, 2272,  44060, 61852,
                8176,  11072, 64736, 54208, 1476,  16236, 7408,  52660, 57232,
                6004,  50956, 27404, 35416, 23064, 40212, 47880, 25876, 53896,
                7172,  23592, 13140, 40280, 57088, 3548,  16008, 38068, 41320,
                23204, 50584, 8856,  36568, 21164, 58448, 24872, 39380, 65492,
                1152,  59228, 40260, 54284, 46344, 30832, 24692, 39120, 57288,
                8604,  62256, 53924, 19096, 61196, 2764,  48708, 30948, 42328,
                10952, 33904, 57004, 65504, 41960, 54512, 6152,  17284, 712,
                50840, 6784,  55480, 27644, 9980,  21532, 57368, 40252, 19252,
                47576, 65508, 6124,  57460, 49484, 36048, 46308, 7608,  64104,
                34568, 23540,
            },
            {
                31924, 15624, 62624, 49164, 12792, 46232, 20576, 37072, 62748,
                33512, 25452, 60364, 54296, 38836, 12536, 34272, 64680, 10076,
                49072, 39932, 24120, 51840, 2988,  65048, 25972, 39176, 49852,
                29064, 23256, 37956, 32360, 17488, 29796, 35948, 1184,  19400,
                8500,  62952, 17516, 58312, 49232, 60664, 19904, 31164, 56740,
                17932, 45992, 35860, 29808, 43148, 58380, 35456, 28860, 41680,
                21496, 16016, 60636, 8288,  57960, 4356,  18480, 37764, 15172,
                30908, 49916, 58848, 10096, 46560, 24612, 29424, 59000, 7096,
                19248, 44896, 57388, 16772, 53668, 5112,  33304, 11968, 49292,
                35876, 9420,  21132, 3664,  36536, 63880, 1960,  50200, 43460,
                21400, 29340, 44760, 32240, 57872, 35460, 14032, 64228, 752,
                24688, 49656, 16056, 29464, 23264, 11688, 60848, 27124, 39228,
                22812, 32820, 36844, 52672, 1104,  46684, 15660, 5312,  52464,
                12996, 37360, 30488, 18060, 61232, 3844,  31284, 15016, 51068,
                19788, 47664,
            },
            {
                54720, 43232, 8952,  19340, 57356, 32480, 27616, 14684, 44736,
                444,   41080, 9812,  15596, 21328, 50964, 42676, 26036, 20108,
                59884, 4112,  34736, 20596, 15888, 44336, 12844, 55316, 34304,
                10240, 59076, 46156, 2952,  62564, 9140,  56420, 44072, 60788,
                28152, 11776, 43036, 21768, 32788, 7248,  51612, 12584, 23804,
                38808, 2852,  59456, 24672, 12816, 50100, 19460, 10244, 63752,
                2832,  37596, 45352, 12420, 31316, 43088, 51156, 9364,  65488,
                44492, 1080,  28240, 33744, 5200,  20092, 44548, 10804, 32384,
                65020, 34476, 27924, 376,   42036, 29772, 45856, 55000, 23380,
                16916, 31204, 62164, 48804, 14956, 55716, 18420, 33572, 13180,
                36736, 916,   15336, 9888,  41252, 6548,  21056, 27152, 52320,
                58372, 39908, 20612, 1788,  46804, 51600, 31448, 44008, 48144,
                10980, 64484, 14224, 18404, 31100, 38708, 62728, 33492, 43304,
                26428, 21892, 45216, 8364,  41860, 20880, 25428, 57908, 124,
                61524, 11872,
            },
            {
                4532,  35692, 24632, 38548, 3060,  61252, 53504, 5140,  58444,
                17552, 49144, 27432, 63912, 31256, 57632, 6780,  14444, 38192,
                29888, 8152,  54516, 47396, 57268, 33144, 7448,  60652, 1608,
                19492, 54212, 15216, 26652, 49264, 22212, 12472, 25960, 39736,
                51332, 34000, 54588, 1784,  25624, 37664, 41996, 64188, 28352,
                49192, 14880, 52188, 21216, 5784,  39116, 60304, 26276, 47332,
                32916, 55752, 24880, 49676, 20764, 63020, 28128, 34220, 56516,
                20268, 39088, 16748, 53660, 63648, 50684, 35788, 54656, 14972,
                48488, 6388,  40136, 13336, 63476, 19308, 8516,  60484, 4460,
                43384, 56644, 25320, 5496,  42528, 28508, 7048,  61176, 26092,
                56272, 48576, 65244, 25208, 51000, 55588, 46184, 38556, 4688,
                13020, 31996, 9028,  60108, 36280, 4440,  15128, 20408, 7844,
                58704, 25928, 4740,  43516, 60800, 23628, 11600, 29000, 58824,
                2068,  49992, 56664, 14232, 38440, 54632, 11204, 44028, 37108,
                27336, 40064,
            },
            {
                22276, 65308, 28940, 51356, 16180, 40604, 10288, 23808, 30704,
                37976, 56088, 6072,  36008, 3076,  47392, 23696, 61732, 52296,
                45736, 17880, 27400, 11504, 40548, 29212, 21740, 46872, 25156,
                41544, 36168, 6684,  43408, 39020, 53104, 32760, 58076, 3768,
                15832, 6504,  47940, 63700, 13812, 46136, 9944,  104,   34644,
                55604, 4736,  41356, 63132, 33596, 45576, 852,   53324, 14308,
                7688,  18172, 41144, 364,   54172, 15640, 6020,  24324, 3308,
                11400, 48428, 61556, 25688, 14132, 41556, 896,   60908, 26292,
                3728,  53212, 59584, 24316, 47728, 51428, 38008, 15208, 27712,
                52220, 38900, 12212, 34076, 51864, 22592, 39540, 11116, 53088,
                5336,  40440, 20664, 3944,  29684, 16432, 59920, 33804, 18656,
                47724, 63720, 43512, 53016, 26288, 40940, 63096, 55904, 34888,
                50520, 38004, 46360, 56732, 8164,  50660, 3708,  54116, 17524,
                9612,  62168, 33012, 3028,  28124, 64576, 49304, 33216, 8720,
                17828, 56796,
            },
            {
                14284, 1300,  45356, 56232, 7156,  48504, 35236, 64336, 46856,
                21156, 13232, 51580, 44048, 18824, 11128, 40664, 33288, 2120,
                56780, 36516, 63952, 380,   59412, 4980,  38024, 63748, 16176,
                50728, 30492, 56848, 65452, 1920,  18148, 7404,  47172, 20644,
                61948, 23940, 40360, 29132, 19100, 56604, 52832, 22168, 17340,
                44592, 31932, 26628, 8712,  56112, 17156, 30880, 22448, 43172,
                61956, 29024, 59264, 35628, 10544, 39200, 46632, 60996, 52064,
                36492, 22552, 42912, 6980,  31740, 9284,  22124, 39632, 18192,
                30824, 21444, 36152, 9908,  32084, 2592,  22796, 34440, 64272,
                748,   47096, 19764, 59676, 9680,  64732, 47924, 44536, 30552,
                17752, 58560, 32728, 45512, 37628, 10860, 24076, 7976,  54336,
                28148, 22728, 5576,  17692, 33208, 10256, 23908, 1384,  29752,
                17204, 2976,  21656, 28168, 34396, 41736, 20192, 47292, 36284,
                40060, 24020, 53396, 19584, 46024, 5980,  16628, 22876, 60252,
                46256, 49784,
            },
            {
                34360, 59924, 11456, 21416, 31800, 25856, 18616, 42972, 2300,
                8596,  59380, 32964, 24728, 60972, 55168, 28528, 16852, 6160,
                22424, 13488, 42312, 24756, 48852, 18588, 52244, 9556,  43832,
                3640,  13108, 21380, 10844, 34540, 25100, 60184, 37496, 30620,
                44840, 35396, 9108,  58852, 3204,  31344, 36424, 59488, 6712,
                61572, 11896, 19908, 47796, 37804, 11432, 65180, 4560,  36672,
                50396, 11916, 4976,  47588, 26256, 57120, 31784, 41588, 13756,
                29304, 55244, 2076,  35080, 58752, 48128, 28580, 56748, 46268,
                11752, 62716, 42704, 16128, 56684, 43968, 58652, 11628, 41296,
                8020,  30008, 15504, 37516, 1692,  32220, 15968, 3052,  24108,
                35928, 8760,  13384, 52764, 63540, 1456,  43752, 62244, 40692,
                2496,  36620, 56580, 13608, 49256, 59368, 54084, 38896, 12964,
                42452, 61472, 53472, 11960, 15848, 59200, 25976, 64800, 6240,
                30164, 15248, 42888, 10924, 35084, 58184, 29964, 2012,  41792,
                5252,  30700,
            },
            {
                52036, 18396, 37188, 41848, 62176, 3928,  57440, 12480, 52820,
                28004, 40848, 16168, 1036,  38308, 7820,  48780, 65448, 44380,
                50608, 30820, 8564,  53608, 35020, 14136, 31628, 26708, 54760,
                33576, 61632, 27780, 46380, 51124, 41388, 14432, 55416, 932,
                12952, 54004, 17228, 50340, 43296, 26280, 14624, 48208, 24780,
                39652, 51360, 57964, 2520,  28964, 24040, 49328, 40416, 57748,
                19668, 33908, 22844, 64636, 16992, 7780,  21584, 4504,  18188,
                59240, 9512,  46000, 19600, 52404, 16480, 63992, 3044,  37164,
                51748, 7684,  49392, 26004, 5676,  28864, 53784, 18036, 49748,
                21920, 62400, 57020, 44792, 27580, 54380, 21092, 56548, 62228,
                49612, 42992, 60884, 26904, 18244, 48932, 31064, 12572, 50648,
                15840, 45724, 61680, 30460, 42580, 7004,  18960, 46080, 65116,
                24912, 32368, 6936,  48580, 39444, 2028,  32036, 13632, 44856,
                57256, 396,   48868, 63072, 25664, 51004, 38952, 53848, 63632,
                25196, 9804,
            },
            {
                43664, 26476, 2824,  54260, 14460, 50408, 29500, 36960, 61092,
                22500, 47868, 64100, 30404, 52972, 21808, 35616, 12224, 25980,
                58564, 39624, 60812, 20708, 44900, 62232, 1844,  58088, 22740,
                7628,  40132, 59096, 16680, 5340,  29508, 63456, 9588,  49376,
                27944, 64816, 5564,  37760, 11748, 62636, 4508,  41684, 9536,
                29904, 35584, 14700, 62060, 42228, 54692, 7964,  15656, 26644,
                2504,  55072, 43604, 30632, 52452, 37584, 62824, 44460, 49400,
                33124, 26028, 63212, 38156, 11576, 24508, 5908,  43800, 14044,
                32816, 58492, 2204,  39520, 20140, 63740, 36604, 4048,  26448,
                32868, 53232, 4744,  24288, 50316, 41460, 6896,  38280, 12036,
                28468, 120,   22520, 5740,  35648, 57472, 21232, 26236, 58776,
                32980, 10100, 25020, 320,   21504, 37052, 28500, 4056,  51452,
                9336,  58224, 36708, 19732, 63572, 54400, 9864,  50364, 22172,
                61460, 27120, 37304, 4352,  21732, 7640,  14876, 11832, 35916,
                20456, 55908,
            },
            {
                6620,  64148, 47148, 8976,  33776, 23220, 45044, 7752,  17372,
                33428, 4064,  10516, 45288, 14264, 59016, 42568, 1728,  19160,
                33996, 3376,  16216, 5680,  28440, 10864, 41512, 47684, 35868,
                19392, 49456, 116,   37276, 53492, 22252, 43884, 18872, 24784,
                40660, 33020, 19848, 23568, 54748, 33508, 20540, 52300, 64904,
                1244,  21964, 44960, 6212,  18772, 34560, 60288, 31612, 46480,
                9696,  61296, 14192, 40116, 1548,  12628, 27932, 54352, 2488,
                39544, 15140, 5164,  56608, 30176, 41784, 34492, 54424, 26884,
                22340, 17564, 29880, 52900, 47180, 13956, 8768,  46092, 60004,
                43288, 12728, 35880, 9076,  18456, 13644, 63368, 33356, 46068,
                17076, 51432, 39744, 55428, 14820, 42148, 3020,  39028, 4764,
                19952, 55276, 40000, 51240, 64060, 11524, 48616, 33972, 15964,
                21952, 44216, 492,   30132, 45428, 23832, 35748, 42264, 18268,
                8440,  32580, 55104, 17920, 41216, 60524, 44416, 27636, 48112,
                16240, 39468,
            },
            {
                32900, 22128, 28876, 58404, 17980, 40380, 64828, 544,   55380,
                38668, 50796, 57124, 20320, 27088, 4588,  31856, 51896, 56004,
                10236, 47480, 24480, 55172, 50240, 38512, 17188, 5184,  64724,
                14020, 44608, 24472, 32072, 8840,  58344, 2836,  36268, 52624,
                4216,  45896, 60528, 48876, 1016,  45008, 57516, 28208, 17964,
                47056, 55336, 32316, 26304, 52352, 13200, 1452,  21676, 53368,
                36116, 28308, 5732,  50824, 19432, 59940, 35060, 9036,  23112,
                61068, 51312, 21236, 44572, 112,   50300, 19120, 61292, 9788,
                45688, 65328, 35680, 11176, 59328, 33660, 23636, 38876, 16084,
                2484,  20488, 40528, 60860, 47008, 57880, 29384, 3920,  25404,
                60016, 7840,  33632, 29324, 47204, 9172,  52552, 34880, 65024,
                48260, 28912, 6104,  17052, 44328, 56328, 25588, 59912, 41200,
                53304, 27356, 13680, 57084, 16824, 4124,  28660, 56236, 4964,
                39532, 52132, 11764, 46204, 30716, 56416, 33416, 4892,  62004,
                516,   59336,
            },
            {
                12328, 15312, 37808, 49800, 4524,  11564, 52544, 26984, 13916,
                42892, 24456, 6832,  36024, 41300, 62760, 16760, 45788, 28288,
                62380, 36900, 43004, 64372, 32708, 22288, 56104, 25792, 30192,
                53084, 10508, 62488, 55860, 13540, 47488, 33376, 60936, 11500,
                57208, 14860, 8024,  26968, 39492, 16012, 7452,  36980, 12444,
                39064, 4228,  63176, 10016, 48632, 37948, 44480, 64236, 41276,
                17148, 48732, 23732, 58140, 45084, 25120, 48400, 16248, 42592,
                30984, 12368, 35824, 27200, 64524, 15580, 7524,  38728, 56940,
                1460,  40936, 6716,  24852, 42956, 312,   51080, 55912, 29152,
                47512, 65148, 31200, 26700, 724,   34780, 52676, 10340, 41208,
                54464, 20672, 64640, 11700, 58416, 24468, 61316, 16736, 8048,
                43520, 14064, 59740, 35556, 31420, 8068,  14268, 3004,  37784,
                5852,  62448, 49804, 40516, 8156,  51436, 62588, 13244, 47456,
                60348, 24000, 15908, 65268, 1320,  10080, 19068, 52804, 23352,
                29716, 50136,
            },
            {
                53864, 45328, 1600,  62360, 25004, 35656, 46984, 31860, 19268,
                59332, 29708, 12520, 60728, 49868, 9280,  23864, 38420, 5204,
                21552, 14484, 30320, 704,   12828, 7204,  45840, 60580, 2264,
                38248, 18084, 28236, 41280, 19784, 26688, 40164, 16904, 29688,
                22428, 42828, 31796, 56344, 10820, 30504, 60852, 24596, 49436,
                58324, 15400, 41436, 22880, 59656, 28664, 4896,  25436, 12028,
                6940,  62956, 38476, 32028, 10696, 4192,  29932, 65232, 56308,
                6704,  47184, 54908, 9900,  58176, 31928, 46424, 23836, 29388,
                49860, 15064, 54180, 61868, 17824, 31340, 20884, 62268, 7416,
                11048, 51940, 55564, 6052,  15032, 43692, 19192, 23192, 48092,
                14572, 37436, 44188, 2016,  19300, 32548, 45372, 28104, 22076,
                54076, 38096, 23728, 2268,  52956, 19672, 63184, 32672, 55520,
                18284, 11008, 31792, 20428, 59772, 33184, 38228, 26480, 19508,
                34668, 3516,  29180, 36680, 50780, 25992, 40056, 46456, 37056,
                9236,  41896,
            },
            {
                26120, 19212, 55740, 31088, 43128, 20476, 56836, 8572,  63048,
                2348,  45612, 53948, 17640, 32688, 40,    54648, 12316, 57568,
                49416, 8168,  53384, 19016, 59588, 35640, 51548, 9776,  33956,
                43220, 58524, 3128,  50828, 4944,  64980, 7708,  49740, 592,
                62800, 51420, 36864, 19132, 64476, 50852, 43336, 1880,  33408,
                20820, 29700, 7680,  35952, 16452, 56692, 19592, 49608, 55456,
                34160, 20712, 244,   15684, 55224, 36308, 40712, 20728, 984,
                37568, 18356, 24360, 2736,  39884, 20948, 53592, 4020,  12116,
                33412, 20204, 27652, 48068, 5048,  57044, 12832, 36240, 41988,
                25036, 17500, 37708, 22820, 49628, 38948, 64160, 58916, 2920,
                31492, 6100,  26396, 49284, 40584, 4916,  13224, 50308, 952,
                31908, 10572, 62796, 47936, 27720, 41672, 22688, 45132, 28760,
                48040, 24284, 36260, 2368,  46448, 22488, 10776, 860,   44836,
                57624, 49856, 42360, 6228,  22216, 59044, 13556, 64084, 16688,
                57416, 3884,
            },
            {
                35140, 40268, 6844,  10056, 60268, 14552, 5120,  41340, 49388,
                34264, 22184, 5604,  39896, 47312, 29092, 63876, 34936, 41900,
                26080, 61468, 39248, 45180, 27204, 41768, 15900, 20776, 48816,
                23516, 7440,  32492, 46320, 36800, 23368, 55376, 43656, 37860,
                26216, 5808,  13604, 2532,  47024, 21736, 6012,  55228, 10344,
                61584, 44112, 54200, 47300, 604,   32872, 39588, 9040,  30180,
                59120, 47116, 41924, 51932, 61872, 8012,  49960, 13768, 27556,
                51708, 62932, 43704, 33936, 49224, 13696, 59732, 36544, 63640,
                44404, 58544, 37180, 10280, 40068, 45424, 26944, 4008,  49236,
                33324, 1560,  45116, 57620, 9348,  32132, 7040,  27192, 36164,
                56536, 62192, 52196, 15632, 60452, 55016, 36796, 63628, 41712,
                56636, 18544, 45464, 4284,  58548, 10144, 50676, 1472,  60492,
                8476,  43104, 64744, 55312, 14708, 41584, 53544, 63988, 31496,
                8772,  14172, 61692, 54316, 45272, 33964, 7532,  2416,  31952,
                21036, 61120,
            },
            {
                11836, 65200, 23400, 48288, 27916, 53636, 38312, 24736, 16552,
                11060, 37280, 64908, 25776, 14108, 21280, 43816, 18688, 2420,
                15944, 33488, 3564,  23740, 56672, 4788,  64016, 29428, 13008,
                62732, 52516, 16568, 12144, 60040, 30340, 15000, 10328, 20688,
                58756, 33168, 54420, 41568, 25628, 35052, 16716, 40840, 27504,
                51216, 3660,  24960, 65412, 11884, 51012, 62376, 14788, 43688,
                3832,  25864, 12668, 28916, 22852, 17596, 60160, 45568, 57280,
                32260, 8436,  15728, 60864, 28228, 5964,  42356, 25616, 17452,
                8220,  52000, 2772,  23044, 32896, 64708, 15744, 54416, 59292,
                20056, 63584, 12376, 29476, 61204, 17100, 51624, 12940, 42284,
                18704, 10124, 23020, 34732, 29016, 20984, 9924,  24844, 6516,
                35280, 26844, 13684, 39688, 33176, 17304, 36028, 13060, 39392,
                16484, 52440, 5380,  25908, 29784, 6656,  47772, 16344, 23080,
                40764, 27772, 20588, 11080, 17292, 28572, 49220, 55144, 43200,
                27900, 47764,
            },
            {
                15540, 32272, 51688, 17528, 34344, 3328,  62496, 30120, 58384,
                52120, 28032, 55856, 2980,  50688, 59624, 6576,  52924, 30664,
                58608, 48168, 12080, 52372, 9120,  31880, 37620, 1308,  54828,
                39548, 26872, 44344, 21388, 53796, 1556,  34928, 63412, 45928,
                17380, 48660, 9412,  62456, 29480, 57920, 12084, 63956, 36756,
                18472, 14112, 34484, 20044, 42844, 27080, 2896,  23632, 53572,
                18712, 36648, 64780, 5992,  44068, 34452, 25264, 3324,  11004,
                21916, 41188, 3972,  52340, 19408, 10792, 31188, 50960, 596,
                39604, 30152, 14512, 60468, 48880, 7296,  22028, 37844, 9872,
                28212, 52240, 35568, 42700, 3492,  21516, 55484, 45968, 1228,
                29996, 46976, 39700, 7508,  1440,  43696, 48312, 58728, 16040,
                51632, 61788, 21692, 49864, 5716,  65464, 54248, 26560, 57168,
                21564, 34104, 12564, 39068, 58164, 19000, 35184, 60192, 2932,
                55808, 33616, 38616, 1864,  63588, 41500, 36920, 24008, 10368,
                52084, 4632,
            },
            {
                45752, 58244, 1144,  39300, 44144, 12392, 21076, 46784, 676,
                43092, 19920, 9488,  41464, 33748, 11216, 37500, 46544, 9264,
                21068, 65072, 36400, 17464, 46620, 21480, 58720, 42752, 18512,
                6280,  35452, 61140, 8964,  40496, 48420, 24892, 52228, 3524,
                30656, 39436, 23684, 15348, 4956,  44180, 52916, 31240, 7396,
                45716, 58420, 39948, 5452,  30624, 57300, 46340, 38124, 60360,
                7872,  31484, 57068, 47788, 1752,  54596, 30544, 39824, 49048,
                35684, 58668, 25464, 45392, 37880, 65088, 56976, 46688, 21348,
                62420, 54436, 26420, 41904, 18376, 52776, 30680, 43064, 2800,
                46936, 16308, 5688,  25576, 48376, 40448, 34228, 24652, 62992,
                50756, 14176, 58204, 53284, 65228, 17968, 31436, 39168, 3644,
                30072, 43564, 8740,  55352, 30452, 23788, 46384, 3916,  31248,
                63008, 716,   50568, 44396, 61952, 9676,  51348, 25140, 43188,
                5744,  48432, 52968, 25552, 57140, 5040,  14524, 62416, 19056,
                56908, 37400,
            },
            {
                8680,  29012, 21944, 54776, 59608, 8484,  50604, 33164, 14932,
                6976,  61716, 48476, 16100, 62552, 28460, 24232, 55260, 39924,
                26556, 5088,  42988, 28788, 62344, 13672, 50532, 25128, 47884,
                11404, 31116, 2696,  56092, 28376, 18816, 5900,  41852, 12784,
                56012, 7228,  59788, 51508, 37480, 28,    20272, 23564, 48928,
                2460,  26848, 53100, 61016, 9580,  16820, 21292, 33544, 11612,
                48952, 40900, 15480, 20648, 37600, 12264, 62472, 7148,  18660,
                64048, 13960, 30308, 55528, 1816,  23316, 35216, 13928, 9204,
                34212, 45152, 4036,  11412, 35784, 824,   58084, 13984, 62484,
                39644, 23068, 60320, 53856, 14148, 64892, 10672, 6316,  37352,
                20812, 4052,  32372, 25360, 36080, 13576, 56008, 22536, 46036,
                11972, 59276, 4,     38012, 15356, 43136, 19232, 8000,  41508,
                47240, 28080, 17544, 23196, 32520, 4784,  28592, 37016, 12980,
                64820, 18516, 9248,  45828, 31308, 21940, 50380, 33280, 328,
                40776, 24760,
            },
            {
                63756, 42368, 5988,  14688, 26392, 36720, 64292, 23692, 54864,
                39376, 34988, 30408, 23032, 6340,  57144, 17768, 1808,  14500,
                50784, 32468, 56060, 1000,  39660, 7484,  33140, 4180,  55732,
                65292, 23972, 50152, 15652, 37032, 58256, 32892, 60544, 27040,
                21536, 45236, 34328, 18272, 47844, 61132, 9920,  57320, 38860,
                63144, 12292, 22804, 47744, 36988, 51468, 6144,  63168, 27864,
                884,   52408, 24988, 9444,  59468, 51764, 16104, 43248, 28312,
                52884, 6276,  47668, 10260, 17312, 43500, 4816,  28972, 60056,
                18752, 24048, 57428, 50044, 63996, 28568, 46172, 24552, 33232,
                56220, 7568,  31160, 36992, 0,     29032, 18400, 58904, 44104,
                28280, 61320, 41548, 11168, 44876, 3040,  51884, 7620,  63452,
                25216, 19632, 34608, 27016, 57824, 11288, 52288, 36460, 59896,
                10668, 54108, 37444, 3132,  48500, 14580, 57452, 45908, 21420,
                58672, 29988, 15188, 35908, 60344, 12008, 43900, 7972,  59688,
                30448, 17180,
            },
            {
                35332, 50720, 60868, 46548, 31616, 18868, 4384,  44984, 11352,
                25896, 58600, 1552,  52584, 43364, 47804, 35500, 63680, 44400,
                60628, 10620, 19352, 24340, 53340, 60296, 27600, 16448, 36464,
                45548, 20404, 42040, 62272, 7104,  10936, 44540, 14824, 36156,
                65128, 2452,  28676, 12380, 25988, 32144, 41120, 15768, 28984,
                33784, 17744, 43944, 1408,  32340, 14048, 41348, 55332, 44276,
                18132, 35432, 63832, 29656, 41552, 22564, 32848, 56788, 23904,
                1344,  37328, 20704, 33328, 61824, 39764, 50496, 55236, 47644,
                38168, 6136,  31592, 15896, 39904, 20216, 10312, 4456,  50836,
                18004, 11712, 49364, 20464, 44860, 56388, 33024, 51844, 15500,
                8644,  55004, 17244, 50072, 22032, 60752, 27940, 37084, 32732,
                49112, 53696, 41824, 64064, 46964, 3228,  61900, 25640, 14672,
                20824, 6796,  56624, 64584, 41060, 19696, 53652, 8,     39976,
                7000,  50732, 42332, 1272,  19720, 52476, 38956, 25820, 46272,
                54132, 12268,
            },
            {
                1888,  20040, 10652, 38392, 2356,  57484, 40472, 29144, 60460,
                18256, 46472, 13284, 20388, 38908, 4096,  12404, 31004, 22416,
                5884,  37336, 49012, 15432, 34984, 44520, 10436, 57820, 40096,
                32,    13364, 32524, 26468, 46624, 53600, 23140, 660,   49500,
                52940, 10292, 43008, 54448, 63828, 6836,  45532, 50572, 4412,
                53908, 8820,  56468, 28404, 64560, 24712, 58856, 4312,  23152,
                12868, 54528, 6408,  45220, 2588,  49440, 4884,  10452, 46240,
                60708, 41412, 51136, 58488, 27396, 7716,  15224, 24984, 1220,
                12568, 62996, 43644, 8188,  23236, 55964, 61392, 37384, 43992,
                27768, 64368, 40812, 58508, 9584,  26068, 39708, 4512,  48240,
                23980, 38640, 572,   30280, 57780, 9628,  18764, 42760, 14100,
                2580,  17092, 10160, 6076,  22080, 33480, 29680, 40228, 50360,
                34712, 45120, 25168, 12048, 29884, 34340, 63044, 10744, 27496,
                33336, 22708, 62136, 56224, 4620,  28724, 65076, 16116, 3604,
                21456, 48784,
            },
            {
                32576, 55892, 28224, 23320, 53072, 48252, 15820, 7496,  51120,
                3636,  37156, 54488, 65396, 32232, 25680, 58536, 53536, 41644,
                29448, 57900, 45972, 64492, 3184,  21836, 50052, 19080, 29568,
                8460,  59460, 51628, 3168,  19448, 39016, 61516, 31364, 17252,
                38568, 20028, 58096, 4668,  35604, 21308, 24828, 13252, 60124,
                36148, 21604, 49432, 6996,  38384, 19184, 10556, 30252, 49900,
                39320, 32452, 61064, 14756, 26856, 36868, 19476, 64760, 34756,
                14984, 26016, 11980, 2428,  45032, 21908, 64312, 31396, 41592,
                51720, 27872, 54096, 34656, 47120, 5400,  32104, 15096, 54832,
                3036,  34108, 6192,  16612, 47656, 2640,  21880, 62048, 12656,
                35152, 63808, 46276, 6664,  34140, 39512, 47308, 4752,  61932,
                57380, 38904, 31580, 51092, 13788, 55944, 17592, 2072,  5780,
                57888, 31828, 16348, 52368, 8196,  44096, 24060, 16700, 48000,
                54952, 12652, 37556, 25348, 48568, 34128, 10640, 58060, 36772,
                41780, 62644,
            },
            {
                15620, 40300, 7044,  65028, 12920, 34176, 61828, 22380, 31080,
                42176, 10324, 28012, 6740,  15780, 51020, 8908,  19648, 304,
                16304, 25524, 8148,  12748, 31048, 42064, 5504,  63564, 53172,
                46908, 22860, 35768, 64296, 12696, 29452, 4812,  57544, 8356,
                47136, 24240, 30484, 40560, 16244, 52272, 56784, 1724,  43404,
                26196, 40520, 61652, 15952, 43152, 52864, 46712, 36088, 62040,
                2316,  20136, 47676, 8892,  58648, 51372, 40580, 54348, 29848,
                7868,  56424, 18508, 31732, 53560, 37736, 10920, 56768, 35404,
                16884, 21320, 3488,  60324, 13124, 52184, 42532, 25920, 19748,
                46952, 22744, 52980, 30288, 60720, 35964, 54376, 42076, 31040,
                58276, 19576, 26992, 53620, 14604, 64628, 25416, 52168, 21392,
                28764, 45580, 24148, 60696, 36912, 43728, 48412, 63768, 22836,
                42024, 61648, 1536,  38472, 59560, 3628,  50440, 36216, 60088,
                2456,  44624, 8944,  17012, 41128, 6564,  23372, 47592, 29836,
                5824,  26496,
            },
            {
                58656, 45368, 50116, 17760, 43208, 26204, 9860,  44676, 55476,
                62860, 33984, 23280, 48232, 61276, 40324, 34680, 45016, 63156,
                49896, 33700, 39516, 56592, 27132, 60892, 37412, 25060, 33452,
                15456, 41084, 6528,  55536, 43724, 48896, 25448, 41884, 35004,
                63080, 3452,  13752, 61220, 48644, 9656,  37940, 32612, 65008,
                18948, 10688, 3380,  33520, 57944, 280,   26704, 7908,  15784,
                57324, 28292, 42308, 24116, 31688, 17060, 12700, 76,    21476,
                43908, 49356, 35832, 63100, 5032,  48220, 19144, 2796,  44600,
                59028, 9636,  39264, 25024, 17904, 29528, 2276,  63816, 9020,
                59384, 39024, 13500, 25012, 43192, 10876, 15928, 27888, 7800,
                3760,  43484, 11652, 22936, 49372, 2500,  31180, 11436, 35912,
                8400,  54300, 1496,  19044, 27964, 7276,  11392, 53364, 28832,
                9428,  19312, 47680, 27224, 21696, 56140, 29320, 6596,  41304,
                20392, 31848, 63796, 29600, 53988, 61360, 19376, 55836, 13796,
                52012, 9312,
            },
            {
                34752, 3632,  29756, 36788, 188,   56896, 5508,  37900, 19928,
                1116,  13688, 56952, 43960, 2260,  21884, 11596, 28428, 7072,
                60120, 21120, 2600,  51616, 18348, 46252, 14328, 2156,  10896,
                56960, 27808, 18156, 31564, 9484,  15104, 20780, 54744, 11840,
                27464, 51316, 33028, 43956, 2544,  29084, 17580, 46896, 5948,
                30604, 52020, 47368, 23008, 12692, 31324, 63104, 21828, 44616,
                52100, 11308, 37112, 55268, 5512,  60608, 45528, 27764, 62616,
                39220, 3712,  23736, 15976, 41872, 28900, 60440, 24012, 6960,
                49004, 30884, 65080, 50368, 36600, 57956, 40224, 48584, 12680,
                35356, 1084,  62984, 49952, 5220,  20952, 65340, 48800, 52648,
                17436, 56532, 37932, 61484, 41676, 17016, 56868, 43612, 59804,
                16208, 41224, 65036, 5320,  57612, 39312, 25336, 35128, 45668,
                14452, 36256, 51132, 12664, 41840, 17764, 10372, 62712, 14776,
                24752, 57644, 49984, 3424,  14276, 45020, 35336, 620,   43184,
                61676, 23984,
            },
            {
                20484, 11588, 53616, 59408, 21616, 32224, 51992, 47520, 16068,
                27044, 50452, 38464, 18360, 30292, 59100, 54188, 47376, 36916,
                13356, 54992, 43836, 10528, 35284, 7352,  54616, 48976, 38844,
                62684, 50480, 1236,  60588, 37592, 52164, 63868, 2104,  45240,
                16848, 59184, 6296,  20256, 25400, 62912, 58744, 22772, 55152,
                14184, 59616, 27816, 37696, 53876, 18176, 49020, 39832, 33628,
                4128,  18424, 65196, 1668,  21900, 34648, 52976, 7324,  33036,
                57772, 11244, 52692, 58840, 8264,  13440, 33696, 54304, 27440,
                40732, 14908, 220,   45312, 10828, 7300,  21276, 32448, 57016,
                27996, 45244, 17948, 31964, 56080, 37908, 668,   34308, 24292,
                45964, 31708, 1376,  28476, 8536,  20556, 34604, 6204,  49184,
                26668, 12740, 33564, 46728, 50384, 15212, 20552, 59208, 336,
                56028, 65276, 30612, 4684,  60640, 45728, 32212, 38884, 53188,
                46844, 5664,  18628, 38688, 26636, 51424, 8352,  31060, 38028,
                17268, 47340,
            },
            {
                39592, 63724, 25528, 14600, 41752, 63040, 11364, 29356, 59496,
                41428, 8504,  63464, 5192,  36336, 14976, 25852, 4004,  18464,
                41148, 24860, 30064, 63772, 23252, 58968, 31748, 17064, 21524,
                5332,  43040, 25692, 46524, 22468, 33792, 5936,  30368, 39920,
                22832, 37828, 56036, 35444, 53176, 39988, 12504, 600,   41368,
                35772, 44640, 6808,  64136, 42080, 9736,  5372,  55568, 13724,
                59720, 25840, 49664, 44264, 39292, 10632, 48728, 25324, 14472,
                19948, 44824, 30540, 26244, 45772, 38724, 51044, 63484, 11608,
                20048, 56208, 33348, 22656, 28048, 62744, 53332, 16872, 24220,
                4676,  55100, 41028, 8284,  26868, 44800, 58584, 12788, 40012,
                63304, 9684,  50848, 36380, 55196, 45332, 62368, 23928, 416,
                38416, 52828, 21184, 29536, 9660,  42628, 63228, 31108, 6284,
                24544, 40036, 8432,  54560, 34740, 23328, 49768, 1108,  27740,
                35160, 10788, 43096, 32796, 60300, 11664, 21352, 64620, 27324,
                5392,  56380,
            },
            {
                1820,  43848, 8368,  49064, 5000,  18460, 36096, 23228, 3392,
                53296, 32456, 24604, 46656, 10204, 51244, 42820, 64840, 57548,
                32560, 5836,  15844, 48460, 732,   41044, 26612, 44560, 60100,
                30112, 35664, 13904, 8192,  58136, 41668, 18668, 50188, 61708,
                8672,  48212, 11056, 15484, 45600, 7792,  33752, 50820, 25236,
                9176,  16972, 20836, 1976,  26000, 60792, 36480, 23536, 29812,
                41728, 8320,  32176, 15340, 28652, 63288, 17720, 41564, 61052,
                1992,  37492, 6084,  65320, 20960, 1156,  17448, 3960,  36036,
                47028, 61464, 5720,  51560, 41828, 14340, 1596,  44116, 38328,
                51168, 11616, 21832, 61548, 14900, 51664, 19180, 29800, 5624,
                21688, 14468, 58480, 25000, 5076,  13784, 51836, 29640, 18968,
                63216, 44904, 4120,  61364, 36176, 2884,  52056, 17664, 49200,
                43864, 21260, 16076, 26500, 2624,  63660, 13884, 7896,  59040,
                22052, 65160, 54664, 1316,  24056, 56576, 40756, 48924, 54476,
                12916, 31984,
            },
            {
                50068, 15996, 33584, 38272, 27292, 46028, 55864, 7444,  64876,
                45144, 12548, 20252, 54792, 60336, 31960, 828,   22400, 9188,
                52532, 45868, 60512, 37212, 19724, 52688, 12428, 3332,  55644,
                9844,  19336, 64432, 54068, 3428,  28116, 12096, 55108, 26112,
                224,   31528, 65512, 27992, 3724,  58000, 19272, 46428, 28928,
                62340, 57172, 48556, 34632, 50940, 15676, 45160, 2892,  63916,
                47816, 20888, 58412, 51672, 6600,  57052, 3568,  35608, 54980,
                47716, 51620, 16960, 34728, 54772, 47908, 30812, 23132, 43492,
                8420,  25704, 38200, 18536, 60240, 47176, 35804, 59068, 7024,
                64600, 29660, 48344, 36532, 3980,  32856, 9212,  61288, 53576,
                42484, 34844, 18440, 46768, 39612, 32700, 10104, 40916, 55512,
                7948,  32156, 16768, 23356, 54852, 26540, 13132, 33344, 37916,
                10996, 52756, 58476, 47380, 37540, 19960, 42856, 55680, 16944,
                39968, 29172, 14636, 47612, 36124, 6096,  16416, 2576,  45260,
                24884, 59692,
            },
            {
                21888, 29176, 57304, 62336, 2752,  51460, 13800, 31044, 39296,
                16532, 35532, 1948,  41004, 27180, 17216, 39472, 48624, 35624,
                12388, 27696, 4220,  55960, 8832,  33588, 65516, 36388, 24600,
                48088, 51740, 32608, 38584, 16920, 48688, 59848, 36688, 14940,
                42964, 21016, 52044, 38480, 23436, 31012, 63576, 14920, 53508,
                3256,  38664, 31736, 10348, 56192, 29148, 19744, 53528, 11104,
                38220, 524,   35304, 12000, 24212, 46228, 30964, 21624, 9896,
                27884, 23096, 42860, 12312, 8916,  61332, 40596, 53468, 58360,
                16152, 29608, 53912, 12092, 4376,  30420, 26108, 10424, 19644,
                33432, 16280, 1924,  24740, 57076, 43084, 47500, 26172, 3124,
                48692, 60368, 27968, 2676,  65120, 22392, 59576, 47824, 14992,
                36468, 57432, 10900, 59544, 39536, 47284, 7776,  58072, 61836,
                27608, 5588,  32388, 41408, 9824,  51784, 30440, 25164, 45500,
                3648,  50592, 8884,  20120, 43768, 63220, 29844, 34436, 19516,
                9192,  36840,
            },
            {
                6440,  54028, 19076, 11160, 23784, 60208, 42740, 20652, 49708,
                25376, 56504, 62204, 48964, 6252,  13404, 56204, 29088, 19640,
                62892, 39888, 22940, 43012, 14740, 28304, 46168, 7140,  15524,
                39720, 424,   23208, 44892, 10364, 31232, 24188, 6688,  46432,
                33840, 59084, 5748,  17484, 49712, 43560, 10448, 36820, 6472,
                22620, 43708, 13036, 24196, 40364, 6604,  59484, 34156, 26508,
                17108, 55792, 62248, 42672, 18940, 53556, 38056, 13572, 64040,
                5160,  32556, 60372, 56512, 28920, 25124, 14216, 4888,  32808,
                49084, 2144,  63236, 40816, 34296, 57584, 21788, 50332, 42408,
                60772, 45936, 53168, 40348, 63320, 11788, 22584, 38656, 16516,
                31508, 7884,  55876, 11452, 50168, 16832, 4364,  27228, 2440,
                24416, 41612, 49324, 28628, 1916,  18944, 42128, 22328, 3696,
                45552, 19288, 64332, 13544, 60112, 4984,  34268, 62188, 11376,
                36320, 61080, 31980, 57308, 26432, 50936, 12516, 60420, 52892,
                42496, 64156,
            },
            {
                46372, 540,   40748, 45028, 29968, 8940,  34600, 936,   58268,
                4828,  10044, 30244, 22724, 34672, 63336, 45340, 7380,  59232,
                2404,  16788, 49424, 31620, 61804, 50924, 21972, 57012, 42224,
                62652, 27620, 5632,  56828, 62096, 42404, 2308,  64256, 19136,
                56432, 12704, 40340, 61428, 2648,  55060, 26792, 41656, 60896,
                49988, 18652, 58436, 65240, 784,   47408, 14016, 43072, 50508,
                5352,  45824, 29432, 4500,  27032, 61440, 1432,  43684, 57716,
                40652, 15916, 680,   38412, 5800,  44432, 18892, 64568, 10196,
                37372, 21176, 44812, 24504, 8932,  48540, 15044, 55032, 2984,
                27744, 13032, 8124,  30736, 17712, 5388,  35092, 54696, 63900,
                12912, 40988, 23576, 36560, 30188, 42656, 54524, 34124, 63976,
                45320, 20300, 5916,  14156, 35144, 64888, 54164, 29960, 14584,
                55968, 36744, 1516,  29412, 23160, 17788, 48192, 232,   52784,
                21000, 41876, 4660,  17416, 38360, 7988,  22868, 39980, 4132,
                27948, 13944,
            },
            {
                34564, 25176, 52144, 14944, 37076, 55008, 17044, 61988, 28452,
                46516, 37204, 44044, 18384, 53428, 3212,  24476, 37552, 33160,
                52116, 25800, 58032, 11132, 1360,  39092, 4844,  17868, 30792,
                11224, 53200, 18880, 13992, 35036, 20676, 51776, 38184, 9996,
                29200, 47752, 25772, 9052,  32912, 13940, 21240, 968,   34232,
                30556, 8360,  27724, 51660, 35792, 22048, 30828, 8032,  60616,
                23344, 33076, 14564, 40084, 50748, 8704,  16600, 33652, 49216,
                24980, 52848, 46388, 21464, 49844, 35200, 52136, 39752, 27852,
                60176, 13692, 56816, 17884, 664,   65436, 37528, 6368,  40040,
                23652, 35468, 56160, 21200, 49032, 59256, 28464, 1732,  45412,
                20696, 52208, 368,   44996, 58040, 19628, 8580,  38680, 12052,
                30472, 53836, 62316, 51292, 43736, 24592, 10332, 34192, 50620,
                40808, 24908, 49316, 54204, 44688, 56860, 38528, 14960, 28172,
                56176, 24460, 13344, 49472, 59268, 1672,  55184, 32068, 48436,
                17800, 57704,
            },
            {
                61116, 8136,  32100, 65384, 3220,  47436, 22896, 40344, 11740,
                52212, 15372, 7012,  61076, 11444, 42140, 50724, 15220, 9768,
                41052, 45540, 6572,  35724, 20068, 54652, 26124, 60152, 48996,
                34676, 58996, 46540, 29360, 49716, 7468,  26640, 54472, 44244,
                1112,  22092, 52428, 36272, 45192, 64592, 48552, 58152, 16500,
                55256, 46748, 3872,  15856, 42376, 54836, 63344, 38972, 18248,
                53112, 9548,  65016, 21344, 35944, 46868, 59540, 29040, 19504,
                6928,  10380, 30048, 61688, 13340, 57836, 1940,  22600, 46664,
                3612,  50552, 30840, 52908, 43244, 26712, 32436, 18720, 58556,
                50892, 64168, 152,   38120, 43524, 14404, 51176, 9940,  57832,
                25892, 33468, 60956, 13880, 5752,  25712, 51788, 60456, 48320,
                17272, 580,   37408, 31780, 7088,  16816, 47964, 868,   63112,
                7900,  16976, 11188, 34592, 6608,  26416, 9476,  63496, 33064,
                7320,  44488, 64548, 34488, 28780, 45732, 15328, 62548, 10024,
                22140, 38768,
            },
            {
                12184, 49720, 20916, 58052, 26520, 5912,  50816, 32000, 19636,
                64596, 26552, 55264, 39748, 32152, 27628, 19372, 65056, 55500,
                21064, 28732, 13828, 64280, 47820, 43792, 12708, 37296, 2536,
                8092,  24936, 38240, 3012,  41036, 58264, 13116, 33380, 16548,
                63188, 31868, 4468,  57376, 18208, 29516, 7132,  38352, 25228,
                12148, 39496, 61396, 33004, 19812, 5036,  11780, 28424, 1248,
                48364, 41688, 57008, 2632,  54672, 11496, 23872, 4236,  56212,
                39576, 64932, 34136, 17988, 26472, 8524,  31500, 54976, 16808,
                34664, 25872, 5448,  36092, 12256, 61148, 46884, 9368,  30016,
                44604, 16888, 10944, 26232, 61768, 32348, 19564, 36644, 41940,
                6432,  17620, 39140, 48828, 64128, 32844, 15260, 4868,  23284,
                57284, 9668,  26340, 21712, 56440, 61124, 38044, 27140, 21144,
                57164, 30732, 61792, 42344, 20604, 58900, 40492, 46596, 18520,
                51448, 37536, 3420,  20524, 10704, 41524, 25652, 35392, 51952,
                2680,  30020,
            },
            {
                41652, 4640,  17264, 43332, 38820, 13720, 56700, 8036,  42432,
                2132,  34020, 22116, 49520, 1128,  58972, 46236, 5644,  31360,
                196,   60548, 53144, 23796, 31988, 8688,  29436, 63140, 45204,
                15136, 20940, 65000, 10112, 17176, 61620, 23460, 4864,  40072,
                49492, 58864, 14288, 41936, 24172, 11292, 51648, 43324, 4576,
                63504, 22372, 44768, 9856,  26596, 49476, 57220, 45036, 24888,
                36372, 12892, 30536, 26200, 17892, 32064, 63256, 45172, 36496,
                12640, 54344, 43160, 3352,  38000, 48016, 63728, 12020, 43156,
                62372, 9572,  40156, 58388, 21040, 3896,  51752, 23092, 13808,
                5064,  33896, 48048, 54604, 7600,  3412,  24432, 65052, 48184,
                29728, 55120, 9252,  28184, 21956, 41132, 46124, 36044, 29028,
                43504, 39676, 60180, 46760, 12180, 4492,  52620, 44832, 13616,
                38848, 47104, 2292,  51636, 14124, 4268,  30036, 22464, 1972,
                12476, 58692, 30728, 54356, 48224, 60136, 18768, 5880,  43760,
                56596, 47652,
            },
            {
                63492, 35148, 53704, 9956,  29580, 63076, 35808, 24848, 59320,
                45988, 5104,  57916, 16968, 9232,  36172, 23612, 13056, 48872,
                43740, 36956, 17520, 40484, 2212,  56428, 16928, 22204, 52872,
                32528, 56320, 43228, 51220, 30572, 47064, 36752, 56048, 27692,
                10808, 19760, 37848, 54224, 2444,  62508, 34792, 20172, 28692,
                53692, 35900, 1364,  52444, 60164, 37420, 15556, 33264, 62720,
                20724, 58960, 5968,  49936, 38580, 42396, 372,   15364, 50320,
                27320, 22412, 6300,  51728, 59808, 20832, 40724, 25384, 6744,
                19480, 53992, 46176, 15528, 28904, 41968, 55948, 38332, 62300,
                41096, 59432, 22676, 29604, 40256, 46112, 53768, 12168, 4140,
                15512, 62388, 35712, 3828,  54016, 10664, 1368,  62464, 20000,
                52224, 3264,  14884, 34552, 41804, 30168, 19472, 33068, 3816,
                60412, 22908, 27448, 37040, 65344, 33484, 48864, 54860, 61640,
                42008, 26564, 16668, 7768,  23992, 504,   37256, 64968, 27824,
                14304, 23524,
            },
            {
                1780,  27076, 59976, 22312, 45636, 772,   18492, 49412, 14952,
                28792, 37812, 12836, 44940, 29548, 53028, 62112, 39232, 26300,
                57576, 10984, 4824,  50576, 34708, 59748, 41600, 49596, 5436,
                39460, 960,   26920, 34148, 6184,  21412, 1824,  45284, 60196,
                7940,  25592, 46104, 6256,  31244, 47364, 15560, 59220, 8924,
                49080, 18252, 14104, 30508, 23724, 41264, 2408,  9300,  53640,
                3656,  16576, 46212, 62008, 8416,  52816, 57896, 20376, 9000,
                60216, 32184, 16956, 45980, 29244, 15076, 136,   36012, 58892,
                49660, 31888, 23232, 64320, 7812,  34116, 17400, 1748,  26440,
                52600, 19168, 2688,  13204, 57984, 17776, 34368, 39428, 59472,
                23188, 46644, 43780, 18624, 59252, 49940, 26924, 56184, 12608,
                32336, 65424, 18864, 50288, 8380,  25280, 64300, 55348, 10140,
                43168, 6880,  54012, 18748, 8896,  44212, 16284, 25476, 9784,
                35308, 50080, 45400, 39608, 56892, 31764, 12756, 53736, 8616,
                33568, 19196,
            },
            {
                38264, 46776, 15068, 7160,  51392, 33552, 55300, 40092, 10068,
                60844, 53632, 24100, 63424, 40976, 19004, 2732,  15736, 7772,
                33872, 22504, 62540, 27540, 14540, 6852,  25412, 11268, 28708,
                58452, 19240, 13816, 54940, 63524, 11728, 53312, 18056, 30284,
                42804, 64916, 33704, 21620, 60316, 26876, 39572, 55652, 3208,
                32620, 42100, 58456, 47704, 7256,  64668, 18956, 46920, 29596,
                39084, 43020, 33852, 21520, 13764, 24800, 29524, 34548, 40500,
                47640, 2248,  63200, 35232, 10276, 56948, 53280, 45364, 28532,
                3284,  13052, 37988, 980,   50272, 59700, 11844, 48180, 30952,
                8096,  45916, 37472, 63692, 50304, 9400,  27788, 20900, 51224,
                32180, 1132,  25508, 12896, 30720, 37996, 16496, 42572, 8088,
                47600, 5764,  27732, 54728, 59680, 1448,  48448, 16908, 35164,
                50404, 15344, 32296, 41488, 56372, 1436,  60496, 38112, 5440,
                19456, 64252, 4168,  14384, 61724, 50504, 21556, 46652, 39952,
                58736, 50740,
            },
            {
                11252, 57532, 30844, 40932, 61400, 12436, 23660, 3904,  31584,
                47624, 20060, 7420,  34520, 4280,  56864, 31660, 50260, 61172,
                52788, 18796, 46520, 42632, 54580, 19936, 64116, 36556, 46400,
                62184, 9208,  44720, 37680, 24584, 49056, 40928, 35324, 13792,
                2664,  51456, 16268, 49052, 10604, 240,   44420, 13160, 22892,
                64328, 25932, 5360,  54332, 12352, 34944, 56016, 23016, 61224,
                51652, 10868, 27536, 55676, 37288, 4636,  44724, 64348, 5788,
                25816, 55424, 13396, 24560, 42084, 7244,  32664, 22696, 16636,
                61888, 42124, 26816, 56376, 19208, 43924, 23824, 35816, 64900,
                15168, 55744, 32696, 25292, 43116, 796,   62084, 6632,  42688,
                10568, 57244, 52704, 40612, 64400, 6456,  21220, 35008, 24308,
                58596, 38080, 44484, 22652, 36724, 13920, 40632, 21672, 28848,
                57976, 25804, 63000, 11680, 23964, 28612, 51532, 13192, 31920,
                53980, 22980, 29236, 34324, 9700,  27280, 42756, 2548,  16752,
                29876, 6552,
            },
            {
                54060, 24296, 35936, 3536,  19684, 28264, 44708, 64392, 17428,
                5636,  42116, 27576, 50952, 11668, 47004, 21868, 43348, 28656,
                40148, 1580,  30912, 8828,  37796, 48216, 31668, 4484,  15948,
                23624, 50540, 3460,  60404, 15740, 28640, 7648,  62140, 24152,
                55064, 39052, 6988,  56400, 36224, 18100, 52920, 30236, 37548,
                46264, 16512, 39972, 21228, 27568, 43880, 31728, 4548,  13376,
                25352, 6724,  64824, 2568,  49504, 59944, 16652, 11336, 52140,
                18664, 43932, 38072, 58712, 50156, 19452, 65284, 10964, 37116,
                55176, 5308,  47692, 33040, 9640,  29416, 53916, 6128,  20656,
                42352, 10504, 4800,  21436, 16092, 36596, 31152, 54884, 14812,
                37776, 20084, 7612,  34016, 47984, 3296,  54796, 61576, 50428,
                30352, 2724,  16188, 10888, 31216, 45984, 5944,  62468, 37712,
                48,    46352, 4748,  49160, 36404, 20512, 45704, 40784, 57352,
                47428, 2160,  43544, 53260, 38204, 18372, 5260,  60236, 25564,
                63128, 44684,
            },
            {
                44,    17476, 64812, 48900, 53480, 8880,  57048, 34612, 50640,
                37144, 62076, 56292, 15192, 25340, 64444, 36352, 10028, 4648,
                13876, 57064, 65440, 24348, 12484, 58884, 2024,  43676, 53584,
                34512, 30224, 20544, 42056, 32860, 356,   57336, 20452, 46260,
                32160, 11540, 28204, 63612, 41812, 25276, 61980, 7364,  49552,
                10904, 60340, 816,   36420, 62036, 9704,  50212, 38084, 17976,
                57636, 47712, 31464, 19264, 41328, 32908, 22544, 56488, 36428,
                30528, 9564,  21568, 1212,  28144, 4664,  39900, 51980, 46636,
                8428,  21368, 14260, 63384, 40248, 3480,  61108, 38132, 49560,
                58832, 28700, 51832, 59968, 45384, 56660, 49280, 24128, 46876,
                63788, 28252, 60140, 15940, 23476, 29116, 44864, 14072, 10252,
                19332, 41636, 55736, 61200, 51280, 26888, 56544, 9324,  52700,
                12744, 19988, 39848, 59260, 8084,  64428, 6036,  17540, 26452,
                11312, 60080, 15880, 6772,  63392, 47544, 33224, 55596, 36668,
                13456, 21716,
            },
            {
                33952, 43584, 26716, 13836, 37392, 42576, 1168,  25560, 13088,
                22572, 2188,  31008, 44620, 39040, 684,   17560, 59860, 25708,
                48736, 34776, 17144, 5464,  51544, 28364, 22104, 56588, 39680,
                11580, 57932, 5768,  46960, 54660, 11016, 38300, 50120, 4644,
                59892, 19412, 44888, 22756, 3888,  32504, 57756, 20720, 4304,
                34028, 56132, 29664, 45156, 51908, 15956, 1988,  59908, 45056,
                35048, 40564, 15004, 53780, 9224,  46408, 1392,  27364, 48844,
                3740,  62572, 53892, 32284, 47784, 61004, 14716, 24424, 29400,
                33940, 59872, 51140, 24696, 16776, 45564, 12948, 27084, 456,
                17644, 33876, 40396, 12840, 27048, 4360,  10872, 18736, 6244,
                34796, 2008,  49556, 43276, 9392,  58952, 39640, 292,   27136,
                64012, 47160, 7632,  34280, 2040,  19756, 42516, 32632, 24168,
                60840, 44252, 30044, 16336, 34120, 53000, 31028, 656,   62324,
                36052, 40076, 20716, 25796, 57988, 11488, 23432, 51912, 8764,
                49044, 40140,
            },
            {
                56096, 9976,  6676,  58212, 21948, 31132, 15932, 59684, 46824,
                55112, 10304, 18680, 58772, 8160,  51880, 55248, 32288, 45484,
                21100, 54040, 38892, 44508, 60952, 33816, 14296, 7424,  18556,
                27108, 63776, 36836, 25684, 17928, 65376, 43376, 27304, 16464,
                35040, 1240,  52392, 13520, 48012, 9044,  15508, 43496, 27812,
                52336, 18496, 13292, 6524,  24916, 20248, 40852, 27144, 8592,
                21684, 420,   61696, 24300, 29288, 63372, 13316, 42560, 59388,
                15516, 39760, 45196, 17320, 8272,  35288, 43556, 2700,  57424,
                18096, 43024, 1512,  36448, 57176, 52108, 32364, 22496, 55296,
                46052, 7816,  61928, 2400,  35276, 65428, 39524, 29956, 57860,
                41952, 25868, 12528, 55724, 32060, 18924, 51320, 36732, 53720,
                32924, 21924, 39284, 25096, 63580, 16744, 49320, 3560,  15116,
                34884, 50884, 2432,  22968, 55516, 13368, 42840, 23688, 49652,
                8816,  32392, 46160, 50900, 30376, 1048,  41256, 15692, 31208,
                4872,  60716,
            },
            {
                28516, 50424, 32600, 47140, 4732,  62756, 52088, 38704, 7220,
                28336, 40944, 49468, 34424, 23440, 28180, 13068, 41468, 7100,
                62212, 10676, 26912, 552,   19416, 41192, 46780, 62080, 51944,
                1044,  48360, 8784,  14008, 51528, 30548, 22776, 8948,  62680,
                40884, 58160, 29872, 38100, 60660, 35292, 50876, 39136, 63084,
                23876, 41660, 47944, 64952, 57700, 33404, 53612, 63732, 30752,
                56564, 51208, 11808, 37388, 5640,  50312, 38540, 19700, 7888,
                34500, 23492, 12512, 60000, 26208, 55616, 20740, 63488, 6372,
                38624, 26952, 11000, 30480, 4472,  19828, 8480,  62584, 41580,
                14324, 24768, 48024, 20116, 53952, 22732, 44160, 14252, 51196,
                4044,  62528, 38712, 22084, 4728,  65104, 25260, 6560,  17404,
                57764, 4024,  14496, 52852, 11384, 37884, 58348, 28140, 65168,
                38816, 7252,  57540, 27604, 10188, 47572, 37840, 58808, 28752,
                18744, 56200, 2868,  13660, 37048, 44504, 19872, 65432, 26460,
                45832, 19108,
            },
            {
                38748, 22944, 63764, 16948, 40384, 27792, 10584, 19556, 44260,
                32588, 64068, 13976, 3672,  61684, 48280, 19980, 37484, 2228,
                50944, 15496, 31820, 56392, 50020, 9752,  23880, 36292, 32196,
                43164, 22616, 40216, 58936, 2208,  35480, 5472,  54308, 14220,
                47280, 25136, 7696,  55524, 17740, 26352, 1032,  12244, 56924,
                3476,  9332,  31332, 37928, 2728,  14400, 46892, 11492, 4300,
                17056, 26212, 45768, 59088, 16360, 33764, 58068, 24896, 52008,
                64080, 29440, 6156,  50868, 37192, 11396, 30084, 48268, 13424,
                54176, 61380, 49008, 44356, 64656, 39800, 47648, 34836, 3836,
                29488, 57032, 38608, 31744, 15800, 49892, 8388,  61048, 32724,
                20752, 16648, 52732, 46088, 35196, 14784, 48124, 42440, 12156,
                45444, 28488, 60264, 48496, 30096, 44712, 6112,  21640, 54264,
                11092, 18468, 46436, 41180, 63824, 20400, 3252,  15252, 52564,
                6412,  41536, 64424, 22792, 59932, 8232,  54000, 34468, 57364,
                3148,  14796,
            },
            {
                52480, 1592,  12124, 44816, 55420, 2376,  35256, 60968, 24176,
                6488,  53064, 21744, 38488, 43624, 5016,  57092, 64764, 29484,
                23260, 42152, 36112, 6720,  63908, 30148, 2768,  59172, 12644,
                16540, 29532, 55584, 19608, 45264, 61240, 48768, 37892, 32056,
                21136, 11192, 43680, 3404,  22148, 65112, 45800, 29720, 17248,
                36636, 21356, 54880, 26140, 8344,  43076, 22308, 36212, 49152,
                39508, 32992, 3396,  42908, 22004, 55232, 4060,  11076, 44436,
                268,   55400, 46996, 41156, 19220, 1912,  52332, 40524, 32432,
                22272, 16412, 7808,  23364, 13748, 27760, 57892, 11640, 53128,
                19156, 64076, 10420, 5572,  59072, 28320, 128,   25092, 55208,
                37252, 10220, 29520, 1008,  60648, 8804,  54404, 30228, 62560,
                35488, 8960,  41820, 864,   20364, 34992, 9528,  47620, 41452,
                30888, 24944, 60144, 32400, 5284,  51724, 34912, 45000, 62908,
                12400, 26324, 34172, 52936, 17508, 28316, 5528,  49332, 11920,
                42596, 61544,
            },
            {
                30092, 36004, 59448, 25368, 20620, 50664, 13912, 47276, 57028,
                16780, 1028,  59428, 26484, 31456, 11136, 16036, 34700, 8656,
                54704, 60824, 47540, 13484, 25632, 17496, 39960, 54544, 4564,
                62816, 50104, 6864,  33476, 26408, 12112, 17224, 57396, 1768,
                51696, 63820, 33992, 49828, 40552, 32168, 53764, 5924,  61264,
                49228, 44828, 59284, 16376, 50344, 62672, 29096, 5984,  60748,
                55052, 19800, 64920, 28268, 8060,  46788, 31272, 27096, 40376,
                16476, 20972, 32532, 4256,  64480, 57784, 23780, 9136,  45380,
                948,   36192, 51640, 33088, 54700, 17556, 5820,  24388, 44156,
                36828, 1800,  50908, 44944, 40992, 35744, 12684, 46828, 42512,
                6972,  48948, 58392, 43964, 27128, 40636, 22644, 1852,  19500,
                50996, 23244, 54928, 16000, 63088, 56304, 26284, 60600, 2864,
                51352, 14228, 560,   37080, 16584, 26656, 9504,  22088, 31416,
                38144, 48944, 4264,  10128, 47020, 39484, 62328, 21096, 37216,
                24804, 8900,
            },
            {
                19780, 5868,  42288, 33668, 7960,  30676, 38236, 5268,  29300,
                42292, 36936, 50512, 17936, 46968, 53496, 40640, 25460, 45616,
                17820, 3372,  21012, 58800, 44152, 52740, 47856, 20860, 27036,
                45072, 10228, 38700, 64340, 3364,  42272, 29272, 24132, 44776,
                6520,  28372, 16392, 58924, 13852, 9496,  20200, 42168, 24776,
                33548, 11796, 536,   40172, 34380, 53972, 18028, 25424, 41912,
                13152, 9360,  52204, 35992, 14424, 61832, 18572, 49524, 58780,
                35844, 61416, 25784, 14316, 43712, 34024, 16712, 60656, 27176,
                56840, 63980, 41420, 3472,  59800, 38852, 31176, 61096, 49368,
                15636, 33156, 26744, 23116, 17512, 62968, 56980, 30496, 18576,
                64676, 24400, 34064, 12296, 17368, 56384, 32828, 38672, 58296,
                4260,  26804, 37468, 32312, 5208,  40432, 13028, 17748, 33720,
                44512, 56652, 21084, 48920, 62284, 54740, 40008, 59796, 944,
                55360, 19768, 43488, 57828, 24312, 32716, 15088, 576,   31516,
                55772, 48140,
            },
            {
                39784, 53588, 15616, 46704, 56444, 61920, 17812, 51812, 65364,
                12416, 33444, 6700,  9316,  63736, 21284, 260,   59856, 49956,
                11880, 38928, 30992, 5348,  34516, 10884, 7264,  37748, 57080,
                35044, 24332, 14736, 53716, 21544, 51048, 7860,  59436, 40004,
                19740, 55188, 37576, 25408, 4908,  47548, 35872, 55816, 14904,
                7392,  64268, 27468, 20360, 4556,  10612, 45440, 58108, 1652,
                31068, 23656, 48008, 832,   40864, 53360, 34244, 2296,  13288,
                7168,  52608, 9968,  48684, 28432, 6568,  39196, 47508, 4600,
                18952, 12496, 28704, 20380, 46808, 10460, 2060,  41984, 21300,
                8008,  56244, 60396, 11956, 53272, 2660,  21148, 39056, 4336,
                14728, 53788, 3348,  21180, 63608, 5660,  48672, 10784, 15628,
                46980, 64844, 11852, 45124, 49336, 21928, 52448, 37672, 5568,
                64344, 27920, 8072,  42744, 12236, 30080, 6948,  48172, 25200,
                15876, 61424, 29976, 14004, 2720,  41756, 59400, 52016, 44692,
                16640, 62664,
            },
            {
                11848, 24096, 64456, 340,   11172, 26360, 40920, 2584,  21196,
                25192, 55364, 44700, 30004, 56408, 36100, 5920,  28356, 33192,
                63556, 24588, 53236, 41444, 27736, 65136, 15308, 31772, 52,
                17944, 60476, 5128,  46860, 31644, 36876, 62408, 15028, 34744,
                11768, 61344, 200,   43068, 53300, 29160, 59728, 1656,  50224,
                38736, 30848, 52744, 47076, 60356, 30380, 37636, 15320, 51260,
                62876, 38760, 16696, 59392, 25644, 10512, 21660, 64556, 46120,
                29216, 39452, 56332, 22316, 62032, 51920, 10892, 55388, 37152,
                31776, 44660, 7316,  50256, 35676, 25944, 52400, 65188, 12848,
                29008, 46416, 39528, 6092,  31224, 48444, 9408,  51644, 61300,
                36416, 28636, 41692, 46608, 36928, 51876, 25748, 61244, 42608,
                29952, 6824,  18836, 59816, 28796, 1984,  62312, 30416, 24032,
                10700, 38792, 53020, 34152, 23072, 58396, 18476, 42252, 11124,
                33808, 6024,  51772, 36588, 63552, 48676, 10856, 22476, 7432,
                27484, 3608,
            },
            {
                43216, 32792, 29036, 48956, 22384, 35440, 44964, 54584, 9576,
                47808, 61420, 15420, 40100, 24364, 12872, 51972, 42472, 15008,
                7660,  44932, 1264,  18640, 57732, 23060, 50432, 61448, 42812,
                48740, 29928, 41240, 58432, 18276, 10408, 45584, 2816,  54072,
                30876, 46312, 8560,  22480, 64044, 18012, 11356, 27024, 22208,
                57648, 18212, 43124, 13736, 23332, 55636, 7936,  21896, 33536,
                44340, 6264,  56020, 29868, 45040, 5892,  37960, 55412, 24496,
                43108, 3272,  17616, 36780, 760,   31268, 19956, 24876, 14544,
                61528, 53224, 24112, 62924, 14800, 56568, 18180, 33684, 38108,
                51536, 696,   19488, 64176, 42600, 34532, 27388, 43716, 22768,
                11380, 49516, 59864, 8676,  31168, 13856, 2372,  20520, 34772,
                56708, 39004, 52896, 24748, 9320,  42616, 15404, 55668, 46696,
                19496, 59316, 16212, 4324,  45932, 1876,  36328, 53792, 65248,
                46200, 39384, 21592, 8652,  26168, 18312, 29544, 56336, 38564,
                34760, 58796,
            },
            {
                20784, 8496,  57604, 39096, 4540,  59012, 14292, 32484, 28808,
                37604, 4832,  19996, 1884,  62320, 46100, 18684, 58604, 21856,
                55832, 36964, 61560, 9988,  46556, 4720,  39580, 12040, 21376,
                8876,  52488, 12596, 27584, 1068,  55996, 25780, 20396, 49564,
                26828, 17028, 50700, 35108, 31560, 39272, 45052, 62776, 41708,
                3952,  9440,  61896, 35352, 2628,  40824, 65408, 49000, 3600,
                27188, 20008, 35228, 12360, 63500, 51180, 31912, 15052, 8324,
                19924, 33236, 59904, 46504, 13436, 42400, 65336, 49932, 2256,
                40780, 10020, 33864, 160,   42936, 6424,  45556, 23164, 4516,
                58616, 25396, 36180, 14676, 24080, 56000, 16192, 6732,  58220,
                32936, 1524,  25132, 19020, 55760, 43300, 28536, 54120, 8868,
                22720, 236,   14632, 33388, 57728, 36000, 50656, 7360,  40644,
                1088,  48484, 30924, 63440, 26252, 50564, 14492, 28504, 20352,
                3448,  27216, 57184, 44280, 53940, 34204, 6376,  46140, 65032,
                13848, 49996,
            },
            {
                2760,  45456, 16876, 52888, 19532, 63420, 7180,  49764, 17072,
                59636, 43260, 50692, 28056, 33308, 8980,  38408, 30608, 2808,
                47828, 26772, 13652, 31472, 54816, 35780, 29316, 55712, 26040,
                3100,  64652, 36104, 22540, 40044, 63276, 32572, 43380, 6348,
                65256, 41284, 58100, 14012, 3080,  52200, 6856,  16108, 32444,
                36760, 48916, 25380, 6920,  51792, 26892, 18380, 11952, 54492,
                60380, 8776,  47596, 23040, 2636,  18844, 41432, 58172, 47836,
                62700, 50844, 5432,  26776, 57556, 8684,  35472, 28200, 45680,
                21628, 59132, 17200, 38988, 27848, 60096, 30388, 9464,  62108,
                16988, 43660, 54536, 49700, 10456, 3416,  62604, 47080, 18260,
                54312, 40196, 15444, 62124, 6028,  38560, 64536, 16492, 45764,
                50628, 63036, 41204, 48340, 3668,  18016, 26840, 11860, 61336,
                35068, 24676, 12820, 55024, 41312, 6584,  61860, 32020, 8988,
                59236, 49244, 16632, 1056,  12580, 60552, 40708, 1840,  17256,
                24924, 54708,
            },
            {
                62068, 26972, 34420, 12768, 30592, 42836, 24996, 55972, 1332,
                23408, 34920, 13472, 58116, 53800, 3756,  49576, 64836, 10916,
                34068, 17424, 51104, 43144, 20332, 2216,  16572, 45496, 58632,
                32644, 44192, 15472, 54668, 7996,  50472, 13524, 38376, 23496,
                10956, 3940,  28588, 21152, 61088, 10356, 24628, 47296, 53808,
                12976, 55896, 29940, 16580, 59116, 46676, 31816, 36500, 42624,
                15072, 40108, 53400, 58740, 43440, 28368, 4848,  25924, 36296,
                11544, 30056, 16020, 40292, 23640, 53628, 16860, 6172,  56804,
                4104,  30616, 47616, 54724, 20616, 13256, 53596, 40828, 47524,
                12288, 32204, 7572,  28776, 59992, 40428, 36748, 30240, 26316,
                4760,  45292, 35192, 50128, 22420, 46916, 11572, 36060, 4792,
                31524, 26932, 10748, 60536, 20996, 44752, 65380, 32648, 22108,
                43824, 51344, 8444,  37684, 17916, 21840, 44764, 38800, 52808,
                13704, 41816, 32780, 64200, 37488, 24412, 20496, 52640, 32200,
                9800,  37036,
            },
            {
                5500,  40144, 60596, 1120,  51416, 36232, 10552, 39400, 46208,
                6320,  64460, 9936,  21500, 41732, 26260, 15836, 23572, 54220,
                40736, 59812, 6044,  63780, 25372, 48528, 62056, 7164,  37868,
                18888, 50040, 4116,  31072, 46220, 17672, 3572,  57236, 60400,
                36340, 45844, 55720, 40208, 48572, 35920, 57808, 28748, 19384,
                1892,  64520, 21548, 43804, 39080, 16,    9240,  24052, 62736,
                29652, 1188,  32416, 16396, 37136, 61748, 13508, 53548, 912,
                22512, 54732, 44500, 61908, 1896,  48532, 32536, 60704, 38380,
                26132, 51760, 11620, 64572, 3508,  36616, 50004, 1660,  26948,
                35180, 57108, 2564,  22388, 44912, 13332, 20236, 52860, 9180,
                65324, 12972, 56932, 27528, 904,   32352, 57656, 24624, 59504,
                19032, 44296, 37168, 5616,  30264, 54712, 38460, 2312,  15884,
                57840, 4876,  28216, 58912, 33516, 11240, 56152, 2512,  23912,
                35504, 4912,  22020, 29632, 51084, 4424,  47896, 58428, 42648,
                29280, 47440,
            },
            {
                22508, 14372, 55844, 8904,  47356, 21360, 28276, 61280, 18700,
                52236, 29924, 38532, 47980, 5376,  60504, 36612, 44168, 7192,
                29104, 856,   22188, 38780, 9124,  33080, 14092, 51768, 23212,
                11032, 40628, 59508, 25184, 61728, 35328, 28036, 52412, 19224,
                14936, 33016, 7832,  25788, 18196, 692,   42420, 61784, 5656,
                38448, 45720, 9972,  33832, 14028, 61660, 56076, 48668, 5996,
                21336, 50716, 10848, 25056, 7176,  49396, 34036, 45176, 65044,
                39208, 7036,  33884, 10136, 37464, 21384, 11944, 43112, 19296,
                14396, 35188, 7460,  43744, 25160, 32740, 18280, 63416, 21452,
                39636, 48884, 18620, 64692, 52040, 32836, 740,   58804, 42164,
                37780, 20956, 30652, 10248, 53740, 18328, 7120,  41696, 52528,
                1848,  15380, 56528, 24208, 12860, 51408, 7060,  47476, 29676,
                53748, 20020, 41964, 64396, 296,   47660, 26988, 60044, 17328,
                46420, 61944, 55408, 10464, 44740, 15084, 27456, 7584,  12908,
                63856, 18728,
            },
            {
                49732, 32440, 43720, 24356, 65132, 4972,  15272, 57856, 32972,
                42188, 25848, 15108, 57116, 18488, 30256, 11900, 62240, 19260,
                47412, 56756, 35268, 12604, 58760, 53776, 41272, 27392, 888,
                62972, 28632, 6032,  19884, 12088, 43048, 9024,  47144, 30788,
                1772,  50012, 63016, 12044, 53032, 31840, 14272, 22056, 34448,
                51164, 26516, 58104, 3796,  53164, 20132, 28408, 16828, 44268,
                57672, 38148, 64704, 45712, 56188, 21652, 9756,  18044, 28736,
                14464, 59520, 19128, 47460, 28852, 64180, 51464, 7124,  55220,
                63192, 40896, 22248, 57904, 46848, 8728,  55344, 14356, 5816,
                59340, 10320, 30584, 37792, 5384,  25720, 48332, 15240, 23604,
                50764, 3092,  59780, 43640, 39868, 63428, 48316, 12636, 28948,
                64152, 49408, 32496, 40720, 62764, 17184, 25952, 58748, 10404,
                35704, 13320, 49488, 24104, 15732, 52160, 40796, 30524, 10040,
                50516, 25900, 7020,  18388, 40032, 62928, 53824, 34292, 38692,
                2152,  54276,
            },
            {
                7152,  59632, 17356, 29472, 38764, 54644, 44464, 3016,  12724,
                8216,  62640, 388,   34364, 53004, 46020, 1964,  33188, 52284,
                14680, 26512, 42232, 46048, 29468, 4032,  18528, 36236, 44820,
                54904, 34624, 47224, 57468, 38924, 332,   64588, 25492, 40368,
                54960, 22712, 37824, 43896, 5068,  64740, 46464, 55768, 7668,
                41856, 17412, 30928, 48392, 24968, 36084, 40956, 4224,  33248,
                12764, 26752, 18972, 2124,  40532, 31156, 60592, 51904, 3400,
                43120, 50024, 25608, 56144, 3528,  15564, 30808, 24464, 672,
                48812, 28484, 4568,  16660, 38268, 60436, 28956, 41800, 45648,
                25040, 53964, 15604, 42732, 58092, 11112, 35764, 63284, 7284,
                33500, 17532, 47968, 5424,  15056, 26024, 33608, 20624, 38976,
                9352,  22436, 4212,  46928, 8852,  34856, 43180, 22640, 40124,
                62948, 45608, 3484,  32136, 36972, 7560,  20100, 63804, 3792,
                38128, 43280, 31680, 58308, 35732, 172,   19992, 23404, 60168,
                44352, 25828,
            },
            {
                35892, 40856, 1828,  11256, 33948, 19688, 50456, 36852, 23028,
                48520, 55096, 44652, 23884, 9452,  39892, 57844, 22576, 38176,
                8756,  64232, 3260,  16912, 50392, 23952, 65520, 56992, 8132,
                16880, 13400, 52004, 23708, 32192, 53984, 21440, 13084, 4704,
                61856, 16664, 29056, 58228, 20840, 27168, 38732, 10768, 24384,
                59980, 13512, 63108, 7112,  43284, 11184, 64316, 50328, 55040,
                8296,  59424, 34872, 52984, 15464, 5940,  25480, 36432, 57456,
                23088, 31872, 5152,  12464, 41528, 58588, 39448, 46488, 34032,
                10992, 59820, 32024, 51264, 12824, 2776,  23052, 52156, 31968,
                360,   62936, 7864,  27880, 46356, 17908, 55988, 29184, 44696,
                54408, 27184, 62752, 21960, 36664, 51608, 2956,  55496, 46032,
                35520, 60332, 53828, 19664, 28556, 59824, 4756,  50060, 1532,
                18128, 26676, 9200,  56628, 61152, 43912, 13092, 34276, 57340,
                14804, 20964, 52356, 12508, 45860, 29124, 9084,  48564, 16324,
                31084, 10580,
            },
            {
                20572, 63480, 52948, 48536, 59056, 7676,  26156, 61964, 30140,
                39620, 20316, 5864,  29156, 65236, 13772, 27328, 4368,  49252,
                31052, 53580, 20756, 60048, 6268,  32140, 11560, 47604, 21824,
                30916, 42488, 2848,  9624,  16052, 49248, 59304, 36804, 44476,
                48384, 10052, 6556,  34932, 51328, 2712,  16368, 49168, 33136,
                53424, 900,   37504, 22324, 56680, 15672, 2232,  30276, 23012,
                39488, 46300, 4804,  29044, 63508, 43588, 47832, 12620, 40728,
                8668,  63940, 35496, 45212, 52696, 22040, 9088,  61032, 17008,
                43620, 20336, 54092, 26008, 42548, 35484, 65468, 11248, 37992,
                19868, 35360, 50856, 22960, 62044, 4480,  21560, 41016, 2080,
                13504, 38936, 10680, 31700, 58252, 9008,  61796, 17152, 6108,
                26820, 14360, 42612, 468,   38752, 52260, 15184, 30648, 61040,
                33892, 54056, 39476, 21124, 28608, 5300,  53448, 23184, 48292,
                28836, 2468,  65164, 5520,  25032, 60712, 55828, 41268, 5168,
                51924, 57096,
            },
            {
                46912, 3780,  27316, 22344, 14068, 41664, 1092,  16644, 52032,
                10656, 60828, 16272, 36728, 47212, 51148, 17836, 60736, 43980,
                11660, 24660, 39716, 34636, 55672, 43444, 38604, 1704,  59312,
                26368, 63688, 37968, 61168, 44908, 28720, 7196,  18412, 26740,
                32464, 59928, 42060, 13444, 56108, 30628, 43212, 61328, 4108,
                20112, 29108, 45976, 50704, 32328, 60220, 26312, 47188, 17568,
                62788, 14256, 21996, 49872, 9516,  20164, 33268, 84,    54428,
                20692, 15748, 60796, 27416, 18428, 33204, 2352,  26720, 50216,
                36712, 3592,  8624,  61820, 47972, 19468, 7340,  57072, 16156,
                47776, 42180, 12992, 2844,  39348, 31768, 52572, 9740,  60820,
                24328, 57160, 46040, 88,    42052, 24036, 30364, 44036, 57948,
                50324, 31384, 10388, 64964, 24788, 12240, 56988, 20432, 47128,
                6380,  14128, 65100, 50484, 17096, 46552, 38984, 59648, 9064,
                42184, 55592, 37332, 16980, 50088, 32424, 13824, 36700, 64716,
                28112, 13360,
            },
            {
                39240, 16708, 44788, 32376, 54924, 63224, 46648, 35016, 57348,
                3892,  32764, 42880, 58208, 2392,  21172, 41364, 6884,  33756,
                62304, 92,    47928, 14052, 8528,  27508, 15576, 52388, 34084,
                6588,  49452, 19464, 25304, 4836,  40696, 33928, 56620, 1324,
                52312, 20536, 24708, 47256, 18592, 62880, 9056,  26084, 36136,
                57972, 40924, 11868, 18160, 5224,  40104, 9692,  53688, 36572,
                616,   32112, 41832, 56856, 37104, 59128, 17192, 62432, 29864,
                51060, 38776, 1648,  48608, 7272,  38104, 65152, 54844, 13080,
                23076, 56752, 39672, 14752, 1224,  30672, 50232, 27244, 4068,
                61180, 29372, 57884, 34248, 55272, 14780, 47472, 36856, 17172,
                49148, 34180, 19704, 53644, 14428, 48984, 38428, 12068, 1484,
                23084, 17900, 56260, 36300, 48836, 32540, 43920, 8424,  38016,
                24500, 42644, 30104, 11324, 3684,  25584, 31904, 744,   18884,
                26228, 33384, 47304, 22524, 40648, 1908,  19256, 44804, 3136,
                22636, 34216,
            },
            {
                6816,  61736, 9164,  37572, 4592,  29248, 19204, 8860,  43988,
                27264, 13264, 53760, 24236, 8660,  35348, 54080, 28564, 15152,
                55924, 19012, 29264, 63620, 50064, 22972, 61388, 19844, 41596,
                12164, 54184, 36016, 57724, 13936, 52960, 63328, 9716,  15520,
                39420, 65184, 5480,  36924, 792,   39252, 22000, 46288, 15048,
                51504, 8180,  65300, 27684, 54608, 34464, 20912, 44076, 7236,
                51396, 60964, 11556, 24700, 2880,  27804, 49120, 44744, 4716,
                25936, 42792, 11532, 58192, 53804, 14808, 30108, 42032, 5216,
                62424, 29060, 45836, 34208, 23776, 59080, 43844, 39108, 22044,
                52556, 9072,  25392, 18916, 44376, 7344,  26528, 64184, 31076,
                3688,  8104,  28324, 65108, 5844,  18672, 27592, 63160, 33240,
                41244, 60816, 45700, 6964,  21668, 2328,  63240, 28008, 52588,
                59564, 920,   48276, 58520, 36684, 56052, 63260, 12004, 51668,
                62216, 6756,  11480, 58768, 8508,  62708, 54064, 25444, 9792,
                50144, 59512,
            },
            {
                29900, 49140, 25572, 57668, 11648, 40284, 24320, 50192, 64688,
                21636, 38772, 49348, 31104, 63760, 12384, 59664, 23176, 37292,
                46364, 6420,  41628, 36196, 2480,  44948, 56824, 3856,  29540,
                45632, 17336, 700,   30024, 47352, 20264, 23560, 43400, 49696,
                30172, 11908, 54920, 27384, 50904, 58908, 12524, 53884, 5072,
                30796, 24124, 43468, 1504,  47416, 62220, 12752, 57312, 29456,
                25672, 19292, 45504, 54592, 7736,  40160, 13532, 23488, 10192,
                56920, 31380, 19072, 34504, 24244, 44472, 10032, 47888, 17608,
                32636, 52076, 11304, 64028, 17128, 53684, 9880,  13700, 33436,
                45420, 1500,  64784, 11632, 50292, 58440, 756,   20608, 42316,
                51188, 59960, 39956, 44532, 35560, 59156, 50984, 7796,  54412,
                37380, 3800,  27196, 15644, 54716, 40876, 18448, 11568, 35252,
                15988, 22864, 33100, 8372,  44240, 21484, 14980, 40912, 35840,
                45300, 16148, 30640, 52500, 27980, 35156, 47736, 30444, 56720,
                42412, 18740,
            },
            {
                2560,  41020, 21052, 51288, 16260, 60732, 33312, 3156,  14896,
                55804, 6540,  708,   17676, 26852, 45448, 3320,  49648, 10116,
                58676, 26240, 52720, 12456, 17600, 31724, 9916,  37240, 62280,
                24164, 8488,  64976, 32968, 10988, 3704,  37616, 27984, 59600,
                2744,  34288, 45168, 15812, 32236, 6916,  42144, 33976, 63244,
                38824, 19124, 59404, 14376, 37356, 24536, 2856,  16672, 64632,
                39712, 3948,  35176, 16380, 31304, 65476, 55624, 34320, 37944,
                60444, 6532,  46932, 63512, 2784,  20856, 59352, 25328, 38916,
                272,   20640, 6904,  26756, 41640, 36444, 4992,  63296, 55900,
                17588, 40464, 31196, 37172, 23120, 41260, 29824, 10316, 55576,
                13048, 22624, 15912, 25568, 11096, 3000,  21764, 46216, 14076,
                19692, 9876,  51960, 29916, 34684, 59296, 4288,  46492, 55936,
                5272,  62152, 40308, 17872, 27836, 52868, 7028,  49624, 23472,
                60516, 1624,  39012, 43996, 20800, 14244, 6176,  17112, 37312,
                12552, 53320,
            },
            {
                33712, 14212, 63532, 35572, 216,   47448, 53888, 42088, 29860,
                36348, 47036, 61972, 42340, 52524, 39316, 19808, 31624, 42664,
                1836,  21048, 32864, 60328, 54292, 25540, 40536, 51656, 14504,
                48972, 56172, 39132, 43876, 60700, 50980, 55784, 7724,  18792,
                47552, 22780, 57572, 8792,  63864, 23668, 28648, 17288, 56348,
                3248,  49912, 33540, 8800,  51828, 30568, 42216, 49480, 33644,
                9152,  48272, 58932, 51916, 42620, 20928, 1932,  50108, 16856,
                21968, 52772, 13272, 28540, 40080, 51272, 6292,  35448, 55600,
                60572, 42984, 49964, 57996, 2148,  47556, 19328, 28908, 24488,
                6956,  48712, 54280, 5084,  60060, 16140, 35516, 62580, 44968,
                37724, 5232,  54008, 48516, 32748, 55920, 42276, 31032, 26128,
                64540, 48648, 43056, 62512, 12884, 25308, 50352, 22220, 38368,
                29696, 51116, 12560, 54944, 64192, 3108,  34584, 29556, 19824,
                5468,  56396, 25072, 63968, 3520,  57968, 41048, 60424, 812,
                64132, 23828,
            },
            {
                46188, 55192, 6188,  43596, 28028, 22632, 7516,  12056, 58180,
                18920, 23676, 10760, 33728, 14724, 4652,  61564, 55212, 16740,
                65084, 38796, 8304,  47360, 4320,  63920, 20368, 6932,  33648,
                27844, 18788, 4396,  22120, 26568, 16436, 34892, 41844, 13408,
                61892, 5044,  40024, 19728, 43532, 49792, 1696,  47324, 10208,
                26724, 45304, 21988, 56928, 17444, 61084, 5060,  21756, 54988,
                14508, 28332, 23004, 12488, 5516,  47156, 30012, 8700,  63208,
                45060, 1232,  36696, 55356, 16564, 61412, 31136, 49272, 14408,
                8808,  28520, 34652, 13208, 22816, 31696, 59620, 50800, 37876,
                60960, 14948, 27348, 20212, 45952, 2916,  52800, 25240, 17888,
                33292, 29136, 63792, 1388,  38832, 61764, 16756, 4308,  57744,
                33664, 23856, 1720,  38804, 7480,  44388, 31460, 64808, 9456,
                19284, 43796, 1900,  24352, 37580, 46152, 10852, 58472, 42720,
                32096, 48892, 18068, 12408, 37060, 50492, 22956, 32960, 45232,
                27344, 8208,
            },
            {
                58956, 19396, 31228, 10600, 56872, 17624, 37952, 63148, 26096,
                50788, 6640,  59196, 28800, 56760, 8204,  25500, 35412, 11804,
                28980, 51236, 24344, 42876, 15300, 29380, 58340, 47060, 1352,
                59836, 42500, 52792, 13260, 46496, 1416,  63544, 24256, 31676,
                52508, 36228, 26488, 53540, 13072, 35104, 60768, 21396, 37000,
                62088, 12800, 5724,  39224, 27296, 46296, 11332, 36884, 59696,
                44256, 1020,  62452, 37124, 25860, 61024, 39112, 13964, 27648,
                41400, 32704, 25364, 9344,  43528, 3700,  11800, 19404, 45076,
                23348, 64860, 16836, 39360, 55116, 9196,  44516, 2860,  11772,
                33900, 43392, 9448,  63652, 32736, 13372, 48380, 8280,  3820,
                59168, 46460, 9416,  19440, 23744, 28600, 12532, 52748, 40408,
                6728,  11888, 53228, 21020, 17080, 58336, 308,   14840, 41516,
                26388, 59880, 48508, 31992, 15668, 61012, 26536, 51988, 14152,
                9340,  54552, 34412, 46600, 7928,  26760, 53968, 10176, 20080,
                50932, 38524,
            },
            {
                3680,  48960, 25836, 40016, 61756, 51512, 32772, 44844, 1928,
                31548, 39912, 46144, 18364, 37728, 50376, 21600, 48028, 44748,
                6224,  57496, 18756, 34572, 10796, 37004, 43916, 23456, 12808,
                36132, 9664,  31512, 37512, 58876, 29636, 6068,  49060, 10012,
                44000, 16704, 392,   59288, 30360, 4380,  54896, 41112, 15180,
                52024, 29828, 42476, 64528, 12,    53328, 32300, 25388, 6924,
                18104, 40480, 53148, 32840, 9808,  17716, 57788, 52460, 4084,
                58988, 18804, 49880, 64956, 21632, 34088, 27544, 57876, 37416,
                1832,  52304, 46180, 4612,  61508, 27372, 15716, 40792, 22356,
                56832, 156,   51324, 23584, 55468, 38536, 28068, 56696, 21312,
                50812, 12712, 42480, 57056, 51540, 7728,  44288, 36472, 18976,
                47424, 61376, 35728, 55884, 28076, 47708, 36860, 53832, 5928,
                57488, 34860, 7720,  21316, 4692,  39656, 18848, 64,    65460,
                40504, 27360, 2596,  61008, 30532, 42504, 15324, 4164,  61496,
                29804, 16096,
            },
            {
                35116, 64928, 13708, 2048,  45900, 4544,  14488, 21432, 9796,
                54372, 65456, 13024, 1180,  42904, 63656, 31144, 2424,  40320,
                14412, 62836, 612,   49260, 56148, 3500,  51968, 18032, 64644,
                54564, 24824, 62516, 7136,  20064, 53868, 39684, 57776, 21268,
                28296, 64032, 48468, 10660, 38696, 18408, 25496, 8108,  32052,
                2888,  55204, 23932, 34100, 15280, 19620, 41384, 63068, 50388,
                30836, 21080, 46604, 4204,  55748, 43580, 20528, 35424, 23680,
                46816, 12100, 6968,  56632, 38756, 46716, 53052, 62656, 5696,
                41172, 30000, 10912, 19984, 49088, 36140, 53568, 64664, 18568,
                48052, 29712, 35940, 41416, 6760,  18600, 43268, 61636, 36276,
                40392, 26320, 31184, 15552, 34948, 47792, 64216, 432,   58920,
                27412, 15304, 30780, 42196, 9652,  62292, 23504, 32832, 46336,
                16940, 11036, 63004, 52364, 43028, 56880, 48084, 23772, 36504,
                45744, 16088, 22568, 56964, 19140, 64448, 47860, 35588, 56496,
                42136, 11232,
            },
            {
                44040, 20744, 54100, 36652, 23860, 29188, 59888, 41360, 48372,
                16596, 35112, 22432, 26780, 53120, 10536, 16124, 60520, 54624,
                26864, 37248, 30468, 22404, 61812, 27236, 8184,  39336, 30220,
                5180,  48132, 16608, 50400, 42960, 11108, 15452, 36552, 2280,
                54532, 7540,  33560, 23948, 44576, 51028, 65124, 46572, 59876,
                20220, 48060, 7348,  59492, 49756, 9480,  56068, 3024,  13668,
                58128, 8392,  64360, 15476, 28728, 48640, 316,   31948, 6056,
                61776, 37340, 30424, 15284, 636,   24280, 8464,  17104, 32712,
                25604, 55560, 59124, 33196, 24044, 3468,  7876,  32332, 25824,
                5556,  62356, 16524, 12308, 58688, 1956,  30872, 10832, 15816,
                624,   63348, 5704,  60084, 3276,  20896, 25212, 31944, 9520,
                22444, 45524, 6180,  2340,  51440, 19364, 12460, 3652,  27756,
                50436, 38628, 24812, 29048, 12032, 33612, 8620,  30960, 4936,
                59064, 51676, 7304,  38744, 11484, 1204,  25172, 39992, 7092,
                24092, 52248,
            },
            {
                27020, 32256, 60244, 7580,  18800, 52964, 33740, 6080,  56412,
                28196, 3336,  49884, 60072, 36316, 5292,  24808, 34040, 20848,
                9552,  52664, 45864, 12660, 41316, 32744, 15704, 57936, 45544,
                20924, 40768, 56,    33960, 27728, 23144, 65280, 32568, 46956,
                18820, 41140, 61104, 15936, 55684, 5976,  13832, 34536, 27516,
                38996, 17504, 11932, 36768, 44564, 28968, 22664, 38872, 47316,
                27272, 34828, 38296, 24484, 41104, 12272, 63028, 51096, 16444,
                54972, 44020, 27052, 51808, 41916, 59784, 12864, 49040, 43748,
                20808, 7712,  14096, 38252, 62900, 42604, 12876, 46016, 58592,
                10412, 44052, 53244, 27164, 48940, 65204, 24516, 46948, 54360,
                34056, 22612, 45472, 38256, 53464, 41440, 14056, 55020, 50096,
                39732, 56764, 65332, 24992, 34400, 39940, 56500, 43628, 64364,
                20852, 55084, 956,   45816, 60156, 15148, 53216, 62000, 20480,
                12332, 29492, 35056, 43432, 52596, 32292, 54912, 12856, 18300,
                62832, 508,
            },
            {
                5444,  46888, 10212, 41624, 49100, 64284, 11220, 37476, 24260,
                59048, 44140, 8744,  30744, 19544, 45948, 58168, 49308, 3840,
                42940, 17048, 59528, 4856,  19944, 53440, 48772, 2620,  35260,
                26980, 10500, 57360, 61408, 45940, 3280,  51100, 5356,  58980,
                25832, 11784, 36816, 2652,  28952, 21876, 42456, 10480, 1356,
                57436, 43600, 63672, 26548, 2092,  58280, 33360, 5236,  17472,
                54676, 11696, 1640,  51872, 56900, 7836,  36812, 26116, 40404,
                8548,  21208, 3304,  63364, 19676, 31540, 36312, 57200, 2384,
                64220, 50488, 44848, 520,   17688, 57292, 28300, 52128, 38432,
                34284, 20124, 3436,  39668, 21720, 34972, 52456, 7576,  19356,
                58188, 49616, 9060,  18092, 27588, 11052, 33780, 62116, 17396,
                2900,  37196, 13220, 49672, 16264, 59868, 7756,  30184, 37120,
                14200, 5596,  41376, 18224, 36344, 3584,  26732, 39276, 44204,
                49208, 24648, 63164, 5188,  46404, 20816, 59556, 29364, 49128,
                33848, 58044,
            },
            {
                38232, 17456, 29704, 55488, 15124, 26576, 1012,  46756, 18052,
                12452, 39424, 62904, 14704, 55708, 41176, 12356, 38092, 64796,
                31884, 7336,  25660, 35120, 63196, 10400, 25252, 60036, 13740,
                63092, 52652, 30688, 13216, 18064, 56284, 40588, 30160, 14160,
                43572, 52176, 31016, 58004, 48248, 62144, 37752, 53080, 23392,
                50696, 4372,  32276, 21576, 52912, 14080, 48440, 65212, 42448,
                23424, 44980, 61492, 31588, 17124, 22240, 58940, 29852, 13620,
                48456, 57812, 33120, 10800, 45628, 5972,  27988, 22604, 39332,
                15924, 35136, 26248, 53860, 30668, 48424, 21324, 16136, 1284,
                24272, 61368, 55992, 31532, 9304,  14792, 44392, 3268,  40908,
                25956, 13064, 30296, 56100, 62976, 48300, 5828,  43928, 28620,
                8228,  31412, 20600, 44308, 27220, 53412, 1292,  22848, 49104,
                9296,  57640, 31452, 63960, 21820, 49724, 56136, 17300, 2704,
                9472,  57216, 18152, 13840, 27460, 3532,  37208, 9156,  45268,
                22300, 14480,
            },
            {
                50132, 62412, 23452, 2992,  39372, 44272, 21128, 62500, 52424,
                32404, 4672,  21708, 47952, 6868,  28420, 264,   17940, 27368,
                56452, 50596, 39808, 47452, 1076,  44064, 37428, 22112, 6196,
                42696, 19040, 38644, 7504,  36108, 25344, 10092, 20680, 61968,
                6644,  22760, 45880, 9592,  19164, 4796,  31832, 16804, 60508,
                29368, 14928, 40232, 8864,  61480, 18932, 37272, 10432, 29768,
                59136, 6232,  20004, 48160, 5124,  42364, 46192, 2056,  64712,
                23916, 35756, 17240, 54020, 38088, 50804, 60756, 9732,  55132,
                29576, 11272, 60464, 6616,  40540, 10168, 5012,  60020, 42048,
                50016, 14000, 46820, 5952,  62688, 37456, 28716, 60448, 32308,
                64384, 37012, 6332,  43176, 23468, 1760,  36576, 20072, 60076,
                53752, 47048, 61876, 4948,  10296, 32984, 46924, 17992, 61340,
                34316, 51804, 25248, 47528, 10524, 7240,  42380, 32824, 64304,
                37308, 31312, 54140, 39936, 61752, 50348, 15904, 65172, 6088,
                41032, 54256,
            },
            {
                11344, 35208, 58368, 6860,  31124, 56668, 35672, 8308,  28804,
                42220, 54272, 25636, 37688, 32988, 51476, 61644, 23172, 44924,
                10732, 14856, 22232, 57448, 29612, 17696, 55304, 31656, 51604,
                47088, 2128,  23840, 53960, 44236, 60688, 48200, 35372, 54948,
                39100, 1676,  64504, 14660, 40200, 25880, 56352, 43868, 7128,
                35924, 48680, 55800, 46064, 5628,  41632, 26056, 532,   52736,
                15572, 35492, 39756, 27680, 63548, 34376, 10716, 19580, 52384,
                43132, 4244,  61592, 25284, 1564,  14388, 18636, 47920, 4712,
                41484, 46720, 19816, 23556, 33620, 65472, 37264, 53160, 32124,
                8200,  29192, 17840, 43060, 51016, 20224, 55492, 11264, 17020,
                46060, 1172,  51184, 15368, 31892, 59032, 52104, 25472, 41136,
                11464, 23276, 57868, 36332, 42556, 63408, 13276, 39600, 26928,
                44636, 16024, 2656,  39032, 29344, 59476, 23272, 13184, 27716,
                46764, 780,   22260, 7412,  33108, 23900, 42236, 55764, 26180,
                30972, 2064,
            },
            {
                42944, 27476, 45536, 18984, 49832, 11760, 60012, 3784,  50524,
                15412, 58304, 1540,  64440, 10180, 15984, 42416, 35524, 53316,
                2184,  62444, 33152, 6360,  12944, 65524, 8376,  40128, 15788,
                28472, 33888, 64580, 49136, 5144,  15204, 1024,  27300, 17388,
                49512, 33072, 27828, 53344, 35000, 50268, 284,   12924, 64672,
                19804, 10736, 24376, 34388, 30068, 50372, 56736, 21224, 32952,
                49404, 9596,  54456, 2788,  14248, 50252, 55540, 38140, 6984,
                31640, 12772, 46944, 40336, 29624, 65140, 43196, 24180, 32884,
                63248, 3196,  51380, 55824, 13780, 45812, 27172, 22856, 12228,
                63676, 36628, 25044, 59160, 408,   26436, 47688, 4624,  24072,
                53456, 21008, 57560, 40288, 19048, 7944,  45228, 14872, 3852,
                35012, 16728, 628,   29228, 18896, 24408, 54764, 3868,  58636,
                6344,  20768, 62224, 12624, 54984, 34904, 51200, 5028,  60576,
                52684, 16312, 42920, 48396, 57476, 10200, 1336,  34804, 17796,
                59104, 20864,
            },
            {
                8544,  52944, 13892, 64020, 33932, 25244, 16536, 38968, 23508,
                10880, 34876, 18964, 43840, 56164, 21304, 59764, 7980,  30864,
                48636, 20016, 38336, 42976, 52436, 26900, 48492, 3084,  60980,
                11912, 56436, 9132,  20976, 29164, 32920, 42268, 63596, 8404,
                12780, 58716, 20356, 43308, 7512,  62504, 22364, 46116, 39008,
                26880, 58660, 4000,  62612, 16544, 12132, 61000, 45248, 6892,
                63948, 24608, 43852, 60780, 22684, 30940, 25700, 16320, 60276,
                27612, 56584, 8920,  21608, 58200, 34640, 8056,  52668, 13460,
                36808, 27552, 16720, 42796, 1468,  58080, 18596, 3880,  44468,
                56552, 2740,  40332, 10428, 33924, 13728, 41720, 58624, 38880,
                33580, 9992,  27952, 63400, 34720, 11212, 55284, 64864, 27268,
                48544, 51848, 39816, 55820, 8300,  50280, 37320, 30892, 11528,
                52796, 41472, 33300, 44148, 18000, 1444,  45640, 19860, 40388,
                25220, 11316, 35848, 60284, 19092, 28040, 45672, 52812, 12072,
                37612, 63516,
            },
            {
                32228, 3224,  22900, 41196, 24,    54784, 48148, 43508, 30804,
                60560, 46800, 27112, 5904,  49092, 29856, 2840,  13744, 39556,
                25724, 54736, 3992,  60740, 23664, 34352, 19360, 57676, 36476,
                25316, 43416, 17352, 40316, 55156, 59516, 22840, 52752, 45224,
                37132, 24720, 3300,  55532, 11672, 17344, 29196, 33392, 54328,
                2332,  42304, 51704, 21984, 39504, 2684,  35760, 18020, 40436,
                28696, 13120, 37888, 18444, 7552,  58112, 1164,  40476, 48796,
                20284, 36716, 53220, 16220, 49688, 4040,  26764, 56460, 21024,
                58828, 7536,  61532, 38304, 29792, 8052,  50988, 60916, 34576,
                20036, 48912, 54848, 22180, 64324, 52512, 30204, 8044,  62160,
                14608, 43464, 48044, 3288,  24744, 49496, 29732, 38208, 6840,
                60832, 31704, 13388, 62348, 46244, 2108,  15720, 64496, 47976,
                23720, 28328, 58024, 7852,  25468, 65220, 37020, 14476, 57596,
                6448,  63664, 29592, 3444,  13496, 39124, 62916, 22668, 49644,
                4432,  47608,
            },
            {
                16120, 57036, 36528, 60776, 9560,  28496, 20468, 64832, 7388,
                3160,  52152, 39768, 11996, 36976, 24664, 53976, 46700, 64616,
                17212, 9128,  29256, 15532, 46452, 10644, 41492, 5136,  21700,
                52780, 464,   61916, 37044, 13312, 3616,  10684, 18696, 30136,
                5540,  61192, 40680, 30636, 46640, 38424, 60116, 9276,  49288,
                18080, 31812, 13584, 45660, 27796, 55136, 9100,  23548, 59844,
                1584,  57712, 32408, 52504, 47112, 35592, 43456, 10000, 62296,
                5288,  44628, 876,   32544, 42204, 11604, 38516, 45520, 164,
                31320, 48664, 11704, 24904, 34096, 47564, 15588, 39188, 28448,
                5612,  14968, 31024, 45684, 17220, 4200,  36056, 19216, 50084,
                26224, 5536,  59644, 37460, 13416, 42788, 988,   21212, 17756,
                44176, 9356,  25112, 20088, 34312, 28192, 21864, 43312, 8628,
                35428, 184,   14868, 48968, 53564, 31036, 9720,  28220, 48612,
                33464, 44312, 20920, 51744, 55200, 32304, 5228,  15428, 29648,
                40468, 24644,
            },
            {
                44452, 50528, 18452, 30344, 46740, 5092,  13732, 36440, 55872,
                17440, 22460, 62236, 31632, 59108, 17972, 41744, 4536,  33396,
                57980, 44580, 36128, 55928, 848,   51404, 30216, 62480, 45664,
                32604, 49780, 6828,  26956, 46592, 51024, 34968, 40052, 57100,
                48004, 15316, 51252, 19104, 6356,  57464, 24952, 14852, 5684,
                63264, 37316, 56364, 6748,  64996, 32668, 43632, 53308, 48776,
                15804, 45760, 5176,  11472, 28284, 65480, 13948, 51900, 33364,
                23216, 13296, 64000, 24772, 59660, 19304, 62884, 15544, 50948,
                40412, 18344, 44208, 54112, 20632, 63848, 10548, 23820, 53708,
                43240, 62844, 8448,  38292, 27352, 57332, 47220, 23360, 2088,
                55980, 31056, 18116, 53900, 22332, 61940, 57180, 51284, 33676,
                58320, 3072,  41836, 53380, 5860,  57248, 60172, 39344, 18760,
                55704, 62556, 37964, 20416, 3788,  60092, 39500, 22164, 4072,
                55932, 9148,  37912, 24580, 47032, 8520,  43552, 59752, 55848,
                9764,  61624,
            },
            {
                27972, 6888,  11520, 38372, 52568, 59144, 41948, 49632, 26104,
                33860, 44988, 14664, 1100,  50336, 8936,  61704, 27748, 22376,
                11684, 49980, 20760, 7268,  64248, 37692, 17492, 13600, 8588,
                28156, 15092, 20196, 57848, 31336, 24336, 64684, 7556,  26808,
                728,   34144, 23292, 65496, 36324, 2792,  52380, 40952, 44920,
                22672, 28856, 10224, 20308, 48228, 14744, 25808, 4416,  30708,
                36384, 22408, 55368, 40868, 20232, 4088,  24836, 17852, 29408,
                58724, 39160, 47236, 28388, 7436,  54148, 30356, 35100, 9380,
                23648, 64768, 5416,  2556,  56936, 41088, 31428, 528,   59464,
                12412, 25488, 51956, 1348,  61324, 9644,  42392, 12220, 65176,
                44568, 35408, 9384,  40888, 7016,  31936, 15808, 10616, 47000,
                22964, 29004, 63984, 36524, 10596, 45308, 13592, 31876, 5280,
                50468, 12104, 29832, 42256, 46316, 16328, 51032, 63268, 42112,
                18124, 50556, 14996, 252,   64516, 17576, 26768, 36160, 1216,
                19752, 34764,
            },
            {
                3860,  65260, 42660, 25668, 15700, 22736, 32380, 1304,  11548,
                63360, 8384,  56884, 42684, 21112, 35312, 15144, 52048, 38452,
                1776,  41380, 59344, 32044, 27004, 22200, 43984, 58696, 55076,
                38676, 64144, 42020, 10072, 2360,  44024, 14836, 55044, 21160,
                61992, 42000, 9184,  13452, 48352, 27500, 20448, 34824, 60876,
                1764,  50416, 59604, 41352, 564,   36776, 61568, 11088, 19536,
                64356, 7992,  26604, 49300, 61256, 38596, 56340, 47852, 2036,
                54364, 9216,  16844, 35708, 2644,  48100, 22348, 4860,  57528,
                28044, 52992, 35644, 14768, 27116, 6848,  49972, 45508, 35968,
                18436, 32488, 41228, 20560, 34588, 15536, 54496, 38500, 28272,
                21044, 15196, 51488, 63840, 25556, 45604, 36248, 4528,  54104,
                38868, 14024, 48108, 17136, 51492, 25612, 2748,  54036, 27060,
                44364, 22352, 59532, 8872,  24556, 35032, 12804, 1588,  32344,
                25844, 61912, 30052, 58628, 34092, 41448, 21852, 51412, 46440,
                13148, 53076,
            },
            {
                22452, 33320, 55628, 2032,  62072, 8512,  54508, 19528, 40240,
                53036, 29740, 38048, 27288, 5008,  55028, 44316, 30596, 6476,
                63600, 25148, 16508, 12936, 47328, 53624, 2604,  34696, 5888,
                23620, 47872, 33812, 53276, 61616, 18304, 49176, 38216, 11988,
                45488, 29068, 53252, 59324, 32252, 43940, 10396, 54444, 30748,
                13104, 25144, 34428, 17040, 52580, 22296, 57760, 39144, 51172,
                42096, 59176, 34480, 480,   15584, 31712, 43784, 8080,  35204,
                41888, 21296, 51308, 57720, 43668, 13900, 39944, 60764, 43328,
                12500, 32356, 46752, 60212, 37704, 17856, 62288, 22100, 8848,
                55940, 47372, 5956,  58928, 49736, 24420, 31976, 5052,  52472,
                60024, 168,   47132, 28936, 3120,  19036, 59376, 41520, 26060,
                7788,  59760, 384,   22224, 30400, 39876, 61020, 34956, 16356,
                64912, 1944,  33496, 54868, 4852,  58484, 27444, 53436, 10492,
                47568, 6140,  40424, 12060, 53008, 5580,  10740, 62568, 30464,
                38620, 59480,
            },
            {
                15080, 45852, 19940, 49312, 34964, 44732, 28020, 59952, 46304,
                23520, 2616,  18200, 48620, 65400, 24136, 12280, 57240, 19236,
                48196, 54460, 35344, 3588,  61744, 40264, 11276, 25536, 50536,
                17960, 1252,  12600, 22284, 36360, 6464,  29744, 3340,  33256,
                58020, 17536, 4960,  38660, 1628,  16364, 63636, 7312,  18432,
                47200, 39724, 55088, 5116,  45492, 31300, 7604,  27260, 2084,
                13480, 17636, 44808, 9848,  52708, 22748, 12276, 60672, 26404,
                63380, 4240,  31220, 11180, 64940, 26832, 52592, 17604, 1428,
                49824, 19708, 7376,  22588, 42868, 52264, 12932, 28340, 39412,
                3540,  64992, 29564, 12796, 44108, 3056,  62864, 18524, 8732,
                41164, 33052, 11892, 37972, 55404, 48856, 13016, 30076, 65388,
                19892, 32500, 55128, 42536, 63064, 7968,  19736, 47232, 10120,
                37648, 49548, 17832, 39864, 48548, 19840, 44672, 38036, 60784,
                21760, 35364, 45388, 18692, 28016, 48336, 57564, 16048, 24680,
                7656,  41604,
            },
            {
                27156, 8836,  30868, 12960, 40048, 3936,  16932, 37064, 5244,
                14416, 50832, 58196, 10812, 32012, 46392, 108,   40572, 34184,
                9884,  29220, 43424, 50888, 8824,  19444, 29736, 63052, 42524,
                60200, 30412, 56640, 40176, 27380, 58240, 42768, 52996, 24084,
                7932,  50048, 26308, 20892, 55868, 24216, 49028, 36640, 57864,
                3764,  64552, 9256,  28260, 61996, 12248, 43052, 54228, 47800,
                33132, 24448, 56648, 28688, 64072, 37220, 5408,  50248, 18992,
                14336, 45904, 24224, 38496, 19624, 32876, 8648,  36792, 25312,
                62820, 39048, 56288, 10712, 30396, 2140,  34240, 58580, 15712,
                53068, 23124, 37268, 17468, 56268, 27892, 46588, 35796, 49196,
                25256, 58144, 22560, 17260, 61456, 34168, 23384, 1756,  49848,
                44324, 10752, 35688, 4296,  14764, 50172, 23588, 58400, 42044,
                6304,  24900, 28444, 13924, 62724, 30384, 7464,  17160, 56276,
                14280, 2020,  57264, 23464, 38508, 2448,  33180, 44556, 3624,
                49812, 56656,
            },
            {
                764,   62784, 52172, 57600, 23128, 64532, 11300, 55796, 31120,
                60924, 34372, 41184, 7056,  37284, 17084, 59140, 26464, 62208,
                14656, 6652,  22828, 38136, 57940, 33212, 46084, 15036, 36896,
                9116,  46724, 4272,  15824, 50644, 9772,  20532, 64036, 15696,
                41124, 62532, 35376, 46412, 61160, 40452, 12736, 28812, 42508,
                21140, 32956, 15020, 23616, 49948, 18040, 35220, 21204, 60068,
                6536,  38632, 49776, 2972,  19916, 46660, 58216, 29764, 40592,
                33944, 58788, 53124, 404,   49212, 5584,  56556, 46056, 31128,
                15608, 4628,  26672, 64416, 49240, 55280, 5728,  47092, 42092,
                25948, 10376, 51008, 7208,  40272, 21728, 10648, 60836, 13908,
                5756,  44972, 53476, 4404,  9828,  43100, 7368,  56972, 37532,
                16528, 51996, 24524, 57492, 45784, 33056, 1328,  28884, 13248,
                52052, 56480, 61228, 10352, 42748, 484,   51388, 24304, 41212,
                28932, 49684, 65444, 8336,  53904, 61356, 20508, 37236, 64208,
                18532, 35212,
            },
            {
                47988, 39180, 17792, 6116,  28976, 47124, 51572, 25116, 8636,
                43576, 20748, 26624, 62632, 22528, 52616, 5776,  50140, 21428,
                45676, 55548, 60556, 17376, 27252, 428,   54612, 5324,  21188,
                55660, 25912, 65304, 32948, 44172, 60132, 288,   34800, 47840,
                30956, 3732,  13932, 9728,  30332, 6460,  52844, 452,   25916,
                51472, 44884, 37128, 57104, 1196,  39836, 64224, 4016,  15176,
                29948, 62252, 11864, 41236, 34940, 25676, 16144, 1256,  55788,
                10440, 6592,  28480, 16504, 59776, 40832, 22752, 61436, 11148,
                53840, 41540, 34908, 44956, 14988, 25008, 20988, 63624, 31556,
                804,   45804, 59308, 31000, 64056, 1268,  55148, 38980, 30760,
                19880, 64748, 28188, 39820, 51588, 30860, 62856, 14348, 27548,
                4768,  61720, 29080, 19024, 11656, 37508, 53936, 62428, 18120,
                31372, 3440,  45588, 35972, 22924, 32564, 64196, 36968, 4160,
                9832,  33296, 42544, 15860, 31348, 12928, 26736, 9932,  51892,
                29268, 12492,
            },
            {
                43272, 24840, 59416, 37176, 2284,  42280, 33692, 18644, 63172,
                48032, 12688, 2924,  54568, 14568, 42460, 30208, 36392, 2304,
                39856, 31652, 3924,  12304, 49076, 64172, 23416, 51564, 12904,
                34132, 41396, 19572, 7288,  23800, 13640, 26684, 39236, 11236,
                19976, 55308, 51936, 43292, 19112, 34252, 58984, 15828, 62524,
                10948, 6692,  60808, 29780, 46772, 8708,  26332, 55556, 44284,
                22932, 53152, 18612, 4820,  60912, 54168, 8984,  38260, 22996,
                48348, 42668, 62872, 35736, 44780, 13820, 28880, 3192,  20668,
                47912, 59004, 440,   19052, 8912,  40276, 37068, 11592, 18108,
                61208, 35240, 20160, 13676, 43652, 33920, 16132, 26188, 50176,
                42680, 1968,  35132, 15236, 25656, 19152, 46248, 54620, 21256,
                48064, 39308, 8212,  41456, 63972, 25964, 5020,  44136, 40152,
                34784, 20348, 8064,  53772, 16256, 57420, 12012, 46536, 54752,
                61712, 19896, 25740, 4448,  46940, 39772, 55976, 42260, 5692,
                60408, 21408,
            },
            {
                6648,  10576, 33008, 14912, 55396, 21116, 7084,  39628, 1004,
                29052, 37896, 57276, 45200, 33672, 9012,  61404, 12556, 17728,
                64936, 51984, 24972, 35800, 41308, 8004,  43888, 38652, 29352,
                59152, 1520,  53024, 48188, 37440, 54232, 45452, 56912, 6500,
                60532, 24928, 37368, 65012, 3180,  23552, 45560, 38244, 32188,
                54900, 18776, 41288, 13624, 20548, 52256, 32128, 11084, 36412,
                47100, 8172,  32584, 43316, 13156, 30660, 44880, 51568, 64696,
                32248, 12648, 20908, 25756, 9004,  54964, 34060, 51036, 38020,
                7760,  24088, 32904, 53192, 60624, 29260, 56540, 50076, 7732,
                53656, 27472, 5304,  49340, 22992, 52760, 4292,  59500, 7804,
                54396, 11360, 57148, 48524, 60232, 588,   38612, 34820, 9616,
                32320, 58460, 976,   53184, 47196, 16044, 21816, 57952, 9892,
                48832, 59568, 27064, 38636, 49928, 5316,  20844, 26384, 15408,
                44232, 52604, 36740, 58820, 50276, 436,   16984, 23532, 46616,
                31972, 54628,
            },
            {
                64772, 51484, 45656, 26896, 49428, 61936, 13236, 52636, 58732,
                16404, 50112, 4940,  19324, 27912, 48304, 24436, 54236, 43776,
                28060, 10008, 47168, 57404, 20148, 32164, 16460, 61588, 4408,
                45692, 17420, 11940, 28400, 61236, 3236,  17784, 29668, 49444,
                42012, 1124,  27856, 16620, 49816, 55508, 8492,  20660, 48368,
                4152,  24520, 50296, 2540,  34620, 59768, 42160, 16896, 1812,
                58064, 26800, 65144, 50792, 23848, 58640, 20564, 3676,  27480,
                18088, 1856,  57024, 50160, 3908,  61904, 18264, 42928, 65216,
                16060, 29816, 12440, 48724, 5096,  43568, 1952,  33528, 44716,
                14748, 40820, 36996, 57800, 9672,  28996, 46072, 36200, 23764,
                17808, 31488, 22096, 6168,  42388, 12364, 23976, 3900,  63836,
                44032, 13612, 23268, 30272, 35172, 6580,  50588, 27784, 1868,
                14916, 24016, 63204, 2396,  44056, 29208, 58992, 34960, 1192,
                30784, 5872,  11792, 22456, 28784, 63860, 34508, 58424, 3140,
                36692, 16472,
            },
            {
                28512, 4144,  19484, 60188, 9492,  31280, 44784, 26400, 35244,
                21976, 32516, 10764, 64728, 40420, 1804,  58056, 5256,  20280,
                38588, 7032,  15488, 62492, 2508,  27896, 55036, 10132, 25108,
                50672, 39440, 63604, 32120, 9388,  21492, 34260, 64464, 14832,
                22472, 33640, 46900, 11552, 29944, 40744, 63432, 13308, 29224,
                43904, 35856, 65316, 27492, 56492, 5404,  24372, 63436, 50196,
                39212, 20344, 14612, 176,   37660, 6164,  34748, 15268, 41608,
                60804, 46376, 37772, 30088, 39964, 23816, 11744, 2004,  26748,
                54556, 45944, 38932, 63060, 22532, 16624, 26632, 62364, 24068,
                19656, 55572, 2708,  61848, 42132, 18808, 63708, 12672, 40760,
                61312, 45416, 37608, 64108, 32776, 52560, 58684, 49568, 17032,
                26456, 50864, 18560, 60724, 10392, 55452, 32512, 65224, 37224,
                54268, 46368, 11856, 33604, 17956, 9432,  41000, 47956, 60480,
                39416, 18008, 62392, 45140, 7280,  19612, 52348, 9500,  26184,
                13560, 40980,
            },
            {
                22916, 38944, 35272, 20,    41248, 23428, 4772,  63928, 9032,
                54156, 44120, 59988, 23324, 52148, 16784, 35384, 31112, 49924,
                59840, 34236, 23608, 44444, 52628, 13412, 47760, 33592, 20324,
                35776, 6736,  23108, 57132, 41476, 51524, 5736,  38180, 10216,
                52644, 7548,  57204, 59984, 6000,  35576, 26160, 2912,  59580,
                56072, 7472,  16276, 39560, 9904,  48700, 19172, 12984, 29284,
                4696,  34392, 56456, 46324, 28384, 62780, 48744, 55956, 9816,
                52360, 33636, 7664,  14628, 53348, 47636, 31844, 57504, 35632,
                5412,  59712, 20184, 9632,  36188, 51276, 13136, 58352, 4608,
                48388, 30232, 12344, 25068, 32680, 6952,  49108, 632,   27092,
                51516, 3176,  8716,  14576, 26784, 18604, 40296, 29788, 6820,
                56812, 41748, 37348, 2812,  45024, 39728, 12616, 17612, 42736,
                30116, 4452,  40672, 56712, 51556, 64868, 23544, 7052,  13348,
                49532, 24856, 53500, 40880, 32684, 14364, 38572, 43820, 62896,
                49968, 56792,
            },
            {
                8276,  62156, 53204, 14520, 57796, 17708, 38160, 47288, 15132,
                40892, 6288,  29972, 13980, 38064, 7528,  45960, 63744, 13648,
                720,   55664, 30512, 39664, 4920,  36580, 64648, 892,   56404,
                60692, 14164, 43892, 2116,  16292, 47244, 58812, 24200, 43412,
                62696, 19276, 38520, 44100, 21748, 15548, 53264, 42248, 18324,
                11424, 32552, 52416, 21448, 45064, 31172, 37740, 54548, 44004,
                61600, 10628, 40444, 17408, 8556,  42708, 12108, 25596, 39204,
                5024,  23376, 59372, 19616, 64112, 6236,  43972, 21512, 50056,
                14352, 41760, 1260,  27976, 57000, 31408, 41976, 38196, 10184,
                34688, 65368, 45296, 17120, 39000, 53416, 21424, 58664, 33968,
                16028, 54936, 29296, 47516, 56264, 2516,  9928,  45100, 35436,
                11924, 4252,  64452, 28164, 58048, 20420, 24764, 49636, 8040,
                62012, 18988, 22516, 31148, 14392, 36208, 53932, 19792, 32932,
                56984, 29404, 1736,  10264, 57684, 47496, 3776,  30348, 20936,
                1208,  46988,
            },
            {
                31672, 12076, 44528, 29460, 48824, 33772, 55840, 2388,  20340,
                58036, 25296, 460,   47700, 55728, 26908, 10488, 22628, 40568,
                26412, 48588, 11100, 17036, 59356, 22440, 42436, 17668, 27232,
                46576, 30944, 54324, 26144, 36564, 12200, 27936, 55048, 1528,
                30692, 26076, 13076, 54048, 1980,  62028, 31356, 50632, 38828,
                23644, 61596, 47216, 13996, 63012, 836,   60308, 7920,  25532,
                21724, 47960, 30260, 54124, 23100, 51756, 19856, 31212, 63396,
                16800, 29120, 45004, 11140, 27276, 36540, 16428, 60488, 10064,
                25188, 33400, 53520, 44916, 7292,  64436, 3292,  21252, 54144,
                27308, 7596,  51496, 1680,  57328, 14592, 29840, 43788, 9744,
                39352, 20296, 61976, 43204, 34336, 22556, 63388, 60392, 20460,
                54388, 32116, 22680, 15396, 48236, 6040,  61188, 100,   35512,
                26796, 50860, 59832, 6260,  1544,  27564, 45164, 4012,  61384,
                8664,  37656, 15992, 65060, 22976, 27028, 55448, 60984, 11476,
                34224, 17860,
            },
            {
                55288, 26296, 20580, 6788,  65404, 10908, 27820, 51368, 30896,
                36548, 49460, 63352, 34524, 19064, 61520, 43360, 53672, 18184,
                4444,  62260, 21088, 46328, 6908,  29692, 50420, 8584,  40716,
                3976,  10692, 49352, 19316, 60384, 33200, 4436,  18228, 46396,
                35828, 50600, 4092,  32628, 48020, 24732, 10268, 46004, 68,
                28344, 5840,  36880, 3812,  26584, 41980, 16372, 33372, 52632,
                2236,  58292, 5608,  62092, 34004, 3172,  60028, 47012, 972,
                36032, 57156, 50636, 4332,  40456, 55696, 256,   29672, 39616,
                63340, 18504, 48604, 12016, 24248, 17324, 49600, 44360, 14924,
                59180, 41152, 22236, 35820, 61268, 47016, 4328,  23936, 64852,
                48820, 4932,  24616, 11984, 6992,  49360, 37936, 14848, 25736,
                46476, 51692, 9220,  42444, 34444, 53604, 31328, 44408, 57688,
                15292, 10780, 43000, 48116, 39076, 62460, 16692, 41504, 26088,
                46840, 20772, 42872, 34916, 49620, 5344,  17024, 36356, 24684,
                42372, 59716,
            },
            {
                38320, 2176,  50356, 42120, 24404, 3932,  45276, 12956, 62520,
                7952,  16864, 11428, 41880, 5600,  30372, 2164,  33104, 58232,
                36676, 42724, 52092, 34840, 55984, 24820, 12396, 62052, 53248,
                21872, 37564, 65040, 7672,  42068, 50880, 63356, 39916, 11440,
                61580, 16300, 42296, 60680, 18168, 36624, 6764,  64420, 20584,
                57792, 42552, 54252, 30268, 56744, 22704, 49904, 11328, 37148,
                18240, 42916, 12760, 26272, 15392, 37520, 7500,  41024, 53512,
                13176, 42468, 21364, 34012, 60992, 23968, 51780, 47208, 6768,
                56312, 3744,  29336, 60872, 40192, 55124, 33820, 25420, 72,
                47108, 18540, 31760, 11320, 26588, 8540,  37904, 55612, 13528,
                31392, 35740, 59704, 40184, 17648, 53392, 27904, 1684,  5488,
                39860, 17572, 61772, 1480,  26544, 13112, 38912, 9712,  22064,
                54088, 37432, 32676, 24620, 56324, 9144,  50772, 34580, 12152,
                55688, 52192, 6496,  59552, 11904, 44496, 39268, 53680, 7744,
                51444, 4724,
            },
            {
                63308, 16072, 35096, 59548, 37820, 18392, 61044, 39704, 22648,
                43548, 53432, 28712, 21860, 57500, 46568, 15776, 50464, 8140,
                28524, 14440, 9604,  1404,  63924, 38400, 44952, 31252, 35088,
                16332, 57660, 29252, 60,    23956, 14732, 30316, 22060, 6812,
                56116, 23580, 9272,  57400, 29232, 41056, 55216, 12716, 33524,
                15264, 49344, 17996, 10840, 35464, 6636,  59024, 45796, 27656,
                63520, 31192, 39036, 50284, 45888, 55916, 28892, 17700, 24724,
                8748,  65392, 2436,  46528, 18144, 8992,  13844, 33000, 20700,
                44132, 37716, 15332, 35548, 20932, 5476,  10564, 29992, 61668,
                36484, 5592,  63568, 54820, 42464, 19716, 33460, 51204, 2464,
                18220, 45652, 52536, 212,   30724, 58284, 43592, 33148, 55240,
                59052, 29892, 36068, 49808, 57344, 19380, 63332, 47044, 28548,
                2940,  64008, 7200,  20096, 13596, 29984, 21992, 908,   64088,
                30816, 3312,  24204, 32032, 18780, 29204, 1644,  64276, 14772,
                28392, 45092,
            },
            {
                23396, 30040, 12852, 54804, 8780,  32172, 52836, 26356, 1604,
                33272, 59744, 3804,  38436, 52252, 25048, 11948, 39356, 64164,
                23756, 55056, 32280, 26936, 19060, 15440, 2672,  6048,  59900,
                25620, 13144, 39648, 47864, 56724, 9600,  44348, 53460, 48748,
                33424, 27312, 37652, 4880,  14084, 22196, 51152, 26608, 38172,
                60256, 7748,  24956, 64884, 51552, 19972, 39192, 4232,  14620,
                56368, 7588,  20104, 736,   64700, 10520, 21648, 58260, 49820,
                32692, 27008, 15352, 30580, 53404, 43264, 35996, 64612, 27332,
                11028, 50912, 58784, 1464,  46584, 63800, 51712, 56616, 39692,
                13168, 50968, 28816, 3432,  49276, 15232, 62828, 40688, 28088,
                57480, 8340,  26836, 15276, 64468, 20796, 10468, 48264, 13712,
                23448, 6944,  12232, 44224, 4388,  24228, 16224, 5248,  40800,
                49572, 17704, 44092, 61072, 53240, 45956, 59444, 38468, 17912,
                44088, 14560, 40180, 62352, 50712, 56128, 22244, 46444, 33032,
                19888, 9948,
            },
            {
                58012, 52216, 46148, 2876,  22136, 47916, 14448, 6396,  57152,
                19368, 47164, 14644, 65092, 8692,  35620, 59612, 20276, 47768,
                6148,  45280, 60280, 41868, 51268, 47468, 56612, 20516, 42772,
                51948, 3064,  45108, 20152, 34692, 60008, 3752,  37244, 19116,
                808,   43872, 65500, 49680, 46128, 1380,  62668, 43540, 3244,
                30800, 46564, 39996, 1096,  44320, 31720, 62132, 24492, 49024,
                34472, 44424, 53812, 24832, 31956, 42016, 35400, 5368,  61272,
                39216, 48080, 56464, 6624,  62396, 25728, 3552,  57512, 16796,
                54504, 23336, 7904,  26664, 31352, 42864, 16172, 19932, 7984,
                23480, 58552, 16952, 45336, 24568, 57060, 6804,  21644, 11208,
                61444, 23156, 41920, 50612, 37632, 7384,  24964, 36520, 62544,
                41704, 20208, 64544, 28232, 37856, 51852, 59936, 30740, 55692,
                35028, 12140, 27832, 2320,  35984, 5004,  25884, 49124, 7828,
                58408, 27708, 47272, 9820,  35296, 7332,  41648, 12252, 59008,
                49540, 40996,
            },
            {
                496,   18900, 33244, 64240, 27668, 58148, 42300, 35700, 50000,
                10308, 40372, 26812, 32216, 18140, 4572,  42900, 30968, 208,
                16936, 38188, 21612, 11636, 7644,  61504, 32976, 28072, 48992,
                8796,  62980, 31836, 54780, 16032, 24976, 27956, 63964, 12176,
                58332, 15164, 52712, 19912, 31572, 35168, 9016,  17772, 56820,
                11284, 21332, 53728, 28160, 13012, 16724, 9424,  53256, 1636,
                21800, 10728, 59916, 16080, 4136,  52080, 13516, 45148, 1740,
                19836, 11340, 35264, 22580, 38356, 12448, 48696, 39828, 6416,
                32264, 41232, 62772, 49640, 13664, 38152, 3104,  28248, 48716,
                43368, 32944, 9544,  38720, 35124, 688,   30564, 47504, 43700,
                36872, 54024, 12880, 32624, 3316,  44796, 55888, 17196, 2692,
                50744, 33828, 47320, 56240, 10588, 33044, 42968, 8472,  20732,
                24796, 58700, 51520, 31504, 40860, 15972, 11068, 33048, 54196,
                19968, 36584, 53092, 132,   16764, 26580, 61488, 3748,  24368,
                35660, 6384,
            },
            {
                37924, 25888, 43356, 11164, 38864, 4488,  17332, 29444, 63904,
                23868, 54484, 872,   44368, 49420, 56024, 27452, 62020, 53372,
                34196, 57816, 2996,  29552, 35812, 23600, 39564, 10968, 16840,
                36452, 23388, 4924,  11196, 38960, 51856, 7592,  41576, 32040,
                23168, 39852, 7216,  11200, 26136, 58944, 53676, 23812, 47720,
                36204, 63528, 6008,  34528, 60888, 47780, 42240, 36784, 30300,
                64140, 40544, 28596, 36632, 48096, 61984, 23340, 30196, 16200,
                54576, 42348, 59420, 2944,  51108, 19420, 60128, 29128, 1148,
                47628, 19200, 35076, 10360, 55608, 24444, 59668, 35388, 63152,
                1296,  54052, 21060, 64244, 13552, 52932, 60148, 16792, 33228,
                1860,  5856,  19680, 62436, 47732, 28024, 59948, 31432, 53516,
                9648,  26064, 412,   15864, 22012, 3580,  48908, 14088, 65372,
                820,   45460, 9228,  22268, 64308, 55880, 42980, 60948, 23988,
                4340,  12544, 64500, 21772, 57112, 48156, 38276, 31476, 54452,
                16316, 62848,
            },
        },
        {
            {
                31132, 57864, 18816, 28644, 64212, 31716, 52172, 8392,  14328,
                58644, 47228, 34676, 10304, 16420, 38316, 13176, 31724, 22944,
                28860, 35340, 49372, 24756, 41608, 61528, 20068, 1320,  52112,
                59120, 6012,  53776, 45844, 22968, 4888,  39652, 47264, 1104,
                23724, 62516, 5900,  55488, 11960, 51708, 30608, 14628, 60360,
                42380, 1080,  47888, 13376, 9156,  3888,  24796, 50228, 40208,
                8672,  18076, 26564, 63436, 41304, 30492, 4084,  43820, 38360,
                3140,  15096, 42404, 27760, 46216, 38740, 2960,  36224, 44888,
                28824, 23012, 35328, 20408, 32360, 40848, 62420, 10612, 44416,
                63780, 48156, 36600, 56932, 1652,  32012, 62692, 5820,  20152,
                51924, 2396,  49388, 6984,  58440, 20284, 61024, 54056, 31780,
                18608, 51828, 34984, 42612, 9228,  50584, 1704,  18020, 13288,
                42684, 30200, 55904, 61836, 40232, 50536, 64204, 28948, 41792,
                60672, 49480, 23676, 44204, 59888, 29960, 50032, 62544, 23464,
                44768, 38184,
            },
            {
                50908, 41420, 8880,  54664, 11628, 21808, 40356, 43636, 24712,
                38052, 6224,  60924, 41544, 49956, 58736, 3016,  43408, 17320,
                6168,  59304, 2800,  53220, 16768, 7484,  30916, 26496, 40796,
                24048, 38496, 9720,  61720, 18476, 28868, 65080, 33028, 58468,
                51496, 31028, 38604, 48896, 24912, 20044, 36460, 55088, 7488,
                32240, 24252, 64848, 29160, 54580, 61588, 46400, 14640, 2092,
                60512, 34204, 53172, 2912,  23324, 50896, 17924, 27244, 52864,
                11864, 57172, 35384, 7332,  53652, 16492, 49064, 60784, 53188,
                15632, 49920, 61812, 9168,  59216, 4520,  48836, 55836, 31836,
                24760, 5544,  29448, 11176, 45212, 53276, 38708, 28720, 56400,
                16872, 40568, 61672, 31060, 10952, 42360, 13984, 25436, 44720,
                7524,  48432, 26500, 58052, 20120, 61580, 26076, 54684, 38300,
                21464, 8560,  51936, 35936, 1124,  18700, 32632, 15404, 2144,
                35528, 19528, 52220, 40312, 16284, 25780, 5052,  36780, 7752,
                56044, 2292,
            },
            {
                16052, 62196, 24568, 48948, 37084, 4920,  61640, 1476,  55512,
                20456, 12296, 52224, 30204, 22468, 7668,  33988, 56156, 52036,
                39668, 63000, 14032, 33252, 45952, 57636, 11480, 49708, 14544,
                3688,  29796, 34924, 57548, 49824, 7532,  37280, 14380, 26764,
                44428, 8776,  16728, 1932,  58336, 63252, 3796,  44924, 27136,
                57192, 51004, 16224, 40132, 44028, 19076, 31320, 57224, 21600,
                43752, 11488, 31552, 46852, 58384, 12976, 35240, 61204, 21520,
                47932, 29808, 19684, 59964, 1216,  25580, 21996, 11236, 7936,
                26432, 248,   46036, 14144, 37220, 23528, 27140, 3060,  17300,
                58680, 51320, 42888, 18152, 25836, 8112,  22932, 11884, 44240,
                34128, 9256,  21500, 45920, 26700, 996,   49812, 35968, 63796,
                3388,  56496, 15112, 11452, 38780, 45604, 5744,  33888, 60648,
                2992,  46540, 23564, 14632, 44056, 58880, 53416, 6988,  45968,
                25140, 57812, 3500,  8368,  63852, 45936, 20128, 41824, 13892,
                29320, 21664,
            },
            {
                46652, 35640, 776,   32948, 59908, 45676, 17292, 29508, 33516,
                63212, 44688, 26376, 60,    18576, 46132, 62076, 10436, 25552,
                20368, 47156, 9224,  27292, 22432, 64256, 34824, 43452, 54900,
                63480, 48576, 16924, 56,    25300, 43804, 11148, 53252, 3040,
                19240, 61392, 35508, 43368, 32992, 9628,  39936, 17988, 11796,
                38820, 3336,  35580, 6472,  26104, 11688, 38368, 7544,  35828,
                25636, 49516, 6056,  20180, 37876, 6808,  45480, 54772, 784,
                40088, 64268, 9616,  49728, 32500, 40388, 57612, 43308, 34032,
                64088, 39464, 30560, 18836, 50844, 54008, 12688, 35432, 40468,
                8340,  14108, 38212, 64836, 34304, 58580, 15976, 47776, 816,
                60376, 24908, 53868, 36672, 64700, 56136, 16452, 9008,  21840,
                39280, 28924, 33560, 1236,  52908, 29900, 16332, 11784, 49440,
                57792, 31472, 63920, 5660,  26004, 11988, 38260, 21824, 61396,
                31364, 12444, 38164, 28500, 55020, 10940, 33312, 53032, 48072,
                65420, 10468,
            },
            {
                58444, 27724, 6092,  20208, 13816, 26320, 54216, 10676, 49128,
                4544,  15972, 57616, 36352, 54244, 40636, 27624, 15172, 36220,
                2032,  30704, 42604, 55932, 576,   39224, 5068,  18896, 7824,
                22180, 45200, 12248, 40320, 31240, 60760, 56600, 22444, 48020,
                39824, 55796, 12740, 21468, 28568, 47788, 53024, 23028, 63956,
                47200, 20916, 61128, 52880, 59336, 944,   51288, 64192, 16624,
                54096, 59524, 14964, 64788, 42540, 25368, 9808,  32828, 16136,
                5884,  24636, 37348, 44824, 13804, 63392, 5616,  29600, 17396,
                54548, 4112,  58616, 41976, 6780,  65344, 44616, 20628, 47612,
                61624, 22728, 376,   54516, 3972,  49868, 40824, 63516, 31548,
                50728, 4484,  13236, 18208, 5220,  40040, 33432, 60052, 52616,
                17700, 43184, 62120, 47436, 23240, 65380, 36848, 42000, 27976,
                19096, 10008, 39792, 34052, 56548, 29512, 48644, 9852,  42736,
                54328, 17176, 48220, 34488, 22136, 1816,  58784, 17424, 3712,
                32236, 40252,
            },
            {
                18456, 52920, 43864, 56964, 38896, 51488, 7004,  35704, 42336,
                22664, 39368, 8660,  64312, 13276, 31148, 3960,  48392, 64808,
                51080, 58420, 17248, 36732, 12768, 50864, 28228, 60564, 32688,
                37528, 27276, 62164, 52436, 20436, 14980, 4704,  34596, 29480,
                7244,  24660, 49568, 62588, 6048,  15368, 59788, 548,   34240,
                28996, 14712, 9772,  31892, 22336, 47620, 28052, 42228, 4260,
                30240, 39716, 76,    28588, 56024, 17428, 62232, 28944, 58948,
                43068, 52440, 17676, 3716,  28328, 51336, 20040, 37748, 48376,
                12372, 21340, 25332, 10072, 33704, 28892, 1404,  56796, 30152,
                52516, 33188, 27536, 45772, 19944, 29756, 6308,  36632, 19016,
                27060, 58656, 43092, 30412, 48064, 22720, 28116, 2592,  46968,
                10672, 24732, 58516, 6520,  18948, 8604,  56236, 2248,  24432,
                53572, 44672, 16048, 51452, 19788, 4136,  65228, 35092, 72,
                27356, 63320, 4604,  60132, 42968, 50416, 38572, 26804, 60948,
                23360, 6812,
            },
            {
                37872, 14852, 62724, 9816,  30692, 23144, 65076, 19180, 59808,
                27568, 50768, 32544, 46692, 20776, 6724,  56476, 22552, 43848,
                11780, 5828,  23788, 62292, 54240, 20856, 47728, 15304, 57072,
                10508, 2540,  55428, 6568,  36488, 47112, 42060, 64208, 16400,
                59352, 32088, 3480,  54092, 38192, 25308, 31392, 42712, 50316,
                5272,  56544, 45272, 41372, 17064, 34796, 13124, 56688, 23756,
                10296, 45628, 21288, 34736, 48540, 3612,  51352, 36424, 47344,
                11280, 31844, 60720, 55348, 23164, 10440, 56648, 2344,  60480,
                44452, 36124, 52216, 46792, 61188, 17724, 39036, 10848, 15596,
                5636,  42500, 10408, 60004, 13264, 24448, 55828, 9812,  14196,
                46496, 38252, 8328,  61348, 51540, 11428, 43988, 55356, 31184,
                37628, 4312,  14200, 41116, 31616, 51272, 44264, 13916, 62740,
                7072,  36464, 804,   61200, 46408, 40852, 24264, 52512, 14028,
                20588, 39736, 8916,  24824, 14624, 30908, 7324,  12800, 54784,
                43384, 50540,
            },
            {
                25564, 34028, 4144,  48124, 41392, 3048,  45368, 14744, 968,
                55756, 11852, 3732,  25160, 52456, 42244, 60228, 32888, 18312,
                28524, 40448, 33816, 46320, 8744,  31524, 3460,  41044, 24784,
                51136, 42896, 18308, 33400, 24140, 9368,  26976, 920,   51624,
                45144, 9876,  41376, 18780, 45596, 11476, 57664, 8736,  17420,
                61712, 37108, 24080, 3012,  54788, 8080,  62132, 19392, 49068,
                36748, 61708, 52804, 12448, 8524,  41184, 22864, 13624, 1888,
                26596, 20832, 7604,  35876, 40940, 45948, 33052, 15360, 24460,
                7652,  31520, 56976, 15004, 4712,  22772, 49688, 59332, 25852,
                63160, 37472, 17520, 49120, 35124, 62032, 42884, 51648, 23112,
                57048, 3260,  34516, 15968, 25616, 63152, 6888,  14656, 19680,
                64528, 49104, 54540, 35308, 25964, 59536, 21580, 33644, 47952,
                30088, 58232, 22324, 27464, 13064, 8356,  17744, 59712, 45084,
                33504, 51620, 46236, 56020, 18736, 64612, 47644, 20900, 35376,
                1424,  11544,
            },
            {
                59444, 55112, 22048, 28372, 13024, 58104, 32884, 37360, 47528,
                29924, 62576, 18388, 57948, 36964, 16164, 9924,  38268, 808,
                60928, 52700, 14944, 2536,  26012, 44272, 63520, 35296, 5656,
                59456, 30116, 13692, 63688, 50120, 58756, 54036, 38152, 20912,
                13304, 36196, 27856, 65192, 2384,  35108, 52016, 20664, 40168,
                26740, 12224, 48344, 64712, 29452, 38672, 44680, 5728,  31732,
                2316,  17264, 57924, 24896, 32128, 63672, 18908, 57672, 39208,
                53468, 65404, 48264, 14596, 524,   62896, 26656, 49648, 40092,
                64636, 19224, 948,   40788, 26960, 54992, 43624, 35644, 3384,
                47064, 22216, 56672, 7728,  39364, 4648,  31940, 1252,  65316,
                29076, 20036, 41760, 54596, 216,   21084, 38576, 57500, 34112,
                26756, 8832,  44512, 16984, 396,   11228, 39476, 3808,  17356,
                41292, 10776, 49212, 32900, 54756, 37668, 30796, 2928,  57104,
                5192,  29628, 11540, 36620, 1012,  40500, 57180, 28700, 45612,
                63616, 30628,
            },
            {
                8220,  45148, 17004, 63880, 50800, 7420,  25476, 54600, 9468,
                21800, 40372, 34660, 44820, 2088,  28840, 49788, 55004, 26324,
                8408,  47820, 57956, 19756, 39592, 56312, 17468, 12452, 48312,
                20216, 38012, 46136, 1456,  40592, 12528, 18084, 32404, 62184,
                23984, 57012, 50876, 15024, 22860, 60444, 28288, 46972, 1784,
                53364, 32936, 6900,  19664, 50680, 15320, 25664, 59240, 51812,
                41592, 27620, 7076,  43688, 37524, 54720, 28120, 45204, 6684,
                16916, 33984, 43616, 28732, 58812, 18460, 9040,  53856, 3420,
                28828, 11708, 50588, 62492, 34188, 12424, 30656, 8264,  18884,
                54356, 31584, 2516,  28488, 52628, 21028, 45624, 17228, 39712,
                49028, 10620, 59720, 31748, 46112, 28396, 50216, 41556, 1868,
                51968, 22172, 29884, 56624, 63900, 49344, 28144, 60540, 55324,
                25600, 64776, 15572, 2628,  41936, 62456, 50132, 26188, 42268,
                16028, 61920, 21760, 26508, 52620, 33060, 4228,  9604,  16500,
                38828, 19608,
            },
            {
                52268, 41064, 664,   34744, 38416, 20088, 61100, 17200, 42536,
                5284,  50060, 14044, 8180,  62012, 23592, 13040, 64000, 45240,
                21396, 35568, 30048, 65356, 10932, 28652, 51604, 23412, 61676,
                8908,  27036, 56636, 22292, 5368,  29424, 44228, 8008,  2748,
                47176, 4820,  31152, 8252,  55320, 42080, 6580,  13464, 63868,
                22588, 43900, 58864, 36284, 668,   55800, 10772, 22212, 34356,
                12904, 65108, 47580, 15652, 1572,  5428,  49144, 10588, 31352,
                59836, 2832,  23768, 11788, 51980, 38024, 21552, 34532, 42596,
                58236, 22960, 37704, 45060, 5548,  20672, 64140, 51736, 39948,
                13412, 44708, 64492, 15328, 58876, 25688, 11700, 55056, 34252,
                7056,  24720, 52260, 13684, 36716, 9396,  60072, 16908, 12072,
                61656, 3952,  40464, 12992, 37288, 7028,  45696, 20364, 8304,
                37916, 5096,  44336, 57400, 20168, 6412,  11912, 22488, 36800,
                8484,  47136, 57892, 6204,  44020, 14148, 61156, 25076, 53620,
                48404, 3540,
            },
            {
                61960, 31328, 24064, 10128, 55784, 44304, 2576,  31752, 52732,
                64644, 26928, 59104, 31388, 48548, 41008, 34324, 3468,  16708,
                42552, 5092,  13316, 48932, 6784,  36856, 328,   33244, 42088,
                3128,  52372, 16584, 32892, 60496, 48232, 25352, 54844, 39120,
                59636, 42824, 20132, 37944, 48144, 32620, 16964, 38824, 29876,
                55560, 8996,  14208, 27660, 42120, 31404, 63528, 39964, 3680,
                54464, 20424, 30780, 51244, 61004, 21712, 35020, 62472, 25064,
                49944, 41560, 55872, 4848,  30836, 44744, 6408,  62204, 13884,
                47304, 31920, 9120,  16772, 56268, 47568, 1460,  24052, 60248,
                27000, 9804,  41120, 34684, 47260, 8608,  62980, 27564, 14580,
                61516, 44476, 3628,  18696, 64448, 5424,  32452, 24576, 45812,
                36232, 58012, 47856, 18812, 23624, 53464, 34436, 14176, 52504,
                30992, 50612, 35540, 24768, 29212, 47308, 33764, 64504, 55160,
                19536, 31276, 40104, 63544, 17920, 49948, 22040, 41784, 58556,
                12712, 27460,
            },
            {
                37324, 14220, 50260, 59260, 15472, 26652, 48748, 10864, 23488,
                36056, 1468,  16804, 21224, 54060, 6364,  19492, 51920, 31108,
                61736, 24848, 53736, 41404, 22060, 61124, 46208, 54656, 15048,
                30688, 64112, 43260, 10172, 37300, 14456, 64736, 34584, 10904,
                15624, 26228, 63420, 12368, 172,   25384, 61620, 50796, 3844,
                35408, 18212, 49392, 60712, 21124, 4740,  48040, 17688, 26448,
                45352, 9384,  38848, 24420, 11680, 41048, 17360, 43964, 14352,
                8652,  19708, 36508, 63800, 15808, 49176, 56696, 27732, 17824,
                4352,  54400, 60164, 24964, 29484, 41688, 15604, 36700, 32832,
                4620,  50620, 23188, 1060,  19192, 37840, 50220, 42280, 1744,
                37172, 30668, 56520, 40800, 22828, 47624, 56008, 42624, 20440,
                6324,  27484, 32896, 9540,  62092, 30432, 1620,  43212, 63052,
                22952, 12156, 18276, 61560, 9692,  52464, 15012, 44624, 1360,
                51216, 12864, 2668,  34980, 28308, 7768,  36804, 30232, 5736,
                34616, 21116,
            },
            {
                57284, 46544, 6368,  42152, 29636, 4988,  62960, 40120, 13620,
                46372, 55628, 39160, 10480, 45668, 26176, 36908, 58856, 8948,
                56068, 38128, 1728,  32212, 57780, 17012, 9792,  25824, 58564,
                39472, 5916,  23960, 50660, 55548, 3536,  21380, 52156, 18872,
                30068, 50020, 34088, 53436, 58920, 19212, 43352, 10528, 57392,
                46212, 25556, 2332,  37644, 53872, 43772, 8212,  58204, 35300,
                62300, 2120,  56500, 33648, 58908, 27392, 55060, 360,   38088,
                57472, 28164, 52944, 22456, 40364, 1312,  24300, 10980, 51860,
                39268, 20184, 35168, 7096,  52168, 61424, 10752, 57404, 48756,
                17368, 54888, 61876, 29856, 57420, 6352,  32432, 17668, 58188,
                20696, 48532, 11572, 27384, 51412, 1296,  14320, 29772, 53372,
                65088, 15628, 51268, 43576, 4676,  58964, 26196, 17196, 39324,
                3404,  46920, 54432, 832,   40604, 59028, 5032,  38888, 27876,
                60184, 24672, 48504, 54268, 10640, 56780, 46424, 280,   64392,
                44348, 9216,
            },
            {
                17932, 2652,  65288, 21884, 36528, 54192, 33020, 20516, 58492,
                28760, 5924,  33636, 62460, 56816, 260,   41940, 11968, 28180,
                47016, 14908, 19244, 51044, 27328, 44652, 4832,  35752, 20556,
                12568, 47312, 34528, 18244, 28168, 7556,  45636, 41796, 1092,
                56900, 5384,  9268,  44900, 28908, 4396,  36372, 23880, 15528,
                31860, 41676, 65488, 11208, 33276, 24456, 13256, 29372, 49584,
                15084, 42560, 18544, 4732,  46564, 7844,  51940, 30372, 64800,
                46948, 5504,  12772, 34392, 9476,  60328, 31740, 36604, 45572,
                65040, 188,   43728, 14016, 38460, 4048,  27900, 45176, 7504,
                21140, 36016, 43108, 11160, 52340, 24612, 45080, 4320,  52832,
                23816, 8188,  63568, 16352, 34944, 38676, 61456, 7596,  10968,
                39580, 880,   24940, 56288, 13756, 40924, 49660, 55536, 9392,
                59696, 32740, 26436, 36784, 16704, 31368, 23904, 18220, 10216,
                42636, 33336, 16484, 22608, 38832, 61892, 19480, 51712, 15736,
                24328, 54612,
            },
            {
                38916, 33472, 25840, 12036, 47340, 17844, 1892,  50720, 8168,
                42816, 18708, 49236, 23084, 30180, 15748, 64812, 50240, 22556,
                3652,  33860, 63280, 7400,  12152, 39080, 64512, 49624, 53076,
                28920, 60300, 732,   62788, 40236, 58156, 31416, 24228, 62116,
                46836, 39448, 23492, 16120, 40712, 49472, 63276, 53020, 7568,
                59192, 20348, 50648, 6068,  16952, 62016, 51628, 38492, 6240,
                23268, 52980, 28480, 64032, 13824, 36368, 20192, 4080,  23024,
                32916, 18080, 44600, 62628, 47960, 19268, 42328, 58404, 7500,
                29800, 26116, 50208, 63124, 32660, 18468, 23120, 40096, 65300,
                12936, 27404, 3296,  16268, 39784, 64580, 13440, 28452, 61060,
                40332, 33004, 4564,  43652, 58256, 19088, 26024, 33240, 48780,
                59380, 34732, 46784, 19280, 37520, 21908, 6300,  35152, 28660,
                20308, 44496, 64348, 7624,  49488, 55036, 46428, 63344, 35948,
                56924, 5396,  65368, 45052, 3412,  13648, 26676, 32744, 40492,
                48900, 28728,
            },
            {
                7724,  43816, 51652, 60212, 9264,  41436, 61680, 37372, 24588,
                64260, 52836, 2784,  13136, 43876, 7832,  33248, 18480, 45160,
                39492, 59572, 24532, 43064, 54196, 30928, 23244, 3172,  17364,
                8096,  43872, 22176, 11484, 49540, 15844, 37008, 9856,  13548,
                35312, 20352, 64332, 58036, 6864,  12856, 21300, 33512, 1144,
                27980, 13180, 39396, 29788, 56540, 45304, 19920, 232,   61068,
                32912, 10764, 37812, 49280, 23860, 41732, 61108, 11196, 50616,
                40608, 58840, 2024,  25092, 29196, 51464, 3836,  13020, 22644,
                15636, 54964, 9344,  21912, 47416, 58732, 53452, 796,   31116,
                55980, 46960, 59308, 32788, 48236, 20244, 55508, 10108, 35712,
                15192, 46548, 54592, 29272, 10264, 3288,  52576, 44316, 22316,
                17552, 28768, 3444,  62940, 31264, 11340, 64852, 47288, 15220,
                52704, 4700,  13604, 42248, 21892, 11376, 3988,  29180, 51796,
                20584, 8872,  27432, 51108, 31172, 42844, 7212,  60468, 11412,
                1916,  62644,
            },
            {
                58176, 20144, 30880, 16068, 56264, 27612, 6488,  31024, 15052,
                11120, 35136, 27824, 59864, 37904, 51748, 24808, 61224, 5804,
                55448, 10048, 48492, 35908, 904,   14796, 60596, 33200, 40952,
                57564, 36760, 54412, 32716, 26136, 2664,  53324, 59736, 27800,
                50736, 1964,  32568, 26884, 54012, 30208, 38704, 47296, 43844,
                62200, 54876, 47804, 2680,  35964, 26920, 9824,  41628, 57656,
                46744, 20928, 55452, 1204,  31656, 16828, 45152, 55644, 26276,
                15264, 9172,  54488, 36844, 6820,  16380, 55936, 34288, 49384,
                61064, 40912, 31372, 2892,  12288, 42648, 34760, 15180, 50124,
                37800, 5848,  22624, 9028,  36916, 64,    31288, 42792, 50696,
                1996,  19860, 60024, 24564, 49700, 40660, 63440, 5336,  56876,
                12764, 42368, 54568, 8124,  51468, 44044, 25584, 68,    57724,
                37688, 24044, 30260, 56736, 34812, 59972, 43628, 14512, 392,
                48080, 40164, 15092, 36592, 58132, 21344, 55676, 35088, 53604,
                22928, 14576,
            },
            {
                47768, 5028,  40324, 504,   34964, 23208, 49576, 57920, 46096,
                54776, 40220, 4456,  47672, 20404, 57148, 1944,  36244, 14036,
                30812, 21496, 16824, 28636, 58356, 19760, 46232, 11064, 48764,
                25152, 15348, 5704,  46900, 19128, 65364, 43192, 5040,  18028,
                55792, 42484, 10856, 47832, 18384, 3068,  57240, 9652,  24548,
                17852, 34412, 8928,  21592, 63992, 14728, 54044, 31188, 24804,
                16292, 7216,  43152, 62396, 9512,  58596, 34272, 6284,  37788,
                48228, 30064, 61336, 43356, 21688, 39888, 64136, 25424, 1656,
                46304, 18192, 36148, 56132, 26552, 5676,  62240, 9756,  25220,
                19068, 44580, 63492, 51020, 25808, 60456, 7328,  22784, 62664,
                26748, 38376, 6032,  13328, 35624, 20996, 15276, 31248, 37956,
                9092,  61328, 35892, 24120, 15712, 58584, 33652, 18880, 41364,
                11016, 50160, 62704, 1516,  19276, 26868, 38320, 62248, 32652,
                25084, 54824, 61176, 1968,  10288, 47488, 17080, 4876,  44804,
                30348, 36884,
            },
            {
                55212, 24628, 64060, 45828, 53084, 12708, 43060, 18352, 1340,
                21352, 25444, 62772, 16444, 31412, 10792, 40648, 26460, 53292,
                42284, 2432,  65144, 50672, 39860, 6304,  55336, 27764, 63084,
                1512,  31040, 61820, 41836, 8780,  34404, 22696, 48172, 30944,
                7476,  37620, 15124, 61184, 44276, 35840, 64880, 14440, 51480,
                4244,  42436, 26008, 58460, 43400, 4916,  48888, 37356, 3320,
                65072, 34800, 13488, 25872, 51368, 3504,  28564, 21304, 64144,
                3072,  18824, 33552, 13508, 53420, 9940,  44700, 30376, 37972,
                11320, 6496,  59848, 44560, 51600, 17404, 29864, 41524, 57292,
                2376,  29208, 11744, 40488, 14604, 53448, 45516, 17792, 57552,
                11776, 48848, 30344, 65516, 45344, 55040, 1848,  23896, 46164,
                27500, 49380, 20592, 5196,  39624, 48032, 4376,  29364, 61408,
                7348,  46168, 16844, 40064, 9444,  52356, 6252,  22364, 57688,
                12184, 45540, 19164, 29692, 24160, 62732, 38008, 25892, 50744,
                18368, 9712,
            },
            {
                33920, 13460, 28224, 8632,  19776, 63012, 4940,  38980, 59656,
                33508, 8820,  42740, 55600, 6180,  45868, 64116, 17580, 59056,
                47444, 33540, 8016,  13056, 44592, 23952, 34512, 8596,  38596,
                21268, 51340, 12684, 58592, 28972, 51780, 14248, 39360, 63356,
                25408, 58304, 52312, 22652, 6072,  25696, 41344, 20504, 29496,
                59824, 37716, 12560, 51764, 32048, 18776, 59480, 12428, 28072,
                52032, 40456, 48164, 30632, 19284, 39552, 53208, 14236, 42756,
                57620, 24040, 50440, 1208,  27224, 58992, 4348,  19144, 61844,
                53800, 28264, 21012, 14092, 64744, 38236, 48936, 21460, 60940,
                52624, 35100, 17348, 56640, 2740,  28232, 34884, 39340, 3832,
                32596, 53752, 42440, 18112, 7376,  32848, 59880, 51572, 62784,
                17112, 676,   53068, 30488, 60644, 9980,  55668, 22636, 53236,
                36048, 31912, 26080, 54336, 44920, 33364, 47532, 17224, 41572,
                7880,  37392, 5076,  34476, 52376, 41204, 13084, 2812,  64980,
                41804, 60896,
            },
            {
                2328,  50100, 38652, 58464, 32096, 36520, 26392, 10532, 29904,
                48608, 52240, 13832, 35924, 23432, 50428, 29020, 7540,  11932,
                22748, 51948, 26768, 37388, 61260, 4212,  52604, 43424, 18120,
                56528, 45104, 24316, 3372,  37448, 55360, 272,   11004, 45332,
                20540, 4076,  34572, 9920,  31544, 50444, 400,   55108, 32904,
                7236,  48720, 16528, 872,   39644, 24016, 8072,  44872, 21740,
                56764, 17768, 1960,  57972, 45504, 11260, 61668, 47212, 8020,
                35880, 11596, 45452, 63400, 38568, 32516, 48068, 14424, 23620,
                50196, 40248, 32932, 852,   24220, 8844,  3860,  33396, 13908,
                7168,  43128, 59604, 32064, 48760, 19688, 64784, 8680,  47132,
                15944, 23308, 616,   58284, 10060, 38852, 26424, 12264, 6188,
                40760, 33120, 44740, 13484, 26000, 36544, 17280, 43116, 13828,
                2724,  20812, 59820, 4336,  14956, 64160, 29516, 2208,  53584,
                31284, 63788, 50140, 15920, 58896, 8404,  32220, 56756, 28388,
                7132,  21844,
            },
            {
                53780, 17736, 43160, 4204,  15204, 51092, 56372, 45004, 15928,
                65260, 5568,  28356, 18968, 61604, 512,   34164, 44664, 38280,
                3228,  62444, 17908, 55704, 21004, 31612, 15476, 64308, 11868,
                4864,  33132, 40052, 16676, 20052, 26664, 59560, 32760, 16556,
                49748, 28876, 40944, 63112, 45660, 16464, 60336, 12080, 46160,
                62944, 22012, 56960, 28580, 62288, 53392, 34156, 63444, 38892,
                10492, 6428,  62024, 22740, 36252, 5688,  24840, 32376, 27720,
                54732, 40732, 5976,  20796, 16992, 52188, 6948,  36060, 57020,
                3544,  9296,  47368, 57676, 35736, 43680, 55304, 26988, 47684,
                38696, 24536, 9880,  22204, 6060,  42520, 12648, 24956, 55556,
                63040, 36696, 28296, 51804, 22120, 47860, 43156, 19220, 36256,
                56252, 10964, 65044, 55116, 41928, 1604,  62152, 50928, 27372,
                64560, 48288, 12032, 37560, 23544, 58448, 11036, 49556, 25568,
                59584, 21280, 43072, 27312, 1056,  44972, 20384, 47996, 15144,
                39428, 46248,
            },
            {
                31824, 26808, 60608, 23128, 47916, 1036,  20448, 60792, 23500,
                2756,  38048, 46752, 57332, 39620, 12652, 53892, 20532, 56340,
                31312, 41152, 46344, 10432, 352,   49596, 26140, 36796, 30044,
                47892, 61460, 7952,  50256, 63536, 46640, 41520, 8128,  53676,
                60580, 1528,  13732, 54144, 19388, 28156, 36344, 39840, 3484,
                26648, 43704, 35464, 10860, 5556,  45912, 15392, 1568,  30172,
                50152, 33600, 27308, 43640, 15512, 50396, 56484, 18428, 100,
                15648, 60764, 29528, 58288, 25364, 10380, 41748, 65372, 27528,
                43564, 16788, 63256, 30164, 18852, 50724, 11444, 61612, 20024,
                484,   64244, 45288, 54312, 37900, 60112, 50788, 30012, 43592,
                5608,  11252, 41156, 60932, 14088, 29752, 2448,  50456, 58940,
                23496, 28344, 7304,  18376, 21928, 46888, 6660,  32476, 39300,
                8792,  43748, 56684, 28612, 41872, 7464,  35604, 19856, 39444,
                4268,  13388, 9744,  56196, 23340, 63144, 35532, 52656, 24092,
                58560, 12088,
            },
            {
                64148, 6380,  9908,  55072, 29588, 40576, 7796,  34792, 41728,
                54188, 32308, 10332, 25900, 4580,  42568, 16152, 63572, 25012,
                5512,  14856, 28412, 35504, 59404, 40772, 57488, 2672,  54316,
                22356, 27284, 58272, 35184, 11336, 4328,  30028, 22208, 35064,
                43504, 24704, 38432, 57700, 8228,  4736,  53112, 23716, 15564,
                8828,  53844, 18504, 41252, 49872, 20712, 26244, 56088, 42516,
                19372, 54608, 12828, 38560, 59652, 2988,  34740, 65132, 39124,
                22388, 49404, 33820, 3088,  47080, 55200, 30892, 20032, 12188,
                34444, 22576, 54116, 12968, 5380,  58744, 39460, 16536, 31704,
                52064, 28508, 14708, 4104,  26532, 33720, 16448, 1336,  20420,
                33108, 49464, 18560, 4056,  34556, 55152, 64428, 15852, 31816,
                3292,  48284, 38620, 51140, 34700, 29740, 11952, 58604, 24008,
                19184, 33944, 1192,  50056, 17784, 55052, 44352, 62568, 15576,
                46976, 29016, 36564, 48356, 40036, 17704, 4632,  10148, 30584,
                4,     36412,
            },
            {
                44808, 21064, 38200, 33768, 63564, 11916, 53244, 27768, 12920,
                17856, 58804, 21748, 51212, 60492, 31668, 27788, 46040, 10036,
                60016, 49316, 53696, 7388,  23568, 13144, 45656, 19532, 9796,
                42220, 14172, 1452,  44160, 25560, 15864, 57188, 64908, 5532,
                18332, 10136, 48768, 32352, 22104, 63888, 42692, 49396, 59188,
                31964, 65292, 1936,  30124, 60984, 37568, 12500, 59716, 4072,
                24132, 46820, 64152, 8100,  30456, 20780, 10756, 48660, 44948,
                8920,  53152, 13120, 43604, 37168, 15724, 560,   60256, 46120,
                52300, 2320,  37712, 42156, 25112, 28952, 3160,  46156, 7972,
                57216, 36144, 49436, 18804, 62392, 10720, 40380, 58660, 52876,
                61688, 26160, 56552, 45184, 24352, 6956,  37804, 9728,  45916,
                20172, 60528, 14940, 4124,  63408, 56984, 16340, 49152, 3936,
                53612, 14680, 60864, 22200, 5244,  31972, 27232, 704,   56760,
                33696, 53108, 65216, 6720,  31864, 12752, 55224, 42488, 60308,
                49740, 16748,
            },
            {
                56972, 51428, 14416, 46688, 18100, 24836, 44220, 2196,  62936,
                49544, 7176,  44588, 15308, 36400, 8288,  49024, 2788,  37632,
                33916, 19640, 43176, 64692, 17332, 33360, 62064, 6656,  38680,
                49468, 31556, 52348, 20852, 54700, 39192, 49208, 12300, 37660,
                51868, 61940, 27272, 2408,  47108, 34432, 13212, 29380, 19516,
                38836, 46936, 13980, 23916, 7940,  52608, 32852, 47640, 9472,
                35808, 16612, 1044,  51852, 25224, 54308, 42424, 28392, 4444,
                25704, 62272, 19468, 6716,  63584, 23736, 50656, 39380, 5596,
                25944, 57492, 8152,  49564, 62068, 21360, 35360, 63540, 42764,
                22804, 11560, 40920, 30556, 56356, 2856,  45472, 23512, 6840,
                37552, 15060, 9096,  39536, 62156, 20992, 52856, 41612, 26180,
                35256, 54000, 43228, 23852, 9432,  40316, 26296, 36004, 62916,
                41028, 30504, 46052, 37044, 64840, 13228, 51732, 24176, 8452,
                42680, 18984, 2264,  25896, 50552, 61576, 28100, 37724, 19424,
                25732, 4996,
            },
            {
                40288, 28572, 1800,  60092, 4568,  36640, 57876, 47496, 30436,
                35516, 25132, 39800, 1484,  64372, 19952, 53004, 23372, 57964,
                12812, 1616,  25744, 38120, 4692,  48464, 29264, 24884, 56092,
                65244, 17636, 37384, 6572,  62140, 33076, 892,   23812, 45284,
                31268, 14748, 55592, 41308, 17416, 60664, 9560,  1004,  56732,
                5996,  51388, 34216, 58216, 44572, 4688,  18052, 28832, 62244,
                41616, 58076, 32152, 45168, 39832, 17456, 63056, 14260, 37312,
                55952, 31896, 41296, 28076, 56792, 35068, 9064,  29100, 18036,
                62720, 31068, 15372, 33372, 46800, 9612,  54604, 13748, 26268,
                1808,  59408, 6564,  47336, 21100, 52288, 35816, 29028, 12420,
                47204, 30280, 2156,  48668, 31968, 11996, 28456, 48,    62760,
                12816, 8176,  29432, 33192, 52688, 45632, 468,   20920, 44268,
                11324, 7280,  25468, 54420, 9556,  40736, 47852, 59740, 38096,
                11828, 22836, 58928, 15784, 43996, 21432, 2676,  47372, 8708,
                62580, 32816,
            },
            {
                22616, 9404,  53724, 42400, 31204, 22096, 8692,  14268, 20616,
                4292,  60032, 54892, 30128, 12212, 56988, 40344, 16996, 43972,
                62800, 51084, 30396, 56388, 11164, 52592, 43484, 14592, 34232,
                3416,  11756, 59892, 29340, 9584,  46384, 18960, 28384, 56676,
                3396,  21316, 36580, 6740,  24604, 51188, 38092, 45044, 26204,
                22540, 11328, 42272, 27496, 15672, 64016, 40144, 22436, 53328,
                5212,  27448, 21632, 11948, 3756,  57652, 6236,  33316, 50224,
                17132, 1364,  11076, 47896, 21212, 13680, 59300, 44408, 48816,
                10512, 40916, 20392, 59704, 648,   17652, 40400, 49828, 32680,
                53568, 17052, 65256, 34228, 13812, 8040,  63452, 17720, 55616,
                41448, 65008, 19704, 57816, 16376, 43268, 50164, 56428, 18060,
                47244, 57576, 2388,  61552, 18992, 13544, 55680, 6000,  28316,
                59948, 17436, 50972, 2744,  33596, 16428, 20356, 3244,  29908,
                62108, 45852, 33144, 38928, 7888,  57996, 33908, 14932, 54460,
                43832, 13156,
            },
            {
                48292, 65412, 25456, 16148, 50248, 55456, 63984, 40060, 52444,
                43076, 9976,  16504, 47664, 34316, 26044, 4196,  32996, 29036,
                7676,  21964, 14916, 41360, 59040, 20256, 1168,  60616, 21784,
                47188, 26708, 43344, 23364, 40184, 14240, 52752, 60344, 41696,
                9020,  64084, 44216, 11816, 58056, 30328, 14972, 64420, 54280,
                36676, 61780, 4192,  20600, 55460, 35392, 268,   49156, 11456,
                15292, 38144, 49820, 60768, 35648, 47400, 22124, 9788,  61684,
                24032, 58452, 52460, 38804, 5180,  54088, 32344, 1732,  22448,
                35856, 55352, 5856,  43780, 24076, 56720, 29568, 4696,  20160,
                37588, 44096, 27864, 23688, 39212, 43096, 25864, 50352, 592,
                22264, 34348, 53636, 26888, 3568,  60288, 33836, 6036,  24500,
                36816, 40016, 22340, 49524, 27300, 37608, 64336, 49908, 34132,
                57036, 38428, 21292, 62416, 28684, 44860, 55240, 35768, 14716,
                51040, 5888,  54752, 27648, 11548, 52336, 40936, 23628, 29572,
                1368,  35984,
            },
            {
                57296, 5392,  34632, 11244, 38480, 6272,  26952, 33100, 17764,
                61632, 28380, 23556, 51676, 5724,  42796, 60140, 11044, 48324,
                53852, 35180, 46552, 3576,  32560, 27252, 37328, 9212,  40420,
                54652, 6176,  51220, 56768, 3036,  63424, 35228, 5408,  16892,
                50288, 32988, 28096, 53732, 46588, 20640, 3212,  33348, 7616,
                16168, 48788, 31156, 52012, 9056,  46128, 25292, 31960, 56432,
                44296, 64676, 8312,  18856, 26040, 52296, 29396, 38532, 46008,
                4068,  43172, 34212, 18340, 64756, 26312, 42240, 16064, 52580,
                64100, 13008, 27740, 50516, 37868, 64664, 11896, 45600, 61056,
                9136,  50944, 3156,  55416, 5868,  58728, 31856, 10024, 60404,
                5024,  44388, 11084, 37196, 7840,  39116, 20652, 14140, 30292,
                65416, 4952,  15908, 42456, 10704, 7068,  31684, 23352, 14340,
                3560,  47696, 42276, 11752, 58832, 23536, 5072,  63944, 25976,
                41264, 17788, 388,   48844, 61888, 19160, 3824,  63896, 49640,
                59624, 17608,
            },
            {
                20492, 30524, 44696, 62144, 19108, 47872, 2048,  12084, 46004,
                36708, 700,   58088, 38420, 20836, 46292, 15464, 65268, 168,
                19080, 38960, 62052, 9840,  17672, 64436, 44908, 50708, 30224,
                18508, 36104, 15384, 32552, 19872, 48812, 26248, 11264, 38076,
                25080, 59552, 204,   18320, 5276,  39512, 59084, 43396, 28208,
                19232, 40272, 2180,  59768, 38396, 13340, 61008, 6796,  19632,
                29140, 2108,  54508, 33168, 13356, 728,   41960, 59780, 19800,
                27600, 14904, 30140, 8612,  45872, 3224,  37860, 61020, 7360,
                25072, 45724, 3768,  31572, 14976, 7416,  34644, 25288, 57748,
                15684, 31448, 12780, 61572, 19440, 48596, 15496, 45728, 38208,
                28532, 14376, 51564, 24276, 46644, 62476, 54972, 49364, 45120,
                9552,  51972, 34268, 54852, 59420, 47476, 17984, 43044, 53540,
                8896,  26820, 31832, 1112,  49772, 39200, 8740,  48560, 31952,
                10356, 56660, 24408, 35428, 30532, 45088, 37260, 10572, 26484,
                7584,  41540,
            },
            {
                38072, 52748, 568,   56344, 28804, 41644, 58500, 24668, 54208,
                7496,  32116, 13780, 63364, 8748,  55188, 36008, 22768, 27412,
                57228, 5216,  24888, 55132, 49288, 23004, 5800,  57276, 12700,
                63696, 320,   59472, 45112, 8280,  29984, 42916, 54816, 20720,
                47824, 13564, 40776, 35196, 62624, 25576, 49856, 10692, 52892,
                63196, 45424, 12120, 23136, 28668, 17892, 50568, 41324, 58572,
                36596, 23832, 40596, 45736, 63336, 56508, 15932, 7252,  53608,
                11588, 62316, 50524, 59036, 21888, 12668, 48000, 28884, 19408,
                40000, 33848, 60148, 54128, 21752, 41860, 52128, 1696,  40072,
                22656, 47220, 36764, 41704, 29712, 3980,  35592, 23092, 56716,
                18764, 63864, 31560, 58720, 17184, 10516, 27744, 2244,  41088,
                58164, 25856, 20084, 1436,  28980, 39044, 3032,  61988, 36340,
                19764, 65236, 16000, 56112, 34692, 18792, 13796, 42776, 58300,
                20496, 38356, 64728, 13208, 21696, 5612,  16812, 53080, 31916,
                46076, 12292,
            },
            {
                60296, 23912, 14396, 8912,  21960, 34080, 15768, 49224, 65020,
                21512, 43812, 49844, 18240, 29088, 2512,  31260, 40968, 51232,
                44340, 12068, 30040, 41800, 36328, 2700,  15848, 33772, 24624,
                42600, 27780, 21652, 37584, 12836, 57912, 1860,  64648, 31440,
                4044,  56140, 23316, 51928, 8300,  30800, 14024, 22244, 772,
                35492, 25344, 57752, 33544, 44012, 65120, 2584,  34448, 9988,
                48240, 16372, 5564,  30420, 9608,  36996, 24268, 32428, 48872,
                36264, 40436, 304,   24780, 35576, 55908, 31928, 9784,  57644,
                51692, 1008,  17536, 10412, 62216, 46560, 18820, 28928, 62572,
                6940,  54908, 120,   25192, 51580, 11692, 63120, 53748, 7648,
                41100, 49016, 6004,  1288,  42904, 32984, 22848, 36032, 18360,
                13100, 32052, 63168, 44868, 14876, 23908, 51884, 12028, 30256,
                58724, 44504, 37936, 6704,  24828, 61304, 53916, 28104, 1700,
                45992, 6976,  52640, 42200, 47504, 60516, 57796, 39928, 22304,
                55396, 4792,
            },
            {
                49744, 27132, 46716, 36660, 50920, 60840, 3148,  30804, 9660,
                40392, 3780,  56212, 25172, 41844, 48060, 13052, 59200, 8324,
                17136, 33868, 59724, 14084, 52868, 28772, 47864, 61704, 7440,
                53456, 10576, 49448, 62608, 51908, 23280, 16128, 39508, 7756,
                44668, 61944, 10444, 16620, 47232, 37492, 57272, 41588, 60236,
                5232,  15288, 54524, 7976,  4716,  53300, 22372, 14548, 26896,
                62904, 52184, 20924, 60416, 49724, 18536, 43892, 3356,  65444,
                21356, 6128,  55064, 43376, 16696, 62532, 49952, 4856,  14616,
                43052, 23384, 48864, 36068, 26540, 4232,  56564, 13368, 49368,
                32800, 19836, 64388, 16328, 57132, 43736, 27332, 1796,  33356,
                12544, 25320, 36524, 20464, 54104, 63596, 7256,  50600, 61332,
                4200,  47800, 6348,  35940, 60368, 55720, 7900,  25984, 49408,
                5472,  22876, 12868, 51160, 46260, 31012, 4668,  22504, 62864,
                33960, 16396, 26348, 2420,  9380,  28124, 34600, 1492,  13864,
                64620, 34184,
            },
            {
                2956,  16932, 42288, 64216, 5112,  12704, 44308, 38324, 19000,
                27504, 60944, 34168, 10876, 53088, 61716, 4944,  20176, 37852,
                63960, 46704, 1992,  21328, 8704,  56576, 39968, 19784, 37200,
                46616, 31796, 3456,  40840, 5740,  34192, 46880, 53160, 26668,
                18488, 33612, 29204, 42236, 65448, 6480,  19020, 33064, 46476,
                29128, 49692, 39176, 20444, 47944, 36944, 30544, 43132, 55104,
                3760,  12820, 42216, 26216, 1844,  54288, 28696, 58192, 13504,
                26488, 47452, 31628, 11108, 7160,  27176, 39588, 21060, 36704,
                60844, 29928, 6708,  58472, 39128, 31192, 8968,  43412, 37760,
                26772, 10360, 45420, 34368, 8444,  21480, 39720, 17640, 51132,
                61636, 45360, 57368, 28272, 39896, 12000, 44472, 56644, 29576,
                38400, 53988, 22408, 9780,  41132, 18932, 34012, 45848, 39880,
                2096,  54416, 28748, 63512, 10244, 40688, 16856, 52196, 37616,
                12392, 49676, 61116, 31852, 55032, 15208, 50476, 25284, 44076,
                18952, 29232,
            },
            {
                61756, 7444,  56892, 29580, 18432, 54744, 23380, 52488, 58820,
                14568, 46748, 6492,  37208, 16072, 23356, 32636, 56420, 26832,
                6372,  24124, 50504, 62264, 43580, 25656, 11312, 59100, 4448,
                17400, 56056, 14784, 25972, 18920, 28836, 61592, 10232, 36668,
                58528, 50520, 1372,  55252, 26396, 3000,  52676, 24292, 9204,
                17496, 64220, 11360, 26780, 62020, 16516, 9316,  60464, 46336,
                31608, 37440, 58080, 34036, 11424, 39572, 46596, 8428,  34724,
                17236, 57060, 38844, 19888, 52760, 44752, 2040,  65012, 25816,
                54440, 11652, 45256, 14232, 19808, 53192, 65164, 17120, 2588,
                58872, 50576, 39184, 4840,  30680, 59144, 46904, 5628,  29988,
                21832, 9032,  15020, 3804,  52252, 16240, 24716, 680,   20732,
                14348, 43340, 26752, 50812, 30724, 472,   64076, 14228, 57760,
                17616, 32756, 42192, 20428, 80,    35652, 60260, 45124, 8172,
                29464, 57728, 22328, 36532, 40784, 19924, 62564, 7048,  48176,
                10020, 40828,
            },
            {
                35548, 22592, 10988, 33164, 39884, 26184, 8184,  35348, 1396,
                49632, 22824, 30056, 63508, 564,   44052, 39876, 49768, 14540,
                54344, 30940, 38984, 18664, 35000, 628,   32108, 45456, 23404,
                29776, 63856, 35448, 59728, 43968, 56436, 696,   13964, 22440,
                4908,  39756, 15044, 21240, 34956, 44136, 12980, 61696, 56452,
                36296, 42532, 30936, 2068,  56072, 40816, 25148, 1536,  19356,
                8116,  23232, 47908, 6280,  64328, 15248, 22800, 61448, 42264,
                51192, 1676,  63820, 29092, 59220, 13836, 32324, 46684, 16840,
                4412,  34400, 63200, 41468, 2136,  24516, 47412, 35468, 22144,
                54576, 14068, 24136, 61324, 53500, 13148, 36728, 65480, 54692,
                41684, 34264, 48664, 58972, 31128, 37512, 47360, 34804, 59384,
                8588,  64440, 17164, 57428, 12436, 48340, 28064, 21900, 36012,
                62436, 9112,  48436, 59196, 26600, 56724, 13016, 24376, 2708,
                42944, 18260, 4816,  10868, 46324, 3220,  56956, 37428, 31384,
                58308, 53864,
            },
            {
                13936, 51772, 43684, 48412, 4324,  58068, 45688, 63832, 32340,
                11760, 42992, 17836, 50832, 57408, 28044, 10248, 3392,  36052,
                45100, 11620, 4428,  57856, 15960, 49268, 65472, 13972, 52092,
                41260, 2556,  48648, 11892, 8056,  38264, 31956, 48968, 42524,
                60808, 24724, 47716, 11068, 59904, 50900, 28188, 39808, 5912,
                21528, 3900,  51904, 44852, 33712, 12320, 58412, 51512, 38800,
                63624, 16780, 28512, 56360, 19936, 50580, 36496, 5448,  30744,
                12020, 24152, 45888, 5004,  36080, 22844, 56148, 7896,  40696,
                49400, 27892, 22168, 56848, 51148, 33584, 11792, 60264, 5260,
                29748, 42104, 17996, 1592,  48556, 19636, 25548, 10336, 16220,
                500,   60884, 23648, 7120,  19928, 64916, 9960,  27428, 49184,
                40336, 32300, 2980,  36980, 61092, 42632, 4548,  52564, 6744,
                43712, 24996, 14620, 37416, 5968,  18876, 33160, 50368, 63692,
                39220, 48500, 54272, 65476, 24512, 29832, 42732, 12464, 21536,
                0,     26124,
            },
            {
                38584, 59940, 2280,  20236, 62808, 13132, 16752, 28468, 5684,
                56028, 59952, 39400, 8380,  20880, 46604, 65036, 18092, 58916,
                21516, 63060, 41532, 27832, 8144,  54680, 37648, 6132,  27056,
                9548,  19956, 54340, 24792, 46468, 21216, 65016, 17244, 52596,
                30108, 7044,  56516, 32120, 38220, 7928,  16312, 48272, 31932,
                53756, 14504, 59476, 23420, 18072, 6456,  49048, 28092, 34336,
                44528, 52600, 84,    43084, 31944, 25604, 2504,  53336, 19136,
                60188, 37896, 9568,  16188, 42580, 50412, 10996, 59928, 19512,
                53488, 428,   9892,  30416, 16208, 7300,  27240, 39704, 45136,
                8808,  64004, 36024, 27968, 43244, 33012, 3928,  50188, 44412,
                27004, 38288, 11876, 44944, 40664, 2908,  55496, 18448, 4964,
                52344, 23248, 45744, 7460,  19644, 24180, 38944, 56804, 11464,
                30624, 51528, 4108,  53552, 41580, 46808, 54560, 30272, 20892,
                9732,  27228, 14464, 32824, 51060, 8292,  16488, 53036, 63372,
                45400, 17904,
            },
            {
                6468,  28740, 24556, 53356, 30772, 37948, 50104, 42028, 19876,
                36392, 26052, 2340,  32060, 13280, 34704, 24688, 52984, 29552,
                7632,  33616, 51808, 23328, 46780, 30212, 20840, 60520, 44356,
                58100, 32396, 39484, 16364, 5880,  53536, 26892, 2916,  35120,
                12164, 62688, 43888, 2364,  19464, 63744, 22936, 624,   45392,
                62540, 26236, 9400,  47708, 38276, 64608, 21708, 15036, 4304,
                10744, 60068, 13988, 39164, 9496,  62548, 44420, 57344, 27628,
                47652, 33228, 55696, 61532, 26900, 1308,  34564, 29720, 24892,
                36348, 60756, 46864, 38084, 43312, 63608, 55144, 49140, 19116,
                32272, 51668, 11580, 56304, 7736,  63016, 40352, 58568, 20692,
                56284, 31828, 53496, 61972, 28624, 50472, 13492, 43496, 61164,
                29912, 12096, 56084, 34620, 53772, 15212, 32528, 45532, 18532,
                61316, 21120, 34492, 64732, 28268, 10828, 7060,  15524, 60992,
                35484, 764,   44508, 38296, 19288, 60876, 36028, 27888, 4868,
                33476, 50280,
            },
            {
                65096, 46804, 15904, 35796, 8852,  800,   22352, 54468, 10004,
                47920, 15080, 51988, 62268, 55048, 5316,  40716, 1164,  43164,
                48396, 15152, 1764,  59884, 12524, 42572, 2828,  17160, 35232,
                50016, 1028,  62328, 29228, 36472, 60316, 41596, 9176,  45800,
                20156, 27164, 15940, 40928, 53512, 30820, 57388, 37516, 11168,
                19012, 34840, 41256, 29224, 1076,  54048, 31496, 41808, 56708,
                26292, 20704, 35760, 55288, 48884, 17556, 12628, 40100, 7684,
                14952, 2876,  21728, 52324, 18420, 39312, 62524, 15000, 44620,
                6836,  13296, 18528, 58264, 5120,  21408, 14136, 780,   24392,
                58752, 3496,  46868, 21648, 15876, 52716, 28756, 13472, 35904,
                5292,  18252, 2236,  15716, 35476, 21692, 33008, 24852, 37100,
                16260, 1716,  63216, 26468, 48640, 9068,  64936, 2008,  26300,
                47380, 39528, 1472,  16300, 22852, 59556, 44152, 3408,  40844,
                56332, 23080, 59180, 6508,  54896, 2000,  48364, 23368, 59684,
                41268, 9684,
            },
            {
                31924, 12124, 42352, 61452, 57504, 44936, 25792, 60696, 4420,
                64280, 23260, 44328, 28628, 37816, 48920, 16360, 61440, 10696,
                25916, 56012, 38784, 18904, 36100, 62844, 25760, 53100, 10976,
                23008, 14428, 43004, 55492, 3924,  13044, 49560, 23776, 58508,
                54632, 36740, 50604, 9848,  25008, 4528,  49604, 27388, 43008,
                55852, 6884,  49940, 61136, 13584, 36020, 45784, 8440,  61372,
                47724, 30176, 7040,  23560, 4020,  29060, 34312, 22496, 64748,
                50192, 41388, 30572, 44332, 8728,  47464, 5664,  57432, 51660,
                41384, 64856, 32784, 25312, 48424, 35208, 29164, 61500, 37212,
                16560, 40564, 26232, 60624, 38412, 940,   23408, 8856,  46268,
                63908, 51200, 26020, 47520, 8092,  56620, 63300, 6340,  46472,
                58220, 20988, 39812, 43852, 3968,  29540, 57788, 37112, 14000,
                55180, 9456,  50072, 31604, 55984, 37836, 25868, 33500, 51872,
                17620, 49360, 31244, 13076, 26688, 39924, 11180, 44432, 14648,
                19780, 56628,
            },
            {
                23040, 54940, 2940,  19300, 32844, 51196, 14448, 39912, 34828,
                30708, 41328, 3424,  18464, 9860,  22148, 57152, 31764, 36724,
                20568, 64432, 30660, 5972,  50284, 8724,  56276, 40228, 30952,
                64668, 7560,  25876, 47000, 20740, 34140, 30644, 15444, 39500,
                88,    5520,  64356, 46444, 33948, 12556, 61444, 15432, 2692,
                65048, 22688, 16664, 4452,  56396, 19752, 25088, 3144,  17980,
                38588, 50980, 65204, 41460, 57860, 52368, 46284, 820,   36688,
                25752, 58296, 4672,  13408, 64412, 23856, 32140, 20480, 2936,
                22912, 28540, 54672, 2392,  11536, 53352, 42092, 9640,  50652,
                57064, 12900, 6076,  31036, 44040, 35076, 49040, 59632, 30424,
                42204, 10888, 39180, 59372, 42800, 12412, 44,    40892, 53564,
                9912,  50112, 31632, 13524, 17964, 51052, 21972, 42100, 6144,
                28992, 60544, 42696, 12656, 5136,  18668, 48008, 64236, 12308,
                28784, 8932,  62484, 46840, 21948, 64124, 34116, 52572, 30428,
                1740,  37232,
            },
            {
                5252,  49880, 27708, 38488, 6748,  10768, 29300, 18096, 57896,
                8044,  12732, 55528, 59620, 45556, 6464,  27696, 51324, 13872,
                47252, 3876,  53532, 43928, 23836, 33892, 15492, 45336, 5036,
                48352, 37680, 17708, 59340, 9736,  63488, 51704, 6648,  61084,
                32228, 21716, 29240, 17776, 59412, 44080, 21200, 36292, 31296,
                52468, 39728, 33568, 27544, 42908, 10224, 63220, 52896, 33808,
                11732, 1180,  15436, 32576, 19548, 8412,  61400, 16032, 54880,
                11332, 19380, 34728, 53592, 27520, 37364, 55328, 49056, 12316,
                38528, 9100,  45672, 16432, 39016, 62776, 3696,  31700, 22344,
                44456, 34648, 65056, 53640, 9868,  18704, 55172, 4164,  15140,
                21128, 34040, 5872,  19208, 28132, 52672, 31208, 18032, 26948,
                34460, 6872,  24224, 60320, 38248, 62104, 10540, 34120, 52228,
                16796, 24004, 35996, 46296, 62512, 52680, 8036,  22132, 2380,
                43224, 36648, 4720,  41516, 15460, 56480, 3800,  8344,  58696,
                26172, 45928,
            },
            {
                40656, 63080, 13928, 21952, 47516, 64792, 53796, 2192,  47084,
                21436, 50212, 32192, 25124, 35304, 64720, 41456, 2528,  59748,
                8336,  39960, 16088, 27776, 11380, 59044, 208,   28648, 19656,
                56832, 33296, 52924, 40368, 1332,  44692, 27608, 18216, 43492,
                47984, 13380, 56424, 7688,  38336, 54552, 6404,  47828, 9272,
                25508, 12396, 46584, 59920, 50884, 30540, 37792, 47100, 23088,
                43120, 58776, 26016, 45096, 12304, 37796, 26880, 31580, 43252,
                6084,  47164, 60592, 40640, 192,   10220, 16656, 43364, 63324,
                33828, 60080, 50892, 20684, 57528, 26592, 18772, 46592, 7472,
                28056, 1756,  20864, 47564, 14324, 25328, 62668, 39432, 27268,
                52232, 56824, 24496, 64540, 36828, 49216, 22712, 60620, 14308,
                65156, 42364, 54804, 972,   46220, 5304,  25256, 45008, 63448,
                332,   57164, 20304, 3532,  26412, 30008, 14500, 39000, 57264,
                24868, 53296, 19736, 33152, 50356, 29276, 18572, 38188, 49572,
                62048, 16412,
            },
            {
                10016, 30016, 52648, 59360, 43032, 24148, 36168, 41856, 26556,
                63328, 37292, 15540, 152,   53060, 11496, 17532, 23800, 43536,
                35024, 21780, 57976, 46116, 63816, 37892, 22076, 52248, 60584,
                11920, 3092,  27236, 13500, 22840, 35740, 57376, 11272, 24620,
                53232, 35404, 41632, 23700, 1172,  27884, 16764, 41764, 60384,
                55216, 19308, 276,   7904,  21788, 14344, 5536,  57416, 16296,
                29136, 53760, 6248,  62208, 49684, 2348,  59280, 20956, 51520,
                63500, 29428, 14636, 21924, 45940, 58544, 29964, 4332,  25668,
                15260, 1048,  30284, 7320,  43540, 34100, 14412, 52100, 60144,
                55636, 17592, 41180, 29408, 58416, 2580,  32780, 45576, 7708,
                60700, 1020,  43856, 13956, 3508,  9668,  45164, 38992, 2712,
                47276, 19216, 11468, 27964, 35416, 56004, 14608, 19004, 30852,
                39352, 48568, 8892,  33204, 41408, 58524, 44904, 34784, 61764,
                15952, 46312, 55928, 408,   10052, 61340, 24484, 43080, 12236,
                21252, 33024,
            },
            {
                55236, 7372,  18224, 34440, 464,   5788,  16080, 56144, 11284,
                3676,  43668, 60736, 19744, 39032, 49300, 33740, 54768, 28692,
                62536, 10396, 32668, 2216,  18356, 49252, 41320, 7884,  35316,
                24520, 43552, 64048, 31872, 47844, 54164, 4524,  38352, 65492,
                3624,  9312,  62096, 48688, 32456, 51776, 64128, 22992, 5156,
                34592, 43760, 57532, 37160, 62348, 41312, 26516, 1872,  64028,
                9196,  20188, 40244, 35080, 23732, 54148, 41832, 10068, 3664,
                24528, 37656, 8156,  33344, 51316, 18752, 61752, 35512, 53240,
                40152, 56416, 23792, 36876, 49080, 4644,  64524, 24424, 38160,
                10660, 49416, 35900, 5048,  51436, 42980, 21796, 12272, 17576,
                36644, 48196, 31884, 54544, 17020, 57848, 33756, 5712,  56100,
                29556, 37036, 51384, 59112, 20960, 31868, 49932, 59516, 7680,
                27092, 12880, 65312, 54704, 17024, 10608, 868,   50460, 5500,
                31424, 11408, 64972, 27216, 39480, 45732, 6136,  54436, 35564,
                1024,  47948,
            },
            {
                44128, 25020, 39688, 12472, 51632, 30816, 61476, 20080, 33268,
                52476, 29388, 5772,  46504, 26704, 7452,  58332, 14388, 5720,
                48852, 52076, 25400, 54968, 5352,  31088, 13800, 62956, 46728,
                16440, 50880, 5692,  58092, 16800, 8472,  20268, 42388, 30384,
                16252, 27096, 19824, 58652, 14284, 10656, 39144, 30620, 50064,
                14720, 27716, 52068, 31772, 17740, 49112, 55196, 44876, 32440,
                36928, 48012, 3120,  30332, 17440, 13848, 46700, 33872, 39680,
                17628, 48924, 57584, 2404,  42460, 24652, 6436,  47320, 9480,
                19908, 45264, 62188, 12972, 58740, 10132, 40888, 12,    32700,
                15500, 63236, 23196, 60912, 9248,  37508, 54304, 64816, 50392,
                28608, 10632, 20096, 39664, 29444, 62680, 25844, 20768, 52272,
                9160,  23772, 16192, 3840,  43320, 8900,  40792, 2844,  54136,
                36248, 42960, 21868, 50780, 37376, 24116, 60392, 20756, 27952,
                40808, 7352,  22972, 36140, 13704, 59256, 31180, 15708, 64684,
                28008, 57944,
            },
            {
                37120, 2544,  64044, 56868, 26932, 44536, 49420, 7172,  40308,
                58904, 23348, 14104, 55656, 62752, 30760, 21544, 45308, 988,
                19132, 39260, 14860, 42956, 36136, 57076, 26544, 2764,  29676,
                39088, 10000, 21620, 41244, 29064, 61468, 50168, 33460, 58976,
                51552, 46192, 2600,  36836, 44464, 4416,  56204, 18840, 1628,
                62008, 9656,  23804, 4480,  11804, 34892, 7408,  22724, 13088,
                51820, 60560, 11352, 55812, 64404, 7152,  28024, 57032, 62380,
                12172, 53836, 27220, 64252, 11104, 38168, 55024, 13844, 28352,
                50364, 3476,  17548, 27684, 32104, 22068, 54960, 29700, 43304,
                6276,  45704, 27304, 13068, 31316, 16544, 26164, 5952,  41176,
                23032, 4088,  61760, 46200, 7148,  50276, 42448, 15160, 60100,
                32384, 40432, 63844, 48684, 13612, 62600, 26416, 22736, 47024,
                61212, 15812, 2368,  28544, 6728,  47292, 32736, 55584, 17780,
                49168, 44236, 52256, 57448, 20624, 3020,  50952, 23172, 40960,
                4780,  19720,
            },
            {
                14868, 31688, 47224, 21068, 9364,  37344, 13392, 25212, 46848,
                17000, 8732,  32496, 42384, 2228,  10548, 41200, 60804, 36840,
                31236, 65496, 7288,  21148, 61320, 9672,  44928, 53136, 19260,
                59896, 55420, 36448, 2412,  45700, 25520, 14292, 640,   22896,
                12328, 39892, 55228, 29012, 22112, 62696, 26132, 47092, 35720,
                40452, 58648, 47744, 42704, 65504, 20372, 29292, 58368, 40908,
                16960, 27192, 22064, 42772, 49000, 36824, 19828, 896,   31144,
                43516, 5984,  20100, 30616, 15900, 60232, 32252, 22676, 65500,
                41944, 33392, 6264,  39408, 51896, 46140, 18288, 59668, 50184,
                20400, 57100, 2492,  40404, 55520, 49096, 1140,  34968, 58892,
                55632, 33468, 52756, 12792, 23900, 2356,  36044, 11092, 44556,
                1184,  56828, 6908,  30508, 38016, 53180, 34084, 11844, 18316,
                4596,  31820, 57044, 44648, 61648, 14064, 39956, 3932,  63612,
                12948, 33880, 1612,  16724, 42556, 47508, 34160, 7764,  52852,
                11316, 60216,
            },
            {
                23696, 53912, 6244,  41188, 17268, 62880, 57684, 4188,  35216,
                65136, 51424, 37968, 49548, 24676, 35636, 16368, 48308, 26580,
                56788, 11592, 46524, 28196, 49960, 17068, 23656, 34552, 6756,
                48708, 14696, 32644, 62684, 11584, 56260, 37920, 43952, 7612,
                63224, 34588, 6124,  17192, 50676, 33488, 8068,  42856, 12636,
                21376, 29544, 16384, 2268,  53740, 39076, 50268, 4784,  62428,
                404,   45976, 8060,  33408, 4028,  24616, 60828, 50772, 14928,
                23476, 36332, 47388, 41104, 49928, 4824,  44892, 684,   37148,
                57784, 12192, 54084, 63432, 1956,  14588, 7792,  26456, 11664,
                37556, 52844, 33480, 18808, 44460, 63004, 20748, 46924, 14572,
                8536,  43472, 26604, 37612, 57260, 19168, 64624, 28108, 49336,
                18296, 25540, 46252, 22224, 17632, 584,   45116, 57596, 40140,
                52040, 24880, 35352, 11136, 19312, 51640, 26208, 8684,  36288,
                53964, 24380, 29620, 63044, 9716,  28528, 60800, 18552, 44732,
                30720, 49352,
            },
            {
                8888,  35200, 60788, 28220, 2772,  32164, 22008, 53860, 28704,
                928,   21204, 12040, 57624, 18940, 64176, 54224, 8204,  22620,
                3332,  52420, 33928, 40384, 1240,  58600, 12680, 41900, 64904,
                532,   27804, 23872, 52520, 19724, 6400,  30036, 53624, 48004,
                18712, 24944, 61052, 41768, 11824, 58124, 15892, 53388, 64468,
                5832,  55504, 37484, 26336, 60036, 14836, 9416,  36108, 25304,
                31636, 56748, 38616, 59900, 15820, 53340, 40588, 8624,  45716,
                56048, 3524,  61432, 8904,  56812, 26844, 17888, 52556, 8528,
                20700, 26388, 47704, 23176, 42788, 37188, 61772, 34352, 3764,
                65140, 24680, 8924,  59932, 6608,  28444, 11472, 39372, 30580,
                18568, 63132, 364,   16132, 41620, 31168, 53936, 7772,  38444,
                61536, 33728, 11528, 55008, 60372, 28556, 6052,  64320, 29880,
                9200,  48868, 63676, 41820, 1176,  30756, 57600, 43144, 21772,
                46412, 58512, 6064,  38524, 55168, 25060, 12988, 37476, 62816,
                26452, 40068,
            },
            {
                3308,  50564, 13788, 55996, 45980, 51028, 42432, 11060, 39560,
                45380, 60356, 27872, 6664,  44084, 4152,  28964, 13532, 59676,
                43328, 17488, 23948, 62408, 5496,  37396, 54020, 31596, 21044,
                38512, 43648, 7788,  47784, 40752, 35544, 64548, 15692, 27336,
                57556, 9600,  31080, 932,   47780, 38000, 3572,  31308, 24872,
                49704, 34016, 8636,  46180, 31488, 23300, 44144, 18588, 47712,
                53224, 13900, 19648, 29000, 44612, 12044, 32408, 27700, 65068,
                18160, 33140, 25596, 14180, 21872, 35116, 40268, 46788, 29972,
                15732, 60088, 35380, 9952,  19264, 28640, 55608, 47104, 41484,
                16632, 29248, 48388, 14832, 36076, 23780, 53544, 60964, 3816,
                22428, 47792, 51372, 9536,  60448, 47240, 3552,  23264, 14188,
                52664, 4380,  42652, 35756, 9060,  50836, 24356, 14528, 37004,
                21184, 5280,  15168, 53792, 22744, 38472, 62364, 16792, 2436,
                10896, 32160, 19416, 49752, 3448,  41672, 53408, 6260,  196,
                57480, 16760,
            },
            {
                65112, 42804, 19360, 33760, 24552, 8240,  15416, 59596, 18592,
                5328,  34076, 14912, 53196, 31876, 47356, 40412, 34332, 50424,
                38348, 6904,  55220, 14404, 30168, 47700, 25700, 9244,  57336,
                51684, 12228, 59136, 17328, 3920,  58200, 22160, 2904,  41480,
                37144, 51832, 44756, 54324, 23580, 27940, 60872, 19892, 45180,
                2696,  18304, 63028, 12492, 50752, 1376,  55664, 63628, 11212,
                41924, 5456,  64752, 50292, 1412,  58392, 22128, 4432,  37500,
                11396, 50320, 39564, 52884, 2884,  59424, 11448, 62876, 55776,
                2160,  43512, 6588,  31408, 58688, 49924, 6444,  13172, 21596,
                51840, 1116,  39584, 57424, 43548, 2044,  50640, 32628, 41876,
                56712, 36364, 28292, 33136, 20952, 12200, 35264, 59172, 41224,
                29504, 20968, 65452, 15720, 48132, 39864, 19292, 43976, 55340,
                60504, 45984, 27748, 33088, 7804,  47144, 12600, 29108, 49304,
                65188, 41284, 15312, 59648, 34960, 22084, 16096, 48676, 33436,
                20724, 46272,
            },
            {
                29616, 22904, 11124, 39244, 588,   64164, 36492, 30568, 49060,
                55660, 25356, 63640, 38720, 21988, 11276, 61556, 436,   20488,
                27960, 64376, 32732, 44212, 20660, 59612, 17840, 3284,  46348,
                16076, 30564, 63660, 33660, 25128, 10320, 49884, 32952, 12876,
                5640,  21260, 14556, 64968, 18040, 6768,  41080, 51560, 10520,
                38788, 57112, 41896, 20936, 6256,  39768, 33556, 28300, 3516,
                21764, 35252, 24740, 9708,  37104, 18412, 55016, 48520, 42324,
                59840, 380,   29944, 63932, 44368, 28792, 6876,  24332, 32764,
                50868, 38556, 64300, 53156, 16408, 552,   24820, 32468, 58184,
                44280, 62160, 11056, 31564, 19520, 64656, 16172, 9972,  25348,
                6912,  13188, 64476, 4912,  55716, 43884, 26940, 16784, 48716,
                5992,  58312, 1464,  26092, 31216, 57144, 2056,  33580, 12132,
                3556,  40600, 18104, 59664, 55736, 4368,  35164, 53048, 25612,
                37052, 7856,  45484, 27208, 11500, 64396, 43708, 28844, 55384,
                9928,  36536,
            },
            {
                52284, 4360,  61748, 57236, 47300, 27396, 53428, 3488,  23484,
                7864,  42912, 48028, 2428,  17604, 58028, 24336, 53684, 15280,
                9768,  48488, 2060,  11172, 52008, 7588,  42528, 62764, 36040,
                26680, 5404,  37600, 45412, 53040, 29084, 42928, 61908, 46492,
                59600, 29440, 2400,  35896, 32304, 55820, 13272, 34948, 59460,
                26212, 15184, 29008, 53600, 61740, 25672, 58436, 16512, 49192,
                59512, 30988, 46760, 40976, 61492, 26860, 34480, 7620,  15752,
                24572, 19976, 46876, 10196, 17116, 37740, 49220, 14480, 19072,
                4748,  21848, 12804, 25608, 41112, 45388, 63724, 38900, 18164,
                5300,  35140, 23000, 53840, 4948,  27480, 36832, 46416, 59508,
                50036, 17752, 45072, 24280, 40012, 1912,  52048, 63476, 8984,
                32580, 44436, 51304, 38196, 12960, 7188,  62780, 52708, 26444,
                30740, 51420, 36064, 10368, 24788, 43924, 20228, 60636, 6452,
                56656, 23288, 52144, 860,   31464, 57016, 8508,  39064, 23828,
                14332, 59272,
            },
            {
                26200, 7836,  43836, 30828, 17876, 12344, 20868, 40532, 61852,
                13656, 35916, 10236, 29348, 51504, 7580,  42472, 37060, 31344,
                56940, 41144, 25388, 60824, 38792, 27844, 33084, 50632, 10736,
                22568, 55280, 1908,  19552, 14112, 57052, 1152,  17028, 25912,
                55308, 39868, 50828, 43272, 9232,  46536, 62888, 22420, 116,
                48944, 33040, 4392,  44568, 35524, 13768, 8272,  43524, 37592,
                54860, 7340,  15896, 52384, 4936,  13436, 45236, 62884, 30404,
                53380, 57108, 7104,  33884, 21548, 57716, 53976, 42056, 34668,
                46148, 57312, 29236, 48792, 7988,  34296, 11520, 53576, 30076,
                9072,  26788, 46084, 13552, 49256, 41036, 55204, 2888,  20736,
                30100, 38756, 53432, 9116,  61520, 30640, 19508, 37676, 24692,
                11728, 55808, 17696, 22856, 61132, 42344, 20860, 47328, 8208,
                64924, 22300, 176,   48116, 62672, 13896, 40820, 1544,  17448,
                31008, 12896, 39384, 61932, 47004, 19884, 3940,  51644, 63264,
                1644,  41072,
            },
            {
                34712, 16336, 54444, 37408, 5932,  60856, 51476, 44788, 32584,
                57820, 20012, 54856, 62768, 33340, 45444, 26872, 63772, 6088,
                45900, 17940, 34752, 22256, 54872, 14992, 964,   19696, 40296,
                60472, 48752, 41864, 62192, 8308,  39216, 21556, 36452, 7144,
                10584, 18924, 23148, 60168, 26384, 3852,  16684, 29328, 43908,
                8516,  63884, 22976, 10796, 47816, 19540, 52056, 23160, 752,
                12540, 62592, 20316, 29400, 57316, 23540, 51008, 1768,  39136,
                12260, 36152, 43124, 62252, 26252, 1324,  5680,  64864, 27188,
                9632,  62112, 3132,  36500, 19772, 61384, 23212, 4040,  50344,
                60656, 55892, 38408, 63560, 7892,  22156, 33452, 11848, 43012,
                62676, 1100,  34248, 15984, 48956, 13336, 6536,  57580, 47500,
                40672, 27752, 34424, 3792,  46508, 28584, 35556, 16904, 38380,
                14164, 42020, 58224, 16980, 31640, 27016, 51104, 33492, 47912,
                44656, 58924, 20716, 6080,  16532, 36972, 26724, 32460, 45372,
                18860, 49456,
            },
            {
                57000, 22196, 13200, 49088, 24184, 34576, 9492,  1948,  16720,
                27156, 3880,  41192, 23808, 15744, 1428,  11720, 19304, 50572,
                2836,  61916, 13284, 49528, 4172,  45712, 65124, 56560, 29612,
                13744, 7108,  28240, 23928, 51252, 31620, 65328, 48876, 52800,
                32080, 63388, 5132,  12984, 37536, 50332, 53948, 39320, 57844,
                19252, 52712, 37236, 60292, 2012,  57632, 29872, 64600, 32956,
                26072, 44500, 38776, 2900,  33812, 41712, 10372, 21112, 59016,
                27784, 5228,  51396, 13420, 40676, 47680, 30808, 12508, 52132,
                38856, 16968, 42752, 54456, 14840, 56580, 27668, 47752, 15660,
                41416, 20212, 1348,  17128, 29296, 57604, 61248, 15284, 26408,
                6172,  56952, 21932, 27672, 59464, 36572, 44792, 22056, 52,
                14764, 62744, 7492,  58676, 52208, 10852, 2656,  59616, 55652,
                25440, 45128, 6668,  53660, 38932, 8088,  64052, 22604, 55604,
                9752,  27912, 34820, 42864, 54764, 49092, 13724, 59796, 6776,
                11928, 30084,
            },
            {
                3884,  46376, 64556, 1440,  42196, 59000, 29284, 38548, 65468,
                50764, 45864, 6512,  37832, 60136, 48736, 35728, 59228, 23020,
                29144, 38448, 8136,  58496, 31100, 36280, 23632, 5752,  44380,
                33528, 18004, 53908, 35220, 11860, 44780, 4092,  27352, 14484,
                44124, 34772, 47168, 57756, 31096, 20320, 10284, 33904, 5624,
                14076, 41400, 30736, 16580, 26504, 39056, 5936,  41752, 17808,
                48036, 53476, 9420,  60000, 48740, 17540, 63740, 31760, 44072,
                47924, 18372, 23036, 32320, 60612, 16700, 37064, 20536, 59856,
                1864,  22704, 32180, 50992, 6104,  39344, 1776,  43240, 35584,
                12168, 25492, 32696, 51720, 43444, 4516,  39296, 48604, 52536,
                36192, 46908, 10880, 42108, 3492,  55028, 32924, 64740, 29560,
                53968, 49004, 20432, 40004, 15828, 31976, 23608, 48280, 5100,
                33320, 11532, 28752, 21492, 2716,  57628, 12012, 36872, 5172,
                15868, 61160, 2476,  24468, 62920, 9080,  22404, 56000, 42668,
                37252, 62136,
            },
            {
                9644,  39676, 27472, 32112, 19740, 53896, 14776, 47608, 22240,
                11048, 31476, 53308, 12912, 20824, 55880, 30536, 40980, 14520,
                55532, 43588, 26612, 20396, 10684, 41668, 16424, 52828, 9620,
                38332, 58148, 46660, 5236,  16572, 59156, 19452, 41648, 60936,
                516,   25004, 17188, 41160, 2932,  64532, 45460, 24600, 61564,
                27280, 50512, 3400,  55724, 46436, 33924, 50272, 10648, 56180,
                3848,  35668, 22460, 14072, 27080, 55960, 37048, 7964,  16056,
                3720,  65528, 55760, 2640,  9288,  54352, 24596, 6996,  44844,
                48592, 26524, 58712, 11132, 29896, 21572, 59388, 31576, 65532,
                6968,  57840, 47448, 36396, 10564, 24216, 19396, 31356, 8320,
                18128, 65232, 30884, 51532, 19148, 25380, 8772,  17180, 42828,
                5168,  35852, 10028, 26108, 44552, 64576, 53344, 40932, 19564,
                62424, 49500, 60424, 34688, 46556, 41724, 18656, 29652, 43204,
                50360, 38816, 53028, 12128, 29768, 40340, 34200, 2112,  27956,
                20252, 51792,
            },
            {
                33876, 58780, 17432, 11144, 49888, 4272,  26068, 7204,  56948,
                36444, 18272, 62468, 27808, 43088, 8556,  4764,  51848, 9964,
                33456, 64324, 5604,  53828, 47208, 62308, 27492, 49100, 60916,
                21456, 324,   64080, 26144, 37992, 55248, 30496, 8952,  22384,
                57200, 53280, 7808,  49376, 27792, 14468, 56296, 1328,  48416,
                35480, 11716, 65324, 21020, 8468,  14356, 61792, 22000, 28332,
                15272, 63976, 30948, 43440, 184,   5904,  24156, 53704, 49864,
                34224, 26084, 37928, 45348, 29200, 49680, 41700, 33116, 55848,
                13860, 35672, 4180,  44252, 63808, 47008, 8812,  18016, 53104,
                22640, 3056,  61896, 14692, 54480, 63032, 444,   44784, 59564,
                13248, 24340, 4812,  38504, 62004, 48044, 12432, 52180, 38812,
                23844, 61172, 31164, 55544, 1400,  36416, 13696, 8496,  30500,
                1064,  37960, 17692, 51864, 14408, 24664, 54388, 61824, 492,
                23680, 19204, 32472, 46380, 3472,  51616, 17344, 65332, 48260,
                14820, 24900,
            },
            {
                45608, 5760,  43460, 55472, 62964, 35396, 40148, 61292, 43196,
                1692,  24928, 47928, 3352,  34244, 64688, 25776, 18728, 46964,
                200,   17036, 49972, 37096, 14256, 1488,  34496, 4592,  25376,
                13060, 32000, 42112, 10488, 50116, 1832,  47424, 13096, 39576,
                35932, 29404, 11880, 61948, 21636, 39976, 32392, 18012, 42160,
                22108, 7128,  44716, 39672, 25048, 53516, 1600,  37308, 46064,
                58044, 7180,  39232, 51588, 60692, 46608, 40768, 29520, 11808,
                61148, 41996, 14420, 19996, 58588, 12076, 316,   61232, 17916,
                8940,  62368, 40160, 16012, 24916, 35036, 13468, 50176, 27052,
                44884, 39996, 30196, 7640,  27444, 49988, 34664, 56064, 28216,
                41316, 54108, 45924, 15132, 936,   35008, 28812, 60352, 2532,
                46024, 13072, 17868, 49784, 6632,  21412, 58388, 26912, 44196,
                54532, 24028, 9428,  4652,  65464, 32076, 6920,  48636, 35368,
                56800, 8656,  64172, 15412, 58084, 25828, 43824, 7600,  38744,
                53996, 312,
            },
            {
                61724, 21792, 37444, 7820,  23456, 45768, 12476, 17272, 32480,
                54376, 9292,  39940, 58328, 16160, 45664, 38728, 54536, 60728,
                24244, 40504, 31660, 21968, 59284, 29736, 18832, 39696, 57168,
                45692, 51816, 29172, 20140, 61072, 23524, 33484, 63904, 50808,
                3788,  19120, 43528, 54800, 36888, 5348,  51568, 8784,  60436,
                53700, 31380, 58172, 17716, 29244, 59640, 43480, 32004, 19084,
                49696, 24936, 12144, 20380, 33216, 15356, 19328, 56616, 1448,
                22248, 8572,  52940, 5644,  34624, 64184, 22312, 27108, 38100,
                30220, 50400, 20232, 56704, 6584,  54380, 41964, 8,     37352,
                11296, 56216, 20608, 46184, 17660, 38592, 12008, 22036, 3548,
                36936, 7436,  32664, 57836, 22888, 44032, 56376, 7164,  21076,
                33444, 57516, 41148, 63312, 29116, 39072, 50756, 16944, 63768,
                12624, 42224, 57984, 28152, 44312, 38580, 10892, 26904, 13720,
                44484, 28880, 41500, 5896,  36272, 20764, 60192, 31804, 10616,
                57544, 29104,
            },
            {
                12884, 52808, 30964, 15136, 28016, 844,   52508, 21324, 28476,
                60076, 14264, 51156, 30184, 23052, 11568, 2148,  28712, 35976,
                12592, 57692, 3616,  45520, 10124, 54620, 43336, 64628, 7776,
                16956, 55376, 3192,  15056, 35660, 43912, 6576,  27124, 15560,
                59960, 46756, 32868, 1532,  24364, 57696, 45040, 12996, 28212,
                37780, 15380, 412,   51500, 35872, 11992, 4884,  63524, 9300,
                41032, 2460,  61864, 55588, 28200, 9704,  64472, 35664, 45468,
                58208, 32056, 48188, 28140, 38808, 16144, 43500, 47588, 3784,
                53768, 23712, 1232,  32776, 48536, 28036, 19660, 62852, 58952,
                16140, 33996, 3948,  60652, 42012, 5696,  64408, 15912, 48512,
                61700, 20324, 26908, 63940, 10076, 40548, 18444, 49312, 27296,
                53528, 8716,  3184,  24932, 45064, 10644, 34148, 5860,  47032,
                32232, 36132, 20820, 49032, 1500,  17936, 58548, 21264, 52396,
                3528,  59700, 22272, 54168, 49264, 12744, 4424,  46984, 23336,
                35800, 18044,
            },
            {
                4488,  40612, 47012, 61028, 57004, 33588, 64780, 48820, 4168,
                45208, 35284, 20204, 7248,  56456, 61288, 50388, 21152, 43776,
                8024,  51612, 26344, 63620, 6424,  24072, 50436, 12196, 36228,
                22700, 33124, 63116, 40620, 8720,  58000, 53508, 20848, 41908,
                9388,  25472, 64820, 16680, 10152, 30020, 19748, 63632, 2964,
                23976, 46500, 62824, 6312,  41476, 48892, 23604, 16660, 27408,
                52820, 35160, 44848, 6448,  37768, 50296, 4036,  25628, 13708,
                39316, 17252, 63460, 2976,  55476, 50956, 6636,  57524, 11152,
                41336, 64932, 45428, 12664, 61044, 38988, 4836,  31904, 24000,
                8200,  48256, 53816, 23344, 31808, 52416, 25712, 33304, 44448,
                10780, 52236, 17100, 49912, 5780,  30288, 13560, 36336, 64488,
                15824, 37828, 48112, 19960, 14660, 59708, 55232, 22884, 2684,
                60460, 7308,  15620, 53900, 62236, 34872, 47272, 40180, 63048,
                16976, 33284, 10496, 26800, 39612, 30612, 55580, 16288, 62700,
                42996, 49668,
            },
            {
                64284, 25208, 2300,  18768, 10280, 42820, 6216,  37696, 11188,
                24416, 63076, 604,   42700, 36788, 31996, 5700,  15008, 63148,
                30452, 18380, 35364, 41428, 16456, 37864, 31532, 1904,  27692,
                44360, 5508,  50692, 25240, 47764, 17928, 2304,  31728, 55516,
                36652, 5200,  52308, 39648, 49124, 59816, 41708, 34008, 50048,
                55968, 10948, 33112, 26864, 20508, 54472, 34220, 45488, 57396,
                13352, 30676, 21532, 17336, 47332, 59496, 42492, 30912, 51508,
                4708,  24648, 42976, 21180, 9996,  25532, 31044, 19448, 34024,
                15196, 28436, 7424,  36468, 17508, 10084, 57356, 51056, 44256,
                28620, 64652, 36768, 13784, 9888,  58844, 2520,  39708, 56652,
                29804, 1388,  42476, 34292, 55296, 46144, 61048, 1576,  24088,
                43920, 29492, 61352, 32608, 52292, 712,   40284, 29684, 18976,
                52424, 25052, 40440, 30408, 10160, 24324, 3076,  29940, 6604,
                37032, 50148, 45820, 716,   17948, 61120, 41884, 2508,  27112,
                7032,  32532,
            },
            {
                20792, 55988, 35084, 50528, 39100, 25632, 16108, 30316, 59368,
                40876, 17088, 53504, 48128, 25452, 18056, 46664, 40728, 58716,
                2612,  48140, 56904, 13092, 52948, 60832, 20620, 58344, 48692,
                61608, 10748, 37772, 13608, 60048, 29004, 38328, 45824, 12692,
                58788, 29596, 20632, 13772, 35292, 5920,  26100, 15860, 6992,
                39608, 18176, 43608, 57196, 14080, 8276,  60920, 1444,  37996,
                4612,  65224, 54364, 26640, 1032,  14736, 21940, 8416,  54740,
                61484, 11524, 36176, 59568, 46368, 13944, 62636, 37296, 60532,
                49512, 21448, 58796, 52660, 25516, 42852, 22116, 14508, 3008,
                40256, 18636, 1068,  27028, 43220, 49204, 18440, 8836,  22940,
                60776, 14056, 38136, 21720, 3620,  25512, 39084, 58552, 10544,
                4924,  50644, 12340, 7232,  36664, 26088, 62848, 43716, 49260,
                37460, 11220, 56880, 5184,  43140, 51844, 14848, 55260, 42564,
                24700, 13104, 56240, 64948, 34864, 9012,  51760, 22092, 38372,
                53644, 10064,
            },
            {
                44396, 29040, 12532, 7100,  59872, 54392, 20376, 50960, 55788,
                8028,  32960, 27936, 12696, 65248, 9408,  52636, 23720, 37844,
                10924, 21736, 33016, 3956,  28548, 43808, 5064,  15072, 40884,
                18928, 56680, 30728, 21676, 3916,  52448, 10308, 63316, 23480,
                92,    48056, 42952, 62840, 56448, 1884,  47592, 54040, 22252,
                59436, 30600, 49452, 2100,  64592, 40028, 28460, 19412, 50432,
                24544, 41892, 10032, 39096, 61276, 34360, 56936, 37404, 46256,
                19896, 29948, 50128, 660,   33256, 40484, 52560, 2084,  9048,
                44184, 4504,  39788, 31300, 1596,  63468, 46812, 33852, 55884,
                11940, 52000, 57608, 47048, 21276, 62820, 28788, 54496, 36088,
                6196,  47184, 57508, 63036, 12052, 16820, 52028, 32704, 20048,
                54924, 40480, 22956, 57808, 45880, 17728, 10116, 4464,  14132,
                64884, 27360, 45384, 61192, 21876, 33700, 64036, 19580, 8972,
                60852, 30932, 20104, 4900,  23996, 48296, 28872, 11908, 46356,
                59076, 15316,
            },
            {
                39504, 60572, 48216, 23068, 31332, 44120, 2500,  13260, 46020,
                22412, 3740,  58612, 38688, 2868,  56172, 33952, 4872,  27812,
                53876, 45076, 61996, 25136, 9360,  47492, 34708, 54404, 8448,
                25980, 652,   46124, 64928, 34776, 42296, 26660, 16480, 50668,
                33776, 17820, 7240,  27820, 22808, 32200, 12512, 65104, 36208,
                9740,  4552,  25392, 37012, 22812, 47060, 31736, 10960, 59852,
                15856, 48360, 6760,  31708, 51700, 11420, 28148, 1988,  16104,
                40864, 7000,  57124, 26796, 17972, 5892,  22024, 28176, 55412,
                25044, 18324, 56320, 13888, 49976, 28864, 5308,  37664, 60252,
                25252, 30956, 7084,  34256, 38140, 4664,  14668, 41228, 50492,
                26512, 19320, 32184, 28464, 49188, 42180, 6880,  27420, 47180,
                35144, 17168, 63828, 3428,  28280, 56052, 21624, 35400, 54156,
                6732,  33232, 18676, 224,   37636, 12536, 48476, 28520, 45496,
                1556,  53012, 38716, 44000, 58792, 14880, 37124, 63668, 18632,
                34064, 636,
            },
            {
                52148, 4128,  17372, 37248, 63092, 5432,  34888, 27040, 64196,
                36852, 49628, 44244, 16092, 21756, 29828, 44736, 13164, 63652,
                16316, 1256,  39988, 50704, 18868, 65212, 23296, 29868, 38304,
                51880, 33788, 16020, 54716, 7392,  19832, 57536, 4476,  40644,
                61012, 37728, 53960, 10680, 45548, 38880, 19052, 42616, 28536,
                45404, 61652, 53264, 11812, 16600, 51900, 6484,  44292, 55408,
                33976, 20512, 62508, 43768, 23460, 18484, 49108, 64832, 25216,
                33628, 62708, 14892, 43896, 54016, 65276, 48384, 42212, 12860,
                32908, 47256, 64360, 35456, 7868,  19916, 54512, 16436, 9188,
                20500, 44676, 61220, 16248, 10484, 58136, 31444, 284,   65460,
                11736, 44016, 2168,  9124,  54236, 23228, 64888, 14224, 56896,
                140,   30684, 43256, 14924, 39008, 47964, 61904, 30364, 42044,
                23284, 47648, 15076, 51036, 59008, 25764, 3864,  39188, 58348,
                34716, 15580, 27288, 7536,  32132, 54408, 1748,  5840,  56152,
                30440, 23820,
            },
            {
                64960, 27200, 41952, 9488,  14768, 49532, 56980, 40260, 10312,
                19104, 30472, 5964,  62224, 51176, 41096, 60124, 19420, 49340,
                35680, 30112, 7260,  55764, 37132, 11724, 1548,  59048, 13432,
                62376, 43428, 23956, 11516, 38924, 49272, 45324, 32224, 8332,
                14012, 25056, 58160, 2644,  51264, 60820, 7984,  52568, 544,
                14936, 19972, 40572, 58704, 34108, 3276,  63404, 25736, 36960,
                296,   29500, 13952, 55832, 2880,  40552, 58628, 44200, 5340,
                53056, 47216, 9340,  23204, 38044, 11356, 30460, 16648, 58884,
                38884, 3064,  10384, 23396, 45448, 41772, 61956, 32016, 48480,
                41196, 3252,  50404, 23824, 52840, 42544, 46204, 21540, 34068,
                53628, 16184, 61808, 40192, 33736, 4540,  37400, 44576, 9320,
                60952, 24816, 53144, 11012, 33640, 7760,  51280, 16404, 3564,
                59580, 55856, 9460,  31232, 41584, 7636,  56292, 17644, 10668,
                22536, 49736, 62872, 11648, 40428, 21364, 45320, 26028, 41780,
                48728, 8540,
            },
            {
                35596, 45644, 57660, 21936, 53472, 29324, 24240, 136,   60888,
                14452, 54944, 25948, 35236, 11364, 916,   24976, 8236,  55092,
                42808, 21016, 59504, 14920, 32020, 41868, 49664, 45108, 21048,
                6804,  3300,  50888, 59320, 29792, 1636,  22980, 64224, 52912,
                29456, 43560, 20272, 35608, 30216, 16212, 26316, 57988, 34612,
                48624, 32028, 6848,  27988, 42428, 56840, 18596, 13520, 41332,
                8760,  59128, 46100, 26472, 6116,  34988, 9664,  30476, 13152,
                20576, 36692, 28968, 3660,  60400, 34452, 1316,  51228, 7184,
                61728, 27100, 52824, 17108, 59212, 25932, 12180, 1284,  27860,
                65064, 36212, 12892, 29724, 63204, 26304, 17684, 9000,  59688,
                24472, 36720, 48616, 20904, 59244, 18132, 29168, 50860, 21332,
                36260, 5876,  49520, 58240, 20412, 1084,  44300, 26568, 12356,
                39732, 36428, 28252, 63892, 20136, 44940, 52072, 32176, 64764,
                41948, 2604,  46396, 18684, 50784, 60908, 35048, 16552, 61968,
                13428, 19880,
            },
            {
                54360, 12004, 1684,  33532, 46488, 7512,  17288, 43508, 32424,
                51356, 42016, 8620,  57884, 18116, 48408, 37452, 32572, 4052,
                26672, 10724, 46656, 24320, 61568, 4264,  18024, 26632, 55692,
                31776, 41232, 27088, 18564, 62660, 14816, 36480, 10884, 18964,
                47160, 5476,  62180, 12608, 48456, 41816, 3908,  22088, 10820,
                54792, 24304, 64200, 50144, 9356,  21992, 30444, 47616, 53656,
                23220, 50108, 37888, 17952, 63928, 52024, 22260, 38752, 55120,
                2272,  59312, 49504, 42640, 55816, 19336, 46012, 24128, 35616,
                20784, 43724, 30264, 37532, 6320,  50940, 39388, 53632, 22072,
                55712, 6620,  19024, 40188, 1880,  5808,  54836, 49240, 39700,
                6692,  3344,  27064, 11460, 46264, 55896, 1852,  62100, 12252,
                41576, 18512, 27704, 37152, 65456, 23876, 32524, 60604, 52780,
                19488, 2020,  46088, 5044,  24260, 36116, 14336, 27664, 6200,
                54708, 25296, 31280, 58128, 3736,  29268, 7064,  10292, 51380,
                38908, 4724,
            },
            {
                23532, 31004, 62256, 19124, 38516, 64012, 58956, 48564, 21192,
                4316,  37700, 23504, 46340, 28364, 63284, 53204, 14664, 58048,
                64760, 38864, 52104, 6292,  28656, 53400, 35928, 8640,  64276,
                16596, 36956, 47268, 8816,  33180, 43868, 55924, 26032, 59124,
                848,   39028, 55192, 23392, 7092,  64452, 37224, 46912, 62036,
                39104, 17664, 1564,  14160, 45620, 38364, 60552, 2256,  33036,
                62984, 5148,  10824, 42300, 32264, 15104, 47936, 61800, 17516,
                24856, 32712, 11740, 16004, 6344,  26608, 63684, 10088, 56160,
                14812, 48908, 244,   63068, 32676, 15508, 4388,  35060, 45312,
                14168, 59416, 32276, 48052, 57184, 37164, 32840, 12364, 29476,
                44796, 57668, 52480, 31200, 7432,  14700, 39276, 25504, 32032,
                47808, 59440, 2200,  45904, 14152, 40616, 57028, 9524,  48348,
                33912, 61436, 16100, 54476, 11552, 62388, 856,   48880, 38464,
                19816, 35776, 8800,  14688, 38028, 24192, 47688, 57460, 31988,
                27572, 59208,
            },
            {
                15324, 41128, 50824, 26288, 3748,  13364, 35324, 9916,  28040,
                56588, 64984, 2296,  33308, 13456, 5460,  40528, 23048, 45260,
                17812, 36,    34144, 19768, 43828, 14060, 58292, 39920, 48136,
                11936, 54200, 22032, 57568, 3580,  51956, 6164,  40948, 50232,
                30968, 15836, 26968, 34196, 52788, 18180, 31624, 13752, 27836,
                4344,  44424, 35992, 55276, 25624, 4492,  50948, 11636, 27596,
                16024, 20772, 57340, 28976, 54132, 7672,  600,   27848, 45356,
                40396, 64228, 52176, 22220, 38996, 31484, 52932, 43136, 40124,
                4472,  57732, 12484, 53928, 22476, 47536, 58228, 18524, 8956,
                26440, 51524, 43476, 10276, 20744, 15236, 60684, 22880, 64008,
                19384, 15696, 41236, 63384, 24248, 43568, 34832, 52872, 7812,
                55612, 15504, 30904, 10140, 54564, 5224,  17564, 29992, 22192,
                7124,  25420, 50380, 40444, 32540, 57092, 21904, 42720, 58532,
                11924, 61976, 52860, 43468, 55844, 65200, 17884, 42136, 21560,
                1156,  44832,
            },
            {
                48856, 6816,  10376, 55380, 42592, 30464, 52528, 5412,  39952,
                12604, 17460, 52952, 43020, 60312, 20148, 30552, 9192,  50496,
                27904, 12788, 57128, 48808, 63464, 1792,  32564, 23588, 5240,
                30104, 372,   61868, 13868, 38644, 28600, 17256, 21668, 35224,
                9016,  63716, 45244, 10424, 59520, 1668,  43348, 56668, 8316,
                51608, 59152, 21244, 29660, 63100, 16900, 35104, 56164, 40524,
                44864, 36384, 3340,  46440, 23416, 59752, 37076, 11200, 56928,
                4408,  8256,  28472, 46952, 60976, 13344, 2704,  17912, 28560,
                33032, 25156, 19152, 39828, 7664,  28984, 42164, 63804, 30896,
                38668, 4208,  23692, 64264, 28632, 52540, 42048, 836,   35724,
                51172, 9152,  33932, 1352,  59812, 49508, 17476, 64368, 2808,
                22348, 44364, 62932, 25324, 50984, 35072, 43000, 2480,  63952,
                38172, 43792, 9760,  27880, 18416, 46928, 8568,  30312, 16460,
                3348,  45856, 27024, 21612, 32,    11440, 36128, 5560,  54024,
                63704, 36984,
            },
            {
                28960, 61300, 34328, 17044, 58476, 21656, 44748, 25572, 60688,
                46736, 30924, 22600, 10200, 36408, 47972, 56032, 2968,  62360,
                32348, 42072, 7704,  37572, 22708, 10652, 18648, 51076, 60816,
                26064, 42340, 34656, 46044, 24508, 65304, 47668, 12160, 60208,
                42620, 3104,  51276, 21248, 39772, 29280, 49716, 20460, 25032,
                41012, 32412, 5824,  47392, 10104, 42036, 20288, 6108,  61512,
                24388, 52276, 63592, 13112, 35276, 19344, 41640, 51184, 15664,
                34172, 20108, 42176, 828,   56336, 35028, 49916, 8192,  59248,
                47316, 64964, 35864, 45048, 61464, 11088, 24728, 1052,  48696,
                60524, 16732, 34808, 2496,  45580, 7700,  26620, 47468, 4828,
                56256, 28276, 22628, 13204, 37304, 4788,  20964, 11140, 28932,
                40048, 34236, 3752,  38768, 20572, 61600, 27552, 46712, 56328,
                13840, 52304, 1276,  59396, 36612, 4064,  52928, 61076, 24012,
                51284, 32372, 6824,  40628, 33796, 51976, 26308, 46228, 13904,
                9596,  18828,
            },
            {
                53256, 2360,  39332, 24772, 48208, 840,   15352, 54428, 19728,
                35708, 7200,  49608, 57512, 1072,  25592, 16644, 43948, 37040,
                20708, 60176, 25236, 52996, 29816, 41272, 55772, 35436, 44372,
                15668, 53008, 6616,  19444, 9944,  56572, 32284, 5296,  54676,
                24644, 18068, 32388, 57540, 6516,  15116, 61540, 11204, 35040,
                65424, 16196, 12736, 37884, 59944, 53332, 31592, 46300, 14364,
                692,   30604, 8964,  49248, 26944, 4032,  65152, 30228, 24084,
                61256, 54648, 36812, 10328, 18688, 23884, 63172, 37548, 21660,
                11304, 5288,  51536, 2116,  17212, 56016, 37180, 53140, 20656,
                12388, 56488, 40536, 54152, 13716, 38124, 58684, 17060, 11840,
                39092, 61296, 48912, 45020, 54928, 26720, 58456, 42760, 51592,
                60628, 17992, 53272, 12964, 6652,  48940, 15616, 10728, 35572,
                19040, 30368, 23440, 65004, 15068, 44140, 26272, 39436, 35244,
                13332, 62912, 18624, 59052, 48828, 15680, 30548, 60332, 40112,
                33576, 22924,
            },
            {
                59532, 46528, 12588, 65432, 7980,  37380, 62756, 32880, 2948,
                42064, 63496, 15764, 28084, 39468, 64496, 34560, 11600, 53588,
                4436,  15088, 46328, 2488,  16508, 61308, 46844, 3256,  9348,
                21080, 58248, 31436, 50004, 41220, 1212,  36516, 15480, 44964,
                28736, 38032, 13036, 47544, 26280, 36560, 45536, 4976,  54064,
                228,   48552, 57288, 27584, 1900,  23388, 7828,  26744, 58152,
                39156, 18800, 43148, 58608, 16688, 55364, 44684, 5796,  47404,
                12888, 49720, 25728, 58996, 45228, 29488, 4860,  43916, 54912,
                15816, 42256, 28136, 22832, 31500, 49484, 14532, 5204,  33044,
                46360, 7036,  29532, 18944, 60904, 32536, 50372, 24284, 31376,
                43316, 20056, 5376,  16864, 30768, 9588,  33272, 47524, 14216,
                24452, 8512,  45684, 28320, 57932, 32316, 23616, 59528, 5108,
                41840, 55164, 47560, 34048, 7220,  56408, 11008, 20028, 1804,
                42412, 47876, 29188, 10452, 4000,  63248, 20196, 3004,  55572,
                50172, 5164,
            },
            {
                15760, 27260, 20388, 32492, 51784, 28724, 11836, 47556, 9128,
                58808, 23980, 4560,  44816, 12760, 50968, 7412,  23840, 58624,
                48528, 31304, 10348, 65428, 33676, 6644,  25404, 14280, 28816,
                64516, 36900, 12048, 27376, 62924, 53440, 23116, 60900, 50384,
                4256,  62796, 55480, 812,   64168, 22520, 51892, 42084, 18280,
                30252, 22984, 43960, 19332, 50700, 34464, 64584, 48400, 11240,
                54812, 33092, 22188, 2636,  40024, 33440, 9676,  21256, 38760,
                2220,  31456, 6800,  16156, 51728, 33792, 14496, 48976, 26328,
                32260, 60676, 56776, 10228, 35212, 59756, 43880, 27344, 41492,
                23292, 62500, 50848, 25796, 9832,  4500,  20604, 65128, 53956,
                8064,  35004, 57484, 64092, 39636, 53016, 536,   37652, 5812,
                31516, 56512, 36680, 63636, 42208, 448,   39488, 51124, 26828,
                61768, 3208,  11368, 21092, 41352, 50076, 31588, 63708, 54956,
                8232,  57176, 37000, 24920, 54068, 38872, 44092, 8552,  25068,
                29820, 42304,
            },
            {
                35536, 44288, 57136, 4928,  41424, 22796, 55748, 17656, 38392,
                29344, 52496, 34092, 55436, 18744, 60996, 29976, 41412, 17524,
                27264, 39228, 55300, 43432, 19652, 52152, 38424, 56664, 49052,
                40580, 1640,  45396, 16256, 4600,  18784, 30300, 7992,  40080,
                20264, 10272, 34384, 41496, 16884, 8504,  32836, 26996, 60836,
                39348, 9448,  63228, 6924,  39872, 15240, 42920, 18196, 36236,
                5436,  50336, 60172, 28184, 12360, 52352, 25144, 62356, 57872,
                18556, 43488, 64588, 40704, 3200,  61380, 9280,  39748, 528,
                19584, 7468,  39328, 46680, 3692,  20276, 8688,  65384, 57252,
                10636, 36880, 160,   42972, 48316, 35772, 40880, 1416,  45816,
                15176, 27440, 2552,  25360, 12716, 22872, 59544, 19560, 65180,
                48916, 1940,  21944, 16852, 10192, 54884, 19932, 8248,  45280,
                32856, 16472, 38600, 29120, 58772, 17472, 5620,  46244, 28028,
                22716, 16668, 4992,  45328, 13516, 22280, 34880, 57380, 17584,
                64520, 11000,
            },
            {
                54588, 8376,  49852, 18236, 60732, 2132,  43296, 26264, 50012,
                13348, 20020, 40744, 9464,  32136, 22052, 5540,  45756, 572,
                63268, 7916,  22268, 35692, 28020, 12204, 59776, 22452, 32196,
                6388,  54348, 24608, 59484, 39308, 48460, 44024, 12756, 27044,
                52956, 46404, 23660, 29760, 53880, 59784, 12232, 3364,  56584,
                14272, 36200, 54828, 32208, 12112, 56468, 3976,  29436, 61924,
                25316, 15028, 44932, 63872, 7156,  47980, 36184, 14584, 28420,
                52900, 34408, 10936, 27644, 20328, 54112, 22868, 57832, 64464,
                36216, 52960, 13116, 63340, 25644, 54520, 38176, 1780,  18328,
                30824, 15520, 55256, 22164, 63380, 16200, 56784, 29840, 11248,
                62320, 48184, 41368, 50936, 6936,  46696, 42396, 15388, 27172,
                40860, 12332, 51300, 26096, 34056, 46484, 29704, 64552, 12840,
                22548, 53444, 49036, 63184, 292,   24108, 37764, 14560, 51888,
                40076, 33856, 64672, 50312, 31664, 61404, 1660,  12380, 48732,
                38228, 424,
            },
            {
                61596, 24524, 30468, 38500, 14312, 34788, 58196, 6432,  62000,
                264,   46352, 63024, 3632,  48796, 38312, 53168, 57268, 33536,
                50684, 13580, 58968, 4616,  49644, 1196,  44112, 8712,  17284,
                61796, 20260, 34364, 52004, 9580,  33184, 61856, 56884, 35848,
                2760,  58640, 14184, 5440,  48776, 37976, 20980, 45996, 49612,
                24800, 2444,  51492, 28496, 46532, 61040, 22424, 52428, 9484,
                41092, 1788,  37116, 31400, 19796, 55840, 42576, 452,   46276,
                8284,  23216, 55956, 48368, 37092, 31084, 46460, 11944, 27852,
                50304, 24112, 17900, 42464, 30352, 15252, 51144, 33840, 45876,
                50024, 60044, 5480,  33236, 12280, 27120, 7336,  51740, 23548,
                37820, 32600, 55732, 18756, 36160, 62804, 29680, 8268,  55208,
                34972, 61912, 43992, 3164,  60388, 15032, 4284,  36948, 57244,
                40984, 6356,  26584, 9508,  45492, 54384, 34396, 61000, 11616,
                1248,  58260, 19432, 9372,  26528, 41660, 52108, 28256, 45140,
                32168, 21504,
            },
            {
                41004, 12496, 3052,  64072, 47116, 10448, 52696, 31784, 21700,
                36268, 56380, 26056, 16832, 58980, 27556, 14732, 10184, 19792,
                37068, 26224, 45552, 18168, 40408, 62736, 29920, 36092, 46516,
                13224, 41972, 3108,  28596, 14384, 21880, 5816,  24752, 17340,
                42708, 31000, 62504, 18740, 43660, 27972, 6916,  64804, 34096,
                17452, 44392, 5980,  20340, 25428, 1148,  37756, 33300, 45832,
                21232, 53848, 10928, 57436, 23988, 17056, 5360,  31992, 59692,
                39844, 16912, 62556, 6376,  13808, 1688,  42316, 16816, 6612,
                45276, 3152,  33288, 6024,  48100, 61252, 11504, 22648, 26156,
                7448,  39544, 28432, 47124, 53212, 38968, 44840, 59680, 17312,
                4844,  21452, 9236,  14276, 57740, 2240,  33664, 51752, 23376,
                5016,  18364, 30576, 56772, 38956, 48452, 52768, 25460, 17872,
                1924,  61104, 35388, 13476, 19156, 29956, 3588,  43672, 25800,
                48472, 30052, 42812, 3512,  55912, 16244, 7284,  58836, 19296,
                5580,  51088,
            },
            {
                57720, 34060, 45028, 20932, 28428, 24980, 16888, 39804, 45248,
                8564,  30848, 11772, 43520, 34540, 1588,  64796, 24988, 44148,
                3116,  61784, 30696, 52792, 15232, 24736, 55068, 4956,  51688,
                27508, 56920, 48024, 63656, 37752, 55500, 556,   50916, 65092,
                7656,  38964, 49836, 1712,  35460, 57828, 15300, 40156, 29392,
                10580, 58432, 62488, 41680, 15916, 49796, 59116, 7080,  17204,
                65308, 48448, 27632, 3724,  39252, 49964, 63308, 25708, 13220,
                51120, 4140,  44176, 26340, 57280, 50912, 60160, 35056, 29980,
                56224, 38664, 59400, 53980, 21040, 2804,  40700, 56208, 63936,
                13360, 44404, 17596, 3708,  58024, 20000, 2312,  33832, 42308,
                50376, 60488, 30784, 43840, 26840, 21072, 45544, 11672, 59204,
                47660, 37940, 9516,  20548, 6944,  23184, 11184, 33196, 44516,
                49896, 30972, 42948, 55824, 39568, 50308, 65252, 8980,  20680,
                53972, 15148, 60380, 37228, 47076, 21576, 40044, 35780, 63556,
                14672, 27148,
            },
            {
                4132,  17144, 59276, 54004, 7364,  49672, 60268, 3176,  13996,
                64640, 53260, 23200, 6788,  52080, 39532, 46896, 31228, 54556,
                41528, 11432, 7024,  56820, 34020, 9844,  20668, 64944, 38772,
                23192, 10944, 6792,  17760, 44596, 30856, 40496, 46676, 33564,
                11288, 21400, 25924, 55596, 9724,  24360, 50792, 580,   53688,
                22232, 37824, 32484, 55728, 9800,  30144, 43324, 12564, 26636,
                58212, 32100, 15772, 44068, 61376, 8520,  34520, 20676, 54284,
                36388, 30484, 19472, 38684, 32928, 24144, 18788, 8840,  61900,
                21980, 14844, 10712, 27400, 36356, 44988, 29332, 6504,  35424,
                52404, 21524, 62728, 37192, 24640, 10352, 64724, 26464, 13252,
                54448, 356,   39524, 62084, 5488,  54032, 16576, 39740, 3608,
                26380, 14760, 53716, 64828, 28048, 42260, 62632, 58276, 8648,
                20876, 15156, 24488, 59548, 6844,  22532, 15924, 56852, 32464,
                38628, 7020,  23636, 10816, 29256, 62584, 2560,  24972, 10092,
                53548, 47604,
            },
            {
                23180, 8756,  39416, 31292, 756,   36684, 42512, 55124, 28764,
                18136, 47692, 37136, 61356, 18892, 12832, 21336, 4852,  59492,
                16928, 23108, 47472, 38112, 2620,  43300, 48104, 14252, 212,
                32268, 60724, 35496, 25500, 59140, 8696,  19612, 27816, 15484,
                60200, 53228, 44968, 13644, 32336, 60600, 19968, 42132, 62976,
                7276,  47044, 13876, 4572,  36164, 63128, 19428, 55244, 39496,
                180,   6208,  35560, 52936, 12916, 29672, 46816, 41812, 9688,
                1392,  60428, 47456, 11256, 65400, 3596,  40216, 52736, 47632,
                864,   43208, 64344, 50040, 8360,  16476, 58708, 19348, 1584,
                42168, 31256, 8700,  49904, 32420, 15376, 48680, 36432, 8108,
                29052, 24256, 47720, 10760, 49808, 37024, 64292, 29024, 43292,
                61284, 32520, 49792, 44896, 35784, 17240, 2864,  29192, 37736,
                53832, 64400, 4640,  11400, 47600, 37016, 28440, 41692, 4252,
                46188, 63964, 35320, 52848, 13856, 49764, 33524, 55464, 42504,
                30592, 37576,
            },
            {
                64500, 43644, 48860, 14828, 63140, 20004, 9748,  23944, 34500,
                4804,  41084, 2016,  57456, 28376, 49936, 62604, 8120,  35912,
                51656, 27920, 64460, 18864, 58380, 26048, 30480, 59252, 18200,
                45512, 53132, 41348, 2260,  49636, 12216, 54124, 58360, 6476,
                37156, 29524, 3604,  63232, 38520, 46828, 5484,  12584, 31272,
                25660, 18400, 52116, 27316, 22660, 47460, 2820,  34720, 51100,
                24312, 42748, 19544, 58868, 22528, 2184,  56352, 16356, 64248,
                28204, 22368, 55648, 7516,  15872, 45560, 28492, 13028, 36772,
                25788, 31880, 19140, 40128, 24632, 62860, 32708, 46768, 54780,
                25528, 12116, 57024, 656,   61548, 43024, 55900, 20828, 45440,
                63348, 18424, 34860, 15100, 22752, 32044, 1264,  19984, 55672,
                8352,  22892, 124,   12596, 5188,  55368, 50464, 13640, 46512,
                1380,  34748, 27840, 52136, 32872, 952,   58372, 12488, 50932,
                18496, 27416, 612,   43620, 58576, 18140, 8424,  46060, 13232,
                984,   19044,
            },
            {
                34992, 28088, 11644, 25280, 56908, 33224, 46308, 61196, 50500,
                58520, 26368, 15336, 32296, 9332,  41880, 33716, 24368, 14612,
                39980, 792,   32328, 13244, 51180, 6624,  35612, 49848, 39916,
                8384,  28516, 21036, 15688, 64564, 34308, 24060, 42832, 1876,
                48700, 18184, 41756, 7780,  16672, 27488, 54760, 35248, 49444,
                59956, 39292, 1228,  61280, 40832, 53952, 8456,  28956, 14676,
                61964, 47636, 10012, 38308, 48928, 26728, 6600,  37276, 44004,
                13636, 49588, 41056, 34904, 53492, 59096, 20484, 63192, 4684,
                55156, 48832, 5668,  56536, 1984,  52120, 13568, 9324,  38256,
                60244, 16568, 45252, 27588, 39596, 6360,  30072, 4148,  58748,
                38456, 3232,  56300, 42356, 58320, 8860,  51576, 13712, 48252,
                35128, 17816, 40904, 58848, 31692, 21768, 39556, 25968, 61936,
                18268, 41636, 45648, 16744, 63368, 19852, 44164, 23920, 59996,
                10144, 54976, 21296, 32156, 5464,  39024, 28012, 21000, 60780,
                57372, 50732,
            },
            {
                6688,  61504, 4340,  51348, 41568, 16776, 5552,  12952, 21056,
                8048,  44660, 49328, 22508, 55096, 4160,  45012, 58008, 48824,
                55964, 9284,  45764, 21776, 40896, 61368, 11156, 22908, 4364,
                63272, 12808, 55948, 47040, 29784, 3812,  38540, 51516, 21684,
                64040, 34104, 23752, 57908, 51404, 21476, 2336,  43700, 16116,
                9984,  56104, 44760, 33156, 11668, 17016, 64296, 45500, 21804,
                55888, 6972,  30120, 63728, 17412, 33592, 60956, 51016, 24348,
                32612, 5988,  18108, 26120, 1580,  31348, 10096, 42008, 33692,
                16616, 60432, 12468, 29608, 35188, 42900, 22764, 28248, 49496,
                3772,  34416, 22632, 52916, 18672, 12660, 23932, 51068, 16320,
                11640, 52796, 30340, 6028,  26420, 45964, 60240, 24476, 5652,
                38564, 63980, 51204, 25096, 46884, 60640, 9764,  33080, 6268,
                23472, 56752, 8348,  38608, 25680, 54160, 7716,  34344, 30268,
                40172, 15332, 62952, 48168, 24464, 51992, 65508, 3236,  40224,
                24684, 15588,
            },
            {
                45508, 53804, 21440, 37684, 2140,  29308, 54204, 39248, 31468,
                62480, 35804, 11656, 38972, 65512, 17388, 26972, 2440,  19436,
                29648, 60568, 34780, 54372, 3464,  28408, 15728, 53728, 57640,
                33672, 24984, 37368, 6140,  43744, 18580, 62080, 13536, 27540,
                9436,  55424, 12208, 45592, 36456, 30032, 64572, 57384, 23508,
                3996,  29096, 20728, 6680,  50480, 24776, 37808, 3640,  33260,
                12136, 40360, 25340, 4224,  54252, 41172, 11972, 19732, 57256,
                2996,  59540, 62856, 44724, 38272, 23016, 47376, 51484, 24396,
                8260,  39048, 21860, 46388, 16036, 58020, 4728,  62280, 19900,
                41108, 65264, 6952,  47584, 35980, 54504, 60704, 40872, 33548,
                27168, 44492, 21108, 65208, 38736, 17308, 33068, 41340, 29716,
                57308, 10500, 27868, 14552, 7528,  18912, 43696, 52668, 58480,
                48612, 12620, 31336, 60968, 2944,  13700, 48248, 62304, 2212,
                47056, 36156, 4120,  41848, 11768, 15956, 30776, 36036, 48992,
                10624, 32692,
            },
            {
                41124, 17804, 30296, 59760, 47196, 65052, 24580, 43544, 3440,
                53216, 18516, 888,   52096, 29368, 13400, 59992, 37488, 11032,
                43420, 4584,  25024, 17484, 63416, 43956, 37784, 46668, 19028,
                42716, 1720,  52200, 61096, 10832, 57008, 32148, 6772,  45300,
                40584, 31032, 1188,  61644, 6112,  14460, 9044,  39012, 33428,
                48652, 63548, 36556, 15040, 59448, 30876, 42660, 57928, 18344,
                52772, 59980, 35260, 15064, 45316, 768,   30920, 8664,  46028,
                39684, 29124, 11632, 50840, 7876,  58140, 14368, 2468,  56740,
                44772, 27452, 53460, 64868, 7228,  38020, 47880, 33368, 11492,
                51756, 30324, 14724, 57572, 2372,  32040, 9696,  45804, 912,
                61776, 8592,  48952, 13616, 1896,  54952, 10956, 61952, 3728,
                20612, 45860, 2308,  56464, 37964, 63648, 992,   15424, 29472,
                36892, 3672,  22028, 50532, 41068, 29260, 37432, 17124, 21816,
                56492, 9208,  28284, 53484, 59288, 7608,  44628, 54424, 5080,
                27320, 56368,
            },
            {
                1812,  9520,  35676, 13912, 7344,  19100, 10252, 57116, 15548,
                28616, 25232, 59132, 45988, 5468,  35172, 47868, 22332, 53368,
                63972, 39272, 12644, 47676, 8492,  31480, 900,   25936, 9308,
                30960, 49132, 16524, 22376, 35696, 25928, 48088, 53404, 17032,
                59608, 51112, 25748, 19716, 49044, 42836, 26476, 52904, 18584,
                12944, 41472, 26240, 54832, 45796, 5140,  10188, 49020, 27324,
                1624,  43936, 20644, 50636, 58004, 23868, 64940, 52588, 35600,
                15536, 54212, 21416, 33220, 16868, 64484, 40476, 30188, 18396,
                61496, 36188, 368,   11028, 30868, 25848, 17944, 55752, 1520,
                25168, 44488, 9180,  38876, 25908, 64108, 21608, 49332, 19200,
                37540, 24864, 31768, 59804, 35588, 20296, 44544, 49656, 15408,
                52744, 33936, 42032, 30748, 53820, 34768, 26992, 40780, 20092,
                65000, 43232, 55940, 16112, 9412,  58120, 52364, 5524,  43028,
                25804, 61616, 19404, 32656, 37672, 22472, 62296, 19008, 14432,
                60012, 22004,
            },
            {
                64380, 49840, 57888, 44916, 27524, 52052, 33956, 48096, 37264,
                63948, 40764, 9104,  32648, 20220, 54624, 41168, 7292,  32248,
                15608, 27728, 52244, 36084, 55876, 19980, 50420, 58488, 65376,
                13300, 40520, 58936, 29080, 8076,  39764, 340,   21164, 36628,
                4288,  13948, 35372, 40020, 55000, 31984, 58824, 20,    47020,
                60060, 8132,  51432, 2352,  19368, 62712, 22756, 35868, 14304,
                64716, 30980, 10604, 5792,  28172, 37020, 13396, 21052, 4628,
                26848, 47732, 16,    37268, 25188, 4008,  53720, 35016, 11984,
                5176,  49348, 20280, 41020, 51912, 59868, 43380, 14096, 36404,
                59348, 21220, 61816, 18156, 42964, 13184, 28780, 6184,  56364,
                14492, 54080, 41512, 4460,  51996, 29252, 7852,  26696, 37480,
                23748, 62948, 17084, 6120,  22524, 12440, 47140, 51584, 5356,
                10596, 33404, 25248, 46940, 35192, 20648, 27144, 64872, 33172,
                50988, 12548, 44976, 2572,  49324, 26548, 28,    33980, 42408,
                47152, 37420,
            },
            {
                31672, 25768, 4932,  22792, 38912, 61660, 344,   21836, 6544,
                13168, 50776, 22964, 43200, 63136, 10460, 25120, 61144, 24,
                49600, 21196, 5864,  59324, 23744, 41016, 14756, 34472, 4536,
                54232, 20580, 3436,  45840, 64132, 14644, 55176, 60880, 29656,
                43720, 23076, 65100, 2952,  16756, 11020, 21172, 36952, 4968,
                30380, 23044, 32796, 12064, 39004, 28856, 56096, 41248, 8148,
                51544, 38484, 47396, 62648, 17296, 42040, 49012, 59296, 43464,
                63288, 9148,  56116, 60740, 44064, 49172, 20884, 26924, 57772,
                42768, 32124, 15296, 62384, 23552, 3432,  8668,  50088, 28676,
                6100,  53712, 33748, 48420, 4752,  51664, 59160, 39992, 34280,
                63292, 6676,  18148, 46740, 23132, 39776, 64120, 1524,  59164,
                12576, 8372,  58280, 49780, 44284, 3168,  57444, 24308, 59876,
                14600, 53284, 1224,  63088, 6156,  44524, 11512, 460,   15120,
                38536, 7016,  54840, 16548, 40956, 58072, 10316, 51596, 29352,
                6592,  12404,
            },
            {
                53248, 43016, 16216, 54528, 31092, 12348, 42548, 60152, 31840,
                46288, 3196,  57764, 15256, 2188,  49876, 16988, 28776, 45808,
                56992, 41716, 33292, 2720,  11404, 62432, 7052,  27712, 44060,
                24236, 38040, 11388, 32356, 50180, 41984, 24436, 11832, 7660,
                49424, 56700, 9904,  28312, 60324, 46464, 24708, 62988, 43632,
                15776, 55992, 65340, 44180, 49580, 16416, 620,   59344, 19616,
                25884, 2768,  22512, 33212, 55440, 7872,  1976,  24948, 33632,
                18048, 31220, 40304, 13600, 29356, 10208, 5956,  46720, 63824,
                22668, 9276,  55804, 45616, 34876, 39540, 19592, 63912, 46108,
                38224, 15840, 480,   56120, 31160, 36744, 16712, 2736,  22464,
                44384, 10560, 28032, 57960, 12100, 15936, 55272, 31900, 42984,
                47324, 29852, 36476, 26364, 19588, 62716, 8960,  30984, 37732,
                45220, 27948, 39972, 18612, 30900, 59428, 54256, 41488, 48620,
                24208, 60796, 30236, 21444, 64680, 13792, 36436, 55700, 23576,
                62616, 20060,
            },
            {
                956,   34908, 10472, 63748, 3992,  48972, 15948, 26588, 55568,
                18284, 34852, 27196, 37988, 30392, 55780, 39656, 34696, 13728,
                9184,  18604, 64844, 45092, 29732, 47088, 36976, 52492, 17772,
                48224, 62992, 55860, 26816, 5600,  19172, 34208, 46452, 62812,
                17680, 32172, 41932, 52360, 5648,  34628, 50596, 7572,  53680,
                40680, 19992, 35272, 5632,  25028, 52964, 31528, 46724, 34544,
                61664, 15804, 57588, 43612, 12416, 29916, 39796, 53924, 10592,
                50488, 5256,  23612, 19500, 52684, 59064, 36380, 16264, 39204,
                1420,  51072, 29032, 6192,  12908, 27180, 57232, 32292, 22692,
                11348, 42068, 27340, 23668, 10420, 63552, 46620, 25488, 52816,
                30844, 50008, 37128, 62056, 3360,  34420, 48840, 18404, 5364,
                21168, 40512, 432,   54640, 14120, 39404, 33972, 16836, 50052,
                21604, 8296,  60772, 48092, 13320, 37580, 22416, 29384, 18936,
                56836, 34976, 4236,  47548, 8104,  31644, 46172, 3584,  15592,
                39752, 48160,
            },
            {
                56872, 18692, 59236, 40416, 20940, 35812, 7932,  51416, 39600,
                10740, 60812, 51964, 8004,  45224, 19576, 62212, 3776,  58764,
                23520, 37332, 51000, 26256, 16940, 54848, 21916, 61036, 9896,
                30156, 708,   35764, 16628, 53164, 59072, 2524,  38508, 26916,
                4892,  37340, 20332, 45156, 14192, 39148, 18228, 29420, 11888,
                26776, 1304,  10212, 60304, 37848, 13660, 7380,  43388, 11624,
                54028, 40556, 5324,  26760, 51024, 61480, 20112, 36576, 14804,
                61088, 46392, 35452, 65024, 2424,  41912, 31460, 12728, 54644,
                25920, 34124, 17756, 58504, 48276, 1608,  52316, 16716, 4176,
                62336, 51332, 57936, 45192, 20016, 6896,  41076, 12352, 58408,
                19316, 148,   42128, 24764, 53072, 44952, 9144,  25272, 56968,
                51788, 65360, 10700, 60196, 28912, 48496, 43168, 4508,  63988,
                2324,  54904, 34072, 24168, 3668,  50804, 7748,  62412, 3044,
                10180, 43788, 27152, 39440, 52720, 25164, 18392, 42840, 60224,
                9084,  26984,
            },
            {
                5960,  44584, 29148, 50924, 25524, 45892, 57204, 30244, 23600,
                4960,  42252, 21284, 63836, 12720, 24696, 6552,  48588, 42932,
                53316, 31120, 6868,  12956, 39908, 1160,  5420,  32820, 14156,
                41440, 7396,  59988, 22732, 43784, 9504,  31064, 51260, 57768,
                12844, 54808, 908,   25860, 57088, 64024, 2204,  33448, 61228,
                49076, 57680, 45720, 28244, 54868, 21104, 64232, 23940, 3896,
                29624, 48996, 9956,  35824, 18232, 3216,  45032, 56524, 27104,
                1344,  57648, 43036, 8140,  26492, 17612, 51672, 62652, 7296,
                45188, 60716, 42140, 21732, 65172, 35860, 10044, 40708, 47484,
                30096, 7800,  13940, 35836, 60204, 28416, 54572, 33496, 8388,
                65084, 15488, 33056, 7404,  20468, 29604, 63072, 38108, 13240,
                34636, 16468, 24296, 32768, 6980,  53376, 23272, 12220, 56912,
                29996, 17600, 10908, 42860, 57712, 16084, 33416, 45884, 36712,
                51328, 16972, 58424, 12456, 62900, 5452,  57084, 28576, 33424,
                50028, 37880,
            },
            {
                23740, 32748, 14052, 8544,  3084,  62640, 19228, 1408,  64956,
                14476, 47408, 31428, 644,   54072, 36988, 32448, 10840, 26876,
                15700, 1752,  61312, 57160, 35044, 59844, 49860, 43284, 56556,
                20064, 51456, 46796, 12312, 39304, 64988, 20552, 15544, 41504,
                22820, 48772, 61848, 30360, 7860,  23332, 54276, 44440, 21428,
                15164, 38764, 31888, 17732, 3452,  47992, 33864, 57096, 17040,
                37932, 21308, 58888, 46224, 65280, 24036, 31800, 7268,  41452,
                22140, 29152, 11712, 54932, 33896, 48268, 4768,  24188, 19812,
                37316, 10900, 4512,  14652, 30664, 44104, 54616, 24068, 59968,
                34372, 18988, 39816, 3248,  50156, 16236, 1724,  47900, 38440,
                27424, 45376, 56168, 60860, 12924, 40900, 1128,  59364, 27944,
                4016,  42728, 50448, 45528, 2452,  17956, 59588, 27072, 38860,
                44856, 35884, 49160, 65240, 28368, 40376, 55264, 25832, 14008,
                64444, 23320, 884,   32972, 20416, 37412, 50964, 1828,  21628,
                12268, 64992,
            },
            {
                2616,  47052, 61264, 53744, 38624, 12504, 33680, 43944, 53580,
                36420, 25952, 58828, 17504, 41288, 50200, 60028, 20560, 63760,
                39196, 47748, 21508, 42416, 18452, 9036,  23100, 27348, 62792,
                38148, 25100, 33376, 28708, 1632,  48600, 25484, 4116,  63244,
                33332, 9820,  36180, 16008, 40996, 48328, 9992,  36752, 4308,
                52544, 7312,  23072, 62612, 42724, 9544,  40292, 27640, 50324,
                63104, 1480,  32380, 15244, 6332,  51984, 13452, 47904, 63680,
                17072, 38712, 50044, 15188, 21368, 60412, 39932, 30092, 57560,
                49804, 32024, 53560, 39112, 25640, 6860,  19804, 13672, 2064,
                26360, 53312, 64896, 22320, 32204, 43372, 62168, 20888, 13628,
                51800, 3912,  22916, 48440, 36204, 17232, 50760, 21976, 46576,
                54996, 8032,  36776, 20844, 63776, 41280, 34568, 7712,  51048,
                14360, 25000, 5088,  21032, 9376,  1200,  19600, 59732, 5764,
                31020, 42116, 54492, 49284, 44640, 10080, 14792, 62220, 45292,
                54668, 16588,
            },
            {
                58376, 36096, 17968, 22480, 42664, 27636, 59912, 16592, 7136,
                49232, 9936,  34276, 4760,  28448, 14564, 45524, 2688,  29780,
                55316, 8052,  33048, 25720, 52972, 46056, 31252, 11964, 2796,
                16204, 4756,  61744, 57364, 14896, 35268, 56080, 44764, 28404,
                6856,  45932, 19188, 51696, 59264, 32288, 13764, 57992, 25888,
                64696, 46612, 12552, 35972, 51312, 15832, 60668, 5732,  12672,
                44632, 25756, 53044, 41024, 28672, 37744, 58180, 9884,  34176,
                53524, 6020,  31652, 63108, 508,   43796, 10664, 47128, 2176,
                15888, 22948, 61880, 720,   57804, 49136, 62520, 37720, 56348,
                42608, 5152,  48444, 9636,  25684, 56608, 4980,  36552, 24480,
                60040, 40200, 10340, 31396, 54220, 6336,  43532, 10536, 30976,
                18540, 60980, 26192, 14436, 55688, 11216, 46892, 21312, 62260,
                156,   54484, 59860, 38180, 52392, 32368, 43236, 10836, 47736,
                38476, 19092, 8480,  27796, 61364, 24288, 41740, 35632, 7356,
                30588, 40328,
            },
            {
                25464, 48984, 6396,  31452, 56384, 9528,  50452, 24344, 39856,
                20472, 62040, 44712, 56440, 22900, 8824,  57328, 35788, 17408,
                43732, 11800, 62352, 4772,  14444, 38632, 65116, 48724, 34900,
                45036, 53200, 9220,  18972, 42420, 7552,  52328, 12092, 37912,
                60276, 54100, 26856, 2464,  5948,  20752, 28424, 43248, 17896,
                40472, 29892, 2780,  56888, 25268, 240,   30716, 19912, 54984,
                34936, 7968,  11384, 19048, 60340, 384,   43112, 26400, 19964,
                3024,  45836, 24744, 58144, 37640, 28068, 14316, 35684, 64364,
                8644,  42468, 27576, 46448, 17976, 9532,  31948, 28236, 45752,
                12108, 17376, 36868, 59032, 13196, 40724, 29824, 53708, 7196,
                43800, 28664, 18652, 1952,  63644, 26784, 58032, 34136, 64544,
                40460, 3124,  52552, 31508, 38340, 28800, 5160,  57592, 32416,
                18436, 41140, 30860, 13416, 46196, 63164, 23892, 53668, 28160,
                3600,  60588, 35344, 16060, 2752,  31540, 58932, 18248, 26824,
                51856, 10256,
            },
            {
                43216, 13128, 62440, 132,   45760, 37436, 4384,  32280, 57940,
                2484,  29668, 12384, 51724, 38840, 65352, 26036, 5576,  52484,
                23888, 36904, 50252, 28680, 56176, 2076,  17588, 7592,  55392,
                22228, 40264, 31056, 50236, 24384, 61244, 30304, 17528, 520,
                22044, 14536, 42124, 65272, 38976, 46916, 62044, 688,   34376,
                9252,  60180, 48988, 20344, 32976, 53096, 38948, 47280, 23644,
                42004, 62060, 56324, 48484, 33824, 23060, 15992, 55432, 49148,
                62128, 40868, 10400, 18888, 7732,  52164, 56244, 25416, 18760,
                54120, 33352, 51344, 12852, 35488, 41276, 51916, 6228,  21392,
                63352, 32804, 29336, 45024, 416,   51032, 18996, 10992, 64096,
                16392, 35280, 58988, 46632, 21564, 39060, 15420, 4680,  23784,
                13652, 47512, 9836,  59292, 1300,  49356, 15988, 25448, 43940,
                9948,  48108, 6712,  26616, 17316, 2564,  36120, 15400, 58116,
                51168, 12616, 45432, 55740, 39760, 6296,  53616, 46932, 824,
                63792, 20544,
            },
            {
                57496, 34300, 52160, 26220, 15516, 19844, 63968, 13540, 42096,
                54332, 25260, 36792, 19196, 1664,  42312, 32980, 46772, 13512,
                58016, 456,   19456, 44260, 34680, 60120, 25200, 41956, 58760,
                28112, 11232, 64188, 37564, 2728,  21096, 47552, 40624, 57968,
                31936, 49164, 11072, 34604, 23596, 54916, 12148, 50092, 53396,
                23708, 15584, 37604, 7116,  44444, 58668, 10040, 64536, 14684,
                2252,  31136, 21672, 4156,  44480, 64456, 8196,  30712, 36756,
                13920, 28324, 54688, 34868, 47940, 22392, 3112,  59764, 45268,
                38132, 5332,  20948, 3304,  63860, 23864, 15428, 1244,  58428,
                49816, 8012,  54936, 23444, 16040, 62836, 34320, 46072, 31744,
                48964, 5672,  52140, 13556, 32604, 8436,  53184, 49084, 56412,
                28592, 35552, 20224, 44100, 23572, 63176, 40652, 53064, 36860,
                64596, 22560, 58268, 35156, 60988, 49592, 6040,  21852, 40748,
                33128, 25184, 20896, 65396, 48152, 11268, 22596, 13852, 32640,
                38068, 4372,
            },
            {
                29176, 8084,  21320, 39616, 58672, 53812, 29056, 46980, 8796,
                16880, 63472, 6628,  48656, 15440, 54696, 10056, 20452, 60440,
                31212, 41040, 64316, 8600,  22080, 12460, 30520, 51472, 19692,
                128,   14752, 47036, 6328,  54948, 34652, 9864,  63064, 24844,
                44008, 4800,  56864, 29932, 17008, 7564,  37240, 19504, 5588,
                44880, 55484, 28004, 63604, 13372, 17960, 28896, 5344,  36568,
                50816, 16636, 38080, 27204, 13048, 51376, 39356, 2284,  59068,
                22296, 4880,  44132, 15740, 65336, 42024, 32556, 7088,  28940,
                11392, 61080, 56844, 30080, 44548, 59184, 54264, 34284, 39172,
                25036, 41536, 3312,  60396, 38204, 6392,  26128, 58064, 22184,
                1120,  56504, 24172, 41968, 61884, 44956, 19340, 37244, 924,
                42584, 62552, 6832,  51400, 17500, 35052, 8576,  3904,  13676,
                29304, 2816,  51876, 11820, 41920, 29888, 55372, 44996, 8988,
                62332, 348,   7456,  30652, 26352, 36240, 57868, 43860, 60848,
                49492, 16228,
            },
            {
                46580, 41624, 65196, 30888, 10844, 2152,  36072, 21584, 51256,
                33388, 45232, 59148, 30840, 22484, 61508, 28552, 51460, 38036,
                4400,  26712, 15600, 47760, 52776, 39456, 46420, 5056,  37088,
                33208, 59832, 25960, 17220, 44192, 28192, 13968, 5748,  53384,
                35688, 19572, 62452, 2840,  45196, 60536, 31196, 26332, 63096,
                32444, 11436, 41508, 3644,  24540, 49892, 42688, 56108, 26112,
                60576, 46152, 57980, 6524,  54292, 18520, 24960, 46624, 11604,
                33412, 49712, 60548, 1108,  30192, 12612, 51096, 16740, 40692,
                23104, 49244, 14800, 40116, 7644,  19248, 27512, 47836, 10204,
                18620, 13424, 52584, 20248, 28744, 49292, 42508, 9352,  14872,
                40424, 27580, 36968, 11224, 3240,  29584, 25412, 60348, 10920,
                16276, 53944, 26560, 32624, 11956, 56692, 27616, 60220, 19484,
                55540, 38940, 44520, 20300, 8432,  24748, 63876, 13776, 18732,
                37424, 49776, 43416, 17156, 52692, 3324,  19400, 28340, 8884,
                24196, 55744,
            },
            {
                1540,  17880, 50560, 6500,  48332, 43276, 61216, 4984,  27184,
                39392, 476,   11300, 41300, 35420, 3636,  44208, 7264,  17712,
                49432, 11128, 55136, 32672, 6044,  62968, 16692, 10268, 61788,
                48784, 53664, 41564, 31720, 57040, 60476, 49296, 39068, 16348,
                28796, 8628,  50348, 25716, 40300, 13632, 48628, 42076, 57900,
                1560,  21856, 47248, 59976, 34580, 38468, 1508,  21188, 33620,
                7508,  11080, 40988, 30136, 62176, 35096, 42672, 63412, 17092,
                56568, 39984, 19940, 25204, 38640, 5372,  61980, 46648, 55100,
                34916, 164,   25784, 47364, 36608, 12024, 3592,  65296, 30636,
                44324, 62172, 35444, 46456, 10716, 55640, 3080,  36000, 61420,
                54176, 7784,  65520, 17140, 50872, 57468, 5776,  47068, 31676,
                21640, 38660, 58536, 4636,  41776, 47968, 21484, 44836, 49476,
                32864, 15800, 25772, 57140, 33968, 4468,  38292, 48516, 31504,
                26716, 54140, 60064, 13592, 38920, 63600, 50328, 5768,  40540,
                34928, 12056,
            },
            {
                58768, 36820, 24952, 14296, 34044, 18144, 23936, 52976, 13080,
                56076, 19364, 24440, 50856, 57800, 13308, 25180, 63736, 33800,
                42376, 61692, 23236, 37504, 1760,  27828, 56404, 24876, 43764,
                3328,  22684, 8216,  12408, 1280,  19948, 23764, 3684,  64104,
                47148, 12276, 36924, 53940, 21680, 56592, 4744,  10100, 16496,
                35920, 51116, 18748, 29968, 8848,  52524, 62404, 12284, 44248,
                53648, 19556, 23924, 48244, 980,   10156, 28988, 3872,  52764,
                6220,  27532, 8476,  52280, 56036, 34504, 21144, 26732, 8676,
                64240, 18264, 52412, 62448, 32092, 22780, 51012, 38064, 6892,
                57248, 26736, 5528,  32256, 15340, 64708, 24412, 18640, 44912,
                29848, 20124, 47740, 33668, 22396, 39516, 13192, 34940, 64272,
                49620, 2028,  45588, 15216, 65284, 30448, 736,   35960, 10800,
                6420,  63764, 1220,  47656, 60104, 18008, 52532, 2080,  59092,
                11704, 5000,  22672, 33608, 9576,  42744, 31756, 15656, 62148,
                53348, 21616,
            },
            {
                32072, 4896,  61472, 55468, 27996, 58340, 38436, 7744,  64424,
                47572, 29184, 62324, 7924,  17324, 38612, 47120, 55868, 21744,
                2276,  29376, 45896, 18660, 58540, 42480, 20788, 35744, 13992,
                29936, 18256, 39168, 65408, 51152, 36588, 43040, 32752, 56228,
                21024, 41656, 61416, 15704, 108,   34896, 28688, 52408, 24492,
                39140, 64180, 6700,  54748, 14968, 22988, 46568, 17104, 29412,
                65028, 2848,  32488, 59432, 15792, 21384, 49308, 37856, 23584,
                31696, 45748, 63916, 11372, 17392, 44776, 58252, 14244, 4060,
                43100, 29640, 10300, 5844,  56192, 43288, 59660, 17648, 14116,
                52020, 672,   40804, 59488, 21956, 39336, 53148, 32812, 11856,
                51408, 4656,  42892, 59936, 112,   54368, 44172, 18292, 7112,
                28808, 9900,  24232, 36308, 18900, 7956,  52812, 62028, 23640,
                42188, 51556, 29952, 13976, 40720, 10416, 27756, 43332, 20972,
                35288, 41464, 46600, 55404, 1384,  24428, 56856, 46316, 2608,
                26936, 43360,
            },
            {
                10476, 52548, 44984, 39132, 3100,  11684, 45584, 30788, 16176,
                36304, 4024,  44088, 33604, 53888, 1356,  31104, 10600, 15108,
                53116, 59356, 8400,  13596, 50408, 31016, 6736,  54052, 63580,
                50688, 58108, 34548, 25116, 46520, 6928,  10804, 52044, 26428,
                2004,  30308, 6596,  32504, 46956, 64900, 19668, 60108, 46176,
                2776,  12932, 27008, 40740, 59224, 32216, 4576,  57324, 40632,
                35956, 14100, 51744, 37456, 43448, 55080, 61428, 12784, 57776,
                41916, 14488, 35748, 48508, 29048, 1552,  39692, 31788, 50068,
                58944, 35844, 39376, 21208, 15644, 1496,  29072, 46628, 34456,
                23728, 19176, 49412, 8764,  29316, 1840,  46992, 7008,  58700,
                38344, 25432, 14040, 30872, 9828,  27772, 62228, 24560, 52992,
                57212, 40544, 61524, 50544, 55012, 25940, 39240, 13448, 58816,
                27916, 18716, 37336, 54980, 23168, 46032, 64860, 14984, 56532,
                8224,  63396, 28128, 17860, 61208, 29728, 20636, 12572, 37984,
                18492, 64304,
            },
            {
                29836, 15268, 22612, 8164,  19676, 63240, 49756, 1972,  22284,
                41744, 59328, 10464, 21176, 49228, 27688, 59792, 41788, 35656,
                5572,  39632, 26060, 34856, 65060, 10872, 48016, 40204, 4660,
                9648,  45640, 2124,  13820, 61388, 30060, 18064, 60284, 14524,
                45476, 59080, 52724, 23156, 39836, 11900, 7316,  42780, 30512,
                55972, 33380, 49116, 44156, 724,   37184, 26624, 9700,  49616,
                6096,  27256, 45972, 9132,  4796,  25496, 33656, 7692,  19236,
                2872,  59916, 21588, 4608,  61268, 23448, 10164, 54076, 19700,
                24372, 12676, 45960, 53412, 60748, 40856, 25244, 8488,  55620,
                42644, 62868, 36624, 56996, 43556, 12928, 62972, 26956, 16876,
                2548,  62088, 56472, 35888, 50204, 16564, 4904,  37320, 42936,
                14788, 20476, 33176, 12244, 3700,  44048, 33624, 17148, 2660,
                46636, 8864,  62528, 3892,  32364, 6540,  36316, 30388, 24444,
                51236, 3264,  14004, 37692, 48076, 6752,  35732, 59800, 50904,
                7720,  47576,
            },
            {
                1268,  35620, 49200, 41432, 32860, 53920, 25480, 35012, 57156,
                51364, 13880, 25724, 39904, 63332, 6232,  19032, 24020, 64352,
                48380, 17208, 44704, 56280, 1040,  24056, 15964, 32720, 25820,
                20524, 28348, 41904, 56392, 21812, 53932, 40348, 34920, 8000,
                37924, 18628, 10504, 57348, 27128, 51296, 35792, 14824, 22516,
                8944,  17560, 61032, 21160, 10984, 51548, 63376, 18616, 23652,
                61112, 56272, 22016, 63208, 30864, 17380, 39780, 46944, 50592,
                26372, 54320, 38936, 33096, 51440, 42332, 64876, 36636, 47884,
                2052,  63428, 27984, 4404,  33000, 49460, 12848, 64288, 2632,
                30996, 10916, 15880, 25692, 52400, 18204, 36300, 49652, 41736,
                21920, 45708, 7628,  19064, 41212, 47284, 58800, 11508, 31052,
                2568,  48192, 6384,  59316, 29360, 21984, 57300, 50080, 40560,
                55624, 34996, 12724, 50624, 19904, 60632, 53480, 740,   47956,
                39152, 19672, 32400, 52204, 10688, 43280, 54180, 3656,  32876,
                40240, 25196,
            },
            {
                53268, 17624, 62596, 57904, 28900, 14204, 43456, 9572,  18956,
                6672,  32512, 54820, 2920,  15452, 36940, 46080, 9004,  51240,
                31792, 21404, 4356,  52212, 38796, 19712, 43584, 59232, 55552,
                38116, 62284, 16280, 32920, 5852,  27012, 608,   49800, 64604,
                24968, 43104, 48904, 5248,  17216, 44608, 61928, 3368,  58664,
                52728, 36920, 5268,  25536, 16324, 55312, 39744, 47348, 33732,
                15344, 288,   41548, 11612, 53128, 58692, 1292,  65176, 29644,
                10552, 44376, 17796, 7960,  13404, 27212, 5784,  15796, 30400,
                7848,  55708, 17480, 23124, 9564,  36896, 19496, 52320, 38724,
                21372, 48336, 3648,  33724, 5592,  60892, 31140, 9140,  55388,
                34508, 29460, 53520, 23972, 63188, 1920,  33752, 22572, 51308,
                63588, 27160, 37776, 46000, 15780, 62656, 10240, 5388,  31176,
                21008, 24812, 44232, 39108, 27032, 9500,  41988, 17304, 12336,
                61828, 44188, 57320, 25396, 64416, 16232, 27548, 23140, 14392,
                56596, 60508,
            },
            {
                42872, 4240,  12104, 24404, 6152,  308,   60116, 39548, 65392,
                28604, 48200, 60364, 44728, 30000, 52432, 58244, 13384, 40,
                28260, 60680, 10404, 33280, 28852, 61344, 50096, 3096,  12640,
                47428, 4184,  52388, 10628, 48584, 43692, 13216, 55184, 21272,
                3272,  33684, 29220, 63296, 1672,  31848, 25176, 39520, 47432,
                27368, 41380, 65484, 45944, 35112, 30024, 2624,  7740,  43984,
                54228, 37980, 19112, 27992, 35472, 44960, 13664, 20984, 36548,
                60752, 1836,  30648, 58852, 47352, 56808, 19060, 45132, 60084,
                40084, 33572, 42924, 47628, 62312, 29416, 56632, 6316,  27116,
                45056, 58984, 53764, 40276, 20800, 44812, 23760, 540,   14808,
                64616, 11696, 3188,  38636, 13588, 27048, 55920, 45416, 7428,
                17832, 55524, 23056, 41136, 336,   48704, 36584, 26576, 64952,
                7548,  59376, 1088,  15532, 56460, 48544, 22100, 58396, 26404,
                34484, 9328,  5940,  21348, 252,   39724, 49552, 62560, 45464,
                11040, 20804,
            },
            {
                27604, 32312, 56308, 45364, 37072, 51716, 16608, 46708, 23684,
                4300,  37624, 17864, 22580, 11116, 33420, 25544, 42848, 38244,
                54300, 14960, 42148, 57704, 7224,  13924, 35700, 9024,  30516,
                18844, 35336, 23968, 63996, 38388, 59452, 16936, 31324, 46104,
                11564, 60660, 15128, 37056, 53824, 20808, 56184, 7012,  18724,
                10788, 31360, 1928,  57068, 13576, 59592, 21820, 27436, 12632,
                31072, 60156, 48120, 3316,  50484, 23672, 6160,  42656, 55944,
                15676, 24400, 52332, 40136, 21896, 34836, 788,   52988, 25648,
                11100, 50996, 14128, 57880, 440,   41720, 15756, 34816, 60868,
                13496, 7948,  29632, 64912, 11660, 50300, 58060, 38952, 46732,
                20072, 50748, 43980, 60420, 32008, 8976,  19988, 39852, 36112,
                59924, 13140, 9164,  53092, 31712, 18848, 54184, 42588, 14472,
                47796, 52380, 29664, 62892, 33328, 2972,  37496, 30832, 5012,
                46856, 54396, 29536, 42292, 58632, 31600, 8768,  18980, 30276,
                6008,  36440,
            },
            {
                48572, 9108,  39452, 18916, 63840, 21568, 34260, 31124, 53000,
                12796, 42172, 8364,  63664, 1824,  55292, 19868, 5060,  62828,
                34952, 23664, 47420, 18000, 25588, 46068, 63720, 22760, 53784,
                60708, 43048, 7696,  29288, 19820, 34640, 5020,  25812, 57916,
                40108, 51952, 22920, 41980, 10120, 46332, 13292, 33520, 63156,
                49732, 15568, 23256, 50740, 9240,  41828, 48712, 64064, 52084,
                4588,  24504, 8580,  64340, 16124, 54920, 32944, 27248, 8936,
                48632, 33708, 63752, 4004,  11192, 28916, 38156, 62996, 22308,
                3204,  36512, 20292, 6932,  25996, 51392, 46280, 3964,  24100,
                42144, 17828, 37140, 1504,  26836, 16640, 7368,  28304, 60008,
                33104, 25992, 6764,  17352, 52612, 48212, 64976, 15396, 1260,
                29548, 43656, 35032, 64480, 25016, 12060, 60484, 2232,  34608,
                17748, 10392, 41444, 19256, 45776, 11980, 61412, 51064, 15224,
                65348, 18336, 37204, 50264, 13688, 34848, 55084, 3136,  38732,
                51960, 65148,
            },
            {
                1016,  54636, 14772, 30132, 48960, 6964,  10344, 59168, 2456,
                56444, 26852, 50976, 35952, 47172, 40212, 59392, 15996, 49008,
                8616,  2648,  64768, 39900, 54584, 2172,  38544, 27380, 40972,
                300,   14780, 50556, 57440, 2824,  46872, 62340, 9260,  36172,
                18796, 5416,  56744, 7520,  27924, 50628, 60452, 748,   36276,
                54736, 44540, 6416,  38404, 32616, 19056, 3828,  36932, 17076,
                57452, 34536, 43404, 29812, 38216, 10732, 46572, 62908, 39288,
                5264,  20336, 13960, 45568, 61180, 50244, 8804,  14948, 43596,
                58960, 28060, 65388, 31812, 39040, 18088, 11416, 64020, 31680,
                51824, 55344, 22236, 47072, 56728, 42392, 35944, 53288, 10784,
                4248,  41600, 36540, 57852, 28484, 4532,  42348, 25676, 54172,
                49536, 21088, 3856,  47192, 6548,  39284, 28888, 45172, 23276,
                58400, 37908, 26168, 5944,  54712, 24104, 7944,  43572, 40196,
                3380,  23224, 10568, 60960, 5104,  24584, 47756, 42988, 58860,
                16304, 23516,
            },
            {
                46480, 61168, 25652, 4624,  58040, 41396, 27736, 44468, 15612,
                20596, 62072, 30752, 14288, 24212, 6528,  29468, 44644, 26520,
                20528, 51448, 12016, 31420, 6016,  20116, 49320, 16308, 58324,
                32548, 45956, 26684, 39820, 12520, 23304, 41664, 50340, 28716,
                876,   48428, 32084, 63848, 17444, 38700, 23312, 29592, 41888,
                20688, 26572, 62620, 28848, 52500, 60972, 25104, 45340, 10112,
                40684, 20008, 13444, 56040, 21864, 59012, 3944,  18408, 50852,
                58112, 36736, 42372, 26644, 17464, 56124, 41240, 32964, 48672,
                5208,  54724, 45408, 9452,  53596, 59468, 21488, 37952, 48468,
                2852,  10428, 33992, 61860, 14048, 31908, 3376,  63924, 18172,
                23096, 52088, 14400, 45780, 21424, 34672, 12240, 59108, 6960,
                32940, 61840, 27680, 15884, 57476, 51248, 20360, 5116,  55576,
                32068, 3704,  49832, 64056, 14212, 35820, 28904, 20240, 56316,
                32728, 52192, 27652, 44636, 16920, 63812, 20164, 9932,  26536,
                33264, 12400,
            },
            {
                20520, 35332, 43436, 52264, 12256, 23468, 63008, 35716, 49276,
                39236, 5400,  44108, 57412, 18600, 52124, 10168, 61240, 32684,
                56916, 37464, 27896, 44980, 59576, 34524, 10456, 52632, 6852,
                21724, 65168, 9680,  17800, 55768, 31648, 53692, 15640, 64892,
                44604, 24656, 13108, 35500, 44168, 5928,  57304, 14996, 9304,
                4280,  58168, 11764, 16860, 420,   56220, 14124, 33384, 28504,
                62748, 47236, 5908,  51360, 1096,  41652, 31536, 25228, 11976,
                29216, 96,    53904, 32144, 6288,  24096, 2472,  19352, 30248,
                12140, 17172, 40668, 23064, 1772,  34468, 27516, 7384,  14900,
                58352, 25880, 40836, 5320,  51224, 20564, 24860, 48304, 44320,
                56200, 30764, 63532, 10364, 488,   62124, 38232, 22492, 45680,
                10556, 18348, 55684, 42940, 35988, 8992,  63700, 48800, 13004,
                42184, 60128, 21136, 31048, 39424, 52740, 59176, 1680,  25988,
                13032, 46364, 57736, 1136,  40508, 29156, 37080, 56612, 2104,
                62400, 40812,
            },
            {
                7996,  16808, 2984,  32588, 38284, 18472, 976,   53424, 8244,
                25108, 33900, 11344, 496,   64996, 34756, 38656, 2164,  13312,
                41992, 4216,  61992, 14740, 22680, 55916, 29688, 43600, 13268,
                37028, 3868,  51944, 34428, 61236, 1432,  6440,  21704, 11112,
                37708, 59672, 19848, 54796, 2224,  26356, 45908, 52060, 64384,
                48804, 33324, 39660, 47540, 42676, 22564, 5708,  49984, 54452,
                2036,  23400, 60556, 27076, 36320, 16948, 61732, 52652, 43188,
                22500, 64704, 9076,  48372, 59644, 35628, 52888, 64156, 57708,
                25264, 35356, 49992, 14516, 62344, 47848, 56860, 43932, 30336,
                63020, 19604, 45788, 29112, 8788,  59020, 38240, 13012, 34340,
                7912,  2596,  39640, 24904, 54988, 48980, 16604, 30732, 52968,
                41000, 37272, 2072,  12872, 24024, 32772, 17048, 37596, 25620,
                8160,  16272, 46764, 220,   9624,  45068, 16540, 48300, 63712,
                38552, 7208,  35412, 21828, 53808, 7480,  48744, 14704, 44532,
                29744, 50712,
            },
            {
                60280, 55044, 29068, 64824, 47480, 56060, 13932, 31568, 59984,
                17512, 55268, 48180, 40992, 27068, 49760, 22288, 53884, 17544,
                47028, 24200, 53176, 9088,  36616, 960,   63576, 25956, 47988,
                60020, 30160, 23848, 44344, 48148, 28080, 38868, 58484, 33072,
                53124, 7816,  41216, 30320, 61628, 11608, 20200, 31512, 37176,
                18188, 24592, 53672, 7908,  35520, 30528, 64920, 38104, 8876,
                16044, 39052, 32508, 44828, 8420,  48524, 13740, 35148, 7140,
                46432, 16848, 39020, 13324, 21132, 44036, 10912, 38060, 46016,
                7920,  61016, 3820,  29044, 42444, 5444,  10812, 17492, 39256,
                49900, 104,   36484, 16736, 53788, 43392, 1132,  62496, 27364,
                16388, 60500, 50136, 19036, 33148, 43756, 5756,  57952, 3280,
                25372, 49980, 62832, 30596, 59024, 46048, 1000,  61152, 29708,
                51636, 34912, 55284, 27928, 61584, 23252, 5516,  34844, 10324,
                19568, 60056, 15468, 31076, 61872, 12324, 33804, 52784, 22584,
                5584,  36808,
            },
            {
                25276, 45564, 21388, 9872,  6640,  26792, 42628, 22400, 45792,
                3268,  29132, 61140, 20976, 15228, 6556,  58900, 30148, 63304,
                7192,  34180, 19460, 48204, 42880, 17096, 40032, 4808,  19272,
                54260, 39420, 15552, 7576,  20076, 13736, 43056, 49072, 17260,
                27084, 3028,  50508, 15556, 47840, 39264, 59088, 3520,  43264,
                13160, 2416,  61804, 20944, 59828, 11292, 44224, 19624, 26152,
                58912, 51932, 11904, 18680, 65436, 24272, 54248, 2288,  60408,
                30672, 55140, 25904, 61832, 29564, 4220,  16180, 27476, 744,
                22020, 55976, 38648, 52644, 19476, 24204, 32968, 53320, 4296,
                22816, 12748, 56248, 65220, 23452, 11556, 30356, 50548, 40964,
                21644, 46776, 36312, 28936, 8460,  13572, 26980, 61360, 19840,
                14988, 8396,  53984, 20872, 5492,  39848, 52452, 22152, 43676,
                2896,  65160, 12580, 41164, 18548, 31980, 42868, 51444, 29844,
                55076, 42232, 2860,  50468, 43664, 25040, 4556,  64508, 17572,
                57360, 11676,
            },
            {
                144,   40280, 15016, 59268, 33780, 50664, 61984, 10708, 39944,
                52472, 13000, 37284, 8532,  32724, 42320, 45740, 11748, 26148,
                40176, 57352, 30700, 3180,  60096, 51208, 23424, 57744, 33964,
                10972, 1736,  62372, 57056, 36324, 63756, 8868,  24992, 4496,
                45652, 63504, 22276, 9440,  33688, 23796, 53556, 8500,  26812,
                56128, 30004, 45216, 15364, 27560, 1708,  57120, 48084, 34764,
                42452, 6532,  29860, 56944, 39604, 4972,  28400, 40756, 20312,
                11024, 36360, 1648,  51128, 41060, 57220, 47772, 51768, 33464,
                42876, 17848, 31492, 12376, 64772, 36992, 59628, 46092, 26692,
                60744, 42496, 31432, 6696,  46860, 35204, 18500, 54500, 5084,
                58096, 12176, 3984,  57080, 65524, 41552, 51164, 36656, 46240,
                33744, 28360, 43740, 48048, 11308, 27468, 14864, 57520, 10388,
                19524, 38004, 23964, 6560,  53992, 58364, 13668, 62276, 3744,
                22544, 47812, 27020, 9052,  18124, 58636, 39628, 28000, 42052,
                32188, 47976,
            },
            {
                63484, 34932, 53360, 24164, 38904, 1980,  19628, 35832, 5124,
                64568, 23428, 44400, 56604, 53120, 4276,  19376, 36504, 760,
                50820, 15200, 65440, 11576, 27656, 32436, 13760, 8464,  46996,
                28336, 42784, 49996, 26260, 2792,  31112, 51292, 60272, 35176,
                54628, 29764, 37216, 57824, 256,   65320, 17152, 35132, 50000,
                62928, 38452, 6148,  50716, 41208, 53052, 17276, 5128,  12776,
                22380, 61488, 632,   49196, 21228, 33940, 47096, 16016, 58316,
                50084, 44992, 6212,  18644, 34000, 8548,  23992, 59060, 13528,
                63260, 9968,  47440, 26284, 1272,  50608, 7272,  14020, 54864,
                34388, 9336,  51680, 26964, 40056, 4012,  59500, 25336, 9776,
                32592, 44800, 23704, 53304, 17568, 1856,  23152, 10260, 56188,
                63456, 236,   38056, 16896, 64660, 55332, 35440, 6828,  32244,
                49968, 59004, 45296, 33784, 48580, 2732,  37468, 25740, 17384,
                39412, 33844, 65032, 54296, 37544, 21156, 46988, 2924,  9592,
                51836, 19620,
            },
            {
                6460,  12940, 44116, 4100,  46832, 15788, 57464, 27592, 49428,
                31340, 18300, 1820,  27908, 63784, 24924, 48320, 61544, 55128,
                9424,  44564, 20816, 38384, 5312,  55148, 45000, 36376, 64632,
                16344, 20908, 32808, 12516, 41604, 55444, 15980, 20760, 10524,
                41852, 13480, 18720, 45016, 27932, 41356, 5144,  46672, 22360,
                10808, 19324, 34004, 58620, 22776, 37256, 32792, 63732, 28820,
                54728, 45436, 36864, 14888, 10176, 55564, 62464, 8584,  24832,
                32332, 63360, 22996, 55864, 12480, 65184, 30792, 19596, 4776,
                40008, 54660, 5716,  57208, 43904, 16520, 29312, 21472, 38748,
                2128,  19864, 15448, 61272, 21236, 49180, 14372, 43180, 36912,
                64068, 27676, 14884, 38692, 34380, 47596, 29696, 5008,  40516,
                12824, 22788, 59772, 4440,  31224, 23436, 41052, 47384, 63180,
                26628, 4096,  16652, 11096, 27456, 21420, 8752,  46824, 56232,
                11052, 5836,  14300, 32036, 596,   11872, 55400, 34152, 15456,
                59744, 26480,
            },
        },
        {
            {
                58328, 52048, 6064,  23572, 14548, 59628, 33244, 45408, 23328,
                36152, 26784, 53416, 10592, 37104, 4752,  21588, 63464, 9044,
                31028, 61916, 35452, 16220, 31488, 53848, 13720, 44124, 11076,
                5636,  51684, 37468, 55956, 23200, 42568, 1404,  35888, 13044,
                41736, 29136, 8884,  22688, 38264, 31904, 48372, 3628,  41708,
                18316, 6368,  62876, 54856, 48752, 22780, 43308, 18396, 980,
                45464, 7580,  38020, 44588, 26040, 2372,  42868, 28020, 12624,
                1916,  30228, 36452, 26016, 49864, 63064, 34864, 52432, 7736,
                64284, 40384, 17248, 61956, 33672, 7784,  60832, 15608, 46844,
                41292, 57136, 13712, 7424,  46480, 16168, 49176, 40228, 9776,
                46188, 19360, 59904, 38176, 44744, 2808,  59508, 41536, 27076,
                20544, 61180, 29152, 23744, 46176, 53828, 19000, 5768,  39792,
                52568, 20704, 26340, 56048, 32568, 23056, 59440, 35804, 2992,
                56524, 25800, 40892, 14728, 18852, 64220, 12388, 36572, 49932,
                62192, 25104,
            },
            {
                15640, 47340, 19140, 38160, 54684, 48888, 17672, 11668, 50832,
                15340, 43024, 1156,  46928, 61164, 16520, 34396, 2400,  58012,
                44464, 12476, 22168, 59156, 9804,  49468, 36696, 57896, 63624,
                40316, 18592, 27428, 8224,  64000, 30972, 53444, 27620, 21696,
                5008,  57144, 14276, 62744, 27332, 7260,  56372, 61396, 34392,
                26440, 38968, 31816, 20328, 11640, 59716, 38208, 30156, 50660,
                58132, 22276, 53808, 19220, 5036,  64196, 35328, 54272, 49508,
                44448, 22932, 59432, 3776,  11412, 30952, 14640, 55904, 26400,
                21248, 29768, 3660,  23776, 13884, 28464, 36944, 49696, 9544,
                63420, 18356, 33840, 61644, 28628, 1012,  25728, 58048, 22432,
                33480, 41896, 5688,  29492, 48632, 33836, 22656, 52604, 692,
                38524, 51468, 32988, 12308, 62524, 37804, 14332, 56564, 28264,
                68,    35024, 49968, 64060, 2356,  40776, 54260, 50432, 16904,
                21264, 37304, 62056, 28376, 44172, 55348, 31208, 41996, 4288,
                29412, 10200,
            },
            {
                34388, 27208, 65336, 3732,  41604, 26032, 504,   56788, 7308,
                64068, 21064, 55356, 29952, 40392, 56312, 51240, 14500, 37328,
                19148, 3656,  27792, 45748, 39684, 24516, 7148,  28800, 21484,
                32240, 856,   45156, 13672, 47076, 19800, 10812, 60516, 44980,
                37568, 18184, 48116, 40964, 1164,  45404, 17280, 23584, 13732,
                50696, 9536,  58284, 42068, 27164, 4692,  64460, 14628, 25936,
                4116,  63428, 10060, 30424, 56832, 39884, 16556, 21136, 9236,
                61284, 14308, 51716, 40836, 56912, 20164, 39380, 4840,  45132,
                49392, 10976, 54724, 51176, 41796, 1104,  56908, 23252, 31032,
                1948,  26612, 50344, 39840, 20900, 35540, 12580, 64832, 7044,
                50732, 14512, 61996, 11480, 19840, 55060, 8408,  16276, 62788,
                10636, 55688, 17520, 48764, 8064,  26808, 22428, 44804, 10080,
                62932, 46908, 13272, 6180,  19464, 45572, 9824,  29676, 43672,
                12600, 48908, 1560,  10268, 35132, 23560, 51720, 16504, 21284,
                54128, 38864,
            },
            {
                2916,  56984, 12628, 32432, 9380,  21148, 62688, 31620, 28084,
                39784, 33812, 5500,  12744, 23356, 7844,  26304, 47472, 29408,
                55036, 41472, 50728, 65384, 180,   18148, 61312, 46880, 15412,
                49760, 54400, 60660, 34708, 3824,  38236, 48932, 7316,  32744,
                63324, 3104,  25400, 59696, 12184, 51972, 36468, 42680, 2540,
                64824, 47556, 15264, 1652,  51620, 34168, 8740,  40704, 47920,
                32448, 36248, 42328, 49704, 13780, 24032, 47060, 5724,  26688,
                37464, 33528, 17848, 7472,  27432, 47676, 62128, 9896,  33844,
                58652, 38252, 32188, 6988,  46356, 64472, 20220, 12232, 38692,
                53948, 45372, 11256, 4380,  60028, 52216, 44492, 37976, 17424,
                54444, 24824, 36384, 57280, 4296,  27440, 40412, 45856, 35320,
                24232, 43948, 3648,  39732, 58244, 2072,  50408, 60460, 32760,
                17712, 23396, 42352, 30904, 37272, 61156, 15012, 24788, 63164,
                4880,  53116, 32824, 59520, 46976, 7824,  2788,  63028, 45300,
                8252,  60772,
            },
            {
                49608, 22264, 44084, 53616, 60240, 47028, 37256, 43700, 52760,
                10172, 48820, 58748, 19452, 65088, 32376, 43516, 10848, 63000,
                24108, 5924,  32256, 14436, 34328, 53340, 42316, 4576,  36156,
                9632,  25300, 41508, 22360, 58080, 16296, 55716, 24436, 14684,
                51108, 54412, 30828, 34664, 21556, 57496, 28608, 8864,  53928,
                20924, 30192, 24744, 36676, 45724, 17884, 53872, 21088, 56436,
                12364, 17340, 27268, 52,    59188, 32900, 51872, 62780, 42148,
                56456, 660,   46192, 65248, 32244, 2524,  51464, 24544, 18608,
                904,   15000, 62916, 25680, 17632, 34552, 52272, 43444, 58800,
                7192,  21840, 30044, 56408, 17096, 8960,  31096, 3632,  28012,
                47540, 152,   30768, 43572, 50144, 64328, 12756, 30640, 58528,
                5168,  28280, 65020, 21212, 30868, 42904, 35864, 4656,  54256,
                39260, 7880,  59176, 55484, 27188, 48728, 688,   34572, 38560,
                58592, 22604, 41696, 15352, 20268, 56380, 37148, 26564, 33252,
                13764, 41256,
            },
            {
                30440, 7484,  36512, 16812, 28836, 4596,  13568, 19340, 2192,
                16640, 25604, 36356, 44648, 50244, 1500,  57580, 4940,  39548,
                16424, 60292, 9456,  20684, 57304, 26824, 12208, 30784, 56744,
                64348, 3364,  11976, 30480, 51560, 27168, 42912, 124,   40540,
                20572, 11200, 8036,  44072, 15856, 4412,  62172, 32532, 46200,
                37800, 6100,  55984, 62740, 12804, 60752, 28372, 2520,  43944,
                6028,  61972, 45804, 20636, 8012,  37912, 11576, 3868,  30996,
                19900, 12296, 54516, 42876, 21764, 15652, 35660, 41528, 60984,
                28400, 43840, 21876, 57408, 39664, 9356,  4224,  27120, 32352,
                16548, 65500, 36368, 48200, 41984, 23420, 55480, 60852, 20080,
                40984, 63356, 8800,  21896, 16780, 37884, 1588,  48376, 18656,
                50616, 13984, 34796, 9712,  54884, 14932, 19532, 11644, 29428,
                25528, 49472, 2676,  16804, 11468, 21956, 56636, 7696,  47072,
                17756, 30612, 6220,  27416, 64556, 39980, 11896, 48408, 58664,
                24148, 18732,
            },
            {
                61936, 51692, 1712,  24668, 40708, 51396, 58468, 23636, 55256,
                63316, 40952, 3240,  15104, 28468, 38312, 18652, 22284, 48924,
                35428, 53856, 43056, 48256, 38048, 3012,  22944, 52004, 17404,
                43884, 38004, 18828, 62204, 6200,  9856,  65220, 35500, 58872,
                29240, 46856, 37360, 64696, 50040, 40112, 19044, 24560, 11780,
                59596, 16864, 43352, 3516,  32112, 23444, 49524, 37836, 58712,
                24736, 51148, 39240, 54760, 65092, 28804, 45004, 16120, 58316,
                48880, 24696, 29228, 9372,  38332, 58868, 12584, 54280, 5352,
                47524, 53028, 12068, 2412,  30168, 55560, 14224, 49360, 60600,
                2392,  40812, 13360, 26180, 1496,  33664, 14696, 45732, 11084,
                34616, 58344, 13900, 53492, 33324, 25932, 56784, 23124, 7548,
                61468, 41576, 52524, 46060, 24764, 59388, 47680, 62220, 41920,
                14520, 65148, 35152, 44136, 52968, 32496, 64392, 41308, 26408,
                11024, 55128, 44324, 49668, 1872,  31004, 18344, 5476,  53164,
                172,   46156,
            },
            {
                38612, 14404, 55948, 64204, 10688, 32012, 6672,  35716, 45568,
                30136, 8304,  61416, 54012, 11404, 60076, 33676, 52200, 12996,
                988,   30096, 25200, 7088,  59764, 44952, 62904, 34884, 7232,
                27980, 48384, 54096, 34108, 44904, 21220, 31796, 48064, 17600,
                5060,  56124, 23604, 1416,  26980, 7512,  47996, 54720, 772,
                41152, 27672, 52676, 20016, 10192, 40208, 7352,  15672, 34672,
                30392, 9672,  14288, 33552, 2908,  18584, 52900, 23308, 35488,
                40176, 6148,  60372, 52636, 4524,  26592, 45252, 30676, 21024,
                9196,  36100, 32564, 61512, 48048, 42264, 19372, 36712, 45636,
                24616, 52084, 8476,  57468, 62268, 50224, 38480, 4972,  52324,
                23888, 2756,  38792, 46268, 6156,  60652, 10488, 43680, 32064,
                37608, 26284, 492,   17956, 5856,  38548, 33208, 2328,  51876,
                6136,  20652, 30084, 8924,  39616, 3584,  18928, 13696, 52024,
                2584,  62884, 36684, 13180, 23716, 54072, 61680, 42088, 28684,
                35640, 10788,
            },
            {
                26924, 21488, 33644, 44752, 18336, 48940, 60896, 26868, 11544,
                50472, 33152, 22424, 46368, 25096, 7552,  42436, 62656, 26488,
                45480, 64488, 18476, 13936, 31948, 10544, 19664, 49452, 14836,
                58356, 23428, 968,   14144, 40076, 50292, 2696,  12264, 25912,
                62160, 41972, 13624, 52944, 31628, 60364, 35824, 14464, 63792,
                33540, 7176,  47664, 35608, 57932, 44788, 54244, 63332, 19564,
                1772,  59976, 22308, 48316, 25948, 42020, 61316, 9144,  1576,
                64040, 14676, 32552, 18484, 47296, 62728, 1528,  57040, 16948,
                64940, 40800, 6488,  15892, 23196, 26416, 63664, 6296,  10780,
                20576, 34376, 44016, 18084, 6104,  21036, 26728, 63912, 30072,
                16628, 49744, 27000, 62596, 19436, 40664, 51244, 15644, 55360,
                20508, 11600, 57028, 30216, 63968, 8940,  27768, 22548, 57212,
                37124, 46580, 55140, 61460, 23720, 50736, 45628, 29436, 57996,
                33080, 19984, 8008,  59612, 33880, 45120, 9152,  22116, 16004,
                64820, 56680,
            },
            {
                3268,  48432, 8320,  28380, 3608,  38076, 15376, 42504, 20584,
                5248,  57792, 17784, 36004, 4696,  56376, 16748, 30984, 9688,
                58248, 5616,  40544, 50620, 55772, 27344, 41368, 2008,  38932,
                32404, 9420,  61140, 26352, 56196, 16940, 60408, 53412, 39368,
                8860,  19896, 34352, 57888, 17448, 10824, 43160, 29472, 20556,
                51196, 23792, 13532, 65396, 29780, 408,   26084, 11428, 41980,
                52068, 37264, 44080, 6536,  57508, 12148, 31304, 47172, 54960,
                27264, 43480, 49936, 36308, 23232, 11128, 34144, 39060, 24420,
                49244, 27084, 51496, 58444, 38060, 3132,  52616, 57972, 30548,
                55176, 63188, 28392, 47384, 32120, 12092, 43544, 57164, 9336,
                42004, 56056, 32072, 11684, 3368,  29232, 35448, 2128,  47420,
                5052,  62244, 34264, 43124, 48856, 53456, 15952, 44908, 10760,
                26472, 17868, 1424,  13320, 27668, 59412, 5324,  36948, 22436,
                48508, 39748, 25716, 51440, 17188, 4148,  38196, 50764, 32268,
                6188,  42808,
            },
            {
                19672, 59924, 41396, 62888, 53544, 22380, 56264, 320,   63924,
                38672, 13548, 43428, 52848, 29104, 48028, 39600, 2724,  20800,
                37420, 54944, 22784, 2508,  36380, 61484, 6348,  53516, 65000,
                43272, 20108, 46740, 37316, 5356,  28816, 22132, 36288, 45832,
                30316, 50912, 3604,  44340, 22680, 38940, 2576,  57088, 45136,
                9508,  38788, 55092, 5644,  17056, 21680, 46556, 31896, 56884,
                4520,  28064, 63732, 15768, 50708, 39880, 4304,  37520, 16884,
                21292, 10668, 2880,  56396, 41240, 16256, 51356, 7020,  43648,
                13352, 144,   19972, 45236, 8964,  13072, 34032, 17256, 40292,
                4628,  14752, 332,   39184, 60968, 54460, 35332, 2204,  19188,
                37096, 6844,  21752, 43968, 53064, 59828, 24324, 64224, 27808,
                39144, 49912, 22724, 13856, 4160,  20096, 39568, 59784, 34456,
                63736, 49664, 42728, 38192, 33668, 10700, 16872, 43580, 9132,
                60868, 15148, 872,   42552, 29552, 55576, 58612, 13396, 47204,
                24996, 52424,
            },
            {
                31120, 16384, 35072, 13112, 25396, 45892, 9744,  29712, 52104,
                47884, 27336, 2068,  64876, 20356, 12468, 60804, 51000, 44216,
                15036, 32920, 47788, 29480, 16352, 45512, 21672, 25672, 12872,
                29312, 3948,  52600, 11632, 64032, 43864, 8032,  58144, 4724,
                16064, 64212, 26788, 48392, 6848,  63172, 52516, 25316, 4968,
                16508, 59848, 27012, 42480, 50656, 60724, 35956, 14076, 48524,
                23992, 17972, 10728, 35208, 29504, 19708, 24492, 59008, 33860,
                51668, 61848, 30660, 7832,  64344, 28528, 20344, 58392, 31672,
                62004, 55344, 34904, 29916, 60380, 42936, 27664, 47144, 22896,
                50492, 36456, 58512, 23820, 8820,  16252, 48864, 24892, 60496,
                45984, 64976, 15140, 36196, 48344, 17688, 8164,  13432, 45204,
                17028, 9208,  58816, 25856, 36192, 55664, 28976, 868,   7452,
                15056, 31664, 4636,  56776, 21052, 47872, 64104, 52772, 4336,
                28420, 53904, 46340, 65380, 10996, 20992, 26792, 1248,  36956,
                60824, 11592,
            },
            {
                39356, 54976, 1312,  50904, 7112,  32692, 61528, 36516, 17392,
                24228, 60032, 8684,  31976, 37944, 3844,  23520, 34236, 27616,
                52828, 11464, 62452, 8084,  58796, 11156, 33776, 56860, 17768,
                49796, 59248, 34024, 23928, 31388, 49368, 14724, 33536, 24424,
                54932, 11100, 36636, 59372, 13216, 28732, 18428, 34272, 62236,
                48588, 31220, 2284,  33872, 10404, 39456, 3488,  64252, 8336,
                39972, 53792, 45904, 60548, 728,   52704, 63072, 13492, 6744,
                40368, 46364, 25428, 14400, 44976, 53892, 2264,  47724, 9816,
                37572, 4388,  17668, 53484, 22036, 49996, 1720,  64992, 7912,
                43712, 11496, 19808, 52244, 27892, 5020,  40332, 31496, 13608,
                51304, 732,   26080, 58224, 4612,  31052, 42340, 57808, 33792,
                53756, 31372, 1660,  41012, 46128, 11648, 62320, 50976, 41136,
                21844, 53976, 25520, 62428, 7236,  39900, 30912, 25180, 35380,
                12272, 19488, 32136, 7168,  35916, 40712, 49216, 63588, 18712,
                8496,  45472,
            },
            {
                58060, 6032,  21736, 37752, 59264, 19052, 41700, 3944,  12116,
                34192, 40856, 55224, 16144, 49736, 42596, 57720, 8240,  63760,
                268,   19568, 25756, 43632, 38832, 51696, 880,   47352, 36496,
                8372,  41120, 16044, 2200,  40204, 19096, 62036, 716,   42172,
                47492, 20348, 1680,  40848, 32836, 50348, 8548,  42112, 37300,
                11500, 22564, 46672, 19184, 58364, 24636, 52752, 20532, 29156,
                58992, 33420, 6004,  21948, 42428, 8972,  48020, 44096, 28428,
                1120,  57604, 19144, 38708, 4464,  35040, 60596, 25984, 15108,
                22640, 42192, 64176, 11364, 5368,  38752, 15616, 31788, 54208,
                26088, 61712, 31224, 42276, 46276, 64480, 21640, 53840, 10652,
                28516, 33764, 40408, 9704,  23608, 55668, 38164, 21596, 6132,
                60604, 19588, 52116, 64476, 7652,  32332, 17940, 24660, 47376,
                58964, 9692,  44384, 18448, 51480, 14052, 500,   55612, 42300,
                62580, 38764, 56844, 24140, 60052, 14496, 4568,  33524, 42512,
                23168, 28760,
            },
            {
                63136, 26076, 48844, 44116, 29068, 14576, 54376, 46852, 57864,
                6664,  20808, 45836, 10968, 62696, 26436, 18544, 14320, 46252,
                40676, 55992, 35652, 3924,  18068, 24372, 30100, 61776, 4832,
                27464, 63128, 53712, 45812, 57732, 9880,  51376, 28168, 38084,
                60368, 30944, 52700, 23784, 16192, 45852, 56544, 21268, 640,
                52808, 56080, 63212, 13668, 43964, 6732,  31436, 15756, 43260,
                1696,  13056, 27048, 56488, 37188, 31424, 17352, 55848, 22556,
                11756, 32260, 52184, 59720, 23332, 11492, 30752, 40224, 53024,
                45964, 27864, 33140, 48144, 25308, 61376, 56104, 20492, 35232,
                3492,  17516, 56260, 2092,  14124, 34040, 57456, 3192,  43360,
                59496, 19844, 49268, 63140, 44504, 12588, 51528, 296,   26708,
                11236, 43896, 28668, 15884, 22076, 57680, 3360,  35316, 12920,
                30176, 37336, 2952,  34768, 28740, 45840, 58688, 17764, 22012,
                3620,  48108, 16236, 2428,  52268, 45656, 29304, 55332, 15824,
                53232, 2504,
            },
            {
                14012, 32284, 17416, 8632,  65460, 2800,  26696, 22664, 63712,
                30324, 51404, 740,   35808, 29832, 5340,  52380, 37128, 31328,
                6480,  60376, 28812, 48812, 64720, 55440, 14360, 39388, 44576,
                22888, 12396, 20252, 6216,  26568, 35372, 22384, 55468, 12636,
                17548, 8708,  43928, 5664,  64784, 3004,  26580, 61332, 15152,
                29196, 40248, 4360,  36444, 27468, 55500, 47544, 38028, 62180,
                50768, 18996, 49092, 65244, 14140, 25028, 5156,  34964, 64092,
                49660, 15968, 36328, 8436,  48404, 43372, 17444, 6676,  61700,
                1388,  56732, 7884,  16448, 35884, 41884, 9232,  13316, 45216,
                48948, 40124, 10044, 37364, 50276, 6184,  18128, 38280, 24268,
                7264,  53360, 16308, 3624,  29384, 18816, 35032, 65256, 48528,
                40340, 35908, 3032,  47796, 38588, 50084, 42320, 54912, 6416,
                65456, 16092, 52824, 60980, 11368, 24116, 37660, 8184,  50112,
                29936, 10076, 27184, 43604, 20304, 9156,  25288, 61432, 6864,
                35028, 47000,
            },
            {
                10392, 41992, 57044, 34656, 53080, 39772, 49748, 9304,  37212,
                16680, 42468, 25236, 58888, 21568, 47832, 56576, 10436, 24056,
                50252, 15868, 22068, 12456, 7216,  42060, 20592, 10112, 58948,
                32520, 50392, 37384, 30584, 65332, 14344, 3640,  45360, 6504,
                63700, 26320, 58124, 35852, 54532, 38680, 11900, 32336, 44352,
                7808,  25108, 17200, 50748, 9396,  59804, 2836,  11052, 23080,
                55252, 35940, 9812,  40116, 3224,  60340, 53836, 38808, 9424,
                42220, 61404, 2776,  26956, 55700, 65392, 21324, 49780, 36060,
                12812, 20036, 51644, 58900, 29796, 620,   52032, 28016, 59224,
                6424,  62820, 29148, 22840, 60788, 26300, 44640, 63484, 14624,
                35872, 46572, 31880, 39020, 60040, 7496,  45924, 15136, 23300,
                55544, 62472, 13300, 24916, 60948, 9840,  27356, 19860, 45584,
                39384, 26264, 21252, 41912, 48852, 5660,  64740, 33000, 44284,
                61160, 54040, 35288, 64272, 32528, 58408, 39468, 11964, 50508,
                20748, 38024,
            },
            {
                59052, 51216, 524,   23828, 12564, 20168, 32180, 44408, 13832,
                56060, 6024,  62164, 12728, 39112, 3036,  33416, 65372, 44376,
                2320,  38152, 53488, 45556, 34856, 31296, 52500, 3048,  48284,
                16740, 57172, 236,   54540, 42748, 48180, 60008, 37048, 31912,
                40572, 48832, 13892, 21432, 29484, 18968, 48360, 59556, 51436,
                35692, 65104, 45780, 32464, 21616, 41208, 18160, 34752, 26380,
                5408,  44940, 29600, 20932, 47608, 43504, 28044, 18768, 4232,
                24428, 30292, 19980, 40528, 13928, 33568, 3180,  58116, 25612,
                31536, 43992, 39008, 4648,  46484, 22472, 64268, 38144, 24016,
                33372, 19524, 14872, 53316, 32732, 7764,  48480, 30012, 54940,
                1136,  61568, 11584, 21404, 56552, 26948, 53124, 32356, 4988,
                18244, 8484,  33356, 54584, 4856,  31136, 14900, 58532, 72,
                50540, 8504,  56496, 1884,  31572, 19692, 52416, 15696, 12544,
                24860, 6236,  41356, 13292, 49396, 300,   18352, 31484, 43332,
                64772, 24896,
            },
            {
                18588, 6768,  28204, 47656, 61740, 5740,  58860, 1476,  52332,
                33760, 27948, 46768, 18192, 54204, 43084, 15612, 27916, 20332,
                59500, 29944, 9172,  61380, 1224,  26736, 63268, 35920, 25196,
                6796,  41300, 10984, 24064, 17432, 8288,  29320, 19508, 53308,
                23176, 4136,  33896, 51544, 9668,  42348, 6460,  23516, 3344,
                19716, 11004, 56960, 1580,  61772, 29556, 52180, 64444, 42304,
                15236, 58216, 48,    53188, 16204, 32540, 12032, 59196, 48460,
                44896, 56976, 52768, 5732,  46232, 28624, 10284, 38304, 15712,
                63868, 9300,  23732, 61176, 13564, 18680, 43512, 3068,  11300,
                50924, 41560, 46208, 700,   56820, 11988, 40972, 20272, 9452,
                27712, 42084, 24840, 49948, 1732,  42812, 10444, 38476, 59344,
                30080, 50792, 45268, 20416, 37092, 44248, 63948, 34648, 23116,
                29772, 61912, 33796, 14648, 60160, 25816, 40920, 57544, 1232,
                39092, 59340, 17380, 22844, 5048,  36476, 46676, 56924, 6408,
                3208,  29980,
            },
            {
                40148, 62644, 36024, 15368, 42664, 37824, 29396, 24384, 61144,
                19852, 40120, 2292,  31568, 9332,  24808, 60676, 7028,  12820,
                55072, 41532, 17908, 23632, 57480, 15284, 43760, 18708, 60228,
                29220, 46624, 61496, 33276, 51792, 39836, 25440, 1316,  57248,
                11048, 61288, 45728, 348,   62968, 57780, 16016, 37668, 54252,
                30288, 41360, 49044, 15588, 38520, 5928,  13160, 48388, 31516,
                8780,  61108, 33740, 24208, 62960, 7016,  51332, 22084, 34308,
                14664, 820,   37596, 63284, 23524, 51744, 41760, 53932, 46840,
                5280,  54992, 28572, 49180, 34516, 57576, 31676, 55096, 17184,
                57940, 4248,  65496, 25668, 37768, 17112, 59752, 33532, 50600,
                57500, 15832, 5756,  64072, 36508, 14220, 20616, 63180, 2272,
                42244, 26360, 58228, 1612,  16668, 51444, 6000,  12052, 48296,
                17344, 43076, 11104, 46920, 36720, 4156,  45544, 27904, 34940,
                20392, 47444, 30696, 56348, 52056, 63340, 28032, 23652, 16360,
                48396, 54876,
            },
            {
                13232, 44612, 20496, 56652, 8784,  54404, 17604, 46016, 11528,
                8056,  50552, 58196, 64232, 37020, 51936, 47552, 40016, 35036,
                48468, 3840,  33136, 51236, 37016, 4792,  49988, 8760,  53956,
                38640, 14700, 20824, 3496,  58336, 12908, 62784, 50028, 15840,
                42952, 28112, 18364, 39628, 25344, 33020, 46584, 27776, 62660,
                13500, 6728,  23188, 27144, 54676, 45128, 24672, 2440,  56172,
                21424, 39120, 46936, 10912, 37288, 55820, 1980,  40264, 64880,
                26684, 10816, 32592, 17472, 6940,  59244, 18936, 356,   21800,
                33424, 42196, 17016, 2312,  37796, 10412, 7252,  47252, 27192,
                36348, 30740, 9148,  43988, 21900, 52252, 2828,  5440,  23292,
                44796, 38328, 32456, 46916, 29564, 55320, 51272, 24700, 47520,
                15572, 11264, 38944, 62188, 28336, 24128, 41524, 56320, 38188,
                3596,  55244, 24012, 52936, 18188, 8044,  55792, 10600, 63548,
                51400, 3176,  9796,  26092, 42572, 7756,  12340, 40888, 60224,
                33912, 9664,
            },
            {
                53388, 31352, 2152,  24812, 33304, 49416, 4152,  63672, 35304,
                43340, 26188, 22512, 13604, 4132,  20692, 30200, 468,   58520,
                16080, 26528, 62592, 11132, 46924, 40760, 22452, 12892, 33788,
                2252,  64164, 27484, 36908, 44812, 5240,  35408, 46056, 32480,
                6636,  37232, 56072, 14556, 5084,  53652, 10684, 2484,  43412,
                58076, 32924, 60880, 35508, 9756,  20032, 58792, 37216, 16856,
                27908, 50884, 4084,  19224, 27104, 42608, 30196, 17788, 7632,
                53940, 58156, 42416, 49576, 12432, 35788, 31068, 62200, 26672,
                57464, 12288, 51476, 62936, 26356, 22120, 52804, 40816, 14328,
                60728, 20460, 50076, 13324, 29400, 61624, 47084, 36376, 64756,
                12236, 19308, 58856, 8968,  17324, 3876,  33864, 6340,  57768,
                35760, 22776, 6660,  47184, 54020, 10196, 60220, 31524, 20660,
                63280, 6928,  28648, 39796, 64256, 30920, 48976, 23316, 16704,
                32388, 44632, 60888, 38156, 15800, 33348, 21020, 50740, 1844,
                37584, 25736,
            },
            {
                4916,  59380, 47148, 64896, 12160, 27752, 21624, 39624, 55436,
                16024, 5588,  48288, 33972, 44132, 16964, 62964, 10892, 23780,
                52756, 44088, 21144, 5876,  28196, 64500, 31144, 56476, 24584,
                42844, 50756, 9184,  55868, 22900, 29972, 18664, 9436,  59572,
                20840, 52316, 64984, 31280, 48456, 22080, 35840, 17484, 50640,
                21176, 47012, 680,   51112, 42616, 63104, 32756, 7388,  53044,
                43576, 12856, 64700, 57260, 45540, 14304, 61548, 50288, 36104,
                46012, 21128, 29076, 61880, 24648, 44496, 8836,  14516, 48320,
                36628, 7720,  20444, 39908, 59212, 44740, 836,   64008, 24500,
                5160,  39648, 55968, 34364, 6720,  40532, 26100, 15456, 54092,
                29008, 2360,  53016, 22876, 62072, 41392, 45292, 28644, 18636,
                50064, 64768, 30884, 14084, 33876, 18752, 1064,  46280, 26348,
                15084, 34868, 50636, 2848,  21372, 13140, 37928, 43320, 4756,
                57836, 13488, 21824, 53784, 1088,  62048, 55388, 29356, 45848,
                64132, 21916,
            },
            {
                51556, 16924, 38300, 7068,  41408, 52696, 14800, 612,   31344,
                60400, 29000, 53192, 57268, 38996, 26768, 54408, 46108, 32032,
                8200,  38556, 56756, 14448, 53592, 19088, 632,   45968, 59428,
                5100,  19388, 47316, 16056, 41192, 53568, 64292, 26700, 40748,
                2012,  24388, 11980, 7836,  42176, 58956, 63788, 40288, 25868,
                8580,  38712, 18420, 14552, 4912,  29012, 12268, 47304, 61888,
                1036,  35200, 25116, 8080,  32416, 5040,  22984, 10108, 25516,
                3452,  15792, 5992,  39108, 1776,  56432, 51276, 40576, 3980,
                65084, 45104, 32080, 3684,  15020, 30488, 19152, 48964, 33708,
                11588, 46516, 1800,  63728, 18524, 57788, 9976,  31888, 42840,
                7812,  48092, 39504, 35136, 26456, 10608, 60276, 13156, 40144,
                8856,  55016, 43660, 4376,  40796, 50212, 36796, 8428,  53692,
                41144, 59148, 10156, 44628, 56748, 60808, 368,   53628, 29300,
                40268, 7288,  35344, 28596, 48584, 44068, 10532, 18628, 6832,
                13992, 43040,
            },
            {
                35588, 10360, 29032, 19404, 60848, 34812, 57992, 46820, 9560,
                38140, 18704, 1964,  11948, 7988,  60268, 5092,  36744, 19424,
                64960, 1784,  29840, 35836, 60248, 8512,  39832, 16644, 35960,
                11124, 29336, 32800, 61600, 7368,  2824,  49156, 13888, 55552,
                46692, 35784, 50316, 27060, 19272, 1032,  29096, 13084, 4712,
                56492, 30848, 63980, 54008, 25388, 57112, 40108, 19032, 23340,
                30568, 41468, 15744, 49580, 38032, 59736, 47244, 55404, 39588,
                63660, 31512, 52556, 47604, 19264, 27588, 60956, 22460, 17328,
                29048, 24612, 53480, 47628, 11016, 54936, 37008, 8652,  59504,
                28292, 54228, 16052, 24004, 45176, 50684, 164,   21008, 56244,
                24900, 63004, 14588, 50524, 1264,  56368, 21760, 51820, 32228,
                276,   26624, 20956, 61196, 57140, 22588, 64208, 29632, 13052,
                48692, 18612, 25244, 32884, 15764, 26904, 35164, 19392, 62736,
                25628, 50632, 65228, 18920, 4624,  25012, 37100, 59548, 32664,
                56956, 27008,
            },
            {
                132,   55692, 44780, 49920, 23528, 4852,  43144, 25780, 50856,
                22104, 65208, 41420, 31916, 47264, 22800, 15004, 42624, 51036,
                13208, 41164, 49640, 24412, 44912, 33384, 52164, 26628, 63100,
                53496, 39040, 57176, 25176, 12360, 34028, 38124, 22088, 31048,
                4776,  16936, 61708, 38616, 45264, 55724, 33380, 52172, 60612,
                44728, 11660, 36736, 21812, 46384, 33996, 3312,  50384, 10372,
                55364, 59308, 21216, 53664, 28912, 1620,  20064, 33648, 12772,
                43748, 58544, 9584,  35348, 13836, 32348, 2996,  37920, 55412,
                9680,  58736, 5652,  35636, 61720, 27472, 43424, 16824, 51640,
                21796, 42556, 32076, 37576, 27804, 12484, 34800, 60184, 37756,
                45504, 3708,  20408, 30836, 43816, 16544, 37368, 5576,  63236,
                46216, 36596, 17052, 11176, 27640, 2840,  16076, 44196, 5720,
                62104, 2144,  37348, 47392, 6604,  41016, 49600, 11916, 5708,
                45088, 15512, 9580,  57084, 41612, 12964, 53084, 2568,  40388,
                49232, 19724,
            },
            {
                31024, 60024, 14568, 3256,  32264, 10924, 17920, 61648, 6328,
                13648, 45340, 25144, 58052, 51764, 35600, 29544, 61424, 3564,
                27568, 59300, 17568, 4632,  11872, 20976, 3396,  47972, 13772,
                6600,  21184, 1152,  42672, 51728, 58784, 17932, 43456, 60884,
                10388, 57024, 32764, 5776,  15208, 9960,  47772, 21364, 16616,
                27204, 49628, 2608,  41448, 8920,  16528, 65184, 26972, 36792,
                5332,  45280, 3124,  11816, 62312, 42064, 16792, 51632, 27512,
                272,   18004, 22792, 54388, 64672, 42700, 11356, 49292, 34316,
                43536, 15492, 19500, 41352, 22716, 3168,  56772, 6196,  38960,
                2516,  62356, 9252,  4652,  61088, 53452, 41108, 5224,  17316,
                11376, 33496, 57584, 8256,  65316, 47916, 28040, 54688, 24332,
                14600, 58296, 48980, 52480, 38408, 32872, 54708, 24580, 38980,
                31872, 55808, 21788, 65096, 52348, 9248,  58348, 22644, 32768,
                55812, 37436, 23836, 46932, 33756, 30312, 63204, 15920, 23448,
                8488,  63648,
            },
            {
                42028, 24588, 39300, 53960, 64384, 37828, 56344, 29328, 33868,
                54324, 36392, 4216,  16312, 20180, 892,   10096, 55684, 23944,
                34044, 9120,  46760, 63900, 55172, 42156, 61572, 30280, 23508,
                44236, 49596, 64608, 15100, 27828, 46488, 8380,  120,   52108,
                28888, 41304, 22860, 53212, 63416, 25052, 37628, 2900,  42532,
                34496, 6836,  61764, 29448, 58672, 52624, 43064, 13880, 60528,
                32976, 18172, 39404, 26256, 35000, 9064,  56856, 5676,  61080,
                36540, 49184, 41000, 25952, 6712,  46164, 20404, 59960, 4904,
                52012, 31320, 63944, 48748, 12844, 33196, 24664, 65004, 30960,
                14596, 47232, 57964, 20284, 48068, 15604, 23216, 30052, 49404,
                61500, 27444, 52220, 41924, 12680, 4512,  33992, 19552, 9972,
                41024, 2512,  30636, 6852,  42564, 9408,  48140, 61064, 19744,
                51664, 12312, 28776, 4640,  17696, 30948, 44244, 61756, 2704,
                48808, 29692, 3712,  59112, 6952,  20436, 50372, 27692, 46604,
                36472, 12736,
            },
            {
                5932,  47640, 9088,  20884, 27604, 47240, 14976, 41092, 2124,
                48428, 10632, 61200, 28148, 43104, 63708, 48616, 39160, 44824,
                20312, 53112, 37412, 31660, 27132, 15988, 36432, 9412,  58924,
                33600, 18012, 36964, 3880,  22560, 31836, 62920, 35948, 25580,
                13536, 48564, 18856, 1640,  43164, 30148, 58660, 8112,  64948,
                23640, 56096, 19056, 13136, 24244, 392,   38336, 21016, 47064,
                8396,  52112, 63852, 48188, 22004, 44488, 31336, 24172, 45916,
                8676,  29892, 62484, 2656,  16572, 57220, 30468, 23976, 13556,
                26224, 1824,  38564, 8144,  57816, 52572, 44760, 18972, 49336,
                35696, 25276, 11944, 39336, 33448, 2016,  64936, 43184, 54316,
                6260,  22552, 36232, 18312, 25468, 39892, 60492, 49784, 44700,
                61520, 34888, 22172, 65048, 19160, 59352, 13820, 35212, 440,
                8916,  42136, 45816, 59692, 39500, 24960, 14040, 36616, 18404,
                42080, 13148, 62360, 17120, 38496, 54776, 9964,  42648, 4324,
                58640, 52796,
            },
            {
                28452, 17156, 61748, 35832, 1132,  51812, 7156,  23068, 62832,
                19712, 30968, 40012, 7348,  53552, 12876, 31504, 17008, 7040,
                58160, 14248, 148,   22440, 6772,  50812, 56600, 1508,  40184,
                54056, 10820, 29088, 56176, 40660, 11580, 54240, 20380, 44532,
                65428, 7540,  59104, 35416, 12712, 50404, 19484, 53788, 39864,
                14528, 51672, 47124, 36124, 32220, 49128, 6060,  30512, 56556,
                23900, 29192, 14720, 6556,  54384, 13236, 65412, 39176, 15404,
                20908, 56236, 12932, 38212, 51772, 34872, 8872,  41252, 63240,
                45764, 54668, 21816, 28956, 16444, 240,   39916, 10524, 6980,
                55920, 28892, 44156, 52296, 7992,  56516, 26112, 9512,  13776,
                39172, 47196, 2148,  62464, 54988, 30464, 996,   15504, 7460,
                28744, 53908, 12572, 45800, 25848, 3204,  29188, 44960, 23020,
                57716, 26572, 15384, 34436, 53476, 900,   57388, 27984, 7864,
                54212, 21376, 51380, 26516, 44372, 812,   34880, 60928, 18564,
                32360, 21880,
            },
            {
                50048, 57800, 31528, 45024, 11972, 19256, 59216, 43704, 26616,
                55860, 14280, 50092, 59552, 21652, 36692, 25864, 4196,  62700,
                28716, 49032, 42508, 59040, 39432, 11204, 46300, 20208, 25996,
                4892,  61868, 47248, 7692,  59884, 48500, 4976,  16212, 38036,
                3252,  32004, 39976, 26364, 46544, 4804,  33888, 11556, 27648,
                31308, 1380,  43252, 10272, 60892, 55564, 17368, 63400, 11488,
                2780,  43376, 37220, 58056, 2024,  28348, 49964, 3540,  59484,
                52744, 4364,  32712, 47852, 27696, 61576, 608,   49992, 17400,
                36608, 10888, 59460, 42332, 35180, 63012, 26932, 51268, 59876,
                22052, 1624,  63992, 17752, 21636, 30980, 45612, 35668, 19848,
                29176, 58852, 15820, 10252, 46124, 22872, 51656, 38592, 57552,
                24920, 17592, 5196,  36252, 57072, 51932, 40652, 16636, 64128,
                49476, 37040, 3692,  63052, 10288, 20696, 50036, 46548, 63848,
                34048, 1708,  40008, 32052, 12144, 64412, 25220, 14644, 55936,
                38796, 1616,
            },
            {
                13864, 41064, 4348,  25304, 56016, 39052, 32580, 9600,  36648,
                5464,  46044, 24456, 33628, 3052,  45608, 56716, 50820, 40596,
                34680, 10320, 25148, 63472, 18552, 33184, 28940, 65292, 49860,
                16400, 31696, 42940, 24592, 19084, 33772, 27320, 51056, 58068,
                23700, 54852, 9884,  62224, 16656, 56796, 23108, 60336, 45304,
                5496,  62548, 21600, 39136, 7592,  25988, 45124, 34780, 39956,
                48848, 59972, 20292, 33096, 41656, 17872, 36708, 10164, 33808,
                41864, 24448, 43976, 18976, 10432, 22124, 38780, 55976, 28320,
                6500,  32608, 49504, 5136,  46536, 21276, 13716, 32772, 38092,
                16136, 41648, 5828,  36776, 59800, 49960, 4228,  62240, 55316,
                3184,  50664, 32652, 43156, 7024,  58256, 12284, 20184, 32476,
                43472, 63276, 39556, 48652, 31840, 11392, 5812,  55124, 32292,
                6812,  19076, 47120, 23492, 41620, 32224, 5976,  38508, 11712,
                23384, 59684, 9008,  57032, 46476, 20960, 49284, 29868, 7444,
                45560, 65464,
            },
            {
                34196, 8568,  53584, 15584, 64012, 29524, 49560, 17264, 53032,
                64816, 476,   18100, 38900, 9116,  64368, 19080, 11708, 22236,
                55080, 16464, 3372,  44564, 53300, 14008, 3852,  43432, 35216,
                12792, 21704, 52344, 2548,  14072, 39132, 62252, 8776,  29924,
                45940, 15344, 21104, 52192, 28860, 37392, 2044,  48920, 17596,
                35492, 57688, 15636, 28632, 53880, 20052, 13636, 4672,  27080,
                53200, 16768, 8796,  25712, 51296, 62020, 21592, 47568, 27128,
                16892, 63636, 6816,  59912, 53632, 14620, 45352, 4188,  20120,
                60668, 14240, 25364, 18632, 56292, 8992,  60972, 3532,  45988,
                29736, 53504, 48648, 26828, 14296, 40312, 11180, 16920, 41624,
                24980, 64144, 21540, 37892, 27376, 34876, 64704, 48228, 3392,
                10520, 51120, 596,   15788, 23668, 62492, 20520, 37580, 27816,
                12816, 52880, 58636, 29284, 13504, 56144, 60572, 26128, 16132,
                44772, 49080, 28284, 16488, 6248,  37280, 3472,  41668, 51880,
                10740, 26196,
            },
            {
                19380, 44180, 36860, 22184, 46796, 6404,  2620,  60564, 12912,
                28344, 41732, 51900, 58416, 15528, 29376, 43324, 33120, 1804,
                46532, 62052, 30916, 36000, 6080,  57336, 23148, 60344, 8560,
                41380, 58768, 37224, 64408, 45080, 54524, 1220,  42676, 12652,
                35684, 49436, 588,   41704, 7116,  44056, 64492, 9596,  40464,
                52776, 24036, 49572, 4004,  64860, 41036, 51008, 60500, 21644,
                32132, 748,   63148, 46428, 11868, 5064,  55800, 1056,  58280,
                12464, 50488, 36180, 2032,  31636, 25460, 64304, 33748, 42772,
                52256, 39672, 65368, 2308,  53040, 28416, 41004, 24496, 57380,
                12292, 61236, 9628,  33652, 400,   57664, 22952, 52820, 31744,
                8616,  45184, 12532, 1644,  54148, 16320, 5252,  25688, 37524,
                56596, 21316, 29660, 60192, 8768,  43892, 47436, 1320,  59832,
                43288, 9604,  34136, 1564,  50980, 17824, 43588, 2288,  31012,
                53604, 4580,  34536, 62828, 23932, 54592, 60320, 17736, 35736,
                22804, 58908,
            },
            {
                30020, 49688, 396,   59536, 10916, 41852, 24568, 34568, 45140,
                22040, 31732, 11184, 25740, 48272, 54372, 5784,  60820, 26744,
                37592, 7600,  50228, 20896, 26916, 47368, 38380, 51500, 568,
                54652, 27580, 6008,  10556, 30672, 25788, 17452, 21960, 59688,
                4272,  64192, 33024, 56136, 25016, 14200, 31060, 20480, 26540,
                6824,  12508, 33432, 45876, 31180, 1996,  36556, 10504, 43116,
                56780, 13940, 35620, 54756, 30432, 40028, 24576, 44764, 31156,
                39032, 8816,  28780, 47104, 40488, 56628, 7940,  12140, 23320,
                47528, 9272,  30844, 37136, 43984, 34092, 49104, 17928, 7476,
                35512, 19812, 23832, 63176, 44704, 29108, 37548, 47944, 5264,
                35076, 56580, 18776, 49460, 62084, 41364, 31216, 44596, 61008,
                14180, 34260, 41776, 52672, 26480, 34860, 14592, 50800, 22596,
                39760, 25336, 64620, 45388, 38132, 8292,  35272, 20424, 65300,
                40728, 10384, 19816, 42752, 50480, 31768, 12568, 27548, 63044,
                2300,  54888,
            },
            {
                6692,  63404, 17572, 33056, 27848, 52540, 19408, 57696, 38512,
                6788,  56420, 62000, 36400, 2116,  22868, 39460, 14372, 52644,
                18684, 58088, 12960, 41236, 61224, 9728,  15080, 31040, 25328,
                18280, 33920, 46896, 20564, 50680, 36324, 56352, 48252, 40304,
                27820, 17860, 38848, 11876, 61100, 51540, 34584, 58840, 55276,
                43192, 63252, 38576, 9480,  18052, 57392, 24608, 47776, 7396,
                28144, 39080, 23164, 43716, 6920,  15876, 34624, 60748, 18780,
                53348, 23288, 57504, 20756, 16592, 49556, 37540, 29728, 54952,
                3524,  16900, 57704, 22628, 11436, 15028, 808,   63884, 51960,
                43448, 3804,  47400, 15960, 54060, 7428,  19208, 59576, 27160,
                14428, 61124, 39524, 29292, 23788, 10716, 52544, 18080, 8468,
                27688, 47312, 6400,  18844, 58064, 4076,  30684, 63656, 17856,
                56256, 5548,  15812, 21244, 27540, 61744, 52720, 47336, 14160,
                58016, 36760, 55988, 13768, 208,   39252, 9084,  44652, 47948,
                15620, 38580,
            },
            {
                12828, 23988, 54196, 39764, 13872, 62416, 48724, 8944,  15896,
                47624, 3936,  19492, 42636, 8368,  59236, 30500, 49212, 9448,
                42944, 23920, 29460, 55504, 1600,  33588, 20228, 64056, 49004,
                12256, 56664, 61120, 15320, 3920,  63124, 7012,  10232, 31780,
                53728, 24204, 8160,  45648, 18836, 3284,  47980, 5032,  16328,
                29708, 184,   59664, 22520, 51868, 14540, 62248, 33924, 18804,
                63940, 49480, 3552,  59468, 20560, 64396, 51604, 13516, 6224,
                2452,  65036, 43684, 13248, 5188,  62792, 924,   18540, 59252,
                35248, 27544, 46116, 5920,  61252, 55240, 21180, 37640, 30120,
                26064, 55492, 38852, 31340, 41268, 12780, 65520, 1936,  43468,
                51628, 22620, 3764,  7796,  47140, 59184, 952,   22148, 50236,
                55416, 2416,  65524, 12988, 38072, 54600, 41052, 10492, 3172,
                36032, 32420, 48956, 54484, 3020,  12064, 24776, 5208,  29512,
                23180, 2772,  26712, 46680, 29672, 58376, 20688, 53396, 4400,
                32144, 42996,
            },
            {
                26964, 47088, 9240,  44712, 4876,  36388, 2168,  26308, 63796,
                33492, 51064, 13456, 27292, 46440, 34000, 20112, 62860, 412,
                34596, 64916, 4760,  45660, 17836, 52992, 40404, 44800, 6808,
                37704, 2344,  40284, 43728, 23796, 33268, 45900, 15860, 61876,
                5236,  49756, 58724, 29576, 36644, 22636, 40132, 11080, 45420,
                36108, 19668, 48208, 27524, 42292, 5916,  29976, 44472, 1512,
                52468, 11060, 17420, 29348, 48332, 9124,  27248, 37156, 41768,
                48984, 33284, 9928,  35612, 27988, 52376, 24732, 44328, 49084,
                13552, 63604, 41500, 51388, 25884, 32328, 45396, 9476,  60152,
                13692, 6448,  59132, 2380,  21460, 49532, 25020, 36096, 16716,
                9872,  31288, 54864, 36412, 15520, 33044, 39180, 63644, 35772,
                40324, 24556, 32168, 45600, 8000,  24104, 48324, 28512, 44892,
                52508, 62016, 9076,  30048, 40948, 59116, 33592, 39448, 49916,
                8416,  60872, 52088, 17024, 63372, 5600,  35308, 25284, 59952,
                19580, 57036,
            },
            {
                34080, 3200,  58452, 29212, 22376, 56332, 31204, 43552, 21308,
                40744, 29668, 55380, 65348, 16888, 52932, 11448, 44852, 27956,
                16084, 39076, 49568, 12076, 36972, 62588, 4040,  28312, 58176,
                22684, 31828, 51840, 28756, 12724, 58848, 38040, 26392, 43452,
                21172, 14120, 41900, 2104,  63412, 54604, 28124, 59900, 24708,
                52400, 13188, 56640, 4100,  37172, 55032, 11936, 21416, 58588,
                36876, 25348, 41548, 54216, 33016, 76,    46100, 56604, 20040,
                29884, 25572, 59488, 46464, 41100, 60592, 32208, 10136, 39056,
                21940, 1448,  8772,  19232, 38572, 4200,  50068, 16796, 41640,
                22732, 34116, 51164, 27684, 10564, 56100, 45676, 30016, 57620,
                40276, 47712, 64236, 42656, 26376, 57348, 12708, 28916, 4436,
                15964, 43616, 19904, 51704, 61968, 16348, 20712, 60156, 13308,
                25992, 19368, 43496, 14384, 56508, 45700, 18580, 62760, 16000,
                44400, 31808, 41140, 10852, 22568, 43636, 49344, 14432, 40160,
                10260, 51520,
            },
            {
                21464, 37284, 64596, 18716, 50848, 15400, 61136, 11792, 54528,
                768,   9576,  23416, 38464, 2612,  6192,  40900, 22528, 54116,
                57756, 7360,  20512, 26372, 30896, 8764,  24440, 14352, 53668,
                10072, 16928, 65136, 8060,  55488, 3156,  19104, 52520, 784,
                35972, 55764, 32656, 9828,  50868, 14636, 7064,  33300, 64312,
                2956,  40492, 32148, 17012, 60816, 25088, 49616, 39852, 15452,
                31332, 4572,  61696, 12940, 38492, 58752, 24044, 10992, 62544,
                15212, 55580, 6964,  17852, 2716,  21352, 14964, 5868,  61960,
                53564, 30656, 34148, 56684, 13020, 64692, 27964, 56340, 2960,
                48464, 64308, 18376, 37260, 62328, 32888, 4316,  8124,  61628,
                20880, 528,   12072, 18912, 6140,  44732, 20604, 47616, 9284,
                56872, 59984, 10884, 26996, 36916, 2736,  34068, 57092, 7340,
                38540, 80,    23620, 36624, 5712,  22192, 9900,  864,   55052,
                35664, 20352, 3800,  37740, 55728, 32912, 8128,  65024, 29268,
                912,   61988,
            },
            {
                49012, 7556,  12324, 41284, 33332, 46648, 5304,  37480, 18284,
                60176, 44884, 35396, 58716, 49592, 31812, 61348, 36372, 3912,
                47280, 32096, 59948, 51796, 56200, 42492, 46804, 59672, 35644,
                41436, 48084, 26176, 20864, 39308, 50072, 29596, 64264, 10920,
                47612, 61032, 17512, 25508, 46756, 21240, 42984, 18032, 49420,
                9888,  23440, 62436, 7544,  34356, 45500, 2692,  65420, 9020,
                47324, 56452, 44556, 6376,  22020, 15784, 43216, 5552,  34416,
                3248,  50908, 22752, 37064, 54188, 48224, 57740, 34920, 26692,
                43336, 16632, 60416, 47016, 23008, 43224, 7976,  36200, 31900,
                11804, 45316, 7048,  15168, 42900, 52428, 19528, 13824, 48996,
                28132, 34504, 24480, 59920, 53748, 2256,  51168, 61516, 23592,
                34020, 49400, 664,   29948, 55608, 6160,  50512, 42376, 30800,
                65008, 47572, 51612, 60692, 32028, 48816, 28224, 51780, 25228,
                12376, 64564, 48576, 59064, 25812, 1716,  18616, 54800, 36120,
                42368, 16304,
            },
            {
                31140, 43788, 55300, 1144,  25452, 9496,  53236, 28728, 50044,
                25608, 14960, 6696,  19912, 12880, 24888, 56480, 18208, 13580,
                25432, 10508, 41076, 3560,  15120, 21664, 1256,  32440, 19648,
                4936,  61752, 128,   34620, 46764, 16216, 5996,  33636, 40824,
                23096, 28356, 4964,  40000, 35060, 57424, 1284,  38224, 56148,
                31036, 47328, 43348, 53744, 11064, 20068, 28768, 53052, 34932,
                22692, 27452, 18228, 35976, 51768, 63880, 31228, 53852, 47752,
                38776, 44256, 11408, 63772, 29572, 9204,  39860, 18668, 50836,
                4312,  11548, 37052, 28576, 1956,  18136, 52352, 24524, 61372,
                20048, 40336, 25696, 57256, 796,   23280, 39952, 36580, 54456,
                5128,  43932, 50152, 38876, 29640, 35596, 16808, 41096, 31044,
                5628,  38444, 15044, 63780, 45272, 39936, 24936, 9988,  15472,
                54780, 4392,  27624, 11484, 16620, 64080, 37164, 40832, 57684,
                30720, 6272,  15564, 28932, 45432, 13332, 51352, 22144, 46236,
                5216,  24964,
            },
            {
                52692, 14396, 27996, 59932, 39012, 63876, 22968, 42256, 3460,
                32636, 63516, 53844, 47904, 42704, 29848, 9032,  50780, 43364,
                62604, 34420, 18512, 63804, 36332, 48772, 61280, 11068, 50956,
                29888, 44204, 57052, 11884, 59956, 42708, 25240, 55008, 13920,
                58488, 8464,  52764, 65200, 12472, 30144, 61148, 26368, 13700,
                4812,  20700, 27736, 14972, 38652, 56980, 6568,  17364, 41400,
                844,   63020, 10064, 48008, 28840, 2332,  40684, 25448, 17536,
                60624, 27216, 32796, 14564, 42784, 25272, 264,   65100, 45460,
                22652, 55144, 7320,  49068, 62388, 39412, 11096, 58644, 1240,
                50716, 28984, 54048, 9788,  30700, 61004, 45368, 26280, 64776,
                17612, 56904, 14692, 10120, 7240,  63808, 26884, 11208, 54508,
                19344, 43136, 53228, 21756, 11620, 18292, 61564, 47040, 22488,
                33548, 18516, 40420, 57148, 1992,  44200, 7744,  19304, 3428,
                46664, 23264, 53536, 9164,  34928, 62516, 39232, 27932, 11372,
                56704, 61152,
            },
            {
                3768,  35048, 20668, 48544, 17276, 6268,  35720, 58600, 12280,
                46452, 39420, 27920, 1272,  34600, 64216, 3080,  38844, 28660,
                53752, 1108,  45160, 24224, 6992,  28136, 39204, 17032, 55048,
                7800,  37952, 15416, 23684, 31252, 9276,  61632, 20316, 45524,
                1608,  31072, 43416, 19684, 3484,  51516, 9112,  44684, 53272,
                36832, 58988, 636,   51232, 63336, 24100, 31940, 60708, 49884,
                14112, 53824, 33272, 59120, 20152, 8040,  57728, 12164, 21340,
                8912,  1172,  58676, 52124, 6516,  56892, 33292, 13740, 30240,
                59704, 40592, 25844, 15216, 54108, 33312, 29720, 45992, 16036,
                37692, 4764,  63552, 35472, 49160, 16732, 11672, 3244,  8988,
                31264, 22720, 61440, 32948, 47484, 21648, 57064, 3940,  46684,
                62272, 8700,  26000, 35780, 32084, 57548, 1752,  29180, 37356,
                59328, 8332,  46196, 34900, 24136, 54368, 13784, 33100, 62184,
                43108, 38096, 61072, 19856, 42108, 4032,  58388, 7084,  33156,
                18248, 38432,
            },
            {
                8956,  45788, 57596, 10792, 31020, 44420, 14668, 20828, 55328,
                8020,  17332, 21944, 10336, 57912, 16324, 46020, 20964, 7536,
                15224, 58236, 30372, 50184, 13024, 57236, 43724, 25592, 64076,
                21312, 27480, 50328, 4276,  53812, 36560, 2748,  50784, 38424,
                17108, 35584, 49100, 59724, 24728, 39044, 16932, 22348, 33956,
                64576, 17844, 41072, 30224, 5436,  47836, 44108, 10648, 37832,
                25732, 43300, 4212,  38688, 14744, 45424, 50284, 37024, 65252,
                30604, 49064, 40560, 19300, 23104, 46148, 50180, 10428, 36544,
                3076,  19784, 57920, 44008, 3636,  21156, 6012,  41988, 55776,
                22824, 13644, 43368, 19420, 3952,  58276, 33400, 47668, 51660,
                37536, 42308, 1976,  53296, 40252, 15468, 43600, 36928, 14088,
                28388, 58680, 3064,  48624, 7092,  51408, 14348, 54160, 5568,
                43152, 12992, 63612, 20500, 30008, 49904, 59420, 26584, 11012,
                16896, 50864, 2084,  32128, 56044, 24400, 16712, 48152, 63492,
                50652, 23128,
            },
            {
                30476, 65516, 2232,  24564, 53684, 61536, 49356, 324,   30132,
                62440, 36404, 60360, 50996, 40436, 23656, 54680, 61684, 37108,
                47912, 22156, 40220, 9532,  53448, 32980, 2164,  6332,  36172,
                12496, 45980, 41084, 62692, 18168, 48232, 28200, 11596, 63160,
                26108, 56792, 10344, 14572, 32952, 47620, 62808, 5820,  48840,
                11676, 7960,  45632, 22616, 13312, 35220, 1852,  58536, 20836,
                7076,  30652, 55632, 23812, 63456, 27052, 32412, 4356,  55180,
                44820, 16108, 35188, 4432,  62664, 37936, 16700, 60996, 27328,
                52512, 8704,  31276, 35524, 12672, 64928, 48304, 26596, 8392,
                62140, 31112, 47288, 24380, 53280, 27600, 40692, 19820, 59940,
                12968, 28500, 5904,  19328, 25412, 59312, 56,    51964, 23968,
                32724, 40556, 16832, 65164, 43508, 38896, 25720, 63228, 21084,
                49136, 26500, 51748, 3456,  38272, 5192,  41432, 22096, 52876,
                35792, 8272,  26976, 14232, 49500, 10496, 29896, 43000, 288,
                12952, 41260,
            },
            {
                54696, 15916, 37320, 41636, 5112,  33180, 26936, 40236, 51968,
                24468, 44664, 6016,  31804, 13428, 27228, 4868,  32840, 11996,
                26484, 5880,  65296, 35368, 19704, 62340, 22808, 48040, 59276,
                53276, 1364,  31928, 57640, 7060,  22764, 40580, 32488, 53132,
                3928,  22160, 41340, 6520,  55192, 28156, 972,   40424, 29516,
                24800, 57216, 33104, 60844, 54084, 18272, 26420, 52464, 15844,
                62296, 46576, 11820, 17828, 52240, 616,   42716, 13660, 24520,
                6888,  53392, 26852, 11960, 54420, 28828, 2176,  21120, 41644,
                47532, 63748, 17820, 51492, 24712, 58864, 36764, 18488, 50460,
                34216, 11164, 57420, 6800,  37444, 14136, 62948, 1428,  24088,
                54700, 45596, 64116, 49188, 11776, 30452, 34532, 64424, 5268,
                49352, 55028, 12200, 22964, 29852, 19444, 9592,  33144, 40852,
                652,   31416, 15392, 58460, 10500, 17480, 47828, 65512, 404,
                30272, 60120, 47132, 40380, 64724, 37032, 53764, 21164, 35568,
                26144, 60432,
            },
            {
                5736,  27632, 51084, 19756, 13276, 57116, 8648,  18092, 34844,
                3788,  15088, 48532, 56688, 1828,  43996, 52360, 18908, 60064,
                42920, 55908, 16772, 2980,  46204, 14264, 29776, 41888, 16124,
                33948, 19384, 9952,  25892, 13708, 44924, 58648, 15664, 7868,
                46520, 29364, 64148, 37388, 44536, 20396, 57748, 14212, 53724,
                42688, 50128, 2760,  38232, 9016,  64932, 40216, 29452, 36564,
                50564, 2648,  40956, 60584, 9140,  36420, 57128, 18536, 62176,
                38568, 21808, 60296, 42212, 32000, 9320,  44524, 55448, 5488,
                14056, 38628, 656,   45880, 9748,  40696, 14796, 3332,  60256,
                40104, 40,    17648, 65224, 42484, 9608,  30332, 43860, 35520,
                8072,  15592, 33488, 38204, 55676, 7772,  46272, 18220, 10224,
                20624, 37240, 1024,  46496, 58032, 3744,  60232, 46064, 11932,
                53572, 61016, 35656, 44840, 55160, 27900, 34008, 6856,  45412,
                20224, 54692, 23476, 5948,  19092, 3576,  45076, 8544,  58920,
                47220, 19168,
            },
            {
                11308, 45036, 33984, 62400, 47760, 22760, 43140, 64836, 11904,
                58220, 41688, 28748, 19276, 37820, 64748, 34736, 9472,  49304,
                32,    31960, 51828, 27372, 58576, 39440, 3796,  51624, 7948,
                28256, 64996, 55132, 39196, 34448, 63556, 552,   20620, 43196,
                60356, 18140, 50504, 2572,  24008, 11136, 35548, 60644, 19204,
                4528,  16184, 27844, 20732, 48212, 43220, 4888,  57520, 8480,
                22516, 34312, 26636, 31508, 47740, 21572, 28724, 49620, 33476,
                10672, 47892, 1392,  15304, 49972, 64528, 25168, 34472, 59080,
                23488, 32808, 27024, 55892, 5364,  30208, 53740, 22408, 28680,
                54904, 45528, 26132, 32152, 49820, 22064, 55640, 5232,  58084,
                50580, 21716, 27096, 2812,  60792, 22908, 41280, 28656, 56588,
                44060, 60480, 26156, 53104, 34400, 15808, 50724, 36924, 17556,
                24716, 7956,  19472, 40024, 23748, 60536, 13596, 39364, 57208,
                15188, 42440, 11296, 34548, 59348, 28248, 56296, 15668, 32044,
                2920,  39096,
            },
            {
                53240, 58400, 9296,  1016,  29648, 36840, 3196,  47032, 53108,
                21456, 9352,  62952, 25256, 7284,  47396, 15816, 29040, 40768,
                23192, 12664, 37456, 43872, 23892, 10516, 56460, 61448, 37780,
                21848, 43264, 46972, 3152,  50004, 29548, 51980, 37744, 55720,
                35104, 9740,  32396, 15480, 61980, 52212, 6996,  45960, 32452,
                37544, 63872, 11432, 59844, 30596, 15256, 24272, 12612, 45456,
                63560, 18568, 58304, 4924,  15748, 39632, 7296,  59464, 2764,
                43776, 29628, 56184, 35456, 20072, 4060,  39776, 17552, 48684,
                10664, 61204, 43200, 21440, 62344, 49380, 33508, 44308, 7596,
                13344, 20856, 36228, 52588, 2640,  15724, 47512, 26620, 18308,
                39708, 62672, 10292, 43500, 17236, 50816, 13280, 62584, 35176,
                6580,  14740, 31576, 9228,  41684, 5420,  27644, 55944, 2944,
                64584, 29528, 47988, 5792,  2436,  31548, 51300, 9800,  25748,
                37072, 2904,  61804, 31324, 50932, 12644, 24320, 41440, 64140,
                49976, 22332,
            },
            {
                30692, 42528, 17740, 25556, 54080, 59084, 15680, 26552, 31104,
                38916, 33716, 50220, 53796, 11516, 22112, 57488, 3736,  54804,
                63132, 19940, 59796, 6432,  49672, 34256, 18640, 25820, 12240,
                50388, 5952,  15016, 24260, 18456, 11056, 6152,  26656, 13172,
                24376, 4252,  54632, 27392, 47300, 40092, 30376, 25704, 9492,
                55852, 23712, 51652, 34788, 2156,  56468, 52188, 32400, 136,
                41608, 13684, 49248, 54132, 43400, 64616, 12640, 53252, 26024,
                17268, 63816, 6040,  24180, 46696, 57396, 12676, 52384, 30340,
                6764,  53424, 15908, 36336, 11912, 18224, 1384,  64420, 38676,
                49076, 61608, 4544,  11568, 59744, 38992, 62276, 33828, 12428,
                948,   31668, 47924, 54452, 36028, 4984,  25600, 1456,  52152,
                22456, 40140, 63532, 48448, 19936, 62280, 44692, 22320, 32184,
                43212, 57512, 14356, 53068, 62404, 21280, 44004, 18620, 63244,
                29280, 52308, 46380, 16436, 39964, 1740,  48452, 35064, 6740,
                27172, 14188,
            },
            {
                36272, 4104,  64552, 38984, 11716, 50496, 7280,  19560, 61452,
                2140,  5692,  16608, 42500, 30064, 61544, 39400, 25836, 35700,
                45668, 8736,  30832, 15740, 64156, 4444,  30380, 46524, 416,
                55960, 31656, 60332, 36080, 57160, 61620, 41936, 31520, 65432,
                49060, 38436, 42296, 59304, 18648, 1756,  13484, 62980, 48896,
                508,   44548, 39544, 7680,  46748, 17900, 36888, 60784, 27660,
                55284, 38184, 9916,  25048, 1300,  33036, 23568, 46312, 35820,
                40788, 13264, 51724, 38000, 8852,  28352, 61652, 1124,  37204,
                20188, 46504, 2740,  57376, 41948, 27476, 58332, 10152, 24868,
                16728, 56668, 28164, 41712, 19316, 24820, 6648,  45084, 52852,
                60104, 41788, 6984,  20044, 29920, 58524, 38904, 48072, 30620,
                17724, 54792, 2688,  11968, 24620, 38684, 7716,  13560, 49856,
                10540, 37968, 26104, 42124, 8908,  36208, 58184, 1176,  48488,
                5312,  22024, 8680,  26556, 57700, 20464, 61024, 10352, 18408,
                55840, 61420,
            },
            {
                24192, 7904,  48700, 32008, 21856, 43952, 34292, 41384, 54816,
                45704, 23808, 59660, 36144, 1520,  45384, 5528,  13204, 17508,
                53012, 2340,  48340, 41324, 21612, 55044, 36884, 42288, 62812,
                17804, 40624, 27300, 8140,  47956, 22260, 45520, 16280, 1868,
                57288, 21580, 12004, 5140,  34124, 22788, 56816, 41916, 17064,
                21228, 29244, 14420, 58448, 26192, 42120, 9708,  21544, 47056,
                5680,  20092, 29580, 61044, 37160, 50840, 19324, 4036,  9216,
                58252, 21536, 31316, 60552, 43028, 15676, 33264, 44444, 25972,
                64956, 39968, 28968, 8260,  23172, 47260, 35464, 52276, 30628,
                43344, 6096,  33464, 47004, 55000, 30932, 10028, 37416, 22272,
                28484, 15184, 24772, 65360, 11228, 44332, 16100, 60148, 8716,
                46096, 26988, 36036, 57284, 30508, 59228, 52420, 35388, 62924,
                20340, 1360,  33768, 16800, 49752, 27628, 13036, 33256, 40784,
                56008, 35648, 64468, 44104, 5624,  37228, 29704, 53984, 43168,
                780,   46136,
            },
            {
                52000, 20288, 56156, 14820, 60048, 5392,  63720, 10300, 14192,
                28456, 48476, 12448, 55924, 18728, 32296, 50896, 60128, 42852,
                32660, 23760, 57356, 26912, 12416, 52096, 8312,  14716, 23408,
                34792, 4280,  52628, 13452, 38804, 2912,  33560, 52860, 19736,
                8636,  46708, 29864, 53268, 64504, 37716, 50960, 5772,  35848,
                60020, 54336, 4800,  19924, 33360, 62824, 3512,  50468, 16148,
                64884, 34816, 44672, 7616,  16648, 56592, 30616, 62308, 54544,
                27156, 45516, 188,   18256, 53660, 4552,  22100, 50380, 9432,
                58308, 13724, 34072, 50012, 63500, 6360,  15060, 20200, 2668,
                37672, 63752, 22600, 14804, 724,   51072, 64320, 17148, 56028,
                3544,  50308, 57448, 34556, 2260,  53204, 21516, 32908, 12552,
                64668, 5172,  43596, 50332, 15552, 252,   17984, 28848, 4408,
                47516, 55068, 59416, 22980, 65016, 45860, 6688,  53508, 23980,
                15308, 11008, 19064, 33076, 52576, 13868, 46864, 23284, 32704,
                38704, 11624,
            },
            {
                28048, 34360, 40452, 1396,  46668, 27400, 24024, 37708, 57324,
                18296, 39644, 6924,  26296, 63032, 9760,  21348, 28008, 6724,
                64864, 10628, 34700, 38856, 1268,  61304, 28616, 47672, 58340,
                10572, 45064, 64048, 20376, 55524, 28844, 10416, 58684, 40772,
                35564, 60664, 43608, 25296, 14868, 9220,  45000, 27728, 11520,
                32364, 41032, 65364, 49708, 12440, 53468, 23844, 39480, 31368,
                12024, 52504, 58104, 22592, 47576, 10736, 41516, 14780, 38136,
                6684,  50536, 11728, 34980, 25548, 63288, 40460, 55736, 18440,
                5116,  24236, 53676, 17628, 31108, 39068, 55272, 61012, 46184,
                12788, 53220, 8892,  58916, 35828, 43644, 27652, 5120,  47356,
                32620, 38504, 12924, 46048, 40996, 27944, 5560,  55596, 37324,
                41328, 19176, 23012, 9848,  34244, 47008, 41388, 60436, 24532,
                39608, 7612,  30408, 11316, 2896,  38324, 17892, 61300, 30256,
                43396, 59368, 49716, 60,    24828, 61728, 8936,  3896,  63932,
                15548, 59036,
            },
            {
                43520, 62712, 10452, 17948, 30680, 52928, 3836,  49792, 32812,
                228,   65232, 51648, 34088, 41372, 46780, 55004, 37132, 14708,
                49280, 18824, 45868, 59448, 16604, 43784, 33232, 3592,  19720,
                50892, 25312, 31000, 42684, 5132,  62136, 24656, 49720, 14272,
                28000, 3904,  17304, 7276,  49120, 31152, 19268, 62600, 52656,
                3748,  22836, 8948,  37484, 29800, 44428, 7200,  60080, 26388,
                1372,  40876, 4956,  27772, 63892, 3420,  25484, 49096, 20596,
                33428, 64376, 42180, 58764, 48416, 7932,  13060, 30816, 35740,
                48240, 42712, 60520, 436,   12604, 43936, 4416,  23452, 32688,
                26476, 48600, 18764, 40196, 24052, 11652, 57828, 21080, 42452,
                9612,  63232, 18820, 23036, 8432,  49428, 63068, 25136, 1148,
                51328, 58164, 28600, 62108, 54192, 6632,  21496, 55916, 11880,
                15828, 44920, 51024, 41584, 54496, 32320, 21552, 48172, 8236,
                3500,  36952, 28088, 41132, 56848, 17020, 39276, 50628, 28920,
                19260, 5648,
            },
            {
                25120, 3448,  49648, 36728, 58028, 42104, 20012, 62196, 8896,
                29804, 44128, 21096, 14896, 4172,  24396, 852,   58772, 40136,
                29760, 2972,  54220, 5592,  20996, 50008, 24528, 64988, 37440,
                56204, 1488,  16180, 48876, 34628, 17888, 38180, 1324,  63512,
                22608, 55584, 32944, 61788, 40300, 58696, 2884,  24624, 16188,
                43228, 47236, 26876, 17708, 736,   57660, 15156, 35432, 55748,
                49228, 17812, 32680, 14216, 38636, 34276, 44864, 60904, 1952,
                28904, 16472, 23212, 3820,  29508, 38484, 45784, 1432,  62732,
                27504, 10872, 37792, 21884, 56992, 26968, 50444, 9892,  41756,
                59480, 1764,  29964, 62748, 3976,  49868, 34580, 14256, 60920,
                25036, 244,   54488, 31404, 60468, 17456, 35012, 14292, 47332,
                31164, 16264, 2988,  39152, 13464, 26160, 45392, 31640, 37044,
                63704, 27316, 18868, 61792, 14772, 5096,  57852, 39576, 25824,
                62988, 54644, 13388, 7448,  31608, 44552, 21236, 54920, 35528,
                57340, 47708,
            },
            {
                33008, 53924, 21564, 61656, 6468,  11456, 15852, 45844, 23016,
                36244, 53920, 10836, 61128, 49052, 31100, 17588, 11748, 52812,
                22884, 61924, 26204, 36660, 31248, 10424, 40868, 7140,  13196,
                28324, 40056, 8556,  59892, 11908, 52132, 44988, 31832, 7776,
                47584, 37604, 51896, 352,   20940, 35680, 54824, 48380, 38316,
                59532, 13460, 54036, 61596, 48536, 40764, 20060, 9992,  46072,
                21908, 62776, 43312, 54844, 51320, 19012, 8268,  56356, 12460,
                53380, 39728, 9636,  55200, 19868, 15052, 57516, 21620, 52584,
                16588, 4320,  46264, 33336, 61944, 18860, 35628, 65440, 16956,
                7576,  36716, 44020, 15388, 55108, 31532, 7676,  39740, 52736,
                29372, 36076, 47992, 5972,  39304, 52356, 43960, 9724,  21288,
                61344, 8156,  44240, 32392, 49316, 65320, 1704,  9268,  52120,
                4796,  34324, 604,   24852, 36316, 28876, 51532, 11572, 16536,
                45468, 19828, 23112, 47404, 64920, 11476, 26832, 1912,  42240,
                8188,  12860,
            },
            {
                65328, 16484, 29236, 44696, 25944, 33520, 39844, 55636, 59888,
                3308,  17208, 38620, 27780, 55796, 35876, 63432, 44876, 6052,
                33452, 43172, 9328,  14064, 63348, 57008, 52924, 18124, 44456,
                60908, 32156, 54284, 22916, 27148, 5908,  57536, 20628, 42780,
                12668, 25916, 9700,  44152, 13988, 28172, 11272, 6116,  33164,
                1532,  30056, 35352, 6716,  24364, 32524, 64324, 28952, 2632,
                36996, 7564,  24944, 11160, 1540,  59864, 31592, 23936, 36448,
                43796, 61824, 47080, 32088, 51016, 65260, 26468, 6616,  40924,
                32216, 56180, 51048, 8152,  40280, 2616,  47024, 29344, 54344,
                21400, 57180, 51280, 25664, 20336, 45672, 65056, 17408, 4452,
                44736, 59260, 15132, 26452, 12136, 2700,  28704, 56316, 40752,
                35932, 52660, 23848, 55472, 19440, 35812, 16664, 57048, 42604,
                20892, 49132, 59092, 10040, 47476, 44100, 64404, 1112,  34936,
                52620, 30668, 2528,  58812, 35096, 5124,  49564, 14956, 60636,
                23156, 38268,
            },
            {
                376,   40960, 8444,  14032, 54820, 48704, 1196,  28316, 13348,
                25564, 47692, 58824, 4920,  8692,  42280, 19944, 26776, 48336,
                16292, 58092, 51028, 39680, 46868, 28052, 684,   34164, 49008,
                21328, 4476,  46348, 36896, 41228, 65108, 15540, 28556, 53004,
                59172, 18472, 64416, 30204, 57784, 46628, 41272, 63184, 21892,
                55856, 19172, 44768, 11336, 56752, 3916,  43656, 52832, 13104,
                60800, 30780, 57752, 47784, 28736, 41752, 15432, 52072, 4744,
                18360, 7292,  25216, 1688,  37516, 10472, 34132, 44468, 61184,
                12168, 19752, 29536, 25588, 15448, 53172, 11956, 5508,  39932,
                13468, 33824, 4588,  10140, 38820, 1184,  27288, 48804, 23512,
                10964, 19676, 40872, 56840, 64856, 33236, 18520, 59808, 6512,
                27020, 12864, 62972, 4180,  10560, 41156, 28188, 24092, 61172,
                12832, 30088, 39360, 56052, 22392, 7668,  19244, 26952, 42132,
                60560, 9468,  40608, 50876, 18368, 38448, 62420, 32560, 45336,
                29604, 51948,
            },
            {
                20020, 47020, 34724, 60812, 4688,  22232, 64712, 37960, 51044,
                6172,  41176, 33032, 22288, 13908, 50928, 2448,  60392, 10480,
                37916, 21936, 2336,  19196, 6572,  23072, 15312, 58616, 26220,
                9944,  63800, 13968, 19124, 472,   48352, 10536, 36304, 2316,
                40512, 6304,  34640, 50644, 23648, 7408,  17372, 26508, 51184,
                9316,  64688, 40168, 50544, 21344, 37972, 17128, 25648, 49536,
                40616, 15948, 19892, 5864,  37432, 21668, 64912, 46324, 27952,
                58268, 34576, 14380, 56248, 43280, 17716, 2940,  49728, 23644,
                37060, 58728, 1328,  63984, 44792, 57860, 32384, 25204, 49272,
                60472, 28208, 46776, 62284, 58140, 12732, 36656, 60188, 55620,
                34104, 2180,  50420, 22028, 45772, 37948, 24292, 48504, 624,
                45376, 18008, 38292, 30152, 47456, 59788, 50772, 6860,  38016,
                46292, 17244, 5940,  52328, 13924, 33188, 38228, 55168, 14812,
                5372,  33544, 25356, 14376, 28928, 56212, 22544, 9820,  17412,
                4208,  56804,
            },
            {
                11656, 59144, 24504, 51188, 30172, 43384, 16564, 8792,  31556,
                62944, 18744, 52792, 46260, 64636, 34112, 39808, 29316, 54576,
                4980,  65304, 30320, 56364, 35556, 44888, 61816, 38860, 3436,
                42048, 35284, 57328, 51688, 31148, 24220, 54872, 32876, 61532,
                22248, 49016, 15260, 38744, 1908,  54232, 60348, 37068, 42924,
                15040, 25212, 3128,  28192, 13932, 62704, 8328,  58568, 4368,
                33800, 53680, 45320, 63120, 12956, 50080, 3084,  10084, 38872,
                20948, 48440, 63820, 29732, 22340, 60444, 53292, 29028, 8596,
                16008, 48160, 42252, 10348, 22828, 37428, 20024, 62772, 43720,
                44,    22976, 18240, 41928, 32436, 52448, 21192, 8016,  30428,
                42724, 63376, 7104,  29684, 9868,  3860,  53144, 13876, 64064,
                32544, 50936, 7568,  58108, 21928, 2480,  14284, 31236, 54140,
                3072,  64656, 27212, 41832, 61640, 4176,  59616, 49712, 22704,
                46848, 63448, 57644, 44028, 6112,  960,   41276, 53284, 36240,
                48756, 26928,
            },
            {
                42600, 6316,  37720, 18152, 11000, 58000, 35084, 20776, 44320,
                56024, 10644, 29476, 628,   26012, 16968, 56696, 23472, 13612,
                32936, 42544, 25900, 48660, 9004,  51864, 31876, 12104, 55424,
                49888, 17172, 29016, 7724,  62572, 43356, 3780,  17212, 46408,
                26560, 56708, 3992,  61948, 20300, 33632, 12936, 30460, 5516,
                48412, 32972, 60244, 45564, 54436, 31128, 35228, 46448, 22316,
                10400, 26872, 160,   31860, 55784, 34524, 25928, 59400, 31364,
                932,   52716, 11508, 40588, 5044,  46720, 12896, 38736, 63352,
                3756,  34052, 54440, 31184, 6228,  50744, 2964,  16224, 9520,
                38416, 52844, 7328,  15112, 28864, 5164,  40724, 16260, 46616,
                13268, 24984, 36900, 55212, 16028, 59356, 40468, 30268, 21772,
                55780, 42372, 25340, 15600, 34540, 43528, 63580, 19768, 33720,
                11020, 23324, 35952, 19956, 30804, 46172, 10720, 29352, 2384,
                18112, 37380, 11332, 20744, 52288, 46332, 25888, 59856, 6876,
                64052, 31772,
            },
            {
                15752, 54628, 2080,  63824, 40372, 3816,  53884, 49296, 2004,
                24512, 15244, 36756, 42888, 59528, 9308,  3760,  44044, 47292,
                52608, 17692, 59676, 11664, 40808, 2020,  18604, 28652, 21888,
                6292,  24816, 45208, 11736, 39328, 20796, 58708, 37712, 6956,
                12012, 31688, 41820, 28640, 47108, 10036, 44544, 53156, 564,
                56500, 36268, 16724, 7032,  39392, 936,   52264, 19016, 61760,
                38488, 57232, 42668, 23852, 7996,  40844, 14756, 44264, 54736,
                16752, 42400, 7820,  26660, 33124, 58148, 19332, 25392, 55708,
                44932, 20752, 26432, 14024, 59284, 41480, 34840, 55120, 30396,
                57960, 35572, 64300, 48056, 56288, 24124, 49924, 62724, 3956,
                53324, 18216, 60420, 47180, 34252, 20004, 44252, 8192,  36688,
                11212, 2056,  62424, 45952, 53328, 26724, 9024,  40700, 50164,
                59016, 43808, 56888, 1352,  51060, 16756, 25268, 43232, 56464,
                32288, 53428, 6708,  30492, 62640, 34076, 10932, 19416, 13996,
                39472, 21992,
            },
            {
                50108, 44848, 28304, 32700, 47156, 23380, 27552, 13228, 59820,
                39000, 61952, 7188,  54348, 20056, 49928, 35268, 61676, 20648,
                7860,  24,    34912, 15420, 24340, 64100, 53464, 43268, 62408,
                37816, 58808, 33248, 4372,  53760, 27068, 14228, 50172, 64536,
                43752, 18916, 9212,  55324, 25156, 65240, 39224, 18348, 23948,
                62528, 21100, 10752, 58776, 23084, 26812, 12016, 42204, 29968,
                7120,  14612, 48580, 18380, 60932, 52412, 20088, 5204,  61660,
                24160, 35336, 62756, 51260, 16172, 36396, 224,   41552, 30732,
                5512,  61692, 38128, 48988, 18304, 64900, 11840, 27112, 45428,
                20364, 12536, 25980, 2628,  44336, 10864, 34384, 58260, 26772,
                32308, 39920, 484,   11152, 27880, 5316,  49464, 61860, 17036,
                28448, 39824, 19920, 4816,  11940, 37184, 56004, 176,   25140,
                15332, 47876, 8172,  13108, 62900, 39496, 35544, 8352,  65448,
                13592, 40272, 23876, 49224, 16208, 37700, 58200, 29904, 47272,
                1308,  61856,
            },
            {
                25056, 35424, 7672,  20192, 14852, 62444, 42268, 5752,  33484,
                45864, 27936, 48424, 32232, 13088, 39424, 29844, 11460, 27448,
                37776, 62460, 50972, 29500, 56728, 36992, 7464,  14476, 46968,
                2624,  51160, 14916, 61308, 47976, 35192, 9104,  29608, 23048,
                53092, 59584, 36960, 51124, 5832,  14888, 58136, 28028, 8148,
                46032, 29836, 51852, 42720, 49192, 64296, 15924, 55888, 2804,
                51564, 65112, 36264, 11416, 3520,  30028, 37616, 49892, 12212,
                29184, 47428, 4008,  21232, 54968, 9060,  64788, 50188, 11784,
                17044, 52028, 9400,  1052,  28672, 46828, 23100, 6932,  60960,
                3704,  51204, 32844, 39532, 60260, 18872, 1092,  38172, 20276,
                8244,  44144, 51340, 22832, 64628, 54656, 25644, 2868,  56676,
                46980, 52396, 33412, 57632, 30588, 48076, 17744, 61932, 29308,
                5104,  38644, 31784, 54928, 23564, 3384,  59320, 20944, 50424,
                26748, 336,   61116, 42540, 8600,  2556,  22796, 43012, 56020,
                33396, 9360,
            },
            {
                57712, 13200, 60648, 55536, 50964, 9516,  36488, 56440, 19216,
                9968,  21712, 4500,  58476, 23868, 64036, 6204,  54052, 57948,
                41664, 22736, 46308, 5416,  44476, 20928, 33040, 26716, 10548,
                30384, 19832, 41204, 23800, 17960, 1304,  57592, 41048, 4900,
                33976, 20,    16420, 22468, 32380, 2552,  35324, 50124, 41376,
                13364, 37864, 2040,  18840, 32676, 4184,  37140, 47580, 24752,
                33316, 20172, 28240, 58516, 43420, 25192, 64004, 33656, 8612,
                18672, 58396, 37724, 13828, 44584, 28548, 47600, 22532, 59396,
                35560, 24676, 40520, 57296, 33388, 4052,  54044, 39116, 49448,
                17216, 42632, 8448,  22044, 14364, 30092, 55020, 45356, 12332,
                63524, 56560, 15268, 30632, 38116, 41744, 13092, 35112, 22140,
                60756, 9048,  14036, 23880, 65156, 7248,  22312, 44560, 35688,
                60016, 21256, 17308, 42696, 27508, 47128, 53804, 15632, 4620,
                45312, 34988, 19068, 55304, 27592, 64708, 51092, 5492,  26836,
                52320, 18812,
            },
            {
                40656, 47848, 4164,  38536, 25464, 28,    29960, 49852, 15568,
                65532, 53512, 41572, 16912, 1344,  44948, 47936, 18924, 2816,
                13912, 31848, 10128, 17996, 59408, 2932,  49828, 60640, 54536,
                36008, 65236, 8620,  56336, 31448, 44480, 63536, 19660, 48604,
                13212, 27868, 46160, 63116, 42880, 48236, 20764, 60760, 4300,
                54964, 63592, 26492, 57564, 9488,  44984, 28772, 58996, 11036,
                41220, 46000, 5700,  53952, 16344, 47880, 704,   57108, 44776,
                40400, 1924,  31408, 60688, 24780, 39756, 3700,  32748, 7356,
                54740, 45436, 13288, 63060, 21524, 43048, 15408, 35880, 24684,
                30976, 62668, 57316, 53000, 35768, 65344, 7196,  49264, 28820,
                24304, 34832, 4932,  48708, 8868,  18388, 51732, 31588, 6628,
                42960, 26504, 49952, 39084, 3144,  42056, 51524, 12720, 54300,
                10100, 49484, 64812, 34732, 6588,  11952, 32596, 41592, 29824,
                58500, 48496, 11796, 32932, 14736, 45956, 39656, 17000, 11844,
                36976, 3388,
            },
            {
                65152, 22328, 28884, 43440, 17388, 58656, 45228, 22404, 39636,
                2228,  30708, 36212, 51372, 26960, 32960, 15232, 36524, 25456,
                49300, 63676, 55444, 27764, 38112, 12764, 42336, 16844, 39940,
                444,   25828, 47460, 4112,  37508, 11808, 52076, 25532, 38472,
                56828, 60740, 39924, 8228,  12412, 56240, 26184, 10304, 31652,
                16356, 34412, 6284,  40640, 14680, 61844, 20844, 7664,  17684,
                61272, 2268,  23004, 31736, 39236, 10228, 21740, 14468, 27740,
                23132, 52668, 48740, 5716,  10776, 56836, 18900, 15144, 43128,
                28072, 19700, 5000,  30536, 50880, 7964,  61984, 10596, 55648,
                1796,  13132, 27876, 4708,  47704, 41180, 22740, 16072, 52564,
                1928,  42248, 61212, 21448, 58564, 1436,  62392, 45828, 16032,
                37084, 592,   18792, 60068, 35240, 16316, 27608, 32428, 4340,
                41044, 26072, 984,   57600, 51536, 19612, 37812, 63224, 9588,
                22496, 6168,  38248, 60036, 4384,  30956, 20984, 57904, 61240,
                43856, 30264,
            },
            {
                16176, 10132, 34268, 53432, 12176, 32868, 63292, 7420,  26216,
                57724, 46836, 11992, 62256, 9072,  55964, 42976, 60284, 52868,
                7684,  39292, 1116,  33892, 53120, 64436, 25068, 5684,  21688,
                57988, 13424, 42788, 53500, 28092, 16048, 5804,  32736, 9732,
                17500, 3720,  24464, 53820, 30252, 37932, 17760, 44208, 51552,
                23248, 47424, 20232, 53776, 49848, 24284, 34120, 54268, 50948,
                38364, 13524, 56824, 62432, 7468,  51428, 36116, 60448, 55280,
                6612,  65376, 16820, 42796, 34564, 62212, 52916, 37980, 63696,
                1848,  59868, 47500, 37152, 17580, 56532, 27612, 44828, 19636,
                40456, 34668, 45552, 17504, 10528, 26292, 3380,  33364, 59668,
                38720, 17748, 11532, 46640, 36176, 24912, 28588, 57156, 11044,
                63860, 53780, 29208, 45020, 55248, 9280,  63260, 57240, 19060,
                46140, 14416, 30688, 9288,  44484, 60840, 2220,  25504, 55816,
                17240, 43676, 51776, 24756, 54028, 10092, 35312, 48572, 7688,
                23708, 54416,
            },
            {
                50588, 45344, 5288,  62148, 20672, 3528,  41212, 51596, 13752,
                34204, 6464,  24252, 19876, 38668, 5072,  22128, 10724, 30344,
                20440, 44120, 14944, 22296, 47748, 9040,  31084, 46036, 51324,
                29340, 33832, 18320, 61444, 22212, 58628, 46912, 61976, 42956,
                29392, 51804, 35056, 19804, 2136,  58312, 7008,  64648, 676,
                39024, 61292, 10948, 29616, 36504, 4704,  41808, 496,   30368,
                26276, 48640, 34500, 44212, 26796, 17700, 41976, 30592, 46600,
                12424, 35420, 26676, 21044, 29520, 792,   48968, 23344, 11312,
                50596, 34060, 9500,  25040, 40256, 764,   33052, 48712, 5404,
                63904, 50436, 21356, 60504, 38008, 51432, 62112, 43080, 9656,
                55896, 31240, 26548, 53184, 14584, 44232, 5580,  39408, 23244,
                48444, 32792, 5824,  11800, 21296, 47440, 1252,  24076, 37876,
                61076, 52872, 22244, 40040, 16388, 28720, 47868, 14020, 50568,
                31932, 64280, 1472,  19536, 42228, 62336, 2348,  26544, 14424,
                33160, 696,
            },
            {
                20280, 58432, 26756, 48732, 37552, 55368, 9388,  28476, 61000,
                18692, 54664, 41944, 58836, 48300, 28216, 64660, 34716, 4240,
                47152, 61580, 26604, 59004, 3988,  19200, 35484, 56068, 11224,
                63480, 6456,  49288, 9460,  40484, 35276, 1584,  20360, 55076,
                6936,  45192, 64168, 15200, 41784, 49540, 27760, 34696, 14092,
                25640, 57200, 3148,  45492, 65340, 13260, 57796, 44388, 63844,
                16112, 6320,  20540, 11504, 53060, 64260, 5424,  2720,  19680,
                39072, 8980,  50440, 59296, 13184, 45768, 7704,  31284, 26332,
                40884, 57672, 15580, 65276, 52732, 12576, 21972, 58980, 14604,
                25616, 7848,  31800, 292,   55512, 30356, 19516, 14100, 48276,
                21912, 6484,  64872, 33072, 3324,  55528, 20156, 50760, 8104,
                17772, 41168, 58008, 25252, 34224, 39724, 30128, 50096, 7788,
                33960, 3556,  63864, 36140, 54848, 23412, 5400,  34908, 40552,
                7524,  27272, 36568, 15160, 29216, 50116, 38500, 55732, 46420,
                63508, 39316,
            },
            {
                35968, 8196,  41904, 13904, 30560, 24112, 16380, 48004, 36012,
                43800, 3864,  29764, 14832, 372,   52224, 13028, 41312, 57120,
                17292, 36312, 11472, 42692, 51944, 38972, 61800, 2000,  16040,
                44048, 37680, 24368, 3220,  30076, 14196, 49984, 25128, 37248,
                13540, 22188, 48128, 10188, 32588, 61428, 21492, 46296, 53356,
                8756,  43060, 32316, 18116, 26740, 8452,  23160, 19476, 9940,
                36784, 60476, 40600, 1076,  29724, 22744, 48176, 33500, 54248,
                61324, 44036, 3808,  55828, 40992, 18044, 36352, 56360, 14204,
                2984,  20872, 44608, 29004, 4504,  60828, 42560, 29876, 37856,
                53716, 57868, 42024, 12056, 24316, 44396, 7640,  36224, 27724,
                2396,  44000, 49980, 9956,  38256, 63016, 29784, 34632, 61588,
                27276, 13860, 2456,  64604, 51892, 15380, 59732, 12972, 43908,
                17288, 27972, 48912, 6896,  12384, 59364, 43560, 62648, 20204,
                11888, 59044, 47388, 56220, 7936,  12132, 22304, 18024, 4716,
                28552, 11564,
            },
            {
                24768, 54840, 18300, 2076,  59100, 45640, 64828, 896,   21792,
                10928, 50448, 63692, 37056, 32556, 45100, 18760, 25636, 50356,
                2224,  55084, 32368, 6976,  29084, 13588, 23908, 41452, 27812,
                20848, 59164, 54136, 45484, 64752, 56300, 41812, 10796, 62996,
                31272, 920,   38740, 55752, 24796, 3984,  11924, 40156, 59280,
                19656, 37372, 15192, 52128, 59604, 39584, 50352, 32820, 46248,
                55148, 24216, 49644, 56428, 15360, 37840, 58692, 13848, 25000,
                28808, 16416, 23236, 31700, 6356,  64316, 21932, 61256, 43584,
                53072, 32304, 6972,  49364, 18072, 34644, 46656, 3680,  10104,
                18332, 35904, 47372, 15556, 63680, 4192,  58024, 53260, 62632,
                39660, 57332, 18796, 23804, 41960, 16988, 12368, 1084,  42628,
                54896, 45736, 36056, 19132, 7384,  42200, 3292,  20912, 54480,
                59012, 11396, 45476, 20028, 32624, 50984, 1604,  29624, 54104,
                45056, 3480,  23268, 32176, 63744, 45308, 34424, 59608, 43132,
                51336, 60620,
            },
            {
                40732, 31920, 63112, 51992, 34720, 6748,  39848, 53964, 31868,
                58180, 26020, 17564, 8824,  23024, 59712, 7244,  62564, 29928,
                9556,  24200, 65356, 49764, 17952, 58320, 53988, 33028, 4584,
                50688, 10088, 32036, 17220, 26764, 4960,  33168, 18532, 57760,
                51384, 26704, 59644, 6308,  44392, 52712, 17140, 31432, 2280,
                28424, 55672, 5988,  47192, 1524,  29168, 56520, 14732, 2496,
                28096, 5256,  31964, 44928, 61084, 6820,  43176, 9720,  51076,
                2036,  63460, 37452, 53420, 48136, 27424, 9996,  49588, 5320,
                38320, 60180, 24048, 55232, 39228, 8904,  26008, 52304, 64560,
                23392, 2532,  28792, 52144, 34444, 40072, 17296, 23460, 10620,
                34184, 15240, 29124, 60744, 7336,  51984, 58404, 48012, 21384,
                4536,  60300, 30260, 48892, 56620, 27388, 61908, 36828, 25524,
                32092, 41332, 584,   62080, 25976, 38816, 17980, 8604,  25064,
                37764, 51912, 17152, 40344, 64,    52664, 25660, 6492,  31480,
                16372, 2652,
            },
            {
                13224, 6020,  44572, 22444, 11168, 27404, 19592, 13400, 37648,
                6176,  46592, 40584, 56484, 53312, 3336,  35800, 39376, 14128,
                45776, 40820, 20612, 37012, 312,   44260, 8456,  48484, 62868,
                36428, 14656, 832,   39612, 21660, 52392, 8096,  46228, 3016,
                43404, 16396, 34492, 19312, 28712, 36984, 63496, 7508,  50132,
                61132, 22820, 33784, 63036, 20196, 36872, 6912,  62532, 41956,
                52404, 13164, 18896, 9096,  35092, 27116, 20448, 55660, 40100,
                45976, 34332, 10744, 14484, 1188,  39992, 34064, 15980, 29488,
                19048, 10804, 35912, 13152, 62864, 20780, 57056, 15300, 40188,
                44656, 60916, 6592,  20580, 26164, 49844, 45328, 32024, 5964,
                51584, 47408, 452,   36360, 45884, 25188, 31452, 37472, 15176,
                24452, 8804,  38920, 11344, 22624, 33108, 46948, 10048, 4720,
                51824, 22476, 56896, 36904, 14392, 55372, 42184, 57884, 14848,
                65264, 33580, 5456,  60004, 20728, 14456, 39396, 10324, 56948,
                21560, 47344,
            },
            {
                35292, 50256, 28604, 15864, 57264, 42488, 60324, 50804, 24704,
                62752, 2768,  29200, 12528, 42848, 27348, 48836, 21436, 58556,
                54076, 5472,  15196, 46964, 60280, 27108, 12400, 22536, 18496,
                42908, 55912, 60836, 48104, 11724, 62480, 36300, 28796, 22920,
                39676, 9028,  64732, 49724, 13220, 57068, 42168, 24288, 46984,
                13368, 44160, 9772,  41248, 12492, 24864, 48216, 17608, 34892,
                59708, 38936, 63572, 23596, 51884, 1668,  64968, 31420, 18036,
                5748,  57364, 21388, 60316, 44268, 24240, 58880, 55116, 2172,
                64016, 45620, 27460, 280,   48100, 30572, 2744,  33596, 49256,
                11268, 31540, 54520, 58360, 9504,  1468,  13244, 60212, 20076,
                25676, 64108, 12248, 54276, 19788, 3724,  9852,  65324, 53656,
                33704, 50500, 16908, 44508, 428,   53208, 17624, 39800, 65192,
                15248, 7968,  30160, 49824, 9924,  4848,  46688, 31444, 21512,
                48364, 10440, 27436, 44112, 30644, 49236, 62132, 36276, 53580,
                27220, 65040,
            },
            {
                19456, 55996, 3376,  37500, 48420, 1452,  30040, 8120,  44276,
                16784, 49200, 34432, 20000, 64532, 16160, 10376, 31728, 1040,
                34528, 26096, 62852, 30928, 4344,  34836, 40180, 57152, 30412,
                6264,  24196, 28432, 33692, 42408, 58128, 14828, 48952, 61220,
                12380, 53772, 3872,  45936, 21784, 364,   10460, 35128, 18596,
                38260, 3900,  26536, 50828, 55100, 31468, 43612, 10840, 21836,
                26404, 3400,  47652, 41556, 16568, 46632, 37076, 12300, 24408,
                49328, 27720, 38836, 30608, 50648, 19252, 7984,  47136, 41804,
                21832, 51856, 57772, 40472, 16840, 43832, 59792, 7608,  27040,
                19396, 38440, 13788, 42832, 37080, 65204, 28520, 54712, 37784,
                43592, 4548,  41008, 33112, 57004, 27456, 44312, 40440, 5564,
                28544, 62120, 57528, 26232, 63768, 6640,  29368, 55744, 44944,
                35108, 47592, 18392, 63392, 23240, 28116, 61384, 2832,  36416,
                6596,  41844, 54392, 12500, 57452, 3908,  18480, 46212, 1676,
                42448, 8280,
            },
            {
                24692, 39888, 61364, 9784,  24168, 63468, 18076, 35624, 55040,
                11032, 22880, 60732, 37728, 4748,  46112, 52016, 60568, 17732,
                43980, 50240, 9780,  55760, 23484, 51508, 16300, 63928, 1856,
                45692, 53128, 8276,  19164, 3280,  25444, 5860,  20148, 56120,
                33556, 25684, 30748, 38596, 58892, 33148, 27304, 51128, 54396,
                64944, 30184, 57944, 16612, 4736,  63936, 84,    58288, 49880,
                8100,  33744, 57276, 11628, 28232, 54100, 4128,  58372, 42992,
                62836, 8688,  17300, 5056,  65180, 12648, 36700, 25764, 31080,
                13964, 8644,  32460, 5956,  24884, 53832, 36536, 22572, 51344,
                63368, 860,   47860, 16764, 32904, 21608, 47068, 3296,  16232,
                57560, 30416, 22200, 14608, 49240, 59544, 17864, 13068, 46784,
                19604, 2444,  41040, 14156, 34660, 49440, 11720, 20412, 1820,
                26532, 60124, 3116,  39512, 43916, 34772, 52552, 12748, 59336,
                50484, 17896, 62396, 24904, 35204, 8876,  28832, 23504, 15296,
                32864, 59840,
            },
            {
                11760, 17144, 30736, 51816, 33516, 45592, 4644,  40416, 59324,
                32640, 540,   51576, 8400,  30716, 55624, 23400, 41216, 7740,
                28688, 12836, 38912, 18880, 42324, 7364,  47208, 10712, 38460,
                34428, 13840, 65216, 38964, 51292, 44968, 31716, 37652, 1068,
                42040, 18204, 62292, 14504, 8092,  43524, 61560, 5604,  15172,
                1572,  21224, 45108, 34488, 39816, 22936, 52972, 37984, 29740,
                15180, 53624, 20768, 31092, 6548,  61828, 21996, 33728, 15128,
                432,   53288, 47256, 55952, 33328, 2796,  53096, 62288, 4820,
                38828, 60508, 18644, 50296, 65028, 10580, 14104, 41564, 4496,
                55392, 29256, 23952, 61836, 4992,  53472, 40308, 8524,  34560,
                49772, 9912,  61504, 6792,  38696, 1896,  35504, 61060, 23688,
                55196, 36816, 8248,  22056, 42964, 59656, 38276, 24444, 57856,
                42660, 32672, 12228, 54224, 6968,  16332, 19780, 26168, 39856,
                29700, 22540, 1940,  38660, 47168, 53264, 64184, 40096, 57844,
                4884,  48796,
            },
            {
                43868, 54616, 6396,  42032, 13916, 21576, 52648, 25656, 14936,
                27912, 43100, 57848, 39744, 25536, 13952, 2752,  36968, 64244,
                54796, 22364, 59180, 1460,  62092, 32048, 54548, 21004, 25784,
                58540, 49220, 22344, 11216, 55680, 16944, 63564, 52596, 10944,
                46876, 6760,  51600, 23972, 48664, 17680, 56040, 23404, 31864,
                41680, 48936, 6804,  60720, 13688, 28076, 18784, 5704,  45912,
                64716, 41080, 1008,  60088, 44220, 38604, 9128,  50592, 29132,
                40904, 35632, 11196, 23904, 40496, 28472, 43476, 17496, 49340,
                55928, 23348, 43256, 34848, 3464,  28364, 46904, 60700, 17912,
                34636, 45092, 8088,  50668, 26856, 11732, 59444, 24460, 63088,
                19192, 27308, 53048, 46432, 24360, 30068, 8520,  52148, 32612,
                11248, 48680, 30336, 56088, 3108,  16404, 31844, 52748, 8360,
                14368, 50304, 21872, 29752, 58456, 47968, 63812, 744,   45688,
                9292,  56400, 14880, 32040, 6164,  21060, 43296, 12752, 50608,
                37004, 28260,
            },
            {
                1348,  34948, 19932, 64496, 2588,  56584, 10316, 48720, 5872,
                65528, 20528, 12020, 18016, 45016, 62380, 33220, 19776, 47364,
                4028,  35480, 45144, 26760, 14764, 36812, 28620, 60736, 2844,
                17428, 41840, 29560, 284,   35436, 27596, 8076,  23500, 60396,
                28060, 57384, 36064, 1840,  29748, 39372, 12088, 46700, 36440,
                59200, 11612, 25488, 51788, 9176,  47412, 61524, 35172, 12436,
                23996, 18096, 36704, 49152, 14488, 25084, 19036, 46068, 56608,
                25752, 20468, 59540, 45324, 15228, 21140, 58188, 10624, 33988,
                1212,  29144, 11552, 54560, 38552, 20952, 56800, 31200, 9404,
                39812, 12948, 58832, 36136, 18932, 44192, 31244, 14952, 45896,
                1280,  41428, 36480, 17180, 64568, 56132, 20524, 42260, 4600,
                64120, 43888, 18104, 61340, 26720, 46424, 4560,  61784, 19248,
                40604, 64336, 5764,  37656, 25576, 10308, 41476, 33216, 54156,
                35708, 64744, 42476, 50944, 61352, 16596, 576,   31440, 9620,
                20476, 62716,
            },
            {
                46168, 59160, 26424, 47764, 29272, 36408, 62152, 31644, 38632,
                53732, 47044, 35924, 6392,  50320, 58368, 27832, 11288, 51412,
                15732, 31692, 5596,  49432, 53008, 11864, 4872,  43204, 50788,
                32696, 7000,  62116, 47952, 58208, 43072, 13656, 49456, 39948,
                16152, 20860, 44432, 65400, 53972, 4080,  20472, 63328, 8384,
                28228, 54892, 19432, 37512, 32100, 56112, 3216,  42496, 51512,
                7332,  55476, 26992, 10216, 33296, 53020, 64612, 6036,  12608,
                61612, 4604,  30308, 1920,  63540, 50360, 6624,  24856, 46028,
                14580, 63600, 47700, 7324,  16376, 44348, 1444,  25420, 52492,
                64524, 21804, 4284,  41184, 57016, 1968,  52292, 38724, 6560,
                56504, 32192, 11852, 4268,  43744, 13704, 50428, 25772, 15728,
                27836, 1492,  53320, 12704, 37492, 51708, 10868, 44516, 35992,
                27368, 0,     55188, 45220, 14876, 51836, 4440,  23372, 13948,
                6780,  19356, 24600, 10748, 27840, 34712, 55864, 60328, 26228,
                52528, 15684,
            },
            {
                23140, 11380, 7952,  39596, 15872, 23704, 43540, 19112, 1464,
                9312,  24348, 56128, 29664, 2236,  41764, 8624,  38760, 24540,
                61864, 56084, 20640, 39288, 65076, 24248, 46256, 19624, 9528,
                56032, 37400, 15976, 24720, 5656,  20100, 30872, 1972,  35052,
                4448,  31584, 13080, 9428,  26644, 34544, 42828, 51928, 2304,
                16660, 40564, 64484, 1208,  44168, 15372, 21112, 25860, 59772,
                30824, 44580, 62768, 4204,  58232, 41748, 2488,  31632, 37760,
                43792, 17808, 52260, 38344, 9244,  35984, 31392, 61232, 40988,
                37168, 19628, 26244, 59204, 31996, 63220, 49700, 6496,  15316,
                33472, 27788, 49408, 15704, 29416, 9644,  64684, 20664, 26652,
                50300, 60964, 21548, 54424, 33612, 4,     39332, 59048, 36088,
                56740, 39704, 23436, 31552, 5840,  65012, 22412, 29540, 57104,
                48744, 17228, 33404, 61104, 20656, 31464, 62264, 57076, 28176,
                49632, 46304, 3140,  58632, 39688, 48044, 23924, 44440, 38404,
                7004,  32504,
            },
            {
                57532, 37144, 51116, 54184, 60856, 5296,  12452, 59072, 28408,
                42008, 15848, 33064, 63908, 22672, 16828, 46444, 53996, 8,
                42456, 29080, 10364, 7416,  17904, 33908, 57124, 40240, 63296,
                27224, 12224, 52956, 45240, 39488, 60944, 53540, 64352, 45616,
                55600, 59652, 41504, 49788, 60704, 15220, 56944, 30484, 24948,
                45708, 33260, 6288,  23260, 28908, 62576, 38800, 9836,  33780,
                2188,  16088, 39780, 21952, 29056, 17656, 47896, 23276, 55012,
                7816,  33604, 48472, 57436, 13728, 26908, 55556, 18328, 5444,
                50100, 57472, 2324,  41520, 22676, 12900, 36052, 40916, 57900,
                47808, 212,   61732, 54176, 24184, 33804, 47508, 36804, 13004,
                43180, 8720,  25112, 48628, 28268, 62892, 9568,  47416, 6452,
                20160, 45920, 8932,  49488, 41600, 18600, 34280, 2728,  13392,
                7344,  23896, 39828, 9100,  3008,  37000, 43956, 17720, 39416,
                59780, 30820, 36912, 53992, 5200,  13744, 8264,  18800, 2820,
                65168, 41828,
            },
            {
                4352,  29856, 18164, 12,    32780, 45824, 50396, 55088, 35124,
                48280, 60616, 3728,  51752, 12840, 36204, 59728, 19348, 34232,
                13652, 58608, 48612, 43940, 59916, 1048,  29468, 14884, 4056,
                47448, 20608, 31256, 3232,  10640, 33952, 17348, 26116, 11348,
                22256, 7052,  37888, 24300, 18808, 5544,  38468, 10356, 62100,
                49900, 13896, 57540, 53404, 47812, 4952,  50340, 58004, 18992,
                47548, 53224, 11360, 49768, 7212,  36280, 13944, 59632, 27872,
                15596, 64276, 24572, 20116, 41456, 46404, 36,    52548, 22972,
                9948,  29816, 35144, 51364, 9256,  53860, 28964, 18660, 23764,
                10940, 38748, 19460, 7528,  42444, 60112, 17532, 5180,  62348,
                30532, 16476, 58112, 5520,  37852, 18944, 23256, 31376, 12516,
                62368, 33732, 60308, 14440, 54752, 25852, 59152, 51012, 42612,
                63380, 53376, 46956, 58560, 26448, 50612, 11252, 7392,  21656,
                1296,  12328, 43804, 17272, 22180, 63608, 51580, 30940, 46944,
                54352, 13064,
            },
            {
                49528, 61900, 25360, 42384, 10012, 21168, 26820, 17088, 6084,
                21988, 11536, 44380, 39004, 26848, 48688, 7172,  30232, 64664,
                5936,  22396, 36320, 25880, 52020, 11328, 37956, 53896, 23368,
                35376, 43016, 64868, 57100, 49028, 23424, 7768,  42432, 50264,
                29568, 16852, 52976, 16,    45112, 33012, 48184, 22508, 4484,
                19028, 36084, 42364, 11140, 18088, 34972, 13176, 41860, 28160,
                65472, 23612, 34728, 61260, 56812, 44316, 51448, 1336,  40128,
                46588, 10800, 3164,  29424, 6252,  62840, 16288, 32708, 43972,
                65116, 13680, 45756, 17660, 4556,  59740, 46620, 2856,  61268,
                44032, 30648, 35860, 46244, 12596, 3432,  55164, 23184, 51680,
                976,   35440, 40896, 45944, 14704, 59816, 50020, 42988, 54000,
                3812,  28404, 17540, 556,   36160, 46860, 8516,  16060, 38532,
                31268, 19732, 12156, 29908, 16364, 56188, 48000, 34960, 63756,
                51288, 25792, 61264, 33724, 28988, 41496, 57928, 35724, 15904,
                27704, 21272,
            },
            {
                33624, 45012, 13760, 56724, 64380, 38056, 2784,  40792, 63452,
                31720, 57824, 8048,  19728, 62372, 1632,  55532, 25224, 44956,
                50576, 40736, 2472,  15544, 31460, 21196, 62024, 48868, 8552,
                59456, 1876,  17992, 28308, 14816, 38516, 58740, 3848,  36600,
                61188, 46564, 34304, 27700, 63960, 57976, 13420, 54112, 39928,
                59096, 27360, 7760,  63668, 30964, 25004, 55152, 2948,  7804,
                37880, 14260, 328,   30448, 19228, 26056, 9160,  62560, 21420,
                35088, 53720, 43004, 50156, 58324, 37624, 11088, 26140, 39476,
                3356,  56252, 24836, 61356, 33212, 38352, 25876, 55452, 8136,
                51904, 14336, 63304, 25960, 50752, 31852, 40480, 27496, 44936,
                57188, 20248, 65484, 10900, 53056, 29932, 2424,  16972, 35580,
                25280, 51032, 43664, 57352, 29636, 4772,  21820, 61992, 26860,
                1168,  36492, 5012,  43068, 65468, 1700,  23856, 29116, 41720,
                15276, 55308, 8584,  3588,  49552, 11092, 876,   25044, 60252,
                9768,  40320,
            },
            {
                1904,  8220,  20256, 35676, 48860, 30300, 59860, 51784, 14312,
                36988, 25404, 53168, 30792, 42584, 33688, 15252, 38400, 11440,
                17968, 28272, 57460, 63976, 46704, 41716, 4948,  17196, 32536,
                26520, 54880, 36920, 6756,  52092, 44708, 30364, 54464, 20420,
                14316, 3100,  55884, 11144, 21408, 7892,  29324, 43828, 1892,
                32060, 51172, 22092, 800,   39492, 46352, 61476, 21700, 32516,
                56404, 46500, 42388, 6412,  39324, 12444, 32804, 54636, 4836,
                18156, 60044, 31820, 13240, 22224, 34180, 48400, 60544, 53816,
                19916, 36284, 7532,  43036, 21124, 10832, 15956, 42036, 34200,
                22448, 5432,  56632, 2108,  21508, 58620, 14860, 10264, 38052,
                7520,  32600, 27056, 4328,  24068, 34320, 8296,  56392, 63616,
                40864, 10180, 20388, 64516, 11788, 40548, 56036, 44688, 10676,
                49204, 60000, 54648, 22524, 34128, 40364, 13796, 54004, 4908,
                31500, 18876, 47308, 36532, 56692, 20084, 45444, 39168, 5336,
                52080, 56108,
            },
            {
                63628, 47964, 28300, 4064,  16020, 7152,  24120, 10680, 45244,
                840,   49516, 17284, 4404,  56512, 23052, 8844,  52340, 61664,
                4472,  54236, 33856, 9652,  6784,  55940, 24748, 44368, 51740,
                14048, 10456, 42044, 21768, 60632, 672,   12244, 63920, 26780,
                9036,  39528, 62504, 42580, 16272, 51700, 35944, 25560, 65128,
                15984, 12152, 44668, 59600, 52044, 14984, 9676,  43628, 50872,
                16684, 25384, 63568, 53608, 59140, 48928, 23468, 42652, 28696,
                38372, 7492,  26240, 1812,  55024, 18412, 7840,  4592,  30520,
                15440, 49260, 28360, 53216, 63156, 1552,  50400, 29420, 65444,
                18200, 47960, 28296, 39124, 8732,  35224, 47188, 61036, 18196,
                49208, 13804, 55340, 42592, 48768, 61584, 39428, 21468, 14244,
                47200, 6704,  37896, 24416, 48900, 32124, 17964, 52592, 33900,
                24152, 14992, 45164, 8460,  18576, 59452, 46540, 10032, 62124,
                44592, 38784, 22948, 64548, 26800, 14968, 32984, 62848, 22612,
                30276, 17524,
            },
            {
                24484, 38888, 59020, 52912, 41492, 55832, 34484, 19884, 56972,
                41800, 28636, 65172, 13416, 46396, 60424, 18884, 41940, 31400,
                46008, 21748, 14116, 39516, 19512, 28560, 36048, 488,   63896,
                38288, 46744, 62520, 32248, 25208, 34992, 17528, 41148, 47716,
                32896, 50776, 24644, 30664, 38244, 4108,  60992, 19572, 6668,
                47468, 56764, 37532, 29092, 4740,  19760, 36424, 27064, 58464,
                4860,  10328, 20904, 28244, 3408,  35896, 16580, 65060, 47284,
                14560, 51080, 61780, 44900, 40720, 64580, 24968, 45808, 42188,
                58896, 708,   12352, 47432, 31580, 40172, 58624, 12660, 3772,
                36640, 60288, 42816, 17004, 53576, 64600, 29360, 6240,  52680,
                22696, 63200, 1648,  37396, 17544, 12196, 44868, 27856, 1096,
                59256, 30564, 53248, 2680,  60672, 13792, 37252, 2196,  7268,
                64372, 28384, 39216, 51460, 30924, 5744,  27284, 20296, 35496,
                548,   59292, 11744, 5836,  42760, 54152, 8176,  50136, 12560,
                43088, 36044,
            },
            {
                14904, 10148, 31892, 22772, 12304, 45720, 28108, 62800, 5328,
                32604, 22280, 10056, 39904, 34740, 50052, 5152,  26600, 35752,
                720,   58828, 25292, 48596, 52788, 61020, 11696, 57096, 20676,
                29812, 4684,  18848, 50624, 9376,  46288, 53088, 5176,  22816,
                58036, 6344,  18984, 1332,  48784, 45364, 56000, 10704, 41580,
                34056, 23916, 17796, 8668,  33172, 53944, 62332, 1988,  39996,
                30840, 48516, 34240, 13756, 44916, 8216,  56280, 712,   10904,
                57440, 23752, 35672, 9792,  15324, 28880, 51392, 12036, 21852,
                38012, 64124, 23556, 17132, 5960,  26648, 19620, 45496, 53980,
                24972, 10240, 31708, 13000, 44568, 20140, 448,   41824, 26316,
                34760, 44140, 29464, 21056, 57968, 6544,  54308, 33092, 51844,
                23032, 16408, 34188, 44416, 7980,  27992, 23120, 56900, 42016,
                20008, 58204, 4044,  12848, 62928, 37504, 49276, 57228, 24312,
                52444, 28492, 17116, 50916, 30812, 2408,  58936, 19040, 46800,
                3188,  60464,
            },
            {
                51284, 44844, 776,   64908, 18424, 50196, 2604,  39104, 14912,
                47768, 60084, 54728, 2560,  24876, 29788, 12108, 64456, 53708,
                8536,  43008, 62496, 36852, 2892,  32200, 15460, 41196, 7588,
                48800, 25940, 58152, 3212,  14996, 57012, 27884, 37748, 61456,
                15524, 43304, 54168, 59424, 13528, 32788, 22576, 28708, 52176,
                60060, 3888,  50160, 63444, 42804, 23456, 48112, 12976, 18324,
                60556, 52780, 38600, 62636, 18956, 51004, 31792, 20988, 40504,
                30000, 4460,  19292, 48828, 38528, 59316, 3644,  35392, 54660,
                8696,  32848, 56868, 41724, 35816, 55228, 8660,  33004, 49424,
                39736, 1748,  56168, 50368, 6916,  24276, 33444, 55220, 4680,
                12080, 58972, 9324,  50940, 31680, 25472, 64808, 4616,  36936,
                11400, 62508, 56228, 19336, 40968, 51484, 63476, 45752, 30164,
                10936, 36016, 47488, 25124, 55456, 16880, 2644,  41868, 14176,
                7708,  47048, 34520, 62376, 40636, 24096, 37736, 28152, 34368,
                54948, 6900,
            },
            {
                20452, 40432, 54552, 26640, 36612, 8588,  58508, 24628, 52684,
                18904, 6904,  36772, 45332, 17800, 58704, 39348, 20920, 47320,
                16916, 12740, 29160, 5640,  18176, 45932, 50692, 23616, 34096,
                60404, 12128, 39988, 36256, 43032, 65404, 19644, 1548,  11772,
                29652, 35300, 9648,  27584, 64540, 7160,  17252, 36844, 1536,
                14672, 26512, 39284, 11860, 56444, 28328, 6540,  34592, 44616,
                22500, 7728,  1072,  42072, 24280, 59964, 37900, 26172, 52908,
                43820, 63076, 32776, 54024, 944,   21068, 31684, 62624, 16776,
                27176, 50476, 2864,  45008, 60304, 13336, 22708, 62208, 6608,
                16584, 21708, 63436, 27004, 37588, 57776, 46328, 15484, 62216,
                48052, 39752, 16532, 3300,  46284, 40508, 15396, 19576, 49676,
                40048, 46132, 25804, 5300,  59076, 10176, 15776, 4664,  49872,
                17232, 53532, 256,   21152, 33564, 43904, 10860, 64852, 32016,
                40032, 58240, 3612,  21012, 13100, 47840, 16200, 65408, 10576,
                25060, 29612,
            },
            {
                62608, 16456, 33440, 5004,  60976, 43044, 29860, 11276, 35260,
                43764, 27420, 13956, 62992, 51756, 8108,  1480,  57224, 30908,
                24392, 50968, 40780, 55592, 10380, 26944, 64964, 2276,  54596,
                43736, 16688, 52372, 22484, 29988, 6944,  33392, 45116, 55900,
                50412, 21000, 47096, 37644, 24028, 41264, 49320, 61872, 44272,
                54032, 31260, 46808, 21412, 88,    15944, 40756, 63216, 54640,
                26208, 15328, 57692, 29532, 11260, 5536,  46884, 13640, 3024,
                58904, 16284, 7132,  27340, 56964, 13976, 43556, 47268, 6524,
                40648, 11608, 20792, 29496, 4784,  51104, 38168, 43092, 30180,
                59624, 45224, 34756, 4220,  18108, 60712, 8528,  30108, 36868,
                19128, 24724, 54172, 62816, 35360, 10708, 59620, 43692, 28508,
                8900,  1888,  14760, 30024, 36484, 21108, 33436, 39344, 25492,
                62556, 31600, 41460, 61248, 6428,  51100, 29060, 21500, 54144,
                17776, 26252, 10204, 44528, 56928, 6088,  52280, 304,   41692,
                57636, 37200,
            },
            {
                49000, 9056,  46508, 13356, 22016, 48776, 16868, 55396, 64240,
                220,   50584, 31168, 22864, 42076, 33176, 15692, 44404, 36552,
                3468,  63632, 33936, 21968, 43408, 58072, 38216, 19596, 9696,
                27852, 31980, 828,   61168, 49036, 10160, 52724, 23360, 38988,
                3404,  63504, 5460,  16096, 51848, 11692, 56624, 6092,  20720,
                10212, 64800, 7304,  35936, 60764, 49412, 30712, 10220, 3352,
                37556, 46316, 32740, 49112, 53936, 17068, 63836, 33460, 9260,
                49520, 36820, 22216, 46900, 41244, 9364,  25708, 52952, 19376,
                57880, 61636, 48348, 15464, 25496, 64804, 18264, 340,   27196,
                14172, 54328, 11384, 42096, 48540, 13372, 40444, 22152, 51956,
                1340,  32856, 7620,  28140, 22060, 52632, 200,   24176, 54956,
                63620, 32584, 50280, 54364, 43148, 48168, 60796, 1672,  55376,
                13404, 8424,  46492, 27732, 14808, 58788, 36652, 48124, 5780,
                60272, 37632, 51616, 28972, 36112, 32116, 60092, 21360, 45172,
                12776, 3668,
            },
            {
                23768, 59700, 28440, 57360, 40940, 1768,  32172, 7412,  21188,
                40568, 59068, 9624,  4048,  55880, 27180, 62156, 49376, 10512,
                54808, 19696, 7660,  59908, 30992, 13576, 4488,  35120, 48016,
                63344, 5696,  55872, 19120, 13544, 27044, 40936, 62068, 14536,
                26136, 32272, 42396, 57708, 380,   34440, 30388, 25964, 40192,
                34984, 57612, 18144, 43284, 52920, 25080, 19792, 58424, 51208,
                12188, 65212, 21116, 3716,  36344, 27244, 41676, 56700, 18892,
                25100, 40064, 11680, 64364, 34604, 59824, 4828,  37332, 29940,
                1636,  34692, 38824, 54016, 31988, 9844,  56192, 48204, 36748,
                50720, 8208,  23672, 31312, 52484, 28340, 3444,  65124, 10312,
                59516, 45032, 56876, 42144, 14168, 48944, 31188, 7056,  35616,
                18028, 21984, 60164, 3664,  12216, 24256, 7872,  29100, 44040,
                37488, 23092, 57168, 18572, 39580, 9200,  25436, 940,   45488,
                12760, 22912, 2112,  64160, 18756, 9000,  39572, 27296, 50120,
                18180, 31412,
            },
            {
                43388, 5800,  35184, 18740, 51040, 62536, 37344, 46732, 13436,
                25908, 33884, 48292, 19288, 38360, 12408, 21868, 5984,  25924,
                39604, 15008, 48264, 216,   52840, 24680, 49940, 41288, 15356,
                23136, 44832, 38812, 34340, 46528, 58096, 2296,  17876, 47756,
                7828,  60264, 19240, 28564, 46040, 63248, 17832, 3568,  48356,
                15500, 24336, 29260, 4928,  13340, 38368, 7792,  42520, 33964,
                28504, 18132, 43316, 9340,  61704, 22032, 2052,  45060, 29264,
                60952, 52236, 5376,  29808, 15536, 50376, 23296, 63272, 12420,
                44644, 22300, 8500,  3340,  40976, 45532, 34288, 5888,  20432,
                58744, 40068, 64096, 1276,  20324, 56416, 34468, 43240, 49836,
                26864, 12980, 37460, 4332,  18528, 64400, 39148, 57428, 13096,
                47564, 41652, 38348, 27324, 57192, 17840, 52196, 65272, 19988,
                5540,  51712, 34916, 2684,  64248, 53600, 19608, 34176, 63108,
                30248, 42860, 55520, 48716, 14164, 24740, 54916, 4676,  34152,
                63384, 54428,
            },
            {
                14452, 52248, 39296, 11112, 4016,  23432, 27516, 52904, 60940,
                3228,  56612, 15936, 65072, 45580, 53588, 34824, 58412, 42620,
                61192, 28824, 35928, 45348, 17576, 8388,  61920, 29140, 57484,
                11524, 51312, 8232,  24956, 64356, 5396,  36892, 30236, 56988,
                34652, 12524, 52988, 39552, 8348,  22252, 53736, 42800, 60656,
                51608, 1400,  45820, 55656, 32720, 64188, 47688, 1292,  23144,
                55836, 5068,  59000, 51996, 40036, 12684, 54768, 7712,  35748,
                14660, 668,   42820, 55288, 19108, 2660,  32616, 47856, 16652,
                55420, 26804, 63652, 58984, 17204, 23272, 12172, 62324, 25324,
                2968,  17168, 33576, 44872, 61056, 15648, 7380,  24000, 16976,
                31348, 20384, 62260, 46812, 33700, 8984,  26328, 44756, 3416,
                61820, 9864,  16240, 6420,  34612, 45680, 40088, 14472, 32928,
                48368, 59980, 11836, 30188, 43460, 49252, 13844, 56412, 40348,
                8672,  16116, 33512, 6472,  40908, 45972, 61928, 15880, 10256,
                38284, 1832,
            },
            {
                20760, 64792, 25320, 30876, 54832, 15496, 44620, 9444,  18056,
                39220, 43668, 23912, 28788, 7656,  3040,  30304, 17488, 1960,
                52136, 11752, 23540, 64440, 39100, 33340, 55056, 20916, 1736,
                36520, 60012, 30580, 16140, 41968, 21504, 53620, 10604, 42824,
                22584, 49388, 3964,  25264, 14632, 59024, 36732, 12356, 31624,
                8752,  38108, 59268, 21584, 10476, 27716, 17464, 60576, 14096,
                49584, 38664, 25368, 15532, 29644, 33084, 50016, 23864, 64796,
                47176, 21332, 59220, 25624, 45684, 39188, 56808, 9980,  41316,
                51736, 33200, 13616, 48884, 28612, 52408, 57616, 31016, 42740,
                47212, 28220, 55704, 9904,  26052, 37292, 46636, 58604, 39788,
                6120,  54608, 2048,  24472, 52980, 58552, 15912, 20820, 51368,
                29792, 24848, 53352, 63996, 30352, 2376,  58484, 10464, 27016,
                1028,  41124, 16540, 25776, 6072,  37788, 23312, 3412,  27028,
                50672, 61724, 21304, 28184, 58760, 1440,  31088, 51452, 23064,
                47116, 28068,
            },
            {
                41628, 8212,  48312, 60352, 42548, 6828,  63764, 35704, 29288,
                51416, 5852,  11232, 36292, 50452, 41104, 63040, 46460, 21132,
                32632, 7072,  57368, 3996,  27036, 12404, 46472, 6372,  43464,
                25760, 18748, 3932,  54924, 12260, 33048, 50060, 27312, 6280,
                64088, 37176, 55696, 43780, 33620, 49108, 5728,  27744, 20236,
                63096, 14068, 34284, 49944, 3828,  40396, 53460, 36092, 30936,
                44636, 7624,  34968, 388,   47480, 59872, 16816, 4124,  39140,
                9136,  30760, 34296, 11320, 7572,  61604, 27532, 21160, 204,
                6212,  38296, 19496, 4728,  43188, 1060,  37940, 15836, 7128,
                53768, 13124, 38772, 5148,  50528, 12592, 53520, 520,   63208,
                35996, 47816, 40860, 12044, 30116, 42896, 5144,  37276, 55756,
                33940, 1368,  46888, 42272, 20104, 50520, 23336, 36980, 55516,
                63256, 22292, 45696, 56660, 61796, 10612, 31564, 59768, 17988,
                46320, 4608,  38200, 52364, 12372, 20136, 36836, 42856, 7080,
                60608, 56540,
            },
            {
                32832, 17160, 2212,  36940, 20212, 33088, 56936, 1140,  22336,
                59360, 32252, 54812, 61296, 15280, 25172, 8832,  12884, 55644,
                37296, 44660, 50248, 19468, 42152, 59564, 16560, 31612, 65188,
                53336, 40232, 48620, 61320, 45052, 116,   62540, 19320, 45796,
                16228, 1568,  29224, 10468, 17616, 64780, 992,   45028, 56472,
                41056, 26148, 18460, 43852, 61812, 24296, 46080, 2852,  11072,
                62144, 21720, 56712, 63856, 10552, 20456, 42472, 56164, 26260,
                52892, 17636, 49020, 54356, 36824, 14132, 44012, 30900, 65492,
                57804, 24992, 46336, 62488, 34828, 21780, 9616,  50056, 65144,
                35444, 23028, 61768, 19948, 41848, 32196, 28948, 21828, 9808,
                15048, 27500, 59880, 7400,  22008, 61556, 49144, 27860, 11604,
                18688, 59568, 36592, 13572, 9416,  61052, 6552,  44720, 18556,
                31116, 8888,  52208, 33916, 20216, 47792, 52640, 42344, 35020,
                11832, 24952, 57556, 43880, 8576,  64736, 26396, 55540, 18500,
                35364, 11688,
            },
            {
                24536, 44880, 57568, 13512, 26984, 50000, 10880, 40672, 47820,
                13116, 19928, 46644, 532,   21472, 57924, 38728, 49332, 27260,
                63888, 15708, 30472, 9920,  54380, 1244,  37448, 48308, 9092,
                13476, 34172, 27940, 7480,  37476, 24596, 14252, 58504, 38948,
                32140, 60988, 20888, 57572, 51252, 30744, 39128, 22928, 47940,
                7312,  2096,  54620, 30328, 6364,  15560, 58120, 20484, 26880,
                51888, 16708, 40516, 27976, 44972, 5844,  36664, 31752, 12480,
                60780, 41568, 2132,  63440, 23624, 3424,  52312, 17356, 35536,
                49808, 11340, 30496, 7888,  53364, 60484, 27088, 40428, 19212,
                3320,  30112, 48512, 57876, 2600,  64624, 17312, 44520, 51132,
                57344, 32752, 17924, 51924, 35732, 14492, 648,   40688, 63684,
                44076, 7272,  22812, 56920, 28036, 39164, 33192, 15292, 48788,
                4000,  39944, 14016, 5184,  28700, 1236,  15804, 7488,  65308,
                54984, 112,   31396, 15288, 34460, 48248, 3548,  13304, 29912,
                516,   50604,
            },
            {
                4732,  62568, 30004, 52168, 3892,  45536, 17460, 25584, 62032,
                3616,  38396, 26940, 42424, 33944, 53180, 31076, 19004, 6380,
                848,   23060, 39464, 62096, 33848, 21628, 51568, 28592, 23676,
                58820, 2184,  21976, 16676, 57676, 29592, 41464, 3504,  54568,
                8340,  24804, 46400, 40328, 7100,  25808, 14268, 61276, 10980,
                52052, 36364, 65140, 11700, 38928, 33640, 9012,  55264, 41876,
                5228,  32160, 12504, 54088, 24132, 51068, 63196, 1228,  45744,
                20868, 6384,  28444, 15688, 32816, 48080, 60312, 7124,  42128,
                22208, 15348, 55932, 39712, 17728, 13168, 32784, 46152, 58804,
                11172, 43652, 16340, 8628,  36680, 24552, 54716, 34952, 3752,
                25092, 42576, 2536,  65488, 45400, 31472, 56720, 8848,  24060,
                48096, 15940, 31992, 49544, 4516,  52784, 25072, 62628, 54120,
                27748, 59208, 36672, 64388, 44996, 39272, 58428, 22048, 27708,
                37408, 19116, 49372, 59992, 21980, 40044, 52960, 59168, 46752,
                63952, 40152,
            },
            {
                54788, 19428, 9572,  38716, 22748, 65388, 53648, 31192, 34976,
                55508, 16512, 64572, 6840,  10616, 44856, 2876,  60216, 35532,
                43020, 57984, 7780,  46716, 25596, 14148, 63320, 5504,  18048,
                42236, 46104, 63152, 49804, 9860,  52560, 34808, 22648, 11848,
                50416, 36132, 13816, 4308,  62364, 35476, 55064, 18560, 32956,
                28532, 16068, 46468, 20852, 51144, 60116, 29112, 48032, 37404,
                64728, 49172, 2240,  34380, 8664,  18340, 28896, 15096, 57412,
                35168, 51636, 38384, 45152, 57492, 20264, 39312, 10396, 27072,
                59232, 3112,  44184, 26124, 49048, 1760,  55236, 5448,  23960,
                52228, 33980, 55548, 27380, 47036, 11768, 6532,  61048, 39452,
                13032, 49072, 21260, 10340, 39280, 25540, 19504, 52740, 33468,
                3236,  54572, 40260, 65068, 17468, 43900, 10876, 168,   41616,
                12120, 23984, 50200, 17396, 54500, 25620, 32916, 12868, 50336,
                45440, 9876,  41224, 6700,  28536, 5108,  16984, 24928, 37528,
                21604, 14712,
            },
            {
                33820, 48592, 43248, 59968, 35280, 7928,  14508, 4896,  44064,
                8640,  48904, 30296, 51140, 17944, 62908, 14300, 24604, 54468,
                48492, 29172, 12696, 52816, 2732,  44512, 39820, 57320, 35080,
                54556, 11540, 38100, 32340, 19520, 43688, 5260,  47504, 65044,
                27572, 42232, 54064, 19584, 47660, 9640,  43824, 2404,  58884,
                41952, 23496, 57204, 2936,  25796, 44228, 18624, 804,   14616,
                24940, 19400, 43912, 61436, 37844, 58584, 47380, 40628, 10280,
                23352, 65064, 8808,  25872, 12008, 4468,  29456, 54492, 46560,
                33904, 62872, 6440,  37268, 23044, 63972, 42420, 29744, 15272,
                63488, 140,   40084, 21724, 60144, 42916, 49956, 20368, 29688,
                8052,  58844, 54360, 28664, 5808,  50560, 59088, 12344, 37696,
                60412, 26460, 11920, 1724,  30504, 21428, 58040, 34464, 19556,
                47052, 32444, 2388,  21452, 11192, 6884,  41816, 61028, 2592,
                30544, 63528, 24212, 53800, 62028, 45652, 32716, 10368, 43924,
                7908,  28276,
            },
            {
                6048,  25652, 12556, 824,   27488, 58352, 41344, 21632, 60912,
                24688, 12520, 58548, 36340, 22924, 28488, 47452, 40944, 9736,
                16672, 21040, 34748, 59376, 18980, 31824, 10184, 49444, 3136,
                30400, 24988, 6232,  60580, 1200,  55208, 15072, 58968, 31356,
                17060, 644,   57820, 33068, 28960, 22464, 52612, 38412, 5848,
                49056, 9108,  32068, 40352, 5344,  12904, 63364, 34996, 7584,
                59332, 29716, 55616, 22204, 13736, 3264,  25376, 54836, 30528,
                3968,  48560, 18720, 53640, 41880, 61228, 36188, 14000, 1192,
                18276, 51180, 31292, 58168, 10068, 35160, 19548, 8508,  37860,
                26004, 48972, 7224,  31172, 14532, 1808,  32468, 15992, 63832,
                45792, 34084, 37116, 14988, 62796, 1864,  16692, 43620, 29044,
                6584,  19872, 46084, 36296, 61392, 47696, 38044, 6352,  55460,
                63308, 7636,  44188, 61336, 35196, 57252, 48036, 19640, 53368,
                5584,  14792, 34220, 18120, 12048, 36632, 57184, 2924,  61388,
                51456, 58020,
            },
            {
                36848, 62044, 16552, 54476, 31648, 18948, 47928, 51760, 28852,
                39244, 19772, 1484,  41660, 5080,  56152, 37676, 6652,  31560,
                65280, 4292,  49776, 38392, 6960,  60924, 26664, 21208, 16244,
                64464, 52036, 14480, 45200, 28900, 25744, 39804, 20708, 6872,
                45260, 23464, 11108, 63408, 3120,  15064, 60532, 26576, 20132,
                63988, 14184, 52284, 62088, 36184, 54340, 22400, 50556, 41404,
                46568, 10856, 5816,  52040, 42640, 32276, 7404,  61904, 17104,
                43696, 33660, 59756, 420,   31776, 22728, 49732, 64200, 25260,
                40020, 21092, 12692, 53612, 16516, 46892, 60100, 51548, 45188,
                58292, 12768, 19280, 62448, 52984, 37424, 57624, 41412, 26272,
                5024,  18444, 23600, 48120, 41484, 32344, 46832, 22176, 64288,
                50032, 42536, 56304, 23580, 7900,  13016, 25920, 51308, 29680,
                14524, 38880, 26444, 52232, 30420, 4260,  16500, 36148, 25160,
                39768, 44176, 57668, 51316, 928,   26496, 48556, 20772, 30704,
                17664, 41740,
            },
            {
                46824, 22136, 50888, 40212, 45284, 3792,  36528, 10808, 2708,
                54504, 46512, 31956, 52884, 61508, 11892, 19976, 51976, 60540,
                25968, 43836, 55384, 27968, 15024, 42892, 53696, 46220, 38700,
                8440,  41596, 21864, 56996, 36780, 63784, 10764, 34508, 53256,
                60108, 38840, 51052, 35900, 41544, 49832, 31300, 11856, 36548,
                44304, 108,   28024, 16744, 46240, 30796, 10016, 27240, 56656,
                32548, 17084, 39700, 26680, 63024, 49876, 20240, 38656, 52940,
                12716, 27352, 37560, 14908, 46144, 5672,  17080, 43276, 56388,
                8788,  48260, 28872, 41780, 5668,  27200, 2544,  22108, 31856,
                4244,  35728, 56116, 43492, 28104, 4420,  23208, 10116, 55824,
                52368, 1408,  60440, 11512, 55156, 26028, 8608,  35008, 3856,
                15444, 10236, 33736, 52652, 16696, 59560, 41296, 3316,  17652,
                22992, 59056, 9908,  41060, 14152, 46004, 28436, 62844, 10276,
                59512, 21528, 8532,  29620, 42140, 63192, 13812, 39536, 54320,
                11420, 1944,
            },
            {
                32484, 5028,  9932,  63776, 23860, 13808, 56536, 61940, 17620,
                34140, 64172, 8364,  15996, 24432, 44296, 3580,  34100, 13620,
                39692, 1216,  11188, 22416, 64028, 35964, 248,   12540, 57588,
                27756, 33712, 48156, 4396,  17584, 7944,  48992, 42768, 2596,
                27924, 14460, 8300,  25332, 17476, 45760, 7748,  57476, 53876,
                24080, 34336, 59592, 20320, 7096,  58472, 39016, 15032, 2208,
                21204, 60588, 36260, 260,   15656, 9524,  34852, 46436, 1556,
                57840, 21300, 7896,  55652, 62684, 28236, 9984,  32512, 4144,
                36740, 60488, 1816,  64840, 33616, 55972, 38584, 10960, 63084,
                16452, 41128, 23548, 9832,  50824, 17976, 63144, 47804, 13628,
                38624, 43380, 31008, 7228,  20428, 38884, 61464, 52300, 56864,
                39984, 30852, 63312, 5620,  28396, 49168, 32108, 64640, 44784,
                35844, 53864, 1180,  19888, 65080, 23552, 54828, 572,   50700,
                31724, 47276, 3784,  37992, 19964, 55136, 6880,  34804, 23728,
                65120, 27636,
            },
            {
                59476, 55876, 18468, 34784, 7376,  49740, 30556, 26212, 6056,
                42312, 13328, 27100, 48552, 35552, 29900, 57524, 49612, 23000,
                58572, 18252, 47360, 32508, 50676, 5088,  24792, 31456, 61616,
                18416, 2792,  59812, 13076, 31936, 52156, 23224, 61896, 18724,
                32648, 48196, 61244, 55184, 4564,  65284, 39028, 2060,  16336,
                47588, 9552,  41856, 49896, 25152, 3348,  44436, 52460, 65476,
                5272,  48872, 54068, 30436, 44816, 58300, 23692, 28584, 6044,
                63724, 50192, 42948, 23940, 39540, 51248, 58928, 21776, 53076,
                26816, 15972, 44860, 23588, 18676, 49124, 14408, 43876, 53672,
                28496, 48192, 1596,  34036, 46024, 7456,  35468, 29248, 20972,
                33368, 64904, 27232, 50216, 58380, 3096,  18060, 13128, 27576,
                24084, 580,   18940, 38356, 43708, 2160,  20552, 10588, 56572,
                5556,  46960, 32992, 50364, 37636, 8404,  42928, 13256, 38908,
                16876, 26896, 64848, 49684, 15516, 25248, 45040, 3260,  50532,
                43292, 15436,
            },
            {
                48736, 25164, 39208, 28460, 58876, 42736, 20632, 37924, 45888,
                58720, 21920, 56284, 39436, 156,   63396, 17076, 8812,  45716,
                30516, 37028, 62232, 8540,  41348, 19740, 55112, 47648, 7036,
                43488, 51096, 24632, 38148, 55464, 41160, 29656, 12808, 57244,
                5292,  21684, 43568, 30284, 19616, 34016, 22940, 27784, 61832,
                30756, 4140,  55768, 12548, 37196, 61400, 18464, 23840, 35236,
                42392, 12204, 24640, 19352, 64332, 13012, 51212, 41772, 16156,
                31924, 35776, 11388, 3044,  18384, 34764, 13296, 47164, 40928,
                57936, 11616, 35340, 51988, 9080,  30524, 60876, 24760, 5968,
                19824, 57000, 65480, 12512, 26412, 58912, 40356, 51488, 3884,
                8880,  16860, 46416, 14044, 35604, 44604, 31704, 48024, 42160,
                60200, 51264, 46372, 58100, 13852, 53560, 34896, 24784, 39872,
                12800, 27556, 16980, 60208, 3740,  31360, 58700, 20588, 34452,
                5360,  55572, 11824, 32860, 59760, 9752,  29992, 60864, 18964,
                8492,  37120,
            },
            {
                13120, 44288, 3600,  53148, 11636, 480,   65052, 15116, 51952,
                2364,  31196, 6644,  50844, 10956, 21396, 41336, 27676, 2564,
                53704, 5900,  15796, 56224, 28980, 59852, 14784, 37908, 22220,
                35384, 11116, 65452, 45416, 20308, 460,   6528,  46596, 38428,
                53912, 35068, 1204,  11984, 58940, 52496, 13412, 41112, 50464,
                18864, 38420, 63640, 28540, 33224, 54772, 9348,  31056, 47496,
                28080, 57608, 7852,  40644, 4492,  37236, 2672,  56328, 19748,
                47684, 25768, 59928, 54588, 30728, 6128,  65268, 1100,  19008,
                7516,  31428, 62708, 3508,  41232, 57832, 536,   34476, 40296,
                8840,  37208, 30444, 52688, 20400, 15760, 956,   44748, 61688,
                57400, 24880, 54292, 344,   22848, 62304, 10756, 4540,  20736,
                7504,  36040, 11028, 22768, 26840, 61964, 8284,  59240, 50920,
                30888, 62880, 22356, 41872, 12100, 26236, 48088, 52888, 62412,
                45232, 23664, 42360, 1420,  36588, 46996, 56424, 40524, 33408,
                53636, 1000,
            },
            {
                20784, 31160, 62384, 16428, 46552, 32996, 54432, 8976,  24072,
                35244, 63008, 19024, 44624, 60900, 36880, 54748, 12688, 64520,
                42972, 20488, 26312, 34924, 1728,  44808, 9344,  51940, 62676,
                1412,  30540, 16460, 7644,  27396, 57952, 63092, 25480, 16600,
                9464,  64136, 40932, 27092, 47720, 8472,  44356, 56940, 5884,
                11292, 46772, 22268, 15716, 1544,  48568, 41188, 13664, 6752,
                17436, 61852, 32964, 46388, 52856, 26920, 33608, 62008, 10144,
                6776,  40360, 15124, 44500, 48960, 27140, 37308, 43920, 24924,
                50204, 38868, 22000, 46788, 13252, 26752, 20536, 47348, 51256,
                61592, 14776, 3960,  42164, 60524, 32280, 55408, 23576, 12616,
                36500, 39716, 6256,  41836, 29444, 56012, 38088, 53152, 65288,
                28580, 16128, 55400, 32408, 4864,  41908, 47464, 16012, 19688,
                3476,  54996, 7732,  45712, 57292, 36236, 17640, 2352,  29128,
                7916,  58212, 18288, 53440, 21964, 5608,  16524, 12252, 22904,
                28996, 63916,
            },
            {
                40980, 6736,  49492, 22580, 36752, 26048, 18736, 39868, 48672,
                11812, 41792, 25832, 14924, 32372, 4844,  46988, 23772, 33288,
                50140, 38976, 60716, 48548, 12916, 39508, 25408, 17644, 28100,
                56968, 41488, 54312, 49680, 33132, 15068, 36072, 50272, 30776,
                45096, 23480, 50808, 17048, 37600, 2976,  31544, 20936, 35460,
                54124, 26036, 58960, 44964, 6476,  20644, 62476, 56952, 51192,
                964,   36864, 22352, 10732, 61068, 16992, 21860, 44052, 53384,
                29204, 64452, 1656,  20980, 9660,  61328, 16104, 55628, 29996,
                60860, 4944,  56076, 17704, 52996, 36856, 64512, 11560, 17224,
                27492, 45212, 22504, 49652, 5412,  37868, 9188,  28568, 17816,
                49140, 30880, 60132, 51156, 18952, 8004,  25380, 14824, 33128,
                49548, 44300, 1684,  63828, 18040, 37192, 28676, 756,   44092,
                38080, 49040, 34372, 15576, 5220,  64112, 24356, 10660, 40828,
                14856, 50176, 31212, 39696, 63360, 27960, 51588, 62012, 4808,
                45928, 57648,
            },
            {
                51920, 35100, 9484,  59988, 42092, 6076,  56672, 60684, 3652,
                28212, 55788, 52436, 8572,  57956, 28936, 17780, 60056, 7500,
                14948, 10568, 4424,  22668, 53408, 63840, 6144,  34228, 46736,
                13380, 20968, 37088, 3440,  60768, 10784, 43052, 20812, 2464,
                59588, 14080, 6436,  54908, 62956, 24488, 59648, 15928, 64360,
                760,   32684, 8828,  39896, 53528, 35796, 27528, 23204, 38892,
                44292, 54972, 14840, 50208, 30624, 7180,  38608, 13520, 49624,
                23376, 36620, 57312, 33752, 52100, 40164, 22756, 8308,  12112,
                45740, 15164, 27888, 33204, 7924,  44344, 5308,  55428, 32892,
                2580,  58420, 34820, 11040, 25544, 47092, 62468, 42968, 53036,
                4700,  21476, 11220, 15660, 63544, 34608, 45644, 41200, 3288,
                59192, 24548, 39352, 48636, 52536, 10000, 56384, 33696, 64508,
                23816, 10296, 27032, 40052, 53524, 30576, 43640, 52008, 60096,
                36788, 26060, 3696,  13384, 8656,  35004, 43740, 25500, 38104,
                10656, 17136,
            },
            {
                23956, 27896, 14652, 53700, 2028,  29332, 13632, 32056, 44424,
                17100, 37564, 1004,  22240, 49088, 39264, 2120,  51472, 36584,
                29872, 57736, 45948, 32212, 18232, 29252, 55568, 42792, 4096,
                59436, 9052,  25352, 44836, 18888, 39912, 53304, 7856,  55804,
                33572, 39560, 29440, 20144, 34588, 10896, 46392, 27564, 38120,
                43812, 13972, 51228, 30244, 64972, 16760, 11280, 4660,  31884,
                18960, 64632, 26068, 2432,  41632, 56768, 59996, 544,   32324,
                5016,  17376, 46412, 12632, 25568, 4780,  58440, 33344, 53868,
                36128, 2492,  64024, 40200, 58272, 22660, 30104, 41444, 24264,
                53436, 39036, 17916, 63956, 54264, 14340, 19764, 35256, 2088,
                64652, 33680, 43756, 26888, 47844, 1932,  21692, 57272, 12192,
                19652, 8712,  30124, 14108, 21676, 59640, 25424, 7164,  51800,
                14388, 60512, 56160, 18832, 104,   21744, 47632, 6620,  32472,
                19992, 64644, 44360, 47780, 59384, 19100, 424,   49348, 55980,
                34156, 2248,
            },
            {
                47888, 58756, 39156, 19632, 64448, 47216, 51136, 20680, 63596,
                5572,  58976, 46180, 33584, 65424, 10844, 26632, 44164, 19908,
                53916, 24832, 62976, 41932, 3160,  50232, 11284, 61092, 23228,
                32492, 51808, 63964, 29020, 48760, 5484,  24308, 64764, 27256,
                11928, 47100, 61736, 232,   42732, 49800, 6948,  52440, 4256,
                23040, 61884, 19544, 3276,  24328, 42356, 57764, 46724, 60172,
                8728,  29296, 5276,  47596, 34248, 19284, 45448, 26268, 62984,
                42884, 55432, 8068,  61892, 31132, 43392, 316,   48060, 20548,
                41540, 24164, 50860, 10772, 48520, 1080,  16248, 62804, 9384,
                46728, 6868,  29276, 92,    40912, 31064, 7184,  57908, 26324,
                46344, 8592,  59128, 38452, 5896,  53888, 61472, 27680, 50952,
                36808, 63056, 43484, 6300,  35116, 45248, 12888, 41088, 20724,
                30140, 5912,  46224, 33060, 58932, 13192, 37996, 16624, 56568,
                1356,  11148, 54304, 23076, 29380, 56852, 41424, 14748, 20504,
                30212, 61672,
            },
            {
                12700, 3688,  45296, 31384, 24844, 10828, 38308, 8024,  34776,
                23660, 13144, 29696, 18432, 42412, 14788, 56092, 62616, 6108,
                40500, 1628,  16432, 9180,  35148, 21076, 37664, 15076, 48164,
                39564, 17124, 888,   12220, 56644, 31972, 14688, 35140, 44224,
                18372, 5076,  52864, 25904, 58192, 14412, 19072, 30360, 41672,
                57432, 47228, 36604, 10208, 49024, 33308, 2460,  52456, 13444,
                36068, 42776, 53900, 59288, 23872, 12336, 8316,  54296, 15364,
                39268, 28288, 20372, 37620, 50260, 18700, 64340, 27536, 16440,
                61552, 6324,  31596, 19236, 26288, 60456, 35576, 50104, 13800,
                59524, 21200, 52064, 44716, 60236, 23152, 50168, 12784, 39652,
                16736, 23736, 55352, 13472, 29820, 17936, 10008, 39212, 5384,
                31492, 16960, 56064, 27236, 61668, 4264,  31952, 48780, 62456,
                37496, 42932, 3056,  23088, 49384, 9168,  62552, 28128, 23680,
                42224, 33852, 38456, 16392, 4168,  32728, 10020, 65312, 6444,
                53332, 43208,
            },
            {
                8564,  35892, 56308, 6388,  16952, 43120, 57628, 27364, 54164,
                40716, 50324, 61096, 7108,  53344, 4020,  34676, 23364, 12392,
                32940, 48792, 27656, 56736, 45380, 64760, 26524, 1984,  29956,
                7372,  55204, 36168, 43436, 21728, 62300, 38388, 1592,  59028,
                51348, 22420, 36576, 9440,  32204, 39200, 63168, 54624, 12180,
                7604,  16996, 28784, 55336, 60936, 14980, 28412, 39640, 25076,
                49840, 21320, 10988, 16492, 37612, 51088, 29584, 35404, 22372,
                10768, 48668, 2872,  59108, 14844, 7208,  54764, 38068, 9368,
                52836, 44024, 57196, 13584, 39064, 45604, 4120,  19796, 31968,
                37340, 26732, 12124, 33240, 16196, 4480,  37244, 54472, 62064,
                31740, 51504, 816,   35988, 42404, 49512, 32852, 44724, 24144,
                53548, 47644, 2928,  40080, 50704, 19600, 54980, 1788,  17336,
                11240, 53100, 27824, 65436, 40680, 35016, 54612, 45068, 5244,
                61040, 49692, 7300,  62856, 52728, 37036, 24352, 46120, 27252,
                38924, 18268,
            },
            {
                22988, 26588, 61360, 50088, 33320, 1504,  62228, 15736, 3536,
                45168, 10332, 25700, 37112, 21368, 48644, 29072, 46456, 58480,
                18504, 61488, 38064, 13284, 52800, 5760,  41320, 57916, 50712,
                19952, 45872, 59136, 26336, 4668,  52472, 17880, 47736, 8952,
                30404, 40632, 56548, 16412, 45908, 22960, 2064,  35980, 25416,
                33684, 59776, 40244, 5788,  20176, 44148, 8132,  64084, 18064,
                96,    61412, 31604, 40740, 63716, 5612,  46652, 60628, 1836,
                52476, 64952, 24652, 29756, 41964, 34684, 22388, 46792, 13040,
                29404, 1376,  35296, 63584, 8204,  29164, 54512, 42872, 57308,
                1780,  64588, 40004, 8724,  56760, 47536, 27412, 20516, 10024,
                5348,  44452, 19180, 63688, 22632, 58732, 14920, 64888, 100,
                13008, 59032, 22072, 10584, 15720, 37848, 28752, 58944, 24908,
                35400, 57372, 14940, 7204,  17492, 25476, 2244,  12488, 52452,
                29588, 14208, 21732, 27384, 44540, 12656, 60204, 50796, 2860,
                33352, 63424,
            },
            {
                55260, 40668, 13600, 20788, 53196, 36800, 22856, 48824, 30988,
                19880, 56448, 2100,  31760, 57744, 39520, 16836, 788,   42000,
                10004, 51916, 7136,  30808, 23740, 18788, 33456, 10052, 61808,
                24508, 12852, 33932, 9264,  40620, 29828, 11424, 42524, 25024,
                62060, 13408, 2500,  65352, 28524, 5796,  60384, 48148, 20740,
                51420, 916,   45576, 26384, 53176, 37312, 22712, 55604, 34012,
                45180, 57080, 7436,  26608, 3000,  20804, 56916, 40060, 18000,
                43564, 33928, 5388,  45588, 11244, 57444, 2216,  31944, 59936,
                39592, 23220, 49308, 17040, 52488, 21532, 11444, 25232, 6124,
                46052, 18260, 49604, 24156, 61716, 43328, 1288,  34300, 41512,
                48916, 28692, 56616, 9548,  4088,  26344, 7220,  55292, 20128,
                34944, 42012, 26428, 32828, 64676, 46608, 8748,  42464, 5380,
                47800, 21480, 44680, 31380, 51592, 59404, 20716, 32236, 39256,
                18492, 46088, 58668, 1020,  40476, 18020, 7876,  21520, 57656,
                15700, 49116,
            },
            {
                30724, 196,   44280, 9564,  28332, 59580, 12320, 7440,  39088,
                64680, 16576, 42644, 61840, 12620, 8996,  63080, 54200, 36436,
                26200, 21392, 43732, 59680, 384,   47824, 54860, 16468, 42864,
                31756, 3092,  65196, 16268, 49312, 60696, 21028, 57980, 4012,
                50024, 34208, 20816, 48676, 53160, 41116, 13508, 43844, 9224,
                64228, 14768, 31748, 11352, 62040, 1880,  48328, 12316, 4996,
                29024, 14892, 48696, 52964, 44460, 33228, 13480, 25372, 7920,
                30860, 14004, 56276, 50572, 17320, 63300, 26044, 51224, 4456,
                19448, 56216, 27136, 5632,  32576, 59392, 40804, 62076, 15336,
                30456, 53136, 3832,  35856, 28992, 18548, 12000, 65132, 57652,
                15424, 25184, 39192, 34404, 52708, 47560, 37352, 41020, 29064,
                46940, 8324,  60680, 52896, 1044,  23724, 13980, 51832, 62912,
                32880, 9540,  61492, 38220, 4072,  46872, 41892, 64020, 8420,
                56140, 36216, 10448, 31712, 47732, 55496, 35156, 29432, 42216,
                11304, 5980,
            },
            {
                19412, 34212, 47932, 64924, 4788,  41172, 46376, 55588, 26152,
                51360, 33816, 5532,  46816, 27928, 51220, 24344, 5212,  30600,
                65032, 3672,  34348, 15428, 40536, 62352, 36932, 28252, 6068,
                53140, 38768, 46952, 22700, 54180, 360,   33280, 37376, 28004,
                16848, 44676, 7648,  37684, 10408, 18212, 32644, 55312, 28368,
                38376, 56648, 23544, 50456, 34744, 17384, 30772, 42460, 63048,
                38544, 23964, 35356, 10124, 17560, 54868, 28764, 62720, 48132,
                59472, 19968, 37988, 23304, 7752,  36220, 40880, 14544, 44412,
                65508, 9936,  42380, 46660, 3028,  36460, 47908, 7560,  34344,
                56208, 22324, 42516, 14324, 6680,  54900, 50312, 22228, 30764,
                2468,  60196, 12984, 45452, 17092, 30864, 11452, 61112, 15780,
                51572, 4068,  18452, 44364, 35868, 56880, 31232, 19960, 38648,
                26900, 908,   13440, 55296, 28480, 10420, 15932, 49164, 26248,
                4768,  23824, 50992, 64544, 5480,  25840, 62680, 1860,  37688,
                45276, 60452,
            },
            {
                50988, 15092, 37904, 24976, 18236, 31908, 15204, 35252, 752,
                11120, 23628, 54564, 20876, 15508, 35264, 43848, 19836, 49196,
                14060, 45664, 56932, 51152, 25552, 12084, 4092,  22368, 45256,
                14236, 57300, 26844, 36020, 7700,  43768, 13748, 55844, 6312,
                52532, 62896, 30856, 56272, 23536, 59272, 25896, 2888,  17072,
                5524,  42764, 19480, 4428,  41028, 54672, 25692, 59124, 8412,
                51424, 19540, 58580, 42100, 65504, 884,   36724, 10972, 3972,
                53556, 41416, 512,   60776, 28944, 48608, 20260, 54332, 6892,
                34160, 15888, 37964, 62612, 13676, 23884, 17792, 50900, 27280,
                38756, 9764,  63520, 58496, 32572, 39320, 45196, 8168,  37772,
                53596, 6576,  20832, 62652, 3272,  58384, 21924, 1692,  25512,
                63576, 38956, 30060, 12824, 6208,  40612, 59836, 3572,  16720,
                54448, 49496, 41588, 18772, 24188, 36768, 57020, 456,   34704,
                61208, 42836, 15628, 38340, 19864, 13048, 49656, 16788, 53244,
                23756, 27520,
            },
            {
                8356,  54580, 58952, 2636,  49908, 57132, 21904, 63388, 43624,
                29880, 60072, 37872, 41068, 1792,  59060, 7256,  56232, 38952,
                60388, 10696, 28924, 19020, 8180,  32500, 58492, 50268, 63264,
                8744,  20244, 2368,  62620, 29388, 18988, 61540, 46092, 22956,
                39720, 12276, 26464, 43244, 600,   35516, 45996, 50852, 62168,
                35044, 49148, 59492, 9716,  65160, 14444, 3304,  21032, 46992,
                13376, 2476,  28120, 5944,  22892, 39444, 50548, 45148, 24020,
                34512, 27124, 12060, 52336, 32424, 4236,  10908, 61408, 24040,
                30788, 50516, 21072, 53644, 30036, 57892, 1900,  64980, 12348,
                45048, 464,   17440, 48268, 25032, 2664,  60428, 26668, 17164,
                43236, 48220, 32668, 40448, 27800, 49816, 42980, 53688, 33504,
                10168, 22996, 49324, 54784, 21336, 27796, 47636, 11764, 45072,
                64844, 22480, 34004, 60136, 5428,  62764, 30892, 21444, 53400,
                12028, 28868, 2712,  54704, 30348, 43772, 9392,  59896, 32164,
                4024,  64044,
            },
            {
                40376, 21380, 12040, 29052, 42588, 10248, 52580, 5284,  19156,
                47864, 14028, 7972,  49812, 64428, 29120, 12096, 32968, 17360,
                1128,  23532, 37808, 42756, 64180, 47112, 39484, 17260, 34956,
                30552, 42116, 47900, 11704, 51020, 39248, 25724, 9192,  32104,
                2420,  19136, 59944, 5468,  15488, 64432, 39876, 8376,  22196,
                12796, 30220, 24932, 37732, 46076, 28180, 52140, 36464, 32020,
                40840, 55216, 61480, 47744, 31124, 14928, 20568, 57416, 17176,
                6508,  47160, 64592, 15772, 42052, 57872, 38240, 28056, 47224,
                58780, 192,   6244,  26120, 10244, 39048, 43096, 31984, 20600,
                59748, 51676, 29984, 37180, 10484, 20124, 52388, 13496, 35712,
                63740, 23696, 10952, 55740, 8028,  36164, 18696, 5892,  57404,
                44992, 36668, 58596, 16496, 64152, 9684,  51200, 24476, 35744,
                7628,  30032, 3328,  46360, 51860, 43532, 13960, 47728, 39960,
                18400, 45624, 60140, 22852, 34480, 58172, 39668, 22164, 46712,
                13448, 35412,
            },
            {
                1160,  31476, 45548, 62512, 35756, 6908,  27560, 39340, 61368,
                3676,  57812, 32300, 18508, 24872, 45508, 52948, 41728, 25956,
                47984, 52204, 6336,  55712, 3060,  21048, 27152, 1664,  54732,
                24404, 60168, 15624, 33116, 4824,  58264, 16480, 49636, 64892,
                54744, 46612, 34408, 50148, 53968, 19996, 11740, 29036, 58044,
                53372, 45044, 308,   16164, 33376, 6656,  57060, 10692, 62500,
                25132, 16368, 33968, 11828, 52160, 59636, 8344,  32628, 62316,
                39620, 55268, 19296, 8116,  45288, 22492, 1744,  17676, 42744,
                12928, 35764, 56528, 44600, 48656, 19364, 54288, 4508,  25632,
                35592, 5452,  23388, 55104, 42284, 62940, 31764, 4532,  56324,
                28856, 1260,  52060, 15900, 25008, 65416, 12944, 31176, 48244,
                14864, 7432,  3088,  42208, 34688, 1516,  31616, 61216, 43112,
                15476, 58436, 11324, 38732, 17192, 8928,  27408, 6564,  65176,
                32312, 9068,  51908, 6276,  48436, 560,   26892, 7144,  56268,
                18436, 43548,
            },
        },
        {
            {
                23192, 35524, 14776, 58172, 24132, 20448, 37600, 57544, 12084,
                60824, 38992, 7128,  25408, 49448, 4996,  40528, 65200, 10264,
                32528, 17000, 56100, 20044, 60964, 27908, 54576, 41104, 25680,
                3964,  28708, 13408, 62080, 42164, 29464, 51732, 14180, 64948,
                48176, 56752, 21496, 33324, 18648, 49628, 41556, 28288, 52572,
                48200, 32616, 23852, 50912, 34196, 14792, 55484, 3396,  61572,
                53908, 39480, 19756, 5564,  41924, 36036, 54612, 4000,  52256,
                18200, 6508,  28556, 22344, 35180, 62260, 47264, 53692, 31904,
                22268, 3580,  36848, 13464, 51804, 62908, 38244, 21068, 49920,
                14844, 55780, 26188, 30696, 37880, 12132, 28816, 44392, 8656,
                38920, 4880,  45856, 632,   13952, 20628, 53452, 16616, 46568,
                40588, 49192, 15928, 26208, 52564, 30620, 19300, 64412, 37100,
                27236, 9864,  38852, 51236, 34488, 61456, 8176,  31508, 54704,
                20360, 64736, 45248, 52412, 5024,  61592, 38832, 57148, 28516,
                41720, 11740,
            },
            {
                61452, 44036, 296,   46948, 33084, 3592,  16528, 29060, 49068,
                35068, 14944, 54980, 11032, 59136, 27952, 53708, 15448, 21404,
                60184, 1632,  41516, 36040, 51824, 6860,  46952, 10308, 65168,
                34636, 44148, 58884, 17732, 292,   10432, 19968, 34540, 41472,
                27516, 11832, 62924, 29312, 5404,  14100, 65424, 22496, 36064,
                18936, 58652, 40700, 6904,  44324, 63816, 48872, 31316, 22400,
                35676, 13212, 64016, 30536, 9652,  48496, 32044, 11692, 43564,
                33656, 50396, 60008, 41044, 51892, 4712,  13720, 36316, 64416,
                44932, 55568, 40536, 19808, 45760, 2112,  54376, 4092,  64820,
                35800, 10324, 61520, 16876, 46100, 57404, 24604, 19008, 63360,
                31692, 49268, 17268, 41508, 32916, 28376, 57400, 5672,  62272,
                12036, 32216, 64068, 9320,  43584, 1524,  41296, 49796, 12044,
                57184, 61960, 42592, 29192, 12672, 48928, 21920, 43248, 4336,
                36584, 7760,  17652, 39760, 27652, 46668, 14564, 2632,  24480,
                59132, 32284,
            },
            {
                38248, 9324,  27128, 64672, 39660, 50448, 62804, 54588, 1616,
                19752, 30168, 41560, 8,     44100, 19128, 33768, 46332, 37764,
                51356, 30072, 8844,  23224, 12464, 33188, 18524, 59692, 14968,
                49600, 8308,  37180, 50740, 26516, 47640, 58284, 6228,  53776,
                17492, 8648,  50228, 42800, 54084, 38120, 31876, 2356,  44564,
                8316,  12808, 27560, 53684, 20644, 9704,  26440, 6428,  16504,
                44068, 760,   50184, 55452, 21684, 59388, 27388, 16324, 63092,
                940,   25960, 12256, 2992,  18584, 57784, 27208, 8204,  1968,
                16680, 26020, 9412,  58496, 32992, 23564, 29248, 40892, 46776,
                24640, 20136, 50844, 7468,  33668, 2736,  15060, 54648, 34816,
                11612, 21620, 58128, 52820, 10840, 39464, 2384,  43308, 35376,
                55272, 3352,  20592, 38524, 60648, 22424, 28248, 16084, 34008,
                4684,  20516, 16496, 5964,  24700, 37680, 56484, 27812, 62164,
                14200, 50712, 32736, 12012, 58504, 21396, 31640, 51360, 44784,
                6336,  16188,
            },
            {
                49272, 21812, 52900, 18656, 12356, 7400,  25424, 42372, 9792,
                46676, 64788, 52160, 16632, 35944, 62376, 8232,  3208,  25140,
                13384, 57668, 47748, 64420, 43720, 56612, 37932, 24388, 2580,
                39756, 19848, 23912, 5568,  32592, 64060, 38508, 22468, 30948,
                45408, 37404, 23664, 508,   57564, 10040, 47772, 16456, 55952,
                63224, 38356, 216,   61036, 16928, 36020, 41808, 58084, 52012,
                60252, 27228, 34160, 17008, 2708,  37536, 7136,  47364, 23816,
                56968, 38136, 46268, 64704, 32700, 43656, 23884, 49852, 41564,
                57492, 30020, 47700, 5380,  15992, 60884, 8512,  13724, 59188,
                5736,  44192, 39352, 58812, 22484, 48804, 39968, 42972, 1260,
                61172, 27620, 36076, 7552,  25028, 60208, 50900, 18204, 26812,
                22940, 51976, 45100, 13436, 34396, 55136, 7512,  62792, 45136,
                52408, 32084, 47120, 54328, 63656, 900,   18044, 11040, 46400,
                58032, 25760, 42312, 63364, 1264,  54432, 36744, 10404, 65244,
                19988, 54976,
            },
            {
                29968, 4980,  41096, 56996, 30612, 45460, 36484, 14308, 32676,
                23176, 4436,  26756, 57860, 21960, 48736, 29576, 55532, 63524,
                43292, 6180,  26952, 16112, 1036,  28884, 5432,  52672, 45648,
                30084, 63020, 53384, 57028, 12780, 43604, 16108, 1852,  62036,
                4628,  58716, 34164, 15444, 20268, 27144, 60352, 4856,  25572,
                30604, 21188, 49352, 31848, 56616, 46996, 1984,  24208, 38720,
                11424, 7492,  40988, 24732, 45440, 52908, 61328, 39980, 19864,
                9112,  31552, 15524, 21320, 55196, 10484, 38052, 61668, 20376,
                33604, 11772, 63680, 37684, 51044, 43184, 34760, 18644, 53336,
                31936, 372,   26744, 52804, 11148, 65392, 29812, 7968,  50796,
                14424, 44876, 4396,  62636, 47592, 22080, 31436, 12860, 61400,
                8808,  29232, 6220,  58632, 17540, 47588, 10924, 38104, 23384,
                1756,  26276, 58904, 9332,  43968, 30104, 51828, 40188, 3144,
                30960, 20792, 5828,  47824, 24012, 16936, 7256,  26196, 40512,
                34128, 2220,
            },
            {
                46084, 59296, 15352, 34852, 3928,  60656, 20384, 56232, 51296,
                59312, 37344, 9052,  32048, 12744, 4704,  39152, 11104, 18324,
                32884, 39908, 20508, 35428, 49496, 60796, 21212, 32212, 58240,
                11208, 16340, 1416,  41204, 29064, 9000,  25196, 55960, 48640,
                13568, 26072, 52728, 61292, 39324, 45752, 35120, 50736, 41004,
                53060, 14716, 43756, 5624,  11796, 28424, 65208, 14376, 33064,
                20776, 46788, 56360, 63424, 10688, 14696, 29728, 4496,  50916,
                42668, 53552, 27820, 6924,  48684, 360,   29596, 14568, 52052,
                3984,  53924, 24716, 21356, 804,   56668, 25848, 48064, 37888,
                12224, 63668, 15984, 36256, 4036,  19464, 25840, 60020, 37508,
                23352, 56600, 18744, 42216, 15548, 38092, 6024,  44612, 36372,
                57156, 41596, 49768, 31356, 25568, 4412,  51536, 30008, 57884,
                41772, 12996, 39000, 35040, 21384, 14492, 59308, 23940, 35608,
                53892, 9844,  38216, 56240, 33836, 43956, 60212, 52760, 48124,
                13540, 62788,
            },
            {
                36320, 25020, 51568, 9992,  22976, 48188, 2140,  28180, 6688,
                40032, 18084, 49956, 45332, 41216, 61096, 52476, 23164, 48080,
                2324,  53276, 61884, 9868,  54820, 39376, 13884, 8560,  42212,
                26764, 48320, 34028, 21148, 46972, 60176, 51060, 33020, 40172,
                19392, 43760, 7728,  30524, 12252, 3468,  22700, 18092, 7016,
                10376, 59416, 23900, 34744, 40116, 18476, 53404, 4200,  48984,
                61708, 28752, 3372,  19160, 35496, 43716, 25748, 33808, 12656,
                58908, 3204,  62884, 39752, 34432, 60560, 17820, 45624, 7280,
                35816, 43736, 14240, 40820, 30928, 10788, 63232, 6352,  22724,
                56132, 29184, 42576, 47476, 32532, 53884, 43936, 17036, 31776,
                3224,  53376, 33412, 29468, 9756,  55520, 64856, 2028,  48356,
                15408, 19584, 2888,  64656, 37188, 42460, 61104, 20188, 15540,
                6628,  65220, 18548, 3296,  49708, 32908, 5572,  64668, 16788,
                44832, 61480, 14532, 27784, 3340,  12796, 38644, 18612, 29128,
                23160, 9676,
            },
            {
                17556, 6572,  28352, 62544, 55112, 41692, 16596, 64276, 44072,
                13372, 62028, 3008,  25076, 56660, 16056, 34480, 27716, 59472,
                14616, 30300, 45820, 5020,  26120, 18064, 47496, 63980, 35560,
                5976,  55264, 61800, 38096, 3444,  18568, 36008, 6652,  11004,
                28764, 64860, 2420,  49052, 41828, 62264, 53644, 32560, 64360,
                36800, 47188, 2852,  62872, 50460, 22120, 44172, 36676, 9108,
                54408, 15224, 32220, 50744, 59656, 124,   54916, 65056, 18272,
                35992, 23220, 44968, 13824, 25436, 52872, 40620, 23056, 65304,
                27836, 58196, 8252,  61784, 49976, 17748, 28632, 45216, 2768,
                50316, 19956, 8408,  62128, 13576, 57068, 5076,  10180, 48144,
                39732, 12072, 63952, 228,   49912, 26676, 20432, 53080, 32900,
                24168, 39212, 11660, 53420, 21924, 14128, 172,   46764, 35704,
                53900, 48040, 28276, 61208, 56116, 41348, 46592, 9212,  28720,
                1620,  49488, 19520, 59488, 50820, 30484, 63720, 5792,  780,
                57120, 43416,
            },
            {
                60468, 39440, 44360, 13164, 32008, 38100, 25232, 11128, 30852,
                34388, 21488, 54708, 29360, 6468,  20264, 788,   42840, 8912,
                37336, 57080, 23736, 12388, 33176, 43228, 2180,  23132, 52000,
                19220, 13676, 24968, 8092,  52388, 14392, 63776, 23500, 55092,
                46640, 36760, 21968, 17552, 56604, 8772,  28396, 14044, 43128,
                25864, 18820, 29760, 57380, 9892,  6620,  61112, 30240, 25168,
                42940, 38968, 5504,  23724, 40168, 9048,  21468, 47828, 7560,
                28484, 49252, 10636, 19948, 56848, 3688,  9404,  32264, 49228,
                1380,  19280, 47196, 33460, 3100,  36572, 53832, 15304, 39976,
                60812, 34928, 24408, 2440,  40392, 28040, 22208, 62744, 52072,
                25252, 20256, 45756, 59532, 35464, 40680, 14288, 29680, 7440,
                62772, 58932, 28452, 46276, 8312,  33140, 56040, 27248, 9544,
                31468, 43060, 23584, 8300,  12336, 25964, 19796, 51028, 39040,
                33144, 23312, 41864, 36120, 7800,  45620, 22128, 54684, 41404,
                32404, 50492,
            },
            {
                3124,  20904, 49344, 476,   19352, 7832,  53448, 48704, 57392,
                956,   46424, 36340, 10412, 65404, 38320, 47256, 62228, 51068,
                4116,  18824, 41892, 52284, 62716, 55980, 27900, 59524, 38648,
                4104,  44672, 31928, 59212, 28388, 45308, 41844, 31264, 768,
                59696, 14276, 52056, 33104, 24544, 37828, 47356, 1644,  57720,
                5180,  54744, 38872, 15572, 27336, 33620, 55620, 16732, 972,
                20260, 64504, 12544, 57920, 46204, 30824, 16012, 37548, 41740,
                56420, 1116,  61724, 30320, 37148, 47596, 59100, 12596, 55012,
                37876, 15700, 25792, 55504, 21100, 42824, 58868, 7680,  32100,
                11064, 44600, 57780, 17620, 51156, 36804, 45592, 33920, 6460,
                58332, 28672, 7480,  17396, 23692, 4772,  57812, 43500, 10756,
                45552, 1020,  35188, 17076, 61796, 49984, 24296, 39724, 63200,
                2328,  16704, 51620, 37648, 31868, 436,   58192, 14040, 54500,
                62776, 4896,  11460, 55712, 25472, 16148, 34332, 11688, 61464,
                25592, 14444,
            },
            {
                29844, 33932, 56224, 63620, 46636, 35532, 60380, 4540,  19028,
                26320, 52048, 14820, 43188, 50568, 31636, 11920, 25796, 21944,
                33716, 64544, 28652, 472,   36888, 7668,  15360, 11008, 29984,
                49396, 65344, 40600, 11444, 22492, 57476, 6404,  16968, 49812,
                26308, 9040,  40468, 63168, 3756,  15856, 59872, 20700, 50244,
                33960, 12200, 45044, 1492,  48236, 41492, 12416, 51132, 45512,
                56640, 34132, 49380, 17828, 26888, 62824, 51412, 3820,  60684,
                32316, 17276, 51844, 43112, 5892,  16600, 27016, 20788, 42032,
                29796, 60432, 10248, 39524, 5292,  12436, 24232, 65100, 19192,
                26496, 53472, 5596,  31368, 60044, 11252, 712,   16440, 42044,
                14168, 36284, 43588, 48600, 11548, 51880, 61880, 37748, 18708,
                50976, 26388, 40972, 54888, 4860,  43436, 18300, 57776, 12728,
                21640, 59936, 4960,  54968, 45880, 62352, 36032, 21880, 6872,
                26520, 44292, 17616, 64432, 1384,  52124, 39996, 48484, 19896,
                8128,  53596,
            },
            {
                38528, 10988, 24340, 6264,  27648, 14172, 22784, 42652, 39204,
                62500, 8404,  59556, 24008, 18172, 57672, 7324,  54136, 15040,
                45632, 10336, 48844, 16588, 21172, 50528, 45456, 34368, 57220,
                21608, 16808, 40,    50292, 34216, 3076,  38512, 62628, 20696,
                35012, 43408, 5204,  28960, 55064, 44856, 10816, 31192, 40604,
                23480, 61984, 52600, 20344, 64664, 23140, 37328, 4468,  60172,
                10116, 28048, 7004,  37284, 2032,  11440, 44044, 22928, 13984,
                25984, 9096,  39392, 23704, 64484, 34088, 44892, 62288, 3044,
                6504,  50592, 44456, 63584, 28200, 52232, 46672, 35232, 49432,
                1052,  38076, 14908, 48628, 21072, 26092, 55460, 64984, 30596,
                53756, 1932,  62212, 55428, 31476, 26180, 1512,  33616, 22352,
                56156, 6348,  13708, 23216, 30904, 11392, 3108,  36184, 44924,
                49520, 34572, 41036, 18980, 14832, 29176, 43412, 10572, 48136,
                59084, 31000, 37308, 47300, 28488, 9604,  58832, 3648,  36556,
                46264, 65080,
            },
            {
                43116, 16724, 59344, 40832, 54372, 31428, 50908, 10200, 16268,
                28772, 32924, 5416,  40140, 1844,  35364, 28196, 41504, 59076,
                38744, 6008,  60264, 40592, 31888, 25304, 61404, 2600,  41616,
                8936,  26992, 36756, 56400, 18772, 53712, 27404, 10872, 45776,
                53224, 57952, 12924, 20060, 49884, 34908, 26184, 65260, 6096,
                17040, 8624,  36096, 31480, 6260,  57616, 17840, 25664, 31844,
                40256, 22308, 53180, 42436, 55732, 33000, 58724, 5652,  53780,
                46912, 35140, 55632, 13460, 2048,  50484, 8724,  36728, 24632,
                53788, 32476, 22440, 13872, 56872, 31484, 3588,  16388, 41252,
                56180, 28936, 64052, 41972, 8544,  46956, 38588, 19568, 48412,
                23432, 10140, 39136, 21140, 14976, 42468, 9164,  47800, 15516,
                65504, 32424, 48584, 58536, 38016, 64224, 27604, 53332, 6304,
                25004, 28728, 10032, 64124, 24392, 6048,  52252, 33724, 39900,
                15484, 2248,  53124, 20732, 33400, 61788, 22816, 31112, 17244,
                27244, 4400,
            },
            {
                22612, 51508, 2160,  36072, 18252, 44824, 1424,  64964, 35056,
                55824, 44664, 21016, 48888, 63468, 13876, 46808, 19952, 2728,
                30764, 23400, 56184, 12560, 53580, 5000,  38252, 18964, 54392,
                47160, 62852, 6716,  43088, 13380, 47416, 59500, 32472, 15616,
                1840,  24116, 31700, 60532, 8080,  42288, 452,   54292, 46456,
                29088, 58848, 43264, 26476, 11168, 49820, 44568, 63192, 14064,
                48516, 3940,  61688, 15264, 24952, 20080, 36336, 28316, 40952,
                63544, 20640, 6780,  60124, 28232, 19344, 58336, 14472, 47760,
                18036, 8368,  36092, 1584,  19500, 38448, 9668,  61696, 21836,
                7080,  12668, 23580, 34024, 61272, 5012,  28572, 12748, 3460,
                60596, 33264, 27532, 5920,  63916, 35224, 54276, 59816, 28836,
                3336,  43024, 18156, 8020,  52064, 20856, 47564, 32020, 62536,
                14160, 56776, 1388,  46800, 38732, 58672, 2896,  18312, 55856,
                24256, 61124, 12520, 41448, 7072,  14628, 43952, 56260, 49712,
                12772, 57844,
            },
            {
                7540,  45508, 26680, 12308, 61188, 8224,  57240, 24916, 47816,
                3696,  11948, 53740, 26064, 30488, 55928, 4988,  61836, 52028,
                16984, 34996, 26624, 46704, 8416,  64756, 14540, 29648, 23456,
                12444, 32684, 60084, 24216, 30448, 8644,  22100, 42068, 6932,
                64988, 39268, 47688, 16688, 37616, 23072, 13452, 18960, 38184,
                51420, 2272,  14892, 55164, 40104, 252,   28968, 8044,  36360,
                54564, 19072, 30492, 46412, 7732,  64040, 48108, 9872,  17788,
                2636,  30100, 43488, 47968, 33124, 41176, 54760, 30644, 39812,
                64816, 57524, 42228, 49240, 62708, 45416, 25976, 51368, 30624,
                58752, 46140, 52624, 2064,  18228, 56532, 43872, 35556, 52668,
                40656, 17728, 51040, 57856, 46524, 18940, 4252,  24776, 39004,
                10904, 36312, 61264, 26580, 1184,  40664, 15632, 9092,  19856,
                37512, 43180, 30976, 53828, 20920, 48976, 27524, 65484, 7948,
                30096, 45528, 5120,  57684, 51100, 37544, 25844, 316,   40616,
                62756, 34672,
            },
            {
                54644, 31884, 64196, 49048, 21176, 29304, 38684, 13832, 20300,
                41376, 61484, 17164, 37916, 10392, 42984, 22320, 36952, 9460,
                44364, 63760, 1916,  18732, 42276, 33752, 48364, 59244, 51364,
                4536,  44752, 15836, 52896, 40156, 3848,  61808, 36276, 50664,
                28268, 10092, 56648, 2656,  63504, 50632, 58068, 33056, 61744,
                11868, 24764, 47984, 19884, 59920, 32964, 52192, 20980, 58964,
                2156,  43780, 11668, 38776, 52316, 1440,  14580, 56596, 50756,
                38180, 12056, 52568, 15768, 25024, 10792, 4716,  22172, 144,
                12260, 28608, 25056, 16128, 34020, 6720,  55304, 14084, 43048,
                18544, 36604, 10444, 40056, 31704, 50780, 9280,  24936, 58620,
                7216,  44620, 11960, 2288,  29668, 40936, 13512, 45356, 51464,
                20028, 53836, 46476, 13216, 34892, 44676, 60664, 55176, 2668,
                51024, 59704, 17440, 6820,  11140, 32744, 13428, 37220, 42432,
                50196, 21572, 35648, 27728, 18620, 64772, 11268, 53812, 5988,
                29068, 20064,
            },
            {
                10060, 15216, 38172, 3236,  43516, 33256, 53588, 5880,  27928,
                50752, 31788, 1076,  59324, 6756,  50608, 33156, 25380, 55288,
                13356, 39232, 51244, 58432, 22240, 27264, 872,   9808,  39992,
                35624, 28144, 1548,  19776, 49672, 57072, 26376, 17480, 55024,
                21124, 34116, 43688, 25888, 30196, 6828,  45488, 27664, 3612,
                41612, 34712, 64116, 5052,  37552, 16624, 11876, 42308, 24848,
                33740, 64648, 26908, 60040, 34884, 29412, 41672, 24376, 32544,
                62396, 22248, 57740, 1556,  63936, 45740, 51796, 61308, 34736,
                46500, 6104,  52764, 59348, 10784, 22844, 40180, 608,   33572,
                5684,  62932, 27168, 59396, 13776, 22296, 63132, 15996, 31136,
                20872, 64608, 25324, 37384, 50060, 22848, 56844, 7032,  63120,
                31272, 2184,  23544, 29964, 57436, 4968,  24780, 33528, 39804,
                22796, 12240, 26972, 36288, 60912, 44668, 23644, 54748, 752,
                10296, 63084, 14936, 48904, 3712,  31724, 45800, 21964, 16564,
                38704, 47888,
            },
            {
                42476, 24312, 58008, 6700,  51252, 16464, 59264, 45952, 63252,
                9288,  35712, 23256, 45424, 14788, 65036, 17588, 188,   48832,
                28576, 5744,  31248, 11120, 36712, 54560, 44140, 17744, 61812,
                20812, 56288, 64904, 38008, 34444, 11764, 44636, 720,   13156,
                46560, 4912,  18696, 52292, 11356, 35824, 21156, 16320, 9748,
                56744, 21916, 44848, 8352,  27448, 62696, 46992, 56608, 6196,
                15712, 48196, 9180,  17608, 4548,  21504, 45640, 59740, 8140,
                4844,  26984, 40344, 34308, 7824,  37348, 17780, 26540, 43136,
                55640, 19872, 37704, 2880,  30348, 48084, 60904, 27896, 57012,
                47072, 24212, 3136,  44208, 48324, 37208, 156,   42088, 47740,
                3952,  34784, 55664, 15496, 61664, 9776,  34228, 27164, 16576,
                42072, 56364, 9408,  64540, 50080, 17624, 11264, 28552, 46004,
                5420,  48680, 63348, 40884, 51600, 3508,  15932, 62000, 19136,
                33240, 25368, 39604, 55376, 9060,  41764, 58304, 33988, 51936,
                61000, 1824,
            },
            {
                30496, 62232, 19564, 35192, 26048, 9848,  23064, 3000,  39560,
                19208, 57460, 52960, 27552, 39364, 29860, 56800, 40716, 61428,
                21644, 16076, 45680, 63104, 3936,  15116, 57820, 31148, 25296,
                6120,  45844, 9200,  14524, 24972, 6648,  63856, 29348, 38296,
                58852, 63116, 40896, 14872, 54972, 59404, 39508, 49112, 53296,
                30304, 17384, 50048, 31860, 53532, 23184, 3252,  38656, 29832,
                51580, 41304, 23436, 57504, 54424, 49388, 12476, 18860, 37076,
                44448, 49492, 17236, 55424, 21144, 30040, 59056, 14152, 9172,
                31740, 62828, 13284, 50516, 43444, 17172, 35952, 20312, 11288,
                50584, 15480, 54796, 19936, 29560, 6616,  57836, 26688, 54596,
                14192, 39692, 7912,  46988, 32136, 464,   59144, 37968, 49200,
                4832,  35656, 15236, 39344, 21648, 43376, 52592, 63812, 58004,
                15404, 31808, 888,   24712, 8816,  57576, 30208, 39156, 45268,
                53016, 59140, 2596,  29908, 20564, 61436, 24724, 7436,  13392,
                26136, 56128,
            },
            {
                4488,  46220, 13264, 40676, 55644, 61748, 36432, 30900, 15324,
                43648, 4804,  12632, 48720, 20852, 4004,  11276, 46532, 7992,
                34348, 59492, 41460, 24928, 49760, 7240,  38912, 52016, 13028,
                48552, 41540, 53940, 31768, 59896, 42500, 48436, 32928, 52836,
                23376, 8196,  31236, 47172, 4364,  24512, 1328,  64840, 6072,
                37288, 61016, 4088,  14008, 43344, 9960,  34552, 61256, 13148,
                19744, 856,   36456, 31156, 7392,  39456, 65188, 268,   53140,
                31040, 10156, 61736, 12852, 42600, 50376, 5620,  39084, 48536,
                2268,  23084, 41236, 26452, 64588, 4020,  52468, 7888,  63672,
                31184, 38936, 34620, 8964,  65320, 51968, 17560, 34148, 10128,
                61152, 51228, 27628, 22076, 18208, 52552, 44372, 20452, 11084,
                24452, 59916, 47600, 5932,  32304, 28,    36840, 7524,  20208,
                35732, 55748, 44300, 18168, 34808, 21196, 48140, 5004,  27944,
                7292,  13180, 42856, 47408, 35956, 15752, 1288,  48796, 43960,
                36860, 18120,
            },
            {
                49964, 7828,  53020, 28864, 24,    47352, 12376, 50024, 54912,
                25080, 64156, 34056, 7904,  60712, 53724, 35504, 19784, 26168,
                52348, 2540,  56164, 18604, 33596, 28820, 21088, 64172, 3268,
                33820, 16676, 22280, 4676,  51036, 18100, 21476, 9252,  15908,
                3312,  27732, 37332, 20332, 60840, 32952, 42756, 28784, 44460,
                11956, 25392, 58316, 39256, 55908, 17856, 48688, 26232, 44996,
                58816, 53428, 63484, 14684, 43232, 24660, 33524, 28216, 14932,
                58968, 24840, 3016,  46904, 27292, 1088,  65048, 24280, 53416,
                15916, 56960, 35184, 7288,  21120, 55564, 14600, 45324, 41172,
                22480, 5440,  60536, 42916, 25212, 12652, 39520, 45560, 23024,
                30088, 1536,  43420, 63636, 5768,  40324, 13404, 64784, 55228,
                29684, 43868, 19116, 27456, 54764, 61220, 13920, 26268, 41976,
                9948,  49740, 29284, 12432, 60344, 41996, 54340, 10844, 36068,
                50436, 17500, 22896, 64096, 11412, 54272, 38844, 28952, 64316,
                10544, 32984,
            },
            {
                39100, 20988, 65476, 32588, 16888, 42684, 21540, 59640, 1728,
                29168, 47084, 17416, 41880, 24404, 31404, 43464, 63792, 13280,
                38612, 30176, 8884,  12472, 60456, 46572, 42752, 10356, 36816,
                59320, 29416, 63128, 38560, 27708, 1452,  58060, 40372, 61236,
                45832, 51312, 57000, 12932, 49904, 9512,  18408, 15052, 22512,
                47580, 34264, 548,   20864, 29152, 65020, 1904,  21680, 7756,
                32672, 10916, 27472, 46328, 2796,  60472, 9568,  56896, 48024,
                19592, 39104, 32608, 52816, 35884, 57512, 19524, 33108, 44912,
                28280, 60328, 10424, 47348, 32848, 38652, 29392, 25184, 1332,
                57640, 13304, 18440, 50008, 1576,  31548, 60220, 4272,  49336,
                19404, 56152, 11860, 36632, 58372, 30340, 26264, 4148,  33736,
                1372,  52704, 62456, 10720, 41052, 23284, 47792, 51668, 59512,
                2380,  22624, 65108, 52364, 4240,  26336, 14736, 64336, 24240,
                60540, 31908, 56500, 4512,  27108, 50660, 19408, 57284, 3800,
                23040, 59908,
            },
            {
                27188, 11516, 44264, 24644, 5720,  62896, 8188,  33600, 41008,
                11096, 37008, 58784, 3548,  50700, 15244, 1344,  57464, 5632,
                49460, 45228, 23676, 36380, 53240, 432,   16988, 56020, 26116,
                50128, 2132,  11328, 56592, 46548, 13636, 35440, 54200, 25724,
                11672, 34460, 80,    41400, 26780, 62348, 35548, 57676, 54244,
                7724,  63140, 51192, 42264, 7120,  36132, 52208, 57060, 40912,
                37756, 4140,  50116, 21008, 17428, 51516, 38024, 22284, 3860,
                42232, 64100, 5992,  22596, 16208, 9380,  41120, 12364, 6672,
                36896, 18688, 51216, 480,   61936, 11604, 59208, 49128, 33916,
                53352, 27312, 46612, 36168, 56688, 21628, 54228, 8492,  38032,
                62516, 33080, 16452, 48052, 23876, 8600,  46208, 50540, 17340,
                38348, 14228, 7652,  35288, 57796, 16760, 5660,  29624, 33756,
                17488, 39640, 7164,  37392, 46584, 33160, 2196,  38764, 19868,
                46176, 20,    40296, 34312, 44596, 6300,  31408, 41656, 16644,
                45792, 54036,
            },
            {
                15084, 56836, 3344,  50840, 36832, 56328, 27704, 18192, 53432,
                6000,  21024, 55420, 27956, 10008, 62484, 37372, 28780, 16944,
                55456, 20808, 65480, 40336, 5948,  27252, 62344, 22740, 7508,
                40648, 19376, 44096, 24416, 5492,  62584, 29960, 6948,  18784,
                43796, 65336, 17060, 22956, 6868,  45668, 51528, 2572,  39896,
                32064, 18876, 27352, 15300, 47028, 11980, 31056, 16736, 63164,
                13828, 24064, 61120, 35328, 29752, 55284, 12948, 45548, 35216,
                8664,  55796, 13948, 44260, 62560, 31140, 49716, 61376, 55108,
                3644,  42340, 30692, 22664, 44492, 16860, 4420,  19608, 9156,
                42676, 62820, 4620,  10596, 40728, 16124, 27788, 43864, 14488,
                25256, 5148,  42204, 3172,  54880, 34708, 61956, 21500, 59024,
                42648, 23100, 49844, 31092, 3264,  44916, 37964, 12016, 62132,
                45384, 57212, 27980, 16144, 20560, 56060, 49556, 58252, 8000,
                28668, 53192, 15532, 9840,  59832, 22392, 13064, 61888, 8792,
                35616, 876,
            },
            {
                41940, 34912, 60856, 19312, 40024, 13916, 46528, 23916, 60752,
                48588, 14336, 39772, 33116, 45084, 19268, 48312, 24004, 41932,
                33728, 1692,  15024, 58276, 31256, 48896, 13316, 32804, 45644,
                54580, 13968, 35764, 52992, 32896, 41792, 50216, 22852, 39036,
                4900,  28956, 53920, 58928, 38060, 30572, 5124,  24920, 13816,
                60136, 10300, 38028, 55512, 60732, 24600, 44272, 4672,  28480,
                47668, 43004, 56584, 8464,  41132, 5284,  25968, 62652, 17004,
                29848, 49528, 26560, 37252, 53932, 3920,  27752, 21820, 46296,
                25376, 63248, 13228, 57572, 26740, 40472, 54496, 65008, 36616,
                15580, 29948, 24000, 33088, 61460, 48264, 2628,  64644, 35416,
                52024, 58108, 28756, 50340, 18868, 14456, 41484, 10420, 6204,
                28508, 63692, 46324, 19560, 26584, 64832, 54544, 20660, 4012,
                25132, 53344, 10704, 43220, 62316, 6480,  30580, 17016, 43696,
                12096, 36836, 63528, 25696, 47612, 55804, 37596, 52088, 25156,
                49056, 29544,
            },
            {
                7104,  22892, 48344, 8840,  30432, 52768, 2092,  35084, 43044,
                30148, 65148, 412,   22980, 59436, 7096,  53136, 11540, 60052,
                7860,  26828, 51268, 10172, 19228, 42116, 52660, 37352, 3892,
                64880, 30744, 58776, 8616,  20372, 15704, 10564, 57268, 62052,
                47648, 32660, 9760,  48852, 14640, 19640, 64400, 42868, 29452,
                49404, 45064, 5888,  22380, 2692,  33488, 49968, 9340,  54488,
                19204, 31500, 1420,  16100, 64584, 49076, 32628, 1888,  52584,
                59756, 20648, 10692, 724,   47376, 17892, 39680, 8372,  34260,
                15644, 38444, 8788,  53068, 35516, 7496,  31288, 45564, 21708,
                2908,  51920, 58180, 19468, 6516,  53048, 30780, 20752, 10072,
                40000, 6908,  21764, 61060, 38532, 652,   31380, 57116, 36580,
                53792, 2096,  11708, 39196, 51956, 9360,  32816, 42268, 49152,
                14368, 36000, 1148,  32320, 23788, 13044, 41020, 25484, 61676,
                48836, 5468,  20668, 42160, 17564, 1316,  33252, 4936,  20552,
                58152, 63144,
            },
            {
                32228, 45480, 15808, 26252, 58384, 11384, 63548, 5256,  16580,
                8760,  26068, 52248, 11916, 41032, 35244, 3068,  31720, 46832,
                37996, 61692, 43944, 35444, 24276, 2016,  60768, 9564,  21456,
                27744, 17080, 588,   47464, 63560, 27084, 45392, 1864,  36468,
                13536, 21436, 3196,  42060, 34076, 53148, 11556, 56012, 21480,
                1188,  34484, 63984, 52424, 40752, 17736, 57960, 36976, 22932,
                59736, 11856, 52860, 38640, 20420, 10464, 43932, 23380, 39904,
                6788,  33804, 41884, 57348, 23848, 60392, 13740, 56352, 50676,
                2216,  58820, 48044, 20076, 1752,  50180, 24320, 12348, 60200,
                48068, 39956, 13820, 44648, 37264, 24752, 12892, 46720, 59220,
                17596, 44748, 31996, 10712, 45940, 65460, 26912, 48340, 15660,
                24792, 33348, 56644, 60916, 15228, 648,   24156, 60132, 7424,
                30364, 63676, 46984, 58988, 50964, 35196, 55000, 1476,  21992,
                33516, 57692, 30776, 50832, 7768,  28812, 64944, 46520, 40388,
                17884, 12608,
            },
            {
                3904,  54076, 64676, 2740,  33800, 44184, 21736, 38892, 50944,
                57912, 37412, 45884, 17648, 55736, 26544, 64476, 21560, 14252,
                54992, 17708, 4064,  47972, 56496, 29100, 15964, 46712, 56992,
                39748, 51276, 42976, 23608, 38384, 4708,  52164, 30324, 17284,
                55156, 60128, 27540, 63180, 23936, 8040,  47128, 35844, 16792,
                58948, 26024, 14980, 30128, 11212, 62124, 27420, 108,   39940,
                6800,  33828, 25284, 47216, 28512, 59440, 56784, 13300, 27596,
                46384, 15576, 65440, 30332, 35320, 4760,  43812, 29636, 64708,
                18160, 32384, 28260, 43752, 63784, 15104, 57188, 38160, 6240,
                26860, 34412, 336,   63968, 9188,  57328, 41372, 33956, 1092,
                27232, 55672, 13584, 53476, 24432, 7740,  20504, 52400, 3988,
                45316, 18456, 7360,  22260, 43728, 28920, 37088, 16964, 55976,
                40280, 21852, 18416, 9020,  4756,  44828, 64572, 10540, 38556,
                54232, 3292,  13396, 60272, 39384, 53944, 15240, 26600, 9956,
                36152, 51016,
            },
            {
                28128, 20000, 37420, 40948, 49840, 17228, 28732, 56112, 12992,
                32012, 20288, 6200,  61196, 30316, 15436, 50052, 43192, 5220,
                29324, 23212, 32624, 12700, 63796, 38380, 6568,  34184, 25268,
                11428, 6368,  60436, 13188, 54852, 33924, 60800, 41368, 25188,
                7112,  44332, 50992, 39240, 920,   57560, 27116, 4520,  40940,
                50380, 7312,  37272, 47872, 3728,  20884, 43340, 15392, 65360,
                51004, 44880, 61588, 3504,  34732, 6380,  18464, 36776, 61976,
                54848, 3476,  50784, 19172, 12004, 52708, 37588, 20968, 6316,
                40540, 24088, 11308, 4700,  33944, 41580, 29340, 18500, 52684,
                10536, 55344, 23412, 17312, 50852, 28088, 4836,  62048, 48952,
                23824, 63976, 34564, 2680,  37080, 58612, 43072, 12332, 38908,
                62944, 29824, 41064, 35064, 47428, 57888, 51304, 10380, 45872,
                2776,  52824, 28660, 37900, 15192, 26788, 18716, 29900, 47224,
                15968, 27476, 45328, 35612, 23200, 11576, 43940, 59496, 1796,
                55300, 42272,
            },
            {
                8444,  57112, 13472, 6580,  23984, 59904, 1108,  46852, 24456,
                63212, 3328,  42880, 48404, 9540,  1772,  39144, 58480, 10656,
                62704, 40400, 52876, 8344,  21168, 42948, 54288, 18956, 63392,
                48532, 31608, 36232, 28544, 18268, 8324,  21632, 12268, 49476,
                34492, 19940, 11100, 15744, 31032, 18540, 43748, 60412, 31732,
                10364, 54600, 23652, 44508, 56716, 34980, 53572, 48256, 10700,
                29528, 18068, 13600, 22136, 54320, 42004, 51092, 368,   21784,
                10076, 38820, 25104, 7632,  45300, 62768, 26608, 10328, 48776,
                53556, 45908, 61552, 55860, 22348, 9516,  60960, 3212,  43600,
                62920, 30380, 42224, 47336, 32420, 20416, 54180, 16548, 8868,
                37860, 5296,  41916, 15948, 49824, 30312, 17672, 34144, 60116,
                9124,  48756, 54928, 3032,  12636, 6044,  62072, 20220, 34172,
                25372, 12420, 60672, 48208, 57892, 40580, 52092, 4256,  60892,
                41840, 8548,  19444, 62620, 2436,  32800, 49816, 21412, 31292,
                62864, 23468,
            },
            {
                34524, 44352, 61776, 31384, 53128, 9624,  35984, 41536, 7160,
                49908, 34716, 27500, 22224, 36524, 53468, 19924, 24992, 34352,
                48268, 140,   44920, 58168, 25912, 2512,  50404, 13752, 41280,
                1656,  21052, 57992, 3564,  48932, 65132, 43800, 820,   59096,
                38152, 4128,  55868, 47312, 62280, 36928, 51992, 13680, 20860,
                64804, 39576, 18428, 61448, 13492, 8112,  31564, 23996, 5216,
                36444, 56884, 40772, 8660,  64184, 15552, 30592, 47944, 33352,
                43776, 29056, 58956, 48360, 32868, 2604,  16764, 60096, 34548,
                200,   13500, 37312, 16912, 51108, 46900, 25860, 36028, 49328,
                20888, 15568, 7376,  60276, 3380,  39580, 13456, 43568, 30848,
                51944, 19076, 47556, 27764, 63204, 5900,  55068, 2460,  27572,
                21200, 14596, 25820, 64024, 17988, 32120, 27204, 38988, 4556,
                65408, 41872, 31880, 280,   23188, 33892, 7916,  21520, 50020,
                36188, 24332, 55432, 48376, 5796,  57416, 18116, 38552, 6472,
                47100, 16392,
            },
            {
                50604, 244,   25456, 47244, 20740, 62732, 15156, 54728, 19308,
                10944, 59620, 14728, 56544, 62064, 32256, 46252, 7696,  54692,
                14588, 19428, 28056, 35936, 16716, 32344, 59364, 29976, 9028,
                61416, 44496, 15600, 52840, 40220, 25788, 31276, 16156, 53172,
                28616, 64256, 32584, 25716, 5196,  9672,  23528, 3492,  46316,
                28796, 1484,  32988, 5096,  27960, 41212, 62976, 18900, 59156,
                26488, 1740,  31968, 49316, 37608, 26224, 11472, 56376, 4784,
                63492, 17248, 13020, 54512, 21996, 39224, 55480, 42604, 22992,
                58368, 31164, 26748, 7044,  39216, 1284,  56540, 12204, 32764,
                4972,  58400, 37640, 11964, 25640, 64884, 34844, 57840, 22168,
                61488, 11632, 56904, 21604, 9696,  44236, 23332, 40860, 50928,
                57656, 4824,  37500, 52500, 23728, 49572, 42996, 13908, 56936,
                50304, 16552, 7140,  53564, 11360, 63648, 55792, 13572, 31300,
                852,   65224, 12456, 29208, 42704, 25492, 9348,  52940, 27304,
                13052, 60508,
            },
            {
                29584, 10732, 18176, 39436, 4280,  33676, 44628, 26004, 30276,
                38668, 52776, 692,   40368, 12156, 4212,  17092, 63556, 29708,
                37656, 61052, 10020, 49092, 65176, 5692,  45612, 37236, 27028,
                54008, 38548, 24484, 7168,  34584, 10740, 56632, 46108, 22824,
                9232,  14744, 43512, 21104, 39936, 48692, 61140, 34888, 56624,
                15900, 52512, 48548, 59688, 21796, 50792, 2484,  38848, 45420,
                52688, 42540, 60992, 23628, 3084,  45208, 60080, 19600, 24520,
                52372, 35356, 1704,  41488, 27632, 6176,  14028, 29520, 8248,
                19216, 43928, 64240, 54740, 19812, 30004, 15020, 65368, 24076,
                41012, 51084, 28328, 55920, 45340, 49532, 2256,  7020,  28444,
                45448, 128,   33100, 38956, 53236, 14384, 62020, 36576, 11564,
                31152, 45576, 42484, 33648, 1248,  58540, 7960,  46684, 29744,
                22068, 35724, 44060, 19668, 46616, 37212, 25576, 43168, 59048,
                18348, 52576, 33632, 39644, 14796, 59924, 34768, 64520, 41248,
                3428,  37084,
            },
            {
                58560, 42772, 55872, 51876, 28896, 12528, 57152, 2308,  63736,
                45536, 17604, 24652, 29492, 44420, 51640, 26816, 41124, 2168,
                50732, 23068, 3876,  41324, 53272, 12000, 19612, 22920, 4904,
                17088, 32772, 12452, 47552, 62876, 20112, 3024,  35688, 61248,
                40928, 2052,  52344, 58636, 12616, 54224, 29896, 41644, 8460,
                25900, 36592, 11896, 42740, 16712, 35780, 55788, 11600, 15680,
                7336,  20476, 12100, 17108, 53492, 34276, 6736,  40888, 14440,
                38424, 8088,  60012, 19804, 64552, 51444, 47200, 62412, 37092,
                49732, 4752,  10968, 33208, 42296, 59828, 45012, 7752,  47836,
                18336, 792,   33816, 21720, 8424,  17836, 24112, 53664, 36264,
                15232, 41080, 60048, 4120,  31348, 26240, 47708, 944,   19380,
                64972, 7544,  16384, 61144, 10896, 19636, 36548, 62648, 2008,
                9944,  55248, 27008, 61940, 30152, 2532,  16948, 6192,  45780,
                38288, 10188, 3368,  46360, 19976, 51116, 1412,  44652, 17048,
                54532, 20692,
            },
            {
                4480,  33316, 15348, 65352, 8180,  48632, 40224, 21408, 5716,
                32716, 8764,  48092, 64992, 6884,  35572, 21700, 57748, 12692,
                43972, 56732, 30808, 15596, 25116, 39172, 57352, 48132, 62720,
                50924, 59708, 276,   55416, 28736, 42712, 51448, 5756,  26892,
                48988, 18616, 34556, 7176,  27824, 16748, 212,   19724, 45104,
                62400, 5704,  23968, 54668, 8652,  46632, 24808, 29712, 63888,
                33884, 47208, 56296, 30360, 62372, 9896,  27940, 47124, 57488,
                29916, 49860, 44728, 31804, 10148, 34904, 24528, 1508,  57248,
                16032, 52844, 24856, 48116, 3880,  22908, 52524, 37776, 27348,
                61392, 54160, 14496, 62384, 42384, 31736, 38756, 62992, 9788,
                50536, 25352, 18488, 46596, 7856,  58720, 16864, 33564, 56088,
                49116, 28560, 22396, 55128, 40292, 26796, 53852, 15344, 24740,
                41152, 59528, 4068,  14012, 50848, 40864, 60860, 28596, 49384,
                21892, 27032, 61912, 54208, 7688,  22800, 28704, 11644, 32260,
                24540, 49088,
            },
            {
                9492,  26876, 38232, 2576,  23948, 36240, 61092, 16116, 49992,
                58312, 37364, 55296, 20124, 15668, 60060, 10124, 47676, 33016,
                18484, 6708,  35020, 46444, 62216, 8420,  33040, 2760,  42412,
                10256, 29792, 45284, 22672, 14344, 58944, 17600, 39296, 13276,
                57144, 30920, 62640, 46756, 38464, 65420, 33280, 50856, 13268,
                58404, 30676, 39648, 64528, 696,   32648, 59452, 4312,  49564,
                37728, 27276, 512,   39448, 43224, 21516, 51736, 64252, 948,
                22384, 12076, 25932, 56876, 4428,  18184, 44328, 11820, 32444,
                28148, 39868, 61528, 14204, 35668, 9772,  17320, 2528,  31652,
                10748, 44028, 36292, 3828,  52276, 11396, 58972, 47036, 20116,
                5496,  55168, 65204, 13060, 35600, 51316, 43120, 9984,  24420,
                38488, 13564, 3524,  47076, 30464, 5872,  44972, 33376, 51808,
                48148, 18256, 38752, 33628, 8728,  20800, 54384, 11156, 33036,
                4644,  56952, 15816, 31764, 37164, 63028, 48752, 56920, 6916,
                61164, 45788,
            },
            {
                63156, 21660, 59868, 44012, 18908, 54284, 27148, 10196, 42152,
                23048, 12696, 2824,  42616, 31244, 53688, 39020, 4780,  64492,
                25540, 52656, 60188, 20596, 828,   28900, 55084, 15056, 26084,
                35680, 18944, 40532, 4812,  37136, 8756,  33076, 64748, 24128,
                10360, 44528, 21888, 3584,  24844, 10648, 57548, 22676, 37836,
                2712,  18020, 48016, 20436, 28440, 13440, 42964, 19448, 9368,
                22620, 58016, 6444,  50428, 15520, 4112,  36500, 18080, 32500,
                5556,  61312, 16168, 48120, 38780, 54080, 60624, 41316, 22036,
                46352, 2792,  20500, 57880, 28828, 63960, 41164, 55052, 59176,
                21368, 6632,  25992, 48764, 29448, 16288, 1360,  27372, 34204,
                43392, 30576, 37672, 22680, 28988, 3096,  20544, 60724, 4892,
                41904, 62472, 51572, 34608, 59504, 12760, 64696, 21264, 4872,
                30684, 11928, 63732, 23844, 57732, 47304, 35908, 14880, 64264,
                50980, 40508, 44120, 25236, 12968, 4028,  40836, 35048, 19264,
                39120, 14292,
            },
            {
                41716, 5328,  50628, 30548, 13332, 46628, 540,   31944, 52420,
                28740, 62292, 34796, 26416, 50156, 1516,  23784, 29008, 15440,
                37844, 9272,  42076, 13344, 51408, 36864, 44444, 64480, 21756,
                6936,  56552, 63336, 49220, 25648, 53736, 46580, 29944, 50680,
                928,   37016, 15312, 51832, 55752, 42660, 4612,  47320, 27340,
                54012, 44152, 9560,  34984, 57204, 52536, 39880, 61856, 54144,
                43976, 14148, 64624, 35136, 24748, 55468, 12720, 44388, 40208,
                54308, 35508, 42300, 2552,  21544, 30064, 14660, 5904,  65144,
                9388,  54944, 37560, 7588,  44124, 49168, 24468, 34268, 15428,
                46492, 39244, 64752, 18872, 60636, 40668, 56124, 22508, 13900,
                57928, 1912,  10624, 52856, 62436, 39164, 54996, 27360, 50040,
                31512, 8568,  19916, 24824, 32,    17960, 42404, 8716,  57344,
                36868, 1276,  52952, 44700, 6464,  26532, 544,   42904, 24096,
                8184,  18692, 1800,  60872, 47360, 55368, 16212, 26356, 664,
                52216, 29156,
            },
            {
                55200, 17432, 33976, 56656, 6760,  59160, 35172, 63912, 4580,
                18412, 46848, 6552,  58664, 13912, 46024, 61768, 43388, 54688,
                49456, 4136,  27568, 57788, 24052, 18088, 11376, 52724, 39572,
                47764, 13176, 31592, 16720, 61316, 2432,  20744, 6308,  41648,
                54696, 60580, 8216,  40132, 29400, 17668, 35576, 7840,  63928,
                32656, 14900, 62044, 50092, 6100,  23308, 16844, 1696,  26212,
                32828, 18368, 10528, 29780, 46388, 60400, 27052, 58280, 7380,
                24472, 51176, 63416, 28024, 8256,  58844, 35880, 51780, 25204,
                34108, 16872, 50464, 31544, 800,   19012, 12108, 4308,  50760,
                28916, 56536, 12512, 23552, 4624,  35344, 44940, 7704,  64064,
                48792, 17316, 41676, 47472, 6232,  15500, 45452, 11888, 36124,
                17300, 58208, 44168, 36980, 55588, 48676, 38688, 26564, 61424,
                46136, 27872, 20048, 40028, 15824, 30880, 62420, 55688, 37740,
                29048, 59456, 34428, 10344, 30012, 20836, 42872, 65024, 58200,
                11292, 36388,
            },
            {
                24384, 1940,  47812, 10492, 26060, 40780, 20108, 44928, 14356,
                56788, 40480, 10320, 22072, 36544, 18892, 8096,  12068, 34596,
                19492, 62888, 32116, 39788, 48416, 6388,  30832, 3432,  60316,
                28240, 1292,  52172, 9740,  34932, 44276, 12800, 57724, 16176,
                27700, 19900, 32456, 62204, 12428, 49568, 59684, 20816, 11968,
                41440, 24568, 3912,  27096, 38264, 11092, 47248, 35568, 7356,
                59252, 48160, 52460, 41188, 19832, 1924,  9416,  49040, 30568,
                14280, 19620, 10608, 33520, 50072, 45052, 18844, 424,   47456,
                57128, 42472, 13004, 26436, 62120, 56072, 38140, 63244, 42100,
                9076,  1128,  32612, 53516, 47548, 9988,  31320, 51684, 38500,
                4204,  27988, 60588, 19824, 25980, 34044, 656,   64632, 22356,
                53804, 2664,  14704, 63448, 10496, 28548, 53300, 15756, 22944,
                13224, 55312, 33096, 10252, 59316, 51896, 19096, 4388,  12488,
                48300, 53704, 22536, 38944, 52636, 5028,  8744,  31464, 22436,
                50088, 7564,
            },
            {
                58796, 43244, 64652, 38924, 16428, 53324, 23560, 8768,  37580,
                25048, 50656, 32176, 54420, 60396, 40724, 30804, 56572, 676,
                22736, 46364, 11052, 2612,  61660, 42864, 57008, 34152, 19340,
                45080, 37872, 23796, 42588, 27800, 59720, 38156, 23576, 49012,
                35412, 3836,  47040, 23012, 1684,  25812, 44400, 30416, 51752,
                1012,  59268, 17368, 43460, 60448, 31052, 65256, 51704, 28912,
                39276, 2496,  23748, 5392,  62340, 33832, 38584, 22016, 65348,
                46728, 1320,  56772, 39736, 13248, 25996, 62564, 38996, 11372,
                29052, 4516,  59568, 22452, 45708, 33004, 6164,  25532, 20672,
                59548, 17180, 43372, 37948, 58488, 15464, 26152, 19304, 54476,
                23868, 36560, 32780, 8608,  56140, 59276, 42236, 29772, 6900,
                46876, 25660, 40900, 30352, 20612, 6804,  34004, 2276,  50580,
                6124,  42860, 65116, 3064,  48732, 35240, 22828, 45728, 32148,
                16652, 6224,  44704, 14256, 61964, 49044, 37480, 46504, 14808,
                44164, 32812,
            },
            {
                28032, 12928, 21460, 29724, 61280, 2788,  49832, 60336, 29988,
                1336,  65212, 15760, 3040,  27984, 5128,  51180, 64092, 26672,
                42568, 58764, 16960, 35468, 21380, 14608, 50444, 24492, 7940,
                15804, 57792, 65384, 4224,  17716, 54524, 8148,  31340, 64140,
                11300, 59028, 43352, 53280, 36620, 63272, 5980,  16604, 56212,
                37456, 48128, 33664, 55360, 20164, 4572,  14692, 41804, 20908,
                13204, 63440, 36368, 54236, 15892, 43744, 51696, 17168, 5656,
                37144, 43356, 60772, 22552, 4356,  55880, 7244,  31912, 23060,
                61384, 19844, 39656, 52036, 9904,  15132, 53368, 47756, 30796,
                36412, 51860, 26604, 6984,  21192, 62968, 404,   42280, 11584,
                61868, 14740, 44216, 51120, 12764, 23344, 48860, 16232, 38340,
                60564, 33492, 51712, 4484,  57944, 45180, 62388, 40404, 58860,
                36056, 17876, 24628, 29240, 14124, 42184, 8940,  60480, 40128,
                64900, 27640, 57500, 896,   24412, 17984, 56428, 27064, 2372,
                60584, 19544,
            },
            {
                38300, 8480,  52136, 35864, 5748,  46920, 32968, 12088, 55544,
                43820, 20520, 39260, 48892, 44736, 23324, 16772, 47480, 13532,
                38404, 7624,  50096, 29504, 55208, 9700,  27636, 63432, 36476,
                54068, 11680, 48488, 33244, 22124, 50232, 384,   45376, 40236,
                5388,  26464, 17912, 8996,  14448, 55048, 33296, 40484, 10352,
                28160, 21728, 12876, 8556,  52924, 45920, 25000, 57804, 9596,
                55104, 45320, 27536, 8588,  31628, 12504, 59536, 27948, 55320,
                31964, 25224, 15896, 52304, 29940, 42440, 17256, 49608, 54540,
                44860, 8396,  33888, 3120,  27864, 42828, 18448, 61472, 13140,
                4032,  46608, 64280, 13684, 30112, 49944, 55696, 34704, 46944,
                6340,  58260, 1640,  29564, 40068, 3744,  53396, 10592, 20068,
                55924, 9240,  13424, 49124, 37644, 24120, 11808, 18852, 30472,
                9508,  49420, 60700, 39024, 5604,  57872, 27060, 1364,  50868,
                10744, 20304, 35812, 42464, 29000, 33652, 12160, 62848, 40360,
                6040,  53700,
            },
            {
                63428, 76,    44756, 56940, 15096, 27196, 41980, 17976, 5424,
                34820, 27368, 7748,  12604, 62548, 33396, 9616,  36164, 4292,
                31584, 53488, 24980, 64980, 38748, 72,    46884, 40412, 3908,
                21216, 30396, 6564,  41112, 14076, 62800, 29032, 19324, 14888,
                51792, 56476, 38896, 30076, 48540, 21972, 3348,  46648, 58392,
                6812,  61840, 42016, 29980, 37752, 616,   34780, 49372, 5676,
                32104, 17052, 21624, 47844, 56824, 24508, 488,   40300, 10304,
                49400, 3824,  34420, 8732,  46368, 64724, 35340, 13608, 1952,
                36596, 15812, 62712, 48924, 57192, 35744, 1776,  39372, 55076,
                10044, 22884, 34092, 2556,  45212, 39140, 9204,  27680, 17124,
                31556, 49284, 22088, 62616, 18152, 36376, 63780, 32068, 45204,
                1736,  28296, 22808, 65284, 16368, 32372, 48048, 53772, 25808,
                44800, 420,   52696, 21160, 46448, 53952, 18136, 33952, 24496,
                55056, 4236,  49748, 63220, 8436,  51160, 3700,  21040, 34924,
                49156, 16800,
            },
            {
                46600, 33592, 23992, 19044, 10292, 64000, 54252, 22404, 46196,
                61764, 57224, 52240, 18736, 38040, 57904, 52680, 20036, 56252,
                61204, 15628, 3132,  43672, 18760, 32712, 58876, 12580, 44156,
                61756, 51604, 25064, 45936, 36924, 10524, 57228, 35024, 61244,
                24196, 33120, 1240,  64456, 42144, 27612, 60968, 19152, 24744,
                35628, 50888, 2960,  63536, 16068, 59240, 28236, 18292, 38728,
                60292, 42408, 4076,  61916, 37840, 6592,  46460, 63176, 21220,
                13756, 61864, 57648, 38036, 20396, 3152,  27616, 58164, 41728,
                24876, 52772, 30120, 12080, 20708, 65496, 8152,  23828, 28332,
                59928, 41668, 56828, 18232, 60348, 24036, 4692,  62408, 20804,
                40984, 37192, 53972, 8120,  46420, 27496, 5396,  24548, 40432,
                61556, 36100, 42744, 54860, 836,   8304,  42284, 3852,  63332,
                13700, 34860, 16396, 31852, 11072, 37000, 62512, 14544, 43980,
                38712, 30436, 13244, 17028, 37300, 59712, 45676, 54700, 25580,
                9820,  30264,
            },
            {
                13744, 59368, 41528, 50204, 31160, 39528, 1876,  37120, 14408,
                10012, 23956, 41544, 31444, 2772,  6808,  25500, 45504, 28700,
                41336, 22032, 11816, 47704, 5540,  52652, 23104, 16536, 26840,
                34632, 1064,  18264, 59644, 52928, 3636,  22832, 41532, 7672,
                47316, 12360, 20704, 50644, 7048,  11588, 37884, 52444, 13664,
                44944, 17484, 26140, 48624, 22572, 10832, 43892, 64216, 24108,
                1880,  51380, 34356, 14284, 50148, 30116, 17636, 35052, 53144,
                44408, 28844, 18056, 47996, 12020, 53460, 22360, 9640,  63216,
                18816, 46248, 6112,  41076, 25736, 47132, 31780, 44132, 49752,
                15728, 31144, 7604,  52308, 35868, 11804, 43140, 57396, 51468,
                2856,  10812, 25544, 15196, 33904, 59732, 51332, 14020, 57232,
                17548, 4016,  46976, 19548, 27260, 61148, 56628, 29320, 21424,
                39500, 55372, 7736,  64308, 23616, 2564,  29084, 47940, 7100,
                58588, 22164, 47008, 56136, 23672, 6256,  30908, 15204, 42480,
                58000, 22600,
            },
            {
                5312,  27580, 3552,  61720, 7408,  25316, 59108, 51800, 29444,
                49288, 340,   15472, 59956, 28104, 43108, 64568, 11060, 984,
                49540, 33588, 59444, 36252, 28256, 62856, 7124,  56068, 49320,
                10160, 57608, 39076, 8428,  31616, 26948, 48636, 54608, 2352,
                29308, 43856, 59836, 34756, 17324, 57556, 30892, 116,   65164,
                32368, 5304,  57340, 39168, 33216, 55060, 7464,  52152, 12216,
                29404, 57236, 10104, 26368, 20020, 43236, 58296, 8124,  24244,
                2872,  41060, 6532,  25468, 60260, 40504, 45096, 29384, 6600,
                32568, 59860, 860,   55172, 16836, 59280, 4376,  52812, 19732,
                516,   38616, 21548, 48456, 26832, 16240, 32428, 47396, 14120,
                29292, 65500, 43368, 57604, 220,   41788, 21076, 8364,  47820,
                30924, 12104, 52004, 33576, 40064, 10580, 36820, 16924, 5832,
                59820, 47684, 27280, 42928, 56508, 40048, 51452, 9924,  19052,
                63988, 2812,  32976, 10516, 44348, 39972, 19032, 65180, 1700,
                36148, 51232,
            },
            {
                39988, 17928, 55716, 36704, 13448, 47656, 19964, 4956,  33304,
                64364, 44396, 35408, 54988, 20936, 50332, 14896, 39396, 18004,
                62300, 8280,  25728, 51136, 13836, 40636, 31312, 37824, 20320,
                42748, 29020, 14648, 63840, 20160, 44356, 16052, 11364, 63496,
                37812, 15336, 53824, 3464,  25428, 44160, 48204, 9552,  40924,
                21092, 53728, 8916,  13292, 46472, 2392,  40868, 20472, 36536,
                15792, 46164, 40284, 64580, 2280,  52220, 12344, 38984, 32732,
                64940, 50220, 55784, 31324, 120,   33980, 16276, 56264, 51172,
                39564, 14352, 22720, 37780, 33792, 9572,  14052, 36272, 63832,
                11340, 61068, 43552, 5044,  54104, 64424, 976,   37816, 23052,
                7576,  55096, 19316, 30708, 48616, 11792, 37848, 54368, 26760,
                64332, 22148, 6816,  59508, 14464, 49828, 23568, 52264, 43640,
                31044, 1732,  19212, 12988, 33248, 4652,  60268, 26144, 42580,
                35748, 51980, 28044, 61636, 388,   26392, 53212, 11996, 47900,
                7924,  62160,
            },
            {
                53896, 11044, 44852, 22056, 33876, 42956, 55008, 9224,  40628,
                17712, 26032, 4524,  47096, 8584,  33764, 23872, 53568, 30980,
                37376, 45928, 3556,  57532, 19800, 9860,  45724, 2544,  64864,
                5056,  54892, 47004, 35176, 6216,  60548, 39712, 32944, 18856,
                58340, 26732, 8968,  40072, 62448, 22564, 55992, 15880, 26656,
                59412, 43448, 29652, 61620, 19452, 27216, 60204, 31516, 48308,
                62440, 6792,  22420, 32536, 55700, 5280,  28644, 59960, 16064,
                20132, 10664, 36876, 14876, 63820, 49020, 4576,  37104, 11676,
                47728, 27088, 64376, 44472, 50452, 28640, 56272, 41856, 26396,
                32600, 54956, 14232, 29212, 9576,  40396, 19664, 60544, 52352,
                34976, 46120, 39404, 6364,  23812, 63508, 17064, 32664, 5208,
                43712, 35728, 56064, 25448, 45200, 2104,  32180, 64808, 12064,
                24988, 37452, 50616, 62008, 45628, 16848, 21340, 31108, 56808,
                15416, 5508,  40964, 17720, 49248, 59068, 33408, 38592, 28600,
                20636, 32308,
            },
            {
                24648, 48884, 29892, 2752,  64952, 16648, 28228, 60720, 22948,
                56412, 11352, 62676, 38352, 13308, 61564, 2188,  58572, 5772,
                20768, 54924, 15876, 42024, 24140, 61224, 53760, 15676, 25172,
                32768, 11608, 22904, 50240, 2720,  56084, 25016, 52428, 5008,
                49452, 21372, 31572, 46488, 13000, 36784, 4024,  34052, 50432,
                37676, 1624,  24248, 35768, 51096, 15100, 5088,  56672, 24892,
                248,   53844, 17280, 36972, 45540, 24984, 41984, 48388, 1032,
                53964, 27324, 43964, 23028, 8976,  19284, 58748, 25088, 20892,
                2320,  53604, 7880,  19240, 3184,  62092, 21948, 6824,  47660,
                3944,  23460, 35392, 45932, 56900, 25008, 31088, 5856,  11456,
                26800, 16896, 3284,  60068, 50544, 28680, 45036, 2504,  60820,
                39536, 10096, 16160, 29828, 62100, 6236,  20272, 46680, 7996,
                58992, 54204, 5356,  23136, 8668,  54676, 37980, 748,   49560,
                11320, 24772, 54360, 36552, 13632, 8268,  22236, 4348,  57452,
                44000, 14332,
            },
            {
                37772, 61192, 8604,  57496, 51064, 11684, 924,   45616, 36348,
                6524,  31084, 51616, 19436, 29172, 40956, 48696, 44268, 26928,
                12936, 65196, 29164, 33380, 912,   48464, 28676, 35736, 59940,
                51708, 39852, 16956, 27968, 43252, 13220, 30408, 8348,  36652,
                42764, 568,   64968, 52868, 6160,  28636, 19888, 60844, 7284,
                12380, 17948, 48372, 4260,  64496, 38452, 45056, 9476,  34972,
                41284, 28188, 11144, 60792, 7900,  14244, 62812, 22160, 36140,
                45748, 6944,  61340, 39916, 52096, 28684, 42012, 61848, 31784,
                57408, 35588, 42120, 30936, 12624, 46112, 17740, 39488, 58224,
                16508, 50688, 62308, 1996,  17644, 50084, 58892, 45168, 41084,
                63744, 56704, 33392, 14216, 36436, 7896,  57124, 22836, 51460,
                19476, 49176, 3536,  53672, 37360, 42420, 57424, 34280, 14984,
                40840, 17528, 35268, 30252, 41748, 47884, 13588, 65464, 33580,
                45292, 62480, 21136, 46868, 31080, 63264, 42700, 50620, 17128,
                63996, 2176,
            },
            {
                6152,  34384, 19604, 41180, 24956, 38568, 32168, 53412, 14156,
                49472, 42084, 58472, 24688, 3780,  54352, 17420, 10236, 35380,
                40176, 51864, 8872,  44580, 37708, 6436,  12776, 21472, 8472,
                45060, 1720,  62964, 58824, 37292, 21060, 64056, 47168, 23228,
                57164, 14476, 35132, 17864, 59052, 48968, 42520, 54088, 30800,
                63032, 44680, 55524, 10448, 30912, 22748, 53372, 18664, 13524,
                59036, 43852, 51144, 30628, 48936, 19100, 34100, 9732,  56880,
                30584, 13008, 57964, 32376, 1920,  46916, 13352, 5260,  44796,
                9804,  16356, 23752, 60148, 54732, 34508, 25248, 52488, 29552,
                9228,  42416, 20724, 39088, 12716, 34392, 3668,  15028, 21668,
                49032, 8932,  25160, 53196, 20620, 41420, 11160, 34948, 14436,
                26512, 59808, 41396, 23276, 18104, 11304, 25924, 53028, 28620,
                4056,  49016, 64152, 11188, 58476, 27904, 5844,  23356, 39928,
                8332,  28928, 1896,  58388, 5820,  25528, 55020, 34968, 10144,
                27440, 46692,
            },
            {
                58924, 52520, 15180, 28980, 5636,  48012, 17872, 62356, 26900,
                21064, 1716,  15656, 46072, 37012, 7504,  32232, 63368, 22372,
                47340, 4340,  59624, 18984, 56492, 64028, 50272, 42816, 58012,
                18712, 26112, 7024,  33388, 10004, 51140, 1528,  17192, 60936,
                10804, 29632, 45032, 25092, 9268,  38932, 1228,  14632, 24828,
                39588, 20764, 33416, 58188, 42452, 7592,  28564, 49780, 62840,
                23364, 3528,  20240, 38792, 1468,  58092, 40812, 4648,  51496,
                25180, 17932, 4300,  21244, 55204, 35076, 17404, 37820, 52504,
                27996, 62504, 48560, 332,   38208, 5708,  10920, 64800, 1208,
                33288, 59796, 27048, 6980,  63860, 43032, 53504, 28588, 37424,
                1224,  30932, 43924, 63088, 1868,  47500, 61360, 55152, 30508,
                904,   46716, 32452, 7792,  63712, 50968, 196,   38696, 62540,
                21984, 44076, 25652, 1212,  19764, 36532, 60940, 53096, 18832,
                59300, 15988, 43704, 12176, 39124, 19688, 14940, 1068,  39888,
                56080, 22776,
            },
            {
                42056, 11420, 45072, 63320, 55552, 21824, 59092, 4052,  8904,
                39308, 65280, 33464, 10472, 61044, 56304, 26076, 50480, 288,
                16244, 31440, 23524, 11336, 26460, 16544, 30520, 3616,  37380,
                31708, 54108, 48816, 41944, 15260, 55540, 45220, 34380, 27452,
                40808, 55132, 3012,  50816, 32024, 21844, 64208, 45712, 9880,
                52528, 2244,  26976, 13672, 17136, 60928, 1392,  39920, 32124,
                6296,  55844, 15292, 65228, 25460, 52968, 28156, 15584, 64300,
                42360, 49872, 38604, 63068, 26704, 10764, 50296, 64284, 20232,
                3640,  39984, 14000, 29332, 51692, 59064, 43644, 15332, 36736,
                46540, 11976, 54156, 30608, 47616, 24272, 19168, 8060,  62156,
                55476, 17684, 11636, 39872, 28292, 16344, 24896, 6624,  38304,
                64872, 12316, 36424, 58436, 14088, 31624, 48008, 19252, 9392,
                54964, 13516, 33220, 56568, 51520, 15128, 42780, 3020,  30632,
                50640, 36128, 55912, 32824, 52748, 48000, 29840, 61780, 49224,
                31492, 17956,
            },
            {
                3480,  25552, 32852, 1340,  36300, 10416, 42332, 34472, 44840,
                55472, 28832, 52556, 22588, 43000, 13976, 19540, 41344, 57864,
                62168, 38128, 53064, 46464, 34436, 40096, 55220, 24344, 14328,
                61276, 10908, 21596, 29424, 3664,  25220, 39228, 7448,  52140,
                4908,  20576, 37960, 61952, 12424, 57356, 27676, 17032, 35200,
                59788, 6056,  50684, 37868, 45808, 54780, 34764, 11800, 47080,
                26596, 36688, 45996, 33132, 7068,  11628, 46744, 37216, 20428,
                1304,  32956, 46308, 15000, 7616,  58984, 24172, 6360,  33308,
                55328, 22476, 45600, 7692,  18804, 26724, 21580, 49636, 56096,
                23944, 18132, 4656,  37488, 58100, 10480, 32864, 59780, 45604,
                23128, 34876, 51960, 58144, 4424,  32880, 50132, 44540, 17900,
                52972, 5160,  20976, 45088, 28084, 42988, 5448,  59288, 35828,
                30028, 3176,  39652, 45288, 7344,  31752, 25420, 10880, 46376,
                6776,  26328, 22752, 4172,  64688, 9036,  44576, 21616, 12660,
                7064,  36788,
            },
            {
                50284, 60288, 21028, 47296, 14272, 52744, 30052, 24084, 49100,
                12840, 18468, 5264,  47652, 2808,  29580, 35016, 6276,  12484,
                28656, 7844,  42628, 1648,  63000, 5212,  9656,  46908, 51436,
                60,    39608, 45636, 57332, 65004, 18836, 59676, 31832, 13996,
                63328, 47932, 16444, 7584,  43572, 36108, 6476,  55616, 47308,
                29388, 42000, 65452, 22212, 3816,  25752, 19908, 63892, 16204,
                58460, 10080, 53948, 42720, 22524, 60360, 31696, 55728, 8520,
                61500, 11776, 28892, 52588, 43840, 40248, 30700, 47876, 42844,
                12180, 58856, 36160, 63436, 33548, 41292, 4040,  31588, 7612,
                62520, 44288, 51388, 21528, 14720, 160,   50524, 39408, 5320,
                13612, 48524, 6744,  19660, 37540, 62332, 9344,  21872, 41784,
                28500, 57160, 50352, 24696, 2848,  55224, 16420, 23696, 46280,
                51288, 60680, 16892, 63260, 22792, 48948, 55120, 63808, 38884,
                14184, 61608, 42348, 18344, 27692, 37020, 2964,  59660, 26220,
                53328, 65448,
            },
            {
                29144, 8752,  56724, 40244, 27488, 7224,  64148, 16236, 1160,
                61252, 36180, 25516, 58864, 38428, 64032, 53184, 44952, 48596,
                24308, 55516, 20292, 14584, 49680, 22108, 59812, 18352, 27320,
                63316, 34496, 17084, 5360,  35520, 49468, 11200, 22648, 44052,
                26324, 58052, 33912, 24068, 52604, 19020, 40872, 2976,  23360,
                12868, 18888, 9152,  31580, 58648, 49444, 5644,  43336, 51872,
                21256, 29336, 4864,  14112, 49900, 18052, 2252,  43860, 23768,
                35776, 58320, 5764,  21956, 36220, 716,   18840, 60932, 15608,
                2124,  26000, 49276, 10240, 16192, 56768, 61412, 12904, 28164,
                40380, 2200,  34516, 65276, 41524, 54724, 25272, 16412, 30192,
                41868, 27512, 64188, 24040, 46768, 13236, 54680, 59168, 3112,
                14772, 34060, 8960,  18592, 38948, 60444, 32432, 41232, 12292,
                6312,  20996, 27804, 9932,  37784, 1676,  18096, 33320, 21612,
                53608, 260,   47608, 11616, 57752, 51672, 16364, 41116, 34640,
                43792, 15388,
            },
            {
                46028, 38284, 4408,  17472, 49772, 58132, 20276, 38772, 56424,
                31208, 41620, 8172,  50356, 16812, 10820, 21780, 1892,  31956,
                16972, 36692, 61100, 33008, 28412, 44144, 36812, 32192, 42244,
                7796,  23476, 13152, 52740, 27176, 41428, 776,   54472, 36916,
                2780,  10280, 29532, 46708, 1024,  60408, 30816, 51384, 62588,
                34248, 56380, 47948, 36396, 15012, 40908, 30060, 8856,  37516,
                624,   60756, 40688, 35448, 63388, 26612, 39452, 52368, 13840,
                48472, 26260, 54772, 16700, 64680, 56220, 9584,  27832, 51984,
                39280, 31748, 5276,  54092, 44432, 1956,  47460, 38004, 19748,
                54508, 16280, 25700, 9384,  28808, 46372, 35684, 63340, 56588,
                2020,  53796, 9888,  31304, 40684, 352,   29620, 35716, 25636,
                61820, 40212, 47272, 64012, 51884, 10932, 7276,  62996, 26456,
                56912, 42680, 34364, 47840, 59148, 13068, 56980, 44536, 8824,
                28336, 58528, 33940, 39684, 6148,  31728, 23892, 56796, 10604,
                500,   19912,
            },
            {
                24152, 12324, 31332, 61908, 43172, 2516,  33268, 46092, 5984,
                11076, 53768, 20728, 44520, 32932, 26964, 56192, 60028, 40356,
                51344, 3388,  10768, 57736, 6912,  54300, 12152, 2704,  58596,
                48604, 55556, 30972, 60692, 8704,  47332, 14988, 62668, 19616,
                50064, 41964, 65184, 13896, 54752, 26472, 10892, 15400, 44344,
                7188,  26700, 64,    62060, 11220, 53744, 23144, 56276, 32836,
                48504, 24996, 18756, 56512, 3360,  10976, 58656, 5336,  30748,
                18332, 42912, 3408,  33196, 49260, 13168, 44424, 34692, 21524,
                57760, 65456, 19224, 24664, 30068, 22048, 35060, 52204, 10024,
                59112, 32028, 49148, 60740, 6892,  19064, 3980,  11012, 21400,
                44484, 18128, 35272, 57176, 15684, 51524, 20092, 44180, 48728,
                7028,  30996, 12980, 1096,  29488, 35292, 21816, 37592, 48824,
                672,   14536, 53860, 5608,  30156, 40544, 26104, 4132,  36748,
                49500, 19700, 15172, 63196, 45144, 20168, 47768, 5156,  61040,
                32440, 54876,
            },
            {
                63540, 51432, 35148, 22316, 10268, 26316, 13652, 51336, 22708,
                27736, 64640, 14680, 640,   61388, 6972,  36992, 14500, 8820,
                64728, 26768, 46836, 38864, 17640, 25208, 65268, 15776, 20960,
                37464, 4328,  43768, 19488, 38176, 24300, 33148, 28580, 57168,
                7744,  17520, 36024, 5456,  21636, 39532, 49536, 37032, 20844,
                53560, 40160, 16432, 44004, 28368, 2928,  61064, 17372, 12496,
                64852, 44500, 8388,  31924, 47292, 28940, 45240, 21272, 64072,
                37440, 61004, 10620, 39776, 29856, 25280, 61732, 3104,  7884,
                45816, 11748, 42516, 38240, 59700, 14208, 6732,  64560, 24560,
                42108, 5040,  44532, 12752, 38960, 58344, 50188, 32324, 59988,
                38012, 49300, 4508,  61168, 43104, 7664,  65236, 11132, 17072,
                52180, 22592, 59460, 42532, 25300, 15840, 54568, 44368, 18384,
                30828, 65252, 36852, 18768, 23760, 62380, 52188, 16300, 64912,
                41712, 5324,  29932, 9316,  25684, 54196, 12804, 36740, 27816,
                49120, 39688,
            },
            {
                2312,  7076,  58568, 47716, 53840, 37324, 62684, 42040, 59588,
                35264, 47212, 39252, 30212, 51740, 42316, 47544, 25144, 19852,
                43984, 30844, 22616, 52548, 968,   41100, 46040, 29804, 52832,
                9968,  25896, 64260, 2520,  50828, 58696, 5852,  45368, 39696,
                23840, 31400, 56200, 44560, 61492, 33340, 8432,  64468, 3356,
                31896, 59912, 23708, 51928, 20224, 34628, 45692, 39044, 27400,
                2616,  15048, 51676, 22716, 38308, 16664, 54356, 34648, 9168,
                68,    51584, 23108, 46820, 7116,  19184, 50768, 37856, 54812,
                29196, 17204, 844,   51576, 8692,  49548, 27668, 45280, 528,
                14904, 36224, 21748, 55936, 30740, 23932, 42980, 14188, 27240,
                7772,  25108, 12220, 22064, 28120, 34324, 24888, 38804, 56076,
                2228,  36428, 19276, 46236, 57516, 5784,  50192, 3572,  58912,
                9816,  24464, 50548, 2972,  43680, 8156,  47260, 31620, 11652,
                23244, 51372, 55996, 37936, 60192, 1368,  41464, 64548, 18496,
                8500,  14852,
            },
            {
                26096, 30256, 19508, 15688, 3720,  29484, 8200,  18676, 1884,
                16348, 4792,  55676, 23764, 12272, 18048, 54824, 4476,  35324,
                58380, 5488,  13844, 62868, 34328, 50004, 8076,  57036, 33680,
                40384, 14552, 47060, 35004, 10936, 17100, 53232, 12740, 3288,
                61028, 52268, 11464, 28000, 18588, 4080,  46516, 25044, 56856,
                12724, 48920, 5576,  38280, 64268, 8220,  49348, 4596,  58952,
                41408, 35204, 55268, 59844, 5916,  62608, 13508, 25412, 40520,
                56764, 27880, 14268, 59592, 54172, 4368,  41680, 14948, 23520,
                48316, 36080, 55896, 32688, 63040, 41088, 17112, 33824, 55256,
                29420, 50600, 62724, 2508,  16908, 53248, 36664, 744,   47024,
                52328, 64428, 39288, 55332, 1592,  47424, 59856, 5192,  31836,
                63284, 27100, 53676, 9140,  38412, 64448, 13556, 27684, 34012,
                39924, 45500, 55900, 11828, 60808, 20840, 35252, 880,   27080,
                60632, 45904, 2700,  21860, 34124, 16136, 51916, 30856, 22780,
                55652, 42456,
            },
            {
                36260, 62572, 44716, 40440, 57280, 24292, 45584, 54952, 31816,
                49848, 26404, 9464,  62644, 34532, 2828,  28768, 63268, 50896,
                10520, 40644, 55240, 19372, 28804, 11592, 21912, 4608,  62116,
                18296, 51664, 29108, 21808, 41652, 30840, 63596, 20924, 35308,
                48168, 15708, 636,   40920, 50476, 58484, 16312, 42808, 28432,
                19260, 35672, 10512, 30636, 57040, 14016, 25732, 21692, 50996,
                30728, 19480, 11620, 27076, 1144,  42104, 50564, 6864,  47488,
                17680, 44232, 34048, 20948, 36352, 63184, 31488, 58364, 10428,
                62664, 4828,  25320, 12548, 20780, 3088,  58120, 22576, 61752,
                10684, 18920, 47236, 26824, 6284,  63644, 9548,  60980, 19644,
                34068, 15284, 30356, 44864, 17476, 52932, 13780, 21376, 41768,
                45140, 12328, 4284,  33404, 17536, 30688, 41800, 21484, 61584,
                7148,  16472, 28624, 32728, 38952, 14856, 53808, 58704, 39476,
                17508, 33024, 13096, 43628, 48840, 27140, 7416,  44956, 3808,
                58936, 47204,
            },
            {
                9160,  52384, 12712, 5640,  33788, 65216, 39116, 10928, 21740,
                36480, 57528, 40788, 20316, 46624, 59672, 39728, 21848, 32292,
                16852, 48332, 36568, 2948,  44632, 60256, 38116, 27184, 43332,
                532,   58256, 6680,  60996, 55660, 1408,  38484, 27256, 43324,
                9308,  26128, 64120, 37564, 23440, 30600, 9828,  52184, 2084,
                41144, 62452, 46244, 1112,  18280, 43256, 32520, 56108, 10228,
                61876, 6976,  43560, 48564, 33428, 20496, 29920, 60728, 11892,
                31676, 65136, 2300,  8688,  43260, 17020, 27104, 1200,  33948,
                20252, 43624, 60780, 46796, 30896, 37200, 47952, 5536,  39624,
                52852, 7816,  38664, 34336, 41776, 46068, 22692, 29188, 54920,
                5864,  41752, 3656,  58184, 10164, 37432, 29260, 49960, 8260,
                16380, 50824, 62240, 23444, 48592, 796,   55316, 11184, 46860,
                52472, 1816,  22184, 57628, 49596, 25388, 5876,  42692, 9876,
                54864, 7212,  64296, 19108, 58036, 10644, 62428, 38856, 12508,
                33660, 16916,
            },
            {
                1688,  23076, 27868, 49792, 20424, 784,   15160, 51504, 60828,
                43012, 13432, 5776,  30964, 53264, 8004,  14960, 44312, 448,
                25600, 61700, 8484,  23980, 56728, 16448, 53544, 13336, 47896,
                23368, 32792, 12600, 44108, 25452, 8848,  49624, 60024, 17764,
                55832, 32740, 46260, 7388,  54460, 13128, 35988, 60836, 32692,
                55028, 15068, 26432, 58668, 40524, 52796, 6012,  37156, 16416,
                46408, 24264, 37928, 63752, 17208, 52916, 36612, 23428, 55324,
                5700,  49764, 24832, 57680, 53088, 11992, 46356, 49948, 40352,
                52540, 7208,  28520, 16200, 54116, 9976,  26576, 13552, 43836,
                31984, 24488, 59196, 11944, 51532, 15044, 32844, 40320, 11720,
                59256, 23712, 50720, 20716, 33012, 63956, 2864,  56928, 24360,
                34828, 39220, 56356, 28448, 44084, 60668, 36668, 25948, 19396,
                35584, 64228, 42036, 9300,  3760,  63456, 29280, 19712, 47780,
                36892, 24924, 29592, 41016, 4192,  36136, 54664, 24784, 50328,
                20824, 60628,
            },
            {
                31416, 38364, 56736, 42584, 60064, 30704, 47056, 6712,  28008,
                3216,  64232, 45040, 16920, 37224, 26820, 50176, 11880, 57508,
                53480, 29408, 45864, 33384, 41300, 5224,  31612, 63844, 36116,
                9468,  54604, 39248, 18864, 47560, 15452, 33484, 5664,  51500,
                3772,  21896, 59164, 19080, 2944,  62780, 47520, 20916, 6016,
                24056, 8320,  49940, 21300, 28996, 11388, 47528, 64008, 3300,
                28880, 53592, 2368,  13972, 57024, 9680,  4168,  43684, 39056,
                19788, 35484, 13628, 39944, 29816, 22412, 6372,  64776, 24868,
                14260, 56964, 38700, 1460,  42892, 64356, 50208, 19680, 57336,
                3316,  16036, 64132, 1044,  20796, 58216, 4380,  49620, 17964,
                45656, 36488, 62464, 7320,  25952, 40460, 18848, 46428, 60896,
                30184, 1836,  19920, 6728,  14848, 9620,  51428, 3992,  58236,
                30440, 13036, 48104, 37472, 17784, 44792, 34304, 52356, 13956,
                62312, 2988,  49980, 53548, 23016, 32096, 18244, 48,    29116,
                41332, 54100,
            },
            {
                45916, 63396, 17660, 7228,  11436, 37044, 24668, 56560, 33908,
                18608, 23988, 49164, 59184, 2340,  23000, 65472, 33236, 37984,
                6772,  20444, 14652, 64628, 10576, 51588, 25828, 2236,  19904,
                50300, 27924, 62748, 2932,  36268, 64892, 22812, 12884, 39380,
                30228, 41812, 14320, 34964, 44384, 27408, 39412, 17144, 50892,
                43632, 37624, 65068, 34560, 4640,  61072, 24580, 20180, 33896,
                40340, 58044, 21232, 31280, 45896, 26364, 62752, 15512, 996,
                58684, 28028, 62528, 47436, 3324,  37800, 56000, 18212, 31220,
                9364,  35400, 19484, 59632, 23204, 5100,  34660, 60676, 28876,
                36636, 49208, 45132, 27508, 30952, 54096, 8412,  64592, 26176,
                1636,  28848, 14364, 48400, 54124, 11884, 43200, 5528,  14404,
                52736, 10772, 41608, 58840, 34416, 22432, 40008, 16952, 43616,
                6440,  54620, 27132, 23572, 59892, 12116, 56664, 1480,  22216,
                31716, 58828, 15972, 8888,  46152, 13580, 58592, 47860, 65092,
                6748,  10876,
            },
            {
                14688, 3532,  34600, 25784, 52788, 16316, 63256, 40852, 9528,
                54400, 35696, 28868, 10856, 54908, 42852, 4740,  47468, 17952,
                41820, 59468, 1872,  48908, 21724, 38836, 17376, 45772, 41632,
                59384, 14816, 5804,  30372, 51932, 55232, 27724, 45964, 57084,
                62012, 10136, 49296, 53084, 23976, 9296,  56452, 320,   30044,
                57580, 12148, 2348,  16940, 38900, 54480, 14972, 44416, 59664,
                12788, 8504,  50708, 35840, 6036,  40692, 48788, 29096, 53732,
                45464, 7232,  16756, 51308, 10108, 61304, 34568, 44476, 4444,
                60552, 47928, 51200, 12236, 32252, 39860, 17156, 9084,  53968,
                22220, 7088,  56028, 18012, 38200, 43312, 23232, 34592, 39108,
                56520, 9184,  43904, 35296, 604,   60364, 27912, 36324, 21508,
                64160, 48248, 26124, 45824, 53696, 65128, 29428, 49744, 61892,
                33560, 15384, 56,    52132, 32056, 7308,  40288, 27284, 46344,
                5560,  43424, 38372, 28096, 61944, 39496, 5308,  43396, 16020,
                36772, 26872,
            },
            {
                48980, 21584, 43948, 55720, 47736, 2424,  21108, 44640, 13112,
                50652, 52,    39876, 15380, 31916, 19860, 60372, 28076, 9716,
                51240, 31268, 36684, 26920, 58140, 6668,  61176, 53996, 8384,
                34720, 24356, 44980, 20984, 40792, 10680, 184,   34616, 19200,
                24728, 1900,  28904, 65520, 4728,  33272, 42724, 14520, 63292,
                26896, 48284, 32268, 44780, 51292, 8944,  31012, 1992,  49864,
                26480, 18032, 42636, 65524, 23688, 11756, 60056, 20972, 32776,
                11056, 23744, 42696, 32052, 20096, 25504, 13040, 53612, 41512,
                22228, 26692, 2672,  45116, 56188, 25240, 47048, 152,   42788,
                13108, 40416, 33992, 10064, 62692, 13712, 3036,  47180, 16096,
                52272, 59968, 18968, 31068, 23120, 16512, 50636, 57660, 32696,
                7852,  37444, 16196, 30940, 5552,  12492, 2480,  24908, 8928,
                21744, 40816, 63076, 44024, 20100, 48668, 64312, 16476, 35404,
                55124, 11216, 18928, 34212, 2240,  55892, 26424, 20228, 33504,
                52116, 57592,
            },
            {
                40308, 60476, 29880, 9336,  38824, 32720, 58420, 4796,  30836,
                61560, 21392, 46008, 63512, 8108,  50808, 35472, 13416, 24564,
                62508, 3868,  54640, 12532, 42944, 34180, 28404, 13172, 30820,
                3604,  55852, 49376, 60864, 7520,  17024, 59080, 42968, 6604,
                53936, 36172, 17176, 38840, 12408, 60428, 52068, 22272, 35760,
                9612,  19424, 60168, 23260, 56812, 27564, 42356, 35128, 55744,
                5060,  61364, 28540, 312,   53120, 17408, 34788, 2724,  50408,
                64600, 36960, 56416, 2192,  59116, 45972, 28648, 1000,  16728,
                38168, 63764, 30124, 15456, 6988,  62248, 52148, 30664, 65316,
                26272, 58608, 2316,  47832, 25100, 59380, 51256, 29888, 10960,
                22028, 4084,  41056, 65432, 55500, 38880, 9752,  3452,  44552,
                24584, 55876, 396,   61656, 19232, 38580, 45256, 55584, 36464,
                47784, 57172, 10980, 29612, 5108,  36996, 24336, 8776,  59544,
                49648, 25764, 64004, 52436, 22408, 49368, 11480, 60004, 8552,
                23388, 4992,
            },
            {
                11784, 18776, 600,   64896, 14068, 27888, 49612, 18016, 25856,
                37904, 57440, 5600,  26468, 41624, 57016, 1308,  39340, 56196,
                44788, 15716, 23340, 47192, 18952, 1028,  50120, 22880, 65300,
                40016, 17940, 11736, 32540, 25488, 37556, 47448, 30716, 63872,
                14724, 48408, 44608, 58292, 20148, 30916, 47108, 7040,  39828,
                54332, 4460,  41356, 13296, 6768,  18976, 64512, 11900, 22584,
                38812, 45444, 32940, 13852, 47516, 37952, 56680, 7764,  40428,
                13940, 27332, 5524,  52792, 40164, 7716,  64352, 50612, 57304,
                11624, 8276,  53188, 42428, 36060, 19768, 10556, 37988, 15788,
                50400, 20204, 31856, 53308, 5476,  35756, 19416, 41796, 62256,
                33544, 27520, 50152, 12708, 4944,  47368, 29992, 61760, 18552,
                13496, 50260, 41240, 22788, 52076, 33152, 59428, 14092, 18700,
                28492, 4196,  38660, 17212, 58644, 54132, 13648, 42020, 30296,
                20880, 832,   40804, 7620,  15124, 36648, 29656, 40652, 46288,
                63768, 31992,
            },
            {
                53000, 25288, 49916, 41912, 22760, 6052,  62176, 55088, 43100,
                9016,  14508, 52440, 33072, 23268, 17232, 46840, 30200, 20820,
                5696,  33696, 40264, 8016,  63376, 55576, 38044, 16272, 47288,
                58080, 36144, 1300,  43580, 62844, 53252, 21240, 12092, 26632,
                40196, 8524,  23004, 5688,  50168, 1768,  26148, 17908, 58712,
                45608, 25136, 30452, 49416, 61300, 580,   37176, 48648, 16480,
                9724,  54072, 19760, 58264, 8864,  29972, 44616, 25668, 19060,
                59372, 47156, 17752, 22580, 33844, 14928, 36708, 23588, 31128,
                47016, 34360, 21312, 59476, 27848, 3240,  57852, 22888, 45432,
                4296,  61516, 11780, 43608, 15692, 27892, 7636,  57312, 2056,
                54840, 44820, 7248,  34840, 25920, 20676, 42128, 53632, 26944,
                34244, 6424,  58532, 10992, 47228, 26492, 7780,  42388, 1780,
                60976, 50748, 25416, 46932, 34136, 1972,  50512, 61320, 6528,
                45296, 12944, 58136, 32400, 44604, 63036, 5780,  54836, 17692,
                1596,  37620,
            },
            {
                56972, 15832, 35036, 58760, 46096, 36956, 11316, 33712, 3596,
                48296, 19676, 36520, 59304, 12024, 3976,  61408, 9284,  50288,
                64460, 27796, 52580, 58548, 30420, 10956, 44468, 4360,  9372,
                27152, 20572, 52200, 28948, 14956, 2952,  57372, 5344,  51056,
                60496, 31672, 54636, 27932, 35260, 41128, 64304, 13340, 32332,
                2464,  62736, 15828, 38312, 33556, 45108, 25628, 57700, 30424,
                62988, 24708, 3516,  41000, 61088, 22288, 5168,  63588, 31952,
                42376, 9536,  30368, 61896, 49340, 10852, 19356, 43204, 6328,
                61012, 17308, 4788,  49640, 12832, 40764, 48328, 34168, 7384,
                29028, 36296, 55448, 22520, 39616, 64848, 46404, 37476, 13548,
                24428, 17460, 38400, 58288, 15412, 63464, 1272,  11468, 49000,
                39600, 64792, 29004, 35892, 2836,  15556, 64020, 31328, 52996,
                34936, 21012, 13012, 65436, 9032,  22096, 27584, 18432, 33328,
                37528, 53668, 28320, 47956, 18508, 2584,  24976, 50136, 13400,
                28172, 44252,
            },
            {
                9684,  30872, 4456,  8036,  20040, 52224, 16424, 40084, 23504,
                63700, 28364, 1496,  49180, 44544, 27344, 53228, 37320, 42640,
                18248, 11536, 36448, 3260,  21116, 25908, 32572, 61644, 53208,
                42552, 6856,  60240, 46604, 9720,  35388, 41496, 23772, 34040,
                908,   18764, 11568, 62108, 15884, 56976, 9248,  53152, 36872,
                21512, 51632, 10828, 55144, 20616, 8812,  52944, 3840,  43056,
                6664,  51424, 36356, 27180, 50144, 16080, 55212, 12500, 51396,
                3004,  54128, 38624, 176,   44228, 56792, 62740, 3364,  25988,
                39768, 56016, 44904, 24260, 63152, 31712, 55244, 14400, 59840,
                41416, 17572, 47112, 668,   31016, 50860, 10660, 21204, 32144,
                48180, 63100, 9688,  51552, 45580, 32364, 37048, 59104, 22292,
                2488,  16868, 44804, 20128, 56864, 40316, 48972, 23116, 10456,
                45152, 6704,  56172, 42596, 30048, 39284, 46160, 55740, 3520,
                64732, 16360, 23348, 9064,  60424, 42352, 31496, 58576, 35360,
                21776, 62492,
            },
            {
                47620, 40708, 61344, 54260, 26028, 29820, 988,   58148, 44452,
                31524, 54056, 9900,  39792, 65060, 19348, 33964, 14264, 25264,
                520,   59716, 45492, 15252, 42248, 51624, 17808, 35492, 23328,
                12896, 31412, 39200, 24500, 19140, 64960, 48528, 16404, 45172,
                55916, 37072, 42324, 46928, 4268,  23488, 44340, 28860, 48960,
                6272,  42732, 26864, 3092,  58800, 29616, 12964, 21828, 39360,
                33636, 14964, 46804, 11512, 1588,  34224, 39008, 47976, 23864,
                35700, 15796, 57980, 24860, 6852,  28152, 32752, 50276, 54040,
                13732, 29300, 524,   37240, 9328,  18376, 2060,  26616, 51760,
                10384, 24820, 63160, 8284,  56888, 25596, 3540,  59772, 52664,
                42392, 132,   30384, 23180, 5732,  56292, 18108, 8456,  45912,
                31864, 52236, 7608,  25068, 53508, 4404,  29264, 17676, 62032,
                39808, 27192, 16468, 4532,  52228, 62328, 14764, 10388, 24552,
                48812, 40088, 4696,  35808, 54516, 20964, 10652, 39312, 7328,
                51852, 3320,
            },
            {
                18884, 23556, 13856, 33552, 42320, 64516, 47404, 13192, 6956,
                17120, 60196, 22196, 15208, 30564, 7352,  57968, 4528,  47696,
                55260, 32392, 22332, 49428, 6188,  60460, 56692, 704,   48740,
                64212, 16028, 54404, 4288,  50416, 30220, 7264,  27056, 13072,
                63604, 21904, 7548,  52404, 33068, 38608, 19928, 660,   60484,
                17288, 34664, 65232, 46348, 40492, 16768, 62524, 47892, 59420,
                18828, 56008, 64928, 21112, 43660, 61988, 28568, 4060,  19932,
                62600, 28984, 11760, 47412, 20928, 40968, 16656, 37468, 9660,
                20012, 65084, 33172, 52616, 58600, 43676, 21952, 38632, 62360,
                3888,  49604, 33372, 14116, 18880, 44512, 34988, 16180, 5452,
                27220, 19652, 60088, 40436, 13924, 28408, 41704, 24800, 54736,
                60616, 14248, 37912, 62460, 32724, 13132, 58772, 37172, 684,
                54768, 32796, 58920, 36332, 19532, 1152,  35008, 41948, 58020,
                30516, 12144, 62016, 50904, 27024, 14732, 46736, 65380, 16556,
                42876, 27272,
            },
            {
                56504, 38724, 50268, 3748,  9996,  18444, 55692, 38204, 26916,
                50012, 41836, 35456, 5184,  55776, 43508, 23448, 62096, 38868,
                29676, 8900,  63748, 27000, 38504, 13728, 9024,  40740, 27768,
                6500,  57472, 33532, 43896, 11940, 37232, 59584, 51996, 39816,
                3708,  29364, 58416, 25608, 13992, 63384, 54492, 11924, 40004,
                56392, 22636, 14132, 7808,  36104, 50696, 32504, 26016, 10208,
                808,   28932, 7908,  32004, 52964, 17768, 10288, 59572, 45984,
                8160,  43404, 32512, 64324, 51908, 13232, 55836, 1552,  48220,
                43536, 22916, 6836,  15288, 27432, 47644, 6268,  54168, 31232,
                20088, 42784, 37716, 59000, 53132, 40496, 29132, 64048, 37924,
                57064, 11048, 35692, 54184, 2764,  48028, 61860, 12536, 35940,
                5092,  27660, 42396, 10204, 48368, 22112, 45672, 8748,  50836,
                14220, 23464, 47584, 10728, 49700, 60140, 26372, 6880,  52644,
                20248, 300,   44296, 33032, 6676,  57200, 1448,  24476, 33624,
                54024, 12172,
            },
            {
                112,   63992, 28712, 44724, 58888, 24504, 5140,  34536, 62904,
                11088, 3148,  46892, 25612, 51472, 36656, 12556, 50588, 20324,
                16120, 41196, 53244, 1860,  33472, 46772, 30000, 20296, 45016,
                36752, 18564, 2264,  62192, 22448, 56268, 1400,  19672, 32576,
                47864, 10408, 17444, 50044, 2800,  30532, 46036, 26384, 7260,
                31940, 48184, 29112, 54248, 24048, 2376,  5840,  43524, 37068,
                53536, 45260, 39848, 24352, 57772, 5960,  41600, 25588, 37112,
                54896, 1708,  18072, 37972, 4320,  59984, 23716, 30752, 60900,
                34772, 57252, 50704, 42064, 11176, 64088, 35552, 16104, 45496,
                12440, 28384, 2416,  21752, 6492,  11448, 48660, 22340, 8048,
                45924, 49988, 16532, 64916, 33568, 20388, 51224, 3512,  30792,
                21332, 50420, 58160, 18800, 6156,  41160, 64764, 26868, 19696,
                43876, 3736,  63972, 29236, 22376, 44524, 32244, 17656, 46940,
                36932, 59292, 22900, 17200, 41744, 37248, 29524, 49412, 4868,
                60776, 31116,
            },
            {
                35312, 7272,  21184, 15944, 37024, 51300, 30788, 21440, 53220,
                19132, 58452, 32080, 62432, 17904, 9448,  28220, 2012,  46064,
                6920,  57728, 12276, 18340, 61992, 24180, 55116, 63228, 11240,
                51716, 25244, 47724, 28460, 14860, 42612, 9832,  24768, 61020,
                54044, 43772, 35044, 40516, 59680, 9628,  36452, 16004, 62808,
                43432, 3456,  10584, 61920, 42140, 20052, 56700, 64192, 14664,
                22632, 60568, 3724,  12836, 36084, 48744, 31180, 52196, 14812,
                22532, 58896, 50360, 25944, 9784,  35212, 45364, 7568,  26932,
                12392, 2916,  18624, 38496, 30392, 1216,  24364, 59340, 8528,
                55904, 51208, 63828, 46116, 31660, 61604, 1204,  54268, 14484,
                32564, 24016, 43208, 7092,  26052, 10100, 37408, 44256, 64396,
                15952, 46480, 264,   34528, 55964, 30376, 3128,  35820, 57832,
                31536, 40748, 7976,  38220, 13200, 5072,  54528, 64176, 9292,
                28924, 13656, 48304, 63684, 10272, 53304, 61504, 18536, 39720,
                22388, 45704,
            },
            {
                10696, 53004, 47924, 62700, 2784,  41520, 14480, 45948, 8056,
                38796, 23808, 13504, 428,   40216, 59976, 54316, 32640, 64532,
                35420, 25880, 44212, 37116, 50308, 3732,  15420, 34684, 4592,
                59248, 41272, 8832,  38196, 53484, 31392, 47032, 35852, 5228,
                14924, 22324, 65088, 6280,  19296, 48432, 22764, 57456, 51900,
                20340, 37396, 58880, 17468, 31352, 52380, 12296, 28348, 49184,
                17896, 33228, 50884, 63280, 16252, 20604, 440,   64716, 11432,
                34140, 6964,  40456, 30404, 53904, 19728, 41660, 63372, 17576,
                40112, 46844, 25348, 61704, 53156, 20492, 49256, 40568, 33692,
                5104,  25972, 14868, 36204, 24444, 17220, 43484, 26804, 39596,
                62948, 2292,  29716, 52692, 46748, 58624, 17456, 55436, 7460,
                40376, 25632, 62912, 13360, 23304, 51540, 16332, 11112, 53424,
                24736, 61076, 18260, 56300, 51744, 41276, 15732, 23680, 39300,
                3488,  55356, 26100, 31576, 2456,  21536, 12856, 43528, 8244,
                15120, 59360,
            },
            {
                38056, 18000, 25084, 32960, 11864, 54652, 60708, 1136,  28468,
                43492, 56256, 48800, 29160, 45468, 21716, 15272, 42148, 23836,
                48708, 4776,  29288, 20956, 58808, 8144,  39548, 53496, 23088,
                30884, 13444, 63932, 20656, 6004,  61616, 17580, 57388, 39236,
                28356, 284,   31560, 56248, 26988, 42252, 1664,  33356, 4920,
                13560, 49936, 26248, 356,   38784, 46924, 8272,  35276, 40932,
                6484,  27036, 9420,  42672, 29456, 55072, 45192, 39388, 27884,
                48420, 61576, 15888, 46576, 2164,  57624, 13964, 50124, 4732,
                54212, 59120, 8488,  32548, 4632,  56576, 13100, 29104, 61356,
                18216, 39264, 57916, 9532,  54932, 49692, 33772, 59044, 19836,
                5868,  55760, 12792, 21712, 39444, 816,   29024, 22972, 33204,
                52904, 10628, 37700, 28840, 44428, 60748, 39080, 48060, 4888,
                13624, 45764, 1268,  33432, 26736, 59804, 1856,  35208, 61628,
                43428, 51280, 7396,  40476, 58584, 47420, 27920, 34800, 55724,
                50520, 26536,
            },
            {
                3472,  57108, 43620, 29548, 8516,  22712, 35280, 49080, 65428,
                16484, 5788,  33860, 63696, 7712,  37664, 3652,  58272, 9068,
                17524, 52516, 60956, 10868, 31656, 46232, 27224, 43360, 17888,
                49332, 492,   55560, 33456, 50952, 2844,  26592, 12008, 62660,
                45184, 51104, 11124, 37796, 14300, 53044, 63880, 28536, 40720,
                23960, 55528, 34752, 44740, 14416, 63412, 24852, 57808, 1356,
                59888, 46016, 55840, 37852, 3156,  24960, 8948,  58176, 5236,
                19880, 43328, 24368, 12212, 65292, 28100, 22768, 32996, 37184,
                29344, 21296, 14576, 36384, 45876, 17252, 43160, 2868,  22504,
                44764, 48352, 1712,  20556, 41876, 3896,  12972, 8800,  51440,
                35912, 41468, 60524, 32156, 15200, 63404, 43068, 12136, 59748,
                2388,  20004, 49028, 55192, 8392,  1804,  20328, 32668, 63624,
                28788, 50104, 37124, 21004, 10436, 46184, 29868, 49504, 11700,
                21032, 33476, 14836, 19056, 35980, 52340, 5728,  62836, 1296,
                30444, 41684,
            },
            {
                13696, 6140,  61544, 50724, 39132, 58424, 17920, 25740, 10340,
                36732, 54028, 20172, 11664, 52448, 26500, 49808, 29952, 12872,
                55600, 38692, 136,   41780, 16524, 56460, 2444,  10152, 60516,
                36328, 45396, 27608, 16740, 40240, 44088, 22500, 49544, 6876,
                19972, 55080, 24592, 60464, 46304, 7876,  18112, 11656, 45484,
                61548, 9256,  6136,  60384, 29836, 4688,  20356, 51124, 30336,
                11732, 18996, 22428, 14004, 62152, 34952, 51048, 17196, 32388,
                53340, 3192,  56464, 34604, 38436, 5752,  47452, 10400, 61852,
                1168,  42608, 52260, 63476, 27712, 10112, 37496, 64380, 53976,
                11272, 34792, 30260, 65076, 27384, 38468, 61324, 30512, 47388,
                25536, 18060, 9588,  48216, 54064, 5064,  34864, 50028, 26856,
                45652, 61512, 5112,  17544, 35964, 26508, 56316, 41588, 22968,
                8700,  16540, 59204, 42952, 64812, 7172,  19364, 57688, 25744,
                5548,  56488, 65064, 44624, 9304,  24608, 16376, 46048, 23124,
                19120, 65000,
            },
            {
                46536, 35092, 20664, 1520,  14828, 42848, 5068,  31980, 57664,
                41412, 2820,  25344, 42804, 59352, 18480, 35104, 61476, 44592,
                33512, 22668, 47572, 65160, 25152, 49668, 33200, 63612, 21416,
                5648,  57272, 24184, 11520, 7640,  36200, 55972, 30544, 34644,
                42380, 16164, 32920, 3248,  21336, 35432, 50160, 57896, 38112,
                31376, 19328, 48784, 22176, 41816, 53984, 16400, 37732, 43212,
                65156, 34120, 49024, 5928,  59040, 43808, 11872, 23492, 63640,
                37504, 29276, 9484,  50672, 18516, 43124, 53160, 16608, 56620,
                25148, 11840, 48664, 5908,  23732, 57704, 49632, 31532, 26280,
                5532,  59016, 13788, 50388, 7428,  56988, 16640, 22060, 364,
                44380, 57132, 3276,  27808, 37304, 19472, 8628,  57384, 15624,
                38544, 29996, 32876, 43240, 65372, 14236, 47140, 5944,  58124,
                35100, 54444, 4384,  24904, 14296, 32644, 53888, 40736, 16328,
                47628, 38388, 28284, 3900,  60600, 30760, 39160, 58348, 11108,
                33092, 53764,
            },
            {
                28136, 24092, 55308, 47692, 27044, 62244, 52920, 46224, 13324,
                22012, 61160, 47776, 31452, 14556, 1612,  40844, 7152,  20200,
                3996,  28132, 14468, 35144, 6088,  19412, 12704, 38564, 29876,
                15220, 41832, 48428, 65528, 52956, 60324, 14196, 1244,  63804,
                9312,  48448, 5924,  40232, 62196, 30144, 25032, 5628,  1104,
                15564, 52648, 28064, 11904, 56780, 33880, 9600,  47536, 23240,
                4164,  54324, 40152, 26088, 31132, 19244, 47880, 1252,  41392,
                13848, 46200, 21236, 62220, 308,   31008, 59884, 6608,  39556,
                45348, 31420, 18364, 34576, 41156, 148,   20736, 8136,  16352,
                40880, 52060, 18576, 43636, 24136, 46452, 35604, 52280, 62476,
                14708, 33028, 65264, 22656, 43832, 61184, 24940, 40876, 3624,
                21980, 12460, 52720, 23924, 10016, 51000, 29604, 18196, 44720,
                11552, 39740, 30736, 52128, 48112, 37568, 3028,  9800,  61056,
                31124, 468,   22868, 49952, 54336, 12676, 42644, 4144,  49584,
                37628, 7196,
            },
            {
                60816, 9920,  16660, 37152, 7332,  33872, 19516, 2232,  29072,
                51088, 8708,  38344, 4660,  56860, 46000, 64780, 24796, 51164,
                57104, 62556, 8068,  53092, 40612, 59152, 54808, 44128, 52040,
                8712,  34288, 1948,  31472, 20940, 28400, 46588, 17860, 25836,
                38432, 59776, 27828, 55636, 44488, 13772, 53916, 41956, 65532,
                46960, 35948, 62580, 3796,  38460, 25688, 2212,  62284, 14144,
                28212, 8580,  16976, 52432, 2472,  38476, 27656, 54616, 60520,
                6448,  25928, 57428, 40424, 14412, 26772, 23300, 34424, 20084,
                64368, 4316,  55684, 61284, 14916, 52912, 60108, 33928, 47328,
                63044, 22188, 37736, 1084,  33060, 11836, 4492,  28428, 40560,
                10600, 37760, 49728, 6020,  12412, 51940, 31600, 47508, 54628,
                63112, 7488,  58356, 40144, 3804,  60120, 38148, 1456,  63016,
                26296, 19740, 61536, 688,   22472, 58212, 27576, 44744, 35336,
                52020, 13412, 41480, 17568, 33996, 20848, 64036, 26056, 56676,
                15280, 43380,
            },
            {
                51656, 32076, 41264, 64596, 12612, 56148, 24228, 39856, 64108,
                34256, 16572, 27556, 53268, 23112, 10488, 32480, 15780, 39316,
                11812, 36672, 45596, 30724, 22144, 1232,  28272, 5016,  23420,
                58244, 62224, 18796, 39304, 4600,  10216, 40760, 52112, 57276,
                20756, 12576, 51484, 17292, 2152,  9456,  20008, 33864, 26848,
                22728, 8620,  42960, 18316, 46148, 59284, 51652, 31424, 56048,
                36392, 59596, 46780, 11248, 62604, 57020, 8440,  34036, 16612,
                50108, 36192, 4448,  32484, 54900, 44684, 51476, 8972,  13716,
                49484, 27136, 38048, 10508, 29460, 44900, 39016, 12400, 55188,
                4552,  29496, 9744,  60284, 53112, 63408, 19596, 56144, 7252,
                24636, 53656, 18924, 30016, 58728, 16520, 1352,  10776, 34456,
                17772, 49592, 45124, 27844, 16248, 34252, 22368, 53600, 32272,
                49512, 7036,  45720, 12824, 41328, 16996, 63924, 21308, 6656,
                24804, 63096, 57948, 7364,  45360, 1600,  36400, 8636,  29912,
                22132, 392,
            },
            {
                19188, 57612, 4584,  50344, 28664, 44316, 10068, 49312, 6076,
                57140, 44064, 62980, 19536, 35920, 49552, 28380, 54712, 964,
                48396, 26192, 16824, 60100, 11280, 48492, 36968, 17512, 46556,
                26940, 12284, 50488, 44732, 25228, 59180, 35164, 5680,  43920,
                2696,  32152, 35872, 23860, 63300, 37976, 48964, 59332, 12032,
                54872, 2500,  57632, 32620, 13476, 6320,  21252, 40776, 18392,
                100,   43652, 32172, 23952, 41928, 14604, 22000, 45372, 30456,
                10824, 64500, 17868, 48940, 11524, 2744,  62688, 36608, 42112,
                53820, 1488,  22276, 47744, 7056,  18904, 24556, 2408,  27120,
                36344, 45696, 56696, 25524, 15860, 38816, 43276, 48440, 32036,
                60388, 42260, 1848,  46144, 35896, 40092, 63756, 28748, 43176,
                24620, 36228, 564,   20392, 55492, 48096, 12552, 8296,  41048,
                15320, 36156, 55864, 28692, 34728, 5436,  49236, 11532, 55464,
                18808, 46652, 36856, 11116, 27480, 59948, 48476, 18028, 53288,
                62916, 38800,
            },
            {
                13420, 25768, 45784, 21316, 1324,  17224, 59008, 36600, 14316,
                21180, 232,   11528, 41244, 5376,  59608, 8012,  18896, 43320,
                61200, 20992, 3308,  42344, 33848, 64248, 14360, 60924, 40944,
                2984,  55216, 36844, 15340, 54052, 62952, 13544, 23156, 29568,
                64828, 47088, 7296,  42760, 57936, 31308, 45380, 6548,  17800,
                39176, 29904, 50172, 23920, 64472, 29044, 44104, 10452, 50068,
                26668, 63568, 20032, 4432,  35108, 49244, 3484,  59744, 52632,
                20568, 43596, 24756, 59020, 38324, 21568, 29036, 5188,  60592,
                30784, 16408, 59172, 41604, 65488, 32748, 50392, 62188, 42716,
                19772, 14396, 49216, 3792,  31252, 8116,  26484, 13616, 3600,
                21592, 15920, 56760, 26784, 6996,  23272, 55812, 20688, 4124,
                60512, 13812, 31840, 63884, 5400,  42776, 61596, 25400, 57520,
                2920,  64932, 24268, 9176,  60784, 52884, 31892, 43096, 39048,
                28776, 4452,  32208, 50468, 23336, 55036, 13960, 43916, 33260,
                6032,  47276,
            },
            {
                35848, 9044,  31188, 39484, 60504, 52032, 30656, 42572, 26256,
                32348, 47144, 55252, 30080, 15276, 44976, 38292, 63800, 34080,
                30132, 6560,  52296, 56160, 8540,  23880, 51592, 30412, 9812,
                21128, 32468, 7348,  29200, 272,   48056, 32788, 8496,  50440,
                15872, 54856, 11256, 26712, 21292, 14884, 192,   28436, 51544,
                60660, 15016, 40904, 9664,  53628, 35636, 16000, 61352, 38392,
                7556,  12984, 51032, 55488, 28472, 64844, 37604, 26216, 7012,
                40848, 952,   29432, 7812,  15820, 46240, 55612, 18124, 24516,
                45556, 11504, 35644, 25936, 3304,  56636, 9780,  16692, 58064,
                6344,  33704, 64868, 40036, 21228, 50812, 58680, 36504, 64556,
                51392, 34748, 11404, 62632, 49324, 13376, 51272, 8564,  46320,
                57196, 39784, 53312, 10224, 37168, 29120, 17852, 46124, 30556,
                21000, 51324, 42708, 18288, 47064, 2568,  25904, 15164, 59464,
                1100,  53624, 16092, 61624, 40732, 30868, 2660,  39400, 11192,
                23152, 59764,
            },
            {
                2000,  63296, 54868, 14952, 35096, 23208, 3500,  8468,  62136,
                50880, 37416, 23508, 60888, 51644, 25956, 3856,  13704, 23392,
                10548, 58628, 39584, 27424, 18740, 45000, 2468,  56832, 35540,
                49196, 64044, 58520, 42528, 22104, 18424, 57056, 41968, 36724,
                19828, 61216, 39504, 3884,  49924, 54296, 36516, 64272, 21792,
                34284, 5164,  48088, 19088, 892,   46888, 4616,  54936, 24440,
                33812, 57868, 39432, 16132, 9644,  44304, 18660, 12912, 32652,
                55444, 62468, 47672, 52976, 33608, 64080, 40552, 9768,  50776,
                33744, 6796,  55160, 49212, 13368, 21768, 37612, 46552, 30756,
                52620, 24104, 10476, 44336, 54784, 17916, 596,   46396, 29136,
                44196, 8680,  39292, 31528, 18596, 44008, 32756, 38188, 16900,
                29884, 6084,  25672, 50312, 22660, 58780, 1172,  52332, 11152,
                38636, 5048,  32448, 13488, 37356, 20348, 56388, 8192,  34900,
                65444, 43724, 21732, 9196,  5136,  19528, 64340, 26628, 57076,
                50552, 28792,
            },
            {
                43472, 17996, 48076, 6612,  11400, 64388, 46292, 19256, 56480,
                4744,  17516, 9128,  2332,  34680, 21096, 57092, 53024, 46172,
                50140, 35480, 14712, 47848, 32436, 38740, 7132,  25824, 43028,
                13208, 17096, 24616, 47012, 10940, 38236, 5288,  53116, 24680,
                1072,  28856, 44572, 33732, 62112, 18720, 8052,  42304, 10672,
                44844, 25052, 63108, 37160, 26776, 60304, 21760, 30812, 44964,
                3160,  47484, 22548, 6132,  31644, 52144, 1660,  57368, 38980,
                15140, 9940,  35652, 22544, 3672,  13084, 26292, 536,   57636,
                21288, 63728, 39668, 18240, 43540, 53320, 27644, 556,   40976,
                12916, 59216, 2344,  28036, 34696, 61932, 11728, 23452, 5860,
                19144, 54800, 24572, 59516, 6420,  16,    64684, 24148, 52452,
                12028, 62104, 19040, 45224, 14840, 40576, 7528,  35460, 62764,
                16088, 48292, 60104, 54592, 29736, 63480, 40080, 23172, 50936,
                11912, 26080, 37720, 45836, 58736, 51840, 35332, 45232, 3812,
                15664, 40348,
            },
            {
                52612, 24532, 58204, 28140, 41568, 53912, 25616, 38976, 12188,
                28976, 43740, 65272, 39420, 48520, 10332, 32032, 40744, 1760,
                17848, 25112, 65124, 376,   12120, 58464, 63072, 19820, 53524,
                5824,  39512, 3776,  52416, 32040, 64452, 27592, 58804, 10440,
                49264, 14572, 56384, 6488,  12352, 30552, 47752, 26200, 55412,
                2732,  58156, 12768, 31212, 56436, 41320, 8532,  13904, 65288,
                17776, 27772, 62656, 41200, 59484, 24524, 47152, 61904, 21868,
                50496, 26996, 18188, 43132, 60736, 31364, 48608, 36280, 42812,
                14388, 2652,  28092, 31872, 4184,  62984, 34652, 8720,  61440,
                20468, 37004, 47148, 15004, 6992,  41952, 31568, 57260, 40260,
                61024, 2040,  14460, 47664, 41636, 27156, 58104, 35804, 2912,
                48468, 42400, 34500, 2204,  65096, 32132, 26172, 55592, 44080,
                23252, 27484, 8356,  12,    44888, 11380, 5152,  47568, 31196,
                18520, 6512,  49724, 33344, 12844, 24760, 17152, 9504,  61980,
                21048, 32704,
            },
            {
                12192, 4324,  37724, 20480, 592,   31540, 15800, 49292, 59972,
                33480, 53164, 14072, 27012, 58524, 16016, 63004, 27856, 8676,
                59636, 42196, 5968,  55184, 22804, 49308, 15432, 29756, 46644,
                61580, 28176, 35000, 60144, 14428, 1980,  45352, 16932, 31260,
                60368, 37768, 21576, 25364, 51948, 40136, 59124, 14512, 38192,
                32708, 20588, 52300, 6400,  16744, 49464, 34400, 51764, 37660,
                10796, 53752, 1124,  36044, 20120, 11496, 34804, 29272, 5460,
                45400, 2296,  57824, 38084, 7340,  54632, 19272, 60016, 29644,
                52676, 47220, 60788, 10884, 23236, 58300, 15468, 48848, 25444,
                50972, 30180, 55348, 63552, 22020, 52212, 48072, 16048, 26936,
                34016, 49856, 29776, 36912, 53256, 20192, 15256, 10084, 55396,
                21324, 26924, 58508, 9072,  54440, 48568, 13240, 18668, 3972,
                33180, 56684, 41184, 21936, 52008, 26528, 61080, 16224, 57712,
                42156, 54344, 62496, 28888, 1432,  55820, 38440, 30500, 47504,
                54428, 7200,
            },
            {
                65308, 45472, 33644, 50876, 61740, 44376, 7084,  35900, 2892,
                20440, 24372, 47268, 6896,  43020, 19876, 5252,  37788, 53988,
                21448, 31224, 45700, 28264, 36796, 41848, 9916,  34032, 1280,
                21884, 56244, 9144,  19148, 49696, 40312, 22876, 35788, 7684,
                42884, 3332,  47136, 65172, 35256, 1580,  22684, 4848,  61828,
                17424, 46760, 39552, 43612, 11224, 23756, 1936,  57376, 20408,
                42336, 30188, 49928, 14452, 43532, 54940, 7920,  16636, 41920,
                53960, 31072, 65332, 14060, 24944, 45076, 10984, 5268,  23700,
                17184, 7836,  36716, 50692, 44436, 7000,  38628, 18780, 42820,
                3244,  6092,  17504, 39428, 25816, 1672,  37280, 7868,  53364,
                11172, 64348, 22092, 9192,  4048,  62068, 29244, 45860, 40448,
                31796, 5176,  17104, 38492, 29328, 21556, 42124, 61008, 50256,
                10276, 64612, 14036, 36420, 17972, 32972, 43280, 1472,  27852,
                35960, 3576,  14804, 20584, 48172, 42992, 5956,  59576, 13692,
                36052, 27780,
            },
            {
                22936, 16260, 59448, 8784,  13520, 56168, 22860, 62728, 10372,
                40916, 55548, 1220,  36496, 30720, 50572, 56092, 45004, 34296,
                11036, 61296, 16060, 52312, 19048, 3168,  60236, 51812, 38420,
                12592, 45516, 42080, 30428, 25624, 55884, 11908, 48260, 62936,
                52888, 18472, 29864, 9428,  16024, 45344, 54416, 29224, 49524,
                9356,  26956, 1396,  59564, 29508, 63600, 45588, 26652, 5116,
                60640, 7656,  25036, 61524, 4244,  48868, 26340, 63628, 36900,
                12168, 20308, 49108, 3948,  29220, 51260, 39092, 64384, 34072,
                41220, 58024, 25464, 19736, 33332, 55764, 29076, 11716, 63864,
                57088, 35284, 45024, 9400,  60556, 13144, 58268, 20456, 45188,
                3080,  41260, 16856, 58976, 43452, 34232, 50668, 6928,  57540,
                14024, 63608, 51972, 46496, 11724, 6184,  36216, 2428,  28724,
                38972, 46392, 5408,  49892, 59800, 55364, 8672,  38576, 64168,
                10532, 23804, 57096, 40660, 8804,  64660, 22008, 25800, 44,
                41444, 56984,
            },
            {
                48644, 2620,  42488, 29376, 19124, 39840, 27416, 51376, 45404,
                29640, 59432, 18436, 63516, 11768, 23080, 3232,  25440, 13800,
                49868, 4156,  39844, 7952,  63564, 32112, 24100, 56892, 26808,
                17844, 65356, 2816,  53980, 7180,  61716, 4276,  20280, 26648,
                33688, 13136, 57324, 41268, 59996, 19648, 7644,  42492, 36212,
                53400, 64976, 21932, 34724, 53856, 18556, 38904, 32128, 15368,
                47240, 34588, 56056, 17400, 32596, 39328, 58996, 1832,  22872,
                56956, 8640,  40828, 34960, 59392, 22204, 16184, 2940,  54112,
                12280, 45976, 756,   14104, 65028, 4940,  47604, 52780, 32312,
                23316, 13936, 28124, 48956, 32840, 43164, 29936, 63240, 35632,
                25404, 56344, 32224, 47184, 24380, 12196, 17804, 37520, 22640,
                2088,  43016, 35112, 24160, 61540, 56932, 51564, 24864, 58512,
                20608, 16152, 31020, 24692, 3376,  12524, 22924, 49072, 19440,
                46664, 31504, 51680, 36940, 17824, 32936, 53440, 44896, 51020,
                18532, 9884,
            },
            {
                31632, 55140, 25332, 36808, 48240, 1944,  33044, 17688, 5032,
                13880, 38068, 8920,  52324, 33612, 60500, 41256, 18144, 64616,
                29140, 36112, 56208, 26132, 13104, 47020, 5272,  43144, 7596,
                48876, 35436, 23660, 15784, 37832, 32300, 42564, 57004, 444,
                39540, 23720, 6108,  49036, 27412, 32552, 63276, 24436, 12888,
                31336, 5444,  15184, 48452, 8452,  3872,  12688, 58448, 53008,
                22460, 40204, 11824, 45888, 21600, 9436,  30468, 44872, 51756,
                33540, 25892, 46440, 17816, 55408, 9592,  43732, 48392, 26884,
                31372, 61680, 38376, 53032, 41756, 26640, 21552, 1964,  40796,
                7956,  59792, 53928, 19632, 5428,  51196, 24060, 4568,  18164,
                49060, 13312, 6724,  38672, 1560,  54256, 65328, 26428, 48856,
                60072, 28372, 7864,  19684, 992,   32236, 17260, 43284, 7268,
                34872, 53816, 61924, 39836, 44660, 35168, 29572, 52640, 14224,
                60160, 4544,  26288, 61568, 2024,  29480, 10668, 15092, 33900,
                63048, 38760,
            },
            {
                20952, 61632, 11348, 5372,  63716, 57800, 12468, 54452, 64744,
                48880, 21800, 44020, 27528, 15308, 47916, 8212,  58324, 46300,
                2068,  20380, 43784, 59228, 34896, 50872, 20460, 15176, 62076,
                31204, 11000, 58352, 51168, 46752, 12908, 28168, 49932, 16140,
                44984, 63904, 53716, 2588,  38000, 11232, 55988, 732,   17160,
                44588, 57996, 38020, 41724, 26236, 62404, 43288, 49356, 9352,
                2832,  62040, 28340, 644,   65012, 52936, 13768, 18460, 5896,
                61804, 15072, 236,   63304, 6332,  32280, 37028, 62576, 19004,
                8376,  22052, 4216,  29872, 16684, 9728,  37060, 62392, 17992,
                46864, 37572, 620,   65396, 40192, 10848, 15424, 54660, 39368,
                61332, 28696, 52156, 20532, 60608, 31460, 4928,  44440, 9432,
                33336, 15868, 56284, 44884, 49424, 39800, 14164, 64144, 47372,
                12288, 1500,  27300, 8008,  19036, 63352, 58444, 2304,  40408,
                33840, 7964,  45428, 12952, 49280, 57588, 39676, 60356, 3440,
                28456, 6848,
            },
            {
                14304, 45568, 34300, 52376, 23416, 43992, 8536,  41696, 35072,
                24676, 31900, 3968,  56712, 39636, 208,   26568, 32496, 37944,
                52464, 24236, 10184, 17264, 1544,  40596, 28584, 36988, 53800,
                36,    39964, 27460, 5080,  21260, 64220, 8028,  36564, 60716,
                10808, 30588, 34956, 21788, 15032, 51320, 46824, 39704, 60224,
                50036, 23640, 10964, 55828, 30956, 20772, 35512, 24204, 29700,
                37460, 19320, 43084, 50412, 35640, 25164, 58072, 37808, 28800,
                42424, 50164, 38680, 27876, 52320, 24316, 13764, 1348,  57320,
                51152, 35080, 56312, 48280, 60848, 43988, 57180, 13080, 30652,
                51772, 26344, 16008, 22608, 35032, 57956, 46688, 31648, 8896,
                456,   44584, 58408, 10028, 27492, 41040, 13864, 51612, 20720,
                40124, 53108, 11936, 37036, 62784, 26572, 9276,  29800, 55380,
                23496, 41024, 50932, 56408, 47804, 15076, 6172,  26728, 44224,
                20832, 56236, 16672, 42176, 22772, 4948,  19336, 47796, 23780,
                43468, 53168,
            },
            {
                1016,  50532, 18356, 39272, 15296, 30860, 26880, 20016, 840,
                58688, 51788, 7536,  61532, 35156, 21152, 54816, 10780, 15648,
                6644,  63060, 48716, 30984, 54240, 64796, 8988,  58940, 23032,
                45148, 19552, 61108, 41908, 34200, 17216, 54388, 24588, 4916,
                19588, 48020, 8476,  42448, 58872, 26012, 4208,  20528, 33452,
                28504, 3280,  19000, 61228, 408,   51952, 6212,  16840, 64524,
                54552, 5616,  59236, 15504, 7472,  41552, 3924,  47988, 10568,
                21420, 58616, 11988, 19332, 45236, 60312, 40276, 29368, 42220,
                15008, 45156, 10836, 23516, 6640,  33936, 49576, 24424, 3768,
                42736, 11324, 56320, 44656, 28852, 2560,  62004, 26588, 42900,
                21856, 33984, 15328, 36792, 47712, 18672, 56396, 35660, 324,
                63836, 24964, 3692,  30232, 5816,  21900, 52336, 2840,  37584,
                60156, 17632, 31824, 10724, 22416, 38260, 32556, 54844, 10996,
                64924, 36492, 30160, 53848, 63312, 34236, 27444, 55392, 8100,
                35916, 58916,
            },
            {
                25712, 29268, 8228,  60908, 2716,  53620, 59752, 47440, 39060,
                11016, 16568, 45900, 19068, 12228, 64236, 49720, 44280, 29788,
                55948, 41364, 33968, 5712,  21908, 45812, 12568, 4472,  33420,
                51608, 13784, 9496,  55624, 1788,  45388, 30036, 51596, 40704,
                59260, 27020, 55040, 17880, 62296, 30236, 36408, 10000, 64508,
                7452,  52880, 34468, 44040, 14136, 47092, 58732, 39592, 45176,
                13932, 33720, 23820, 46980, 31076, 20352, 54140, 63500, 33184,
                2748,  54948, 31100, 35720, 3400,  10088, 49408, 20748, 5952,
                26312, 64328, 18380, 39960, 28416, 980,   20144, 53436, 35904,
                61496, 6660,  32340, 49656, 8208,  20548, 52108, 37052, 12052,
                64824, 50364, 25124, 3620,  63356, 7700,  23684, 60984, 29356,
                17496, 47116, 59232, 41584, 18236, 60420, 45688, 33468, 20024,
                6396,  44176, 65324, 36016, 824,   42768, 51264, 17336, 24192,
                49008, 344,   25556, 6208,  10284, 37896, 45664, 13116, 65016,
                16784, 40756,
            },
            {
                32980, 63948, 42896, 22256, 46020, 33500, 6588,  13988, 62184,
                29556, 55708, 40980, 25356, 30876, 42188, 5808,  23512, 60880,
                19624, 13088, 27200, 59980, 15372, 38272, 26008, 48620, 17344,
                63308, 29252, 38316, 49104, 23832, 58220, 11752, 3544,  33440,
                13944, 37532, 936,   46180, 5332,  13320, 43844, 54148, 15940,
                40420, 47380, 25708, 37636, 8572,  28112, 32356, 11260, 1828,
                51348, 27740, 10220, 56004, 61372, 12048, 27296, 16584, 43996,
                24612, 7892,  47068, 65052, 23020, 58056, 33364, 54164, 60876,
                36660, 2336,  32164, 54904, 59328, 15620, 64700, 9100,  27964,
                57764, 18812, 38916, 62860, 13024, 41352, 16752, 4232,  56916,
                18580, 6416,  55932, 42240, 53056, 33192, 45944, 11476, 42924,
                7060,  32832, 13528, 50456, 35028, 55772, 11484, 42092, 27756,
                49588, 14432, 4096,  25564, 57364, 62276, 28872, 8612,  59668,
                39612, 14768, 60764, 51776, 40992, 17352, 3680,  31284, 49800,
                21832, 10900,
            },
            {
                57032, 4108,  36624, 12540, 56516, 18932, 37692, 25512, 50016,
                21452, 2284,  36768, 48760, 3676,  53512, 17304, 35860, 3060,
                39472, 47636, 584,   52104, 42504, 57924, 30288, 55968, 43496,
                2860,  25096, 6960,  32140, 14864, 37056, 62672, 21564, 44404,
                49644, 65296, 23612, 32604, 40012, 57048, 24836, 49172, 22232,
                58552, 12300, 62252, 17532, 54716, 63740, 19996, 25120, 56924,
                42180, 21020, 38676, 4504,  34512, 39864, 1444,  51404, 36364,
                60576, 14340, 40800, 17360, 6544,  38088, 12588, 16820, 46272,
                9440,  50576, 43156, 12372, 48916, 37228, 45684, 40824, 13688,
                47104, 23484, 1376,  54364, 25584, 48164, 60620, 32892, 24024,
                47492, 39832, 31064, 12680, 17068, 28300, 2676,  50212, 36508,
                57292, 54000, 22328, 8448,  25620, 700,   15652, 63460, 57816,
                9480,  54032, 30140, 45892, 20184, 12640, 5036,  46816, 34880,
                3832,  19816, 46080, 32464, 21688, 57360, 61432, 25308, 2004,
                54672, 47000,
            },
            {
                19944, 15736, 49144, 27776, 51352, 10120, 65248, 4764,  44112,
                34320, 9148,  63772, 14672, 59580, 10132, 28592, 57848, 50764,
                9396,  62320, 35248, 24656, 10460, 3560,  20056, 7708,  34740,
                40228, 58792, 52176, 64876, 19368, 42536, 27976, 8876,  56908,
                17364, 6576,  12040, 52360, 19780, 8740,  63520, 2144,  32412,
                5836,  29764, 1540,  23280, 41308, 4808,  48772, 36208, 61156,
                7932,  48424, 63400, 52560, 15844, 45436, 57136, 22528, 5740,
                49136, 28612, 52752, 25832, 55800, 43884, 27468, 2956,  30640,
                23928, 58096, 20944, 4248,  25276, 7204,  31756, 22192, 4372,
                34344, 50556, 15364, 30968, 36460, 2876,  29204, 43548, 53520,
                10368, 27068, 1048,  61968, 22244, 57976, 39320, 15168, 21056,
                26844, 1812,  44284, 65104, 37956, 47540, 31772, 4924,  24348,
                35500, 39188, 60000, 16220, 49888, 40272, 54380, 21772, 30540,
                57568, 42620, 27688, 11984, 48380, 7484,  52608, 43052, 34668,
                38516, 5580,
            },
            {
                44244, 31456, 59900, 224,   23856, 41860, 30460, 53316, 17968,
                57768, 27720, 52288, 22856, 32516, 38808, 46312, 25704, 15112,
                32288, 21208, 44696, 16832, 63420, 36824, 50324, 60804, 11572,
                15612, 21676, 45028, 11028, 5520,  48004, 1176,  52828, 31432,
                25432, 43216, 29472, 60972, 48156, 34916, 28324, 17328, 38080,
                42920, 51700, 57316, 45968, 33212, 15720, 53200, 3140,  14560,
                31172, 17700, 104,   29316, 25384, 8836,  64836, 30476, 18912,
                39096, 10468, 62088, 708,   31820, 48336, 63852, 51912, 41340,
                62324, 13804, 34992, 29396, 61420, 52756, 17348, 62796, 55044,
                59648, 43268, 6060,  64760, 21284, 9636,  58360, 14380, 6692,
                63572, 37448, 44812, 49096, 35316, 5172,  52080, 64408, 9632,
                59880, 40076, 16216, 28972, 53348, 20212, 40564, 51784, 17392,
                46700, 21840, 7500,  32872, 2136,  26968, 37064, 64288, 13808,
                52120, 8860,  63652, 1764,  36304, 29256, 14528, 18528, 9580,
                27376, 62368,
            },
            {
                24396, 53444, 40052, 7936,  34832, 57308, 15588, 2684,  40304,
                12480, 46812, 6520,  42496, 19556, 54792, 2128,  64908, 43092,
                4668,  55848, 29476, 6784,  54348, 31828, 23548, 46192, 28244,
                53784, 680,   30136, 35300, 56036, 61084, 39460, 15136, 60092,
                36588, 1976,  56308, 38396, 15396, 4100,  53500, 45768, 61684,
                13860, 20336, 35424, 7804,  11844, 60036, 29216, 39700, 46188,
                23396, 34812, 59408, 43692, 20176, 49880, 12880, 42208, 59072,
                2536,  34000, 43272, 19708, 13796, 8992,  21876, 5340,  18684,
                37804, 256,   54448, 46076, 39332, 10736, 42368, 1908,  29688,
                11744, 26664, 39632, 56748, 45304, 50984, 38456, 19692, 34856,
                22644, 15848, 59004, 19432, 8592,  43592, 24288, 30344, 47344,
                4588,  32108, 49392, 11020, 3228,  57420, 13076, 27124, 60704,
                1564,  12232, 41852, 55808, 61180, 9928,  17792, 43848, 6392,
                25676, 38268, 16492, 54312, 59652, 23896, 39628, 63876, 58040,
                50320, 13668,
            },
            {
                37244, 11180, 18400, 63472, 44480, 21044, 48780, 60320, 32360,
                24324, 62056, 572,   35384, 61132, 12940, 7572,  22336, 36540,
                51628, 12128, 59560, 48700, 40572, 13660, 2404,  18304, 37696,
                62416, 48808, 41824, 16780, 26752, 23540, 33672, 20368, 46872,
                10760, 50648, 21864, 7776,  26552, 41900, 11580, 22688, 55984,
                9244,  27428, 65240, 50252, 25872, 43700, 21588, 9824,  64712,
                54584, 11488, 38064, 5912,  55756, 32240, 36004, 6888,  54216,
                15528, 23600, 50000, 57876, 35924, 39948, 59356, 34220, 56556,
                26556, 49188, 16292, 6456,  19656, 58468, 24284, 35708, 45956,
                20400, 8336,  33300, 17832, 12448, 24884, 4664,  61612, 46132,
                51456, 29080, 3960,  53652, 32488, 61444, 12396, 18040, 37368,
                54828, 19704, 61368, 34688, 23632, 62424, 5884,  43440, 33868,
                49232, 64720, 28716, 23624, 45120, 51072, 34176, 2804,  56448,
                22704, 49688, 33784, 19576, 43504, 6080,  46228, 884,   22116,
                30388, 7052,
            },
            {
                60952, 2260,  47980, 29720, 12976, 5384,  27364, 9736,  37792,
                20216, 50224, 30032, 16776, 45196, 49436, 31448, 58708, 17980,
                27548, 38940, 1196,  19236, 26644, 44320, 65400, 55648, 4952,
                24932, 9216,  58228, 4344,  51492, 12784, 7660,  42136, 3384,
                64320, 17696, 32808, 45980, 63188, 58604, 31360, 35968, 240,
                48384, 40556, 18320, 2968,  37712, 56332, 17012, 49132, 6244,
                26228, 41664, 51012, 14920, 62180, 3048,  24080, 48224, 27308,
                45572, 63704, 29380, 4352,  53748, 25476, 15492, 45868, 11284,
                7848,  42512, 32760, 62940, 28016, 48652, 7820,  51220, 15956,
                53456, 62172, 48900, 180,   28688, 54280, 41380, 31228, 1652,
                57432, 9444,  41572, 14108, 26304, 40332, 56176, 1960,  44768,
                25480, 12900, 42256, 7432,  46212, 37204, 30092, 54884, 8984,
                19512, 52988, 36700, 5512,  13364, 19992, 62552, 47808, 31596,
                41028, 60308, 5084,  10552, 30648, 51248, 12628, 33424, 47788,
                41136, 56432,
            },
            {
                43544, 34084, 25868, 51820, 59128, 35996, 54984, 45988, 64084,
                6952,  41688, 56348, 10500, 27392, 24020, 40500, 53292, 9764,
                47720, 63660, 23196, 34836, 8104,  51836, 10676, 33780, 41224,
                14548, 32188, 20784, 37268, 44816, 62144, 54832, 30204, 57296,
                27588, 40200, 53176, 13032, 3052,  20156, 50200, 6684,  59848,
                24328, 32820, 54720, 6168,  30992, 62760, 764,   35536, 32060,
                58564, 2492,  21304, 28392, 46436, 18024, 40100, 61288, 9220,
                20364, 37388, 12244, 7872,  46740, 61508, 1784,  29012, 51112,
                64536, 23296, 55596, 2476,  37428, 13868, 32016, 65312, 38828,
                4604,  23596, 41992, 59876, 35304, 63824, 16488, 49684, 11952,
                24124, 33760, 65376, 47868, 21388, 6752,  49972, 34376, 63024,
                5668,  51636, 58428, 27268, 17756, 49996, 21492, 15188, 39884,
                26044, 3420,  16516, 58900, 30668, 39416, 26348, 8072,  15560,
                12060, 46128, 27840, 56024, 63056, 37576, 25916, 54152, 16980,
                4332,  20140,
            },
            {
                14676, 55404, 22200, 3844,  39568, 15108, 23740, 1164,  17380,
                53680, 13928, 36880, 59724, 5812,  63288, 1604,  15536, 33536,
                5200,  42556, 14668, 56720, 61336, 37992, 21464, 28944, 58768,
                46732, 52628, 64200, 28072, 2072,  18148, 35972, 48500, 14324,
                23248, 5724,  35488, 57644, 25176, 44032, 39516, 17448, 29664,
                44760, 15036, 61212, 46380, 13196, 23888, 40672, 53040, 19248,
                13748, 45412, 63944, 33312, 10176, 52716, 57288, 13888, 33748,
                3392,  56104, 41208, 18372, 23168, 33048, 43008, 20244, 39012,
                31048, 18180, 12320, 45008, 52544, 21988, 43476, 1080,  26352,
                56032, 30944, 14624, 46256, 5472,  21976, 7944,  27396, 44048,
                18988, 38336, 55604, 348,   29924, 59552, 10036, 27696, 15904,
                22300, 31024, 38572, 2100,  56120, 10396, 63788, 204,   59852,
                45852, 32184, 48192, 42172, 54804, 628,   51888, 58048, 21080,
                65040, 1132,  35856, 22488, 3864,  15696, 58960, 8340,  65364,
                36176, 28052,
            },
            {
                62596, 7648,  46168, 10560, 64888, 31760, 50112, 42728, 29296,
                34096, 25884, 2996,  51904, 19292, 37940, 43708, 56404, 21268,
                60032, 29732, 46512, 4188,  31200, 16880, 1724,  49676, 5800,
                18972, 12184, 7368,  39780, 49788, 10888, 25560, 4816,  43880,
                61816, 9912,  49204, 30528, 15748, 9012,  55016, 62532, 12312,
                53216, 8164,  38620, 21224, 51328, 9376,  44220, 60760, 28604,
                8684,  56524, 36780, 6740,  22964, 37864, 1140,  29784, 43036,
                51648, 60076, 31396, 50368, 65416, 13184, 52700, 5516,  57584,
                3072,  35928, 60180, 25940, 5240,  57536, 18636, 60572, 12164,
                34920, 8956,  19892, 51964, 11204, 40148, 58660, 36472, 54004,
                61136, 6116,  16592, 44908, 36012, 18504, 43064, 38108, 57216,
                45804, 9524,  64936, 14800, 41148, 31988, 44516, 35796, 24072,
                56216, 10752, 62900, 7600,  22996, 11696, 36248, 43456, 28476,
                38788, 53260, 17924, 44016, 48748, 40268, 20912, 29692, 44708,
                11508, 50716,
            },
            {
                38964, 31120, 17612, 42096, 56440, 19288, 7316,  60452, 11560,
                58028, 48724, 22560, 45840, 32784, 28628, 11972, 48612, 26040,
                37096, 8892,  50804, 24900, 41424, 54176, 45252, 60416, 24144,
                35220, 43900, 54624, 22464, 59604, 31060, 63868, 20868, 37340,
                54060, 18680, 41500, 868,   65120, 37632, 27092, 2640,  22364,
                41476, 4176,  27860, 34464, 58412, 26504, 15544, 4044,  38708,
                50800, 25072, 16372, 43788, 59932, 49652, 26244, 64436, 17148,
                22156, 6596,  25780, 460,   38224, 9520,  27072, 61032, 15152,
                46828, 8908,  49664, 41312, 33584, 10168, 29516, 40044, 50236,
                44936, 58308, 38132, 25508, 55352, 32632, 47680, 14372, 2644,
                31812, 50384, 11136, 24684, 52984, 63908, 13288, 51204, 1060,
                33224, 19388, 48244, 25012, 52892, 4304,  28060, 50788, 6248,
                14140, 38472, 27936, 18748, 60944, 46508, 17356, 3684,  32296,
                49508, 7144,  25772, 61652, 12264, 32204, 1808,  52484, 57652,
                23532, 484,
            },
            {
                59272, 26836, 52948, 34776, 1744,  28192, 38276, 45312, 20876,
                4748,  39820, 9056,  61600, 14644, 53012, 8024,  65192, 2688,
                18140, 54788, 13056, 20488, 64440, 11500, 27328, 14992, 39424,
                61900, 30172, 0,     15912, 34192, 46044, 8240,  51768, 2208,
                32636, 27972, 58980, 46088, 21328, 51748, 32460, 47044, 35784,
                49876, 64204, 55384, 18652, 1988,  48480, 64464, 33136, 20900,
                47280, 1428,  30328, 53872, 12756, 19792, 5500,  45744, 11068,
                35228, 47908, 14048, 44092, 55276, 34112, 48460, 22820, 41700,
                54456, 28496, 20412, 15740, 64112, 53864, 47284, 6324,  22864,
                16804, 4180,  63616, 1572,  29372, 17628, 4800,  64564, 26036,
                20600, 39892, 28744, 57708, 3784,  31948, 6412,  23964, 61948,
                54548, 4964,  36088, 59600, 20896, 12828, 61048, 18284, 41708,
                21276, 52456, 4160,  35592, 32680, 25396, 52808, 63064, 10052,
                58676, 14636, 34316, 55236, 8780,  59824, 42796, 6376,  17388,
                33708, 48276,
            },
            {
                20632, 4264,  12516, 47400, 22744, 61772, 14176, 53956, 35564,
                63688, 31096, 18276, 42008, 4,     58116, 22952, 40040, 34372,
                45068, 60868, 39180, 1008,  35832, 7532,  33368, 56944, 2900,
                9856,  48508, 26708, 56548, 42192, 13484, 57696, 39952, 16228,
                60644, 12368, 24304, 7516,  56368, 13468, 5544,  61260, 18948,
                30616, 15636, 9980,  42292, 37040, 29852, 6384,  53584, 11416,
                58836, 41628, 63324, 4720,  34868, 40856, 32160, 55816, 52492,
                38600, 62488, 29124, 16816, 59200, 6292,  18788, 37140, 10800,
                32408, 62816, 736,   38596, 23904, 3412,  14516, 36640, 62612,
                28252, 33232, 48572, 41576, 60652, 44808, 22984, 51212, 42200,
                8852,  46784, 62592, 15064, 41760, 48712, 20236, 40364, 26904,
                11452, 43348, 29016, 7928,  39620, 46056, 33360, 8952,  57988,
                30680, 65412, 44412, 49580, 56456, 13792, 5592,  40488, 20072,
                23668, 46656, 3188,  39072, 28224, 23320, 49756, 36944, 26420,
                41384, 9120,
            },
            {
                44188, 63532, 37108, 57972, 5760,  51284, 9836,  25328, 2080,
                15924, 50948, 55940, 26400, 36416, 46856, 30504, 16308, 51548,
                5364,  27672, 31688, 58076, 48228, 52712, 42052, 22304, 50772,
                18492, 36628, 64604, 6584,  19980, 3632,  27316, 23008, 48252,
                44504, 5352,  39192, 34624, 17760, 42908, 25692, 10912, 58544,
                772,   24880, 46660, 60376, 12664, 56868, 17188, 43040, 35876,
                23636, 17944, 9796,  27464, 48544, 62024, 15772, 8288,  24252,
                2044,  56852, 9116,  21352, 40960, 30480, 63124, 3628,  45128,
                25260, 6540,  56816, 51660, 30280, 43912, 59728, 19960, 55292,
                51856, 8380,  15248, 21248, 12112, 7720,  34408, 56340, 12816,
                60232, 33164, 1928,  22040, 35372, 9236,  60688, 46336, 36680,
                15144, 50500, 17592, 55656, 932,   63632, 25604, 54412, 47432,
                1312,  24816, 15976, 9692,  2224,  38124, 59376, 27356, 44464,
                54656, 30292, 63580, 19104, 47444, 15592, 64488, 12732, 55440,
                61128, 14592,
            },
            {
                32380, 54776, 24788, 15864, 30560, 43576, 33112, 48036, 59784,
                28080, 44712, 11296, 5612,  62880, 19504, 6844,  55340, 61928,
                10716, 22180, 43196, 15960, 25852, 19360, 4852,  63576, 31792,
                43480, 12956, 24400, 51340, 38368, 53528, 62568, 35396, 10048,
                30272, 54520, 50596, 62972, 28528, 48580, 36644, 54220, 44116,
                39824, 33700, 52396, 5464,  22556, 45524, 25548, 62268, 7420,
                31344, 51188, 39128, 54960, 20680, 552,   57828, 28524, 41888,
                19092, 32904, 44836, 53100, 2592,  49736, 13644, 53616, 58440,
                16828, 35580, 13016, 46340, 8292,  34656, 26620, 10860, 400,
                43076, 24912, 36908, 53052, 57596, 39336, 27040, 612,   37132,
                16904, 54396, 7108,  51052, 26720, 55888, 17132, 30244, 4416,
                57448, 64392, 32196, 23408, 49616, 35368, 16708, 5244,  36936,
                12564, 40120, 58556, 19840, 42832, 30164, 48512, 16172, 35152,
                1404,  12684, 42028, 6064,  52732, 34940, 2648,  31004, 21928,
                4820,  50960,
            },
            {
                18420, 2400,  42328, 8796,  39068, 64444, 17464, 21532, 41068,
                7628,  38328, 23536, 33684, 49896, 12584, 43816, 25216, 32888,
                14612, 50248, 3416,  63148, 9472,  46156, 13640, 28020, 54504,
                3704,  58756, 46432, 32856, 11368, 29608, 43556, 18388, 740,
                61728, 20464, 15212, 2148,  9648,  57744, 4464,  20932, 14420,
                7984,  62680, 28304, 19112, 38520, 32336, 96,    49064, 14756,
                60216, 2448,  45960, 14032, 25360, 37744, 45164, 11848, 50864,
                64692, 5232,  26444, 61792, 35088, 23264, 28012, 39032, 20828,
                50728, 42548, 61832, 22084, 17936, 56044, 49776, 40784, 32248,
                13120, 46936, 65152, 2412,  31104, 46216, 62960, 19720, 48672,
                29512, 24200, 39184, 45264, 65072, 12404, 1628,  53360, 22988,
                8980,  40632, 3272,  45112, 14056, 9936,  29768, 43824, 22456,
                62084, 32092, 50956, 26548, 61640, 22312, 6636,  64344, 9008,
                51080, 37436, 59628, 22696, 10616, 44308, 57412, 8264,  46052,
                39664, 28532,
            },
            {
                35932, 48864, 21280, 60340, 26696, 380,   55336, 4876,  57484,
                13328, 64180, 53648, 3220,  58700, 28004, 38072, 1256,  47920,
                41072, 59400, 37484, 53868, 29928, 57208, 39468, 35772, 7676,
                16628, 40328, 21656, 1464,  59952, 15080, 7304,  50348, 56580,
                26300, 37668, 42656, 32088, 22604, 40696, 64768, 30024, 50424,
                23592, 55680, 3448,  47904, 65516, 8880,  55280, 40444, 20152,
                27748, 56372, 33796, 65044, 6536,  30864, 60744, 22840, 35528,
                47384, 16304, 12820, 38408, 10192, 46968, 7300,  64996, 1504,
                30988, 9852,  27624, 39708, 1820,  64620, 5588,  23292, 61240,
                57300, 29040, 18560, 6028,  23908, 15936, 9712,  52596, 5484,
                42836, 58580, 10348, 18600, 31036, 42444, 37920, 49304, 60164,
                34440, 20404, 27916, 62236, 38144, 56740, 60248, 52392, 18704,
                48556, 3788,  7456,  34520, 53076, 11640, 55580, 32352, 19628,
                57444, 26408, 16884, 32860, 62364, 27380, 19716, 37296, 61844,
                53576, 11228,
            },
            {
                65032, 6840,  56228, 45020, 11788, 49364, 36440, 29540, 46416,
                34448, 19156, 30660, 15148, 40640, 21344, 56336, 64636, 8236,
                23372, 18452, 6356,  33856, 21348, 416,   60832, 23288, 45276,
                62208, 30308, 9104,  55536, 47732, 25312, 65512, 39492, 31668,
                13256, 4932,  58744, 46792, 52496, 16264, 34188, 1608,  45848,
                17436, 36512, 11080, 41936, 15724, 50988, 29092, 12304, 36720,
                43804, 4984,  22516, 11312, 42524, 18008, 53380, 2980,  7784,
                39912, 30400, 57908, 51816, 18396, 56468, 43316, 15640, 34676,
                47912, 5936,  57984, 53072, 31684, 15316, 47176, 37316, 17116,
                3932,  39716, 54048, 35544, 50372, 41452, 59032, 32912, 22152,
                14080, 63452, 34848, 4564,  57264, 21428, 6764,  26284, 43668,
                13592, 47632, 52864, 5972,  19180, 26412, 1680,  40548, 10948,
                27792, 56280, 38736, 14312, 45272, 328,   37652, 41388, 24032,
                44868, 5480,  48348, 54020, 496,   41168, 51556, 24448, 16668,
                1192,  25776,
            },
            {
                40996, 13604, 30216, 34292, 19196, 52980, 14824, 24188, 9972,
                51728, 1156,  45532, 60988, 48288, 9488,  17240, 30248, 35160,
                53388, 28204, 45828, 13160, 42168, 48820, 17452, 51688, 11648,
                25804, 49516, 36884, 17664, 41988, 33972, 20684, 3256,  44688,
                48656, 24044, 10864, 63344, 6128,  27004, 11932, 53464, 39028,
                61672, 31296, 57940, 26108, 34612, 21132, 59424, 3180,  63724,
                50100, 16796, 52508, 38716, 58088, 48444, 33444, 14760, 55400,
                21164, 60488, 4500,  24704, 88,    32276, 59760, 11236, 25756,
                61348, 23404, 19024, 36244, 11852, 43148, 28760, 8596,  50920,
                21696, 10316, 43764, 60244, 12620, 28308, 3116,  38164, 55704,
                45544, 2172,  27436, 53036, 47964, 14752, 33436, 62956, 18076,
                92,    57044, 11024, 31680, 43152, 51008, 7980,  33128, 63172,
                15672, 46012, 23648, 64740, 18640, 29228, 47856, 15088, 3424,
                60528, 29672, 9608,  38928, 14748, 30772, 4636,  12920, 48212,
                32072, 58328,
            },
            {
                17724, 52084, 23776, 2924,  63080, 6432,  41896, 58788, 62140,
                39356, 27160, 22732, 6496,  35888, 52784, 3752,  44136, 12248,
                60112, 2452,  56472, 25200, 65140, 10260, 31176, 4440,  34240,
                56804, 2548,  63664, 27504, 5848,  53720, 10244, 58456, 17044,
                35476, 54436, 29588, 18404, 36404, 45244, 60280, 24576, 9500,
                5132,  48992, 13980, 1180,  53104, 6288,  46060, 24460, 32508,
                9952,  26360, 61396, 29148, 3568,  9080,  25876, 63964, 43828,
                28108, 49360, 34580, 45732, 41456, 28992, 20540, 51128, 40252,
                55100, 44948, 3496,  50264, 62304, 24948, 59540, 54468, 34700,
                62928, 26332, 32000, 7476,  20484, 64292, 48100, 25644, 11492,
                30672, 50280, 16436, 37276, 24672, 59520, 10640, 51720, 29704,
                39148, 25040, 35452, 60696, 15460, 47324, 24164, 36236, 53284,
                2396,  31240, 9088,  41960, 4680,  59060, 26164, 62892, 50336,
                13260, 35792, 56052, 21652, 47052, 58640, 63488, 42544, 55032,
                9208,  44692,
            },
            {
                3716,  37524, 59088, 46564, 38860, 27760, 32416, 20624, 3956,
                16336, 55668, 43080, 11408, 32200, 24596, 63008, 49660, 20536,
                38416, 47624, 16296, 36308, 6556,  52800, 38480, 59964, 41548,
                14996, 20524, 43664, 13048, 60440, 38228, 23424, 28824, 51480,
                8400,  61116, 39932, 84,    50076, 20652, 41228, 57052, 28908,
                19176, 43304, 22652, 60852, 44204, 38256, 17796, 57244, 41288,
                6696,  47524, 35664, 14348, 45716, 19164, 51512, 37196, 1056,
                11164, 18728, 8168,  14656, 64372, 54304, 5368,  36948, 2364,
                13124, 16696, 33508, 41140, 7008,  20712, 3056,  13348, 45092,
                1236,  48944, 15476, 55148, 44960, 33776, 17704, 8084,  61872,
                19572, 41192, 60300, 9004,  1040,  40608, 44776, 4976,  55628,
                8328,  64136, 41432, 21672, 4724,  58232, 12848, 20760, 42932,
                57756, 19580, 60404, 51400, 35836, 53636, 21444, 10056, 33292,
                42624, 18632, 65468, 4008,  25496, 35116, 6968,  18992, 27172,
                34504, 21664,
            },
            {
                49836, 28420, 8576,  16460, 54464, 11244, 44240, 48576, 8508,
                36696, 49480, 64404, 18792, 58500, 38876, 14780, 26960, 4768,
                31920, 9424,  61972, 29180, 44548, 19016, 27212, 22264, 47164,
                7236,  54120, 32328, 51560, 848,   45736, 14700, 63900, 41640,
                2756,  21804, 13892, 32948, 55768, 7972,  14620, 2936,  35596,
                64576, 54188, 33496, 28116, 10212, 63136, 30224, 13388, 53356,
                19496, 59612, 504,   22044, 63208, 41436, 31744, 22960, 57552,
                40624, 61996, 52532, 37908, 23096, 9708,  17140, 47576, 63444,
                30732, 58376, 26448, 10312, 46284, 56708, 38888, 32524, 18628,
                41360, 57932, 23472, 4840,  39112, 164,   56948, 42664, 53408,
                5996,  33640, 22444, 46620, 63708, 28208, 20104, 36196, 22912,
                48828, 17056, 2608,  54264, 30268, 37492, 65112, 28344, 5348,
                48232, 38212, 25340, 12960, 32492, 16620, 7220,  39744, 55004,
                1792,  27544, 51724, 44248, 17332, 10952, 50508, 38360, 59768,
                5316,  63012,
            },
            {
                56652, 12812, 33448, 60604, 22024, 1004,  65340, 56528, 24624,
                29956, 13092, 3200,  28464, 46544, 728,   54484, 42508, 60948,
                51868, 41108, 21084, 1568,  58492, 13596, 55388, 2884,  64164,
                29440, 24220, 9908,  35976, 18916, 30712, 49440, 5144,  34340,
                26684, 47252, 57900, 42560, 23620, 63848, 30768, 51416, 47532,
                12208, 7580,  40108, 3764,  16044, 47960, 35348, 2036,  23036,
                37688, 43400, 30056, 52848, 7444,  55944, 12340, 5412,  16560,
                46880, 31216, 26636, 2108,  58112, 49804, 34824, 27288, 21432,
                8360,  38536, 52044, 65492, 29740, 15508, 53204, 26660, 64128,
                11196, 30108, 36048, 62200, 27992, 51076, 23972, 12864, 36920,
                27112, 3404,  57100, 12736, 31976, 54536, 15604, 58396, 31520,
                45520, 13272, 27600, 50056, 9556,  44992, 560,   55180, 17296,
                11332, 29352, 6144,  49160, 1120,  43908, 58248, 29536, 46964,
                23604, 61380, 8132,  31960, 40060, 56820, 29628, 23044, 46484,
                15356, 40584,
            },
            {
                25128, 2116,  42792, 48912, 26380, 35752, 14212, 18224, 40464,
                59944, 34208, 51488, 41732, 9260,  21940, 34404, 7988,  18572,
                13760, 24536, 53880, 34452, 49084, 42632, 8736,  35236, 16992,
                37892, 59192, 48152, 63236, 26204, 57620, 39796, 11712, 53992,
                19084, 62444, 9452,  28068, 4152,  44772, 17412, 38768, 21704,
                26160, 59224, 18220, 51988, 56444, 25656, 7404,  50560, 64956,
                56564, 15852, 10972, 33852, 38540, 26716, 48996, 60612, 35124,
                54016, 3740,  19984, 42132, 32396, 12644, 61232, 44644, 41592,
                54756, 4736,  18328, 22628, 304,   35620, 49140, 4560,  22756,
                7156,  51848, 13620, 46724, 9264,  16284, 60492, 31168, 47392,
                64920, 43520, 51184, 18360, 39052, 2904,  50472, 11544, 4072,
                60332, 42364, 34064, 62624, 18512, 40020, 25192, 51148, 34944,
                60920, 41092, 63592, 55572, 23148, 36964, 64244, 19384, 5248,
                12648, 37400, 15488, 48032, 3164,  63920, 13736, 812,   53540,
                9964,  31028,
            },
            {
                45796, 64076, 18308, 39348, 5300,  51924, 31388, 47232, 2360,
                53392, 5584,  20512, 16500, 56204, 61648, 47936, 29748, 57716,
                36984, 6252,  63940, 11596, 30888, 23656, 62148, 50624, 44988,
                12124, 6068,  40452, 3436,  16040, 8064,  22140, 60228, 44200,
                31664, 16072, 52100, 36828, 12572, 54572, 34272, 61468, 960,
                42688, 30284, 45992, 37128, 21364, 60152, 42936, 12140, 33052,
                25336, 3916,  49704, 61268, 17812, 1748,  44556, 29500, 9132,
                24812, 13480, 63896, 45336, 55608, 5940,  24720, 864,   59480,
                14912, 34104, 47852, 43296, 59012, 11704, 42048, 57468, 45476,
                60296, 39544, 19420, 54556, 22004, 34544, 40768, 2076,  20580,
                15376, 10232, 35740, 23800, 7412,  42888, 26524, 65508, 38332,
                19456, 53640, 7192,  23092, 12384, 59864, 32580, 14784, 44056,
                7788,  21360, 31932, 14504, 45660, 26852, 10708, 52168, 34156,
                57256, 41736, 59336, 22252, 54192, 26156, 43384, 33284, 62336,
                36904, 20284,
            },
            {
                55508, 7184,  29660, 53660, 10504, 61712, 43364, 8032,  26980,
                22568, 45048, 63380, 31800, 37260, 13252, 24028, 2624,  43888,
                50032, 26792, 46032, 4392,  39672, 15764, 576,   31972, 20196,
                56756, 52480, 22540, 33168, 46468, 55496, 36764, 2524,  24872,
                6940,  39064, 1668,  46104, 59992, 25060, 6832,  10588, 56840,
                15268, 4884,  63052, 11344, 1532,  28964, 39208, 18752, 46696,
                8696,  28360, 45160, 57480, 24224, 14096, 65388, 21036, 40184,
                58516, 37256, 48768, 8828,  16256, 29696, 39764, 19400, 10612,
                50504, 28312, 61824, 7372,  25520, 31604, 17272, 9136,  28184,
                33276, 1436,  43300, 4220,  64104, 49784, 6408,  58740, 52244,
                25720, 55956, 29600, 62832, 59616, 47992, 34476, 8924,  24176,
                29436, 916,   57600, 36528, 46896, 6452,  56324, 3660,  23792,
                47512, 53876, 2120,  18724, 58692, 3608,  40712, 15980, 48272,
                25292, 168,   29808, 10632, 35352, 19460, 8632,  49004, 17584,
                4228,  50940,
            },
        },
        {
            {
                18624, 4924,  39988, 43464, 59476, 33772, 38936, 54392, 29012,
                48196, 62920, 39460, 16068, 6756,  47112, 40276, 52884, 8812,
                22932, 15188, 28156, 56836, 44432, 10024, 40756, 7020,  35968,
                44888, 29076, 2236,  38880, 34000, 11836, 808,   55280, 15292,
                35684, 58472, 30008, 52380, 108,   55036, 63628, 21116, 58180,
                9292,  52328, 28204, 60864, 56632, 38624, 64852, 51328, 42864,
                61364, 37548, 31252, 63688, 5836,  21084, 54420, 228,   64560,
                23600, 7004,  51400, 25860, 38724, 56616, 43984, 63768, 51616,
                36176, 46412, 19152, 54788, 10312, 30660, 64600, 52776, 36832,
                60844, 54056, 7848,  3156,  23896, 42060, 588,   46272, 22468,
                33712, 40480, 47724, 15460, 5260,  62976, 53820, 28148, 10616,
                23948, 15880, 6732,  64428, 13556, 33784, 22316, 10796, 31556,
                54332, 26688, 59280, 51068, 36924, 56176, 60500, 4312,  23512,
                10752, 55740, 1292,  65488, 9540,  19112, 13172, 2256,  6448,
                55432, 37228,
            },
            {
                30240, 57208, 13696, 9436,  20304, 6620,  15396, 64212, 21928,
                37052, 3904,  44860, 26292, 54472, 22280, 29728, 36112, 60292,
                41616, 34756, 62804, 4572,  17624, 23192, 33444, 65400, 52672,
                57976, 23900, 50648, 64108, 58992, 46372, 40592, 32300, 23480,
                10432, 3136,  45060, 13924, 17900, 28572, 5004,  42968, 31264,
                1316,  46756, 23328, 3400,  48636, 11792, 33172, 6868,  25040,
                20360, 48788, 40760, 27736, 11280, 47044, 33600, 41656, 9820,
                57408, 20140, 36060, 1128,  29372, 48828, 7852,  20992, 15528,
                60972, 6292,  12956, 41468, 57764, 16260, 24996, 6392,  28992,
                12776, 31660, 16076, 44444, 56272, 30312, 62872, 27348, 12180,
                50156, 30476, 8496,  52580, 39120, 31828, 42040, 7780,  35732,
                58544, 30964, 50112, 27544, 43008, 60416, 55688, 40496, 48384,
                7524,  38452, 2068,  15400, 28060, 11648, 40672, 17544, 51208,
                58792, 39240, 16648, 25288, 52252, 37756, 49092, 26888, 40432,
                23432, 46808,
            },
            {
                1336,  50176, 36144, 28316, 55564, 47824, 25952, 51340, 8332,
                18568, 12404, 59824, 33696, 10236, 62316, 13764, 2308,  46220,
                19772, 11140, 51848, 47000, 30740, 55204, 668,   27356, 11588,
                18208, 32240, 7300,  15752, 22028, 4596,  18664, 61004, 50016,
                39304, 65272, 25760, 50572, 60868, 35156, 47920, 15764, 26452,
                38904, 64448, 35292, 16312, 30788, 44116, 21816, 58528, 12812,
                53524, 3696,  17328, 55640, 35900, 59256, 25028, 18104, 28096,
                45928, 13480, 43060, 63504, 10620, 59920, 34368, 26116, 4264,
                43120, 31368, 24008, 35416, 4700,  51008, 38508, 48524, 468,
                23020, 40044, 64024, 21080, 36516, 14136, 6032,  57824, 53764,
                4536,  18844, 61868, 27572, 23076, 58048, 16936, 21828, 51952,
                45220, 1184,  39012, 9784,  19332, 5736,  16072, 24556, 65380,
                18220, 46712, 30972, 63184, 5956,  48056, 25108, 31400, 44680,
                7076,  47332, 29296, 43284, 5172,  59052, 44712, 61184, 32604,
                11492, 62184,
            },
            {
                16612, 21776, 65244, 45620, 2228,  40552, 11460, 35012, 42544,
                57648, 28476, 52552, 1816,  19352, 43220, 50824, 27780, 6400,
                54980, 25552, 1636,  37744, 13220, 60384, 49540, 43808, 39048,
                3804,  48304, 41424, 36108, 56328, 30600, 52932, 8476,  27588,
                17176, 55648, 32860, 7756,  40460, 22884, 10068, 61824, 51248,
                12756, 7480,  54524, 40964, 9584,  62120, 2560,  39804, 46660,
                34944, 8932,  62176, 23164, 2244,  49900, 7444,  61940, 53156,
                34288, 4764,  54244, 17124, 22604, 40884, 13180, 54052, 38012,
                49480, 55904, 64084, 28424, 45544, 59540, 19924, 42528, 62792,
                55972, 46492, 4504,  51404, 9504,  49388, 39504, 20492, 34668,
                41948, 59168, 13828, 45100, 2536,  12048, 48076, 3700,  63816,
                18592, 12848, 59952, 53680, 32072, 45472, 37840, 29272, 3304,
                13024, 56424, 23672, 43828, 34256, 53952, 1568,  64656, 20892,
                14204, 35404, 61032, 11236, 33460, 22352, 15060, 8212,  19760,
                52452, 42184,
            },
            {
                34748, 8076,  53228, 24584, 17088, 58292, 61880, 31452, 64,
                46736, 24356, 40436, 48676, 31224, 37608, 57412, 17436, 65140,
                31696, 43976, 59180, 21840, 41176, 6220,  20112, 14852, 24948,
                62752, 55080, 10996, 45652, 26604, 13380, 43884, 37040, 1744,
                46532, 21496, 4716,  12328, 58084, 44096, 2288,  32440, 56772,
                18404, 45160, 20912, 60072, 25136, 50316, 28428, 17756, 56160,
                30368, 26064, 45484, 42084, 13372, 38544, 15844, 31308, 2668,
                40424, 61020, 25468, 32244, 50524, 2472,  46228, 30176, 18348,
                76,    9792,  15828, 21324, 2600,  11924, 32076, 14980, 34404,
                10044, 18500, 28076, 32948, 58720, 26044, 17288, 45524, 10120,
                25080, 960,   38304, 32788, 65336, 36664, 55268, 29340, 40040,
                33256, 26652, 47500, 21924, 63464, 1604,  52192, 59452, 49736,
                35668, 41284, 8716,  20128, 58408, 16252, 37916, 9740,  52908,
                26880, 2568,  49420, 18640, 54300, 68,    30832, 56552, 36752,
                3440,  27608,
            },
            {
                44952, 12352, 31776, 6344,  37824, 29548, 19948, 5256,  54700,
                16168, 64912, 6492,  14788, 61508, 4268,  8720,  23124, 39448,
                49148, 15472, 8248,  29152, 64132, 34152, 53624, 36996, 59400,
                8060,  29232, 19488, 1232,  59724, 5476,  23232, 63552, 56964,
                14304, 42112, 36328, 49212, 30112, 19072, 54060, 36596, 24520,
                3880,  29432, 33392, 344,   13888, 53452, 37800, 5080,  63240,
                14992, 712,   58916, 32120, 20152, 65036, 43724, 50988, 11736,
                21400, 48236, 7880,  56428, 36820, 6108,  64984, 57284, 23680,
                59792, 40132, 47352, 52404, 37708, 61820, 7624,  27012, 52688,
                3048,  37724, 61000, 13304, 42952, 1496,  64360, 31296, 60512,
                48640, 54772, 21412, 51620, 9600,  25800, 43376, 14688, 7460,
                57560, 4360,  36296, 7724,  25584, 14608, 33664, 10468, 21436,
                27308, 61096, 552,   50776, 12232, 28860, 42668, 57384, 32904,
                40488, 56224, 23580, 38248, 63616, 46108, 41728, 24656, 64456,
                47616, 59284,
            },
            {
                23620, 56728, 61516, 42860, 49508, 13624, 52352, 44480, 22360,
                38604, 9876,  35276, 21372, 55528, 25772, 47540, 34144, 12628,
                3544,  36344, 56712, 18120, 47404, 10080, 27948, 3068,  45896,
                33072, 22800, 49828, 64816, 39116, 32032, 48952, 10792, 34052,
                29292, 61700, 53424, 24252, 64168, 14484, 5916,  47008, 39948,
                63456, 49236, 57300, 43156, 35740, 19708, 8816,  44396, 22684,
                49228, 36888, 51796, 10500, 54348, 4808,  27220, 23564, 58628,
                38232, 29424, 14972, 44196, 20260, 27932, 16004, 42320, 11472,
                35076, 26720, 6676,  32964, 43256, 55836, 22572, 49872, 58880,
                43784, 24416, 53736, 7416,  22120, 55416, 5404,  37200, 15056,
                7604,  29388, 16416, 47128, 6096,  19972, 59900, 49836, 23484,
                44668, 54272, 15900, 42432, 49172, 57940, 40608, 4112,  44252,
                15128, 53372, 32116, 36904, 63632, 23072, 3176,  47600, 17460,
                5424,  61944, 8576,  13324, 27208, 6652,  50708, 10592, 17932,
                5996,  14456,
            },
            {
                40512, 580,   35712, 20864, 3608,  64020, 9244,  25840, 59100,
                32164, 50744, 60400, 45440, 29672, 11420, 41880, 62824, 53428,
                60312, 24492, 42728, 52416, 2356,  23452, 58064, 17012, 51344,
                12524, 43212, 35048, 14540, 53632, 17316, 41104, 20640, 51968,
                3792,  18420, 9360,  788,   39180, 27248, 60412, 21648, 8408,
                15308, 11328, 26316, 6760,  47484, 65432, 57924, 32852, 11232,
                41492, 18408, 6956,  28780, 47596, 35464, 57012, 9028,  46124,
                17840, 648,   61976, 10936, 59212, 53212, 33340, 7956,  51672,
                19704, 62584, 54548, 13032, 17956, 29744, 1368,  39548, 16492,
                11264, 30116, 48724, 40520, 35320, 47116, 27800, 52556, 22664,
                43356, 63060, 35528, 58732, 41504, 31644, 2336,  35200, 10816,
                62060, 20108, 30540, 60848, 11628, 28568, 19028, 54836, 63860,
                5884,  24380, 45444, 17680, 48712, 7568,  59652, 25748, 12736,
                45776, 30456, 43616, 51844, 34572, 16532, 55660, 29700, 38864,
                33116, 54112,
            },
            {
                28584, 10104, 50964, 15688, 27224, 47100, 34320, 41524, 12824,
                2912,  27268, 18048, 1664,  38340, 52584, 18944, 472,   28400,
                20580, 6844,  30764, 11716, 62592, 37920, 42120, 30940, 62096,
                5032,  57240, 2664,  24892, 8536,  28180, 57756, 7092,  44940,
                26344, 59488, 47716, 32356, 44652, 10924, 50468, 30888, 42504,
                53268, 59748, 38228, 16800, 23504, 30680, 1720,  26848, 52716,
                60640, 25016, 63880, 40216, 22188, 17036, 3088,  31804, 63276,
                52980, 34580, 49452, 39844, 3556,  24244, 47564, 38108, 1860,
                44332, 29312, 3112,  24084, 65436, 9736,  45708, 35748, 62524,
                21024, 6072,  64848, 14912, 19144, 61632, 12060, 40980, 2860,
                57000, 10932, 4124,  27400, 12704, 52428, 63396, 16804, 28984,
                39440, 940,   52264, 37932, 3140,  23336, 46912, 35172, 30332,
                38860, 57492, 11092, 4492,  40984, 30264, 35192, 51704, 64240,
                22252, 36868, 19040, 1852,  60244, 40296, 21552, 2372,  62048,
                49744, 19900,
            },
            {
                63720, 45888, 31028, 60860, 39344, 56996, 18232, 6232,  63068,
                48700, 55220, 42924, 13672, 58120, 7084,  32752, 44872, 14564,
                40208, 50256, 34496, 46020, 26592, 14452, 49400, 8672,  21812,
                40104, 18792, 52292, 37772, 61012, 47256, 316,   62888, 37288,
                12760, 39940, 22512, 16420, 57460, 35348, 55380, 1400,  18020,
                34300, 28740, 2772,  50692, 55008, 12996, 40004, 46820, 5924,
                14332, 34132, 2672,  55040, 12696, 61388, 42216, 37336, 13084,
                6528,  20652, 25708, 31180, 43340, 13272, 62892, 21396, 60076,
                15328, 48484, 41300, 37116, 49440, 59296, 25852, 4476,  47344,
                32988, 56376, 736,   26724, 32176, 8532,  50536, 17604, 34112,
                25564, 46320, 39040, 18308, 55784, 24288, 37752, 5164,  56992,
                46684, 25076, 9196,  33348, 56624, 62996, 7104,  16656, 9152,
                50080, 20432, 28064, 62016, 55408, 13652, 18972, 39756, 9864,
                3652,  54400, 58560, 24600, 31792, 47944, 12224, 57628, 26168,
                44308, 4096,
            },
            {
                17172, 24840, 54572, 7616,  22436, 2576,  53728, 30024, 36652,
                20836, 8552,  34172, 24580, 64356, 22204, 48284, 59548, 8804,
                55832, 64500, 16452, 4760,  54544, 20164, 384,   55596, 35596,
                26804, 63920, 29764, 44768, 12076, 33180, 23748, 15140, 30208,
                55804, 51272, 5300,  62452, 3188,  19980, 25196, 64756, 45436,
                5496,  22628, 62340, 41636, 9472,  59352, 18904, 36472, 55900,
                21428, 44548, 49512, 30048, 8112,  45680, 52052, 24508, 28640,
                44756, 55856, 64532, 9124,  18416, 56752, 5960,  31752, 26024,
                55068, 9136,  57868, 5200,  15036, 31784, 19268, 55368, 13144,
                51992, 41548, 37016, 58192, 45924, 3688,  59504, 29716, 64008,
                53868, 20476, 32316, 60796, 460,   48472, 43972, 20776, 51300,
                13320, 64764, 18712, 44476, 14432, 40900, 51320, 25412, 59208,
                42232, 1888,  34196, 52324, 22000, 44028, 1036,  56800, 32672,
                27960, 49320, 14864, 10188, 64824, 5684,  43128, 35256, 8540,
                13600, 37760,
            },
            {
                58844, 5108,  12684, 43312, 32792, 14696, 44884, 24180, 58520,
                15868, 46776, 29200, 3852,  50440, 35992, 16836, 30468, 24776,
                36972, 2108,  22724, 58780, 39280, 32916, 60776, 44188, 13228,
                47684, 9756,  16268, 5848,  21236, 54356, 42372, 49360, 19408,
                8124,  33412, 43656, 28880, 37736, 48664, 8592,  13644, 39260,
                57884, 48272, 19340, 32516, 45284, 25420, 3480,  64040, 28376,
                9688,  59632, 16460, 36824, 26456, 56432, 19212, 1540,  60456,
                15864, 38608, 2400,  52424, 35516, 27600, 49984, 40724, 11776,
                36008, 18708, 33540, 22388, 52948, 44280, 40212, 8264,  28840,
                22764, 17332, 9572,  49892, 20364, 39324, 23980, 44008, 1524,
                13388, 6132,  50828, 42480, 9856,  14752, 30580, 7680,  34092,
                41180, 28084, 5948,  49692, 21880, 29740, 204,   37092, 12588,
                64980, 47812, 14836, 38216, 8760,  26464, 62720, 47052, 15820,
                42424, 7040,  38004, 46024, 27648, 52652, 15944, 23416, 60664,
                31560, 52076,
            },
            {
                41220, 35832, 47892, 57760, 37924, 64664, 10440, 51520, 40528,
                5036,  62136, 38688, 56612, 10360, 41092, 2980,  53696, 12308,
                43548, 47776, 28108, 10600, 42812, 7912,  29356, 23628, 5564,
                59272, 31940, 50096, 56644, 39512, 4300,  65456, 35880, 3284,
                61324, 24864, 11664, 54040, 15172, 42092, 31796, 52904, 26744,
                10420, 36772, 14516, 7352,  53620, 34912, 15544, 48332, 41984,
                31732, 116,   39832, 62656, 4576,  33832, 10608, 40784, 48856,
                32524, 11724, 23044, 46664, 60584, 16140, 508,   44820, 64688,
                4540,  46732, 27660, 61692, 11904, 336,   63320, 34604, 58516,
                3100,  44428, 25252, 63140, 13044, 34576, 55892, 15772, 36724,
                48784, 62092, 28920, 22924, 35788, 64136, 54108, 26416, 61412,
                3380,  58832, 35708, 55588, 10832, 61284, 54104, 46076, 32884,
                18336, 23924, 3844,  60948, 31576, 50404, 6472,  36588, 24092,
                57960, 61548, 21284, 33800, 504,   19668, 39312, 55064, 48684,
                1244,  21180,
            },
            {
                9656,  29380, 17948, 1892,  26684, 19892, 48428, 416,   27920,
                12264, 52336, 18584, 44084, 14832, 60024, 27116, 63288, 5940,
                19452, 32144, 61584, 52628, 18428, 49688, 65152, 15904, 53036,
                40568, 1372,  36456, 60768, 26656, 17692, 10076, 27720, 57156,
                41260, 46760, 21276, 63524, 6912,  58376, 22956, 61804, 2396,
                51200, 30140, 63784, 772,   27616, 60764, 21664, 5000,  52044,
                13252, 54128, 19556, 23376, 50848, 14708, 65312, 22248, 57340,
                4120,  51388, 42024, 29924, 7560,  38284, 54176, 20456, 24120,
                51376, 59444, 39604, 7504,  30388, 50588, 24424, 16148, 48872,
                61212, 31076, 54156, 6744,  28340, 4720,  51528, 10072, 22232,
                40244, 8040,  16556, 57832, 4524,  39700, 46928, 17408, 11576,
                22812, 47456, 15824, 31440, 39204, 4180,  20204, 27536, 6552,
                55644, 29476, 45260, 58028, 17444, 12164, 54624, 20316, 2588,
                30396, 11848, 50600, 41612, 56492, 62724, 29568, 4856,  45044,
                27008, 65452,
            },
            {
                56596, 50084, 23368, 60032, 53016, 8024,  31572, 35512, 60720,
                22164, 33416, 26160, 7176,  31680, 21132, 45348, 34312, 51072,
                57864, 38752, 14360, 5092,  35424, 25856, 2952,  37720, 46092,
                18352, 25424, 21916, 13656, 47168, 32460, 44192, 51536, 12664,
                16768, 912,   35600, 50004, 27272, 3864,  46104, 35000, 16316,
                41860, 21156, 56412, 38196, 50612, 40740, 10488, 57404, 37224,
                25900, 46364, 6488,  58572, 42804, 47292, 30736, 7684,  36480,
                17744, 27036, 59332, 20072, 13568, 63112, 33360, 8792,  30544,
                14308, 1788,  16928, 43804, 54912, 19988, 37516, 5456,  41944,
                11312, 36636, 15216, 40584, 57244, 45276, 65264, 31668, 26756,
                58796, 44744, 33768, 52240, 25184, 20032, 1964,  57100, 37128,
                52112, 42496, 1280,  63400, 24148, 43760, 14700, 58692, 9888,
                39000, 52108, 5708,  35212, 42932, 39416, 29244, 65268, 43712,
                52472, 17800, 4036,  25404, 13840, 8200,  35908, 11676, 18508,
                33608, 14948,
            },
            {
                42476, 3624,  34856, 11692, 40260, 62620, 43748, 14372, 56184,
                41844, 3120,  65276, 53908, 48528, 652,   38236, 17976, 9316,
                25588, 1512,  22112, 56064, 45584, 12472, 57348, 33284, 9496,
                55292, 63664, 42984, 7280,  53200, 1936,  59020, 21016, 30824,
                39100, 60452, 9396,  32188, 17484, 40164, 11544, 19836, 55048,
                5764,  45068, 24216, 11988, 16972, 31424, 44716, 24024, 17896,
                63372, 33324, 28976, 11812, 35640, 1148,  27488, 54216, 44616,
                63236, 5904,  34964, 55604, 2992,  25864, 46148, 58332, 42676,
                37704, 56472, 25452, 35008, 64400, 13688, 46944, 26772, 56104,
                20808, 1624,  47676, 33216, 21888, 17824, 200,   42256, 53104,
                19220, 2428,  12564, 47648, 10288, 60284, 32304, 44540, 28924,
                8444,  19680, 26816, 49316, 7732,  53284, 33848, 48560, 41972,
                62932, 21256, 11596, 25780, 568,   22844, 49008, 14532, 9248,
                34236, 39632, 63500, 31592, 47800, 43892, 57668, 51924, 61684,
                38564, 6668,
            },
            {
                25124, 61972, 45904, 15940, 29732, 5332,  24064, 17996, 50212,
                9420,  46340, 36988, 11548, 24212, 57324, 61744, 13040, 46816,
                41288, 64596, 48980, 30020, 39692, 62624, 19888, 51352, 28268,
                4176,  30896, 11400, 34828, 62272, 38224, 24116, 6120,  47464,
                55248, 25816, 52460, 44488, 56944, 65144, 29248, 48344, 59456,
                32396, 8936,  61256, 47324, 3360,  65164, 7720,  49384, 1612,
                59980, 8712,  41036, 55768, 17348, 61224, 20324, 39712, 13140,
                24232, 47912, 10112, 44048, 39316, 49612, 17816, 3968,  11352,
                62488, 21176, 49040, 9996,  4068,  40632, 59996, 9212,  29992,
                63984, 52744, 24512, 61936, 8908,  49640, 37888, 13884, 5548,
                62796, 30516, 37992, 55572, 27688, 41868, 14464, 5792,  65364,
                54716, 34264, 13356, 57292, 37676, 17644, 61984, 22712, 1448,
                36016, 16700, 49476, 64064, 46496, 59956, 5284,  57200, 26812,
                46252, 58576, 6752,  54512, 21736, 16596, 26460, 1940,  22736,
                47392, 53644,
            },
            {
                19320, 32096, 55928, 21636, 52056, 47056, 58608, 38016, 4508,
                30564, 20132, 59008, 16884, 42392, 29448, 4816,  33064, 55196,
                28816, 6840,  34536, 17128, 8636,  844,   22984, 41348, 14928,
                58496, 45156, 50912, 19820, 28540, 15816, 9116,  64244, 34392,
                4404,  14184, 20080, 2604,  23960, 7864,  37260, 484,   25604,
                14868, 39784, 28560, 33752, 20208, 54428, 27020, 35056, 12724,
                43708, 21088, 51372, 4432,  25364, 49864, 9132,  58276, 2620,
                33252, 53584, 16592, 60996, 31104, 22484, 53316, 36232, 28892,
                47148, 5396,  32016, 52896, 28220, 22720, 1180,  50948, 33892,
                18052, 44844, 4288,  12412, 58648, 30168, 25680, 60592, 34492,
                45824, 23956, 17588, 64328, 7316,  35432, 50580, 23984, 16864,
                39540, 59912, 4684,  45256, 29364, 11156, 26448, 5124,  31284,
                13500, 57568, 27896, 32844, 9592,  19200, 31352, 36292, 2152,
                19864, 24264, 10824, 36720, 2884,  61080, 32544, 42064, 8888,
                29228, 13204,
            },
            {
                168,   39284, 9992,  7196,  36460, 1712,  26836, 12548, 63488,
                53360, 27444, 1920,  34080, 49788, 8340,  22396, 52384, 15580,
                20724, 58484, 11540, 53668, 44032, 27448, 59808, 47992, 7072,
                37376, 23828, 40,    56768, 39800, 48956, 54324, 43468, 18060,
                28128, 62800, 42472, 36272, 49144, 13448, 53592, 42956, 62508,
                50200, 4560,  57616, 51876, 42312, 14192, 59404, 39264, 30244,
                55032, 15592, 38072, 64416, 32412, 45660, 37000, 15912, 50900,
                41316, 21160, 28348, 1196,  12608, 7044,  65048, 14988, 24440,
                55152, 18636, 42100, 61228, 16052, 57176, 36548, 43596, 14148,
                7096,  39476, 28000, 35496, 43204, 54004, 7496,  21136, 56036,
                9644,  50328, 40832, 4032,  21824, 53968, 61240, 288,   46588,
                10504, 31116, 22432, 63856, 1880,  41356, 51816, 45844, 59940,
                54204, 43388, 6824,  38308, 53604, 15536, 41244, 55072, 50012,
                61492, 15288, 43408, 27740, 51424, 39268, 12080, 58652, 50008,
                36048, 64148,
            },
            {
                45088, 27212, 59276, 42708, 64800, 18992, 55132, 33272, 44404,
                22972, 40896, 61480, 14408, 56344, 39396, 64072, 44424, 36316,
                3796,  42376, 24804, 61380, 50244, 36080, 32184, 11772, 64140,
                17752, 33856, 61060, 13964, 5304,  25872, 31544, 11708, 37204,
                49912, 57364, 10908, 29912, 61068, 32900, 16808, 21196, 10264,
                35268, 22968, 18068, 1492,  36884, 6800,  22180, 51052, 5132,
                47032, 27904, 1992,  23196, 12868, 6116,  29680, 62444, 26012,
                5236,  64380, 37544, 56708, 51436, 41064, 32708, 58976, 780,
                39052, 8468,  34556, 12452, 45700, 30352, 19096, 53800, 25736,
                60552, 49304, 56604, 15728, 19600, 2484,  39820, 48356, 15920,
                29104, 984,   57672, 14104, 44960, 30636, 18968, 37896, 26336,
                49816, 41940, 52520, 16092, 34776, 56248, 19316, 36748, 8756,
                24156, 18524, 2624,  50840, 62148, 25024, 44440, 7712,  12300,
                38896, 30276, 48180, 64404, 18400, 6000,  46176, 20792, 15616,
                4880,  55176,
            },
            {
                17320, 51448, 22960, 14720, 30980, 49940, 40060, 7868,  16108,
                48620, 10652, 6712,  45848, 25516, 18912, 2704,  27236, 10404,
                62844, 48192, 31756, 14504, 5832,  18536, 3604,  54876, 43552,
                27200, 49848, 9940,  42168, 46592, 58868, 3244,  61392, 22912,
                640,   40092, 7332,  22116, 55584, 5552,  46876, 39348, 28292,
                54652, 46172, 63996, 11324, 26388, 61760, 43472, 18212, 10524,
                62896, 34612, 56724, 48064, 60372, 42848, 20780, 55020, 11452,
                47276, 32148, 8596,  43536, 19160, 26676, 48100, 9624,  44256,
                50616, 63940, 26584, 2316,  49504, 6316,  65396, 11040, 3372,
                37264, 22172, 9744,  64556, 47036, 32572, 62528, 11580, 42868,
                36484, 60212, 25648, 33124, 48932, 8256,  12044, 57552, 33828,
                3316,  19932, 8080,  25248, 48220, 6784,  13816, 65156, 29720,
                39864, 47956, 34700, 21800, 11188, 4336,  28596, 64004, 22400,
                33660, 404,   54180, 8972,  34380, 55544, 29888, 63200, 26092,
                41040, 33316,
            },
            {
                12196, 3952,  47608, 35372, 2852,  11212, 24824, 62056, 540,
                35628, 55480, 31348, 37464, 52124, 11928, 32064, 57828, 50444,
                17912, 38220, 1616,  21724, 40384, 57516, 25204, 39068, 21244,
                2712,  53688, 30672, 22264, 16796, 35760, 19984, 52768, 46056,
                15500, 33112, 52160, 18168, 44692, 25924, 64368, 2128,  57936,
                7184,  13664, 41028, 30188, 47932, 55676, 32488, 332,   58588,
                25504, 19732, 8184,  39428, 17136, 52236, 34884, 396,   44496,
                18000, 59108, 14020, 24272, 62080, 3600,  37020, 21388, 29344,
                17092, 57436, 21964, 37616, 59588, 24328, 40108, 32812, 55960,
                46188, 29828, 992,   51748, 24564, 5600,  27676, 51304, 23132,
                6300,  52592, 18256, 39032, 63508, 28144, 42632, 51872, 62828,
                14676, 55332, 59616, 38716, 61528, 32392, 22124, 50072, 308,
                11964, 61652, 56284, 30400, 45552, 58896, 35680, 17252, 51240,
                58344, 5632,  20896, 41592, 14164, 23844, 1236,  37820, 52368,
                8048,  60576,
            },
            {
                30248, 38464, 62864, 57524, 28360, 45892, 59500, 20416, 51628,
                29068, 18164, 64320, 21872, 5248,  60408, 46900, 40780, 23796,
                8120,  29420, 55372, 45908, 65360, 51468, 8896,  47444, 15584,
                61896, 6568,  37792, 65000, 55296, 8456,  41456, 27232, 6288,
                64612, 25072, 58392, 37680, 3552,  12820, 50972, 31288, 19012,
                43812, 24748, 33528, 53012, 8508,  15748, 24048, 46392, 37356,
                41840, 14008, 52976, 28704, 3980,  10332, 24468, 59888, 38180,
                28996, 49932, 2260,  34768, 45936, 15860, 55888, 63096, 12260,
                33400, 3472,  42616, 14460, 52804, 9304,  44916, 20608, 14712,
                62652, 17572, 34544, 41444, 13360, 37656, 59328, 17808, 33928,
                65132, 44268, 10572, 5128,  54576, 1656,  16528, 23156, 5360,
                27860, 44128, 30184, 12036, 2692,  43252, 28048, 58128, 44796,
                17152, 26708, 7632,  14776, 39384, 20144, 48128, 3204,  40588,
                25164, 44812, 60712, 26900, 49436, 58432, 44156, 10812, 18884,
                48160, 24068,
            },
            {
                55704, 20668, 8288,  18260, 53828, 5348,  32648, 43260, 38076,
                12928, 59080, 2476,  43904, 28404, 16364, 34476, 144,   13896,
                52012, 60264, 34992, 6700,  27988, 12764, 31412, 59140, 34876,
                28456, 12972, 48092, 25600, 1224,  33228, 60328, 12112, 30932,
                42892, 48804, 9236,  29192, 41684, 59524, 35060, 9660,  38124,
                62188, 2892,  59780, 19672, 4636,  40116, 64812, 12504, 54120,
                31048, 2840,  44336, 58256, 32796, 65192, 41508, 15076, 7932,
                22544, 56168, 40372, 53716, 10788, 30752, 5628,  52020, 40080,
                7412,  54240, 47872, 31268, 18312, 35400, 356,   51368, 28200,
                42488, 7260,  58540, 54800, 20660, 45516, 9256,  56332, 1480,
                13752, 31156, 20720, 46508, 24188, 34200, 58460, 39676, 47372,
                9936,  36040, 21720, 46916, 53732, 15956, 37440, 5408,  52992,
                33508, 41792, 51976, 64548, 1396,  54508, 31516, 13312, 56764,
                8296,  15788, 31856, 11504, 36556, 3860,  62332, 31444, 56968,
                4436,  43124,
            },
            {
                1500,  33812, 44000, 12620, 40412, 23292, 15032, 56020, 3648,
                26264, 41148, 50028, 9768,  53872, 38704, 62372, 21124, 56568,
                42468, 25732, 10780, 16680, 43484, 37340, 19128, 1120,  45048,
                56204, 40136, 18488, 51892, 43900, 14944, 50324, 38652, 55576,
                17764, 1692,  13932, 62496, 21292, 48292, 15888, 23104, 52420,
                47264, 16696, 36528, 48888, 57080, 28564, 35508, 20860, 5696,
                49292, 63412, 22332, 36620, 18460, 48672, 27120, 51460, 31812,
                63744, 4692,  19776, 27484, 60340, 49020, 18476, 26028, 45144,
                23420, 61532, 27420, 4676,  63264, 55320, 25096, 60680, 4092,
                48348, 11880, 26184, 31508, 61656, 2956,  29616, 42176, 38344,
                48860, 27332, 56916, 62052, 37244, 11084, 49660, 19628, 31504,
                65008, 708,   58668, 18376, 7656,  63204, 24452, 10840, 20308,
                60016, 3596,  23312, 36792, 26080, 10252, 42692, 63028, 28032,
                34888, 47272, 65520, 51736, 17880, 40812, 22128, 15204, 35548,
                27632, 65004,
            },
            {
                15736, 53048, 26564, 50564, 61420, 36628, 9580,  65524, 47940,
                7996,  22740, 33580, 14436, 58300, 25240, 7652,  48988, 31096,
                4712,  19020, 47360, 61524, 3728,  23708, 62712, 52872, 7448,
                22504, 32620, 3876,  10520, 58480, 29600, 23744, 2832,  21408,
                61092, 36796, 33732, 53276, 5784,  26864, 176,   56280, 29688,
                6156,  27296, 10972, 22460, 44388, 2192,  51396, 9912,  60216,
                16248, 26500, 9300,  46348, 13168, 6536,  56496, 2060,  45812,
                12460, 43176, 33488, 8216,  38112, 42352, 196,   35336, 59316,
                13992, 38468, 20104, 10904, 45940, 40916, 13192, 32468, 38156,
                22456, 53168, 40016, 4876,  16232, 47976, 25280, 54248, 21820,
                61148, 7764,  41032, 3260,  14792, 29324, 60556, 6456,  54768,
                16044, 38504, 51288, 26612, 41004, 34324, 47660, 55424, 39016,
                30776, 13488, 47248, 18288, 59584, 50048, 5208,  22768, 52940,
                19272, 39020, 2556,  6460,  28852, 54220, 8548,  46896, 50768,
                11752, 39856,
            },
            {
                23456, 45800, 6140,  31256, 776,   46844, 27944, 19264, 31536,
                54644, 62604, 46384, 19904, 30412, 3364,  43148, 11476, 36212,
                63908, 39620, 52732, 33268, 57136, 49136, 40800, 26364, 11756,
                59976, 42588, 62956, 21052, 36168, 5932,  63988, 45016, 10056,
                47136, 24620, 56984, 19184, 44572, 38336, 63848, 42272, 12612,
                58828, 39524, 63624, 31872, 14232, 61384, 24924, 43144, 29648,
                34804, 40508, 55244, 784,   61276, 23496, 39056, 20220, 36120,
                53116, 17044, 57572, 23840, 14604, 65316, 21628, 56448, 8800,
                49964, 2012,  34004, 57620, 28912, 6988,  17212, 56884, 9500,
                64840, 18516, 35004, 50664, 10364, 64152, 33132, 12156, 4216,
                16540, 35368, 53300, 18976, 51012, 44020, 2224,  41364, 25172,
                11896, 44912, 32744, 3640,  60784, 13772, 29496, 1556,  63852,
                8608,  50704, 43208, 6104,  29204, 34304, 16712, 38020, 796,
                10956, 55908, 24000, 58108, 45548, 33736, 25216, 60744, 5568,
                19940, 59460,
            },
            {
                9312,  36836, 63560, 21164, 16868, 57464, 51708, 4864,  42360,
                16448, 36104, 1168,  40268, 51252, 64948, 18076, 53404, 23064,
                14872, 1460,  28864, 21484, 13956, 9120,  30192, 17260, 36788,
                48652, 15312, 27176, 53180, 47308, 40948, 17080, 28052, 54048,
                31924, 7048,  41108, 11056, 30300, 51292, 8588,  33420, 20592,
                45500, 1696,  53692, 50184, 6432,  37456, 17432, 54872, 47432,
                4376,  59236, 19516, 32024, 52720, 42940, 30416, 58932, 9724,
                25144, 48844, 61572, 2480,  46636, 51544, 11968, 32252, 28672,
                17404, 43736, 64608, 48852, 22304, 36744, 50780, 43184, 30080,
                45672, 1264,  59368, 23692, 43692, 36524, 18900, 50192, 59148,
                44968, 24576, 9844,  32480, 26248, 63812, 21852, 34880, 48760,
                58020, 7012,  22772, 55732, 10040, 49680, 22040, 44380, 17564,
                27064, 35176, 61428, 54840, 12476, 46268, 57132, 60912, 44264,
                49604, 30172, 42444, 14952, 20336, 63304, 276,   37404, 42756,
                32288, 54556,
            },
            {
                48576, 14200, 55496, 41580, 10984, 34644, 24320, 38684, 60724,
                11364, 25484, 6216,  55952, 12892, 33992, 47544, 27144, 59832,
                44312, 57688, 7312,  37612, 45580, 2252,  51800, 63968, 5336,
                54940, 480,   31124, 8128,  56740, 12700, 34524, 59264, 4412,
                15040, 49832, 65176, 3492,  16900, 60304, 24940, 14888, 55436,
                35312, 24296, 9232,  19260, 41256, 58008, 33264, 7760,  12916,
                23636, 50516, 14660, 37844, 27528, 11064, 15988, 62560, 3452,
                28828, 39608, 6608,  30556, 36500, 26432, 4080,  40924, 53436,
                60224, 25428, 5056,  14820, 53940, 1488,  62992, 20888, 5272,
                26384, 15148, 54288, 29148, 7452,  57396, 128,   39172, 28720,
                5888,  64628, 47532, 56628, 38820, 7264,  13636, 53500, 28420,
                17796, 63108, 30212, 42560, 18952, 38136, 4456,  58208, 52256,
                41068, 22856, 880,   20564, 39932, 24740, 8472,  31932, 13704,
                26232, 63700, 7636,  37076, 10204, 49244, 13468, 56112, 16720,
                28620, 2972,
            },
            {
                24916, 33532, 3776,  29252, 59924, 43908, 2300,  13916, 53128,
                30092, 49804, 59496, 28140, 22152, 9144,  38296, 5680,  31964,
                9920,  50300, 25908, 61156, 54344, 32512, 20796, 44288, 34516,
                23824, 38792, 45840, 19176, 25052, 3220,  50700, 20528, 42620,
                37644, 27168, 22228, 45764, 57816, 35812, 2388,  47440, 4872,
                61216, 16144, 43600, 29780, 26140, 944,   62736, 20740, 38656,
                64892, 2896,  44964, 7556,  63772, 48588, 4980,  46208, 34660,
                55344, 13720, 44172, 20716, 54932, 18096, 62468, 45712, 6404,
                36324, 10700, 39600, 31848, 41824, 26924, 11748, 34784, 59868,
                48864, 37552, 11984, 41008, 20612, 46140, 14420, 53148, 23024,
                42704, 30520, 12964, 2776,  17228, 45828, 59364, 9176,  37364,
                1320,  47508, 14328, 34744, 53060, 64692, 25992, 33372, 6580,
                14804, 56796, 10560, 32956, 65092, 3280,  51808, 18756, 36100,
                5144,  21508, 53288, 32712, 59716, 40392, 30548, 22308, 52092,
                64392, 39364,
            },
            {
                17904, 62324, 52004, 8304,  26108, 48972, 63888, 22236, 45592,
                8516,  37672, 17264, 44996, 41832, 61144, 792,   54528, 16496,
                40564, 34872, 18396, 11592, 41720, 15152, 59760, 27452, 10444,
                58088, 13296, 65256, 33872, 60044, 39576, 30044, 63520, 11272,
                57332, 52528, 33516, 13128, 39916, 19928, 50128, 27956, 31380,
                51776, 38496, 65064, 56640, 48360, 11860, 45200, 52464, 27936,
                42052, 33864, 25200, 56816, 21392, 33404, 52080, 19032, 41952,
                22888, 63944, 10304, 49820, 59672, 8668,  34444, 15248, 23916,
                20444, 47916, 55916, 18384, 8556,  58216, 47080, 16120, 54856,
                8292,  31364, 63864, 2876,  60600, 26824, 32260, 62420, 10760,
                55724, 19568, 35876, 61336, 52280, 29576, 33432, 19896, 61772,
                40272, 24688, 56748, 3096,  7872,  46296, 16152, 11356, 37452,
                62704, 45324, 29408, 48912, 16224, 42364, 28208, 61960, 47460,
                41196, 58768, 16512, 44752, 6196,  25720, 3976,  46048, 7776,
                11572, 44376,
            },
            {
                56548, 21860, 45964, 39760, 19180, 15684, 6900,  33260, 56792,
                19536, 64536, 2048,  32128, 14736, 57144, 24960, 45456, 63652,
                21980, 3944,  47748, 65040, 23652, 6280,  39320, 3004,  18556,
                49944, 41888, 6436,  52832, 16344, 9488,  47736, 2100,  25592,
                18124, 7812,  816,   29328, 54432, 9560,  64096, 42600, 11116,
                21920, 7200,  13336, 3916,  36160, 22516, 31728, 15664, 5368,
                57708, 10852, 53984, 16784, 1508,  39980, 25940, 9012,  58164,
                1900,  16840, 33144, 38408, 952,   42612, 29604, 58204, 52356,
                63748, 720,   29032, 60804, 51224, 23308, 32496, 2460,  40256,
                24428, 44220, 17812, 51660, 34308, 48244, 4604,  17532, 37212,
                1760,  48592, 40824, 22488, 6360,  42264, 24204, 50264, 4408,
                43968, 11160, 50872, 20532, 28608, 41512, 60632, 30376, 49812,
                24348, 2648,  53112, 36468, 58944, 7056,  22528, 54452, 2716,
                11720, 50584, 1044,  28452, 18988, 51084, 62140, 35040, 58000,
                27312, 1104,
            },
            {
                36624, 6428,  12600, 30652, 58304, 53704, 37284, 28676, 3432,
                40700, 24732, 47808, 7088,  51452, 35904, 19240, 29804, 12380,
                52016, 56780, 31196, 284,   28728, 50784, 55816, 46680, 62200,
                30104, 22072, 2384,  28692, 43676, 22832, 54760, 36688, 44976,
                60904, 40940, 48456, 62716, 23404, 6100,  37700, 17372, 58776,
                46504, 27136, 33448, 17860, 54676, 59732, 8900,  50020, 61728,
                18728, 30304, 47756, 36412, 59908, 13244, 62084, 31172, 37304,
                50684, 24848, 53336, 27508, 14288, 21708, 48896, 3128,  12320,
                32724, 38384, 43564, 13576, 4200,  44592, 37960, 61208, 19908,
                50312, 4256,  57108, 22640, 13224, 8992,  41976, 51028, 26020,
                59320, 8424,  15196, 27388, 58056, 388,   12364, 55600, 15924,
                27540, 32292, 63548, 36952, 58804, 23220, 16,    54732, 17992,
                43316, 9296,  19596, 25820, 12656, 45616, 34116, 15168, 30872,
                39008, 23520, 34400, 64972, 54812, 10720, 42032, 15792, 20436,
                49948, 31388,
            },
            {
                15244, 60540, 50904, 34780, 400,   41924, 11940, 61816, 51276,
                10316, 34808, 54172, 21668, 63024, 10584, 3152,  49280, 37472,
                26636, 8404,  42976, 36936, 58512, 16968, 35356, 7080,  33176,
                14860, 57296, 37416, 48832, 12184, 62960, 5528,  32172, 14724,
                21672, 35084, 55728, 16272, 43764, 32892, 25768, 53232, 632,
                40500, 60656, 52704, 44064, 39888, 2848,  25704, 43020, 37248,
                412,   22576, 41216, 8504,  28324, 44672, 53760, 6064,  12220,
                47040, 60080, 4568,  45244, 64736, 56824, 39484, 26320, 44656,
                17024, 7828,  25620, 20904, 65216, 29768, 7228,  12556, 28016,
                63196, 10368, 36552, 29832, 39752, 55844, 65304, 21096, 44948,
                31184, 53648, 63252, 34232, 49076, 38292, 65100, 46260, 35972,
                58368, 5556,  17308, 45132, 9596,  13756, 47580, 34224, 5780,
                57712, 39956, 64044, 4956,  56080, 38448, 63208, 9332,  59848,
                26884, 56484, 46784, 13560, 37620, 31192, 1776,  24796, 38660,
                61620, 43056,
            },
            {
                54448, 24312, 27672, 4912,  63080, 23244, 47196, 25936, 16112,
                44184, 60668, 13288, 27664, 39216, 31244, 43860, 61992, 4932,
                15876, 60564, 19692, 13596, 45692, 9380,  21676, 25884, 43228,
                52156, 9260,  25100, 60756, 34504, 18816, 27148, 58308, 50228,
                10144, 3832,  26392, 11796, 46904, 2808,  57508, 14128, 34916,
                19784, 5024,  23532, 10448, 29016, 62936, 20180, 34180, 46824,
                13800, 52624, 64208, 3456,  49908, 23552, 17672, 42540, 20848,
                28496, 34996, 8152,  19228, 31780, 11172, 5388,  35324, 60896,
                49676, 57544, 53564, 35928, 48756, 16572, 52676, 56500, 46088,
                33632, 15372, 43344, 59984, 1484,  24976, 15848, 6336,  35100,
                12648, 3732,  20348, 44108, 10296, 18044, 30976, 21608, 8224,
                51476, 23536, 39944, 2500,  30984, 52700, 39108, 20368, 61444,
                28008, 14444, 32736, 48208, 29612, 500,   20744, 49108, 43556,
                17084, 4344,  8388,  42744, 22404, 59392, 47788, 53956, 12960,
                4796,  9324,
            },
            {
                19248, 38856, 48300, 17284, 44516, 8904,  19736, 55800, 32432,
                5276,  18964, 57780, 2636,  46408, 17060, 56144, 22624, 41048,
                53364, 46964, 32540, 24592, 62964, 38828, 54132, 12528, 64580,
                1020,  39952, 17612, 53456, 7588,  41368, 46524, 1048,  38512,
                63660, 30804, 58948, 38844, 20328, 50880, 62252, 30356, 9772,
                49116, 64276, 31980, 47416, 14004, 51108, 6816,  56232, 10208,
                59600, 26988, 32952, 15640, 57388, 34648, 4228,  65236, 56308,
                39788, 15284, 62260, 42044, 54584, 24476, 51568, 9016,  19832,
                29900, 4992,  41552, 11044, 1536,  32984, 22156, 41716, 2156,
                25508, 53996, 6188,  19420, 46848, 52856, 38240, 28380, 61408,
                47368, 39244, 24524, 57024, 6684,  52588, 26492, 3148,  41488,
                13980, 61996, 48744, 55144, 26728, 65240, 4752,  24908, 10880,
                36236, 51652, 21580, 42808, 17384, 52360, 25116, 6468,  35780,
                52900, 20200, 62816, 27916, 3336,  17768, 7860,  64344, 28808,
                33380, 46644,
            },
            {
                1700,  64000, 11424, 57236, 31524, 36252, 59624, 1928,  50164,
                37956, 29620, 42164, 9352,  52964, 25208, 6972,  33160, 11180,
                28480, 2688,  55276, 6028,  50148, 1832,  29588, 59216, 47400,
                20396, 32040, 45428, 4024,  30284, 55956, 13012, 24404, 52760,
                18272, 43044, 5872,  53536, 28788, 7928,  41464, 23028, 44644,
                56804, 15948, 37032, 1364,  58068, 38484, 16672, 30864, 23724,
                39224, 5296,  45380, 20952, 10668, 38748, 29988, 49100, 160,
                10160, 23388, 48416, 2240,  36764, 17380, 46616, 63036, 40636,
                14512, 23436, 62128, 27524, 58116, 39404, 60404, 9408,  18920,
                37528, 48680, 28744, 62812, 8356,  31868, 11224, 57892, 892,
                18344, 51168, 29656, 14100, 32808, 61192, 44788, 59016, 53892,
                29276, 33320, 10512, 18948, 36088, 15708, 42240, 48836, 55932,
                45020, 7440,  3324,  60300, 10308, 40280, 57500, 31664, 12496,
                60856, 39564, 48352, 33344, 57456, 44956, 35576, 40428, 51608,
                23032, 58264,
            },
            {
                35456, 30076, 7576,  41084, 53032, 14244, 28784, 40308, 8068,
                63480, 23548, 48732, 34020, 65084, 14116, 59704, 49728, 36288,
                20756, 64552, 39684, 12108, 34208, 42712, 18484, 4500,  36072,
                27480, 57644, 11744, 64036, 38000, 21028, 62216, 29348, 8044,
                47864, 22948, 33728, 15304, 60172, 35488, 17888, 4520,  38068,
                28172, 7064,  54488, 26520, 22032, 45312, 65332, 1996,  49524,
                53772, 18268, 62748, 41980, 51176, 61288, 14196, 25744, 44816,
                32624, 52796, 58380, 29564, 12784, 59440, 28192, 520,   33276,
                56124, 45328, 3352,  18564, 47024, 13316, 24608, 50592, 64288,
                4652,  57036, 12856, 23236, 35572, 43992, 49500, 21568, 41124,
                55160, 9524,  64432, 42492, 1676,  37900, 15680, 11096, 35476,
                20636, 700,   46772, 60828, 6692,  57792, 32600, 2788,  17232,
                30780, 63636, 26156, 54952, 36700, 14224, 65420, 45960, 3416,
                23392, 30036, 10756, 15428, 52632, 25228, 11480, 19576, 3716,
                16204, 42536,
            },
            {
                13412, 50452, 20092, 25120, 3524,  65328, 22080, 45868, 12120,
                55164, 15004, 32,    20900, 37696, 3800,  41768, 17984, 57660,
                8324,  44024, 16724, 26784, 21936, 61920, 52684, 14768, 40876,
                6656,  51556, 23384, 15620, 49760, 2596,  44204, 16228, 36384,
                55668, 12492, 65416, 36,    49084, 10916, 46200, 63536, 52376,
                12500, 20760, 42596, 61988, 8584,  35088, 27796, 42844, 13060,
                33572, 8788,  28604, 1068,  24412, 7584,  54468, 19460, 59412,
                37540, 6916,  20232, 41088, 4056,  44112, 21956, 53220, 38212,
                11516, 51032, 31736, 37344, 54628, 6572,  30084, 35160, 16284,
                43004, 31480, 39828, 59136, 16924, 5572,  63892, 14176, 33552,
                27016, 4528,  36208, 21908, 49648, 25292, 19344, 47836, 6992,
                64508, 43648, 25672, 39228, 22408, 12372, 50812, 62180, 22936,
                40808, 13540, 46788, 33084, 23176, 2116,  27668, 50968, 16640,
                42324, 54804, 1592,  37968, 6312,  60532, 31496, 48984, 55884,
                62432, 26068,
            },
            {
                45252, 60980, 55760, 33484, 48044, 6644,  51656, 18004, 26648,
                35388, 43912, 59060, 28232, 51412, 30756, 45768, 26272, 824,
                52216, 30348, 60296, 47792, 57748, 7392,  31648, 24816, 49044,
                10156, 60360, 42784, 35392, 26360, 9708,  32508, 57848, 41484,
                2900,  25980, 44584, 39908, 21644, 27460, 55540, 24724, 1708,
                59268, 33916, 50160, 17964, 12056, 52984, 4728,  19800, 61120,
                57724, 48032, 37596, 55264, 31608, 46748, 35772, 4724,  12068,
                63932, 16056, 26556, 51236, 34352, 64304, 8284,  16320, 25236,
                5984,  64832, 21340, 8964,  42220, 62980, 140,   45092, 53096,
                22004, 10400, 1140,  51280, 26624, 55700, 3060,  24292, 46672,
                60616, 17200, 45124, 58232, 5952,  55056, 62328, 40000, 28112,
                56528, 15028, 4332,  52212, 29880, 44300, 8628,  27568, 37124,
                716,   58904, 18836, 8564,  49996, 43788, 20008, 33860, 9372,
                58760, 26668, 64224, 49800, 21992, 42904, 28,    27640, 37480,
                9884,  5464,
            },
            {
                39212, 22608, 680,   16444, 43444, 38172, 58316, 32872, 62132,
                5068,  53320, 9972,  17280, 61580, 6052,  12100, 63644, 38532,
                22876, 13852, 34920, 2496,  10384, 37972, 46028, 56544, 63404,
                17208, 33200, 300,   55180, 46692, 59788, 51060, 5016,  20276,
                60596, 31132, 52132, 17416, 57732, 4020,  32800, 14840, 40664,
                29852, 46892, 5416,  39088, 32084, 57152, 47172, 40932, 25368,
                2800,  16396, 22440, 12448, 58568, 15952, 43436, 28196, 40728,
                47592, 31344, 55968, 9812,  14476, 47092, 30536, 57216, 48812,
                43404, 27984, 58968, 16732, 50232, 26572, 14684, 58444, 7596,
                27824, 61908, 47236, 19636, 41996, 37856, 30268, 53664, 7160,
                38920, 52312, 12444, 28580, 34640, 9628,  31600, 2072,  23400,
                50100, 36980, 60004, 17420, 55612, 35052, 19744, 47512, 54372,
                10556, 51488, 28288, 39148, 62756, 5356,  54464, 61304, 37312,
                4836,  19196, 45136, 34816, 17512, 56456, 63220, 14536, 46812,
                32504, 53660,
            },
            {
                28832, 11108, 36448, 62516, 27152, 13660, 10396, 1332,  42116,
                19712, 31332, 47380, 40452, 24056, 54888, 20392, 32720, 49012,
                56100, 6364,  40768, 51632, 20056, 28820, 13052, 2132,  22420,
                39188, 28628, 19560, 6180,  13736, 22244, 38432, 27712, 11804,
                48948, 6772,  36076, 9192,  62484, 37652, 43304, 51336, 18928,
                8876,  64088, 25668, 340,   60612, 23320, 14508, 7888,  30688,
                35916, 44908, 64336, 40456, 3024,  20672, 62292, 51740, 2000,
                22708, 5752,  43560, 62220, 24540, 37668, 4832,  60824, 19400,
                35580, 13808, 2548,  33500, 23252, 36508, 48012, 20268, 32444,
                37368, 54520, 14428, 33020, 9108,  62212, 15936, 43412, 10484,
                20692, 31472, 292,   63428, 15984, 46008, 52116, 13440, 42652,
                8308,  32228, 10784, 41796, 1360,  64308, 4792,  60200, 15476,
                42740, 34464, 6240,  56180, 16156, 30696, 12400, 40928, 24420,
                47584, 13528, 31760, 7108,  11876, 24536, 40696, 8384,  21012,
                59708, 17876,
            },
            {
                4380,  57532, 46932, 8132,  53756, 30724, 23340, 49412, 56208,
                25316, 38092, 7308,  13508, 49844, 35672, 42172, 9588,  16016,
                28280, 62868, 44892, 24192, 55420, 64704, 42584, 35520, 5804,
                53700, 48308, 62276, 40476, 31004, 64292, 17392, 54336, 33536,
                63284, 14816, 23228, 28724, 47412, 13300, 6328,  60220, 23440,
                36440, 52864, 16244, 44544, 28964, 51156, 37944, 62644, 56008,
                10976, 51916, 5088,  33292, 48968, 9732,  25000, 33816, 57252,
                18504, 49576, 35316, 900,   20824, 54880, 41896, 10912, 1416,
                40232, 53056, 45724, 56844, 61568, 3788,  10536, 41396, 65440,
                3404,  23968, 44036, 60056, 2328,  22328, 49340, 34732, 64872,
                57952, 47984, 25012, 41700, 21472, 38084, 60168, 18544, 54532,
                63340, 21200, 27112, 48936, 23768, 14068, 38884, 25492, 30956,
                20940, 64672, 24472, 45080, 35792, 22340, 48520, 448,   28900,
                52956, 57608, 39628, 61688, 51756, 29792, 53948, 35216, 2916,
                49896, 41936,
            },
            {
                14844, 51020, 24500, 19064, 40556, 59228, 45576, 35980, 15200,
                3892,  59656, 64568, 27752, 1084,  57964, 4552,  59992, 47132,
                1756,  18852, 11800, 31948, 3588,  49672, 18128, 25888, 45104,
                15092, 9080,  57052, 24560, 45248, 8108,  1404,  42412, 24460,
                45900, 39660, 56156, 1312,  53384, 19936, 31500, 45860, 2196,
                57268, 12708, 41240, 55812, 10116, 18240, 4224,  21260, 48536,
                27576, 18596, 24172, 59348, 29636, 53888, 38712, 7240,  14264,
                61472, 39292, 27140, 58860, 17052, 51724, 27520, 32664, 23004,
                63212, 29632, 6944,  11936, 39528, 30296, 51668, 56140, 17868,
                12176, 50344, 6600,  35920, 29336, 57148, 12860, 26180, 2816,
                17708, 36712, 6372,  56276, 50668, 3104,  25844, 30028, 4748,
                35236, 46888, 5912,  58012, 33788, 53188, 45148, 7256,  57664,
                3560,  47252, 13188, 1824,  60840, 9516,  57972, 17660, 64700,
                8724,  21216, 2836,  26992, 47160, 4548,  16728, 45420, 64928,
                25520, 30856,
            },
            {
                61840, 43568, 33120, 64620, 2928,  6012,  20684, 63476, 9440,
                51244, 44068, 21836, 34036, 16408, 45228, 30016, 25400, 37072,
                53876, 34060, 57928, 39160, 15760, 8664,  30676, 61244, 58504,
                32056, 36656, 3236,  12488, 52388, 35344, 48516, 58988, 9908,
                3460,  19372, 59608, 42036, 34136, 25576, 64868, 11008, 28328,
                48604, 32668, 21020, 7156,  35696, 64808, 33052, 43672, 976,
                38560, 61748, 7168,  42992, 12864, 17712, 65148, 44504, 30148,
                8948,  53264, 12172, 31832, 7696,  45460, 13400, 49312, 56524,
                16580, 47376, 25796, 18828, 43632, 21768, 5544,  27076, 46032,
                38924, 58696, 26372, 18548, 52784, 46192, 5344,  41296, 29096,
                53312, 44588, 14156, 61376, 8652,  33836, 11532, 44520, 58888,
                16404, 39852, 12520, 62168, 17920, 28652, 10020, 50284, 40752,
                33220, 53368, 18644, 40028, 26908, 52260, 42344, 36912, 33004,
                44492, 15496, 35724, 42524, 10472, 58672, 38388, 22648, 11980,
                55712, 7564,
            },
            {
                35860, 1804,  11524, 29300, 16904, 52532, 33896, 28120, 40036,
                17960, 29776, 10672, 55200, 52100, 19592, 62160, 12748, 22100,
                43136, 6924,  26276, 60760, 46100, 36200, 52480, 1088,  11292,
                50804, 21316, 43824, 27204, 61672, 20052, 15668, 29384, 36900,
                51480, 31800, 11372, 16208, 50196, 7820,  55116, 40304, 17268,
                62668, 3724,  24888, 59576, 49592, 46328, 26352, 13104, 54232,
                31280, 14632, 56408, 35996, 47140, 27472, 444,   56532, 36520,
                24160, 45816, 3956,  41280, 64916, 35768, 3008,  61828, 5744,
                35136, 9364,  38052, 64520, 53376, 15612, 59660, 35020, 1808,
                30944, 15404, 41748, 64068, 10612, 38164, 16632, 50504, 60376,
                9696,  22704, 32344, 27496, 19688, 39236, 65444, 49168, 23284,
                560,   52288, 30624, 42592, 2468,  37208, 21952, 63408, 17068,
                11512, 28960, 62036, 48996, 31836, 14492, 3180,  23476, 5840,
                50832, 56516, 62544, 23172, 18784, 31464, 61272, 860,   33576,
                40192, 19724,
            },
            {
                58552, 22428, 54752, 37584, 48960, 42816, 12880, 55876, 1548,
                48336, 58336, 36676, 3248,  40868, 8488,  38588, 5288,  50924,
                63312, 10180, 49272, 20928, 5112,  56024, 23964, 19756, 41236,
                28364, 64820, 16780, 38956, 32996, 5560,  54860, 22452, 65124,
                6384,  26820, 62380, 37748, 3972,  45004, 22076, 35536, 5908,
                44132, 38328, 53860, 29584, 1680,  15692, 40152, 58464, 8656,
                50132, 41564, 21104, 3584,  52512, 22872, 6224,  50224, 20212,
                15780, 59752, 48796, 21772, 55868, 18880, 39372, 24992, 43952,
                20980, 50252, 55504, 616,   32048, 48584, 7768,  62760, 23348,
                55672, 8856,  48280, 372,   31656, 24672, 62348, 20560, 33652,
                39544, 1164,  54656, 48476, 43052, 15332, 53028, 7516,  28088,
                36644, 56960, 20172, 8644,  55680, 47144, 59612, 26700, 12,
                56600, 37728, 8244,  4444,  55780, 20468, 60160, 47396, 26216,
                11632, 29268, 1408,  48432, 54892, 6984,  49704, 14336, 28296,
                52648, 47664,
            },
            {
                16180, 27264, 45792, 60380, 8772,  26112, 61416, 22824, 7572,
                32520, 24040, 14140, 60748, 26188, 49132, 32592, 56512, 17732,
                30880, 40340, 14652, 29404, 42520, 12336, 62408, 38288, 4732,
                47176, 7640,  54296, 656,   46580, 58436, 11536, 41344, 44640,
                14016, 47828, 53832, 23676, 29816, 61348, 14388, 57896, 26568,
                51232, 13516, 18472, 42228, 10944, 22296, 61520, 4852,  24280,
                17556, 63588, 33952, 10680, 61128, 39388, 32160, 41744, 63244,
                2888,  34072, 26380, 14380, 29256, 9892,  53656, 31012, 15124,
                59880, 4472,  28484, 13676, 23764, 36932, 11208, 40996, 51136,
                20004, 34156, 60448, 21884, 45232, 54916, 7792,  3836,  56556,
                46976, 12900, 63332, 5524,  58196, 2304,  30828, 20764, 41928,
                14032, 63948, 25812, 49960, 32384, 4284,  13460, 35448, 42828,
                46344, 21348, 24612, 34788, 43964, 10336, 38256, 63796, 54456,
                18280, 40072, 33648, 13732, 36856, 25968, 41152, 57304, 44596,
                4920,  10032,
            },
            {
                64124, 6576,  40992, 14268, 20272, 4556,  36424, 46448, 51880,
                62776, 43300, 6128,  46156, 21232, 65356, 15084, 44532, 3484,
                23812, 53328, 96,    64896, 34276, 26616, 48708, 16184, 57488,
                33708, 24728, 60332, 10496, 23240, 50280, 27500, 2392,  33796,
                56904, 20812, 476,   42884, 9932,  18804, 47708, 1376,  31376,
                61008, 8064,  34384, 63352, 55360, 31840, 51688, 35804, 29280,
                47320, 2456,  26512, 44576, 16916, 55364, 13408, 9288,  28748,
                53540, 38992, 5588,  62876, 43428, 80,    46984, 58280, 11396,
                40620, 33564, 62192, 42196, 46552, 58740, 18200, 28952, 44720,
                12968, 3712,  53092, 39480, 13552, 36268, 28532, 43012, 15660,
                25312, 30012, 18580, 34540, 24696, 37428, 55636, 61596, 47268,
                10356, 5228,  45268, 15744, 23808, 40264, 53504, 20028, 6408,
                51840, 65196, 13056, 58240, 51092, 27804, 16376, 31272, 8092,
                43332, 58612, 5084,  65248, 45684, 9280,  20960, 63324, 17576,
                24660, 38264,
            },
            {
                48596, 30292, 244,   34244, 53776, 57916, 30928, 15260, 10256,
                19396, 38104, 54160, 29092, 11128, 2028,  35884, 27820, 60036,
                46568, 36768, 58488, 17908, 51540, 3664,  8860,  30252, 43024,
                13744, 51576, 18820, 40712, 31492, 15144, 37948, 61308, 18040,
                8400,  40052, 35704, 58220, 51332, 33472, 38888, 54152, 11700,
                20420, 48924, 24324, 45912, 5392,  38200, 19280, 44980, 11788,
                54828, 37884, 58580, 30460, 7592,  24984, 47340, 58784, 43644,
                18704, 11384, 57328, 51120, 34652, 23264, 37240, 7060,  26912,
                51096, 22560, 17524, 9848,  3332,  26532, 54096, 6284,  37788,
                64904, 25880, 29708, 5824,  18088, 47768, 58892, 10508, 65028,
                37144, 51936, 42236, 9456,  46304, 12288, 17580, 3920,  33716,
                51860, 30136, 38536, 57684, 62744, 10744, 29072, 61124, 15576,
                31140, 39356, 1864,  17784, 40704, 5312,  46256, 664,   35776,
                21032, 50116, 24692, 17216, 53344, 3856,  30444, 35444, 2112,
                54352, 32936,
            },
            {
                20752, 56700, 62164, 24392, 50000, 39400, 2612,  64796, 41272,
                26796, 524,   16520, 32008, 58600, 51852, 42156, 20176, 9404,
                12980, 32192, 7360,  22176, 37876, 45396, 55024, 22816, 63176,
                2344,  36348, 29060, 44864, 63840, 3580,  53004, 45864, 25032,
                30452, 63544, 16488, 5592,  25284, 64236, 4388,  23324, 43236,
                36404, 56400, 1984,  16380, 27744, 9328,  57640, 84,    65408,
                21456, 6084,  13856, 49000, 62576, 37424, 21000, 34592, 1032,
                25392, 46096, 31392, 7944,  17192, 61372, 49240, 19884, 64720,
                1520,  44936, 57392, 35932, 49588, 63612, 32888, 1056,  57336,
                16600, 49632, 42404, 56520, 63152, 2516,  21040, 31844, 49488,
                2008,  6964,  61796, 21384, 59304, 50640, 26856, 40292, 23188,
                60012, 19148, 1008,  34792, 7164,  43836, 49060, 33884, 55708,
                9824,  25740, 48172, 29556, 62556, 23100, 59424, 52524, 13604,
                61892, 28548, 10716, 32124, 39664, 59112, 50400, 11104, 42004,
                60916, 13120,
            },
            {
                8188,  42348, 16704, 11304, 44464, 18092, 28080, 21756, 45364,
                55992, 34820, 62012, 47624, 39796, 24516, 6660,  54780, 48668,
                63452, 25896, 43164, 56444, 11192, 61468, 32492, 18160, 49596,
                10888, 58764, 4824,  55476, 8708,  35844, 21184, 6628,  55744,
                49976, 11276, 46752, 28384, 13000, 45612, 17020, 59676, 28644,
                6516,  64908, 30596, 40396, 60480, 50716, 14060, 25444, 39532,
                32012, 52808, 42128, 18932, 1968,  51788, 5516,  63976, 16216,
                49840, 60492, 21288, 41812, 13152, 28124, 56028, 32748, 15896,
                38280, 52936, 6484,  30632, 20320, 14856, 42948, 21632, 47240,
                9480,  35948, 23120, 11176, 33744, 27108, 53408, 40360, 23592,
                17780, 55912, 28020, 39340, 88,    31240, 64680, 7876,  54756,
                14808, 42532, 27516, 53252, 21732, 17440, 1620,  23524, 5160,
                45176, 59492, 34420, 7708,  53988, 11604, 33152, 25832, 41052,
                47284, 2640,  55764, 44060, 22320, 14976, 27040, 46064, 19528,
                28824, 51584,
            },
            {
                46880, 26608, 37148, 5232,  32568, 8520,  59416, 52548, 6832,
                12512, 49748, 4424,  8840,  18876, 13612, 61764, 33556, 38672,
                17148, 2696,  50308, 29140, 15064, 1684,  41200, 6648,  26980,
                38420, 46720, 21944, 26240, 17396, 60156, 41184, 13512, 33204,
                2104,  22796, 60124, 54536, 41776, 31612, 9224,  51604, 39792,
                14904, 44708, 10776, 53492, 22140, 46980, 34428, 43604, 16760,
                60144, 9128,  23936, 57316, 32924, 28528, 11612, 53788, 38808,
                29284, 4172,  36916, 55012, 1972,  45240, 5732,  10528, 42824,
                29392, 13236, 24956, 60900, 40332, 7364,  52196, 25472, 60088,
                30900, 6884,  61864, 15644, 51260, 44152, 8180,  13740, 60420,
                44848, 32820, 12692, 54044, 16036, 44168, 35380, 11380, 45836,
                3272,  49756, 64200, 12424, 46488, 58636, 37188, 63444, 40600,
                18532, 50476, 21516, 14560, 37448, 44580, 19848, 3812,  56812,
                16668, 37488, 7028,  63668, 34568, 432,   62440, 56120, 5928,
                39392, 2844,
            },
            {
                22496, 65424, 55580, 51972, 61220, 47588, 13872, 37436, 33544,
                25328, 58152, 22192, 43800, 28336, 56756, 1000,  29696, 22564,
                5280,  57876, 39976, 19868, 47548, 24636, 35024, 57944, 52848,
                15656, 65352, 33328, 12580, 50680, 30280, 48176, 27336, 62808,
                43180, 38556, 34512, 3144,  21168, 57312, 35288, 728,   19648,
                47848, 24844, 33888, 18132, 2812,  7508,  29124, 62144, 4660,
                47740, 27788, 35840, 46380, 15388, 41304, 45360, 22660, 57092,
                13912, 9636,  65376, 24820, 51516, 39488, 22384, 58700, 47996,
                61884, 3516,  54972, 45932, 10952, 56848, 34900, 12468, 38620,
                19088, 54784, 46428, 688,   38160, 19584, 57548, 35888, 3212,
                48136, 8660,  36812, 24072, 49096, 6244,  20428, 57360, 29128,
                36880, 24364, 9180,  40148, 31324, 26104, 14044, 51988, 28236,
                11464, 3948,  64056, 42028, 57444, 6112,  49624, 64616, 9528,
                30448, 23616, 52660, 19080, 49288, 8696,  37132, 24228, 16824,
                59248, 34908,
            },
            {
                15228, 30960, 1240,  19448, 29004, 23836, 40976, 1844,  62336,
                17668, 30588, 38744, 64144, 35808, 50596, 46416, 14896, 52736,
                44624, 34432, 60920, 8140,  54024, 64216, 44296, 20352, 30692,
                360,   41664, 7336,  43524, 54276, 1288,  10464, 57540, 19524,
                15468, 9036,  50876, 18072, 7132,  48392, 62396, 26940, 54944,
                61252, 5184,  58040, 37108, 63008, 41908, 54868, 11260, 20100,
                51612, 1420,  12408, 64512, 3628,  59836, 26144, 6696,  32332,
                43192, 47952, 33904, 18100, 30552, 62968, 35164, 2680,  26268,
                16948, 34280, 21740, 37384, 17928, 28432, 1688,  64184, 43996,
                4464,  27352, 41044, 32236, 24752, 64444, 5060,  30372, 25540,
                62984, 20188, 58400, 3960,  62464, 41312, 25868, 51960, 16456,
                62112, 33088, 18676, 59580, 2452,  54864, 8748,  43072, 32628,
                56076, 36376, 27092, 812,   31152, 17168, 27728, 39552, 33840,
                45056, 59852, 11884, 41704, 29944, 13900, 54068, 42300, 32216,
                52868, 9548,
            },
            {
                49264, 38984, 12772, 43520, 35244, 4340,  57028, 20876, 48552,
                9868,  52788, 15184, 2408,  11636, 25104, 7784,  41076, 64420,
                10324, 26996, 13520, 31432, 37120, 4252,  12652, 8796,  48404,
                24316, 56348, 19052, 61600, 37812, 23444, 34892, 5104,  45480,
                53064, 31316, 26712, 65288, 40880, 24300, 14400, 37904, 10052,
                32328, 13452, 51040, 27492, 48496, 15532, 23364, 38348, 33388,
                58800, 40796, 22220, 54504, 38028, 8872,  50520, 17736, 62392,
                1660,  20676, 58124, 6940,  14692, 11240, 54776, 19296, 52008,
                41660, 9448,  50688, 5704,  62588, 47888, 23712, 50396, 16240,
                53164, 58632, 13404, 61044, 9796,  47428, 17244, 55212, 42716,
                15160, 39968, 51296, 29484, 33956, 13376, 59556, 1672,  39560,
                5176,  56000, 47964, 6504,  35492, 50040, 20576, 3408,  61956,
                22536, 16024, 52972, 47280, 24012, 60204, 52032, 13848, 21240,
                1004,  48556, 27372, 4840,  57820, 21128, 47156, 3256,  11764,
                61788, 45084,
            },
            {
                24980, 60248, 50936, 8960,  63420, 16020, 53332, 44408, 27532,
                5308,  41408, 46688, 54632, 32560, 60876, 21308, 36584, 18440,
                55652, 23780, 608,   51780, 17792, 27928, 58852, 39112, 62156,
                14180, 36116, 27716, 4356,  47472, 15256, 64544, 40128, 24544,
                60348, 112,   56620, 45196, 11152, 30196, 52612, 3568,  43940,
                22380, 41692, 1896,  20184, 9620,  31588, 4052,  56576, 44140,
                7356,  30228, 49404, 16588, 31200, 20804, 34688, 55632, 39876,
                27696, 53136, 38352, 44072, 49980, 26952, 46276, 7808,  37952,
                64256, 30428, 56128, 43092, 32616, 3888,  40816, 30260, 10148,
                33464, 22940, 3356,  20588, 50712, 28848, 38764, 11620, 52224,
                32272, 1012,  10920, 44568, 18628, 8236,  46968, 30876, 21056,
                44984, 13108, 23112, 42000, 28812, 65476, 16896, 37880, 48088,
                6056,  44732, 61076, 12636, 7992,  34980, 42500, 5244,  62852,
                56356, 35752, 15716, 61552, 39252, 33620, 63972, 26260, 19872,
                29180, 6412,
            },
            {
                17804, 4060,  21364, 58104, 26084, 38444, 11320, 32028, 65024,
                34972, 58532, 26420, 19512, 5728,  44240, 57968, 30700, 6272,
                48612, 38572, 46624, 63192, 41496, 22568, 50728, 33596, 3288,
                53552, 46312, 11360, 31876, 57276, 21004, 29572, 50092, 7660,
                37088, 13064, 21996, 35960, 4512,  59224, 17776, 46596, 64376,
                29164, 57248, 38760, 59948, 45000, 64504, 52396, 26576, 18180,
                14208, 63228, 25904, 5856,  44356, 61644, 12004, 46000, 4780,
                15768, 10692, 23928, 59932, 492,   40864, 61296, 33032, 12988,
                23884, 856,   15540, 25932, 11444, 57656, 20648, 60660, 6720,
                37444, 49180, 42964, 35652, 56476, 1812,  44736, 21332, 6632,
                57900, 27164, 65340, 23080, 56084, 49904, 37492, 63872, 10452,
                53152, 27476, 61356, 15384, 9680,  44904, 24852, 56776, 29520,
                10096, 39812, 32152, 20408, 38692, 55140, 10864, 46512, 26280,
                18324, 8100,  43368, 23976, 2124,  51984, 7548,  43980, 57376,
                40408, 36036,
            },
            {
                53748, 33192, 41400, 46180, 31088, 6592,  49156, 18780, 60,
                14080, 22048, 8620,  63072, 39412, 16608, 51152, 13004, 3124,
                60620, 32500, 15512, 6736,  56932, 10268, 45236, 16096, 29456,
                21560, 60040, 8148,  52184, 40968, 2148,  12140, 54808, 17548,
                28392, 42852, 49432, 16304, 54236, 39840, 33288, 20656, 7908,
                53068, 16132, 6520,  34480, 24376, 12888, 36780, 692,   59508,
                46476, 10416, 39680, 57420, 51920, 252,   24088, 29452, 49528,
                63824, 35636, 31540, 6036,  16876, 29840, 21968, 4296,  58816,
                43880, 49368, 60352, 19024, 36044, 52564, 13952, 45212, 55484,
                17656, 65204, 28068, 8000,  15932, 60196, 33108, 63684, 24248,
                37640, 46284, 16624, 35460, 5688,  27996, 15120, 24208, 41816,
                35104, 464,   49332, 38696, 57872, 32320, 2280,  51420, 14580,
                59132, 25476, 4088,  49720, 64884, 1652,  23092, 30068, 53392,
                38300, 50320, 31080, 54996, 10896, 28632, 17296, 13628, 50292,
                1064,  64408,
            },
            {
                47436, 28176, 13484, 1640,  55788, 23304, 61332, 40220, 54492,
                43456, 50552, 37320, 29144, 48020, 1588,  27384, 41992, 53088,
                21900, 28164, 43620, 20280, 35396, 1740,  25808, 64768, 6948,
                43680, 39572, 25660, 17884, 63260, 44056, 33428, 60984, 46404,
                4064,  63836, 32404, 9320,  61488, 27188, 972,   50416, 36444,
                11976, 26196, 48820, 55448, 17628, 8172,  42696, 50168, 32576,
                22848, 35140, 3172,  18832, 36560, 14744, 42428, 59564, 8344,
                22284, 57196, 46656, 51596, 64640, 36464, 56656, 15280, 52740,
                27884, 6380,  38600, 46060, 64592, 7832,  27328, 39308, 92,
                24944, 46468, 12200, 52860, 39636, 25256, 5520,  14108, 49072,
                9604,  54136, 2948,  42896, 61264, 52616, 3376,  58136, 7248,
                55592, 31640, 19156, 4616,  53672, 21700, 12236, 41412, 34904,
                19252, 54168, 43140, 28412, 17496, 58476, 33196, 15112, 61664,
                2488,  12804, 65308, 19656, 47656, 35560, 60536, 38184, 31016,
                23208, 11268,
            },
            {
                6892,  58704, 37524, 19168, 51496, 14984, 35716, 9716,  29460,
                24532, 60436, 4696,  11956, 56560, 34624, 23760, 65428, 36148,
                8004,  57744, 11456, 62208, 53960, 31020, 48540, 55208, 19368,
                34924, 848,   49564, 13196, 36608, 5972,  26692, 9008,  23068,
                39128, 57476, 19436, 25264, 6688,  47844, 14052, 62912, 23700,
                43040, 61188, 1300,  30316, 46884, 62024, 27848, 20388, 6260,
                61704, 55392, 48376, 27940, 65208, 32220, 53780, 37848, 18332,
                1792,  40716, 12252, 19412, 25688, 8612,  42056, 47316, 19968,
                35272, 10408, 22548, 29540, 2088,  49876, 34272, 22044, 62648,
                32264, 5148,  59156, 20076, 30408, 47212, 54560, 41532, 34708,
                19016, 60112, 31212, 13124, 40160, 19664, 33304, 46548, 17704,
                12384, 64564, 25440, 46924, 35588, 62632, 6376,  60068, 46740,
                8,     63232, 9188,  13536, 36000, 48256, 44260, 4656,  41600,
                22088, 34240, 45172, 6352,  41140, 56860, 21932, 3932,  45448,
                55532, 16200,
            },
            {
                25332, 62068, 8880,  43820, 64540, 27360, 5488,  57580, 45532,
                3020,  17592, 33096, 53432, 15344, 59596, 9964,  18984, 45972,
                14824, 49328, 38940, 4108,  23868, 40844, 14260, 38040, 11068,
                57428, 61476, 29848, 22556, 53928, 59028, 19684, 51648, 34852,
                12416, 1452,  51056, 44604, 35112, 41644, 58140, 29444, 4972,
                54604, 32696, 40656, 21844, 3996,  38024, 58044, 15208, 40972,
                29516, 12396, 43624, 7296,  21520, 10172, 3572,  26348, 44500,
                33156, 54816, 28312, 2984,  45344, 54384, 11816, 1828,  31908,
                63724, 54712, 41328, 14312, 56808, 17848, 43232, 11024, 51508,
                15608, 41476, 34988, 2380,  62448, 10352, 18032, 516,   61872,
                28896, 7304,  21940, 48420, 25596, 10688, 62476, 29436, 43248,
                37004, 50696, 8688,  14084, 28492, 43540, 18304, 26776, 31204,
                23424, 37996, 50660, 57400, 21620, 7212,  26904, 59772, 50884,
                9000,  25396, 58856, 15852, 212,   26060, 50960, 9376,  62532,
                33668, 42180,
            },
            {
                20148, 30384, 48904, 33808, 3208,  40736, 48104, 20932, 12840,
                38492, 64192, 46948, 21416, 40064, 44660, 5584,  32052, 55400,
                1868,  24928, 33628, 18024, 51980, 8252,  60952, 4624,  27624,
                51380, 15916, 45600, 4440,  31456, 47908, 15436, 41572, 56228,
                62196, 30436, 15088, 55188, 21752, 11052, 18552, 38324, 45744,
                9576,  19380, 13700, 65504, 52488, 11436, 33624, 54484, 2796,
                51428, 17272, 24164, 59944, 40744, 50608, 58592, 12668, 63052,
                49200, 15556, 62028, 38720, 34024, 60992, 23820, 39672, 50540,
                16644, 3464,  59664, 48232, 31676, 5876,  61500, 54460, 29132,
                8436,  57372, 49552, 23000, 43920, 37316, 57788, 26696, 50740,
                43848, 39152, 53260, 59120, 832,   36332, 49664, 22944, 2164,
                60644, 21188, 40484, 58960, 1308,  51104, 38320, 9556,  53480,
                6564,  16468, 32848, 2936,  40596, 54636, 11440, 19284, 36188,
                55620, 29396, 49396, 38780, 30760, 63808, 13076, 18696, 27816,
                5384,  51896,
            },
            {
                596,   54852, 12484, 22756, 17364, 53464, 30936, 59628, 34344,
                55136, 26748, 9204,  30584, 696,   25728, 50180, 60692, 40684,
                29500, 62820, 44372, 58224, 29048, 46744, 21764, 43280, 33312,
                24096, 8988,  40440, 64652, 10532, 38312, 1872,  28968, 5044,
                47328, 26096, 37420, 6172,  65012, 31820, 52332, 2200,  15996,
                60472, 50492, 36136, 7400,  26752, 44004, 23888, 8912,  46036,
                64112, 38680, 1024,  54320, 33960, 16220, 46936, 30484, 23848,
                5492,  9616,  21960, 50672, 4804,  14500, 29428, 58236, 7244,
                27000, 44792, 23988, 8740,  20340, 40140, 25088, 3032,  36408,
                45640, 18244, 26544, 13504, 51868, 4884,  32476, 21576, 11488,
                15156, 4184,  33560, 17112, 44104, 56480, 6464,  15340, 54192,
                27704, 4960,  45504, 31000, 23488, 55940, 15360, 64952, 48324,
                42768, 61852, 29100, 46232, 24668, 64324, 30916, 39200, 16424,
                63144, 2872,  20704, 10664, 53744, 33944, 42396, 47336, 39328,
                59172, 36532,
            },
            {
                45204, 39912, 57380, 6308,  60424, 25188, 10476, 948,   15700,
                49184, 6876,  41340, 52136, 63040, 17772, 37572, 12660, 20984,
                4768,  50952, 9220,  13340, 324,   36084, 16516, 56588, 63424,
                47832, 2264,  34696, 20288, 55696, 25036, 60128, 44876, 17936,
                21312, 9668,  42384, 58952, 3276,  48564, 24884, 56260, 34772,
                27656, 22952, 57740, 42140, 16564, 49164, 60308, 19404, 36180,
                26436, 31636, 9916,  45492, 28616, 4680,  19788, 39580, 56652,
                35872, 42212, 59300, 31220, 18372, 43292, 48748, 21220, 37028,
                61360, 12572, 34140, 52600, 37484, 58416, 47180, 13284, 21204,
                60524, 39516, 6780,  63964, 28956, 16568, 40616, 63224, 45984,
                55980, 24132, 64844, 27776, 9112,  30952, 63776, 41652, 11656,
                34508, 57120, 16368, 63656, 11308, 33236, 3644,  22092, 35620,
                12584, 19808, 59004, 10100, 15324, 52024, 628,   48008, 6068,
                45556, 13580, 42800, 61436, 23372, 7036,  56296, 2828,  22820,
                14920, 10876,
            },
            {
                24856, 16736, 28940, 35764, 43384, 46828, 37648, 62948, 42636,
                23816, 19216, 58920, 13692, 35328, 56396, 8160,  27784, 54080,
                16340, 36800, 22860, 41672, 64976, 53440, 31564, 3296,  12720,
                18908, 54744, 28012, 43096, 50448, 12544, 7520,  33980, 64332,
                53976, 32580, 50340, 17108, 28544, 39668, 8736,  63272, 43288,
                6160,  47488, 3668,  31624, 55500, 604,   30380, 5212,  12952,
                56300, 21048, 61040, 14236, 37060, 63728, 8376,  52752, 564,
                17584, 47124, 26792, 7192,  56372, 64900, 1016,  10064, 53460,
                18616, 40860, 28648, 65296, 684,   16172, 27768, 63000, 50756,
                30748, 968,   33760, 54440, 46244, 9088,  52960, 2432,  34608,
                7972,  37496, 47244, 12768, 51016, 21064, 38636, 26132, 47796,
                51696, 19480, 37556, 6324,  49556, 39880, 45716, 58072, 27912,
                1176,  55232, 38460, 4324,  43756, 34316, 21044, 60208, 27832,
                32832, 53124, 35824, 4648,  46332, 17016, 28436, 60888, 49780,
                31748, 65200,
            },
            {
                47816, 4164,  52096, 63868, 14320, 8168,  19768, 27732, 51588,
                32692, 2424,  45108, 28272, 22680, 3072,  46872, 42648, 64232,
                48224, 31144, 59480, 27096, 20244, 10844, 49296, 25624, 41576,
                37412, 59536, 5844,  16776, 62304, 36352, 22096, 48900, 13820,
                39336, 908,   61196, 11888, 22696, 46216, 14656, 20520, 30704,
                12464, 39452, 63884, 10588, 21108, 37780, 62848, 40776, 46940,
                50908, 3992,  42336, 48800, 55288, 26732, 22184, 43868, 32756,
                64272, 13428, 53612, 37980, 11740, 35440, 26244, 32656, 46920,
                3092,  56152, 6060,  49140, 11368, 33024, 41732, 4704,  9728,
                43572, 56044, 12128, 24036, 36504, 20196, 58564, 30124, 25664,
                59972, 19172, 1576,  40936, 55452, 4788,  17368, 60356, 8544,
                31984, 2528,  43852, 25640, 54764, 29212, 18752, 7272,  41540,
                50940, 33876, 23096, 49176, 27004, 57076, 12280, 41864, 8580,
                23036, 18080, 58440, 25920, 50816, 38316, 9056,  35376, 19572,
                54380, 7600,
            },
            {
                34340, 58660, 21208, 1584,  32400, 49772, 56592, 4908,  12304,
                55548, 64788, 38048, 10648, 49620, 61900, 32640, 24136, 11100,
                1160,  7372,  52568, 46012, 4964,  38824, 60476, 7024,  29704,
                51592, 10008, 45180, 31628, 184,   27276, 40820, 56056, 30052,
                5936,  25160, 44348, 36476, 54956, 33456, 58656, 220,   51268,
                54688, 17788, 25556, 34328, 52088, 13968, 27280, 57504, 16952,
                32928, 7884,  24964, 18648, 2540,  12132, 51036, 60008, 10424,
                24764, 29844, 2176,  20236, 45216, 51264, 16196, 42436, 63092,
                30272, 15272, 22852, 44216, 19504, 54064, 57576, 23168, 35692,
                15440, 19328, 49028, 60260, 3964,  43076, 12992, 49444, 16188,
                42388, 53596, 31880, 23140, 59288, 35800, 45760, 988,   23740,
                63004, 13608, 59520, 21268, 8944,  62580, 13908, 60700, 24800,
                10596, 16744, 59768, 6524,  63472, 17568, 37252, 54720, 49752,
                64584, 39884, 964,   12096, 31992, 64128, 14028, 44784, 1552,
                26964, 42304,
            },
            {
                17980, 9944,  38480, 27228, 41380, 23108, 62008, 35180, 40228,
                17540, 6020,  31128, 53352, 16124, 4900,  39472, 18772, 55720,
                35204, 40320, 17864, 14424, 32596, 55076, 44304, 17360, 62608,
                21192, 33932, 23612, 14124, 53044, 58388, 3928,  10276, 17620,
                51324, 57904, 20224, 7068,  62856, 4352,  41668, 26828, 36380,
                59744, 45168, 2532,  61180, 48168, 6776,  44632, 9800,  22336,
                65112, 39408, 58492, 34984, 62568, 41428, 28888, 36708, 4148,
                57512, 49880, 41632, 62680, 24044, 3988,  58752, 22052, 7752,
                37892, 50136, 60908, 35480, 26224, 7836,  45752, 29508, 52836,
                64180, 25964, 40020, 8020,  32132, 65464, 22920, 38976, 6076,
                10892, 62832, 50152, 14960, 28300, 11256, 33068, 53568, 43196,
                28092, 40400, 52812, 48260, 34704, 2744,  37276, 47784, 32204,
                53904, 43336, 29472, 40448, 9340,  31808, 1732,  25224, 4976,
                14996, 46728, 28936, 55240, 41828, 21444, 57784, 51904, 39964,
                12432, 61924,
            },
            {
                30364, 50336, 44636, 12792, 53972, 16388, 2920,  29308, 44508,
                48312, 25524, 60752, 21356, 43036, 57752, 28800, 13632, 60336,
                45096, 25536, 63100, 57632, 1628,  35524, 23860, 13588, 46868,
                1800,  55444, 65172, 48412, 38064, 19496, 46600, 43244, 63620,
                35820, 27952, 14188, 47504, 29724, 18224, 49196, 11200, 7528,
                22612, 15072, 29188, 38436, 18528, 24060, 53924, 35688, 3228,
                28416, 52840, 15212, 30772, 46576, 6624,  17096, 48072, 20868,
                39296, 15628, 34188, 9264,  55944, 29116, 39776, 13132, 54936,
                25580, 10244, 58156, 2404,  39464, 63516, 17356, 1440,  38168,
                6004,  47312, 57984, 28612, 16996, 51532, 208,   56896, 46540,
                28796, 36032, 4232,  8232,  64472, 48600, 20048, 57068, 15492,
                36172, 10552, 3984,  30836, 15964, 42960, 52400, 21760, 4828,
                64496, 1908,  14160, 20584, 47192, 53216, 44452, 61952, 29928,
                34840, 9780,  61100, 18692, 6276,  3036,  24664, 29960, 6612,
                56672, 23256,
            },
            {
                4896,  55560, 63032, 6044,  60084, 46292, 10772, 58824, 20696,
                8632,  14076, 34760, 236,   46572, 9512,  36392, 52372, 5892,
                21808, 29916, 8312,  47860, 21304, 27828, 8808,  58112, 39892,
                5616,  26984, 41156, 7644,  11624, 24880, 61140, 32456, 23160,
                2096,  42188, 53020, 10108, 40288, 24456, 54076, 61592, 31916,
                42276, 50356, 9040,  58096, 42908, 1608,  30120, 60280, 48740,
                12516, 44076, 120,   10216, 23572, 56324, 33048, 12940, 54424,
                7284,  60344, 46400, 18720, 32296, 6424,  61164, 45476, 312,
                33644, 18956, 42916, 14356, 31396, 48164, 11828, 60092, 21380,
                13784, 32824, 3000,  11416, 44920, 37664, 26876, 33476, 60932,
                17700, 21548, 45288, 25700, 38268, 41956, 3488,  7380,  22292,
                46992, 58160, 18572, 65480, 26356, 56252, 11508, 28344, 18296,
                45316, 37588, 56116, 60832, 35120, 23448, 13592, 19276, 39072,
                57484, 22064, 43840, 50428, 37196, 47300, 62492, 34532, 16336,
                46184, 37272,
            },
            {
                14772, 33548, 19840, 24336, 36512, 31740, 26076, 52916, 38708,
                63692, 51160, 56716, 40504, 26620, 19804, 65224, 48396, 2656,
                34096, 51484, 12312, 36876, 42684, 64524, 52756, 30200, 36564,
                50456, 15252, 18764, 59800, 35300, 29552, 5412,  15624, 49792,
                8228,  59464, 33076, 64624, 1248,  37172, 15604, 45488, 19204,
                65060, 4740,  55252, 12360, 32660, 64164, 15348, 40688, 19092,
                37832, 57112, 21068, 51172, 61168, 38488, 2340,  64956, 43776,
                23040, 27680, 3428,  53420, 14252, 49088, 35128, 20624, 52188,
                64772, 47492, 29020, 4952,  54684, 24112, 34588, 50528, 43888,
                56784, 41584, 23500, 62344, 54252, 20708, 8848,  14384, 52924,
                2632,  40732, 58024, 54140, 16940, 60708, 30716, 50852, 63512,
                32644, 5724,  38776, 23212, 45856, 256,   40172, 58624, 34860,
                8204,  25560, 50104, 5268,  28136, 2944,  48656, 55808, 7840,
                51572, 2676,  26000, 33100, 15448, 54144, 10964, 19944, 59904,
                49492, 456,
            },
            {
                42644, 28056, 9228,  40472, 1132,  49160, 18364, 6548,  32908,
                2032,  22996, 29172, 12240, 59872, 7008,  31476, 24464, 41528,
                16912, 61636, 55272, 5152,  18520, 49208, 3436,  11864, 20264,
                61604, 32728, 43684, 52152, 1600,  56452, 39520, 54000, 12644,
                37600, 26004, 17032, 21804, 50188, 56704, 6588,  34488, 2652,
                24020, 39740, 35188, 26220, 20688, 49860, 8460,  55488, 25048,
                5644,  62664, 27184, 35540, 42448, 18788, 26040, 49308, 30064,
                35852, 11820, 63568, 38868, 25308, 43588, 10768, 27304, 16060,
                39192, 7220,  51284, 21688, 61608, 41276, 3348,  26832, 9640,
                30000, 52620, 18116, 34412, 48320, 5316,  63300, 43608, 24484,
                48052, 30088, 10140, 32836, 572,   12844, 24768, 37576, 9760,
                26852, 54292, 49992, 12716, 33804, 61784, 6724,  50408, 14488,
                62784, 31304, 11680, 17352, 39688, 64876, 10604, 32312, 42516,
                16856, 59896, 11944, 65484, 7344,  40580, 27708, 43636, 8604,
                26072, 64676,
            },
            {
                53444, 58820, 45628, 51636, 13876, 65168, 41836, 56304, 15424,
                43932, 36816, 4756,  47780, 17132, 53528, 38592, 14416, 59324,
                9672,  44052, 26552, 39124, 32212, 15296, 60136, 44556, 25268,
                55776, 9492,  4784,  27876, 47900, 21616, 44368, 62684, 19384,
                47012, 56096, 4188,  44804, 31044, 12084, 27564, 59188, 47700,
                29608, 52040, 16652, 46520, 59512, 4384,  45120, 34160, 11112,
                31416, 47016, 16536, 8348,  3720,  53100, 14704, 9044,  57064,
                1116,  41988, 51128, 21148, 57920, 2052,  62536, 54260, 3500,
                31100, 12392, 57700, 37112, 8832,  15412, 57308, 19188, 64936,
                6664,  37064, 980,   59432, 15236, 30948, 39716, 55664, 11712,
                37024, 5048,  61400, 20568, 51472, 46628, 58428, 18264, 44468,
                14760, 1476,  42328, 59552, 8640,  20404, 30616, 23912, 54648,
                41568, 46840, 22240, 53768, 43872, 57988, 36356, 24952, 4276,
                28448, 48068, 36732, 30480, 22928, 58716, 1956,  52072, 32140,
                39044, 21404,
            },
            {
                35452, 2608,  16872, 29652, 54968, 22348, 9896,  28284, 60180,
                50120, 19224, 54948, 63328, 33244, 44412, 1428,  56832, 28260,
                50248, 20508, 872,   62904, 23784, 8032,  40624, 34692, 576,
                47224, 23060, 38736, 64264, 12160, 17076, 8940,  31340, 3224,
                28760, 41192, 62076, 7916,  38964, 61160, 20884, 41520, 13796,
                7320,  57172, 10684, 740,   37632, 28752, 22748, 63216, 51720,
                1424,  41588, 49956, 29972, 58936, 45032, 32428, 60944, 19740,
                46836, 16504, 33128, 5608,  30608, 17740, 36896, 41696, 59372,
                22616, 43500, 26524, 18204, 45636, 52440, 32448, 39156, 48368,
                14568, 45880, 25684, 9976,  42748, 22716, 2144,  28152, 19008,
                64856, 49708, 14220, 43360, 35584, 28664, 5612,  40708, 55616,
                61248, 21584, 29220, 36852, 53348, 15564, 48340, 38400, 3840,
                19464, 996,   59048, 33224, 7292,  14968, 21120, 52232, 63256,
                38768, 20448, 188,   55128, 45820, 18604, 35616, 14520, 56004,
                6388,  12124,
            },
            {
                47020, 24700, 62416, 6908,  38036, 33248, 4392,  47228, 24784,
                11520, 31328, 7984,  41640, 23880, 10968, 21512, 46860, 6268,
                36544, 30728, 13280, 46336, 51044, 57564, 27380, 53844, 17388,
                62540, 13268, 30432, 57528, 34800, 26052, 59092, 36400, 51308,
                23604, 11552, 34620, 15100, 25064, 51912, 1976,  54500, 36928,
                63164, 22208, 43080, 61344, 53468, 13684, 42132, 17224, 36632,
                58500, 14040, 23352, 64480, 11996, 21376, 40344, 4936,  37328,
                25500, 54796, 60100, 10348, 45968, 50124, 24184, 8776,  33384,
                47184, 63936, 4488,  35340, 62908, 556,   28440, 5436,  22964,
                54992, 61564, 29160, 50944, 56572, 35660, 60364, 51440, 7172,
                33336, 22464, 26596, 56016, 7960,  64352, 23200, 2644,  31404,
                35132, 48776, 17336, 4260,  25788, 44664, 64660, 11868, 28072,
                60976, 9872,  37324, 27060, 50632, 2020,  30572, 44900, 12908,
                56212, 9200,  43104, 13748, 50792, 4928,  63792, 47636, 24124,
                61968, 18448,
            },
            {
                40156, 32276, 11016, 56880, 19456, 44824, 61352, 35864, 2512,
                39380, 61948, 15000, 27412, 4116,  52104, 34716, 64116, 17676,
                40576, 60588, 55936, 34360, 4144,  19844, 11164, 30816, 6420,
                41608, 50304, 18924, 3912,  41012, 53324, 368,   42724, 65508,
                6544,  53792, 48764, 57908, 18588, 46352, 32060, 9168,  17552,
                49016, 26920, 31064, 18892, 33368, 8016,  56216, 6496,  27584,
                19824, 54496, 39568, 6796,  34628, 55236, 27748, 51432, 63736,
                13436, 7188,  38568, 27856, 65516, 14072, 57204, 5116,  19548,
                14780, 53912, 10724, 49268, 23468, 12944, 44120, 60324, 34720,
                11000, 18424, 38912, 7476,  20036, 12324, 46584, 15840, 44416,
                58352, 41320, 1716,  17468, 38852, 11120, 52272, 47764, 13368,
                7348,  64012, 10568, 55924, 40644, 31988, 6092,  57772, 35072,
                43652, 49580, 16088, 64060, 23668, 40516, 60880, 18144, 5676,
                49740, 32552, 62512, 24780, 39004, 28908, 10728, 41516, 30844,
                3312,  51212,
            },
            {
                59648, 4236,  48780, 42148, 26528, 13048, 53292, 17640, 57612,
                21592, 51804, 45720, 56980, 38088, 61036, 14036, 26296, 54124,
                2880,  23288, 8884,  16628, 42568, 65404, 37976, 45784, 58808,
                35612, 25676, 54984, 44780, 8012,  22588, 49656, 12808, 16324,
                20856, 30712, 1108,  27364, 42928, 5076,  64372, 24488, 33968,
                44136, 6208,  3384,  39164, 50720, 25004, 47652, 60272, 3736,
                45624, 32532, 2616,  25272, 46376, 16888, 488,   10492, 22780,
                31336, 49344, 42988, 22288, 1260,  34296, 44344, 29544, 51932,
                38056, 2204,  27808, 31936, 39996, 58684, 20084, 50636, 42104,
                3752,  32136, 44628, 1748,  62640, 31420, 26148, 3368,  38116,
                9612,  54408, 31024, 62032, 45568, 33612, 59184, 19376, 26300,
                38364, 45188, 23852, 50916, 60572, 852,   22256, 17528, 52644,
                24304, 30308, 5508,  46424, 11616, 54284, 8600,  57260, 34552,
                22224, 27552, 17236, 59764, 7952,  33676, 21692, 60728, 16524,
                44248, 27604,
            },
            {
                13944, 20644, 36156, 64028, 164,   50348, 8300,  29064, 42572,
                6088,  33592, 644,   20068, 29580, 48696, 8316,  42308, 31604,
                12144, 45292, 52436, 29504, 25148, 49336, 1820,  22672, 15432,
                9276,  2352,  63680, 32636, 15600, 61628, 27692, 33524, 39140,
                60120, 45192, 36132, 62572, 10344, 38148, 14640, 58908, 51408,
                12368, 61876, 56508, 15488, 65512, 10928, 30004, 38100, 22160,
                63904, 10260, 51164, 61756, 37476, 57416, 48616, 41128, 59200,
                35304, 3576,  18368, 56188, 51524, 39992, 10940, 62172, 25112,
                60188, 42016, 56536, 16392, 6512,  53052, 9416,  26480, 15632,
                63268, 53176, 58228, 24236, 48460, 40056, 53848, 63608, 18392,
                28944, 12936, 47604, 4004,  21504, 14668, 28356, 41628, 53816,
                57996, 3196,  30072, 19616, 14000, 36052, 47896, 41480, 8036,
                13548, 57016, 39208, 20096, 33780, 43380, 28980, 37740, 46960,
                2784,  41708, 51996, 3868,  45384, 56388, 52724, 1328,  37864,
                9368,  55224,
            },
            {
                7100,  52576, 29808, 15812, 23656, 39616, 32036, 63496, 14348,
                48548, 26340, 65116, 10804, 17056, 43948, 1988,  57520, 20284,
                49516, 63156, 37104, 6168,  56956, 13904, 32868, 61048, 52604,
                47972, 29176, 21564, 11048, 38392, 46080, 5440,  57368, 48000,
                24720, 7672,  13420, 54792, 20292, 50024, 29940, 40904, 180,
                19916, 36276, 47152, 41856, 21224, 1072,  43816, 52828, 33920,
                15832, 41848, 28696, 13640, 8164,  20556, 29820, 14596, 44484,
                53752, 9104,  62924, 15456, 26164, 4612,  20972, 48388, 17008,
                8136,  33972, 21344, 62456, 46452, 36012, 30892, 1392,  38204,
                21696, 27980, 13456, 36024, 16960, 10036, 5028,  34388, 23752,
                52168, 59560, 35956, 25356, 50820, 6356,  63160, 56,    9356,
                16400, 34056, 61848, 39424, 7136,  27396, 54540, 63084, 32960,
                1596,  61464, 51772, 4000,  62772, 14592, 836,   24528, 65056,
                15416, 58252, 11640, 35564, 20064, 15240, 26588, 49324, 58132,
                34736, 23380,
            },
            {
                38892, 44772, 60460, 9092,  46768, 58836, 3508,  22596, 54476,
                36944, 8624,  41228, 55044, 36300, 59408, 24792, 33704, 39184,
                4592,  27244, 18760, 41756, 62224, 10028, 40076, 27972, 18736,
                42832, 36728, 59864, 50088, 54280, 30640, 19652, 10412, 2276,
                55628, 18176, 41292, 32548, 2628,  22788, 56976, 7936,  27560,
                53724, 23776, 8744,  28444, 34596, 58712, 18028, 12640, 7484,
                55384, 48152, 19164, 59572, 43740, 33640, 65080, 6204,  25656,
                19604, 28928, 46356, 37232, 59860, 30344, 55100, 35964, 72,
                45832, 12256, 50044, 2732,  24712, 13572, 65324, 56380, 45688,
                49120, 8732,  4208,  42672, 56060, 21264, 60568, 49724, 42080,
                1092,  20136, 8440,  39768, 55216, 43308, 36784, 31580, 48488,
                21796, 42624, 11760, 49408, 57088, 44988, 10212, 19004, 26128,
                44160, 16664, 36284, 22520, 27240, 58420, 49232, 19492, 9960,
                54016, 38580, 30472, 48424, 64348, 43188, 5448,  12088, 19716,
                30164, 65260,
            },
            {
                17508, 2292,  26928, 34100, 55920, 19140, 35664, 11148, 45588,
                17940, 60464, 23460, 31052, 50236, 5712,  15392, 64440, 10136,
                55256, 14528, 34740, 660,   21976, 46604, 55088, 3064,  5720,
                56352, 13148, 6792,  17836, 1560,  25192, 64464, 36960, 43328,
                29376, 63672, 51624, 26308, 61260, 47560, 34192, 16860, 44840,
                60580, 32368, 14056, 54920, 5796,  50360, 24384, 62600, 39972,
                26468, 5188,  31896, 1632,  52500, 23728, 3420,  50656, 38396,
                61536, 1736,  33408, 11852, 41852, 7668,  14368, 64076, 40312,
                27052, 59076, 30320, 43392, 54600, 19484, 41460, 5620,  17748,
                61088, 34292, 52028, 64604, 26736, 46072, 29984, 7236,  15048,
                45372, 65280, 32104, 16464, 61460, 10696, 18748, 56892, 25776,
                65020, 52888, 5580,  24176, 15804, 30868, 4584,  38008, 56292,
                49124, 29360, 10540, 47084, 7456,  40328, 52892, 32436, 43864,
                26368, 7288,  22424, 1052,  28264, 39720, 32528, 61172, 45164,
                4532,  48608,
            },
            {
                11412, 54900, 42012, 13424, 5328,  51312, 43100, 62500, 28308,
                1516,  51000, 13680, 3252,  20712, 46528, 28592, 51728, 22744,
                44080, 60940, 47712, 53864, 31120, 17256, 35148, 25372, 64632,
                39064, 23144, 33364, 43792, 58188, 40532, 14584, 52824, 22532,
                12212, 4196,  38372, 16012, 5880,  42512, 12052, 64864, 4944,
                38520, 48736, 2544,  63716, 37604, 46324, 30996, 3396,  45332,
                60636, 22476, 57040, 36416, 11244, 40032, 55824, 17476, 10240,
                56988, 48372, 23832, 53076, 20116, 45376, 50368, 32196, 22324,
                52412, 18684, 4396,  38552, 10436, 28248, 51504, 32876, 23952,
                12040, 29860, 39144, 15356, 304,   33376, 11952, 58372, 37360,
                27772, 56192, 4580,  47980, 29304, 23568, 3672,  45408, 13028,
                37372, 2348,  28684, 59164, 35144, 63828, 50604, 23084, 12480,
                2756,  41116, 54116, 60516, 31072, 17684, 12624, 4996,  36216,
                59720, 50260, 62636, 13860, 57140, 8704,  23944, 53608, 14796,
                41056, 25376,
            },
            {
                31744, 49952, 21856, 62100, 30220, 25696, 15504, 7124,  33044,
                56660, 38540, 44472, 34424, 62708, 11992, 41740, 36640, 1532,
                30420, 8052,  24284, 11340, 59968, 6880,  50896, 44600, 11004,
                29920, 47632, 63048, 27324, 9344,  49052, 3320,  32608, 59700,
                49824, 34976, 46432, 9508,  54084, 29120, 36452, 50932, 21896,
                10196, 18656, 43152, 26540, 16548, 10732, 56668, 19624, 34812,
                14396, 38584, 49616, 16352, 27844, 60508, 46612, 30648, 35940,
                42408, 14496, 5920,  64804, 2524,  57624, 25348, 4044,  9816,
                56244, 36280, 63592, 15800, 60672, 7324,  58340, 37100, 2520,
                43916, 58980, 20536, 6124,  48816, 41212, 18216, 51956, 24876,
                10380, 40856, 22104, 13208, 53132, 35248, 60048, 50412, 7268,
                32688, 18152, 47452, 43744, 20604, 1136,  40524, 59428, 33900,
                64784, 19920, 24936, 3592,  35080, 45024, 63952, 56088, 20840,
                2868,  17100, 41452, 34376, 52144, 17632, 47476, 36204, 408,
                57536, 62880,
            },
            {
                20024, 5992,  37300, 46204, 1144,  40652, 54188, 47728, 24032,
                19640, 5852,  58384, 26788, 52572, 7788,  59740, 18608, 56696,
                16116, 39848, 51492, 28116, 42764, 38428, 14628, 58972, 20240,
                52204, 232,   15480, 53532, 20944, 35308, 45808, 17516, 8056,
                26840, 20044, 58036, 24128, 60148, 18936, 1344,  25528, 56368,
                30424, 58964, 51820, 22276, 60968, 41404, 7692,  28240, 53652,
                248,   9840,  63492, 6452,  44164, 20852, 13240, 7436,  26872,
                21468, 59388, 31912, 39736, 28460, 35232, 17724, 61328, 44176,
                13708, 6368,  47772, 25752, 32180, 44460, 21680, 14392, 48004,
                54844, 7900,  25712, 61732, 55492, 22668, 63844, 34936, 2420,
                61104, 49392, 33028, 44456, 896,   38876, 27368, 15320, 41308,
                60956, 55376, 38812, 7904,  13260, 53516, 27124, 8528,  15444,
                46040, 6500,  57880, 14440, 50480, 22068, 9084,  38968, 29224,
                46620, 31904, 10644, 45140, 25876, 5800,  62368, 20784, 29008,
                8368,  34952,
            },
            {
                40356, 59916, 10060, 56360, 18036, 64996, 36196, 9700,  61200,
                42208, 29956, 10284, 16844, 39872, 23788, 32248, 5040,  45740,
                62460, 34176, 3168,  19124, 64280, 4220,  23008, 32700, 7920,
                61720, 35924, 5216,  39500, 60816, 12092, 28656, 62480, 54692,
                41384, 392,   30800, 43216, 13068, 39496, 62364, 45308, 14612,
                40692, 4460,  34348, 12728, 1768,  32880, 49796, 64776, 42460,
                46832, 30328, 25084, 54208, 32864, 2412,  64252, 50216, 54680,
                1096,  45036, 17156, 51684, 7980,  12064, 38612, 48632, 28988,
                41248, 33792, 20516, 51784, 40088, 544,   49852, 63448, 27024,
                40388, 17064, 51112, 36672, 31228, 8844,  44564, 5664,  54848,
                19828, 15264, 7396,  64092, 57652, 9484,  20228, 54568, 29872,
                22472, 10340, 25548, 62436, 32340, 48568, 17968, 43324, 30592,
                52912, 36616, 28688, 39764, 62244, 136,   26800, 48880, 14048,
                61680, 24372, 55716, 1980,  59344, 38916, 11856, 43000, 55508,
                51048, 16040,
            },
            {
                1884,  52668, 28836, 24344, 33520, 12316, 21624, 2720,  49784,
                14096, 53628, 63804, 46988, 428,   55356, 49628, 26600, 12420,
                21792, 48692, 57452, 13564, 36256, 55472, 49548, 27044, 40660,
                17616, 43492, 24756, 31512, 7144,  56132, 23804, 4904,  38144,
                14916, 65232, 48752, 6872,  33964, 51744, 3768,  32100, 48400,
                7976,  63348, 27592, 45952, 53900, 37220, 23632, 11556, 16976,
                21072, 58924, 40376, 18576, 48040, 37160, 23864, 41232, 34028,
                62628, 9544,  25948, 56504, 42876, 63172, 21100, 904,   53448,
                24052, 58076, 3476,  12008, 55864, 17456, 9776,  31176, 5340,
                34564, 11300, 45756, 2360,  13396, 27900, 16428, 39732, 47104,
                29156, 37984, 51192, 26312, 17000, 42872, 62360, 5264,  45996,
                1952,  51316, 16328, 57032, 5220,  36892, 58212, 3744,  61056,
                20948, 49428, 11296, 18300, 55084, 42564, 33740, 58884, 3828,
                51580, 7816,  37084, 21556, 29904, 48964, 33332, 3184,  23360,
                45772, 26632,
            },
            {
                64204, 43224, 14800, 49004, 7404,  44700, 59528, 31428, 38152,
                25464, 3540,  33656, 22016, 36968, 14272, 64588, 43264, 38412,
                54376, 6996,  26056, 44868, 30528, 9900,  45980, 2184,  58148,
                11960, 47304, 57004, 49888, 18872, 42136, 50980, 44676, 31816,
                11080, 52656, 17856, 22692, 56608, 27288, 10676, 20488, 54908,
                23800, 17220, 38908, 58284, 19392, 6136,  59884, 27156, 5012,
                51828, 35568, 3468,  61268, 10800, 15316, 58200, 4608,  18868,
                12828, 48688, 36336, 22792, 15016, 50296, 33012, 59844, 15724,
                9052,  64176, 45536, 28472, 61928, 35848, 24772, 59360, 19236,
                56240, 64888, 22272, 41688, 60704, 48252, 57840, 32388, 12356,
                23396, 57164, 4008,  34824, 22372, 31188, 48216, 12344, 33616,
                64268, 36128, 42548, 28556, 21320, 45320, 23892, 10164, 39168,
                25612, 1444,  44232, 7964,  30784, 23684, 11732, 16908, 44100,
                19416, 40648, 63752, 15052, 53472, 18600, 64932, 14284, 58812,
                37180, 11060,
            },
            {
                18380, 31944, 3816,  58364, 36704, 52172, 27056, 16880, 56264,
                45388, 19044, 41360, 60388, 28932, 8836,  20040, 2172,  29488,
                10376, 33056, 63416, 528,   21060, 60732, 16604, 34956, 52944,
                29136, 22012, 3132,  62952, 14280, 36680, 1284,  21212, 58736,
                26200, 36096, 4040,  45644, 63380, 15796, 37592, 59828, 43700,
                35816, 1228,  50372, 9024,  30664, 15520, 47644, 39256, 32000,
                56760, 13616, 8072,  29236, 51196, 42248, 27440, 31616, 53544,
                38848, 28948, 60488, 6444,  3344,  45596, 26960, 7032,  36948,
                30820, 18292, 38728, 22992, 7628,  53144, 46212, 38376, 43276,
                1352,  28776, 32828, 54072, 19532, 3824,  24396, 62296, 1200,
                43592, 63368, 10656, 41772, 54924, 2964,  51928, 24436, 40196,
                19432, 6888,  48992, 12268, 268,   65188, 54672, 33480, 13864,
                63148, 56720, 34560, 60368, 51888, 4688,  65344, 35556, 53916,
                27840, 32416, 11432, 43488, 4136,  27424, 8980,  41676, 30204,
                4916,  47752,
            },
            {
                53884, 22416, 39376, 63388, 19356, 352,   40536, 5812,  63136,
                8492,  50796, 12208, 4544,  56972, 47760, 34584, 52300, 61616,
                18228, 41884, 15044, 37624, 53476, 41144, 24708, 5096,  64760,
                38516, 8500,  33940, 26580, 10388, 29868, 63876, 16692, 8700,
                47744, 61064, 39904, 29712, 8524,  41680, 49608, 5180,  28732,
                11392, 56744, 42152, 25644, 64188, 43752, 55792, 1112,  61912,
                42980, 24744, 45748, 21596, 55736, 840,   62124, 9148,  45304,
                6808,  20504, 41172, 52348, 31040, 57948, 11468, 41440, 47556,
                55340, 50752, 1380,  42552, 15352, 29676, 4812,  12568, 52228,
                15808, 49056, 10004, 6812,  37808, 51564, 34440, 9460,  53676,
                18136, 30848, 48444, 14524, 60232, 8156,  35984, 58524, 15020,
                27320, 55872, 60392, 31484, 40764, 15368, 50420, 28856, 42664,
                5512,  16660, 27596, 47732, 20540, 38080, 46456, 25244, 5760,
                58356, 888,   50500, 60688, 34684, 46240, 56636, 51964, 20600,
                61668, 34268,
            },
            {
                57728, 12780, 26508, 9424,  47004, 30144, 54664, 13184, 22600,
                35700, 31852, 26504, 52988, 42912, 15588, 24676, 58348, 4892,
                46368, 24108, 59204, 48276, 7824,  28408, 12592, 50512, 20312,
                14932, 60396, 44384, 40284, 52664, 47120, 57128, 34332, 53684,
                2368,  23516, 12376, 19388, 54832, 748,   25168, 17648, 65052,
                52176, 14364, 32840, 21660, 4496,  12272, 34128, 22540, 9384,
                15884, 49684, 65088, 38732, 34396, 17312, 47188, 22656, 14732,
                64668, 55396, 328,   16620, 35184, 23508, 65472, 20216, 3900,
                25176, 13200, 60140, 32732, 48212, 57184, 20880, 62840, 24100,
                35532, 61640, 25972, 57072, 14216, 27456, 42604, 21280, 49448,
                35936, 6680,  27072, 20820, 38676, 28804, 18316, 43480, 1432,
                46500, 9692,  23280, 53412, 35360, 7704,  19632, 2508,  46640,
                52320, 23408, 40892, 12732, 2016,  15648, 56220, 8864,  39596,
                22200, 47552, 16480, 24920, 6928,  21892, 37852, 676,   16432,
                25532, 8220,
            },
            {
                5540,  50032, 42576, 35092, 15136, 24004, 61316, 42752, 47948,
                58272, 1504,  65412, 17872, 37508, 5716,  31208, 40548, 13492,
                35796, 51116, 4192,  31164, 18716, 61808, 56440, 42780, 31632,
                48504, 760,   54564, 17160, 3656,  24224, 5828,  41168, 27852,
                43452, 32408, 57776, 50868, 35240, 61588, 31276, 46696, 34548,
                20844, 3692,  61980, 48580, 37796, 53572, 28504, 51180, 19104,
                36388, 2332,  30492, 4772,  12428, 26036, 57676, 36240, 50472,
                33352, 25304, 43876, 61368, 9464,  39036, 53000, 16264, 57224,
                43944, 35484, 9968,  19244, 64744, 2220,  40348, 32088, 8752,
                3748,  39836, 43832, 17488, 46560, 63928, 5120,  58536, 14964,
                40224, 60780, 56236, 440,   46716, 65460, 11124, 53236, 61932,
                29800, 38244, 3708,  17828, 47536, 26948, 60924, 55996, 36244,
                31552, 9068,  64364, 54196, 32780, 43240, 61112, 30152, 13364,
                62884, 36396, 30992, 57468, 40368, 12904, 62988, 31520, 49584,
                41112, 45064,
            },
            {
                29168, 60696, 2312,  56464, 51076, 4328,  33504, 10092, 28512,
                15976, 39708, 46128, 23184, 10428, 61108, 49776, 20788, 9156,
                28188, 57048, 21848, 11496, 45464, 34064, 1944,  36964, 9684,
                25804, 58268, 29468, 21600, 61544, 38252, 13724, 20088, 10232,
                64412, 15560, 6252,  44748, 21532, 13880, 9532,  58840, 6604,
                40168, 45804, 29208, 59196, 18252, 7748,  45452, 63640, 41724,
                26324, 54616, 59696, 19956, 44420, 53040, 5532,  40828, 2572,
                29796, 11652, 47856, 19500, 27756, 49700, 2216,  33296, 29400,
                62596, 5432,  27580, 54088, 36844, 13836, 26412, 50788, 55428,
                47480, 20332, 59568, 156,   31528, 10992, 38632, 29264, 2208,
                24912, 11840, 44276, 52164, 33756, 23012, 4516,  32472, 49568,
                20500, 13620, 63392, 42920, 57856, 10708, 39332, 22084, 12292,
                18320, 58396, 37768, 6836,  21988, 26428, 49260, 18056, 52544,
                42008, 3512,  9788,  53208, 18796, 48188, 26780, 9924,  55192,
                13768, 36776,
            },
            {
                19588, 16084, 31568, 21476, 62860, 39104, 18388, 53304, 3328,
                20872, 55336, 7532,  58956, 28764, 44704, 3192,  63188, 55328,
                43424, 1040,  65180, 39928, 52140, 5980,  22500, 63356, 46480,
                18480, 6740,  36364, 11920, 49652, 31720, 45796, 59532, 51676,
                36612, 48440, 26716, 3056,  38576, 53400, 42068, 23148, 55260,
                26760, 16688, 10184, 4,     41096, 24968, 14848, 3164,  58584,
                12072, 47524, 8676,  40188, 61404, 32108, 10300, 63292, 17988,
                59032, 7676,  37536, 56900, 5668,  42440, 13948, 46308, 8352,
                20920, 40604, 51644, 23412, 45664, 6852,  43440, 60020, 16740,
                29528, 11704, 34008, 22900, 49916, 54904, 18560, 44896, 53008,
                64424, 31976, 19068, 8484,  15652, 42400, 57836, 25232, 6708,
                41188, 34284, 51132, 24624, 4820,  33036, 45128, 6152,  62412,
                48480, 536,   28184, 50576, 59516, 10748, 2864,  33996, 6960,
                20484, 45944, 28872, 64712, 2076,  42760, 35608, 60252, 4160,
                23624, 65448,
            },
            {
                52392, 40120, 46004, 10988, 7120,  27556, 44228, 59812, 36864,
                50076, 30852, 13384, 35116, 52120, 19056, 32932, 25340, 38524,
                17204, 31956, 14172, 25944, 59604, 16812, 27392, 54416, 13868,
                40544, 51908, 43664, 65120, 8432,  55028, 25140, 0,     30404,
                7276,  18672, 59024, 33584, 63056, 28352, 3872,  49356, 12204,
                37532, 57860, 51100, 35124, 55456, 61052, 32068, 37352, 6560,
                29088, 34048, 17040, 23792, 50140, 14544, 28524, 21152, 43432,
                52748, 23356, 62240, 15176, 31584, 53936, 63436, 24648, 38500,
                59336, 48500, 15836, 3392,  56580, 34104, 10480, 22376, 37988,
                6048,  65388, 52432, 37096, 8196,  25432, 61708, 34168, 22364,
                47968, 3564,  36804, 26288, 62236, 48792, 13112, 37504, 17072,
                60892, 56288, 804,   30904, 16384, 64296, 52448, 25784, 29932,
                41416, 15192, 44292, 18940, 35828, 40404, 63924, 45072, 24388,
                60608, 56316, 15408, 34184, 23088, 5964,  51360, 16988, 45528,
                33148, 1216,
            },
            {
                8780,  58060, 24828, 37156, 48840, 55828, 12788, 25324, 6144,
                63580, 43516, 24860, 20,    41208, 14908, 54036, 11684, 47692,
                6748,  49968, 35068, 44448, 8364,  37872, 48908, 30464, 4708,
                60184, 32816, 23640, 15972, 4620,  34676, 17760, 42888, 62520,
                22804, 54092, 40668, 10900, 16288, 45496, 19120, 31884, 64228,
                2748,  44284, 31060, 23016, 12616, 46652, 20248, 54304, 49768,
                21876, 43780, 57472, 5816,  37056, 2004,  56268, 48240, 38260,
                13256, 33908, 45728, 1528,  21300, 36260, 10868, 18724, 55464,
                1080,  12540, 31888, 62004, 28464, 18668, 63596, 46780, 2056,
                42096, 27080, 14680, 57272, 2740,  41352, 13780, 5484,  9284,
                16484, 42160, 58456, 54360, 39748, 1932,  28588, 54976, 46152,
                9336,  27340, 12104, 48148, 21248, 37500, 13976, 2700,  57232,
                34252, 53708, 24444, 3680,  55820, 14472, 28736, 54396, 37168,
                11560, 26644, 38616, 50352, 12508, 59044, 30060, 20748, 38356,
                56052, 27612,
            },
            {
                49216, 13936, 4888,  64196, 16832, 32168, 868,   47296, 33988,
                15376, 9428,  60648, 48288, 6480,  64080, 36668, 2140,  59960,
                22916, 61964, 53496, 3636,  20936, 56200, 42452, 10856, 35632,
                20296, 2440,  56392, 28256, 39896, 48264, 57556, 11672, 38788,
                14112, 46396, 1468,  25056, 50568, 7800,  60176, 35420, 15336,
                25336, 8028,  18452, 63316, 5252,  28040, 9476,  40068, 16300,
                61740, 936,   52780, 27172, 64396, 45676, 34356, 24896, 6768,
                3268,  28216, 50432, 40240, 26008, 60228, 4248,  50736, 34336,
                26672, 44808, 37432, 9072,  41472, 50620, 14340, 30884, 54148,
                58788, 18840, 48036, 43928, 30128, 59312, 47108, 37732, 55608,
                50556, 29040, 12688, 21452, 7376,  31300, 19644, 63756, 4240,
                22344, 43988, 38900, 59688, 54404, 8680,  43160, 49716, 20156,
                10628, 7148,  63308, 30532, 46248, 8268,  21492, 724,   16828,
                48512, 4012,  7856,  62064, 44740, 25636, 53944, 7620,  63484,
                11908, 42088,
            },
            {
                61504, 34212, 28792, 42900, 52276, 22836, 62288, 19312, 41808,
                53108, 22020, 38440, 55756, 27316, 21228, 45876, 30560, 41960,
                27888, 18808, 10132, 30032, 64032, 15224, 264,   59084, 24868,
                63732, 50108, 12912, 45012, 60884, 20964, 27292, 3632,  50652,
                29052, 35220, 65528, 57220, 30496, 22260, 54896, 41016, 47680,
                53160, 60504, 38664, 49024, 42944, 52640, 2464,  65532, 34832,
                13292, 31068, 39196, 11408, 20480, 16212, 9048,  62352, 53556,
                58664, 16752, 64728, 7384,  56196, 47064, 43088, 30628, 7740,
                64960, 17472, 54312, 21036, 59804, 420,   25360, 36264, 8240,
                23560, 33136, 5204,  10456, 21684, 17520, 27160, 63532, 32348,
                24628, 60684, 4948,  35168, 51812, 44688, 10740, 50036, 36124,
                32208, 52596, 18540, 6616,  35036, 28536, 61456, 23940, 39868,
                59160, 47468, 37928, 12228, 60096, 33140, 50064, 58184, 43028,
                30604, 53636, 41336, 19360, 32112, 24,    40316, 15220, 47928,
                4564,  22480,
            },
            {
                16584, 54032, 20380, 2592,  35384, 9564,  39960, 27964, 58744,
                2660,  29692, 17716, 32376, 10820, 58640, 16280, 8380,  57084,
                13648, 37292, 46804, 40364, 23904, 33436, 46044, 52540, 17480,
                41780, 7844,  31408, 37012, 1572,  9712,  53204, 33356, 61280,
                8372,  20376, 5052,  42464, 12884, 38456, 2040,  5740,  11184,
                29592, 21536, 1324,  14132, 32464, 58688, 22412, 45052, 25456,
                55976, 8336,  47288, 60104, 42776, 30108, 50748, 39624, 18768,
                31960, 42380, 10048, 19952, 29560, 12984, 16356, 22392, 40444,
                58092, 3704,  24360, 47088, 6348,  38784, 57480, 49188, 12332,
                63280, 40008, 51864, 61028, 36068, 53812, 3924,  11428, 19508,
                924,   38804, 46464, 64572, 16944, 59012, 40628, 25060, 14924,
                58172, 2444,  64748, 25512, 46484, 1204,  17944, 32536, 4668,
                16080, 27724, 21728, 51732, 17504, 41560, 25828, 5820,  65072,
                34864, 22168, 57592, 16028, 49080, 9632,  35296, 57352, 26860,
                31864, 37400,
            },
            {
                3448,  58644, 8428,  47720, 60544, 14548, 56856, 6636,  12248,
                49468, 65016, 46300, 4084,  51836, 39696, 24640, 33924, 50888,
                4428,  55524, 1704,  58708, 50436, 11912, 28516, 5772,  38548,
                27100, 46952, 54824, 22896, 17240, 64452, 40872, 15104, 23660,
                43668, 55316, 16576, 32676, 48436, 62424, 27764, 58748, 17652,
                34088, 42268, 56732, 25892, 36648, 17340, 11204, 29532, 4664,
                48916, 18284, 24332, 35412, 5480,  58324, 496,   22676, 11564,
                45112, 25180, 37080, 52408, 34032, 124,   62944, 53388, 48628,
                10460, 29036, 35408, 52208, 31460, 16792, 43168, 20060, 2780,
                45152, 16192, 26016, 820,   31672, 49152, 40420, 45648, 57804,
                43320, 15548, 8828,  23584, 27684, 33720, 52,    61856, 8088,
                42368, 29492, 12604, 40836, 56564, 10736, 50992, 55184, 45400,
                65320, 35472, 1124,  43504, 4484,  62548, 38472, 19964, 9520,
                13544, 27504, 2820,  36860, 64160, 23248, 60528, 42456, 18688,
                52468, 45656,
            },
            {
                25692, 31160, 43924, 38368, 25128, 30236, 50732, 45208, 36984,
                24104, 8272,  35544, 13728, 43528, 62428, 5808,  47904, 64880,
                22108, 43416, 26424, 16436, 7388,  36848, 60792, 22056, 57060,
                10576, 15732, 4984,  59116, 50380, 30808, 46440, 6320,  58312,
                48048, 37380, 26396, 52220, 8868,  19696, 44400, 24028, 51356,
                46648, 64716, 9852,  53548, 6896,  40960, 50624, 60276, 37408,
                41904, 63584, 2940,  52304, 13160, 26440, 33240, 47496, 57116,
                5900,  61716, 2504,  48468, 59856, 39060, 25344, 5640,  36604,
                14664, 44272, 62308, 13088, 9432,  64284, 27548, 34520, 53416,
                29320, 58412, 37628, 13092, 62700, 15776, 8328,  23180, 34248,
                52876, 29976, 49856, 55796, 4480,  13444, 53580, 21480, 46864,
                17428, 50332, 34448, 23056, 15704, 63248, 38276, 29752, 12920,
                8892,  52920, 56908, 24352, 10228, 29788, 14876, 55684, 47852,
                44608, 60236, 51824, 11768, 45608, 28668, 5604,  13308, 1924,
                62928, 10320,
            },
            {
                50172, 13532, 65128, 18108, 55440, 1648,  20976, 32424, 15856,
                54612, 41500, 26472, 57124, 20020, 1876,  37164, 28712, 18276,
                10960, 31436, 35208, 62264, 20016, 54592, 44224, 2064,  33080,
                49248, 62284, 34960, 42580, 25436, 12276, 36308, 26972, 18800,
                864,   11892, 60808, 4156,  35868, 15096, 53796, 37396, 7512,
                13804, 3784,  20460, 30100, 47640, 62940, 436,   19996, 53720,
                14764, 32364, 22008, 38800, 44536, 54580, 64636, 14552, 35504,
                28876, 53980, 15572, 23152, 11284, 43956, 18356, 59072, 32380,
                20552, 56948, 2180,  40176, 23268, 55156, 5320,  60496, 10564,
                21904, 48124, 7128,  44236, 24504, 56688, 29196, 65160, 6008,
                13072, 61752, 20680, 36192, 42972, 48140, 30620, 39024, 26660,
                56172, 3908,  62108, 44728, 5968,  33604, 21092, 3200,  25008,
                41100, 19048, 32740, 48716, 58928, 35896, 52456, 1464,  32324,
                24220, 38980, 7420,  31372, 20632, 54620, 50388, 33588, 56040,
                21604, 39612,
            },
            {
                56924, 36228, 22652, 6184,  11568, 42420, 63676, 4968,  59636,
                19620, 600,   60812, 47520, 30908, 23720, 55092, 14324, 40084,
                60316, 53808, 6176,  47448, 41372, 30924, 14296, 65372, 18744,
                40772, 29440, 732,   19468, 8396,  56336, 2768,  52772, 63564,
                30232, 40236, 21328, 57044, 29464, 64220, 272,   31996, 61576,
                28520, 40416, 36056, 59104, 15464, 23696, 33880, 27252, 9836,
                6556,  57192, 28680, 61320, 17188, 7232,  20916, 42048, 3548,
                49924, 20356, 40100, 31248, 6968,  55148, 28552, 46556, 3424,
                51764, 26400, 17636, 49104, 33984, 44972, 14784, 50488, 40852,
                3616,  32452, 55004, 20000, 39300, 2212,  50836, 18192, 37860,
                26100, 1724,  40912, 11228, 62916, 18432, 57344, 6148,  11528,
                36756, 20424, 9648,  53712, 27880, 47924, 59436, 43108, 50052,
                60652, 28320, 5636,  13416, 45884, 20700, 26664, 40792, 61888,
                4640,  18156, 56136, 42792, 1076,  15784, 37684, 24604, 44436,
                29480, 6728,
            },
            {
                61556, 348,   28044, 53244, 48876, 34016, 26176, 38756, 52148,
                29000, 10200, 33852, 16676, 7016,  52492, 59484, 8560,  45336,
                424,   49988, 15012, 23756, 10280, 4628,  26048, 50724, 8260,
                24152, 12740, 53508, 45948, 63168, 39556, 21640, 43016, 33700,
                8764,  50764, 44880, 24704, 47208, 40640, 11288, 21784, 43772,
                56320, 24588, 50496, 2436,  44392, 11948, 56876, 46700, 35856,
                43628, 49532, 11644, 1296,  48204, 30644, 9928,  38128, 25480,
                63044, 8464,  57716, 46236, 65368, 35604, 13940, 9388,  63740,
                37836, 42788, 7472,  61072, 29892, 1060,  19292, 25448, 36580,
                64828, 17164, 9752,  60060, 34664, 46972, 11032, 42288, 59936,
                46224, 32772, 58508, 7468,  25616, 3080,  34868, 65384, 50976,
                43352, 59712, 31384, 39744, 1772,  18632, 13984, 56364, 7664,
                36496, 16000, 62280, 39096, 2976,  64460, 7648,  16764, 12020,
                49276, 35196, 63780, 26088, 48248, 58996, 8448,  64740, 11584,
                48096, 15960,
            },
            {
                42340, 32920, 46164, 39444, 60820, 16980, 8768,  47688, 13464,
                43688, 62836, 50628, 40336, 12456, 35436, 42244, 21432, 32968,
                25352, 38120, 29412, 64048, 57496, 52712, 39220, 35648, 45040,
                56092, 60628, 37688, 27284, 32564, 15908, 59000, 48940, 13212,
                17464, 62072, 2968,  14168, 7688,  18340, 59692, 48648, 5692,
                16412, 10328, 60936, 18492, 31708, 39080, 4152,  64120, 17004,
                59384, 24832, 19428, 41160, 34948, 51944, 60320, 56420, 17452,
                43640, 12796, 33688, 26124, 4600,  21540, 41436, 49192, 23716,
                30840, 11404, 53120, 21352, 57440, 38644, 47220, 11916, 56540,
                28624, 42720, 48892, 26628, 4588,  31112, 54184, 21368, 3808,
                55104, 16756, 49220, 29536, 52496, 45352, 15484, 22840, 28576,
                1100,  16136, 25212, 49036, 61396, 37068, 10176, 31824, 22492,
                752,   54164, 44044, 23116, 30920, 50508, 42836, 54368, 58728,
                21544, 29756, 9348,  13788, 39824, 32420, 18812, 41536, 2580,
                35352, 20252,
            },
            {
                54228, 8996,  19000, 12948, 2792,  55840, 22988, 58016, 3216,
                35952, 24596, 21076, 2728,  64648, 26548, 48572, 3756,  63336,
                56684, 19472, 4072,  43172, 33636, 21272, 2552,  59340, 16636,
                5624,  21424, 3308,  51712, 10624, 6716,  24988, 3936,  28304,
                56032, 35280, 38772, 31716, 54928, 51024, 34680, 25652, 38528,
                54100, 33452, 46160, 7112,  55468, 26032, 52444, 21296, 30576,
                5452,  38404, 54724, 63832, 4468,  26680, 22892, 46608, 2080,
                29368, 53308, 18776, 764,   51216, 60028, 17180, 53804, 1156,
                59356, 15452, 35728, 4028,  26896, 8952,  62616, 51680, 6864,
                22644, 100,   52852, 12676, 63648, 23644, 14600, 36312, 28756,
                9268,  23920, 13276, 37216, 19348, 60548, 41804, 8416,  47672,
                33440, 52344, 12536, 6904,  23344, 54588, 46052, 27128, 63896,
                48016, 33300, 8392,  57808, 17972, 10660, 33912, 27836, 37636,
                46516, 2084,  53512, 23332, 62232, 4776,  27992, 55352, 50644,
                59592, 25792,
            },
            {
                4204,  49560, 63960, 31488, 26936, 36872, 44928, 30912, 18660,
                7540,  54960, 31972, 44776, 56384, 17924, 29996, 11376, 16160,
                46436, 8968,  51456, 12832, 17536, 47164, 27908, 11220, 30656,
                48644, 34204, 43544, 17832, 47384, 54668, 37236, 64644, 46472,
                22784, 6248,  59396, 20472, 42688, 28212, 3884,  13920, 63344,
                1188,  20988, 27868, 65044, 41752, 14644, 9412,  42736, 1848,
                51256, 13248, 32804, 9032,  45224, 15672, 11900, 32612, 6640,
                49064, 36372, 61612, 42680, 37580, 30508, 8144,  27408, 39588,
                19912, 45272, 48060, 65136, 41764, 16332, 30524, 43396, 33308,
                15268, 61440, 35736, 18248, 41264, 57636, 6672,  61512, 51512,
                43936, 64316, 40468, 56416, 584,   32224, 11972, 54552, 62404,
                19232, 57604, 40908, 64484, 34408, 15276, 42224, 4860,  39808,
                19752, 12532, 52060, 25976, 37036, 60800, 280,   23540, 5672,
                14900, 60288, 43696, 36340, 51464, 12152, 45780, 22808, 7432,
                30796, 13812,
            },
            {
                37564, 57992, 23492, 41136, 52064, 6016,  14480, 64968, 41624,
                59292, 48112, 14936, 9536,  36600, 6508,  51080, 61312, 34672,
                27404, 41072, 59736, 36432, 61676, 7408,  54708, 62692, 41324,
                25020, 64260, 13760, 29772, 60988, 20440, 42072, 31092, 9568,
                40920, 52536, 26204, 1276,  65220, 9832,  57180, 45560, 30336,
                42192, 49884, 12976, 35260, 22752, 48768, 36808, 62268, 27432,
                47260, 60624, 22312, 28904, 58100, 36716, 53784, 64860, 39584,
                59260, 9452,  23872, 15108, 47884, 11344, 56868, 33824, 63120,
                6440,  56148, 32584, 13020, 23876, 54264, 4632,  20344, 57932,
                46632, 39728, 30096, 7896,  44944, 49644, 32372, 39456, 19520,
                2044,  33868, 5460,  21972, 47216, 27376, 39132, 24644, 3612,
                37460, 5324,  44764, 29288, 20124, 2248,  59068, 52508, 16788,
                60240, 29056, 41932, 6040,  47612, 14412, 45076, 56940, 51792,
                40252, 17916, 31148, 3160,  20412, 57424, 34472, 16164, 38988,
                62732, 45296,
            },
            {
                29084, 16708, 1172,  10884, 61232, 20664, 49068, 27976, 11448,
                38872, 1192,  26136, 52728, 60428, 23300, 38952, 43372, 1212,
                54640, 23048, 31056, 2272,  25572, 44040, 32776, 14624, 48,
                52844, 8928,  39028, 55964, 1256,  33776, 12632, 3028,  18888,
                49224, 15300, 11608, 45280, 36736, 16932, 23260, 52504, 18700,
                8852,  37332, 59964, 2168,  5072,  58176, 18860, 53560, 33672,
                16176, 6856,  43960, 18172, 50272, 172,   42204, 18996, 22036,
                13892, 27652, 55556, 5944,  63528, 22584, 45696, 2708,  43796,
                24716, 9956,  28768, 1436,  49600, 34452, 60468, 37912, 2136,
                25220, 10712, 55300, 21612, 27428, 4168,  16472, 10828, 47868,
                26916, 59620, 53084, 10580, 63124, 6920,  59064, 14756, 46288,
                30768, 10872, 25996, 55752, 8692,  49520, 30968, 24196, 9184,
                37712, 1668,  64944, 21748, 55568, 29668, 63360, 20012, 32556,
                7580,  65104, 26868, 49364, 8096,  40676, 61624, 620,   52636,
                10808, 20908,
            },
            {
                55536, 34724, 47988, 43400, 33000, 56828, 3444,  53616, 34084,
                21448, 62300, 29856, 41820, 19288, 4304,  57168, 14276, 20464,
                49496, 6860,  15696, 56256, 40144, 19728, 49572, 23272, 36684,
                18612, 58144, 5896,  22904, 26956, 51148, 44320, 57588, 61660,
                34124, 55304, 29596, 61016, 49732, 33060, 39900, 5168,  61800,
                26172, 56436, 17104, 52308, 28488, 31356, 7368,  12148, 23428,
                40720, 56488, 37568, 3660,  62972, 25416, 8116,  29936, 46132,
                2932,  51348, 33008, 40952, 29112, 19100, 51548, 13844, 37776,
                16772, 52708, 40804, 61176, 18464, 44512, 9720,  28004, 14012,
                64528, 50560, 3044,  60560, 37716, 53272, 65032, 24900, 55984,
                13824, 30568, 17608, 35756, 50424, 42556, 18236, 34836, 50808,
                21864, 60000, 16920, 47204, 38944, 61300, 13472, 35284, 57056,
                46264, 50760, 34040, 16308, 3848,  39780, 9160,  12552, 43296,
                24872, 54884, 11076, 59096, 14728, 28972, 47880, 25092, 32856,
                42640, 5756,
            },
            {
                50464, 59816, 7700,  25928, 15232, 38176, 9064,  24788, 46280,
                5700,  16816, 49920, 12388, 33820, 47076, 26764, 31724, 10248,
                37520, 65228, 46676, 11780, 52180, 4372,  9860,  58912, 45852,
                27436, 42608, 49112, 35656, 10292, 63708, 16748, 24140, 27644,
                5768,  39176, 19880, 7328,  24652, 2320,  59124, 12024, 31548,
                46908, 6296,  24016, 43268, 47576, 37936, 61448, 45300, 756,
                64724, 9988,  31108, 54256, 14152, 34364, 48868, 61844, 56648,
                36020, 43476, 17844, 1456,  59036, 35028, 4848,  62152, 26768,
                48772, 20832, 57736, 36840, 5652,  22728, 52036, 56012, 48024,
                19208, 41968, 33016, 14316, 46084, 20168, 1412,  42484, 36976,
                7180,  46068, 41024, 24340, 13332, 29908, 55552, 1752,  64924,
                41204, 53488, 216,   35720, 22356, 4368,  43580, 26968, 5864,
                20616, 11388, 24904, 44200, 53072, 35252, 26536, 50460, 36568,
                4244,  47356, 37940, 33468, 44648, 21948, 4016,  56692, 18188,
                65292, 27068,
            },
            {
                39248, 13008, 19812, 64312, 47028, 30256, 18468, 63116, 42732,
                55516, 36992, 57888, 2296,  64492, 8512,  53380, 61812, 44932,
                57980, 27968, 18856, 35364, 62044, 29952, 38840, 64708, 5136,
                12560, 30860, 62564, 15068, 46360, 39432, 7152,  53576, 37348,
                364,   46792, 57852, 41900, 14088, 54412, 44148, 21572, 50772,
                34752, 40300, 10764, 63556, 20496, 13960, 55512, 25916, 35784,
                50544, 27028, 19308, 47620, 22688, 41000, 11012, 16560, 5240,
                24760, 11600, 64488, 46956, 9676,  53932, 40380, 31532, 58604,
                8192,  3300,  30812, 12116, 64248, 15740, 39984, 7208,  30708,
                35988, 5776,  24268, 56888, 9172,  29524, 34220, 58296, 51124,
                21716, 63916, 2416,  57356, 4416,  45340, 20924, 9948,  27196,
                7492,  14248, 31892, 63788, 12000, 51832, 18652, 63364, 53992,
                41224, 31596, 58168, 7948,  62728, 18016, 1272,  59220, 62088,
                15872, 22636, 1784,  18864, 63576, 54012, 10192, 36360, 13776,
                46420, 2764,
            },
            {
                31928, 24076, 53520, 2120,  40572, 58448, 51752, 376,   14228,
                30988, 9664,  23772, 44244, 28500, 40200, 17324, 2752,  24240,
                4868,  32680, 42772, 884,   22212, 25912, 55124, 16956, 34528,
                51700, 3536,  19764, 54364, 2364,  21464, 30324, 48328, 13928,
                64752, 22864, 10704, 31684, 63296, 35592, 16476, 28716, 64436,
                152,   15180, 54328, 32944, 3232,  8684,  41892, 17600, 5788,
                58876, 12744, 43532, 5428,  59680, 28244, 55112, 38628, 31704,
                60440, 52296, 21252, 38192, 26120, 15892, 23472, 11484, 46444,
                35224, 54608, 42936, 27468, 47408, 33568, 25544, 59784, 11332,
                45512, 63076, 16992, 51600, 40848, 62040, 12712, 5224,  16852,
                31764, 9880,  52764, 27636, 62672, 37560, 48944, 32768, 57096,
                38332, 44124, 50060, 28036, 57800, 40184, 32972, 48108, 15928,
                3012,  61484, 13972, 28276, 48508, 41760, 32092, 20728, 45632,
                30180, 57256, 52244, 41964, 6596,  31168, 39860, 51220, 59252,
                22580, 9208,
            },
            {
                60928, 44800, 36420, 28252, 5976,  12296, 22760, 36140, 26892,
                60268, 48408, 19992, 52364, 14572, 35644, 22132, 49868, 41448,
                13668, 51768, 8412,  57020, 48080, 13136, 43732, 7736,  21360,
                57288, 40956, 25300, 33164, 59684, 43496, 57160, 9704,  32896,
                42996, 17120, 50276, 3264,  26404, 47804, 8984,  4644,  56936,
                19540, 45424, 58772, 26808, 39272, 51716, 60520, 30224, 48452,
                20996, 39648, 62204, 32308, 1304,  51144, 7204,  19612, 44856,
                3772,  15008, 28868, 7224,  49352, 56912, 44696, 224,   19364,
                63696, 15364, 23556, 51640, 956,   56680, 4744,  43116, 21500,
                54696, 512,   27032, 36956, 6788,  22444, 49456, 27132, 44724,
                61024, 38996, 19608, 34068, 16296, 6256,  23464, 12680, 60964,
                19132, 24496, 3780,  17596, 8104,  23856, 1856,  10016, 29352,
                36184, 49808, 39084, 4940,  23224, 11088, 54388, 6952,  13264,
                39652, 9076,  27892, 12340, 59820, 16616, 25848, 6476,  29664,
                41596, 55120,
            },
            {
                15712, 3676,  50056, 17560, 61736, 34108, 44092, 49252, 7608,
                39508, 3076,  33040, 62740, 6304,  55896, 60132, 11036, 34456,
                63572, 26284, 15992, 60676, 36572, 3240,  32336, 50208, 28468,
                47048, 61648, 8568,  12032, 37816, 27300, 5156,  19136, 51500,
                58680, 29024, 36436, 55880, 20384, 53024, 38816, 42796, 24972,
                36660, 7940,  29980, 22448, 48120, 16048, 34120, 23528, 2268,
                54444, 34844, 8616,  17140, 45736, 36492, 65468, 26704, 49972,
                57720, 34796, 42252, 62768, 32284, 5504,  61768, 29784, 50548,
                38960, 6212,  59468, 9808,  20192, 38416, 61792, 17304, 29240,
                49376, 13392, 32352, 60192, 47516, 18360, 55052, 35552, 672,
                24616, 48364, 12440, 43420, 50920, 58940, 41432, 52968, 46168,
                5576,  34436, 62472, 46704, 36576, 55000, 60432, 45412, 25496,
                56664, 21984, 18456, 55404, 45432, 36004, 58360, 24812, 51188,
                34632, 64576, 48132, 23932, 34260, 49712, 44212, 62248, 920,
                18980, 35064,
            },
            {
                11252, 63376, 21832, 41788, 55060, 9444,  57432, 19440, 65500,
                16348, 42296, 11336, 26304, 38140, 548,   47424, 29080, 19324,
                54704, 37692, 45416, 30504, 18512, 53880, 63540, 39232, 14636,
                1580,  36304, 17892, 52128, 48624, 15636, 62116, 35744, 24684,
                1836,  8176,  40324, 62228, 5860,  12284, 30488, 60872, 49424,
                13176, 52560, 62504, 1916,  12016, 65108, 6820,  46120, 58052,
                14648, 27872, 51856, 24836, 57212, 21788, 13476, 9100,  40012,
                22632, 800,   53396, 13352, 18916, 36692, 41872, 22144, 13116,
                26236, 34012, 41060, 45976, 31968, 13712, 50384, 35264, 3108,
                39092, 58328, 44340, 4420,  11196, 31084, 64104, 39724, 15116,
                59128, 7428,  55860, 1948,  30360, 8920,  26444, 2760,  15968,
                30684, 56164, 52200, 11728, 29836, 41736, 20768, 13496, 63980,
                6976,  43508, 380,   33232, 65492, 16292, 3116,  29260, 43068,
                4800,  19960, 132,   54728, 38932, 3764,  20968, 14292, 47572,
                52792, 26976,
            },
            {
                46016, 38044, 7464,  29748, 14404, 25488, 31688, 4104,  52248,
                29108, 54964, 58556, 46564, 17696, 31232, 43576, 7892,  58872,
                5656,  2492,  21744, 10544, 41388, 6340,  24552, 10084, 60484,
                23576, 44836, 31652, 65496, 624,   22508, 54736, 41392, 12596,
                60740, 44992, 14588, 23996, 34076, 45604, 17196, 22216, 3052,
                33212, 41420, 18064, 44088, 37136, 53964, 26376, 10860, 38272,
                43364, 4672,  63468, 11216, 41604, 2904,  34164, 60256, 17424,
                30792, 47704, 10272, 59144, 25044, 2908,  55284, 9240,  60164,
                53296, 2804,  18112, 64052, 28708, 54516, 7424,  25068, 65284,
                8820,  23648, 19860, 53280, 25724, 43200, 3620,  9748,  52608,
                28884, 32980, 22828, 37868, 62020, 20372, 35912, 64516, 39436,
                9652,  42292, 20452, 876,   15508, 49472, 5020,  34228, 39536,
                51204, 11872, 59928, 29812, 8916,  49484, 38424, 61712, 17412,
                58984, 46112, 14740, 30668, 63132, 10948, 56852, 37184, 31320,
                8360,  57680,
            },
            {
                24400, 32684, 59244, 47960, 52820, 1472,  38648, 46800, 13080,
                24308, 35892, 21008, 8728,  51692, 23216, 64940, 15164, 25296,
                39288, 48296, 62000, 51364, 27180, 58080, 46136, 19816, 52692,
                5380,  55348, 28100, 7340,  40112, 45956, 31236, 4452,  47696,
                28028, 53224, 18896, 49256, 57812, 744,   65392, 54224, 9764,
                59840, 27256, 6332,  57448, 20772, 31008, 42284, 18328, 61432,
                31860, 19676, 47568, 37468, 29876, 53640, 48824, 43612, 55520,
                6416,  64100, 27812, 38832, 46388, 51392, 33684, 17344, 44328,
                30500, 48268, 56404, 11832, 1840,  43728, 20736, 47200, 52800,
                41712, 15524, 34968, 62688, 38476, 16500, 57704, 46764, 20512,
                42508, 65348, 17116, 47668, 44524, 14092, 56920, 28916, 48544,
                22868, 61540, 27104, 38380, 65184, 59040, 26496, 53856, 31136,
                16552, 24432, 37784, 53240, 26560, 13524, 21336, 52816, 8572,
                26740, 36828, 22368, 42332, 7716,  27512, 50856, 23596, 63712,
                43048, 4272,
            },
            {
                50376, 12852, 2824,  18620, 43660, 63088, 21588, 58864, 33844,
                5444,  44684, 1764,  61832, 39640, 12436, 54212, 34044, 42356,
                52680, 31700, 12800, 35016, 452,   15568, 29824, 37256, 33512,
                41164, 11348, 20548, 57228, 13832, 26332, 10224, 64016, 21140,
                37908, 3040,  31772, 10372, 27464, 41800, 36488, 25824, 47348,
                39360, 15380, 51004, 34484, 4448,  59988, 1340,  52268, 7804,
                55772, 23204, 260,   59776, 15552, 19076, 27700, 4048,  24548,
                36940, 16032, 44144, 20816, 7552,  14576, 28104, 64780, 4140,
                37048, 8084,  21656, 25388, 35836, 57844, 16104, 32588, 11660,
                30340, 55748, 1384,  46144, 7924,  29760, 23276, 34372, 12576,
                5596,  51228, 3532,  10836, 25408, 54020, 1252,  18012, 51560,
                13164, 3340,  45540, 33104, 7728,  44016, 19060, 1564,  9468,
                62256, 48144, 19792, 5748,  44828, 41376, 1220,  47060, 31788,
                11932, 56108, 51900, 61084, 18148, 45264, 35108, 1912,  16848,
                39644, 19852,
            },
            {
                55312, 64476, 40180, 27216, 35624, 10516, 16100, 41544, 8452,
                63764, 49668, 15024, 32268, 27620, 5352,  48532, 1388,  20544,
                9004,  17184, 23908, 55172, 43132, 64988, 49348, 3760,  58676,
                16984, 50144, 43460, 60836, 35500, 51444, 17952, 57584, 33748,
                15676, 55624, 63440, 39492, 7052,  51884, 14120, 5536,  20120,
                31292, 55948, 23972, 45564, 13988, 49372, 24808, 36092, 28636,
                12836, 50268, 33764, 43856, 6764,  63992, 10392, 33092, 61204,
                12192, 52084, 2284,  31468, 61696, 56676, 40096, 48884, 24368,
                12816, 43032, 61236, 50984, 40352, 62376, 5064,  38060, 60064,
                6236,  49636, 27924, 13156, 59892, 50864, 2324,  62388, 37392,
                56864, 27084, 40840, 34932, 59640, 6080,  42108, 33680, 7796,
                36908, 58424, 53080, 17300, 23732, 11168, 36152, 57596, 45704,
                40464, 2960,  35032, 55988, 63604, 23736, 57600, 35228, 63064,
                41120, 5400,  2564,  32716, 13328, 59448, 5988,  53836, 12188,
                60444, 28132,
            },
            {
                36536, 15980, 22976, 5292,  56820, 31032, 48380, 54316, 26932,
                18140, 37044, 55324, 41916, 59308, 18960, 36540, 56312, 29512,
                58452, 63800, 46856, 6980,  19424, 39656, 9272,  13432, 26228,
                63128, 1960,  24276, 32200, 3016,  39136, 6264,  49204, 42840,
                9020,  25276, 44316, 21524, 60788, 18196, 32652, 43348, 62320,
                1208,  11316, 64388, 7744,  29184, 40540, 62384, 15756, 46724,
                64340, 40788, 9252,  26408, 55388, 50956, 38972, 46852, 20572,
                41332, 58724, 34928, 48028, 23544, 11132, 612,   19856, 59240,
                54200, 16256, 32912, 4364,  28368, 10088, 23052, 45404, 26328,
                18680, 63760, 21912, 40048, 33208, 19192, 54660, 25608, 48448,
                15420, 31692, 61780, 19336, 49764, 30216, 22148, 46708, 60604,
                24932, 31312, 5100,  47936, 55852, 40888, 50892, 29684, 22060,
                27192, 59376, 14828, 31360, 10124, 16892, 29660, 6540,  19108,
                24168, 49460, 28396, 39076, 47820, 25380, 41252, 30156, 48728,
                33936, 9400,
            },
            {
                704,   52928, 44552, 49536, 61452, 6936,  24408, 240,   60108,
                29736, 22136, 10548, 3084,  24448, 45028, 62400, 11072, 26192,
                43896, 2724,  37512, 28168, 57188, 32080, 22592, 53920, 45508,
                29332, 38208, 8784,  15596, 53852, 62764, 23688, 29948, 768,
                59644, 50800, 5140,  35152, 11688, 46316, 59056, 55108, 28388,
                49284, 37192, 42260, 19564, 58548, 9904,  21704, 5376,  38948,
                2988,  17664, 58616, 21420, 36220, 13344, 23608, 916,   54268,
                8280,  26152, 18436, 4280,  54008, 43272, 36248, 32280, 6228,
                27416, 38796, 47420, 19304, 14344, 53196, 48928, 104,   56952,
                43112, 35976, 3412,  47876, 10632, 43768, 6584,  41132, 9164,
                21488, 192,   52696, 8480,  13100, 38032, 63432, 11136, 53740,
                19720, 43084, 14940, 29028, 61292, 3820,  14212, 63020, 7116,
                12012, 52000, 42856, 4076,  50364, 38928, 54596, 12924, 60052,
                45184, 14880, 64156, 21144, 10000, 55848, 19084, 65076, 21868,
                6704,  46796,
            },
            {
                30612, 59756, 11028, 33496, 14300, 19300, 37764, 45008, 12672,
                39924, 52996, 58004, 47528, 30744, 7968,  14884, 40748, 50532,
                4988,  32784, 16064, 11500, 51088, 1644,  61152, 36224, 5500,
                20160, 48920, 56340, 44760, 27864, 19976, 45788, 13348, 36064,
                19036, 28600, 14256, 54480, 30132, 24572, 4132,  8824,  22980,
                17048, 53172, 3528,  31900, 47072, 35468, 51552, 56788, 32764,
                24480, 53140, 30056, 45520, 2684,  32004, 62696, 42820, 29968,
                14956, 49380, 65068, 39468, 29316, 17028, 63384, 45392, 50392,
                10012, 62660, 1348,  55692, 42200, 31036, 64468, 34148, 16716,
                9216,  52516, 14620, 58848, 26516, 63104, 16684, 30040, 59232,
                45872, 38668, 24260, 44364, 57696, 27344, 4348,  16236, 39936,
                636,   63912, 9804,  38096, 20256, 25956, 34616, 47388, 18008,
                55228, 33692, 23316, 19444, 60852, 26844, 48316, 2024,  37152,
                52340, 7612,  34468, 58468, 592,   36760, 4844,  14616, 43720,
                58032, 25260,
            },
            {
                41556, 20732, 3496,  28508, 42656, 53248, 65096, 32484, 50860,
                7544,  4316,  33948, 17292, 64732, 52068, 22552, 34728, 18412,
                60116, 53256, 40204, 62676, 25460, 48084, 14812, 42660, 59152,
                33168, 12752, 64792, 40612, 6848,  11144, 59088, 52952, 41080,
                64920, 46536, 38640, 61836, 1356,  50204, 36028, 40248, 12896,
                34216, 59796, 26256, 15132, 55168, 532,   18184, 28024, 44612,
                10980, 60772, 14672, 6200,  49300, 56468, 17248, 5364,  57796,
                37660, 32760, 6164,  12872, 58320, 8420,  25756, 13996, 57264,
                20956, 35044, 24144, 58900, 7536,  37388, 20524, 12348, 41020,
                30160, 24568, 55096, 31448, 37296, 1268,  52048, 35428, 12136,
                63900, 54308, 5008,  33396, 17820, 48492, 55016, 32232, 44924,
                26488, 35332, 50288, 57768, 6192,  53824, 2188,  43512, 30732,
                38740, 928,   64964, 45668, 35676, 8320,  42124, 21780, 32256,
                25764, 40680, 17376, 53340, 44560, 31768, 50220, 27628, 53184,
                37828, 17728,
            },
            {
                51140, 63956, 38836, 56460, 25632, 8924,  3896,  21652, 16008,
                62552, 46116, 25568, 13016, 38132, 2232,  55656, 6532,  48184,
                24632, 9552,  21440, 45116, 34896, 7772,  18496, 24080, 10128,
                51948, 44,    17688, 23128, 34636, 49928, 31216, 4212,  24736,
                9984,  2856,  22732, 17144, 42416, 57104, 19700, 64836, 44352,
                48116, 7140,  39420, 63704, 23588, 42628, 13096, 63180, 7000,
                50068, 36696, 41920, 65252, 20300, 40024, 28372, 47232, 11020,
                61776, 22620, 53192, 45988, 20628, 51184, 1728,  30512, 39352,
                5196,  44208, 29624, 11248, 46196, 25628, 59876, 3740,  50676,
                62356, 7252,  45468, 4736,  20596, 56840, 23136, 49048, 3504,
                18740, 28660, 14448, 62312, 40944, 1544,  22880, 9096,  51384,
                57008, 13216, 22524, 45356, 32020, 39704, 64172, 21712, 59668,
                9980,  48808, 28700, 13232, 5420,  58260, 16276, 64384, 9952,
                57692, 2996,  29628, 11808, 22700, 62900, 39816, 10640, 1780,
                61560, 12876,
            },
            {
                34416, 8008,  16128, 46996, 36320, 58244, 48720, 40988, 27048,
                35944, 19580, 42076, 59728, 28224, 43716, 31952, 61340, 38700,
                14024, 30392, 54748, 3684,  58404, 29644, 39276, 63600, 46544,
                26476, 37176, 30288, 58248, 3388,  61860, 37964, 16964, 57280,
                33976, 53840, 49032, 32044, 9392,  27260, 15080, 53484, 2092,
                29640, 20800, 10848, 50484, 4320,  30440, 59420, 39920, 22908,
                19116, 1028,  26944, 33424, 12028, 7988,  52316, 25384, 19476,
                44180, 2036,  27760, 41648, 33724, 37268, 61136, 47840, 17448,
                53588, 65300, 14716, 51416, 17852, 5468,  54436, 28228, 47096,
                22024, 38596, 17400, 65212, 40560, 13584, 44620, 32156, 26252,
                42700, 36592, 50844, 7500,  29896, 59176, 37008, 64992, 18732,
                5648,  29416, 61904, 8276,  17492, 11844, 49544, 15720, 6804,
                25320, 56872, 17276, 40124, 53356, 24680, 29884, 46460, 53896,
                19660, 43844, 61424, 48156, 26212, 7488,  16544, 56736, 33188,
                23992, 4292,
            },
            {
                44324, 54028, 22268, 932,   12244, 18084, 30720, 60736, 10636,
                56048, 51760, 828,   9308,  49696, 20400, 12168, 26640, 45572,
                296,   64696, 42280, 19552, 13036, 52284, 2376,  31712, 56072,
                5192,  42020, 54340, 47628, 14748, 43064, 8208,  21172, 44528,
                27088, 12932, 62788, 6932,  59544, 45992, 33184, 6024,  25132,
                61724, 56068, 32992, 45368, 36428, 53600, 9140,  47364, 34460,
                57912, 54820, 45920, 23664, 58288, 37308, 61132, 4128,  35096,
                55460, 16440, 10152, 63012, 5472,  24256, 54988, 11668, 41912,
                22776, 2736,  38188, 32704, 61916, 40492, 34764, 14376, 57956,
                1796,  33280, 11696, 50996, 28332, 9060,  61116, 6828,  55308,
                10220, 58756, 21112, 44012, 11784, 46668, 25836, 14144, 39592,
                33492, 2448,  48228, 41268, 25156, 58596, 28772, 36740, 51940,
                42144, 34500, 3520,  62780, 32360, 49464, 320,   13792, 38360,
                6396,  50240, 14468, 36368, 3940,  59472, 45732, 20828, 42316,
                29044, 48848,
            },
            {
                60152, 26208, 31920, 63016, 50928, 24200, 2584,  44360, 5660,
                14352, 31620, 23296, 56928, 34940, 63820, 4100,  17720, 57320,
                32940, 49936, 7216,  37140, 25984, 60960, 48704, 16508, 21460,
                13716, 61496, 9608,  19748, 25716, 52476, 28844, 63460, 48464,
                5372,  41876, 19876, 37280, 23704, 39772, 11824, 49416, 36920,
                17500, 41620, 14064, 5868,  19796, 25988, 16820, 2160,  29216,
                14892, 10168, 4400,  51064, 16360, 43584, 29864, 14240, 48976,
                39368, 31260, 59192, 47068, 14556, 18824, 3292,  32632, 28160,
                8976,  58620, 26484, 48660, 1152,  22196, 44852, 8956,  18848,
                42880, 25960, 48200, 59380, 34600, 18444, 52484, 37804, 16372,
                64684, 1904,  25152, 56584, 34848, 19544, 2924,  49128, 60716,
                43704, 55412, 20620, 36164, 52880, 148,   45916, 4308,  62104,
                19780, 12216, 44832, 22300, 8648,  19256, 43448, 34656, 62612,
                27792, 32976, 21528, 41784, 54740, 29964, 34712, 51664, 64300,
                15516, 9828,
            },
        },
        {
            {
                23184, 3540,  31940, 11020, 37552, 54868, 5636,  63572, 50520,
                31372, 62128, 13820, 29628, 37880, 45816, 63628, 15572, 21796,
                5224,  17784, 55852, 6912,  53224, 63868, 46908, 17192, 28916,
                3768,  12156, 38104, 6224,  61620, 3556,  30388, 60304, 50700,
                17648, 40920, 6228,  58052, 27452, 3868,  55936, 30744, 11348,
                37036, 14260, 39900, 25312, 34660, 29588, 2412,  46268, 26676,
                35300, 57932, 19848, 60460, 7532,  56744, 33248, 50992, 25420,
                5748,  18796, 43744, 4888,  60644, 35880, 18588, 64040, 13984,
                47240, 36564, 51004, 15212, 44928, 18656, 63364, 48792, 20184,
                26532, 43900, 17000, 60808, 32712, 9028,  58704, 50096, 11832,
                42736, 24960, 63684, 2092,  48536, 37780, 62988, 54568, 16420,
                51568, 28408, 38312, 18760, 43596, 52684, 62648, 9576,  48716,
                17468, 32700, 11700, 50528, 14736, 26728, 54852, 31984, 20880,
                2844,  61456, 17892, 38492, 58000, 3652,  41064, 5916,  29040,
                14552, 49600,
            },
            {
                26752, 61496, 51816, 43920, 416,   48924, 28832, 44276, 16296,
                39432, 4260,  8556,  59024, 22640, 3068,  52176, 33460, 9360,
                51120, 41860, 36620, 12728, 24092, 39724, 1208,  52324, 35716,
                55840, 64048, 22936, 54252, 46724, 11872, 22228, 36640, 26140,
                8728,  30792, 64460, 14940, 48908, 38980, 63356, 25060, 46944,
                62324, 28460, 4732,  44976, 63976, 52220, 13972, 55464, 42840,
                10540, 53728, 4348,  44764, 11504, 39408, 21040, 9448,  59848,
                48556, 34020, 63672, 27964, 41460, 22648, 55652, 38800, 24120,
                28860, 2316,  8248,  39480, 57000, 1028,  37360, 10380, 7388,
                33320, 5476,  23668, 55332, 27684, 4452,  34752, 23416, 61128,
                4092,  54236, 36188, 28632, 12800, 32324, 19548, 756,   41600,
                8960,  64428, 48152, 33276, 56804, 26832, 20996, 3716,  59232,
                41552, 55300, 7756,  45972, 35220, 1884,  46948, 8440,  60024,
                35308, 42676, 29096, 22568, 49788, 33420, 20200, 56900, 52428,
                62716, 41764,
            },
            {
                6184,  13064, 39552, 21028, 25816, 58352, 12756, 22240, 9652,
                52780, 24760, 48560, 41560, 54308, 11500, 26416, 40320, 57160,
                24716, 65484, 1900,  59068, 20756, 32828, 43520, 13816, 5720,
                45776, 31764, 15852, 26456, 35012, 42700, 15512, 48704, 4188,
                61280, 21300, 51672, 36304, 23048, 9456,  32820, 2000,  7676,
                19332, 49016, 54808, 16964, 7484,  22892, 36688, 18708, 5776,
                32328, 16056, 40724, 25724, 49560, 28804, 64408, 41776, 30736,
                14888, 38400, 10764, 52544, 31712, 12008, 3528,  50016, 9852,
                58136, 54340, 22204, 64856, 33592, 27976, 14252, 53076, 58320,
                40756, 65348, 46576, 10788, 20760, 41456, 45568, 16844, 30420,
                39564, 13852, 18192, 43568, 56496, 4936,  46260, 58140, 35616,
                25480, 3776,  14052, 60144, 7648,  11748, 46584, 34960, 14312,
                28348, 21316, 38660, 59880, 18960, 65480, 40824, 23804, 51208,
                13376, 5904,  53936, 640,   62076, 27100, 44836, 24732, 1400,
                18676, 34572,
            },
            {
                56212, 30600, 59800, 17400, 7776,  32724, 61728, 35592, 56584,
                27512, 60632, 34640, 15348, 32036, 61768, 19236, 46968, 4048,
                14612, 31820, 27904, 46040, 50020, 8424,  62196, 27220, 21868,
                41104, 8772,  51400, 528,   58280, 7380,  64788, 53496, 38988,
                33064, 12580, 42012, 5320,  46284, 17912, 53920, 43792, 58872,
                35336, 23624, 10056, 57856, 32864, 48072, 59772, 39828, 62444,
                51596, 21136, 63724, 34708, 14824, 52660, 476,   18012, 4028,
                45960, 57648, 20884, 1804,  46868, 16380, 42764, 60164, 34176,
                19064, 41524, 12792, 26024, 4336,  48084, 42664, 23316, 29820,
                15552, 2912,  37520, 50248, 14160, 63088, 52700, 168,   57212,
                21716, 47348, 61672, 8092,  51184, 39872, 15104, 29604, 49772,
                20816, 45208, 30564, 40720, 23424, 37472, 65072, 31132, 53044,
                2592,  62520, 25052, 5164,  51960, 28792, 4056,  56428, 30484,
                17096, 63392, 46836, 11016, 36196, 13828, 8340,  60124, 37940,
                10688, 44572,
            },
            {
                48400, 2700,  36168, 47140, 52964, 41608, 4996,  46228, 1356,
                18412, 43152, 6064,  21380, 264,   37168, 7632,  29164, 59904,
                37836, 18684, 10140, 54792, 3980,  16840, 56412, 37120, 59856,
                18444, 48492, 62732, 39248, 20032, 31208, 27464, 10348, 18728,
                45732, 54624, 200,   57620, 29524, 60892, 11820, 39940, 15464,
                51592, 30164, 42624, 38468, 3336,  27864, 11408, 552,   24804,
                29220, 46536, 2352,  55920, 6068,  22904, 36944, 58636, 53956,
                23644, 7988,  26704, 55076, 36784, 65364, 29888, 26112, 5376,
                46000, 31168, 49636, 60244, 17720, 54988, 5892,  63004, 35932,
                51324, 19072, 28928, 59624, 31664, 7480,  36824, 26308, 10400,
                50608, 31740, 2448,  26812, 22028, 65476, 24496, 7052,  10816,
                62332, 54264, 17408, 1676,  55668, 50620, 5424,  19024, 44592,
                10528, 49320, 31452, 12556, 43796, 15788, 36908, 11352, 44848,
                40028, 25640, 32160, 19532, 41912, 54488, 47744, 16644, 31872,
                64088, 21632,
            },
            {
                54356, 15672, 9596,  27460, 64696, 14152, 19732, 38396, 50392,
                31268, 12576, 65120, 47696, 56708, 50624, 42860, 12176, 53952,
                49580, 44460, 60676, 34632, 39240, 25272, 30740, 11096, 53132,
                2360,  29516, 24660, 11400, 44424, 56292, 47668, 1956,  23380,
                58996, 27916, 16684, 24648, 34236, 49484, 21080, 27172, 5692,
                65096, 1004,  13440, 60584, 19584, 44800, 15500, 56688, 49084,
                9052,  38236, 12528, 31572, 59704, 44448, 47912, 27360, 11864,
                35236, 43396, 61884, 5992,  49060, 19924, 8172,  52452, 14228,
                62924, 632,   35992, 10500, 44364, 38612, 20868, 12096, 45620,
                9132,  57440, 43192, 1584,  24520, 55620, 18268, 43476, 64724,
                5712,  35352, 59348, 38528, 11532, 34252, 53208, 42364, 59272,
                38020, 5988,  34600, 48728, 28704, 15204, 42292, 25796, 58404,
                37092, 17092, 40984, 56652, 33804, 22268, 59180, 48672, 20368,
                1960,  58036, 51752, 6824,  59252, 30344, 4312,  40004, 51204,
                6992,  25360,
            },
            {
                29496, 61312, 40624, 22732, 1848,  55584, 29456, 24608, 59312,
                8316,  53716, 38952, 28276, 16972, 23800, 63668, 35076, 21252,
                1468,  7300,  23340, 13144, 42740, 64568, 49020, 6712,  43884,
                34088, 14812, 60392, 36252, 4152,  16592, 34108, 41548, 61824,
                6924,  35928, 44252, 63528, 8296,  3764,  37676, 57032, 32044,
                46696, 21560, 54080, 25440, 50468, 64352, 30908, 42120, 34520,
                17820, 61832, 23848, 42004, 19296, 10336, 7036,  32968, 61040,
                50780, 15268, 39212, 32228, 13420, 24740, 57384, 44552, 38132,
                17164, 53632, 23752, 7048,  29944, 57240, 32616, 3880,  61044,
                27176, 22752, 6756,  47664, 34160, 11988, 49380, 29036, 40276,
                14752, 19772, 44260, 55456, 16284, 48100, 1344,  18604, 31580,
                13656, 26460, 43276, 63796, 21724, 9200,  61268, 1120,  54536,
                22896, 5828,  64060, 564,   52900, 8748,  27764, 6372,  64312,
                34248, 9248,  38248, 15832, 23928, 64800, 20976, 27260, 58292,
                13428, 42720,
            },
            {
                19504, 4828,  51912, 44864, 35068, 49604, 10424, 63412, 15940,
                34880, 23248, 3588,  44936, 9744,  31076, 2944,  15052, 26872,
                41232, 62920, 30104, 52424, 20016, 152,   15544, 22464, 38360,
                57680, 5152,  46740, 54460, 22576, 63248, 12472, 51744, 30296,
                14692, 49780, 10820, 40456, 52904, 14224, 62140, 43104, 17304,
                9316,  40648, 33412, 7184,  36248, 4100,  22692, 53592, 6480,
                57748, 3756,  50532, 54044, 26404, 65184, 16620, 40844, 20712,
                2452,  28960, 18512, 58516, 876,   40776, 34540, 3952,  21932,
                58860, 27648, 40544, 64348, 51520, 14928, 25200, 48952, 34872,
                52580, 40380, 17120, 64332, 38000, 60648, 3580,  22656, 58604,
                53748, 25100, 9336,  29784, 4012,  60328, 27908, 46068, 56572,
                23024, 52144, 4172,  11804, 57720, 33492, 39504, 30080, 13660,
                34276, 47044, 26632, 19664, 45432, 61760, 38148, 50896, 24896,
                14032, 55500, 28468, 44440, 49288, 2208,  11848, 34964, 45236,
                488,   37248,
            },
            {
                33224, 56612, 11900, 18016, 31608, 7056,  39576, 43776, 2172,
                47096, 58144, 14232, 62612, 41352, 51680, 58340, 45564, 55672,
                32660, 16468, 57840, 4904,  47772, 33692, 54708, 61808, 26372,
                50612, 19300, 9472,  27320, 39992, 48896, 8024,  25592, 38264,
                3332,  56224, 21976, 31376, 19020, 47080, 23768, 1888,  55404,
                26300, 62848, 48648, 16752, 55880, 10232, 39180, 13024, 20124,
                45384, 28504, 15260, 34924, 1408,  38368, 49864, 56040, 5228,
                45728, 64188, 9768,  51860, 46264, 63288, 11440, 50516, 31516,
                9340,  47428, 12996, 20332, 2524,  59660, 42328, 18600, 336,
                14240, 31292, 10812, 54076, 21204, 15316, 45852, 8420,  33700,
                1220,  47168, 51384, 62348, 40568, 21128, 36516, 6364,  9644,
                39956, 60924, 19476, 37308, 47264, 16640, 52216, 45016, 8336,
                59600, 50432, 10316, 39072, 30380, 13276, 3564,  17992, 42320,
                46464, 21412, 62500, 5288,  32632, 41440, 56272, 52380, 17476,
                62204, 49168,
            },
            {
                7792,  63968, 38356, 25620, 60616, 57452, 21288, 27292, 52720,
                19412, 37980, 25936, 33388, 6680,  18124, 36768, 4988,  10648,
                49172, 38684, 25836, 11416, 37464, 27992, 45308, 8144,  12560,
                31388, 42020, 65056, 32824, 976,   18300, 58588, 53940, 20252,
                46776, 65400, 26636, 2364,  60036, 36120, 28692, 11044, 49736,
                37224, 4896,  12496, 43392, 28140, 61132, 47784, 26008, 63580,
                32680, 40636, 9796,  60224, 43904, 29644, 13364, 24768, 31584,
                53460, 41976, 26224, 35952, 20924, 16152, 28340, 54960, 42452,
                1748,  61996, 32932, 44968, 36832, 28040, 8000,  54864, 37708,
                62516, 56852, 44748, 28232, 5820,  41076, 51996, 30496, 63252,
                12464, 36292, 17776, 6940,  33060, 54304, 14372, 64044, 50888,
                33772, 220,   44520, 27576, 6656,  24748, 3960,  64704, 19372,
                28016, 41568, 16084, 57556, 23028, 33336, 55192, 60836, 29564,
                35700, 380,   12236, 51428, 19088, 36576, 7540,  24664, 30956,
                10416, 23120,
            },
            {
                27668, 45988, 1192,  15152, 47416, 3684,  12608, 62096, 32248,
                4972,  11128, 55820, 48464, 21112, 61288, 28088, 24060, 64376,
                19832, 7996,  43740, 61252, 55832, 3012,  17780, 40060, 59488,
                1740,  21700, 15400, 52044, 44160, 29072, 36056, 4596,  42000,
                15984, 34404, 7944,  43652, 54940, 6308,  51988, 33528, 19796,
                60308, 30252, 23212, 59100, 19304, 2688,  35196, 51952, 884,
                59148, 23016, 47932, 5004,  21256, 57200, 8208,  62872, 37424,
                12232, 22412, 7420,  55940, 30832, 4652,  60880, 24100, 18652,
                36344, 56336, 5048,  16568, 50212, 10936, 63768, 45836, 21584,
                5332,  24456, 50064, 1976,  32916, 58220, 25980, 17416, 39120,
                55960, 27472, 42548, 23640, 12080, 49004, 26260, 43296, 17824,
                29912, 14996, 55708, 63100, 31768, 49800, 56300, 22308, 37764,
                1840,  53268, 6848,  63192, 2372,  47440, 40072, 10392, 5964,
                53900, 58812, 39584, 26444, 59876, 14272, 63172, 46732, 4052,
                59716, 41972,
            },
            {
                50616, 20640, 55200, 30504, 41368, 53392, 36472, 16976, 49492,
                40304, 60412, 29808, 1044,  52852, 12308, 40020, 47356, 54180,
                736,   35044, 51648, 14020, 21296, 31932, 62972, 23812, 53284,
                46356, 36872, 56668, 6804,  60176, 13464, 22900, 62340, 31568,
                11144, 59408, 50408, 38624, 12544, 16396, 41320, 64656, 45580,
                15296, 472,   52916, 40016, 46168, 31468, 15000, 42928, 8380,
                17188, 37184, 12444, 62440, 52432, 17764, 34072, 47388, 160,
                16780, 58164, 48988, 2196,  43880, 38564, 46900, 7164,  14096,
                49648, 26520, 21480, 53800, 40904, 23608, 33400, 13508, 29668,
                41656, 9520,  36200, 19220, 63588, 48316, 10272, 2892,  44972,
                21016, 5220,  58896, 64616, 45120, 1656,  58156, 5028,  23960,
                59984, 48500, 21936, 9068,  41828, 13780, 35732, 11064, 43536,
                62160, 32148, 25048, 36332, 51504, 19248, 26536, 15368, 49244,
                23792, 17504, 31396, 47464, 9712,  43216, 28740, 20256, 39276,
                54148, 15952,
            },
            {
                2840,  36636, 10924, 63052, 8412,  22868, 29108, 43112, 8052,
                24248, 45228, 15824, 36040, 43896, 7812,  31696, 14884, 42564,
                30016, 59284, 24772, 5436,  41188, 50420, 9664,  35448, 3948,
                27156, 49308, 11048, 24812, 39104, 47796, 8880,  55580, 44496,
                608,   23844, 17948, 29112, 61964, 22064, 25564, 3656,  8580,
                38548, 57704, 33904, 9532,  5624,  65372, 24288, 57108, 29176,
                50388, 55344, 26884, 31808, 39172, 3504,  23452, 42756, 60008,
                29284, 40100, 33636, 14548, 59132, 10984, 52596, 34104, 65112,
                39544, 9792,  59556, 30060, 6536,  57020, 1860,  52128, 48008,
                58596, 16248, 60904, 43208, 13400, 22548, 37148, 53068, 60760,
                14776, 49880, 30924, 9620,  38584, 19196, 31964, 37128, 10596,
                41224, 2808,  34584, 53112, 18436, 61480, 2948,  29780, 48812,
                14436, 18556, 45552, 12568, 8652,  42472, 64244, 57376, 32756,
                44120, 8240,  65288, 3512,  22852, 53568, 1640,  57984, 35344,
                12920, 32124,
            },
            {
                58560, 44464, 25244, 34060, 18632, 50720, 57072, 164,   65008,
                54116, 20188, 5772,  63884, 22848, 59396, 56240, 4540,  21912,
                11300, 17228, 45252, 65216, 28600, 15976, 57168, 42956, 13624,
                30636, 18056, 62080, 34288, 2772,  30436, 17148, 50900, 27248,
                37744, 48168, 56704, 33016, 1652,  46976, 58428, 31716, 54504,
                21436, 44420, 25692, 17152, 49144, 54344, 11636, 38332, 19544,
                3240,  42416, 6456,  15844, 44744, 49740, 27560, 54000, 10456,
                20096, 51604, 24500, 64388, 18980, 27208, 21908, 57612, 29448,
                43772, 3712,  46244, 34456, 14432, 60860, 38076, 19484, 4116,
                25700, 33792, 52784, 7724,  27536, 56164, 6476,  33988, 24828,
                41536, 624,   35356, 16224, 56464, 28000, 52628, 63240, 47132,
                54928, 13268, 38300, 5212,  26500, 46132, 58884, 23912, 54420,
                5124,  40132, 56048, 59944, 28656, 34836, 4376,  21484, 1564,
                37212, 52032, 14604, 56676, 38568, 33348, 16800, 48388, 25904,
                5296,  64896,
            },
            {
                17688, 40372, 6056,  60320, 46412, 4156,  35016, 15304, 26340,
                10720, 33632, 50996, 27652, 38976, 17856, 34476, 50552, 63112,
                38608, 55372, 32368, 1940,  36724, 48428, 6780,  20744, 64080,
                54968, 5172,  41596, 20108, 53404, 64620, 41000, 5980,  20512,
                63656, 14156, 4920,  52980, 42600, 10488, 35900, 50336, 28216,
                12252, 63568, 51508, 29612, 36500, 21172, 4620,  59824, 45460,
                34908, 64496, 23944, 58488, 9264,  63228, 14024, 35396, 6400,
                61720, 3860,  44516, 8152,  36924, 42232, 6280,  432,   17376,
                12708, 24672, 62964, 18640, 49124, 43424, 26956, 31864, 65416,
                11620, 39392, 716,   30412, 45700, 40208, 17788, 65024, 11784,
                28940, 55136, 46624, 23000, 50656, 3004,  13684, 21388, 7312,
                29196, 25084, 65308, 50176, 31056, 40480, 16668, 8216,  33884,
                60796, 26864, 40,    21952, 48972, 54836, 13556, 46324, 61608,
                27060, 41804, 19900, 29504, 45788, 7044,  63632, 10260, 41004,
                52288, 22644,
            },
            {
                29232, 49504, 14348, 54040, 27616, 12248, 39520, 59440, 47900,
                38084, 57540, 13552, 3256,  48220, 9732,  25184, 1660,  27396,
                47708, 6596,  51224, 22824, 12212, 60888, 25624, 33644, 45556,
                38116, 8860,  58360, 44808, 14540, 25784, 33424, 11772, 59176,
                35272, 9140,  39976, 26868, 19180, 63876, 14784, 4492,  18560,
                41664, 2232,  6516,  13960, 62036, 40916, 26700, 48296, 14416,
                30508, 10892, 51244, 36816, 19756, 2028,  56144, 22136, 38932,
                46512, 32176, 12732, 54512, 29932, 61384, 51408, 47320, 32668,
                55860, 52256, 36612, 1324,  11060, 22492, 8308,  54360, 45180,
                17896, 50232, 57156, 20548, 62260, 4368,  47816, 31192, 51060,
                8500,  19060, 61592, 5700,  40196, 60248, 42680, 35096, 16980,
                44544, 58332, 19820, 9736,  57116, 912,   36776, 52248, 20308,
                46504, 10516, 37564, 31744, 6640,  17244, 38740, 30368, 53744,
                11244, 5132,  35412, 62124, 12128, 24980, 51064, 19140, 30748,
                45260, 8260,
            },
            {
                37796, 57572, 1972,  21856, 31948, 63340, 44596, 21308, 30472,
                2268,  42272, 62372, 31124, 53848, 41408, 61440, 44760, 15280,
                35612, 9324,  57936, 18236, 43636, 39112, 52944, 1388,  16520,
                50960, 23192, 28116, 232,   48732, 57296, 3732,  46800, 52692,
                29740, 22160, 46252, 60636, 7252,  24336, 39292, 56992, 48628,
                59792, 23908, 34536, 56244, 44184, 740,   32736, 8844,  61572,
                21832, 1560,  46784, 27828, 32956, 41120, 47828, 30424, 15576,
                57524, 26208, 18052, 48700, 2332,  14788, 23796, 38284, 63680,
                8084,  20804, 40748, 28956, 55068, 61972, 39856, 14840, 5728,
                60204, 23468, 36716, 10492, 15396, 53648, 23808, 1964,  42848,
                58376, 37692, 27008, 12260, 32624, 9032,  26004, 49536, 61844,
                1704,  40256, 33508, 15404, 23004, 43320, 12736, 61944, 28820,
                42060, 15716, 65156, 52336, 44644, 62536, 24996, 8384,  18452,
                60220, 23440, 49596, 2160,  54992, 42400, 36544, 276,   61388,
                55268, 12312,
            },
            {
                62932, 24712, 35432, 42148, 9508,  17276, 5140,  53304, 7860,
                18828, 23696, 45516, 16496, 6688,  21180, 12004, 58316, 30348,
                20168, 40924, 25952, 62488, 3668,  30848, 10064, 59508, 29608,
                12424, 60652, 32692, 36788, 10252, 22700, 38072, 18136, 42092,
                1800,  57764, 15904, 50704, 30876, 54404, 45444, 33212, 9328,
                30248, 38168, 47052, 20020, 9912,  58856, 51848, 18000, 54184,
                41824, 57784, 16540, 53200, 61036, 4788,  11380, 64992, 50480,
                1592,  35968, 62788, 40420, 57176, 34916, 19428, 10840, 42920,
                26808, 50072, 15160, 59136, 33072, 4528,  47764, 34920, 29440,
                42040, 3464,  48372, 32480, 26776, 59596, 38488, 13640, 21672,
                34184, 3896,  44316, 64328, 47688, 18220, 57716, 4220,  31280,
                11828, 53784, 6664,  48800, 62896, 55080, 25628, 5928,  50560,
                3280,  56544, 24352, 19408, 11960, 3132,  58336, 40700, 47872,
                31884, 44532, 58040, 16096, 28004, 21400, 59736, 13912, 26924,
                34316, 3444,
            },
            {
                47504, 18952, 52120, 61024, 48712, 56332, 25296, 33452, 50448,
                60476, 10328, 36616, 26796, 56884, 33392, 37536, 51088, 6384,
                64300, 54144, 12848, 34132, 46492, 55468, 20060, 24252, 48040,
                3800,  40536, 54060, 17556, 61540, 51168, 27592, 63348, 13352,
                25152, 44076, 33484, 3364,  37260, 12448, 108,   22220, 16460,
                62624, 7428,  54036, 15924, 28188, 23292, 39196, 3592,  34156,
                24916, 38500, 6040,  13032, 22952, 44332, 25648, 18680, 7688,
                42072, 23152, 10408, 4960,  27664, 44680, 59956, 3388,  30752,
                58100, 2492,  7156,  44296, 17844, 24884, 51328, 19800, 56592,
                26108, 13348, 64148, 43580, 8184,  34832, 44876, 56308, 63308,
                16864, 53384, 23660, 14916, 54732, 36656, 22152, 51844, 38092,
                23540, 46552, 27400, 36416, 3508,  30532, 47092, 17536, 39124,
                33260, 8536,  48112, 35576, 42520, 28024, 33456, 50908, 652,
                14008, 6916,  39580, 34244, 8980,  47280, 4804,  39304, 49680,
                20944, 42772,
            },
            {
                15712, 10520, 6568,  28588, 796,   37348, 41096, 64564, 14288,
                28316, 55320, 49668, 1372,  65516, 46156, 17496, 3400,  23896,
                43492, 192,   48664, 28224, 5516,  15148, 42048, 36396, 64972,
                44144, 6464,  21284, 13560, 42944, 4636,  8400,  31796, 59748,
                6884,  55000, 11080, 64776, 20328, 28064, 61512, 51288, 43800,
                26476, 12832, 42152, 64036, 31868, 50088, 12296, 63324, 28800,
                7748,  45264, 64136, 31040, 37664, 55392, 34260, 52268, 28524,
                59644, 45712, 53152, 21044, 33124, 8604,  54096, 49204, 16868,
                36124, 23076, 46980, 64888, 38484, 12196, 244,   63312, 9672,
                37528, 52976, 21644, 1200,  51544, 18896, 5096,  10620, 28576,
                47036, 7796,  39788, 30620, 1764,  28104, 5816,  45080, 14608,
                64500, 18284, 59812, 42156, 21012, 11212, 35464, 63932, 21816,
                58656, 14040, 29884, 60404, 5824,  55732, 15472, 20684, 36496,
                64812, 25920, 19716, 52004, 63924, 30188, 54008, 17404, 58664,
                5960,  31180,
            },
            {
                56628, 40392, 33620, 44816, 15032, 20932, 11148, 3072,  44036,
                34856, 5444,  40172, 13224, 22912, 8668,  28840, 55556, 39020,
                31832, 16832, 36708, 21900, 58836, 50808, 7712,  56672, 18212,
                33948, 26828, 63028, 46288, 29708, 56196, 39896, 47920, 19660,
                36116, 49764, 17340, 39556, 47444, 58448, 40940, 6148,  35988,
                57920, 4516,  52232, 1736,  37272, 6360,  46760, 42488, 15056,
                56656, 19672, 50052, 272,   17448, 8496,  58276, 3856,  38672,
                12404, 31288, 16156, 49792, 64676, 14184, 25056, 38716, 61520,
                12480, 52760, 31848, 9272,  27496, 54012, 43080, 31528, 45608,
                6896,  17580, 58328, 39456, 29624, 62692, 24328, 50144, 32404,
                112,   60372, 51396, 11120, 58284, 42348, 61028, 9944,  55812,
                29836, 304,   8976,  56540, 14944, 53000, 7452,  44716, 2128,
                26420, 51692, 39892, 1148,  22728, 45832, 62884, 10440, 53596,
                29488, 56256, 11572, 42876, 2520,  23500, 33536, 11108, 44132,
                25300, 65512,
            },
            {
                52992, 22372, 59700, 26120, 62684, 52848, 30092, 58628, 17696,
                47372, 20520, 62644, 31944, 42496, 52504, 60368, 47476, 13940,
                61256, 50496, 10028, 63820, 39932, 25292, 32108, 776,   13132,
                49528, 9552,  57600, 1520,  35640, 24300, 15476, 53444, 688,
                41640, 26732, 30440, 4552,  24056, 9196,  14332, 19056, 29268,
                48104, 21784, 33864, 25160, 57564, 18552, 60704, 21652, 52252,
                36216, 11156, 26624, 59044, 48136, 40436, 14488, 45008, 20176,
                63676, 788,   56928, 37304, 41464, 6304,  46660, 20396, 4640,
                28320, 40524, 57056, 20164, 35288, 59352, 15708, 21864, 60844,
                48976, 33436, 27448, 11516, 46864, 15540, 55052, 38100, 41932,
                19792, 25940, 35132, 21084, 48044, 16424, 32992, 20436, 41176,
                34324, 49752, 24952, 38520, 32184, 49252, 60532, 28612, 55244,
                9960,  43976, 64260, 17356, 52696, 38144, 7612,  24296, 41152,
                45048, 4512,  60488, 37096, 14712, 46256, 61652, 7548,  51248,
                36316, 1296,
            },
            {
                8736,  37876, 12864, 47048, 6052,  32960, 49212, 23776, 38600,
                8932,  25776, 58024, 15920, 4412,  36036, 19268, 10756, 27012,
                6832,  57368, 19076, 3292,  44356, 11364, 62640, 28728, 53484,
                39064, 23280, 30984, 51360, 19068, 10104, 65432, 34180, 22572,
                60908, 8820,  56076, 62880, 43308, 34512, 53136, 56380, 2404,
                65280, 17268, 40440, 14036, 44628, 9300,  27188, 1300,  31204,
                5292,  62032, 43040, 33100, 21432, 28944, 60940, 24600, 49296,
                35536, 27108, 9180,  23832, 3016,  29900, 56096, 32908, 43696,
                63904, 1280,  14396, 49872, 3560,  6488,  39476, 28976, 2140,
                13964, 41712, 53868, 6472,  60388, 35384, 2960,  13204, 58972,
                9480,  65300, 45220, 3928,  37508, 63472, 7344,  25604, 53488,
                5492,  13328, 43708, 61776, 2344,  18100, 23720, 40540, 15964,
                36904, 20460, 6964,  33696, 12868, 27480, 32204, 49652, 2068,
                16636, 22356, 32080, 50696, 18660, 57464, 26824, 40900, 20116,
                13784, 29016,
            },
            {
                49488, 17812, 2732,  55436, 19508, 42408, 3988,  61436, 54748,
                280,   50132, 29828, 53864, 45216, 24684, 1008,  34556, 41620,
                22408, 45548, 29788, 33624, 55032, 15808, 38288, 20812, 45724,
                60928, 16164, 7128,  40812, 59532, 47020, 3188,  29084, 12256,
                45388, 15108, 52064, 21072, 1152,  48948, 25948, 32140, 38280,
                45256, 10012, 59400, 30580, 49672, 55544, 35392, 40620, 48860,
                55012, 23956, 14728, 4004,  54032, 10708, 1876,  32500, 7168,
                17288, 53944, 47344, 62276, 18692, 52076, 60552, 11884, 8752,
                48668, 18404, 25908, 44888, 63132, 24360, 52012, 57644, 36060,
                25304, 64928, 3832,  23296, 42996, 31176, 21208, 44776, 27732,
                53756, 17624, 13616, 56348, 27084, 50164, 12584, 46184, 38768,
                62672, 57404, 21344, 28184, 46488, 6124,  34484, 12484, 59000,
                31524, 49952, 24808, 56588, 47756, 60172, 18764, 54752, 58716,
                35524, 48624, 8768,  28196, 5808,  38728, 1768,  54652, 32244,
                62300, 45696,
            },
            {
                23200, 41240, 30376, 64280, 35808, 9828,  16324, 28108, 13424,
                43360, 34048, 11392, 6676,  38252, 62240, 50884, 56020, 64716,
                2616,  38096, 53148, 8404,  22996, 48540, 59612, 6128,  35028,
                2796,  43064, 55508, 13876, 37216, 26352, 42316, 56968, 50740,
                37864, 5356,  31644, 36560, 11368, 16504, 61616, 6952,  12520,
                23836, 51636, 5440,  63124, 2856,  22748, 12688, 64444, 17576,
                9612,  38452, 47040, 35192, 65228, 42204, 51664, 56576, 61840,
                43372, 39324, 11524, 33596, 44032, 15384, 26576, 35696, 22840,
                58916, 37488, 55208, 30208, 41404, 12716, 46728, 18104, 10076,
                50328, 19948, 38052, 57352, 17392, 8808,  64012, 48392, 5880,
                33824, 40192, 6800,  31688, 22496, 832,   59632, 18780, 29312,
                2668,  16732, 36448, 10768, 54776, 65208, 42108, 51572, 728,
                45632, 5256,  61340, 42008, 10168, 3224,  39284, 6196,  26588,
                11220, 63804, 40424, 55260, 62424, 12672, 22216, 44912, 16328,
                5160,  56168,
            },
            {
                34840, 58456, 7788,  51128, 25376, 39252, 46540, 57768, 22324,
                37100, 64816, 18736, 59636, 21568, 12992, 27736, 16448, 31432,
                48284, 14756, 26136, 61916, 42468, 2124,  27540, 51028, 10420,
                25432, 64392, 33012, 21788, 4460,  61704, 17136, 6752,  19884,
                62420, 24848, 58492, 44452, 28240, 39040, 57264, 20532, 42112,
                55440, 35232, 27520, 20156, 37756, 43236, 29892, 4344,  44996,
                20852, 59432, 28328, 7492,  18860, 13164, 37228, 22380, 15100,
                4720,  29816, 21116, 57756, 6956,  38060, 120,   42368, 53332,
                5400,  32496, 10772, 7436,  21228, 33752, 61928, 7000,  55248,
                43864, 32432, 13076, 47212, 51488, 26056, 56120, 37052, 2168,
                23952, 49392, 61420, 43812, 54664, 41316, 35652, 52484, 9564,
                23860, 47748, 51908, 4088,  31120, 19976, 26280, 9176,  62752,
                18820, 28908, 14468, 35896, 21940, 29824, 65360, 43024, 14080,
                46016, 20024, 104,   24216, 34300, 47700, 30328, 59560, 36704,
                10152, 26304,
            },
            {
                396,   15248, 44028, 21292, 12160, 53604, 62256, 7068,  31604,
                51824, 4228,  47076, 28744, 41032, 48688, 8072,  44060, 20576,
                10436, 59288, 5796,  18364, 36132, 12880, 32212, 16860, 57852,
                19456, 47804, 28296, 49964, 54508, 11652, 30868, 48128, 33396,
                40604, 1856,  17984, 49916, 63748, 3112,  45940, 50928, 30780,
                1496,  15896, 46808, 8620,  53732, 14968, 57796, 25864, 52652,
                31780, 880,   55628, 50300, 25760, 46424, 30856, 8756,  26608,
                48376, 59912, 2548,  25476, 51276, 65368, 48108, 19728, 29368,
                16172, 45660, 59992, 51232, 16612, 916,   38316, 30944, 27168,
                2964,  62832, 7980,  29028, 680,   40696, 11696, 16268, 52644,
                29356, 57328, 19580, 10340, 28548, 15424, 5368,  31968, 64768,
                41968, 33496, 61324, 40128, 15264, 58200, 44308, 37532, 22976,
                55776, 39424, 52488, 1476,  46692, 16500, 50724, 23140, 33472,
                59724, 30432, 52512, 43500, 15604, 9380,  53028, 3208,  42256,
                51524, 63640,
            },
            {
                46956, 32420, 60696, 3784,  28812, 33728, 1580,  20300, 41248,
                9536,  25348, 15704, 56460, 2204,  33372, 58720, 4900,  54432,
                40640, 34148, 51552, 46532, 56260, 65268, 40712, 52988, 44664,
                37996, 12632, 3404,  8704,  34820, 44852, 22988, 53528, 9356,
                27296, 55308, 13136, 7740,  23392, 33080, 9716,  14804, 25568,
                62072, 58020, 40288, 32192, 60428, 48192, 6876,  34932, 61556,
                12140, 41996, 16444, 60832, 40052, 3064,  62508, 50832, 40788,
                34504, 54916, 13636, 46220, 30812, 9044,  12684, 56508, 62756,
                2324,  39192, 23168, 64904, 43008, 56756, 48888, 22532, 14628,
                39528, 58132, 24368, 36320, 61276, 33512, 20172, 62996, 43264,
                7716,  14800, 37896, 2508,  63828, 47996, 21548, 56500, 45316,
                13888, 7552,  26880, 22256, 50240, 6728,  13128, 33104, 3272,
                48188, 11208, 26484, 62512, 32312, 54156, 8564,  36992, 55972,
                3300,  17464, 37352, 6744,  57952, 20660, 65068, 24800, 17860,
                29292, 12048,
            },
            {
                54516, 18696, 40032, 49948, 56184, 17036, 44840, 49192, 14828,
                63080, 54400, 44672, 35980, 10524, 64076, 24392, 37660, 28968,
                63096, 23312, 372,   28036, 9832,  20648, 24564, 7660,  896,
                29696, 60752, 41756, 18196, 62956, 38964, 444,   59244, 15796,
                64156, 45992, 35000, 42380, 53696, 19044, 58608, 36328, 4472,
                48824, 22344, 11640, 18252, 720,   24484, 41300, 17016, 49660,
                37384, 8252,  23700, 33008, 5888,  21236, 54324, 17064, 548,
                64452, 19204, 42524, 36988, 16896, 22712, 40352, 33344, 25116,
                50152, 13932, 28132, 4840,  35156, 26324, 11308, 60720, 54228,
                45912, 18992, 49516, 15772, 54020, 4748,  45792, 31508, 22928,
                59168, 34668, 46816, 25168, 52008, 39352, 11112, 26204, 1812,
                18516, 57972, 54052, 464,   35632, 46796, 63328, 52940, 29520,
                17112, 58620, 7636,  43660, 19324, 57832, 12636, 5420,  27412,
                48468, 10300, 63320, 50456, 27980, 40740, 35568, 6292,  49044,
                39164, 21556,
            },
            {
                37132, 9500,  25120, 6404,  65232, 37568, 24236, 57220, 27796,
                38496, 924,   30552, 22488, 50792, 18004, 13748, 52440, 3248,
                17116, 11852, 44152, 38512, 30888, 4140,  47180, 58712, 35556,
                22248, 55348, 51116, 24844, 57052, 14148, 29336, 43156, 36720,
                3956,  21156, 30076, 60920, 4980,  40416, 27820, 64884, 43508,
                7972,  37648, 52880, 45324, 29088, 64792, 10080, 21920, 2480,
                28864, 64196, 45720, 13832, 58248, 36368, 44040, 12428, 28872,
                23864, 10344, 5392,  61876, 53540, 58256, 4360,  44256, 7328,
                55296, 36208, 47680, 9008,  53272, 18460, 3264,  42016, 6460,
                34436, 9948,  2424,  43068, 22012, 59520, 9628,  50168, 1284,
                39652, 12204, 61744, 4500,  33176, 17156, 59864, 36140, 50536,
                30880, 38504, 11492, 43196, 28708, 18972, 24324, 4924,  60912,
                41696, 33748, 22384, 37500, 4272,  25548, 39936, 45188, 62016,
                21428, 41924, 25444, 13020, 32340, 1292,  46092, 14124, 61220,
                57700, 4324,
            },
            {
                63156, 28204, 47432, 14056, 42492, 10384, 3552,  33040, 12316,
                59468, 19556, 8128,  61016, 39508, 27392, 46820, 42220, 31936,
                56100, 48632, 59840, 15364, 61700, 54468, 33284, 13920, 63560,
                9580,  15764, 31976, 6880,  46444, 20336, 5968,  52300, 24592,
                49028, 10508, 51448, 25388, 15520, 47124, 11964, 51688, 20840,
                33416, 27416, 62724, 6172,  35840, 55264, 32600, 52068, 57420,
                43416, 19224, 53752, 49096, 27380, 9280,  31540, 56232, 47152,
                37992, 51500, 31972, 26160, 1688,  35052, 28060, 61452, 17804,
                11104, 41820, 20632, 58708, 30444, 45600, 63660, 32524, 21200,
                28084, 52624, 65412, 30732, 39004, 14028, 28284, 56352, 17728,
                26788, 49132, 21000, 30212, 8548,  54448, 42340, 6164,  61688,
                21064, 48052, 64200, 15908, 59324, 55364, 10096, 38640, 14352,
                46056, 2728,  50196, 64436, 15592, 48224, 60292, 30836, 14512,
                33896, 54520, 2740,  46896, 60156, 18940, 55680, 26376, 8888,
                31112, 43612,
            },
            {
                1420,  52092, 58684, 35160, 21676, 30308, 61240, 52500, 5396,
                46436, 35224, 42852, 53892, 2924,  6004,  57816, 21232, 7700,
                36100, 26384, 19996, 6440,  41484, 50004, 18496, 27388, 43836,
                49032, 40632, 2252,  37012, 27544, 65092, 34256, 12392, 61164,
                17712, 39804, 57320, 2164,  38140, 55924, 31756, 616,   16688,
                55044, 13592, 2552,  42424, 19608, 14492, 46580, 39088, 13172,
                25412, 4764,  34996, 1320,  39216, 63276, 18916, 4632,  60776,
                7684,  58528, 45248, 20412, 49832, 14444, 47172, 21448, 51580,
                31312, 63848, 664,   15644, 39588, 23840, 12744, 51292, 16144,
                60068, 37180, 12280, 25464, 48352, 6096,  35836, 42344, 64544,
                5672,  53508, 41060, 57960, 45812, 23684, 14132, 28780, 44196,
                9112,  25344, 3796,  33732, 7408,  41088, 31248, 49532, 27028,
                19816, 55144, 12368, 28488, 52872, 35036, 524,   19908, 52164,
                4664,  38412, 57520, 23032, 36692, 10760, 42160, 51896, 34576,
                16160, 23508,
            },
            {
                12148, 32860, 16788, 3084,  55276, 48308, 18816, 39824, 22688,
                50208, 25800, 16320, 12612, 48300, 33952, 61664, 10880, 45156,
                64368, 2096,  51132, 34312, 23704, 11124, 1604,  38156, 5204,
                25572, 19564, 62392, 54772, 9880,  48532, 41156, 1720,  30976,
                44408, 7356,  33572, 19844, 63056, 8632,  23060, 61112, 40876,
                46792, 60044, 24044, 50348, 58056, 26200, 3440,  7600,  61992,
                31088, 60132, 11360, 22908, 15880, 50976, 25012, 41744, 34724,
                22132, 15172, 39472, 11396, 64192, 41392, 9460,  38336, 2784,
                57040, 25636, 34136, 49056, 61800, 4212,  36508, 56636, 1144,
                40932, 47108, 5060,  55704, 62308, 19560, 52016, 23584, 33032,
                15516, 36152, 10100, 18280, 940,   64720, 34420, 3032,  56296,
                16888, 53092, 37200, 46236, 22992, 51576, 1352,  64996, 35484,
                8428,  60100, 31760, 42916, 7960,  23360, 11252, 44288, 64224,
                8800,  18260, 30180, 15416, 7500,  64380, 29124, 21132, 5300,
                46568, 56388,
            },
            {
                20120, 41200, 45168, 62540, 26708, 7880,  43932, 15676, 64512,
                9936,  56488, 31896, 63480, 29228, 18408, 37952, 24652, 16108,
                29936, 39772, 13388, 58464, 28808, 64160, 45392, 57412, 52516,
                59900, 34700, 12696, 43608, 17512, 58868, 22628, 53452, 57956,
                27488, 64480, 13844, 47648, 26996, 36112, 44536, 5528,  29388,
                10040, 38364, 17540, 31472, 9332,  63196, 36892, 20772, 49748,
                17420, 40948, 47712, 54616, 59404, 44896, 2612,  12964, 52792,
                28192, 1064,  54596, 29732, 6736,  55548, 32816, 59756, 23556,
                13648, 45176, 5952,  54216, 10584, 28368, 43036, 48164, 25900,
                8984,  23056, 18232, 33468, 424,   44932, 8468,  11788, 58876,
                2956,  43848, 62284, 28484, 38420, 48564, 20192, 51348, 39904,
                31804, 62904, 13504, 27816, 61232, 17432, 11952, 21660, 56940,
                39744, 24988, 1572,  18116, 37944, 58304, 55016, 29128, 24616,
                41472, 51140, 61848, 43540, 48504, 53180, 3700,  38772, 61980,
                49400, 36672,
            },
            {
                64640, 5628,  23876, 10916, 38472, 53348, 80,    34296, 28996,
                3372,  37776, 6376,  21456, 44360, 55700, 520,   49436, 54524,
                59344, 4716,  46640, 53024, 7984,  36160, 21612, 16356, 29636,
                8256,  23188, 50068, 30500, 4276,  26272, 14308, 8436,  35920,
                20964, 4660,  52520, 42096, 11476, 54248, 18088, 50732, 24984,
                57652, 34200, 6892,  48016, 40056, 53464, 29376, 45124, 55796,
                33676, 5760,  27776, 36624, 7836,  29560, 33052, 64532, 47948,
                17516, 62376, 35784, 43280, 19000, 25408, 16440, 5216,  42688,
                49808, 29244, 38160, 18096, 22540, 58068, 7708,  19568, 31264,
                64056, 53312, 43872, 14864, 58532, 30132, 40236, 54876, 46432,
                31012, 24700, 50644, 12840, 56732, 25912, 7296,  60612, 11816,
                22520, 5564,  42280, 2416,  55856, 33152, 43664, 47736, 4164,
                15428, 45332, 53660, 63704, 47616, 5072,  40092, 16536, 48864,
                34824, 12268, 26600, 1088,  33532, 24264, 17528, 57260, 13516,
                25616, 8388,
            },
            {
                53536, 28972, 49968, 59620, 31708, 13456, 57828, 24928, 60180,
                45276, 52908, 41132, 59548, 9204,  26736, 14012, 41376, 32952,
                9760,  20416, 25812, 17748, 42420, 4076,  48360, 32704, 11892,
                42300, 76,    53916, 38352, 60916, 33628, 40044, 45784, 16552,
                50188, 38448, 24384, 30556, 60240, 1844,  32676, 62608, 13244,
                3268,  52748, 65356, 22340, 188,   15780, 12076, 23992, 2080,
                10264, 65060, 21528, 42716, 14064, 19676, 56608, 6560,  39988,
                9752,  46020, 23880, 3708,  61048, 48288, 37000, 53372, 65132,
                19276, 8228,  61300, 41644, 33192, 52352, 15480, 35420, 59016,
                13108, 38864, 27952, 36204, 50160, 21880, 26336, 16256, 61192,
                20364, 7908,  54352, 3996,  32532, 15868, 44676, 36528, 27236,
                47000, 58580, 50268, 20152, 38968, 7912,  26364, 62336, 30036,
                51252, 33888, 10372, 21760, 27148, 14320, 32316, 60188, 2468,
                6628,  58956, 20476, 56476, 40284, 11480, 45284, 30124, 448,
                42632, 32008,
            },
            {
                14720, 39656, 1868,  17636, 35972, 21008, 41880, 48604, 11224,
                19308, 14248, 24584, 2716,  34680, 52236, 64932, 6828,  22884,
                44244, 37244, 62548, 31488, 57104, 14644, 54720, 60800, 39632,
                63776, 45904, 15012, 20620, 7572,  47836, 2736,  56680, 62356,
                10944, 368,   55888, 17100, 6860,  22244, 47944, 37556, 42824,
                20348, 45068, 11624, 27888, 41984, 61148, 34628, 58928, 43108,
                51548, 16116, 57164, 804,   60884, 49324, 37380, 21740, 26560,
                59192, 32300, 50788, 13016, 57360, 31156, 392,   28096, 11488,
                34672, 55412, 1552,  13356, 45900, 63372, 2276,  50036, 44620,
                3324,  7028,  60872, 10672, 3816,  63812, 6684,  37516, 1752,
                48796, 39808, 35176, 22676, 41948, 62868, 10320, 53364, 72,
                18044, 34792, 9920,  30456, 52656, 59172, 14560, 36964, 5948,
                19168, 60624, 41720, 3640,  35644, 62360, 9816,  51640, 22404,
                37760, 46380, 31172, 14856, 5656,  63332, 59184, 35152, 50676,
                19144, 60356,
            },
            {
                9888,  21884, 55816, 47596, 4488,  63496, 51756, 6144,  31348,
                36256, 61892, 50040, 17012, 47396, 39060, 19376, 29384, 60236,
                53584, 1196,  49624, 10964, 39076, 23620, 1776,  27004, 19396,
                5912,  25176, 55988, 28884, 63344, 52124, 23636, 31324, 27824,
                43020, 33180, 61484, 45976, 39852, 57876, 8528,  15448, 26296,
                56420, 30692, 36192, 18356, 55664, 50156, 4340,  18984, 38004,
                26936, 30936, 46884, 34696, 25860, 11100, 44084, 3000,  55132,
                14668, 4780,  20444, 38592, 8492,  44700, 14984, 40840, 21328,
                46648, 26388, 58912, 31164, 24284, 6136,  39700, 22068, 29308,
                55432, 24628, 48764, 20568, 56480, 42192, 33208, 52736, 28244,
                13800, 65140, 17456, 59308, 47360, 5284,  19644, 30912, 57396,
                41028, 65492, 25448, 16060, 46592, 23104, 996,   42528, 54984,
                24448, 12972, 29160, 50252, 55804, 43028, 19856, 45312, 28628,
                65320, 8644,  54496, 50440, 36968, 27092, 21020, 7828,  54920,
                4364,  46248,
            },
            {
                34100, 61604, 43092, 25068, 29984, 9188,  16744, 39416, 22376,
                1260,  56820, 28056, 32028, 12060, 57740, 4268,  46152, 12796,
                16580, 34076, 6244,  28476, 65532, 50932, 34648, 9772,  47308,
                57664, 32308, 36592, 10536, 17876, 41836, 12640, 19240, 5844,
                49148, 21656, 14428, 27096, 51148, 34828, 29116, 64180, 5668,
                49684, 2012,  60496, 47404, 6248,  25324, 32060, 46396, 8160,
                61368, 12564, 39532, 4976,  53340, 63044, 16532, 51284, 30288,
                35516, 42512, 64836, 27716, 51940, 22948, 63220, 58172, 50444,
                3624,  37788, 9844,  17488, 48600, 27872, 11428, 62228, 18456,
                34068, 15956, 40708, 31500, 13596, 45512, 18140, 9212,  58076,
                43712, 30652, 10060, 1160,  27772, 55656, 39452, 23308, 48652,
                13764, 7212,  44588, 3212,  61896, 11600, 34304, 49092, 64708,
                8592,  39200, 58396, 17348, 7752,  25180, 1716,  57392, 13092,
                41688, 24816, 18112, 42980, 2680,  47848, 15660, 41416, 24012,
                38120, 27564,
            },
            {
                52332, 7012,  37232, 12668, 57144, 44872, 27324, 54780, 65528,
                46060, 9680,  42244, 6968,  62888, 23780, 36400, 51220, 25684,
                39876, 61152, 22368, 45404, 17428, 6772,  43856, 53424, 37736,
                13476, 3456,  49328, 44140, 1272,  34608, 59452, 38188, 65328,
                53988, 8828,  36796, 2880,  12044, 19632, 53156, 41892, 11032,
                39312, 16288, 23904, 9432,  38756, 14500, 62844, 52932, 21928,
                2756,  55064, 23512, 19084, 8568,  32408, 40592, 23352, 60436,
                7368,  48380, 56080, 2224,  17564, 35628, 10632, 5984,  32280,
                16032, 52616, 43736, 64492, 54408, 36412, 56812, 42816, 8188,
                51600, 46608, 65020, 1436,  53812, 25600, 61588, 23020, 50768,
                4480,  25140, 45876, 37368, 50276, 12652, 34308, 63584, 2632,
                28684, 36984, 54952, 31860, 38664, 56964, 28404, 16240, 20596,
                44796, 32896, 184,   47472, 37820, 31000, 53016, 33612, 16924,
                3260,  35404, 61936, 29432, 9900,  53560, 32964, 61000, 11216,
                57012, 16704,
            },
            {
                2528,  19492, 31328, 50464, 1024,  60692, 33036, 13796, 4728,
                25196, 53088, 37644, 21188, 44168, 1524,  30728, 9896,  56996,
                43356, 3096,  54584, 13856, 58524, 31296, 20844, 61852, 29340,
                17024, 64748, 22696, 60216, 27184, 55336, 4908,  45428, 15792,
                24796, 41036, 57100, 63376, 44540, 4796,  60380, 22812, 31924,
                59200, 53644, 33932, 63856, 43972, 30088, 10644, 40848, 17056,
                35672, 49676, 64128, 42144, 47924, 27548, 1588,  12532, 45596,
                19420, 10284, 24824, 33584, 41592, 54412, 47952, 25992, 39328,
                62408, 23136, 6900,  33656, 20496, 92,    14980, 32348, 60468,
                2984,  37376, 9868,  28388, 36032, 5448,  38816, 12348, 34592,
                41128, 19444, 57124, 61880, 21820, 16252, 7472,  45140, 52364,
                17208, 61352, 21964, 49932, 18824, 6976,  41092, 52492, 3100,
                26028, 53708, 61628, 22452, 14508, 64064, 10824, 38920, 61236,
                49316, 6020,  56796, 13380, 23172, 63816, 45000, 1500,  29576,
                48336, 65148,
            },
            {
                25884, 45764, 62496, 15988, 23052, 37960, 19724, 42056, 49476,
                34604, 18384, 14464, 59628, 50556, 54880, 17872, 63508, 6648,
                20072, 32100, 37680, 47892, 26328, 40996, 12092, 2636,  24644,
                42140, 51944, 6856,  39644, 14536, 50672, 9788,  29812, 33648,
                1084,  47496, 28700, 17320, 32792, 25576, 47600, 764,   18120,
                44740, 28032, 4588,  20704, 51260, 952,   58652, 27140, 56648,
                44324, 7444,  29772, 15188, 56200, 59492, 37072, 54380, 63428,
                28652, 38924, 15456, 62576, 6788,  29804, 61012, 1228,  19524,
                56848, 29332, 13632, 41236, 4760,  50708, 46100, 23288, 26656,
                13208, 58148, 21972, 17848, 50060, 59692, 47248, 236,   63608,
                54188, 14568, 32640, 6276,  52956, 43000, 30240, 58952, 25072,
                9476,  42264, 12376, 4756,  26692, 63900, 10116, 31284, 60184,
                36808, 11932, 6624,  28080, 43964, 49844, 4952,  21888, 46516,
                26844, 20304, 51864, 32072, 39156, 18832, 6192,  36184, 20676,
                13708, 40148,
            },
            {
                33664, 58080, 10332, 41160, 5344,  52284, 8028,  57916, 11464,
                29744, 62668, 2248,  32924, 26664, 13320, 41508, 34580, 49220,
                27948, 11656, 62600, 8448,  760,   35864, 55380, 48880, 59144,
                9088,  34140, 20408, 31552, 47288, 25128, 19860, 58312, 51916,
                12272, 61756, 20820, 7264,  40324, 56312, 14104, 37784, 51928,
                8176,  12704, 57808, 36712, 15732, 23320, 47908, 5088,  32972,
                13044, 25492, 312,   33860, 10748, 21260, 5708,  16872, 33048,
                3932,  52668, 58548, 46428, 11812, 21576, 14860, 42444, 46064,
                11292, 36492, 47832, 60984, 27312, 58680, 37672, 7232,  53972,
                41692, 30488, 44436, 54884, 7556,  14920, 31008, 20956, 28620,
                8168,  47960, 3024,  36272, 27068, 10900, 37912, 4104,  19448,
                33256, 57216, 47732, 35708, 54288, 43860, 23368, 48180, 14636,
                46108, 19040, 40892, 57080, 35168, 18076, 59128, 29864, 55772,
                8504,  36476, 44236, 260,   58008, 48808, 26196, 59552, 51984,
                43804, 8324,
            },
            {
                21732, 49300, 29204, 54336, 35208, 47228, 26564, 61348, 3432,
                39732, 55948, 47888, 8992,  39128, 5176,  46844, 22820, 15028,
                59360, 52876, 24016, 18576, 50680, 64304, 15948, 5616,  38588,
                45440, 56872, 12228, 61796, 2368,  36828, 63476, 40008, 22860,
                43552, 4416,  35876, 53368, 49708, 10172, 65036, 34352, 26716,
                62628, 41148, 48544, 30960, 55492, 39688, 64824, 19628, 37884,
                60020, 51352, 61960, 38348, 45064, 51884, 25368, 41048, 49720,
                43756, 22476, 492,   31100, 50784, 37956, 57948, 28012, 8136,
                51452, 2384,  54832, 24504, 9308,  31024, 17248, 64144, 20044,
                49428, 4800,  63140, 25104, 33516, 41412, 56032, 44340, 17352,
                58208, 40168, 24304, 63164, 18276, 55908, 61060, 47196, 51412,
                39412, 1816,  29676, 59920, 16988, 1176,  34744, 58048, 4020,
                29132, 64336, 51112, 2120,  9244,  25736, 40396, 1156,  15004,
                42288, 11736, 60708, 27924, 8064,  16020, 41748, 10548, 31576,
                56108, 3576,
            },
            {
                38780, 13988, 1932,  24968, 64116, 17328, 31888, 45372, 21532,
                15612, 44380, 23748, 19864, 63908, 28984, 56156, 28,    38220,
                44576, 3500,  41432, 33476, 44116, 28680, 23044, 32384, 18724,
                25680, 4496,  28332, 53804, 17964, 44732, 7780,  3316,  16524,
                55084, 31596, 59964, 15192, 23560, 29952, 5244,  20504, 46668,
                3848,  24316, 18968, 2824,  7964,  12172, 28624, 45808, 9120,
                16280, 22620, 47480, 18240, 4192,  65344, 30364, 9416,  60256,
                13096, 27104, 36644, 18416, 43252, 5580,  53240, 35024, 22556,
                65468, 32472, 19100, 15128, 44696, 52836, 11936, 40444, 1348,
                34988, 15872, 11192, 39148, 3448,  62068, 10092, 26288, 37632,
                51084, 11728, 30512, 44636, 49468, 1100,  22320, 13548, 27656,
                64016, 15752, 8392,  24624, 40376, 12872, 50452, 21216, 39268,
                8060,  24028, 33264, 54676, 13612, 62976, 45708, 53388, 33216,
                64852, 24460, 17180, 46768, 34480, 55188, 64508, 4264,  23476,
                16556, 63152,
            },
            {
                59804, 44648, 18876, 57500, 8628,  13176, 288,   55212, 36844,
                28556, 6484,  52712, 35360, 58468, 11332, 49956, 19104, 65124,
                30628, 6260,  57940, 14528, 54240, 10120, 59668, 46928, 12952,
                63336, 36456, 49372, 40364, 9988,  57064, 27588, 34720, 49448,
                25872, 8196,  38704, 46164, 1404,  43684, 58444, 53836, 39832,
                15388, 60208, 52796, 44952, 62368, 34728, 51840, 1628,  54764,
                41196, 3308,  56868, 28424, 42640, 14196, 57400, 35680, 20104,
                6696,  64416, 55876, 9148,  61792, 25512, 2708,  60284, 16912,
                39696, 43628, 5648,  62000, 38992, 2888,  48268, 32928, 60592,
                27252, 56632, 46188, 51948, 18784, 23412, 48656, 2620,  64952,
                5644,  21968, 59496, 15300, 8708,  31728, 41684, 35212, 5536,
                44240, 21852, 52640, 45944, 63448, 32812, 7084,  53852, 62564,
                43272, 15564, 47604, 20440, 30608, 37312, 23072, 6300,  19316,
                50340, 38476, 4776,  52664, 22080, 12712, 29380, 38080, 49912,
                34984, 27016,
            },
            {
                11260, 52240, 36696, 30232, 43284, 40152, 51320, 24472, 64840,
                10036, 59952, 41324, 3660,  16336, 43248, 32644, 25108, 9072,
                51936, 26428, 36764, 21488, 6888,  39400, 2500,  51268, 41312,
                54472, 1612,  15420, 22560, 32664, 51536, 14016, 59576, 41452,
                64796, 19128, 12648, 27484, 61952, 33560, 17604, 11688, 28436,
                36232, 9288,  32588, 26360, 42076, 16916, 24040, 60784, 31676,
                26584, 36440, 11228, 32888, 7948,  23496, 48592, 3136,  46472,
                53412, 40388, 15916, 33704, 48472, 20636, 13900, 31352, 49784,
                11904, 26344, 57588, 35320, 21680, 28384, 58920, 23772, 43384,
                8876,  21452, 36840, 28688, 60200, 13668, 53692, 35764, 16304,
                32868, 42708, 2156,  55204, 37332, 64560, 25424, 54112, 9864,
                60384, 32224, 37280, 3060,  19880, 57008, 27432, 17396, 30988,
                11200, 58504, 38408, 3192,  59924, 49716, 11496, 57900, 28772,
                2604,  58904, 31416, 62248, 40404, 1944,  44492, 19748, 54532,
                46712, 7136,
            },
            {
                31384, 23260, 5532,  48776, 61708, 20828, 33440, 4568,  18572,
                48692, 13948, 31064, 25924, 51768, 7236,  37288, 60196, 14204,
                46688, 17972, 61228, 48748, 31080, 63084, 27080, 20472, 34384,
                9424,  30120, 60332, 43676, 64240, 600,   21036, 30276, 10828,
                5144,  47876, 57544, 52420, 20940, 41400, 7160,  49996, 63880,
                240,   56564, 21056, 13536, 58732, 5512,  48200, 20788, 43724,
                13896, 63744, 52148, 20376, 60560, 54860, 39188, 17044, 32512,
                29236, 24128, 44948, 4316,  28676, 56832, 41680, 46940, 7124,
                55600, 1396,  30316, 50284, 7432,  54368, 18396, 5092,  14236,
                50216, 65256, 6600,  1824,  32476, 43124, 7616,  29348, 46468,
                61004, 53048, 27696, 20324, 46848, 4812,  16124, 58476, 19124,
                48440, 12724, 56228, 26156, 10360, 42644, 46972, 37596, 772,
                49564, 26036, 6008,  44724, 27308, 7736,  16836, 43532, 35956,
                47964, 10784, 20052, 7528,  25584, 51296, 60344, 8856,  14452,
                556,   42068,
            },
            {
                55844, 63960, 14748, 2792,  27160, 10664, 56640, 46224, 38536,
                54004, 34492, 1696,  62472, 46292, 22580, 57060, 4000,  40628,
                34112, 10952, 2008,  55368, 12384, 45584, 16740, 6208,  57604,
                24008, 48156, 18464, 5208,  25452, 47740, 37900, 54528, 45088,
                23652, 35528, 31052, 3532,  9804,  36972, 55448, 22444, 31244,
                47516, 41040, 51192, 37752, 2376,  29756, 35960, 10484, 6500,
                57864, 48516, 4884,  45588, 34768, 1276,  26572, 63600, 10196,
                58752, 1784,  51392, 12340, 39016, 64108, 9916,  37024, 23480,
                63060, 20536, 46384, 13300, 64524, 10512, 45452, 36464, 55196,
                31252, 40732, 17544, 47676, 57640, 38636, 20148, 56376, 12552,
                24696, 9312,  35276, 40312, 12012, 52000, 28980, 43440, 156,
                39676, 29768, 6616,  50200, 62056, 15068, 4752,  23252, 61428,
                35180, 56028, 18032, 65424, 34056, 40260, 52108, 62636, 25984,
                14648, 41772, 55952, 45804, 15240, 36648, 32320, 57724, 28452,
                62268, 20980,
            },
            {
                17184, 39396, 45476, 54456, 35776, 59652, 15252, 29352, 7412,
                43316, 21348, 55640, 18084, 39364, 12276, 29680, 19692, 53992,
                62736, 27912, 43256, 38404, 24488, 35520, 52140, 42392, 65276,
                14268, 38724, 55216, 35564, 12512, 42064, 7208,  16180, 62760,
                1996,  40176, 50596, 16904, 63564, 26312, 45648, 4580,  13048,
                24992, 17388, 7104,  65164, 45352, 55524, 52372, 62776, 38876,
                18884, 24864, 30404, 16572, 40688, 12748, 50140, 21100, 42948,
                14732, 37140, 62616, 22880, 53624, 17520, 636,   26900, 52132,
                33856, 42544, 17080, 37612, 24944, 41344, 29412, 62620, 456,
                26068, 12244, 53472, 24180, 9724,  26640, 63780, 540,   40992,
                49848, 17936, 6076,  57088, 60712, 33368, 21032, 50308, 35872,
                23576, 65212, 16944, 40872, 34412, 29224, 58800, 52260, 12932,
                20216, 41380, 9776,  53140, 13304, 21876, 384,   31792, 5264,
                53700, 23176, 34292, 28164, 63456, 3468,  17800, 23872, 40532,
                34208, 49232,
            },
            {
                26464, 8328,  32912, 24164, 19172, 41364, 1312,  63360, 25556,
                11520, 60728, 28212, 8460,  48968, 64176, 1164,  44212, 49416,
                23148, 5568,  58192, 19388, 8416,  61900, 316,   28636, 32216,
                3320,  44660, 7664,  27808, 57660, 61856, 19576, 28596, 33668,
                56684, 13088, 22040, 43852, 58536, 14688, 32980, 60296, 42568,
                62044, 34968, 28272, 11584, 23216, 18156, 14208, 27116, 33084,
                504,   42360, 9436,  53780, 61908, 29044, 7288,  36240, 56472,
                5544,  47420, 19684, 7620,  31736, 35084, 59988, 44484, 15340,
                4160,  8964,  59072, 53204, 2072,  33548, 15048, 47852, 19572,
                59764, 44412, 33936, 62388, 14440, 51460, 45092, 5000,  22168,
                31392, 62748, 44804, 15016, 24972, 3360,  8916,  61772, 14188,
                4584,  44488, 55088, 21364, 2076,  49120, 38736, 7352,  43992,
                28144, 2216,  32088, 24160, 61080, 46280, 57204, 19552, 39184,
                8688,  60564, 1332,  50584, 11668, 42628, 47720, 5620,  53300,
                10448, 3972,
            },
            {
                60608, 57092, 12500, 50712, 6156,  47156, 52356, 32468, 16920,
                50352, 4680,  41652, 36568, 15336, 32048, 26292, 35996, 8164,
                16260, 32716, 50172, 30012, 14704, 47864, 22552, 56072, 11508,
                49732, 59940, 21664, 51872, 31724, 2720,  49584, 52912, 9064,
                25676, 60856, 6176,  29552, 38308, 1096,  53360, 19440, 8928,
                52184, 2996,  57624, 49768, 32256, 40204, 7952,  44052, 50544,
                56288, 64752, 37324, 22020, 3968,  58848, 46656, 52348, 24888,
                33520, 60868, 27972, 44020, 49312, 4968,  55980, 20080, 30724,
                61472, 40096, 27712, 48920, 5384,  57712, 22436, 52312, 9524,
                38904, 4708,  21728, 2288,  37256, 30204, 16756, 34280, 58544,
                37712, 52404, 1600,  30184, 49100, 37108, 41420, 56596, 26244,
                52212, 32884, 8540,  60444, 11984, 25000, 18536, 32536, 54740,
                63788, 47188, 50812, 37320, 3892,  29032, 11264, 49444, 63888,
                44396, 29924, 18224, 57300, 38272, 21708, 55376, 64624, 13436,
                45932, 37164,
            },
            {
                28876, 900,   42924, 65396, 30096, 10208, 22508, 37316, 56880,
                45656, 33540, 23420, 58892, 53320, 5868,  56044, 60580, 13168,
                39572, 64936, 1540,  54840, 41436, 58888, 6908,  39740, 18756,
                25540, 34052, 16000, 10612, 39716, 23448, 14384, 36676, 46084,
                41268, 17560, 53976, 47592, 8576,  24200, 48996, 28924, 36536,
                15332, 46508, 20424, 42808, 1288,  61580, 59012, 4072,  21272,
                15776, 12016, 49264, 26332, 44112, 14476, 19048, 31096, 60,
                17172, 39816, 10024, 58508, 24536, 13444, 39228, 10588, 46672,
                54172, 12896, 21840, 32052, 17968, 61360, 42712, 6936,  35116,
                56444, 28756, 50292, 58300, 41736, 8204,  54632, 47788, 6844,
                13576, 26616, 42200, 21360, 64344, 17440, 46348, 30852, 11372,
                18420, 47392, 27600, 36792, 53492, 46316, 62652, 10008, 16052,
                22664, 6408,  14216, 59280, 17732, 42036, 34400, 25208, 15876,
                36104, 12892, 48456, 4964,  26180, 9164,  34904, 30644, 25020,
                19384, 51700,
            },
            {
                15164, 21924, 36172, 17532, 55444, 39464, 62172, 3416,  20248,
                13396, 64912, 420,   10844, 45032, 18592, 41228, 24132, 47328,
                52860, 21320, 45060, 10860, 24880, 34424, 16432, 45348, 53416,
                37460, 62656, 1092,  47112, 60780, 43832, 5024,  64324, 20652,
                56,    31332, 35636, 65404, 12084, 41952, 56948, 3472,  40332,
                59248, 30940, 25632, 9836,  53548, 35324, 29420, 25112, 46364,
                35788, 31196, 2440,  54976, 33364, 9104,  38444, 65172, 11712,
                54948, 50404, 2644,  15740, 41872, 29648, 65028, 50260, 25284,
                1672,  36540, 64228, 7924,  46076, 37356, 28256, 13588, 24388,
                64556, 16400, 11576, 45916, 19352, 61200, 23396, 27920, 64908,
                19452, 9956,  55224, 59020, 11608, 6548,  54140, 2056,  63296,
                38572, 57736, 2664,  14964, 41212, 30540, 412,   42760, 36340,
                57428, 39924, 30452, 44508, 8776,  56140, 5852,  54092, 2764,
                58688, 22760, 41208, 31592, 61336, 51880, 15652, 44232, 2560,
                61812, 41052,
            },
            {
                5372,  59008, 45360, 25428, 2600,  14700, 27768, 44148, 7332,
                53724, 26392, 48512, 29752, 21736, 51256, 2236,  10204, 30272,
                34976, 3824,  27280, 37592, 60788, 4144,  31256, 63836, 9292,
                4836,  42836, 30172, 54728, 17996, 26188, 32652, 58728, 11760,
                56264, 49980, 4744,  26740, 21096, 33924, 16192, 62712, 23084,
                12388, 5364,  63792, 38516, 16100, 19720, 12488, 51584, 57484,
                5836,  60136, 19468, 39912, 62328, 24340, 56988, 42480, 28432,
                45148, 23436, 63548, 35376, 52468, 3600,  21648, 7416,  34488,
                18316, 43176, 56408, 26348, 11404, 55512, 3104,  50604, 45492,
                40468, 32492, 6232,  25708, 35316, 3380,  11160, 43656, 38868,
                50688, 35388, 3788,  32996, 39308, 28344, 23924, 35428, 21588,
                7316,  42784, 24244, 64216, 20428, 6704,  59208, 49900, 26968,
                3644,  52036, 20388, 26048, 64448, 48060, 21024, 38844, 28536,
                51388, 7292,  65204, 45212, 19604, 196,   58984, 39340, 8288,
                56204, 33252,
            },
            {
                50564, 30576, 9220,  48420, 60724, 34888, 49896, 58808, 31400,
                40464, 35916, 15960, 61528, 38884, 33784, 63800, 58364, 16848,
                6948,  62112, 18172, 53764, 13480, 46992, 51704, 21956, 27760,
                14580, 56396, 20688, 6204,  36004, 8760,  48328, 38852, 29020,
                23788, 43100, 14304, 57780, 44820, 51492, 6672,  27344, 54256,
                48236, 33384, 55824, 45416, 6604,  48584, 62728, 39232, 9608,
                41816, 27612, 45300, 7400,  16908, 50084, 3180,  6432,  20764,
                37252, 7200,  31816, 19152, 56400, 44908, 37816, 60660, 28796,
                58564, 5184,  51124, 15568, 40852, 20952, 63416, 31220, 19032,
                2192,  54964, 47992, 62604, 52800, 30776, 57480, 15144, 1308,
                59844, 45448, 25036, 48068, 14380, 51516, 44992, 58296, 15392,
                49544, 31628, 51812, 10668, 34952, 55576, 23524, 12888, 18800,
                34212, 60988, 11684, 1760,  36020, 31624, 13384, 61124, 45860,
                10616, 33852, 16596, 36948, 12116, 29180, 49420, 21356, 27368,
                47508, 11312,
            },
            {
                64316, 38900, 54480, 20796, 6284,  42228, 11592, 24052, 18424,
                9820,  57576, 3840,  43364, 7544,  13836, 27744, 37848, 44612,
                56248, 49364, 39840, 8356,  29600, 19888, 932,   35944, 59240,
                50108, 24572, 40616, 63972, 51272, 13532, 53544, 3128,  16656,
                8220,  62824, 37456, 18636, 1412,  60004, 38696, 9872,  43808,
                17372, 376,   21608, 28236, 58184, 23932, 31496, 1780,  17600,
                22828, 64232, 52744, 29996, 12936, 46832, 35292, 53520, 61304,
                14004, 59332, 47100, 12000, 26748, 8996,  17028, 53500, 14344,
                47260, 22592, 32880, 96,    48184, 35296, 5744,  53080, 10144,
                60600, 22672, 14068, 37868, 17384, 41180, 48980, 33640, 22000,
                29928, 17224, 8124,  63844, 20140, 820,   61560, 9876,  40584,
                27956, 60396, 4096,  17756, 48028, 28552, 44844, 39140, 64988,
                7280,  47576, 55004, 40764, 16372, 50964, 24256, 928,   18400,
                56752, 25496, 3340,  52476, 55808, 42276, 5612,  63700, 36772,
                16292, 24116,
            },
            {
                19148, 1680,  13756, 27180, 32488, 17108, 54788, 61804, 4876,
                44964, 51932, 20372, 25064, 55592, 49820, 3412,  20668, 25308,
                11800, 31840, 22724, 43604, 64404, 55868, 40264, 12036, 44404,
                32852, 2660,  11320, 45752, 28048, 22076, 60556, 41532, 55316,
                46744, 31752, 52180, 10428, 29296, 47352, 32220, 20260, 64644,
                35608, 41444, 52080, 10780, 36572, 3904,  44156, 53964, 33660,
                47940, 11648, 656,   36488, 59204, 22116, 32084, 26724, 17828,
                49152, 29764, 39524, 768,   62272, 33556, 48912, 2112,  41072,
                10412, 38764, 61548, 29428, 58944, 24184, 43420, 15996, 26672,
                34664, 42868, 29104, 404,   9512,  24708, 5864,  53776, 63116,
                4712,  56468, 41012, 53572, 31044, 43288, 33828, 18720, 2952,
                55792, 12700, 44128, 38200, 62296, 3020,  9240,  52480, 31300,
                15528, 27968, 22764, 45244, 59464, 7576,  63264, 43400, 30312,
                40856, 48744, 60572, 20872, 8948,  24444, 34388, 12812, 53840,
                3092,  42636,
            },
            {
                55916, 35768, 43780, 63076, 51424, 37504, 672,   47244, 34336,
                26720, 63284, 30572, 11568, 60424, 32284, 46664, 53828, 41328,
                24,    59808, 14832, 35364, 5840,  25436, 48036, 7840,  61184,
                18180, 38556, 57532, 16404, 34712, 968,   31148, 19160, 25804,
                35280, 3672,  22332, 40612, 62116, 24528, 14516, 4532,  50660,
                29640, 7732,  61676, 15408, 40452, 65236, 13736, 26928, 56860,
                38152, 15840, 25748, 42904, 55676, 9648,  63400, 1376,  41784,
                10292, 5044,  51808, 21988, 42780, 57668, 23972, 30928, 63020,
                27032, 7176,  52724, 17324, 12124, 8368,  62484, 49276, 39728,
                58840, 7392,  50796, 65224, 45172, 60288, 19868, 12548, 42884,
                26496, 10792, 36852, 23064, 5572,  12420, 54892, 24744, 47916,
                36176, 20556, 25960, 53772, 32196, 16016, 21636, 56660, 1052,
                42436, 37292, 9748,  3728,  29004, 33984, 12028, 53380, 36600,
                6120,  13728, 28356, 38748, 46520, 58288, 30920, 18168, 45152,
                61100, 28608,
            },
            {
                10608, 49552, 5468,  22800, 57636, 8584,  29532, 22032, 39096,
                13308, 48780, 1668,  36888, 40968, 18484, 8952,  15636, 64764,
                28752, 51160, 46296, 3488,  58460, 16660, 31620, 23080, 53664,
                5996,  29748, 48496, 8200,  61788, 38204, 44868, 11188, 64864,
                6348,  59052, 13696, 54924, 2724,  36164, 53096, 58224, 23604,
                13140, 56604, 25880, 32528, 18888, 50316, 8524,  20904, 60536,
                7112,  51100, 62852, 5180,  18904, 39448, 15132, 46232, 36212,
                57732, 25720, 55232, 15168, 37112, 6212,  12680, 18616, 55416,
                43752, 20784, 35624, 45888, 56944, 37808, 32336, 1340,  21104,
                56176, 11976, 18716, 32620, 55228, 27376, 35472, 38480, 47512,
                51784, 32832, 15208, 49000, 60352, 28540, 38044, 62796, 6356,
                30604, 65488, 8664,  4956,  41708, 59928, 35648, 48588, 25396,
                61740, 57908, 49496, 17808, 55984, 39492, 20360, 4080,  58408,
                22336, 62504, 33024, 1080,  15836, 62984, 3924,  51216, 39972,
                7348,  33120,
            },
            {
                59544, 15632, 30680, 46036, 12336, 40884, 65088, 15664, 59948,
                8096,  42268, 16932, 53032, 5704,  24036, 56856, 34496, 6420,
                37176, 10032, 19712, 27424, 52472, 37988, 62384, 13724, 42588,
                26064, 65456, 20528, 51532, 24908, 4672,  55968, 48076, 15216,
                50860, 43244, 27624, 48404, 17076, 44208, 8612,  39848, 45928,
                3168,  48484, 42828, 5684,  59752, 46556, 29948, 35712, 3352,
                45604, 30616, 22612, 33312, 49208, 27944, 4240,  52984, 23144,
                64132, 32720, 44188, 19348, 28372, 64680, 46116, 34936, 50924,
                4824,  13644, 65472, 3200,  27780, 18908, 54396, 14000, 30176,
                4176,  46328, 39012, 22432, 14404, 3204,  58004, 8432,  17852,
                696,   62008, 57344, 3596,  45328, 17460, 8560,  42124, 51180,
                16492, 45480, 57492, 22668, 13236, 27212, 44500, 11076, 5596,
                20228, 13336, 32064, 62744, 24776, 46644, 50124, 27500, 16484,
                44980, 50804, 7920,  43516, 53676, 36308, 21996, 9720,  57236,
                25404, 20612,
            },
            {
                52928, 39244, 61888, 3344,  25716, 19652, 49104, 53480, 3080,
                32580, 57316, 21140, 64212, 45272, 27348, 61448, 48312, 22420,
                43212, 62720, 55764, 33448, 42284, 9224,  49816, 4244,  35332,
                46572, 1316,  33680, 12624, 43116, 59416, 8812,  22960, 29464,
                37580, 20340, 33188, 7096,  64384, 28736, 19808, 60316, 33300,
                17572, 37656, 21280, 52576, 1620,  24080, 55420, 42612, 11248,
                18160, 39784, 57996, 9076,  54440, 44376, 60360, 30828, 18468,
                13220, 2200,  8108,  59816, 49804, 3696,  40216, 9636,  25580,
                59540, 32076, 49180, 24532, 40160, 5500,  43816, 47544, 63964,
                25380, 36064, 53064, 5496,  43728, 49928, 30872, 63832, 24156,
                29592, 40268, 20076, 25932, 34756, 52884, 22036, 58796, 13360,
                208,   39844, 33980, 48884, 52388, 1952,  63808, 16956, 39796,
                51512, 35260, 6508,  43168, 132,   14772, 8616,  65076, 34692,
                11004, 25220, 56172, 19932, 12224, 26644, 48928, 29544, 14672,
                47860, 440,
            },
            {
                43076, 27072, 18292, 36860, 56152, 33160, 6428,  27700, 44656,
                24176, 50916, 28784, 35284, 14120, 1056,  39048, 11716, 30464,
                2768,  16392, 24356, 12536, 1244,  21896, 28936, 19208, 59908,
                56696, 15684, 39692, 54300, 27932, 18476, 32272, 40692, 53352,
                592,   61532, 10580, 24976, 56912, 34900, 12208, 944,   26320,
                64072, 9268,  58768, 28856, 12112, 38840, 15588, 64000, 49984,
                26652, 61736, 13768, 2928,  37152, 21068, 11596, 6708,  50312,
                38972, 47336, 35512, 11232, 54292, 31960, 22212, 56960, 1832,
                38424, 16064, 42352, 10552, 51668, 62132, 22060, 34784, 10052,
                59580, 16876, 28516, 62704, 10960, 39980, 15736, 54064, 45796,
                6332,  13288, 43944, 10044, 64596, 2136,  47164, 32744, 27812,
                24364, 60828, 10848, 28760, 19256, 38068, 32904, 54540, 29880,
                45884, 26920, 53440, 21524, 37704, 59780, 31788, 41920, 54108,
                2040,  39664, 30660, 64308, 41476, 2348,  61248, 44072, 38304,
                65284, 34684,
            },
            {
                12588, 8192,  48424, 52100, 10308, 42732, 58236, 36244, 18628,
                10992, 38804, 3900,  9516,  55284, 31772, 51344, 19432, 59380,
                54376, 49140, 40292, 60544, 47292, 63648, 55324, 38832, 10976,
                30964, 7152,  22804, 62580, 3884,  36660, 13184, 63352, 5472,
                17300, 55536, 45856, 38848, 3872,  49404, 42324, 54088, 30228,
                50028, 15008, 35136, 45164, 62812, 32976, 6652,  22704, 1912,
                33768, 52816, 43732, 17160, 25848, 65312, 34620, 41668, 56760,
                25088, 61536, 21444, 41980, 25964, 14300, 61344, 17932, 47184,
                29596, 53672, 21276, 668,   56024, 29260, 16200, 7032,  51040,
                41520, 2684,  55648, 48396, 34032, 23580, 2228,  20316, 34864,
                59892, 55588, 50572, 37588, 30460, 15668, 40428, 5272,  55692,
                37208, 17692, 54068, 6808,  46024, 58212, 4604,  22916, 61432,
                1504,  9000,  64052, 11944, 29492, 55476, 3424,  21824, 48264,
                18332, 59436, 5268,  35188, 17048, 52552, 32772, 6872,  18804,
                4420,  23332,
            },
            {
                56724, 63232, 2108,  21980, 15344, 61292, 1480,  13496, 47880,
                62740, 58700, 16312, 46924, 60456, 21984, 41672, 4328,  45500,
                27044, 8480,  31160, 5956,  35832, 14388, 43488, 2320,  24840,
                52168, 41544, 49572, 9688,  47072, 57896, 43908, 50012, 26668,
                35200, 41928, 13572, 51732, 21460, 15700, 62264, 22588, 6816,
                55696, 41936, 4792,  25316, 18520, 53824, 41500, 57272, 47660,
                20160, 7240,  28920, 56416, 48756, 496,   46404, 16076, 28880,
                3740,  51608, 17104, 30280, 212,   48212, 35780, 43300, 12188,
                63688, 6584,  60120, 36936, 32936, 12300, 58496, 38056, 23548,
                31732, 13240, 20776, 7896,  26916, 58344, 51564, 42224, 9684,
                26252, 32292, 4036,  22472, 59536, 54276, 11824, 19980, 63460,
                50184, 3704,  42028, 64472, 15060, 25532, 10148, 43352, 14140,
                18836, 57112, 40316, 47460, 17484, 51068, 26080, 36820, 12980,
                29080, 46144, 23348, 49852, 8572,  57528, 24676, 11856, 59320,
                53608, 29920,
            },
            {
                44824, 33004, 39880, 28220, 46616, 31036, 23856, 40280, 54464,
                7020,  30480, 42752, 25268, 37480, 7108,  65332, 17128, 35456,
                13752, 38112, 58160, 18216, 26544, 53144, 32392, 17132, 45936,
                64112, 19784, 35888, 29868, 16008, 24544, 2804,  20004, 59676,
                8408,  23408, 29328, 60520, 31912, 9096,  37852, 47236, 33844,
                11708, 20420, 57628, 452,   50764, 9544,  27120, 36424, 12816,
                59976, 38088, 10532, 40600, 31520, 59368, 23940, 9572,  63128,
                33736, 7404,  45112, 57364, 64264, 5504,  52448, 8148,  23284,
                34000, 26952, 44012, 18036, 46176, 25320, 48768, 2016,  54120,
                44924, 64828, 37448, 42684, 17588, 62020, 5632,  36920, 65104,
                15044, 46872, 18248, 7856,  42476, 25524, 48876, 35104, 43428,
                9400,  30112, 21224, 32144, 35948, 50684, 40484, 59680, 48480,
                36956, 33112, 23892, 4880,  43716, 9528,  63644, 5756,  56552,
                62224, 10000, 38224, 14876, 28572, 45592, 20740, 40816, 47252,
                36752, 16204,
            },
            {
                20040, 5640,  59116, 53120, 35380, 4772,  64688, 20392, 27232,
                34544, 560,   52868, 19012, 49700, 28308, 10800, 52960, 23964,
                62928, 576,   50460, 44368, 22984, 4444,  9408,  58864, 27832,
                5480,  12356, 60152, 140,   55024, 33172, 38744, 10928, 31336,
                47840, 64296, 6444,  1336,  57496, 44064, 18324, 2284,  25468,
                40108, 61508, 46204, 38064, 30800, 60268, 16628, 4404,  29976,
                45872, 23232, 63464, 5432,  18948, 13280, 53176, 37984, 20492,
                54908, 14816, 36848, 10368, 39108, 19496, 28052, 40864, 56384,
                2832,  51052, 13864, 8472,  64592, 4136,  40240, 61524, 18544,
                9128,  29700, 57488, 284,   46920, 31800, 12120, 28312, 48828,
                1224,  39968, 61468, 51964, 29192, 752,   62060, 6720,  23516,
                14328, 59720, 48144, 984,   55156, 7596,  20000, 31216, 3172,
                26596, 50568, 15452, 60820, 34596, 20544, 39484, 15200, 33044,
                42796, 508,   54364, 60432, 3808,  36096, 63840, 980,   26856,
                9168,  51080,
            },
            {
                41220, 24932, 13772, 9352,  17644, 55528, 44056, 8476,  50844,
                14544, 22744, 63516, 12328, 4740,  33428, 56976, 46612, 30008,
                42820, 20936, 10112, 55864, 64968, 41020, 49188, 37584, 54660,
                33524, 23460, 43184, 51676, 18912, 65064, 45692, 56800, 52408,
                15608, 38208, 19512, 46704, 36128, 27524, 53256, 65260, 49728,
                15372, 28416, 8372,  22276, 13804, 48448, 43924, 65004, 52560,
                956,   15492, 50484, 55280, 35664, 4068,  43876, 49088, 1616,
                42172, 59728, 27020, 22400, 53820, 33148, 15356, 60400, 48476,
                19092, 38164, 57448, 30568, 49892, 20584, 34128, 27508, 14360,
                47956, 4520,  25132, 50632, 14808, 39500, 22636, 53516, 19264,
                56556, 24612, 36008, 12916, 57872, 16528, 39000, 31684, 57340,
                45420, 27052, 37908, 12364, 23688, 61308, 28288, 56132, 12600,
                65296, 8068,  54296, 1732,  30292, 57696, 45336, 52708, 22756,
                50236, 26768, 20264, 32396, 43144, 50824, 13720, 54208, 33992,
                60972, 3044,
            },
            {
                29208, 49744, 64020, 38184, 26524, 49036, 12220, 31836, 59972,
                46708, 40936, 35976, 55116, 44224, 15828, 39316, 2828,  7668,
                60076, 32708, 15688, 28112, 34364, 13052, 20008, 62052, 3232,
                15292, 48304, 39404, 6412,  26440, 13608, 7536,  22196, 27340,
                4308,  43096, 54160, 24764, 10132, 14116, 4628,  31092, 56748,
                5452,  36180, 54424, 63388, 33500, 6132,  24412, 18900, 34332,
                41100, 25788, 32732, 42912, 21416, 27164, 62136, 30788, 24636,
                12452, 31828, 50360, 2312,  61860, 46652, 24792, 3944,  30044,
                11152, 61216, 22260, 41308, 5812,  54980, 11172, 43600, 56840,
                36444, 21564, 60048, 34396, 55072, 8232,  60944, 42984, 5032,
                30704, 10108, 45292, 21152, 33720, 47580, 10460, 19340, 52600,
                2260,  17944, 51716, 62952, 44284, 16796, 47136, 5792,  39256,
                45036, 21720, 42052, 25820, 48984, 12192, 3608,  28076, 8244,
                17052, 39884, 64468, 10576, 25248, 18432, 7580,  30808, 22272,
                14960, 57596,
            },
            {
                10832, 45760, 21368, 344,   60700, 41580, 2884,  37188, 17088,
                56736, 3568,  9696,  30916, 58948, 26060, 62064, 19528, 13040,
                48096, 40220, 52224, 6216,  46824, 1712,  30100, 25260, 44524,
                9968,  58012, 28816, 62316, 34940, 49936, 40676, 1116,  35572,
                58540, 11844, 33432, 62944, 50664, 59484, 41792, 23256, 11092,
                43700, 17452, 51312, 2624,  42216, 57992, 38936, 10072, 55624,
                59164, 7496,  11868, 60420, 47424, 8692,  57708, 17200, 6724,
                64520, 47208, 18312, 6252,  43224, 12876, 8824,  55328, 34616,
                44348, 808,   26044, 35604, 15760, 62232, 23900, 992,   32260,
                52060, 40896, 10876, 18856, 28848, 3144,  46388, 15816, 34764,
                62708, 51172, 2588,  64220, 5876,  55644, 27840, 41648, 65180,
                34340, 5592,  40704, 8944,  32448, 2920,  35264, 52396, 18596,
                32556, 56920, 10776, 38260, 63404, 18376, 36364, 61380, 58632,
                2760,  34844, 48340, 5156,  55736, 59852, 40412, 47812, 62992,
                43720, 37296,
            },
            {
                55096, 32136, 7504,  57084, 29988, 19360, 24432, 63268, 5660,
                28888, 25484, 19620, 48240, 2244,  22164, 50104, 34320, 54500,
                24400, 3368,  58432, 22156, 38456, 56644, 51164, 17436, 35964,
                53688, 21312, 1836,  16804, 10304, 59608, 30168, 44616, 63612,
                49304, 17744, 29964, 6852,  21744, 38760, 34124, 19004, 48012,
                60668, 31812, 25236, 20112, 11628, 28912, 50056, 3852,  21512,
                46804, 28396, 18020, 2544,  38944, 14780, 51048, 40772, 35860,
                54176, 9604,  39348, 58932, 29392, 36552, 51612, 40164, 62480,
                17316, 47456, 54124, 9884,  51496, 44984, 37928, 58992, 7372,
                16512, 62556, 5764,  44352, 64504, 37344, 25764, 58648, 11484,
                41260, 27288, 17424, 37620, 43344, 23180, 2972,  49272, 13312,
                24408, 56440, 29584, 20692, 54444, 41760, 14292, 24704, 60548,
                748,   28472, 16428, 52208, 5404,  23228, 47056, 31880, 41812,
                54636, 12396, 44400, 15768, 28948, 37076, 3408,  11692, 26032,
                6368,  18936,
            },
            {
                62212, 15812, 35720, 43348, 51780, 14420, 34224, 53084, 45612,
                38616, 51316, 64440, 41540, 13652, 37608, 8904,  42412, 64536,
                29076, 36680, 17840, 63396, 10908, 43056, 60732, 7936,  40548,
                64712, 30952, 46088, 37792, 24000, 54024, 19624, 14760, 9184,
                25416, 39792, 2032,  56600, 16220, 45456, 148,   63440, 27848,
                3536,  9440,  40000, 47204, 55028, 16332, 62528, 31632, 13924,
                37740, 51620, 64248, 54348, 29876, 34284, 22564, 3124,  27440,
                44108, 20624, 33908, 25868, 16188, 65388, 19992, 23244, 7172,
                13368, 28532, 64204, 4912,  31588, 18288, 29048, 13292, 46112,
                25744, 30716, 48904, 23708, 13580, 52508, 32628, 19616, 49452,
                22940, 7704,  59256, 31844, 53924, 15124, 58820, 30544, 7320,
                37056, 60272, 11764, 49460, 25888, 64236, 57844, 9704,  47768,
                37400, 62524, 7216,  33968, 43828, 56316, 14660, 9924,  25040,
                19192, 30056, 62588, 22536, 52172, 45956, 20500, 56836, 34676,
                52604, 1472,
            },
            {
                24048, 48408, 27040, 4024,  10624, 47104, 58792, 8140,  21492,
                11196, 15040, 32848, 6700,  60628, 52540, 27136, 1108,  16272,
                11328, 44272, 7376,  27284, 32288, 15088, 23528, 3584,  27076,
                13712, 5232,  51436, 57028, 42456, 2968,  47548, 33056, 55720,
                21076, 60768, 42668, 48124, 27048, 8740,  54904, 13156, 37544,
                53612, 58660, 34804, 64272, 692,   36996, 25500, 45160, 60528,
                1700,  34896, 24020, 43940, 6112,  62840, 11040, 56768, 60508,
                14076, 864,   52084, 56148, 5328,  48836, 1872,  42960, 32592,
                58988, 39068, 19752, 42584, 58120, 2916,  49196, 65264, 20780,
                53904, 38524, 2044,  57252, 41964, 9656,  1512,  55896, 4684,
                38440, 53036, 46840, 11912, 25828, 8780,  39376, 62428, 21092,
                46996, 43012, 16136, 216,   38544, 6784,  30072, 22056, 42972,
                13120, 49944, 20680, 59080, 29736, 39672, 964,   65032, 51956,
                6592,  38508, 57444, 856,   9080,  32540, 65244, 16368, 49384,
                28716, 42128,
            },
            {
                12960, 38380, 53580, 64832, 22708, 39368, 1796,  31200, 43444,
                61544, 57076, 23272, 45240, 29512, 18500, 57824, 46904, 32112,
                60956, 50600, 54972, 48320, 252,   35620, 53432, 45544, 49464,
                34380, 60300, 20084, 8180,  12860, 28280, 61172, 6528,  37860,
                3792,  51308, 13760, 36028, 62584, 32152, 49992, 24440, 42188,
                16132, 21472, 7060,  14448, 43868, 22544, 8552,  53652, 5896,
                41944, 15756, 10240, 19596, 49068, 26104, 42388, 18864, 47564,
                30144, 63512, 23040, 45572, 11612, 30696, 57776, 26680, 53056,
                3076,  49868, 24720, 34116, 12164, 22736, 39736, 8864,  33956,
                4400,  60604, 15648, 34780, 20968, 47436, 29136, 60864, 43888,
                14524, 29472, 544,   62916, 19640, 50396, 45320, 34644, 17472,
                55220, 4476,  33076, 61956, 45744, 18340, 36452, 51032, 5200,
                31928, 25096, 45704, 11544, 4252,  26660, 48576, 21212, 33568,
                43136, 49656, 26896, 36000, 14176, 42516, 24872, 2628,  38928,
                9632,  59228,
            },
            {
                5056,  19940, 8320,  31536, 17364, 61068, 25528, 54996, 18368,
                4872,  35692, 116,   54284, 10248, 35172, 4424,  21800, 39512,
                5548,  19984, 23884, 41164, 61612, 19080, 58624, 30268, 11168,
                17884, 41752, 25588, 31908, 64168, 35804, 16624, 52096, 44048,
                27876, 31464, 9860,  23432, 3304,  19744, 59316, 5380,  30136,
                46332, 50588, 26764, 30768, 61108, 49048, 34004, 17904, 27724,
                50332, 58760, 30948, 56220, 228,   37392, 52924, 32608, 4668,
                36880, 8588,  41740, 15276, 35676, 39548, 17704, 46172, 11296,
                36760, 15352, 7764,  46880, 61516, 52400, 26940, 55912, 42168,
                12352, 28444, 51152, 7624,  63756, 39660, 16712, 24168, 33964,
                65500, 21088, 36388, 42464, 33196, 5248,  27332, 1232,  11136,
                25192, 51136, 28568, 21688, 53288, 13520, 56728, 2300,  59708,
                17608, 53680, 40520, 63692, 18808, 55356, 36512, 60324, 15936,
                2936,  11284, 18208, 59340, 47904, 54588, 6960,  60848, 45028,
                21696, 33316,
            },
            {
                54716, 46368, 58324, 41556, 50880, 12616, 36360, 49856, 28660,
                41204, 47928, 26892, 39952, 15496, 64004, 43556, 51380, 55928,
                14108, 35080, 29992, 8956,  13124, 38324, 5860,  21948, 63732,
                55568, 37408, 1072,  45192, 50112, 21248, 41068, 24208, 12288,
                65504, 17976, 58228, 53768, 39280, 44512, 10808, 36264, 65000,
                1384,  11808, 57312, 41636, 4572,  10856, 56452, 40012, 65240,
                21184, 7568,  39024, 46128, 60336, 16692, 8120,  13232, 55684,
                25264, 49508, 58600, 3216,  62448, 51420, 8732,  60568, 21756,
                63772, 54604, 30028, 56740, 684,   36460, 6088,  17176, 59516,
                48020, 19768, 44768, 24940, 31380, 53860, 5740,  12152, 48196,
                8936,  57308, 4128,  54612, 15596, 61144, 52528, 58028, 41832,
                64660, 7868,  39748, 59392, 10192, 26552, 33928, 41172, 64980,
                28248, 8968,  2656,  34508, 50944, 13260, 6944,  44828, 27900,
                52996, 62208, 23648, 5324,  39648, 28412, 19228, 31412, 13664,
                50244, 63120,
            },
            {
                29484, 15944, 35128, 708,   27896, 45504, 3812,  15548, 8988,
                65116, 12100, 20132, 50008, 59980, 24380, 30816, 8284,  26276,
                59028, 2392,  65508, 45672, 56000, 25560, 50272, 43764, 2640,
                9084,  48160, 15784, 54704, 4292,  10476, 58412, 400,   56304,
                46416, 34992, 5832,  47484, 14932, 27556, 55788, 17772, 23096,
                53060, 33232, 18428, 37800, 52156, 20664, 29720, 1948,  13196,
                44940, 25256, 33764, 11996, 22396, 29272, 64744, 44728, 20972,
                40120, 17280, 27728, 32984, 20564, 24948, 12,    34740, 28668,
                3920,  44200, 17888, 40684, 20920, 14696, 45820, 32168, 24228,
                2584,  35824, 62456, 10956, 8,     58936, 36276, 51624, 40784,
                26760, 17752, 49236, 24964, 39612, 22500, 12968, 37268, 29728,
                48248, 14092, 35852, 1788,  44180, 48616, 7656,  23300, 46476,
                14820, 38012, 55932, 22504, 30560, 41908, 24136, 9708,  57188,
                38908, 30824, 45488, 34444, 63764, 10796, 52716, 57584, 36748,
                26248, 3648,
            },
            {
                43912, 11132, 24144, 63736, 6452,  21340, 57904, 42888, 55712,
                23100, 33812, 58244, 7652,  3908,  37340, 12764, 18060, 47984,
                38192, 42552, 16948, 51904, 3744,  32508, 16112, 40144, 27692,
                33308, 58188, 23220, 29052, 39372, 62700, 26848, 33376, 8004,
                20672, 41336, 25612, 1528,  63824, 33740, 49256, 7468,  39836,
                43996, 62236, 2992,  24272, 59216, 14740, 47380, 61476, 36520,
                55176, 4044,  63168, 52532, 3220,  41488, 35048, 50380, 1264,
                61748, 10940, 53072, 7040,  44780, 55552, 42396, 14480, 48580,
                38732, 23564, 10180, 33184, 64600, 28072, 50024, 62816, 9972,
                53428, 40252, 56328, 18048, 43072, 27444, 19368, 61584, 2408,
                32240, 44712, 60340, 10280, 6236,  47636, 31996, 18028, 2704,
                20452, 54556, 23916, 17084, 63436, 30632, 18868, 52584, 11240,
                32568, 58640, 44216, 6044,  16416, 62468, 47276, 32400, 4600,
                20432, 13492, 56052, 1772,  15864, 21960, 41576, 180,   8684,
                17832, 40232,
            },
            {
                21160, 48740, 55360, 38108, 52572, 33340, 61764, 29860, 37484,
                1532,  52732, 16376, 31824, 42088, 46828, 53008, 1040,  63224,
                32344, 10628, 22876, 28200, 36348, 61988, 11552, 59420, 52536,
                18932, 12540, 61392, 7424,  34760, 13672, 48356, 53448, 38344,
                15432, 50508, 59696, 30516, 21048, 41876, 3632,  60264, 14112,
                27036, 9172,  48612, 34652, 7188,  44172, 25704, 32728, 9700,
                17612, 49408, 28120, 15080, 47656, 56792, 24172, 6532,  58128,
                30804, 46772, 37924, 64964, 12408, 29528, 59640, 19120, 52320,
                61684, 5508,  58500, 51176, 6904,  42440, 1904,  37512, 21324,
                29248, 14872, 4984,  32940, 49724, 13808, 45380, 22980, 7604,
                56036, 13604, 35140, 29140, 63188, 43768, 56268, 9048,  62596,
                44860, 32596, 58264, 42212, 6504,  55748, 37716, 61372, 1364,
                20644, 25856, 48872, 60748, 204,   52244, 37468, 18380, 64120,
                49424, 42304, 8224,  51628, 26088, 48524, 33140, 45964, 64732,
                51792, 60080,
            },
            {
                2280,  7628,  30108, 13872, 17592, 10136, 25216, 6820,  19036,
                45968, 39288, 27352, 63616, 21420, 56716, 28268, 34852, 20272,
                54244, 4536,  49292, 57592, 19740, 8104,  46748, 22456, 6100,
                36912, 46032, 42116, 51920, 19872, 44016, 17368, 4548,  29172,
                63048, 11580, 6836,  37172, 52840, 12720, 24420, 32200, 57616,
                20448, 30656, 55340, 16824, 63940, 39320, 4304,  53244, 22680,
                43336, 40224, 7144,  20404, 36928, 9592,  18148, 27880, 14868,
                34328, 22468, 3460,  16452, 49760, 40400, 4692,  32452, 11536,
                26448, 36072, 46072, 13812, 25240, 60116, 11720, 54828, 44336,
                7248,  47524, 25696, 58392, 38688, 3496,  54696, 30536, 64172,
                38620, 21776, 1180,  52896, 19252, 25780, 3772,  51464, 38880,
                26184, 4948,  10836, 50892, 28324, 14472, 4216,  34464, 50048,
                40360, 6980,  18108, 35856, 28988, 11616, 58216, 26212, 2100,
                35304, 23324, 29152, 61444, 37932, 58772, 6552,  29716, 24344,
                14652, 33796,
            },
            {
                27244, 62416, 42572, 59108, 46764, 39996, 48528, 51660, 14264,
                60764, 4944,  49224, 11420, 3028,  14952, 61328, 9600,  45364,
                24900, 60060, 13600, 44104, 700,   41292, 53796, 29840, 64940,
                1252,  25852, 31404, 5168,  56276, 24492, 59060, 32104, 46052,
                22720, 56004, 43412, 18540, 8700,  62092, 44392, 50920, 37920,
                5076,  46180, 41428, 12596, 28764, 50376, 19184, 60088, 516,
                56664, 30416, 62088, 58852, 32424, 44004, 63504, 39768, 54056,
                9344,  43304, 57004, 26820, 35580, 23400, 8532,  47084, 56112,
                2864,  17236, 29904, 55460, 39460, 19472, 31504, 16236, 58844,
                34268, 64948, 51416, 21860, 8712,  60656, 35312, 11272, 47068,
                17124, 50864, 42704, 57988, 37452, 11724, 33848, 22112, 59212,
                13200, 48944, 35244, 19708, 40048, 25016, 59032, 43620, 27420,
                63740, 53512, 13056, 42696, 23536, 8304,  40964, 15112, 53876,
                59872, 10980, 17232, 44268, 3396,  13112, 18700, 55532, 10480,
                38856, 57268,
            },
            {
                18644, 51092, 35848, 4372,  20656, 2144,  63980, 35004, 10972,
                31020, 24292, 54688, 35744, 44220, 23676, 38940, 6716,  41264,
                15972, 29056, 38552, 31748, 63604, 24580, 34228, 14592, 38996,
                50320, 16768, 62560, 11460, 36588, 2744,  64784, 9368,  39944,
                1328,  26712, 33960, 54572, 47776, 28992, 292,   17216, 10364,
                64464, 52592, 1692,  22304, 57476, 10268, 36976, 27568, 13880,
                35112, 11288, 24476, 2816,  53344, 12884, 1636,  48272, 60952,
                4644,  51000, 18964, 60212, 1924,  53192, 62808, 20912, 39144,
                64476, 43432, 22016, 8680,  48596, 3612,  51852, 23484, 40752,
                2780,  18872, 12056, 42580, 29568, 16040, 24832, 41372, 4508,
                27756, 9540,  31492, 5304,  15536, 64872, 47340, 41960, 16360,
                28824, 61020, 2488,  65464, 53996, 47224, 12108, 21844, 16388,
                9384,  30384, 46936, 57228, 64692, 33616, 50424, 45224, 6868,
                39628, 47624, 32520, 54784, 21124, 35228, 62192, 42872, 49160,
                5588,  45652,
            },
            {
                12468, 23364, 9640,  26316, 32352, 56712, 28176, 22288, 43680,
                57468, 40244, 8364,  17260, 59268, 50280, 30592, 55612, 48452,
                62492, 5716,  52056, 9372,  17524, 48788, 4700,  56516, 20560,
                9252,  58804, 47752, 21616, 29480, 41256, 50744, 13080, 19636,
                48848, 61140, 5696,  14276, 23632, 39776, 56088, 34812, 27708,
                15084, 24556, 35752, 61644, 32360, 5460,  42196, 47856, 64516,
                51336, 45640, 16664, 38180, 49840, 26516, 31224, 20212, 36068,
                25280, 30052, 41272, 13584, 31648, 45376, 15912, 27804, 34460,
                13216, 50136, 580,   60996, 33352, 37016, 64028, 27852, 8908,
                46160, 30980, 37196, 828,   63000, 52340, 48572, 19936, 58692,
                53760, 62688, 24084, 48724, 40660, 27608, 128,   55596, 6588,
                36864, 45688, 23124, 30892, 9020,  648,   32180, 55008, 4932,
                60168, 38656, 2116,  19656, 5252,  54620, 21504, 30520, 18812,
                26948, 4016,  65352, 7680,  50044, 40664, 27688, 2532,  21692,
                36260, 31532,
            },
            {
                588,   41284, 65220, 54644, 44720, 5848,  16208, 53216, 836,
                62176, 20224, 47064, 33356, 26888, 484,   12284, 18244, 33240,
                2024,  35688, 21808, 55488, 27356, 59124, 12240, 44756, 28168,
                41124, 33552, 2152,  54956, 45528, 16012, 25988, 53932, 35340,
                58112, 16672, 38796, 31340, 65100, 4396,  19764, 45892, 60504,
                40728, 49860, 8012,  17836, 45424, 53656, 15928, 23520, 7768,
                19968, 4768,  55760, 41848, 21764, 5484,  59472, 45668, 16348,
                11732, 64364, 47296, 5976,  37040, 10724, 57728, 42576, 6160,
                53720, 30904, 57192, 26220, 15196, 43164, 6104,  56116, 14060,
                60968, 53228, 57248, 26688, 45144, 7308,  33168, 2060,  37144,
                14340, 44788, 34704, 18608, 60596, 8716,  31304, 20028, 52292,
                9992,  56436, 18080, 43512, 15412, 38136, 61716, 48960, 35588,
                45024, 24552, 33128, 14716, 27632, 37844, 1036,  12412, 62948,
                56284, 36604, 14356, 24860, 30760, 11384, 53908, 16476, 63696,
                52116, 59796,
            },
            {
                53788, 47376, 16884, 37964, 12948, 61116, 41956, 36504, 7696,
                29556, 14564, 4032,  65292, 37856, 52632, 63032, 22460, 26492,
                57692, 42604, 12956, 45684, 40036, 2380,  36884, 61316, 6928,
                53308, 24312, 13968, 38048, 7892,  60276, 33000, 6152,  43140,
                28724, 3164,  45204, 51188, 11920, 57964, 9192,  22796, 53872,
                2340,  30284, 56368, 39468, 26380, 1136,  62460, 30844, 40040,
                58520, 29288, 33360, 9012,  65392, 15096, 54800, 7608,  38948,
                52768, 340,   21332, 55496, 61260, 24788, 49836, 1548,  19392,
                24240, 10296, 40744, 18388, 47284, 11084, 21464, 49396, 34816,
                24464, 16632, 5100,  20240, 39488, 13148, 55608, 65200, 29092,
                22148, 7512,  3288,  54932, 12752, 50548, 44292, 61836, 24936,
                32672, 39440, 5352,  58788, 51788, 23816, 41732, 28480, 17652,
                11548, 56536, 52296, 48432, 61904, 43032, 58744, 47028, 33892,
                22428, 51748, 42252, 58644, 920,   60964, 46632, 24064, 8852,
                28768, 15220,
            },
            {
                25756, 33608, 7464,  29444, 49924, 19960, 9800,  47728, 25956,
                50872, 55872, 42908, 10904, 20768, 5556,  40868, 46140, 9848,
                50984, 16652, 64984, 7324,  32784, 19460, 23356, 51012, 31436,
                17980, 43968, 64488, 27604, 49556, 20356, 324,   56568, 22920,
                10560, 63492, 20864, 25832, 36088, 42508, 49616, 32456, 6692,
                13212, 44832, 20792, 11180, 60140, 48508, 12660, 34788, 44556,
                2464,  25336, 60376, 46680, 36300, 27580, 42812, 32132, 23404,
                58092, 33808, 28124, 40156, 18348, 7800,  29548, 37976, 59304,
                47844, 63532, 35748, 4612,  61948, 53104, 30684, 988,   39780,
                43388, 10600, 50180, 35544, 60108, 23996, 17868, 43132, 10564,
                50772, 40368, 58196, 26256, 38256, 22768, 35468, 4108,  14088,
                64252, 49796, 12292, 26912, 34548, 3516,  19952, 7900,  64604,
                4124,  21752, 40472, 6316,  10220, 24904, 16548, 8044,  41340,
                2676,  10048, 16736, 45084, 19312, 38244, 5464,  32844, 40124,
                44080, 4820,
            },
            {
                62764, 56340, 22348, 2608,  59568, 24424, 33780, 64152, 17924,
                38384, 32380, 23488, 48868, 57880, 31276, 14656, 59996, 37436,
                4280,  30220, 24076, 59868, 54552, 46932, 15556, 63716, 3108,
                11644, 55884, 4224,  34800, 10660, 62876, 39600, 47032, 14924,
                37540, 52772, 7928,  59364, 1032,  29360, 14744, 62164, 37640,
                27024, 63596, 33576, 3228,  36524, 22100, 55400, 18188, 52856,
                10640, 49240, 13372, 18328, 1184,  50912, 11344, 62572, 3436,
                49116, 9276,  44916, 14336, 51056, 34972, 65160, 43936, 15508,
                32440, 7396,  45280, 28788, 23196, 38680, 59056, 19216, 63380,
                4084,  28952, 62464, 47316, 8784,  31272, 4916,  47628, 59336,
                32836, 15848, 46136, 30224, 2256,  59112, 16808, 47896, 28136,
                42620, 1440,  21476, 46524, 63160, 54200, 13468, 58032, 50640,
                43120, 29408, 59832, 18308, 32004, 36044, 50120, 53620, 27856,
                61624, 31420, 48644, 26528, 34532, 55992, 13776, 50836, 57888,
                20292, 37404,
            },
            {
                9152,  18072, 41024, 46148, 52564, 5240,  40348, 56580, 12360,
                6340,  60740, 1508,  16436, 28464, 44476, 8080,  33836, 54192,
                19668, 48840, 38720, 176,   28100, 9976,  41884, 26232, 35532,
                48172, 39436, 22596, 58204, 17336, 43548, 25212, 30156, 61504,
                49260, 32236, 40588, 17060, 47408, 24452, 54672, 18748, 41800,
                48684, 17252, 58828, 51528, 43044, 8636,  27404, 5540,  38228,
                62856, 21552, 40504, 54072, 30756, 22684, 56692, 18792, 40644,
                26468, 17332, 63016, 2432,  54492, 21780, 11908, 4296,  55520,
                22224, 13152, 52276, 1920,  56784, 13744, 8272,  26400, 46300,
                32416, 56356, 22280, 14456, 41008, 54848, 27224, 38232, 428,
                24724, 53564, 19920, 64576, 11256, 53184, 40512, 7224,  55388,
                18524, 35868, 60160, 29940, 9824,  43980, 31652, 25252, 37388,
                33976, 364,   12984, 46604, 55288, 65016, 4844,  20860, 13676,
                39032, 55484, 5084,  63148, 7816,  22096, 29776, 64860, 1424,
                12200, 49540,
            },
            {
                30672, 60104, 13472, 36284, 31440, 15468, 28260, 2876,  22252,
                45396, 53704, 41468, 36076, 51684, 61912, 24656, 1268,  27704,
                63304, 11604, 44228, 35040, 14316, 49592, 56952, 7644,  59424,
                19840, 29616, 52192, 6396,  31512, 50956, 12476, 7564,  18920,
                4856,  26992, 13160, 56060, 63180, 6168,  35348, 2976,  57672,
                9856,  4564,  24396, 14128, 29800, 63928, 46012, 58116, 15312,
                28648, 34164, 7476,  63892, 4456,  37524, 47468, 6544,  34440,
                12768, 59296, 36800, 30360, 5736,  47612, 26172, 40860, 50628,
                27936, 59916, 39680, 17508, 33268, 44088, 36224, 50752, 12072,
                52824, 17940, 6380,  34264, 1608,  51892, 63292, 20508, 12828,
                61364, 6032,  36228, 8344,  43760, 33228, 21264, 61136, 30968,
                9056,  52052, 38700, 5924,  17296, 39684, 2088,  47568, 16048,
                54600, 62792, 26932, 38464, 22964, 1792,  30192, 44472, 59388,
                18352, 23728, 36868, 12820, 53040, 43312, 47620, 17360, 26792,
                34956, 45520,
            },
            {
                23764, 51008, 1544,  64668, 10396, 54668, 42536, 49112, 62632,
                30396, 13976, 26604, 9560,  4236,  18944, 55784, 46984, 15860,
                40580, 5456,  53420, 21164, 62364, 3452,  38128, 32276, 45012,
                1132,  62048, 15224, 46376, 38416, 1980,  55560, 59584, 36376,
                54100, 1828,  43648, 33756, 20128, 44704, 11632, 51456, 21892,
                31140, 46448, 54220, 40716, 19212, 3936,  32900, 23336, 50968,
                68,    43408, 57196, 26412, 45748, 14368, 59932, 28776, 53552,
                44344, 51616, 23784, 42044, 57288, 33304, 62104, 18624, 500,
                37028, 9616,  62960, 25352, 48852, 5752,  65340, 16384, 1756,
                41628, 37628, 25892, 58368, 44068, 16508, 9624,  35500, 56432,
                42504, 28352, 48520, 14904, 51264, 27256, 888,   13316, 44960,
                23572, 57792, 14988, 25792, 55604, 50204, 61984, 22296, 5064,
                9984,  19696, 49712, 7820,  15488, 58060, 40768, 34948, 9660,
                46960, 44,    51240, 29148, 59456, 2400,  9952,  40800, 60772,
                54812, 3804,
            },
            {
                39296, 27892, 43892, 20464, 25672, 58400, 18336, 37204, 8792,
                35088, 19972, 57224, 64528, 38788, 12780, 42176, 35436, 57560,
                22956, 32120, 58452, 17768, 30084, 52228, 22140, 17068, 25076,
                13004, 41616, 9108,  26840, 64184, 23464, 33880, 42448, 21220,
                46304, 64876, 23224, 9388,  38432, 30400, 59828, 26016, 39300,
                65408, 34588, 7340,  37476, 56772, 49620, 12988, 41276, 61160,
                19536, 48080, 10324, 16952, 32636, 41768, 9156,  20752, 64740,
                816,   5236,  20068, 10732, 16228, 45056, 8332,  58612, 31060,
                47004, 20208, 43128, 3844,  55104, 21628, 29324, 57332, 23568,
                60976, 9404,  47008, 64684, 29536, 23088, 49640, 45740, 31028,
                17988, 3384,  39712, 59896, 23976, 57016, 37300, 63784, 50372,
                33800, 4608,  47452, 64092, 20600, 35476, 11916, 27836, 59684,
                44528, 40344, 30024, 53196, 45536, 11880, 24576, 49812, 6620,
                63556, 32804, 41044, 20220, 15728, 38668, 24276, 31992, 6760,
                21600, 14972,
            },
            {
                57688, 8020,  53888, 34528, 5944,  46420, 36,    23988, 59664,
                51304, 3152,  47364, 23128, 32648, 49228, 21620, 10388, 3328,
                51108, 8836,  26592, 45644, 6984,  40976, 10700, 65380, 54760,
                50032, 57132, 36404, 20488, 52972, 4112,  13916, 27552, 9668,
                16264, 30032, 58724, 49520, 3524,  53116, 16184, 47988, 408,
                12856, 27504, 16852, 61460, 1632,  25996, 35736, 9412,  4892,
                31188, 36480, 52828, 23428, 61816, 2836,  49888, 24784, 15236,
                38604, 32796, 48244, 63524, 28592, 37604, 13460, 24416, 53880,
                6660,  34240, 51876, 14880, 31424, 10692, 40560, 45780, 34688,
                27860, 54640, 20232, 4132,  12032, 38784, 2456,  26076, 7588,
                50936, 62252, 20896, 31904, 5408,  17568, 41900, 3876,  19780,
                11164, 40556, 28664, 596,   32604, 8016,  41988, 51868, 31356,
                56956, 24192, 2848,  63936, 33144, 19176, 61104, 28492, 16196,
                56844, 26096, 10712, 46276, 64140, 35204, 57292, 48760, 52152,
                42968, 63488,
            },
            {
                33020, 17616, 12604, 49388, 39800, 63092, 32020, 12052, 44300,
                16728, 7360,  42900, 28844, 52788, 6608,  63540, 30300, 60096,
                37688, 42892, 64372, 12944, 36268, 60280, 47268, 34168, 3828,
                28560, 6116,  32156, 60620, 17716, 44136, 49200, 62396, 39420,
                51720, 6264,  41244, 13788, 28068, 62836, 7524,  41588, 19836,
                55120, 44416, 50904, 10988, 21440, 43460, 64672, 51972, 55768,
                25596, 13488, 59040, 5932,  39220, 54560, 35756, 58064, 30020,
                42992, 55564, 26124, 6556,  53232, 1992,  49664, 40356, 17008,
                64572, 11664, 27492, 58384, 36856, 61780, 19052, 2800,  13680,
                6768,  49348, 15136, 42592, 32552, 52556, 62696, 58152, 14220,
                34204, 54712, 12136, 46544, 9584,  53396, 48232, 30068, 26000,
                54576, 59048, 43468, 13568, 60252, 46400, 18228, 6016,  37832,
                13116, 16880, 36092, 6436,  42164, 54548, 2220,  38176, 52264,
                21804, 43004, 3520,  53628, 7364,  27468, 4352,  13944, 19416,
                10544, 36756,
            },
            {
                48332, 4560,  61600, 28720, 22184, 15036, 51968, 27228, 55904,
                33684, 39708, 61264, 15024, 1984,  17664, 54544, 25008, 13952,
                48460, 16724, 3992,  54372, 23716, 972,   27300, 14852, 43084,
                39056, 23008, 47980, 11140, 40432, 7844,  56364, 31668, 892,
                57460, 25552, 34624, 18788, 55636, 22832, 36812, 32752, 61920,
                5780,  23628, 59260, 31988, 47536, 28868, 14768, 18256, 38652,
                46320, 2020,  42180, 20516, 27620, 11280, 17628, 46484, 7260,
                11896, 61088, 18176, 41296, 58232, 21508, 34428, 61212, 3252,
                43632, 22484, 39332, 1460,  47400, 8088,  53532, 50428, 62820,
                39356, 30784, 59776, 36372, 56824, 6256,  17584, 40912, 22312,
                44684, 780,   24856, 38240, 63144, 34472, 14640, 60588, 8156,
                35372, 16696, 22352, 53004, 37008, 23968, 54856, 65052, 1076,
                47128, 61208, 50756, 22200, 27272, 10740, 47652, 5312,  31360,
                13008, 61416, 34064, 17960, 49344, 22180, 44692, 62220, 30264,
                468,   26240,
            },
            {
                22836, 55716, 41396, 2176,  57136, 9428,  38008, 6392,  65152,
                20748, 25288, 11268, 58072, 37548, 43788, 33604, 46048, 536,
                28544, 34568, 20988, 49248, 31776, 57372, 51024, 20588, 8912,
                63024, 16892, 256,   58272, 29404, 35816, 24224, 19380, 12068,
                36740, 47532, 63952, 2460,  43824, 10896, 46344, 14684, 50256,
                29792, 36220, 3184,  39384, 7832,  58424, 34360, 6312,  22600,
                62404, 29916, 50636, 65192, 45116, 31680, 1368,  63408, 21904,
                49548, 2908,  36016, 13996, 31240, 45676, 10376, 25968, 56424,
                29796, 48956, 60092, 17676, 24068, 42384, 26696, 32656, 22776,
                44632, 18144, 16,    25156, 21376, 47584, 28160, 35940, 9444,
                65496, 29908, 42936, 58592, 27636, 1708,  22816, 44600, 39764,
                64808, 2388,  49828, 10128, 4928,  30372, 14212, 26852, 20824,
                33280, 9588,  43472, 56560, 14324, 59476, 35584, 64880, 45072,
                8300,  39008, 55504, 29304, 59648, 11456, 37636, 54136, 40552,
                58616, 45844,
            },
            {
                13792, 31320, 19016, 44944, 35008, 60816, 18064, 41728, 46876,
                1684,  54936, 31640, 48116, 26988, 61940, 8972,  40200, 19500,
                62344, 55540, 41896, 8488,  39516, 18152, 6000,  44812, 58968,
                54128, 35256, 51724, 45004, 15284, 65048, 5416,  54280, 45472,
                22176, 15640, 8804,  50980, 31952, 60032, 25660, 2048,  57968,
                9764,  42608, 16072, 62552, 19612, 53792, 792,   44456, 57416,
                11024, 16616, 7804,  34868, 14168, 56924, 52088, 38388, 34172,
                28128, 52888, 24428, 8848,  65384, 51368, 19028, 5724,  37916,
                14624, 8648,  52304, 5524,  34560, 57508, 15724, 4784,  11588,
                54260, 8600,  64032, 51364, 9964,  13512, 61204, 2940,  52968,
                49108, 15620, 4724,  18648, 50344, 12504, 57280, 19108, 5688,
                47012, 32436, 27720, 39052, 63136, 57516, 42660, 49692, 39704,
                53504, 29240, 4392,  18344, 32172, 41056, 24104, 20400, 16676,
                50128, 24604, 1168,  14572, 41992, 32460, 2648,  24820, 16148,
                6092,  65376,
            },
            {
                9892,  37560, 52764, 7116,  24680, 30668, 50296, 4468,  29416,
                14256, 50868, 8268,  19704, 4524,  22416, 12940, 58756, 52024,
                6028,  11304, 25972, 60684, 13704, 64084, 28896, 37412, 24112,
                30896, 12524, 26284, 21004, 2988,  48568, 27336, 38824, 61924,
                3888,  59196, 29476, 38268, 20732, 5276,  52740, 40136, 18472,
                22360, 55384, 27132, 51432, 11740, 24320, 40564, 49176, 27000,
                32948, 54428, 39916, 24632, 5412,  19944, 25840, 9488,  44372,
                14912, 59960, 40024, 47692, 20,    27644, 42404, 54048, 33220,
                45436, 20244, 61932, 30820, 12808, 45864, 64760, 38340, 58552,
                35240, 27064, 40104, 45772, 30492, 43480, 55020, 32776, 25472,
                20320, 37136, 55740, 8764,  40960, 32232, 36980, 52820, 25124,
                11676, 56816, 20276, 45132, 17308, 2696,  34884, 10888, 6576,
                59096, 24752, 63664, 38828, 49356, 8076,  800,   53220, 28228,
                57848, 35796, 45996, 20728, 64632, 8720,  57568, 50364, 20456,
                33912, 51372,
            },
            {
                2472,  27152, 62980, 15580, 48024, 11928, 54580, 21500, 61660,
                34892, 39260, 63896, 42616, 36324, 56360, 49128, 31484, 24348,
                38040, 45020, 30236, 1556,  46452, 33940, 53128, 11236, 1908,
                47588, 5136,  61680, 42336, 32516, 53236, 9784,  14296, 42672,
                33444, 24640, 55180, 42184, 13412, 63036, 28364, 34732, 48680,
                64804, 45340, 6140,  33744, 46736, 29956, 59860, 14048, 37116,
                3676,  19200, 61224, 46596, 49696, 59480, 42260, 4200,  62152,
                19436, 5940,  56904, 20992, 37492, 16276, 59760, 11980, 63944,
                1216,  25356, 36608, 41512, 55472, 21304, 1020,  29844, 19284,
                47780, 3820,  14708, 56064, 2180,  19328, 39100, 5020,  46044,
                11840, 60792, 30772, 23828, 64256, 46120, 4384,  60232, 30596,
                15380, 51992, 61176, 8236,  25728, 48384, 22512, 62040, 18924,
                15484, 45868, 1936,  12656, 54896, 62156, 30588, 43916, 4388,
                11512, 62804, 6988,  52752, 26452, 47644, 36024, 12332, 43584,
                28508, 39624,
            },
            {
                45540, 21604, 56512, 42308, 628,   58776, 36952, 26424, 44384,
                10676, 23732, 17144, 840,   53328, 28616, 6972,  17256, 2516,
                65196, 15272, 56916, 51836, 23132, 4556,  16488, 42236, 63316,
                19600, 39812, 55756, 7916,  37004, 58696, 18580, 30476, 51976,
                7120,  48940, 17792, 100,   47024, 56124, 16680, 7092,  12088,
                31616, 960,   14400, 37840, 63420, 2860,  9144,  20948, 64360,
                51708, 42776, 10188, 328,   36468, 30244, 12168, 54232, 32872,
                46636, 29580, 11752, 33996, 54736, 5108,  30624, 23112, 40308,
                47760, 57120, 10680, 3544,  27516, 48784, 9692,  52444, 42896,
                61596, 23156, 33292, 59264, 37444, 24140, 63872, 16700, 57912,
                40340, 6036,  44428, 53376, 2496,  16456, 26980, 9364,  43052,
                37696, 136,   34432, 41700, 12824, 52920, 32092, 55516, 44124,
                37068, 51980, 33816, 27844, 20036, 46628, 15584, 59076, 33724,
                41528, 18844, 30864, 39536, 3964,  17212, 29896, 62352, 7440,
                59968, 54804,
            },
            {
                4696,  35596, 10568, 32688, 20180, 40068, 7968,  16352, 2444,
                59156, 51736, 46564, 33204, 14796, 62900, 39592, 44264, 34012,
                50220, 21848, 35936, 18392, 38460, 60052, 26436, 56788, 50356,
                34796, 13868, 29060, 16960, 23328, 46352, 63280, 1304,  21384,
                60672, 11680, 27240, 35984, 9236,  23760, 43640, 38016, 60840,
                25460, 41448, 58180, 22108, 17660, 43380, 55848, 31556, 6792,
                25024, 58240, 29200, 22440, 16120, 63720, 39620, 17668, 23724,
                1444,  51044, 42988, 24956, 62312, 45052, 49284, 7720,  18320,
                29012, 15120, 51236, 63636, 17816, 39444, 34152, 25092, 6564,
                12508, 50736, 17796, 6320,  27628, 11468, 49336, 8624,  28264,
                50988, 34408, 14100, 20876, 35724, 48900, 55956, 21684, 63552,
                48260, 23372, 5068,  29300, 58372, 37616, 5972,  844,   29856,
                8900,  22772, 58440, 6272,  36336, 25372, 10160, 38216, 22052,
                54648, 14144, 49268, 60756, 22480, 56136, 45304, 904,   23868,
                18672, 15232,
            },
            {
                64008, 25080, 50192, 61284, 46720, 28520, 64772, 49184, 55744,
                30972, 6268,  27532, 60408, 21624, 9904,  47556, 25808, 11672,
                59376, 8112,  3636,  43544, 10088, 48444, 7004,  32032, 22128,
                9116,  59036, 43988, 51376, 5520,  11968, 25976, 39344, 35060,
                44784, 57172, 40336, 65304, 52028, 33108, 59188, 2580,  51656,
                20280, 54204, 8484,  48280, 27328, 52188, 35440, 16104, 39388,
                47792, 12516, 34144, 51104, 56320, 3008,  27144, 48620, 58308,
                36744, 64340, 8264,  17928, 2308,  14588, 33068, 58388, 53248,
                4940,  35124, 43232, 23692, 6388,  54272, 61196, 16140, 56980,
                40836, 29508, 65080, 47160, 53316, 42432, 60484, 32356, 22236,
                356,   63384, 26084, 10084, 61076, 29500, 12692, 40528, 6812,
                17864, 55100, 50776, 65144, 19588, 15992, 46700, 26612, 63236,
                41788, 14496, 49052, 65420, 42800, 3572,  61272, 51404, 7488,
                26176, 3140,  44096, 10356, 34044, 13452, 38124, 53172, 41904,
                32612, 47684,
            },
            {
                38808, 30260, 6468,  17640, 3428,  24096, 13060, 33916, 41624,
                18668, 38032, 12776, 43496, 3680,  35144, 58268, 19280, 54316,
                40736, 27784, 32188, 61092, 54900, 29264, 41112, 15064, 3760,
                46496, 26052, 676,   64612, 34448, 41724, 55152, 50504, 8872,
                16340, 3548,  30700, 18880, 4736,  21108, 29760, 10432, 46756,
                16316, 35704, 30256, 61564, 4832,  11324, 23820, 60824, 44736,
                3196,  18068, 62784, 41604, 8640,  45532, 35328, 6200,  10236,
                15376, 40296, 27192, 57812, 52020, 40988, 10464, 25664, 37668,
                62012, 21496, 32016, 59712, 46208, 13628, 31408, 2356,  45076,
                36420, 10228, 704,   34748, 21544, 15504, 2820,  36632, 54392,
                18132, 47704, 57580, 42804, 38708, 52280, 1204,  34096, 45716,
                31444, 13732, 27056, 44044, 9216,  39888, 60480, 11924, 19956,
                57324, 2812,  31108, 11116, 53684, 17908, 48148, 29572, 45100,
                64276, 56404, 36556, 28392, 51644, 6296,  64424, 26268, 9160,
                57380, 11956,
            },
            {
                52524, 59384, 13904, 40820, 51560, 36532, 53960, 4256,  22144,
                9004,  64024, 52496, 24912, 56128, 49628, 5388,  30612, 348,
                14424, 63452, 46856, 24204, 12664, 20144, 64960, 36664, 54564,
                61752, 38692, 31068, 20700, 57096, 14908, 3160,  28644, 61696,
                23888, 54168, 13272, 43928, 61492, 49440, 14576, 39928, 27384,
                64284, 3480,  13100, 38892, 55056, 41564, 612,   50512, 27984,
                57180, 37720, 6060,  26472, 13956, 21244, 52416, 60148, 31308,
                55616, 20724, 48120, 30340, 35844, 22044, 64892, 44204, 1864,
                13192, 49940, 9320,  300,   28528, 37328, 50400, 22448, 63184,
                26568, 19988, 55256, 39616, 7728,  30528, 57068, 45268, 12572,
                41192, 29968, 4204,  16776, 7196,  19688, 59500, 24372, 56896,
                62664, 10572, 36732, 1648,  32696, 22872, 53912, 49568, 34468,
                38560, 46952, 26984, 21060, 40084, 34080, 13564, 172,   19892,
                32760, 15720, 23592, 1820,  58784, 20592, 16480, 48640, 35504,
                21396, 2036,
            },
            {
                26960, 19404, 44884, 56932, 31428, 9928,  60640, 44192, 29452,
                57152, 47120, 2132,  32000, 17444, 38676, 64584, 23600, 45624,
                36652, 51924, 16996, 5920,  34980, 51072, 1012,  43404, 23616,
                11792, 17760, 48820, 10124, 24868, 47324, 32376, 19156, 46124,
                37972, 33404, 48256, 26040, 8032,  37020, 54912, 58356, 6764,
                43448, 50224, 24188, 45920, 19648, 33164, 64636, 14136, 9468,
                20384, 30992, 48736, 55168, 33380, 64848, 18164, 40908, 23344,
                44956, 3628,  13028, 7132,  60804, 4180,  28604, 19480, 55160,
                46620, 39596, 57436, 19964, 42084, 11008, 56208, 18092, 7976,
                48056, 59820, 13892, 44568, 62660, 26228, 48832, 5260,  61032,
                24212, 9484,  51832, 32764, 64628, 27528, 43688, 15444, 3284,
                39168, 20928, 47060, 61412, 56180, 4184,  30140, 7508,  24668,
                4676,  16856, 55944, 60744, 7824,  23936, 56888, 37684, 53020,
                9812,  40796, 62544, 47500, 43148, 31700, 40328, 4408,  29400,
                61156, 43368,
            },
            {
                37264, 9024,  23664, 1188,  63624, 20892, 16792, 48436, 25508,
                14620, 35444, 20572, 42240, 10804, 28300, 15320, 41676, 8676,
                57184, 21408, 2564,  39428, 57924, 26116, 8924,  16576, 51888,
                34024, 58124, 4852,  53436, 40408, 62908, 7456,  58472, 11540,
                5336,  63864, 1172,  57656, 41852, 23384, 1880,  18360, 34656,
                21668, 31460, 57788, 7808,  15968, 28852, 47520, 22788, 34848,
                53948, 60452, 1104,  23504, 43820, 4504,  29320, 11888, 908,
                62940, 33580, 54592, 38812, 46096, 16516, 51076, 34220, 8396,
                29960, 24560, 16604, 52832, 65428, 26132, 3356,  40248, 34368,
                52948, 31184, 3244,  23444, 51856, 10884, 16544, 20352, 38888,
                34196, 56216, 45924, 21540, 37156, 48140, 9940,  50692, 29632,
                53336, 6108,  25144, 50000, 17264, 41424, 45128, 14664, 63712,
                52272, 43048, 32572, 1892,  50416, 63064, 28152, 42540, 59528,
                4284,  50264, 18568, 8048,  12432, 25188, 61976, 54744, 14680,
                50580, 6732,
            },
            {
                64580, 54816, 34716, 47640, 27372, 39084, 6864,  58572, 32,
                40088, 60980, 5576,  54104, 59092, 7008,  51468, 61400, 33028,
                12024, 27124, 43220, 62568, 48208, 31600, 45576, 61168, 29468,
                7256,  27316, 44880, 35488, 2240,  17168, 36876, 22652, 42788,
                52688, 20380, 29188, 16936, 11276, 32364, 45288, 63484, 11836,
                52464, 2336,  62292, 37032, 55976, 5652,  58880, 38540, 7072,
                42100, 12216, 16968, 39116, 10004, 57608, 47384, 51300, 37440,
                26556, 49576, 18976, 24516, 56448, 11744, 41780, 58576, 14836,
                63244, 2852,  36408, 33200, 7080,  48916, 60900, 29396, 12836,
                5120,  41840, 16992, 37576, 59004, 32684, 43268, 64900, 28564,
                3088,  15228, 62304, 1492,  13228, 55252, 5112,  35908, 18504,
                59936, 42792, 14556, 34452, 28008, 11068, 59220, 35924, 19116,
                57892, 9496,  12904, 36960, 44164, 15532, 5116,  11564, 22088,
                26396, 30708, 38596, 57676, 35408, 52048, 436,   10736, 45496,
                17952, 32116,
            },
            {
                12676, 49352, 16036, 4816,  42652, 12928, 34232, 52344, 31016,
                11052, 49824, 26972, 44708, 22972, 36148, 47144, 19516, 4120,
                48964, 54196, 30200, 7560,  14244, 19260, 56236, 38792, 2052,
                41504, 63984, 19292, 12908, 59504, 29852, 49368, 55312, 26976,
                14520, 40184, 51196, 35820, 60416, 53576, 28428, 48088, 25204,
                40780, 16928, 10068, 42932, 25608, 50816, 2104,  44468, 63260,
                25732, 50972, 46200, 62380, 27672, 36012, 16092, 61356, 7268,
                42880, 14900, 8940,  63708, 32040, 532,   26368, 37432, 5560,
                43180, 48416, 59140, 12320, 45296, 16004, 22856, 43576, 58668,
                20984, 64548, 46500, 28380, 6932,  1000,  36236, 53276, 8376,
                49960, 43016, 26152, 39948, 31368, 23588, 61568, 41868, 26168,
                32208, 9040,  57772, 2436,  64356, 20716, 51476, 320,   25916,
                40212, 29344, 22660, 55124, 19244, 30884, 47176, 35552, 64588,
                44772, 13284, 54212, 5732,  27940, 44560, 19520, 37828, 60056,
                24756, 41168,
            },
            {
                1744,  21836, 59292, 30040, 56056, 62452, 24232, 43672, 20056,
                63500, 17272, 33544, 13884, 62864, 948,   30900, 25688, 58740,
                35508, 17620, 60000, 23672, 37428, 3912,  10868, 21712, 48636,
                14584, 54268, 22584, 50852, 25768, 44024, 10472, 308,   34028,
                59372, 9208,  45984, 6188,  21536, 4008,  15360, 8456,  36780,
                59448, 29692, 48772, 34036, 12772, 21168, 31920, 15180, 18752,
                29848, 3312,  34016, 20836, 53292, 2652,  24596, 32464, 21424,
                58168, 53616, 36312, 5036,  44304, 48992, 19896, 61072, 23036,
                52104, 27584, 20916, 4432,  31072, 38576, 53816, 8868,  35728,
                49156, 25148, 10156, 55660, 50100, 22072, 13620, 57304, 23740,
                18688, 59564, 10212, 53640, 17380, 57752, 46004, 12620, 936,
                63272, 48552, 22604, 39760, 54152, 7760,  38296, 32056, 48932,
                5340,  45484, 64780, 49608, 7204,  25332, 57232, 52548, 17220,
                1248,  34376, 61408, 23240, 15888, 65440, 32856, 22528, 56908,
                5676,  28500,
            },
            {
                62028, 45368, 37968, 10468, 19096, 49632, 3276,  8796,  54768,
                37724, 4436,  56936, 8660,  38960, 52456, 11176, 40656, 14764,
                45096, 1516,  10288, 42376, 64288, 53884, 34348, 25668, 58568,
                31152, 9304,  38172, 33116, 6644,  64844, 15804, 38956, 62120,
                4648,  24220, 31032, 64976, 48696, 38644, 43172, 61820, 224,
                54656, 20736, 5052,  65520, 53600, 39960, 61096, 8452,  52384,
                37048, 58484, 6344,  14676, 41480, 8788,  55292, 46008, 12456,
                3720,  30468, 41348, 21872, 28180, 57348, 9232,  53856, 32544,
                10752, 17596, 41116, 55688, 61712, 25504, 1456,  18584, 56092,
                2932,  14616, 39336, 33716, 18492, 63300, 47724, 30720, 40572,
                5608,  35800, 29660, 49072, 3864,  8212,  33840, 21704, 52308,
                16472, 36836, 44688, 12844, 30796, 46912, 23264, 62280, 13692,
                59524, 17548, 1392,  33712, 41388, 61732, 2868,  9504,  39604,
                49908, 20136, 46548, 3616,  40384, 49512, 9100,  2896,  47572,
                35032, 52588,
            },
            {
                17140, 32584, 7140,  53160, 26548, 36352, 15628, 46812, 28672,
                23160, 41856, 48204, 29684, 17920, 21592, 55572, 5884,  65168,
                22780, 38580, 51588, 32296, 27676, 49456, 44608, 62200, 17292,
                4380,  46528, 61052, 2748,  41280, 53524, 21768, 46916, 28696,
                19488, 44100, 54700, 12976, 26488, 18732, 55900, 23164, 33564,
                13740, 45408, 27304, 17680, 3036,  46028, 24148, 57148, 48004,
                11012, 22124, 64652, 31480, 50148, 60128, 19344, 38896, 65524,
                27088, 17740, 60512, 51696, 13484, 35056, 16828, 40188, 2576,
                46104, 64432, 34084, 8520,  50492, 13860, 47332, 63992, 32444,
                27596, 52376, 5788,  59656, 41660, 8608,  26904, 2296,  45828,
                12924, 63520, 20708, 44652, 37364, 64956, 27428, 40064, 6740,
                56192, 28828, 4624,  18776, 57284, 3492,  16364, 42332, 8656,
                35184, 51804, 27660, 10912, 14844, 37336, 21404, 32264, 27640,
                58780, 7888,  30320, 55836, 11704, 59104, 29184, 53808, 14180,
                42556, 10020,
            },
            {
                57820, 40508, 23544, 65188, 732,   41496, 61244, 32748, 59160,
                13256, 1448,  64700, 25172, 45952, 60492, 43240, 33464, 49920,
                29256, 57512, 19788, 6580,  16024, 88,    12852, 7520,  42136,
                36080, 56828, 27792, 12436, 18704, 57684, 35416, 3348,  50840,
                57044, 16244, 37220, 1536,  33876, 58980, 10404, 30524, 50856,
                7088,  41704, 58416, 37888, 30000, 11436, 35120, 460,   28208,
                43292, 54452, 39380, 47016, 11796, 29008, 43748, 84,    34552,
                50412, 46600, 10556, 1596,  64268, 47304, 6180,  62024, 29704,
                14460, 37812, 872,   23552, 42976, 29672, 36596, 10864, 41216,
                45664, 62800, 31232, 23920, 44920, 15884, 37064, 58380, 52136,
                32332, 55396, 388,   24308, 14632, 51340, 18204, 59120, 46752,
                11036, 24032, 50828, 60312, 35492, 26100, 53164, 29168, 56620,
                20296, 39176, 23388, 55996, 60112, 46216, 50648, 12372, 63204,
                42024, 15176, 25712, 36280, 43484, 18480, 25224, 38712, 63860,
                20480, 26236,
            },
            {
                2428,  48412, 14280, 44312, 31104, 12040, 55752, 18664, 7384,
                51764, 35020, 54328, 6336,  32428, 2304,  16168, 26944, 9396,
                3984,  13408, 61636, 47200, 56084, 22292, 39864, 29156, 53012,
                20344, 24072, 49976, 44752, 30584, 8744,  25232, 13540, 32164,
                10852, 40952, 62780, 7872,  50304, 41888, 3056,  46988, 16720,
                62968, 24540, 9676,  49972, 55728, 63592, 19804, 41184, 16128,
                5008,  26148, 18008, 2084,  24852, 37104, 6524,  16748, 56616,
                7784,  40140, 25004, 37124, 31364, 20524, 43340, 24124, 50576,
                56524, 26144, 59672, 52636, 19336, 5148,  57456, 22104, 6572,
                16068, 20204, 144,   12400, 54332, 4148,  61180, 20284, 9740,
                16648, 27276, 61296, 41144, 10704, 55112, 31212, 2504,  35268,
                63996, 43200, 32068, 8816,  40804, 65460, 11452, 44988, 6080,
                48000, 63468, 5012,  42964, 30004, 3688,  17708, 55036, 22888,
                5552,  48676, 53252, 63576, 1452,  33944, 4868,  45980, 7272,
                31476, 50940,
            },
            {
                5360,  35216, 19736, 58816, 51096, 22364, 4248,  38776, 26012,
                43844, 20696, 15524, 40080, 50820, 12416, 37748, 62772, 46212,
                54164, 41252, 33820, 25928, 37080, 59328, 51200, 33288, 65452,
                10216, 1060,  16028, 62412, 39208, 52072, 63620, 42648, 60072,
                6288,  22968, 52892, 29144, 21828, 14392, 64292, 27216, 38836,
                53260, 34912, 3748,  22084, 15116, 6492,  52040, 32788, 58088,
                63108, 35660, 8276,  60876, 56972, 21268, 62432, 53108, 31116,
                22120, 14412, 58832, 52936, 4428,  27204, 59428, 12020, 35772,
                5428,  47868, 10312, 15932, 63536, 39036, 48856, 26780, 60500,
                51020, 35424, 57208, 38276, 29068, 50524, 34344, 25276, 39560,
                43952, 49876, 7244,  33600, 47540, 4808,  42768, 25384, 13528,
                20268, 53408, 512,   14896, 49040, 21196, 1808,  33832, 24728,
                12900, 31692, 16564, 53668, 8696,  25400, 34612, 44480, 360,
                38036, 32740, 9348,  21120, 13992, 51556, 61376, 56188, 16784,
                36932, 60948,
            },
            {
                29100, 55632, 9756,  27268, 6668,  35600, 46372, 64096, 49480,
                10872, 61656, 28400, 8896,  59412, 24548, 56344, 20800, 35812,
                17656, 23656, 2292,  43948, 10728, 18532, 3156,  25328, 14408,
                43504, 55272, 32876, 7076,  22328, 1928,  17500, 48892, 38028,
                26744, 47528, 35248, 17880, 45880, 54436, 36548, 6024,  20288,
                12344, 59768, 31892, 47632, 43204, 36384, 25136, 45824, 12804,
                49280, 30688, 44900, 14892, 50760, 33096, 47608, 10200, 26432,
                63424, 2484,  42656, 18956, 56324, 39668, 8312,  53984, 17916,
                21516, 41516, 28512, 44548, 31980, 3116,  13068, 33688, 43644,
                9492,  25544, 47560, 61872, 22024, 10716, 46888, 5680,  65324,
                2596,  36108, 19132, 58104, 28732, 21792, 60016, 49776, 38752,
                7448,  27748, 61084, 18304, 37904, 29980, 57632, 50384, 40076,
                58940, 2900,  35684, 47220, 20064, 65176, 40272, 58260, 29212,
                62188, 17240, 45200, 57804, 39568, 23744, 30128, 11088, 22208,
                43956, 13104,
            },
            {
                24152, 45560, 63444, 39264, 17672, 54416, 29436, 15900, 1968,
                31228, 37160, 55424, 47312, 18372, 42940, 3620,  49360, 7180,
                31612, 63920, 52200, 7956,  30408, 63072, 45908, 6496,  60228,
                37732, 47976, 26356, 59276, 35884, 46196, 29120, 5016,  20092,
                55452, 572,   10164, 60960, 3976,  31856, 23756, 57036, 45044,
                812,   40488, 17412, 62592, 28376, 1568,  60084, 8884,  22616,
                2872,  53356, 19680, 40576, 27680, 5488,  13544, 41108, 45196,
                36052, 49412, 32920, 11780, 47972, 15744, 32484, 45508, 64820,
                3692,  33776, 57744, 7516,  20848, 55140, 61500, 17684, 53324,
                4656,  40116, 14284, 2904,  42692, 17956, 31316, 56776, 14600,
                22388, 54484, 11448, 39132, 15156, 63368, 8892,  17204, 56456,
                41916, 33408, 47116, 54944, 44444, 5128,  62180, 14596, 18852,
                52676, 26504, 61332, 38328, 12264, 51332, 6796,  14976, 11000,
                54384, 24588, 3724,  28156, 47672, 7592,  41864, 49008, 296,
                54084, 40228,
            },
            {
                51772, 15696, 32892, 1380,  42744, 60364, 8724,  24512, 41136,
                57760, 5604,  22944, 248,   34516, 61188, 30064, 11356, 40112,
                58900, 12912, 48032, 57336, 15624, 40980, 35460, 49884, 19852,
                29712, 4168,  41084, 11660, 53468, 14956, 57256, 34372, 12304,
                65252, 31048, 43692, 57388, 12788, 41080, 8672,  61404, 49024,
                30048, 25032, 56720, 7932,  13324, 39044, 55164, 18564, 29424,
                38532, 61968, 10932, 55828, 1432,  37948, 59224, 19916, 4592,
                16308, 60064, 28712, 6448,  23376, 61396, 1140,  29000, 38436,
                49524, 14200, 61868, 37192, 50324, 24268, 41140, 1428,  30304,
                58824, 19164, 32808, 55304, 7940,  63216, 52620, 40956, 28148,
                45948, 59740, 25772, 52368, 1236,  45468, 36684, 30712, 2148,
                63008, 22740, 3836,  12184, 25896, 9840,  22924, 36392, 7228,
                43376, 10276, 21772, 1068,  56876, 28020, 33092, 48348, 20888,
                42532, 51036, 36940, 12460, 64100, 19400, 35100, 58252, 27456,
                64924, 7852,
            },
            {
                20908, 4440,  61552, 48708, 21520, 13404, 50436, 33972, 62532,
                19364, 45768, 13936, 52804, 39152, 26800, 15328, 54888, 22284,
                860,   28360, 37804, 20960, 26540, 54132, 12132, 23208, 57424,
                9256,  17020, 63212, 20832, 43436, 7992,  61640, 23596, 50092,
                40516, 16760, 25044, 37728, 49988, 27788, 18620, 34040, 15892,
                10604, 51472, 44388, 21352, 52812, 48548, 32252, 65136, 42596,
                5800,  25516, 34776, 43840, 23012, 63012, 29872, 51728, 56008,
                24480, 9296,  52612, 63760, 34808, 43260, 51800, 10996, 22716,
                54824, 25740, 268,   12180, 45896, 27960, 10352, 36136, 46460,
                22424, 63956, 51280, 27868, 36804, 24004, 744,   11776, 35400,
                8280,  30932, 4616,  42864, 33136, 23984, 53928, 11972, 26784,
                51540, 15308, 37060, 59152, 31260, 41612, 55724, 46860, 28892,
                64552, 48720, 30640, 42080, 15980, 44892, 23204, 2556,  60896,
                34772, 5784,  59592, 31136, 1724,  52756, 15560, 4544,  32548,
                18024, 36288,
            },
            {
                47808, 30324, 11376, 37416, 26580, 57048, 2788,  44504, 11432,
                51440, 26508, 32768, 65444, 10256, 4484,  46480, 62680, 43784,
                51480, 17284, 42484, 5900,  3420,  61464, 31720, 584,   40460,
                53100, 32576, 50848, 1484,  25652, 31548, 39092, 2752,  44584,
                8444,  54028, 5872,  21144, 63916, 2184,  52728, 43228, 4320,
                64920, 37572, 2692,  34736, 26648, 4464,  10696, 16088, 46964,
                51760, 14072, 58672, 7744,  17344, 46336, 9556,  34272, 660,
                39496, 43960, 20608, 37936, 3120,  26216, 19136, 59888, 7672,
                42856, 16900, 30352, 53556, 18768, 6240,  65248, 56160, 13344,
                26628, 5804,  11340, 45040, 15800, 48344, 60012, 19540, 51156,
                62288, 16560, 49136, 64396, 18448, 6084,  47800, 60448, 19876,
                43668, 8512,  48936, 18712, 64068, 51356, 1576,  33296, 16608,
                4332,  54628, 34500, 59784, 5348,  53124, 63652, 39640, 9548,
                26772, 19008, 15244, 55480, 44008, 37872, 61668, 25456, 45680,
                11940, 56700,
            },
            {
                42312, 25228, 58436, 52864, 6416,  30996, 36380, 22844, 17196,
                38392, 7692,  56068, 41632, 20552, 50472, 31448, 36156, 25364,
                9228,  32840, 65040, 46684, 34876, 49164, 17724, 44604, 64412,
                26756, 13332, 35560, 58176, 48092, 18272, 55184, 27752, 14376,
                58960, 29280, 33900, 47192, 15256, 58676, 36356, 22608, 55240,
                27364, 19776, 59616, 15140, 63176, 41332, 24280, 57140, 33652,
                1644,  20312, 48488, 31852, 54724, 26264, 15092, 48844, 64484,
                27800, 13180, 57884, 16816, 46392, 56104, 13840, 36144, 32012,
                63544, 47412, 39224, 58748, 33504, 43464, 50952, 16940, 38872,
                49216, 34120, 40808, 57356, 4064,  31784, 38984, 25644, 43560,
                3236,  21572, 39964, 13036, 29252, 56252, 38820, 34216, 4196,
                39920, 57976, 28172, 34636, 6220,  14044, 20628, 61120, 41016,
                24260, 13448, 19592, 8516,  26092, 36700, 17900, 30160, 57432,
                46080, 49704, 40596, 7876,  24332, 20720, 9124,  50652, 39160,
                60028, 1852,
            },
            {
                33788, 8596,  18548, 40928, 15324, 47088, 64756, 53896, 4300,
                59064, 29972, 2272,  16212, 24404, 58108, 6352,  18740, 3376,
                60576, 13928, 55148, 22632, 11316, 59088, 28748, 8132,  37372,
                4848,  45848, 22932, 41844, 6072,  64540, 10636, 51712, 21992,
                36296, 63040, 1240,  41584, 10656, 30764, 7460,  48048, 13340,
                32304, 42428, 6520,  46560, 30392, 53736, 20012, 7904,  37600,
                61488, 28904, 40180, 63988, 2568,  36736, 42372, 22004, 7220,
                32268, 47300, 5080,  29652, 9016,  39752, 49080, 5280,  56984,
                2540,  20580, 10016, 4232,  62144, 22864, 2572,  31916, 8360,
                60852, 1016,  18440, 61724, 22048, 53968, 13716, 7016,  57836,
                33332, 55428, 35892, 9136,  59460, 22524, 15020, 9780,  64736,
                23736, 53296, 852,   45400, 24836, 39080, 48228, 27264, 11072,
                58556, 50592, 39360, 46408, 61864, 11560, 3780,  52160, 13908,
                644,   65012, 33244, 28900, 60348, 47272, 13416, 30664, 6048,
                22192, 53264,
            },
            {
                13700, 64124, 45524, 604,   60716, 20088, 8352,  28640, 42460,
                46788, 62148, 34416, 48804, 43452, 63432, 12592, 40500, 52680,
                48176, 37240, 29724, 7336,  39236, 24688, 52204, 14936, 21336,
                55964, 62084, 10176, 29460, 15436, 37396, 33324, 46360, 5104,
                18040, 48216, 23948, 53400, 19812, 45344, 26020, 62676, 39636,
                9260,  50288, 57472, 11556, 38320, 568,   60040, 44432, 26804,
                54388, 11336, 23236, 6424,  52412, 12644, 60192, 56864, 18264,
                53740, 62108, 23108, 51212, 33480, 65336, 21240, 28044, 41384,
                24468, 52392, 35148, 26836, 40672, 14164, 29276, 58420, 20236,
                53188, 43704, 25520, 30216, 9980,  35480, 64104, 46676, 28584,
                11184, 47552, 24876, 52196, 44640, 4,     50368, 28496, 46340,
                31564, 16588, 11860, 61692, 54756, 9092,  57548, 3604,  44320,
                36916, 0,     28440, 56532, 22232, 49500, 33872, 43624, 25164,
                38212, 22264, 11472, 52328, 2212,  34392, 54476, 64456, 43592,
                16764, 28420,
            },
            {
                49756, 38648, 23276, 28028, 51144, 32988, 40300, 14364, 24920,
                10444, 21176, 13264, 37652, 8544,  28456, 33708, 56892, 21944,
                27112, 1212,  44000, 19272, 57128, 1916,  41572, 60812, 33920,
                48364, 18840, 2512,  53832, 60260, 24692, 48,    57504, 39820,
                61008, 9036,  13432, 57024, 38860, 60664, 5192,  16716, 56528,
                1128,  24196, 28964, 17040, 22516, 49612, 32024, 13848, 3940,
                46308, 16412, 35064, 43524, 19288, 30840, 4860,  39860, 10968,
                35252, 2004,  41360, 58924, 11948, 824,   53212, 15460, 45840,
                11768, 60688, 16044, 49376, 7772,  54680, 47216, 42296, 24088,
                35912, 12144, 47368, 7192,  51796, 42560, 1488,  18772, 40792,
                15856, 2396,  63200, 5600,  32096, 41356, 62004, 18988, 55352,
                7100,  37276, 22008, 42032, 29216, 17312, 35452, 52316, 31144,
                62860, 17700, 6512,  32372, 14948, 40760, 55236, 20196, 60524,
                4992,  56484, 17072, 42724, 6324,  18508, 40832, 26480, 3176,
                36432, 61056,
            },
            {
                7304,  3916,  54684, 35548, 11992, 5040,  56012, 63068, 50488,
                3048,  55060, 26816, 51828, 620,   54312, 16464, 45628, 10244,
                59744, 15656, 62476, 50748, 12760, 47148, 31004, 4040,  11028,
                26192, 43060, 31900, 39608, 49472, 19904, 43564, 11756, 30548,
                25944, 42832, 32944, 27928, 3664,  35164, 51016, 21748, 33760,
                44624, 64420, 35904, 52360, 61576, 9452,  42500, 18744, 64320,
                50800, 8832,  57800, 61064, 47824, 27740, 50500, 23680, 45636,
                26860, 14792, 37284, 19232, 44668, 25028, 31704, 59588, 8100,
                38024, 29064, 44328, 56520, 19760, 37088, 10948, 62912, 5664,
                15440, 56016, 64916, 37892, 16772, 27196, 60440, 23456, 52436,
                58976, 30428, 20048, 38196, 16984, 26620, 12324, 35668, 3752,
                43188, 59884, 47968, 32800, 5316,  65272, 21468, 14848, 7584,
                23068, 54320, 42844, 64868, 2536,  27436, 7064,  10184, 31344,
                45464, 27996, 48660, 63104, 37236, 56808, 22808, 9996,  58096,
                46240, 20100,
            },
            {
                56392, 31660, 16600, 44092, 59732, 22300, 37624, 18128, 31128,
                35760, 58512, 44904, 18928, 61632, 23532, 4356,  36584, 31236,
                6748,  40828, 32560, 25752, 35368, 64608, 16584, 54692, 38372,
                51652, 6328,  58584, 13188, 8292,  28252, 62244, 16820, 55092,
                49688, 2672,  51820, 65108, 18296, 12380, 43088, 29620, 54016,
                14172, 19464, 3392,  40880, 5908,  34564, 58348, 24508, 36628,
                29832, 40668, 25392, 52,    14484, 38292, 65044, 2980,  52648,
                60464, 8040,  49588, 28448, 55408, 7284,  36428, 48276, 18848,
                63728, 1360,  32504, 5308,  64728, 25824, 480,   32780, 50116,
                27988, 2476,  31456, 21052, 49644, 56372, 4704,  31956, 36896,
                7640,  43616, 49432, 56780, 8464,  53844, 47232, 57928, 24924,
                10684, 52460, 14532, 1988,  50948, 37772, 46440, 26512, 59300,
                48064, 11388, 35656, 20468, 45800, 59084, 37084, 62436, 50876,
                15076, 35828, 8508,  25536, 13000, 31636, 49340, 15692, 51740,
                29664, 11424,
            },
            {
                24892, 48252, 62184, 9568,  26684, 49012, 1256,  45756, 7848,
                41716, 15680, 5856,  39540, 30448, 47716, 41940, 64664, 51484,
                18612, 46588, 53980, 9420,  5188,  20900, 27884, 45412, 23268,
                62936, 17492, 35792, 22316, 45232, 52704, 4864,  35496, 21392,
                6996,  15820, 36900, 22392, 8036,  48292, 58480, 2420,  10496,
                38428, 59512, 48324, 25656, 13012, 28304, 47256, 1896,  56492,
                6632,  18200, 33156, 55040, 21640, 9908,  34524, 16080, 31560,
                42748, 22092, 64164, 4060,  17004, 62400, 42208, 3736,  54844,
                23712, 50716, 13296, 22188, 43160, 16300, 53636, 39720, 18488,
                60540, 41304, 45136, 5936,  13524, 34192, 10224, 48368, 14192,
                61828, 26408, 12104, 23116, 65084, 33672, 1728,  20856, 39868,
                30196, 63912, 27200, 19700, 57276, 9932,  42952, 2328,  33868,
                40680, 29364, 51900, 8920,  24620, 48752, 13248, 18996, 41808,
                3484,  53588, 21192, 59836, 46892, 4688,  62216, 38916, 64,
                34188, 42132,
            },
            {
                13500, 39272, 2064,  20484, 42356, 52952, 13688, 60932, 27476,
                23092, 64944, 33236, 12064, 59788, 9464,  20604, 13392, 25340,
                57552, 2712,  22792, 38912, 59572, 42728, 56548, 7864,  14084,
                712,   30116, 54872, 65316, 2188,  38632, 32128, 47492, 63852,
                39204, 59604, 46192, 54820, 30332, 40476, 24376, 63752, 27128,
                46312, 8008,  31084, 55172, 63208, 16216, 53476, 21580, 11528,
                44248, 61648, 51776, 5196,  42104, 46780, 58964, 7024,  19828,
                56624, 11412, 46272, 33588, 39136, 12740, 21372, 29656, 10592,
                34944, 40608, 57536, 47936, 36048, 28836, 58764, 46716, 22624,
                8348,  53052, 24568, 57868, 62768, 40492, 44280, 21596, 54608,
                784,   39984, 35072, 4288,  45356, 29372, 14724, 61320, 50080,
                17736, 7884,  34928, 40944, 24436, 13084, 60680, 55128, 19424,
                13980, 4448,  60992, 16812, 56280, 33272, 724,   28932, 23612,
                64400, 33088, 40448, 1416,  19188, 43328, 27572, 8116,  59688,
                21812, 63948,
            },
            {
                52844, 28336, 35108, 56764, 7148,  30356, 34356, 57244, 10920,
                47448, 53168, 1664,  25488, 50076, 35092, 55108, 1112,  38376,
                29688, 11204, 62828, 15072, 49332, 3040,  37044, 33132, 50540,
                40652, 47820, 9392,  26908, 14992, 57980, 24108, 9728,  13824,
                29024, 24736, 11056, 848,   61424, 35740, 16408, 5768,  52608,
                33488, 17552, 22172, 43572, 4208,  37824, 41288, 31672, 49904,
                15184, 38628, 27408, 12784, 63256, 24024, 53456, 28292, 40496,
                36580, 1124,  30148, 51444, 58084, 26164, 45108, 52776, 59236,
                16708, 26876, 3476,  9808,  60936, 6540,  12492, 3296,  34860,
                14504, 36668, 11444, 29540, 19356, 2776,  25876, 64648, 29316,
                18528, 57944, 51632, 16232, 40972, 52808, 6612,  43984, 36728,
                3052,  45616, 54796, 61784, 49064, 31876, 28580, 6920,  51228,
                64208, 25844, 37700, 43456, 5584,  39756, 52672, 44564, 57860,
                7276,  12628, 55892, 30336, 52112, 35540, 23824, 55048, 17632,
                45104, 4576,
            },
            {
                58044, 18184, 46456, 65448, 15748, 23852, 40888, 19112, 3148,
                36436, 20540, 43528, 56504, 17480, 6840,  28092, 43332, 60472,
                48608, 34092, 44580, 26964, 30860, 19320, 10652, 58736, 24780,
                60516, 19824, 43588, 33328, 50228, 18892, 41492, 55780, 1624,
                44792, 53712, 34008, 19912, 44176, 13072, 50476, 20808, 42248,
                57408, 332,   60736, 50728, 10744, 19928, 59356, 8056,  65436,
                23304, 1464,  47884, 30676, 36484, 17032, 2264,  45184, 13252,
                62100, 54224, 23980, 15616, 8840,  48816, 124,   32412, 6636,
                43324, 62320, 20616, 31544, 52568, 24524, 41796, 30648, 55800,
                62168, 50668, 868,   39028, 47488, 53280, 15600, 37420, 6776,
                46852, 32564, 9376,  21148, 59444, 26128, 10452, 23472, 31656,
                58908, 12440, 22784, 6012,  352,   16176, 39908, 36084, 23492,
                44856, 1048,  58016, 30152, 21916, 62892, 10504, 16344, 37768,
                49076, 26072, 45736, 9060,  15288, 65128, 11876, 48132, 32388,
                37496, 9284,
            },
        },
        {
            {
                8652,  34276, 26960, 37524, 52168, 17468, 4136,  59972, 10732,
                50264, 57144, 47412, 25560, 58504, 50848, 23444, 63020, 10760,
                3520,  51876, 14244, 9304,  56888, 64028, 36092, 51296, 30972,
                44736, 25384, 49076, 16,    39984, 19504, 54544, 58964, 31852,
                25160, 34452, 6216,  16744, 51556, 33832, 14092, 23996, 17292,
                63636, 38672, 21328, 44784, 62372, 7432,  43004, 19748, 416,
                53832, 21720, 39388, 56148, 24140, 42900, 29220, 54280, 61148,
                17652, 36060, 5072,  42068, 9300,  54936, 16272, 3188,  53768,
                30420, 9964,  24740, 34600, 59716, 46300, 53912, 20120, 13604,
                6404,  59964, 2588,  22316, 44680, 30596, 55208, 34356, 21836,
                2936,  9640,  41884, 65512, 48044, 59648, 38580, 51232, 32140,
                35344, 27488, 4504,  36668, 47952, 15788, 34408, 62616, 46384,
                51788, 59712, 41660, 20588, 49588, 30148, 7596,  12144, 33380,
                49092, 27620, 6528,  19772, 42028, 16288, 9900,  51500, 27540,
                55532, 13776,
            },
            {
                44240, 53824, 12,    10208, 41804, 62228, 32880, 24848, 41200,
                15748, 27524, 38732, 13704, 33436, 5392,  41160, 17928, 54196,
                21744, 48272, 61244, 24088, 45752, 32064, 2444,  22284, 62192,
                5652,  14152, 21120, 56552, 33132, 12536, 49692, 15744, 1348,
                48492, 53116, 59756, 23352, 29156, 48896, 61424, 40972, 5588,
                50116, 13416, 34564, 2160,  18144, 35552, 27132, 63268, 10340,
                32912, 47556, 28380, 2496,  35520, 62484, 14464, 44668, 23552,
                7704,  46244, 20536, 62784, 37864, 23312, 33060, 59208, 40724,
                19752, 63860, 56236, 50628, 8016,  16508, 3708,  44252, 61668,
                30344, 48144, 17132, 51328, 37412, 25232, 5696,  48280, 52164,
                60512, 29096, 19840, 36800, 1160,  26100, 20488, 43120, 57536,
                14436, 61196, 54624, 12016, 63836, 6896,  39492, 23992, 12452,
                18936, 5016,  24776, 65508, 3204,  40088, 24156, 52640, 21296,
                15496, 51752, 64984, 36212, 856,   26268, 57040, 32780, 1808,
                21528, 47996,
            },
            {
                16896, 63060, 19880, 58656, 14248, 27996, 48440, 2628,  64288,
                34596, 7708,  54420, 19728, 65504, 9408,  46160, 26844, 38348,
                32568, 7164,  28272, 39532, 4536,  12232, 47708, 17208, 27552,
                41252, 58824, 37056, 46452, 28664, 6696,  63876, 38004, 21612,
                43720, 13772, 3012,  40580, 9832,  19844, 124,   35880, 58604,
                29760, 54532, 6828,  60448, 48860, 52972, 13628, 45848, 38032,
                60712, 42620, 16512, 59620, 48964, 9392,  19004, 33988, 58984,
                50712, 31036, 53056, 14808, 1788,  49032, 28220, 12560, 6588,
                45708, 15432, 916,   28540, 41360, 32040, 26200, 51640, 38540,
                9788,  26780, 40964, 56988, 8580,  63076, 18608, 39292, 11692,
                32044, 46964, 6788,  53360, 33176, 55292, 13280, 3212,  23240,
                8576,  41736, 17808, 24896, 50788, 30164, 57952, 43016, 55504,
                31300, 38320, 45300, 55060, 33732, 17876, 56796, 61684, 1752,
                39660, 9636,  24564, 13196, 58720, 49636, 38224, 12504, 64272,
                36852, 5364,
            },
            {
                24164, 40596, 31400, 49964, 45268, 7544,  54968, 12772, 44464,
                22444, 59056, 2240,  43060, 49216, 31296, 56444, 1016,  12472,
                59832, 43820, 16468, 54764, 20504, 37436, 60316, 55896, 9444,
                50160, 32176, 4128,  16252, 60600, 41624, 26300, 10380, 30408,
                61088, 18612, 36084, 55888, 65304, 43432, 26748, 46736, 8524,
                18740, 42552, 25456, 31652, 22364, 40120, 4396,  30512, 17736,
                24992, 6372,  12520, 31008, 20984, 54916, 40652, 484,   11816,
                38504, 4116,  25452, 34396, 55876, 18848, 43468, 51892, 36548,
                60760, 33284, 48180, 20752, 62604, 12044, 58600, 18384, 436,
                34968, 63796, 20472, 3724,  33588, 14932, 43356, 58356, 32,
                17200, 40584, 61900, 15040, 9884,  45092, 62712, 29196, 48344,
                52980, 30960, 38420, 1416,  44172, 19720, 3504,  10080, 16484,
                49376, 1244,  28428, 14652, 10388, 47516, 4668,  36816, 28840,
                55808, 32692, 43944, 53440, 29668, 45960, 4108,  18336, 43484,
                29004, 57836,
            },
            {
                50884, 12092, 4888,  25536, 35476, 20980, 39132, 31308, 18248,
                50956, 37504, 29228, 11536, 24192, 36232, 15848, 62316, 50224,
                19708, 34844, 1924,  63700, 51068, 29092, 42412, 34088, 1268,
                19704, 63224, 10860, 23868, 52308, 2552,  45532, 50772, 56360,
                7464,  27516, 46860, 31920, 5288,  15692, 54964, 12192, 23248,
                52296, 64544, 10916, 46032, 15664, 58820, 8844,  54488, 49284,
                3028,  57024, 51660, 37564, 7052,  46016, 25760, 63072, 28328,
                15900, 57364, 61796, 10384, 40028, 7532,  65260, 2720,  21764,
                25944, 10952, 38652, 4736,  54672, 36132, 5740,  46956, 23716,
                55300, 43688, 13064, 29704, 47188, 53432, 23188, 28084, 36316,
                56472, 24416, 49824, 27404, 21912, 40072, 6060,  37380, 18980,
                64464, 4852,  46944, 59788, 33440, 53924, 27424, 64528, 34692,
                59016, 8240,  52568, 22068, 60416, 43052, 26108, 13840, 44980,
                18756, 61156, 5228,  17108, 8532,  21192, 60536, 25184, 53988,
                9428,  33516,
            },
            {
                1976,  37948, 56584, 65268, 16684, 1372,  61728, 57828, 9736,
                4984,  62776, 15352, 53436, 60556, 3680,  21380, 28384, 40168,
                9024,  58196, 47028, 25576, 10564, 6956,  14352, 23360, 45504,
                54092, 39356, 29536, 57156, 35972, 18460, 22564, 34260, 3996,
                39340, 63296, 11684, 24408, 51916, 38616, 30444, 62160, 33260,
                39696, 4188,  36972, 56404, 1088,  28596, 64008, 36004, 20680,
                61792, 33688, 44404, 26884, 65300, 3852,  32840, 53192, 48684,
                42588, 21688, 45012, 29292, 47736, 24248, 56976, 16536, 31564,
                49492, 58144, 42760, 24028, 14912, 50180, 29792, 40712, 61284,
                15580, 6816,  49924, 59380, 38064, 10888, 4500,  64744, 8248,
                44880, 12364, 3916,  34992, 59564, 51684, 16504, 56320, 34236,
                11488, 26432, 15380, 22484, 8684,  13536, 47848, 37708, 21060,
                25512, 41832, 63088, 36076, 30608, 6364,  51104, 64424, 8344,
                48876, 22828, 40840, 35796, 63612, 33344, 39372, 6112,  48960,
                15292, 59960,
            },
            {
                19216, 28296, 46920, 8828,  42644, 50512, 28848, 23580, 47472,
                34044, 26056, 45528, 40512, 7972,  47984, 54872, 44540, 5264,
                30716, 23236, 14940, 40888, 32848, 58452, 48428, 65008, 26536,
                5904,  13644, 43204, 48364, 8068,  64868, 12800, 53292, 15992,
                42924, 20856, 50080, 1152,  59504, 21520, 7076,  49212, 1952,
                15348, 28196, 20012, 50676, 33736, 43804, 12012, 26012, 41604,
                14376, 23204, 9520,  18124, 57676, 14956, 22792, 7816,  18472,
                34920, 2192,  6252,  14284, 59592, 32540, 11836, 38400, 54320,
                8292,  3528,  18324, 64736, 44588, 8428,  21180, 53736, 11236,
                28772, 33904, 22180, 1320,  25964, 61376, 31672, 49196, 21244,
                52644, 30092, 63016, 42708, 1644,  31168, 46484, 24148, 2104,
                43424, 51484, 55944, 36476, 62532, 41288, 5524,  52392, 2232,
                46440, 12768, 18352, 3308,  40508, 57988, 20036, 34732, 31548,
                48,    57820, 27668, 3580,  50112, 14500, 55936, 30324, 41040,
                22616, 45256,
            },
            {
                62048, 53016, 14148, 22372, 32604, 53900, 6492,  15064, 41500,
                56524, 19884, 732,   30832, 17336, 33184, 37692, 13792, 64784,
                51924, 56260, 36540, 4692,  53608, 19472, 2916,  37860, 17040,
                35224, 61028, 20792, 900,   27028, 40568, 31848, 59984, 24856,
                57788, 8712,  33796, 14240, 36376, 45696, 17880, 56296, 42864,
                60208, 47612, 8672,  61356, 23492, 17416, 51852, 7000,  47156,
                780,   39524, 53616, 48172, 41908, 36044, 59932, 39424, 10608,
                55072, 64588, 51072, 36836, 19452, 160,   42468, 27316, 45272,
                61920, 29836, 34724, 56412, 26696, 32660, 59820, 2224,  37912,
                45920, 65016, 52400, 40172, 16440, 45384, 18888, 34524, 14168,
                38800, 5584,  17544, 56984, 20192, 12780, 8708,  58660, 28360,
                61884, 40336, 5812,  18504, 28712, 57424, 24344, 15132, 61912,
                29696, 33632, 56656, 48108, 15336, 24096, 11020, 39396, 52244,
                16236, 12588, 47040, 54396, 24476, 44392, 11172, 624,   64728,
                35376, 7564,
            },
            {
                42036, 31100, 3568,  37016, 59684, 11448, 44780, 36448, 3080,
                60856, 12684, 52020, 64392, 22568, 57848, 10408, 26096, 18692,
                552,   42448, 11176, 27916, 60668, 44364, 30532, 52424, 56876,
                9744,  49788, 32552, 53644, 15280, 44532, 5472,  46960, 1960,
                37316, 30104, 54996, 43544, 27696, 64152, 9740,  25580, 35360,
                22220, 54412, 13508, 41076, 5504,  38496, 57636, 31448, 54976,
                63496, 28076, 4644,  30740, 12208, 1776,  50656, 29672, 45252,
                23608, 31660, 26856, 40960, 53328, 63336, 48512, 5596,  20236,
                14708, 50640, 11672, 696,   39376, 13652, 48240, 17444, 25568,
                4344,  19160, 9972,  57932, 7340,  55796, 41224, 2992,  60292,
                55180, 23536, 48000, 36568, 26852, 65112, 39460, 49608, 14572,
                20832, 9656,  31612, 49888, 52,    45552, 32212, 19408, 43452,
                6928,  50740, 9768,  27200, 53968, 45732, 3820,  62724, 25324,
                42732, 60068, 37124, 9664,  18236, 61472, 27880, 19812, 51384,
                12632, 26248,
            },
            {
                48856, 17264, 55884, 40644, 26792, 19556, 63456, 25352, 49580,
                31720, 38168, 7280,  27272, 49060, 41692, 6468,  59840, 46216,
                33780, 21212, 63128, 47532, 16544, 8076,  22700, 12332, 42080,
                28760, 3976,  24504, 58136, 37560, 61532, 10664, 28680, 19268,
                48920, 62084, 16964, 4896,  52520, 2448,  40604, 12376, 30020,
                5872,  44512, 31856, 26164, 65144, 2248,  45144, 20088, 10900,
                16644, 34972, 59308, 20448, 55244, 26364, 17540, 61440, 3352,
                57752, 12340, 7656,  16472, 22940, 9264,  34140, 59096, 24876,
                37288, 41416, 22624, 46632, 53288, 5408,  62024, 34628, 57268,
                42932, 48648, 31516, 36104, 23260, 28692, 51592, 9588,  26104,
                43824, 32328, 11072, 7368,  53712, 44208, 32904, 6184,  54580,
                35088, 47720, 23604, 64884, 12344, 38920, 9196,  54880, 59244,
                39712, 23256, 65060, 812,   37192, 59524, 30160, 18036, 54780,
                6920,  28792, 20608, 32496, 2744,  58080, 37696, 43092, 31956,
                58752, 2612,
            },
            {
                21272, 10448, 64220, 5544,  47912, 344,   56252, 17004, 9116,
                21844, 55340, 42936, 14848, 35340, 2404,  51176, 29584, 38872,
                15308, 50024, 3588,  24748, 38692, 34348, 62648, 272,   47444,
                19892, 64252, 45788, 8944,  17840, 22808, 55412, 51632, 40236,
                6848,  11948, 25648, 59928, 19628, 32504, 57944, 50128, 63212,
                19188, 460,   59072, 48752, 35572, 14556, 28744, 60232, 37444,
                24904, 50048, 43556, 8800,  64284, 46132, 6936,  41348, 14732,
                37460, 49332, 61756, 45780, 56424, 30372, 13972, 51796, 1832,
                55520, 62800, 7416,  58460, 19484, 30236, 41784, 22988, 7740,
                54140, 27368, 14804, 63004, 4820,  13236, 61908, 47168, 16152,
                876,   63548, 41448, 50256, 15696, 22884, 556,   18944, 37928,
                59500, 2592,  42952, 16444, 52224, 60192, 35588, 27468, 2632,
                12200, 36272, 19136, 32076, 43404, 21540, 8948,  48320, 35428,
                2408,  45800, 65332, 40600, 52584, 5784,  48104, 8540,  16092,
                54112, 38368,
            },
            {
                60864, 33116, 24296, 51268, 13864, 30672, 34716, 40436, 52548,
                28980, 3824,  45968, 61200, 11232, 20208, 63396, 23356, 55352,
                8488,  59104, 31408, 54728, 13312, 51696, 40684, 25848, 59416,
                36184, 14192, 39932, 50980, 1768,  31216, 35444, 14108, 64552,
                21952, 34116, 42092, 47492, 38236, 22952, 46416, 15896, 36672,
                52372, 39012, 11272, 17728, 55188, 22404, 8748,  48052, 3224,
                41184, 6092,  14908, 32084, 38272, 22620, 34496, 53716, 27932,
                20776, 43068, 1328,  34872, 5312,  39416, 18964, 43324, 10592,
                28912, 17228, 32528, 25720, 36764, 10428, 51424, 15568, 63444,
                12256, 312,   40916, 50600, 44608, 37768, 30540, 20284, 34900,
                58760, 28188, 21132, 4120,  60060, 29988, 62068, 46212, 25584,
                11808, 29300, 56280, 33840, 26288, 3960,  21900, 49020, 17476,
                53424, 45428, 60896, 14588, 6496,  51444, 13180, 41312, 61176,
                23032, 11324, 50028, 14980, 25768, 34792, 21736, 63480, 24852,
                5332,  45712,
            },
            {
                28836, 42828, 7976,  36148, 44204, 61696, 7400,  47032, 12368,
                64104, 33788, 17596, 24952, 56812, 31556, 44300, 5048,  12360,
                27044, 36036, 43340, 5992,  20428, 58056, 6908,  32028, 17288,
                55068, 6144,  33744, 27416, 63012, 43024, 4040,  26084, 45192,
                56916, 500,   53868, 9368,  13816, 6348,  28420, 3424,  9048,
                61588, 24680, 29296, 7132,  51312, 42964, 62900, 33500, 52712,
                57400, 22160, 61516, 51464, 368,   12832, 48480, 62996, 9532,
                33144, 54524, 18196, 25308, 50092, 59544, 27116, 64960, 35896,
                46080, 4776,  49324, 14208, 64564, 1680,  45740, 28268, 38376,
                32520, 20840, 59028, 25236, 18484, 1900,  54680, 40076, 6700,
                51156, 12468, 38548, 55776, 35612, 10008, 41000, 51844, 17024,
                63968, 4944,  39684, 20252, 7244,  47184, 13892, 63744, 41528,
                29360, 4444,  25948, 55600, 39056, 63604, 28248, 33188, 15884,
                56680, 38304, 30736, 7912,  57128, 45112, 12080, 30000, 56308,
                35760, 14040,
            },
            {
                1064,  53076, 17872, 57184, 20440, 27856, 23184, 59180, 5096,
                20096, 58132, 39124, 964,   52892, 36968, 16404, 40748, 61556,
                47948, 18348, 1192,  65116, 27980, 45460, 10716, 49356, 4256,
                44220, 23916, 11532, 56620, 20648, 48524, 59668, 8552,  17576,
                37764, 27564, 62380, 30836, 51248, 65244, 55376, 41080, 32924,
                20996, 45160, 56624, 40608, 32196, 4244,  26516, 12716, 18328,
                29976, 10292, 45592, 25896, 56848, 29336, 18496, 4512,  24556,
                58868, 6420,  29736, 62668, 11876, 2788,  47216, 7688,  21108,
                53560, 60472, 38776, 43712, 54664, 21696, 60152, 4624,  50032,
                56784, 46968, 34932, 7644,  52472, 11472, 65316, 23904, 57204,
                45040, 17496, 31724, 46772, 24996, 14180, 57508, 7652,  31940,
                44072, 49980, 14628, 53244, 62652, 43952, 31336, 56528, 8360,
                34272, 50476, 10336, 46516, 23520, 1760,  57712, 4436,  26176,
                52704, 19000, 920,   42568, 62832, 17820, 1840,  51472, 41212,
                19256, 49688,
            },
            {
                26464, 59348, 12276, 39616, 2440,  54024, 15840, 43012, 36592,
                27088, 49920, 9836,  46896, 7536,  27584, 50308, 3156,  33244,
                53804, 23080, 39336, 50384, 16364, 36524, 22500, 63724, 29428,
                52884, 60344, 41516, 16380, 7144,  37300, 12640, 53180, 32700,
                49488, 15004, 4404,  23876, 34884, 18244, 43692, 25460, 58208,
                12848, 48532, 2052,  15268, 59352, 19980, 46732, 39804, 1640,
                64892, 36204, 17016, 3864,  40448, 59852, 43632, 51992, 38120,
                46356, 14388, 40696, 44556, 22096, 52664, 37976, 15304, 31164,
                24200, 12352, 2896,  27444, 8496,  34248, 40280, 18392, 24468,
                10636, 3560,  16128, 60732, 27852, 47540, 32944, 15124, 3592,
                26784, 8500,  64432, 1936,  19908, 48584, 27824, 3324,  36156,
                23452, 9460,  57956, 28100, 36528, 10692, 39184, 24272, 672,
                20724, 58620, 16880, 30660, 35132, 19764, 49116, 43264, 8144,
                46820, 61692, 27480, 54304, 22680, 32068, 38760, 61192, 4488,
                9996,  64024,
            },
            {
                38216, 5916,  30504, 49228, 65052, 10604, 32316, 51588, 1620,
                13688, 41928, 30288, 22216, 65356, 13440, 58772, 19828, 45048,
                10032, 14336, 60016, 30308, 8192,  55448, 42696, 13464, 37828,
                19560, 1232,  31000, 51448, 45404, 28860, 23680, 40992, 2544,
                60340, 20164, 45892, 39864, 59076, 10948, 1200,  50416, 16748,
                5896,  34608, 64664, 23980, 10264, 36900, 61068, 56104, 23168,
                49348, 42052, 55200, 32740, 20520, 10848, 35184, 15724, 1932,
                65464, 10132, 57052, 36152, 16828, 32248, 56356, 4148,  42336,
                57256, 34868, 20116, 51812, 15804, 30696, 56176, 7188,  44268,
                36636, 64228, 30892, 42592, 37536, 21524, 9104,  43112, 49372,
                36804, 60288, 41732, 51644, 6592,  39216, 54484, 62928, 20580,
                60276, 47792, 1892,  17768, 22692, 3484,  60748, 15728, 53976,
                37200, 64596, 42792, 7376,  60548, 54308, 11088, 36352, 22324,
                13448, 33756, 4760,  37032, 10232, 49148, 14404, 26220, 46296,
                23028, 33648,
            },
            {
                47344, 21196, 44476, 23976, 34788, 6892,  46608, 61224, 24240,
                56240, 62596, 16516, 54224, 35024, 39920, 24464, 30852, 63568,
                6308,  25980, 56860, 46392, 2256,  33476, 61660, 3496,  26776,
                47784, 10144, 35696, 65324, 4384,  58436, 18700, 62780, 29880,
                43128, 11568, 56564, 7260,  48472, 21876, 36788, 29488, 60332,
                52592, 27600, 38200, 44144, 53904, 29808, 7600,  15736, 34244,
                5184,  27400, 8412,  47296, 63976, 5732,  54448, 27104, 31692,
                19332, 51020, 26612, 836,   8380,  64020, 25192, 48972, 61672,
                9464,  45808, 63684, 41324, 58892, 46932, 12728, 62528, 29012,
                51336, 22548, 12544, 55636, 6360,  57732, 61296, 29364, 53108,
                18440, 22644, 11696, 28700, 59396, 34072, 17296, 42808, 12796,
                38728, 30264, 34580, 42224, 59252, 51552, 29036, 48792, 44644,
                27320, 13044, 48188, 2508,  40676, 14656, 28572, 64864, 58400,
                41064, 51360, 16388, 45972, 59844, 3184,  58024, 35240, 53732,
                16308, 56516,
            },
            {
                13504, 62868, 3444,  16528, 58768, 40920, 28944, 17620, 38316,
                8320,  33508, 3000,  44452, 4872,  10596, 55844, 524,   37644,
                52032, 42488, 35256, 12132, 20912, 25080, 49764, 17996, 57140,
                40576, 61312, 21704, 25436, 14800, 34560, 54232, 10212, 6128,
                51436, 24780, 33772, 28052, 15540, 63488, 4676,  46320, 39280,
                8704,  20340, 3860,  18252, 47696, 24,    25656, 43316, 53224,
                12088, 59336, 15456, 24672, 50560, 38836, 22040, 60772, 48984,
                39908, 23148, 33652, 53940, 46640, 41032, 12176, 19036, 28512,
                412,   17328, 31844, 5952,  22492, 1448,  25572, 53500, 16980,
                41164, 944,   46312, 26072, 19128, 2180,  39676, 12888, 34536,
                28,    54792, 32224, 15260, 44504, 23780, 10252, 692,   53024,
                25260, 5528,  55684, 11936, 45540, 6968,  33888, 19228, 9112,
                4892,  32320, 21928, 25668, 52432, 18540, 45672, 256,   31076,
                6476,  20772, 63728, 30032, 25048, 19620, 43748, 6868,  29188,
                42540, 8088,
            },
            {
                31364, 52316, 36468, 27112, 51124, 12960, 21660, 4620,  45380,
                53020, 20380, 25252, 60388, 49744, 18764, 42024, 47384, 15388,
                21472, 29160, 4532,  64280, 38248, 52724, 9756,  31332, 5632,
                15184, 53356, 7660,  49596, 42856, 20,    47476, 39228, 21776,
                35984, 64760, 2048,  52808, 42848, 55276, 32632, 12072, 23364,
                42408, 63184, 56384, 33304, 13980, 62808, 50780, 19476, 30804,
                63528, 44624, 37360, 1184,  30012, 13268, 45772, 8560,  3628,
                12760, 58256, 4968,  60592, 20944, 29948, 5540,  35936, 52412,
                39768, 55184, 26488, 10872, 50500, 35164, 39024, 4176,  33396,
                9328,  59188, 53788, 34304, 49628, 15780, 45620, 25248, 63924,
                5976,  46904, 38436, 63252, 3884,  56292, 49844, 32744, 46180,
                65472, 15828, 49604, 19872, 26132, 64312, 14028, 41104, 61464,
                57856, 38972, 55560, 63032, 34152, 8768,  38444, 24864, 55508,
                47880, 11520, 39160, 8216,  52680, 40420, 12252, 64536, 20928,
                88,    60496,
            },
            {
                18628, 41260, 10220, 55852, 1204,  47688, 63324, 57076, 31176,
                12048, 48696, 36952, 28304, 14044, 32488, 62384, 27072, 9508,
                60976, 48680, 18516, 54776, 15572, 41352, 60108, 47072, 34772,
                44656, 27900, 38600, 12400, 57396, 31320, 26272, 15916, 56964,
                44772, 13176, 18076, 38512, 9220,  25840, 19144, 61664, 53984,
                15020, 31032, 49864, 10924, 40476, 58456, 35632, 9412,  39708,
                3292,  21716, 52132, 57260, 18672, 62592, 33320, 56652, 42416,
                29520, 44756, 17528, 37336, 14636, 50736, 43968, 59868, 23076,
                13584, 48212, 36700, 60432, 44044, 65468, 18784, 48644, 61360,
                21396, 14776, 38680, 4828,  28824, 62088, 54248, 7860,  20788,
                42044, 58896, 25952, 8600,  21100, 37228, 61444, 26756, 19360,
                8108,  58288, 39536, 1044,  31748, 37476, 55028, 2964,  24908,
                50580, 10904, 15872, 3716,  29620, 50016, 60780, 5144,  16768,
                35148, 26976, 57372, 1100,  34212, 15636, 56160, 31792, 50836,
                38640, 25040,
            },
            {
                49056, 4756,  59944, 20024, 39408, 33824, 8772,  25636, 42364,
                236,   64656, 40708, 5628,  58664, 22504, 53032, 6900,  34760,
                57216, 1916,  32096, 44108, 6840,  28576, 748,   23252, 19376,
                58368, 2056,  32708, 62460, 20516, 7088,  63936, 50404, 3136,
                28564, 61168, 47920, 30912, 59176, 380,   50652, 35568, 7116,
                2468,  25472, 44956, 6512,  27888, 21984, 4520,  46164, 56640,
                17280, 28212, 34056, 10136, 40560, 5196,  25416, 16284, 36072,
                63988, 9376,  49312, 27528, 62468, 10272, 55568, 2076,  33544,
                62892, 4972,  20896, 15528, 28748, 8164,  13624, 56332, 27500,
                45176, 30664, 64720, 11376, 43460, 23152, 31996, 37440, 50796,
                30328, 17076, 11364, 52380, 48064, 30152, 5800,  14232, 41644,
                36052, 29576, 23224, 62056, 52600, 9904,  21824, 47368, 30728,
                35168, 20328, 46284, 42616, 58496, 12380, 20636, 41536, 53504,
                61852, 44008, 23208, 49872, 61308, 45376, 26724, 3516,  46456,
                11748, 55220,
            },
            {
                27944, 32996, 45340, 24284, 29920, 15552, 53392, 35996, 18860,
                58108, 15816, 9040,  55116, 45616, 1504,  38696, 17380, 44776,
                23988, 39816, 11360, 25692, 62448, 35532, 13292, 51112, 64740,
                11144, 24392, 54648, 16400, 45936, 41256, 11424, 37256, 32468,
                8528,  21020, 4212,  23556, 40844, 14512, 44576, 27924, 48096,
                59760, 36760, 20632, 61248, 16348, 54360, 32112, 24828, 13460,
                62136, 48252, 6792,  60160, 44256, 49748, 54756, 292,   20288,
                52452, 24224, 2212,  40468, 7060,  32024, 25548, 16316, 46572,
                8940,  41676, 57628, 2652,  53184, 32156, 42132, 24120, 5316,
                37100, 7256,  51016, 20388, 57080, 3956,  9968,  59796, 14304,
                2532,  56408, 33912, 40156, 1300,  18120, 43840, 57688, 50576,
                2820,  10944, 44188, 47804, 5372,  17368, 43256, 63000, 14976,
                464,   53760, 27844, 7492,  36260, 23672, 47448, 31888, 9380,
                3988,  18796, 13708, 30900, 4792,  21796, 37196, 9156,  61996,
                34916, 15852,
            },
            {
                65392, 8308,  13836, 62308, 5548,  59068, 44432, 3712,  49988,
                29236, 24052, 34484, 19928, 30388, 12328, 50952, 63680, 29460,
                13660, 54212, 49696, 58420, 21616, 46128, 55768, 39480, 8052,
                43108, 36988, 48248, 6056,  28936, 23272, 55496, 17984, 59924,
                41916, 54716, 51108, 33536, 63096, 56068, 10696, 21420, 39668,
                17144, 56844, 52052, 1040,  42476, 37608, 65452, 50424, 1472,
                36664, 42692, 26148, 15168, 31440, 22412, 38020, 28356, 47160,
                13960, 59292, 34892, 54788, 21700, 42648, 58692, 38476, 19416,
                30108, 26208, 51332, 38612, 23332, 59700, 47356, 212,   52188,
                57908, 17396, 25772, 47336, 35144, 40968, 18308, 48376, 43280,
                27100, 65140, 45860, 23072, 55024, 62720, 11768, 32008, 21504,
                60944, 54528, 13932, 26540, 35288, 57412, 28352, 38024, 6172,
                59628, 40256, 65312, 17572, 51808, 1432,  63760, 14992, 28148,
                51244, 37472, 65136, 41036, 10512, 54200, 18204, 58884, 23284,
                43144, 2128,
            },
            {
                57344, 47916, 35540, 41940, 50472, 10956, 22728, 63944, 13456,
                38412, 46380, 51688, 62952, 41596, 25116, 36028, 3272,  59548,
                19776, 5140,  36444, 8676,  17428, 2904,  32396, 26640, 16928,
                30128, 60272, 2956,  52248, 35336, 61300, 1492,  47172, 25088,
                14916, 35480, 11656, 6952,  17612, 29200, 5188,  64992, 31884,
                8664,  12540, 33776, 29308, 47828, 10324, 7412,  18956, 29832,
                55440, 11400, 53420, 19816, 2860,  64976, 12040, 57920, 6964,
                39652, 30732, 18580, 45928, 65156, 12196, 3984,  48916, 53468,
                64120, 11688, 45748, 34640, 6240,  19512, 10476, 40048, 34064,
                12712, 42772, 62436, 2976,  15036, 52920, 62792, 24396, 6984,
                36332, 19964, 4580,  13948, 28584, 36704, 25476, 52628, 38796,
                6424,  33884, 18864, 40656, 60680, 3236,  12036, 50104, 24048,
                44572, 9652,  22472, 32628, 57032, 26632, 43856, 39268, 58676,
                33404, 1696,  46000, 56672, 27664, 48700, 39612, 5256,  28868,
                52208, 19712,
            },
            {
                37936, 22344, 3888,  54604, 19044, 27640, 40472, 32348, 7616,
                60636, 2188,  11468, 6568,  16688, 58172, 48124, 9952,  42580,
                33080, 46360, 65272, 30576, 42948, 53252, 61636, 4608,  49668,
                56468, 22196, 14060, 19200, 39732, 9400,  30632, 52500, 5412,
                44884, 27356, 58000, 47008, 37752, 42980, 49752, 54164, 24204,
                46540, 41612, 4408,  63320, 23264, 14492, 44384, 59704, 40520,
                22264, 4940,  61460, 35792, 46264, 8460,  50768, 32588, 43152,
                62252, 10652, 5088,  50108, 15800, 28672, 35628, 23216, 6440,
                33212, 912,   17492, 61904, 13968, 54952, 64356, 28896, 20824,
                60452, 31244, 9384,  39320, 27704, 33008, 1508,  29940, 57656,
                53936, 10308, 50132, 42608, 58500, 7956,  2276,  44892, 16148,
                28452, 64696, 49452, 7828,  22996, 52260, 32768, 19016, 56064,
                29960, 48880, 13424, 3536,  37820, 11356, 5532,  19352, 54992,
                12324, 25212, 20604, 7224,  15760, 32464, 63916, 14164, 47408,
                31488, 11004,
            },
            {
                53792, 26580, 16072, 30940, 57648, 768,   48040, 17088, 56052,
                21156, 43208, 26664, 54404, 32564, 4272,  21304, 27992, 15932,
                52948, 22696, 320,   27056, 14468, 37680, 20660, 12120, 41776,
                7160,  34000, 44724, 62876, 26136, 43380, 13492, 21480, 65180,
                38176, 788,   61612, 20612, 24840, 2832,  34880, 15208, 1688,
                61952, 50920, 26764, 17860, 53272, 57464, 34576, 25820, 47308,
                15808, 32264, 49512, 39580, 27384, 59392, 18016, 24736, 1736,
                22028, 53476, 26740, 33812, 572,   56736, 44676, 61108, 14552,
                55980, 42824, 24516, 49080, 30884, 44124, 36768, 15612, 53492,
                2288,  49288, 22984, 54608, 59100, 11312, 45016, 20960, 13232,
                39836, 33112, 62020, 30584, 17180, 48756, 63640, 10056, 23836,
                46596, 120,   56392, 30404, 15856, 46044, 63504, 41828, 7356,
                16540, 34780, 61140, 54468, 45236, 49984, 62004, 23404, 7680,
                42096, 49440, 60376, 35528, 51920, 23848, 536,   42624, 58072,
                7152,  40892,
            },
            {
                1464,  45132, 62676, 8984,  38684, 34648, 61384, 25360, 52028,
                36736, 31184, 59328, 49100, 38840, 44868, 64472, 56024, 38344,
                7520,  60812, 40364, 50604, 10036, 47644, 58316, 24696, 36296,
                64212, 27228, 10504, 54084, 3756,  58112, 49896, 33296, 56036,
                9976,  16672, 31716, 52060, 13208, 60628, 56336, 19336, 29896,
                37236, 21164, 58980, 6556,  39700, 31080, 1980,  5908,  62864,
                9720,  57816, 896,   23548, 13672, 41728, 34448, 56416, 14528,
                48308, 37204, 59908, 41232, 20180, 52328, 9128,  40296, 18816,
                28184, 37540, 59264, 9680,  3244,  26396, 5404,  47944, 8132,
                25488, 44728, 35992, 5796,  17664, 50008, 37796, 64224, 47320,
                16776, 404,   24160, 5644,  38372, 20900, 34344, 54876, 59948,
                40324, 20656, 12384, 43392, 37332, 2088,  10200, 26948, 58712,
                1356,  39256, 24772, 19688, 28816, 15700, 34944, 30396, 47380,
                63264, 17000, 29472, 3696,  44116, 61260, 10052, 34696, 25588,
                16932, 60924,
            },
            {
                12748, 33348, 49420, 24008, 13696, 45408, 6124,  10268, 41568,
                4452,  12816, 17964, 22936, 8572,  14000, 34836, 1424,  49396,
                12600, 25520, 18260, 55928, 63024, 33420, 3228,  45508, 18200,
                51428, 1904,  47860, 31540, 15688, 37000, 6488,  18476, 40680,
                23820, 48784, 43792, 7668,  40108, 27496, 10464, 45700, 40976,
                8200,  13820, 44844, 35828, 11736, 49304, 20500, 52464, 37844,
                27776, 44020, 17408, 64000, 7316,  54220, 3264,  45368, 63872,
                29868, 17268, 7724,  11932, 63288, 30944, 25736, 2620,  50940,
                46668, 6872,  20936, 52848, 40252, 57912, 22116, 62692, 41436,
                56832, 18872, 13200, 63696, 42380, 25932, 8012,  4364,  52612,
                27548, 59480, 43884, 51392, 57376, 26760, 41984, 13436, 31224,
                4340,  51900, 34572, 25100, 59020, 53652, 20336, 35980, 51004,
                46240, 63904, 5356,  42576, 8864,  60048, 744,   53376, 37184,
                4600,  10320, 40192, 57596, 13568, 37676, 19392, 55748, 48904,
                21732, 36264,
            },
            {
                28972, 58584, 5260,  42496, 63972, 21004, 54156, 15676, 28640,
                65288, 47212, 1828,  62064, 29040, 52556, 24976, 19028, 31704,
                58236, 43524, 35440, 5892,  21936, 29124, 8256,  54712, 30932,
                15052, 59488, 40400, 20092, 24100, 46800, 60788, 27816, 2264,
                62812, 29000, 55196, 3608,  63964, 32680, 48688, 3968,  60092,
                53380, 32952, 228,   56032, 24944, 64712, 16332, 43124, 12964,
                55168, 34124, 51768, 30612, 47940, 21392, 28164, 10276, 39996,
                4196,  55528, 43828, 23700, 47484, 38324, 13788, 34092, 64336,
                11392, 31732, 62328, 15440, 35136, 50704, 17908, 33256, 11196,
                28528, 61264, 32636, 584,   22060, 56568, 30776, 34860, 41664,
                22268, 8436,  35424, 14576, 10812, 2948,  47628, 6796,  18556,
                37632, 57556, 8956,  62332, 5164,  29212, 40464, 14224, 22328,
                31344, 12212, 53096, 33464, 57312, 22124, 40720, 12912, 18656,
                26020, 33224, 54324, 21248, 26940, 7548,  45480, 28416, 3396,
                64508, 51456,
            },
            {
                9580,  20480, 17244, 52368, 3200,  29748, 59440, 33576, 50908,
                19968, 55312, 35484, 40220, 57088, 5224,  42780, 61632, 10348,
                46660, 28752, 2112,  15876, 39484, 44100, 13800, 62420, 38428,
                22800, 5380,  56128, 35032, 63452, 8696,  42268, 12116, 51528,
                35232, 13828, 19600, 36936, 23176, 16052, 57448, 21636, 25808,
                16784, 63180, 28644, 46972, 9176,  40832, 29464, 59324, 23920,
                2504,  19784, 5500,  38748, 11844, 60676, 36924, 52148, 19672,
                25744, 34832, 51044, 2064,  57744, 5856,  60024, 21640, 41740,
                56852, 26156, 1780,  43740, 11940, 29248, 828,   46020, 38052,
                3836,  52588, 40452, 46864, 51720, 14448, 60172, 18388, 12032,
                62268, 48544, 55564, 19296, 64972, 33336, 53624, 61124, 24552,
                49740, 27896, 15072, 46868, 17952, 49272, 65040, 4476,  55516,
                8036,  26652, 18364, 48940, 3360,  46604, 26996, 64452, 43668,
                50812, 59248, 45916, 1168,  49972, 62408, 53452, 32236, 14944,
                6356,  43176,
            },
            {
                25140, 55384, 40432, 35748, 25748, 48264, 37868, 1068,  44388,
                24608, 6800,  31904, 45796, 11156, 17096, 50372, 36304, 22436,
                6824,  54724, 64316, 48232, 52088, 60256, 26024, 868,   50436,
                9892,  28108, 43728, 12752, 84,    30028, 54588, 21992, 45488,
                5684,  57348, 42128, 9008,  53172, 44040, 35712, 6864,  38772,
                49716, 42676, 22732, 18772, 60352, 3232,  33532, 7576,  46676,
                36996, 63132, 26424, 56668, 44424, 16844, 32308, 5984,  58180,
                13016, 62824, 31760, 14968, 28504, 18428, 45392, 53660, 4796,
                16608, 36976, 48316, 55424, 24084, 65176, 59092, 14592, 49680,
                24244, 7588,  16736, 27324, 10112, 37548, 45116, 1988,  54076,
                32144, 4008,  29140, 40788, 45652, 22192, 29824, 15952, 44528,
                64332, 1172,  42180, 33012, 23740, 11456, 44056, 33712, 60440,
                37852, 41936, 59344, 14308, 36440, 9852,  31412, 55592, 2300,
                8028,  23576, 15164, 30692, 39152, 17340, 11204, 40856, 58940,
                46872, 37836,
            },
            {
                61024, 36,    32172, 11180, 57228, 8260,  18572, 13156, 39664,
                9760,  59596, 14348, 21260, 26752, 63800, 30072, 3108,  59400,
                41148, 13468, 32916, 23872, 10840, 19356, 35848, 41904, 46936,
                32500, 65492, 51804, 18360, 48568, 58748, 38212, 16180, 60252,
                32276, 25776, 48016, 62132, 30188, 2764,  13288, 65320, 31084,
                9784,  4296,  12532, 52356, 37668, 50260, 21280, 56300, 11452,
                51040, 15452, 41996, 8868,  24092, 64,    65496, 49504, 42504,
                46740, 9032,  21356, 41932, 54496, 35852, 10500, 24728, 49536,
                29636, 8512,  18560, 32664, 5292,  39060, 8972,  53772, 19856,
                31276, 59452, 55916, 35356, 4480,  65488, 28904, 24576, 46724,
                39108, 16304, 25028, 7204,  58792, 1528,  39472, 9276,  34120,
                11924, 21552, 54416, 6400,  38884, 57136, 2752,  25276, 16604,
                47824, 428,   29456, 23384, 51704, 61724, 16028, 20860, 38636,
                60932, 11524, 36552, 65104, 5556,  24592, 34952, 1584,  23328,
                18588, 30792,
            },
            {
                13564, 43940, 65076, 15768, 46096, 61752, 23160, 55664, 63420,
                28004, 49564, 42292, 54616, 156,   47364, 39192, 15032, 53368,
                20000, 27216, 38256, 3912,  56600, 30356, 5008,  58864, 16712,
                20816, 6680,  36696, 24920, 4804,  33480, 26524, 3440,  9728,
                40024, 18032, 560,   11756, 26872, 51032, 19496, 54204, 46552,
                58732, 35192, 55460, 30592, 6452,  15172, 44948, 27264, 61568,
                31572, 4592,  48588, 59660, 34896, 54444, 27140, 14620, 23100,
                3316,  36604, 60608, 48660, 7556,  65048, 31968, 2828,  38528,
                63332, 58060, 40800, 51764, 21308, 45148, 27180, 33960, 41280,
                62932, 12056, 22428, 48304, 19248, 42960, 7508,  57652, 20712,
                10016, 63896, 36436, 50064, 13160, 26920, 51580, 56720, 4992,
                40500, 59228, 29376, 50904, 61240, 20028, 30436, 51948, 10068,
                21012, 54576, 64524, 5836,  40056, 44988, 4140,  34800, 48992,
                27792, 42920, 19524, 48192, 54892, 44032, 57044, 50484, 63732,
                8312,  53296,
            },
            {
                4440,  22572, 49860, 5640,  27644, 41208, 31120, 6076,  52404,
                34048, 17696, 4952,  36860, 60552, 8820,  24016, 33896, 45084,
                9340,  49856, 60808, 17276, 45824, 8472,  53488, 24380, 12184,
                55296, 39504, 60560, 14768, 53796, 41864, 19652, 50584, 64920,
                53980, 22584, 59200, 37904, 45784, 57924, 40764, 23504, 960,
                15628, 26472, 62516, 41428, 24472, 63764, 35468, 800,   17924,
                40060, 21820, 28784, 12528, 20176, 46276, 8092,  39084, 31220,
                56180, 28116, 17248, 980,   26284, 40440, 51624, 15320, 44604,
                12356, 23016, 3780,  61788, 10884, 56732, 17308, 2796,  6136,
                44444, 37372, 1544,  30144, 61956, 52924, 15076, 33680, 49532,
                59764, 784,   43624, 55104, 18064, 62916, 35684, 20084, 49008,
                25592, 46116, 17100, 9396,  35912, 14480, 46560, 40716, 62944,
                34704, 43528, 11840, 32796, 18924, 58648, 24764, 13172, 54708,
                6200,  32376, 58200, 3376,  9332,  29080, 20396, 12852, 27556,
                35932, 48560,
            },
            {
                56264, 29448, 34796, 19760, 51832, 1984,  36276, 45596, 15244,
                2520,  61340, 22756, 31380, 51584, 18976, 57804, 5456,  65228,
                31340, 760,   43088, 35064, 63628, 40020, 48528, 33216, 62292,
                27624, 2476,  45688, 29632, 10536, 62664, 46616, 13136, 29172,
                34956, 48928, 14496, 31656, 4960,  16600, 33920, 10508, 29192,
                38388, 44816, 18156, 2364,  47968, 12004, 53304, 42988, 9084,
                51888, 34252, 64324, 53216, 3980,  41120, 59116, 18160, 62104,
                11284, 51884, 38196, 57588, 12248, 22400, 61428, 19384, 54852,
                28128, 35060, 47452, 14988, 30916, 36884, 60956, 47732, 54752,
                25164, 15708, 51544, 40784, 11372, 23636, 38204, 4840,  41844,
                12936, 30184, 21788, 32956, 9648,  42304, 3620,  30080, 60980,
                13724, 37136, 3924,  63436, 26324, 1820,  55152, 7568,  27892,
                3948,  15428, 50460, 26420, 8424,  52888, 30048, 41236, 64064,
                17364, 51416, 25412, 14256, 34028, 41444, 60284, 5948,  45080,
                39892, 16876,
            },
            {
                10524, 41988, 60228, 38524, 54700, 12052, 58780, 21036, 25708,
                48392, 39068, 44164, 11620, 27576, 14048, 41972, 55084, 25556,
                16416, 52288, 22860, 12552, 26428, 20364, 15116, 1264,  43956,
                9436,  49908, 22136, 57336, 7292,  36012, 23180, 1212,  43800,
                8316,  3896,  40948, 64068, 24644, 7904,  62672, 48676, 59552,
                7016,  50896, 21944, 8628,  56940, 31780, 19612, 59936, 25356,
                57148, 2172,  44332, 17052, 36792, 29860, 51400, 1572,  34436,
                45020, 5216,  20556, 43732, 30304, 47836, 5328,  33792, 144,
                60264, 7580,  43288, 26456, 1304,  50396, 22916, 13384, 29548,
                64572, 9068,  57392, 33360, 3300,  60012, 28020, 56188, 17852,
                26368, 53412, 47148, 5388,  60248, 24040, 47684, 16384, 8232,
                55040, 31728, 21160, 44000, 49852, 32340, 59260, 18916, 37384,
                24104, 57720, 46880, 38492, 62236, 2816,  47676, 9940,  22064,
                140,   39324, 46352, 63236, 22556, 52904, 16044, 31392, 2416,
                58528, 25372,
            },
            {
                63516, 1520,  14644, 8332,  24640, 17556, 42996, 64800, 9320,
                29848, 57360, 7496,  64084, 46888, 34984, 1700,  48300, 11148,
                36740, 59172, 6980,  54864, 3256,  59780, 29136, 38184, 52672,
                35156, 18640, 41504, 32920, 16548, 51300, 56048, 38896, 61828,
                26980, 56664, 20844, 50168, 53312, 35916, 43028, 20432, 54560,
                32764, 13068, 60872, 34768, 39872, 27648, 3840,  37128, 14564,
                46752, 11292, 23476, 6296,  62644, 25756, 13144, 21920, 48596,
                25168, 54828, 14740, 63564, 35104, 8852,  58764, 25344, 41600,
                50208, 20784, 53648, 64792, 18444, 40216, 6720,  42840, 35768,
                19988, 49184, 27000, 18224, 44640, 47488, 8588,  50988, 36068,
                2968,  62576, 39432, 15228, 28056, 52768, 37880, 64036, 43308,
                996,   52136, 58064, 10864, 38468, 22824, 12416, 45280, 53012,
                61508, 30280, 1108,  21672, 34268, 17012, 57892, 36660, 44560,
                60176, 28940, 11160, 36452, 1912,  49388, 38508, 62356, 47668,
                19552, 33692,
            },
            {
                50516, 44592, 26956, 47884, 62848, 31808, 6772,  50464, 35512,
                13480, 53604, 20368, 3804,  38440, 59768, 21980, 29820, 39832,
                19756, 44620, 28480, 33568, 41628, 50228, 7908,  64692, 22780,
                13720, 59672, 3400,  64304, 26356, 5172,  31004, 18776, 15120,
                33828, 46448, 11308, 17460, 28832, 1588,  13472, 27016, 3388,
                40404, 25124, 53036, 46480, 15948, 64360, 48948, 6008,  30392,
                62100, 38916, 33068, 55524, 48088, 8928,  57356, 41812, 64576,
                7748,  32584, 40756, 2972,  49928, 18024, 45884, 37720, 10992,
                16592, 31700, 38460, 9188,  57812, 32436, 52732, 10304, 58432,
                924,   39656, 5320,  61624, 13152, 21040, 31272, 64684, 22976,
                44232, 7924,  19740, 58040, 34388, 2108,  10632, 22348, 33240,
                25076, 40320, 17796, 28156, 5912,  65404, 41684, 16428, 4684,
                9808,  41316, 13824, 56072, 43180, 11492, 27092, 5240,  31788,
                14868, 53112, 7112,  56864, 18396, 26816, 8384,  11828, 23844,
                54292, 7168,
            },
            {
                30772, 21648, 57292, 34328, 4172,  39924, 55540, 27012, 1256,
                41468, 17204, 33352, 25136, 56136, 15892, 6752,  63240, 50588,
                2492,  62112, 14560, 47600, 10332, 57428, 17236, 46176, 4472,
                31236, 54424, 10980, 48152, 39736, 45072, 58988, 9896,  52968,
                24308, 59724, 5488,  44176, 39300, 57036, 60488, 47180, 17812,
                64872, 5592,  29412, 348,   10972, 23096, 54288, 41852, 20764,
                50328, 17988, 27680, 42500, 14780, 31060, 38008, 3296,  16852,
                28876, 10792, 60704, 23372, 27348, 53484, 13376, 29728, 56708,
                62264, 3640,  24420, 13752, 46060, 27784, 63276, 16968, 24528,
                45764, 30904, 53120, 36784, 54888, 42464, 512,   15764, 11044,
                57192, 32388, 49812, 12020, 45724, 40980, 19012, 56788, 49708,
                12292, 6832,  62564, 47044, 55828, 216,   34020, 28660, 48624,
                35956, 19736, 64724, 6388,  51964, 23468, 63244, 50012, 56244,
                19808, 24328, 42868, 32220, 60900, 45448, 55316, 35296, 43200,
                14280, 38180,
            },
            {
                59900, 17032, 11464, 51352, 19976, 46600, 15084, 22776, 60300,
                45296, 62464, 51256, 29028, 10060, 42548, 53336, 26468, 32752,
                9028,  56004, 24600, 38404, 21216, 31624, 36516, 25296, 56440,
                42208, 27392, 36776, 23764, 14136, 20740, 2092,  48636, 42104,
                3116,  37132, 31972, 62288, 8884,  22084, 31476, 37432, 10772,
                44484, 56056, 20264, 37776, 43480, 58012, 8008,  26360, 59288,
                3436,  9344,  52492, 688,   61236, 20372, 46580, 53728, 35560,
                50752, 44456, 56368, 38676, 6940,  59680, 1684,  22576, 43664,
                6524,  47128, 55824, 34584, 2072,  21276, 5452,  48864, 34220,
                62376, 14972, 22376, 7440,  25796, 34032, 58788, 40136, 47856,
                28464, 37988, 4064,  25888, 65252, 54168, 5860,  29224, 46024,
                60040, 35784, 30556, 14392, 24484, 51932, 20460, 60476, 57280,
                23596, 50820, 32288, 45704, 29216, 37916, 18948, 40640, 2596,
                35300, 65388, 48872, 3936,  14032, 21680, 30348, 5824,  64876,
                27788, 2756,
            },
            {
                9100,  41216, 36656, 360,   28892, 58576, 10752, 37728, 31264,
                5108,  11404, 39468, 2040,  49204, 34528, 19152, 12240, 46436,
                17164, 42088, 5604,  53668, 1444,  61044, 12776, 6624,  51064,
                19608, 332,   61132, 7800,  53520, 62568, 34988, 27304, 64088,
                21664, 13008, 51228, 25312, 54984, 15668, 6188,  49712, 23648,
                34480, 15008, 50552, 62768, 32060, 17720, 33976, 12264, 35960,
                45500, 65124, 22396, 35056, 57860, 6672,  24692, 11972, 61496,
                21456, 528,   19180, 15424, 31976, 42016, 64096, 35692, 50696,
                26836, 18768, 40820, 51956, 61344, 44080, 38288, 54980, 12024,
                3792,  41188, 49900, 10712, 63956, 17916, 51672, 6156,  24604,
                54552, 17068, 59132, 21568, 31088, 14744, 36628, 62764, 16104,
                2428,  20260, 51080, 43056, 9312,  37700, 45856, 11884, 7504,
                2164,  39352, 16752, 10216, 1556,  55132, 14360, 8100,  45156,
                12284, 28032, 9256,  40200, 36808, 52100, 532,   57780, 19116,
                48956, 53044,
            },
            {
                45100, 64112, 54088, 23808, 61680, 43536, 5956,  52868, 19088,
                48424, 24456, 58976, 21412, 64852, 45164, 3472,  60620, 57520,
                36200, 30132, 64964, 18712, 48840, 27864, 45024, 63064, 39120,
                15012, 45536, 49552, 33448, 43248, 29812, 11900, 38424, 7304,
                56108, 46952, 17944, 152,   34680, 41712, 52532, 63008, 2732,
                58408, 41264, 7336,  25400, 2604,  52312, 48160, 40668, 16024,
                56448, 31680, 13296, 39824, 49620, 17772, 43464, 33152, 4908,
                26496, 47340, 36872, 52056, 9948,  48340, 20540, 14796, 8964,
                32612, 63168, 11700, 29392, 16116, 9712,  25376, 30480, 19568,
                56896, 27712, 59748, 44344, 2244,  29272, 37404, 13796, 63152,
                1804,  42312, 9876,  51516, 44104, 408,   24688, 9160,  42060,
                26800, 39128, 58960, 3740,  32888, 62180, 15824, 31048, 43900,
                27164, 54160, 59236, 26008, 62000, 33672, 48072, 30568, 61112,
                52268, 22304, 58392, 17784, 62600, 25320, 41872, 16212, 39116,
                32672, 22332,
            },
            {
                26528, 4928,  31416, 13748, 39088, 17896, 33196, 64492, 27708,
                56828, 15536, 36180, 7768,  13988, 30796, 23644, 38784, 4980,
                22108, 51212, 11964, 34428, 40488, 4640,  22612, 32472, 9484,
                58244, 28828, 22176, 17028, 5324,  57320, 19196, 51460, 15984,
                31108, 40692, 28008, 65172, 48288, 12168, 29332, 18832, 27136,
                9572,  30868, 47360, 12628, 59692, 28396, 6540,  63432, 1368,
                24632, 42812, 4156,  26824, 10404, 29400, 63748, 54972, 39624,
                58360, 13220, 64912, 29116, 57680, 25120, 3572,  40312, 53740,
                58992, 608,   22956, 36896, 49296, 2984,  59632, 41952, 47656,
                8204,  35128, 16064, 32384, 55868, 48136, 22164, 45220, 32896,
                20204, 35596, 61736, 7080,  39284, 60216, 47352, 52656, 32716,
                55096, 11168, 23448, 18232, 53860, 5520,  25240, 55728, 63716,
                34416, 14080, 47196, 21760, 42100, 6012,  58092, 24824, 17008,
                38300, 988,   47124, 31284, 6808,  49720, 11504, 60664, 46664,
                3532,  12464,
            },
            {
                35452, 18676, 48408, 8032,  56112, 2608,  49892, 9348,  41708,
                720,   32304, 44112, 53884, 41112, 56644, 50324, 28256, 15108,
                42984, 7952,  26028, 57760, 14812, 60032, 52420, 17504, 35388,
                4288,  65372, 11540, 54956, 40180, 1104,  25732, 59460, 43620,
                3564,  60588, 10372, 21032, 37784, 4240,  59856, 45956, 39508,
                54276, 21592, 61992, 37024, 19788, 44888, 22836, 53816, 29800,
                18900, 50568, 55344, 45932, 59492, 36616, 1516,  15980, 8596,
                30828, 42736, 6472,  2660,  34828, 43812, 62108, 30124, 18000,
                38280, 45520, 5028,  57544, 19096, 33636, 53284, 13944, 65100,
                80,    23964, 39252, 20232, 5844,  11984, 61212, 8280,  53256,
                28952, 46520, 15188, 26512, 34060, 12420, 17832, 21136, 58380,
                7184,  44244, 64516, 47280, 27988, 39940, 49880, 3020,  18968,
                41544, 8568,  4068,  37464, 52716, 11244, 20300, 43560, 5160,
                54668, 34036, 42728, 13132, 55400, 28412, 34656, 8640,  24212,
                56008, 62432,
            },
            {
                50744, 42596, 58388, 28332, 45952, 25176, 35700, 22248, 12708,
                46764, 63120, 4184,  25492, 18240, 9872,  1732,  63500, 46928,
                54492, 62348, 772,   45600, 29912, 8444,  43196, 55288, 25968,
                41720, 51724, 37740, 31764, 61936, 45212, 48824, 9148,  22948,
                35544, 53548, 6016,  33092, 57152, 43496, 24032, 14052, 1284,
                33964, 16280, 42796, 3488,  56676, 14936, 39548, 11080, 36944,
                61960, 8584,  15024, 21144, 5888,  52428, 22964, 44876, 51316,
                18372, 24256, 54312, 22156, 17036, 11776, 55392, 6776,  47964,
                13284, 25828, 52804, 41724, 63556, 27308, 6784,  22128, 36580,
                44764, 54152, 61820, 51532, 42688, 25264, 57824, 40372, 3084,
                11508, 56960, 49548, 22516, 55700, 29608, 50756, 2928,  37208,
                30448, 15400, 1008,  34340, 12812, 59816, 10024, 36312, 22724,
                51272, 60716, 31288, 17688, 64372, 27884, 35536, 50468, 63352,
                10152, 26708, 19696, 64440, 4564,  44752, 21072, 53748, 30064,
                37804, 15632,
            },
            {
                9776,  1724,  32968, 21124, 62984, 14828, 53612, 60632, 38112,
                51616, 20544, 29648, 48616, 61480, 37724, 33800, 20068, 10984,
                31104, 35456, 19520, 50288, 23884, 37556, 2512,  20644, 13100,
                46692, 2016,  23896, 8780,  20524, 14788, 34068, 29344, 63304,
                13804, 45544, 25900, 49840, 17104, 8160,  31648, 56088, 64344,
                48936, 5612,  52192, 29688, 8920,  49704, 32444, 59316, 5060,
                47164, 33764, 38604, 64504, 32544, 40924, 12900, 27432, 62980,
                36256, 49012, 61836, 39436, 46248, 51028, 27408, 22704, 33548,
                65380, 9252,  30624, 14764, 46576, 10688, 40264, 50832, 16804,
                29644, 9956,  13352, 2664,  30928, 15644, 35308, 27372, 17976,
                64928, 38144, 5480,  1512,  62796, 40908, 6040,  63992, 45640,
                24492, 49944, 38268, 56152, 21416, 42584, 16944, 46536, 29764,
                12408, 56080, 24500, 49360, 276,   14704, 40188, 3540,  29984,
                15488, 57608, 49164, 23984, 36288, 16856, 61856, 1340,  43428,
                6548,  59272,
            },
            {
                22912, 39596, 52076, 11436, 6064,  41048, 30956, 3556,  26576,
                16628, 7156,  35324, 11652, 55484, 6072,  26552, 58008, 41192,
                3432,  16476, 39632, 55964, 11380, 61164, 48436, 63656, 30256,
                57548, 16480, 60576, 28060, 50852, 6212,  54640, 38148, 2268,
                18184, 58120, 38648, 11908, 63512, 52940, 20144, 36844, 10440,
                25976, 60280, 23164, 34960, 65476, 25868, 1852,  43552, 21812,
                27024, 57056, 252,   24988, 17624, 47896, 60364, 5120,  32836,
                2272,  10568, 14608, 31768, 8072,  60504, 36904, 4336,  58036,
                43072, 20424, 35524, 5944,  23588, 56592, 31496, 62512, 4284,
                58944, 26384, 46988, 37020, 64308, 48820, 4544,  44848, 52280,
                23568, 43132, 32372, 19428, 44708, 10776, 34940, 27328, 13336,
                59424, 19904, 8912,  61848, 31228, 6544,  53568, 64824, 1096,
                38804, 5792,  42892, 34300, 46040, 59412, 55772, 23020, 46856,
                37280, 7012,  40624, 3004,  59312, 51740, 38940, 14372, 49004,
                19572, 27580,
            },
            {
                46288, 63584, 16436, 36992, 59848, 45004, 18648, 8608,  55720,
                43676, 59588, 52880, 22888, 40768, 16920, 45216, 51600, 22356,
                48352, 28016, 59540, 4416,  32984, 15680, 26992, 7072,  39860,
                33524, 5748,  48092, 35772, 42612, 64296, 24704, 11136, 41968,
                51732, 30580, 22408, 844,   27908, 41056, 3124,  47548, 28984,
                39868, 18728, 12148, 46092, 40628, 16788, 54512, 13024, 51716,
                17316, 41472, 11848, 53308, 30292, 9388,  55100, 20528, 46528,
                57104, 41800, 28756, 58548, 100,   19060, 41980, 15196, 50348,
                1400,  54856, 61608, 48444, 38188, 18320, 1288,  45604, 34492,
                19528, 41640, 57240, 22340, 7996,  55320, 20512, 59664, 13744,
                29972, 8604,  60360, 15596, 53352, 23720, 49048, 17176, 54836,
                4708,  41016, 52488, 26404, 2560,  49180, 24888, 32868, 58160,
                27312, 15240, 62744, 10064, 19456, 7840,  32160, 11960, 62368,
                19008, 53196, 32620, 12388, 29240, 8120,  25660, 31596, 65220,
                34364, 54556,
            },
            {
                3624,  29924, 24868, 55032, 776,   27632, 48656, 65044, 32832,
                39792, 2392,  14228, 47152, 31632, 548,   65276, 12860, 7284,
                61676, 52992, 9480,  42836, 21772, 45316, 36016, 51504, 10628,
                19260, 44372, 56484, 12804, 18436, 3928,  57584, 46588, 21188,
                62140, 5032,  48740, 44312, 60384, 34840, 15860, 51748, 6812,
                43852, 58264, 372,   55736, 6520,  21092, 61868, 36216, 29528,
                59884, 6704,  49576, 61488, 43628, 2924,  34720, 39052, 15532,
                25844, 7040,  21240, 52908, 34500, 25332, 63908, 10104, 29316,
                39420, 16760, 26700, 3552,  59204, 13592, 54096, 25084, 9496,
                49664, 5248,  32688, 17152, 28620, 39840, 10452, 33616, 352,
                50444, 36596, 47768, 26636, 57964, 39048, 7920,  61500, 43224,
                29100, 11812, 33252, 17564, 44712, 14384, 39564, 8404,  19304,
                45308, 51836, 22152, 29496, 53480, 25664, 44036, 51708, 1352,
                27572, 44908, 61452, 22184, 42240, 47284, 57224, 10844, 2672,
                40760, 12680,
            },
            {
                49756, 8340,  44236, 19972, 34568, 13712, 57876, 21468, 11100,
                24072, 29048, 63520, 8504,  56808, 27536, 35068, 38932, 24820,
                32400, 14252, 37232, 26152, 64860, 53708, 220,   59120, 23172,
                62220, 25792, 1364,  53780, 32536, 39440, 27612, 16120, 33048,
                8656,  36140, 15160, 55372, 7844,  24948, 57644, 22236, 62816,
                14288, 31420, 37596, 50344, 28408, 33120, 47648, 3040,  9324,
                45580, 23136, 35592, 14420, 27744, 57480, 23744, 11632, 53168,
                65212, 43680, 37452, 49300, 12516, 45276, 54008, 21692, 47136,
                32460, 11856, 44192, 50968, 33264, 28960, 42860, 60752, 38808,
                14540, 63204, 53784, 1332,  45820, 61716, 24708, 54316, 41860,
                63620, 21492, 12432, 6416,  33868, 1996,  30680, 20672, 35952,
                104,   65348, 48056, 37040, 58572, 63068, 21880, 54988, 35284,
                3736,  60124, 40752, 36652, 2412,  65296, 39016, 16952, 35084,
                58848, 9292,  15140, 50300, 480,   35316, 20020, 52796, 45056,
                17284, 61204,
            },
            {
                32860, 38720, 58688, 4816,  51092, 40408, 6612,  37292, 46228,
                50816, 17960, 36176, 42508, 19596, 49560, 4956,  59052, 18376,
                44700, 1440,  47428, 19084, 5276,  12228, 31752, 17172, 42356,
                50088, 37520, 29700, 9716,  22632, 60880, 49280, 1748,  53528,
                58956, 28444, 40276, 19224, 32012, 12484, 45940, 4732,  33644,
                54000, 8784,  25128, 15444, 60424, 10532, 41836, 19284, 64208,
                39028, 32164, 2124,  20704, 40316, 7308,  62128, 49792, 29840,
                18800, 1676,  59828, 16336, 4104,  28048, 38044, 5376,  61316,
                57540, 24232, 7832,  64364, 21864, 10188, 5700,  20456, 52728,
                30044, 24060, 37368, 12644, 51100, 35464, 15376, 6904,  27592,
                17224, 2940,  55972, 41376, 62660, 51988, 14104, 47056, 54020,
                24932, 56492, 22272, 7944,  4052,  28204, 11012, 42752, 49976,
                13140, 17776, 7064,  48712, 56376, 14400, 4496,  21848, 42556,
                6080,  25104, 37744, 55192, 17616, 62548, 24520, 7108,  58364,
                26448, 21932,
            },
            {
                56060, 15324, 11276, 64204, 31460, 25448, 53960, 16220, 1596,
                61152, 54928, 3828,  25224, 61640, 15092, 52504, 10100, 41308,
                54452, 62976, 30212, 57208, 49600, 40664, 28488, 55500, 8112,
                4028,  14624, 65056, 41100, 45964, 6648,  13380, 37092, 42756,
                23348, 11344, 65340, 2748,  53228, 61540, 38244, 41948, 26656,
                47820, 19780, 63652, 44832, 4268,  23796, 57496, 52068, 26004,
                15624, 56544, 52740, 63524, 48500, 18068, 44808, 4292,  35888,
                9124,  47376, 32380, 23416, 62700, 56716, 8788,  18332, 35420,
                2252,  53160, 18844, 36940, 40936, 56464, 46468, 35404, 2204,
                47712, 7572,  42368, 59808, 19172, 3612,  44184, 48372, 58224,
                32516, 37584, 45632, 28924, 18340, 22840, 59388, 10288, 39784,
                6352,  15464, 30780, 41964, 51856, 34404, 57212, 1720,  30296,
                62364, 26292, 32356, 11648, 44684, 28292, 60720, 31388, 48132,
                53564, 29684, 64764, 4548,  31980, 43880, 13600, 39544, 30600,
                36912, 4552,
            },
            {
                52012, 28440, 42444, 22804, 47816, 9516,  62920, 28612, 34332,
                6332,  43896, 31436, 11304, 37716, 45416, 22628, 33408, 28096,
                4312,  22032, 8896,  35004, 15044, 60144, 20700, 36892, 47620,
                33996, 58552, 20040, 51308, 16884, 56616, 30988, 63788, 19536,
                5508,  48032, 43776, 25672, 49140, 21084, 1236,  17412, 10896,
                58808, 2132,  40516, 34148, 53684, 38464, 14016, 31028, 7516,
                47228, 4616,  28740, 10004, 37180, 25960, 32964, 14508, 58240,
                40740, 26808, 54844, 11140, 40384, 30756, 43972, 50672, 14660,
                42744, 28844, 48128, 13812, 296,   61336, 15716, 27128, 65188,
                17932, 58088, 10376, 26832, 30652, 64680, 21884, 38712, 11552,
                61572, 51612, 24360, 9476,  49652, 3788,  43996, 32816, 18040,
                50544, 45104, 62608, 12620, 19464, 47500, 16488, 24432, 46432,
                37848, 54132, 58512, 23664, 35260, 19220, 50928, 10228, 57612,
                13560, 20168, 40536, 11192, 48828, 27724, 5180,  50440, 64240,
                9692,  46916,
            },
            {
                420,   35812, 61332, 17732, 3260,  44480, 19604, 41460, 59192,
                22648, 13920, 48604, 57972, 7240,  29780, 2360,  56196, 60936,
                16868, 50992, 39444, 24452, 6304,  44012, 2908,  63424, 24316,
                11832, 43736, 27428, 5636,  34432, 24020, 9152,  26624, 50428,
                56092, 33484, 16520, 37408, 9592,  30260, 51412, 64412, 28964,
                36388, 50556, 22204, 7812,  17648, 48508, 1140,  35212, 43292,
                60756, 18876, 41092, 13308, 59584, 700,   55332, 22432, 50644,
                63880, 20732, 6412,  36172, 51680, 892,   22092, 64916, 25940,
                58920, 6860,  32980, 54336, 25752, 31156, 50432, 11928, 43928,
                33040, 22380, 54636, 49108, 34316, 6480,  53148, 26124, 8136,
                19952, 4328,  14984, 65092, 35016, 38572, 25984, 63348, 28524,
                58516, 3372,  37772, 26508, 59572, 39296, 7236,  64436, 20308,
                9552,  5156,  42564, 504,   63676, 7784,  40272, 24440, 36424,
                1772,  46256, 59484, 23116, 35780, 60888, 55660, 21404, 16032,
                42108, 19616,
            },
            {
                57512, 24668, 7084,  53408, 33704, 56956, 12656, 50040, 7876,
                38992, 26820, 53064, 20668, 40380, 64644, 47460, 12908, 43344,
                36648, 11528, 46712, 64012, 54192, 27388, 10424, 52196, 16612,
                31152, 56220, 864,   38608, 61924, 52636, 44088, 39348, 108,
                14068, 29436, 59604, 4144,  56660, 34764, 14520, 44228, 55948,
                5084,  13400, 60920, 30412, 26704, 64840, 58744, 21988, 11712,
                54796, 24536, 33760, 45136, 51056, 7992,  42048, 28276, 10804,
                3240,  15480, 45908, 61000, 18008, 13528, 48772, 33600, 10284,
                38364, 20836, 63412, 43704, 4812,  39912, 23140, 55704, 6564,
                39072, 3832,  14960, 40492, 664,   57332, 14364, 42188, 29872,
                56920, 43772, 31836, 54144, 6068,  13240, 56120, 1484,  9044,
                21676, 33460, 53836, 9924,  752,   29540, 52560, 33748, 41480,
                15272, 51208, 21268, 47992, 16156, 53800, 45828, 5416,  62524,
                17956, 32720, 51148, 3168,  16648, 9240,  45896, 1616,  33740,
                60044, 12988,
            },
            {
                31584, 44924, 39288, 14452, 26900, 37600, 928,   31188, 56076,
                18172, 63952, 33096, 260,   16208, 24900, 35672, 18952, 6208,
                26376, 59128, 716,   31360, 17936, 33872, 45988, 38980, 59636,
                6912,  49884, 22392, 46532, 12496, 3340,  60020, 17704, 35804,
                62896, 7988,  46224, 22676, 41748, 61048, 6264,  24056, 19132,
                32596, 46788, 42656, 54472, 10960, 45076, 5828,  29072, 50152,
                36880, 1872,  64652, 21440, 30272, 16616, 62860, 46568, 38472,
                34176, 54128, 42348, 27652, 57300, 24880, 4524,  55224, 41540,
                2780,  50976, 11728, 17092, 58292, 9184,  19288, 36584, 60428,
                28344, 46556, 63476, 25000, 44928, 18292, 62556, 50100, 36280,
                1144,  47740, 39200, 21372, 60268, 46656, 16636, 49308, 36560,
                42904, 14076, 48556, 23592, 44500, 61476, 13736, 2944,  55696,
                28320, 35844, 60972, 31024, 26668, 12564, 33108, 55576, 28568,
                43744, 8336,  26904, 54440, 42520, 30248, 37944, 24796, 53620,
                27484, 48216,
            },
            {
                21532, 65308, 10076, 50388, 60164, 21168, 62452, 24340, 47388,
                10240, 4224,  45660, 60156, 51996, 8516,  54896, 61976, 49516,
                30428, 53364, 20848, 42768, 12972, 61020, 22172, 1612,  26320,
                41632, 18616, 64148, 32016, 15732, 29304, 21656, 47456, 54252,
                24136, 40152, 19020, 53576, 12664, 27224, 47756, 39880, 8856,
                52928, 25172, 16232, 2636,  37260, 20224, 39976, 16724, 62520,
                8616,  15144, 52416, 4836,  57580, 36416, 19868, 5336,  59168,
                23676, 30656, 1948,  8536,  34812, 39020, 46372, 28608, 16176,
                60400, 30688, 23724, 47060, 34164, 62092, 48024, 1248,  53176,
                11108, 51188, 20072, 7524,  36812, 10020, 32300, 22280, 12648,
                60916, 17116, 26736, 10356, 29152, 41440, 31520, 24112, 54684,
                60848, 18512, 5068,  56872, 36040, 20808, 49660, 25148, 58860,
                45168, 11500, 3288,  39772, 57060, 37328, 2464,  21808, 15276,
                58248, 38824, 64136, 13148, 19984, 57388, 63356, 11824, 40816,
                8020,  4092,
            },
            {
                55172, 17440, 2356,  29368, 46500, 4664,  42832, 34948, 15200,
                29568, 41764, 36712, 12000, 28668, 43040, 32216, 22256, 2100,
                40636, 15404, 8168,  57468, 38100, 5396,  50632, 55544, 14472,
                35432, 10168, 53656, 37152, 58640, 42328, 10472, 6280,  31616,
                11804, 50932, 3128,  64708, 33064, 17636, 520,   57384, 63400,
                37576, 4392,  61940, 57672, 32908, 51648, 56328, 26040, 42116,
                31756, 46848, 39588, 25364, 10640, 48448, 32240, 12884, 51236,
                17708, 62208, 49276, 21388, 64300, 11512, 58568, 19488, 8264,
                45204, 35988, 56532, 2136,  28152, 42216, 14072, 31784, 24336,
                16984, 35380, 31052, 61252, 55036, 27508, 46688, 5648,  52936,
                24968, 7856,  50528, 58100, 1964,  19064, 64124, 11608, 7148,
                39208, 29320, 63768, 32420, 16756, 41096, 10780, 31348, 7592,
                18528, 63448, 23472, 6880,  17752, 50524, 64880, 41652, 10928,
                49868, 136,   24036, 36820, 6660,  48748, 3404,  18116, 51012,
                61616, 37084,
            },
            {
                42680, 34296, 58908, 40896, 18584, 11912, 54408, 7056,  51524,
                61160, 55012, 22876, 18532, 50136, 4948,  14488, 38912, 10968,
                45864, 65460, 23968, 34804, 48020, 28288, 9584,  32120, 62360,
                48416, 28636, 2524,  7620,  25688, 49428, 34548, 56764, 61988,
                26492, 44408, 37552, 29144, 9944,  49524, 35312, 21464, 30220,
                11564, 41636, 28704, 22880, 9092,  14236, 240,   48908, 4316,
                59372, 18732, 28176, 61012, 42820, 2980,  56476, 26924, 43232,
                9432,  39900, 14460, 44368, 52300, 5988,  31144, 51252, 62828,
                26380, 13580, 6316,  39156, 52544, 21336, 7028,  63920, 43780,
                57568, 2704,  42260, 13256, 49416, 39516, 2388,  57900, 41028,
                33880, 64460, 42748, 36620, 53268, 5424,  44596, 34888, 51904,
                952,   46776, 25552, 8448,  52064, 4304,  64932, 47604, 38844,
                34080, 52692, 41912, 46512, 60028, 9204,  25268, 47544, 30084,
                34552, 18520, 45344, 52608, 32368, 27764, 44324, 34736, 23060,
                29516, 15252,
            },
            {
                49476, 23804, 6656,  52912, 32332, 25716, 64108, 38740, 20728,
                1896,  26436, 7764,  59280, 33860, 63672, 26168, 55872, 59976,
                33236, 27860, 51136, 3100,  17348, 63544, 44280, 20052, 3660,
                43140, 23380, 61096, 45096, 19576, 52536, 4072,  40556, 16692,
                1276,  20868, 58332, 5832,  42720, 52764, 60532, 45444, 15364,
                51284, 19516, 48724, 35676, 46204, 59956, 29956, 23584, 12820,
                54272, 35856, 6168,  13868, 53416, 22980, 37984, 65520, 68,
                55280, 28976, 4360,  32668, 17048, 24068, 42204, 472,   37872,
                21740, 53848, 65336, 18316, 10256, 57960, 26660, 49520, 38220,
                9140,  21840, 59560, 6380,  16656, 23656, 63784, 19448, 15096,
                30008, 4468,  20748, 14440, 32416, 23000, 56396, 27260, 16108,
                21000, 59184, 12824, 44160, 37416, 28044, 54704, 22720, 1228,
                15524, 27144, 56132, 13264, 28864, 35372, 19852, 3692,  61932,
                55216, 5124,  59608, 10456, 62280, 21252, 14124, 58412, 6024,
                45248, 10768,
            },
            {
                60700, 27956, 13552, 36484, 57236, 8872,  48536, 16912, 32892,
                44260, 13768, 39360, 47524, 1316,  44968, 20320, 51608, 3872,
                18404, 6856,  43576, 55120, 11588, 39844, 25008, 58028, 36920,
                16864, 55064, 11904, 38864, 14668, 64768, 23292, 28536, 46184,
                35448, 55160, 14888, 62428, 24412, 13408, 3964,  26216, 7352,
                56172, 1636,  65524, 5676,  17240, 39096, 63312, 34076, 45768,
                9524,  21996, 64180, 49968, 33424, 16124, 8328,  20820, 47636,
                35368, 25092, 60392, 56364, 36380, 61872, 13084, 47900, 33552,
                4180,  43064, 49252, 32344, 46136, 36372, 16068, 3584,  55860,
                29380, 46308, 26112, 52396, 34200, 29664, 11252, 44284, 36000,
                55656, 46412, 10748, 49192, 62728, 39992, 8760,  61324, 50060,
                31124, 41400, 55760, 2872,  60436, 19864, 13924, 42628, 59384,
                49232, 5876,  21056, 32616, 724,   44292, 53524, 39204, 14084,
                22292, 42376, 25956, 15964, 40204, 1208,  55020, 37736, 64500,
                52688, 1756,
            },
            {
                20240, 46304, 63468, 40,    21780, 44860, 28436, 3164,  56144,
                50340, 62640, 30752, 57124, 16320, 36856, 8904,  30096, 47788,
                37704, 14024, 59012, 21448, 31068, 44,    53156, 13248, 7196,
                29944, 49724, 33864, 57488, 848,   32232, 9360,  58124, 12424,
                50004, 8044,  32852, 47108, 18468, 36868, 31712, 40952, 59332,
                34108, 43976, 26972, 11880, 54372, 42852, 7648,  19140, 58276,
                38392, 43456, 29680, 1036,  40736, 46112, 60584, 31500, 13692,
                52752, 19156, 10540, 46076, 2384,  27728, 54364, 18216, 57692,
                10708, 15752, 28708, 24428, 764,   62536, 41344, 33364, 18928,
                11988, 65108, 36916, 76,    56224, 47616, 59980, 7756,  51692,
                888,   24348, 58624, 27940, 2888,  17376, 46892, 13332, 3456,
                35436, 9700,  23340, 16396, 33492, 46428, 8060,  35576, 29804,
                62972, 10328, 38060, 51468, 63840, 16196, 58776, 6836,  32260,
                51132, 8860,  29580, 46768, 34292, 49776, 8324,  30516, 17524,
                26068, 33376,
            },
            {
                38944, 8636,  41956, 50248, 15500, 37588, 59256, 12136, 23200,
                35656, 4832,  19636, 11220, 24816, 41672, 58560, 12968, 23652,
                62400, 40928, 26032, 36080, 61524, 49136, 34004, 45944, 65088,
                41152, 4424,  21292, 26504, 43416, 48220, 38276, 19232, 63280,
                42040, 22496, 27748, 39740, 2044,  54044, 64404, 9616,  17472,
                23488, 38360, 50540, 31820, 21508, 25524, 3704,  52668, 26892,
                2676,  16680, 56780, 11708, 24664, 54436, 27740, 6224,  58468,
                41744, 5168,  38488, 49828, 21684, 7884,  39328, 30228, 23400,
                61232, 40412, 55820, 8056,  59432, 12444, 22600, 54100, 5616,
                50840, 40732, 14568, 20376, 43148, 4400,  38620, 22532, 27412,
                62272, 17912, 38480, 43876, 52792, 30700, 37400, 24544, 43512,
                65516, 53588, 38592, 49456, 63216, 26076, 57180, 3672,  17712,
                44488, 24808, 58212, 4324,  40648, 23528, 10784, 27656, 45624,
                57440, 37212, 65528, 3920,  58020, 23736, 43604, 11416, 41532,
                4124,  56268,
            },
            {
                59160, 18164, 30336, 25096, 61776, 5764,  31620, 42120, 65148,
                9364,  27336, 43252, 49028, 53040, 5400,  64776, 35280, 1004,
                53808, 9676,  46592, 4864,  15956, 8464,  19032, 22944, 27476,
                59580, 10092, 51880, 62424, 7004,  16264, 53896, 5004,  30300,
                2376,  52200, 61256, 10824, 57068, 20800, 44744, 48400, 29312,
                53320, 14064, 4220,  60784, 56836, 46548, 36756, 14692, 49432,
                61780, 32756, 47036, 20016, 62788, 3904,  39364, 17680, 45064,
                23800, 62256, 29900, 12764, 64900, 43276, 59516, 3304,  50804,
                44996, 5728,  35720, 19900, 51048, 43652, 29064, 47624, 60180,
                24132, 31840, 7948,  61372, 26860, 17420, 32480, 13548, 48692,
                41756, 8644,  33124, 12696, 6616,  60396, 20332, 57768, 7264,
                27868, 18912, 5244,  30156, 148,   11592, 52840, 38988, 50960,
                13004, 34532, 46696, 19176, 29492, 48260, 36100, 62324, 17060,
                948,   20392, 12856, 53872, 18804, 27448, 59968, 51656, 21588,
                47972, 13120,
            },
            {
                5496,  52304, 34876, 3512,  55164, 20004, 52496, 47596, 18056,
                39144, 54284, 60464, 2500,  33368, 28776, 21460, 44096, 50096,
                32036, 19920, 57084, 29044, 52212, 41508, 56520, 3900,  37992,
                15548, 46496, 18380, 30424, 36500, 23944, 60564, 45372, 25620,
                37508, 17304, 6760,  48804, 30808, 25936, 5768,  12668, 884,
                62296, 20080, 8420,  40848, 16256, 10236, 64044, 30232, 40632,
                22856, 5552,  51596, 37296, 9688,  34360, 50380, 11980, 32508,
                2712,  51840, 16568, 33836, 25484, 53060, 14852, 9856,  34216,
                26628, 63380, 14140, 30844, 37808, 3088,  15576, 9792,  35400,
                1876,  45120, 57112, 49124, 10544, 53152, 64520, 57244, 5268,
                35392, 54028, 21724, 64756, 26184, 49380, 668,   54208, 33072,
                47432, 13668, 58928, 44836, 20596, 41172, 31484, 23124, 60492,
                28120, 1428,  53692, 8148,  14356, 55800, 4880,  50352, 25380,
                40792, 33592, 48732, 7452,  38736, 14368, 2208,  33928, 63228,
                36744, 28492,
            },
            {
                44716, 64820, 11260, 47140, 39876, 14196, 8040,  25836, 1552,
                30276, 13484, 22520, 37340, 17592, 55744, 14536, 7460,  27096,
                16448, 3932,  64428, 12320, 37224, 25288, 62352, 10996, 53632,
                32728, 2304,  39692, 56012, 12836, 50880, 9992,  34224, 55044,
                13060, 58032, 44068, 35008, 14292, 42396, 59772, 34472, 39196,
                56340, 45584, 35640, 27636, 49240, 33564, 1436,  55612, 12440,
                8492,  58816, 28432, 15476, 44348, 59112, 21904, 64444, 55408,
                36780, 8888,  57928, 40812, 992,   20184, 37052, 47316, 17212,
                21444, 1392,  48384, 54824, 25500, 64248, 56496, 39716, 26596,
                62624, 16992, 22072, 29884, 36348, 40616, 1800,  20600, 30500,
                16340, 59060, 2580,  45576, 15492, 35196, 42164, 11076, 16816,
                40112, 51872, 25612, 36188, 55052, 15264, 62096, 9296,  5420,
                20496, 42148, 65204, 32228, 59876, 42916, 21564, 31444, 9540,
                54584, 61388, 23408, 43008, 31552, 62216, 45680, 6368,  16008,
                54592, 23856,
            },
            {
                32784, 16832, 22588, 58464, 27492, 44428, 33292, 61136, 56776,
                51396, 6132,  44812, 62172, 9064,  40148, 46152, 58156, 61860,
                39000, 42888, 33400, 22228, 44992, 1940,  48368, 29556, 43992,
                58376, 24844, 63708, 6344,  42764, 28588, 20584, 3636,  41368,
                65292, 21048, 24612, 1252,  63608, 18132, 52524, 22008, 49808,
                31044, 24480, 64624, 3348,  58084, 20616, 24752, 45124, 18636,
                35620, 42144, 25204, 55176, 3064,  30204, 7636,  26256, 47112,
                19588, 28316, 44688, 48744, 6448,  56232, 30872, 62752, 52152,
                38624, 57884, 42384, 9372,  18136, 45900, 6932,  20048, 52236,
                12924, 38308, 3368,  54940, 6460,  23936, 46200, 51036, 42928,
                10028, 47936, 28624, 40304, 56868, 8476,  29596, 63084, 22012,
                59824, 2696,  10420, 63888, 7140,  47652, 34424, 43472, 48892,
                57016, 36032, 16812, 24076, 38868, 12060, 1796,  63900, 44580,
                14892, 4556,  28260, 52528, 10556, 55552, 20664, 29788, 40744,
                10044, 808,
            },
            {
                42456, 50680, 7320,  36240, 1132,  63508, 21888, 11800, 41384,
                35264, 15584, 57724, 31600, 25200, 51712, 3248,  36196, 10740,
                24004, 54904, 51084, 7192,  60444, 16988, 34368, 21088, 13728,
                7676,  19540, 48552, 34504, 508,   61968, 47088, 15784, 31908,
                7712,  29252, 50508, 54256, 37312, 9036,  28160, 2760,  15864,
                7200,  11092, 17780, 52896, 42308, 6992,  38312, 52220, 62204,
                48640, 328,   65128, 12720, 39752, 49112, 16144, 42572, 640,
                14176, 63392, 22844, 11716, 61688, 27120, 43684, 3816,  7604,
                28800, 12724, 32412, 4300,  52508, 34336, 29740, 42236, 58280,
                32828, 47888, 43588, 63124, 15564, 34016, 12480, 61808, 26116,
                37996, 63300, 23396, 4356,  18660, 51700, 24448, 46492, 37732,
                6340,  32052, 42276, 22636, 28888, 18304, 3992,  25420, 13964,
                29916, 10468, 51144, 6176,  27396, 47744, 52952, 36608, 19344,
                58932, 38048, 47580, 1664,  17156, 36168, 49800, 25432, 58128,
                48852, 60860,
            },
            {
                26772, 13684, 56388, 29844, 18760, 49736, 38156, 3428,  46624,
                24260, 19836, 48388, 1072,  12220, 63868, 21096, 29512, 18452,
                47956, 452,   13676, 30492, 55976, 9764,  39636, 64532, 51420,
                36428, 28140, 53344, 14664, 23368, 56908, 38664, 52800, 59652,
                48176, 11240, 40496, 5444,  32184, 46524, 62120, 40396, 58444,
                44024, 37248, 47800, 32192, 13572, 29184, 59536, 15600, 5036,
                31240, 21008, 52932, 32992, 18828, 61104, 35752, 56944, 53084,
                40004, 5148,  31424, 38088, 17992, 35140, 13940, 23064, 54612,
                45456, 64648, 24760, 59940, 22260, 11216, 61380, 832,   23900,
                5716,  10720, 27840, 19940, 58164, 52460, 28920, 7608,  19072,
                364,   14116, 53236, 32452, 37220, 61800, 13880, 1716,  55732,
                27460, 49572, 14484, 57832, 51164, 39760, 59512, 52744, 63284,
                38000, 45852, 3044,  61520, 57844, 34012, 16496, 26060, 6396,
                30312, 12312, 21024, 57176, 41300, 6732,  63588, 12616, 3096,
                37496, 20060,
            },
            {
                4320,  38888, 61400, 43096, 9908,  53464, 16096, 59800, 28812,
                9548,  64668, 39552, 27720, 43348, 49244, 32900, 53584, 6020,
                60952, 35020, 27520, 41268, 19372, 49940, 25724, 6052,  1468,
                42460, 60840, 9816,  45668, 31312, 18268, 7252,  26160, 2776,
                35756, 19212, 57368, 16164, 23012, 55476, 11740, 25284, 51120,
                28872, 60724, 496,   23232, 62948, 9864,  43848, 34588, 27512,
                11388, 46816, 8792,  23832, 45572, 5804,  28780, 11032, 24188,
                33936, 49400, 60412, 53840, 8400,  50648, 58636, 41228, 10868,
                19432, 35800, 16056, 40444, 47552, 37168, 14840, 49700, 17740,
                65012, 51440, 36108, 8556,  42320, 2668,  38876, 59952, 55112,
                35100, 43920, 58104, 11384, 48672, 6780,  43156, 34624, 19692,
                64660, 17372, 44156, 34976, 2032,  12348, 32296, 21708, 628,
                18044, 55420, 23008, 40228, 19924, 8620,  45452, 56684, 40308,
                51292, 64832, 33312, 26876, 60568, 22900, 34324, 44628, 18096,
                31800, 53992,
            },
            {
                10920, 23960, 33596, 47420, 5220,  26144, 32020, 55492, 6500,
                50408, 33100, 4572,  54736, 17436, 7584,  14324, 41496, 57616,
                44508, 16132, 64176, 47288, 4080,  59360, 31916, 45984, 57660,
                22772, 17252, 39968, 4216,  64804, 36676, 44196, 13656, 22656,
                63044, 45912, 27240, 61320, 43608, 3464,  18544, 35824, 4848,
                14012, 53556, 19684, 39648, 55284, 49040, 2472,  22052, 56572,
                41424, 60852, 38552, 57284, 2540,  51280, 63136, 19912, 7360,
                58880, 16268, 3600,  21232, 46088, 2400,  25208, 33172, 60844,
                176,   49816, 6768,  56936, 2328,  28072, 55580, 44316, 39232,
                31248, 56288, 14100, 25632, 48432, 31864, 16900, 45420, 22896,
                49260, 30088, 5100,  20972, 26716, 59496, 30924, 52824, 11780,
                38752, 8764,  61268, 5712,  26340, 55992, 45356, 7932,  41900,
                27756, 33556, 12952, 49820, 31012, 3616,  63316, 11296, 22968,
                200,   43336, 8728,  46336, 3784,  14864, 51172, 9056,  27340,
                47100, 64276,
            },
            {
                41380, 52160, 2556,  20420, 65428, 12700, 45060, 36412, 18552,
                42124, 14120, 60348, 35688, 23684, 59232, 37572, 1992,  25924,
                21856, 9076,  38384, 24288, 12096, 36292, 15448, 53908, 29340,
                12476, 33724, 55764, 49784, 27280, 11460, 59004, 50892, 41556,
                31456, 9168,  392,   13476, 38584, 30052, 48836, 64828, 21452,
                33876, 9564,  43044, 6268,  35608, 25532, 18052, 63996, 50784,
                13908, 3656,  17184, 35320, 26684, 14096, 42220, 37640, 46780,
                29732, 43716, 26228, 40852, 29372, 65432, 16696, 48048, 39064,
                29864, 53444, 26720, 43352, 20384, 63844, 8388,  25152, 4752,
                21080, 45360, 1660,  61120, 22352, 64488, 11752, 4000,  27152,
                9472,  15616, 41008, 65164, 46052, 16840, 23040, 3732,  47780,
                25060, 54180, 30224, 46948, 19052, 37448, 64968, 15328, 50140,
                60052, 9208,  62736, 43636, 15356, 53920, 27836, 35580, 49052,
                16060, 54696, 18748, 36840, 52916, 29892, 39032, 55336, 59720,
                5920,  15372,
            },
            {
                30644, 55940, 16720, 28232, 57028, 40356, 22608, 1928,  62964,
                25640, 52660, 21256, 10624, 46168, 28788, 62688, 52084, 11916,
                31256, 56168, 3068,  52540, 61092, 43320, 21512, 8996,  38556,
                62844, 2920,  24584, 8084,  20736, 54400, 1860,  28548, 16820,
                55236, 37264, 50076, 33660, 53080, 58300, 7624,  40808, 45232,
                56140, 61984, 27588, 15088, 59268, 31452, 45304, 8064,  36716,
                24964, 54620, 29744, 44060, 59784, 22100, 32428, 1308,  52436,
                64048, 9420,  13616, 54804, 36824, 10244, 56688, 4844,  21816,
                14676, 62580, 10120, 34172, 13576, 32056, 51944, 35364, 12180,
                60100, 28600, 38072, 54048, 6100,  40196, 50844, 35680, 58728,
                62404, 52348, 33708, 2152,  38848, 9244,  56488, 41372, 60076,
                33608, 340,   40548, 22924, 51984, 9824,  29692, 24108, 35808,
                4800,  46912, 24792, 1156,  37140, 21320, 41520, 59836, 4808,
                25528, 31640, 58216, 12584, 24636, 64060, 19888, 644,   42600,
                35944, 21560,
            },
            {
                8124,  44792, 37824, 49156, 34820, 7488,  51140, 57872, 11060,
                30760, 44288, 128,   39164, 55572, 3940,  19700, 34376, 46876,
                40116, 49484, 20044, 33412, 7612,  28696, 936,   50480, 18408,
                44168, 47976, 60220, 31992, 40900, 46836, 33956, 6504,  60404,
                21200, 6600,  63712, 24304, 19340, 10364, 26944, 16772, 1868,
                30976, 24184, 46564, 50732, 1704,  11720, 54004, 40068, 4528,
                32652, 19676, 63376, 10300, 6960,  48600, 55536, 11636, 18128,
                22784, 35272, 57484, 712,   19092, 44092, 32292, 52776, 8140,
                46748, 36688, 23344, 59776, 49316, 4100,  16960, 61880, 41648,
                47772, 8244,  15720, 34184, 19444, 28996, 56016, 18104, 42512,
                6740,  19956, 24460, 55328, 50276, 35644, 14204, 29132, 7348,
                20880, 16004, 58272, 12756, 61864, 42424, 2528,  58588, 53720,
                19792, 39368, 29148, 51324, 58004, 7468,  32456, 13404, 46744,
                62880, 39304, 2516,  42032, 7032,  47928, 11024, 32748, 24124,
                50204, 62212,
            },
            {
                26688, 58736, 10672, 440,   14612, 61448, 29408, 16860, 38284,
                48580, 7936,  62284, 16240, 31960, 50496, 8916,  15316, 64004,
                6744,  58996, 26544, 13608, 46340, 54760, 65416, 25300, 57200,
                5432,  36208, 15384, 51980, 12612, 18524, 64032, 38704, 43916,
                11124, 47660, 15516, 42432, 3008,  46464, 60408, 51908, 12576,
                38520, 4660,  17744, 37428, 65240, 21300, 28764, 16424, 61512,
                46192, 49584, 1456,  52736, 40160, 16908, 28404, 58580, 39556,
                5996,  50864, 31912, 47236, 62760, 23496, 13356, 27832, 58384,
                41888, 17764, 2724,  54564, 39680, 46120, 30016, 54148, 612,
                18488, 58448, 51128, 43136, 10204, 47120, 2340,  14020, 31040,
                38240, 47716, 12896, 28192, 5736,  19056, 63460, 45684, 50972,
                62200, 36624, 49188, 4348,  33276, 17384, 48516, 7176,  31924,
                12428, 55880, 16948, 10548, 64908, 44904, 18176, 56028, 9504,
                19960, 28852, 50192, 60260, 33992, 55932, 44448, 16276, 57408,
                13000, 2884,
            },
            {
                34168, 19240, 63340, 25336, 43568, 20864, 47000, 3312,  23560,
                54456, 34928, 27800, 58140, 24912, 40984, 44380, 23192, 28064,
                660,   17464, 42800, 62040, 37856, 16460, 40996, 34684, 30984,
                10352, 22004, 28264, 204,   57916, 25812, 4160,  22992, 30684,
                53324, 26568, 58708, 29612, 55988, 36956, 31880, 22908, 63792,
                48996, 58876, 9004,  55464, 33428, 42664, 48548, 58596, 10932,
                22920, 41796, 13272, 25704, 33308, 65208, 3356,  44916, 24916,
                60876, 41892, 15436, 26864, 7136,  50332, 40544, 35188, 1312,
                63960, 31208, 44544, 6028,  19320, 24588, 9544,  38352, 22524,
                26928, 32200, 3944,  24936, 63780, 59436, 22748, 44632, 25928,
                56968, 620,   63948, 44820, 59304, 31504, 24172, 1500,  11248,
                26676, 42636, 8396,  25280, 56204, 27688, 38596, 21896, 44492,
                63852, 2168,  42560, 31608, 22816, 4152,  26840, 50616, 34288,
                43640, 6096,  14328, 22420, 17684, 27288, 3144,  61544, 28932,
                39496, 47568,
            },
            {
                5012,  41584, 50392, 31180, 54740, 9672,  33820, 41668, 64340,
                13928, 19380, 2900,  46976, 12104, 5208,  56628, 60468, 38132,
                54368, 35120, 10176, 30488, 2804,  22848, 6272,  12864, 49072,
                63532, 42640, 38952, 61744, 44840, 35516, 55968, 50720, 14188,
                1164,  35660, 40032, 4768,  21400, 14340, 43836, 6820,  34852,
                20008, 42064, 29856, 23212, 13808, 7404,  3160,  26572, 35636,
                6180,  30708, 56928, 37112, 20904, 46832, 14752, 36356, 30880,
                2460,  20196, 8932,  58284, 38092, 3868,  60800, 18400, 48764,
                9928,  26388, 56372, 12300, 28732, 65256, 57728, 14132, 49024,
                63260, 36732, 55136, 13680, 38820, 31708, 8508,  53208, 61916,
                10656, 33156, 17388, 37060, 9708,  42256, 53956, 38968, 56932,
                18192, 30628, 53744, 14316, 45412, 63040, 10360, 52956, 15712,
                36876, 27020, 59644, 48632, 35616, 53628, 39136, 1656,  61712,
                23812, 36408, 65532, 53852, 9628,  40588, 36864, 51388, 8812,
                20624, 54600,
            },
            {
                16596, 11772, 23052, 37356, 4448,  58396, 51568, 26312, 5092,
                59364, 43296, 52284, 21892, 65028, 35964, 18356, 30340, 13092,
                45664, 51376, 24616, 56904, 47592, 51952, 59708, 20100, 55368,
                3720,  17856, 53964, 7020,  10744, 16716, 41992, 8544,  62076,
                46396, 18820, 65412, 9604,  51168, 61748, 1396,  55264, 27952,
                10724, 52788, 172,   45744, 60456, 51052, 38536, 19364, 55608,
                51676, 62544, 17580, 8172,  60596, 4936,  54900, 9812,  52876,
                48812, 64468, 45524, 29776, 52632, 11872, 21596, 55652, 24804,
                15080, 38828, 61220, 50056, 36064, 42872, 2856,  33288, 43764,
                5300,  11040, 45516, 21076, 1412,  49684, 35160, 5236,  20128,
                50520, 41788, 23104, 52456, 3384,  15300, 20676, 33232, 4740,
                44976, 65344, 2220,  39620, 20352, 35092, 680,   59920, 24320,
                47796, 8376,  20200, 5864,  14276, 60544, 11888, 30244, 15888,
                56724, 47832, 704,   31020, 46036, 59468, 5936,  23276, 32204,
                45244, 64996,
            },
            {
                29652, 52620, 7372,  61100, 16112, 39400, 18892, 12028, 31944,
                37188, 9536,  29572, 40124, 33024, 53220, 1580,  49160, 7892,
                21140, 4260,  64416, 19300, 8284,  33208, 28468, 44788, 37544,
                26672, 46612, 30636, 23288, 33168, 49176, 27472, 20924, 32180,
                56748, 24364, 12788, 33924, 48360, 25996, 18212, 40704, 47048,
                15556, 63116, 26308, 36724, 17124, 30620, 63388, 43476, 15016,
                824,   45388, 39452, 29112, 49956, 42660, 23228, 27736, 17500,
                38660, 12928, 34620, 24252, 16700, 43372, 32408, 45904, 4900,
                53396, 33728, 6888,  22240, 16040, 52320, 7888,  20484, 55788,
                16552, 52184, 30120, 60728, 17536, 56400, 27380, 40776, 15908,
                29588, 4484,  61076, 27068, 57904, 47256, 62584, 50052, 13028,
                35736, 22640, 48488, 59196, 6756,  50660, 30076, 41816, 4168,
                33584, 52128, 40008, 57296, 25696, 46220, 21068, 41396, 52252,
                7848,  19108, 38852, 25348, 13368, 49208, 18808, 62440, 14144,
                1420,  36456,
            },
            {
                60008, 43948, 33952, 47680, 28244, 1296,  63104, 46064, 55468,
                48952, 16652, 61600, 6004,  15156, 10528, 42704, 25428, 62244,
                32788, 41452, 15148, 37240, 43580, 11992, 63156, 816,   15796,
                8936,  60684, 13596, 51408, 64944, 5252,  59376, 37760, 2996,
                41276, 6972,  44460, 54348, 30604, 38084, 8480,  58320, 32696,
                5568,  39112, 57516, 4412,  11496, 53952, 23732, 9080,  33088,
                27192, 22036, 11268, 3664,  58544, 13020, 34136, 63112, 56304,
                4696,  21968, 1028,  55308, 62060, 36644, 1944,  64940, 29416,
                41320, 19076, 47692, 376,   31524, 59048, 39764, 28168, 62028,
                23932, 40164, 7552,  26236, 42516, 46544, 11428, 65384, 37628,
                55556, 48620, 8000,  34996, 11852, 29768, 24652, 7364,  58844,
                27960, 9600,  16084, 32524, 24800, 11944, 18620, 56948, 14432,
                63344, 11316, 30464, 17676, 2868,  64400, 32988, 4748,  45032,
                28200, 11288, 58856, 42940, 3812,  34384, 26644, 52960, 41768,
                57172, 24768,
            },
            {
                18920, 2308,  13556, 56184, 20976, 43420, 30508, 7180,  24488,
                20688, 708,   50900, 26600, 58336, 47392, 20228, 55016, 38812,
                57164, 27628, 49728, 2024,  26140, 53724, 39884, 23424, 50072,
                34616, 57756, 2692,  38260, 18592, 45496, 12260, 53820, 15656,
                49392, 28348, 60820, 16376, 5304,  22336, 64348, 13340, 52008,
                24216, 18988, 48004, 34008, 21128, 41132, 2536,  47024, 57420,
                64592, 49256, 35352, 53700, 24956, 18680, 40212, 7232,  44048,
                31464, 59036, 41116, 47528, 6844,  10316, 26476, 58756, 13428,
                8440,  57476, 44016, 63600, 10880, 25816, 46280, 12556, 35652,
                1080,  47700, 63980, 34652, 4904,  14792, 58804, 23688, 2096,
                45424, 13432, 21584, 39728, 44536, 1652,  37900, 17804, 41752,
                52836, 61564, 37636, 43500, 55644, 64140, 47016, 36224, 28028,
                44212, 22932, 54820, 42484, 36024, 49640, 9844,  59152, 24208,
                63580, 35220, 50572, 21316, 63080, 55740, 10668, 38380, 5384,
                47876, 9748,
            },
            {
                50244, 39756, 64388, 25256, 10584, 53512, 35408, 14684, 58192,
                40876, 34632, 45180, 22468, 37580, 63540, 28552, 2772,  16432,
                11036, 6584,  60992, 31204, 58472, 17356, 5448,  56272, 31544,
                43104, 20652, 25600, 41408, 29356, 956,   34688, 25016, 63576,
                10012, 20132, 36284, 400,   56316, 41404, 45656, 29268, 2324,
                60692, 43212, 8300,  55348, 61980, 49424, 29068, 37320, 13048,
                6916,  41004, 16412, 61892, 29968, 47808, 56,    51184, 11664,
                25404, 50304, 15048, 28104, 19948, 39380, 49292, 17588, 51744,
                36340, 23788, 27804, 38160, 55004, 18312, 3668,  60056, 50636,
                31636, 19264, 10620, 53756, 22276, 31172, 50824, 8876,  33684,
                25832, 63144, 57264, 18884, 52000, 64408, 55416, 32168, 46476,
                396,   20576, 5880,  28820, 2716,  39956, 8212,  21324, 5020,
                49932, 1404,  61784, 7632,  27692, 53264, 15060, 37624, 18276,
                54236, 2140,  31592, 16492, 7728,  29388, 44944, 17136, 31536,
                21360, 27976,
            },
            {
                58912, 15960, 31260, 4680,  41244, 61560, 48460, 3448,  27544,
                64780, 8648,  56560, 12976, 3760,  32092, 8960,  46140, 35548,
                23244, 52964, 44648, 13276, 21868, 47728, 35820, 11068, 62476,
                4204,  52684, 11480, 62036, 56460, 48452, 21748, 42716, 30544,
                57664, 40232, 52228, 25388, 47608, 11228, 35112, 20220, 49632,
                36220, 11952, 31148, 27568, 15192, 6152,  58828, 18208, 53104,
                20592, 31428, 2184,  43364, 8364,  37620, 57120, 21204, 60660,
                36144, 8624,  63828, 33456, 54060, 61184, 22664, 44864, 33052,
                2584,  62156, 14312, 4824,  34264, 49088, 22688, 42712, 8892,
                15312, 57700, 44856, 38956, 59812, 180,   36692, 54572, 16668,
                42844, 30112, 2768,  32872, 15588, 6532,  22740, 10296, 14724,
                25740, 57020, 49264, 13620, 52512, 17080, 33228, 54464, 60816,
                37788, 16800, 32704, 12736, 45924, 22560, 4,     29932, 40552,
                5852,  13872, 46700, 41020, 52116, 23756, 58556, 796,   63416,
                55080, 34604,
            },
            {
                3632,  45476, 37076, 51912, 7776,  22672, 17168, 38716, 11616,
                52336, 18548, 30176, 42472, 54340, 18004, 51060, 58888, 41960,
                65284, 18960, 34468, 39008, 8752,  63820, 41780, 19424, 28252,
                15236, 45108, 36748, 5972,  16200, 8180,  60196, 51452, 6320,
                3460,  33164, 13036, 63052, 18012, 59140, 3752,  26828, 56536,
                16560, 63404, 1240,  45436, 38632, 25112, 10700, 34672, 45088,
                25728, 59428, 55624, 23084, 14004, 64352, 32424, 16188, 28804,
                46728, 18940, 5192,  43788, 12692, 3036,  30416, 6228,  55780,
                11604, 48228, 21112, 52564, 41060, 7276,  64608, 28928, 36752,
                53004, 25368, 4096,  28228, 17656, 48484, 41204, 20188, 60696,
                5900,  52988, 10680, 47508, 41156, 28616, 48900, 59892, 35108,
                39796, 64632, 31492, 45184, 23420, 62224, 26880, 42992, 10644,
                25608, 47276, 57100, 39148, 18720, 55892, 61536, 44128, 49016,
                57864, 25108, 61392, 37348, 3176,  12652, 48780, 36112, 43272,
                13164, 8004,
            },
            {
                23460, 62124, 19404, 26912, 46804, 33444, 54268, 59528, 44140,
                32512, 2380,  25056, 60540, 6668,  40064, 24080, 12492, 30200,
                5708,  48664, 184,   28716, 51800, 3768,  25504, 53372, 7900,
                48884, 30704, 23068, 54920, 33520, 28036, 39080, 14212, 19024,
                46828, 23044, 43756, 29128, 6620,  32644, 52516, 42428, 7036,
                39604, 23320, 53200, 20076, 50548, 65456, 42632, 432,   61584,
                5080,  38116, 10832, 46852, 52276, 26460, 5688,  42400, 54720,
                1608,  40084, 52376, 24720, 37308, 58228, 15820, 40052, 26204,
                42804, 57808, 30100, 17072, 58812, 32148, 13544, 55472, 1920,
                21516, 61816, 34084, 13612, 64560, 7220,  26276, 12524, 46808,
                35864, 22460, 59124, 37120, 25072, 61708, 2116,  43440, 52204,
                3648,  18084, 11112, 36020, 6948,  57380, 1084,  14008, 51496,
                20304, 5564,  29616, 64160, 4044,  26412, 11152, 32548, 7980,
                20256, 28520, 10000, 56352, 33432, 64484, 18492, 6652,  25468,
                52784, 41124,
            },
            {
                29244, 55480, 11724, 57784, 14320, 92,    29624, 20160, 4976,
                50036, 36600, 46316, 15412, 49444, 34052, 61436, 2196,  54868,
                15248, 25764, 56648, 60240, 16216, 46156, 31772, 58068, 38096,
                59916, 444,   64184, 46620, 19664, 43540, 10480, 64808, 34912,
                55056, 60988, 8724,  49960, 39104, 22144, 14876, 60372, 9848,
                47576, 33620, 58972, 13412, 4596,  32432, 22148, 52364, 29504,
                15344, 50456, 33656, 19124, 3772,  35716, 45492, 10084, 62540,
                14160, 22488, 57324, 31664, 9416,  50664, 46444, 62912, 19644,
                8304,  37036, 1224,  46236, 10344, 25020, 44600, 18020, 39592,
                48244, 41824, 8196,  51728, 44152, 30764, 55924, 62508, 3412,
                28000, 49904, 14344, 7772,  55836, 19548, 13108, 29992, 21384,
                8740,  27848, 42200, 50504, 20804, 38936, 46908, 31796, 35872,
                58936, 41220, 9644,  50068, 34856, 42324, 51960, 15944, 65152,
                35816, 52696, 17220, 44420, 21628, 27196, 40092, 30364, 61056,
                15468, 49592,
            },
            {
                9724,  1604,  35664, 43328, 64072, 38208, 9684,  61832, 23480,
                13916, 57740, 63772, 21364, 10736, 27276, 37484, 19468, 45636,
                40504, 32960, 10816, 42492, 23532, 35348, 12904, 2736,  22188,
                17892, 40592, 12548, 26088, 5064,  58668, 49036, 24784, 1116,
                27820, 16420, 36832, 3032,  54064, 65160, 45512, 30140, 18836,
                26036, 3416,  42976, 36984, 8832,  55784, 16836, 39540, 48120,
                8484,  64076, 27828, 41420, 60948, 56256, 20348, 24596, 38128,
                29952, 48504, 3644,  64748, 17672, 27936, 284,   34748, 13952,
                50320, 65364, 27004, 39316, 60796, 3148,  51240, 62496, 6336,
                11596, 29592, 56500, 19716, 2436,  38452, 22132, 9560,  32972,
                39944, 17668, 64716, 42652, 4264,  33900, 53340, 37660, 58540,
                47440, 62340, 54056, 4132,  60356, 29084, 9108,  65280, 16372,
                2372,  53636, 24220, 14300, 21052, 59804, 2616,  23704, 39572,
                46388, 1764,  31128, 5044,  50624, 8756,  54184, 45568, 2700,
                20760, 37756,
            },
            {
                47324, 17608, 31928, 6244,  24648, 50964, 56096, 42536, 27248,
                39312, 8188,  29328, 568,   44472, 53144, 5040,  63596, 22544,
                8176,  50912, 63028, 19308, 6120,  55628, 64476, 50596, 43492,
                33356, 9624,  51404, 35836, 54080, 2952,  31376, 37656, 52596,
                41240, 11000, 58592, 19860, 26180, 12068, 984,   37176, 51320,
                63912, 55204, 15968, 29628, 47348, 59904, 27160, 11892, 57552,
                20964, 43816, 1592,  16808, 30848, 12580, 48768, 2484,  53332,
                59656, 35076, 12172, 44940, 41512, 21376, 60516, 53404, 24264,
                33200, 5436,  22112, 54228, 15232, 35384, 20344, 26680, 33496,
                58952, 16408, 46400, 24716, 34700, 59276, 15740, 52384, 45588,
                55252, 616,   31468, 26352, 46124, 9456,  63552, 16588, 6088,
                32928, 23640, 15180, 33968, 12076, 18508, 55812, 23540, 43724,
                28172, 60912, 32792, 45044, 5884,  37652, 29348, 56840, 6580,
                12488, 58176, 62300, 41580, 14444, 60612, 35600, 11348, 58268,
                33128, 65368,
            },
            {
                39972, 53844, 59448, 48988, 19744, 15508, 34460, 5772,  47480,
                17900, 54068, 33272, 41340, 56892, 16580, 30920, 48076, 57500,
                36364, 27808, 3220,  45396, 30316, 39608, 9212,  15772, 25316,
                55272, 61656, 29120, 15068, 41484, 62572, 17232, 21148, 6708,
                63092, 44520, 30820, 48456, 33332, 41012, 56756, 23600, 5760,
                32004, 11352, 21104, 61948, 24168, 44272, 2348,  34512, 5656,
                37160, 24860, 54120, 58536, 9020,  40040, 65192, 32812, 16012,
                8224,  19584, 26796, 6432,  55596, 36640, 4648,  10704, 44180,
                59040, 18168, 47748, 31588, 43572, 8612,  57276, 38340, 49648,
                22736, 976,   37532, 62740, 10484, 49612, 42000, 5232,  24024,
                13052, 61116, 20564, 51636, 38448, 23036, 48720, 26788, 44664,
                40184, 2012,  57708, 26316, 45792, 40864, 52352, 4884,  49556,
                7940,  38588, 17816, 63200, 54628, 48652, 18432, 43936, 51288,
                27108, 20952, 37956, 25004, 47332, 308,   23660, 17488, 51264,
                4780,  24572,
            },
            {
                7824,  21608, 12448, 28236, 40824, 3216,  30216, 11660, 65072,
                50492, 4652,  13260, 61064, 23632, 8900,  39896, 12672, 1120,
                17432, 53516, 14268, 37512, 59736, 21428, 47988, 28544, 5116,
                46008, 1836,  19236, 48196, 7960,  24064, 12268, 57252, 47304,
                14504, 23436, 5980,  55648, 17196, 8048,  61492, 13992, 43864,
                48800, 38656, 648,   51824, 7124,  18744, 40572, 52700, 62472,
                31280, 13364, 47704, 4636,  23280, 51096, 26988, 6032,  43172,
                47064, 39820, 62144, 49992, 31984, 13348, 48708, 29828, 40772,
                2216,  56208, 12236, 6748,  63864, 28988, 46984, 13124, 4456,
                65132, 31948, 53472, 14296, 6384,  29284, 19204, 63804, 27504,
                37088, 43892, 10756, 57984, 15368, 60792, 1032,  12160, 56604,
                19272, 10224, 49760, 36136, 62704, 208,   31268, 37424, 13520,
                21832, 47052, 11408, 676,   26532, 13324, 9224,  32252, 63648,
                15504, 34488, 7384,  54932, 18856, 32360, 56740, 41920, 27616,
                13524, 44248,
            },
            {
                30788, 61484, 2332,  45648, 62968, 53088, 58840, 44276, 21860,
                36096, 25804, 45980, 19348, 35556, 51620, 60116, 25604, 34372,
                43708, 60960, 23772, 49336, 11668, 728,   54352, 42440, 35000,
                58628, 22812, 38928, 56344, 33192, 44692, 51940, 39800, 34196,
                27060, 60296, 36008, 1708,  43260, 50420, 28208, 34864, 18028,
                59992, 26484, 42084, 35416, 30992, 64256, 13780, 22972, 45760,
                17644, 60324, 38860, 34280, 44740, 18180, 36948, 56100, 22552,
                29256, 57220, 1576,  23908, 17128, 59728, 25444, 64172, 15988,
                37780, 27948, 50620, 41332, 23336, 424,   17392, 54376, 41868,
                9888,  44824, 27184, 40428, 55604, 47312, 33512, 1824,  57272,
                50776, 6804,  34612, 3024,  28956, 41760, 35924, 31372, 51220,
                65036, 29024, 43000, 20884, 7820,  16524, 25628, 63704, 56580,
                34744, 51784, 58312, 31064, 40932, 35920, 61188, 22416, 1148,
                42156, 48464, 10820, 28240, 64688, 39040, 5484,  48976, 63208,
                35904, 57288,
            },
            {
                51668, 25912, 38328, 33528, 9868,  23304, 37376, 16572, 1024,
                31396, 55996, 62992, 3092,  29020, 6536,  45292, 20276, 64188,
                9556,  31200, 4164,  57004, 26936, 32624, 17512, 62336, 12688,
                8880,  31016, 65448, 5808,  26616, 60080, 756,   29796, 9280,
                4192,  51652, 20696, 63808, 11432, 38152, 20316, 53028, 2812,
                9120,  57524, 15000, 54500, 10572, 48944, 55616, 28336, 8700,
                908,   51536, 10460, 28796, 63048, 13956, 324,   61652, 10796,
                52036, 14688, 37148, 9620,  52856, 39464, 7540,  46328, 21828,
                9356,  62588, 20172, 33416, 52852, 36320, 61720, 25304, 30380,
                20140, 51216, 17828, 3180,  22296, 61468, 11896, 39264, 16564,
                21368, 30724, 47200, 54772, 19112, 8104,  53140, 21652, 5600,
                37896, 14332, 3364,  54504, 59576, 47300, 10160, 42700, 19328,
                3060,  27596, 43188, 7428,  20436, 56040, 50280, 6160,  38560,
                55428, 59872, 3848,  52108, 44440, 13080, 8632,  30268, 20412,
                1216,  16792,
            },
            {
                47468, 5580,  14772, 55832, 18752, 7048,  27168, 51508, 60072,
                8736,  42420, 11396, 38432, 49344, 14640, 54296, 3476,  28484,
                48716, 39236, 19120, 42168, 64888, 7760,  37080, 51304, 20532,
                40128, 50000, 13856, 17700, 37516, 11420, 19932, 64164, 48704,
                17848, 40524, 45888, 29980, 25652, 58480, 4352,  62684, 45152,
                30060, 22288, 46488, 4568,  25220, 38080, 3052,  33752, 58148,
                41656, 26048, 19660, 56804, 7068,  53456, 47864, 34128, 19208,
                3892,  32576, 64584, 45804, 20684, 34192, 1188,  57196, 52040,
                35604, 44732, 3764,  13876, 58348, 10280, 39852, 5476,  56800,
                35040, 60672, 7916,  58188, 36720, 25340, 44704, 52652, 8824,
                42332, 65444, 13716, 24524, 59600, 44304, 62856, 16904, 25516,
                47132, 61060, 31896, 24376, 38908, 33804, 53280, 29552, 6204,
                61128, 15460, 23952, 64132, 46348, 16344, 25616, 44672, 29404,
                18284, 24236, 31432, 16500, 35508, 22788, 61420, 45472, 54840,
                10580, 40860,
            },
            {
                32272, 20916, 64704, 49952, 42076, 57968, 48116, 13128, 39600,
                20508, 53260, 24400, 17552, 58220, 32608, 41944, 36928, 56116,
                13516, 5664,  51816, 34740, 15484, 46716, 23456, 56588, 2988,
                27032, 44612, 53580, 4012,  47204, 55228, 43244, 22852, 36344,
                59404, 12568, 54656, 7408,  15288, 47640, 32448, 24000, 12948,
                40376, 65096, 33664, 18480, 61080, 43360, 15996, 20888, 48284,
                36300, 64368, 4232,  42944, 22320, 30484, 40248, 26120, 44376,
                60524, 41180, 26440, 5348,  55436, 43032, 28224, 12792, 31736,
                5128,  25228, 60064, 48728, 18880, 27236, 45548, 14948, 48348,
                2424,  13360, 43116, 49848, 32088, 15152, 4208,  29604, 60616,
                35228, 26296, 4712,  50272, 38984, 11612, 4372,  33816, 41168,
                55548, 9136,  17788, 50448, 5340,  13648, 22684, 59136, 48816,
                39828, 32072, 53696, 2236,  11200, 33000, 4024,  57992, 9568,
                12932, 46332, 63100, 40300, 1812,  50200, 26604, 14860, 38068,
                24228, 60828,
            },
            {
                8416,  44076, 28900, 576,   35648, 30360, 3344,  62820, 33580,
                4464,  29288, 47840, 1360,  64812, 21476, 10392, 24276, 17192,
                59420, 44328, 22020, 10312, 54072, 1672,  41052, 30520, 7096,
                59568, 34204, 24196, 61964, 31872, 28372, 3152,  14408, 53776,
                31688, 2644,  23728, 34644, 56556, 41588, 9916,  50212, 37396,
                55692, 1536,  7928,  51512, 29352, 56548, 6716,  62560, 11368,
                29964, 14816, 49948, 33372, 11476, 59080, 16160, 8368,  50924,
                23432, 12064, 49200, 30384, 15220, 63108, 19280, 48328, 61812,
                16976, 53864, 29720, 38768, 2156,  31988, 51896, 21644, 63688,
                24532, 38296, 28560, 20796, 10184, 64232, 54784, 19820, 48148,
                1136,  58704, 17344, 36248, 31132, 22528, 49128, 57940, 13228,
                1792,  35740, 43888, 28324, 64580, 45336, 1488,  17480, 35236,
                12304, 8520,  48008, 37048, 59240, 40288, 52176, 65440, 37420,
                33852, 53640, 7788,  21044, 59164, 54260, 4932,  33808, 58872,
                3392,  53068,
            },
            {
                36404, 13204, 54380, 23948, 11056, 16956, 46072, 25036, 43216,
                15332, 60740, 35012, 44496, 27040, 7664,  50612, 62008, 1540,
                32820, 26480, 62888, 29996, 58304, 26016, 13116, 63816, 48608,
                15836, 10600, 41248, 19104, 8272,  58608, 39272, 50400, 6596,
                27532, 44652, 65196, 38976, 300,   18724, 59616, 6048,  17056,
                26804, 20932, 48080, 36612, 13040, 22796, 46208, 38960, 53544,
                3276,  23824, 55452, 38056, 45328, 1884,  62304, 37104, 4656,
                57592, 18152, 1908,  58652, 36496, 10496, 6164,  40424, 23296,
                37028, 10892, 42372, 8276,  64952, 55456, 6552,  41476, 9752,
                33280, 52900, 61552, 72,    46368, 41088, 7436,  37884, 23628,
                11704, 45332, 53300, 9236,  63484, 268,   27296, 45240, 29772,
                51936, 23056, 58684, 11084, 20064, 56824, 41364, 52112, 25972,
                62956, 43960, 21768, 29260, 18420, 27124, 13852, 23132, 19500,
                49912, 2800,  27436, 43696, 11128, 30784, 18688, 42140, 46680,
                28400, 17532,
            },
            {
                49644, 40484, 4604,  62168, 58696, 38500, 21264, 55304, 9144,
                50336, 56712, 6000,  13328, 39936, 55676, 30284, 46684, 37968,
                53204, 7560,  47632, 4380,  38816, 17824, 45052, 33468, 19848,
                37684, 55984, 288,   51160, 35728, 16360, 24928, 63148, 42300,
                10192, 16940, 48060, 21340, 52092, 27972, 44264, 30888, 63220,
                52572, 43160, 10416, 58328, 41136, 96,    32100, 25872, 18220,
                43660, 60984, 6044,  17792, 25700, 52756, 20628, 29396, 54912,
                34464, 45992, 39176, 21576, 53128, 25496, 51200, 59408, 44964,
                544,   56284, 21312, 15340, 44132, 23512, 37144, 17760, 59520,
                47096, 5052,  15756, 56228, 34420, 26592, 51432, 16016, 56924,
                32876, 27780, 41976, 20692, 56508, 40528, 15028, 61700, 19896,
                7344,  63248, 40260, 4688,  32712, 36964, 8288,  28992, 4588,
                57436, 16228, 54540, 5440,  61648, 48968, 0,     42968, 6436,
                62260, 39384, 56772, 14548, 36828, 48312, 65032, 7968,  12216,
                21956, 62732,
            },
            {
                57316, 26712, 19632, 44696, 33056, 52216, 6604,  65424, 31472,
                19080, 37648, 23196, 61804, 16848, 34232, 4036,  12460, 19316,
                42392, 15128, 36236, 20296, 49480, 61580, 8924,  53276, 3688,
                61052, 29424, 22464, 64616, 44356, 48292, 11860, 33720, 19824,
                55360, 60320, 33004, 8220,  12980, 61896, 22512, 35668, 2228,
                14672, 32640, 61280, 27604, 17320, 64604, 8968,  51780, 58740,
                34728, 10128, 47436, 31512, 65396, 13444, 48184, 41296, 10560,
                15544, 27204, 61208, 7716,  44028, 33300, 3508,  28856, 14112,
                32776, 63036, 47844, 35048, 28112, 12272, 57772, 1176,  26444,
                30840, 19420, 39560, 23880, 12740, 60088, 30936, 3192,  63740,
                39512, 5468,  13640, 46940, 6736,  34516, 51000, 9796,  38124,
                33452, 16632, 48036, 25784, 53996, 15256, 60892, 46292, 23752,
                37876, 1124,  33972, 42020, 9336,  55756, 35744, 46108, 30036,
                25032, 16872, 32080, 60968, 23524, 516,   25676, 56912, 52444,
                34476, 1388,
            },
            {
                15672, 31832, 9772,  48912, 14952, 1956,  26408, 12508, 40940,
                188,   28216, 10764, 47248, 53596, 20872, 43908, 57232, 28632,
                64672, 23816, 56380, 11760, 31868, 2368,  42972, 27968, 24352,
                46992, 42352, 7312,  13588, 27292, 5104,  56696, 1048,  38332,
                24292, 4700,  29104, 57632, 37044, 3776,  47144, 11440, 57472,
                39856, 7100,  24012, 4764,  35248, 54548, 47536, 13900, 2316,
                28080, 21580, 40672, 57452, 8660,  35776, 3604,  23852, 63440,
                6256,  50252, 31352, 13072, 65004, 17352, 38356, 56880, 20104,
                7792,  25916, 52344, 2960,  60136, 50536, 45776, 34240, 54300,
                11764, 43412, 65064, 50488, 6116,  44524, 21800, 10676, 49248,
                19148, 61072, 54960, 30136, 24268, 17716, 59148, 26280, 3952,
                56192, 44896, 12296, 540,   50164, 21752, 31356, 11792, 49672,
                19508, 64896, 51364, 12872, 23440, 31292, 16036, 57564, 11584,
                52996, 47776, 9316,  5624,  51576, 41128, 16020, 38628, 29544,
                6508,  43584,
            },
            {
                64376, 39332, 7212,  60224, 29444, 42004, 47104, 35976, 53928,
                48980, 59300, 42684, 31668, 2296,  63232, 25780, 6140,  49796,
                9576,  316,   40456, 60148, 51492, 22312, 35860, 57576, 11256,
                14900, 52256, 31936, 55048, 39640, 21116, 61032, 30748, 46628,
                52324, 14820, 43400, 49832, 16556, 40796, 54676, 24724, 49448,
                18600, 45836, 55640, 50412, 43600, 30024, 20400, 37604, 42284,
                63408, 50936, 16076, 940,   27364, 44216, 51520, 32940, 59024,
                43304, 19624, 54432, 804,   47892, 22960, 9704,  49916, 60884,
                46272, 40176, 18060, 5744,  31568, 20292, 7684,  16312, 62908,
                49084, 8392,  28180, 2312,  37268, 18140, 55396, 41616, 35268,
                25988, 1380,  50708, 36544, 64168, 2740,  47520, 41856, 52480,
                23428, 30468, 58260, 39044, 62680, 43388, 2844,  55672, 40540,
                8836,  30168, 26192, 46980, 38708, 63776, 6988,  21488, 60580,
                1712,  36328, 44340, 21236, 55816, 32772, 45628, 20124, 61364,
                48420, 24128,
            },
            {
                54692, 20876, 52624, 36684, 22752, 55960, 17660, 61352, 22168,
                15640, 4720,  24676, 52024, 8452,  36360, 40904, 15940, 60572,
                34980, 46196, 29724, 25712, 14632, 6996,  65264, 17332, 40036,
                4584,  35396, 18604, 59508, 10096, 34784, 50716, 17508, 8128,
                62492, 35044, 1624,  25856, 63756, 20108, 33324, 8352,  29076,
                64480, 37284, 21544, 15408, 11264, 3652,  61368, 24508, 7128,
                32808, 12636, 54768, 38744, 22580, 61504, 18708, 14220, 38136,
                2852,  24984, 36124, 40868, 28300, 55248, 42196, 34816, 1844,
                30572, 11580, 54924, 37932, 64396, 24960, 42264, 38516, 4048,
                23156, 35704, 53676, 58016, 32104, 47872, 62840, 28808, 8692,
                46012, 15392, 21948, 8228,  44004, 11560, 31096, 20452, 13996,
                65184, 6876,  18896, 27752, 35208, 6108,  18148, 61628, 33340,
                14596, 44720, 60140, 2640,  18824, 52232, 41696, 26728, 33572,
                40340, 18644, 64496, 29088, 12940, 62716, 9696,  4432,  14272,
                36252, 11212,
            },
            {
                28448, 46324, 3268,  12956, 63292, 4996,  10124, 33908, 7528,
                30616, 64236, 38172, 19532, 14476, 58324, 48404, 21964, 31304,
                54188, 18456, 4632,  44416, 55128, 38484, 48028, 30332, 54384,
                62416, 49068, 1480,  23892, 45880, 3196,  41356, 28720, 13216,
                22448, 39780, 55076, 10912, 30440, 5000,  44932, 58796, 3452,
                12676, 31876, 1056,  62412, 26848, 40284, 56752, 16704, 49436,
                44852, 4368,  59888, 30400, 46720, 4988,  53672, 9172,  28768,
                47080, 57624, 16620, 8680,  62708, 4744,  13664, 26392, 16584,
                63668, 23784, 43564, 14380, 48496, 10588, 56436, 28880, 60624,
                13636, 41456, 20720, 15416, 25424, 7208,  13760, 2708,  52004,
                58800, 32152, 38576, 57304, 27256, 60804, 39488, 55708, 1336,
                36680, 43192, 9860,  54860, 13500, 24384, 47252, 27252, 6640,
                57532, 22208, 5928,  55000, 36532, 11096, 3408,  49096, 14728,
                54832, 5076,  24744, 49876, 2176,  35332, 26304, 59216, 51260,
                42012, 2084,
            },
            {
                17836, 58488, 34132, 43988, 25788, 51088, 28592, 57620, 40096,
                44548, 11864, 55356, 46084, 33248, 28312, 10728, 2004,  39100,
                13320, 51008, 63624, 10684, 33268, 19876, 1272,  9804,  25188,
                20620, 43396, 28508, 63692, 15844, 53124, 25180, 64904, 45312,
                57796, 6644,  18972, 47416, 59732, 52476, 15360, 39220, 50532,
                25992, 41808, 58096, 47192, 34256, 51488, 8548,  31116, 58532,
                19996, 25544, 35928, 17972, 11600, 41608, 32116, 56276, 21176,
                64512, 11788, 33504, 51540, 20956, 31560, 59896, 44400, 50700,
                36192, 3728,  58564, 27768, 33844, 356,   51340, 18280, 32600,
                46760, 1280,  62456, 10040, 43912, 60308, 40132, 22868, 17560,
                65248, 43184, 4716,  13168, 53552, 17260, 5756,  33984, 50292,
                26648, 46796, 59612, 32312, 51356, 64052, 39584, 53348, 35732,
                50220, 42528, 16664, 33044, 28688, 58204, 44052, 61772, 29600,
                10156, 59456, 42212, 15880, 38532, 54264, 47272, 30648, 22120,
                56348, 33084,
            },
            {
                25244, 6276,  50124, 15924, 39688, 20148, 46188, 3048,  52812,
                18344, 27212, 968,   60328, 5360,  51476, 63812, 45284, 57456,
                27008, 7696,  22928, 42228, 58900, 27684, 61036, 45564, 34100,
                5960,  58416, 8992,  37252, 32392, 56820, 11640, 6220,  36396,
                50156, 27344, 32800, 42244, 23500, 9060,  35204, 21872, 60964,
                17868, 9308,  52604, 6444,  18788, 22716, 43860, 564,   37156,
                10260, 64732, 52580, 7744,  49768, 62924, 25408, 60,    40344,
                50020, 6712,  43508, 58836, 38192, 46672, 18568, 7424,  57248,
                9984,  52676, 19412, 6456,  61172, 22360, 44412, 5280,  58352,
                24832, 52140, 29784, 37960, 50368, 34040, 54388, 30452, 36772,
                10876, 24372, 49996, 20056, 35096, 48396, 23860, 9164,  61576,
                16676, 22308, 4428,  37568, 1600,  16328, 9096,  20716, 12224,
                656,   25680, 63572, 48012, 13224, 24424, 19768, 7448,  35304,
                22388, 48084, 31776, 57012, 7780,  18868, 3336,  40388, 16392,
                8024,  61144,
            },
            {
                38108, 10648, 64568, 30172, 820,   59220, 36116, 14604, 23508,
                62696, 50216, 35500, 22508, 43020, 17324, 24812, 4308,  19804,
                32864, 61416, 37616, 16456, 3332,  52860, 13860, 40872, 50828,
                16168, 39168, 12996, 47760, 4856,  19324, 43656, 31056, 492,
                16368, 62072, 12596, 3056,  37800, 65352, 27716, 1880,  43904,
                55444, 36128, 14580, 29232, 39076, 64328, 12876, 53592, 27732,
                47908, 41924, 1816,  28496, 57328, 15176, 36368, 45840, 17424,
                30856, 23544, 3684,  26896, 12512, 1260,  24568, 33316, 39244,
                22244, 29888, 41304, 46636, 36508, 55432, 14836, 40332, 11188,
                35152, 6728,  56124, 17312, 3280,  20152, 5516,  47572, 56612,
                1052,  28040, 40780, 62612, 1968,  45208, 28968, 41552, 56632,
                31644, 12140, 49460, 19564, 42280, 29656, 46028, 58048, 31508,
                60336, 38016, 9612,  40956, 1744,  50608, 65216, 38892, 53812,
                45188, 468,   12412, 27176, 62164, 44636, 23864, 65480, 12308,
                48776, 43768,
            },
            {
                54040, 23024, 41068, 55584, 8408,  32732, 61840, 11116, 42252,
                5968,  32000, 9440,  13976, 39392, 30864, 56020, 36480, 41700,
                49624, 840,   46784, 25440, 49044, 36164, 5344,  23220, 62116,
                30996, 54800, 26264, 51928, 61592, 22904, 39568, 59880, 53944,
                21792, 44296, 55712, 49472, 17632, 56760, 13540, 48200, 30180,
                4860,  63720, 23840, 45400, 3208,  56044, 32948, 16300, 61928,
                22076, 13832, 33848, 19460, 39404, 22764, 6248,  58484, 12732,
                52984, 36960, 55268, 15792, 62232, 49404, 54016, 65436, 2840,
                48336, 12920, 63364, 16824, 8848,  26260, 31072, 64292, 19724,
                48592, 42724, 22540, 64816, 45948, 12660, 26244, 62196, 15216,
                44436, 53000, 7752,  30812, 57604, 10976, 64556, 14920, 5272,
                53188, 39888, 63592, 24732, 60420, 53916, 5560,  23620, 43784,
                17904, 52772, 29904, 21208, 56324, 34228, 15224, 4252,  26052,
                17020, 61004, 37492, 20968, 50196, 11048, 33392, 52832, 36120,
                280,   29432,
            },
            {
                5152,  18668, 12844, 47584, 24628, 18048, 53704, 48788, 26416,
                37920, 58492, 47076, 64612, 53880, 7476,  12156, 60104, 15212,
                9512,  54652, 30320, 11920, 57868, 31952, 18256, 56456, 10828,
                168,   21348, 42404, 2808,  34664, 14216, 47244, 26812, 10396,
                34024, 39036, 8096,  25044, 31628, 6376,  51828, 41196, 10788,
                19592, 33160, 49340, 59532, 10988, 25860, 50148, 5368,  39776,
                3544,  55140, 44768, 61408, 47588, 2864,  32736, 63848, 26556,
                9268,  60368, 45036, 32476, 40140, 28500, 10836, 42884, 16048,
                26868, 56596, 32324, 1888,  49616, 38832, 52976, 2344,  59752,
                27876, 14088, 9180,  33220, 28656, 59320, 41704, 9012,  21632,
                34660, 60528, 17980, 14184, 23108, 37612, 51564, 20780, 35940,
                448,   28284, 44760, 7880,  33612, 14516, 36536, 64924, 3132,
                48888, 14756, 5460,  61412, 45484, 8776,  30964, 42344, 57736,
                9284,  52124, 33700, 4560,  41620, 29816, 5692,  57880, 26744,
                20476, 62904,
            },
            {
                34756, 45320, 60768, 36588, 4276,  43488, 29564, 2000,  20248,
                55804, 16100, 4420,  19656, 27084, 49168, 44516, 22760, 28460,
                64948, 18424, 39724, 21624, 6560,  64156, 43592, 28604, 48140,
                36708, 65080, 17968, 57896, 28908, 7420,  51276, 3744,  18368,
                64448, 2336,  29532, 61228, 46148, 20756, 33916, 62480, 25596,
                54036, 38724, 1196,  17400, 41840, 36888, 19936, 46252, 29756,
                58784, 25064, 8252,  30460, 11224, 55588, 42544, 48760, 20768,
                40880, 1496,  18416, 7896,  21756, 5112,  36400, 19680, 59284,
                7720,  35472, 43648, 24324, 62176, 12372, 21408, 45140, 5680,
                37456, 53888, 57936, 39948, 196,   52944, 37344, 50876, 31696,
                4112,  46708, 38856, 54332, 33016, 3140,  43872, 26128, 47464,
                58724, 17756, 11120, 57340, 2656,  27300, 50728, 10432, 39928,
                34456, 24560, 54848, 37272, 26692, 18812, 60036, 47004, 22056,
                29280, 40836, 63632, 14748, 54596, 59912, 17520, 39140, 47068,
                14872, 50948,
            },
            {
                31528, 1692,  27376, 52048, 57996, 14128, 64056, 39456, 9932,
                45560, 30472, 24620, 37012, 60940, 1632,  33980, 5608,  51972,
                35072, 3800,  59064, 45644, 51628, 9468,  38456, 3908,  25392,
                44564, 7864,  31932, 11328, 45720, 63464, 36460, 56952, 41144,
                47852, 52448, 15652, 11300, 57980, 39952, 116,   15136, 44972,
                7672,  61008, 13784, 31252, 56768, 6944,  60648, 9596,  34380,
                18088, 50668, 36308, 16224, 52104, 27420, 7500,  14696, 34936,
                50996, 29208, 64600, 48540, 57048, 52044, 61216, 30564, 45812,
                50748, 4032,  53856, 18792, 57716, 6676,  34444, 29708, 51204,
                17628, 25196, 4508,  47400, 15420, 23776, 6576,  18632, 64540,
                27244, 10612, 24972, 48668, 6288,  60236, 18300, 8716,  61972,
                33180, 22604, 51860, 38164, 47224, 18992, 55864, 21536, 28724,
                62320, 8208,  43300, 2824,  12188, 51560, 35840, 1128,  13032,
                49220, 19388, 2320,  24788, 45228, 8592,  22652, 2892,  42912,
                10172, 58632,
            },
            {
                39720, 22456, 16296, 9668,  33136, 21804, 7172,  51196, 34676,
                62444, 2836,  42740, 52172, 10400, 40912, 17584, 57600, 46508,
                24684, 43100, 13884, 27048, 33540, 15620, 20552, 60128, 55752,
                14880, 52760, 49152, 40292, 24444, 15904, 20812, 30476, 12808,
                25624, 22104, 37468, 43520, 4924,  53240, 24180, 36572, 57064,
                28456, 22024, 47020, 52156, 27232, 22872, 43808, 52484, 65000,
                12280, 41564, 5724,  63308, 23392, 38292, 18624, 61740, 4516,
                56452, 11996, 23316, 33696, 14416, 42184, 304,   24436, 12124,
                38996, 22048, 10108, 29176, 41084, 47724, 15648, 61348, 10436,
                43076, 63932, 31580, 20444, 62312, 35200, 55688, 44148, 13392,
                40728, 59108, 636,   63492, 41820, 28556, 55912, 38764, 13512,
                6428,  42316, 29452, 15520, 62756, 41024, 5964,  44804, 13012,
                56996, 18536, 31480, 48412, 64456, 23120, 6516,  54732, 62080,
                32244, 7732,  57160, 34908, 28368, 48236, 64264, 32404, 56164,
                25012, 7388,
            },
            {
                48208, 54632, 65120, 42008, 37812, 47812, 26984, 59444, 23004,
                12436, 49320, 58152, 14720, 29476, 21548, 63616, 31140, 12396,
                7984,  37488, 60908, 2624,  54748, 41292, 49508, 31212, 11968,
                34752, 22832, 2284,  61180, 5752,  54816, 43796, 8976,  62504,
                5428,  59368, 32684, 19480, 28024, 64268, 9424,  18112, 50120,
                5204,  43236, 34284, 9960,  62656, 3420,  15284, 28340, 684,
                22384, 48476, 57852, 2600,  43752, 56856, 46924, 30948, 24924,
                38924, 43384, 2728,  46704, 26768, 9780,  17600, 34540, 56248,
                64260, 15472, 60380, 36472, 972,   25144, 55156, 39308, 22668,
                1740,  35868, 12128, 50452, 42524, 10088, 30240, 2564,  49064,
                53600, 19648, 35252, 16172, 21436, 11820, 46260, 30980, 53532,
                49132, 65236, 1848,  35496, 9404,  25132, 60112, 32976, 49544,
                112,   41688, 58904, 15608, 28280, 44468, 40144, 16640, 27052,
                37924, 43868, 51572, 17216, 11412, 37592, 51024, 13732, 36228,
                18092, 61548,
            },
            {
                12244, 3548,  30552, 5816,  18996, 56504, 792,   16660, 41524,
                32364, 19168, 7804,  35488, 55008, 48068, 4672,  38688, 53752,
                20404, 48924, 30496, 22892, 62940, 6324,  24660, 1628,  63984,
                39276, 19292, 56432, 27608, 37528, 33020, 592,   49676, 35080,
                45008, 55108, 1344,  50592, 13632, 33488, 47664, 59676, 30952,
                12880, 64772, 1972,  18448, 36464, 48868, 32560, 39964, 55724,
                37364, 26548, 31604, 20136, 34208, 12704, 456,   52816, 9912,
                60004, 17084, 54240, 37324, 58672, 63272, 39628, 47396, 5868,
                26620, 31324, 49368, 45352, 13192, 65168, 32188, 7640,  48572,
                58832, 54644, 28532, 7216,  56992, 26092, 16740, 58524, 23048,
                37688, 7852,  31316, 57168, 50564, 36492, 7008,  23412, 17256,
                4228,  25864, 56608, 20988, 54344, 30816, 3284,  17112, 37068,
                26252, 53680, 22016, 38568, 9632,  52820, 33076, 58044, 10444,
                23692, 65068, 4916,  40348, 60868, 1112,  20244, 5924,  53008,
                28676, 44568,
            },
            {
                21152, 26336, 50232, 43980, 62240, 11628, 29876, 45224, 55148,
                5056,  28068, 64848, 23924, 224,   43984, 26400, 15776, 61604,
                2020,  56576, 17120, 9820,  34964, 46792, 17548, 53508, 43080,
                7328,  29676, 46504, 13316, 17640, 52080, 60312, 23308, 14600,
                27352, 40240, 8236,  61768, 45692, 38228, 21016, 2688,  41592,
                53948, 38880, 25920, 58168, 41328, 54428, 11484, 59744, 17692,
                8564,  44396, 13912, 60836, 8988,  28628, 65328, 36384, 21784,
                45976, 31900, 7324,  20360, 4060,  29468, 53460, 21028, 51192,
                2432,  42452, 7472,  53400, 20640, 4728,  43504, 17844, 26952,
                13936, 45196, 18696, 40360, 1324,  46844, 60996, 33856, 4612,
                27796, 47512, 13984, 43760, 26608, 2420,  64040, 58076, 34668,
                44900, 40208, 12456, 48256, 43612, 14172, 52332, 46268, 63940,
                6684,  11548, 35276, 4868,  62884, 1564,  20268, 48628, 4016,
                46232, 13844, 30800, 22424, 53972, 26372, 43924, 31092, 59000,
                2144,  38012,
            },
            {
                15972, 57800, 34592, 14456, 23956, 53384, 35832, 9072,  60652,
                38232, 52576, 46364, 40104, 12608, 60200, 9260,  50888, 34412,
                27772, 44584, 40012, 51868, 13344, 58440, 37304, 27172, 10164,
                50984, 58340, 4576,  63664, 42816, 10492, 29032, 39428, 18652,
                52648, 11132, 30720, 17404, 24548, 4200,  54812, 27332, 16324,
                8372,  21668, 47404, 14200, 6632,  20892, 25052, 4724,  46812,
                63892, 3492,  50872, 54176, 40460, 49120, 17920, 42272, 5776,
                13700, 63856, 25480, 50084, 12604, 43084, 7836,  14680, 32760,
                62488, 18412, 23888, 34156, 58428, 28648, 50860, 61016, 38076,
                3252,  33204, 24300, 63140, 37064, 21220, 51548, 12500, 43164,
                62872, 55144, 3500,  61304, 9448,  52864, 40944, 14928, 28012,
                60832, 19164, 32592, 7120,  61432, 23268, 39576, 8668,  20828,
                29420, 50684, 60212, 47292, 30524, 14396, 25800, 60644, 34848,
                54476, 19100, 59084, 9216,  46756, 15056, 62620, 39960, 23128,
                9528,  63192,
            },
            {
                52340, 40660, 8744,  59556, 4492,  39916, 49172, 20940, 14844,
                25508, 2680,  16796, 33140, 56540, 19312, 31684, 42176, 21972,
                11556, 5132,  65420, 25824, 384,   44084, 31964, 21224, 61760,
                16244, 32856, 40480, 25880, 21924, 7092,  47924, 57416, 2548,
                64988, 21620, 59044, 41880, 56816, 11796, 62960, 44136, 35460,
                61276, 32132, 51076, 28736, 63736, 45288, 30860, 51820, 34104,
                21604, 29016, 16532, 35764, 23572, 3972,  26332, 58184, 51776,
                28884, 55680, 41044, 33328, 60744, 35900, 24116, 58308, 44616,
                37392, 9980,  55668, 39848, 16356, 11028, 35180, 21712, 8952,
                49936, 59792, 53092, 10660, 14964, 31860, 5840,  39412, 18264,
                9936,  21572, 39904, 24404, 32676, 18932, 47232, 11016, 264,
                52240, 9272,  57560, 37892, 1376,  27660, 34352, 55364, 58372,
                41336, 15100, 24712, 19068, 40416, 56728, 43168, 7692,  38964,
                28580, 388,   42192, 36576, 33032, 57092, 6924,  11680, 49500,
                46056, 32484,
            },
            {
                29168, 8,     47260, 20272, 32048, 26916, 2292,  62936, 34112,
                43548, 11104, 62248, 22368, 49000, 37480, 5940,  63172, 57684,
                46648, 36520, 31328, 20112, 59996, 8456,  56084, 3320,  47764,
                23712, 36088, 932,   50376, 55260, 62152, 35412, 5284,  32492,
                45348, 36244, 6148,  48356, 33944, 29264, 6764,  49568, 24044,
                652,   57116, 11052, 3596,  35124, 15868, 61040, 9500,  42604,
                56700, 39448, 59224, 6464,  46344, 61764, 14584, 33784, 2260,
                38396, 9288,  16140, 1000,  46900, 18736, 54744, 1784,  28684,
                12916, 60084, 27456, 2784,  47084, 62988, 476,   55856, 41680,
                29852, 16764, 5492,  42360, 56420, 28376, 64144, 45872, 52748,
                30640, 35036, 49772, 15920, 59088, 37500, 29324, 22224, 49656,
                35892, 23940, 29480, 16184, 50360, 63188, 11784, 17940, 4628,
                32284, 44828, 2568,  54948, 33932, 11280, 50296, 22232, 16080,
                62012, 52272, 24872, 49104, 3076,  17456, 28144, 35492, 55092,
                18780, 5672,
            },
            {
                56900, 24496, 36796, 64420, 52016, 16292, 55908, 46644, 7288,
                57432, 50364, 29716, 8152,  3700,  53248, 14904, 24892, 1092,
                18100, 54944, 14424, 49468, 41388, 28392, 14996, 38904, 54356,
                12336, 45832, 9000,  18576, 31136, 12152, 16576, 42112, 23972,
                13904, 50760, 26348, 2120,  15560, 52388, 39808, 18840, 13452,
                45876, 38140, 19668, 43036, 55388, 39248, 2068,  19276, 13488,
                25884, 860,   11868, 32280, 20416, 55632, 10148, 44912, 19832,
                48324, 59156, 22592, 52072, 27928, 10368, 65224, 38788, 49496,
                22200, 51848, 6104,  42668, 30824, 52616, 25292, 45716, 13104,
                64788, 22768, 46996, 34508, 49364, 8432,  23624, 1856,  26196,
                56980, 7024,  64380, 1292,  44936, 4772,  56216, 62632, 42876,
                5780,  64092, 54660, 45432, 20568, 6192,  42956, 48296, 25852,
                65400, 37216, 9352,  61876, 5660,  27560, 64628, 2452,  32108,
                44872, 12572, 6664,  64200, 21172, 51976, 43228, 24368, 64980,
                14260, 42288,
            },
            {
                61404, 10072, 13420, 44320, 6312,  41560, 11956, 22440, 28516,
                18072, 40532, 36728, 59740, 26520, 45324, 30252, 39788, 48796,
                8688,  60708, 24176, 6392,  34096, 51348, 18704, 62804, 5200,
                28948, 58948, 65200, 43436, 37712, 27268, 46584, 53052, 59860,
                8720,  55324, 19544, 63928, 37352, 22252, 9452,  57704, 65408,
                30004, 53212, 6196,  60244, 23516, 27208, 49236, 54032, 37116,
                47240, 62184, 50184, 43672, 27360, 38644, 51224, 30548, 63372,
                25156, 4460,  36336, 62396, 44336, 5704,  32032, 16000, 4388,
                45440, 34924, 64244, 15104, 20216, 37964, 7332,  17948, 32124,
                3844,  39180, 26452, 1012,  18984, 58968, 37908, 16892, 60484,
                13740, 42388, 20032, 27464, 54516, 12992, 8116,  31828, 16728,
                38668, 13372, 2576,  40828, 58520, 31160, 36504, 53428, 1020,
                13532, 51116, 23388, 46048, 17360, 52552, 36432, 47560, 9488,
                56428, 18464, 40620, 29908, 10488, 60188, 38408, 4876,  1568,
                33668, 21496,
            },
            {
                48736, 27964, 54908, 18664, 30560, 59864, 37832, 32932, 65376,
                580,   54536, 13244, 19492, 41772, 64640, 10852, 54392, 33624,
                28092, 42908, 38104, 2456,  64100, 11208, 44952, 24356, 32440,
                40988, 20468, 14784, 51968, 2280,  61292, 20744, 3576,  28728,
                31744, 39212, 43368, 12164, 46408, 60508, 31676, 42248, 5308,
                26584, 15112, 33388, 47932, 12100, 7808,  32352, 65084, 5176,
                30208, 22452, 8184,  17452, 64620, 1460,  7272,  35708, 17148,
                11336, 41548, 30192, 13496, 20408, 53100, 25684, 42056, 57492,
                18228, 8804,  25564, 48112, 58716, 11676, 62044, 35504, 57776,
                51792, 10116, 55124, 61596, 13396, 44224, 50868, 33148, 4280,
                47092, 37072, 10616, 48164, 33560, 41176, 26044, 59464, 20708,
                48380, 26964, 34188, 10140, 24756, 14712, 60480, 21352, 29108,
                57132, 19368, 39476, 30116, 13112, 41576, 19916, 24580, 60000,
                37816, 26560, 34180, 55232, 47220, 14700, 31404, 57840, 45612,
                53572, 37840,
            },
            {
                17140, 3380,  34712, 50312, 1560,  24836, 8840,  51664, 42880,
                5620,  24656, 34776, 48224, 2352,  16708, 22820, 6692,  61844,
                20560, 12592, 52720, 46424, 21728, 56972, 36324, 1076,  48832,
                53164, 6636,  34544, 24512, 6260,  56312, 10552, 40688, 63056,
                17448, 232,   58296, 28132, 4236,  54136, 24884, 1408,  36556,
                50808, 21500, 62148, 41272, 18080, 58424, 44748, 21184, 16192,
                41464, 54688, 34144, 58052, 12984, 42136, 23464, 60604, 54172,
                44120, 50856, 57572, 7736,  39672, 59340, 34556, 11544, 61732,
                30432, 40352, 56156, 33104, 3104,  23376, 41072, 27220, 47424,
                20324, 43240, 29276, 36048, 22212, 30352, 11184, 41140, 54108,
                22596, 29500, 52096, 60736, 23088, 15660, 50792, 44552, 4088,
                53764, 60764, 18852, 52264, 47752, 3796,  9052,  45728, 41492,
                6852,  34392, 60656, 53932, 596,   57948, 31384, 4920,  14056,
                50172, 4076,  62636, 1452,  23144, 7392,  50688, 19796, 12392,
                26080, 6608,
            },
            {
                43644, 59356, 40368, 21852, 63560, 56200, 47672, 19732, 14544,
                60640, 45868, 9248,  56512, 32256, 51380, 58700, 35564, 44800,
                3172,  57504, 32128, 17044, 29660, 7444,  26968, 60688, 16624,
                10180, 57352, 28088, 39748, 48612, 32844, 23092, 50356, 13088,
                34636, 47980, 21484, 7796,  35328, 16732, 48844, 12840, 55952,
                44960, 9088,  2932,  36932, 29484, 164,   51368, 35948, 10412,
                60500, 2396,  25464, 37388, 52468, 29060, 47116, 14824, 5720,
                27188, 736,   33604, 17516, 48564, 2880,  22864, 46840, 192,
                53688, 21228, 13184, 5576,  44352, 54460, 50692, 2036,  14716,
                6328,  63752, 16260, 7248,  53072, 2648,  65232, 27036, 7872,
                62772, 17800, 2480,  5848,  39092, 65020, 904,   35176, 11644,
                30196, 7512,  43220, 64856, 28476, 39644, 32556, 62344, 16088,
                48932, 10936, 4248,  26224, 47264, 9016,  63644, 43932, 53664,
                21344, 41876, 16204, 45172, 37096, 27676, 41488, 60824, 8808,
                30056, 63824,
            },
            {
                13756, 31824, 9608,  45816, 13096, 29524, 4084,  35968, 27284,
                52828, 30676, 21288, 62392, 27920, 5296,  39004, 13896, 17888,
                50284, 25908, 9492,  62276, 42232, 54244, 13212, 38756, 43844,
                22476, 63256, 46068, 12828, 59476, 16452, 43700, 36848, 4964,
                57000, 24280, 52144, 62836, 30068, 44200, 20464, 63660, 32824,
                17568, 57976, 25904, 53536, 63196, 14648, 24332, 57096, 28472,
                48268, 14156, 45364, 4784,  20548, 9192,  56000, 32580, 40016,
                20852, 64384, 23708, 54508, 28652, 63360, 14760, 51344, 27872,
                7268,  35884, 50176, 60776, 29384, 17160, 9732,  31232, 60304,
                37664, 23828, 48848, 58232, 38792, 46004, 19992, 55840, 15296,
                34308, 44984, 58612, 31112, 19396, 9660,  55848, 24540, 40000,
                57404, 22660, 37008, 588,   16780, 58116, 54216, 23612, 36056,
                27452, 63160, 43444, 22000, 33640, 38028, 16996, 27812, 35292,
                7456,  30908, 58508, 12112, 53448, 63164, 16972, 34520, 47868,
                39500, 51760,
            },
            {
                23760, 56792, 26908, 7104,  38336, 16916, 43616, 58252, 11008,
                39172, 16136, 1524,  37172, 12316, 43312, 23912, 47564, 30588,
                63472, 36160, 4704,  39520, 48100, 2028,  19800, 51180, 31192,
                3524,  35908, 18500, 852,   30368, 8348,  54116, 64844, 27156,
                45068, 10048, 41284, 14036, 37972, 59032, 9828,  40564, 6628,
                29204, 39240, 48332, 10940, 43596, 33716, 4376,  42480, 7300,
                19192, 38900, 63832, 30668, 49352, 62016, 18288, 2200,  59472,
                49464, 12288, 38252, 45028, 9988,  37004, 43448, 19640, 38416,
                64836, 16248, 42784, 26240, 38564, 64116, 21940, 45736, 55404,
                33892, 244,   41896, 10808, 25876, 32804, 5788,  48468, 40080,
                24624, 11624, 41432, 53828, 36632, 46104, 29164, 61644, 17884,
                47372, 14896, 32936, 10908, 50316, 20356, 5352,  12744, 1548,
                52120, 18296, 55900, 14736, 59624, 51480, 2488,  56636, 11340,
                60928, 48168, 26064, 19440, 32532, 5572,  2148,  24700, 55920,
                20908, 604,
            },
            {
                62188, 17724, 48520, 53048, 60520, 33028, 50800, 22536, 6484,
                61888, 41848, 54808, 50188, 18596, 64320, 57072, 8816,  132,
                41792, 21960, 55792, 14884, 23768, 58924, 34312, 64548, 8080,
                26232, 50144, 55512, 42296, 61824, 26000, 19944, 2008,  15396,
                30896, 61272, 19048, 5536,  49836, 3016,  27064, 53136, 23668,
                60096, 14428, 5212,  22708, 19400, 55716, 38264, 61328, 50764,
                32208, 53496, 23748, 11332, 40804, 34824, 25892, 44360, 36268,
                29508, 6884,  60460, 15912, 4020,  32572, 5668,  55380, 10800,
                45464, 23928, 57188, 2684,  12008, 34808, 4540,  40044, 25540,
                12892, 52180, 30528, 18188, 62052, 14524, 59696, 36488, 632,
                57308, 50236, 6724,  14832, 26212, 49540, 13300, 6036,  51756,
                3072,  63368, 44660, 55488, 38700, 27080, 46884, 43048, 58680,
                38544, 31892, 7484,  40244, 11744, 25328, 20280, 45676, 39260,
                23564, 248,   51604, 9132,  39988, 57008, 49528, 43332, 15204,
                6284,  36512,
            },
            {
                41716, 5136,  35172, 20492, 1180,  25216, 63176, 2876,  48204,
                33676, 25024, 7396,  46420, 26732, 3856,  33768, 54104, 28136,
                59640, 11444, 50916, 32656, 27984, 10520, 45844, 15512, 53540,
                40700, 12084, 6084,  21916, 38036, 51372, 34160, 48156, 40224,
                7228,  54480, 33384, 25396, 55904, 15976, 34400, 46460, 1728,
                50240, 35624, 64956, 30876, 45756, 8296,  26932, 12944, 22140,
                1648,  57888, 16352, 3112,  54520, 6232,  13304, 57640, 8796,
                52780, 19436, 47012, 26500, 56704, 61944, 24312, 30712, 59008,
                1668,  33940, 8732,  20312, 46824, 52844, 59212, 18564, 7700,
                62852, 21064, 56480, 45200, 3468,  51736, 23112, 10248, 30968,
                19252, 28124, 60904, 21908, 63328, 1864,  33472, 21284, 42160,
                30744, 26588, 6572,  23792, 62500, 34904, 7964,  30456, 64700,
                20948, 3328,  47960, 29056, 64196, 5820,  49780, 32136, 62388,
                14924, 43376, 35052, 64192, 46144, 21432, 29640, 10856, 65484,
                32268, 46376,
            },
            {
                12108, 29052, 55088, 43268, 11164, 40328, 14616, 28388, 55620,
                17864, 12868, 59144, 31576, 10196, 40268, 20992, 49412, 15704,
                37940, 18904, 45128, 6408,  61620, 37672, 3876,  29752, 23484,
                61084, 47208, 31816, 57696, 14532, 3676,  11320, 60120, 22480,
                50724, 37276, 880,   47328, 64636, 42340, 21600, 62032, 12404,
                19244, 42172, 52872, 1060,  59688, 49384, 17188, 64016, 46652,
                9800,  35244, 43832, 60132, 27700, 46404, 63536, 16464, 24388,
                42436, 2792,  33948, 39812, 51772, 18272, 48808, 41412, 13848,
                52292, 28424, 49144, 62628, 31368, 15592, 42072, 50044, 29180,
                47176, 4332,  39344, 8980,  35116, 28364, 43284, 53892, 64080,
                47268, 3748,  35216, 43532, 8908,  39744, 54328, 60204, 36908,
                9920,  58916, 52408, 13388, 2572,  18108, 49268, 14412, 25272,
                10576, 50944, 44308, 22712, 34320, 42532, 55240, 9840,  5024,
                29440, 53876, 17748, 3808,  27076, 14264, 59296, 38040, 52440,
                25644, 57792,
            },
            {
                50268, 3120,  15684, 64752, 30768, 57444, 46100, 8356,  37888,
                44064, 63884, 22080, 36392, 53132, 61396, 13252, 45608, 6688,
                25068, 64936, 1476,  54012, 20920, 49328, 42672, 58344, 19184,
                2060,  35028, 17272, 27912, 43408, 62748, 45996, 28916, 17604,
                56884, 13188, 27904, 20572, 11064, 30536, 7868,  37748, 56744,
                32336, 9228,  24980, 15604, 39980, 33036, 4004,  36980, 29936,
                41392, 26172, 49804, 18716, 32648, 37952, 21332, 48576, 31812,
                55956, 65024, 13888, 22300, 600,   11732, 35788, 7380,  21424,
                61288, 17300, 39704, 55256, 5932,  27148, 1220,  10516, 57460,
                36564, 15832, 26188, 64676, 49224, 19580, 40392, 7296,  16924,
                37980, 12624, 51864, 31740, 56212, 18684, 45556, 24940, 15812,
                49408, 18908, 40100, 31804, 43516, 60184, 53316, 40884, 56692,
                37164, 16936, 61456, 54052, 872,   15928, 26564, 19040, 37108,
                57748, 22744, 11572, 38780, 54568, 7480,  34708, 2912,  17484,
                7044,  20212,
            },
            {
                39528, 60168, 23324, 47504, 6300,  21976, 35724, 53232, 20156,
                30376, 488,   6236,  42776, 17064, 2080,  29712, 35584, 57528,
                42788, 30040, 34440, 40612, 16888, 26344, 7628,  13056, 54884,
                38948, 9880,  65500, 52708, 8268,  24144, 35876, 4788,  42152,
                9232,  63384, 45000, 39188, 58404, 3880,  54124, 45264, 27672,
                3480,  47904, 57764, 29008, 11516, 55212, 23696, 51152, 58476,
                14856, 5512,  64796, 8156,  12204, 52360, 1532,  5464,  39224,
                10964, 27440, 49732, 58644, 29928, 44796, 64128, 26328, 47496,
                37244, 4056,  43964, 12784, 22044, 37792, 65360, 33628, 23616,
                42896, 53388, 32724, 11976, 58616, 1384,  13764, 61528, 26888,
                44920, 57668, 24152, 15936, 4912,  29560, 11576, 6976,  64216,
                740,   46172, 22088, 57108, 9200,  28308, 21556, 336,   6292,
                32884, 27760, 8468,  13076, 36420, 58852, 46472, 63428, 41572,
                2848,  45260, 59988, 31532, 48276, 62552, 23300, 45468, 41108,
                61704, 31196,
            },
        },
        {
            {
                54548, 47212, 13756, 41580, 52760, 11772, 4688,  15168, 42568,
                34924, 4020,  55056, 61228, 31464, 47164, 5148,  34540, 11028,
                54832, 3788,  25580, 8044,  60288, 4300,  24332, 50164, 29740,
                18984, 44860, 976,   23000, 16472, 58368, 52592, 33712, 56472,
                39564, 12040, 59376, 36864, 4,     12416, 42592, 60200, 18432,
                46456, 5484,  61332, 34292, 57160, 22500, 39040, 8624,  46704,
                54772, 31516, 11744, 48048, 64312, 55704, 42336, 60456, 25408,
                50672, 44056, 14176, 23916, 60804, 2124,  30860, 57584, 3592,
                44956, 17780, 7920,  53508, 35556, 23412, 48348, 52524, 35928,
                21660, 50664, 37480, 65348, 12812, 31996, 58064, 4592,  63968,
                36564, 32696, 52160, 41700, 22632, 37324, 60680, 29284, 20900,
                35808, 46468, 59636, 15932, 54044, 31076, 58424, 48984, 14040,
                27184, 5164,  21408, 52968, 520,   49412, 14340, 60324, 17772,
                7976,  2464,  26300, 10040, 60236, 19040, 12244, 56776, 49576,
                32404, 8908,
            },
            {
                1376,  26672, 33728, 58344, 29684, 44604, 55836, 32416, 20716,
                64640, 16816, 44644, 7008,  19688, 14104, 43472, 17964, 38672,
                51332, 44420, 15000, 41128, 34388, 12016, 39232, 15464, 56484,
                41800, 64088, 27620, 51076, 5992,  43944, 40572, 28168, 1540,
                50044, 19012, 5556,  26196, 49584, 33260, 24368, 48356, 2416,
                40732, 26004, 37664, 8016,  29848, 16060, 65220, 27564, 13812,
                41384, 60216, 37332, 8244,  19968, 28584, 15040, 9516,  40052,
                18964, 3608,  33004, 58076, 46104, 17024, 35576, 52428, 13656,
                37636, 32304, 49288, 14984, 63296, 45328, 17016, 2784,  58688,
                11756, 46516, 1552,  19668, 6272,  48260, 22700, 39360, 45876,
                12320, 59660, 25180, 2876,  31220, 8308,  47884, 3828,  13592,
                50600, 184,   18780, 28536, 47852, 8728,  24100, 1036,  33656,
                40464, 60196, 16152, 57660, 26376, 65164, 10460, 45088, 27896,
                35664, 64124, 31384, 14668, 43056, 36364, 27352, 2120,  23464,
                63192, 42280,
            },
            {
                60148, 51276, 10080, 20776, 5928,  37416, 62272, 25920, 1616,
                39924, 11992, 48820, 26748, 38144, 55904, 29280, 62716, 1964,
                22552, 31876, 65424, 18828, 57720, 45196, 52268, 2288,  35916,
                7576,  13864, 57400, 38296, 30912, 19752, 13204, 6860,  24508,
                63464, 29824, 44228, 53856, 65420, 17208, 9144,  29288, 14244,
                63816, 55832, 12052, 49420, 45740, 4676,  51440, 33160, 19568,
                23300, 6304,  50088, 53404, 33016, 1928,  36196, 51932, 30848,
                62268, 7828,  28848, 10940, 39120, 5948,  62976, 9988,  21008,
                61476, 55068, 25272, 42116, 4844,  31672, 10284, 62220, 40876,
                32464, 27008, 53924, 41868, 59848, 35424, 14828, 26860, 54852,
                648,   17788, 39000, 56980, 62172, 19132, 25976, 44352, 39644,
                56712, 23748, 33904, 43876, 65312, 13108, 39104, 60988, 17748,
                55008, 44328, 11808, 36624, 47064, 19240, 34128, 5344,  55480,
                23380, 46864, 40872, 58164, 3476,  48288, 64976, 33984, 52932,
                14420, 29844,
            },
            {
                4552,  24168, 39164, 64008, 47488, 17700, 9192,  49376, 54336,
                29008, 58732, 23104, 53268, 104,   60388, 24000, 8228,  47800,
                59320, 12780, 37216, 916,   28700, 22200, 62488, 27040, 47812,
                21020, 33952, 46220, 2984,  60208, 48532, 62036, 54036, 46244,
                10484, 35232, 14864, 21920, 6724,  38656, 52180, 57712, 35572,
                6512,  31080, 20456, 24296, 59104, 10536, 42552, 60744, 3900,
                56952, 29780, 17632, 25120, 43548, 59784, 46492, 23280, 12772,
                57128, 37328, 48876, 53076, 22080, 26816, 42296, 48112, 29012,
                1492,  6692,  59364, 12228, 28432, 56856, 38084, 25484, 18992,
                7692,  63064, 15504, 23852, 29396, 51396, 3324,  61360, 20144,
                30116, 48584, 9884,  43652, 15024, 49896, 34056, 64692, 16952,
                30768, 60764, 11196, 5372,  55712, 20240, 35420, 51112, 6516,
                29424, 22904, 31768, 7432,  40660, 3564,  29604, 52664, 38516,
                15824, 11504, 51104, 19996, 24928, 5760,  16532, 44656, 10372,
                40292, 19160,
            },
            {
                35116, 43788, 15748, 54808, 416,   34704, 23644, 13380, 43820,
                6312,  36140, 9444,  33492, 42196, 49908, 12308, 32684, 41420,
                27080, 53592, 9488,  49524, 55380, 6972,  17304, 10380, 59608,
                53584, 24392, 11228, 17268, 26232, 9056,  32912, 21592, 38056,
                58700, 2612,  56244, 40952, 27812, 59928, 3224,  44852, 18900,
                50572, 43252, 53724, 620,   39784, 18236, 25576, 36756, 48508,
                44412, 1100,  39432, 62720, 10936, 5180,  17132, 54284, 2684,
                44600, 20680, 15956, 64416, 668,   56176, 32556, 14724, 51196,
                39696, 23188, 46576, 36316, 20644, 56,    51552, 43600, 55312,
                3932,  45376, 34220, 56700, 8348,  44684, 11156, 41496, 52908,
                7548,  63512, 35292, 27880, 55164, 4912,  11416, 52856, 2592,
                8100,  37648, 51500, 26868, 41032, 2916,  46176, 25664, 10600,
                64412, 2808,  49068, 61552, 24040, 59272, 42852, 21260, 62844,
                852,   60088, 6852,  33236, 54316, 39652, 59512, 21368, 7136,
                56188, 48760,
            },
            {
                12152, 57732, 7348,  27928, 31320, 59428, 52328, 38584, 61592,
                18080, 46712, 62892, 15212, 4624,  19016, 37060, 58060, 16780,
                5616,  20496, 45808, 24224, 42304, 33200, 38368, 43840, 29984,
                5752,  40492, 63380, 54912, 42964, 36524, 628,   15632, 42124,
                17984, 31604, 49008, 9812,  46412, 13484, 32120, 22412, 26680,
                9888,  61496, 15424, 34900, 64044, 31732, 52668, 7904,  12272,
                63652, 34660, 14628, 55508, 21584, 37540, 27868, 65148, 33456,
                25604, 5776,  41648, 34604, 12740, 46764, 7572,  19128, 57920,
                34168, 9456,  17464, 52504, 64688, 47700, 13652, 22592, 30368,
                50048, 12496, 39532, 748,   18560, 63860, 37560, 22416, 33540,
                45124, 13960, 23708, 1788,  20984, 37740, 42100, 22572, 28208,
                47988, 42828, 15292, 22092, 61720, 30132, 16292, 57880, 41396,
                52600, 37628, 18456, 54760, 15236, 50420, 12632, 8312,  48408,
                30600, 26920, 37160, 57124, 9496,  28768, 50648, 31452, 37864,
                25672, 61448,
            },
            {
                32100, 21652, 50396, 46212, 11056, 42244, 4232,  20896, 27216,
                2132,  30732, 55260, 21612, 28180, 64948, 45236, 2448,  50816,
                34692, 63720, 29812, 59884, 2932,  13432, 51256, 64680, 180,
                18892, 49392, 31948, 4012,  22268, 56872, 50288, 64568, 27056,
                52456, 4604,  24180, 63252, 19636, 36192, 55356, 64140, 41528,
                2100,  37764, 28392, 46892, 13192, 3272,  56104, 22328, 16536,
                28040, 51652, 8648,  47016, 31192, 58088, 50032, 41040, 9212,
                48240, 59532, 51672, 28024, 60708, 23612, 37276, 63836, 25804,
                44520, 62072, 29776, 40740, 5692,  33468, 8820,  61696, 16556,
                35968, 58908, 22060, 52512, 31928, 27556, 49532, 16212, 4876,
                56060, 40136, 59300, 52196, 46612, 63088, 31720, 59704, 55820,
                19008, 63692, 1596,  34940, 53360, 9436,  48592, 32632, 21236,
                14408, 45492, 9100,  33848, 1152,  26728, 36200, 56156, 18224,
                40616, 45628, 22004, 17004, 43156, 12880, 468,   62852, 47008,
                17448, 1812,
            },
            {
                40944, 65364, 36408, 19260, 25304, 63352, 15416, 56560, 33820,
                50992, 12912, 43124, 39188, 51968, 8712,  30520, 24964, 55136,
                14108, 40240, 4728,  36448, 16492, 56160, 21740, 25784, 35244,
                57680, 15016, 37824, 28440, 47204, 13520, 30324, 5704,  11420,
                44572, 61060, 37608, 29260, 684,   50228, 5908,  11728, 47940,
                17468, 59456, 5676,  21524, 57872, 42016, 29492, 45896, 59800,
                40444, 19972, 25652, 42448, 6504,  13472, 380,   19480, 15108,
                38668, 30676, 3384,  18268, 9300,  44020, 53712, 2240,  6392,
                12988, 55632, 3488,  15184, 26756, 59672, 37464, 41824, 6004,
                28060, 64852, 9880,  46804, 14116, 57560, 2832,  60268, 25992,
                31460, 18368, 5916,  29436, 9532,  16824, 6612,  13852, 36144,
                10100, 25440, 45668, 58736, 13448, 38728, 4916,  62752, 1384,
                28724, 25260, 63264, 40072, 30692, 44596, 64644, 4900,  32792,
                13712, 62144, 2756,  48756, 64488, 34536, 23752, 15060, 5332,
                52336, 28248,
            },
            {
                55544, 3152,  14212, 5756,  51564, 37584, 7972,  47820, 40320,
                9848,  59516, 24756, 6180,  57052, 47516, 11672, 21084, 43204,
                58648, 9816,  19228, 48328, 62080, 31240, 7860,  47492, 12124,
                41480, 51800, 6856,  10408, 61944, 19600, 40636, 59408, 23176,
                34084, 8652,  14284, 57756, 43556, 16524, 25588, 39044, 30308,
                54748, 33496, 51744, 26144, 8848,  49600, 35656, 6420,  1612,
                32976, 57320, 3816,  53296, 60732, 34568, 45508, 23988, 56388,
                62228, 12024, 22568, 55192, 40076, 31552, 16344, 28620, 41604,
                48668, 22244, 32720, 53620, 45256, 18428, 24372, 57324, 48724,
                2348,  19076, 25792, 41100, 6492,  43024, 35340, 9696,  46956,
                65228, 11904, 50036, 42580, 33972, 57328, 26456, 50340, 40680,
                4412,  52344, 32384, 7080,  27916, 19816, 44244, 53932, 36672,
                59352, 6340,  47580, 20088, 58252, 10004, 22400, 53428, 25004,
                58120, 10476, 52176, 6120,  26424, 53776, 45752, 58408, 36728,
                43400, 10136,
            },
            {
                48016, 23876, 60336, 44452, 32704, 58536, 29732, 22984, 3740,
                64580, 19212, 36832, 1728,  17300, 34560, 60144, 38008, 6756,
                32028, 27288, 52436, 24884, 11024, 39260, 44764, 59224, 4124,
                20572, 26992, 59824, 44016, 53420, 2628,  35460, 46004, 54004,
                18572, 49904, 40300, 22172, 32672, 62536, 52068, 58300, 7912,
                23364, 44932, 11940, 40200, 62548, 15220, 18880, 24656, 64584,
                47608, 11048, 37232, 16792, 22140, 29228, 63788, 51328, 26960,
                6848,  44872, 35844, 58860, 47484, 4828,  61484, 50708, 35136,
                18564, 38072, 58448, 11144, 49556, 1816,  31788, 11828, 44676,
                53728, 38308, 33672, 54920, 61392, 23832, 19684, 53920, 39396,
                76,    36456, 24692, 61996, 20044, 38880, 1076,  44844, 30392,
                20472, 62068, 16508, 41928, 56728, 49832, 24684, 17200, 11648,
                42816, 55900, 13384, 3840,  50860, 16644, 37988, 46872, 1556,
                42524, 35636, 29784, 39004, 18360, 41132, 8492,  20652, 30396,
                62112, 18832,
            },
            {
                33736, 38752, 27120, 53852, 2440,  17888, 11620, 43544, 54224,
                26508, 45812, 50008, 32364, 62968, 26912, 14516, 540,   49628,
                61704, 45504, 3180,  41860, 54444, 1572,  27452, 16880, 63440,
                54744, 34124, 23548, 16720, 32320, 25600, 8316,  15188, 27972,
                1224,  56316, 6108,  46728, 10268, 4424,  20108, 35060, 14656,
                3204,  65236, 19496, 1040,  31184, 55124, 38600, 53192, 42676,
                14316, 30272, 62204, 48708, 40192, 11636, 8192,  37016, 2400,
                32428, 49484, 15876, 1184,  25292, 13200, 21504, 56704, 9964,
                65372, 580,   25052, 42248, 7868,  64080, 55340, 21732, 15216,
                30112, 8668,  49372, 3688,  16576, 30900, 50884, 13208, 28508,
                21452, 55140, 45576, 3388,  12888, 53144, 23356, 64844, 54824,
                11980, 47240, 38100, 22744, 328,   65040, 34144, 8180,  30936,
                50268, 22880, 35028, 29668, 61840, 41912, 6992,  28788, 19356,
                49816, 15976, 23284, 61308, 32496, 12044, 2944,  55060, 26176,
                12816, 6500,
            },
            {
                52536, 16256, 9076,  41344, 21508, 62288, 49696, 35328, 15992,
                31180, 13280, 7696,  21972, 52936, 44780, 39968, 54496, 17756,
                22620, 35788, 15240, 65128, 20064, 33880, 50908, 37420, 30668,
                9092,  46292, 1188,  65376, 39840, 50728, 57784, 62440, 39084,
                31488, 64892, 26128, 35880, 55036, 28292, 40984, 45664, 60672,
                29384, 42156, 35944, 48320, 60344, 4860,  10744, 28252, 7436,
                22616, 55456, 2520,  26248, 5268,  59280, 43796, 17744, 54364,
                41716, 20400, 64840, 29320, 52776, 34044, 39304, 6984,  26620,
                46476, 15432, 60432, 29940, 20312, 39240, 35416, 5648,  62308,
                40500, 59576, 22764, 12120, 64100, 37880, 2300,  58920, 43948,
                61024, 16648, 8784,  28052, 58552, 47724, 15756, 5488,  34904,
                28444, 3176,  60204, 9896,  29852, 12976, 54944, 40540, 62056,
                2456,  18136, 39480, 53576, 25864, 12168, 55552, 34596, 63664,
                59120, 8004,  54428, 3844,  57256, 47208, 63164, 35228, 49516,
                40092, 59260,
            },
            {
                4472,  30964, 63616, 13136, 46296, 28388, 6768,  56936, 284,
                58980, 38924, 61168, 42096, 10892, 4008,  24028, 63940, 28904,
                11440, 56600, 7948,  29840, 58316, 13056, 5896,  22548, 43080,
                13912, 50000, 36516, 5464,  12660, 21588, 47428, 5020,  11788,
                43704, 20568, 16272, 60252, 13112, 49388, 1664,  24628, 11068,
                50408, 54248, 16280, 27336, 23108, 34412, 46472, 58948, 50596,
                36260, 18656, 45132, 32500, 50788, 20252, 56164, 30172, 14072,
                60140, 7924,  37956, 10604, 42928, 62824, 18868, 44712, 31868,
                52552, 5228,  34824, 51068, 13068, 27064, 46232, 51444, 25288,
                1004,  17848, 56120, 28184, 43356, 47908, 25548, 7960,  34008,
                4224,  31804, 51560, 41468, 36044, 32312, 9720,  42920, 18240,
                57188, 24944, 51424, 44984, 37068, 48096, 3880,  21204, 46348,
                27420, 60384, 9380,  45144, 676,   21620, 49088, 14644, 3260,
                26372, 39608, 44292, 14016, 28036, 22012, 42772, 15796, 880,
                22692, 44736,
            },
            {
                20060, 56748, 35024, 1268,  55228, 37888, 24836, 40544, 20244,
                51452, 4820,  28196, 55196, 16132, 30976, 8524,  47148, 34120,
                4572,  40776, 48944, 37708, 24212, 46996, 61416, 53140, 2804,
                58444, 19892, 27884, 57064, 42544, 30300, 17852, 36828, 24292,
                52320, 48084, 2848,  41872, 30828, 18636, 63776, 57000, 38532,
                21648, 4088,  8772,  57304, 12916, 43756, 20908, 16504, 132,
                61544, 41704, 9236,  65332, 14940, 35600, 23668, 47816, 3792,
                26220, 52024, 23272, 56796, 5516,  48192, 2156,  59564, 11916,
                38512, 22496, 56192, 43872, 2704,  61304, 16892, 10528, 42608,
                34032, 46624, 37008, 6284,  32616, 10120, 53492, 18640, 49276,
                39672, 13572, 64184, 18740, 6936,  62556, 26152, 49936, 61020,
                40276, 7968,  32776, 15964, 19492, 58432, 25740, 14832, 36468,
                6800,  56924, 15568, 32432, 65368, 36800, 57356, 30508, 45352,
                11476, 32824, 20292, 51024, 37432, 5188,  52252, 31684, 8984,
                64604, 28484,
            },
            {
                11052, 46800, 23628, 50120, 17000, 32344, 10620, 63796, 47436,
                23940, 33436, 18580, 46600, 35696, 58044, 50680, 19300, 43164,
                59692, 52112, 16724, 756,   43376, 9512,  18188, 40736, 25328,
                62820, 39068, 15752, 53496, 9680,  60264, 1476,  54892, 32940,
                9196,  59256, 6868,  38028, 53096, 8372,  34492, 15764, 5980,
                32816, 45940, 63080, 37388, 52392, 3080,  64272, 40252, 33164,
                24452, 12856, 28776, 54120, 38932, 1132,  10312, 63572, 40612,
                34896, 45344, 17372, 32736, 14368, 28152, 24364, 54848, 16412,
                28756, 63336, 9500,  18092, 32452, 58232, 4440,  54332, 29216,
                65212, 12964, 51896, 20692, 60768, 15248, 41056, 63284, 22524,
                56768, 26700, 47292, 23500, 3028,  55184, 21016, 37012, 740,
                13624, 21860, 54344, 62992, 5592,  42492, 52076, 64092, 32752,
                41864, 51376, 24548, 48636, 5540,  18820, 8972,  40980, 23068,
                62196, 48200, 44,    65072, 10044, 58820, 18208, 25692, 55664,
                36712, 51236,
            },
            {
                3432,  39408, 58876, 7688,  42352, 60572, 2900,  15120, 43248,
                8456,  12464, 65472, 2408,  25188, 61456, 37984, 1344,  14112,
                24808, 31816, 20968, 62180, 27384, 56536, 36036, 29912, 11892,
                33368, 6404,  47648, 23236, 34980, 44952, 26764, 63608, 40804,
                15088, 29120, 23632, 61808, 21176, 43896, 25816, 48220, 54736,
                27876, 18548, 41248, 24140, 31952, 7660,  49152, 29372, 56780,
                6456,  46768, 58528, 4772,  25460, 43312, 57856, 32096, 6020,
                12336, 62680, 48,    49808, 60504, 41240, 36404, 7788,  49120,
                42068, 1244,  47320, 24864, 40448, 48692, 37252, 23348, 19796,
                7820,  57060, 2624,  44280, 24088, 55736, 1360,  29504, 37020,
                5960,  10492, 59604, 31004, 38160, 45340, 11292, 52816, 30776,
                48456, 44216, 35596, 1700,  27708, 31448, 7620,  11264, 55388,
                1364,  20476, 10800, 38164, 43680, 27316, 61212, 51320, 4620,
                55088, 17536, 35496, 25016, 30000, 45692, 40632, 61252, 13236,
                42500, 17492,
            },
            {
                62064, 26696, 13784, 30136, 19468, 51960, 27548, 34680, 53780,
                60000, 37344, 49500, 41228, 7064,  21416, 11812, 53308, 27992,
                64472, 6436,  10888, 54264, 33044, 3504,  49868, 7312,  45424,
                55608, 296,   29300, 64220, 6616,  14332, 51072, 4336,  19728,
                57572, 45364, 35200, 50052, 11452, 8,     57916, 40116, 12704,
                61440, 51756, 864,   14500, 59520, 19584, 26428, 10140, 15304,
                51340, 37604, 21560, 16540, 48560, 52652, 18952, 27860, 50968,
                21792, 55564, 29536, 38736, 19360, 10320, 64724, 21456, 33268,
                57772, 20080, 36176, 53908, 14708, 7200,  30556, 61872, 49604,
                15100, 31708, 26976, 39484, 5008,  34720, 46080, 12316, 16928,
                44240, 54008, 884,   42528, 14484, 57788, 27684, 17384, 4940,
                64276, 25656, 10008, 59228, 48936, 38552, 23324, 18624, 47588,
                28204, 63144, 30760, 60028, 54188, 2388,  33768, 15864, 38312,
                28500, 7680,  41788, 56364, 15268, 6548,  34324, 1924,  47840,
                7220,  33048,
            },
            {
                54152, 44356, 36272, 65084, 4716,  45616, 56624, 21896, 5336,
                30184, 25752, 56436, 15256, 29688, 45632, 33264, 40308, 57148,
                47780, 36224, 39720, 44756, 13320, 22272, 64672, 16484, 60040,
                20920, 43240, 51568, 18116, 40152, 31752, 22212, 48740, 37396,
                12396, 2192,  54212, 16812, 28068, 64620, 31456, 4432,  22428,
                36172, 10344, 29596, 47532, 55732, 37940, 43320, 54676, 34944,
                60916, 3628,  30916, 63100, 33716, 7848,  13276, 37852, 59200,
                16220, 42400, 8924,  24664, 53392, 2996,  44408, 51212, 4784,
                13536, 61532, 6268,  27788, 64564, 11668, 44060, 424,   41104,
                35380, 59628, 47524, 17028, 63964, 50380, 25948, 61912, 32264,
                51224, 21224, 34644, 17836, 50548, 8172,  61472, 34336, 42008,
                56772, 15744, 39728, 20504, 12664, 56180, 60684, 43576, 35256,
                14144, 40504, 6264,  17664, 13180, 23884, 47712, 10168, 20460,
                59580, 52272, 12700, 60972, 49144, 23592, 54068, 20272, 29088,
                58384, 23172,
            },
            {
                324,   20700, 10116, 49380, 25388, 37592, 12720, 40208, 17648,
                46280, 1380,  20424, 9940,  54296, 63452, 3172,  17968, 8672,
                22432, 1912,  51228, 17792, 58480, 47228, 38740, 26600, 52472,
                13564, 24644, 36248, 58836, 11260, 55668, 60796, 8184,  43764,
                26400, 62996, 32076, 5936,  47512, 37632, 14772, 45460, 59808,
                7520,  43568, 64832, 34160, 3872,  12076, 62372, 1448,  22912,
                42364, 18872, 45428, 11152, 40936, 60440, 23184, 45084, 1796,
                48728, 35948, 4732,  46564, 59060, 34272, 16092, 30076, 26460,
                39660, 45316, 31236, 50324, 22100, 55840, 18540, 25832, 52848,
                10384, 21360, 6812,  54396, 11060, 19956, 8072,  57956, 3684,
                40100, 60696, 28284, 65400, 24512, 39548, 2096,  47408, 23120,
                7504,  29360, 52308, 45180, 34020, 16784, 2236,  50812, 4632,
                24276, 57876, 45688, 50272, 36048, 58676, 42576, 63980, 31396,
                44504, 2296,  21952, 32108, 3988,  38560, 62732, 9284,  37272,
                14604, 50148,
            },
            {
                31892, 6000,  55696, 16408, 32936, 1148,  63028, 9000,  60976,
                51580, 32468, 59008, 43208, 35508, 50152, 26016, 60228, 43976,
                14920, 28792, 63140, 25508, 4272,  30700, 10160, 1952,  34148,
                41044, 8988,  4248,  48376, 26052, 1948,  35660, 29640, 17340,
                51792, 22864, 9692,  42716, 20280, 55316, 24764, 52544, 18232,
                26548, 50640, 16596, 21480, 25840, 46112, 18060, 31216, 48236,
                9088,  27688, 53476, 616,   26020, 55428, 6528,  29672, 64980,
                10956, 26576, 61812, 31412, 12684, 22188, 56480, 63104, 10732,
                54500, 17628, 2060,  9788,  37772, 46672, 32296, 58656, 5472,
                45384, 62940, 37340, 29132, 33220, 42624, 38416, 48056, 23804,
                14608, 9536,  4460,  45992, 12432, 30412, 20220, 53632, 13028,
                37548, 60800, 2824,  6416,  65244, 26560, 9316,  30532, 62092,
                54452, 9712,  32844, 440,   21156, 29328, 3572,  54440, 14216,
                36724, 26288, 47220, 40400, 18760, 11628, 51412, 25428, 43916,
                64392, 40568,
            },
            {
                27540, 46436, 61320, 41592, 58108, 23484, 48428, 28820, 42452,
                13748, 38196, 4072,  16920, 23420, 5628,  12760, 31092, 52380,
                55852, 32780, 42168, 7944,  35640, 55236, 42800, 18976, 61772,
                56840, 30528, 62856, 15560, 44628, 20336, 41952, 64548, 4964,
                56308, 39708, 58952, 36228, 61084, 2420,  8520,  41028, 33484,
                2800,  38364, 57616, 6776,  53148, 39704, 5512,  51956, 13716,
                64968, 38764, 57628, 14448, 36500, 47060, 16984, 40364, 33916,
                20912, 56992, 18416, 52100, 43340, 7216,  37076, 896,   47196,
                23704, 33872, 58136, 42732, 60616, 3536,  15660, 38952, 28864,
                13544, 24748, 2468,  51408, 59212, 14000, 248,   30100, 18392,
                56660, 48988, 37356, 55436, 6564,  58684, 43728, 63828, 32592,
                18756, 49800, 24148, 31576, 54784, 47100, 41688, 37268, 21816,
                15736, 38424, 26928, 64576, 52988, 8720,  39080, 25124, 5856,
                50116, 9432,  55956, 64500, 28084, 45496, 33256, 55268, 2332,
                18112, 11512,
            },
            {
                52736, 22260, 38188, 11860, 7372,  52248, 19056, 35048, 6208,
                22324, 64700, 48280, 27892, 62124, 38884, 47236, 20132, 36652,
                5128,  11272, 60900, 20544, 15208, 59900, 48812, 28128, 6624,
                45712, 21800, 38860, 52108, 33216, 54168, 9932,  14912, 47772,
                33632, 1068,  18488, 13956, 29656, 50100, 11764, 63612, 56804,
                48104, 30388, 13076, 42064, 63224, 28520, 35548, 60164, 24696,
                33892, 7044,  50040, 21752, 63544, 32008, 3020,  51296, 54608,
                6940,  41152, 14444, 2608,  39060, 27648, 49072, 19756, 41416,
                61172, 14756, 52276, 20228, 25112, 34672, 8616,  63784, 49968,
                56848, 40832, 16332, 44752, 22396, 26892, 64796, 53132, 34504,
                43228, 25756, 31536, 19044, 51532, 35392, 26136, 9820,  4748,
                56072, 44464, 11304, 40144, 21388, 13344, 18132, 53208, 7144,
                48536, 43780, 18844, 12688, 41176, 47256, 16580, 57924, 19716,
                61924, 33592, 16844, 736,   7832,  58644, 15556, 5656,  30320,
                57168, 35692,
            },
            {
                63384, 3860,  17532, 26664, 31128, 44384, 14916, 54236, 57792,
                26256, 11236, 53588, 33352, 8036,  57344, 41752, 204,   65060,
                24196, 49208, 38948, 46432, 52864, 23136, 12064, 37348, 51168,
                17012, 2512,  11688, 27932, 7000,  60068, 24468, 38224, 27264,
                12248, 45840, 51284, 25960, 43892, 34480, 22504, 28116, 15192,
                20860, 5196,  60600, 23276, 9172,  15572, 56344, 20552, 41548,
                1960,  17452, 30296, 42252, 5016,  19884, 60908, 13404, 24360,
                46196, 28796, 50364, 64360, 23856, 59508, 53644, 8812,  30980,
                4152,  36868, 7180,  28764, 12592, 54544, 47808, 23088, 1336,
                19924, 33556, 61428, 5384,  9460,  56184, 46332, 4712,  11592,
                61144, 2032,  63340, 13868, 40576, 500,   16512, 48136, 39340,
                27492, 15200, 35580, 62444, 168,   58348, 34320, 63280, 29136,
                1992,  56424, 60420, 4468,  23168, 61204, 34520, 30772, 45408,
                39760, 12420, 52980, 43176, 36576, 21372, 49748, 41484, 23836,
                48156, 7624,
            },
            {
                14208, 33520, 50320, 60320, 2548,  64432, 39008, 4428,  46820,
                31400, 820,   44616, 14424, 20972, 52708, 16380, 10016, 28100,
                43596, 18048, 2160,  27304, 6308,  31972, 508,   65504, 25076,
                33528, 55048, 47688, 63752, 18884, 536,   49424, 58200, 20608,
                62328, 30892, 6472,  65500, 3656,  58024, 47080, 39476, 532,
                43168, 52724, 36388, 49284, 33084, 46940, 4236,  10636, 50556,
                55400, 46560, 25776, 58620, 11524, 48464, 27636, 38680, 9580,
                62608, 36392, 5640,  33212, 11204, 17336, 34792, 13752, 57164,
                26308, 45996, 65336, 49452, 39940, 61376, 17052, 42288, 36184,
                10808, 55040, 29552, 49040, 39612, 19604, 35972, 16700, 40468,
                21596, 8212,  45476, 23292, 28968, 59868, 52728, 22628, 62812,
                2384,  59036, 51608, 8420,  48432, 27672, 4388,  45080, 12208,
                25204, 35716, 31532, 49620, 28132, 10660, 5216,  51688, 1920,
                22876, 28860, 59872, 24712, 31168, 62424, 12608, 37936, 61104,
                20068, 44492,
            },
            {
                29064, 42272, 54884, 46952, 36128, 24604, 10144, 21132, 60848,
                17104, 39900, 56752, 36780, 4684,  30252, 60004, 34772, 50856,
                58760, 13248, 56544, 34544, 62676, 41188, 45056, 14732, 8496,
                58720, 42464, 23312, 35740, 41076, 31920, 43860, 4100,  53356,
                8416,  42184, 55824, 21364, 38336, 17036, 6760,  54064, 61364,
                32444, 11360, 26332, 18620, 1324,  59624, 43500, 26828, 37144,
                13188, 62280, 8804,  35768, 54124, 43420, 32876, 57184, 17588,
                360,   44128, 20428, 55964, 47000, 1668,  44904, 63680, 21616,
                51100, 11516, 18096, 32512, 2856,  6060,  30832, 27052, 52716,
                4816,  44040, 25296, 13008, 62816, 31424, 6680,  52236, 28320,
                57964, 32952, 54340, 49336, 10352, 43096, 7592,  34116, 30260,
                42240, 19608, 24912, 30688, 16820, 42572, 22948, 50560, 57608,
                40340, 8484,  14664, 54876, 44792, 37372, 55944, 14868, 65300,
                8448,  48804, 3644,  19080, 46964, 4544,  55716, 27312, 2796,
                10680, 59052,
            },
            {
                25000, 692,   9404,  19708, 13668, 56068, 29880, 41936, 34232,
                51048, 8604,  23220, 61756, 49788, 26068, 45964, 22360, 6884,
                38124, 30660, 52008, 9124,  16916, 57620, 20992, 53412, 38440,
                28608, 4960,  13988, 8860,  55996, 16284, 11888, 28988, 34860,
                17484, 37136, 24640, 13336, 52036, 10436, 30824, 19428, 25132,
                8364,  45912, 65088, 54484, 29520, 39236, 22680, 64172, 30872,
                19432, 3892,  40348, 23336, 16112, 2012,  7756,  22096, 52488,
                59252, 31256, 14932, 26624, 60812, 40700, 28488, 5348,  38040,
                42108, 384,   59392, 43972, 21500, 57716, 45640, 13348, 64776,
                59148, 18248, 37580, 1976,  53840, 23828, 43852, 60224, 49988,
                12980, 36484, 3240,  17392, 37944, 61960, 20640, 13764, 53872,
                10952, 6600,  45592, 57080, 37496, 61596, 9952,  32032, 19124,
                64872, 22196, 42488, 1016,  17692, 62584, 21068, 25968, 43436,
                36396, 56788, 41600, 14452, 51940, 9328,  34508, 17572, 52940,
                32504, 39520,
            },
            {
                16108, 57416, 22988, 40220, 6136,  49016, 59116, 1996,  12944,
                63600, 28708, 18936, 42752, 11368, 1868,  40744, 13980, 62980,
                3216,  19748, 23784, 46052, 36976, 26388, 3372,  30932, 48480,
                60652, 20164, 51000, 61616, 26028, 46756, 64820, 22296, 50564,
                60984, 3116,  48948, 33356, 59664, 45388, 63480, 36052, 50300,
                58180, 16736, 37892, 5600,  13540, 51736, 16324, 7024,  57476,
                44508, 33512, 51148, 59728, 29356, 65432, 50488, 34764, 46652,
                25084, 10796, 39376, 49608, 8288,  19276, 53000, 32016, 16024,
                55880, 23816, 28136, 36108, 10304, 52028, 38116, 8096,  22424,
                32044, 46928, 8556,  58048, 34296, 15612, 10308, 1196,  25148,
                19964, 42508, 65056, 26984, 31668, 4644,  46552, 58172, 37292,
                49080, 64984, 33572, 1832,  13244, 52104, 6352,  38876, 15600,
                3696,  51860, 59996, 26644, 33340, 7764,  40204, 50352, 30348,
                18052, 10872, 32892, 64068, 39160, 25608, 42760, 63456, 46396,
                6592,  50864,
            },
            {
                34908, 45944, 63948, 32200, 27036, 43884, 16264, 53172, 26496,
                44912, 3576,  35148, 54252, 31960, 64424, 18280, 53512, 32840,
                48212, 41672, 62120, 4636,  12900, 50444, 63604, 10328, 17712,
                1568,  44584, 36876, 30736, 2412,  38908, 54012, 5448,  40472,
                14740, 44804, 9280,  27144, 1032,  40672, 23352, 4172,  13048,
                41108, 1844,  22144, 44052, 57036, 9484,  35020, 47680, 264,
                53636, 27576, 14220, 5832,  45844, 19692, 39216, 13612, 4368,
                42144, 64096, 54596, 3396,  36784, 24156, 62384, 10184, 48396,
                34040, 7852,  53416, 14416, 62184, 25412, 18796, 50388, 584,
                41116, 14684, 51336, 21000, 42188, 48168, 64348, 38112, 31020,
                47004, 7052,  56868, 14372, 50928, 55812, 25248, 6464,  28188,
                17676, 23564, 40280, 54348, 21088, 25872, 34552, 55700, 48620,
                29316, 45916, 36268, 12908, 54032, 47368, 3284,  12240, 59016,
                372,   53948, 26848, 5352,  20556, 58756, 1644,  29988, 13740,
                21212, 61856,
            },
            {
                28056, 11660, 5036,  50476, 60404, 35680, 8884,  22604, 38388,
                7292,  47892, 57972, 15224, 6112,  47056, 36928, 8156,  25208,
                57472, 11532, 28612, 54576, 32412, 42712, 23244, 39868, 34576,
                55280, 11268, 24248, 58036, 7448,  19184, 33536, 10224, 25396,
                57312, 30624, 64204, 18956, 56368, 15472, 29312, 55264, 48040,
                27428, 34136, 60496, 30708, 25728, 62788, 41440, 18508, 24768,
                11244, 63124, 21804, 36420, 55980, 10984, 26384, 62040, 30268,
                16992, 7148,  21760, 32732, 58104, 13492, 44260, 2068,  59796,
                20392, 63428, 45160, 39624, 1940,  33344, 56372, 28964, 61288,
                35464, 26304, 63268, 30256, 3292,  27780, 18948, 55352, 4348,
                61180, 11232, 23952, 39444, 824,   18860, 34572, 44784, 2144,
                60472, 14528, 29380, 4540,  46912, 63648, 44108, 720,   61088,
                24420, 10392, 5840,  20852, 58580, 29816, 24216, 63180, 34308,
                46660, 22448, 60584, 37044, 45744, 11988, 49112, 23536, 36644,
                55172, 2428,
            },
            {
                18272, 37616, 53860, 14652, 3156,  19048, 65308, 32712, 55724,
                18076, 60284, 24052, 40216, 27756, 21512, 56220, 29540, 44256,
                16276, 672,   39528, 60024, 7804,  15400, 53816, 5940,  46556,
                26524, 64420, 40808, 15740, 49512, 43744, 28148, 60444, 47684,
                1732,  21192, 38556, 51504, 35428, 5316,  43528, 9440,  62684,
                19192, 53292, 7808,  49032, 20092, 2560,  50496, 32340, 59156,
                38268, 42504, 3164,  49508, 31212, 41272, 1052,  57496, 52872,
                35868, 48152, 27952, 42900, 51304, 17580, 31012, 26820, 38832,
                12992, 5580,  30364, 17424, 47708, 5112,  42956, 15532, 48764,
                4344,  10876, 54312, 6980,  39428, 59216, 13172, 33784, 44424,
                16448, 53264, 35280, 48252, 62304, 41900, 10760, 63712, 38264,
                51060, 8768,  43144, 59528, 11160, 16356, 28300, 20040, 13616,
                41908, 31584, 63764, 38632, 50852, 15828, 44304, 19376, 38016,
                6896,  15152, 50224, 30608, 16768, 53536, 7644,  57224, 40688,
                9752,  43696,
            },
            {
                60780, 21984, 41804, 30408, 25228, 39920, 46288, 1328,  49764,
                13824, 31084, 10376, 404,   50468, 60668, 12148, 3780,  61676,
                51380, 34340, 22544, 18608, 48848, 28012, 59236, 20304, 61388,
                14384, 31736, 3812,  21688, 54572, 63320, 13412, 36772, 17556,
                54720, 43132, 11900, 7268,  46324, 60940, 21676, 39028, 32284,
                3620,  44880, 15048, 35792, 39908, 12368, 56296, 28324, 14972,
                8412,  47108, 17592, 61264, 7956,  16336, 44920, 23580, 9416,
                19860, 59712, 12584, 708,   63624, 6144,  47444, 57240, 52304,
                43068, 24780, 50316, 58532, 22248, 64388, 11712, 39300, 23820,
                58220, 43776, 17680, 46340, 23320, 51788, 8852,  49128, 25700,
                41012, 28668, 20844, 8600,  30336, 15844, 22828, 53116, 31520,
                20768, 56652, 35212, 50168, 31940, 39100, 58336, 7676,  37128,
                53244, 56460, 17860, 1312,  42224, 9384,  4788,  56556, 51712,
                28376, 43092, 9636,  3220,  63016, 33948, 27212, 18804, 65156,
                31476, 49828,
            },
            {
                7072,  47732, 58636, 9792,  56264, 51776, 12072, 29056, 41980,
                62632, 36792, 44164, 53484, 34408, 16752, 42876, 38468, 20452,
                9780,  46684, 65176, 36580, 3184,  45036, 33764, 912,   37716,
                8444,  52752, 47744, 34188, 9728,  88,    23604, 52140, 4888,
                33076, 28424, 62188, 24192, 31856, 17136, 52616, 13524, 57632,
                25332, 10828, 64556, 55012, 4648,  46272, 22788, 6240,  64760,
                34172, 51836, 24136, 27980, 53156, 58996, 33140, 49044, 37844,
                3928,  45356, 54972, 39144, 25312, 20840, 36256, 9016,  16600,
                34776, 3544,  55516, 9352,  36716, 27308, 53652, 7652,  30988,
                20624, 37896, 33072, 14140, 61880, 35316, 176,   22036, 63076,
                5236,  57760, 2596,  59740, 44100, 55276, 27512, 7112,  12804,
                46788, 26252, 5920,  18808, 23996, 2860,  52464, 47620, 33312,
                3836,  23100, 49092, 27112, 34804, 60924, 32212, 25808, 11584,
                64344, 39916, 58112, 24724, 43996, 39348, 724,   45304, 14872,
                4420,  25636,
            },
            {
                13376, 33196, 92,    16980, 36856, 23004, 61280, 5588,  21288,
                26688, 4240,  19556, 64788, 25384, 7424,  48716, 32368, 24564,
                55568, 5708,  26356, 13620, 57208, 10628, 23648, 41676, 50204,
                29108, 19032, 42988, 60012, 38140, 30084, 41852, 46008, 8636,
                58556, 14564, 49096, 2652,  57196, 26936, 47364, 240,   36700,
                50632, 41784, 22340, 29268, 17184, 62000, 37528, 43032, 53800,
                20800, 1720,  40080, 12300, 35812, 20960, 6192,  13796, 27472,
                64740, 31276, 15664, 34068, 10560, 41652, 61776, 2680,  22560,
                65132, 28876, 41024, 14172, 32460, 46064, 18596, 60008, 50516,
                3076,  64180, 55216, 5028,  27260, 42780, 56792, 31644, 14560,
                38252, 50588, 32924, 13400, 37120, 5160,  48596, 58976, 39716,
                16888, 272,   64572, 41596, 60840, 9832,  42904, 25456, 65484,
                16200, 8872,  44648, 62660, 13880, 21412, 54932, 46512, 17400,
                1748,  32856, 19516, 54612, 13608, 21492, 60996, 29572, 51880,
                35352, 54724,
            },
            {
                63932, 28456, 45392, 62568, 48796, 7472,  34052, 16156, 53740,
                57272, 8856,  46544, 12696, 30064, 56604, 2016,  63640, 14308,
                40996, 29760, 53272, 43100, 50732, 31444, 63500, 16192, 58328,
                11936, 56240, 6424,  26760, 16676, 55796, 19808, 62472, 26320,
                39792, 18708, 36384, 42308, 10748, 34888, 7980,  63728, 30016,
                18480, 58668, 6684,  33412, 51920, 25916, 788,   30656, 10780,
                45468, 57644, 31848, 63632, 3724,  43200, 61688, 51140, 40852,
                53452, 23140, 7700,  60720, 48884, 56376, 29448, 53660, 44948,
                11428, 48732, 59464, 20872, 6152,  62012, 2272,  34640, 44536,
                12808, 24876, 9452,  47956, 19908, 37488, 11192, 52952, 45240,
                19344, 9652,  46344, 25008, 64624, 20048, 2316,  34732, 62360,
                43772, 52740, 28980, 36952, 14944, 55504, 20904, 29588, 12192,
                59988, 37820, 30572, 6228,  52356, 39600, 2780,  36616, 59040,
                22856, 49028, 5116,  35752, 51064, 6708,  47860, 10968, 59172,
                22452, 39944,
            },
            {
                18160, 36216, 21076, 4004,  26352, 42536, 58188, 45140, 38688,
                31652, 50304, 40304, 59384, 21696, 37596, 52092, 18824, 45860,
                59976, 1212,  17568, 8536,  38392, 19572, 4600,  46796, 25152,
                35632, 2308,  22888, 65288, 12676, 49172, 3964,  35112, 11540,
                50920, 64896, 6028,  53372, 20232, 61708, 41304, 23712, 15640,
                45784, 2924,  39248, 60920, 8996,  48392, 13760, 59696, 17856,
                26900, 5244,  15712, 54820, 47280, 25468, 30080, 18040, 11212,
                2720,  19248, 43668, 27072, 1676,  18004, 13836, 32916, 38400,
                25924, 18352, 20,    37720, 52840, 41836, 25860, 15804, 56892,
                28676, 41252, 52088, 31156, 59920, 3612,  17248, 65352, 6676,
                27404, 61652, 53956, 17008, 40384, 51684, 29212, 23932, 8920,
                32204, 21744, 11736, 48928, 7224,  33956, 50536, 904,   40268,
                46548, 19780, 57048, 24620, 10912, 43432, 15904, 29976, 8632,
                42476, 61780, 27888, 9924,  56684, 25940, 37380, 17164, 42596,
                3212,  8504,
            },
            {
                51576, 41504, 11996, 55112, 31120, 13288, 19504, 2868,  10856,
                24008, 1020,  34788, 15408, 4752,  42056, 10920, 27360, 7996,
                36208, 31728, 62904, 22288, 58880, 27908, 52400, 7664,  39596,
                61796, 44868, 33040, 47132, 39952, 8024,  31636, 57588, 22376,
                1000,  30920, 23808, 45004, 28748, 13124, 55788, 4744,  48980,
                54056, 27480, 12500, 44364, 20740, 56508, 40436, 34984, 50676,
                62296, 36884, 42164, 20004, 10252, 38372, 24,    57292, 34928,
                62560, 47596, 58392, 36984, 51524, 23928, 46228, 8048,  4200,
                61116, 54268, 31372, 46696, 23476, 11348, 49192, 8032,  37336,
                19028, 776,   62724, 15180, 44720, 26100, 49416, 29876, 41328,
                35056, 22860, 1144,  30856, 7552,  11324, 46128, 54436, 14192,
                57140, 3856,  59264, 45288, 26444, 63036, 17440, 57724, 5988,
                53976, 34464, 1736,  48512, 59688, 27644, 65012, 50076, 53880,
                13064, 38300, 46320, 15440, 31328, 64664, 2188,  33036, 55684,
                24960, 47048,
            },
            {
                57284, 60468, 6400,  49704, 39684, 64880, 52032, 29432, 59588,
                63248, 18440, 53164, 28332, 61936, 47468, 33444, 58712, 23288,
                50028, 12692, 40296, 48000, 2476,  34580, 14148, 55348, 30592,
                20744, 14936, 53720, 3300,  28596, 52448, 60772, 14392, 42688,
                46640, 55244, 15912, 59488, 1856,  51092, 37876, 31500, 9804,
                34276, 64164, 24240, 36552, 16052, 31796, 3540,  24396, 8280,
                21840, 48784, 28832, 6988,  51592, 59668, 14520, 45044, 24152,
                5436,  39316, 32220, 12708, 5736,  63020, 40560, 58124, 50292,
                15116, 42612, 9188,  63376, 16972, 57824, 30524, 64904, 53152,
                46496, 32836, 22784, 7132,  39936, 57612, 22336, 1824,  55496,
                12452, 48548, 43196, 56152, 36100, 63084, 42172, 17908, 38548,
                49984, 35404, 18400, 39568, 2352,  30728, 43444, 36604, 23652,
                27436, 13364, 41772, 16884, 32988, 4132,  19720, 6104,  25060,
                32124, 40,    18516, 59556, 39992, 44476, 20328, 49664, 62624,
                15148, 30456,
            },
            {
                1908,  27292, 16440, 24204, 34784, 8576,  22532, 46380, 36148,
                13844, 48748, 43492, 6588,  24384, 54708, 2472,  16660, 42784,
                53936, 4268,  56500, 25772, 11104, 45680, 64456, 42388, 444,
                10440, 58912, 37048, 17624, 24324, 44088, 20532, 38444, 27220,
                7532,  37156, 9864,  33752, 40516, 17500, 21968, 59092, 43428,
                19472, 1688,  56960, 50252, 5284,  46368, 65520, 54272, 43540,
                2216,  58856, 12200, 33936, 64660, 22468, 31204, 49196, 55272,
                28908, 15916, 9180,  21768, 55444, 27912, 35348, 19416, 29916,
                22176, 35820, 4920,  27696, 34236, 1624,  39392, 21256, 4884,
                11008, 58584, 36668, 50540, 12984, 34332, 9808,  37992, 60688,
                16088, 6232,  59304, 25972, 14700, 21980, 58516, 1512,  27824,
                7412,  25764, 64188, 23152, 53340, 10612, 14136, 48736, 60624,
                9540,  64120, 51388, 22308, 38044, 54732, 47200, 35484, 58072,
                21276, 63808, 51972, 4700,  22976, 8368,  12596, 28120, 5668,
                38612, 10836,
            },
            {
                22024, 32788, 46596, 61668, 964,   56552, 17040, 5252,  41448,
                26164, 9036,  56916, 32508, 11836, 19788, 36956, 62524, 6688,
                28824, 20488, 33508, 16076, 60092, 37404, 23296, 17952, 48364,
                27280, 50668, 7396,  63800, 56028, 10360, 2224,  50448, 17900,
                63744, 48476, 25448, 61928, 4060,  46404, 27028, 7116,  61508,
                14276, 40724, 30360, 8608,  59584, 27328, 11136, 17320, 29804,
                14492, 38760, 55708, 26048, 44028, 4448,  36308, 8056,  19476,
                41268, 53176, 65516, 48528, 42300, 16228, 3248,  10608, 63864,
                47756, 56276, 13396, 60260, 50416, 44832, 13976, 26788, 42724,
                61016, 16340, 28044, 55252, 4392,  64084, 18380, 52404, 45764,
                32352, 20172, 39032, 3908,  50196, 33724, 5232,  31580, 61832,
                43532, 55084, 15004, 5068,  46540, 61324, 21320, 3736,  32852,
                18544, 44812, 7276,  29936, 61916, 9676,  44340, 15104, 39892,
                10972, 43160, 34304, 25708, 48416, 54380, 58796, 41920, 52564,
                34892, 44840,
            },
            {
                64600, 53304, 36900, 10388, 43180, 28172, 50904, 32972, 54764,
                2304,  38120, 21396, 65476, 40708, 51124, 30464, 45272, 13924,
                38492, 64812, 44080, 51812, 5396,  29032, 8688,  32644, 62452,
                39124, 21544, 42672, 32392, 13948, 36440, 62232, 32904, 6348,
                56616, 12796, 20752, 51944, 30544, 56404, 11804, 49312, 36776,
                23052, 47124, 52748, 18036, 38964, 22520, 41984, 51648, 36164,
                63132, 46120, 19024, 1300,  16236, 39984, 56996, 12084, 61224,
                4128,  25424, 34728, 592,   31008, 60544, 51748, 25540, 43912,
                1072,  38476, 24804, 40892, 20120, 7124,  54372, 35436, 48564,
                24400, 2728,  40704, 20100, 47496, 30740, 42328, 27608, 7748,
                24572, 51588, 64780, 28568, 10616, 46880, 52660, 19464, 12884,
                47716, 9596,  40908, 29168, 33652, 37392, 56132, 41224, 28712,
                53552, 38580, 3004,  57668, 12768, 26608, 1272,  60704, 29496,
                49824, 2972,  55860, 14252, 30328, 36416, 17716, 804,   23656,
                59852, 18904,
            },
            {
                40924, 4044,  14464, 20636, 58984, 38784, 11944, 64024, 19740,
                60428, 29704, 46720, 15684, 1112,  59656, 9368,  25092, 56860,
                47604, 10236, 728,   18932, 40556, 49836, 58000, 3344,  52764,
                12892, 5412,  29636, 47948, 4320,  26160, 46200, 53912, 29576,
                41428, 35360, 5260,  43800, 14956, 64372, 34880, 664,   54504,
                28784, 3936,  11496, 33980, 62868, 276,   32332, 57056, 4188,
                24792, 9528,  31608, 53604, 49916, 62164, 20848, 27588, 44724,
                51128, 13936, 59204, 18672, 46116, 6260,  39524, 54652, 33068,
                7628,  18324, 52584, 10048, 31860, 57248, 62656, 17488, 9268,
                31976, 56452, 44968, 8112,  24116, 59400, 2284,  14168, 61820,
                36788, 496,   44436, 17956, 41452, 62736, 23968, 39960, 57376,
                34760, 20856, 49640, 58468, 17140, 1220,  51216, 8460,  15160,
                59000, 24948, 49116, 16528, 34092, 41944, 23268, 51784, 7452,
                17844, 27424, 62880, 9592,  45524, 6328,  64076, 32052, 13828,
                47640, 7204,
            },
            {
                29128, 50176, 55296, 31468, 3016,  47432, 23332, 6876,  44012,
                14436, 51556, 4584,  35524, 27224, 48868, 17396, 3308,  34624,
                22224, 27252, 54448, 31820, 61336, 21876, 14880, 43064, 35412,
                25552, 55256, 61064, 18776, 57940, 40068, 21672, 15564, 356,
                23560, 60340, 28408, 39192, 8940,  19876, 24896, 42220, 16636,
                63236, 58312, 43768, 25404, 55072, 14036, 47760, 7568,  20016,
                49332, 60020, 41072, 6288,  28896, 9012,  47192, 33668, 1904,
                38092, 29872, 43244, 11200, 23664, 36736, 14688, 21060, 12296,
                57640, 28752, 65200, 48164, 2484,  23716, 29472, 3600,  51244,
                64492, 12628, 35712, 52144, 15624, 33488, 54404, 21868, 48316,
                57432, 11556, 31332, 56440, 8284,  35896, 15436, 6024,  29392,
                408,   24916, 6664,  62388, 11684, 27116, 23924, 65092, 43608,
                4976,  36016, 21120, 46240, 53228, 63272, 19304, 57068, 34932,
                47384, 37724, 22348, 41088, 57424, 20564, 39592, 50828, 26544,
                55804, 9908,
            },
            {
                17944, 24868, 44320, 63348, 26604, 16028, 35848, 49176, 31616,
                25176, 10412, 57984, 42756, 23144, 54976, 39048, 63724, 51936,
                6044,  58896, 36412, 13240, 7208,  26500, 46588, 56340, 9288,
                17072, 45516, 1044,  34700, 51252, 8040,  11872, 59284, 49956,
                44704, 17280, 3120,  57900, 53384, 45776, 5924,  51312, 32484,
                8468,  20344, 37508, 5596,  45720, 21316, 60884, 39616, 28124,
                11896, 34524, 22160, 57592, 35872, 13732, 24492, 54540, 17020,
                63444, 22136, 7012,  52460, 57904, 64296, 29652, 48836, 61952,
                45696, 34844, 15276, 42640, 36804, 13128, 45936, 41508, 37952,
                21780, 28512, 304,   61752, 38452, 10084, 44084, 5044,  40360,
                34024, 15768, 23368, 52888, 3568,  26768, 59772, 45048, 53896,
                64736, 38012, 51924, 31792, 44972, 40140, 48080, 18800, 32092,
                12144, 62840, 1652,  30816, 8008,  38820, 4536,  31600, 12068,
                61868, 4880,  15884, 51512, 1580,  28264, 10976, 61236, 3084,
                34088, 38232,
            },
            {
                62096, 12348, 5544,  39808, 9256,  52396, 60932, 640,   56708,
                40160, 18124, 63564, 32740, 12264, 7380,  44448, 31700, 19500,
                41732, 15848, 48920, 43604, 63344, 38524, 1504,  20028, 30092,
                65144, 40752, 23684, 14596, 28280, 64136, 42804, 31748, 36996,
                10248, 65396, 47556, 21756, 36684, 31056, 61284, 12828, 38928,
                48132, 27652, 15244, 52300, 29612, 10028, 35208, 16592, 64528,
                53316, 45108, 3672,  17776, 43412, 65020, 39092, 5324,  49020,
                10128, 55920, 35784, 47308, 27320, 1488,  9048,  41956, 4896,
                25876, 1884,  21788, 6520,  61524, 53064, 19340, 59832, 7388,
                15284, 48124, 57564, 18504, 26380, 49976, 63332, 29756, 18788,
                8792,  63900, 45868, 38776, 19744, 49912, 12256, 32372, 17760,
                10368, 42812, 13872, 19520, 2908,  57764, 5784,  35288, 55212,
                27772, 51536, 41120, 56408, 11240, 28256, 15724, 45864, 42444,
                24852, 53996, 31096, 60188, 33868, 49396, 16468, 43648, 21880,
                46180, 58472,
            },
            {
                208,   48088, 35204, 58040, 19100, 42744, 29272, 21056, 8540,
                36636, 52644, 6200,  47676, 20008, 61408, 14616, 28224, 10928,
                61044, 2480,  23044, 29712, 10776, 53136, 33300, 59572, 37676,
                6296,  50372, 9800,  54156, 38032, 2532,  24900, 18336, 54792,
                7084,  25628, 33180, 11280, 15948, 2248,  26776, 58740, 22464,
                55828, 1356,  65068, 41172, 59328, 3188,  50940, 43572, 1256,
                31068, 14812, 61888, 25788, 50720, 392,   30972, 58236, 20188,
                42052, 32324, 3548,  16404, 40016, 19792, 55320, 32144, 17364,
                53572, 40328, 59100, 49496, 27248, 33136, 10704, 26012, 54508,
                34000, 43048, 5424,  31620, 41536, 3472,  13212, 24720, 55076,
                50836, 27176, 2076,  58900, 30040, 62216, 41708, 2220,  48604,
                22104, 59140, 28288, 54780, 36824, 22492, 14696, 61608, 8880,
                45296, 19452, 14576, 60372, 24624, 49360, 65268, 55432, 460,
                20136, 39268, 7908,  12504, 42556, 24260, 36276, 53820, 6096,
                14636, 30672,
            },
            {
                20348, 53516, 23640, 14848, 32608, 4396,  54584, 13504, 62416,
                45528, 26804, 2088,  29480, 37544, 50752, 108,   57316, 47140,
                38324, 33832, 55968, 4516,  18164, 49732, 24104, 12212, 48060,
                26836, 58156, 19784, 32160, 44996, 56740, 48492, 4500,  62132,
                29244, 52364, 39740, 56172, 63000, 48752, 41524, 18448, 4280,
                44068, 34260, 11976, 18204, 24424, 32036, 57156, 22436, 26436,
                7068,  48824, 38108, 8780,  55940, 12012, 22820, 45540, 14440,
                28088, 59936, 24584, 63160, 13092, 34316, 49784, 23760, 60172,
                36492, 11488, 30648, 55584, 16552, 4824,  39736, 47020, 2292,
                62376, 23860, 11296, 53708, 21208, 58744, 45580, 37364, 60192,
                6236,  42600, 35092, 13452, 7704,  22804, 37064, 56864, 26084,
                35528, 4092,  46852, 7596,  64052, 49384, 30152, 52700, 232,
                39212, 23064, 6380,  36768, 44376, 2820,  21892, 35648, 9628,
                58624, 48544, 27152, 64292, 18972, 4652,  58932, 9132,  65388,
                25592, 42208,
            },
            {
                36860, 7772,  28924, 50736, 64920, 46888, 24436, 41080, 33380,
                16368, 22636, 59964, 41816, 56096, 24840, 35376, 22048, 6728,
                51300, 25568, 64996, 45924, 40096, 60580, 5632,  16416, 42460,
                3436,  35756, 12896, 62928, 5728,  16160, 21436, 35184, 41016,
                13672, 19764, 924,   43372, 24024, 8124,  35684, 54128, 10556,
                30644, 60612, 49560, 37140, 6088,  47356, 13532, 36612, 54412,
                41300, 58520, 21164, 32968, 28516, 40432, 60960, 34976, 6752,
                53216, 38236, 9388,  50936, 45052, 57096, 2688,  43740, 14240,
                6748,  47448, 19916, 8876,  44124, 63520, 22508, 58452, 30208,
                17292, 50392, 37556, 63944, 7712,  34868, 16876, 800,   31224,
                12088, 20732, 49240, 54672, 17236, 47272, 5012,  52572, 9072,
                63116, 40404, 16756, 33476, 25648, 10104, 42236, 17724, 26588,
                47464, 57796, 29236, 50776, 32724, 17344, 39860, 51596, 30088,
                13928, 34204, 44864, 2116,  56268, 46504, 28560, 32280, 39492,
                50060, 56880,
            },
            {
                10752, 62796, 44008, 2968,  11704, 37316, 7360,  57704, 3092,
                50216, 11784, 53624, 8980,  15444, 4120,  64316, 43628, 17324,
                30576, 12680, 20248, 8896,  28092, 35892, 55324, 31884, 63820,
                52360, 22736, 46592, 41244, 25556, 30248, 60992, 11176, 51188,
                46284, 59768, 5868,  30332, 50892, 14164, 28528, 63880, 45892,
                24600, 7900,  20704, 55180, 43084, 62292, 8808,  19436, 63368,
                11380, 17068, 2392,  47544, 53544, 18612, 3056,  49860, 63620,
                18008, 872,   42588, 20756, 5796,  29856, 10712, 62888, 28272,
                51384, 64540, 25268, 38936, 34684, 656,   51480, 12824, 36424,
                9104,  44668, 27388, 14460, 47168, 28836, 52688, 22940, 47864,
                39296, 61764, 25464, 32132, 40368, 65152, 28064, 15072, 20216,
                30340, 50436, 12636, 60220, 44528, 2268,  37920, 60736, 12004,
                34472, 64256, 10296, 4656,  62088, 12844, 59344, 6996,  25944,
                62784, 16904, 53200, 23092, 37828, 15012, 51632, 20868, 13032,
                4204,  17128,
            },
            {
                32984, 55132, 39556, 21712, 58872, 27476, 17884, 48244, 30312,
                64480, 39024, 34436, 19084, 45728, 30120, 40120, 9604,  54112,
                60064, 1880,  42796, 53460, 14840, 928,   21280, 45280, 9184,
                29368, 17608, 216,   59500, 8776,  53248, 44000, 1532,  23520,
                27324, 37804, 16456, 34312, 58208, 20796, 5000,  38500, 16516,
                52812, 40476, 27368, 2760,  15980, 29180, 39768, 4292,  49408,
                27844, 34916, 44336, 64020, 14032, 7640,  43552, 25952, 11256,
                30492, 47844, 57832, 26616, 61316, 37040, 22364, 39328, 17940,
                33428, 1564,  45380, 14508, 57332, 28684, 18836, 41472, 56108,
                3692,  20324, 59876, 1848,  56532, 40720, 10220, 64908, 55884,
                15984, 4064,  9756,  58096, 944,   44048, 11100, 33976, 45888,
                58028, 688,   23956, 52944, 19960, 56784, 31716, 21708, 55520,
                3508,  16388, 40988, 54100, 21052, 27236, 47264, 42668, 54804,
                3524,  40860, 5680,  31492, 10516, 63156, 41560, 1740,  61640,
                27560, 47312,
            },
            {
                19320, 1588,  26036, 15328, 45672, 33836, 52676, 9776,  20792,
                43256, 5568,  27528, 62248, 49460, 58356, 13324, 26580, 48412,
                34484, 38956, 57768, 31548, 48664, 62564, 26272, 40512, 14064,
                60748, 54964, 32908, 50520, 39264, 36012, 14996, 56332, 33584,
                7528,  62408, 54424, 44460, 11644, 61412, 47036, 32360, 164,
                58456, 13660, 64240, 33128, 50584, 59640, 24660, 45520, 31844,
                60648, 22920, 5608,  36892, 24824, 59880, 32520, 38620, 55476,
                21700, 35984, 14044, 33204, 17100, 54308, 48656, 8136,  55500,
                42048, 6432,  52868, 23116, 62168, 7844,  45788, 25368, 65172,
                33612, 52624, 39540, 24076, 32996, 19244, 4400,  43008, 34004,
                28228, 45484, 51468, 19440, 36280, 24316, 53832, 60368, 38828,
                7232,  43212, 36472, 29548, 4944,  15068, 50500, 6892,  43296,
                49000, 25256, 31016, 45400, 38240, 900,   34612, 10804, 22808,
                19208, 36336, 61368, 50784, 26420, 6556,  33964, 54924, 44516,
                36040, 59944,
            },
            {
                41820, 52232, 31756, 5264,  62192, 40848, 1372,  60956, 25088,
                14760, 55800, 768,   23700, 6908,  32784, 20668, 62948, 3580,
                22556, 15484, 7308,  23900, 37444, 11172, 58436, 51008, 2872,
                38660, 24432, 6580,  12380, 19232, 27940, 4588,  64916, 18536,
                47392, 10176, 21940, 26112, 3760,  40756, 55580, 23180, 49672,
                36116, 5968,  43988, 22728, 10708, 35620, 53472, 14568, 452,
                52172, 9552,  56488, 29892, 50844, 20520, 57296, 15908, 46132,
                5120,  52612, 62744, 2176,  11632, 41492, 3772,  59188, 26708,
                20948, 61244, 30108, 37312, 11652, 49236, 32172, 15372, 6048,
                47528, 29144, 11780, 6844,  63004, 51108, 26512, 13296, 7496,
                21836, 37264, 63708, 30620, 6668,  16572, 49572, 2584,  23164,
                18372, 54844, 10768, 61984, 40668, 46248, 65488, 28016, 36568,
                18688, 62696, 13340, 8188,  57144, 17872, 50912, 64520, 32164,
                49272, 57624, 12620, 43120, 19812, 47828, 16728, 22368, 9724,
                24144, 6736,
            },
            {
                29140, 64064, 11252, 49320, 19660, 55372, 13060, 37972, 32552,
                58968, 47424, 36152, 41360, 11352, 52048, 37288, 6280,  46044,
                28932, 61208, 51184, 44916, 4936,  18120, 33332, 7508,  19992,
                34956, 48568, 44272, 63188, 22388, 58496, 49648, 40416, 30888,
                53044, 42020, 1828,  50140, 36920, 18376, 7708,  27752, 11444,
                19896, 30036, 47636, 56756, 1776,  19804, 6532,  58260, 38648,
                16480, 42344, 47248, 12792, 1348,  41556, 9840,  3364,  65044,
                27856, 8660,  39220, 45232, 51636, 24924, 32128, 14992, 35832,
                11220, 48832, 17056, 2668,  55092, 40164, 3996,  60116, 22404,
                54632, 17376, 42440, 46012, 15488, 36232, 48300, 57216, 60852,
                53456, 2896,  14160, 42748, 55448, 62588, 29080, 13160, 32612,
                64496, 27616, 47764, 17060, 34652, 25064, 13084, 9648,  54196,
                1288,  33684, 52560, 22312, 61260, 29628, 6052,  14344, 44484,
                1508,  24880, 8388,  29192, 56476, 60080, 39404, 30864, 51900,
                58396, 13420,
            },
            {
                16424, 46036, 57368, 35080, 24388, 6784,  28640, 51096, 45244,
                8140,  18152, 29400, 54664, 16008, 44024, 57228, 18100, 39856,
                55156, 11572, 19512, 64260, 29800, 56588, 46904, 43088, 65324,
                27716, 15800, 56968, 30612, 2020,  45852, 10672, 24592, 5744,
                13808, 28372, 63532, 57744, 30444, 14804, 64656, 43324, 59360,
                52500, 61800, 17312, 37748, 41776, 63492, 28728, 44188, 21828,
                26796, 64728, 19664, 39072, 61852, 27508, 48468, 23588, 34360,
                42776, 18180, 22648, 29028, 59708, 19760, 47416, 65344, 44512,
                4520,  56608, 32992, 43804, 21220, 26812, 51172, 35332, 10256,
                37912, 116,   61568, 31264, 58404, 21332, 1560,  30376, 17388,
                39804, 49404, 26240, 10924, 20940, 39108, 47180, 9020,  41724,
                50944, 3356,  22152, 8116,  59468, 2396,  57500, 39460, 22732,
                59620, 42472, 4416,  48400, 35476, 41516, 55408, 26532, 59632,
                39464, 16428, 46848, 34240, 2180,  13628, 5572,  64800, 428,
                48992, 38136,
            },
            {
                3404,  22056, 8692,  38808, 44120, 16856, 64264, 21812, 2524,
                11972, 61692, 50612, 21496, 65480, 2004,  24860, 31392, 50012,
                832,   35480, 26752, 41576, 2660,  14572, 22660, 53784, 11876,
                59336, 3924,  9308,  37656, 42368, 53752, 17272, 35196, 59948,
                38320, 19644, 33960, 12400, 48068, 53468, 34744, 1944,  39180,
                24344, 3468,  8844,  26964, 13440, 48336, 32564, 11484, 55148,
                36028, 6912,  33324, 4616,  52532, 17044, 36356, 53804, 13776,
                60728, 49352, 56548, 6072,  36432, 9640,  212,   29544, 52000,
                39500, 24496, 8676,  58380, 13876, 64364, 18220, 43280, 56884,
                27852, 49468, 24888, 8628,  52732, 6476,  41264, 45000, 24488,
                9336,  32244, 59860, 45884, 34180, 3648,  25780, 58128, 35800,
                15620, 56064, 38020, 31304, 49448, 19732, 44652, 32192, 15972,
                46876, 29148, 11616, 25800, 15524, 2364,  20884, 37024, 7836,
                30400, 63552, 54160, 21520, 37696, 52860, 45188, 27988, 20096,
                42888, 33244,
            },
            {
                61968, 54016, 27904, 764,   60096, 31368, 53424, 35320, 42540,
                26712, 38712, 4496,  34036, 8732,  27592, 60556, 9824,  14512,
                43232, 58484, 8380,  32728, 49668, 37904, 26000, 608,   31136,
                41444, 21720, 51704, 25512, 13588, 32664, 61636, 3304,  48768,
                55168, 45248, 8332,  24820, 5256,  21656, 45620, 10108, 31712,
                15140, 46040, 33744, 54692, 51056, 23532, 7964,  62148, 2432,
                50876, 14080, 59176, 24416, 31324, 45596, 7300,  58928, 30316,
                10572, 2516,  32832, 14716, 62964, 43352, 57652, 22760, 13148,
                18812, 63220, 28844, 46960, 35884, 1480,  31064, 7584,  13152,
                63328, 19592, 34628, 14152, 38404, 28468, 64508, 12356, 35224,
                62460, 400,   16364, 56564, 6496,  51548, 61648, 18816, 44544,
                5944,  62464, 12644, 41348, 53628, 28316, 4548,  52096, 62324,
                6040,  19920, 38744, 64944, 58276, 44876, 10548, 48064, 51912,
                19696, 3984,  41404, 10420, 62048, 25192, 16116, 36212, 11564,
                56720, 25516,
            },
            {
                18028, 41200, 50400, 14720, 48228, 10580, 4760,  57948, 49876,
                19420, 56292, 14068, 45488, 40796, 48440, 36852, 53528, 21996,
                63300, 24444, 17092, 54464, 60356, 10568, 62420, 39816, 17408,
                47600, 35624, 55924, 18664, 64336, 7440,  27700, 40840, 23028,
                15676, 352,   61844, 42372, 37756, 60376, 26944, 18680, 50840,
                57548, 40992, 65256, 4864,  18944, 59472, 38332, 46160, 18420,
                30048, 47900, 42740, 55692, 11076, 63324, 21252, 932,   38484,
                25236, 41052, 46452, 53608, 26856, 17480, 38176, 7428,  54768,
                34184, 2960,  50156, 41424, 5852,  60896, 52884, 24256, 45724,
                39844, 3440,  59960, 43512, 56004, 18724, 4052,  51600, 47324,
                20124, 54020, 28880, 40372, 22896, 14712, 31740, 1200,  24044,
                30128, 46208, 25684, 16,    64144, 14948, 37424, 8836,  25376,
                35592, 56328, 51144, 8108,  32308, 53836, 23984, 62936, 33364,
                13176, 58204, 27740, 49564, 6932,  32408, 58852, 48188, 8472,
                51420, 4928,
            },
            {
                36584, 11776, 65280, 33940, 21072, 40932, 24744, 15616, 7316,
                32236, 63872, 24128, 30020, 58116, 3052,  17732, 33112, 4740,
                47388, 38900, 3296,  44496, 21064, 5720,  34620, 52324, 8476,
                63416, 5428,  29516, 49188, 1012,  46568, 20448, 57444, 9244,
                31484, 51476, 27584, 56760, 16564, 2648,  54948, 63092, 7032,
                28656, 21080, 12224, 36840, 31032, 1116,  15124, 41432, 25108,
                60608, 8696,  22164, 2888,  39412, 14924, 43520, 51012, 18628,
                55784, 64208, 21476, 3920,  11948, 50872, 32652, 47888, 42480,
                61136, 10300, 22068, 16668, 26064, 38588, 20664, 48520, 32900,
                6376,  53808, 22884, 10908, 46976, 26180, 33376, 58240, 23392,
                4924,  42632, 36120, 8620,  65284, 48304, 37564, 55052, 10032,
                52636, 20236, 59924, 35236, 10540, 23544, 57132, 41664, 48872,
                13724, 844,   43488, 17048, 27900, 4776,  39580, 17380, 12,
                46356, 35520, 23460, 15092, 43848, 19372, 3036,  40496, 22484,
                60448, 30180,
            },
            {
                46832, 6928,  26076, 58324, 3132,  45536, 62876, 28244, 39980,
                46760, 52,    53960, 11188, 20500, 63012, 13020, 42228, 56956,
                29204, 12292, 51824, 30992, 15672, 47996, 27828, 57020, 23872,
                43584, 15176, 11332, 38608, 59388, 34280, 52640, 12252, 43660,
                65224, 35864, 20364, 10740, 49992, 33408, 13332, 40264, 35284,
                704,   48628, 60776, 43924, 26328, 56492, 52528, 33532, 4276,
                37072, 53972, 17416, 34736, 29220, 57692, 26212, 33504, 47120,
                6560,  15860, 35836, 30200, 40420, 60180, 24716, 5288,  15584,
                26540, 37776, 52352, 59676, 12312, 54872, 8756,  15380, 58652,
                29068, 16872, 50256, 30812, 1860,  61864, 15788, 9004,  38644,
                14348, 49252, 59340, 18444, 27712, 43188, 12456, 60552, 40628,
                33644, 16476, 6796,  48276, 44004, 18256, 31428, 63128, 21116,
                29600, 60352, 33796, 22384, 49796, 12372, 61132, 29500, 56272,
                42268, 5604,  65444, 38920, 56844, 52044, 28580, 63844, 33812,
                13920, 43268,
            },
            {
                23400, 54512, 19000, 38376, 29888, 55080, 12948, 36888, 61028,
                9900,  17348, 35668, 49328, 6292,  37948, 26260, 50504, 7580,
                19456, 34828, 65080, 25196, 58216, 42468, 12744, 19312, 3624,
                33288, 60548, 26368, 42032, 23148, 15808, 30148, 37700, 3732,
                24308, 46844, 5804,  39688, 28996, 44588, 22916, 47672, 25136,
                53944, 9716,  16308, 22576, 7128,  47352, 9976,  20824, 62712,
                12516, 28004, 45292, 65436, 49264, 8964,  4580,  62612, 12340,
                28948, 52004, 44688, 8160,  56252, 20356, 1784,  64532, 30720,
                57780, 45832, 784,   29748, 44800, 33760, 42456, 63528, 1024,
                37504, 41972, 65440, 7456,  36324, 40896, 53056, 44200, 29264,
                63588, 25476, 11696, 2568,  52928, 5532,  20784, 25928, 4408,
                49712, 57384, 38960, 29324, 55864, 50644, 1800,  6660,  45552,
                10664, 53332, 40124, 3068,  58632, 36504, 43952, 8400,  25880,
                21344, 50108, 9684,  29796, 1092,  12788, 46152, 5764,  17368,
                55908, 1304,
            },
            {
                61300, 32880, 9572,  51308, 6080,  17212, 49432, 2152,  20600,
                52632, 43692, 27356, 41532, 54996, 31812, 59744, 44964, 23408,
                61184, 560,   40712, 9544,  4792,  36680, 62032, 30372, 50264,
                45792, 53400, 2380,  56200, 8840,  45064, 6016,  60536, 54888,
                17668, 13632, 59548, 52808, 63812, 8488,  3588,  60184, 17576,
                42436, 32184, 57232, 51164, 38724, 63896, 29004, 43140, 57040,
                49900, 6384,  40784, 228,   19856, 54488, 42036, 22612, 37768,
                57076, 3256,  24188, 61580, 46736, 10788, 36368, 49856, 18592,
                12756, 8020,  35160, 19388, 61948, 3708,  27580, 50716, 12096,
                25496, 20396, 55648, 13896, 24588, 57736, 21384, 2776,  32420,
                6712,  55744, 34384, 45464, 31268, 58052, 35272, 46688, 64112,
                14156, 23056, 9464,  2940,  26360, 13016, 34392, 59292, 38064,
                17604, 24532, 63436, 14496, 46408, 20052, 52244, 32888, 15324,
                60104, 54492, 18244, 33400, 61660, 24280, 36476, 53500, 26840,
                38504, 49076,
            },
            {
                29076, 15888, 44660, 63548, 42076, 22992, 33840, 56632, 31316,
                23888, 5328,  60592, 14320, 22276, 1352,  16196, 10260, 53108,
                37208, 15172, 55160, 45456, 21728, 51028, 1684,  40272, 9304,
                37260, 17196, 21348, 32024, 50688, 63684, 19628, 26520, 49108,
                41812, 32356, 1820,  25432, 18864, 31888, 56932, 11300, 36968,
                64404, 3012,  27464, 5340,  34512, 13268, 17296, 1648,  32156,
                23428, 15896, 59440, 24844, 13460, 36460, 31088, 16304, 60272,
                9736,  42636, 18476, 32480, 14816, 54460, 28340, 44204, 40020,
                53008, 24020, 56080, 48436, 7036,  23252, 57480, 17828, 34820,
                47472, 4216,  44848, 33500, 48780, 11312, 17520, 46664, 59568,
                50576, 19396, 41140, 22772, 63276, 9360,  17088, 29020, 1400,
                43824, 31564, 60312, 42312, 65108, 20620, 40440, 52136, 27128,
                4220,  47736, 8644,  31416, 26852, 6156,  64444, 1984,  40256,
                11968, 36676, 3756,  48372, 43072, 20260, 8088,  59804, 10948,
                21004, 4156,
            },
            {
                57696, 12832, 35700, 948,   26940, 59844, 7492,  40816, 12428,
                48076, 57440, 33516, 8832,  64888, 47452, 40044, 28872, 5376,
                31568, 48524, 26640, 59984, 33552, 16588, 54520, 23340, 63760,
                13480, 28680, 61400, 4908,  35760, 14132, 40696, 876,   29344,
                9644,  57404, 35240, 43560, 15084, 38844, 49340, 21644, 29728,
                14184, 46364, 40032, 20116, 61004, 24968, 54740, 40180, 47104,
                9140,  38352, 52040, 33120, 46336, 60980, 51200, 1724,  47752,
                25980, 34636, 65100, 456,   39284, 22180, 6780,  59864, 2828,
                32808, 63560, 42004, 15868, 38432, 31872, 10340, 40640, 54184,
                62276, 9224,  58916, 28624, 344,   39468, 64192, 27300, 10280,
                37600, 15540, 700,   48540, 13684, 38564, 51404, 55916, 37236,
                19364, 51992, 35408, 17220, 53884, 46384, 7744,  15344, 32316,
                60636, 54456, 35888, 42564, 57088, 11040, 23824, 47572, 53380,
                30472, 45432, 26504, 56112, 15596, 39196, 49892, 31544, 44472,
                64756, 41584,
            },
            {
                51988, 24580, 48224, 20288, 53688, 10480, 46020, 64588, 28020,
                3408,  38980, 18888, 51492, 25988, 34800, 58564, 20036, 63556,
                42924, 12172, 18852, 7088,  29736, 10640, 46772, 26988, 6652,
                55640, 48720, 39852, 10792, 58012, 47904, 23448, 52368, 37124,
                64156, 21832, 47536, 7260,  61520, 54636, 5368,  45260, 53548,
                7916,  59028, 49248, 10900, 53256, 45116, 4356,  60100, 19984,
                64056, 30004, 55764, 3748,  10452, 27392, 7244,  19348, 39748,
                54200, 13736, 49980, 27536, 57996, 51820, 34836, 16840, 10152,
                21268, 27284, 4036,  11676, 51732, 64864, 44992, 2544,  21724,
                30496, 38636, 15900, 19524, 50800, 54644, 35192, 3960,  42704,
                24200, 62804, 29944, 54980, 4640,  42844, 24672, 7340,  12100,
                62332, 5748,  25164, 11460, 4492,  30216, 23912, 56640, 44172,
                11604, 21988, 448,   18304, 51364, 38704, 34268, 19020, 60904,
                7556,  22600, 64072, 11224, 5524,  28808, 57180, 1544,  16372,
                34208, 7792,
            },
            {
                19236, 5660,  61976, 39184, 30204, 15044, 36608, 19448, 52924,
                16044, 62540, 29968, 2376,  44224, 12352, 4068,  49676, 24464,
                56100, 3096,  39536, 51668, 64460, 35168, 57684, 19072, 43564,
                32544, 160,   24356, 44264, 18168, 30928, 8084,  61468, 12524,
                16568, 4800,  51356, 28200, 12180, 23908, 35604, 1672,  26528,
                18344, 33756, 23964, 320,   30896, 36708, 8320,  27244, 35096,
                12196, 5548,  17972, 42652, 22380, 64452, 44180, 56628, 30104,
                21400, 7884,  45300, 5168,  11816, 42420, 25668, 48968, 61908,
                46260, 37096, 58832, 29564, 20196, 55064, 25224, 14640, 49580,
                5824,  26556, 43464, 63168, 6640,  30844, 22408, 14004, 52388,
                56824, 8060,  35452, 26596, 61836, 20204, 33396, 58372, 48808,
                27660, 45284, 39788, 59056, 49644, 37652, 63152, 1936,  18968,
                40964, 64608, 49552, 30060, 15164, 61892, 2948,  28472, 42736,
                16452, 604,   39904, 51344, 35220, 63096, 13164, 23032, 52876,
                26200, 37248,
            },
            {
                45264, 55768, 32492, 3000,  58924, 49292, 5460,  25356, 43880,
                35076, 11132, 46648, 55960, 21484, 37788, 53828, 17124, 9120,
                32896, 61108, 22676, 44488, 1156,  13900, 41460, 4096,  59600,
                36020, 20404, 65356, 54376, 27692, 3328,  34432, 55524, 45800,
                26060, 39204, 56204, 19836, 41932, 30788, 64108, 58412, 42872,
                38456, 62484, 15392, 43476, 65104, 18652, 14432, 48380, 52408,
                41288, 25184, 61972, 50096, 37476, 15128, 34328, 11692, 2732,
                59192, 36332, 61036, 31704, 19912, 63360, 2200,  29924, 55396,
                13584, 5636,  50332, 43816, 64,    35720, 7464,  59324, 33088,
                61148, 52672, 11760, 23936, 41136, 9584,  60936, 45736, 32232,
                19872, 47296, 11824, 51780, 16544, 46088, 2340,  40956, 22076,
                15720, 54208, 412,   21340, 33452, 14748, 9624,  47412, 34632,
                28352, 6948,  25032, 58140, 4576,  44960, 54280, 12624, 56928,
                49932, 32660, 58152, 19220, 24752, 46268, 41236, 5004,  47508,
                62668, 11832,
            },
            {
                40048, 27872, 16924, 12236, 42912, 22220, 57200, 420,   61380,
                7880,  23316, 41000, 6168,  15528, 62828, 30220, 41320, 46024,
                36480, 15320, 28476, 54392, 37412, 25264, 49544, 29416, 16000,
                11408, 51720, 8256,  14976, 38220, 59296, 42792, 21136, 2168,
                31432, 59820, 33884, 556,   46500, 8716,  16084, 20864, 12472,
                52148, 3976,  56036, 27848, 50684, 39676, 58524, 22892, 56528,
                2108,  33420, 46924, 28312, 996,   58272, 52332, 24936, 49124,
                42160, 16164, 23972, 53224, 37848, 15452, 45032, 7288,  38756,
                18200, 25364, 33888, 61560, 16944, 40224, 27944, 46424, 18408,
                37320, 1416,  56828, 47644, 36312, 58092, 17924, 5584,  38428,
                2092,  59240, 40428, 5724,  31288, 60212, 10552, 29556, 64912,
                34664, 8684,  61684, 28736, 43716, 54524, 26204, 61000, 51516,
                13264, 37520, 46580, 10216, 40600, 21568, 25720, 35988, 9356,
                37964, 27060, 44692, 4164,  8796,  54216, 33308, 18072, 58704,
                30416, 2708,
            },
            {
                60724, 8892,  47344, 63044, 26316, 34244, 51664, 39732, 31944,
                54356, 48916, 59616, 27524, 33676, 58008, 7212,  26192, 252,
                56984, 50456, 5432,  10088, 61920, 21172, 7636,  53664, 62512,
                47400, 40520, 25868, 45604, 5696,  51156, 11392, 17444, 50192,
                41312, 14268, 10060, 62900, 53928, 38000, 50144, 32680, 6960,
                47084, 30160, 9176,  21680, 11956, 2536,  32516, 6524,  43984,
                16584, 59220, 13416, 8752,  20168, 40760, 5076,  18192, 32472,
                63768, 4136,  28648, 47964, 9112,  56352, 23128, 32548, 52556,
                42088, 57396, 8932,  22960, 12820, 49104, 56448, 9980,  41764,
                13556, 21468, 31924, 5132,  15032, 26472, 49680, 29296, 65232,
                24760, 15820, 44160, 21776, 37640, 54648, 50280, 14204, 4304,
                38464, 47788, 18176, 12864, 58160, 2744,  40000, 22204, 5056,
                59180, 55288, 17228, 33924, 63472, 31724, 48616, 5480,  65076,
                21108, 13788, 62480, 31164, 16016, 60968, 27632, 38676, 8064,
                21352, 50284,
            },
            {
                33688, 23636, 52692, 36696, 7296,  18148, 10124, 14260, 28632,
                20560, 16948, 37180, 1284,  50756, 11376, 47480, 22128, 65064,
                12556, 20516, 31784, 47768, 17240, 58492, 33116, 38604, 22908,
                2360,  31024, 60676, 33816, 22300, 62156, 29752, 36572, 64804,
                6188,  22752, 49012, 18312, 25640, 3912,  57268, 27744, 61396,
                23508, 36740, 41780, 60132, 34768, 46148, 62596, 29052, 10772,
                38596, 21948, 51132, 35860, 54896, 29560, 62412, 46108, 38272,
                8464,  54536, 12764, 40336, 1172,  35004, 60644, 11308, 20604,
                65468, 2772,  47152, 53796, 32268, 63996, 5576,  23688, 62740,
                51660, 44728, 28212, 64428, 53068, 2460,  43304, 55488, 10052,
                33856, 53736, 28524, 63792, 1136,  25572, 19392, 44344, 56680,
                23160, 31364, 42044, 52132, 6336,  19564, 32580, 16208, 43112,
                31284, 1432,  20592, 52384, 7724,  13700, 59160, 18600, 51692,
                46000, 2452,  52780, 42128, 22304, 49744, 980,   12468, 54908,
                42584, 14680,
            },
            {
                1440,  19948, 41696, 3948,  55000, 44564, 65452, 47832, 57960,
                2964,  12384, 63960, 24056, 43060, 19144, 39172, 52836, 34284,
                43828, 59684, 40412, 26536, 43052, 1968,  13052, 45020, 27604,
                6344,  17584, 55560, 13156, 1228,  47656, 25116, 8640,  53616,
                27180, 58016, 43664, 29996, 35924, 13780, 44540, 40228, 960,
                54568, 18764, 48912, 5140,  16788, 51804, 24440, 19264, 54352,
                64748, 31508, 5912,  60492, 44908, 23384, 9916,  14056, 26440,
                57288, 21168, 44372, 18568, 64104, 50820, 27520, 46488, 4628,
                29352, 15760, 37200, 27084, 1692,  20712, 38204, 29900, 3448,
                35052, 8360,  17332, 40316, 34100, 20156, 12640, 22292, 40008,
                48204, 6648,  13352, 49492, 9656,  42412, 35708, 7564,  59828,
                51316, 3064,  26464, 63396, 35612, 46992, 50628, 65492, 8916,
                26864, 45120, 60912, 38276, 28696, 44252, 1248,  41464, 30232,
                24428, 8164,  35336, 59812, 11080, 36936, 44308, 65240, 24348,
                35388, 58004,
            },
            {
                29248, 64168, 13096, 59132, 31356, 27792, 21296, 5732,  41960,
                33108, 45972, 52992, 9508,  31436, 61160, 14856, 2980,  29404,
                8568,  16176, 3520,  63924, 35564, 55376, 51040, 9448,  64436,
                57508, 36204, 41924, 52568, 19324, 39388, 57428, 15036, 33052,
                46124, 2748,  38904, 60864, 5792,  22084, 65448, 17316, 10336,
                33568, 13580, 64200, 26264, 56392, 7896,  41368, 36084, 1084,
                45824, 25844, 41916, 2600,  16352, 32812, 48384, 59780, 32,
                29904, 49688, 33900, 58340, 25172, 5400,  17260, 39832, 59492,
                34352, 51540, 11544, 41436, 59652, 44980, 16144, 52976, 47116,
                25760, 55540, 60044, 10844, 46100, 62264, 36964, 59372, 464,
                17544, 61644, 36532, 57980, 32104, 17080, 62640, 27408, 33172,
                11552, 16392, 56116, 9972,  23620, 28960, 12132, 54856, 36528,
                23488, 48800, 14480, 4332,  22952, 54960, 34648, 15460, 61272,
                39380, 56416, 17420, 26404, 5888,  57600, 30012, 19368, 52264,
                5408,  46184,
            },
            {
                38568, 49280, 25420, 16396, 39288, 1276,  51032, 35900, 24320,
                62028, 38708, 29200, 4868,  57016, 35492, 6536,  55720, 24732,
                48404, 37500, 53312, 23568, 6956,  19880, 30264, 40800, 15388,
                21096, 49132, 3728,  28556, 10240, 43396, 63200, 3980,  37624,
                19864, 11472, 16064, 52084, 9128,  55740, 46776, 25168, 50960,
                58568, 43784, 2812,  38096, 30968, 13104, 59840, 49256, 15312,
                9504,  52772, 19904, 12280, 64000, 39420, 6468,  52960, 42628,
                16628, 37436, 3048,  10676, 31504, 43332, 13680, 49344, 9420,
                21916, 44236, 62340, 18896, 8052,  55952, 33392, 10992, 59152,
                19616, 652,   49464, 23672, 4364,  29952, 7516,  51292, 31472,
                27024, 45128, 20384, 24484, 3244,  55512, 48012, 5304,  40064,
                21024, 45676, 37732, 1064,  41280, 59896, 6836,  39280, 3112,
                57452, 11032, 40888, 63060, 50068, 10404, 57992, 27208, 4612,
                11724, 48972, 33320, 46836, 40856, 15512, 3500,  47912, 27000,
                16832, 10896,
            },
            {
                32252, 6584,  52060, 9208,  46252, 62492, 11800, 55328, 14988,
                8224,  18284, 49048, 21936, 44556, 26676, 49944, 40528, 63008,
                18332, 58628, 11508, 29000, 14536, 46812, 61436, 24856, 34164,
                45816, 7864,  61248, 24208, 56380, 31260, 7164,  23208, 48672,
                55556, 63580, 31776, 26416, 41112, 33944, 3352,  30596, 38788,
                7256,  28496, 19768, 53696, 45012, 22168, 4308,  27676, 62284,
                30460, 58224, 37856, 50020, 27728, 56592, 19096, 35400, 22472,
                65208, 7588,  52052, 60500, 22332, 55096, 35904, 57844, 432,
                25724, 55176, 3552,  30680, 24328, 49820, 4764,  39640, 13992,
                42656, 37112, 32084, 15836, 54180, 26124, 47328, 14336, 42284,
                56084, 4116,  52604, 41124, 12580, 44824, 22624, 14796, 53748,
                64516, 49164, 28220, 61444, 17708, 44396, 21516, 15352, 30024,
                52628, 34176, 17640, 31512, 25912, 19976, 46532, 37456, 53240,
                22824, 63772, 192,   20616, 54148, 62952, 32596, 9344,  39964,
                56128, 60408,
            },
            {
                44772, 18484, 35772, 57084, 22652, 33548, 18876, 43656, 58612,
                27296, 3464,  54756, 13312, 64476, 16964, 10264, 21328, 31596,
                1392,  44296, 33608, 50532, 38772, 56312, 436,   5892,  53848,
                11844, 31624, 38408, 14356, 47028, 34948, 17156, 52256, 27680,
                41616, 148,   22420, 49720, 59124, 12164, 20648, 54240, 15028,
                62704, 23228, 48216, 9836,  34452, 57544, 39364, 18184, 42992,
                5456,  22836, 34420, 8216,  44092, 24536, 3160,  31044, 11536,
                45416, 25768, 15204, 46828, 39292, 7284,  28400, 19200, 64328,
                32436, 38192, 48172, 14352, 36072, 64704, 27432, 21448, 62864,
                29184, 7720,  61572, 43700, 39276, 57776, 18728, 62972, 23248,
                11344, 35012, 7992,  64936, 28660, 37032, 60604, 244,   30684,
                8276,  34912, 4596,  13820, 50212, 33628, 58416, 46780, 63504,
                19824, 7192,  59268, 564,   43272, 5644,  64732, 8396,  18496,
                30904, 43792, 13704, 28916, 7740,  23776, 50804, 36348, 62004,
                2892,  22232,
            },
            {
                28240, 572,   63628, 42080, 4208,  27012, 49184, 2344,  31280,
                39764, 61068, 34380, 41728, 940,   37932, 59088, 46692, 13120,
                54552, 6176,  20876, 60152, 9668,  42848, 18524, 37092, 59084,
                26468, 50724, 63992, 20436, 828,   60048, 44404, 12732, 61516,
                8900,  34992, 45660, 5296,  18064, 63108, 42880, 48996, 1368,
                35440, 41964, 60276, 16188, 196,   63856, 11400, 32980, 55360,
                47652, 13292, 61500, 744,   53992, 14672, 58964, 50492, 61352,
                38836, 56300, 29376, 19936, 63172, 3972,  51460, 45192, 40868,
                12552, 17560, 6452,  58500, 40564, 9904,  44276, 53792, 2880,
                50692, 56888, 12476, 21572, 1956,  10036, 32828, 5932,  38800,
                59648, 16664, 47024, 32860, 18856, 9984,  49740, 25632, 58144,
                43424, 19204, 24516, 53464, 30344, 5820,  25216, 1600,  41884,
                27800, 50880, 45656, 36292, 54616, 13232, 33000, 50484, 41768,
                3616,  58776, 51584, 36836, 56692, 44664, 12424, 20084, 43288,
                14612, 53504,
            },
            {
                48580, 12936, 55024, 29868, 15588, 61296, 38104, 10832, 51628,
                20376, 46884, 9876,  23944, 30800, 52956, 4436,  28028, 41992,
                35940, 25352, 47796, 2752,  26980, 32300, 64648, 22228, 44072,
                3228,  17636, 42320, 6216,  53324, 26312, 39648, 3276,  30588,
                18848, 57828, 13692, 39312, 29164, 36732, 8236,  27092, 57728,
                11596, 31832, 5300,  50808, 29208, 46508, 24540, 51400, 2424,
                26492, 40656, 18668, 46808, 29608, 36648, 41060, 9332,  18320,
                5452,  33404, 2436,  42548, 11928, 34096, 15716, 24060, 8260,
                53768, 60308, 27864, 51904, 20160, 796,   17064, 32276, 24064,
                35552, 17960, 26968, 47984, 65136, 35912, 45312, 53732, 2696,
                29764, 50404, 25080, 58400, 2500,  53028, 39456, 33876, 16900,
                11664, 62516, 40716, 57348, 10504, 38212, 54388, 12924, 35260,
                9556,  24012, 14876, 62500, 28760, 40004, 24176, 15856, 55460,
                35172, 9632,  25360, 40128, 16704, 2496,  59048, 30564, 6276,
                25100, 35188,
            },
            {
                7540,  20728, 39156, 9744,  47696, 5788,  54136, 25144, 64972,
                6604,  15688, 57556, 50472, 18520, 62620, 33776, 8000,  57340,
                65340, 15368, 39496, 61956, 51700, 16360, 12600, 48624, 29308,
                9200,  57484, 35504, 29744, 48272, 10584, 22052, 56900, 37184,
                50900, 25104, 64868, 52824, 1804,  56288, 15892, 22596, 45440,
                52580, 18528, 26216, 55936, 40772, 20372, 6880,  36444, 15636,
                65496, 57100, 32348, 10756, 21872, 63668, 4680,  48020, 27936,
                54132, 13792, 50080, 58868, 25140, 54408, 60964, 48004, 29980,
                2724,  43152, 22964, 46072, 34444, 55856, 48644, 59916, 6720,
                46816, 40608, 3664,  51984, 28928, 15648, 23960, 49140, 19656,
                63920, 40864, 13304, 5816,  42968, 62100, 20928, 6820,  46420,
                52520, 36664, 1456,  45168, 20996, 64380, 48484, 18616, 60788,
                56464, 38696, 2936,  21232, 9992,  47836, 61100, 1768,  27484,
                62872, 19528, 47992, 5536,  64376, 34368, 26792, 49472, 54776,
                64924, 41356,
            },
            {
                59776, 51204, 24816, 58708, 34716, 21424, 42520, 13932, 33992,
                44776, 29532, 37188, 6444,  26140, 45556, 14884, 23008, 51192,
                19180, 10728, 30908, 7236,  22812, 35724, 5356,  52800, 39628,
                61816, 13228, 23792, 55892, 15780, 33480, 64300, 45988, 6388,
                15644, 42000, 32240, 7560,  24016, 47872, 32964, 61536, 4720,
                38448, 65024, 8440,  37080, 12848, 61788, 43624, 53888, 9872,
                21324, 37576, 6744,  51488, 55972, 16828, 33844, 24236, 44196,
                20684, 64036, 36240, 8612,  30696, 38700, 1308,  20816, 36924,
                63400, 33292, 10860, 4372,  61556, 29508, 12388, 38248, 15056,
                64028, 10324, 33148, 55976, 8152,  42416, 59144, 11864, 26884,
                8656,  35688, 22124, 56400, 30548, 15520, 27380, 56020, 3492,
                28892, 22716, 31916, 8120,  16244, 27272, 4160,  31040, 43812,
                6008,  47420, 32820, 53284, 58352, 5032,  20360, 38080, 46192,
                13012, 30196, 59892, 22608, 11336, 47172, 18576, 39136, 1548,
                11180, 32676,
            },
            {
                17308, 5224,  31200, 44716, 17704, 56696, 29072, 332,   58828,
                22900, 54468, 2256,  43108, 59460, 11576, 39724, 48712, 112,
                27204, 44856, 53320, 57024, 40792, 45444, 58304, 1104,  33132,
                20268, 46016, 2280,  40928, 60564, 7776,  19280, 28692, 54796,
                2228,  47160, 11128, 20512, 60640, 43300, 10148, 40536, 30168,
                14312, 21908, 47668, 32488, 3456,  17096, 57336, 27196, 31248,
                48424, 4048,  44640, 27488, 41660, 1520,  60820, 12056, 58572,
                39876, 476,   23444, 48444, 17768, 44148, 10652, 57172, 14200,
                50764, 18736, 39416, 15772, 24596, 8132,  42392, 26208, 57276,
                20952, 44032, 23696, 60888, 19352, 172,   31572, 38128, 55200,
                44096, 1696,  51884, 46140, 37868, 11388, 48788, 41476, 64716,
                14364, 60052, 47920, 51980, 61612, 34012, 40344, 52768, 22480,
                16896, 64928, 25732, 41512, 17512, 30944, 51352, 57280, 7932,
                42864, 4076,  54292, 32188, 41608, 53080, 8512,  61340, 15476,
                23204, 46352,
            },
            {
                37728, 62352, 13468, 2620,  63404, 7940,  37448, 50056, 18260,
                39948, 12404, 63312, 20880, 35152, 3668,  56012, 30304, 37004,
                59348, 5024,  33920, 17816, 11364, 28140, 63372, 24680, 15940,
                54968, 27568, 50348, 37052, 4796,  51768, 42948, 13140, 23468,
                34376, 58592, 27156, 50624, 35544, 55300, 3144,  19536, 50464,
                58284, 1808,  43932, 60400, 24220, 49444, 34212, 892,   41412,
                60576, 17612, 58196, 23868, 13892, 37204, 49804, 29348, 6672,
                32088, 45716, 15136, 53396, 6164,  65036, 28368, 46144, 25472,
                5308,  58936, 48420, 54288, 65248, 35824, 52756, 1436,  31132,
                4296,  54656, 37376, 13548, 39872, 51052, 62468, 5276,  14792,
                30244, 61076, 16808, 25828, 63676, 1048,  35304, 24228, 8740,
                38420, 18700, 4812,  25584, 42396, 13648, 58940, 292,   10588,
                29512, 50608, 13036, 7800,  44936, 35828, 11372, 24788, 33748,
                16908, 49756, 37468, 14304, 712,   24380, 29568, 36156, 43724,
                57436, 28616,
            },
            {
                3764,  53864, 48852, 39912, 27168, 52712, 11656, 45684, 61664,
                7484,  27200, 48092, 31988, 52696, 17644, 61768, 21144, 9276,
                41880, 62916, 24872, 1780,  47896, 19704, 8252,  42916, 38280,
                6480,  65276, 10428, 18016, 30924, 25220, 62552, 39096, 49840,
                61112, 17992, 39776, 5492,  16460, 13116, 27964, 63568, 25392,
                34140, 10656, 54840, 28396, 39560, 9376,  63656, 14096, 23516,
                52340, 11840, 34532, 64304, 8532,  54592, 42860, 17896, 52892,
                10112, 57520, 27460, 59856, 41144, 21796, 34832, 55404, 8508,
                42120, 30716, 26916, 2564,  43760, 21376, 17932, 60484, 45608,
                50312, 17120, 29648, 5864,  26364, 45224, 21844, 34548, 47112,
                57700, 20676, 9944,  33576, 7364,  53204, 18292, 59136, 51116,
                32568, 43908, 55336, 11124, 36288, 6816,  23744, 46680, 37532,
                57812, 34428, 2140,  60892, 27832, 63292, 484,   40768, 52540,
                64248, 26872, 20176, 57976, 62836, 45636, 56572, 3360,  20388,
                51864, 9296,
            },
            {
                25536, 19196, 33096, 22208, 43416, 15988, 24500, 31048, 3312,
                33588, 51916, 16128, 8960,  41352, 28356, 5416,  47276, 54752,
                16312, 12492, 50188, 38652, 60948, 55424, 31984, 51280, 12448,
                47284, 34404, 22756, 44044, 59076, 11708, 35608, 636,   9348,
                30424, 3852,  45076, 63212, 31388, 37840, 46076, 53348, 5836,
                42484, 49060, 15080, 19156, 4484,  53120, 20764, 46256, 7140,
                38840, 28884, 2136,  47136, 30820, 21048, 3640,  26148, 63540,
                38592, 19424, 35176, 3072,  12612, 31612, 4244,  16296, 62456,
                20408, 36660, 12932, 9492,  32256, 51176, 6368,  39584, 11608,
                34808, 8584,  63596, 47728, 57380, 10720, 17252, 53940, 24904,
                4340,  38816, 48556, 42332, 59940, 21736, 44500, 29332, 12284,
                2956,  22132, 58420, 29864, 49812, 19404, 54640, 62776, 14692,
                43116, 20508, 55232, 38132, 22800, 48460, 14008, 58728, 21624,
                3044,  10192, 44388, 7320,  34192, 17204, 50004, 12960, 32116,
                63448, 41148,
            },
            {
                35104, 55772, 10688, 60232, 1636,  36112, 64288, 55344, 19548,
                42960, 60348, 24980, 988,   64032, 44624, 13488, 25564, 35560,
                31628, 22344, 43708, 29044, 14892, 35364, 3336,  21856, 61480,
                30028, 1468,  56348, 52476, 3400,  47624, 20892, 56972, 44208,
                14844, 22488, 56248, 52720, 24520, 316,   58840, 9008,  16936,
                36628, 23492, 62956, 31648, 58692, 36068, 42380, 30288, 55284,
                62236, 19980, 50544, 40532, 15924, 61672, 35672, 48640, 13372,
                5248,  50832, 24124, 43732, 61312, 51544, 48044, 39224, 52444,
                256,   60364, 55120, 46896, 58672, 14100, 28936, 62748, 22868,
                26808, 41968, 53032, 3752,  36060, 31060, 64956, 8328,  41540,
                12536, 63244, 27108, 3444,  30752, 14224, 57008, 5664,  47440,
                63256, 40452, 15196, 1316,  65392, 44268, 28176, 32260, 5672,
                25560, 9024,  47068, 16216, 4288,  54416, 18464, 29896, 46416,
                32456, 54880, 39512, 29160, 22240, 4692,  40388, 27192, 7040,
                48008, 14556,
            },
            {
                61328, 45208, 6964,  29720, 57252, 46900, 5184,  9664,  49268,
                36376, 13604, 39504, 56676, 22456, 37116, 58696, 51616, 7280,
                64176, 3460,  58548, 6900,  52056, 10188, 45016, 27096, 57192,
                18840, 41840, 14296, 38460, 28336, 16040, 32628, 53188, 26924,
                65116, 36688, 7416,  11044, 19108, 48792, 40352, 21564, 29276,
                57244, 1108,  40604, 7476,  46868, 11216, 25836, 1708,  16096,
                35432, 9796,  25128, 55948, 6888,  44288, 58716, 23060, 33252,
                46400, 56196, 30440, 8760,  37900, 18424, 26592, 9968,  43928,
                32688, 23404, 17112, 40664, 24408, 37680, 2052,  48864, 55756,
                15096, 59232, 19672, 23948, 13888, 43480, 1424,  28436, 50460,
                35932, 17988, 55116, 51708, 10596, 40132, 25488, 37684, 17232,
                34876, 26732, 52832, 33792, 9096,  38916, 16940, 2712,  49180,
                60632, 39828, 29252, 59064, 32668, 42836, 9232,  39064, 5496,
                61384, 16104, 48552, 60072, 11732, 51676, 65412, 36764, 58752,
                23372, 96,
            },
            {
                27736, 38288, 17528, 51240, 13984, 20656, 40584, 28732, 23396,
                57928, 6448,  30284, 46360, 10696, 33248, 3100,  19296, 40488,
                45932, 53692, 37084, 20468, 25504, 64684, 40288, 16068, 4724,
                48856, 8928,  26032, 59912, 7512,  63748, 40188, 5052,  18604,
                48608, 41048, 33336, 28428, 43184, 62104, 14428, 32588, 64836,
                45436, 52348, 13196, 27148, 51016, 17928, 64964, 57120, 48284,
                43580, 5048,  60156, 33028, 12728, 28348, 2528,  9708,  41680,
                17152, 2036,  64632, 14764, 53964, 1452,  22852, 63740, 14060,
                28600, 50124, 7600,  4192,  63984, 18732, 43496, 33636, 7020,
                46308, 624,   32656, 40024, 51888, 60460, 48256, 19940, 59420,
                32072, 2208,  22936, 45772, 34356, 64816, 50480, 100,   54904,
                61072, 6552,  19900, 48148, 23156, 60176, 12344, 53648, 34812,
                18296, 51752, 12560, 6324,  24636, 52164, 65160, 26104, 36244,
                12860, 23880, 1296,  35628, 55984, 25344, 44760, 18908, 10232,
                42644, 52548,
            },
            {
                20936, 4808,  64848, 41796, 25044, 34588, 53524, 12160, 61032,
                17720, 2644,  54300, 20112, 49624, 16232, 62312, 30640, 9856,
                27276, 17356, 11964, 33184, 47076, 364,   59444, 37256, 53016,
                33104, 62768, 36400, 19868, 45872, 50296, 23600, 10472, 60280,
                1420,  13168, 51872, 59644, 2836,  35108, 6196,  49948, 11480,
                4456,  20200, 33664, 61292, 38284, 23132, 4080,  32056, 12836,
                27448, 52124, 17472, 46312, 39244, 20212, 49852, 54040, 62404,
                27136, 39712, 21240, 45060, 28072, 58824, 35588, 56584, 5200,
                37976, 61632, 45068, 34996, 27172, 52704, 58192, 10356, 21432,
                38544, 28812, 62316, 9068,  17728, 25956, 37996, 5440,  55844,
                14592, 43192, 60532, 6224,  16548, 20352, 8272,  31896, 22536,
                42708, 10196, 44924, 57884, 3876,  26956, 41284, 45804, 64128,
                23456, 792,   44740, 62152, 35308, 19640, 2320,  49156, 57260,
                43468, 52820, 19444, 41856, 8516,  15052, 1840,  33280, 55440,
                16148, 31124,
            },
            {
                56876, 12652, 48964, 31956, 8392,  62504, 936,   48024, 38216,
                43408, 32920, 65460, 27468, 7488,  56092, 42856, 48100, 23376,
                57672, 2204,  61980, 42532, 55748, 18492, 30764, 23892, 11276,
                43868, 22284, 54984, 5388,  12800, 30164, 34960, 43504, 54868,
                31408, 25096, 46084, 16348, 22156, 56076, 26116, 53988, 38576,
                24576, 42808, 55872, 15920, 5884,  44380, 53984, 40244, 20888,
                62672, 36932, 23260, 808,   65184, 57456, 36136, 14228, 31656,
                7444,  58460, 11448, 49224, 32584, 7176,  41408, 46840, 19288,
                54204, 10448, 21200, 56260, 12520, 3128,  30448, 16488, 65000,
                54384, 12752, 49136, 56456, 2740,  33988, 11768, 22668, 46216,
                8588,  26236, 37152, 29716, 56088, 41976, 61736, 46572, 14704,
                29096, 51368, 13508, 31144, 36588, 55392, 7092,  21308, 10172,
                30984, 37284, 55596, 14780, 47880, 40948, 11072, 16612, 29464,
                6872,  63912, 27396, 31980, 62356, 47552, 29644, 60628, 38972,
                3108,  46304,
            },
            {
                9220,  36096, 22356, 3416,  58228, 45136, 16384, 31112, 26184,
                9028,  51272, 14544, 41308, 24092, 37784, 472,   13360, 52128,
                34752, 39152, 49260, 29440, 5092,  13464, 50524, 6064,  57552,
                15252, 1176,  31496, 41068, 52220, 2056,  58244, 15492, 37620,
                20484, 5772,  62908, 8968,  39668, 44892, 30748, 18364, 8384,
                58376, 28384, 2164,  48844, 30212, 10524, 60032, 34676, 8560,
                3428,  41644, 54952, 11108, 30292, 25324, 4840,  18404, 46944,
                23452, 52064, 37304, 3904,  62760, 16744, 12060, 30616, 2556,
                25416, 41064, 15728, 31588, 47804, 37368, 50696, 25284, 41612,
                35180, 4980,  22796, 45072, 41208, 64132, 52420, 30044, 39604,
                63408, 53040, 11148, 49960, 1660,  12648, 26844, 35980, 58768,
                2664,  38152, 24608, 64048, 17508, 49996, 14504, 28492, 58504,
                3944,  42408, 26552, 8336,  22216, 30420, 60692, 54660, 21864,
                34252, 10424, 46480, 3268,  37960, 21748, 53916, 6772,  24652,
                50344, 63468,
            },
            {
                26668, 43720, 53184, 38848, 27412, 19724, 55204, 5972,  63304,
                22000, 45904, 35840, 4452,  52648, 60112, 29124, 20588, 64764,
                6540,  15508, 24668, 9248,  21884, 63576, 35032, 41744, 46732,
                28464, 64900, 48352, 17412, 61988, 26568, 21472, 8480,  64384,
                47924, 53012, 27776, 36912, 57820, 12220, 4000,  47216, 61012,
                35252, 12712, 62808, 36656, 19848, 51604, 24632, 14488, 49700,
                58576, 31300, 15364, 48472, 7780,  51764, 40248, 43292, 60296,
                36,    34416, 18768, 42892, 25068, 53112, 22120, 50896, 65464,
                34064, 49316, 59244, 5768,  62708, 22704, 7988,  60620, 1712,
                51868, 18412, 27364, 31380, 15132, 7156,  57936, 16692, 488,
                33224, 17876, 21440, 62572, 38976, 23528, 48776, 5320,  18068,
                54072, 60524, 6256,  41636, 588,   33928, 61460, 39132, 48612,
                52920, 16988, 63208, 33800, 58248, 4836,  36896, 45472, 644,
                39836, 51456, 14292, 57568, 17564, 49776, 11924, 41572, 19560,
                13964, 34196,
            },
            {
                59504, 18024, 62060, 14776, 10716, 42112, 35040, 49084, 13272,
                53668, 1280,  19272, 62116, 12104, 34156, 8976,  45612, 41008,
                31272, 55128, 60508, 44700, 52964, 39400, 26132, 61220, 20148,
                8168,  37568, 24728, 10012, 35512, 55624, 45548, 29104, 3452,
                41552, 17596, 33012, 2552,  50768, 20696, 64152, 41336, 16056,
                50952, 22112, 46028, 7872,  41624, 63892, 220,   29036, 45768,
                18452, 25936, 63516, 44184, 21636, 33600, 62644, 10632, 28160,
                54788, 13640, 29836, 56908, 9620,  59592, 36812, 43172, 14524,
                8744,  28868, 568,   43592, 19104, 39780, 46276, 14380, 32532,
                43620, 58600, 9476,  61164, 36796, 21124, 46744, 27640, 42316,
                56816, 47936, 3808,  44360, 32272, 57952, 52576, 9520,  40748,
                32396, 20956, 46236, 11320, 53168, 19772, 43864, 4956,  32540,
                12724, 20412, 46224, 1524,  49736, 52788, 14028, 25884, 62336,
                18912, 59404, 24480, 42824, 6012,  27044, 35084, 61792, 44940,
                29196, 4256,
            },
            {
                7732,  24080, 516,   50104, 29920, 61008, 2488,  24568, 58816,
                39772, 28328, 57136, 31244, 46980, 16772, 25996, 59080, 3556,
                18460, 11396, 37164, 1204,  31864, 16260, 3252,  10444, 32760,
                52684, 4016,  59476, 44064, 14404, 5552,  39436, 51232, 13080,
                59368, 10068, 23512, 61372, 14968, 29304, 34460, 23780, 680,
                31052, 5500,  53700, 25824, 32800, 16632, 57576, 38892, 54684,
                6704,  35312, 1744,  38208, 13284, 55912, 2404,  16500, 37672,
                48940, 64232, 5976,  39932, 48140, 1140,  27228, 5716,  60488,
                55600, 23048, 36104, 53160, 26780, 10020, 57176, 28308, 21032,
                6832,  38184, 47476, 54116, 3528,  50512, 10668, 61748, 6100,
                13144, 25156, 36088, 8068,  14624, 19036, 28548, 45212, 65168,
                25452, 15260, 49632, 30280, 26612, 57580, 23692, 9240,  65124,
                25300, 57300, 38316, 10488, 28404, 18288, 43044, 7480,  32388,
                48176, 3824,  36360, 30880, 64876, 55928, 2576,  9312,  57392,
                52020, 38244,
            },
            {
                55472, 40684, 33316, 46448, 56324, 21536, 37860, 8352,  31964,
                17032, 10456, 44212, 22792, 5104,  50328, 38328, 56228, 23192,
                51496, 48160, 27600, 20224, 56576, 46188, 59196, 49728, 17784,
                56148, 39972, 21772, 49368, 32524, 19624, 60380, 24132, 31560,
                35780, 44732, 54400, 40056, 47568, 7252,  54956, 10508, 57264,
                44348, 39036, 18584, 56580, 11500, 47704, 5064,  22008, 42820,
                10432, 61040, 52792, 19088, 59380, 29460, 24616, 45544, 20720,
                8700,  25856, 44828, 21716, 14900, 32868, 19820, 45948, 38852,
                17684, 47360, 11908, 64332, 16100, 34344, 2692,  48600, 54800,
                63832, 16672, 24120, 12928, 26484, 43264, 35036, 22932, 38088,
                51520, 64696, 29256, 54716, 60424, 4872,  34952, 12216, 956,
                56384, 35804, 8304,  63184, 37780, 2312,  47592, 16464, 36064,
                41896, 6828,  30604, 61176, 24004, 39988, 64752, 11464, 56688,
                28076, 15968, 9052,  52508, 12952, 40408, 23800, 32048, 16172,
                21336, 47664,
            },
            {
                30808, 13804, 65272, 9480,  5088,  17404, 52480, 47376, 42616,
                64668, 36380, 7076,  54256, 40912, 63288, 1460,  13768, 43220,
                33372, 5560,  61512, 42216, 8028,  13100, 24264, 36512, 43588,
                30032, 12112, 27068, 188,   64212, 9364,  46948, 1888,  16204,
                56636, 6700,  19540, 1096,  25964, 43636, 37228, 49660, 26772,
                60528, 14012, 64464, 2676,  37808, 27924, 61876, 34152, 13676,
                24300, 48108, 27760, 4848,  41500, 49844, 6740,  52596, 57896,
                31824, 2976,  60880, 52188, 35920, 63032, 55032, 10816, 30840,
                2920,  40728, 58020, 6920,  50092, 42680, 62008, 36744, 12092,
                30636, 200,   41168, 32864, 65456, 17736, 2064,  54476, 31148,
                18916, 9548,  40028, 21848, 46784, 42092, 62368, 22664, 39384,
                50824, 3700,  43456, 17892, 54900, 13600, 60016, 28232, 52452,
                3168,  49428, 15776, 44612, 54620, 6212,  34988, 22280, 51220,
                38060, 43808, 61140, 22064, 45564, 18744, 48572, 36980, 63916,
                1628,  11316,
            },
            {
                44620, 19532, 25712, 36988, 44220, 28532, 62728, 14472, 3968,
                20548, 51160, 60876, 14852, 27748, 19888, 35384, 29788, 8724,
                63928, 25056, 15020, 34852, 51892, 65524, 28420, 1680,  5956,
                62508, 15668, 57516, 52260, 41340, 28564, 54244, 37212, 62600,
                26908, 49304, 30404, 65532, 12000, 59432, 16560, 3392,  19408,
                33708, 8292,  29176, 43004, 50376, 20596, 8904,  51952, 30432,
                56736, 15872, 44896, 32380, 9612,  36760, 64808, 34200, 14052,
                42428, 38812, 17224, 11624, 28572, 3604,  50220, 24528, 61528,
                52432, 26296, 19680, 32932, 29708, 23660, 4608,  18660, 25812,
                45104, 59332, 52224, 5404,  56644, 29060, 60300, 14620, 45320,
                58476, 1008,  50184, 16400, 3040,  26040, 53684, 31352, 14248,
                58664, 28800, 23036, 33472, 6484,  40524, 31772, 44768, 21628,
                56516, 12268, 33544, 20916, 2324,  58888, 14896, 46444, 1404,
                19460, 54472, 5240,  34756, 348,   29016, 54604, 5700,  42356,
                27596, 59108,
            },
            {
                50356, 3104,  53128, 57604, 22720, 12136, 34076, 58464, 26336,
                30480, 1632,  24160, 33144, 10964, 48388, 59448, 53868, 17764,
                45900, 54712, 39680, 3956,  30468, 18436, 39016, 54276, 21404,
                47288, 33464, 7392,  36132, 17256, 22444, 12436, 42984, 4444,
                21300, 39176, 14468, 34848, 51716, 22252, 32196, 62528, 40828,
                47980, 52184, 22776, 34964, 15288, 59168, 45228, 1320,  65528,
                36372, 3024,  20440, 62448, 12940, 23216, 17936, 4084,  27344,
                59700, 49636, 23616, 56512, 7060,  42620, 38256, 15852, 7616,
                35276, 13512, 45372, 1444,  60664, 10868, 53980, 40508, 50988,
                7544,  22580, 35732, 20000, 47972, 8452,  40284, 49228, 5156,
                26952, 43136, 33152, 59536, 37572, 10288, 48648, 6788,  20420,
                46628, 9148,  61588, 52156, 48248, 20584, 4104,  10332, 62052,
                38888, 26832, 64060, 51572, 37176, 48660, 31528, 25348, 62616,
                10272, 30540, 24984, 50084, 58748, 10812, 61344, 14272, 51260,
                22644, 35536,
            },
            {
                6360,  16240, 41892, 31556, 1396,  49780, 40460, 6128,  45028,
                56412, 49204, 40212, 58264, 45172, 2912,  6184,  22656, 37832,
                12032, 236,   21184, 59968, 48840, 10904, 44748, 58604, 9564,
                40976, 50760, 24552, 45780, 61604, 5996,  58388, 32948, 51436,
                10820, 45276, 58616, 3884,  42376, 9228,  56320, 6252,  24688,
                12408, 1500,  60036, 54992, 4476,  25320, 39076, 18676, 41904,
                6440,  49520, 38868, 53376, 57308, 42696, 47960, 55464, 10988,
                20056, 632,   34400, 46204, 65052, 19400, 31968, 57708, 21944,
                43380, 63824, 55780, 38024, 21284, 46520, 57464, 14728, 32224,
                63308, 10228, 15156, 44136, 37484, 12204, 24976, 20836, 34688,
                11432, 63732, 23848, 13224, 30096, 56656, 17824, 41156, 64216,
                36508, 16268, 38804, 11740, 25480, 64512, 55536, 35120, 17948,
                128,   42660, 7160,  17352, 10024, 27836, 3940,  42260, 57944,
                39344, 13840, 63868, 41296, 16776, 32620, 26072, 38768, 18144,
                8764,  63136,
            },
            {
                28912, 38864, 59888, 8572,  61732, 19152, 54320, 10164, 17672,
                37688, 12668, 7816,  16796, 36816, 65196, 26088, 42180, 50132,
                31680, 57688, 26720, 42700, 7100,  23496, 33996, 15112, 26932,
                63904, 19328, 11492, 2904,  38992, 30512, 48036, 18704, 24740,
                63488, 7892,  54076, 27816, 18356, 37980, 46584, 29632, 53656,
                44248, 37440, 17840, 10364, 46620, 32400, 13044, 54580, 26972,
                22476, 58848, 29340, 7876,  25676, 1340,  30956, 37836, 44788,
                61884, 29444, 8992,  53368, 25888, 13220, 49024, 968,   47268,
                28380, 4568,  9764,  51324, 16696, 34780, 27088, 6204,  38664,
                48296, 28544, 55868, 61240, 3584,  31312, 63228, 52496, 56336,
                38716, 17996, 51612, 7016,  44744, 60836, 1164,  33772, 51848,
                5468,  26692, 56236, 888,   45332, 29660, 15076, 49884, 24784,
                47032, 30072, 59596, 23736, 44036, 61356, 53408, 17796, 33732,
                5876,  46096, 20820, 36948, 7824,  47128, 4380,  43844, 57852,
                33360, 46632,
            },
            {
                55644, 21160, 13392, 26704, 46920, 36428, 24336, 63972, 32424,
                21976, 62256, 28576, 55044, 20256, 30744, 52680, 15356, 62128,
                8148,  47404, 35908, 16224, 55672, 62348, 51416, 5204,  37492,
                868,   31420, 58972, 54560, 26172, 14360, 53672, 1056,  41092,
                35368, 16140, 32000, 23308, 49488, 63364, 28,    15396, 20932,
                64940, 30836, 26340, 40552, 63120, 6952,  51044, 60816, 9372,
                33232, 12324, 17460, 45704, 35000, 14824, 63112, 21808, 5124,
                51464, 41004, 16036, 37296, 4856,  60480, 40592, 10576, 62520,
                18172, 31696, 41376, 25524, 7408,  65188, 43856, 3148,  58608,
                17800, 1756,  33680, 23416, 50024, 16732, 41668, 1332,  7784,
                29452, 47456, 3320,  55004, 35676, 21640, 28216, 11920, 23768,
                44416, 31196, 60560, 18468, 41256, 8176,  37300, 5096,  60860,
                53772, 14024, 34840, 55304, 39256, 12616, 21292, 8680,  49848,
                55612, 27768, 1872,  53084, 57104, 23212, 65120, 52292, 1208,
                12008, 24476,
            },
            {
                4180,  49324, 34516, 42896, 3804,  15296, 29224, 46156, 84,
                52916, 42980, 4032,  48116, 41216, 6968,  11528, 34816, 2184,
                28672, 19140, 64560, 12508, 32328, 2148,  28804, 47632, 56136,
                22768, 43392, 35540, 16740, 64536, 44480, 9116,  60844, 28716,
                56668, 47740, 2328,  59956, 12852, 34028, 25520, 40324, 57808,
                7812,  3868,  49880, 56140, 21264, 29696, 36080, 16680, 48208,
                44324, 63240, 51808, 2700,  59484, 49760, 9292,  53904, 26280,
                13576, 32288, 58660, 44444, 22108, 28940, 33908, 56256, 24800,
                37220, 54324, 14388, 60240, 48632, 29772, 12288, 20724, 53212,
                25028, 45624, 42140, 7184,  54432, 27552, 59480, 45920, 15448,
                22588, 61464, 40580, 26348, 14744, 49940, 42512, 62016, 53824,
                2764,  13884, 49168, 33348, 53540, 22640, 57528, 43752, 31836,
                9608,  21960, 2196,  48052, 4984,  29388, 64952, 36488, 24272,
                15812, 60928, 11328, 42932, 29932, 13720, 36056, 19928, 30948,
                41632, 62076,
            },
            {
                18712, 54144, 10624, 64792, 56224, 50700, 5528,  39636, 60168,
                9408,  25444, 35264, 14188, 23772, 60436, 45600, 56812, 22148,
                40112, 53072, 4524,  45348, 38496, 20812, 41292, 17432, 61404,
                13552, 52412, 7028,  49148, 4404,  33304, 22996, 37460, 12360,
                5652,  20772, 38624, 43028, 6732,  55680, 9920,  51208, 35776,
                13500, 32700, 60752, 15656, 2792,  43684, 23980, 388,   57032,
                4384,  37308, 24352, 40916, 20032, 28156, 39976, 17176, 35956,
                63716, 6824,  19092, 55412, 2368,  50308, 15680, 7524,  51816,
                2716,  45512, 22708, 33692, 524,   39620, 51680, 35472, 61992,
                30784, 10824, 64744, 13800, 38984, 19852, 9660,  36284, 31992,
                57840, 10400, 33276, 19700, 65296, 8868,  4352,  36848, 19380,
                58992, 39656, 21460, 10512, 3532,  62140, 27704, 12332, 19632,
                38344, 41948, 58132, 15960, 32872, 51828, 45100, 528,   41588,
                31676, 48120, 35100, 18216, 3232,  49476, 9080,  27160, 56208,
                15420, 37132,
            },
            {
                44708, 29964, 772,   23680, 32136, 20300, 58296, 13696, 34300,
                16912, 50232, 57468, 63204, 1060,  33024, 50660, 17188, 43672,
                59412, 10064, 23812, 49792, 26168, 54164, 11412, 6428,  44680,
                29908, 9732,  39956, 27976, 20416, 41792, 50616, 17516, 52904,
                45164, 64768, 26684, 51928, 17920, 30704, 44884, 60452, 18996,
                48360, 23076, 45324, 38228, 11424, 53352, 64408, 40156, 28304,
                21044, 30996, 10544, 56040, 6576,  33740, 61196, 45908, 1088,
                48816, 42936, 24612, 10132, 46656, 64308, 39336, 20540, 42648,
                12116, 61600, 6300,  56052, 18960, 58956, 23976, 15332, 8076,
                37884, 49480, 21532, 57364, 34608, 2656,  51432, 64368, 25716,
                5740,  53436, 44168, 1716,  46700, 58212, 31524, 16032, 46060,
                29112, 7172,  63840, 35964, 47616, 16048, 50888, 1168,  65316,
                52588, 7248,  62772, 26276, 19384, 7952,  59748, 13972, 56904,
                4360,  21604, 64040, 25048, 58428, 38340, 60596, 45420, 5476,
                50412, 8404,
            },
            {
                60256, 40012, 47968, 16656, 38200, 7188,  42292, 26800, 47504,
                21028, 30664, 6460,  39308, 19052, 28460, 8824,  26324, 3716,
                36452, 30228, 15708, 58280, 8012,  63412, 36168, 57152, 24908,
                34468, 18692, 60516, 55660, 12080, 62136, 2580,  58084, 25212,
                31480, 14964, 10212, 35124, 62400, 22028, 2044,  28192, 5880,
                41748, 54588, 8736,  27968, 20072, 47692, 33696, 8432,  14400,
                42976, 54088, 16288, 64772, 43968, 13040, 3796,  23240, 57236,
                11720, 28540, 60136, 35456, 31376, 12716, 26656, 59012, 30068,
                35584, 16420, 47340, 28412, 10848, 43364, 5060,  46676, 55776,
                1180,  17180, 28236, 5984,  47044, 24184, 43384, 12876, 39576,
                49052, 16860, 36540, 12576, 27724, 39324, 24096, 55152, 10204,
                50620, 26228, 56672, 42884, 30276, 24940, 40040, 33620, 46644,
                23524, 29116, 36092, 43688, 56016, 23024, 38172, 26996, 51020,
                9740,  39632, 53812, 6916,  44112, 31932, 16328, 21684, 63196,
                33896, 26096,
            },
            {
                22116, 13664, 57740, 27256, 53564, 62896, 11020, 52228, 2444,
                65028, 43960, 11932, 53344, 46136, 56284, 37472, 64228, 13688,
                48368, 61740, 34048, 1604,  41844, 19332, 31540, 49004, 2000,
                64884, 46300, 3676,  23556, 47560, 29676, 35736, 8592,  40312,
                548,   48924, 55384, 4664,  40968, 48072, 15996, 38488, 63884,
                25340, 836,   34656, 62380, 57596, 5520,  18084, 59760, 49872,
                62240, 5780,  47348, 35748, 26044, 50972, 30188, 54728, 32848,
                20580, 38940, 51392, 3660,  17880, 53328, 288,   44632, 5444,
                63524, 25932, 40196, 50956, 37352, 64224, 31632, 27004, 41740,
                60568, 33092, 40376, 62636, 52880, 30056, 18276, 55468, 80,
                21704, 29792, 63392, 56428, 21040, 52288, 6084,  63072, 41720,
                33100, 268,   17904, 13260, 4564,  55604, 9260,  59560, 17652,
                13088, 2636,  49964, 10564, 3652,  48696, 34108, 17912, 62084,
                30008, 46752, 14768, 28360, 11036, 54988, 156,   39516, 12460,
                53280, 3136,
            },
            {
                36340, 63584, 4952,  9340,  33864, 45336, 23012, 36872, 58780,
                5272,  27668, 36320, 22512, 15608, 2788,  41712, 52120, 20924,
                6568,  54696, 44312, 27100, 52204, 13744, 4988,  40456, 21692,
                53036, 14196, 37612, 32640, 6544,  15816, 44568, 63696, 20192,
                34072, 60716, 23360, 28900, 13256, 58588, 53756, 32180, 12188,
                57116, 17504, 49400, 13308, 41520, 31172, 26628, 36720, 11140,
                24892, 32796, 2504,  22044, 8376,  60108, 18340, 41876, 5220,
                65384, 15692, 6408,  41684, 62544, 37384, 23020, 49216, 56620,
                19268, 53232, 8800,  21784, 1864,  13968, 18156, 53612, 10208,
                22968, 50848, 12392, 3832,  15412, 59732, 7104,  33824, 45648,
                58804, 4804,  41204, 7608,  48144, 17540, 35156, 3380,  14412,
                22236, 38396, 52744, 60656, 45152, 19932, 36944, 5392,  44104,
                57112, 32208, 61096, 15308, 39812, 64236, 11700, 2008,  43216,
                20320, 5136,  36916, 61156, 19588, 50248, 24172, 47332, 29692,
                42868, 17328,
            },
            {
                31108, 45756, 51180, 40596, 19316, 1536,  14752, 31404, 17976,
                40764, 55420, 8324,  49056, 61080, 31072, 11096, 24084, 32768,
                40396, 17620, 11588, 22392, 46916, 61548, 28112, 16376, 58332,
                9272,  26824, 43020, 59764, 51840, 56764, 26448, 11612, 54000,
                43516, 16932, 8196,  45368, 36808, 2992,  24232, 9108,  50868,
                43452, 29808, 37428, 21556, 2356,  51724, 44988, 3848,  55016,
                39424, 17288, 53020, 58544, 38156, 48312, 14676, 36640, 9912,
                52912, 47144, 23720, 27400, 55224, 8208,  32604, 14928, 10592,
                34372, 4512,  30552, 60824, 57460, 49100, 36252, 62396, 3032,
                19292, 58784, 44232, 26224, 37360, 41988, 10276, 63068, 27504,
                13728, 51620, 24776, 32692, 11284, 60760, 28920, 44400, 58808,
                47660, 65004, 7404,  24036, 32292, 48952, 64004, 28480, 51760,
                25040, 41184, 19136, 53844, 27624, 30868, 45500, 55208, 25252,
                58068, 32572, 52216, 2212,  41364, 34556, 64396, 4932,  58944,
                8944,  56432,
            },
            {
                1124,  23992, 11984, 28840, 60868, 55636, 48268, 62108, 24832,
                50948, 14236, 33208, 58148, 25704, 5208,  62944, 47612, 57028,
                308,   29468, 65408, 37928, 8956,  34796, 54916, 44156, 37108,
                30352, 49716, 120,   17752, 39664, 21600, 3124,  49972, 29364,
                4324,  38380, 57676, 51508, 19412, 65048, 34016, 46460, 21180,
                3480,  61828, 7612,  55100, 65304, 23840, 15576, 58640, 20964,
                46608, 28704, 42136, 12328, 27444, 124,   62576, 24920, 44524,
                29484, 34348, 58884, 13568, 45568, 20208, 61348, 38872, 24996,
                46792, 41392, 15280, 44144, 33624, 23732, 7400,  45452, 39332,
                30380, 35296, 8564,  56444, 19840, 31936, 23112, 49952, 38360,
                19116, 35764, 62044, 43676, 920,   38792, 50180, 25072, 18928,
                10880, 34456, 27996, 40548, 11884, 1464,  22712, 15468, 10464,
                37712, 7684,  47072, 22584, 4708,  58864, 16432, 7056,  40356,
                12548, 65360, 17192, 26396, 57220, 8128,  14256, 37148, 26896,
                20760, 48652,
            },
            {
                15064, 54328, 35144, 43640, 6716,  26080, 39020, 3632,  9468,
                45572, 20432, 860,   42996, 18308, 35532, 44796, 15880, 8664,
                60248, 24704, 49308, 3516,  58800, 20736, 1232,  11856, 63780,
                6924,  20296, 61896, 12184, 30792, 8912,  36820, 59952, 14264,
                24648, 62580, 31692, 11404, 26432, 42720, 6628,  60128, 40168,
                13832, 25896, 46824, 32960, 11288, 38436, 7228,  34104, 63496,
                1264,  9168,  61584, 19576, 45848, 32020, 53744, 4260,  57888,
                18716, 1388,  8816,  50368, 30952, 2604,  43016, 52280, 59248,
                1576,  65112, 54860, 19648, 3744,  28636, 12784, 52440, 25620,
                15696, 48888, 812,   46392, 63672, 52168, 2244,  43940, 56832,
                3888,  8948,  16184, 54304, 22840, 13856, 55628, 8232,  31152,
                51908, 2028,  16136, 56044, 50924, 58512, 42348, 34696, 54680,
                60120, 732,   63176, 34228, 9568,  37168, 51976, 21228, 35068,
                50112, 8596,  44548, 48448, 21488, 31036, 45952, 18348, 51856,
                61728, 38968,
            },
            {
                6316,  65096, 17552, 58288, 21632, 33008, 52828, 12968, 35016,
                56520, 29948, 63852, 39208, 12572, 54192, 28720, 20332, 37524,
                42908, 33804, 53196, 15428, 41220, 31744, 50636, 23864, 47300,
                33460, 56232, 25316, 53640, 45092, 65252, 47928, 32928, 41888,
                19736, 7120,  47584, 1752,  55760, 14584, 30500, 17820, 53580,
                34564, 58904, 16520, 42060, 28944, 48504, 53532, 43276, 13000,
                31104, 49300, 35128, 56988, 6656,  39928, 21528, 11600, 49724,
                39508, 62024, 22032, 36464, 64400, 17436, 5564,  28956, 13456,
                21912, 36496, 26648, 10736, 48196, 56648, 41656, 59816, 5176,
                64960, 55028, 23624, 13944, 28644, 5900,  17360, 12488, 25892,
                47720, 59540, 31340, 27784, 46172, 62988, 2928,  36620, 57388,
                42764, 61268, 46092, 36264, 19068, 8500,  31188, 3376,  48340,
                20748, 30224, 14288, 51152, 42788, 25696, 63536, 46856, 492,
                28828, 23788, 38076, 3332,  11520, 60712, 55572, 1640,  42384,
                11164, 33384,
            },
            {
                25984, 41020, 2840,  47040, 10156, 16080, 64352, 42132, 27332,
                59932, 4140,  23388, 47932, 6508,  59544, 1916,  50580, 55652,
                4144,  10980, 19488, 45928, 5688,  26584, 61216, 17496, 4904,
                39168, 14908, 41736, 35132, 5904,  23756, 17116, 952,   56912,
                52116, 44176, 34968, 22256, 39352, 61656, 48892, 992,   28080,
                9428,  51288, 4660,  20184, 300,   60944, 17832, 27348, 23424,
                55992, 16072, 4504,  24404, 51696, 16436, 63956, 35516, 26880,
                15144, 43536, 55752, 12156, 40836, 25492, 57804, 48676, 9032,
                32080, 50980, 6572,  39896, 62692, 35248, 17428, 31900, 21104,
                11084, 42516, 33176, 39744, 59452, 36304, 54936, 33416, 62364,
                20804, 41692, 51120, 5828,  19336, 33228, 41808, 21100, 15384,
                4528,  26108, 21820, 5360,  60360, 26904, 44944, 62924, 16956,
                25900, 40392, 45584, 56596, 18000, 2952,  13316, 31808, 54368,
                58724, 14820, 61860, 53104, 27496, 40148, 33704, 24312, 7384,
                29616, 57412,
            },
            {
                19164, 52192, 31308, 36940, 50780, 552,   45096, 19172, 7376,
                48960, 16764, 36904, 52424, 26740, 32448, 39884, 22844, 14124,
                26516, 62860, 30192, 56612, 37056, 12956, 54228, 42952, 28952,
                59416, 9828,  48976, 3280,  28772, 55588, 40236, 9772,  27268,
                12412, 64340, 16248, 28744, 51080, 9760,  36024, 23740, 57540,
                38348, 43900, 63216, 24932, 55688, 35396, 9600,  5080,  51348,
                41084, 37692, 62984, 44392, 9784,  29592, 42940, 54704, 7668,
                32576, 4892,  28652, 46708, 6620,  34256, 54480, 37924, 18988,
                46032, 60416, 16640, 53520, 22516, 60,    9060,  47096, 37816,
                28364, 51264, 4176,  9672,  21244, 49036, 41332, 8340,  29412,
                38036, 2988,  11848, 36908, 57836, 9164,  48588, 29428, 63688,
                38684, 49692, 12540, 30504, 39472, 14736, 53024, 11748, 36544,
                59020, 4968,  11120, 23540, 29476, 60464, 39356, 9152,  19112,
                42176, 5040,  34132, 43644, 16848, 5800,  49504, 63804, 16020,
                44284, 47916,
            },
            {
                4672,  12260, 23016, 63040, 26488, 59068, 30652, 23896, 38292,
                54028, 11384, 44468, 8092,  15300, 65008, 9928,  46604, 61056,
                35744, 44560, 7672,  23232, 64196, 46968, 9252,  35816, 1836,
                51948, 18784, 22460, 62700, 13860, 50772, 20380, 61504, 37088,
                31100, 4704,  53900, 59128, 3864,  19064, 45976, 64440, 6148,
                21668, 12020, 31664, 49920, 14300, 39572, 45732, 64652, 32804,
                1972,  20632, 27076, 13516, 33860, 60160, 1792,  19800, 47784,
                58680, 24108, 52608, 16012, 60740, 10348, 22540, 984,   63508,
                41848, 3088,  29960, 12440, 45112, 27532, 50744, 57656, 3060,
                60832, 18696, 62764, 45148, 15516, 24956, 64484, 480,   46428,
                14960, 55308, 25244, 65260, 15952, 52516, 25820, 12140, 54084,
                7328,  33564, 62244, 55332, 47252, 4148,  23764, 41544, 7352,
                50592, 32636, 64932, 37588, 7168,  53704, 44816, 26876, 62428,
                36748, 22316, 9852,  30484, 58360, 20780, 12444, 37028, 52804,
                21764, 62344,
            },
            {
                38572, 55240, 43360, 14456, 7736,  40176, 56024, 5108,  61760,
                31880, 21924, 62848, 34528, 57448, 19308, 42040, 29172, 6248,
                17916, 51624, 39756, 716,   16300, 32756, 21196, 57912, 25436,
                64708, 32040, 46168, 37812, 58764, 33524, 6696,  45828, 49708,
                21964, 40900, 8864,  25020, 43328, 12588, 31440, 41324, 15336,
                56032, 47380, 36592, 7928,  59992, 22372, 29624, 19484, 57408,
                11340, 59736, 46484, 54420, 3916,  49440, 25500, 37104, 10840,
                64988, 512,   38004, 20660, 50072, 31296, 45252, 27340, 14904,
                34856, 24412, 58440, 38720, 33020, 64504, 14600, 24708, 43712,
                13216, 35976, 26724, 56504, 6488,  31208, 52896, 18472, 23608,
                60476, 49684, 34060, 44132, 1128,  39272, 60060, 3952,  45480,
                23264, 16836, 43348, 9392,  20128, 63888, 33296, 57492, 18924,
                28008, 54104, 1704,  47848, 20628, 34364, 50064, 15936, 2264,
                47564, 55364, 50424, 63148, 816,   46908, 54864, 27188, 2588,
                34884, 9216,
            },
            {
                59276, 27032, 3200,  49548, 35216, 17864, 10932, 46068, 14888,
                2640,  41760, 29048, 396,   24676, 49528, 3800,  53440, 56944,
                33080, 12048, 59792, 49064, 28108, 52900, 3820,  40940, 12484,
                43936, 7324,  27376, 1240,  10692, 42084, 24988, 14860, 1876,
                58560, 18316, 47188, 37908, 33616, 60084, 53300, 26244, 2084,
                29700, 18556, 3784,  27140, 43148, 2232,  54176, 13368, 48264,
                42728, 7760,  30756, 17524, 40260, 21928, 56716, 14020, 41096,
                17660, 44976, 33580, 62932, 3560,  42340, 59436, 52296, 11356,
                56420, 4948,  49456, 18748, 6944,  55616, 20672, 40480, 7840,
                53220, 32060, 1252,  47628, 37400, 58892, 12872, 35356, 43236,
                10000, 6976,  19596, 29156, 22564, 47156, 32112, 18500, 36000,
                58540, 28048, 51740, 2864,  36972, 28856, 49348, 612,   45700,
                22192, 13816, 43284, 16688, 58812, 12232, 4780,  24068, 57636,
                29524, 12972, 18532, 26024, 38640, 32528, 8580,  41192, 57748,
                14088, 31760,
            },
            {
                46528, 16712, 30452, 61488, 21420, 53336, 27764, 49200, 36160,
                57752, 51484, 17656, 55104, 45412, 36236, 13596, 22320, 38780,
                2540,  25660, 20024, 42692, 8220,  38260, 61744, 18252, 50200,
                35072, 15944, 56744, 52240, 18044, 30240, 54628, 63660, 35140,
                26888, 55188, 13328, 63432, 0,     21148, 8144,  48128, 61232,
                52284, 40844, 63908, 57092, 17084, 51136, 33912, 5844,  26412,
                36188, 23844, 50428, 64468, 35444, 5808,  61452, 31764, 51876,
                28984, 55484, 9704,  26632, 13436, 18328, 6116,  38540, 21380,
                46376, 28452, 62252, 10468, 42768, 2040,  35500, 29420, 59552,
                48496, 22816, 16180, 41828, 10096, 28000, 4756,  50440, 57524,
                32152, 40108, 62496, 5144,  56808, 13708, 8296,  42012, 64524,
                312,   11468, 40780, 24848, 58100, 15404, 10784, 38304, 61128,
                8436,  35852, 62160, 25624, 40084, 30632, 63024, 43448, 33056,
                8248,  40624, 6140,  45268, 15008, 23040, 65192, 17980, 44808,
                24544, 50652,
            },
            {
                20316, 41832, 6068,  56896, 44532, 1216,  32708, 65416, 19712,
                25848, 8952,  12776, 39228, 6632,  59756, 31336, 62208, 16624,
                47776, 63636, 30536, 57536, 13772, 24340, 55792, 30780, 5812,
                23676, 60828, 40036, 21464, 48332, 60520, 3340,  38988, 10396,
                43508, 32248, 6808,  49864, 29408, 56664, 17476, 38616, 34724,
                10056, 22688, 32884, 12376, 37516, 9040,  62832, 40652, 60396,
                15316, 56356, 368,   11004, 27984, 43316, 8552,  45980, 3192,
                22924, 6904,  39700, 47500, 57360, 36296, 29292, 65320, 8264,
                17244, 53760, 36344, 22684, 47540, 52784, 61620, 17276, 4668,
                11112, 55532, 65428, 19776, 51640, 61200, 39140, 21092, 1772,
                27232, 53388, 16800, 48684, 37704, 61052, 24972, 49772, 29860,
                20808, 47576, 54360, 31908, 7196,  44140, 53560, 17456, 30960,
                55416, 50016, 29240, 2816,  56056, 9616,  51796, 38148, 20264,
                64552, 53596, 35652, 59396, 56124, 3424,  48292, 29584, 5964,
                61188, 336,
            },
            {
                64148, 34216, 9936,  37744, 12868, 23552, 40824, 8200,  4168,
                43904, 61092, 33240, 47824, 20832, 11012, 26292, 50932, 8704,
                44432, 5364,  36436, 52316, 1980,  34496, 44580, 9960,  48028,
                54260, 4252,  12108, 33168, 7888,  44116, 23344, 16864, 52972,
                4328,  60952, 22944, 15628, 36556, 44440, 4560,  24560, 14076,
                45220, 6244,  54564, 48324, 25528, 45404, 22980, 18140, 31000,
                4112,  38824, 20944, 53892, 15832, 48688, 19168, 26572, 62664,
                35936, 15496, 60292, 21992, 53052, 1892,  24032, 50160, 43368,
                33424, 576,   40648, 15272, 31160, 25240, 12604, 45644, 32748,
                38508, 25748, 44368, 4040,  33660, 24112, 16004, 46616, 64016,
                13428, 24448, 44552, 11088, 34424, 3236,  53288, 15592, 56396,
                6364,  37736, 13848, 60124, 22040, 65016, 26268, 41232, 5620,
                24268, 11820, 42212, 6644,  21580, 46524, 17808, 140,   14328,
                46860, 23728, 2080,  27808, 42024, 33932, 10520, 53444, 39800,
                35704, 11792,
            },
            {
                28816, 52376, 25160, 48452, 62960, 16320, 54812, 58788, 30568,
                37916, 23072, 56168, 28032, 64628, 43224, 1516,  40960, 33596,
                55528, 22972, 11064, 18056, 47224, 65292, 21152, 15544, 62604,
                28888, 36704, 45652, 64268, 28096, 13528, 34248, 50452, 29336,
                20012, 46048, 39796, 26636, 52152, 11436, 64828, 57932, 50260,
                27960, 61964, 20276, 3420,  30176, 58268, 1292,  48904, 52852,
                43040, 63460, 46056, 33440, 60316, 37660, 52468, 13004, 57800,
                41996, 49592, 30428, 4996,  44608, 32624, 41164, 14180, 55816,
                60540, 25852, 7368,  63476, 56540, 3896,  39372, 51248, 20076,
                62320, 7304,  29040, 13636, 40620, 55896, 7796,  30516, 42232,
                36960, 2884,  55548, 31116, 21272, 41196, 27796, 10500, 44672,
                33700, 62912, 18300, 1852,  35644, 50432, 3768,  33640, 48308,
                63232, 19176, 58080, 32476, 53092, 36032, 26284, 60660, 28780,
                51088, 11116, 61424, 19284, 13424, 49888, 20828, 59716, 15376,
                22748, 54940,
            },
            {
                7460,  17812, 60076, 2616,  29456, 35616, 50916, 11568, 46932,
                14648, 50384, 6172,  3140,  16120, 52416, 57892, 19004, 28164,
                14324, 39588, 58596, 28604, 41180, 4992,  26716, 39116, 2508,
                42192, 18632, 25544, 260,   53276, 57988, 5508,  41620, 65512,
                12564, 57504, 1528,  55620, 8424,  30876, 21576, 40484, 32064,
                1484,  42264, 16444, 39200, 65508, 14532, 35064, 10072, 27572,
                12748, 6804,  25232, 29768, 5212,  1988,  24048, 32228, 9956,
                1408,  19944, 63876, 11852, 17076, 59316, 9844,  20104, 4228,
                30476, 12036, 44696, 49652, 19256, 9324,  27920, 57848, 1412,
                35728, 49436, 53556, 59096, 600,   48344, 62200, 11000, 19216,
                50240, 59788, 8408,  64612, 51528, 18644, 62176, 2024,  39820,
                23328, 51428, 26660, 42256, 46328, 12528, 20140, 59112, 14808,
                1260,  44928, 38384, 15792, 65264, 12480, 41212, 56920, 5848,
                34920, 40060, 31592, 52620, 7604,  62432, 38048, 4108,  26744,
                43388, 47052,
            },
            {
                38480, 13408, 41488, 20180, 46164, 5420,  26120, 21216, 1608,
                63700, 18772, 41372, 35952, 31800, 23596, 37240, 7004,  64012,
                48184, 144,   61576, 32536, 9064,  51852, 59724, 33124, 57012,
                11184, 51268, 59032, 15652, 38896, 20528, 46440, 26056, 8748,
                37512, 33328, 17740, 62300, 42204, 47244, 3348,  18264, 53252,
                8788,  59308, 34224, 52208, 7096,  43920, 21312, 55248, 61540,
                36632, 19224, 58320, 13952, 42200, 55592, 64720, 39088, 47092,
                54220, 34396, 25644, 38520, 55108, 27956, 48500, 62592, 36124,
                52080, 21888, 58792, 37224, 32560, 42424, 64712, 22528, 15228,
                41628, 9588,  18088, 32004, 21932, 35268, 26452, 54232, 5100,
                33788, 22832, 28552, 14540, 4736,  45796, 56964, 30796, 16748,
                59208, 4488,  9560,  30584, 56732, 7936,  28992, 39440, 54528,
                35372, 27016, 8888,  48956, 23572, 3288,  30940, 9156,  44636,
                21608, 16968, 4480,  43836, 24460, 29100, 46388, 32744, 9320,
                56852, 1760,
            },
            {
                59024, 50552, 33192, 55932, 8828,  39452, 61784, 43000, 34264,
                28972, 53260, 10180, 62020, 48880, 13444, 45880, 10700, 53004,
                25276, 19952, 45040, 15340, 23904, 48576, 12656, 19148, 45984,
                7500,  22956, 32216, 49220, 9424,  35996, 61932, 2128,  54096,
                22088, 48680, 6056,  24504, 34488, 14788, 60328, 26480, 36328,
                12904, 45760, 24252, 10884, 27164, 49656, 40884, 5504,  31688,
                46668, 52012, 9264,  49212, 34616, 21664, 7648,  18032, 28628,
                14632, 43524, 8080,  50740, 780,   34872, 7240,  25336, 39448,
                46972, 17108, 28592, 1900,  14396, 53236, 5084,  47952, 30580,
                55656, 24284, 45532, 64244, 6596,  14836, 38628, 43956, 58168,
                16316, 47304, 39692, 43128, 35300, 24828, 7984,  36880, 46936,
                12672, 54700, 48908, 38732, 16496, 61152, 51772, 23200, 10244,
                47176, 21112, 61432, 5712,  42664, 59752, 50128, 15552, 54060,
                62628, 48744, 58308, 35876, 55808, 660,   16804, 52016, 64616,
                18920, 30628,
            },
            {
                22072, 4312,  27048, 64860, 14952, 31344, 54080, 17172, 57512,
                7356,  45200, 25736, 20536, 56524, 2048,  60756, 30140, 34748,
                4028,  37752, 54024, 6764,  56280, 36692, 1080,  29884, 53364,
                35044, 63388, 3484,  43036, 29620, 55876, 18104, 31852, 14552,
                59356, 43748, 30052, 11960, 50964, 38180, 54928, 5624,  63976,
                48488, 21428, 56212, 31292, 61256, 18212, 57868, 25616, 15580,
                752,   23084, 39552, 62920, 16604, 44900, 50172, 61048, 5380,
                56800, 21356, 61724, 45708, 18588, 64596, 42496, 15480, 2672,
                10292, 54600, 6320,  61904, 24700, 45956, 10644, 34448, 59720,
                5872,  12996, 37280, 27732, 42028, 53048, 20524, 2172,  29828,
                62476, 10200, 72,    58988, 49568, 11796, 53676, 21540, 65380,
                28104, 33284, 20444, 63424, 24524, 1472,  32716, 43484, 63848,
                4532,  31232, 55676, 34708, 28852, 18940, 37760, 24736, 33064,
                1428,  26408, 9948,  14548, 63484, 40184, 23096, 12276, 41260,
                48860, 36180,
            },
            {
                61940, 11208, 43612, 23440, 48772, 152,   10496, 24472, 37536,
                12920, 59980, 39752, 5312,  28588, 43616, 18388, 40644, 58508,
                49768, 11868, 29232, 42072, 64324, 21304, 43992, 61804, 25024,
                16616, 40424, 13300, 59904, 5192,  24072, 11640, 50792, 40904,
                27416, 3704,  53124, 63704, 19552, 504,   22928, 43888, 29488,
                17148, 38856, 840,   6412,  36600, 13132, 3368,  38692, 63988,
                44152, 56836, 28976, 3496,  26344, 10996, 30884, 40676, 25012,
                36992, 3412,  32376, 29528, 13252, 23124, 52492, 57908, 32140,
                60700, 43460, 35008, 40380, 56048, 20612, 38412, 17688, 26736,
                50236, 61624, 3636,  47520, 10316, 60856, 49616, 13072, 24796,
                37244, 51728, 21036, 27104, 18108, 32648, 60872, 1192,  40920,
                7108,  43832, 2852,  14120, 45156, 37100, 6160,  18396, 13940,
                40788, 50976, 17216, 11548, 52844, 2112,  46716, 64160, 12736,
                42560, 38748, 30236, 20464, 47460, 8548,  34476, 59424, 6092,
                25372, 14800,
            },
            {
                45836, 7264,  52948, 37036, 18196, 60512, 45448, 51844, 3776,
                31904, 48180, 15840, 34668, 54172, 9084,  24556, 14784, 5684,
                21900, 62800, 16716, 33648, 26652, 4264,  14660, 38536, 5612,
                55728, 46740, 20368, 27020, 48232, 64356, 44888, 35088, 7344,
                61564, 16760, 39368, 8936,  45216, 28144, 58292, 10684, 33156,
                7716,  50704, 62648, 42360, 53488, 47184, 32764, 50568, 20688,
                11580, 33968, 6132,  59972, 37452, 52200, 2252,  58364, 12176,
                47548, 52372, 9524,  58960, 40172, 4024,  46316, 20284, 28296,
                50360, 23796, 18684, 12128, 48700, 29680, 62688, 280,   44192,
                39996, 19844, 32944, 57532, 17284, 31176, 5952,  57044, 44836,
                7656,  31660, 56000, 63592, 6372,  38912, 15536, 51360, 25688,
                18512, 35960, 58256, 52072, 27124, 56144, 47868, 60056, 28276,
                57372, 340,   25612, 44456, 58488, 22440, 8428,  55576, 4852,
                18552, 60792, 50336, 54816, 2632,  27516, 53180, 44576, 29928,
                2336,  54688,
            },
            {
                33432, 39880, 29724, 3680,  57204, 34440, 27456, 40692, 64448,
                19832, 23724, 2736,  62652, 49924, 37644, 65180, 47792, 33060,
                43012, 56948, 1620,  46984, 9892,  60244, 52484, 48932, 10724,
                28416, 1584,  36752, 53480, 16852, 39488, 696,   54668, 20152,
                47260, 25308, 35796, 56216, 31912, 49928, 14476, 41276, 54052,
                59612, 25908, 13904, 19620, 28664, 22780, 9748,  60304, 27804,
                54140, 18128, 42604, 47232, 20308, 55144, 15360, 33780, 19120,
                63756, 41640, 16608, 26476, 55368, 33852, 11168, 37552, 7768,
                14048, 1496,  65216, 32772, 8344,  4284,  58032, 15604, 53600,
                8528,  23080, 51656, 1160,  25596, 40088, 22264, 34864, 65032,
                16868, 41400, 13644, 3712,  45560, 29024, 43308, 10076, 56496,
                47372, 62532, 10764, 31348, 8268,  21140, 11880, 34600, 23920,
                37968, 9204,  64636, 36076, 14580, 40232, 31780, 26948, 35568,
                45820, 23224, 6124,  33272, 43404, 18228, 61192, 20976, 37872,
                58056, 17868,
            },
            {
                49596, 63356, 20480, 12840, 42104, 7728,  21392, 14232, 6356,
                50208, 56976, 43076, 11680, 30804, 20988, 596,   27840, 53448,
                8708,  25380, 39252, 51036, 19652, 35856, 30852, 23196, 42432,
                65140, 58176, 31640, 11016, 7152,  33388, 22740, 28740, 10532,
                58184, 1764,  13476, 21904, 6220,  60808, 2492,  36560, 20980,
                3720,  45008, 34972, 57488, 4184,  64676, 45204, 1896,  36220,
                7336,  48896, 14084, 31828, 24244, 65328, 7856,  44428, 29092,
                23436, 1592,  35488, 48900, 6636,  61124, 24992, 53680, 62792,
                41160, 45396, 57212, 27132, 52212, 42276, 25400, 35324, 31028,
                64592, 14092, 36300, 43336, 63316, 53968, 46748, 4196,  27612,
                53432, 47976, 23472, 35448, 50748, 55292, 22184, 34592, 4508,
                29580, 15784, 23576, 42404, 38944, 64856, 53568, 3512,  50276,
                46264, 19580, 32148, 49296, 5860,  61276, 50604, 15928, 59312,
                52996, 9472,  14280, 65404, 37408, 10960, 4556,  13908, 47316,
                9136,  27628,
            },
            {
                760,   15548, 59004, 25532, 53876, 47876, 61712, 30356, 54532,
                33828, 38168, 27372, 7332,  17144, 57968, 45184, 12532, 61420,
                18792, 31252, 13496, 55492, 41388, 6864,  57648, 17804, 34080,
                7752,  21552, 50656, 41316, 60392, 56304, 13996, 62780, 38356,
                31140, 42684, 64280, 48704, 40104, 18012, 24772, 64824, 47944,
                16684, 30724, 9400,  49364, 38068, 16976, 41036, 14376, 55988,
                23432, 38996, 62436, 9688,  1236,  41456, 36520, 49536, 4972,
                55848, 59680, 13024, 21548, 43736, 18020, 224,   47856, 30560,
                19612, 36004, 6076,  16708, 37668, 21852, 49612, 11560, 46464,
                5072,  56568, 28504, 11092, 7048,  15264, 37800, 12028, 19828,
                1656,  38472, 10416, 59496, 18980, 8104,  63048, 13916, 59288,
                38052, 53716, 5292,  49752, 908,   17616, 29188, 41568, 7272,
                15732, 60412, 42840, 3008,  28688, 20576, 10916, 43344, 1120,
                29956, 40812, 57664, 28268, 24376, 56360, 50984, 31360, 64284,
                40880, 23304,
            },
            {
                56008, 34740, 45308, 9700,  31840, 1932,  16652, 44332, 10944,
                856,   18384, 46724, 59836, 53060, 33560, 39544, 4464,  50136,
                35468, 59044, 2768,  23504, 64116, 27820, 376,   13184, 52984,
                45588, 15348, 4056,  24952, 18648, 43260, 49160, 4832,  51644,
                17696, 53788, 27544, 9868,  34584, 52796, 44592, 27948, 11716,
                39112, 63056, 23480, 54836, 6792,  32336, 25280, 59072, 30156,
                50796, 4768,  57004, 28344, 51472, 17548, 26564, 61700, 10864,
                39012, 45960, 31228, 51996, 64992, 29152, 39824, 15456, 58844,
                4376,  24288, 47336, 54828, 13560, 63644, 2276,  60332, 19060,
                24456, 39148, 49356, 59184, 21248, 30144, 61824, 49232, 33808,
                60784, 56468, 30488, 26092, 848,   39848, 27500, 49540, 2104,
                44820, 19508, 61680, 33604, 45748, 13388, 61120, 32956, 56724,
                22696, 54108, 12568, 24812, 56820, 46536, 36596, 63736, 25036,
                19544, 48828, 5152,  16620, 46636, 41756, 22016, 35164, 3196,
                52312, 11912,
            },
            {
                29084, 6396,  38796, 50508, 65204, 36352, 23256, 39052, 57864,
                25200, 63052, 13356, 41380, 2372,  25796, 22872, 15700, 42148,
                7420,  46372, 37792, 16996, 10852, 43440, 47396, 37192, 59164,
                26156, 39128, 61716, 47748, 30436, 2572,  26392, 36008, 45360,
                7632,  23584, 3596,  57352, 15500, 30124, 4696,  8300,  51004,
                57816, 972,   41940, 27720, 12364, 61848, 48032, 18720, 10648,
                43104, 20492, 34712, 45024, 58772, 12544, 32440, 53764, 21012,
                16252, 27240, 8356,  4316,  36536, 11752, 55888, 8544,  34288,
                50996, 11456, 60588, 30384, 43632, 9396,  33188, 40332, 55220,
                43980, 3264,  16960, 34112, 45176, 2072,  55452, 24304, 42832,
                7468,  16124, 44316, 64544, 46988, 33448, 17168, 42324, 24164,
                32012, 12304, 26784, 57072, 21824, 51832, 25744, 10092, 37172,
                2260,  29992, 39056, 62756, 17160, 7536,  54516, 4212,  32600,
                12512, 37796, 62260, 33720, 68,    8204,  61900, 15704, 26188,
                59524, 44076,
            },
            {
                17264, 62212, 22672, 3316,  18752, 27656, 59908, 48516, 8240,
                51372, 29820, 22020, 36548, 9996,  51084, 64252, 57108, 28260,
                20708, 62224, 29972, 49244, 54048, 32176, 20340, 62884, 5172,
                9860,  33032, 54624, 11952, 37000, 64784, 9576,  21188, 15232,
                61628, 32068, 47088, 41564, 20788, 62392, 55020, 35992, 22352,
                32168, 14980, 51964, 19252, 43964, 2844,  53100, 35344, 136,
                63260, 25904, 16168, 3208,  38528, 22724, 6036,  47012, 544,
                34936, 63420, 54556, 40972, 19676, 49588, 25680, 42944, 21956,
                64320, 27664, 39888, 1028,  19988, 57860, 27440, 6840,  14588,
                29832, 61492, 9044,  26008, 52656, 40820, 17600, 9768,  28448,
                50528, 21128, 5280,  14128, 54092, 11248, 57420, 6332,  53088,
                63936, 9160,  40588, 6608,  36844, 3992,  42972, 19188, 63952,
                45856, 50712, 9412,  34500, 42152, 30712, 22848, 39864, 51448,
                56940, 44300, 21444, 50244, 53952, 29664, 39320, 45968, 6032,
                20020, 37196,
            },
        },
        {
            {
                1628,  28332, 16116, 53948, 10080, 2548,  51788, 39768, 57384,
                43908, 54736, 20824, 2800,  13336, 29628, 58084, 35744, 10892,
                21508, 33304, 57344, 4680,  41272, 22360, 32224, 3692,  18636,
                64292, 1184,  57048, 27580, 16216, 6060,  24108, 56316, 7652,
                63460, 13672, 35880, 56740, 9164,  33604, 22856, 37828, 25628,
                5580,  32540, 10076, 64564, 29848, 35000, 54728, 5356,  33532,
                18612, 1376,  23168, 35332, 63340, 49824, 22556, 29480, 34656,
                876,   45984, 28764, 57588, 31596, 60904, 45264, 22912, 59408,
                6884,  52576, 21560, 9020,  50672, 45472, 31056, 54240, 40316,
                14340, 50244, 9952,  45136, 15340, 48304, 5028,  58340, 8352,
                46472, 57088, 39196, 17640, 21596, 51876, 47368, 36532, 57068,
                4464,  18192, 45952, 56672, 20872, 36492, 44508, 3796,  24028,
                8764,  42584, 1936,  56100, 22436, 41636, 29328, 52444, 8384,
                57656, 13972, 34720, 28872, 62236, 51032, 752,   43584, 17516,
                65184, 22412,
            },
            {
                52448, 44800, 6844,  37048, 29444, 45524, 34588, 62412, 19188,
                8848,  15104, 41324, 33996, 26448, 44764, 48444, 4244,  63372,
                27904, 43928, 12844, 64424, 50392, 15068, 61300, 56476, 34872,
                25064, 48232, 61472, 11132, 51864, 44808, 36156, 28780, 19540,
                49672, 26736, 47036, 968,   40124, 17384, 4412,  61340, 19388,
                41748, 56448, 22076, 39192, 12704, 2012,  20560, 43632, 63636,
                13148, 57080, 44652, 30832, 16664, 46312, 10072, 42892, 64700,
                54456, 12640, 19440, 51132, 6100,  26608, 13868, 10592, 36524,
                42388, 2292,  31824, 40612, 36152, 23236, 16772, 7604,  26320,
                65000, 4656,  33720, 54492, 63484, 24332, 12620, 51200, 29340,
                15548, 64948, 11624, 25732, 3076,  60036, 33524, 13132, 21968,
                48628, 29124, 34100, 5632,  25932, 59204, 51148, 19456, 34668,
                60464, 27772, 18008, 33292, 5764,  49544, 36984, 2752,  17248,
                24048, 65020, 40776, 48608, 7092,  20444, 38160, 57436, 30148,
                11356, 33972,
            },
            {
                39848, 24836, 63696, 48464, 60224, 21924, 26528, 12192, 30740,
                47956, 3940,  50492, 59592, 9724,  61940, 19704, 41152, 54360,
                15780, 47860, 7848,  24800, 30160, 164,   38432, 9168,  42664,
                12280, 22144, 31864, 4612,  41548, 58700, 572,   14804, 53456,
                42340, 10748, 23308, 52240, 62112, 45200, 49760, 31012, 13008,
                52880, 35552, 4080,  60280, 46524, 27176, 61528, 9292,  28976,
                48100, 38024, 7840,  54380, 20736, 2824,  52400, 14648, 26324,
                7672,  40568, 60264, 3276,  37352, 47716, 64628, 32952, 55532,
                17144, 27560, 64172, 56660, 3660,  59716, 10992, 52136, 44048,
                36940, 19368, 41404, 21524, 7096,  32012, 43360, 61284, 35968,
                20712, 5544,  33220, 55820, 30108, 41700, 7660,  27800, 61236,
                9964,  52036, 63032, 38816, 48096, 16072, 11844, 39624, 6600,
                49348, 45632, 13456, 53404, 60840, 10088, 63040, 33836, 44744,
                50648, 31148, 1480,  10368, 59468, 33332, 13280, 4596,  25636,
                49968, 60688,
            },
            {
                9564,  18880, 31968, 11920, 17236, 5272,  55212, 376,   38832,
                64764, 25508, 56300, 22020, 38392, 856,   52584, 25232, 6000,
                37752, 60292, 18760, 40132, 53500, 45004, 20180, 27504, 51792,
                55184, 39252, 46460, 17644, 34032, 21400, 62272, 39740, 32240,
                61028, 3704,  58600, 30268, 14684, 26360, 10264, 58316, 1524,
                27352, 16192, 43940, 8280,  23788, 50296, 36612, 16888, 52556,
                24624, 4188,  61364, 26924, 40952, 58896, 32232, 38912, 56340,
                21576, 48740, 34172, 24680, 16572, 42944, 20640, 4520,  49688,
                24080, 46120, 12912, 19376, 47440, 28248, 34376, 61392, 2992,
                47912, 13092, 59580, 29176, 52092, 37500, 17844, 2092,  26244,
                54164, 44308, 38368, 50428, 14688, 45768, 19348, 53924, 38064,
                43144, 604,   13760, 54856, 8208,  32696, 63400, 54200, 29828,
                21120, 64212, 38036, 25144, 44016, 19972, 15044, 26848, 54808,
                11876, 20976, 43520, 26312, 15940, 52220, 41076, 54848, 45156,
                15432, 3332,
            },
            {
                58684, 43480, 1312,  56960, 40264, 50544, 43208, 59292, 16048,
                35268, 5788,  29768, 43512, 17180, 31448, 14124, 34164, 9944,
                30444, 55752, 2680,  34304, 10328, 58060, 63796, 6168,  17224,
                2196,  60012, 8368,  26664, 55580, 49188, 9672,  25308, 6604,
                18112, 34400, 38628, 20540, 5368,  54568, 35176, 39404, 46804,
                63692, 50932, 30676, 57604, 18584, 55408, 41780, 2892,  59348,
                33176, 19840, 15028, 49528, 34556, 12676, 5952,  62044, 17044,
                2132,  30528, 9760,  63368, 51992, 56616, 8776,  29712, 39824,
                60788, 9660,  38008, 5824,  54096, 14884, 39444, 21304, 32088,
                25160, 8756,  55488, 872,   45876, 11028, 57756, 39724, 48600,
                9384,  62188, 236,   22104, 63748, 10408, 58776, 4164,  25260,
                16560, 31608, 19928, 23240, 28420, 3024,  42812, 24944, 1244,
                57392, 10540, 3548,  31600, 7280,  40276, 58232, 4140,  38828,
                6760,  60504, 35992, 56936, 45836, 23028, 64112, 8156,  32592,
                21636, 36708,
            },
            {
                53436, 47776, 23248, 35292, 27888, 8044,  32704, 24236, 52824,
                19744, 46168, 12856, 8084,  63604, 47932, 57632, 45220, 65392,
                22392, 49780, 27024, 46868, 16104, 23404, 32948, 36956, 48640,
                30112, 14164, 35916, 63632, 2660,  13408, 29736, 45924, 50832,
                56032, 12772, 48204, 8780,  41516, 64524, 18172, 23556, 7248,
                20592, 11188, 38332, 580,   33388, 13924, 6736,  30180, 11928,
                46856, 38888, 9764,  64256, 1816,  24004, 47280, 28144, 36888,
                45816, 57908, 42172, 13332, 28480, 2424,  35336, 58952, 14784,
                668,   52196, 33924, 61748, 24876, 43320, 1448,  57536, 17028,
                50568, 62228, 35716, 15812, 26780, 64672, 5872,  22960, 13432,
                31436, 17132, 28256, 7060,  35712, 24400, 40008, 50060, 34212,
                64640, 45276, 58448, 41044, 61788, 47116, 18428, 50732, 14100,
                35728, 41964, 17400, 52320, 56532, 28788, 46128, 23400, 64060,
                29620, 47972, 17764, 4748,  31540, 2388,  28176, 18608, 49168,
                62356, 27160,
            },
            {
                30484, 4240,  15332, 52208, 64448, 20456, 13864, 48368, 9860,
                40760, 61380, 53960, 36364, 23600, 4960,  39228, 17912, 1940,
                41988, 12952, 36568, 62404, 4756,  50904, 13124, 43528, 62060,
                20524, 41384, 50868, 23296, 43340, 37912, 19372, 64840, 1848,
                42948, 27712, 62796, 53160, 31984, 44588, 3456,  57148, 33916,
                42512, 60728, 24532, 49016, 62576, 39896, 45440, 65004, 22328,
                57416, 42080, 53180, 29152, 43612, 55836, 19428, 51144, 10852,
                23060, 53384, 5096,  19664, 39256, 22684, 45096, 53616, 18924,
                43040, 28384, 21168, 49148, 30532, 10212, 64556, 46024, 6944,
                41860, 30628, 22172, 40476, 49372, 19020, 33380, 44668, 56372,
                51548, 41400, 58208, 47896, 53484, 31356, 1504,  13396, 56384,
                6348,  9464,  49308, 4780,  33956, 10856, 37468, 60156, 30864,
                46336, 22968, 62520, 47820, 13276, 2256,  35832, 18252, 51808,
                14300, 40232, 9396,  50780, 61504, 37832, 11688, 57160, 39564,
                6380,  12812,
            },
            {
                20192, 41508, 61388, 10744, 45664, 4536,  36616, 62684, 1864,
                28080, 32256, 3020,  50724, 27292, 55640, 11380, 28952, 52052,
                58796, 20472, 7668,  56436, 40512, 28736, 1584,  55376, 9956,
                26012, 5040,  58636, 31648, 7204,  53680, 58280, 33536, 23988,
                15468, 36208, 21860, 108,   24980, 15880, 50256, 29660, 14012,
                54672, 4152,  15796, 28960, 9844,  19816, 26564, 51524, 35752,
                1040,  25632, 5852,  16760, 36688, 59640, 8392,  41660, 65204,
                15216, 35132, 61700, 32112, 49440, 60496, 11100, 25988, 32392,
                65404, 7636,  57404, 16376, 4792,  51872, 35464, 13676, 27672,
                53856, 4768,  11660, 58760, 2816,  53408, 28444, 61188, 3512,
                35376, 25004, 11612, 19736, 43648, 60588, 17832, 47212, 22252,
                29348, 36652, 25612, 52952, 16420, 56236, 22128, 7844,  4540,
                54800, 28008, 8704,  33660, 20860, 60964, 49152, 10596, 32572,
                836,   24916, 55876, 21704, 42400, 14996, 47456, 34512, 52,
                43128, 55316,
            },
            {
                37360, 7436,  32900, 24684, 38904, 57944, 30232, 43696, 56344,
                17036, 21956, 59172, 14640, 41768, 20200, 62728, 35892, 24704,
                4600,  32692, 44664, 25484, 18500, 32136, 59444, 22356, 46960,
                34804, 54064, 18160, 11056, 45448, 16088, 4392,  40660, 10056,
                60664, 7292,  45760, 58836, 11524, 37288, 60580, 9656,  48000,
                27056, 36648, 52704, 46256, 58804, 4468,  54916, 15312, 8752,
                18528, 60636, 32156, 47900, 22080, 13632, 33384, 64,    30080,
                6824,  26196, 44464, 1384,  55596, 16296, 5728,  47588, 3732,
                38616, 13368, 45064, 36380, 41452, 19908, 23468, 56256, 38420,
                18436, 63968, 47164, 24668, 37028, 7464,  42156, 15416, 20568,
                8188,  60220, 2524,  37872, 15012, 5804,  27264, 42072, 61456,
                54012, 18960, 12604, 39764, 976,   27664, 44524, 65424, 40128,
                15484, 58716, 524,   38676, 26040, 41352, 7812,  54544, 58996,
                44132, 62560, 34756, 28564, 5460,  53008, 25744, 59520, 23144,
                16684, 64720,
            },
            {
                26308, 50680, 59196, 17524, 992,   54016, 15748, 25332, 7052,
                49768, 38092, 10276, 45436, 33788, 240,   48912, 8644,  46352,
                61656, 16400, 53716, 11048, 64620, 42656, 8204,  38324, 16272,
                3412,  65268, 29012, 48796, 36308, 22380, 28280, 47536, 52352,
                30460, 55424, 18344, 40904, 28508, 52428, 20264, 40188, 1108,
                65452, 21580, 6468,  41168, 34520, 22988, 38580, 31472, 43692,
                49592, 55636, 11460, 63472, 3368,  26764, 54516, 45536, 37716,
                58532, 48480, 18228, 9352,  24648, 36280, 41228, 61780, 51088,
                22408, 54952, 25464, 1188,  62928, 58652, 32720, 8516,  49660,
                68,    33920, 43432, 14244, 31644, 62148, 10920, 47644, 38800,
                30652, 45420, 54760, 29108, 65324, 34368, 51688, 10896, 38500,
                2888,  44816, 63564, 31388, 59960, 51168, 13196, 32648, 25236,
                49424, 19140, 44200, 51748, 64484, 16528, 30208, 22896, 37668,
                19244, 3996,  46548, 11316, 65516, 20032, 8496,  31764, 51480,
                9992,  46108,
            },
            {
                2356,  13312, 44056, 29584, 47988, 21416, 9480,  51360, 33328,
                42164, 64856, 25808, 5520,  54540, 16656, 57812, 31248, 13512,
                39980, 50364, 37332, 616,   47552, 14188, 52456, 27464, 60760,
                44248, 12980, 40056, 352,   60048, 54832, 63356, 12860, 20384,
                38244, 4092,  49748, 34060, 2724,  63068, 5740,  32828, 45340,
                58000, 17636, 31720, 12736, 63752, 1344,  47492, 61632, 4900,
                27844, 20868, 37848, 44804, 51980, 39620, 60976, 16336, 20956,
                52792, 12208, 39024, 51840, 64284, 30508, 12616, 27688, 18024,
                34840, 59876, 10472, 48632, 28524, 12528, 4032,  44128, 26024,
                59464, 20692, 9000,  57272, 50208, 21920, 55452, 25800, 51156,
                63156, 13188, 22016, 49344, 8772,  56548, 21252, 4732,  31804,
                15300, 57496, 7448,  22820, 47000, 19512, 36596, 2720,  53876,
                10144, 36112, 31108, 12308, 5032,  56876, 2728,  42672, 12884,
                27156, 53440, 15804, 30724, 38372, 56112, 44732, 3492,  61964,
                28844, 34944,
            },
            {
                60500, 22588, 56268, 5836,  35480, 64180, 39680, 59832, 13268,
                3568,  18516, 47700, 61808, 29208, 38668, 22864, 43268, 2908,
                27704, 21640, 6636,  29604, 58244, 20268, 33976, 6272,  49772,
                31140, 23668, 51972, 19828, 26596, 8868,  32792, 2408,  43820,
                26072, 63960, 14088, 23224, 55980, 46644, 26404, 14632, 23640,
                10708, 43108, 56132, 49800, 27288, 18356, 10296, 57056, 13524,
                41292, 33608, 7544,  15708, 29940, 10396, 5168,  24872, 63316,
                2792,  28924, 33568, 42688, 6332,  20004, 56988, 44252, 8708,
                2872,  31052, 16056, 53176, 38884, 18276, 47596, 61692, 15716,
                30260, 52412, 39308, 27912, 17232, 2152,  34028, 6328,  17988,
                548,   41220, 33080, 4364,  25836, 39712, 44052, 62664, 24708,
                50460, 28100, 35184, 41844, 10044, 5324,  58124, 42756, 21348,
                62704, 5940,  59636, 23916, 45208, 34836, 48312, 63496, 50896,
                6776,  61116, 40624, 48920, 1208,  17296, 35924, 40984, 14196,
                48548, 18788,
            },
            {
                37984, 9032,  40548, 52480, 11736, 27400, 4864,  46008, 23084,
                56892, 30524, 35772, 21000, 12184, 6856,  53272, 63924, 18916,
                59268, 55472, 34536, 62128, 23712, 39592, 2640,  57448, 17908,
                9856,  36912, 56684, 5816,  43176, 15308, 39548, 50508, 59104,
                17088, 8736,  31400, 43148, 7048,  17704, 35936, 60040, 53828,
                38660, 2500,  25000, 7692,  44980, 53804, 30092, 36204, 24524,
                64688, 53048, 732,   57956, 22832, 49084, 42364, 32776, 47100,
                8144,  56116, 22876, 14628, 59128, 48768, 736,   53868, 39148,
                64040, 46456, 41640, 6920,  33824, 24076, 55480, 40544, 11128,
                36372, 3552,  62888, 5744,  45476, 40960, 64936, 37340, 57752,
                27452, 53976, 16428, 58728, 46900, 18072, 13764, 53692, 36424,
                46084, 1896,  17608, 55096, 26392, 62032, 30124, 16556, 47540,
                27388, 40868, 14580, 53324, 19776, 28056, 10996, 17508, 31920,
                36028, 21476, 9068,  24408, 59828, 51128, 27088, 21096, 57672,
                4840,  54208,
            },
            {
                30576, 16168, 62500, 25208, 18552, 43316, 32124, 16212, 37464,
                53004, 9012,  2448,  41628, 58404, 49624, 26460, 34972, 9096,
                47780, 12144, 44144, 15676, 9208,  54576, 45852, 26108, 64068,
                42224, 1996,  61232, 34080, 46904, 62144, 24216, 6520,  35096,
                53992, 46920, 37036, 57600, 11684, 51296, 41608, 8240,  48532,
                29336, 63376, 36352, 16436, 60232, 39476, 21528, 51292, 2884,
                16808, 46624, 27104, 62388, 35236, 18440, 56860, 13556, 36844,
                19028, 43912, 61488, 4100,  26640, 37392, 32924, 23760, 14296,
                50776, 26348, 21388, 56964, 63560, 1924,  29448, 6224,  22108,
                54628, 47792, 13572, 24252, 51736, 29740, 11936, 46536, 22704,
                9124,  44148, 35656, 11204, 63848, 30236, 768,   8140,  19596,
                60140, 12048, 64404, 49080, 14000, 38136, 50796, 8052,  57016,
                33432, 1644,  49008, 37460, 7888,  60384, 39684, 24856, 372,
                55168, 44432, 57508, 33372, 13412, 6504,  63092, 10516, 33512,
                24908, 42768,
            },
            {
                512,   47392, 33800, 3388,  49488, 60864, 56068, 660,   63644,
                26112, 48588, 60776, 14676, 31924, 45204, 1256,  15800, 41328,
                31636, 4988,  25324, 50936, 41744, 30908, 12708, 35272, 21164,
                47672, 13900, 28120, 17364, 10508, 29696, 57512, 21276, 12356,
                28836, 772,   60712, 20796, 27748, 64740, 31092, 22024, 4772,
                13580, 20052, 51636, 32384, 3524,  12984, 6588,  43996, 58524,
                8856,  20168, 38872, 43556, 9216,  53652, 1876,  59908, 27020,
                50328, 11164, 39528, 53028, 16944, 46564, 10740, 60836, 30044,
                18716, 4496,  36820, 9888,  14072, 51000, 44880, 34816, 65260,
                43004, 18580, 33004, 60724, 9604,  20252, 59884, 15156, 31496,
                48844, 61252, 3272,  23856, 38708, 49996, 33864, 56956, 41980,
                23368, 32484, 39456, 4036,  21652, 33948, 152,   24140, 12064,
                45384, 17936, 64952, 30312, 56280, 4712,  52280, 46464, 61928,
                14232, 28228, 4948,  19424, 47256, 42644, 30320, 52736, 45848,
                7584,  64132,
            },
            {
                19932, 27992, 58188, 13916, 36440, 8220,  20428, 29708, 11224,
                42784, 17428, 38984, 52336, 24096, 10068, 37492, 61468, 22012,
                52856, 64544, 37208, 1684,  19196, 62900, 7500,  52268, 4304,
                55332, 39312, 23016, 53468, 49420, 36540, 1632,  45072, 64552,
                41420, 24724, 49552, 5600,  39288, 2376,  15808, 57060, 34748,
                61148, 46752, 9540,  56896, 42508, 62716, 33980, 26536, 49068,
                31132, 60148, 12800, 4168,  32052, 25708, 40428, 6880,  31020,
                63808, 1000,  34904, 29352, 8296,  65052, 21696, 41928, 6684,
                57988, 44140, 61288, 49040, 26972, 19708, 59700, 16164, 8952,
                25752, 1140,  38504, 55588, 43944, 35860, 1476,  53144, 5256,
                38376, 19460, 29384, 51952, 7144,  20812, 61584, 27036, 10308,
                52536, 6488,  29304, 44500, 54176, 60096, 42132, 63452, 38848,
                54448, 25748, 10280, 21972, 42804, 15628, 33624, 20700, 9668,
                41256, 49852, 38260, 64560, 25904, 36896, 2192,  16820, 39164,
                12888, 50032,
            },
            {
                56500, 9816,  44852, 22440, 54048, 41036, 46884, 51228, 34828,
                6664,  22268, 28488, 5848,  65152, 19644, 56428, 50424, 4016,
                28212, 17668, 57236, 46724, 27652, 58788, 38612, 16180, 29612,
                60008, 32884, 3372,  63420, 7452,  40432, 18964, 52876, 32228,
                9156,  15444, 18748, 33140, 53648, 45976, 24952, 50656, 43504,
                284,   26876, 39224, 22888, 28908, 19332, 54876, 14796, 40324,
                22592, 36628, 52364, 47756, 15328, 65244, 50840, 21800, 16368,
                45992, 23852, 58076, 19576, 44336, 50588, 3708,  35088, 54640,
                12512, 33232, 344,   30924, 42328, 37768, 2936,  53332, 31988,
                58276, 50456, 28964, 15660, 7220,  26548, 56768, 41776, 24884,
                63652, 13244, 55832, 41556, 15788, 44716, 3736,  47228, 37992,
                16288, 48212, 59296, 18804, 8456,  27828, 15436, 20088, 31040,
                5300,  61156, 36056, 2476,  50608, 27052, 63976, 3696,  30440,
                58856, 17772, 1740,  52940, 9260,  55768, 22508, 61500, 58780,
                23804, 36064,
            },
            {
                15392, 41912, 4460,  65300, 28940, 2016,  24568, 14584, 62448,
                55512, 45396, 59428, 1736,  34044, 42888, 29680, 12988, 46228,
                40184, 8648,  14064, 32908, 6120,  22348, 48876, 43956, 25132,
                8940,  19168, 45500, 12492, 25428, 56576, 14928, 4852,  48568,
                37884, 55044, 62336, 43252, 13480, 59232, 9908,  38048, 18388,
                31684, 53328, 15316, 5012,  49504, 1076,  45872, 10240, 64012,
                1828,  6024,  28088, 56704, 19360, 44436, 10376, 37488, 54676,
                41732, 12696, 5244,  55380, 31624, 13836, 58928, 27620, 47188,
                24212, 52328, 17148, 22468, 56456, 11800, 48256, 23532, 40304,
                12752, 45356, 4628,  22456, 64456, 48032, 32636, 17120, 50316,
                7996,  34628, 2332,  59984, 25540, 54556, 12504, 30636, 65100,
                1212,  56216, 24948, 36168, 11712, 46840, 52612, 3604,  49864,
                43576, 14128, 47612, 58100, 39348, 12272, 45168, 37064, 53884,
                23268, 34960, 12412, 44172, 31852, 14776, 48652, 28740, 4576,
                32388, 53072,
            },
            {
                26420, 60552, 38020, 32072, 10948, 17336, 60372, 39304, 3828,
                32352, 13576, 37020, 49156, 55056, 16584, 6984,  25248, 60304,
                34276, 54752, 24020, 61280, 51684, 36192, 500,   11992, 63712,
                41432, 50824, 58152, 35408, 43368, 31208, 62436, 27680, 22472,
                57864, 3728,  26436, 8000,  36116, 21284, 29040, 5856,  63740,
                11696, 57712, 45016, 64968, 36872, 59860, 24396, 35384, 50784,
                18204, 42144, 61064, 23616, 34672, 2392,  29688, 58736, 3348,
                33492, 61784, 48428, 25816, 40596, 1640,  38600, 16372, 9408,
                36592, 64492, 40200, 8136,  62724, 4804,  29100, 61756, 6996,
                20152, 62940, 34432, 52988, 39484, 12596, 3896,  21344, 58888,
                44788, 27420, 47480, 32164, 9308,  35780, 58696, 22804, 19364,
                34424, 13508, 42876, 63288, 2748,  32000, 57432, 36972, 9732,
                23196, 28576, 32912, 18300, 8512,  24304, 55892, 16812, 7104,
                47292, 27428, 59900, 20408, 62252, 6156,  42000, 19032, 10860,
                40880, 1928,
            },
            {
                48804, 6028,  18600, 51416, 45772, 35868, 54748, 27224, 19092,
                50108, 8524,  20816, 26232, 11388, 47184, 58628, 36696, 656,
                19016, 44748, 2964,  42436, 10532, 17476, 56404, 33932, 53764,
                4860,  15176, 28252, 1912,  21436, 51696, 9584,  35748, 41972,
                12652, 45488, 30132, 51384, 1248,  61712, 47108, 54304, 41448,
                21840, 25380, 6676,  33040, 17264, 12256, 29932, 6956,  57652,
                32600, 53860, 11904, 7736,  39668, 62096, 13904, 47004, 27340,
                20564, 9152,  36704, 17304, 63352, 22848, 53656, 61560, 20344,
                2540,  28808, 45504, 14984, 51596, 35328, 18248, 46696, 36668,
                27632, 55816, 10336, 17904, 43468, 57572, 29252, 36740, 10864,
                40180, 14812, 21816, 64208, 18236, 39540, 43644, 4700,  51440,
                40700, 28236, 5340,  50700, 22156, 40300, 25256, 64360, 17064,
                55840, 60000, 6404,  53336, 62912, 29728, 1364,  60256, 42380,
                10696, 51164, 3432,  39648, 25192, 35768, 57804, 51248, 45296,
                64496, 20772,
            },
            {
                35036, 54476, 24404, 13592, 58420, 6908,  48468, 9904,  43420,
                56952, 30096, 64032, 41676, 4368,  31220, 39872, 22304, 64160,
                11704, 49292, 28816, 38920, 65240, 26372, 46040, 20380, 30560,
                23300, 37820, 48808, 65012, 17484, 6076,  46068, 60772, 20,
                19756, 34248, 65384, 23260, 40108, 17676, 32956, 14936, 2124,
                36052, 48836, 9364,  40748, 52172, 55856, 43168, 20996, 47080,
                26192, 16000, 45284, 30504, 49984, 55240, 24860, 7080,  52784,
                15568, 50372, 29052, 43604, 7376,  11312, 32840, 42696, 49872,
                57556, 6300,  55436, 25468, 32084, 42844, 57308, 13504, 3184,
                41652, 49160, 276,   31564, 23876, 8408,  62172, 52604, 1236,
                61032, 54068, 5556,  50440, 204,   53504, 26812, 8060,  62376,
                46356, 9836,  54600, 17544, 61216, 14052, 44712, 6916,  34848,
                41148, 644,   45640, 20604, 41692, 35048, 49404, 21492, 32456,
                64956, 15428, 30744, 54596, 47924, 208,   13076, 33132, 7256,
                55604, 30028,
            },
            {
                46852, 8960,  63336, 42680, 21004, 2468,  33640, 22576, 62996,
                80,    38012, 15396, 52004, 61332, 18400, 53728, 9136,  50972,
                32424, 57064, 21192, 14612, 31668, 7624,  58632, 3308,  40408,
                61424, 7396,  10880, 32676, 40224, 54964, 13608, 24896, 49596,
                59068, 10132, 16040, 48860, 55132, 9524,  58228, 27444, 51764,
                62072, 30344, 59036, 20288, 26784, 2364,  37688, 61696, 9556,
                3768,  36780, 64260, 1284,  21448, 17440, 36188, 43072, 64464,
                39388, 59572, 104,   56880, 52256, 46184, 25384, 4324,  30248,
                38032, 12836, 21204, 48788, 1540,  9620,  24228, 52836, 60276,
                15912, 25196, 59160, 37640, 51172, 15140, 45468, 25028, 33496,
                17420, 30544, 38044, 24156, 45636, 14016, 33748, 16840, 56692,
                31296, 24172, 38236, 33128, 48288, 1288,  29904, 20768, 51540,
                12824, 26100, 37512, 14860, 51280, 4436,  12788, 39240, 25100,
                5432,  37948, 45152, 8400,  16920, 63556, 21632, 27004, 38220,
                16352, 12488,
            },
            {
                39812, 964,   31556, 26888, 38796, 61496, 29364, 40652, 12712,
                25312, 46776, 6316,  23172, 35188, 2284,  28052, 43500, 15920,
                25868, 7140,  60456, 3836,  52964, 48356, 43032, 12720, 16360,
                51780, 56272, 25776, 46764, 58552, 28868, 37224, 16860, 31584,
                53212, 38592, 5868,  42736, 26140, 3204,  37708, 43488, 12060,
                18064, 4404,  44536, 14160, 63268, 5792,  50368, 14592, 40644,
                58920, 22744, 52020, 41812, 10112, 60404, 4048,  31224, 11464,
                23316, 5772,  32548, 21128, 14292, 35820, 58596, 18888, 15324,
                63656, 43672, 33448, 59100, 39544, 65128, 20008, 31124, 7612,
                33676, 44888, 20580, 6708,  64020, 2756,  40708, 19636, 48700,
                7536,  42868, 57868, 9708,  62132, 29484, 59856, 49196, 21384,
                1564,  14740, 58440, 7460,  26756, 59284, 10660, 55148, 62012,
                47484, 31780, 65396, 10024, 27312, 60924, 17868, 44300, 58452,
                52656, 19724, 57440, 23660, 29556, 40212, 52484, 59864, 2668,
                23896, 62296,
            },
            {
                44404, 17396, 57244, 49132, 11340, 16032, 56232, 50620, 17852,
                53800, 60448, 32400, 44876, 57792, 13960, 47760, 62396, 4848,
                38364, 41824, 46648, 34988, 18012, 23732, 36764, 63296, 27980,
                34508, 42516, 18992, 1196,  22176, 4084,  44468, 64088, 5020,
                27348, 22388, 56764, 13376, 32044, 60204, 21996, 5660,  46660,
                23792, 56724, 38644, 32480, 47384, 23116, 29120, 18648, 32292,
                54616, 28416, 13564, 33896, 48620, 27524, 45908, 56992, 18812,
                35104, 47636, 41112, 27128, 65420, 2704,  39752, 48072, 7760,
                53192, 26692, 10304, 17528, 5612,  28240, 45568, 50360, 38860,
                63548, 11300, 54028, 30060, 47380, 34776, 27976, 11892, 55524,
                64692, 26508, 13128, 35056, 20360, 41488, 2772,  11344, 39088,
                44448, 63412, 50052, 19752, 36244, 45868, 39524, 33740, 5776,
                18060, 3188,  23112, 57772, 46948, 33248, 54356, 9328,  28300,
                884,   35932, 11492, 61140, 3812,  34496, 9936,  43664, 49332,
                56648, 28304,
            },
            {
                52064, 22844, 4952,  36260, 52916, 3884,  44516, 7756,  36488,
                3120,  28688, 11608, 19892, 8416,  37428, 23908, 56160, 34000,
                18892, 53892, 12536, 62960, 29532, 9696,  54532, 2168,  21604,
                5148,  49708, 14408, 60328, 35108, 12152, 51956, 8904,  40728,
                46956, 1560,  63384, 36016, 47768, 18564, 50412, 64676, 34452,
                53132, 27960, 1420,  11256, 53908, 35080, 43916, 65328, 8,
                47888, 5016,  19520, 62964, 6356,  38596, 14864, 50852, 2776,
                53536, 61980, 9680,  17724, 49432, 31144, 10588, 61852, 21740,
                34524, 912,   51160, 61248, 36252, 54608, 12068, 1980,  17292,
                26904, 4000,  39924, 13380, 22696, 56932, 60368, 5220,  36932,
                22372, 3108,  49972, 54680, 4556,  48024, 25412, 55552, 35460,
                27656, 5092,  42076, 12020, 53768, 3968,  15688, 22524, 28500,
                50200, 43244, 54072, 39396, 5864,  21240, 2836,  61936, 48272,
                31796, 14260, 49536, 42312, 55440, 47248, 15136, 19104, 31952,
                4624,  10960,
            },
            {
                60352, 33812, 8540,  65008, 19760, 24620, 32272, 63888, 21612,
                42968, 57216, 49496, 40756, 65500, 51756, 30476, 11176, 1748,
                59892, 27732, 23072, 92,    50380, 40012, 59712, 32144, 45652,
                57964, 9288,  30928, 38880, 48580, 62548, 24368, 18700, 57324,
                34724, 20208, 44272, 28936, 6700,  10220, 40504, 30432, 15520,
                8312,  41760, 62440, 17452, 59496, 6784,  9788,  56544, 24824,
                11840, 44928, 37660, 52948, 24740, 59412, 21988, 41428, 29576,
                25268, 13588, 44352, 55672, 6792,  23816, 54192, 27812, 44708,
                57132, 40720, 23548, 46340, 14268, 41368, 22484, 58356, 35400,
                55760, 48484, 61152, 18660, 43296, 9184,  16532, 51620, 46268,
                29276, 15460, 44120, 31100, 17740, 37732, 65164, 7212,  52552,
                18524, 57376, 23456, 31068, 64684, 25456, 60620, 41464, 57348,
                12304, 35208, 8420,  15404, 30172, 42596, 37256, 23948, 16708,
                40692, 63916, 27204, 5860,  20608, 26136, 65504, 7676,  41392,
                36512, 20396,
            },
            {
                39360, 48284, 29616, 45656, 40192, 59540, 13848, 48108, 26572,
                9300,  16084, 34252, 892,   27072, 5392,  42536, 21144, 45556,
                49796, 8508,  37092, 44624, 15188, 5996,  19080, 25728, 12088,
                40648, 64344, 52908, 27080, 6828,  42624, 29404, 32996, 14668,
                60160, 11564, 51652, 16280, 55372, 61948, 25672, 340,   58860,
                21008, 36232, 49616, 26132, 21608, 41024, 51656, 16316, 39232,
                33120, 61316, 8460,  16396, 31868, 780,   10932, 64168, 7872,
                58880, 37936, 1228,  34216, 59944, 42984, 36632, 16724, 5156,
                13704, 30032, 7556,  19404, 32160, 4760,  63188, 29800, 42640,
                7828,  23952, 32472, 1204,  50396, 25588, 31360, 41624, 488,
                59064, 39188, 62828, 8308,  58372, 12316, 22620, 32268, 14472,
                47260, 9420,  34576, 396,   17096, 51368, 7936,  46488, 1880,
                63640, 25692, 60052, 19228, 52232, 63072, 13316, 56736, 45044,
                7504,  22280, 52936, 33472, 38104, 1580,  58288, 29140, 53232,
                61960, 14468,
            },
            {
                6492,  58128, 16540, 12012, 27476, 792,   35436, 5712,  53236,
                39436, 62700, 23288, 54968, 18508, 60300, 15504, 38936, 62780,
                31544, 17068, 58772, 55812, 33348, 64792, 47076, 51984, 35504,
                17208, 23888, 3244,  20612, 16252, 56188, 552,   53392, 49004,
                7560,  26728, 41344, 2296,  23392, 38284, 52408, 12172, 43196,
                54896, 4808,  14412, 46016, 30556, 3320,  36412, 27584, 58692,
                20792, 51020, 29280, 57724, 42676, 55188, 35252, 45304, 16932,
                32076, 21212, 51800, 28552, 19688, 11956, 1808,  46972, 60520,
                38068, 49208, 64624, 58056, 53244, 26176, 48900, 10104, 19904,
                46516, 15244, 53260, 37372, 65380, 6368,  35652, 61484, 19148,
                11072, 33728, 20924, 51708, 27728, 46280, 42780, 60444, 3000,
                29560, 62740, 40384, 48884, 43740, 36924, 30072, 14404, 20260,
                38516, 31344, 48052, 148,   45704, 27116, 10176, 34196, 51256,
                1920,  59816, 10456, 17532, 44540, 13344, 50500, 22416, 46072,
                504,   26480,
            },
            {
                43192, 23680, 2696,  54644, 51064, 42652, 57628, 19272, 60972,
                29564, 4784,  45796, 13056, 32032, 48424, 7992,  54340, 24848,
                11972, 2832,  41236, 20484, 24640, 10900, 29212, 1156,  56612,
                7116,  43832, 37344, 61628, 46176, 10600, 39820, 21720, 4060,
                37572, 62524, 31200, 58380, 34408, 45596, 19580, 32628, 47488,
                28804, 64052, 10108, 33880, 61520, 54740, 12776, 49048, 6276,
                1932,  41320, 23264, 4348,  47216, 13468, 26752, 49988, 39280,
                5376,  47424, 63464, 15320, 40704, 50128, 64280, 32876, 20420,
                26292, 2712,  11764, 34740, 43852, 324,   16208, 39172, 61460,
                4512,  58680, 10840, 27588, 45584, 55252, 12632, 23284, 53100,
                7236,  56804, 24784, 1568,  36332, 4956,  16696, 50812, 39220,
                20556, 54524, 13236, 58556, 21316, 10476, 56820, 34332, 52760,
                4644,  55024, 11232, 23692, 35852, 58748, 3668,  18680, 25436,
                38972, 29580, 46820, 57144, 63088, 31392, 39652, 10356, 17412,
                32932, 55444,
            },
            {
                50204, 31372, 37772, 62596, 21984, 9972,  31120, 15492, 44412,
                11504, 37632, 51852, 59028, 36384, 43800, 28472, 3756,  34552,
                52168, 47204, 28208, 5104,  53252, 43160, 38588, 14536, 62192,
                48220, 27936, 54696, 13180, 31304, 58468, 35180, 64656, 45312,
                24008, 17792, 48208, 5548,  14228, 8536,  59988, 3088,  17368,
                37956, 23200, 51216, 2052,  42500, 17696, 23956, 62764, 43656,
                14664, 64500, 36176, 10048, 18048, 62040, 20764, 3160,  58112,
                54092, 23848, 10004, 3620,  56584, 29776, 22924, 8812,  51964,
                55956, 41852, 17916, 24196, 8972,  37328, 56432, 51844, 31340,
                24920, 34936, 41084, 20804, 17324, 3228,  39556, 48672, 28368,
                43012, 47088, 14500, 40988, 54388, 63720, 9984,  34236, 26412,
                45800, 7628,  25116, 4308,  28172, 61680, 5724,  48260, 23176,
                44408, 16912, 41396, 64348, 6812,  39808, 48936, 31244, 65068,
                55844, 14724, 35060, 2760,  8192,  24144, 54928, 4424,  35948,
                64372, 12700,
            },
            {
                9220,  61060, 18736, 34048, 6864,  47516, 38856, 23936, 49384,
                1432,  20648, 25984, 8792,  2984,  22344, 64532, 56484, 19420,
                37940, 63680, 14104, 61056, 35640, 8468,  59528, 21904, 32336,
                19236, 9800,  33764, 1956,  49336, 25784, 17184, 8304,  28748,
                55992, 10800, 53872, 43284, 22052, 65084, 49876, 27168, 55716,
                7304,  59664, 39956, 20080, 58064, 8488,  46908, 30868, 34712,
                19192, 53932, 48512, 27776, 38620, 52100, 33636, 8900,  29332,
                13252, 43568, 35884, 26016, 45616, 6312,  39284, 58740, 15596,
                5624,  31892, 50996, 46640, 62092, 28612, 22008, 12648, 45240,
                1596,  63308, 49560, 8096,  57316, 30472, 62280, 15352, 34808,
                64468, 3356,  31680, 48980, 19536, 30140, 23572, 57488, 1332,
                59844, 37120, 16480, 33344, 53476, 39364, 18484, 26932, 62648,
                8264,  29248, 56628, 33008, 21016, 15580, 54536, 43204, 5232,
                11868, 21172, 41800, 27376, 19008, 48792, 61624, 27884, 41880,
                47656, 20788,
            },
            {
                39804, 4572,  45132, 14008, 26432, 64776, 3240,  52768, 33476,
                56516, 64152, 30852, 16832, 40444, 50528, 10872, 14756, 41832,
                7480,  23036, 32768, 48720, 18476, 3156,  50164, 26048, 41252,
                3992,  51404, 59332, 39400, 21180, 5288,  42836, 51244, 14080,
                40536, 34084, 3176,  26284, 37320, 29864, 40860, 11144, 35564,
                15000, 45100, 26656, 12000, 29256, 36928, 4704,  57036, 10728,
                25832, 6728,  32244, 60116, 1616,  43936, 24552, 64824, 41888,
                18776, 59360, 32444, 62120, 18092, 53732, 13292, 34984, 44388,
                28956, 63736, 38976, 13736, 5088,  58312, 33724, 7088,  18100,
                54684, 29220, 13820, 44136, 33616, 51588, 22028, 9576,  5296,
                25652, 18044, 58768, 11772, 60916, 6284,  39996, 52888, 12260,
                18980, 43256, 50688, 64988, 46588, 1280,  42992, 11568, 58328,
                36476, 1544,  49732, 12944, 52540, 27940, 9088,  23080, 36264,
                47608, 50964, 61372, 54296, 37072, 43764, 6068,  13684, 59356,
                2372,  25408,
            },
            {
                52140, 28188, 58884, 54036, 41296, 17416, 59364, 28068, 8028,
                14220, 41708, 46608, 54212, 61276, 33168, 26376, 45492, 30396,
                59340, 2008,  44112, 11296, 57204, 30956, 45256, 55124, 12196,
                65412, 44900, 24428, 14448, 63212, 55368, 30620, 61024, 1464,
                58668, 20412, 62852, 51528, 16704, 56912, 5884,  21040, 61272,
                52828, 692,   32740, 48908, 64812, 53168, 21420, 39760, 51408,
                61112, 44644, 12600, 21656, 55884, 5844,  15228, 54724, 37280,
                292,   51544, 7420,  48736, 37540, 1152,  63124, 25016, 48352,
                10668, 21824, 1320,  54332, 20084, 42308, 49144, 64980, 40360,
                36348, 22636, 5436,  60764, 25940, 696,   41540, 59236, 45840,
                55920, 38768, 8560,  35808, 26712, 45012, 15192, 47856, 31816,
                63008, 3616,  29692, 9256,  13784, 24052, 31084, 51516, 15148,
                40472, 19848, 26052, 61200, 2744,  45300, 59108, 62444, 17736,
                756,   24992, 9388,  3636,  15556, 31900, 22004, 51564, 17984,
                30796, 57032,
            },
            {
                15972, 36768, 10756, 100,   49916, 36076, 12328, 43748, 22244,
                60532, 35704, 5924,  12124, 19696, 280,   57676, 5068,  53068,
                17332, 50064, 25616, 40216, 21360, 63116, 16476, 5960,  38148,
                34320, 17060, 29720, 7644,  46812, 36004, 11436, 22916, 38128,
                46348, 31688, 8652,  12312, 44392, 1668,  47908, 31000, 42452,
                25228, 18604, 56348, 6528,  16340, 44332, 2924,  27972, 15956,
                404,   37236, 57832, 16980, 40480, 49312, 29936, 11304, 47196,
                23108, 28272, 16224, 11172, 22272, 30884, 41712, 19208, 3504,
                56608, 36320, 60508, 26744, 31464, 15944, 2264,  25148, 11136,
                50420, 58204, 16720, 37792, 47296, 11620, 18792, 36980, 29324,
                50040, 21280, 42616, 51476, 568,   56152, 20968, 36132, 8656,
                27132, 55320, 40496, 21156, 35520, 60824, 55800, 5264,  33584,
                63952, 47208, 9752,  42432, 38188, 34264, 5808,  30640, 41040,
                57492, 29024, 33776, 45548, 63724, 10620, 56588, 38632, 34336,
                44648, 7288,
            },
            {
                63728, 48224, 32436, 24360, 20752, 30288, 4172,  54840, 38440,
                18296, 2004,  29260, 24528, 49256, 36044, 39328, 21108, 62344,
                34764, 8580,  55792, 29092, 824,   36312, 10260, 28020, 22868,
                57452, 368,   53452, 41476, 19344, 4480,  50104, 27068, 18168,
                6420,  49396, 27820, 59728, 34728, 24776, 63620, 13952, 50600,
                4284,  62348, 35900, 41108, 23824, 9900,  33644, 63272, 42276,
                24280, 47420, 30500, 8360,  26160, 62532, 35816, 20388, 61328,
                5108,  40308, 60424, 44688, 57136, 52680, 8448,  59968, 28040,
                50356, 17000, 43792, 9500,  52740, 38192, 60956, 45668, 28276,
                8436,  42788, 2596,  31428, 55748, 64760, 24472, 53944, 12876,
                1900,  15576, 61996, 23412, 33744, 65508, 3868,  42360, 50284,
                22772, 14636, 47160, 58304, 6532,  17612, 38656, 48856, 21544,
                3500,  29924, 57372, 16464, 24088, 19316, 51436, 10664, 14848,
                37604, 52980, 19772, 59084, 40104, 49296, 25532, 664,   61124,
                12040, 22940,
            },
            {
                2828,  18912, 42724, 61988, 57740, 9004,  46392, 62732, 26208,
                51328, 47676, 55720, 42692, 63576, 15680, 9744,  47692, 28012,
                13016, 38476, 63992, 14696, 51340, 46664, 61224, 52808, 42864,
                47592, 9224,  61476, 26444, 59116, 32712, 44096, 64992, 54040,
                14600, 42304, 22232, 39576, 54784, 19560, 7608,  33468, 38404,
                9628,  46528, 13012, 30200, 60460, 57260, 48300, 12908, 55192,
                19880, 4360,  65220, 53052, 34140, 2600,  45464, 7772,  56452,
                31444, 53136, 34716, 2852,  26908, 14988, 33944, 46376, 12556,
                40060, 7200,  23488, 34312, 47864, 12524, 6056,  55964, 19212,
                33144, 62916, 52944, 20628, 6040,  43460, 34500, 4560,  63120,
                32552, 44684, 6924,  29820, 13040, 17584, 28328, 59052, 5716,
                61932, 33436, 96,    26020, 52548, 43960, 28316, 10136, 25476,
                44916, 52016, 7020,  32236, 64832, 47904, 56552, 26264, 46568,
                64116, 7148,  12928, 23632, 5172,  17344, 29320, 8640,  47236,
                53528, 41096,
            },
            {
                29880, 55556, 6564,  14116, 38956, 52440, 33900, 15232, 6392,
                10956, 32888, 20920, 7788,  59312, 31336, 51888, 43224, 23584,
                2384,  45232, 19944, 5328,  41896, 23920, 32176, 4216,  13664,
                19656, 31628, 36272, 12456, 51880, 15876, 2116,  9868,  35196,
                58080, 24,    63236, 4528,  15512, 45564, 52488, 58752, 22572,
                28396, 55028, 20828, 51676, 5608,  18852, 37816, 31308, 7684,
                50168, 35288, 10648, 43464, 22700, 13232, 50956, 16608, 42732,
                25516, 13720, 19380, 64228, 38436, 50536, 4136,  21536, 64536,
                30552, 54820, 62736, 3200,  57644, 29380, 22512, 35632, 51352,
                13716, 23976, 39988, 27508, 15132, 49140, 9128,  17552, 52072,
                26748, 37580, 58272, 54620, 48372, 39892, 53396, 11644, 38088,
                18216, 42796, 10520, 37096, 64340, 12948, 1792,  62340, 57916,
                18004, 35608, 12016, 54088, 388,   13712, 4820,  42960, 21936,
                1788,  49908, 43680, 31072, 36276, 52744, 42528, 65064, 20256,
                26772, 35456,
            },
            {
                9572,  49676, 25348, 45160, 1536,  17836, 23048, 41360, 29788,
                64932, 57744, 40028, 13812, 27220, 3236,  18652, 6548,  56060,
                60244, 30304, 53816, 33772, 59524, 7980,  17556, 58216, 39424,
                64232, 49884, 22224, 3408,  39028, 23656, 56784, 40936, 29892,
                24608, 51752, 32364, 37012, 10400, 29240, 40096, 1048,  17004,
                65448, 43472, 2304,  39520, 27096, 45228, 1400,  25352, 61944,
                39500, 14856, 28028, 57388, 18768, 60176, 29028, 37576, 63664,
                1624,  48648, 9200,  42324, 6252,  24440, 58500, 43288, 36656,
                176,   14652, 45180, 19720, 41048, 16492, 64004, 43444, 456,
                59016, 7024,  45936, 11076, 61576, 30424, 59380, 41032, 21836,
                47500, 10624, 19416, 4096,  35340, 8996,  46304, 25300, 30496,
                57400, 54148, 48244, 20068, 30656, 49920, 33892, 42040, 14172,
                39040, 60940, 22400, 43624, 39776, 29216, 35888, 61664, 32408,
                16956, 27864, 55116, 62712, 2416,  58120, 14788, 33600, 4664,
                13404, 60152,
            },
            {
                16484, 37124, 64752, 33124, 28660, 60556, 56488, 50388, 2780,
                44612, 17328, 4824,  37156, 46032, 54256, 33984, 65336, 36800,
                10096, 48360, 16596, 26788, 11580, 38240, 49176, 26260, 1716,
                29076, 7152,  56028, 45912, 63492, 28392, 48248, 20400, 7552,
                46468, 11916, 18772, 56700, 48040, 60980, 25136, 12120, 47584,
                34956, 7352,  15060, 32680, 62800, 11560, 52560, 16876, 58876,
                21896, 45744, 51796, 928,   41216, 6080,  55492, 23564, 11096,
                33300, 59180, 21928, 46496, 30024, 16736, 55140, 9332,  18504,
                26620, 50952, 32360, 24852, 8852,  49324, 4384,  38964, 30844,
                17716, 48696, 33632, 54868, 38252, 960,   25432, 35912, 56776,
                2436,  61132, 42152, 24928, 62360, 21660, 1572,  64140, 15076,
                3380,  24044, 7508,  59484, 16108, 4816,  23484, 53164, 7424,
                28744, 2464,  49472, 26524, 58132, 20492, 53172, 9916,  40164,
                57940, 35044, 8584,  19488, 11724, 46560, 22748, 50612, 38780,
                56036, 44776,
            },
            {
                51896, 3360,  21200, 11884, 48120, 8196,  36996, 12544, 21152,
                35120, 25876, 52312, 61968, 22308, 11896, 42160, 25052, 14460,
                40980, 22132, 4396,  43700, 64636, 54936, 21572, 34940, 53420,
                44784, 15644, 41180, 18456, 10284, 5484,  37032, 62320, 16852,
                59776, 43028, 26768, 6672,  21592, 3496,  42320, 31376, 53388,
                57596, 25548, 59688, 49252, 22316, 55924, 34372, 41716, 3840,
                30292, 6772,  63868, 24900, 36772, 31992, 47496, 3780,  40116,
                54228, 28340, 51364, 35444, 61648, 11752, 32812, 48152, 60592,
                39472, 6464,  58148, 37100, 61920, 53488, 26952, 11528, 55348,
                25340, 65348, 3304,  22088, 16904, 50856, 45696, 7324,  13892,
                29196, 16520, 50820, 31516, 13624, 52388, 41208, 32716, 44728,
                51316, 35116, 28884, 39700, 44556, 55860, 36516, 20432, 47148,
                56388, 32916, 5524,  63552, 15784, 7300,  48676, 24556, 3536,
                13496, 47576, 25180, 42008, 37968, 27436, 6952,  59612, 31216,
                1308,  23812,
            },
            {
                29128, 40084, 58164, 53512, 41876, 4072,  25128, 46212, 61048,
                54932, 9456,  30540, 48240, 2048,  58912, 29144, 50416, 1292,
                32500, 52860, 58292, 36228, 1012,  15304, 46320, 5456,  62672,
                11548, 24372, 59024, 30036, 34540, 54196, 13172, 50872, 32672,
                4984,  38952, 64596, 49948, 34684, 54464, 63536, 19292, 5140,
                10008, 37860, 18376, 42208, 3508,  8592,  28428, 47028, 13828,
                56572, 33844, 17968, 12148, 49244, 8692,  62616, 20744, 45336,
                17776, 7016,  14564, 744,   53788, 41132, 27448, 2716,  22292,
                52712, 11348, 46952, 1856,  13756, 34692, 21444, 59948, 44812,
                8248,  36032, 41856, 29472, 57500, 9924,  19676, 63500, 53496,
                34056, 45308, 38304, 7648,  57760, 27372, 18520, 60364, 8268,
                20876, 13104, 63704, 2268,  11664, 61760, 27356, 9516,  65480,
                15296, 45372, 19500, 10604, 38396, 30980, 60396, 18256, 44280,
                65212, 52188, 4584,  61212, 31676, 54188, 17768, 10504, 41680,
                19624, 61616,
            },
            {
                34296, 5424,  15288, 27196, 19480, 63776, 32064, 16348, 38916,
                364,   14440, 41080, 19076, 34708, 8344,  16884, 56464, 20368,
                62864, 7796,  28568, 50172, 23696, 30536, 9132,  42188, 18988,
                32800, 37796, 50196, 2040,  60528, 25056, 44232, 1444,  22728,
                54884, 14772, 30316, 9520,  16620, 38200, 13300, 45328, 22768,
                51080, 29036, 46412, 12484, 64300, 36504, 19544, 60736, 49764,
                9804,  38132, 43896, 60452, 54980, 16264, 27640, 13548, 57560,
                36220, 65312, 25664, 39160, 19300, 5444,  63600, 44656, 34900,
                15792, 65140, 28364, 19052, 43828, 31368, 6384,  47976, 15212,
                19872, 52732, 12892, 5496,  62544, 43136, 31064, 39688, 3792,
                22260, 9824,  63964, 304,   46592, 35872, 4932,  49184, 37416,
                56220, 41632, 47060, 25044, 33200, 17760, 51460, 648,   38140,
                24492, 41176, 54328, 35024, 51936, 46236, 1676,  55508, 36484,
                29316, 21852, 15996, 56664, 224,   49844, 44956, 63304, 26088,
                48384, 8276,
            },
            {
                43056, 63004, 46824, 9424,  36416, 50764, 57636, 6260,  27968,
                48988, 64192, 57992, 24444, 43784, 63280, 38536, 47064, 10908,
                45176, 39604, 18224, 12432, 61740, 40100, 56204, 60228, 28264,
                3096,  55420, 8404,  14708, 46580, 19700, 39596, 27608, 58648,
                35420, 2896,  45484, 23536, 59156, 28180, 2080,  33416, 40768,
                62308, 840,   56540, 31532, 24744, 53620, 39208, 4,     23320,
                26672, 52968, 4828,  29180, 22184, 35008, 42544, 52432, 2112,
                30516, 43676, 10028, 60100, 49600, 23612, 13240, 57096, 8224,
                31004, 41728, 23740, 55944, 51120, 61184, 40120, 1276,  28712,
                38400, 59260, 49052, 26868, 35300, 14792, 23964, 48604, 59328,
                27152, 56524, 18744, 24276, 54384, 15988, 11756, 29136, 23764,
                3904,  16356, 58660, 53344, 7072,  48348, 42200, 31812, 59392,
                12236, 29868, 2788,  62176, 22692, 13248, 26044, 41008, 11120,
                6808,  33108, 39276, 12340, 23960, 34800, 6048,  15180, 36620,
                54580, 12840,
            },
            {
                50564, 22236, 31060, 55612, 2068,  22936, 11184, 43076, 53428,
                22204, 33172, 4388,  11588, 51692, 6088,  26860, 31740, 3956,
                60476, 25440, 34568, 5352,  46864, 20076, 33504, 13472, 50960,
                47240, 20696, 63820, 42336, 31784, 7068,  64912, 10964, 48432,
                19056, 52884, 62004, 40552, 6044,  52224, 47452, 60440, 26384,
                14748, 36040, 6928,  16992, 50112, 4616,  14924, 44752, 32004,
                63172, 19768, 40616, 1348,  46636, 6624,  59308, 24184, 5564,
                48452, 21224, 55428, 33212, 45956, 30228, 36968, 50436, 20212,
                60076, 3532,  37664, 5276,  10000, 16872, 24328, 63520, 55648,
                32780, 2944,  22956, 45236, 55948, 1328,  52260, 6872,  12096,
                36104, 41956, 49896, 29968, 40280, 58916, 43392, 63292, 52640,
                32184, 9312,  27996, 19924, 38896, 60572, 22640, 4476,  16964,
                49952, 56976, 8076,  27708, 43000, 5912,  33664, 63344, 20364,
                50792, 59112, 47152, 42704, 64244, 20216, 29992, 40404, 3080,
                57948, 18468,
            },
            {
                996,   37856, 60028, 17580, 44424, 29228, 62328, 34516, 18704,
                8568,  45896, 37684, 29756, 55772, 15120, 49412, 22608, 37040,
                54184, 15648, 43564, 57012, 52176, 2856,  25272, 6896,  16716,
                39036, 26592, 35484, 23212, 57264, 52672, 16112, 29920, 42036,
                8424,  26228, 12672, 33020, 19900, 57544, 10804, 17976, 8232,
                52912, 21392, 44284, 59812, 41376, 27740, 61716, 56264, 11748,
                7524,  58528, 15360, 51276, 64396, 10632, 39132, 32944, 62972,
                12540, 37824, 16892, 3068,  7428,  15516, 43044, 1116,  26216,
                54100, 45480, 13460, 58784, 29636, 36100, 49288, 12580, 42248,
                9448,  16668, 61592, 7680,  18808, 39200, 32612, 64608, 16948,
                44628, 2868,  13992, 5304,  33488, 8544,  20300, 1844,  39216,
                60744, 45108, 36536, 1264,  14108, 29432, 11108, 54788, 43988,
                36224, 20980, 47436, 17392, 59552, 49880, 57332, 16432, 45588,
                3180,  26352, 9264,  28572, 4764,  55108, 59992, 46880, 23628,
                28064, 33028,
            },
            {
                45824, 25480, 13680, 7216,  39460, 48188, 14728, 3908,  40484,
                57296, 26124, 16548, 61728, 21140, 2300,  42428, 58712, 13360,
                29984, 620,   63772, 10016, 27536, 37748, 65256, 53848, 44400,
                58924, 10092, 3944,  12668, 48748, 860,   36356, 61440, 4292,
                56052, 37672, 47664, 592,   44188, 25080, 36528, 42628, 31168,
                47920, 64904, 11444, 33000, 22520, 9416,  34868, 17932, 42492,
                48400, 33240, 36716, 23884, 30816, 18764, 50188, 15620, 41944,
                28716, 58400, 51304, 27236, 63220, 59212, 52624, 10576, 39636,
                16960, 33284, 48488, 19936, 43312, 54584, 7400,  21668, 46148,
                25944, 51904, 29944, 41240, 11020, 60400, 27900, 46600, 20584,
                30672, 52932, 61108, 22032, 64480, 50696, 25844, 46888, 17480,
                12724, 22476, 51104, 64884, 56380, 46452, 34440, 61924, 26268,
                5820,  64272, 32976, 11288, 38080, 21568, 528,   30716, 37084,
                54376, 62644, 18416, 52696, 37420, 14216, 10300, 51224, 7380,
                65428, 11236,
            },
            {
                58968, 4696,  51560, 64388, 32848, 54656, 24812, 59492, 49692,
                13200, 1592,  53696, 47400, 40004, 35308, 64716, 8632,  51388,
                19824, 48552, 22840, 32372, 16984, 42744, 11364, 34820, 380,
                31080, 61832, 51744, 29048, 39264, 18848, 24520, 44768, 33364,
                21440, 16388, 64444, 51568, 30480, 14888, 63380, 2564,  54996,
                5004,  28468, 39656, 1488,  54796, 47056, 5676,  52288, 29004,
                21676, 3952,  55448, 12300, 44884, 57336, 26688, 7732,  54552,
                464,   23164, 9040,  39908, 20376, 34204, 22904, 29236, 63920,
                6568,  60820, 24580, 588,   64252, 26820, 33508, 4604,  57516,
                37304, 64868, 34116, 48104, 21792, 53524, 4836,  37788, 9232,
                57004, 25500, 38944, 46132, 11412, 36836, 55900, 31212, 6084,
                57320, 33856, 6696,  43036, 24788, 3580,  18656, 8660,  40080,
                14952, 52380, 2340,  41868, 29508, 53544, 14368, 43452, 7752,
                23452, 13036, 35212, 1144,  44520, 22948, 32316, 17632, 42488,
                35356, 53564,
            },
            {
                40072, 28796, 42576, 21488, 10224, 704,   19504, 36876, 28636,
                65136, 43212, 32020, 7012,  10420, 25396, 17652, 32868, 5208,
                39012, 59376, 41600, 7516,  50476, 60428, 21248, 48708, 24300,
                18592, 41192, 14840, 45716, 62932, 6744,  54632, 11848, 50988,
                60272, 28452, 5224,  22792, 56788, 7724,  49540, 19668, 38756,
                23880, 58388, 15964, 50704, 19176, 63524, 25556, 38056, 1960,
                65468, 41200, 27300, 61044, 5656,  1868,  35636, 20448, 46288,
                61804, 35316, 43844, 48176, 12820, 56556, 6036,  47508, 36136,
                11996, 51856, 31424, 9360,  38712, 14588, 51604, 60948, 19112,
                13752, 116,   6208,  15672, 58336, 25676, 13424, 50496, 62256,
                700,   34660, 6180,  17576, 28400, 3132,  14912, 41304, 62384,
                26992, 48848, 10204, 30368, 16028, 40888, 52956, 48824, 31324,
                23004, 45404, 60128, 25040, 8528,  63784, 47712, 27280, 61136,
                40540, 48396, 31484, 58544, 50132, 62116, 39372, 56292, 2312,
                21028, 15092,
            },
            {
                8104,  18176, 57356, 35876, 27424, 61660, 45728, 6008,  10684,
                22764, 34996, 19852, 58092, 28988, 52776, 43728, 56424, 46200,
                26116, 11452, 35788, 55608, 1972,  30088, 14956, 57532, 8680,
                54500, 36660, 5084,  21784, 32536, 59004, 27064, 15256, 2176,
                39916, 9960,  42880, 36108, 11864, 40320, 26776, 59972, 34152,
                12564, 8600,  45580, 36448, 29808, 13536, 59536, 45128, 16016,
                54160, 8836,  14372, 47632, 38124, 43356, 63972, 51672, 11024,
                31860, 18328, 4008,  53772, 30128, 2812,  42124, 18836, 55340,
                44504, 21788, 41260, 57980, 17692, 45732, 2624,  40928, 31768,
                49980, 24060, 55708, 43236, 36120, 2172,  32972, 42592, 22584,
                15380, 49320, 58504, 54116, 43948, 60184, 23784, 52212, 424,
                18448, 38300, 54300, 20800, 58988, 35944, 62892, 27752, 172,
                57728, 12024, 34348, 18940, 56128, 3848,  35112, 11016, 19712,
                56440, 5552,  16144, 25780, 19836, 8012,  4208,  27296, 60720,
                49452, 31188,
            },
            {
                63512, 47352, 3152,  49584, 15984, 52984, 38992, 30976, 51432,
                56076, 4440,  48724, 14524, 61076, 816,   12384, 22212, 62100,
                2732,  29112, 18240, 63080, 24912, 39968, 46476, 5568,  28220,
                43324, 58364, 25908, 52804, 9204,  42300, 37608, 49500, 30804,
                57876, 46988, 19096, 53060, 61052, 32488, 4660,  46404, 16856,
                52508, 62620, 26028, 56304, 4908,  40740, 10332, 23348, 31384,
                49124, 19672, 34092, 52216, 22284, 15960, 29552, 24592, 40256,
                14424, 58984, 25872, 64788, 16568, 37180, 61904, 26708, 1904,
                15112, 28092, 3852,  49588, 34696, 62468, 23204, 29088, 10512,
                44948, 38468, 27792, 8928,  60988, 46000, 17664, 55464, 29232,
                40336, 10316, 31252, 19984, 6988,  32872, 47916, 9544,  35472,
                44740, 13488, 63896, 2164,  47668, 6160,  10916, 21244, 54972,
                37904, 5308,  50640, 15344, 44156, 39084, 22724, 52468, 32892,
                1824,  44012, 63844, 10644, 41908, 30448, 47852, 36884, 12792,
                44244, 24752,
            },
            {
                38528, 33828, 11516, 24064, 43684, 8272,  59200, 13852, 41348,
                16752, 62668, 38216, 23940, 40840, 50376, 36240, 31036, 15976,
                53844, 42928, 48092, 13940, 4328,  53540, 37004, 63592, 33400,
                12576, 2244,  64224, 17284, 47416, 4180,  20108, 65432, 23076,
                7196,  34416, 26060, 1428,  15764, 43884, 21308, 56520, 264,
                42100, 31512, 21692, 2240,  43788, 33588, 57580, 6816,  36976,
                61764, 25368, 57824, 6552,  31632, 9532,  60016, 2700,  56904,
                5176,  50344, 38932, 10388, 45688, 23808, 48996, 9084,  57764,
                34284, 65332, 54024, 6244,  25640, 12108, 54352, 7816,  59752,
                52660, 17376, 63584, 20416, 30116, 51512, 11192, 65056, 4280,
                47224, 24664, 63264, 42056, 12796, 38116, 65512, 21060, 29420,
                59796, 23480, 42024, 32428, 26256, 14396, 44996, 33412, 17076,
                65132, 41072, 29764, 62104, 26724, 49684, 6128,  58908, 17100,
                38340, 27780, 53596, 35896, 57732, 15172, 62920, 23180, 9336,
                55388, 480,
            },
            {
                59300, 14576, 53612, 62836, 29836, 3776,  20704, 25304, 46924,
                1160,  27604, 9316,  45628, 5396,  18304, 64136, 6752,  38388,
                9340,  60560, 21148, 34460, 44348, 10712, 17496, 22920, 51152,
                20036, 49260, 35280, 30104, 13772, 55916, 34072, 10820, 45380,
                17240, 62472, 55780, 48560, 29744, 63676, 10560, 50452, 28832,
                38120, 7348,  49352, 61088, 15144, 20676, 51060, 46936, 416,
                12552, 39632, 3416,  63488, 41724, 55060, 19288, 48684, 44372,
                28456, 34772, 20044, 7064,  32652, 59760, 13520, 31280, 50860,
                38412, 19632, 10404, 30416, 47432, 38176, 19780, 42884, 35168,
                3384,  12816, 47252, 4936,  40832, 23580, 7028,  38096, 19652,
                59416, 35612, 2632,  52716, 27332, 56364, 16500, 5196,  53836,
                50136, 4064,  8616,  57620, 51820, 39464, 60168, 50080, 7836,
                25404, 47308, 20460, 9864,  1060,  31520, 14140, 64916, 46156,
                23728, 50540, 7368,  21700, 2648,  45344, 52588, 18052, 34664,
                51012, 20020,
            },
            {
                26000, 6936,  41864, 18588, 37780, 56644, 35216, 63948, 31960,
                54520, 19572, 59764, 29996, 33668, 56140, 26816, 46520, 51700,
                23992, 31736, 6352,  50272, 26188, 58964, 31972, 956,   60480,
                40564, 9664,  44660, 23780, 39572, 59848, 28,    28864, 54408,
                38204, 3328,  12968, 23312, 38980, 5908,  35720, 24348, 14208,
                65236, 18120, 35172, 10952, 53548, 26452, 63980, 17752, 28484,
                56596, 44340, 16728, 27892, 46440, 13304, 37532, 33404, 8152,
                22676, 61588, 56172, 41656, 51584, 196,   21424, 40584, 6632,
                24384, 46492, 43164, 61336, 16156, 57252, 1720,  64400, 26944,
                22516, 57312, 31884, 54004, 14980, 34444, 57976, 27252, 53660,
                14112, 8500,  17172, 49116, 22652, 1192,  45756, 40364, 12284,
                33992, 17124, 28424, 37152, 18756, 22632, 1804,  28892, 36572,
                13388, 3628,  59420, 53280, 36144, 56792, 40956, 8624,  29536,
                11728, 60680, 14436, 33152, 40204, 28320, 4968,  57256, 42860,
                6112,  29724,
            },
            {
                46196, 56176, 32328, 1268,  50880, 10148, 48276, 6524,  12092,
                43532, 36580, 52636, 12900, 49444, 2024,  20840, 10912, 58548,
                40916, 32,    56400, 65124, 39128, 7908,  55260, 45960, 14068,
                26520, 56696, 3584,  62220, 7472,  50560, 25960, 43260, 14360,
                51444, 31788, 41524, 9076,  58088, 18544, 53604, 60180, 45620,
                3980,  55156, 23516, 46152, 38820, 31996, 8628,  42460, 34892,
                22452, 51300, 10688, 36096, 24056, 748,   52308, 65116, 16996,
                12032, 47244, 3672,  15276, 28768, 54572, 44288, 63508, 16924,
                59568, 12964, 1032,  36340, 22300, 50816, 32056, 14256, 49032,
                7252,  41968, 37292, 864,   62788, 48044, 43372, 3280,  32096,
                45316, 40732, 29860, 60700, 36856, 10536, 31560, 58180, 26476,
                63020, 43264, 55784, 11080, 63388, 5668,  54420, 43804, 61416,
                51760, 32016, 42752, 24084, 45700, 17568, 21396, 48960, 36748,
                5052,  42348, 56332, 47348, 64296, 11308, 24480, 32420, 13732,
                64800, 39944,
            },
            {
                10792, 61840, 21480, 27408, 44756, 16132, 23544, 40156, 60844,
                3680,  22688, 7264,  42280, 16100, 60784, 44440, 29428, 15056,
                34200, 18948, 12396, 28680, 15480, 20436, 41924, 29600, 6900,
                37784, 16660, 32700, 53704, 21588, 18096, 36560, 63128, 5372,
                20496, 60360, 27268, 45084, 50140, 31404, 2316,  40492, 26912,
                8984,  42952, 58368, 28972, 5832,  2584,  59740, 14024, 54236,
                6248,  30992, 58972, 48904, 61428, 20908, 5572,  26584, 42792,
                53748, 36796, 26148, 64028, 18740, 34388, 10848, 27008, 2972,
                51924, 33164, 56104, 27648, 9140,  4716,  45212, 38808, 55336,
                18548, 61880, 28436, 9896,  25724, 18320, 12416, 22060, 61400,
                50976, 25284, 56996, 4636,  44176, 63700, 18624, 47724, 7312,
                21600, 49280, 3336,  46104, 30984, 41456, 16300, 24376, 9700,
                19252, 39108, 15664, 5800,  11332, 62324, 26396, 1608,  55620,
                32212, 20060, 25892, 112,   18424, 49756, 39092, 59852, 1680,
                48656, 17012,
            },
            {
                3760,  34924, 52564, 13044, 64548, 58140, 33756, 28000, 50068,
                18000, 57996, 26416, 65440, 39420, 23604, 36676, 3844,  63348,
                47688, 54268, 44936, 36824, 2532,  48556, 62300, 23748, 51536,
                64804, 47872, 28292, 12084, 42612, 48508, 10032, 57520, 24024,
                47624, 35160, 6484,  64816, 15152, 22500, 12164, 33964, 48916,
                20276, 32520, 13308, 62992, 17252, 49232, 40240, 20228, 47460,
                64732, 1752,  18556, 8164,  32988, 55584, 40424, 30632, 59704,
                1452,  32008, 8956,  49476, 38852, 6172,  56928, 36868, 48436,
                29824, 42268, 18864, 64076, 40436, 60388, 29132, 10812, 24660,
                34344, 46508, 15464, 59220, 52404, 38684, 49836, 8068,  36456,
                252,   11648, 20848, 33564, 14916, 52376, 24092, 35844, 1812,
                39004, 14328, 24804, 34916, 58616, 13120, 49024, 34260, 57172,
                868,   63168, 54924, 28408, 58604, 33428, 51100, 43752, 63200,
                16444, 52452, 37888, 61508, 30420, 7076,  54512, 21084, 27836,
                36672, 23840,
            },
            {
                50076, 42720, 8040,  38460, 5348,  19832, 2096,  54220, 13996,
                30572, 46504, 34544, 10228, 5628,  31700, 54648, 50604, 8504,
                25828, 4796,  59456, 24272, 52864, 32856, 11652, 4224,  35560,
                19352, 1796,  59124, 38512, 4776,  29644, 33484, 2064,  39904,
                11672, 17464, 52720, 676,   37172, 43328, 62276, 57040, 16332,
                53208, 37440, 916,   51252, 22200, 55756, 34088, 25764, 11476,
                37620, 43380, 26760, 39168, 15096, 44944, 10868, 49648, 14304,
                20452, 45224, 24200, 58568, 13660, 46316, 23276, 62028, 20372,
                8112,  53964, 5320,  14624, 23460, 52580, 17424, 58480, 5780,
                51532, 2588,  20892, 43720, 6544,  32804, 63440, 27720, 56148,
                17732, 64568, 47360, 6940,  42204, 28800, 8924,  55100, 61708,
                30004, 59888, 53664, 19584, 7944,  27208, 64728, 4508,  29852,
                44564, 21868, 35220, 48332, 2928,  38680, 8080,  13160, 24124,
                3644,  46432, 8828,  13328, 43536, 34220, 15600, 45904, 9236,
                53220, 60716,
            },
            {
                14948, 29412, 24960, 55504, 30920, 41416, 46056, 8720,  63544,
                38292, 328,   55644, 51716, 20328, 59216, 13116, 18284, 39056,
                42800, 21768, 31256, 9640,  40696, 18288, 59956, 44216, 54716,
                10172, 22396, 44908, 15260, 56800, 64220, 16512, 53552, 44640,
                30280, 58376, 25688, 19916, 55804, 27556, 8120,  3144,  24136,
                64148, 6656,  41840, 25460, 45032, 9676,  62428, 29972, 3880,
                57304, 22872, 51816, 62956, 2616,  57820, 24560, 6972,  38308,
                55152, 62224, 40864, 2404,  53156, 30308, 4004,  15652, 40144,
                11860, 25512, 37480, 44492, 48140, 33988, 436,   43140, 65148,
                13372, 30900, 36692, 55172, 23932, 2100,  41492, 14596, 44592,
                30364, 39392, 53248, 25864, 59176, 3676,  39920, 13440, 50752,
                17600, 9748,  42480, 312,   50892, 40064, 21104, 37656, 47276,
                12608, 26204, 7228,  41848, 14488, 20784, 30272, 59896, 40656,
                34596, 28840, 57828, 22984, 50720, 56852, 2860,  63660, 41596,
                32860, 19764,
            },
            {
                392,   47136, 59252, 17200, 49108, 61384, 25656, 35584, 21068,
                43876, 11732, 17040, 29084, 48144, 43552, 1104,  27964, 33596,
                61196, 15952, 49328, 64356, 5916,  56168, 26300, 14484, 30848,
                41136, 61612, 34096, 25024, 8556,  50884, 22192, 27048, 61360,
                7900,  49448, 41000, 10500, 46264, 30904, 51612, 38732, 47532,
                28600, 10724, 59400, 30800, 36160, 15048, 6848,  41300, 53672,
                16392, 32504, 12664, 46992, 29464, 19492, 34920, 64156, 28904,
                4688,  17116, 10312, 33572, 19716, 43448, 59500, 34076, 55476,
                46780, 63044, 58024, 3424,  30760, 12468, 20624, 37232, 26544,
                40628, 57464, 48116, 11480, 29540, 59872, 20040, 54060, 4352,
                23752, 9404,  35084, 16780, 49508, 62600, 19860, 44552, 33100,
                26840, 46700, 37424, 31896, 61860, 56092, 10772, 16968, 54056,
                60928, 51056, 18036, 65248, 52868, 57000, 46980, 18372, 54484,
                10836, 64880, 41764, 4548,  27468, 19436, 37644, 24796, 12872,
                6480,  56744,
            },
            {
                65232, 10352, 36584, 22616, 3480,  11272, 15684, 52316, 4832,
                57360, 24596, 62200, 4144,  40664, 25212, 63764, 56008, 10944,
                51828, 1728,  37588, 13552, 45940, 34676, 520,   38876, 49632,
                7432,  27660, 53024, 47388, 652,   40236, 35984, 12992, 3032,
                37704, 15488, 63232, 34272, 3928,  61512, 11796, 18968, 34844,
                44736, 14616, 54204, 18832, 2420,  57616, 46332, 19304, 49092,
                61596, 8712,  36304, 4236,  53464, 41592, 16188, 51188, 46112,
                23444, 36020, 50432, 27240, 65264, 7716,  25088, 51336, 628,
                31524, 21760, 28404, 9616,  53424, 61792, 56248, 49932, 8920,
                18184, 22972, 3752,  63828, 16864, 46252, 9036,  33340, 49228,
                61872, 57268, 2568,  12248, 37696, 31260, 23128, 56728, 6296,
                2844,  64460, 22600, 15816, 6616,  24436, 43616, 2380,  32688,
                5504,  39792, 9980,  31264, 23744, 36468, 2000,  26940, 6596,
                49856, 20880, 16232, 36284, 60952, 8340,  47172, 59504, 29656,
                44780, 38552,
            },
            {
                21336, 31964, 6688,  43772, 53140, 39244, 32452, 59904, 27928,
                41288, 50304, 32728, 37476, 9044,  14560, 35732, 44828, 6960,
                20836, 57104, 26580, 29804, 19808, 51184, 22980, 62604, 17020,
                57720, 4568,  12628, 19040, 31160, 60132, 45964, 20100, 55312,
                32040, 23192, 5292,  53200, 21364, 14900, 42344, 57504, 5896,
                60672, 21752, 39052, 48324, 64852, 27248, 37448, 23304, 0,
                27896, 44840, 21232, 58584, 25560, 9372,  60488, 2320,  12728,
                42368, 60872, 55932, 14384, 47788, 37812, 11428, 18156, 42856,
                5892,  15040, 50332, 18480, 36444, 25012, 6496,  45820, 32284,
                62420, 53228, 7444,  34008, 38548, 51392, 26644, 36880, 13296,
                40972, 21684, 46328, 27140, 54564, 636,   10432, 48020, 35528,
                58248, 12848, 54652, 48184, 29064, 35800, 49700, 58520, 27756,
                22536, 34880, 58956, 45068, 5008,  11932, 43200, 60676, 32248,
                38180, 776,   47572, 53984, 11676, 31652, 51824, 17212, 2328,
                54764, 26164,
            },
            {
                41224, 51320, 61868, 13884, 28284, 64216, 1548,  47936, 18104,
                7404,  13696, 22136, 53368, 60656, 19468, 49636, 23340, 31744,
                41092, 46976, 63028, 8320,  42648, 59032, 9820,  29360, 45144,
                21464, 35248, 63824, 43064, 56276, 10216, 5036,  50048, 65284,
                42916, 47928, 28888, 59980, 36944, 26684, 48420, 23576, 384,
                31912, 50716, 8476,  4420,  32928, 12392, 52748, 60416, 33860,
                13668, 40112, 65464, 6460,  48992, 38004, 32168, 20984, 54308,
                8160,  30384, 984,   22072, 5132,  56668, 28776, 62564, 35532,
                60136, 38924, 43892, 64796, 1992,  41188, 16344, 28588, 2672,
                14288, 44296, 27600, 42244, 12896, 57896, 1508,  65456, 18148,
                7000,  31616, 60692, 43092, 18996, 64900, 41056, 16308, 25156,
                39136, 20012, 44004, 9568,  3464,  60412, 19048, 13356, 41364,
                63328, 15632, 428,   50384, 29072, 62456, 16764, 51592, 23256,
                14528, 61976, 29376, 23716, 43248, 3784,  40588, 22660, 33940,
                49516, 15412,
            },
            {
                4288,  18644, 46740, 34700, 55704, 19308, 23472, 10208, 45248,
                34564, 64600, 1396,  45680, 30564, 5696,  57364, 2396,  53812,
                12972, 17800, 4248,  33648, 54908, 3056,  37132, 12956, 53980,
                1760,  48784, 38156, 15740, 23672, 28852, 34436, 17256, 26220,
                11320, 7568,  18368, 44868, 1672,  55572, 33036, 62720, 16988,
                41616, 27876, 56640, 24416, 43416, 17300, 9252,  42260, 5332,
                51004, 55568, 18188, 30284, 43780, 14184, 57184, 26732, 34504,
                48264, 17880, 39508, 45408, 32936, 41560, 49416, 23416, 8796,
                54076, 25856, 12052, 32820, 47372, 22188, 60996, 34652, 59096,
                37624, 50572, 18928, 61208, 24816, 6388,  22044, 44164, 55764,
                28260, 52076, 15264, 5596,  34392, 50248, 29496, 59628, 52708,
                7892,  28048, 33716, 62976, 40416, 53204, 31724, 45764, 7960,
                55180, 48616, 25336, 19508, 38652, 54364, 34420, 9092,  40532,
                55468, 44760, 6968,  34184, 64240, 14032, 56636, 62904, 7408,
                11420, 60652,
            },
            {
                57564, 30332, 1052,  25372, 7928,  42416, 37188, 58192, 51920,
                29032, 39356, 55880, 11260, 26920, 42332, 16052, 64928, 28560,
                35448, 60432, 39212, 23768, 15108, 47964, 25920, 65224, 41792,
                31528, 9188,  26680, 51072, 6732,  62312, 42104, 58812, 1016,
                39428, 57716, 50664, 13352, 40312, 9948,  6888,  52236, 37452,
                11152, 63880, 13832, 35540, 61620, 47180, 21288, 56772, 30964,
                24604, 3224,  35040, 11240, 23704, 708,   62168, 40076, 3712,
                64576, 24732, 58184, 9440,  63364, 16408, 1724,  13528, 46064,
                20404, 3764,  58412, 7700,  54984, 13448, 49240, 9304,  54124,
                24112, 10776, 1124,  30808, 54312, 48060, 40252, 32556, 3476,
                48516, 10788, 38664, 58384, 24632, 8744,  13728, 4108,  45416,
                62136, 1056,  49928, 23560, 17624, 11400, 26180, 1696,  21368,
                37112, 11116, 42904, 57852, 13620, 6360,  21216, 48336, 3560,
                26532, 11104, 58560, 17572, 49096, 26716, 20224, 37264, 46172,
                27920, 35676,
            },
            {
                9412,  53736, 39504, 12268, 59372, 49556, 5136,  14800, 25096,
                3044,  20236, 16468, 49136, 59440, 33688, 39000, 46048, 8832,
                25320, 49220, 11200, 52144, 61516, 19060, 32808, 5268,  17472,
                22860, 60900, 55796, 2452,  45720, 20884, 37764, 13944, 53988,
                21584, 32376, 35412, 24736, 64588, 29844, 20308, 43724, 25768,
                3448,  45692, 19444, 53776, 1128,  29588, 6788,  37996, 63444,
                15336, 46136, 61376, 50024, 55064, 17104, 45748, 51036, 10924,
                15196, 52600, 35908, 27612, 20620, 51236, 31104, 57300, 39836,
                34104, 50800, 29488, 19184, 38508, 26380, 43172, 5044,  20160,
                40824, 46932, 57328, 36164, 16424, 10140, 62840, 14056, 24000,
                35020, 61724, 21020, 1440,  53356, 44204, 37920, 22312, 31908,
                18532, 42292, 14572, 55928, 4592,  38264, 64104, 51396, 57552,
                29396, 32796, 3048,  64268, 27908, 46744, 31352, 63140, 57176,
                19004, 36344, 51424, 1964,  39440, 9356,  30604, 720,   52820,
                41740, 23844,
            },
            {
                48296, 63932, 16448, 44920, 21184, 33356, 29676, 63588, 54108,
                42896, 61296, 36360, 8132,  22528, 3864,  12448, 51900, 21628,
                58704, 192,   42228, 30756, 6756,  44544, 38724, 50756, 57872,
                47096, 14060, 40388, 33528, 11820, 52728, 30732, 4040,  47324,
                27788, 61260, 2968,  53492, 16824, 58160, 49104, 14096, 59276,
                55000, 30040, 49480, 6192,  40752, 58708, 25680, 48892, 9848,
                19876, 39468, 27392, 8100,  36848, 31712, 6256,  20352, 29500,
                43304, 5508,  46876, 12624, 3800,  37196, 60056, 6800,  26884,
                64316, 15476, 45028, 61540, 272,   31576, 57688, 63196, 29668,
                15064, 32964, 64744, 4444,  45124, 20488, 29192, 50524, 57736,
                7916,  41588, 26804, 47652, 32748, 17228, 63112, 55432, 9828,
                51580, 36608, 30192, 58764, 47132, 34180, 42524, 14700, 6204,
                47600, 16800, 51908, 8788,  41060, 23648, 1008,  37528, 13140,
                29148, 42284, 22228, 32620, 60044, 44184, 55364, 16700, 61564,
                13656, 19844,
            },
            {
                6152,  36792, 28704, 52164, 61736, 3340,  17720, 40168, 10612,
                6336,  32132, 44864, 25848, 53084, 62936, 18408, 30376, 6136,
                37952, 16564, 55536, 20464, 27956, 56512, 1096,  11532, 35152,
                7044,  29684, 19788, 64488, 25364, 57456, 8492,  44416, 63756,
                15916, 9596,  41480, 6052,  45900, 37868, 2324,  34788, 22000,
                8020,  33500, 38792, 22976, 15424, 51376, 12204, 34856, 43608,
                53628, 1220,  59368, 21552, 42028, 64188, 26128, 56040, 37880,
                61776, 19128, 31300, 60804, 55276, 44028, 24464, 17888, 53056,
                2904,  10196, 36644, 22672, 51724, 11268, 16636, 36000, 2252,
                52000, 7620,  22492, 26880, 39112, 52752, 84,    37544, 18728,
                45828, 15936, 64176, 12008, 56564, 28708, 2784,  47332, 26636,
                60252, 5124,  12212, 25552, 8260,  20104, 28268, 59404, 24232,
                39784, 61240, 20636, 35392, 55960, 60740, 17204, 52340, 45364,
                7512,  64016, 4608,  53920, 12748, 24380, 6644,  47804, 31772,
                3364,  43640,
            },
            {
                59868, 32816, 2056,  10200, 26036, 56088, 48048, 35140, 22824,
                50692, 12932, 56872, 1036,  40528, 46964, 35976, 57100, 43272,
                62504, 33060, 46796, 9580,  64084, 14324, 24040, 61992, 27136,
                43240, 52112, 4416,  36852, 16260, 48968, 19000, 34268, 23892,
                37220, 52300, 19996, 31116, 23336, 11500, 27432, 61144, 40944,
                17112, 62476, 10784, 27244, 65168, 32304, 18536, 60472, 5076,
                28880, 33276, 13612, 52372, 3892,  12252, 48160, 33628, 8356,
                23724, 53576, 88,    40032, 22340, 8620,  47648, 12072, 41244,
                31664, 48948, 42448, 6072,  56196, 46716, 39608, 24756, 48776,
                60196, 41672, 12460, 55936, 61012, 8800,  33692, 59748, 4980,
                54872, 31128, 3208,  36404, 20244, 7392,  41992, 34912, 15284,
                40136, 21320, 64528, 44116, 53300, 220,   50192, 9712,  35988,
                3824,  44896, 26440, 14320, 4740,  43812, 10780, 33180, 58936,
                24292, 49696, 15900, 46180, 28348, 37596, 64680, 21260, 35028,
                56968, 25240,
            },
            {
                54712, 14144, 22708, 42976, 37944, 7592,  13808, 43708, 60268,
                28016, 18016, 64736, 29544, 14736, 9772,  24192, 2028,  48760,
                13640, 25976, 3452,  36508, 41336, 31408, 52928, 45792, 20988,
                59660, 10528, 55204, 41472, 1224,  62052, 29436, 5760,  59616,
                12584, 48868, 43544, 57224, 62676, 54232, 5500,  46940, 52528,
                596,   43868, 55652, 4296,  46408, 41804, 2020,  55328, 22384,
                64368, 47604, 18244, 45000, 24488, 58416, 16012, 2120,  41248,
                59228, 13844, 48964, 34456, 15832, 29424, 61908, 35136, 56492,
                21460, 62656, 25376, 17488, 28344, 65360, 8116,  21236, 55292,
                28112, 18432, 35452, 48172, 15440, 24644, 44580, 13400, 27816,
                39800, 23528, 49628, 43660, 60024, 52152, 11352, 23820, 57696,
                1424,  48380, 32844, 37744, 16096, 62088, 32024, 55624, 17972,
                65488, 11708, 53460, 30452, 38496, 50400, 27412, 20144, 2804,
                39888, 31232, 9840,  60800, 18572, 2508,  51324, 11520, 40376,
                8348,  49612,
            },
            {
                38804, 19120, 65444, 50928, 58320, 30352, 20548, 56,    55048,
                4588,  37616, 47708, 21188, 33204, 61000, 53632, 28288, 20112,
                8672,  54508, 60468, 22464, 48872, 5664,  17604, 39560, 3040,
                15456, 32404, 23344, 46540, 27308, 11140, 43384, 39352, 53780,
                228,   26492, 7924,  34592, 14520, 40068, 18944, 32140, 24032,
                12572, 50088, 19280, 35840, 58052, 9048,  24964, 37108, 15696,
                39840, 10640, 57108, 35644, 30892, 38788, 51372, 63360, 18088,
                45280, 27044, 9912,  65208, 43116, 4196,  50616, 6536,  27528,
                932,   13616, 53832, 35372, 3248,  33440, 14224, 45360, 3932,
                10656, 43796, 1532,  30568, 5204,  41280, 63244, 21912, 51732,
                10700, 61128, 6456,  14816, 25772, 38464, 65216, 17660, 50836,
                28820, 9180,  55280, 5692,  23476, 40608, 13068, 43188, 22148,
                49568, 33540, 2528,  60112, 22432, 7884,  64708, 54336, 14236,
                35972, 56212, 21052, 41120, 34616, 58004, 43084, 29780, 62652,
                17108, 27804,
            },
            {
                11012, 46872, 4920,  34360, 16632, 45804, 63396, 49276, 32756,
                24484, 42004, 7664,  51400, 4112,  42912, 16644, 39120, 65340,
                35144, 29956, 18128, 51932, 12036, 59288, 33928, 8728,  64860,
                37276, 50268, 5512,  61084, 35124, 56336, 50404, 15560, 21664,
                30712, 65172, 17680, 4116,  28732, 49868, 9716,  64644, 37252,
                28104, 59584, 31312, 22140, 14428, 53076, 29928, 44356, 50224,
                7340,  26796, 2692,  62196, 5400,  9476,  22404, 28996, 36080,
                4580,  32148, 51960, 20272, 25684, 55904, 38268, 19256, 45888,
                58264, 38728, 8980,  59964, 50484, 40736, 57340, 31480, 61816,
                38296, 53376, 64328, 50676, 19240, 57180, 31904, 2184,  46424,
                35672, 19036, 56920, 33420, 576,   46772, 30464, 4272,  42748,
                61320, 13860, 45400, 18708, 59920, 27440, 2952,  46892, 29524,
                7188,  56984, 40672, 47836, 16304, 34384, 46296, 41648, 6200,
                61420, 26696, 60,    48800, 7580,  25068, 14852, 4268,  23376,
                44964, 492,
            },
            {
                60960, 29592, 41164, 24820, 2536,  11416, 26556, 39320, 9512,
                15776, 62624, 11984, 56324, 35396, 58824, 6320,  11760, 51052,
                45040, 6716,  38228, 43736, 1576,  27484, 56072, 47328, 24344,
                28540, 58032, 18332, 13708, 20864, 7932,  25060, 3460,  58036,
                41232, 36092, 46208, 60920, 21864, 44672, 1600,  56480, 15368,
                42464, 8564,  2960,  39868, 48388, 5528,  63216, 11788, 58944,
                32660, 53400, 41704, 20424, 54852, 43516, 47340, 12028, 54264,
                57796, 7364,  38736, 58720, 1484,  13000, 30988, 63580, 15532,
                32968, 43308, 20732, 46372, 11808, 23384, 18640, 5880,  25444,
                16416, 32784, 21856, 26568, 9296,  37740, 53288, 16552, 7820,
                64416, 29832, 41504, 53416, 21428, 9796,  54772, 34412, 20572,
                37408, 25660, 31184, 51848, 35324, 49120, 9016,  63100, 38164,
                14556, 25572, 19296, 9852,  55272, 1492,  24880, 30100, 17820,
                51264, 44040, 13264, 62896, 55560, 32196, 46800, 54000, 59608,
                36392, 52504,
            },
            {
                21032, 56760, 9348,  62164, 53092, 36788, 56568, 18308, 51740,
                59140, 45512, 30660, 26872, 18696, 24600, 46380, 32036, 21916,
                2940,  57212, 14692, 24728, 63516, 40816, 19532, 13476, 53952,
                348,   39864, 43904, 31420, 52544, 38152, 62552, 33096, 47300,
                9884,  54828, 13216, 52128, 38224, 59240, 25896, 34148, 6516,
                52892, 63504, 45496, 25792, 60540, 18068, 34648, 20684, 140,
                23596, 17316, 48752, 29772, 14480, 25624, 60200, 844,   20028,
                42184, 24072, 46484, 16732, 36200, 44832, 52772, 9756,  24120,
                49888, 6184,  26488, 30600, 1392,  62372, 43156, 51892, 47848,
                58852, 12936, 6500,  45776, 60756, 12332, 28192, 49012, 42632,
                25356, 3960,  12364, 48344, 27628, 59508, 15700, 45120, 5992,
                56580, 1968,  63816, 10680, 4376,  57128, 20176, 33216, 52596,
                924,   61344, 44496, 28216, 63036, 39412, 12232, 59676, 47364,
                9644,  23252, 37864, 28948, 17956, 10616, 38900, 20116, 7284,
                13100, 33288,
            },
            {
                48956, 6324,  15592, 31572, 21884, 42636, 5900,  29468, 1700,
                35680, 21504, 3656,  40208, 49728, 496,   63248, 55224, 41496,
                26316, 61900, 47556, 9468,  31076, 36088, 4088,  44384, 32984,
                11088, 49028, 7240,  63876, 2188,  45608, 12904, 17272, 27768,
                6140,  19888, 24988, 3112,  30328, 11700, 41528, 19912, 48252,
                23156, 16880, 33680, 55912, 10340, 28616, 51468, 40440, 46684,
                61824, 36460, 11252, 65372, 34160, 7172,  39432, 31568, 48840,
                14908, 63836, 10940, 28828, 60240, 21976, 7208,  40808, 60932,
                2148,  53320, 64412, 16544, 55668, 36900, 27692, 9652,  35592,
                672,   41976, 55392, 34404, 40244, 4648,  21272, 33620, 54756,
                59040, 38112, 17688, 62776, 7564,  40196, 24148, 63104, 11824,
                49936, 41444, 22168, 15872, 39020, 42932, 26152, 16908, 58640,
                22760, 50324, 36300, 32188, 4372,  50748, 21072, 35500, 5160,
                64264, 32824, 4020,  52192, 44856, 1660,  65040, 50212, 30736,
                42220, 25472,
            },
            {
                17940, 38636, 44452, 50096, 59564, 13652, 47960, 64908, 23800,
                41512, 13876, 54768, 61480, 10124, 37164, 14548, 29164, 8236,
                17560, 33260, 52272, 20716, 58260, 16044, 50596, 62528, 21540,
                60064, 16672, 26612, 22680, 55268, 28944, 40684, 59780, 50044,
                63152, 43440, 33708, 57888, 8108,  17280, 62808, 55176, 3260,
                38740, 29700, 13024, 1024,  37336, 43588, 4228,  13800, 57152,
                27396, 5828,  44968, 1704,  58144, 51136, 17092, 62640, 27360,
                35284, 3036,  50276, 54664, 5480,  33876, 48136, 27256, 18272,
                36808, 12780, 34188, 40368, 48644, 4736,  14672, 54588, 19856,
                29452, 62784, 23520, 49752, 17564, 57484, 62068, 452,   14824,
                9484,  22964, 52180, 35240, 43932, 3072,  32368, 51664, 27916,
                18144, 32752, 48008, 58868, 29368, 53784, 12288, 45624, 4620,
                41356, 8184,  11964, 17448, 58904, 42252, 15540, 57052, 27184,
                39248, 54960, 14992, 21496, 57612, 34040, 27108, 16092, 58284,
                3600,  63208,
            },
            {
                54636, 1512,  27724, 35736, 3444,  19412, 33464, 9148,  57928,
                46436, 6948,  32980, 17260, 52764, 44108, 20656, 48928, 60124,
                12588, 37260, 852,   42540, 5080,  54460, 25972, 8244,  29200,
                42088, 36776, 51464, 34476, 10180, 19312, 4432,  35304, 22568,
                812,   39048, 14456, 49468, 45560, 36420, 51096, 31716, 10040,
                58020, 46384, 61204, 21112, 52724, 24308, 64096, 30932, 8216,
                38576, 16204, 56056, 24544, 19796, 37000, 44488, 3972,  56208,
                8824,  40680, 31944, 18684, 42604, 64808, 14380, 55324, 30340,
                59072, 47144, 19940, 8056,  60792, 22548, 32192, 64844, 39856,
                46916, 8716,  14432, 2664,  30896, 43968, 24892, 47748, 39788,
                31468, 45972, 1892,  29568, 14092, 55360, 19684, 38760, 7036,
                60356, 36084, 8316,  24636, 532,   64584, 6748,  35072, 30616,
                63408, 27520, 54528, 47284, 24324, 7492,  30584, 53080, 952,
                18688, 43096, 60340, 7800,  40572, 48064, 5684,  22532, 37516,
                9928,  45932,
            },
            {
                12868, 23756, 64472, 11040, 55660, 25928, 40036, 53412, 16616,
                27548, 62884, 48196, 22796, 30020, 7388,  34752, 2400,  40160,
                56012, 45892, 27172, 64420, 23124, 12688, 38052, 48012, 2276,
                55688, 5812,  13340, 59188, 42392, 48636, 56916, 11776, 53040,
                31228, 56296, 27084, 64072, 23396, 1252,  27744, 14276, 43424,
                25488, 5144,  50016, 41088, 8596,  59256, 17520, 49172, 22248,
                54144, 31888, 49940, 40688, 29068, 9160,  12756, 23504, 52068,
                20740, 61436, 26104, 12352, 1988,  25036, 38144, 4076,  10544,
                41568, 3168,  25528, 29288, 44608, 11036, 50628, 2900,  25152,
                16900, 57860, 37628, 27188, 53888, 10292, 36684, 6976,  65344,
                19100, 55976, 60284, 25592, 49060, 10676, 58644, 46188, 15124,
                2644,  43336, 52524, 13780, 46556, 37484, 21624, 56392, 48572,
                14240, 39008, 20392, 1352,  64820, 37692, 44952, 10288, 48688,
                24564, 11792, 29632, 36396, 24976, 11264, 60968, 44240, 55984,
                29296, 34572,
            },
            {
                60236, 40856, 52304, 30704, 7372,  62084, 44604, 2200,  31320,
                37592, 11180, 1372,  39336, 57624, 26172, 64476, 53472, 23428,
                5476,  19064, 9920,  49368, 31604, 40932, 60648, 34136, 19268,
                65088, 24516, 46096, 988,   31828, 64324, 7540,  25956, 44844,
                16228, 7108,  10488, 19012, 41332, 5888,  60684, 21520, 36992,
                65120, 18292, 34792, 27952, 15032, 32492, 45712, 1336,  42376,
                10568, 61580, 3148,  14076, 60600, 53268, 46912, 30512, 42808,
                16148, 37472, 57396, 45532, 52520, 20904, 49360, 57836, 45184,
                22812, 63540, 54368, 15036, 58220, 37600, 42532, 59696, 6704,
                33804, 43668, 50728, 5640,  63832, 20600, 15828, 51944, 28356,
                11968, 41920, 5284,  16412, 63404, 36948, 22740, 30780, 65096,
                25316, 55968, 20532, 61800, 31940, 17892, 50864, 9472,  24456,
                2996,  57568, 43976, 33768, 52048, 13908, 21948, 58576, 34052,
                63416, 46144, 50644, 2088,  63056, 18132, 52496, 13768, 716,
                50908, 19616,
            },
            {
                47952, 4532,  20896, 15292, 46732, 36452, 22884, 13088, 50684,
                20728, 55744, 43052, 60708, 4668,  13192, 46324, 16184, 32560,
                28728, 61548, 35148, 57444, 16648, 6508,  45112, 10564, 14880,
                30836, 52996, 38836, 21376, 27404, 15452, 37200, 20148, 40348,
                61160, 47744, 37356, 52840, 58256, 34480, 46668, 54744, 6612,
                52160, 12532, 2688,  62768, 54400, 5876,  36212, 57412, 25492,
                19276, 34860, 44228, 21964, 5200,  33544, 64440, 6576,  59436,
                468,   48392, 6932,  34128, 62760, 29624, 6424,  16916, 32516,
                52132, 7596,  35468, 49564, 76,    18116, 27276, 20948, 52924,
                11656, 61744, 22152, 32356, 41016, 46160, 59512, 34232, 3820,
                49820, 35928, 20888, 32460, 41028, 6116,  72,    53792, 9736,
                39748, 29732, 4028,  10768, 40844, 27120, 60568, 42572, 16496,
                36700, 61220, 10584, 26120, 6020,  61636, 28308, 2764,  40668,
                16600, 5592,  20520, 54688, 31916, 42084, 28096, 33580, 39568,
                25760, 7948,
            },
            {
                31708, 37736, 58464, 42412, 51272, 17900, 4448,  57208, 63532,
                6376,  24760, 15116, 35476, 19568, 51084, 9632,  42064, 59080,
                49904, 14152, 39260, 3344,  53348, 28084, 22116, 57008, 49088,
                41552, 8688,  17188, 61556, 4176,  53724, 46788, 58732, 1768,
                28932, 22096, 32296, 2592,  12804, 30152, 16312, 9228,  40332,
                32312, 22752, 44268, 47940, 20632, 39948, 11908, 29188, 64768,
                51988, 7384,  48124, 26652, 56460, 38692, 18472, 25580, 35804,
                13364, 28668, 22952, 9492,  15704, 41412, 59808, 36012, 27100,
                13516, 18972, 39972, 30764, 6016,  55896, 34824, 13976, 47736,
                29812, 1020,  55244, 18348, 12764, 1908,  56328, 25900, 22368,
                61608, 45292, 9052,  57192, 52296, 26808, 44396, 34548, 48112,
                17176, 50532, 35416, 45260, 57608, 6148,  1772,  33376, 53444,
                29060, 19224, 50352, 31460, 41892, 18200, 36268, 53636, 8428,
                31364, 59916, 38704, 14420, 8676,  58484, 4916,  21064, 64184,
                56888, 17024,
            },
            {
                62408, 9528,  24984, 268,   28756, 60984, 33732, 26852, 41440,
                46004, 32544, 49548, 27860, 54392, 31180, 38380, 21312, 3632,
                7764,  44212, 24768, 20504, 46968, 63568, 684,   37136, 25740,
                5228,  59472, 33872, 49608, 43088, 9976,  33192, 12940, 50900,
                6344,  65400, 42060, 49644, 24496, 62184, 20280, 49112, 59148,
                28860, 56228, 38312, 9740,  25820, 58892, 50556, 3612,  14204,
                37916, 30720, 62332, 16624, 42168, 2456,  11004, 50184, 55248,
                41668, 63988, 51028, 38344, 55740, 1100,  11392, 44104, 62516,
                2444,  47568, 59560, 22112, 64540, 45376, 8740,  61004, 41276,
                36840, 25964, 9276,  48828, 39068, 29476, 8008,  43508, 13604,
                53936, 1028,  29372, 47548, 12684, 18844, 58072, 13964, 21644,
                59624, 7532,  63944, 23928, 16036, 52664, 47704, 12660, 65228,
                7440,  45968, 3576,  15500, 56444, 47084, 11536, 49248, 23900,
                56808, 27212, 44872, 23052, 35588, 49680, 46572, 10552, 42608,
                3116,  45036,
            },
            {
                53304, 13748, 34908, 56124, 11888, 7880,  49020, 38424, 9792,
                18360, 2880,  65436, 8608,  44036, 120,   63024, 27124, 35940,
                55128, 65016, 30728, 12140, 42140, 9248,  32632, 52108, 62508,
                19748, 28664, 12568, 56156, 22928, 30016, 62660, 24844, 36236,
                18136, 55728, 15004, 4472,  54416, 38488, 43836, 184,   25084,
                4964,  15400, 60668, 1456,  31396, 16604, 41796, 23352, 46368,
                18220, 1652,  54472, 12444, 28792, 58948, 45244, 21048, 32656,
                17168, 5928,  44560, 19528, 32120, 26328, 53120, 23512, 39176,
                55628, 10064, 28044, 42180, 11872, 24672, 31752, 53968, 5488,
                17672, 65112, 43132, 57584, 23796, 61884, 50948, 37896, 32204,
                18084, 39984, 24540, 64696, 37444, 7792,  61876, 29116, 42716,
                1764,  27272, 13136, 39064, 19956, 30372, 37636, 25296, 20852,
                40392, 54780, 23372, 38000, 59532, 5216,  20668, 64320, 43352,
                13080, 256,   51124, 65368, 3428,  17468, 25140, 54052, 15224,
                35952, 27840,
            },
            {
                50004, 21564, 43864, 65092, 40048, 20440, 54976, 14444, 60288,
                52516, 36548, 12768, 58116, 23540, 17308, 56624, 48496, 11292,
                18616, 51008, 1860,  60308, 35604, 56200, 17500, 13644, 44196,
                3164,  46760, 36288, 1528,  40524, 16776, 5000,  56796, 43600,
                9072,  26296, 34992, 45884, 8072,  28168, 11160, 35424, 64364,
                46092, 19396, 49964, 35580, 54956, 63000, 8396,  33084, 60332,
                56732, 40420, 8820,  35524, 23736, 52512, 30812, 8480,  62260,
                2180,  25184, 58408, 12476, 61676, 46240, 17244, 7952,  49716,
                21332, 33780, 5408,  52756, 16128, 49388, 38824, 3220,  22828,
                50348, 14212, 32788, 2932,  35756, 16136, 20156, 5440,  63792,
                10192, 59244, 15608, 2820,  22092, 32960, 49816, 4876,  38256,
                55520, 32060, 46584, 54468, 2556,  62772, 9244,  44032, 57280,
                300,   34488, 63432, 8664,  25608, 33256, 28900, 3888,  38212,
                34396, 19144, 9832,  40996, 28448, 61864, 38520, 30880, 59932,
                23280, 5920,
            },
            {
                58828, 2428,  30612, 16320, 47268, 32664, 23832, 1340,  30580,
                25996, 21648, 47176, 38968, 6416,  33696, 13920, 44932, 24104,
                40852, 33884, 15968, 26212, 47948, 5420,  23360, 40620, 31096,
                57900, 24352, 51556, 64124, 8332,  45648, 52692, 20348, 38948,
                48944, 30684, 58624, 21340, 63284, 18352, 57420, 51728, 13912,
                32896, 41156, 11424, 27760, 5984,  20128, 43760, 52832, 4500,
                26080, 21688, 48772, 43972, 65180, 6408,  15508, 48148, 40220,
                36624, 52992, 29856, 3468,  49056, 35092, 4356,  65520, 31112,
                14844, 43756, 61324, 36908, 1756,  57288, 19524, 62580, 28856,
                44892, 59248, 7224,  27284, 53256, 10844, 46420, 55544, 27540,
                44368, 35156, 51076, 54992, 41720, 45516, 11280, 24224, 51832,
                19200, 61544, 10164, 42396, 33960, 58980, 14868, 49776, 31936,
                17164, 11880, 51712, 42684, 13600, 45572, 51332, 54668, 15860,
                61492, 46792, 53108, 32344, 12852, 55632, 7852,  888,   47464,
                11496, 39316,
            },
            {
                17808, 46400, 26336, 6476,  53020, 3964,  63456, 41828, 45104,
                5708,  55600, 62232, 29512, 42256, 51448, 59732, 2736,  30136,
                5840,  58168, 8552,  38752, 52972, 29272, 59672, 50308, 10692,
                7100,  39292, 14316, 18840, 27192, 34760, 59092, 356,   13824,
                62952, 2980,  11432, 40412, 50100, 31856, 3104,  42440, 21776,
                7572,  61432, 52976, 23140, 46768, 38604, 13584, 28556, 36480,
                11060, 63084, 32100, 19088, 124,   38568, 60104, 26968, 57292,
                13692, 22036, 42996, 8896,  38696, 20528, 28512, 57460, 37808,
                600,   56408, 18040, 25916, 30156, 46708, 13544, 34464, 10652,
                55076, 37724, 21208, 48036, 63276, 31032, 41372, 1656,  22196,
                48880, 4120,  25796, 30708, 5968,  57044, 17380, 64288, 35064,
                14764, 3640,  25840, 50708, 6852,  28520, 22732, 4976,  62264,
                26368, 47844, 29264, 19784, 60812, 1068,  40296, 22992, 7632,
                30052, 24692, 57932, 5468,  21372, 44304, 34864, 51344, 19960,
                64504, 32380,
            },
            {
                8804,  35364, 61524, 11112, 37504, 28152, 48440, 16744, 57844,
                33252, 10236, 15496, 1588,  18828, 26588, 9240,  37612, 62076,
                54288, 46612, 22580, 63256, 12760, 19640, 3808,  36500, 65316,
                21356, 55416, 32568, 60608, 48456, 11248, 23552, 31832, 27984,
                52116, 17656, 44480, 24208, 5752,  15736, 37116, 25536, 55032,
                29292, 2360,  36916, 15948, 65296, 58104, 640,   49744, 17152,
                45660, 5704,  53796, 13384, 50848, 34256, 23020, 3688,  10580,
                46692, 33296, 64508, 55020, 14496, 59768, 42580, 12156, 25112,
                46060, 50844, 11196, 63252, 40804, 8148,  59820, 42288, 25400,
                408,   16440, 40780, 12344, 3920,  23388, 60576, 14896, 32764,
                58136, 19260, 9600,  62868, 14264, 38812, 26936, 828,   47784,
                40812, 58172, 36860, 21136, 17356, 47128, 56136, 41904, 36036,
                7876,  40016, 3012,  55136, 35600, 17432, 63596, 11156, 56356,
                42520, 1732,  37056, 16896, 48624, 60896, 24316, 15856, 28652,
                43080, 54436,
            },
            {
                24576, 51472, 41664, 55808, 19248, 59020, 7804,  12336, 36964,
                20140, 49712, 43408, 35348, 53640, 64000, 22324, 49656, 14656,
                19408, 28196, 42560, 412,   33264, 41688, 45684, 26032, 16248,
                47992, 28412, 2768,  42472, 5680,  37988, 63616, 41932, 46736,
                7488,  36936, 61572, 34068, 54136, 59604, 46784, 63804, 10232,
                48320, 57120, 44180, 4544,  31164, 10120, 24696, 54816, 60780,
                30120, 39408, 58268, 27848, 42316, 16816, 55092, 44360, 19820,
                52096, 788,   17992, 27472, 23984, 51240, 6196,  53608, 19336,
                8388,  32624, 23092, 3572,  35352, 20544, 53196, 6004,  49036,
                63908, 29840, 51500, 58496, 44456, 37144, 8088,  50472, 39732,
                12508, 43060, 36728, 46808, 20776, 52780, 59304, 31760, 8472,
                22336, 29948, 12380, 53752, 64748, 39236, 1368,  13096, 53292,
                60604, 21940, 58832, 31432, 6340,  49300, 27060, 32668, 19896,
                50320, 13932, 63940, 27564, 39300, 3140,  10344, 40872, 57480,
                3664,  13872,
            },
            {
                63260, 1520,  21456, 14156, 44572, 34168, 23184, 61092, 27180,
                54280, 4564,  24412, 59048, 11856, 40172, 31276, 5240,  43540,
                36336, 9932,  51196, 17360, 56020, 7904,  60852, 53588, 1412,
                34896, 44472, 12160, 52264, 20932, 54844, 15692, 3716,  57912,
                22424, 55548, 27144, 1176,  13084, 28544, 6860,  18820, 39600,
                14768, 34144, 20296, 26552, 48816, 40764, 33784, 21088, 42192,
                14780, 2520,  24132, 9452,  60908, 6868,  31776, 63840, 28492,
                36128, 59056, 7456,  41184, 46388, 2104,  34024, 30184, 63800,
                39492, 58588, 42816, 55016, 49792, 28072, 16740, 32528, 22048,
                36436, 8936,  19432, 33652, 25992, 16236, 56708, 28584, 64056,
                6104,  53276, 59928, 28160, 2236,  34324, 10764, 44692, 54880,
                63176, 42872, 5120,  45148, 9176,  34876, 24160, 29548, 18632,
                44328, 10720, 15184, 24264, 43376, 9560,  37584, 47200, 59824,
                5768,  34580, 45808, 8412,  59144, 52088, 33044, 62748, 6268,
                37368, 48472,
            },
            {
                27512, 39416, 5756,  29672, 49576, 3268,  40896, 51356, 472,
                39516, 64604, 28984, 48056, 7084,  45736, 16592, 55396, 25600,
                2488,  64164, 31836, 58820, 37496, 21748, 14516, 31348, 9704,
                62988, 18032, 58932, 25020, 30784, 8844,  50012, 19464, 33684,
                10160, 14876, 47764, 41772, 20664, 52024, 44000, 31284, 584,
                24256, 59744, 11976, 62036, 52572, 17856, 3284,  63528, 7984,
                35244, 51192, 19648, 46832, 36736, 49788, 12644, 40508, 4632,
                49216, 15536, 30792, 62568, 36920, 10380, 60484, 16680, 48492,
                2608,  27684, 15616, 6692,  12472, 65024, 44076, 2260,  56048,
                46360, 61096, 3324,  54140, 5416,  47564, 20120, 35100, 516,
                24288, 31492, 15932, 7720,  49620, 41564, 24512, 18904, 3948,
                16176, 49992, 26280, 60640, 31316, 15564, 52060, 63064, 3420,
                33048, 50580, 38360, 53016, 62752, 57380, 14392, 3484,  39796,
                25696, 54216, 22180, 31412, 19328, 13152, 26004, 43900, 22296,
                31236, 18212,
            },
            {
                45752, 59548, 33244, 56752, 64784, 10128, 17456, 31088, 46672,
                14508, 8912,  17756, 32940, 21328, 1944,  57936, 34308, 60912,
                47396, 13648, 23160, 5584,  48536, 28772, 50808, 39644, 23068,
                56656, 37296, 4708,  40788, 62268, 36124, 26896, 45140, 40356,
                60632, 30824, 4912,  64024, 38028, 8880,  35032, 62488, 50300,
                54248, 41900, 8128,  38556, 6144,  56756, 44696, 12520, 48132,
                26008, 56004, 64664, 30492, 1300,  21380, 25812, 57428, 9784,
                23776, 43348, 56396, 12692, 19612, 50144, 22852, 40848, 13436,
                36060, 21732, 61452, 46860, 37756, 24588, 57648, 40088, 15080,
                11336, 27380, 43016, 14168, 65476, 40140, 10348, 52332, 45216,
                18340, 39016, 56620, 22460, 64972, 13628, 60816, 51428, 28592,
                37568, 33808, 1092,  19740, 56716, 5948,  42712, 47408, 57684,
                26828, 5280,  46656, 28140, 1404,  19484, 23244, 30408, 64920,
                17460, 11456, 41572, 56836, 2216,  46688, 54940, 16652, 50156,
                56024, 10984,
            },
            {
                7688,  52008, 16220, 23912, 37300, 26576, 53712, 62484, 21672,
                34964, 55908, 41788, 51720, 61952, 38624, 28580, 12220, 20512,
                52384, 39156, 27164, 44476, 11448, 3028,  62204, 42700, 6124,
                27232, 49356, 14660, 46844, 712,   56420, 12740, 64996, 2060,
                25264, 50584, 17864, 54504, 23996, 59120, 15852, 22264, 6396,
                27500, 46576, 16940, 29980, 23436, 36292, 27460, 31932, 59424,
                5192,  38688, 11008, 15924, 45048, 62536, 52568, 18080, 38084,
                60748, 2516,  34208, 26068, 4844,  54904, 28928, 44636, 56080,
                5212,  52356, 31048, 18936, 33656, 9684,  5072,  30412, 52148,
                24068, 38448, 31956, 49956, 22668, 30400, 59320, 26832, 13260,
                61732, 48292, 3196,  44512, 35696, 29716, 6288,  39296, 56828,
                9392,  59724, 13064, 40592, 48316, 27344, 11604, 20780, 36256,
                8808,  65484, 17952, 12420, 33520, 41960, 50504, 55404, 9056,
                44576, 51212, 4552,  63192, 28460, 38720, 35596, 8916,  1180,
                60992, 34932,
            },
            {
                20324, 42476, 12372, 948,   46164, 7356,  42052, 11900, 5188,
                25564, 60108, 3016,  13168, 23568, 9100,  50292, 41620, 4264,
                8372,  33188, 60436, 16488, 54836, 34688, 19264, 13032, 53116,
                32080, 20204, 8324,  33592, 22664, 17436, 29912, 6668,  53880,
                37204, 44092, 11720, 33012, 2368,  46124, 12180, 55848, 37404,
                3440,  32724, 64648, 51268, 972,   61164, 15348, 53372, 22496,
                18140, 43404, 33280, 54084, 4068,  41388, 34492, 6240,  29976,
                45864, 20856, 53112, 48004, 38276, 65044, 180,   8892,  25648,
                62756, 11228, 43636, 980,   53644, 60936, 48756, 19868, 62688,
                7128,  59652, 804,   56504, 36664, 7860,  2352,  41820, 54776,
                33064, 8548,  26332, 11680, 52620, 17876, 308,   47476, 21220,
                25200, 45544, 52920, 23064, 35492, 61844, 2472,  39496, 16172,
                23524, 41468, 56416, 37044, 61408, 6572,  45880, 35684, 1888,
                27832, 37384, 33320, 14476, 49224, 6448,  21296, 65276, 29308,
                40520, 25076,
            },
            {
                54260, 63788, 29104, 48660, 60848, 19732, 33092, 57776, 38544,
                49532, 44960, 30768, 37060, 54452, 46416, 17616, 65108, 24832,
                45020, 53912, 920,   41340, 64580, 25568, 46308, 59600, 1064,
                44972, 64780, 54008, 60296, 43412, 51928, 38764, 48404, 15420,
                21452, 8300,  57692, 40248, 27328, 49520, 30664, 42824, 18560,
                57528, 13796, 21728, 43816, 11376, 48668, 40636, 9008,  46244,
                1744,  28696, 60348, 23508, 8532,  27496, 47808, 13928, 54824,
                64200, 11216, 16524, 7600,  42920, 31656, 17924, 59388, 33752,
                37976, 49664, 24168, 58864, 39340, 14308, 27260, 35824, 44420,
                16612, 48016, 20940, 11636, 17860, 62400, 46996, 16580, 4724,
                36864, 20340, 63240, 40020, 55724, 32524, 42848, 63076, 14760,
                31612, 6720,  65104, 16024, 7964,  54320, 32340, 49740, 60492,
                54812, 30840, 4428,  21744, 52276, 10628, 25216, 15584, 58212,
                62212, 22776, 18664, 60704, 24764, 42988, 58456, 12216, 47800,
                15472, 3376,
            },
            {
                37524, 5576,  22480, 35380, 55264, 14464, 44124, 1800,  28632,
                15656, 18952, 6232,  63148, 26984, 232,   34948, 30348, 58488,
                14344, 18784, 29300, 21900, 9624,  6680,  30144, 36544, 17032,
                38940, 24616, 11692, 28024, 3084,  58360, 10392, 24476, 60208,
                35012, 29460, 62588, 19692, 5452,  65460, 7808,  25252, 61604,
                47980, 9436,  39536, 54120, 34292, 19992, 4336,  58028, 37324,
                63872, 49484, 12864, 39720, 56824, 17408, 59488, 22816, 540,
                32904, 40716, 28672, 58472, 22612, 14180, 51624, 47092, 20688,
                15732, 7040,  29000, 17160, 4276,  22756, 42020, 1712,  10416,
                33088, 53448, 40560, 29020, 43856, 25360, 51912, 30996, 23620,
                57284, 49572, 29016, 15728, 6660,  24152, 58656, 10824, 36068,
                51220, 42272, 2160,  38280, 29816, 18496, 45320, 25328, 13452,
                156,   44600, 48084, 14960, 29168, 59316, 32532, 20708, 39660,
                12080, 48028, 7776,  52844, 360,   30940, 17288, 51260, 33552,
                44224, 57808,
            },
            {
                31028, 50448, 16844, 40460, 3984,  30456, 51608, 22276, 65356,
                53864, 9688,  58392, 20952, 43456, 15544, 55696, 6620,  38316,
                49180, 35796, 57076, 50760, 38532, 47832, 57788, 22900, 10384,
                51112, 5636,  41752, 15864, 36296, 19812, 31872, 46012, 5112,
                55004, 16,    51232, 45324, 16404, 35700, 52668, 40500, 1272,
                34016, 28596, 60084, 6840,  26704, 63180, 30968, 24808, 16384,
                34612, 7136,  20576, 51640, 30776, 36712, 10548, 50736, 44256,
                25204, 4052,  49456, 61984, 35792, 10704, 39588, 27576, 2976,
                53760, 40884, 64652, 48164, 32440, 55736, 51348, 64144, 57472,
                24748, 3720,  63160, 5976,  34640, 55732, 13688, 38472, 61016,
                11000, 42620, 1388,  59720, 46396, 20596, 49376, 27852, 4504,
                18816, 57116, 26472, 49200, 59460, 10936, 64064, 6444,  52796,
                35004, 10188, 26304, 63668, 38428, 1164,  47288, 53432, 6188,
                30592, 56240, 40712, 45368, 10672, 55684, 39384, 23356, 6820,
                26964, 11508,
            },
            {
                59432, 45192, 8452,  62364, 11064, 25388, 59012, 7972,  46928,
                35980, 24392, 41052, 34020, 11640, 50876, 61168, 21432, 10492,
                3976,  63648, 12168, 3100,  32644, 14940, 53508, 2484,  63424,
                33868, 56368, 30520, 62064, 50576, 7832,  40284, 63760, 17996,
                41816, 25804, 13804, 38524, 23120, 55528, 11092, 20936, 14932,
                51868, 23492, 3748,  17960, 56712, 13968, 42828, 52348, 10816,
                55496, 26988, 44380, 1044,  65376, 5736,  42044, 61856, 15668,
                37928, 56944, 18872, 6064,  45252, 1612,  56252, 63204, 43436,
                12960, 57228, 34952, 10480, 44260, 7528,  13052, 18152, 30688,
                45788, 37396, 14744, 58044, 47472, 9368,  3988,  21076, 45724,
                6212,  18672, 35232, 52028, 9144,  38040, 2232,  54292, 40828,
                61912, 12368, 34156, 20660, 43476, 3064,  40968, 28528, 21044,
                38960, 58224, 18692, 50124, 8064,  42404, 17824, 64308, 43888,
                2544,  25852, 15992, 33904, 27624, 36680, 60072, 3128,  63096,
                53284, 19136,
            },
            {
                1324,  24388, 28372, 56940, 49460, 34356, 18020, 39780, 12996,
                31272, 2652,  52644, 47996, 3700,  31928, 39844, 25880, 47156,
                42420, 23096, 27416, 44428, 62584, 19980, 41536, 28296, 44024,
                19448, 47524, 3912,  21804, 45288, 26076, 1784,  52904, 11512,
                33560, 48228, 9108,  59516, 32596, 2136,  60584, 30296, 46028,
                63912, 42128, 36184, 50288, 45784, 37964, 1952,  48216, 21264,
                40744, 61256, 15020, 33840, 47376, 25720, 20248, 29416, 7912,
                52472, 12200, 34036, 26200, 53852, 30648, 23772, 8380,  32200,
                21908, 4488,  26344, 18900, 61536, 37520, 28076, 40344, 21472,
                8168,  50944, 27456, 22644, 18824, 63860, 41268, 33952, 53808,
                26900, 65196, 30788, 13500, 25220, 33208, 64336, 16688, 30252,
                23000, 46620, 8212,  55700, 14332, 23836, 57188, 33072, 15908,
                62968, 42668, 3264,  33672, 24240, 56868, 13348, 28036, 35368,
                50768, 19476, 58008, 64772, 4752,  14284, 20016, 46704, 9592,
                40836, 35864,
            },
            {
                64944, 13444, 42772, 20132, 37924, 612,   63060, 43344, 5720,
                55680, 62140, 14492, 28184, 64520, 8336,  17796, 53308, 29792,
                60536, 34224, 17340, 55824, 5796,  24932, 11032, 60644, 7740,
                14148, 26504, 37312, 12680, 54624, 59184, 14680, 29284, 22800,
                61396, 57112, 28676, 4720,  43008, 25596, 48940, 37776, 4800,
                10036, 57984, 30668, 12112, 22560, 8200,  28164, 62540, 6292,
                31640, 3488,  58352, 18444, 11576, 54888, 3256,  34852, 58620,
                46444, 22364, 64428, 42096, 15268, 60192, 18512, 50228, 37068,
                60732, 51860, 46276, 624,   23332, 58572, 50056, 2432,  55308,
                61104, 12116, 42356, 1948,  32208, 50464, 28204, 188,   48576,
                16080, 39832, 55120, 44836, 60596, 41872, 11408, 45812, 5388,
                36904, 60120, 1496,  31508, 52032, 37316, 47968, 9028,  50924,
                5472,  29640, 55040, 11440, 61192, 36636, 4904,  22628, 9768,
                60884, 38744, 7124,  23032, 49724, 53824, 43184, 26340, 32092,
                15836, 48180,
            },
            {
                4256,  33888, 53668, 5652,  47264, 14720, 27924, 51016, 21408,
                26168, 44568, 18620, 37652, 22944, 44924, 56748, 5056,  14808,
                1200,  52012, 8440,  39960, 48268, 36368, 52120, 31840, 39044,
                49524, 57424, 62860, 9272,  32608, 42564, 35344, 50092, 38988,
                2628,  20196, 15896, 46984, 63476, 12808, 17628, 54708, 27480,
                20056, 16140, 320,   53928, 64848, 33268, 58848, 17828, 35920,
                45948, 53240, 23220, 37800, 42660, 62696, 49128, 39992, 17136,
                1776,  31548, 9024,  48280, 3396,  38776, 11768, 46632, 5532,
                16456, 29492, 39480, 53900, 31620, 15840, 5248,  46944, 32300,
                17052, 35544, 53744, 59336, 38840, 15640, 57276, 12880, 62024,
                23868, 8576,  4684,  21992, 2920,  19172, 56864, 25968, 53012,
                14904, 50160, 40000, 27932, 63052, 4204,  18780, 61412, 44152,
                23424, 14280, 46836, 20024, 40800, 51600, 30876, 55220, 47752,
                14504, 31800, 46272, 12436, 41068, 30008, 1620,  61180, 56780,
                22420, 52156,
            },
            {
                39124, 18164, 60068, 31268, 23232, 56352, 9320,  33148, 59280,
                39076, 10244, 820,   57540, 49960, 12500, 35664, 63480, 20476,
                38016, 45388, 13288, 58240, 29988, 760,   16152, 55304, 23152,
                2988,  17840, 40900, 1084,  24220, 17008, 4652,  65072, 8432,
                44548, 35960, 52628, 39772, 21620, 57768, 34768, 6980,  44940,
                62156, 40288, 48704, 26236, 43580, 15712, 39704, 55088, 24320,
                13788, 9588,  50232, 27948, 7296,  31792, 14020, 24188, 61448,
                28004, 55864, 36832, 20292, 52244, 27200, 34608, 65524, 25168,
                42484, 13392, 8284,  63628, 11668, 36024, 25584, 65160, 43388,
                6436,  24128, 46036, 4996,  10464, 24700, 44680, 6796,  31192,
                37436, 58200, 47444, 29516, 35624, 49272, 7272,  31696, 63428,
                20304, 9504,  24548, 17348, 45412, 12224, 35516, 26740, 808,
                37728, 60248, 32588, 26516, 2248,  15752, 43524, 62924, 476,
                25424, 54224, 3540,  36072, 62480, 8988,  17592, 34976, 11980,
                7164,  29456,
            },
            {
                25224, 8696,  40992, 11744, 64304, 43924, 37052, 2956,  16196,
                47528, 29916, 60696, 33452, 6628,  26916, 42740, 31332, 48068,
                59132, 26096, 32580, 22444, 18896, 60060, 45916, 4972,  65416,
                34980, 30644, 46344, 53000, 60868, 49072, 21196, 56288, 27092,
                13324, 55344, 6804,  30204, 10268, 3060,  41736, 51784, 31536,
                23972, 56284, 33832, 6132,  21024, 3404,  10484, 49164, 904,
                43152, 29872, 64376, 4260,  57524, 21324, 52788, 10152, 45088,
                4892,  41312, 13724, 59192, 7056,  44008, 56980, 1472,  54348,
                33308, 59448, 21176, 43628, 48076, 56844, 19604, 38640, 9880,
                28760, 62216, 20136, 29952, 64704, 35296, 54892, 21468, 42972,
                50984, 11784, 17620, 61532, 52464, 14644, 39032, 43952, 544,
                34780, 42016, 58512, 55256, 6556,  22164, 54168, 31172, 57784,
                10876, 53032, 7232,  49604, 65032, 9196,  21500, 37556, 18464,
                40640, 59708, 28700, 20172, 56180, 25072, 47040, 50940, 37908,
                45056, 58664,
            },
            {
                62804, 46116, 50468, 1976,  17352, 25948, 52056, 61892, 20284,
                4868,  52396, 22160, 41408, 15844, 55080, 1984,  23944, 7412,
                10904, 52960, 3528,  64432, 49704, 11716, 26960, 42136, 20356,
                51496, 6876,  27364, 14004, 36640, 30068, 10832, 46448, 32116,
                18232, 62824, 24504, 34012, 50336, 26892, 64960, 14400, 9080,
                2460,  38184, 11168, 59396, 47428, 53096, 31152, 26948, 57884,
                61368, 20332, 40092, 16508, 46480, 35428, 832,   38076, 54480,
                18732, 50408, 23624, 63320, 29900, 16068, 22448, 10460, 19608,
                48732, 4124,  37680, 26928, 2280,  7308,  52800, 13568, 58436,
                48412, 564,   39452, 52460, 17512, 49076, 2336,  60020, 18884,
                34064, 1296,  25736, 41284, 9280,  23684, 59324, 28108, 12296,
                61348, 47616, 3924,  32864, 38584, 65320, 42112, 15204, 47468,
                20756, 39744, 17388, 35740, 44984, 29172, 57920, 6304,  33396,
                45092, 10556, 51648, 15572, 44072, 33016, 5312,  64048, 20336,
                48,    15024,
            },
            {
                54488, 20760, 28684, 34884, 55164, 7320,  30752, 13028, 40876,
                55540, 35620, 11780, 46220, 62880, 19216, 51044, 39204, 62368,
                18316, 35204, 40796, 6264,  43292, 34124, 7856,  37852, 13596,
                58428, 10060, 63392, 19804, 43232, 6428,  59596, 39852, 936,
                42240, 48376, 3804,  60216, 44344, 18596, 36988, 22056, 59952,
                49408, 19072, 14828, 28620, 36432, 61896, 41884, 18720, 37240,
                7744,  33700, 55852, 12100, 26560, 60752, 43332, 25576, 65036,
                33068, 11208, 35616, 296,   46100, 39716, 32048, 62496, 40892,
                28608, 52360, 17216, 62000, 34428, 30680, 22220, 41952, 33912,
                16244, 56184, 12424, 33312, 7588,  26364, 41004, 10116, 28312,
                63688, 56192, 45980, 32220, 64836, 2348,  54560, 17372, 51108,
                22488, 29652, 16240, 11384, 48924, 28916, 8484,  51040, 4896,
                25700, 62628, 3352,  58976, 12188, 53956, 24868, 49316, 13108,
                62292, 23420, 7484,  38928, 2044,  58540, 13416, 23676, 53380,
                42108, 32880,
            },
            {
                36828, 12656, 5448,  59800, 38908, 21876, 45116, 49340, 24652,
                8520,  64724, 26272, 4132,  31880, 9324,  28980, 57164, 14604,
                44720, 27616, 54860, 15388, 24572, 56908, 61956, 29160, 48524,
                23860, 44632, 39180, 55232, 3400,  25420, 53532, 16004, 23644,
                58308, 36600, 11284, 15248, 53916, 632,   56632, 46204, 27868,
                40912, 54424, 64080, 45604, 7576,  23056, 12716, 4524,  52040,
                15088, 47796, 2128,  30860, 49804, 19152, 8768,  14968, 30056,
                6472,  57880, 48476, 25980, 9536,  53708, 4924,  47404, 14356,
                7496,  58476, 23608, 12636, 50220, 45392, 59996, 3052,  26500,
                50652, 23148, 43652, 60660, 47140, 14716, 37216, 53216, 4492,
                15724, 38444, 6364,  13208, 20000, 37700, 34004, 4880,  45452,
                7976,  37380, 52416, 25668, 59556, 1304,  18336, 61020, 33556,
                44824, 28752, 42600, 22932, 31948, 1636,  40940, 16460, 52676,
                2560,  31292, 47272, 65192, 27544, 49944, 40396, 31156, 10444,
                27316, 4200,
            },
            {
                57960, 24036, 48780, 43048, 14752, 63436, 432,   34300, 57816,
                28992, 16972, 38672, 48832, 59804, 36756, 43840, 3300,  21944,
                49464, 168,   60564, 31488, 20644, 51628, 1916,  17492, 55692,
                32260, 400,   16284, 33736, 48156, 12404, 64196, 34636, 8840,
                52228, 20516, 29440, 40272, 23208, 32252, 6724,  12076, 34560,
                4516,  24584, 31732, 1088,  16976, 56360, 34288, 65280, 44376,
                25504, 22064, 42372, 63732, 38452, 3316,  51996, 58672, 47124,
                21848, 16768, 42568, 56308, 19164, 61008, 37168, 24656, 56508,
                35572, 44700, 444,   39708, 55288, 8976,  18028, 62820, 5688,
                36732, 8604,  28032, 2796,  57084, 21832, 62876, 30388, 44976,
                23904, 49392, 54132, 27676, 43596, 48128, 57700, 25520, 40784,
                64332, 55868, 2656,  44084, 21124, 41128, 36564, 23864, 13220,
                54440, 8948,  14704, 55996, 19400, 47012, 35052, 59480, 26624,
                37364, 55216, 17964, 34680, 21772, 8256,  16796, 55676, 61820,
                47232, 18384,
            },
            {
                39512, 65408, 9172,  32180, 27032, 52608, 11124, 19132, 5360,
                43396, 50968, 1232,  20616, 13204, 24260, 54180, 33852, 11740,
                58176, 37564, 8036,  47660, 10572, 36148, 45024, 12452, 40468,
                5384,  60088, 26792, 62180, 21616, 30548, 38320, 49848, 27696,
                4484,  44792, 64572, 7824,  49512, 62608, 42552, 16936, 52688,
                60892, 8636,  43776, 51704, 39580, 26720, 49832, 9876,  29504,
                59648, 53124, 6432,  10980, 55104, 23136, 33360, 40920, 4184,
                36576, 61812, 2676,  29044, 33908, 12428, 50668, 2492,  18260,
                31240, 11084, 64940, 29344, 15384, 24720, 38564, 29896, 53600,
                45644, 64516, 18196, 31876, 42148, 4940,  50624, 12292, 34340,
                58016, 8784,  21736, 60380, 4344,  15624, 10156, 20992, 32128,
                12732, 19180, 35312, 62080, 14040, 53188, 57140, 46248, 3744,
                64036, 35440, 51284, 38492, 7112,  64248, 10732, 4332,  20092,
                42424, 5536,  57668, 11912, 43716, 61040, 37212, 3520,  43216,
                7316,  51504,
            },
            {
                29244, 2288,  55592, 17596, 4340,  58724, 36784, 46552, 61444,
                13936, 32832, 58396, 53364, 45444, 5980,  17072, 65304, 41808,
                25008, 29760, 18856, 63996, 39928, 3684,  26512, 63332, 21980,
                50744, 36812, 10988, 51940, 7476,  43860, 2208,  18492, 61080,
                13112, 31704, 56312, 17788, 26632, 38072, 58108, 30048, 47356,
                20124, 13948, 35692, 21556, 59000, 3872,  46300, 20164, 128,
                38844, 13284, 34532, 27716, 16712, 44724, 62792, 12744, 26288,
                54272, 39584, 14248, 52344, 6912,  22120, 44168, 27736, 62948,
                41604, 51484, 19976, 47680, 5184,  43428, 56604, 11584, 20844,
                14352, 39816, 51456, 11404, 36248, 25620, 19320, 59588, 728,
                17708, 41948, 31136, 35648, 51024, 62512, 29596, 53676, 59836,
                6164,  46728, 28376, 9112,  33024, 5616,  26600, 10440, 39448,
                20316, 30820, 796,   60860, 24468, 49720, 28432, 44616, 62708,
                30216, 14872, 48896, 25344, 1468,  52872, 28848, 20072, 24956,
                34484, 14944,
            },
            {
                21812, 45424, 35956, 47816, 20912, 40380, 30168, 24100, 54604,
                41264, 22124, 7988,  29608, 40292, 60832, 28132, 8864,  51412,
                4788,  45988, 13060, 52700, 23408, 58992, 32992, 54080, 8176,
                29748, 46500, 19044, 41144, 56848, 15220, 59044, 46628, 22892,
                54252, 37268, 5492,  47520, 13840, 3556,  9968,  25032, 1500,
                39876, 55616, 62424, 10736, 30404, 15272, 61356, 41160, 32604,
                17404, 56144, 61652, 48692, 37140, 5064,  29400, 19516, 50592,
                10248, 31580, 24012, 45844, 64876, 40152, 57840, 15528, 8700,
                54700, 3856,  25704, 33104, 60828, 35512, 2684,  48200, 32564,
                57848, 900,   22564, 61292, 54316, 7244,  43984, 38892, 27824,
                47728, 63864, 2708,  11948, 18932, 39100, 248,   42980, 48972,
                15128, 39628, 23664, 57800, 47984, 17312, 30076, 50520, 59344,
                15820, 48408, 27320, 43124, 17536, 33368, 14028, 38772, 54496,
                22648, 9636,  60172, 39096, 32584, 46260, 13980, 58324, 49380,
                63228, 11276,
            },
            {
                41784, 61176, 26092, 13156, 62452, 50152, 7332,  15884, 2220,
                27516, 10972, 62152, 34812, 3436,  19324, 47544, 38348, 32152,
                16380, 61568, 34584, 42936, 5516,  15160, 48448, 18488, 42384,
                14252, 57592, 2496,  24204, 35556, 28648, 10608, 40340, 33704,
                12,    43068, 21352, 61552, 34704, 53592, 21764, 50740, 65060,
                33456, 27336, 2636,  48680, 37548, 24424, 55208, 5404,  51572,
                23432, 45456, 2504,  20536, 8964,  59772, 56720, 536,   43548,
                63892, 7696,  58592, 17700, 36008, 1692,  30428, 47840, 34644,
                22736, 38352, 58960, 7768,  50916, 19108, 64108, 27000, 41696,
                7032,  35200, 46364, 29156, 15868, 48764, 65308, 9988,  54864,
                14376, 37552, 26084, 56676, 46080, 23440, 8724,  35128, 26668,
                4232,  62812, 51660, 880,   40456, 64864, 35776, 2344,  43744,
                22808, 7708,  58424, 12136, 4812,  56652, 52044, 7956,  332,
                45996, 35388, 51812, 19384, 6512,  63928, 9808,  36324, 680,
                31692, 53224,
            },
            {
                16848, 6652,  33076, 1216,  42940, 10256, 56816, 34108, 65028,
                48080, 37760, 44704, 15008, 50480, 26252, 56224, 1112,  59644,
                22604, 54612, 2140,  27572, 56924, 30356, 38232, 6440,  60624,
                25392, 33576, 65200, 6032,  47772, 54692, 3724,  25888, 64044,
                16676, 51492, 10884, 28724, 40820, 59680, 31552, 44020, 15888,
                5700,  45856, 18724, 53184, 6964,  43496, 12240, 35320, 63468,
                7780,  30772, 42116, 25756, 51288, 32320, 46720, 15168, 35076,
                21484, 48592, 3624,  26856, 50252, 11324, 20508, 5060,  60376,
                12264, 45552, 16276, 41212, 13272, 23464, 9892,  52648, 16328,
                61836, 24968, 9692,  58580, 4192,  33112, 20916, 30588, 3596,
                23008, 51632, 7336,  42444, 32264, 53148, 65144, 17588, 58688,
                21820, 36804, 30960, 12276, 20468, 55144, 24448, 13532, 62460,
                32028, 54032, 39900, 46544, 36376, 22216, 61888, 18676, 26184,
                31588, 64756, 4252,  28244, 41532, 16536, 54988, 23024, 39880,
                5412,  57028,
            },
            {
                10100, 38416, 51668, 54944, 23272, 28380, 39072, 19600, 51508,
                5936,  17648, 53756, 23188, 63904, 11372, 35576, 14132, 44324,
                10020, 39380, 19620, 47312, 11572, 21532, 64408, 688,   45780,
                11488, 39736, 53036, 16628, 31416, 61404, 19592, 49492, 7868,
                30608, 57476, 25292, 45540, 2480,  18868, 7268,  38456, 12056,
                60344, 23380, 41500, 32468, 64352, 28640, 17080, 49064, 26864,
                11052, 58196, 14620, 64976, 39664, 11812, 24180, 41316, 28060,
                55484, 38248, 13956, 32708, 56412, 62636, 42764, 53740, 26156,
                49924, 1136,  28360, 55112, 31440, 46596, 37836, 1516,  30436,
                49192, 53820, 19084, 43228, 38168, 12924, 41116, 57664, 45188,
                34784, 62124, 16364, 59544, 4692,  14084, 28336, 50264, 11360,
                45052, 54804, 16788, 43220, 6608,  45576, 9444,  52652, 37160,
                5236,  18908, 25108, 65288, 2036,  29224, 41424, 47512, 58296,
                15928, 43020, 11816, 56600, 24340, 48364, 30596, 44316, 18668,
                46828, 25416,
            },
            {
                49100, 19920, 60144, 30468, 14836, 63572, 3124,  43848, 24904,
                30944, 57168, 288,   32508, 6836,  42924, 21056, 52284, 30196,
                25716, 50036, 62856, 7180,  41064, 50444, 34280, 52532, 27488,
                19968, 49736, 9116,  22332, 44860, 12044, 42216, 36552, 59840,
                13560, 39488, 6176,  63612, 15084, 55296, 47740, 28148, 52852,
                35628, 56592, 9064,  14336, 57408, 21268, 4676,  59576, 39964,
                19628, 47880, 35812, 5540,  17896, 54548, 7168,  62692, 5344,
                18312, 60260, 44992, 40964, 8748,  24828, 17048, 37932, 31196,
                64512, 18976, 36752, 62248, 3588,  57468, 43592, 59936, 21516,
                11952, 37192, 2916,  27112, 63224, 50176, 24500, 6540,  18124,
                49400, 10808, 29572, 21256, 48328, 37244, 2440,  41308, 33352,
                8024,  3216,  27636, 61268, 34328, 59208, 29356, 17812, 41436,
                49436, 10828, 34240, 15372, 50920, 9460,  32772, 12480, 37008,
                3172,  50488, 21132, 37376, 59924, 2228,  7656,  62008, 12176,
                59136, 29528,
            },
            {
                64888, 3544,  44460, 8288,  18180, 47016, 35764, 12828, 59684,
                9192,  41684, 26700, 60092, 48716, 39152, 58792, 6012,  64616,
                3772,  36428, 14820, 32396, 58344, 24780, 8884,  15768, 36760,
                59668, 29888, 1416,  62816, 38288, 27368, 5100,  52900, 23592,
                47068, 34120, 20720, 49828, 36744, 10320, 24244, 63132, 800,
                17804, 49268, 26408, 3392,  46896, 38408, 52080, 44264, 3004,
                33116, 53700, 940,   57156, 30224, 22040, 49000, 33324, 53044,
                1836,  11600, 29520, 21952, 52200, 36,    46652, 13744, 6412,
                41576, 9376,  52248, 21780, 7008,  26276, 17684, 34316, 5048,
                63900, 45164, 31820, 55872, 17196, 40,    53516, 60004, 26976,
                38572, 1408,  55656, 40352, 25448, 61228, 54128, 19792, 57548,
                24268, 64236, 48596, 38700, 22208, 3860,  47892, 26424, 212,
                60888, 55828, 28352, 57904, 43872, 19988, 55160, 5644,  61264,
                24712, 54792, 8376,  32416, 13816, 52476, 34604, 27040, 51176,
                2604,  33968,
            },
            {
                22504, 13228, 40676, 34736, 50888, 58432, 5648,  53568, 21712,
                47640, 14512, 37080, 19156, 9872,  28476, 16292, 33336, 46756,
                18392, 55776, 28388, 45432, 1780,  18208, 61972, 43768, 56084,
                5336,  42496, 34176, 54412, 14608, 58252, 33156, 1820,  18364,
                9720,  56164, 1132,  30212, 42708, 57892, 33480, 20960, 43364,
                6768,  30912, 40176, 60796, 34472, 10324, 30324, 24248, 15588,
                62416, 22908, 27592, 43732, 38100, 61308, 9996,  43180, 36892,
                25276, 47320, 64100, 6280,  36328, 61704, 33516, 59692, 22256,
                56884, 44772, 33792, 11828, 48728, 39860, 14048, 54660, 28692,
                40604, 15200, 7160,  22540, 46604, 35904, 9344,  32216, 14036,
                64632, 43688, 33272, 8092,  17944, 12320, 6764,  29796, 47052,
                37720, 12784, 51968, 8816,  15164, 57220, 33236, 64436, 13984,
                44596, 20672, 3136,  7416,  37388, 62984, 27148, 48952, 16952,
                39932, 29784, 46676, 63608, 43712, 17784, 39184, 20972, 42048,
                15208, 37412,
            },
            {
                53520, 27880, 56840, 24364, 556,   31756, 26844, 39940, 61720,
                33712, 4312,  63016, 44068, 54592, 2308,  24284, 56900, 41484,
                9380,  22784, 60808, 12612, 38784, 53688, 31288, 2912,  13256,
                23448, 17872, 47048, 25104, 6712,  20808, 46076, 64592, 28784,
                43808, 61100, 52124, 17276, 26676, 5672,  39936, 14044, 51312,
                58564, 11540, 22352, 53904, 19068, 420,   63312, 54948, 37076,
                9120,  46512, 12376, 50804, 16120, 3472,  26836, 13700, 19472,
                59076, 31748, 15364, 54900, 19204, 27568, 10524, 49236, 29388,
                2552,  15612, 25288, 59008, 30188, 65364, 51140, 8572,  47628,
                24628, 57352, 51576, 62056, 12388, 29392, 42408, 20220, 47192,
                3900,  22656, 51768, 58492, 46020, 35760, 42760, 63184, 15356,
                2108,  31980, 18796, 25912, 42296, 53360, 11328, 39696, 23216,
                35660, 30948, 47696, 40464, 24164, 13776, 908,   44796, 35228,
                10012, 19116, 1380,  26224, 5364,  57856, 10168, 64128, 55516,
                6904,  45740,
            },
            {
                17816, 4452,  48520, 10760, 62160, 20184, 45592, 7864,  17156,
                29008, 52324, 22836, 12548, 31456, 50712, 62244, 13320, 37508,
                608,   52436, 42820, 5128,  49580, 21412, 26648, 47944, 39332,
                64312, 51804, 10796, 60164, 39912, 50236, 11368, 55068, 40976,
                14972, 24612, 36048, 8908,  65492, 53104, 3292,  61244, 28912,
                36472, 45944, 64828, 15376, 42852, 27944, 13492, 48528, 5932,
                41940, 18568, 59784, 6740,  34528, 57708, 45920, 64660, 51068,
                7784,  41644, 2612,  49812, 38748, 43964, 3832,  55228, 40328,
                35848, 62492, 47024, 18380, 508,   36216, 23100, 4156,  59476,
                19564, 10272, 33820, 39080, 4944,  58744, 50992, 61644, 36464,
                54720, 11468, 15524, 28720, 62020, 560,   26540, 50772, 22552,
                40576, 53996, 60320, 45172, 1260,  30084, 19220, 5784,  55052,
                8124,  51420, 60188, 17712, 53316, 58800, 31380, 22428, 52616,
                56528, 61688, 41460, 49976, 22288, 29408, 47412, 44,    24312,
                30972, 59880,
            },
            {
                8612,  63164, 36316, 42236, 15636, 54344, 36720, 12004, 49808,
                2076,  56832, 46044, 7548,  40580, 35548, 5464,  20928, 48888,
                30696, 25244, 34624, 16928, 65496, 36400, 10436, 58096, 33136,
                7712,  28464, 35996, 3936,  30872, 17140, 26064, 37088, 5560,
                32068, 3608,  48236, 13020, 44584, 23040, 18800, 47072, 8228,
                25048, 2204,  33056, 5164,  49912, 59164, 35276, 20480, 57072,
                32104, 28656, 52684, 24460, 40448, 21036, 31176, 460,   39104,
                23588, 56948, 34112, 25952, 12516, 58444, 16692, 24432, 19964,
                8464,  53264, 5316,  41996, 55456, 10428, 45076, 32496, 42588,
                37016, 1552,  27324, 44044, 18752, 24420, 2412,  16008, 7176,
                25496, 31452, 41836, 4408,  20320, 55952, 33816, 8860,  59452,
                4928,  28156, 35508, 7184,  63744, 36960, 50000, 46428, 61848,
                15892, 25884, 11544, 4728,  34632, 8684,  41984, 64872, 12916,
                6096,  32640, 15072, 34888, 55036, 38208, 16160, 33424, 49640,
                11944, 39676,
            },
            {
                51916, 32744, 21724, 28896, 57968, 3008,  24924, 64008, 43276,
                19728, 35164, 25788, 65176, 17780, 58676, 27016, 44364, 53940,
                63812, 11472, 58756, 8032,  28968, 55664, 4316,  19556, 44208,
                15848, 55200, 21300, 62352, 44312, 56972, 8172,  51452, 22320,
                63108, 56812, 39612, 28548, 58608, 32920, 38172, 55988, 15824,
                41612, 20312, 52812, 38608, 23636, 7704,  45600, 26056, 4148,
                65080, 38328, 1360,  61916, 9548,  49892, 55412, 17444, 29268,
                10888, 45268, 21080, 63300, 8180,  47812, 32576, 64984, 50980,
                12832, 27492, 32280, 37900, 14416, 20680, 26980, 62288, 16588,
                53720, 64612, 50148, 14552, 55500, 33184, 40924, 28624, 57024,
                45348, 63324, 50216, 37024, 24296, 48612, 13636, 17548, 44276,
                56496, 11552, 48820, 16256, 21892, 58040, 25164, 3740,  29096,
                42456, 38648, 64092, 44220, 27808, 49304, 19452, 3876,  28532,
                47684, 24888, 58460, 7920,  11804, 62944, 4320,  43104, 58300,
                19864, 26240,
            },
            {
                44320, 1356,  13420, 47304, 6216,  51680, 40632, 30380, 58536,
                9552,  39060, 14976, 1004,  29884, 47344, 10184, 15448, 4400,
                18920, 39268, 46532, 22240, 43560, 14176, 40228, 61640, 24452,
                49784, 160,   41544, 14136, 48744, 2144,  34620, 59912, 11940,
                45528, 17540, 21092, 6592,  50548, 244,   11396, 30856, 50180,
                62464, 57092, 9428,  28644, 62108, 12132, 40372, 51552, 17016,
                10928, 47580, 14540, 33052, 43920, 13212, 5620,  36520, 62208,
                52848, 4024,  17056, 51616, 29752, 4744,  42068, 37308, 896,
                43824, 59792, 22544, 63716, 47828, 57200, 52084, 2864,  13048,
                30336, 6092,  21880, 46140, 8992,  63780, 48540, 21292, 10052,
                38996, 18460, 5964,  54396, 10372, 40044, 65188, 30952, 37536,
                20232, 24772, 62592, 32684, 9940,  41100, 13004, 34228, 59152,
                20096, 764,   32512, 21872, 57368, 15408, 39324, 51644, 35724,
                43880, 20748, 2848,  52392, 45460, 19340, 27596, 53480, 36172,
                5732,  60856,
            },
            {
                16640, 56320, 38484, 65292, 18452, 33848, 22068, 13988, 5024,
                53896, 32332, 60548, 50340, 55460, 22596, 37272, 61120, 33796,
                27856, 2620,  55084, 32172, 48852, 1664,  51772, 30264, 6892,
                59632, 37284, 31976, 9612,  23688, 28124, 18876, 42964, 30064,
                1648,  53840, 34244, 61600, 15240, 41936, 63984, 26456, 4012,
                34192, 13164, 47220, 17928, 32348, 55352, 1884,  30000, 60544,
                42504, 22100, 55972, 19284, 27384, 58808, 48088, 25644, 42776,
                14120, 58236, 35536, 39756, 61304, 14732, 21828, 28324, 56044,
                16268, 48984, 9776,  2156,  29648, 6560,  34796, 39672, 47044,
                24792, 41580, 57240, 31204, 724,   37348, 13176, 53128, 60612,
                2948,  34364, 14272, 59656, 29664, 2840,  45428, 6452,  51948,
                1120,  41520, 46216, 3516,  56016, 51204, 17728, 44292, 8876,
                52104, 13896, 55788, 46616, 3092,  62680, 25712, 10968, 60080,
                16516, 62432, 36816, 30276, 40148, 59940, 23328, 9608,  14364,
                47032, 29960,
            },
            {
                24176, 9508,  50516, 25860, 44064, 10424, 61172, 45928, 27644,
                48168, 20944, 42840, 11988, 4044,  41196, 7260,  57124, 42556,
                51380, 62316, 15980, 6584,  25452, 63448, 18056, 34744, 46816,
                12408, 26356, 63852, 52592, 57972, 39368, 64924, 14568, 37980,
                49668, 26956, 10452, 44100, 23872, 54912, 19588, 7192,  45060,
                22716, 39828, 944,   58900, 44192, 21548, 15252, 37176, 53972,
                25124, 2808,  35836, 63708, 7360,  39376, 2084,  21708, 33316,
                6780,  47336, 26628, 2224,  44528, 54244, 10976, 59264, 6228,
                35224, 19552, 40452, 25524, 43120, 17320, 60768, 9104,  19232,
                59736, 11068, 35784, 17848, 61036, 26496, 5260,  43704, 30300,
                23364, 47620, 26660, 43492, 19548, 52816, 22376, 27216, 60628,
                34448, 13484, 53624, 30164, 37456, 27012, 1604,  65528, 23500,
                36388, 48460, 29704, 10412, 37560, 7276,  53580, 31672, 740,
                41104, 9212,  49284, 25824, 15760, 1148,  50912, 31504, 64392,
                3232,  41124,
            },
            {
                61828, 35360, 3916,  31016, 6992,  56108, 37432, 144,   17504,
                63684, 7968,  25092, 35856, 19160, 52424, 31808, 24840, 13072,
                21796, 9812,  35568, 41756, 58516, 37648, 8760,  22624, 54104,
                4056,  20048, 44912, 16804, 3312,  47316, 5944,  54444, 24336,
                7640,  59088, 39144, 2740,  29876, 35432, 48340, 37840, 59224,
                17128, 51116, 25176, 35688, 4640,  63768, 47732, 6308,  33196,
                9284,  50552, 45508, 30488, 53296, 15604, 60876, 54704, 18264,
                65352, 30700, 9728,  23324, 18628, 33460, 49840, 24940, 46748,
                30936, 62908, 54172, 58156, 12104, 50788, 23292, 31476, 55284,
                48812, 27988, 3816,  50632, 40040, 54324, 19680, 35488, 15720,
                51836, 64452, 7940,  56680, 35264, 62832, 12324, 47868, 16472,
                57020, 8292,  18412, 22996, 6220,  58844, 47504, 31044, 54152,
                5116,  63012, 24972, 18712, 43112, 34380, 21228, 47120, 57232,
                23924, 54432, 13144, 65532, 6580,  33936, 42092, 56376, 37960,
                21680, 54020,
            },
            {
                12464, 19356, 58940, 40400, 15280, 48932, 23828, 52632, 31728,
                38272, 3252,  56244, 45612, 28536, 63136, 16200, 47876, 336,
                45272, 30256, 49860, 19660, 12228, 28224, 56468, 14832, 42548,
                38540, 56096, 7616,  35260, 30360, 21404, 11044, 32852, 61072,
                20188, 46680, 16748, 51180, 62848, 9060,  13888, 28200, 54372,
                10636, 64892, 30392, 53572, 8888,  41020, 23980, 56856, 18860,
                62380, 39884, 4884,  11832, 23496, 43300, 28628, 37844, 11592,
                50120, 40792, 56536, 60324, 37804, 7468,  64384, 1832,  39344,
                13880, 8252,  3648,  33032, 36952, 65048, 45520, 216,   36604,
                7120,  63672, 44484, 13792, 23088, 9488,  47104, 59060, 1840,
                40268, 11596, 32476, 440,   16020, 38864, 8732,  32324, 42832,
                25172, 64380, 39616, 50280, 61772, 14920, 20588, 39952, 11852,
                16756, 41140, 2072,  60616, 50240, 58872, 12128, 17192, 28812,
                5152,  33392, 20240, 44676, 57576, 47824, 11628, 18268, 7328,
                28496, 48504,
            },
            {
                32760, 44820, 52500, 28232, 62240, 19952, 34928, 58068, 12496,
                41584, 50020, 61768, 14544, 9432,  58348, 38196, 6108,  64712,
                56260, 38716, 60312, 3212,  52492, 44904, 784,   48308, 62744,
                32448, 27784, 50084, 13428, 42232, 62632, 52204, 44160, 484,
                36496, 12920, 32288, 55576, 21756, 4888,  60524, 42212, 2512,
                33160, 5972,  43484, 16124, 27532, 50028, 11616, 29204, 43992,
                13740, 26800, 57248, 17744, 51208, 34732, 8932,  4212,  45196,
                22712, 1168,  14388, 52252, 45832, 30176, 16836, 42908, 20652,
                52184, 27796, 44988, 16076, 21012, 4856,  27668, 15052, 40600,
                21808, 16792, 33612, 58012, 29184, 65472, 32836, 7156,  55196,
                28128, 21512, 50828, 42032, 24716, 49364, 56064, 2572,  37184,
                5428,  29056, 45352, 11148, 33408, 43280, 8444,  35192, 28116,
                57952, 45332, 33548, 14312, 27076, 3592,  40724, 63956, 45008,
                51308, 39272, 3288,  28440, 37128, 22084, 26604, 60316, 44444,
                15552, 448,
            },
            {
                57680, 8364,  23044, 1872,  11836, 42956, 4128,  8668,  26468,
                16324, 21844, 30240, 34352, 1708,  49264, 20900, 27552, 34600,
                17980, 26144, 5748,  23700, 33760, 65076, 31008, 25280, 10496,
                17748, 2876,  61752, 57704, 24864, 37104, 27304, 15756, 57196,
                29080, 64204, 3788,  40516, 26248, 45676, 18420, 50636, 24356,
                20964, 47448, 12440, 57660, 20064, 38336, 60336, 1080,  35404,
                48976, 21716, 59976, 32616, 132,   64276, 47884, 58048, 26428,
                61672, 36408, 19948, 27872, 3564,  12592, 55712, 35828, 57780,
                61668, 23720, 49212, 39640, 53352, 10084, 57640, 50008, 61312,
                53684, 25936, 1460,  52420, 5988,  41916, 16576, 24996, 37712,
                61796, 45672, 18540, 60512, 4380,  29312, 62016, 18956, 51520,
                59272, 21160, 200,   55300, 26484, 4300,  56472, 62572, 50660,
                7004,  21932, 52368, 8944,  55356, 30916, 23228, 6832,  35668,
                14348, 58612, 61464, 8968,  16828, 54920, 2576,  50116, 34832,
                62872, 39140,
            },
            {
                26400, 17084, 64120, 37676, 47112, 55400, 30012, 64964, 44848,
                59384, 53312, 6236,  40260, 24536, 43572, 12244, 53556, 42196,
                8872,  14200, 55012, 43100, 16452, 39116, 20552, 5180,  58332,
                40908, 23132, 45708, 9780,  1852,  18576, 4992,  50312, 9268,
                42012, 24676, 48664, 11244, 37148, 57748, 30624, 35016, 62980,
                39692, 55384, 36196, 62048, 4220,  32276, 53088, 17220, 65272,
                4104,  37876, 8048,  46088, 41172, 19800, 13184, 31328, 16064,
                53560, 32732, 5380,  42352, 48500, 62392, 25924, 9496,  4672,
                32108, 11484, 1072,  60516, 29964, 42728, 24116, 32464, 12560,
                8588,  42900, 47720, 37248, 11960, 21116, 50072, 44620, 10364,
                14532, 5252,  57924, 36724, 13464, 46284, 22720, 10360, 44532,
                14452, 35256, 48864, 16784, 38356, 22780, 46232, 18324, 848,
                25604, 37972, 61684, 19496, 44080, 38560, 49044, 54276, 10252,
                19024, 24688, 48192, 31844, 41204, 63936, 5956,  31024, 23968,
                10752, 51488,
            },
            {
                42468, 31660, 49204, 5588,  33228, 25120, 17920, 49428, 36556,
                1172,  11212, 47020, 18108, 61352, 55888, 32308, 3296,  60880,
                46712, 29908, 62304, 48712, 10716, 7344,  60032, 49900, 35964,
                53064, 14964, 33444, 54404, 48544, 40488, 59788, 33224, 21960,
                54112, 17268, 60212, 6372,  51884, 15772, 1688,  11924, 7896,
                15236, 28604, 316,   24936, 42264, 8004,  22788, 46224, 10448,
                30220, 55236, 15744, 28136, 53152, 25188, 56120, 40024, 7132,
                44088, 10468, 63856, 55072, 21888, 34468, 18280, 50260, 38868,
                46848, 19124, 35068, 55940, 6400,  18404, 37892, 47168, 4160,
                35496, 62556, 17884, 30828, 60420, 55564, 2212,  63048, 30692,
                53340, 34132, 26616, 9400,  31500, 55008, 39976, 26824, 31848,
                53584, 7748,  62612, 30748, 60392, 52292, 13540, 32432, 40556,
                48584, 11632, 30244, 4872,  64668, 16060, 136,   32736, 62284,
                27700, 43024, 2032,  52896, 11220, 46192, 14712, 40052, 54284,
                19884, 4456,
            },
            {
                56560, 12976, 59352, 21100, 41380, 61068, 14192, 6648,  23012,
                32412, 27764, 63884, 37228, 51092, 7520,  16868, 23652, 39532,
                51776, 1316,  19392, 36180, 26996, 54428, 45860, 13224, 28504,
                43980, 7728,  38384, 20724, 30888, 65388, 12400, 46292, 38480,
                2580,  31592, 44236, 20832, 28516, 65156, 43620, 54044, 48416,
                59620, 45080, 18076, 51976, 48564, 14892, 58816, 39728, 25544,
                51048, 42176, 60944, 11556, 5904,  37400, 3652,  60408, 18396,
                50512, 24016, 38060, 15100, 8328,  41012, 260,   54156, 28824,
                15016, 62844, 44060, 26464, 13856, 63624, 1556,  58840, 20500,
                28876, 56688, 23708, 3192,  39552, 27228, 34968, 18984, 40948,
                23104, 1240,  49652, 43400, 63144, 6640,  16504, 1436,  65252,
                40772, 24508, 12348, 42120, 2272,  9648,  28920, 64896, 54732,
                15904, 58508, 36140, 47560, 26388, 12360, 59168, 20820, 46052,
                38108, 5604,  56000, 17948, 22880, 35708, 27028, 59756, 8016,
                47008, 36588,
            },
        },
        {
            {
                62684, 20716, 8424,  33708, 44996, 56928, 3572,  50880, 14664,
                46364, 60952, 23012, 57648, 2284,  39264, 31676, 59440, 41120,
                49496, 26696, 38512, 16572, 24084, 36460, 656,   45060, 16816,
                6204,  56200, 31248, 10824, 19300, 37700, 29792, 64096, 39200,
                58540, 50196, 62456, 9132,  12976, 20592, 63776, 1832,  37672,
                28900, 3540,  58316, 48584, 11972, 20096, 41944, 5556,  26964,
                64900, 31112, 13972, 57328, 49700, 16948, 26612, 54644, 36720,
                62676, 33688, 25076, 19556, 54780, 16592, 44372, 11368, 47572,
                15256, 31084, 940,   38800, 60220, 14160, 50628, 47472, 40304,
                61580, 26932, 21148, 9108,  16736, 13576, 59624, 25936, 42436,
                53604, 63448, 44904, 39920, 30088, 49544, 688,   32108, 51064,
                27772, 47336, 18148, 36868, 44820, 8168,  58428, 34496, 40984,
                56020, 5900,  58080, 47200, 11760, 62636, 38836, 29880, 20760,
                40972, 56272, 32476, 51620, 2848,  59136, 13496, 40452, 18896,
                46944, 2336,
            },
            {
                42720, 29448, 52896, 16172, 38936, 6464,  22484, 64584, 34480,
                696,   17376, 37012, 5812,  50560, 46280, 63872, 4308,  7828,
                13984, 54544, 2524,  63316, 46748, 60144, 28612, 57744, 25512,
                37472, 22280, 1312,  43672, 50876, 60592, 23884, 14188, 1596,
                18872, 44012, 29144, 22684, 48456, 4464,  42180, 16012, 49036,
                61132, 43912, 8664,  31464, 63880, 2924,  53200, 16812, 10848,
                40476, 4040,  20028, 7504,  34764, 3300,  58760, 30888, 46536,
                2408,  58060, 13704, 51672, 368,   32224, 25492, 57960, 4624,
                41860, 54976, 23040, 17492, 27628, 34324, 1980,  24032, 4120,
                13088, 54528, 41912, 52080, 45216, 36828, 29732, 19080, 8604,
                35208, 31860, 2588,  16284, 57052, 10204, 41948, 58732, 16048,
                39964, 6224,  2772,  56248, 49828, 24584, 3576,  64036, 13116,
                51852, 43864, 21308, 28892, 348,   25452, 48104, 13464, 60656,
                49348, 1580,  28372, 19532, 45448, 34996, 30504, 4016,  52460,
                23156, 31912,
            },
            {
                12868, 5692,  46328, 63792, 27360, 48900, 42280, 9276,  28148,
                40332, 48560, 31144, 62784, 14812, 20044, 28184, 52412, 22644,
                58076, 35348, 10988, 41856, 31472, 14908, 7960,  39836, 11704,
                61868, 47596, 13928, 26056, 33632, 4128,  45124, 54068, 32196,
                11616, 5404,  35944, 60636, 39664, 32408, 58208, 24628, 55240,
                19544, 40028, 22612, 54776, 27260, 44892, 36936, 30108, 47124,
                22948, 59752, 51240, 27456, 63608, 45032, 10092, 14452, 22704,
                42500, 28680, 38016, 6072,  60860, 39720, 8280,  63236, 37204,
                19932, 50376, 65220, 46668, 11228, 41968, 63064, 30868, 56412,
                36424, 6936,  33832, 636,   23712, 56836, 3888,  48964, 61188,
                11680, 57812, 20564, 7872,  36468, 25560, 46432, 4036,  34812,
                53972, 21948, 60276, 33340, 11352, 30776, 18388, 47288, 26408,
                16800, 31008, 9072,  40444, 52224, 17876, 55452, 4360,  34700,
                16984, 37604, 64028, 8564,  15976, 48872, 61252, 7220,  44376,
                35900, 58736,
            },
            {
                39876, 54956, 19676, 1836,  58176, 13260, 35972, 18952, 52664,
                60088, 11236, 7596,  26860, 43404, 34264, 11484, 38024, 16976,
                30808, 45352, 19840, 52172, 5016,  21884, 48528, 53152, 3328,
                18660, 34756, 54244, 63336, 17124, 9516,  49744, 21500, 42056,
                57376, 47396, 25980, 14480, 53612, 7300,  27752, 9736,  33228,
                5520,  13340, 35332, 1092,  15484, 51148, 8092,  60812, 54948,
                1396,  43644, 32780, 12040, 41172, 23908, 37780, 51316, 18812,
                64960, 9312,  49464, 20936, 45488, 29484, 22524, 52072, 13768,
                28752, 9664,  33480, 6852,  58388, 21000, 52444, 16104, 44084,
                19420, 49728, 59200, 28784, 11316, 63968, 33200, 43804, 15636,
                24624, 41452, 50596, 28792, 54472, 62260, 18260, 23812, 12988,
                63920, 44208, 26816, 14420, 41668, 61976, 53180, 6908,  38428,
                1760,  35584, 63208, 14356, 60016, 37272, 31660, 43488, 10544,
                53780, 23048, 42124, 57692, 25716, 39248, 22364, 55964, 27836,
                9548,  17152,
            },
            {
                61996, 24468, 36840, 10504, 32460, 25352, 55148, 44616, 4484,
                24288, 56196, 21368, 51504, 59256, 3420,  55276, 46880, 62152,
                828,   25028, 61148, 27644, 57100, 33896, 64808, 42836, 30224,
                58296, 45300, 7452,  28876, 40920, 55924, 36016, 27332, 7616,
                61276, 20144, 51204, 216,   43684, 17964, 62896, 37020, 45920,
                52432, 65180, 47964, 59192, 40992, 20632, 33868, 12828, 25396,
                18460, 36488, 56716, 16180, 48384, 420,   61392, 32112, 4352,
                54144, 26460, 34600, 57236, 11028, 15348, 56000, 1828,  43400,
                59620, 3740,  40312, 25096, 49208, 36192, 5776,  28188, 10288,
                64620, 25204, 14572, 46520, 18076, 40588, 21604, 7112,  55448,
                38248, 5684,  64276, 47644, 12448, 5236,  38884, 52696, 29392,
                7820,  37448, 19688, 50832, 860,   36496, 21692, 44148, 60804,
                57264, 49260, 23140, 45560, 3144,  8276,  22472, 64684, 26632,
                46376, 6948,  13144, 33508, 52700, 316,   11204, 15344, 64744,
                50504, 1220,
            },
            {
                48204, 28864, 43972, 52068, 60492, 16888, 2872,  63080, 30424,
                14168, 45484, 36248, 40036, 18204, 9152,  24056, 41916, 6704,
                50144, 37312, 9812,  3480,  38832, 12832, 17208, 26404, 9400,
                23032, 39028, 2668,  20788, 59984, 12536, 916,   65096, 16120,
                34252, 40148, 10364, 31292, 56584, 21688, 49884, 12492, 1652,
                23776, 16876, 28740, 10448, 24160, 63248, 4552,  49944, 39632,
                62612, 9324,  5328,  52836, 29520, 19568, 55524, 7964,  47044,
                39412, 16036, 1480,  61552, 40884, 48432, 33204, 25832, 35560,
                17224, 46012, 56596, 31036, 14992, 2380,  60824, 39208, 47864,
                31800, 3232,  38276, 61016, 54268, 2124,  51336, 27372, 46956,
                30664, 356,   17068, 22312, 34352, 43284, 31728, 59292, 48548,
                2092,  56436, 10116, 46068, 57736, 27488, 9228,  32880, 12332,
                19896, 5116,  55032, 27048, 33856, 58164, 50892, 19360, 1024,
                61380, 30936, 50704, 5360,  62488, 29368, 46840, 37596, 41872,
                21144, 34080,
            },
            {
                8400,  14308, 4944,  22040, 41112, 7700,  50364, 38996, 20524,
                33588, 5936,  65060, 964,   32020, 48668, 63440, 28664, 15612,
                33148, 55600, 18248, 48032, 43964, 51468, 5912,  36688, 61152,
                49708, 13380, 52456, 31844, 48316, 25484, 38464, 46700, 30092,
                3436,  55048, 24480, 63536, 35444, 3856,  39140, 58648, 30904,
                42380, 56936, 37868, 6928,  32184, 43052, 57220, 28364, 15064,
                45284, 31344, 21944, 64516, 38808, 11076, 26004, 35928, 59912,
                12312, 44604, 24340, 30896, 18492, 3892,  64728, 7552,  51048,
                61652, 22244, 12100, 53032, 19120, 44708, 55116, 22996, 7732,
                57932, 42972, 21400, 6600,  30952, 35528, 9836,  63212, 12864,
                36204, 59432, 44664, 26744, 57992, 1636,  9372,  21048, 15368,
                40612, 32080, 24528, 62916, 5508,  16620, 54752, 47904, 25060,
                41380, 30064, 10704, 42912, 15992, 47728, 12772, 29016, 39024,
                56224, 15320, 36808, 20840, 43352, 17560, 54936, 32124, 4276,
                25912, 59220,
            },
            {
                18184, 57084, 31176, 65292, 46996, 34228, 28400, 10900, 59512,
                48332, 54024, 16408, 24992, 57160, 13348, 35872, 20744, 60096,
                11812, 41348, 64236, 29576, 24352, 60376, 20948, 55136, 128,
                33168, 16428, 64384, 42560, 4568,  18496, 58416, 22784, 51576,
                13156, 44712, 17204, 7824,  28408, 47080, 15728, 26412, 61688,
                19736, 3080,  51348, 45692, 53756, 17668, 308,   36012, 7088,
                58908, 49332, 26928, 2364,  46316, 58704, 43392, 16936, 22208,
                29268, 63528, 51944, 8580,  54448, 37336, 44976, 20172, 10840,
                39120, 28348, 164,   37404, 64416, 26672, 33660, 13600, 18224,
                52328, 34912, 11060, 50372, 24948, 44484, 17016, 56344, 23800,
                19172, 52364, 10280, 40244, 14296, 50912, 65380, 45964, 27152,
                60872, 51652, 18060, 34420, 40000, 29596, 63832, 2168,  15088,
                51984, 65296, 37936, 19032, 60764, 3904,  40840, 34848, 7768,
                44300, 24504, 48476, 2424,  58140, 24020, 6780,  60648, 13328,
                53808, 44828,
            },
            {
                39508, 49724, 37172, 192,   19104, 13472, 57780, 43924, 1680,
                26140, 41732, 11764, 29564, 52724, 40848, 2384,  44680, 4948,
                52984, 22496, 7684,  14724, 1996,  31608, 11432, 41136, 44776,
                28160, 57276, 24312, 7944,  35256, 53788, 9888,  41708, 5860,
                61800, 36848, 49044, 59372, 42876, 11116, 52952, 6700,  44948,
                9616,  34932, 14568, 26560, 60580, 11720, 47936, 23348, 54184,
                20152, 12280, 40804, 34248, 14928, 6872,  32924, 53024, 6500,
                48676, 3360,  35468, 42976, 23136, 12792, 27808, 55692, 32036,
                5960,  48804, 58284, 42888, 9008,  4380,  50084, 41492, 63364,
                1368,  27876, 55936, 62544, 13956, 39012, 60508, 4900,  41824,
                48452, 7628,  62232, 31252, 56060, 24488, 18380, 36068, 5984,
                11260, 43796, 3320,  13396, 49040, 22680, 42576, 37004, 58028,
                33544, 8100,  668,   53332, 31052, 25112, 56980, 52216, 17484,
                60236, 10768, 28644, 65264, 40064, 9828,  35776, 49308, 20236,
                30260, 2592,
            },
            {
                62600, 25164, 11756, 55468, 27068, 52440, 22976, 36400, 18068,
                8740,  61288, 37728, 46916, 7544,  18680, 58596, 50688, 26608,
                31404, 46284, 36432, 59156, 54240, 34888, 47168, 63576, 19096,
                5192,  37992, 11612, 45180, 29180, 62988, 15380, 33104, 56444,
                27684, 21836, 2320,  32396, 19808, 64776, 23436, 33044, 54832,
                39660, 23056, 64032, 5044,  20228, 30396, 40096, 65012, 33004,
                43208, 3544,  61888, 56540, 23588, 50228, 63124, 1268,  40932,
                57952, 20016, 15000, 56984, 5140,  49476, 60432, 15688, 41788,
                63272, 24380, 14032, 34256, 21408, 56912, 11356, 30816, 24436,
                45544, 16136, 40464, 19620, 6052,  49164, 29040, 33760, 2472,
                27820, 34840, 21328, 5080,  37548, 3240,  42016, 54304, 30532,
                22116, 55264, 36724, 58904, 7788,  53044, 11300, 5248,  26924,
                17508, 49936, 22640, 44736, 62652, 5924,  11452, 46228, 22384,
                3356,  54424, 34216, 18916, 13944, 52056, 26840, 42536, 64080,
                11036, 34680,
            },
            {
                21896, 16200, 43004, 6648,  61812, 39612, 4328,  64280, 55916,
                32788, 51016, 20108, 3584,  63960, 34056, 23788, 9500,  38760,
                56644, 2920,  19164, 40564, 49480, 6592,  15504, 25556, 9148,
                53628, 48884, 21104, 59772, 50984, 3156,  25800, 45952, 19196,
                9036,  40672, 54652, 14128, 5492,  37544, 50020, 684,   17636,
                58128, 29148, 49236, 42020, 36924, 56484, 2664,  15640, 9464,
                26104, 52208, 17884, 36796, 8836,  19860, 30316, 25292, 37100,
                11164, 32348, 26788, 39540, 63952, 33988, 892,   21964, 52764,
                3104,  46244, 17556, 52060, 29004, 61416, 46768, 38204, 6188,
                59740, 8980,  30384, 47176, 660,   57956, 21180, 53260, 13432,
                64648, 54876, 15668, 49976, 45620, 28308, 12232, 62076, 48960,
                208,   63940, 25756, 19268, 31564, 60992, 45136, 20856, 61964,
                46672, 39288, 28712, 14176, 36316, 20756, 33032, 64424, 27232,
                38284, 42112, 6152,  45764, 56460, 32632, 1012,  16588, 46468,
                5516,  51828,
            },
            {
                8720,  60468, 33460, 29476, 45752, 9668,  16364, 48728, 29908,
                5348,  23452, 57740, 27384, 44024, 14088, 30348, 61432, 16960,
                12820, 62500, 25376, 10564, 28120, 21412, 58016, 39520, 30104,
                60632, 34340, 1256,  17548, 40128, 12932, 37000, 49788, 448,
                64456, 34672, 50868, 25328, 57672, 41672, 28608, 62316, 13000,
                47252, 4192,  11200, 52776, 8344,  24844, 44524, 49548, 60060,
                38436, 5640,  45848, 28556, 54284, 42000, 47836, 14436, 54668,
                62156, 46264, 51564, 9168,  17948, 29840, 46824, 38364, 9432,
                35684, 30596, 60032, 6976,  40772, 2764,  14924, 19272, 50568,
                54320, 36544, 23540, 65460, 32728, 25868, 37432, 9092,  46640,
                43552, 24192, 39776, 61100, 19744, 58972, 8388,  16892, 33936,
                39672, 14892, 46260, 41800, 3980,  27552, 14588, 35172, 55184,
                2428,  11716, 59124, 55696, 8524,  42408, 49268, 1304,  14276,
                31528, 61348, 49796, 25272, 4208,  59812, 38692, 22984, 55748,
                28304, 40744,
            },
            {
                47436, 1564,  54028, 49988, 20436, 35520, 25572, 41880, 12512,
                44896, 15176, 35216, 10628, 39700, 55140, 49780, 1944,  42800,
                48380, 34212, 52420, 43832, 65464, 912,   36572, 51060, 4044,
                13940, 42672, 26888, 55236, 31568, 22556, 57492, 60736, 29688,
                16424, 43836, 11636, 30628, 47580, 15616, 8904,  35668, 43368,
                21232, 32212, 59260, 17096, 34300, 63068, 12768, 28888, 18760,
                57420, 31700, 14104, 504,   64720, 11564, 4720,  59308, 18280,
                7020,  22808, 1540,  60036, 43920, 25252, 12016, 59412, 18992,
                26780, 55404, 11712, 44720, 23704, 35880, 55232, 26276, 32992,
                2016,  13356, 44112, 52564, 11804, 42440, 16016, 59272, 18548,
                31368, 6476,  11176, 792,   30780, 35408, 47284, 24700, 52524,
                6980,  29316, 9856,  51268, 57300, 38480, 49624, 8776,  23964,
                41624, 32440, 4816,  25920, 51184, 17824, 38896, 58304, 53136,
                7640,  17200, 12120, 21540, 30584, 44704, 9188,  62080, 36232,
                14036, 58520,
            },
            {
                18348, 38008, 24008, 14552, 3452,  59520, 51756, 836,   54960,
                38356, 62864, 47880, 1188,  59560, 21632, 6076,  36396, 19948,
                29020, 7996,  3828,  16484, 32452, 55708, 45948, 18152, 62248,
                22728, 47348, 9948,  65052, 6832,  43468, 4060,  11000, 39044,
                23560, 55960, 4592,  61316, 1872,  22120, 60124, 26164, 3016,
                53916, 38188, 26804, 45624, 3388,  21864, 54980, 36664, 7204,
                23172, 42696, 49860, 60308, 24936, 33188, 38920, 44504, 28084,
                35312, 42096, 31244, 14252, 53324, 6556,  56416, 49092, 42828,
                65072, 3868,  38724, 20120, 62700, 48652, 9652,  64432, 42224,
                21472, 60920, 17912, 6760,  56656, 3780,  50904, 62116, 1748,
                35860, 56812, 27008, 52028, 14180, 55804, 3596,  43080, 20292,
                60456, 56368, 23260, 33612, 17180, 1416,  62912, 30344, 18024,
                52604, 64608, 15624, 45860, 60268, 4156,  24420, 29168, 19884,
                46752, 57232, 37320, 63468, 53636, 14996, 19892, 50492, 2288,
                25372, 31156,
            },
            {
                5136,  64536, 11108, 56276, 40272, 32052, 63764, 21644, 28108,
                8068,  19316, 52332, 31640, 17136, 26076, 46464, 54140, 63748,
                24356, 58244, 38168, 60884, 13224, 24240, 7728,  11588, 41336,
                31980, 57344, 15964, 36388, 46452, 53328, 19872, 34016, 48376,
                6964,  51992, 20444, 36148, 39708, 44348, 55088, 50380, 18408,
                65156, 7652,  14084, 60960, 40396, 50428, 32092, 1356,  46924,
                64016, 10672, 20656, 35424, 16768, 52820, 22272, 2040,  51080,
                10316, 65376, 48824, 20540, 37888, 34540, 2296,  23328, 32088,
                7696,  16324, 50584, 33312, 1624,  28656, 17388, 5352,  57824,
                29512, 48228, 39884, 27896, 34860, 22572, 39180, 29444, 20952,
                48236, 40916, 63048, 44472, 21968, 38308, 64820, 27932, 12648,
                36952, 48340, 5224,  64328, 11788, 25228, 44404, 58692, 5852,
                48108, 37148, 21548, 29872, 34472, 12924, 63652, 10236, 43540,
                33596, 26172, 436,   40652, 7384,  27916, 41844, 33232, 11688,
                54552, 43120,
            },
            {
                51464, 33052, 44924, 27980, 6004,  18256, 10680, 46792, 34468,
                56960, 24592, 4888,  42644, 65172, 8444,  33140, 11468, 14788,
                41060, 51008, 22048, 47400, 28732, 42412, 52912, 35432, 26344,
                2044,  50396, 4912,  29632, 24832, 14140, 63424, 27736, 42432,
                59424, 13852, 28776, 64100, 17412, 11128, 31088, 5800,  34148,
                41260, 22928, 48448, 30484, 18932, 5576,  62176, 16452, 39336,
                26740, 55808, 2984,  40808, 46424, 6656,  56680, 31436, 60676,
                15760, 24796, 3136,  57832, 27288, 63156, 17308, 40316, 13228,
                53904, 46516, 25796, 57628, 13976, 52148, 39376, 45492, 11632,
                35760, 3396,  9352,  63692, 49452, 14368, 45648, 5184,  54680,
                12944, 8492,  16740, 33656, 49664, 10096, 6092,  53428, 32152,
                2660,  43980, 19496, 40328, 53712, 35448, 21112, 39936, 13412,
                27128, 9784,  43500, 156,   53608, 41008, 48060, 36228, 3072,
                61764, 51680, 10932, 31936, 48740, 57600, 64988, 5768,  46056,
                60280, 21080,
            },
            {
                26472, 15180, 36740, 60908, 49344, 43224, 58204, 14684, 3600,
                41268, 61216, 29096, 13488, 50292, 38536, 57316, 43700, 44,
                30948, 4572,  9632,  18756, 34356, 2836,  59400, 19812, 64292,
                54428, 38676, 21464, 61364, 41048, 8512,  50512, 17240, 1728,
                25212, 33016, 46152, 8212,  49280, 24548, 59180, 14688, 46180,
                10228, 57644, 1072,  54592, 10968, 27960, 43856, 53364, 12468,
                33768, 51256, 30048, 9340,  58528, 27392, 13048, 19132, 45640,
                36656, 53804, 40548, 12228, 47148, 9068,  50956, 59964, 28936,
                36032, 62368, 21564, 5808,  43520, 60188, 24024, 31988, 52976,
                15740, 55348, 25288, 19312, 31192, 58364, 10024, 26420, 64512,
                32032, 23832, 58024, 2368,  26180, 18764, 59508, 40040, 16432,
                62036, 26856, 14112, 30944, 47280, 7764,  56292, 2580,  33852,
                60780, 55500, 18880, 62044, 7992,  22828, 16248, 56348, 6588,
                21888, 15500, 45312, 23576, 18344, 3512,  36752, 22724, 16540,
                38928, 8500,
            },
            {
                62564, 47180, 456,   22184, 8748,  25444, 30176, 37460, 49960,
                17544, 9300,  46164, 36168, 2796,  22812, 27732, 17984, 60496,
                35344, 45676, 57476, 54008, 63556, 6324,  49144, 30252, 9416,
                13352, 44332, 17732, 58160, 812,   31716, 55772, 36088, 61808,
                10368, 38568, 57496, 3176,  53224, 34884, 92,    38908, 28840,
                51708, 25680, 33492, 42680, 37628, 58340, 24668, 8312,  60104,
                18616, 5124,  63184, 15652, 37088, 49568, 62436, 39804, 8224,
                4080,  29248, 19468, 5920,  43496, 31364, 21436, 5056,  45196,
                1376,  10292, 41428, 30660, 35220, 8780,  19604, 996,   63132,
                21816, 41604, 60120, 44448, 512,   53588, 36680, 17764, 42612,
                51388, 37880, 5804,  61672, 42024, 30440, 45340, 23412, 49284,
                8852,  57552, 51872, 508,   60240, 27716, 15868, 45776, 51152,
                23284, 4460,  49484, 38552, 32584, 58496, 27056, 50624, 30872,
                41724, 55256, 35188, 60180, 53216, 13672, 26692, 48120, 56560,
                29928, 4112,
            },
            {
                54156, 19004, 57748, 13188, 34896, 54600, 1784,  61872, 23292,
                54252, 32428, 59028, 20604, 55508, 62096, 5456,  47856, 53212,
                25476, 15772, 38820, 12028, 23520, 39812, 15272, 46188, 37592,
                24336, 6668,  48588, 34660, 12108, 47196, 20620, 6256,  44668,
                53872, 15884, 21568, 41316, 27432, 19100, 63380, 55432, 20844,
                61668, 16884, 6016,  64152, 20248, 14800, 49400, 35808, 22156,
                41212, 48000, 25148, 44204, 21264, 1588,  33360, 25456, 55544,
                50440, 63548, 34084, 59748, 54564, 37096, 14544, 56128, 24888,
                18728, 49636, 54608, 12760, 64704, 48132, 56212, 37996, 27976,
                49716, 7128,  33216, 13604, 6408,  40924, 23232, 59544, 2752,
                45960, 19780, 53748, 35092, 14920, 56508, 11592, 1608,  33520,
                20724, 35984, 41744, 23748, 10844, 38292, 65500, 19572, 30708,
                42244, 11996, 28940, 14596, 46268, 3916,  19372, 37904, 12564,
                64752, 8996,  28800, 1792,  39176, 43456, 9920,  61172, 34640,
                12664, 42568,
            },
            {
                31592, 40508, 28412, 50972, 65356, 41532, 19976, 44608, 12080,
                6360,  39372, 26616, 15872, 11136, 40496, 30552, 9216,  20432,
                64240, 6900,  28196, 32388, 44356, 51656, 26824, 396,   55504,
                33320, 62892, 28428, 52716, 25952, 60256, 39084, 29312, 23212,
                50016, 4268,  30676, 59916, 12000, 47552, 43544, 8660,  4104,
                36616, 13128, 45364, 50992, 2624,  32376, 6416,  46084, 540,
                55496, 31556, 11408, 60756, 53232, 6160,  43260, 11516, 21904,
                16692, 42548, 13552, 23104, 26392, 8,     64824, 40988, 34200,
                61548, 38184, 27144, 3060,  17812, 25304, 5060,  14696, 46492,
                10812, 36324, 51668, 28744, 62040, 47976, 34180, 15372, 8204,
                29948, 12220, 27856, 47332, 8080,  50724, 37296, 63044, 54920,
                45968, 4480,  63676, 17692, 50360, 32620, 3672,  53976, 8944,
                62688, 36104, 56428, 24476, 64148, 10400, 42944, 60808, 2584,
                25248, 46864, 19940, 6156,  63188, 32564, 51580, 20256, 1272,
                52480, 23968,
            },
            {
                9380,  2724,  37224, 6732,  16344, 32028, 5028,  52084, 29300,
                63796, 47972, 772,   53120, 34044, 44512, 51444, 36524, 13368,
                42204, 49264, 59452, 1796,  18172, 8960,  58612, 21844, 60768,
                16232, 40376, 3456,  9592,  43256, 15396, 2960,  63636, 12580,
                34268, 42964, 65308, 6804,  36928, 15300, 32188, 23856, 41084,
                48920, 59780, 29340, 22620, 40156, 61228, 52996, 29012, 65344,
                14496, 38432, 3092,  35088, 17900, 29568, 64804, 48248, 58740,
                32128, 2260,  47524, 9240,  51648, 17644, 46052, 11304, 52344,
                7988,  15120, 58508, 45080, 40200, 51360, 33256, 42008, 59392,
                2480,  24184, 17280, 57200, 21004, 10536, 26748, 63788, 49652,
                56864, 39972, 76,    65336, 21040, 24940, 5592,  29200, 15172,
                26152, 12204, 30244, 6516,  56648, 43532, 14396, 26008, 40364,
                1724,  17172, 44556, 5836,  52796, 28368, 48472, 33992, 53380,
                17060, 40204, 50176, 56972, 24452, 16028, 7532,  28332, 45076,
                64448, 17040,
            },
            {
                55832, 59784, 43916, 23036, 48072, 10232, 59096, 25160, 36800,
                14512, 21684, 42364, 7708,  64624, 17480, 2120,  23204, 55840,
                4048,  33872, 21640, 55056, 63108, 36560, 31432, 42700, 4640,
                50432, 19652, 57248, 22940, 36696, 54444, 19072, 46404, 57108,
                26944, 18188, 54944, 25064, 50952, 2036,  57760, 52476, 11296,
                26604, 34388, 7248,  56636, 9772,  25636, 17512, 42092, 9176,
                26988, 58924, 51116, 23444, 57044, 40428, 14816, 35916, 6876,
                27412, 39164, 56104, 35368, 61096, 30756, 5388,  28140, 22448,
                32328, 4108,  20280, 29504, 56756, 9796,  62728, 29884, 20752,
                53064, 65196, 40008, 45512, 1704,  37584, 53084, 5368,  32316,
                22256, 60340, 17100, 43152, 33172, 59164, 40436, 18664, 43476,
                60476, 52924, 39756, 46908, 20972, 36344, 59692, 48148, 22368,
                57528, 50160, 32896, 20976, 39652, 844,   14956, 21660, 6888,
                58116, 36212, 14040, 30448, 45756, 37068, 59468, 40104, 4920,
                35840, 48348,
            },
            {
                33224, 12336, 26228, 61708, 55464, 39260, 18524, 46984, 3776,
                56880, 33396, 59796, 49556, 24752, 28648, 57984, 46252, 61256,
                26796, 10604, 40960, 46384, 5188,  14012, 53668, 11192, 28704,
                46800, 12908, 31520, 65000, 48852, 6712,  32484, 40760, 8672,
                648,   38192, 10208, 45656, 20600, 62072, 28580, 17608, 64796,
                1208,  54888, 18948, 44168, 37420, 12780, 57500, 34636, 21204,
                45052, 17148, 7104,  46760, 10016, 26512, 392,   53496, 45372,
                10692, 62812, 20720, 6344,  43736, 39636, 49232, 56492, 60728,
                43240, 48068, 63784, 35980, 464,   23020, 16832, 7288,  44164,
                12836, 27300, 4748,  32068, 14848, 55900, 41700, 19060, 44452,
                35764, 9612,  26080, 54280, 13788, 3524,  52320, 48232, 8724,
                896,   34372, 58516, 25036, 9484,  29052, 4696,  11532, 34740,
                27308, 7612,  61636, 11312, 59340, 36416, 56288, 30940, 44004,
                26724, 61904, 9672,  4256,  21744, 54640, 11500, 49692, 25264,
                14364, 21620,
            },
            {
                6036,  46020, 19352, 3408,  28984, 13900, 34060, 62540, 40980,
                8928,  17684, 30016, 11892, 38020, 5176,  14532, 32148, 38636,
                18752, 52520, 15860, 29244, 24964, 49656, 20188, 38748, 63840,
                8012,  35440, 42084, 988,   25424, 11388, 62184, 21840, 52164,
                59488, 49024, 14456, 31236, 40556, 35136, 4964,  44980, 38324,
                21664, 47408, 14968, 31428, 62292, 47748, 1844,  50284, 4996,
                54296, 30544, 63436, 42396, 32936, 50404, 20296, 60204, 24176,
                17944, 49872, 29400, 15200, 24704, 12292, 19972, 2540,  16504,
                38912, 11016, 24344, 53292, 13612, 39064, 49768, 55172, 34380,
                37712, 58464, 9076,  50720, 63032, 29464, 24684, 12584, 2144,
                52484, 48980, 5724,  38000, 45916, 10492, 31644, 63780, 23656,
                38416, 19708, 14164, 3292,  49964, 62192, 53564, 18996, 63852,
                46156, 15224, 30024, 42172, 18100, 24900, 50648, 64464, 11768,
                1556,  33120, 52712, 41364, 65036, 36,    18668, 31132, 56808,
                41988, 63076,
            },
            {
                30036, 39452, 50032, 35896, 8288,  51604, 204,   22608, 54468,
                26688, 51088, 1772,  45244, 19804, 62928, 54056, 8088,  43568,
                532,   64880, 35248, 8328,  61776, 43492, 33780, 1492,  25648,
                54872, 22288, 51932, 16932, 58616, 44844, 27740, 36044, 16496,
                29680, 24052, 61492, 3036,  56704, 9220,  49380, 14016, 32748,
                58896, 41448, 3376,  53352, 5896,  23180, 29956, 59364, 39852,
                11340, 36732, 22436, 13912, 5760,  61728, 37820, 3972,  31784,
                41972, 57904, 1756,  54264, 64044, 33116, 58948, 36160, 26464,
                7232,  50768, 33748, 5864,  59132, 46680, 25768, 61884, 5288,
                19404, 48224, 23336, 42768, 18340, 4248,  46704, 58348, 62372,
                15296, 29156, 63552, 22652, 56148, 27612, 36360, 15744, 55284,
                29628, 51308, 64892, 45164, 17264, 38132, 31856, 42980, 584,
                38804, 52384, 3192,  55092, 47012, 4344,  8420,  40532, 19536,
                49168, 23480, 17772, 47296, 27912, 34392, 44072, 60472, 9040,
                2328,  52788,
            },
            {
                10944, 54788, 15060, 64576, 42688, 58816, 45652, 30972, 12364,
                43308, 35756, 61620, 56176, 40592, 34204, 47620, 21752, 12148,
                30612, 48576, 57356, 39824, 2860,  55632, 17252, 59300, 45128,
                14316, 61332, 5888,  29456, 38040, 53672, 13992, 5628,  47444,
                3812,  34368, 42512, 51856, 18564, 27020, 23224, 54440, 6720,
                10332, 24760, 63492, 27608, 35720, 43116, 18360, 14360, 25208,
                60880, 3820,  49564, 57676, 28572, 16664, 43888, 55644, 13528,
                7400,  33928, 37504, 46204, 4636,  50448, 9424,  44928, 54044,
                62112, 30392, 15468, 43656, 19608, 28880, 11256, 2280,  30572,
                15364, 54572, 32888, 11992, 59956, 35964, 8756,  31708, 38336,
                20068, 42452, 34068, 1620,  17924, 61200, 21240, 2492,  42620,
                6740,  11156, 33092, 41328, 26956, 7064,  13388, 23936, 56016,
                9924,  22152, 35328, 26000, 63340, 14220, 34288, 28448, 45260,
                58792, 38296, 56028, 12724, 7460,  51952, 15080, 23240, 38088,
                27428, 18336,
            },
            {
                61080, 1280,  24572, 32168, 20492, 26916, 16864, 6548,  64124,
                20260, 5444,  15252, 23760, 10044, 3064,  26548, 51260, 60488,
                24652, 5688,  19668, 13208, 23492, 30764, 10184, 51244, 4832,
                31992, 39748, 47908, 9936,  20532, 2252,  60296, 41536, 56232,
                64104, 20060, 7580,  12788, 36756, 63744, 43364, 60796, 29848,
                50916, 37076, 17064, 12144, 49784, 8376,  64400, 52396, 46408,
                33400, 19296, 40880, 1812,  35508, 52608, 10356, 25712, 48296,
                65300, 22476, 11040, 26968, 17624, 40624, 29996, 23396, 852,
                18824, 40308, 55200, 2676,  65024, 36888, 41640, 59684, 45408,
                38768, 64172, 360,   26072, 39868, 53660, 22044, 48628, 26832,
                6788,  11052, 57856, 47984, 40524, 7656,  50660, 59376, 46868,
                26044, 56740, 22340, 1360,  58232, 51736, 61212, 48500, 29332,
                17336, 58640, 44628, 6660,  38484, 21424, 59764, 54288, 3076,
                15608, 5308,  29824, 60692, 24764, 39688, 62976, 4788,  50944,
                47388, 34876,
            },
            {
                21860, 44840, 57504, 4844,  10328, 53404, 36824, 49604, 39172,
                57324, 32256, 48292, 53236, 29580, 17800, 58424, 41456, 16148,
                37824, 54360, 44196, 27496, 60084, 47688, 41372, 21812, 36892,
                27132, 18220, 57460, 43732, 34800, 50008, 31140, 23164, 10928,
                38460, 45868, 27968, 58488, 47404, 2112,  16196, 39344, 19724,
                284,   56832, 45972, 60056, 20828, 39100, 32524, 728,   9712,
                27864, 56152, 64884, 24524, 47092, 21288, 62632, 30508, 39820,
                16320, 51332, 43280, 61920, 21280, 52992, 14264, 63628, 47776,
                12592, 58600, 27236, 22416, 49416, 7136,  16928, 23944, 52740,
                8484,  21252, 43624, 6960,  49508, 16144, 3012,  56468, 43708,
                60904, 52840, 24756, 14200, 29920, 44280, 12372, 33816, 39056,
                14464, 61596, 48012, 36256, 10556, 20624, 34632, 4424,  40796,
                64972, 32640, 12168, 48916, 30328, 51868, 10796, 24204, 36836,
                63104, 9412,  43556, 35556, 2864,  45704, 20088, 31876, 12916,
                58720, 42356,
            },
            {
                8176,  29452, 37836, 47876, 41632, 61916, 2068,  24272, 10712,
                46232, 26160, 3940,  37764, 65208, 44792, 7172,  35456, 1548,
                63820, 32516, 9588,  51760, 36492, 6768,  15188, 65184, 56504,
                11216, 1788,  62304, 24220, 15308, 64760, 6972,  18520, 52916,
                32380, 15480, 55212, 22440, 31100, 5980,  33976, 11576, 26288,
                43932, 33640, 5624,  30832, 54536, 4124,  57296, 22696, 43988,
                38224, 7016,  15732, 12132, 42656, 7884,  34280, 2632,  6444,
                59160, 29028, 664,   56136, 7916,  36704, 3936,  42664, 34004,
                7720,  37300, 45140, 10392, 31648, 57140, 34488, 47940, 12488,
                27956, 35496, 56920, 60956, 29036, 33920, 63904, 13104, 18004,
                592,   31104, 39340, 4100,  65064, 53868, 23580, 5276,  19136,
                31388, 4144,  17752, 29736, 54480, 44304, 16160, 26192, 8284,
                50884, 2300,  19924, 60628, 16532, 296,   42968, 48004, 18048,
                31832, 50352, 21504, 54088, 16616, 58256, 9556,  55668, 25948,
                2956,  16336,
            },
            {
                52360, 62616, 18104, 13404, 55944, 33704, 15492, 29528, 59704,
                18288, 8372,  60724, 12920, 20916, 31956, 55060, 11272, 28532,
                21024, 46964, 4436,  17788, 62136, 33268, 124,   28812, 49140,
                43048, 52220, 32792, 4296,  54624, 27376, 39444, 44760, 26028,
                376,   5200,  62852, 10152, 40284, 54000, 59904, 49296, 64360,
                53004, 14472, 23728, 10004, 42104, 26444, 15116, 48252, 63028,
                17628, 53424, 48924, 31688, 55132, 58868, 17816, 44848, 54460,
                20208, 35008, 13800, 47484, 32072, 60932, 27440, 57408, 20412,
                25136, 51404, 3640,  60484, 14284, 53320, 1172,  20036, 63300,
                4052,  51520, 13804, 18596, 45824, 9980,  24924, 41148, 36380,
                46368, 21520, 49712, 17156, 34908, 9308,  28040, 62932, 56040,
                51084, 43748, 8656,  64320, 39904, 3048,  59024, 47156, 37120,
                22980, 42616, 56132, 27036, 39800, 62256, 33164, 7188,  57768,
                41192, 26376, 11984, 64404, 32804, 27664, 49336, 41432, 36660,
                65288, 33144,
            },
            {
                24836, 49488, 6376,  27900, 23288, 3900,  50576, 44068, 54720,
                35000, 40368, 51436, 43160, 1904,  50264, 15452, 48112, 59276,
                42268, 14268, 57132, 39284, 22788, 55004, 45684, 19336, 7600,
                25032, 13508, 21172, 41184, 9336,  48408, 12544, 58568, 61844,
                49180, 37184, 43180, 50668, 24984, 19228, 45376, 8324,  22264,
                2944,  40888, 62668, 49084, 18644, 61192, 36080, 11700, 29376,
                34452, 4400,  26060, 39744, 440,   27624, 37276, 12340, 50116,
                25520, 41896, 9844,  39228, 24404, 16908, 49632, 10748, 53680,
                30520, 64264, 17320, 35056, 41864, 25052, 29120, 44040, 58672,
                39808, 31680, 24116, 1740,  38104, 51168, 6084,  54032, 28416,
                62492, 10172, 54884, 60380, 37840, 20512, 48764, 40860, 248,
                24596, 37288, 52732, 27104, 22576, 12732, 30880, 62872, 11276,
                53176, 14980, 35780, 5952,  9820,  55008, 20324, 28212, 13252,
                53516, 1700,  46548, 7284,  38756, 700,   13772, 22176, 5376,
                45612, 11172,
            },
            {
                40812, 800,   35664, 60372, 39676, 19140, 64984, 7052,  22360,
                620,   14868, 28316, 33804, 23408, 61260, 26268, 38152, 4972,
                24092, 51036, 30380, 2932,  26572, 12196, 40076, 59356, 35572,
                63020, 38412, 46820, 60552, 29348, 36612, 2612,  16656, 33844,
                9064,  21228, 17164, 29812, 1572,  13560, 36532, 28232, 31940,
                17780, 57260, 37636, 28856, 1352,  44676, 53648, 6580,  59540,
                21608, 43036, 61660, 10252, 20344, 64116, 47868, 22792, 60556,
                4864,  63464, 52076, 57936, 5932,  45460, 2816,  39796, 15324,
                6508,  38492, 48264, 20784, 8028,  61780, 37540, 5876,  16316,
                9644,  48908, 42028, 65456, 56008, 31260, 20484, 59108, 4324,
                14904, 32908, 7160,  26492, 1744,  45724, 58836, 13440, 34440,
                10440, 60304, 15260, 6028,  49204, 33676, 55796, 19176, 5152,
                28240, 58484, 31780, 47700, 24912, 45432, 36576, 65068, 4764,
                22772, 58956, 35888, 19332, 51616, 43124, 60572, 53392, 29788,
                18732, 56144,
            },
            {
                14876, 31012, 44544, 53128, 8972,  47248, 30684, 12756, 41956,
                62400, 52932, 19320, 57184, 5440,  39976, 8584,  18856, 63060,
                34628, 10624, 45044, 63672, 48696, 8364,  52644, 15972, 30836,
                2876,  55412, 17196, 6472,  57640, 19816, 52536, 23912, 41676,
                28560, 54728, 59616, 35116, 65144, 56076, 42136, 58984, 4512,
                47532, 34684, 7608,  12964, 51764, 31868, 23888, 40400, 2140,
                50200, 13516, 56396, 36200, 51472, 15104, 30796, 8868,  38664,
                33028, 14852, 18804, 30284, 22196, 54692, 35180, 62608, 44328,
                58812, 1520,  24016, 56528, 51056, 11856, 46528, 55608, 33756,
                26432, 54348, 20852, 7948,  16716, 12192, 43044, 34780, 47560,
                23808, 40748, 44592, 57452, 15576, 31596, 6232,  18192, 43096,
                28924, 46148, 20008, 62228, 41488, 960,   45304, 25044, 38960,
                43688, 1324,  64108, 12744, 51716, 2456,  15512, 43264, 49984,
                38992, 29164, 15084, 61480, 24128, 30908, 17108, 8576,  47952,
                38592, 59168,
            },
            {
                7940,  64184, 21224, 12160, 57472, 34404, 24720, 56304, 37140,
                26528, 9360,  48364, 64296, 11828, 46632, 56156, 31736, 52584,
                2048,  58268, 16400, 36176, 20116, 32384, 42848, 4768,  22336,
                50968, 10664, 34172, 26108, 44108, 32064, 63564, 5648,  57008,
                14068, 45088, 6708,  11552, 47328, 5744,  16396, 51300, 10668,
                61564, 25592, 54812, 21496, 65416, 16376, 9292,  56004, 18868,
                46072, 30188, 24104, 3648,  45440, 5972,  42496, 57032, 1180,
                27804, 46340, 3148,  37156, 65048, 13024, 28848, 19716, 25820,
                32864, 12632, 42716, 28676, 4560,  32172, 18552, 22816, 62576,
                13124, 2392,  45412, 35924, 27920, 61600, 50104, 1020,  18128,
                64636, 51964, 12480, 22072, 39856, 53196, 64204, 50852, 22852,
                58000, 2360,  32040, 56496, 36536, 17576, 51592, 9012,  59928,
                49628, 21052, 17524, 40616, 22292, 57920, 31204, 10296, 17664,
                33732, 44644, 9048,  47736, 4556,  56952, 34904, 63936, 26912,
                2128,  23000,
            },
            {
                46852, 50828, 17420, 26992, 2548,  43316, 16236, 49904, 4088,
                32512, 44772, 3276,  30156, 35700, 16488, 43628, 27512, 13664,
                41620, 21772, 28920, 6436,  60544, 55700, 25404, 64552, 47720,
                27880, 42068, 61876, 820,   14280, 50500, 10584, 46420, 37720,
                1400,  51772, 26264, 39604, 32352, 22776, 26684, 38640, 20588,
                44476, 15420, 41772, 3656,  38876, 46836, 34972, 26196, 62900,
                37796, 7436,  17024, 58376, 33436, 25812, 62820, 16828, 49512,
                55456, 43244, 59864, 50656, 9344,  42264, 388,   49016, 8180,
                52920, 46696, 60800, 15748, 40180, 64980, 116,   42940, 50604,
                39020, 60548, 30516, 58032, 5380,  39916, 25536, 10540, 56804,
                27500, 6340,  36164, 61856, 29212, 3056,  25140, 36972, 7920,
                54180, 39360, 13652, 24348, 10792, 29080, 65504, 15076, 35068,
                30376, 7048,  55488, 33604, 4412,  61788, 27076, 53876, 59988,
                2156,  63428, 55372, 27352, 40656, 11744, 3180,  44120, 12940,
                54312, 33644,
            },
            {
                28440, 4348,  41932, 37556, 63008, 6096,  58964, 10632, 20780,
                60664, 17428, 38648, 24564, 53508, 21552, 276,   60152, 7836,
                49432, 37516, 54232, 43892, 2484,  13824, 38620, 9280,  57580,
                13284, 19284, 37080, 53488, 22028, 40120, 25296, 18036, 30308,
                62604, 22252, 18576, 61028, 53008, 2700,  63524, 55216, 33424,
                724,   30300, 50836, 57548, 28192, 5596,  58884, 42740, 11144,
                32736, 54376, 64784, 40212, 12008, 53204, 21636, 35920, 9764,
                20136, 23932, 7180,  26132, 17424, 32668, 61124, 56188, 16492,
                36788, 21760, 9640,  34892, 53716, 25992, 48084, 8956,  28504,
                4664,  18368, 23388, 48172, 15024, 53776, 21832, 45668, 38700,
                31480, 2200,  49032, 18884, 9496,  47660, 12064, 33240, 15944,
                63052, 5164,  47020, 50764, 6992,  42492, 54384, 4252,  23092,
                11960, 62960, 44960, 25632, 38032, 47140, 7592,  40872, 21444,
                25100, 6572,  14000, 20668, 52580, 37736, 23556, 50208, 19868,
                62412, 36912,
            },
            {
                10408, 60864, 55528, 32372, 22240, 51808, 29064, 39252, 46656,
                54208, 12900, 58324, 42032, 6180,  63444, 47540, 34400, 19220,
                25724, 65468, 10160, 24252, 51100, 18316, 29588, 35236, 1484,
                44972, 31180, 4492,  47240, 65384, 7392,  35476, 58460, 49396,
                4668,  34456, 42776, 8548,  14896, 44284, 29460, 7352,  13264,
                48196, 62404, 8920,  23392, 12556, 49008, 20544, 212,   15156,
                51296, 3172,  27660, 19560, 48568, 1916,  29900, 41344, 60684,
                32392, 12608, 40300, 34824, 53920, 45160, 23352, 39148, 4456,
                29756, 64008, 2348,  56940, 19864, 12928, 37112, 59240, 15912,
                35120, 51996, 9892,  42392, 3288,  62848, 33304, 7924,  59972,
                16184, 43088, 54136, 34224, 56632, 41540, 59304, 44368, 49448,
                21524, 27408, 35336, 61836, 20388, 58556, 26148, 38216, 47568,
                41108, 52892, 15400, 10040, 50996, 18968, 12992, 35240, 45844,
                51140, 30272, 42584, 33308, 62160, 17688, 58852, 31588, 41472,
                5680,  16556,
            },
            {
                45220, 24292, 13428, 520,   48240, 18264, 35144, 7152,  64668,
                23448, 1068,  27744, 50128, 15276, 10456, 29620, 39368, 55556,
                4244,  46240, 15656, 33500, 39988, 47244, 62208, 21452, 52112,
                60200, 24060, 57016, 9968,  28824, 55648, 2508,  15416, 41196,
                27168, 12524, 47796, 57512, 37132, 19588, 50028, 40912, 59344,
                24812, 37572, 18808, 35364, 63648, 32324, 53460, 29668, 61848,
                21536, 47368, 43428, 8476,  37188, 59296, 6536,  14152, 52012,
                2376,  63180, 48308, 58064, 3624,  13932, 27840, 10788, 59212,
                41784, 49428, 24636, 44852, 5908,  51512, 30196, 21556, 55304,
                40800, 64408, 32140, 56420, 37124, 19036, 28492, 50672, 12348,
                21028, 63656, 24448, 7304,  15108, 27180, 19436, 4396,  30216,
                9868,  42176, 16528, 30692, 2904,  45552, 33428, 18208, 24,
                58212, 27792, 36332, 1768,  56788, 29292, 64376, 5008,  16008,
                56996, 36968, 10880, 48392, 20,    6468,  46588, 9456,  25856,
                52308, 57784,
            },
            {
                20416, 31208, 52728, 39644, 8880,  59888, 26032, 14076, 42300,
                30788, 36060, 8300,  61864, 33488, 56556, 23228, 51272, 12652,
                61244, 30884, 1592,  59528, 56168, 11328, 4220,  41476, 16480,
                6736,  38892, 14652, 43020, 18472, 32988, 45556, 60792, 9088,
                53812, 64020, 31452, 188,   24656, 60448, 10568, 34804, 17296,
                4940,  52988, 46040, 1888,  41420, 16720, 7496,  45332, 39544,
                25008, 35512, 12972, 61084, 23472, 55044, 44184, 27248, 46168,
                37976, 16608, 29112, 6268,  20012, 64428, 47228, 51936, 20408,
                7216,  32424, 17392, 39432, 33812, 63544, 2828,  43296, 6836,
                24928, 12528, 780,   26592, 13636, 46912, 6244,  35688, 55776,
                40016, 3556,  30124, 45784, 61436, 416,   38792, 65136, 53396,
                57104, 1100,  60044, 52312, 39948, 13904, 8128,  50240, 62052,
                20272, 5616,  32236, 60348, 22872, 42984, 48720, 32588, 59564,
                1692,  18652, 65452, 24500, 54588, 29140, 35824, 61300, 14520,
                1316,  38840,
            },
            {
                49756, 7500,  65260, 16024, 43560, 55792, 46032, 2628,  52812,
                18784, 48708, 43976, 20900, 38244, 3100,  45280, 5548,  17720,
                35932, 43136, 20032, 27084, 7540,  23088, 31916, 54836, 27636,
                48256, 33496, 61556, 50316, 26024, 11784, 51632, 20476, 24264,
                37440, 17588, 6820,  55476, 28468, 45932, 4084,  22488, 57188,
                42832, 27776, 10992, 59940, 54908, 25860, 36964, 59552, 10112,
                57148, 4872,  31352, 50296, 15804, 33664, 18484, 65440, 24384,
                8428,  55336, 22500, 43600, 37488, 30800, 1868,  35140, 14580,
                55052, 62724, 11740, 27140, 57804, 15568, 46896, 11044, 61356,
                49776, 45812, 20688, 59612, 53124, 43252, 65076, 1776,  25736,
                48048, 52544, 10076, 36844, 21336, 51208, 32660, 23552, 13692,
                37416, 25764, 47920, 11472, 23592, 64180, 56968, 31128, 10660,
                24920, 43752, 52100, 17144, 39424, 3564,  25332, 10592, 38856,
                27884, 53184, 44492, 8228,  39908, 16280, 21592, 55328, 44540,
                28672, 34648,
            },
            {
                58800, 3704,  26792, 36328, 30040, 11308, 22456, 33260, 61164,
                5720,  57360, 11848, 53852, 16972, 26168, 59396, 41496, 28528,
                9316,  48876, 52756, 37920, 64352, 44384, 50224, 13008, 63264,
                9084,  20508, 4828,  36632, 476,   63128, 38740, 30140, 3508,
                44716, 50508, 21928, 41028, 14204, 36220, 65080, 50988, 32436,
                14004, 63116, 30792, 22104, 4212,  49720, 13132, 19784, 2692,
                52416, 17960, 63760, 28176, 680,   40516, 10800, 4476,  57568,
                31840, 50120, 11560, 61304, 51212, 9204,  56764, 24512, 43348,
                28852, 46380, 720,   49920, 8408,  23272, 38124, 31820, 27600,
                17344, 34512, 38948, 8760,  29944, 23420, 16448, 57716, 32000,
                18468, 14028, 60176, 42188, 55480, 11372, 46848, 5928,  17820,
                44988, 33336, 19700, 5036,  36356, 27264, 16296, 41928, 37760,
                55168, 13092, 46648, 8684,  62216, 14212, 55604, 20360, 50260,
                5996,  22528, 14564, 31952, 58564, 49672, 11428, 40728, 4140,
                63024, 17532,
            },
            {
                41876, 22964, 46888, 60532, 19748, 5104,  63100, 37688, 16352,
                40560, 25024, 29264, 1732,  63876, 31768, 13740, 52212, 63328,
                22904, 57820, 4876,  14376, 34132, 18000, 952,   36352, 24780,
                43928, 56432, 29964, 53600, 22648, 42292, 7132,  55332, 14100,
                58844, 35024, 10708, 62296, 52588, 18508, 8812,  26340, 2272,
                48480, 7860,  39040, 15532, 34348, 43212, 65056, 28996, 33000,
                47808, 42564, 38136, 7416,  58688, 51548, 47220, 36504, 20904,
                42376, 1436,  35412, 25488, 15292, 41056, 18176, 62144, 38424,
                4232,  21596, 60516, 40824, 54496, 19244, 59756, 52500, 1992,
                56848, 5696,  63684, 50812, 3688,  40828, 11392, 38156, 8032,
                45100, 34492, 27708, 5512,  16304, 29548, 62680, 41444, 58068,
                50592, 8076,  61420, 55688, 43408, 53304, 6060,  48208, 2500,
                28684, 64024, 34624, 21384, 49388, 31080, 36260, 45208, 63404,
                41608, 60264, 35500, 3616,  26084, 62732, 6100,  32692, 24568,
                48192, 8744,
            },
            {
                31732, 11780, 56796, 1688,  41200, 53568, 47912, 27756, 50932,
                9284,  59504, 34924, 47312, 39440, 49552, 7240,  20480, 33416,
                640,   39924, 25516, 10216, 46932, 56892, 29796, 59232, 40540,
                2448,  17744, 46508, 12952, 59020, 16728, 33716, 47608, 64656,
                27316, 5652,  32296, 47504, 3160,  29888, 43896, 54748, 40528,
                20664, 56160, 45404, 52116, 57868, 7012,  23564, 40172, 55928,
                14548, 22668, 10528, 45236, 19764, 25156, 30248, 53616, 13700,
                59708, 17920, 46064, 5076,  58680, 32808, 48044, 6748,  12500,
                53632, 33024, 14912, 35732, 29936, 3492,  42260, 13716, 36456,
                22212, 43756, 14708, 32508, 18580, 48320, 61160, 54004, 22180,
                62452, 3920,  56576, 23992, 48636, 35608, 8768,  25256, 2076,
                28872, 12708, 39680, 30164, 1384,  21068, 34136, 60588, 23128,
                18140, 6616,  40684, 4184,  26984, 58804, 408,   8296,  16260,
                30184, 11320, 47788, 52508, 18768, 37528, 45772, 57396, 19788,
                12948, 54016,
            },
            {
                37812, 50784, 14916, 34612, 24880, 8464,  31604, 13164, 88,
                21700, 45436, 14556, 5340,  23612, 10596, 57116, 37160, 46132,
                16176, 62120, 31384, 53620, 41764, 21528, 6380,  15632, 52296,
                10948, 64964, 39592, 27420, 49392, 9192,  25728, 19548, 1216,
                38984, 43288, 15780, 25356, 60228, 38604, 12096, 59216, 16152,
                35800, 25196, 1300,  18708, 27468, 11132, 46596, 752,   62552,
                6064,  26536, 60092, 34760, 55260, 14820, 2436,  61944, 5656,
                26864, 39940, 63268, 28912, 52852, 12,    22800, 27540, 50684,
                58548, 8988,  25880, 45660, 6332,  63196, 24816, 48788, 8912,
                62332, 28456, 53572, 24076, 58928, 34916, 26948, 60,    30368,
                50608, 40512, 19596, 64768, 1372,  52816, 18736, 60540, 36956,
                53908, 22688, 63696, 17300, 50108, 10264, 45240, 14148, 56268,
                50700, 30636, 57888, 52752, 11696, 38540, 18892, 54616, 34260,
                25568, 2244,  57124, 42312, 13624, 29592, 2388,  51452, 36028,
                27556, 64324,
            },
            {
                6228,  19248, 28340, 44808, 64076, 17236, 58848, 43852, 55980,
                32696, 64560, 51752, 19656, 60560, 43144, 27344, 3744,  54676,
                12116, 44256, 19520, 60324, 3008,  27696, 62764, 48572, 34428,
                22452, 31928, 6140,  35648, 2680,  62840, 44396, 52972, 11424,
                22732, 57880, 54492, 20216, 7760,  49616, 22848, 33524, 5324,
                61328, 10344, 63680, 37756, 32336, 60576, 50636, 17120, 36108,
                53856, 31756, 49372, 3444,  64260, 43072, 39008, 23252, 49500,
                34244, 55096, 9600,  21176, 13312, 38052, 64268, 44724, 36084,
                19500, 41340, 65408, 17672, 51584, 10908, 33792, 55424, 17956,
                40280, 46532, 1116,  10308, 42360, 5272,  13876, 43660, 16944,
                9576,  36564, 12400, 45584, 31808, 39728, 13288, 44420, 32544,
                15916, 3404,  46392, 35028, 58636, 25184, 65204, 31812, 39400,
                9448,  45548, 15712, 24068, 42844, 64996, 46372, 22912, 51728,
                62524, 39016, 21212, 7792,  64712, 23668, 10192, 61324, 15864,
                43156, 460,
            },
            {
                57816, 40456, 55016, 10376, 49816, 2804,  39648, 19424, 26480,
                38148, 3944,  41320, 30464, 54052, 35416, 17616, 65028, 24396,
                49948, 29008, 7516,  36464, 50728, 12404, 38900, 8348,  45384,
                26296, 60712, 53992, 20748, 56908, 40412, 30900, 36428, 60504,
                49108, 29728, 2564,  45796, 36620, 64336, 544,   27672, 52884,
                44264, 29852, 49404, 4528,  14260, 21680, 38824, 28564, 9756,
                20832, 41704, 12696, 17748, 28524, 8220,  32264, 11940, 44136,
                7100,  15812, 31512, 47680, 42484, 56608, 8700,  15896, 3332,
                31356, 48600, 1136,  56164, 38672, 20940, 43648, 26936, 6068,
                31684, 12516, 60584, 36876, 56108, 21132, 52280, 63000, 47476,
                59336, 28688, 53760, 6752,  26216, 56900, 21756, 49116, 6552,
                59472, 42812, 8892,  27536, 13532, 41012, 6892,  19644, 556,
                26252, 36004, 61000, 2012,  33036, 6880,  29188, 13064, 4880,
                43692, 15408, 28452, 33384, 45048, 55856, 41024, 31136, 7340,
                46688, 22548,
            },
            {
                26224, 61196, 4000,  32576, 36672, 23508, 52532, 7096,  61032,
                10684, 16856, 25620, 8336,  13148, 1900,  47692, 31652, 8940,
                38500, 4312,  57348, 16772, 33056, 24376, 55704, 19192, 58584,
                3964,  13572, 43032, 9908,  16004, 23768, 4752,  14312, 7036,
                17992, 41600, 9988,  32964, 13780, 55320, 42184, 19112, 47544,
                13468, 22308, 40788, 58532, 54816, 43536, 3628,  52680, 45092,
                61684, 4700,  57976, 37752, 22192, 52120, 46940, 58196, 19624,
                64932, 51004, 24824, 3020,  34692, 18956, 29952, 54704, 25104,
                61360, 11684, 23296, 29220, 14116, 47392, 60160, 1644,  57584,
                50412, 19936, 25864, 48892, 29640, 7576,  38852, 31924, 24932,
                2760,  22604, 42456, 16076, 61772, 9916,  3652,  63976, 27852,
                38208, 24464, 56876, 48704, 4548,  54928, 37072, 51276, 43524,
                62796, 12816, 47764, 20652, 56352, 16872, 59828, 49460, 35956,
                58608, 10012, 48976, 53912, 1260,  17216, 49996, 20644, 59348,
                52356, 33652,
            },
            {
                17008, 8000,  47072, 20796, 60116, 12176, 29492, 48268, 33972,
                46360, 54868, 58660, 44780, 62556, 22332, 40704, 58132, 15132,
                61308, 42504, 22720, 47668, 64772, 380,   43880, 28964, 37036,
                17048, 50800, 47420, 29708, 61744, 51572, 46272, 55864, 28008,
                34720, 62460, 51960, 58772, 26016, 16784, 31284, 8632,  57720,
                34920, 7196,  17248, 26332, 34556, 8072,  64212, 24672, 13920,
                34240, 47956, 25048, 54800, 268,   62432, 16676, 35384, 27504,
                1196,  37632, 57684, 61544, 5632,  49524, 59592, 39292, 6524,
                52144, 43148, 37352, 4820,  62172, 8152,  30784, 16080, 38044,
                63712, 34412, 3516,  15048, 65244, 45272, 18392, 11152, 35772,
                6024,  56140, 50392, 32868, 37192, 46780, 30292, 51524, 19684,
                11080, 53096, 17768, 31620, 21244, 62300, 29192, 16508, 58264,
                22716, 4064,  29912, 51028, 37236, 9508,  40620, 25188, 1028,
                31220, 19208, 61908, 5612,  37772, 27400, 35104, 3568,  13828,
                38472, 11004,
            },
            {
                49536, 42276, 30556, 15284, 54176, 43056, 5052,  63320, 14332,
                22032, 1016,  36568, 32248, 49732, 28200, 6728,  51980, 20376,
                34332, 53472, 26940, 10048, 39448, 15008, 52408, 5780,  63844,
                32556, 2116,  38788, 25428, 33900, 224,   19328, 65332, 39548,
                24620, 12892, 5820,  21880, 39880, 4408,  50964, 60840, 38332,
                2648,  65372, 53280, 45792, 11620, 20020, 30748, 56736, 1892,
                18748, 29216, 8992,  44408, 31228, 40764, 5148,  10200, 53844,
                41984, 21932, 11412, 45012, 27204, 10272, 22624, 13676, 47188,
                33892, 16416, 58476, 53928, 34844, 24212, 41272, 52888, 21952,
                9560,  43548, 53884, 40164, 23488, 2132,  57776, 54276, 41652,
                64188, 14172, 20100, 60012, 1044,  24144, 43012, 14624, 34712,
                41352, 748,   61112, 36244, 44548, 2088,  11540, 47016, 8252,
                33512, 54804, 41848, 5840,  27212, 44748, 62992, 20920, 54992,
                46980, 40268, 26184, 22268, 59644, 10832, 63580, 24540, 56356,
                29000, 62536,
            },
            {
                35836, 24188, 64948, 744,   38260, 26144, 18200, 56240, 41076,
                27952, 51068, 19064, 5448,  15588, 38068, 11916, 45568, 1104,
                30172, 13040, 3448,  44872, 58940, 30876, 22100, 11528, 41868,
                54420, 19672, 57848, 6812,  12384, 36804, 43640, 9364,  50308,
                2340,  47672, 43868, 29284, 63592, 45380, 11512, 20864, 26960,
                43324, 23840, 31548, 1144,  61452, 48664, 41936, 37264, 50356,
                59672, 39300, 65216, 15684, 12268, 49112, 25564, 60132, 30436,
                47384, 14340, 32708, 17700, 52472, 36708, 43724, 62364, 2964,
                27948, 21268, 8952,  44788, 18888, 49680, 4096,  12872, 46136,
                28768, 4968,  18080, 59908, 10164, 33692, 27228, 15428, 48356,
                29516, 8936,  40012, 48856, 11988, 18044, 54636, 58748, 5232,
                62800, 47088, 13832, 8048,  52372, 24036, 34564, 53440, 26776,
                39792, 15036, 64612, 18696, 52784, 12660, 3660,  33476, 15952,
                7308,  12352, 52928, 43440, 14660, 48216, 39912, 46172, 6124,
                44080, 2104,
            },
            {
                18816, 12656, 50980, 57696, 9428,  45744, 32364, 2640,  37084,
                8612,  65340, 43388, 24124, 60156, 55040, 64156, 25308, 36100,
                55768, 63508, 17756, 50532, 7404,  35020, 48140, 57192, 26892,
                9028,  23244, 62472, 41332, 53640, 59252, 22672, 31328, 16192,
                59696, 20092, 53624, 36556, 1912,  55852, 34012, 48492, 14728,
                56692, 10128, 50588, 38712, 14460, 27292, 5496,  16668, 11084,
                22988, 6456,  52772, 34704, 21072, 56856, 36856, 18540, 3424,
                62944, 7428,  55244, 40500, 64548, 1084,  32144, 19392, 53444,
                41052, 63732, 31488, 1800,  26500, 64852, 56604, 33020, 61500,
                24792, 57136, 36116, 30324, 46396, 50220, 61720, 21316, 692,
                44732, 34696, 25960, 5704,  29196, 65360, 38980, 8712,  32084,
                26124, 22480, 29860, 39352, 59268, 18608, 42196, 63332, 2684,
                20184, 48648, 10220, 34992, 23456, 57964, 29440, 50420, 56884,
                36288, 65324, 2744,  34192, 30408, 8036,  54540, 16680, 32872,
                21728, 53092,
            },
            {
                56824, 6564,  27720, 34176, 22132, 62220, 51888, 24576, 58824,
                12904, 30728, 53580, 10588, 34124, 3632,  19636, 42932, 9368,
                48436, 22464, 37580, 28152, 19848, 62328, 2844,  15940, 45712,
                35676, 49584, 14448, 28488, 17600, 48344, 3488,  27004, 55152,
                37900, 33316, 8056,  14384, 18528, 24072, 6844,  29976, 62584,
                4740,  35672, 18848, 59568, 22092, 54504, 34832, 63504, 43820,
                32060, 46300, 3308,  27520, 62092, 7736,  51904, 43780, 23112,
                38844, 50088, 28336, 4852,  23948, 14836, 58148, 25808, 7228,
                11508, 50436, 14516, 47656, 39960, 10652, 17168, 37508, 628,
                48520, 14676, 51500, 6596,  13296, 41036, 4496,  37464, 53192,
                23376, 62236, 58676, 52464, 45924, 35656, 2784,  20824, 48784,
                43444, 57072, 3348,  15928, 48168, 28144, 5988,  13484, 59808,
                30236, 55888, 4644,  60536, 46456, 37932, 8456,  42080, 22972,
                45156, 28512, 17652, 50928, 58404, 20200, 120,   27668, 59880,
                9608,  38968,
            },
            {
                31304, 45168, 40260, 4092,  48484, 11752, 16096, 6296,  44296,
                49072, 17604, 1632,  40388, 47704, 29308, 51136, 16596, 59032,
                5412,  40976, 60400, 10940, 43024, 54236, 25340, 38564, 61168,
                928,   31584, 5064,  44488, 9824,  33212, 63112, 6560,  42860,
                11220, 64220, 26336, 57892, 41664, 61644, 37680, 54344, 17460,
                40892, 46960, 28268, 3664,  44816, 9392,  51180, 484,   26100,
                55376, 58796, 19292, 42448, 47548, 1348,  32628, 14704, 10512,
                57048, 34424, 19964, 59092, 48524, 8908,  45896, 38716, 56088,
                34772, 58996, 23220, 36548, 54860, 6860,  30136, 44000, 19704,
                8788,  39216, 64316, 22140, 25972, 58384, 19108, 8264,  32456,
                11748, 17656, 3236,  15280, 22012, 56840, 27096, 53576, 16792,
                11184, 50692, 33404, 64952, 9940,  55736, 37232, 46116, 23280,
                38612, 43776, 25616, 32076, 496,   15436, 63856, 19368, 1880,
                10732, 61008, 39524, 4692,  24268, 37388, 64412, 41980, 50268,
                14536, 63192,
            },
            {
                25020, 16292, 61568, 20000, 42368, 29636, 54932, 39076, 21248,
                61424, 35488, 25844, 58288, 20988, 7928,  39460, 32784, 26716,
                14348, 31048, 2148,  51628, 33008, 6208,  13292, 30112, 18228,
                51792, 40660, 64864, 52828, 20464, 56592, 39992, 14764, 51880,
                21776, 1128,  47160, 31972, 50572, 9680,  46104, 180,   25892,
                52176, 12300, 63956, 33444, 40216, 57536, 30312, 18252, 40568,
                13872, 8256,  37536, 11776, 24064, 39420, 28604, 64660, 46388,
                25976, 2096,  12316, 42992, 30336, 35184, 51744, 17416, 29280,
                364,   42868, 5344,  28736, 60620, 21616, 50012, 59604, 54220,
                27768, 42728, 32156, 1516,  54512, 44272, 29712, 63772, 56340,
                49408, 42100, 38228, 30588, 9900,  44212, 13460, 40676, 63516,
                37304, 5460,  19756, 41072, 25388, 304,   51012, 32740, 16992,
                7756,  51864, 12996, 18320, 41040, 54340, 26700, 48200, 59476,
                32752, 25604, 53784, 13764, 44260, 49096, 11692, 35012, 22868,
                4216,  47164,
            },
            {
                1032,  54400, 8968,  58320, 35796, 1840,  64348, 33484, 27784,
                4508,  55384, 14832, 45064, 62420, 13308, 56720, 64,    62936,
                47260, 53864, 24884, 16444, 46544, 64472, 41500, 58820, 7852,
                21768, 12212, 25588, 37280, 1460,  24088, 47316, 35820, 29760,
                60520, 44132, 17476, 4800,  28028, 21512, 13860, 42764, 32704,
                8160,  22832, 55068, 15924, 7056,  24736, 12200, 61508, 49152,
                22236, 29696, 52188, 60852, 16636, 54988, 58668, 5252,  17828,
                53360, 61044, 38112, 16464, 62008, 2496,  22052, 64136, 13120,
                48952, 20076, 62884, 16612, 45608, 12600, 35400, 2756,  23524,
                62424, 11660, 18604, 60352, 35016, 10032, 39408, 15072, 46076,
                26848, 6624,  57636, 51304, 34092, 61928, 7076,  31324, 1856,
                24224, 60412, 52600, 45024, 30688, 21460, 58536, 10852, 64392,
                1920,  57036, 36136, 62644, 49888, 6912,  31200, 13524, 36872,
                5196,  47128, 20932, 8888,  61488, 31376, 18420, 6856,  55636,
                29496, 37208,
            },
            {
                12152, 33088, 51660, 26576, 13792, 49848, 18556, 8496,  46900,
                11348, 41852, 31920, 2936,  36264, 23764, 52736, 43320, 18480,
                35084, 7324,  57332, 36444, 20472, 3400,  23192, 48988, 55120,
                34528, 59852, 46024, 16020, 30416, 61712, 7716,  18928, 3784,
                25120, 38272, 56336, 12424, 60168, 35784, 55720, 64968, 19364,
                58856, 37408, 1656,  48464, 60748, 20392, 37964, 45520, 6088,
                35132, 64056, 2024,  33564, 49420, 8644,  20956, 36276, 41508,
                31668, 8332,  45336, 27220, 54744, 10776, 40608, 6192,  44764,
                25364, 33372, 10072, 51724, 4188,  25940, 41808, 15508, 6412,
                34524, 50564, 4076,  46772, 15876, 52496, 5228,  24368, 1340,
                35940, 20504, 64596, 24044, 100,   18744, 50072, 58372, 46920,
                28696, 34968, 13244, 7060,  62128, 14964, 35452, 41728, 26596,
                47612, 21016, 28760, 10432, 44428, 20680, 58260, 40224, 52488,
                16952, 55884, 42304, 34728, 57040, 3852,  26484, 52592, 43076,
                59416, 19832,
            },
            {
                65168, 43960, 38076, 23116, 6940,  60292, 43788, 25172, 58100,
                51460, 19144, 64676, 9044,  48700, 27648, 6388,  37652, 10372,
                22324, 44388, 12636, 61980, 39468, 27844, 32164, 10496, 43356,
                5384,  28756, 3256,  54712, 10688, 43200, 58480, 13080, 53800,
                48840, 9180,  33980, 51532, 40240, 3184,  47988, 29288, 5088,
                50000, 44200, 30916, 27364, 42744, 52596, 32716, 2620,  57152,
                17056, 41920, 26556, 5020,  43992, 31072, 12548, 51020, 332,
                23568, 49132, 20420, 4332,  50188, 24364, 57968, 31496, 60892,
                53596, 38096, 56600, 40852, 31320, 64564, 57444, 52876, 30388,
                45268, 58228, 25524, 38280, 28896, 21736, 48732, 61296, 31852,
                54632, 9596,  13756, 47888, 41712, 26328, 36296, 15360, 21260,
                54648, 9540,  18272, 37892, 55220, 49056, 3604,  19048, 53824,
                31764, 40056, 60008, 3528,  24776, 34900, 2184,  9972,  23620,
                62888, 30120, 740,   27544, 16328, 46004, 38236, 62816, 14080,
                8124,  49520,
            },
            {
                27904, 5588,  17396, 47960, 31888, 39684, 53244, 15328, 3304,
                37872, 29072, 23216, 53896, 16808, 61128, 30644, 50052, 58328,
                65248, 29320, 1336,  49876, 8884,  52200, 57060, 16156, 37864,
                19384, 48128, 63148, 39088, 32496, 50324, 27184, 40384, 65152,
                31280, 15764, 62124, 23924, 18656, 8528,  25368, 15700, 35036,
                10212, 62716, 13232, 17896, 4892,  9944,  65104, 28424, 51664,
                10736, 46968, 55812, 14224, 23052, 61828, 45872, 27656, 63036,
                56612, 14612, 35288, 64840, 32684, 13568, 36764, 18500, 2968,
                8260,  14412, 23516, 1388,  19076, 37436, 8148,  47860, 18028,
                40048, 13708, 7520,  56392, 65120, 42040, 12308, 57928, 18476,
                40768, 44912, 29396, 5280,  60480, 55444, 11556, 44968, 4472,
                40068, 59328, 47768, 27164, 4916,  24324, 44152, 61472, 8532,
                14208, 5524,  46352, 16744, 55064, 65124, 28016, 50872, 45324,
                6792,  12980, 39160, 64860, 51428, 10960, 21492, 1716,  31752,
                23868, 40372,
            },
            {
                58436, 2452,  54904, 63520, 11292, 232,   28692, 21796, 63004,
                33752, 7084,  45528, 39220, 11900, 42464, 20264, 2520,  15516,
                25888, 41140, 33440, 19240, 14720, 60980, 44696, 320,   26784,
                58136, 13748, 23548, 6796,  17104, 21740, 4660,  34536, 20052,
                412,   42048, 6132,  29100, 44804, 63360, 53480, 41752, 57988,
                21560, 25000, 39944, 54100, 59652, 36748, 23200, 14484, 39620,
                24968, 20312, 60164, 36812, 40584, 54500, 6396,  16848, 38628,
                10804, 43564, 59456, 7320,  41256, 46708, 52324, 27336, 42524,
                47872, 29980, 59388, 50416, 46276, 11544, 21996, 27324, 96,
                63568, 22796, 32652, 19564, 2416,  8572,  33884, 27584, 3324,
                51812, 61796, 16440, 39116, 21804, 30772, 7724,  64284, 32984,
                23820, 1496,  41804, 64672, 32344, 11264, 38744, 29276, 50148,
                58292, 23780, 32884, 51364, 38220, 14388, 42476, 18584, 60612,
                36072, 20328, 49360, 8208,  24000, 59980, 41556, 48532, 56700,
                35660, 15708,
            },
            {
                52204, 21984, 29704, 42052, 19852, 56652, 36368, 42852, 12668,
                48956, 60212, 1000,  57168, 34676, 5284,  55280, 32844, 45468,
                52568, 4300,  56184, 47740, 25108, 4688,  35244, 22460, 64740,
                51400, 40580, 33888, 53048, 60604, 47060, 56488, 10028, 45388,
                58200, 22536, 47756, 54880, 2020,  37500, 11572, 30712, 864,
                46716, 56220, 3580,  34048, 45016, 19552, 49752, 7200,  31932,
                63096, 568,   8764,  29612, 18840, 2396,  33764, 24784, 53552,
                30928, 2976,  28076, 22676, 17848, 880,   9268,  56044, 21100,
                63988, 35420, 17284, 6532,  61704, 33912, 55024, 59924, 36484,
                10388, 43508, 51796, 47492, 36348, 55028, 45188, 14748, 37804,
                7388,  25276, 34664, 53832, 2356,  50752, 38368, 57224, 16672,
                52492, 30008, 13752, 20536, 51112, 57272, 16112, 22220, 1240,
                35320, 42652, 11188, 60888, 7712,  30732, 48136, 4932,  33152,
                25280, 56640, 44640, 31016, 3124,  33696, 17452, 27244, 5156,
                44656, 10480,
            },
            {
                32848, 46428, 13952, 35124, 6280,  60704, 47428, 9264,  55560,
                18136, 25984, 14740, 21432, 48056, 26652, 63620, 39668, 12840,
                8304,  21600, 59716, 38108, 30208, 41300, 54784, 11936, 31796,
                8144,  2316,  44180, 11676, 26588, 1552,  38176, 15332, 52808,
                27704, 36092, 13500, 59524, 32956, 20884, 50452, 61428, 18276,
                37220, 8216,  51124, 15844, 26620, 1112,  57632, 41296, 54660,
                45152, 34828, 49120, 52088, 11536, 65224, 47500, 60004, 8488,
                19888, 39848, 48360, 54112, 58432, 30452, 62088, 15556, 39560,
                4732,  11104, 44380, 28312, 24236, 42212, 14700, 4008,  49356,
                29108, 59048, 5296,  26136, 16956, 60944, 23684, 50544, 63460,
                20892, 11268, 58448, 46628, 13332, 18300, 27580, 43344, 10036,
                48612, 60672, 7568,  45308, 34988, 2800,  63204, 46016, 55312,
                18040, 63964, 27568, 20024, 428,   22952, 58944, 12156, 54604,
                1584,  41020, 14524, 58580, 52620, 37456, 12692, 63716, 51192,
                20140, 61560,
            },
            {
                25416, 7904,  62408, 50516, 24152, 16520, 26732, 5120,  32012,
                38864, 51784, 43944, 30256, 58752, 9808,  17584, 23660, 49248,
                61240, 35548, 16312, 10464, 6540,  62512, 19768, 49100, 45928,
                17892, 28948, 55828, 19680, 35544, 63908, 30460, 23896, 62280,
                7512,  50920, 17872, 10380, 41400, 26380, 15140, 6352,  43632,
                28352, 64748, 31872, 10828, 62904, 47380, 29656, 11708, 17832,
                4448,  27592, 15444, 58112, 43016, 21648, 37564, 13936, 44436,
                51900, 63916, 15204, 11180, 37660, 44688, 23860, 34088, 49412,
                25872, 54456, 58360, 39036, 2196,  52548, 31744, 20576, 41092,
                16412, 54700, 13076, 39332, 31152, 10720, 1708,  30076, 41160,
                47716, 32604, 3956,  42752, 24080, 62012, 35204, 784,   20984,
                26112, 37284, 56012, 17736, 25448, 9328,  40928, 31516, 12860,
                38072, 4240,  49188, 44744, 53820, 41284, 36780, 27120, 17332,
                64232, 9120,  28624, 18796, 5752,  43484, 55272, 8816,  30220,
                548,   37196,
            },
            {
                41696, 56172, 3004,  39104, 44432, 33528, 52800, 65420, 22504,
                2560,  61204, 7968,  37064, 3996,  53080, 43572, 604,   37496,
                30964, 27072, 46100, 53880, 43220, 14120, 27596, 3768,  59776,
                36580, 62164, 24444, 46480, 8736,  49968, 41396, 12208, 43728,
                2372,  39364, 30856, 64556, 4652,  57320, 47640, 33940, 53188,
                12640, 22864, 42400, 20132, 39052, 6544,  35604, 61736, 22016,
                59680, 38508, 23600, 6632,  32192, 3924,  56112, 28264, 1052,
                35524, 5468,  25704, 33180, 3612,  51228, 6420,  12740, 60140,
                1296,  31976, 21576, 13964, 64788, 9684,  44832, 62744, 7636,
                24108, 33984, 1088,  64160, 53504, 42624, 58356, 18724, 5716,
                56544, 27312, 65268, 8772,  31380, 54724, 6756,  46860, 63432,
                12880, 33192, 3520,  42748, 53952, 59404, 27268, 6684,  52868,
                24740, 9020,  57292, 34128, 13544, 62508, 9544,  52392, 45424,
                49772, 38128, 22892, 47304, 62660, 25760, 20876, 46624, 39932,
                59676, 17012,
            },
            {
                12796, 28496, 19156, 11056, 58976, 1536,  14380, 40644, 45840,
                28080, 12172, 19084, 64176, 32932, 14444, 28764, 62348, 55764,
                5968,  18920, 2652,  64856, 24320, 50864, 33452, 39384, 10068,
                14864, 42044, 6272,  57980, 16360, 54124, 5024,  19124, 33388,
                60068, 24952, 46484, 21892, 52348, 9032,  38860, 23904, 2440,
                60332, 49012, 4068,  58872, 55676, 24408, 14736, 50068, 42808,
                9636,  53036, 47236, 63312, 40084, 25988, 16996, 50400, 31500,
                22428, 57256, 46488, 60896, 20792, 28576, 65428, 42948, 19456,
                46448, 52036, 7328,  36096, 47636, 17988, 26880, 56236, 37396,
                60564, 45476, 19260, 48460, 22128, 7184,  46436, 35224, 12596,
                52904, 16764, 37044, 20396, 49908, 40480, 14932, 57900, 38736,
                51720, 6316,  61748, 21876, 14592, 48792, 36940, 19484, 47944,
                60316, 29992, 40088, 16868, 25776, 32180, 2556,  21352, 30240,
                3676,  34036, 59420, 11160, 32360, 1476,  35724, 14392, 24256,
                6168,  49080,
            },
            {
                43824, 64800, 32056, 54148, 21920, 29524, 49256, 10104, 58108,
                35568, 54824, 41404, 50276, 23072, 39896, 46828, 20356, 11852,
                51216, 58576, 39740, 32132, 20820, 1560,  58412, 55164, 22884,
                52124, 31412, 3228,  38380, 32824, 22656, 27964, 57144, 48164,
                15820, 55340, 3760,  35588, 43268, 28064, 63496, 19028, 55036,
                36272, 16732, 27252, 33096, 44316, 1960,  53652, 27116, 33700,
                2716,  30580, 12960, 19212, 45276, 9212,  59556, 11896, 62384,
                40292, 18144, 8924,  42192, 54896, 16552, 38872, 8608,  35740,
                27908, 16696, 41596, 55940, 29844, 4500,  50036, 12440, 5400,
                30360, 51108, 9504,  28136, 37784, 15356, 26552, 61832, 23464,
                39608, 444,   45452, 11604, 61088, 3312,  28296, 23100, 30824,
                18688, 45780, 27764, 39456, 31044, 11480, 152,   62320, 42148,
                14792, 21720, 1712,  64944, 6264,  48716, 56624, 42540, 14236,
                62104, 7032,  53076, 16476, 56476, 41992, 49852, 64272, 57216,
                34096, 53240,
            },
            {
                22876, 3952,  36480, 46692, 7688,  61068, 37328, 19800, 3416,
                16108, 47804, 25260, 1720,  10700, 59932, 4824,  33860, 24696,
                42332, 14944, 7796,  47760, 11492, 37476, 44784, 7440,  18412,
                26400, 64300, 49384, 13372, 60976, 45072, 564,   64916, 36312,
                9468,  29240, 12044, 61716, 17116, 68,    13592, 31748, 44740,
                6772,  40468, 9568,  52180, 12812, 18620, 37952, 7676,  63596,
                17020, 57912, 36132, 788,   54580, 33996, 48016, 42704, 6688,
                53268, 27100, 13436, 36144, 16,    47616, 25016, 53528, 57240,
                3464,  63728, 11224, 23732, 60196, 20284, 34120, 40276, 22300,
                15240, 42284, 57596, 3264,  62788, 32340, 52152, 2880,  50076,
                29768, 59732, 55552, 26256, 34940, 17488, 52504, 8016,  43512,
                10904, 57080, 49956, 2704,  65432, 52240, 23956, 32836, 6056,
                35472, 55968, 52016, 46048, 37136, 18312, 59968, 38704, 24956,
                19576, 44216, 26952, 39320, 4388,  28928, 10268, 18400, 3224,
                27888, 9580,
            },
            {
                58700, 15244, 51188, 26036, 17356, 42840, 55316, 24532, 63752,
                33672, 6080,  29652, 56748, 44796, 18084, 54996, 8916,  48488,
                63888, 35864, 29472, 53160, 61680, 16468, 28260, 63040, 47416,
                920,   43720, 20708, 29424, 9584,  51844, 40252, 7376,  21628,
                52936, 41548, 20288, 58056, 32732, 51368, 47104, 59800, 10964,
                29540, 62340, 22412, 46728, 64052, 30144, 59264, 47968, 21192,
                43872, 25496, 48984, 61004, 28436, 4676,  23544, 19752, 36648,
                2084,  48864, 63584, 29780, 59036, 5788,  31476, 14476, 22352,
                44652, 33408, 50408, 38776, 528,   43952, 53836, 59148, 2100,
                65436, 25696, 35352, 12852, 54372, 40724, 10484, 19616, 43380,
                8480,  14840, 22160, 6372,  42648, 47464, 59052, 33840, 64396,
                4612,  24852, 16228, 35112, 8716,  44692, 17372, 57700, 46604,
                27404, 9984,  42908, 12576, 22836, 28988, 7412,  11112, 50924,
                55596, 36036, 12888, 48772, 21044, 61076, 52400, 31540, 44900,
                20528, 38476,
            },
            {
                6612,  40868, 62004, 384,   34752, 13028, 5536,  31168, 44940,
                53088, 21404, 60732, 15012, 36124, 31348, 50540, 38572, 28236,
                3984,  21828, 57664, 25644, 42552, 4680,  35308, 13016, 32320,
                39760, 54328, 6724,  58708, 34732, 17916, 25740, 14428, 31216,
                45820, 6284,  49252, 37924, 24432, 5796,  41080, 20968, 25420,
                50680, 56668, 876,   35072, 6336,  23880, 41264, 11232, 56080,
                5528,  39588, 9976,  21912, 15496, 38400, 64724, 30152, 55684,
                16204, 32980, 43396, 19876, 11820, 40820, 62344, 49532, 10508,
                58780, 6236,  26372, 14860, 62592, 8556,  16456, 46816, 32136,
                52428, 7528,  17864, 44520, 21580, 4576,  58920, 36760, 64580,
                33468, 48608, 38196, 63664, 30084, 1752,  21036, 14044, 53644,
                36624, 41220, 60148, 20196, 56252, 28916, 37716, 12432, 3772,
                63560, 19200, 31760, 4132,  61144, 52960, 34336, 47100, 5452,
                31448, 732,   65368, 24100, 45708, 7120,  37268, 13408, 62308,
                55420, 48284,
            },
            {
                30132, 23804, 10452, 56512, 28180, 63088, 51516, 39736, 8676,
                12560, 37644, 42792, 7848,  64936, 25876, 2108,  60444, 16820,
                12936, 45816, 148,   18600, 9384,  56548, 48848, 23984, 59104,
                10284, 36184, 15264, 24516, 47564, 63228, 44020, 37356, 56244,
                62664, 26416, 1512,  14900, 54396, 10020, 36420, 55588, 3712,
                15584, 38092, 19664, 42636, 57572, 16040, 3168,  34520, 28004,
                14324, 54092, 32852, 50740, 42336, 57412, 13196, 51636, 10168,
                60284, 24608, 7448,  57024, 23624, 52340, 34448, 1932,  37852,
                29432, 18432, 55180, 47152, 31372, 37056, 28112, 24260, 10532,
                37988, 48152, 28620, 60900, 49436, 30616, 25216, 46616, 16912,
                27492, 4148,  53704, 13236, 50444, 10188, 40080, 25924, 46304,
                600,   12284, 32216, 47892, 6276,  42604, 60972, 21388, 53980,
                39380, 50760, 25284, 57284, 39764, 14952, 26284, 63900, 17128,
                41144, 59120, 9528,  33820, 57064, 15988, 41000, 24688, 980,
                35064, 17084,
            },
            {
                59884, 49812, 44092, 38264, 20608, 48368, 2896,  18488, 59588,
                26704, 48744, 480,   51928, 20692, 11948, 43956, 22880, 55512,
                41368, 61404, 32688, 51384, 39404, 30620, 20040, 52572, 4224,
                17712, 27868, 56572, 41716, 2224,  6520,  54096, 11644, 2908,
                17648, 33876, 59060, 44768, 65528, 30624, 17728, 62856, 43472,
                32904, 48416, 13548, 27588, 49928, 31816, 61272, 52104, 46564,
                65532, 3976,  18108, 62084, 2176,  7752,  46660, 26448, 4416,
                44648, 38848, 50936, 2948,  46560, 17140, 26872, 20660, 43704,
                61460, 40228, 3040,  22024, 12612, 57940, 5092,  43340, 62148,
                20104, 56472, 240,   38924, 15648, 6864,  55904, 12736, 1956,
                56976, 41720, 19068, 24600, 35652, 56208, 62628, 17704, 29024,
                61224, 22108, 54856, 26836, 14780, 1500,  24748, 48820, 30480,
                16264, 948,   34852, 47516, 8828,  44236, 2688,  56216, 22540,
                45508, 29088, 18980, 53968, 2488,  27688, 59384, 50756, 5748,
                42348, 12124,
            },
            {
                33684, 18860, 4596,  31400, 14636, 45224, 23928, 30096, 55848,
                15892, 62780, 33276, 28536, 40488, 53860, 6604,  34984, 30228,
                49220, 6512,  37164, 11860, 64004, 44364, 1664,  60680, 41180,
                45984, 65400, 50804, 11956, 21988, 32976, 28700, 59460, 24028,
                50908, 39472, 7776,  28460, 22076, 47812, 1908,  26680, 8864,
                58592, 4904,  64828, 10324, 54560, 38904, 8356,  19460, 23276,
                36900, 30192, 45068, 25908, 36048, 31460, 18456, 40380, 34188,
                21088, 64112, 14828, 31256, 37428, 8448,  64756, 54300, 7044,
                15784, 49124, 34652, 63616, 41812, 51176, 18712, 55144, 13968,
                3968,  34436, 23688, 11064, 64252, 42660, 34748, 50976, 23324,
                61400, 32272, 7624,  60408, 44188, 5480,  32104, 48972, 7488,
                42308, 51132, 9480,  39004, 63164, 51848, 35856, 10468, 6776,
                44952, 60244, 13512, 21656, 63012, 18464, 31212, 36772, 12180,
                4228,  51372, 14256, 36960, 43616, 8628,  32760, 21320, 46724,
                64692, 26272,
            },
            {
                2312,  52020, 63476, 8292,  58504, 54216, 10888, 36552, 42116,
                4264,  23344, 10100, 17928, 47792, 57436, 15192, 63284, 9724,
                19648, 25176, 56820, 15776, 23080, 7492,  26996, 14672, 33724,
                21216, 6144,  31028, 38440, 61268, 49160, 19188, 41276, 9052,
                46948, 13744, 19772, 42508, 11524, 51492, 39296, 34444, 52872,
                23332, 29700, 45444, 21484, 2172,  25804, 44220, 12776, 136,
                41792, 59936, 8540,  12288, 52768, 22856, 62692, 56048, 50044,
                576,   27892, 10404, 53688, 59656, 41908, 12956, 46028, 32648,
                24156, 11404, 53560, 8024,  25916, 1160,  33584, 48544, 30020,
                39952, 59872, 45980, 31788, 52556, 26844, 18740, 8552,  44944,
                38728, 11096, 47208, 28168, 15664, 22568, 2852,  38140, 58552,
                15168, 34116, 2988,  45540, 18352, 31536, 55972, 41104, 64836,
                27240, 55020, 37912, 5040,  29272, 50236, 42236, 61220, 53684,
                25596, 39580, 62672, 23132, 49304, 63688, 17808, 53140, 29176,
                9356,  54796,
            },
            {
                45588, 22532, 35552, 26496, 39888, 1244,  32944, 64140, 7464,
                46472, 53336, 58300, 38100, 3052,  31884, 26884, 45976, 39500,
                51768, 4236,  28860, 46892, 35176, 54892, 50368, 38300, 9232,
                55396, 25348, 324,   43800, 16404, 53228, 3680,  35232, 64588,
                30740, 56564, 61732, 36064, 4356,  57828, 15816, 61136, 12136,
                18428, 36600, 40844, 59636, 17028, 35340, 63412, 58156, 49640,
                15840, 55568, 20848, 47696, 58420, 43132, 5160,  10752, 16560,
                58252, 47172, 36000, 20176, 25200, 4176,  29048, 56780, 624,
                60184, 28244, 44964, 19632, 59080, 38528, 10108, 64500, 22544,
                8108,  50900, 17404, 4984,  57896, 1600,  37364, 63160, 29356,
                20736, 52160, 352,   57352, 39844, 50748, 55176, 11672, 25300,
                19720, 64976, 29364, 56816, 23636, 4200,  13444, 20612, 33848,
                17804, 8396,  24168, 52832, 58144, 10920, 264,   20488, 8468,
                33548, 46652, 6320,  31068, 11808, 40220, 3736,  13656, 36216,
                39496, 15832,
            },
            {
                58220, 41776, 13320, 48796, 17972, 60384, 21676, 50548, 27724,
                19488, 34532, 43848, 13360, 61940, 6012,  22036, 59332, 1004,
                13884, 65088, 43000, 59112, 2432,  18212, 63224, 29724, 47576,
                62028, 13820, 35644, 58936, 8196,  26660, 13072, 45232, 21480,
                5580,  26020, 2052,  53956, 32724, 24744, 6692,  44552, 49292,
                716,   56284, 6368,  51860, 31392, 48036, 4808,  29404, 24496,
                33620, 6104,  27516, 38596, 1152,  14732, 28540, 37532, 24172,
                32244, 7000,  42404, 62808, 16308, 48220, 35300, 39552, 18972,
                49868, 37392, 3896,  31804, 13644, 46556, 27800, 15900, 42960,
                54260, 26504, 12896, 41628, 21152, 46988, 14228, 53764, 5320,
                16252, 34976, 65396, 12684, 30956, 18976, 63512, 34792, 43968,
                48040, 5336,  37704, 11248, 42060, 61632, 47256, 6116,  58864,
                43592, 49196, 31964, 41032, 15676, 35844, 27320, 48176, 57392,
                16700, 60364, 1164,  19820, 55408, 26068, 45248, 61176, 57088,
                23996, 6000,
            },
            {
                62444, 10640, 29236, 53484, 6120,  43588, 15164, 38444, 12060,
                59444, 2160,  29664, 24840, 50664, 41736, 35628, 17036, 53544,
                32892, 37980, 20700, 8560,  31904, 41464, 12296, 22328, 2832,
                42640, 52616, 19980, 46844, 30060, 63152, 37748, 57872, 16116,
                52316, 38632, 48748, 17316, 45800, 64312, 29224, 20836, 41768,
                32276, 62776, 24200, 13240, 8844,  40020, 20728, 53996, 10064,
                40688, 51776, 61056, 17516, 34304, 64920, 48644, 53220, 44228,
                60848, 13300, 55592, 1612,  51420, 6248,  22752, 63220, 14344,
                8860,  41952, 65164, 56388, 23648, 52704, 60740, 5644,  35152,
                1452,  63232, 37608, 28972, 62048, 33040, 24472, 40044, 59152,
                48160, 26096, 42820, 23208, 5792,  45956, 8432,  28480, 1884,
                54036, 22916, 60500, 16708, 50056, 26120, 36920, 29772, 51692,
                1344,  12484, 63532, 3088,  45828, 23264, 65092, 38340, 12676,
                43668, 28324, 52748, 36268, 59176, 7584,  34360, 20244, 84,
                49184, 32160,
            },
            {
                37468, 20812, 1940,  64888, 31720, 47432, 25668, 4540,  52516,
                40780, 16460, 65496, 8508,  19828, 56092, 11020, 44144, 23844,
                48624, 27080, 11580, 53944, 49104, 25744, 5844,  57312, 37028,
                16964, 28124, 10416, 56672, 4520,  23468, 48508, 1060,  33176,
                43176, 12164, 29604, 22372, 9104,  13172, 37412, 55472, 26428,
                9880,  16256, 45892, 28292, 61852, 56688, 14772, 37484, 46608,
                64368, 3532,  30984, 9704,  42076, 21296, 7704,  30840, 2304,
                19128, 39428, 26488, 33800, 29856, 58644, 10952, 52292, 26640,
                54736, 21820, 16168, 7092,  35968, 2736,  40188, 21116, 56116,
                45748, 18836, 9100,  49824, 6136,  55324, 11208, 2820,  31332,
                9160,  55572, 3804,  60744, 33580, 53420, 16516, 41176, 58008,
                13660, 32044, 52236, 7420,  33736, 2308,  54548, 15792, 23168,
                38452, 26904, 20816, 59244, 33944, 7572,  51696, 4420,  31396,
                22408, 7264,  41096, 13924, 47508, 29744, 51832, 11048, 41152,
                27972, 17536,
            },
            {
                50496, 45056, 56228, 23772, 36640, 9816,  55712, 62376, 30548,
                23064, 48388, 54940, 37368, 46584, 28548, 3496,  62824, 8084,
                57944, 5268,  62000, 16852, 39080, 60988, 44636, 51988, 32616,
                7268,  60192, 39476, 33648, 42064, 17716, 54388, 9768,  25344,
                63456, 6944,  55356, 59976, 40904, 51096, 5244,  60568, 2780,
                53248, 38652, 50644, 34168, 3468,  43412, 26440, 1420,  18844,
                12496, 22084, 44752, 50580, 57668, 26436, 55204, 12056, 62532,
                23076, 50232, 9236,  45728, 20772, 40340, 43760, 31468, 2552,
                45400, 34516, 29988, 50520, 43908, 18244, 49300, 29676, 11464,
                32480, 23836, 53512, 43060, 35272, 16348, 60696, 50848, 41900,
                18324, 36596, 14416, 49324, 38576, 25508, 61376, 21696, 36716,
                9404,  26720, 40132, 45008, 21292, 63948, 9624,  42544, 61264,
                57268, 8984,  47780, 17640, 55248, 13492, 41588, 19348, 55728,
                61336, 50472, 24616, 63860, 2536,  16600, 23016, 65232, 44288,
                54688, 8784,
            },
            {
                60336, 33332, 7164,  16276, 41552, 19732, 884,   34776, 45036,
                10420, 5756,  33048, 228,   15220, 60436, 32412, 39212, 19236,
                30476, 36372, 45120, 23312, 588,   29228, 14760, 19452, 63892,
                25224, 49684, 1328,  21780, 51780, 12624, 31264, 40600, 58788,
                18636, 36588, 44480, 112,   34040, 19184, 47008, 15004, 34928,
                20160, 65160, 5700,  22492, 18444, 48620, 60616, 31656, 58624,
                35372, 54664, 24904, 14056, 2608,  36604, 18112, 40596, 46256,
                4704,  35728, 59608, 14408, 64992, 7620,  17552, 37092, 62132,
                6040,  57908, 10716, 61520, 25600, 12784, 64092, 7672,  58048,
                39328, 60136, 14440, 2228,  25968, 19960, 45084, 28060, 22400,
                64064, 45696, 29608, 20308, 10984, 1120,  50344, 6312,  47268,
                63344, 18588, 52,    59580, 28500, 12532, 46412, 19432, 5636,
                34820, 29184, 40060, 4792,  30668, 25144, 59856, 45292, 1468,
                35580, 10856, 18452, 32776, 43300, 37832, 58368, 4672,  14972,
                24648, 3384,
            },
            {
                39256, 13596, 26656, 52552, 60940, 48680, 27936, 14560, 57400,
                18828, 61484, 26568, 43292, 53696, 24332, 13096, 51264, 47120,
                15552, 3260,  56324, 33420, 50304, 10124, 35908, 3928,  46224,
                11844, 43460, 15980, 65084, 6884,  45768, 61676, 28396, 5068,
                50600, 23852, 15268, 27088, 62276, 30852, 24520, 57484, 44116,
                29800, 12324, 42160, 55428, 36712, 11816, 8052,  50732, 41924,
                28096, 7272,  39196, 63644, 48092, 5544,  59172, 32840, 52528,
                27928, 16752, 54792, 3244,  24976, 48944, 56924, 13160, 23460,
                48540, 19584, 39048, 28,    55252, 32940, 42608, 26828, 16580,
                46876, 5372,  30468, 65020, 48312, 57288, 7812,  38004, 12472,
                1380,  57764, 7108,  54116, 59712, 44192, 29256, 14644, 33064,
                42424, 55744, 49444, 15136, 35360, 56332, 39312, 31724, 50320,
                13632, 54080, 44044, 62240, 50808, 37180, 9200,  28628, 15464,
                48276, 39436, 5220,  57532, 27340, 8976,  48596, 31064, 35816,
                62068, 29428,
            },
            {
                19204, 58620, 43328, 3800,  11836, 38784, 63700, 7604,  50816,
                39780, 35752, 12020, 59140, 20568, 40628, 6108,  64732, 26456,
                53116, 42460, 12808, 64192, 20180, 59576, 41312, 56448, 22960,
                30000, 54060, 38164, 27416, 57112, 36520, 2768,  20428, 34508,
                11732, 47084, 57172, 8536,  52140, 4468,  10816, 39236, 49696,
                7660,  24848, 59144, 452,   30600, 63868, 23740, 15724, 4364,
                47344, 61892, 17088, 33272, 29092, 43936, 23916, 14948, 9792,
                64040, 42256, 31612, 38816, 52032, 34464, 868,   27388, 53732,
                41232, 15352, 28636, 47320, 22168, 4936,  53584, 37776, 1228,
                51232, 21356, 36156, 10872, 40324, 32120, 4056,  61476, 34620,
                51392, 24972, 32856, 40472, 16272, 35228, 64940, 54696, 3592,
                24244, 11520, 30700, 4848,  22768, 52512, 3684,  24980, 65392,
                764,   22392, 16340, 11364, 2256,  21544, 53556, 64440, 33668,
                25848, 62272, 54556, 46120, 13136, 53284, 20908, 40176, 51480,
                11872, 46856,
            },
            {
                704,   49900, 35380, 30456, 56872, 21168, 32636, 25080, 42724,
                3620,  22320, 51700, 30996, 8692,  49376, 36112, 2220,  20896,
                9564,  38520, 28204, 7224,  46764, 25552, 52824, 7740,  33864,
                62356, 3140,  9752,  32464, 24132, 14292, 49076, 55920, 42220,
                64764, 3164,  38952, 20628, 35712, 42520, 63388, 17932, 1924,
                61412, 33364, 47624, 16648, 44800, 52744, 40112, 33952, 55660,
                20764, 10976, 956,   53056, 20384, 8368,  61232, 37708, 536,
                48592, 21868, 6952,  11608, 18624, 29436, 45632, 60020, 4744,
                32772, 63296, 8064,  59280, 35812, 17564, 61280, 10312, 24216,
                62392, 43464, 55740, 17784, 52352, 23500, 15128, 54196, 43792,
                17256, 9740,  48760, 3188,  22664, 26540, 8600,  38964, 20704,
                59064, 45760, 37984, 62656, 42152, 8668,  60512, 17292, 36728,
                48328, 56860, 26248, 34208, 47216, 40716, 18052, 3692,  42884,
                7364,  20636, 30524, 552,   23784, 62640, 3024,  17360, 5660,
                57028, 22512,
            },
            {
                9928,  62792, 24556, 17836, 8876,  44536, 2404,  54272, 16916,
                29172, 64488, 45212, 2740,  17844, 57076, 43876, 33636, 58272,
                61248, 23044, 54672, 16544, 31792, 2644,  38384, 18012, 13648,
                47772, 20800, 58840, 50060, 18820, 44572, 8728,  26476, 16300,
                53464, 32100, 28804, 59536, 48756, 14092, 54352, 22820, 28596,
                51544, 13840, 38752, 27272, 9912,  19844, 2696,  58276, 25828,
                44632, 36500, 58712, 41248, 12620, 55992, 50676, 30444, 19612,
                57852, 26648, 44876, 62748, 58332, 41832, 8964,  21312, 35952,
                11400, 51292, 24712, 41816, 13580, 45192, 29972, 48640, 34364,
                13864, 29068, 8316,  26976, 280,   63084, 47584, 29468, 20928,
                59184, 27816, 63480, 42372, 56520, 12272, 47824, 52156, 17232,
                28720, 7028,  51596, 19660, 26868, 47708, 30932, 44440, 10500,
                28376, 39000, 7908,  63724, 58052, 31424, 13896, 49368, 58764,
                12216, 51788, 16688, 37168, 41884, 33244, 58808, 45700, 26092,
                34432, 42252,
            },
            {
                15996, 53900, 4836,  46460, 51644, 36440, 59572, 12672, 48496,
                58036, 7912,  15044, 39096, 62752, 28056, 11240, 16140, 30032,
                46140, 908,   35632, 43764, 62740, 11920, 49212, 28552, 57948,
                36120, 42416, 4980,  39724, 62756, 840,   60260, 37768, 22444,
                6452,  45564, 10656, 17368, 1252,  26232, 32968, 45904, 9492,
                36236, 20672, 57612, 6020,  49228, 62056, 28944, 13448, 5564,
                51712, 30288, 23320, 49276, 27064, 34724, 3336,  45516, 39840,
                12392, 55300, 35564, 2268,  23752, 14752, 52848, 64484, 17312,
                44312, 3460,  31060, 55652, 2180,  51800, 7140,  20516, 54732,
                41116, 3152,  58572, 45888, 37048, 11952, 41244, 8516,  4608,
                38976, 13540, 35704, 19480, 4896,  61984, 31968, 736,   41760,
                61572, 35904, 10412, 15836, 57380, 1392,  13996, 21348, 58468,
                4544,  52272, 19540, 43276, 6492,  23952, 55360, 27872, 38504,
                22708, 45132, 60232, 9960,  50288, 14888, 28968, 9436,  54764,
                64616, 30820,
            },
            {
                37568, 27148, 40640, 14124, 65252, 28104, 23268, 5484,  33828,
                37856, 25748, 55760, 32580, 21980, 46612, 24772, 51624, 7068,
                40952, 18532, 50776, 4812,  56792, 22376, 41756, 61512, 132,
                25900, 16684, 53256, 12460, 28508, 34296, 51824, 30568, 13060,
                62504, 41252, 24328, 52628, 64308, 40344, 5472,  60172, 55616,
                4004,  64652, 41580, 23872, 54900, 35692, 43516, 38516, 65508,
                17776, 8732,  60964, 15288, 6172,  64592, 17432, 24872, 61932,
                5300,  16164, 51024, 31896, 48324, 4340,  37908, 25240, 47452,
                56456, 38608, 18384, 21624, 65312, 27304, 39132, 57884, 4776,
                18376, 64452, 49764, 32492, 20460, 60040, 25360, 34272, 55860,
                50524, 900,   46332, 53432, 29904, 44864, 37248, 14272, 56948,
                25324, 2748,  31532, 64780, 39268, 34588, 54584, 41384, 63176,
                32680, 45836, 13032, 29232, 36980, 184,   61948, 9476,  34768,
                4956,  31908, 55516, 26384, 6292,  63736, 44032, 38488, 19600,
                6904,  49048,
            },
            {
                60720, 21448, 58880, 32468, 1576,  19276, 41840, 62204, 45900,
                19944, 10744, 49840, 5108,  53500, 1224,  60420, 38084, 56100,
                13640, 64492, 25232, 29500, 15124, 33504, 6680,  53932, 10548,
                45392, 31020, 64072, 22348, 6816,  42956, 17568, 57996, 48096,
                2732,  55564, 34108, 7476,  47536, 21300, 12428, 38012, 30860,
                16056, 44932, 12012, 1532,  31572, 15312, 8120,  22096, 48180,
                32488, 40144, 3876,  46756, 37212, 43040, 10560, 53384, 33960,
                29416, 42596, 9348,  19448, 40356, 61584, 30744, 6496,  13216,
                28072, 60760, 9804,  49004, 35052, 11628, 15808, 43612, 31552,
                25320, 10580, 38616, 15316, 5848,  51916, 18016, 43528, 65284,
                22248, 31420, 60816, 10224, 24048, 17676, 50152, 7808,  21612,
                46440, 54760, 49600, 44276, 23004, 6652,  50484, 26048, 2212,
                15968, 23664, 60788, 54772, 17260, 50736, 44500, 19388, 47924,
                64244, 1816,  40664, 18772, 35504, 22064, 1140,  52640, 24680,
                13668, 2248,
            },
            {
                44244, 11456, 7424,  47752, 38232, 10352, 53000, 15768, 30488,
                400,   60996, 43064, 36056, 13816, 41616, 9696,  20168, 4032,
                32768, 48052, 10348, 38828, 60356, 46636, 18900, 37376, 23692,
                51072, 8244,  35260, 47412, 55724, 25072, 10740, 5032,  39656,
                27448, 19580, 13980, 36992, 30068, 56800, 44156, 18424, 50140,
                25852, 52856, 34384, 61040, 18704, 51156, 59088, 2056,  57012,
                12184, 26292, 54128, 19428, 31076, 56616, 22080, 47424, 1488,
                58500, 23148, 65240, 54408, 26760, 10936, 57020, 34312, 51608,
                1320,  42320, 32548, 5532,  44684, 59044, 22920, 62720, 47192,
                35576, 53452, 22164, 44528, 57516, 30180, 2064,  47928, 14632,
                6984,  26352, 15956, 39696, 5956,  58888, 26896, 64304, 34396,
                11772, 16980, 5420,  28048, 12912, 60252, 9872,  18964, 38304,
                48968, 8624,  40756, 4908,  33248, 11976, 39968, 25996, 15440,
                53312, 29296, 13052, 61140, 49272, 56660, 11396, 32232, 47852,
                55912, 34816,
            },
            {
                18720, 52424, 56424, 23816, 29828, 57656, 49240, 7556,  39772,
                53940, 23644, 28568, 17364, 65140, 30980, 48660, 35264, 62188,
                27616, 21940, 55492, 7804,  51528, 2032,  27024, 65448, 13908,
                40504, 56888, 2600,  14940, 38456, 61640, 46644, 21020, 32608,
                50460, 59128, 42880, 61524, 3864,  26908, 8596,  63140, 288,
                22388, 7900,  28716, 47440, 40160, 24808, 45464, 28116, 34744,
                52248, 42756, 63612, 168,   59408, 28276, 6628,  14744, 38556,
                49572, 7072,  37024, 45636, 336,   15544, 43420, 19692, 59596,
                23188, 16100, 63452, 25624, 52404, 760,   37960, 9004,  3792,
                14216, 60836, 1040,  26468, 9396,  62924, 40032, 11072, 32996,
                38036, 57480, 48692, 42996, 33728, 55196, 1848,  40572, 29420,
                59848, 42856, 37664, 20112, 56828, 32828, 46788, 28980, 53532,
                34784, 57608, 26580, 47272, 65112, 22056, 59920, 6828,  57244,
                42936, 8620,  24148, 46216, 3872,  28284, 42232, 16900, 61724,
                40404, 28616,
            },
            {
                5976,  33536, 15040, 42240, 61840, 4196,  26204, 35108, 63944,
                13324, 47292, 59016, 6356,  21476, 57304, 25708, 15340, 53100,
                45416, 1332,  42468, 34596, 20732, 43604, 31456, 58236, 3808,
                17852, 29116, 60048, 19968, 31880, 1508,  53936, 36592, 65212,
                15904, 776,   9904,  23256, 48632, 15644, 53476, 32796, 42132,
                59840, 39184, 56580, 3960,  9524,  13112, 62972, 5396,  20204,
                9620,  15880, 23960, 34316, 12984, 50556, 41280, 63352, 19280,
                27476, 12844, 17528, 33352, 59040, 22584, 48896, 39536, 9112,
                46308, 36008, 54576, 13020, 19016, 30364, 55584, 27508, 49972,
                57212, 29832, 42340, 49000, 33680, 54620, 23300, 28100, 61692,
                53816, 19376, 3992,  63924, 12644, 20944, 44568, 9536,  53132,
                3732,  23876, 62596, 52004, 312,   41660, 61860, 14616, 3200,
                63588, 11668, 19836, 1824,  30052, 52944, 36152, 2776,  32532,
                20980, 37252, 52048, 33432, 15248, 37696, 59116, 7632,  22596,
                3364,  64344,
            },
            {
                25012, 50384, 560,   36140, 17620, 12476, 44612, 22232, 18304,
                3044,  33220, 9552,  38940, 51104, 1896,  43228, 7444,  39488,
                11148, 58456, 17004, 62588, 14304, 54332, 11252, 49612, 34484,
                43172, 48260, 26312, 52760, 9304,  44064, 28724, 12504, 7276,
                25676, 45144, 53072, 31692, 39828, 57756, 20676, 35696, 11436,
                48124, 14504, 19992, 63992, 35960, 53448, 30720, 41684, 36944,
                61912, 46568, 6048,  38988, 45228, 9332,  25236, 32536, 55124,
                43812, 60868, 51160, 3608,  29804, 52880, 5260,  27452, 62520,
                3084,  29336, 39232, 7844,  48028, 41528, 34196, 16660, 20996,
                39860, 6808,  18700, 13200, 36984, 4632,  16940, 50824, 2996,
                44824, 9096,  36300, 23108, 28484, 51380, 15336, 36528, 19148,
                48868, 32208, 8392,  15560, 35948, 24716, 5096,  44916, 22632,
                31316, 42580, 51556, 37916, 15112, 45532, 10444, 28092, 49064,
                63376, 17044, 58452, 6044,  65444, 20368, 45176, 26920, 35988,
                10648, 46712,
            },
            {
                38320, 30400, 63292, 21488, 46872, 55208, 29128, 59996, 51496,
                42248, 27032, 54916, 44920, 34616, 11968, 29692, 60216, 19000,
                31772, 24544, 37492, 29360, 4012,  25128, 39752, 8132,  21672,
                61824, 5816,  11656, 40004, 63324, 17052, 22844, 58604, 41388,
                56364, 34980, 17324, 63928, 12344, 2672,  44812, 6032,  27760,
                55072, 2912,  32284, 26064, 43084, 16624, 22204, 49312, 1192,
                56724, 27528, 51172, 21372, 56084, 17888, 60396, 6216,  2072,
                35848, 9172,  24632, 41692, 63756, 13344, 36284, 56032, 17500,
                11868, 50792, 57448, 21784, 59820, 4168,  64688, 11344, 45644,
                61804, 24640, 52128, 64248, 45740, 21064, 59532, 41820, 13812,
                24960, 30724, 52184, 59248, 2824,  47136, 61960, 25752, 57424,
                13892, 40192, 56328, 47900, 30200, 11420, 55532, 39192, 59008,
                17160, 5872,  56568, 23432, 7936,  57876, 18876, 41892, 55100,
                12236, 632,   43772, 30648, 10340, 54248, 1528,  49596, 53016,
                16132, 57796,
            },
            {
                13176, 44076, 9848,  52780, 7260,  39624, 1860,  10052, 37104,
                57864, 4988,  15100, 19464, 63624, 23796, 55788, 49916, 4580,
                64632, 52168, 5568,  49060, 57228, 45524, 63500, 16052, 30304,
                55536, 24296, 35876, 46124, 3844,  33744, 49932, 4760,  47800,
                30012, 20776, 5364,  37380, 28252, 59736, 49864, 23976, 65108,
                16880, 37260, 46400, 51448, 4772,  60428, 7380,  54596, 13584,
                18440, 32252, 59688, 2616,  29480, 36784, 47884, 52052, 21792,
                30332, 46108, 56728, 19900, 38764, 7780,  47004, 32300, 24460,
                42896, 33712, 5424,  26600, 14528, 43948, 23972, 51540, 1472,
                31032, 35316, 10080, 2708,  56868, 31416, 7588,  47512, 34328,
                55984, 17792, 41240, 11332, 32744, 38796, 5892,  31296, 1168,
                65424, 27176, 4376,  21716, 63384, 51340, 19400, 27860, 8236,
                50180, 35532, 27188, 62772, 40184, 31712, 61388, 24892, 4860,
                39204, 22580, 27284, 48280, 39528, 24908, 13848, 60320, 32016,
                40816, 20148,
            },
            {
                61092, 4584,  27824, 59072, 25180, 32612, 64628, 16268, 31120,
                47816, 24392, 60912, 31616, 3912,  46936, 16420, 36448, 21340,
                46080, 13680, 41568, 9932,  22148, 35284, 672,   52136, 38532,
                3216,  13696, 51248, 21108, 57056, 26708, 60828, 38080, 14328,
                9256,  60460, 51816, 47032, 8436,  18592, 31288, 38944, 51896,
                8932,  61536, 23368, 12112, 58088, 28432, 33836, 38348, 25540,
                43620, 10784, 40484, 15092, 65148, 4780,  12572, 40696, 16188,
                64332, 11876, 4716,  28356, 15788, 61072, 21668, 1156,  52300,
                64532, 19640, 45504, 61516, 31704, 36916, 8568,  56760, 38688,
                15052, 54964, 41208, 28476, 15680, 25532, 37928, 10864, 64872,
                1676,  60772, 6784,  49748, 16704, 55752, 20468, 42572, 46504,
                11488, 34184, 45472, 17268, 42628, 6672,  37128, 47356, 64904,
                12436, 43716, 468,   47724, 13192, 3756,  50716, 15800, 46720,
                34408, 61756, 51588, 18544, 56360, 35040, 42732, 4952,  28868,
                8104,  54568,
            },
            {
                23120, 35492, 41188, 14620, 48712, 19796, 43676, 50712, 6696,
                20696, 11120, 41324, 53104, 38420, 7888,  28224, 42072, 48,
                33932, 26520, 56064, 18120, 32308, 59196, 27692, 19524, 46600,
                59668, 32448, 64924, 7144,  15228, 42388, 244,   19344, 54820,
                24208, 40732, 1604,  25944, 42480, 55844, 14372, 1212,  43432,
                20580, 29932, 41512, 608,   19160, 45428, 10088, 64708, 47932,
                61060, 6664,  35624, 54324, 24304, 44252, 33392, 26176, 57384,
                37616, 48444, 33776, 54432, 50280, 44292, 10996, 58280, 37956,
                15016, 8856,  48932, 32,    53724, 17904, 28884, 48400, 19928,
                5240,  59844, 22304, 50244, 43828, 61740, 53376, 19824, 28952,
                39280, 22276, 27548, 43744, 63720, 24856, 9272,  59768, 52632,
                23304, 37360, 54356, 9896,  59804, 33456, 1264,  15392, 32048,
                25432, 55156, 18116, 33600, 53720, 21824, 36864, 7360,  59228,
                29940, 9460,  14600, 3268,  7832,  62712, 22824, 17936, 63836,
                48064, 1696,
            },
            {
                45576, 51676, 18268, 55896, 2952,  37896, 11980, 56932, 27444,
                62696, 35004, 2324,  49588, 13268, 57604, 62624, 10176, 55076,
                60688, 7344,  38408, 62844, 44052, 6676,  12628, 41468, 9420,
                25784, 17456, 44016, 39304, 29868, 53052, 11288, 31984, 64288,
                45360, 33808, 13384, 63356, 21976, 35160, 61284, 11008, 33356,
                57652, 6252,  54768, 35464, 50212, 40248, 15628, 23144, 2464,
                30496, 20684, 46000, 50004, 9748,  19912, 61444, 53340, 8112,
                524,   19024, 62212, 24112, 2512,  35376, 30004, 25684, 4736,
                41068, 28708, 35892, 11460, 40648, 58960, 3208,  62984, 33916,
                27000, 45256, 7480,  12464, 33592, 260,   6504,  49192, 14804,
                46288, 53060, 35060, 13168, 492,   36880, 29344, 14400, 6584,
                18924, 61320, 2344,  29716, 25824, 56944, 22468, 52964, 40492,
                4392,  21012, 60856, 9080,  29104, 64128, 43204, 27040, 19408,
                53492, 44564, 37216, 58152, 32116, 47112, 12128, 51892, 37848,
                25612, 33552,
            },
            {
                11724, 65188, 31172, 6528,  62464, 34476, 23236, 300,   39516,
                45804, 55460, 29552, 17384, 26188, 22284, 44560, 31116, 15160,
                23632, 49216, 29784, 2164,  15540, 54040, 48904, 64060, 36676,
                53708, 1284,  49660, 10008, 24012, 62108, 47352, 37052, 27564,
                4160,  17856, 57156, 30632, 48020, 5868,  52792, 24828, 45732,
                15672, 48272, 26644, 14008, 63216, 31440, 56300, 6148,  52844,
                57576, 16084, 27196, 1108,  58784, 30968, 3368,  42004, 22636,
                29584, 39984, 14096, 9320,  42128, 56532, 17228, 49576, 60452,
                53924, 21208, 62468, 25220, 50424, 22228, 13684, 42532, 10476,
                53020, 65524, 37332, 19324, 58308, 40288, 23356, 32024, 56676,
                4724,  9700,  19056, 58248, 45852, 54152, 48844, 40348, 32600,
                50652, 41904, 15144, 49088, 39928, 13204, 44460, 62360, 10132,
                57620, 36188, 45596, 41564, 15424, 2192,  57068, 49668, 12856,
                1132,  64832, 25932, 20596, 40712, 28020, 80,    43844, 9136,
                58352, 15960,
            },
            {
                3296,  38252, 22172, 10244, 44860, 28960, 60404, 52288, 18196,
                14288, 7508,  43272, 65516, 37244, 1660,  51688, 18640, 39484,
                53388, 11864, 20124, 46740, 36172, 25244, 31188, 21304, 5856,
                57860, 28652, 20164, 35212, 55680, 5500,  21724, 15696, 7784,
                49792, 53156, 10460, 37732, 2728,  16644, 40536, 28208, 63848,
                3344,  38548, 60328, 8800,  20912, 3860,  27484, 42892, 36860,
                11568, 40864, 62736, 34292, 39156, 17192, 47264, 11324, 50464,
                58168, 44756, 52708, 27728, 20520, 65520, 7372,  32916, 12024,
                43372, 2792,  56696, 16224, 7004,  33132, 47064, 25732, 39124,
                16724, 2504,  30296, 47604, 54132, 17000, 63972, 42952, 36292,
                25884, 62940, 40996, 30128, 5216,  17348, 21856, 64496, 3032,
                58040, 24428, 8320,  64144, 20252, 3824,  50388, 28248, 16840,
                48244, 30168, 6384,  51396, 23308, 34688, 10180, 39416, 32712,
                22712, 41656, 6460,  50456, 15756, 54108, 60100, 34500, 20992,
                54864, 29560,
            },
            {
                60028, 42632, 50480, 26236, 54336, 13988, 47712, 6008,  32960,
                25436, 58952, 21200, 10336, 32664, 59868, 5824,  47652, 35292,
                2888,  64908, 41940, 59832, 10256, 57508, 3832,  16568, 33628,
                42120, 11880, 62768, 46044, 2972,  32560, 51560, 43584, 59628,
                39512, 23484, 62484, 43808, 20500, 55540, 50204, 8136,  35884,
                18612, 53316, 23940, 33196, 46744, 59584, 51312, 17908, 33012,
                49020, 22060, 7892,  51968, 13416, 54756, 28012, 63244, 35168,
                15704, 6212,  60416, 36128, 47488, 3696,  39464, 46036, 23532,
                18164, 34836, 30652, 44856, 37940, 55296, 61052, 4868,  57588,
                22900, 50780, 11068, 25168, 3796,  28280, 9184,  12720, 59500,
                1972,  49472, 23028, 33560, 61496, 8820,  26208, 36052, 12356,
                28288, 47024, 35252, 31636, 55188, 37860, 6988,  34552, 24456,
                2412,  64480, 12260, 26532, 60108, 47376, 17996, 62804, 5560,
                46552, 56068, 34284, 10896, 62180, 4488,  24416, 13916, 40208,
                5708,  46344,
            },
            {
                19952, 8172,  56992, 16788, 40152, 3988,  20924, 42472, 63144,
                38056, 50092, 3700,  47224, 53840, 41584, 28132, 9248,  58092,
                25472, 32912, 6424,  28544, 22564, 51000, 39112, 61792, 47460,
                51732, 24308, 15384, 58756, 38972, 13480, 56956, 28796, 19996,
                1176,  34944, 26564, 7244,  29532, 60916, 13100, 22260, 58096,
                30960, 11504, 44088, 1456,  39956, 12396, 24692, 7212,  64216,
                1804,  59068, 25788, 44104, 5772,  22956, 37400, 4164,  19760,
                32356, 25664, 1408,  12508, 30412, 51432, 14880, 27224, 62952,
                5712,  51884, 9788,  63808, 1424,  19988, 29748, 9124,  43608,
                32404, 62032, 35788, 44588, 60624, 38916, 51200, 21256, 30656,
                44956, 15572, 55620, 11624, 51120, 42352, 56552, 44984, 6112,
                53252, 16904, 56,    43436, 10820, 60668, 18672, 58724, 42864,
                39068, 54952, 19956, 37684, 53536, 4728,  30696, 54968, 25380,
                14184, 59860, 17520, 30232, 45204, 37008, 48516, 31336, 64228,
                50164, 26752,
            },
            {
                32596, 35748, 768,   61656, 31960, 36364, 57592, 9604,  28024,
                12276, 54840, 35600, 16380, 24560, 12552, 20332, 63136, 14584,
                44240, 51344, 17440, 54848, 500,   43216, 13184, 27356, 8648,
                1936,  36996, 30844, 7316,  18832, 25660, 41636, 9140,  65320,
                12416, 48616, 15152, 58440, 46496, 33128, 38328, 220,   49176,
                41440, 6744,  65364, 50080, 16392, 35616, 54172, 44992, 28844,
                14064, 46804, 31944, 18940, 63864, 56712, 45944, 9288,  58980,
                48816, 55228, 38696, 63172, 23124, 57560, 9676,  54524, 37648,
                58172, 47744, 14608, 40420, 27296, 12588, 53308, 17304, 48992,
                652,   13888, 55392, 7356,  15236, 33100, 46952, 6432,  54740,
                18364, 39564, 3788,  28228, 37800, 1544,  14784, 30920, 62880,
                20872, 38376, 61468, 22700, 26668, 47468, 30512, 12752, 52040,
                8192,  15596, 32672, 832,   41156, 14668, 44836, 20712, 37176,
                52000, 27700, 1876,  39704, 21960, 8044,  19008, 56096, 1988,
                17448, 12304,
            },
            {
                63372, 48100, 23476, 11908, 49364, 24820, 52864, 17276, 46212,
                1236,  22692, 30848, 6260,  61952, 38708, 50192, 30432, 4428,
                22000, 40256, 12700, 36932, 63572, 33184, 18788, 45344, 56320,
                21852, 64680, 53796, 44660, 50300, 61020, 2216,  33720, 45264,
                30752, 55224, 41216, 4316,  18356, 10600, 24612, 44464, 61616,
                15068, 21008, 26800, 57096, 29720, 61700, 9728,  39240, 20372,
                55932, 37256, 9996,  41796, 272,   33072, 26388, 40632, 13152,
                42916, 21848, 7856,  45356, 17760, 41740, 34560, 20072, 808,
                30056, 25504, 21396, 59380, 50960, 45876, 34156, 64524, 37144,
                21164, 28816, 41016, 19472, 23440, 57164, 992,   63488, 25084,
                35268, 8504,  65276, 46676, 21392, 60832, 18560, 48188, 40232,
                9128,  57372, 50256, 5828,  15020, 53412, 1404,  21324, 46144,
                62832, 23744, 48512, 60928, 10836, 28444, 58012, 8696,  3948,
                42692, 9840,  48888, 64420, 53368, 58832, 28300, 10360, 44140,
                37588, 52668,
            },
            {
                41288, 15488, 28588, 45004, 7348,  64868, 2840,  30212, 59516,
                39628, 64200, 42712, 57808, 45104, 472,   55548, 36644, 46664,
                56784, 61024, 30864, 7748,  47068, 25688, 5132,  60700, 30148,
                11380, 40136, 3476,  27436, 10972, 36196, 22420, 53296, 17632,
                5992,  61648, 21788, 36412, 50632, 64364, 54484, 5740,  28044,
                34576, 53960, 37340, 5316,  19168, 2232,  23024, 58472, 5676,
                49896, 16760, 60224, 29160, 49608, 14576, 52688, 3068,  64640,
                29876, 16372, 52336, 31828, 2420,  59788, 6448,  50124, 44596,
                11796, 42324, 3536,  36240, 7952,  23424, 2900,  41520, 26116,
                59436, 52024, 4172,  62876, 50040, 36436, 27796, 13420, 41836,
                52228, 32444, 58512, 13608, 25640, 34964, 52368, 4372,  24164,
                33924, 13180, 28788, 45028, 37240, 65200, 40792, 33380, 5256,
                36320, 28772, 56056, 43092, 22068, 35388, 65388, 49540, 31900,
                61312, 19712, 34856, 24800, 13620, 3720,  41560, 34344, 61576,
                22396, 6392,
            },
            {
                2940,  59316, 38144, 55128, 18936, 41168, 13796, 34880, 50708,
                19916, 8440,  14108, 27136, 18984, 33344, 8096,  16804, 26360,
                10428, 1668,  19516, 49836, 58408, 15232, 40876, 51224, 34592,
                16216, 48800, 20220, 33068, 63092, 16060, 47212, 58044, 37600,
                25316, 51912, 9692,  27560, 2000,  42228, 30812, 16564, 57488,
                8948,  45936, 13004, 42588, 48424, 52064, 31308, 43168, 34864,
                3128,  24232, 53456, 7280,  21992, 61340, 18180, 35392, 24484,
                5940,  60776, 10572, 36404, 48396, 25124, 29056, 15720, 60876,
                33112, 65032, 53848, 16584, 31624, 62324, 57180, 6484,  15476,
                9520,  46540, 12360, 31096, 43636, 9964,  17580, 45908, 60948,
                4756,  20452, 49704, 43192, 7332,  55952, 10676, 27712, 59012,
                42784, 2060,  55012, 32268, 17708, 9016,  55628, 24732, 59224,
                11212, 17868, 3392,  7692,  51284, 17436, 2544,  23672, 40108,
                12680, 57336, 44340, 6840,  32400, 46784, 51956, 14960, 25468,
                57840, 30472,
            },
            {
                53468, 20740, 9956,  4280,  31268, 57916, 22188, 44516, 4960,
                26324, 52388, 34568, 48736, 10808, 52980, 23504, 64812, 42088,
                34024, 53768, 27996, 38932, 22932, 9884,  55364, 1124,  24096,
                43816, 58988, 6480,  56036, 42676, 4152,  8184,  28360, 13616,
                432,   40072, 47448, 34152, 57800, 13400, 20404, 47948, 39864,
                23596, 1288,  63120, 25152, 33608, 15032, 64504, 11964, 27028,
                61236, 40552, 12704, 34276, 47028, 38344, 10756, 57308, 50252,
                45832, 39816, 27012, 54476, 13476, 64120, 40140, 55640, 23496,
                7772,  18912, 48832, 39060, 10764, 44048, 19232, 48504, 31996,
                38888, 56264, 34584, 24864, 3096,  54228, 58900, 38212, 23084,
                11384, 29260, 16500, 756,   31240, 39224, 45628, 64600, 17032,
                49468, 20004, 63056, 23604, 3724,  48212, 27848, 14244, 50336,
                39980, 45396, 31560, 63304, 38360, 26820, 46500, 53752, 15848,
                29252, 936,   54256, 16796, 38588, 62968, 20648, 424,   49028,
                8824,  45672,
            },
            {
                26628, 33532, 62868, 51600, 26012, 36584, 48564, 10260, 62416,
                16244, 56732, 2240,  61592, 40520, 29132, 59320, 3472,  49244,
                14232, 61820, 44424, 3220,  32520, 65304, 36180, 19256, 8308,
                62196, 26756, 13248, 37228, 23428, 30924, 60344, 49760, 43984,
                63884, 19504, 15432, 60288, 23316, 53068, 37676, 3248,  62284,
                50488, 31504, 58588, 9876,  40752, 56408, 7192,  46320, 18240,
                54972, 31024, 44344, 65228, 5392,  27780, 43452, 31000, 20804,
                1080,  56376, 18008, 4452,  44060, 20584, 9992,  3120,  46812,
                35404, 27676, 5540,  55976, 24944, 29488, 13108, 54516, 22764,
                60820, 5784,  16632, 65116, 40432, 21124, 7312,  30576, 2008,
                47648, 56664, 40408, 63280, 51320, 22616, 3112,  14020, 30268,
                7116,  38064, 10424, 51876, 35196, 61116, 43184, 1976,  20064,
                61936, 53288, 25312, 14988, 57388, 12004, 33076, 7024,  62704,
                36628, 48300, 22908, 59700, 29968, 11416, 27124, 55344, 36884,
                18032, 13220,
            },
            {
                38780, 48780, 16524, 42108, 12884, 60984, 1736,  28904, 38560,
                32500, 46444, 24400, 36744, 5916,  15524, 45864, 20400, 37576,
                5620,  24372, 16896, 51836, 12368, 45572, 28320, 48372, 53828,
                38256, 2276,  51924, 46928, 18372, 53892, 39504, 11196, 21416,
                32832, 56120, 29740, 7548,  43696, 4992,  26876, 11124, 34008,
                18692, 14492, 52612, 20320, 2892,  27984, 38288, 22088, 51376,
                9144,  1200,  20456, 15932, 55784, 2460,  59724, 8140,  15208,
                37656, 32432, 62476, 23676, 34736, 50856, 31092, 37612, 52192,
                13272, 58104, 45328, 676,   61108, 50064, 35484, 1780,  43236,
                27464, 19880, 45200, 49620, 28472, 14352, 51424, 62568, 33796,
                53164, 24728, 36076, 8900,  19012, 58072, 37372, 53700, 34548,
                44672, 56768, 26900, 41004, 12716, 21140, 58224, 38028, 32928,
                4604,  9440,  21360, 48604, 108,   41100, 59464, 44096, 21056,
                10384, 26584, 41976, 5292,  50820, 8020,  40776, 60080, 31628,
                43112, 65480,
            },
            {
                6716,  23380, 984,   45940, 7976,  18088, 55484, 52676, 20880,
                43416, 7560,  13304, 21508, 55988, 42816, 32228, 11932, 57176,
                29984, 40416, 55716, 7468,  21060, 59876, 3816,  14508, 31360,
                42216, 21440, 29684, 9720,  65040, 1648,  25692, 35516, 6300,
                52968, 3440,  46184, 12252, 64664, 31600, 49952, 59648, 41544,
                7168,  28908, 44392, 36476, 48008, 16756, 60112, 4976,  35324,
                62496, 39308, 49052, 26052, 51196, 22896, 40264, 53108, 64168,
                12264, 49504, 42164, 8752,  56988, 4,     58968, 17660, 62312,
                22112, 40708, 32368, 15604, 21196, 7668,  40124, 64460, 4648,
                52804, 37444, 9224,  612,   58396, 36020, 44728, 18448, 41680,
                13808, 5764,  59600, 28068, 43940, 11928, 26212, 61604, 20808,
                1292,  60784, 15908, 4656,  45912, 30736, 7484,  17080, 55948,
                29204, 44128, 37384, 33784, 54012, 19444, 4440,  30540, 51476,
                56296, 3428,  64896, 19440, 35736, 46236, 16072, 4928,  22628,
                2712,  56316,
            },
            {
                29816, 61628, 35680, 58560, 32292, 23792, 40296, 5436,  11648,
                63704, 58380, 30500, 51684, 62448, 2916,  25460, 50328, 63708,
                9388,  47048, 34956, 62964, 42516, 37724, 24676, 57544, 10868,
                63472, 15920, 58392, 33880, 44268, 14424, 57428, 17092, 62388,
                41292, 27172, 36304, 18676, 39276, 55416, 16044, 21220, 45680,
                56260, 65044, 4620,  23360, 61956, 54200, 30560, 43860, 24388,
                14656, 28932, 58992, 36820, 11444, 33788, 17380, 46592, 26424,
                21956, 4840,  29352, 15580, 47300, 27632, 12052, 41888, 25956,
                3836,  9472,  64012, 36636, 52900, 26516, 58524, 17696, 33556,
                14132, 62100, 30276, 54684, 11832, 23680, 4404,  26636, 9648,
                64388, 21588, 46192, 15376, 34000, 5128,  47276, 8340,  41780,
                24040, 48656, 33124, 54488, 64716, 25672, 49804, 11024, 47052,
                64084, 13628, 59792, 7176,  28404, 64572, 25056, 13564, 38732,
                17212, 45580, 32860, 14156, 57728, 24916, 63348, 53548, 35148,
                50168, 11284,
            },
            {
                41224, 14712, 20084, 50844, 27944, 64816, 47132, 34020, 27108,
                17352, 104,   48412, 39032, 9852,  19088, 54392, 35804, 1808,
                22748, 18516, 26812, 516,   30528, 16384, 50528, 5432,  46684,
                35304, 3548,  49676, 7296,  27640, 40692, 50948, 31632, 47820,
                23708, 9260,  61384, 51820, 1864,  24896, 8896,  36896, 796,
                25780, 12072, 39640, 33252, 8808,  13140, 144,   49892, 57680,
                10860, 46884, 3644,  7876,  42928, 62836, 29892, 6288,  35276,
                43900, 51840, 61408, 36768, 19380, 64540, 44884, 7252,  55108,
                30804, 49880, 19512, 43104, 11924, 47364, 30640, 9832,  46208,
                25528, 48336, 21936, 39712, 33080, 61588, 47528, 55380, 38584,
                49648, 31672, 3552,  53984, 62288, 50840, 30072, 18064, 52540,
                12568, 29304, 9060,  18864, 36692, 572,   53208, 40052, 22124,
                26316, 1628,  50096, 17504, 43424, 10144, 47144, 35044, 61292,
                8352,  28592, 40100, 52672, 1148,  30716, 43768, 12712, 27832,
                18776, 45500,
            },
            {
                24604, 54436, 5488,  10396, 43480, 3372,  13856, 57524, 49856,
                36336, 41960, 25068, 14856, 34716, 46576, 27828, 16128, 41228,
                60128, 51280, 13836, 58192, 10276, 54164, 33464, 40440, 20056,
                25440, 55732, 39136, 23068, 60272, 4024,  20960, 10576, 824,
                56896, 15828, 48724, 22020, 34220, 42824, 62528, 48936, 30672,
                53772, 17496, 50312, 57456, 27192, 42316, 20048, 33824, 40168,
                17840, 64224, 32568, 19592, 57120, 14072, 404,   60716, 54984,
                10892, 2788,  25088, 53988, 5832,  39188, 22992, 34112, 14488,
                45996, 59480, 28032, 5728,  56452, 3204,  23404, 52008, 41436,
                6924,  57772, 3340,  17464, 42788, 7472,  16032, 30100, 1008,
                58240, 17940, 37316, 41408, 23720, 196,   39716, 57708, 63668,
                5428,  38468, 59896, 42168, 22936, 13960, 62428, 33280, 5600,
                42344, 35428, 55268, 23696, 39316, 58180, 52576, 21800, 1428,
                49316, 58632, 11912, 22520, 48536, 9252,  20304, 39568, 60652,
                7880,  58336,
            },
            {
                32528, 48288, 39356, 56684, 16776, 37324, 20316, 8804,  22924,
                53744, 61012, 5000,  29380, 65512, 7864,  58728, 4712,  44868,
                32720, 6848,  39144, 48420, 44308, 22592, 64340, 8188,  59728,
                29060, 44908, 17572, 12764, 53524, 35828, 63824, 45600, 37524,
                29924, 43360, 4180,  13316, 58120, 29136, 5312,  14640, 60076,
                34644, 22404, 2444,  46512, 15888, 63416, 53416, 7536,  26508,
                4516,  51748, 22780, 45616, 53692, 24988, 49832, 39072, 18504,
                58344, 33516, 40940, 13376, 31184, 49740, 3280,  60640, 51704,
                37520, 1064,  16748, 33740, 62708, 38060, 14808, 59744, 19416,
                34952, 12876, 53168, 63996, 27380, 51128, 59492, 22588, 44124,
                11360, 25896, 8164,  55956, 12420, 20380, 32756, 15692, 35620,
                26712, 46008, 50860, 3884,  57004, 44580, 28036, 18536, 58492,
                15592, 61184, 10636, 31836, 2468,  15056, 5476,  31040, 42488,
                18904, 25652, 5904,  63800, 36652, 56052, 4368,  33324, 51408,
                580,   37040,
            },
            {
                13392, 3840,  26396, 62268, 30828, 51804, 60072, 44468, 2596,
                31524, 10980, 19740, 56380, 43108, 23372, 49580, 37668, 11012,
                54844, 29076, 61816, 4284,  19044, 28216, 2980,  14824, 51236,
                11584, 176,   62200, 32260, 47676, 8060,  25092, 14052, 54120,
                19288, 59632, 32472, 39788, 53356, 10780, 46416, 19920, 40832,
                9776,  44028, 61624, 6400,  31148, 37792, 23248, 59664, 44532,
                55876, 29808, 37620, 2292,  40668, 28384, 9208,  21652, 30680,
                14796, 48304, 20964, 59368, 9572,  55780, 16240, 28464, 20560,
                11140, 24768, 41996, 54020, 21420, 45172, 32200, 200,   63600,
                28836, 44416, 36976, 10084, 20496, 2568,  33956, 13504, 36452,
                52436, 61104, 47844, 28828, 64700, 44880, 9732,  48812, 2576,
                55624, 21284, 16640, 31300, 10620, 34960, 2656,  48556, 8872,
                51920, 29032, 20364, 48088, 63252, 36384, 27052, 55656, 62572,
                37624, 54168, 33624, 42012, 14756, 28172, 62336, 47832, 16548,
                22380, 64068,
            },
            {
                9116,  44444, 21572, 35048, 1412,  24196, 40700, 28328, 63984,
                16220, 46092, 39616, 50536, 1364,  12800, 31484, 18628, 63816,
                21532, 15456, 24788, 34164, 55440, 36776, 47600, 43028, 31276,
                38240, 54084, 6304,  42296, 19396, 29572, 58696, 40948, 5356,
                26676, 65008, 7800,  24316, 18096, 63484, 26544, 56124, 3716,
                51032, 28392, 36408, 19308, 10924, 47916, 2884,  12444, 35480,
                15212, 8384,  61528, 11800, 16436, 63804, 35832, 47360, 43140,
                6932,  65476, 944,   27216, 45716, 35156, 63276, 42920, 7896,
                57364, 65132, 48080, 13056, 8544,  27092, 49924, 39492, 54612,
                5304,  22744, 49128, 31108, 57092, 46580, 40968, 65472, 5736,
                19792, 32540, 1968,  15216, 35296, 52284, 25580, 60528, 41164,
                13556, 8040,  64436, 25400, 52244, 61464, 39324, 55112, 24580,
                37816, 45184, 3728,  40896, 51416, 18168, 45040, 7236,  11168,
                15388, 46312, 2516,  59816, 18436, 44936, 24424, 10876, 41484,
                27200, 53272,
            },
            {
                18132, 57404, 50100, 14844, 8268,  55816, 12748, 5204,  49172,
                35200, 58932, 7524,  21908, 33376, 61664, 53148, 26764, 47784,
                904,   43304, 51948, 8452,  12224, 60300, 24120, 57440, 18216,
                63740, 22344, 26524, 37032, 50272, 2604,  16472, 34104, 51552,
                11824, 47040, 35868, 50348, 976,   44100, 37640, 32800, 23616,
                64528, 13868, 54292, 58916, 41460, 52092, 28832, 65328, 49528,
                21096, 42600, 47456, 34232, 57792, 5964,  52376, 3108,  56440,
                24644, 36816, 51164, 42036, 18284, 23536, 1948,  48828, 32572,
                40024, 6428,  29600, 35640, 61752, 18512, 6636,  11792, 25268,
                17112, 58892, 1432,  14872, 38388, 8004,  24588, 49736, 28380,
                56256, 39396, 22840, 42272, 6328,  57204, 4260,  22316, 29916,
                44412, 33348, 37808, 47496, 6568,  15304, 21428, 12328, 32288,
                5084,  65348, 14192, 25840, 8416,  59312, 22860, 34100, 50772,
                29324, 21376, 8840,  31444, 51940, 3412,  55084, 6824,  59076,
                46776, 31004,
            },
            {
                61064, 40236, 28668, 64356, 42528, 46976, 33136, 18572, 38772,
                24868, 13732, 27940, 54412, 36904, 16576, 4028,  40460, 8832,
                56588, 30892, 38048, 65100, 41688, 16860, 1248,  5732,  34872,
                9164,  46248, 14648, 59428, 10552, 63392, 44320, 56072, 22804,
                1928,  42200, 15448, 56536, 31164, 6200,  52264, 16124, 7816,
                39244, 488,   25392, 32644, 22136, 5572,  17340, 39768, 25480,
                708,   58776, 27156, 18568, 23400, 42144, 31740, 13036, 62016,
                17188, 29412, 11736, 61448, 4684,  38684, 13776, 54224, 25628,
                18092, 52684, 22552, 2080,  51220, 57196, 43576, 60924, 47308,
                34320, 41044, 52044, 26624, 62396, 18292, 54716, 10644, 16604,
                3432,  46220, 62480, 50744, 18960, 31544, 38660, 16780, 62516,
                53144, 848,   59100, 18416, 41592, 30116, 46732, 59236, 42796,
                18988, 56772, 34580, 54076, 30356, 12452, 39168, 56628, 340,
                64912, 40856, 49156, 23652, 12140, 38392, 29624, 34656, 14196,
                37972, 2152,
            },
            {
                33448, 11664, 6128,  36736, 20888, 52636, 60844, 9744,  56708,
                51328, 2236,  63368, 42668, 10472, 45808, 24136, 59360, 34668,
                13276, 46348, 3504,  19984, 49844, 29752, 53436, 39732, 27256,
                49492, 56308, 1964,  33264, 53172, 21032, 27680, 8636,  38668,
                62548, 29660, 20340, 10060, 61368, 22296, 12380, 59900, 48116,
                43580, 17860, 49320, 8404,  44624, 60644, 54808, 33540, 9688,
                52560, 31864, 5144,  54416, 49148, 10524, 26300, 20232, 45536,
                39900, 8704,  54404, 32676, 57540, 47204, 30564, 60388, 11884,
                36792, 4432,  46292, 41204, 31824, 14320, 36908, 21076, 30340,
                8616,  64508, 11448, 32812, 43504, 5048,  30420, 45108, 37560,
                58656, 13720, 7972,  26768, 11496, 60000, 47076, 9296,  50172,
                27280, 12068, 23384, 56620, 3588,  26308, 63896, 236,   8380,
                27992, 49328, 22008, 1820,  43680, 61352, 5100,  47520, 25500,
                17400, 54532, 35100, 61988, 57520, 43312, 21188, 65236, 51640,
                5168,  20552,
            },
            {
                24300, 48580, 53656, 3212,  16064, 25656, 644,   29644, 21704,
                43664, 6640,  31160, 19152, 57788, 5944,  50136, 29384, 63016,
                17596, 23196, 58652, 26304, 6576,  33296, 45348, 61968, 12456,
                20224, 30320, 43740, 24440, 39784, 4320,  48352, 31576, 13712,
                49516, 59004, 25548, 45420, 36852, 41524, 27788, 56904, 34572,
                29764, 62252, 55872, 12320, 35996, 1852,  14240, 46736, 19904,
                62024, 40392, 13364, 64644, 1444,  38352, 60608, 53592, 33576,
                1684,  48928, 22600, 15460, 6764,  26772, 20868, 8652,  44360,
                57836, 63396, 15716, 55292, 24492, 4924,  28156, 1672,  53300,
                15852, 45496, 20572, 3484,  59836, 50640, 21872, 60936, 33412,
                25116, 52196, 30760, 36224, 44224, 54104, 3028,  34416, 20440,
                5552,  40604, 35596, 48860, 10240, 54508, 37308, 33300, 52956,
                16388, 61924, 11032, 46972, 37116, 15936, 20424, 32096, 42420,
                13336, 4288,  27424, 7148,  15824, 1036,  48880, 17468, 26220,
                45368, 55996,
            },
            {
                14540, 43376, 27460, 58664, 32240, 39832, 45480, 64844, 34604,
                16824, 59484, 39872, 48948, 27016, 14680, 37968, 20348, 2028,
                41236, 53736, 36340, 10612, 62860, 14336, 21972, 7980,  58400,
                41504, 5208,  65404, 17980, 12032, 61048, 36252, 57468, 19356,
                6800,  34376, 3252,  53408, 5264,  49980, 18780, 2572,  10436,
                21160, 5880,  38120, 26664, 63288, 30704, 23584, 57924, 36948,
                6896,  16628, 45788, 35436, 28600, 44456, 15600, 5672,  58216,
                27572, 64132, 43188, 36040, 52692, 40900, 64736, 51052, 888,
                33616, 28052, 20212, 8792,  59640, 39272, 63812, 49352, 58188,
                5948,  38624, 55824, 24412, 36392, 12412, 40448, 616,   9056,
                20268, 41416, 64544, 1464,  21516, 15096, 28516, 42684, 65280,
                55800, 16332, 60392, 30988, 43332, 14404, 19508, 23756, 45252,
                40360, 35852, 6620,  24360, 64048, 28256, 52648, 58004, 9780,
                59992, 37844, 20192, 47096, 40320, 33208, 60524, 7456,  10520,
                40936, 63240,
            },
            {
                35032, 8588,  17968, 62168, 11092, 50332, 6364,  14060, 54048,
                47628, 9024,  22968, 35592, 328,   65016, 55436, 44176, 11640,
                50616, 5072,  28420, 47324, 42592, 55868, 37828, 25384, 51740,
                16088, 35912, 50024, 7744,  46904, 25928, 15984, 804,   43904,
                54160, 40908, 17592, 62948, 14300, 32280, 65128, 45992, 24724,
                42156, 53112, 15660, 46324, 19116, 51092, 43068, 4504,  26280,
                50048, 30372, 22760, 57464, 19492, 8412,  50216, 23716, 36668,
                18684, 4588,  12688, 59660, 19224, 3708,  32332, 16836, 23900,
                42208, 12824, 49560, 34796, 45020, 11600, 18908, 41480, 23528,
                35080, 27368, 49068, 17328, 57704, 28520, 15472, 63660, 48768,
                55520, 4628,  17220, 48076, 56404, 61372, 37744, 24876, 13084,
                45148, 28780, 8116,  21456, 62980, 4524,  50796, 57616, 3196,
                13068, 59960, 31664, 55192, 40784, 8240,  3000,  35280, 49456,
                29616, 44324, 64000, 11244, 56312, 23892, 28344, 37016, 57844,
                29896, 172,
            },
            {
                22216, 54828, 37944, 44496, 23176, 56400, 20156, 27328, 37480,
                3848,  25816, 62020, 11904, 51568, 33060, 8272,  30608, 25412,
                57752, 33568, 16068, 60208, 1448,  18332, 4336,  47664, 31848,
                596,   60312, 28220, 55308, 34028, 41124, 52108, 64520, 23160,
                27348, 11888, 47684, 29388, 23060, 39892, 7680,  54708, 36376,
                60600, 1204,  32552, 58860, 3560,  9376,  38880, 64476, 12036,
                55368, 41360, 10056, 4116,  52468, 32820, 61696, 42804, 10148,
                47184, 55612, 30492, 24660, 45604, 10616, 48996, 56776, 38524,
                6644,  54188, 61156, 1504,  26320, 52452, 7336,  32624, 14024,
                61344, 10956, 1056,  42328, 6608,  47108, 53400, 35128, 26368,
                43336, 12188, 38496, 23864, 33156, 10248, 6916,  49808, 1048,
                52132, 38644, 3272,  53348, 34460, 27044, 38956, 9452,  30428,
                48468, 21236, 1076,  18020, 50132, 13880, 45112, 23096, 18404,
                1568,  24552, 52720, 31312, 2812,  51440, 44036, 12968, 53040,
                18800, 49688,
            },
            {
                59288, 4292,  29152, 7156,  34052, 2532,  41964, 61532, 11336,
                32416, 41132, 55756, 18160, 46088, 21748, 40720, 16924, 61608,
                45456, 8848,  21276, 39576, 30912, 52268, 35076, 64164, 10728,
                39348, 23512, 13536, 19856, 58768, 4804,  10140, 30080, 38372,
                59760, 8360,  56280, 36512, 72,    58912, 26980, 16988, 13280,
                50468, 28632, 11280, 41064, 22516, 56844, 29124, 17880, 34236,
                932,   59944, 47592, 62620, 25544, 40196, 3316,  29776, 14500,
                52096, 39596, 34608, 0,     62064, 36832, 27208, 14144, 62996,
                29544, 46524, 22736, 37788, 16712, 56500, 29328, 47372, 4072,
                43784, 54364, 30264, 65004, 33160, 10120, 22756, 2720,  18396,
                31892, 58084, 28164, 5884,  52660, 41612, 19476, 58828, 32592,
                22888, 61972, 18156, 47848, 12408, 45736, 59448, 17680, 64956,
                25964, 53676, 43248, 58184, 34140, 26736, 60596, 38160, 62828,
                55820, 7648,  14692, 36208, 18072, 6176,  21712, 64208, 3908,
                32656, 39480,
            },
            {
                62908, 13588, 47992, 52276, 63828, 15412, 31124, 49136, 58444,
                52232, 15752, 43840, 2868,  28388, 59188, 5332,  53012, 3132,
                37452, 13736, 64696, 54316, 24004, 12388, 27276, 57280, 20860,
                46176, 53740, 42988, 2928,  31696, 45720, 21128, 13456, 50340,
                2352,  33236, 20768, 44352, 51484, 10300, 48404, 33948, 4796,
                44232, 20080, 63540, 52256, 35164, 48808, 14136, 46096, 53344,
                20448, 27924, 14432, 35992, 17444, 11944, 56964, 21332, 63400,
                26156, 7040,  16536, 57660, 50788, 7956,  21732, 43884, 3352,
                18308, 10000, 31776, 5212,  65412, 42904, 21344, 62760, 50940,
                16356, 21764, 39556, 51412, 19340, 58716, 38656, 45856, 61784,
                7368,  51040, 60752, 44600, 13760, 63408, 30028, 46832, 15796,
                36516, 10696, 56372, 41308, 24284, 5608,  32420, 42428, 6996,
                37344, 15028, 28808, 5004,  10516, 20276, 54308, 5464,  12076,
                33084, 47840, 39388, 58544, 61896, 48752, 41376, 15528, 47036,
                24536, 9800,
            },
            {
                17132, 41572, 25700, 19412, 36684, 46336, 8520,  24248, 872,
                21468, 6488,  30280, 63256, 38448, 14048, 47732, 34788, 23152,
                49224, 27620, 42924, 5584,  49800, 8156,  44160, 16368, 6184,
                30204, 62580, 9408,  37876, 63932, 25040, 54924, 34808, 46808,
                16920, 63604, 25792, 13844, 62264, 18648, 41648, 57368, 61880,
                23920, 39152, 7080,  17072, 25904, 5416,  60440, 7984,  24664,
                43448, 38600, 6956,  31580, 45320, 54368, 48440, 37948, 2208,
                59824, 44888, 22944, 11840, 29084, 39996, 54452, 33472, 59208,
                51352, 40636, 58020, 48836, 12728, 34504, 2188,  10136, 37424,
                26240, 59204, 8248,  13452, 25132, 4708,  29536, 55104, 11100,
                36608, 15948, 21156, 256,   35612, 26088, 3880,  55080, 8592,
                44336, 27524, 31224, 1308,  63980, 15548, 55836, 22144, 52260,
                11440, 61612, 46476, 39572, 64380, 47632, 30976, 41128, 16652,
                43164, 27112, 20352, 9628,  25772, 29820, 33968, 8232,  55664,
                27812, 44700,
            },
            {
                53276, 32008, 1616,  58124, 11548, 28088, 55160, 18328, 39392,
                45688, 60064, 36308, 8680,  53792, 26040, 10760, 63640, 18296,
                56192, 1952,  32876, 19216, 58312, 36988, 61992, 40956, 51252,
                2264,  25720, 18232, 48940, 15148, 6348,  42556, 61208, 4204,
                40736, 53264, 6440,  38680, 4384,  29864, 22224, 2636,  31196,
                9532,  47828, 56524, 32004, 2332,  42444, 37512, 31340, 63168,
                3380,  57564, 51324, 64088, 1184,  23628, 8200,  34064, 19020,
                31408, 49424, 41644, 63768, 19776, 47232, 1640,  12376, 25576,
                35708, 21084, 15404, 27604, 53964, 24708, 40336, 18716, 57252,
                33572, 2808,  47996, 36508, 45288, 63200, 41392, 14768, 49288,
                25408, 40576, 31232, 56752, 49592, 17724, 39092, 22004, 64468,
                4600,  50888, 60660, 20240, 35540, 49440, 28660, 2004,  47588,
                33904, 3748,  23824, 17288, 35976, 7868,  24804, 52380, 3764,
                49912, 63308, 372,   45000, 52304, 3500,  57712, 37884, 61504,
                2400,  36024,
            },
            {
                6196,  56464, 21900, 39108, 60968, 3752,  42772, 64928, 34308,
                26452, 11728, 48428, 23828, 19304, 57724, 43652, 31492, 7664,
                40944, 59892, 11476, 46160, 28956, 1096,  22740, 32204, 13044,
                36280, 59056, 33328, 52940, 28748, 57208, 18792, 9512,  27480,
                22508, 30992, 48672, 57732, 34948, 54520, 46796, 14984, 37552,
                54072, 27532, 13424, 45096, 65272, 53028, 19908, 11088, 49200,
                15536, 23008, 10304, 19528, 41516, 27472, 59000, 44008, 13784,
                9708,  55352, 4020,  33284, 6240,  57000, 17184, 64848, 46196,
                5664,  61900, 1276,  43996, 8640,  60424, 51508, 30160, 44508,
                6968,  62140, 28992, 54656, 16968, 712,   33772, 21924, 5604,
                64876, 2992,  46356, 8688,  62380, 11376, 51972, 33368, 41088,
                25192, 12848, 38404, 7124,  54212, 9716,  38720, 63260, 19252,
                41424, 30592, 59284, 51488, 856,   56744, 13520, 59720, 29508,
                22432, 56480, 35396, 13728, 40544, 17244, 11652, 22660, 19264,
                50476, 12520,
            },
            {
                65484, 45988, 9488,  49644, 16512, 32764, 51976, 13724, 4856,
                56180, 17272, 51076, 33288, 1440,  40116, 4784,  28128, 51536,
                15564, 24996, 35716, 62648, 14776, 48684, 54464, 17976, 46572,
                55580, 7432,  44576, 11596, 292,   39600, 31948, 51288, 36700,
                59548, 14628, 9860,  19696, 43596, 12084, 26200, 64372, 49940,
                18236, 816,   59952, 36540, 23340, 16092, 58684, 27652, 40092,
                33964, 55000, 29408, 37108, 50248, 16212, 4444,  65316, 52652,
                38868, 27060, 15520, 37060, 51456, 24140, 31272, 38316, 9444,
                28976, 55400, 48144, 32900, 38268, 4616,  15736, 47056, 12244,
                23724, 40964, 20556, 10884, 60360, 27396, 51908, 57340, 44056,
                28584, 53888, 18852, 23988, 42708, 29044, 1984,  60024, 15196,
                48552, 57556, 17408, 46112, 26364, 43196, 13212, 57432, 25336,
                7932,  54852, 12604, 21380, 40740, 32920, 45316, 19092, 37692,
                10916, 6868,  31056, 59948, 23700, 64568, 46060, 53948, 43100,
                30768, 23980,
            },
            {
                18124, 28728, 35096, 43384, 26128, 7292,  21184, 47980, 28820,
                61768, 38312, 6920,  64444, 14604, 46992, 61180, 21136, 37152,
                47340, 53540, 3932,  21584, 39692, 8460,  26808, 4136,  60464,
                23736, 39128, 20640, 61540, 47480, 23364, 62956, 13012, 2136,
                45880, 41576, 65492, 28000, 968,   60248, 7840,  40680, 5668,
                32816, 21272, 41748, 10320, 29836, 7208,  50612, 140,   44172,
                62060, 5112,  46296, 60708, 12616, 55908, 35536, 29672, 24180,
                972,   61396, 21364, 45592, 59324, 13948, 42872, 52948, 19180,
                41412, 13256, 23848, 17952, 64196, 22200, 55892, 252,   63632,
                52860, 14976, 49820, 34544, 42760, 8008,  19628, 12540, 35768,
                9760,  58628, 33108, 38200, 14716, 47556, 56024, 6404,  20616,
                30352, 3116,  34160, 59496, 22356, 32176, 4884,  17076, 35936,
                61120, 44400, 27576, 48912, 9660,  61440, 27988, 2204,  65352,
                42288, 53520, 18632, 48024, 4564,  33292, 28640, 924,   7564,
                60052, 40836,
            },
            {
                4532,  14248, 59352, 344,   63072, 54380, 37200, 41304, 10156,
                2856,  44248, 20548, 29648, 52908, 35192, 55672, 12256, 40,
                65488, 9864,  29960, 43712, 56384, 32700, 63912, 41828, 9952,
                29372, 14884, 3668,  26972, 34936, 16288, 43008, 55388, 21092,
                7408,  53664, 24276, 32952, 50896, 17176, 35460, 23184, 57128,
                45664, 62920, 51612, 4536,  55880, 46200, 32948, 13688, 21808,
                9000,  17740, 25584, 1764,  32220, 8796,  47116, 20336, 50696,
                42900, 56516, 30536, 10488, 3284,  27748, 7256,  61036, 2528,
                57416, 36472, 51144, 10608, 29556, 42780, 35792, 26348, 32312,
                37740, 5012,  57688, 2476,  24228, 47000, 31508, 62560, 39532,
                16844, 48156, 4272,  60368, 7008,  35356, 26244, 44584, 37348,
                54628, 42140, 10912, 51356, 160,   62224, 53372, 45884, 50112,
                1524,  15620, 38116, 4304,  54064, 23608, 16000, 51044, 34076,
                46620, 15184, 26852, 38172, 55576, 9244,  50156, 37292, 15856,
                34032, 48776,
            },
            {
                55288, 38180, 50620, 23640, 12092, 30604, 18056, 59084, 25004,
                32304, 54192, 12104, 42384, 25836, 9196,  17472, 24280, 44620,
                32972, 19052, 58876, 6164,  50552, 12240, 20128, 52656, 34144,
                49340, 65192, 41356, 52448, 57972, 5180,  8708,  48184, 26728,
                34072, 16208, 3636,  38396, 56204, 45116, 30044, 52624, 10772,
                14468, 25608, 34708, 17540, 38812, 24860, 61456, 56916, 35744,
                48688, 53728, 38544, 43232, 22424, 62352, 40352, 5504,  12088,
                17612, 7644,  47896, 40424, 64256, 35636, 49992, 21708, 33908,
                26260, 62440, 6308,  45296, 54204, 7396,  19728, 58744, 9656,
                17796, 46128, 30404, 65256, 38580, 54912, 5408,  50184, 1232,
                22560, 27160, 12804, 52252, 21916, 64604, 19040, 12248, 61760,
                8472,  23572, 65176, 29208, 14936, 39584, 20300, 28272, 10196,
                23416, 32504, 64792, 18944, 34868, 43128, 6220,  57624, 9156,
                24508, 5172,  62244, 12048, 42736, 21120, 25464, 63420, 56852,
                10724, 26356,
            },
        },
        {
            {
                62816, 35304, 21968, 15908, 36956, 25396, 59224, 28616, 48904,
                53708, 31604, 41612, 46824, 7920,  40088, 62308, 53476, 11340,
                18116, 21940, 29092, 10140, 39820, 62888, 5560,  49776, 38132,
                14792, 24876, 48364, 11336, 51528, 41896, 46832, 31972, 15092,
                4972,  51340, 2700,  63352, 12156, 26264, 37860, 44164, 33328,
                20724, 6784,  12872, 25480, 32780, 56328, 28472, 45864, 3808,
                54080, 49272, 60116, 36116, 46252, 63920, 39832, 14956, 57620,
                27728, 22248, 39460, 5032,  14336, 59332, 11184, 55584, 28136,
                21208, 37988, 10612, 14940, 53300, 5952,  37728, 62752, 25624,
                41628, 19184, 37544, 57684, 41016, 34308, 52352, 536,   48388,
                21824, 30384, 57756, 37184, 16580, 63924, 28480, 12104, 49780,
                1496,  29508, 36372, 3580,  41064, 16408, 34052, 51388, 19064,
                25900, 40680, 7280,  22112, 30228, 11496, 43812, 23480, 308,
                29856, 41536, 46220, 20592, 55592, 62516, 23092, 27304, 33296,
                59896, 18340,
            },
            {
                6832,  49792, 27640, 47184, 42756, 2780,  34268, 5824,  14724,
                36200, 2624,  17140, 52588, 29512, 1876,  22972, 35196, 6576,
                59600, 44080, 48572, 3948,  36388, 23956, 57392, 33608, 64324,
                28812, 7364,  35184, 21688, 29864, 280,   27012, 10524, 64716,
                44136, 23776, 57024, 32204, 21060, 53172, 59064, 4800,  50440,
                60096, 40656, 47660, 58264, 5528,  22236, 50264, 35816, 58572,
                26820, 8940,  41576, 12440, 17660, 53432, 10212, 49380, 6324,
                36616, 12044, 54816, 19584, 31092, 41408, 23120, 16908, 2312,
                45444, 62020, 25764, 48252, 31824, 28356, 17592, 13484, 33100,
                2272,  59732, 48736, 9760,  5468,  64028, 19512, 44776, 15904,
                40140, 2500,  13404, 27356, 8452,  58892, 40244, 24584, 33220,
                57984, 20560, 52008, 12920, 60996, 55192, 1948,  42480, 9108,
                58184, 65060, 14372, 45060, 60808, 39028, 16596, 47152, 58688,
                36068, 15748, 60220, 4752,  34468, 9116,  42132, 58120, 5336,
                50928, 39892,
            },
            {
                31288, 2196,  58936, 8232,  63816, 18880, 56356, 45752, 62324,
                20392, 58364, 26408, 65276, 13600, 49948, 43528, 30928, 26068,
                56160, 1448,  33044, 61780, 16516, 46596, 8840,  21040, 44884,
                18212, 54040, 59292, 39120, 16428, 62620, 36480, 53932, 39456,
                18192, 34684, 9432,  49192, 40924, 396,   30372, 18860, 10672,
                27428, 37092, 844,   52028, 18364, 43028, 2496,  62144, 16972,
                31584, 19932, 2124,  56740, 25768, 3660,  32892, 24108, 42172,
                62892, 1048,  47820, 43804, 63348, 3688,  47452, 35288, 51800,
                40420, 18324, 8008,  58992, 612,   43856, 60840, 50188, 4884,
                44736, 52960, 23400, 16520, 46904, 25660, 11152, 56140, 7824,
                33272, 54704, 61904, 46452, 22524, 50768, 18768, 7052,  46056,
                16120, 41908, 25616, 48204, 8712,  37588, 29948, 46420, 22480,
                33416, 48356, 28080, 53856, 2608,  51580, 26152, 32568, 5976,
                52976, 8772,  26772, 38516, 18068, 52632, 12360, 47856, 21448,
                15580, 55728,
            },
            {
                11532, 19792, 39232, 32384, 51144, 12188, 30284, 23572, 39440,
                10688, 43380, 6304,  33304, 38344, 9748,  18804, 51884, 15412,
                38868, 12420, 51220, 26560, 55388, 30644, 2300,  40760, 11600,
                50704, 1652,  42824, 9548,  56992, 4780,  49616, 22412, 6816,
                47836, 29308, 62372, 15940, 7432,  45844, 65140, 14172, 42640,
                62288, 22484, 15720, 30796, 64528, 39548, 13968, 10328, 44384,
                38088, 64456, 51016, 29960, 43616, 37056, 59104, 20280, 52004,
                30524, 16600, 34688, 8072,  27608, 53948, 9332,  61416, 30308,
                12708, 54304, 33472, 41616, 20512, 11160, 56292, 39400, 21376,
                65340, 35180, 29288, 1368,  32376, 58576, 37104, 28540, 63196,
                49784, 19132, 10308, 31816, 43368, 144,   35868, 55872, 65404,
                10744, 5396,  62808, 32052, 18620, 24088, 64016, 13828, 52948,
                5596,  17004, 37560, 10716, 34852, 5096,  56876, 13396, 21380,
                62436, 43240, 50168, 2468,  64296, 29192, 44232, 584,   36656,
                28352, 45112,
            },
            {
                25220, 56976, 14600, 22780, 4376,  41208, 53944, 588,   48436,
                28196, 50788, 55976, 21808, 46140, 61228, 57292, 5240,  63604,
                46204, 23512, 19420, 7224,  42536, 14096, 53316, 61048, 25708,
                32584, 62056, 23036, 31620, 45920, 19844, 12620, 26260, 60928,
                1272,  13316, 52936, 38364, 27844, 22736, 54664, 32692, 48292,
                3364,  56108, 34972, 9592,  26952, 53828, 33768, 25384, 55460,
                6572,  47332, 14400, 21628, 60952, 7956,  46456, 13144, 5092,
                56488, 25532, 59984, 50284, 15004, 20796, 38684, 24592, 6164,
                57964, 3104,  27664, 64552, 49496, 35620, 24524, 7680,  27892,
                10272, 14000, 40104, 61724, 43252, 50948, 3552,  23192, 42240,
                4596,  25120, 38004, 52760, 6144,  61140, 14024, 30332, 39316,
                27116, 35360, 53444, 700,   44160, 56580, 4416,  27376, 59824,
                39932, 468,   57448, 24280, 20296, 42016, 64488, 49028, 39816,
                28832, 11624, 23560, 46700, 32492, 15292, 25072, 61304, 54084,
                8740,  63556,
            },
            {
                33992, 52544, 43544, 60980, 35484, 26516, 16008, 64476, 8352,
                18440, 35340, 3812,  16048, 25252, 268,   36248, 27776, 41288,
                9176,  34428, 57728, 38196, 64648, 22304, 35508, 47540, 3868,
                37576, 13904, 6180,  27200, 52436, 34908, 59180, 41104, 32116,
                56464, 42708, 25064, 4560,  57668, 35124, 9796,  39076, 25332,
                7700,  50760, 44604, 59524, 4040,  46488, 19576, 60852, 100,
                22576, 34324, 10556, 39768, 54196, 16696, 28120, 64748, 39080,
                45072, 10292, 22616, 40516, 1928,  34008, 56164, 42980, 15928,
                47956, 36888, 22684, 13608, 4968,  16384, 53620, 31508, 47364,
                58056, 18800, 54860, 7336,  21016, 13112, 17636, 60212, 35336,
                14072, 45176, 58480, 16044, 26668, 41412, 53960, 20360, 3000,
                47816, 59200, 21884, 14236, 38852, 50544, 10856, 36272, 20672,
                44488, 31428, 50240, 61420, 46428, 29888, 8864,  18336, 1352,
                33872, 58140, 19568, 55280, 7276,  58604, 35272, 18928, 3908,
                48972, 41388,
            },
            {
                5860,  1656,  29332, 9868,  49196, 58300, 45192, 37596, 32004,
                59020, 13044, 63324, 41748, 53412, 31656, 12716, 20868, 49288,
                3924,  60344, 28588, 676,   49492, 10548, 28116, 16628, 57604,
                20148, 55296, 48724, 63544, 15460, 1872,  44376, 8136,  16028,
                35872, 20572, 46068, 60740, 17736, 50392, 1764,  59640, 16604,
                29328, 20008, 12280, 38472, 23396, 7476,  49612, 30324, 40248,
                52292, 27564, 62304, 4976,  32228, 820,   50128, 33824, 18908,
                2512,  36072, 52908, 29868, 65244, 46368, 11668, 28940, 63528,
                19984, 44428, 51596, 60352, 38756, 45828, 1540,  63104, 41832,
                3072,  50428, 26388, 46288, 38268, 30304, 53364, 47572, 9264,
                31416, 62312, 2924,  21176, 34396, 47224, 9604,  24296, 51128,
                17036, 42688, 7416,  28840, 61092, 33200, 16860, 47848, 7656,
                63032, 12096, 18556, 7004,  14972, 55068, 37880, 25572, 45636,
                53720, 4608,  37108, 13860, 41152, 51192, 10156, 39580, 31540,
                23172, 17072,
            },
            {
                38104, 47480, 65240, 18204, 6568,  21008, 11776, 4056,  22348,
                53012, 39180, 29536, 47788, 7900,  59736, 45052, 55220, 15956,
                31048, 52800, 13564, 18140, 44728, 5256,  60084, 40052, 8432,
                44320, 29032, 41468, 10404, 38252, 24192, 55624, 28516, 65004,
                51852, 3092,  10168, 29848, 12948, 42476, 21648, 45100, 63580,
                52400, 41768, 31688, 63152, 16704, 57572, 42364, 12464, 15728,
                58612, 45164, 18236, 49264, 24780, 41948, 55964, 8552,  26584,
                48428, 62260, 12928, 6616,  58044, 18520, 5016,  50984, 96,
                32596, 10324, 6900,  30812, 25896, 57316, 19604, 34328, 15148,
                23528, 36604, 4664,  64196, 10632, 56864, 640,   39788, 21764,
                55084, 28184, 50176, 11732, 56048, 63740, 4848,  33024, 61664,
                12056, 31396, 55612, 46268, 19772, 2664,  42236, 58412, 29644,
                25052, 54444, 42872, 27188, 33744, 2504,  59240, 11248, 61560,
                16132, 42452, 63812, 30532, 1744,  22064, 45680, 64612, 56664,
                11328, 60224,
            },
            {
                20844, 13984, 26016, 53728, 39880, 33456, 55428, 61388, 25360,
                44132, 1416,  19876, 10864, 34820, 23464, 3252,  37924, 64976,
                24928, 47088, 40856, 35028, 56960, 32176, 51624, 24200, 34476,
                64508, 4764,  17572, 33556, 60520, 4124,  47496, 18732, 11456,
                23704, 48692, 37376, 55332, 64032, 31916, 5648,  36424, 11284,
                4856,  56680, 1184,  47148, 35632, 28464, 3536,  65096, 36844,
                8412,  3076,  56504, 36252, 12012, 63480, 22088, 14528, 59592,
                40904, 21740, 16996, 43952, 32400, 24060, 36620, 59328, 26632,
                41176, 61816, 17992, 55608, 42652, 9064,  29196, 51212, 11476,
                60856, 44664, 32816, 16432, 22632, 34496, 27272, 6432,  65180,
                17400, 43004, 8036,  38652, 30008, 17756, 40396, 44916, 56980,
                37688, 1900,  25208, 52000, 9500,  65008, 23556, 53072, 14692,
                34928, 1908,  38476, 63952, 49336, 41188, 22456, 32284, 49908,
                28020, 7884,  24564, 47764, 60716, 27316, 6264,  14800, 28568,
                43900, 51808,
            },
            {
                32188, 58020, 35796, 45460, 1016,  29956, 42284, 49544, 14816,
                7328,  51208, 57660, 27312, 62776, 50212, 18400, 42420, 10448,
                7160,  21488, 2804,  62924, 25920, 19940, 14504, 1720,  48100,
                12076, 52344, 25948, 54560, 21256, 50444, 30340, 39828, 5916,
                61764, 32484, 41380, 7480,  19784, 25228, 47740, 57832, 26976,
                34224, 22660, 14564, 25996, 54764, 10064, 50932, 22172, 32056,
                26144, 42556, 20776, 29236, 7084,  44608, 37820, 51360, 30096,
                3728,  56948, 28200, 49940, 9372,  41988, 54752, 14240, 48576,
                23028, 53180, 34628, 1940,  13136, 21868, 59372, 5852,  40416,
                55272, 28048, 8684,  58240, 48328, 62032, 41656, 51452, 12460,
                36760, 1916,  46728, 23300, 59708, 892,   25868, 15120, 8112,
                22380, 63672, 40532, 13504, 36396, 27896, 6656,  39412, 3884,
                50960, 60564, 10440, 20872, 4656,  52928, 14016, 5640,  36476,
                20388, 52308, 10724, 34652, 17388, 53528, 37856, 49620, 904,
                24480, 7800,
            },
            {
                41184, 2900,  10432, 22764, 63120, 16944, 8704,  27996, 35608,
                63592, 32180, 16316, 40916, 5384,  14260, 29160, 58080, 32912,
                60924, 36904, 54392, 11820, 8268,  42148, 55548, 61352, 30416,
                22160, 37136, 784,   45676, 8464,  12660, 58564, 34548, 43888,
                54140, 13992, 26920, 2428,  51324, 38492, 14760, 53400, 17656,
                40944, 48772, 62368, 6740,  19520, 44176, 38708, 17308, 55168,
                46876, 62988, 14344, 52124, 58872, 17264, 2204,  34588, 11220,
                46004, 6388,  38168, 53680, 1396,  63008, 20556, 7192,  39228,
                3980,  15452, 46400, 38276, 50172, 65040, 36012, 47896, 17108,
                1080,  20188, 52452, 38968, 2688,  14476, 19388, 45736, 24252,
                32948, 57464, 52268, 13800, 35684, 53468, 47932, 28864, 50328,
                34760, 5424,  18152, 59776, 49472, 32264, 57220, 44780, 18368,
                22356, 46516, 31796, 16420, 45248, 25312, 57140, 65420, 43500,
                120,   59320, 40124, 56240, 3300,  42664, 20468, 33096, 55528,
                36384, 62868,
            },
            {
                18600, 27796, 50232, 56420, 13284, 47736, 60292, 19644, 3204,
                38300, 46608, 21876, 54956, 36804, 44468, 53676, 1620,  26544,
                50884, 17556, 45200, 28392, 49732, 38012, 6056,  45096, 16308,
                41076, 56620, 62044, 31984, 42716, 63080, 16228, 25644, 152,
                20840, 59792, 16932, 57072, 43444, 61348, 8348,  792,   30492,
                59268, 9440,  38016, 52020, 32544, 59032, 1996,  61244, 6228,
                11256, 34128, 1052,  39224, 31628, 47328, 25620, 61528, 54708,
                18720, 64428, 34976, 25496, 15796, 46548, 30736, 34300, 11876,
                64192, 30012, 58436, 24196, 28304, 4196,  43688, 32224, 24324,
                61792, 30828, 13192, 42972, 25484, 32028, 7984,  54524, 3928,
                60060, 26764, 18868, 41880, 6212,  64828, 10464, 19552, 61020,
                54100, 42448, 26732, 45532, 528,   21156, 10732, 61992, 30348,
                8156,  55576, 36676, 58900, 29004, 8628,  38816, 18032, 29840,
                46848, 15356, 31832, 23128, 12760, 63228, 9284,  59212, 16936,
                12612, 46788,
            },
            {
                5316,  59844, 42820, 31020, 37480, 5880,  24364, 54276, 58760,
                12840, 9556,  25972, 2288,  60824, 8828,  20300, 48256, 39336,
                14728, 5668,  31204, 59300, 22896, 64760, 33976, 27536, 9776,
                50276, 6860,  13756, 18844, 27452, 37724, 5164,  47024, 50840,
                8996,  36264, 46160, 29412, 11784, 33072, 23752, 65520, 45620,
                20900, 4348,  27404, 42704, 12192, 24336, 48160, 29548, 41240,
                24984, 49148, 60208, 22532, 9996,  53572, 5684,  20984, 41704,
                31732, 13744, 22288, 60420, 11004, 40180, 26912, 59976, 55368,
                43940, 19760, 7424,  10708, 53912, 18960, 13948, 8068,  51680,
                38112, 46028, 64004, 5932,  56204, 50148, 60968, 29652, 39308,
                15528, 9156,  62060, 31284, 21952, 44008, 33736, 39320, 2308,
                12800, 30816, 8584,  56636, 16192, 38100, 53700, 13920, 41404,
                49024, 24236, 13008, 764,   62700, 34568, 48164, 11928, 55724,
                25748, 61276, 5412,  50028, 45608, 29172, 25124, 47944, 3636,
                29788, 54048,
            },
            {
                38360, 15056, 20580, 3528,  52832, 34228, 44480, 40388, 30676,
                51716, 43080, 65352, 48796, 31044, 24600, 34208, 11908, 64264,
                22492, 56912, 42500, 156,   13492, 17944, 3676,  54208, 20476,
                60180, 34932, 24188, 48992, 2776,  53296, 22312, 56908, 38908,
                31016, 64568, 22876, 6044,  49340, 18832, 36032, 41820, 12368,
                50252, 35568, 55988, 17564, 64576, 5108,  14844, 35208, 19828,
                54120, 16484, 4760,  27580, 42984, 65524, 14772, 28604, 49408,
                8756,  44544, 2808,  47868, 57512, 4320,  50608, 17236, 920,
                25140, 48984, 32528, 62268, 45360, 39556, 57916, 26908, 55800,
                3920,  10344, 23044, 34824, 17780, 37436, 11212, 20992, 43668,
                48448, 34968, 4876,  49852, 12196, 57820, 4308,  24880, 58680,
                44668, 21440, 48740, 33764, 64100, 24976, 6280,  35244, 26936,
                64804, 5492,  39528, 43348, 19904, 51116, 5936,  22888, 2736,
                35520, 9152,  38620, 19384, 58148, 35372, 6444,  38828, 65016,
                44044, 23644,
            },
            {
                48712, 61620, 9392,  25340, 64836, 17596, 11192, 780,   21588,
                16076, 4860,  34604, 18308, 13268, 59048, 41700, 55560, 4116,
                46464, 35628, 10048, 52568, 40256, 48128, 24620, 36784, 46032,
                2280,  29124, 57640, 40796, 65336, 33752, 10740, 28396, 18060,
                12848, 53048, 1960,  39712, 58740, 52680, 3644,  55440, 25832,
                15416, 61680, 1440,  31012, 46308, 37268, 57356, 51176, 8608,
                64068, 37664, 45544, 57176, 18528, 35396, 39836, 58764, 852,
                56416, 38964, 52232, 29240, 18848, 37080, 43312, 9124,  52536,
                36156, 40956, 3576,  16224, 34892, 728,   63780, 20996, 34508,
                15740, 49832, 29096, 58676, 496,   47064, 26956, 63240, 2424,
                56408, 23496, 40724, 27552, 55148, 17256, 30188, 49316, 15008,
                37048, 62080, 3024,  11856, 40684, 51572, 47096, 60252, 2244,
                17284, 51764, 57184, 27444, 15076, 54904, 31456, 40576, 62400,
                52860, 42896, 27792, 54124, 1476,  14884, 41688, 51920, 21056,
                10756, 34088,
            },
            {
                7000,  32044, 55128, 40036, 47180, 28768, 57348, 50084, 62300,
                27000, 56456, 39448, 52672, 6544,  46040, 932,   17688, 30424,
                25056, 60456, 19428, 33376, 61864, 29884, 58028, 11644, 63844,
                39376, 17428, 11120, 44372, 7580,  15432, 59236, 45020, 62636,
                6948,  42636, 32644, 26204, 15768, 10112, 28900, 59952, 6708,
                32292, 40120, 23864, 10424, 52728, 21580, 27644, 304,   43728,
                30240, 13140, 32756, 8104,  50196, 3472,  11924, 32524, 23420,
                17188, 26508, 63392, 33680, 5732,  24376, 65148, 21972, 13708,
                28504, 56624, 60464, 22768, 49764, 30152, 11572, 47428, 42608,
                60560, 19764, 40392, 44640, 14688, 53240, 6520,  33028, 13292,
                18320, 51876, 64332, 200,   37252, 46584, 62916, 9420,  52700,
                27868, 17716, 54952, 23276, 29104, 4024,  19404, 31340, 44284,
                21680, 33632, 11216, 47008, 4372,  25100, 58552, 44744, 13596,
                17268, 21616, 64724, 11200, 32804, 62124, 18424, 55680, 26980,
                332,   58180,
            },
            {
                17976, 28056, 44948, 1520,  13524, 22944, 8028,  33192, 37164,
                47568, 8884,  29448, 23160, 62704, 36692, 27268, 52312, 38224,
                7940,  49956, 14380, 26700, 4384,  8372,  43032, 15220, 31836,
                52612, 5048,  54992, 21484, 31168, 51028, 40240, 1144,  24528,
                48508, 56104, 20024, 61552, 47636, 43792, 21552, 37472, 46804,
                19436, 54076, 44500, 60124, 7376,  41788, 62980, 11472, 23588,
                59572, 2652,  52496, 25260, 61012, 21840, 47140, 53092, 60620,
                36868, 12596, 7548,  42568, 14992, 56024, 30632, 47040, 61372,
                6084,  18428, 12020, 42900, 8384,  52924, 24796, 6172,  31040,
                2012,  53892, 8436,  25000, 64864, 21556, 42116, 57944, 36588,
                45800, 10220, 31904, 15368, 20364, 6864,  40272, 22408, 1508,
                41652, 5924,  35788, 59412, 43300, 15420, 9948,  57676, 38284,
                8032,  59632, 29312, 36224, 63292, 9456,  18252, 33916, 7468,
                49156, 30584, 3992,  44976, 24288, 48524, 8208,  31052, 13264,
                37140, 51024,
            },
            {
                22344, 11692, 63400, 35744, 52184, 60012, 43476, 3852,  14452,
                19528, 59484, 1936,  42736, 14944, 21052, 57652, 11236, 63376,
                43508, 5188,  54296, 45048, 21888, 55884, 50436, 1008,  22792,
                26604, 47144, 61564, 36884, 25716, 18656, 5536,  35480, 30204,
                14408, 37020, 9272,  3988,  34572, 63516, 364,   51348, 11136,
                64444, 2888,  14008, 29344, 19040, 34036, 16328, 49632, 38648,
                55408, 20744, 41528, 36412, 15052, 29896, 41992, 9388,  44276,
                4644,  50652, 21240, 59576, 53496, 11292, 39428, 2168,  33828,
                45196, 38520, 54328, 27324, 36672, 59604, 17920, 56548, 37112,
                13320, 62544, 33116, 3788,  37876, 30564, 9488,  50352, 24696,
                28608, 5856,  60176, 43064, 53456, 26352, 34728, 56296, 31260,
                65400, 47272, 50764, 7612,  32236, 63380, 54320, 26364, 48864,
                13684, 53600, 1800,  42264, 20748, 39164, 48336, 28484, 580,
                59276, 51612, 35920, 15872, 57704, 38152, 4740,  60228, 45772,
                62564, 42048,
            },
            {
                53628, 38904, 4676,  16552, 20196, 30036, 38408, 63840, 25472,
                45132, 11780, 54104, 33356, 49892, 8476,  31368, 47556, 16020,
                21428, 28712, 36024, 65248, 38740, 16760, 34888, 62784, 41780,
                7976,  33452, 13900, 1560,  58556, 45900, 61920, 54840, 21316,
                51924, 60408, 28012, 40852, 13168, 24352, 30892, 16648, 41160,
                27068, 34492, 48964, 38928, 4784,  56144, 46656, 30972, 5956,
                27020, 17144, 48064, 5472,  54972, 63020, 1792,  19176, 27288,
                62536, 31432, 47256, 25404, 35280, 4220,  49872, 20056, 57932,
                24672, 31424, 1412,  63172, 15440, 4152,  44660, 40616, 21928,
                51032, 27064, 46312, 11972, 48988, 60132, 16148, 1780,  62380,
                55600, 38460, 21976, 48284, 11252, 61700, 13912, 45316, 19108,
                10692, 24992, 13556, 20408, 45820, 37204, 22740, 288,   41180,
                64308, 24112, 16768, 50836, 60612, 3480,  53924, 64096, 37488,
                25256, 41096, 8668,  53332, 27300, 42772, 19748, 34924, 24052,
                15576, 2472,
            },
            {
                32312, 58904, 26184, 56560, 49440, 6068,  10788, 55112, 50844,
                31896, 40364, 64608, 26172, 3108,  44600, 60736, 4480,  34264,
                41364, 57244, 12328, 2536,  47276, 9980,  28288, 19712, 48744,
                57412, 17824, 42816, 51828, 9636,  28100, 12664, 41732, 8320,
                2796,  44076, 17340, 57636, 53328, 48748, 7584,  58944, 55288,
                22388, 8532,  58320, 51520, 25924, 60836, 3064,  13692, 61760,
                44352, 10076, 64636, 32140, 11388, 38136, 24648, 57812, 35076,
                14696, 40300, 20,    17296, 45592, 28172, 62096, 15336, 7816,
                52024, 10428, 48800, 21320, 46724, 33440, 28296, 65372, 9304,
                16476, 57848, 39324, 18988, 56460, 27492, 35332, 20236, 40528,
                14704, 3176,  34060, 58376, 29976, 2268,  51412, 5180,  59652,
                39212, 34160, 57204, 27684, 3360,  52044, 12208, 34484, 17884,
                30540, 6120,  45188, 32300, 26548, 13936, 23208, 10136, 45840,
                19468, 12924, 60772, 22128, 1032,  11864, 49748, 54160, 9720,
                28996, 48384,
            },
            {
                18712, 43204, 9012,  45816, 34528, 23660, 42036, 15944, 284,
                21340, 6660,  16560, 36916, 56784, 19208, 39152, 23688, 51804,
                524,   61096, 18612, 25468, 31564, 59008, 53260, 3648,  38260,
                11712, 28868, 64776, 23592, 39060, 49608, 33512, 16728, 64380,
                47204, 32428, 22612, 5700,  37612, 19076, 42940, 33084, 4360,
                46228, 37076, 15960, 21168, 11056, 40876, 22872, 36780, 51756,
                34516, 57420, 708,   27848, 45960, 16900, 50852, 7868,  48212,
                53780, 10828, 56608, 64412, 8788,  41580, 23456, 36556, 43344,
                64672, 17068, 40712, 57148, 7312,  12720, 52480, 16,    48872,
                35572, 2992,  23124, 7348,  52932, 4440,  43648, 47880, 8868,
                53568, 45380, 17836, 7804,  24596, 36820, 42252, 16212, 28592,
                53752, 952,   62740, 16844, 40044, 60588, 6936,  57012, 47464,
                61484, 37444, 11792, 56244, 7564,  35172, 43456, 16264, 57092,
                32332, 5656,  47012, 33952, 64180, 29752, 59004, 3444,  40992,
                64732, 7248,
            },
            {
                36772, 61044, 14548, 30904, 2112,  62132, 58316, 27760, 36084,
                61084, 47704, 24160, 51392, 11012, 29364, 13944, 54872, 26708,
                10264, 32404, 44364, 51104, 6256,  41292, 13780, 24924, 60592,
                5488,  36364, 53988, 6696,  19996, 3956,  56352, 23976, 38332,
                26864, 10412, 50132, 63800, 29368, 11468, 62156, 25268, 13508,
                61008, 30448, 1680,  63680, 33152, 45088, 53756, 18664, 8248,
                24724, 20044, 39908, 53368, 22424, 59700, 41004, 30980, 4036,
                22844, 28648, 37716, 20852, 32888, 53100, 11976, 55504, 30128,
                4636,  27028, 35540, 24012, 61548, 39136, 19580, 25160, 59968,
                29492, 45220, 42084, 61832, 30900, 13084, 24448, 63864, 32080,
                23068, 28040, 65036, 39424, 50364, 19988, 63148, 33144, 23416,
                7836,  46752, 43076, 9380,  30076, 48720, 19824, 43712, 27888,
                3232,  51784, 20272, 39644, 48636, 59208, 30120, 51708, 1944,
                62352, 27756, 55208, 17200, 39952, 45288, 14332, 21532, 33428,
                55980, 24884,
            },
            {
                2920,  50160, 20976, 53360, 38788, 12984, 18920, 46008, 52556,
                10056, 29832, 58504, 4660,  41716, 62348, 48948, 7372,  64496,
                37432, 48408, 15064, 56656, 22968, 62200, 34432, 45988, 50532,
                21624, 44264, 32072, 14684, 47456, 62944, 29756, 576,   53000,
                59264, 14852, 35308, 1256,  45464, 54808, 3068,  39620, 51976,
                19680, 43268, 54500, 47952, 7516,  29552, 1084,  64248, 42212,
                30432, 49876, 14632, 7308,  35944, 2748,  12776, 64948, 43860,
                16108, 60380, 6620,  50872, 3324,  59424, 18208, 1188,  47528,
                59916, 13148, 51180, 2284,  29664, 54092, 43236, 32440, 5892,
                14608, 63968, 10576, 34024, 16776, 38616, 51540, 816,   58024,
                11488, 49284, 5104,  14252, 54744, 9768,  45880, 57656, 12356,
                49924, 20680, 36028, 24736, 55760, 14304, 33404, 23736, 9004,
                15680, 58484, 24656, 756,   65392, 18548, 5192,  38172, 41812,
                21272, 49884, 37064, 3172,  23904, 7880,  52220, 37568, 17312,
                46072, 12816,
            },
            {
                28416, 41336, 4988,  63496, 25152, 48580, 7512,  32932, 3800,
                39452, 14220, 44248, 33772, 20728, 1380,  35492, 17584, 43304,
                21196, 5552,  28984, 39256, 9396,  19084, 772,   30132, 16140,
                55876, 2140,  25696, 57968, 34564, 41356, 11840, 45068, 7568,
                19288, 42864, 57132, 25808, 20896, 15984, 35732, 47600, 9232,
                28124, 5864,  39240, 12804, 59796, 26312, 16512, 57892, 12456,
                3968,  60516, 43932, 62788, 32908, 56640, 20596, 25908, 54648,
                33528, 46552, 41108, 13576, 30548, 44800, 26124, 39604, 21228,
                33140, 42676, 19344, 45764, 9508,  16688, 56736, 11516, 46468,
                51972, 20804, 26192, 55060, 47644, 60580, 6996,  36696, 18896,
                41644, 35116, 56936, 43152, 31132, 592,   26636, 4332,  37948,
                64604, 30696, 2940,  52796, 62068, 5232,  38376, 64024, 54712,
                35968, 42656, 31572, 46328, 9772,  27936, 54332, 12228, 25008,
                7664,  14396, 43704, 11024, 56892, 32108, 60704, 26336, 5692,
                62084, 52756,
            },
            {
                23012, 58452, 35896, 9892,  29436, 56308, 41256, 22256, 54660,
                62428, 18592, 8044,  53616, 27076, 59636, 46320, 30516, 11524,
                58868, 53076, 3136,  63552, 35676, 53920, 43272, 8400,  64300,
                40136, 11372, 61908, 8572,  17612, 51636, 21732, 60936, 36488,
                30536, 4436,  51608, 40000, 7332,  58608, 30964, 22704, 65052,
                56684, 17248, 24548, 35824, 20320, 40340, 49372, 33928, 45824,
                37824, 26900, 17952, 10860, 47280, 28936, 50048, 8388,  36508,
                1244,  18984, 25012, 63356, 48816, 35836, 6344,  63916, 9924,
                52896, 7032,  57716, 64176, 37956, 4564,  34756, 23524, 58516,
                36452, 40344, 1808,  5548,  19668, 27672, 44972, 15448, 30004,
                62008, 2464,  25692, 21568, 61592, 36148, 17792, 52512, 41484,
                14840, 56332, 18100, 11452, 41212, 27264, 1232,  45656, 12416,
                29184, 5812,  61068, 13740, 34336, 22092, 44260, 60156, 35564,
                64252, 52816, 29568, 61808, 47308, 19128, 1616,  49132, 43652,
                10544, 31756,
            },
            {
                19056, 46932, 15352, 44420, 17528, 968,   64584, 12140, 26528,
                35536, 48156, 23492, 65424, 40008, 15176, 6512,  56528, 25420,
                33712, 40952, 16884, 24608, 12264, 47852, 58520, 27416, 33668,
                20144, 48652, 28632, 45476, 38084, 5828,  27592, 14468, 48976,
                65488, 23112, 12472, 33288, 62836, 48628, 42680, 14404, 540,
                33672, 49576, 61328, 2760,  52392, 56212, 9712,  23212, 6348,
                54900, 51284, 1456,  24440, 41892, 4720,  15180, 39532, 61928,
                11700, 51372, 57116, 9164,  22008, 15932, 56336, 49804, 28548,
                37240, 23584, 15172, 31080, 25892, 50492, 62844, 1268,  18132,
                8376,  30452, 49176, 58932, 42484, 9588,  56056, 22680, 47072,
                54044, 8260,  16456, 46436, 11368, 58956, 48324, 22284, 28324,
                5664,  44484, 33216, 47196, 21948, 50472, 58836, 18876, 52572,
                21796, 49656, 17616, 55052, 40840, 50340, 2800,  16532, 47756,
                32476, 19648, 4688,  26004, 35092, 41400, 13244, 29164, 57836,
                39088, 4,
            },
            {
                55360, 7740,  32840, 59508, 51896, 37572, 31592, 45420, 16184,
                1624,  57368, 30140, 12004, 2820,  52040, 22592, 37992, 49388,
                1100,  61264, 44596, 50404, 31152, 21392, 4236,  14388, 51728,
                5084,  36552, 53624, 22952, 59948, 2616,  57380, 33808, 40988,
                9516,  54640, 46532, 16852, 27464, 10664, 4328,  54240, 38080,
                44100, 11288, 30864, 46020, 14836, 5036,  30560, 61032, 15276,
                20440, 32480, 59060, 37040, 55500, 63852, 18456, 44880, 58500,
                27480, 32152, 4568,  39140, 28880, 1740,  43048, 12996, 17640,
                61752, 452,   48224, 41556, 12256, 21120, 47380, 42024, 26676,
                61672, 53348, 15888, 24232, 35844, 32040, 62680, 4640,  12412,
                38676, 33684, 50744, 28968, 40032, 7056,  32112, 62432, 10000,
                58096, 24208, 61148, 8124,  36428, 16276, 31720, 6876,  39420,
                62960, 1644,  37356, 26112, 4532,  63564, 31024, 56388, 10408,
                464,   40716, 58908, 15596, 8936,  53968, 63024, 21992, 15992,
                34400, 65332,
            },
            {
                13028, 48760, 23800, 5672,  26996, 20140, 4352,  49488, 60112,
                41784, 6676,  37460, 49968, 33180, 43116, 18692, 63280, 9236,
                14152, 20308, 28088, 7140,  56360, 41320, 37832, 60448, 24844,
                42888, 16876, 40,    13344, 30956, 43164, 54152, 24692, 17816,
                3544,  29024, 37660, 1472,  60240, 51900, 19276, 29524, 25720,
                58012, 21100, 8000,  63788, 27732, 43592, 36420, 48132, 39888,
                65492, 28796, 8524,  13700, 21512, 52416, 30412, 2640,  48424,
                21068, 14936, 43588, 61636, 53824, 33508, 60288, 25212, 40868,
                55236, 34380, 5404,  53724, 60428, 8060,  32000, 55596, 13612,
                44828, 38228, 10920, 65196, 3304,  14124, 40012, 52012, 25956,
                58616, 19292, 63372, 3272,  53044, 15280, 43440, 2520,  35636,
                50904, 39008, 48,    28944, 54540, 65072, 10420, 43956, 26608,
                33240, 11100, 46692, 57760, 8980,  19484, 38644, 23564, 27548,
                45560, 55552, 37796, 48584, 23432, 2896,  36876, 50496, 7452,
                25540, 42188,
            },
            {
                28244, 36764, 62576, 11612, 43336, 61596, 55944, 24272, 10600,
                28168, 20312, 45000, 17020, 58780, 4380,  28508, 54984, 45308,
                31844, 53812, 36980, 10700, 64076, 1692,  18264, 46720, 10200,
                62732, 56192, 33040, 64800, 49396, 19812, 11188, 7208,  45352,
                62108, 57884, 20248, 42308, 23856, 34364, 39564, 63456, 6664,
                16176, 51276, 34740, 40560, 18104, 58168, 44,    25648, 11464,
                3392,  44968, 49152, 40660, 6464,  33960, 10584, 24216, 35300,
                7848,  52772, 36972, 47416, 10780, 19884, 46408, 3836,  8656,
                30044, 44444, 19452, 27512, 38604, 17496, 36092, 3732,  29396,
                6048,  20876, 33816, 54912, 43680, 49988, 20152, 29056, 172,
                42916, 10512, 23952, 37276, 57328, 20708, 25608, 55140, 17472,
                11968, 20260, 48500, 14248, 42288, 3840,  24000, 57008, 48040,
                15140, 59876, 23360, 29488, 44060, 14656, 59588, 6772,  52192,
                13252, 21524, 7164,  64936, 33048, 43296, 11352, 56828, 46416,
                3556,  52360,
            },
            {
                21012, 2516,  53928, 40176, 34388, 14820, 7812,  39236, 33956,
                51236, 62608, 55860, 8724,  25600, 61924, 35556, 6956,  24304,
                39472, 4172,  59548, 16168, 46000, 26792, 34368, 54344, 30092,
                6776,  21212, 40644, 9092,  26060, 38784, 60636, 35888, 52216,
                31808, 13476, 48888, 8880,  56496, 5612,  45180, 12316, 59660,
                47244, 1836,  24392, 53264, 10776, 22152, 62392, 51012, 19284,
                54156, 35012, 16796, 62728, 26504, 44088, 60320, 38992, 55144,
                65112, 312,   23568, 17392, 5348,  26800, 57972, 38388, 22268,
                50816, 65136, 11232, 58704, 2260,  52328, 64384, 24420, 58424,
                51488, 47832, 1176,  27880, 17732, 8540,  57824, 36548, 64520,
                48952, 32012, 5148,  45500, 13224, 33948, 65416, 46908, 41048,
                31412, 63484, 26404, 58716, 33496, 49992, 37428, 19888, 2772,
                54008, 40200, 5912,  35648, 53108, 48664, 32268, 41444, 63272,
                35352, 30568, 18084, 51568, 27008, 60312, 19544, 30160, 39844,
                17276, 59000,
            },
            {
                44772, 30804, 17900, 4768,  50804, 46860, 29452, 18672, 54352,
                13748, 2656,  22648, 32060, 14368, 39824, 48400, 12088, 17396,
                64812, 47592, 21772, 32744, 52212, 22864, 5736,  12888, 48832,
                37340, 27292, 45684, 52428, 4712,  15344, 47368, 27852, 21916,
                40132, 2432,  26308, 63832, 30820, 15224, 50096, 22556, 32132,
                41368, 36808, 57296, 5460,  31372, 46740, 7144,  33576, 41944,
                27576, 58496, 22620, 776,   55888, 50780, 4180,  13888, 17700,
                42128, 27964, 57624, 31316, 62968, 36172, 52140, 12148, 61120,
                16164, 6440,  24824, 33112, 45376, 14752, 42804, 10660, 39196,
                16864, 63068, 22996, 59232, 31392, 45872, 24416, 6364,  13100,
                17272, 55472, 61072, 27340, 49996, 1064,  9084,  28872, 6488,
                53128, 3532,  44912, 7400,  18256, 12184, 28280, 62496, 8516,
                30288, 18000, 64152, 21088, 3288,  11508, 25392, 17408, 2292,
                57720, 42868, 10712, 1132,  46152, 14256, 4068,  24532, 63980,
                33260, 9736,
            },
            {
                38400, 63460, 57308, 25816, 21728, 58768, 1180,  64968, 44124,
                6060,  36232, 41116, 46924, 53176, 944,   21164, 57228, 51160,
                29668, 42408, 420,   13352, 58468, 39740, 43484, 61704, 57516,
                15540, 3036,  59220, 18296, 34832, 56744, 1044,  64112, 6508,
                16440, 44188, 35428, 52764, 18376, 37420, 1020,  55008, 9624,
                28192, 19472, 64944, 13656, 43172, 37592, 16400, 56272, 13932,
                4408,  9800,  38236, 31436, 12100, 19348, 29220, 46880, 32360,
                50068, 9020,  12976, 41344, 48548, 15476, 2408,  31676, 42788,
                35228, 48528, 40488, 56652, 21516, 28920, 49420, 20000, 32256,
                7892,  35312, 41384, 11356, 4880,  62048, 54292, 33924, 47300,
                41068, 22536, 7672,  38972, 18156, 58996, 36576, 21756, 60336,
                15996, 37636, 55900, 22560, 39876, 60024, 52236, 42856, 35168,
                45540, 50388, 12668, 42404, 56120, 60572, 37624, 45856, 6168,
                49232, 22984, 62192, 39480, 31200, 55400, 35952, 52696, 6052,
                49212, 14916,
            },
            {
                672,   47492, 8168,  12756, 32684, 42348, 9816,  23412, 31300,
                48804, 26736, 63472, 10996, 29052, 60432, 43960, 32852, 5176,
                10116, 26344, 55232, 36324, 8792,  28664, 2212,  19588, 24276,
                32104, 50564, 11048, 62900, 44120, 23352, 30712, 12396, 50088,
                55720, 59404, 10892, 4816,  46324, 60416, 25796, 43552, 62720,
                15676, 4260,  48532, 26564, 54452, 59880, 28960, 24100, 64404,
                45304, 52084, 61676, 47976, 42512, 23636, 63936, 57952, 5572,
                21216, 61380, 37892, 55632, 7172,  24572, 44856, 56816, 27368,
                20632, 1236,  13884, 51352, 8844,  5228,  62160, 256,   46496,
                55520, 13764, 44784, 52768, 37232, 15032, 40228, 21368, 1760,
                29376, 52624, 14872, 44140, 30620, 52224, 42104, 12812, 48940,
                25156, 33876, 10376, 30256, 46928, 1432,  6524,  14780, 21656,
                58280, 1864,  25048, 33448, 28360, 8728,  51172, 20540, 29612,
                14536, 26168, 53704, 5376,  16500, 59396, 9256,  41012, 21424,
                60724, 27160,
            },
            {
                23468, 52912, 41352, 36284, 61880, 16676, 52092, 37584, 60316,
                15828, 20012, 57724, 5040,  18272, 37288, 24908, 15048, 62596,
                38548, 18872, 60972, 48768, 17024, 63768, 51504, 35368, 8020,
                42508, 38504, 22044, 28856, 7708,  40436, 54404, 19476, 37744,
                32408, 24408, 20912, 29516, 40892, 13004, 20332, 34768, 7360,
                52260, 42604, 33700, 21344, 3100,  9120,  49760, 1292,  39968,
                30400, 20544, 15028, 7840,  34312, 2052,  37272, 9852,  25436,
                43916, 33620, 1848,  18812, 29428, 64728, 10148, 17796, 5264,
                53504, 63624, 37264, 30692, 59312, 35488, 53792, 40792, 24940,
                60504, 29800, 2488,  25820, 19260, 48648, 27476, 9096,  63012,
                58800, 35696, 4612,  64272, 10640, 24464, 3760,  56708, 44212,
                1772,  61756, 51936, 15260, 63824, 24956, 56576, 32340, 26844,
                10176, 37124, 61900, 47460, 16700, 128,   65292, 34452, 55336,
                60812, 36968, 47000, 33860, 22124, 43896, 28176, 46892, 12868,
                55644, 34992,
            },
            {
                10720, 19456, 29340, 56376, 2788,  27604, 47716, 6980,  12220,
                55696, 2240,  43140, 33820, 50396, 8204,  54636, 3084,  46628,
                34252, 52332, 6712,  24628, 31632, 45480, 11640, 55588, 46912,
                65436, 1204,  53640, 13696, 33580, 60540, 46808, 3468,  42948,
                62168, 8180,  48452, 54380, 65048, 2988,  50408, 58492, 29792,
                38384, 56964, 11072, 61392, 39596, 18764, 35412, 12204, 47468,
                5772,  36208, 25592, 56916, 60604, 52560, 17080, 40540, 54716,
                14964, 51752, 46364, 22756, 53424, 34672, 49712, 39632, 59028,
                46356, 10928, 25364, 44280, 17740, 22880, 12492, 27944, 15836,
                5576,  21204, 57440, 64912, 33092, 3704,  56028, 43148, 17692,
                11752, 49448, 26244, 56052, 19948, 34848, 62716, 17100, 28432,
                38856, 20048, 5556,  41860, 35700, 19352, 49120, 39004, 64904,
                53188, 5280,  14132, 54864, 39476, 23288, 43984, 12404, 40648,
                9352,  2568,  19008, 11800, 64548, 50360, 1700,  17860, 31100,
                3936,  43644,
            },
            {
                65120, 15700, 49936, 6368,  45028, 20648, 54616, 33504, 25280,
                39708, 30168, 52472, 23616, 13368, 65152, 41712, 27440, 58420,
                22028, 12388, 44512, 3340,  40664, 21416, 4276,  27400, 14184,
                17960, 25880, 58232, 48208, 4264,  16236, 25432, 10240, 28104,
                14804, 236,   35804, 16640, 26776, 32648, 10252, 44996, 23404,
                18112, 444,   24936, 45472, 31108, 51592, 62480, 22000, 58896,
                54336, 17544, 43640, 4824,  27484, 13120, 45336, 29908, 3720,
                63260, 26960, 4852,  60484, 12296, 42208, 520,   21700, 32856,
                29080, 15684, 6824,  61200, 3524,  47656, 38496, 64304, 50668,
                34116, 47780, 38184, 10052, 50888, 13480, 60628, 30940, 37344,
                23784, 45340, 32716, 480,   47916, 39468, 6016,  31960, 51140,
                11408, 47360, 58312, 27876, 9024,  44560, 12028, 3052,  16836,
                46544, 41056, 20788, 30600, 7008,  58980, 32008, 18300, 4312,
                51672, 28564, 42060, 55972, 7528,  25672, 36836, 62640, 51812,
                39772, 58308,
            },
            {
                33128, 3280,  38896, 23116, 63412, 14128, 40832, 600,   64184,
                45888, 9916,  59568, 38160, 47268, 20936, 31588, 16896, 9168,
                39864, 30404, 56080, 14796, 61628, 57912, 33900, 39276, 60648,
                31232, 6960,  35460, 43220, 20644, 51904, 36816, 64504, 49352,
                56932, 40320, 45468, 59800, 7156,  38936, 56076, 14768, 5944,
                60832, 49452, 13516, 53776, 4944,  15640, 43504, 26448, 32556,
                9336,  63360, 49640, 38932, 21508, 32940, 50216, 59712, 19192,
                35708, 10644, 39352, 30992, 16748, 57476, 26432, 8332,  62760,
                38452, 50268, 55092, 41668, 32676, 57692, 9288,  2136,  42488,
                54588, 7748,  17460, 41708, 23156, 45424, 6988,  52200, 2524,
                57320, 8336,  16376, 41932, 60988, 14392, 53196, 22056, 7864,
                64440, 23940, 33212, 368,   52688, 60140, 30396, 55656, 23452,
                34012, 27952, 60308, 51532, 10896, 49060, 26848, 46168, 58060,
                24488, 62332, 48568, 31364, 39016, 14004, 57048, 5728,  20664,
                8664,  24856,
            },
            {
                46164, 54440, 59868, 31320, 10532, 35776, 58104, 18736, 50320,
                22012, 4272,  17048, 28164, 6592,  192,   60268, 50024, 53672,
                2172,  64460, 25980, 36568, 50332, 18508, 9524,  48492, 22292,
                51704, 41088, 10648, 61944, 29596, 55832, 7096,  32792, 17924,
                21804, 30376, 11764, 51832, 23144, 19488, 48180, 62928, 41724,
                27632, 33264, 37620, 64544, 28476, 8128,  57480, 2328,  41264,
                14276, 29476, 292,   55556, 10680, 65268, 2708,  8236,  23180,
                48264, 56540, 43260, 6916,  47392, 37012, 51420, 14228, 45280,
                3752,  23268, 18724, 27132, 13384, 51928, 29964, 18840, 24120,
                12072, 31124, 62012, 644,   29076, 35820, 19636, 25884, 40040,
                63996, 21096, 50968, 29604, 9608,  27396, 44956, 59092, 36204,
                42304, 14292, 56236, 17516, 22212, 37580, 4812,  42196, 50596,
                8856,  924,   44460, 17948, 36152, 2400,  63520, 14312, 37792,
                33624, 21092, 15156, 656,   60276, 22432, 43464, 34064, 29532,
                49072, 13464,
            },
            {
                27680, 7668,  17760, 48136, 26228, 43580, 8220,  28328, 32240,
                13664, 35904, 54556, 62560, 40964, 34744, 12504, 43604, 35876,
                19616, 48236, 7944,  43016, 628,   29128, 53544, 6196,  44568,
                2976,  55188, 16720, 24076, 660,   12488, 39764, 45232, 3112,
                53768, 61536, 4424,  34020, 43412, 1572,  30052, 35928, 3520,
                54376, 20780, 9180,  42124, 22956, 48088, 37072, 20040, 53212,
                46480, 23888, 35220, 18436, 47076, 40780, 25272, 37500, 58192,
                32316, 13548, 20588, 64044, 23820, 3412,  61640, 19944, 54024,
                10504, 34692, 65408, 1580,  37672, 21644, 62652, 44424, 36600,
                56392, 46300, 26216, 58236, 53244, 63076, 15804, 48200, 12384,
                34236, 4284,  53972, 37520, 22776, 56368, 1984,  18752, 48968,
                4580,  29224, 38636, 46080, 63252, 13544, 26496, 62276, 19840,
                58428, 12876, 62828, 25520, 56500, 41960, 21956, 5920,  54516,
                10804, 6888,  44548, 53416, 27812, 9976,  47640, 15788, 54896,
                61028, 37628,
            },
            {
                22080, 42936, 35056, 1732,  52600, 61652, 4488,  47084, 55572,
                60728, 44564, 24816, 8820,  49100, 56472, 23080, 26580, 4844,
                59284, 15792, 22360, 32656, 11884, 62772, 24784, 36192, 13216,
                63964, 27088, 37928, 46200, 50208, 63200, 23072, 58464, 27220,
                9172,  37936, 25328, 13952, 63692, 53452, 16832, 25180, 11952,
                46896, 60004, 16356, 1708,  56124, 33812, 11300, 64040, 4156,
                38468, 59608, 7932,  62104, 30996, 14540, 53908, 44404, 16804,
                940,   51084, 27676, 54436, 9928,  34192, 28084, 40688, 31156,
                59816, 43192, 48472, 57108, 8256,  46632, 15328, 6380,  59956,
                3032,  20696, 14348, 39348, 10792, 3944,  42436, 55404, 28440,
                59184, 43612, 13444, 62664, 5100,  34352, 40872, 12860, 26104,
                60920, 54468, 2916,  10228, 49652, 34504, 7560,  15864, 47424,
                35840, 31120, 39176, 6340,  15468, 32956, 52372, 29500, 47604,
                41480, 64856, 17864, 35008, 50688, 3476,  64204, 40100, 5024,
                18980, 0,
            },
            {
                51860, 63092, 12332, 56596, 15096, 21496, 37788, 17552, 11344,
                40212, 1820,  51840, 19960, 30480, 16320, 64128, 9708,  45884,
                28972, 38608, 52508, 57284, 46976, 39692, 16124, 59252, 30260,
                20020, 33644, 8080,  59892, 31280, 18952, 35716, 14596, 43256,
                50684, 17464, 47560, 57336, 39496, 7552,  49984, 57816, 40352,
                6756,  30912, 52112, 44764, 18316, 60424, 24916, 49828, 28644,
                17220, 12552, 52032, 42612, 21084, 5896,  28264, 9676,  63724,
                41736, 6332,  38436, 45080, 17696, 58336, 49096, 2076,  16092,
                25640, 5588,  12300, 29576, 40588, 34032, 53656, 26660, 32144,
                40772, 52072, 6780,  34172, 49536, 22084, 31596, 8852,  936,
                23600, 18568, 31780, 46940, 17040, 49688, 65044, 31404, 8228,
                34772, 20424, 43136, 24892, 31500, 59084, 52456, 40608, 28456,
                4064,  54496, 22404, 49848, 45716, 9700,  61580, 18932, 1220,
                26328, 56100, 38588, 24096, 58796, 31736, 20176, 26372, 11648,
                44904, 58068,
            },
            {
                9652,  39356, 31076, 25080, 41588, 29116, 51268, 65380, 23380,
                6828,  27424, 33972, 14040, 42760, 2728,  38980, 32700, 55024,
                13128, 61732, 5436,  26484, 19100, 3608,  54888, 42616, 1604,
                50560, 57068, 15000, 42724, 4620,  9864,  48348, 1920,  29728,
                65260, 5328,  31804, 20052, 27780, 10304, 32776, 21232, 64984,
                14456, 35320, 26500, 38720, 5392,  30080, 14576, 6580,  43568,
                55856, 33196, 26828, 4476,  57744, 48028, 60556, 35036, 52656,
                30156, 24624, 61180, 4412,  31452, 12564, 42572, 7784,  56468,
                37828, 20924, 53120, 17488, 24328, 58588, 476,   11396, 47980,
                16412, 62912, 29688, 44540, 17676, 59988, 38304, 64932, 45708,
                50452, 39656, 7728,  26552, 58644, 10296, 24184, 53556, 46240,
                15572, 51228, 64120, 6208,  18072, 44272, 2360,  23920, 57524,
                10100, 42844, 18616, 65084, 27716, 4200,  40296, 36020, 59308,
                12600, 30612, 4012,  8680,  13436, 42884, 36592, 53032, 62672,
                33844, 28928,
            },
            {
                4820,  49724, 19228, 6124,  59544, 9268,  2632,  31552, 43364,
                36712, 57200, 63628, 46652, 5808,  52992, 60764, 20600, 50228,
                988,   35032, 44464, 9936,  65000, 34028, 22016, 46520, 10560,
                38488, 5420,  21908, 52916, 26712, 40508, 54388, 60872, 20720,
                11592, 41308, 55436, 2224,  45416, 62024, 37372, 664,   43584,
                55100, 23248, 9912,  62848, 47236, 53404, 40928, 35048, 61828,
                22628, 1320,  49368, 39588, 36216, 17680, 1632,  22468, 11704,
                19092, 47432, 14720, 36304, 51892, 64900, 22676, 35424, 51152,
                63388, 46128, 32688, 62252, 5116,  49296, 20532, 64560, 35560,
                23236, 9376,  57172, 24556, 5840,  53820, 26524, 14976, 35692,
                11580, 61844, 55684, 2448,  38060, 42696, 20100, 908,   37236,
                58276, 10876, 28312, 39572, 55468, 12116, 21124, 37772, 63764,
                14544, 51080, 1360,  34460, 12444, 58132, 24580, 15752, 50776,
                22244, 45344, 49840, 61968, 17116, 47124, 2692,  7492,  49348,
                23140, 16672,
            },
            {
                44344, 35856, 64532, 53420, 45404, 34440, 48848, 58052, 14628,
                53304, 18108, 10436, 22672, 58776, 36008, 25192, 7808,  17332,
                41508, 23756, 15548, 30832, 49168, 7660,  28216, 14112, 62072,
                25560, 48076, 64892, 35188, 12808, 57800, 23816, 33208, 37704,
                52564, 24312, 34940, 60072, 12932, 23488, 48144, 28916, 18488,
                5160,  48856, 58728, 13220, 21364, 416,   58152, 9032,  19612,
                45984, 11052, 64512, 24284, 13288, 55056, 31864, 45320, 38752,
                58384, 8364,  55036, 20428, 72,    26324, 46612, 18608, 13552,
                28412, 1056,  9788,  41676, 14340, 37244, 43948, 28720, 54740,
                4096,  41976, 50720, 36700, 12960, 47376, 3256,  20328, 57580,
                4928,  29336, 21384, 33492, 51148, 14960, 29984, 61440, 4888,
                41540, 22720, 33276, 1568,  47132, 61660, 30688, 7036,  45616,
                32736, 25104, 60324, 38264, 47760, 52724, 31356, 43480, 5340,
                63428, 33596, 19916, 40776, 54932, 25448, 57060, 30700, 14136,
                41168, 56044,
            },
            {
                26728, 13560, 2180,  22596, 16136, 39048, 12244, 20168, 26236,
                512,   48484, 41260, 32552, 28340, 12136, 44028, 48372, 31576,
                63300, 55836, 51552, 37752, 59468, 41640, 51964, 35812, 56476,
                31752, 16404, 44064, 29504, 900,   45916, 16692, 3768,  8472,
                46476, 15284, 6564,  43104, 17348, 54268, 6448,  59432, 51868,
                32460, 40756, 3116,  36756, 27504, 32364, 16112, 50672, 38796,
                30624, 59068, 16772, 43832, 8296,  62876, 27180, 50164, 6036,
                63256, 32704, 41224, 28640, 60048, 38952, 10104, 57616, 4536,
                43556, 23812, 55692, 50484, 26280, 61176, 8760,  18280, 39072,
                13848, 60844, 19168, 1072,  63644, 29044, 43008, 32836, 52104,
                25136, 41128, 45324, 12164, 62880, 6428,  44496, 55096, 26592,
                17588, 48620, 60816, 14272, 35576, 25556, 53324, 49512, 19072,
                56312, 5332,  29588, 16452, 8152,  20252, 1832,  56692, 11144,
                37600, 7576,  27204, 400,   35432, 10812, 21284, 60304, 37364,
                1192,  61712,
            },
            {
                20240, 47708, 56900, 33176, 27472, 60644, 3964,  63268, 45784,
                30180, 8360,  62224, 3620,  50796, 19104, 2048,  57124, 27364,
                10884, 3004,  20920, 5128,  12944, 24588, 116,   18684, 6456,
                40784, 3080,  9580,  59340, 19608, 38632, 63584, 49912, 28156,
                58304, 62964, 30488, 50900, 26748, 40404, 34436, 11688, 15644,
                24756, 63868, 19416, 46060, 56064, 42312, 65444, 26072, 2904,
                53136, 6452,  27836, 33688, 51864, 20352, 4112,  41840, 16152,
                25728, 2612,  49032, 11992, 44152, 6476,  53980, 30192, 33800,
                60328, 38764, 19124, 34856, 3436,  31360, 56396, 2152,  45932,
                31696, 48676, 27032, 33612, 40160, 55512, 10628, 62484, 17424,
                47696, 8700,  53716, 18468, 57840, 25356, 35656, 9896,  50516,
                31700, 7704,  56552, 20192, 41876, 8988,  16068, 208,   36000,
                11788, 40732, 45076, 54212, 23656, 42292, 62668, 27968, 48404,
                54064, 16252, 59908, 51648, 29852, 65476, 44144, 18136, 46732,
                9276,  32356,
            },
            {
                4208,  38432, 10468, 42956, 7368,  49864, 23960, 40380, 35064,
                56532, 21684, 37652, 16240, 54604, 39484, 65116, 14652, 36328,
                40204, 46120, 60796, 29416, 17184, 57796, 63648, 45496, 53872,
                21128, 61368, 24496, 50756, 54724, 6912,  31740, 13164, 41024,
                18808, 348,   36436, 20792, 8640,  64360, 2928,  46212, 37708,
                56772, 8808,  29924, 51328, 6968,  11092, 23032, 47732, 12904,
                36624, 42076, 55516, 2572,  46920, 37916, 57504, 12712, 53992,
                36544, 18292, 57192, 23100, 34540, 16708, 62556, 24684, 49320,
                15376, 6992,  64996, 11380, 46792, 15912, 51048, 25148, 59520,
                6116,  21612, 9060,  58040, 16072, 22640, 7060,  38664, 2084,
                59820, 14288, 36920, 80,    31880, 48056, 21856, 64312, 13104,
                37848, 44720, 4048,  51736, 65272, 29304, 58844, 42776, 62448,
                27456, 21300, 64012, 2936,  58664, 35924, 13712, 33284, 19324,
                24460, 41572, 46540, 9536,  15036, 39396, 6292,  27788, 52076,
                24092, 54368,
            },
            {
                62996, 29656, 59044, 52188, 18948, 30976, 55172, 10284, 5644,
                51988, 13388, 58968, 24536, 45440, 30276, 5724,  22364, 52580,
                6884,  24988, 33932, 43384, 48312, 38924, 31308, 10832, 27700,
                49592, 37360, 32964, 11988, 42204, 26476, 21852, 56084, 44868,
                25092, 53736, 11460, 47584, 57248, 31520, 22760, 61024, 27712,
                1552,  43404, 13632, 35072, 17668, 59960, 4708,  34056, 57224,
                18348, 21924, 61584, 14616, 30760, 9496,  23640, 61768, 29936,
                45040, 8960,  64020, 51656, 3448,  47512, 19992, 40828, 1816,
                45456, 21788, 52964, 28660, 58340, 22320, 39696, 10340, 35132,
                53016, 65188, 43980, 4392,  51912, 46648, 26036, 50192, 29876,
                33944, 22512, 65412, 28128, 39684, 4980,  16572, 40864, 1332,
                58624, 23552, 27620, 11384, 38444, 5540,  33564, 23604, 47228,
                8420,  51856, 14980, 31088, 10348, 47060, 40116, 7288,  3144,
                64472, 31064, 4268,  22072, 54472, 49244, 2208,  34348, 58812,
                12964, 42580,
            },
            {
                17604, 24868, 14432, 1312,  62524, 36076, 15392, 43448, 18504,
                27540, 32868, 43056, 7012,  10924, 60400, 34456, 47444, 58672,
                18052, 62840, 12380, 54792, 8976,  4016,  22372, 34792, 14520,
                7592,  58156, 17828, 46484, 3976,  34960, 60396, 9532,  4996,
                33572, 39492, 61492, 5900,  43428, 13300, 16952, 51020, 41060,
                20948, 54224, 61632, 25700, 39956, 52812, 44180, 30020, 62476,
                8212,  49628, 25368, 40584, 59460, 17476, 48808, 34284, 1556,
                20732, 39388, 27240, 14244, 31920, 37864, 8528,  52592, 12480,
                58952, 31664, 42044, 37036, 5776,  43772, 63420, 18884, 28372,
                15040, 37952, 11696, 29200, 34944, 60356, 13124, 19180, 44440,
                56988, 3960,  43020, 49624, 55820, 9068,  59912, 52196, 29016,
                19044, 54096, 34408, 48108, 17480, 21716, 55040, 13624, 57644,
                4060,  37884, 34040, 49648, 26348, 18128, 60700, 51824, 55768,
                44644, 11956, 38200, 58000, 33016, 24748, 62696, 16548, 40144,
                5372,  48936,
            },
            {
                7992,  34044, 46116, 40804, 22272, 46984, 2716,  64980, 57996,
                48232, 1392,  63732, 49808, 27036, 20132, 340,   42496, 9340,
                31772, 50144, 1260,  19756, 26648, 51260, 56320, 44156, 62000,
                41416, 1028,  29968, 64600, 14988, 52116, 38128, 17536, 64232,
                50140, 14376, 29284, 23964, 38144, 55532, 35588, 4692,  10356,
                47352, 33316, 6528,  48752, 912,   20636, 37940, 15296, 27052,
                45568, 4088,  11420, 35640, 252,   44316, 28016, 6788,  52888,
                58860, 46800, 4728,  42072, 61396, 56800, 28888, 64288, 36132,
                26852, 9104,  18076, 2332,  13956, 54360, 32024, 88,    42216,
                47160, 23924, 55244, 18220, 41460, 632,   63408, 37212, 54188,
                7972,  16800, 25840, 11484, 20096, 45792, 24720, 32916, 42836,
                61516, 15108, 2628,  62600, 43800, 50868, 1988,  39616, 30232,
                18388, 60624, 44052, 6132,  56256, 1500,  29268, 22852, 15236,
                35660, 26840, 61112, 17908, 6976,  43188, 11448, 55668, 30724,
                22440, 36456,
            },
            {
                64280, 56424, 26928, 6296,  12160, 54128, 28800, 24920, 38500,
                11804, 21148, 35668, 15564, 39184, 52172, 56276, 24228, 15652,
                27900, 38680, 57288, 41940, 36468, 65432, 16220, 5992,  25664,
                53228, 47936, 23296, 8536,  55804, 44356, 1424,  28500, 46844,
                22504, 3120,  59152, 18628, 1364,  48640, 28420, 59324, 18516,
                63688, 24020, 15840, 58592, 28452, 64132, 10124, 2588,  53964,
                39552, 33160, 64868, 50992, 54804, 21596, 62852, 38208, 11392,
                15820, 31228, 55160, 10088, 25040, 744,   16248, 22656, 5308,
                50528, 56156, 62408, 47864, 26480, 59944, 8200,  49692, 57164,
                5608,  61840, 2740,  31856, 45760, 21892, 6504,  27864, 11128,
                41200, 60524, 52096, 34316, 62292, 14448, 37280, 3600,  7872,
                46996, 10152, 40368, 30708, 25076, 59488, 9040,  45976, 25964,
                53832, 11176, 24300, 20832, 64852, 38716, 43620, 8848,  48932,
                20108, 840,   51184, 47044, 29208, 36124, 20028, 45996, 61248,
                1004,  51492,
            },
            {
                10872, 20112, 50000, 37964, 61432, 33332, 8912,  51248, 4176,
                30880, 41952, 55792, 3208,  61132, 31104, 12556, 37168, 64464,
                44836, 5220,  22192, 13988, 29928, 2216,  46536, 20092, 38256,
                11720, 33988, 19392, 39904, 25476, 31376, 20220, 57940, 10984,
                41544, 31820, 52080, 45744, 65324, 25320, 7636,  52872, 38900,
                30200, 3744,  37284, 11716, 45236, 32164, 56248, 49324, 23580,
                59080, 15988, 19400, 27372, 8644,  13736, 41884, 50372, 25172,
                65192, 35752, 22164, 18680, 49600, 43988, 33980, 47156, 39996,
                14680, 44204, 23748, 34420, 38984, 16988, 22940, 36176, 13172,
                20356, 26612, 50700, 59108, 14552, 52548, 57852, 33252, 48616,
                23696, 31160, 6000,  38732, 2600,  29732, 49964, 57808, 21964,
                26992, 36728, 57136, 6856,  12672, 35764, 16444, 63820, 33036,
                48340, 692,   41792, 52660, 15884, 12132, 34864, 31680, 63284,
                57240, 42472, 10032, 14412, 63976, 2932,  53068, 8764,  27624,
                15264, 44084,
            },
            {
                41372, 4680,  30196, 58196, 16788, 42324, 19684, 60348, 45452,
                16196, 62356, 25308, 9728,  46132, 18772, 7216,  49116, 2528,
                53480, 60828, 34532, 49308, 58924, 10004, 32960, 60660, 29040,
                2824,  63188, 57680, 49728, 12524, 61468, 7296,  51044, 36404,
                16396, 56572, 9016,  34708, 11772, 15732, 32500, 42976, 13096,
                45896, 57548, 53236, 42028, 22652, 7428,  18944, 35044, 12536,
                6728,  43128, 31212, 46828, 57736, 4748,  32540, 18244, 2948,
                43168, 7392,  51844, 60032, 38760, 12684, 54132, 60668, 3212,
                32348, 19440, 6892,  53532, 10868, 51304, 45700, 28732, 63872,
                33168, 39652, 7820,  37028, 10320, 25300, 40192, 3588,  15400,
                63784, 20504, 46856, 56780, 43912, 18268, 63888, 12112, 52736,
                16716, 64788, 49088, 19980, 52460, 28240, 42156, 21352, 6260,
                14296, 61476, 35972, 28400, 46900, 59380, 54536, 3748,  24164,
                6680,  39280, 32640, 25628, 56096, 41120, 48828, 23384, 38596,
                59076, 31956,
            },
            {
                24644, 53232, 14032, 140,   23944, 49124, 5072,  36260, 23016,
                52828, 6632,  48868, 28980, 34560, 43160, 57896, 26256, 32880,
                17452, 11724, 7620,  43472, 18512, 53080, 40484, 22692, 55180,
                43072, 16036, 5708,  36180, 3192,  42644, 33888, 23712, 63156,
                6272,  26356, 44148, 20472, 40476, 54996, 62360, 21828, 164,
                26440, 9076,  34344, 17260, 50504, 61784, 40520, 46968, 26644,
                63512, 51944, 3040,  39220, 24436, 36380, 60752, 29564, 55524,
                47988, 13360, 26468, 3592,  30592, 6704,  27520, 21044, 10952,
                57456, 28556, 64656, 1104,  30740, 61488, 2892,  40980, 10120,
                53208, 44236, 16392, 48188, 30364, 64700, 18632, 45104, 51156,
                36236, 984,   13856, 27164, 8468,  22892, 39544, 34136, 4556,
                43660, 32372, 23760, 388,   45488, 60532, 3632,  54736, 37192,
                57732, 19872, 31352, 7488,  2552,  25736, 18996, 45768, 14064,
                28512, 53156, 59784, 21720, 6300,  17172, 33204, 13180, 5516,
                54576, 18012,
            },
            {
                62452, 47484, 36392, 64764, 44824, 31408, 10552, 56848, 13460,
                33724, 39584, 20068, 54788, 4364,  14568, 62204, 20944, 40412,
                47384, 23692, 28892, 63588, 25928, 4704,  48120, 13052, 7964,
                50460, 31140, 45660, 26972, 54420, 17728, 47608, 14036, 39680,
                54168, 3712,  61836, 29828, 2492,  50036, 5344,  35448, 60104,
                48480, 20088, 64988, 1860,  27704, 4572,  14496, 57628, 596,
                36892, 21668, 10908, 59972, 15012, 52972, 45240, 10008, 22708,
                39536, 33236, 62208, 45812, 55364, 17456, 64008, 42532, 51644,
                35604, 48536, 40536, 13448, 43848, 21260, 55452, 5300,  19156,
                59596, 3828,  23680, 61100, 1932,  35416, 7352,  56112, 29048,
                9436,  59884, 52840, 33308, 61000, 54492, 1532,  46064, 29100,
                56484, 6080,  13640, 55248, 38696, 31932, 11504, 49460, 26576,
                40552, 9584,  51404, 63540, 38312, 10652, 41084, 50716, 61212,
                36016, 18004, 2584,  46260, 37032, 28736, 57168, 65160, 44988,
                35248, 2840,
            },
            {
                9244,  40312, 21280, 7200,  27636, 54976, 38840, 63424, 27224,
                1168,  58160, 11416, 65300, 23444, 38040, 50572, 10360, 5388,
                59656, 36848, 54672, 1140,  38948, 15560, 58112, 27612, 36708,
                18744, 61240, 9964,  22204, 65172, 29700, 58708, 300,   28064,
                46208, 18224, 38216, 59452, 23872, 46412, 27584, 16828, 52248,
                40284, 13620, 55636, 31116, 38048, 53744, 23936, 30320, 44688,
                17316, 55844, 28696, 48836, 7220,  20372, 1608,  63216, 16348,
                57416, 564,   20224, 36196, 9560,  49000, 37692, 1640,  25528,
                8328,  16460, 58888, 25088, 37256, 15636, 32800, 26180, 47120,
                14280, 29440, 55628, 19688, 41964, 53292, 13196, 22264, 43096,
                25524, 38188, 16948, 41584, 11436, 48552, 25688, 15196, 59456,
                19624, 41220, 35384, 63184, 17364, 8140,  22564, 65200, 15616,
                3168,  44680, 23256, 17576, 48784, 56764, 32516, 21540, 8300,
                43836, 12288, 49836, 63436, 10392, 51444, 652,   25732, 20500,
                50556, 29020,
            },
            {
                16300, 32632, 56968, 11808, 50336, 15704, 2644,  19120, 47408,
                43928, 30220, 17320, 41476, 46864, 616,   27192, 56348, 30896,
                42552, 14428, 50080, 31952, 21252, 44592, 33616, 64644, 712,
                24152, 56384, 38576, 14320, 40816, 8340,  51820, 11428, 32924,
                21820, 49164, 10616, 14464, 36104, 8012,  56432, 10904, 29972,
                5928,  24256, 44208, 8244,  47616, 60800, 10820, 34120, 62548,
                8508,  41332, 33012, 62236, 43744, 34244, 28112, 37532, 50912,
                8144,  29248, 41664, 14892, 59216, 22920, 31980, 13108, 61776,
                45788, 22100, 4684,  54600, 46672, 7252,  57748, 65468, 36960,
                50312, 33836, 38804, 10656, 26924, 49188, 32748, 62520, 3292,
                57664, 48008, 4400,  21676, 29460, 6376,  63112, 37096, 8992,
                24456, 48396, 10500, 29648, 26076, 51296, 42924, 34640, 46840,
                28988, 54116, 33652, 60236, 14168, 4472,  29176, 65124, 160,
                57564, 25316, 30808, 40264, 15268, 43392, 59904, 8176,  41512,
                12172, 60640,
            },
            {
                26128, 1668,  42928, 60300, 34996, 22516, 41684, 51956, 8800,
                59852, 24316, 51516, 6940,  32048, 54060, 35916, 16496, 64228,
                2248,  19676, 9568,  56732, 62216, 11076, 6128,  52384, 42012,
                46796, 4524,  32496, 53320, 2356,  20784, 36048, 43736, 63560,
                57156, 7180,  31172, 51692, 41916, 18956, 64580, 43852, 37644,
                62644, 33776, 58804, 21472, 16624, 42332, 5980,  19088, 49056,
                54216, 14060, 1980,  23732, 17788, 56860, 48260, 13012, 24492,
                44296, 64104, 53524, 25680, 5200,  44860, 57344, 18892, 53472,
                39024, 33280, 63620, 10460, 28204, 51064, 41452, 11444, 21672,
                8048,  1340,  63144, 46688, 5884,  58568, 16656, 39732, 11196,
                19548, 31644, 64592, 45272, 35984, 52056, 18044, 42456, 50828,
                62232, 2420,  53464, 44740, 57460, 1484,  59528, 19232, 5156,
                62036, 12588, 1444,  42316, 26824, 36976, 53392, 16424, 39156,
                34016, 54308, 19504, 5620,  23708, 34836, 19020, 31728, 48280,
                37740, 55412,
            },
            {
                45692, 51996, 19244, 29324, 4188,  62396, 25836, 32824, 14712,
                37536, 3156,  35156, 60880, 13432, 20284, 8216,  44900, 23240,
                52608, 27972, 46188, 4228,  36080, 49464, 23060, 16492, 30216,
                12372, 60000, 26088, 49240, 44672, 60748, 25032, 15164, 4716,
                40652, 25904, 62280, 884,   58076, 33484, 22636, 2036,  47984,
                18356, 4420,  50944, 12032, 35664, 26860, 51600, 38960, 22328,
                4496,  26332, 37996, 51424, 11500, 5580,  40604, 59012, 4224,
                32160, 18408, 11584, 50060, 33760, 2368,  27860, 42032, 6804,
                29904, 668,   49540, 19664, 35224, 2436,  17624, 30668, 44040,
                53056, 25380, 57264, 13604, 22600, 30040, 624,   45648, 35760,
                54184, 7712,  14088, 55828, 352,   58272, 12648, 31440, 4300,
                28092, 37972, 14316, 21132, 5868,  39784, 13964, 30840, 49980,
                24616, 39020, 57772, 21328, 47520, 6764,  44888, 24104, 10044,
                48176, 4128,  58964, 45576, 62460, 55536, 12980, 53060, 3740,
                22900, 6192,
            },
            {
                64000, 13768, 38272, 48604, 9752,  45852, 56016, 5704,  64216,
                20816, 49412, 10492, 55956, 28688, 62796, 39928, 48788, 12180,
                34004, 61376, 40740, 24864, 18056, 29036, 55604, 40216, 62920,
                20860, 8832,  35104, 18420, 5524,  30720, 55736, 50732, 19048,
                35212, 53648, 16104, 45948, 28052, 12312, 53956, 26288, 14876,
                55200, 28376, 40016, 2792,  54944, 64220, 1284,  58004, 29148,
                45912, 65480, 56288, 42740, 31536, 63568, 26512, 21028, 54056,
                36704, 47548, 6964,  60200, 38156, 65304, 15668, 48728, 11308,
                60876, 26080, 42576, 13276, 58416, 62404, 23876, 5796,  60204,
                15712, 40812, 18708, 44348, 37060, 64924, 52676, 25128, 61448,
                28348, 50612, 23884, 40004, 27128, 20656, 34412, 46988, 22752,
                55996, 16924, 33740, 61496, 48068, 35892, 23292, 10268, 56668,
                35468, 18124, 8448,  31264, 11628, 62244, 19412, 59296, 41672,
                14740, 28212, 38020, 9200,  26756, 1952,  38880, 28388, 62780,
                17980, 34624,
            },
            {
                57260, 8088,  24752, 54692, 17160, 35848, 12624, 29740, 53372,
                39716, 26740, 45312, 15744, 42764, 5356,  25724, 59316, 3220,
                38464, 6484,  57584, 13672, 43632, 60064, 8276,  3376,  37216,
                50996, 43052, 54780, 15536, 64140, 10216, 37648, 1328,  29348,
                12232, 44388, 22252, 4164,  38768, 49816, 7024,  41244, 61272,
                9360,  32560, 60036, 44528, 24400, 31496, 15464, 43372, 11912,
                33400, 16660, 8784,  19976, 2724,  46636, 15800, 9348,  61428,
                2236,  23448, 40752, 28956, 20764, 8888,  55136, 35500, 21460,
                57052, 17208, 37720, 53604, 31476, 45364, 39284, 55936, 48016,
                34860, 28576, 2812,  32124, 50548, 9044,  20436, 14328, 6352,
                41248, 17744, 33520, 10248, 61996, 43316, 7236,  65236, 10636,
                40292, 58972, 43740, 7968,  25500, 52296, 63440, 41848, 6972,
                52988, 44520, 64736, 49424, 55004, 34800, 2024,  50380, 31480,
                63940, 21800, 51204, 33364, 16480, 49916, 46340, 57592, 9952,
                43768, 30172,
            },
            {
                41340, 20808, 32396, 2144,  40692, 58824, 22020, 43320, 440,
                17872, 58940, 4588,  22208, 50940, 33352, 17964, 53192, 30460,
                21348, 16312, 51292, 31764, 112,   47724, 34356, 26032, 14224,
                58620, 1356,  28144, 39668, 47664, 22528, 42744, 59560, 48716,
                64960, 8484,  32128, 59364, 17748, 63756, 30704, 20536, 36516,
                45600, 23056, 13832, 19572, 48320, 9872,  37464, 60712, 6808,
                50204, 36636, 59532, 29372, 53288, 35232, 39332, 49480, 28236,
                43356, 13812, 57232, 16820, 52304, 45876, 24380, 40188, 2876,
                51168, 44400, 7760,  4084,  20956, 9204,  27280, 216,   12884,
                62708, 9416,  54164, 59720, 5260,  42144, 34096, 47564, 56648,
                1728,  44292, 59056, 5484,  48288, 16324, 54844, 25912, 52240,
                1428,  20316, 29744, 12040, 32280, 2264,  16564, 46396, 28060,
                604,   14604, 25324, 3496,  28624, 40460, 13348, 26424, 7544,
                55300, 1112,  12344, 60776, 42112, 21336, 7196,  24260, 15348,
                51256, 376,
            },
            {
                12336, 60948, 50072, 65396, 27104, 7168,  50708, 61612, 9956,
                48240, 34176, 30476, 64876, 38292, 1228,  57056, 9888,  43488,
                64632, 46964, 8908,  36924, 62832, 11824, 53104, 18476, 45688,
                31648, 24032, 7380,  57804, 32592, 13340, 26684, 7688,  21332,
                39780, 56060, 25456, 48004, 10080, 35452, 3244,  52256, 57360,
                384,   50256, 6360,  35004, 62212, 4756,  18660, 52784, 21288,
                25284, 248,   47404, 22696, 12436, 58632, 7092,  32936, 18028,
                55480, 34912, 63468, 188,   32616, 12520, 4912,  31268, 63160,
                14140, 33692, 24568, 64376, 48392, 15024, 52552, 33380, 19140,
                38008, 24292, 46472, 12508, 26968, 17240, 58392, 29276, 37552,
                22460, 12952, 53388, 36564, 29420, 2116,  38688, 13588, 35724,
                45612, 5508,  48928, 64468, 56956, 37608, 19968, 60256, 31908,
                21664, 58744, 37640, 43156, 16928, 22724, 57256, 45448, 37208,
                17832, 43684, 29760, 53804, 5012,  65056, 36520, 32580, 59804,
                38568, 26892,
            },
            {
                45868, 36120, 5044,  44032, 14832, 18836, 33000, 37328, 23908,
                14076, 41728, 54312, 8416,  12780, 47744, 23988, 36496, 14104,
                27460, 54428, 24084, 19116, 27004, 41596, 22196, 65348, 49184,
                10288, 61556, 44892, 19880, 2680,  62148, 53900, 34240, 3616,
                15600, 36340, 5236,  41568, 54688, 23896, 43764, 13116, 17304,
                27688, 65100, 42524, 53800, 25988, 57216, 46272, 28276, 41284,
                54800, 14624, 63312, 41796, 4136,  44684, 24668, 64896, 51408,
                10968, 6536,  26136, 49128, 43024, 59920, 56184, 27568, 47320,
                9944,  58508, 28908, 54820, 36724, 41800, 61956, 22500, 50032,
                42528, 4240,  63880, 21528, 39248, 53148, 3224,  10900, 62320,
                49772, 31488, 64260, 18924, 24500, 61104, 50244, 30436, 18460,
                60388, 55080, 27208, 15772, 42228, 4248,  54236, 12448, 40324,
                48956, 33780, 10572, 61108, 51164, 8324,  62976, 3672,  32784,
                60136, 47536, 23620, 39512, 19160, 10912, 28948, 44708, 3372,
                19780, 55932,
            },
            {
                16360, 9148,  23196, 29628, 56444, 47672, 4032,  28148, 55840,
                62744, 2756,  25548, 19708, 60384, 28788, 45024, 62340, 2316,
                33592, 5008,  39404, 58652, 3248,  56372, 6684,  28836, 3804,
                40940, 35768, 12560, 52016, 38220, 49504, 17044, 45924, 57424,
                27988, 51272, 19856, 62932, 14612, 28536, 61696, 46992, 8660,
                32088, 38668, 11068, 16012, 30628, 40112, 2004,  34212, 61460,
                10260, 31344, 38540, 26948, 18760, 54488, 14360, 1172,  21520,
                30116, 45504, 38792, 19628, 8408,  36652, 16156, 19212, 42080,
                22552, 38592, 992,   18328, 11860, 30776, 3276,  58580, 7616,
                54756, 31332, 15880, 35392, 48544, 61128, 24676, 45412, 19560,
                7588,  26656, 3012,  46600, 11536, 56584, 21564, 41828, 8460,
                33184, 23484, 39368, 9612,  21784, 50304, 25084, 8560,  63204,
                4544,  18648, 53588, 5748,  29916, 35728, 19868, 53280, 24896,
                14364, 8916,  3308,  34724, 58548, 48152, 52396, 13864, 63232,
                6700,  48468,
            },
            {
                31420, 58748, 52228, 38876, 10980, 60608, 40496, 51732, 7736,
                17152, 46296, 36140, 52176, 40696, 16352, 7068,  51544, 19852,
                41852, 60472, 12892, 49280, 44760, 31492, 14888, 38572, 55124,
                17164, 22912, 56888, 29112, 5292,  24504, 9052,  30420, 41376,
                60908, 10476, 46292, 1856,  33920, 7256,  20888, 39944, 59828,
                24360, 55652, 4120,  22240, 51232, 8604,  12940, 16992, 48792,
                3700,  57400, 5940,  50784, 61980, 32288, 37668, 47472, 41044,
                57876, 3404,  62172, 23576, 29772, 50648, 62684, 1844,  53852,
                7028,  50184, 45044, 61168, 6308,  25536, 46424, 16544, 28152,
                11512, 44864, 57528, 1972,  29996, 8316,  14708, 32904, 40900,
                52716, 16088, 42800, 38068, 32652, 5272,  44516, 3356,  64048,
                12680, 51536, 68,    44844, 33968, 59584, 30144, 36044, 44196,
                26984, 57104, 45728, 24136, 14788, 47116, 42256, 11040, 49580,
                40720, 64432, 56632, 26488, 15604, 832,   38108, 22572, 54624,
                34804, 25424,
            },
            {
                62176, 3492,  19328, 33912, 768,   25188, 13296, 20968, 43876,
                31928, 58600, 11576, 4828,  33032, 57788, 22472, 31000, 56188,
                48244, 8404,  29820, 17052, 35420, 51632, 9112,  60984, 33464,
                50728, 760,   64392, 15496, 34788, 43572, 63396, 19496, 64,
                13648, 23424, 31256, 53428, 38232, 57924, 49896, 2532,  34680,
                18700, 48700, 62736, 36560, 43776, 58268, 64616, 23504, 36996,
                20376, 45328, 16364, 35376, 10704, 7792,  27556, 60904, 16976,
                35828, 52088, 15084, 54748, 11836, 5816,  40072, 24912, 34632,
                64808, 15568, 32096, 21768, 51768, 56524, 34828, 40172, 65012,
                36788, 24812, 6796,  51640, 20160, 43904, 54948, 65388, 244,
                35084, 58840, 55392, 8736,  51336, 15716, 58476, 28384, 47580,
                17360, 36680, 62440, 56404, 13976, 47240, 6140,  17084, 51796,
                13488, 1880,  31292, 39068, 63944, 276,   58668, 28160, 5076,
                30856, 18372, 50636, 43000, 31768, 61340, 8804,  27768, 42728,
                11208, 40024,
            },
            {
                51036, 14308, 42560, 63612, 53560, 45228, 35600, 64756, 2088,
                38456, 22040, 27412, 63344, 49596, 316,   39000, 11140, 15104,
                36796, 23324, 63248, 956,   21480, 59244, 25204, 19656, 44336,
                27168, 8288,  41804, 47252, 54964, 11872, 58344, 38624, 49976,
                35380, 55540, 44012, 17504, 25776, 11108, 30236, 52956, 5948,
                12748, 45276, 28772, 2324,  19732, 32664, 26588, 42376, 51696,
                28976, 63752, 25108, 42912, 22392, 56036, 49604, 5140,  12240,
                25552, 7888,  33396, 41560, 58880, 45928, 31948, 13452, 48140,
                28320, 10760, 59256, 37496, 42840, 14028, 9444,  20828, 916,
                49356, 18916, 61916, 41312, 10568, 36992, 27948, 5616,  48696,
                23664, 12428, 20508, 27508, 61912, 24068, 34620, 39216, 54564,
                25584, 6372,  30616, 19148, 27648, 2848,  64640, 38672, 23220,
                61688, 41912, 20568, 11680, 54456, 33300, 16536, 38092, 55920,
                22420, 36484, 11916, 6840,  20444, 45892, 56132, 17492, 49556,
                1712,  21308,
            },
            {
                45012, 5988,  27932, 22340, 8380,  16912, 57208, 28708, 10336,
                53888, 48504, 14984, 42096, 9472,  24944, 44632, 65232, 5520,
                26628, 52324, 43340, 55580, 11084, 40632, 47048, 6064,  13048,
                62416, 31560, 20524, 25984, 2480,  32032, 22444, 6420,  26380,
                62112, 8188,  3996,  65376, 48316, 15408, 43040, 63704, 26520,
                41276, 59148, 7844,  14764, 54272, 46684, 6636,  1376,  55896,
                7916,  12124, 53576, 928,   59760, 39192, 19736, 31180, 43308,
                63776, 46852, 18640, 848,   26880, 20340, 61268, 3268,  57080,
                43692, 19928, 4512,  26988, 2008,  61668, 47628, 53612, 32464,
                58820, 13236, 27112, 32020, 56196, 60512, 22148, 17532, 39488,
                29136, 63132, 3820,  47812, 41396, 7016,  12364, 20380, 1784,
                59476, 49644, 43280, 8648,  52868, 40788, 20928, 55356, 9648,
                34132, 49660, 59472, 7876,  48420, 25944, 21244, 8568,  63036,
                46316, 1308,  54484, 62540, 40276, 25020, 4008,  33320, 63652,
                29720, 57492,
            },
            {
                18096, 32812, 58396, 37896, 47412, 31060, 4904,  50008, 18352,
                60128, 6276,  30916, 55688, 19360, 35176, 52808, 58920, 32444,
                46676, 18748, 4528,  33516, 27248, 64348, 3164,  35940, 54364,
                38808, 48944, 11000, 60544, 37308, 17772, 53024, 47004, 15184,
                42372, 29392, 21104, 40148, 59228, 696,   22868, 36964, 16632,
                55000, 32320, 23768, 50448, 61872, 11312, 38584, 60284, 15396,
                32208, 40328, 48192, 30148, 14052, 46016, 3456,  53088, 58408,
                21652, 29088, 56116, 49484, 36500, 9740,  51848, 22980, 35800,
                8184,  54552, 40372, 49740, 33852, 18216, 29212, 23992, 42244,
                4340,  45776, 38124, 16664, 1316,  46504, 12972, 52404, 59580,
                9824,  44788, 37508, 31112, 17876, 53160, 63668, 45400, 10164,
                32944, 14672, 37812, 23880, 61052, 11588, 31968, 44936, 28964,
                5568,  16260, 36860, 28524, 44104, 4092,  60188, 51952, 42852,
                15504, 33720, 28828, 48460, 14584, 52048, 36684, 58976, 13980,
                7768,  37152,
            },
            {
                55204, 12236, 49360, 2764,  61736, 13924, 42660, 23932, 33540,
                40860, 25512, 45436, 2836,  60480, 13540, 27808, 17444, 1752,
                40056, 61952, 13616, 38844, 49736, 15404, 30608, 57780, 23264,
                16880, 5204,  57020, 51136, 7832,  44628, 64236, 3772,  33360,
                57508, 52148, 12348, 36136, 32612, 8932,  56588, 46432, 3132,
                9988,  20276, 36220, 40336, 30184, 17384, 22796, 34948, 45056,
                19060, 61500, 5172,  34696, 18040, 65308, 24764, 33880, 9220,
                2564,  39432, 13868, 4924,  65032, 44396, 14828, 39568, 29932,
                17016, 63748, 24664, 12576, 57436, 8368,  63532, 6184,  15916,
                55812, 9292,  63836, 50136, 6516,  25848, 35456, 42784, 2576,
                32412, 51004, 13972, 57880, 720,   35992, 49052, 29252, 41008,
                22032, 65516, 2460,  57044, 34636, 47884, 15132, 680,   58828,
                52640, 25044, 2548,  65264, 18148, 40236, 35144, 12768, 3260,
                26360, 59724, 9356,  23184, 5496,  30588, 10940, 43824, 22824,
                47796, 26212,
            },
            {
                64696, 8708,  40908, 19660, 26024, 36060, 52252, 62504, 1576,
                13016, 36228, 64436, 21608, 38296, 47920, 7600,  41764, 22168,
                56536, 9212,  24612, 53836, 6928,  43796, 19028, 51476, 1840,
                45520, 34220, 28728, 40800, 23724, 13876, 28256, 39104, 9680,
                24244, 17896, 45908, 5120,  25352, 51000, 18888, 29000, 60692,
                51288, 44288, 65024, 108,   5592,  48876, 58404, 27668, 9132,
                50740, 22232, 26224, 57908, 7300,  37528, 11608, 41680, 16372,
                51244, 35292, 59288, 23376, 30580, 54144, 5628,  59492, 49080,
                448,   45484, 31248, 53140, 22520, 44096, 39084, 36088, 52144,
                30124, 20256, 34512, 23308, 53692, 62800, 30500, 7628,  56752,
                19692, 25408, 6008,  21864, 60548, 25864, 15436, 4072,  55716,
                50880, 27308, 46172, 17404, 4936,  51344, 26640, 63792, 18856,
                40568, 46508, 56728, 13788, 50384, 55432, 23544, 29900, 49104,
                38532, 19776, 44220, 57976, 39304, 65484, 19508, 552,   53636,
                39608, 3892,
            },
            {
                51652, 23792, 30104, 60152, 53860, 10592, 7076,  20856, 47220,
                58172, 8944,  16568, 50520, 5284,  30212, 55396, 11620, 50644,
                29144, 36336, 60248, 31704, 21372, 58856, 10184, 27976, 41504,
                65132, 14188, 19652, 508,   61852, 35532, 55808, 19972, 59788,
                48684, 1156,  61572, 55384, 42000, 63088, 13040, 39540, 34104,
                6760,  27524, 12224, 56224, 25304, 42004, 53272, 3348,  64500,
                39092, 1636,  54784, 43512, 51356, 28344, 47284, 55672, 63124,
                26268, 45732, 10964, 42164, 17520, 34100, 25544, 19600, 11892,
                61708, 36640, 5028,  15832, 47188, 2660,  59668, 11520, 25112,
                60688, 40548, 3596,  43516, 11364, 38892, 20668, 14620, 47508,
                64704, 40164, 54796, 46592, 42416, 8904,  33188, 61544, 18540,
                7464,  12640, 39884, 30456, 21264, 42140, 8116,  37684, 32576,
                11324, 21748, 34644, 8920,  31544, 5400,  61648, 10012, 17412,
                64340, 53336, 2160,  35020, 17012, 55196, 47172, 27596, 61280,
                16180, 35160,
            },
            {
                45756, 14216, 43624, 148,   16652, 46108, 31792, 39128, 54720,
                29300, 43060, 53116, 33064, 24632, 62164, 44092, 34360, 64136,
                4304,  48488, 16780, 560,   47324, 37388, 62040, 33248, 7268,
                25464, 58256, 47792, 53396, 31240, 11116, 50308, 2972,  41428,
                30748, 37188, 27656, 14756, 21636, 31252, 5876,  48924, 23176,
                15088, 58188, 18576, 46124, 35264, 14472, 20608, 31148, 12052,
                47476, 36532, 14636, 9940,  20700, 60496, 548,   22804, 6400,
                32276, 19704, 61408, 7924,  48704, 1460,  43408, 55496, 28224,
                40896, 21324, 58008, 9844,  35112, 28036, 19908, 50292, 92,
                48080, 13720, 27528, 58536, 17376, 49076, 60168, 27124, 36240,
                10588, 16436, 34296, 28520, 11996, 56428, 39032, 23020, 44300,
                36184, 58244, 62656, 9992,  54680, 60756, 23668, 56924, 48912,
                6732,  62568, 43676, 26760, 38920, 20660, 46284, 41824, 56324,
                7128,  30844, 13020, 51052, 26200, 4864,  33628, 41928, 9520,
                31548, 20552,
            },
            {
                55864, 5476,  33588, 38480, 56844, 27600, 65344, 14948, 3692,
                18544, 26340, 568,   12008, 40600, 19188, 2228,  15168, 25968,
                20492, 43424, 12340, 40884, 26796, 5432,  14572, 54580, 50316,
                11560, 36828, 4836,  43452, 17540, 5968,  46036, 22744, 15632,
                64752, 6652,  53508, 9804,  34884, 44896, 1384,  54396, 63828,
                42592, 38320, 52780, 31812, 9668,  63328, 6392,  44184, 55256,
                17136, 28932, 62588, 32848, 39984, 16792, 31136, 12916, 49684,
                37324, 2020,  52420, 28380, 57740, 37780, 64064, 8872,  32572,
                6224,  51472, 26304, 42428, 64352, 54728, 31868, 16756, 43044,
                33568, 65220, 8264,  46144, 31692, 1904,  5604,  54464, 44624,
                1088,  52492, 4020,  62488, 19240, 50736, 1716,  29988, 53916,
                5088,  24344, 33344, 1216,  47016, 35780, 12544, 3388,  16488,
                29432, 54112, 1804,  58224, 15624, 52744, 488,   33524, 24472,
                36580, 47448, 21752, 41316, 8612,  60108, 23272, 13408, 49668,
                2580,  58852,
            },
            {
                10784, 25824, 63128, 50112, 21712, 4604,  41984, 23628, 49676,
                34904, 63308, 59192, 46448, 56032, 9080,  51628, 58700, 37764,
                7420,  54856, 33548, 63664, 56868, 22924, 44128, 17956, 39204,
                21436, 29836, 60664, 24180, 39464, 63804, 26816, 58360, 33648,
                12608, 43460, 20076, 49508, 60992, 57268, 17940, 26196, 9328,
                30280, 1888,  5368,  22428, 59072, 50920, 39848, 25760, 34092,
                58524, 23024, 6148,  49960, 4080,  54252, 44732, 61984, 41028,
                56996, 15244, 24404, 40268, 13076, 21904, 16032, 50272, 45140,
                14156, 60360, 18036, 48416, 3508,  13304, 39972, 56920, 5504,
                22496, 53760, 18776, 36448, 56340, 24320, 40744, 32796, 22144,
                61088, 30296, 24172, 37352, 7100,  45580, 64356, 13496, 48072,
                16964, 43036, 52516, 14660, 28492, 18432, 44820, 39516, 51668,
                24640, 41500, 19236, 48172, 60744, 11148, 27908, 62872, 14440,
                4344,  59160, 28140, 63332, 46084, 15816, 37804, 53884, 64072,
                28676, 40580,
            },
            {
                37172, 47912, 19004, 7852,  12636, 32876, 60468, 9832,  55784,
                7388,  38744, 21704, 15660, 36288, 28552, 22584, 45652, 30824,
                61364, 49880, 18728, 10020, 51120, 30944, 2984,  47992, 63432,
                1372,  55228, 9072,  33372, 13572, 54288, 36440, 8100,  51916,
                47260, 25232, 38536, 3716,  28704, 40108, 11984, 36612, 47588,
                20988, 61344, 49756, 26884, 37100, 3060,  15776, 61716, 8636,
                2636,  41532, 45980, 25348, 59176, 36100, 26856, 8824,  18564,
                29204, 46704, 63444, 3548,  31684, 53664, 5112,  35792, 24224,
                56012, 1704,  37304, 33132, 22848, 7952,  25684, 62724, 10580,
                29264, 38824, 2676,  51376, 15124, 9632,  63728, 49920, 12568,
                17848, 43228, 48556, 58064, 15116, 21476, 26292, 34076, 9764,
                60436, 27572, 20412, 49384, 64708, 5440,  31196, 58784, 63616,
                33868, 13080, 4956,  35440, 22904, 37564, 44432, 50348, 19596,
                52952, 39200, 10764, 888,   32600, 57212, 3124,  44036, 7356,
                21912, 17056,
            },
            {
                1452,  53352, 29704, 59088, 44796, 52368, 36952, 46756, 30068,
                13892, 44304, 31848, 6088,  49092, 64680, 3876,  39916, 12824,
                1304,  24004, 28808, 4104,  36752, 60888, 8952,  25184, 32324,
                42548, 16200, 45780, 49856, 20616, 1592,  44696, 18184, 29608,
                2320,  62552, 55908, 16448, 7996,  23436, 51484, 59664, 33124,
                16576, 55928, 13188, 43276, 17968, 46812, 29880, 21692, 48308,
                52704, 18852, 64784, 10604, 14284, 21640, 2192,  52684, 34280,
                4932,  10824, 43576, 35328, 60052, 47304, 27420, 62140, 19272,
                39776, 28992, 11708, 53844, 61292, 45564, 51240, 35268, 47632,
                59740, 44656, 26096, 60864, 30064, 42700, 19336, 6844,  38420,
                57028, 5288,  9984,  40524, 31568, 54232, 41972, 4192,  57076,
                39312, 2252,  36528, 7904,  40440, 55616, 25568, 212,   20712,
                8676,  46256, 61788, 30572, 7532,  56644, 3500,  31800, 8440,
                42960, 16504, 55544, 20564, 50044, 29084, 18964, 26056, 34764,
                47136, 61184,
            },
            {
                42056, 14488, 35516, 23332, 3484,  17892, 26176, 1248,  20584,
                62136, 51384, 2968,  25132, 57188, 10852, 32972, 53876, 17768,
                57340, 41752, 46680, 13732, 40128, 19956, 55968, 35832, 12060,
                59684, 27100, 4140,  61988, 28860, 57696, 11964, 40208, 60444,
                22096, 10888, 34404, 31072, 46616, 65224, 42440, 5768,  2544,
                44576, 39612, 6868,  34200, 62512, 54448, 11604, 57432, 37904,
                13372, 30652, 34472, 39148, 56820, 48036, 42876, 64256, 23340,
                59696, 54880, 21248, 7744,  17820, 11540, 41592, 2732,  9300,
                46952, 65144, 7048,  41252, 15372, 30056, 20344, 1296,  17652,
                14232, 7396,  21396, 12036, 48304, 34256, 58636, 25612, 28952,
                52152, 35024, 65252, 27260, 84,    60892, 36316, 18312, 50524,
                30672, 62792, 11664, 53460, 22544, 34156, 10496, 49084, 15708,
                38308, 55020, 26116, 50876, 17932, 40912, 12592, 59992, 26904,
                65028, 34808, 24800, 45264, 36432, 9312,  62992, 51280, 11424,
                56612, 31936,
            },
            {
                6156,  64660, 9564,  56152, 40220, 63712, 14664, 58128, 41516,
                10992, 27432, 59772, 40288, 16644, 43496, 20292, 27096, 47860,
                8576,  34048, 65228, 53512, 26052, 44944, 15672, 52412, 6492,
                49140, 19356, 40592, 34784, 52364, 6556,  24476, 32636, 54916,
                6176,  42544, 50468, 58648, 240,   20684, 14880, 28044, 53084,
                24140, 64160, 29028, 9820,  23368, 1672,  32996, 5680,  43144,
                26964, 460,   53816, 6752,  28744, 17356, 31860, 12432, 39508,
                48844, 15608, 37984, 51496, 25296, 56676, 30636, 52300, 58596,
                32200, 22048, 50180, 26752, 58116, 4404,  43132, 38000, 55340,
                32248, 57496, 40504, 64368, 5792,  500,   53996, 15648, 47100,
                2200,  23136, 14196, 19556, 50220, 11936, 8252,  44476, 22188,
                14160, 25224, 43732, 58656, 16808, 46192, 61656, 42224, 28716,
                58084, 2860,  43264, 14716, 64420, 24708, 46092, 21220, 48376,
                1628,  30504, 6320,  61684, 14384, 4592,  42160, 38512, 15152,
                4144,  24904,
            },
            {
                20304, 49672, 27148, 43996, 31556, 48116, 6612,  34272, 53976,
                37300, 17984, 47504, 8312,  52900, 35712, 59128, 5020,  62492,
                37200, 15824, 21388, 6836,  58284, 180,   29660, 64208, 23328,
                37632, 56840, 10364, 22960, 14912, 42180, 62856, 48840, 15656,
                37336, 18972, 26672, 12968, 39344, 35960, 55648, 32136, 11676,
                48184, 35852, 19912, 58372, 45092, 40060, 26048, 49252, 59352,
                16616, 61116, 45536, 20268, 62528, 4168,  51312, 7912,  25976,
                176,   32504, 27832, 64940, 2064,  44452, 21444, 38204, 16620,
                12900, 43400, 360,   18792, 34656, 9724,  52408, 64116, 24704,
                4788,  50344, 34952, 27388, 45428, 22836, 37316, 63064, 10736,
                40808, 59616, 45644, 33268, 56136, 47344, 24768, 64144, 53132,
                6628,  47516, 32380, 3264,  28248, 6268,  13364, 35964, 51556,
                22260, 32344, 10728, 36540, 796,   53216, 33480, 38032, 15472,
                57376, 51440, 11748, 39636, 53340, 57948, 30836, 23388, 60440,
                53008, 45284,
            },
            {
                59676, 37320, 17280, 876,   11944, 20980, 28724, 49704, 23440,
                2132,  64448, 33408, 21452, 29108, 380,   14164, 50464, 24776,
                44332, 2120,  31852, 48624, 11660, 33476, 41432, 4920,  46012,
                14068, 31600, 864,   65384, 47688, 36912, 9504,  27544, 1076,
                46388, 63416, 54028, 8956,  24680, 49812, 62316, 19316, 8692,
                60216, 15628, 3452,  49944, 53356, 15304, 65080, 19340, 8860,
                35680, 50432, 24424, 9512,  41624, 37176, 55916, 45008, 19724,
                57960, 42684, 5268,  47668, 34112, 14592, 62444, 3952,  26472,
                35624, 61720, 56508, 38956, 46604, 60008, 22712, 12580, 28560,
                42464, 19752, 9452,  16920, 51872, 13456, 32092, 43868, 20836,
                30848, 54652, 7640,  38064, 4872,  29672, 16244, 33600, 1596,
                40640, 59872, 19480, 37604, 51252, 56700, 19892, 3640,  64188,
                7320,  18412, 48456, 58660, 29060, 6204,  56180, 10172, 4540,
                41148, 23232, 46360, 17508, 27060, 20932, 44392, 272,   18172,
                34340, 28852,
            },
            {
                13184, 7264,  54544, 61884, 38656, 52880, 60368, 8816,  16100,
                45268, 5456,  55116, 12692, 60788, 46148, 39132, 30920, 9664,
                54656, 60016, 28096, 39124, 61144, 17988, 50812, 20824, 27912,
                59120, 42904, 53204, 18080, 30356, 5360,  20184, 51776, 58348,
                23392, 4624,  30252, 45124, 59628, 3128,  6416,  46280, 40092,
                54940, 26452, 38440, 6112,  30380, 12176, 36840, 55564, 23000,
                4964,  39872, 12880, 57556, 33056, 22452, 14640, 61588, 30684,
                52136, 13060, 60280, 19196, 9900,  40748, 49344, 8040,  55416,
                51368, 6240,  24348, 12084, 30888, 17112, 7176,  33908, 49256,
                2452,  62376, 54548, 38428, 59932, 3816,  56776, 8284,  51088,
                3464,  26464, 17416, 63636, 21872, 42432, 58208, 38544, 27320,
                55448, 10316, 15204, 65452, 41228, 23728, 32068, 45160, 26284,
                53784, 41052, 62064, 23244, 44816, 19848, 49404, 26392, 63548,
                18976, 35980, 59500, 2128,  33660, 49204, 7632,  65280, 47748,
                9792,  40028,
            },
            {
                51620, 22748, 33228, 45584, 25116, 3832,  35912, 43092, 56364,
                26396, 30444, 38372, 42540, 24428, 6792,  64816, 21920, 42192,
                18756, 12788, 52132, 23608, 8016,  55776, 35476, 62076, 9880,
                49900, 4452,  35996, 26492, 55852, 44448, 59924, 31964, 12080,
                35324, 40888, 15212, 33840, 18048, 42792, 29892, 34612, 23316,
                732,   43548, 32832, 61812, 22104, 42108, 1040,  28600, 45204,
                32368, 63492, 27344, 52340, 1280,  47808, 28132, 3384,  36320,
                8876,  39188, 23084, 53868, 29464, 57372, 23844, 32472, 45152,
                19936, 29780, 48496, 3516,  62296, 54584, 44020, 39964, 59144,
                15252, 46392, 29684, 6640,  24116, 40984, 28232, 18532, 61876,
                36856, 12956, 44252, 51960, 10620, 61252, 3564,  13092, 20692,
                49680, 35276, 29784, 572,   8496,  48688, 59096, 39112, 10160,
                15020, 34732, 4508,  12284, 39244, 15936, 61444, 30392, 43880,
                53580, 7188,  28792, 62908, 10196, 54924, 13536, 36408, 26368,
                56220, 3504,
            },
            {
                42768, 29776, 58248, 9320,  14324, 63276, 18668, 31944, 12852,
                59164, 51076, 10024, 18904, 48764, 55764, 16868, 34292, 57980,
                6136,  36108, 46076, 2996,  42748, 25804, 13704, 1824,  38940,
                17292, 24036, 8280,  60860, 12688, 3512,  38560, 17060, 43284,
                56264, 8120,  51564, 64148, 22476, 12272, 57032, 15896, 64840,
                51740, 13312, 57856, 17708, 46560, 7796,  50600, 60272, 11260,
                48412, 2868,  16064, 43376, 18584, 65368, 10988, 50116, 24544,
                17888, 63988, 6396,  45628, 36936, 736,   17132, 63476, 39852,
                10488, 15360, 42344, 36492, 27252, 50792, 996,   19304, 25876,
                36748, 11404, 21180, 32720, 48440, 64920, 14928, 34392, 48916,
                23336, 60164, 32456, 808,   28528, 35784, 50576, 45668, 7304,
                63056, 23408, 46748, 54412, 26768, 12516, 17180, 2096,  60768,
                47532, 27752, 56440, 31328, 51780, 7412,  35740, 2852,  11636,
                32952, 14200, 50420, 43212, 37944, 23836, 58540, 41440, 31992,
                15780, 61224,
            },
            {
                18492, 6216,  48776, 20488, 41808, 27820, 47420, 6460,  39624,
                20904, 3240,  62016, 35900, 1688,  27488, 11376, 51684, 2340,
                29592, 62612, 20884, 16160, 63856, 47720, 30344, 54596, 44616,
                32976, 63608, 46440, 40400, 21504, 50480, 25196, 64564, 2392,
                21160, 28028, 48296, 1480,  54356, 38116, 27296, 49392, 8148,
                21004, 29360, 9876,  54180, 25400, 63808, 33732, 17088, 38140,
                20416, 54832, 30752, 60364, 6020,  33984, 39408, 58832, 43784,
                54592, 32624, 27140, 14268, 50808, 59964, 11940, 27840, 5224,
                59280, 52904, 64316, 20800, 8768,  14208, 32424, 65364, 5216,
                51428, 57300, 44108, 740,   55424, 10256, 45604, 1828,  42100,
                5764,  55660, 39264, 47612, 24356, 14968, 19200, 56600, 30716,
                42632, 4456,  17712, 61156, 43672, 35472, 55492, 30024, 22400,
                42648, 19492, 1276,  64668, 25504, 42272, 59408, 22608, 47648,
                57784, 25588, 20612, 3776,  16272, 30072, 5416,  19564, 1660,
                50108, 24716,
            },
            {
                37044, 64284, 1884,  35120, 52128, 56788, 24,    50476, 64212,
                24852, 46444, 15532, 53144, 32468, 63168, 37980, 45396, 25636,
                48464, 39840, 56228, 32608, 10668, 37224, 6540,  22316, 57612,
                11268, 51940, 29480, 15200, 33432, 54620, 10036, 29712, 46176,
                61256, 39736, 11492, 25740, 6932,  45956, 61332, 3932,  40976,
                36716, 47840, 4992,  35256, 40196, 13808, 55940, 4316,  26120,
                58916, 41140, 9252,  36880, 25016, 46460, 20516, 7136,  15552,
                1060,  47972, 41872, 4428,  20716, 34196, 43720, 48268, 37260,
                23228, 1200,  33424, 57280, 39704, 23900, 45704, 56040, 29980,
                8512,  23740, 63180, 17600, 36188, 26568, 20640, 53984, 29804,
                11348, 16268, 20104, 57992, 6104,  53688, 39992, 9552,  25676,
                59392, 13668, 39500, 32672, 21200, 5832,  63900, 37932, 50660,
                6600,  53536, 36908, 9808,  45992, 13816, 55012, 17972, 38404,
                432,   64712, 40224, 54172, 48520, 62188, 45256, 52532, 63176,
                10800, 46088,
            },
            {
                13652, 53584, 40064, 16284, 30936, 11900, 23088, 37148, 9144,
                33804, 41420, 29380, 5124,  43968, 22464, 7504,  15584, 60264,
                9228,  14212, 3684,  28272, 59416, 53384, 17784, 41604, 27332,
                2620,  20128, 5964,  56852, 260,   48112, 36648, 6496,  14300,
                53004, 18416, 32548, 58368, 35704, 20080, 10388, 32084, 25036,
                58864, 63224, 44872, 19900, 2456,  31208, 22296, 43176, 51748,
                6736,  13928, 22820, 49720, 54052, 12936, 56492, 31524, 63108,
                35736, 11440, 56768, 62256, 24760, 54668, 3048,  19460, 56812,
                31056, 46512, 17420, 6108,  49700, 61204, 11124, 37540, 16540,
                42280, 34184, 13280, 40152, 50964, 7324,  58724, 38368, 63932,
                51380, 27448, 8720,  62580, 31304, 43564, 64972, 33532, 1024,
                37052, 53840, 10180, 49376, 2784,  52448, 25372, 9140,  13636,
                32968, 61300, 16468, 49224, 21468, 33412, 4948,  29180, 52648,
                44928, 8056,  34980, 11432, 26664, 33488, 9128,  22004, 38860,
                28108, 32696,
            },
            {
                3316,  23284, 27080, 61804, 7456,  60056, 44756, 17500, 58260,
                13724, 55076, 20016, 59752, 12276, 57000, 41036, 31528, 54868,
                23224, 35060, 50656, 44556, 23980, 1096,  49708, 64792, 14560,
                59248, 35504, 42600, 61848, 27048, 16344, 42088, 59448, 24124,
                35052, 3764,  44256, 62940, 15592, 41968, 52664, 56252, 18176,
                2072,  15256, 27236, 52712, 59504, 49248, 10060, 62196, 28448,
                35584, 45664, 64388, 1812,  29356, 4576,  41488, 26156, 50580,
                21880, 28684, 18196, 39676, 9424,  30428, 64916, 15100, 8076,
                51432, 12724, 26108, 43872, 29352, 4132,  20180, 53036, 2000,
                49064, 58928, 28408, 3916,  61136, 31184, 12308, 24148, 3296,
                33796, 45940, 41360, 36212, 12828, 2856,  21632, 16592, 51576,
                46304, 22784, 28180, 57100, 15964, 41268, 46580, 18780, 58736,
                44648, 24552, 29404, 57152, 2720,  41032, 60332, 9280,  24484,
                15320, 31068, 60144, 18344, 57364, 2348,  15068, 43532, 55712,
                6924,  59428,
            },
            {
                42176, 57532, 10516, 46620, 21064, 40768, 33060, 52920, 28268,
                2220,  48588, 7828,  26028, 50676, 17800, 47996, 620,   19820,
                43244, 64484, 5720,  19032, 40252, 34204, 9364,  31244, 38096,
                45368, 50936, 12496, 23052, 38832, 63860, 19864, 31484, 49292,
                57272, 8796,  28700, 22884, 49136, 336,   29280, 13256, 43844,
                50616, 32984, 11552, 42276, 7472,  37004, 15980, 47128, 104,
                18572, 32532, 56832, 15900, 43992, 62264, 17844, 9480,  38508,
                2484,  59564, 52488, 6688,  36064, 49932, 42300, 27256, 39372,
                60508, 35548, 63288, 10236, 54644, 33964, 41132, 26556, 62508,
                21932, 6072,  54300, 19700, 44992, 15612, 49228, 43084, 17808,
                59388, 22068, 4776,  52876, 24840, 49160, 57472, 29072, 61236,
                5984,  18824, 64516, 31388, 7696,  62120, 34276, 27804, 196,
                40080, 5296,  11960, 38348, 63236, 26536, 47968, 35580, 62688,
                49800, 21356, 42516, 6356,  46184, 38148, 60720, 25856, 35252,
                17232, 47772,
            },
            {
                30388, 50456, 4516,  37396, 54812, 2648,  26084, 6248,  61604,
                43540, 36360, 65472, 39672, 30576, 35408, 4868,  61152, 37492,
                53284, 27244, 11172, 55640, 60340, 13440, 46696, 21136, 3440,
                24788, 7340,  30744, 47104, 8896,  4100,  55276, 11088, 1964,
                39360, 46760, 12992, 54260, 5408,  39524, 64664, 34488, 23632,
                6384,  38660, 55028, 22992, 29556, 65168, 25096, 39040, 53904,
                60412, 10844, 25872, 38036, 21576, 34536, 52864, 58380, 47192,
                14580, 42624, 32184, 45832, 16340, 22060, 11736, 53312, 1664,
                45296, 21408, 3284,  38564, 18380, 59688, 14776, 46528, 31924,
                12048, 36852, 47340, 25248, 33708, 1492,  57384, 36504, 8292,
                47684, 13916, 61076, 29680, 18480, 45108, 7724,  38996, 11852,
                42400, 35596, 1612,  44504, 38120, 21780, 11240, 55304, 49860,
                65208, 21236, 53248, 31840, 7652,  15756, 19788, 43180, 12092,
                5212,  56168, 27660, 52280, 23200, 29920, 50972, 11544, 372,
                64628, 20916,
            },
            {
                14108, 34448, 18448, 29140, 64872, 14924, 48020, 19720, 11044,
                31624, 16056, 21744, 1408,  10092, 62820, 23672, 28896, 12740,
                7064,  16856, 47248, 30648, 4232,  26436, 52060, 62152, 56072,
                15924, 60960, 53712, 18472, 35140, 50100, 43636, 27784, 21832,
                65204, 17776, 37524, 60964, 26320, 21048, 9428,  57752, 47296,
                62228, 19620, 60148, 1160,  17560, 56216, 3860,  12604, 21116,
                41816, 7684,  52388, 5780,  49048, 11816, 3228,  30512, 23344,
                8172,  64620, 25492, 3844,  61320, 57596, 6236,  33728, 24832,
                16256, 30136, 57928, 50856, 23364, 48368, 2832,  8924,  56748,
                43600, 17228, 64860, 10188, 41836, 61964, 21024, 52524, 30100,
                25704, 328,   55324, 37852, 10480, 63708, 34736, 26232, 15312,
                48644, 55156, 24576, 13840, 60260, 48048, 3584,  30468, 15228,
                34988, 46248, 17868, 43520, 50752, 55984, 2080,  58732, 32604,
                39380, 1120,  36296, 13716, 62088, 3848,  18008, 46868, 40848,
                54016, 7860,
            },
            {
                52264, 62628, 24980, 44024, 12200, 50976, 35388, 57112, 39100,
                24040, 51992, 57428, 46708, 54568, 15060, 44416, 51448, 58460,
                33544, 42328, 38392, 63368, 20480, 36308, 41280, 8052,  28532,
                33704, 40428, 4896,  57956, 25812, 13356, 32564, 59748, 15544,
                51216, 30656, 6880,  33348, 42992, 52064, 17032, 37348, 3236,
                26600, 8984,  45516, 35152, 48344, 32328, 44228, 58772, 34520,
                28876, 46644, 63244, 30984, 40376, 59848, 27348, 45136, 36512,
                55816, 20032, 12392, 48104, 28008, 38212, 43536, 62184, 55620,
                47440, 8696,  41920, 13644, 6624,  28468, 64268, 38324, 23788,
                228,   29216, 51468, 6908,  54852, 28004, 5052,  11320, 40308,
                65256, 34304, 49588, 15844, 41900, 2032,  55880, 51224, 3400,
                30440, 59440, 9628,  53020, 27212, 18016, 57468, 42964, 24008,
                9780,  2396,  61436, 24960, 10956, 36628, 29636, 23152, 53652,
                17764, 65360, 48044, 10068, 41304, 33656, 57064, 22116, 31744,
                27156, 37732,
            },
            {
                45208, 5652,  39640, 684,   59336, 22664, 41904, 3408,  63676,
                45156, 4724,  13156, 32512, 41996, 26624, 38180, 8812,  21360,
                49112, 1776,  23804, 50368, 15240, 58216, 424,   18288, 43892,
                49220, 11020, 21660, 44552, 61740, 804,   52824, 40764, 5676,
                44700, 24728, 56020, 10944, 59024, 2412,  46052, 31640, 12656,
                53308, 30544, 40708, 13532, 52204, 9640,  27108, 6024,  50680,
                14700, 23476, 17368, 964,   55444, 19672, 15192, 62956, 6100,
                50848, 39936, 34464, 54284, 76,    18396, 30876, 14116, 4548,
                19836, 63876, 36668, 32420, 55380, 44532, 35080, 19396, 52940,
                60452, 40672, 34716, 14204, 23312, 38912, 32884, 16812, 55924,
                43560, 19380, 6608,  27816, 59100, 31900, 19112, 23004, 62804,
                39912, 20652, 33340, 5000,  41124, 35356, 7212,  51332, 37912,
                58528, 54072, 28612, 39796, 6028,  64156, 46524, 13796, 8620,
                44716, 25068, 31312, 20452, 53896, 7440,  63384, 14480, 4652,
                60596, 10916,
            },
            {
                17104, 58388, 48680, 9596,  33468, 30268, 7980,  17324, 27084,
                8972,  29064, 59196, 19264, 3028,  60100, 5752,  17916, 64880,
                29992, 56412, 5312,  9856,  32588, 45904, 12252, 54980, 23828,
                65440, 2556,  37196, 30016, 16128, 38076, 23500, 9600,  35496,
                62584, 1152,  47752, 19804, 14808, 28032, 63508, 21960, 50020,
                57144, 16296, 64168, 4324,  22368, 61360, 19808, 40184, 64088,
                2952,  58200, 44172, 37084, 25200, 7936,  42384, 33088, 1336,
                17352, 29408, 9208,  59420, 23520, 51616, 9972,  49008, 40556,
                27628, 52644, 636,   25264, 60868, 10768, 15972, 49844, 26688,
                12772, 4796,  48560, 18580, 58288, 46948, 63000, 50488, 3612,
                23076, 9696,  53796, 46232, 24168, 5480,  47356, 10932, 44112,
                7292,  17096, 46572, 50824, 63448, 23148, 12652, 61868, 19080,
                32168, 4672,  14176, 49584, 33892, 20812, 52356, 41436, 27928,
                60488, 6768,  56448, 16524, 45300, 24468, 38552, 49796, 43216,
                55488, 23540,
            },
            {
                32628, 28488, 21560, 52732, 15760, 62420, 55744, 46936, 53608,
                34084, 50016, 40704, 36160, 23168, 48516, 53408, 34524, 45492,
                14356, 25784, 60848, 54280, 39792, 22436, 63048, 30552, 36004,
                14532, 26876, 51200, 55372, 6312,  63744, 47908, 19432, 57844,
                13760, 28924, 41236, 37128, 50512, 34712, 4940,  39896, 43324,
                7020,  24264, 37452, 28300, 42832, 55104, 15764, 47168, 36536,
                10016, 32392, 51792, 12584, 62472, 48512, 53632, 22932, 58032,
                47596, 63696, 42220, 15520, 45148, 36332, 65356, 21776, 34916,
                59832, 11888, 16872, 46044, 21708, 5784,  42268, 3416,  33008,
                57408, 21412, 63736, 30784, 1164,  8544,  26300, 13584, 59464,
                36948, 32196, 61336, 39064, 12732, 52708, 37504, 60676, 26680,
                35884, 57084, 28444, 532,   16096, 30048, 45528, 2028,  26944,
                47876, 42380, 22988, 60652, 16508, 428,   59764, 4212,  37776,
                50064, 34896, 3424,  61220, 36644, 1300,  29256, 9660,  19172,
                35644, 1724,
            },
            {
                41620, 12988, 55240, 37016, 45724, 25656, 1968,  37384, 20200,
                11844, 61568, 16388, 7204,  63984, 12376, 28260, 224,   40356,
                10676, 36932, 19144, 43752, 27916, 2956,  48052, 5660,  52720,
                8600,  59368, 45508, 20384, 12128, 43012, 27040, 31660, 53684,
                4648,  21696, 61308, 8356,  54000, 23848, 60088, 10204, 18484,
                34164, 408,   59272, 48780, 7720,  33444, 1436,  24712, 29456,
                55960, 41552, 21456, 3896,  34232, 28748, 10396, 38692, 13324,
                26220, 20880, 3488,  32336, 5624,  56380, 26896, 7648,  2184,
                43864, 31276, 49516, 57396, 39728, 29528, 53500, 62388, 46180,
                9616,  39328, 44408, 53448, 37120, 42248, 20420, 44876, 29120,
                48564, 15332, 1108,  20908, 64364, 33784, 16332, 1488,  53848,
                13424, 65528, 42780, 10836, 37760, 59812, 53772, 40492, 8428,
                63684, 11760, 36144, 55348, 44472, 26428, 31940, 18552, 11276,
                22052, 14848, 42504, 26872, 48980, 13208, 53224, 59516, 26296,
                47232, 65176,
            },
            {
                7772,  61472, 3044,  18992, 6500,  40972, 13392, 60160, 23548,
                42848, 860,   25916, 54460, 44984, 31976, 20228, 58108, 61608,
                48012, 31504, 52316, 7448,  16736, 59936, 34424, 19632, 42352,
                38976, 17252, 32488, 1544,  34840, 56604, 7676,  16940, 39096,
                43716, 49696, 32732, 17568, 1756,  45128, 13400, 55732, 29384,
                62572, 45016, 20216, 53540, 11896, 39056, 57708, 61932, 13308,
                19164, 7284,  27384, 59140, 45332, 18228, 4892,  61284, 44536,
                7556,  37476, 51124, 61532, 12000, 39172, 16584, 53096, 57860,
                23596, 37888, 9188,  3088,  64832, 18676, 36988, 23188, 14736,
                28740, 7072,  25288, 15856, 11812, 60372, 34320, 2696,  7496,
                57608, 41100, 26620, 44620, 8564,  28760, 42412, 49400, 31128,
                21812, 6040,  24452, 55772, 48672, 3856,  21184, 33756, 16712,
                56884, 20432, 29952, 6648,  9464,  40676, 57688, 46960, 55032,
                63500, 30176, 52484, 10232, 21192, 64408, 32772, 40332, 3560,
                14732, 51456,
            },
            {
                37872, 25004, 47928, 34260, 64224, 29232, 51604, 32192, 5848,
                47828, 57040, 29872, 37548, 9620,  50772, 42692, 16636, 6904,
                22184, 2560,  62864, 13228, 38316, 50620, 9528,  25932, 56232,
                3864,  61576, 48900, 25216, 40232, 59856, 50540, 3184,  64372,
                25176, 9860,  56148, 27072, 64848, 31144, 38044, 48196, 25596,
                51008, 9476,  14668, 31608, 25888, 18092, 50224, 4960,  44908,
                53052, 48860, 64676, 15508, 39688, 51316, 56316, 30640, 34556,
                53112, 57352, 28228, 24064, 48608, 20116, 30768, 46660, 13820,
                63364, 19268, 52208, 26164, 34288, 12352, 56268, 828,   51396,
                60092, 35748, 56344, 4468,  29692, 49568, 54920, 24508, 64652,
                18392, 11064, 55108, 50584, 3792,  56436, 20036, 59040, 4396,
                39520, 51980, 34752, 18940, 31876, 13896, 25576, 49928, 36928,
                44724, 1464,  52844, 48248, 65076, 13428, 23624, 5632,  33792,
                43964, 1896,  59136, 39744, 5500,  45968, 17524, 8224,  56904,
                21860, 30408,
            },
            {
                18024, 10936, 57332, 43112, 21836, 10540, 58100, 17440, 39144,
                65064, 4280,  19500, 14860, 62632, 3900,  25412, 38288, 55328,
                27724, 44768, 34936, 24444, 56928, 29748, 44436, 62660, 11656,
                22548, 29400, 13880, 54200, 9908,  22220, 29168, 12764, 33788,
                46380, 15620, 37756, 5324,  42620, 20972, 6848,  16080, 2380,
                57648, 36348, 40728, 65456, 2596,  43388, 34900, 22604, 31004,
                38352, 296,   35720, 9644,  22336, 2508,  24888, 14484, 19424,
                1912,  10748, 16984, 43292, 976,   59940, 41600, 3656,  34068,
                28584, 5512,  41496, 15444, 48332, 7644,  44492, 31776, 40464,
                18064, 48884, 20760, 46276, 62328, 17704, 9968,  39560, 51968,
                35164, 30520, 23216, 62712, 36268, 13792, 24792, 10224, 45936,
                17432, 60392, 47216, 8488,  41172, 57236, 64084, 10352, 4804,
                61412, 23852, 33312, 17336, 37088, 28404, 51436, 39036, 16416,
                8928,  24828, 18796, 35652, 30728, 55312, 24128, 43808, 35204,
                60820, 45596,
            },
            {
                896,   50832, 27516, 15072, 4464,  49040, 44676, 7856,  27468,
                34704, 52440, 45588, 41080, 22832, 34816, 53696, 9028,  13960,
                49328, 57712, 5252,  42168, 21020, 2060,  15492, 32896, 51772,
                46564, 37748, 7132,  64036, 17852, 47028, 36740, 61160, 19920,
                52,    52432, 60576, 11948, 48732, 58164, 53276, 33324, 41980,
                21620, 4616,  28844, 55216, 47776, 58792, 8456,  63660, 11104,
                16904, 25240, 56880, 46744, 32260, 62364, 42008, 49824, 65104,
                39980, 47068, 32620, 63116, 35672, 9100,  55912, 50924, 10840,
                45548, 61508, 55016, 30328, 59540, 22216, 62812, 27016, 11204,
                5800,  65512, 33156, 2336,  40564, 22964, 32232, 14784, 45836,
                4492,  43436, 6532,  16060, 31512, 47488, 41000, 64056, 33640,
                26252, 12476, 1288,  62180, 27496, 5756,  43596, 29764, 54772,
                14500, 27336, 41740, 3696,  58912, 19952, 1092,  60568, 47948,
                28804, 51060, 57004, 13420, 61384, 60,    50280, 11400, 28580,
                5820,  54264,
            },
            {
                33148, 60184, 39752, 31652, 61312, 36632, 24520, 55376, 14460,
                21984, 10372, 31708, 55780, 1208,  59768, 46872, 30164, 65320,
                33256, 18384, 12016, 63796, 48820, 54348, 6588,  40628, 18332,
                724,   57452, 44016, 32212, 2684,  42668, 5716,  54632, 41456,
                57904, 23252, 29844, 35240, 18248, 24432, 9192,  63892, 47108,
                12424, 61616, 23764, 6692,  16172, 21000, 27940, 41272, 54532,
                60552, 43208, 29152, 6560,  53376, 11672, 36608, 5452,  28912,
                58788, 22700, 4552,  54136, 15128, 29468, 21980, 25668, 17620,
                39416, 21268, 1224,  36280, 42880, 4148,  17076, 49868, 55048,
                24340, 42920, 13628, 27828, 52636, 6944,  57500, 36,    27232,
                61216, 20628, 49304, 39340, 59444, 1196,  53256, 7124,  37456,
                57568, 30028, 44048, 22716, 37868, 52120, 18164, 22076, 46888,
                39384, 57868, 50248, 11060, 43820, 53660, 35128, 12548, 22352,
                64684, 3736,  41888, 46640, 27276, 15852, 38192, 62616, 19796,
                40844, 13272,
            },
            {
                44652, 22800, 3368,  9240,  53484, 19332, 28,    62692, 42520,
                50004, 6596,  61036, 12744, 28460, 17548, 5636,  20548, 41524,
                1516,  25936, 39720, 8716,  27616, 36292, 58948, 23972, 64540,
                35344, 26420, 20484, 52804, 27992, 50120, 15232, 25860, 31188,
                8424,  45172, 4296,  40636, 62464, 1036,  30368, 39012, 26456,
                17484, 52468, 44524, 32432, 36864, 12540, 51932, 1788,  33436,
                5564,  20464, 13568, 58876, 18088, 26868, 47844, 21224, 8556,
                43944, 12532, 26720, 38872, 49428, 5836,  64400, 37008, 60492,
                7624,  49012, 13472, 24604, 9920,  53436, 38780, 34580, 2040,
                58176, 37556, 8948,  59360, 44068, 36128, 63504, 47728, 37808,
                53808, 12500, 57388, 26836, 9468,  18912, 28656, 22180, 15160,
                2960,  50056, 54400, 16004, 33076, 12216, 60040, 32,    32156,
                8064,  19740, 5444,  62936, 30872, 25416, 8480,  40996, 55508,
                31760, 37156, 10608, 6284,  22132, 33832, 52788, 4044,  48124,
                25844, 63928,
            },
            {
                7536,  18740, 56592, 46780, 26248, 41040, 12512, 30352, 35956,
                57872, 18496, 25792, 48096, 37908, 51744, 44240, 11036, 36244,
                50692, 60684, 53492, 32252, 19464, 44840, 13064, 30596, 8892,
                49752, 14864, 11028, 61744, 34616, 22928, 63140, 11264, 38724,
                17468, 50260, 13852, 27696, 51712, 45952, 14588, 56128, 3096,
                34480, 59648, 10528, 50288, 39940, 63060, 45512, 15364, 24044,
                47544, 50980, 35364, 39860, 44572, 1252,  60172, 54520, 17168,
                35404, 51092, 56824, 19152, 10312, 45356, 41020, 2044,  52464,
                27380, 56756, 32172, 64688, 47052, 28668, 14048, 45740, 21036,
                30360, 16608, 47176, 24952, 19136, 11904, 29600, 21792, 8588,
                17448, 32712, 2416,  42720, 34880, 60932, 44312, 51056, 61892,
                41464, 20064, 9056,  64908, 3420,  49236, 42392, 36096, 53184,
                64320, 12856, 37512, 22816, 15968, 46816, 61404, 5060,  44924,
                15480, 20624, 48908, 63072, 39988, 59172, 8652,  31272, 56984,
                16304, 35932,
            },
            {
                28292, 49788, 37800, 15848, 64524, 33856, 51320, 5448,  46716,
                2148,  39756, 32808, 64536, 8004,  23996, 57276, 62128, 22384,
                14436, 4356,  43780, 15812, 960,   62028, 47552, 3140,  55072,
                42460, 60192, 4216,  40824, 8192,  56008, 1420,  47712, 58684,
                34140, 65296, 22120, 56836, 10368, 36596, 20528, 43068, 49020,
                7948,  29260, 19220, 5004,  26000, 3216,  57792, 29560, 38612,
                62468, 10508, 4000,  64796, 23516, 33716, 14092, 38944, 63656,
                30860, 456,   62500, 33884, 24900, 58848, 31216, 15948, 11548,
                34664, 43884, 17212, 5960,  59156, 19696, 56452, 7540,  63908,
                52272, 4108,  61600, 54372, 1924,  49524, 15724, 55176, 41772,
                25428, 65212, 52444, 23040, 13240, 55064, 4736,  32872, 11684,
                48024, 24968, 35808, 40360, 57488, 23984, 9484,  15500, 28076,
                24512, 48756, 32864, 52616, 2068,  56512, 33420, 18936, 27772,
                1388,  52620, 25788, 54776, 17628, 44848, 12572, 24384, 43120,
                1512,  53200,
            },
            {
                58252, 31384, 12120, 42860, 6604,  21548, 59260, 17148, 28000,
                23304, 53344, 4252,  15588, 42712, 34544, 2176,  31448, 47904,
                55704, 29272, 23840, 58324, 39168, 51676, 26132, 17288, 38244,
                21544, 28680, 46156, 16528, 37368, 19372, 44224, 28816, 6552,
                53124, 2188,  42152, 6984,  33120, 61192, 4160,  58884, 23104,
                37968, 54408, 42388, 61260, 22140, 53168, 35236, 8092,  18900,
                56068, 31712, 27708, 54884, 8752,  50324, 29736, 4288,  25024,
                46500, 7524,  15784, 42588, 3880,  53380, 20456, 47208, 58092,
                23348, 3756,  51524, 36800, 40612, 880,   25744, 32980, 42136,
                10380, 22580, 40836, 34720, 31884, 39800, 5064,  60732, 34956,
                3784,  46348, 7604,  38424, 48540, 16288, 26064, 39160, 7788,
                30660, 56284, 5068,  27408, 18636, 30556, 60912, 45348, 55344,
                2712,  40456, 44636, 59536, 28848, 13520, 37920, 49564, 62592,
                57936, 35436, 13688, 32352, 1956,  28636, 49768, 65448, 21072,
                39044, 10796,
            },
            {
                4792,  23912, 61172, 1240,  29624, 54828, 10416, 43416, 63212,
                13776, 60068, 44612, 21144, 54432, 12268, 26716, 16956, 7436,
                38056, 9732,  46664, 34776, 6716,  21140, 10624, 34144, 58204,
                5804,  32416, 53864, 25336, 59376, 51480, 32728, 14492, 24836,
                20244, 37412, 47292, 26384, 17936, 52968, 29632, 11728, 16464,
                64964, 220,   14120, 46980, 31872, 17204, 11740, 49068, 43108,
                608,   14556, 46384, 21404, 16916, 42052, 57600, 10684, 53064,
                40920, 22176, 48656, 28628, 61056, 12736, 36460, 7232,  28780,
                41328, 61624, 20960, 11032, 30800, 49456, 61972, 12820, 36352,
                50412, 27924, 14100, 7404,  64740, 26460, 57764, 44268, 11316,
                51096, 28332, 19248, 61504, 30208, 404,   64396, 57312, 18304,
                45640, 62624, 12908, 52740, 46776, 7228,  38528, 20964, 5248,
                58136, 17648, 10472, 6412,  20740, 42444, 9296,  24016, 6748,
                39600, 9828,  43288, 5544,  58332, 37616, 7108,  34812, 14868,
                47692, 62424,
            },
            {
                44380, 19716, 52692, 47892, 36792, 24932, 49476, 38556, 7444,
                34500, 48592, 9840,  30248, 58628, 39260, 63896, 52156, 41392,
                59480, 20072, 62860, 12844, 55316, 31516, 63640, 43468, 48812,
                13680, 65184, 412,   48476, 12168, 3628,  64164, 39760, 60232,
                11528, 30112, 62248, 14856, 49552, 39660, 24332, 45696, 32452,
                51068, 27436, 36376, 58432, 7244,  40468, 64336, 27044, 59756,
                23676, 41164, 61188, 37700, 2100,  47336, 61772, 19376, 36344,
                13380, 59620, 55284, 9136,  39504, 23716, 50712, 65312, 556,
                53740, 14524, 46372, 26916, 55184, 16016, 44752, 3312,  18604,
                60760, 45572, 56516, 20336, 51700, 9400,  17996, 30932, 22396,
                14644, 36768, 57632, 10456, 43396, 21400, 36276, 51948, 23824,
                2604,  15696, 33292, 42628, 856,   62896, 34576, 51264, 14080,
                30792, 35464, 62100, 26400, 50424, 65284, 54928, 17300, 46352,
                29824, 21464, 64452, 47676, 23372, 19256, 60600, 55120, 2816,
                26784, 33172,
            },
            {
                7780,  40280, 34660, 13836, 57900, 18020, 3352,  31472, 56712,
                19444, 26148, 37392, 504,   49716, 5888,  19312, 45552, 2880,
                32928, 27228, 4668,  49560, 41956, 28308, 3604,  52996, 8344,
                24156, 40432, 18864, 36168, 28068, 21988, 8780,  42968, 54936,
                4204,  44932, 57404, 9404,  1124,  64080, 5972,  55824, 8776,
                40936, 20604, 11156, 24372, 55152, 44952, 2964,  20520, 51300,
                5760,  35616, 11132, 52424, 32752, 7080,  23916, 28496, 44832,
                2364,  32016, 18828, 35200, 1976,  16848, 43924, 32448, 19000,
                38024, 8596,  33584, 64200, 7044,  22936, 38484, 58016, 24368,
                5428,  31224, 1212,  42932, 36444, 24056, 46492, 648,   53548,
                63220, 41324, 3196,  32388, 54036, 6160,  46712, 10752, 42068,
                59840, 37676, 50552, 22024, 55748, 17000, 11596, 25960, 64720,
                43420, 22448, 48092, 38328, 800,   31032, 34748, 3668,  60632,
                51788, 14900, 56092, 27904, 41112, 52276, 9448,  30464, 41844,
                51588, 17120,
            },
            {
                56004, 63848, 3008,  27176, 8632,  44960, 65164, 13152, 41776,
                5196,  52884, 16336, 61452, 22976, 28824, 35524, 10276, 24972,
                15248, 52748, 36732, 17664, 23472, 60780, 15316, 19728, 37220,
                29860, 61356, 9836,  43696, 57544, 31176, 49500, 16116, 34796,
                50896, 18444, 33560, 22624, 36472, 43760, 19252, 34844, 61016,
                2672,  48432, 63264, 4628,  28752, 37408, 15424, 34372, 30780,
                55476, 17912, 58448, 25992, 63452, 15016, 40096, 54824, 5584,
                64952, 26092, 51512, 45524, 63044, 56808, 26616, 10208, 59204,
                48596, 24028, 57252, 2344,  42596, 52168, 9812,  29244, 54012,
                48660, 38800, 16892, 59112, 12728, 49044, 62384, 39116, 33664,
                7184,  25940, 50052, 17064, 59692, 27136, 14352, 34000, 29316,
                19964, 7500,  26276, 10244, 59052, 29228, 41348, 49300, 1548,
                54504, 7152,  12644, 56300, 16612, 45168, 11412, 41072, 26020,
                33080, 472,   36524, 11228, 4028,  15876, 45804, 24652, 12632,
                59016, 22136,
            },
            {
                11112, 29812, 16000, 42296, 60080, 21172, 52288, 28252, 23108,
                62344, 45392, 32992, 40348, 12896, 57700, 47804, 61800, 56172,
                42492, 65128, 45184, 12,    8744,  39592, 46556, 56940, 1748,
                50908, 45212, 15860, 54676, 6096,  62676, 38028, 972,   23780,
                27172, 7240,  41424, 54020, 27960, 58560, 12484, 25580, 52628,
                15484, 30060, 43184, 17644, 52376, 10096, 61860, 47164, 12836,
                43816, 8444,  29156, 3428,  20460, 50664, 35108, 9540,  48084,
                16280, 38448, 11180, 6668,  29816, 47940, 4388,  40624, 13580,
                29484, 44360, 17244, 37292, 31380, 19768, 62984, 34600, 14824,
                8392,  63956, 26724, 52504, 3984,  29540, 6004,  16144, 57016,
                20208, 46112, 12320, 23428, 39700, 48880, 63096, 2016,  54608,
                44692, 65532, 40076, 31724, 3824,  45944, 22828, 8592,  32724,
                39300, 19224, 29716, 61064, 5136,  23204, 53220, 58072, 20212,
                7776,  44852, 18232, 49532, 63340, 33636, 38280, 61896, 5788,
                35936, 46884,
            },
            {
                38704, 53940, 23260, 50624, 31784, 38072, 704,   35444, 49108,
                10972, 3180,  55268, 7928,  43524, 17856, 1584,  30764, 22036,
                6896,  11796, 29768, 58444, 54424, 32148, 11304, 26012, 34592,
                59496, 21500, 4444,  32988, 24820, 12792, 19640, 46956, 56304,
                63948, 13660, 60680, 3796,  16188, 48168, 31296, 4808,  44792,
                22280, 36036, 56628, 33108, 49004, 22508, 57536, 2056,  24964,
                65496, 38836, 49444, 45632, 42200, 12452, 58144, 30604, 61124,
                22668, 43352, 57560, 21032, 15044, 34416, 22332, 55260, 63720,
                3152,  51072, 5712,  60792, 48996, 13160, 232,   47092, 43492,
                20736, 33164, 10696, 22540, 35096, 41864, 59672, 27184, 9692,
                43360, 54628, 64480, 1344,  35880, 8580,  17880, 58036, 24516,
                4840,  13176, 53252, 48896, 19052, 37468, 60804, 52752, 16024,
                58296, 46784, 25376, 42092, 50800, 37400, 28428, 14056, 48032,
                62336, 39052, 58640, 30884, 26160, 53516, 1676,  20448, 49364,
                27736, 320,
            },
            {
                44804, 61080, 4584,  47372, 12144, 6644,  55756, 16784, 58712,
                31096, 20752, 27872, 63524, 25168, 53592, 38052, 14424, 50416,
                39436, 19448, 35088, 24744, 50296, 20848, 64492, 42796, 7344,
                13204, 27720, 48216, 36896, 52476, 41204, 59124, 29696, 10332,
                39364, 46136, 30960, 52068, 10084, 38340, 63596, 54768, 39920,
                9368,  60020, 13000, 1012,  39812, 7460,  27884, 41608, 54248,
                4484,  21944, 15512, 31788, 55868, 868,   26888, 19012, 3724,
                52284, 32820, 184,   62220, 41756, 50236, 7716,  37448, 19592,
                25800, 35552, 11616, 22416, 27824, 39960, 59348, 25460, 6244,
                56724, 40932, 61524, 45372, 55212, 14416, 19524, 51500, 36872,
                30244, 4700,  33224, 27740, 52892, 21112, 41744, 31192, 37024,
                47624, 16960, 34876, 5908,  63336, 14148, 2884,  35528, 28024,
                10848, 63632, 2440,  32304, 9260,  18784, 64060, 1768,  35512,
                10596, 24144, 54508, 6872,  21572, 13248, 43196, 56972, 16752,
                64424, 32668,
            },
            {
                20124, 9412,  36664, 18168, 62764, 26100, 41192, 8520,  46576,
                39252, 15188, 50152, 34700, 4244,  46224, 9196,  33336, 54908,
                2404,  59744, 48276, 14264, 5744,  36900, 3332,  52668, 18260,
                39392, 55308, 64108, 1404,  17812, 8096,  50588, 4520,  22108,
                33232, 2092,  19540, 25292, 42988, 21848, 492,   18144, 28284,
                50300, 6316,  26040, 20164, 58984, 45972, 16040, 32768, 19016,
                51040, 36256, 59356, 5184,  23772, 64172, 36660, 46096, 40880,
                13468, 27744, 36736, 53268, 25244, 12292, 59980, 45624, 32064,
                52856, 62272, 41856, 56616, 7752,  54192, 16744, 36984, 51560,
                28764, 1504,  17840, 7508,  30868, 3188,  64888, 47524, 23008,
                13528, 58456, 40596, 15692, 60900, 45244, 3940,  50988, 9756,
                61164, 22228, 58400, 27500, 42672, 24848, 54848, 44116, 20940,
                5600,  51416, 35908, 15080, 60028, 46216, 6468,  55464, 42468,
                29424, 15456, 3148,  40968, 47400, 61196, 9048,  31460, 40316,
                7040,  13908,
            },
            {
                25652, 51548, 28776, 57552, 33904, 44308, 20404, 64240, 24700,
                2164,  60528, 42396, 11480, 18716, 59036, 27120, 63960, 23808,
                42892, 27920, 9744,  62948, 44168, 16472, 29640, 47872, 61820,
                23720, 31532, 10280, 43200, 60476, 26412, 35192, 44704, 15920,
                59780, 50092, 36776, 62412, 56280, 7988,  35316, 60940, 14084,
                46768, 65088, 38412, 53732, 30664, 63760, 10880, 37696, 60708,
                8836,  26696, 11920, 52540, 39292, 9088,  15868, 54220, 6820,
                58488, 48600, 9408,  18120, 31468, 3708,  54512, 16888, 1128,
                9308,  15340, 29808, 18536, 45004, 33368, 3664,  22788, 65508,
                12024, 46764, 38356, 58692, 49972, 24560, 40020, 8160,  872,
                61728, 18652, 46668, 6672,  11272, 29388, 56896, 13784, 26140,
                40948, 168,   30772, 11280, 56520, 7732,  48148, 30908, 61540,
                40620, 24176, 44412, 56944, 21936, 40068, 24996, 32680, 49332,
                20892, 51908, 65156, 34108, 17372, 36688, 27328, 51816, 23064,
                54760, 59552,
            },
            {
                43248, 3160,  40424, 14952, 1892,  52108, 30312, 13072, 53952,
                32860, 6012,  56208, 22908, 36832, 51508, 40736, 5696,  16232,
                47112, 4368,  33004, 55788, 38848, 26240, 58544, 12400, 34248,
                2352,  46568, 14996, 20704, 30300, 56200, 13512, 65504, 40572,
                54612, 7408,  12324, 29520, 15144, 45796, 51132, 31672, 24080,
                2444,  33536, 43700, 15112, 3336,  24388, 50124, 44340, 264,
                30084, 47620, 63052, 33604, 20156, 48892, 60244, 32036, 25060,
                20864, 61316, 4908,  44328, 63016, 39804, 28436, 23508, 43100,
                58512, 38728, 47964, 1684,  63536, 10948, 49548, 42944, 31996,
                15380, 54324, 26316, 20396, 36056, 11868, 44324, 33104, 52852,
                38748, 25564, 50012, 55796, 23860, 37960, 62604, 33848, 19204,
                54244, 45720, 38396, 51112, 20172, 33748, 17008, 12212, 1348,
                55892, 18240, 12796, 7120,  30272, 52652, 11652, 16680, 61512,
                5740,  44004, 9656,  24688, 59716, 2540,  45384, 4744,  11556,
                47500, 35100,
            },
            {
                6336,  65020, 55252, 24204, 10400, 61232, 5532,  48060, 17580,
                37248, 45252, 29008, 65500, 324,   30988, 13232, 21108, 56796,
                36112, 61060, 18588, 22644, 788,   53520, 8624,  21432, 41472,
                6188,  57444, 52244, 39648, 5464,  47784, 23132, 56,    28316,
                18360, 24268, 47036, 4952,  40048, 26540, 58584, 10960, 41760,
                57160, 18644, 9672,  51660, 35948, 5872,  55992, 21736, 13804,
                57768, 40700, 16816, 6252,  42356, 28756, 2768,  43628, 10816,
                35016, 38884, 15388, 56688, 20368, 47680, 10444, 64572, 49888,
                34148, 4832,  24308, 52348, 36584, 26272, 59864, 19816, 5080,
                61040, 34384, 9576,  2304,  53488, 60656, 16968, 27764, 56952,
                10108, 31616, 3328,  34872, 42260, 16684, 1796,  48824, 5208,
                64624, 8548,  15216, 61692, 2256,  41296, 64276, 50628, 37900,
                27392, 33392, 48000, 64992, 748,   36944, 59512, 3056,  27144,
                56400, 37712, 30680, 48300, 12468, 55708, 20060, 63192, 38176,
                29616, 17192,
            },
            {
                21724, 32764, 46404, 19096, 49820, 34964, 38864, 26692, 58220,
                9704,  21152, 14192, 52980, 8196,  47824, 61936, 44456, 10520,
                30472, 51760, 13412, 42040, 49432, 31580, 60784, 44812, 51196,
                28336, 37068, 25236, 62756, 34072, 11168, 53564, 37844, 9248,
                43756, 57576, 32216, 52792, 64416, 1852,  20136, 6548,  37736,
                49216, 29296, 22300, 61948, 26932, 41136, 17684, 62824, 34124,
                25164, 3320,  44980, 55484, 24540, 13056, 51464, 18180, 64884,
                46944, 29796, 50748, 26596, 2388,  33068, 37160, 6436,  19832,
                27284, 12912, 60404, 21076, 31400, 14512, 55164, 8504,  40384,
                24220, 49268, 43828, 64052, 28820, 6472,  48272, 21420, 63464,
                15300, 44712, 20576, 65216, 8964,  52596, 21900, 27652, 43840,
                35592, 24132, 29132, 47264, 36400, 26188, 22728, 6744,  45808,
                59860, 9224,  54256, 25444, 42360, 19408, 47348, 34152, 40480,
                13500, 18704, 204,   53644, 28072, 41448, 32296, 15324, 56792,
                824,   50508,
            },
            {
                39272, 13376, 8024,  29912, 41696, 59344, 22308, 752,   42584,
                61824, 49180, 25952, 39808, 17124, 33676, 24248, 50952, 1468,
                38640, 24860, 6584,  64092, 11332, 35860, 2828,  17720, 13772,
                64772, 9316,  17028, 1600,  45216, 19024, 61400, 31348, 49260,
                63136, 2872,  38248, 16588, 22772, 34668, 44200, 54108, 16732,
                60196, 4076,  45260, 54876, 13132, 31464, 47960, 7572,  38580,
                52984, 19300, 10132, 30948, 61940, 58212, 37836, 5904,  55224,
                1736,  23096, 8272,  42752, 12628, 52520, 60896, 14932, 56568,
                46376, 54088, 39856, 7116,  44000, 2752,  38524, 28208, 46236,
                57088, 4184,  17512, 39664, 32120, 13752, 42064, 2592,  37380,
                5364,  51400, 28884, 13200, 56716, 47056, 31668, 60120, 11224,
                57672, 17904, 5688,  53028, 59704, 10072, 57324, 30264, 14444,
                20756, 3568,  38988, 16084, 50860, 28596, 9460,  54068, 23280,
                63992, 46100, 58440, 21896, 5996,  50956, 8732,  23832, 43656,
                26572, 61424,
            },
            {
                53440, 57864, 25752, 63296, 4696,  15892, 12064, 53552, 31336,
                6920,  35148, 2908,  55676, 43708, 58292, 5244,  28368, 18188,
                58756, 45408, 54836, 28784, 19860, 47212, 25604, 38628, 54732,
                22916, 32828, 58328, 50356, 28672, 41636, 3624,  15808, 25440,
                35136, 20772, 10536, 60460, 50200, 13024, 30496, 63164, 23652,
                12260, 33052, 8240,  37516, 1264,  59808, 43232, 11360, 28364,
                49276, 64956, 35612, 47388, 716,   21296, 34080, 27360, 40512,
                13872, 52944, 63488, 34584, 59116, 22856, 29472, 688,   41232,
                31748, 9688,  17436, 64780, 48668, 58720, 17804, 52164, 13068,
                30852, 21600, 11244, 57888, 22812, 52320, 59900, 25388, 33864,
                55132, 40960, 59612, 484,   25852, 38820, 7520,  14908, 40408,
                50536, 32272, 43224, 13088, 19296, 4292,  42424, 52180, 35988,
                62972, 43976, 30756, 58816, 4916,  13824, 62768, 43664, 4256,
                8108,  31716, 36164, 15664, 39296, 64692, 34376, 59132, 48708,
                7272,  11096,
            },
            {
                34180, 42996, 1324,  37312, 47080, 55904, 28544, 45432, 19280,
                51364, 12704, 63100, 29292, 20288, 11164, 37404, 42120, 64768,
                8748,  14744, 32508, 40668, 5168,  56480, 62532, 30484, 4004,
                48612, 40472, 6952,  13332, 55700, 21760, 58988, 52052, 6424,
                13416, 54340, 42232, 27588, 7260,  40448, 46736, 3396,  27984,
                52336, 40168, 64744, 16208, 49952, 20400, 23928, 2376,  56760,
                14904, 4632,  22488, 39976, 15952, 50076, 11296, 45672, 19516,
                60500, 31988, 17092, 4336,  39288, 18404, 44580, 50568, 24872,
                61856, 3020,  35348, 26808, 11768, 23048, 34608, 62004, 1648,
                37132, 63084, 50892, 34168, 136,   45748, 9344,  18200, 49312,
                11756, 23648, 16840, 36300, 45224, 19308, 63912, 2296,  54700,
                21304, 3540,  63404, 38776, 28220, 33552, 46836, 16824, 356,
                24240, 49528, 11080, 35000, 22568, 56260, 32244, 20264, 51688,
                26788, 49636, 10772, 61288, 44920, 18284, 1696,  13032, 37332,
                30508, 19368,
            },
            {
                46024, 15524, 22688, 51460, 18500, 33696, 8396,  64824, 40304,
                24396, 16740, 38380, 46244, 4052,  61004, 15272, 54204, 22276,
                34444, 48852, 3460,  59728, 23612, 15428, 8132,  42952, 11568,
                19532, 60956, 44368, 24808, 36368, 8304,  32536, 39208, 46344,
                59888, 30032, 48360, 812,   56176, 18816, 58960, 9544,  35756,
                48220, 19068, 25344, 56556, 29068, 35284, 53936, 61520, 31892,
                41548, 26416, 59400, 54384, 8308,  29676, 63004, 56660, 7764,
                24804, 37180, 45084, 9884,  48380, 55740, 7384,  11564, 37976,
                19364, 45120, 51100, 57036, 41520, 30336, 6220,  47768, 25516,
                44808, 7908,  42368, 26744, 15976, 56000, 38916, 31028, 62904,
                6724,  47288, 30528, 62240, 5132,  32740, 49144, 24636, 29496,
                36744, 9684,  47924, 56804, 22324, 54316, 65316, 11832, 26812,
                55952, 6288,  18696, 61464, 45556, 39868, 2276,  37840, 16052,
                60616, 41632, 24024, 5144,  30000, 26376, 47312, 22200, 54528,
                63600, 2912,
            },
            {
                49520, 59928, 30788, 10300, 61976, 26208, 3972,  36040, 57120,
                2108,  59436, 32660, 9036,  50604, 25712, 31220, 46772, 2156,
                52500, 26780, 18560, 36720, 44588, 52776, 33388, 50156, 57304,
                35216, 29320, 2844,  17380, 64556, 48444, 11932, 26832, 19536,
                3912,  36940, 23164, 65092, 33020, 25488, 37424, 15736, 55088,
                132,   42808, 6232,  46332, 10040, 4432,  44508, 17216, 37416,
                7148,  46264, 12528, 32920, 44056, 23700, 3872,  52100, 42336,
                980,   49572, 57588, 21536, 26444, 31236, 65428, 16380, 58356,
                28424, 5320,  21816, 15308, 392,   53788, 10028, 20232, 55264,
                14180, 18788, 60076, 6152,  36464, 64820, 3888,  27800, 43608,
                57540, 20952, 9184,  50916, 54416, 11632, 41924, 58124, 17176,
                44940, 60916, 25772, 15384, 7596,  2232,  39948, 31164, 59384,
                37816, 41660, 52528, 27748, 8500,  48960, 25828, 11848, 57828,
                34780, 1116,  55352, 14020, 51984, 56856, 41720, 4500,  16556,
                40500, 27532,
            },
            {
                9000,  5152,  41300, 54032, 13996, 43908, 50104, 21276, 11008,
                47396, 27560, 53596, 21996, 41156, 57180, 35864, 7088,  12784,
                61296, 41648, 9572,  63576, 12304, 28188, 20688, 1992,  26008,
                14920, 46972, 54004, 38736, 30292, 836,   42320, 63208, 54960,
                43972, 17196, 7960,  14420, 45388, 52604, 5352,  62456, 31636,
                22156, 60976, 14144, 32436, 63028, 40260, 13212, 50500, 21584,
                63316, 52160, 18452, 3016,  61208, 38700, 17224, 35544, 31036,
                11828, 28572, 15136, 62092, 2744,  36356, 42828, 3652,  34216,
                52820, 62884, 38888, 33244, 60944, 44244, 36572, 64292, 39900,
                32308, 52848, 28904, 49016, 24072, 12248, 20204, 51720, 14516,
                2372,  37000, 40452, 26652, 15556, 22732, 35068, 8084,  52036,
                12864, 544,   30732, 35688, 50724, 44584, 18624, 8900,  47900,
                21188, 13592, 32708, 1536,  16668, 64596, 53764, 6800,  46588,
                29620, 17724, 47800, 38600, 32520, 9492,  61960, 35400, 50820,
                58752, 23684,
            },
            {
                38240, 65464, 34596, 20348, 2384,  37784, 60296, 29708, 15288,
                42732, 6328,  64344, 13844, 516,   17608, 63320, 23796, 38812,
                20084, 29940, 56088, 8,     49172, 38772, 58808, 65460, 41492,
                6720,  62116, 10564, 21080, 58532, 52584, 16204, 23356, 9848,
                33940, 62052, 50632, 39724, 10976, 21492, 42020, 49200, 11744,
                26940, 57776, 37228, 53152, 19744, 27216, 58352, 5380,  25452,
                10192, 29944, 36416, 55948, 27196, 49036, 13260, 65068, 20620,
                59624, 53748, 6032,  40820, 51584, 12676, 54480, 23460, 48228,
                9216,  13336, 46916, 7692,  24740, 49904, 16724, 27692, 5276,
                57920, 10808, 1400,  39444, 46196, 54176, 33500, 41144, 24660,
                60536, 32076, 55848, 1148,  44216, 59556, 3904,  63304, 27092,
                40156, 55568, 20120, 41692, 58472, 25028, 61748, 35296, 53668,
                5056,  63700, 44964, 56872, 36052, 20496, 31324, 43328, 22948,
                61888, 8672,  21592, 63884, 2668,  19800, 25144, 11880, 6480,
                32844, 13728,
            },
            {
                51888, 17344, 26704, 57988, 48352, 23968, 7316,  55292, 33384,
                51188, 18688, 36824, 45144, 30316, 49436, 10128, 43724, 58088,
                50696, 15688, 45860, 34676, 22840, 4772,  16980, 9784,  35976,
                22588, 50864, 27856, 5844,  45292, 34920, 4600,  49744, 29580,
                1588,  56704, 24948, 31084, 60584, 2980,  28692, 18160, 39268,
                45964, 8968,  3680,  23744, 48632, 1000,  30952, 34868, 47576,
                57196, 1528,  45116, 20820, 9384,  41936, 5304,  25632, 47700,
                37768, 43432, 24048, 33996, 18464, 29188, 60672, 19896, 39748,
                26044, 30968, 55044, 18596, 29444, 12152, 59644, 2104,  23320,
                43332, 35772, 21312, 60884, 17156, 8688,  62284, 4984,  49468,
                17632, 46104, 12696, 62748, 29724, 38416, 18524, 46820, 31912,
                6200,  48920, 64588, 4504,  10484, 14044, 29584, 2944,  23536,
                16216, 28652, 9932,  24212, 41216, 13036, 59188, 4460,  14812,
                40084, 52836, 27980, 43920, 36312, 59168, 45712, 29012, 43088,
                20676, 56544,
            },
            {
                30088, 124,   44748, 6076,  31612, 40544, 17928, 63972, 1068,
                39108, 25756, 56720, 4448,  34648, 54968, 27352, 4900,  32100,
                2704,  25508, 7484,  60696, 31444, 43124, 54476, 46336, 30924,
                55752, 17752, 39924, 32900, 13676, 25276, 61324, 37680, 41196,
                12700, 46624, 18968, 6404,  54148, 34552, 57128, 64328, 1868,
                51724, 35040, 17672, 41564, 60376, 15488, 52576, 65328, 42184,
                16368, 39576, 62648, 14508, 51308, 58696, 34332, 54572, 8616,
                2476,  16764, 9716,  64244, 46048, 344,   7896,  43936, 15208,
                58048, 1524,  42564, 64928, 3780,  40444, 33896, 51880, 47032,
                15516, 63716, 50376, 31888, 3200,  44072, 29544, 58228, 10384,
                35260, 7112,  20004, 24412, 51664, 10144, 56564, 21844, 14676,
                37656, 23868, 17328, 32788, 53040, 47316, 38712, 57096, 43748,
                50040, 37296, 59996, 47652, 3572,  51108, 27152, 37484, 49416,
                33460, 3432,  57484, 13328, 7104,  17128, 49208, 53880, 64124,
                3380,  47020,
            },
            {
                41868, 12408, 54684, 61644, 9904,  52380, 12832, 44192, 22224,
                9008,  61456, 11980, 23532, 59836, 19924, 14896, 47200, 37144,
                64844, 41092, 54892, 13940, 51836, 19216, 27024, 12752, 948,
                60044, 43788, 3344,  63904, 56672, 18820, 7608,  55320, 21292,
                59680, 27956, 36812, 42812, 16292, 47436, 8492,  23948, 14748,
                55456, 29572, 63572, 45036, 10452, 38164, 7756,  20324, 12108,
                23892, 33136, 6092,  28620, 22808, 436,   30224, 19320, 45032,
                61480, 32220, 50400, 56696, 25780, 37116, 49664, 32760, 63772,
                6408,  50592, 34984, 22860, 57520, 45848, 20768, 9160,  55664,
                30108, 6812,  12616, 26804, 54988, 22376, 14012, 38336, 26532,
                65108, 53164, 42340, 4732,  47888, 33936, 2864,  42908, 54228,
                61796, 9324,  45064, 59304, 27056, 21604, 6852,  62952, 12068,
                31828, 1136,  54696, 18276, 34188, 63040, 8164,  55420, 19036,
                65288, 11016, 24772, 50640, 31008, 39628, 1564,  14648, 37440,
                9960,  24732,
            },
        },
        {
            {
                17352, 37824, 46912, 27696, 59644, 31676, 3744,  26544, 34272,
                48,    51448, 62488, 28932, 47560, 59788, 51560, 65260, 22348,
                16784, 39184, 63032, 21220, 29128, 42084, 10664, 22824, 49608,
                61164, 45656, 19892, 32456, 47100, 2360,  59424, 11832, 19600,
                57768, 512,   51752, 7288,  42336, 20600, 31000, 4460,  53156,
                35980, 20960, 42112, 24168, 18220, 61904, 35596, 57872, 19712,
                52336, 29756, 49472, 22640, 40448, 19536, 38476, 980,   62208,
                18856, 7192,  13256, 40068, 608,   29788, 36312, 7868,  27984,
                59256, 43964, 31044, 57528, 768,   49980, 29372, 10804, 36396,
                42924, 4632,  20284, 38964, 10300, 22040, 57580, 36612, 12724,
                49352, 42684, 30408, 38336, 11636, 27600, 33036, 232,   38896,
                21232, 36096, 4724,  45880, 28396, 58016, 32500, 47428, 10164,
                54224, 6596,  14188, 62420, 11152, 4928,  28536, 328,   58440,
                14948, 22480, 34292, 11456, 47544, 31788, 21928, 50044, 1176,
                14236, 65396,
            },
            {
                51204, 56444, 664,   18924, 53576, 39528, 15724, 62852, 43680,
                30512, 54916, 10528, 37124, 3248,  31656, 11744, 26388, 2280,
                55044, 8340,  44320, 53708, 15752, 37492, 63756, 32992, 2448,
                14328, 27816, 35912, 4396,  15076, 29544, 44580, 5524,  40556,
                30816, 48128, 21768, 27016, 64636, 38384, 47648, 14276, 44880,
                61316, 1300,  56824, 37924, 7428,  27528, 50936, 1444,  23680,
                45096, 5796,  60048, 13412, 47108, 55216, 34560, 11580, 49720,
                24404, 46424, 63100, 26528, 56552, 49528, 61844, 18536, 53548,
                39508, 11328, 25444, 37132, 14512, 42260, 6220,  65408, 48080,
                25340, 57876, 32476, 45232, 5652,  16032, 43168, 60776, 900,
                33448, 54944, 17416, 2452,  60680, 48068, 53860, 16684, 26412,
                44464, 55508, 15436, 40604, 19376, 50340, 1476,  16348, 23180,
                44360, 29236, 51540, 21456, 32732, 53648, 48692, 35236, 45076,
                54792, 6032,  59492, 40208, 61660, 6584,  17596, 39800, 54260,
                43364, 21236,
            },
            {
                25568, 10428, 30056, 62004, 22484, 5828,  47832, 12044, 57132,
                7628,  21344, 15024, 44072, 23488, 17548, 7324,  43304, 35848,
                58176, 27448, 32320, 11824, 47784, 3860,  19352, 51920, 43200,
                38196, 7504,  54524, 50692, 60732, 23676, 53104, 34132, 25388,
                15592, 61512, 12584, 34288, 16796, 55552, 9524,  33664, 18492,
                26488, 49144, 9800,  46296, 32748, 14748, 40540, 10228, 31976,
                15840, 36844, 53900, 32988, 28008, 5152,  58452, 15832, 41360,
                31800, 59600, 36588, 9712,  21716, 4920,  12768, 43032, 23872,
                33080, 5604,  51140, 61756, 19560, 32956, 54140, 21572, 15324,
                8040,  50376, 17164, 54080, 64368, 34736, 51320, 28020, 40628,
                15068, 25032, 57700, 34404, 51388, 5344,  40832, 9252,  61856,
                11768, 51288, 3008,  62840, 9556,  37676, 25608, 64440, 39316,
                59268, 35916, 1024,  46396, 38140, 7488,  23084, 12388, 26736,
                38300, 49592, 19092, 28228, 14528, 44492, 55984, 10608, 27672,
                32100, 6108,
            },
            {
                35764, 48764, 41144, 14056, 34920, 51736, 25252, 36720, 18348,
                41628, 49448, 34616, 64292, 52776, 56268, 40188, 62188, 14704,
                47436, 18884, 828,   23848, 59556, 55396, 8080,  30160, 58216,
                24372, 62668, 10604, 41828, 18244, 37964, 8568,  63312, 55188,
                4228,  37696, 57184, 46208, 2168,  24588, 50580, 59056, 6688,
                39892, 30612, 12964, 52388, 60056, 21732, 64304, 47576, 55808,
                62624, 41960, 208,   10872, 44516, 24156, 64128, 29392, 21484,
                6388,  1976,  51932, 16560, 41804, 34088, 46648, 55292, 1248,
                64116, 16276, 55752, 9288,  45836, 26368, 58616, 1620,  34868,
                61652, 40128, 2308,  27464, 13168, 24024, 3832,  9364,  21136,
                65076, 6592,  44680, 10020, 21396, 24512, 30924, 58476, 20152,
                34904, 29500, 24580, 33184, 59496, 13672, 55760, 30716, 4284,
                12680, 18356, 27172, 57856, 15572, 64884, 41752, 60468, 17252,
                9760,  63748, 1372,  53532, 37280, 23776, 33880, 62688, 4156,
                46008, 59568,
            },
            {
                12808, 63120, 4784,  57616, 44348, 9492,  64948, 28984, 1752,
                23772, 60364, 5244,  27068, 1584,  46252, 33428, 21412, 4676,
                51904, 40992, 63944, 35408, 39388, 27208, 45288, 12168, 17048,
                48352, 20656, 32024, 26120, 1352,  47976, 13248, 20764, 50096,
                43516, 9900,  28232, 41336, 5912,  62792, 29116, 42776, 22416,
                54968, 65024, 17484, 25424, 6512,  43060, 4432,  18488, 28476,
                8052,  25676, 21380, 61668, 18092, 50888, 2376,  43336, 53444,
                57276, 14004, 44808, 28504, 64764, 57928, 30736, 9008,  37556,
                20604, 47652, 40848, 29936, 3580,  12492, 38516, 51852, 29068,
                44792, 22448, 56880, 36964, 48536, 61488, 31160, 55428, 46584,
                50372, 29336, 38656, 62888, 13956, 36264, 43984, 1768,  46832,
                6116,  57380, 41876, 48520, 22036, 5848,  43092, 46948, 8516,
                52760, 60988, 41096, 9604,  50232, 19608, 30216, 2208,  52588,
                33540, 22044, 30804, 46228, 8980,  3068,  58540, 20188, 15504,
                52572, 39296,
            },
            {
                18796, 23020, 32972, 27140, 17544, 2916,  40048, 58708, 53420,
                46448, 32172, 13468, 39256, 19556, 10392, 29240, 59160, 24936,
                12620, 29636, 9156,  50332, 5760,  20916, 62392, 34460, 484,
                40232, 5324,  45728, 64680, 55656, 29016, 40952, 31808, 2736,
                17532, 32880, 22916, 52548, 19248, 36632, 12028, 32,    15484,
                35040, 3700,  44220, 37120, 57448, 29596, 34608, 50312, 38644,
                13900, 58520, 35472, 48640, 39524, 7608,  34956, 10068, 37776,
                18616, 33592, 23560, 38624, 3012,  19840, 15052, 25532, 60788,
                28292, 4696,  34644, 22304, 49376, 63808, 43464, 18640, 9112,
                13844, 31528, 6492,  10864, 19764, 44460, 39116, 17668, 35580,
                12112, 1508,  18848, 52580, 47228, 55664, 17388, 63940, 38408,
                52028, 14556, 18304, 604,   36152, 53464, 27576, 19704, 33900,
                49064, 22612, 32116, 5196,  25404, 36536, 45936, 57604, 6984,
                48008, 42272, 13404, 57192, 26068, 50600, 41692, 48164, 30724,
                24456, 8176,
            },
            {
                42620, 2080,  45812, 54344, 50704, 21424, 31504, 11172, 15912,
                38212, 8196,  55536, 50220, 61724, 16088, 48768, 8280,  38192,
                45208, 54168, 60712, 17412, 14068, 42324, 52196, 25328, 56168,
                60192, 51116, 12868, 35748, 16520, 7064,  57584, 60940, 26804,
                46792, 65184, 58084, 7872,  48500, 59764, 32108, 53248, 47304,
                60428, 8772,  49868, 20500, 908,   53924, 12364, 24280, 2632,
                52456, 43172, 5372,  29632, 54800, 14976, 57780, 27092, 46084,
                62588, 49944, 8480,  54724, 47956, 6500,  52964, 40360, 50004,
                44356, 13580, 56896, 60112, 15968, 35996, 5640,  24364, 56008,
                46916, 64088, 52444, 42136, 57956, 480,   7988,  59264, 25964,
                62120, 43024, 32532, 26848, 3184,  7492,  29024, 12736, 22664,
                27348, 8584,  64900, 45316, 11324, 61664, 15636, 40744, 63804,
                2192,  14608, 44976, 62712, 56396, 3708,  13912, 24128, 39560,
                27624, 61524, 4772,  35988, 64984, 17952, 11048, 37744, 128,
                64332, 55296,
            },
            {
                50176, 60000, 15160, 37496, 7532,  61020, 42516, 48892, 4056,
                20880, 63872, 25096, 30080, 42264, 36244, 57208, 2172,  64836,
                34360, 3308,  22476, 32888, 46736, 2092,  31116, 9388,  37060,
                15588, 27868, 22672, 3112,  44156, 52032, 23384, 10896, 37212,
                51596, 5284,  13164, 39480, 15988, 26320, 45048, 20252, 38772,
                24712, 28520, 32608, 63124, 15304, 39824, 60928, 45192, 57180,
                32696, 16948, 64060, 11556, 20580, 32132, 60700, 22292, 12728,
                5268,  30700, 58988, 12260, 26908, 61868, 32488, 11120, 2368,
                18400, 52312, 8180,  25308, 244,   54556, 32252, 61056, 39564,
                2044,  17584, 26360, 34020, 23156, 29416, 15388, 51396, 4036,
                22216, 56496, 9472,  59172, 41588, 61356, 33996, 48900, 42796,
                60288, 32668, 54280, 24976, 30292, 49848, 3532,  24196, 57464,
                36940, 28616, 53892, 11384, 42500, 34524, 51196, 60764, 18648,
                10968, 54124, 16372, 21184, 32080, 6460,  28108, 60836, 13688,
                35052, 26872,
            },
            {
                32164, 10288, 29152, 64212, 12560, 34576, 24840, 55796, 28092,
                35340, 44296, 11716, 636,   6424,  23176, 52504, 27020, 19804,
                16328, 43640, 28024, 57880, 37600, 54508, 65528, 19200, 44628,
                6352,  33200, 61956, 58048, 38588, 18972, 33600, 360,   15264,
                41608, 21364, 29880, 34196, 55676, 3756,  9788,  63524, 4808,
                13324, 57940, 41888, 10500, 47188, 26628, 7368,  19432, 10048,
                22712, 47364, 1668,  25928, 44840, 52100, 2760,  48012, 40964,
                25520, 440,   42936, 17852, 35624, 41536, 22200, 59240, 36196,
                64928, 31424, 39408, 43140, 29132, 47352, 11424, 20432, 28308,
                50048, 36652, 4392,  12876, 62500, 54148, 46172, 33348, 40644,
                13920, 48028, 37204, 15648, 20996, 25248, 10696, 53868, 5552,
                2692,  37272, 20428, 39876, 6528,  56380, 33156, 9592,  46436,
                18680, 6884,  39004, 20824, 27536, 17560, 8876,  31508, 44504,
                37612, 660,   49236, 45080, 40060, 55164, 51496, 44100, 21512,
                47176, 4900,
            },
            {
                58164, 20532, 40392, 23616, 1376,  47476, 18700, 6544,  62592,
                51556, 17856, 59296, 54424, 46020, 32976, 13740, 40576, 47620,
                55476, 10828, 6720,  48668, 24920, 4552,  12060, 23032, 49708,
                39404, 53704, 10764, 30624, 8108,  49024, 63800, 45348, 54812,
                25000, 61160, 2240,  44216, 62372, 23276, 41476, 30588, 55172,
                51360, 18512, 2224,  22316, 51976, 31032, 35964, 65492, 40672,
                28628, 59352, 37704, 55964, 41316, 6020,  35968, 16820, 65192,
                56156, 37316, 21428, 63768, 46996, 1596,  15920, 45916, 24560,
                6280,  20920, 10888, 62536, 17172, 52088, 41080, 6640,  14600,
                57024, 44016, 60512, 48072, 38836, 18380, 10312, 64672, 27804,
                6236,  54336, 30704, 4340,  50952, 38888, 57724, 19216, 30040,
                16444, 50568, 58760, 12856, 44572, 17236, 37988, 59964, 13280,
                31148, 50896, 61416, 132,   48672, 64588, 55740, 2764,  21828,
                63348, 33892, 25292, 59880, 12608, 1900,  24032, 57684, 9084,
                40788, 17408,
            },
            {
                36684, 6172,  49000, 56840, 43960, 30128, 59724, 14476, 40712,
                2608,  31344, 39464, 15380, 21052, 62948, 10084, 4748,  60472,
                30936, 39124, 63296, 13456, 18620, 41184, 60904, 27384, 57460,
                3480,  47080, 21164, 43196, 25796, 13720, 28528, 59588, 6208,
                35616, 48260, 53108, 19100, 11868, 50656, 35440, 16828, 7272,
                36984, 45800, 33656, 61556, 56596, 5172,  16432, 54516, 3364,
                51176, 14140, 33896, 8764,  18444, 61912, 28400, 9664,  31776,
                53016, 14408, 50336, 9972,  29856, 54496, 8104,  51444, 28700,
                55940, 48320, 58212, 34484, 3084,  22112, 58632, 65176, 32780,
                24788, 9680,  21644, 6824,  31420, 2900,  24476, 36212, 44204,
                19400, 61496, 24136, 46292, 65488, 404,   44396, 35604, 62708,
                46184, 9612,  26716, 1160,  63536, 22496, 52612, 27108, 1804,
                42880, 56068, 24996, 15196, 33168, 40304, 46336, 13380, 52272,
                29528, 6192,  56224, 8712,  28972, 36852, 15676, 33700, 3448,
                53824, 62012,
            },
            {
                45400, 26372, 53528, 16588, 4360,  38084, 50576, 10672, 46256,
                23500, 8968,  27296, 48584, 35184, 57744, 28224, 51308, 24416,
                1136,  21544, 35080, 59524, 51100, 29716, 36268, 7712,  32240,
                14404, 18068, 64552, 1816,  56040, 40748, 4028,  20376, 31388,
                11416, 16672, 8320,  38448, 28644, 748,   60156, 48324, 27148,
                64852, 24320, 8356,  38828, 13064, 43880, 24636, 46796, 32028,
                6760,  20972, 63000, 48924, 24408, 13156, 50784, 39588, 19516,
                7016,  44832, 33468, 4104,  25364, 58596, 33956, 12996, 42396,
                38208, 4412,  14552, 45256, 27368, 8716,  36452, 45980, 1332,
                53400, 35308, 16580, 52040, 41152, 56660, 49644, 58888, 1268,
                51972, 8192,  35004, 11664, 17928, 32004, 7812,  13512, 23308,
                40404, 55968, 34424, 48224, 29424, 42168, 7704,  49120, 64832,
                21616, 10648, 35680, 59008, 8128,  23300, 5184,  26696, 58828,
                38976, 19684, 15296, 41012, 62412, 47684, 19032, 64000, 30272,
                25088, 14248,
            },
            {
                868,   31660, 9344,  34112, 63440, 20184, 26492, 33424, 57280,
                36404, 53168, 65364, 7080,  3092,  43372, 18204, 36824, 45340,
                15760, 52884, 42832, 8948,  1984,  45968, 16584, 53112, 42176,
                59288, 25024, 36804, 49928, 34384, 17104, 52800, 37452, 43856,
                58156, 26764, 64028, 32756, 45736, 57376, 13664, 20548, 43424,
                2888,  15520, 50484, 28172, 20780, 316,   37748, 60420, 12172,
                41832, 27168, 45444, 940,   31096, 56900, 43704, 59960, 2520,
                23908, 28000, 56744, 61004, 16988, 39488, 20408, 62716, 2624,
                17824, 32636, 25040, 54360, 39000, 50856, 12812, 29660, 19128,
                42228, 59072, 28772, 64236, 25808, 8644,  21072, 12348, 16448,
                29696, 39148, 58376, 43068, 28248, 56196, 48312, 26340, 52636,
                5156,  19824, 14400, 59816, 4404,  36064, 19660, 14720, 33436,
                39204, 47180, 3500,  44252, 30424, 54544, 36828, 17248, 42432,
                10096, 50864, 45984, 23008, 53312, 4840,  43484, 49924, 38832,
                10884, 51796,
            },
            {
                65232, 19264, 57884, 12916, 41796, 7676,  54520, 352,   16248,
                60612, 19408, 12972, 41088, 23812, 54924, 8088,  32008, 64584,
                5692,  57100, 27164, 32460, 22560, 56352, 64248, 11104, 180,
                45432, 29200, 5396,  12404, 58836, 9704,  61980, 23724, 51244,
                6728,  40512, 55140, 21660, 5600,  25200, 39972, 33928, 56220,
                11304, 53692, 40652, 58916, 47500, 63672, 29476, 18276, 56948,
                52576, 36164, 15616, 54244, 38452, 4584,  21868, 34284, 47076,
                62512, 40740, 12444, 36060, 48628, 5488,  52824, 44848, 23568,
                59944, 50272, 63788, 7216,  18432, 61536, 24188, 55236, 6096,
                49112, 12080, 4012,  46780, 14724, 43848, 38108, 32948, 63064,
                48344, 21884, 14488, 5568,  51552, 21252, 61060, 36900, 1880,
                64512, 32228, 43820, 24400, 53524, 11004, 62476, 55032, 5176,
                29140, 17100, 53120, 19960, 63636, 11052, 50000, 61684, 31860,
                4424,  65480, 34932, 2380,  31572, 11864, 26344, 7152,  59324,
                22088, 43120,
            },
            {
                27800, 38348, 46392, 23240, 52452, 29012, 62280, 22240, 42488,
                5352,  45272, 32796, 51044, 29400, 62180, 13436, 49244, 40864,
                11252, 19260, 47972, 14232, 38768, 4968,  25972, 34416, 21540,
                38060, 61388, 54144, 22972, 31224, 44700, 27604, 824,   13540,
                19340, 47800, 2544,  14532, 52004, 63140, 9272,  4100,  29244,
                61944, 31804, 18932, 6332,  35048, 10520, 50092, 7948,  22836,
                2292,  61000, 10436, 20156, 63620, 8164,  27048, 15164, 10388,
                54036, 18100, 1504,  22548, 42984, 30740, 10956, 27680, 36640,
                8576,  12280, 30028, 42652, 836,   33496, 43852, 15440, 63656,
                39544, 22780, 37268, 33268, 92,    60412, 52724, 26692, 5336,
                41252, 54864, 25704, 63816, 2336,  40088, 16340, 11140, 42212,
                28488, 50028, 7412,  38844, 16612, 31064, 45644, 40944, 25396,
                57488, 61592, 7032,  27676, 41460, 14832, 22084, 2664,  47996,
                25064, 13260, 27728, 57308, 20484, 61220, 35572, 55420, 17604,
                3072,  34872,
            },
            {
                15508, 6772,  60200, 1972,  35720, 48356, 14120, 38372, 30536,
                49860, 25136, 2200,  59096, 16608, 38308, 552,   21668, 25436,
                58748, 34708, 4004,  62236, 50672, 43932, 18392, 48248, 52140,
                13972, 8528,  47200, 41472, 19768, 7144,  48612, 39672, 65292,
                34716, 29600, 61024, 37056, 42780, 17988, 47084, 51304, 22220,
                37808, 45024, 2272,  25984, 55168, 15916, 32788, 44508, 39932,
                25748, 49460, 29804, 42980, 33380, 47984, 52348, 58820, 36580,
                29748, 49972, 63696, 8268,  55588, 15084, 65508, 376,   56632,
                46864, 40064, 20772, 56984, 48912, 10412, 58140, 3356,  32016,
                27136, 8804,  62216, 55612, 19528, 30044, 7316,  56940, 46148,
                2884,  10136, 32432, 36596, 46628, 8760,  33564, 54552, 59040,
                18328, 12456, 61788, 56312, 3052,  50624, 22800, 760,   9740,
                48828, 13376, 36492, 51480, 1048,  58148, 34036, 44836, 56664,
                18864, 53700, 43868, 8004,  38568, 48472, 14432, 40904, 29724,
                47116, 56400,
            },
            {
                12024, 40572, 49984, 26184, 17400, 4428,  44420, 8532,  56428,
                11432, 63812, 35896, 20892, 9404,  44896, 53344, 60728, 30320,
                45768, 39328, 53904, 28992, 9676,  59024, 31248, 6344,  63280,
                27400, 17784, 33116, 1512,  62856, 55744, 16172, 31944, 54292,
                8768,  45356, 23296, 10768, 31192, 26564, 35360, 59468, 16440,
                7320,  49436, 13600, 60952, 41772, 22068, 58332, 4836,  64848,
                34860, 17444, 3796,  59140, 12908, 40960, 17716, 88,    45336,
                5792,  25528, 38800, 32836, 46320, 25180, 49808, 18036, 31448,
                53136, 15728, 4080,  35516, 22944, 28684, 37792, 46920, 20496,
                50560, 53584, 17040, 42856, 48276, 10640, 23580, 16868, 35208,
                20088, 59920, 50472, 17260, 57096, 29408, 24672, 44908, 5916,
                22872, 46660, 35156, 26596, 20272, 60944, 37308, 58780, 33888,
                43368, 21956, 31908, 46108, 24612, 38860, 9180,  28656, 6756,
                36200, 40196, 59604, 16992, 51800, 1612,  23932, 9716,  63288,
                6112,  51132,
            },
            {
                61640, 21224, 30344, 10360, 64628, 54988, 32136, 60040, 18372,
                27732, 40008, 6580,  47916, 57440, 26540, 34048, 6384,  15672,
                9308,  1888,  21196, 16252, 24292, 36960, 12684, 41556, 54600,
                3024,  39292, 57816, 50912, 14192, 24816, 36996, 59052, 3696,
                26104, 17084, 58532, 53592, 6308,  56920, 964,   12204, 41604,
                27420, 63508, 20164, 36176, 51584, 1244,  28068, 53316, 14152,
                9724,  44996, 55400, 22096, 5924,  24792, 64960, 32064, 21312,
                42140, 14540, 58516, 19788, 3716,  60524, 41148, 34544, 21940,
                6324,  59092, 26868, 64472, 12692, 53844, 17780, 7100,  60556,
                41124, 13636, 5496,  24964, 36236, 61460, 51064, 38796, 64508,
                13872, 28620, 42476, 22756, 12292, 62672, 3548,  51864, 38332,
                31460, 264,   41216, 8756,  47988, 11944, 28576, 15372, 18968,
                54616, 3096,  65068, 11560, 18164, 60980, 52836, 16240, 63252,
                23200, 400,   10788, 30104, 64196, 33752, 43288, 53752, 18964,
                32468, 24876,
            },
            {
                44340, 33916, 57364, 42624, 37884, 20616, 24860, 47300, 1120,
                51720, 14952, 54808, 31280, 12432, 42044, 18660, 37540, 51380,
                63412, 56412, 43072, 33024, 64056, 2656,  56996, 22004, 29808,
                44664, 24192, 10836, 35348, 28524, 5376,  44032, 21564, 12532,
                41268, 50284, 2108,  39032, 20788, 44400, 64932, 24484, 54016,
                33300, 56532, 9600,  30148, 46492, 11964, 37800, 19440, 48380,
                31352, 62244, 38760, 50964, 28948, 56572, 38040, 11592, 54348,
                61892, 9128,  51164, 28564, 12820, 37336, 5320,  10044, 62332,
                43180, 37176, 45700, 51376, 2296,  40428, 62788, 25256, 34952,
                1952,  29604, 58608, 32376, 3404,  13192, 41912, 1496,  30852,
                44636, 52560, 6196,  1660,  49368, 40936, 19588, 13860, 63744,
                56740, 15300, 52804, 65160, 32520, 43904, 6248,  40504, 62908,
                7576,  37872, 26876, 56276, 5228,  48868, 30900, 41356, 12472,
                50480, 33204, 47752, 26012, 4224,  21392, 57848, 28016, 46076,
                37752, 84,
            },
            {
                52876, 8372,  3788,  14868, 51820, 6624,  12312, 41100, 35196,
                22652, 61760, 43692, 24140, 4568,  65004, 2712,  47392, 23436,
                11840, 27560, 49488, 5976,  46440, 40284, 51340, 8816,  15904,
                59384, 49348, 20172, 64740, 40908, 53480, 9872,  48000, 61772,
                35648, 29048, 63472, 13752, 33012, 49136, 15756, 38412, 5512,
                47624, 3136,  39936, 24216, 5744,  63916, 43208, 59476, 23416,
                7116,  26472, 1440,  15392, 35188, 8648,  48780, 3560,  27632,
                47132, 35528, 1920,  54828, 44128, 23700, 57788, 51924, 29168,
                13652, 24208, 9060,  19080, 32964, 43660, 15008, 9456,  56640,
                44616, 22572, 49712, 64992, 46420, 27404, 55324, 22000, 58388,
                8476,  24800, 61976, 38120, 31636, 59672, 35600, 27304, 9940,
                48960, 29272, 19308, 24552, 4116,  55876, 51236, 23336, 30856,
                52680, 47324, 14924, 41816, 35068, 21000, 1968,  25880, 58536,
                43324, 20528, 61484, 14572, 45508, 36332, 7528,  13088, 4848,
                58812, 16068,
            },
            {
                27216, 19172, 46972, 62632, 27940, 33308, 57556, 63552, 4280,
                29940, 9732,  37248, 17228, 49588, 59628, 28924, 55732, 32248,
                41564, 17776, 36068, 60572, 14036, 19256, 27000, 35012, 62264,
                4504,  32036, 7636,  46236, 2588,  60204, 31072, 18420, 52076,
                5672,  22776, 46576, 7840,  27564, 52768, 9520,  30496, 21508,
                43492, 17880, 59912, 14780, 54784, 31952, 16676, 3108,  35860,
                41348, 54628, 20512, 46960, 62916, 44124, 19120, 59836, 15580,
                23448, 40376, 18184, 64648, 33696, 48728, 16388, 20328, 47012,
                1276,  54636, 61332, 29704, 5188,  58584, 47844, 31204, 51040,
                18152, 11484, 38924, 20044, 9052,  16072, 33740, 4984,  47912,
                11980, 34784, 19208, 56464, 15880, 7556,  47792, 43244, 22148,
                6848,  37524, 58980, 12944, 35640, 17572, 59484, 1700,  44728,
                10948, 19732, 29844, 59532, 8436,  45564, 64008, 36976, 3940,
                55052, 8284,  38248, 56228, 51336, 17940, 60752, 39924, 49668,
                22656, 64420,
            },
            {
                40228, 56160, 36308, 23536, 1724,  44052, 16564, 49204, 19424,
                46072, 58860, 6224,  53884, 33832, 39776, 14284, 20640, 7476,
                62040, 1232,  24932, 9044,  52620, 31604, 568,   48052, 43152,
                38264, 13424, 56604, 16960, 23100, 38404, 26548, 1396,  33292,
                15544, 42640, 56656, 19412, 61192, 42004, 2740,  62336, 57520,
                11216, 26428, 52084, 35384, 49128, 8600,  26928, 56628, 50840,
                10580, 60568, 32740, 4520,  12612, 24080, 30744, 51912, 34092,
                5232,  58116, 30908, 11020, 7256,  26968, 2828,  38668, 64132,
                41648, 34148, 16228, 49092, 38804, 20588, 26996, 396,   37244,
                62472, 54032, 6508,  42628, 56820, 37472, 51916, 18612, 63612,
                39312, 54088, 27900, 46308, 10856, 24284, 54644, 1180,  33724,
                61988, 45456, 2124,  41880, 47468, 27516, 38572, 13980, 34740,
                25500, 61700, 200,   50792, 23808, 55528, 11344, 15608, 32368,
                18516, 28328, 5064,  31396, 41736, 10560, 25164, 54588, 31068,
                35276, 10808,
            },
            {
                43524, 30100, 12180, 50476, 60396, 38296, 7800,  26196, 55584,
                39180, 13892, 28208, 21632, 108,   25468, 10212, 46000, 53416,
                38116, 57560, 29640, 44148, 22268, 55520, 65532, 11196, 21276,
                25060, 53768, 28264, 33856, 50400, 11160, 57968, 45504, 64500,
                53888, 10936, 37660, 592,   34040, 23924, 17308, 36688, 48820,
                32412, 64272, 984,   20952, 41516, 62324, 21992, 44960, 13508,
                29884, 17656, 42928, 52496, 57936, 39684, 1148,  42368, 63164,
                13028, 45620, 50584, 22092, 42108, 61272, 55608, 31088, 11548,
                26116, 5872,  57392, 23288, 11060, 65196, 55008, 13560, 24044,
                4592,  33800, 29148, 25392, 944,   60872, 30224, 43940, 26828,
                15148, 368,   42340, 4608,  65300, 30092, 18604, 58716, 51664,
                16152, 25636, 54976, 10168, 62356, 21348, 8228,  64444, 48152,
                56832, 6008,  38944, 32916, 17256, 28508, 39848, 48144, 53140,
                45148, 62464, 13528, 23024, 2148,  65000, 28908, 4052,  14900,
                47644, 2252,
            },
            {
                6644,  61780, 17608, 5068,  20728, 30448, 53520, 10728, 32524,
                2572,  64400, 50364, 44980, 57136, 41820, 63184, 34880, 3784,
                15624, 50108, 12220, 59612, 39392, 5656,  37168, 17372, 51952,
                3124,  61480, 44484, 6080,  63564, 14752, 20796, 40268, 8264,
                24304, 30352, 59668, 50620, 12628, 55304, 45752, 27936, 14124,
                7796,  38508, 45540, 29628, 12424, 4072,  34028, 39228, 324,
                63772, 25092, 37304, 8380,  27976, 16876, 55204, 7388,  20560,
                37948, 26100, 732,   53604, 36128, 15060, 45180, 8472,  52888,
                18836, 36864, 51600, 2476,  45428, 34392, 6840,  40240, 60068,
                43452, 16536, 48984, 63776, 14016, 47124, 10452, 23644, 7296,
                57652, 49280, 21948, 33828, 51080, 37096, 41392, 4068,  12200,
                39252, 20632, 29852, 33476, 50616, 4492,  54468, 31888, 16792,
                41944, 22032, 54008, 13136, 43644, 61424, 5620,  21740, 9736,
                24908, 40716, 49196, 59348, 34324, 47284, 43652, 19624, 62664,
                52448, 21588,
            },
            {
                26076, 48708, 34012, 54472, 40804, 45740, 14484, 61580, 22144,
                42520, 35344, 18684, 12052, 30952, 6788,  16892, 48452, 27032,
                43280, 21056, 33804, 3352,  46944, 14688, 28548, 58736, 32776,
                40168, 10268, 19240, 36132, 42144, 54420, 4220,  28120, 36416,
                49272, 4548,  16556, 26760, 39372, 7208,  60456, 3896,  53084,
                22940, 55496, 5024,  17516, 57076, 47340, 60248, 18888, 54300,
                46668, 5612,  49884, 21552, 65268, 35932, 11264, 49688, 29252,
                56232, 9068,  17156, 59360, 28776, 4288,  21400, 33844, 58864,
                47872, 13444, 60552, 31308, 41744, 15716, 52856, 28088, 46568,
                21404, 58240, 8272,  36092, 21904, 54652, 4192,  40508, 60308,
                36348, 30828, 17624, 60840, 7136,  14244, 26156, 63240, 31984,
                49288, 57820, 6456,  14660, 44168, 24248, 40216, 28408, 3524,
                10536, 35948, 63424, 7908,  49512, 2384,  58120, 34476, 64556,
                904,   36144, 53908, 20040, 7732,  16404, 57484, 36880, 8240,
                32192, 57976,
            },
            {
                38188, 15480, 8940,  58264, 24064, 432,   35592, 47852, 4888,
                52460, 25016, 8620,  62292, 38360, 52624, 60096, 23116, 31600,
                64816, 8452,  53828, 18144, 62724, 23544, 49804, 43748, 7672,
                26600, 57396, 49408, 40,    23744, 31724, 47928, 57168, 13336,
                62616, 19816, 43572, 47412, 65428, 21240, 31144, 42956, 19104,
                40636, 61752, 25368, 51160, 37164, 7668,  27592, 23984, 9500,
                33392, 15744, 59952, 3508,  32000, 48040, 24540, 59772, 43972,
                33124, 64392, 46616, 39492, 11704, 49192, 63540, 24620, 156,
                43340, 28348, 21112, 8708,  25056, 62060, 19148, 10112, 32400,
                3556,  12664, 52180, 41384, 31288, 17176, 34568, 50880, 13616,
                44704, 3636,  54388, 11572, 43668, 56752, 47024, 22732, 9116,
                44524, 2648,  64860, 36636, 18716, 60740, 12372, 49652, 59116,
                52236, 26880, 46232, 30236, 19540, 37500, 26264, 50848, 30484,
                17588, 6476,  11940, 26060, 39484, 51576, 720,   23944, 41576,
                11684, 45592,
            },
            {
                1340,  64340, 42676, 28180, 11900, 63236, 18228, 56300, 29324,
                15536, 58044, 1684,  47580, 27212, 20140, 2892,  11288, 56540,
                4656,  36924, 25904, 42072, 30992, 9124,  35520, 1824,  61960,
                16200, 45944, 12616, 29192, 62076, 9316,  17492, 44308, 1196,
                34444, 54880, 32032, 9952,  2972,  35684, 51084, 11588, 58636,
                34268, 9176,  14968, 32632, 64956, 42312, 14172, 50164, 58840,
                28732, 40260, 55680, 45252, 14320, 41228, 2496,  18160, 5984,
                13732, 23168, 3504,  31716, 19044, 56560, 37380, 40976, 16664,
                7328,  64312, 39060, 47120, 56100, 4728,  37420, 49836, 63444,
                56460, 38620, 26532, 2112,  59236, 45716, 62656, 28256, 20696,
                9420,  64208, 23472, 38164, 29060, 19388, 5304,  52480, 35708,
                17096, 27920, 41296, 51536, 56092, 740,   34664, 20904, 43584,
                18148, 1380,  14136, 60008, 23968, 42424, 11516, 14980, 44008,
                55472, 60224, 46384, 32908, 56308, 30012, 13820, 61540, 27860,
                54156, 18668,
            },
            {
                22636, 31012, 52352, 3808,  37848, 49752, 26000, 7176,  39764,
                44856, 32120, 37104, 55372, 14960, 43828, 35660, 50832, 40440,
                14420, 45824, 61184, 1168,  51460, 57140, 13348, 20736, 52988,
                38652, 32324, 21856, 58444, 50908, 37836, 25768, 60676, 22180,
                40984, 6852,  58928, 23584, 52492, 15492, 25980, 47032, 28,
                28452, 49180, 44672, 20356, 3340,  30284, 53356, 2424,  36712,
                43948, 11648, 19444, 26856, 8848,  62092, 54540, 27832, 37340,
                52532, 41632, 50248, 62256, 26624, 5756,  9816,  29848, 54752,
                50396, 33356, 2012,  12676, 52104, 29420, 42816, 864,   24464,
                17676, 30384, 47924, 19984, 6744,  11740, 24944, 2928,  55980,
                33164, 51840, 26612, 47808, 1864,  59712, 40076, 30460, 55440,
                60328, 21728, 12040, 25492, 9164,  30228, 46696, 5032,  64104,
                32428, 39428, 55312, 34172, 4108,  52832, 57372, 7352,  38236,
                20944, 28468, 41788, 3192,  63972, 10116, 45276, 49900, 33764,
                4604,  59916,
            },
            {
                35892, 13076, 47264, 19832, 32720, 58924, 43044, 10372, 65416,
                20308, 12396, 61248, 22912, 5816,  29892, 46768, 58968, 18780,
                28692, 55432, 21700, 11724, 38488, 26888, 64652, 48116, 24812,
                5576,  55060, 3564,  40796, 14860, 6712,  33108, 52924, 10904,
                50088, 27344, 13736, 37512, 42536, 61296, 7764,  18040, 64184,
                38020, 5720,  58056, 54456, 10912, 22900, 61472, 18112, 6168,
                22100, 63912, 916,   38576, 51856, 22408, 34108, 47268, 61292,
                20124, 8328,  35416, 14668, 44040, 53180, 46788, 13544, 62152,
                20192, 26268, 57264, 17828, 35220, 22336, 58768, 15172, 34640,
                44092, 8924,  53200, 65424, 42256, 37288, 57592, 48816, 39444,
                14816, 41620, 7472,  16232, 62836, 34604, 10964, 15248, 176,
                46644, 7404,  33092, 48304, 62780, 39164, 15764, 54084, 25884,
                7444,  22592, 48704, 9340,  65420, 18316, 31912, 47348, 63272,
                4600,  50224, 14344, 23768, 18384, 37560, 21820, 6428,  16956,
                40156, 48944,
            },
            {
                57088, 26832, 7720,  55200, 14584, 5240,  22464, 34448, 51756,
                27828, 4084,  48956, 41612, 10416, 64524, 704,   24312, 8808,
                34232, 5636,  49088, 16748, 33332, 6564,  41212, 17844, 30368,
                11284, 63404, 34968, 18596, 44812, 64924, 2420,  39424, 16320,
                30076, 64092, 18960, 1616,  57036, 29040, 40112, 54872, 32072,
                21912, 13096, 27060, 40408, 34892, 47952, 38724, 45388, 32264,
                53896, 48292, 34876, 56980, 30088, 16652, 4976,  12552, 1884,
                55280, 25176, 57680, 1172,  21580, 60804, 34136, 23720, 3296,
                36972, 9256,  45560, 41132, 61560, 7240,  11480, 47680, 54660,
                5480,  60528, 14428, 23404, 32536, 16668, 8456,  30176, 18772,
                620,   46168, 57256, 31216, 53212, 22460, 49484, 25940, 42908,
                64052, 38008, 53668, 17792, 5664,  22948, 59664, 36532, 12896,
                42180, 58088, 16344, 44964, 28072, 40436, 2016,  23312, 27044,
                10492, 34400, 61208, 52420, 7420,  59548, 42696, 55900, 65284,
                25284, 9844,
            },
            {
                3240,  62832, 39264, 44740, 61656, 29432, 53556, 16492, 884,
                45788, 58604, 17712, 35136, 54428, 32864, 16168, 39220, 52144,
                63584, 43772, 24960, 60076, 53644, 46340, 3100,  36512, 60772,
                43184, 47492, 26844, 8932,  54312, 28660, 20564, 56848, 46612,
                3856,  43896, 54268, 47308, 33048, 20964, 12356, 3256,  44140,
                52812, 60760, 16724, 1320,  62972, 7512,  26444, 15512, 59156,
                9184,  24972, 13960, 6616,  46112, 42592, 64660, 39112, 31612,
                45044, 29188, 10852, 40532, 30608, 16500, 4956,  42996, 48688,
                59708, 31404, 14684, 3864,  49948, 27584, 32588, 64168, 20180,
                40296, 28516, 36384, 1020,  49880, 54068, 43916, 63572, 22980,
                59544, 35476, 12128, 20468, 3244,  41280, 7776,  57636, 19048,
                12924, 28856, 3516,  41776, 56792, 44632, 28276, 10088, 51384,
                61812, 30120, 5168,  37036, 12208, 51316, 61744, 36116, 16700,
                54936, 39948, 236,   30348, 46896, 34996, 27460, 1448,  31384,
                14728, 43688,
            },
            {
                21044, 33644, 16384, 24508, 2352,  41180, 47060, 60380, 38700,
                24736, 30964, 7416,  51224, 26308, 20976, 47836, 59892, 13092,
                4096,  31132, 40124, 9892,  29044, 21332, 56728, 8636,  50664,
                15704, 860,   22472, 60148, 48832, 11948, 36372, 24112, 59404,
                8864,  35296, 25108, 10720, 6272,  49912, 62940, 35088, 24724,
                9496,  41920, 30844, 51052, 19184, 55948, 12184, 51392, 1740,
                41416, 30576, 62240, 20444, 59736, 9780,  25576, 51012, 15188,
                59104, 18128, 48564, 65516, 51908, 38204, 58544, 27532, 18136,
                39996, 54212, 65056, 24924, 19204, 43908, 38520, 2408,  25536,
                51520, 12156, 46580, 58112, 26808, 4820,  13200, 34380, 5736,
                52912, 25648, 65116, 37796, 44948, 28200, 61908, 36256, 32176,
                51108, 23372, 58792, 35116, 13420, 19992, 1356,  32604, 46964,
                2784,  20208, 50144, 25456, 57112, 21388, 8308,  44720, 59120,
                48748, 19988, 43596, 25548, 15700, 53724, 12076, 19920, 51220,
                37008, 53572,
            },
            {
                28568, 48004, 58192, 10976, 36528, 20836, 6836,  12940, 55264,
                9720,  63824, 40192, 13676, 57608, 6200,  42612, 27952, 37192,
                19568, 57340, 15140, 240,   64068, 42412, 13976, 23904, 28124,
                33760, 57292, 38216, 41836, 31964, 5964,  51544, 42472, 14268,
                31632, 62088, 38776, 17044, 59016, 26800, 45376, 15400, 57904,
                48340, 4088,  36056, 46280, 24360, 33492, 43224, 28900, 65212,
                18496, 36240, 50120, 3196,  28144, 35536, 55620, 21720, 7572,
                42092, 3316,  35956, 6860,  25688, 12640, 8624,  50628, 1532,
                11880, 21896, 6684,  36208, 58104, 53088, 9544,  16680, 59656,
                31608, 4060,  63008, 17908, 21548, 39512, 60580, 47072, 28048,
                42204, 9488,  4420,  50448, 13856, 55756, 16816, 4992,  46140,
                1996,  61304, 9424,  26676, 49320, 52728, 65084, 39984, 24012,
                14372, 34612, 41108, 60312, 1008,  33160, 14696, 29524, 2908,
                12672, 32288, 5668,  62612, 9000,  58652, 40888, 4720,  45688,
                60032, 7252,
            },
            {
                41844, 504,   52148, 31648, 64800, 50204, 26140, 32436, 36112,
                20080, 44104, 23464, 2548,  36660, 63300, 11440, 1796,  55268,
                23236, 50180, 45876, 35724, 18868, 48920, 31544, 59380, 52748,
                6904,  19636, 13292, 4016,  25208, 16916, 61628, 380,   19620,
                49476, 5092,  21776, 52264, 41660, 712,   37376, 19680, 6856,
                28952, 21152, 64696, 14168, 5832,  58224, 3760,  37720, 22748,
                54228, 11280, 44708, 39896, 52932, 17728, 212,   47904, 37528,
                63180, 24008, 54632, 20648, 45640, 32332, 55048, 62772, 34624,
                29084, 46572, 51772, 30884, 348,   13384, 46752, 55924, 35496,
                22660, 44344, 9032,  41580, 29828, 56236, 10400, 20220, 15528,
                49248, 30840, 18048, 59196, 32832, 24796, 10292, 52544, 21304,
                38472, 43564, 16180, 31640, 5260,  37416, 8652,  17292, 56488,
                53660, 63784, 9820,  17620, 47528, 55340, 42188, 53220, 38968,
                64252, 23148, 55560, 50024, 38068, 22244, 33040, 63264, 24348,
                10772, 18192,
            },
            {
                63936, 23608, 13928, 45468, 8552,  56456, 16984, 61804, 2040,
                50572, 15796, 54760, 46356, 29780, 18240, 49560, 31928, 44552,
                9224,  63132, 27292, 7384,  54688, 11392, 37432, 1840,  40544,
                65336, 44468, 50240, 62884, 56188, 45532, 34452, 53992, 26744,
                40516, 56520, 29308, 13812, 64996, 30564, 55712, 10952, 61444,
                51868, 38512, 55220, 9792,  40500, 61712, 20204, 48460, 8184,
                57612, 4756,  16056, 23912, 58292, 12232, 32680, 60816, 27196,
                11532, 33536, 14336, 61676, 676,   40916, 19020, 23604, 43592,
                57504, 16160, 10344, 42584, 63116, 23400, 29304, 41196, 7436,
                49820, 15312, 54176, 33792, 3344,  51024, 36300, 1292,  61732,
                39080, 56524, 22752, 43252, 948,   47748, 40148, 64676, 30140,
                12064, 55248, 47812, 62640, 21712, 57628, 46080, 29676, 4516,
                43576, 27772, 6548,  30708, 37648, 23732, 4008,  19116, 26476,
                46004, 9828,  35704, 17704, 3172,  47568, 14132, 29108, 49656,
                34376, 54824,
            },
            {
                30980, 5224,  37608, 19480, 28056, 4184,  43276, 47788, 29180,
                58416, 5028,  33500, 60820, 8096,  52652, 24504, 61308, 35056,
                16396, 3128,  39440, 33232, 62072, 21932, 5820,  46748, 17740,
                11056, 27352, 30776, 36568, 21316, 10512, 29428, 7500,  64408,
                11652, 45108, 2228,  47696, 8616,  23152, 50352, 40996, 32828,
                25412, 1096,  17808, 30264, 26224, 50360, 16296, 31956, 45580,
                27228, 34152, 61076, 31208, 7964,  46600, 42440, 19428, 5008,
                53408, 49572, 43292, 29488, 58152, 48160, 14620, 5928,  37368,
                3844,  61636, 26652, 38764, 17944, 34180, 60196, 4448,  19924,
                64972, 38152, 24692, 61084, 13612, 45496, 25512, 32308, 53800,
                7200,  12948, 36456, 63324, 8660,  19284, 34768, 58352, 26136,
                6576,  18800, 35696, 1000,  42216, 14824, 25320, 61748, 36284,
                12360, 22420, 52216, 45636, 63020, 11244, 60768, 49536, 7396,
                15944, 58804, 29728, 42692, 27040, 57188, 6240,  43436, 1940,
                15408, 39696,
            },
            {
                9216,  48712, 61064, 53760, 40580, 33848, 22368, 11472, 38792,
                8904,  41940, 26604, 20908, 38420, 14768, 40484, 5528,  21328,
                53496, 59428, 24324, 13668, 51404, 43380, 26084, 61228, 34672,
                22808, 54908, 8448,  1312,  15328, 41092, 48284, 37804, 17932,
                33580, 23692, 60416, 36704, 19052, 34620, 3628,  16592, 46844,
                13112, 43212, 59804, 45784, 53712, 35608, 11072, 992,   63224,
                13480, 41724, 48824, 1632,  37812, 64360, 14984, 56592, 30544,
                39636, 8016,  17836, 4232,  36204, 10004, 27284, 63960, 49780,
                20628, 33088, 48504, 3028,  54756, 8428,  48928, 26944, 12596,
                56544, 804,   28748, 48184, 6540,  58040, 16856, 64356, 21676,
                44624, 3568,  26356, 51356, 29436, 54816, 4476,  14884, 42484,
                49008, 60448, 23868, 29104, 11620, 50680, 33328, 2436,  19564,
                49068, 39248, 58528, 15908, 5368,  35192, 28580, 56852, 32852,
                37912, 52356, 1184,  12376, 64784, 19736, 53544, 36084, 62168,
                21444, 58940,
            },
            {
                42656, 17136, 25240, 2488,  57796, 14520, 63344, 51996, 24776,
                19132, 64700, 12304, 48940, 972,   58588, 27892, 47144, 12452,
                42772, 30332, 48204, 4996,  58184, 29648, 16040, 56020, 49556,
                3824,  39724, 46044, 61132, 51068, 57064, 22724, 2552,  58992,
                50764, 15688, 6480,  41976, 53608, 62248, 26672, 58236, 5108,
                63692, 22340, 33960, 7204,  3056,  23344, 42860, 56340, 38956,
                24928, 52516, 18056, 21876, 28648, 51232, 25084, 2616,  45624,
                22164, 58824, 26188, 51572, 21600, 55500, 39176, 31112, 53440,
                8688,  13760, 56584, 22764, 43804, 15088, 52468, 36428, 45528,
                31060, 42576, 10616, 18880, 23164, 37284, 42348, 8992,  28824,
                48468, 33508, 60052, 15748, 38540, 45152, 23636, 50872, 2580,
                33412, 9100,  40092, 53720, 63892, 7464,  44284, 53144, 60188,
                9012,  32516, 196,   26192, 21168, 40704, 13776, 43580, 21736,
                4464,  62068, 24488, 50496, 39568, 31900, 9976,  24916, 51696,
                6732,  27236,
            },
            {
                56780, 35552, 12056, 30616, 44900, 6464,  36732, 44,    55908,
                45720, 35676, 54056, 31592, 44012, 10260, 54840, 65288, 2060,
                36944, 18376, 9644,  40676, 20032, 2532,  38056, 7876,  31140,
                13552, 58972, 25316, 18288, 32712, 5060,  63308, 25936, 9412,
                39640, 28500, 56136, 31400, 10104, 14240, 43980, 37908, 29796,
                53128, 10408, 39844, 57148, 15104, 64528, 28936, 51344, 20872,
                5824,  9620,  62508, 55368, 11296, 41016, 7060,  36524, 63320,
                10868, 35064, 62320, 12824, 44940, 60684, 3304,  17232, 44564,
                25400, 39860, 60744, 6632,  31560, 64136, 20584, 2512,  61776,
                16336, 50488, 34980, 63316, 53452, 30440, 2160,  52208, 14468,
                57588, 41040, 10680, 21076, 5776,  61188, 12192, 28392, 63664,
                55816, 15860, 46196, 4268,  37920, 16912, 22816, 40488, 27100,
                14436, 64904, 43188, 56388, 50104, 54236, 1664,  64492, 10480,
                47692, 18792, 33876, 44604, 7552,  60644, 16412, 46728, 38424,
                12956, 45572,
            },
            {
                812,   52928, 63840, 21048, 50824, 27380, 16688, 59832, 29836,
                3848,  14844, 6980,  61144, 22904, 36136, 19360, 33144, 25116,
                57012, 51056, 62720, 32876, 45480, 53688, 11656, 63832, 42888,
                19744, 52160, 35836, 11460, 43440, 28752, 13588, 46324, 54448,
                43824, 3156,  64112, 24832, 49420, 1012,  20688, 50676, 7924,
                17496, 48112, 27852, 20120, 49260, 37300, 7920,  17152, 60620,
                31340, 36484, 44116, 3376,  34580, 59884, 20416, 54096, 16920,
                48228, 1548,  41532, 29872, 7112,  33692, 23992, 11932, 58220,
                1832,  34896, 18472, 47088, 28004, 38032, 10668, 24824, 40492,
                6916,  22132, 57156, 3948,  44316, 11848, 59440, 39352, 19716,
                24952, 76,    53324, 47152, 31548, 56448, 41496, 17984, 37708,
                21672, 31128, 59308, 26864, 20280, 58076, 30540, 55460, 5424,
                46724, 18724, 37152, 6952,  31680, 18140, 46100, 25300, 30572,
                39360, 56280, 5680,  14492, 22508, 55648, 1656,  28340, 58340,
                19024, 32340,
            },
            {
                22708, 4692,  39364, 9468,  61396, 41328, 48840, 21748, 33364,
                40480, 50724, 17896, 28596, 3412,  51616, 15360, 7640,  44748,
                28632, 4320,  15064, 22928, 27156, 60044, 34216, 23928, 47204,
                28220, 2636,  64656, 6892,  38840, 53472, 20068, 35336, 30788,
                18936, 12652, 21384, 46028, 38904, 59956, 33572, 55736, 23936,
                36336, 62296, 3684,  32128, 60280, 11756, 46716, 34440, 4164,
                48108, 58304, 15236, 25728, 46924, 30436, 13472, 43460, 27124,
                32224, 56244, 15604, 49940, 19344, 53080, 42404, 47596, 65080,
                29368, 50804, 41708, 12648, 640,   58508, 48208, 33132, 59392,
                52984, 27644, 9144,  38232, 15576, 25724, 33980, 49716, 4664,
                62140, 35804, 17600, 64728, 25776, 1696,  35072, 7784,  44716,
                11176, 464,   51256, 13124, 34116, 45600, 1328,  11352, 34632,
                51332, 23960, 29396, 12668, 61568, 9384,  36148, 58564, 16212,
                53232, 27780, 60004, 37108, 29988, 49216, 41992, 34304, 5712,
                65108, 50188,
            },
            {
                29328, 15488, 46760, 34000, 18592, 7076,  12504, 44432, 9916,
                62608, 25876, 46672, 57964, 42012, 63084, 39500, 49576, 60860,
                11488, 38376, 54376, 7980,  48732, 764,   17644, 40416, 5676,
                57676, 15652, 31392, 22324, 60424, 48192, 1704,  61840, 6660,
                58472, 36324, 52628, 5036,  16400, 27876, 11408, 65204, 2132,
                41600, 13244, 54656, 43892, 24556, 164,   41236, 25996, 55096,
                13044, 28312, 39940, 19152, 65276, 5516,  49480, 58512, 8720,
                22876, 4732,  38092, 25464, 62912, 256,   14776, 36392, 6212,
                21416, 9464,  62044, 53960, 23660, 43100, 55696, 14536, 3764,
                19320, 46056, 31940, 61152, 47756, 20844, 65356, 8324,  45896,
                30248, 43320, 7508,  37952, 14156, 50216, 22864, 52816, 62520,
                48412, 24708, 41256, 64964, 8736,  49228, 60916, 16012, 38932,
                57152, 2324,  59592, 48436, 39516, 22552, 51704, 2988,  7748,
                42156, 11784, 588,   46348, 63352, 20092, 12244, 52616, 24028,
                10328, 41128,
            },
            {
                60108, 54896, 25148, 57568, 31104, 52484, 24316, 54768, 5044,
                57092, 20760, 38028, 8752,  12568, 24544, 30020, 320,   22020,
                17356, 46656, 64484, 35712, 41900, 12980, 51000, 61464, 9912,
                36740, 50100, 45312, 55156, 9280,  16512, 24688, 41780, 10860,
                48804, 26860, 62872, 32404, 8384,  42452, 47704, 19180, 31368,
                46096, 26592, 6072,  17024, 52296, 56944, 19748, 63364, 43272,
                22740, 1364,  51092, 8612,  53580, 23696, 468,   33708, 39368,
                51836, 64560, 46092, 10464, 58312, 31736, 40080, 56888, 27588,
                54844, 32812, 16272, 36808, 5332,  18860, 8220,  28836, 35876,
                63764, 12136, 41424, 612,   55328, 5840,  28372, 16696, 56336,
                12800, 21488, 27508, 57828, 45260, 10200, 59680, 29284, 5880,
                33016, 18308, 53928, 36896, 4580,  24096, 42532, 28572, 63848,
                20820, 9864,  44620, 16376, 3964,  27456, 43416, 62124, 32568,
                23044, 50268, 17528, 25844, 8748,  39716, 4124,  61500, 44784,
                17684, 36356,
            },
            {
                6936,  13068, 43676, 5456,  1264,  36988, 65524, 29092, 34984,
                13680, 1044,  31696, 53376, 34412, 5904,  55348, 42672, 34808,
                58276, 31176, 3484,  18996, 59164, 29196, 21756, 32184, 54020,
                25708, 20396, 52,    41060, 26992, 37260, 51700, 32620, 55684,
                17312, 472,   41204, 23528, 59128, 54252, 36868, 5592,  15028,
                57512, 61328, 35032, 38396, 8276,  30036, 14672, 32676, 9960,
                53196, 62212, 45164, 31780, 37784, 16568, 41948, 61740, 18736,
                12344, 29312, 20876, 35364, 7616,  48448, 22328, 11032, 18196,
                44324, 3020,  48992, 26524, 30792, 64596, 39348, 50740, 43736,
                25572, 49308, 17384, 23756, 35172, 51624, 42504, 38676, 32624,
                52960, 61260, 49316, 4776,  32808, 20132, 42344, 16008, 38892,
                58480, 3592,  28080, 14940, 56124, 32112, 19076, 52872, 7048,
                26152, 41344, 32480, 54584, 64288, 34964, 19416, 14224, 48064,
                37984, 65520, 57120, 35532, 54044, 15984, 32020, 26328, 56508,
                2500,  47992,
            },
            {
                27088, 21704, 38548, 49400, 59328, 15792, 47016, 39884, 17752,
                42732, 48048, 64352, 16424, 45116, 59540, 18692, 13516, 51880,
                8692,  26536, 43716, 24116, 52772, 5296,  38612, 3188,  14388,
                43684, 62400, 12368, 34008, 59044, 4796,  13868, 65476, 22256,
                37880, 45308, 14316, 50392, 20108, 2980,  62032, 25552, 50116,
                20928, 10076, 51152, 22272, 64780, 40444, 49776, 2756,  37680,
                18236, 6448,  35380, 12400, 59596, 55208, 27756, 6752,  48100,
                57428, 2388,  42668, 55076, 17268, 26588, 61836, 4544,  51612,
                63372, 38268, 13572, 59868, 45476, 52436, 11616, 22176, 1912,
                57048, 6368,  59940, 30388, 10220, 14048, 58400, 1456,  23588,
                9856,  3116,  39920, 16976, 53912, 63988, 2176,  26280, 54736,
                12556, 44112, 47212, 22212, 62800, 38732, 12332, 2844,  46808,
                36464, 13308, 50816, 24772, 6656,  57760, 11096, 54024, 28704,
                4936,  20244, 31412, 2840,  44240, 59248, 50744, 37172, 14200,
                30592, 63420,
            },
            {
                34816, 10532, 61848, 18988, 28112, 8468,  22060, 3428,  60316,
                51680, 23136, 28352, 2652,  21800, 48512, 27492, 64156, 38928,
                2412,  55888, 14652, 10596, 33608, 45072, 55592, 65152, 48136,
                28888, 7276,  56332, 17868, 29916, 47020, 43488, 2628,  28096,
                8068,  30688, 57620, 9956,  35228, 29360, 12176, 44332, 39028,
                33324, 1192,  29172, 47216, 12352, 5560,  57920, 26560, 61140,
                47700, 29364, 24696, 20064, 2984,  46160, 10744, 35796, 25324,
                15120, 32464, 60668, 5252,  45240, 52892, 30912, 42236, 34720,
                24040, 7652,  57268, 20480, 2444,  34532, 16884, 61016, 32380,
                15376, 36784, 53064, 40120, 62968, 44824, 26732, 19356, 63632,
                46988, 34912, 24768, 11804, 30024, 37076, 47876, 8972,  34208,
                20720, 61948, 31004, 9988,  6612,  43404, 49612, 60536, 29792,
                55360, 62200, 736,   18556, 30316, 46364, 40396, 1432,  60176,
                44992, 10024, 41504, 13440, 23392, 6092,  21040, 42712, 9004,
                19576, 53048,
            },
            {
                58328, 1828,  42088, 32356, 54460, 44936, 11856, 56212, 26292,
                7452,  10996, 57324, 40764, 36504, 9784,  4140,  32104, 23380,
                47236, 61972, 39904, 49444, 63088, 16256, 8592,  24592, 18748,
                34748, 39188, 49220, 23480, 63232, 8488,  20848, 58180, 48240,
                52168, 62448, 3676,  39868, 46544, 52692, 17568, 58432, 7952,
                53856, 42812, 18300, 60540, 25160, 45020, 33964, 21300, 42148,
                15252, 56712, 39808, 64016, 49336, 30732, 21264, 63464, 44300,
                52704, 40016, 24308, 11360, 38580, 13916, 1676,  59488, 15272,
                29248, 47248, 40876, 10508, 25452, 42172, 5584,  53792, 46416,
                9692,  28300, 19512, 2744,  21972, 48872, 4972,  36352, 50984,
                14996, 41984, 55616, 60136, 44540, 5660,  23216, 60896, 50160,
                40664, 1540,  51476, 36024, 58996, 16620, 24756, 34340, 9208,
                15824, 22404, 43332, 38592, 8916,  50228, 21804, 25620, 33464,
                16644, 55688, 27176, 62368, 47172, 29444, 64580, 124,   49256,
                39796, 45396,
            },
            {
                16408, 29784, 51656, 13728, 23452, 63576, 33836, 49584, 30528,
                38536, 62360, 32612, 14272, 54396, 60660, 43508, 50960, 11712,
                20168, 35524, 29700, 6268,  21096, 27768, 36424, 42828, 1388,
                60372, 11248, 5312,  42232, 52540, 15720, 35300, 39668, 12120,
                18120, 34084, 24160, 15476, 27052, 64608, 1812,  30968, 23688,
                62812, 14368, 55904, 3460,  36928, 16292, 54400, 4344,  11192,
                51828, 2020,  9172,  43116, 15924, 4536,  38696, 56192, 1004,
                8888,  19468, 49888, 28212, 65512, 21284, 36496, 9560,  49792,
                19144, 700,   32328, 55084, 63040, 48812, 29768, 23304, 38532,
                65124, 42308, 47572, 7896,  33360, 54972, 11044, 60500, 30812,
                6176,  27636, 20360, 456,   51744, 18532, 27964, 56084, 14992,
                6372,  25712, 18508, 54904, 27596, 276,   53412, 20624, 40100,
                57912, 6056,  33736, 53240, 60596, 15040, 35972, 63884, 5732,
                51768, 37072, 49080, 7544,  35108, 52916, 11396, 57336, 32708,
                23620, 4456,
            },
            {
                25700, 64692, 7604,  36048, 4876,  39400, 560,   18528, 15200,
                45888, 5096,  19332, 50384, 7000,  25268, 16616, 37768, 57444,
                6888,  17272, 53552, 1124,  59768, 46772, 12688, 58232, 50732,
                22056, 54076, 32344, 26496, 3152,  60236, 31408, 55088, 26148,
                5144,  42444, 55956, 59828, 6828,  21724, 41924, 48348, 36168,
                4924,  27356, 39612, 30784, 52012, 8740,  64036, 30204, 36232,
                59684, 32628, 27260, 53224, 58356, 24172, 51144, 13624, 28680,
                34812, 62260, 3552,  56440, 32872, 46260, 54132, 26032, 43248,
                63928, 52660, 22680, 36980, 13780, 7900,  59344, 19068, 148,
                12936, 56344, 24300, 59136, 14604, 28968, 39572, 17304, 44440,
                57664, 9136,  64844, 34068, 13708, 39836, 10332, 43036, 30584,
                37412, 65376, 12196, 41332, 32560, 46452, 64172, 4416,  44436,
                26932, 47880, 65208, 12340, 27912, 2516,  56672, 42720, 12796,
                21216, 58964, 1032,  18956, 14848, 38992, 26168, 17864, 60972,
                12984, 55828,
            },
            {
                38600, 44084, 17888, 48364, 57436, 43240, 25304, 61492, 35248,
                53944, 42552, 24328, 46892, 34820, 30132, 62980, 632,   27916,
                45996, 60480, 24784, 40760, 31916, 55972, 3984,  39472, 29508,
                16880, 64164, 14312, 45648, 38292, 10620, 23184, 1104,  64188,
                46328, 20312, 10892, 49012, 32280, 37636, 13476, 9616,  51788,
                20440, 45960, 11148, 59220, 20024, 43356, 23964, 49020, 17840,
                45664, 22120, 13060, 36672, 7120,  33528, 41540, 18080, 58788,
                47232, 23088, 16044, 41376, 7268,  18252, 3868,  57348, 38808,
                12712, 6132,  58372, 17120, 27412, 40040, 44780, 54900, 35100,
                27180, 4804,  51484, 35872, 43760, 64376, 25020, 2540,  53636,
                22644, 37232, 49268, 45860, 31212, 57416, 63356, 3228,  46700,
                22608, 57860, 48948, 5216,  21860, 10732, 36544, 13952, 51468,
                31496, 1924,  19492, 23648, 45764, 31832, 18344, 48444, 8628,
                30188, 40616, 24896, 32512, 61412, 45268, 3668,  41972, 36272,
                6452,  50348,
            },
            {
                9772,  21524, 54112, 27480, 11340, 20536, 50844, 58228, 9796,
                2216,  28744, 64916, 3904,  58796, 10776, 20864, 41380, 52220,
                33444, 13620, 43920, 10124, 50556, 18560, 25856, 33812, 9148,
                47316, 2220,  36040, 56576, 19996, 47676, 51292, 40932, 14208,
                35744, 53484, 29732, 580,   44568, 54356, 61180, 16940, 57564,
                33060, 65392, 336,   48604, 33788, 6256,  13556, 61344, 928,
                40880, 5288,  62556, 46840, 19612, 65048, 10304, 31664, 5436,
                43008, 54392, 37584, 10660, 48508, 62816, 30504, 22924, 8132,
                33596, 28796, 45420, 3436,  51128, 32848, 4368,  15820, 50184,
                62164, 31864, 17068, 10572, 1128,  19192, 50544, 33904, 13212,
                41000, 3852,  16788, 25560, 7460,  21872, 35244, 53092, 17280,
                7792,  33648, 44652, 15816, 55884, 61092, 29572, 18788, 59632,
                9836,  55540, 37112, 41680, 7284,  59692, 38224, 26712, 54684,
                45964, 63304, 11092, 50648, 56436, 21844, 8892,  53948, 27896,
                46496, 31372,
            },
            {
                1688,  59260, 33512, 3064,  63104, 30200, 7308,  14080, 38036,
                21440, 56588, 12016, 17328, 39052, 44676, 55724, 15136, 8836,
                65412, 2912,  37364, 21664, 61764, 14792, 6064,  52668, 59180,
                43432, 24288, 28284, 8144,  62996, 17160, 29576, 6648,  60592,
                27468, 8112,  39244, 63660, 18576, 25744, 4196,  29000, 43548,
                7180,  25052, 15180, 41172, 26636, 57108, 38320, 28944, 55480,
                25584, 51208, 16776, 29692, 54288, 1528,  48720, 61040, 26272,
                12804, 2096,  29540, 59660, 25732, 13716, 35628, 51548, 16416,
                60144, 48656, 40956, 64752, 21420, 11836, 61096, 24036, 9540,
                38900, 21108, 45976, 60332, 40780, 56764, 6948,  46124, 62440,
                29832, 59784, 11356, 54892, 62184, 1928,  48580, 26700, 11184,
                61672, 28672, 1836,  38636, 26208, 49500, 42456, 6812,  38920,
                25384, 16504, 50036, 61916, 11260, 52796, 4896,  22684, 2940,
                35480, 16324, 5540,  42648, 2304,  30468, 48908, 64232, 19752,
                15192, 62100,
            },
            {
                41484, 13596, 37832, 45500, 16236, 40924, 33000, 46240, 26072,
                48568, 41320, 31836, 52860, 23668, 2288,  49740, 32272, 23016,
                48180, 26504, 54912, 7088,  35428, 45452, 64432, 41264, 19336,
                11888, 61264, 51792, 40204, 4572,  34144, 57704, 44392, 21640,
                50416, 16712, 58896, 23316, 11796, 34828, 49892, 40032, 22848,
                54996, 37596, 50780, 18676, 62008, 2724,  46632, 20556, 9756,
                34336, 60020, 43412, 11400, 38528, 25072, 44208, 20832, 52980,
                35864, 63260, 18928, 51028, 43468, 72,    40140, 61600, 44412,
                2480,  19280, 24828, 8988,  53184, 35740, 57352, 29052, 41664,
                48424, 7092,  53812, 26648, 30108, 22208, 37688, 10008, 26920,
                19572, 50852, 33112, 42636, 38200, 14180, 19496, 42060, 39056,
                54572, 50504, 20420, 58884, 9360,  3588,  23524, 54012, 46052,
                63368, 34752, 4144,  29176, 15320, 34096, 49456, 64760, 13304,
                56956, 27752, 20260, 59640, 37616, 25376, 11896, 34368, 4688,
                56732, 23268,
            },
            {
                28764, 52372, 6680,  24224, 54868, 60356, 4188,  53000, 63888,
                6276,  15404, 35588, 61680, 8012,  27132, 36736, 62156, 4708,
                39792, 16052, 59420, 30596, 48884, 27496, 23144, 932,   31256,
                37552, 3872,  21520, 31712, 13020, 54208, 24732, 10276, 37900,
                3776,  32652, 42124, 2788,  46588, 57792, 8548,  63108, 2268,
                12580, 60736, 29656, 9220,  53736, 12132, 32256, 52300, 15596,
                39660, 7612,  22832, 3544,  57176, 33968, 6832,  16132, 39772,
                8824,  45696, 33288, 22412, 6572,  55748, 19916, 10584, 26748,
                55272, 14020, 37888, 31172, 776,   43504, 47164, 19448, 1892,
                63012, 14836, 36884, 4236,  13316, 61352, 48608, 16148, 55384,
                848,   44500, 23444, 4680,  27944, 51860, 30824, 58272, 3988,
                24264, 13396, 43480, 31576, 35612, 62460, 14800, 33068, 1280,
                12468, 21516, 43056, 57656, 25196, 40412, 17404, 43996, 31076,
                39944, 52052, 47804, 33548, 14032, 55108, 17500, 39664, 51524,
                43952, 35508,
            },
            {
                47456, 65328, 19288, 48952, 10244, 36216, 22604, 17804, 29440,
                59000, 8,     50892, 19972, 47148, 57784, 12864, 18744, 29228,
                56972, 45056, 19828, 11212, 3740,  39008, 12976, 57432, 54548,
                48156, 16096, 44556, 58460, 49032, 42468, 1964,  15888, 65244,
                47948, 56384, 13344, 51980, 27116, 37424, 15532, 31300, 19664,
                47396, 34260, 3932,  44648, 36000, 23708, 42052, 64824, 4616,
                56860, 30420, 49036, 63468, 27652, 13964, 60268, 49828, 30488,
                56452, 4064,  14568, 58252, 37024, 28712, 47272, 31992, 5428,
                35036, 50652, 57908, 61852, 17760, 26408, 13884, 5508,  51992,
                30832, 24680, 55580, 44080, 34488, 52384, 3060,  31376, 65252,
                39232, 8036,  58740, 17472, 61088, 46132, 8596,  63520, 16360,
                36188, 6844,  64224, 17700, 51644, 46664, 20000, 40872, 58424,
                30416, 52292, 8944,  47136, 4,     55012, 21080, 9944,  59168,
                1516,  23852, 9428,  62644, 6608,  44328, 61552, 1112,  26512,
                7700,  10940,
            },
            {
                16860, 372,   31932, 57004, 28184, 2392,  44536, 12284, 37468,
                42288, 24900, 10440, 29908, 40312, 5192,  43128, 53012, 9624,
                34680, 784,   41892, 53364, 63048, 50996, 17664, 34924, 28708,
                6588,  25828, 63548, 9444,  18224, 27416, 61548, 51632, 34936,
                25696, 19404, 30064, 60724, 5724,  21084, 55672, 44312, 53172,
                40684, 16772, 58772, 21004, 63024, 6976,  17436, 49404, 26688,
                45872, 12988, 35736, 18904, 40608, 46164, 22184, 520,   64076,
                23876, 27504, 48600, 10284, 64468, 16808, 52520, 62564, 41916,
                21324, 46012, 11124, 6516,  40692, 56048, 63900, 38132, 34356,
                58368, 10792, 20320, 64276, 9016,  18808, 41584, 23996, 35868,
                47936, 14052, 53488, 35412, 10748, 22620, 0,     32736, 44592,
                48372, 29984, 56692, 952,   12020, 28168, 56116, 8152,  26288,
                48200, 38080, 18452, 64020, 32900, 7340,  61692, 26316, 35952,
                6444,  45172, 18088, 38744, 28676, 22600, 48076, 30944, 58004,
                21448, 60564,
            },
            {
                37184, 26176, 42992, 39156, 14612, 62984, 51464, 57368, 7648,
                20508, 54452, 45264, 64572, 14624, 32912, 60252, 22312, 50436,
                64320, 24728, 14360, 32920, 22424, 8056,  43308, 59972, 10184,
                41884, 52204, 34320, 448,   37100, 22260, 39704, 30684, 12092,
                6260,  45140, 8920,  39888, 33256, 64464, 1384,  24872, 9824,
                27796, 6048,  48896, 25992, 39332, 29204, 57740, 508,   36788,
                21132, 61864, 2128,  52944, 9076,  55232, 36952, 11892, 42416,
                17332, 38256, 53816, 41704, 1848,  23684, 7932,  13408, 58756,
                1256,  28956, 23900, 34056, 49428, 30308, 10132, 22936, 45440,
                16900, 42352, 49744, 384,   26296, 45660, 57240, 6152,  11540,
                21248, 26616, 30604, 49240, 40840, 56432, 37732, 25868, 55240,
                10320, 21624, 40180, 25008, 42716, 4176,  36768, 65072, 16848,
                5804,  60072, 23952, 14064, 28540, 45328, 38480, 48716, 15004,
                53916, 64084, 31752, 57548, 4204,  53132, 9848,  40968, 14496,
                33188, 53836,
            },
            {
                5132,  12836, 59124, 8100,  20744, 34512, 25476, 30916, 47616,
                60648, 34224, 3260,  18104, 56076, 25420, 1552,  38368, 17004,
                7168,  47552, 37444, 60452, 29452, 46284, 1820,  20700, 65148,
                23340, 13648, 56328, 29556, 45884, 7380,  58752, 3036,  46976,
                53788, 62868, 23712, 17148, 49968, 42400, 13896, 48092, 61136,
                32996, 56180, 12316, 1572,  51708, 14044, 44876, 31564, 10704,
                54364, 24648, 43752, 16460, 30948, 5384,  58952, 33008, 52060,
                6328,  59968, 20608, 34120, 30664, 45352, 38936, 25968, 36220,
                18324, 64968, 52948, 15952, 2676,  20136, 53808, 7132,  60496,
                3160,  28104, 39040, 32284, 59176, 15412, 28840, 50920, 59812,
                42820, 63376, 1692,  6484,  20256, 62408, 13104, 51372, 18280,
                5256,  60504, 52228, 33524, 62116, 49176, 22524, 11136, 44844,
                54700, 2932,  35636, 41636, 53160, 3664,  12212, 56736, 29208,
                19864, 40728, 11764, 49604, 15500, 35900, 19268, 64300, 3216,
                49468, 44956,
            },
            {
                63456, 23204, 50528, 45744, 53988, 5276,  40632, 1564,  16016,
                10984, 27552, 39456, 7072,  36508, 49072, 11228, 45632, 31380,
                27120, 55000, 10064, 4556,  56564, 25916, 14936, 37236, 49172,
                5080,  40020, 19464, 60404, 50244, 14356, 54668, 17648, 21296,
                41324, 36104, 112,   59504, 11188, 28324, 35488, 4652,  38708,
                22264, 65064, 41936, 34864, 55132, 8424,  60964, 19796, 41284,
                59436, 6944,  33852, 50012, 65256, 26236, 19436, 44452, 28740,
                9656,  46560, 3348,  13072, 61224, 51812, 57060, 4384,  49704,
                44288, 8208,  39676, 59820, 47404, 42448, 26080, 32656, 50464,
                13748, 56356, 7804,  22388, 52756, 10468, 37568, 32592, 3268,
                17036, 55976, 33484, 44884, 15812, 29552, 3728,  42464, 65220,
                28640, 38044, 12928, 7736,  15784, 31080, 52896, 39788, 28848,
                33244, 20840, 50408, 10040, 58144, 17876, 22964, 33784, 4952,
                50940, 24852, 716,   43328, 26948, 59976, 46276, 24760, 38260,
                28916, 18012,
            },
            {
                41372, 35292, 2704,  29156, 62016, 17964, 49168, 64892, 43500,
                22820, 50500, 62560, 53288, 21308, 29064, 63676, 57424, 3016,
                40948, 62396, 21260, 17936, 39780, 51620, 58808, 30312, 54380,
                33096, 47240, 3384,  24268, 10736, 43220, 64496, 33628, 28140,
                57844, 14508, 31628, 44196, 55092, 19524, 51420, 57840, 15396,
                7376,  18728, 30428, 46200, 16548, 22716, 37876, 50604, 4032,
                15416, 28492, 38752, 12108, 22500, 47208, 3692,  13808, 62416,
                24572, 35784, 55284, 26780, 18540, 11040, 21652, 33376, 15332,
                55124, 31284, 12860, 36572, 6040,  57316, 11792, 38816, 65464,
                19272, 36156, 46684, 62160, 40456, 5160,  63920, 20392, 46136,
                39652, 9104,  23800, 60588, 54284, 36520, 47900, 23492, 34316,
                2400,  46596, 19780, 44200, 58320, 5644,  19156, 2000,  61436,
                13080, 43632, 63148, 30988, 25756, 47508, 65500, 43048, 60440,
                9648,  37256, 61808, 21256, 33872, 2612,  55660, 6896,  12116,
                57124, 8564,
            },
            {
                26728, 55448, 16128, 10188, 32752, 24232, 12732, 36304, 55100,
                5700,  31580, 13768, 46352, 9072,  42248, 15644, 20084, 52536,
                13228, 34184, 50016, 44776, 12384, 33452, 3768,  10796, 18084,
                27332, 8728,  63192, 36120, 31672, 26256, 38440, 4624,  9700,
                49188, 5548,  25764, 37684, 7984,  2596,  24108, 30956, 44864,
                53920, 49564, 3368,  25012, 62848, 5564,  33304, 26392, 64404,
                45392, 52344, 57944, 816,   41112, 54676, 35420, 57200, 39804,
                49672, 15628, 64864, 43784, 49004, 6784,  41008, 63860, 27684,
                3360,  61448, 25028, 21128, 28416, 62308, 17672, 788,   44076,
                29824, 24744, 3644,  12436, 18172, 48660, 25348, 55228, 13724,
                27968, 52132, 35324, 12140, 26352, 7688,  58672, 10916, 16136,
                50304, 59652, 24120, 54532, 26892, 35016, 46144, 55716, 24564,
                48488, 6568,  16332, 800,   36100, 7828,  39744, 2768,  16436,
                46784, 30372, 56140, 7540,  51472, 13604, 42056, 32232, 50592,
                61364, 20540,
            },
            {
                31620, 48084, 60340, 42700, 38552, 57260, 8292,  28260, 19740,
                59900, 41488, 25960, 668,   33668, 61012, 4752,  35844, 44192,
                23948, 6076,  29964, 184,   63852, 22196, 41768, 62300, 44916,
                57172, 15212, 42304, 53424, 17144, 2064,  52416, 23052, 44576,
                62020, 19784, 53164, 64296, 46816, 58976, 40000, 62900, 10192,
                27316, 37068, 59216, 10932, 39864, 56368, 48516, 12296, 18412,
                35788, 9276,  20400, 31972, 61104, 8304,  17564, 30180, 1404,
                21436, 5220,  31728, 37940, 564,   29888, 59200, 46380, 9324,
                19096, 42728, 51676, 1852,  46088, 33704, 52512, 23068, 48456,
                9392,  59572, 54192, 33952, 30392, 43764, 1424,  36052, 7300,
                62824, 47640, 19064, 4452,  50608, 43604, 20752, 39624, 53936,
                31472, 9508,  36420, 428,   40828, 63908, 14332, 8524,  36468,
                57536, 27472, 38880, 59936, 55128, 19644, 51944, 32044, 27188,
                54184, 12832, 18312, 44744, 64720, 28296, 17336, 22492, 36012,
                44584, 1436,
            },
            {
                53044, 13008, 5980,  19164, 51188, 576,   45472, 52600, 33004,
                2792,  17000, 57220, 38276, 54596, 23064, 49636, 28196, 8400,
                61392, 55464, 38868, 26200, 53476, 47764, 7260,  25004, 38144,
                988,   49544, 22352, 58060, 7104,  62552, 48428, 13128, 56492,
                39952, 29924, 11420, 15928, 22012, 33384, 13084, 17920, 3828,
                42792, 21636, 14396, 52116, 28156, 44108, 1644,  31016, 53684,
                3324,  24388, 48268, 14616, 27340, 51740, 45780, 11132, 53060,
                58712, 42300, 9368,  23108, 56016, 14876, 35316, 23780, 38460,
                48632, 32952, 58200, 10688, 41412, 14656, 55832, 4860,  37228,
                16140, 50956, 41480, 21504, 57716, 14784, 51936, 60812, 22400,
                31524, 2432,  57384, 41084, 64476, 29932, 832,   61860, 27324,
                6356,  42280, 62628, 17736, 11404, 50112, 23092, 30212, 42356,
                18292, 11000, 51124, 22188, 45284, 10540, 14212, 58284, 21708,
                41396, 5728,  35372, 23864, 38748, 9876,  60784, 54196, 5608,
                15472, 40108,
            },
            {
                24440, 65168, 36848, 22380, 58728, 26416, 15524, 39872, 63608,
                47280, 10012, 50020, 18992, 30464, 11596, 57984, 40072, 18368,
                46488, 14808, 59076, 9396,  16952, 35776, 14112, 60240, 20552,
                34520, 29876, 11640, 39088, 45808, 20384, 29112, 35756, 16840,
                1580,  34240, 50924, 4812,  42688, 27620, 52868, 48980, 34916,
                56028, 64144, 296,   33728, 17832, 8668,  21352, 61824, 42008,
                58492, 39260, 63600, 43712, 5968,  37756, 24244, 63948, 34212,
                27792, 18108, 46936, 52684, 63268, 20096, 50532, 2800,  12548,
                54620, 6532,  16496, 30048, 64412, 7864,  39700, 26984, 31756,
                63028, 6488,  27640, 2528,  64568, 8420,  26656, 41272, 11128,
                44212, 16184, 37136, 23232, 13496, 33136, 17088, 45488, 57576,
                14464, 21604, 48848, 28512, 33072, 4208,  59032, 52184, 1368,
                64820, 32904, 41432, 4436,  30116, 26424, 63480, 38016, 1040,
                50040, 63004, 57688, 3892,  31456, 49292, 188,   46872, 29952,
                62976, 10624,
            },
            {
                45848, 28216, 3660,  47344, 30928, 61456, 6960,  21744, 12248,
                36796, 24428, 59400, 6408,  44376, 15220, 3876,  63380, 32572,
                2720,  36816, 20636, 42932, 32048, 56060, 28812, 50972, 5780,
                54240, 64724, 3836,  27028, 33884, 10108, 41524, 61148, 7656,
                26632, 59228, 45436, 37000, 56892, 852,   8744,  60636, 24628,
                6296,  31364, 40860, 46932, 60984, 57252, 38240, 25580, 15692,
                6380,  29520, 11436, 55020, 33280, 20028, 3820,  42940, 13924,
                6560,  61280, 35884, 11884, 28456, 7740,  43028, 57052, 26680,
                62572, 21788, 36956, 47368, 24576, 19224, 49600, 60824, 13216,
                20572, 45908, 11376, 35268, 47728, 38364, 18572, 33196, 50060,
                55624, 28964, 60260, 9168,  53492, 49508, 37692, 3780,  24684,
                34468, 52808, 5208,  45084, 55892, 39448, 16216, 44692, 21368,
                6316,  54204, 14500, 61468, 35212, 48932, 6724,  44448, 33584,
                9332,  25720, 47584, 15152, 53652, 19772, 41164, 25900, 57640,
                18720, 34432,
            },
            {
                8020,  56512, 41808, 13864, 9832,  35224, 43800, 56056, 30364,
                4388,  42208, 27904, 65132, 35400, 48212, 26580, 52740, 10420,
                24424, 51588, 48264, 4824,  65380, 2364,  39900, 10424, 46464,
                17028, 41288, 14376, 51636, 59840, 416,   54640, 23804, 43792,
                53536, 20692, 9416,  24384, 62432, 31276, 20128, 38444, 45680,
                11828, 19364, 50900, 23628, 4816,  35556, 13368, 49700, 45708,
                34596, 22136, 50324, 2568,  16516, 61888, 57916, 48188, 22048,
                39072, 49896, 24868, 1772,  40200, 32684, 61432, 17732, 30652,
                41076, 152,   52732, 59332, 4172,  35700, 57084, 3236,  43040,
                53360, 39020, 56644, 16544, 23600, 54464, 5852,  59636, 292,
                20936, 6780,  47052, 25860, 5212,  19896, 56608, 11428, 47432,
                64888, 38504, 12764, 60476, 20072, 9228,  26708, 32096, 60036,
                38072, 24452, 47068, 18664, 2832,  55404, 23592, 17056, 60508,
                20076, 29912, 39680, 11028, 59320, 34764, 7768,  13240, 37996,
                2484,  51136,
            },
            {
                21024, 16944, 33272, 54384, 50316, 18628, 2936,  48328, 62028,
                52340, 19968, 13712, 53620, 1756,  39192, 20476, 43012, 56256,
                29348, 62192, 12708, 27668, 23188, 44732, 19548, 58428, 25604,
                31644, 60900, 23376, 47564, 18756, 30984, 15364, 49116, 3888,
                32448, 64632, 13548, 49684, 17212, 41352, 47864, 14840, 28872,
                59364, 55252, 9140,  15356, 29680, 52940, 3044,  65156, 9748,
                56208, 18656, 60244, 26704, 46704, 40520, 28780, 9564,  31996,
                56112, 4112,  16508, 60060, 45588, 14412, 5088,  47732, 10052,
                34600, 14524, 43360, 28188, 11668, 45132, 22384, 10000, 34052,
                24656, 596,   29184, 61508, 43132, 30760, 12760, 46032, 37548,
                14288, 63996, 40308, 34592, 43872, 62936, 27524, 41292, 7888,
                18484, 30380, 1392,  24888, 36008, 64120, 47740, 3320,  11752,
                49956, 27924, 8784,  57892, 37536, 11168, 42068, 50808, 13220,
                56320, 45536, 1744,  23056, 43816, 27956, 62480, 55424, 23244,
                43392, 59412,
            },
            {
                39620, 62228, 892,   23504, 37844, 64436, 28852, 25272, 16104,
                34044, 45596, 9120,  31948, 23424, 60664, 7600,  34364, 16028,
                680,   40688, 35204, 8632,  59896, 49184, 13320, 36296, 1472,
                43700, 8504,  37388, 4844,  57288, 39596, 63652, 11700, 37240,
                18248, 40252, 29276, 2976,  35092, 7220,  54320, 65104, 1788,
                36564, 42988, 26332, 63276, 39556, 44280, 19700, 27764, 32148,
                1284,  42976, 38160, 7424,  35464, 13152, 224,   53628, 18740,
                65448, 44132, 29920, 55036, 50772, 22124, 37332, 53680, 24180,
                65172, 50292, 8188,  18340, 63328, 54248, 30444, 16896, 47540,
                64940, 18760, 49332, 9160,  4300,  50976, 63544, 22812, 27104,
                51776, 32188, 17744, 54500, 12700, 1524,  31120, 51888, 22720,
                58904, 43788, 54964, 49276, 6816,  42724, 14220, 56724, 34960,
                17020, 62504, 43352, 30896, 20816, 64180, 27392, 31744, 4472,
                36836, 53032, 18028, 65236, 50636, 2948,  16528, 46548, 32416,
                4640,  29300,
            },
            {
                11688, 48548, 27076, 58936, 6252,  42960, 11956, 39396, 8124,
                12,    58556, 40292, 56680, 16804, 51296, 12268, 64200, 46264,
                22376, 58100, 18548, 53204, 38252, 5360,  33240, 63428, 53036,
                21060, 56284, 12528, 34204, 25660, 7880,  45212, 27548, 58128,
                51672, 5316,  46288, 55788, 61072, 11448, 26552, 22512, 5492,
                33252, 17112, 49084, 6700,  32896, 12088, 55056, 22952, 59316,
                47964, 14184, 52528, 64448, 22880, 56816, 50932, 25140, 37220,
                7852,  12592, 34584, 19696, 8500,  26432, 58316, 1600,  44364,
                20148, 56476, 38704, 25100, 33404, 40708, 1856,  60116, 38076,
                7008,  31876, 58912, 14712, 40152, 19956, 34528, 3408,  41528,
                56556, 10324, 2600,  21960, 58436, 36932, 16164, 61252, 33796,
                3956,  15012, 27612, 32820, 17340, 23036, 52844, 29020, 41208,
                7044,  53716, 528,   13696, 46188, 52696, 2248,  40036, 62676,
                8344,  26312, 33992, 6420,  31024, 37620, 21584, 10692, 49864,
                63828, 15080,
            },
            {
                52848, 44244, 9320,  31588, 15584, 51628, 20652, 54884, 60184,
                49052, 22504, 28768, 5056,  47288, 26040, 37588, 4148,  30640,
                50356, 6028,  43740, 14444, 30136, 55852, 16756, 27264, 6652,
                50208, 28980, 64216, 42800, 53732, 20232, 3300,  34988, 22804,
                8680,  60164, 25740, 20980, 43312, 30680, 50084, 39828, 45228,
                52316, 21596, 57388, 2284,  60104, 47588, 7580,  40772, 36124,
                5052,  25260, 30152, 17800, 10036, 32352, 42704, 15468, 59792,
                47252, 27072, 41672, 3140,  38984, 63740, 33152, 13268, 28960,
                6704,  31568, 2684,  46980, 13628, 6340,  49840, 27192, 55844,
                12512, 42608, 36812, 25644, 55196, 28588, 58348, 8336,  16524,
                61108, 47932, 29532, 50640, 25360, 45912, 6436,  48300, 10920,
                38848, 51240, 8996,  57948, 62104, 37456, 1116,  61156, 19304,
                46640, 22624, 38952, 33808, 24940, 8048,  16428, 58684, 22280,
                47892, 15316, 57284, 42548, 12960, 60484, 54484, 41624, 6284,
                25652, 37064,
            },
            {
                2332,  34280, 21996, 46856, 55944, 1784,  35456, 30184, 43992,
                17512, 36948, 64080, 10816, 33716, 41932, 58324, 19544, 54740,
                10180, 33100, 25428, 61428, 1296,  41500, 23704, 47612, 11076,
                40056, 18352, 4076,  15788, 49264, 30156, 62288, 47440, 16844,
                42128, 31904, 14880, 38352, 1240,  18296, 58456, 14416, 60884,
                8536,  12564, 37356, 28720, 23824, 18264, 63700, 15936, 51312,
                11240, 61632, 41764, 5808,  58572, 46036, 3420,  29352, 6068,
                20756, 52996, 62404, 57000, 11544, 47064, 18024, 51964, 60628,
                40900, 16108, 61800, 52976, 58364, 21204, 43936, 15560, 23112,
                51020, 4884,  20776, 46520, 1220,  11748, 48620, 43976, 35920,
                23632, 5000,  39044, 65452, 8684,  40884, 20808, 55412, 26448,
                63932, 19296, 46300, 40656, 12236, 5956,  50192, 25112, 10384,
                31732, 65348, 5248,  56868, 60892, 49360, 35652, 44372, 29028,
                11444, 39460, 61784, 20056, 47512, 8788,  27396, 34900, 58620,
                18896, 56128,
            },
            {
                7496,  64936, 18032, 60016, 40328, 24536, 63732, 10556, 5688,
                26796, 13204, 50668, 20684, 53976, 1056,  14392, 44472, 27376,
                39092, 63128, 47408, 20324, 51784, 9480,  58032, 36748, 61924,
                31188, 46068, 24752, 60408, 10264, 40620, 13372, 52592, 684,
                55356, 11876, 50276, 63212, 53348, 9632,  35644, 3612,  24856,
                31264, 62704, 46512, 40528, 53780, 34948, 756,   30756, 45424,
                20316, 55864, 34160, 49524, 37516, 21088, 63720, 39604, 49284,
                32792, 920,   23104, 16368, 30520, 43076, 3944,  36032, 9880,
                49104, 22432, 34772, 10612, 37408, 29472, 62784, 7916,  35128,
                61276, 28792, 53560, 63876, 32760, 60616, 18436, 53116, 30880,
                12520, 45100, 19140, 35024, 14744, 60292, 29996, 16,    42540,
                35820, 23828, 2100,  30656, 21568, 45524, 33944, 39816, 59356,
                15108, 48808, 27280, 17692, 41388, 12288, 20336, 5752,  55504,
                51088, 344,   30096, 4048,  24652, 51400, 17468, 820,   45320,
                30676, 23640,
            },
            {
                38584, 42508, 12308, 3952,  28320, 14148, 45292, 50148, 33456,
                61964, 40844, 2536,  45404, 60932, 31312, 24204, 8000,  59648,
                16648, 2728,  12000, 35432, 28012, 45124, 4712,  21432, 14856,
                20,    54848, 37728, 32772, 2416,  57988, 24492, 38564, 28876,
                65360, 36252, 6336,  23896, 32616, 47464, 28148, 56200, 42604,
                18816, 54692, 4980,  14804, 9608,  43316, 25952, 58028, 38484,
                3804,  28304, 15144, 1140,  24616, 12828, 54404, 18256, 10844,
                61032, 44904, 37764, 51564, 5772,  24720, 55640, 20868, 27288,
                57540, 45776, 4740,  26304, 18652, 912,   54952, 39536, 10752,
                45856, 2804,  16968, 9376,  23388, 37816, 5972,  25540, 1680,
                62892, 54984, 27760, 57356, 3472,  33264, 52968, 17428, 13036,
                58172, 7440,  53208, 56504, 64504, 27820, 13948, 55512, 3992,
                42664, 36724, 9652,  54860, 30008, 1868,  64428, 32440, 25380,
                17788, 35320, 45748, 54804, 38288, 32648, 64576, 39540, 13364,
                61736, 50524,
            },
            {
                27300, 53280, 32212, 49200, 36888, 7836,  57996, 19108, 3440,
                23220, 56036, 29684, 9040,  17076, 36540, 65264, 52044, 34588,
                48916, 23128, 57160, 39732, 17296, 65016, 52792, 33064, 42896,
                58696, 8872,  51412, 21784, 43944, 18124, 33768, 4760,  20544,
                45156, 26664, 17848, 44232, 4296,  62036, 21180, 38500, 49044,
                24,    34188, 26980, 21016, 59448, 49372, 13144, 52552, 8936,
                23396, 63964, 53372, 43912, 60708, 27908, 7536,  34836, 56372,
                25896, 14448, 8200,  35104, 60028, 48368, 39600, 64396, 624,
                14216, 30556, 63400, 42564, 51456, 47448, 32144, 25144, 19720,
                57516, 41240, 34220, 44588, 51512, 14256, 42736, 56988, 40184,
                47592, 15656, 5800,  43192, 11624, 48572, 22744, 44428, 61616,
                31700, 48120, 15828, 35152, 3212,  17884, 51604, 8212,  29568,
                20112, 52008, 3120,  22228, 45928, 37088, 52424, 42480, 58248,
                9684,  63448, 21656, 7976,  15116, 56788, 5128,  22364, 48280,
                9532,  16064,
            },
            {
                58468, 5892,  19724, 62176, 54576, 22360, 30600, 37840, 53460,
                47196, 14764, 59376, 39132, 21964, 6024,  47000, 12460, 3916,
                30328, 42968, 7372,  50288, 2084,  30192, 11628, 25204, 49100,
                19776, 27840, 13284, 64716, 7164,  55068, 49856, 60852, 8460,
                57272, 48800, 10820, 59084, 41248, 12752, 16080, 7684,  63504,
                11780, 51156, 41520, 65388, 30324, 6364,  19552, 62328, 33544,
                48032, 40336, 18504, 10632, 32092, 47336, 51328, 42384, 1944,
                30260, 40592, 64880, 19008, 28428, 13524, 8812,  32444, 43472,
                53260, 7884,  38388, 16552, 60092, 12604, 5860,  65324, 48736,
                30580, 13332, 25912, 58500, 4272,  30084, 64920, 19912, 10496,
                32584, 21892, 36876, 52248, 25188, 63452, 38220, 9088,  4496,
                26960, 41244, 10800, 25288, 43588, 38492, 61048, 46536, 24184,
                57900, 63092, 33568, 13428, 59552, 8140,  23612, 14864, 4636,
                48552, 40700, 27336, 60012, 44444, 11112, 29388, 52712, 42744,
                3416,  35576,
            },
            {
                46204, 10908, 25668, 43732, 15892, 2860,  63168, 11312, 42200,
                7568,  35200, 25632, 51528, 43560, 55416, 27988, 41068, 20988,
                54412, 63712, 26812, 14292, 43836, 59112, 37436, 6136,  63256,
                2904,  35832, 40452, 30340, 46224, 26420, 12188, 41848, 15864,
                34688, 1560,  30636, 54704, 36992, 27096, 52328, 45064, 29948,
                23048, 16904, 56548, 1980,  38356, 35424, 44800, 2964,  29292,
                13772, 7012,  35448, 57344, 5464,  37896, 22976, 13388, 59048,
                20160, 53332, 46756, 4136,  57304, 22796, 51112, 17124, 36760,
                19792, 24956, 56096, 3292,  23428, 35560, 40988, 15856, 2048,
                54100, 7516,  63220, 39212, 21176, 48700, 35264, 7036,  27160,
                61236, 50068, 8896,  59520, 19508, 2180,  28560, 56748, 50420,
                20984, 36920, 59564, 52200, 6084,  21212, 11660, 31852, 356,
                16112, 44096, 26460, 39616, 48140, 18784, 61940, 28076, 37828,
                12440, 53980, 1716,  34104, 49300, 19596, 61080, 25836, 33416,
                55772, 20968,
            },
            {
                39344, 60492, 268,   34944, 47768, 40640, 50756, 17184, 27272,
                60832, 19844, 4488,  11672, 32396, 1408,  62484, 15044, 37328,
                9660,  18064, 33488, 60848, 23496, 19040, 54064, 41260, 16456,
                56032, 47604, 59244, 17032, 1272,  37216, 63060, 30996, 23796,
                52764, 40084, 19252, 22688, 3000,  60632, 33712, 5432,  57756,
                36656, 47056, 8580,  24336, 53152, 15996, 57244, 22192, 42020,
                59928, 50440, 26436, 21148, 65240, 2732,  17128, 62340, 45280,
                6348,  24368, 10340, 32260, 41592, 2428,  45188, 62304, 5180,
                59232, 49568, 32844, 10176, 45088, 28584, 58408, 51732, 22392,
                37640, 18672, 46720, 140,   11604, 55524, 16636, 45920, 53428,
                39268, 628,   29664, 41428, 34552, 16392, 45816, 32388, 15092,
                65128, 1536,  18388, 29720, 63072, 48936, 57248, 36556, 41028,
                49992, 11108, 6204,  55668, 31108, 956,   50380, 33316, 45380,
                19936, 30972, 16188, 23988, 6164,  36440, 40980, 1228,  17368,
                64192, 28636,
            },
            {
                49364, 14872, 56172, 29412, 9036,  24432, 32664, 55436, 792,
                45028, 30300, 64604, 48252, 57072, 19016, 25276, 50076, 58704,
                45732, 1088,  38464, 51968, 4372,  9352,  46900, 27364, 33864,
                22528, 11208, 4780,  24084, 53832, 9584,  20896, 47712, 5856,
                60168, 14164, 64416, 50516, 9296,  47720, 24448, 19756, 41020,
                3648,  61372, 32504, 43108, 11952, 27572, 47360, 9408,  54716,
                16764, 272,   45040, 52852, 29740, 41444, 55300, 26976, 31452,
                38784, 49904, 16224, 54744, 61116, 38004, 29860, 10760, 26824,
                46608, 13048, 40104, 64308, 54188, 20352, 7696,  33636, 44236,
                27848, 61044, 31684, 52752, 23860, 28412, 60336, 3168,  13840,
                24340, 17860, 64624, 13184, 47856, 54128, 6820,  39748, 11364,
                53368, 42420, 47096, 9248,  33652, 14812, 26608, 4856,  64944,
                19328, 34456, 60276, 24380, 16540, 43268, 10900, 56864, 2848,
                64736, 58560, 51008, 43144, 63248, 14076, 57496, 9260,  44952,
                12588, 5040,
            },
            {
                32040, 42284, 21812, 65384, 13224, 58948, 6216,  21032, 36576,
                13576, 52364, 37624, 15976, 40476, 9432,  35484, 6920,  30728,
                22896, 55820, 11524, 48024, 31220, 35760, 56924, 13700, 7392,
                44724, 31428, 51960, 61832, 42752, 32944, 58068, 2688,  38284,
                45032, 28896, 6972,  42916, 31720, 13816, 55600, 64012, 11380,
                28344, 14732, 20492, 48864, 59860, 4512,  63716, 39548, 25408,
                31468, 62748, 38872, 11256, 14988, 49312, 8880,  35792, 11916,
                3332,  63284, 34800, 26220, 12704, 19632, 52072, 56804, 35304,
                1748,  22076, 29468, 17276, 492,   47332, 61900, 11960, 3800,
                56784, 9996,  15256, 34696, 43400, 8288,  37352, 41568, 57836,
                33816, 45092, 55960, 4244,  26112, 58692, 21936, 62052, 25068,
                34692, 5116,  23360, 55764, 40280, 2776,  45408, 22704, 55004,
                8024,  28724, 52284, 4408,  64100, 35828, 53300, 22008, 39552,
                25556, 8956,  37032, 3724,  54888, 31516, 22140, 50536, 37716,
                54048, 24076,
            },
            {
                8364,  51280, 37324, 4764,  52108, 38980, 44496, 63592, 49328,
                57776, 8416,  22440, 2264,  29032, 60488, 53796, 42680, 3816,
                61520, 27244, 42000, 20228, 63904, 24912, 536,   62544, 49768,
                60208, 38996, 26660, 19000, 48972, 13584, 27980, 16708, 56000,
                11200, 24604, 35856, 57660, 17364, 39012, 1956,  35392, 44488,
                50308, 54276, 39784, 6676,  30168, 18116, 34156, 2312,  12488,
                36552, 19472, 4660,  24068, 33516, 60956, 19976, 46888, 53148,
                57572, 21684, 42824, 248,   48420, 6188,  23856, 15228, 40912,
                62956, 52524, 6312,  43112, 37052, 26324, 14912, 39280, 24376,
                49440, 42032, 5476,  58640, 50700, 62344, 18468, 30920, 5932,
                50988, 20924, 8352,  38024, 31492, 10460, 43236, 796,   48740,
                30252, 59132, 12660, 28044, 61244, 17524, 53052, 31028, 12152,
                47420, 37488, 44456, 21356, 40496, 7632,  27056, 15424, 6088,
                47628, 17872, 28884, 11612, 46848, 26584, 6708,  60400, 29504,
                18764, 62920,
            },
            {
                3164,  16980, 27064, 45804, 19488, 30940, 16120, 27960, 4212,
                24600, 33044, 43420, 62828, 46620, 20672, 12500, 47828, 17536,
                34420, 14104, 7456,  58484, 15664, 44144, 39856, 29492, 21036,
                3520,  15428, 8976,  35512, 6412,  39324, 64772, 51608, 34080,
                19760, 49504, 364,   62752, 26924, 46732, 52472, 18844, 8300,
                25616, 2584,  64668, 35992, 55484, 24020, 50712, 58420, 45792,
                53952, 48540, 59304, 42276, 56052, 6440,  960,   25224, 40144,
                18004, 7904,  29536, 59204, 36376, 63752, 43636, 32168, 8368,
                18920, 59988, 34060, 57736, 9596,  50980, 31704, 54820, 62896,
                17660, 36328, 20856, 26096, 2640,  12904, 22520, 47388, 63704,
                11412, 28472, 42048, 61028, 52192, 14680, 36644, 19036, 56968,
                15632, 45612, 20300, 37312, 50540, 8560,  34840, 42864, 61896,
                15900, 57972, 2036,  13796, 47920, 30296, 57648, 61624, 42320,
                32216, 55824, 62224, 40588, 19276, 35404, 15516, 41548, 2116,
                47220, 34652,
            },
            {
                59064, 55604, 10708, 61068, 1344,  56964, 9516,  35564, 12148,
                40768, 17264, 53276, 26684, 5876,  38660, 32508, 24884, 64840,
                50328, 39432, 53384, 28604, 4768,  51500, 9896,  17592, 36344,
                52936, 42428, 63500, 57232, 656,   21976, 43428, 25356, 4444,
                61324, 40856, 53664, 21816, 11644, 6104,  29752, 62364, 32372,
                57864, 16656, 22332, 10316, 44684, 13640, 41560, 20680, 6012,
                29100, 15780, 9668,  27744, 45364, 37252, 30396, 62584, 13944,
                32928, 51212, 46024, 14692, 9588,  27628, 53852, 2592,  50212,
                45552, 25352, 14424, 48596, 22668, 60384, 19176, 6716,  45384,
                996,   30480, 65012, 45852, 38988, 33504, 54432, 26816, 1064,
                36560, 55160, 16468, 2076,  22768, 64048, 28832, 51504, 4180,
                39144, 63136, 6748,  43672, 644,   25472, 58908, 19980, 5944,
                27428, 23484, 32452, 62204, 18580, 51436, 11844, 34636, 3492,
                50064, 13488, 23224, 652,   52556, 65096, 48560, 56828, 11024,
                26248, 43376,
            },
            {
                20472, 36316, 41160, 32860, 23212, 48744, 42740, 53880, 62252,
                47224, 58580, 3468,  11368, 56516, 15352, 59396, 192,   8900,
                21772, 45200, 2232,  37028, 32580, 23072, 46472, 59720, 55860,
                24528, 11676, 32540, 28360, 46444, 55080, 12328, 7936,  46128,
                29624, 9476,  15540, 33128, 44136, 56248, 23264, 14176, 42596,
                37736, 46940, 28668, 52024, 496,   60808, 31968, 8856,  65032,
                23460, 35112, 1640,  63216, 21320, 16204, 49616, 58280, 43532,
                4260,  64540, 24152, 55704, 40380, 17092, 58660, 21124, 36700,
                28864, 10488, 39436, 4744,  29928, 2204,  41824, 34760, 27080,
                13848, 51580, 55804, 10944, 16864, 49040, 7680,  43956, 15096,
                59512, 24676, 46484, 34728, 49252, 5420,  45060, 9132,  33780,
                26384, 10756, 53176, 31784, 64748, 14008, 48636, 39024, 2668,
                41136, 50776, 10516, 55212, 38740, 68,    24748, 45628, 20888,
                59460, 37904, 7752,  44828, 33236, 9696,  4596,  37440, 22968,
                61984, 13432,
            },
            {
                5404,  28804, 53588, 13992, 63880, 7124,  25920, 19888, 1792,
                29668, 21532, 36004, 49772, 30356, 44180, 51352, 36668, 54856,
                30112, 11272, 61992, 18632, 55320, 64684, 12900, 5920,  31244,
                1708,  44912, 19420, 51408, 15844, 61120, 31152, 36772, 18052,
                57204, 48392, 37976, 4200,  59456, 39752, 48652, 1052,  60584,
                5112,  12264, 33912, 62652, 39172, 25816, 17140, 37196, 43096,
                49800, 57644, 40344, 51760, 12104, 54436, 7280,  22736, 10128,
                28032, 37652, 1736,  20332, 33776, 4908,  47048, 12780, 65008,
                3924,  53292, 18268, 56612, 46428, 64644, 11708, 53392, 59004,
                8512,  40820, 22760, 5864,  28460, 56932, 62740, 20008, 40248,
                50056, 32152, 7196,  19592, 57672, 39504, 24780, 60272, 55016,
                21160, 48256, 17300, 35936, 22908, 54476, 9764,  29512, 56012,
                63728, 44920, 35492, 5072,  28688, 42920, 65496, 16720, 9920,
                26916, 30632, 54028, 60844, 25760, 21156, 30276, 16936, 54180,
                32492, 49736,
            },
            {
                44596, 24596, 2700,  46708, 17432, 38328, 34256, 60228, 15204,
                39384, 8256,  65136, 41760, 18944, 23040, 6876,  27276, 41176,
                16692, 59036, 25820, 48388, 8484,  41696, 35124, 26124, 40272,
                48888, 58024, 7128,  38316, 24088, 3456,  40732, 52840, 62732,
                1608,  23172, 27488, 65140, 19196, 8044,  35904, 27712, 21012,
                50072, 56772, 18544, 7664,  48624, 4912,  52784, 56324, 2748,
                11332, 31136, 18976, 4940,  32824, 26964, 39820, 34656, 47040,
                54732, 13052, 42364, 60756, 52400, 30068, 25036, 38916, 55548,
                31828, 42160, 61232, 26884, 33676, 16036, 38172, 23508, 48296,
                19604, 37180, 31840, 61408, 42496, 1592,  35620, 30268, 10232,
                3596,  65248, 13484, 42900, 10700, 30836, 17448, 13276, 41996,
                2560,  60992, 28240, 3624,  41868, 46120, 18272, 33680, 21852,
                7304,  16264, 25812, 20516, 58576, 8312,  36820, 56184, 51928,
                40816, 2328,  17996, 14364, 42876, 58600, 51104, 45224, 1420,
                39728, 8820,
            },
            {
                64912, 51592, 59776, 9868,  30208, 56420, 50788, 5896,  43900,
                52708, 26144, 13300, 33420, 4576,  60868, 10592, 63628, 47520,
                5588,  33624, 43520, 14480, 21528, 3748,  50720, 16176, 61704,
                21888, 14296, 65280, 34140, 10016, 49148, 20292, 26452, 33840,
                11064, 44668, 52244, 13132, 31260, 51440, 16572, 54980, 9884,
                39916, 24640, 42948, 30888, 15280, 59480, 28084, 21864, 46504,
                14576, 25684, 61124, 47376, 43264, 64776, 1304,  19316, 61796,
                16632, 31360, 49764, 6932,  11564, 62804, 43124, 1100,  15868,
                22984, 7140,  12336, 51216, 21100, 8032,  44772, 3424,  28736,
                63360, 2340,  46304, 52956, 18892, 12720, 24500, 51832, 58628,
                22488, 38400, 52720, 26336, 61920, 54296, 1132,  36368, 46312,
                31416, 56132, 39412, 12428, 58052, 6244,  61476, 37928, 49964,
                12004, 59608, 53020, 46540, 13360, 49048, 23076, 32600, 5500,
                47444, 63796, 35692, 49388, 5940,  38780, 12412, 62532, 27036,
                57824, 15640,
            },
            {
                37188, 18752, 33620, 22168, 40464, 1060,  12788, 23664, 32208,
                58880, 48772, 2212,  57360, 53804, 46332, 34908, 13784, 20176,
                56408, 1556,  52112, 38824, 31500, 60652, 28532, 54352, 8952,
                37400, 4400,  26940, 43104, 56104, 60084, 13940, 5104,  42644,
                16728, 58920, 35436, 2576,  41968, 24396, 62444, 44928, 30244,
                64344, 2144,  53256, 21500, 45940, 36360, 10196, 33212, 63648,
                38680, 54776, 8160,  36364, 23228, 14204, 56296, 50760, 4588,
                25888, 58892, 22344, 36160, 18564, 48480, 8696,  34540, 59848,
                49304, 45128, 36180, 64,    41056, 54796, 59464, 32156, 16660,
                56292, 10504, 14964, 26044, 59824, 38756, 47480, 5356,  42240,
                17968, 29268, 47980, 4668,  34076, 21628, 50828, 64384, 6432,
                24104, 15240, 8432,  50460, 30648, 26092, 14828, 1092,  43728,
                28424, 32892, 2004,  39304, 62864, 30172, 2660,  15000, 61544,
                20100, 11468, 23352, 55776, 29120, 2752,  19168, 34328, 6968,
                21480, 30864,
            },
            {
                11788, 4620,  42616, 49140, 61688, 27788, 45220, 64324, 10092,
                18360, 37532, 22628, 16420, 28380, 39284, 25048, 52656, 30500,
                37672, 23080, 62540, 7028,  57708, 44388, 340,   19324, 42544,
                30256, 52716, 47688, 17816, 1080,  29280, 45952, 64424, 54956,
                30752, 7084,  21028, 55988, 47780, 4916,  11812, 34344, 6556,
                14908, 37080, 58020, 11220, 62960, 1712,  41712, 17756, 5380,
                50260, 29124, 2028,  17064, 59108, 9852,  29376, 36780, 44480,
                8404,  40324, 2188,  45548, 28664, 57728, 22016, 52016, 28100,
                19348, 62424, 24704, 30892, 63560, 14096, 25640, 6148,  51036,
                42024, 35940, 49796, 4944,  33560, 8676,  64256, 15496, 34312,
                56600, 556,   60088, 16352, 45196, 7644,  40424, 27540, 19656,
                52488, 62128, 44408, 21968, 64108, 35328, 51984, 56272, 23956,
                64908, 18016, 42220, 9980,  34480, 18696, 54560, 43812, 38096,
                28552, 45368, 33192, 8556,  41872, 64460, 24716, 53192, 48496,
                41044, 55800,
            },
            {
                46556, 26772, 58744, 7928,  14896, 20412, 53864, 36620, 3704,
                28816, 46524, 63336, 42912, 8572,  880,   17748, 58360, 4120,
                42296, 12600, 28176, 17556, 10644, 24344, 36616, 47044, 63096,
                11320, 23552, 59444, 32364, 40044, 21760, 37372, 8796,  24520,
                49664, 39740, 62648, 15848, 26644, 38128, 57332, 19884, 52128,
                46876, 27184, 4500,  33588, 25172, 51424, 57008, 45104, 24220,
                60600, 20748, 43844, 52056, 32236, 48316, 41024, 21340, 12816,
                53456, 32924, 65316, 56044, 14708, 38468, 2852,  13252, 40096,
                9812,  4216,  56364, 17320, 10296, 48532, 34628, 39340, 60924,
                20036, 23972, 30304, 44600, 55492, 21120, 27432, 53744, 44696,
                11760, 25236, 37404, 10060, 29968, 57852, 14144, 47380, 10992,
                32644, 36728, 60,    17216, 46968, 3936,  20116, 40468, 8700,
                48060, 4364,  57624, 23476, 51200, 6140,  59948, 25076, 7992,
                58344, 500,   52324, 60064, 15980, 46868, 36676, 10236, 14664,
                61348, 56,
            },
            {
                52584, 16964, 36036, 55112, 3200,  33020, 48360, 16208, 42132,
                55564, 5520,  11508, 50928, 31652, 61880, 44528, 48544, 9580,
                65456, 51032, 46828, 34804, 54932, 49628, 13764, 5684,  55652,
                34264, 2164,  15564, 8148,  53728, 12276, 50768, 34248, 19136,
                168,   27884, 45484, 9064,  32692, 60880, 808,   43088, 23520,
                60140, 17360, 48576, 41116, 14252, 20020, 8076,  31084, 13328,
                34548, 9576,  39644, 63484, 4000,  25460, 6292,  57932, 62380,
                27452, 17960, 10356, 24472, 4948,  31892, 63580, 44260, 58504,
                33172, 47372, 37956, 43408, 52780, 22596, 4480,  46016, 12392,
                7832,  65460, 412,   58800, 17208, 40776, 1988,  31440, 7844,
                63176, 20644, 54564, 42076, 63708, 23332, 34788, 54960, 4560,
                58644, 42436, 25672, 56616, 38948, 11084, 31920, 59996, 13824,
                36412, 29980, 54040, 15156, 27240, 45820, 36140, 12240, 50748,
                17072, 40300, 26400, 12756, 21780, 31708, 56876, 3928,  44256,
                28988, 22996,
            },
            {
                8884,  62144, 29620, 24060, 39236, 62776, 8860,  24984, 31324,
                60888, 20240, 35240, 24548, 55152, 13160, 36248, 22024, 33520,
                15992, 25752, 5812,  41004, 20676, 64264, 32424, 26556, 17284,
                40548, 51268, 45176, 64140, 25592, 58000, 4648,  61932, 14888,
                59688, 35928, 12480, 50368, 22236, 52864, 29296, 13356, 39320,
                31688, 9268,  54304, 61708, 29404, 39160, 64804, 53632, 3584,
                47548, 58256, 26988, 12380, 19484, 54340, 34348, 16300, 172,
                46216, 51368, 35216, 42064, 47260, 54328, 26052, 18076, 6052,
                53436, 14904, 26956, 2120,  60444, 29264, 18552, 57480, 31268,
                26720, 53872, 13936, 37892, 9808,  50948, 61820, 23596, 48856,
                39692, 51180, 32764, 3272,  18768, 49660, 1636,  38524, 17948,
                29256, 49424, 13172, 6928,  28268, 53784, 48852, 5272,  26516,
                45068, 19868, 61648, 38380, 936,   64388, 20956, 42116, 31432,
                55928, 5120,  63432, 37604, 1896,  51324, 27580, 59276, 19932,
                33756, 38648,
            },
            {
                13236, 43696, 5328,  51184, 12100, 43388, 57868, 696,   52096,
                13908, 39532, 58412, 2944,  40792, 19460, 6672,  28940, 59368,
                1224,  53820, 31036, 60348, 1916,  8704,  39376, 57800, 22128,
                60256, 27932, 35960, 20012, 2716,  44060, 28432, 41064, 47816,
                53504, 3960,  42372, 64176, 5748,  17504, 46532, 7788,  65468,
                2260,  35812, 22288, 6016,  45604, 160,   26160, 36108, 41980,
                16084, 55224, 924,   37476, 46360, 61176, 42528, 49908, 30644,
                38048, 5836,  21556, 61336, 16488, 8116,  50492, 36292, 22892,
                30124, 65120, 20860, 7548,  35816, 40852, 63976, 2992,  51716,
                36288, 47744, 42868, 22320, 28280, 46220, 12892, 36756, 4152,
                14560, 28064, 6232,  46564, 12272, 26840, 60704, 43776, 64792,
                9152,  21336, 59876, 33940, 62760, 23132, 42784, 16736, 63228,
                52092, 11292, 7228,  32408, 49464, 10248, 29088, 3576,  47128,
                22920, 34436, 19112, 49156, 42524, 7856,  17380, 40332, 6504,
                64664, 49212,
            },
            {
                18832, 57476, 31960, 21140, 47104, 17324, 28132, 37156, 22108,
                49384, 7820,  27436, 15772, 45360, 64004, 49872, 56148, 39964,
                44188, 19084, 37224, 14504, 45244, 29564, 52440, 3572,  43928,
                9804,  4932,  13592, 49548, 38692, 32724, 17192, 22676, 10072,
                31304, 19836, 25044, 55644, 34776, 40824, 58464, 25848, 49028,
                55556, 44304, 12844, 58720, 17956, 48836, 11100, 59808, 19384,
                29800, 23320, 50444, 31820, 14972, 22632, 7824,  11080, 24252,
                64064, 13804, 55576, 1492,  29992, 40564, 11968, 60212, 752,
                42120, 49324, 12224, 57216, 50736, 15284, 9672,  44276, 21688,
                16752, 5580,  32804, 59864, 2680,  34124, 57888, 19452, 43568,
                59780, 17720, 62312, 36480, 56952, 31272, 6804,  14956, 24436,
                53124, 3880,  40372, 45412, 15176, 1400,  57312, 35180, 30628,
                2824,  40052, 22396, 43084, 56844, 17820, 52672, 62024, 14788,
                59224, 10576, 44972, 29316, 55380, 61528, 11144, 46156, 24228,
                54772, 2024,
            },
            {
                25772, 40612, 63856, 1488,  34792, 54368, 6668,  65304, 10980,
                43148, 62232, 33984, 53972, 30552, 10272, 25504, 4528,  12256,
                23888, 61584, 10364, 55532, 22988, 48860, 16812, 12540, 33640,
                47516, 61876, 30872, 56916, 9196,  62952, 54708, 5696,  57508,
                45168, 38628, 14304, 29976, 1588,  10784, 21192, 37160, 15308,
                19368, 30472, 26904, 52252, 37820, 33344, 22860, 51096, 5340,
                62196, 8168,  43260, 65180, 4704,  28612, 59300, 40132, 18600,
                52952, 43720, 27008, 48220, 57924, 33924, 20344, 46212, 56144,
                9096,  38712, 32528, 44752, 23572, 27812, 33720, 55544, 38788,
                61720, 11236, 54612, 18824, 63644, 7096,  52608, 26056, 30808,
                54920, 8868,  23060, 52788, 43064, 20884, 51532, 33352, 46676,
                36380, 30492, 18332, 51648, 9688,  38012, 20276, 8084,  24352,
                55992, 48336, 59732, 14264, 25628, 34856, 40648, 8236,  38608,
                27004, 53040, 6468,  14280, 23412, 32268, 35852, 52232, 15056,
                31316, 37200,
            },
            {
                45616, 52904, 15368, 9452,  59372, 25484, 41312, 32704, 46604,
                17972, 3996,  23196, 48408, 104,   59676, 38344, 17652, 52648,
                35000, 49496, 4044,  42196, 27656, 58816, 36192, 63640, 25716,
                54116, 21496, 41756, 16596, 23892, 46152, 13176, 37128, 26132,
                62568, 7892,  59208, 48232, 43476, 61320, 32544, 53304, 6416,
                62880, 9908,  41652, 4312,  63396, 8392,  56776, 14300, 45204,
                38104, 33772, 11464, 18284, 55920, 35368, 51168, 45756, 3148,
                32576, 8216,  36500, 10740, 23832, 3264,  64380, 27740, 15444,
                24808, 62172, 17424, 5632,  59932, 516,   48376, 13608, 26024,
                1732,  29288, 49492, 24412, 41812, 15696, 39108, 10812, 2968,
                45684, 33532, 40236, 300,   16000, 9572,  38972, 57712, 11904,
                888,   55700, 61384, 25544, 32496, 50052, 65340, 46692, 12848,
                36856, 17220, 28440, 6520,  45464, 2672,  58064, 20304, 32672,
                1544,  65200, 36400, 57596, 2348,  20348, 63496, 27220, 3464,
                60828, 10652,
            },
            {
                6620,  28316, 22840, 38908, 49060, 12484, 19820, 2316,  59980,
                29896, 56024, 37544, 13000, 20448, 42756, 33180, 46688, 63332,
                28484, 7956,  32312, 64808, 19812, 2644,  7484,  41032, 18940,
                436,   37712, 7160,  52280, 1872,  29076, 50944, 3276,  33480,
                17768, 52432, 3656,  23272, 16464, 50728, 27948, 3208,  39584,
                57552, 34192, 47496, 24392, 16100, 31052, 40348, 27512, 1844,
                53284, 24820, 58900, 47140, 39104, 2032,  14060, 25908, 60796,
                56204, 20804, 63036, 39928, 17616, 52392, 43968, 5460,  34852,
                54120, 3380,  29460, 52408, 37292, 19948, 62904, 7960,  41192,
                57752, 44820, 37144, 8844,  31584, 47416, 56700, 22580, 61516,
                50412, 12792, 29072, 65312, 25656, 60304, 4872,  27928, 63052,
                22868, 43160, 6496,  12544, 58336, 3476,  27844, 40784, 53968,
                60688, 280,   33972, 64612, 54136, 11372, 24052, 47724, 51508,
                43556, 17628, 24948, 41492, 47796, 39276, 5388,  49812, 44068,
                21612, 56688,
            },
            {
                47656, 35632, 61404, 4832,  44608, 29320, 56652, 52268, 14932,
                24804, 40400, 6392,  64480, 51900, 27152, 7156,  14228, 2276,
                21472, 45052, 15452, 38556, 12048, 46764, 31556, 56768, 50860,
                14588, 58848, 27320, 65372, 34940, 42768, 58548, 20812, 48648,
                40568, 11316, 27360, 35376, 64564, 7432,  13140, 45772, 23564,
                50456, 896,   20732, 59888, 55136, 2996,  43860, 64268, 21360,
                48492, 16220, 5616,  27224, 20852, 53596, 31476, 7264,  37644,
                15964, 28448, 4684,  49920, 59752, 14092, 30532, 38184, 49076,
                19580, 41300, 46636, 11512, 56696, 42904, 30660, 51568, 22544,
                18132, 64328, 14340, 3312,  52152, 27856, 6100,  35140, 42184,
                20048, 58268, 6404,  47940, 35332, 45300, 49976, 19236, 41224,
                15800, 48288, 28996, 39168, 21840, 44164, 15548, 5308,  31520,
                9924,  44380, 20784, 49760, 30764, 37484, 62796, 15556, 5444,
                30412, 11988, 50200, 60532, 10144, 16780, 55024, 13148, 40972,
                33932, 17632,
            },
            {
                2772,  12912, 50708, 32056, 16832, 63568, 37972, 5148,  35028,
                49996, 9936,  45972, 17008, 35548, 56472, 22888, 60296, 53616,
                40212, 57764, 50320, 25564, 60608, 52828, 23940, 10208, 28880,
                44424, 33104, 46984, 12036, 18408, 39308, 8984,  15048, 64032,
                30360, 44048, 57044, 38432, 19872, 54536, 41720, 60640, 17540,
                29340, 13932, 44228, 36800, 11388, 49552, 18060, 35148, 9768,
                57296, 31200, 41308, 62744, 8852,  42952, 64260, 22308, 48124,
                41956, 12416, 34308, 45016, 25692, 7524,  55936, 10172, 62944,
                13024, 60264, 22960, 8244,  26484, 16268, 3840,  46412, 35656,
                5200,  32640, 55848, 21824, 59500, 18336, 64708, 13876, 1308,
                26792, 37360, 17316, 55856, 21464, 13660, 30976, 2456,  54108,
                9204,  34796, 63896, 17688, 52900, 33684, 61216, 23556, 18952,
                51848, 26508, 41048, 13312, 4788,  18900, 42512, 27372, 59728,
                54604, 21980, 34648, 7224,  28608, 33216, 25372, 59012, 8376,
                29960, 65144,
            },
            {
                42572, 58308, 20488, 54608, 332,   24004, 9264,  43796, 21536,
                62436, 28160, 58204, 31320, 3280,  9236,  49096, 30712, 36276,
                11308, 18416, 548,   34472, 6036,  42808, 16288, 36860, 62660,
                4292,  9080,  21924, 56004, 5140,  61604, 24332, 54940, 1484,
                22696, 5868,  14440, 47008, 444,   31252, 25344, 35008, 10336,
                53956, 61872, 32336, 5016,  26260, 53448, 29612, 6808,  60692,
                13032, 37040, 1108,  50156, 34396, 12012, 17580, 56936, 708,
                51808, 61256, 54780, 2508,  18984, 33056, 42152, 21796, 26912,
                1216,  32384, 36472, 49412, 64980, 34100, 12884, 60352, 53624,
                10472, 26396, 40356, 46272, 11808, 37860, 43528, 49620, 31868,
                62764, 8672,  52644, 2008,  42588, 10552, 57212, 37148, 60856,
                26216, 51272, 4792,  45544, 1076,  10240, 56796, 48172, 37780,
                63044, 7364,  57732, 23716, 61196, 52360, 1212,  33296, 8840,
                39580, 45556, 144,   56316, 64160, 46244, 19808, 36764, 856,
                53540, 23788,
            },
            {
                37868, 26952, 7400,  41544, 34676, 59620, 27716, 48680, 13056,
                54720, 1144,  19292, 44152, 61268, 39152, 15808, 43004, 4564,
                27308, 62636, 55104, 29456, 21792, 57600, 1800,  49916, 20368,
                40224, 60160, 25944, 48876, 31512, 44264, 27724, 34128, 45992,
                60388, 52908, 32768, 62924, 9048,  49208, 58396, 3920,  38876,
                6764,  48484, 19028, 41404, 63056, 21880, 39420, 47028, 25220,
                42600, 55392, 19088, 23760, 60460, 29708, 44796, 35888, 25332,
                9984,  30568, 22692, 38812, 63340, 48528, 58732, 5100,  51948,
                44272, 58096, 18404, 2556,  40552, 55308, 24260, 39300, 28892,
                16732, 62112, 50124, 600,   33748, 24584, 4524,  55192, 16092,
                46884, 40012, 24664, 29820, 33732, 64072, 23752, 40340, 6776,
                21020, 31764, 12096, 55408, 36460, 25216, 41852, 13272, 2864,
                34352, 16260, 47600, 38820, 9568,  35452, 46624, 56836, 13532,
                63392, 26252, 19228, 14472, 42380, 3688,  51264, 61572, 15684,
                48968, 10100,
            },
            {
                62876, 31196, 15420, 46836, 12008, 51724, 18480, 39756, 7248,
                32596, 41896, 23836, 11528, 50552, 26108, 20196, 64876, 47736,
                23192, 8388,  44176, 13832, 40524, 64336, 32564, 45864, 12776,
                54580, 34492, 15660, 38112, 744,   12968, 51728, 7356,  37740,
                10712, 18396, 25832, 41400, 22252, 12624, 18840, 43620, 65060,
                22516, 27200, 56088, 9092,  13684, 58208, 460,   15852, 52188,
                3972,  28624, 46104, 15036, 52632, 2920,  58688, 6228,  40536,
                19520, 46508, 7292,  14548, 28236, 10924, 35668, 16476, 39064,
                29744, 14352, 53396, 6996,  28204, 19692, 9284,  1348,  48272,
                42764, 20668, 7744,  29760, 60672, 53100, 9964,  28860, 22772,
                57500, 11992, 44756, 60024, 49648, 4376,  17724, 52332, 14116,
                47092, 41656, 59432, 19124, 28600, 64520, 6988,  30196, 54408,
                44988, 27996, 4040,  55116, 29688, 15208, 25152, 21288, 49452,
                4308,  35840, 52404, 38244, 30848, 23324, 11552, 40288, 27544,
                45000, 18708,
            },
            {
                5708,  50264, 57080, 22536, 64756, 4348,  30768, 57228, 63292,
                16004, 36968, 53216, 6120,  34032, 55568, 976,   12744, 57952,
                32392, 52068, 37208, 48792, 10376, 18820, 7056,  27972, 24276,
                2780,  51192, 7808,  63724, 53516, 19904, 58944, 16628, 42028,
                29448, 49756, 2372,  59696, 37592, 55768, 29864, 52412, 15128,
                46476, 37448, 2152,  51172, 34388, 45296, 31100, 65352, 20380,
                33148, 62132, 10544, 39760, 7620,  48440, 27484, 15876, 54160,
                32660, 65444, 57492, 43656, 51016, 228,   24420, 46048, 64592,
                9200,  23816, 46460, 61504, 42492, 50916, 63532, 36228, 58724,
                4440,  54648, 37392, 15736, 44852, 19500, 41052, 62148, 6696,
                34176, 3076,  20372, 15460, 8252,  27476, 45144, 35272, 29560,
                62604, 1628,  23976, 48752, 39708, 15260, 50884, 19672, 59704,
                22556, 11336, 64856, 19944, 43744, 58956, 6160,  40940, 29344,
                17188, 60936, 10352, 47504, 59292, 7448,  55932, 32360, 4468,
                34724, 54444,
            },
            {
                24864, 1416,  40004, 9448,  36916, 44816, 24740, 2564,  45796,
                26364, 60284, 48384, 14452, 63408, 30000, 45608, 35808, 5544,
                41188, 15940, 2924,  61440, 25988, 53056, 35944, 56264, 62992,
                42708, 18500, 29464, 41276, 23328, 35284, 46480, 2880,  65028,
                21696, 54672, 35504, 16076, 44712, 7024,  26244, 1200,  33920,
                11036, 30960, 60780, 17240, 23656, 38228, 7728,  11632, 49340,
                36664, 5648,  57400, 25892, 32276, 20404, 38148, 62452, 49580,
                12932, 1860,  25612, 17520, 31336, 62096, 54488, 20060, 3632,
                56568, 33576, 37264, 11736, 31348, 15600, 45568, 22116, 14128,
                33028, 25440, 65320, 12324, 57144, 2140,  35500, 47660, 17892,
                39140, 53772, 63624, 36488, 41368, 54992, 58984, 2956,  10028,
                56260, 16316, 34024, 8828,  3908,  58380, 33440, 42892, 544,
                39200, 52304, 32160, 36892, 1936,  50612, 63820, 11232, 54092,
                44548, 32244, 22104, 2856,  28060, 17904, 43616, 62524, 21592,
                58764, 13208,
            },
            {
                43284, 60172, 33220, 28272, 17344, 55572, 13756, 34884, 19952,
                10256, 4880,  28928, 21764, 41072, 8608,  24048, 50968, 19188,
                28464, 59756, 21272, 31020, 5204,  46908, 39768, 15016, 9548,
                31924, 58108, 47532, 3608,  60324, 9840,  31092, 26172, 48400,
                13492, 8348,  62496, 4628,  31748, 50388, 59080, 40136, 62596,
                20612, 57164, 42660, 5596,  47896, 27836, 55636, 41200, 59984,
                22884, 43348, 17980, 53764, 64536, 42840, 10140, 23408, 5236,
                35096, 41740, 52736, 37012, 6180,  40316, 11976, 29096, 42580,
                49680, 5996,  21408, 57668, 3048,  25784, 5716,  56304, 29220,
                9948,  50796, 46904, 21808, 31844, 26620, 51896, 13504, 59068,
                30400, 26036, 50172, 5292,  22956, 11816, 19900, 48196, 24700,
                43448, 37992, 53656, 63244, 44172, 23004, 11012, 26456, 62580,
                17200, 8224,  48676, 12952, 26300, 18000, 33744, 37760, 2300,
                57524, 7692,  40668, 65344, 53508, 49844, 14544, 26020, 9108,
                47608, 37464,
            },
            {
                15804, 19964, 48096, 53380, 5948,  40624, 61200, 47868, 52464,
                39084, 56360, 43768, 2344,  59312, 16928, 38496, 61312, 55148,
                10848, 44984, 53748, 35396, 17204, 58612, 22616, 220,   49632,
                37004, 21988, 11680, 27128, 14920, 55720, 40600, 5412,  56668,
                33228, 39136, 27720, 47168, 11692, 23652, 17636, 9348,  48432,
                4020,  25080, 13196, 53004, 63736, 19860, 3336,  16836, 26820,
                2136,  13500, 47460, 35388, 304,   14108, 59560, 44932, 29944,
                56480, 20288, 10280, 60720, 22356, 47820, 58932, 34888, 14796,
                26668, 63388, 17060, 47672, 39812, 52596, 33868, 62576, 43536,
                18732, 40256, 2952,  6900,  38884, 62696, 8800,  24144, 43228,
                288,   10156, 46404, 31856, 61928, 29144, 39016, 65100, 32472,
                19004, 7020,  27424, 13644, 31040, 49936, 56416, 5264,  35144,
                45576, 24496, 57992, 42164, 61532, 7848,  46744, 22428, 27520,
                48880, 15732, 25432, 12740, 33400, 36936, 1452,  39688, 51988,
                3452,  29380,
            },
            {
                64316, 6872,  11204, 62352, 21832, 1876,  31532, 8172,  23252,
                63992, 32140, 18208, 36028, 53676, 47536, 4352,  13996, 33660,
                7360,  25264, 1500,  65440, 42568, 12164, 29588, 61612, 44924,
                17132, 5884,  64828, 44292, 49988, 36628, 18096, 62108, 23440,
                43708, 17392, 52476, 20792, 56072, 64456, 37020, 43216, 28288,
                54624, 35728, 39720, 29816, 10056, 33976, 59020, 45956, 32012,
                51284, 62620, 29080, 7068,  24296, 51804, 36744, 3616,  16740,
                61716, 27708, 45900, 3392,  33176, 16356, 7944,  52080, 2404,
                37944, 53756, 10308, 27872, 59616, 19396, 8332,  37916, 120,
                53612, 61284, 34732, 55912, 44028, 17196, 49224, 4892,  64544,
                21680, 56404, 14012, 17460, 43544, 1960,  53244, 14740, 5572,
                50596, 61420, 46552, 21092, 1324,  17508, 37520, 47636, 14700,
                54264, 29652, 3284,  21460, 53188, 31164, 56216, 13264, 60376,
                19616, 36260, 42872, 58008, 6264,  20932, 45036, 63616, 18428,
                32840, 54836,
            },
            {
                23884, 44544, 35824, 26276, 49960, 42964, 57980, 28728, 16312,
                388,   12632, 50136, 25132, 10716, 31332, 26972, 43136, 63152,
                48416, 40116, 14632, 49732, 8316,  33336, 51780, 4828,  26048,
                56812, 52880, 39288, 32932, 7968,  21008, 1156,  51432, 12216,
                6636,  61368, 116,   41700, 34004, 2868,  13988, 52136, 19060,
                7624,  62000, 16480, 692,   50252, 43512, 13836, 53932, 8724,
                38436, 56176, 11068, 40920, 58260, 31124, 19312, 55452, 49132,
                7596,  37960, 14380, 50152, 56908, 26064, 44868, 19456, 60220,
                23140, 44336, 32784, 1480,  42292, 13460, 24100, 49392, 11984,
                27084, 15932, 23260, 29516, 10972, 58524, 28036, 37092, 32196,
                52376, 39992, 34408, 58296, 50904, 9328,  35732, 26180, 56708,
                40596, 11776, 36184, 57832, 52820, 40420, 65436, 28420, 9744,
                60360, 6472,  40176, 35984, 16024, 840,   44020, 39968, 5004,
                51348, 63144, 476,   47112, 30072, 55780, 10152, 27500, 59760,
                12300, 40384,
            },
            {
                204,   56872, 31816, 14676, 37864, 18876, 10780, 51488, 36444,
                41688, 60152, 45924, 7052,  65044, 56676, 19728, 672,   21920,
                29480, 57692, 37576, 23120, 27408, 55468, 18916, 40736, 34556,
                12920, 29164, 2356,  23784, 60792, 54332, 28152, 45460, 37856,
                30696, 57320, 10628, 25864, 45932, 30052, 6144,  60656, 32484,
                47296, 22648, 45008, 59536, 25792, 21188, 39976, 4276,  24644,
                18908, 34332, 21692, 45332, 16116, 47824, 9020,  26128, 33372,
                42104, 23916, 63488, 29812, 39840, 1036,  64712, 31156, 40812,
                13004, 4960,  61968, 50836, 35772, 65036, 55388, 46040, 31896,
                59452, 7708,  47424, 52256, 3488,  20660, 41436, 14384, 47156,
                18704, 5952,  27664, 3772,  24444, 60576, 45892, 17900, 424,
                30032, 23256, 4644,  33140, 9212,  25416, 3388,  21560, 32716,
                42748, 18812, 63460, 50868, 10160, 28436, 64952, 24844, 9528,
                32936, 28592, 11180, 23512, 61608, 15896, 48976, 35568, 22232,
                5416,  49676,
            },
            {
                16800, 52036, 8664,  59516, 4964,  65432, 25488, 45236, 6304,
                55144, 27616, 21452, 34064, 15112, 40460, 50564, 36840, 53568,
                11920, 17764, 4240,  59872, 44760, 2256,  63588, 48168, 8060,
                20456, 62388, 43204, 16624, 46992, 13564, 33948, 64532, 25104,
                48904, 19532, 35976, 60100, 15956, 49832, 57720, 24880, 41616,
                3496,  12144, 55692, 31668, 6004,  36224, 64688, 29496, 60624,
                49852, 1568,  63924, 27440, 3372,  60300, 39736, 64872, 1604,
                12252, 54748, 4252,  18624, 11584, 53264, 36584, 9512,  49296,
                56288, 29764, 18320, 22296, 6864,  30404, 16532, 3680,  21172,
                35924, 41792, 64616, 13296, 33284, 61576, 53776, 1652,  57696,
                11500, 61204, 42560, 37508, 20524, 13180, 32960, 41904, 63836,
                47472, 59284, 15276, 43080, 61128, 45520, 13116, 51764, 57472,
                49056, 23844, 12656, 32076, 58592, 47356, 18212, 35780, 54948,
                15216, 45184, 53740, 18912, 39208, 3400,  41908, 7332,  57632,
                45948, 28808,
            },
            {
                60800, 20452, 40756, 29224, 46880, 54052, 13888, 31980, 62808,
                19244, 4092,  38940, 58676, 3032,  44404, 9188,  59272, 6128,
                45704, 32180, 50660, 20144, 35716, 11276, 15568, 24016, 58852,
                37428, 50168, 10444, 31236, 57804, 3980,  42080, 9336,  2196,
                14928, 40892, 52064, 7588,  22160, 39880, 10368, 36692, 15232,
                65216, 27692, 38852, 9636,  51876, 17016, 56080, 11600, 46952,
                15448, 41856, 6288,  51516, 32980, 13016, 22540, 44248, 52308,
                20740, 47004, 59584, 34428, 43296, 21372, 24660, 15712, 6536,
                26232, 46116, 58300, 38728, 11088, 44088, 26204, 39712, 57020,
                50140, 1908,  19212, 38616, 45672, 23532, 8248,  36044, 30164,
                25212, 45248, 8656,  64796, 48644, 55276, 6768,  52564, 10588,
                20268, 37936, 50820, 28716, 18608, 55260, 35540, 5900,  16280,
                37852, 1776,  44888, 26164, 38544, 6000,  22248, 49344, 4024,
                59852, 40752, 7592,  35084, 51148, 31480, 24980, 53068, 14028,
                33612, 38456,
            },
            {
                24836, 11772, 43988, 3132,  22788, 34564, 1164,  40276, 23820,
                48104, 10432, 52428, 30772, 23728, 18464, 28820, 34372, 25480,
                64516, 40808, 9024,  62048, 29772, 39476, 53328, 31616, 43832,
                844,   55792, 25524, 5448,  36436, 52000, 17992, 58664, 22284,
                55516, 32200, 3288,  63952, 28904, 54440, 1728,  20576, 48696,
                52688, 34744, 19800, 62684, 47716, 2472,  23432, 44224, 7756,
                35688, 30508, 56808, 18524, 38280, 54372, 5452,  29580, 15836,
                36648, 26464, 8908,  51048, 5468,  61360, 46824, 54876, 62820,
                34976, 420,   14516, 54072, 47908, 59088, 5992,  62692, 10204,
                14772, 25120, 30932, 5084,  50508, 27112, 15972, 64096, 49532,
                21576, 53272, 16888, 31292, 2820,  29160, 22852, 37044, 27312,
                3512,  54568, 7760,  24124, 2320,  31168, 63516, 41440, 26740,
                8928,  62056, 53340, 4168,  55344, 13788, 41716, 61816, 30692,
                20340, 25516, 57300, 2072,  63476, 11728, 59992, 44000, 19300,
                64812, 2524,
            },
            {
                48588, 35352, 63492, 52744, 17976, 57128, 9400,  59192, 16472,
                35076, 61644, 13704, 46592, 55040, 63436, 12420, 48036, 16020,
                3536,  54712, 14088, 24200, 56760, 6956,  46456, 4128,  27444,
                13904, 18188, 40432, 64148, 21468, 26836, 38716, 30016, 62456,
                43540, 26520, 12496, 44764, 18044, 34228, 61696, 43876, 30516,
                7716,  1084,  42944, 13632, 27144, 41640, 33032, 19648, 53024,
                25948, 59856, 10840, 48056, 25168, 62284, 35256, 10456, 57068,
                62600, 40368, 30800, 15132, 57468, 28028, 2156,  31484, 39216,
                20712, 43016, 8640,  33556, 23840, 18636, 51668, 34572, 28652,
                53984, 43724, 61052, 57328, 10524, 55184, 44120, 39576, 6400,
                2464,  35020, 56884, 11872, 39096, 59576, 14820, 62064, 44892,
                31824, 12992, 62988, 48332, 39632, 10380, 22444, 47276, 59184,
                29972, 34660, 21244, 17168, 63968, 33408, 9292,  52368, 1464,
                37772, 12636, 47772, 16852, 28384, 21280, 37628, 3884,  27880,
                9992,  54728,
            },
            {
                5888,  15348, 27328, 7520,  38272, 49016, 29956, 43808, 51248,
                7232,  28356, 42848, 780,   36604, 6868,  51416, 60548, 22172,
                37980, 28052, 44024, 392,   48844, 16932, 21836, 65088, 35168,
                60516, 47400, 32556, 54172, 12228, 49516, 7148,  46712, 11008,
                5784,  50588, 37724, 58776, 48016, 5012,  26788, 11800, 57040,
                17116, 54788, 24164, 58488, 6552,  37564, 56992, 62492, 524,
                38688, 22152, 6736,  43256, 15020, 1188,  45516, 50684, 18448,
                6360,  96,    45712, 23672, 38000, 12524, 41952, 17912, 10684,
                52320, 60560, 28444, 64372, 3176,  31056, 41748, 968,   20596,
                47184, 7336,  37348, 17440, 34756, 728,   19856, 32052, 58072,
                14460, 41304, 26348, 47036, 19640, 43020, 50008, 5348,  17452,
                56928, 41456, 33988, 16824, 60544, 53696, 14040, 100,   19220,
                51276, 11860, 42884, 48524, 29356, 23920, 46248, 16308, 27256,
                44532, 64348, 32744, 41408, 49724, 8396,  55596, 16048, 47256,
                41220, 30780,
            },
            {
                20940, 58092, 46516, 32296, 12320, 60976, 20900, 26088, 2960,
                38392, 56856, 21268, 17080, 26380, 39960, 31596, 1780,  42392,
                56904, 18584, 51712, 33084, 37016, 59740, 41156, 10120, 55028,
                6060,  23348, 8604,  44736, 3144,  61828, 15072, 33248, 53352,
                20504, 34972, 15680, 23736, 9484,  41668, 51120, 22468, 63684,
                39520, 46176, 32552, 50236, 61400, 15384, 10396, 28800, 50644,
                13692, 54512, 65332, 32420, 20460, 58012, 26784, 41420, 23028,
                33908, 49396, 60908, 19688, 53008, 33120, 65504, 49748, 5624,
                24056, 47320, 15960, 37668, 49816, 12840, 61300, 16060, 39120,
                64124, 12576, 22828, 29592, 42388, 63076, 11908, 46652, 28332,
                61376, 50344, 7188,  62932, 648,   34236, 9304,  25312, 52120,
                22052, 1252,  46180, 25824, 5536,  36624, 43664, 33368, 56528,
                4508,  39988, 60968, 8064,  1016,  57104, 39452, 59408, 6800,
                55812, 9860,  22992, 4532,  59148, 44872, 32068, 63016, 23284,
                51492, 61288,
            },
            {
                1672,  39648, 54220, 23368, 42192, 5704,  14852, 53468, 64732,
                11720, 32348, 49596, 62844, 9932,  58288, 19880, 45416, 8780,
                34928, 11452, 5048,  63204, 25600, 12784, 28760, 49640, 19848,
                37968, 51872, 29736, 16716, 35460, 42332, 57408, 24532, 40660,
                3792,  65296, 56484, 532,   31232, 59844, 14636, 37344, 5768,
                28128, 9440,  21064, 2796,  30240, 19012, 47968, 4324,  41140,
                17812, 44804, 27252, 36476, 9628,  52992, 5136,  30672, 64600,
                13792, 55836, 10252, 43444, 3732,  7972,  58648, 26576, 36448,
                55176, 3976,  40800, 20224, 7564,  54504, 45304, 25788, 56624,
                32688, 3232,  52920, 58448, 5400,  25588, 51452, 22588, 8588,
                37480, 18688, 32800, 23516, 54852, 28252, 58784, 40164, 35664,
                65228, 28920, 11156, 58188, 20768, 51892, 27608, 64744, 24460,
                46368, 15340, 26468, 36680, 51652, 11608, 34844, 20620, 31228,
                48664, 18200, 35768, 52048, 13452, 25596, 540,   34992, 6524,
                12408, 36696,
            },
            {
                43756, 9436,  17052, 3720,  62736, 36548, 45828, 33672, 18476,
                41508, 24480, 5908,  53672, 34700, 47244, 14456, 25192, 64024,
                29904, 58960, 47484, 20364, 8260,  44968, 2244,  32700, 15740,
                43052, 1460,  63680, 59152, 20724, 28116, 876,   18712, 59748,
                29616, 45160, 26212, 39336, 53096, 20104, 33688, 2504,  47840,
                60464, 35444, 12872, 40696, 45216, 54324, 25124, 34832, 63512,
                31180, 5844,  2604,  49520, 39912, 61564, 16744, 48132, 38100,
                7408,  29036, 25496, 36016, 17012, 46680, 21984, 14196, 44560,
                30548, 11996, 56976, 26724, 59904, 35060, 21908, 4356,  9372,
                50548, 27776, 45012, 14716, 48788, 38428, 59908, 16144, 55376,
                3204,  44512, 11300, 51684, 16380, 45324, 12448, 3396,  48148,
                14628, 8072,  38416, 50080, 31048, 3040,  9460,  17180, 38340,
                7244,  31328, 58124, 22452, 18424, 44416, 4264,  62928, 14160,
                42040, 2896,  61172, 29212, 39068, 19708, 53600, 40560, 57028,
                18440, 26404,
            },
            {
                31772, 65052, 51228, 25852, 55916, 28696, 2056,  57960, 8440,
                48020, 61112, 15244, 43648, 3328,  29004, 55180, 4316,  52288,
                15708, 23548, 38664, 54272, 30868, 58404, 52212, 62756, 24132,
                56580, 13120, 25840, 39100, 10568, 55256, 48176, 36908, 13012,
                8508,  17456, 49416, 6692,  12084, 63068, 44944, 54000, 24988,
                16192, 55996, 51816, 64976, 7780,  36776, 59100, 21944, 8496,
                46928, 55364, 60696, 13288, 22792, 34072, 2068,  11576, 59336,
                21228, 41840, 53848, 63688, 30904, 56376, 40364, 724,   60344,
                18948, 63864, 32968, 1288,  43344, 10600, 30336, 48396, 36516,
                17796, 60320, 40244, 21068, 7912,  32300, 2104,  41864, 34300,
                64768, 25780, 59280, 4328,  37932, 63980, 21292, 30796, 18852,
                60716, 54680, 44004, 15552, 60080, 41596, 56064, 48756, 12160,
                62428, 54592, 2396,  48044, 65164, 28040, 53940, 24468, 50424,
                37140, 26376, 54764, 5960,  65404, 10348, 48236, 14892, 29136,
                45832, 59508,
            },
            {
                22224, 14000, 35120, 47384, 19372, 10988, 50404, 22080, 37384,
                27864, 1624,  31008, 39036, 23280, 11928, 60132, 40724, 36172,
                7040,  43612, 1260,  13656, 41860, 18020, 5788,  39272, 9244,
                34684, 46956, 50428, 4864,  45372, 31536, 6796,  51072, 43164,
                62084, 55316, 34508, 22568, 42360, 27556, 18156, 8028,  30620,
                41732, 36,    23364, 17480, 27688, 3444,  12068, 50296, 15124,
                37656, 24608, 18456, 42224, 28480, 55872, 44044, 25680, 52276,
                46344, 5504,  14756, 2492,  48760, 10960, 28556, 52380, 34496,
                8912,  46860, 23248, 49108, 52568, 15612, 65224, 58160, 42852,
                24236, 452,   11116, 35288, 64220, 18180, 53996, 28976, 13188,
                47452, 20828, 30060, 42460, 7996,  33616, 57224, 52972, 42100,
                5164,  32728, 24848, 19652, 7004,  35260, 23096, 29332, 44184,
                20704, 34244, 41800, 8776,  13520, 32304, 40928, 10148, 252,
                60128, 20912, 46316, 16124, 31624, 42632, 22456, 62136, 8204,
                3600,  49696,
            },
            {
                38312, 772,   29584, 8136,  38960, 59744, 31796, 43552, 13448,
                55244, 58856, 20664, 51260, 65400, 18216, 49164, 31792, 20568,
                61620, 50520, 27700, 64244, 35544, 22532, 47884, 27232, 59472,
                3736,  30144, 18168, 57532, 23012, 14736, 64620, 26008, 21752,
                31988, 2812,  14592, 60876, 48216, 3652,  58196, 38052, 63956,
                10876, 49160, 33076, 38560, 57456, 44160, 32124, 61884, 41784,
                29260, 58680, 10880, 51844, 7348,  63384, 19676, 36340, 31444,
                17916, 61936, 33460, 23624, 38604, 19940, 62272, 4868,  25128,
                42408, 16768, 38180, 6124,  29056, 19504, 38736, 6604,  13800,
                53404, 62276, 30432, 56240, 25932, 45868, 10216, 23540, 50128,
                6940,  39076, 54416, 14024, 48592, 24000, 1360,  10564, 27204,
                46800, 36904, 584,   64280, 47488, 13352, 61380, 1720,  50696,
                5364,  27704, 16640, 52676, 38304, 5740,  56716, 17772, 29548,
                44064, 12476, 8408,  34712, 50992, 1992,  58700, 37276, 33260,
                55072, 11492,
            },
            {
                61212, 53252, 57748, 44788, 4484,  15432, 25232, 64284, 6792,
                17680, 45692, 10832, 35752, 7660,  44640, 260,   26640, 9356,
                46804, 17376, 33340, 57032, 3812,  10448, 55288, 15032, 44268,
                21208, 37320, 61792, 8792,  33860, 40868, 53320, 1904,  11224,
                39608, 52500, 28336, 37084, 10224, 31768, 51060, 14084, 21648,
                57056, 44688, 4700,  62376, 13968, 20992, 53388, 1764,  6300,
                20464, 34168, 284,   45840, 30820, 14916, 3900,  58244, 9552,
                688,   39852, 50688, 59700, 7212,  43924, 15292, 36912, 49788,
                58552, 2696,  53876, 13392, 61768, 41728, 2460,  26568, 32984,
                19908, 47556, 5532,  42252, 51660, 4672,  36592, 62768, 56468,
                80,    17488, 34848, 62348, 26756, 18176, 50256, 38856, 63200,
                22156, 12516, 56704, 52156, 26776, 38672, 32316, 18460, 53964,
                36872, 58844, 23764, 45676, 60912, 20388, 47328, 62316, 33604,
                52164, 57772, 40024, 24272, 56152, 27824, 5628,  16908, 24516,
                42096, 19668,
            },
            {
                6156,  41364, 16872, 23160, 62516, 54104, 40352, 488,   52224,
                33820, 40680, 3968,  54152, 24256, 33052, 56536, 14072, 39496,
                58496, 4612,  11852, 24904, 45760, 37632, 51004, 31872, 65020,
                11936, 51940, 216,   42836, 27808, 59580, 19392, 36080, 47160,
                57896, 18008, 46064, 5764,  55880, 24764, 40896, 2088,  35160,
                26900, 18804, 29900, 9640,  48464, 25736, 39224, 17288, 56800,
                47944, 53076, 64788, 38176, 22064, 50300, 41120, 47888, 55064,
                28544, 44644, 12464, 26752, 57116, 31488, 54004, 22204, 11820,
                30452, 63156, 26240, 33632, 55632, 22784, 50716, 56720, 45136,
                8540,  36088, 16704, 22300, 12880, 59796, 40172, 19380, 31812,
                42844, 58384, 10636, 3528,  45448, 60180, 35280, 15100, 7172,
                58876, 17576, 30748, 40720, 10476, 5076,  57452, 42376, 9300,
                14580, 64152, 11164, 30288, 1336,  26004, 36752, 14640, 5124,
                21900, 3224,  63792, 18588, 47192, 13108, 44768, 52664, 64040,
                47524, 28244,
            },
            {
                35584, 25448, 32204, 49952, 12572, 34296, 19628, 48556, 29232,
                22728, 62860, 26500, 59388, 15668, 41684, 62220, 52176, 22372,
                29644, 42804, 51688, 20236, 62548, 28868, 1412,  18776, 7344,
                40388, 26284, 54696, 16576, 48988, 12536, 4380,  30220, 63172,
                7468,  23456, 43000, 64488, 20424, 16244, 61588, 46408, 53564,
                7184,  60864, 51300, 42268, 35044, 6664,  63760, 45004, 27648,
                9776,  14308, 25244, 4716,  33320, 59624, 11476, 24072, 15884,
                65040, 21476, 36072, 18568, 10032, 1520,  64548, 46500, 6628,
                39592, 20052, 9904,  44476, 36320, 7724,  12032, 29484, 63080,
                39908, 58656, 50280, 28192, 33432, 1468,  15620, 26228, 7772,
                46528, 27992, 22856, 51884, 31296, 5988,  41512, 54316, 28376,
                43176, 48476, 3004,  23000, 45668, 16196, 63188, 21608, 25800,
                47668, 39656, 4160,  50032, 34824, 54216, 8520,  41468, 49152,
                27388, 37460, 30876, 6912,  60368, 36076, 20800, 30720, 9728,
                3180,  14596,
            },
            {
                45112, 64640, 8832,  2872,  43300, 27012, 10080, 60960, 38088,
                8008,  12852, 49728, 30376, 47312, 5936,  11348, 36792, 3104,
                64704, 35232, 55868, 6908,  14648, 41552, 59716, 24212, 56392,
                47760, 32856, 22564, 63844, 6880,  38684, 56424, 50632, 15768,
                27268, 35176, 1236,  12508, 50132, 33312, 5280,  28828, 12072,
                39832, 15168, 24240, 1152,  56620, 31356, 11536, 23356, 36020,
                59252, 40264, 44056, 17420, 54256, 8120,  27736, 38912, 34252,
                6600,  53236, 3432,  47776, 51824, 40584, 34200, 17552, 27660,
                57196, 51640, 47664, 4540,  18364, 60120, 48244, 16364, 1316,
                24632, 11496, 3640,  64988, 54832, 43624, 48308, 53296, 60948,
                14348, 37396, 65484, 39980, 13100, 56648, 21492, 1932,  32220,
                51748, 8960,  36280, 60748, 55444, 28784, 48776, 35132, 308,
                31464, 56252, 19852, 42760, 16996, 58036, 23208, 64364, 11016,
                55332, 43600, 15344, 50468, 10548, 41232, 408,   56756, 39128,
                50800, 58624,
            },
            {
                11568, 52124, 20200, 56348, 37296, 59028, 5408,  45844, 16304,
                57016, 43396, 19072, 2052,  38596, 21200, 28844, 17612, 48796,
                8544,  15776, 26852, 38640, 49432, 33208, 4852,  43780, 36432,
                15456, 4132,  10656, 35672, 46376, 20296, 24624, 43608, 9928,
                41036, 59340, 53228, 29672, 37580, 59060, 42788, 22584, 65472,
                47708, 36388, 54528, 20016, 60124, 16156, 52260, 49380, 2184,
                32292, 7312,  29384, 61996, 46192, 20212, 63196, 1948,  46740,
                58832, 31692, 41676, 61168, 28388, 23740, 13416, 60432, 43496,
                136,   31936, 15268, 58080, 25296, 31552, 38124, 43232, 21116,
                54164, 32088, 45904, 37664, 20004, 9380,  29840, 35468, 21916,
                2876,  49824, 19160, 8412,  44368, 25804, 16760, 64204, 11912,
                24668, 62384, 20216, 13984, 32884, 4248,  8464,  52396, 59924,
                45344, 6964,  27888, 62680, 12748, 6320,  44656, 31240, 17640,
                59188, 1204,  25280, 54060, 33388, 23576, 61596, 26480, 16452,
                32816, 22436,
            },
            {
                1428,  29868, 40028, 47292, 17108, 30456, 53072, 24092, 32940,
                3080,  53640, 35880, 9968,  64044, 55628, 43888, 61008, 23980,
                32060, 46468, 1648,  61728, 9752,  22276, 53336, 12644, 27568,
                62268, 58136, 44384, 28788, 60436, 31880, 1068,  61952, 33396,
                21144, 13852, 47960, 6396,  25156, 9312,  55488, 616,   18644,
                31436, 3620,  10484, 45512, 28164, 37572, 5392,  41448, 18828,
                61100, 21620, 50812, 1208,  13568, 35312, 41964, 51364, 18072,
                12716, 25664, 8492,  15336, 38324, 55728, 3912,  50452, 9232,
                36608, 22700, 64896, 41988, 2236,  52240, 63780, 6376,  35164,
                61340, 15224, 26768, 6992,  57236, 24176, 63556, 4904,  11732,
                57808, 32868, 55336, 1072,  29216, 60920, 49124, 38156, 46060,
                34516, 5472,  54200, 39240, 44352, 24568, 19232, 38864, 11696,
                15948, 23420, 53028, 33224, 39196, 51692, 26936, 3752,  34500,
                39468, 20716, 46432, 62468, 4336,  14100, 49540, 43840, 7816,
                62728, 37116,
            },
            {
                57420, 15288, 6924,  25872, 63528, 1576,  14324, 41104, 65112,
                28364, 22072, 60392, 32084, 50196, 25956, 872,   53512, 40220,
                57404, 19584, 42328, 54664, 30476, 17924, 64452, 48404, 8360,
                20708, 51076, 1808,  17476, 51956, 13400, 54232, 6044,  46776,
                55708, 3604,  62964, 39956, 19056, 45584, 14760, 34704, 51428,
                57412, 25624, 62700, 40160, 8156,  21952, 64868, 55840, 14644,
                46400, 54480, 11268, 37504, 24356, 55896, 5300,  30232, 22576,
                56636, 43456, 63668, 20948, 5484,  31184, 45988, 19476, 26028,
                55120, 12696, 38632, 28404, 10404, 20248, 13880, 27748, 51252,
                9028,  40472, 49232, 18232, 41928, 13828, 50752, 38720, 45492,
                41340, 25228, 15872, 36716, 52508, 34016, 9708,  3672,  57812,
                18260, 41644, 27248, 1028,  49784, 63596, 56164, 30004, 43384,
                65368, 36832, 9536,  59528, 1760,  21104, 46820, 60604, 52640,
                13040, 7560,  29008, 36408, 42316, 19928, 30280, 54928, 4736,
                18980, 48684,
            },
            {
                23880, 53268, 61240, 34780, 21848, 50512, 36600, 10676, 48784,
                7108,  46268, 13232, 42036, 16484, 8296,  33936, 14260, 5440,
                10824, 60216, 25184, 13536, 36708, 45724, 572,   34536, 39628,
                30668, 41464, 24892, 37700, 8092,  42016, 26896, 16924, 38168,
                28912, 24148, 16284, 31848, 60996, 49692, 27784, 39416, 7860,
                43628, 16604, 49932, 33472, 13744, 47860, 30560, 25508, 4256,
                34504, 27024, 42556, 58132, 31760, 48616, 15464, 60520, 9888,
                37788, 2836,  48996, 34668, 58568, 10724, 64228, 40836, 33552,
                61452, 5020,  46812, 53432, 34164, 44860, 56684, 3220,  46372,
                22844, 55456, 312,   34464, 60232, 2708,  31884, 17708, 62080,
                28756, 9504,  47632, 58668, 6184,  19876, 42244, 23748, 29776,
                13340, 47848, 59416, 10928, 17396, 35800, 13464, 3712,  26440,
                49624, 2816,  18520, 48088, 30524, 15184, 37048, 9240,  24524,
                48964, 65188, 56120, 16972, 8964,  58168, 38064, 12124, 46388,
                28368, 41452,
            },
            {
                32548, 4304,  42972, 12772, 8444,  44660, 55784, 20520, 58868,
                17464, 39048, 24968, 4800,  56960, 20264, 38136, 45652, 29712,
                49356, 34276, 3540,  50876, 6740,  58872, 25924, 56500, 16600,
                5556,  55352, 11520, 65272, 56912, 22028, 35356, 49876, 64240,
                9192,  44036, 35908, 54492, 11504, 2620,  58392, 23468, 63604,
                11972, 29608, 4988,  59144, 53500, 2808,  43864, 11924, 39380,
                63160, 8732,  17348, 3088,  65308, 8232,  26572, 44612, 52700,
                33276, 28136, 54308, 17244, 44520, 23124, 52172, 14944, 2468,
                48724, 21376, 17224, 7940,  62528, 23792, 39356, 17696, 59800,
                30860, 12888, 63208, 28496, 52604, 46852, 26016, 7480,  53840,
                20592, 4332,  63984, 22932, 45120, 14564, 63112, 56108, 50432,
                7584,  65092, 31740, 23292, 53308, 7236,  41572, 61036, 20436,
                33824, 57544, 40320, 25336, 43156, 55952, 63416, 18872, 41168,
                31540, 3252,  43884, 25976, 52020, 2440,  63868, 24992, 35252,
                59212, 9056,
            },
        },
        {
            {
                26876, 33328, 50060, 54336, 36344, 64896, 47828, 52448, 26964,
                21984, 50540, 12948, 62648, 25724, 52972, 2552,  56936, 20664,
                13036, 42372, 9960,  46488, 61496, 8184,  22300, 45120, 16320,
                57512, 21164, 43960, 58848, 30856, 22872, 42668, 14024, 28972,
                20500, 48360, 30720, 64412, 53008, 2144,  49576, 4864,  27360,
                8868,  23736, 62112, 11248, 4292,  42444, 58024, 46196, 23940,
                34168, 48104, 2200,  37268, 56620, 33332, 12328, 46064, 42228,
                62556, 2148,  51760, 64216, 24244, 36036, 54848, 14300, 19520,
                35256, 41004, 62212, 4104,  51460, 7444,  63084, 15916, 45640,
                31268, 54824, 38204, 23312, 5020,  16716, 61356, 19808, 8632,
                65316, 52532, 43884, 18208, 36956, 7336,  12428, 16144, 35464,
                30092, 92,    20544, 32172, 64520, 6460,  22216, 15732, 5204,
                45320, 64324, 14692, 53068, 21396, 60640, 35612, 26300, 1240,
                15520, 47776, 61232, 18136, 42824, 64400, 48428, 57076, 43576,
                1288,  58400,
            },
            {
                53232, 13628, 4220,  44660, 8680,  19332, 32076, 4356,  15328,
                63652, 30120, 40196, 19712, 57848, 43116, 14736, 34396, 5056,
                51632, 55652, 23436, 32736, 36728, 53312, 25944, 60156, 11380,
                50948, 39196, 1116,  7208,  35596, 51220, 3140,  62220, 52820,
                5344,  39940, 24492, 7584,  44656, 33040, 56120, 16140, 43128,
                35340, 972,   57232, 38748, 50688, 14264, 64912, 6524,  55416,
                19976, 8800,  29464, 14808, 25356, 5200,  21036, 54552, 35232,
                57276, 22352, 11836, 44932, 18228, 48380, 27916, 63500, 3280,
                50088, 10012, 55152, 13344, 38288, 26580, 41424, 53976, 2284,
                21060, 5812,  47856, 12712, 58596, 41960, 27692, 1476,  47492,
                16152, 35792, 10836, 22716, 61756, 41032, 45336, 64612, 50728,
                56260, 43432, 15296, 53628, 12156, 48112, 59124, 30232, 41096,
                23936, 3740,  26992, 42468, 1876,  39636, 54012, 57608, 30968,
                21128, 4840,  32580, 13192, 7108,  27096, 21496, 9204,  50736,
                23668, 36268,
            },
            {
                46760, 61580, 22564, 58740, 26040, 41496, 56940, 38048, 43944,
                9516,  48052, 4012,  35972, 7216,  28512, 61872, 45864, 25236,
                39084, 63700, 700,   16388, 40804, 13996, 3112,  41932, 29604,
                8540,  24760, 53744, 14956, 61380, 38876, 17520, 26152, 34328,
                45776, 60524, 16616, 57700, 12904, 22104, 9808,  30168, 65320,
                13748, 41148, 28816, 16892, 34704, 24984, 28532, 112,   39008,
                13404, 61064, 50628, 45020, 62968, 39320, 49832, 26728, 14428,
                6704,  29536, 38952, 33540, 3736,  60312, 11328, 38448, 46388,
                24516, 17656, 29396, 44240, 57080, 19160, 11024, 49176, 34960,
                39892, 61156, 27136, 33512, 45088, 9860,  37528, 55028, 24120,
                31664, 60012, 4964,  49244, 54220, 32772, 1756,  21924, 5604,
                11116, 23920, 59504, 38484, 26716, 44196, 18840, 1028,  61732,
                50804, 55360, 18404, 62640, 32452, 23532, 10644, 6868,  43984,
                62836, 55828, 39820, 46060, 57676, 1960,  52604, 38916, 29540,
                5568,  16912,
            },
            {
                7532,  30128, 39264, 204,   14704, 29224, 6264,  61300, 17972,
                34140, 58504, 23220, 45316, 16920, 51084, 21824, 9176,  48476,
                18488, 30548, 7360,  50208, 27996, 63340, 19716, 49696, 35764,
                65496, 46172, 32812, 20316, 28156, 45304, 9288,  55176, 21816,
                12356, 2460,  36568, 27848, 41504, 51212, 37004, 59428, 45792,
                20980, 53928, 47528, 7304,  60892, 9740,  53088, 48868, 31704,
                57376, 26576, 40720, 3504,  18644, 10052, 60020, 488,   41280,
                19552, 52936, 59264, 26044, 56264, 7796,  21592, 53012, 32224,
                59204, 4776,  34280, 22772, 948,   60208, 31992, 24180, 64080,
                9644,  17136, 51028, 368,   56068, 21268, 63896, 49624, 12208,
                43208, 20216, 40000, 26608, 14320, 29104, 17600, 58880, 37576,
                27772, 34180, 48676, 8200,  3632,  36096, 57252, 33024, 13160,
                35484, 8756,  29680, 48292, 5744,  46184, 16624, 50276, 34888,
                11936, 27764, 8428,  23324, 36336, 16556, 61744, 33724, 12940,
                65144, 41860,
            },
            {
                49640, 10692, 34628, 63388, 51480, 48156, 12144, 24772, 53260,
                912,   27564, 54324, 10612, 65492, 38632, 32760, 3188,  60412,
                12116, 36176, 54020, 45396, 57940, 5124,  32188, 55860, 2036,
                17616, 58612, 4064,  49208, 11244, 57372, 316,   41356, 65052,
                29984, 53916, 49524, 19652, 63244, 512,   24892, 17868, 7044,
                3676,  25472, 32152, 58700, 19724, 45036, 36000, 15988, 43160,
                22016, 6932,  34520, 58632, 23564, 52196, 36308, 30240, 64592,
                48752, 10460, 5060,  16620, 40724, 30068, 44124, 364,   15852,
                42456, 62792, 52280, 47268, 14984, 36588, 6604,  43836, 14036,
                29612, 57868, 41060, 25128, 14900, 30492, 4052,  34028, 6608,
                57936, 53136, 2048,  64100, 6448,  56484, 47440, 41720, 52700,
                61516, 2292,  17132, 63436, 22848, 51240, 9664,  25212, 46928,
                21040, 39964, 58140, 11600, 37224, 59444, 28836, 65476, 24828,
                19036, 53000, 48976, 59668, 30472, 42332, 4940,  46432, 25056,
                55548, 20572,
            },
            {
                27464, 57764, 16344, 43056, 20864, 55376, 35984, 46048, 21388,
                41372, 14360, 60852, 31512, 2316,  13312, 54768, 41452, 57640,
                27204, 43296, 20968, 10076, 23980, 38232, 12276, 44640, 23004,
                27428, 13520, 42472, 36628, 62724, 23604, 33056, 48480, 15272,
                6836,  43656, 9680,  32684, 5208,  46660, 54696, 39192, 33952,
                61672, 51640, 12736, 1552,  40424, 22812, 62340, 3292,  10452,
                63840, 17288, 54648, 12376, 32044, 46892, 15608, 7872,  44404,
                24904, 37288, 62048, 45928, 51004, 13140, 65264, 36184, 27004,
                8436,  12240, 38892, 6412,  64620, 28372, 50608, 56496, 1800,
                46612, 20304, 6940,  62960, 36800, 53328, 18616, 40480, 25864,
                15300, 29764, 37996, 45800, 34112, 11376, 24400, 8364,  19948,
                14172, 44628, 39972, 29080, 55920, 41812, 16036, 65072, 53568,
                2024,  44168, 16180, 25684, 54508, 20540, 160,   39392, 42940,
                4364,  37664, 2372,  14088, 20336, 54748, 10736, 18892, 59752,
                3768,  37056,
            },
            {
                1644,  53332, 24588, 4760,  32296, 9456,  3016,  64616, 30628,
                8588,  48536, 37352, 43256, 26092, 49732, 18260, 29480, 5492,
                15440, 64736, 1812,  34896, 60140, 16788, 48260, 62776, 9744,
                40412, 55148, 7616,  30480, 52464, 5172,  18760, 37096, 26804,
                58744, 39060, 23320, 60872, 15004, 28860, 11420, 58284, 48496,
                9372,  42560, 36900, 54720, 30020, 6008,  56524, 26892, 51956,
                46492, 36932, 1620,  42808, 27868, 4000,  55688, 22656, 58200,
                13052, 31960, 1276,  20532, 34380, 24216, 54444, 18808, 58152,
                49512, 30712, 19688, 25112, 54212, 40840, 18004, 21872, 38600,
                53772, 35472, 11888, 48636, 44040, 8880,  59540, 46300, 51372,
                61420, 9984,  23336, 19184, 51272, 59660, 36536, 30808, 4376,
                49724, 32556, 53860, 20264, 12852, 31064, 5404,  38320, 28172,
                7424,  61008, 34512, 3896,  50876, 13796, 33104, 9320,  56736,
                15108, 61256, 44072, 33476, 64268, 49828, 28296, 40708, 51716,
                31880, 44236,
            },
            {
                63876, 13300, 47160, 38408, 58936, 27340, 17208, 39344, 56364,
                4640,  52164, 19300, 5912,  22444, 56888, 45580, 36632, 23272,
                51580, 47100, 31544, 55280, 26896, 42592, 6548,  29476, 34352,
                50396, 61136, 21476, 15748, 25984, 44248, 12508, 60596, 51352,
                1816,  17700, 56480, 35500, 51704, 42172, 21356, 30916, 15856,
                22980, 27740, 18340, 64700, 14860, 48268, 33552, 19100, 39692,
                30364, 24480, 48800, 60620, 19912, 63572, 40280, 34700, 18388,
                51792, 42844, 28548, 56032, 9464,  3864,  46940, 6504,  39936,
                22376, 56272, 44356, 3644,  32640, 11368, 8064,  62296, 26416,
                5312,  31120, 60316, 23156, 2448,  28760, 13284, 21884, 1544,
                35732, 44344, 7316,  62852, 14876, 2648,  43612, 54556, 65240,
                26192, 10720, 6880,  62216, 532,   46012, 49948, 59244, 18560,
                32096, 49508, 22736, 64064, 41292, 44772, 61540, 53540, 22092,
                47812, 29092, 25276, 9924,  6600,  22672, 392,   35248, 14780,
                9080,  17876,
            },
            {
                40448, 33860, 7620,  19312, 61632, 44432, 50888, 13004, 33620,
                25060, 16028, 63364, 35304, 59756, 8960,  152,   62336, 11084,
                38864, 7540,  19368, 12832, 3888,  51816, 20700, 54572, 996,
                18176, 37808, 3336,  47568, 64552, 39564, 55592, 8320,  21156,
                31108, 45876, 11672, 26488, 7244,  64772, 1908,  44732, 4912,
                62680, 49844, 2732,  44552, 24576, 8716,  42232, 12568, 59056,
                5040,  14572, 9368,  53084, 37620, 7016,  11752, 48236, 2488,
                6312,  63240, 14952, 48028, 60584, 37980, 31824, 62624, 14684,
                1612,  34972, 61916, 16736, 59512, 49076, 45060, 34108, 57344,
                15964, 51788, 42440, 18184, 33352, 56384, 38508, 63496, 32104,
                17816, 55668, 49592, 31496, 40520, 27568, 21256, 16756, 39132,
                57016, 46864, 23808, 37304, 58380, 34084, 21740, 10748, 43180,
                13860, 56188, 9836,  30004, 18180, 4780,  26232, 31012, 7068,
                17872, 36016, 58232, 52488, 47036, 37908, 62448, 58080, 45596,
                24188, 60276,
            },
            {
                21380, 50016, 56460, 29628, 11748, 760,   23548, 62592, 42324,
                58564, 29296, 46456, 11860, 27644, 40772, 33948, 16776, 53696,
                25560, 41664, 62732, 49012, 36696, 60988, 39704, 14792, 25252,
                45448, 31468, 58780, 10728, 33896, 1444,  28744, 47260, 34868,
                42336, 63744, 3296,  48968, 38548, 17156, 55428, 36112, 52720,
                32808, 38860, 11548, 57108, 35556, 51768, 61056, 1924,  54832,
                21344, 64900, 33828, 26324, 16348, 45900, 29668, 59972, 25988,
                54352, 35864, 23012, 39880, 17568, 26628, 11676, 43284, 28752,
                53124, 10300, 47668, 27104, 37420, 484,   23700, 13220, 41624,
                2868,  27792, 9208,  64200, 50460, 4824,  47644, 26948, 5896,
                41572, 12808, 24692, 57744, 5248,  48124, 62124, 9964,  1628,
                35032, 15536, 42096, 50908, 17440, 26856, 4540,  63000, 25004,
                37188, 46376, 2692,  58768, 35628, 11196, 46804, 38376, 50632,
                62928, 11796, 1376,  40232, 15884, 31336, 12168, 26480, 4036,
                52748, 30252,
            },
            {
                10796, 2308,  25720, 41892, 53664, 36584, 47520, 19092, 3848,
                10060, 38324, 1404,  55284, 51132, 20688, 47792, 30488, 58228,
                3232,  33492, 56640, 16044, 23648, 30704, 8040,  57316, 64060,
                5572,  52860, 23048, 42708, 54480, 19876, 14724, 62152, 4060,
                24348, 14044, 55100, 30344, 22460, 9408,  59136, 25156, 13848,
                7520,  59988, 20776, 31248, 4096,  17380, 28092, 38136, 31724,
                44472, 50716, 40632, 332,   58588, 22924, 51052, 32844, 20048,
                41192, 12160, 7848,  57628, 2208,  52704, 21076, 50000, 58560,
                24160, 41308, 5620,  13824, 55220, 30176, 52128, 65504, 19768,
                47460, 58828, 37456, 13816, 25624, 40776, 16196, 10340, 58296,
                53288, 29020, 140,   37196, 11784, 33848, 51612, 45116, 30064,
                22280, 61036, 3416,  28884, 9104,  44176, 52560, 40300, 54860,
                6184,  20016, 27572, 39996, 48992, 23208, 57160, 14604, 3960,
                42276, 32464, 23640, 55020, 20096, 43792, 56312, 18556, 48456,
                6688,  37472,
            },
            {
                16524, 45256, 65380, 5820,  15836, 32440, 8160,  28424, 54364,
                49748, 22180, 43784, 32028, 15012, 3904,  64384, 12496, 42632,
                21420, 9684,  28712, 720,   44760, 11040, 47544, 35296, 41488,
                12724, 28152, 16664, 7468,  49960, 25184, 37740, 56840, 10228,
                52428, 39720, 19028, 60624, 33900, 47284, 40792, 28988, 19268,
                46460, 42088, 26736, 53544, 62080, 46924, 22256, 49496, 10288,
                16820, 7144,  28764, 55352, 13064, 43088, 5128,  9892,  65164,
                45080, 27700, 50432, 33032, 44268, 64576, 8224,  35300, 4548,
                18476, 31948, 63976, 21744, 42712, 17296, 6324,  39020, 9820,
                32328, 22416, 45128, 612,   54944, 20944, 61936, 36120, 43968,
                19536, 64728, 47192, 21944, 60676, 18512, 25672, 6768,  58088,
                52684, 12604, 32380, 54948, 64748, 13740, 35408, 1348,  16312,
                32940, 51200, 63304, 15752, 53712, 7528,  64336, 20852, 27528,
                59588, 45808, 5444,  65000, 28484, 2784,  8308,  41228, 32888,
                63280, 55208,
            },
            {
                28004, 58540, 34640, 22132, 49384, 57284, 40652, 60456, 14344,
                34832, 64944, 17764, 7920,  60500, 37632, 24248, 6260,  50252,
                61404, 46084, 52284, 38564, 65060, 18032, 27068, 2336,  21048,
                49432, 61532, 37208, 32796, 63008, 4720,  41676, 17320, 31940,
                49272, 28120, 5652,  43992, 272,   12880, 51048, 3892,  64320,
                55772, 876,   15516, 9632,  40488, 5808,  13620, 63372, 25564,
                35108, 60760, 47312, 19504, 36156, 61904, 38868, 17212, 55752,
                800,   15620, 60216, 5156,  24624, 29936, 39064, 16272, 61388,
                45428, 38360, 49572, 8340,  58628, 34336, 61028, 25268, 49796,
                55780, 5776,  61556, 30332, 34504, 7996,  51692, 23884, 2724,
                33044, 15308, 39968, 8668,  43196, 53876, 13936, 41656, 37972,
                18860, 48852, 5828,  38848, 20084, 46952, 23404, 58084, 28680,
                60776, 43552, 12264, 544,   25636, 33932, 41208, 1768,  36420,
                52096, 18296, 13964, 35400, 47516, 51532, 61932, 36516, 23152,
                13000, 42384,
            },
            {
                50772, 4500,  39376, 12764, 62304, 25312, 1940,  20716, 45172,
                5220,  24800, 56244, 41556, 26936, 53172, 44992, 17352, 35856,
                27128, 13304, 19900, 7116,  31560, 57928, 50856, 55056, 32416,
                8896,  44684, 180,   57176, 13416, 29552, 45024, 60368, 22108,
                7552,  58652, 37044, 15896, 53660, 61812, 23780, 32508, 10996,
                37124, 50116, 34132, 23804, 45188, 36756, 58368, 1044,  42604,
                56580, 3932,  23872, 9008,  31424, 2828,  52992, 24412, 34968,
                30272, 47256, 36824, 19364, 53924, 13412, 47956, 100,   54532,
                11816, 26148, 2740,  53284, 28008, 46508, 3196,  12552, 28888,
                35916, 14904, 18484, 48756, 42848, 12404, 59492, 29972, 49412,
                7040,  52344, 25348, 56132, 4100,  31640, 64208, 944,   27924,
                9472,  62964, 43140, 25744, 2428,  60296, 31616, 11552, 48232,
                8380,  22508, 38812, 31388, 59324, 44576, 17248, 55616, 30340,
                9312,  25192, 57768, 39372, 10768, 22076, 15056, 29036, 58964,
                704,   19680,
            },
            {
                24064, 31124, 8912,  18832, 46256, 30468, 10844, 37368, 52500,
                31592, 12076, 47988, 3064,  34300, 10908, 57508, 30076, 39848,
                1948,  55356, 34072, 60408, 40956, 4160,  22696, 15028, 38984,
                59472, 18492, 25804, 47428, 20784, 9552,  53396, 1252,  35840,
                12612, 45544, 65348, 20768, 26904, 8328,  38300, 17264, 43872,
                21468, 29808, 59160, 63632, 18524, 28468, 32688, 51452, 20012,
                14836, 38208, 53264, 63288, 44132, 27296, 14272, 48440, 59208,
                8504,  22344, 62748, 40864, 9732,  57300, 42732, 27176, 33748,
                20632, 56760, 15600, 36620, 10852, 19408, 40856, 57156, 44244,
                1560,  52724, 39340, 26972, 64876, 3828,  17444, 45488, 39124,
                11068, 61512, 35096, 28572, 16584, 48552, 23268, 34568, 59580,
                46672, 22884, 35600, 57128, 16504, 50712, 6592,  42048, 19040,
                36324, 55476, 5852,  47396, 20248, 9696,  50168, 13028, 61276,
                43304, 48436, 6792,  32244, 26924, 60284, 4700,  44820, 53380,
                9592,  47076,
            },
            {
                35280, 55008, 60968, 52216, 3716,  43040, 55724, 63664, 16836,
                40276, 59696, 29448, 19448, 62788, 22436, 4952,  51652, 9040,
                63880, 24088, 15792, 48520, 25580, 12280, 43536, 63176, 3392,
                29392, 11636, 53896, 39980, 34200, 50464, 64272, 27416, 40568,
                55396, 24632, 3588,  31148, 41732, 58048, 46692, 52864, 5300,
                57380, 8012,  12960, 3208,  49144, 7188,  55636, 10020, 27164,
                46464, 30256, 12124, 17796, 49616, 57812, 20904, 39876, 3520,
                43260, 12608, 51120, 1804,  32332, 22960, 17824, 63168, 50944,
                5332,  60260, 43940, 31300, 64496, 23304, 51384, 15936, 63208,
                21176, 60016, 10896, 7160,  23072, 37684, 54756, 26624, 14612,
                57140, 21016, 1984,  45812, 60220, 37596, 10732, 51296, 15472,
                4712,  30348, 13492, 7812,  45212, 27392, 38176, 53704, 3040,
                64640, 26956, 15156, 61956, 52784, 28268, 4928,  23356, 38028,
                2624,  21516, 63596, 53188, 1056,  40596, 49852, 17784, 31660,
                38976, 63964,
            },
            {
                2560,  16260, 40952, 26276, 36356, 15096, 22684, 7440,  27292,
                476,   50364, 8836,  44588, 39200, 48708, 14180, 60204, 18976,
                47232, 43236, 5640,  53600, 36760, 8388,  46744, 34836, 52008,
                24164, 42904, 63560, 2964,  16212, 23888, 5784,  14556, 19228,
                32964, 47556, 10616, 50556, 35208, 14252, 1416,  28520, 62204,
                24932, 47320, 39540, 53984, 42996, 15744, 22856, 39952, 65436,
                6400,  60288, 41640, 1572,  33676, 5768,  10684, 32312, 62068,
                26596, 55456, 17260, 28352, 46212, 59816, 6328,  36172, 9224,
                29580, 22760, 39832, 940,   48596, 6648,  33300, 38088, 8712,
                25000, 32896, 43456, 50500, 57672, 33672, 20332, 788,   63512,
                31448, 41680, 50592, 12892, 19704, 7348,  56860, 43620, 20412,
                55496, 39548, 53240, 61492, 33084, 21272, 62860, 13252, 24224,
                50052, 44332, 33616, 3684,  40452, 35756, 65332, 46600, 32916,
                56944, 16372, 41768, 12944, 19308, 56048, 34176, 7052,  25788,
                56764, 13884,
            },
            {
                44948, 29212, 48208, 6892,  58300, 32960, 50672, 46968, 61396,
                35880, 23116, 56752, 16108, 1608,  26344, 31524, 41604, 35336,
                28220, 11488, 59112, 30276, 20952, 61400, 27592, 16512, 5888,
                57984, 36276, 7376,  31104, 58980, 37884, 43444, 48772, 57556,
                2548,  62284, 17500, 54672, 6144,  63568, 19644, 33708, 41344,
                15084, 35264, 20000, 26296, 30500, 61188, 36144, 47924, 2544,
                34216, 21836, 50920, 25836, 37264, 64560, 46412, 52252, 19188,
                6760,  38436, 34048, 64356, 4120,  38004, 49820, 12988, 41936,
                46980, 52756, 13448, 18292, 61220, 27148, 54536, 4072,  30016,
                47740, 55708, 2300,  28688, 13216, 46912, 42292, 9216,  47932,
                36064, 4852,  23760, 65432, 30852, 40296, 3872,  24964, 32396,
                63908, 1824,  18468, 41716, 10248, 172,   55804, 34952, 30028,
                9768,  17996, 57536, 12104, 24340, 18544, 55180, 14652, 8088,
                26732, 50928, 35848, 30112, 45728, 23860, 64804, 11288, 43012,
                21304, 6364,
            },
            {
                59048, 23092, 10072, 20132, 64956, 2904,  28820, 10464, 18992,
                43416, 13348, 32596, 52092, 36388, 65488, 54260, 3712,  22020,
                56804, 276,   39156, 17924, 3212,  50704, 56152, 31632, 20108,
                47976, 14312, 22028, 44848, 52628, 11016, 61212, 30616, 8392,
                42532, 27992, 38708, 22468, 44592, 25780, 48988, 11096, 56084,
                2540,  51300, 64556, 10368, 116,   56596, 12732, 52660, 18328,
                44480, 58164, 8036,  55924, 13440, 23060, 41900, 29332, 14756,
                58432, 49256, 23632, 10792, 15676, 53476, 30612, 25260, 19172,
                63848, 4264,  34064, 57264, 45232, 10500, 42508, 58672, 17852,
                12744, 40608, 19920, 63416, 16448, 5328,  59088, 52508, 28748,
                17892, 55260, 8484,  44304, 53112, 26916, 59188, 48332, 36716,
                11728, 49884, 23568, 29180, 51604, 46628, 16012, 43080, 59560,
                39836, 1424,  51944, 30872, 59792, 42584, 744,   29908, 39664,
                62744, 3564,  10204, 58332, 4420,  37860, 15316, 47724, 61572,
                36604, 51244,
            },
            {
                33172, 63028, 38712, 53596, 13068, 42480, 55312, 39768, 5028,
                58920, 25908, 62268, 7232,  20876, 11628, 45648, 8316,  50880,
                15464, 32024, 48940, 64232, 42244, 13656, 37916, 860,   65360,
                40268, 9304,  55432, 28300, 18396, 692,   26132, 20624, 37100,
                52976, 14840, 4580,  60908, 30676, 8736,  37360, 59532, 22984,
                31284, 7140,  44280, 38620, 33548, 21208, 5708,  25588, 31508,
                10756, 15948, 28592, 40172, 18632, 53988, 360,   9052,  36332,
                44360, 2220,  54708, 40600, 43936, 21124, 58764, 1068,  55296,
                38292, 27968, 8452,  24276, 36884, 14564, 21596, 50340, 65124,
                35008, 6240,  52608, 37084, 32040, 25364, 39584, 22064, 12200,
                60672, 26116, 38096, 15888, 34412, 356,   18224, 14364, 8424,
                27620, 45668, 5996,  35920, 59016, 25776, 4604,  19592, 7388,
                47916, 22080, 37712, 45272, 6616,  10244, 47848, 52264, 20560,
                44572, 17904, 48132, 61448, 28192, 8580,  51776, 30640, 2028,
                27380, 17528,
            },
            {
                11932, 380,   31060, 44556, 25032, 35364, 16420, 22052, 31088,
                53196, 2416,  45956, 40908, 29360, 56112, 17252, 24852, 37756,
                62028, 44104, 25996, 9440,  33628, 24356, 45412, 11076, 26436,
                50240, 33176, 60044, 4312,  46868, 35360, 50820, 13020, 64632,
                23468, 33776, 58160, 11764, 40380, 51416, 3400,  16464, 45832,
                54292, 18656, 13664, 59020, 50296, 46152, 63820, 42684, 59936,
                37772, 48192, 63104, 4284,  31316, 48608, 60040, 25956, 62888,
                20828, 30484, 5524,  61592, 27676, 7728,  34748, 45400, 10984,
                16020, 61628, 49436, 53688, 30864, 3356,  39476, 25856, 256,
                44968, 23748, 60080, 8252,  48136, 61988, 3500,  33928, 45000,
                1764,  49344, 58180, 11416, 47412, 62356, 43028, 54528, 61252,
                40036, 57756, 16680, 65176, 12936, 39404, 31780, 63376, 52916,
                28104, 61292, 13328, 26320, 63692, 34148, 22776, 59068, 11808,
                56064, 25676, 34456, 22160, 53676, 42236, 20644, 59876, 5240,
                54940, 41196,
            },
            {
                50260, 57004, 5460,  18060, 48928, 1796,  62464, 57384, 48076,
                37144, 10952, 17984, 49728, 4448,  61108, 42988, 33908, 58220,
                6444,  12964, 54868, 21328, 57900, 6888,  60612, 53776, 42772,
                5236,  17024, 38840, 24032, 63856, 41180, 56252, 5212,  43996,
                2104,  49764, 46272, 18416, 56172, 21300, 65020, 42388, 26400,
                35720, 62696, 28812, 24128, 4288,  17304, 28288, 8760,  54412,
                23368, 1632,  35228, 45736, 57432, 11580, 33816, 16440, 52676,
                46688, 12716, 33408, 17980, 48100, 14208, 65480, 51544, 31568,
                22168, 43164, 1616,  19928, 58992, 46936, 61520, 9556,  31256,
                56932, 14872, 27872, 43680, 11240, 18748, 54104, 15032, 64292,
                40884, 32124, 5116,  21028, 29164, 7964,  23364, 30664, 20028,
                2260,  33428, 53532, 21568, 3120,  48516, 56492, 23308, 10744,
                33336, 5036,  54248, 19732, 50108, 15768, 28024, 36936, 2232,
                32336, 6968,  39276, 12812, 508,   16852, 32656, 39900, 24512,
                14584, 46624,
            },
            {
                21180, 26000, 37112, 59684, 52036, 9484,  33384, 6188,  14328,
                28332, 64380, 24364, 33420, 38676, 13724, 27712, 1188,  47760,
                19260, 29456, 4108,  37276, 51060, 15652, 30432, 18848, 22624,
                62544, 29584, 11972, 53512, 14892, 8100,  31700, 16808, 58720,
                29156, 9588,  26052, 35968, 340,   28824, 33060, 13092, 5140,
                9636,  47844, 1440,  41176, 55452, 37068, 14496, 40312, 5176,
                19888, 51884, 14256, 24376, 7636,  21452, 42920, 5108,  38540,
                8048,  23932, 59452, 37220, 56248, 24316, 3428,  39744, 6124,
                59688, 35584, 12348, 40588, 7400,  16468, 33980, 54284, 19216,
                49040, 38256, 3836,  21348, 56324, 30572, 49868, 37480, 7088,
                22908, 16864, 54656, 63792, 41636, 51864, 38416, 4696,  50692,
                44652, 25020, 9992,  29860, 43584, 8820,  36820, 14916, 44692,
                41024, 17032, 36204, 2400,  56436, 39408, 5268,  61216, 42016,
                14928, 64468, 50332, 43896, 55768, 63396, 46092, 10524, 57788,
                34548, 7948,
            },
            {
                43604, 62172, 15192, 29812, 22644, 40076, 26928, 45572, 20288,
                42816, 51748, 7588,  59964, 21672, 52808, 9700,  64012, 22792,
                53580, 41848, 62700, 46552, 1408,  40932, 35080, 47784, 2744,
                36780, 57152, 45600, 1704,  34492, 27316, 22252, 47680, 38536,
                19708, 61416, 41368, 15488, 53496, 8056,  48396, 60756, 39232,
                52656, 58276, 20284, 33036, 11008, 61584, 31068, 58040, 49484,
                32924, 61176, 41428, 27796, 64916, 36844, 51008, 29064, 55904,
                64288, 41156, 50180, 604,   10212, 42892, 28696, 18660, 56608,
                25668, 48368, 29268, 64664, 24756, 51216, 27716, 5000,  41856,
                11680, 63752, 52076, 34444, 40248, 684,   26208, 10180, 57248,
                29956, 46016, 36392, 25644, 2524,  13864, 60224, 34788, 12180,
                63048, 15432, 37532, 50144, 61184, 19004, 26452, 51380, 716,
                58996, 47176, 62424, 30192, 42888, 8844,  48888, 18408, 53620,
                46556, 29288, 24044, 3800,  35680, 26188, 6684,  49284, 19120,
                65392, 28336,
            },
            {
                3168,  54472, 11048, 7084,  46544, 64548, 12780, 53824, 59116,
                212,   35072, 15616, 55388, 3352,  46588, 31308, 40044, 15180,
                35980, 8872,  32792, 17620, 27076, 64416, 10216, 58784, 13788,
                51596, 8608,  20224, 49800, 62272, 43324, 60088, 10764, 54264,
                32668, 6652,  50572, 63080, 23996, 44008, 56684, 17300, 22348,
                30460, 15044, 43564, 51388, 6844,  47308, 21640, 724,   25828,
                12640, 9120,  55760, 17184, 44796, 2996,  60480, 18960, 1676,
                14716, 27480, 20168, 54028, 32092, 62432, 36416, 49940, 9072,
                44500, 3652,  17960, 56000, 37556, 44128, 13700, 62644, 35828,
                23128, 29716, 7772,  58396, 16576, 45124, 59708, 19568, 42928,
                51968, 12796, 8852,  49808, 32720, 17800, 56088, 26988, 47452,
                7200,  55612, 41772, 1132,  57256, 33984, 5376,  64280, 29364,
                20720, 7792,  25204, 11392, 23008, 64976, 31796, 26032, 10588,
                21296, 58428, 8508,  18744, 61032, 13804, 29560, 37600, 1392,
                52880, 39072,
            },
            {
                17792, 48524, 41668, 35640, 55528, 18432, 4320,  37644, 23712,
                9912,  40780, 29916, 44456, 25876, 18520, 58916, 50532, 3060,
                57296, 24336, 52276, 11940, 21800, 55172, 5628,  23972, 28188,
                41212, 32944, 25968, 39388, 17888, 5512,  51360, 888,   25100,
                14244, 45168, 3080,  30800, 12284, 37468, 4676,  27300, 46316,
                3036,  36860, 24804, 64940, 18116, 27024, 53460, 44428, 63640,
                39144, 46812, 29872, 5764,  53340, 12464, 32372, 25168, 47612,
                35384, 11004, 44984, 5772,  16252, 47156, 21072, 12560, 63384,
                15048, 33636, 52812, 5416,  10476, 21316, 1192,  57240, 48040,
                17704, 2000,  46580, 25324, 12928, 64860, 6180,  35132, 27744,
                62952, 1448,  60552, 20352, 58460, 44904, 9448,  22400, 40744,
                19416, 32168, 27828, 22948, 13704, 46360, 40432, 54932, 11908,
                38304, 50032, 32600, 53096, 15020, 45148, 232,   57136, 35048,
                4704,  38164, 52316, 31976, 40924, 51224, 59524, 23148, 44700,
                12304, 31736,
            },
            {
                6108,  33592, 20840, 60072, 1048,  32080, 43616, 29276, 61528,
                48460, 19272, 62452, 11372, 38064, 6672,  34308, 12224, 27324,
                45092, 5424,  60168, 43124, 48844, 31520, 38196, 45496, 54180,
                16316, 64872, 3436,  58520, 30652, 12888, 36360, 41048, 65180,
                34912, 56948, 21680, 39464, 59144, 19460, 34240, 64048, 8972,
                54068, 59340, 11992, 1980,  35428, 41872, 9624,  16076, 36248,
                19132, 2788,  34612, 59508, 21064, 38388, 49652, 9108,  42520,
                58064, 61920, 30204, 39096, 23432, 59228, 2584,  52148, 30868,
                40992, 22072, 61736, 46512, 30516, 60320, 39624, 31680, 9012,
                53180, 60468, 42176, 32920, 50412, 22272, 38720, 48532, 4452,
                16500, 24572, 38456, 42064, 28200, 3484,  37232, 65012, 1564,
                52300, 59320, 4908,  63772, 17592, 7904,  31160, 23520, 16112,
                58044, 43808, 2704,  60184, 36708, 20340, 51000, 40808, 13684,
                44508, 63112, 16040, 47048, 10364, 20480, 4772,  55200, 15736,
                60532, 25108,
            },
            {
                56676, 63332, 27972, 14080, 25256, 50012, 57760, 16288, 6708,
                52376, 32908, 2496,  57656, 49400, 64464, 42568, 21216, 60980,
                17232, 37476, 28472, 2424,  14688, 61820, 19020, 284,   60388,
                6376,  44100, 11252, 23596, 46364, 55676, 21172, 28724, 8672,
                18008, 52420, 27952, 9276,  48916, 1732,  50864, 42160, 14368,
                32088, 40340, 49888, 29232, 56332, 60436, 32360, 4180,  48776,
                57612, 22840, 50696, 41184, 26748, 15848, 63124, 54516, 22584,
                17396, 3480,  50792, 55628, 8660,  34668, 27508, 6744,  37836,
                58704, 384,   26864, 35396, 14588, 50732, 24380, 45500, 15248,
                26872, 38516, 19476, 10268, 3344,  53656, 30924, 13872, 55848,
                33764, 46436, 54952, 6860,  14616, 48788, 54244, 30040, 16376,
                34592, 12628, 48648, 43120, 53272, 36592, 60108, 48392, 4236,
                26820, 35060, 18440, 5752,  56092, 27892, 7412,  61960, 30236,
                24468, 1452,  55060, 27580, 2916,  43380, 34988, 30960, 8076,
                40304, 50472,
            },
            {
                10624, 2636,  42880, 52668, 9748,  39628, 22152, 11976, 36436,
                25516, 45952, 14104, 22956, 28864, 16124, 664,   54136, 30824,
                49132, 10760, 55712, 34268, 40376, 8024,  50824, 35700, 30504,
                21448, 49212, 37364, 53104, 7688,  61248, 15456, 43816, 59648,
                47800, 5876,  41780, 63592, 16024, 26100, 55140, 23160, 60848,
                18740, 7056,  21848, 44900, 4980,  13276, 23952, 54736, 28896,
                7908,  14468, 61620, 10520, 304,   44476, 7072,  29632, 4820,
                40404, 33788, 25596, 13476, 61272, 41524, 45844, 54692, 24524,
                17420, 49112, 8832,  41996, 18920, 6812,  55588, 3524,  65508,
                34920, 5184,  55232, 62384, 27448, 43636, 8356,  61880, 41564,
                21660, 10976, 30744, 63680, 35308, 23728, 11916, 61244, 45608,
                24652, 38132, 9264,  30324, 25452, 2756,  19868, 41536, 9844,
                52408, 61436, 13132, 41284, 47572, 10432, 38724, 17524, 59024,
                48836, 34008, 21988, 37140, 64512, 58356, 25708, 47620, 62516,
                19440, 36232,
            },
            {
                23660, 16496, 47212, 37516, 4992,  60752, 44952, 1604,  65256,
                56732, 40420, 5564,  53192, 36876, 9612,  47264, 24096, 7816,
                40852, 65312, 20008, 46728, 23212, 57804, 11440, 25732, 42724,
                13512, 57548, 27680, 17268, 34392, 2952,  50236, 31788, 4204,
                23412, 36856, 13532, 33216, 46268, 31032, 11184, 5500,  36164,
                27948, 47496, 63524, 16240, 37864, 51160, 20200, 39420, 64332,
                43448, 25292, 54328, 31036, 47964, 33892, 56660, 37088, 13832,
                47236, 64676, 20712, 1200,  48912, 15692, 19500, 10804, 64852,
                44056, 13268, 57716, 54124, 63712, 37344, 28428, 20380, 43268,
                11652, 23508, 47468, 14816, 36348, 57600, 17336, 25248, 1036,
                59008, 49520, 18200, 4152,  52156, 19828, 42736, 7484,  3884,
                50744, 57144, 21340, 62320, 46972, 11520, 54592, 65424, 31912,
                45456, 21548, 30088, 24784, 63992, 33296, 23456, 53208, 5476,
                11756, 42952, 7928,  14112, 50148, 17508, 11104, 53792, 44,
                28536, 45972,
            },
            {
                59332, 54628, 30420, 64140, 18952, 33744, 54856, 30812, 17756,
                9324,  27804, 20520, 62996, 32376, 56424, 39556, 61840, 34864,
                2824,  15476, 26244, 6584,  62812, 29664, 17480, 53700, 64000,
                8708,  33396, 1236,  62496, 41752, 25608, 38412, 11920, 53364,
                27120, 58364, 636,   56056, 21116, 60024, 40148, 45280, 51892,
                20,    57864, 10528, 25340, 33760, 62128, 46792, 10980, 1108,
                32992, 36832, 4332,  17292, 65268, 23720, 18712, 60724, 27672,
                53440, 10492, 43720, 38020, 29376, 32800, 56956, 2128,  35116,
                29204, 4392,  32740, 23180, 1828,  12352, 49348, 58120, 32264,
                52024, 59664, 29780, 40436, 2444,  20832, 46676, 51712, 29032,
                7392,  37400, 44452, 26708, 59400, 39288, 31872, 56052, 27776,
                18144, 42076, 40,    15764, 39432, 33804, 28216, 14476, 1060,
                38768, 6476,  54164, 1784,  50152, 15556, 3860,  46284, 28708,
                19852, 56628, 61296, 30564, 40640, 6480,  22488, 38496, 14380,
                42052, 5392,
            },
            {
                12788, 35284, 21472, 7760,  26536, 14412, 48152, 24272, 51208,
                42884, 34364, 49632, 3240,  44584, 26072, 19316, 13424, 50868,
                44316, 54608, 32836, 49944, 1760,  44140, 38688, 3324,  47996,
                40344, 19572, 52080, 46028, 10872, 21888, 63928, 44864, 19516,
                62084, 16712, 43216, 49980, 7960,  2988,  18028, 65396, 13844,
                32592, 38912, 53344, 42896, 2564,  6344,  27140, 17476, 59676,
                52168, 20476, 46140, 39640, 12388, 50020, 9236,  43328, 1720,
                32068, 57588, 6572,  52404, 63100, 4740,  22832, 49812, 39168,
                53216, 20676, 47204, 40112, 30316, 45064, 16612, 7708,  40896,
                36,    17884, 8080,  51064, 64444, 32132, 10740, 35344, 40164,
                63192, 13376, 54064, 9512,  656,   15452, 47372, 10404, 36460,
                64360, 33016, 6428,  59776, 51428, 7076,  58476, 24500, 50300,
                17240, 62380, 36444, 44148, 19640, 58648, 42412, 35328, 65188,
                39764, 25176, 51620, 1928,  45040, 28212, 34340, 57876, 51172,
                65036, 32572,
            },
            {
                48972, 40696, 936,   50980, 57836, 41412, 3704,  37152, 6980,
                61548, 15276, 58836, 12084, 17308, 7080,  59184, 4028,  28132,
                22516, 10272, 59364, 37536, 13660, 56616, 32480, 22668, 6104,
                59720, 24640, 14544, 31364, 57224, 6560,  54728, 2196,  33704,
                39924, 10040, 29252, 24100, 38520, 34684, 27432, 56412, 24660,
                20824, 7344,  29348, 19148, 59028, 54992, 31652, 41092, 49300,
                13668, 8752,  57968, 26840, 54912, 2908,  38556, 20928, 50972,
                15184, 23912, 18212, 26888, 12816, 41916, 59568, 26084, 7324,
                62468, 15088, 58960, 9856,  52272, 61100, 35140, 26336, 63216,
                22196, 37228, 45144, 13080, 24608, 5232,  60332, 15208, 56060,
                19564, 23812, 34020, 29768, 64104, 50416, 25320, 60824, 22176,
                13472, 53648, 29800, 44844, 23044, 18676, 43684, 35380, 56080,
                47248, 11356, 26548, 56644, 7840,  30952, 12260, 54876, 592,
                9652,  15420, 35804, 19056, 55488, 62760, 47072, 4024,  20420,
                25464, 8572,
            },
            {
                18320, 27696, 62244, 44372, 10220, 29492, 63052, 12572, 21432,
                55248, 1964,  37896, 30292, 41124, 46888, 52212, 31324, 38668,
                63892, 16972, 41576, 28604, 19628, 9376,  46088, 15304, 28316,
                55604, 37568, 4080,  49780, 28028, 36648, 15952, 29848, 51348,
                6956,  46400, 54000, 63796, 12088, 51552, 42364, 9292,  49056,
                44324, 61596, 12692, 48204, 35888, 15128, 45196, 6924,  24284,
                29844, 63460, 42436, 5756,  30284, 34716, 59036, 26352, 62936,
                41120, 36292, 60568, 45484, 34252, 8488,  46960, 17280, 11700,
                42972, 27688, 36612, 5804,  25044, 20124, 3732,  53920, 11124,
                48148, 56900, 28160, 34212, 54348, 42152, 49356, 26980, 6156,
                2264,  47832, 58292, 43496, 21512, 35184, 4804,  44212, 2604,
                38776, 48080, 26024, 11180, 36788, 62012, 3184,  9036,  20960,
                29308, 6668,  33656, 14340, 40124, 63040, 26160, 21480, 49964,
                32708, 47688, 59888, 4944,  12968, 24192, 9568,  16532, 30912,
                43812, 55776,
            },
            {
                60196, 4432,  15588, 36244, 22988, 18428, 52380, 45816, 27132,
                32532, 47564, 23892, 54216, 63544, 21668, 36008, 8396,  55976,
                45752, 1184,  5472,  53168, 24328, 60000, 51316, 63484, 34872,
                10968, 44408, 64716, 9268,  42756, 19024, 60308, 47820, 13436,
                21484, 36088, 4508,  18852, 31812, 60700, 3780,  16184, 37028,
                1736,  57200, 40368, 24008, 9432,  64964, 20892, 37900, 56516,
                2164,  35392, 22464, 16068, 61760, 44288, 13308, 7264,  46532,
                4308,  9948,  49460, 156,   55544, 21004, 37452, 54452, 31804,
                51112, 708,   18608, 64740, 48688, 13352, 38472, 44448, 31212,
                15508, 4920,  60748, 19432, 1684,  37992, 21572, 44872, 30860,
                52440, 41396, 16604, 5624,  12268, 40132, 18532, 57872, 51332,
                8376,  58944, 16476, 1840,  56636, 47664, 13456, 41824, 64136,
                39220, 60324, 51796, 23132, 48252, 3508,  37708, 17004, 62024,
                6912,  43064, 27356, 39304, 32100, 52188, 41240, 36068, 61552,
                1496,  37544,
            },
            {
                24016, 53484, 33140, 48412, 56176, 2768,  38900, 59692, 9156,
                42008, 19064, 5316,  10532, 27440, 52,    60240, 14512, 24996,
                20116, 34260, 47940, 62712, 35812, 6864,  40004, 836,   20724,
                52780, 17684, 32776, 22600, 58108, 56,    40644, 24392, 65120,
                55984, 26340, 58840, 14732, 44724, 22316, 54612, 29968, 62736,
                33680, 17272, 4800,  52836, 30476, 768,   51024, 11928, 61180,
                17952, 53524, 48264, 10652, 51108, 23956, 17576, 53616, 33276,
                29176, 56392, 22236, 31288, 15576, 65484, 28460, 3512,  60984,
                23088, 45984, 56656, 33496, 42204, 55436, 28552, 58576, 2060,
                40100, 23656, 52360, 9952,  63300, 16792, 57272, 10456, 64924,
                37136, 9128,  28000, 62308, 51924, 55692, 31332, 14828, 28584,
                33936, 20072, 63688, 40200, 31204, 27420, 54224, 32696, 25512,
                15844, 624,   43644, 18764, 59528, 10232, 52368, 44540, 30216,
                23440, 53492, 10664, 21248, 64284, 2484,  58556, 48172, 13920,
                51664, 11280,
            },
            {
                30288, 42984, 7888,  12680, 64564, 31168, 6100,  34224, 16652,
                56380, 64908, 50580, 45228, 34776, 16568, 48920, 43168, 11028,
                51668, 58492, 30584, 11620, 43272, 17160, 26644, 30992, 48528,
                41456, 2900,  26268, 53768, 46796, 12336, 34644, 8932,  3412,
                31596, 41744, 49276, 1176,  39012, 6000,  47384, 25900, 11276,
                50936, 21832, 27556, 44824, 59476, 41748, 26172, 33440, 46340,
                28248, 39780, 4424,  32828, 37724, 1352,  57652, 39884, 19872,
                64172, 13044, 38488, 43772, 5368,  50720, 40900, 13900, 35088,
                6436,  39300, 10956, 29676, 4324,  17044, 8780,  21728, 64392,
                50156, 35940, 42820, 31932, 46648, 7204,  25784, 34824, 14396,
                59760, 20432, 33184, 1356,  45612, 26428, 6876,  65288, 42628,
                24052, 45740, 5080,  52184, 21308, 7268,  17448, 50812, 5224,
                46896, 54640, 30524, 5580,  34856, 28168, 56108, 2508,  13388,
                58136, 36684, 16648, 49204, 43856, 29344, 18368, 7048,  27376,
                21100, 46192,
            },
            {
                5932,  58004, 19492, 40428, 25704, 45312, 14288, 49336, 24448,
                1384,  29356, 13840, 39104, 57988, 5796,  32072, 62276, 39908,
                6316,  27100, 14812, 21864, 3616,  50492, 57580, 13320, 8532,
                61964, 58956, 14408, 5048,  38528, 62980, 27596, 52980, 44208,
                17188, 11100, 61996, 28668, 34764, 58240, 19472, 41068, 6948,
                55556, 38796, 63452, 13572, 34840, 18344, 55104, 3620,  8624,
                43212, 13960, 60164, 20144, 64508, 45620, 27320, 10332, 48196,
                3276,  25228, 51936, 61800, 18732, 10472, 24976, 56104, 49000,
                20036, 63368, 15580, 53308, 24108, 62184, 47188, 33772, 12452,
                17772, 6304,  26960, 14136, 55216, 29520, 41324, 51016, 3164,
                24568, 54964, 49928, 38008, 23200, 10084, 36580, 48764, 528,
                12004, 55372, 36372, 10156, 43188, 62816, 34984, 58716, 10688,
                20616, 36972, 12424, 64660, 42032, 47200, 20488, 33432, 40580,
                5644,  61084, 1212,  54700, 8600,  37800, 23464, 56540, 34556,
                64688, 38972,
            },
            {
                16480, 61708, 28148, 51896, 428,   21544, 54288, 62660, 40832,
                36560, 8256,  59592, 21068, 25448, 55324, 19156, 28608, 2056,
                36492, 64760, 44852, 56200, 38960, 61612, 34100, 45368, 23816,
                37760, 29052, 34980, 50084, 19968, 30936, 15612, 59812, 49816,
                36576, 20360, 7596,  23836, 52012, 9896,  64600, 14004, 45840,
                31416, 2640,  48904, 9124,  5712,  22940, 48492, 15828, 64016,
                23392, 56708, 26568, 7844,  30660, 52764, 5520,  36044, 60128,
                15804, 42168, 7924,  27808, 36572, 59616, 44672, 1292,  30532,
                59172, 26688, 7168,  45224, 40288, 1592,  36988, 51808, 25416,
                44960, 61144, 57468, 20972, 49744, 480,   62480, 18600, 46356,
                6692,  43692, 12548, 17028, 56584, 60148, 20272, 52952, 39140,
                17956, 59672, 26968, 15036, 49200, 24560, 1524,  44792, 40252,
                61544, 27280, 56928, 16900, 24332, 7420,  15144, 63696, 25592,
                46056, 18940, 34284, 26068, 14424, 62224, 3472,  45052, 9824,
                54560, 2676,
            },
            {
                23112, 35524, 47000, 10148, 59500, 37880, 29024, 11120, 20040,
                52040, 44516, 31744, 2800,  42548, 11500, 50384, 46000, 23064,
                53488, 18084, 7832,  31888, 24840, 10056, 19348, 2632,  55000,
                16656, 6920,  44976, 56772, 8156,  43148, 2872,  22568, 5964,
                57500, 39668, 46148, 54988, 16308, 32192, 49956, 36012, 23376,
                60460, 16072, 20552, 37180, 56612, 61952, 39108, 31080, 51672,
                36524, 220,   49692, 42196, 16564, 12632, 41044, 21292, 29892,
                54304, 34452, 58168, 2132,  47864, 22000, 33452, 8464,  17376,
                42796, 37636, 51368, 31928, 57728, 19624, 60180, 5680,  30264,
                53864, 3056,  9184,  33368, 38700, 12580, 23024, 58592, 32444,
                39244, 29884, 64260, 3708,  41620, 32408, 14452, 4336,  62200,
                30220, 47692, 2392,  32804, 57496, 38384, 12856, 29924, 22976,
                4136,  49532, 32584, 1988,  51976, 36508, 57320, 48600, 9172,
                51724, 29896, 42104, 57504, 46932, 32544, 51136, 41076, 14932,
                31944, 49060,
            },
            {
                11876, 55836, 3952,  32164, 15512, 44076, 7228,  58688, 4188,
                26996, 61408, 15908, 53760, 35944, 64372, 7980,  34040, 14008,
                58148, 41588, 49648, 680,   54112, 47336, 29588, 65004, 42240,
                51780, 21732, 63784, 11692, 24952, 54704, 47716, 33880, 26552,
                13888, 29852, 63012, 3992,  42056, 27084, 5632,  448,   56996,
                28140, 42500, 52944, 32400, 45652, 27980, 10504, 4684,  58472,
                18092, 11560, 33200, 54980, 58968, 24864, 62488, 50816, 660,
                45016, 23052, 17668, 31656, 12760, 39760, 54088, 62924, 47032,
                11744, 4164,  22408, 13612, 9348,  27184, 15408, 42616, 10776,
                37676, 19196, 41144, 65456, 45512, 4608,  36480, 53560, 15860,
                10576, 57404, 25832, 47476, 7564,  28184, 45888, 34752, 25468,
                8776,  41740, 21820, 61372, 5860,  19076, 55076, 65196, 15924,
                53228, 9360,  39036, 45560, 61228, 11664, 27864, 124,   38356,
                22308, 11944, 65296, 4048,  21440, 11476, 28240, 60608, 20392,
                26404, 42068,
            },
            {
                63204, 18536, 39748, 24884, 63988, 34904, 48768, 17384, 33392,
                46976, 12000, 23228, 48224, 4884,  17332, 40760, 60540, 4112,
                26724, 10380, 21212, 37704, 60008, 15872, 4924,  36456, 12120,
                26560, 576,   39708, 33020, 17660, 61684, 40284, 10632, 64460,
                50872, 828,   35452, 12024, 21400, 59836, 44568, 19080, 39436,
                7600,  11460, 64996, 24936, 984,   14456, 42116, 21196, 44744,
                29184, 47660, 38788, 21636, 2796,  46420, 33716, 7732,  14120,
                63360, 10704, 49672, 55740, 64524, 5576,  15552, 28556, 23924,
                58112, 33836, 54896, 65136, 46652, 35332, 49660, 56280, 63108,
                23236, 48848, 28832, 16300, 24664, 60076, 8324,  28056, 49156,
                1100,  21712, 35268, 18928, 53764, 63432, 22424, 57980, 51124,
                13516, 53908, 35876, 45404, 28872, 51320, 8280,  46772, 33404,
                43092, 18988, 26156, 6084,  22008, 41108, 17736, 32468, 62176,
                55328, 16156, 7180,  37020, 53036, 39904, 17776, 6564,  57620,
                804,   52056,
            },
            {
                28808, 44460, 6136,  50076, 20512, 1844,  55680, 24024, 41652,
                56672, 1032,  39452, 62684, 26284, 29788, 21920, 52228, 31472,
                47216, 62912, 34276, 13184, 27308, 40768, 22896, 50424, 59272,
                31412, 47484, 60404, 4524,  28596, 36548, 1688,  20856, 42792,
                53992, 18752, 24960, 48448, 56276, 37932, 14636, 63264, 54008,
                30780, 50380, 4484,  18772, 48096, 60664, 53412, 34320, 62568,
                7556,  25136, 65460, 5420,  10808, 37388, 18072, 57132, 26720,
                39520, 36152, 3812,  25300, 41612, 20792, 51592, 2476,  38988,
                50140, 19084, 1152,  42108, 29256, 4988,  20736, 296,   32880,
                14084, 58892, 2016,  52332, 56016, 31684, 19508, 43796, 62064,
                41016, 51568, 60588, 13792, 38328, 3092,  11148, 42676, 6088,
                19752, 64108, 3956,  16552, 11688, 41420, 25616, 36260, 424,
                57748, 13488, 63984, 54600, 30596, 59552, 50124, 43228, 5188,
                24988, 44884, 31056, 48896, 25396, 2780,  63956, 34728, 47548,
                37652, 8696,
            },
            {
                33308, 14548, 60736, 53280, 29932, 12708, 38888, 8724,  65464,
                30428, 19292, 34540, 9240,  51204, 45140, 12296, 56892, 38772,
                16032, 2348,  44512, 51280, 6568,  62164, 45604, 33660, 8176,
                15372, 19284, 53740, 44180, 13932, 49080, 59080, 16276, 31968,
                8700,  45332, 61068, 30828, 7992,  52288, 33468, 10292, 47224,
                25520, 15112, 58256, 40384, 36264, 8244,  26256, 16744, 3492,
                55940, 14192, 52580, 44228, 28932, 61328, 49500, 31280, 42932,
                19936, 59748, 46204, 29240, 8052,  35616, 45684, 32476, 9956,
                60692, 26444, 36692, 16208, 59732, 52456, 38920, 25712, 46144,
                7696,  42288, 36160, 11260, 5600,  39716, 47416, 12868, 25172,
                6396,  33012, 9228,  43200, 49304, 31020, 16936, 36848, 48652,
                32268, 39652, 24712, 56852, 31792, 62844, 20564, 5544,  60652,
                24028, 29424, 47420, 35492, 10064, 3544,  20728, 13680, 53296,
                34944, 58288, 19144, 60244, 9252,  54916, 43252, 29756, 12740,
                23960, 59032,
            },
            {
                3172,  21852, 35868, 9160,  42144, 59960, 27708, 51752, 14884,
                5972,  43752, 54432, 14144, 58940, 37384, 456,   7384,  19320,
                25224, 55236, 29444, 57944, 18372, 10964, 55516, 20576, 3928,
                56604, 37448, 9944,  22328, 51984, 7756,  24444, 56284, 38192,
                27844, 5308,  40756, 16800, 2040,  23772, 28700, 3236,  20976,
                35708, 44080, 32932, 22996, 12360, 56972, 31444, 50316, 41036,
                35688, 19664, 32140, 40192, 15840, 54184, 22712, 2180,  5832,
                53528, 9628,  15160, 52396, 58304, 17464, 62100, 56128, 14420,
                43976, 6796,  48632, 11964, 23572, 8124,  63920, 17424, 54312,
                30508, 61664, 21336, 27200, 50232, 15288, 63816, 3368,  35948,
                56576, 16668, 28776, 1744,  58808, 26396, 61412, 55684, 728,
                28088, 59828, 9840,  47488, 2752,  54148, 49316, 39332, 11464,
                50336, 41756, 2736,  16456, 38568, 45164, 56572, 28848, 8784,
                39336, 2116,  13128, 41592, 33484, 15716, 21976, 50676, 56308,
                16984, 45992,
            },
            {
                41104, 54644, 26248, 47592, 17512, 4732,  45452, 35216, 22232,
                49628, 60508, 28224, 3152,  20660, 32832, 62260, 49184, 43412,
                65224, 9112,  36924, 23136, 32548, 2608,  38420, 27944, 43804,
                63328, 41312, 27216, 34484, 65376, 30956, 46008, 3908,  62540,
                49564, 14164, 58144, 36668, 65044, 46496, 42000, 60976, 55500,
                6064,  62324, 2572,  52296, 63508, 44928, 1556,  21776, 60084,
                10552, 48740, 932,   59456, 26484, 8264,  12984, 44764, 34796,
                64968, 23904, 32868, 1548,  43352, 11912, 25860, 4788,  21656,
                30048, 63604, 39844, 31604, 57052, 43600, 33884, 12372, 40356,
                4520,  48092, 17964, 44312, 33784, 58128, 22580, 30148, 53668,
                20440, 64928, 46448, 23300, 52504, 19456, 7932,  44840, 22788,
                14680, 52372, 18244, 43540, 37508, 13976, 28432, 17284, 32308,
                55304, 7944,  19952, 52636, 62552, 23692, 33960, 64488, 17348,
                47228, 51344, 23264, 63400, 27000, 46440, 1324,  10144, 39596,
                4876,  65152,
            },
            {
                49240, 11660, 76,    63540, 32744, 55332, 19756, 2368,  57436,
                11268, 37964, 24528, 47804, 42272, 15808, 24200, 28500, 34792,
                53032, 14284, 4260,  42696, 47128, 64124, 52848, 13952, 48876,
                23584, 1164,  58252, 6036,  18364, 41952, 13324, 35644, 10496,
                22892, 54524, 32820, 20896, 11512, 50748, 18424, 13712, 38880,
                48704, 17328, 10048, 28260, 19224, 6368,  39056, 13732, 45784,
                28664, 63076, 23708, 6068,  47360, 35996, 62808, 56268, 16980,
                28116, 47436, 38364, 61452, 22124, 31276, 40208, 47272, 36200,
                54140, 17588, 2044,  51392, 20044, 3404,  27220, 50264, 58268,
                23732, 9880,  55228, 62956, 608,   8228,  37924, 43232, 10256,
                48780, 39600, 12036, 36528, 4736,  40016, 33448, 11564, 65172,
                41884, 34688, 6964,  27092, 22112, 60172, 8944,  45896, 63824,
                22548, 34928, 59108, 27492, 11708, 5100,  49488, 1204,  26220,
                59924, 30392, 6944,  38040, 4268,  57400, 62416, 35820, 25640,
                31904, 19304,
            },
            {
                27876, 58260, 38964, 23472, 14296, 37628, 61564, 26556, 41084,
                31452, 17076, 5072,  64536, 8544,  58124, 54636, 11332, 5252,
                40516, 21716, 49580, 58856, 16296, 25092, 30424, 9260,  35536,
                17200, 31304, 12584, 48032, 55520, 1884,  26408, 52348, 19244,
                42540, 992,   26544, 44596, 4216,  57172, 34632, 8584,  27524,
                58900, 24548, 37796, 43052, 49172, 30300, 55572, 24848, 53148,
                7892,  37436, 17088, 57220, 41868, 19844, 29548, 39824, 51504,
                11736, 3976,  18844, 49492, 6632,  55016, 63356, 72,    50448,
                9248,  24732, 59544, 45292, 10488, 38044, 61308, 15392, 2176,
                35460, 28476, 38628, 13676, 25036, 52996, 17364, 61200, 2064,
                26104, 5900,  31720, 56916, 62780, 14240, 47328, 54344, 29880,
                4512,  58544, 48140, 63644, 1396,  35024, 57440, 4888,  42700,
                1112,  14124, 40056, 47912, 31972, 41844, 15380, 37428, 42836,
                10564, 20188, 45508, 54360, 12040, 29076, 20320, 51988, 44292,
                60120, 8044,
            },
            {
                15436, 31180, 51508, 43476, 6212,  10640, 52312, 48620, 8288,
                62940, 45096, 52460, 35432, 30716, 39048, 1680,  45444, 17612,
                31228, 60520, 27488, 11960, 68,    40960, 61004, 5336,  55120,
                50668, 60380, 44632, 37016, 21584, 61720, 39536, 58748, 30080,
                64168, 47712, 9468,  60372, 39800, 24372, 30756, 53048, 16,
                45416, 32288, 54684, 13136, 60272, 35448, 16172, 64220, 33248,
                3044,  43832, 52248, 32824, 11060, 50404, 1304,  24668, 6928,
                59180, 42180, 57088, 35056, 10372, 27928, 15776, 19604, 57920,
                42328, 28456, 14140, 35136, 55092, 29656, 47656, 21764, 45048,
                64604, 51868, 6248,  41808, 32412, 46076, 28632, 50064, 34196,
                15712, 54276, 44624, 17988, 27584, 50280, 21552, 2668,  25612,
                38944, 20324, 12820, 31264, 51688, 40572, 18972, 25488, 30456,
                51576, 62060, 25024, 7032,  56992, 53852, 21616, 61316, 52044,
                33028, 56416, 15944, 35292, 42156, 48664, 16748, 6016,  13200,
                54100, 37260,
            },
            {
                21276, 4224,  61196, 18568, 46700, 30376, 24084, 16264, 34416,
                564,   21560, 55784, 12140, 18612, 51436, 22740, 61852, 48084,
                56020, 8092,  38264, 33504, 50800, 19480, 45380, 22368, 39688,
                2884,  25960, 8060,  15972, 28904, 50100, 8548,  14856, 5748,
                33832, 17144, 51836, 29328, 15332, 6356,  63972, 41692, 22244,
                15832, 6612,  65292, 1956,  22784, 9060,  3964,  41736, 20296,
                58604, 13364, 28124, 22044, 4660,  59912, 15656, 33872, 45716,
                21564, 30540, 14200, 25064, 52160, 45344, 38644, 34248, 12784,
                32012, 3876,  64984, 22592, 4648,  18160, 7396,  55956, 11216,
                31712, 16644, 20528, 60364, 56180, 3868,  12432, 20204, 57336,
                41788, 63420, 22184, 264,   9428,  42356, 35952, 61716, 16916,
                50612, 55856, 8180,  24320, 45192, 15220, 54900, 48644, 11904,
                37160, 17112, 44780, 2464,  18624, 29736, 7956,  13168, 24744,
                6024,  2888,  64248, 23000, 8284,  32020, 60804, 40584, 23348,
                1900,  47472,
            },
            {
                41596, 34244, 9752,  26764, 57244, 64176, 3660,  42852, 58712,
                13852, 28876, 40360, 25920, 3268,  43840, 13812, 36432, 26140,
                3672,  20384, 64440, 54024, 6732,  36564, 56356, 13372, 32968,
                18956, 64584, 41660, 53604, 3440,  33204, 46452, 24832, 55820,
                40560, 3228,  37372, 56240, 20408, 46248, 49372, 11864, 61312,
                36948, 50536, 19996, 41172, 27028, 57472, 46900, 50992, 10636,
                25992, 48952, 39620, 65064, 53564, 44380, 38068, 63780, 9168,
                53868, 61868, 952,   40888, 64780, 3176,  22824, 60248, 6228,
                46396, 53128, 38316, 49192, 42968, 62840, 33304, 39480, 26420,
                42308, 1368,  49392, 35896, 9596,  23108, 64708, 38756, 6856,
                11264, 29160, 35276, 51800, 59132, 31072, 6556,  57196, 10080,
                32928, 43728, 36952, 62316, 3548,  10184, 33112, 64844, 21148,
                56560, 8804,  33800, 59516, 38852, 65408, 45868, 35652, 58224,
                47788, 39672, 28016, 50284, 58724, 536,   26096, 34692, 63004,
                29648, 56156,
            },
            {
                64476, 45676, 53404, 1580,  39944, 32876, 20780, 54192, 37852,
                46344, 60800, 6508,  48328, 63608, 29428, 58636, 9196,  32636,
                41480, 15492, 43640, 23252, 29732, 10384, 63160, 27552, 47604,
                57476, 10828, 34760, 23664, 62716, 12108, 37976, 20232, 44032,
                60860, 22992, 12488, 63268, 33068, 2748,  35696, 18704, 56100,
                30060, 9704,  46380, 33652, 53692, 38372, 18168, 31052, 36688,
                61616, 568,   16492, 7988,  30388, 25664, 12620, 49320, 18052,
                27156, 35784, 47860, 17192, 32748, 8744,  54588, 49116, 26500,
                61772, 20416, 16304, 8864,  25880, 51948, 13360, 60036, 4896,
                53672, 62348, 24420, 14384, 47720, 30932, 43420, 52600, 25308,
                47292, 3300,  60544, 13164, 24436, 45924, 19648, 39736, 47008,
                14532, 572,   27352, 17168, 57800, 28780, 38612, 6884,  3012,
                46644, 29056, 52968, 23020, 11172, 26848, 50956, 812,   30776,
                17920, 11516, 43596, 19352, 14204, 53572, 44996, 9968,  49296,
                18140, 11496,
            },
            {
                6984,  14760, 22836, 36544, 12720, 48416, 7504,  11352, 25476,
                18256, 33400, 9776,  56792, 34652, 20100, 5740,  49536, 62772,
                52584, 12172, 47024, 2944,  58512, 17128, 42192, 1456,  6416,
                51288, 29572, 45944, 5532,  18348, 56744, 51600, 280,   10544,
                49768, 28052, 7212,  42612, 25208, 10188, 59608, 27404, 5120,
                40120, 25016, 14028, 62864, 4708,  12072, 59904, 23616, 6728,
                45632, 55184, 34368, 42688, 19108, 2516,  57828, 31784, 3420,
                39948, 5836,  12392, 56668, 20992, 29084, 43000, 15216, 10620,
                40680, 2556,  34580, 58908, 30412, 400,   23416, 36484, 17896,
                44680, 29040, 7664,  39916, 58684, 18056, 832,   61076, 33192,
                17652, 38100, 48964, 40788, 16132, 55244, 2020,  28040, 64008,
                23644, 60092, 53800, 40008, 48372, 22556, 52632, 43532, 26652,
                60924, 40872, 14920, 48996, 3980,  42956, 15592, 20872, 62916,
                37156, 54144, 61464, 33688, 5228,  37832, 20796, 57116, 5596,
                38592, 25728,
            },
            {
                58236, 28908, 50244, 60452, 17540, 55640, 28588, 61924, 50056,
                2272,  52100, 22312, 15504, 41908, 54160, 38444, 16908, 24156,
                1136,  30452, 60100, 26564, 49288, 34636, 53176, 24680, 38816,
                15376, 21132, 37256, 59992, 42564, 30444, 26824, 64960, 31984,
                35656, 18360, 46916, 54440, 50656, 39696, 15100, 44308, 53880,
                1648,  59120, 51396, 17064, 29312, 44164, 1504,  52400, 15076,
                28440, 20772, 10004, 62280, 47324, 37036, 54784, 23176, 43472,
                52556, 59624, 24544, 44484, 51100, 5408,  58404, 36072, 31172,
                23552, 55824, 45616, 11828, 41512, 56376, 46132, 49788, 9444,
                21404, 57740, 32788, 52324, 5944,  27652, 36408, 14644, 9028,
                58192, 21360, 4752,  26880, 8164,  62168, 34136, 12400, 52416,
                31396, 7252,  19932, 34604, 4916,  13644, 62612, 17788, 35100,
                12308, 268,   20180, 63116, 35992, 32184, 56224, 6752,  41484,
                25752, 8812,  2244,  23716, 47020, 65032, 30620, 16060, 42640,
                51728, 32428,
            },
            {
                44232, 19360, 41040, 4748,  25028, 44836, 35404, 41336, 5824,
                30732, 64696, 44024, 25216, 808,   11296, 27608, 45300, 57212,
                34780, 40140, 18948, 37380, 8272,  13868, 45576, 31492, 58928,
                62328, 8472,  54928, 2920,  14108, 47928, 7704,  16368, 53324,
                4020,  58320, 14184, 30268, 780,   62148, 23360, 32348, 48168,
                21320, 35240, 42504, 8028,  22532, 48568, 33348, 64752, 41864,
                57348, 38784, 50724, 24280, 59212, 6440,  16048, 10884, 62096,
                19776, 29700, 37656, 10044, 62376, 39612, 19128, 712,   64228,
                51636, 6788,  27424, 63668, 18820, 5448,  15196, 31092, 65516,
                2612,  38056, 11444, 45772, 63888, 22224, 49228, 55160, 44924,
                29748, 51548, 65128, 32120, 53244, 37548, 44048, 21748, 3972,
                41064, 49784, 11312, 45432, 59148, 31192, 8416,  47456, 23684,
                58420, 31668, 55144, 44364, 8512,  60648, 23504, 52364, 13452,
                48804, 56924, 29100, 51812, 41248, 11164, 26776, 2792,  61052,
                21980, 4156,
            },
            {
                62208, 54820, 8924,  31908, 65092, 672,   14632, 20024, 58100,
                38924, 13116, 36300, 59636, 47772, 32284, 61240, 4784,  21532,
                10320, 63728, 6140,  56140, 21856, 65168, 4296,  18788, 11696,
                43144, 28320, 49836, 25652, 34076, 40040, 58812, 23192, 45208,
                41204, 62884, 21620, 38140, 6380,  17452, 57292, 8188,  12672,
                64396, 6244,  31244, 56836, 62032, 37868, 19356, 26164, 12544,
                5092,  31232, 2092,  13500, 33264, 28236, 41704, 48504, 34264,
                7192,  14400, 49028, 1920,  33240, 26120, 12792, 48128, 43712,
                17120, 37936, 32388, 48700, 22276, 35648, 59348, 25316, 41012,
                55044, 16168, 26112, 19452, 42080, 4480,  12288, 39544, 24132,
                2568,  42656, 11848, 19344, 46564, 5244,  17484, 58016, 47424,
                35156, 16472, 65200, 26440, 1280,  41928, 56204, 37744, 5488,
                49792, 39352, 25492, 4692,  28792, 18128, 38464, 44880, 3320,
                32948, 17116, 36316, 14764, 7408,  59932, 49904, 36428, 54256,
                13784, 34552,
            },
            {
                1144,  12412, 37376, 47640, 57364, 10348, 52224, 23764, 46620,
                27272, 8876,  56028, 3936,  18668, 51832, 14672, 41580, 54668,
                48312, 15904, 43900, 50860, 29068, 40916, 54296, 23744, 48908,
                33664, 1096,  17056, 64204, 10944, 20092, 5096,  37120, 12864,
                25796, 8748,  33712, 55992, 26700, 42944, 36340, 52072, 28404,
                39076, 18480, 46944, 26520, 11300, 3804,  55484, 8888,  35808,
                46988, 63448, 17844, 44776, 53304, 21052, 64732, 856,   26252,
                45504, 55392, 64264, 22472, 16596, 60560, 52540, 28900, 21840,
                9572,  59596, 14308, 1480,  52804, 39756, 8108,  51484, 12696,
                34312, 47512, 60904, 1668,  34876, 56832, 62668, 32356, 6712,
                61704, 15672, 35180, 57444, 24592, 10324, 29432, 60916, 25400,
                9200,  56780, 30036, 22140, 51628, 15320, 20592, 28084, 64528,
                16452, 9648,  51980, 14040, 47676, 58904, 11020, 27520, 64788,
                21108, 61016, 43368, 58196, 21628, 32680, 18280, 45692, 9532,
                25148, 48304,
            },
            {
                39164, 27512, 22680, 16436, 29264, 34036, 43320, 63232, 2960,
                17008, 49128, 21460, 29616, 39484, 64296, 8556,  36188, 28560,
                31844, 2156,  25440, 12512, 33120, 408,   59824, 37132, 6828,
                57092, 39648, 22428, 53816, 44436, 50976, 61948, 55540, 31084,
                1268,  51904, 46424, 11256, 65236, 49260, 20156, 2240,  45532,
                61192, 14676, 1120,  52800, 40648, 16004, 50172, 43264, 58972,
                21876, 54196, 27284, 39860, 3696,  9132,  51148, 38272, 58568,
                17580, 31672, 4724,  42652, 36904, 45660, 4084,  57424, 40880,
                3072,  54056, 44620, 26468, 10988, 61984, 28436, 3604,  44152,
                22588, 7008,  53428, 30760, 50896, 15340, 28304, 19152, 53960,
                48424, 27180, 40440, 1532,  49196, 63936, 39356, 14388, 848,
                53872, 36864, 4532,  44388, 61508, 33092, 10848, 53392, 3360,
                43628, 33996, 19848, 63776, 41220, 2576,  54052, 35128, 5916,
                46832, 10312, 64,    26136, 39428, 55256, 1740,  29512, 41376,
                64404, 17456,
            },
            {
                51772, 58988, 42776, 5728,  53636, 19048, 37824, 7272,  32204,
                54872, 61776, 34932, 6620,  45820, 26272, 22708, 3068,  59052,
                19636, 52048, 38736, 61660, 17516, 47736, 9908,  26952, 15780,
                63120, 46592, 9220,  35964, 30784, 2112,  27748, 17360, 48820,
                59704, 19580, 39560, 15892, 3180,  31260, 59040, 10296, 24700,
                33612, 56344, 23188, 35752, 60472, 32488, 28840, 24440, 104,
                15572, 33920, 10608, 57596, 61164, 35780, 29516, 12748, 23476,
                41188, 9716,  50964, 28360, 54664, 7852,  11524, 35388, 30988,
                25336, 62948, 19404, 34160, 56904, 46716, 16704, 20064, 63492,
                56472, 28800, 10480, 40388, 23620, 8460,  47116, 37324, 10948,
                21936, 59176, 8496,  54392, 31348, 20648, 43668, 33160, 50584,
                41988, 19380, 13224, 38928, 7004,  48472, 36196, 41260, 23256,
                60812, 29452, 55596, 36892, 22660, 30896, 15668, 50248, 24232,
                38224, 29740, 52652, 48272, 5112,  12924, 61644, 22964, 6640,
                56816, 31132,
            },
            {
                20348, 8016,  63716, 49224, 1968,  60820, 26060, 13916, 51268,
                40540, 10924, 43248, 15404, 54060, 12096, 50132, 42876, 62856,
                11588, 45556, 58012, 5396,  36132, 21384, 44740, 52120, 32116,
                20228, 29012, 4076,  58424, 15024, 23992, 41724, 10108, 35164,
                43288, 6784,  29408, 53836, 22744, 35420, 14248, 40548, 53624,
                4652,  42344, 48616, 8796,  20656, 45440, 5924,  62992, 52104,
                41056, 7260,  49404, 24956, 19600, 47536, 16696, 56700, 2876,
                60728, 35928, 19576, 13380, 59564, 24748, 20436, 65276, 15172,
                50524, 36880, 7456,  41964, 4856,  31400, 37584, 50048, 33064,
                520,   38512, 18664, 64580, 44424, 4092,  58948, 41436, 108,
                33976, 45176, 18108, 37696, 13340, 3532,  56004, 7312,  23080,
                28420, 63504, 58572, 24360, 55524, 17708, 1912,  57708, 13384,
                46908, 6200,  1360,  11852, 45248, 7984,  57624, 19428, 42524,
                55064, 14152, 18572, 62608, 35000, 43760, 51444, 38104, 15292,
                46604, 3316,
            },
            {
                44536, 32996, 14772, 23852, 40272, 11840, 45296, 59100, 22096,
                28616, 228,   24656, 59744, 33144, 57312, 37612, 17236, 6952,
                34384, 23232, 9044,  28312, 54776, 63528, 13808, 41444, 5548,
                55024, 13024, 49468, 40372, 60960, 7476,  46788, 64492, 21972,
                14020, 62076, 24676, 57852, 44468, 5192,  48048, 63756, 30056,
                19772, 12192, 27660, 65100, 2816,  57428, 13356, 18816, 37128,
                59908, 29596, 43664, 14440, 5076,  32512, 43292, 7604,  53356,
                45916, 25972, 62656, 184,   40060, 32228, 47840, 55472, 5584,
                46292, 11812, 23592, 60716, 14516, 53448, 24520, 9996,  42788,
                15468, 59416, 48016, 13056, 35832, 54428, 16880, 25504, 52640,
                64096, 4768,  28992, 60492, 51856, 26704, 62264, 17344, 45516,
                11436, 3764,  47064, 31916, 9920,  28856, 63320, 25692, 32392,
                18380, 50760, 59800, 27152, 49008, 61796, 33280, 1156,  63916,
                8344,  59232, 32148, 23224, 7248,  28508, 19760, 58076, 33608,
                26412, 11320,
            },
            {
                62004, 56428, 27880, 35816, 55448, 30224, 48692, 3748,  18252,
                37280, 65524, 47900, 19532, 6404,  2344,  30308, 47704, 27348,
                55720, 41224, 15984, 49600, 3272,  30748, 24820, 1852,  60844,
                37928, 44204, 25976, 19012, 32528, 55132, 37248, 3628,  52244,
                33128, 2304,  49596, 37540, 9140,  60556, 27124, 16832, 7640,
                57192, 50228, 37492, 16996, 30908, 39644, 47904, 33008, 10308,
                21376, 3984,  64628, 37520, 52624, 61976, 26860, 39324, 21576,
                33196, 5452,  48956, 43724, 53132, 16136, 37484, 2340,  42516,
                18124, 58536, 29600, 48960, 20756, 2776,  62704, 57204, 5864,
                26264, 52208, 22804, 2520,  27720, 32432, 60888, 12028, 30400,
                49456, 15168, 24236, 42720, 10120, 47560, 35052, 38664, 59460,
                51912, 34596, 16360, 40828, 50308, 20884, 45108, 52144, 7624,
                38896, 21948, 34916, 40628, 17096, 24968, 39092, 13096, 27736,
                35476, 3840,  40820, 46160, 15864, 65532, 10132, 4272,  48088,
                54084, 37072,
            },
            {
                372,   41520, 9812,  4492,  19988, 62192, 7976,  34420, 56856,
                44644, 5160,  31112, 51376, 41132, 22040, 64020, 13616, 52432,
                908,   60592, 32616, 43460, 19124, 39208, 57808, 48316, 34460,
                23260, 10192, 65324, 136,   50652, 11740, 16848, 29228, 56696,
                26184, 40764, 12056, 17780, 31936, 21352, 52140, 38740, 45264,
                34720, 820,   59260, 43744, 52920, 25800, 7896,  56136, 51188,
                26920, 46516, 54660, 11896, 23076, 632,   50092, 10600, 65104,
                13752, 17228, 29388, 10112, 22652, 7172,  28740, 61472, 26432,
                33444, 52672, 1000,  38172, 45028, 27724, 40468, 13544, 46296,
                36092, 31048, 8152,  62364, 19488, 48728, 6764,  43440, 20152,
                39528, 7560,  55716, 32516, 4300,  22292, 14492, 1596,  30568,
                8648,  26048, 53980, 84,    62140, 5588,  37604, 11984, 43184,
                65528, 56588, 14824, 6776,  54772, 4372,  52612, 44272, 48188,
                21436, 50788, 11540, 54468, 37396, 49352, 24844, 41792, 59768,
                14096, 22192,
            },
            {
                50456, 17000, 46468, 52484, 38780, 16120, 42408, 23580, 10128,
                53152, 16544, 12588, 36352, 60660, 25916, 9452,  44256, 39112,
                20536, 25392, 12312, 65016, 6748,  52904, 11412, 17688, 7748,
                56808, 16236, 53368, 35736, 27656, 21508, 43528, 59432, 9380,
                19236, 45524, 64084, 55012, 324,   42752, 12540, 3364,  25072,
                61236, 21880, 9504,  13892, 23288, 4756,  63624, 42660, 15324,
                1436,  35224, 17676, 31240, 41340, 57356, 16100, 30000, 47184,
                37640, 54812, 58340, 34784, 64052, 49988, 57328, 11188, 21312,
                39796, 9420,  61972, 16960, 55408, 7648,  34296, 21768, 60352,
                17748, 55276, 40936, 44908, 56228, 38828, 24424, 3020,  57544,
                35212, 61284, 46876, 18872, 65520, 40592, 56988, 48984, 19884,
                64112, 43788, 22116, 36296, 14236, 31040, 58468, 16888, 3636,
                29220, 10176, 44972, 31460, 63220, 20800, 10536, 30696, 61324,
                16084, 56756, 26316, 2700,  30032, 56420, 1308,  31840, 18516,
                7516,  29988,
            },
            {
                24992, 59380, 32256, 64656, 26332, 1064,  50852, 31876, 63272,
                39840, 25404, 57564, 8144,  45680, 54388, 34188, 18120, 57820,
                6252,  36464, 51096, 46004, 22916, 33668, 27364, 61860, 43020,
                29752, 3820,  46280, 41268, 6800,  62256, 47896, 38956, 4796,
                50904, 34820, 5724,  28324, 47636, 36040, 63172, 54496, 30592,
                15740, 41544, 33220, 54996, 46680, 35724, 19860, 29792, 38584,
                62332, 56768, 24872, 8020,  45492, 20120, 35588, 6040,  59376,
                1600,  24168, 41700, 3700,  19044, 39364, 14128, 45352, 53932,
                5988,  43748, 25264, 31820, 12368, 64032, 51908, 29864, 1488,
                49480, 4584,  12188, 33696, 15628, 10160, 64936, 50440, 14012,
                22616, 10668, 988,   37252, 51520, 29380, 6220,  24728, 41820,
                12972, 55996, 6756,  59312, 46052, 26600, 54504, 23096, 34304,
                52736, 25888, 500,   50204, 42128, 36660, 57528, 2108,  24176,
                38576, 5956,  63708, 44416, 20744, 13272, 61088, 35976, 45548,
                64144, 39216,
            },
            {
                43632, 12896, 6848,  21556, 11508, 56476, 45724, 13948, 28064,
                2528,  49084, 18944, 29116, 1012,  14992, 49756, 3128,  31648,
                62360, 10028, 29500, 15480, 59408, 560,   46848, 37300, 21092,
                51196, 33152, 24264, 57484, 14752, 31628, 1856,  23908, 15824,
                30976, 61152, 21892, 14568, 59308, 24072, 18908, 9988,  46312,
                50836, 6288,  64348, 28284, 1708,  60192, 12204, 50428, 22876,
                6472,  11364, 49604, 60696, 2852,  63904, 53432, 25912, 43400,
                20608, 51180, 12420, 46768, 32692, 25576, 1264,  35664, 30072,
                64540, 14968, 50960, 36540, 4472,  19328, 42860, 10088, 37720,
                63312, 24944, 28892, 59628, 644,   36240, 27056, 45284, 31688,
                52716, 28524, 43104, 25736, 12092, 16268, 61468, 53212, 32492,
                3204,  38340, 29120, 18548, 10428, 49068, 2360,  39400, 60880,
                47628, 19088, 58052, 22732, 13576, 27628, 18044, 46712, 8788,
                42760, 31352, 18444, 34440, 8196,  40544, 51648, 22704, 11132,
                52892, 2956,
            },
            {
                19660, 54208, 36744, 48924, 29640, 35084, 61060, 20816, 6336,
                58680, 43492, 32872, 64424, 38312, 21464, 61456, 28076, 40616,
                47136, 53456, 3576,  38168, 41828, 8848,  54564, 14060, 5984,
                63740, 11952, 18728, 60684, 9736,  52320, 36328, 64304, 54484,
                42528, 10392, 39252, 51540, 7500,  32848, 40512, 58068, 4280,
                37184, 25632, 18336, 48980, 38244, 16824, 57044, 3332,  45932,
                31760, 43436, 39424, 27908, 33228, 13672, 40332, 9756,  32208,
                63344, 7876,  27752, 61752, 5592,  55736, 59868, 47744, 18384,
                3048,  57880, 21604, 46208, 59156, 48660, 24240, 57308, 15760,
                43560, 20148, 47348, 51680, 22264, 53588, 18392, 41028, 6352,
                62408, 17460, 54792, 59480, 34584, 45084, 9192,  36772, 17944,
                47120, 60576, 50912, 34128, 41628, 64472, 20192, 44000, 13432,
                4516,  37048, 61748, 32564, 7436,  48748, 64868, 33700, 54044,
                60484, 12292, 50400, 59296, 47524, 28196, 16896, 5664,  26756,
                57280, 34372,
            },
            {
                28952, 63284, 4612,  58640, 41364, 3560,  17436, 39268, 54272,
                35548, 22388, 10316, 52832, 6032,  42856, 55696, 10540, 24000,
                16592, 21720, 56912, 26664, 19960, 63260, 31376, 25140, 48672,
                40080, 55384, 1088,  38392, 26308, 44028, 19400, 12164, 27832,
                49180, 2652,  26004, 55504, 43928, 1588,  28676, 14912, 21188,
                62240, 56528, 11308, 44012, 8212,  32236, 40844, 27260, 53840,
                18356, 59328, 15060, 21284, 51900, 4812,  47392, 18436, 55964,
                14800, 35692, 39868, 53336, 21904, 15624, 42284, 8948,  23560,
                34004, 41080, 10888, 29028, 344,   39316, 32912, 5088,  27412,
                54512, 8420,  39468, 5736,  30576, 61680, 8980,  58408, 2364,
                38152, 48488, 8216,  4176,  21228, 50324, 1400,  57576, 27344,
                11128, 23516, 15604, 1232,  25012, 7544,  31740, 56432, 9032,
                30328, 16056, 40408, 11112, 55212, 2892,  38084, 14356, 20468,
                27032, 432,   36828, 23696, 4400,  54320, 63228, 38092, 48484,
                15956, 9064,
            },
            {
                47196, 24172, 14464, 44504, 10196, 23332, 51756, 8412,  46820,
                13008, 62232, 16052, 47816, 24912, 35004, 13248, 45588, 4632,
                63864, 33924, 43424, 12576, 49364, 34992, 17124, 58660, 2656,
                27600, 34480, 45700, 30124, 49004, 4328,  56360, 33456, 6916,
                20696, 58600, 36732, 18300, 11804, 65364, 48020, 53072, 43556,
                33968, 2536,  30844, 23752, 53276, 62636, 21324, 9884,  65468,
                36872, 784,   8684,  55580, 36404, 23740, 58796, 29188, 2100,
                45788, 49952, 17596, 30948, 10512, 37404, 51512, 27852, 62036,
                49152, 54752, 7164,  25604, 61604, 17640, 13880, 65512, 50840,
                34936, 12976, 62528, 16856, 42624, 46632, 13556, 34608, 20932,
                25096, 14628, 32904, 41088, 63036, 24300, 31016, 42716, 64648,
                4232,  54808, 44200, 62604, 52588, 38072, 16628, 49968, 26768,
                63952, 45884, 51476, 24776, 43036, 59740, 29820, 4444,  51764,
                39632, 44912, 55552, 14976, 10412, 32384, 43888, 844,   30688,
                61816, 40668,
            },
            {
                2332,  37820, 18164, 27224, 55464, 33232, 65080, 29044, 25200,
                1388,  30588, 41540, 3816,  60188, 18332, 30816, 58412, 37408,
                50512, 7672,  60264, 1656,  52732, 4872,  45420, 10932, 42132,
                20260, 15368, 7488,  64904, 22320, 16784, 60392, 40068, 47276,
                62752, 15280, 45860, 30636, 60060, 22396, 35252, 8520,  26680,
                13076, 40228, 50292, 58800, 15228, 5608,  47696, 34024, 13244,
                24428, 48544, 30464, 63072, 43776, 10912, 50508, 38296, 61524,
                25068, 6388,  59128, 968,   44920, 64988, 32216, 6080,  13088,
                38504, 16684, 31548, 51424, 42448, 55532, 8112,  45964, 21952,
                3156,  31432, 56964, 37040, 1936,  23652, 56368, 28716, 51164,
                64436, 45796, 29244, 58212, 18204, 38744, 12676, 48500, 20668,
                35760, 40028, 8208,  28388, 12660, 58436, 5052,  35568, 22504,
                54232, 6452,  1516,  34848, 17412, 21632, 45976, 25428, 62880,
                7916,  17624, 29636, 64764, 41684, 19248, 24684, 58308, 13584,
                21588, 52204,
            },
            {
                57084, 32448, 49984, 62404, 80,    47588, 14648, 43172, 59788,
                37424, 50040, 56884, 27616, 39152, 8408,  51492, 2312,  25868,
                14852, 31800, 19164, 39136, 29720, 24456, 56300, 37764, 62896,
                50360, 59892, 53956, 12840, 41764, 51824, 9148,  28980, 1624,
                23848, 34356, 8372,  3668,  49920, 38604, 4556,  17472, 60952,
                55136, 7020,  19556, 35632, 1124,  26016, 42484, 51972, 3988,
                57552, 39988, 19296, 5816,  26528, 15980, 3264,  20888, 34192,
                12932, 42620, 22572, 54652, 26584, 18856, 3252,  56044, 20704,
                44708, 2456,  63800, 12248, 34228, 20464, 37172, 29940, 41164,
                59576, 48508, 19220, 26540, 10352, 49396, 39248, 3756,  43548,
                11384, 55052, 880,   10152, 52592, 4680,  55980, 15816, 6276,
                30352, 50036, 18692, 33340, 45720, 20948, 41972, 60476, 11284,
                39224, 19588, 28412, 63256, 52960, 9640,  57480, 41416, 11712,
                34080, 57964, 47080, 2236,  35800, 60448, 49736, 8576,  34052,
                42780, 6208,
            },
            {
                11404, 60572, 7660,  39872, 20684, 36600, 5780,  19168, 55204,
                4880,  17324, 9864,  21416, 53400, 44172, 65212, 20424, 47944,
                42020, 54576, 9928,  47288, 64884, 14596, 21648, 8096,  29152,
                3624,  24152, 36792, 31964, 2812,  25384, 37356, 14212, 43824,
                57120, 51168, 40980, 27664, 13504, 56440, 42200, 29528, 49032,
                23204, 46416, 64808, 28464, 45204, 60384, 37872, 17068, 61484,
                28348, 45376, 53724, 33736, 60096, 40892, 64892, 44888, 54016,
                9496,  32420, 63428, 48116, 13780, 40024, 50004, 60440, 35776,
                25884, 57412, 46668, 23388, 4252,  60896, 53388, 1528,  11924,
                24740, 7380,  44392, 53812, 64836, 33256, 16256, 61488, 7404,
                19208, 35672, 22972, 47552, 26812, 33740, 45384, 60972, 26380,
                58552, 11504, 61784, 56040, 3424,  25948, 48036, 352,   32864,
                57396, 44748, 49428, 12916, 31576, 5704,  35528, 16080, 49684,
                23484, 5540,  20820, 26712, 51876, 12912, 3396,  46220, 55560,
                17548, 26124,
            },
            {
                43952, 22332, 30196, 12884, 53372, 59084, 31236, 10992, 22888,
                45392, 34476, 62976, 32004, 904,   15428, 34732, 29248, 6360,
                62576, 22952, 27560, 57604, 6164,  36100, 44116, 53224, 33124,
                17632, 46872, 10356, 44336, 59168, 19528, 63440, 54404, 31536,
                11204, 19000, 64376, 53584, 24716, 19816, 62524, 11348, 12,
                32144, 38052, 13908, 9396,  56292, 20280, 7240,  30984, 22512,
                9232,  15568, 2256,  12556, 47672, 30892, 7644,  18076, 27548,
                57584, 37944, 4040,  29804, 7548,  34388, 24012, 10584, 41672,
                30052, 8004,  18652, 37952, 49424, 9564,  26472, 47808, 17196,
                63164, 36376, 14748, 29196, 5084,  20640, 52152, 26028, 31728,
                39932, 59764, 42464, 16384, 63392, 37464, 8592,  21700, 53144,
                42036, 2432,  23316, 37512, 14660, 53788, 65068, 17940, 29096,
                14964, 8260,  23796, 60360, 40352, 48176, 1748,  54968, 30840,
                59956, 42452, 53576, 9508,  40256, 30212, 22808, 38660, 28228,
                65304, 36108,
            },
            {
                48220, 54416, 16608, 45704, 3144,  26532, 41584, 51560, 63940,
                39032, 25548, 13280, 46696, 60680, 24388, 10676, 57028, 39492,
                13032, 748,   45072, 16804, 32252, 50188, 2184,  59220, 13084,
                39788, 57420, 5260,  27236, 49740, 34448, 7256,  47884, 22172,
                5216,  37912, 840,   32676, 46656, 6044,  33864, 51736, 44252,
                58000, 5272,  18284, 41160, 51488, 33564, 12344, 48840, 55344,
                43016, 36028, 50700, 21408, 55156, 23968, 35788, 51964, 8,
                49416, 15784, 20220, 41912, 58208, 61908, 16404, 47092, 524,
                51156, 14524, 59844, 43024, 28544, 15232, 57932, 43572, 33524,
                55644, 51328, 196,   41904, 57792, 37876, 12300, 44832, 57332,
                14196, 5856,  29868, 2772,  50552, 12256, 30788, 236,   38856,
                17608, 32460, 48828, 43480, 7356,  31100, 10284, 38588, 52268,
                62876, 42648, 2404,  37032, 16876, 25928, 65244, 20056, 37988,
                3596,  14280, 63200, 32756, 16508, 57616, 61560, 5016,  52908,
                14392, 1248,
            },
            {
                58804, 4368,  34324, 64156, 23680, 48948, 8456,  33528, 15356,
                2352,  50660, 54920, 7284,  36216, 42124, 49444, 18020, 53100,
                33048, 36652, 60884, 41000, 11788, 26240, 19436, 62188, 27860,
                48816, 21084, 63128, 16116, 55908, 1272,  39448, 17020, 60580,
                26784, 58360, 43376, 16484, 9692,  59268, 37216, 15584, 25536,
                21444, 53536, 59736, 30104, 24704, 3468,  62044, 39784, 388,
                64672, 24876, 57960, 38884, 9976,  4816,  58480, 13928, 39516,
                61092, 25544, 45804, 11432, 52436, 28128, 4616,  55340, 20080,
                64796, 34892, 53844, 1752,  32604, 64364, 22560, 6272,  39272,
                20364, 10444, 31896, 22912, 60792, 8536,  47684, 1208,  23380,
                53080, 48612, 61836, 20604, 57924, 24368, 44068, 65284, 47708,
                13744, 8904,  63064, 27244, 59248, 35112, 21796, 46884, 3720,
                25412, 34620, 55116, 21104, 58416, 10024, 44956, 28480, 7836,
                46080, 25344, 36192, 48736, 6636,  44828, 18868, 10820, 32156,
                20836, 40508,
            },
            {
                24600, 28640, 10648, 42600, 37588, 17720, 60856, 56520, 28408,
                6456,  42580, 29576, 19420, 58272, 3200,  27840, 5468,  46304,
                58788, 8476,  20252, 51412, 55584, 5400,  38120, 42744, 8884,
                200,   35352, 41560, 31164, 23248, 12996, 28756, 45828, 52552,
                35152, 13636, 49292, 22664, 54568, 28112, 2848,  64180, 40688,
                8644,  35844, 1728,  46816, 15680, 44188, 54076, 26668, 14480,
                32860, 5952,  29652, 17012, 62796, 46288, 42352, 29976, 22404,
                8596,  34812, 56216, 1896,  36608, 21784, 44376, 32840, 39052,
                27504, 11316, 24416, 7136,  40240, 50604, 12380, 30164, 2716,
                59996, 26828, 44940, 49760, 18016, 34152, 28072, 64212, 36672,
                9904,  27240, 33500, 38476, 7452,  15140, 55224, 34472, 28164,
                57772, 52476, 20308, 4360,  16160, 51068, 41552, 5992,  56872,
                18588, 11988, 50844, 6280,  29508, 42256, 53408, 13124, 60764,
                51336, 18672, 56880, 60,    24076, 53848, 37104, 42556, 50680,
                62560, 8772,
            },
            {
                18100, 51252, 61688, 6840,  54760, 13256, 776,   21940, 46112,
                18220, 62228, 9336,  23352, 39956, 51916, 64132, 21896, 14276,
                25700, 30360, 3456,  24092, 63184, 46540, 30648, 15312, 52000,
                24928, 54424, 11656, 7676,  51264, 43132, 65280, 9804,  3328,
                41476, 7632,  30208, 62756, 39328, 45008, 20268, 48576, 12776,
                27648, 61504, 49892, 11168, 64368, 35092, 21136, 8968,  47448,
                19468, 60344, 45100, 3536,  33488, 26780, 18640, 63720, 47504,
                4624,  31368, 65352, 18448, 48724, 8976,  62944, 13548, 6216,
                57304, 48108, 44600, 20936, 56400, 18308, 36212, 53200, 47132,
                15724, 63768, 6804,  13408, 39416, 3580,  54540, 15284, 19908,
                43700, 3776,  17804, 46224, 53300, 40848, 3592,  19104, 6372,
                23756, 36180, 45672, 39608, 55608, 24644, 12980, 61864, 27704,
                45044, 31532, 60828, 47380, 14708, 33780, 824,   22852, 39804,
                32368, 10376, 29088, 41296, 13496, 64252, 27760, 2728,  25456,
                46828, 35624,
            },
            {
                56776, 39204, 32084, 20508, 47096, 30140, 35356, 40972, 53092,
                37660, 57696, 48624, 31360, 16064, 11824, 32612, 37788, 42960,
                62196, 49088, 39116, 14972, 34660, 10136, 21808, 57024, 33600,
                64544, 18580, 46108, 60796, 36488, 57752, 19324, 32056, 24736,
                61764, 20444, 56648, 2320,  11996, 31764, 7300,  57236, 52472,
                31156, 42804, 19016, 23460, 39044, 4344,  29108, 59488, 37524,
                53044, 11604, 40188, 50184, 13392, 53468, 1808,  11208, 51292,
                41492, 54240, 13204, 24408, 40484, 30440, 51880, 25376, 42208,
                17536, 2688,  31428, 63136, 9332,  45540, 4348,  58852, 23944,
                41508, 33956, 52572, 56444, 24764, 62584, 42040, 31440, 50072,
                60112, 56008, 11868, 63760, 30248, 25816, 48556, 61600, 11428,
                42424, 2124,  29484, 10816, 64388, 1180,  30632, 49912, 36816,
                7692,  40684, 2984,  24472, 38552, 63968, 49072, 56256, 16952,
                3096,  63660, 47144, 20956, 59640, 8172,  34348, 15544, 58444,
                6120,  12900,
            },
            {
                892,   48356, 14908, 2928,  26672, 57452, 64428, 5132,  10712,
                26212, 13772, 35044, 4468,  59920, 45424, 148,   55536, 9980,
                4588,  18552, 53900, 44084, 1020,  59420, 49476, 4140,  44496,
                6488,  29300, 38968, 2436,  25808, 5936,  14528, 55412, 38144,
                48068, 16224, 34524, 26360, 51468, 17856, 35516, 24196, 3380,
                16204, 6028,  33316, 57724, 7768,  55732, 50988, 16928, 42936,
                6824,  28396, 21908, 56688, 24696, 37792, 59916, 35604, 20908,
                16672, 27496, 44676, 6660,  60832, 16088, 1172,  58392, 35316,
                50388, 60548, 38184, 15448, 25772, 33356, 61668, 28012, 9816,
                19256, 37672, 1724,  29952, 10172, 46444, 21844, 4808,  7952,
                39180, 24860, 35676, 1776,  21868, 8664,  36704, 58924, 31708,
                49992, 62676, 54300, 17728, 33576, 43916, 19956, 59556, 14844,
                21704, 63308, 17092, 53996, 9100,  19464, 5616,  27268, 35872,
                43736, 52760, 4848,  38180, 31024, 50080, 46068, 22088, 52232,
                31128, 43800,
            },
            {
                27604, 22552, 60252, 52536, 44120, 8900,  24056, 50024, 32352,
                20844, 1320,  65140, 43112, 53952, 25408, 20020, 50216, 28144,
                57912, 31572, 11480, 65384, 25188, 28772, 17812, 39616, 12872,
                22928, 59572, 15528, 48144, 21160, 34032, 49856, 460,   44060,
                10508, 53108, 5516,  46200, 64888, 42184, 59872, 46844, 39772,
                63736, 54132, 45592, 26572, 14116, 41276, 32136, 1432,  24344,
                63672, 35884, 2412,  61636, 8656,  44444, 29340, 7220,  57408,
                38648, 2924,  58832, 33212, 53420, 38228, 46120, 19880, 9900,
                23140, 12688, 28768, 54152, 49264, 492,   13992, 43096, 50272,
                4968,  57996, 14796, 48004, 18680, 36148, 57712, 13264, 52480,
                29072, 15704, 42704, 49636, 57260, 13428, 44668, 16844, 39100,
                21044, 14304, 26304, 6192,  48452, 38108, 9316,  3568,  46784,
                52032, 34828, 26740, 44548, 58884, 31340, 41784, 11584, 61724,
                23916, 14712, 26504, 55632, 17832, 1412,  10700, 61888, 40032,
                19008, 64744,
            },
            {
                53592, 34376, 11192, 36784, 40108, 19176, 14316, 59280, 42316,
                56072, 47172, 28176, 17664, 6780,  36764, 63212, 14744, 35168,
                40812, 22360, 47908, 36852, 7740,  41944, 53320, 62848, 36056,
                51056, 32500, 9524,  54884, 63516, 41876, 29828, 62536, 22476,
                28244, 59384, 40176, 23372, 9096,  864,   13760, 28736, 9780,
                22036, 37052, 436,   60444, 48464, 20396, 62596, 12624, 49516,
                55288, 15264, 47992, 41984, 19668, 15796, 49160, 23292, 32048,
                62288, 9848,  49388, 22356, 12132, 4636,  26492, 63960, 31828,
                55124, 3584,  43908, 6512,  41300, 21812, 37080, 55624, 31836,
                22596, 65452, 27576, 42428, 60668, 2820,  26424, 33768, 44784,
                65028, 21012, 6020,  61044, 33320, 52064, 27888, 1128,  53376,
                8104,  46116, 35716, 60336, 22540, 58372, 53052, 25764, 32032,
                11136, 57060, 1016,  13236, 37076, 22260, 52512, 58036, 46764,
                7884,  34516, 58644, 11980, 43108, 36552, 54520, 28804, 3348,
                37000, 9620,
            },
            {
                41888, 3924,  17404, 63140, 6320,  31456, 45912, 4144,  16976,
                38532, 7936,  51404, 22520, 41072, 30320, 8996,  46856, 2072,
                61804, 6464,  16176, 56680, 33156, 20448, 11228, 1636,  25924,
                57368, 3496,  43948, 27836, 12220, 52792, 8448,  17100, 36776,
                4116,  12756, 32324, 55968, 37348, 20996, 53348, 33728, 57844,
                50192, 12232, 17880, 29888, 36124, 9876,  44108, 27964, 34424,
                18964, 9760,  29728, 32936, 5412,  64504, 56208, 756,   45856,
                14260, 42460, 19444, 35904, 28720, 56296, 43224, 7288,  36740,
                47404, 62108, 19112, 34808, 58500, 64420, 17172, 7480,  45996,
                12364, 39256, 51952, 32540, 6988,  54280, 39808, 61828, 17644,
                584,   47608, 31252, 10252, 40052, 19560, 4936,  64828, 24464,
                30528, 56372, 2972,  41708, 12684, 28672, 16292, 40216, 65340,
                42976, 7036,  29316, 50044, 61892, 4256,  15720, 28796, 2832,
                18696, 40604, 47880, 6196,  65220, 20652, 25360, 45112, 13912,
                56288, 24476,
            },
            {
                46096, 59012, 28684, 47936, 24916, 54488, 61352, 35860, 29372,
                62692, 11492, 33292, 57456, 13176, 59436, 52108, 24220, 18040,
                43912, 26816, 52384, 2696,  45104, 58824, 48420, 31220, 16340,
                47012, 20740, 40324, 17636, 36228, 2032,  24484, 56196, 45276,
                60948, 19740, 49024, 15696, 29592, 44264, 61288, 17244, 5384,
                25584, 43764, 62820, 6700,  23868, 53716, 3948,  56864, 6528,
                45768, 39896, 58708, 53236, 26144, 38432, 12184, 35120, 54632,
                26516, 52392, 63852, 1520,  47300, 61428, 18096, 51516, 14232,
                40444, 27080, 8928,  52112, 24768, 11176, 29608, 51340, 61884,
                26180, 916,   9280,  20988, 16220, 49620, 11612, 22724, 8740,
                38148, 56720, 53612, 25572, 15120, 46880, 57856, 34860, 43240,
                10696, 61304, 19240, 51696, 32624, 88,    56076, 5364,  23032,
                19276, 38268, 46548, 17760, 32956, 8348,  45216, 38156, 50624,
                32496, 62800, 22728, 29992, 51408, 33412, 4644,  60788, 16932,
                49872, 6596,
            },
            {
                21096, 13484, 32588, 56532, 628,   12112, 50940, 22188, 2440,
                53516, 25372, 44524, 1880,  48120, 5676,  38792, 33116, 55832,
                12472, 59864, 38992, 29824, 14048, 24004, 6236,  61364, 38656,
                8312,  65024, 55888, 5440,  60356, 47152, 39656, 13856, 30560,
                51848, 42216, 1836,  62900, 6332,  51176, 26676, 2844,  47700,
                38940, 31620, 55988, 51152, 40264, 15812, 59724, 38352, 25680,
                65252, 22296, 13604, 1972,  50648, 42964, 20588, 28956, 5136,
                17604, 7804,  30740, 41152, 15424, 8856,  24620, 30608, 648,
                21540, 59288, 16328, 32108, 2172,  47016, 40364, 4388,  19676,
                33756, 44112, 56116, 62440, 25104, 43428, 30108, 46352, 59300,
                27532, 13060, 35900, 3856,  62904, 29696, 7460,  21932, 50328,
                16228, 40160, 27500, 7820,  64068, 44520, 47752, 35956, 61168,
                14268, 53828, 60348, 23704, 41304, 64800, 25660, 56164, 21264,
                10016, 53972, 1696,  16216, 39524, 9024,  57860, 41244, 34844,
                30940, 63520,
            },
            {
                38284, 51804, 8568,  42692, 20504, 38396, 27372, 9084,  43580,
                14948, 20176, 60268, 36032, 16884, 27472, 20516, 64864, 3832,
                31028, 49360, 20112, 9392,  64328, 34408, 54828, 43048, 27632,
                13016, 34724, 24144, 31556, 50924, 19116, 33272, 64352, 7128,
                26464, 10164, 34696, 24896, 57228, 11724, 40796, 35936, 64076,
                14448, 8728,  19372, 2380,  33460, 47220, 20916, 31200, 600,
                49864, 7868,  36368, 60784, 17968, 9492,  62832, 48540, 40464,
                57736, 37168, 24228, 59856, 50832, 34164, 38808, 54340, 65208,
                5432,  49932, 44936, 38404, 55252, 21056, 57952, 36076, 15396,
                59976, 48324, 13608, 37844, 34768, 2004,  55728, 4464,  33288,
                50344, 18916, 44016, 22392, 49568, 41924, 37828, 12564, 55460,
                31920, 3844,  36476, 49104, 22060, 18012, 9400,  30136, 50392,
                1672,  27900, 9916,  3784,  51440, 12460, 35288, 320,   60928,
                14064, 43404, 27536, 59600, 46104, 13152, 19424, 48296, 292,
                11224, 26932,
            },
            {
                18352, 2328,  35236, 65404, 15532, 45260, 57888, 33876, 64128,
                40156, 49644, 4592,  29944, 62548, 54548, 45468, 10548, 42596,
                35704, 6908,  25120, 46736, 40700, 17900, 3968,  21752, 49908,
                53292, 736,   46236, 14668, 27188, 10448, 57796, 3100,  47984,
                21140, 54804, 38492, 46824, 18968, 32644, 8352,  20568, 53784,
                23512, 60292, 42044, 27812, 10788, 63404, 13232, 51996, 17372,
                42728, 55040, 28516, 45460, 24384, 31552, 3244,  59344, 22544,
                11212, 46720, 4668,  12860, 21412, 3476,  59000, 11064, 42476,
                35520, 28944, 11624, 63088, 14540, 6232,  27544, 43004, 52200,
                3052,  23784, 28968, 6168,  51044, 64044, 19728, 14600, 41896,
                7104,  63900, 1660,  58728, 9668,  17672, 61588, 552,   44320,
                59284, 25280, 62144, 14560, 55128, 39080, 26236, 12492, 58456,
                33732, 44464, 36624, 57568, 20496, 29964, 48012, 17432, 39928,
                24596, 49664, 36224, 4128,  55400, 24792, 29192, 52852, 22844,
                61344, 54448,
            },
            {
                45708, 58676, 24048, 49236, 30336, 4396,  18148, 47356, 6256,
                31208, 55864, 10436, 42340, 23896, 14372, 1052,  50376, 23144,
                16548, 53040, 62344, 308,   56404, 51012, 36896, 10396, 30736,
                18932, 58244, 41648, 7744,  61788, 43716, 22868, 36020, 41264,
                16560, 58876, 5064,  14016, 65368, 50128, 43372, 58496, 29684,
                1260,  35076, 48696, 57572, 24808, 43880, 5940,  35824, 62092,
                10416, 32632, 14988, 5292,  56148, 37332, 43972, 15756, 33868,
                50564, 64636, 32176, 55748, 43008, 46188, 26788, 18984, 47960,
                23528, 56512, 7372,  25628, 34600, 49332, 60644, 10752, 31008,
                8300,  40500, 59060, 17560, 10900, 39444, 26940, 60232, 52452,
                24324, 30692, 39348, 54796, 33568, 24816, 52856, 28784, 20372,
                6308,  46616, 10328, 42680, 2012,  33284, 62472, 41804, 7156,
                19992, 48684, 25244, 15148, 54456, 6544,  43084, 59044, 5104,
                31732, 64408, 8360,  18312, 32724, 63336, 6808,  37108, 44088,
                4892,  40336,
            },
            {
                14664, 28280, 6172,  40692, 10680, 53964, 61104, 23856, 12992,
                26604, 17712, 37272, 52020, 7568,  33992, 39212, 29324, 61048,
                57104, 37736, 28384, 11648, 32008, 15072, 26200, 60600, 44492,
                4948,  62868, 37212, 21232, 54676, 5264,  29280, 53640, 12176,
                62600, 27624, 44868, 30624, 956,   24260, 16188, 5848,  46176,
                12396, 52548, 15932, 3656,  37552, 53508, 29488, 22524, 46524,
                25132, 40128, 20052, 64160, 47408, 12440, 52256, 8000,  27264,
                4,     18528, 38932, 28364, 16600, 63476, 6736,  32972, 14444,
                1952,  51584, 17316, 43344, 39732, 0,     22780, 16964, 54972,
                64880, 20208, 46328, 54228, 31184, 22944, 48072, 9092,  35220,
                16780, 11072, 47868, 27048, 14160, 4976,  36664, 48064, 15636,
                34012, 54040, 23280, 29644, 57896, 5892,  51860, 23628, 54764,
                16676, 63996, 2504,  39500, 61112, 33312, 9708,  27044, 52844,
                12324, 45340, 21912, 39128, 50764, 41956, 11576, 59236, 16424,
                31768, 9656,
            },
            {
                51144, 33520, 55880, 62476, 21760, 36532, 2276,  51660, 38836,
                58376, 260,   63316, 20804, 47632, 59832, 55320, 18780, 8248,
                13564, 4436,  42908, 19412, 45628, 63564, 2420,  55168, 40020,
                15960, 28644, 11732, 33072, 48404, 39312, 17948, 176,   50356,
                32160, 8732,  22380, 52352, 40224, 55792, 36968, 27328, 62436,
                39576, 31436, 22068, 64992, 18152, 9152,  59076, 4456,  56340,
                2472,  59772, 51140, 34184, 1220,  22012, 29836, 61208, 55032,
                42392, 24856, 9076,  51820, 2136,  36636, 53436, 58132, 41128,
                62372, 31136, 58912, 20128, 53608, 63844, 33080, 47732, 38188,
                25936, 34144, 1224,  42784, 4764,  57448, 44696, 336,   38368,
                55944, 61368, 5868,  20184, 46020, 65336, 56820, 9256,  40836,
                63676, 13288, 38344, 50196, 16944, 20920, 45948, 3488,  37804,
                30900, 42872, 11452, 28732, 46100, 22764, 50212, 36980, 19916,
                55972, 1832,  28540, 60428, 15260, 3000,  46836, 20484, 56096,
                25528, 63912,
            },
            {
                22156, 3432,  18716, 13568, 44712, 27936, 32536, 42496, 9164,
                48792, 34288, 44612, 28568, 12196, 3540,  25164, 45968, 40984,
                32900, 48240, 22340, 59004, 7684,  39184, 23120, 33388, 8084,
                49100, 24584, 52336, 2140,  15132, 56632, 64776, 24116, 44348,
                38032, 3788,  61392, 35124, 10336, 60104, 3008,  48932, 18796,
                9660,  43688, 6072,  46748, 34116, 40868, 49560, 15240, 38236,
                30496, 12728, 7432,  42024, 26292, 57664, 39776, 19496, 35144,
                13800, 61980, 45244, 58760, 12008, 23840, 29876, 8952,  21960,
                37692, 5360,  46476, 10000, 29460, 13592, 7572,  43780, 3728,
                12384, 50368, 15728, 63016, 36320, 13368, 18376, 64500, 28880,
                22024, 42420, 34468, 51644, 39992, 2084,  31500, 22612, 51092,
                26656, 1584,  60124, 7808,  35912, 65084, 28492, 13588, 61376,
                8808,  50488, 58580, 5656,  18272, 64516, 1284,  14728, 62400,
                41608, 34908, 48244, 10580, 54128, 30804, 27012, 34996, 1160,
                48712, 36644,
            },
            {
                60816, 47464, 38424, 52696, 7656,  59372, 15352, 19836, 64704,
                22640, 14608, 5004,  56724, 32248, 16812, 36424, 64608, 2280,
                26900, 62056, 52084, 35040, 29468, 50576, 13100, 46756, 19832,
                57464, 35540, 63748, 45076, 26616, 30964, 9500,  34508, 14336,
                57908, 19632, 47252, 16732, 28948, 21424, 13640, 32524, 54176,
                25656, 60488, 56388, 28704, 12340, 61424, 26660, 20196, 44368,
                53940, 17820, 48628, 62428, 10096, 16796, 4004,  49448, 6972,
                47624, 3680,  21024, 33972, 40656, 48216, 17992, 50600, 44716,
                12436, 25872, 34124, 2840,  50268, 23948, 57036, 19264, 61940,
                29840, 58028, 24068, 8720,  27444, 53016, 33224, 5388,  49552,
                13716, 2616,  57840, 16412, 28692, 12012, 43452, 17108, 58184,
                4544,  45636, 19540, 32364, 43828, 10892, 49016, 40904, 33844,
                25620, 20368, 35748, 53268, 13208, 38732, 44564, 30752, 25460,
                7864,  17584, 58008, 24396, 6420,  38248, 64596, 51528, 13528,
                42120, 6896,
            },
            {
                11396, 43276, 30792, 616,   25568, 48056, 55744, 5556,  30008,
                52424, 39752, 25824, 61900, 41548, 53256, 9308,  50848, 20680,
                55884, 15660, 10280, 1316,  17556, 57208, 4672,  27228, 61700,
                10916, 4008,  40860, 7276,  19736, 60136, 47056, 6004,  52828,
                26260, 41460, 54936, 6516,  50520, 42188, 45268, 63828, 7184,
                35412, 14788, 668,   50408, 23496, 3028,  32456, 7736,  64652,
                35424, 27932, 23284, 37784, 31924, 52940, 43520, 65444, 31076,
                37888, 57056, 29000, 15484, 63096, 5504,  56396, 556,   64756,
                15928, 54852, 61608, 41432, 59496, 36048, 27468, 39580, 52516,
                5356,  36944, 41776, 48432, 20764, 59796, 40736, 24868, 54420,
                44224, 31864, 9608,  63156, 24080, 53452, 60992, 7580,  35368,
                30372, 40064, 52664, 14864, 56564, 24920, 2644,  59484, 15640,
                56236, 412,   47728, 23544, 32560, 56508, 4628,  60028, 51248,
                46740, 3724,  31816, 61768, 43348, 16572, 22336, 8984,  59808,
                18980, 29216,
            },
            {
                24908, 16092, 57644, 34924, 63580, 10592, 37780, 44096, 1500,
                60836, 11140, 18720, 48388, 6536,  22100, 43676, 12952, 30184,
                6900,  39508, 44704, 25444, 65192, 41200, 36128, 53520, 43176,
                30836, 17724, 23540, 55036, 12652, 42296, 37560, 21688, 63092,
                2468,  31152, 12664, 64448, 25484, 58312, 4528,  38608, 23100,
                51432, 41516, 19864, 37904, 43364, 52740, 57948, 47532, 11092,
                288,   57008, 4340,  45756, 58864, 13444, 27116, 22820, 9480,
                17564, 25740, 53220, 7940,  44156, 31848, 26696, 39680, 34940,
                28920, 49120, 21372, 11608, 17544, 44788, 1512,  15236, 46572,
                22204, 10940, 55088, 31752, 1868,  45892, 11616, 7576,  62292,
                19696, 26640, 47948, 36868, 6176,  45568, 38428, 20344, 48900,
                12516, 23820, 62492, 28036, 6408,  39292, 53652, 18896, 7112,
                43508, 29980, 63620, 41232, 8552,  27824, 19252, 10672, 22452,
                39700, 13764, 36512, 20088, 49712, 2188,  56980, 45520, 31380,
                39028, 55316,
            },
            {
                65088, 8480,  50484, 20300, 41100, 23296, 17388, 33824, 26836,
                46072, 35728, 54200, 2160,  38160, 27896, 63152, 34236, 47168,
                60424, 36252, 54188, 31952, 47444, 7368,  15204, 21996, 404,
                38124, 51536, 60720, 32628, 49164, 1072,  58388, 29320, 17572,
                45656, 8992,  39172, 34088, 504,   19096, 30684, 11716, 17216,
                59304, 29708, 9388,  64244, 6532,  17428, 13828, 39660, 21612,
                42644, 51276, 14700, 20388, 6624,  34772, 1332,  54732, 59884,
                41992, 50664, 1040,  60516, 37624, 22496, 11144, 59780, 19340,
                7716,  3460,  38824, 32292, 5696,  65260, 55788, 9560,  33372,
                59356, 18264, 62412, 6664,  16432, 51464, 29692, 36472, 15540,
                39412, 59860, 3572,  55700, 18500, 32716, 224,   27396, 64624,
                55240, 9300,  46808, 852,   60876, 21772, 31144, 46260, 36272,
                51324, 11844, 5168,  16724, 59104, 47240, 42672, 53720, 33584,
                65412, 55564, 26328, 52876, 11088, 40220, 34060, 25284, 4572,
                52068, 1712,
            },
            {
                33268, 44756, 28356, 13904, 5960,  53684, 59928, 50708, 13460,
                57916, 7120,  24136, 31392, 58976, 15164, 56124, 1228,  24540,
                18288, 3376,  22932, 12456, 20244, 59700, 29172, 48572, 64028,
                13720, 46332, 8692,  36384, 28100, 15968, 51708, 11220, 35684,
                49916, 59848, 23036, 15772, 48376, 36468, 52692, 56812, 45908,
                2120,  48320, 33108, 55756, 26368, 36304, 30536, 62016, 25084,
                33588, 63296, 30156, 40504, 60864, 50320, 46640, 38764, 14832,
                33808, 11528, 46252, 19972, 14332, 54924, 2948,  47340, 52620,
                43524, 58216, 25304, 54092, 48564, 22648, 29672, 51304, 25688,
                2228,  44192, 28292, 35204, 39812, 65132, 23016, 58096, 1004,
                52004, 11160, 22676, 41976, 13148, 50112, 59716, 15360, 42664,
                5032,  37012, 18324, 42148, 34572, 50468, 8444,  64188, 27332,
                22240, 61124, 38076, 26168, 50560, 1916,  35908, 15252, 28984,
                164,   9284,  44916, 4868,  63636, 28368, 13260, 60940, 17104,
                48008, 21520,
            },
            {
                59612, 39920, 3004,  61728, 46136, 30556, 3692,  8940,  21664,
                40528, 16428, 62520, 43152, 8636,  19376, 49464, 40212, 10772,
                57040, 43332, 62728, 50900, 4200,  34316, 55368, 10140, 24724,
                57668, 27008, 3372,  20788, 65420, 5732,  24948, 41288, 4044,
                56408, 27304, 43964, 53884, 62588, 9488,  40740, 27368, 5968,
                24212, 62828, 12828, 22056, 45408, 53548, 1652,  49924, 5788,
                16708, 9328,  54120, 25600, 11344, 17860, 29384, 21192, 5716,
                63600, 23688, 29856, 35532, 48824, 64344, 42060, 24628, 13048,
                31188, 63464, 9676,  15344, 42552, 35324, 13316, 40664, 63056,
                37812, 48760, 12600, 52752, 20160, 4192,  43488, 48812, 33632,
                27920, 46036, 35592, 64240, 28452, 53968, 9792,  24508, 33516,
                51500, 22220, 30772, 55892, 11400, 16860, 38572, 13536, 1464,
                54368, 18132, 45528, 33360, 13336, 21492, 62664, 6468,  57516,
                49248, 21692, 41440, 30668, 18456, 47148, 54308, 7508,  42948,
                35504, 12480,
            },
            {
                53248, 23800, 16760, 55196, 35192, 26384, 43312, 64972, 32200,
                47968, 28960, 51356, 12032, 33464, 45348, 4832,  26984, 52028,
                31196, 14776, 28328, 8828,  38640, 45288, 18648, 42140, 6060,
                34624, 17164, 43980, 56504, 38800, 46704, 54624, 62160, 33000,
                13652, 19484, 1992,  7292,  29352, 21916, 14404, 61476, 34528,
                43608, 38280, 16640, 3408,  59632, 10424, 19672, 43820, 60004,
                38016, 47952, 2408,  44856, 36700, 62824, 8384,  58484, 43740,
                2296,  53948, 40620, 4148,  9384,  27856, 33164, 5976,  16772,
                36024, 20636, 46164, 28208, 496,   61080, 7776,  20240, 5508,
                15996, 56876, 24020, 8240,  60712, 26760, 13240, 9296,  18804,
                56464, 5196,  16872, 7668,  31692, 21088, 38980, 47376, 3220,
                58344, 13480, 62988, 48336, 3104,  59620, 24556, 56664, 42380,
                31860, 9800,  57184, 3136,  60052, 52492, 40244, 25160, 18236,
                37592, 60512, 14500, 56316, 35660, 1256,  23196, 38216, 62632,
                27668, 5280,
            },
            {
                49452, 29920, 37752, 7828,  48944, 12320, 19032, 38580, 56824,
                2596,  5612,  20612, 37748, 64312, 54740, 22440, 36736, 61992,
                7196,  48248, 41600, 57692, 25552, 63248, 2204,  30928, 51828,
                61444, 39984, 50352, 14056, 9544,  30764, 18316, 22480, 8296,
                38624, 64680, 31988, 41568, 57956, 47304, 2712,  50568, 19608,
                8604,  54436, 28044, 49148, 34804, 41644, 29208, 55348, 12616,
                27540, 57832, 22372, 32232, 5256,  55648, 24612, 32884, 49168,
                27312, 57688, 15524, 62252, 51104, 18036, 56920, 39308, 61808,
                49368, 2620,  56456, 37984, 18576, 57560, 49876, 43932, 54724,
                30980, 3088,  41728, 32976, 46124, 55272, 38692, 31312, 63616,
                40552, 24708, 60772, 49188, 57992, 1864,  43924, 63584, 17312,
                28344, 40556, 7280,  26216, 19784, 43876, 29816, 47648, 4240,
                35608, 49688, 23588, 39016, 27956, 9412,  31488, 43568, 11572,
                46348, 32672, 7024,  24604, 49544, 59252, 15820, 31756, 9580,
                56716, 18936,
            },
            {
                1892,  10840, 57164, 20940, 63252, 96,    52236, 24060, 14848,
                53780, 44808, 60032, 25436, 464,   29904, 13896, 58324, 17340,
                34428, 20868, 688,   33236, 16324, 53076, 13144, 37504, 21528,
                11768, 1008,  29292, 23244, 59164, 35172, 1340,  44384, 48784,
                53500, 25380, 46320, 11000, 18064, 33148, 25088, 55876, 11872,
                31044, 58752, 40136, 5380,  61856, 15224, 7524,  23864, 35488,
                4244,  18276, 63868, 14032, 41836, 51116, 15876, 39380, 12528,
                18736, 36796, 6960,  32036, 43892, 21504, 1568,  53140, 10560,
                26632, 41528, 6384,  53060, 30884, 11292, 33888, 23500, 27732,
                36680, 64276, 19212, 58616, 132,   16128, 50952, 21956, 2452,
                52984, 14176, 43592, 37496, 12128, 26088, 6904,  55424, 35260,
                10344, 45472, 54048, 37444, 33416, 52520, 5672,  61832, 21112,
                16244, 65184, 7064,  44340, 19280, 48340, 4460,  64300, 54400,
                2932,  27232, 63224, 10780, 39088, 5700,  44420, 52868, 21260,
                46044, 40876,
            },
            {
                32344, 60328, 42992, 25424, 32892, 40292, 58984, 9852,  36640,
                27064, 34256, 17148, 8068,  48796, 41688, 9712,  46308, 3148,
                39592, 64980, 54236, 10864, 22412, 49124, 27904, 58952, 46240,
                55512, 33100, 63324, 6588,  52888, 41532, 12484, 61280, 28660,
                5648,  15500, 51308, 35576, 61120, 5800,  39604, 65148, 37064,
                46156, 312,   18812, 52304, 22492, 32272, 47580, 65308, 51872,
                40564, 46428, 52948, 28400, 548,   20760, 46992, 6484,  64948,
                52564, 22384, 46840, 25760, 11956, 59644, 35196, 45236, 30384,
                58820, 22484, 13708, 64452, 24940, 47296, 3284,  62132, 13736,
                46500, 9548,  50256, 12136, 29624, 36448, 62388, 7724,  46800,
                34496, 29260, 3308,  20428, 33132, 52796, 14936, 30368, 20164,
                61652, 23768, 1700,  15632, 64712, 9056,  14432, 39912, 26944,
                10628, 52240, 29472, 14072, 61944, 56036, 15920, 35200, 21200,
                50436, 17224, 42320, 52388, 19840, 28504, 64152, 3192,  25792,
                13648, 63552,
            },
            {
                16336, 47500, 5692,  13956, 45940, 17496, 30152, 6576,  47244,
                63472, 12636, 57188, 39552, 53352, 18904, 61500, 26064, 51256,
                12520, 24452, 45136, 59804, 36808, 7608,  42924, 4016,  18156,
                8432,  26376, 44896, 19620, 48384, 25848, 16956, 57064, 37148,
                20596, 59584, 3772,  22864, 54604, 43860, 15268, 27816, 6680,
                23428, 14488, 63684, 26620, 11468, 37856, 57488, 980,   20456,
                10468, 33556, 7720,  37668, 57340, 61844, 35068, 30100, 2912,
                41360, 10100, 60944, 56144, 38832, 4688,  24124, 15092, 7536,
                18880, 51624, 34172, 44688, 8404,  40104, 58756, 17828, 7012,
                53552, 22324, 39864, 26372, 45160, 5528,  24552, 42576, 18232,
                59396, 10260, 54620, 65356, 46480, 40976, 60616, 48860, 38336,
                5436,  51036, 57704, 29136, 41840, 22700, 49840, 59092, 36976,
                46024, 57460, 40728, 34092, 740,   37732, 23964, 29148, 8368,
                59784, 36940, 816,   57112, 33180, 12648, 40636, 34736, 50200,
                7152,  36724,
            },
            {
                22432, 27912, 52964, 35580, 3920,  61692, 50288, 42396, 21020,
                1380,  31420, 43868, 22620, 4900,  35188, 31956, 55656, 43316,
                29016, 4564,  18420, 30356, 2532,  64092, 32276, 23900, 62020,
                49588, 41020, 15548, 37024, 4184,  64184, 33472, 2808,  10400,
                43300, 30656, 40396, 13292, 28940, 1460,  48000, 20472, 50224,
                54032, 42072, 33720, 48364, 44300, 5884,  17492, 43100, 30972,
                26108, 61148, 16200, 44260, 11900, 25500, 9212,  55068, 45132,
                28020, 33812, 976,   16632, 29760, 50220, 65076, 40328, 47388,
                62564, 37460, 2976,  27884, 16104, 21168, 36964, 50636, 41920,
                32060, 4088,  59680, 33988, 17696, 52680, 57288, 12656, 27388,
                49676, 38948, 23988, 6112,  17716, 9068,  22816, 516,   12836,
                43060, 31852, 18112, 11632, 46596, 55364, 248,   30708, 18596,
                2980,  24352, 6116,  20752, 53680, 11792, 46852, 58032, 40996,
                13728, 30144, 45360, 22212, 8168,  47892, 60228, 17836, 57776,
                29704, 54596,
            },
            {
                772,   9460,  65328, 19612, 56468, 11568, 26800, 55004, 15868,
                60564, 51720, 10240, 28648, 64832, 14508, 2248,  21220, 8136,
                37564, 58104, 51992, 40456, 47852, 15596, 53416, 39396, 14100,
                35012, 2660,  54464, 60212, 9272,  29420, 45388, 50008, 24460,
                54816, 63724, 47352, 8908,  37312, 63044, 56212, 10388, 34964,
                59940, 29772, 3852,  8920,  60936, 53804, 24720, 12824, 59352,
                50136, 3340,  54716, 23448, 31892, 49044, 16752, 60068, 19692,
                14068, 53904, 48640, 42868, 19824, 8840,  53472, 27436, 376,
                31748, 11484, 55900, 48212, 61116, 52884, 792,   29688, 25496,
                63236, 15212, 55048, 2512,  64784, 9352,  38112, 31600, 61824,
                928,   15708, 57720, 31320, 51228, 36396, 28080, 58176, 53832,
                25768, 47764, 63532, 36504, 5872,  26356, 34664, 12592, 54380,
                63764, 32484, 49328, 59944, 27108, 42376, 3464,  51616, 5484,
                24836, 49048, 61160, 15684, 26592, 53944, 1576,  23824, 10936,
                44800, 39684,
            },
            {
                58872, 48880, 41052, 29112, 38252, 22528, 2376,  33644, 38844,
                24924, 4168,  37164, 49440, 45780, 59360, 38760, 50028, 16096,
                63536, 34220, 13924, 9188,  25524, 20400, 56828, 9720,  28136,
                57976, 20580, 30680, 12316, 22920, 39280, 56184, 19788, 14168,
                34708, 128,   18192, 52544, 24040, 16536, 31644, 41140, 2500,
                17936, 12772, 55912, 20880, 35512, 28612, 39040, 47876, 5340,
                35176, 41472, 19800, 64684, 1888,  39148, 42840, 4172,  36220,
                63480, 23276, 5464,  62300, 37060, 13156, 33640, 21244, 57012,
                43696, 17752, 23776, 41332, 5044,  32304, 12848, 56844, 9972,
                44528, 37716, 20556, 48884, 41384, 23040, 47208, 4596,  35572,
                21736, 45624, 42368, 26648, 4384,  62508, 45364, 16816, 33560,
                8120,  21428, 3116,  40092, 58248, 20356, 61740, 44812, 7912,
                42028, 15336, 10104, 45476, 16580, 64644, 32000, 19052, 62116,
                33904, 10656, 4576,  35444, 65156, 43356, 30832, 38012, 4404,
                62312, 18924,
            },
            {
                32504, 24432, 14940, 51676, 7328,  44860, 48680, 64456, 7824,
                41880, 58328, 16400, 19964, 6628,  25368, 10784, 57072, 23524,
                46860, 5980,  27336, 61792, 43500, 35376, 4656,  45880, 50808,
                6348,  65056, 42252, 46780, 51420, 16248, 6292,  61040, 41760,
                7968,  27288, 58816, 33168, 44768, 59728, 7000,  26456, 62512,
                45696, 24580, 39532, 49716, 15648, 1716,  63860, 21600, 55292,
                14888, 29124, 47040, 7092,  52132, 57780, 21488, 26908, 50476,
                11032, 40168, 31636, 26508, 55188, 45988, 4380,  60528, 14552,
                6820,  50620, 59292, 9528,  35312, 65472, 43340, 23168, 18756,
                47600, 6580,  27976, 11780, 30396, 14804, 56012, 18784, 51088,
                8824,  63980, 12100, 34432, 55660, 14156, 39460, 3760,  60900,
                41632, 55480, 15348, 28256, 9936,  48856, 16332, 37920, 23328,
                28308, 51744, 39120, 2288,  36080, 22604, 8688,  38696, 15000,
                53160, 42812, 55704, 39456, 18800, 6656,  14220, 47060, 52328,
                26684, 13108,
            },
            {
                5352,  45748, 2896,  61440, 31772, 58776, 14092, 18876, 30096,
                11892, 54680, 27784, 33940, 53808, 41980, 29784, 44412, 1468,
                31612, 19548, 55112, 48588, 188,   59140, 30520, 12876, 38332,
                23608, 33916, 640,   26460, 59468, 32780, 3084,  25756, 52824,
                38260, 48720, 21580, 5908,  39728, 12236, 50984, 21080, 36912,
                52712, 5668,  64572, 31884, 57020, 10904, 45240, 32768, 8560,
                62460, 38200, 11304, 25820, 34500, 13688, 8500,  31356, 58860,
                46384, 7364,  15788, 57904, 2388,  23928, 40676, 48308, 36452,
                25852, 39296, 29948, 19656, 14740, 26392, 49380, 5880,  60840,
                32704, 57884, 51364, 36496, 62040, 1848,  43708, 60504, 28496,
                40204, 53164, 2992,  19288, 49108, 9576,  22128, 29436, 48656,
                11156, 35620, 62088, 52088, 43136, 32660, 2152,  53004, 59404,
                3900,  20076, 62352, 29960, 58696, 50348, 56192, 26792, 46336,
                21032, 28600, 1860,  23612, 51456, 58792, 33424, 21644, 56448,
                42264, 35104,
            },
            {
                63704, 56976, 37200, 18188, 10420, 25480, 36380, 43244, 50548,
                23340, 46372, 61848, 884,   13396, 63180, 17928, 35028, 61096,
                52900, 41500, 11760, 38080, 21964, 16520, 52260, 63444, 18592,
                44728, 10572, 16988, 54072, 7856,  36616, 44276, 64768, 29928,
                11556, 56784, 15064, 64148, 3744,  28580, 47068, 57648, 14944,
                9604,  43156, 27248, 3516,  19136, 41392, 52340, 26844, 17180,
                472,   51032, 60420, 18248, 45760, 63196, 54904, 38468, 208,
                18688, 52896, 35272, 43852, 11772, 18460, 64196, 29496, 10276,
                54460, 63576, 1792,  47108, 55308, 37816, 2600,  54036, 40116,
                13944, 1084,  23844, 8468,  53888, 33380, 25232, 6492,  13540,
                32320, 23636, 58316, 38024, 25508, 43396, 64848, 52816, 19068,
                25080, 920,   31004, 23164, 6540,  65096, 25272, 11884, 46404,
                35468, 55264, 13040, 25240, 6552,  11668, 43732, 120,   63468,
                7320,  59336, 32212, 12080, 48060, 27768, 9832,  62920, 16700,
                7784,  50528,
            },
            {
                11336, 22688, 27640, 41940, 53064, 1104,  55192, 5760,  60396,
                3260,  35348, 9344,  40492, 22796, 50304, 4552,  9136,  26524,
                14924, 4956,  65372, 32652, 8072,  25008, 40940, 2216,  27196,
                56692, 61128, 49312, 40144, 20328, 13632, 47972, 22284, 17400,
                2480,  42740, 30820, 51656, 35440, 18496, 42268, 596,   33076,
                59196, 21968, 36168, 47432, 60064, 7308,  23084, 36920, 58336,
                43624, 20860, 32180, 40716, 2684,  28936, 16352, 44044, 24672,
                65448, 29128, 20964, 60432, 49772, 32620, 52524, 5840,  17016,
                21860, 33088, 6340,  43768, 60200, 21608, 30672, 16992, 35016,
                27016, 64256, 43220, 16380, 46684, 19388, 39024, 49268, 56968,
                17140, 47576, 7352,  30200, 59952, 1312,  32568, 6160,  37728,
                59216, 45068, 50420, 13172, 18504, 54864, 40712, 29712, 5456,
                17256, 42912, 33376, 48044, 40944, 18104, 31116, 34956, 13196,
                49608, 37244, 15976, 61928, 44616, 3796,  41448, 37316, 2008,
                30084, 20276,
            },
            {
                43920, 54584, 6992,  48400, 34400, 21456, 63408, 28448, 39488,
                16840, 20708, 52192, 30312, 58448, 37340, 46732, 56168, 39568,
                22228, 45356, 28868, 49720, 56336, 60536, 34156, 47332, 7752,
                31564, 35988, 4972,  29236, 63188, 55508, 31580, 9356,  58020,
                36104, 46216, 24824, 9688,  61140, 22756, 55672, 7880,  29660,
                49340, 17084, 11856, 54844, 30436, 14052, 63148, 4860,  48200,
                28564, 9616,  55840, 6096,  49752, 22628, 35932, 5276,  49020,
                12752, 40800, 4716,  8620,  38308, 26912, 872,   45200, 57388,
                41948, 12956, 52768, 27384, 8516,  11640, 63144, 46244, 7332,
                52776, 21184, 37968, 59224, 28996, 3388,  65204, 10824, 36280,
                252,   62248, 41796, 14348, 51572, 17488, 56500, 12444, 42212,
                16008, 7780,  57416, 39228, 47164, 34560, 59968, 21368, 49880,
                63656, 8764,  57892, 1536,  52528, 21364, 60780, 54780, 24104,
                41272, 19764, 56868, 5304,  35552, 17912, 53820, 23876, 48468,
                59820, 39240,
            },
            {
                216,   31696, 16768, 62504, 12252, 45552, 15460, 8628,  32728,
                57100, 44560, 65160, 14880, 2664,  27212, 19812, 12644, 50932,
                58548, 35564, 18452, 3528,  14624, 43384, 11200, 20600, 54712,
                14324, 22364, 43044, 12052, 24980, 1140,  38440, 51524, 5148,
                62444, 19780, 54376, 1692,  48352, 38680, 13508, 65440, 24312,
                40416, 62236, 1996,  44396, 25328, 50776, 40236, 33508, 12060,
                53204, 24148, 36520, 58736, 12500, 61696, 52688, 9940,  60144,
                33364, 56536, 47368, 22904, 62892, 15116, 59200, 35320, 24208,
                49308, 61876, 36656, 18828, 40748, 50740, 24812, 56552, 41468,
                10200, 48284, 4904,  12212, 55844, 42012, 22576, 51888, 27172,
                44136, 20672, 34204, 10124, 45744, 23068, 36136, 47508, 26744,
                63020, 33852, 22032, 4132,  27040, 1428,  9536,  14592, 38116,
                31480, 26636, 22800, 15700, 65344, 38240, 5720,  45180, 3216,
                29912, 8528,  52708, 26496, 21900, 64040, 30888, 13296, 55872,
                26348, 15008,
            },
            {
                46904, 36364, 58440, 3916,  30172, 40660, 51852, 24184, 49220,
                11776, 26224, 6056,  48580, 32608, 43504, 63832, 6872,  30132,
                1092,  62620, 9788,  53184, 27484, 36888, 5288,  51232, 62484,
                38460, 3648,  59368, 52924, 46168, 18628, 60280, 42248, 23672,
                28928, 13012, 40496, 32260, 16636, 27400, 53056, 35148, 45464,
                5560,  52356, 33856, 8936,  37440, 20748, 1344,  16540, 61576,
                42100, 2384,  15644, 45328, 26228, 31808, 41800, 20404, 27728,
                15400, 1472,  53728, 31292, 42864, 11052, 20104, 40184, 8400,
                3224,  15912, 30848, 144,   58452, 32552, 15136, 3032,  19856,
                29744, 62156, 32520, 24292, 35456, 14996, 8008,  31328, 60160,
                4996,  54544, 26128, 64116, 4196,  28392, 61260, 2224,  20620,
                53856, 10568, 30188, 64056, 16444, 56600, 51564, 44220, 61204,
                2760,  53464, 46956, 35712, 29368, 10068, 26204, 51284, 17276,
                64308, 47124, 38544, 43280, 10440, 49656, 2672,  45480, 8612,
                33792, 64816,
            },
            {
                10236, 24268, 50372, 19204, 26280, 55800, 2936,  35636, 60048,
                348,   41404, 37008, 18636, 10960, 53028, 23676, 34056, 16868,
                42768, 24488, 39068, 46032, 21724, 64164, 31176, 17504, 24636,
                45004, 49996, 16528, 34536, 7096,  27276, 11036, 33624, 15900,
                49936, 7612,  56652, 59980, 6296,  44020, 10868, 3444,  19804,
                15176, 28180, 57492, 18216, 64340, 55404, 47088, 25932, 56908,
                19544, 30512, 63628, 38820, 8204,  17692, 3132,  57148, 45688,
                39572, 25048, 17976, 36596, 3824,  51312, 55576, 29900, 65116,
                26512, 45980, 55444, 22768, 44352, 5372,  38904, 34740, 59520,
                45056, 16764, 50640, 696,   53116, 63132, 47400, 17680, 40084,
                12020, 49096, 16000, 37488, 40912, 55084, 9016,  31216, 50756,
                4828,  40392, 44720, 49704, 36060, 41816, 29060, 24504, 5792,
                18000, 41256, 12416, 7060,  43904, 59852, 14640, 34532, 57532,
                11832, 32988, 900,   14820, 55108, 34120, 60628, 40012, 19700,
                4476,  52172,
            },
            {
                29048, 41116, 7472,  60808, 37940, 10032, 64004, 18024, 46324,
                30724, 55440, 22692, 61712, 57520, 4208,  40088, 60496, 48864,
                54580, 12272, 32064, 56952, 2324,  48668, 40752, 58532, 468,
                28204, 9888,  30996, 65232, 39888, 56568, 48592, 3448,  64316,
                36996, 45152, 25296, 34656, 21280, 63024, 31372, 56352, 50444,
                61360, 39472, 23172, 13188, 42220, 5928,  29284, 8268,  44752,
                34756, 4440,  49472, 21500, 55072, 47316, 64532, 36916, 6816,
                51732, 62572, 9116,  57592, 28272, 46264, 6424,  33596, 12536,
                52412, 38400, 10264, 60688, 49136, 27940, 64920, 54492, 12016,
                26364, 6976,  57632, 39496, 27588, 44544, 4428,  34016, 56160,
                22148, 30024, 6432,  58620, 18824, 48348, 13940, 44064, 38452,
                18240, 58896, 12920, 23624, 7340,  19392, 11324, 58060, 48184,
                33912, 55936, 62908, 20060, 49776, 55080, 4408,  42136, 21524,
                27780, 61072, 23788, 58732, 29336, 6720,  16280, 27516, 57732,
                43204, 21828,
            },
            {
                16416, 57068, 13212, 44876, 33684, 22636, 42636, 6740,  21152,
                13560, 52496, 8132,  34332, 45536, 29144, 15188, 9404,  36804,
                4932,  20136, 61264, 7236,  16408, 25896, 12804, 8816,  34044,
                55336, 41852, 19792, 2076,  23184, 14216, 44216, 30544, 53156,
                19192, 2592,  11536, 51400, 47084, 796,   41464, 26056, 37116,
                8220,  30944, 46636, 3808,  51844, 32280, 58932, 11408, 38348,
                52468, 14188, 59656, 27452, 1168,  33752, 14620, 29272, 12152,
                21536, 34544, 44052, 14000, 63944, 21928, 17060, 48584, 59548,
                19596, 4504,  34360, 17732, 13400, 8292,  20848, 47044, 1640,
                37336, 43392, 22084, 13464, 9872,  19744, 58352, 24888, 1420,
                62872, 45960, 52840, 32712, 3108,  22420, 34576, 62768, 24752,
                55852, 28276, 192,   61000, 52568, 32052, 62652, 37192, 21684,
                9048,  28032, 37840, 452,   32220, 23240, 39236, 48180, 2252,
                50768, 7428,  40612, 19980, 47480, 37948, 24564, 50892, 11448,
                36768, 62180,
            },
            {
                5552,  47280, 31516, 924,   53316, 14720, 49976, 57180, 28340,
                62672, 38500, 16688, 26796, 1780,  50796, 21392, 55916, 25940,
                46608, 28788, 50884, 43756, 34852, 60132, 53756, 45824, 21236,
                62932, 13120, 58156, 51932, 37204, 61676, 20924, 8956,  26076,
                58092, 40072, 61480, 29564, 17768, 14352, 58668, 22248, 12332,
                48288, 1148,  59984, 26692, 36312, 21788, 49216, 17740, 65272,
                23724, 7416,  40964, 11056, 43660, 50968, 23828, 53912, 60868,
                4488,  49668, 30700, 1820,  40672, 26080, 37616, 2804,  43484,
                23792, 41252, 63348, 30116, 53020, 42588, 24304, 31384, 51920,
                18620, 64036, 33840, 49280, 61172, 30448, 38220, 50096, 13836,
                36004, 8704,  42224, 25180, 11232, 59840, 51496, 6724,  16148,
                9672,  34096, 45872, 37892, 14980, 5320,  43672, 1296,  52872,
                14224, 45904, 25124, 57352, 16488, 11456, 28976, 61656, 18708,
                36500, 45012, 53480, 12700, 64432, 1904,  44648, 62984, 31000,
                2764,  53732,
            },
            {
                25892, 38636, 21204, 65300, 27636, 4416,  39384, 32196, 10660,
                44532, 5296,  50068, 42164, 12228, 64824, 31608, 41712, 396,
                63456, 14472, 37700, 10304, 19584, 30304, 38672, 6152,  49504,
                36412, 26288, 7496,  46752, 28924, 5164,  54960, 35768, 42900,
                13876, 6076,  22720, 37956, 8304,  54204, 35740, 4792,  63948,
                18508, 54396, 43388, 10516, 63380, 15080, 41112, 240,   27800,
                32700, 47140, 56592, 35772, 61896, 18472, 8892,  39816, 43076,
                26868, 18836, 58528, 54156, 11388, 50544, 61640, 15152, 56848,
                28628, 7592,  47780, 1300,  36084, 57392, 5920,  15688, 60376,
                9340,  28912, 55600, 2968,  42536, 6520,  16460, 54116, 43848,
                28376, 20140, 15496, 64692, 38652, 27612, 41408, 1872,  30280,
                48228, 65400, 19796, 25748, 55620, 47652, 27256, 17176, 64840,
                30796, 60632, 4536,  42800, 52052, 64568, 6852,  33604, 55764,
                9764,  15992, 31584, 4560,  34948, 51928, 18912, 7788,  14580,
                48716, 34232,
            },
            {
                61340, 9728,  50588, 17048, 59096, 46232, 24780, 60964, 1328,
                34976, 23424, 56548, 19200, 59652, 36400, 7512,  18196, 59152,
                33188, 22880, 3688,  55468, 65112, 1372,  23480, 14960, 29168,
                2864,  18088, 43648, 33252, 10596, 49420, 17648, 420,   63648,
                32856, 48344, 55868, 43652, 64820, 32316, 49680, 45308, 28444,
                40476, 14484, 32752, 19720, 55492, 4664,  45252, 60400, 54268,
                5324,  20292, 16164, 25116, 30644, 4960,  58888, 32128, 588,
                47748, 14868, 38752, 23492, 5636,  45220, 35064, 9540,  32300,
                51740, 12068, 55960, 25540, 19336, 62708, 39160, 33544, 48408,
                40928, 5008,  15068, 25288, 35852, 22608, 64952, 10880, 2860,
                60340, 56788, 49540, 444,   55268, 46392, 20548, 57524, 36960,
                52788, 42432, 4600,  8564,  59948, 22500, 34680, 40920, 49848,
                7448,  39440, 19072, 9796,  34900, 46536, 26036, 13696, 43408,
                24296, 62456, 57684, 26752, 21992, 10556, 56748, 40472, 23344,
                42764, 19384,
            },
            {
                1932,  56024, 41316, 6644,  34648, 11968, 19684, 53644, 16364,
                47756, 64120, 30604, 4352,  46484, 24256, 53384, 48256, 11720,
                44636, 52016, 40524, 27036, 47540, 42488, 51588, 57680, 61348,
                39976, 54840, 63548, 15564, 25144, 60464, 39176, 23536, 51700,
                28232, 16728, 2080,  27020, 20032, 3600,  24536, 9436,  52644,
                34616, 58508, 24616, 7492,  38716, 29556, 23384, 34564, 12504,
                38996, 62688, 52116, 2192,  38212, 48872, 22120, 55796, 11080,
                35892, 65216, 8276,  33324, 60604, 19940, 27476, 440,   64668,
                21240, 39000, 44160, 14768, 50504, 4124,  45920, 11424, 25716,
                58280, 20808, 45184, 52928, 59276, 46948, 32112, 40784, 25952,
                33436, 5428,  36992, 30404, 13552, 8128,  17152, 63412, 10972,
                23028, 14132, 32340, 39188, 50916, 10604, 3556,  58624, 12524,
                23976, 33096, 48732, 58844, 22004, 40260, 3912,  59316, 51472,
                29832, 540,   38804, 49252, 43068, 60912, 29140, 33136, 54408,
                59448, 12244,
            },
            {
                30260, 36236, 23124, 28620, 43588, 56624, 8148,  36556, 41388,
                27144, 13988, 9144,  40320, 33532, 14696, 2856,  38524, 29440,
                8236,  16232, 58072, 5688,  12596, 17552, 32952, 10876, 44988,
                5068,  22936, 51072, 1664,  42084, 56740, 30768, 12408, 44804,
                9784,  60708, 35960, 52956, 11948, 59192, 42280, 16660, 21144,
                6012,  2168,  50104, 46724, 62104, 51940, 9600,  18724, 49700,
                42916, 29796, 9424,  45644, 64588, 13180, 44004, 17036, 61224,
                25076, 52060, 44488, 29524, 16396, 55932, 52576, 41380, 46664,
                17204, 5348,  58488, 32784, 9180,  28380, 22144, 61568, 580,
                53796, 37092, 31272, 12064, 1216,  18540, 8032,  51684, 21224,
                47888, 16828, 44608, 23396, 51128, 35436, 31980, 44296, 26772,
                3304,  56712, 61336, 27820, 16720, 43360, 30244, 19512, 53444,
                1976,  55812, 15412, 28416, 1336,  54172, 17808, 37292, 21008,
                5684,  47104, 18172, 13580, 6936,  37328, 16356, 964,   5948,
                27052, 46180,
            },
            {
                52292, 63588, 15104, 48936, 2356,  62616, 31096, 49376, 3552,
                57972, 21288, 50996, 54788, 28096, 61912, 21624, 57268, 25040,
                62628, 35160, 20692, 31140, 36812, 60740, 7628,  26852, 19232,
                34816, 30904, 12668, 36676, 20376, 7700,  46984, 4056,  58348,
                20812, 41008, 7296,  46228, 15052, 38560, 29304, 62580, 54888,
                39360, 60932, 27456, 12216, 35744, 2588,  58868, 26196, 56488,
                3256,  22828, 14536, 57816, 26976, 7124,  34588, 29200, 40732,
                6300,  21000, 2096,  48892, 39676, 4316,  13332, 24204, 60116,
                36720, 30380, 22968, 64088, 37608, 42312, 55568, 14292, 30160,
                17628, 8192,  50752, 62784, 39852, 28060, 55664, 61332, 13072,
                38728, 63884, 9476,  58692, 3944,  61968, 54500, 6052,  40316,
                49972, 18700, 45372, 896,   54168, 64072, 36908, 47028, 26884,
                35560, 62072, 44944, 11880, 63836, 31408, 48276, 8860,  63352,
                41352, 55420, 33244, 65388, 53360, 25152, 46584, 50452, 63060,
                39260, 17356,
            },
            {
                43032, 3640,  11012, 54880, 20312, 38480, 13112, 23216, 61268,
                6676,  44400, 37308, 868,   11044, 43956, 50176, 6048,  42092,
                54332, 1944,  49612, 63808, 24532, 53424, 40824, 48604, 56232,
                64812, 8916,  47872, 53892, 28068, 62000, 34208, 15200, 37500,
                53504, 31476, 24880, 62120, 33820, 50784, 24,    47052, 11236,
                30880, 44284, 22268, 16940, 42748, 32016, 14460, 46040, 6832,
                35544, 60920, 32576, 41136, 18884, 54908, 49824, 2720,  53556,
                14148, 28252, 56728, 11704, 62740, 36288, 31344, 6996,  50164,
                10716, 3248,  52728, 47616, 2068,  18412, 51236, 34800, 40096,
                65500, 43704, 23056, 5284,  34104, 15444, 43336, 3312,  30012,
                6224,  52932, 19616, 28652, 42512, 25352, 21332, 14656, 59536,
                29776, 36284, 11592, 33648, 21708, 6132,  14416, 60304, 9364,
                41328, 20900, 5660,  38000, 24140, 42360, 14228, 27112, 34672,
                12532, 23832, 9584,  28828, 4212,  58516, 20584, 10208, 34676,
                22208, 7972,
            },
            {
                25432, 31676, 60056, 33944, 26176, 46504, 16692, 52596, 28844,
                34292, 17744, 62764, 25196, 58524, 16284, 35796, 31856, 18400,
                13756, 27756, 45732, 9724,  43544, 1076,  22164, 4568,  38128,
                16192, 25648, 59896, 39792, 4744,  18304, 50616, 26448, 65416,
                1772,  17408, 49556, 4412,  22304, 57824, 7860,  25964, 36208,
                15256, 56220, 8440,  63424, 53752, 20236, 40348, 64932, 28528,
                17052, 47980, 51784, 676,   37240, 24036, 10484, 63732, 46568,
                37680, 60176, 42304, 18792, 25556, 47180, 58608, 20524, 43464,
                26832, 62052, 16516, 12476, 26008, 60956, 6716,  10036, 48164,
                3288,  27072, 13968, 47796, 58172, 24900, 49708, 36140, 22748,
                57636, 40968, 34712, 12048, 48744, 38036, 960,   46560, 52988,
                8652,  24252, 62420, 49052, 40624, 25696, 50312, 4276,  44440,
                16904, 29996, 49548, 52444, 7652,  60256, 55164, 3608,  51076,
                58772, 45664, 19524, 40048, 35372, 14896, 44092, 30600, 54984,
                13980, 58116,
            },
            {
                51040, 18900, 45436, 40308, 4836,  64224, 328,   42608, 9932,
                55896, 12800, 31588, 48024, 19904, 3452,  65228, 9416,  47364,
                61320, 4984,  37776, 19180, 15244, 59712, 29004, 13624, 51840,
                44144, 28,    21696, 45764, 10360, 55952, 42400, 22136, 9020,
                43844, 56984, 28624, 10724, 43468, 18068, 31868, 65008, 19896,
                4304,  49408, 37572, 752,   29008, 5152,  50644, 10920, 37432,
                54476, 8492,  20912, 12044, 45324, 61384, 31716, 17864, 22048,
                32980, 8616,  51184, 34488, 3124,  9772,  53252, 14784, 34000,
                57000, 40460, 45564, 35508, 56328, 31832, 44604, 23444, 57540,
                19924, 60152, 36440, 54616, 8988,  19140, 64480, 10812, 46128,
                15016, 25812, 2088,  63032, 17848, 7764,  60768, 33004, 17040,
                39284, 2268,  54976, 15384, 8116,  58384, 31900, 55300, 23576,
                65248, 3384,  58204, 32852, 18776, 35924, 21792, 44328, 17080,
                31224, 1540,  61648, 47920, 52220, 2396,  63924, 41168, 4728,
                47836, 37176,
            },
            {
                65040, 6284,  22752, 9828,  53632, 30296, 58656, 36984, 24788,
                47112, 39856, 4844,  52744, 42300, 28576, 38684, 53736, 21252,
                40400, 30728, 57096, 50480, 33344, 55096, 36256, 62396, 32240,
                7100,  57216, 29504, 33260, 14436, 62972, 3612,  30416, 48512,
                34880, 13984, 39712, 63556, 36712, 54744, 13596, 40180, 51960,
                60300, 23452, 33052, 46964, 57660, 25052, 59900, 22632, 1508,
                43516, 25388, 63932, 29568, 56304, 6500,  42828, 3940,  58584,
                25840, 32,    15940, 64792, 22900, 41216, 28728, 63612, 1364,
                18996, 8336,  29544, 6092,  20452, 39724, 15040, 28916, 52176,
                32732, 11108, 42112, 30408, 3996,  39368, 32404, 48,    60636,
                50684, 31296, 53936, 44980, 56544, 29132, 50596, 11272, 27060,
                65048, 45848, 20212, 42492, 28656, 37648, 18864, 11644, 34404,
                38872, 13768, 26020, 10224, 46996, 300,   28964, 62136, 6204,
                37960, 26388, 7900,  56556, 12148, 27460, 23420, 17908, 61012,
                27788, 732,
            },
            {
                16016, 35036, 57324, 13776, 49860, 18048, 7464,  20628, 51192,
                2836,  64484, 22448, 9004,  34464, 56704, 12704, 25844, 244,
                59064, 23104, 11596, 7176,  25220, 2940,  47004, 10008, 19984,
                41968, 61432, 17392, 53120, 36116, 24404, 38936, 12448, 54316,
                61020, 6148,  20068, 52616, 1492,  24288, 48160, 3076,  43864,
                27988, 10092, 41616, 18564, 13104, 35020, 45032, 15880, 32436,
                56960, 39828, 4068,  35244, 15664, 26588, 52912, 13420, 39960,
                49584, 43988, 30228, 54080, 45712, 36748, 5012,  48444, 39004,
                54956, 23600, 49804, 59412, 168,   53708, 63804, 5496,  38616,
                1196,  46520, 16924, 62500, 21780, 44736, 54096, 27984, 17468,
                42404, 9088,  5024,  23880, 14376, 36052, 42004, 22288, 57124,
                34744, 5144,  31404, 60704, 3024,  47596, 63292, 652,   51448,
                46276, 54108, 41696, 62804, 15388, 56452, 40536, 11152, 48832,
                54356, 42416, 22200, 16588, 31776, 38380, 51080, 8792,  33572,
                12468, 42348,
            },
            {
                29412, 48560, 24972, 38060, 43192, 27252, 33580, 61780, 14576,
                30920, 57360, 27684, 15560, 60996, 5480,  44964, 51020, 33208,
                15800, 44036, 52772, 64856, 39740, 43072, 17072, 27408, 49896,
                24112, 37284, 8768,  2680,  50120, 59392, 19748, 45156, 620,
                27168, 23488, 32648, 46408, 8640,  29620, 56800, 34068, 7028,
                58264, 14520, 64192, 51556, 6392,  39588, 9000,  49232, 63068,
                19396, 13524, 46576, 59388, 50516, 38596, 20004, 34272, 55380,
                10800, 61624, 20256, 6268,  12844, 59464, 16948, 25904, 10860,
                32816, 13820, 65108, 42544, 25332, 46708, 11800, 19060, 61460,
                24688, 56320, 7224,  26476, 51260, 14504, 59604, 6496,  37392,
                20984, 62368, 33796, 39512, 19892, 63812, 2708,  5904,  44184,
                14076, 49900, 10708, 23408, 56796, 13600, 43512, 25420, 20460,
                7132,  28264, 2492,  22536, 37580, 51608, 24796, 33480, 20172,
                13972, 63788, 34344, 44892, 60732, 6128,  59072, 46920, 55928,
                21652, 53748,
            },
            {
                7680,  60236, 2052,  62532, 5180,  55948, 45936, 39504, 10408,
                42980, 18604, 45584, 37848, 49140, 23400, 18268, 63872, 8140,
                48280, 3160,  35496, 29400, 20732, 12768, 56348, 63488, 1788,
                54252, 13228, 47344, 40948, 26808, 5536,  32472, 64236, 16968,
                41348, 50828, 59256, 15124, 42260, 60744, 11820, 21676, 17368,
                37768, 31540, 2580,  26612, 54892, 61132, 29404, 2880,  52648,
                27160, 6772,  31484, 22860, 9868,  1244,  48300, 60416, 23984,
                4620,  37416, 27208, 56896, 32664, 50780, 21756, 61536, 44816,
                52124, 2628,  37236, 16740, 9520,  33536, 36752, 50496, 42832,
                13692, 34436, 49324, 40816, 2212,  35668, 11360, 24496, 57048,
                48808, 13468, 55816, 46776, 10116, 52180, 31996, 59880, 18768,
                54688, 26312, 37296, 52136, 17532, 34884, 8676,  61024, 31528,
                57032, 17932, 59424, 44580, 30180, 4496,  64720, 7800,  58664,
                28636, 3792,  10168, 50160, 416,   19944, 40704, 25532, 4032,
                37688, 45852,
            },
            {
                10928, 40988, 20492, 30580, 16392, 11684, 23556, 1484,  59240,
                35416, 6696,  54836, 1024,  32424, 10856, 41236, 29724, 37412,
                60656, 27088, 17916, 57784, 4228,  49036, 31624, 38276, 5844,
                33964, 28488, 64024, 18464, 11544, 55808, 47824, 8232,  36840,
                57168, 10832, 3664,  35836, 18888, 26012, 38908, 62720, 46528,
                54372, 49064, 19820, 44664, 32764, 21804, 17416, 36836, 11472,
                41832, 62008, 54004, 17840, 43308, 65428, 28852, 7712,  17220,
                31504, 47584, 15364, 40576, 8964,  1080,  42572, 30876, 7148,
                58056, 27192, 21120, 47768, 30552, 58188, 22456, 3880,  28048,
                59440, 9244,  18684, 64724, 30044, 53024, 46368, 32984, 40532,
                3752,  29532, 25980, 764,   58464, 27960, 48548, 24648, 40152,
                8332,  62392, 46472, 4248,  41400, 28108, 54384, 38704, 49528,
                11340, 36928, 33692, 8524,  49092, 12908, 18080, 45836, 41320,
                52308, 37320, 24308, 54800, 29416, 35480, 11532, 15416, 31464,
                63276, 18584,
            },
        },
        {
            {
                39756, 15164, 25976, 48764, 38400, 908,   47332, 59076, 21980,
                10716, 62720, 40576, 54652, 28592, 20880, 50676, 25644, 18696,
                60100, 24060, 34144, 63920, 47136, 17928, 32368, 27980, 62936,
                11420, 31588, 23384, 63492, 27612, 52612, 41420, 13992, 64896,
                27844, 16728, 58208, 1016,  34256, 7268,  56540, 19040, 25984,
                268,   41676, 18220, 31184, 22032, 26632, 15696, 65228, 46316,
                56984, 4748,  10288, 58836, 52244, 40644, 46604, 30344, 4856,
                16272, 24860, 47280, 28960, 6884,  62192, 27632, 38936, 34156,
                2716,  42840, 51412, 31352, 64876, 17268, 29488, 61252, 2856,
                26308, 39820, 9364,  28780, 13168, 62772, 56376, 4564,  50336,
                42276, 6548,  54064, 25304, 35008, 28236, 43088, 48624, 8612,
                64688, 51108, 11752, 20752, 17220, 37488, 65108, 22220, 44020,
                17624, 63240, 53060, 42004, 14836, 3020,  40196, 21096, 63476,
                55784, 41260, 18772, 59548, 28872, 34336, 57168, 15548, 64560,
                7456,  52104,
            },
            {
                33352, 60288, 9000,  55952, 21232, 12820, 40888, 27748, 55636,
                37064, 45584, 18476, 14080, 9124,  44464, 1908,  41300, 31444,
                52752, 3244,  42412, 11960, 52088, 1972,  13660, 38628, 49148,
                19536, 58368, 51152, 37992, 2128,  16848, 61004, 31848, 49904,
                43612, 37112, 24152, 49328, 30928, 22124, 42296, 50460, 32464,
                64664, 23620, 56992, 10088, 47696, 43920, 52796, 11200, 24424,
                41544, 28700, 37376, 31932, 15376, 23896, 60800, 34796, 43960,
                37840, 60048, 12684, 40200, 18392, 50552, 44468, 13020, 52400,
                18016, 26400, 37852, 12680, 47672, 8740,  34268, 19836, 11432,
                48748, 64300, 44952, 34712, 18436, 2384,  46696, 16556, 9248,
                58720, 31928, 13184, 44984, 16860, 63560, 13844, 21572, 33116,
                40536, 4548,  57880, 30504, 48704, 7924,  41572, 51832, 29124,
                33148, 47448, 12464, 59072, 27616, 56580, 49956, 9464,  33676,
                23736, 1388,  32608, 51840, 5480,  12860, 43184, 26008, 19560,
                46432, 57452,
            },
            {
                23156, 50436, 2492,  42460, 62520, 31908, 51528, 6028,  14964,
                49852, 31160, 3344,  51604, 64320, 32852, 60944, 7612,  13752,
                46660, 58124, 21896, 8408,  37284, 56404, 61736, 23880, 7448,
                948,   40980, 14816, 9612,  34160, 56892, 6496,  23716, 1584,
                21056, 9696,  55084, 64056, 17980, 10944, 61156, 3644,  15664,
                54608, 5188,  34860, 39368, 63168, 2388,  29944, 38664, 7044,
                54504, 19240, 63020, 48156, 6220,  56104, 776,   11156, 19904,
                54536, 49456, 3800,  33896, 54012, 24316, 824,   59956, 30004,
                62856, 9776,  56060, 20172, 59236, 4308,  43196, 56288, 38152,
                30760, 21204, 4728,  57940, 51928, 37348, 26600, 53536, 39752,
                23680, 20616, 61980, 3632,  56032, 31016, 50776, 656,   59868,
                18464, 46228, 35328, 2436,  53924, 25868, 15360, 68,    57356,
                6456,  24900, 37200, 5080,  31688, 18136, 44048, 6468,  52572,
                15988, 47856, 10756, 44280, 21108, 38372, 54216, 9120,  32324,
                892,   11968,
            },
            {
                44060, 18412, 35844, 27992, 4816,  16368, 25120, 65144, 42924,
                8500,  20336, 58132, 26636, 37340, 23064, 16320, 55980, 38652,
                26904, 17412, 50300, 30544, 44792, 20036, 33804, 43676, 53804,
                30676, 26528, 55228, 21340, 48644, 29372, 39372, 46204, 59088,
                13136, 30164, 41224, 6424,  44740, 53412, 39268, 28516, 36640,
                43764, 12644, 27800, 51552, 6180,  20504, 13956, 60776, 34800,
                50808, 1172,  12768, 43012, 20424, 29140, 39028, 64180, 26344,
                7460,  30020, 22208, 62800, 8620,  15752, 36124, 47872, 21728,
                40672, 6704,  45204, 35408, 52520, 27516, 23616, 63232, 14020,
                1276,  54792, 15716, 24828, 6276,  43356, 11728, 64220, 34036,
                1004,  48308, 36096, 10692, 40240, 23796, 6212,  37468, 54412,
                28100, 24080, 14488, 63320, 43288, 10576, 60304, 36144, 20016,
                62224, 40444, 51536, 21440, 62732, 35352, 24348, 38836, 60596,
                29088, 36756, 64828, 25732, 58320, 2108,  62248, 48028, 41068,
                60876, 29044,
            },
            {
                5772,  64084, 10360, 54916, 45784, 37456, 57640, 1448,  35052,
                23424, 61676, 40116, 6280,  11732, 42756, 48712, 29452, 4388,
                64552, 35712, 296,   62700, 25960, 4772,  9840,  59240, 16652,
                46556, 64992, 3732,  44112, 62348, 11984, 18264, 54760, 34792,
                50964, 61588, 3888,  26032, 35416, 14420, 23228, 8468,  49508,
                19868, 60540, 45912, 16672, 59348, 32628, 55612, 23240, 45892,
                16548, 27164, 57856, 35932, 8720,  53624, 45460, 16880, 33368,
                58780, 14688, 37080, 44744, 57364, 31644, 42480, 11124, 4112,
                58300, 32832, 23268, 260,   15092, 39720, 49708, 7116,  46024,
                36112, 41612, 62040, 47844, 32528, 59912, 19744, 29704, 14312,
                57288, 42740, 27480, 52756, 19380, 46096, 61232, 12340, 43956,
                9316,  49744, 6648,  38592, 21892, 32600, 46828, 27812, 49496,
                9104,  16500, 3184,  45748, 9820,  54364, 456,   57664, 12396,
                20372, 4512,  54924, 14772, 30920, 17452, 35260, 23532, 14184,
                53680, 37888,
            },
            {
                15600, 58148, 31172, 23608, 52552, 19752, 11320, 48100, 29384,
                53944, 12892, 33440, 45944, 56568, 868,   20200, 53440, 10152,
                45124, 12512, 40396, 55180, 15488, 48172, 29004, 40056, 22040,
                12604, 36324, 32644, 7996,  24704, 41932, 2688,  27016, 7424,
                15952, 38564, 19920, 48252, 62672, 1620,  46424, 55840, 63652,
                2520,  31648, 7920,  24816, 36732, 42052, 49120, 8584,  4068,
                63828, 33188, 22104, 49356, 62200, 24960, 3900,  51508, 42464,
                1784,  46844, 52744, 18916, 2676,  26160, 65004, 55056, 28256,
                16532, 49992, 63800, 29328, 60360, 10860, 32192, 18072, 26456,
                52892, 8324,  28608, 11180, 39452, 22728, 7928,  45496, 5364,
                50996, 17192, 8016,  65268, 2720,  33592, 26672, 56756, 17056,
                32000, 62064, 52704, 29360, 57208, 16868, 5832,  54876, 13160,
                43916, 34864, 58032, 29468, 13744, 48608, 16752, 27272, 46032,
                41868, 50532, 8292,  39960, 47004, 6672,  51092, 28252, 3188,
                21268, 49348,
            },
            {
                26104, 40592, 47412, 13548, 7372,  33512, 40312, 60544, 17168,
                44416, 2536,  52060, 18092, 30308, 60488, 35440, 24852, 58988,
                33260, 19656, 24076, 7408,  60432, 51236, 35012, 2620,  52732,
                6368,  59884, 49664, 15484, 58252, 51936, 36760, 64516, 22584,
                44976, 32460, 56292, 11112, 29628, 58576, 33152, 26416, 11740,
                39736, 22444, 56584, 48268, 1232,  11700, 17920, 29180, 39252,
                43556, 11316, 2584,  41008, 15152, 32076, 12260, 57480, 21380,
                10308, 63632, 27668, 11844, 39652, 49268, 6384,  19592, 38404,
                53136, 12544, 41684, 18664, 46508, 53884, 2344,  57280, 61172,
                22544, 33404, 18920, 56852, 532,   53720, 49192, 35852, 62500,
                25032, 31388, 38384, 58112, 13944, 49088, 7328,  21292, 35664,
                41732, 1180,  19172, 12124, 45188, 2992,  64404, 37752, 22604,
                31108, 52632, 26136, 65528, 42376, 37936, 33168, 63708, 3584,
                31440, 61248, 34532, 21504, 63212, 43580, 11428, 55688, 65532,
                8508,  34596,
            },
            {
                4468,  20540, 148,   56128, 42728, 63884, 3396,  21976, 8036,
                38232, 27692, 62880, 24380, 8756,  50308, 14644, 41188, 6428,
                47780, 52460, 30976, 43236, 37164, 18592, 11520, 62820, 42396,
                25772, 18700, 39180, 28220, 20676, 5384,  30948, 47776, 10364,
                60120, 24,    51908, 15168, 41716, 20784, 5408,  17528, 35268,
                52800, 42688, 15176, 65060, 27188, 52936, 57496, 61700, 25152,
                51492, 56156, 29664, 59732, 5460,  38352, 47876, 28388, 35380,
                40188, 24048, 5028,  34760, 61672, 14088, 23100, 46196, 34028,
                2232,  25684, 8416,  37036, 6092,  21668, 42652, 38552, 12572,
                3200,  50912, 45408, 14440, 64956, 27004, 16260, 40832, 3168,
                54772, 11368, 21676, 44644, 29132, 41120, 52236, 64012, 4164,
                48112, 58528, 26576, 40380, 34460, 51628, 25160, 41508, 1088,
                61412, 7216,  19292, 2392,  23548, 5720,  21036, 51360, 9256,
                18528, 24788, 12888, 864,   26492, 57252, 18960, 38872, 31456,
                46012, 59668,
            },
            {
                51968, 63244, 38004, 27212, 16100, 30200, 51020, 26232, 54988,
                59288, 14916, 4800,  47168, 38580, 64960, 2932,  28136, 16896,
                62184, 2000,  57216, 14292, 4172,  27832, 23192, 56880, 30520,
                47768, 388,   56056, 10092, 45540, 61668, 13908, 54192, 17960,
                42280, 28288, 23756, 7168,  37192, 54204, 44180, 59564, 48524,
                28668, 4404,  10316, 33520, 38840, 20840, 4952,  35604, 14380,
                19972, 7496,  45620, 17568, 54160, 22824, 65348, 18632, 7188,
                60000, 49064, 55512, 43808, 29404, 51920, 58908, 9648,  56704,
                44344, 62336, 48800, 55644, 34376, 65472, 27628, 48508, 16296,
                35520, 62892, 42408, 4456,  37592, 31916, 9324,  60332, 20272,
                33700, 47116, 60744, 72,    24004, 16616, 37648, 10016, 30372,
                14912, 23164, 54592, 8204,  60528, 20468, 10196, 58628, 17408,
                48072, 11152, 39516, 54992, 49916, 60376, 11832, 55764, 40688,
                58176, 44840, 53848, 48824, 29940, 36504, 4380,  24432, 42316,
                16948, 12392,
            },
            {
                43760, 32784, 9916,  49288, 59796, 5196,  45452, 12264, 32472,
                48744, 35688, 42428, 21664, 12296, 31972, 43972, 55040, 36844,
                21780, 26348, 9928,  46888, 65352, 54028, 44588, 8688,  37436,
                13988, 63992, 34708, 50768, 40984, 1736,  33648, 25800, 39504,
                4528,  57248, 34652, 65488, 49264, 25180, 9472,  14016, 660,
                62852, 19528, 45720, 60400, 7264,  44340, 32056, 47524, 28,
                64624, 37828, 33880, 26960, 49964, 9892,  44120, 644,   53152,
                15836, 30840, 20520, 8872,  17628, 444,   36608, 31728, 16124,
                5100,  20280, 30936, 14528, 24604, 11672, 4192,  30492, 59228,
                7756,  24664, 29412, 20528, 52032, 23588, 55908, 43820, 28332,
                14696, 6904,  53052, 34816, 5308,  56108, 59656, 25636, 45360,
                62416, 38324, 5088,  47120, 15392, 30580, 50088, 33484, 27124,
                56740, 45044, 31732, 14776, 35096, 44612, 29780, 25436, 36040,
                15108, 3076,  38440, 60212, 16340, 9884,  62032, 49704, 1184,
                54656, 29460,
            },
            {
                7272,  17768, 57564, 22892, 35216, 20120, 39096, 62136, 704,
                19052, 9780,  29832, 58072, 53324, 18552, 7064,  59384, 11924,
                49968, 42000, 33436, 38568, 20160, 31780, 2224,  16480, 60180,
                21060, 6864,  23832, 30016, 17300, 57924, 22200, 7912,  62492,
                49800, 13472, 44600, 17112, 2364,  30820, 61684, 40768, 22736,
                37756, 54784, 30264, 51024, 23872, 16712, 59376, 10560, 41596,
                54548, 23392, 12428, 61396, 2800,  36280, 58408, 32732, 41308,
                11980, 38016, 3420,  60784, 47688, 41440, 25476, 64456, 42724,
                28688, 59292, 39712, 952,   60868, 51116, 44004, 20180, 54248,
                40828, 47444, 11012, 57316, 6788,  46612, 12744, 1456,  50616,
                39540, 63100, 18432, 41836, 49860, 12224, 32424, 18768, 1876,
                51408, 11680, 27752, 36136, 63356, 3548,  44292, 6588,  13932,
                36560, 21908, 62028, 27512, 8096,  17596, 1032,  47636, 6984,
                62532, 28680, 22704, 5420,  42632, 53028, 33508, 14708, 58680,
                22628, 36708,
            },
            {
                50136, 25440, 2924,  46560, 13904, 8264,  53080, 24024, 43172,
                56344, 25556, 63952, 1932,  48980, 34940, 23408, 46460, 30244,
                5072,  63408, 15744, 58244, 7700,  50900, 40344, 47140, 33604,
                52372, 43884, 12164, 54448, 6128,  37604, 44904, 52652, 35980,
                19948, 31284, 8868,  58412, 39292, 19588, 50756, 33584, 57512,
                26212, 15808, 5544,  41200, 12792, 55788, 26516, 50476, 30080,
                15920, 46804, 6360,  42572, 20656, 29884, 14040, 25024, 46964,
                63488, 56820, 27048, 34204, 53868, 21516, 5928,  12896, 51172,
                22860, 10912, 53384, 45916, 18368, 37712, 57012, 9572,  33136,
                788,   17952, 60532, 38964, 33224, 63604, 17576, 35584, 58736,
                23056, 9700,  26024, 30040, 64636, 46528, 7156,  54336, 43156,
                21100, 31520, 57732, 18192, 42112, 22768, 55696, 39160, 64784,
                1868,  53128, 4688,  42044, 56512, 64392, 39908, 52960, 19736,
                33808, 49740, 11896, 32444, 46736, 20928, 6472,  41428, 27920,
                10600, 61224,
            },
            {
                39996, 62872, 29048, 55080, 41516, 65236, 31096, 16736, 36544,
                7112,  51164, 40088, 16248, 6188,  27192, 62424, 9176,  39204,
                19056, 53084, 1460,  28640, 23944, 12672, 56396, 24924, 4460,
                27556, 38804, 61208, 47660, 26964, 64412, 15192, 11076, 28076,
                1344,  54884, 23052, 47340, 27280, 12232, 3736,  45324, 7428,
                10788, 47128, 34952, 2048,  64316, 37324, 3992,  21152, 7864,
                59976, 26124, 51540, 56352, 39404, 62736, 3884,  50668, 19460,
                5496,  22340, 45100, 11392, 14960, 62660, 37896, 55452, 2020,
                47816, 35412, 7796,  26728, 32316, 5984,  15280, 23540, 63132,
                51800, 26608, 13940, 50268, 2560,  25632, 41920, 30220, 4924,
                45952, 56696, 37568, 1968,  15596, 22732, 39152, 35448, 60500,
                5900,  40100, 48544, 332,   52656, 12364, 28900, 19100, 46584,
                24840, 9956,  18328, 50704, 12568, 32052, 22548, 59060, 9932,
                43716, 55252, 65080, 17968, 57084, 25656, 63756, 37864, 47704,
                19372, 5132,
            },
            {
                15456, 44708, 11616, 37256, 1288,  27672, 48192, 4120,  60984,
                13776, 33020, 45424, 20812, 37536, 54424, 42968, 15256, 56828,
                24964, 35360, 43344, 48660, 61636, 36508, 17984, 64304, 10768,
                58140, 15824, 1920,  20056, 42008, 3248,  32392, 58612, 46876,
                40668, 63948, 36728, 5428,  62252, 53012, 56492, 28944, 17700,
                63276, 51776, 58692, 19852, 28436, 48116, 32996, 62460, 40304,
                35476, 1832,  31532, 17296, 11024, 45876, 54268, 28572, 9116,
                35324, 52196, 40780, 1408,  31408, 49680, 28172, 18964, 32552,
                61276, 17272, 56216, 41528, 64432, 48684, 28556, 40092, 45268,
                35796, 5556,  30740, 43980, 19828, 8788,  54376, 61752, 20580,
                12044, 33256, 51752, 44512, 9080,  58896, 28500, 13676, 26620,
                16852, 65396, 9652,  34516, 25912, 61520, 8492,  50396, 60016,
                34340, 30064, 38128, 47208, 26092, 3388,  37364, 15636, 27828,
                4540,  24580, 412,   38976, 8168,  30120, 12940, 1844,  55456,
                34780, 53248,
            },
            {
                3428,  21816, 33056, 51312, 19652, 58648, 10572, 44448, 54060,
                22636, 59036, 10880, 29292, 61356, 12620, 312,   32752, 50860,
                3808,  60428, 10460, 21428, 5784,  45224, 872,   30396, 41860,
                49060, 32564, 36016, 9548,  55996, 24124, 50376, 18536, 6544,
                25328, 15812, 10668, 32724, 42068, 15044, 35960, 21804, 42616,
                38108, 30940, 23416, 43964, 9200,  53588, 17456, 45244, 11252,
                57220, 19804, 63428, 5256,  34416, 22940, 15588, 43204, 57944,
                30516, 16768, 61832, 24436, 58768, 6936,  42076, 9620,  45284,
                39132, 3564,  24336, 11800, 20908, 1536,  58452, 12648, 3372,
                21576, 55420, 65096, 36880, 58264, 48196, 15120, 38508, 49476,
                27388, 16972, 3668,  60932, 19888, 55268, 49568, 4768,  52468,
                44076, 23672, 54808, 14828, 46016, 37832, 32216, 2944,  15572,
                41656, 55388, 58956, 21004, 62808, 7508,  45668, 51044, 61712,
                41412, 36272, 14388, 52548, 44476, 49512, 60344, 17068, 43836,
                24476, 30412,
            },
            {
                49084, 60132, 7980,  56900, 14852, 24692, 34824, 39356, 17900,
                27156, 5560,  47716, 3224,  50412, 40364, 22368, 65116, 28052,
                47160, 13448, 31088, 55144, 40152, 33176, 51852, 20524, 54212,
                7024,  23012, 63416, 52092, 30528, 13384, 60808, 34996, 43216,
                53404, 60248, 50056, 20592, 544,   24884, 48144, 5844,  59832,
                1416,  13592, 6840,  55568, 14812, 39632, 800,   24592, 52780,
                14200, 42948, 37736, 47744, 27424, 60600, 7468,  36872, 64940,
                2304,  47860, 10184, 37004, 46224, 16176, 56428, 25776, 63956,
                14304, 30136, 59444, 43684, 50956, 36216, 53852, 31264, 60824,
                47884, 16392, 7252,  23764, 11564, 28492, 33912, 700,   7908,
                53264, 64168, 39948, 24804, 35172, 31696, 41236, 884,   62164,
                32900, 7240,  29872, 58776, 20940, 5992,  56548, 43924, 23420,
                11964, 6792,  460,   14384, 43048, 33504, 57780, 11656, 30660,
                18496, 47252, 58448, 27444, 34392, 4700,  23260, 32648, 58420,
                9284,  63620,
            },
            {
                36288, 17744, 26072, 42048, 5504,  62804, 50180, 2556,  30824,
                64564, 41276, 55888, 35888, 24896, 57524, 17172, 44988, 8184,
                20636, 37872, 17492, 64036, 25724, 14600, 9768,  61108, 26364,
                13572, 46324, 17432, 40348, 45084, 4584,  38856, 9988,  21968,
                2036,  28868, 38420, 44412, 58892, 30584, 64888, 11652, 53444,
                26924, 49784, 33840, 63812, 25792, 59484, 31328, 6232,  49196,
                28936, 22492, 8616,  50792, 55572, 856,   40220, 24892, 12736,
                19080, 55596, 26940, 4124,  20560, 53056, 34412, 5116,  21752,
                52312, 48580, 5532,  33372, 15668, 8708,  25124, 18288, 41724,
                10484, 27340, 39556, 52396, 3976,  62640, 45636, 21760, 59684,
                42800, 30684, 13576, 48540, 6824,  11284, 17796, 46872, 21616,
                12144, 38728, 50672, 2400,  41340, 62796, 17600, 27592, 53856,
                64196, 49572, 26916, 35996, 53628, 17360, 23988, 1496,  55028,
                5688,  22316, 9720,  62100, 19908, 11492, 42372, 51620, 5640,
                13896, 40836,
            },
            {
                328,   55340, 47600, 37908, 29860, 45280, 21548, 8320,  52740,
                12676, 33956, 19768, 15028, 9048,  31344, 4616,  34912, 53424,
                41136, 58656, 6636,  51168, 3176,  46436, 57384, 39076, 34928,
                3016,  29080, 59452, 568,   21252, 27868, 65212, 49104, 31384,
                56700, 12700, 6972,  17976, 52028, 8856,  39780, 34284, 19628,
                45036, 40384, 21344, 4128,  36312, 43376, 19456, 56176, 34784,
                64832, 4304,  59604, 13036, 30812, 20408, 46712, 52724, 32100,
                50140, 38780, 42896, 33012, 64332, 11196, 39564, 59152, 164,
                36488, 26912, 19212, 62260, 38548, 46708, 63844, 6160,  34744,
                57408, 50636, 32544, 43316, 18384, 55536, 40616, 25116, 10784,
                36000, 5040,  22468, 58228, 43728, 62848, 25720, 56924, 35700,
                59596, 45312, 18624, 25584, 11748, 49020, 35792, 9600,  38712,
                20088, 31544, 40880, 60184, 9476,  29172, 49112, 39680, 35064,
                63456, 50600, 31740, 3012,  54496, 36540, 64408, 27088, 20472,
                46492, 28384,
            },
            {
                22912, 12456, 64820, 3568,  11300, 54472, 15544, 60828, 42568,
                23816, 732,   51316, 63216, 43104, 60388, 48492, 11620, 56476,
                1332,  26260, 33872, 42384, 22296, 29724, 17060, 5200,  44236,
                55552, 50096, 37008, 8452,  57156, 14920, 53840, 5476,  17472,
                45740, 34456, 63224, 26772, 55964, 13868, 23744, 3208,  57000,
                15632, 61812, 10492, 46444, 50992, 12236, 62708, 9460,  39156,
                16900, 45448, 25888, 41152, 17384, 63852, 10592, 58844, 3936,
                22236, 6872,  60708, 13756, 49468, 24100, 30348, 47204, 13260,
                42440, 57972, 10540, 55140, 2124,  27996, 51500, 22836, 44812,
                14780, 252,   20992, 59332, 13076, 30844, 5748,  15592, 56960,
                46728, 18608, 54068, 1292,  28804, 51564, 38464, 8544,  14908,
                27884, 4080,  63308, 34764, 52820, 30196, 1036,  60356, 47648,
                3896,  16864, 46188, 22488, 3092,  65328, 44336, 19984, 13780,
                26716, 42880, 16388, 39972, 46072, 14524, 1136,  39376, 57112,
                34280, 53076,
            },
            {
                16264, 44256, 31944, 20360, 59456, 26908, 32748, 36808, 47420,
                58360, 28620, 45760, 6784,  26760, 38680, 18600, 29276, 23248,
                15524, 62356, 48224, 9368,  61460, 36864, 48820, 65024, 23504,
                19180, 11020, 62764, 24956, 33028, 46884, 35940, 26028, 42060,
                61628, 23232, 48560, 2648,  36424, 46592, 42852, 61000, 28848,
                7340,  31988, 54968, 17736, 29052, 2508,  23068, 27324, 47720,
                1296,  31976, 54932, 36376, 3380,  43688, 33932, 27224, 15848,
                44676, 57432, 29492, 17800, 2060,  43488, 7608,  18596, 63028,
                32676, 6692,  45512, 22188, 31120, 12912, 40308, 4360,  61268,
                29716, 9168,  64016, 35564, 2568,  48564, 37500, 64868, 50516,
                33084, 26800, 63384, 36784, 15316, 20248, 5312,  31428, 48672,
                53548, 39984, 9880,  56356, 21812, 15340, 44260, 24216, 33476,
                52972, 8020,  56804, 13096, 51400, 32960, 5148,  58552, 8120,
                46760, 57052, 6340,  25312, 61032, 29772, 49428, 18048, 10804,
                62392, 7500,
            },
            {
                49836, 58240, 35472, 8824,  41484, 49204, 1404,  18932, 5204,
                10252, 16940, 36520, 54380, 21532, 1720,  51076, 64288, 37124,
                44540, 30956, 13028, 19600, 54276, 156,   12160, 27824, 7764,
                41540, 31316, 16228, 52440, 42928, 11756, 19724, 58588, 3156,
                10368, 15320, 41040, 31852, 21300, 6252,  17232, 48708, 37676,
                51420, 272,   25008, 38596, 58476, 33456, 52444, 42564, 13644,
                61516, 20868, 10068, 51792, 23300, 56312, 7204,  39276, 62908,
                36220, 9812,  25020, 35132, 52084, 61708, 55096, 27400, 50684,
                15776, 24676, 39104, 52604, 17224, 60020, 55936, 19572, 37316,
                48896, 54452, 23868, 46040, 26504, 53880, 19960, 28924, 8148,
                3068,  12952, 41100, 9344,  47564, 55656, 41668, 64824, 2764,
                22412, 16996, 32412, 6236,  46796, 65056, 5020,  57820, 14036,
                28444, 63420, 36472, 25592, 38812, 16516, 28156, 52668, 23104,
                37724, 11872, 33924, 51748, 21360, 8692,  55500, 24652, 4336,
                42624, 30128,
            },
            {
                25896, 2076,  53712, 24092, 14540, 38584, 64144, 25196, 55008,
                40232, 61148, 32016, 11852, 59576, 14456, 41948, 9680,  5768,
                52664, 3540,  57980, 38824, 25176, 32204, 59388, 51348, 34312,
                57628, 47556, 5584,  38724, 2380,  63792, 7464,  50012, 29436,
                37328, 55068, 7824,  51228, 64216, 57864, 33300, 25504, 12376,
                21712, 41632, 65404, 8432,  48476, 15564, 60724, 7244,  55608,
                35160, 49644, 5236,  59316, 29116, 14148, 48528, 18780, 50884,
                1700,  53916, 46632, 40992, 4664,  22004, 37224, 9748,  40588,
                56452, 3072,  64260, 35436, 7820,  43752, 33736, 10792, 26648,
                16304, 41844, 6620,  39220, 16944, 9944,  61024, 42236, 23564,
                44900, 60484, 52452, 30608, 24020, 33796, 12368, 58444, 25420,
                44192, 61652, 50292, 42212, 26980, 37380, 19812, 40328, 10688,
                42836, 21500, 1216,  44720, 62044, 10908, 42132, 60740, 204,
                30292, 64256, 18736, 2104,  58824, 44648, 35864, 32156, 47480,
                59764, 37768,
            },
            {
                6296,  19296, 63360, 43284, 51868, 6888,  57448, 12880, 30360,
                20588, 48904, 2836,  23636, 45220, 28412, 57656, 32580, 47440,
                20596, 27184, 42860, 7384,  16528, 46004, 40208, 20364, 14824,
                1808,  61508, 21792, 28728, 55944, 23612, 31952, 40192, 53092,
                25080, 59428, 19668, 12072, 27544, 1280,  52992, 9336,  62680,
                56252, 34600, 14132, 43532, 22052, 4972,  37076, 19012, 30072,
                24448, 41912, 16384, 38660, 45920, 152,   61772, 24504, 31268,
                13144, 20648, 64980, 15580, 57108, 12372, 31048, 1352,  20604,
                46664, 29624, 12012, 49228, 25388, 968,   47344, 65468, 3400,
                57932, 33288, 12756, 62412, 51616, 34632, 4352,  14128, 55164,
                37916, 17644, 21164, 4868,  61476, 1652,  45292, 18760, 36244,
                10552, 29672, 116,   13500, 59344, 8972,  55032, 31204, 51468,
                61012, 34208, 50028, 6264,  55372, 19568, 48216, 35304, 15040,
                54588, 47808, 41884, 28552, 38748, 13068, 3316,  65436, 15380,
                20780, 12068,
            },
            {
                40528, 47036, 31288, 3932,  17716, 28664, 45640, 35004, 51068,
                42760, 7952,  65296, 38416, 53484, 3772,  35836, 24680, 16300,
                55808, 63744, 34068, 49636, 60880, 4752,  9992,  55036, 44136,
                26536, 36572, 12664, 49520, 17564, 45560, 60652, 12908, 16716,
                20,    46468, 30336, 43060, 36856, 15612, 40236, 44364, 5492,
                18556, 46332, 3368,  30596, 54388, 26880, 40940, 46788, 2024,
                62512, 11712, 27396, 65248, 19816, 33140, 42444, 8964,  57916,
                39944, 28112, 6024,  33048, 26152, 48384, 44088, 59336, 34856,
                61384, 5108,  19408, 54836, 41184, 14684, 21800, 30496, 52896,
                45128, 20316, 1212,  30296, 57204, 21508, 47496, 31716, 58868,
                356,   34220, 49260, 56720, 39312, 16044, 52120, 28244, 6804,
                54052, 57792, 38424, 23900, 18408, 35124, 48516, 2368,  25068,
                6760,  18488, 14752, 27856, 31516, 3444,  24524, 8628,  40664,
                20692, 4840,  9964,  61948, 16988, 54132, 23328, 41460, 28232,
                51580, 56124,
            },
            {
                62004, 27200, 13340, 60644, 33884, 21600, 9980,  2544,  62208,
                14928, 26720, 33696, 16704, 49972, 19896, 7480,  60452, 11100,
                37764, 1204,  14188, 21660, 52924, 30432, 36320, 24308, 62704,
                7172,  53192, 41984, 59440, 9016,  34948, 5152,  20956, 43968,
                64928, 34304, 5536,  61480, 50124, 23468, 60056, 29168, 32408,
                50464, 26156, 61132, 10312, 49936, 63912, 12688, 58888, 8816,
                52064, 44948, 32180, 7564,  54644, 11336, 36964, 52988, 4836,
                45664, 60028, 49996, 9356,  38544, 63388, 17092, 51556, 13784,
                24148, 43124, 32948, 58788, 28060, 62632, 50760, 39688, 8656,
                24984, 59928, 49908, 27268, 43648, 7696,  40008, 25268, 51008,
                9096,  28148, 11976, 43008, 25620, 8568,  62644, 48992, 40796,
                14492, 20304, 46476, 51712, 62748, 4680,  28764, 16088, 46116,
                54156, 39668, 57912, 46748, 37060, 59296, 51004, 62752, 32656,
                27232, 58308, 50836, 24588, 34484, 5760,  50092, 58472, 8344,
                1040,  33660,
            },
            {
                16980, 9596,  39004, 55116, 48272, 58796, 37480, 53552, 23376,
                5348,  46928, 57244, 9836,  29972, 43476, 62832, 40316, 51196,
                45428, 29596, 9216,  40868, 25972, 64632, 17908, 3636,  47364,
                29844, 19000, 32696, 772,   25548, 54660, 38072, 49212, 28448,
                9656,  22564, 56408, 18444, 3612,  10164, 55580, 20632, 2016,
                38536, 15796, 57860, 35376, 19840, 2736,  32436, 23516, 35696,
                15648, 56796, 3504,  22652, 50400, 60572, 26172, 15216, 21796,
                34188, 17312, 42676, 54748, 2412,  23096, 5416,  27912, 8172,
                53788, 38188, 16520, 6260,  10180, 36772, 4636,  18316, 13476,
                34844, 5280,  37672, 15912, 11384, 64596, 2212,  18460, 62052,
                15332, 45492, 65388, 6268,  30156, 37272, 21584, 32640, 2332,
                64308, 34324, 3816,  31572, 12032, 41148, 56772, 63848, 36388,
                22812, 11524, 64880, 8160,  21216, 13128, 43480, 17692, 3952,
                45848, 36200, 13840, 43448, 29540, 47032, 19576, 11724, 36696,
                45504, 22088,
            },
            {
                42496, 50708, 7128,  23748, 228,   43832, 15956, 27360, 39664,
                60164, 19256, 41476, 520,   55700, 13704, 26424, 4588,  18332,
                23304, 54620, 59700, 47804, 2724,  11996, 42200, 58512, 14004,
                39024, 10744, 65272, 45024, 15128, 30324, 62552, 2592,  57808,
                41172, 51576, 13884, 38980, 48052, 33820, 45568, 13376, 64528,
                53828, 8164,  23276, 40060, 44964, 17140, 56384, 49156, 28080,
                20688, 40560, 37412, 46960, 17404, 2468,  29764, 47616, 64124,
                1424,  24728, 11624, 30212, 19696, 58560, 41160, 31904, 47628,
                65016, 364,   50456, 45216, 23640, 55896, 32320, 60796, 53836,
                42104, 63700, 47604, 56028, 24300, 36384, 54176, 29588, 41700,
                35424, 23184, 53568, 19032, 60040, 47260, 12748, 59028, 24456,
                43424, 27180, 55984, 7984,  44996, 25740, 20748, 9416,  43660,
                472,   32808, 26604, 41636, 52888, 1312,  29188, 55712, 11236,
                64812, 22664, 7132,  57308, 716,   63996, 39744, 25540, 61316,
                29288, 53572,
            },
            {
                4812,  64936, 35708, 19008, 29580, 11868, 64344, 7840,  49484,
                31924, 11760, 36628, 63684, 21740, 34676, 48180, 31576, 58632,
                35104, 6332,  15552, 36700, 31404, 56120, 49764, 33876, 21464,
                50804, 55716, 4320,  22140, 39872, 52152, 11176, 24384, 16160,
                35764, 6660,  26408, 31224, 63804, 24516, 7040,  36612, 27696,
                42292, 47952, 4432,  29296, 53328, 7380,  38336, 43440, 576,
                64192, 13348, 58592, 24944, 34568, 43668, 56588, 39304, 8480,
                55448, 37088, 61176, 51980, 35524, 46000, 10884, 55748, 36292,
                21480, 11508, 29352, 61988, 19772, 48160, 1640,  44400, 28224,
                22404, 10236, 19112, 3304,  32928, 45600, 13240, 49108, 5488,
                57680, 3576,  39788, 32268, 1060,  55244, 17236, 5528,  53016,
                9624,  18028, 61584, 15472, 37168, 60628, 48860, 14240, 29948,
                55880, 49588, 4452,  15852, 34300, 61756, 37800, 25428, 49080,
                39352, 18616, 52356, 31680, 9672,  54944, 16012, 32904, 3112,
                56764, 13724,
            },
            {
                24848, 31600, 44656, 59592, 52164, 40632, 32956, 21032, 55240,
                2072,  28380, 51896, 25012, 44872, 8152,  54296, 2284,  12140,
                43132, 27520, 62596, 19740, 44536, 23188, 7364,  28180, 1320,
                60908, 26080, 35608, 47848, 58752, 7308,  19552, 46856, 32236,
                63348, 44808, 54104, 2324,  42512, 16204, 58164, 52040, 18376,
                10648, 33472, 59708, 13816, 63128, 26064, 11232, 61064, 6012,
                52748, 29876, 6752,  10524, 63568, 5968,  13088, 19584, 31632,
                49400, 44300, 14360, 6688,  26696, 62496, 15248, 3116,  18168,
                26300, 59716, 42028, 34476, 13124, 40176, 25880, 15756, 6812,
                58084, 30704, 40244, 52280, 59436, 21720, 8548,  60996, 26452,
                20004, 47584, 14432, 51052, 10340, 43644, 28748, 39232, 46704,
                35848, 50068, 40408, 23000, 53492, 2256,  33992, 5744,  39420,
                62340, 18800, 44784, 58976, 23624, 47828, 9276,  14948, 33724,
                1776,  59352, 41556, 15100, 36948, 21612, 44268, 7008,  48960,
                18848, 38180,
            },
            {
                47088, 2776,  15468, 10156, 25472, 57132, 3196,  46160, 17480,
                43028, 58668, 14584, 5228,  60300, 17756, 38460, 64976, 21352,
                50480, 39476, 384,   51732, 10568, 60108, 16284, 38040, 45592,
                9064,  42788, 17040, 12316, 28012, 33776, 42096, 55628, 4244,
                12132, 18712, 58872, 21460, 11324, 49360, 29688, 484,   61420,
                25144, 55516, 21148, 1300,  50828, 31364, 22232, 16568, 33252,
                45516, 19120, 48008, 51612, 41196, 27820, 52636, 61976, 23432,
                4280,  28348, 18648, 40012, 736,   33220, 50640, 43528, 57992,
                39548, 7104,  52640, 3516,  57148, 8796,  64816, 51160, 34132,
                46400, 476,   14932, 27440, 6364,  42576, 17336, 38692, 33448,
                11256, 63740, 27796, 36720, 24700, 63068, 33988, 57240, 22020,
                12240, 832,   28280, 6440,  30712, 19232, 51476, 59328, 27204,
                22304, 12128, 53648, 30552, 5860,  19268, 41284, 58004, 53820,
                6084,  28692, 24044, 46512, 60756, 27476, 51704, 63148, 41652,
                10544, 58672,
            },
            {
                53000, 21176, 63040, 48548, 6104,  36224, 13952, 62896, 38312,
                23472, 9412,  48532, 32532, 40204, 51088, 28484, 15292, 45756,
                8840,  57320, 24952, 33536, 4980,  40264, 53356, 63336, 19676,
                52380, 31536, 5660,  63968, 49960, 1472,  61388, 26128, 39176,
                50372, 28988, 37452, 7904,  62176, 34892, 40124, 5792,  44664,
                37972, 15384, 45824, 40736, 34840, 47352, 59084, 41768, 54856,
                36564, 26276, 60052, 32288, 21964, 17016, 464,   35276, 42012,
                10740, 57816, 65424, 47892, 54556, 24264, 20460, 9136,  30692,
                47216, 14228, 24340, 17588, 49660, 31228, 20912, 38104, 11640,
                54824, 61656, 36996, 49308, 62628, 31008, 55148, 50840, 696,
                56524, 42424, 7760,  53976, 3216,  20232, 14896, 4580,  60480,
                31620, 54776, 65076, 47348, 57772, 11288, 42344, 16232, 46340,
                2816,  36184, 8724,  39064, 62784, 51904, 2904,  30860, 21688,
                44876, 61940, 11548, 49884, 2404,  5016,  13176, 30648, 22776,
                35280, 28020,
            },
            {
                8076,  33888, 38924, 28536, 42792, 22576, 50608, 30160, 4820,
                61216, 35312, 54940, 20564, 1540,  10988, 24112, 34092, 60816,
                30672, 17892, 12956, 64420, 47224, 30192, 25536, 3308,  35256,
                14160, 56784, 23884, 38208, 53956, 21632, 15528, 9112,  59460,
                22676, 760,   52680, 46256, 26644, 55824, 19280, 23136, 50628,
                7876,  30540, 65176, 5944,  18884, 9440,  2780,  12980, 23800,
                4496,  8960,  15244, 3300,  37860, 57212, 45048, 59608, 15136,
                51072, 32344, 21684, 8196,  12692, 59788, 38332, 63184, 4384,
                54372, 33092, 63768, 37048, 27432, 46148, 2424,  60312, 17784,
                24364, 43144, 20396, 11308, 25256, 3856,  14348, 23720, 45800,
                29656, 21776, 16608, 60128, 40040, 45508, 49324, 8748,  25544,
                41936, 18504, 13984, 43604, 38100, 25060, 63468, 8024,  31880,
                50008, 57888, 25308, 43364, 14028, 27536, 46092, 64100, 17276,
                38448, 8064,  33124, 19832, 35180, 57176, 39000, 54676, 16220,
                648,   61364,
            },
            {
                18132, 12332, 54356, 1144,  56152, 18684, 59264, 8396,  27076,
                46740, 16464, 41752, 29988, 63064, 44072, 56360, 6404,  3104,
                49160, 37520, 55532, 42556, 21492, 14800, 57788, 11224, 48832,
                28912, 61300, 41688, 18180, 4396,  44924, 36444, 48348, 31040,
                43516, 17108, 33360, 14672, 41580, 4260,  12812, 62980, 32944,
                57768, 11892, 26808, 53140, 37396, 57088, 28928, 51372, 64720,
                39808, 55916, 43704, 63156, 11476, 48620, 30408, 8648,  26336,
                38648, 1936,  45308, 35616, 42888, 29512, 16808, 49068, 27328,
                21848, 1104,  42716, 10408, 59040, 6204,  41064, 53288, 29040,
                4900,  8364,  32652, 57844, 46816, 40752, 64768, 36680, 7084,
                61616, 34904, 50324, 31416, 11936, 56608, 28208, 63872, 37784,
                51924, 3740,  33656, 8560,  21168, 1752,  35244, 54408, 40816,
                13536, 65440, 17992, 740,   34608, 22532, 10188, 35824, 13072,
                50664, 25920, 53580, 65260, 43388, 17748, 25668, 8624,  45612,
                51028, 42436,
            },
            {
                24376, 45896, 60620, 15948, 32208, 11332, 44760, 34116, 53488,
                12848, 3036,  25900, 7284,  58376, 13720, 36268, 52336, 42300,
                23212, 7636,  28364, 1992,  53832, 6680,  37236, 44372, 22512,
                7988,  128,   46824, 11340, 32420, 28600, 63536, 12808, 5284,
                55208, 65484, 10132, 59908, 23844, 31720, 47308, 53256, 16856,
                42700, 1860,  49096, 16144, 24140, 62148, 44452, 20264, 31604,
                48936, 18160, 28560, 34504, 51988, 24104, 4692,  20384, 53732,
                62956, 17316, 56668, 25608, 3604,  52712, 5828,  11160, 35728,
                45276, 60220, 19620, 51676, 15264, 22592, 33596, 13528, 48716,
                36132, 64284, 52488, 2152,  17880, 34016, 9832,  52188, 19400,
                13432, 2628,  46840, 5548,  23836, 35632, 1148,  16692, 44460,
                20732, 59048, 27020, 61728, 50588, 56036, 28792, 18364, 23244,
                6040,  28452, 45628, 53900, 57180, 49788, 60404, 3716,  55760,
                40524, 172,   15204, 29240, 6564,  48616, 59192, 32296, 64376,
                5852,  37260,
            },
            {
                31312, 4044,  49612, 26296, 40876, 64396, 3472,  39072, 21240,
                56832, 65360, 50004, 38000, 17512, 47108, 26556, 20012, 63732,
                16032, 59524, 45996, 35220, 18980, 61452, 31824, 51284, 65124,
                40172, 33704, 25924, 54756, 58624, 7692,  50612, 19516, 24768,
                35428, 39824, 6392,  49932, 56732, 2192,  38240, 8660,  28084,
                36400, 22288, 60904, 33268, 41444, 4376,  14712, 35752, 712,
                11772, 60444, 21512, 2012,  40544, 14744, 61228, 37056, 47172,
                6400,  29148, 50148, 14496, 63928, 33652, 58748, 41848, 55752,
                13740, 7736,  38540, 28612, 56012, 44520, 63524, 25940, 56952,
                45288, 15960, 39604, 27688, 55548, 21836, 59372, 28544, 43444,
                39764, 54952, 26628, 65168, 19148, 41740, 53728, 32036, 11140,
                5960,  48208, 36452, 16096, 45356, 12592, 4416,  47840, 61876,
                52008, 36960, 10780, 31216, 6724,  19520, 29508, 42612, 23592,
                31656, 58696, 46404, 37408, 10836, 22948, 40772, 12468, 20532,
                27256, 57468,
            },
            {
                63480, 21928, 9952,  35892, 6964,  51932, 24172, 48984, 17324,
                31772, 43348, 10772, 22936, 53620, 32384, 1084,  9920,  31256,
                38864, 11920, 25448, 50572, 9708,  40960, 26724, 2696,  16812,
                20216, 59896, 49368, 14892, 22120, 37608, 41132, 56788, 3280,
                47572, 28044, 21696, 17772, 29260, 43252, 64428, 20300, 59396,
                5412,  46360, 55876, 7484,  10972, 47968, 27472, 56388, 8232,
                38204, 45372, 53964, 7440,  26812, 56144, 43908, 31252, 12516,
                23016, 42208, 10464, 39784, 20344, 46480, 23600, 18164, 25844,
                64592, 31856, 47540, 4544,  35024, 9088,  996,   19016, 10580,
                4056,  23224, 30856, 13092, 44572, 6044,  48844, 1264,  24304,
                58088, 10260, 37496, 15076, 59500, 9480,  50160, 22524, 62216,
                29032, 56744, 208,   23308, 31156, 41704, 58968, 38960, 32728,
                1500,  21076, 60664, 41336, 14404, 38484, 47748, 8904,  17936,
                61836, 5260,  20788, 55380, 62504, 2096,  51296, 34884, 3528,
                47916, 13872,
            },
            {
                39844, 53132, 43692, 57264, 20516, 29736, 14052, 61328, 7640,
                36932, 136,   29120, 60292, 5064,  40972, 62012, 48876, 54004,
                3360,  56440, 32616, 64060, 15416, 48152, 58952, 12992, 55132,
                36692, 9300,  3980,  42620, 62912, 1256,  26700, 16068, 62324,
                53196, 13640, 45412, 61500, 35756, 11404, 15220, 24992, 50408,
                40076, 13380, 18808, 30448, 53700, 38632, 22108, 50440, 63060,
                25400, 15804, 30688, 47608, 64612, 10244, 17620, 1176,  54796,
                34056, 60336, 2840,  58144, 31464, 8596,  16,    51156, 40128,
                3480,  53316, 17000, 24688, 61808, 50260, 39396, 32064, 61124,
                41968, 51560, 59844, 9224,  62376, 36196, 15364, 31756, 63688,
                17636, 32768, 52620, 44264, 29952, 3940,  57340, 38760, 46392,
                13596, 40912, 54024, 10328, 64764, 6852,  25532, 14968, 9828,
                43560, 16552, 48972, 25736, 58848, 4572,  64312, 26944, 52524,
                44212, 12412, 34680, 25348, 42544, 16828, 28972, 61104, 44184,
                54852, 7568,
            },
            {
                33732, 28032, 1468,  17188, 62696, 46636, 2416,  41364, 58284,
                25628, 52316, 19564, 44480, 34968, 12836, 21748, 27972, 44252,
                17972, 41504, 5796,  21624, 608,   30384, 23804, 6216,  43584,
                28188, 53164, 31092, 19116, 34740, 52256, 10548, 44056, 31844,
                8288,  34176, 944,   54184, 4960,  49220, 55352, 31380, 140,
                34328, 62220, 26968, 44816, 63904, 1600,  58616, 17856, 33068,
                41496, 5288,  58192, 36528, 19728, 33528, 49716, 39608, 63648,
                18716, 51972, 25208, 36428, 48940, 15940, 62436, 35140, 29416,
                10080, 22392, 59172, 41560, 12436, 20804, 54844, 27120, 47724,
                34728, 20188, 12,    49376, 25528, 41156, 54680, 46364, 7856,
                49944, 3340,  21656, 6632,  48064, 26256, 34004, 15888, 7320,
                25204, 32604, 18416, 37892, 51280, 20116, 34868, 53508, 57256,
                27716, 63292, 3624,  33628, 22828, 54668, 35296, 15604, 2376,
                37928, 50240, 7848,  30804, 52956, 6488,  56780, 24608, 37804,
                15992, 19404,
            },
            {
                61924, 11096, 48596, 38576, 32660, 53436, 26792, 34604, 11304,
                47152, 15404, 63676, 8088,  55664, 16580, 58640, 36828, 7488,
                24508, 60140, 35644, 52772, 44828, 39336, 56532, 33964, 47360,
                22596, 64028, 12212, 45604, 5616,  29640, 60636, 22996, 38196,
                20400, 58316, 41312, 26488, 22464, 39692, 7716,  44232, 57836,
                10280, 52576, 3944,  23780, 16352, 35164, 12552, 46008, 3496,
                52940, 23140, 13304, 43000, 2684,  24296, 57620, 5704,  27804,
                8860,  44492, 13664, 5600,  55484, 27728, 44396, 12932, 60608,
                46028, 36820, 49460, 1780,  29864, 43848, 5392,  16436, 13284,
                7676,  57368, 38020, 16680, 33320, 4852,  19820, 11272, 38684,
                28024, 42660, 63096, 36340, 12484, 61116, 20488, 1440,  63544,
                49856, 59320, 3692,  44320, 28352, 47312, 61052, 2336,  22752,
                45808, 38060, 52980, 8540,  42964, 11916, 46076, 31892, 58328,
                24256, 19068, 63936, 47336, 13936, 39512, 20028, 10284, 396,
                30876, 50656,
            },
            {
                3892,  25088, 58416, 13508, 5628,  9504,  44960, 20136, 55184,
                4180,  38848, 32992, 49872, 25040, 46408, 4356,  51944, 65040,
                14308, 47468, 29008, 12632, 62188, 17156, 9212,  61200, 3476,
                15688, 37868, 58052, 24716, 49036, 57036, 13480, 47048, 2632,
                49808, 16688, 10040, 62688, 32452, 18516, 60864, 28304, 16620,
                21472, 42088, 37024, 49720, 6748,  42944, 31320, 8908,  28584,
                59828, 48408, 9716,  62312, 51588, 29552, 45868, 14576, 21852,
                37656, 48120, 30532, 65112, 21544, 38772, 52852, 6352,  20440,
                56632, 15724, 7936,  33944, 65392, 57588, 37132, 52132, 63696,
                29376, 24796, 54136, 42932, 65072, 28904, 57704, 23356, 61068,
                14176, 56792, 24800, 18284, 55092, 41128, 45804, 53348, 36172,
                42904, 21784, 8752,  62776, 13204, 4992,  17476, 41900, 31484,
                7048,  13292, 19164, 29792, 50424, 612,   20972, 62244, 9452,
                41328, 55892, 27732, 1140,  60088, 33072, 49616, 64972, 46548,
                59052, 41368,
            },
            {
                55100, 36148, 18744, 42940, 23296, 60172, 15916, 64272, 30260,
                23792, 59680, 12328, 28428, 2308,  39296, 30588, 10980, 33920,
                20860, 3088,  55900, 7872,  25640, 37188, 51640, 20512, 41416,
                50132, 8124,  2644,  40492, 16656, 36360, 7280,  27348, 55048,
                64920, 29900, 44652, 51740, 13520, 47976, 36052, 4600,  47060,
                65252, 32340, 14460, 56508, 61392, 19928, 54768, 64748, 40136,
                18836, 34876, 27436, 39168, 16764, 7880,  59992, 41916, 53272,
                62272, 764,   16932, 41760, 11108, 2248,  24544, 32252, 41092,
                2664,  28368, 53816, 25672, 18232, 10200, 23444, 1396,  40896,
                45728, 3592,  10112, 22192, 13412, 2288,  44188, 52100, 31376,
                1492,  34216, 8400,  49048, 3108,  31616, 10356, 27848, 17684,
                12120, 30484, 55960, 34232, 24708, 52716, 37352, 10952, 49576,
                54868, 59528, 35512, 64756, 15880, 56160, 39228, 28524, 48900,
                4492,  15088, 36660, 10628, 44016, 23088, 4724,  35748, 26584,
                12868, 21528,
            },
            {
                8440,  45180, 64772, 30548, 56276, 35572, 39648, 1564,  48320,
                43228, 6556,  20896, 41996, 57672, 18468, 62320, 43324, 27136,
                50696, 38528, 42600, 19276, 48588, 1852,  28648, 11904, 31752,
                26164, 59636, 33424, 52844, 21524, 63504, 42864, 18576, 39980,
                4704,  24132, 37244, 6772,  1884,  56008, 23688, 11540, 53464,
                25848, 1480,  9060,  29104, 39140, 26508, 48184, 22680, 13848,
                280,   55872, 4932,  21756, 54108, 35876, 11988, 31400, 4888,
                34472, 24720, 56876, 51432, 33280, 61288, 57960, 17604, 50452,
                63380, 43416, 12724, 39164, 45596, 48640, 32476, 59068, 19540,
                35936, 61492, 31136, 47024, 60208, 50432, 35768, 6196,  17028,
                46504, 53860, 40004, 60580, 14924, 22708, 64676, 5300,  58404,
                39456, 2772,  45548, 15928, 40964, 58092, 29652, 64024, 20236,
                26292, 1464,  41240, 23488, 45164, 26096, 6976,  13388, 53472,
                34000, 21844, 46220, 57272, 17944, 29632, 54224, 15424, 43200,
                5400,  29524,
            },
            {
                52832, 14784, 916,   50204, 4644,  27392, 51700, 8316,  18212,
                53636, 36068, 62728, 51820, 9268,  23084, 54220, 15252, 1064,
                58120, 10076, 30848, 64120, 33400, 58396, 43992, 62576, 53864,
                17804, 43308, 13820, 29204, 9940,  336,   32520, 51864, 11264,
                34684, 14620, 59688, 20484, 41640, 64064, 29636, 43540, 37580,
                19340, 50816, 58904, 46656, 11876, 2428,  5892,  36684, 51248,
                44736, 60980, 31236, 43452, 65428, 1392,  25748, 49640, 19988,
                58604, 9272,  28540, 45796, 19364, 7416,  47304, 26836, 10456,
                34640, 22672, 59380, 3460,  55652, 6308,  14876, 27936, 50040,
                11880, 17204, 53116, 38768, 7648,  19060, 25956, 40708, 64232,
                21112, 11676, 26552, 29536, 45032, 37620, 50488, 34668, 24260,
                52364, 61344, 26860, 49436, 616,   22380, 7832,  44092, 14448,
                33848, 48056, 6072,  10404, 58256, 37128, 61332, 18732, 42224,
                65376, 30960, 51872, 3688,  63300, 41176, 9180,  50580, 57464,
                63192, 38888,
            },
            {
                48344, 26220, 34072, 10796, 19980, 42148, 13228, 32776, 58900,
                26592, 14508, 804,   33772, 47328, 37596, 6148,  48600, 36004,
                61688, 23476, 16108, 53452, 5908,  14256, 22416, 6808,  37564,
                1508,  65240, 46300, 56544, 25652, 60296, 44956, 23208, 61680,
                55912, 48728, 27788, 53928, 33040, 15800, 8224,  57584, 3524,
                61264, 41072, 34564, 22872, 18128, 53172, 59168, 32976, 16592,
                25916, 10984, 49280, 13544, 19228, 46700, 40068, 61524, 15440,
                43748, 38612, 13064, 3392,  36308, 40532, 14156, 54580, 37920,
                5916,  16288, 49548, 29336, 20308, 35480, 63036, 43544, 8192,
                56648, 25828, 5332,  33500, 14976, 56308, 29968, 9568,  57996,
                36804, 51120, 4288,  57332, 19036, 316,   55816, 9192,  47384,
                14500, 7404,  19432, 54936, 10588, 35904, 60700, 3620,  56020,
                39188, 61968, 18012, 31660, 51320, 3352,  29980, 47488, 1716,
                25184, 7720,  39444, 12728, 25780, 34700, 1504,  19436, 24064,
                32400, 17332,
            },
            {
                60316, 22452, 39840, 62864, 45652, 54728, 61596, 21496, 38388,
                10336, 45884, 29616, 17240, 59424, 12852, 31840, 25572, 18692,
                44892, 4908,  39532, 46344, 24988, 35884, 56184, 47756, 10896,
                23812, 34808, 4652,  19472, 49452, 37472, 4100,  15712, 30600,
                8144,  1100,  45684, 10680, 24980, 39196, 49236, 22028, 13456,
                30916, 16168, 5432,  44332, 63432, 30564, 41960, 8716,  63092,
                38480, 6376,  23784, 34260, 57812, 28856, 9976,  54920, 3128,
                23724, 48488, 64448, 55248, 22408, 62760, 30268, 1680,  60960,
                46252, 31624, 64240, 9352,  42204, 24616, 51848, 2752,  21692,
                39464, 64480, 44444, 23080, 49488, 42020, 61788, 900,   24428,
                15420, 43620, 7556,  33816, 62256, 13112, 41888, 20544, 29084,
                43788, 33184, 37876, 64488, 30828, 47244, 18296, 51716, 23892,
                28532, 12184, 53352, 42472, 20720, 14868, 34956, 55560, 11144,
                58484, 16380, 60716, 20976, 48188, 55680, 61436, 37836, 47064,
                11220, 3408,
            },
            {
                55220, 7548,  58336, 29604, 16644, 6532,  25432, 2528,  49580,
                55620, 64660, 40756, 24772, 4300,  42848, 63196, 56340, 8940,
                52908, 29420, 12288, 61008, 2956,  41828, 18972, 30620, 51080,
                40032, 55272, 8576,  31192, 41052, 12968, 58220, 47792, 20156,
                42264, 38076, 17584, 62988, 51136, 5864,  61804, 34848, 46044,
                54664, 8604,  27420, 56040, 36448, 14568, 47076, 21452, 28548,
                52528, 58540, 41500, 2616,  51816, 7256,  17244, 33044, 36668,
                52328, 6740,  31260, 16452, 26268, 49988, 8956,  44304, 24108,
                19204, 52812, 288,   37428, 57372, 13452, 60368, 30124, 34528,
                47960, 540,   29092, 59536, 4144,  11048, 34736, 45688, 49032,
                32232, 63328, 22964, 39040, 48372, 25468, 32024, 59212, 2272,
                62128, 23120, 4672,  42664, 13108, 25560, 40728, 32700, 9144,
                46420, 5004,  57444, 25376, 63864, 8804,  40248, 22724, 43724,
                49756, 36780, 28344, 42684, 6152,  31684, 14116, 27524, 7028,
                42828, 35488,
            },
            {
                20092, 41728, 13604, 2132,  37612, 48000, 35248, 30216, 43756,
                4860,  19532, 7868,  50584, 54764, 20856, 28396, 3384,  40548,
                34168, 21644, 57136, 48964, 26988, 9292,  64280, 15348, 58832,
                27724, 20552, 61120, 16192, 51744, 7100,  28308, 35184, 64556,
                25768, 57484, 31672, 3436,  43936, 19396, 27908, 728,   38468,
                24324, 64652, 48536, 1380,  10752, 25228, 5268,  56928, 1124,
                19500, 45856, 15892, 62556, 37728, 21476, 42856, 63272, 26876,
                11556, 20624, 59932, 42500, 5972,  53476, 35384, 15556, 57044,
                40680, 11528, 26952, 47940, 17420, 4196,  40472, 9960,  16460,
                55600, 13276, 19580, 37492, 54328, 27240, 20716, 13928, 55136,
                3252,  18652, 53556, 10208, 16576, 52624, 6444,  38092, 11484,
                54436, 48284, 17120, 57532, 51036, 2864,  59008, 15684, 65276,
                21208, 34396, 37700, 496,   45676, 49364, 59568, 27552, 5304,
                19664, 33232, 92,    51056, 10776, 45008, 22252, 58184, 51668,
                62152, 28596,
            },
            {
                46328, 31968, 50524, 53252, 23168, 57092, 11836, 62600, 16040,
                23776, 36824, 32120, 61180, 10992, 38012, 45532, 14224, 50192,
                65508, 96,    17372, 37220, 52484, 33108, 45340, 1128,  6172,
                44196, 12108, 47356, 36024, 63780, 24624, 54864, 10652, 2576,
                52964, 13320, 22136, 35956, 54732, 14452, 58920, 10372, 51600,
                17840, 42876, 32968, 20192, 52384, 61696, 39324, 49624, 13048,
                35736, 30420, 10228, 26368, 31792, 56116, 47936, 1240,  58236,
                45624, 39792, 34664, 452,   46892, 12092, 21212, 60092, 28772,
                5320,  34008, 61660, 22460, 32048, 53912, 46080, 26816, 50772,
                62588, 32632, 7944,  46524, 16236, 51480, 65152, 39264, 6356,
                29268, 41744, 60364, 27740, 3720,  43120, 65464, 18228, 50576,
                27196, 35596, 9296,  29392, 21328, 7292,  37108, 53968, 1768,
                41620, 50732, 17104, 29444, 12876, 33356, 3752,  15660, 64372,
                54628, 13148, 57476, 62068, 18240, 53456, 2456,  40024, 16512,
                12492, 640,
            },
            {
                63656, 17484, 9456,  27144, 65512, 18896, 8080,  40104, 51336,
                59284, 12920, 47508, 26656, 1632,  33244, 17844, 58016, 24472,
                10724, 44160, 31180, 6988,  13252, 19940, 60104, 38052, 24200,
                33740, 53652, 3916,  22080, 992,   43548, 18968, 45900, 40272,
                6708,  49176, 60836, 8680,  29584, 40540, 46780, 33444, 56692,
                4984,  12424, 39992, 58392, 29060, 17256, 44124, 31872, 64796,
                55112, 3804,  60492, 50364, 4804,  12200, 24628, 14864, 30312,
                17988, 8852,  57016, 28248, 64032, 32524, 39016, 2760,  43328,
                50872, 14552, 55440, 43664, 7300,  65172, 20000, 37704, 5840,
                43240, 22172, 40336, 61284, 1612,  31412, 8488,  43736, 57284,
                24712, 12532, 36276, 46520, 58740, 34900, 29756, 22376, 45260,
                15172, 3484,  63628, 39272, 45836, 62088, 27148, 48500, 30464,
                13636, 58356, 62984, 7788,  55928, 23916, 52056, 38504, 31504,
                45132, 8260,  40620, 23280, 38120, 34888, 65092, 30872, 49116,
                25360, 54084,
            },
            {
                22312, 5604,  39392, 34152, 43032, 3796,  46464, 26804, 33468,
                300,   56460, 43432, 15640, 53180, 63784, 48124, 7444,  36396,
                27880, 58928, 39244, 62528, 22876, 54164, 29184, 49656, 17860,
                63424, 41404, 30084, 56664, 38896, 32492, 14444, 61904, 29136,
                33296, 16984, 42992, 24468, 65492, 4048,  21124, 25932, 62112,
                30612, 22580, 50512, 7472,  37416, 3652,  22056, 9564,  26084,
                42432, 23292, 39596, 18604, 43880, 64496, 38972, 53684, 6016,
                62124, 51424, 24204, 14260, 18988, 55720, 48772, 24576, 63236,
                8412,  20628, 39300, 2988,  25816, 35196, 12192, 57632, 1892,
                25028, 53004, 11512, 56776, 25688, 36624, 22472, 17652, 33680,
                62568, 50700, 812,   21396, 8112,  14024, 56860, 1220,  40052,
                60396, 32224, 24492, 55832, 13864, 33460, 10276, 19348, 44356,
                5580,  26340, 21956, 39068, 43944, 18256, 62120, 10064, 21084,
                48256, 25756, 30024, 15112, 3876,  26868, 9668,  19716, 4844,
                33832, 37640,
            },
            {
                43860, 59532, 49040, 56016, 12976, 30780, 60324, 54428, 21592,
                9912,  29440, 20260, 5404,  41084, 22480, 28976, 2452,  51824,
                20420, 15016, 49420, 46304, 2868,  40612, 5264,  9904,  57988,
                14180, 6924,  25992, 11104, 49752, 58352, 5176,  51048, 20820,
                59652, 38620, 248,   47656, 12312, 52768, 37584, 7352,  15368,
                44552, 2480,  63024, 14832, 45964, 60320, 34624, 53372, 16508,
                7068,  48412, 13972, 54552, 34908, 8444,  20084, 33768, 42216,
                48512, 36464, 2940,  41468, 44684, 4964,  9856,  16624, 31004,
                35776, 47580, 58724, 10852, 51976, 15708, 49316, 31196, 59952,
                17488, 35284, 29548, 4912,  44548, 50020, 59000, 2672,  47796,
                9132,  16028, 30724, 40260, 51780, 25056, 47904, 10500, 55128,
                7220,  42976, 52720, 18784, 104,   42248, 57292, 24120, 60688,
                36588, 9636,  54228, 48892, 2848,  35540, 28204, 42364, 1820,
                53168, 5976,  63596, 43280, 49828, 59612, 55000, 41104, 45480,
                61204, 10708,
            },
            {
                29936, 15428, 24736, 1284,  20796, 36968, 15296, 6284,  42400,
                38456, 64848, 50868, 35060, 59512, 9576,  39120, 62076, 42816,
                55236, 4408,  9228,  25356, 33864, 56204, 43588, 26756, 36164,
                46428, 52320, 19644, 60696, 16444, 27760, 9304,  37040, 24036,
                11068, 52248, 27372, 56956, 31804, 17676, 60408, 49596, 40932,
                55332, 35508, 24416, 54152, 26984, 11804, 49248, 1788,  62648,
                36776, 58980, 27900, 124,   46992, 29272, 59744, 2136,  27284,
                22100, 12708, 32160, 53808, 59572, 26216, 37084, 52424, 56636,
                1372,  27140, 17672, 29892, 61956, 44884, 22852, 39924, 9388,
                46236, 64580, 49180, 20240, 15240, 63136, 12188, 28300, 53760,
                38512, 44668, 56328, 64236, 18404, 32468, 61992, 36364, 26568,
                20388, 12176, 47504, 6156,  35872, 51144, 16416, 3440,  50036,
                32964, 46176, 15412, 31200, 60176, 6796,  56964, 14272, 34076,
                59220, 18500, 55460, 32804, 12152, 46532, 6956,  23352, 14172,
                56224, 3080,
            },
            {
                52456, 33128, 8340,  62096, 44880, 53032, 49700, 24528, 61576,
                17132, 2916,  46068, 25300, 13564, 55740, 31888, 16904, 11848,
                34264, 61016, 30240, 18620, 64780, 16196, 12588, 61548, 21652,
                360,   31012, 43952, 35016, 2208,  41168, 46988, 64352, 2812,
                55356, 15184, 5980,  20072, 35832, 45092, 1428,  11060, 27964,
                18348, 47612, 9492,  32764, 19684, 40648, 57228, 29252, 44832,
                20112, 32308, 9852,  22400, 57540, 16104, 50648, 44772, 9796,
                56244, 64844, 17064, 8220,  29928, 20220, 62196, 12008, 45556,
                22428, 64020, 37940, 54256, 41384, 972,   6520,  27836, 54840,
                14368, 2516,  41840, 8936,  39148, 32572, 6744,  42532, 19316,
                23536, 4832,  26900, 11684, 6436,  43420, 4340,  16432, 51276,
                30488, 38200, 60992, 28956, 22272, 64380, 30784, 40000, 12716,
                62472, 1628,  41872, 20252, 11928, 25192, 46484, 50864, 22808,
                39580, 10536, 36436, 984,   21764, 17376, 29680, 36028, 50080,
                27104, 18584,
            },
            {
                37944, 46864, 57676, 40464, 26596, 10256, 2292,  34944, 47436,
                12360, 57892, 31508, 7592,  18944, 47836, 4184,  25964, 45764,
                22332, 41012, 53500, 36476, 50624, 1704,  32152, 48148, 39724,
                54688, 8780,  65324, 48808, 22756, 54312, 18676, 31076, 43352,
                34560, 46412, 41304, 63716, 9076,  58440, 23692, 34212, 64252,
                4444,  59752, 38796, 420,   65188, 5440,  15260, 23512, 39212,
                4504,  50496, 61944, 42812, 38304, 4956,  25232, 61604, 18644,
                40140, 4232,  46152, 38436, 50344, 1812,  40988, 33156, 6616,
                42284, 13788, 4224,  9140,  20404, 33284, 51272, 63252, 37092,
                21704, 33636, 27404, 51436, 58152, 24908, 55076, 35372, 62056,
                13856, 59988, 33420, 47132, 54148, 37900, 23204, 57796, 45000,
                64156, 1744,  15788, 57928, 9888,  44724, 54468, 7596,  26844,
                22528, 56444, 28708, 63776, 52760, 37556, 17540, 65256, 4740,
                29112, 45948, 25084, 61468, 52148, 39320, 63124, 57760, 6540,
                42492, 64776,
            },
            {
                12300, 22600, 5344,  17292, 29804, 64500, 19084, 31868, 55364,
                27000, 21924, 37524, 53952, 42188, 63612, 36232, 50976, 58128,
                1132,  27500, 6668,  11164, 38988, 24220, 57828, 6132,  17648,
                34380, 25240, 15236, 5596,  38260, 61220, 13116, 25716, 7688,
                58096, 28844, 22908, 50680, 29920, 13920, 42804, 52540, 20292,
                31148, 12944, 51084, 28832, 42796, 47368, 33620, 52864, 8884,
                13600, 55400, 17364, 33984, 11720, 53540, 31592, 13700, 35656,
                52728, 28504, 58280, 22920, 61212, 14592, 48236, 28176, 18152,
                57748, 50312, 34480, 47028, 25136, 59312, 13180, 18540, 43744,
                5712,  61620, 54244, 17708, 3996,  21088, 45872, 80,    30204,
                52016, 36992, 1756,  20224, 59032, 28888, 13712, 2844,  34364,
                8924,  23888, 49652, 40808, 25832, 4448,  19488, 59080, 37444,
                51888, 17252, 5484,  33844, 8952,  43812, 796,   31996, 13016,
                60588, 50152, 16064, 8436,  43892, 26204, 3492,  10968, 32688,
                21348, 256,
            },
            {
                28056, 50316, 35212, 53920, 13316, 38132, 59016, 7432,  40924,
                4096,  44628, 10888, 60168, 620,   28116, 21052, 14748, 8820,
                64136, 48468, 15496, 60272, 46212, 19556, 52616, 29428, 62916,
                11884, 56148, 42164, 29108, 10376, 33016, 4072,  49824, 62768,
                16676, 11500, 2216,  38284, 5272,  55052, 26352, 39620, 7512,
                56968, 44176, 16212, 55624, 22268, 10812, 58296, 26708, 63900,
                45936, 29564, 2280,  24228, 65264, 7660,  41108, 47552, 3040,
                23968, 33228, 12100, 6052,  35388, 25216, 9288,  64600, 53520,
                24208, 30284, 60760, 16008, 55684, 28784, 48520, 10428, 57004,
                24136, 46848, 12584, 31276, 37212, 64108, 10584, 48364, 17200,
                8736,  41820, 50272, 25452, 8272,  40900, 62380, 49184, 19788,
                42160, 56488, 32144, 12948, 36596, 47164, 33564, 14792, 43520,
                10136, 47516, 40488, 24068, 49124, 58924, 27112, 41208, 55924,
                19692, 42304, 3236,  56576, 31784, 13620, 48060, 52944, 16668,
                45880, 59872,
            },
            {
                8948,  42184, 63076, 1624,  48004, 43484, 22924, 50384, 15052,
                63960, 52168, 29964, 16376, 34184, 6396,  53524, 39288, 44012,
                32512, 19924, 55528, 43116, 3672,  27784, 9376,  41644, 45332,
                3064,  50216, 20276, 59012, 52200, 44856, 56520, 39832, 21196,
                36652, 52948, 60832, 18832, 47632, 62420, 16788, 2056,  46852,
                23004, 36632, 26112, 6292,  61036, 18068, 37204, 1224,  41380,
                21372, 35308, 60200, 48840, 44220, 28132, 21008, 55392, 63572,
                8684,  49552, 44204, 19272, 52136, 56468, 43500, 4848,  38112,
                76,    10872, 40368, 7076,  37300, 2220,  41696, 35128, 30536,
                588,   39616, 8200,  58820, 43492, 14664, 26512, 39520, 56912,
                22972, 64932, 12856, 45264, 17460, 52824, 31936, 10808, 54912,
                26896, 5292,  18252, 53268, 63160, 2052,  50248, 29156, 65032,
                548,   35404, 61860, 13680, 3312,  54088, 21160, 9860,  37184,
                6848,  34544, 27504, 64460, 37904, 19244, 60224, 34896, 24488,
                39496, 55124,
            },
            {
                15048, 19168, 32084, 25512, 9496,  57068, 3412,  28508, 35716,
                9280,  19944, 39644, 49352, 23332, 45588, 60976, 11216, 25072,
                3132,  37668, 30036, 22772, 34440, 64368, 14072, 36236, 22356,
                59980, 26872, 37388, 1712,  17504, 23948, 14480, 28200, 912,
                47300, 31568, 25048, 44288, 32908, 8728,  35916, 50228, 59244,
                10612, 63672, 2756,  34252, 49164, 30468, 7708,  50224, 16344,
                53664, 6492,  39892, 19160, 14364, 57832, 1112,  37684, 17508,
                41832, 15072, 62484, 888,   39796, 30972, 16920, 34020, 21264,
                49404, 62868, 18864, 45488, 22396, 63836, 5024,  59840, 16544,
                53428, 26120, 48912, 22128, 2296,  50468, 33412, 60568, 4880,
                28424, 32352, 3852,  59756, 35032, 5696,  22688, 47292, 15304,
                39416, 59620, 45520, 8116,  21556, 60236, 24464, 6712,  55044,
                20964, 31308, 57056, 18548, 29812, 35928, 15616, 63140, 47888,
                23216, 53564, 11468, 46732, 22424, 7140,  41588, 2596,  63500,
                5820,  30768,
            },
            {
                58520, 3864,  46208, 60592, 21156, 33384, 16992, 61828, 45792,
                24392, 58180, 2500,  63176, 13668, 56348, 29608, 18428, 51940,
                57304, 62144, 12616, 8256,  53560, 48956, 58204, 4808,  32028,
                16328, 8328,  47460, 31524, 64248, 36300, 8504,  59792, 42456,
                6256,  55508, 14596, 3756,  57300, 21520, 12404, 28168, 32148,
                18588, 41320, 52420, 13912, 39672, 44700, 62788, 24968, 32172,
                58708, 10436, 27008, 3836,  52108, 33272, 10900, 30012, 59416,
                25516, 53744, 29264, 21972, 58972, 11472, 47568, 60072, 13492,
                44024, 26448, 54368, 32272, 14108, 52304, 26888, 20124, 44504,
                64708, 14340, 35340, 62132, 29076, 55676, 7584,  19252, 52532,
                44596, 15704, 54996, 39128, 27288, 63588, 1940,  36916, 29424,
                64836, 500,   34960, 30248, 43164, 10656, 39192, 17468, 41372,
                12088, 26036, 51264, 7656,  46756, 61044, 42140, 5296,  51328,
                30172, 60136, 17320, 552,   54616, 30340, 57232, 27292, 12884,
                51288, 36432,
            },
            {
                23684, 52900, 40036, 13812, 37136, 52048, 41988, 11704, 54908,
                31064, 6140,  43168, 26420, 8404,  37588, 1848,  42368, 34520,
                5716,  47068, 17400, 40184, 416,   25344, 18840, 46932, 54100,
                40516, 61928, 13464, 43392, 5964,  49284, 54260, 33516, 19096,
                65020, 10844, 40608, 28724, 51240, 42288, 64680, 39088, 53696,
                5436,  45780, 29072, 21416, 54964, 11396, 19892, 3676,  42548,
                13392, 46672, 55588, 36764, 63288, 22880, 45608, 50916, 6764,
                34448, 3628,  45812, 36568, 7516,  27012, 2328,  55192, 29368,
                6536,  35568, 2768,  58860, 9360,  47232, 38932, 7248,  49928,
                32756, 9872,  42064, 5588,  17520, 38028, 24320, 41536, 10916,
                36328, 21788, 48176, 9608,  20500, 50016, 57440, 44172, 8428,
                21188, 13684, 51656, 26088, 15768, 56680, 32924, 52404, 46128,
                59660, 4328,  43828, 38740, 22228, 11312, 25352, 33164, 1900,
                13736, 38816, 43320, 33780, 50296, 10468, 44584, 17760, 48440,
                20496, 43840,
            },
            {
                64364, 10516, 28000, 6876,  56180, 5112,  26828, 400,   38340,
                48352, 18372, 35228, 54400, 32020, 47732, 22084, 64732, 15276,
                28228, 23364, 50944, 60728, 31244, 42640, 10920, 37732, 28580,
                2164,  51140, 24936, 57388, 29020, 22048, 11688, 45484, 26040,
                49696, 22696, 34368, 61692, 17124, 6412,  23852, 184,   14764,
                25260, 62024, 8588,  57560, 1580,  27244, 59724, 51664, 37632,
                64476, 21960, 31220, 16604, 41232, 5692,  61440, 13852, 39576,
                20448, 57196, 12740, 49276, 65416, 42240, 23580, 38792, 61640,
                17440, 51220, 41180, 23876, 29732, 56164, 34136, 12180, 23448,
                4020,  55396, 20640, 51964, 45028, 12824, 64452, 47072, 3520,
                62308, 58056, 1192,  30152, 13524, 40788, 16664, 25480, 53124,
                58332, 46680, 38656, 62580, 4136,  47896, 27300, 2660,  22868,
                36752, 15700, 62900, 32304, 2588,  52672, 56872, 45572, 19092,
                64048, 25676, 6344,  59148, 14972, 24372, 36744, 60704, 7960,
                29480, 1152,
            },
            {
                33744, 17248, 49016, 62612, 30808, 19388, 49912, 65288, 22176,
                14296, 61528, 51364, 11084, 16648, 59760, 6952,  49728, 9752,
                38756, 43900, 4208,  35484, 55256, 15576, 59768, 6932,  63976,
                20556, 10512, 35056, 18628, 244,   41036, 62816, 3960,  16240,
                38588, 7576,  58288, 2300,  37020, 48240, 55120, 44240, 60032,
                35536, 49028, 16584, 37820, 33096, 47560, 15656, 34656, 8040,
                28292, 632,   50064, 8976,  25384, 48104, 18720, 27412, 44040,
                63972, 9056,  32356, 18020, 54456, 14676, 51692, 9028,  46172,
                21988, 11636, 64924, 16152, 5424,  43292, 17792, 63048, 57708,
                29356, 38356, 61096, 26624, 32116, 680,   57324, 29880, 14624,
                25764, 33180, 43036, 51960, 60968, 6304,  31692, 11692, 34488,
                3600,  24556, 7232,  19704, 58936, 36484, 13580, 64104, 8772,
                55724, 28812, 10412, 48968, 58568, 17228, 28284, 39988, 8600,
                55504, 47268, 29068, 20900, 40628, 65496, 1976,  32508, 54292,
                40952, 56824,
            },
            {
                4936,  38328, 22036, 42764, 46968, 15032, 57756, 34104, 8180,
                41480, 2804,  28216, 21140, 44580, 39804, 26004, 57688, 30988,
                53236, 62724, 19824, 8916,  27896, 22180, 49600, 33304, 24272,
                56004, 46108, 39388, 60472, 53400, 14736, 36796, 31940, 59468,
                28468, 53844, 46192, 20572, 11860, 27176, 32836, 18756, 9804,
                30800, 4088,  43108, 24096, 65100, 5124,  43932, 23132, 56868,
                18280, 44640, 61080, 57616, 35084, 3288,  54980, 37104, 232,
                52076, 24312, 41076, 28324, 5328,  35020, 19024, 31368, 3572,
                56760, 32884, 47928, 38176, 60856, 50660, 1576,  26192, 41532,
                46628, 2252,  15268, 48300, 8676,  54168, 35112, 20984, 50100,
                39868, 7728,  18204, 23752, 35588, 56656, 47752, 62824, 42732,
                17436, 30720, 50196, 41972, 9948,  53988, 30984, 20100, 49832,
                33712, 41864, 18876, 24612, 6996,  37616, 14136, 62116, 22112,
                35976, 3328,  51856, 9756,  56000, 5248,  46216, 22076, 11496,
                15356, 25952,
            },
            {
                59704, 52000, 12440, 2952,  35136, 9924,  24232, 44776, 30056,
                58792, 37332, 55972, 5880,  62432, 720,   35720, 19544, 12248,
                2080,  25456, 14032, 46020, 65196, 1792,  44752, 13608, 41660,
                16660, 3320,  32264, 7672,  26208, 48040, 56724, 9500,  50936,
                1328,  13732, 43220, 30908, 52284, 63152, 6944,  57024, 40848,
                51472, 20452, 58784, 12348, 53232, 30108, 9736,  40440, 53584,
                4716,  38944, 11744, 15224, 42936, 32092, 58804, 11452, 30524,
                16180, 47104, 61488, 1692,  44692, 59448, 25496, 62620, 37344,
                43136, 26564, 600,   10028, 28260, 21640, 36408, 14680, 53292,
                10320, 19064, 36128, 58744, 23528, 41204, 16536, 6320,  60328,
                53304, 11788, 65408, 45820, 4720,  27564, 19680, 2156,  37288,
                60240, 55920, 12524, 33960, 23576, 928,   44488, 39084, 6020,
                61512, 1376,  53812, 65300, 44908, 33940, 32,    50320, 42476,
                11764, 32112, 61280, 18396, 34592, 49072, 28124, 38916, 62400,
                50156, 45072,
            },
            {
                18816, 30224, 55316, 26540, 60736, 39964, 1552,  53336, 17352,
                12148, 46920, 24780, 33388, 50252, 14544, 55576, 46640, 42040,
                59324, 36600, 56456, 32300, 51836, 38148, 6416,  58468, 52464,
                29484, 62180, 50188, 12272, 43780, 30116, 19936, 23860, 41488,
                17940, 63764, 24784, 8744,  39912, 15820, 46516, 22588, 13008,
                63824, 28160, 1244,  39876, 17896, 49256, 60792, 13916, 32124,
                62756, 24452, 29836, 52600, 21288, 7544,  49668, 23220, 60460,
                5800,  35460, 20680, 56228, 10856, 39544, 48460, 13200, 7360,
                52288, 15060, 58160, 19308, 54316, 45096, 8532,  61540, 31356,
                22792, 65064, 43700, 4796,  12080, 63444, 27044, 45696, 2792,
                36516, 28808, 55384, 15780, 38892, 10940, 54240, 22900, 9072,
                45380, 26892, 4940,  65156, 47316, 17216, 60656, 25612, 15008,
                46376, 27364, 13040, 39592, 21044, 29984, 59740, 6088,  54116,
                24280, 15944, 44732, 38236, 26188, 13560, 53064, 17548, 6208,
                35420, 9196,
            },
            {
                36,    64948, 36888, 7784,  50800, 20132, 63856, 27208, 49312,
                7016,  63280, 18748, 42908, 9628,  30232, 22904, 5068,  33540,
                16560, 48804, 5656,  41272, 11504, 17964, 26048, 34916, 9844,
                4824,  37432, 22840, 18108, 65412, 2580,  6524,  61144, 47092,
                32704, 36240, 55744, 2860,  60860, 34272, 4228,  26252, 37400,
                6076,  47388, 55200, 34540, 7332,  21484, 26464, 1836,  45832,
                20244, 48848, 36480, 1572,  27304, 65204, 17616, 44972, 40656,
                53616, 26664, 13628, 50568, 32788, 22144, 2936,  55104, 29852,
                20608, 63940, 35808, 41028, 30640, 59044, 3240,  47624, 40216,
                6388,  50116, 33572, 29612, 52360, 37644, 49300, 31724, 18888,
                43400, 22436, 748,   32284, 50740, 42452, 63832, 29684, 52096,
                15336, 39728, 21136, 52860, 36912, 28656, 55412, 10924, 51728,
                22164, 36084, 4256,  48584, 56516, 11040, 46768, 19392, 27916,
                64668, 48248, 1028,  58564, 8104,  63640, 3272,  30836, 59100,
                24724, 42504,
            },
            {
                48044, 14588, 22896, 46240, 16600, 32820, 43160, 4656,  36380,
                22644, 31992, 3748,  53776, 38364, 64916, 27032, 51996, 61160,
                8460,  28816, 23972, 20760, 60616, 30300, 55492, 62744, 21332,
                47708, 27336, 55800, 41268, 52676, 33800, 37796, 54712, 11352,
                5164,  21772, 49620, 27840, 19344, 42628, 54464, 50536, 59852,
                33064, 15140, 24564, 45076, 62524, 37692, 51784, 58012, 35348,
                10624, 5668,  55424, 59776, 46800, 39432, 13288, 33344, 2504,
                9640,  63056, 38316, 6656,  28760, 63752, 17500, 41584, 46572,
                24168, 4036,  50112, 6720,  12452, 23912, 34492, 18560, 56504,
                13208, 26524, 57968, 15584, 1772,  21820, 9864,  62316, 13324,
                58840, 48884, 8696,  61296, 25100, 4016,  14316, 33684, 48680,
                112,   61748, 31768, 8372,  14064, 3488,  40884, 32496, 57172,
                42832, 60004, 31292, 16796, 7620,  25552, 41116, 14808, 35000,
                39184, 5324,  30568, 23108, 42768, 20392, 40224, 45616, 54720,
                20852, 32576,
            },
            {
                39856, 57488, 28916, 3256,  59200, 11004, 54048, 14252, 60392,
                56684, 41140, 13132, 59860, 16968, 2652,  44224, 11292, 19088,
                40156, 63664, 53960, 744,   48096, 43572, 3120,  40044, 14276,
                44560, 59736, 1260,  14956, 9684,  45724, 25708, 16136, 28952,
                58516, 44592, 12720, 37932, 57784, 10448, 30168, 708,   18140,
                43464, 9444,  57848, 28920, 11608, 3924,  30968, 16732, 43100,
                64512, 14988, 40920, 18672, 10036, 3704,  54524, 28480, 57652,
                18580, 48076, 42328, 15896, 58524, 52116, 8880,  34776, 57544,
                10704, 31964, 45716, 17136, 52568, 62564, 38948, 27764, 51760,
                108,   37268, 19976, 46336, 61340, 42820, 56596, 7032,  39380,
                26468, 34508, 56304, 20340, 46536, 36748, 59176, 7160,  41360,
                25288, 18340, 58400, 42636, 50416, 62092, 20712, 7748,  2084,
                18456, 9488,  51000, 63144, 35272, 53104, 61840, 3024,  58260,
                9848,  55836, 52496, 13004, 49844, 34120, 57104, 15512, 2188,
                11820, 62780,
            },
            {
                5812,  10564, 51724, 44272, 38064, 30304, 21488, 39524, 28636,
                9040,  46156, 25824, 48576, 21536, 35880, 58272, 49480, 31044,
                4264,  45376, 13760, 37100, 8892,  16056, 24636, 49888, 32684,
                19264, 8052,  35968, 30996, 58340, 21356, 63116, 51416, 39236,
                592,   64648, 17004, 6576,  46244, 23820, 14604, 65364, 39676,
                22008, 53276, 2896,  19416, 41976, 47900, 56072, 8348,  24872,
                28456, 51100, 31164, 23324, 34352, 61568, 24756, 37528, 50920,
                21832, 31548, 1168,  23712, 36648, 44940, 26360, 1984,  62204,
                14532, 38716, 59936, 26148, 42916, 4576,  10104, 15764, 63540,
                44608, 8928,  54196, 24916, 6068,  34084, 17532, 23992, 53972,
                3172,  16208, 40676, 12064, 30416, 18044, 54788, 21904, 11600,
                56088, 35168, 4168,  27508, 45272, 24240, 35500, 48880, 64700,
                38228, 23928, 28464, 42128, 1484,  22640, 31864, 50472, 43456,
                21436, 33716, 18032, 27040, 59888, 10728, 6604,  28752, 37760,
                50304, 26316,
            },
            {
                36008, 54396, 19108, 25508, 5224,  62212, 47772, 1888,  51244,
                19140, 35200, 224,   29520, 52324, 7724,  14076, 38820, 22336,
                57328, 34332, 25580, 59096, 31900, 63364, 53460, 5648,  11360,
                64544, 51524, 24328, 48376, 39888, 3556,  12564, 7532,  42912,
                23076, 33640, 52512, 31068, 61664, 49876, 35948, 8012,  47812,
                31116, 61792, 35072, 51180, 63724, 13876, 22748, 39968, 59112,
                284,   46320, 7668,  57096, 43800, 49372, 15460, 7176,  64152,
                12048, 44432, 60632, 49712, 5240,  12556, 19884, 40508, 49544,
                28496, 53208, 896,   33976, 20476, 57348, 49464, 32364, 23008,
                40748, 30176, 59940, 11824, 39700, 28820, 51308, 47040, 31240,
                64736, 44980, 5168,  62996, 52448, 1412,  43596, 28036, 64588,
                45932, 38500, 10480, 54252, 16928, 12116, 58024, 29824, 15312,
                53716, 47176, 4944,  14144, 58596, 38048, 17728, 8072,  26052,
                13716, 46048, 62236, 6476,  37152, 47020, 24600, 52808, 61196,
                43600, 17640,
            },
            {
                30512, 1588,  63720, 33664, 13408, 56052, 8380,  24284, 61256,
                15740, 54188, 63704, 11168, 42776, 32972, 62968, 26244, 1400,
                53108, 10508, 18564, 51032, 41748, 22024, 28392, 36440, 56988,
                39256, 29284, 4592,  62140, 17680, 27468, 49760, 35456, 19192,
                54440, 9320,  25876, 40288, 2228,  19760, 27948, 57032, 12344,
                25368, 6244,  16000, 38676, 26684, 32864, 4864,  52976, 34032,
                18008, 61236, 13440, 38032, 4608,  20416, 30436, 41032, 3140,
                34692, 27452, 9052,  56644, 29828, 64788, 54960, 32588, 6992,
                18156, 22516, 9032,  64744, 13728, 29016, 45456, 2928,  58600,
                18936, 5244,  35092, 49592, 64052, 14424, 4628,  37228, 11460,
                20724, 28240, 48448, 23460, 38452, 9588,  34588, 49216, 16016,
                2464,  20204, 30112, 63460, 5912,  39584, 3100,  25760, 43696,
                11208, 33132, 55640, 20824, 45644, 11900, 48312, 29668, 63376,
                51532, 4236,  40028, 31036, 1308,  65128, 18792, 33212, 22132,
                8000,  59472,
            },
            {
                41548, 47084, 21984, 39612, 50728, 17812, 36120, 41924, 32796,
                44456, 5764,  39172, 23456, 56076, 5008,  18268, 41980, 47668,
                15964, 65104, 44632, 6448,  2196,  12576, 46716, 17868, 60,
                44168, 21828, 12028, 54860, 45144, 32596, 56372, 60820, 29592,
                47652, 62228, 14212, 44836, 5452,  59404, 42272, 51892, 1696,
                45208, 58992, 49564, 10024, 1000,  60268, 20152, 45436, 11016,
                36688, 21768, 53784, 27320, 63564, 10556, 52480, 59412, 46312,
                54416, 17184, 39732, 20668, 35204, 15520, 47008, 24012, 60260,
                44392, 56100, 48288, 41060, 37320, 55224, 8044,  36060, 12784,
                53184, 47544, 16400, 1364,  42320, 22184, 58436, 52984, 392,
                60308, 35784, 7344,  14168, 59644, 25864, 61908, 5036,  31836,
                51956, 59924, 48360, 23148, 32936, 52276, 46608, 62812, 20064,
                368,   59628, 36264, 26280, 64224, 5512,  57416, 40804, 688,
                36532, 59128, 23560, 44036, 16184, 54008, 40948, 12636, 3988,
                48700, 14728,
            },
            {
                24820, 11644, 58912, 7540,  43948, 26480, 65400, 11668, 2828,
                26972, 59188, 31100, 19792, 45988, 60940, 28520, 54544, 8536,
                37508, 30752, 27312, 39308, 57192, 34584, 61536, 54352, 25908,
                60676, 33908, 15860, 41952, 8800,  1008,  23848, 14628, 6108,
                3028,  36800, 18612, 55868, 34424, 22952, 15464, 31960, 37276,
                18400, 41048, 29676, 23344, 46832, 55060, 41472, 29304, 64184,
                5508,  49012, 40720, 2564,  33264, 44680, 17876, 26176, 13836,
                22980, 61856, 3968,  53228, 43212, 556,   9984,  38140, 4776,
                35944, 12208, 30900, 3284,  24180, 17752, 61612, 26740, 42720,
                65332, 23984, 27864, 55464, 32220, 9164,  26712, 44000, 18324,
                41252, 54692, 32388, 50832, 17196, 41852, 55780, 12816, 23704,
                40800, 7236,  13976, 42340, 56940, 18236, 36676, 8552,  31492,
                41044, 50392, 16876, 8888,  43004, 32448, 24224, 54672, 20284,
                16684, 28576, 11792, 56628, 48988, 8900,  29708, 45752, 62360,
                34836, 55772,
            },
            {
                5136,  32620, 52408, 15932, 492,   30992, 46620, 20988, 57268,
                48692, 13224, 8228,  49952, 14992, 2356,  35636, 12416, 58104,
                23360, 3560,  52536, 20228, 49168, 15508, 7652,  40820, 10736,
                6372,  52192, 58276, 36552, 20576, 64988, 38516, 46272, 41800,
                58960, 27648, 11592, 48228, 64336, 10344, 53772, 4332,  63324,
                21580, 54360, 13244, 64856, 36168, 17580, 8268,  14724, 50908,
                25396, 31436, 16280, 58876, 23904, 56272, 36248, 684,   38444,
                7972,  47820, 32780, 11552, 25248, 59504, 51736, 29028, 62884,
                16472, 26852, 58344, 43824, 6240,  52072, 48756, 20892, 1988,
                33428, 9632,  39032, 60560, 19444, 46552, 63120, 33556, 14996,
                24540, 10144, 57596, 44544, 1880,  30744, 20356, 45564, 37460,
                57224, 26376, 35652, 9868,  4440,  28276, 60844, 14652, 55376,
                22992, 6080,  61776, 29100, 52128, 2260,  14632, 34920, 10056,
                46792, 53020, 7088,  21104, 34972, 26180, 57572, 1896,  38880,
                27532, 19848,
            },
            {
                64752, 42980, 27924, 35592, 59948, 54700, 5996,  38860, 52932,
                17416, 36944, 64484, 33900, 25336, 40332, 51464, 21068, 32568,
                43256, 59784, 8004,  62368, 24400, 29320, 44764, 32096, 19440,
                47432, 28048, 2608,  49136, 30460, 53120, 10396, 17348, 33052,
                21936, 52020, 38752, 30404, 7052,  25492, 39332, 46688, 27596,
                7916,  34060, 3192,  43640, 6064,  27264, 58224, 38192, 1436,
                55736, 43508, 8520,  47188, 13188, 6644,  49144, 65148, 30652,
                58652, 42108, 27960, 63316, 48872, 18300, 40784, 21364, 54036,
                2420,  50500, 20288, 62296, 14768, 33024, 11128, 39952, 57060,
                15124, 45384, 6408,  51352, 13268, 2572,  38272, 6756,  50128,
                62104, 4480,  26932, 21736, 37032, 6224,  53364, 8640,  62480,
                604,   18660, 65008, 45316, 50692, 22204, 1824,  48304, 44328,
                27600, 37952, 13248, 47052, 19020, 38868, 60556, 49980, 64908,
                41816, 32668, 61764, 38412, 4732,  63728, 14484, 22848, 51828,
                9728,  49792,
            },
            {
                2980,  57472, 9328,  18868, 48752, 23932, 14740, 9544,  29904,
                23256, 43272, 3124,  53708, 58664, 6700,  45104, 63584, 1024,
                48556, 14904, 34124, 10864, 38088, 3820,  63344, 50604, 23252,
                37548, 64000, 12964, 24408, 5012,  59580, 26948, 50852, 56816,
                8284,  62544, 132,   20208, 43020, 50348, 17560, 59904, 11716,
                56232, 50032, 16372, 61188, 51544, 31764, 48472, 22496, 61416,
                12020, 19860, 63312, 35392, 29776, 20932, 42516, 10044, 51064,
                19376, 4928,  14208, 36904, 3084,  30796, 7400,  13588, 46296,
                42080, 10224, 34660, 38640, 54512, 28644, 46448, 4996,  30508,
                52500, 62328, 21392, 36092, 29712, 56932, 23340, 53996, 28908,
                46808, 39884, 13488, 65344, 48164, 58944, 27904, 16316, 33340,
                47880, 29348, 54344, 15840, 32140, 40096, 57948, 34096, 10392,
                64604, 3776,  53800, 33788, 57048, 6868,  22116, 27528, 5756,
                2524,  24656, 13772, 50276, 43384, 19076, 32560, 42308, 59808,
                17356, 37532,
            },
            {
                14068, 22800, 39340, 62280, 4132,  41432, 63400, 34200, 58216,
                4620,  47200, 27584, 10760, 21628, 30736, 17020, 9800,  27128,
                18872, 40556, 55708, 46580, 17660, 57904, 13220, 1488,  54832,
                16360, 43468, 33100, 40936, 18824, 44164, 34724, 1872,  12864,
                25276, 43984, 15196, 54684, 60788, 32888, 2172,  35332, 23652,
                42196, 30560, 39092, 24752, 10996, 19220, 3880,  42884, 33616,
                28420, 39284, 51808, 1960,  60780, 55088, 27572, 15784, 34360,
                24808, 56836, 52580, 22196, 45172, 58532, 34344, 65420, 23440,
                31664, 60420, 7800,  25280, 340,   64360, 22760, 59124, 37264,
                17828, 25364, 42256, 4212,  48416, 16892, 42868, 11648, 20256,
                1160,  31640, 52792, 18512, 34404, 11240, 41056, 50428, 22572,
                12076, 43264, 3956,  24568, 61424, 12764, 6568,  25700, 17544,
                49692, 20568, 41624, 24932, 10732, 44824, 31152, 39860, 17884,
                45476, 55280, 30008, 59664, 596,   47264, 54600, 11536, 5672,
                30376, 45148,
            },
            {
                26288, 54080, 46276, 31608, 11768, 28408, 45160, 50236, 19200,
                60680, 39760, 16120, 62048, 36468, 48292, 55004, 38940, 60576,
                52224, 29908, 4668,  25808, 51380, 32800, 40372, 26768, 58940,
                5464,  9584,  61112, 55940, 7768,  14344, 63668, 47712, 39408,
                31584, 49172, 36156, 5120,  28476, 13336, 57080, 44712, 5632,
                62684, 19636, 1544,  57504, 34772, 54016, 60148, 9188,  15732,
                47596, 4220,  24548, 18224, 45404, 5572,  39852, 62488, 2448,
                46868, 39480, 8376,  16816, 55284, 10956, 49676, 1348,  37652,
                56284, 17072, 49392, 44748, 19156, 41804, 13512, 8312,  49004,
                1512,  10632, 54880, 32920, 59276, 7792,  64416, 34236, 57868,
                37384, 60124, 8792,  43404, 3376,  24792, 63552, 2472,  55152,
                39800, 59848, 8784,  37960, 52516, 19916, 42520, 54432, 36920,
                59104, 31028, 1596,  60204, 16276, 63180, 54284, 13100, 58508,
                35620, 20832, 10476, 16824, 27620, 8188,  36212, 24928, 62548,
                56048, 34052,
            },
            {
                8300,  924,   16740, 51548, 21280, 37356, 1340,  25836, 7744,
                12472, 31360, 50824, 440,   42332, 13300, 4116,  24212, 35088,
                7192,  12540, 64200, 22092, 42956, 7392,  20268, 35464, 45904,
                30616, 21444, 49816, 27332, 36664, 53940, 23584, 20436, 4476,
                57752, 17912, 65308, 23172, 41448, 8760,  21376, 52260, 15300,
                48424, 9560,  27928, 46976, 14096, 44860, 36940, 26392, 65120,
                53296, 57700, 11120, 37392, 32680, 13960, 53204, 22344, 57920,
                11628, 32088, 64092, 43300, 28828, 24404, 39976, 19596, 27660,
                5212,  12420, 30048, 57624, 5848,  52784, 33968, 27248, 61324,
                40424, 28940, 63680, 13792, 24332, 39568, 27108, 45860, 5336,
                15288, 22612, 49056, 29504, 56332, 14716, 45928, 31052, 6920,
                19660, 34580, 49320, 28164, 1524,  46984, 63556, 29324, 4412,
                14124, 46380, 8920,  50332, 28616, 36536, 216,   48136, 8828,
                51992, 63576, 41332, 37812, 57420, 21648, 51268, 40112, 2144,
                20376, 48648,
            },
            {
                64244, 58432, 34980, 5568,  61164, 56676, 15924, 53340, 42668,
                35316, 55692, 20184, 24904, 57428, 28980, 65232, 43720, 20764,
                56748, 45060, 36492, 380,   60748, 53932, 10488, 65384, 14520,
                52392, 38696, 524,   17176, 45536, 3448,  30092, 42084, 61824,
                7288,  27464, 10764, 51708, 46948, 38068, 63924, 26832, 32184,
                36716, 54892, 40120, 64004, 6736,  23552, 0,     40692, 20920,
                7004,  29956, 41520, 59392, 50328, 26100, 8768,  35828, 43928,
                18308, 26532, 4,     36032, 6508,  62404, 15408, 53320, 61560,
                47680, 40624, 22440, 62860, 36620, 15900, 45960, 56420, 20348,
                35156, 50796, 44008, 19952, 8,     52156, 9816,  18568, 50592,
                42036, 26200, 62840, 5780,  39488, 20916, 60372, 37824, 26520,
                56672, 13396, 62344, 21864, 10976, 35192, 16504, 8304,  44208,
                22456, 55932, 34536, 40296, 21744, 4872,  42712, 23924, 26996,
                32816, 3004,  6136,  48356, 31488, 65500, 15208, 45864, 28676,
                12624, 41552,
            },
            {
                38156, 28968, 43732, 24912, 47392, 8712,  32012, 65012, 22660,
                2820,  62712, 46264, 5856,  33612, 9024,  51772, 14796, 1764,
                32212, 50232, 16772, 27756, 13968, 30076, 48928, 2708,  42156,
                25000, 62676, 12284, 57120, 32456, 59800, 11248, 52204, 35744,
                13832, 45136, 33408, 55988, 4160,  30896, 18104, 784,   60476,
                12388, 4004,  20836, 32812, 17392, 58884, 30772, 51112, 12704,
                34556, 46136, 22168, 16148, 724,   61128, 47400, 4324,  29408,
                51952, 60668, 48340, 20776, 51216, 32772, 3912,  44868, 8864,
                35144, 55212, 3048,  26388, 10620, 31396, 3904,  24160, 11940,
                3056,  16572, 7276,  47424, 36976, 55856, 32248, 61496, 2808,
                54508, 12040, 32840, 17388, 53280, 35236, 8524,  51456, 17148,
                43592, 4744,  32396, 53752, 40812, 59408, 24660, 50972, 38496,
                60900, 26384, 18124, 65292, 13356, 52344, 59308, 18704, 61784,
                14328, 43976, 23092, 53644, 12252, 3296,  34428, 7164,  57660,
                52856, 17592,
            },
            {
                22148, 9824,  14280, 55108, 19320, 40916, 12936, 48452, 38476,
                28788, 10300, 15148, 39460, 60552, 18356, 37240, 47240, 27384,
                59720, 10004, 40108, 57604, 46140, 38036, 23196, 55848, 18080,
                34224, 4604,  47944, 23024, 39588, 8244,  49076, 25616, 18852,
                55020, 40168, 1188,  20584, 14656, 57852, 49804, 43912, 40776,
                24292, 58372, 49380, 42584, 52644, 11344, 47404, 61552, 18120,
                56392, 5092,  63188, 9760,  43040, 31668, 19784, 64900, 15352,
                38244, 5208,  14008, 57720, 10248, 42536, 59816, 25872, 13400,
                32008, 18856, 43428, 50736, 60468, 39440, 48380, 64696, 42784,
                52928, 57436, 26500, 60096, 30148, 15624, 41392, 23072, 28876,
                36036, 59024, 45400, 2312,  47980, 12840, 28672, 168,   63032,
                48068, 24344, 15728, 44624, 6056,  30592, 56840, 436,   32860,
                11956, 3180,  53144, 6048,  45192, 31372, 7900,  34716, 38784,
                50780, 29380, 17704, 60956, 25660, 42972, 19524, 61720, 23936,
                32896, 4032,
            },
            {
                50120, 62836, 31472, 2348,  35840, 27860, 59508, 4040,  18076,
                51428, 44376, 54460, 26580, 49000, 22624, 56320, 7476,  41408,
                18796, 24088, 62928, 3680,  19608, 33524, 6168,  61644, 28192,
                9004,  58684, 29472, 43804, 19748, 64912, 15832, 2340,  46676,
                28824, 59520, 24196, 63016, 35468, 26228, 11424, 6484,  53360,
                16468, 30000, 7604,  26068, 2624,  37776, 27356, 3684,  44068,
                38268, 24844, 52208, 27984, 36512, 53756, 24016, 40732, 10048,
                55728, 24856, 45440, 40064, 30044, 16760, 36556, 22068, 51916,
                628,   64568, 14872, 7712,  21296, 54584, 18244, 6500,  29396,
                38044, 33104, 10432, 21456, 5516,  65224, 12528, 48568, 6896,
                19900, 9592,  40552, 21868, 64892, 57424, 23520, 41568, 10232,
                35788, 57716, 2640,  65136, 18860, 48592, 13496, 20772, 63964,
                47248, 42016, 29248, 37516, 23428, 16640, 56188, 47520, 2264,
                11088, 57152, 46496, 35548, 9156,  56256, 39044, 49340, 10828,
                44116, 59364,
            },
            {
                6732,  46164, 40320, 52176, 61936, 6228,  45016, 25408, 57188,
                7616,  21000, 34932, 1856,  11596, 31124, 3844,  62172, 34636,
                53468, 5356,  31084, 47824, 9332,  52628, 13060, 39424, 44820,
                51192, 37144, 14408, 5956,  52764, 27872, 34648, 61432, 38288,
                5456,  10176, 49344, 43412, 7528,  39384, 62240, 22320, 34192,
                46568, 65160, 35972, 14608, 62508, 57020, 20040, 8896,  32480,
                14044, 48740, 2268,  17696, 57312, 12984, 3148,  58348, 49540,
                31056, 34552, 7844,  23044, 64712, 1816,  56496, 48784, 39240,
                58048, 28624, 37196, 46916, 30428, 1336,  35344, 58828, 14572,
                22560, 62876, 45692, 39916, 50368, 2088,  44532, 58064, 38560,
                63228, 51812, 30572, 25936, 7228,  44044, 33708, 54172, 19912,
                30236, 39704, 26840, 51504, 37716, 9468,  27976, 40456, 54140,
                16164, 9012,  58028, 48856, 61472, 10636, 41228, 25784, 63796,
                21872, 7056,  39932, 1636,  28068, 51900, 15908, 29896, 100,
                36160, 27308,
            },
            {
                56132, 16476, 24252, 12104, 21092, 49736, 16188, 37208, 33208,
                64140, 29932, 58732, 43268, 63404, 40360, 15828, 44896, 11864,
                49556, 14412, 37096, 55304, 26704, 41736, 58108, 16132, 1724,
                21244, 64096, 55072, 32876, 1316,  41112, 12596, 55660, 21824,
                32440, 16964, 52776, 30280, 19176, 48024, 54744, 28768, 3404,
                9896,  19648, 55404, 45396, 22788, 33828, 41212, 54320, 64644,
                29056, 58932, 40480, 33308, 47832, 7772,  44348, 27216, 17448,
                1068,  62964, 52160, 18812, 53896, 27656, 11992, 6432,  19808,
                45420, 10820, 24596, 59160, 41288, 13104, 27100, 45168, 49776,
                8216,  964,   17144, 54928, 35444, 27676, 18352, 33764, 25164,
                16024, 1096,  56432, 13964, 38216, 17780, 3232,  60216, 14844,
                50060, 7600,  11932, 22364, 34140, 61484, 44912, 3780,  31168,
                36176, 25320, 19384, 1076,  33728, 53984, 4148,  30088, 15400,
                52080, 33548, 62268, 14104, 59208, 21732, 4948,  46056, 64464,
                13768, 19872,
            },
            {
                37948, 1432,  34296, 58044, 8388,  30816, 55732, 564,   11116,
                41576, 14092, 5608,  17836, 52052, 24164, 54560, 29144, 20848,
                26012, 64044, 43524, 21320, 880,   62636, 23728, 31704, 47492,
                26372, 11296, 18292, 49224, 24868, 58168, 44920, 6928,  48780,
                42152, 64536, 4284,  12900, 60348, 1420,  15792, 37708, 59516,
                51368, 39900, 624,   12060, 50784, 5144,  16336, 49408, 23632,
                7148,  20480, 11268, 63820, 22684, 38876, 62456, 35040, 21420,
                47016, 12496, 37968, 3784,  43796, 47512, 34388, 62452, 30980,
                4428,  54232, 17328, 3292,  53008, 63000, 9704,  56472, 24824,
                41680, 60352, 30924, 13256, 23740, 56948, 9764,  53048, 4524,
                46720, 42072, 34872, 48456, 61868, 51224, 31808, 46624, 5952,
                63788, 44144, 59232, 54020, 1796,  15188, 56436, 23404, 49984,
                5372,  62396, 44248, 13796, 27604, 20996, 46352, 59748, 36956,
                44368, 25052, 19332, 42388, 48260, 33076, 37464, 54800, 25064,
                51260, 42656,
            },
            {
                10264, 29644, 64952, 47256, 38912, 43248, 62408, 23128, 52876,
                46388, 24920, 50084, 36860, 9436,  33240, 120,   60892, 38080,
                8448,  2744,  58580, 16744, 35188, 10604, 49628, 5612,  61292,
                35800, 42708, 7348,  38168, 62888, 20756, 16696, 29840, 11044,
                23940, 57076, 37484, 26824, 41564, 33276, 24560, 44380, 13764,
                21540, 32128, 27944, 43152, 59920, 30708, 10528, 45908, 1056,
                35628, 43504, 55428, 4424,  29516, 14376, 51188, 5000,  60112,
                41216, 28628, 58764, 32332, 14980, 9020,  40872, 24116, 15672,
                42960, 63772, 33580, 49008, 22280, 32080, 19216, 38600, 4060,
                34172, 20128, 52124, 48328, 7224,  63452, 41600, 31336, 12268,
                59196, 28340, 19480, 10052, 4624,  27736, 11660, 24644, 37116,
                21248, 28568, 16908, 40844, 31832, 47376, 8252,  18200, 60084,
                11696, 55320, 39660, 51148, 64384, 35660, 6960,  18448, 12228,
                528,   54444, 31072, 4268,  8356,  63512, 11488, 17776, 7312,
                32108, 60116,
            },
            {
                48572, 53592, 18148, 26044, 2784,  14400, 28088, 19224, 38408,
                4520,  61580, 28416, 56484, 20444, 45680, 6612,  48220, 57260,
                41512, 33480, 29740, 50820, 45744, 27956, 37964, 18724, 53332,
                3060,  30068, 56464, 46668, 10032, 3700,  36284, 53948, 60692,
                180,   34444, 20296, 47012, 56140, 10304, 64276, 49896, 4660,
                56612, 63340, 7356,  17816, 36636, 25316, 57336, 38396, 62232,
                51672, 15904, 26484, 46348, 53044, 19352, 32060, 10748, 55064,
                16080, 8176,  49724, 25460, 61900, 20864, 54780, 59368, 50108,
                36884, 26992, 9264,  39640, 6036,  43792, 51376, 15856, 64968,
                55328, 10684, 26396, 43380, 2732,  37052, 20328, 60764, 50720,
                23316, 7836,  64440, 54384, 22244, 42772, 58312, 40340, 55804,
                792,   34832, 48664, 4792,  24996, 64492, 42380, 26928, 38556,
                33172, 21944, 29888, 2680,  9744,  42644, 49688, 57548, 40280,
                65164, 9616,  50356, 56944, 23544, 27496, 44200, 57956, 40824,
                21596, 3872,
            },
            {
                13264, 43784, 22620, 5788,  55196, 33568, 9644,  49092, 59144,
                16564, 31980, 1672,  12612, 39656, 65140, 27116, 17516, 11000,
                52816, 19448, 5232,  13308, 64760, 6980,  56936, 44324, 12504,
                59056, 22096, 14668, 27072, 59820, 31712, 51128, 40268, 26272,
                45336, 14880, 7644,  52228, 2852,  17848, 30452, 7892,  36088,
                42124, 24276, 48420, 53260, 61564, 3336,  19360, 27852, 12408,
                31512, 59780, 9252,  37512, 240,   61168, 42872, 25884, 36460,
                45816, 22740, 4784,  57144, 36192, 2432,  29600, 7020,  12780,
                876,   21064, 60520, 14416, 29216, 61608, 1664,  36044, 27768,
                45956, 5752,  39124, 61504, 15540, 29224, 46144, 3456,  17180,
                38904, 44384, 31596, 37280, 15620, 60952, 2604,  18636, 52428,
                13900, 61848, 9808,  58020, 20492, 12400, 35432, 52840, 636,
                48048, 7552,  17632, 58636, 24188, 14952, 31540, 26732, 22248,
                47192, 28312, 17260, 35992, 39696, 14336, 52504, 2888,  35232,
                62624, 27892,
            },
            {
                7624,  35900, 61244, 41088, 50540, 63592, 44936, 35624, 7000,
                54812, 21992, 43632, 48788, 59696, 15180, 36116, 31476, 23112,
                44496, 60604, 25392, 54308, 40048, 22744, 15568, 32380, 25296,
                40596, 50688, 34696, 848,   43576, 22668, 13192, 5052,  18572,
                62428, 50548, 28660, 58996, 22916, 40160, 46288, 54488, 27448,
                16308, 2796,  34408, 14216, 9204,  44312, 40448, 55276, 5252,
                22224, 42056, 18472, 33192, 57744, 23480, 7588,  48932, 1948,
                30368, 65456, 18272, 42560, 11856, 51788, 38992, 18480, 46644,
                57592, 52416, 34976, 46064, 56736, 24144, 8528,  48264, 12752,
                31468, 18112, 57728, 33668, 21912, 56068, 9308,  26168, 34616,
                55408, 14324, 344,   52212, 47288, 34064, 29232, 8912,  45444,
                31496, 26428, 43296, 37680, 50388, 29572, 5676,  58464, 15872,
                63332, 43368, 53368, 46616, 37788, 61092, 52184, 6004,  3000,
                34432, 13460, 59880, 45656, 1156,  61884, 19624, 29992, 46944,
                16072, 56368,
            },
            {
                51652, 31424, 15644, 10948, 30180, 21024, 3544,  25128, 13612,
                40696, 63256, 34128, 7896,  24352, 2700,  54976, 50284, 4368,
                14140, 36984, 46956, 9524,  34148, 1732,  52220, 61892, 8608,
                5044,  65200, 17688, 54820, 38608, 63888, 47476, 56196, 8876,
                32004, 38788, 11444, 43332, 35492, 61020, 13196, 20876, 62084,
                10932, 58620, 39776, 29996, 22504, 50980, 63520, 15012, 34548,
                48696, 2692,  65028, 50208, 13084, 39716, 17128, 62920, 56768,
                13436, 38252, 52904, 33248, 47972, 26920, 64228, 43472, 32404,
                25500, 41264, 4276,  18680, 11448, 42100, 54500, 20740, 60080,
                52696, 23596, 424,   47764, 12580, 51384, 65044, 40724, 49608,
                6780,  61464, 27024, 5576,  24484, 12384, 50176, 36332, 65208,
                22212, 6580,  53792, 18304, 1604,  62592, 44564, 21368, 31708,
                25604, 12988, 34820, 27684, 4084,  20080, 11664, 56528, 38732,
                62072, 42892, 6912,  21388, 54596, 32824, 5948,  50280, 10876,
                24156, 39500,
            },
            {
                19356, 46060, 468,   59268, 37792, 16936, 56572, 52340, 29448,
                47416, 10832, 18440, 52848, 30392, 42592, 9876,  58816, 40476,
                63808, 768,   30576, 20816, 57976, 43396, 28964, 36348, 49272,
                20744, 45984, 31128, 11212, 7560,  19476, 28716, 34804, 42336,
                23396, 1924,  65244, 16348, 4372,  26320, 6460,  32692, 51324,
                43740, 19236, 54112, 46860, 1200,  32488, 24744, 7396,  45240,
                53992, 29244, 25212, 6684,  45576, 28264, 53496, 32984, 20696,
                41456, 8700,  24460, 408,   16200, 5728,  22308, 10124, 60192,
                15864, 8156,  30424, 64548, 49780, 27064, 32892, 37988, 3500,
                43996, 40520, 63264, 8100,  37420, 31828, 1616,  19196, 11776,
                57576, 21412, 46052, 41280, 63372, 19504, 57184, 4108,  16496,
                49100, 40500, 11816, 32844, 55524, 14560, 39008, 10292, 51200,
                41348, 2908,  56264, 8984,  64088, 41808, 33080, 48168, 18100,
                53376, 25004, 31448, 11376, 48316, 26304, 37816, 59728, 42604,
                1828,  64296,
            },
            {
                5444,  57376, 23028, 48020, 26776, 7804,  42392, 61916, 828,
                37120, 26544, 3964,  57360, 38344, 20948, 47220, 18656, 33852,
                27700, 55844, 17096, 61320, 48736, 4484,  19144, 11080, 39224,
                26652, 2496,  41352, 51948, 25752, 58584, 3828,  15476, 52308,
                57692, 48496, 20664, 30888, 53532, 56976, 39216, 47760, 352,
                37072, 28404, 5872,  65452, 11584, 57516, 18036, 37368, 59280,
                11192, 39136, 15372, 56092, 35528, 4472,  10936, 47284, 3640,
                27368, 60524, 55308, 45156, 62652, 31500, 53220, 35724, 2824,
                54848, 47548, 22712, 36936, 1120,  16792, 5620,  63692, 15156,
                9792,  29496, 16612, 26284, 44864, 54724, 23524, 43564, 28720,
                33120, 37984, 16960, 30904, 54236, 7200,  38952, 43816, 25324,
                55904, 2704,  60732, 46416, 27492, 23032, 48324, 56716, 6800,
                19028, 60160, 22360, 30912, 16404, 45368, 23708, 7940,  29220,
                15000, 1656,  58460, 40944, 64800, 15212, 8632,  21876, 55204,
                33624, 26936,
            },
            {
                38264, 10096, 34748, 53892, 65444, 12476, 33144, 18928, 22888,
                60068, 50968, 44096, 64524, 12336, 28212, 62448, 5732,  24512,
                8668,  50988, 11912, 41256, 26324, 14548, 63516, 53604, 59180,
                15972, 56412, 61404, 13648, 48244, 37156, 44576, 61076, 12036,
                26668, 6584,  34384, 44424, 49604, 10472, 18084, 64208, 23152,
                9148,  60280, 25244, 16524, 35368, 42232, 49240, 27644, 1800,
                20952, 61140, 43616, 19412, 62572, 22432, 59224, 43080, 15516,
                50748, 35320, 11612, 20024, 39864, 56908, 13828, 49432, 19368,
                38524, 61724, 13360, 52660, 59552, 40744, 56980, 46784, 25464,
                35252, 55792, 49188, 60928, 4312,  14060, 59136, 5876,  47588,
                62924, 8844,  2236,  59544, 11188, 22984, 28372, 59960, 13568,
                34620, 29500, 19932, 37292, 8764,  4140,  34288, 28744, 65340,
                36900, 47080, 39684, 50444, 54528, 52,    59624, 37140, 63616,
                43844, 50812, 35532, 18764, 3972,  45056, 52368, 29388, 17536,
                13484, 44568,
            },
            {
                52508, 29796, 14848, 43232, 2484,  39492, 50184, 45980, 9340,
                14352, 32584, 6716,  16636, 35148, 980,   52180, 14720, 43768,
                37624, 45972, 3144,  32592, 38172, 7412,  44888, 22552, 33576,
                6124,  29300, 35676, 23604, 5500,  32416, 21336, 584,   40416,
                18216, 37748, 60196, 14196, 2872,  24496, 30188, 42580, 15232,
                56064, 31580, 45020, 51764, 21920, 4092,  62792, 13800, 52560,
                33532, 7968,  30252, 3044,  50940, 31788, 37360, 25404, 64072,
                6832,  30656, 48836, 3164,  27680, 8280,  23996, 44916, 29340,
                5664,  25812, 33204, 44388, 20104, 10964, 30748, 22348, 51688,
                59712, 2116,  21312, 34244, 18532, 30480, 38844, 51636, 15996,
                24888, 53656, 49052, 35116, 43192, 51392, 32868, 960,   46168,
                8424,  63104, 50492, 15628, 57840, 62020, 41876, 17288, 1452,
                26240, 14640, 10868, 5084,  35912, 26460, 12608, 49948, 4676,
                22324, 10380, 28008, 55592, 23864, 33996, 39848, 62668, 6328,
                48688, 61308,
            },
            {
                21120, 59064, 7196,  25044, 20096, 28092, 6060,  58644, 30444,
                39056, 55496, 21308, 25568, 49424, 56448, 39772, 31060, 60804,
                54624, 21220, 65448, 23664, 51876, 56800, 31324, 56,    41776,
                47988, 9688,  44296, 18056, 63080, 54740, 49900, 29752, 64040,
                45772, 54576, 9404,  28360, 61996, 38212, 58040, 4904,  34880,
                48888, 2948,  13404, 37884, 55176, 9732,  30680, 39904, 46088,
                23656, 55480, 47700, 12972, 40996, 9036,  17996, 860,   56112,
                38920, 22384, 17076, 58456, 42704, 37296, 60464, 1444,  64984,
                41604, 57696, 9408,  3032,  28040, 35988, 7452,  42696, 13624,
                6032,  39428, 11576, 53516, 41772, 63988, 10120, 35396, 2900,
                40600, 12776, 27548, 18912, 4612,  14984, 63760, 20936, 52700,
                39812, 23768, 5624,  44216, 25168, 51568, 12844, 54492, 45348,
                32428, 53072, 62332, 29432, 20944, 56604, 40840, 18948, 32516,
                57392, 38900, 62976, 7572,  49292, 12304, 324,   56972, 36012,
                25156, 3432,
            },
            {
                40356, 16976, 47428, 62716, 51440, 56888, 36064, 15868, 64080,
                1760,  48092, 41436, 59856, 8848,  45320, 22648, 11184, 2204,
                17464, 35288, 5468,  58756, 16420, 10240, 27540, 50212, 13056,
                64540, 21132, 52588, 2008,  39956, 10664, 16632, 7776,  24128,
                32828, 4240,  22292, 41796, 51460, 19800, 46268, 11812, 53176,
                20684, 40504, 62364, 6728,  27132, 43652, 19188, 60264, 5352,
                16884, 64204, 36840, 26404, 53300, 58212, 45296, 29332, 12668,
                52472, 44616, 61312, 5804,  31884, 51684, 16748, 34436, 12652,
                21680, 50528, 17524, 47236, 64188, 55668, 50340, 61348, 18804,
                32176, 45184, 28004, 47532, 7436,  22608, 26744, 55884, 61028,
                21116, 44944, 64792, 57400, 39572, 47536, 9232,  36980, 17656,
                58008, 12520, 32280, 40704, 276,   30832, 21192, 9516,  58292,
                23464, 7664,  42988, 48484, 15500, 64852, 9220,  28740, 46764,
                14288, 3464,  45828, 17088, 31304, 59944, 27568, 20060, 43188,
                11412, 32256,
            },
            {
                56168, 976,   36928, 30700, 13080, 3664,  44660, 23976, 53024,
                27236, 12056, 35868, 3712,  29692, 18892, 34240, 58268, 26612,
                49796, 29784, 13312, 41904, 46820, 36704, 61572, 18544, 54984,
                25488, 38184, 59812, 31556, 26748, 57380, 42952, 36736, 52596,
                11572, 57500, 48432, 16804, 1520,  32536, 8028,  63484, 25928,
                29228, 59132, 18196, 32716, 50164, 56920, 448,   25728, 34768,
                10840, 42032, 1476,  21716, 15736, 4568,  63204, 49232, 20456,
                9536,  33216, 26184, 13556, 47640, 21072, 9996,  55756, 46280,
                7208,  31392, 54264, 39036, 24072, 12352, 348,   26864, 37552,
                54484, 65220, 15816, 57684, 1108,  49864, 43436, 17340, 30184,
                7948,  33452, 572,   23648, 30388, 55444, 25860, 61192, 29280,
                4392,  47956, 53704, 64584, 18208, 36108, 48724, 39316, 4052,
                35100, 19300, 59356, 2068,  33008, 44620, 5932,  51648, 61744,
                25380, 54120, 20768, 36548, 42356, 51484, 15324, 47056, 8360,
                64704, 50404,
            },
            {
                23952, 44796, 10420, 59600, 40564, 33792, 9108,  19880, 42172,
                4896,  18420, 61796, 53796, 14300, 63436, 6916,  47996, 40968,
                62384, 8496,  53408, 25648, 2548,  21040, 5724,  40432, 34500,
                3536,  15748, 7524,  47408, 13420, 22388, 3596,  60920, 47228,
                15104, 29308, 35648, 64864, 26312, 55156, 36972, 16428, 43340,
                1236,  10212, 46456, 23964, 11952, 64504, 36644, 47184, 52004,
                58544, 28460, 49528, 61732, 32848, 39472, 23760, 35028, 41812,
                2032,  65132, 36584, 53904, 2488,  62948, 24972, 39740, 27576,
                59432, 36812, 14700, 3984,  42324, 34248, 16484, 48428, 4876,
                21808, 9092,  24740, 36344, 31080, 13272, 59260, 3768,  46692,
                54636, 50924, 37424, 16448, 5524,  12216, 41608, 1864,  44804,
                35556, 21884, 27168, 10148, 56552, 7036,  60436, 26496, 64148,
                50876, 41396, 13352, 25524, 37956, 53724, 23116, 36296, 1916,
                41316, 33760, 11052, 61456, 2968,  6452,  23144, 54420, 37976,
                28488, 14204,
            },
            {
                34356, 53576, 27624, 22216, 18060, 55292, 63296, 49532, 30028,
                58076, 45776, 32360, 23312, 43512, 37772, 52684, 4152,  15444,
                36416, 19732, 44852, 32168, 63644, 48864, 30632, 58480, 9380,
                45736, 28320, 56208, 35336, 51520, 65088, 33756, 27608, 19776,
                1044,  59864, 39928, 6164,  44528, 12928, 58800, 23284, 50596,
                38996, 55436, 35820, 3788,  40928, 16244, 22256, 8672,  14612,
                31452, 19996, 7740,  56296, 11388, 27776, 6528,  54340, 14372,
                57140, 23452, 8056,  40604, 29760, 43708, 5060,  53100, 17904,
                940,   23036, 62036, 29744, 58188, 20644, 63172, 30500, 43988,
                59480, 50168, 3332,  40712, 62992, 20548, 34048, 38320, 25416,
                14536, 10424, 61820, 43888, 59248, 49304, 33608, 15760, 51016,
                8140,  60064, 14884, 38300, 46132, 29316, 13808, 44156, 16872,
                10644, 28376, 56400, 4892,  60988, 16924, 11784, 58964, 18188,
                8572,  49748, 29456, 57740, 25904, 40080, 63072, 30764, 1728,
                60152, 19136,
            },
            {
                7932,  62944, 5448,  43096, 2532,  47000, 25968, 6552,  13804,
                37148, 9580,  50848, 236,   27344, 10848, 21568, 31820, 24880,
                56996, 672,   60156, 38532, 11416, 54612, 14504, 24008, 52268,
                63200, 41720, 20140, 676,   44404, 17824, 6192,  40976, 55296,
                45052, 24620, 10400, 20736, 52880, 29648, 3260,  47124, 5936,
                31652, 15068, 63112, 52996, 28992, 49044, 5520,  42748, 54736,
                39112, 3648,  45580, 35356, 43052, 47868, 18688, 60884, 31104,
                45888, 16540, 48996, 19452, 56480, 15448, 35740, 32244, 63860,
                49472, 44064, 11276, 51608, 8248,  45708, 53672, 10352, 39348,
                14284, 33488, 18388, 44928, 54872, 10504, 51860, 5380,  64356,
                40956, 19304, 28144, 31984, 22240, 53748, 19864, 65380, 24268,
                55300, 42588, 2408,  20212, 52492, 3696,  22616, 33464, 1052,
                54404, 47372, 31812, 21620, 40352, 47800, 28016, 31460, 43680,
                63980, 22804, 15772, 44516, 52916, 12828, 34812, 16956, 45472,
                4712,  48816,
            },
            {
                39628, 13584, 50716, 31860, 61304, 15064, 38144, 32856, 54300,
                21900, 64616, 16488, 39940, 59584, 56136, 45544, 64884, 42312,
                12096, 47740, 23040, 17008, 7580,  28064, 44316, 1228,  18000,
                31272, 12084, 60228, 24356, 9712,  29912, 49632, 12380, 37336,
                7964,  51344, 33644, 61592, 15876, 41164, 35176, 62604, 21228,
                59672, 27220, 7628,  19044, 57556, 33032, 61896, 27712, 59360,
                23368, 65192, 13416, 17552, 60044, 488,   51252, 10216, 25996,
                4272,  37844, 59496, 27068, 11588, 60564, 46540, 8296,  13044,
                40428, 4828,  33868, 18260, 27052, 5396,  36228, 23492, 1532,
                56688, 25744, 60924, 5816,  28096, 16324, 47456, 29568, 22060,
                56380, 1648,  48400, 6640,  39400, 9520,  3512,  38096, 11708,
                32504, 28652, 63084, 34452, 59256, 41292, 50288, 62540, 38488,
                20368, 6880,  36256, 63736, 9428,  560,   50984, 56624, 4488,
                38308, 54972, 852,   7504,  19640, 47692, 9160,  58676, 21940,
                42176, 56616,
            },
            {
                29924, 20960, 35896, 57236, 9556,  22960, 52044, 1072,  60612,
                41788, 26332, 7336,  49024, 30332, 2740,  17872, 7816,  51040,
                28440, 34040, 5808,  56364, 42736, 36204, 61184, 50724, 39012,
                4200,  48276, 36768, 52912, 40072, 61100, 57100, 25948, 63868,
                17160, 30364, 56424, 2028,  48828, 25564, 8996,  17720, 52332,
                11632, 41928, 22816, 45256, 2964,  38392, 12600, 18116, 1048,
                34292, 50788, 25840, 54124, 30228, 22796, 40436, 35816, 62960,
                42900, 52216, 6560,  33688, 48,    50104, 23920, 20708, 28884,
                58548, 25096, 47592, 55768, 41080, 61444, 15328, 50632, 28632,
                42648, 11736, 52708, 37476, 22956, 196,   42252, 60144, 8092,
                44484, 36304, 52692, 13860, 63412, 45232, 26476, 56620, 46972,
                17820, 6480,  48948, 11328, 16004, 25444, 9008,  56248, 30144,
                45004, 59156, 15652, 52140, 43044, 24584, 35068, 14516, 20148,
                26140, 48132, 35560, 31744, 65368, 38472, 27152, 51132, 32240,
                11568, 25852,
            },
            {
                64872, 40,    48016, 17608, 27092, 40164, 45304, 12256, 19284,
                5160,  34704, 55348, 12356, 20176, 35504, 39060, 14084, 59836,
                3588,  40300, 49840, 64340, 25340, 19464, 32760, 10608, 21952,
                57568, 26784, 16492, 4988,  32540, 14760, 2320,  20620, 48336,
                4316,  41992, 22844, 36524, 12136, 57896, 44728, 33292, 38624,
                44,    48552, 34400, 51756, 9740,  25148, 44108, 53156, 47784,
                11072, 41792, 5156,  38380, 8368,  14944, 55368, 3268,  20044,
                12904, 29096, 22372, 64424, 41880, 38360, 4156,  55992, 52436,
                36180, 15200, 65504, 21948, 2244,  32672, 57644, 19720, 64532,
                47276, 7180,  30636, 49132, 61964, 57128, 34644, 12872, 17428,
                32612, 26616, 59804, 23824, 17084, 34988, 29868, 61152, 404,
                41024, 52920, 22276, 44932, 31176, 37232, 4632,  18336, 12004,
                24836, 2712,  27380, 12656, 19208, 62284, 7492,  45700, 59476,
                9664,  13236, 60684, 42844, 23676, 2352,  55232, 6000,  61760,
                37404, 16292,
            },
            {
                45924, 10444, 41356, 53416, 3832,  64160, 30624, 57800, 48212,
                28564, 44308, 61400, 23808, 47196, 62740, 52968, 26132, 22156,
                55216, 18788, 30380, 13696, 2984,  53420, 6324,  46112, 64132,
                34112, 8048,  55172, 63508, 46232, 22932, 36336, 43628, 33088,
                10700, 62388, 46596, 18976, 5180,  64672, 53908, 6592,  28776,
                56812, 63660, 13688, 29976, 58608, 64628, 21144, 36352, 7852,
                29620, 20728, 62168, 32292, 48656, 63916, 24864, 46588, 32736,
                57608, 48140, 9372,  54076, 17100, 30440, 61336, 43068, 18380,
                1740,  9152,  38764, 29212, 12916, 44844, 9432,  38432, 4648,
                16952, 35036, 20872, 14860, 9604,  39708, 24948, 53068, 49500,
                63008, 11580, 2996,  40324, 57492, 4708,  49524, 20980, 14732,
                35920, 26688, 58248, 1608,  54708, 64716, 43360, 47964, 60548,
                53688, 41424, 49772, 32880, 56916, 29560, 38672, 52868, 32276,
                41764, 28328, 50712, 16832, 57528, 30288, 14332, 19184, 43872,
                2884,  52688,
            },
            {
                59272, 32952, 23272, 60912, 6940,  35048, 16596, 8808,  37012,
                51592, 15272, 3724,  32040, 40700, 5868,  9692,  33328, 46396,
                36868, 61048, 10272, 45196, 37996, 59616, 15560, 27744, 40792,
                13540, 43856, 20048, 28860, 11348, 50764, 7860,  53872, 59340,
                28732, 14320, 53224, 39116, 27428, 31332, 20888, 14704, 24056,
                46036, 18752, 4596,  39828, 16916, 1980,  31552, 56300, 61408,
                16140, 45080, 57636, 2788,  18844, 44028, 11456, 59120, 38828,
                16092, 1944,  35612, 45152, 13516, 25856, 7732,  11840, 32132,
                45108, 59692, 51356, 6676,  54904, 49412, 25272, 30868, 53216,
                41248, 58060, 63044, 45660, 4780,  31132, 19612, 2552,  38276,
                5740,  20904, 46936, 31340, 51340, 12536, 42268, 8352,  54000,
                63876, 10496, 38928, 19260, 7680,  27816, 14636, 33720, 21636,
                8060,  36740, 65316, 4976,  46308, 2976,  16780, 23288, 1684,
                64324, 5888,  21560, 3760,  11008, 46372, 40404, 49820, 34964,
                24052, 7632,
            },
            {
                18452, 38492, 14048, 29012, 43636, 20660, 54032, 25520, 1964,
                22012, 58504, 10388, 53448, 17164, 27780, 57352, 42524, 1688,
                15980, 6464,  26976, 51644, 23236, 31480, 48088, 55584, 2148,
                23980, 51912, 37560, 904,   41324, 56856, 26432, 16084, 38644,
                1208,  24776, 8236,  60848, 43084, 1676,  49208, 41000, 60936,
                9900,  36724, 26380, 54888, 42780, 49504, 13892, 5924,  39448,
                51680, 26692, 13000, 53612, 35580, 27408, 5376,  30456, 7536,
                53088, 24480, 62276, 19512, 56560, 50960, 36576, 63816, 48916,
                26856, 19856, 34180, 42352, 17732, 35780, 58704, 844,   22780,
                11136, 27640, 2044,  25484, 42488, 55672, 65320, 46284, 28984,
                58100, 43112, 55344, 9100,  19288, 25576, 60024, 33812, 23412,
                44148, 3848,  32376, 50752, 60948, 23264, 52252, 39600, 64,
                31144, 17152, 11172, 22072, 40468, 53936, 61448, 12244, 48776,
                36500, 55336, 39484, 33316, 52240, 60340, 26016, 8932,  62656,
                55540, 28588,
            },
            {
                47864, 3220,  56192, 51096, 10960, 48464, 59676, 41524, 63248,
                32980, 38744, 43336, 65476, 780,   49880, 19796, 12508, 64068,
                48852, 32500, 42116, 56884, 188,   8812,  20672, 36392, 61800,
                5684,  58364, 48204, 16888, 61240, 31024, 4436,  65480, 19496,
                50024, 57936, 34196, 16912, 51596, 12796, 36188, 55432, 3096,
                32664, 51488, 61844, 7120,  22716, 35108, 27940, 45712, 23336,
                1324,  33952, 8832,  22448, 60276, 41648, 52024, 18064, 65460,
                43260, 27932, 40660, 6176,  33380, 2728,  21316, 40292, 5056,
                55472, 13672, 61780, 23572, 4252,  63208, 14888, 43064, 60768,
                48676, 37720, 54572, 33036, 50932, 16268, 12280, 7704,  23500,
                34080, 13924, 27316, 35964, 64572, 38832, 1956,  46200, 16172,
                28704, 59252, 13468, 45500, 35544, 11436, 4076,  57164, 62008,
                44440, 48480, 59588, 25988, 35152, 9312,  28072, 43148, 18880,
                25664, 14436, 45212, 62972, 18344, 36876, 1304,  21708, 31948,
                12548, 41940,
            },
            {
                21128, 62904, 26060, 36672, 956,   31564, 14692, 4640,  46440,
                12628, 19004, 24696, 29532, 36592, 45012, 60244, 30732, 25036,
                53632, 20828, 62156, 17396, 39528, 65280, 43496, 11380, 18176,
                30756, 26056, 9724,  34308, 21304, 45236, 13152, 34936, 46952,
                41672, 30032, 5552,  45844, 23372, 9260,  59876, 26680, 16356,
                43712, 21016, 12772, 46684, 59116, 9512,  53480, 60036, 17664,
                64400, 42672, 50044, 37924, 15676, 664,   48036, 34468, 21672,
                13824, 49924, 10928, 59420, 29400, 46184, 58232, 15968, 24412,
                30932, 512,   47736, 9968,  28296, 46724, 20700, 8084,  33836,
                13748, 19048, 6816,  10172, 22044, 60284, 35640, 41376, 53676,
                17044, 62476, 820,   52584, 6820,  48436, 30272, 55648, 5192,
                51880, 40760, 25596, 17424, 48652, 29024, 42528, 19700, 25200,
                13980, 28932, 55012, 5824,  19548, 58000, 50564, 33004, 6892,
                58728, 30880, 9540,  24552, 6380,  29152, 56212, 44768, 16396,
                50948, 5316,
            },
            {
                35120, 8664,  45344, 17888, 65000, 23508, 39280, 27952, 54828,
                6828,  50380, 56660, 8512,  14152, 22520, 4532,  39636, 7828,
                37504, 11092, 4296,  35044, 13996, 28880, 54480, 50844, 33552,
                46836, 14120, 64348, 54540, 6248,  39896, 24288, 52836, 2880,
                11440, 21468, 37544, 64268, 56080, 32196, 19964, 47396, 7368,
                65304, 29800, 756,   38616, 32372, 19328, 3212,  36788, 11036,
                30788, 56844, 4696,  29000, 63268, 10324, 24176, 56728, 3660,
                37044, 55024, 840,   22508, 38852, 12444, 53504, 8420,  43940,
                59004, 37000, 53148, 41344, 32104, 56640, 38688, 51232, 28948,
                64656, 44428, 57460, 47272, 39080, 28268, 1516,  50000, 59464,
                10012, 45632, 22700, 42144, 18024, 14268, 22160, 62108, 37628,
                19844, 8132,  64264, 2656,  55416, 6652,  63284, 37016, 9348,
                51012, 2180,  38256, 42168, 47148, 15080, 4008,  37912, 63260,
                53040, 1080,  47620, 57200, 42244, 13368, 48732, 4204,  64608,
                39144, 58688,
            },
            {
                54288, 30776, 13024, 57712, 42596, 53112, 9208,  34984, 17208,
                61060, 30964, 3532,  47096, 61872, 34228, 50904, 56200, 15436,
                47156, 58500, 26548, 45416, 49396, 6836,  25112, 3152,  59772,
                38968, 42608, 2476,  28512, 50244, 60012, 9068,  57456, 27988,
                62440, 54096, 14220, 428,   27792, 39020, 3728,  53308, 34420,
                40276, 57884, 49296, 25224, 14820, 63448, 49920, 41896, 25704,
                47912, 14464, 20076, 44696, 54128, 32912, 40228, 61952, 8384,
                44756, 30644, 61532, 48396, 17212, 64804, 27160, 34024, 50556,
                11532, 21448, 16588, 64904, 2636,  12480, 26020, 5636,  54040,
                3764,  23828, 30476, 15004, 63600, 6288,  18096, 25264, 30352,
                4292,  39360, 32312, 60536, 28120, 57876, 9244,  33364, 12220,
                48952, 57412, 31032, 21880, 39208, 32548, 16456, 46384, 56024,
                34108, 23020, 64076, 12308, 31528, 62264, 23960, 44408, 11560,
                21224, 40284, 16216, 34924, 22152, 61704, 38160, 33160, 27376,
                10672, 24420,
            },
            {
                15480, 49492, 2440,  33588, 7092,  20008, 49388, 58700, 44472,
                22264, 37440, 40496, 26588, 53764, 9908,  17916, 43624, 28840,
                2176,  22656, 52272, 31736, 60232, 21400, 41464, 16128, 8456,
                20412, 52884, 23436, 37660, 18900, 14856, 31776, 43128, 17368,
                7108,  33112, 44032, 50352, 17852, 42508, 62292, 13424, 24500,
                10192, 17932, 5884,  53876, 44800, 29164, 5220,  21256, 54648,
                2200,  38956, 58852, 7304,  26264, 17712, 13364, 46564, 27036,
                19324, 15344, 35080, 25692, 7212,  42692, 2112,  19420, 62060,
                39880, 6860,  26676, 35076, 49444, 18828, 60060, 45852, 16708,
                36252, 41784, 432,   52292, 33936, 44084, 55796, 47948, 37308,
                20380, 51572, 11948, 47180, 2540,  53528, 41124, 45120, 24192,
                508,   35692, 43304, 10160, 59492, 50652, 12292, 27296, 4348,
                59968, 17764, 8136,  53432, 26848, 580,   55948, 17444, 29788,
                49848, 26956, 65052, 5032,  51208, 9788,  17832, 53392, 20312,
                920,   44152,
            },
            {
                40652, 62352, 22484, 38720, 28316, 60720, 3744,  26328, 11028,
                192,   64468, 15084, 20164, 42448, 1368,  24440, 58948, 33196,
                64684, 41192, 18040, 9484,  1528,  56500, 37372, 63108, 29748,
                57276, 5104,  12204, 61376, 46472, 808,   36404, 48908, 22784,
                39100, 60512, 25140, 9552,  57984, 6572,  48944, 29256, 45788,
                55632, 21856, 36372, 60812, 11408, 40084, 57072, 16112, 61852,
                32720, 10000, 23484, 35672, 48632, 59824, 5564,  31188, 51060,
                58424, 41144, 4400,  52736, 57296, 30892, 37600, 47464, 4500,
                29660, 59892, 45524, 55324, 8836,  24032, 40144, 32556, 10676,
                61932, 19956, 58716, 26196, 11808, 21408, 61088, 8988,  14428,
                64808, 55168, 7520,  24668, 36852, 30944, 5708,  65336, 17724,
                54704, 27968, 14512, 47324, 25016, 1092,  54072, 21700, 40132,
                30276, 43552, 36104, 49732, 20704, 40572, 33824, 60872, 8008,
                36152, 3416,  54716, 32072, 45088, 26076, 6768,  57764, 46292,
                60888, 28836,
            },
            {
                18708, 4420,  55820, 51292, 16252, 45976, 13532, 41592, 32940,
                53388, 48128, 7388,  57508, 32188, 63496, 38056, 12024, 6144,
                54272, 13708, 36420, 61912, 43852, 12732, 33312, 46100, 26764,
                49252, 34372, 44276, 30320, 55188, 25820, 64724, 6120,  52264,
                12488, 46812, 19708, 36260, 31748, 23176, 16224, 36988, 2460,
                63636, 43372, 27808, 1568,  33860, 23388, 8332,  36048, 27488,
                46084, 52952, 64964, 41628, 3264,  52296, 22016, 37848, 1252,
                10596, 64328, 21564, 13164, 45200, 9676,  23048, 56044, 15024,
                24640, 51632, 1272,  13880, 43016, 62692, 3364,  57776, 27664,
                49940, 8308,  38116, 48296, 4464,  40580, 31296, 2892,  42808,
                27580, 34524, 16784, 62952, 19992, 50892, 15536, 26820, 38220,
                60256, 51444, 6348,  63012, 18820, 36496, 44104, 64728, 7260,
                48792, 13692, 2912,  58572, 45300, 10660, 5540,  51448, 46120,
                14248, 43024, 19312, 11972, 59092, 29960, 41096, 14468, 35760,
                51512, 7752,
            },
            {
                37176, 47500, 25172, 10440, 64164, 30328, 36908, 56324, 23956,
                18484, 29036, 36056, 11372, 51176, 28400, 48636, 19968, 45528,
                27028, 49648, 4556,  29544, 24764, 18996, 51496, 10296, 504,
                14940, 64116, 18052, 7324,  10384, 41296, 20792, 58492, 3424,
                30208, 56316, 2120,  63908, 53692, 11280, 60624, 51204, 6516,
                31112, 14056, 52412, 17304, 47984, 64692, 51256, 44052, 13216,
                6312,  19152, 29312, 15976, 11888, 33392, 63528, 43056, 55956,
                24536, 46912, 32484, 39508, 60852, 18508, 49868, 63532, 32624,
                42484, 18088, 37856, 31000, 20612, 34348, 52300, 14580, 22260,
                44604, 31700, 13652, 62468, 17024, 53512, 35516, 58136, 22864,
                46652, 84,    40740, 57116, 43876, 9972,  58496, 48200, 4516,
                11032, 21048, 34464, 40864, 29576, 8392,  14756, 33348, 56600,
                19508, 61740, 24760, 29196, 15284, 54332, 38084, 21888, 25076,
                56336, 60076, 38884, 23732, 88,    49200, 63608, 3616,  23556,
                11356, 31432,
            },
            {
                57536, 13428, 43904, 35224, 6336,  21432, 1912,  62940, 50420,
                4788,  60412, 45552, 22720, 5644,  16060, 3052,  60672, 39560,
                8564,  21840, 57908, 40420, 47908, 54816, 5836,  23380, 55852,
                41892, 36712, 25220, 58808, 48280, 33972, 13444, 27588, 35240,
                41856, 16312, 26440, 5904,  44992, 40252, 27096, 21028, 41616,
                58712, 8592,  39344, 57580, 4756,  30400, 18740, 1928,  59540,
                40412, 50052, 200,   55564, 45040, 27260, 8464,  18276, 14236,
                29768, 3136,  50508, 6008,  28028, 35924, 144,   12640, 7144,
                54520, 10084, 64292, 58156, 28140, 6776,  46908, 37172, 1548,
                54532, 5652,  56536, 28336, 45352, 24532, 10904, 50952, 18492,
                61648, 5896,  31760, 12788, 28736, 3204,  33904, 22856, 42180,
                31248, 45968, 2276,  57952, 52036, 61136, 48080, 25980, 1840,
                37808, 52608, 9424,  34512, 65216, 18520, 31020, 62444, 1668,
                28852, 9384,  33416, 51844, 16052, 37304, 20924, 33632, 55312,
                43176, 65372,
            },
            {
                21284, 27708, 480,   59184, 40256, 54056, 47112, 16800, 9508,
                34320, 14000, 40892, 54568, 61972, 44080, 34852, 53244, 30696,
                17280, 63392, 33692, 15132, 2612,  65048, 38668, 31212, 60584,
                20168, 3928,  52112, 38984, 1708,  16836, 53980, 45176, 63004,
                9396,  51452, 38296, 48628, 15056, 33544, 1020,  54900, 18144,
                34752, 25332, 46248, 22692, 12560, 42416, 26248, 55704, 33376,
                21996, 62584, 31796, 38008, 20052, 58384, 47920, 35264, 53344,
                61372, 37160, 19756, 54632, 15532, 58324, 25108, 40856, 46452,
                34756, 22284, 47924, 4344,  50644, 12660, 56752, 18640, 64996,
                25236, 42540, 34496, 19616, 1748,  64508, 6908,  29464, 13372,
                38124, 49536, 54516, 23668, 47576, 38636, 61864, 13888, 53640,
                63880, 17116, 26412, 12804, 23200, 4176,  17308, 41964, 11260,
                28528, 46648, 40392, 6512,  42824, 3228,  47044, 12000, 41780,
                17740, 47992, 6420,  61056, 44500, 56556, 8992,  46752, 26660,
                17048, 5360,
            },
            {
                48332, 33336, 52388, 18524, 31800, 12448, 27352, 42996, 38572,
                58036, 25132, 3824,  18992, 37664, 26436, 13332, 23320, 56708,
                292,   43072, 52012, 11056, 27252, 35208, 16840, 44100, 13280,
                47296, 32712, 11132, 22540, 63712, 30728, 50200, 6948,  18984,
                23660, 60008, 31140, 20600, 58864, 8944,  65036, 44352, 11604,
                49128, 62828, 6184,  32916, 60916, 53668, 36412, 9528,  48796,
                15036, 25788, 10720, 7072,  61072, 23644, 1660,  40400, 6608,
                26472, 11464, 41824, 62464, 9128,  44788, 30356, 52068, 61544,
                27172, 1904,  39836, 16700, 25092, 44140, 8644,  41020, 29820,
                16036, 50560, 10616, 59304, 37572, 48812, 40212, 56172, 44360,
                26224, 9660,  20428, 59900, 7812,  52432, 18952, 1592,  36076,
                8240,  50072, 38808, 55976, 45116, 36832, 31280, 53780, 63220,
                21860, 16256, 60504, 23496, 50172, 55520, 26560, 58772, 34720,
                52804, 64620, 21276, 25804, 4188,  28800, 13656, 61768, 1384,
                53212, 40376,
            },
            {
                60772, 15844, 63368, 8636,  49416, 22884, 61920, 7136,  20196,
                30952, 48668, 64740, 10220, 32032, 58536, 7420,  46980, 10696,
                37068, 24572, 6900,  45328, 21012, 58488, 8516,  52788, 26108,
                6624,  61880, 28108, 56236, 42468, 4760,  24812, 37312, 56904,
                264,   43276, 12168, 3920,  53284, 24244, 30100, 38136, 3452,
                28356, 15692, 51624, 20144, 38800, 3356,  16412, 64008, 5736,
                39108, 57724, 46896, 52544, 42260, 13552, 30628, 49684, 16332,
                57824, 46180, 23700, 33752, 4736,  38736, 20968, 3276,  18908,
                14564, 55604, 31636, 60448, 36100, 63052, 21724, 33332, 59648,
                3812,  38376, 23160, 7808,  30792, 15228, 21180, 60508, 5048,
                33000, 63464, 36356, 41664, 15936, 27080, 45248, 56592, 24672,
                29364, 59556, 5592,  33060, 10020, 19632, 58760, 49244, 5140,
                35364, 57804, 220,   12996, 32932, 20320, 8704,  37576, 4916,
                14100, 31348, 39816, 11796, 35452, 42192, 19780, 50144, 37980,
                30132, 9448,
            },
            {
                35984, 3348,  24648, 42120, 55860, 2920,  35576, 56896, 51660,
                2512,  44436, 28196, 53036, 1012,  50504, 17036, 40484, 61040,
                50076, 32260, 55776, 61816, 39364, 48388, 29916, 1268,  57040,
                40860, 49976, 15492, 45732, 19876, 12324, 61260, 40684, 14900,
                28892, 34276, 62536, 46744, 36208, 16776, 50220, 56240, 22328,
                59216, 36892, 376,   44672, 10824, 29344, 46600, 23568, 43608,
                28896, 4012,  35704, 17012, 33496, 5184,  55264, 62608, 21260,
                31920, 2168,  53096, 17788, 48368, 56220, 63932, 10800, 37252,
                58856, 44508, 11364, 6200,  53188, 28604, 372,   49336, 12156,
                45992, 53740, 63548, 43896, 52352, 27056, 34088, 2372,  17948,
                53200, 14476, 1356,  30096, 5276,  64640, 32740, 11228, 40388,
                47484, 15020, 22064, 42552, 65184, 2140,  25284, 14356, 8732,
                44132, 31896, 51212, 27876, 39328, 63840, 15984, 49152, 24084,
                45064, 1164,  57404, 46996, 53544, 63892, 32044, 6844,  59632,
                22988, 12800,
            },
            {
                51184, 46104, 29128, 37740, 14392, 45392, 29700, 11244, 15680,
                60712, 22500, 12696, 39052, 43180, 21272, 63580, 27416, 4884,
                20068, 14232, 29160, 18248, 3708,  13052, 63748, 22000, 37180,
                18360, 34316, 2444,  8652,  36072, 54640, 31912, 47320, 9784,
                49892, 54476, 22300, 7884,  27704, 41956, 4968,  13032, 46488,
                9236,  30864, 54564, 63352, 25696, 57160, 50544, 19424, 59140,
                12760, 54604, 21020, 24940, 65516, 45432, 27560, 10520, 37880,
                43776, 8032,  64840, 28696, 12320, 24396, 32336, 47380, 7956,
                29200, 49516, 23788, 41712, 19072, 14840, 56260, 36792, 6112,
                26796, 19248, 14396, 932,   58196, 11828, 47664, 55016, 42228,
                23840, 46544, 50928, 57396, 43656, 22476, 49768, 3860,  60964,
                19484, 988,   54200, 28432, 51768, 46356, 38224, 29856, 40904,
                55160, 19764, 10332, 45468, 57064, 6116,  43536, 29728, 60660,
                54956, 18668, 28184, 8556,  16756, 2092,  24832, 45388, 17380,
                56808, 42744,
            },
            {
                18924, 6272,  57872, 20508, 5172,  65084, 24248, 47684, 41244,
                33324, 6664,  17572, 56416, 34164, 5920,  30472, 45112, 35600,
                58068, 2040,  42360, 53888, 25588, 33396, 51124, 42984, 10712,
                24520, 63164, 29808, 58304, 51404, 26736, 1996,  21404, 65432,
                18172, 2972,  39748, 13948, 57552, 63396, 20020, 33928, 61368,
                42920, 18312, 23124, 13504, 41388, 7780,  34992, 1116,  31628,
                40764, 61888, 2184,  51104, 8776,  39536, 19128, 2748,  52232,
                59488, 14804, 35680, 50360, 936,   43092, 15432, 53240, 41016,
                21384, 5096,  65520, 33492, 47852, 9392,  43568, 23348, 62372,
                57668, 31612, 35400, 40636, 24976, 6504,  36580, 61380, 9240,
                28272, 39248, 11908, 19672, 34012, 8968,  37744, 14244, 30852,
                43864, 62844, 35772, 11944, 16844, 7124,  60384, 22632, 64112,
                2832,  25892, 62304, 17556, 35684, 2064,  52348, 21324, 10072,
                40908, 36020, 62516, 23400, 50856, 58080, 40568, 10892, 34672,
                2876,  26000,
            },
            {
                54044, 62300, 11780, 33948, 52172, 39552, 17956, 752,   53736,
                36836, 63088, 46500, 24360, 59400, 15116, 54208, 11544, 23856,
                47932, 64576, 10116, 37780, 5988,  46260, 16628, 7060,  60752,
                53660, 5340,  44244, 39468, 17344, 60188, 7152,  34788, 44128,
                27084, 58984, 32432, 45140, 51396, 816,   38708, 25292, 6628,
                51796, 39436, 5128,  48924, 32164, 59972, 15396, 52828, 10100,
                26572, 48108, 37688, 14660, 29720, 57028, 48976, 34628, 23180,
                26788, 41004, 19712, 57344, 38348, 59964, 27460, 3908,  61960,
                35732, 16364, 54948, 2360,  38428, 59996, 51584, 30256, 17080,
                42136, 8128,  48868, 21608, 62932, 44848, 15608, 20324, 31956,
                65524, 3508,  59560, 25944, 52872, 60456, 17924, 55968, 51516,
                24236, 7976,  39624, 25624, 57124, 32228, 49812, 11400, 16424,
                52592, 37448, 6856,  48768, 23908, 14012, 61716, 33784, 3468,
                51332, 13120, 5472,  33200, 38520, 14192, 29176, 54752, 49384,
                64860, 32136,
            },
            {
                8476,  40716, 48232, 27060, 60896, 8980,  32348, 59916, 26116,
                10060, 28796, 50744, 2100,  9184,  37028, 48920, 2960,  40020,
                16408, 33916, 51720, 22820, 60252, 56280, 39936, 27636, 31876,
                46940, 14616, 21172, 10348, 24040, 48392, 42024, 12832, 53596,
                37216, 5388,  16156, 24748, 10640, 30716, 48404, 58172, 15452,
                27484, 64944, 35292, 56096, 2600,  21184, 38248, 45704, 64128,
                17612, 6696,  32792, 43672, 60380, 4764,  16076, 63624, 12196,
                47788, 4064,  30664, 10128, 22352, 7096,  34572, 19104, 56848,
                25188, 50520, 13232, 30884, 25796, 20432, 3656,  13140, 38908,
                2004,  52144, 11148, 55812, 3608,  30316, 51304, 38700, 5680,
                48012, 17264, 36080, 7012,  45840, 320,   41400, 27772, 4684,
                34576, 58116, 48720, 4028,  41756, 20464, 692,   43312, 33980,
                47212, 29208, 58812, 40640, 31560, 55556, 27652, 38776, 16440,
                26144, 59108, 44780, 48612, 19604, 7080,  59984, 4560,  21588,
                14788, 38280,
            },
            {
                29696, 17052, 22556, 2160,  15504, 44716, 49440, 21200, 57520,
                4536,  19688, 41708, 31816, 65356, 26236, 19468, 61272, 57008,
                8108,  27228, 44284, 13328, 30604, 2316,  20332, 12112, 57964,
                176,   36896, 50484, 64520, 3324,  30556, 15884, 62664, 22976,
                9172,  48084, 62288, 55468, 35804, 17864, 8212,  54180, 32988,
                1952,  11480, 46772, 17032, 28712, 62616, 25424, 4508,  42404,
                23060, 57900, 53380, 20108, 11204, 24388, 41908, 31208, 55828,
                7316,  61632, 54092, 43868, 46880, 63304, 51884, 12432, 45228,
                536,   39768, 59020, 46124, 7184,  63472, 34612, 55488, 46904,
                24732, 65068, 27888, 33856, 17668, 59300, 26356, 12960, 57600,
                22928, 42680, 55288, 29816, 15160, 23696, 64172, 48032, 10108,
                21424, 13632, 18320, 63984, 29064, 53396, 36316, 57736, 24632,
                8580,  14556, 3948,  20844, 11516, 43208, 8484,  47676, 65312,
                42220, 21932, 304,   30140, 64176, 43408, 35552, 26780, 47528,
                44420, 516,
            },
            {
                50896, 58200, 43140, 36656, 56564, 28864, 3840,  13172, 38024,
                44228, 15308, 55864, 12052, 39200, 52476, 43076, 28996, 32636,
                21552, 652,   62000, 18956, 36456, 49560, 65284, 34732, 51804,
                25412, 41692, 28684, 54696, 33672, 38704, 56268, 668,   50880,
                29556, 40852, 20808, 3160,  43244, 64388, 22832, 40584, 45068,
                20352, 58916, 24368, 43224, 9084,  50888, 12500, 55476, 29528,
                35668, 13156, 308,   28152, 47452, 54392, 36952, 1644,  44968,
                18004, 35300, 13616, 25712, 3008,  16020, 30104, 41492, 8472,
                32872, 28472, 10268, 18184, 42348, 49448, 28128, 10140, 60416,
                19492, 43460, 14264, 37696, 47472, 7376,  40460, 53608, 1196,
                33748, 49584, 10816, 62560, 40148, 32068, 12460, 35856, 44524,
                61360, 52564, 32744, 45672, 9772,  14936, 62080, 5776,  19336,
                60840, 44704, 54304, 64436, 50448, 836,   58380, 19428, 6100,
                31420, 56864, 10452, 54144, 25104, 12172, 52648, 17808, 9420,
                60648, 24184,
            },
            {
                34828, 63944, 6096,  11064, 53160, 24000, 42092, 64212, 52376,
                30060, 48604, 60640, 22944, 6968,  16720, 4024,  13012, 45768,
                55260, 49140, 40452, 54324, 6600,  9936,  44556, 22536, 16048,
                8068,  60516, 18424, 13212, 7296,  59164, 25680, 45464, 18940,
                33432, 60496, 12276, 26444, 51984, 28756, 4216,  13344, 61984,
                29476, 37540, 6316,  53824, 33560, 40512, 19132, 48504, 8336,
                60972, 50264, 39260, 64472, 34044, 8276,  62016, 21916, 27720,
                39412, 51696, 20884, 32708, 58880, 36848, 48760, 23772, 64444,
                53600, 21236, 61352, 35952, 56652, 14112, 22568, 5216,  36616,
                32200, 6596,  54348, 160,   61084, 20536, 10416, 45252, 63896,
                14648, 27276, 2668,  21080, 51388, 56712, 19124, 2396,  54804,
                30052, 6300,  38292, 1560,  23628, 47728, 27456, 40016, 50048,
                31300, 38368, 26752, 33892, 17284, 36816, 24876, 53312, 35496,
                14472, 49332, 36924, 16820, 39920, 2240,  61624, 31232, 41452,
                55360, 12712,
            },
            {
                3552,  45648, 30668, 18728, 61556, 33236, 8208,  17496, 34688,
                5368,  25372, 212,   34100, 47144, 58388, 35860, 63440, 10008,
                37636, 5700,  15388, 29192, 25252, 58660, 41220, 4000,  54896,
                45940, 35812, 4364,  43772, 47644, 20800, 10204, 36368, 4552,
                54280, 7684,  46368, 39048, 15096, 59640, 46900, 36140, 9532,
                52556, 48444, 14164, 61600, 3792,  58428, 36604, 1556,  32328,
                21604, 16724, 44636, 25600, 3580,  18776, 13296, 50620, 57648,
                9400,  65180, 1248,  42420, 55544, 10532, 20032, 4920,  57612,
                15144, 46924, 5940,  24684, 1360,  40180, 53768, 44688, 51300,
                16116, 58556, 22968, 50256, 41944, 31676, 24028, 35908, 29236,
                18904, 60424, 38164, 47100, 9044,  5076,  59204, 38604, 24444,
                15720, 26640, 42752, 60440, 55616, 33600, 3868,  12924, 56084,
                10168, 1804,  22764, 7888,  46776, 61428, 29848, 12016, 45364,
                4104,  26884, 62160, 7992,  46576, 22420, 49672, 4248,  37248,
                28104, 20652,
            },
        },
        {
            {
                3492,  45316, 50180, 1564,  61948, 35240, 48988, 17960, 64472,
                21412, 39548, 53352, 31092, 49588, 54560, 33676, 17316, 29680,
                60844, 49116, 44096, 6784,  11272, 23272, 4584,  27108, 59920,
                37920, 29988, 23940, 47812, 14576, 55360, 30652, 17656, 62216,
                41440, 11012, 32348, 60508, 46648, 9200,  848,   37764, 41752,
                24780, 7292,  39932, 280,   48288, 11312, 6076,  41932, 46236,
                53180, 26140, 49724, 30340, 20388, 63320, 26836, 43876, 9704,
                16688, 54260, 47516, 14080, 31584, 55600, 39024, 62856, 5540,
                24756, 44776, 15680, 31476, 43164, 38752, 28656, 17692, 45200,
                57976, 26908, 4128,  19016, 29912, 7736,  57840, 17268, 47696,
                54800, 34856, 19924, 8348,  31984, 3224,  20952, 63016, 53996,
                36860, 1376,  58852, 51776, 4404,  41848, 15240, 63948, 46872,
                2904,  58036, 5444,  63732, 39144, 7448,  17796, 64872, 20748,
                31948, 50168, 36120, 29200, 2480,  48688, 31668, 19736, 35064,
                10556, 60864,
            },
            {
                55888, 37744, 11648, 30880, 20968, 55124, 6664,  13404, 42896,
                5188,  58892, 16640, 3728,  43236, 25276, 2024,  47576, 39308,
                8976,  34928, 20424, 55212, 38332, 52612, 18732, 35188, 12272,
                21532, 56404, 10728, 3400,  42552, 64444, 22468, 38088, 34028,
                2376,  20016, 51684, 16116, 40168, 35072, 63220, 51468, 10868,
                47588, 17916, 44452, 22412, 34276, 53692, 27220, 63492, 31600,
                14892, 8424,  56988, 42844, 10736, 55036, 6964,  29848, 39136,
                64228, 1412,  28624, 40988, 4412,  23956, 10928, 29676, 58696,
                37524, 9724,  53828, 49360, 6440,  57076, 14176, 50724, 23876,
                9772,  15700, 50400, 60828, 12036, 43288, 38488, 32568, 9652,
                28028, 1140,  24996, 37484, 52800, 13780, 29616, 6756,  12228,
                42704, 49292, 19272, 26000, 34428, 60000, 23596, 53644, 9520,
                41328, 52076, 12956, 43888, 23080, 56240, 50660, 4532,  53480,
                8108,  62036, 5008,  57388, 52776, 39464, 15032, 58616, 816,
                47028, 17936,
            },
            {
                7984,  24568, 59888, 43028, 26572, 10216, 38696, 57392, 27280,
                47200, 33380, 10312, 62012, 20708, 36388, 14328, 62984, 22152,
                56384, 196,   16112, 32308, 62276, 2132,  41768, 64312, 53832,
                7888,  43940, 61300, 36044, 28040, 7176,  47080, 12816, 49456,
                24284, 44852, 65264, 26380, 5980,  19668, 56904, 22640, 4748,
                55300, 30044, 13480, 64276, 57420, 15448, 19408, 38336, 3180,
                21912, 60036, 35892, 23792, 15516, 33968, 59176, 22228, 51880,
                5456,  25028, 58400, 17956, 63420, 35980, 44392, 20012, 48216,
                2948,  28008, 62148, 18740, 24520, 34240, 64348, 4668,  37604,
                54636, 32124, 40700, 46376, 35420, 3420,  64692, 23648, 52148,
                44436, 58004, 49104, 65332, 42800, 59396, 24376, 50828, 57728,
                27180, 31184, 62784, 10624, 56096, 28884, 6188,  37460, 32912,
                17012, 27120, 34660, 19900, 59796, 29184, 11624, 35596, 44580,
                27700, 12328, 41516, 18212, 25424, 7764,  64796, 27288, 50708,
                41712, 30244,
            },
            {
                64116, 15128, 34356, 48440, 3160,  63444, 32144, 52208, 1636,
                37012, 23960, 55516, 28172, 40928, 58168, 6976,  44656, 27152,
                51796, 42680, 58684, 25376, 45688, 28780, 15480, 47740, 30900,
                484,   51360, 25524, 17336, 58584, 32364, 900,   60616, 53336,
                8248,  29916, 3688,  55188, 31644, 11544, 44004, 15008, 33524,
                61512, 36992, 51976, 8676,  3876,  42808, 50968, 9452,  55528,
                47968, 40184, 6284,  61860, 3628,  46772, 40532, 13016, 48188,
                33276, 37332, 11440, 49220, 8404,  52124, 60920, 16032, 33112,
                56680, 13968, 36072, 192,   46416, 41696, 8736,  30156, 47720,
                20736, 63728, 1240,  24900, 53456, 20368, 55916, 6024,  14728,
                18968, 31076, 12012, 5020,  17428, 47132, 35620, 40688, 2928,
                15136, 46020, 5580,  17092, 43892, 48708, 12060, 19792, 56908,
                44832, 63024, 3856,  49448, 15328, 232,   40960, 24884, 59164,
                16332, 38276, 23012, 46920, 33916, 10976, 45032, 36380, 22376,
                12796, 53324,
            },
            {
                38860, 20528, 6512,  58224, 17836, 45656, 22744, 15496, 61592,
                18924, 50544, 13092, 576,   48024, 18036, 52764, 31928, 12004,
                18652, 5112,  37284, 13164, 7492,  50808, 9892,  23820, 39004,
                19920, 33252, 13816, 40384, 49648, 20460, 43436, 27060, 16392,
                57804, 37664, 14028, 41908, 50248, 60220, 28524, 48640, 25628,
                1868,  19788, 46324, 26684, 32008, 23208, 35984, 59360, 28772,
                12204, 32432, 17532, 51164, 28488, 53552, 18364, 2116,  62404,
                19796, 45380, 56204, 22540, 32468, 608,   26296, 6576,  42060,
                22504, 50964, 40592, 59916, 11916, 21716, 52344, 59268, 2700,
                42960, 13044, 28240, 8824,  16420, 48052, 29252, 41556, 37236,
                62728, 7760,  40148, 22576, 27724, 604,   9580,  19500, 64224,
                22420, 37772, 54592, 40372, 2188,  21952, 65372, 39640, 1312,
                24268, 8384,  30396, 54608, 36968, 45952, 62768, 31072, 2728,
                56012, 49468, 63952, 1236,  60284, 53912, 16752, 3636,  61676,
                5500,  26344,
            },
            {
                44400, 1356,  51460, 40448, 12732, 36076, 4744,  41684, 29416,
                8080,  44428, 64788, 35384, 30508, 9732,  61488, 3392,  40348,
                65424, 29480, 48008, 61044, 20692, 33844, 59760, 56636, 4724,
                61912, 46812, 55628, 3792,  8668,  54296, 11724, 39604, 34904,
                21840, 62772, 47736, 18260, 23560, 2720,  38656, 64632, 6452,
                42636, 57880, 11800, 38976, 60456, 47316, 1300,  18048, 44336,
                24208, 63128, 904,   44912, 37804, 24820, 8516,  31316, 54452,
                15140, 6792,  29068, 42912, 14500, 39524, 47072, 55280, 10576,
                64616, 3980,  30480, 26252, 55772, 32864, 15792, 26972, 18556,
                34944, 56940, 60308, 39452, 32944, 61536, 12432, 1496,  26492,
                46660, 60112, 33768, 51664, 56476, 62080, 31880, 55028, 47984,
                33492, 8372,  59560, 25124, 35108, 53072, 31308, 14816, 51264,
                60484, 47308, 40044, 10888, 23952, 7260,  18160, 48112, 21476,
                9684,  32344, 6084,  28992, 42676, 21528, 30796, 40292, 55484,
                32968, 49072,
            },
            {
                62568, 30672, 56540, 24052, 28064, 53924, 60140, 46972, 11632,
                55020, 39508, 22076, 5296,  56544, 25884, 37936, 23104, 49884,
                34268, 10184, 23612, 53516, 41544, 2956,  36624, 27244, 17524,
                41948, 10988, 28356, 65068, 36880, 24636, 61684, 6544,  46400,
                1812,  10700, 33160, 7280,  58252, 34864, 9428,  17496, 54268,
                35736, 29332, 7104,  16252, 54616, 5908,  65360, 14040, 50452,
                7076,  41796, 54936, 20280, 12384, 64900, 57764, 42804, 26748,
                38996, 58988, 2456,  65316, 53260, 19564, 59432, 29400, 34996,
                18256, 45196, 8216,  17028, 47924, 6028,  63120, 38672, 10424,
                45600, 6548,  50580, 22620, 4844,  44300, 57208, 50704, 21068,
                54512, 3320,  13620, 44844, 37016, 15568, 43256, 11840, 28368,
                4200,  51144, 13852, 29820, 10088, 61296, 46276, 7800,  28168,
                35328, 13152, 21268, 62204, 56156, 33044, 52180, 13036, 42932,
                53608, 36876, 19464, 14480, 51116, 7328,  57716, 47276, 19424,
                9992,  16076,
            },
            {
                22604, 11596, 7040,  33460, 64620, 8956,  19248, 31544, 24556,
                2432,  32888, 15772, 51908, 43472, 13904, 46672, 59140, 15552,
                55896, 44996, 1676,  17424, 30404, 55096, 11716, 45208, 50020,
                6568,  35316, 22408, 16828, 44516, 30560, 51012, 18700, 54928,
                29196, 42868, 51320, 26184, 40796, 52980, 21472, 46512, 12744,
                23436, 51016, 63020, 21492, 41268, 25688, 34604, 30424, 38780,
                58388, 26948, 9372,  33656, 49032, 5172,  35148, 46408, 10108,
                21828, 34208, 49756, 25612, 9776,  36832, 3360,  13276, 49676,
                24868, 53884, 39716, 61844, 35772, 1504,  42056, 54444, 49472,
                24644, 30468, 14744, 54972, 36116, 18456, 9720,  31356, 35500,
                16632, 24572, 29860, 8932,  20272, 5616,  25440, 39536, 58492,
                18264, 44440, 63228, 20472, 49460, 5160,  17468, 42268, 58188,
                4708,  18232, 49936, 42360, 2152,  27944, 39120, 65144, 3972,
                26936, 61704, 40444, 58748, 25976, 34952, 13076, 64,    28528,
                59400, 36888,
            },
            {
                55024, 47532, 42776, 14776, 49004, 400,   38620, 50832, 58424,
                63140, 48276, 27384, 60256, 20208, 7608,  33108, 964,   20804,
                7032,  26756, 62060, 38812, 5756,  64496, 15268, 21448, 62764,
                31604, 59036, 52096, 1260,  57540, 13112, 33544, 3132,  64252,
                23304, 57028, 12984, 61468, 92,    15256, 62592, 31976, 39800,
                58992, 824,   44632, 31568, 49912, 9736,  19708, 53584, 2996,
                21316, 35708, 15844, 59456, 30180, 22496, 16352, 360,   52280,
                61096, 12912, 41560, 17344, 30984, 45716, 22720, 63100, 38540,
                5924,  58268, 21296, 10836, 51724, 23356, 29188, 13592, 4700,
                65392, 20004, 41396, 564,   62584, 25084, 40064, 64380, 6708,
                58952, 41664, 53300, 64100, 47260, 59992, 50132, 1688,  53220,
                23444, 36000, 41280, 68,    55324, 37712, 25932, 22740, 54032,
                38868, 32724, 26732, 6312,  16312, 60244, 9208,  22464, 35012,
                16044, 46588, 11196, 2096,  48408, 63812, 38700, 24624, 52164,
                42300, 3944,
            },
            {
                25952, 18616, 61024, 37140, 21180, 44188, 26624, 13752, 6340,
                17736, 37180, 4164,  10784, 41160, 64096, 53804, 29064, 43120,
                37208, 51552, 13440, 48892, 44280, 24108, 51948, 39352, 2004,
                25284, 14464, 47536, 38308, 7708,  21228, 48488, 40120, 15072,
                36196, 5088,  19152, 31380, 45060, 28732, 49576, 3632,  26952,
                8872,  18192, 34980, 13572, 4068,  56760, 45252, 62072, 11832,
                48056, 52692, 63624, 2228,  42496, 56400, 39904, 62780, 18992,
                29940, 3756,  56756, 5696,  60560, 54088, 7960,  43668, 27896,
                808,   33732, 14200, 31340, 44352, 57248, 19232, 61152, 33440,
                36912, 53216, 8948,  48652, 28120, 46500, 14472, 2424,  44024,
                48956, 11064, 38644, 2688,  23236, 35124, 30936, 16668, 64800,
                7464,  11116, 27680, 15984, 45004, 33712, 59280, 10584, 45660,
                836,   63196, 52664, 57696, 35660, 43804, 48968, 30124, 58296,
                6344,  56252, 23896, 30576, 43484, 9516,  17456, 61104, 45676,
                7948,  34252,
            },
            {
                39628, 1668,  29300, 57988, 5620,  53064, 56820, 34936, 42140,
                21736, 45644, 57492, 30352, 35824, 48744, 18016, 12100, 62980,
                57872, 31376, 18532, 34988, 10448, 28396, 33296, 8140,  57164,
                43092, 10116, 29752, 63700, 42316, 27712, 60796, 9968,  45408,
                25760, 49232, 39072, 54324, 10316, 37532, 7060,  56752, 42812,
                64052, 48280, 54896, 28232, 61080, 37132, 15804, 27528, 40604,
                6180,  24800, 45536, 13500, 27308, 51692, 6768,  25280, 47956,
                37304, 44948, 23652, 35788, 48312, 13840, 32976, 57204, 16660,
                52044, 46944, 63984, 25944, 6704,  38124, 9204,  47052, 2192,
                44104, 16568, 59828, 31936, 11848, 58472, 52744, 22896, 27488,
                19268, 33352, 15208, 28696, 57500, 8152,  12684, 45784, 38352,
                32544, 48496, 56900, 62340, 30784, 13052, 3332,  64516, 29692,
                15512, 22064, 11940, 46244, 24476, 19840, 13688, 780,   51156,
                44116, 18096, 33708, 52996, 20832, 55072, 5260,  32624, 21624,
                15188, 65368,
            },
            {
                53768, 49376, 9660,  24592, 32428, 17136, 11160, 62112, 29768,
                9232,  54328, 14436, 22892, 2292,  25676, 5664,  40012, 23760,
                9316,  2384,  22608, 59448, 55500, 3776,  60704, 47156, 18052,
                36436, 54232, 4040,  24080, 17864, 55960, 4560,  53452, 30388,
                58772, 8484,  63764, 22796, 16924, 60960, 24112, 19372, 32996,
                14960, 37752, 22352, 6972,  42264, 23772, 1500,  32900, 57384,
                18564, 31132, 38432, 8584,  19640, 35820, 10940, 32304, 14540,
                55284, 8192,  63788, 19696, 26860, 1536,  39900, 20808, 11220,
                59792, 40612, 18528, 2796,  50384, 59952, 15428, 27856, 56120,
                22444, 26528, 39036, 3200,  20488, 34436, 7668,  37556, 61172,
                55920, 4324,  62352, 50732, 18276, 42016, 55260, 26188, 20628,
                60176, 3028,  24516, 5880,  21768, 51644, 40668, 19552, 48832,
                36096, 41376, 8412,  31636, 3412,  54492, 63612, 41148, 25220,
                37344, 8488,  64556, 3528,  14760, 36440, 28228, 57080, 48576,
                11804, 30240,
            },
            {
                17820, 43568, 13568, 63496, 40620, 45936, 3524,  23660, 49696,
                936,   65256, 39456, 51180, 61384, 44744, 56952, 50632, 34716,
                46828, 53632, 42700, 6752,  37508, 20412, 41656, 12928, 27140,
                49796, 20740, 60608, 34728, 46552, 12000, 37192, 22204, 16604,
                1388,  34568, 42188, 4312,  46968, 34064, 53400, 12088, 51560,
                25900, 2216,  57928, 11224, 52560, 47912, 64592, 8440,  51476,
                43144, 3276,  61332, 54596, 46804, 64644, 43216, 60092, 3072,
                28132, 40404, 11148, 51464, 43044, 62104, 29908, 49556, 25100,
                35628, 8744,  28940, 54932, 34404, 42356, 21108, 32276, 50824,
                6352,  12832, 63360, 45448, 55448, 41604, 17520, 49736, 31144,
                9212,  46876, 25748, 43692, 724,   37244, 61140, 3988,  51680,
                14800, 43636, 53732, 39360, 46964, 8756,  56644, 26840, 6592,
                55412, 60820, 50484, 27692, 59820, 37048, 10720, 32684, 15124,
                59636, 28504, 47636, 39592, 50532, 62920, 42632, 2036,  40160,
                60164, 4804,
            },
            {
                22344, 56044, 34504, 2504,  50664, 27920, 59660, 38776, 43932,
                32984, 26736, 18604, 33976, 8444,  13516, 31188, 16500, 4272,
                28108, 14364, 64856, 25972, 50656, 15712, 63184, 896,   32152,
                58368, 6324,  15132, 40504, 364,   51440, 32804, 65504, 43920,
                50672, 20116, 12704, 27884, 58112, 1152,  39924, 44020, 4780,
                59904, 45864, 40648, 16864, 30572, 19280, 35492, 13628, 22560,
                59060, 16512, 34148, 23252, 29468, 1208,  17608, 49560, 57456,
                21092, 53464, 15384, 34080, 4328,  17552, 55396, 6372,  64676,
                2204,  45424, 22812, 12480, 47744, 3952,  62088, 10968, 40864,
                58916, 35564, 49604, 24292, 5064,  29284, 65020, 1804,  13832,
                40452, 20780, 34724, 12188, 53412, 21892, 30004, 10172, 40880,
                28824, 34748, 12192, 18124, 36460, 60432, 16260, 34624, 44200,
                13412, 1748,  18676, 39092, 15912, 7108,  47480, 53212, 21372,
                2372,  55556, 12680, 23240, 6900,  26096, 11132, 18792, 24440,
                51400, 36656,
            },
            {
                27448, 7436,  60584, 16108, 21032, 54700, 8180,  19492, 15488,
                52864, 11432, 5208,  42068, 55576, 21004, 37072, 62160, 58184,
                20056, 39260, 33016, 11040, 45680, 29964, 53132, 24532, 38692,
                11276, 44268, 52696, 30840, 62188, 25096, 7392,  13716, 39224,
                26228, 55656, 62420, 31712, 49828, 14628, 26564, 64492, 9880,
                20608, 28700, 34160, 63096, 3872,  44592, 55776, 39808, 29160,
                47148, 10844, 50200, 5460,  57024, 12820, 38704, 23804, 34456,
                6264,  46136, 30756, 24608, 58532, 9912,  37388, 46736, 14808,
                41884, 56424, 31804, 63112, 17100, 25020, 37528, 53572, 292,
                18744, 30212, 8428,  15672, 52068, 11068, 57148, 26372, 45028,
                54820, 59364, 6856,  64564, 31816, 15444, 47008, 62816, 6816,
                19520, 58832, 1428,  65240, 25040, 29840, 2312,  50688, 22404,
                32200, 25408, 64068, 45368, 56728, 23116, 29256, 4692,  61696,
                45672, 41816, 19196, 31428, 60504, 44556, 34512, 54408, 8524,
                31892, 45428,
            },
            {
                64168, 39236, 47096, 30624, 37684, 12324, 35352, 63676, 4276,
                57040, 46216, 60664, 28864, 47568, 1272,  26440, 10468, 43184,
                48224, 36,    61112, 17904, 3484,  35424, 57056, 5760,  48528,
                64936, 23052, 27564, 8876,  17484, 57224, 48940, 28896, 59380,
                5360,  9860,  37476, 44540, 5956,  21704, 55076, 30824, 36052,
                48432, 13892, 50760, 24564, 9000,  59368, 26080, 6364,  12,
                63972, 37136, 26968, 41432, 20360, 52340, 30916, 9216,  41732,
                60936, 1768,  65152, 39028, 44716, 52304, 22160, 33348, 26740,
                19380, 51852, 4796,  39188, 58088, 7484,  29684, 14224, 64424,
                47844, 43712, 57832, 32904, 38020, 42916, 22180, 36176, 5256,
                16612, 23944, 38600, 48800, 5084,  57128, 35916, 23564, 49984,
                55480, 45100, 31580, 49056, 6248,  42908, 11768, 62912, 58516,
                38064, 48088, 5656,  11176, 33888, 51448, 40420, 17600, 35764,
                26336, 9412,  34156, 53416, 508,   16308, 58468, 46988, 62332,
                14296, 1228,
            },
            {
                17016, 11092, 24812, 5424,  43268, 58736, 48080, 26436, 31284,
                20820, 37780, 23832, 16048, 64292, 50920, 40840, 54000, 6928,
                30360, 23360, 54908, 49752, 40772, 22276, 9792,  43296, 18988,
                34104, 2112,  59492, 37380, 45140, 21148, 41332, 2544,  18856,
                46360, 52900, 23880, 17684, 60716, 8868,  38396, 16968, 56524,
                6660,  61952, 972,   53920, 38112, 12528, 49888, 32592, 20080,
                52972, 14796, 7928,  61712, 44496, 3832,  63272, 47592, 15972,
                27520, 19332, 50124, 7500,  13776, 28576, 8,     59260, 11924,
                61708, 36520, 9556,  21240, 42968, 52568, 45628, 34924, 23328,
                27100, 4520,  20872, 62988, 1308,  18392, 60212, 47212, 33024,
                51504, 29688, 10068, 19108, 26472, 43996, 1524,  11452, 38252,
                26580, 16184, 9144,  21132, 56124, 53024, 40076, 19748, 3692,
                9504,  54128, 30080, 20484, 43312, 16,    65460, 12880, 57516,
                50476, 5200,  62800, 38084, 49164, 21856, 29852, 3892,  37240,
                20932, 53172,
            },
            {
                33568, 49948, 57592, 52600, 216,   22888, 6920,  13996, 40608,
                50520, 1728,  34824, 7220,  12456, 32376, 22488, 17592, 63748,
                13248, 36668, 8784,  27452, 60060, 14152, 62664, 29136, 15504,
                55164, 41188, 12864, 48232, 5024,  33596, 10564, 62616, 36032,
                30956, 15296, 1792,  34424, 41368, 51220, 46280, 3820,  23752,
                43284, 39688, 18064, 31720, 21668, 42248, 16724, 61004, 45092,
                40408, 24648, 58084, 32104, 35864, 14432, 25904, 58340, 36936,
                53856, 11892, 23244, 34956, 60376, 18824, 54776, 38856, 48332,
                3228,  30164, 49212, 15048, 27524, 2156,  61236, 17744, 9320,
                55432, 40388, 13012, 54116, 25664, 49016, 6676,  12696, 63572,
                212,   42792, 61820, 52736, 40040, 60320, 17576, 32880, 63980,
                5436,  41728, 61008, 37264, 27376, 14572, 33236, 46288, 28580,
                16836, 41404, 61828, 14488, 58872, 26820, 8156,  31112, 44108,
                23500, 15904, 56020, 11572, 27024, 7744,  40664, 12260, 25400,
                42408, 58936,
            },
            {
                8720,  40784, 19468, 28344, 65112, 33260, 54136, 44692, 62292,
                10464, 59688, 54240, 45104, 58632, 38820, 4432,  46404, 34188,
                51604, 58592, 44788, 4640,  32488, 47596, 1452,  37212, 50696,
                7244,  21636, 31824, 64004, 15952, 56036, 50960, 22844, 54272,
                42788, 63824, 48660, 57612, 25264, 12656, 28448, 61360, 32552,
                11776, 58576, 26208, 47264, 65400, 5520,  35136, 54732, 11264,
                30300, 2576,  18724, 49260, 10300, 55160, 21756, 468,   7972,
                43652, 31884, 56428, 46848, 3808,  42332, 31264, 7172,  44152,
                17000, 25368, 57200, 64884, 33644, 11240, 50032, 38900, 3192,
                60048, 29580, 35676, 44908, 9944,  31180, 40708, 28340, 20524,
                57844, 14768, 34896, 3084,  12852, 29356, 48396, 54664, 20760,
                52512, 13280, 34088, 46796, 316,   63744, 8344,  24448, 56772,
                49624, 34420, 22636, 4920,  37868, 54748, 48292, 19416, 37368,
                1680,  29156, 46540, 18784, 43360, 65096, 52740, 56848, 48684,
                5736,  29088,
            },
            {
                23364, 2708,  36216, 11828, 15716, 39000, 18428, 28876, 3112,
                24752, 17248, 30132, 20020, 26120, 9328,  55452, 28404, 2536,
                20696, 25708, 16016, 38888, 19760, 54160, 24160, 59144, 45512,
                26764, 57560, 52276, 3156,  29288, 25372, 38756, 13992, 7128,
                27136, 11528, 4244,  30232, 19284, 64964, 268,   53308, 20392,
                50164, 7980,  36760, 14612, 56844, 2404,  27940, 22924, 7716,
                63364, 46856, 54024, 4504,  28492, 40092, 45836, 33612, 49780,
                63632, 4704,  39728, 16348, 62272, 25572, 10672, 64148, 22400,
                53908, 40340, 6148,  46112, 23164, 55268, 19888, 31040, 42708,
                48536, 16808, 5868,  51708, 61504, 15760, 55832, 50092, 8252,
                37808, 25892, 46504, 22240, 56148, 7568,  24848, 2556,  45472,
                30024, 3908,  57196, 23340, 18144, 51328, 39124, 5252,  61196,
                12120, 1416,  52796, 45148, 32236, 24304, 11676, 60040, 51820,
                63512, 40588, 33192, 59940, 3188,  31100, 15560, 35036, 19020,
                63072, 44984,
            },
            {
                54912, 61076, 48928, 45664, 59152, 9012,  50112, 57216, 36852,
                48604, 43212, 5308,  52132, 41192, 63252, 15300, 37040, 61092,
                42472, 10704, 55608, 65012, 8004,  42052, 11948, 31272, 4204,
                10508, 38908, 17696, 42956, 9072,  61212, 44412, 1600,  57924,
                34860, 21352, 40312, 55648, 8120,  37100, 42388, 15580, 35212,
                45308, 3312,  29748, 52380, 10580, 43132, 59804, 47748, 38740,
                15852, 34520, 42024, 23484, 64640, 16652, 60228, 12500, 19176,
                24792, 28832, 9344,  51444, 20668, 37032, 49592, 15272, 35168,
                60592, 32444, 13240, 1064,  42028, 36300, 6852,  58192, 25508,
                11756, 65280, 24176, 20300, 38844, 2208,  34960, 23552, 44816,
                53196, 4104,  31620, 50932, 64984, 37636, 42604, 15292, 61772,
                39788, 25988, 50456, 10124, 44176, 58724, 29600, 20928, 43604,
                36924, 25724, 64232, 9136,  17152, 40952, 2584,  34312, 6212,
                14732, 9728,  22548, 6932,  50536, 23844, 39064, 9472,  868,
                32216, 13636,
            },
            {
                16528, 27048, 6732,  31920, 24220, 42540, 4984,  21112, 12584,
                31888, 64812, 14516, 35508, 432,   47460, 23888, 50560, 5960,
                31460, 48712, 832,   34328, 27880, 51848, 17236, 63816, 20588,
                34780, 62388, 23840, 47348, 35956, 19864, 32164, 49816, 17988,
                47020, 60540, 51584, 16180, 32892, 48132, 58056, 5352,  27424,
                59668, 63324, 23120, 40180, 17296, 31324, 50512, 19588, 1040,
                26616, 59020, 12920, 7368,  36556, 51268, 3580,  30100, 57372,
                41772, 14096, 59464, 33696, 1008,  45240, 57732, 27704, 2516,
                9360,  19608, 50840, 61852, 17340, 28412, 63332, 14636, 52532,
                37272, 644,   33508, 47464, 27592, 58096, 4968,  64552, 16908,
                11672, 60956, 19460, 9460,  16388, 33660, 57876, 11136, 35536,
                7876,  16768, 64752, 31664, 3424,  12700, 35100, 54360, 15676,
                47680, 30884, 18984, 50148, 56052, 61248, 46252, 21700, 27584,
                57968, 49136, 54992, 36628, 13208, 57836, 46896, 26668, 60548,
                51424, 37776,
            },
            {
                64440, 42152, 20548, 52160, 1348,  63256, 34888, 27216, 60552,
                7832,  39700, 22616, 56960, 11748, 32264, 19192, 57584, 12604,
                38100, 18196, 59864, 22016, 46820, 5164,  36208, 43988, 55572,
                49788, 14580, 256,   59596, 12664, 53720, 4364,  65124, 29028,
                9256,  37436, 25712, 5640,  44576, 21000, 10680, 24100, 40516,
                13392, 19076, 54540, 46492, 6624,  25104, 36344, 12308, 62448,
                51940, 30708, 21288, 56972, 44016, 26100, 8928,  38616, 52660,
                2488,  48096, 22736, 53944, 29716, 12840, 5668,  52580, 42196,
                47684, 29224, 37792, 25076, 53724, 10132, 46568, 3680,  21988,
                44416, 57684, 54280, 8076,  13580, 43624, 19296, 32520, 41704,
                27112, 55128, 39416, 44356, 28212, 4784,  50232, 23296, 46996,
                59600, 20164, 42420, 36664, 48836, 26712, 62408, 2328,  9956,
                59440, 6360,  39192, 27952, 4092,  15204, 29920, 53320, 38864,
                43936, 19716, 25844, 42596, 63924, 2380,  19928, 53936, 41028,
                22280, 4608,
            },
            {
                10136, 34008, 57000, 12924, 29304, 17388, 53960, 45192, 50720,
                1532,  54612, 46868, 28680, 60264, 43720, 7772,  27300, 62720,
                44228, 29656, 52884, 40392, 13812, 57756, 24876, 2100,  28764,
                8452,  40640, 31500, 50928, 27708, 7584,  39600, 24400, 14388,
                43396, 748,   63412, 12536, 53664, 29228, 62548, 50488, 55380,
                32260, 9532,  980,   34480, 57708, 64088, 3948,  55780, 40924,
                9160,  45500, 5152,  49488, 32680, 19784, 62636, 45404, 34332,
                17392, 64816, 36612, 7316,  41048, 62792, 17480, 38116, 24124,
                59108, 6692,  56512, 44612, 4576,  39976, 34336, 50368, 31760,
                9396,  18468, 29060, 40224, 62008, 52292, 10484, 48736, 58588,
                1696,  35680, 5432,  48076, 21724, 62996, 456,   31972, 27516,
                52028, 5768,  54220, 14520, 22080, 39760, 52328, 18488, 33080,
                42252, 21720, 57472, 12900, 43392, 35724, 8056,  64912, 12092,
                680,   31728, 4568,  16380, 33648, 28420, 44388, 7140,  12224,
                29664, 47240,
            },
            {
                18480, 2824,  38804, 60332, 48268, 40744, 9808,  14920, 37904,
                19188, 26404, 9312,  17716, 4812,  49856, 34844, 53840, 2324,
                23448, 15396, 4228,  9052,  32284, 61268, 38440, 16276, 47640,
                60240, 25492, 5552,  42536, 21924, 56856, 46108, 59092, 34100,
                54876, 22964, 31852, 41344, 59416, 3864,  35740, 17060, 2300,
                43872, 37964, 49204, 28408, 21224, 14392, 44740, 32916, 23744,
                18032, 37336, 61748, 16244, 692,   11304, 55876, 23092, 6164,
                27492, 11208, 43564, 19584, 25396, 55988, 32828, 21140, 65480,
                11112, 15848, 33072, 21944, 12564, 58856, 20148, 61100, 26328,
                41624, 63576, 15412, 2952,  21760, 35408, 25012, 7156,  29816,
                15040, 23148, 61908, 12452, 53520, 38316, 18608, 55964, 13032,
                38800, 2236,  28676, 57776, 8760,  4580,  44012, 24480, 65268,
                49404, 436,   52924, 32676, 62660, 20504, 49300, 25348, 17728,
                59084, 47804, 61156, 52184, 10452, 37572, 61928, 17380, 56888,
                35988, 50480,
            },
            {
                44080, 54812, 23000, 7164,  33000, 25448, 3880,  59016, 23512,
                63388, 33668, 42764, 61764, 38988, 22048, 14312, 40852, 10084,
                46580, 56588, 64180, 26364, 49972, 20348, 7232,  53676, 11200,
                33884, 17768, 54300, 63200, 14996, 36884, 18804, 10936, 3444,
                47940, 16716, 50640, 19392, 26808, 38788, 8692,  47280, 29784,
                64436, 25256, 61020, 10960, 41320, 52868, 8160,  29240, 48752,
                2884,  58376, 28012, 40020, 53424, 42760, 29576, 14716, 60576,
                46844, 54744, 1604,  60980, 49268, 9836,  46516, 2104,  44068,
                35884, 53184, 580,   63720, 49040, 30136, 16272, 1464,  55636,
                5476,  36336, 50780, 45904, 30544, 56488, 38504, 63068, 45696,
                51916, 41096, 56948, 30828, 7096,  42860, 60124, 8944,  44896,
                21808, 62760, 34324, 46212, 61280, 16040, 56640, 29464, 11692,
                37468, 26964, 8776,  24056, 46888, 2828,  39896, 56604, 5588,
                34792, 38360, 27772, 7724,  56352, 21596, 49420, 32252, 4140,
                59620, 25656,
            },
            {
                62828, 28480, 15752, 46264, 65000, 19828, 55420, 30056, 44204,
                2844,  12792, 51984, 30456, 2056,  55684, 65396, 29424, 19836,
                50996, 33448, 36632, 43052, 12128, 45756, 30688, 40944, 65252,
                21440, 37472, 46640, 10196, 33168, 4916,  52056, 30552, 40472,
                60752, 8256,  57548, 2360,  45652, 14748, 23440, 56764, 20492,
                7044,  51784, 16416, 4368,  58764, 37560, 19316, 61532, 15284,
                54600, 34060, 12676, 22456, 65292, 35728, 48564, 2744,  37660,
                20496, 33004, 30228, 16156, 38964, 5324,  28336, 14160, 50728,
                30260, 19008, 41456, 25588, 36596, 7296,  44000, 38588, 13312,
                47724, 23920, 11016, 59972, 6260,  17276, 388,   12828, 20800,
                33908, 9088,  17944, 26388, 35080, 14888, 24736, 33340, 49264,
                16964, 41776, 12064, 24772, 31012, 49960, 36004, 7000,  46664,
                14880, 60976, 41672, 17308, 58564, 11292, 31196, 52436, 44820,
                23232, 13100, 19256, 41116, 46224, 13856, 332,   42624, 23520,
                14396, 7920,
            },
            {
                37480, 11248, 51688, 500,   36784, 12440, 49664, 34588, 6136,
                47820, 56880, 36516, 16088, 24744, 47088, 6488,  37260, 59680,
                25416, 7056,  17244, 3088,  54688, 62696, 1116,  24416, 50624,
                4644,  57704, 2252,  29308, 59388, 43852, 25336, 64268, 20828,
                27480, 37872, 34580, 11416, 65008, 52552, 41104, 60032, 34644,
                13656, 41960, 22520, 31900, 47436, 25580, 540,   51292, 42656,
                10224, 25824, 47328, 7720,  3976,  18476, 24616, 57852, 41444,
                7776,  50308, 58796, 12284, 35360, 52420, 61744, 57916, 24544,
                4064,  60748, 46180, 9528,  56840, 51396, 23056, 64916, 28520,
                53056, 18952, 33536, 26560, 54044, 43812, 49344, 59544, 27916,
                4300,  65520, 49956, 1192,  46632, 52348, 64188, 2748,  29780,
                59052, 6844,  51756, 1032,  19164, 40984, 22644, 2436,  58392,
                20248, 54172, 34220, 5320,  50872, 37056, 27008, 15472, 9024,
                63880, 48912, 1964,  62468, 24332, 30852, 54696, 65340, 39460,
                52984, 33084,
            },
            {
                20892, 41608, 59752, 31116, 43856, 8072,  61792, 41552, 17504,
                28076, 21396, 7432,  58460, 10616, 32992, 17612, 43588, 1296,
                12824, 41580, 58236, 22816, 28596, 18584, 35936, 14756, 44660,
                27900, 12936, 41584, 23396, 49988, 16816, 960,   54580, 13048,
                6032,  44708, 24652, 48884, 28496, 4936,  31192, 10596, 2980,
                54420, 45848, 36416, 56360, 12484, 65104, 35032, 6748,  39212,
                20216, 63376, 43868, 56320, 50904, 31952, 9692,  52516, 13004,
                64264, 23460, 3840,  26700, 43864, 22024, 17256, 41088, 8896,
                38348, 55200, 13880, 28320, 17948, 2924,  32612, 58436, 8284,
                42524, 2048,  62688, 41196, 8908,  32400, 23072, 40108, 55376,
                16132, 42988, 37248, 59336, 20256, 11080, 40972, 6172,  54156,
                36084, 26612, 64944, 38300, 55296, 10204, 63604, 51128, 32244,
                39608, 3468,  28724, 45296, 64568, 19344, 932,   59728, 41956,
                29120, 53756, 32548, 57508, 4904,  36244, 27352, 10156, 18592,
                46560, 2128,
            },
            {
                49248, 6088,  24668, 18152, 57812, 27016, 22272, 53248, 10092,
                64328, 50448, 39644, 44528, 63076, 52608, 27360, 55108, 48588,
                63472, 30196, 52528, 38880, 48484, 5712,  59644, 55932, 8904,
                32604, 53048, 19264, 63904, 38936, 8616,  47068, 31896, 41748,
                50856, 62960, 15376, 20152, 55272, 36184, 17408, 49512, 25640,
                63440, 28116, 1596,  18404, 8472,  29624, 45080, 23060, 59040,
                31248, 1872,  36984, 28820, 13700, 39648, 61108, 26552, 34464,
                17148, 45632, 39892, 53780, 63536, 272,   31216, 34492, 48680,
                20964, 6080,  33428, 62844, 40412, 46776, 11808, 20316, 35116,
                15068, 57176, 37356, 21056, 13916, 64400, 5312,  35920, 10364,
                47708, 30216, 23496, 5772,  55796, 28928, 37908, 22916, 19100,
                47236, 9624,  15748, 32848, 45260, 13988, 27188, 43788, 17212,
                25048, 62868, 9308,  14180, 23280, 42920, 55040, 33752, 21272,
                3920,  17280, 39668, 11684, 50264, 16268, 45464, 6540,  61016,
                29876, 57152,
            },
            {
                16084, 63292, 34120, 52748, 3384,  39616, 14964, 4492,  38076,
                32504, 13460, 26068, 108,   20176, 35836, 13936, 4464,  22432,
                32816, 8280,  15464, 61756, 10248, 33512, 42260, 21008, 38668,
                61864, 47948, 7212,  33996, 14088, 28152, 61292, 35516, 18284,
                3220,  56220, 33396, 720,   42928, 7792,  62396, 44348, 19680,
                38884, 5892,  59216, 49652, 40536, 52488, 16224, 54944, 13156,
                49500, 5596,  17416, 58484, 21292, 45980, 5952,  43496, 872,
                29632, 56564, 10024, 19444, 47852, 7696,  55972, 12024, 64288,
                27324, 43976, 52496, 23588, 4868,  60464, 54224, 26088, 49820,
                45304, 29952, 4352,  51040, 47036, 29264, 18696, 52268, 26116,
                2296,  61484, 13512, 33076, 44220, 15964, 50244, 58124, 13336,
                42820, 56492, 23680, 49396, 3376,  59956, 35112, 7684,  53512,
                12112, 47468, 35648, 56928, 30668, 7204,  12540, 48228, 37860,
                61088, 6980,  26272, 43764, 20448, 59976, 38520, 22672, 51080,
                12472, 26420,
            },
            {
                38612, 45288, 9612,  13436, 48000, 29652, 56248, 46192, 23948,
                54804, 58964, 5212,  30592, 49620, 41200, 8900,  61428, 38180,
                18768, 46668, 2580,  25228, 44864, 13528, 51792, 26476, 3772,
                16916, 24688, 58260, 1664,  51216, 55740, 22340, 10964, 58756,
                23692, 29796, 10372, 61404, 38424, 23160, 57792, 12772, 33664,
                52880, 15572, 23932, 32972, 61584, 21360, 2320,  27872, 33812,
                61276, 24372, 53800, 9004,  34940, 63696, 16216, 54584, 20768,
                49000, 36496, 5196,  32836, 15000, 25148, 42328, 51620, 2856,
                16720, 59384, 10764, 36160, 15344, 29856, 37544, 204,   6904,
                59812, 17940, 11680, 25000, 56008, 1336,  60556, 44560, 57536,
                20640, 39348, 53932, 9440,  62644, 25960, 1824,  63716, 31940,
                5216,  28860, 61732, 20616, 6288,  29556, 56328, 21824, 696,
                41360, 59576, 20856, 49812, 40208, 52704, 62140, 25272, 10040,
                46124, 51420, 34672, 64500, 8640,  55612, 31628, 1552,  35052,
                43376, 3716,
            },
            {
                30992, 55368, 22512, 36472, 64008, 20284, 60372, 34400, 2120,
                19000, 42312, 47444, 17076, 59768, 23784, 56024, 29204, 51196,
                42736, 57904, 35304, 20772, 56744, 31364, 152,   64708, 46060,
                30584, 11796, 36660, 42484, 19764, 5372,  44320, 38236, 7144,
                48284, 40196, 46080, 52120, 14076, 27240, 47512, 1932,  29540,
                9116,  42600, 47076, 10856, 37668, 4840,  57760, 41164, 10664,
                46368, 38080, 42380, 27276, 49304, 3584,  30876, 38416, 59764,
                12396, 63204, 27472, 58152, 38580, 60064, 29328, 20192, 36748,
                46344, 31096, 1080,  41708, 50900, 19156, 43776, 63736, 22684,
                38972, 53396, 62256, 43188, 34444, 38120, 15812, 12052, 33472,
                7416,  51136, 18216, 46328, 4800,  41600, 35244, 9156,  45612,
                37616, 616,   51988, 39320, 42348, 16592, 48772, 37596, 62228,
                31328, 26844, 5896,  2072,  16488, 27976, 3164,  18576, 31108,
                57908, 22780, 13320, 29932, 3104,  47652, 14852, 53076, 64212,
                19592, 59288,
            },
            {
                50620, 5140,  28176, 41668, 908,   8568,  43292, 11920, 51872,
                29008, 8208,  62296, 34692, 11704, 6048,  45220, 16324, 892,
                10860, 27160, 64200, 50268, 7532,  37816, 18072, 54532, 6244,
                41204, 60692, 53448, 26980, 46696, 31136, 15100, 65508, 51624,
                27560, 16980, 5032,  20700, 31960, 53980, 4656,  40836, 60804,
                55580, 20644, 64596, 30656, 13884, 26232, 47764, 64072, 7488,
                19660, 15024, 44,    59520, 12268, 23568, 51640, 8732,  25472,
                3916,  41176, 22292, 51252, 11120, 45568, 6328,  61672, 13860,
                55016, 25836, 21252, 65100, 55828, 8036,  12844, 48208, 33692,
                9764,  26996, 31744, 5684,  21764, 8496,  50036, 23860, 42172,
                64688, 27428, 36620, 30924, 60188, 20844, 53228, 17396, 25212,
                60420, 14380, 33932, 8432,  64332, 25356, 45572, 9948,  14712,
                50796, 18088, 46028, 38192, 64032, 34012, 44936, 56016, 14652,
                40508, 1172,  53956, 37280, 24976, 41644, 28184, 10644, 40492,
                24708, 8332,
            },
            {
                14168, 61784, 17624, 58548, 50988, 32080, 17048, 26596, 64784,
                36720, 15184, 22452, 40016, 52848, 32580, 65052, 36376, 24420,
                54568, 14060, 5232,  39840, 28936, 60008, 23432, 49316, 35524,
                14736, 22044, 9408,  3308,  63156, 56696, 25536, 320,   33268,
                12416, 60440, 57012, 35804, 8456,  64452, 18040, 34828, 48868,
                25116, 6276,  372,   57272, 51896, 43196, 16676, 35388, 30272,
                52244, 62824, 32648, 44920, 39784, 18524, 56956, 33036, 47164,
                15532, 53536, 44304, 1072,  17984, 34720, 23640, 3800,  39556,
                9056,  58300, 48568, 5652,  32692, 24988, 28784, 57464, 15788,
                41688, 2396,  48980, 16960, 58624, 40336, 61992, 30748, 3092,
                47580, 13868, 40,    23972, 56672, 12848, 28704, 49552, 56128,
                22008, 47864, 18864, 54244, 12352, 31064, 57588, 4852,  23800,
                34488, 55328, 11540, 57644, 23480, 8832,  51072, 36820, 6104,
                63592, 19936, 44468, 58940, 17536, 62016, 6836,  58156, 16796,
                48428, 33964,
            },
            {
                21156, 38912, 46744, 11188, 23700, 54704, 39476, 6528,  48252,
                56368, 45044, 1644,  57596, 26784, 4136,  20784, 48196, 58744,
                41032, 33368, 19832, 47528, 16096, 11864, 43308, 8752,  28036,
                57476, 47544, 32500, 39312, 12748, 35684, 18720, 41640, 54304,
                21740, 43592, 2200,  25444, 39384, 44928, 22856, 9960,  13496,
                37444, 28616, 45588, 34284, 18948, 6552,  22524, 50040, 2528,
                25184, 55512, 21524, 6156,  29272, 64968, 1548,  42872, 62136,
                19820, 35320, 30700, 8340,  65288, 48148, 54284, 32600, 50464,
                43340, 17616, 34612, 12168, 45820, 40200, 61432, 5040,  52112,
                20836, 56196, 65056, 44376, 11184, 52520, 26788, 6092,  56272,
                17084, 59008, 41020, 10296, 48036, 38232, 2540,  7024,  40276,
                11000, 43024, 28004, 59136, 36548, 1472,  20076, 52688, 43728,
                65220, 7756,  42340, 32188, 13676, 59664, 20580, 42668, 25896,
                33028, 48592, 9524,  4332,  50648, 21940, 32856, 45576, 37568,
                24,    54204,
            },
            {
                43808, 2572,  26500, 34736, 3992,  44712, 60680, 20776, 3024,
                24336, 31640, 10164, 50108, 18112, 43780, 13428, 8200,  30052,
                2984,  44924, 61356, 52396, 1980,  55804, 63192, 32876, 19512,
                1740,  62540, 17772, 50992, 23892, 7940,  49372, 61964, 9108,
                37156, 29636, 63392, 50612, 15564, 55716, 30488, 58336, 51428,
                62360, 16240, 40548, 10260, 60888, 54872, 39292, 59584, 13456,
                42976, 37112, 10504, 50876, 46440, 14460, 36312, 10752, 27768,
                6412,  59536, 13260, 56048, 26416, 40520, 15424, 62752, 20712,
                27760, 2260,  38524, 62288, 21984, 1408,  17852, 37116, 46364,
                30640, 13740, 35832, 23424, 568,   32920, 19104, 45928, 37748,
                21968, 34248, 50468, 63344, 16680, 43512, 33588, 65108, 30304,
                61328, 3408,  6072,  23336, 50228, 41216, 60596, 27572, 38708,
                2820,  29604, 19752, 52948, 20,    27620, 47932, 3720,  12040,
                60356, 28512, 15656, 35224, 43088, 11736, 56296, 4616,  27072,
                63532, 29900,
            },
            {
                10280, 50076, 65088, 57004, 15756, 29552, 12576, 35272, 51404,
                40912, 13588, 63656, 37196, 29516, 60880, 39392, 53392, 63348,
                17628, 9840,  23608, 30988, 36200, 26408, 4684,  39088, 52896,
                44756, 26032, 5400,  54968, 44064, 59840, 4112,  27196, 15468,
                47808, 5764,  19312, 11984, 46932, 6348,  1052,  41808, 19768,
                3676,  54320, 23068, 48364, 26876, 3848,  32100, 9484,  28324,
                57308, 4348,  17588, 60512, 26124, 54720, 22188, 49516, 55540,
                40360, 24152, 50548, 37624, 21400, 3056,  11740, 29892, 6804,
                53372, 59712, 14684, 30072, 49712, 53236, 11124, 59220, 24144,
                4016,  8864,  39268, 28484, 47336, 55548, 12948, 63672, 9544,
                53296, 28024, 7540,  31800, 4372,  26300, 54424, 19504, 15020,
                36020, 52424, 32404, 46092, 15416, 7468,  33500, 16984, 10400,
                48876, 59172, 25060, 46420, 39792, 35396, 62600, 17680, 54396,
                39424, 23388, 52008, 65120, 30140, 39804, 19172, 52264, 22824,
                13180, 59568,
            },
            {
                18760, 32436, 7372,  21656, 37972, 47904, 63224, 9224,  28268,
                59540, 19636, 53684, 7084,  47404, 1020,  22116, 34876, 50380,
                27744, 38344, 58052, 6864,  41952, 21556, 47168, 13940, 59684,
                11288, 36980, 41308, 14068, 28916, 20276, 38632, 45420, 31652,
                59000, 52888, 41392, 33828, 28088, 59980, 36700, 26152, 33328,
                43944, 7828,  31524, 63568, 12992, 36828, 44164, 65428, 19136,
                46768, 40556, 30908, 34968, 8088,  3044,  39300, 31904, 4652,
                17156, 45940, 1952,  61416, 32112, 43004, 56716, 46656, 37028,
                10476, 45324, 25608, 55624, 8408,  42164, 27800, 34232, 43260,
                63108, 50152, 58452, 18060, 61408, 4912,  35412, 24424, 42684,
                3616,  45096, 19844, 61224, 23044, 58064, 51100, 9392,  44780,
                25388, 56692, 17884, 64076, 11168, 54956, 45036, 63048, 22072,
                13948, 36280, 5504,  63680, 15944, 6712,  51632, 30116, 8652,
                46056, 792,   56088, 7320,  25332, 2084,  62552, 35852, 47232,
                5796,  41260,
            },
            {
                37200, 55344, 13756, 41936, 52964, 5632,  25036, 57604, 16480,
                43352, 4360,  33264, 25920, 16052, 58568, 11412, 25152, 4012,
                14600, 46600, 54404, 12720, 65432, 16744, 51188, 29620, 24164,
                7748,  57084, 31140, 64848, 1156,  33856, 57368, 11508, 22676,
                1520,  24980, 9648,  62456, 21692, 52400, 17128, 65148, 11696,
                50500, 59724, 38920, 1232,  21244, 52448, 15900, 49224, 1956,
                23368, 61760, 15064, 53384, 43348, 63008, 18388, 58960, 12856,
                63996, 29172, 10056, 18860, 48920, 5612,  59436, 18164, 24456,
                64216, 40832, 4964,  18356, 36112, 64532, 14228, 6720,  19572,
                53948, 26200, 2792,  42336, 14680, 51340, 40132, 29420, 60088,
                15460, 55732, 11996, 39764, 46616, 14012, 2028,  37372, 22300,
                48560, 344,   39936, 29032, 37064, 24948, 2172,  30128, 51696,
                56876, 41984, 18544, 54144, 10636, 22580, 43772, 33652, 58540,
                20980, 36712, 13084, 18496, 44732, 49352, 14772, 8468,  31048,
                57364, 25204,
            },
            {
                3128,  46476, 27948, 61288, 536,   31840, 19064, 46012, 2016,
                38228, 49348, 22304, 62000, 44872, 41000, 32020, 56412, 43272,
                61916, 21380, 424,   33480, 44384, 9188,  60840, 1324,  34772,
                50028, 17004, 22112, 47880, 10188, 52432, 17204, 50068, 64344,
                35200, 56660, 38492, 4292,  13912, 43208, 8528,  47976, 23852,
                29100, 14908, 18400, 45856, 56100, 29868, 7552,  25120, 33920,
                54456, 5820,  48792, 21852, 12048, 29444, 25208, 47456, 34236,
                52728, 43724, 35960, 54552, 14172, 27372, 35172, 50800, 248,
                33064, 21072, 48776, 31848, 3476,  23268, 56612, 47152, 31684,
                16208, 10220, 34800, 22332, 31564, 7728,  20868, 49012, 956,
                32596, 37068, 51536, 5528,  35040, 28984, 41692, 59776, 5808,
                62836, 13488, 8476,  59496, 49608, 19092, 40560, 6492,  47032,
                25956, 1360,  32528, 28728, 49156, 61576, 25456, 13604, 3456,
                63460, 42272, 31256, 61324, 34244, 54996, 27436, 60004, 43572,
                16228, 51832,
            },
            {
                62860, 8972,  23604, 17188, 49444, 35776, 40436, 53864, 30848,
                65356, 12280, 35700, 8920,  54764, 4980,  19412, 48880, 7304,
                35944, 29732, 51716, 18712, 27248, 37080, 55892, 40812, 19420,
                63552, 42856, 3068,  35840, 60792, 26196, 40324, 6356,  42740,
                14524, 18368, 46016, 49720, 30184, 56332, 39184, 5000,  54504,
                2196,  36148, 57900, 25768, 10756, 41928, 62212, 58136, 39720,
                10916, 27840, 36692, 1284,  57888, 51596, 41300, 8952,  392,
                21684, 25604, 6728,  39704, 64652, 22944, 9248,  41740, 16028,
                52712, 12444, 57520, 61452, 44324, 39468, 51332, 636,   60072,
                37836, 48924, 57320, 44800, 63968, 54140, 11388, 57736, 18412,
                65524, 23468, 26768, 57260, 18028, 64208, 10776, 49920, 26540,
                31104, 35228, 43332, 21844, 4500,  56080, 62164, 34392, 9480,
                38292, 59892, 12668, 44972, 35872, 2020,  56688, 38216, 17264,
                49640, 27096, 9904,  47644, 4536,  22120, 10768, 37680, 844,
                21204, 34000,
            },
            {
                40576, 30416, 57972, 44288, 11884, 56776, 8608,  14128, 24016,
                6776,  56244, 27968, 50908, 14044, 30012, 38008, 64780, 16848,
                10080, 41468, 59808, 5532,  48992, 22172, 4212,  31860, 46100,
                6016,  28264, 53964, 15320, 44424, 8016,  21176, 30324, 3700,
                54716, 27476, 32708, 59740, 104,   25576, 34264, 19524, 44956,
                62092, 41084, 6780,  32844, 50364, 4820,  35848, 20196, 13900,
                43912, 63608, 18272, 46064, 32948, 6044,  16004, 56116, 37736,
                62208, 15600, 58420, 30608, 45684, 3684,  60636, 29708, 55900,
                7120,  38368, 28236, 9500,  16484, 26692, 11820, 21428, 29040,
                41428, 6448,  12572, 27684, 1560,  36408, 25428, 43304, 6628,
                46068, 9884,  41336, 2716,  47952, 21728, 33228, 15420, 54988,
                18796, 46488, 53740, 27444, 14720, 31764, 11976, 52756, 16316,
                20636, 64732, 4696,  55364, 20008, 8852,  41716, 29460, 52856,
                5708,  57768, 24200, 38960, 58816, 17192, 41720, 53748, 64836,
                49636, 12276,
            },
            {
                54476, 19740, 34544, 6232,  64464, 21544, 28964, 60968, 48444,
                20144, 44312, 916,   39848, 21468, 59512, 2532,  52456, 26992,
                22820, 57100, 14020, 39540, 62924, 11476, 52944, 14916, 24920,
                56576, 12544, 38168, 23592, 32212, 56192, 63328, 36492, 46760,
                60732, 11032, 7200,  22808, 16936, 63148, 10920, 58956, 31152,
                22004, 13144, 53040, 64544, 22840, 17208, 46728, 284,   32064,
                50256, 8144,  59124, 39132, 24580, 64832, 20732, 44840, 30996,
                11688, 47832, 2872,  19724, 52036, 12184, 37944, 44484, 25420,
                62628, 46424, 22828, 1580,  54060, 33484, 62384, 45796, 4596,
                65156, 23908, 53080, 18844, 59800, 47792, 16172, 38676, 28516,
                53564, 14212, 62144, 30496, 8228,  53292, 44680, 1132,  37812,
                58948, 10376, 2460,  65308, 48204, 42020, 23024, 58456, 27876,
                43248, 49476, 23824, 39836, 14912, 60756, 48248, 22096, 10804,
                44448, 35468, 15668, 2428,  51652, 29508, 33240, 6736,  24660,
                29072, 4992,
            },
            {
                15096, 48184, 2264,  38836, 26604, 50864, 2644,  41512, 36256,
                58828, 32712, 17528, 63752, 46784, 25088, 11004, 33560, 43824,
                47700, 2176,  31736, 24444, 45436, 29384, 58536, 35552, 8860,
                61776, 50628, 4120,  59228, 48824, 460,   13792, 53364, 24700,
                19884, 51076, 40996, 45144, 36724, 51868, 42460, 15276, 49832,
                8684,  27396, 47524, 2808,  38296, 28888, 61000, 52556, 26340,
                56208, 22312, 30328, 14360, 10524, 49060, 35372, 4860,  59708,
                50884, 26816, 42044, 55840, 33572, 17224, 49324, 20540, 2604,
                13712, 34744, 41868, 60620, 49848, 37148, 7356,  18056, 54752,
                35528, 15612, 32884, 39816, 8820,  30800, 3924,  62892, 33740,
                21452, 49932, 36372, 16760, 59532, 25304, 40004, 61728, 7100,
                24204, 40632, 16944, 34200, 7808,  37784, 712,   46320, 5132,
                35576, 8048,  30964, 52316, 27408, 34020, 4248,  65456, 32632,
                54628, 19776, 63848, 8360,  43128, 61868, 12808, 47268, 18896,
                44092, 61124,
            },
            {
                26044, 59180, 42796, 53628, 16496, 33176, 46548, 5728,  15644,
                9576,  26312, 53596, 5332,  34984, 42292, 15632, 55984, 5792,
                61552, 37552, 54792, 8596,  16560, 912,   41824, 48136, 17908,
                39996, 30364, 20380, 10436, 41424, 18176, 28548, 9964,  38876,
                2964,  33924, 65040, 13504, 55104, 6424,  28144, 3852,  37876,
                55436, 34932, 19236, 42996, 14648, 57440, 11144, 40368, 5524,
                16788, 3416,  42556, 52936, 2052,  55400, 28500, 17932, 39576,
                22440, 7616,  36452, 10256, 24540, 63424, 6256,  57652, 31696,
                51840, 18432, 5420,  30020, 20704, 14188, 57752, 25500, 43160,
                49252, 2040,  61232, 46104, 21872, 55700, 50396, 11952, 59156,
                6496,  1540,  55976, 43552, 32536, 4296,  13028, 28652, 20812,
                50976, 30648, 56920, 52484, 25268, 59704, 18128, 30292, 54768,
                13472, 59200, 17648, 1016,  57572, 45776, 18224, 26292, 13876,
                300,   40468, 30520, 48904, 26128, 20656, 4208,  58348, 38980,
                8780,  36424,
            },
            {
                32192, 11228, 23100, 7864,  63028, 12712, 55384, 24872, 62608,
                51208, 38956, 13300, 58008, 8492,  50136, 62500, 28828, 20920,
                12476, 18444, 49992, 64524, 34272, 20464, 60580, 26672, 6416,
                44768, 33816, 65200, 27208, 46208, 34812, 50740, 62304, 43460,
                16576, 57376, 29176, 1860,  20864, 32116, 47376, 24704, 64244,
                1084,  59780, 10064, 30888, 54336, 7376,  24236, 45000, 65060,
                37384, 48436, 34620, 62680, 26076, 40736, 61132, 13372, 1432,
                54092, 62896, 29352, 59076, 656,   43328, 28100, 36768, 9932,
                64980, 48012, 58580, 11368, 44636, 4168,  39964, 31572, 8532,
                11640, 28508, 51780, 5784,  13960, 42112, 24428, 19128, 45124,
                40680, 27548, 23380, 10904, 19872, 51824, 56504, 34912, 45460,
                63264, 12580, 3604,  20480, 44464, 10908, 50416, 64140, 40052,
                25516, 45312, 63340, 37084, 11780, 7288,  40844, 51740, 58656,
                47040, 23628, 55744, 11500, 36844, 52812, 34408, 15592, 55532,
                49208, 1304,
            },
            {
                18180, 51244, 36952, 29532, 20168, 44032, 37644, 30312, 18704,
                43368, 1836,  28572, 22584, 31296, 18936, 380,   39912, 46184,
                35296, 29968, 4688,  25876, 39156, 51240, 3568,  55568, 22764,
                12904, 1632,  52368, 15936, 6600,  60344, 22164, 5564,  31828,
                49356, 8624,  23936, 39340, 61940, 10176, 58596, 18832, 40400,
                16200, 45352, 25832, 63312, 49188, 35504, 20452, 32868, 13080,
                28092, 60104, 18976, 6620,  45880, 21564, 8660,  47384, 34048,
                43704, 14424, 18508, 46940, 39412, 12876, 54384, 22392, 41312,
                15876, 26632, 38172, 24076, 52876, 64204, 48304, 21708, 62740,
                55868, 20244, 37440, 26004, 64128, 35104, 968,   56980, 31780,
                15724, 48372, 60496, 38464, 46304, 64356, 16140, 9152,  49180,
                5512,  42628, 37268, 28376, 61680, 33456, 14632, 22128, 3760,
                9668,  32660, 21084, 41924, 50016, 23004, 62308, 35120, 9496,
                37888, 6584,  60984, 17664, 1448,  44404, 62916, 27340, 30368,
                21620, 64056,
            },
            {
                45452, 3704,  60916, 47352, 57952, 184,   10388, 59836, 4960,
                34396, 56356, 49168, 45332, 60192, 36892, 52816, 24912, 7548,
                59844, 53940, 43276, 11020, 45852, 13696, 30864, 37296, 62684,
                49276, 57048, 36464, 24436, 54076, 39448, 2108,  58040, 26220,
                12932, 53976, 45488, 17508, 48756, 35368, 43408, 53348, 7852,
                33116, 51256, 21900, 4588,  39404, 1848,  59588, 47756, 8604,
                53524, 23204, 11344, 32148, 15696, 36396, 52224, 30636, 57284,
                23968, 50156, 3912,  31980, 52820, 16648, 61240, 47492, 1788,
                33516, 8024,  54848, 3036,  35140, 16708, 27420, 148,   34280,
                41120, 16804, 59276, 44508, 10544, 29076, 52832, 37676, 9920,
                64844, 3960,  34536, 7636,  29336, 2268,  41220, 26776, 18640,
                32652, 58636, 15200, 48724, 7132,  1944,  57660, 36788, 47556,
                52172, 16124, 5776,  55004, 28224, 14568, 31288, 3432,  20680,
                28996, 14904, 42660, 31916, 57720, 23200, 10160, 3356,  41892,
                12404, 53892,
            },
            {
                39928, 26760, 14948, 6056,  40584, 23992, 52384, 48384, 21264,
                64904, 11320, 16780, 40568, 4400,  14252, 10356, 65260, 49392,
                15168, 1488,  21936, 56720, 61380, 7456,  53688, 16336, 9768,
                28256, 18980, 43152, 9008,  13424, 29476, 44724, 18636, 37500,
                63912, 34460, 6152,  60144, 26720, 14240, 4976,  23320, 29016,
                12144, 58024, 41896, 13760, 18120, 29760, 52000, 16376, 41068,
                752,   56932, 43672, 51312, 58652, 2596,  64316, 19228, 5328,
                9700,  37364, 65408, 21344, 7496,  35208, 25436, 5276,  56652,
                45476, 60800, 19656, 43056, 30660, 9908,  59564, 51508, 13872,
                46752, 2908,  32096, 4736,  49432, 15404, 61200, 6832,  26084,
                51452, 21384, 54916, 13772, 50508, 24008, 36728, 60632, 54380,
                384,   25540, 22316, 55444, 40308, 19252, 29492, 42176, 26928,
                56568, 61888, 35784, 47100, 2232,  60184, 56228, 44728, 49196,
                64468, 53660, 25244, 48452, 6096,  39564, 51020, 46480, 58808,
                6916,  33316,
            },
            {
                62184, 10052, 56032, 31616, 17448, 34776, 27832, 14288, 41288,
                31452, 24548, 6392,  61936, 27052, 55464, 22148, 31812, 41784,
                27176, 38184, 33200, 16988, 28808, 24028, 42412, 46860, 33144,
                40228, 5628,  60812, 47648, 64108, 33496, 55820, 11164, 48260,
                21548, 2420,  41588, 30616, 51108, 1332,  55872, 37548, 62280,
                48300, 6456,  36320, 55664, 61768, 44420, 6160,  25968, 63852,
                30564, 38012, 4788,  27212, 39620, 24504, 11992, 42240, 27636,
                59292, 41304, 26352, 55712, 44128, 10552, 50844, 31228, 21124,
                12376, 28816, 50756, 14416, 57324, 39252, 44868, 6948,  25224,
                61540, 53156, 23528, 56316, 19004, 40456, 22260, 47184, 42280,
                17300, 30384, 39768, 63004, 19068, 57984, 6524,  43828, 11792,
                38648, 64504, 45336, 10648, 35364, 62708, 49836, 8000,  12836,
                88,    30492, 24020, 10384, 19532, 39244, 25460, 11352, 17556,
                40304, 2012,  34132, 12588, 63484, 19036, 14384, 25704, 37144,
                17160, 23964,
            },
            {
                44444, 36068, 21480, 50144, 54392, 63784, 8400,  58500, 3316,
                46928, 37788, 53360, 20132, 33592, 44044, 47772, 3120,  18292,
                57672, 9572,  63884, 50604, 5704,  36128, 20212, 64740, 116,
                57964, 25648, 31704, 3100,  21312, 16636, 50916, 6992,  40704,
                28688, 15452, 56852, 10656, 20416, 64648, 31496, 17056, 44652,
                19908, 480,   31944, 27200, 22868, 11404, 54728, 34884, 21048,
                14300, 47144, 18304, 63116, 8300,  48820, 45248, 54620, 33488,
                17072, 12960, 48480, 1904,  29260, 62284, 38572, 17808, 64408,
                35572, 40152, 1160,  62972, 6532,  22788, 18612, 36752, 29832,
                10880, 39504, 7892,  36272, 30280, 58044, 2972,  33672, 12016,
                58912, 596,   44548, 10392, 33360, 47788, 15236, 31036, 21044,
                51676, 7460,  29960, 4408,  53020, 15976, 24248, 44812, 59348,
                18336, 39864, 44260, 65028, 53148, 33432, 7628,  52032, 36152,
                5356,  57016, 21172, 45552, 28472, 36024, 55308, 31648, 65344,
                52232, 688,
            },
            {
                29164, 58228, 4456,  42448, 11972, 2340,  44980, 36400, 16428,
                55672, 9196,  29428, 50192, 1652,  38872, 59032, 12636, 35696,
                52680, 45160, 3452,  40000, 47828, 58640, 4108,  12172, 51520,
                22156, 15196, 54940, 42048, 38428, 26588, 708,   58804, 23716,
                61920, 52792, 44512, 36356, 24968, 46604, 39488, 9068,  59552,
                25308, 52476, 65404, 8188,  49996, 37940, 2788,  42532, 61052,
                50324, 10292, 35780, 55180, 21780, 14864, 29948, 348,   62440,
                52440, 4828,  34848, 18828, 58256, 14616, 42136, 2512,  53240,
                9168,  23872, 47900, 26468, 33416, 54168, 47416, 64404, 55520,
                20660, 43964, 14692, 65324, 46152, 9176,  27080, 63644, 53068,
                36504, 48696, 25112, 5036,  27512, 53144, 1660,  62224, 46688,
                34608, 17452, 41124, 58848, 26664, 32796, 2800,  53796, 21484,
                33716, 51004, 14112, 3884,  29340, 16532, 43140, 62776, 22968,
                28104, 60816, 10096, 53000, 4572,  60128, 8504,  2472,  20796,
                47664, 13956,
            },
            {
                7604,  18900, 48124, 25476, 38260, 30444, 22356, 51704, 26448,
                61364, 43108, 13160, 63960, 16120, 8724,  25568, 62956, 6868,
                23632, 30484, 19800, 25528, 13980, 32024, 54556, 27268, 44380,
                35132, 48456, 7308,  12340, 52460, 62620, 43708, 35448, 31388,
                9016,  3860,  18116, 59628, 6656,  12988, 54684, 2968,  34452,
                14408, 40084, 43252, 16000, 46864, 59068, 19244, 31672, 7240,
                24824, 58328, 28804, 2992,  33704, 60896, 37848, 20232, 7964,
                39528, 22648, 60952, 46084, 24232, 5860,  49148, 27040, 57448,
                44900, 16628, 58980, 4444,  42404, 11580, 15440, 1512,  6140,
                33140, 50056, 26060, 804,   52428, 13396, 43576, 18200, 5788,
                22536, 14968, 62040, 56260, 42096, 36136, 23212, 9512,  27936,
                57480, 13728, 49980, 20304, 47372, 12420, 64776, 38284, 10528,
                60988, 5904,  56984, 37512, 48476, 59528, 1004,  13228, 47204,
                32644, 50024, 16192, 38404, 42168, 22988, 49084, 43628, 11140,
                38240, 60528,
            },
            {
                53312, 34320, 64584, 14372, 56548, 18080, 62676, 5468,  33300,
                19720, 2740,  23536, 35400, 57288, 30120, 19060, 50552, 42640,
                14420, 61272, 55508, 10812, 62312, 42032, 16784, 38792, 8880,
                63248, 18800, 58928, 36528, 29408, 9632,  19376, 14268, 46140,
                50524, 39076, 26856, 32616, 49200, 42516, 22100, 50700, 27388,
                56804, 10864, 21552, 4116,  34068, 28748, 13304, 56280, 46076,
                1268,  40616, 12952, 44524, 51160, 5404,  57460, 25672, 50356,
                31576, 43424, 9864,  53408, 30832, 36292, 11856, 22364, 32704,
                6876,  38000, 50280, 19012, 36404, 31160, 51032, 27660, 40220,
                58216, 17272, 60708, 34584, 21088, 37880, 49192, 31548, 55756,
                28924, 8560,  38484, 20204, 12252, 59160, 16888, 39220, 54008,
                3244,  24620, 62928, 888,   35948, 43104, 6604,  30372, 48192,
                41576, 28112, 22960, 8768,  20092, 26428, 40216, 55616, 19048,
                7068,  43432, 25992, 408,   30068, 17516, 34472, 57344, 26872,
                30912, 41480,
            },
            {
                23128, 10600, 28072, 1168,  45812, 9040,  40936, 49244, 11216,
                39256, 58648, 48636, 6608,  40752, 45236, 54468, 33096, 4620,
                36812, 47356, 860,   34300, 51612, 6536,  22624, 59856, 30692,
                2892,  26216, 43380, 4028,  23276, 47484, 57552, 4896,  65512,
                21076, 12380, 54980, 220,   62732, 16008, 29772, 64000, 4792,
                47824, 31240, 60196, 53444, 24432, 63620, 9376,  51236, 16700,
                37536, 65004, 53652, 23324, 17032, 41980, 10252, 47116, 13784,
                64720, 27820, 16144, 1220,  40656, 63920, 55588, 43700, 61376,
                14004, 30400, 64104, 10016, 56028, 59816, 24288, 62556, 45164,
                12568, 3664,  29140, 55060, 6740,  24536, 62544, 1692,  40980,
                60652, 46164, 52052, 32296, 2876,  49824, 44884, 65416, 5944,
                42828, 31276, 38108, 9604,  60428, 22936, 54780, 14936, 25080,
                3124,  17140, 46444, 63180, 35456, 53904, 4680,  30944, 38072,
                58244, 11588, 63652, 54364, 61116, 13196, 51876, 5076,  62904,
                15400, 2848,
            },
            {
                45184, 59392, 40396, 52088, 31484, 60364, 24392, 54924, 15316,
                46156, 31236, 26704, 52988, 21568, 264,   11060, 27536, 59872,
                21304, 39664, 26828, 18132, 44672, 29024, 1900,  47208, 50788,
                14332, 55392, 32196, 61436, 16136, 54368, 33388, 25092, 37428,
                29512, 60476, 41820, 23532, 35472, 9988,  38436, 18928, 44256,
                36100, 6884,  17572, 38952, 1440,  44684, 41040, 22516, 27596,
                33124, 19960, 8392,  31444, 62612, 27500, 34992, 18736, 55304,
                2888,  36740, 49524, 57296, 20200, 8552,  17420, 3116,  51304,
                20972, 488,   27456, 46200, 21588, 2368,  8244,  18012, 36944,
                22248, 47872, 42544, 10284, 40320, 58428, 15828, 11484, 35048,
                25792, 4728,  17596, 63860, 29744, 7404,  26036, 32924, 19340,
                12044, 51764, 46116, 16976, 28388, 50572, 18880, 58160, 36364,
                62044, 55696, 32664, 43448, 11860, 50312, 15684, 64560, 25004,
                2344,  35940, 19980, 33468, 45228, 9364,  39612, 19184, 24352,
                54948, 49908,
            },
            {
                19972, 5824,  16304, 36284, 20976, 4008,  35204, 28712, 988,
                64324, 8664,  17464, 61228, 14284, 37488, 64888, 48900, 16520,
                9664,  53728, 58304, 7504,  65176, 38004, 56344, 19704, 35644,
                40476, 10652, 21192, 50188, 38828, 8184,  41348, 1264,  53344,
                7216,  16792, 47760, 5144,  57648, 52352, 2520,  58864, 12372,
                23216, 61724, 26496, 48828, 55980, 6252,  35076, 61816, 4428,
                58900, 46908, 3564,  49020, 39216, 1012,  60280, 44856, 5940,
                21580, 59208, 12548, 25316, 33684, 47888, 29376, 39756, 35044,
                58344, 41484, 53772, 13116, 39180, 44040, 34140, 49564, 54080,
                5636,  31688, 63480, 51300, 18340, 28188, 46732, 53836, 20408,
                50924, 13736, 43896, 35844, 21884, 55460, 14456, 48788, 36296,
                60868, 21712, 55708, 7228,  33896, 40068, 5244,  43952, 9336,
                49896, 13420, 744,   29608, 6012,  23984, 41852, 9084,  45340,
                52708, 14656, 48064, 23644, 3736,  27828, 58600, 44196, 36524,
                8064,  32312,
            },
            {
                38036, 56976, 26136, 62264, 49760, 13212, 58320, 44492, 20596,
                52012, 36192, 42352, 4156,  47188, 32016, 24388, 56396, 2412,
                43840, 31208, 12972, 49400, 24328, 15312, 9148,  61812, 24728,
                5984,  64432, 46448, 1976,  27864, 13308, 62872, 18560, 48960,
                43444, 10592, 32224, 20140, 27912, 45728, 24852, 32784, 55216,
                40484, 51488, 9124,  13668, 31624, 20228, 15456, 52928, 11172,
                42040, 14672, 26268, 55056, 11592, 15800, 51556, 24492, 38456,
                32464, 41832, 7080,  45292, 61880, 4752,  54564, 25756, 15596,
                9324,  47716, 33292, 5380,  25924, 61692, 15088, 28460, 10820,
                57400, 25168, 14840, 36304, 2780,  33420, 5336,  42848, 9092,
                58784, 39368, 48,    53224, 10344, 60328, 41448, 1128,  57608,
                29012, 4080,  25984, 64456, 12896, 59204, 1516,  31492, 27252,
                21388, 40236, 52412, 19220, 57240, 60948, 33532, 21628, 59720,
                28660, 40828, 56992, 7180,  51176, 64988, 32040, 1076,  48368,
                12448, 64124,
            },
            {
                42608, 2332,  11448, 46524, 7868,  42092, 17784, 6388,  38736,
                25164, 12180, 55048, 28920, 58332, 8116,  18452, 41208, 34540,
                62668, 23132, 4216,  35180, 41536, 29944, 53784, 12552, 43172,
                28564, 52952, 34024, 18168, 59784, 45456, 31200, 56860, 26984,
                35344, 64020, 55792, 39324, 12916, 62380, 42664, 6336,  15780,
                584,   46000, 28352, 64704, 37000, 59240, 25812, 47944, 29788,
                56708, 35404, 63740, 21948, 33616, 57616, 29740, 8980,  61516,
                54348, 28768, 17228, 52080, 38204, 22680, 10924, 44488, 65116,
                4288,  23704, 62524, 16852, 50204, 55252, 3540,  41836, 65072,
                39632, 416,   20604, 45924, 55748, 60312, 22328, 64992, 29044,
                32508, 23904, 61524, 27908, 47060, 38060, 24012, 17816, 8416,
                44456, 38624, 15524, 42304, 47784, 23868, 53092, 63448, 45916,
                34640, 7028,  65384, 37152, 27116, 47412, 3996,  17732, 49528,
                5364,  10396, 30764, 17180, 41864, 20984, 14952, 26176, 55944,
                17756, 28468,
            },
            {
                21904, 34004, 54460, 30436, 23616, 65284, 33372, 55704, 48324,
                62832, 2316,  33948, 22256, 50692, 39388, 12532, 46584, 6468,
                28056, 52260, 46936, 59300, 20396, 3208,  45604, 33204, 1104,
                58404, 14848, 38140, 5416,  24308, 51384, 10972, 21424, 5556,
                14400, 23308, 2860,  50868, 8040,  17164, 49976, 30028, 63404,
                21496, 34040, 18836, 42208, 2756,  45388, 7796,  38220, 208,
                18756, 9240,  43736, 5876,  45896, 19304, 40776, 4100,  14828,
                48656, 10640, 64080, 100,   31304, 13920, 59252, 49940, 36792,
                19088, 29628, 56836, 37416, 7824,  31044, 22716, 47380, 19112,
                33232, 50296, 61968, 9488,  26356, 13484, 38824, 49496, 2248,
                16012, 48244, 6652,  19364, 13188, 4416,  31560, 52324, 63556,
                11464, 32556, 54340, 2760,  29436, 36680, 20064, 15928, 11284,
                54656, 24832, 16344, 44784, 9996,  14092, 38768, 54964, 35008,
                26040, 63152, 38448, 59968, 34364, 47564, 10480, 38992, 61580,
                4488,  52956,
            },
            {
                48948, 60672, 14484, 39812, 112,   52576, 26628, 10532, 30632,
                15152, 45816, 19276, 61640, 5168,  26192, 63516, 53116, 20944,
                14780, 38148, 17668, 10792, 63668, 51512, 37432, 60288, 18644,
                49332, 22692, 9296,  56392, 42080, 35716, 2552,  40268, 59096,
                52024, 44792, 28848, 59612, 33904, 22592, 36976, 10712, 53600,
                38272, 57600, 5480,  52192, 11752, 55172, 21276, 63400, 50836,
                24000, 39372, 52320, 28276, 1884,  49580, 65388, 34796, 44252,
                20048, 24196, 35928, 43244, 19336, 55496, 27696, 1876,  32248,
                53152, 42840, 10548, 45592, 19944, 59672, 35444, 8848,  13640,
                27336, 6892,  43640, 30500, 41388, 52544, 7816,  18852, 36704,
                57172, 41056, 54644, 34560, 64656, 42400, 56676, 27320, 46624,
                20432, 49708, 61260, 18504, 10340, 50980, 6220,  38560, 58012,
                2288,  47992, 59452, 3296,  30772, 51672, 64012, 260,   12212,
                44272, 20688, 1920,  13220, 54200, 3256,  57868, 22696, 34768,
                45804, 9384,
            },
            {
                25328, 6036,  44500, 19824, 59044, 16020, 37628, 3796,  22724,
                56792, 39948, 9588,  43968, 16236, 36324, 3140,  29764, 56084,
                42512, 544,   57020, 31872, 6680,  25852, 16064, 8204,  26988,
                40748, 31676, 45016, 62344, 28960, 15252, 64852, 47428, 32412,
                37796, 9456,  18772, 41940, 1216,  46716, 60648, 27344, 3576,
                47796, 13972, 61316, 24604, 30528, 16564, 34096, 43032, 13524,
                32140, 62048, 16412, 60120, 37044, 13408, 23316, 53028, 27740,
                2776,  57332, 6420,  60236, 48240, 8460,  41504, 21864, 7004,
                60572, 13808, 3172,  27668, 52284, 1164,  42768, 55736, 60640,
                37288, 53108, 58132, 17200, 4036,  32960, 24588, 62884, 12400,
                44564, 9976,  22168, 30144, 50140, 9096,  16212, 35900, 2348,
                39984, 24840, 6912,  35336, 57632, 43608, 61944, 26156, 41856,
                29804, 19540, 33408, 41264, 56464, 20108, 25160, 42560, 32012,
                56616, 51904, 46564, 24732, 28032, 43884, 6824,  50912, 29872,
                40488, 16548,
            },
            {
                31932, 63080, 36708, 27808, 8740,  46892, 41780, 60224, 49596,
                6068,  27164, 52308, 57488, 32272, 48464, 59824, 10660, 33724,
                8316,  61904, 27124, 44600, 40528, 21960, 55688, 47016, 64668,
                3740,  53340, 12780, 356,   19568, 54188, 7952,  23724, 17700,
                3560,  55152, 62820, 25480, 14928, 56892, 7272,  41180, 17776,
                25680, 32220, 8840,  39828, 49860, 59624, 26568, 5836,  58144,
                3040,  46232, 7780,  25016, 31540, 58448, 7008,  11056, 62128,
                32280, 51372, 40112, 26024, 15704, 34052, 63216, 46228, 17088,
                39500, 49184, 22940, 63908, 39992, 12248, 24992, 49124, 16692,
                2212,  23924, 11824, 21444, 48984, 59296, 45648, 53916, 31344,
                26944, 3300,  62412, 15112, 1380,  44156, 22772, 59716, 6184,
                13932, 55948, 30816, 46024, 22664, 120,   32720, 14264, 49408,
                8816,  61348, 12616, 23172, 7712,  36348, 47172, 15764, 61256,
                6480,  18084, 35932, 8680,  60908, 32352, 19120, 64036, 13720,
                784,   55724,
            },
            {
                51636, 12008, 2752,  56508, 50568, 31424, 12512, 20312, 33776,
                62444, 36836, 552,   23584, 13544, 42216, 18956, 25052, 45564,
                50352, 22832, 35692, 12860, 49808, 1816,  34732, 30392, 11424,
                20996, 57424, 36172, 25056, 49872, 33848, 43516, 27720, 61608,
                13560, 30060, 48296, 36252, 52644, 31172, 20420, 51064, 65312,
                43664, 55644, 21916, 45840, 984,   36684, 9716,  48692, 28636,
                53288, 21672, 11628, 55408, 40156, 17752, 47252, 42204, 37516,
                16736, 45944, 12608, 30288, 2280,  52444, 37360, 12244, 55848,
                25860, 34972, 58148, 30728, 15744, 33680, 61568, 5292,  28796,
                40792, 46472, 34556, 62300, 37892, 15056, 624,   6320,  20320,
                59984, 51836, 36104, 39832, 58220, 26620, 53124, 32872, 62436,
                50680, 42728, 16508, 65084, 12612, 28208, 55388, 20756, 4632,
                53860, 37884, 44264, 52672, 62392, 4436,  28912, 9268,  22544,
                39176, 29792, 63528, 14984, 50240, 37900, 11384, 42392, 26324,
                59256, 20128,
            },
            {
                37824, 47504, 17216, 34228, 22056, 61628, 1612,  54724, 16408,
                43528, 11872, 30844, 64064, 7692,  54828, 2060,  38548, 65436,
                16584, 4764,  54692, 18916, 58672, 62888, 14324, 43456, 51352,
                6052,  39056, 17368, 46372, 60740, 4336,  11520, 57676, 49548,
                40872, 6880,  21800, 11052, 4872,  39692, 12892, 34524, 1772,
                11428, 36056, 4712,  62336, 14548, 20068, 64928, 40500, 18000,
                37656, 34212, 42744, 50116, 4832,  27368, 53992, 788,   22336,
                56064, 8620,  21052, 64920, 57800, 23664, 5392,  28788, 61480,
                588,   8936,  18632, 47540, 6936,  45156, 53004, 19476, 32336,
                64700, 56872, 3344,  27552, 10208, 40676, 28976, 35656, 42492,
                16232, 46712, 24172, 6008,  19548, 47296, 11564, 38528, 21120,
                28380, 9432,  37448, 4424,  52360, 39332, 46976, 63500, 35672,
                25188, 15840, 852,   27816, 17868, 40188, 50224, 59048, 53640,
                1444,  48872, 41736, 2764,  21300, 57140, 4252,  53616, 35652,
                8136,  43204,
            },
            {
                4388,  23928, 64176, 41024, 6684,  25840, 45364, 29084, 7912,
                24720, 47780, 18776, 39588, 50596, 28856, 21732, 58180, 12368,
                29928, 39920, 47472, 9416,  28632, 5120,  37756, 23728, 60644,
                27764, 32608, 63756, 9028,  30264, 39820, 21144, 36500, 1456,
                53636, 34476, 64536, 43540, 58836, 24500, 62120, 45900, 28908,
                58504, 16876, 51232, 29584, 54112, 44568, 23780, 3824,  55860,
                14008, 63692, 1196,  61032, 20156, 33248, 64336, 14336, 30684,
                60676, 4856,  49772, 38636, 43400, 9972,  19436, 50304, 32060,
                44308, 53356, 37424, 4048,  56288, 22040, 10328, 38144, 44140,
                13908, 6476,  18764, 51960, 55584, 22384, 63868, 57556, 49904,
                8388,  32820, 12312, 54020, 61372, 31024, 3724,  17512, 48200,
                944,   58076, 26108, 19292, 59572, 7872,  17240, 10832, 42816,
                58176, 31552, 64600, 48916, 11556, 56896, 31740, 13600, 35276,
                27012, 11608, 57896, 24132, 44772, 29572, 16328, 48612, 22596,
                62416, 28568,
            },
            {
                58280, 30524, 9748,  53700, 14376, 36928, 51768, 64956, 39200,
                59340, 52828, 5676,  60352, 15784, 35280, 44332, 6564,  33020,
                52064, 1556,  60944, 24656, 33336, 46312, 53428, 8764,  15996,
                48676, 42548, 1960,  14116, 51964, 59192, 16264, 45888, 25560,
                19440, 15248, 46800, 28044, 156,   17184, 54132, 8148,  22416,
                48976, 38852, 25192, 8312,  33780, 57816, 11728, 31756, 45532,
                8356,  26244, 30224, 15492, 46396, 9300,  36456, 50436, 25252,
                44584, 35248, 27312, 17288, 33212, 55132, 47056, 39712, 14956,
                21744, 11380, 65300, 28432, 40572, 62516, 26960, 224,   54832,
                23108, 49668, 31056, 42324, 7640,  46148, 17824, 11036, 25856,
                3968,  65528, 41492, 27868, 37312, 8836,  57428, 41660, 64184,
                34152, 53668, 44052, 32132, 49388, 23808, 34376, 1760,  51980,
                21964, 6004,  46004, 34760, 24384, 3016,  43620, 19576, 65212,
                45964, 17460, 33244, 52752, 6144,  64540, 39656, 32956, 1984,
                45760, 14932,
            },
            {
                40204, 19224, 49240, 44292, 58932, 3956,  18840, 10852, 34712,
                3440,  21816, 32936, 45384, 26276, 10264, 61720, 49308, 25820,
                57044, 20908, 36248, 43020, 16440, 56824, 20544, 40684, 3216,
                58284, 19848, 54856, 23344, 27056, 6940,  32908, 55184, 5288,
                60772, 32072, 8276,  56300, 37292, 48584, 33208, 41596, 14980,
                5592,  63892, 43064, 18820, 2416,  41744, 27144, 50940, 60148,
                22436, 48816, 58524, 38248, 24256, 57444, 3536,  41080, 19080,
                11008, 54332, 1420,  61900, 13184, 3940,  25684, 63436, 2724,
                58356, 41860, 24672, 50768, 13388, 34900, 48908, 16292, 60276,
                8772,  36908, 63040, 25320, 35084, 1648,  32328, 53176, 39160,
                20940, 56868, 17312, 704,   50072, 15012, 45392, 25072, 7324,
                13820, 22308, 11268, 2840,  40760, 29568, 61556, 45540, 27468,
                13288, 40440, 9172,  19976, 59848, 38408, 7936,  25700, 4900,
                41112, 8916,  60500, 37104, 13452, 26020, 8636,  56308, 18708,
                10632, 54680,
            },
            {
                5852,  33436, 448,   27356, 21648, 32656, 42772, 49480, 14988,
                27404, 57640, 12872, 1672,  53808, 40860, 3672,  18436, 14644,
                41380, 10748, 6508,  63476, 11980, 240,   30588, 65032, 26012,
                35220, 10740, 45440, 36848, 40800, 47668, 65516, 10572, 43224,
                38364, 51752, 23288, 12592, 20088, 29972, 3752,  60296, 26880,
                55836, 30928, 11152, 52648, 61988, 35664, 5516,  15796, 36816,
                2304,  41248, 11360, 5856,  43676, 52564, 13264, 62316, 7252,
                58972, 40088, 23348, 46032, 31060, 59420, 36360, 8220,  51804,
                34576, 17120, 31508, 1592,  20224, 7812,  57468, 31988, 39988,
                28332, 45520, 12132, 16068, 59168, 48812, 14052, 61144, 28532,
                47728, 35000, 44408, 22804, 62656, 32292, 20564, 55176, 29000,
                39420, 60744, 46188, 63704, 15164, 56516, 5584,  38392, 18328,
                60836, 56324, 29220, 50956, 14216, 54624, 46900, 61064, 51044,
                22200, 55536, 3252,  30804, 20264, 46748, 51528, 60076, 27932,
                35332, 50220,
            },
            {
                25556, 60524, 38444, 12496, 62712, 56456, 7340,  24092, 55248,
                46904, 41756, 37092, 64608, 20252, 31080, 55680, 37328, 64156,
                46348, 28968, 53760, 48220, 27496, 51336, 37060, 44460, 13284,
                50748, 31836, 4304,  62744, 17636, 648,   22052, 29884, 13864,
                27148, 2508,  62576, 41004, 50444, 64136, 9868,  44976, 20512,
                36744, 736,   45968, 23016, 14660, 47616, 21184, 63844, 53140,
                19160, 33504, 54888, 17064, 31556, 20532, 27128, 47300, 32840,
                29056, 49092, 15052, 6224,  52932, 41172, 17912, 22728, 29488,
                44888, 6236,  55812, 60532, 46088, 42576, 25144, 4760,  18548,
                53704, 2592,  21364, 56496, 4740,  38128, 23712, 43756, 6516,
                2692,  13692, 52632, 10104, 58528, 5176,  35632, 48616, 2392,
                51920, 18460, 35144, 8196,  51276, 21100, 11904, 53792, 32004,
                48420, 4356,  36476, 63768, 1720,  32636, 16904, 29728, 36036,
                14620, 28288, 39280, 50556, 62252, 668,   41452, 23192, 13060,
                43860, 65464,
            },
            {
                15664, 8592,  54072, 46984, 16884, 30308, 39652, 63660, 812,
                29720, 17896, 8688,  48764, 24140, 13748, 47704, 8172,  23040,
                2444,  32492, 18940, 39068, 22656, 59652, 5316,  18108, 61620,
                7596,  56200, 28552, 58792, 12108, 53104, 34752, 56648, 49328,
                18484, 57668, 45188, 6268,  25808, 15988, 38928, 53044, 13200,
                49540, 61012, 33032, 27756, 58876, 38712, 9596,  30284, 43220,
                7224,  62200, 27988, 50196, 64048, 368,   35828, 55996, 16572,
                3340,  36636, 62968, 20476, 27736, 10620, 48848, 55552, 13056,
                61420, 48236, 10500, 35756, 15776, 52256, 63936, 11844, 48072,
                61500, 33120, 51324, 43324, 30088, 64748, 8988,  19260, 55348,
                31260, 63356, 25024, 29824, 39012, 42652, 12096, 16992, 59548,
                31732, 5280,  23556, 28148, 37124, 43168, 26148, 65352, 2148,
                23976, 16296, 44344, 26520, 21992, 42376, 9740,  57956, 976,
                63288, 48168, 18268, 43832, 10128, 15992, 32320, 6380,  38568,
                3272,  21196,
            },
            {
                51580, 41792, 23048, 35816, 3032,  10192, 52636, 20112, 36412,
                5872,  61624, 51524, 32384, 5236,  58976, 28136, 34764, 43080,
                57452, 60856, 4420,  13944, 55316, 9280,  33900, 47620, 24240,
                40100, 20912, 15120, 48932, 25644, 42296, 8576,  62076, 4988,
                39856, 33012, 10416, 35760, 55292, 1892,  59284, 34288, 24560,
                4340,  17584, 40756, 7016,  54068, 3060,  49800, 25772, 57072,
                13356, 23988, 45068, 4776,  38748, 59248, 10332, 42288, 22492,
                52356, 12116, 56624, 42520, 34168, 132,   64828, 38056, 4600,
                20124, 26868, 39008, 22860, 29248, 3588,  37412, 20788, 41496,
                7152,  25716, 10516, 39584, 17724, 27264, 50712, 36540, 58776,
                40296, 8132,  49052, 19144, 2064,  55852, 27192, 65036, 9304,
                43404, 47284, 54864, 58200, 1000,  16624, 46532, 8600,  35464,
                41912, 11600, 55288, 6572,  49728, 37576, 53272, 24760, 45116,
                11812, 6696,  23796, 58884, 27132, 36276, 52844, 63452, 56748,
                47980, 30972,
            },
            {
                59484, 1756,  27996, 61368, 43124, 49048, 26444, 13360, 45668,
                58080, 22952, 11400, 43760, 39380, 62952, 524,   52248, 11744,
                16516, 50172, 36016, 26304, 45128, 41472, 15604, 29520, 1832,
                53420, 35924, 43656, 6436,  31464, 37932, 19628, 44644, 24740,
                15856, 60612, 21536, 30612, 47120, 18280, 28584, 7336,  43440,
                64548, 10404, 56216, 19992, 13008, 34600, 44916, 17252, 1136,
                40032, 35028, 9848,  21776, 15004, 46220, 25296, 6688,  65364,
                39084, 26656, 8480,  47424, 58432, 15368, 44124, 24916, 32512,
                41972, 2500,  62504, 54708, 8320,  58608, 32768, 27256, 56164,
                15260, 35544, 63208, 1044,  54852, 6040,  45860, 11332, 22108,
                16220, 43100, 35356, 60904, 14900, 45932, 53036, 21512, 37076,
                25864, 14740, 11048, 39164, 61740, 32516, 50592, 59056, 19912,
                52376, 60044, 32340, 18520, 62480, 13340, 4280,  20136, 40692,
                31472, 53952, 34412, 3652,  56000, 21572, 45984, 29152, 17880,
                9568,  34920,
            },
            {
                19852, 12220, 55884, 32780, 18596, 5344,  59244, 31748, 40768,
                16444, 34664, 26648, 56336, 15388, 19132, 45700, 25928, 21200,
                30176, 40192, 8500,  63260, 1352,  20220, 61060, 57280, 49732,
                10048, 64480, 3372,  23168, 60660, 51024, 2640,  29216, 54880,
                48152, 884,   52700, 13624, 63036, 41416, 22588, 57340, 31448,
                51900, 37216, 47180, 29980, 60268, 23248, 65196, 32288, 52104,
                60992, 47584, 58552, 55404, 29756, 33728, 53708, 19148, 49660,
                1280,  32392, 17676, 24244, 5536,  30036, 51084, 8084,  57912,
                52780, 14788, 49380, 18372, 43696, 13468, 50440, 45180, 2184,
                59932, 46816, 22880, 49096, 32380, 14752, 62068, 33952, 53752,
                52,    26744, 51992, 5732,  23696, 33308, 7428,  41132, 528,
                50716, 63000, 30532, 19556, 4056,  22912, 13476, 29592, 5228,
                27184, 39032, 9044,  47000, 29276, 58712, 34216, 48796, 61388,
                17020, 38212, 64976, 14048, 42100, 7476,  12320, 1528,  25752,
                61688, 44796,
            },
            {
                38808, 48388, 7508,  14624, 64736, 38136, 22012, 55012, 7856,
                48404, 4512,  53252, 2832,  30556, 37492, 9812,  54524, 61636,
                4884,  47048, 23620, 56632, 51200, 31868, 37204, 12236, 25520,
                18440, 32484, 46692, 55236, 16432, 10304, 63380, 13192, 35580,
                7904,  27576, 43180, 37640, 4452,  51500, 11536, 48516, 15232,
                20860, 25828, 1704,  42836, 51112, 4880,  39208, 11180, 21024,
                28736, 3964,  18380, 41364, 1800,  61216, 12028, 43584, 29348,
                59656, 45468, 54816, 63336, 37504, 21368, 61656, 17748, 35888,
                11512, 46120, 30412, 34316, 444,   64672, 23740, 9784,  38916,
                18248, 28980, 12804, 58092, 20400, 40780, 24880, 4284,  29404,
                64488, 46992, 12788, 57436, 38320, 62096, 30064, 18252, 58544,
                34344, 4888,  42508, 47800, 53200, 40884, 36168, 55956, 45120,
                15084, 64164, 164,   22956, 40144, 5780,  27316, 55232, 3108,
                26236, 8968,  50848, 29432, 47860, 60384, 37896, 55256, 41128,
                14232, 4932,
            },
            {
                58140, 29956, 53304, 24116, 45432, 51008, 3288,  11968, 60400,
                28840, 64340, 41636, 21516, 58208, 49424, 6716,  35520, 42072,
                58488, 14444, 33804, 11372, 27796, 6064,  46300, 4172,  42716,
                38832, 57884, 13000, 27656, 40652, 33892, 45640, 22920, 39440,
                57740, 64220, 18780, 24640, 8728,  60200, 35156, 336,   39344,
                5800,  62020, 13896, 35488, 9036,  27204, 15968, 55228, 7700,
                36940, 14072, 25544, 51124, 8376,  48100, 16284, 36948, 5056,
                21152, 14104, 41272, 3004,  12740, 48172, 40248, 1796,  28192,
                23260, 63456, 4996,  25596, 41036, 19988, 52904, 30780, 5156,
                54292, 42128, 7652,  37052, 3368,  44212, 60156, 50252, 18244,
                39696, 9620,  31784, 20572, 3648,  49740, 10796, 46628, 13344,
                22752, 56668, 16356, 25512, 7380,  64448, 10708, 2704,  48840,
                24632, 33928, 50752, 57032, 16540, 45832, 11024, 15028, 42464,
                22500, 45548, 428,   20684, 16656, 32980, 23464, 49464, 30696,
                52296, 21896,
            },
            {
                26480, 3748,  41532, 36580, 10428, 27824, 34084, 44036, 24404,
                18932, 36508, 14136, 10612, 44480, 24216, 64940, 17304, 32076,
                2080,  20404, 53436, 43792, 17660, 65420, 22508, 54516, 61660,
                29052, 7300,  20560, 52584, 96,    58660, 18288, 53576, 3644,
                31968, 11356, 46316, 55112, 30032, 16536, 26708, 45176, 58844,
                52960, 28972, 56812, 18148, 62944, 45528, 58276, 41412, 48748,
                62528, 44224, 32896, 64628, 38068, 22852, 27292, 62900, 56184,
                10148, 35416, 52004, 27172, 33880, 53776, 10512, 45508, 59924,
                54416, 37840, 9424,  56140, 60408, 36648, 15920, 47572, 62056,
                34032, 25132, 65136, 51856, 30740, 55784, 12292, 8100,  35480,
                58324, 24068, 44536, 54632, 16820, 42752, 25180, 64388, 52536,
                28612, 38132, 9844,  60204, 33564, 27924, 43364, 18024, 61616,
                37452, 11936, 20444, 42672, 31052, 63780, 36932, 59480, 51600,
                33412, 62260, 57356, 40212, 10324, 63172, 4320,  18512, 7148,
                64892, 34468,
            },
            {
                16024, 62124, 19140, 56388, 1344,  17036, 62976, 53612, 39428,
                56,    51720, 47360, 33056, 60248, 1400,  28592, 12800, 51532,
                26396, 63580, 36764, 7420,  40056, 49368, 13800, 34820, 16772,
                1712,  44748, 63124, 36916, 24860, 6132,  30008, 49256, 15436,
                42064, 21680, 2032,  34176, 49612, 40848, 65336, 19072, 32160,
                8304,  41500, 22236, 49484, 32452, 2912,  24088, 30600, 404,
                22372, 53488, 5828,  11560, 30428, 54400, 2736,  40712, 31876,
                50328, 23508, 60172, 7660,  17324, 57348, 24528, 32088, 6632,
                20436, 16456, 44120, 50288, 12344, 6820,  26888, 58288, 11784,
                20956, 676,   45624, 10360, 16776, 27076, 38732, 21920, 47548,
                15180, 6240,  62724, 34652, 28160, 60380, 2496,  36568, 6908,
                43500, 1808,  54192, 20180, 50176, 14604, 57700, 21860, 30252,
                6956,  59608, 3460,  52940, 8020,  24032, 1492,  20044, 30336,
                4944,  13104, 36240, 27296, 53136, 43548, 56584, 36368, 46832,
                10984, 42980,
            },
            {
                51088, 8996,  46240, 30968, 59788, 49216, 21208, 13844, 31420,
                9228,  62248, 26132, 6192,  40096, 19756, 53848, 46412, 38728,
                10044, 47776, 57416, 29560, 920,   59616, 31320, 9352,  52336,
                24312, 50392, 33104, 9820,  48264, 43480, 65140, 8960,  26608,
                59328, 50896, 62004, 14208, 6812,  56152, 3324,  23540, 12888,
                36444, 46436, 3764,  11096, 38096, 53852, 19096, 12812, 34916,
                59648, 16464, 39980, 20592, 44952, 60336, 18568, 8060,  46704,
                15720, 1896,  43744, 30712, 63964, 4072,  42688, 14564, 51408,
                39916, 1184,  28692, 22036, 32256, 46036, 2648,  37984, 43072,
                50104, 15352, 59604, 35600, 19996, 49336, 1864,  61192, 53592,
                31020, 41296, 1100,  50684, 8564,  39968, 15484, 32732, 20824,
                49028, 63416, 30676, 45272, 5844,  37024, 732,   51968, 40124,
                54808, 44628, 26716, 13660, 34880, 56168, 49776, 39636, 9448,
                53888, 18464, 48644, 6860,  31280, 2336,  25156, 13648, 28304,
                59524, 308,
            },
            {
                24272, 39060, 13296, 23096, 35436, 8128,  40596, 4716,  57724,
                45056, 17356, 55880, 30152, 15832, 35952, 57160, 7840,  22712,
                42480, 3768,  15116, 24072, 19388, 45268, 26464, 37716, 58248,
                40868, 12200, 18848, 59428, 14820, 38904, 20724, 55808, 35800,
                4824,  38160, 28280, 43980, 20340, 37820, 10380, 48140, 54356,
                63060, 16588, 25392, 60624, 43596, 7160,  64696, 51956, 46824,
                9928,  27648, 57524, 48544, 4732,  14280, 34360, 57828, 24724,
                64924, 37324, 19484, 49080, 39100, 21584, 28892, 61884, 34416,
                47400, 65488, 58360, 36012, 17800, 62536, 55660, 23196, 7992,
                32368, 27788, 39684, 56600, 6116,  63900, 33148, 42900, 4720,
                27716, 56188, 19956, 13564, 47320, 22428, 53556, 57528, 10440,
                26284, 17172, 12296, 39752, 23404, 62508, 47216, 26092, 9644,
                16168, 32480, 48336, 38764, 62848, 17260, 28080, 47124, 64824,
                25632, 44208, 59912, 22628, 16072, 51120, 60932, 40624, 20916,
                54660, 32300,
            },
            {
                48344, 57484, 63896, 5340,  43372, 55092, 29128, 47692, 25172,
                34908, 22092, 2448,  49900, 63956, 43688, 4144,  31360, 61976,
                18624, 59832, 32388, 50860, 55488, 8164,  62632, 4256,  20792,
                47112, 30344, 3168,  55340, 28220, 1996,  32788, 12708, 46792,
                23980, 17320, 9888,  57312, 31592, 25464, 60452, 41888, 28052,
                6444,  50784, 30808, 55668, 14784, 28560, 36212, 25940, 4528,
                43036, 31856, 1700,  35796, 52920, 26904, 42036, 51660, 11644,
                29388, 5888,  56040, 13040, 8828,  46740, 53088, 2464,  10276,
                24972, 13616, 7944,  53440, 4196,  41528, 29472, 51712, 18648,
                63468, 53896, 3656,  23020, 30108, 46308, 13764, 24396, 18500,
                10828, 65532, 37992, 26144, 59508, 30836, 4972,  45344, 61668,
                41352, 35056, 59236, 3328,  55824, 31832, 12868, 34976, 64588,
                4260,  19204, 60832, 22196, 2284,  10724, 42568, 5544,  15348,
                34684, 1248,  38608, 11156, 64392, 33748, 8912,  45712, 4660,
                17348, 36808,
            },
            {
                27540, 2920,  19596, 52236, 26172, 16160, 65276, 11552, 60720,
                37960, 54096, 41612, 12516, 8812,  24936, 48732, 14404, 27792,
                53168, 35088, 10996, 41224, 36048, 13244, 42276, 53560, 15608,
                7088,  64612, 36188, 42644, 23408, 61588, 52252, 7196,  60392,
                40644, 53500, 64476, 572,   46144, 51628, 14924, 33792, 1784,
                20520, 39884, 34636, 768,   21660, 49112, 41144, 16932, 61264,
                55192, 19428, 63796, 24040, 10320, 62648, 20992, 236,   38604,
                44620, 53476, 26640, 62512, 35588, 16504, 59112, 37768, 19452,
                56380, 31268, 43264, 25916, 48964, 10432, 14556, 34868, 1776,
                44876, 9752,  14156, 42432, 52452, 8508,  57220, 36804, 51484,
                45084, 34340, 6616,  52392, 42888, 11568, 36856, 18868, 244,
                51172, 7184,  47868, 28364, 17920, 8096,  41400, 20324, 56700,
                42732, 50432, 6672,  29808, 46520, 54248, 32752, 60928, 21748,
                57688, 29380, 55044, 42180, 47324, 19368, 37692, 29896, 49968,
                63372, 7612,
            },
            {
                15324, 44664, 34256, 10540, 38304, 1684,  32964, 19856, 3472,
                15156, 7672,  27612, 58924, 34056, 21064, 57992, 37456, 828,
                45088, 6316,  26064, 63160, 2676,  48356, 23184, 28672, 33800,
                60096, 25980, 51568, 5072,  48852, 16424, 44596, 30200, 19384,
                3176,  33700, 13508, 22776, 36512, 4956,  18332, 62084, 44248,
                57232, 12080, 63840, 46708, 9600,  59700, 2620,  11492, 22972,
                37916, 7916,  50048, 15768, 39204, 46272, 7332,  31352, 61320,
                16928, 22192, 33092, 1340,  42452, 23756, 6760,  27688, 45320,
                50336, 5016,  16188, 39304, 63928, 20904, 57292, 38660, 60028,
                25484, 48628, 37088, 62476, 18140, 26460, 40316, 496,   61736,
                15392, 48672, 23420, 2244,  17644, 64040, 28436, 54844, 22992,
                31772, 15304, 24856, 53460, 44244, 60068, 51648, 1912,  29236,
                24060, 12084, 36780, 58028, 40260, 18672, 24904, 12256, 38016,
                50588, 4384,  14204, 24764, 5832,  58196, 952,   23748, 57252,
                11444, 42088,
            },
            {
                60460, 54952, 30188, 61664, 49880, 46204, 58264, 40892, 52472,
                49140, 31528, 61984, 45988, 17632, 52136, 41140, 11472, 65172,
                49416, 15816, 56608, 21324, 30516, 17436, 57796, 352,   45396,
                39940, 9260,  13788, 21348, 39568, 10872, 35340, 56472, 26504,
                49944, 43316, 29108, 48540, 58944, 9080,  55312, 30380, 23084,
                7860,  52152, 17968, 24932, 38720, 56284, 29660, 51348, 33876,
                46428, 13108, 42200, 28440, 59212, 33320, 55032, 13644, 50564,
                4148,  47368, 10588, 60012, 49520, 31504, 55720, 12632, 40660,
                62424, 22532, 58868, 33132, 2240,  28372, 47044, 6124,  12728,
                31232, 21460, 55924, 33852, 5304,  12164, 59004, 21812, 28476,
                8964,  31292, 60084, 56592, 33556, 8264,  49340, 40300, 12776,
                56800, 38500, 65164, 10732, 36576, 22396, 14704, 47732, 38724,
                61392, 53016, 15624, 26204, 8992,  59556, 3212,  52676, 45504,
                8448,  19684, 62152, 32620, 52364, 27784, 44544, 13444, 53256,
                31588, 22144,
            },
            {
                47920, 6280,  24692, 13724, 21612, 7236,  28196, 24184, 9508,
                44172, 21644, 5264,  38640, 1764,  29292, 6764,  32228, 19496,
                23776, 39148, 43820, 9468,  52588, 38536, 64144, 11576, 50472,
                19812, 54824, 32324, 57264, 62864, 25340, 716,   64112, 9420,
                15636, 61284, 6744,  19868, 41228, 26556, 39052, 13020, 49176,
                37176, 27780, 42892, 4612,  32756, 19876, 44432, 63628, 5440,
                57656, 30872, 54252, 6108,  2900,  18208, 23224, 42584, 27604,
                36144, 58116, 40272, 14844, 19616, 3304,  64768, 33736, 17640,
                180,   36156, 9492,  51592, 44372, 17332, 23548, 52732, 40896,
                64372, 16664, 1256,  28216, 46700, 50004, 32316, 43420, 55100,
                4460,  41652, 20372, 39496, 14064, 44640, 3744,  25732, 62588,
                4756,  43532, 20440, 1568,  30472, 5560,  63308, 33052, 10180,
                3836,  44892, 34260, 304,   49852, 43644, 35904, 63352, 27224,
                31396, 41592, 48784, 36800, 16556, 9264,  63916, 18308, 39104,
                2612,  36092,
            },
            {
                17604, 40344, 632,   57096, 42456, 16972, 54208, 62520, 34808,
                18188, 56620, 25872, 53432, 15544, 62704, 47440, 54312, 58968,
                3284,  29588, 60668, 4396,  46956, 32776, 6788,  24956, 35964,
                16704, 3696,  43416, 27676, 6872,  18376, 47220, 41876, 38280,
                23256, 55372, 35568, 11604, 63232, 53128, 2868,  45992, 64416,
                3784,  58104, 15340, 60872, 53192, 7424,  14348, 26516, 17892,
                620,   25196, 20676, 65304, 36468, 48176, 57328, 9536,  64272,
                18684, 6556,  24804, 54176, 29544, 44828, 51548, 10036, 25804,
                48092, 54760, 29904, 13224, 61600, 8124,  34092, 58476, 3572,
                44060, 8792,  51296, 40060, 60588, 20028, 3152,  36428, 16940,
                62692, 47108, 10696, 27672, 51092, 61180, 19712, 35284, 46608,
                9292,  33788, 58232, 50216, 40888, 46132, 25908, 53880, 18588,
                27964, 21332, 65296, 54740, 30420, 13124, 20960, 6128,  16300,
                56292, 23152, 10780, 1724,  59460, 40932, 34304, 48492, 25764,
                61472, 9940,
            },
            {
                28272, 52724, 34964, 64420, 31176, 36996, 3148,  13136, 39532,
                464,   64728, 12020, 35896, 42444, 22708, 9628,  26288, 36872,
                13732, 51224, 34548, 20032, 27284, 14812, 43148, 55472, 61932,
                30376, 47840, 61040, 37168, 50812, 33764, 54108, 12752, 3780,
                32420, 51760, 1384,  45224, 31088, 15536, 34172, 20988, 25744,
                10444, 35268, 30476, 47552, 12232, 37308, 50120, 42344, 35624,
                62100, 38968, 51728, 44608, 11636, 26028, 39776, 1820,  30904,
                52312, 43992, 34296, 61148, 8168,  37348, 21996, 41844, 60160,
                5964,  39016, 19780, 24680, 56176, 37648, 49844, 29836, 19356,
                26832, 35860, 57920, 24096, 14972, 9852,  64820, 52804, 12424,
                25232, 35392, 54528, 764,   24044, 30940, 55968, 15576, 53032,
                29648, 17720, 27168, 13088, 55220, 17068, 35428, 7788,  58860,
                48768, 40992, 6432,  17564, 23544, 60480, 33100, 48160, 39572,
                2864,  65048, 44240, 54104, 29640, 21284, 4132,  55144, 7192,
                44072, 50888,
            },
            {
                59232, 12332, 46048, 19328, 10244, 45216, 51096, 59676, 29316,
                48472, 32424, 7968,  50284, 60216, 3488,  33860, 45356, 17740,
                42144, 7620,  64396, 11896, 54028, 59880, 21784, 3204,  41060,
                7932,  23576, 11880, 2000,  14860, 60232, 29312, 21520, 58604,
                44316, 27304, 17976, 57132, 24340, 6208,  60080, 51368, 17352,
                41524, 55208, 20648, 1204,  64908, 28588, 22448, 59424, 10692,
                47688, 14976, 8572,  32688, 17168, 50372, 62052, 15500, 46612,
                21216, 12056, 600,   27464, 48004, 13928, 4032,  53532, 28684,
                15016, 64280, 46544, 1780,  42120, 4928,  15224, 10336, 60468,
                53404, 12428, 45720, 6772,  31436, 37956, 44944, 22224, 29936,
                50460, 6984,  18316, 64604, 38208, 6204,  42004, 2768,  22296,
                60520, 47892, 7116,  63712, 23812, 2652,  61872, 43280, 13584,
                31768, 57380, 11104, 36604, 46952, 42192, 9804,  57864, 51436,
                13532, 34704, 18580, 26392, 50412, 12644, 62116, 30664, 14836,
                20344, 33068,
            },
            {
                23124, 5108,  26804, 55148, 6724,  22928, 26320, 5496,  15908,
                41920, 23412, 45596, 27984, 19360, 55320, 12560, 63408, 376,
                55936, 47896, 25032, 40672, 1368,  35128, 49100, 10676, 27544,
                51744, 57944, 20584, 45704, 39952, 24192, 8380,  49544, 16892,
                5752,  61980, 40172, 8708,  50128, 37316, 42620, 140,   31996,
                61832, 7072,  45256, 24276, 40332, 56560, 2896,  6520,  31408,
                55760, 23428, 2140,  61160, 28744, 4524,  33960, 7752,  37584,
                56736, 63064, 39552, 17540, 56076, 62580, 35184, 19116, 44296,
                32760, 11300, 50972, 34852, 27028, 63504, 22232, 47876, 39492,
                2484,  32628, 20936, 62180, 54984, 26376, 5672,  57784, 1608,
                39248, 59196, 45484, 33216, 13328, 48552, 58820, 11236, 36772,
                40564, 840,   51808, 34528, 39020, 11028, 49452, 22084, 37968,
                1108,  24908, 51772, 62368, 4496,  27416, 1588,  19472, 25592,
                30720, 60712, 4664,  38716, 7908,  46592, 23672, 41976, 37008,
                65192, 2124,
            },
            {
                39868, 16524, 61136, 38044, 49924, 62808, 33580, 57576, 20336,
                54084, 61208, 14352, 5068,  39596, 30752, 48888, 38040, 28640,
                22176, 32456, 16384, 57008, 45580, 18472, 30948, 63684, 38224,
                16164, 34196, 65348, 31220, 56416, 4604,  42368, 64304, 30980,
                37588, 13796, 48348, 21632, 65232, 12692, 27104, 54488, 48028,
                13520, 37760, 51576, 16320, 9464,  33624, 46468, 19320, 52140,
                27380, 43556, 40380, 57980, 52652, 21752, 41988, 54276, 24496,
                29092, 49044, 6400,  22632, 41152, 10980, 24944, 58720, 1088,
                56552, 23156, 7820,  59356, 17900, 54428, 31456, 43660, 24896,
                65376, 17040, 49628, 176,   40432, 16288, 48360, 32824, 42160,
                14708, 27928, 21392, 9356,  52592, 17432, 28464, 45876, 64236,
                25216, 14260, 44364, 20624, 31128, 57112, 28428, 5680,  60852,
                45884, 16368, 29396, 39744, 14532, 55120, 64260, 37520, 43848,
                7452,  47396, 54960, 15160, 58904, 34872, 136,   56784, 10896,
                53100, 47344,
            },
            {
                58072, 43616, 31820, 1060,  14428, 40696, 11488, 44700, 37352,
                9592,  2812,  35540, 52220, 59028, 16196, 24512, 5488,  52640,
                60924, 10716, 5116,  28124, 37220, 8224,  53472, 13400, 46380,
                1188,  42476, 6060,  26884, 17996, 52892, 35308, 11460, 46556,
                55492, 25344, 34384, 2220,  30744, 18728, 58292, 9760,  22484,
                28988, 3232,  58700, 27000, 62532, 53496, 13652, 36688, 61652,
                16856, 4816,  34676, 19544, 13376, 45556, 60360, 10992, 18348,
                4004,  14340, 32472, 52860, 46044, 30628, 7384,  48500, 38268,
                17284, 41240, 30084, 45300, 3136,  38452, 8548,  13432, 6308,
                55544, 37108, 42784, 29364, 11988, 51828, 63708, 19488, 10788,
                61632, 55604, 3248,  43492, 62560, 22884, 34740, 8052,  19308,
                31692, 56500, 9236,  60304, 3504,  42436, 15336, 53012, 34112,
                19968, 56452, 8028,  34484, 21956, 49296, 30040, 16472, 53368,
                11316, 23816, 41872, 32084, 19808, 52188, 44964, 16728, 26260,
                6580,  29448,
            },
            {
                13368, 8396,  48716, 21028, 29212, 53624, 18692, 1908,  30512,
                49312, 26920, 63464, 22320, 10456, 42220, 56712, 8540,  18748,
                43632, 39516, 58840, 49952, 62564, 23400, 4220,  26008, 57228,
                19984, 54788, 10236, 50348, 13684, 61308, 340,   27588, 19632,
                3936,  9640,  59064, 53492, 43536, 46676, 4876,  35236, 40716,
                64120, 19400, 34668, 43228, 5004,  30536, 22032, 39680, 45172,
                11660, 64664, 48944, 9276,  37980, 25784, 796,   30680, 64876,
                44504, 58508, 36132, 60884, 2588,  15860, 65448, 52100, 26824,
                4948,  63304, 53084, 11912, 21500, 50052, 62168, 35152, 51732,
                27972, 4052,  23472, 57168, 8436,  35376, 24300, 3548,  46912,
                34948, 24940, 49508, 30868, 39772, 1092,  57352, 53376, 4224,
                49616, 37724, 27504, 53844, 18240, 47340, 63044, 25692, 9712,
                41632, 49804, 2664,  59928, 44880, 10876, 5816,  59500, 35020,
                21188, 61848, 2732,  27364, 64508, 9584,  29124, 60600, 39140,
                19200, 34516,
            },
            {
                55416, 24892, 64240, 36268, 59516, 8068,  47752, 65444, 24064,
                54976, 40948, 17360, 45012, 33384, 1460,  65064, 35744, 46508,
                13612, 34136, 1856,  20552, 14872, 44328, 40140, 59948, 31908,
                36616, 24344, 47192, 38516, 32952, 44144, 23416, 39112, 62908,
                50736, 41616, 16460, 28020, 7580,  38356, 24712, 60860, 15432,
                45020, 7740,  49684, 11868, 18004, 48508, 60316, 8296,  516,
                24316, 29368, 53712, 31956, 6460,  56916, 47848, 35024, 51288,
                39264, 25584, 10240, 19604, 27860, 43240, 34508, 20816, 9400,
                46456, 36584, 14588, 33988, 28608, 57744, 16764, 46332, 20456,
                59592, 14764, 47608, 61716, 18572, 44368, 58644, 29148, 53620,
                7528,  16672, 37340, 6000,  60180, 14528, 24808, 42416, 12436,
                61120, 16452, 5644,  40788, 24156, 12508, 36224, 1852,  31536,
                64860, 12392, 26636, 53280, 18620, 41072, 24788, 46352, 560,
                39196, 57124, 49132, 12768, 37864, 5900,  22564, 48856, 3596,
                63176, 50676,
            },
            {
                2628,  41700, 17560, 4480,  44112, 27272, 34628, 15264, 58164,
                7012,  11928, 31480, 4308,  51272, 26104, 48448, 29484, 21608,
                62156, 24768, 54412, 30248, 9952,  33136, 51184, 16880, 7168,
                11932, 63132, 2600,  59088, 20900, 8800,  48416, 57544, 14356,
                32356, 22360, 36536, 63488, 13272, 57092, 50260, 2804,  52908,
                31372, 55440, 24600, 57532, 41572, 2632,  25780, 56480, 51316,
                35720, 58760, 14992, 21976, 62964, 40968, 17292, 13984, 8104,
                21136, 1476,  55088, 49680, 5864,  39096, 55912, 13348, 31924,
                61612, 24252, 2276,  60260, 43068, 5184,  25364, 820,   41244,
                10848, 34388, 7064,  31312, 39672, 4864,  13416, 38460, 21232,
                43136, 65216, 12136, 20000, 50944, 28776, 36236, 47712, 21060,
                33632, 45204, 65412, 30604, 50540, 7648,  56092, 39872, 17472,
                46680, 23376, 36988, 31124, 3828,  63636, 32576, 51952, 15244,
                28740, 5080,  33088, 17848, 44084, 56340, 41184, 31440, 11708,
                44552, 22000,
            },
            {
                10008, 33280, 52824, 12596, 56704, 22836, 3600,  42308, 38512,
                20728, 46836, 60700, 57036, 19916, 38384, 11420, 15820, 2816,
                51788, 7664,  47628, 41804, 56572, 64996, 188,   48068, 28300,
                43924, 52408, 15708, 29712, 5284,  54900, 17124, 29112, 6952,
                45244, 60492, 948,   47600, 20876, 32184, 17500, 27748, 11436,
                21640, 700,   38576, 29116, 65452, 35096, 15868, 32048, 42212,
                18964, 4160,  47024, 43156, 2408,  28260, 52992, 23656, 57412,
                42832, 30236, 46572, 62428, 12032, 23720, 59744, 1392,  50528,
                41996, 18904, 54884, 48596, 9828,  39232, 32572, 56724, 29844,
                63432, 45052, 52656, 21880, 54840, 26536, 48704, 61352, 640,
                31224, 57396, 26940, 54868, 45072, 10144, 64084, 2940,  56256,
                25912, 10308, 128,   20104, 34804, 61960, 44648, 21616, 52200,
                5912,  56996, 43520, 61056, 14100, 37976, 8308,  56460, 19580,
                63144, 45492, 52464, 25324, 62236, 1316,  50936, 15080, 57712,
                36560, 28392,
            },
            {
                47288, 61700, 25996, 37688, 50276, 10228, 61544, 52300, 32408,
                60,    27844, 36920, 14120, 7632,  61876, 55068, 40808, 58496,
                31724, 37928, 12688, 17812, 26932, 5988,  38584, 19624, 61604,
                22284, 40856, 34840, 25552, 64772, 41356, 36392, 2180,  53872,
                11668, 25728, 52128, 6300,  40920, 9100,  62484, 42904, 48308,
                34128, 61644, 14668, 9560,  20880, 54004, 45516, 12660, 63600,
                10000, 38680, 26576, 55856, 12356, 36600, 60788, 3380,  33364,
                63368, 13120, 17444, 36756, 32168, 53052, 44940, 17116, 29392,
                57504, 7124,  27632, 15960, 22792, 64880, 13216, 49688, 17760,
                23864, 38028, 1616,  15888, 64764, 10420, 33376, 17712, 52144,
                9672,  40328, 47996, 2272,  33164, 16248, 40820, 7848,  29976,
                39272, 52808, 48532, 58128, 14724, 4152,  26348, 9184,  59408,
                29828, 476,   16620, 9688,  47420, 22244, 50080, 26800, 11656,
                41408, 7732,  14368, 36652, 10232, 19932, 34144, 26052, 7208,
                18008, 59352,
            },
            {
                38840, 14584, 5448,  30932, 19508, 46128, 29412, 13464, 18076,
                62812, 53548, 24460, 49692, 42984, 30724, 22948, 5936,  44160,
                26280, 63188, 4232,  59692, 35688, 23088, 11516, 55240, 33520,
                4096,  9104,  57188, 46340, 12336, 51168, 24296, 62220, 33980,
                42724, 17672, 30000, 56276, 34632, 23572, 54180, 37004, 5300,
                58108, 40288, 46880, 51860, 3624,  37652, 5948,  27836, 22480,
                50060, 61072, 7400,  33220, 49440, 18872, 9780,  45764, 49892,
                5472,  26916, 41012, 3012,  21464, 8716,  26424, 4236,  39944,
                11044, 37396, 45636, 34788, 52468, 6808,  44180, 54448, 4344,
                8808,  58740, 27752, 50012, 41800, 2988,  57068, 45824, 25620,
                15172, 35232, 22932, 5204,  58620, 23900, 52416, 18680, 59964,
                13836, 23064, 36384, 42692, 28456, 53504, 32772, 38380, 13888,
                49436, 35808, 20884, 54508, 28880, 58512, 2564,  43836, 35912,
                31252, 23284, 54756, 59072, 30460, 47248, 54012, 64296, 40552,
                52156, 492,
            },
            {
                20352, 54572, 42852, 64028, 1484,  40164, 58688, 7264,  48860,
                43336, 5240,  10168, 33944, 16952, 1212,  52176, 36288, 18100,
                9552,  45976, 21320, 49428, 43580, 53060, 29672, 45584, 58920,
                14504, 49272, 18320, 740,   32036, 19732, 9876,  57996, 47256,
                20612, 39296, 3500,  65024, 15728, 45348, 2136,  14000, 19652,
                25248, 7344,  17548, 32700, 24212, 59320, 47836, 55356, 1436,
                30580, 44760, 16092, 21168, 63976, 40416, 30860, 25488, 38652,
                19700, 59504, 51864, 56580, 47476, 64460, 35484, 60784, 49120,
                23144, 64024, 32,    60604, 30716, 19744, 35640, 26524, 40580,
                48108, 33452, 12068, 19688, 36088, 30256, 22660, 37704, 6668,
                63564, 59868, 18960, 43900, 37924, 62604, 31432, 49172, 44572,
                6176,  62324, 3048,  17488, 10944, 60292, 48164, 19024, 64192,
                42612, 26016, 62652, 40628, 6636,  33548, 17052, 64804, 5976,
                59896, 49024, 1936,  39888, 5700,  22700, 12652, 3804,  45416,
                23996, 31864,
            },
            {
                49632, 7644,  23384, 57664, 16280, 33972, 22472, 26212, 35616,
                55332, 21972, 40136, 63808, 58380, 28084, 46596, 64680, 13552,
                56232, 34292, 28956, 14676, 3000,  63856, 16404, 7348,  40080,
                26660, 36904, 53508, 60544, 39168, 44904, 3348,  15192, 27088,
                7256,  59024, 48548, 10760, 26412, 51104, 59772, 28872, 64248,
                50344, 30452, 43916, 62328, 12208, 41256, 18660, 33992, 14508,
                58464, 36164, 25236, 52668, 28,    6560,  58364, 15104, 54772,
                10808, 34204, 7768,  15520, 28452, 42712, 19208, 14276, 32740,
                54464, 20292, 13536, 41324, 3640,  57780, 10772, 62852, 21688,
                15360, 61508, 43116, 53820, 5720,  59376, 13704, 42720, 50320,
                1928,  28760, 52968, 13976, 7036,  27732, 11816, 996,   34648,
                26780, 54344, 31796, 46960, 22220, 40816, 1972,  24684, 6944,
                11260, 32416, 3900,  51228, 45708, 24464, 39624, 53388, 20620,
                45844, 15940, 27904, 18944, 43796, 61188, 38176, 28664, 16148,
                62132, 11256,
            },
            {
                59748, 29360, 35292, 48060, 9380,  51736, 4440,  64864, 11348,
                15924, 29888, 2656,  44988, 12488, 20364, 4508,  32744, 24408,
                50212, 652,   54100, 40352, 9348,  32448, 50952, 24472, 2468,
                62880, 30732, 22864, 6408,  27600, 63828, 50184, 31068, 37740,
                52772, 13664, 31964, 44008, 21376, 37988, 8176,  33184, 42108,
                11128, 56364, 1328,  53656, 27020, 6800,  63240, 9128,  39824,
                51456, 3544,  11252, 57060, 27852, 47604, 43320, 21788, 4000,
                65224, 44968, 22872, 38288, 856,   11456, 55336, 5368,  46040,
                8420,  30192, 51344, 25136, 47672, 16844, 38552, 51028, 1224,
                29524, 56060, 3436,  24928, 46336, 63092, 8464,  18396, 55816,
                32172, 40908, 9140,  48728, 56056, 21220, 41648, 57856, 15660,
                19816, 40072, 8328,  57316, 63284, 6396,  34708, 56348, 45040,
                53232, 58752, 14560, 19124, 10140, 61520, 13448, 660,   26384,
                10900, 33756, 64044, 55524, 9192,  50236, 33008, 56436, 8632,
                35712, 41508,
            },
            {
                3612,  17828, 44472, 12624, 27532, 56072, 42132, 45872, 38796,
                60340, 48104, 56788, 25948, 37496, 54036, 40940, 59128, 7520,
                38596, 19396, 60760, 25696, 58120, 37600, 18876, 43000, 54648,
                10120, 46168, 12940, 42488, 16360, 35264, 21248, 56312, 4772,
                63280, 23884, 35668, 61836, 628,   55424, 18492, 47304, 4076,
                22460, 37828, 14824, 20508, 36732, 49568, 29992, 44136, 24004,
                19672, 65016, 42428, 38244, 33528, 12964, 62572, 35532, 51052,
                27332, 31412, 48620, 61996, 58692, 30788, 51152, 37188, 25468,
                59412, 43948, 4836,  34044, 55476, 28540, 6296,  59468, 32588,
                47012, 7956,  39364, 16872, 29132, 51148, 34532, 26956, 47392,
                12300, 24320, 36332, 64384, 33344, 45956, 4376,  51924, 37256,
                64960, 45132, 12640, 25176, 37020, 14512, 51356, 30548, 17372,
                22324, 39288, 28200, 36080, 55800, 31016, 49288, 36964, 57496,
                40764, 51392, 3064,  36260, 14184, 25736, 992,   18384, 46780,
                53244, 26696,
            },
            {
                56964, 63596, 1988,  37592, 61244, 19044, 31608, 1036,  20888,
                7704,  32928, 18516, 5992,  50292, 9708,  15864, 47960, 30204,
                62672, 11532, 43200, 6112,  21876, 47496, 12520, 59936, 33048,
                20576, 50636, 1732,  58612, 54164, 7524,  11468, 40636, 44216,
                17860, 9444,  50000, 5968,  29644, 40464, 12148, 57936, 52616,
                27616, 61204, 48624, 33304, 3264,  56864, 15916, 60448, 4940,
                28544, 48412, 7676,  17780, 23180, 54152, 8924,  17404, 41568,
                1624,  12672, 18204, 4476,  24612, 40728, 21260, 63872, 1948,
                15824, 39480, 18552, 63588, 11908, 42528, 24136, 45048, 13964,
                19516, 36572, 63944, 22216, 10348, 40460, 144,   21504, 62176,
                4552,  59188, 20060, 2712,  15176, 25740, 61956, 9744,  23220,
                49784, 3404,  29724, 52240, 19324, 43232, 27092, 62172, 9476,
                1572,  48328, 65160, 2916,  43768, 21820, 5384,  28944, 62736,
                7996,  22996, 30356, 47388, 20752, 59372, 40244, 64948, 22140,
                5060,  14304,
            },
            {
                39444, 21164, 32128, 50792, 23912, 5716,  14832, 53876, 28384,
                52168, 13148, 62840, 35160, 23008, 61788, 26848, 2356,  21676,
                45076, 16912, 35300, 31512, 53284, 1576,  27980, 4676,  36532,
                15692, 65468, 38760, 33444, 23624, 48400, 61476, 25452, 1056,
                28932, 58440, 19964, 42440, 14884, 60824, 24576, 35556, 16436,
                7588,  41292, 10004, 64484, 45780, 22348, 11480, 52620, 34368,
                55864, 13596, 31680, 61924, 45168, 2088,  30096, 24952, 60776,
                37700, 57256, 42952, 50504, 34496, 16364, 9564,  48392, 27960,
                35164, 52784, 9060,  58316, 21832, 3096,  35432, 65328, 9800,
                26164, 53380, 48524, 33552, 57360, 15280, 53096, 43740, 38200,
                16608, 51800, 45024, 30448, 54672, 39396, 17964, 32460, 28312,
                6988,  60424, 34016, 55640, 756,   48896, 5104,  38164, 57932,
                33636, 42000, 24964, 8272,  60300, 16596, 46576, 52788, 18808,
                14468, 44088, 56732, 11324, 42104, 51844, 7284,  29452, 43384,
                49236, 33604,
            },
            {
                7112,  45912, 55276, 8696,  41064, 47228, 63276, 34892, 58212,
                24224, 42948, 39708, 420,   55352, 31752, 42504, 36488, 52116,
                56828, 3896,  49160, 64760, 15332, 41008, 61796, 56808, 48520,
                26056, 5508,  28424, 18068, 45608, 30268, 14196, 51996, 34192,
                46924, 54520, 37320, 26812, 51936, 46620, 32180, 2852,  44340,
                59116, 31336, 25496, 18044, 6228,  42244, 26452, 38024, 672,
                46160, 21592, 58060, 5348,  50064, 39972, 58824, 47816, 6292,
                52976, 20948, 8796,  28648, 54196, 6020,  57960, 43176, 12600,
                60964, 22980, 45560, 29736, 48808, 54436, 16372, 51572, 40104,
                58384, 5268,  13072, 2440,  45276, 61492, 5600,  29528, 56944,
                7988,  27432, 41520, 10952, 48272, 5928,  60020, 43748, 55928,
                14452, 41812, 17376, 9896,  64152, 21836, 12360, 45996, 15732,
                52604, 19892, 12760, 50764, 38544, 33912, 10568, 41628, 25360,
                35016, 5048,  63428, 26728, 3660,  32524, 15892, 36716, 10912,
                62232, 24888,
            },
            {
                52572, 12316, 28644, 17008, 57244, 26544, 11760, 38376, 3428,
                9924,  47676, 27644, 16900, 45740, 12240, 58680, 8352,  14140,
                29096, 23668, 7880,  26488, 45808, 9916,  30760, 23032, 8236,
                44668, 55592, 60940, 10492, 3184,  57788, 5604,  38312, 64932,
                15648, 3508,  11340, 63012, 4468,  8748,  19448, 65228, 49716,
                21416, 412,   54496, 38944, 58664, 50984, 31120, 63548, 18300,
                40824, 9816,  36008, 27084, 14492, 20308, 34680, 11364, 15948,
                32360, 64196, 36140, 14024, 62320, 46496, 31392, 19728, 55560,
                6472,  32564, 876,   14316, 37232, 7560,  30776, 20840, 532,
                28792, 43012, 60272, 31332, 25652, 19620, 36484, 11204, 23136,
                63584, 34308, 1620,  58556, 22612, 13492, 35608, 296,   48804,
                21080, 36672, 47520, 27664, 39044, 32032, 60728, 24508, 29324,
                6796,  63032, 31532, 56912, 27348, 4540,  55264, 61448, 1180,
                58396, 49704, 16832, 38508, 54064, 61564, 24264, 57104, 2492,
                54904, 16616,
            },
            {
                41788, 172,   65380, 36552, 3812,  20720, 50316, 44824, 18228,
                61176, 51260, 6588,  65204, 21408, 4516,  49036, 19676, 63616,
                41564, 37852, 59756, 55752, 20184, 39048, 52108, 17688, 41828,
                13024, 21796, 40280, 51496, 35792, 43096, 18920, 22212, 8888,
                31404, 24748, 44736, 33796, 22368, 55904, 38664, 11700, 28252,
                36960, 14148, 47964, 10884, 34656, 2960,  14448, 8656,  55000,
                24864, 61424, 51412, 43488, 2932,  64716, 56224, 28244, 46008,
                23708, 4808,  44752, 26112, 324,   38772, 23856, 3512,  37564,
                41840, 49536, 62948, 25312, 40804, 61648, 46636, 56936, 34180,
                49492, 18184, 23076, 38744, 9048,  49840, 55652, 32560, 45892,
                13744, 49584, 18600, 37540, 64840, 29280, 51748, 24984, 11072,
                63760, 2668,  24188, 58204, 7572,  42284, 50516, 3336,  55596,
                36868, 44992, 4,     17652, 47128, 23492, 13680, 31660, 39544,
                45328, 21888, 31204, 7896,  19600, 46296, 13128, 48376, 20384,
                38340, 30896,
            },
            {
                47292, 58716, 22704, 44360, 31776, 59580, 7020,  28836, 32696,
                22652, 14308, 35612, 30568, 52836, 39040, 25964, 33628, 1940,
                54124, 10948, 32932, 520,   13540, 35704, 3260,  64016, 34076,
                53528, 684,   30976, 14596, 63940, 26792, 54288, 40904, 59960,
                48936, 57180, 39956, 17112, 60136, 29864, 42884, 53328, 6464,
                56664, 61804, 43048, 23764, 28708, 60912, 20896, 43816, 49152,
                3928,  29564, 16812, 54576, 23392, 38328, 7844,  51932, 1396,
                39284, 55764, 50044, 17544, 60052, 10668, 52500, 65180, 15372,
                26912, 17872, 10472, 55224, 19612, 2568,  12412, 24628, 8804,
                15144, 62700, 6196,  52540, 41964, 64160, 17024, 1096,  40732,
                54388, 6376,  25384, 53008, 8536,  46284, 19536, 58068, 39676,
                30620, 52388, 44192, 13256, 54500, 16100, 34700, 19052, 10604,
                40364, 22784, 59640, 9220,  42116, 63836, 51132, 20268, 7544,
                28416, 12348, 55844, 35548, 42748, 592,   28156, 33424, 60848,
                9064,  26256,
            },
            {
                19084, 34036, 14192, 9696,  53544, 39736, 15980, 64360, 54588,
                2168,  41460, 59632, 8884,  43648, 56372, 15628, 60536, 45376,
                22268, 17176, 50644, 42780, 62376, 24524, 48760, 6896,  27812,
                60436, 37612, 47632, 7188,  23504, 49412, 1480,  12716, 28128,
                6700,  20296, 252,   50948, 13576, 48128, 1888,  16340, 25624,
                33868, 19168, 4636,  52596, 16544, 46052, 56596, 36108, 32496,
                12288, 39660, 6216,  33784, 10892, 31148, 18324, 42124, 60656,
                19772, 11900, 29984, 41420, 21928, 33836, 44132, 29500, 8032,
                51380, 58788, 44848, 33744, 28308, 51928, 43732, 59120, 39172,
                45620, 55080, 36124, 29180, 14272, 4484,  46948, 27992, 20328,
                61396, 30348, 44100, 15548, 32640, 2976,  42396, 5224,  33576,
                16756, 61344, 4924,  20288, 30016, 1244,  58888, 47064, 62024,
                27232, 13848, 49228, 34616, 29208, 2164,  37580, 16396, 48460,
                65492, 2364,  59996, 51036, 10060, 64132, 39908, 5648,  43716,
                50668, 4624,
            },
            {
                56408, 37844, 48780, 62492, 24696, 1372,  42756, 10520, 47160,
                37400, 57236, 24776, 18688, 1364,  28716, 10488, 36316, 5576,
                30704, 47624, 27624, 6272,  53736, 29812, 57892, 15528, 46452,
                11212, 20240, 55992, 16748, 34564, 9756,  61808, 32532, 43412,
                53276, 35460, 64376, 27652, 7360,  36432, 23188, 63664, 40544,
                46248, 8984,  32396, 64284, 37708, 7388,  26180, 1708,  65484,
                22252, 58408, 47364, 63212, 44520, 59488, 48972, 13380, 26688,
                33584, 63640, 7624,  57808, 4188,  48632, 13324, 56484, 35856,
                21600, 3236,  38784, 5884,  64576, 15880, 35748, 4088,  31400,
                26796, 1716,  11664, 21328, 60016, 34552, 24452, 58048, 9984,
                36676, 4124,  59304, 38940, 56884, 27508, 63052, 14016, 54920,
                8092,  26852, 36264, 48668, 65496, 38388, 25936, 5692,  52196,
                32988, 3708,  54216, 20664, 56420, 11620, 60768, 33324, 53568,
                24784, 40600, 15540, 26332, 22288, 48952, 17580, 58780, 23300,
                12076, 63268,
            },
            {
                28060, 2400,  7444,  29596, 18668, 51588, 35196, 26048, 19952,
                5660,  49076, 12104, 33476, 46096, 63988, 23312, 51816, 40956,
                57116, 65076, 14640, 21404, 39876, 10032, 19056, 43428, 33040,
                2636,  26160, 62804, 43752, 29048, 58480, 37800, 17568, 47856,
                14608, 24228, 10816, 45972, 54544, 31912, 59404, 4180,  51280,
                12264, 58308, 22264, 1024,  41052, 12784, 51060, 17856, 42468,
                52760, 14944, 26924, 20024, 276,   25240, 5028,  36592, 54372,
                3464,  46528, 16152, 37252, 53648, 27568, 18424, 1468,  61856,
                46852, 30952, 13672, 23848, 48720, 9076,  22476, 61440, 19180,
                50608, 64896, 43476, 48700, 56468, 8240,  39436, 52216, 15308,
                48648, 24084, 12156, 21436, 47176, 10212, 35976, 22388, 45212,
                49644, 41284, 10800, 23452, 53120, 45152, 12556, 21652, 42696,
                17132, 64712, 43928, 5996,  39356, 25880, 44676, 3520,  10112,
                36352, 47140, 5248,  33984, 53816, 30792, 14036, 55064, 36588,
                32332, 15932,
            },
            {
                41620, 22184, 54860, 39240, 46460, 13292, 56304, 60132, 31368,
                15364, 27776, 52212, 60724, 38156, 5092,  48212, 13768, 19300,
                8940,  2548,  35512, 44960, 61220, 4380,  37912, 52404, 22768,
                59012, 41136, 6120,  52624, 3496,  46172, 21980, 5840,  56768,
                3144,  39108, 60808, 41760, 18344, 9656,  44276, 20536, 28604,
                35768, 49504, 27156, 54212, 30172, 47196, 58800, 10268, 29104,
                4672,  37728, 8628,  50892, 40916, 56520, 15896, 62240, 22136,
                43112, 51540, 24348, 32120, 9436,  64060, 38984, 24484, 41540,
                9788,  53540, 60024, 42876, 56068, 30092, 41232, 52872, 13268,
                8044,  23736, 33284, 3552,  17980, 30596, 44396, 2672,  65500,
                33464, 41724, 62400, 880,   54264, 17400, 51192, 60252, 29504,
                19032, 59104, 3712,  33272, 16920, 8268,  32000, 57184, 28852,
                9548,  36204, 24120, 14856, 52332, 17924, 57628, 29996, 22556,
                59264, 18884, 43956, 62788, 2944,  38892, 7688,  26548, 2008,
                52504, 45412,
            },
            {
                60416, 49928, 11548, 61252, 33060, 5136,  22028, 8288,  44028,
                63236, 40036, 3732,  21932, 16476, 54480, 32672, 26892, 62196,
                38400, 24844, 51388, 11392, 31300, 56628, 25548, 64528, 7592,
                50584, 30168, 14164, 36180, 19480, 11876, 54040, 26360, 63648,
                30296, 19880, 33400, 5388,  25888, 57432, 38372, 14476, 62064,
                6828,  17804, 43544, 15036, 62624, 20468, 33824, 24168, 61068,
                45444, 31656, 57580, 35440, 11772, 30208, 46764, 8112,  28628,
                10824, 39128, 1144,  59332, 20356, 45136, 5804,  50424, 32476,
                16468, 26456, 19692, 1124,  37164, 17764, 5412,  46432, 57820,
                37420, 40720, 15220, 53984, 26240, 62108, 12724, 21560, 28620,
                6840,  18312, 50420, 31168, 7396,  25796, 40740, 1844,  6100,
                37996, 13548, 56104, 43076, 59852, 2936,  62876, 38924, 50208,
                776,   60152, 46808, 31792, 63104, 8324,  49128, 41340, 14108,
                55244, 7412,  28348, 12504, 20852, 57052, 43084, 47508, 64792,
                19896, 9284,
            },
            {
                25404, 31416, 17708, 3292,  26976, 64172, 37224, 50360, 556,
                54052, 35260, 9900,  43192, 30076, 8012,  59316, 160,   44476,
                55456, 29232, 59148, 17832, 47408, 1112,  14940, 35252, 11964,
                17992, 38496, 47768, 24324, 65208, 32052, 39580, 42580, 9340,
                51544, 46980, 58412, 15640, 49284, 1640,  51892, 30768, 55168,
                3196,  38896, 60516, 9616,  2608,  36896, 5396,  40356, 55008,
                19560, 2680,  62748, 16996, 22760, 52840, 38628, 33128, 58768,
                17888, 63168, 29776, 48180, 14792, 34836, 60472, 11956, 57564,
                4024,  63832, 35068, 50744, 11396, 62936, 25660, 33808, 772,
                28536, 60688, 47660, 6428,  42384, 35592, 50600, 55116, 37672,
                46384, 56444, 27236, 36328, 44624, 64092, 32716, 12388, 24356,
                62932, 51944, 21040, 28140, 36564, 25064, 46388, 14220, 20188,
                56004, 40876, 11944, 28048, 2684,  37172, 21036, 5100,  63524,
                32204, 39316, 52288, 48320, 60108, 35380, 18156, 11232, 29536,
                39560, 4084,
            },
            {
                14700, 36776, 44696, 57860, 40976, 15808, 45960, 29244, 12304,
                18448, 23488, 57512, 62372, 50404, 20052, 41676, 11616, 34432,
                16204, 6648,  41992, 23264, 34072, 54796, 28292, 48608, 42156,
                57136, 76,    60900, 9120,  55620, 1916,  49792, 14320, 34592,
                23332, 864,   12140, 29036, 63396, 35256, 23600, 11192, 42184,
                25352, 48504, 33256, 23112, 52084, 56684, 16036, 50088, 8212,
                13708, 48044, 25532, 42076, 5464,  64572, 1628,  20376, 49572,
                5972,  53468, 12408, 42236, 4628,  54472, 22908, 28756, 43908,
                37696, 47884, 7136,  28164, 58372, 45112, 14696, 55052, 21540,
                10608, 18420, 31488, 57120, 22800, 9680,  228,   17196, 59876,
                10628, 3080,  13316, 59080, 20500, 15356, 49064, 57692, 35092,
                45868, 31008, 328,   48032, 10020, 18812, 54016, 6596,  34596,
                26532, 5128,  18360, 51416, 59476, 34124, 45828, 26752, 50776,
                312,   24128, 16448, 9540,  31468, 25292, 924,   51140, 58896,
                33832, 55908,
            },
            {
                62032, 51888, 7976,  21256, 53604, 10152, 24960, 59344, 42008,
                32832, 48040, 26584, 1840,  14032, 35868, 24368, 47500, 53580,
                21868, 49672, 2880,  63772, 8712,  40116, 19940, 5492,  61896,
                21792, 27400, 33608, 43504, 16400, 28812, 60488, 4392,  18540,
                62640, 37408, 55136, 44704, 39740, 18892, 7680,  46196, 65272,
                20072, 57144, 13068, 45372, 26484, 42592, 31084, 64624, 27728,
                38532, 34116, 54236, 9824,  44932, 27460, 13064, 56112, 43984,
                25668, 34440, 21356, 57336, 27260, 39960, 17384, 8292,  52548,
                21212, 13364, 42228, 23176, 32584, 3364,  38092, 48512, 64412,
                43016, 52020, 2308,  13384, 38264, 63560, 48144, 32800, 25616,
                43508, 22748, 52768, 39432, 5044,  9288,  42320, 53988, 18220,
                7656,  40376, 15836, 63876, 51480, 41488, 61184, 30320, 44284,
                64308, 48572, 22732, 55156, 42564, 9404,  16684, 61028, 11524,
                44184, 34816, 64580, 41968, 5376,  45800, 62356, 15092, 22020,
                5744,  46720,
            },
            {
                23676, 396,   29700, 34688, 49068, 2076,  19648, 55728, 4548,
                65320, 7440,  16824, 45732, 39228, 56780, 64320, 3388,  30220,
                61336, 14248, 37952, 45772, 12364, 60388, 52040, 44532, 31808,
                10496, 54256, 4892,  51432, 20556, 36544, 25140, 46536, 56924,
                43600, 27392, 8644,  21336, 3280,  59736, 53696, 32808, 16056,
                37128, 80,    29880, 7516,  63164, 4192,  11732, 22380, 44720,
                612,   61780, 20600, 31164, 49964, 60628, 35812, 40524, 15408,
                3868,  47104, 37620, 8788,  61528, 452,   49656, 64520, 31244,
                2388,  56212, 61572, 17096, 9020,  53968, 19404, 29496, 7472,
                24828, 35216, 61596, 27044, 45524, 18888, 29668, 7480,  40512,
                64364, 16580, 33332, 45948, 61340, 30432, 22552, 2836,  27892,
                11336, 55676, 25788, 5180,  32748, 13140, 1656,  23768, 10688,
                15616, 38256, 33312, 1584,  14256, 25108, 56440, 30332, 38556,
                20100, 58020, 3268,  53332, 22668, 56236, 40428, 36420, 27628,
                42924, 11888,
            },
            {
                38684, 65080, 42648, 12976, 59884, 31992, 39860, 35992, 14688,
                51204, 37392, 31156, 54604, 10684, 27956, 6384,  17476, 40284,
                8336,  33224, 57408, 26900, 30644, 16896, 24816, 2224,  37404,
                16128, 46256, 38948, 12468, 58880, 6968,  40256, 10956, 31596,
                5484,  16600, 52204, 33936, 48316, 13804, 26264, 5192,  50408,
                9936,  59988, 54120, 36220, 19240, 39748, 48664, 17220, 58732,
                51248, 5848,  15216, 58000, 3608,  18136, 23732, 8232,  59732,
                31844, 65248, 16644, 51056, 24104, 33172, 46072, 36608, 15060,
                25720, 39796, 33600, 51812, 43972, 62452, 12764, 41108, 3932,
                58996, 15736, 50100, 40252, 5096,  58444, 54668, 14592, 51472,
                4020,  29080, 56968, 664,   25432, 51284, 38196, 65188, 47432,
                60412, 36864, 44168, 58708, 21432, 39152, 57680, 36064, 53264,
                58668, 8008,  29192, 60764, 39780, 48084, 3396,  52016, 7248,
                47244, 14440, 27848, 37632, 17620, 30104, 6640,  9636,  54736,
                50428, 16956,
            },
            {
                28284, 53812, 18788, 25848, 6368,  63540, 47136, 8580,  27580,
                21160, 43784, 61892, 4264,  22848, 33640, 52228, 59472, 44616,
                25300, 51492, 19288, 5052,  49144, 36308, 7352,  64684, 56132,
                26308, 62756, 23036, 30408, 47560, 64256, 53868, 21420, 50496,
                61444, 41092, 24596, 64636, 30112, 42372, 56740, 38816, 28904,
                41680, 22688, 47004, 14144, 51516, 60684, 32372, 9332,  35604,
                24836, 29696, 41436, 37276, 11720, 32764, 48156, 52524, 28316,
                11100, 54712, 2616,  29924, 43960, 11280, 20040, 6484,  54048,
                60100, 10840, 5148,  28952, 84,    25068, 49600, 32028, 55788,
                45788, 9112,  20848, 32940, 11496, 24552, 35476, 21236, 61548,
                37160, 49008, 10076, 18996, 35876, 12160, 16740, 5748,  32540,
                23476, 19348, 50084, 9164,  29168, 46884, 16840, 49700, 3788,
                21104, 45792, 51000, 11296, 20672, 64744, 32792, 17704, 23636,
                62488, 33188, 10536, 49744, 61304, 13176, 48424, 63724, 19860,
                32156, 4648,
            },
            {
                48020, 8844,  36644, 57064, 44048, 16800, 22984, 52916, 60736,
                928,   11308, 57748, 18716, 49916, 42364, 13632, 21116, 36824,
                11328, 65244, 1508,  42588, 54892, 59324, 21340, 41372, 12980,
                49868, 7924,  3008,  42012, 18104, 800,   14868, 35324, 29144,
                1880,  12628, 55952, 6168,  10460, 17972, 61400, 21016, 3052,
                63864, 17108, 33956, 3620,  28204, 24180, 1544,  56448, 42964,
                6924,  53716, 47332, 22104, 55196, 63088, 42572, 548,   21488,
                45736, 39732, 19432, 59132, 14132, 55468, 62940, 27804, 42992,
                18020, 49364, 45908, 20764, 38480, 59224, 35636, 22124, 17328,
                37464, 28296, 2660,  64512, 53208, 42940, 1424,  46724, 12524,
                5932,  24024, 41548, 63296, 55084, 46756, 58812, 42936, 52716,
                1320,  14124, 34572, 3516,  64756, 24676, 6504,  42944, 26644,
                31700, 63136, 17840, 43560, 27556, 4848,  37228, 43040, 57212,
                1744,  54440, 44808, 21576, 4268,  34352, 26168, 43880, 2160,
                57624, 60564,
            },
            {
                21664, 14804, 2524,  50332, 29440, 3844,  12216, 30856, 41476,
                48996, 25288, 29548, 40024, 8544,  63792, 47084, 1968,  28844,
                56268, 46356, 23352, 32440, 15108, 10368, 46788, 33760, 29372,
                19212, 35908, 53112, 57940, 33840, 27608, 44804, 8556,  58676,
                38412, 48864, 19804, 36900, 47092, 728,   35312, 12756, 52628,
                44688, 25636, 58908, 8648,  44056, 54484, 37732, 20260, 62028,
                12996, 18416, 64724, 1736,  25868, 7536,  38468, 17080, 61824,
                34832, 25412, 7276,  41764, 35880, 4472,  48256, 38688, 1404,
                23228, 35192, 65476, 57192, 15688, 9980,  5920,  63776, 11652,
                51656, 41900, 56528, 47452, 17440, 8260,  60056, 31520, 27484,
                57948, 44764, 15076, 31212, 7048,  27032, 21456, 9676,  29296,
                40232, 62192, 53824, 41944, 12152, 55632, 33940, 61460, 13952,
                40964, 0,     6888,  35004, 56160, 52684, 9252,  13132, 29344,
                39520, 25816, 7836,  41212, 59308, 52492, 38932, 15556, 24280,
                35288, 41044,
            },
            {
                30736, 44860, 61128, 24036, 39512, 55140, 35584, 58312, 17876,
                34108, 14344, 53764, 35732, 3448,  24468, 31716, 53928, 5624,
                16552, 35060, 7556,  61800, 39472, 26676, 472,   53788, 4592,
                45280, 61312, 14544, 24924, 11108, 51212, 62472, 23828, 42616,
                16060, 26652, 32068, 60404, 43800, 54676, 24260, 48468, 31000,
                6612,  11612, 50652, 40028, 63056, 15884, 11244, 47612, 26896,
                33288, 39276, 9608,  28900, 44836, 14496, 30440, 57404, 5572,
                13236, 53744, 63544, 46840, 26600, 21908, 32092, 10408, 58560,
                30276, 7564,  12464, 26400, 41464, 47656, 53268, 27068, 1276,
                60780, 22900, 14236, 30820, 26432, 38048, 50804, 40424, 18236,
                52480, 34448, 20924, 49532, 2772,  39408, 33872, 63520, 16080,
                47988, 6960,  26680, 18172, 31004, 51564, 19904, 9796,  57108,
                36956, 53900, 25260, 60208, 15288, 23028, 45920, 59444, 20428,
                50160, 16176, 65236, 30540, 18912, 440,   10412, 55564, 46916,
                7364,  11764,
            },
            {
                63300, 54148, 33820, 10288, 20036, 62432, 7784,  45008, 3076,
                64660, 5608,  46484, 61464, 16104, 57268, 38188, 18972, 62244,
                40128, 49748, 13096, 29456, 51912, 17952, 63244, 23580, 58728,
                9872,  31516, 39328, 47908, 4952,  37832, 56136, 19352, 3480,
                54548, 63992, 4564,  22568, 14412, 28000, 8368,  62496, 40540,
                57824, 37376, 18752, 32044, 22132, 6304,  30504, 52072, 2256,
                45544, 59312, 50820, 35972, 60996, 53164, 48844, 24488, 37036,
                50272, 30920, 10100, 940,   57300, 16712, 61560, 50096, 14896,
                52852, 44076, 55204, 32240, 2352,  20084, 30148, 42880, 46464,
                33152, 7576,  36340, 4176,  62464, 12176, 22528, 3352,  65132,
                9244,  1048,  61164, 54056, 13924, 58100, 45232, 4060,  55940,
                24148, 37184, 58272, 45616, 1028,  38152, 47936, 4316,  29268,
                22208, 46644, 12460, 39336, 48600, 31632, 1288,  63688, 34184,
                3984,  11788, 36480, 47488, 56532, 28180, 32736, 64300, 18628,
                51668, 27036,
            },
            {
                1120,  17144, 5916,  42232, 47916, 14664, 27228, 50492, 23688,
                39080, 10352, 21308, 43344, 27004, 12196, 48740, 8892,  43844,
                26368, 58984, 21604, 55428, 6332,  43684, 48580, 37720, 15956,
                42256, 20332, 1752,  65472, 22068, 30892, 6996,  49108, 33392,
                11712, 45768, 9132,  41252, 51068, 58520, 33720, 19644, 15740,
                1292,  28444, 56108, 2624,  49088, 59900, 41016, 63888, 23516,
                15228, 5408,  21772, 17232, 3020,  11084, 20160, 43612, 2092,
                60696, 22424, 18352, 33772, 52048, 40240, 7900,  42224, 25380,
                36696, 19040, 5012,  50300, 62992, 37300, 56172, 13828, 18408,
                39400, 58352, 49876, 20120, 45264, 57276, 34164, 15476, 43356,
                25044, 37856, 28400, 41236, 8588,  24716, 18908, 50880, 12072,
                32208, 20568, 10012, 61036, 14536, 23456, 63508, 43680, 16256,
                51884, 65184, 9032,  28668, 5740,  18664, 41168, 26072, 53188,
                44148, 58172, 23788, 5688,  14292, 45284, 22572, 41916, 4768,
                58016, 38564,
            },
            {
                49320, 25128, 59696, 52092, 32176, 124,   37608, 19216, 54316,
                28720, 58572, 32136, 51972, 512,   34348, 60568, 22904, 30812,
                1200,  10272, 45744, 37120, 2560,  34224, 12124, 28948, 6644,
                50852, 57088, 27888, 53672, 43060, 16948, 13172, 61412, 40008,
                29612, 52508, 35496, 17788, 2044,  38476, 5812,  46260, 53588,
                23140, 65168, 46652, 12968, 25776, 35176, 18092, 4240,  36972,
                57568, 31788, 43388, 55768, 41076, 34372, 64368, 8512,  28556,
                15380, 42148, 48340, 38032, 12908, 29320, 4184,  20744, 63804,
                1924,  60368, 39844, 23524, 16732, 8700,  59944, 24380, 3592,
                65044, 10072, 25008, 54536, 41316, 5568,  28248, 53160, 48352,
                56144, 11852, 17492, 47312, 64428, 36040, 30160, 38472, 61496,
                43452, 2696,  50008, 34756, 41276, 5548,  26864, 58628, 32812,
                1992,  19528, 34380, 56796, 62364, 51364, 36228, 11076, 7600,
                17412, 32268, 40176, 51608, 62268, 8704,  50376, 36028, 13168,
                31348, 20716,
            },
            {
                43464, 36448, 12492, 28868, 22088, 63800, 56832, 11584, 42424,
                15508, 48120, 6500,  17792, 56376, 41880, 4448,  51376, 15148,
                63384, 33156, 18816, 57304, 25200, 61084, 20652, 54184, 63932,
                24796, 35560, 12620, 8612,  60116, 46392, 35348, 24412, 200,
                21012, 62796, 25504, 56432, 30960, 64352, 11504, 26508, 43468,
                36232, 9388,  33356, 42500, 54836, 8364,  44604, 28800, 54228,
                9856,  48116, 27440, 7312,  24048, 31384, 56656, 47224, 39484,
                58032, 53204, 3240,  65092, 23292, 59100, 54376, 46268, 32728,
                11408, 47928, 29004, 12944, 45064, 33864, 5724,  48016, 31708,
                52720, 28752, 16696, 288,   32056, 10560, 60444, 19664, 7092,
                30316, 59184, 33196, 5272,  21848, 53316, 168,   15212, 6200,
                54784, 28016, 16492, 56556, 29704, 52912, 11376, 36640, 7884,
                40480, 45108, 24360, 3556,  42864, 14056, 22060, 57156, 48048,
                28328, 63316, 19948, 3532,  26772, 38324, 17104, 2068,  61972,
                46176, 8520,
            },
            {
                65440, 5124,  57620, 39376, 8672,  46292, 4556,  33540, 60168,
                1828,  35996, 62596, 23372, 38592, 28360, 20172, 45360, 36840,
                53680, 41156, 3904,  50616, 15652, 40264, 8856,  32668, 1252,
                45724, 17928, 40724, 32860, 2452,  26592, 58416, 50388, 43904,
                15588, 7268,  47972, 13232, 42824, 20516, 49384, 59272, 3668,
                14084, 50816, 20632, 4908,  16860, 58704, 50340, 13556, 21020,
                40048, 65128, 12648, 60396, 52272, 14244, 18536, 4544,  25564,
                11960, 32232, 6848,  26724, 9272,  43008, 34784, 17364, 27328,
                56264, 7408,  35452, 58240, 51700, 26224, 54308, 40900, 21096,
                12736, 43524, 48380, 38848, 63820, 14552, 45752, 36060, 39852,
                22876, 3816,  50772, 44232, 27064, 10932, 59860, 48556, 25600,
                40496, 64972, 46684, 7512,  21804, 61840, 18656, 49280, 55692,
                13608, 60972, 49768, 16868, 54432, 30048, 2784,  60324, 38052,
                760,   54640, 10028, 42972, 33688, 55504, 59908, 24664, 28600,
                54652, 14876,
            },
            {
                33180, 27412, 19028, 53484, 15584, 41384, 24364, 29800, 51308,
                9368,  26316, 45496, 13332, 7804,  64952, 11088, 57772, 5452,
                23836, 12884, 27640, 64808, 31032, 47448, 57848, 43300, 15044,
                49864, 5428,  62716, 52372, 20096, 39116, 9832,  5220,  53972,
                28096, 57636, 37096, 4044,  23684, 54352, 29020, 17044, 32108,
                61168, 56536, 28068, 62460, 37948, 31028, 23916, 60876, 1148,
                34500, 18296, 3888,  38104, 45748, 72,    36736, 50576, 61048,
                21280, 35752, 44588, 60348, 19456, 49764, 1176,  13824, 51616,
                41100, 21508, 64868, 2476,  20236, 10744, 61456, 1068,  35968,
                62348, 7052,  34696, 56180, 22600, 51048, 26248, 2144,  63084,
                13252, 41904, 61752, 15872, 56816, 39724, 45480, 33620, 17844,
                9180,  22976, 32852, 13204, 38420, 45400, 504,   31612, 25800,
                21128, 30464, 36796, 10200, 26512, 47272, 33404, 42084, 13352,
                24584, 45692, 15620, 60892, 21696, 11836, 44236, 6404,  52060,
                39880, 22756,
            },
        },
        {
            {
                20112, 58232, 25836, 15280, 50004, 18008, 44896, 35592, 468,
                53820, 38276, 25844, 17700, 52596, 40092, 26628, 43720, 19984,
                48276, 27220, 56376, 44404, 47488, 12064, 32904, 15720, 49808,
                38208, 33864, 12512, 41464, 28436, 9768,  55228, 49364, 60668,
                8128,  62868, 19996, 37996, 13084, 7580,  45556, 41388, 36112,
                58260, 49404, 23428, 5468,  15908, 51292, 8412,  37764, 18408,
                43404, 14380, 32252, 45292, 54924, 39736, 10916, 21732, 5772,
                36900, 23792, 156,   31712, 62900, 12264, 4072,  39076, 21972,
                54996, 42264, 4956,  29680, 60620, 48952, 25600, 8572,  13476,
                18640, 2488,  29648, 50948, 58684, 45648, 11352, 63728, 33044,
                57180, 6608,  14300, 45276, 22528, 48976, 60452, 20748, 2656,
                25488, 35132, 17092, 1664,  5924,  26264, 9696,  29548, 6808,
                65176, 51340, 25772, 14164, 35368, 18916, 39668, 23708, 13140,
                25812, 38396, 7248,  22588, 32172, 27584, 52832, 18480, 34384,
                42288, 29024,
            },
            {
                39980, 12508, 44072, 62852, 29632, 41120, 58716, 27260, 46948,
                23476, 11792, 60744, 8992,  34760, 3232,  12336, 60104, 9788,
                1372,  63888, 17676, 30624, 7780,  37300, 53676, 23332, 5676,
                62640, 54424, 24784, 528,   60300, 32720, 17844, 4688,  26496,
                43240, 16068, 54536, 28312, 64708, 34000, 17824, 3952,  30024,
                8164,  18380, 52848, 56928, 41888, 65260, 25260, 47920, 62196,
                53844, 40748, 5232,  20808, 49604, 944,   58076, 41800, 61504,
                17172, 55612, 40880, 58772, 6112,  44924, 24732, 61584, 30932,
                16364, 9548,  33340, 53312, 45460, 17980, 34640, 41016, 30532,
                61396, 24976, 42204, 7616,  14956, 3324,  20444, 39100, 8848,
                17840, 50188, 60772, 164,   53608, 4748,  41524, 29028, 45144,
                65432, 10744, 53292, 28520, 49772, 21308, 43372, 53068, 46732,
                16968, 32336, 44012, 444,   55156, 31112, 64728, 4696,  57740,
                51908, 62400, 34344, 50240, 64608, 10084, 36680, 57868, 49384,
                15780, 55036,
            },
            {
                32612, 48804, 1672,  37068, 8536,  4112,  13320, 61792, 5564,
                31692, 57840, 49568, 28876, 22364, 63264, 46968, 29932, 36864,
                54752, 33484, 40464, 4404,  52056, 25984, 64964, 42616, 10256,
                28768, 45624, 16616, 48256, 22420, 43780, 13156, 37196, 47132,
                21676, 34960, 50748, 308,   40492, 25648, 52220, 48368, 22064,
                63668, 39692, 13060, 2344,  34844, 20184, 31184, 1684,  6548,
                34300, 23864, 59252, 62596, 29500, 25100, 14836, 30900, 51396,
                4132,  34508, 48264, 10124, 18560, 33840, 53100, 13884, 2928,
                36592, 59376, 19716, 26840, 12060, 6556,  57700, 1316,  53792,
                45904, 38576, 57136, 22196, 34580, 30872, 43748, 51876, 59808,
                27668, 21700, 38044, 32240, 26360, 35656, 13620, 58244, 33100,
                6688,  22484, 43728, 60644, 37060, 63768, 15624, 2392,  24232,
                56764, 4540,  40104, 22116, 46024, 8852,  27020, 48204, 17576,
                9368,  28972, 14144, 41908, 4116,  46952, 24300, 6280,  62296,
                22748, 9312,
            },
            {
                59852, 18256, 23464, 52508, 57164, 25292, 32948, 51072, 18868,
                40308, 15972, 44600, 4808,  42152, 55824, 18464, 6308,  22856,
                15436, 50584, 24328, 11464, 59608, 19012, 1452,  31444, 57400,
                20388, 3948,  37128, 56844, 6292,  53408, 63460, 30148, 58448,
                11064, 5576,  31088, 45008, 10036, 57660, 14468, 60048, 11084,
                44248, 32912, 28472, 61092, 9672,  46184, 58140, 13732, 28084,
                50332, 10808, 37288, 16404, 8248,  35608, 64736, 46564, 9540,
                27384, 20976, 14408, 29400, 37960, 56976, 47668, 27736, 43356,
                51724, 48500, 924,   63220, 43188, 37408, 23220, 64872, 20660,
                4800,  16328, 11052, 48632, 53936, 63064, 26392, 13236, 2476,
                47220, 42428, 10736, 56160, 16708, 63372, 9476,  47632, 18040,
                54384, 39120, 14312, 4900,  31872, 11208, 55392, 33684, 38744,
                12916, 62352, 28604, 51888, 60800, 15700, 56260, 42672, 36048,
                21416, 45448, 2212,  19952, 57308, 16936, 39548, 13340, 28272,
                2684,  45896,
            },
            {
                5416,  30004, 63956, 11672, 20044, 42900, 55264, 37428, 9188,
                63348, 2584,  35792, 53280, 13772, 32236, 38720, 49836, 61284,
                44884, 3064,  62588, 36248, 43512, 48660, 15228, 40032, 51016,
                12932, 33348, 64436, 9492,  26164, 39784, 19496, 1356,  52444,
                24836, 62360, 56264, 18532, 23156, 3072,  32216, 37912, 26520,
                784,   55172, 47956, 24576, 37152, 16268, 53472, 38892, 44712,
                17704, 56140, 2676,  47040, 52796, 43136, 3356,  18760, 38792,
                60740, 44036, 53648, 65128, 23432, 1308,  8624,  20668, 64064,
                11372, 24956, 39660, 55864, 15308, 48056, 28616, 10216, 49676,
                34164, 27408, 62140, 324,   37636, 6760,  17372, 54780, 35600,
                30668, 63968, 5136,  20004, 44596, 50604, 23876, 37828, 856,
                26544, 50016, 57932, 19392, 24904, 48140, 41480, 7704,  59692,
                20012, 35588, 7208,  18280, 11104, 37524, 1932,  30308, 6224,
                54328, 39036, 61588, 25288, 53652, 30492, 61080, 43852, 33608,
                52608, 37988,
            },
            {
                15200, 50536, 40672, 35260, 46812, 900,   15496, 22640, 48768,
                29428, 56912, 19708, 26976, 8272,  59464, 748,   25520, 41636,
                12680, 28904, 20588, 56708, 8448,  27348, 34800, 6368,  60052,
                25456, 43024, 51700, 29880, 14976, 34624, 49824, 9052,  41836,
                36032, 13812, 39244, 48612, 61052, 43436, 54480, 6236,  19640,
                62224, 15472, 7140,  21000, 43012, 4052,  63548, 7884,  21360,
                61620, 31800, 27252, 40072, 23004, 11540, 55080, 33204, 24224,
                56572, 2096,  6508,  41904, 50520, 16508, 60532, 40636, 32448,
                17552, 7216,  34608, 21536, 3724,  32232, 52840, 41988, 14252,
                58124, 43964, 31392, 19148, 46200, 23836, 40372, 58756, 6136,
                15892, 24928, 51716, 39136, 2300,  29772, 6996,  55600, 61996,
                30484, 9908,  34464, 46224, 1416,  61148, 17412, 30228, 50624,
                25884, 47608, 54016, 41884, 33260, 49736, 24908, 63376, 12500,
                58236, 31864, 48640, 11164, 35672, 8440,  820,   56036, 18572,
                64712, 25116,
            },
            {
                56524, 10148, 3256,  26424, 58112, 30696, 61168, 6344,  43660,
                24840, 11252, 47188, 65104, 21788, 51312, 16692, 9976,  31324,
                58388, 6944,  47152, 32428, 13824, 54204, 63500, 21828, 47048,
                37692, 1088,  18752, 60468, 46228, 3320,  61248, 22980, 16420,
                46492, 2716,  27316, 7160,  34364, 16084, 27932, 50596, 46364,
                35460, 40856, 52268, 58932, 49540, 29896, 25852, 33696, 51692,
                42272, 5824,  13516, 64008, 19876, 59616, 48328, 7220,  14036,
                49564, 36292, 12376, 32124, 26448, 45868, 35760, 5396,  57692,
                46784, 54224, 28900, 59048, 9104,  62324, 18816, 5752,  38084,
                24028, 8056,  51208, 12740, 59868, 29388, 9772,  49532, 22368,
                45052, 57512, 13032, 61828, 33888, 58620, 41976, 20612, 14020,
                42852, 3496,  64332, 40172, 28988, 52140, 22452, 3700,  44656,
                11560, 988,   63948, 23296, 3808,  59300, 44632, 18956, 51060,
                23596, 15652, 4792,  28112, 42452, 51336, 22008, 48304, 11300,
                7360,  42580,
            },
            {
                47068, 34200, 61692, 16816, 7984,  39180, 52124, 12972, 33844,
                54724, 38180, 1548,  41004, 33368, 44460, 36572, 62464, 52296,
                18732, 39188, 53124, 248,   23424, 41620, 3400,  30396, 17032,
                8288,  55828, 23624, 11488, 41212, 54936, 28100, 32652, 57548,
                64012, 30460, 53372, 21484, 11968, 65348, 41520, 9552,  56952,
                23660, 12168, 31400, 2936,  10368, 18360, 57048, 12236, 1256,
                24480, 48460, 57348, 34684, 316,   30580, 26340, 41632, 61576,
                29324, 17944, 58748, 21604, 9596,  55684, 13696, 29588, 22496,
                1920,  42092, 13272, 50008, 44408, 36484, 26512, 60308, 54768,
                1964,  64316, 36148, 55872, 3932,  42564, 33116, 65328, 912,
                37284, 32152, 8664,  47440, 26484, 15736, 10920, 35336, 46936,
                52976, 22200, 16676, 55972, 13376, 9108,  36720, 63556, 58132,
                39524, 31952, 16180, 56016, 29640, 13780, 39448, 8608,  34592,
                180,   43380, 65280, 19268, 59524, 14404, 38328, 26764, 31500,
                36352, 20804,
            },
            {
                28096, 53172, 22724, 44224, 49124, 19356, 27888, 63620, 4448,
                17224, 59128, 30436, 15076, 6000,  57144, 27536, 46596, 4756,
                34556, 24776, 64664, 16852, 37460, 58980, 45616, 11688, 51988,
                62808, 32632, 49068, 5512,  36140, 19980, 7548,  12016, 43616,
                5724,  18024, 49912, 58824, 37168, 24696, 4308,  30608, 17540,
                1944,  64304, 26128, 38804, 61772, 35248, 46376, 40020, 54844,
                15584, 37748, 9700,  45184, 51196, 15264, 37536, 3996,  21192,
                8896,  46508, 40148, 51436, 62952, 3460,  39384, 52576, 62192,
                26096, 37568, 64780, 17104, 24408, 544,   11888, 47008, 29920,
                16788, 41176, 21752, 26632, 15456, 47748, 20372, 12316, 53040,
                27292, 41560, 18972, 3416,  21948, 52316, 64984, 28164, 4752,
                59960, 37456, 25532, 5876,  33092, 43428, 54444, 14612, 27452,
                19416, 8780,  43992, 37080, 6840,  20900, 53588, 27092, 61352,
                47500, 36804, 53260, 7408,  33160, 45932, 3632,  62172, 52204,
                59240, 1288,
            },
            {
                63388, 6024,  11988, 32808, 59604, 3608,  36176, 45680, 21624,
                50840, 8976,  48512, 61548, 24308, 12076, 2768,  20932, 14500,
                56412, 8720,  44240, 28484, 50128, 9344,  19452, 56804, 40128,
                27808, 14656, 44360, 59084, 25508, 64724, 47916, 37660, 54192,
                25804, 39904, 9688,  44188, 620,   47464, 53552, 39096, 59196,
                43808, 34104, 47416, 14456, 53888, 22624, 4528,  64992, 30544,
                20332, 60416, 28008, 18316, 6612,  42976, 62652, 55656, 52000,
                34772, 64404, 644,   27996, 33932, 18772, 44196, 8120,  15876,
                48940, 10596, 4140,  31396, 57312, 40868, 51604, 33856, 20324,
                45420, 10528, 32492, 50348, 7384,  60828, 39164, 56780, 16964,
                6784,  49996, 61048, 55260, 44200, 37920, 1116,  49460, 18648,
                31972, 10604, 44828, 62840, 48416, 18968, 24316, 5244,  35108,
                46836, 52652, 61004, 25664, 49012, 64476, 45912, 3192,  30572,
                17232, 10196, 26232, 40736, 55380, 24060, 17024, 6492,  40292,
                13124, 17732,
            },
            {
                38748, 30260, 41972, 55372, 14608, 24400, 10380, 57520, 41940,
                26820, 35348, 19188, 42540, 53712, 38608, 50612, 63796, 29672,
                40960, 49048, 12796, 4160,  61500, 33112, 25392, 35548, 6080,
                22320, 2504,  38928, 30144, 16996, 40,    51868, 14816, 21092,
                2272,  34720, 63040, 15236, 32132, 19544, 61460, 13640, 22136,
                5376,  55688, 19760, 7796,  42644, 28716, 16944, 9120,  44212,
                49968, 2380,  33156, 53976, 58476, 24452, 10688, 32072, 16724,
                44280, 23972, 5856,  14652, 48176, 24712, 60304, 31860, 20872,
                56516, 34056, 45252, 19548, 53752, 7964,  15224, 63680, 5320,
                59352, 53688, 3052,  62964, 36896, 28764, 2252,  25160, 46292,
                63024, 29560, 14400, 34188, 9876,  31044, 56944, 24144, 40444,
                7760,  57640, 50996, 28004, 232,   59844, 40480, 49516, 56784,
                1776,  21892, 12836, 4236,  31624, 16624, 11068, 41924, 55176,
                58856, 22124, 49856, 2408,  12256, 63536, 30512, 57152, 47776,
                24860, 45072,
            },
            {
                57936, 51652, 20124, 1852,  64240, 40560, 53860, 31532, 80,
                13744, 64200, 3468,  29152, 7664,  17304, 32144, 43112, 23272,
                1960,  59756, 36268, 21152, 54652, 15000, 1556,  65428, 42408,
                50460, 60812, 53868, 9028,  56696, 33040, 42040, 28620, 61608,
                46028, 55544, 23132, 28064, 57568, 7496,  34928, 50952, 28568,
                10056, 49588, 30272, 60884, 292,   51236, 59024, 36872, 26000,
                5280,  62728, 41364, 12880, 36392, 20488, 48996, 2028,  27416,
                13072, 57072, 37720, 53584, 58148, 11148, 36664, 1076,  51268,
                41012, 5968,  27912, 61484, 36912, 22228, 28952, 49096, 25140,
                38752, 14120, 42720, 23224, 18276, 54804, 44304, 9316,  35780,
                20868, 4444,  40112, 24492, 59872, 7168,  13840, 45552, 62368,
                16028, 34904, 20476, 12772, 38008, 31260, 8416,  11840, 29524,
                65372, 33824, 41592, 59064, 38692, 57204, 34292, 24284, 5880,
                13688, 39388, 32300, 60456, 20468, 36036, 43200, 10296, 21252,
                34412, 3012,
            },
            {
                15252, 26020, 9584,  34956, 28400, 47820, 16524, 7544,  60624,
                38968, 54600, 21916, 45200, 60116, 35776, 58304, 9684,  54928,
                18220, 31772, 26012, 40252, 47536, 29280, 44652, 53436, 17756,
                12116, 34220, 20896, 13556, 45832, 22896, 4744,  58964, 8540,
                31196, 12784, 4644,  52464, 42180, 11376, 25272, 44880, 37364,
                62404, 16164, 40168, 35936, 24584, 11036, 47752, 31888, 14372,
                55324, 22716, 47144, 26784, 3580,  65424, 38660, 45676, 60320,
                6752,  41236, 20136, 30128, 42528, 4856,  46416, 28772, 12448,
                62804, 23240, 47452, 13736, 2540,  58160, 43936, 3712,  35388,
                8696,  27844, 57772, 48788, 11160, 33544, 14848, 51740, 30956,
                57904, 11856, 54172, 47784, 17688, 42888, 21312, 51780, 27776,
                2236,  54832, 41624, 4336,  22704, 52712, 62408, 18156, 43040,
                25296, 15832, 51028, 9412,  28016, 2536,  51760, 19648, 36528,
                63660, 44592, 4632,  28360, 46456, 51412, 504,   27760, 54052,
                49624, 7812,
            },
            {
                37164, 60856, 43208, 50484, 6380,  37816, 20724, 51536, 24964,
                47044, 33064, 10872, 49756, 25840, 1064,  13552, 45944, 5848,
                50228, 62608, 10760, 7260,  58052, 4840,  38096, 23084, 8480,
                27200, 47196, 4088,  37296, 61956, 50932, 11684, 35404, 17300,
                41252, 49760, 20176, 38436, 47836, 64588, 5692,  18484, 1096,
                53112, 22924, 3984,  45716, 57972, 18832, 41068, 63444, 7104,
                39360, 17500, 9236,  31776, 52980, 7868,  29176, 17820, 52540,
                32856, 48572, 9328,  61832, 17336, 22768, 65200, 54292, 38604,
                18020, 58544, 9440,  50884, 39796, 32476, 11460, 52836, 62108,
                17212, 46440, 31576, 484,   39584, 59080, 5284,  63840, 38864,
                23604, 43584, 48,    36332, 28340, 63792, 3212,  35720, 11844,
                47340, 24628, 64828, 44260, 58676, 15196, 46156, 35988, 3616,
                55048, 6572,  46540, 19216, 23068, 43948, 61456, 47604, 29168,
                50660, 9160,  18076, 54820, 14820, 8084,  38824, 61796, 16440,
                31508, 65220,
            },
            {
                22552, 46152, 17628, 56688, 23600, 61424, 12180, 43812, 29332,
                17872, 6348,  57296, 15596, 39312, 65300, 20684, 28168, 35000,
                38800, 15856, 43268, 51348, 19836, 13040, 61800, 31224, 49432,
                57808, 63912, 29664, 42876, 17996, 26876, 40028, 47548, 24648,
                56812, 62388, 35808, 1472,  16668, 29604, 54808, 59928, 41060,
                31480, 11956, 64800, 27152, 14024, 52616, 2316,  21064, 34916,
                51688, 59484, 43180, 61560, 15428, 40520, 56916, 11712, 22404,
                63576, 25564, 36264, 1480,  50812, 33384, 8008,  14908, 26204,
                4260,  35428, 43308, 25284, 16336, 64916, 19292, 23860, 41336,
                56060, 21076, 6460,  65092, 26428, 22388, 46736, 19076, 2904,
                49488, 16212, 62536, 51132, 6444,  32628, 55412, 39596, 61212,
                31512, 6540,  17440, 28948, 33960, 7052,  26584, 50276, 20772,
                60832, 39664, 30096, 63056, 53540, 13000, 7952,  15628, 676,
                25708, 56924, 33768, 41276, 64640, 25460, 19828, 57652, 4128,
                41752, 12052,
            },
            {
                5952,  33096, 608,   13768, 30860, 3728,  34120, 58176, 2048,
                63380, 37136, 24000, 4060,  31004, 42244, 54216, 48316, 24508,
                57076, 800,   30044, 64272, 24896, 35676, 55588, 16292, 500,
                39632, 7156,  24020, 55204, 1376,  7968,  65052, 52972, 2120,
                6936,  27076, 10508, 58908, 23612, 33784, 12904, 48856, 26312,
                7596,  46976, 55884, 33820, 6156,  30904, 44508, 56828, 24432,
                12068, 29472, 1120,  20848, 49372, 24216, 34432, 448,   43252,
                5332,  15916, 58788, 12820, 27304, 39572, 56356, 44308, 48376,
                32028, 53824, 84,    29688, 55408, 7152,  47912, 1320,  29272,
                12568, 37076, 49880, 15976, 53176, 42100, 8016,  28200, 60824,
                33760, 9464,  26460, 20244, 58336, 14864, 25076, 8884,  18580,
                53200, 37444, 50636, 9996,  55936, 39196, 1696,  58188, 32200,
                9972,  13980, 36748, 1364,  33968, 40500, 56080, 32024, 38508,
                60036, 11976, 23020, 1984,  48180, 31028, 12592, 34688, 45404,
                26788, 54956,
            },
            {
                48428, 28180, 52516, 40668, 64516, 46464, 54452, 9036,  41392,
                13528, 53000, 44400, 60792, 50740, 8356,  17572, 3528,  11520,
                61112, 21760, 33936, 46388, 2736,  41544, 9868,  46044, 33452,
                20032, 44084, 15388, 34972, 59804, 31280, 21560, 14292, 33248,
                19252, 44520, 30620, 53432, 46536, 4852,  38952, 20600, 57480,
                35844, 17776, 42492, 21296, 38168, 61132, 10108, 28348, 3424,
                48692, 37552, 56028, 33468, 6268,  44776, 59224, 51508, 37200,
                55608, 31344, 46500, 53324, 43364, 18812, 2400,  61156, 21728,
                11228, 63984, 19680, 59828, 37796, 44444, 34268, 60552, 51484,
                4764,  59256, 44096, 34440, 11932, 30540, 57108, 37188, 13144,
                54828, 40016, 30820, 45652, 11204, 41528, 49896, 45000, 22288,
                872,   60172, 14140, 45804, 20360, 63256, 12696, 41776, 23628,
                53300, 48348, 25636, 57824, 4932,  18336, 24016, 65108, 43096,
                20228, 46048, 35920, 51596, 6312,  60512, 40408, 52548, 9008,
                18400, 59724,
            },
            {
                39464, 20304, 58424, 10048, 25828, 19456, 38640, 27524, 22264,
                48688, 34656, 18764, 27032, 12184, 33524, 62760, 29700, 40532,
                51916, 14368, 8964,  54560, 17272, 28336, 59880, 52208, 25888,
                62292, 11788, 53188, 47904, 10248, 39048, 45468, 55888, 37204,
                63268, 49276, 14776, 40284, 8644,  63804, 43832, 2636,  14544,
                63160, 50124, 9784,  1640,  54072, 47968, 17876, 40108, 64976,
                15864, 45560, 26240, 10912, 64136, 13896, 18444, 27716, 8980,
                20712, 40344, 3108,  23192, 62264, 10484, 30464, 36420, 7024,
                51636, 41492, 14112, 5992,  22032, 10956, 26740, 15476, 39284,
                32356, 25440, 9448,  19844, 60476, 4872,  50652, 16824, 24572,
                47736, 5828,  56728, 1652,  36936, 65440, 3852,  29576, 58044,
                41048, 27080, 32664, 3900,  25348, 48580, 29980, 17136, 44784,
                6036,  64096, 16456, 42500, 49788, 27580, 46868, 10480, 3436,
                53696, 7040,  28836, 58308, 16568, 21768, 3024,  47524, 23668,
                36652, 2452,
            },
            {
                63328, 7528,  35512, 44864, 14844, 6564,  50736, 16408, 61316,
                30504, 3288,  7412,  55644, 21848, 46828, 37528, 56304, 7236,
                43744, 26660, 59308, 38336, 50084, 6316,  22244, 13976, 5264,
                37192, 57468, 2828,  27708, 19696, 51660, 4780,  28464, 11744,
                41804, 4208,  22436, 60376, 18820, 28052, 50688, 25028, 33088,
                6336,  29224, 23936, 62152, 15084, 25376, 35304, 51188, 5664,
                60100, 19288, 4040,  53240, 38912, 30692, 3640,  60732, 46984,
                12348, 64912, 29108, 7632,  34488, 49184, 58072, 25688, 16632,
                38236, 28208, 46444, 50052, 31276, 53348, 63132, 8276,  46148,
                21716, 54100, 63404, 1456,  40544, 22044, 44940, 64084, 2196,
                42556, 21556, 60316, 17436, 23228, 53716, 34424, 16480, 12104,
                47936, 7792,  64312, 42504, 54272, 36660, 8408,  60312, 33852,
                2992,  28792, 21596, 11484, 61364, 7416,  52692, 35416, 30232,
                17352, 63124, 13632, 38316, 44392, 33140, 28044, 56440, 61392,
                30536, 15828,
            },
            {
                12428, 49928, 23268, 31464, 53908, 59440, 33288, 772,   55944,
                11168, 64604, 41984, 38760, 59152, 364,   15032, 23540, 18880,
                35372, 47856, 5032,  20416, 31920, 44676, 65080, 40600, 29764,
                50220, 23032, 32404, 41648, 63820, 16160, 23920, 61412, 7836,
                57580, 25752, 54732, 236,   32196, 36600, 9860,  60948, 54280,
                38480, 45400, 56752, 40616, 30292, 7508,  58652, 12800, 31552,
                23492, 41848, 35680, 57900, 21736, 48084, 41436, 24768, 34052,
                50648, 17236, 57392, 38408, 15052, 21180, 5156,  45048, 54256,
                60204, 23508, 3744,  34980, 57420, 1864,  42008, 18724, 56732,
                2912,  13180, 47940, 36516, 27324, 56132, 32648, 10524, 35604,
                29336, 51600, 12596, 32136, 48492, 27376, 6192,  62220, 38496,
                56336, 19068, 35472, 22692, 15816, 356,   50420, 19832, 56660,
                38716, 46400, 35660, 54156, 31652, 38100, 14604, 22132, 59032,
                50120, 41784, 25492, 9376,  54760, 63988, 10664, 14440, 5548,
                43664, 54148,
            },
            {
                26412, 41476, 1360,  62132, 4216,  42276, 47396, 23956, 37672,
                44936, 20680, 15504, 51992, 29004, 6068,  44496, 50132, 64112,
                2084,  30496, 61468, 12148, 55476, 35824, 9420,  56876, 2012,
                45532, 17804, 7996,  58984, 3920,  36060, 48776, 43604, 31092,
                17020, 34792, 38664, 13332, 51940, 58352, 16664, 46764, 21364,
                916,   17000, 11780, 51540, 3716,  46756, 21452, 43940, 55344,
                9044,  50428, 13384, 28496, 7844,  62940, 15812, 55196, 1468,
                42516, 5976,  25616, 44680, 51588, 63716, 40596, 13224, 1760,
                32376, 9072,  62332, 16124, 39828, 12388, 24720, 49260, 35456,
                28524, 42924, 31144, 17720, 52284, 14388, 4048,  48972, 19532,
                62348, 8336,  37876, 43816, 61308, 9716,  20560, 51828, 31200,
                25164, 4536,  51224, 58828, 11620, 61880, 40352, 26864, 12352,
                51332, 8944,  60152, 780,   19796, 44540, 62716, 1840,  39528,
                4588,  32828, 21080, 48564, 120,   19492, 42480, 38040, 50616,
                21288, 33536,
            },
            {
                46072, 56460, 16464, 37424, 20196, 28540, 13108, 8560,  60196,
                26444, 32352, 48320, 10000, 34856, 61868, 26136, 32380, 10576,
                57340, 16256, 42200, 24932, 648,   15772, 26416, 19244, 33640,
                63068, 14424, 37992, 46900, 25816, 12980, 54276, 1168,  20300,
                50544, 45372, 64508, 7648,  43128, 23592, 5108,  12520, 41992,
                27476, 59232, 35992, 19376, 32840, 62856, 37944, 176,   27008,
                33948, 63936, 46016, 1280,  51668, 33076, 10716, 37404, 21120,
                61260, 32044, 53992, 11284, 332,   31132, 27124, 56136, 47268,
                18964, 42396, 52680, 26376, 20272, 59640, 30444, 4188,  64452,
                16076, 57780, 5784,  61028, 8460,  43860, 25144, 58952, 39776,
                15716, 26656, 54012, 2552,  14828, 40420, 46632, 1296,  43156,
                14996, 9848,  44576, 31780, 28456, 46304, 6640,  32560, 65212,
                24204, 15260, 41824, 26752, 5748,  56368, 25120, 47984, 28136,
                12012, 57200, 60944, 6456,  27216, 51896, 35004, 24744, 1800,
                64892, 9536,
            },
            {
                6004,  59952, 30328, 10556, 51672, 65272, 34848, 18080, 50352,
                5708,  54404, 1656,  24192, 17444, 55140, 13048, 41540, 53480,
                38500, 22024, 52388, 45888, 62932, 39168, 48132, 53400, 42796,
                10708, 60192, 52364, 30084, 56012, 9084,  33516, 40036, 60484,
                10100, 3560,  27464, 20716, 48040, 29796, 39872, 62828, 51120,
                32100, 8508,  65524, 42972, 26248, 10284, 53892, 18296, 60820,
                5308,  16004, 39228, 25220, 18676, 44244, 4500,  58744, 28684,
                13908, 47656, 19424, 36760, 60540, 17488, 8772,  34664, 22164,
                65352, 12072, 37064, 47708, 6356,  44748, 51768, 37652, 22232,
                10460, 39956, 50380, 23408, 37928, 64772, 30132, 53356, 376,
                46036, 57984, 6740,  23712, 34920, 55720, 58976, 22812, 36152,
                63952, 55012, 39224, 3120,  17556, 56092, 21664, 43704, 2208,
                18284, 55280, 30704, 47564, 13396, 33664, 9360,  18216, 54700,
                44004, 15836, 36624, 45872, 30928, 62308, 16952, 58636, 47100,
                28704, 18132,
            },
            {
                52416, 22660, 39896, 48324, 25576, 6844,  55020, 39192, 43400,
                14208, 63676, 36584, 59836, 39696, 47612, 2580,  20268, 7848,
                27660, 3944,  34460, 7476,  28984, 58668, 3612,  21996, 6040,
                28532, 23952, 304,   20816, 42632, 18112, 65520, 28024, 23112,
                52916, 15540, 33316, 60996, 53628, 1876,  34880, 25096, 55768,
                3548,  47508, 23496, 5668,  56276, 48828, 14124, 29544, 41340,
                49272, 21200, 58404, 30892, 61924, 54504, 23560, 48844, 39868,
                8740,  63248, 3244,  23420, 41580, 48868, 58852, 38996, 51076,
                6696,  29340, 58944, 488,   33168, 62628, 14068, 7708,  54664,
                46920, 26480, 792,   33512, 12620, 18556, 6100,  34744, 11152,
                20972, 31756, 49784, 64720, 28480, 18568, 11748, 7224,  28760,
                48664, 19804, 26208, 62572, 49448, 35856, 9776,  52968, 37504,
                48636, 4096,  63300, 39436, 58560, 51628, 42368, 63884, 35128,
                7672,  24352, 52844, 10764, 39928, 3572,  13132, 7612,  54548,
                39212, 35924,
            },
            {
                2600,  44464, 13788, 4920,  57044, 45124, 22476, 1940,  31012,
                20132, 28188, 42336, 8176,  21380, 29948, 63560, 35860, 58500,
                48884, 64876, 13796, 18984, 50696, 11296, 31588, 37488, 56332,
                48672, 41040, 35240, 6756,  58084, 45664, 5104,  14004, 37396,
                47080, 57744, 41516, 11280, 18152, 56896, 15116, 9472,  19040,
                37548, 13368, 52424, 16192, 39220, 2248,  23000, 45432, 36276,
                11060, 53704, 3660,  9424,  42064, 12976, 34692, 17356, 52564,
                25924, 44964, 34860, 55348, 28868, 4420,  14060, 25244, 2976,
                44660, 15544, 23760, 54380, 10876, 28228, 18200, 43244, 34076,
                61964, 19560, 58872, 45656, 55772, 41020, 48284, 23024, 62944,
                43104, 36676, 17240, 39908, 3956,  44792, 32900, 53332, 60576,
                2560,  12780, 34008, 6152,  23676, 14228, 59012, 29032, 60860,
                25764, 34708, 20340, 7144,  23028, 16644, 3740,  29776, 21336,
                49804, 1340,  65188, 18432, 22500, 56032, 44268, 31648, 20960,
                11356, 61876,
            },
            {
                32668, 27796, 63800, 35696, 17548, 32436, 11112, 62512, 58024,
                51440, 3340,  12384, 57436, 45272, 4768,  51676, 14812, 24412,
                45092, 30348, 40644, 54856, 24056, 43148, 15328, 60400, 18260,
                12912, 63692, 51164, 15768, 31636, 25036, 50320, 59356, 2496,
                30340, 6984,  25356, 36828, 5916,  44512, 64220, 48800, 30612,
                43392, 61320, 28776, 34632, 58380, 31304, 63788, 7592,  57068,
                24692, 32532, 47120, 37144, 27128, 6872,  60108, 1680,  31496,
                5472,  57664, 16088, 10436, 43588, 64208, 20380, 53328, 31584,
                57252, 41196, 34952, 49904, 21112, 40232, 56868, 24804, 3420,
                30804, 14968, 9024,  29380, 4176,  51984, 15296, 56736, 28652,
                4628,  13216, 55052, 9924,  47676, 62732, 25632, 38684, 16984,
                41460, 45464, 58292, 52504, 39852, 1036,  42296, 16680, 5528,
                13120, 44364, 10620, 50644, 28420, 36964, 60392, 12308, 38944,
                59284, 32280, 41932, 28756, 48784, 34516, 60660, 26676, 42236,
                49924, 15744,
            },
            {
                8456,  53772, 19596, 1496,  60064, 41656, 49076, 15936, 25152,
                46600, 34260, 53508, 16252, 32292, 25952, 11136, 43424, 18136,
                1336,  10012, 59392, 5556,  35684, 62072, 8364,  45752, 33944,
                2652,  25860, 9280,  38416, 61488, 11320, 41136, 34112, 19204,
                43752, 64044, 21924, 51504, 31836, 39032, 27264, 4424,  59096,
                22012, 548,   46284, 9300,  20316, 42664, 51300, 17316, 2840,
                61700, 13860, 18224, 64252, 51240, 21652, 46272, 40984, 65256,
                12472, 38644, 21436, 51644, 32620, 7800,  46844, 36556, 10648,
                63100, 18848, 8020,  61376, 2688,  47164, 64956, 6148,  50892,
                38812, 53340, 43652, 63580, 20696, 26624, 37348, 8688,  46796,
                59404, 25792, 51352, 31060, 21672, 14412, 524,   51140, 9620,
                31320, 21908, 7716,  29592, 47584, 64216, 33004, 21044, 51144,
                38848, 57176, 31628, 61640, 46468, 476,   52932, 44744, 5764,
                26220, 13740, 57440, 4896,  8504,  14900, 51360, 708,   5732,
                57444, 24124,
            },
            {
                46880, 58792, 42652, 50720, 26540, 7516,  29064, 37240, 4452,
                9324,  40100, 23100, 60736, 49348, 37940, 55328, 61016, 33304,
                52912, 37596, 21384, 47628, 27232, 960,   50036, 23252, 29216,
                55404, 44080, 19720, 53956, 47516, 1540,  21748, 53244, 8832,
                56444, 12648, 47932, 968,   61496, 20280, 12908, 52964, 40848,
                16696, 54916, 25728, 62700, 4412,  13088, 27288, 35216, 40136,
                49884, 28204, 44276, 492,   56956, 15368, 29248, 55100, 18980,
                49472, 28292, 60448, 876,   25572, 39792, 60980, 16312, 28080,
                1244,  46324, 25760, 13792, 37780, 30200, 16168, 35084, 12672,
                60560, 2304,  23752, 36368, 10968, 58468, 1504,  32368, 40096,
                19932, 2224,  62000, 7480,  58228, 42420, 35648, 56740, 24112,
                65252, 54164, 37736, 18756, 10940, 25208, 54252, 8116,  46612,
                63672, 1968,  18492, 41508, 14692, 25328, 33152, 19668, 55584,
                47568, 17584, 35724, 45608, 63212, 25052, 37892, 18952, 64672,
                34216, 40312,
            },
            {
                3316,  30376, 11804, 22056, 38964, 14176, 52856, 60572, 43828,
                18608, 63812, 27508, 6904,  144,   19536, 8620,  40888, 4440,
                26860, 62584, 15532, 31848, 56884, 17728, 37836, 64932, 5844,
                39756, 58900, 30924, 4520,  28296, 36692, 15356, 63152, 26528,
                38756, 16820, 28916, 54880, 9284,  45976, 23980, 35524, 7948,
                31992, 11948, 36728, 40332, 53768, 47720, 59056, 10488, 20752,
                55572, 5316,  34352, 23372, 39328, 11124, 36220, 2968,  24588,
                33956, 7016,  45520, 55912, 13268, 49296, 5160,  23108, 55168,
                39176, 51012, 32804, 55756, 43116, 9560,  53952, 22048, 45488,
                27748, 17468, 48684, 31252, 42112, 50328, 18268, 65420, 53756,
                14800, 43996, 34196, 38324, 27824, 49688, 19460, 6032,  46560,
                13292, 3912,  59936, 15420, 44020, 3536,  57600, 36316, 30284,
                23264, 27140, 54336, 9012,  56500, 6436,  64596, 41008, 9804,
                62560, 3412,  53216, 21056, 40216, 54920, 10608, 29660, 47840,
                12952, 17740,
            },
            {
                37700, 63184, 6440,  34272, 55876, 64560, 712,   20840, 31948,
                55424, 13532, 47208, 36412, 42860, 63088, 30684, 22992, 57504,
                12532, 50412, 45304, 3636,  11632, 41712, 54392, 10092, 21104,
                47116, 12140, 16756, 64168, 23780, 56724, 44228, 5800,  48796,
                33104, 3780,  42736, 35188, 15396, 59584, 50396, 2792,  56988,
                64856, 48516, 5812,  18420, 33164, 22808, 1612,  65132, 31472,
                43108, 9164,  60816, 48392, 7164,  62736, 43708, 52164, 9372,
                61904, 42016, 17008, 36004, 30640, 19388, 58612, 33896, 42996,
                12268, 6896,  60348, 22584, 4560,  26960, 58836, 49812, 8108,
                41300, 57132, 5452,  13708, 61036, 7340,  45936, 24280, 11480,
                29952, 48704, 22428, 16716, 2740,  11860, 61360, 29520, 34504,
                42840, 26148, 49868, 35320, 62320, 28036, 19448, 11896, 42968,
                15256, 4580,  34548, 38368, 21156, 49692, 29988, 15880, 23868,
                37728, 31360, 27812, 12476, 1500,  32592, 59248, 22540, 42828,
                55368, 26916,
            },
            {
                52404, 23580, 48876, 17120, 45240, 9748,  24428, 35900, 50576,
                5736,  29992, 58432, 21796, 11448, 50936, 15268, 47900, 36104,
                20108, 7644,  39148, 24152, 63084, 33404, 28396, 14872, 52880,
                36088, 696,   33372, 42148, 8212,  51520, 11920, 30220, 59664,
                22596, 52112, 62604, 25656, 4576,  31292, 41596, 27876, 19520,
                43724, 23340, 51696, 60276, 28300, 14516, 45160, 38192, 25252,
                15668, 53364, 19280, 30508, 25700, 16808, 32552, 59316, 14216,
                47732, 22736, 4460,  64460, 52128, 8880,  44760, 3448,  63688,
                20820, 30708, 16932, 48312, 63980, 18784, 37016, 1344,  31916,
                19792, 64248, 34004, 54340, 21940, 28092, 34672, 4960,  37772,
                60580, 9112,  56312, 64428, 45408, 52428, 40204, 15688, 59132,
                1380,  20312, 31608, 6636,  52028, 39984, 48168, 8,     59060,
                52744, 49236, 62356, 45320, 12864, 58988, 35828, 1568,  51948,
                43516, 7336,  49240, 62028, 44432, 16896, 49640, 6324,  2640,
                60460, 9652,
            },
            {
                43612, 13848, 57372, 4736,  31468, 42028, 48008, 59188, 11752,
                39340, 17280, 2324,  53016, 34024, 56640, 27676, 2076,  64336,
                42744, 29160, 59932, 51820, 18988, 48148, 5224,  43432, 60912,
                24612, 51108, 60068, 48356, 20728, 34704, 39500, 18428, 268,
                45812, 10544, 20024, 39644, 55704, 18148, 63028, 10236, 38552,
                14276, 29984, 3196,  9840,  42468, 57532, 7460,  48812, 4600,
                58848, 35748, 12032, 40360, 56244, 49592, 5648,  27248, 19684,
                38232, 57384, 11864, 27380, 41028, 24156, 14736, 53988, 26200,
                36776, 52308, 1868,  40080, 33996, 11020, 44284, 61536, 25464,
                38404, 10224, 46452, 836,   39916, 15952, 57492, 43020, 50608,
                28,    27208, 40760, 6208,  25304, 32440, 8684,  23128, 53316,
                48568, 38360, 56820, 9724,  22848, 14108, 32504, 64612, 37696,
                17976, 10384, 31052, 24484, 2612,  27672, 46800, 61384, 12176,
                55128, 17960, 58268, 23208, 36340, 7904,  53728, 38772, 31416,
                20632, 35476,
            },
            {
                64,    28368, 38584, 60244, 26172, 19248, 14948, 3880,  26868,
                45776, 65364, 41608, 25604, 7464,  18592, 45032, 5964,  53836,
                32580, 16512, 1216,  35112, 8820,  58752, 22548, 2176,  31536,
                18104, 9000,  27228, 14184, 5344,  57940, 62232, 25276, 54372,
                36360, 14432, 57716, 6676,  49136, 12720, 45324, 844,   54352,
                59288, 34544, 46716, 63528, 17388, 36024, 54428, 19644, 29352,
                51316, 1332,  46744, 64536, 3444,  21324, 54144, 41424, 636,
                29900, 50852, 33344, 2484,  61040, 37500, 47816, 29928, 10184,
                59712, 45820, 14064, 57628, 28672, 52896, 15432, 6016,  55900,
                51708, 16432, 24460, 29780, 52084, 63092, 12288, 20956, 55216,
                17588, 35768, 13416, 54168, 20088, 36968, 63072, 4356,  27604,
                11452, 63568, 16888, 45572, 60628, 4844,  55376, 26268, 7552,
                22732, 41092, 5808,  53796, 42668, 18708, 8724,  39316, 21400,
                26736, 34500, 2760,  40964, 13916, 27440, 64432, 24528, 45284,
                15680, 62100,
            },
            {
                32972, 46588, 21088, 50304, 1904,  53764, 63052, 33780, 52004,
                21316, 9256,  32112, 48584, 62436, 38160, 13580, 40260, 24600,
                11040, 56436, 44252, 26576, 12944, 55088, 37232, 45704, 63712,
                41032, 56144, 37896, 44944, 29348, 2480,  15564, 41892, 8496,
                64704, 32268, 44296, 29316, 37004, 21632, 33556, 26760, 48144,
                7032,  21020, 25960, 39688, 472,   31812, 24176, 11044, 63344,
                41720, 21912, 27680, 14764, 37248, 10204, 45860, 34156, 64068,
                8680,  44716, 55308, 20828, 7108,  57604, 17708, 68,    40744,
                19232, 7272,  25044, 43408, 4244,  23344, 47444, 30080, 13024,
                42312, 2984,  60716, 36788, 8380,  47404, 3408,  32772, 25032,
                63592, 46832, 31000, 60020, 3204,  48172, 44056, 55524, 19168,
                41896, 6096,  30224, 24916, 41160, 35032, 20240, 44604, 29804,
                47596, 60424, 33756, 64384, 14744, 56900, 51232, 32140, 5348,
                65492, 45920, 10040, 30604, 60508, 47432, 19104, 11016, 58536,
                50060, 7756,
            },
            {
                17640, 65112, 11648, 8472,  36080, 40400, 29012, 6952,  57212,
                36852, 60444, 15064, 3452,  22936, 57860, 30060, 60176, 47056,
                19276, 7092,  49556, 65240, 40208, 29600, 15820, 7240,  25972,
                12556, 3176,  21976, 65148, 49780, 32576, 53072, 46724, 28352,
                4360,  22960, 50844, 1604,  63860, 8576,  52088, 61192, 16792,
                37608, 11256, 56172, 52736, 13312, 49680, 60352, 39024, 16460,
                33432, 8492,  60688, 43796, 31268, 58548, 24188, 13244, 59812,
                17668, 24972, 14924, 42688, 49520, 12356, 32672, 65312, 50524,
                55700, 34896, 63416, 9824,  37952, 59660, 20204, 65168, 35432,
                49548, 21184, 56632, 44336, 18852, 26644, 59832, 41284, 10068,
                6692,  43340, 22040, 50756, 10896, 17012, 29732, 13720, 33440,
                47460, 36304, 50972, 2424,  53876, 11600, 49368, 3404,  15640,
                56360, 12572, 20412, 28460, 4252,  35168, 23968, 60268, 41832,
                14092, 49952, 56536, 20400, 52816, 1708,  34064, 5184,  41328,
                26032, 54980,
            },
            {
                39608, 24104, 56504, 32180, 61020, 12852, 22788, 44236, 18172,
                1068,  42792, 28840, 54756, 35332, 10452, 4488,  51680, 34400,
                62184, 37592, 23524, 2944,  20288, 47264, 51624, 33220, 57552,
                48096, 35360, 53980, 7764,  17796, 23616, 36656, 11660, 19880,
                60380, 39992, 16992, 26328, 58720, 41288, 24164, 5436,  31452,
                42716, 62468, 4200,  19036, 44132, 27804, 5836,  46180, 2724,
                55788, 48472, 24652, 5780,  18060, 51552, 2592,  39392, 48384,
                5840,  37352, 62632, 29184, 35220, 23504, 44972, 27168, 4864,
                22520, 30992, 48924, 16828, 54944, 33128, 432,   39740, 8764,
                26832, 31856, 11720, 4316,  34180, 14596, 49668, 37560, 30012,
                52228, 15692, 1564,  34668, 26904, 57820, 39476, 61804, 744,
                59036, 21628, 15024, 65096, 27864, 17784, 58564, 63016, 40052,
                35848, 1148,  50900, 45672, 38424, 48836, 11396, 328,   29220,
                17216, 37584, 25232, 6804,  42992, 38464, 29208, 56308, 14136,
                36320, 3532,
            },
            {
                52748, 44848, 5572,  42284, 16684, 47160, 55332, 10032, 49580,
                25360, 12112, 46860, 19480, 50856, 44352, 16856, 21984, 404,
                28696, 14888, 54032, 32108, 59216, 4692,  10324, 62744, 21160,
                17176, 30524, 43520, 11344, 40152, 61000, 6704,  57028, 48732,
                34136, 52872, 9460,  13336, 46520, 18860, 35308, 14688, 57360,
                49208, 22728, 30468, 35736, 64936, 9240,  56864, 20720, 29692,
                13508, 52460, 39816, 56932, 34812, 63048, 29048, 20732, 55980,
                32344, 53028, 1224,  9916,  54636, 4020,  61780, 39492, 10788,
                14496, 42208, 2264,  27968, 50896, 12292, 45392, 17400, 57256,
                5444,  61440, 53440, 39348, 64660, 22620, 54420, 2172,  19332,
                61920, 56100, 38636, 65204, 42488, 4716,  23704, 8816,  52132,
                25988, 10704, 56168, 43440, 8324,  37972, 31440, 23688, 9836,
                53580, 26452, 58812, 8196,  16528, 63480, 20644, 54676, 44152,
                59200, 3688,  31984, 63080, 12684, 16976, 51008, 63776, 21232,
                47600, 30280,
            },
            {
                12920, 19616, 28636, 51812, 25472, 2320,  64276, 30848, 39132,
                58728, 33416, 62832, 7744,  40488, 26472, 64812, 39060, 9304,
                58204, 41616, 11388, 43768, 17536, 27840, 38908, 42684, 140,
                52624, 5412,  61848, 26984, 55628, 1164,  44456, 26492, 14632,
                2156,  43284, 62480, 31896, 55620, 3156,  53308, 44516, 1328,
                27512, 8368,  51528, 40720, 15480, 23304, 34312, 42108, 62284,
                36756, 19260, 284,   12608, 45472, 8332,  15944, 43220, 11156,
                27012, 46344, 18720, 40896, 58496, 20220, 15748, 52100, 35728,
                56840, 46320, 61688, 21108, 7892,  25184, 62692, 29192, 48160,
                42836, 23992, 16580, 46264, 28048, 6868,  58588, 12924, 44168,
                27504, 23180, 11784, 7608,  53572, 49408, 19728, 45980, 37472,
                31180, 3816,  39936, 33268, 47800, 972,   52324, 43888, 6104,
                32584, 18416, 43272, 24220, 31724, 40460, 6848,  26536, 33624,
                51420, 8900,  22332, 47824, 54316, 26160, 9380,  44468, 2292,
                61216, 8804,
            },
            {
                35796, 63624, 3820,  59312, 38308, 20968, 34780, 5980,  14732,
                52440, 4544,  23692, 56292, 1764,  14000, 32796, 55624, 45476,
                25092, 49168, 6020,  35184, 61144, 55592, 24132, 13752, 36288,
                59556, 24624, 15344, 34348, 50620, 30988, 21516, 38900, 59192,
                30072, 19228, 37304, 6092,  28148, 38764, 11616, 64680, 20516,
                39296, 12876, 60776, 2284,  47376, 53680, 3924,  50024, 25644,
                9644,  31936, 65396, 27148, 22912, 49872, 36780, 57980, 4868,
                22020, 64944, 13596, 50424, 25548, 47456, 8112,  29584, 59772,
                18528, 6056,  38648, 33876, 58164, 41324, 2964,  52448, 20484,
                13760, 37868, 1592,  51248, 10300, 42120, 32400, 47308, 35536,
                4468,  48668, 32872, 17764, 30212, 13148, 34304, 63972, 15884,
                43960, 61288, 18884, 22972, 59876, 26756, 12624, 21040, 47084,
                60840, 37704, 11548, 62004, 3168,  52496, 58252, 47192, 12432,
                19120, 61856, 41108, 34600, 1072,  59400, 36972, 23900, 32724,
                16396, 57304,
            },
            {
                22792, 48268, 31380, 15108, 8152,  50140, 41748, 60528, 19960,
                28124, 42256, 17264, 37224, 30172, 61068, 48028, 5076,  18072,
                31036, 64036, 20952, 52336, 1636,  45380, 7512,  50532, 29080,
                46488, 39352, 8784,  47576, 19088, 12892, 62976, 9056,  54188,
                5292,  24344, 47924, 57784, 21348, 50392, 25056, 32512, 59068,
                46472, 54816, 18328, 26168, 31900, 11964, 58144, 17476, 6732,
                59156, 47348, 42472, 5256,  54632, 30748, 1616,  61684, 51400,
                34864, 39580, 29720, 3388,  33464, 37108, 42908, 2680,  23400,
                49228, 26564, 12868, 53776, 44832, 15376, 36704, 32456, 9484,
                63844, 34396, 59340, 30784, 20564, 61732, 17600, 24736, 8604,
                57228, 40256, 61240, 44876, 24872, 60056, 2140,  55568, 27848,
                6764,  53640, 13204, 50488, 4524,  16388, 56636, 63880, 28960,
                15048, 4604,  55300, 47960, 29448, 14348, 36240, 23064, 1956,
                37840, 55224, 15080, 28152, 45760, 19320, 6140,  41944, 50496,
                27484, 40612,
            },
            {
                512,   43572, 10832, 55952, 45352, 26776, 11584, 54040, 260,
                47672, 64580, 9156,  45816, 51968, 19852, 11184, 36688, 53120,
                2296,  13280, 39888, 26292, 16132, 31484, 21776, 63836, 10828,
                18516, 56664, 2500,  64364, 41604, 4888,  36116, 46316, 16588,
                50784, 65360, 10328, 14212, 44176, 61636, 8044,  4380,  16044,
                29460, 35056, 5712,  43676, 63916, 36948, 28244, 45128, 39240,
                14472, 53064, 20904, 38072, 60264, 13660, 41308, 25368, 16920,
                9180,  44572, 59276, 7244,  61100, 11936, 63400, 55320, 32688,
                9948,  64268, 756,   30100, 19164, 5040,  60144, 23376, 56176,
                4616,  49376, 25716, 6352,  55896, 36552, 680,   53096, 64172,
                15404, 20616, 1172,  54364, 36504, 6332,  41148, 21896, 11340,
                48576, 25020, 35172, 63456, 30344, 37228, 41688, 33904, 2020,
                49892, 40584, 22648, 34496, 19656, 45076, 9808,  60340, 43004,
                30020, 48504, 4848,  11176, 64260, 31128, 55984, 14504, 60500,
                5152,  53736,
            },
            {
                17220, 61884, 37308, 23608, 33652, 62708, 17692, 30256, 37808,
                22740, 34648, 12560, 59972, 24920, 6468,  28156, 41348, 22396,
                59432, 34204, 7840,  62372, 48024, 38052, 57336, 34736, 4496,
                43080, 32452, 22268, 28732, 52824, 58248, 23136, 32052, 42332,
                27436, 35340, 40516, 1192,  33528, 17544, 41260, 36796, 51932,
                10188, 21864, 58444, 50752, 8264,  20496, 1136,  62904, 23076,
                34100, 3224,  29120, 16728, 9268,  46236, 19636, 32952, 47952,
                2812,  55552, 23844, 18140, 53296, 21668, 27392, 16628, 45500,
                41232, 35912, 51204, 23060, 62044, 47304, 11572, 28000, 40380,
                44044, 18992, 12108, 45568, 40696, 14244, 49032, 28544, 38244,
                31424, 51816, 10600, 28116, 14724, 47200, 51136, 31432, 37980,
                59204, 336,   41396, 9524,  46080, 7076,  54772, 10308, 19428,
                59600, 27276, 8828,  64160, 684,   55964, 27696, 15804, 65064,
                7424,  24952, 58000, 21804, 39772, 51372, 9892,  46584, 20784,
                11872, 34172,
            },
            {
                26236, 7300,  19432, 52408, 1704,  6184,  56904, 43864, 7828,
                50376, 55840, 3216,  31524, 42784, 57188, 49612, 63244, 15176,
                44144, 55804, 29304, 42576, 10428, 5580,  14572, 53780, 26556,
                61972, 51856, 11496, 36836, 14796, 26364, 424,   61276, 11636,
                4004,  22144, 60148, 30420, 55200, 48380, 27172, 56648, 63136,
                42700, 136,   38412, 24592, 16264, 40396, 55008, 30760, 51612,
                10824, 57416, 63368, 49616, 35208, 56184, 6196,  64320, 52852,
                27988, 14560, 37768, 32088, 46572, 40040, 1152,  50944, 7012,
                20596, 14860, 57388, 42788, 8808,  39004, 54704, 49948, 852,
                15548, 29816, 53668, 65100, 22272, 5192,  58368, 11240, 45220,
                22684, 6852,  42596, 57708, 64696, 19052, 9740,  56448, 16332,
                45264, 20336, 28680, 57908, 17888, 22400, 51408, 25416, 44124,
                36452, 54004, 17060, 50808, 42132, 5744,  38832, 49084, 32600,
                18460, 51784, 35048, 44092, 16640, 4172,  35508, 28940, 38196,
                65400, 49188,
            },
            {
                56940, 39924, 60140, 28416, 48508, 40388, 21704, 13888, 59656,
                26196, 16544, 40068, 21572, 14840, 720,   34996, 9392,  25544,
                3588,  50732, 17772, 23440, 58524, 49980, 20320, 40876, 1196,
                16712, 48852, 6620,  45196, 55968, 8420,  47228, 51472, 20048,
                55764, 45512, 15932, 52320, 5324,  20920, 12760, 3432,  23832,
                49344, 14100, 61368, 33276, 56996, 44384, 13544, 5304,  46696,
                18912, 25568, 43764, 812,   22544, 27332, 39320, 11776, 21424,
                42808, 63076, 10172, 50100, 4676,  14260, 30808, 58196, 37400,
                60960, 28280, 5852,  31816, 17164, 26228, 34804, 20924, 63236,
                58880, 37468, 8236,  35124, 27044, 32776, 43196, 18576, 3752,
                62644, 49600, 34012, 23772, 39080, 5140,  27116, 61448, 3364,
                33196, 62532, 14104, 49452, 38980, 1660,  65288, 31840, 14360,
                6404,  46424, 12324, 30068, 61024, 24656, 20844, 53056, 2448,
                40592, 13152, 62876, 192,   27088, 61508, 22880, 57656, 2604,
                44560, 31008,
            },
            {
                14556, 3000,  46528, 12028, 15940, 64716, 32332, 36336, 46104,
                5176,  63476, 33792, 53256, 45068, 61432, 18940, 39324, 31740,
                46664, 11628, 37024, 440,   33420, 28104, 65404, 46332, 30240,
                23336, 38776, 60752, 19724, 31168, 40476, 16144, 38068, 33616,
                63748, 7772,  38796, 24728, 9912,  64496, 46436, 34740, 18520,
                31852, 27772, 46260, 10796, 4084,  49796, 26900, 35816, 61152,
                41428, 7292,  37484, 12576, 61084, 52120, 3896,  58648, 30452,
                35788, 152,   58032, 26192, 56044, 44324, 64760, 10356, 25148,
                46640, 2228,  53104, 48764, 65484, 3684,  14168, 7440,  32048,
                24424, 46644, 3568,  49608, 16500, 62096, 54912, 25060, 35468,
                59160, 12304, 17028, 2780,  30720, 44696, 35752, 22804, 42076,
                8616,  51732, 4612,  23984, 34388, 12196, 47292, 57508, 2892,
                61472, 23392, 38460, 33312, 8100,  58288, 35620, 11416, 56760,
                26548, 46544, 8732,  55576, 32192, 47492, 52520, 7676,  18440,
                24084, 9436,
            },
            {
                42512, 21428, 31820, 36064, 54512, 24468, 3284,  51884, 10980,
                19936, 29512, 47540, 7940,  11400, 28656, 54472, 4712,  58180,
                64420, 20892, 53024, 61252, 44356, 4408,  12096, 36260, 7900,
                58092, 12992, 34048, 2164,  64880, 23872, 58572, 28144, 2564,
                13988, 29532, 49320, 57448, 42936, 28608, 38120, 60788, 9092,
                55232, 6560,  52756, 19964, 29968, 62688, 22636, 9336,  16392,
                56052, 32364, 48624, 29832, 15684, 33924, 44856, 18012, 7572,
                47384, 16236, 41356, 19676, 8064,  35280, 22796, 18600, 54284,
                33488, 13228, 40432, 20120, 36560, 56104, 45880, 52188, 42508,
                11456, 55816, 20192, 60524, 10504, 1692,  39656, 8548,  51084,
                29528, 40800, 47052, 54852, 62860, 51572, 11836, 48360, 54572,
                26180, 37264, 30752, 55416, 60356, 42404, 26704, 20020, 40328,
                28740, 55928, 3644,  19080, 47792, 14644, 45296, 30332, 4732,
                60168, 22820, 38020, 19004, 41780, 11564, 15088, 39056, 62784,
                50156, 54792,
            },
            {
                27144, 63960, 50816, 4364,  43300, 9060,  60916, 27904, 42400,
                55144, 1200,  58344, 24608, 37668, 50864, 22508, 42416, 16276,
                27752, 6372,  40816, 15332, 24948, 55548, 17884, 60092, 52432,
                42228, 26156, 55000, 5408,  43564, 11116, 49428, 6188,  53788,
                44000, 36444, 18768, 732,   32712, 15276, 51804, 1820,  44348,
                39988, 16148, 65308, 42188, 37760, 1572,  58428, 39952, 50432,
                2168,  21188, 65048, 54208, 8000,  41080, 24436, 50284, 61672,
                53872, 23172, 33588, 62200, 29076, 49388, 39256, 4036,  42988,
                62792, 8296,  24256, 60252, 10548, 28780, 39788, 61552, 2056,
                17492, 28412, 39276, 43848, 31460, 52012, 21032, 45984, 14944,
                132,   21408, 26684, 8292,  14464, 20692, 1060,  59320, 17456,
                13012, 64620, 19340, 44788, 5632,  16836, 7976,  52556, 34028,
                16008, 49212, 64448, 41456, 54460, 2816,  62528, 17860, 43260,
                33940, 15184, 29036, 50820, 6072,  60000, 26048, 43732, 33148,
                864,   36596,
            },
            {
                57316, 5832,  17648, 58996, 29824, 20580, 48196, 16412, 38228,
                62524, 13360, 41184, 17408, 64928, 2396,  32700, 12276, 49044,
                36012, 56588, 29996, 9648,  48748, 37884, 21840, 31720, 3540,
                15660, 47876, 21220, 29452, 53108, 35820, 18488, 62824, 22632,
                26008, 59748, 11500, 62684, 22372, 6892,  58940, 25876, 20296,
                57800, 34228, 4740,  25412, 13192, 47412, 18632, 33692, 28676,
                14012, 25000, 4812,  45956, 19904, 56872, 1992,  13420, 27100,
                38064, 11380, 5116,  51872, 13288, 1440,  57016, 15724, 27212,
                50492, 17324, 30324, 44192, 88,    15928, 21800, 25592, 34092,
                58048, 6596,  53368, 23176, 13628, 36132, 28284, 64684, 56320,
                33188, 60940, 37148, 57844, 41668, 34576, 29172, 39332, 6812,
                46964, 40996, 1432,  11024, 50512, 29252, 37708, 63148, 10500,
                43912, 6724,  26404, 11544, 22108, 28380, 37056, 9444,  49312,
                63900, 53636, 2008,  58172, 21520, 35948, 3828,  53960, 29268,
                16488, 11732,
            },
            {
                46032, 33552, 25384, 11328, 38592, 56388, 2364,  35072, 7472,
                23200, 31616, 50236, 34836, 6816,  57224, 46256, 62064, 8512,
                23300, 2848,  45696, 32764, 63616, 7280,  43312, 50764, 10416,
                35244, 61164, 39484, 8524,  46516, 14356, 32248, 41820, 9468,
                47704, 4024,  51296, 40956, 46004, 36192, 10728, 49152, 30584,
                12156, 23056, 50836, 60180, 31968, 55672, 7028,  53664, 62972,
                42072, 59816, 38400, 11196, 28028, 36520, 63636, 31804, 43656,
                3472,  58816, 46396, 25248, 42248, 60096, 32080, 47712, 6728,
                36084, 58968, 53840, 34244, 48448, 57612, 8180,  50444, 12824,
                47020, 36908, 62416, 2988,  47884, 59860, 4480,  18124, 42128,
                11276, 6240,  49164, 17684, 4232,  52808, 24824, 61736, 32888,
                22212, 28224, 58644, 33748, 62396, 23236, 48216, 4,     56960,
                21532, 36696, 52944, 32204, 60408, 39804, 52036, 25128, 21148,
                5636,  12172, 40944, 48064, 31364, 64960, 46296, 9712,  20144,
                61416, 39540,
            },
            {
                52728, 8204,  41664, 62208, 45364, 14240, 52524, 26176, 43976,
                53916, 4332,  20672, 27120, 43632, 14928, 19732, 26384, 38512,
                60032, 51036, 13676, 19780, 53412, 2072,  27724, 62600, 24740,
                56608, 320,   17672, 63524, 26744, 59676, 1792,  38536, 54764,
                16296, 34552, 30136, 55920, 25176, 17612, 54048, 41612, 62992,
                3332,  37324, 45064, 9616,  16720, 43544, 26084, 11728, 3100,
                45308, 34756, 51516, 17156, 58936, 48236, 22156, 9388,  55420,
                19220, 30156, 65324, 35528, 17184, 9332,  21072, 64184, 11848,
                41256, 2644,  21488, 5364,  63772, 41704, 31568, 38288, 64764,
                19592, 30432, 9792,  16288, 26516, 54576, 7924,  38528, 25200,
                53164, 44820, 23012, 31844, 65444, 45732, 10932, 49300, 2836,
                56512, 52452, 15580, 20856, 39560, 54508, 13604, 32640, 25436,
                60928, 14308, 2068,  45452, 7700,  16224, 796,   56540, 41844,
                31880, 36620, 25800, 7944,  17084, 13956, 24372, 56384, 48484,
                5272,  23260,
            },
            {
                14524, 19652, 49908, 224,   22532, 31664, 65164, 18888, 59164,
                11092, 46360, 60600, 54988, 10564, 36940, 30776, 52592, 5248,
                17396, 41484, 26888, 59148, 35636, 40288, 13344, 46576, 19284,
                29960, 44568, 12604, 51364, 22360, 4860,  56748, 20188, 28308,
                61900, 6624,  20936, 13636, 2544,  64360, 33080, 6124,  15592,
                47232, 28356, 57300, 996,   21772, 64140, 36944, 48988, 19464,
                22836, 8308,  30312, 1632,  43032, 5384,  15172, 34256, 49252,
                39508, 14648, 7116,  22248, 54332, 45644, 37724, 29200, 55556,
                23840, 46824, 14676, 26920, 11656, 18836, 54984, 3228,  6408,
                43288, 24852, 51388, 40784, 33656, 43644, 12548, 50092, 30184,
                2616,  62012, 13364, 27528, 8904,  55104, 19672, 15112, 42544,
                8344,  36456, 43920, 3264,  7288,  45640, 18028, 5252,  41512,
                46700, 30336, 18652, 58528, 50356, 34328, 65412, 13484, 46980,
                59136, 18376, 62188, 54400, 44908, 39640, 2728,  34964, 41584,
                31068, 58464,
            },
            {
                26768, 37624, 55124, 28132, 36040, 9752,  5908,  49220, 33988,
                29276, 15912, 38880, 2956,  48936, 63468, 1324,  56000, 44544,
                34160, 65124, 612,   10616, 23764, 16804, 55792, 4148,  38292,
                49444, 6604,  32788, 36916, 42600, 30672, 49660, 45636, 12344,
                37328, 44580, 59372, 39768, 8744,  47868, 28828, 21724, 38896,
                60924, 18672, 52876, 34924, 40860, 29872, 4300,  52532, 32572,
                57832, 13276, 55284, 64408, 24500, 40076, 52384, 61556, 832,
                25892, 57484, 50792, 40536, 2092,  26552, 5448,  51476, 18204,
                32684, 61656, 39588, 52456, 36480, 45796, 23348, 28644, 59532,
                14088, 55740, 1860,  63204, 19108, 56992, 22464, 63876, 35272,
                20016, 56808, 39564, 43536, 828,   35392, 38596, 30676, 63920,
                24544, 12460, 60844, 26836, 58152, 29964, 36180, 65036, 51280,
                8876,  55120, 38092, 23244, 41640, 27188, 20104, 29976, 3800,
                8988,  50404, 28584, 624,   22300, 60784, 51556, 27096, 13080,
                63008, 1688,
            },
            {
                44312, 64028, 4656,  16428, 57636, 46932, 39516, 13520, 1028,
                41416, 64024, 8304,  33108, 24792, 21456, 40972, 13256, 9080,
                21960, 30480, 48300, 57000, 42592, 29364, 60432, 32460, 9516,
                64908, 23816, 58340, 54464, 10592, 15988, 7736,  65388, 24160,
                640,   50108, 31976, 26500, 52648, 60296, 43700, 480,   54872,
                10312, 26504, 7276,  13392, 60436, 46352, 8856,  15808, 62376,
                41140, 26532, 46684, 18936, 32780, 7880,  28664, 45504, 21140,
                10372, 44628, 33176, 12164, 61336, 56888, 15148, 43456, 584,
                9936,  57544, 6912,  29924, 62520, 9148,  16104, 52136, 47864,
                35076, 21684, 31876, 46404, 6244,  29372, 1080,  15756, 46276,
                9432,  32996, 16204, 51928, 60536, 22340, 58656, 4392,  51488,
                18272, 48000, 31744, 50984, 9832,  23916, 56128, 12056, 20480,
                28108, 3624,  63648, 12496, 4516,  9816,  44620, 54688, 35400,
                23724, 43344, 13248, 37640, 32860, 10572, 57756, 7404,  18048,
                50780, 34144,
            },
            {
                47832, 11944, 31964, 42268, 60544, 20556, 51824, 25768, 56468,
                22700, 19352, 52928, 28548, 57912, 16608, 46808, 61840, 25736,
                51548, 15284, 36844, 5788,  46064, 7668,  50928, 21552, 53828,
                14132, 40780, 18872, 1544,  26068, 60520, 40296, 32932, 53532,
                17800, 10456, 56704, 15160, 36496, 19020, 12420, 24620, 49224,
                35744, 42552, 32116, 49864, 19864, 24380, 56208, 28796, 37620,
                172,   5740,  35764, 49508, 11680, 60256, 55960, 17288, 36816,
                63164, 4436,  28172, 18332, 47400, 30716, 34820, 62920, 38544,
                25928, 44396, 20432, 49340, 3784,  58888, 33252, 1420,  39940,
                61532, 7720,  38920, 11908, 59236, 36524, 52780, 41956, 59580,
                3692,  48280, 24944, 6060,  29844, 12612, 41440, 26356, 56220,
                34752, 692,   39900, 63364, 16340, 42648, 48860, 1348,  40000,
                34884, 16112, 48412, 31072, 56832, 61776, 49596, 39236, 15800,
                60464, 55744, 4968,  64492, 49112, 19528, 29904, 38560, 43420,
                22080, 9308,
            },
            {
                24880, 19016, 52788, 7824,  24520, 3904,  30416, 61436, 43168,
                35288, 50300, 4680,  43804, 11304, 35732, 6920,  29044, 39108,
                58896, 4008,  54436, 20624, 63856, 12564, 34616, 26280, 43320,
                3076,  28660, 61580, 48192, 44376, 35928, 19784, 4624,  27388,
                42800, 62276, 22780, 3276,  41980, 6928,  33796, 57356, 61988,
                14696, 5480,  64700, 55236, 2000,  39368, 12272, 48232, 21852,
                58860, 43776, 15612, 53784, 22208, 38388, 4104,  12888, 31136,
                53424, 58616, 38580, 23360, 52732, 6836,  11096, 22764, 48116,
                54112, 13584, 34480, 17404, 24764, 41292, 44844, 26040, 11028,
                18056, 28088, 51032, 15512, 25432, 49100, 10180, 23828, 27916,
                37832, 53492, 62984, 19336, 45328, 50152, 17248, 7560,  11244,
                44756, 22644, 6248,  20080, 37140, 4428,  32480, 58576, 26140,
                60472, 53520, 43216, 24940, 36432, 14664, 6772,  25908, 1620,
                20552, 30884, 40884, 25584, 15324, 44548, 4596,  53484, 59944,
                3296,  56152,
            },
            {
                40648, 61012, 35664, 14980, 45172, 37936, 11676, 17200, 2920,
                9932,  59244, 14552, 38176, 60932, 53732, 49440, 3152,  18840,
                45544, 11604, 32656, 27428, 38472, 17736, 1396,  61992, 47436,
                37376, 16636, 34184, 5656,  22092, 51748, 13160, 56164, 48408,
                34724, 5996,  46904, 30744, 64784, 50768, 45332, 29572, 3668,
                20100, 46752, 25496, 16584, 44340, 33704, 63824, 6948,  17816,
                51944, 10440, 29916, 63140, 2244,  26400, 41840, 50828, 24012,
                7048,  42760, 15840, 2512,  64100, 41204, 19540, 58212, 4608,
                29296, 60272, 1804,  38048, 55440, 12816, 20884, 64212, 54288,
                5228,  57584, 43556, 64848, 2852,  40424, 17964, 61812, 6980,
                14116, 31160, 10388, 36172, 1796,  64548, 32568, 47060, 62144,
                28596, 52528, 57496, 12988, 46864, 54080, 22240, 14436, 45104,
                9636,  6512,  21208, 460,   46552, 19296, 33300, 52024, 63596,
                47660, 10008, 52784, 7176,  56296, 35376, 63304, 24032, 13920,
                36576, 30304,
            },
            {
                6712,  2800,  49500, 29104, 55848, 64532, 33744, 53500, 45764,
                26420, 30916, 47652, 23288, 100,   20028, 31912, 56652, 35096,
                63332, 42292, 23732, 59552, 49972, 41280, 55696, 20000, 7256,
                58064, 52272, 12228, 55660, 9132,  30168, 63944, 2524,  15448,
                59268, 38564, 19360, 54136, 25064, 10880, 16600, 22356, 40416,
                53428, 37812, 30656, 9856,  59628, 4508,  26120, 42088, 31520,
                61180, 24324, 39964, 7296,  33408, 59228, 47448, 65068, 35052,
                19504, 48992, 9844,  33800, 45356, 25776, 49936, 36612, 16556,
                8700,  41920, 51052, 63360, 31192, 4484,  50340, 36736, 30408,
                47368, 23744, 35876, 20520, 55160, 30520, 34324, 56836, 44928,
                21608, 55916, 42640, 58096, 27364, 38976, 54264, 3556,  36792,
                14712, 42380, 31428, 26396, 60044, 8372,  29088, 64328, 18976,
                51972, 32824, 62484, 39636, 54884, 60080, 29000, 11952, 43760,
                37180, 17244, 34088, 21932, 46852, 2624,  28364, 9720,  48628,
                16980, 65248,
            },
            {
                46428, 21500, 58600, 10668, 604,   19564, 48716, 6272,  62664,
                40604, 18068, 7468,  65276, 27560, 41732, 13992, 24228, 8284,
                15948, 1004,  52804, 9040,  4288,  14728, 45376, 30192, 10580,
                24496, 31312, 42412, 62672, 26792, 46060, 37184, 42004, 24388,
                29488, 8568,  12960, 40688, 2280,  56452, 36768, 63544, 8844,
                58864, 1516,  50832, 22564, 36892, 53092, 14592, 50104, 35080,
                2528,  46132, 55736, 17132, 44888, 20148, 8516,  15312, 392,
                27180, 61860, 55512, 29760, 59780, 13852, 1044,  31684, 64832,
                45844, 20736, 26996, 10848, 46688, 58136, 8268,  15616, 41684,
                244,   13252, 32520, 9428,  4708,  48108, 12764, 628,   50664,
                39344, 4196,  16700, 23512, 7748,  13400, 20256, 24668, 56528,
                18564, 9096,  64016, 1520,  40244, 35576, 50176, 3812,  37356,
                41764, 13220, 28504, 17072, 10272, 3596,  41088, 22440, 57476,
                4368,  27556, 62548, 58472, 12144, 39288, 18800, 41796, 32596,
                52868, 26092,
            },
            {
                38268, 16060, 33016, 43336, 23684, 39724, 27540, 14032, 22068,
                36400, 51160, 56772, 33984, 45056, 10132, 62648, 51100, 39488,
                30724, 48440, 19840, 36536, 31704, 61116, 22756, 35892, 64616,
                49800, 3856,  19420, 38384, 72,    17364, 58392, 10652, 20796,
                53284, 62768, 49288, 33508, 60964, 28512, 5688,  47976, 32736,
                27104, 11984, 42156, 56672, 19396, 29040, 61824, 9244,  57712,
                20448, 12896, 27468, 36160, 51632, 11336, 29552, 57276, 40724,
                46052, 12424, 36540, 20792, 8040,  53864, 39836, 56352, 23632,
                52232, 33868, 5796,  16016, 22680, 35180, 28668, 60684, 19688,
                51684, 62420, 45140, 59468, 26284, 42160, 62788, 28996, 18704,
                65208, 33736, 49352, 46172, 61108, 51308, 31020, 43756, 60568,
                5532,  35036, 49560, 23728, 45192, 17936, 11760, 55992, 24868,
                1888,  49292, 57060, 36016, 24336, 48184, 53036, 7728,  31632,
                14428, 50384, 104,   42864, 30756, 51496, 61952, 56700, 1140,
                44064, 12280,
            },
            {
                54892, 27980, 51756, 62472, 7584,  54212, 58264, 32508, 1824,
                59884, 11696, 3908,  15728, 54744, 37088, 5120,  59456, 46140,
                27788, 55072, 65040, 44112, 12020, 26640, 53748, 6052,  40220,
                13784, 46372, 59848, 22948, 51040, 33676, 7068,  49852, 31956,
                43388, 920,   26568, 15220, 20284, 43872, 52256, 13712, 18584,
                45100, 61272, 15924, 5896,  47476, 2804,  45316, 23388, 38380,
                6452,  49284, 64628, 1240,  58108, 23636, 37872, 54228, 32516,
                4344,  52048, 24676, 2860,  43044, 18160, 28032, 10320, 3696,
                13380, 40752, 61668, 53904, 38600, 44184, 2356,  25280, 54940,
                39040, 7056,  17380, 37656, 53616, 16172, 22664, 37132, 8096,
                26060, 11220, 29768, 2412,  35424, 41132, 10200, 396,   48592,
                39456, 21224, 54320, 15416, 5016,  62772, 33388, 43132, 59824,
                30840, 22036, 63760, 5004,  44768, 61600, 34288, 18412, 64148,
                46380, 39008, 20076, 24672, 7532,  15588, 26332, 5588,  22608,
                59588, 8672,
            },
            {
                61444, 2108,  40852, 13036, 36236, 17340, 43924, 9592,  47552,
                25080, 42960, 29752, 21440, 49332, 25732, 18320, 2192,  21164,
                12444, 6516,  24788, 17452, 57364, 47640, 564,   18108, 56892,
                28068, 32988, 8788,  14704, 41564, 55288, 25540, 60692, 16208,
                4836,  35880, 45840, 55836, 7492,  38680, 24092, 188,   54628,
                36020, 23320, 31188, 65060, 34492, 12736, 41076, 17508, 30628,
                53404, 42316, 15464, 30960, 5044,  43608, 62392, 6120,  21816,
                17344, 59516, 38988, 63696, 47692, 32836, 62288, 44980, 37392,
                59624, 29784, 1220,  47512, 18536, 65488, 13588, 48616, 10632,
                31708, 23144, 28812, 50296, 11428, 5936,  46056, 60164, 54440,
                43476, 52172, 59720, 15152, 21952, 63644, 17480, 52920, 28976,
                25820, 11236, 59220, 32224, 28444, 52352, 21144, 6340,  15648,
                46300, 8124,  40808, 11868, 27688, 15444, 1832,  37908, 25156,
                54568, 9732,  61232, 35068, 55024, 47288, 37648, 33592, 49104,
                19572, 30824,
            },
            {
                6008,  18452, 47364, 25848, 4704,  50880, 29624, 64204, 38016,
                18540, 53708, 62216, 40144, 7064,  32184, 56340, 43412, 34652,
                61572, 38332, 3240,  41760, 7872,  33972, 38448, 51116, 21492,
                43060, 1952,  53528, 64176, 29128, 4384,  12792, 46660, 39812,
                56880, 65180, 23052, 11268, 50448, 59004, 30440, 63096, 40660,
                4388,  49752, 7732,  39160, 52252, 25132, 55456, 63608, 3772,
                25976, 60228, 22280, 39800, 48396, 18196, 26476, 13560, 49140,
                44732, 28288, 6992,  14896, 51080, 22600, 5296,  16324, 49420,
                19468, 25524, 56620, 10176, 30976, 6216,  57576, 34296, 43472,
                3652,  57104, 63412, 2240,  34016, 57988, 31104, 1772,  14316,
                23960, 4828,  40184, 55032, 27256, 6168,  57536, 33600, 14320,
                64528, 47028, 4032,  40908, 56544, 9452,  47760, 39304, 27448,
                54128, 19412, 51024, 32396, 55616, 20944, 49848, 59272, 12552,
                29164, 4976,  44180, 17368, 58904, 2376,  10784, 64392, 13700,
                41116, 35864,
            },
            {
                49964, 23120, 64816, 31932, 60040, 21744, 2932,  14480, 56280,
                6736,  33740, 1012,  13440, 59176, 46988, 10988, 28908, 51452,
                14668, 58088, 32076, 52076, 62336, 28700, 14616, 63436, 9952,
                59124, 36672, 24748, 47532, 18920, 34932, 62564, 21888, 27612,
                9828,  18088, 29972, 41768, 3576,  16780, 34452, 9664,  20756,
                56196, 26800, 11004, 58100, 20584, 1024,  32928, 10412, 47684,
                36044, 7536,  11596, 33628, 60968, 9216,  35804, 53156, 64004,
                10144, 34948, 57184, 30548, 11764, 36092, 58320, 27060, 53048,
                42572, 7008,  36344, 23500, 41864, 52412, 27236, 21660, 61648,
                15500, 46616, 19584, 41600, 25344, 49244, 17632, 41220, 34520,
                47680, 19908, 32316, 9212,  37260, 48016, 44812, 23036, 37588,
                7860,  42904, 19664, 35908, 13656, 24488, 60896, 0,     64512,
                12812, 34540, 2708,  60636, 39208, 9004,  44100, 6592,  41792,
                21480, 52060, 31404, 12984, 40276, 21304, 29116, 43332, 52392,
                25196, 57452,
            },
            {
                44164, 14868, 37964, 10812, 55648, 45632, 35064, 26948, 50172,
                22800, 46136, 27872, 52200, 36212, 16752, 64732, 23048, 39844,
                8752,  26184, 48980, 20404, 11212, 22988, 45720, 3116,  26224,
                48964, 17168, 6528,  39280, 10992, 44160, 1112,  37860, 54300,
                6500,  33360, 52400, 60216, 36980, 25920, 48920, 43600, 14952,
                60652, 46768, 17856, 42892, 28720, 48892, 60888, 16020, 43276,
                58276, 19404, 45700, 51272, 55904, 2888,  28924, 40124, 1048,
                24088, 42104, 20232, 2160,  54108, 40768, 9272,  76,    34068,
                11532, 64144, 15156, 48956, 61228, 3044,  17208, 40364, 7816,
                36744, 53148, 9200,  13820, 38140, 64412, 21168, 8912,  62092,
                56256, 12588, 63272, 50700, 18188, 1716,  12200, 60708, 3140,
                53940, 30176, 61936, 2128,  50796, 44624, 18128, 31328, 36752,
                56776, 42848, 24812, 16928, 29836, 64676, 26024, 32704, 56616,
                36096, 1476,  65500, 49324, 26936, 53660, 60360, 7096,  17296,
                408,   10164,
            },
            {
                33716, 54500, 3148,  41964, 19512, 7896,  40044, 61768, 11008,
                42308, 16032, 63708, 9356,  24564, 41816, 5988,  456,   54596,
                45564, 16948, 35984, 1508,  59612, 39552, 55732, 31332, 35040,
                13020, 54356, 30424, 59968, 51444, 57020, 31656, 49696, 14280,
                43500, 48212, 1388,  20352, 12848, 64284, 6600,  52904, 28928,
                33024, 1720,  36632, 63312, 14152, 5236,  38212, 22148, 27492,
                54660, 256,   29860, 24512, 15060, 21264, 44032, 58056, 31944,
                15644, 50364, 62032, 45712, 24996, 65172, 47136, 21612, 60480,
                30236, 45780, 55484, 20452, 33324, 38156, 11644, 50716, 55820,
                23976, 27940, 32812, 44172, 55256, 6428,  29180, 52820, 36856,
                26324, 4076,  28888, 42948, 56768, 24876, 31680, 41224, 19160,
                48904, 15340, 22708, 57608, 27372, 38568, 7484,  51800, 22376,
                10104, 48532, 5768,  52604, 46076, 704,   53928, 14564, 17664,
                59988, 46160, 23884, 8004,  34528, 3788,  45484, 31904, 38248,
                62180, 27704,
            },
            {
                59260, 7308,  30052, 26300, 49248, 58664, 16924, 1856,  30908,
                57996, 4220,  39372, 31540, 19436, 50712, 57596, 30264, 19860,
                63284, 4816,  43048, 30560, 53352, 6364,  18824, 42388, 4724,
                60956, 44428, 21116, 2692,  23924, 15956, 8444,  19700, 59116,
                25012, 62116, 39116, 28516, 46940, 55500, 22612, 2704,  39016,
                12380, 23788, 54036, 7232,  32212, 45584, 51920, 8736,  34620,
                12884, 39472, 62340, 4640,  37124, 65216, 11472, 47004, 19364,
                4572,  55040, 8340,  37492, 17912, 29084, 15072, 54960, 38628,
                18264, 5760,  1600,  28588, 8528,  58776, 45924, 29292, 752,
                64688, 5024,  49748, 60496, 1412,  24260, 12128, 43712, 2568,
                16876, 46312, 39084, 7188,  34872, 64840, 51544, 27632, 58116,
                9076,  34816, 45436, 5920,  11652, 33060, 58956, 15096, 3292,
                26856, 59476, 37776, 13612, 22568, 11144, 37036, 48756, 3488,
                28176, 10692, 38788, 15848, 57160, 19488, 11708, 23416, 55084,
                46668, 20420,
            },
            {
                50872, 16740, 63736, 53288, 13404, 32832, 24376, 52812, 46804,
                35884, 21496, 54232, 59576, 2788,  45204, 13328, 37600, 33084,
                49716, 58484, 13428, 24680, 46432, 15316, 64396, 50968, 24024,
                38304, 8760,  65460, 34232, 42324, 27328, 63228, 36716, 29620,
                4476,  10952, 16128, 57956, 8676,  32540, 18348, 45724, 57456,
                62568, 41996, 50160, 20116, 56788, 24660, 59384, 2744,  64224,
                49072, 17308, 52764, 42464, 32360, 49328, 6392,  25672, 61140,
                39112, 27836, 33256, 12728, 4884,  42768, 32244, 7576,  50632,
                24688, 40640, 52488, 44128, 14052, 24820, 62908, 19156, 42432,
                35328, 12244, 20888, 17292, 30772, 46888, 57264, 33180, 50164,
                60248, 23124, 58220, 14476, 21616, 10264, 16056, 5168,  38348,
                62848, 25696, 52536, 40508, 65472, 20068, 47284, 41200, 63464,
                45084, 30528, 19692, 62924, 33564, 57916, 40940, 24080, 63572,
                43368, 20536, 51128, 62364, 42140, 48092, 36952, 63828, 14576,
                5640,  40620,
            },
            {
                11608, 23856, 44988, 824,   36376, 43484, 5620,  65464, 19208,
                13100, 27164, 7044,  47896, 34308, 26080, 61604, 7504,  23696,
                10332, 27792, 39172, 62580, 8640,  36700, 27992, 10976, 57328,
                16316, 28784, 48588, 13880, 6132,  46776, 52888, 2032,  41412,
                55312, 34416, 23544, 44388, 3872,  41352, 26372, 35852, 8956,
                17016, 3940,  27732, 10432, 40056, 16416, 29676, 41644, 19736,
                31728, 6656,  26664, 9632,  59072, 18308, 53568, 35344, 13876,
                56576, 47788, 21132, 52960, 59416, 48948, 2824,  62540, 12312,
                34732, 58440, 63324, 22380, 36000, 49144, 6300,  31548, 14768,
                48136, 57560, 40680, 52708, 37256, 61820, 15100, 40160, 19976,
                10540, 31408, 53160, 24,    48848, 43552, 55452, 46496, 32880,
                13064, 992,   17788, 30952, 55096, 1740,  24756, 53596, 34140,
                17516, 7988,  54864, 42320, 4180,  50200, 19060, 31212, 8428,
                55528, 32960, 5540,  29540, 928,   25944, 7364,  51368, 30160,
                2268,  35496,
            },
            {
                47872, 31884, 39264, 21568, 10076, 59916, 28184, 38696, 9168,
                50588, 44104, 61032, 15520, 10752, 53032, 17748, 40188, 44636,
                54140, 2352,  19184, 48760, 32304, 21196, 228,   47796, 32956,
                3512,  53180, 19300, 56472, 39124, 59688, 21540, 13016, 45596,
                18384, 64920, 50640, 36460, 53564, 62892, 14392, 59800, 47988,
                30428, 34680, 45080, 65296, 660,   36052, 12372, 54876, 44720,
                23444, 60648, 47296, 14200, 22416, 39972, 2184,  30476, 8892,
                43232, 736,   64324, 26488, 39704, 23308, 56824, 44816, 27544,
                16344, 3628,  31108, 10776, 57100, 40048, 2112,  54244, 59716,
                22944, 7060,  26612, 3136,  10316, 22316, 7488,  27568, 65468,
                5128,  41828, 35772, 25500, 60616, 30664, 3336,  23488, 20040,
                56416, 42832, 48344, 9400,  37012, 13928, 29420, 4876,  11012,
                39408, 50776, 1156,  25308, 28824, 9780,  59908, 4980,  47428,
                14028, 37316, 53420, 12080, 59636, 33644, 17524, 58384, 43688,
                26468, 60836,
            },
            {
                8132,  57460, 4456,  62056, 54900, 15920, 49040, 22308, 56628,
                108,   30388, 37372, 22784, 42356, 28728, 1780,  50308, 63852,
                15568, 34524, 56592, 6088,  60700, 54540, 40240, 61852, 44984,
                22696, 41772, 36424, 948,   30648, 10232, 33412, 26320, 49336,
                8172,  31204, 5260,  20744, 11820, 29644, 520,   51456, 21176,
                55252, 13112, 58120, 22276, 51056, 47256, 61764, 4880,  9964,
                37856, 57088, 35136, 4292,  55364, 27444, 45548, 62816, 23620,
                36904, 17608, 11360, 7120,  34892, 14056, 19408, 37120, 8984,
                48488, 43248, 19604, 7124,  51568, 16504, 33764, 27112, 9692,
                38128, 46144, 32472, 64056, 42536, 50964, 35140, 1056,  54868,
                47772, 16300, 62256, 7752,  18628, 36932, 40376, 63992, 50268,
                6448,  61008, 27552, 21280, 58396, 44148, 49944, 61760, 21636,
                58020, 14460, 61372, 36440, 44764, 16560, 53044, 35596, 26728,
                61784, 18032, 24556, 45188, 21052, 49512, 41264, 9896,  21832,
                53204, 18392,
            },
            {
                14972, 51404, 27860, 18856, 46620, 31556, 4144,  34332, 41724,
                63144, 18096, 54580, 5012,  64796, 57052, 35980, 21720, 5428,
                29748, 41728, 22844, 12332, 43504, 17312, 25388, 14096, 7148,
                58596, 11692, 62872, 24148, 50672, 17004, 63908, 5508,  57192,
                39628, 14520, 27204, 61420, 47236, 39932, 24292, 7604,  43120,
                4684,  25992, 38740, 8088,  32564, 19000, 24924, 27964, 52064,
                15732, 1756,  29492, 44008, 63816, 10984, 50028, 15360, 58360,
                52184, 31996, 45216, 49988, 61872, 1912,  30912, 51852, 21876,
                64868, 55004, 38256, 26044, 45348, 61704, 21692, 43444, 51924,
                17984, 60756, 13436, 54296, 16672, 29868, 59388, 44016, 24048,
                38012, 28804, 11612, 45428, 51244, 27068, 12528, 8656,  29852,
                35932, 14960, 39520, 2764,  63384, 16812, 7588,  36204, 46908,
                27820, 32220, 20180, 48188, 12296, 65072, 39612, 21216, 212,
                42080, 57848, 3184,  39248, 64940, 14748, 55596, 3392,  32864,
                64244, 37464,
            },
            {
                24504, 42880, 34748, 11736, 40988, 24312, 8260,  52160, 14912,
                11332, 25312, 33212, 46336, 8716,  13704, 31908, 60332, 11444,
                46916, 59040, 52180, 37680, 29368, 4700,  51184, 35560, 30488,
                18300, 49464, 28072, 8392,  45088, 54152, 42628, 35552, 22932,
                60364, 52636, 42984, 2492,  17560, 54932, 34152, 64624, 37564,
                18500, 61408, 52724, 15492, 43348, 3380,  58672, 40336, 33816,
                64836, 21008, 50744, 17124, 38284, 33192, 19776, 6888,  40588,
                3088,  21420, 57040, 28648, 41432, 54496, 5792,  59956, 40140,
                436,   32748, 13716, 59856, 29908, 4464,  12704, 65084, 5616,
                29424, 220,   36228, 24172, 3992,  48228, 19192, 9068,  13912,
                52108, 20964, 56324, 33596, 2088,  58876, 53548, 17504, 45524,
                24448, 54448, 33668, 51260, 23452, 32384, 42164, 25428, 54456,
                2288,  43680, 6916,  56908, 24168, 5984,  30400, 56392, 45884,
                10404, 49976, 30876, 8352,  27396, 4992,  36604, 28848, 12804,
                45336, 1276,
            },
            {
                30456, 5240,  63420, 54120, 2468,  58540, 62656, 45148, 29068,
                57976, 38984, 2384,  51156, 40392, 19848, 48468, 25620, 38632,
                18388, 8916,  26112, 1300,  48456, 59512, 9628,  64000, 39292,
                54620, 4372,  34060, 59280, 20172, 2996,  14708, 29072, 52,
                19124, 10364, 32276, 37320, 58660, 9176,  15016, 49876, 30880,
                10676, 47064, 1584,  28752, 63192, 53800, 13260, 48928, 6956,
                45688, 11556, 41244, 7916,  24536, 380,   61564, 29140, 46568,
                25612, 60880, 14416, 8932,  18304, 24100, 46088, 16012, 28372,
                11288, 23852, 50940, 2820,  18180, 48076, 34888, 56008, 40632,
                49720, 20688, 57128, 45152, 10856, 38904, 56396, 61492, 32012,
                3096,  63452, 5804,  42584, 15132, 22472, 62432, 41736, 600,
                58300, 11232, 5000,  46392, 9956,  56720, 3840,  12636, 17792,
                63924, 10212, 38516, 52572, 34376, 2104,  51112, 13652, 33832,
                22916, 16360, 62624, 34392, 52476, 47544, 23328, 61540, 40340,
                49728, 59420,
            },
            {
                56148, 47472, 22052, 14256, 37956, 26940, 20212, 36488, 3396,
                48556, 21340, 61612, 16656, 27356, 56064, 664,   43176, 55068,
                3592,  32536, 65304, 14824, 21824, 32032, 44492, 19928, 1668,
                24616, 47240, 13468, 40916, 37244, 25588, 61984, 48012, 38216,
                45768, 55988, 24864, 50272, 21600, 28304, 45480, 1980,  22104,
                59296, 33448, 41052, 23204, 35832, 9232,  31140, 17868, 22876,
                56316, 26244, 61256, 53228, 58208, 44752, 36208, 52740, 12252,
                34436, 5960,  38620, 43972, 33520, 64120, 36500, 8028,  57948,
                47648, 61944, 36136, 41916, 54520, 39020, 10136, 25336, 15860,
                32284, 8584,  41696, 63496, 28328, 33892, 5060,  26252, 41192,
                46216, 36244, 25180, 49648, 30276, 37532, 4904,  32916, 49256,
                20544, 64480, 42636, 18656, 27028, 52244, 39672, 59428, 33836,
                50676, 22760, 29244, 16988, 59396, 42696, 18508, 63112, 28140,
                6672,  54556, 41372, 19744, 11436, 42448, 57624, 17752, 6960,
                20512, 10864,
            },
            {
                16376, 40012, 8796,  32068, 49704, 43508, 10964, 17080, 53832,
                6856,  43056, 31080, 10684, 36772, 63520, 7540,  15656, 50708,
                61968, 36640, 45260, 55956, 41312, 53384, 12936, 27404, 56964,
                42568, 16452, 65004, 5660,  55336, 31604, 11588, 52016, 6644,
                64692, 13672, 3672,  44424, 6464,  63260, 41172, 56188, 26608,
                52144, 13756, 6776,  56968, 19620, 46212, 39088, 60584, 888,
                37072, 32644, 5124,  30596, 14084, 21576, 9728,  56224, 17484,
                65408, 48340, 54084, 26896, 3500,  12408, 51124, 31436, 42476,
                17392, 5368,  20764, 8668,  27400, 59112, 22088, 2520,  46480,
                61056, 26680, 51220, 18372, 14148, 53360, 22604, 50564, 16424,
                10644, 59348, 18468, 8188,  57216, 46780, 10420, 25580, 38888,
                28572, 13900, 31256, 37688, 60680, 15124, 30208, 20620, 45864,
                6320,  41400, 61708, 12856, 46096, 26308, 9248,  40524, 60188,
                47852, 37252, 2232,  59820, 25968, 15600, 464,   31832, 54368,
                26440, 35292,
            },
            {
                65244, 28324, 53252, 18252, 61632, 940,   56216, 33352, 65024,
                24368, 14220, 58516, 52472, 22996, 45208, 29408, 34588, 23896,
                20236, 12160, 28492, 7356,  17916, 4248,  37480, 62676, 7696,
                32256, 52512, 28932, 22480, 45988, 8840,  58504, 17180, 33632,
                21028, 27880, 36076, 60656, 16844, 33584, 12536, 5356,  38700,
                17512, 43988, 64344, 49052, 26508, 3960,  55464, 28468, 44120,
                14760, 49360, 19172, 39556, 46972, 27588, 3756,  42184, 23188,
                30940, 1104,  20492, 59644, 46692, 56744, 21016, 1464,  25784,
                55948, 33424, 45280, 64484, 14452, 31456, 50216, 63320, 36648,
                13028, 4948,  35016, 2696,  59912, 7428,  43576, 64504, 532,
                29188, 53196, 40228, 13348, 65136, 20128, 52800, 16092, 61400,
                7660,  50504, 55388, 2148,  47644, 7972,  64852, 252,   36644,
                27156, 55244, 3544,  32676, 7196,  49764, 20916, 36216, 3328,
                24796, 12900, 30088, 56096, 7380,  45928, 64152, 51448, 37880,
                43820, 2040,
            },
            {
                45740, 5888,  59052, 36100, 7368,  23232, 46504, 27712, 40528,
                9224,  35204, 47112, 6388,  2924,  59668, 12832, 53268, 42032,
                5628,  48824, 58592, 24640, 35028, 60748, 46476, 23092, 50000,
                10800, 36380, 2144,  60780, 18512, 50248, 40280, 23888, 43384,
                59448, 9264,  40792, 53600, 30632, 51608, 20392, 48372, 61788,
                2960,  29440, 10152, 37612, 15872, 61836, 12644, 5912,  52704,
                63472, 10112, 57292, 1872,  64472, 51192, 59680, 38736, 8024,
                50860, 36564, 16320, 10352, 30320, 39424, 14940, 62880, 37616,
                49828, 12208, 24524, 52368, 764,   43824, 7212,  17760, 42620,
                53848, 58580, 21388, 39676, 47744, 30972, 37740, 19988, 35276,
                47908, 21964, 32604, 1700,  27360, 34824, 42728, 2332,  56344,
                44736, 23572, 5864,  35632, 21464, 25652, 43088, 49820, 10928,
                58624, 14752, 19884, 37544, 64376, 57268, 30700, 54312, 15904,
                52300, 45036, 21292, 49672, 39432, 35232, 29260, 9520,  13724,
                60808, 22652,
            },
            {
                50908, 12860, 25240, 42224, 29892, 51936, 13464, 5300,  59788,
                49584, 18960, 26388, 41096, 32548, 20500, 39404, 9396,  57196,
                31164, 1436,  39620, 52264, 10400, 30108, 556,   15608, 40264,
                19444, 57880, 43568, 12808, 34776, 27340, 4972,  1484,  29656,
                54416, 48536, 15364, 808,   23468, 10020, 58012, 35448, 24264,
                32264, 54712, 21780, 1228,  34044, 51004, 41404, 20880, 34408,
                25552, 41968, 22628, 35164, 11592, 17952, 33236, 15144, 45756,
                57872, 25408, 42712, 63292, 4832,  23736, 53468, 44692, 10060,
                3928,  29712, 58768, 38392, 19808, 34360, 57524, 28448, 23652,
                9668,  30372, 45312, 25192, 55268, 16052, 9556,  56712, 13056,
                61176, 6976,  54904, 44608, 50112, 5684,  60396, 29480, 11812,
                34080, 17620, 63352, 41812, 57564, 13164, 53624, 17448, 31808,
                24472, 46884, 52900, 41860, 23368, 580,   12440, 43736, 5220,
                65504, 33332, 9800,  61716, 17420, 4068,  24040, 19800, 48960,
                4376,  32992,
            },
            {
                18944, 38200, 2900,  48728, 15552, 63612, 39400, 21136, 30780,
                53344, 540,   61892, 56568, 16196, 49708, 64896, 25756, 18212,
                61076, 44380, 16024, 63564, 21068, 50508, 55460, 33568, 59736,
                5340,  24184, 48124, 7420,  38852, 53060, 64072, 45416, 12468,
                37452, 5972,  26124, 63700, 46944, 38352, 7320,  44204, 14932,
                59180, 40556, 49920, 45236, 58844, 24120, 31316, 8580,  47280,
                16940, 3520,  29940, 54408, 43800, 24296, 5592,  29684, 63684,
                3252,  13308, 51980, 33068, 48560, 7920,  28012, 34712, 19380,
                61332, 41064, 16156, 4924,  47688, 11424, 52988, 39452, 1444,
                48984, 16764, 65316, 11824, 1788,  60592, 27624, 45536, 3704,
                25536, 38132, 15784, 62236, 24036, 17828, 37732, 22220, 48208,
                40164, 26892, 52612, 9988,  28688, 33580, 62164, 5328,  40132,
                61060, 2608,  29808, 9968,  16248, 47636, 34340, 59304, 28488,
                18184, 38356, 984,   26852, 52956, 43464, 62592, 56668, 41084,
                30164, 55468,
            },
            {
                43164, 57928, 21320, 60672, 9140,  35352, 56792, 2700,  44996,
                15676, 37804, 12004, 29496, 45824, 5180,  35960, 2312,  46992,
                34236, 12580, 27548, 36876, 6412,  42168, 25528, 11884, 45120,
                27976, 53456, 30556, 62928, 14772, 20160, 32716, 57556, 16596,
                61268, 19788, 34828, 56596, 42196, 17920, 28800, 64980, 3720,
                11876, 19096, 7392,  27700, 11056, 18120, 65512, 54840, 37440,
                58068, 62616, 49932, 8252,  37932, 61592, 55680, 49148, 21448,
                9580,  28624, 38240, 18064, 58724, 40844, 56268, 2572,  51468,
                22952, 46188, 55064, 27684, 62844, 21544, 60596, 14916, 33028,
                56948, 37432, 6824,  34240, 49940, 41144, 23256, 33308, 50848,
                43036, 57732, 30736, 11536, 41208, 9032,  58508, 54044, 4228,
                62020, 14856, 768,   46340, 19476, 3312,  37644, 22776, 44860,
                8316,  51264, 35712, 62748, 55604, 25900, 39364, 6940,  21756,
                49036, 42352, 57232, 14756, 32064, 12232, 47320, 7628,  25832,
                15980, 10304,
            },
            {
                6432,  52660, 27500, 32036, 45268, 18712, 26276, 11292, 34280,
                64388, 23824, 43792, 8556,  21872, 52348, 14044, 28600, 54728,
                7712,  23404, 55860, 3028,  47696, 18552, 65076, 38364, 3476,
                61816, 16620, 41376, 740,   56508, 25808, 9852,  49088, 23104,
                41024, 51880, 2712,  30248, 13496, 4584,  53880, 25824, 47344,
                52560, 36120, 63484, 56024, 38884, 5420,  43448, 32,    13568,
                27472, 6260,  45176, 20592, 13932, 27072, 508,   40836, 35200,
                44264, 53912, 61124, 908,   21936, 11892, 16532, 65012, 31024,
                13616, 6768,  32920, 9348,  36764, 3200,  25112, 41808, 5132,
                62128, 20836, 43896, 29112, 19056, 5696,  53084, 62344, 17196,
                8860,  20524, 4728,  52328, 33724, 46120, 14376, 31288, 8092,
                21048, 57320, 32092, 39012, 51228, 58948, 48308, 11972, 55976,
                19176, 14080, 27320, 43480, 5728,  18780, 51020, 63288, 10464,
                54644, 24128, 8148,  60344, 37096, 22184, 2532,  33540, 59648,
                36588, 63340,
            },
            {
                23720, 34612, 12136, 412,   54484, 6648,  41036, 49200, 58892,
                28248, 5868,  54796, 33900, 63012, 38484, 58200, 42328, 20576,
                50976, 62380, 40576, 32488, 58532, 14344, 8160,  31208, 49684,
                20996, 9100,  34252, 22384, 50788, 44524, 4168,  36280, 7132,
                31516, 11128, 45708, 21956, 62168, 49024, 33700, 20052, 41488,
                30728, 23312, 2200,  32884, 14892, 52068, 29264, 48968, 23216,
                40692, 32708, 16400, 60412, 34072, 47560, 11172, 18748, 57540,
                15996, 23932, 7200,  31412, 47216, 36348, 43840, 24988, 48864,
                39072, 59436, 18496, 51744, 43952, 30568, 50704, 46288, 8948,
                26336, 52072, 13492, 54964, 63640, 38204, 14196, 1208,  29288,
                36812, 64808, 48352, 27756, 20,    63904, 25124, 50560, 42920,
                35904, 45044, 11468, 65332, 7344,  15796, 26464, 29936, 64108,
                33000, 39204, 21648, 1976,  59796, 32176, 44960, 14536, 30552,
                35624, 3480,  45344, 28816, 49504, 54972, 40116, 18312, 52008,
                1264,  47124,
            },
            {
                39808, 17056, 64860, 47996, 36960, 61528, 22940, 52892, 3764,
                19956, 39680, 50580, 17900, 1404,  24716, 10948, 31732, 3848,
                36492, 17128, 10584, 44708, 21980, 28628, 52544, 57632, 36028,
                13512, 55396, 47096, 38076, 11916, 61544, 28500, 42532, 55188,
                65384, 27268, 58512, 8052,  36880, 55560, 10756, 344,   61200,
                8648,  16516, 48740, 42260, 25484, 60200, 20508, 35176, 61236,
                10900, 51172, 2064,  53944, 39600, 6628,  52632, 31688, 62048,
                3876,  50080, 41872, 26672, 63180, 54648, 4348,  9088,  57464,
                420,   42756, 26212, 63848, 15392, 57760, 12188, 19084, 56112,
                31696, 40268, 3864,  24244, 46176, 8636,  22096, 47572, 42144,
                54180, 12688, 23072, 39716, 56692, 19552, 36928, 3128,  28268,
                18044, 54684, 24808, 30244, 22296, 43460, 36072, 1232,  50224,
                5048,  58868, 53920, 48272, 12996, 36976, 23480, 1092,  57588,
                40504, 16652, 63868, 20072, 4636,  10448, 64996, 28076, 43908,
                13844, 29356,
            },
            {
                58804, 3980,  42436, 20648, 28748, 12840, 16152, 31600, 46624,
                10208, 14512, 57592, 30964, 47520, 60720, 16520, 44836, 64348,
                48712, 26600, 12,    53744, 61464, 4776,  42732, 23380, 1248,
                44040, 26408, 64556, 2972,  31792, 16040, 58780, 19344, 3084,
                14444, 50032, 18084, 43052, 25072, 16096, 39960, 57960, 45456,
                37044, 56532, 62268, 12504, 46280, 2908,  55716, 8136,  44904,
                19116, 56552, 25340, 29568, 22312, 65156, 44688, 24912, 38724,
                12872, 33396, 59332, 10340, 14784, 19612, 33812, 28536, 37384,
                21580, 11088, 34808, 5716,  23212, 1748,  38440, 64768, 35584,
                15900, 48020, 10700, 58412, 34944, 31100, 56204, 59752, 25896,
                32816, 3524,  59328, 16348, 43784, 6820,  53232, 59896, 10120,
                62720, 2360,  48816, 4952,  59760, 55676, 9528,  40932, 17680,
                24580, 44472, 7524,  29384, 41100, 9288,  62036, 52344, 27160,
                47172, 12152, 51328, 31700, 42980, 15896, 24568, 56980, 8400,
                20780, 50416,
            },
            {
                10052, 55116, 25712, 7636,  59264, 44140, 1828,  63632, 36568,
                42932, 61452, 26076, 4416,  41496, 8280,  27768, 55504, 6652,
                13856, 59564, 30852, 19100, 34992, 39252, 11000, 48464, 17904,
                59536, 6160,  29580, 19064, 53376, 8012,  39604, 47888, 25332,
                34560, 38840, 1188,  31988, 52032, 5352,  29056, 22328, 13176,
                27056, 4912,  29728, 21432, 9624,  37716, 16900, 30936, 3776,
                64340, 36544, 42344, 9364,  57748, 15664, 1712,  28936, 8300,
                55400, 21004, 48260, 2596,  39616, 51392, 45592, 61940, 17088,
                50500, 60804, 46384, 54236, 40568, 48736, 27488, 7264,  22488,
                372,   60848, 28344, 18228, 2260,  40664, 15968, 11216, 6164,
                18896, 46460, 35116, 9220,  31592, 12452, 22468, 47388, 32388,
                41168, 16228, 38060, 33804, 13324, 19756, 52436, 46720, 61724,
                11368, 34984, 15292, 65160, 25380, 55780, 17472, 5928,  20504,
                59544, 33860, 25236, 7328,  58740, 34832, 46872, 37904, 3248,
                62680, 32608,
            },
            {
                35972, 14852, 48880, 38556, 51808, 33356, 56400, 24392, 6472,
                29156, 20948, 48900, 12284, 35580, 53416, 19712, 39652, 34132,
                24276, 38344, 46840, 7908,  50804, 15792, 26848, 62864, 33660,
                38828, 51096, 23644, 57896, 43648, 35236, 22192, 62988, 10424,
                44772, 54668, 62620, 12676, 59208, 47700, 64440, 33216, 50400,
                18244, 52224, 39840, 54116, 33472, 63044, 50088, 40248, 27240,
                14988, 47880, 4928,  13296, 49496, 37292, 19704, 51616, 63392,
                45224, 16704, 35784, 29468, 56432, 23152, 5404,  13044, 53972,
                3272,  29404, 14008, 31596, 19924, 61476, 33916, 44976, 54492,
                51068, 42804, 37684, 63276, 52768, 26880, 49480, 62712, 38032,
                51532, 61616, 24532, 55272, 49304, 62968, 38768, 26560, 13904,
                57432, 21548, 53700, 45744, 63200, 25668, 32260, 3552,  28388,
                57244, 38476, 19384, 51320, 3280,  34176, 49180, 42444, 38260,
                9588,  2556,  62228, 39592, 52668, 208,   12844, 30832, 54324,
                23472, 45528,
            },
            {
                19196, 61932, 31232, 3160,  18612, 9456,  40468, 17320, 50692,
                55664, 296,   38420, 65016, 23168, 46116, 59368, 1552,  50056,
                10252, 42872, 57284, 22872, 64576, 3300,  55760, 30576, 7684,
                14508, 10516, 40912, 5064,  48744, 13200, 56,    52156, 30384,
                4772,  20608, 23880, 7252,  35952, 19764, 3004,  42680, 8184,
                60428, 2004,  44476, 7112,  26304, 58272, 22580, 12360, 53276,
                59684, 33712, 21096, 31172, 43904, 59424, 34420, 12008, 40564,
                5212,  25448, 61328, 43192, 8048,  64164, 26924, 32016, 43328,
                39532, 23944, 8080,  56984, 3600,  10672, 17188, 59592, 13600,
                30180, 19856, 5460,  13008, 44640, 7432,  20140, 43592, 672,
                29616, 14048, 41360, 2044,  28408, 17928, 4124,  44724, 6480,
                49484, 8708,  27832, 1624,  40580, 6908,  37420, 14532, 22492,
                53464, 92,    31120, 60504, 22028, 46192, 28708, 13952, 31248,
                54708, 22688, 44076, 17940, 28376, 22004, 49644, 60156, 16372,
                41856, 4512,
            },
            {
                28452, 57688, 12036, 23008, 64456, 27132, 47588, 12488, 60280,
                33672, 9144,  52312, 17496, 32324, 4916,  14984, 30028, 21352,
                61088, 16448, 2328,  29196, 12708, 40804, 44488, 19556, 53444,
                45784, 60696, 32120, 17424, 63808, 27920, 57352, 33456, 15092,
                61072, 49064, 29308, 41056, 45876, 26816, 15044, 55056, 38688,
                24552, 35092, 65020, 14192, 18688, 1124,  47012, 6232,  43360,
                24288, 592,   55692, 63036, 6084,  26884, 3040,  47380, 22180,
                31236, 52644, 96,    13776, 18620, 37984, 48452, 9872,  59508,
                18340, 65356, 49792, 35836, 42548, 52588, 24236, 4936,  40992,
                9192,  25556, 57672, 32940, 23552, 55924, 36124, 32160, 22904,
                58040, 8328,  45412, 20548, 36356, 52212, 59380, 33680, 64656,
                24304, 36184, 61652, 12132, 18164, 50656, 58296, 64280, 42012,
                48724, 9600,  44916, 40484, 6708,  11080, 63832, 1424,  60796,
                50408, 36364, 14568, 55532, 10016, 63104, 40628, 5500,  26688,
                9184,  52236,
            },
            {
                46760, 596,   40176, 44452, 53516, 35420, 4352,  30636, 22076,
                45228, 14672, 42340, 28020, 57952, 10656, 43928, 62820, 51964,
                6400,  33052, 54124, 36860, 48872, 24856, 60220, 5644,  22668,
                36188, 27108, 55480, 2188,  36996, 24200, 45996, 18288, 43068,
                38080, 9504,  56424, 16748, 53672, 10636, 61844, 29788, 48600,
                11512, 20860, 31064, 49620, 41444, 36416, 55016, 32500, 9704,
                38164, 18368, 50988, 39848, 23784, 53556, 17724, 60564, 56124,
                37572, 10220, 49120, 58372, 32964, 54968, 4204,  22112, 34476,
                848,   45288, 26604, 15208, 62916, 28892, 38312, 32848, 48520,
                63588, 36408, 46092, 50436, 2648,  15020, 64236, 10168, 53692,
                17564, 39052, 65380, 5464,  56604, 10344, 14936, 42116, 728,
                18904, 51656, 31336, 43596, 55996, 29912, 9136,  20456, 5388,
                33504, 26256, 13500, 58452, 16744, 35964, 56520, 24456, 19044,
                45300, 4556,  30092, 47360, 3620,  35572, 18616, 44740, 33336,
                64788, 37268,
            },
            {
                24440, 54468, 32784, 6532,  15400, 58568, 41876, 63020, 7184,
                36372, 25040, 62500, 2752,  48048, 54672, 25560, 36668, 40772,
                27496, 46040, 20164, 61912, 8208,  17276, 34536, 2388,  50144,
                65292, 6700,  20928, 42060, 11772, 54592, 8376,  5380,  59452,
                26720, 1808,  34468, 63032, 5516,  37604, 21764, 632,   57724,
                16776, 46220, 56116, 9884,  59976, 27620, 16108, 63780, 29396,
                61160, 45588, 7600,  15888, 10960, 30188, 42692, 8072,  14700,
                27564, 64424, 19892, 41128, 28508, 45440, 16244, 50600, 57148,
                12640, 37800, 5496,  21024, 46252, 2180,  58104, 20464, 55508,
                1512,  17052, 11348, 61752, 39232, 27336, 46604, 4272,  48544,
                27816, 33848, 50800, 25964, 31784, 46740, 21812, 29736, 55076,
                39976, 15408, 58912, 4164,  22720, 48036, 35480, 45244, 16240,
                55240, 62428, 22956, 49436, 27868, 51620, 43528, 39336, 8360,
                27272, 12248, 65228, 38940, 57260, 25108, 53644, 12124, 58696,
                21128, 13748,
            },
            {
                7692,  17580, 60388, 49396, 20656, 25228, 1528,  48808, 18844,
                54528, 59480, 6204,  19900, 39000, 34208, 7520,  17696, 12544,
                56548, 4156,  11200, 43548, 31476, 51772, 57816, 27740, 40324,
                12732, 16116, 47148, 51512, 62300, 29716, 40156, 50168, 22864,
                52720, 13264, 20216, 48200, 30944, 43696, 51736, 33476, 41112,
                4940,  28260, 38280, 2460,  22412, 44332, 3504,  20260, 49392,
                13608, 26344, 35152, 57804, 46772, 64888, 33500, 50464, 1972,
                44504, 34940, 6800,  24252, 3092,  11432, 63656, 25504, 41960,
                29464, 51864, 55432, 33572, 11404, 51324, 8192,  14484, 44068,
                27952, 53220, 21808, 7928,  30588, 19032, 57112, 41180, 13092,
                60608, 1812,  11580, 16492, 61312, 38504, 3236,  50264, 25424,
                6972,  46592, 9928,  38728, 26956, 13548, 1268,  61172, 28556,
                39624, 2080,  37100, 4620,  32968, 20084, 3648,  15164, 53192,
                34448, 58400, 21392, 16648, 32308, 8388,  48656, 29328, 1588,
                49960, 42348,
            },
            {
                62112, 27896, 10444, 37048, 30040, 13128, 52700, 38372, 27744,
                11140, 40652, 50136, 29556, 12956, 22456, 64568, 576,   49020,
                23648, 63764, 38624, 25916, 976,   14272, 37916, 9888,  43932,
                31220, 59028, 25216, 33272, 4056,  14804, 20568, 34912, 64380,
                31492, 41912, 58632, 3308,  24076, 14528, 8552,  59672, 25628,
                63896, 7936,  53152, 61676, 13172, 51584, 34764, 57668, 5020,
                41368, 54588, 2980,  22204, 37344, 4944,  19200, 39184, 24936,
                61120, 16660, 46892, 53632, 60072, 35440, 39944, 7396,  19140,
                62412, 9252,  23484, 60976, 41500, 18680, 26316, 64488, 31716,
                4340,  40796, 34936, 59092, 43292, 52688, 6720,  24196, 36784,
                21060, 44528, 54544, 23324, 42656, 8076,  57864, 63940, 11912,
                35212, 60988, 20988, 33364, 65000, 53816, 41240, 24320, 50208,
                10860, 18644, 46816, 64756, 55796, 9276,  60224, 31796, 62204,
                47980, 5544,  51460, 42844, 2620,  60516, 22524, 38000, 6256,
                55340, 34472,
            },
            {
                47468, 39748, 2204,  56408, 64180, 44808, 8592,  57792, 32020,
                15852, 932,   33280, 45852, 57140, 51180, 41572, 59292, 30796,
                35296, 15352, 47504, 59728, 54888, 21564, 63252, 47892, 19212,
                53536, 496,   39104, 9544,  44316, 59924, 48004, 1292,  10820,
                16960, 46208, 38452, 27948, 65232, 55800, 19324, 45808, 12600,
                50292, 35688, 19484, 32228, 25468, 39684, 8692,  46168, 23520,
                32892, 10712, 28832, 61596, 49524, 13052, 27960, 58584, 52304,
                12044, 31940, 57056, 21272, 14540, 30296, 52480, 1532,  33228,
                47372, 15604, 2796,  30152, 58704, 35968, 47620, 39380, 23164,
                61348, 50064, 13960, 24960, 112,   33400, 16548, 62492, 29956,
                57852, 39648, 6860,  48916, 28552, 13944, 32464, 19368, 44220,
                27780, 56252, 2368,  48908, 17596, 5488,  57620, 31828, 7556,
                58924, 52760, 30064, 12668, 41880, 25692, 46448, 22352, 384,
                40948, 24984, 10128, 28196, 46524, 13300, 41124, 64088, 16284,
                30816, 19624,
            },
            {
                12216, 52580, 22120, 42440, 5204,  34316, 17848, 22556, 47092,
                65392, 53088, 23808, 16972, 8972,  4396,  27828, 19372, 44532,
                53592, 7856,  29432, 18472, 40676, 33788, 3456,  24248, 7080,
                61748, 35196, 21900, 56072, 18176, 27296, 37580, 56936, 25936,
                4672,  54788, 7724,  11808, 49900, 40272, 36256, 4324,  31340,
                22964, 43944, 14720, 47528, 5392,  55908, 30300, 65516, 15320,
                52104, 58708, 44232, 17144, 1392,  55316, 41948, 8824,  21708,
                5680,  43280, 840,   38088, 8532,  48164, 61340, 22816, 58604,
                43376, 26944, 38712, 49000, 6524,  13444, 616,   53804, 10520,
                6884,  17896, 56348, 47332, 65140, 11528, 45016, 49844, 3220,
                9512,  18700, 63504, 35020, 400,   53496, 36992, 48476, 4648,
                16380, 41716, 30112, 52716, 43876, 11524, 19948, 37888, 14904,
                43496, 34660, 21368, 6744,  16384, 38376, 54160, 11640, 29392,
                17528, 36984, 54564, 63232, 35160, 18352, 52952, 26812, 45628,
                57992, 3208,
            },
            {
                63600, 26188, 32340, 14780, 48060, 27352, 60728, 2884,  42752,
                6988,  35976, 58556, 43856, 61524, 36808, 13964, 55448, 10192,
                3008,  22304, 50528, 4784,  10660, 46164, 52396, 30412, 57096,
                11724, 28424, 49468, 6144,  63448, 31148, 12940, 51848, 42724,
                62160, 35284, 22592, 17968, 33012, 1644,  26524, 61720, 17160,
                55492, 880,   58920, 40828, 62696, 18016, 12000, 37784, 36,
                19328, 36608, 8348,  39920, 32408, 24272, 63240, 30764, 47992,
                36684, 62800, 28860, 51356, 42136, 26368, 17252, 37116, 13240,
                4080,  52276, 64636, 17644, 44300, 22424, 28744, 57380, 33884,
                45232, 37380, 29744, 5084,  21244, 38816, 27052, 35532, 14620,
                51892, 32164, 26436, 56364, 17284, 62248, 21696, 9208,  52152,
                63208, 23576, 13728, 8140,  25872, 35324, 47024, 63652, 23412,
                2872,  60908, 26648, 45496, 58216, 2772,  35044, 49712, 64356,
                44436, 13648, 20460, 6172,  56852, 868,   33172, 10748, 7564,
                24140, 37176,
            },
            {
                44216, 5776,  58972, 18908, 55520, 11308, 51148, 39396, 14488,
                26712, 10476, 19820, 3188,  31676, 24844, 48156, 39568, 33244,
                62776, 42456, 37336, 57488, 65032, 27000, 13864, 36472, 17520,
                40456, 45444, 15232, 52992, 41228, 3016,  23584, 8660,  19740,
                47484, 28852, 60548, 52080, 44416, 58408, 9960,  47808, 51648,
                38104, 10816, 28712, 3564,  22160, 49476, 28128, 43468, 60232,
                47352, 26052, 63604, 21204, 53020, 45368, 15300, 4064,  18036,
                53996, 25512, 13808, 19508, 65224, 5072,  10892, 45792, 56088,
                31040, 20540, 10396, 54616, 32420, 61908, 50916, 16048, 19972,
                60012, 26116, 12456, 41700, 53128, 8776,  60984, 56056, 21844,
                45964, 42220, 11796, 3972,  47248, 43692, 25172, 39864, 31076,
                1108,  36232, 59104, 39356, 62424, 54640, 352,   28212, 49308,
                54260, 5188,  40084, 51288, 62980, 31368, 19312, 7652,  57064,
                4224,  59792, 50072, 30288, 39912, 21944, 49060, 62040, 42612,
                51492, 15440,
            },
            {
                35012, 9568,  50180, 40756, 60,    36168, 24136, 31240, 62796,
                55868, 48336, 28980, 40864, 52684, 64124, 6588,  16688, 26928,
                59528, 12436, 24636, 32320, 16908, 5944,  42084, 60872, 48648,
                1836,  64308, 26016, 10836, 33952, 45892, 58800, 39300, 32148,
                15128, 272,   40920, 6076,  13764, 29944, 21472, 41504, 7136,
                25016, 64792, 32944, 53452, 36224, 7324,  54268, 24240, 6536,
                31580, 3376,  13924, 48944, 6048,  11076, 57344, 34372, 40832,
                60328, 9656,  46704, 3348,  34728, 57752, 53076, 28160, 6796,
                40776, 35452, 25364, 1928,  37752, 8032,  40820, 4120,  42820,
                49740, 1448,  62740, 33072, 48832, 18596, 31352, 6504,  1000,
                64264, 24348, 59076, 38656, 30032, 5608,  12524, 59732, 55844,
                46896, 19132, 50628, 4984,  21620, 31272, 9920,  17332, 39216,
                13448, 30800, 18364, 10280, 23516, 13996, 43624, 27608, 22000,
                39468, 33648, 26100, 8836,  44664, 14740, 59108, 3804,  31752,
                20800, 55148,
            },
            {
                27852, 61280, 23436, 29608, 63432, 45388, 7932,  53720, 18344,
                5008,  37968, 60876, 15272, 11504, 21300, 56476, 2432,  51720,
                19816, 47724, 16,    53208, 44648, 21260, 55776, 9128,  23356,
                32272, 5208,  37852, 21236, 61808, 17432, 53604, 5872,  64924,
                51044, 11108, 56020, 25104, 37040, 53932, 62812, 15636, 34796,
                57220, 19608, 42044, 15104, 45384, 10260, 58060, 15992, 41268,
                51304, 56656, 35708, 29708, 60676, 38116, 26692, 50048, 2132,
                23080, 32980, 56212, 40368, 29888, 22504, 42912, 18436, 63408,
                50572, 59776, 15036, 57008, 47016, 12368, 26968, 64952, 31116,
                9416,  22216, 54396, 15192, 3384,  59008, 25404, 40952, 54176,
                36924, 7332,  15188, 50680, 20700, 54624, 34248, 15960, 26912,
                7448,  11740, 33184, 45492, 15424, 42776, 60588, 56428, 34336,
                8464,  65152, 48296, 36396, 1100,  55636, 37864, 52288, 10840,
                47756, 1428,  17068, 61292, 52332, 24892, 37556, 28808, 12416,
                40384, 1736,
            },
            {
                17880, 46068, 12744, 38152, 4552,  20984, 58156, 12828, 43532,
                34020, 22100, 50772, 1936,  46680, 35300, 41900, 29876, 45028,
                8600,  34900, 14792, 29564, 61680, 38136, 2784,  28968, 51564,
                18636, 55208, 43880, 50288, 904,   29148, 13504, 36272, 26456,
                21640, 44992, 32800, 63704, 18476, 2372,  45856, 5196,  27972,
                12660, 47176, 6264,  23940, 60336, 31156, 20168, 34404, 61740,
                11852, 22284, 44500, 17268, 760,   43076, 20364, 64776, 12412,
                44440, 16576, 7088,  59764, 49628, 14788, 1488,  33688, 23948,
                9572,  4824,  43684, 29756, 18692, 51960, 23316, 36300, 14328,
                58168, 39708, 45612, 28576, 37112, 43984, 12120, 16840, 47624,
                29236, 19144, 33808, 62888, 9820,  41380, 61660, 3344,  44136,
                38112, 65480, 24340, 57924, 27480, 3464,  49956, 20292, 41652,
                25316, 57396, 43000, 28316, 59744, 46268, 5036,  62252, 15576,
                30888, 42956, 55352, 35252, 11364, 4664,  47408, 18736, 57404,
                65008, 48792,
            },
            {
                33048, 57080, 7452,  53320, 33428, 15764, 28276, 47276, 1236,
                25684, 9152,  57892, 32096, 26572, 7992,  58640, 23532, 37792,
                64584, 56084, 41152, 7036,  49492, 11900, 33992, 45132, 59044,
                12692, 35444, 59996, 9260,  24760, 57572, 48888, 43580, 2628,
                60136, 16368, 38872, 8384,  49028, 23140, 31308, 59904, 38704,
                50728, 1900,  61888, 52216, 38144, 1032,  47944, 5172,  38960,
                28692, 8968,  64192, 54656, 24072, 32692, 7956,  55092, 29612,
                37280, 52828, 27024, 20636, 10492, 36464, 60760, 47108, 13692,
                38676, 48820, 21376, 63892, 33976, 60076, 6376,  55892, 46728,
                2748,  25676, 19128, 6296,  64544, 22536, 50920, 34676, 58280,
                10376, 52484, 45212, 1352,  27424, 47812, 23536, 51500, 20036,
                29344, 53984, 2420,  40304, 52492, 12716, 32752, 6748,  46548,
                1608,  15004, 21084, 6496,  32524, 17428, 25640, 34484, 57704,
                23908, 65508, 5756,  19772, 41448, 63508, 54096, 34116, 8596,
                25400, 5492,
            },
            {
                14448, 42740, 22516, 60664, 49656, 41420, 64156, 35640, 51840,
                61756, 40192, 16916, 44320, 63540, 18808, 49652, 12084, 15760,
                4492,  27244, 22744, 59740, 18900, 25948, 63196, 15632, 39376,
                27016, 6576,  30116, 15508, 40980, 33496, 19580, 8224,  54992,
                29704, 48152, 4304,  27572, 59000, 42952, 10944, 53264, 20376,
                44052, 31868, 17624, 27308, 11668, 43064, 64564, 25420, 55136,
                18684, 50184, 6304,  40788, 14264, 59504, 46020, 17892, 5700,
                61380, 48220, 368,   39268, 63996, 44824, 4280,  53812, 30844,
                56564, 26272, 128,   41248, 10740, 3180,  44048, 16956, 32636,
                52468, 11048, 59596, 33604, 55428, 9196,  2136,  62024, 26828,
                3676,  23088, 38124, 56492, 32208, 17416, 7788,  35568, 58376,
                14324, 48404, 9532,  18240, 62316, 37368, 22852, 63424, 28864,
                51976, 35492, 61300, 49576, 12656, 53968, 41532, 8868,  3036,
                49204, 12328, 27136, 50904, 31356, 22676, 312,   15556, 45424,
                52860, 38784,
            },
            {
                63004, 952,   27456, 18828, 9404,  2416,  24404, 10588, 19752,
                29840, 55108, 3988,  12484, 53900, 38820, 1272,  60284, 30948,
                53524, 50324, 10724, 36128, 43224, 4012,  53852, 22224, 1996,
                47828, 65528, 20436, 53924, 46912, 3664,  62960, 39440, 23796,
                10680, 35408, 56796, 13460, 52116, 33732, 16872, 3732,  25264,
                63108, 8996,  35556, 56556, 21524, 29668, 51092, 13536, 2880,
                45728, 32896, 25880, 36884, 49092, 10640, 52168, 35312, 24992,
                41672, 14396, 31420, 54584, 16904, 24548, 28564, 7376,  19632,
                62456, 35268, 16736, 54028, 27900, 38444, 50196, 20824, 28432,
                63720, 37900, 49108, 13644, 40704, 47184, 31764, 15288, 39752,
                43716, 65196, 14128, 6288,  60240, 40008, 12768, 64032, 124,
                42240, 31928, 22060, 34968, 5372,  47616, 56856, 10624, 17636,
                55060, 4964,  24212, 9676,  39152, 63740, 21508, 51464, 18548,
                36196, 40064, 46196, 58884, 8320,  38252, 27956, 42392, 60260,
                21444, 30392,
            },
            {
                51048, 55536, 37156, 47844, 31748, 59344, 38612, 56248, 5596,
                14236, 48032, 36616, 24208, 28956, 6252,  34108, 46532, 42916,
                17780, 33296, 1676,  56920, 47212, 31284, 8800,  50452, 58080,
                10908, 42424, 36476, 5028,  61224, 27520, 12788, 50212, 32416,
                64256, 18604, 41952, 21240, 1128,  40024, 65532, 36712, 55784,
                13412, 40684, 48840, 3968,  59472, 8156,  17348, 36512, 57728,
                42280, 62496, 3868,  19940, 30680, 1744,  27644, 62704, 3144,
                21988, 10972, 58732, 34784, 8520,  50516, 58036, 41740, 11316,
                51000, 6200,  45800, 58840, 14160, 24360, 62120, 7688,  42376,
                4720,  23548, 452,   29596, 17832, 26056, 53236, 21228, 7784,
                54388, 30516, 49664, 21688, 26064, 54064, 45516, 49816, 24772,
                6716,  56560, 61104, 44024, 29212, 16000, 25780, 43124, 38320,
                31488, 40936, 45576, 58004, 27720, 200,   31228, 44868, 60236,
                29228, 22072, 1648,  14268, 17852, 62016, 55932, 11992, 35868,
                3964,  10348,
            },
            {
                24700, 16136, 6220,  43524, 13068, 51212, 17148, 45828, 42176,
                32432, 64904, 21212, 42764, 50476, 62104, 20424, 25624, 8864,
                62612, 21468, 39728, 13424, 64040, 16572, 40928, 34512, 25452,
                17712, 31892, 51428, 23640, 9944,  37844, 17112, 43236, 656,
                51792, 5336,  46420, 62136, 24664, 8924,  28220, 47592, 6632,
                30268, 22892, 54612, 15136, 38668, 46648, 61296, 31560, 21880,
                10292, 15964, 51256, 60120, 56600, 42172, 15704, 38340, 55472,
                44556, 51088, 26260, 3860,  43144, 21100, 33320, 37676, 15788,
                43916, 23364, 31448, 8588,  48128, 33492, 56716, 12480, 36708,
                15672, 54068, 43488, 61712, 56800, 5216,  45012, 37028, 59568,
                18668, 11100, 35396, 43072, 1880,  9980,  29536, 33480, 17932,
                39028, 26980, 11392, 54824, 980,   51380, 8240,  59704, 2436,
                13196, 64132, 19588, 16120, 35144, 48400, 13972, 4568,  55028,
                10140, 62912, 52908, 33376, 44484, 49172, 6664,  23912, 47328,
                19240, 40968,
            },
            {
                45692, 60900, 35228, 65368, 25596, 29412, 3768,  22928, 60724,
                26972, 8424,  652,   59408, 10080, 14964, 56300, 40824, 4212,
                58128, 48720, 27652, 5560,  23800, 28844, 60084, 4592,  44372,
                54800, 280,   13800, 45508, 55728, 31528, 59560, 22292, 57408,
                15140, 26152, 37172, 30828, 55212, 49544, 14548, 43492, 18996,
                61044, 276,   44956, 26216, 32820, 6416,  41548, 24476, 668,
                54020, 29020, 38860, 12192, 23148, 5276,  47420, 19568, 32852,
                7204,  65184, 18744, 47336, 61624, 13116, 1132,  64292, 27344,
                60184, 2444,  39860, 65376, 18004, 1160,  44552, 51424, 31048,
                59964, 47424, 20440, 8068,  35564, 11996, 64128, 1040,  48444,
                28528, 4264,  58224, 16496, 37744, 62996, 20760, 4044,  61568,
                51640, 15412, 46412, 20248, 40472, 64884, 35652, 46000, 22168,
                50344, 26352, 3372,  52340, 7852,  56404, 37976, 26004, 42068,
                16540, 6228,  37712, 57236, 25608, 3352,  32288, 51776, 64020,
                57768, 33292,
            },
            {
                28592, 11928, 2952,  20472, 56756, 8228,  53504, 37084, 11816,
                50044, 35024, 52948, 18236, 31296, 37360, 47972, 12396, 29696,
                35412, 15372, 44564, 51524, 36832, 55496, 11664, 20912, 37820,
                63060, 28252, 58436, 40496, 19272, 3080,  47480, 7864,  29052,
                40624, 59900, 7100,  11192, 17384, 60704, 3060,  32768, 57616,
                51576, 34636, 10844, 64824, 19256, 53136, 12300, 49264, 63732,
                34444, 47260, 7640,  44780, 35148, 64400, 53008, 11756, 59840,
                28736, 14876, 36328, 39764, 30016, 56004, 23996, 48424, 5164,
                54740, 19824, 51832, 28896, 37032, 22444, 5360,  26104, 19236,
                9496,  34148, 27280, 39824, 50232, 22888, 32444, 25268, 13836,
                40552, 63516, 32060, 51200, 47496, 55632, 14492, 44292, 57676,
                8756,  36868, 3492,  32728, 23700, 13816, 30864, 18052, 6964,
                55708, 33776, 43212, 30036, 61728, 23280, 18292, 64520, 33908,
                47224, 23848, 30048, 10780, 19916, 40572, 13680, 29300, 17036,
                1408,  7888,
            },
            {
                49128, 22656, 52936, 39068, 46608, 33136, 62148, 19308, 2720,
                58324, 15536, 39856, 45024, 26716, 5524,  53724, 22772, 65344,
                348,   54812, 10160, 61948, 19092, 2508,  45940, 49356, 15304,
                8612,  21904, 35120, 5584,  26700, 64592, 35940, 52628, 11904,
                33636, 48932, 20572, 45060, 34716, 52676, 38856, 21680, 27004,
                8200,  16860, 42304, 50556, 2464,  57428, 28320, 16232, 38296,
                5476,  20276, 61196, 26696, 16604, 31644, 8564,  24704, 40892,
                49528, 816,   57084, 10276, 6488,  52752, 17048, 40540, 31824,
                9124,  34840, 13572, 57828, 11132, 54412, 62948, 39144, 58240,
                3032,  64972, 14332, 2060,  57920, 16884, 42896, 52292, 56488,
                20876, 44800, 8652,  23904, 5816,  27648, 34700, 40976, 22868,
                30592, 48252, 63120, 58760, 9940,  53132, 48596, 27596, 62304,
                37512, 9296,  58488, 12320, 39760, 44668, 5604,  11032, 52092,
                2808,  59360, 50116, 43008, 64900, 35740, 55296, 9320,  43396,
                38652, 54376,
            },
            {
                59652, 41756, 31032, 16892, 5144,  14304, 40316, 43980, 30380,
                47156, 21968, 6680,  57004, 63872, 2052,  33444, 44060, 17116,
                38808, 24828, 31960, 7372,  42772, 33872, 64864, 30204, 5900,
                52192, 59940, 46956, 10560, 50552, 14808, 24752, 44256, 18696,
                3892,  53884, 63756, 1580,  42056, 24064, 5948,  12240, 46748,
                40120, 62240, 24332, 29416, 37628, 21172, 4472,  44644, 58764,
                25480, 14156, 56192, 51764, 1304,  43540, 57412, 37216, 4820,
                22260, 43884, 32372, 25444, 20200, 45020, 36732, 11408, 58332,
                46124, 25068, 49380, 3584,  41576, 46580, 29856, 16036, 48540,
                41408, 23820, 54896, 46368, 29456, 62576, 9864,  6876,  34356,
                2776,  26872, 54304, 13188, 19264, 58808, 2404,  11576, 53620,
                4804,  18660, 27928, 42940, 38456, 5940,  57280, 1948,  40732,
                15248, 24420, 47764, 20368, 1560,  50012, 32328, 57836, 27908,
                39444, 19028, 7776,  15488, 1084,  58820, 46484, 21644, 61744,
                25788, 14248,
            },
            {
                34908, 1892,  9564,  62668, 49744, 23292, 57500, 9756,  25744,
                63216, 33612, 12776, 24268, 49192, 19812, 59548, 8256,  51952,
                46928, 20152, 57720, 39832, 14716, 22884, 54000, 25956, 39576,
                32412, 42824, 16880, 29848, 60936, 42096, 2220,  55488, 61664,
                38836, 23352, 27764, 14296, 56120, 29504, 64540, 58356, 35704,
                1752,  53448, 13488, 7004,  47928, 60712, 35544, 54680, 8628,
                42360, 32936, 4296,  39512, 21584, 48288, 27784, 14180, 63584,
                54516, 16760, 59336, 48248, 63928, 3792,  27892, 61404, 21928,
                15516, 43028, 62476, 27636, 20676, 7912,  34584, 12212, 5904,
                52376, 31544, 10804, 36144, 4652,  19944, 37948, 46708, 61516,
                17716, 38708, 59980, 35840, 41928, 30984, 45748, 64844, 26072,
                35872, 12928, 55436, 1384,  17064, 25396, 44108, 20776, 32056,
                54360, 4532,  65320, 28392, 15868, 54716, 36920, 21404, 13684,
                45908, 61096, 34852, 53144, 23116, 27432, 5112,  49636, 32696,
                4192,  18424,
            },
            {
                46848, 27664, 58016, 43084, 35996, 28560, 340,   52248, 17108,
                5552,  54608, 43256, 38428, 9904,  30472, 41468, 13828, 28632,
                61388, 5704,  26952, 49632, 60852, 10904, 788,   17652, 57092,
                12392, 24052, 1768,  53948, 38468, 32868, 21656, 9744,  30352,
                16064, 7456,  47180, 32660, 9508,  19024, 43844, 16216, 49424,
                30360, 20788, 58628, 33144, 43324, 10360, 23756, 30368, 18448,
                65476, 49400, 28880, 10336, 62460, 17972, 3172,  50912, 30688,
                10924, 35264, 2516,  41156, 13868, 31264, 49840, 7436,  53476,
                2660,  37160, 6684,  17464, 59172, 51704, 24364, 57156, 37416,
                20308, 44840, 16444, 60160, 41868, 53304, 27576, 14580, 30788,
                50684, 11476, 47140, 1020,  62440, 51796, 8436,  16800, 49268,
                39996, 60864, 22324, 50204, 33132, 64288, 12632, 60208, 46652,
                10996, 51664, 42124, 35100, 59572, 25008, 9116,  62660, 516,
                31056, 25868, 11384, 44920, 31788, 38972, 17956, 12404, 52380,
                37388, 64668,
            },
            {
                55652, 13104, 24708, 19400, 54132, 12204, 64740, 32188, 48604,
                36636, 1908,  27984, 62312, 16436, 55812, 36008, 23456, 2844,
                37000, 12040, 53612, 3736,  29312, 46712, 36324, 41692, 62552,
                4136,  49196, 64076, 19920, 7304,  13304, 58916, 48432, 35464,
                50960, 57272, 40924, 62156, 37436, 51788, 2668,  25300, 10268,
                55248, 4548,  38924, 17992, 64052, 15056, 50244, 3124,  46636,
                12492, 22616, 53116, 36840, 58456, 33620, 42232, 60128, 20532,
                45968, 26668, 53184, 8232,  22860, 56156, 42484, 18232, 33964,
                64632, 30140, 55808, 39820, 32744, 2340,  43968, 64368, 28332,
                288,   62764, 26132, 7400,  22344, 48524, 896,   57680, 23448,
                42560, 6484,  22016, 28856, 15012, 24416, 38224, 20604, 59212,
                6044,  31388, 45992, 10552, 37008, 52240, 28612, 7172,  36284,
                22976, 17660, 30712, 8060,  3484,  47128, 39140, 17768, 55940,
                49176, 41320, 3976,  56496, 62176, 7444,  42708, 60612, 28788,
                9016,  21884,
            },
            {
                6360,  50568, 33708, 3748,  7876,  38572, 45668, 21372, 41556,
                60324, 19472, 50980, 7316,  45600, 4312,  52568, 62756, 49056,
                43416, 16472, 33240, 44704, 18948, 58480, 50668, 8812,  21396,
                34264, 44872, 36800, 25912, 56684, 46112, 27856, 5612,  65284,
                25136, 1180,  12700, 21792, 5080,  48244, 60556, 40412, 33728,
                63396, 27460, 45960, 52020, 936,   26844, 38488, 61628, 34036,
                40708, 168,   16100, 45340, 6284,  12964, 24116, 8920,  39892,
                5424,  62556, 18892, 38616, 60060, 35616, 560,   25904, 47664,
                12540, 22900, 45736, 10228, 61188, 13940, 49572, 17040, 9172,
                54088, 34456, 49732, 57032, 32040, 11224, 63488, 35316, 3708,
                54060, 65236, 33560, 56180, 49016, 4184,  57288, 33032, 300,
                43304, 15776, 7624,  62068, 19872, 2864,  42036, 16536, 56328,
                1212,  38540, 61208, 45156, 14224, 53336, 31736, 43092, 6108,
                23664, 64104, 16864, 20940, 50592, 14288, 34696, 54860, 160,
                40548, 44344,
            },
            {
                17192, 38948, 63532, 44028, 59020, 16564, 26724, 6212,  14364,
                10392, 30920, 56484, 22336, 34320, 26652, 11180, 18876, 30216,
                58328, 21860, 64060, 38524, 24008, 6568,  31672, 26588, 55180,
                13968, 29060, 9064,  16184, 51064, 388,   41072, 14660, 43352,
                19180, 34084, 45180, 58928, 26288, 31216, 13872, 20356, 6520,
                42608, 12584, 23044, 8488,  31564, 56236, 20652, 53896, 5820,
                27628, 57856, 63176, 25740, 30792, 54736, 47780, 36296, 52600,
                15240, 33200, 49156, 29284, 10768, 15984, 52052, 58688, 8704,
                41188, 51376, 1144,  26296, 36468, 21284, 30812, 40356, 23384,
                42384, 13472, 4100,  38532, 15460, 40300, 19072, 44700, 26596,
                13284, 39128, 9048,  18728, 36548, 11272, 44932, 28580, 52772,
                24108, 55112, 27048, 40452, 57696, 23672, 47964, 33980, 63744,
                49160, 25680, 55356, 20064, 27592, 63336, 22408, 12100, 52280,
                35692, 8872,  37824, 29800, 2124,  26772, 48104, 20224, 24888,
                58416, 30616,
            },
            {
                53080, 1576,  20768, 28944, 49412, 35104, 61304, 52696, 57324,
                25048, 46856, 38036, 13212, 42524, 57736, 47088, 1732,  40348,
                6924,  9764,  55132, 1400,  12652, 62272, 15844, 43868, 61064,
                5624,  52924, 59140, 39496, 33460, 62448, 23184, 37212, 54976,
                10496, 29828, 53004, 16484, 8544,  54784, 35956, 46408, 57036,
                17140, 49080, 36404, 59892, 41680, 13388, 44588, 9760,  17256,
                51216, 35520, 8452,  42928, 19600, 2868,  64752, 28384, 1284,
                57116, 23656, 42856, 2940,  65336, 45108, 20812, 38876, 28236,
                60288, 19576, 15244, 54488, 48224, 7072,  55668, 3368,  60992,
                47324, 19440, 29436, 62060, 24516, 55752, 8468,  51344, 59500,
                17228, 30000, 45684, 60640, 26796, 41332, 63932, 14420, 9708,
                62636, 35516, 48708, 30448, 5652,  14284, 59696, 8928,  29792,
                13432, 42812, 4268,  10868, 35888, 49832, 1816,  57788, 29124,
                15168, 58736, 46556, 54308, 40404, 65120, 10072, 45112, 15452,
                63128, 11516,
            },
            {
                26348, 32624, 57012, 13592, 9680,  22968, 1260,  39968, 33208,
                3824,  64468, 17656, 536,   60604, 31620, 15348, 65088, 24560,
                34596, 50724, 26708, 41272, 48044, 35384, 52260, 3056,  37576,
                18000, 47036, 21784, 3936,  11072, 30008, 17592, 60004, 2672,
                46996, 63168, 3832,  37104, 42520, 64804, 216,   23556, 29256,
                61864, 2588,  53380, 19048, 4328,  34768, 58692, 24644, 31176,
                47272, 21412, 13948, 56284, 49116, 39460, 16220, 21820, 44448,
                61220, 12756, 6968,  55220, 33880, 24968, 5720,  14040, 32528,
                4000,  35700, 63664, 29628, 42252, 62544, 11880, 37788, 32876,
                57968, 6180,  51996, 44156, 1848,  47728, 33912, 5200,  21712,
                42212, 49860, 5956,  1596,  52372, 21512, 6780,  50468, 39064,
                19152, 2100,  12340, 17924, 54024, 44952, 25996, 39412, 19316,
                6116,  51560, 33020, 58192, 41216, 7388,  18588, 38548, 44968,
                25224, 5056,  32984, 12364, 23284, 6580,  57424, 32128, 3132,
                36532, 48548,
            },
            {
                62052, 46108, 6828,  40608, 54720, 47204, 29812, 18928, 50316,
                43956, 8500,  54240, 25720, 49992, 5484,  21256, 37272, 53460,
                45396, 14636, 31244, 18208, 57248, 28216, 20628, 10732, 40900,
                24604, 31348, 64968, 42024, 56040, 49132, 7656,  52044, 26988,
                20408, 39544, 23964, 50388, 28232, 18456, 11800, 52420, 37756,
                9500,  32544, 25660, 65116, 29144, 47860, 6900,  63724, 42372,
                1916,  61428, 37496, 5312,  32348, 11324, 57776, 35088, 9480,
                31612, 51252, 27184, 17988, 40428, 48644, 53684, 62076, 46656,
                56200, 43672, 7820,  23812, 4320,  18248, 27640, 50868, 15708,
                25748, 10612, 36156, 20852, 12664, 27616, 64296, 30980, 37208,
                10628, 62836, 23892, 55276, 31924, 16592, 59520, 34212, 22712,
                47104, 42496, 56680, 64600, 37632, 32844, 688,   52984, 62444,
                45972, 22512, 16140, 64524, 24632, 46676, 61136, 33828, 3260,
                63512, 50888, 19008, 59708, 43772, 35944, 17096, 50760, 39732,
                6064,  22256,
            },
            {
                9340,  16356, 35896, 25088, 65340, 15740, 5052,  62488, 12092,
                22152, 35812, 29376, 39420, 10472, 41660, 55360, 28056, 12088,
                3104,  59324, 61980, 5148,  8484,  44412, 64352, 33548, 59496,
                50076, 1884,  13004, 27196, 19304, 44900, 35644, 13136, 32556,
                58552, 8908,  14076, 60868, 6012,  32908, 58256, 48676, 4760,
                43668, 15468, 39876, 11260, 50824, 21588, 15008, 38868, 55152,
                11492, 28640, 52552, 23680, 60124, 27368, 50924, 4092,  41296,
                19748, 46876, 37220, 62724, 8888,  12464, 30644, 1844,  22752,
                16848, 11312, 50472, 33224, 46348, 39504, 59492, 22128, 1008,
                46008, 65416, 31300, 54532, 60132, 41628, 16352, 53396, 724,
                56848, 14880, 34788, 39948, 11832, 47948, 2548,  29232, 57648,
                5092,  32004, 25256, 10156, 4016,  50256, 21740, 11420, 35060,
                57376, 28404, 38220, 964,   31124, 14384, 9488,  53856, 21592,
                40764, 10028, 27312, 776,   52996, 29476, 21276, 61184, 27728,
                56372, 42660,
            },
            {
                19500, 51956, 60292, 1988,  43152, 32008, 53248, 36956, 26808,
                59888, 15120, 46240, 63156, 18324, 32924, 61264, 7568,  49232,
                20008, 42704, 23016, 36824, 54076, 25024, 416,   14232, 45772,
                8104,  57368, 36436, 52656, 6396,  24980, 62956, 40512, 4796,
                43892, 54008, 34572, 47556, 22236, 45116, 40180, 26580, 20664,
                60212, 54524, 46248, 700,   57796, 37092, 27532, 3884,  19524,
                33436, 16304, 45360, 41384, 716,   18504, 46100, 63864, 25168,
                54476, 15484, 264,   56612, 22140, 58608, 36508, 42676, 26804,
                52584, 38148, 20832, 57684, 13668, 52940, 9408,  35008, 42964,
                56464, 17948, 7740,  38188, 3508,  23040, 8936,  45916, 25372,
                48068, 28820, 19768, 44272, 64464, 25704, 38024, 43628, 9204,
                60972, 15572, 51728, 20056, 28920, 42944, 60440, 17100, 41344,
                3068,  9228,  54200, 44116, 20264, 56380, 27692, 49008, 16784,
                31660, 47356, 61644, 38844, 14628, 48420, 3656,  10888, 53560,
                13936, 34564,
            },
            {
                58028, 4108,  28264, 38272, 20384, 10696, 45096, 7348,  41164,
                55564, 2308,  6424,  52500, 23928, 3440,  44328, 16308, 57884,
                38916, 33056, 10792, 49724, 16200, 39700, 30836, 55384, 22832,
                28992, 16912, 43636, 32168, 60492, 1628,  16280, 22188, 48552,
                30196, 17328, 956,   56228, 10004, 15620, 2116,  63172, 13452,
                34868, 24004, 18144, 31504, 7804,  62124, 44804, 51432, 60488,
                48240, 6660,  65028, 9604,  55192, 36724, 14352, 7768,  29884,
                59120, 5520,  32680, 43560, 28440, 4276,  18716, 64188, 6792,
                60024, 30868, 360,   64948, 25932, 2916,  30252, 63224, 5268,
                28040, 40212, 48088, 14588, 50192, 34976, 58184, 19348, 38956,
                61480, 3048,  51712, 8712,  4432,  56272, 18740, 13076, 53012,
                27224, 36068, 40740, 62088, 48332, 8220,  56108, 31084, 26616,
                47804, 13316, 61960, 50068, 5288,  36388, 60112, 42604, 6616,
                13232, 55164, 22572, 34428, 8036,  63440, 40904, 46328, 24384,
                588,   30856,
            },
            {
                44208, 22576, 50148, 13208, 55292, 59100, 24068, 17908, 48388,
                30732, 20864, 34032, 58008, 12516, 48652, 36448, 30404, 24900,
                1524,  64572, 28964, 3916,  46308, 62884, 19112, 50956, 4284,
                39092, 63492, 21248, 9880,  38520, 46244, 57172, 53212, 11440,
                64652, 25476, 38936, 28884, 62080, 36848, 54848, 30740, 8216,
                50336, 5672,  56232, 42020, 13096, 25004, 32484, 9812,  22920,
                40196, 26636, 35192, 21124, 31016, 61976, 22828, 40060, 49916,
                11264, 38492, 24356, 52640, 13804, 46128, 50360, 39536, 15028,
                9984,  47580, 41568, 17460, 36252, 44584, 48700, 15180, 54056,
                12048, 21836, 59540, 25188, 29864, 62752, 11924, 42868, 6880,
                32588, 13184, 59016, 23160, 31096, 35756, 62008, 49416, 21332,
                1924,  45900, 6584,  13480, 34628, 24724, 14680, 4668,  65056,
                37276, 18932, 33284, 25856, 40448, 11704, 2832,  23096, 65496,
                37520, 2336,  43316, 17836, 57888, 25320, 32496, 18924, 37340,
                64820, 48080,
            },
            {
                39428, 10024, 63000, 30660, 4504,  47392, 34644, 204,   61324,
                10240, 50368, 38184, 42816, 27656, 20428, 63628, 9660,  53272,
                47316, 17532, 52176, 56584, 12948, 34128, 9612,  60404, 35156,
                11980, 54344, 2732,  49888, 14340, 29724, 26108, 7980,  36740,
                3516,  59784, 20096, 43204, 51384, 24464, 18804, 42748, 46960,
                27284, 64116, 36308, 20908, 48912, 53656, 17044, 1204,  56068,
                13408, 58712, 2632,  48780, 12024, 53168, 44288, 1784,  34124,
                61512, 17744, 48100, 63308, 35500, 9576,  31980, 23588, 57208,
                34876, 28120, 5100,  55516, 10936, 58960, 24044, 19992, 37508,
                32084, 51900, 6544,  116,   44536, 52792, 4028,  28240, 54604,
                17604, 36428, 41536, 46788, 54184, 15336, 6932,  41000, 33036,
                65144, 54776, 23460, 58420, 1460,  39480, 53488, 44616, 51152,
                23564, 58796, 7268,  15524, 53224, 47000, 29360, 33532, 50992,
                25980, 58992, 30124, 4660,  49536, 12968, 55580, 4996,  59444,
                15752, 7084,
            },
            {
                26144, 17864, 34224, 41852, 16176, 26932, 64232, 39156, 14172,
                56816, 25516, 16612, 8476,  59632, 892,   54952, 39744, 13984,
                6176,  27064, 35612, 21856, 43172, 26036, 5884,  41316, 48496,
                26908, 45168, 24396, 59144, 34380, 63976, 18664, 42216, 32976,
                50540, 45620, 7620,  12628, 5400,  32732, 3304,  58520, 38264,
                11828, 3888,  29444, 60368, 2756,  38992, 64552, 43452, 36628,
                29652, 51912, 17812, 42884, 25940, 5068,  17076, 28476, 55640,
                21476, 42052, 7960,  2216,  20640, 59864, 54908, 1536,  44796,
                19136, 62780, 49700, 21868, 32392, 7128,  40872, 60952, 3360,
                45604, 62280, 34284, 41044, 16072, 20092, 33120, 48364, 23528,
                65192, 50020, 10288, 20252, 27924, 860,   45248, 24848, 10720,
                16772, 38300, 30564, 18776, 46820, 63188, 32792, 20740, 10568,
                184,   43228, 29740, 56580, 21344, 62516, 17568, 57472, 8748,
                19628, 45580, 11120, 52864, 37052, 27800, 44672, 9796,  42048,
                29096, 52884,
            },
            {
                61204, 48696, 6420,  57812, 51904, 9292,  20992, 53512, 28912,
                43620, 2876,  64748, 31760, 46512, 15696, 34168, 22840, 45136,
                60892, 41452, 8404,  61916, 2456,  57944, 30076, 20528, 15384,
                57644, 7312,  18116, 41104, 4788,  51480, 44,    56288, 13372,
                23396, 16468, 57964, 35224, 47848, 65456, 14684, 53576, 22560,
                17376, 52360, 44480, 15528, 10408, 34040, 28060, 7500,  20060,
                46012, 4240,  63428, 38780, 56480, 32156, 60632, 47076, 6128,
                14016, 27420, 54104, 30356, 39884, 16796, 27040, 5892,  52040,
                12724, 25084, 37848, 43640, 52356, 13832, 29016, 56048, 8960,
                25864, 18792, 11248, 56240, 64092, 38028, 9352,  57124, 14640,
                1184,  26432, 5456,  63116, 39044, 60372, 52096, 29516, 59088,
                4400,  43836, 8940,  52196, 11552, 27944, 5568,  16768, 60764,
                34532, 49776, 38672, 3760,  9640,  35256, 43740, 884,   38932,
                14600, 61128, 33772, 22172, 64500, 1016,  19912, 51176, 23248,
                35716, 12260,
            },
            {
                20704, 1252,  37924, 23196, 44368, 3164,  32468, 46356, 4988,
                35504, 22176, 53052, 40224, 5600,  51276, 28256, 4256,  57516,
                31384, 20156, 49280, 14832, 38172, 45660, 53392, 62504, 804,
                32740, 37332, 28724, 48120, 12400, 21528, 44088, 27600, 62384,
                39712, 30652, 52776, 26748, 21504, 40436, 29240, 9608,  34192,
                61896, 39720, 25324, 56860, 47740, 23276, 57876, 49984, 61520,
                10512, 24884, 33752, 14624, 8768,  50260, 36988, 10532, 65268,
                35380, 45004, 58832, 11940, 49552, 64648, 42780, 36888, 31376,
                59168, 8632,  3836,  16080, 63752, 1052,  34368, 50312, 16552,
                41936, 48620, 28872, 7352,  22672, 26824, 46232, 59992, 40440,
                30600, 45040, 56676, 32424, 17260, 8396,  35436, 12752, 55748,
                48896, 26732, 60920, 35916, 22252, 57168, 42460, 38056, 54696,
                26992, 12712, 64228, 45952, 24684, 51580, 13540, 27300, 48752,
                54552, 41304, 6384,  16476, 47244, 39880, 58492, 33232, 63316,
                3112,  56972,
            },
            {
                32296, 55444, 29508, 63784, 13456, 36580, 60028, 17360, 11960,
                62084, 48072, 18544, 13168, 24832, 61928, 42364, 18192, 11624,
                37540, 148,   55304, 24780, 32312, 17616, 10772, 23776, 43264,
                51844, 65264, 9992,  55540, 61344, 31640, 37516, 9384,  48292,
                6476,  2256,  63552, 9900,  55724, 1176,  60088, 44912, 6552,
                48480, 552,   30996, 7284,  41588, 18100, 4892,  13892, 31372,
                54692, 41472, 21036, 59700, 1312,  23768, 19896, 40656, 25352,
                51752, 4152,  22460, 33124, 6788,  24540, 14336, 48436, 21328,
                40320, 54220, 30316, 59920, 26088, 47072, 19660, 38732, 64988,
                1724,  58316, 36384, 53740, 50096, 13352, 2576,  34604, 7808,
                21268, 37412, 51168, 13664, 24096, 48112, 41676, 21920, 18396,
                33720, 428,   40700, 15560, 49456, 2856,  30968, 8432,  46924,
                22432, 6028,  18404, 32104, 59984, 40236, 5504,  64048, 32000,
                22984, 3428,  26964, 56624, 9020,  29820, 13524, 6864,  16868,
                46628, 43140,
            },
            {
                14992, 40812, 10536, 18864, 27516, 48528, 55880, 24596, 40840,
                8144,  27412, 58784, 37236, 10044, 55712, 35800, 49768, 65452,
                46204, 29092, 10088, 51416, 64196, 7180,  48772, 35484, 13624,
                4908,  21992, 40200, 16260, 25724, 5860,  53544, 59364, 17204,
                20980, 44728, 36312, 15040, 42588, 18524, 37312, 51284, 24444,
                19868, 55184, 14188, 35540, 63296, 52672, 26620, 40004, 35668,
                16732, 6276,  46672, 52148, 30120, 43900, 58700, 15712, 2152,
                57332, 18356, 38432, 60904, 45836, 2472,  56456, 10096, 62244,
                572,   45164, 18788, 35364, 41708, 6832,  54748, 11412, 27936,
                23568, 14204, 44892, 3844,  31572, 61356, 42624, 19224, 53140,
                62452, 10884, 3796,  43160, 58312, 2024,  62328, 5536,  45788,
                63356, 23988, 54248, 7000,  64444, 44420, 24816, 62260, 14584,
                57764, 42000, 53964, 2276,  15824, 29204, 57120, 20328, 8244,
                36820, 62936, 50876, 43100, 35488, 24800, 48680, 54196, 38108,
                26380, 8792,
            },
            {
                23748, 52620, 46084, 60768, 2036,  39416, 5932,  30768, 50440,
                56644, 33008, 2348,  45256, 30364, 21356, 1896,  7680,  24180,
                16188, 60384, 34988, 41788, 3268,  28428, 39780, 55856, 26592,
                60016, 30500, 47168, 2896,  35356, 45540, 14072, 23740, 33928,
                51860, 57240, 25212, 31768, 50280, 5088,  27884, 11508, 32616,
                64416, 43296, 10116, 59620, 21012, 3020,  44948, 60420, 196,
                49316, 64644, 26028, 37448, 12748, 63280, 7192,  49004, 32760,
                29136, 43184, 8728,  15204, 53808, 35628, 29008, 17808, 34276,
                27176, 13564, 51104, 10468, 57024, 22084, 31668, 43788, 5096,
                59948, 33576, 20708, 40088, 17152, 6328,  55852, 29132, 15216,
                47716, 25204, 54836, 18796, 34096, 27192, 52664, 31152, 37664,
                8312,  13356, 30056, 20396, 33328, 12224, 18092, 50480, 1368,
                33920, 28192, 48052, 37476, 22908, 44612, 11188, 53760, 47300,
                16832, 30104, 11700, 19516, 60008, 2428,  62508, 21460, 30896,
                59488, 50252,
            },
            {
                4564,  36056, 7020,  33264, 54432, 22348, 10244, 65044, 14920,
                20208, 43016, 6668,  64080, 48608, 14884, 58460, 32756, 53388,
                39260, 5440,  13576, 22392, 58364, 19888, 61244, 2016,  44684,
                18624, 8168,  58348, 51592, 19968, 64300, 39200, 1728,  29484,
                40716, 11768, 3604,  47252, 58680, 22824, 61696, 54348, 15380,
                2664,  27084, 38004, 46792, 29320, 33392, 12220, 56076, 22448,
                29100, 9736,  57528, 3680,  18168, 34568, 27036, 54092, 45948,
                12616, 63964, 50372, 26780, 20344, 41552, 50040, 58284, 7228,
                47032, 65436, 23716, 38588, 2440,  15212, 62896, 51836, 37020,
                47768, 54948, 10064, 64372, 25796, 36108, 49216, 23804, 1492,
                41744, 36200, 29636, 64744, 8952,  39272, 11716, 16272, 56420,
                50548, 42192, 59412, 47312, 39308, 55832, 59768, 36164, 40712,
                20348, 10852, 59184, 7832,  62212, 50456, 2948,  33380, 42300,
                59460, 568,   45848, 55044, 15068, 40728, 5432,  44852, 11568,
                240,   65448,
            },
        },
        {
            {
                15476, 45124, 57228, 17488, 40268, 55732, 8792,  42168, 46192,
                61748, 11756, 5868,  59788, 17564, 47648, 23060, 1168,  50612,
                59516, 9128,  21416, 28240, 34892, 47016, 65216, 43444, 37340,
                29564, 23912, 47876, 31448, 11948, 18060, 58044, 21540, 9816,
                35552, 60376, 51424, 20376, 29684, 36876, 61640, 47804, 8268,
                16884, 228,   56124, 41160, 33924, 15732, 28988, 42140, 1624,
                14612, 40488, 9772,  56788, 46800, 53096, 6620,  41736, 10500,
                53524, 47636, 12780, 4240,  16304, 34436, 10672, 26936, 17964,
                57092, 9552,  47892, 36964, 56016, 17340, 3720,  48796, 63724,
                5928,  25824, 45628, 2524,  48640, 39560, 62364, 19060, 116,
                65032, 39936, 32012, 47940, 15100, 63984, 6844,  26720, 436,
                15500, 64568, 4672,  33836, 10520, 47072, 3412,  29356, 58632,
                21680, 5936,  17652, 64592, 46468, 7728,  56996, 24008, 44576,
                51608, 58492, 18996, 48976, 36844, 8088,  62432, 57080, 19620,
                41032, 60184,
            },
            {
                33016, 2328,  52788, 27036, 36212, 14188, 33188, 51112, 16020,
                53560, 27856, 34496, 39660, 13672, 52436, 61940, 28584, 6236,
                35940, 45700, 63108, 820,   58028, 10912, 31608, 5288,  60944,
                16080, 40884, 9540,  4044,  61148, 38084, 49288, 64676, 29448,
                44332, 13432, 38544, 57676, 3700,  14896, 5632,  34192, 52772,
                59548, 32384, 10828, 25776, 60036, 7204,  63208, 25048, 51876,
                34976, 65520, 21048, 33536, 12948, 18464, 50276, 36260, 60048,
                20884, 988,   32952, 56472, 29556, 59464, 2112,  42816, 38236,
                50832, 29404, 60988, 25708, 33560, 7548,  43412, 20792, 35892,
                16032, 55636, 37728, 19564, 30020, 5440,  12544, 36864, 51136,
                28472, 16836, 5700,  61136, 29060, 37696, 22960, 45280, 50692,
                38644, 24448, 57788, 42072, 21992, 37500, 15376, 41152, 9076,
                55564, 33160, 38148, 11164, 25716, 20264, 40352, 3940,  16240,
                7108,  41256, 28856, 60272, 2656,  22040, 42964, 12532, 28208,
                5100,  51636,
            },
            {
                38024, 12984, 64404, 5212,  49484, 61208, 20620, 25104, 38496,
                1020,  19468, 43968, 24140, 8952,  31188, 42216, 12156, 40220,
                16780, 31380, 13012, 52588, 39208, 23528, 54948, 20404, 50496,
                56,    56704, 52160, 34480, 42712, 28120, 6336,  16108, 40820,
                55216, 7264,  17944, 45864, 31660, 50360, 58580, 26648, 20840,
                44148, 28684, 50100, 3912,  37992, 48020, 19960, 11904, 55256,
                17780, 27920, 4804,  42956, 62264, 25832, 2584,  30260, 15952,
                44432, 26284, 63384, 40092, 45068, 23644, 48628, 14244, 62276,
                6048,  20824, 3972,  13688, 46148, 62372, 11960, 53564, 31024,
                1216,  41832, 62964, 23228, 60212, 54516, 25648, 58144, 4332,
                22444, 45424, 12136, 41616, 2892,  53188, 34488, 59940, 11304,
                31524, 8428,  49352, 1760,  54300, 61016, 25248, 34868, 48928,
                2212,  43608, 59336, 49916, 552,   61708, 32236, 48672, 37688,
                65196, 25316, 12968, 45952, 55516, 32184, 17052, 49596, 36044,
                63760, 24888,
            },
            {
                58152, 41796, 31068, 23108, 44076, 3124,  29976, 7084,  63620,
                10360, 58036, 49464, 64968, 55112, 2084,  19220, 56828, 64260,
                48820, 24956, 7244,  42884, 18536, 4352,  45148, 13724, 35016,
                27348, 44560, 18428, 22252, 13608, 53692, 24328, 2144,  50592,
                25756, 33552, 448,   63772, 24904, 41712, 9712,  37912, 2848,
                13528, 39884, 63444, 22336, 13856, 57064, 43652, 32580, 8108,
                45152, 49324, 59836, 29872, 8624,  55176, 40360, 64100, 7540,
                38036, 51712, 18764, 5548,  54772, 8740,  19800, 35840, 32356,
                55600, 44392, 39204, 53880, 19176, 28364, 37864, 23840, 46828,
                59324, 28056, 7648,  13832, 43256, 32732, 9416,  40704, 49152,
                61768, 35828, 55020, 24272, 48712, 18648, 13704, 4056,  20528,
                61840, 46444, 27808, 18052, 12236, 50744, 5788,  65220, 18316,
                30976, 23668, 15488, 28304, 35792, 52504, 12656, 59820, 9328,
                21108, 53372, 35052, 6080,  38672, 10792, 61264, 1204,  45504,
                9288,  20712,
            },
            {
                6124,  48552, 9744,  18984, 54572, 12012, 46556, 56592, 42784,
                35816, 29044, 14644, 6548,  36920, 26700, 46728, 34084, 3728,
                21988, 54536, 60764, 33436, 49284, 29888, 59136, 38224, 63532,
                10396, 32380, 58836, 2688,  63824, 46592, 36548, 59908, 11000,
                20904, 61756, 48236, 11676, 21664, 54860, 17324, 65344, 48452,
                57272, 18740, 8420,  35924, 51412, 30296, 2352,  39740, 58416,
                22712, 1160,  38924, 16356, 47908, 34536, 21240, 46160, 13128,
                56884, 31700, 9956,  15224, 37160, 28052, 64324, 52276, 2908,
                23104, 11016, 31468, 64976, 488,   50732, 57476, 4392,  9768,
                17792, 51324, 34700, 49400, 2812,  17256, 46636, 29380, 14908,
                8096,  19276, 59424, 32504, 9392,  58104, 43576, 29636, 55624,
                40632, 14768, 36084, 59244, 32532, 39844, 20952, 45528, 13192,
                51704, 62792, 9636,  56020, 6248,  42392, 18532, 26828, 44976,
                30932, 892,   17524, 63316, 23416, 52236, 26644, 40288, 30640,
                55368, 16536,
            },
            {
                43840, 25944, 61856, 35444, 39120, 59964, 33632, 16916, 52080,
                22112, 3272,  32304, 45676, 21420, 60932, 51724, 15700, 10044,
                38120, 27736, 2660,  14620, 64512, 9196,  26404, 2276,  48464,
                24204, 6932,  39400, 49932, 30412, 7772,  18960, 32300, 56536,
                42768, 28920, 53060, 39544, 35256, 6492,  45404, 30616, 24456,
                33484, 4816,  45576, 61084, 16516, 24228, 5908,  62088, 27156,
                36756, 13376, 53512, 24420, 3220,  11096, 58760, 28704, 23724,
                1704,  47996, 25280, 61444, 50072, 536,   42088, 12724, 26500,
                59332, 49428, 15936, 41592, 8844,  34844, 15400, 42384, 62112,
                38964, 22032, 11216, 58288, 37004, 63664, 20504, 53380, 33948,
                43868, 3740,  27504, 51232, 768,   39164, 65496, 25244, 7420,
                51612, 2544,  23144, 44452, 9036,  29032, 100,   57396, 27096,
                37200, 3820,  40444, 21984, 46196, 30388, 2692,  55296, 36412,
                57580, 49752, 41884, 29700, 48228, 4068,  19340, 59144, 13940,
                50152, 34648,
            },
            {
                28812, 752,   51804, 15644, 7496,  23872, 1504,  27136, 12960,
                48192, 62640, 53868, 17656, 39524, 11748, 4836,  43320, 30644,
                59272, 45872, 40724, 20808, 36688, 52344, 16456, 42220, 19612,
                54568, 61972, 15792, 26424, 11328, 54984, 40436, 14576, 4464,
                37196, 8732,  16248, 3924,  27876, 61044, 14284, 1612,  50804,
                11500, 53580, 41300, 26528, 55400, 11052, 47068, 52716, 17120,
                9216,  63476, 31668, 57432, 43752, 51100, 17460, 4972,  53000,
                42156, 58996, 34656, 40612, 21380, 30504, 57692, 17712, 45356,
                7208,  36852, 27860, 58520, 24972, 21208, 48536, 32196, 26584,
                55044, 5160,  31240, 44820, 27188, 6984,  56664, 904,   24844,
                64608, 37904, 11232, 46248, 15828, 22044, 35264, 47556, 17156,
                33024, 63636, 5292,  55052, 62376, 15196, 53320, 42128, 7592,
                60664, 47196, 33396, 17164, 58900, 63048, 24564, 15136, 5076,
                11628, 61124, 7944,  14700, 58056, 43972, 34212, 8224,  24264,
                3572,  62900,
            },
            {
                56948, 12808, 40208, 58484, 29416, 49608, 63088, 41068, 37412,
                4956,  25652, 42008, 8772,  58600, 29224, 24320, 62160, 19904,
                53100, 7672,  11596, 56156, 23708, 46688, 6708,  57848, 31228,
                12732, 45932, 35560, 43224, 21104, 61324, 45144, 26988, 65096,
                47200, 19768, 59260, 44140, 51160, 56660, 19252, 40860, 36856,
                59020, 28944, 20236, 716,   35184, 31256, 41708, 20564, 33712,
                50564, 45704, 6136,  19512, 36156, 64864, 39496, 32704, 62368,
                14788, 19708, 3368,  12076, 54096, 46772, 4888,  33716, 39828,
                62984, 19636, 4072,  51968, 44296, 5360,  59540, 13120, 1780,
                45852, 64656, 19076, 52552, 14624, 23180, 38676, 49956, 13220,
                57552, 21604, 40984, 62580, 31084, 56480, 4896,  12016, 54340,
                38268, 26996, 41468, 19160, 47256, 36636, 22256, 34096, 11512,
                19744, 25052, 54228, 13400, 7292,  37252, 51500, 47596, 40100,
                23284, 34032, 27392, 20500, 37304, 11812, 64348, 46600, 54496,
                37772, 21696,
            },
            {
                46064, 19212, 33136, 4328,  43520, 17904, 53736, 9720,  56912,
                19828, 59620, 34464, 464,   50016, 55592, 36576, 47512, 1488,
                34688, 16880, 49908, 29524, 940,   34160, 62776, 39256, 22720,
                3416,  52892, 6428,  58376, 1360,  34116, 5480,  49720, 22920,
                2032,  54060, 31888, 12092, 23364, 33764, 8652,  62888, 22052,
                5784,  15744, 63864, 49160, 7880,  57244, 65076, 14196, 4236,
                60300, 22168, 40896, 27540, 14008, 7688,  25188, 49456, 9516,
                36996, 29808, 44988, 65284, 8068,  15584, 24760, 60952, 9676,
                47900, 30916, 55376, 12552, 38220, 63152, 29336, 53752, 36160,
                15884, 24592, 40564, 2444,  60116, 42188, 10352, 31420, 45368,
                6104,  29264, 17784, 52736, 7676,  26364, 42600, 60864, 20868,
                8932,  49988, 13268, 31476, 25336, 6108,  50300, 64016, 30264,
                56600, 1332,  49708, 28632, 41676, 10640, 20352, 31980, 64392,
                18208, 43584, 52028, 4564,  55444, 31544, 104,   28352, 17672,
                41328, 10012,
            },
            {
                52944, 65420, 25112, 48136, 11320, 21532, 32752, 6392,  30696,
                15300, 46808, 11004, 22516, 31868, 16280, 5952,  13004, 41664,
                63560, 25596, 58180, 43812, 60588, 13768, 27100, 9640,  50096,
                33496, 28008, 18496, 38532, 25136, 52280, 16928, 30208, 10328,
                34752, 61412, 25884, 38928, 868,   48852, 43128, 26556, 54768,
                46860, 32848, 43672, 13280, 37324, 22964, 2004,  44200, 38320,
                29572, 11736, 660,   55196, 59604, 30740, 2520,  45888, 21808,
                56168, 6648,  51196, 26968, 32984, 38432, 55780, 51640, 22120,
                14776, 920,   23980, 33132, 17860, 9936,  40996, 19980, 7380,
                50324, 57416, 8832,  33844, 48232, 28468, 62796, 17064, 54036,
                35504, 49040, 1512,  59796, 13288, 37316, 50660, 1984,  29864,
                45072, 57404, 1272,  61408, 10464, 58332, 2996,  45076, 15468,
                39412, 44004, 5276,  64984, 34632, 45472, 636,   53700, 8744,
                56676, 1860,  46832, 62632, 39788, 22572, 14392, 52416, 61528,
                5412,  31216,
            },
            {
                2708,  35900, 8404,  55484, 61548, 36668, 46396, 64544, 43152,
                51296, 28096, 62248, 40372, 45012, 65280, 27300, 51288, 22188,
                31328, 3920,  37632, 19108, 5692,  41116, 54744, 17404, 44608,
                65416, 57096, 10208, 47484, 63672, 13336, 60060, 39272, 56820,
                42104, 14204, 46240, 7140,  64488, 16396, 30000, 12912, 3192,
                39100, 9344,  24968, 52268, 59316, 17696, 28408, 54332, 24812,
                56900, 47108, 34208, 49700, 42608, 18632, 53844, 12424, 41620,
                61196, 16896, 23432, 59780, 43452, 18616, 1712,  28860, 35492,
                42960, 50636, 64440, 46284, 56792, 2532,  47604, 26064, 61180,
                43020, 29580, 37952, 12512, 55456, 4716,  20212, 2156,  58772,
                25424, 10072, 43620, 33164, 23128, 46460, 18768, 34092, 64360,
                24220, 16540, 35580, 52864, 42408, 38396, 18516, 26632, 8712,
                60008, 32076, 18612, 22636, 56184, 26012, 61560, 28296, 13284,
                37900, 30604, 15580, 25572, 9472,  48076, 42892, 34584, 24636,
                49504, 15528,
            },
            {
                59552, 22932, 42024, 28084, 3012,  14824, 26180, 916,   23316,
                38004, 3848,  54208, 14072, 2484,  19216, 38172, 57632, 9164,
                53944, 14876, 48196, 10560, 32408, 51336, 21200, 4900,  36356,
                604,   14940, 41500, 22548, 32372, 8416,  44356, 20096, 51340,
                6012,  18404, 29012, 52084, 21428, 57520, 35928, 52868, 60152,
                18444, 62340, 29912, 2864,  40576, 47316, 11372, 49924, 8628,
                18248, 63744, 15228, 23064, 9716,  37240, 63236, 33828, 27700,
                92,    47796, 36108, 4132,  10504, 48992, 62712, 12836, 5916,
                59828, 39488, 7708,  28200, 36664, 22832, 52912, 34460, 11568,
                128,   22236, 18436, 64008, 25624, 44488, 33264, 37676, 41512,
                14848, 64220, 19320, 39020, 55100, 4176,  58316, 14264, 40844,
                6316,  48564, 28016, 21688, 14500, 32852, 55436, 48024, 23332,
                52352, 12128, 37804, 51372, 14812, 4436,  17556, 42412, 50188,
                24200, 60516, 5836,  35352, 57544, 18140, 6756,  58904, 11472,
                37172, 44764,
            },
            {
                29308, 13148, 17360, 50460, 39012, 60124, 52220, 11860, 58832,
                18292, 8104,  35588, 25592, 60400, 48300, 7048,  33172, 44352,
                60024, 39608, 28244, 64724, 23828, 46020, 61488, 30716, 59168,
                25536, 48684, 29600, 55896, 4148,  36968, 28132, 296,   25320,
                63900, 37404, 56136, 9948,  40484, 4052,  45112, 8000,  23864,
                49408, 34912, 55072, 21068, 6204,  33028, 63116, 36208, 5056,
                31784, 39808, 2952,  61040, 26580, 46552, 5232,  15720, 54992,
                39304, 31264, 13736, 57804, 40964, 21776, 31852, 44872, 56956,
                26324, 20536, 16720, 11356, 59072, 5672,  14736, 65404, 31776,
                58292, 45572, 54000, 6020,  15808, 51780, 61808, 22820, 7416,
                50396, 27768, 3244,  61300, 30112, 8236,  27164, 51824, 10300,
                37084, 60328, 4120,  44180, 64764, 29132, 7068,  62124, 35128,
                2124,  28068, 61728, 6828,  40692, 48212, 57984, 35476, 7308,
                45276, 19448, 53440, 41844, 28004, 50380, 64824, 3288,  27344,
                20688, 56224,
            },
            {
                1328,  63260, 45936, 31496, 7476,  19332, 34580, 40940, 30332,
                47508, 56652, 20964, 50536, 29928, 41872, 12248, 24692, 17580,
                556,   20996, 6772,  56204, 35616, 1616,  15392, 39156, 11712,
                43236, 20240, 6152,  62636, 50468, 16344, 46524, 59044, 12280,
                31736, 3236,  48188, 34204, 25036, 59512, 19008, 32428, 41868,
                132,   11828, 16068, 44712, 61056, 26252, 14548, 42704, 58920,
                20852, 44544, 53140, 7312,  30364, 51080, 20948, 58972, 24568,
                8540,  64120, 19632, 51868, 26612, 54560, 7436,  36704, 15412,
                3432,  33884, 54260, 41928, 31152, 49656, 44640, 39500, 17468,
                4368,  48896, 34748, 39912, 30304, 9248,  47692, 12328, 56328,
                31740, 45508, 53208, 11912, 48000, 16864, 62940, 43388, 22580,
                31136, 53660, 18992, 9104,  50556, 872,   12756, 40320, 17236,
                43068, 56504, 45980, 9880,  33616, 30124, 22060, 3208,  59448,
                32496, 10488, 63632, 2292,  12940, 21484, 30892, 38872, 53544,
                7748,  40392,
            },
            {
                34148, 51956, 4440,  57648, 22508, 44120, 55036, 4212,  16208,
                63940, 43376, 32604, 10140, 4380,  55116, 36196, 52668, 63204,
                30408, 50256, 42536, 13260, 52428, 8812,  28080, 49268, 56296,
                8056,  53416, 34824, 12812, 40876, 21452, 55288, 34024, 49300,
                22844, 43684, 15928, 62428, 12416, 28444, 50892, 14300, 65204,
                27080, 47176, 58120, 36532, 8968,  50544, 22652, 3388,  52328,
                27776, 11120, 34884, 56240, 17004, 40780, 1548,  44448, 11604,
                49748, 42976, 6260,  34556, 2260,  16768, 64792, 24128, 52636,
                48572, 62524, 45712, 1408,  61028, 19580, 25532, 8616,  55380,
                27924, 13628, 23716, 59304, 1368,  42728, 27400, 18036, 60140,
                5228,  35308, 21116, 41224, 24404, 37448, 33424, 55924, 236,
                46068, 12524, 56872, 33700, 23924, 46504, 53892, 21084, 59156,
                4224,  14532, 20472, 25632, 63044, 53984, 12004, 38716, 15876,
                52320, 23132, 47624, 38176, 33528, 55132, 44260, 16124, 46740,
                61988, 18520,
            },
            {
                9780,  25656, 36496, 12100, 64720, 27196, 9860,  49448, 24676,
                6332,  13300, 39180, 62616, 17088, 58888, 21576, 2984,  46004,
                10724, 34888, 61172, 25332, 18736, 44964, 63068, 23036, 17956,
                32744, 60316, 38800, 1876,  26056, 60896, 3660,  9428,  39796,
                61240, 7440,  20608, 53400, 1096,  46780, 37092, 5008,  55728,
                38912, 22372, 5268,  29536, 19152, 39636, 57144, 30744, 16292,
                64816, 48432, 24124, 38392, 12680, 64384, 32928, 53220, 36656,
                28700, 22248, 61888, 45812, 30048, 39988, 47076, 11264, 38624,
                29608, 9536,  22432, 27296, 13104, 37844, 2988,  51344, 21344,
                41744, 63700, 10292, 52768, 20328, 36728, 55104, 2956,  38956,
                24852, 15372, 9176,  58228, 2280,  50268, 5864,  20272, 15632,
                63832, 26040, 37776, 41196, 15972, 62832, 36320, 26152, 32136,
                48632, 29648, 36956, 50060, 1672,  18308, 44584, 64704, 27240,
                41188, 29820, 4396,  17452, 58524, 9184,  900,   24160, 12472,
                31932, 49108,
            },
            {
                21900, 55472, 41288, 16664, 47436, 1212,  32908, 61304, 36796,
                28620, 53376, 208,   24392, 44216, 28740, 14108, 40296, 26816,
                58224, 16060, 4208,  32152, 40732, 58464, 37224, 6312,  2768,
                46920, 24104, 15552, 51592, 31144, 43100, 18688, 29228, 14820,
                52188, 26848, 38052, 30508, 41532, 9004,  61068, 20756, 30848,
                10188, 51452, 42648, 64068, 53260, 1724,  11944, 46900, 37156,
                8156,  352,   19096, 58136, 3904,  48704, 26116, 7364,  18288,
                59676, 3996,  15680, 55532, 8960,  58568, 20848, 572,   60324,
                18708, 6320,  40480, 50872, 56580, 33752, 62824, 30188, 58844,
                35452, 7080,  32332, 45064, 61228, 14508, 33684, 63224, 43792,
                48360, 54152, 65260, 30656, 45032, 13152, 61740, 40336, 29392,
                49436, 7720,  2540,  60600, 30152, 3736,  43768, 7368,  10412,
                65452, 54616, 5780,  58816, 42228, 24432, 8800,  49584, 496,
                56800, 13568, 60884, 43404, 25992, 48728, 63232, 35416, 57048,
                5460,  42796,
            },
            {
                14800, 62740, 6172,  29464, 53052, 38364, 13896, 56120, 21188,
                45884, 60520, 18668, 35320, 47460, 8348,  64796, 49192, 6588,
                37812, 23100, 47428, 53632, 2152,  11832, 30436, 51104, 42468,
                64388, 28912, 9352,  46084, 63240, 7036,  54168, 36388, 58212,
                45196, 2008,  64844, 57076, 17604, 23896, 53980, 44164, 15508,
                59248, 2528,  33504, 13592, 27528, 44960, 34180, 61900, 25040,
                41216, 60164, 32064, 45784, 29280, 22152, 60572, 14140, 56648,
                41128, 31652, 37608, 24824, 13224, 49628, 27260, 32820, 43980,
                54896, 35904, 63716, 16320, 4764,  47308, 10660, 14952, 44064,
                848,   48384, 16992, 25924, 5028,  49252, 23468, 8032,  19764,
                28192, 520,   36644, 17848, 26380, 54732, 22896, 9724,  35524,
                59028, 42732, 21460, 51492, 10992, 19432, 57948, 51012, 38760,
                23384, 41248, 12376, 34184, 15304, 60916, 31344, 36020, 19664,
                7428,  34376, 50960, 20812, 6176,  29216, 40052, 19668, 51664,
                26956, 60440,
            },
            {
                256,   45732, 33452, 19864, 58604, 24000, 43468, 17640, 2724,
                41072, 11092, 49848, 57364, 4992,  31376, 52340, 33788, 18388,
                55344, 43296, 8164,  64604, 27192, 21192, 16288, 34272, 54612,
                13080, 19700, 59064, 35592, 22304, 11896, 48440, 24276, 4936,
                19704, 10680, 34864, 13536, 49668, 32872, 3396,  26124, 35976,
                47920, 18392, 24912, 60548, 7640,  17436, 54460, 4296,  20312,
                51212, 13468, 54048, 6484,  40020, 9700,  43204, 35064, 1940,
                47080, 10512, 53044, 43484, 61108, 35692, 5024,  51808, 14836,
                24984, 12196, 31772, 43192, 24084, 18920, 39652, 54472, 24960,
                19824, 37388, 57624, 29760, 39436, 10948, 52372, 31056, 57336,
                12816, 42240, 51300, 6812,  60532, 32864, 43264, 52516, 4860,
                18104, 32652, 14012, 55180, 27524, 47852, 34156, 14632, 18000,
                268,   45844, 19720, 27496, 56028, 3516,  46496, 53404, 25816,
                62500, 46104, 10216, 37204, 53148, 14704, 60020, 10776, 4008,
                37884, 30432,
            },
            {
                53948, 39424, 11276, 49860, 3520,  8328,  62856, 31124, 52172,
                7620,  33224, 27356, 22212, 40584, 16368, 10804, 24964, 856,
                61600, 29892, 14592, 49896, 38792, 56784, 61876, 8884,  25228,
                39576, 4852,  41932, 1292,  56552, 16996, 40252, 64112, 33120,
                50576, 42052, 22768, 45804, 6956,  62228, 39684, 12020, 63796,
                6476,  56628, 38336, 49648, 41476, 22904, 58048, 29812, 9680,
                43956, 27124, 35564, 16192, 63464, 55264, 19004, 51248, 27272,
                23032, 65440, 2916,  20436, 6896,  17180, 41688, 64380, 3560,
                57576, 48844, 1132,  58640, 52368, 29148, 7352,  60680, 3896,
                50260, 63040, 12388, 54072, 1916,  64848, 45696, 16808, 3936,
                62436, 33228, 21764, 11272, 38616, 2784,  15296, 27712, 57212,
                48172, 24932, 64820, 44832, 39540, 6072,  24136, 60136, 53144,
                28932, 61912, 8092,  51672, 38420, 22124, 10720, 39644, 14124,
                42184, 22524, 31076, 65392, 1544,  42368, 32988, 45336, 23304,
                48156, 17836,
            },
            {
                8636,  24356, 27828, 60320, 42020, 34980, 12436, 48692, 25628,
                65328, 15456, 59564, 1708,  53620, 62140, 36812, 56984, 41004,
                21508, 11552, 36120, 19080, 4940,  43036, 46144, 300,   49112,
                57932, 30184, 52832, 26332, 32116, 44604, 3232,  27576, 13804,
                59856, 30004, 55812, 2728,  27668, 58268, 19596, 51668, 29084,
                45488, 21456, 9752,  31248, 1108,  36188, 48132, 15368, 38776,
                62756, 2644,  21768, 49644, 25388, 33072, 4980,  62092, 39244,
                15944, 50388, 29368, 33812, 46228, 54220, 30420, 22816, 39228,
                45584, 28520, 21236, 9876,  36840, 65088, 45728, 34260, 27560,
                41236, 8064,  22940, 42532, 19304, 34120, 26652, 41084, 37312,
                54692, 24580, 46016, 58496, 49272, 20024, 64200, 45616, 984,
                38876, 10364, 6872,  35708, 1256,  62604, 31176, 42736, 4756,
                35824, 57716, 31560, 48084, 4868,  64460, 17200, 59132, 2160,
                55704, 4924,  16332, 56620, 24876, 18820, 7956,  54928, 63848,
                13512, 57912,
            },
            {
                35540, 65080, 52632, 15464, 21868, 46516, 28916, 56780, 4304,
                38468, 45212, 36048, 12476, 43872, 19772, 28312, 46120, 5380,
                51292, 44328, 59592, 25028, 32684, 10644, 28484, 22532, 36884,
                17080, 7732,  47532, 14056, 62352, 50320, 9856,  37360, 53496,
                7916,  17408, 38556, 48696, 15696, 35624, 8824,  42492, 508,
                34164, 53916, 14628, 61816, 51936, 11724, 65264, 5604,  32904,
                55808, 47432, 59340, 11440, 944,   44796, 13076, 30532, 8264,
                54944, 42448, 12072, 62536, 26084, 58804, 8228,  11080, 19228,
                33404, 7280,  61968, 41108, 14084, 2116,  22264, 16696, 11656,
                53136, 31936, 15784, 46672, 60052, 6972,  13476, 58828, 48660,
                9504,  5192,  15764, 28516, 41484, 8820,  36040, 23728, 13056,
                60252, 31400, 53048, 16104, 22320, 56568, 9072,  13296, 49076,
                25288, 10812, 16352, 42112, 24540, 35312, 30008, 49368, 27204,
                33556, 48152, 38972, 28552, 49588, 58980, 36472, 27688, 2304,
                31684, 43040,
            },
            {
                4748,  18676, 1944,  37576, 6696,  54584, 18552, 40424, 10068,
                19984, 23336, 55000, 30416, 8788,  50500, 3752,  14324, 63944,
                31104, 7400,  54556, 1640,  63668, 51404, 55840, 13352, 60336,
                33284, 65348, 20816, 38020, 5520,  18344, 23236, 61288, 46520,
                168,   25732, 63164, 10096, 43604, 54484, 23204, 60908, 16868,
                25160, 40464, 5156,  28172, 44124, 19908, 26464, 42500, 24012,
                18168, 7840,  37020, 28656, 53256, 38412, 59084, 48248, 21196,
                1440,  36204, 19180, 5308,  39760, 14436, 48168, 36780, 61268,
                55740, 15356, 53424, 26592, 50588, 31128, 56368, 48004, 59844,
                36252, 5316,  56560, 28368, 36808, 50364, 23288, 976,   20316,
                29900, 35388, 61744, 53344, 3772,  31880, 51420, 55412, 29028,
                41768, 18568, 61832, 27056, 50696, 46500, 19188, 40032, 64576,
                21316, 44644, 1824,  62460, 13244, 55352, 6256,  44128, 19048,
                60700, 8180,  12672, 44008, 4340,  11156, 46412, 16052, 40856,
                21348, 51056,
            },
            {
                28808, 44516, 48516, 30972, 63192, 24980, 516,   61848, 31892,
                58736, 47348, 2844,  63080, 26092, 38980, 59832, 23784, 34992,
                17684, 26852, 47448, 40428, 16176, 35640, 19808, 41808, 3604,
                53272, 11040, 24664, 43284, 58764, 35100, 55772, 30712, 20080,
                41232, 32824, 57732, 20924, 29412, 4320,  32308, 49440, 12580,
                57000, 64224, 47320, 17504, 59076, 33848, 2912,  53104, 60728,
                13252, 31196, 41600, 20216, 64800, 17028, 23832, 34548, 57060,
                26684, 60692, 48892, 56196, 23680, 288,   52300, 27340, 42944,
                2368,  47216, 35668, 18588, 4356,  42824, 37984, 9144,  20088,
                44168, 25216, 64284, 3108,  10204, 54816, 32276, 44780, 63828,
                51536, 39792, 19012, 12588, 25200, 62688, 17000, 5364,  21060,
                49336, 3384,  43504, 38156, 11880, 34320, 28672, 54528, 3776,
                37380, 52360, 56808, 33564, 20612, 40240, 52184, 10024, 37272,
                23496, 54064, 63396, 21140, 34244, 61544, 23740, 53088, 5772,
                62164, 12108,
            },
            {
                59000, 23392, 56384, 9592,  41456, 13060, 50056, 35784, 16360,
                51216, 5752,  14172, 41940, 17380, 32728, 47780, 54852, 10172,
                38580, 58460, 13036, 22860, 8608,  30084, 5920,  48376, 26548,
                39072, 45692, 952,   51380, 28496, 12584, 2592,  43708, 14928,
                49768, 5164,  52260, 12992, 37260, 64696, 46464, 6748,  38504,
                30160, 2628,  36308, 23312, 8116,  55604, 39832, 10436, 49884,
                44540, 4080,  55024, 46896, 5484,  9108,  42276, 3908,  14960,
                45176, 10296, 30720, 16796, 44628, 34332, 63948, 17568, 6444,
                24816, 30776, 10592, 63292, 58384, 24288, 6092,  63676, 29980,
                460,   51564, 40348, 21564, 14492, 61512, 39000, 17192, 26016,
                11212, 2492,  47820, 55776, 37228, 43148, 47036, 10920, 58724,
                33376, 8916,  23996, 57960, 5924,  2060,  59888, 15320, 32424,
                26780, 17868, 8220,  29080, 47276, 2624,  25980, 63008, 15000,
                44,    41148, 31312, 16980, 50936, 1252,  29624, 56716, 38380,
                47380, 34448,
            },
            {
                7432,  39680, 14160, 33328, 58340, 27676, 45452, 7700,  43052,
                28728, 25132, 34764, 57596, 52548, 6504,  932,   20736, 42764,
                49792, 2512,  33620, 53176, 60724, 44028, 57056, 63076, 15348,
                31288, 18744, 61340, 32772, 8312,  40128, 52456, 26368, 65020,
                9336,  34720, 24052, 44860, 1228,  18180, 26044, 59784, 21164,
                9628,  52092, 43212, 12952, 49344, 30036, 15980, 36128, 21828,
                27604, 63028, 15044, 25580, 35380, 50168, 29024, 52924, 62816,
                40048, 6560,  52016, 37932, 3888,  57256, 31452, 12032, 50180,
                57724, 38748, 22228, 44948, 13096, 33624, 49480, 15724, 53996,
                12576, 33012, 17592, 48240, 43132, 29072, 4472,  8040,  57172,
                42204, 59584, 32740, 7860,  21936, 484,   30204, 39736, 65492,
                36804, 53900, 14064, 48048, 63968, 21728, 42152, 45972, 7260,
                61680, 41400, 49804, 11820, 60524, 16856, 57604, 32016, 50000,
                45788, 27592, 5388,  58608, 36860, 41772, 13304, 19284, 9424,
                26156, 16804,
            },
            {
                64164, 53940, 20844, 2460,  51512, 18064, 22756, 59616, 11384,
                54452, 63812, 38304, 9804,  22488, 44696, 13716, 61176, 28788,
                6768,  64956, 20192, 46244, 25976, 37664, 1484,  23368, 9896,
                54424, 6500,  56852, 13912, 21520, 47652, 59896, 4568,  36936,
                56528, 16976, 61624, 39976, 58480, 50328, 14068, 41556, 35012,
                55544, 18824, 26236, 61612, 1176,  45320, 64368, 6892,  57876,
                76,    38936, 33048, 10824, 56332, 61556, 19712, 11784, 32596,
                24444, 20184, 64908, 28028, 9272,  21948, 46448, 40596, 20144,
                60876, 4876,  52940, 760,   40284, 55464, 28356, 21168, 45896,
                39016, 62132, 8620,  57988, 34304, 24180, 52756, 47132, 34956,
                14840, 23048, 28076, 45128, 60592, 14312, 56864, 26388, 16036,
                2184,  44760, 28268, 31576, 16940, 35896, 51720, 24156, 12688,
                57376, 832,   35124, 22892, 44884, 38296, 7604,  21112, 35372,
                56556, 10780, 52792, 24700, 8508,  64972, 44848, 32412, 61000,
                51928, 712,
            },
            {
                30800, 27092, 46664, 37056, 5540,  64860, 38688, 3120,  33064,
                21212, 1140,  18828, 49176, 31272, 64412, 35764, 25060, 56068,
                16464, 31004, 41552, 4248,  11412, 17720, 51036, 34424, 40888,
                46968, 27780, 36608, 42900, 63380, 17740, 24280, 11108, 29988,
                20828, 47960, 7348,  31192, 10864, 28328, 54136, 3252,  45092,
                62916, 4884,  33300, 38744, 57264, 20048, 24668, 32664, 46480,
                18424, 51208, 60016, 22376, 3016,  44308, 38640, 640,   47392,
                59252, 35340, 13328, 43928, 53420, 15220, 61936, 7820,  28548,
                33932, 16116, 48804, 27012, 18216, 61080, 9796,  2464,  58672,
                23968, 4220,  27508, 50876, 1756,  11780, 64644, 20660, 30584,
                54148, 5088,  64240, 17576, 51912, 34732, 48840, 6584,  23008,
                50972, 19676, 56540, 7544,  40076, 55276, 9616,  30168, 47964,
                38740, 19568, 54296, 63312, 3952,  27276, 42660, 12616, 3492,
                64428, 18212, 39168, 47684, 15236, 27032, 49216, 3248,  22824,
                35752, 42948,
            },
            {
                49620, 10976, 61788, 16408, 43644, 29908, 55968, 47828, 15252,
                40244, 46252, 60568, 7072,  27308, 53820, 40272, 11056, 45624,
                37212, 51768, 14208, 55008, 62408, 29584, 7188,  59208, 20164,
                64532, 4112,  23856, 50356, 1964,  33732, 38520, 45648, 53160,
                42516, 26764, 3380,  54912, 21732, 34140, 7980,  24388, 16256,
                29588, 11504, 50480, 15152, 28420, 4476,  41996, 52192, 13780,
                29616, 8604,  41316, 48100, 16496, 31564, 25972, 54604, 17376,
                5804,  49964, 1848,  40808, 58424, 26336, 36432, 1184,  55208,
                42416, 9568,  65388, 32092, 7020,  37040, 47612, 42488, 30592,
                35808, 54972, 19784, 44748, 37644, 16692, 58812, 41120, 728,
                49520, 36896, 10648, 40416, 3152,  20468, 9820,  41580, 60384,
                32568, 11188, 63264, 25456, 284,   60296, 20160, 3356,  65200,
                25964, 15576, 30528, 9084,  33184, 52468, 48272, 59716, 29680,
                23160, 44300, 32520, 2268,  20772, 53856, 6592,  38896, 58500,
                12848, 18904,
            },
            {
                40264, 3612,  24840, 54420, 34264, 12676, 6568,  26328, 61992,
                53584, 29644, 13064, 42528, 16172, 2476,  19224, 5080,  59600,
                22612, 8768,  26752, 35548, 21932, 48780, 43552, 13840, 31876,
                51980, 11796, 16732, 30488, 55080, 6824,  57680, 15264, 616,
                60452, 13524, 38192, 64144, 43548, 47060, 62204, 39452, 57856,
                47728, 23012, 40868, 54412, 9356,  60784, 37012, 10980, 63320,
                54752, 25076, 4904,  36300, 57828, 13084, 7884,  63512, 41916,
                29716, 56476, 22872, 31976, 18540, 5528,  50992, 23164, 47056,
                13044, 37816, 24248, 44480, 53684, 14240, 23412, 62828, 17048,
                6576,  41368, 13888, 60892, 31204, 6188,  46128, 26736, 13356,
                19344, 44400, 57872, 25440, 31480, 62592, 54544, 28388, 46796,
                38344, 4012,  42308, 34432, 46996, 13816, 43772, 33476, 53428,
                5220,  42508, 49012, 57852, 13964, 24172, 17620, 54392, 37320,
                6784,  13760, 55556, 59416, 34920, 61576, 30620, 16452, 54968,
                46008, 5708,
            },
            {
                52108, 32468, 59060, 8564,  22976, 50292, 19364, 36064, 9280,
                23300, 4272,  36708, 55860, 58968, 33868, 47716, 50952, 31840,
                688,   63400, 47296, 58328, 2552,  39332, 56324, 26992, 52,
                38300, 44688, 60880, 40404, 9520,  48268, 27908, 64328, 32612,
                8940,  35160, 50736, 18968, 15568, 1868,  51760, 19592, 32132,
                1276,  65152, 7104,  34988, 46032, 17612, 48332, 22984, 3704,
                33800, 44184, 20464, 65336, 28012, 51784, 46416, 21656, 36992,
                14920, 8880,  45476, 61132, 11484, 39160, 64344, 32340, 17288,
                59956, 51652, 2348,  20320, 57980, 34196, 4576,  51988, 11680,
                49672, 64940, 9244,  25252, 52408, 22588, 39140, 8868,  55220,
                62956, 29240, 5880,  50420, 12792, 43984, 35720, 1552,  13420,
                57268, 16636, 53016, 22020, 49992, 27772, 37752, 17160, 58448,
                11032, 35420, 21692, 6160,  40136, 64124, 700,   9620,  41324,
                62060, 49952, 28536, 9220,  40412, 12172, 43820, 25012, 8664,
                28428, 60620,
            },
            {
                15316, 19996, 44612, 39380, 444,   62812, 41956, 58716, 51848,
                44228, 17732, 50076, 24756, 10224, 21612, 62912, 27764, 13232,
                43416, 39816, 18976, 6668,  33176, 16196, 9984,  22944, 62128,
                8424,  19420, 58108, 26288, 13572, 20244, 41724, 23188, 18040,
                48760, 24384, 57472, 30176, 5728,  26192, 36456, 12892, 9192,
                53080, 43268, 20304, 56892, 25340, 31512, 1988,  57956, 40624,
                15380, 61072, 49820, 10580, 1472,  39888, 33468, 4460,  58344,
                26860, 52740, 34948, 4616,  48788, 28716, 44336, 8944,  56932,
                27480, 5940,  30736, 62392, 10772, 28228, 39588, 56484, 20944,
                32812, 28616, 46932, 35036, 3824,  56608, 62104, 32168, 35724,
                2940,  23424, 38492, 16616, 56292, 7616,  22440, 18172, 61924,
                25152, 30544, 8536,  60968, 11524, 4580,  63128, 23448, 45900,
                28748, 52000, 62348, 45132, 28032, 35984, 47092, 20672, 33440,
                25804, 2944,  19520, 46372, 23532, 420,   50760, 64196, 34152,
                21780, 37152,
            },
            {
                63020, 26944, 12428, 56952, 28608, 47100, 14656, 31364, 2308,
                34112, 65456, 7984,  31048, 45948, 40568, 6004,  37548, 57108,
                16660, 53936, 29936, 11940, 44924, 65276, 50036, 35748, 53688,
                46880, 32768, 3216,  35364, 54032, 62188, 37072, 5684,  55920,
                44672, 4204,  41040, 11380, 60604, 42676, 56320, 28996, 59960,
                37828, 15908, 27932, 5096,  13320, 62732, 53184, 19156, 27696,
                37780, 7192,  30548, 17912, 55696, 24096, 62300, 19036, 48984,
                11924, 64216, 17776, 24836, 55788, 20860, 14104, 2064,  35804,
                21480, 45908, 41060, 16220, 49276, 43592, 1912,  25660, 45440,
                38264, 24,    58912, 15540, 42208, 11348, 17796, 47872, 51756,
                15008, 60652, 46876, 33708, 64648, 27332, 48488, 37824, 51468,
                5580,  45748, 19248, 39428, 56220, 32228, 51044, 8260,  1464,
                39284, 15108, 3676,  18840, 11728, 55688, 30888, 15208, 51732,
                57448, 43024, 15880, 63328, 52968, 36672, 18012, 4256,  48416,
                56632, 1496,
            },
            {
                45780, 7116,  35188, 51192, 17364, 37468, 5200,  22076, 11516,
                27232, 57368, 38216, 48,    61884, 14856, 52744, 25240, 9548,
                34680, 23408, 61552, 51168, 25512, 20020, 41276, 5716,  28948,
                14636, 24596, 49936, 6348,  43880, 31260, 1748,  52136, 12604,
                29156, 63300, 21220, 53780, 33472, 47848, 22324, 6496,  45480,
                24208, 33740, 60672, 51268, 39604, 35860, 8216,  44932, 50580,
                12260, 56712, 45988, 35572, 42656, 13556, 8512,  44460, 2564,
                39376, 31348, 43044, 4,     36492, 62176, 53328, 40116, 49376,
                61344, 11768, 55284, 37440, 7988,  18940, 64244, 14648, 60404,
                5872,  18608, 54644, 21888, 49424, 29744, 1452,  21124, 26088,
                7408,  42604, 10988, 20832, 188,   41100, 54200, 10612, 32964,
                42828, 65092, 35240, 1820,  24412, 15640, 41492, 20724, 60676,
                54396, 25436, 33308, 59232, 50372, 2556,  61256, 44160, 4560,
                10900, 38516, 34224, 6056,  30308, 57992, 27680, 42472, 14272,
                10588, 41052,
            },
            {
                24004, 55108, 3040,  61468, 10092, 25232, 63840, 55572, 39720,
                48460, 13784, 20628, 53616, 28792, 18560, 43976, 3112,  64596,
                49052, 6536,  1896,  38012, 57892, 31504, 3436,  60716, 18108,
                57280, 41784, 64464, 22144, 17176, 10616, 46368, 59376, 33972,
                39460, 16328, 2284,  37392, 8336,  14772, 64552, 18088, 50516,
                3644,  11632, 47540, 372,   43904, 16672, 29512, 23632, 59508,
                928,   21568, 25856, 3956,  60176, 31916, 54380, 28864, 52064,
                22316, 6456,  59952, 46856, 9784,  16588, 31008, 4324,  25968,
                18340, 34048, 3304,  24832, 59392, 32048, 52828, 36100, 9996,
                50416, 27456, 40440, 7812,  61312, 36700, 44092, 65464, 40144,
                54812, 30212, 58456, 52544, 5384,  30900, 15244, 58256, 3080,
                26920, 14456, 54416, 48744, 28560, 45304, 57492, 36056, 30704,
                47096, 9468,  43704, 22464, 38612, 26456, 8460,  23744, 65460,
                29200, 21556, 60068, 48264, 12904, 8196,  21356, 61464, 32364,
                53456, 29560,
            },
            {
                15816, 49308, 31292, 20728, 42952, 33360, 52796, 44440, 18780,
                59712, 5044,  42224, 47456, 10816, 35520, 59536, 31568, 20580,
                45532, 28152, 42064, 15292, 8456,  55012, 12876, 45644, 38764,
                10180, 1072,  36932, 55700, 28824, 50864, 24800, 20012, 7296,
                26472, 50080, 61164, 25148, 52024, 31140, 812,   39040, 55236,
                41920, 63324, 30316, 21972, 58372, 10880, 64740, 5736,  41692,
                32360, 63816, 53384, 16152, 49580, 20424, 37744, 60980, 15760,
                34708, 57100, 13272, 29312, 51068, 41520, 22616, 59056, 43516,
                8316,  64684, 29260, 51188, 22140, 46544, 4952,  29692, 22764,
                42852, 63684, 12320, 32068, 24192, 57348, 14280, 9476,  32932,
                4428,  17508, 36980, 25724, 45088, 60904, 19032, 24068, 47260,
                8152,  21464, 59104, 10492, 6552,  63952, 11988, 4864,  18332,
                13392, 64812, 56492, 7132,  15896, 42160, 53568, 36712, 18132,
                54884, 44864, 1312,  25084, 41308, 55128, 45544, 38112, 5296,
                19404, 65248,
            },
            {
                36624, 40496, 12788, 58684, 48208, 7512,  1080,  29672, 8528,
                35296, 25876, 32296, 56248, 23944, 4628,  50224, 13576, 39392,
                56420, 11076, 60596, 22208, 35276, 47976, 27380, 23208, 52264,
                30464, 48512, 13616, 7464,  61596, 38996, 14376, 65428, 42568,
                58156, 9976,  46344, 18732, 41340, 58020, 12116, 35172, 26744,
                20648, 15184, 7976,  54028, 26052, 33532, 48308, 54820, 14360,
                36628, 47464, 10196, 39732, 5608,  26616, 1040,  10688, 24672,
                48472, 40792, 19116, 26132, 65244, 6808,  33816, 55608, 13424,
                52104, 46840, 15216, 41944, 1012,  12504, 40260, 58140, 54344,
                16084, 34268, 45916, 3176,  53064, 5524,  28380, 51088, 45940,
                59972, 23044, 49812, 13764, 9224,  39308, 34876, 63644, 41696,
                52284, 37452, 30360, 40736, 18580, 34316, 53612, 27024, 49676,
                37920, 332,   29236, 35220, 47756, 60128, 12772, 32668, 48772,
                5556,  14256, 51332, 32272, 64424, 16224, 2148,  51736, 26204,
                47136, 8808,
            },
            {
                56848, 23196, 5396,  27664, 38840, 15020, 23592, 61204, 16600,
                49940, 63404, 1696,  16004, 61112, 41156, 54144, 26240, 1416,
                17304, 30780, 53156, 49732, 18620, 62404, 1528,  34052, 63516,
                20568, 59568, 26200, 34444, 45180, 4096,  32644, 53528, 308,
                36096, 13484, 32280, 4784,  28348, 23192, 47040, 61732, 5372,
                31992, 57584, 36952, 40852, 18068, 4604,  38152, 20508, 27484,
                3456,  18512, 30044, 43896, 63156, 57320, 47240, 41912, 64080,
                3796,  8572,  54248, 38048, 2340,  44980, 11100, 28140, 38368,
                21784, 5612,  36928, 55976, 60832, 33720, 17636, 26076, 8044,
                1844,  59772, 19500, 39024, 47952, 17196, 62400, 20308, 38164,
                12368, 3004,  64212, 28756, 47896, 55228, 1908,  11920, 28320,
                17128, 62600, 512,   44736, 51776, 22540, 2324,  43280, 62288,
                23504, 41804, 52804, 19656, 3552,  28308, 20836, 1776,  40740,
                58668, 26704, 38008, 9892,  19972, 35356, 23808, 11640, 60560,
                34516, 540,
            },
            {
                50784, 62136, 44376, 18468, 64484, 51132, 36404, 54808, 40932,
                11128, 21916, 45464, 37192, 9348,  19900, 6920,  33900, 47528,
                62936, 36828, 3928,  25684, 6276,  44152, 40648, 16180, 8272,
                4752,  43028, 16848, 52812, 21388, 9748,  47944, 18312, 29764,
                22544, 55692, 43336, 64232, 53924, 7816,  15932, 42920, 51504,
                10468, 45260, 49616, 2620,  62032, 51676, 45116, 8644,  61636,
                50024, 59184, 23920, 7424,  32780, 12996, 35336, 18044, 29728,
                52472, 32856, 21912, 57564, 16140, 49116, 60256, 19316, 596,
                62468, 32084, 26444, 10080, 20616, 49020, 63352, 44036, 37800,
                48392, 30180, 56392, 26596, 10284, 35544, 42292, 560,   25344,
                56264, 34508, 43080, 20136, 6508,  31872, 22604, 49944, 56724,
                4912,  33724, 13448, 25404, 61008, 38604, 14792, 58408, 8148,
                32880, 16300, 61020, 11900, 50684, 62836, 45200, 55640, 10336,
                30576, 17456, 61592, 6636,  56680, 50012, 40380, 30652, 57760,
                43112, 17900,
            },
            {
                10884, 32460, 2700,  54324, 9544,  30852, 4164,  46100, 27788,
                6140,  58060, 52432, 30492, 27016, 65084, 44344, 57252, 22968,
                8596,  43368, 14192, 38968, 32004, 58576, 12272, 56416, 37620,
                47376, 32212, 56992, 2236,  60240, 41352, 26796, 57148, 6028,
                49508, 39124, 16820, 2080,  49156, 34080, 59308, 19920, 1832,
                24240, 65516, 28164, 13160, 23520, 30632, 15604, 56976, 32956,
                12124, 39144, 55248, 15704, 51032, 21184, 54088, 6052,  44536,
                14544, 62044, 46332, 12160, 30876, 24468, 36176, 53108, 41764,
                47800, 58196, 17216, 45264, 6848,  28648, 3588,  52364, 11468,
                21856, 14176, 6524,  64524, 22792, 55280, 31704, 48824, 15728,
                52384, 8656,  39616, 57468, 61984, 14432, 43836, 38128, 7744,
                45660, 20780, 58068, 48180, 9012,  32044, 28908, 47420, 19912,
                54688, 5020,  26028, 31508, 37544, 6628,  24848, 34968, 63932,
                22396, 52752, 42616, 47516, 28448, 3628,  63132, 7664,  14708,
                22204, 28836,
            },
            {
                38372, 47644, 24624, 35044, 41704, 21436, 12844, 57208, 19208,
                33648, 39132, 13340, 4036,  49084, 38488, 12232, 15684, 29456,
                59752, 20180, 55332, 64376, 9776,  50920, 29352, 24168, 53476,
                27656, 10624, 39692, 23948, 12916, 36364, 63776, 15112, 44656,
                62012, 8352,  27364, 21312, 37896, 11124, 29816, 39776, 56508,
                36336, 17316, 7268,  34604, 59360, 41464, 1740,  25408, 43076,
                22048, 80,    45388, 27952, 38032, 2568,  59572, 23568, 39336,
                26564, 676,   36572, 5416,  42928, 61720, 4188,  9268,  14640,
                24196, 3264,  40160, 64416, 53264, 36436, 14884, 24332, 32056,
                61472, 50764, 44676, 41056, 2680,  13384, 60780, 6832,  27740,
                63344, 30996, 16900, 1100,  26588, 53072, 18600, 60004, 25620,
                30088, 53908, 36772, 3696,  17528, 55360, 5808,  40508, 11332,
                36600, 44788, 48568, 58740, 9576,  43116, 17132, 13648, 46380,
                6368,  40,    33080, 24224, 12408, 44716, 18276, 53396, 46692,
                4768,  55372,
            },
            {
                13612, 6720,  58032, 16008, 60636, 48740, 25768, 62296, 1840,
                44016, 64084, 24712, 18024, 55252, 31904, 3144,  52224, 36000,
                50208, 780,   27992, 46984, 22360, 2536,  18128, 44704, 64776,
                912,   19852, 62020, 48652, 30076, 51996, 3364,  33664, 23876,
                12104, 31516, 60340, 57464, 45232, 5124,  63596, 26408, 13740,
                53480, 43292, 48444, 21136, 54436, 10368, 63752, 36288, 52808,
                6296,  64612, 34576, 10104, 61944, 46560, 31016, 11400, 56840,
                50316, 18880, 58588, 28020, 51540, 17872, 33112, 56804, 29540,
                50568, 35344, 12796, 30764, 22452, 42524, 59268, 55844, 40056,
                468,   35704, 16744, 33336, 28816, 51148, 43900, 21272, 40192,
                11352, 46620, 23212, 51060, 37064, 33108, 3196,  10308, 64516,
                15232, 40836, 11448, 62332, 43424, 21304, 65356, 51480, 25524,
                61784, 2852,  14296, 39916, 22876, 56988, 51616, 28828, 39564,
                59976, 36476, 15004, 57928, 38780, 21600, 32656, 36892, 26072,
                41628, 64652,
            },
            {
                51972, 20456, 27940, 44724, 788,   7836,  38092, 51680, 31412,
                15436, 47984, 7612,  60744, 42748, 21796, 58692, 25500, 5532,
                40676, 11888, 34404, 16532, 42232, 61156, 36544, 6976,  14152,
                34640, 51464, 15688, 6068,  44316, 8640,  20752, 39600, 54588,
                1316,  51656, 35740, 14156, 24856, 17928, 50728, 46836, 8888,
                32528, 3856,  57728, 39136, 4916,  46628, 19616, 29232, 13564,
                47700, 18416, 57460, 25056, 5544,  41436, 16832, 65028, 7304,
                34088, 10116, 40468, 64292, 8024,  22108, 39372, 45816, 63704,
                20428, 7504,  55096, 48088, 1196,  10944, 18792, 5900,  46604,
                9840,  25868, 53472, 62236, 19192, 8308,  36648, 58768, 4140,
                34044, 55328, 5336,  44580, 12488, 58172, 42196, 48108, 35040,
                51620, 1384,  23292, 28412, 50344, 35536, 13264, 956,   33500,
                21760, 56140, 29832, 18956, 64788, 6120,  32996, 3228,  54564,
                20188, 49536, 9040,  65136, 52152, 2288,  61812, 10252, 59196,
                1388,  31488,
            },
            {
                35656, 61372, 40024, 11932, 54016, 34020, 17688, 42620, 10000,
                22400, 53724, 29172, 35244, 392,   10248, 46640, 61864, 18244,
                44572, 63084, 23892, 53572, 5664,  32492, 55804, 47948, 21840,
                59224, 43188, 37916, 25616, 55664, 17708, 59048, 47012, 26164,
                42328, 19684, 47564, 3692,  55172, 42176, 33736, 164,   22352,
                59916, 29016, 11864, 25120, 31808, 16160, 50512, 7596,  56360,
                39440, 31492, 14524, 49600, 20276, 53040, 35832, 27324, 48612,
                21492, 53644, 24720, 47024, 13144, 55508, 2104,  26112, 11036,
                59824, 44052, 16612, 27152, 61088, 34412, 49728, 30064, 65316,
                20552, 57744, 3932,  11600, 46184, 56496, 24820, 16460, 47768,
                61352, 18692, 29484, 65232, 7388,  20936, 29056, 23972, 5508,
                18348, 46476, 56256, 33004, 4504,  26352, 45580, 59344, 17264,
                42404, 8488,  52620, 35688, 736,   26756, 47192, 62048, 11928,
                25760, 43392, 28252, 17624, 5704,  48216, 28984, 15916, 49360,
                23352, 17536,
            },
            {
                9312,  2856,  49868, 30760, 22600, 65408, 28396, 55904, 3332,
                36724, 59280, 12176, 40256, 50968, 16488, 33628, 39048, 7752,
                30872, 57140, 3036,  38160, 49564, 10820, 25940, 40892, 30832,
                9236,  28344, 2992,  63988, 32040, 35648, 11184, 62356, 5208,
                15812, 64560, 9692,  40112, 30844, 7520,  62448, 52532, 15748,
                41228, 63880, 18892, 51368, 61400, 37276, 42668, 26420, 62724,
                23564, 2336,  43632, 60512, 29948, 1076,  13464, 45168, 3872,
                42356, 15284, 2884,  30692, 35272, 43344, 49384, 15440, 37528,
                6044,  32764, 39552, 56672, 8412,  25352, 38864, 15940, 43528,
                27760, 37572, 48716, 23772, 39536, 1236,  30936, 53976, 12804,
                2192,  41584, 25124, 38940, 49664, 16520, 62752, 55656, 38476,
                30784, 61608, 7712,  42120, 15520, 57816, 9480,  38168, 49000,
                27892, 63212, 46188, 12456, 49984, 41544, 21332, 36152, 16424,
                7184,  32112, 56228, 40784, 24752, 34384, 42576, 54824, 7168,
                39456, 45652,
            },
            {
                55712, 25504, 16828, 58868, 46288, 4344,  13596, 61580, 25100,
                45512, 5592,  19380, 63124, 23544, 55824, 28332, 13204, 53824,
                20968, 47712, 15132, 29492, 58644, 20684, 13624, 61792, 4276,
                49316, 57260, 12500, 52984, 21752, 532,   49572, 29364, 37792,
                33260, 56116, 27988, 21920, 60824, 12776, 20232, 37416, 26068,
                46036, 35612, 3056,  44288, 9380,  708,   58548, 34284, 4084,
                11228, 54708, 35140, 8132,  38720, 55612, 22936, 58712, 32160,
                63004, 55908, 38200, 60536, 17472, 5496,  62744, 28844, 54100,
                23516, 51004, 3000,  20932, 44852, 62948, 51748, 2168,  54724,
                13480, 7284,  31624, 59444, 15076, 64732, 42832, 27292, 35956,
                21616, 52060, 56880, 9708,  35208, 46080, 140,   13976, 9432,
                59068, 21032, 12300, 37216, 63712, 19496, 51800, 30280, 6264,
                15012, 4116,  24416, 32052, 17368, 55032, 10056, 59320, 45292,
                52956, 62872, 2872,  46648, 11140, 60648, 19572, 13184, 58272,
                29516, 63808,
            },
            {
                33392, 42752, 6400,  35944, 10264, 40744, 48908, 20364, 38976,
                50620, 33488, 27476, 47220, 6904,  43160, 2252,  64624, 49340,
                26396, 10388, 41892, 7092,  65224, 45632, 400,   53224, 18884,
                39288, 23976, 17100, 46484, 42200, 57688, 14520, 23088, 7584,
                44024, 51064, 1968,  35268, 49920, 44912, 29324, 58472, 10556,
                6384,  54004, 14412, 28116, 55312, 23152, 13132, 20512, 45832,
                40952, 50848, 19348, 26800, 48032, 62036, 10356, 37332, 17992,
                6724,  28880, 11476, 50748, 23148, 57508, 33508, 19480, 1112,
                42252, 64912, 29844, 14252, 36748, 11868, 5444,  23052, 33768,
                62608, 41732, 52508, 19892, 35164, 5724,  9828,  49532, 63140,
                7232,  32404, 14752, 3744,  59652, 26672, 32252, 52100, 41036,
                49200, 27532, 44600, 53804, 24716, 2792,  40960, 22840, 60644,
                54320, 36564, 40696, 58784, 62572, 6652,  30536, 38536, 1364,
                23328, 34872, 13660, 21268, 37668, 52732, 584,   25984, 36592,
                3532,  20928,
            },
            {
                14692, 51428, 62696, 19680, 26568, 53028, 32336, 8144,  58752,
                16628, 1244,  56584, 14380, 31656, 59804, 20104, 35136, 5132,
                37492, 61284, 33344, 24552, 16792, 35980, 27472, 32232, 43580,
                63448, 34472, 7480,  37052, 5012,  27588, 39772, 52684, 61376,
                18076, 13140, 24516, 57372, 16772, 4856,  39188, 2472,  55796,
                33292, 49080, 21024, 39596, 65060, 47584, 31372, 53232, 61004,
                16936, 29204, 64832, 12624, 4292,  16596, 41948, 52196, 25460,
                49528, 20592, 45828, 0,     27256, 9556,  39696, 47504, 59240,
                11608, 17856, 47020, 58840, 53884, 32288, 18928, 60208, 46108,
                10324, 17744, 4196,  28952, 44476, 51456, 22928, 17352, 38620,
                45516, 60668, 43032, 18304, 22500, 54216, 43992, 4820,  25020,
                35464, 1884,  17068, 5856,  47736, 34336, 56960, 12088, 32748,
                45016, 19640, 10796, 3044,  22368, 44312, 14668, 51796, 27884,
                18184, 48408, 59864, 29836, 6228,  64476, 32140, 43916, 47608,
                53192, 10868,
            },
            {
                28180, 46756, 272,   38408, 57308, 15288, 2040,  43912, 29396,
                11540, 64780, 42320, 37980, 10052, 52880, 24604, 45820, 16372,
                57428, 1580,  44416, 55124, 50476, 40420, 56464, 6460,  15096,
                10764, 51284, 29508, 59900, 19932, 65352, 9884,  45428, 30968,
                4532,  59456, 47868, 41284, 11008, 65144, 25068, 47244, 18100,
                23600, 62656, 30608, 7012,  17112, 36508, 4720,  27548, 8672,
                37648, 6096,  57132, 24284, 46212, 33980, 29732, 1728,  9088,
                60180, 40168, 33076, 65184, 42032, 52248, 13640, 6868,  31088,
                25568, 35428, 8200,  4684,  24020, 42376, 48556, 29268, 39656,
                25564, 56172, 47972, 61048, 12152, 55200, 33428, 58744, 632,
                11788, 26024, 30424, 50788, 37892, 11308, 62320, 19968, 15504,
                54632, 65104, 31996, 60104, 28900, 8776,  43828, 17888, 240,
                64992, 27236, 52988, 48344, 35304, 25840, 57420, 4932,  64920,
                42244, 8796,  39640, 57004, 15624, 49696, 9588,  22436, 16716,
                62156, 40776,
            },
            {
                23732, 60360, 8472,  29956, 45220, 64188, 24380, 55384, 36216,
                22016, 52476, 25880, 18364, 48956, 3212,  39556, 12048, 31208,
                51240, 22836, 19016, 12380, 3812,  9384,  22184, 48576, 58444,
                26096, 1460,  55584, 47620, 15636, 33608, 1792,  55396, 20676,
                26416, 38208, 32420, 8232,  19992, 52324, 31732, 60220, 41720,
                12208, 964,   43824, 59300, 10704, 42084, 50004, 57976, 14588,
                33348, 48764, 600,   39296, 53676, 20488, 64076, 44648, 56312,
                35528, 14340, 5252,  16296, 56624, 20112, 36276, 61276, 44108,
                52820, 56972, 41124, 28436, 64032, 1588,  15344, 58116, 6596,
                36332, 320,   22240, 38328, 26480, 3476,  40672, 29432, 20068,
                56132, 48884, 5596,  64456, 2016,  28664, 40096, 7668,  57644,
                45668, 9760,  38196, 21276, 52800, 14680, 62220, 26144, 49332,
                38656, 8344,  57916, 29712, 12620, 40156, 49636, 20296, 31920,
                12252, 54696, 3464,  24080, 44816, 27384, 40132, 59380, 7316,
                34832, 4144,
            },
            {
                18136, 33124, 54664, 22096, 11480, 34996, 18156, 5888,  61248,
                47720, 4076,  8324,  34628, 62804, 29332, 55156, 61036, 41632,
                7228,  64136, 29128, 38984, 59996, 30524, 63852, 34132, 19652,
                45184, 38064, 22380, 12096, 42596, 25636, 48960, 35496, 11668,
                43248, 63728, 472,   54728, 27672, 36316, 14052, 7556,  50912,
                35396, 57352, 26552, 51744, 34040, 24368, 1948,  19408, 44708,
                62528, 22392, 42640, 30520, 59172, 14476, 4828,  27420, 12448,
                22704, 48016, 54720, 24884, 29804, 6344,  49048, 23256, 15268,
                3200,  20776, 49972, 16572, 37760, 51904, 9412,  21224, 54272,
                13868, 64584, 32572, 16404, 8748,  62788, 14512, 47112, 7564,
                35484, 23216, 13952, 33660, 55548, 16956, 48220, 34520, 30148,
                23136, 13100, 42504, 50104, 1500,  40152, 36088, 55452, 4996,
                22216, 15628, 42016, 1732,  18028, 63176, 9100,  36380, 46960,
                60852, 26516, 33860, 51384, 18844, 35716, 2332,  54252, 30312,
                48456, 56836,
            },
            {
                43776, 49852, 14096, 41048, 3076,  48116, 51628, 42048, 14132,
                39348, 30964, 45056, 57952, 22596, 15192, 5620,  19524, 26872,
                10240, 36164, 48288, 53500, 25300, 45912, 13216, 2416,  41168,
                61928, 5744,  32000, 60796, 52564, 7800,  62004, 17252, 58400,
                50672, 15432, 22640, 46452, 61064, 43784, 3276,  21812, 28980,
                16440, 38664, 5424,  20028, 13908, 56300, 63688, 39148, 25796,
                10168, 54580, 18112, 7376,  10744, 37124, 51244, 40980, 61424,
                31032, 2300,  43300, 60708, 34736, 1556,  63548, 38712, 27784,
                9608,  62880, 33152, 11436, 46224, 26520, 61236, 34288, 41828,
                49756, 27568, 45448, 57500, 50280, 43088, 24048, 59648, 53116,
                42364, 61760, 39340, 46652, 9000,  21952, 60920, 51000, 2452,
                62932, 27264, 6532,  18092, 58572, 23628, 11012, 46024, 30472,
                51700, 34364, 47048, 61280, 32868, 54596, 4100,  24376, 16552,
                624,   38228, 14368, 5628,  57752, 10756, 62612, 14004, 20412,
                38588, 12044,
            },
            {
                26512, 5280,  36440, 63112, 59188, 25688, 32756, 9940,  28072,
                19172, 59544, 12420, 248,   37564, 43748, 50632, 34108, 46940,
                56356, 14748, 592,   17272, 5260,  35400, 52536, 16132, 28372,
                54052, 10152, 18256, 40224, 2780,  30156, 23404, 38944, 3424,
                8984,  28992, 34724, 6364,  17424, 39368, 56336, 64024, 45456,
                54328, 9500,  48524, 61296, 45776, 29868, 7656,  52704, 32028,
                4020,  36012, 50456, 26832, 62992, 47792, 23804, 16712, 7224,
                52856, 37836, 18472, 9252,  50880, 45760, 12364, 32032, 54492,
                46792, 39812, 756,   22888, 57672, 5052,  44444, 30560, 2372,
                19308, 10996, 39984, 4632,  20600, 35024, 1924,  31860, 16484,
                10280, 3376,  19000, 58348, 27452, 43624, 12608, 6112,  41292,
                53004, 35796, 47336, 61844, 33320, 28416, 7256,  63576, 19424,
                13156, 59112, 24908, 6884,  21328, 51328, 42540, 29316, 56920,
                43568, 53548, 21064, 63584, 46044, 31540, 42592, 50720, 24708,
                1476,  65044,
            },
            {
                31520, 52592, 16924, 28724, 7636,  20692, 56772, 4644,  63392,
                53112, 24292, 48644, 55416, 27552, 9644,  63624, 2136,  24144,
                60200, 44204, 32192, 62000, 20636, 42772, 58192, 7560,  22760,
                49568, 36228, 57052, 26820, 45060, 13600, 54736, 46668, 31856,
                41848, 53800, 57820, 12716, 49912, 25892, 10736, 33636, 1652,
                24464, 62860, 32620, 3784,  22812, 37592, 17248, 47472, 13000,
                58240, 61696, 15188, 43744, 31712, 1520,  34676, 58352, 45800,
                21144, 63720, 11992, 27360, 39868, 21668, 16724, 58936, 4156,
                18596, 60364, 29696, 53412, 35736, 17788, 12628, 24256, 62972,
                52852, 7180,  60888, 30172, 54384, 12256, 65372, 27864, 37944,
                49280, 25700, 31244, 52576, 1008,  37308, 32896, 25468, 18908,
                15168, 56532, 22196, 4180,  44652, 15544, 57284, 41264, 2272,
                53848, 36972, 10276, 28384, 45328, 14224, 37652, 10652, 64312,
                6912,  49028, 27960, 40452, 23620, 3964,  17136, 28736, 46404,
                59008, 9340,
            },
            {
                21636, 58164, 1024,  46548, 53808, 38852, 15912, 44768, 36016,
                2704,  33592, 16448, 40772, 20788, 32328, 54140, 17648, 38352,
                6356,  21904, 40176, 55760, 10712, 26912, 37908, 46992, 30956,
                64904, 60,    14996, 48484, 63372, 36912, 20912, 5128,  64632,
                18684, 25064, 37628, 62588, 3852,  31360, 19544, 48164, 42280,
                13472, 17968, 27904, 41024, 11700, 55424, 172,   42380, 21280,
                28136, 40292, 2748,  22160, 57236, 18912, 39688, 9808,  29276,
                3860,  48912, 32456, 54020, 57968, 6820,  42664, 51840, 25420,
                36760, 43648, 14480, 7112,  63860, 40396, 48120, 56144, 36716,
                15832, 47264, 23900, 37540, 17188, 44792, 51596, 5344,  21008,
                55848, 62896, 7328,  45492, 15036, 65016, 54264, 59792, 45860,
                38832, 8724,  31200, 11996, 54204, 35132, 50336, 25956, 32208,
                43732, 4312,  64664, 39932, 55752, 1352,  59880, 22288, 34572,
                18240, 31132, 11844, 8452,  58856, 52400, 37532, 7852,  54608,
                34528, 41240,
            },
            {
                48584, 37888, 24608, 43240, 10260, 30728, 60396, 49188, 21968,
                42808, 8648,  64616, 5712,  60488, 13588, 47968, 42420, 30400,
                12704, 51788, 28568, 7792,  48864, 63388, 2832,  17940, 12000,
                43564, 24976, 33968, 19848, 5976,  9836,  58860, 27444, 49044,
                11180, 1232,  45540, 21372, 41376, 53128, 60732, 5932,  57944,
                35672, 52444, 44496, 59468, 50860, 34452, 25196, 64932, 49488,
                5820,  33644, 46312, 11836, 52908, 6232,  65324, 54352, 25308,
                41416, 15800, 36172, 3024,  24612, 64528, 29472, 34440, 8012,
                11740, 56740, 26964, 50484, 20624, 32104, 3308,  8692,  28544,
                42788, 33200, 55816, 804,   26792, 8520,  39632, 57712, 13712,
                43272, 35104, 11592, 40072, 23512, 20008, 4824,  10692, 28596,
                500,   63708, 50648, 25360, 39860, 1308,  21016, 9016,  61644,
                17700, 23028, 49928, 15780, 19728, 32220, 47292, 25860, 50616,
                3164,  62084, 36676, 47384, 19296, 33052, 12520, 61012, 3480,
                18432, 14560,
            },
            {
                5840,  11672, 61776, 19840, 35216, 13708, 1880,  26604, 55652,
                11876, 30324, 51280, 45796, 36256, 3828,  26208, 8592,  57656,
                65332, 3100,  46320, 34900, 15328, 23692, 33032, 60448, 56176,
                39464, 4524,  59700, 50924, 40688, 32240, 43596, 15716, 35712,
                59580, 52032, 14984, 33448, 9324,  28392, 15868, 38276, 22280,
                30136, 7392,  2248,  20460, 8896,  15672, 57328, 30688, 9824,
                19068, 54952, 60624, 37368, 26412, 48304, 15484, 33784, 12644,
                61500, 56692, 7868,  44232, 19464, 14016, 380,   49256, 61120,
                23456, 47324, 2128,  41876, 10372, 59660, 52164, 22364, 58528,
                4468,  20252, 13024, 62096, 48276, 59436, 23344, 32472, 47152,
                1288,  17660, 28744, 60492, 48080, 31620, 42028, 50020, 57936,
                34056, 19476, 42684, 59980, 17212, 64436, 46440, 13580, 38480,
                48140, 30024, 57392, 35280, 5568,  63252, 12636, 8292,  41320,
                56112, 14828, 43844, 54488, 1180,  64752, 39724, 26304, 22552,
                45248, 63500,
            },
            {
                27228, 33480, 54628, 3600,  51436, 65320, 40592, 7088,  18752,
                61836, 38100, 24788, 18296, 28876, 56044, 22412, 49784, 36588,
                16420, 25516, 19144, 59256, 53532, 41672, 6404,  50392, 21660,
                9264,  53352, 29768, 12536, 22864, 56760, 52836, 3280,  23664,
                40196, 30396, 6616,  56088, 64272, 46376, 376,   50124, 11112,
                65512, 56000, 37284, 26228, 63216, 40088, 4584,  45100, 36944,
                51252, 16096, 23888, 8700,  30244, 42432, 20960, 684,   45160,
                50376, 22536, 29792, 60284, 47452, 37132, 55684, 41076, 20300,
                15956, 65436, 30812, 35076, 16776, 25740, 37872, 14992, 45048,
                64916, 9932,  51204, 41988, 34812, 3992,  18440, 9972,  63956,
                25092, 54508, 50452, 3160,  57204, 8364,  36292, 17036, 24212,
                12764, 47476, 3688,  29740, 5764,  37136, 55856, 28276, 59432,
                2736,  12036, 7900,  41908, 27224, 52128, 44132, 58992, 38348,
                20716, 26904, 5844,  24236, 29880, 15260, 49088, 42776, 56436,
                30320, 50916,
            },
            {
                40108, 57736, 17512, 29632, 47488, 22608, 57152, 32828, 46400,
                52964, 14572, 1420,  58820, 10304, 40148, 63064, 664,   33400,
                44592, 55016, 38992, 11684, 1172,  29840, 45004, 13900, 28108,
                35324, 18484, 62568, 46324, 1116,  17116, 28940, 7924,  62024,
                47144, 19868, 26276, 43380, 35964, 23576, 59164, 26924, 41244,
                16840, 45720, 12884, 49260, 29688, 47168, 23340, 13772, 62388,
                27584, 1088,  44268, 64000, 3524,  51900, 60096, 37820, 27756,
                5004,  17812, 39668, 11116, 53588, 26272, 9308,  31664, 4316,
                38568, 52976, 5256,  55336, 46532, 62644, 20,    49236, 34368,
                26624, 39512, 31012, 21772, 14784, 29004, 50928, 36612, 41836,
                30228, 6940,  21804, 37684, 13452, 26524, 53712, 2376,  61664,
                38080, 55764, 21876, 53288, 11396, 32844, 23584, 6656,  52748,
                33832, 45028, 61432, 54552, 21172, 3668,  29660, 16476, 396,
                32080, 60900, 51688, 35096, 57480, 21244, 6480,  10484, 35880,
                112,   8964,
            },
            {
                44520, 14348, 5120,  41880, 26104, 9048,  16376, 36976, 28712,
                4064,  41852, 48476, 33204, 53360, 43852, 14040, 20120, 59728,
                10100, 6000,  31368, 62184, 20916, 36372, 64636, 55140, 2216,
                48184, 42148, 6980,  25896, 36376, 65268, 49604, 33808, 13584,
                53840, 10564, 60816, 3500,  17736, 13276, 7892,  53888, 34004,
                4960,  23820, 32060, 60820, 17840, 1764,  54376, 32636, 6608,
                59284, 39716, 33216, 56736, 18236, 12384, 32036, 55616, 9440,
                62596, 35120, 51772, 1648,  33240, 16676, 61980, 46272, 58200,
                28284, 10656, 43892, 21500, 7704,  29468, 41444, 11064, 54364,
                18144, 1936,  57224, 6916,  63652, 44032, 55356, 2312,  60032,
                15616, 39780, 62272, 44776, 18564, 64152, 46352, 30248, 43200,
                6556,  27396, 15516, 40928, 45228, 49864, 19232, 42912, 15120,
                26484, 18888, 24524, 38988, 14136, 48752, 36360, 23988, 53468,
                45156, 9208,  12980, 41936, 2632,  45984, 59708, 51600, 16760,
                61308, 20392,
            },
            {
                25184, 64192, 49320, 37516, 59808, 884,   43588, 63520, 11088,
                23224, 60224, 20532, 7444,  16604, 30540, 5404,  27844, 52336,
                23780, 41460, 47444, 26788, 49824, 7696,  17520, 25488, 38416,
                57588, 14924, 32416, 58952, 44212, 10184, 39468, 19328, 42744,
                1804,  37560, 32148, 48496, 51908, 39324, 31536, 43920, 19508,
                62216, 55028, 3452,  42044, 8476,  58976, 38016, 19488, 43228,
                10940, 21632, 13984, 46700, 36504, 23480, 6632,  40664, 47536,
                24512, 13792, 42344, 64056, 21832, 6432,  42972, 23904, 12920,
                49976, 36328, 25168, 60996, 13348, 56572, 19624, 31884, 5140,
                60112, 46952, 24532, 52844, 37588, 11752, 25888, 20700, 47860,
                12404, 52056, 31944, 9800,  34560, 4660,  21544, 14832, 51176,
                9740,  64964, 35632, 59500, 7928,  63016, 216,   58428, 39968,
                65472, 51028, 1428,  9660,  31792, 62488, 56412, 10904, 65008,
                19124, 48144, 37692, 63296, 18080, 33280, 27796, 23868, 32216,
                38704, 53484,
            },
            {
                3832,  31924, 21512, 11240, 33672, 54792, 20324, 50552, 6148,
                56764, 35068, 27000, 65004, 38772, 49452, 61328, 46508, 37144,
                15428, 56548, 3540,  13720, 53036, 39672, 11048, 60172, 4872,
                22632, 51532, 11460, 20804, 54624, 5092,  24232, 56108, 27888,
                62820, 23092, 58072, 6244,  21392, 55724, 63284, 1976,  48252,
                10428, 38856, 51860, 21468, 34904, 12068, 50948, 26168, 48292,
                55800, 29604, 52512, 4776,  26884, 50332, 63580, 16184, 20196,
                57620, 31168, 4360,  27412, 49172, 38292, 57120, 2900,  19272,
                62996, 948,   17728, 39824, 33920, 51164, 23552, 63924, 14352,
                42692, 35376, 9484,  16428, 49056, 4848,  58188, 33760, 5876,
                24300, 57524, 176,   27544, 49760, 56640, 38648, 60280, 33368,
                23700, 48200, 2712,  31180, 25380, 13092, 29196, 35876, 10572,
                4608,  46804, 35108, 59644, 43696, 18192, 6736,  41092, 33988,
                4364,  28220, 23068, 7452,  55676, 39416, 11824, 64448, 2816,
                47928, 12684,
            },
            {
                36536, 60304, 7468,  52020, 46200, 13656, 30428, 25836, 47284,
                39880, 13088, 44900, 54656, 3548,  24500, 11764, 32672, 2412,
                64508, 18716, 34836, 58636, 23276, 31864, 43180, 46964, 28964,
                34252, 63992, 1516,  38660, 29972, 15444, 59756, 46872, 6816,
                17232, 45288, 14336, 41504, 26432, 10876, 29076, 15920, 24724,
                36072, 27440, 15352, 58004, 28576, 44068, 64712, 15996, 2648,
                35700, 8084,  62892, 41260, 58564, 10528, 43856, 28892, 2320,
                52424, 44772, 59608, 18868, 54980, 14552, 29596, 34852, 52700,
                45136, 31500, 58948, 47668, 6388,  2384,  44412, 37372, 49980,
                28800, 20072, 62284, 41028, 27620, 31456, 17820, 45120, 64640,
                36092, 42700, 16204, 61336, 23248, 41428, 11688, 1564,  19300,
                57424, 40340, 17440, 54880, 20900, 52348, 47440, 56604, 22676,
                31092, 16700, 27936, 55716, 23272, 2440,  46716, 25904, 15416,
                50756, 59640, 31340, 14344, 49660, 4548,  53872, 43788, 8072,
                57608, 29440,
            },
            {
                44880, 55148, 16988, 24504, 62884, 38952, 2788,  60828, 15844,
                32204, 136,   18460, 10016, 36112, 21408, 57568, 43072, 8244,
                50048, 29940, 45236, 6580,  63648, 424,   19356, 53748, 8480,
                16472, 45420, 26900, 48860, 52292, 3180,  32120, 35744, 11816,
                51304, 29852, 34796, 65052, 2668,  36888, 45740, 59452, 7340,
                50384, 61060, 5720,  46868, 440,   23244, 31332, 5832,  40668,
                61260, 24620, 15420, 19880, 276,   32592, 54876, 35604, 5964,
                38456, 12296, 8396,  36068, 46644, 10384, 63628, 7144,  40200,
                26356, 9420,  14964, 54192, 28160, 61960, 10036, 25988, 6640,
                58776, 3068,  32924, 55168, 576,   61532, 38444, 53704, 10060,
                29188, 19876, 7856,  54084, 47560, 5680,  29480, 54400, 45792,
                27748, 12712, 6076,  44404, 33888, 38932, 15924, 6880,  43208,
                53928, 62056, 7344,  40756, 13020, 51940, 29300, 58780, 38184,
                54964, 1596,  43640, 62052, 26176, 20140, 35684, 15756, 22404,
                40708, 18640,
            },
            {
                10048, 2072,  41508, 28832, 5564,  18804, 42840, 55420, 8612,
                52596, 62672, 29848, 50644, 59636, 40588, 15852, 52916, 26348,
                38692, 22312, 10476, 41020, 27520, 49104, 14248, 62080, 37472,
                58720, 41272, 6308,  18376, 62144, 42588, 21628, 64072, 39676,
                232,   54916, 8980,  49592, 18368, 57156, 51460, 20588, 42144,
                30808, 13028, 19820, 39952, 56012, 10008, 60528, 53488, 21160,
                33000, 50144, 44944, 37508, 48520, 22696, 13200, 61416, 49556,
                26308, 65132, 23168, 41172, 456,   25540, 50740, 20984, 15984,
                56388, 4404,  37300, 22948, 18924, 40840, 57036, 16912, 53180,
                12168, 45608, 23492, 13744, 43372, 7808,  22956, 14600, 2904,
                50248, 46360, 37832, 32448, 13428, 18196, 35872, 63148, 7796,
                37672, 50236, 62728, 59016, 9572,  2424,  27208, 60072, 19452,
                38068, 11732, 49060, 20576, 37024, 32832, 64140, 19688, 7788,
                11848, 21128, 34380, 40448, 9704,  46916, 59096, 28676, 62716,
                50624, 25800,
            },
            {
                61736, 33008, 50116, 35884, 58276, 10472, 32736, 23616, 37116,
                20704, 42012, 46276, 25260, 7236,  31984, 1224,  19660, 61880,
                12988, 59484, 54312, 16524, 36740, 57028, 33408, 24900, 2504,
                21340, 12832, 56232, 34928, 25144, 13408, 7724,  48280, 26036,
                57824, 20656, 24696, 43964, 31708, 4984,  13664, 33668, 1068,
                55212, 44660, 62680, 32312, 49816, 37000, 17932, 46124, 13292,
                56944, 9872,  3844,  27284, 64688, 7692,  40384, 17044, 20732,
                33568, 53408, 16648, 30456, 57292, 33904, 43532, 59472, 48600,
                33272, 65072, 43252, 50680, 12340, 31080, 34696, 47808, 21528,
                40140, 36236, 65212, 51728, 19112, 47680, 59408, 40636, 26540,
                62480, 57008, 1808,  25692, 65432, 43688, 51864, 24804, 15176,
                32264, 644,   19844, 24644, 41564, 48920, 64296, 32788, 3512,
                50840, 25640, 1120,  44568, 57128, 9524,  3292,  42680, 49136,
                24656, 46296, 56260, 16872, 3460,  52688, 32548, 692,   13240,
                5884,  56380,
            },
            {
                15988, 46528, 12856, 22128, 53520, 44744, 64924, 48604, 5116,
                27948, 56376, 2676,  14732, 63964, 48044, 55432, 28904, 45672,
                6224,  33056, 1620,  47480, 21096, 9396,  4228,  42040, 51120,
                29376, 32648, 47120, 9656,  58468, 37584, 54292, 17760, 30756,
                42848, 4540,  60416, 12184, 38668, 61568, 23056, 40548, 64432,
                9696,  24188, 3592,  15964, 25672, 6876,  42272, 28040, 1412,
                39036, 30144, 59668, 18228, 52200, 31352, 56756, 46432, 1248,
                42424, 6732,  47424, 62324, 3984,  18556, 12640, 5104,  23776,
                10808, 28532, 1324,  59236, 46428, 4772,  64176, 860,   29704,
                56072, 9032,  4544,  30748, 25720, 11368, 34740, 52696, 31612,
                17072, 11908, 21996, 40460, 58756, 9564,  3268,  20972, 57800,
                42720, 53716, 29592, 36500, 55748, 11980, 23124, 45736, 14540,
                34828, 63736, 30476, 16236, 60844, 27244, 14804, 54132, 35112,
                30352, 61440, 6064,  28064, 65148, 23648, 42868, 38900, 48548,
                31324, 37356,
            },
            {
                4704,  27512, 63928, 836,   16480, 26000, 3372,  13892, 58964,
                17328, 10676, 38316, 21940, 34820, 11428, 41960, 4636,  36052,
                51632, 25364, 43164, 63220, 30928, 52640, 60356, 46052, 10984,
                65188, 53324, 23696, 992,   44072, 28216, 2740,  59988, 10200,
                14504, 36604, 51932, 28868, 48404, 7680,  26608, 52584, 16764,
                28696, 38104, 53336, 34936, 11692, 58988, 51448, 22700, 63200,
                16952, 53200, 34880, 12568, 42980, 4452,  24680, 28612, 60504,
                10784, 55892, 13796, 27556, 51408, 36788, 61484, 30940, 41820,
                38140, 55232, 20448, 35180, 8704,  24776, 42440, 14268, 59672,
                26504, 17936, 48784, 60308, 37808, 57196, 5776,  20524, 3768,
                43064, 34408, 54540, 5032,  30096, 49240, 34232, 39104, 61224,
                10596, 47688, 14168, 4948,  17532, 30980, 52900, 8400,  57440,
                40604, 21672, 55364, 5980,  47404, 23040, 40104, 62960, 17572,
                184,   39032, 13696, 49292, 36524, 11444, 17828, 8256,  53772,
                19936, 23484,
            },
            {
                39892, 54448, 8588,  42180, 38060, 51356, 29668, 35328, 40180,
                49640, 31716, 60424, 52168, 26928, 58220, 17852, 23296, 65384,
                14976, 57812, 18644, 7588,  13324, 38812, 26708, 15992, 19956,
                39756, 4656,  17384, 62664, 49632, 20400, 40880, 50812, 32980,
                45960, 62556, 18852, 1208,  15976, 55756, 45268, 35676, 4032,
                48012, 58620, 21524, 46608, 65300, 29620, 4268,  35680, 48580,
                43660, 7016,  23760, 47332, 62292, 10164, 37840, 15068, 50192,
                31556, 22732, 38796, 44752, 8920,  22276, 53728, 47228, 2224,
                17668, 51844, 13636, 26892, 61244, 16640, 52652, 37876, 7516,
                50772, 34076, 42092, 15312, 1164,  28924, 44264, 64236, 49460,
                9056,  59760, 46028, 18584, 14212, 26668, 45656, 17092, 5944,
                27312, 22628, 65000, 44984, 59800, 1692,  39212, 19028, 28680,
                4744,  10608, 42520, 36480, 51128, 33364, 4184,  45252, 10064,
                51272, 58308, 20248, 31408, 44800, 60368, 56708, 27084, 63604,
                44500, 59524,
            },
            {
                34532, 48176, 30588, 60216, 20292, 56408, 9124,  46848, 22668,
                63248, 6900,  43184, 3816,  45484, 8356,  38484, 30660, 49120,
                10136, 39848, 28588, 35776, 55480, 22884, 5560,  50288, 35196,
                7952,  59732, 37080, 30624, 14760, 34200, 5148,  64928, 22340,
                6752,  54668, 24296, 34756, 40976, 63732, 10836, 19872, 60444,
                12740, 42460, 7940,  2220,  41540, 19884, 55320, 12932, 9052,
                26636, 57616, 1784,  39520, 19360, 33464, 59140, 3028,  40924,
                63792, 35248, 1752,  59744, 15860, 40456, 6088,  26292, 57668,
                62196, 7384,  44408, 40000, 56340, 48324, 31720, 20036, 44808,
                3804,  61860, 21740, 9912,  46764, 54468, 23452, 12936, 36824,
                27956, 24072, 51084, 37352, 61684, 52996, 1044,  55628, 31304,
                50716, 40552, 8660,  33304, 50112, 25296, 63000, 43328, 48428,
                61668, 52448, 17896, 26260, 13164, 8468,  59504, 29640, 22244,
                26676, 42372, 7576,  53244, 2640,  21652, 5488,  33960, 15016,
                1856,  10460,
            },
            {
                18816, 13844, 23076, 3976,  33520, 12464, 61964, 18624, 1004,
                53460, 13112, 29288, 20116, 15360, 62504, 54188, 360,   44468,
                21496, 3836,  50932, 61396, 2140,  45096, 63820, 28776, 56896,
                43260, 25668, 46488, 11208, 57444, 54040, 26224, 16500, 39112,
                28104, 42116, 9364,  58944, 31416, 21956, 5464,  29736, 50424,
                32444, 18148, 26080, 57084, 31796, 15692, 38596, 60928, 32940,
                20284, 64316, 14888, 29248, 55916, 48676, 21960, 53968, 18200,
                6304,  26048, 19692, 49496, 29676, 64868, 34144, 20108, 14448,
                29272, 48924, 32476, 2964,  22064, 5240,  10604, 63564, 24480,
                12976, 55632, 32248, 39292, 63364, 17168, 7096,  40968, 61100,
                15104, 1508,  30924, 6744,  11192, 23360, 41372, 63936, 13516,
                35756, 2664,  56364, 20420, 37848, 15240, 6964,  35224, 12708,
                23548, 32640, 608,   57836, 65228, 19792, 38252, 48636, 55876,
                12496, 62344, 34012, 15824, 25916, 40400, 51180, 47396, 29952,
                41700, 52036,
            },
            {
                26812, 62380, 57072, 43864, 49140, 6168,  40956, 27404, 45300,
                33928, 24572, 57652, 36736, 50964, 32968, 25224, 12784, 60912,
                56348, 32536, 24520, 47656, 17244, 41212, 11452, 32124, 252,
                14020, 52404, 21324, 3316,  42284, 8504,  47880, 12312, 58648,
                1892,  49444, 13932, 47000, 3184,  52960, 43340, 38548, 56104,
                672,   62312, 36732, 51976, 10176, 47660, 24364, 368,   53656,
                40700, 46260, 36240, 51764, 5060,  8248,  27468, 45036, 12344,
                52048, 42628, 57880, 4640,  12084, 55468, 45500, 9964,  43004,
                36408, 23080, 11616, 64708, 37476, 29408, 54076, 34768, 40560,
                47572, 28100, 2496,  52556, 25448, 34908, 50824, 31804, 19140,
                56048, 38888, 64588, 44116, 57684, 33548, 19776, 9256,  25428,
                59480, 17884, 43496, 28768, 10840, 58552, 54764, 27160, 2836,
                59356, 38044, 46408, 28968, 43556, 53680, 1660,  16048, 32400,
                5600,  37756, 46256, 55404, 9508,  63180, 36940, 11856, 24148,
                56208, 3616,
            },
            {
                45996, 7472,  36620, 17060, 24924, 54160, 31548, 58800, 15064,
                8556,  39248, 64852, 1688,  48112, 6992,  40644, 17400, 35392,
                5696,  42304, 14304, 9068,  34344, 53860, 20908, 59100, 48608,
                18492, 33584, 64364, 28044, 38212, 61872, 19556, 36296, 52580,
                31096, 63424, 20596, 37536, 26896, 60972, 14516, 25256, 8696,
                22716, 44824, 14028, 28456, 63540, 6328,  43440, 50464, 16800,
                30480, 4060,  11508, 25016, 43560, 16592, 65368, 35664, 30804,
                61520, 9408,  46812, 33148, 38028, 24348, 548,   50884, 63072,
                4596,  54368, 58440, 18680, 45968, 15200, 59220, 16,    16888,
                58096, 8784,  19292, 43756, 12112, 4216,  57116, 44828, 3132,
                47904, 8600,  21252, 15740, 28264, 3880,  48848, 45044, 38696,
                52452, 5084,  63340, 47224, 292,   31836, 21716, 44920, 17596,
                50656, 9156,  15248, 21424, 5016,  34492, 25452, 41304, 64108,
                19460, 2196,  22848, 59696, 29292, 18336, 544,   58788, 20208,
                64564, 37996,
            },
            {
                50136, 11612, 29504, 336,   65156, 37768, 20856, 2732,  49996,
                55568, 42908, 18152, 11744, 28236, 21260, 59080, 47064, 26760,
                52412, 19484, 64452, 57316, 27060, 4336,  38312, 24940, 9452,
                39960, 55660, 7064,  50244, 23756, 796,   32760, 45836, 4496,
                24660, 17012, 55620, 6272,  11300, 49788, 17864, 64756, 47580,
                33540, 53920, 4700,  40324, 21704, 34860, 59088, 26984, 7536,
                61908, 22728, 56924, 60500, 32200, 39840, 56596, 28,    23956,
                38452, 16284, 28184, 22028, 52460, 17484, 60080, 31276, 39172,
                16260, 25476, 1296,  41096, 50404, 9304,  26128, 44248, 22296,
                51508, 30880, 62720, 37464, 59004, 29796, 21376, 10448, 26916,
                53236, 33748, 41668, 50084, 54936, 36468, 62472, 6416,  29476,
                21836, 33020, 12240, 24508, 53196, 38736, 49492, 64784, 40992,
                29664, 56500, 62668, 40236, 11388, 49516, 57340, 9232,  44804,
                52228, 28036, 50068, 13756, 42940, 48800, 32368, 45080, 7972,
                15668, 31616,
            },
            {
                21288, 61096, 41384, 52672, 14328, 47004, 10664, 62476, 35936,
                22408, 4556,  31816, 52240, 56860, 43712, 3064,  63052, 10632,
                30092, 38056, 960,   44876, 31108, 61144, 14676, 43948, 62868,
                4724,  29876, 13168, 44240, 17228, 59148, 29152, 56032, 10760,
                43680, 40332, 33880, 45568, 58204, 32108, 36124, 2808,  40660,
                11580, 60100, 17412, 49900, 2604,  55488, 18720, 10956, 37248,
                42612, 13992, 48832, 2180,  19132, 10144, 47732, 14468, 7156,
                50092, 55708, 1932,  63188, 7764,  41496, 13196, 27220, 7324,
                46748, 34712, 52664, 28048, 6840,  32164, 62840, 38848, 4832,
                35412, 14604, 6324,  23720, 49064, 15592, 65292, 36368, 60576,
                23768, 12600, 62808, 32,    25680, 11804, 17428, 58504, 14764,
                49380, 56064, 40816, 16436, 61920, 5428,  9944,  14200, 3840,
                33956, 7040,  24060, 52980, 31472, 61252, 14580, 23096, 36180,
                60412, 11072, 39904, 7120,  35292, 4612,  55144, 26712, 40804,
                53664, 4964,
            },
            {
                25272, 55792, 34276, 19336, 27980, 7824,  44176, 30288, 16960,
                26640, 60260, 46624, 9024,  24032, 34624, 16168, 39092, 7508,
                55768, 48256, 11976, 22564, 52144, 7716,  49348, 34660, 20388,
                46844, 58160, 35920, 54180, 9360,  39864, 14536, 64672, 22072,
                7932,  60640, 28644, 84,    22620, 42132, 7336,  27824, 57496,
                20896, 29944, 37428, 25088, 45808, 13652, 32808, 64988, 47184,
                54840, 33912, 28492, 38256, 53056, 27008, 62208, 42436, 20572,
                60632, 34216, 11148, 44456, 58184, 35480, 48592, 21596, 54920,
                59264, 19240, 12572, 61540, 36784, 21132, 53668, 11892, 48620,
                60232, 42000, 56096, 46316, 704,   40212, 7740,  42560, 17268,
                46156, 5848,  30564, 18936, 40084, 47028, 32452, 23836, 42476,
                1264,  8764,  35168, 27852, 43848, 20124, 36384, 26232, 54008,
                19440, 47524, 36916, 2364,  18488, 46088, 5972,  29968, 3296,
                17240, 32100, 57532, 25108, 65396, 16120, 22284, 60344, 12896,
                34788, 44236,
            },
            {
                16608, 9260,  2876,  48944, 59920, 23812, 57296, 39768, 3584,
                53312, 13960, 37128, 41268, 63616, 12652, 30984, 51140, 20348,
                24736, 60660, 16444, 36200, 40812, 18812, 1992,  53304, 27684,
                15588, 1468,  25548, 21040, 48340, 4780,  51524, 26856, 36800,
                49904, 15332, 52904, 12396, 63972, 19576, 54276, 50940, 15276,
                43600, 5732,  63228, 10452, 58364, 41980, 29320, 1152,  20864,
                4484,  17704, 8052,  63836, 22068, 5324,  33412, 54652, 30052,
                45316, 26120, 40164, 19584, 29528, 2896,  64036, 4840,  42360,
                9532,  30372, 44144, 48400, 15564, 57200, 1720,  18116, 28440,
                25000, 10028, 33040, 17844, 27976, 54112, 50368, 31388, 1996,
                58700, 38204, 51740, 56776, 8912,  61032, 3344,  53960, 36868,
                65312, 19412, 59768, 3088,  48668, 57860, 30708, 60108, 42288,
                63552, 12304, 27996, 43780, 56040, 25864, 39076, 63528, 48596,
                54284, 43464, 1016,  20032, 52464, 38560, 9648,  50032, 29116,
                1560,  58508,
            },
            {
                64400, 30840, 43456, 38708, 5644,  32712, 51264, 12480, 47916,
                64492, 7056,  29184, 19376, 5420,  53776, 58680, 45360, 1952,
                42512, 33600, 3868,  58284, 26212, 63976, 30328, 10852, 60856,
                37480, 41896, 64064, 30904, 60564, 34616, 42716, 2020,  19164,
                46040, 3980,  25604, 39360, 47912, 30676, 9908,  61616, 23676,
                896,   48320, 32020, 51488, 6676,  22560, 53284, 39384, 50664,
                25372, 59396, 45680, 40916, 50228, 16072, 37060, 11648, 2580,
                17084, 5236,  53788, 13956, 24496, 51416, 32188, 15164, 37148,
                62068, 22868, 3188,  6292,  24408, 41824, 33892, 44868, 64996,
                55068, 2600,  21448, 63184, 11456, 34568, 24436, 13460, 55244,
                28060, 21080, 14916, 34692, 43728, 21744, 50492, 27372, 10768,
                46072, 30232, 51476, 13344, 23428, 6564,  45612, 15960, 1200,
                22512, 50176, 58092, 16312, 9868,  34352, 51828, 19756, 8284,
                26572, 14112, 47236, 61104, 30596, 45416, 3504,  63144, 18964,
                47304, 37184,
            },
            {
                12372, 22176, 53280, 13916, 63336, 17356, 1736,  21364, 34428,
                43544, 24648, 55980, 49476, 580,   22100, 27168, 36460, 14852,
                65116, 28564, 50312, 9460,  46660, 13808, 43276, 56964, 23084,
                5668,  51124, 11644, 7240,  18284, 24424, 12516, 53444, 58704,
                30240, 63136, 34952, 56868, 16876, 3324,  37384, 45840, 33792,
                39264, 56400, 15864, 26680, 36076, 61824, 9388,  15404, 57300,
                31648, 10708, 35260, 13416, 880,   58892, 47032, 24352, 57516,
                49304, 64880, 36528, 46892, 57104, 8584,  18652, 45956, 26280,
                50776, 33544, 57908, 39408, 63872, 50572, 30224, 7996,  14088,
                39996, 36448, 51888, 44196, 59692, 4600,  19748, 62520, 44564,
                9488,  48504, 2972,  64728, 28692, 6208,  39052, 13880, 57024,
                5304,  25044, 41840, 38552, 55120, 34104, 11084, 51172, 39080,
                8632,  32176, 4844,  40476, 62496, 260,   12852, 42316, 57040,
                37948, 22260, 34396, 7776,  12224, 25788, 35472, 40524, 23596,
                7032,  51092,
            },
            {
                27724, 4172,  46656, 25608, 36032, 54532, 42300, 27004, 58676,
                10272, 17208, 33268, 38356, 61404, 41860, 8368,  48708, 11068,
                54996, 6412,  38836, 20872, 54512, 5184,  33940, 39200, 48040,
                19264, 32324, 44636, 55900, 39396, 47188, 62260, 32960, 6200,
                38724, 10892, 23176, 6572,  43460, 60476, 13508, 26220, 6776,
                65252, 11956, 20368, 41080, 1812,  18400, 45352, 27600, 5828,
                42984, 62396, 23748, 56036, 30772, 26656, 41864, 8276,  39236,
                21028, 31744, 9980,  2132,  42136, 61216, 39004, 55524, 10972,
                316,   17432, 53596, 27816, 10620, 19944, 3756,  52896, 23464,
                47160, 5756,  30452, 15340, 39448, 56428, 36640, 6716,  39944,
                32920, 60120, 25004, 11404, 46736, 16932, 31760, 63280, 20516,
                34296, 15796, 61516, 9376,  18380, 64536, 28804, 56460, 24740,
                35568, 46384, 60704, 21216, 30552, 47340, 23608, 59912, 32796,
                4376,  64048, 41716, 55348, 17948, 50944, 57636, 14556, 31644,
                56520, 41856,
            },
            {
                33512, 61828, 58084, 20064, 10524, 29788, 60696, 6396,  38844,
                52072, 2776,  59632, 14936, 46888, 30628, 17476, 62492, 32464,
                23792, 18300, 44956, 30952, 61524, 24588, 17756, 2960,  9188,
                54776, 27052, 59984, 15840, 356,   28896, 8996,  21756, 16348,
                55064, 44492, 18896, 50984, 27708, 54748, 21368, 50212, 17716,
                52572, 29048, 44812, 60804, 54024, 49128, 34312, 63960, 37764,
                48768, 2804,  17020, 51836, 6996,  64444, 18544, 60472, 51984,
                28128, 15080, 59892, 23184, 33876, 27464, 4152,  22164, 64744,
                41588, 47408, 8208,  36264, 13852, 46236, 59560, 35028, 61752,
                19136, 57700, 9436,  25492, 1344,  47840, 27316, 51580, 23016,
                17480, 52972, 42256, 36816, 54372, 58940, 1084,  52120, 43144,
                48064, 2840,  53356, 31680, 652,   43220, 20696, 4000,  62652,
                17516, 52180, 13388, 26160, 53648, 36752, 5816,  16028, 29088,
                10968, 48424, 2436,  28300, 59440, 5368,  43420, 244,   62192,
                10128, 17808,
            },
            {
                6420,  37716, 1128,  40516, 7412,  45312, 49036, 14496, 19856,
                31308, 44680, 22992, 7448,  11624, 25848, 52500, 3204,  40376,
                58436, 51200, 13040, 12,    36024, 52252, 58808, 29460, 65484,
                14308, 35508, 4400,  22420, 36556, 50108, 57344, 40904, 27076,
                48560, 876,   61320, 33332, 9368,  40312, 1376,  31768, 59352,
                36764, 3580,  8320,  32284, 24728, 11424, 3404,  23000, 13052,
                20176, 29384, 39404, 44364, 21300, 33096, 3352,  13068, 34972,
                6116,  43960, 48588, 54592, 17008, 12648, 49224, 35144, 29920,
                14568, 21100, 31572, 62180, 55440, 26400, 38884, 16576, 11196,
                27964, 42296, 50088, 64600, 17980, 31912, 10232, 14416, 64132,
                524,   5576,  30284, 19444, 7784,  23656, 40232, 26136, 9664,
                37208, 58628, 22088, 26740, 50272, 36792, 47500, 14184, 41408,
                28232, 1700,  43172, 7628,  17972, 64660, 49744, 43884, 54864,
                19196, 52420, 15408, 39700, 23264, 32688, 48532, 21120, 26976,
                46092, 54480,
            },
            {
                14684, 48284, 29020, 51320, 65480, 22828, 34940, 8,     62540,
                55160, 27572, 50700, 64304, 34240, 57168, 44272, 21588, 5648,
                35232, 27044, 63896, 42388, 10568, 15160, 44700, 40368, 50284,
                24360, 46616, 41640, 53068, 63696, 12956, 43876, 3116,  64892,
                36116, 13812, 29552, 57628, 15776, 35888, 63908, 43096, 10112,
                23308, 47128, 55552, 14664, 38436, 58748, 30276, 56280, 52560,
                7828,  61452, 54796, 10084, 36400, 49764, 46060, 55316, 24056,
                63104, 492,   38448, 30032, 7360,  62944, 44616, 58584, 7052,
                52720, 60372, 44192, 2240,  22456, 6156,  43348, 696,   48364,
                32616, 4800,  35600, 22104, 45000, 53708, 58596, 41132, 46220,
                35032, 57124, 14044, 63092, 49100, 33516, 12824, 60960, 18096,
                28668, 7484,  45108, 63036, 12052, 5392,  60144, 33140, 54212,
                10120, 59040, 31592, 56284, 39300, 3140,  26716, 8872,  35204,
                62684, 24920, 45020, 36140, 8816,  64408, 11716, 38144, 52316,
                35080, 23860,
            },
            {
                44044, 57248, 18548, 12596, 32892, 16680, 56928, 25384, 41136,
                8296,  35948, 4500,  39872, 20016, 1568,  37168, 60464, 15056,
                45992, 8464,  55744, 22080, 49124, 27632, 6644,  21248, 1092,
                61572, 11288, 7936,  30448, 17420, 25368, 33864, 19940, 7372,
                52840, 23476, 41760, 3724,  49328, 24428, 5248,  47980, 28196,
                16316, 41528, 62764, 21000, 4488,  44276, 16728, 40800, 46204,
                34804, 14932, 24576, 1628,  58252, 15472, 29160, 8848,  42336,
                17220, 56824, 11024, 20556, 51348, 40684, 1304,  18508, 25736,
                37348, 4456,  40124, 17664, 49072, 30016, 58488, 53228, 24252,
                62924, 54740, 13032, 59492, 7272,  37444, 3528,  20268, 29136,
                49968, 25748, 38540, 43692, 3300,  56576, 45620, 5172,  49844,
                55048, 14488, 33728, 39252, 17096, 57088, 25960, 6764,  23280,
                48980, 37376, 20940, 45920, 11360, 33388, 57708, 21444, 40388,
                852,   31636, 6280,  60456, 19672, 53896, 29452, 16644, 3420,
                8188,  60868,
            },
            {
                31164, 2636,  25932, 54788, 43500, 5000,  52140, 10888, 47312,
                18032, 13364, 58112, 48720, 16024, 54116, 28288, 10404, 49396,
                30556, 17764, 4308,  38284, 33156, 63172, 56688, 31640, 37680,
                18672, 33612, 58432, 48688, 3640,  60288, 10228, 56304, 31384,
                45964, 11376, 62304, 20784, 44836, 55864, 12728, 19516, 53956,
                58100, 1644,  34512, 26960, 50232, 64396, 9276,  25908, 412,
                59296, 31788, 47632, 41424, 27028, 61996, 20004, 37088, 51560,
                32128, 26536, 46000, 35728, 57940, 23460, 32512, 53988, 10192,
                48396, 28340, 12468, 51992, 65240, 34392, 10256, 20084, 37600,
                15620, 2432,  40640, 26692, 30120, 16380, 62240, 24184, 6888,
                11720, 60468, 8516,  21844, 28376, 16568, 36512, 22472, 30664,
                65068, 42248, 4444,  52928, 21088, 30572, 46048, 39764, 18832,
                65476, 3576,  15032, 61564, 24044, 52232, 14320, 46680, 61220,
                12276, 50548, 55948, 13924, 46948, 4508,  42780, 57436, 63332,
                40016, 20172,
            },
            {
                11060, 36452, 62256, 8976,  39152, 28176, 60836, 37336, 30340,
                63780, 21864, 43428, 25328, 31236, 6988,  42236, 64872, 24460,
                52888, 41192, 61160, 25556, 2052,  16756, 9624,  51360, 13540,
                44188, 55932, 27172, 20764, 37328, 45496, 51400, 39708, 16432,
                58848, 26376, 38572, 8004,  30300, 60204, 34728, 39084, 6612,
                31444, 11792, 51872, 39920, 6780,  32676, 19456, 37032, 50844,
                21504, 5068,  65036, 12264, 52848, 5328,  40188, 2480,  60608,
                21852, 4944,  65444, 15480, 3096,  27832, 13248, 39108, 63692,
                22752, 57756, 33372, 8568,  25348, 14616, 41332, 6704,  60612,
                30768, 46420, 20480, 52388, 11136, 43924, 51052, 32552, 56216,
                42036, 18020, 53348, 32256, 50988, 10696, 62444, 40712, 212,
                20056, 11160, 25236, 48880, 2120,  62384, 9136,  51924, 12868,
                44080, 35004, 29340, 49612, 1284,  41548, 30468, 4200,  18072,
                28432, 37788, 21976, 26824, 40760, 34168, 24704, 48988, 13500,
                27624, 50852,
            },
            {
                59496, 41812, 21944, 49736, 3448,  20476, 14388, 44952, 2172,
                50308, 33656, 3484,  62064, 12080, 47148, 56252, 19064, 764,
                33640, 57504, 13312, 47364, 54440, 41604, 45904, 60616, 25304,
                6300,  39580, 1604,  14880, 62584, 5656,  28092, 22804, 2088,
                48808, 4712,  34188, 51528, 17296, 2672,  26544, 49948, 63272,
                44972, 24268, 18396, 60940, 13644, 45756, 56980, 61712, 11252,
                28636, 44088, 17692, 35788, 22520, 31280, 10916, 56212, 47088,
                12548, 50296, 41752, 8948,  61388, 47788, 43700, 56512, 16308,
                1928,  45340, 19816, 59724, 37104, 46712, 57020, 28124, 43612,
                50608, 9148,  64044, 33588, 57276, 2684,  39260, 13748, 46956,
                35836, 4384,  63588, 44968, 1684,  58300, 26496, 53624, 46720,
                32884, 55972, 59872, 35848, 43312, 15128, 33460, 27088, 56796,
                5180,  25128, 54504, 8384,  18980, 64208, 36036, 59128, 53464,
                43384, 8304,  65172, 2860,  58088, 10600, 18220, 824,   32484,
                45588, 5472,
            },
            {
                53672, 33580, 15452, 46356, 64276, 34808, 58000, 23884, 7500,
                54620, 27716, 9596,  51916, 36820, 22744, 34636, 13904, 39420,
                5136,  21552, 8992,  29348, 19608, 34308, 23252, 3136,  29916,
                64264, 47600, 52780, 31988, 42556, 55324, 35300, 13124, 43156,
                63168, 19588, 54932, 13968, 65160, 42844, 9864,  22004, 15204,
                36444, 58864, 9112,  47748, 29208, 22772, 3960,  42400, 15612,
                54164, 38564, 56728, 7736,  49652, 45364, 63892, 25600, 16416,
                34468, 28772, 37736, 19696, 31432, 52304, 5272,  8372,  36168,
                29824, 41608, 53744, 6380,  968,   21816, 51472, 3568,  13360,
                18452, 36004, 23612, 5188,  47764, 25936, 19796, 65376, 816,
                22672, 29884, 14864, 39804, 20484, 34220, 6712,  14968, 9020,
                38472, 16400, 28272, 7848,  23156, 54672, 38096, 452,   63268,
                40824, 16688, 60720, 38860, 44904, 27140, 10544, 23388, 6032,
                48204, 32948, 15024, 49876, 30040, 52764, 61904, 37264, 55880,
                22980, 17332,
            },
            {
                1492,  29328, 6852,  25264, 55536, 31064, 9920,  41624, 16824,
                60064, 39320, 18784, 59052, 41488, 2388,  63032, 27752, 44548,
                48904, 62548, 37256, 51520, 6220,  59228, 14744, 36696, 54660,
                9332,  17016, 22592, 11800, 24928, 8828,  18572, 59688, 30012,
                36552, 10432, 28764, 40656, 47164, 31948, 53576, 57220, 1300,
                29856, 4792,  53316, 37712, 1028,  54976, 35556, 31060, 8100,
                48868, 2608,  24040, 59404, 14724, 34172, 19244, 42756, 7300,
                53592, 1572,  59124, 55240, 25212, 17276, 34564, 21676, 60028,
                50640, 13560, 26140, 63912, 40356, 16784, 31528, 63244, 25664,
                58556, 304,   42456, 61364, 17372, 37068, 8432,  59368, 27280,
                54704, 48748, 57780, 25268, 12008, 47724, 42396, 60756, 23588,
                51312, 3620,  64128, 47696, 12120, 58884, 17924, 49468, 21336,
                30668, 47180, 11696, 32836, 5048,  50868, 56744, 16156, 39752,
                25612, 60188, 18932, 44928, 39220, 7024,  21308, 43524, 9116,
                64960, 38948,
            },
            {
                48368, 57304, 43176, 11916, 18232, 1220,  52948, 48436, 26316,
                36028, 13176, 45688, 29568, 5852,  17184, 52612, 9900,  58624,
                30948, 16504, 26148, 42696, 64580, 11256, 40028, 50240, 19916,
                43140, 35176, 58932, 38288, 50264, 64936, 840,   47300, 52728,
                6488,  57356, 21708, 220,   24536, 5168,  19148, 39624, 48312,
                43796, 25920, 20228, 41968, 65412, 17320, 51256, 25376, 63360,
                19184, 33276, 27596, 41648, 4512,  54244, 36,    28168, 60092,
                39528, 23116, 45596, 10800, 3788,  42348, 64556, 27048, 47280,
                4588,  18592, 32576, 10832, 48948, 54924, 35408, 9400,  45520,
                39056, 53552, 28628, 14364, 55560, 30344, 51604, 44384, 11340,
                40556, 5456,  9096,  37484, 53008, 64760, 18260, 30128, 35152,
                57188, 41612, 19072, 31728, 39876, 3224,  29124, 45332, 10496,
                6376,  51820, 23200, 59032, 14148, 21152, 42932, 180,   34800,
                55092, 10952, 1920,  35780, 23824, 58876, 15660, 28336, 51388,
                12872, 25792,
            },
            {
                8500,  20372, 63056, 51696, 40008, 36488, 61392, 20224, 5992,
                64980, 480,   49680, 21584, 56888, 32692, 47704, 24120, 6660,
                54236, 11968, 3340,  46232, 31692, 740,   27896, 58076, 4168,
                26296, 62200, 2428,  6516,  44936, 27328, 33256, 40280, 15804,
                25528, 44664, 49772, 61356, 37512, 59420, 13016, 33576, 63868,
                11176, 60236, 32608, 14372, 27840, 11576, 6164,  39480, 57488,
                13308, 46732, 62212, 10400, 30236, 61348, 37596, 51316, 9668,
                18480, 64228, 14868, 33168, 48776, 58040, 13752, 38272, 10236,
                55500, 61620, 38700, 44720, 28752, 23652, 4880,  56524, 20520,
                32624, 7624,  49840, 10532, 40408, 5792,  21748, 33068, 16076,
                34792, 19280, 62224, 31696, 2720,  27252, 5660,  49512, 1772,
                13236, 26392, 45164, 5216,  60496, 50896, 24772, 36768, 61656,
                34000, 56024, 2516,  28580, 36248, 61892, 47856, 31052, 64012,
                20332, 50980, 28976, 63460, 12592, 4232,  48056, 35316, 2488,
                31688, 59860,
            },
            {
                37096, 15052, 34136, 4620,  27104, 45400, 13532, 29104, 44136,
                32392, 55996, 25204, 8716,  61628, 14228, 37720, 42820, 18724,
                35952, 60768, 22644, 56468, 19128, 53332, 16164, 47924, 32792,
                12840, 46760, 29520, 54156, 19604, 13604, 57592, 22384, 3800,
                63800, 12668, 34372, 8496,  15904, 51660, 27436, 6908,  23024,
                17492, 50816, 7288,  56084, 45256, 60772, 48448, 21232, 43472,
                784,   36616, 51884, 16852, 21072, 47772, 23488, 13848, 44588,
                30752, 50028, 5960,  37028, 28256, 20168, 924,   45084, 31336,
                22576, 2612,  52540, 7600,  15156, 62560, 42164, 12528, 48336,
                16040, 64952, 22724, 35084, 60196, 46816, 63012, 3488,  53296,
                58388, 46308, 50676, 22332, 15856, 45284, 59160, 38904, 21204,
                62952, 9604,  54348, 22192, 34552, 13544, 7552,  57536, 15280,
                42096, 19324, 38500, 46336, 54388, 4092,  9120,  27692, 13668,
                42192, 7492,  46424, 56100, 40788, 32584, 54012, 62484, 17880,
                46436, 41536,
            },
            {
                3032,  50524, 29964, 60156, 8736,  22292, 56812, 2888,  51568,
                11236, 16336, 40948, 35608, 44000, 3672,  27064, 65380, 1456,
                50340, 40872, 33688, 8388,  38088, 24808, 61076, 41412, 7904,
                21648, 56876, 15892, 34236, 41144, 61460, 9732,  51440, 37928,
                31120, 55720, 18856, 41336, 30200, 46116, 56152, 43092, 37180,
                54712, 2652,  35460, 40248, 24116, 1956,  33772, 29036, 9952,
                54548, 23348, 6944,  43944, 39128, 57704, 5400,  33104, 56148,
                2356,  25676, 41992, 61492, 53796, 8484,  50540, 62660, 16216,
                42580, 35856, 25432, 59400, 19312, 1340,  36776, 29652, 60332,
                2560,  27288, 44324, 19044, 1448,  25644, 13800, 41952, 28960,
                23932, 1260,  12692, 41680, 56372, 32804, 10424, 54056, 44084,
                29096, 37112, 49312, 16268, 62576, 43136, 52932, 20892, 1124,
                27040, 65048, 8300,  16388, 24796, 40544, 18848, 52600, 58404,
                3564,  37960, 21720, 16492, 26448, 8856,  43168, 24556, 6424,
                55580, 21896,
            },
            {
                10684, 44472, 23684, 54856, 47288, 16736, 63888, 34600, 39444,
                23372, 62148, 53292, 30336, 19268, 49392, 55204, 11548, 21640,
                29168, 14948, 47104, 63120, 13520, 44504, 4676,  35584, 65168,
                51576, 39116, 10444, 23880, 49228, 3084,  28872, 17892, 46944,
                6804,  42996, 26492, 2344,  64480, 20356, 10636, 4160,  61588,
                28224, 46588, 21056, 9832,  58208, 18524, 37924, 63904, 16188,
                59680, 26952, 31184, 65108, 2824,  12064, 26532, 63376, 36284,
                20640, 58536, 11584, 17076, 22648, 40512, 34784, 25080, 57164,
                6512,  49692, 12900, 33776, 56768, 50856, 45308, 24684, 52784,
                38428, 5332,  58016, 53812, 32072, 37624, 50408, 18124, 8668,
                64468, 38600, 26660, 61428, 7844,  36268, 24440, 14332, 17920,
                6132,  58560, 1432,  30960, 10416, 27728, 38652, 48068, 31532,
                44784, 12696, 50652, 59628, 32088, 11528, 62980, 35236, 45444,
                25292, 31844, 61648, 52332, 720,   60040, 19888, 38460, 13864,
                27744, 63600,
            },
            {
                53276, 17348, 39800, 192,   12200, 31392, 41904, 7460,  19084,
                48096, 6544,  2100,  12924, 58412, 7148,  32024, 39096, 59576,
                53492, 5588,  26600, 2176,  52484, 28780, 10788, 23240, 17988,
                30132, 64,    43196, 63756, 6952,  36680, 44348, 54900, 24164,
                62420, 11284, 52380, 58176, 6024,  35968, 24892, 50132, 16056,
                31580, 12880, 64248, 49412, 30636, 53268, 5504,  50808, 41280,
                3664,  46292, 14260, 53876, 33964, 49420, 17544, 46584, 7252,
                39784, 52296, 47468, 3312,  31000, 55596, 4424,  12188, 29176,
                20644, 65288, 46208, 30080, 40044, 21860, 10748, 6540,  18048,
                33976, 41516, 14808, 47544, 11708, 7044,  59024, 54432, 36060,
                44668, 6100,  30908, 17644, 46076, 4108,  63660, 41440, 51716,
                33604, 46708, 25096, 40936, 55040, 3052,  17500, 63488, 4740,
                54232, 22172, 35432, 1836,  43400, 48380, 22556, 6284,  17124,
                50148, 10108, 14120, 34772, 48352, 30164, 11220, 50768, 58264,
                35628, 30988,
            },
            {
                7580,  57904, 26836, 62252, 37988, 50220, 25356, 53852, 60000,
                27848, 37344, 46164, 26256, 64020, 22736, 45708, 16584, 8580,
                32944, 43720, 57884, 20708, 49168, 32096, 60160, 55164, 48224,
                5356,  59276, 27144, 55644, 20744, 31900, 59092, 14420, 1056,
                33352, 20956, 37408, 14688, 48296, 32268, 40752, 58924, 1632,
                38528, 42464, 25820, 4536,  15460, 43084, 12440, 22852, 32560,
                19548, 35972, 58320, 8448,  20336, 41140, 29628, 54940, 10220,
                15596, 28424, 33936, 64040, 44556, 15084, 61116, 46924, 52992,
                38332, 224,   17724, 9320,  4264,  63612, 32540, 55852, 61952,
                48812, 9456,  23844, 29180, 63656, 43360, 22356, 31436, 11204,
                20492, 56936, 49552, 52872, 21396, 55528, 27820, 404,   60388,
                22416, 8804,  65532, 19812, 45392, 58296, 23712, 33908, 9528,
                60168, 39532, 29284, 5676,  54988, 27184, 57512, 39276, 2508,
                65528, 41356, 57920, 5584,  44380, 22972, 64680, 2380,  45052,
                4812,  48092,
            },
            {
                42108, 13552, 32900, 20632, 6664,  58368, 3652,  14596, 10160,
                33296, 56128, 17552, 51880, 34348, 42056, 808,   25544, 48160,
                63568, 17996, 10320, 35348, 39316, 16620, 3468,  41776, 36924,
                14288, 33692, 46140, 16508, 11532, 50352, 26108, 8936,  40536,
                60084, 49164, 28592, 44292, 23004, 62316, 12360, 52624, 19628,
                47812, 7912,  54356, 60580, 34340, 27132, 59120, 8008,  47988,
                62624, 11268, 42888, 24152, 51116, 1788,  60432, 22492, 43316,
                61800, 528,   23952, 38400, 9672,  27072, 19120, 36392, 8340,
                60076, 42124, 26772, 54408, 47864, 15336, 42896, 28144, 13372,
                21360, 55056, 60748, 2420,  39344, 16136, 27352, 120,   47992,
                59852, 14564, 2204,  35020, 11464, 38820, 48348, 30648, 12348,
                37956, 56288, 14716, 32588, 5572,  12288, 36516, 51516, 14816,
                25312, 47008, 17344, 62692, 20100, 14220, 8212,  33324, 53300,
                29992, 19732, 26948, 37456, 17916, 55988, 39612, 32888, 16392,
                25508, 19716,
            },
            {
                64252, 2000,  51108, 44048, 36132, 18320, 46264, 30392, 42760,
                63368, 21724, 5300,  39704, 9296,  14180, 57240, 51432, 36520,
                3508,  41112, 24600, 55476, 64420, 7864,  45384, 20052, 26100,
                62928, 8376,  53212, 38248, 1768,  41740, 62700, 35288, 53816,
                16920, 3948,  9316,  54828, 628,   17464, 8252,  27492, 34524,
                62708, 23508, 17608, 37496, 2316,  45560, 55184, 39328, 25276,
                52052, 1240,  29068, 38508, 63784, 14652, 37008, 4928,  31044,
                49548, 18776, 59116, 5912,  56856, 49132, 41728, 3680,  23376,
                31404, 11840, 58052, 35516, 23580, 59532, 1592,  37748, 46112,
                3808,  30944, 36232, 18372, 49740, 56440, 4968,  61804, 40532,
                33820, 28280, 42212, 24944, 60736, 16276, 8060,  19472, 44940,
                50044, 2404,  28260, 52488, 39504, 49196, 61796, 28956, 43724,
                6604,  57796, 10376, 41220, 51396, 37432, 61024, 44320, 12192,
                23660, 47412, 8444,  53012, 3172,  13436, 27608, 8680,  54184,
                59740, 38232,
            },
            {
                34588, 23556, 55260, 9560,  28212, 64828, 22996, 38728, 50712,
                656,   12452, 44360, 60684, 28500, 20152, 62028, 29656, 12412,
                21848, 60656, 30868, 1680,  13836, 27868, 53864, 58452, 11144,
                50008, 23984, 18952, 30672, 58132, 23260, 4480,  19428, 29776,
                45876, 24784, 65360, 35148, 41652, 30100, 55680, 45224, 4988,
                56700, 30404, 11544, 48680, 21644, 64840, 18756, 4348,  30724,
                15600, 57176, 18264, 47372, 6864,  27148, 57332, 46096, 12164,
                35200, 40832, 53368, 13732, 21488, 32432, 52044, 64256, 15872,
                45636, 50908, 19416, 5996,  39364, 50520, 17632, 25744, 64748,
                7456,  51236, 44304, 25464, 8140,  33380, 45524, 52068, 23704,
                17260, 8956,  65040, 4412,  43804, 32244, 57216, 64148, 25588,
                35088, 17676, 43000, 59684, 22084, 8288,  18448, 160,   37976,
                20548, 53204, 30864, 34328, 1320,  24396, 28324, 17816, 56544,
                936,   59656, 42636, 63640, 33780, 49032, 61504, 41636, 22116,
                49796, 11104,
            },
            {
                46492, 29800, 61496, 15560, 48700, 2596,  54524, 7608,  16984,
                58244, 25412, 31828, 54844, 2696,  47360, 35000, 4668,  44632,
                53364, 15364, 48480, 42968, 50704, 23020, 34544, 1000,  31148,
                39068, 5896,  43932, 64504, 13188, 47568, 52816, 43352, 6672,
                38756, 57832, 13412, 19532, 50904, 61192, 37076, 22056, 14904,
                43436, 1052,  52040, 41296, 32436, 9284,  14076, 36832, 61436,
                45040, 6372,  34924, 54700, 10872, 33220, 17312, 52680, 25220,
                65448, 8172,  27836, 46632, 35916, 1192,  12356, 25784, 56268,
                34612, 2976,  63452, 28528, 12660, 31748, 8836,  54108, 33420,
                39940, 15736, 58392, 10964, 64268, 20384, 37660, 12228, 6624,
                55944, 37036, 46820, 54336, 22272, 51816, 2920,  40900, 5492,
                54124, 10740, 62848, 4244,  26312, 47044, 31396, 55292, 64888,
                27248, 3428,  48952, 13008, 64372, 46056, 4920,  49180, 40068,
                35644, 15768, 31116, 11660, 24988, 19976, 45468, 416,   30612,
                14860, 3760,
            },
            {
                57184, 18056, 5688,  40848, 32348, 12800, 60276, 34228, 27648,
                47676, 36652, 15180, 50052, 23232, 38424, 16948, 55964, 8728,
                38808, 26688, 5740,  36220, 9372,  60392, 40864, 17588, 47208,
                61092, 56092, 3148,  35844, 26436, 9300,  33180, 15496, 63428,
                11344, 49836, 32524, 26840, 6948,  2924,  47252, 10480, 64668,
                39028, 26476, 59932, 6792,  57868, 28972, 49808, 53756, 22504,
                40912, 26452, 62976, 21004, 43288, 59556, 40060, 3632,  20360,
                55828, 2108,  16624, 43120, 62412, 57864, 29000, 7320,  38768,
                21092, 10020, 43324, 53792, 47416, 61672, 22308, 58024, 11772,
                19924, 28640, 748,   40972, 53120, 30444, 15172, 26508, 63100,
                49356, 19396, 30056, 13860, 10244, 27268, 36464, 15116, 58652,
                22880, 31896, 38524, 13316, 34716, 57924, 15656, 40580, 11420,
                45204, 61128, 16892, 42428, 22148, 55192, 9652,  31800, 63356,
                7216,  21908, 50528, 4288,  56396, 35996, 7000,  58792, 37400,
                62772, 26732,
            },
            {
                9060,  39240, 22012, 53652, 44992, 24936, 19804, 41420, 52132,
                9844,  3444,  65236, 40920, 7528,  11364, 63764, 24076, 32708,
                59036, 18760, 62676, 56488, 20156, 29420, 5264,  52876, 12928,
                21572, 28708, 16272, 51040, 40316, 21044, 60792, 28452, 54960,
                22784, 1636,  37616, 44100, 59328, 16228, 25164, 31940, 50584,
                19392, 34176, 13172, 16708, 24244, 44096, 732,   33212, 10732,
                3336,  13440, 49560, 31224, 152,   23764, 50160, 9496,  44512,
                30252, 33596, 51152, 24484, 9212,  18660, 47664, 41380, 61700,
                49364, 30684, 24952, 15072, 1036,  41204, 5312,  43244, 46884,
                52708, 60428, 34540, 22780, 46612, 3284,  59944, 43572, 32352,
                2044,  41448, 5824,  61604, 39584, 49624, 20540, 47356, 29256,
                8408,  45944, 50820, 20440, 43664, 2820,  49688, 22856, 7248,
                33796, 24560, 37176, 4448,  29436, 59348, 15148, 20416, 53828,
                26844, 46300, 38000, 62168, 29108, 15256, 52776, 10668, 18656,
                51648, 43832,
            },
            {
                35048, 49500, 60540, 568,   36580, 10332, 56908, 4736,  62008,
                22232, 45408, 18972, 29372, 57360, 52724, 27704, 42340, 50604,
                144,   46988, 11520, 33100, 45548, 15028, 65340, 36720, 43012,
                8892,  33984, 45880, 7404,  56732, 1900,  48732, 36136, 4284,
                46572, 17548, 55928, 9168,  21320, 40504, 54068, 58612, 7644,
                3608,  61868, 45024, 54780, 38340, 62704, 19988, 56684, 47888,
                60148, 37420, 53452, 8184,  15948, 64340, 28556, 35440, 61448,
                14472, 39484, 60436, 4300,  38260, 31460, 53740, 14048, 1744,
                17448, 55520, 59840, 38076, 34300, 18252, 27880, 36244, 2760,
                24628, 6760,  49068, 13212, 56748, 6192,  39664, 54196, 9916,
                21620, 58708, 25172, 34072, 56940, 1156,  63024, 12040, 42264,
                60408, 612,   56188, 28488, 63980, 9968,  36324, 59848, 52112,
                14384, 54648, 58064, 8684,  50836, 39088, 34608, 43636, 12664,
                2928,  57888, 17952, 9444,  43736, 40344, 22584, 47640, 32600,
                24312, 4572,
            },
            {
                11964, 30816, 14452, 27376, 64184, 50140, 30376, 15788, 38308,
                32696, 59204, 12316, 35488, 1436,  45684, 20000, 6436,  14036,
                29960, 22776, 40080, 3648,  54128, 24544, 48648, 2880,  26664,
                62416, 53964, 24316, 59876, 30568, 18408, 42800, 13492, 25400,
                41344, 61772, 27916, 52156, 63348, 29748, 12444, 35988, 42724,
                28508, 48128, 21820, 2548,  35008, 5320,  27020, 15900, 42672,
                28524, 18008, 24372, 45744, 38828, 54288, 12720, 5636,  47248,
                22988, 57412, 10924, 20800, 45848, 64772, 5196,  23640, 36568,
                45296, 8708,  4088,  21412, 65176, 48816, 56344, 14672, 62464,
                31424, 16812, 63420, 36596, 19100, 29544, 24100, 17108, 47520,
                35852, 51156, 15664, 45640, 18636, 7524,  32720, 52288, 35252,
                25808, 18944, 15308, 6800,  53172, 25008, 17388, 29780, 42936,
                1268,  20060, 31848, 45536, 18280, 25728, 344,   60808, 51684,
                41396, 33232, 24400, 54240, 1144,  60740, 26932, 55784, 2296,
                40796, 65424,
            },
            {
                20432, 55960, 46580, 7652,  17768, 42564, 2208,  46776, 25900,
                6008,  54080, 43408, 24996, 62508, 16044, 33456, 60752, 37504,
                55084, 64060, 16968, 27176, 42324, 8160,  57324, 31764, 18476,
                49416, 328,   14408, 37856, 11556, 65064, 53600, 8280,  58336,
                31832, 5512,  35056, 14396, 148,   45592, 6452,  18164, 23536,
                56968, 9612,  30820, 50440, 12060, 52660, 39948, 8756,  65164,
                5748,  34420, 58532, 2716,  32716, 19788, 57840, 42332, 18016,
                3408,  48940, 29164, 54304, 34760, 16064, 27320, 59428, 52256,
                28360, 33092, 42080, 51892, 11316, 7136,  23688, 51184, 9200,
                39060, 42260, 26980, 9852,  44220, 51392, 65296, 888,   27792,
                62016, 11760, 3156,  53732, 28464, 43996, 23848, 16544, 4708,
                65012, 37288, 41392, 47588, 33192, 39820, 61368, 5616,  47816,
                26384, 62280, 40172, 12308, 64096, 6688,  47784, 23396, 29244,
                7876,  64944, 13980, 48900, 20256, 34504, 5860,  13208, 59884,
                16244, 52528,
            },
            {
                6408,  38632, 24092, 58796, 34704, 21884, 55540, 63308, 9464,
                49888, 16844, 31212, 4192,  51584, 39224, 10440, 49232, 2756,
                44012, 9728,  51024, 59924, 34248, 13680, 21800, 38916, 11172,
                58512, 40908, 20444, 44756, 4124,  33852, 22328, 38680, 50204,
                20044, 10752, 47744, 23436, 38908, 33252, 48828, 64320, 52860,
                14236, 40304, 17280, 63856, 24828, 58956, 46912, 31632, 22740,
                51896, 12024, 40472, 62844, 10268, 48008, 24992, 37292, 62544,
                26908, 7920,  41644, 13404, 324,   50252, 43508, 10540, 20076,
                63496, 13504, 57560, 26360, 30484, 44368, 33524, 59312, 19760,
                45928, 54848, 1904,  57792, 32292, 7396,  38280, 13972, 56056,
                42740, 31320, 39184, 64572, 9632,  37708, 59764, 54600, 49208,
                10932, 30028, 57964, 22468, 12760, 1888,  55388, 21076, 38180,
                10456, 49524, 3968,  28292, 52244, 35912, 56456, 10716, 17040,
                36900, 45868, 4004,  31040, 38636, 63876, 44732, 30348, 36880,
                46180, 27800,
            },
            {
                34008, 43940, 3256,  51308, 11492, 39572, 28512, 13884, 35908,
                41816, 21560, 56816, 8392,  46864, 22900, 28504, 57640, 21012,
                25696, 31484, 6700,  37120, 972,   61384, 51832, 45976, 6240,
                35228, 29932, 62968, 25872, 51792, 46676, 27488, 844,   16200,
                63748, 43304, 59388, 51276, 19024, 56752, 4492,  26344, 37460,
                2868,  60584, 33676, 42068, 7176,  19288, 14000, 35992, 1664,
                43908, 55492, 21156, 27216, 50752, 6960,  30824, 1064,  52496,
                33208, 56192, 36948, 62852, 25496, 57740, 32660, 7356,  40616,
                2188,  46696, 17984, 4892,  60776, 40184, 88,    13116, 29428,
                4680,  22476, 50184, 15772, 61232, 21256, 46540, 34416, 8524,
                19860, 4808,  26060, 21700, 48036, 13936, 31172, 1796,  21440,
                44844, 7760,  52480, 4128,  63456, 45552, 31552, 13920, 58324,
                35156, 16632, 54224, 22684, 42480, 14424, 31156, 62532, 40520,
                21292, 59736, 27416, 51920, 11808, 8360,  17800, 50600, 22220,
                9584,  58304,
            },
            {
                18384, 63920, 15272, 31584, 61508, 6268,  52524, 19600, 60268,
                280,   27368, 64496, 36416, 13824, 61480, 43060, 5036,  35456,
                14872, 47124, 56696, 19948, 44388, 29388, 17204, 64004, 23752,
                53556, 16148, 9228,  56004, 7008,  14092, 61456, 56616, 29904,
                36348, 25772, 2800,  8492,  28404, 61200, 11560, 44112, 31780,
                8120,  47592, 26868, 432,   45564, 30220, 53952, 60508, 49296,
                16748, 29720, 5224,  15092, 41748, 59968, 16548, 44740, 11776,
                21296, 15512, 45172, 18864, 6040,  38804, 21980, 48508, 56444,
                34896, 24132, 38592, 49244, 15492, 20668, 55868, 47212, 36960,
                63680, 34100, 12284, 25912, 41176, 4408,  53092, 24864, 59476,
                50560, 46280, 60936, 35360, 57400, 5904,  41984, 26940, 38384,
                61916, 14444, 34388, 27120, 19088, 50304, 8332,  27812, 44424,
                64700, 7124,  32860, 61188, 1668,  20132, 44908, 2828,  50128,
                5348,  55392, 16128, 42060, 58420, 25752, 54904, 3884,  62864,
                41924, 1864,
            },
            {
                53508, 29252, 8576,  48028, 24868, 43056, 2936,  33128, 45240,
                55004, 11296, 40276, 19092, 32156, 1104,  54456, 18272, 65140,
                52096, 39964, 12336, 24308, 49828, 10288, 4420,  41756, 27636,
                1600,  48072, 42544, 36080, 32008, 19056, 39900, 43952, 5176,
                12828, 54640, 40204, 32224, 14796, 41524, 22200, 55088, 16656,
                20888, 52088, 57844, 22912, 55460, 37972, 4252,  10848, 25520,
                62336, 38108, 46340, 56448, 34256, 22496, 53972, 39388, 65128,
                49432, 2616,  60628, 31428, 52760, 12460, 64308, 16556, 29896,
                11572, 62232, 54576, 9044,  35800, 64300, 24872, 52712, 17396,
                7756,  60228, 38512, 47936, 30104, 11152, 63504, 17540, 29212,
                40600, 12564, 16652, 196,   50888, 19436, 62784, 55588, 17496,
                48412, 24324, 43476, 59488, 36632, 41044, 23356, 51964, 428,
                18704, 25552, 46724, 39192, 9172,  59180, 26888, 54464, 32816,
                25176, 9792,  33872, 2228,  22692, 46824, 39992, 35384, 14128,
                26264, 49096,
            },
            {
                22660, 40308, 56076, 20128, 37140, 57900, 16580, 49004, 23816,
                30580, 4592,  47616, 52568, 59152, 24652, 11936, 38240, 8112,
                29112, 59412, 3092,  62244, 33044, 54956, 37940, 59292, 34016,
                12744, 57452, 21624, 3240,  59948, 48996, 23796, 9904,  51624,
                21400, 47272, 17600, 65272, 52884, 36192, 1188,  49404, 62876,
                39216, 11248, 35696, 14660, 8928,  64500, 27968, 41596, 20584,
                8048,  32344, 476,   64168, 11432, 3876,  28540, 8036,  24748,
                35512, 29040, 10032, 23500, 46684, 1872,  27612, 42872, 3764,
                51752, 19384, 31812, 1576,  43816, 28688, 5244,  10628, 42904,
                27972, 51096, 18988, 680,   55820, 33696, 43764, 2468,  37244,
                5652,  54868, 33036, 43488, 24688, 8192,  34620, 11280, 2752,
                32508, 5516,  55832, 9988,  2360,  15968, 62172, 33824, 55936,
                42004, 12560, 57612, 29496, 16000, 48388, 37108, 11984, 18696,
                43988, 63592, 47552, 36984, 61676, 6936,  29756, 19832, 56272,
                32912, 10936,
            },
            {
                62100, 35596, 828,   45604, 12508, 27640, 65208, 7660,  13256,
                38608, 62736, 16100, 7160,  28936, 41568, 50488, 46132, 33196,
                21792, 6528,  17628, 41404, 26972, 14728, 22092, 8560,  49248,
                18788, 63508, 38440, 28732, 11704, 53836, 2400,  31036, 62552,
                34028, 58124, 5968,  24016, 45192, 7220,  58984, 25828, 30140,
                5340,  43216, 61956, 19040, 50708, 44040, 16364, 34848, 47328,
                58616, 50900, 18772, 24528, 36684, 48468, 61660, 18356, 56844,
                4976,  42792, 55340, 39908, 58732, 35732, 50164, 60788, 37964,
                6928,  45432, 26724, 59176, 12768, 50448, 33084, 39928, 58356,
                3536,  23936, 44340, 9080,  59288, 20288, 49856, 14144, 58080,
                23444, 63788, 10124, 28248, 53628, 38892, 46216, 29612, 51856,
                60840, 39744, 18268, 30384, 46348, 54104, 11972, 38868, 4652,
                21924, 49372, 35660, 5376,  53020, 23940, 65468, 7276,  39712,
                57664, 14116, 28848, 52920, 16972, 12352, 51644, 108,   59588,
                47232, 4648,
            },
            {
                13496, 50688, 17336, 62904, 5072,  53340, 35072, 41576, 57068,
                51144, 26468, 22000, 44684, 35060, 3320,  15712, 58128, 26184,
                54176, 44244, 35212, 48148, 1656,  64836, 45772, 52124, 30216,
                4136,  44060, 24632, 47400, 17152, 64876, 37016, 15324, 45600,
                27720, 996,   37936, 12324, 29532, 19752, 10156, 34400, 15140,
                46928, 1800,  27628, 32316, 3392,  24036, 59748, 52616, 2448,
                14404, 39956, 9404,  44464, 52440, 13632, 42172, 32516, 46936,
                15124, 51556, 19928, 7100,  13988, 18084, 8752,  25024, 21472,
                55304, 14956, 41200, 22664, 53768, 17772, 62768, 22132, 14892,
                55136, 32388, 64896, 16264, 36540, 24304, 27804, 7780,  45664,
                30884, 18576, 47844, 60044, 14024, 21228, 65500, 15820, 42876,
                22180, 12888, 49724, 64092, 21476, 26460, 6968,  30896, 60248,
                28480, 9580,  63440, 20396, 42992, 33804, 3596,  51444, 29724,
                22460, 1136,  5892,  20740, 41012, 32440, 65120, 42880, 16252,
                38388, 28460,
            },
            {
                57720, 25852, 33384, 41016, 23380, 31072, 10312, 21432, 17832,
                2408,  32976, 60380, 10076, 55300, 63804, 20344, 5152,  10516,
                63304, 14060, 51224, 30500, 57748, 19492, 6216,  25192, 36988,
                15388, 53252, 9756,  6472,  32628, 43104, 25812, 57408, 7872,
                19536, 41656, 61152, 50800, 56160, 48492, 42836, 64172, 53308,
                23800, 55872, 58896, 40748, 48528, 11884, 30836, 7060,  22484,
                29220, 54760, 26776, 60292, 30796, 21180, 1336,  58916, 10576,
                37556, 61932, 25952, 33856, 63608, 44280, 31972, 57600, 12292,
                34652, 65304, 48856, 4280,  37796, 8080,  46980, 2068,  36184,
                48624, 12612, 39568, 52952, 6060,  41800, 61764, 54308, 39356,
                51484, 1524,  36272, 41164, 5812,  2500,  57112, 25764, 6860,
                58540, 27660, 4028,  34932, 41660, 57232, 48104, 17752, 43656,
                51020, 15648, 40064, 1032,  56352, 13332, 17824, 45768, 60420,
                36304, 49220, 55884, 60872, 26876, 46152, 8780,  25032, 21684,
                6252,  44252,
            },
            {
                20280, 9600,  6680,  60012, 51496, 1980,  46976, 60976, 44208,
                5948,  36836, 48540, 14032, 39196, 23788, 31676, 48968, 36424,
                40620, 3556,  22924, 9492,  38920, 12632, 43124, 60544, 55888,
                33244, 62108, 40500, 58908, 51544, 20380, 5352,  39508, 54716,
                48052, 10584, 16512, 25396, 33116, 2048,  39472, 17680, 6796,
                37296, 13136, 20988, 34776, 6084,  63256, 37656, 56516, 45380,
                65524, 35192, 3348,  16412, 6036,  55448, 39648, 27112, 22568,
                3716,  30912, 45892, 864,   50796, 28024, 3988,  47204, 40328,
                388,   29296, 9848,  33680, 61536, 30516, 43536, 26020, 59612,
                29828, 5040,  20992, 28652, 47156, 2012,  11872, 34064, 15708,
                21964, 9140,  26320, 55956, 31752, 50172, 35284, 10148, 47832,
                33236, 38124, 53392, 9028,  14972, 1060,  37188, 65112, 24024,
                3168,  58880, 32260, 24860, 47836, 62360, 27336, 41560, 9888,
                15676, 33060, 42708, 11168, 37640, 4024,  55272, 50040, 36340,
                60964, 54316,
            },
            {
                39432, 48964, 29488, 43448, 15524, 55952, 37732, 25156, 29064,
                53152, 58012, 19528, 28112, 1480,  52212, 42916, 59664, 27564,
                19204, 56316, 61820, 25948, 53832, 34360, 28572, 620,   11224,
                21352, 2772,  27872, 13776, 35820, 68,    61688, 12972, 22908,
                30108, 64356, 36396, 4732,  58592, 13996, 22680, 60712, 31908,
                49892, 8760,  46012, 16704, 53760, 26576, 18700, 13676, 40944,
                10344, 19724, 49016, 42856, 63408, 8876,  34984, 50156, 64620,
                54256, 17416, 12204, 56656, 38324, 20340, 53912, 16212, 61948,
                51352, 18604, 25704, 57044, 15556, 20796, 55496, 11324, 18628,
                52116, 45724, 9684,  56832, 63196, 31956, 19352, 58660, 4728,
                64808, 42496, 60860, 16908, 23112, 44896, 18808, 40540, 61704,
                1356,  19736, 45344, 62328, 25180, 51852, 32936, 8168,  13456,
                53992, 36352, 45244, 10860, 30268, 7832,  21580, 55060, 2744,
                24584, 63884, 7152,  18664, 23440, 59012, 14844, 28888, 10856,
                31600, 2764,
            },
            {
                13928, 64804, 22024, 36148, 11488, 19504, 7200,  64180, 12216,
                15888, 41964, 8544,  61652, 46472, 6128,  17300, 12736, 2096,
                47352, 32840, 15532, 5112,  45216, 63436, 17104, 39548, 48500,
                44916, 18120, 49684, 23672, 56164, 30680, 44528, 49544, 34644,
                2164,  52004, 44156, 20720, 46576, 28720, 51548, 3624,  26864,
                43888, 63740, 30060, 59596, 2796,  43540, 50976, 32776, 724,
                25584, 59372, 38404, 32544, 24112, 45556, 18796, 14080, 43740,
                7572,  41104, 60556, 23572, 6440,  42572, 10552, 36280, 23220,
                5536,  59936, 41972, 47388, 3104,  39280, 6196,  64288, 41008,
                800,   61292, 35424, 25472, 13788, 50428, 44840, 26880, 37100,
                48356, 29092, 13072, 52644, 7908,  63768, 28784, 54636, 15144,
                23908, 56564, 12056, 29996, 18004, 58312, 42632, 20876, 47052,
                27408, 6180,  18420, 61184, 53640, 39176, 35092, 58960, 30992,
                51952, 46328, 27928, 53720, 48316, 33896, 2036,  40768, 62516,
                18224, 47084,
            },
            {
                34128, 53448, 1540,  57548, 26808, 48372, 40528, 33756, 50216,
                408,   31296, 24744, 56060, 33652, 37520, 64900, 30380, 53540,
                8972,  43396, 37236, 49204, 21148, 7992,  51008, 24108, 57556,
                30788, 37852, 64628, 7172,  46596, 16788, 26440, 8908,  58656,
                14944, 26804, 7004,  60484, 9736,  35532, 41900, 12400, 57312,
                18900, 72,    24640, 11260, 38244, 21788, 8532,  62648, 46392,
                52936, 5284,  14584, 51588, 11664, 29008, 57388, 3072,  32320,
                25484, 48876, 35404, 29360, 14712, 64768, 58260, 26784, 31220,
                45764, 37220, 11952, 22428, 53032, 34672, 50064, 27984, 15088,
                32500, 23140, 43232, 17056, 38576, 8028,  22796, 53084, 10508,
                56904, 3128,  46512, 33992, 648,   38376, 12332, 4432,  50780,
                31356, 41432, 5064,  48328, 39044, 2948,  11312, 61476, 31112,
                56424, 41472, 22344, 50508, 2396,  16232, 4664,  48420, 12132,
                17584, 36692, 668,   13380, 39492, 64336, 21176, 45208, 52692,
                7212,  26560,
            },
            {
                60348, 16468, 5984,  45272, 62424, 30828, 3792,  58872, 20604,
                45104, 63432, 40036, 11636, 21880, 7684,  25712, 57456, 39516,
                23400, 60984, 28796, 1400,  58852, 31596, 13620, 3892,  61380,
                9512,  4696,  53932, 41364, 10928, 62908, 38880, 53436, 18876,
                42644, 37524, 55268, 31232, 17908, 54172, 65100, 5552,  38788,
                53164, 35116, 41252, 55308, 48792, 60948, 28760, 35932, 17224,
                57764, 30460, 22348, 62076, 1148,  47776, 60848, 38136, 52392,
                11200, 19552, 4048,  52076, 46568, 33144, 1404,  49264, 8548,
                14912, 55672, 2024,  29548, 59364, 8900,  24548, 45324, 37552,
                57572, 7196,  49704, 3900,  55152, 60460, 264,   40764, 17308,
                31820, 20408, 24764, 40432, 58688, 48656, 26004, 36744, 44552,
                8240,  64692, 34484, 59904, 22036, 28316, 50120, 35636, 16012,
                96,    63484, 12556, 37164, 28600, 43356, 64540, 20148, 40720,
                8020,  62040, 43492, 56720, 30072, 9240,  16752, 57784, 35760,
                23320, 42988,
            },
            {
                10348, 38360, 32480, 24176, 51072, 8864,  17024, 54500, 23852,
                35772, 5800,  47956, 18352, 51552, 43760, 14464, 49540, 20220,
                5436,  12244, 52272, 18328, 41064, 36312, 54832, 42452, 26748,
                34780, 15836, 22624, 29192, 33744, 2456,  21548, 5476,  32972,
                63556, 11652, 23324, 40628, 792,   48244, 24088, 32724, 16112,
                22300, 47932, 13728, 6856,  31968, 15424, 3864,  24064, 12220,
                42352, 37280, 8204,  54892, 40608, 16964, 6788,  21052, 28188,
                62116, 57768, 39268, 8720,  21872, 54676, 18500, 41388, 62268,
                20496, 32680, 64052, 43716, 18176, 13488, 62800, 3544,  19256,
                54092, 12860, 65400, 27200, 33944, 30356, 43860, 63536, 6360,
                35500, 61632, 54888, 15384, 9812,  21608, 62120, 17616, 55512,
                20820, 26488, 16144, 52676, 7888,  44020, 54784, 4624,  39728,
                24768, 48736, 33088, 8440,  60056, 24340, 52648, 32396, 26728,
                58168, 22156, 33416, 6144,  24792, 51228, 4688,  27616, 12820,
                564,   50444,
            },
            {
                20976, 56668, 42076, 19020, 13396, 37636, 42812, 28204, 10088,
                13872, 57016, 29500, 3440,  54756, 61360, 772,   32172, 46176,
                63524, 34596, 56916, 26188, 9924,  65124, 22008, 46768, 18712,
                52148, 44436, 59384, 13368, 56644, 50724, 43512, 59528, 25284,
                47496, 4416,  50412, 61420, 14100, 27652, 7964,  44428, 59816,
                9292,  62452, 27128, 58476, 19896, 43008, 52204, 56308, 47760,
                2244,  26628, 19400, 44372, 33432, 24780, 35864, 55504, 42548,
                384,   16016, 44996, 63060, 26268, 37580, 13228, 5760,  53248,
                38732, 50436, 7076,  26032, 48756, 39620, 31724, 51944, 9788,
                29984, 41780, 21264, 47740, 11044, 19168, 14528, 24488, 49008,
                12180, 44620, 4796,  30292, 43108, 51948, 32488, 3060,  59212,
                11132, 47344, 588,   37568, 13108, 25480, 63916, 20040, 10132,
                58008, 43616, 17804, 55648, 46904, 14484, 10380, 5408,  45396,
                54268, 14216, 49780, 19104, 61212, 46536, 31960, 43936, 54680,
                63416, 30700,
            },
            {
                47884, 28000, 2980,  53216, 59460, 980,   65192, 46744, 52216,
                60812, 39008, 16088, 34324, 27108, 41208, 36420, 10316, 27900,
                16860, 3400,  42312, 47548, 14696, 6016,  29444, 2056,  12140,
                62780, 37364, 776,   47172, 19260, 27388, 36144, 14232, 8436,
                29772, 17292, 57896, 33340, 20880, 37860, 56404, 49800, 29400,
                3684,  33704, 45140, 1444,  36908, 65056, 7896,  39312, 33356,
                63096, 58544, 48916, 11020, 65024, 51048, 14316, 9688,  47268,
                23928, 34280, 31160, 11588, 2572,  60312, 29924, 44856, 24028,
                27944, 10844, 16668, 36008, 55076, 908,   58380, 23268, 47116,
                59776, 35960, 1372,  39748, 58360, 52012, 37612, 57288, 28156,
                53516, 38040, 18916, 63160, 26696, 6600,  13684, 40828, 35448,
                29052, 42860, 62456, 31440, 57180, 17444, 33952, 46136, 30192,
                52608, 7028,  27536, 1584,  21020, 35652, 40040, 62620, 1348,
                37396, 29140, 3780,  41684, 35468, 11392, 58928, 37868, 18836,
                40216, 8016,
            },
            {
                15652, 64280, 11408, 46268, 33996, 25444, 19952, 6924,  31964,
                1960,  22424, 48936, 62748, 8840,  23560, 18748, 59704, 55576,
                37436, 50792, 23960, 32376, 61140, 38752, 50628, 33572, 56276,
                8124,  31100, 25392, 40012, 9008,  61052, 3656,  49184, 54804,
                39628, 44484, 2932,  10004, 42100, 63412, 2028,  12484, 20204,
                52628, 39232, 16564, 50668, 24732, 30116, 10960, 22448, 18728,
                6520,  15572, 31672, 3008,  27912, 5204,  38824, 30272, 63480,
                5956,  49872, 56636, 17960, 41888, 48260, 57540, 34840, 4040,
                56236, 46456, 60480, 42624, 5796,  20544, 34960, 43432, 15752,
                4388,  25324, 17392, 62072, 8688,  5108,  46172, 2264,  20748,
                9092,  59868, 1048,  49960, 34668, 57528, 46852, 24616, 53504,
                5988,  50432, 23120, 39896, 3496,  49024, 6212,  59216, 14440,
                22268, 38116, 65308, 41360, 51016, 30368, 56612, 17976, 48972,
                23736, 9928,  64948, 16092, 55428, 22808, 2256,  14608, 25844,
                3636,  59928,
            },
            {
                23636, 36584, 6340,  40004, 30068, 14460, 57136, 36428, 41452,
                26216, 44532, 5452,  58148, 13180, 46564, 51076, 6184,  44056,
                13820, 8176,  20680, 312,   53784, 17032, 59108, 23524, 41184,
                48124, 20452, 5500,  64332, 53696, 16904, 31268, 23172, 65504,
                20260, 34964, 26244, 52208, 46788, 30256, 17144, 36104, 42864,
                64156, 23420, 11564, 54800, 5468,  46972, 60192, 50200, 28624,
                45376, 53240, 39972, 20980, 55912, 59812, 45412, 19052, 58396,
                13048, 40652, 27644, 52824, 6836,  22708, 15548, 9180,  19200,
                64548, 1852,  23072, 13944, 30196, 62308, 12148, 7532,  28088,
                63844, 51376, 44524, 32564, 23916, 29304, 34944, 64520, 42552,
                33248, 14740, 23604, 39852, 8852,  16528, 65256, 1396,  19780,
                60956, 14400, 18188, 9204,  53764, 28604, 41792, 36904, 2576,
                61780, 32632, 12144, 4104,  59624, 7632,  26340, 12392, 34124,
                59200, 44624, 52656, 38816, 27432, 8136,  62920, 48872, 52604,
                34744, 44596,
            },
            {
                31928, 49880, 20200, 62152, 54908, 9132,  48836, 61852, 17148,
                11416, 51708, 19540, 37880, 29708, 2392,  65508, 31028, 24880,
                39856, 58280, 64160, 45460, 27180, 9152,  43800, 4788,  27732,
                16324, 58216, 50996, 33916, 12540, 42444, 37724, 45824, 340,
                12752, 55836, 61716, 15060, 7128,  24472, 53636, 58236, 26620,
                6684,  31604, 57696, 41000, 18412, 34664, 14164, 55984, 744,
                36484, 12864, 60924, 24216, 42484, 16560, 1532,  51960, 22800,
                35576, 2076,  20920, 65364, 36848, 32916, 61896, 52008, 37704,
                28852, 40680, 33444, 53388, 47672, 24948, 57032, 49776, 39924,
                31284, 11336, 56244, 14276, 49144, 54520, 11852, 16684, 26008,
                52356, 47752, 29424, 44892, 55668, 21384, 31464, 43668, 10700,
                37232, 32876, 58232, 44888, 64884, 20664, 10728, 25996, 51572,
                17284, 44224, 49576, 24896, 15608, 45856, 52376, 21536, 41788,
                7164,  31300, 19648, 204,   46908, 33840, 42688, 29576, 21592,
                9960,  55736,
            },
            {
                12964, 58516, 27500, 16340, 4260,  43808, 22224, 3020,  28820,
                56008, 64116, 32700, 43016, 53432, 16384, 35332, 21284, 4016,
                53076, 28840, 34476, 12492, 37784, 20092, 49148, 13828, 63468,
                36872, 10408, 43676, 22916, 28476, 4552,  57660, 10392, 51260,
                29100, 6468,  37268, 21928, 59192, 39352, 4516,  45716, 9992,
                15448, 47628, 2092,  61332, 27068, 44256, 4372,  40228, 25780,
                64204, 8988,  34456, 49472, 7488,  37048, 32800, 26428, 8904,
                54872, 44396, 15212, 10420, 45924, 3712,  25928, 43308, 48932,
                11620, 17140, 58728, 8076,  39064, 3260,  36560, 18528, 54428,
                21036, 6464,  37424, 1972,  42924, 19644, 61316, 40364, 6728,
                57776, 3944,  63492, 12212, 5448,  52520, 38584, 25560, 49212,
                55940, 27516, 1816,  24260, 34684, 13700, 60132, 46752, 55992,
                8860,  28928, 57160, 19560, 37700, 32144, 64088, 2588,  57380,
                15396, 61584, 25576, 12700, 50596, 59992, 17172, 5640,  57996,
                41480, 4520,
            },
            {
                46784, 364,   42268, 51364, 24916, 38188, 53124, 34592, 47140,
                8128,  14980, 124,   25140, 10340, 57484, 41348, 49092, 61692,
                11028, 18160, 48544, 6352,  56452, 30920, 60352, 34060, 3360,
                52396, 30496, 1280,  60088, 48888, 19964, 62564, 25208, 34292,
                18456, 43480, 48724, 31952, 11536, 50196, 33288, 19388, 60760,
                34916, 51812, 21516, 37488, 8676,  63544, 32180, 16816, 52308,
                43364, 19236, 3748,  29120, 54404, 11292, 64472, 48664, 39592,
                60800, 31020, 50828, 58696, 29144, 56052, 13692, 200,   21736,
                54360, 4760,  44728, 19836, 27424, 65180, 15048, 45188, 348,
                61168, 47368, 27212, 58824, 22748, 36344, 3328,  50348, 30792,
                21712, 37740, 18360, 35768, 27428, 60552, 13548, 3732,  62760,
                7960,  40728, 47824, 5432,  38684, 50532, 31588, 504,   23472,
                39836, 4908,  34356, 60992, 10040, 43048, 5624,  27448, 38960,
                47708, 36224, 5144,  56200, 30732, 20760, 10908, 37604, 65488,
                26372, 18504,
            },
            {
                38464, 63276, 34856, 10896, 60244, 31252, 12908, 18800, 58664,
                40572, 36660, 49832, 20768, 62988, 27580, 7804,  14292, 44508,
                32784, 3048,  42652, 61976, 24336, 2200,  40716, 54044, 21824,
                46168, 18860, 39476, 14900, 35868, 7968,  53024, 40300, 14720,
                56588, 63288, 2232,  41312, 16944, 64736, 1392,  28400, 55188,
                38784, 5768,  29860, 12944, 49716, 23540, 57192, 10536, 21892,
                30860, 58248, 45752, 15848, 61724, 20652, 44104, 14428, 3472,
                18204, 6288,  25116, 38132, 7768,  19368, 39980, 31628, 60264,
                35436, 63472, 30856, 50400, 11244, 52176, 32556, 9160,  23992,
                34500, 41180, 17748, 64716, 7948,  28660, 57012, 10212, 46388,
                13876, 59520, 43332, 1608,  50504, 33316, 46304, 17876, 30600,
                22136, 12432, 52492, 61272, 16740, 7568,  42804, 18872, 35812,
                62628, 14756, 48616, 1716,  22688, 55224, 49712, 18324, 11496,
                54280, 22480, 43628, 64028, 40120, 3708,  45348, 51220, 14356,
                31316, 54444,
            },
            {
                22788, 29344, 7332,  21404, 47492, 1536,  64856, 26768, 4528,
                22528, 30512, 55408, 45008, 3916,  33492, 38072, 56432, 19740,
                26172, 54836, 38628, 14780, 50956, 17560, 8380,  26248, 12268,
                57808, 6692,  64104, 54684, 26196, 44692, 1676,  31756, 47576,
                4692,  23624, 27460, 9064,  54476, 44420, 22952, 12748, 41976,
                24344, 64352, 45436, 18948, 53904, 2968,  46704, 37044, 60688,
                6448,  50472, 25416, 37968, 156,   31824, 23964, 57772, 28148,
                34816, 62512, 42584, 22388, 49388, 63996, 53168, 46364, 6740,
                24452, 14988, 41572, 1380,  22656, 59720, 40492, 28884, 57384,
                13444, 51692, 10548, 32992, 53132, 15536, 44172, 24476, 34036,
                53604, 8552,  25072, 56080, 20560, 9764,  41088, 57972, 54120,
                35620, 44284, 19372, 29784, 56180, 26300, 63572, 52312, 12028,
                45372, 27116, 53536, 40988, 30440, 13948, 35368, 62440, 29752,
                1424,  33312, 9448,  15040, 27304, 53608, 24492, 34068, 1828,
                48060, 8924,
            },
        },
        {
            {
                9056,  40524, 30504, 59004, 44100, 54500, 31904, 21888, 62688,
                8540,  47648, 39212, 13340, 34552, 25360, 52588, 60676, 42256,
                32472, 46332, 62084, 2924,  23760, 41356, 17152, 12052, 63752,
                42884, 22412, 13368, 2852,  40220, 5396,  27408, 19608, 36492,
                48056, 25456, 33992, 14136, 40132, 128,   31940, 44440, 12724,
                2300,  19696, 63396, 56104, 45564, 17840, 65200, 43396, 53800,
                21372, 5920,  46580, 1528,  41900, 23000, 32452, 44384, 19168,
                25000, 47352, 52864, 9772,  46024, 26784, 49516, 31308, 23428,
                44864, 35280, 1600,  30028, 44004, 52016, 65484, 28432, 57068,
                54004, 10972, 43288, 5408,  64664, 55656, 376,   27028, 58884,
                10472, 24760, 17352, 64024, 45900, 20600, 43104, 2972,  47056,
                27404, 16524, 34916, 55344, 48796, 18212, 42976, 59880, 51728,
                1540,  34520, 47712, 6608,  33172, 21072, 61152, 36308, 55960,
                18504, 40540, 8820,  660,   12900, 29468, 3988,  11220, 54584,
                49072, 5416,
            },
            {
                44744, 14888, 25080, 3604,  12328, 63552, 4812,  37108, 1044,
                18476, 30868, 54360, 2512,  44756, 9896,  16844, 48392, 14332,
                58556, 7580,  29388, 53204, 5348,  32920, 51488, 536,   31040,
                9248,  53444, 59788, 50004, 65276, 31392, 8832,  56136, 12864,
                5008,  43756, 56808, 8240,  49464, 58772, 24124, 36116, 27696,
                51924, 16004, 36928, 7000,  49956, 22180, 57360, 9500,  38692,
                33824, 56896, 14544, 36904, 59988, 55600, 13648, 65360, 37144,
                8524,  34184, 63492, 28944, 12968, 35392, 15756, 54244, 18288,
                7104,  48672, 20460, 62420, 16764, 3840,  22040, 48364, 41724,
                15296, 30648, 22396, 51432, 19680, 34044, 41072, 46168, 4564,
                61360, 32328, 56316, 1052,  11092, 54420, 33600, 56788, 9356,
                39636, 52380, 7568,  20520, 30120, 57792, 37992, 6908,  31192,
                24888, 18608, 60408, 43752, 15504, 54928, 8428,  47000, 4028,
                50052, 30072, 57112, 53112, 25444, 62700, 37368, 59340, 16436,
                23104, 61416,
            },
            {
                19140, 64720, 52928, 35928, 39548, 17520, 26420, 46792, 41192,
                59928, 23916, 15708, 64000, 40608, 61636, 29956, 5628,  39056,
                21812, 50300, 17760, 37836, 14916, 45760, 61052, 21160, 55744,
                38876, 26152, 29436, 15440, 37140, 21832, 48636, 62232, 24104,
                16568, 52024, 30564, 18928, 27948, 46248, 6176,  55528, 9140,
                60032, 41412, 25100, 10404, 31016, 35532, 520,   25552, 61452,
                2988,  30932, 49964, 27492, 17856, 5228,  48040, 2664,  27740,
                57696, 16096, 4604,  38372, 19644, 58228, 2908,  40784, 62760,
                10992, 58848, 38140, 54664, 40880, 10424, 32020, 6284,  34796,
                2224,  63036, 46652, 39588, 12972, 24464, 53888, 17084, 21704,
                51416, 42456, 27632, 39084, 50404, 15520, 25064, 64468, 18696,
                31132, 2424,  62708, 45188, 24612, 100,   12160, 47052, 21464,
                42020, 56184, 10140, 30056, 63608, 40084, 28524, 12672, 33504,
                65412, 21552, 36696, 45040, 18076, 48736, 21132, 42048, 1344,
                31184, 36896,
            },
            {
                4172,  10668, 47628, 21452, 7536,  60672, 52268, 9852,  56920,
                34272, 11424, 51188, 27612, 19620, 46704, 23528, 1212,  55596,
                27292, 10432, 43056, 56292, 64276, 7036,  26768, 35248, 48420,
                17716, 2240,  45620, 10752, 54880, 43216, 880,   33312, 41380,
                58840, 37820, 2044,  61980, 11920, 38360, 65336, 14928, 22244,
                49016, 32708, 46400, 58040, 13868, 42248, 51548, 29600, 12452,
                18808, 44584, 64420, 10684, 39384, 33628, 21156, 54108, 11456,
                42016, 50612, 23468, 44808, 61004, 51272, 8656,  25268, 46328,
                28184, 32360, 14144, 5064,  27156, 50704, 60752, 44816, 18516,
                52744, 26052, 4608,  56704, 29604, 2912,  62664, 37672, 7688,
                34896, 13508, 19524, 8288,  61920, 30220, 6144,  41492, 51116,
                60124, 26164, 36924, 14084, 41396, 51052, 61036, 16192, 64744,
                52768, 4412,  38168, 22228, 2416,  51152, 19104, 58492, 24644,
                1956,  15592, 10764, 32416, 6624,  57576, 34548, 8756,  46848,
                27164, 55956,
            },
            {
                41496, 33396, 27764, 57688, 42376, 32380, 23272, 14620, 28772,
                6420,  44328, 37960, 4224,  8108,  35308, 58176, 51776, 32868,
                63332, 35984, 2140,  24316, 30480, 40432, 9644,  14140, 59492,
                5876,  41416, 61852, 34060, 6380,  17932, 52388, 11312, 26980,
                6620,  45504, 22312, 35076, 54332, 3172,  19784, 43492, 29944,
                996,   5840,  64052, 20824, 3972,  60932, 16664, 46008, 39848,
                53492, 22044, 7400,  24556, 52248, 60984, 43728, 30508, 62020,
                18772, 896,   56156, 32080, 5720,  42540, 21660, 33648, 56796,
                16,    52544, 23216, 46940, 36020, 56220, 24828, 12120, 38268,
                58544, 9572,  37192, 16504, 60296, 49332, 10696, 30876, 48032,
                63804, 3952,  53380, 44064, 22520, 48148, 37748, 12904, 4480,
                21672, 48500, 10380, 58372, 5032,  28488, 33536, 36456, 8464,
                27684, 13736, 49384, 59140, 26104, 6132,  44436, 37752, 52592,
                41572, 48324, 60128, 39352, 2464,  28008, 14616, 53844, 63992,
                12596, 50064,
            },
            {
                22768, 60020, 296,   15484, 50308, 2844,  44872, 65408, 48884,
                20112, 55320, 60432, 32512, 49184, 13676, 42680, 18220, 7144,
                45404, 20448, 59984, 12700, 47504, 18688, 53792, 43836, 23532,
                32052, 51284, 20352, 24548, 58092, 28272, 63896, 36744, 20220,
                61124, 10072, 50556, 15832, 42464, 26296, 33344, 52400, 61192,
                39828, 18112, 53084, 28224, 38332, 54832, 23516, 6760,  57952,
                4072,  34724, 41348, 58624, 20,    15460, 8952,  24092, 39720,
                35212, 48168, 26432, 11028, 15560, 29228, 63844, 12004, 39192,
                16528, 42372, 64800, 9220,  19308, 1124,  16044, 64176, 28160,
                47808, 21376, 32428, 44544, 6816,  26824, 42896, 15144, 56948,
                24024, 28644, 57988, 36216, 1996,  59428, 18104, 55668, 34848,
                43576, 15796, 32172, 53908, 22556, 17740, 55572, 2832,  40552,
                45660, 32544, 17508, 35676, 46512, 31736, 15172, 9792,  61572,
                5696,  26868, 54240, 22872, 63440, 51380, 43600, 24328, 5644,
                39076, 17132,
            },
            {
                7856,  45116, 54164, 19916, 29856, 11992, 35496, 4960,  39332,
                25564, 628,   16284, 22732, 53752, 64504, 26320, 11008, 40200,
                15108, 49164, 6220,  52288, 33980, 58812, 1500,  36540, 64724,
                12264, 56264, 8272,  47624, 39512, 2588,  45208, 14860, 48964,
                31944, 55184, 29156, 63740, 7448,  47904, 57940, 9868,  13036,
                35336, 24768, 8672,  44012, 34016, 10916, 48932, 32168, 27188,
                62744, 47760, 13324, 29016, 49380, 37296, 46648, 4736,  53188,
                13780, 7452,  64808, 37540, 52968, 46996, 35616, 4120,  49124,
                20336, 6748,  29812, 59588, 33140, 40312, 49520, 7188,  42324,
                1736,  14220, 61328, 54064, 19068, 35364, 52952, 20864, 2380,
                45596, 40336, 16932, 10540, 32436, 26492, 45144, 9816,  28604,
                59136, 1496,  39860, 45980, 64272, 9476,  49004, 25808, 20444,
                57240, 62904, 620,   53324, 12012, 64400, 56856, 22552, 34676,
                18228, 31340, 13376, 45460, 18864, 10352, 30404, 59020, 20092,
                32684, 62008,
            },
            {
                29320, 36136, 10160, 64040, 38804, 55532, 59748, 17388, 53132,
                30448, 62892, 40980, 9752,  29592, 2248,  37592, 60720, 31228,
                55232, 25368, 29080, 38748, 4148,  22324, 28128, 49412, 5132,
                29848, 38220, 16312, 63176, 13256, 31064, 22920, 57392, 4548,
                39144, 1060,  13560, 24228, 40144, 17812, 4832,  22680, 45924,
                57164, 50388, 15736, 61720, 2940,  19732, 64860, 14700, 37352,
                9928,  17424, 55896, 32792, 20004, 26644, 64092, 17616, 58508,
                28612, 43496, 20304, 57404, 2144,  18196, 58532, 27056, 54580,
                61792, 36996, 51212, 13196, 44196, 57624, 22320, 30772, 54992,
                34316, 50740, 24136, 40732, 12168, 656,   59172, 38912, 9672,
                33696, 6672,  60876, 50820, 14208, 64704, 3484,  49668, 23396,
                63468, 52356, 19352, 6224,  29916, 37764, 43864, 61488, 12416,
                6788,  23732, 42860, 8708,  20312, 41016, 4004,  27996, 47312,
                43488, 84,    56476, 7584,  33792, 40792, 3268,  36532, 45944,
                1924,  51584,
            },
            {
                14240, 40656, 24064, 5464,  46984, 26384, 21720, 8904,  43484,
                12884, 36040, 48076, 56688, 44152, 20724, 5244,  46880, 22176,
                324,   65196, 44040, 17164, 61832, 42288, 15008, 55220, 19716,
                45428, 176,   26736, 35188, 53744, 42800, 9472,  51808, 25728,
                18692, 43580, 34688, 58600, 49968, 32208, 62168, 38028, 27376,
                1988,  41928, 31148, 56068, 47024, 26556, 41324, 1220,  51228,
                21200, 42776, 2584,  61284, 6716,  11520, 56192, 31508, 1532,
                22740, 51428, 33104, 40644, 24800, 8172,  43912, 14300, 22876,
                10052, 45676, 24660, 2996,  27576, 4756,  36720, 11128, 18824,
                63104, 8796,  3516,  30044, 62484, 48720, 31520, 25056, 65364,
                54912, 19700, 47216, 23176, 42040, 53036, 20436, 39096, 31660,
                7888,  13276, 35772, 24528, 58908, 936,   15096, 53572, 31260,
                47032, 38600, 29108, 60784, 25556, 49960, 37076, 54648, 8004,
                51740, 64672, 36856, 24756, 50232, 61232, 16240, 53340, 11900,
                56424, 25956,
            },
            {
                48072, 59264, 52764, 17964, 34212, 1324,  49776, 32312, 57684,
                2976,  19316, 7092,  25120, 14748, 50656, 34644, 57832, 52576,
                13284, 35276, 8328,  56708, 11288, 32256, 7080,  62872, 24892,
                41160, 60072, 50644, 4088,  21140, 59656, 5564,  34132, 64992,
                46928, 60496, 6148,  21092, 11716, 2692,  29500, 51836, 15128,
                64712, 6252,  21592, 12368, 36628, 7720,  54168, 58964, 44916,
                30036, 25032, 52976, 38620, 44420, 50940, 40748, 35028, 45672,
                10436, 59544, 5368,  12796, 30740, 49440, 63316, 32068, 41504,
                1812,  34240, 17920, 54204, 63872, 15508, 51956, 60204, 45012,
                26716, 36452, 46896, 55856, 21260, 7844,  44968, 16484, 49868,
                12340, 30156, 37812, 196,   28028, 34692, 5804,  57776, 16620,
                42708, 26688, 56732, 51140, 11344, 41856, 22020, 34492, 4728,
                18820, 51096, 15396, 55032, 33580, 1912,  17180, 62272, 12372,
                21536, 15700, 28896, 58336, 5296,  47536, 27348, 22336, 65216,
                42084, 6452,
            },
            {
                21188, 2352,  30884, 42940, 60528, 15020, 40864, 62616, 24180,
                45748, 51536, 33488, 59076, 38980, 62284, 17640, 9900,  27276,
                41600, 19472, 50524, 23928, 46096, 53524, 38440, 47868, 9260,
                17464, 55924, 32856, 11228, 46548, 17176, 28968, 40888, 11112,
                15800, 28520, 52516, 36908, 46184, 55376, 43088, 8224,  19924,
                33688, 39380, 58636, 49780, 29852, 17104, 23404, 33352, 5324,
                62356, 8476,  35428, 15964, 21824, 3192,  24324, 14328, 63152,
                38536, 48340, 16956, 61620, 54260, 20008, 38264, 5552,  60304,
                28660, 48308, 59228, 38952, 20980, 47084, 40844, 23892, 5608,
                14608, 58368, 17456, 39024, 27756, 14060, 36100, 5148,  42128,
                26424, 4368,  57008, 63076, 8984,  55464, 12532, 47340, 62072,
                37224, 4644,  47948, 32100, 17404, 60588, 28132, 49276, 65344,
                10624, 58012, 35852, 6512,  13328, 46316, 30340, 24468, 41880,
                33204, 39232, 10280, 42656, 20368, 13632, 38800, 8576,  29936,
                34756, 16916,
            },
            {
                39280, 63188, 12628, 7992,  55880, 28300, 11248, 5044,  38036,
                15604, 64652, 28172, 3744,  11832, 30380, 6696,  43628, 32364,
                3084,  60428, 37528, 29776, 1892,  20376, 26512, 35316, 13092,
                30832, 6364,  22748, 40232, 62444, 36312, 56200, 49320, 23628,
                2884,  55728, 9176,  26316, 64260, 16336, 24792, 60080, 48212,
                54464, 10268, 24448, 464,   43332, 63612, 39040, 13528, 18760,
                56888, 47184, 12784, 65088, 57884, 29888, 54300, 7504,  19768,
                26860, 2608,  35784, 42624, 27928, 1408,  11420, 51412, 16724,
                56172, 12464, 5896,  31144, 10748, 7944,  29056, 1016,  33336,
                49808, 42684, 10728, 4788,  64044, 51832, 59712, 22548, 53608,
                61860, 35288, 14740, 44244, 17572, 40292, 25184, 29656, 1056,
                45100, 21400, 63116, 8728,  44208, 37060, 3712,  55700, 39484,
                26276, 1228,  44672, 21512, 63904, 38216, 57772, 5716,  59680,
                3524,  47892, 53580, 1612,  63684, 31964, 55592, 44036, 1020,
                49672, 57620,
            },
            {
                32824, 26576, 51132, 36592, 22828, 46092, 52200, 19896, 29832,
                54404, 9424,  21896, 41708, 54924, 46552, 23660, 63976, 48980,
                54680, 16092, 6548,  63620, 14056, 58332, 4936,  60944, 52620,
                42752, 65496, 48828, 14464, 25792, 7912,  828,   19880, 62740,
                38172, 31180, 42404, 19112, 232,   40308, 35636, 12668, 3704,
                44352, 28236, 18316, 60320, 53180, 4560,  9676,  50108, 28076,
                40376, 1524,  32324, 26836, 46068, 10000, 43052, 60228, 50368,
                31084, 55648, 22032, 8864,  46700, 57316, 34356, 44752, 25472,
                36780, 22028, 42996, 50356, 62064, 35116, 56812, 53056, 65212,
                20024, 31020, 54408, 25156, 46240, 31984, 2428,  39496, 18832,
                9192,  46692, 51520, 21820, 31504, 49264, 61068, 19268, 10364,
                54396, 14652, 29104, 2816,  53896, 20592, 13972, 7696,  23336,
                42452, 32072, 60060, 28156, 48360, 9496,  43188, 19136, 50132,
                26440, 61856, 17336, 23812, 35388, 51288, 4468,  18888, 62276,
                24436, 10244,
            },
            {
                46804, 3420,  44728, 18976, 65068, 6384,  34684, 61248, 42964,
                148,   36536, 49508, 18136, 60852, 1216,  36920, 20044, 12500,
                39552, 24988, 47292, 33176, 43316, 39884, 49600, 18444, 1076,
                27924, 37292, 2756,  58160, 30488, 53480, 44092, 13804, 33364,
                45184, 58516, 13064, 51460, 32980, 5540,  57972, 21900, 31388,
                62660, 36756, 47344, 14148, 34516, 25448, 45728, 61548, 35860,
                54772, 20920, 52104, 5648,  18456, 37604, 33592, 16356, 4032,
                41588, 13188, 63780, 51952, 24456, 14500, 64484, 20540, 8152,
                61048, 52800, 356,   15812, 25296, 43888, 17524, 12084, 38008,
                7708,  60592, 1536,  35060, 18308, 43412, 11884, 57824, 29508,
                33016, 1672,  24924, 58316, 6688,  3272,  36656, 51316, 33388,
                59380, 40752, 35444, 24056, 50000, 63848, 33248, 45712, 61464,
                12124, 17900, 51892, 3240,  16168, 23512, 53680, 11868, 36332,
                31632, 8408,  44656, 56800, 11792, 26292, 46600, 59716, 36992,
                14004, 54112,
            },
            {
                7008,  61288, 29716, 15556, 39960, 1940,  25292, 47940, 12964,
                56848, 26084, 44992, 13532, 33948, 52696, 26708, 57380, 4352,
                29164, 59048, 10556, 21036, 55988, 8628,  31304, 24080, 44980,
                55016, 20508, 10376, 45780, 18340, 39172, 60452, 27428, 50320,
                4144,  7420,  22284, 46884, 62324, 27724, 49396, 41996, 16820,
                52208, 2480,  8020,  40488, 30468, 20472, 57532, 2332,  16140,
                24432, 10712, 42244, 58804, 49444, 752,   62932, 24856, 47196,
                58952, 6964,  33324, 18128, 37552, 5096,  30348, 40204, 2916,
                46764, 32812, 28164, 38508, 58628, 3644,  21804, 46488, 27172,
                41624, 22668, 48540, 13728, 61396, 8056,  24116, 49448, 15740,
                55368, 38404, 11768, 41528, 65000, 13664, 43808, 23268, 7476,
                26180, 48412, 18236, 57132, 10868, 38828, 25992, 16688, 48648,
                54192, 36760, 8788,  40568, 57192, 34528, 29068, 944,   56132,
                14428, 21172, 38144, 30068, 6804,  40476, 16172, 9068,  28948,
                42520, 20236,
            },
            {
                38476, 23576, 10600, 57856, 53552, 31656, 59084, 17592, 7532,
                32616, 63500, 4048,  29588, 7808,  40284, 15572, 9308,  51276,
                45008, 35692, 1028,  52752, 27524, 16464, 64304, 11572, 34652,
                14988, 61748, 24908, 51648, 34004, 3464,  22612, 9880,  54904,
                16872, 25752, 60140, 37312, 14920, 10008, 54572, 6756,  25696,
                11696, 57108, 22812, 64360, 48912, 11268, 41972, 31772, 7740,
                48180, 63892, 33968, 14512, 22884, 28976, 52832, 11408, 20960,
                39088, 28532, 45864, 544,   41944, 58836, 48908, 27016, 54800,
                13800, 18644, 64740, 6944,  53928, 32420, 49572, 62468, 2516,
                15464, 29460, 57916, 37516, 53700, 28484, 41060, 64692, 6360,
                43920, 62312, 19404, 28996, 33956, 54228, 16672, 57428, 2336,
                64184, 12456, 5784,  43180, 396,   30084, 58376, 6304,  2032,
                28860, 21208, 64608, 25748, 50336, 4720,  62516, 39668, 46200,
                65284, 52028, 2536,  60976, 49152, 58068, 22580, 52916, 34096,
                2892,  56736,
            },
            {
                16420, 48280, 35108, 42604, 8140,  13784, 50408, 38936, 23652,
                41096, 20256, 50564, 58708, 22980, 65480, 43024, 31196, 60700,
                22052, 17488, 62920, 38340, 48860, 2952,  36732, 59224, 50236,
                7312,  31480, 41720, 6480,  62952, 12924, 42676, 37092, 64620,
                29884, 41080, 52756, 2496,  30728, 44452, 19976, 65384, 38776,
                33452, 43944, 27776, 55000, 6392,  17536, 53668, 38392, 58992,
                44164, 3476,  27144, 39440, 61020, 8644,  41284, 56416, 35872,
                15324, 54468, 25320, 62536, 11784, 52836, 16176, 9244,  62144,
                35528, 42496, 11132, 23752, 41028, 14100, 9264,  36024, 55768,
                44768, 51676, 10156, 3816,  20512, 47540, 676,   34680, 21616,
                27180, 3832,  53232, 47744, 9108,  26404, 39676, 46800, 29872,
                37628, 52892, 32308, 59960, 46628, 19596, 51424, 40616, 34780,
                56204, 43100, 5712,  31644, 13896, 45192, 20660, 16856, 24784,
                6500,  41660, 27336, 18368, 13548, 32040, 412,   45200, 64812,
                25676, 50152,
            },
            {
                31032, 4464,  63952, 26388, 21740, 44504, 28428, 2792,  60368,
                52856, 9920,  16304, 37620, 46992, 2868,  19396, 48284, 34084,
                7348,  41744, 13044, 24572, 6368,  42208, 46392, 22188, 3932,
                39836, 57452, 47908, 16628, 28168, 56872, 48864, 5340,  20580,
                46532, 34088, 8932,  18448, 56824, 24232, 35944, 1012,  47928,
                59756, 18752, 14444, 37792, 904,   61840, 26732, 21980, 13492,
                29728, 19400, 56064, 6396,  46708, 17600, 31344, 5116,  64256,
                2572,  9708,  50176, 19856, 35216, 23168, 32152, 44972, 21408,
                4800,  51400, 57304, 48528, 19572, 29084, 59764, 24300, 6464,
                19096, 33644, 40472, 25788, 63164, 32236, 14412, 56876, 11240,
                50500, 36844, 16148, 60148, 580,   21116, 62956, 11188, 19084,
                42576, 22384, 15712, 27608, 36272, 13612, 8680,  62804, 22600,
                11108, 15384, 47260, 58552, 37484, 9148,  60348, 30952, 49376,
                34028, 10932, 54496, 35924, 43608, 63100, 38908, 19076, 12072,
                7864,  60268,
            },
            {
                41128, 13244, 52108, 560,   55588, 36496, 63140, 11496, 34460,
                46408, 4880,  31152, 55432, 11664, 28000, 56372, 13660, 1620,
                53856, 28456, 57016, 32740, 61664, 19504, 29924, 55812, 26636,
                18260, 9944,  35824, 976,   53156, 24040, 31680, 15272, 59216,
                648,   12292, 62612, 49744, 40020, 61356, 12992, 51848, 28480,
                8824,  4324,  50512, 31428, 44748, 35232, 51580, 4128,  63144,
                49824, 37152, 52924, 12404, 35136, 51704, 44288, 26360, 48588,
                32768, 43244, 60492, 29432, 7544,  57056, 1972,  38416, 59364,
                25528, 30192, 2152,  34224, 63220, 4012,  43400, 52740, 31468,
                64532, 12712, 59668, 7332,  45032, 17796, 52084, 38716, 60596,
                45476, 8128,  23920, 42260, 35344, 51540, 6036,  32816, 54960,
                8460,  61640, 50296, 3660,  65148, 53964, 25108, 45104, 31856,
                60864, 52432, 19232, 144,   23016, 53932, 42692, 12556, 1960,
                58140, 21908, 47532, 3856,  24544, 5752,  51712, 29928, 54932,
                36668, 21272,
            },
            {
                27832, 57196, 33088, 19760, 15336, 47584, 6536,  18408, 56272,
                27244, 43416, 62452, 24496, 35012, 5904,  61472, 38756, 25824,
                64220, 45352, 9648,  49892, 15456, 54072, 8936,  13144, 61200,
                52032, 29424, 64516, 21012, 38960, 45536, 8568,  35420, 53784,
                25380, 43764, 22068, 27932, 6528,  3552,  32644, 16244, 21844,
                41112, 62368, 24428, 57880, 20192, 12804, 46592, 9400,  33216,
                16576, 108,   24944, 65292, 22160, 1808,  13712, 19708, 58736,
                23360, 17308, 39900, 3804,  47828, 42344, 65436, 12616, 17220,
                47208, 39664, 8860,  15660, 45648, 37020, 11260, 16904, 39168,
                788,   47088, 22540, 35648, 55920, 4372,  9456,  24372, 29968,
                18632, 32764, 57412, 13496, 29484, 45224, 59700, 14680, 48744,
                1360,  25640, 39816, 10476, 21280, 41908, 4988,  17284, 38164,
                3504,  26808, 39904, 63588, 34640, 27844, 6584,  51492, 38284,
                63812, 28724, 16056, 61728, 56140, 10180, 26788, 15212, 47404,
                1764,  44084,
            },
            {
                9380,  46768, 5780,  39580, 62032, 30308, 51824, 23296, 38524,
                14388, 20956, 796,   40044, 53348, 44900, 21468, 50208, 16032,
                36228, 4740,  20308, 40152, 180,   35936, 44844, 33000, 2040,
                41988, 23648, 44348, 12164, 59936, 3348,  63504, 18568, 50560,
                38448, 56392, 14820, 36712, 47580, 55316, 42356, 58740, 46132,
                54272, 34428, 10340, 40224, 6916,  29536, 60732, 23600, 39712,
                57752, 43636, 9804,  30960, 41068, 56700, 62172, 37816, 7840,
                53252, 12496, 55996, 36848, 14736, 26168, 50448, 29020, 6416,
                53756, 61616, 23116, 55120, 26520, 58436, 21524, 48772, 25916,
                57096, 28804, 15960, 50744, 41752, 27260, 61608, 43112, 1784,
                54756, 5516,  49248, 61944, 2960,  38892, 23128, 27732, 37104,
                58644, 33876, 45684, 56740, 31332, 48356, 28464, 59252, 50604,
                8184,  45876, 30416, 10700, 48552, 17272, 57416, 25408, 19360,
                44912, 8356,  40500, 32852, 19980, 49024, 41848, 34856, 58104,
                61984, 17444,
            },
            {
                50596, 37008, 59744, 24044, 10084, 42416, 2104,  32996, 63900,
                7644,  58792, 47672, 13072, 17732, 8484,  32280, 42308, 10772,
                23300, 52548, 31120, 59920, 27104, 23720, 65176, 48700, 37692,
                15060, 58816, 4716,  33960, 16296, 26584, 41576, 11292, 28916,
                7316,  3036,  64948, 31076, 10572, 17676, 24948, 7376,  30112,
                1488,  15412, 48260, 64700, 53016, 17124, 42348, 55172, 5136,
                27804, 47640, 59576, 17988, 49084, 5856,  28912, 33480, 47008,
                2200,  27080, 31008, 62068, 19048, 8972,  55360, 33852, 20564,
                36508, 11384, 42696, 31932, 1092,  51280, 7680,  61280, 5344,
                42600, 53724, 3264,  10636, 31072, 20904, 49096, 35140, 15588,
                63728, 40420, 26660, 20400, 9724,  17692, 56404, 4328,  44048,
                20552, 16836, 5672,  14124, 35640, 820,   54944, 11936, 34524,
                21008, 58052, 14168, 55632, 4920,  40948, 61524, 35664, 14704,
                31780, 3140,  52636, 12836, 36916, 59384, 7216,  22300, 4408,
                32368, 24620,
            },
            {
                65008, 2476,  16580, 44820, 27460, 58400, 12872, 49252, 41560,
                53692, 28920, 33892, 51032, 64316, 29324, 60112, 2184,  57704,
                47784, 62192, 12788, 43772, 7424,  57676, 17704, 5908,  20868,
                50784, 8520,  55540, 46696, 30436, 49752, 57080, 21288, 61236,
                33136, 46388, 42052, 20160, 60276, 52824, 35804, 64160, 11800,
                38436, 26948, 21412, 3584,  36372, 25932, 1308,  49576, 14784,
                20084, 34620, 7204,  38668, 26012, 54556, 11184, 16288, 40588,
                64748, 44184, 4580,  51260, 22800, 45492, 40,    41232, 58136,
                3224,  49856, 18396, 64660, 13384, 39812, 29336, 33868, 13836,
                36428, 19440, 60484, 38464, 65380, 13312, 58648, 7384,  47012,
                23256, 12152, 35828, 53944, 48132, 65220, 31892, 52320, 11332,
                63836, 30048, 50860, 62424, 19212, 60280, 23880, 15716, 41536,
                64780, 1700,  24412, 44280, 32788, 21756, 1392,  10144, 47140,
                55980, 60420, 23228, 46636, 1332,  29548, 64296, 11416, 52456,
                40116, 12820,
            },
            {
                20648, 30168, 53408, 7260,  35576, 54976, 20364, 6104,  25068,
                17004, 10580, 3032,  22472, 37264, 5160,  54596, 19080, 26416,
                33864, 3568,  17372, 37952, 51748, 11340, 40928, 28256, 62652,
                31888, 25532, 39976, 19612, 52984, 5776,  36292, 1348,  48224,
                16692, 23968, 8852,  50804, 26828, 5248,  44704, 20668, 50116,
                60684, 55964, 31320, 45332, 12272, 57216, 32488, 37508, 64300,
                11300, 53452, 62464, 14088, 3716,  44772, 61240, 23864, 51888,
                20380, 35488, 10356, 58872, 32848, 37988, 63876, 24356, 14492,
                44848, 28204, 7108,  35628, 46784, 20064, 55520, 44072, 64060,
                23488, 8784,  32504, 46076, 24836, 156,   40028, 19344, 29200,
                51984, 2472,  43716, 29780, 6996,  41452, 13916, 25412, 39164,
                7672,  54532, 24636, 40664, 9396,  44392, 37940, 6572,  47224,
                32316, 52252, 37768, 16492, 63008, 53372, 29528, 50168, 38844,
                6040,  27384, 42636, 17320, 54040, 25044, 43924, 18932, 46236,
                27308, 55780,
            },
            {
                42756, 34252, 48240, 22212, 61088, 15004, 46464, 37656, 30592,
                59900, 45384, 56228, 42176, 25484, 15292, 46052, 38492, 9028,
                41164, 56148, 28808, 21848, 46840, 31572, 55128, 3152,  45312,
                11956, 36460, 492,   63324, 10164, 14576, 43432, 25180, 40060,
                53532, 58896, 13524, 39148, 132,   32976, 14352, 40404, 2672,
                17788, 42908, 6604,  50776, 18796, 61864, 9040,  22268, 44404,
                29788, 24052, 41964, 32000, 21424, 36064, 49840, 516,   8400,
                57680, 14196, 25440, 48416, 6092,  16600, 11608, 52384, 31800,
                62600, 22376, 59160, 53300, 4796,  24712, 9864,  17360, 1412,
                47792, 51636, 57344, 4996,  17736, 53392, 33724, 56480, 9280,
                38444, 57800, 16604, 60256, 22676, 34772, 308,   47204, 59768,
                34136, 1852,  46904, 28052, 4256,  52760, 30412, 61320, 26392,
                18576, 9936,  28260, 49140, 7556,  12432, 42744, 24248, 64600,
                20496, 11660, 34980, 63000, 5320,  38992, 15608, 56892, 35656,
                160,   7716,
            },
            {
                14296, 57976, 10912, 3388,  40764, 32460, 688,   65304, 52240,
                4456,  35796, 19752, 62996, 49204, 11316, 52172, 30632, 64908,
                14400, 45444, 6820,  63748, 1384,  15540, 35464, 23436, 59220,
                51972, 16720, 57444, 28600, 23388, 59660, 31652, 64204, 11848,
                4844,  29576, 34972, 55892, 45848, 63084, 57596, 29000, 24508,
                35368, 9592,  59164, 23400, 38660, 28684, 48376, 3076,  58048,
                5880,  51020, 1632,  46688, 57084, 28100, 18264, 39224, 32356,
                29132, 42264, 53376, 18704, 39992, 60520, 27700, 4316,  48288,
                9568,  38528, 16196, 30024, 41604, 61844, 49328, 58564, 30892,
                39532, 26328, 14548, 43384, 29740, 49680, 6456,  44380, 64208,
                25276, 32796, 5060,  46444, 10792, 56080, 51396, 19872, 16204,
                43144, 21944, 12444, 36416, 57768, 21180, 11192, 49540, 2628,
                56496, 42268, 4400,  60412, 35324, 18668, 58288, 3768,  16184,
                51884, 30800, 48496, 8640,  58224, 32516, 50788, 9184,  30596,
                63240, 49644,
            },
            {
                25308, 38040, 28500, 62840, 18292, 50716, 27240, 11644, 22736,
                40516, 14180, 8612,  28528, 320,   33532, 58876, 21520, 1724,
                24536, 50164, 37000, 53164, 26192, 60024, 49032, 9564,  19152,
                42804, 6784,  48476, 41472, 34872, 45808, 8028,  50972, 19252,
                37644, 62636, 21388, 6924,  17300, 22756, 10816, 51544, 47460,
                63544, 14048, 33428, 53396, 5280,  43272, 16292, 54536, 39624,
                18364, 35072, 59408, 16780, 9152,  64248, 12412, 54844, 60576,
                46832, 6664,  63200, 1316,  30328, 46288, 56768, 35372, 19624,
                43136, 952,   56644, 11860, 33604, 2524,  27288, 37044, 12692,
                5676,  20468, 63452, 35732, 11576, 61800, 26932, 15364, 21256,
                12680, 49532, 62712, 19276, 38000, 26592, 62268, 30668, 9452,
                56464, 62796, 50120, 17476, 64416, 33288, 15308, 40240, 23092,
                63128, 13588, 54024, 22592, 39856, 26696, 46036, 32936, 36728,
                57340, 572,   40360, 21728, 26560, 13356, 3160,  59980, 23372,
                39444, 17632,
            },
            {
                60364, 5312,  45916, 52936, 23980, 8548,  57072, 44316, 17588,
                61484, 32228, 47228, 59644, 38964, 17912, 7160,  42044, 35568,
                59740, 18424, 11024, 33120, 19992, 41580, 4696,  38720, 29868,
                61668, 32912, 13308, 20644, 2696,  55292, 15876, 27536, 57612,
                1664,  44256, 49224, 28040, 53148, 42144, 31624, 7976,  4572,
                54528, 27472, 41200, 1424,  65348, 13372, 25424, 33332, 10656,
                63724, 27224, 12844, 49352, 40808, 5124,  43792, 25220, 3528,
                15976, 22652, 36716, 55652, 12944, 21020, 41680, 7560,  61252,
                27012, 50244, 23808, 45448, 52368, 15288, 21696, 54428, 45164,
                60888, 52560, 3136,  40964, 22696, 54916, 2216,  36692, 41812,
                54080, 1152,  28848, 41116, 14856, 6072,  44148, 2768,  37228,
                27352, 5188,  31688, 41532, 2244,  45952, 55724, 5940,  36632,
                45328, 34204, 30776, 47704, 840,   55380, 6956,  61040, 9616,
                43956, 14076, 61432, 52716, 45512, 36680, 42388, 19860, 47280,
                4208,  54456,
            },
            {
                9612,  20052, 32024, 13032, 43248, 38704, 34564, 5724,  49476,
                54908, 26112, 3612,  51720, 23820, 55928, 45240, 12828, 54740,
                29420, 5304,  43164, 57120, 7592,  62568, 13884, 56144, 24820,
                2084,  53740, 26804, 65500, 50288, 5640,  36596, 22464, 46744,
                32624, 9636,  15408, 60980, 2264,  37804, 59792, 19756, 36256,
                44520, 21584, 17324, 30744, 47104, 36056, 59292, 51940, 21304,
                45928, 7128,  37572, 23748, 30284, 53476, 34248, 20924, 37968,
                51244, 11244, 26352, 33704, 50020, 3452,  24708, 15112, 54792,
                32736, 17724, 65240, 8092,  36164, 62876, 6740,  42104, 9544,
                34112, 28192, 48352, 8896,  18648, 46484, 31036, 59572, 48096,
                7900,  35272, 22928, 58768, 50764, 32400, 54372, 23568, 49300,
                39924, 14232, 53816, 24256, 8200,  26744, 51912, 29612, 58468,
                17144, 7760,  20176, 11264, 65192, 14672, 29920, 22092, 49212,
                25648, 19412, 29256, 4568,  17392, 64852, 55148, 28304, 11764,
                32888, 44052,
            },
            {
                51048, 36072, 64776, 2932,  58172, 16084, 62044, 29700, 1820,
                37452, 20732, 10976, 43344, 14852, 30400, 62528, 26568, 3672,
                47176, 61556, 14592, 23364, 28536, 45056, 34396, 50712, 16320,
                46324, 36860, 9216,  39544, 17792, 29640, 42592, 62212, 13432,
                53400, 25780, 40636, 34444, 11980, 24724, 46860, 16188, 56728,
                2872,  62248, 49608, 10416, 56044, 20104, 6124,  300,   30844,
                42400, 55772, 61444, 20068, 984,   62440, 47148, 8804,  58156,
                65316, 41480, 45592, 7748,  58384, 38604, 64372, 51756, 11380,
                37336, 2928,  40520, 28476, 19660, 47544, 31544, 57036, 17984,
                23320, 15124, 59296, 32096, 57468, 38880, 4364,  11120, 25996,
                17248, 56692, 45408, 3628,  9796,  64864, 17880, 12288, 58352,
                20360, 61392, 44372, 10960, 58988, 38728, 19800, 12296, 1300,
                25716, 51340, 59876, 37916, 43688, 51988, 17672, 41400, 54324,
                2548,  38380, 55900, 34664, 10856, 24176, 1780,  49208, 62120,
                15724, 26888,
            },
            {
                1120,  23120, 48556, 29116, 7184,  21052, 48024, 24584, 13596,
                41764, 58000, 34728, 63928, 4932,  36588, 9208,  49972, 19888,
                39064, 31456, 52552, 37636, 48536, 312,   10900, 21056, 63580,
                6140,  57852, 22840, 44696, 59872, 11252, 56904, 3316,  38976,
                7208,  64288, 20836, 57228, 50452, 6164,  64772, 30000, 39820,
                12520, 33836, 23940, 6896,  44212, 27980, 40468, 63016, 50012,
                15372, 3636,  32972, 51352, 39160, 14768, 27256, 17928, 31764,
                2188,  29028, 19388, 61112, 16512, 28560, 44232, 31200, 5420,
                46016, 60048, 13976, 57628, 4036,  39244, 25544, 1692,  49728,
                64944, 37588, 888,   44076, 25016, 15632, 64480, 51408, 20116,
                62944, 31464, 13388, 37632, 25416, 46876, 35768, 41888, 7016,
                33404, 3124,  29252, 18828, 49400, 34540, 61628, 43528, 48184,
                64080, 41032, 28124, 3520,  24396, 33512, 4504,  62412, 12216,
                32196, 63648, 47116, 6856,  50008, 31816, 39240, 8000,  34892,
                41076, 56944,
            },
            {
                61032, 38376, 14864, 53368, 41508, 32664, 55716, 9372,  64624,
                52496, 6928,  28020, 19172, 49052, 22292, 40692, 53656, 15944,
                25088, 2568,  9668,  63888, 17240, 55272, 40428, 31348, 27636,
                42112, 12720, 32036, 712,   51968, 33820, 23976, 19092, 49360,
                30688, 16704, 43916, 4236,  28320, 14236, 43280, 51960, 8720,
                26712, 59456, 53732, 38084, 61164, 16668, 12188, 23184, 37164,
                10112, 26228, 17648, 43616, 11216, 59256, 5812,  55828, 44176,
                13720, 53516, 4496,  48428, 10332, 23004, 1980,  19052, 48728,
                25476, 21540, 42872, 53244, 10632, 51068, 59732, 13164, 29456,
                40736, 6004,  53564, 12252, 50284, 7484,  28620, 34208, 44524,
                39472, 5476,  52732, 61340, 20684, 29496, 792,   60556, 26348,
                52872, 47776, 36560, 65416, 1684,  15148, 6808,  23708, 32204,
                5084,  15360, 56656, 9940,  49996, 59192, 36388, 26884, 45824,
                8588,  16268, 22964, 41616, 60704, 57296, 14976, 21544, 52900,
                5932,  18716,
            },
            {
                12192, 43852, 8360,  26224, 59912, 288,   36404, 18840, 44508,
                31448, 15828, 46380, 60800, 1072,  56864, 32756, 6180,  65460,
                58076, 45520, 33548, 21656, 6508,  25580, 60172, 51804, 3112,
                48616, 19188, 62320, 26208, 14800, 47664, 8260,  54436, 35556,
                58328, 420,   47944, 37096, 55556, 33472, 22844, 1088,  20244,
                47924, 42392, 15168, 1944,  51436, 33180, 57012, 48156, 29052,
                60716, 52488, 58008, 7824,  29280, 48548, 34572, 23604, 50036,
                36480, 21748, 40392, 32672, 54252, 37080, 57260, 62640, 34820,
                55244, 9024,  30276, 34140, 23668, 16388, 35660, 43880, 8504,
                55512, 26184, 21128, 61952, 36176, 42384, 22276, 55044, 380,
                10040, 27372, 42528, 16544, 49456, 8196,  56160, 14780, 44800,
                9628,  16476, 22688, 54460, 41692, 30372, 56236, 52344, 38468,
                21204, 35404, 45532, 31328, 18880, 13760, 7632,  40012, 21148,
                52204, 58856, 27620, 13116, 19260, 29512, 44876, 64064, 25332,
                46976, 30620,
            },
            {
                54384, 33564, 63820, 17944, 46372, 11944, 51532, 27076, 39504,
                2616,  22924, 53972, 38396, 13180, 44688, 27052, 10844, 35512,
                41740, 13380, 50160, 57308, 43116, 36032, 14292, 8748,  38488,
                56632, 35176, 53824, 6888,  38196, 64524, 28816, 41424, 10740,
                26472, 22504, 61648, 9468,  18548, 60456, 38580, 54620, 63340,
                35560, 5200,  31700, 21796, 26172, 8216,  43404, 2676,  6568,
                41024, 21528, 46028, 36360, 64560, 20620, 41768, 1296,  62752,
                9712,  57168, 25588, 63964, 14420, 42820, 7320,  12268, 39448,
                15596, 61756, 552,   46872, 64036, 6328,  20876, 61480, 47988,
                18544, 34476, 46440, 30816, 17212, 4700,  60848, 13240, 47772,
                59040, 23408, 55548, 2268,  32032, 39864, 51452, 23484, 31432,
                62988, 39620, 6028,  12080, 25348, 46396, 10060, 17916, 62460,
                11948, 54008, 1008,  22508, 63736, 47832, 54564, 57964, 428,
                35192, 43172, 3784,  37700, 6052,  52040, 852,   10184, 37048,
                58712, 3844,
            },
            {
                27416, 50200, 21780, 5112,  39880, 23556, 63068, 5792,  47796,
                59952, 11152, 34268, 7952,  30128, 17708, 59824, 48016, 21080,
                844,   18400, 27708, 4628,  29992, 47412, 19648, 64796, 24208,
                16712, 10220, 45912, 21876, 42880, 17468, 2192,  59776, 45488,
                14588, 52468, 40904, 31224, 25072, 6648,  46032, 11756, 29224,
                17720, 57424, 10088, 62348, 47020, 36652, 64968, 19776, 54052,
                34508, 13288, 1976,  24864, 4724,  16024, 54208, 12960, 39268,
                30624, 17288, 6872,  46676, 708,   30088, 24304, 52080, 27656,
                5744,  50432, 18728, 37896, 27860, 54320, 39780, 31868, 4448,
                14456, 63828, 10212, 2296,  57664, 53100, 40160, 18944, 30064,
                37072, 14572, 34912, 45300, 11376, 64196, 18592, 37220, 3332,
                56976, 27908, 49088, 58380, 33744, 60652, 3944,  27216, 43096,
                7368,  48944, 59520, 41596, 37416, 3064,  24008, 29564, 14644,
                49232, 30984, 65456, 54196, 46724, 26060, 40280, 32496, 13932,
                20664, 41676,
            },
            {
                16864, 1396,  56356, 35432, 29272, 58280, 14440, 33764, 17400,
                56224, 25884, 42612, 50188, 63632, 23612, 3880,  51916, 30628,
                61632, 54148, 39696, 62896, 10776, 53308, 1160,  33292, 45132,
                5020,  30184, 61012, 27312, 4180,  51660, 32188, 21384, 6324,
                63524, 34172, 4680,  12808, 51440, 42236, 16440, 3368,  50304,
                24740, 44720, 52884, 39956, 18180, 12484, 30908, 25060, 15980,
                61520, 31276, 56076, 39700, 50908, 32620, 60248, 28220, 47616,
                3364,  58968, 35756, 50884, 20232, 45004, 59532, 17492, 41832,
                32464, 57540, 43984, 13536, 58800, 2592,  11988, 24672, 57144,
                52124, 37732, 43044, 27752, 48612, 23984, 32668, 8368,  65452,
                51996, 5820,  62180, 21500, 57128, 26756, 4764,  47320, 12736,
                19868, 42828, 124,   17448, 38276, 20916, 50912, 34956, 57784,
                30616, 24764, 17184, 27972, 9412,  33116, 42484, 18032, 61076,
                9976,  19920, 24336, 11856, 34036, 61376, 18168, 55428, 48900,
                63248, 8704,
            },
            {
                37924, 61804, 47768, 10664, 42276, 53852, 8536,  49796, 29784,
                4416,  62152, 19824, 2280,  54788, 41328, 15268, 37788, 43748,
                24520, 8732,  35064, 15988, 23224, 59708, 41108, 27484, 54560,
                58948, 39252, 13264, 48708, 57160, 35940, 12172, 55276, 39340,
                28344, 17660, 57268, 47096, 65388, 27420, 58432, 32832, 61096,
                37248, 13468, 540,   28216, 55872, 3912,  58716, 51056, 44636,
                8964,  49156, 18616, 62968, 11708, 44860, 8332,  25048, 19228,
                53000, 42480, 11652, 27136, 55912, 34012, 9632,  65156, 47720,
                3412,  21136, 26044, 8624,  33084, 42224, 50880, 45752, 28372,
                252,   22692, 59336, 6976,  15232, 38188, 45540, 2648,  25548,
                41020, 18116, 28692, 49028, 7284,  33912, 43440, 54296, 61684,
                35540, 52136, 32028, 64108, 8496,  13504, 44760, 2064,  40184,
                13828, 65116, 4188,  51880, 12996, 62016, 46624, 52796, 6408,
                38324, 44932, 2128,  56548, 15932, 8440,  43124, 5528,  23888,
                29304, 45252,
            },
            {
                13176, 24980, 31048, 15856, 3616,  20072, 37004, 43712, 22240,
                38968, 45680, 14812, 36864, 27720, 33124, 9092,  56984, 5916,
                12980, 58512, 46272, 2784,  50852, 37448, 12748, 7996,  21312,
                50072, 2436,  34128, 19040, 8664,  62244, 46352, 25140, 49612,
                1796,  44484, 37664, 20784, 920,   35744, 22232, 9640,  40960,
                7064,  64568, 22392, 34764, 49120, 7628,  41736, 37524, 1516,
                27112, 36180, 6032,  28668, 22724, 57480, 37348, 4336,  64988,
                33168, 22248, 62376, 15788, 3108,  38808, 5268,  22896, 36552,
                11748, 63080, 53108, 48892, 22808, 17192, 65064, 8088,  19348,
                40996, 33612, 11272, 50652, 20704, 62400, 12392, 58072, 49388,
                10876, 44416, 3668,  38616, 13332, 59512, 16384, 29452, 9968,
                25208, 7396,  22868, 46148, 53836, 30020, 62792, 23308, 18988,
                47016, 33508, 38252, 43672, 58388, 21460, 4772,  26268, 56956,
                32728, 62584, 49916, 28812, 36952, 22108, 51332, 59916, 2620,
                35312, 53028,
            },
            {
                6236,  59528, 43536, 51944, 65524, 27588, 61092, 800,   12740,
                51796, 32220, 6552,  57408, 11688, 46816, 20300, 64320, 28764,
                49524, 32304, 20032, 26428, 44356, 31360, 18472, 63108, 43192,
                15152, 25468, 64292, 41064, 22988, 29504, 15640, 7264,  19496,
                61080, 11164, 30164, 8264,  54184, 14028, 49480, 55740, 19176,
                47876, 31292, 15676, 42932, 60804, 20348, 13956, 22976, 62672,
                11256, 54316, 42928, 47764, 192,   17204, 41896, 50104, 14312,
                45528, 7852,  40252, 31052, 60916, 48760, 53632, 15068, 56680,
                31216, 40248, 1508,  35292, 61132, 4972,  30364, 36468, 59804,
                15532, 63312, 44804, 31236, 56128, 4928,  29180, 34744, 21788,
                54852, 31524, 60396, 52276, 25432, 41420, 22264, 1264,  50032,
                39408, 57992, 14720, 4576,  26056, 41088, 56436, 6680,  52096,
                10168, 54848, 20288, 7732,  30472, 15928, 36060, 40412, 12420,
                21956, 15320, 41104, 5212,  47164, 63800, 30696, 39112, 11448,
                57872, 19932,
            },
            {
                40364, 2016,  22288, 34804, 7704,  46112, 24500, 54996, 58504,
                9132,  64716, 24720, 48716, 60780, 12,    52460, 35452, 16552,
                40696, 4964,  53404, 65096, 7044,  57356, 1872,  47728, 36192,
                31956, 6764,  51420, 45000, 240,   53628, 38368, 42616, 56820,
                33664, 52188, 23816, 59232, 39868, 31928, 4876,  44940, 28540,
                2288,  58220, 52332, 9736,  26072, 32988, 52980, 29768, 46364,
                57212, 21184, 14804, 59924, 30860, 53776, 34808, 10464, 29488,
                56188, 1104,  52192, 24704, 10392, 19060, 29276, 25612, 45908,
                7428,  18060, 28468, 14264, 45420, 39120, 55336, 12572, 48492,
                53240, 6312,  25680, 17604, 36076, 53472, 42124, 17000, 63184,
                1552,  14876, 35436, 19804, 9212,  54968, 46920, 65152, 33384,
                18096, 43876, 60968, 36852, 49828, 11156, 16896, 36140, 29240,
                60760, 364,   26720, 45664, 56096, 50532, 64632, 988,   29260,
                51564, 8044,  25700, 58612, 18836, 32,    13668, 26252, 17484,
                46888, 32640,
            },
            {
                26996, 49472, 56828, 18124, 12276, 40548, 31668, 16132, 34984,
                19292, 28676, 40328, 17140, 22048, 42188, 26024, 44904, 3196,
                23452, 59604, 36348, 13812, 41980, 22720, 52648, 28396, 11104,
                58576, 55200, 17556, 10528, 60292, 32876, 4348,  64708, 27744,
                13936, 3836,  48408, 16156, 43868, 63816, 24932, 16824, 60288,
                39528, 24196, 36524, 4296,  47376, 63680, 6268,  39516, 16516,
                33828, 2864,  25912, 38672, 7932,  63508, 20720, 58928, 26076,
                38176, 17636, 47416, 35976, 44144, 57748, 41208, 36,    51764,
                62224, 43516, 59464, 54592, 9768,  20372, 26272, 1932,  23044,
                29736, 3324,  57548, 40320, 9548,  760,   47652, 26852, 7772,
                39500, 23996, 48140, 63636, 2556,  37040, 30912, 6248,  12916,
                53484, 3284,  29040, 19796, 32984, 1416,  59704, 24716, 39672,
                48916, 15468, 63056, 35008, 3128,  24588, 10324, 48480, 60136,
                43884, 55012, 34944, 11224, 53304, 33484, 42088, 49872, 55132,
                7908,  64232,
            },
            {
                15076, 9924,  37084, 28800, 59884, 2396,  50676, 5456,  47852,
                43036, 2560,  54060, 36268, 5076,  31968, 13552, 54484, 62196,
                11020, 46912, 17940, 29628, 9716,  34888, 15680, 61332, 4672,
                40688, 23492, 30300, 36980, 13688, 25900, 50544, 9652,  22124,
                37088, 41704, 62772, 26908, 2984,  10316, 37868, 53048, 12260,
                7916,  20188, 45272, 12880, 55164, 18436, 748,   59600, 8808,
                42184, 65396, 52236, 32592, 13080, 24580, 3120,  43844, 5760,
                60552, 21352, 63428, 4096,  14020, 6600,  64440, 33736, 12840,
                20944, 37568, 24164, 4428,  32404, 49468, 58396, 42988, 34080,
                46416, 38092, 21064, 49712, 62856, 23836, 13512, 59012, 51628,
                43828, 57460, 5768,  28404, 44868, 15420, 60708, 20908, 41052,
                25672, 46596, 10096, 64548, 54656, 43080, 46356, 13152, 5980,
                31824, 42028, 22992, 11976, 39152, 18996, 42664, 33780, 16608,
                20152, 2440,  64444, 39716, 23704, 45508, 61880, 4544,  36120,
                23276, 43760,
            },
            {
                53296, 62328, 3992,  48188, 54524, 21040, 63932, 38148, 23024,
                61424, 14204, 50172, 10512, 63596, 58900, 7968,  18964, 27704,
                38504, 51364, 696,   56212, 62332, 45996, 25952, 38236, 20456,
                49076, 1568,  62648, 47308, 20088, 58028, 43632, 17624, 47824,
                55932, 7380,  19164, 35048, 51180, 20476, 29636, 61892, 34732,
                50312, 56912, 62572, 29896, 40828, 35040, 27648, 50640, 24100,
                48484, 19120, 4524,  45840, 40724, 51484, 47992, 15536, 53528,
                33052, 11808, 28140, 55504, 31748, 21772, 38244, 48444, 9136,
                27940, 2552,  51016, 16592, 41224, 62556, 6772,  17860, 64456,
                8284,  11804, 60236, 14716, 28792, 45096, 32692, 19664, 36240,
                30032, 12492, 18392, 33748, 56256, 10736, 27092, 51304, 58852,
                34964, 56420, 22512, 38724, 5364,  17884, 27880, 63512, 56936,
                18792, 4292,  58212, 47388, 52436, 29892, 58632, 7308,  26880,
                37676, 47368, 30104, 14612, 7100,  27668, 9876,  20812, 58860,
                1368,  30580,
            },
            {
                19300, 39124, 24840, 32712, 41976, 14068, 26892, 10396, 30540,
                7480,  56180, 25712, 44424, 29140, 39272, 49560, 34152, 56596,
                6668,  32872, 25264, 39892, 19704, 2968,  50420, 56868, 13272,
                44068, 27252, 8444,  39784, 54660, 6060,  35476, 29264, 940,
                61460, 31280, 11544, 55060, 46772, 58188, 6892,  42300, 340,
                26752, 32612, 3532,  15896, 22848, 44500, 10824, 56468, 37480,
                13488, 29404, 56056, 10588, 61752, 17868, 36184, 30012, 39792,
                8944,  50540, 42164, 16432, 51872, 59992, 24484, 17076, 54348,
                61044, 34624, 57336, 46224, 11336, 29208, 36568, 14564, 50864,
                55776, 26624, 43276, 34748, 6484,  55196, 3184,  64996, 10004,
                4044,  62096, 53664, 41816, 23012, 48972, 38956, 3864,  8816,
                16708, 812,   49128, 14268, 30924, 52168, 8764,  21576, 50568,
                36548, 53736, 27304, 7924,  14776, 63668, 4200,  53560, 13304,
                61676, 4648,  22500, 50960, 60540, 55804, 32112, 15824, 40556,
                51512, 11836,
            },
            {
                56492, 46280, 5788,  16928, 7988,  35848, 52284, 44708, 59064,
                39596, 33260, 20428, 1320,  14868, 23052, 45836, 2052,  42832,
                21504, 63940, 12744, 43840, 53828, 6472,  30384, 9160,  33032,
                64404, 53260, 16364, 32140, 2344,  24028, 59976, 12472, 52600,
                44828, 24640, 40276, 15180, 2008,  32500, 23208, 14092, 48100,
                17364, 43372, 9048,  49636, 59068, 7004,  64332, 20560, 32084,
                6128,  60028, 35224, 22496, 27876, 7212,  58620, 876,   64876,
                22940, 45396, 1744,  34876, 8156,  45764, 2304,  30500, 44272,
                40036, 13640, 7576,  26136, 21296, 53512, 452,   24680, 40128,
                19744, 31080, 1756,  51928, 18488, 37860, 48292, 40760, 22524,
                49836, 25812, 38204, 7744,  432,   64224, 19044, 31312, 43812,
                63404, 27624, 41796, 61876, 24132, 58796, 40716, 34232, 3244,
                43968, 10648, 62504, 33228, 21252, 36420, 44652, 23832, 40212,
                55536, 32252, 43520, 17540, 1588,  48060, 37884, 64820, 45228,
                28056, 34596,
            },
            {
                2484,  22128, 60564, 43872, 57200, 62148, 316,   19672, 16260,
                3684,  47256, 62720, 35876, 57900, 53212, 17408, 11488, 60108,
                30708, 15352, 47936, 8876,  33984, 23124, 60472, 41792, 18332,
                5452,  35932, 22400, 61672, 42152, 49304, 15808, 39012, 33296,
                20260, 4264,  59008, 28232, 43524, 65160, 38648, 52720, 56296,
                21568, 64136, 36796, 53104, 25936, 38988, 15052, 46608, 1868,
                52728, 40112, 15644, 2376,  44188, 55176, 20408, 12580, 26588,
                56324, 18712, 62336, 25316, 40648, 56852, 11648, 62908, 5764,
                18980, 48952, 32088, 65164, 38676, 58016, 44928, 33160, 60872,
                4116,  47248, 63480, 23140, 59124, 11564, 27268, 16228, 56716,
                44260, 13984, 31488, 16876, 57912, 45288, 12548, 54284, 36644,
                21596, 11480, 33348, 7508,  37012, 2212,  48108, 12516, 29312,
                24408, 17264, 45992, 40820, 736,   50084, 17460, 28840, 48336,
                8096,  11324, 57700, 35696, 41540, 19964, 12336, 4884,  23872,
                8376,  62928,
            },
            {
                14452, 31252, 51084, 27476, 12136, 30824, 23772, 49536, 55388,
                28316, 12108, 51448, 8668,  31548, 5376,  64228, 25176, 36600,
                50848, 4040,  58588, 27356, 63192, 37776, 14836, 48048, 25204,
                59016, 50552, 12104, 45304, 9200,  27800, 65024, 6876,  47264,
                10196, 62940, 36948, 50080, 7948,  18312, 26088, 9984,  5256,
                40332, 28392, 12396, 19568, 1372,  33356, 54448, 28580, 62128,
                43380, 25760, 47860, 63496, 37908, 31744, 49784, 41808, 33776,
                47724, 5668,  37696, 13360, 29296, 19788, 50340, 36580, 26812,
                58700, 23064, 52372, 3372,  18208, 5884,  48812, 9620,  13200,
                53948, 35668, 16060, 8996,  41844, 31716, 61156, 7224,  34196,
                2232,  60088, 47152, 52036, 27516, 34900, 24320, 5976,  60532,
                50060, 45516, 56048, 52708, 20216, 44252, 16640, 64280, 55100,
                39116, 59752, 6340,  30496, 60772, 11624, 57320, 3040,  34960,
                20296, 62704, 25012, 5576,  52944, 30896, 26260, 54780, 49648,
                18268, 41800,
            },
            {
                36556, 55452, 9460,  18936, 37956, 47700, 6732,  34440, 65136,
                42092, 24824, 18416, 43512, 38400, 27324, 47636, 41092, 8308,
                18896, 54776, 39236, 16920, 2000,  51900, 10840, 55620, 1132,
                39204, 29364, 3592,  57476, 19564, 31372, 53676, 22628, 57048,
                27004, 17024, 53936, 22192, 12716, 46080, 61184, 36036, 30864,
                58652, 2772,  44700, 61368, 47916, 5468,  42072, 18012, 23412,
                12148, 7440,  19056, 53576, 9444,  24292, 3876,  61120, 8228,
                15392, 30796, 53704, 59484, 48052, 4452,  33212, 15628, 42212,
                836,   10372, 35980, 43892, 13960, 30948, 63260, 23284, 28508,
                42704, 5536,  25988, 56300, 46300, 152,   50504, 24200, 54116,
                19832, 36444, 5356,  21332, 9512,  61712, 40824, 15216, 28572,
                2404,  18376, 5176,  13696, 61084, 26924, 9688,  32524, 50928,
                1480,  13820, 48432, 26016, 54084, 22772, 42296, 64132, 15480,
                52272, 45152, 29196, 13748, 59000, 8700,  61468, 33964, 772,
                58296, 47180,
            },
            {
                25196, 4612,  40720, 64380, 3508,  54288, 41168, 13912, 9520,
                37176, 6260,  60956, 54540, 21640, 12656, 3080,  56348, 29248,
                44624, 23800, 35016, 46160, 21216, 31400, 43452, 28552, 20480,
                46056, 15728, 33872, 51976, 37648, 13768, 2728,  41264, 35608,
                1328,  42472, 31172, 5664,  34584, 57232, 1048,  51256, 14416,
                47464, 24596, 34560, 55404, 29972, 13908, 60404, 9376,  36612,
                50816, 56672, 33432, 29212, 13724, 45580, 17072, 52820, 38796,
                63708, 43732, 3148,  21648, 9720,  61544, 23656, 55088, 64112,
                47160, 29220, 56284, 61256, 27020, 41460, 55444, 37196, 17116,
                58756, 50620, 39804, 29932, 20316, 36648, 13220, 63652, 29396,
                40104, 11740, 64792, 42544, 30296, 4156,  53388, 46952, 56928,
                32388, 39776, 25400, 30524, 48844, 35592, 58428, 4792,  19508,
                23136, 56772, 34948, 18764, 8872,  38480, 32048, 6616,  26776,
                38004, 328,   50040, 39612, 22752, 46960, 43328, 14992, 38888,
                28984, 11140,
            },
            {
                62000, 33760, 50028, 22984, 15744, 59436, 25980, 20768, 58164,
                30136, 46668, 960,   15312, 58732, 50252, 34708, 61736, 14200,
                368,   65372, 11704, 5636,  57484, 61936, 4260,  36380, 64816,
                7468,  60788, 41132, 25820, 5264,  59072, 45652, 18176, 62204,
                51620, 14288, 48200, 64008, 40072, 28680, 23536, 42900, 20252,
                63168, 10628, 17160, 7488,  22656, 38388, 49036, 57936, 31248,
                3156,  64564, 39692, 1364,  60648, 57124, 35520, 28340, 22380,
                11472, 25608, 35800, 16612, 41756, 52216, 38136, 7520,  12760,
                21004, 39704, 16684, 8268,  51040, 20488, 1184,  7304,  46828,
                21456, 10872, 65448, 14424, 4624,  58244, 42856, 17688, 9188,
                48696, 26200, 55264, 15748, 49976, 37864, 19724, 10996, 23160,
                8132,  58744, 64732, 42180, 1624,  21860, 39560, 46244, 28700,
                43184, 37688, 65352, 4080,  51468, 14508, 46640, 54736, 58500,
                9904,  32424, 16800, 65224, 3436,  36188, 18812, 6200,  63640,
                21580, 52788,
            },
            {
                17296, 7648,  45084, 28776, 35832, 46312, 32180, 2888,  43832,
                53584, 22824, 33720, 28016, 40704, 6900,  17872, 22608, 52996,
                33040, 50024, 27672, 40912, 48332, 24916, 18216, 50348, 13120,
                24096, 53872, 10488, 21448, 63732, 49064, 33004, 11172, 24788,
                29516, 9016,  58124, 19996, 3456,  55136, 16400, 33460, 5836,
                53804, 37232, 50492, 42516, 65256, 26336, 19944, 4848,  45128,
                27072, 16108, 21884, 49880, 42632, 20056, 6652,  48488, 220,
                55204, 50324, 58128, 46612, 26952, 30152, 3796,  45292, 32128,
                53280, 2520,  34328, 24952, 46384, 11932, 33376, 52112, 61732,
                31104, 1828,  34468, 54544, 25224, 47588, 32392, 3260,  52904,
                44024, 1200,  32880, 23312, 59132, 2092,  33848, 63432, 43292,
                37428, 50988, 11588, 15692, 53424, 7600,  55788, 13396, 62948,
                6796,  10304, 31628, 44896, 27760, 61580, 2688,  19124, 23804,
                43388, 60664, 20820, 55624, 11500, 28436, 54336, 50148, 31864,
                44560, 3008,
            },
            {
                58064, 20500, 60640, 12976, 1452,  10236, 63388, 51104, 16520,
                5180,  62596, 49200, 10788, 64684, 44944, 30584, 37756, 9480,
                43028, 19936, 55412, 7408,  14340, 33232, 9532,  40528, 56388,
                32260, 48196, 52,    43896, 16008, 27344, 7784,  39616, 55888,
                4436,  44468, 37728, 26240, 12100, 50468, 8908,  60832, 45932,
                27280, 3648,  31024, 57088, 56,    11640, 34176, 53832, 41040,
                13048, 47192, 36104, 5224,  26100, 11844, 32224, 59392, 40920,
                18844, 7328,  32836, 1800,  64900, 14484, 58784, 18732, 25960,
                60844, 48252, 58056, 5208,  64604, 39940, 56996, 15080, 24392,
                38820, 45196, 18240, 49740, 7684,  62832, 38640, 22972, 60744,
                35264, 18640, 62060, 7936,  45216, 14108, 26728, 48804, 16632,
                72,    20672, 34368, 44948, 24036, 61916, 17608, 33636, 51856,
                25520, 54748, 15760, 58084, 21572, 41372, 36284, 30144, 12644,
                47680, 4888,  34836, 48876, 41664, 23696, 60212, 9908,  25632,
                13228, 41212,
            },
            {
                27584, 38128, 31712, 52220, 56260, 41824, 19016, 37392, 27044,
                12588, 39732, 18868, 36688, 2132,  25076, 56776, 47044, 59688,
                5260,  16412, 30428, 60972, 36772, 53536, 59420, 26968, 2776,
                17528, 37948, 29760, 56980, 35912, 53568, 20504, 60372, 47040,
                21976, 34304, 16756, 60188, 46536, 41700, 30160, 38348, 13292,
                58472, 40876, 21356, 14696, 44428, 51528, 60952, 8692,  23884,
                59676, 55392, 9988,  62052, 52564, 38572, 65324, 15208, 45720,
                30352, 62448, 12528, 20716, 39876, 9256,  50180, 35908, 42960,
                10524, 15196, 22960, 37036, 30568, 17832, 27916, 43340, 3800,
                9332,  60544, 28268, 41668, 21076, 11680, 27784, 15564, 5892,
                57020, 13336, 39248, 29308, 53652, 41464, 56376, 6272,  59696,
                29660, 54156, 27040, 3228,  37808, 31088, 10688, 41524, 604,
                48568, 18492, 39140, 1876,  11908, 49704, 8168,  64556, 51632,
                39428, 26396, 8572,  30552, 1420,  16236, 38296, 4592,  56508,
                35840, 48584,
            },
            {
                54724, 9416,  4908,  25160, 47604, 22544, 7464,  61148, 48304,
                57180, 30972, 8032,  55280, 51784, 15908, 12056, 3740,  26500,
                40096, 63296, 24160, 45704, 1000,  20880, 44140, 6524,  63268,
                45868, 22660, 62296, 5740,  12664, 42620, 1580,  30732, 15184,
                52264, 63264, 344,   31512, 6992,  21620, 64880, 2660,  24664,
                18744, 9420,  63400, 36096, 27852, 18036, 39424, 30204, 2504,
                19364, 33024, 28652, 17268, 44764, 22936, 3144,  27232, 9464,
                36544, 24172, 43064, 56584, 34632, 54480, 22484, 400,   63796,
                29368, 7172,  41296, 52712, 2904,  9808,  48136, 63288, 36168,
                52888, 13300, 56908, 5080,  35760, 52328, 56152, 45896, 30644,
                50648, 24920, 47408, 4676,  20860, 10132, 31880, 24156, 36500,
                46144, 9820,  62788, 57744, 47072, 50456, 27572, 59848, 22864,
                35820, 60836, 29400, 53248, 24388, 32652, 59328, 16976, 1116,
                54176, 14668, 63532, 57004, 44840, 52128, 33096, 46760, 64640,
                15580, 76,
            },
            {
                61208, 43708, 16212, 65520, 38920, 14556, 29880, 34124, 672,
                23736, 42740, 60224, 21752, 32720, 41240, 62960, 20596, 49172,
                33660, 13008, 51072, 10384, 39228, 16064, 29580, 49848, 35020,
                14448, 8384,  51172, 19032, 32580, 49812, 65492, 37148, 6676,
                11812, 42068, 24384, 56608, 53596, 35720, 17684, 51820, 55884,
                34436, 49852, 46616, 6920,  54516, 4308,  48396, 15200, 64348,
                50872, 42284, 592,   58132, 8040,  35032, 54816, 48784, 57668,
                4552,  52844, 16416, 47804, 5900,  26408, 45964, 17156, 38652,
                55580, 50464, 19820, 62208, 44592, 58704, 22372, 6112,  19588,
                26332, 48504, 32240, 16068, 61308, 2060,  42772, 9508,  20200,
                40780, 10864, 63840, 36000, 60040, 51028, 17876, 65476, 13000,
                4220,  39888, 18620, 13844, 6720,  20240, 4092,  44472, 14104,
                4976,  45696, 8928,  63344, 47284, 4484,  44168, 27448, 21912,
                35864, 43048, 18900, 24668, 13104, 61700, 21244, 8100,  18860,
                30292, 23620,
            },
            {
                33368, 50424, 20100, 34992, 57656, 4356,  51780, 55076, 10644,
                45892, 15388, 4272,  27848, 47696, 6412,  36048, 29392, 53968,
                1712,  57288, 35396, 4900,  56100, 64964, 23676, 11204, 41352,
                55216, 27992, 39104, 58108, 23640, 3748,  25968, 17756, 58776,
                28492, 49392, 38644, 9760,  14584, 27736, 47600, 6296,  43284,
                29652, 1572,  59524, 23316, 32572, 62476, 25344, 35808, 45060,
                6292,  38056, 24956, 47848, 31060, 12508, 41368, 17976, 21816,
                39524, 61024, 28748, 2360,  63052, 31404, 12020, 60008, 6428,
                24516, 32244, 11552, 34736, 25856, 13948, 54512, 33988, 41220,
                59508, 372,   22780, 39788, 47364, 25004, 33540, 64892, 3924,
                58616, 33184, 608,   15848, 27816, 40088, 2900,  43472, 54796,
                49908, 32108, 23496, 42780, 34160, 64932, 53360, 38856, 56660,
                31948, 21068, 40348, 15220, 34716, 19632, 38512, 56524, 10724,
                31828, 58820, 6276,  50508, 36988, 2456,  27332, 43128, 38972,
                52604, 11352,
            },
            {
                6448,  40772, 26916, 1304,  11700, 44032, 25496, 40148, 19288,
                63988, 35132, 50668, 11508, 61656, 18500, 58580, 43592, 9272,
                17120, 22164, 42764, 27264, 47564, 32012, 52724, 3752,  19616,
                59904, 1776,  44780, 10176, 61412, 47708, 40168, 55028, 45368,
                33164, 2980,  18536, 45072, 62728, 2028,  39628, 12176, 62384,
                15772, 26284, 10908, 39760, 13624, 42972, 9600,  57956, 21328,
                11916, 54892, 14632, 63704, 20452, 51504, 62316, 1156,  33048,
                14032, 50436, 11044, 38256, 18672, 51608, 41936, 34020, 48852,
                2604,  45716, 57488, 17216, 892,   39264, 28452, 49684, 16372,
                10988, 43952, 64376, 8304,  29448, 12208, 17596, 37640, 48792,
                27116, 52652, 23084, 55476, 45820, 7692,  33936, 22072, 27480,
                15912, 61016, 56620, 51552, 9124,  29668, 15992, 25096, 10024,
                62436, 50956, 6108,  26412, 55228, 12800, 51196, 7196,  62428,
                46304, 3384,  28872, 40388, 10272, 31352, 59168, 55664, 3656,
                62900, 45724,
            },
            {
                13764, 59504, 54304, 30700, 61408, 47188, 16908, 60076, 28556,
                2432,  38628, 54092, 24604, 44736, 8,     13572, 24108, 38248,
                64344, 31160, 60392, 18484, 13604, 7360,  37880, 61448, 33444,
                47076, 24728, 31460, 17060, 34612, 7456,  13992, 21032, 9288,
                63584, 22836, 57440, 34628, 49936, 23948, 59476, 32784, 21920,
                37840, 53556, 44812, 20280, 56384, 3320,  18652, 52704, 31560,
                28032, 60792, 34592, 4168,  39016, 6700,  28336, 59268, 46208,
                25572, 7236,  34464, 45028, 23632, 8292,  57272, 15244, 20804,
                28108, 61516, 7652,  42116, 51948, 64940, 8884,  4512,  56532,
                30388, 37016, 54616, 19416, 50272, 58956, 53384, 22440, 7788,
                13628, 43176, 18960, 37544, 11676, 61976, 57208, 47888, 9324,
                37912, 6152,  992,   25984, 37244, 59248, 2164,  48940, 34812,
                18100, 28548, 57892, 42760, 208,   64236, 29708, 23664, 40952,
                15860, 49452, 21120, 63180, 54096, 17348, 47428, 14532, 25500,
                35448, 21716,
            },
            {
                2800,  48000, 17768, 8404,  36244, 22012, 5520,  32916, 8892,
                58412, 20652, 31436, 7404,  40004, 33888, 55796, 48080, 52988,
                7084,  45984, 2488,  40000, 58520, 21636, 43664, 26368, 16256,
                6000,  53140, 12856, 56060, 43588, 27236, 51264, 36252, 4160,
                52860, 27468, 13316, 5152,  30212, 19428, 42436, 9276,  57312,
                48328, 4940,  65104, 30544, 50664, 34616, 61384, 38924, 2236,
                46504, 8828,  23112, 53488, 43820, 16020, 23788, 9892,  42568,
                55752, 64120, 20968, 58868, 53860, 29752, 4312,  65268, 39808,
                54632, 13184, 36972, 29960, 47400, 20856, 32680, 45412, 60764,
                23368, 2948,  14408, 34788, 6540,  41408, 1112,  61604, 45496,
                29840, 63236, 5592,  49284, 30256, 24796, 1584,  18512, 63096,
                30460, 41820, 20692, 47236, 12116, 44192, 21968, 40884, 54808,
                3560,  12244, 47476, 22208, 36940, 17436, 48220, 2564,  33680,
                54828, 25832, 12624, 44512, 34116, 23480, 5232,  41776, 9916,
                50824, 28820,
            },
            {
                32648, 38364, 24168, 63212, 41552, 50068, 13608, 53072, 43564,
                47896, 12848, 56720, 17236, 65280, 28952, 21380, 3700,  27808,
                15552, 50828, 24832, 9732,  54292, 48920, 392,   12228, 50752,
                40680, 62592, 21664, 38112, 1204,  64048, 57568, 30016, 42700,
                16344, 39724, 61268, 44112, 55004, 6948,  52088, 28640, 588,
                18440, 35120, 8120,  15436, 27836, 45792, 7664,  26144, 15956,
                50076, 40640, 18372, 59552, 29984, 50412, 36976, 53808, 30600,
                3328,  16636, 39912, 644,   12632, 47592, 25736, 36248, 9840,
                44284, 23764, 19240, 3872,  59176, 24744, 11892, 38656, 18352,
                51792, 47244, 27780, 62632, 45080, 25600, 31268, 15620, 35564,
                56484, 2708,  40580, 59280, 14908, 53092, 36840, 44732, 12476,
                49168, 54440, 34672, 63628, 17544, 53748, 32564, 7776,  64308,
                44984, 38712, 61388, 9180,  31760, 59356, 43700, 11604, 61428,
                8516,  37240, 60104, 496,   7956,  51268, 64492, 36764, 18540,
                60616, 56840,
            },
            {
                15696, 52452, 44568, 12300, 2056,  56108, 27272, 64472, 23444,
                36008, 4744,  26376, 45560, 49944, 10056, 59724, 42108, 35816,
                11524, 32596, 62880, 36892, 28288, 34276, 64012, 30324, 57836,
                35492, 8916,  28928, 49460, 5604,  18388, 23144, 11516, 59832,
                48596, 704,   35992, 10504, 16948, 64496, 36352, 14656, 44388,
                61576, 24736, 41644, 60016, 1288,  22912, 13016, 42364, 56844,
                64680, 32760, 1256,  11656, 47304, 3664,  65056, 13460, 19576,
                35544, 49056, 27124, 31872, 62800, 41656, 17924, 51220, 1448,
                32776, 49564, 63696, 53420, 14848, 35524, 1848,  62912, 26656,
                7192,  40896, 10124, 56120, 20780, 4284,  57984, 39396, 10092,
                17744, 23824, 34024, 9524,  20932, 42720, 32476, 4824,  59624,
                24616, 14816, 8412,  28956, 4872,  60608, 26904, 14036, 20276,
                30200, 24424, 16128, 51772, 4140,  27032, 53960, 20808, 39568,
                17780, 29352, 57024, 19580, 38940, 28092, 55160, 31804, 1388,
                43552, 7340,
            },
            {
                64760, 20632, 5220,  29176, 33676, 18848, 38736, 3444,  16144,
                41388, 52360, 59440, 1436,  14636, 37252, 5736,  19280, 61828,
                57140, 17996, 44400, 4112,  14788, 19792, 8336,  45616, 23344,
                2920,  19248, 44964, 59208, 32408, 41272, 47124, 6640,  33464,
                25036, 19972, 51680, 31812, 25840, 40188, 22132, 56216, 49748,
                31552, 12676, 52632, 37724, 56008, 48848, 62692, 36296, 20516,
                5508,  24492, 55092, 35792, 26600, 21152, 8176,  40856, 58348,
                6136,  52472, 44020, 14944, 7656,  34908, 22308, 56444, 61144,
                16972, 5412,  27596, 9352,  41252, 56328, 50812, 43408, 13584,
                33796, 53768, 16768, 32004, 37736, 13296, 51516, 47844, 27868,
                54216, 46348, 51240, 26524, 64172, 7140,  56564, 28360, 21808,
                39436, 3176,  58232, 42640, 38108, 50136, 632,   47716, 58340,
                36476, 2328,  56368, 34008, 41432, 14044, 5868,  24628, 50428,
                45400, 5480,  32628, 48456, 42644, 15424, 11004, 20948, 49288,
                39412, 26640,
            },
            {
                10120, 35260, 57256, 50900, 60692, 46436, 8532,  31300, 60332,
                10848, 29380, 34380, 22492, 42864, 31960, 54140, 24564, 48604,
                808,   40596, 22712, 52416, 60160, 41312, 55984, 16732, 39004,
                53592, 65044, 15056, 24852, 10296, 53920, 15576, 38520, 55240,
                65128, 8372,  45556, 60480, 2964,  47968, 4892,  10808, 26840,
                2752,  21164, 46964, 10188, 19100, 30928, 4528,  53900, 10744,
                29160, 44552, 14356, 62512, 42064, 56932, 32992, 46072, 25152,
                62036, 10964, 23132, 57040, 60344, 3640,  45472, 11580, 30076,
                40660, 58144, 37548, 45732, 31620, 17512, 6172,  29648, 58664,
                20272, 64284, 612,   48896, 61264, 23448, 8220,  65320, 20440,
                5968,  61212, 13348, 136,   38540, 47452, 16200, 10952, 50228,
                64428, 33300, 51840, 19948, 10208, 23608, 35128, 41712, 10564,
                52676, 7148,  48836, 19148, 65312, 46040, 57716, 35356, 63444,
                1832,  53508, 13480, 63936, 25212, 4136,  57968, 61364, 23988,
                13412, 54812,
            },
            {
                45644, 304,   40016, 25644, 15092, 22572, 43108, 56744, 49312,
                20212, 6576,  62608, 17612, 51464, 63672, 12696, 44952, 29092,
                8472,  54720, 30608, 6436,  47788, 25084, 1628,  49628, 26848,
                11684, 33552, 4776,  51980, 36516, 61872, 26928, 3380,  43476,
                13796, 29492, 22956, 54256, 12988, 57960, 33264, 42060, 63028,
                37112, 58476, 33556, 3852,  27296, 40536, 15584, 33752, 46712,
                60436, 38612, 49832, 9052,  17628, 52020, 244,   15348, 29372,
                38428, 1984,  33840, 48228, 19848, 53256, 38076, 26800, 7180,
                47684, 12868, 20640, 48,    25740, 62280, 22256, 46780, 3016,
                39184, 24996, 44556, 5944,  29120, 42912, 34360, 1460,  41952,
                31580, 36424, 44180, 29360, 58684, 19452, 52316, 35964, 1108,
                45148, 17256, 25876, 46452, 56164, 15684, 61824, 29144, 17752,
                63336, 43644, 25708, 11364, 29036, 37876, 9624,  31240, 14984,
                27300, 41036, 22816, 9008,  52324, 30976, 44716, 34752, 6012,
                47500, 30676,
            },
            {
                22664, 16996, 62308, 11512, 5860,  53848, 34884, 848,   26156,
                37848, 46088, 55108, 39680, 7848,  26692, 2736,  38452, 16120,
                34648, 64840, 13556, 37824, 10660, 32840, 63516, 36208, 6256,
                60380, 41940, 48116, 29444, 140,   20536, 56780, 31356, 18816,
                50944, 2048,  41444, 17680, 37796, 28004, 19480, 51524, 16116,
                6844,  54388, 17312, 63592, 45052, 59316, 50624, 24748, 7728,
                17968, 3004,  22168, 30752, 4828,  37536, 63416, 21932, 49984,
                54508, 18328, 42420, 27748, 9692,  30996, 15204, 64164, 54864,
                24020, 34388, 52392, 59360, 48928, 11212, 37212, 55056, 9596,
                50360, 35256, 12232, 56456, 18420, 52772, 15248, 26188, 58276,
                11208, 16728, 22536, 54568, 4388,  33800, 25260, 61512, 41892,
                30312, 5828,  12772, 62624, 32248, 6940,  54488, 3848,  22224,
                31724, 14568, 39756, 59568, 21964, 716,   48268, 18452, 55868,
                7460,  36888, 58496, 45856, 17280, 38124, 2036,  53292, 18084,
                62732, 37520,
            },
            {
                59112, 49584, 42712, 31912, 48092, 65208, 10224, 18044, 61808,
                13484, 23964, 2012,  11712, 30512, 47864, 55840, 21856, 58452,
                49736, 19492, 26596, 45320, 57472, 21192, 13944, 44124, 30576,
                19876, 56092, 22528, 39304, 13540, 44648, 48732, 9872,  62808,
                37132, 57648, 34264, 6188,  64068, 44920, 8768,  60260, 23420,
                44228, 29412, 39640, 8060,  22304, 12344, 948,   64452, 42476,
                52100, 56764, 34312, 61372, 47112, 27896, 12132, 43656, 8580,
                59720, 13752, 64588, 4424,  40396, 49664, 1100,  42876, 18232,
                3252,  63024, 42160, 15452, 7072,  33412, 42672, 27564, 15996,
                61696, 30336, 21668, 40032, 59812, 10192, 46104, 55224, 38496,
                50112, 2532,  62828, 40256, 10028, 45904, 13952, 7712,  20840,
                57616, 53540, 37216, 2072,  28012, 44276, 39036, 51088, 45972,
                57732, 1728,  33528, 54548, 5192,  62508, 51556, 23500, 43208,
                62092, 19488, 33756, 4820,  60584, 26844, 12612, 40984, 28400,
                9036,  4104,
            },
            {
                33896, 26900, 20076, 3736,  38012, 27340, 40932, 30280, 51644,
                43720, 33200, 50528, 58560, 20040, 35612, 61228, 5444,  10956,
                39988, 1952,  60884, 4748,  51324, 28636, 7412,  55068, 50768,
                2180,  16036, 8868,  58832, 63864, 6560,  35024, 23560, 4444,
                26504, 46868, 11160, 50260, 24700, 1748,  30440, 39132, 13424,
                120,   48264, 25104, 51936, 57248, 32144, 38068, 20128, 28784,
                12852, 26216, 41148, 10832, 54652, 24236, 58940, 34952, 3828,
                31284, 36624, 46796, 23432, 55472, 60992, 21088, 35588, 51376,
                31500, 8848,  28592, 38928, 19816, 53236, 65204, 5068,  19272,
                57760, 7616,  47472, 1788,  27048, 36932, 3732,  19436, 23960,
                60156, 46680, 27360, 18556, 31796, 64696, 55104, 39028, 27528,
                10568, 48924, 22564, 41340, 59344, 9604,  20184, 12936, 27212,
                37384, 9348,  50056, 45088, 16480, 41280, 32216, 12016, 3544,
                47484, 28932, 51100, 10980, 21940, 48832, 65512, 20108, 57464,
                50736, 14336,
            },
            {
                56020, 8160,  53064, 59864, 15904, 21336, 58488, 7280,  55576,
                3896,  16536, 64408, 41904, 25188, 15036, 43256, 32176, 46220,
                24480, 52624, 33640, 15924, 41188, 62040, 18276, 39480, 24204,
                63216, 37024, 46836, 25776, 32484, 17200, 41332, 60604, 52476,
                15816, 21248, 31832, 61896, 15000, 53464, 47488, 56676, 33716,
                65508, 10640, 35424, 15012, 43060, 5928,  48992, 55072, 35200,
                58880, 1608,  15940, 44660, 5704,  18584, 39288, 49048, 20576,
                26132, 51752, 6156,  17056, 33224, 12584, 25828, 57588, 11068,
                45024, 22424, 56396, 4152,  60524, 24532, 13024, 46156, 32808,
                41428, 52940, 14892, 64644, 33008, 50948, 62532, 30560, 5308,
                33908, 14528, 7528,  53068, 49260, 23180, 5432,  43972, 60300,
                15836, 34288, 65516, 17852, 49732, 25124, 35384, 64180, 6080,
                60768, 23780, 20000, 28116, 7228,  35888, 26048, 60216, 39180,
                54220, 14260, 28,    42904, 31324, 54876, 6596,  33492, 43768,
                24184, 39216,
            },
            {
                2712,  44892, 12816, 35228, 50580, 1920,  46584, 12256, 35480,
                20792, 27396, 6068,  37432, 3308,  9336,  53456, 17808, 63924,
                29408, 8112,  43800, 22968, 36412, 2600,  46176, 9748,  33440,
                12604, 28740, 43156, 3576,  54712, 49764, 28356, 12636, 38080,
                43980, 59304, 7124,  42232, 19028, 35536, 5844,  21060, 26620,
                50792, 59388, 20048, 3248,  61492, 27388, 16652, 10248, 4664,
                47568, 38308, 21224, 65340, 32772, 52444, 2220,  15636, 63784,
                44884, 11620, 58264, 29076, 37888, 7876,  46252, 5072,  39688,
                60132, 13776, 47372, 50856, 35724, 30180, 1280,  49876, 22888,
                3492,  36408, 24692, 43348, 17848, 8556,  13620, 40892, 44236,
                10548, 56248, 37400, 42412, 3100,  16444, 29792, 36232, 1660,
                47332, 7288,  30680, 4380,  14096, 57364, 24,    47288, 53060,
                15284, 42156, 55356, 64476, 12704, 52280, 57064, 9104,  17580,
                35144, 23844, 62836, 57944, 36128, 15612, 46448, 1196,  10804,
                30268, 64084,
            },
            {
                18984, 29216, 23520, 61724, 30872, 43200, 24188, 63664, 38860,
                48372, 57572, 31876, 46428, 61956, 49660, 27976, 404,   36736,
                13940, 54952, 59080, 11284, 30916, 53176, 27176, 60244, 48868,
                57116, 6064,  52352, 19008, 10852, 21916, 2068,  55756, 8448,
                30108, 508,   55064, 39364, 27788, 58604, 11928, 43236, 17080,
                4492,  40492, 31752, 56208, 37156, 23252, 45160, 63092, 31368,
                23828, 60360, 50760, 29416, 9112,  57728, 42500, 30272, 7624,
                55904, 40176, 524,   62524, 43140, 52256, 65504, 28376, 16792,
                33316, 744,   26232, 17252, 10300, 44312, 55908, 38060, 59432,
                28516, 11780, 55552, 5100,  48996, 23172, 58592, 54368, 21348,
                64436, 28788, 24376, 61324, 35156, 11752, 57028, 51384, 19004,
                24732, 55720, 44908, 37904, 54160, 32752, 43204, 29560, 18564,
                33944, 3028,  30612, 38224, 46808, 19384, 2348,  30808, 45956,
                6468,  27456, 40840, 18252, 8164,  26080, 39648, 22460, 60628,
                16160, 47676,
            },
            {
                37124, 55116, 41512, 9932,  6240,  57388, 15240, 28760, 9116,
                52868, 14212, 22820, 11372, 55544, 19656, 40120, 59908, 22436,
                47980, 4248,  19844, 49280, 65420, 6592,  21724, 15668, 1468,
                41584, 23096, 34636, 65080, 38592, 58324, 45828, 33860, 63760,
                18360, 47668, 25376, 10016, 49640, 3904,  61624, 37564, 55008,
                9312,  28440, 45860, 12812, 52912, 7964,  40296, 51572, 18612,
                42768, 7768,  13860, 35776, 46956, 22632, 12332, 59956, 24632,
                33424, 17952, 22476, 48436, 13852, 19464, 2364,  23588, 49268,
                54688, 37380, 63968, 41140, 6768,  62292, 18524, 14272, 8324,
                63764, 45568, 31692, 60340, 39376, 29236, 35004, 2176,  47276,
                16208, 51684, 480,   19284, 58824, 44856, 26464, 63616, 39368,
                13140, 61744, 27772, 20464, 10680, 62868, 22468, 8616,  39824,
                49484, 57500, 10496, 5496,  25024, 61692, 42448, 22280, 65092,
                56040, 11388, 47836, 52484, 3500,  63412, 49792, 56268, 35104,
                52508, 7276,
            },
            {
                62548, 476,   48592, 18356, 39260, 53468, 34120, 19216, 640,
                40564, 60208, 7020,  43908, 29964, 34720, 12348, 7324,  56756,
                41672, 32860, 38984, 25732, 14360, 42304, 55564, 37928, 31636,
                18140, 59368, 14376, 48028, 30840, 7096,  26092, 13900, 50924,
                23260, 36084, 14540, 64936, 33416, 22456, 30968, 46756, 24364,
                62848, 49720, 1768,  17888, 64264, 30316, 680,   12220, 55424,
                26912, 3392,  56684, 18080, 924,   27652, 37136, 49912, 3060,
                53764, 36400, 9156,  26920, 32212, 55732, 35960, 42004, 61688,
                8596,  20228, 30880, 57508, 23868, 33544, 27464, 40836, 21440,
                51616, 16744, 352,   20568, 10252, 52572, 6208,  26304, 37892,
                8072,  32076, 40544, 5816,  48064, 21604, 8592,  3488,  32412,
                42032, 9168,  1188,  52140, 40632, 16676, 4840,  60424, 25636,
                12792, 63304, 44116, 17128, 34320, 49104, 14308, 39536, 50808,
                4472,  37052, 30244, 21548, 44080, 10152, 28504, 13580, 4376,
                40940, 25704,
            },
            {
                12112, 33652, 14796, 64972, 26456, 4652,  49760, 62488, 45376,
                25460, 35000, 18256, 65236, 1740,  24352, 52004, 45784, 26488,
                16548, 9888,  60812, 1236,  46524, 34776, 10440, 62472, 45176,
                53496, 9172,  27644, 624,   43928, 16772, 62156, 40064, 4176,
                43000, 60464, 53152, 2836,  44832, 16468, 52524, 7444,  13684,
                35684, 21364, 42220, 25848, 34792, 48192, 21864, 36608, 61888,
                32948, 39044, 44320, 63700, 52736, 40504, 62048, 19540, 46180,
                14468, 41468, 60612, 51200, 6344,  59036, 10880, 30360, 14956,
                5440,  43540, 12096, 46284, 52308, 1860,  54604, 48112, 5488,
                34608, 42736, 26724, 47984, 63328, 14344, 42280, 60796, 19684,
                49620, 62164, 13996, 54020, 29952, 34244, 15088, 49372, 53648,
                22788, 59188, 47552, 35244, 24592, 49980, 45588, 55948, 36572,
                1504,  28064, 21764, 53988, 58484, 584,   27184, 9980,  33056,
                19840, 58720, 13808, 60856, 33576, 38352, 58248, 19444, 31124,
                44792, 21484,
            },
            {
                50292, 28588, 57724, 45796, 22112, 37356, 12008, 30720, 56544,
                3728,  50632, 27824, 38472, 48068, 15304, 61652, 30944, 2340,
                64336, 50128, 29152, 56344, 17644, 51696, 3824,  28412, 24380,
                5300,  40108, 61708, 51136, 23896, 36872, 53720, 20532, 31860,
                10760, 6532,  29608, 19772, 38156, 57636, 768,   40776, 29568,
                58984, 5284,  55640, 10104, 59896, 15236, 57188, 5548,  16248,
                49424, 10508, 20572, 25508, 6780,  15568, 9484,  31936, 5584,
                64924, 29348, 44664, 1604,  17092, 40268, 21800, 46048, 53760,
                27432, 51080, 22716, 4056,  15764, 36788, 10720, 61716, 30004,
                56448, 12652, 58844, 37424, 33188, 18072, 45988, 57176, 11440,
                34484, 44296, 26772, 10032, 64752, 42652, 56640, 37128, 5156,
                17356, 30848, 13792, 57204, 3416,  32300, 28744, 14940, 52852,
                19224, 47240, 40424, 31164, 8236,  37456, 60680, 55156, 16232,
                44336, 25940, 968,   54424, 16644, 24120, 2112,  47440, 53912,
                9360,  59448,
            },
            {
                3916,  42688, 7920,  31980, 1648,  55416, 43464, 8544,  21428,
                16216, 41956, 57796, 10204, 53916, 4632,  36828, 19204, 43336,
                35604, 21608, 6100,  37512, 23072, 32008, 57840, 19956, 13268,
                47456, 36280, 20816, 12564, 57044, 9960,  2724,  59628, 45324,
                27192, 56232, 40508, 48564, 12248, 26944, 63572, 20292, 51252,
                16264, 38932, 31108, 50728, 6556,  43724, 28880, 46664, 24264,
                53924, 30060, 60736, 47736, 34064, 55816, 43364, 57928, 26300,
                11532, 20628, 54984, 24128, 34832, 48144, 64104, 4228,  38024,
                60712, 34512, 58408, 39208, 65100, 19780, 44376, 24812, 2796,
                39460, 22220, 7860,  4192,  54696, 23508, 1268,  30820, 24904,
                5292,  55940, 22368, 38412, 18020, 1136,  25040, 60316, 28696,
                44172, 63032, 38212, 19456, 8076,  65016, 11476, 42024, 6792,
                33816, 59548, 3928,  51344, 13128, 23596, 45776, 29824, 3220,
                63808, 48888, 41440, 7132,  50692, 64952, 42492, 6476,  61924,
                34180, 16748,
            },
            {
                54900, 36784, 20284, 52052, 60004, 14008, 24912, 48236, 33684,
                61508, 13124, 5700,  32264, 22056, 40872, 59088, 10812, 55488,
                13832, 52964, 44956, 11952, 63120, 8212,  39728, 49652, 64536,
                29836, 54732, 3212,  32748, 41916, 28936, 48516, 35088, 13204,
                51588, 17376, 23056, 62056, 7896,  35348, 54760, 10260, 43780,
                2720,  65028, 46100, 18664, 23616, 38336, 2652,  63856, 40604,
                8276,  1356,  14152, 37556, 4232,  29136, 22076, 48608, 35600,
                51732, 4008,  38344, 61868, 12888, 28716, 9088,  18800, 25008,
                1544,  16936, 8924,  47528, 28104, 32468, 59324, 14120, 53012,
                17800, 64592, 44632, 50372, 28752, 9364,  40612, 65444, 53032,
                17168, 2392,  59536, 46044, 6960,  52500, 12284, 20404, 46924,
                10708, 6244,  25604, 54172, 43676, 58788, 21532, 37964, 61780,
                23972, 9696,  16040, 64828, 43016, 18552, 5360,  52748, 35548,
                22908, 9516,  37712, 31988, 11492, 27520, 36264, 14580, 23148,
                39744, 27008,
            },
            {
                60912, 24312, 11276, 40140, 17440, 28196, 64388, 6336,  38096,
                52536, 23924, 46684, 63744, 17432, 26356, 49272, 7500,  23956,
                32676, 3548,  59244, 41172, 26516, 45948, 184,   16164, 35036,
                8716,  59204, 16596, 46192, 63420, 5852,  18992, 24896, 64352,
                1164,  36832, 59108, 4460,  31380, 42588, 24540, 47900, 33804,
                22636, 27068, 12684, 34228, 61816, 54264, 11032, 19808, 35184,
                58032, 43652, 23356, 51192, 59500, 11964, 17552, 212,   39592,
                59236, 16180, 32528, 7088,  49924, 57764, 43624, 32908, 55800,
                49160, 42488, 31288, 13076, 51336, 6024,  41008, 8648,  36156,
                48684, 31728, 25384, 15888, 61596, 35624, 47420, 13068, 37476,
                43212, 28460, 35236, 50184, 31140, 61836, 40272, 33064, 2168,
                56000, 50964, 40976, 33632, 15720, 27316, 720,   46560, 50488,
                29732, 56580, 38812, 26864, 34828, 49844, 62372, 39872, 13420,
                56832, 28348, 61784, 21196, 45708, 18624, 56556, 30080, 51664,
                956,   45960,
            },
            {
                6852,  48296, 63712, 2656,  34220, 50332, 41136, 19740, 58260,
                1948,  29148, 36504, 55308, 264,   44488, 29684, 62776, 39708,
                47336, 27912, 19484, 15260, 34324, 54088, 60504, 25252, 43692,
                4816,  41300, 26856, 23340, 52000, 14404, 58292, 39432, 8480,
                47520, 32804, 15136, 44516, 50684, 18184, 13600, 5848,  60168,
                56252, 8752,  52972, 336,   41728, 14828, 32296, 26604, 52072,
                17068, 31492, 65172, 18952, 40796, 45852, 53604, 63884, 8116,
                23456, 43068, 46776, 19244, 25616, 3596,  52668, 14712, 61612,
                7068,  20616, 62540, 25312, 44,    56124, 22784, 45832, 27640,
                57672, 1476,  10984, 42012, 56792, 19372, 3256,  21904, 51000,
                7548,  63124, 14476, 11064, 21472, 27160, 48532, 15444, 63972,
                35896, 22576, 12576, 3116,  48088, 52528, 35352, 18156, 5492,
                13644, 44056, 20604, 54804, 2804,  11016, 25880, 20788, 6840,
                43296, 17344, 2204,  53284, 59060, 3564,  49116, 10232, 63040,
                19548, 12600,
            },
            {
                35596, 15920, 30432, 44368, 57264, 9912,  4108,  31472, 45552,
                11568, 18636, 42460, 8900,  14364, 51248, 35092, 5172,  16508,
                56572, 61768, 9120,  51560, 2820,  21268, 30836, 11548, 56880,
                19432, 50268, 10948, 38200, 1352,  34504, 54504, 30664, 43228,
                20760, 54032, 9844,  26068, 55684, 3232,  62656, 39832, 30792,
                15784, 37900, 48472, 29520, 58100, 7220,  45740, 60376, 4852,
                12424, 47396, 3096,  27368, 7240,  32104, 24816, 34404, 14064,
                30196, 56032, 10292, 64460, 41056, 35468, 21928, 39176, 11732,
                27600, 36088, 53208, 43856, 18404, 63464, 34144, 16112, 60896,
                6660,  20976, 38568, 52364, 4992,  32444, 59972, 26444, 55456,
                33080, 19960, 41484, 58456, 2744,  37316, 57448, 8416,  42956,
                28136, 17936, 60816, 30252, 63568, 23776, 10860, 57740, 63196,
                33044, 1732,  48020, 7904,  31476, 58724, 46568, 33240, 59888,
                51328, 47480, 35056, 14284, 40192, 25256, 7356,  38424, 43976,
                32124, 58204,
            },
            {
                42216, 53796, 5352,  25924, 21100, 37028, 60900, 15448, 54248,
                25844, 50660, 61900, 33372, 57504, 20060, 12000, 52808, 21684,
                1208,  37652, 30092, 43084, 65036, 7736,  48440, 37996, 52684,
                33308, 63956, 29620, 60740, 21028, 44464, 6580,  16804, 62240,
                4068,  28704, 39608, 65376, 21828, 36944, 27792, 52160, 19560,
                45244, 4656,  62552, 20392, 24524, 36672, 49308, 21960, 42808,
                38680, 56408, 35712, 10532, 54936, 62132, 3792,  44408, 50548,
                60904, 1760,  27960, 51600, 15496, 60180, 1032,  30172, 47468,
                41288, 4300,  59212, 10048, 37472, 48448, 3812,  11960, 50516,
                42840, 33468, 63880, 13516, 27692, 49316, 15340, 44444, 10596,
                60,    48276, 25172, 54068, 44724, 5184,  18588, 24648, 53500,
                5572,  45372, 57536, 9300,  38460, 4508,  45124, 28568, 40352,
                25236, 52780, 36820, 63772, 17776, 41780, 14964, 204,   29428,
                10672, 24560, 5104,  30396, 44324, 64192, 34384, 16656, 54716,
                22316, 27864,
            },
            {
                9584,  19024, 61796, 51168, 12728, 47596, 23384, 43264, 63284,
                7796,  39328, 3048,  22832, 27580, 38772, 64896, 42292, 25568,
                45736, 11536, 49864, 23324, 35972, 57808, 17652, 3968,  23076,
                14496, 2320,  47572, 8192,  56116, 49344, 27036, 11432, 23840,
                50800, 57720, 12824, 168,   33280, 46404, 9020,  56836, 1456,
                25624, 35084, 43076, 9836,  16496, 55628, 3360,  64312, 9076,
                29072, 24488, 61204, 49060, 21492, 16332, 37440, 11052, 18660,
                40384, 21436, 37324, 5000,  31820, 45632, 9340,  53876, 64756,
                16376, 23328, 33100, 14744, 29112, 55036, 40124, 31096, 25796,
                18596, 55448, 46632, 23304, 36676, 8744,  40216, 64016, 30008,
                38548, 61644, 17012, 9084,  32336, 65140, 51024, 12952, 31232,
                39468, 332,   49196, 20584, 42508, 14012, 55568, 19828, 8344,
                16000, 60512, 12028, 28768, 23852, 50672, 37740, 55252, 65428,
                19064, 38868, 62220, 56016, 21736, 11896, 28796, 60000, 2088,
                6084,  49800,
            },
            {
                37976, 33620, 116,   39292, 55372, 6864,  27936, 1376,  35572,
                30812, 16680, 48176, 59964, 6516,  46128, 3536,  30356, 60536,
                33900, 15656, 55300, 5028,  26820, 13716, 40972, 44564, 62456,
                26308, 40372, 35856, 17452, 13028, 32480, 40848, 59652, 37268,
                34072, 45440, 19236, 49240, 59816, 15880, 41556, 12036, 48988,
                64636, 13692, 51932, 59996, 39932, 27248, 13280, 31272, 17480,
                52448, 532,   15044, 43992, 40048, 28864, 52164, 57560, 26096,
                6044,  49008, 54784, 12236, 23992, 56520, 18040, 25768, 5428,
                57652, 51368, 2256,  47132, 24144, 7880,  20420, 52692, 61964,
                4684,  9284,  29540, 804,   54036, 58540, 5612,  18004, 23424,
                57148, 4332,  35996, 29060, 40532, 22236, 46740, 35332, 60152,
                55268, 14824, 33524, 26496, 53316, 31576, 61680, 36004, 51476,
                42824, 22000, 3720,  45624, 57640, 4588,  9436,  22064, 34588,
                42440, 53940, 8180,  49044, 764,   18480, 46520, 51292, 41000,
                15040, 64364,
            },
            {
                56960, 23240, 45628, 29012, 16456, 33028, 58752, 52212, 13464,
                21700, 55764, 43804, 11180, 31596, 54452, 17972, 9784,  57328,
                7012,  40816, 63272, 18924, 47524, 32384, 60656, 29032, 9972,
                55408, 6228,  58780, 42916, 24420, 64464, 5328,  53040, 1888,
                15524, 7980,  42252, 24844, 6020,  29624, 21108, 58808, 32056,
                23008, 6936,  28584, 32780, 1720,  50352, 44292, 34968, 58932,
                41216, 20776, 33208, 5404,  59024, 8772,  1140,  46492, 31212,
                62768, 34488, 17420, 42536, 63308, 38696, 49408, 34784, 44220,
                37744, 19908, 61352, 41784, 56652, 65004, 1148,  43816, 35988,
                14972, 39684, 60192, 17188, 42096, 21588, 47296, 34704, 52816,
                12540, 46292, 51508, 13928, 60936, 1084,  10264, 27688, 6932,
                19132, 23944, 64324, 37388, 2292,  17560, 6704,  48840, 1336,
                30320, 56336, 34100, 39948, 14000, 62012, 27628, 47972, 5960,
                16012, 27148, 13148, 33012, 37344, 58096, 9004,  23912, 36052,
                26324, 47048,
            },
            {
                5016,  14248, 59284, 8888,  65120, 42140, 18940, 38192, 48712,
                4704,  64500, 26220, 36440, 15104, 49900, 24368, 37504, 47884,
                20912, 27796, 3092,  36900, 53436, 8836,  816,   51360, 19264,
                46256, 30692, 21320, 52152, 512,   28904, 18776, 47100, 22344,
                63560, 31244, 53636, 37720, 61820, 50484, 36300, 3296,  44104,
                38832, 54764, 18052, 47064, 6356,  62812, 19312, 54676, 7156,
                47812, 63528, 12608, 53728, 25228, 35408, 64912, 22008, 41876,
                14280, 8552,  58608, 29124, 2764,  7336,  20984, 13444, 28656,
                10836, 31264, 6952,  12552, 34308, 16960, 28036, 11124, 57904,
                48752, 24212, 34260, 51092, 11672, 31316, 60880, 2848,  27592,
                42272, 20952, 6492,  26004, 55836, 17244, 43596, 62520, 41292,
                47492, 11080, 51308, 7884,  59796, 46516, 22916, 26264, 63676,
                11320, 47444, 7564,  53352, 20344, 32440, 43604, 52880, 59240,
                2260,  45600, 60116, 22900, 41748, 53004, 31384, 4204,  61116,
                18780, 11356,
            },
            {
                27680, 31844, 40788, 20892, 49436, 2732,  11116, 24216, 60620,
                9228,  40056, 18676, 1916,  58192, 41260, 63064, 652,   14320,
                51920, 31896, 60068, 45360, 12544, 21760, 56412, 34536, 15100,
                37616, 49896, 11396, 62024, 34800, 45068, 57300, 10368, 39996,
                26480, 58464, 11528, 2780,  18024, 55604, 8396,  26548, 15228,
                60724, 10284, 42336, 57492, 22444, 37596, 11404, 24148, 3200,
                26536, 37288, 45636, 30724, 18348, 50316, 11904, 16840, 54552,
                3352,  47952, 22352, 45308, 52556, 32536, 58120, 61760, 416,
                54820, 50124, 45700, 25888, 39000, 51692, 46720, 22296, 31920,
                3468,  63720, 44536, 7292,  55972, 25664, 14640, 38740, 9588,
                64788, 32576, 59648, 38104, 48460, 33880, 53008, 21176, 2444,
                36512, 58640, 30140, 43012, 13096, 56400, 33692, 41384, 15140,
                37532, 59128, 17412, 25724, 1080,  37184, 11540, 18192, 24220,
                38208, 50748, 29908, 17148, 6444,  62676, 14488, 45020, 55852,
                32968, 53228,
            },
            {
                50044, 62544, 3708,  54120, 34920, 26616, 56168, 43456, 32556,
                28420, 46556, 52644, 33672, 7928,  21276, 27952, 34816, 42816,
                55660, 10560, 16952, 25336, 42332, 63548, 39388, 24252, 4768,
                65244, 2956,  25592, 16352, 8452,  38560, 14160, 33380, 50936,
                5620,  19528, 48128, 28388, 14052, 33784, 63448, 47436, 52376,
                20204, 456,   34768, 26344, 14980, 59260, 30464, 40460, 51320,
                16572, 9884,  57352, 4052,  61440, 38916, 44988, 28324, 33068,
                38064, 61304, 25620, 36148, 10516, 16396, 41152, 46412, 23792,
                39968, 15704, 63916, 20888, 2856,  59736, 8436,  37372, 53644,
                19160, 12832, 27872, 20708, 2124,  63160, 45048, 50048, 55324,
                16616, 1444,  44240, 8780,  23544, 4516,  29692, 14316, 56860,
                25428, 4952,  16220, 21868, 39052, 28756, 9368,  54212, 5288,
                21168, 28296, 44008, 64736, 49552, 55808, 30304, 63020, 8036,
                33448, 10796, 64252, 3300,  47424, 26508, 20700, 10200, 38292,
                2312,  43508,
            },
            {
                15548, 37320, 24848, 12144, 46980, 6076,  62852, 15916, 504,
                54392, 13964, 23376, 61540, 44836, 51404, 12352, 59196, 5088,
                23164, 38852, 50220, 1640,  30660, 5936,  47856, 28332, 59116,
                43948, 32636, 40076, 54704, 48300, 60748, 23644, 2492,  55992,
                44088, 34824, 62716, 39220, 45768, 42172, 23692, 4856,  30788,
                37284, 64244, 49656, 4252,  53088, 45872, 8528,  65464, 33596,
                55688, 28648, 21368, 41448, 8064,  23856, 1884,  60324, 5952,
                51372, 18768, 13132, 1312,  65332, 26896, 50480, 4064,  35632,
                8812,  27820, 57980, 5708,  32492, 42532, 14660, 62764, 6048,
                41320, 56500, 47612, 37276, 40832, 33500, 5140,  19556, 24268,
                36776, 30232, 52568, 19000, 57660, 12040, 64584, 39652, 50276,
                31996, 45316, 62136, 49492, 2860,  65392, 18008, 45436, 60164,
                50520, 32956, 4196,  9964,  40600, 15428, 5240,  46140, 41404,
                58036, 20412, 43368, 54644, 35168, 39908, 51596, 29616, 64768,
                22796, 7780,
            },
            {
                20080, 56312, 44676, 18000, 30784, 39008, 50536, 21236, 36880,
                58936, 6544,  29820, 11040, 37984, 3376,  30992, 17584, 45172,
                65048, 7604,  56816, 35380, 61420, 20124, 16252, 50904, 7832,
                13052, 18680, 57864, 27884, 5832,  20332, 31168, 64888, 28084,
                15872, 8992,  21988, 54940, 776,   10348, 18724, 58572, 12620,
                45264, 23232, 29004, 11760, 41544, 19812, 2172,  48452, 13568,
                43704, 1168,  63208, 51724, 13924, 34760, 48756, 55708, 10892,
                27128, 43148, 57104, 39420, 53996, 19520, 30684, 14188, 62668,
                55820, 18700, 43932, 36704, 53096, 23220, 29680, 49188, 25300,
                284,   30736, 15652, 60660, 10676, 52700, 29064, 58976, 7112,
                48768, 62340, 14772, 28284, 41144, 46420, 35208, 7252,  18248,
                10076, 54636, 34452, 26872, 6520,  51828, 35196, 24416, 260,
                39796, 13172, 57384, 23380, 34936, 60960, 21324, 26120, 52464,
                444,   28328, 14072, 23876, 8492,  60500, 1004,  16884, 48740,
                41084, 59092,
            },
            {
                28908, 664,   9388,  63960, 58444, 13700, 28044, 8660,  44996,
                18160, 49624, 42036, 55876, 19448, 63472, 48688, 53672, 25896,
                33072, 19324, 28176, 13476, 45920, 54016, 10536, 35900, 41012,
                52908, 22640, 46152, 1928,  35952, 42784, 52312, 12092, 41608,
                49068, 4320,  25800, 59584, 31756, 51604, 27552, 54600, 41004,
                7756,  16940, 55704, 62360, 32120, 36752, 27412, 22932, 60908,
                18756, 36376, 46856, 25948, 31708, 59352, 19736, 40464, 22708,
                64116, 6812,  31396, 48628, 8740,  44396, 59180, 6496,  22172,
                47548, 31444, 2080,  13544, 62252, 9800,  18244, 58944, 44300,
                35340, 65248, 8364,  49700, 23464, 17472, 42556, 12940, 34604,
                40264, 11232, 3312,  60668, 51036, 1240,  24872, 59836, 43504,
                22856, 616,   40944, 19892, 14808, 58216, 43448, 12024, 31156,
                62028, 19196, 46732, 54136, 29048, 2356,  48656, 12388, 36172,
                17428, 56360, 37932, 49768, 31908, 12896, 56900, 34076, 25216,
                11628, 35124,
            },
            {
                48004, 51592, 32800, 41788, 4600,  23088, 53020, 61104, 33788,
                24984, 65300, 2208,  33236, 26964, 6848,  14932, 39752, 10108,
                3024,  49144, 41360, 58676, 3688,  32560, 22184, 64076, 104,
                30236, 61904, 9660,  50592, 15376, 58672, 7764,  18280, 37436,
                60352, 53288, 36152, 17340, 7060,  65232, 37704, 32932, 2368,
                60336, 34364, 47212, 5560,  15528, 59840, 56648, 4912,  39360,
                30672, 6184,  11304, 54884, 16892, 44028, 4836,  30040, 15252,
                46004, 34996, 16868, 23332, 4288,  37444, 25144, 34056, 52532,
                38836, 10800, 51128, 25524, 46060, 56452, 39416, 6776,  12380,
                21600, 54132, 26652, 4060,  32064, 55848, 1404,  63656, 45608,
                21836, 56340, 25964, 37180, 20740, 31792, 15884, 52196, 28736,
                63948, 13168, 47872, 61108, 38184, 29340, 21692, 48388, 8048,
                51624, 25536, 37492, 6612,  17020, 43744, 58728, 31732, 65168,
                5888,  46824, 61172, 4280,  19336, 45380, 41684, 6904,  53052,
                3424,  62104,
            },
            {
                13848, 25688, 20712, 54364, 35788, 46196, 1616,  40852, 11720,
                5460,  38872, 15792, 22764, 59932, 43168, 35296, 22144, 61176,
                55020, 36488, 16048, 24076, 8508,  39188, 27196, 56364, 45180,
                16980, 25092, 38900, 32896, 63788, 26176, 46812, 1040,  24240,
                32136, 10924, 42984, 47200, 13364, 21480, 48816, 15368, 24696,
                51060, 21024, 39308, 9996,  25136, 50636, 42320, 12316, 53880,
                49804, 23744, 58692, 39964, 3104,  9656,  63060, 53640, 37468,
                564,   50756, 60284, 28608, 62916, 55048, 12088, 43360, 3292,
                17108, 64960, 41364, 20096, 34344, 1272,  27084, 52688, 31648,
                46748, 17032, 40408, 36112, 59496, 46564, 20432, 27120, 53772,
                4488,  32928, 47800, 42944, 5504,  10484, 57876, 38624, 3936,
                35644, 55612, 31208, 6884,  10388, 53428, 2096,  63556, 35968,
                15188, 3580,  42432, 63908, 10904, 51876, 39080, 8320,  23724,
                40956, 30444, 9948,  26580, 55296, 64152, 22080, 28024, 58060,
                18336, 43696,
            },
            {
                5732,  39896, 61160, 15948, 8300,  30148, 19376, 57524, 48968,
                31176, 55180, 45812, 52584, 9580,  47880, 57092, 468,   29172,
                46344, 11924, 63372, 52148, 43960, 60064, 48748, 13652, 6824,
                34108, 59692, 3540,  12764, 21284, 29996, 55188, 40496, 62560,
                20028, 3012,  56952, 29664, 40180, 1636,  44572, 9448,  62684,
                43152, 3908,  28352, 53780, 44460, 1244,  18164, 34424, 8244,
                15380, 64512, 33408, 20896, 50876, 35904, 26608, 47380, 12504,
                58308, 20988, 10256, 40812, 14692, 47348, 18948, 60476, 28144,
                57236, 7272,  29816, 59684, 6288,  49952, 16380, 36304, 60776,
                2636,  57396, 11328, 44096, 7392,  14172, 37708, 9700,  30536,
                16308, 8636,  65440, 17908, 54128, 62404, 47128, 21432, 8392,
                49604, 17384, 24960, 42788, 57060, 33624, 17208, 40804, 27540,
                54476, 58680, 30484, 22216, 33812, 26468, 19320, 55212, 14844,
                51004, 20996, 43796, 16340, 35704, 2100,  38180, 14732, 46424,
                30640, 37068,
            },
            {
                10616, 55952, 2400,  27512, 50144, 64528, 37624, 14524, 26676,
                62604, 20900, 13040, 36564, 4132,  30556, 18656, 13440, 42196,
                5336,  26664, 20588, 1248,  30188, 18148, 4384,  36740, 20940,
                53696, 48036, 41196, 56592, 44496, 4340,  10444, 34416, 49592,
                14224, 27284, 64020, 23152, 51236, 59308, 26976, 55492, 30532,
                12280, 58312, 18804, 64904, 36220, 22348, 30124, 62820, 45668,
                27712, 41864, 64,    46360, 28892, 60624, 17980, 23212, 7472,
                42408, 32964, 3680,  52044, 35728, 1656,  32352, 9488,  49488,
                36436, 23900, 14872, 54588, 44216, 22364, 64392, 42564, 9252,
                24688, 19512, 50976, 29188, 64216, 25324, 51800, 61564, 42136,
                49348, 58404, 23712, 13252, 29552, 34660, 41248, 14380, 27320,
                44528, 62200, 2624,  20636, 47080, 26116, 5588,  60508, 45524,
                18876, 9536,  49256, 13788, 60096, 724,   47252, 4968,  35068,
                59828, 2760,  63012, 52680, 31416, 11348, 51208, 60308, 8432,
                21104, 65368,
            },
            {
                24060, 32520, 47272, 43392, 11392, 24276, 54768, 3588,  43160,
                8948,  1176,  28208, 40988, 64200, 25248, 51500, 62724, 32548,
                59844, 49708, 34236, 38052, 10716, 54400, 26020, 62076, 43120,
                11600, 28112, 7816,  18048, 52184, 37856, 16540, 58240, 5316,
                52948, 45236, 8632,  35052, 5624,  19156, 34040, 4688,  17416,
                36664, 48368, 32552, 7432,  13592, 60384, 48664, 3496,  55936,
                37972, 18956, 10412, 56288, 13756, 5388,  39520, 56784, 30888,
                63852, 25972, 55676, 45424, 22084, 27328, 64148, 41884, 21124,
                68,    46588, 38328, 11816, 32276, 3448,  28632, 13744, 48524,
                38456, 62480, 33920, 1036,  22004, 48084, 3960,  33584, 19340,
                384,   36068, 39800, 45452, 2148,  25328, 6300,  56516, 33020,
                52664, 11596, 36368, 59772, 14132, 38948, 50016, 11668, 22700,
                33256, 1824,  39348, 53588, 45016, 36876, 63360, 29572, 42948,
                25452, 13232, 38700, 6684,  47816, 23672, 42444, 4540,  33392,
                48984, 53328,
            },
            {
                14600, 59540, 17864, 38608, 58696, 6376,  33608, 16740, 47656,
                35096, 59592, 49420, 56664, 16324, 7232,  45120, 37364, 23236,
                17008, 6980,  57376, 45268, 65424, 41336, 15404, 50920, 30656,
                1680,  64384, 23192, 35268, 59468, 25020, 47432, 28384, 21948,
                41920, 37576, 16888, 55692, 11592, 39200, 63564, 52876, 42076,
                23524, 568,   52012, 26372, 56992, 39748, 10608, 25804, 21292,
                5236,  52396, 61552, 24772, 31840, 44880, 52504, 1836,  49236,
                19072, 13576, 37800, 17232, 58904, 6332,  52672, 13500, 56380,
                33328, 63252, 51860, 19200, 61536, 40052, 57544, 7164,  55052,
                30240, 5040,  14604, 42340, 54860, 17496, 39656, 59808, 13004,
                55420, 27676, 7344,  52812, 60824, 50628, 19536, 64628, 884,
                22952, 40560, 29288, 51716, 8256,  64396, 31100, 3964,  55236,
                43356, 65060, 28088, 5756,  20728, 9012,  15868, 57556, 10460,
                48956, 32940, 58168, 28248, 19116, 61960, 54888, 17396, 26448,
                40368, 1128,
            },
            {
                29192, 36212, 7588,  21776, 30900, 51672, 41228, 60924, 23700,
                53272, 18092, 32268, 22944, 11360, 34284, 55644, 1816,  10136,
                40040, 53660, 13216, 24600, 2876,  21652, 33844, 8724,  58480,
                38636, 14664, 49528, 31584, 928,   13400, 65108, 8312,  33252,
                62100, 228,   30936, 60728, 49020, 25244, 14244, 46212, 8208,
                60820, 38356, 16072, 45804, 2824,  31092, 17100, 43860, 59144,
                33156, 14520, 47316, 36224, 8144,  65296, 16584, 11196, 34352,
                44160, 5120,  61008, 8584,  30768, 48124, 24512, 38708, 16828,
                7676,  26364, 4440,  9428,  48692, 24568, 16808, 34580, 21044,
                44264, 52440, 26572, 58180, 35716, 11864, 6712,  28988, 23196,
                43376, 63576, 21112, 32060, 11200, 16500, 43132, 38048, 10020,
                58152, 16736, 5912,  45168, 23564, 18144, 35552, 58284, 14624,
                25240, 48312, 16944, 56972, 31588, 24348, 40684, 52144, 18604,
                1484,  22116, 54328, 9196,  44788, 80,    29912, 36884, 12156,
                63072, 44976,
            },
            {
                51012, 4084,  61476, 48808, 13708, 4476,  20396, 28940, 10584,
                38732, 6120,  44000, 2632,  61312, 47496, 19628, 27132, 58916,
                43460, 28824, 18788, 31808, 47996, 60940, 5500,  46228, 24308,
                55440, 19928, 45344, 61180, 6656,  43020, 39600, 54492, 18188,
                12048, 25928, 46476, 21404, 42552, 2384,  32156, 28308, 19988,
                56028, 11724, 63256, 41760, 20612, 50892, 54344, 36640, 8500,
                49580, 29440, 1964,  42668, 21972, 38120, 27544, 59636, 23932,
                54236, 28472, 49904, 40480, 12068, 43680, 2160,  62116, 29672,
                44796, 55124, 41500, 30408, 36108, 53620, 46124, 600,   59780,
                10452, 18064, 47036, 8340,  31004, 62628, 45572, 53820, 50364,
                9128,  16016, 3820,  57828, 35148, 48544, 28448, 54144, 31532,
                46936, 63348, 33952, 55516, 1752,  42120, 27444, 52340, 40324,
                7700,  37304, 11036, 34480, 50844, 62176, 3428,  27968, 37844,
                64928, 46136, 16124, 40440, 34392, 13892, 57820, 50328, 6016,
                56528, 19500,
            },
            {
                23416, 39604, 55332, 26760, 44740, 63132, 56512, 200,   45640,
                64920, 13876, 26288, 54124, 40484, 29584, 38288, 14216, 49040,
                5692,  62936, 51444, 9072,  37332, 54868, 27556, 17844, 36012,
                2700,  41376, 9540,  27208, 51224, 21084, 30176, 3432,  48520,
                59052, 52244, 6160,  15032, 54708, 9664,  58524, 50992, 3304,
                34544, 24968, 29328, 9344,  33916, 5632,  64096, 13224, 41044,
                23048, 63280, 54684, 18204, 58112, 3164,  48624, 41688, 8956,
                36356, 964,   21688, 64868, 33976, 19036, 57908, 35240, 11088,
                50192, 20132, 14128, 58764, 3052,  12732, 27888, 63364, 40868,
                24012, 36792, 65076, 3000,  40300, 19968, 25512, 1840,  37500,
                31112, 47956, 39072, 44604, 22624, 8084,  2840,  24400, 14080,
                4212,  19256, 25816, 12356, 59424, 48644, 9328,  19884, 732,
                62364, 21644, 59200, 2308,  42868, 12928, 47964, 56072, 34032,
                6168,  26680, 60828, 3860,  24972, 64032, 21444, 41564, 25540,
                34148, 9044,
            },
            {
                43548, 12312, 32732, 17036, 9432,  34408, 37420, 15664, 50444,
                31412, 58200, 35812, 20844, 8980,  50724, 4868,  64824, 32376,
                21420, 35672, 272,   59664, 15028, 40400, 10968, 50100, 63300,
                12912, 32828, 53984, 16776, 36956, 56552, 14948, 44576, 24224,
                35764, 39296, 29168, 63392, 34300, 38016, 22860, 65184, 40244,
                44692, 5036,  53952, 47560, 58584, 24048, 27920, 692,   60200,
                16700, 6588,  26564, 11972, 51160, 33272, 14176, 20424, 62696,
                46064, 56924, 15316, 25684, 51656, 4784,  54056, 22704, 3624,
                60356, 37172, 24000, 65432, 44476, 21936, 7812,  50952, 32132,
                4244,  55584, 13472, 22516, 49756, 59444, 15328, 34168, 56712,
                18748, 60144, 26544, 13864, 64140, 56024, 41520, 61496, 50204,
                36936, 43252, 51232, 39284, 30760, 15500, 63172, 44340, 32844,
                28664, 45692, 53840, 25976, 38896, 19692, 7512,  23020, 14480,
                43684, 11308, 49884, 36576, 53312, 46852, 10448, 2644,  47752,
                16448, 62160,
            },
            {
                1556,  52064, 65012, 3336,  41872, 22432, 53600, 27616, 7872,
                18916, 3760,  46864, 63204, 12932, 57692, 17712, 24928, 53868,
                11632, 46656, 26212, 42228, 20248, 30344, 3344,  56668, 42936,
                29008, 59056, 4636,  24624, 63536, 10652, 34496, 61532, 7540,
                20384, 10360, 44044, 1596,  18852, 48172, 6564,  16624, 12776,
                49816, 18528, 60488, 14040, 36384, 17664, 44776, 48780, 31720,
                35456, 45688, 39128, 30456, 61404, 7268,  55792, 29632, 4500,
                17960, 31452, 38752, 6880,  42428, 28900, 13840, 46216, 40164,
                28120, 6264,  48976, 16988, 33728, 55500, 38260, 19180, 14952,
                49108, 43572, 33768, 29284, 53200, 9780,  42748, 4780,  63004,
                11280, 6280,  51108, 2276,  29472, 36324, 11772, 20736, 57324,
                27716, 8456,  61128, 21516, 4956,  36464, 24192, 56428, 12560,
                51300, 17568, 4732,  15064, 61220, 29532, 45112, 62992, 53460,
                31128, 58424, 19640, 29828, 7640,  17316, 32288, 38432, 59276,
                54312, 30424,
            },
            {
                20680, 28060, 46496, 24940, 58660, 48320, 12200, 61136, 42724,
                39344, 24140, 52492, 28836, 700,   33360, 41772, 45136, 2544,
                39400, 16460, 57580, 6728,  48936, 64340, 34576, 22804, 7200,
                20752, 48244, 39256, 45772, 6204,  49192, 27508, 1276,  55400,
                49920, 16276, 54076, 59472, 27452, 41868, 30856, 56624, 26540,
                37280, 31900, 22760, 1908,  40304, 8080,  56432, 20148, 10732,
                57564, 52232, 4164,  20796, 43300, 46988, 23716, 39464, 50616,
                12512, 60448, 53024, 10884, 61908, 48724, 32532, 64168, 17956,
                57136, 9728,  31424, 1224,  40728, 10692, 58360, 30228, 61028,
                25764, 7516,  57432, 17516, 224,   38408, 48344, 22744, 28444,
                46544, 41840, 20544, 33128, 52840, 17772, 46232, 5484,  33496,
                16076, 172,   54836, 11060, 46576, 53612, 2936,  38760, 6692,
                60632, 35584, 31116, 48872, 57156, 10320, 36432, 3948,  18132,
                38564, 1144,  42148, 23200, 55832, 43532, 61276, 27828, 6708,
                14024, 36968,
            },
            {
                58020, 8604,  38532, 18720, 6800,  31608, 1968,  20972, 55384,
                33516, 11000, 16348, 43312, 37488, 22852, 61380, 8600,  56504,
                29900, 62184, 33132, 23848, 53276, 12688, 45284, 26612, 52512,
                15160, 62304, 484,   31376, 22156, 58116, 17992, 41924, 30492,
                63984, 33152, 5024,  23348, 13408, 62392, 10480, 52616, 360,
                61968, 8880,  42844, 51064, 64728, 30716, 53224, 37760, 3216,
                24504, 14628, 64488, 9392,  35100, 16812, 488,   63776, 33712,
                26140, 44204, 2680,  35164, 17040, 21564, 8380,  460,   25516,
                42980, 51736, 62088, 46480, 25420, 52068, 5532,  42812, 1472,
                37056, 11444, 61764, 46164, 26236, 63768, 31556, 54280, 14432,
                35328, 24576, 55560, 62004, 7608,  39944, 25668, 49180, 65112,
                40916, 45464, 29796, 34712, 64268, 16912, 28188, 49176, 22096,
                43032, 9952,  40708, 23592, 248,   33268, 50248, 24876, 59892,
                9268,  52056, 14960, 64552, 5424,  12524, 50796, 18968, 24284,
                42716, 49504,
            },
            {
                33060, 15752, 54152, 60644, 35780, 50896, 26000, 45456, 5512,
                62352, 48380, 59404, 7076,  55672, 15072, 49416, 27060, 19552,
                50916, 5144,  44016, 9848,  37168, 1548,  17380, 61056, 38304,
                10044, 35580, 19088, 43352, 53120, 13456, 37780, 47144, 11728,
                25340, 40448, 46376, 36800, 50400, 3168,  45092, 34868, 21096,
                46820, 57708, 28264, 16368, 4556,  26124, 13020, 42992, 62588,
                28828, 41392, 47748, 55248, 25392, 59400, 11504, 42240, 54012,
                8424,  19108, 47608, 24280, 55328, 58912, 39404, 36200, 54608,
                14936, 34188, 19940, 13100, 28972, 63688, 17892, 23472, 47068,
                54200, 20928, 40700, 5656,  35892, 12140, 18972, 7496,  58760,
                39324, 868,   16272, 44224, 12648, 59460, 1520,  30940, 10080,
                22684, 58304, 18628, 25192, 41612, 7940,  32568, 57632, 14228,
                26484, 65400, 5680,  54948, 20388, 63920, 12428, 41268, 46264,
                28152, 34500, 47780, 25756, 39336, 35516, 872,   46108, 62980,
                10928, 4924,
            },
            {
                64072, 44128, 636,   11584, 41184, 15176, 64520, 37160, 17576,
                29804, 2460,  26672, 20208, 46120, 31220, 3984,  35300, 12592,
                37920, 22332, 15116, 55508, 27956, 59348, 41652, 5964,  30988,
                57292, 49820, 28240, 7668,  65228, 32904, 3900,  59728, 21680,
                2448,  57816, 8912,  19532, 31640, 58196, 25148, 17828, 41180,
                5556,  12308, 54224, 33568, 46084, 61244, 21992, 6316,  50588,
                34296, 18464, 2668,  31188, 37872, 49888, 28876, 5252,  21952,
                37064, 57608, 30132, 40968, 5596,  27984, 44548, 12128, 49596,
                23028, 5728,  38904, 55864, 8024,  36316, 49244, 13872, 33276,
                64784, 28244, 15416, 51908, 22120, 60468, 43424, 3788,  51392,
                65356, 10172, 48856, 27140, 36748, 21984, 50772, 55968, 14192,
                38576, 52408, 6096,  13416, 50984, 59560, 19728, 45204, 2156,
                53184, 18428, 47324, 37236, 43568, 27840, 16784, 56308, 6008,
                21392, 62576, 3180,  31516, 58892, 21316, 56604, 30588, 34456,
                53192, 22356,
            },
            {
                26668, 48152, 31028, 23108, 28512, 53684, 8248,  57348, 13212,
                41436, 53320, 35920, 65528, 39952, 10620, 52180, 63048, 41568,
                58924, 908,   65084, 48044, 18740, 32284, 51184, 46644, 2500,
                25028, 12320, 54956, 16104, 40156, 26064, 9240,  53788, 35680,
                15224, 29344, 52008, 64764, 15852, 38768, 6828,  55484, 29860,
                63824, 24016, 38944, 19604, 10116, 35844, 48652, 17136, 58272,
                10336, 22892, 60388, 7620,  13824, 20172, 56320, 46116, 14904,
                61776, 1792,  13428, 65260, 50696, 15648, 3556,  63368, 30288,
                61072, 2452,  59100, 44060, 22088, 1900,  59760, 39928, 4360,
                9000,  43988, 32200, 2596,  56176, 28732, 49340, 33808, 25492,
                21240, 30516, 57244, 5048,  32292, 63460, 18856, 43232, 27496,
                2740,  35956, 62564, 47092, 1668,  37272, 10940, 62860, 39068,
                33932, 30208, 58864, 14708, 8204,  52176, 2992,  31676, 37460,
                14116, 54964, 11076, 44120, 16452, 52660, 8712,  14256, 41124,
                2324,  38848,
            },
            {
                12440, 19420, 56588, 61660, 4712,  44596, 20528, 32724, 24032,
                50088, 9764,  22328, 14184, 1564,  58300, 18520, 23908, 7636,
                46968, 29332, 34740, 40356, 7372,  24608, 13320, 21712, 63520,
                40576, 44960, 60252, 2004,  22880, 51164, 44640, 18912, 63088,
                48660, 43584, 5600,  26196, 42368, 11428, 61292, 48600, 14688,
                36484, 1592,  49860, 59948, 3056,  56916, 40620, 31848, 216,
                45584, 54048, 39736, 51668, 43788, 1704,  64612, 31788, 39276,
                27000, 49136, 34092, 9504,  20800, 32900, 46528, 19332, 9236,
                40672, 27108, 47932, 31540, 15900, 52848, 29764, 24936, 56940,
                50836, 18560, 59640, 47384, 37784, 9576,  16848, 41100, 13352,
                45480, 37832, 18320, 53124, 41156, 8760,  3608,  34280, 47756,
                60868, 9304,  26340, 31484, 22488, 55256, 29292, 24296, 7552,
                50460, 11820, 3996,  25404, 34448, 59612, 48008, 61948, 24088,
                51120, 40740, 20036, 60756, 4276,  28380, 48208, 25372, 57860,
                17292, 60196,
            },
            {
                7388,  36620, 42524, 9704,  34348, 49500, 1260,  39048, 59580,
                3776,  43824, 57076, 30572, 48012, 27488, 43620, 33320, 53708,
                17064, 25872, 11736, 51896, 2788,  62396, 36132, 54744, 16404,
                6752,  19864, 29988, 35152, 47912, 61568, 28844, 6404,  38816,
                23264, 12708, 55616, 34928, 860,   53412, 22388, 33108, 9316,
                52296, 42132, 15776, 27856, 43940, 24752, 13772, 28228, 65180,
                36520, 26008, 16052, 62784, 27532, 34852, 8968,  17112, 52348,
                6352,  44668, 23584, 54752, 37608, 60312, 25692, 57368, 34976,
                53544, 17784, 11464, 37412, 63112, 9832,  45296, 20144, 11996,
                36092, 26160, 7052,  13732, 24112, 63980, 1068,  53688, 57996,
                8148,  3208,  60920, 47268, 15048, 25200, 54232, 59096, 16924,
                20964, 42328, 56628, 14752, 44332, 4620,  49012, 16424, 43640,
                57100, 21416, 63232, 46012, 39488, 22104, 10780, 18496, 44492,
                1064,  29844, 7792,  49724, 37204, 42352, 64984, 5568,  45432,
                29644, 50280,
            },
            {
                32348, 2944,  52260, 16152, 26040, 62924, 12180, 47356, 27892,
                63912, 18708, 33884, 8008,  61772, 38032, 5824,  13436, 57512,
                3620,  44924, 61456, 20884, 57172, 43420, 8988,  28280, 32588,
                49112, 37612, 10128, 57496, 4808,  14912, 11560, 33616, 57280,
                1688,  31496, 61500, 20832, 47328, 28964, 44108, 3356,  26684,
                57516, 21308, 31600, 63224, 5796,  54988, 8696,  51124, 20324,
                6632,  12376, 32632, 4420,  21768, 48680, 41276, 24868, 60948,
                11616, 58356, 18028, 42396, 4752,  51688, 13156, 1400,  42952,
                49296, 6984,  24684, 55288, 3132,  41960, 34412, 61300, 47620,
                548,   62500, 42316, 55080, 34340, 45064, 20140, 31976, 27400,
                62824, 34940, 23572, 11776, 29184, 44600, 37256, 11056, 30548,
                4984,  50384, 33708, 18432, 64980, 40068, 34668, 60696, 500,
                27560, 41344, 17304, 54628, 1440,  28832, 42000, 7028,  58364,
                35700, 64188, 25284, 33220, 11872, 23204, 18200, 35836, 10408,
                21220, 55396,
            },
            {
                14384, 46336, 22588, 57604, 40452, 18412, 53980, 31296, 15840,
                5660,  37208, 52428, 24860, 16532, 50708, 21744, 63832, 30904,
                49404, 9440,  38160, 30476, 15804, 48764, 39356, 58536, 3940,
                53196, 64368, 25396, 43668, 18580, 39584, 54612, 42732, 26740,
                17224, 50888, 40668, 7972,  16648, 58828, 37660, 18284, 65328,
                45604, 7040,  11012, 47840, 35080, 18904, 38228, 60084, 42596,
                56560, 49216, 45044, 58004, 11072, 60044, 55192, 3188,  20608,
                36448, 29096, 112,   61988, 31044, 8252,  39772, 22448, 28884,
                15120, 64916, 33420, 19544, 50472, 27424, 4640,  16760, 55496,
                31364, 39476, 21300, 29544, 4404,  59300, 39108, 5800,  47976,
                15820, 42968, 51768, 292,   56696, 64884, 6308,  51480, 24072,
                63380, 38884, 832,   11236, 51388, 6832,  20744, 12948, 52596,
                37120, 9232,  31672, 49940, 13088, 64872, 33148, 53128, 26940,
                12460, 16696, 45800, 52420, 58256, 440,   54576, 61188, 39936,
                4024,  62124,
            },
            {
                27200, 38320, 65288, 29480, 5472,  36344, 7800,  23156, 41048,
                11084, 55312, 45364, 2704,  41804, 55976, 4,     35708, 40584,
                18088, 23188, 55748, 34048, 5864,  25784, 19468, 11852, 45788,
                22596, 14304, 0,     52060, 30764, 22404, 63352, 3920,  47740,
                59596, 10544, 36868, 25364, 62736, 12364, 4804,  55084, 34372,
                14016, 38792, 59184, 53080, 22728, 1380,  46620, 15600, 29948,
                2420,  23100, 38420, 18600, 30604, 37100, 14516, 46972, 32600,
                50572, 40992, 14348, 46260, 26648, 16488, 47644, 54640, 61260,
                4944,  38684, 57868, 46204, 13260, 59516, 22948, 38232, 7960,
                14504, 52404, 10240, 48948, 17656, 12468, 50868, 22620, 10420,
                55168, 30116, 19208, 39632, 33656, 21396, 16316, 40800, 46916,
                13656, 27964, 45576, 59480, 24808, 28496, 56052, 32044, 46752,
                23580, 61436, 4860,  36276, 20340, 56616, 16028, 3676,  49356,
                39556, 56472, 3088,  20992, 40444, 15300, 30652, 48632, 33560,
                24452, 43072,
            },
            {
                49132, 9404,  236,   13208, 50376, 59860, 44712, 56612, 61168,
                49228, 26780, 21048, 13192, 30096, 10468, 46276, 26528, 8052,
                59488, 42852, 2408,  12908, 53812, 64832, 1284,  35320, 61400,
                27568, 41316, 34164, 60568, 7168,  49364, 36028, 9064,  20164,
                30100, 44852, 3280,  52580, 32508, 49000, 41236, 24244, 51204,
                424,   30052, 25836, 16880, 41256, 32540, 64004, 25652, 10308,
                34400, 62620, 54412, 8352,  784,   43556, 27904, 5400,  63644,
                9556,  56488, 22024, 53172, 64572, 36708, 58460, 32148, 10344,
                45036, 21344, 348,   30520, 8776,  40288, 53716, 29244, 64616,
                24884, 45544, 2508,  63148, 26552, 54472, 35360, 64836, 40512,
                25436, 4708,  59148, 8280,  49928, 26992, 2528,  58392, 32272,
                55636, 8560,  22100, 53976, 35252, 43008, 9788,  62748, 3692,
                15256, 58024, 45484, 26380, 7436,  38484, 47060, 23552, 61348,
                19688, 8316,  31664, 63384, 6320,  44616, 26988, 8068,  12780,
                53216, 16984,
            },
            {
                34932, 55736, 20764, 42312, 32924, 25388, 2136,  20120, 35304,
                3472,  32704, 63276, 57812, 37600, 60572, 18884, 53344, 14760,
                51112, 28148, 62844, 47576, 40172, 29696, 44132, 50720, 16796,
                7660,  55916, 11148, 45280, 15984, 58268, 25904, 13740, 53264,
                64540, 23032, 14832, 56240, 6432,  20484, 29800, 9756,  46944,
                19304, 43560, 62496, 3512,  49512, 12488, 57896, 4536,  52412,
                41732, 16788, 26876, 50732, 64856, 23684, 53884, 19184, 38840,
                25744, 2748,  34864, 6836,  11096, 20328, 2412,  25272, 18016,
                36080, 50092, 26280, 42512, 63488, 17896, 5956,  48880, 43260,
                19652, 58252, 32960, 37684, 42924, 7868,  30712, 14880, 1696,
                44824, 61912, 36912, 13112, 46500, 60964, 10276, 36204, 18532,
                3404,  61584, 37464, 15024, 2284,  48580, 17052, 37852, 19748,
                41860, 29808, 11556, 52092, 62492, 42548, 30704, 10216, 34700,
                43544, 28708, 47960, 37032, 24900, 60548, 51816, 19912, 58888,
                1708,  63424,
            },
            {
                6400,  30376, 60560, 53616, 15472, 47232, 10520, 28992, 14372,
                43500, 8684,  17548, 47004, 6644,  34156, 24084, 65032, 32608,
                6088,  36768, 20524, 16968, 24068, 9856,  59940, 21612, 32184,
                38996, 48468, 19584, 28720, 38116, 2896,  32604, 46432, 38664,
                728,   34000, 42008, 27504, 45756, 64056, 15276, 57844, 61000,
                5948,  55944, 35948, 9384,  54728, 29024, 44360, 36364, 21676,
                47508, 5380,  13108, 40676, 33456, 48404, 11460, 58568, 45500,
                16816, 61216, 43320, 29980, 40024, 50440, 44200, 6196,  52896,
                62380, 11888, 56568, 15016, 51708, 36396, 32332, 1180,  11876,
                35740, 6504,  15968, 60184, 23040, 4020,  47220, 56536, 20548,
                50584, 32372, 17172, 23460, 41632, 29904, 54028, 44308, 23624,
                52120, 42380, 31204, 20196, 64128, 5992,  57584, 26028, 50608,
                54840, 1096,  34376, 17696, 22568, 5128,  13988, 57644, 668,
                52932, 15084, 55480, 9776,  13392, 34192, 2640,  37584, 46468,
                40900, 23060,
            },
            {
                44936, 17728, 37980, 3884,  22360, 62580, 39740, 51076, 64576,
                53440, 23688, 40592, 27768, 50256, 3236,  44136, 11744, 39768,
                1804,  45248, 58656, 7416,  52336, 36236, 4980,  13448, 57600,
                2120,  61996, 5652,  23796, 65404, 55340, 20872, 41640, 6636,
                57400, 18508, 60328, 11612, 39300, 35044, 2076,  38312, 26816,
                32892, 21840, 14764, 39984, 24344, 6744,  19460, 14324, 61992,
                55760, 31592, 60892, 20656, 4432,  15936, 36016, 7116,  31740,
                52228, 12640, 48160, 23904, 56244, 14896, 33400, 59452, 41456,
                29356, 3400,  33732, 5448,  24776, 20936, 57736, 62080, 26796,
                56088, 51312, 28780, 11368, 53076, 18460, 61140, 38272, 28080,
                12032, 6388,  57372, 52792, 1492,  20136, 5616,  64508, 12324,
                27436, 7048,  49696, 11636, 45336, 28868, 40628, 33476, 8488,
                65272, 24332, 39156, 60176, 49292, 55112, 27392, 63868, 37580,
                25232, 5332,  62344, 22252, 50416, 42892, 56352, 15952, 28068,
                31916, 11468,
            },
            {
                51576, 25628, 48384, 9740,  35884, 56332, 31564, 17904, 5196,
                36700, 58960, 1024,  14968, 54180, 22152, 56440, 29476, 48820,
                61560, 25920, 12060, 31536, 56600, 46572, 27064, 42660, 51904,
                24992, 30852, 53992, 43268, 12756, 50968, 9860,  60988, 16392,
                24404, 47660, 8232,  50680, 4660,  22584, 53548, 17532, 42192,
                11144, 52480, 45212, 64448, 32092, 60120, 51348, 39508, 1432,
                28252, 9296,  37680, 45656, 57756, 29524, 62300, 41592, 21708,
                27988, 4896,  59032, 18468, 1172,  63484, 27152, 8564,  13704,
                22792, 39372, 47632, 60516, 43904, 9992,  46892, 14156, 41120,
                22140, 46296, 1896,  39456, 44684, 25944, 34068, 9224,  42360,
                63600, 47732, 26244, 35112, 62140, 14792, 33304, 39032, 48104,
                16860, 56964, 39564, 25128, 60100, 53332, 21892, 13248, 4584,
                47688, 14536, 44448, 9552,  2272,  32456, 45816, 21016, 50780,
                17820, 40228, 45388, 32344, 18324, 4520,  65144, 21872, 8688,
                61316, 57052,
            },
            {
                14276, 5004,  64676, 43740, 27812, 916,   8104,  45884, 26128,
                11880, 48400, 29748, 61060, 10068, 39060, 17748, 7524,  15544,
                21852, 54672, 34840, 41516, 856,   15492, 62780, 19012, 37396,
                9080,  46900, 15732, 35748, 972,   31568, 26632, 48668, 29464,
                36144, 54308, 31852, 25892, 61504, 56636, 31012, 7596,  50496,
                61812, 1232,  27944, 4100,  17228, 46948, 10820, 33904, 24656,
                43936, 17504, 52628, 23352, 1856,  55040, 10012, 51144, 556,
                64648, 37496, 33940, 9032,  53956, 37936, 20772, 45392, 49496,
                65264, 18908, 55524, 16408, 29744, 2372,  38764, 54380, 31000,
                8620,  17328, 65468, 31836, 56804, 13616, 576,   52132, 16660,
                22420, 3072,  39980, 10612, 43660, 49632, 25860, 59272, 388,
                35180, 62884, 3956,  32656, 16300, 932,   36824, 61932, 42728,
                31776, 20696, 53432, 28624, 37308, 16088, 41476, 11436, 7152,
                60444, 27228, 1512,  58548, 38556, 29044, 11788, 53364, 48576,
                4016,  35416,
            },
            {
                41968, 29596, 58668, 12568, 19356, 52424, 60220, 41648, 21628,
                55712, 32944, 19676, 43776, 35376, 63376, 31768, 58080, 42580,
                51472, 4532,  18120, 64688, 22136, 29876, 49544, 3044,  33196,
                12240, 63604, 22260, 40924, 59396, 18920, 44628, 5108,  62644,
                12212, 2828,  43480, 14560, 19128, 47156, 12536, 44612, 23860,
                35484, 19852, 48620, 57780, 37376, 22196, 5276,  56540, 65472,
                48788, 6628,  59556, 12044, 26704, 39136, 18296, 25164, 44680,
                55460, 15672, 42080, 25852, 46320, 31256, 4392,  57184, 35172,
                1340,  26532, 7296,  36812, 53220, 64356, 25464, 18784, 3888,
                59332, 36636, 49616, 5392,  20756, 62888, 48348, 29688, 58448,
                36340, 54268, 28980, 18684, 7032,  55304, 21732, 8800,  30368,
                45156, 19476, 10604, 50832, 43444, 9100,  48464, 18224, 58920,
                26452, 56968, 6372,  63228, 19712, 58972, 3756,  34860, 48272,
                30804, 12892, 52540, 8608,  23952, 47300, 41176, 33668, 25352,
                39452, 20168,
            },
            {
                53044, 22776, 33192, 38864, 48960, 24004, 34556, 14756, 62264,
                3980,  38744, 7724,  52304, 4268,  25864, 46932, 472,   36684,
                27900, 9204,  48120, 38044, 44540, 6424,  60240, 40008, 54432,
                58320, 28960, 4216,  49692, 8652,  53144, 37200, 15280, 56112,
                40624, 21556, 64156, 37404, 276,   40260, 27096, 64544, 5164,
                58980, 13236, 31068, 9060,  43240, 53168, 27664, 41488, 12768,
                20492, 36320, 32164, 42628, 50156, 34292, 47820, 14112, 32192,
                6440,  20416, 49988, 61972, 13060, 51864, 17048, 40756, 11100,
                52608, 32320, 42796, 10552, 22416, 50212, 6192,  34880, 48704,
                43612, 23768, 27604, 10328, 41948, 35160, 24272, 6056,  44432,
                8188,  14472, 59852, 45880, 31784, 63716, 40860, 13344, 51852,
                23476, 54340, 28408, 58208, 34924, 24260, 54872, 29376, 10756,
                1644,  35652, 40456, 12400, 47920, 25576, 52656, 65072, 21996,
                56084, 43220, 33972, 63136, 15332, 55348, 6868,  59820, 16716,
                528,   63292,
            },
            {
                46508, 7804,  2196,  16224, 57252, 4596,  10592, 30264, 49992,
                44304, 17028, 65532, 23540, 13904, 49772, 19984, 12384, 62416,
                32952, 59416, 13880, 26436, 53520, 10828, 24652, 14884, 20688,
                6912,  45108, 17948, 33996, 25168, 63996, 3204,  23292, 33036,
                8960,  51992, 46308, 29100, 58688, 9744,  49432, 32744, 16100,
                38516, 41620, 54856, 25288, 14552, 62228, 96,    16832, 30756,
                54376, 2252,  63408, 15344, 5808,  60840, 3340,  63156, 39920,
                58236, 11048, 29128, 3396,  23288, 7352,  62964, 28712, 24360,
                15164, 58344, 61940, 46368, 13816, 31972, 41984, 60808, 11796,
                54896, 14648, 60400, 52300, 17044, 3288,  54980, 40092, 19192,
                64596, 33588, 49092, 24472, 2388,  16328, 36196, 57888, 3068,
                41932, 37188, 14728, 4904,  20356, 64240, 3276,  38788, 51652,
                45936, 16564, 50344, 33076, 4692,  42888, 29300, 9492,  38152,
                16360, 3408,  19764, 49428, 36724, 1904,  20676, 44588, 31420,
                50392, 10896,
            },
            {
                18056, 61588, 54692, 44608, 28212, 64124, 40196, 54100, 1844,
                27220, 11412, 57032, 29204, 42200, 60052, 6216,  40236, 23068,
                45416, 55608, 20848, 3572,  31952, 57804, 34656, 43308, 48560,
                26400, 38596, 61196, 55260, 13520, 29772, 39540, 58088, 49932,
                26936, 17824, 5996,  34420, 23504, 54920, 3892,  20624, 52640,
                28628, 1864,  63044, 6860,  35868, 32396, 49568, 60460, 38824,
                8348,  46604, 28364, 22480, 55860, 19668, 29940, 9096,  22204,
                52612, 45276, 36336, 59616, 39312, 33740, 43436, 59104, 47120,
                37828, 3772,  20268, 27500, 1676,  57220, 16752, 28416, 21508,
                164,   33520, 6988,  37668, 45612, 61884, 31336, 11296, 50996,
                23244, 980,   12184, 38500, 51156, 5524,  28072, 46472, 17816,
                65132, 7244,  47448, 61100, 40416, 45468, 32116, 13968, 22060,
                63660, 25052, 8420,  61848, 23080, 57920, 14924, 88,    46340,
                24964, 59784, 40316, 10920, 30216, 26248, 64144, 13564, 56276,
                37340, 26972,
            },
            {
                34216, 40380, 24460, 9320,  35752, 20828, 47172, 17804, 37328,
                59800, 34432, 46540, 2612,  32660, 55208, 27380, 52776, 8136,
                17016, 1652,  41696, 63320, 50216, 17764, 8296,  64432, 256,
                52520, 31604, 10228, 1772,  42468, 47108, 19720, 10936, 44248,
                1576,  60264, 54000, 12752, 16556, 62816, 36660, 42584, 57436,
                10312, 44412, 22616, 50476, 19368, 46172, 10400, 21624, 57848,
                25356, 51964, 40936, 10492, 45232, 37560, 53504, 43304, 34988,
                26108, 1716,  17564, 48316, 55784, 19424, 92,    9828,  21488,
                6212,  50840, 40344, 54352, 35736, 63356, 7364,  38316, 46460,
                64028, 40744, 30224, 57152, 21144, 26628, 15192, 59632, 37408,
                27728, 56576, 42100, 60860, 20012, 53712, 61740, 32584, 9824,
                25908, 56460, 30920, 11940, 27100, 16992, 7752,  60012, 42168,
                5204,  56280, 30260, 44644, 18272, 39320, 54044, 31684, 61592,
                50700, 28200, 5168,  45968, 58420, 51296, 42204, 8976,  23280,
                5436,  60092,
            },
            {
                43900, 3480,  30332, 50932, 13980, 59372, 6460,  25596, 8844,
                22644, 51456, 19392, 39660, 10100, 16588, 36964, 64412, 30964,
                49148, 36124, 28852, 12204, 39196, 46268, 28276, 36808, 13136,
                23248, 57224, 16136, 35504, 22340, 52116, 6968,  62216, 36288,
                31136, 21232, 38552, 43196, 47360, 30392, 8012,  25488, 13856,
                48204, 34052, 16428, 59156, 40080, 3020,  26832, 43964, 4124,
                13636, 18172, 34600, 58416, 2468,  24976, 12860, 59856, 5972,
                14996, 65124, 31884, 8016,  13672, 27088, 50096, 54624, 34844,
                64656, 29976, 17088, 8940,  44456, 24152, 48904, 52992, 9680,
                25868, 18300, 50576, 12408, 1428,  48676, 7828,  43004, 5092,
                47392, 17332, 30828, 9560,  26476, 43468, 12480, 21560, 39844,
                50264, 34472, 22272, 596,   52956, 49336, 56012, 35412, 28564,
                19328, 37944, 13084, 52076, 2812,  10888, 35880, 20852, 7024,
                12360, 33572, 54416, 22408, 15768, 4076,  35472, 53448, 29268,
                48776, 14900,
            },
            {
                19952, 12064, 62756, 45940, 188,   41992, 33244, 55468, 62972,
                43092, 14684, 5372,  62112, 48424, 21496, 2024,  44268, 14460,
                25132, 61600, 53160, 7492,  23680, 4396,  56196, 19904, 61704,
                5544,  39972, 49556, 64848, 4948,  58996, 28616, 17196, 13404,
                48512, 65188, 10036, 27548, 1192,  51356, 18572, 60600, 2808,
                65052, 26984, 5772,  52920, 12448, 63756, 55824, 35508, 31528,
                64668, 48164, 6232,  61788, 31924, 50688, 17836, 28924, 46844,
                56988, 51216, 40712, 22036, 44532, 63476, 31408, 15516, 41716,
                11912, 25656, 57000, 33112, 13012, 19900, 2880,  31440, 15476,
                55436, 4184,  44620, 24440, 64976, 35460, 54356, 32448, 21792,
                62108, 3696,  36584, 55152, 6488,  35284, 48572, 4668,  55284,
                2684,  15688, 44784, 59288, 37716, 23728, 3808,  10576, 48232,
                53416, 824,   34256, 59608, 26256, 49220, 63792, 44156, 56760,
                41548, 18152, 64804, 8124,  39392, 32688, 18304, 62388, 2580,
                39576, 57932,
            },
            {
                52804, 32864, 56004, 22672, 17096, 52292, 29724, 12436, 48640,
                1252,  31612, 56304, 24476, 35204, 59028, 51176, 4916,  57284,
                10500, 42848, 18736, 34200, 60440, 16080, 51700, 42648, 30636,
                46660, 27024, 18872, 44364, 32716, 25772, 38780, 55680, 3780,
                41836, 24804, 5216,  55352, 59564, 39572, 33772, 45976, 21880,
                36984, 55696, 30980, 38072, 24444, 29720, 15780, 7120,  53356,
                38280, 22148, 27364, 15864, 39852, 8388,  64100, 408,   38088,
                19380, 9956,  28096, 60036, 5012,  38100, 2716,  23936, 61064,
                49080, 45260, 912,   60232, 51868, 41828, 62256, 36604, 58184,
                42504, 34532, 60636, 9292,  39092, 28672, 17620, 58440, 10768,
                52156, 13920, 45076, 64956, 23440, 15488, 59152, 29088, 62288,
                19144, 42056, 63540, 9164,  33092, 18384, 43736, 64424, 25560,
                15844, 61344, 46908, 21360, 41204, 5684,  28544, 16280, 24340,
                1464,  36616, 44580, 25720, 60928, 47516, 11828, 21784, 44888,
                10064, 25112,
            },
            {
                4864,  38132, 7256,  27168, 36160, 64580, 3976,  19600, 38240,
                28048, 53116, 40908, 7300,  29704, 11968, 26312, 39492, 33144,
                22200, 5872,  58148, 48216, 26748, 40768, 1936,  33436, 8856,
                59672, 11824, 2228,  54700, 14368, 9144,  45548, 50196, 33284,
                18892, 57788, 35400, 15892, 22452, 12224, 5580,  54188, 41304,
                9684,  18344, 684,   46672, 8676,  49100, 41628, 19020, 45340,
                11168, 864,   56752, 43784, 54444, 21476, 48824, 33340, 42424,
                23548, 3632,  35916, 54276, 12076, 51568, 46716, 18380, 7612,
                36836, 5056,  22648, 38588, 28888, 6116,  26792, 11136, 22532,
                6736,  29316, 20320, 53336, 13680, 41912, 2576,  46000, 25504,
                39840, 28596, 19220, 49948, 31992, 268,   39020, 45356, 24780,
                36328, 12984, 51640, 5688,  28292, 56884, 12660, 31704, 39316,
                7248,  43224, 9408,  32340, 14676, 55364, 34696, 8920,  48256,
                59968, 29384, 14164, 50396, 2108,  55144, 27660, 57276, 36472,
                31056, 65308,
            },
            {
                17668, 42608, 49368, 10784, 58044, 39916, 24552, 45348, 60580,
                16852, 10704, 64844, 45844, 15432, 42560, 62680, 18068, 47136,
                51812, 38544, 900,   30528, 13452, 64212, 21924, 54824, 15132,
                50980, 38020, 29756, 41636, 21228, 63244, 436,   23392, 61272,
                7892,  51612, 45256, 30780, 48800, 63860, 26204, 29712, 15512,
                50224, 59868, 43428, 62236, 21212, 57948, 2116,  61336, 23784,
                59120, 50380, 30748, 13160, 3868,  36528, 26032, 11692, 55096,
                62864, 49324, 43996, 16900, 25480, 34104, 58748, 28688, 56036,
                32160, 19516, 63944, 15616, 47168, 55140, 17700, 45744, 51008,
                63624, 14724, 46788, 3600,  57456, 22996, 51148, 7176,  63692,
                33700, 1992,  59944, 8512,  41244, 52960, 18108, 10456, 6972,
                53864, 30248, 21000, 47556, 40572, 61296, 2020,  50600, 20156,
                57712, 29676, 23756, 65040, 3724,  59044, 19592, 51744, 38784,
                4304,  53892, 10428, 20020, 31616, 5924,  41852, 15264, 51044,
                740,   46736,
            },
            {
                28856, 22016, 62188, 15624, 47028, 5748,  9608,  56748, 33928,
                3460,  23036, 36392, 20016, 49688, 280,   54972, 8468,  28312,
                65252, 15972, 54668, 44344, 9812,  37300, 6348,  45220, 25584,
                19408, 62260, 7572,  57420, 51844, 35440, 11484, 30420, 15356,
                39764, 28180, 10944, 3652,  36804, 42836, 8792,  57332, 4616,
                32884, 23740, 11400, 34568, 14596, 52224, 32432, 36816, 28576,
                7820,  40652, 34904, 19296, 62408, 46020, 59320, 16560, 5384,
                29864, 13888, 32232, 61528, 7136,  20556, 42920, 10288, 14436,
                52784, 41308, 50140, 8736,  33832, 2540,  59620, 40340, 32700,
                780,   38632, 49788, 26700, 32016, 61224, 36960, 20224, 15400,
                55024, 43648, 22604, 12920, 57592, 26036, 62984, 49716, 32696,
                57924, 1292,  38584, 25660, 15156, 53624, 22904, 45888, 36260,
                4760,  54520, 17500, 49676, 37772, 45140, 11452, 30956, 63436,
                22560, 46456, 34332, 58528, 38384, 23636, 64088, 7836,  20264,
                60272, 13056,
            },
            {
                56724, 51760, 1560,  34336, 20180, 28368, 50240, 14396, 42572,
                51788, 27204, 59376, 5988,  57552, 31696, 21264, 37116, 3440,
                11840, 24880, 35500, 19836, 59412, 49588, 28424, 57528, 36044,
                4240,  47512, 33924, 25116, 17260, 46728, 59312, 43040, 54104,
                2316,  64328, 21340, 60416, 53136, 17276, 20748, 62432, 47692,
                39100, 53904, 28728, 3764,  40100, 26764, 5272,  12956, 48248,
                16472, 63456, 25304, 52828, 10148, 29232, 7056,  51496, 39644,
                58596, 1880,  21632, 40208, 55844, 756,   65020, 38688, 4000,
                60688, 26460, 12196, 57520, 24288, 37260, 12876, 19636, 7984,
                24676, 60056, 35620, 17368, 8840,  44480, 12304, 30452, 49548,
                38300, 5984,  48508, 35688, 29556, 5052,  37360, 14252, 22428,
                60996, 43848, 11176, 65488, 4344,  33856, 9528,  27440, 14392,
                62976, 40436, 11984, 26960, 53268, 448,   25280, 42792, 14840,
                6724,  26956, 61928, 43324, 16964, 52048, 46188, 32820, 40136,
                26148, 35220,
            },
        },
        {
            {
                21100, 58588, 44260, 5232,  27144, 40648, 57272, 30048, 38384,
                58492, 12420, 36540, 47348, 2084,  24412, 12768, 44468, 31672,
                10040, 19584, 37388, 24988, 1988,  47696, 11460, 51920, 32364,
                1432,  35720, 50900, 27600, 4148,  54208, 48424, 26348, 40656,
                52204, 21432, 1460,  27056, 37020, 19468, 4364,  24672, 8612,
                46540, 62084, 6696,  23484, 37528, 61404, 4248,  25756, 18180,
                62416, 7116,  37908, 28504, 61452, 18544, 44944, 2328,  14460,
                7612,  63840, 42420, 59836, 38700, 53124, 61832, 21928, 32960,
                51472, 26552, 5764,  15668, 63236, 41232, 34868, 28916, 22140,
                37952, 18052, 43916, 4308,  36384, 21744, 40532, 50744, 55692,
                47368, 11780, 60520, 26780, 4840,  43400, 12252, 49480, 17468,
                62672, 2952,  12972, 51184, 41096, 864,   43444, 19704, 31504,
                10468, 40012, 22472, 46800, 33148, 9676,  19640, 45364, 15072,
                7400,  35304, 19240, 12528, 54104, 8200,  21864, 11544, 35288,
                54964, 32452,
            },
            {
                50512, 28820, 12548, 52608, 35376, 49464, 21296, 14024, 42768,
                18324, 50632, 8516,  32732, 16436, 28480, 49952, 59660, 7348,
                27260, 46744, 13400, 51004, 63316, 38620, 29300, 6728,  44076,
                59492, 9604,  22876, 17632, 61436, 37404, 10844, 31136, 17112,
                5528,  38400, 60484, 11444, 46672, 52524, 40996, 60732, 20576,
                36732, 27584, 52044, 43784, 55556, 19884, 48196, 58300, 44712,
                29956, 23560, 13972, 47756, 53992, 12176, 40276, 57536, 25124,
                36880, 54932, 16796, 31064, 6128,  14356, 43144, 55536, 16,
                12160, 58820, 30084, 49308, 24876, 2048,  57864, 6096,  54480,
                61300, 50880, 30416, 62316, 12672, 1592,  58256, 7248,  14588,
                24476, 53548, 31784, 18424, 51992, 63676, 31108, 21368, 53268,
                5188,  37848, 29748, 65484, 16072, 59900, 56280, 5400,  60832,
                45284, 64900, 1520,  57236, 13788, 62404, 3028,  58348, 31924,
                41812, 1576,  49392, 61736, 36712, 47064, 57828, 28104, 4408,
                44668, 15572,
            },
            {
                6800,  41448, 56584, 17220, 8984,  64504, 740,   60596, 6520,
                27252, 62852, 21412, 56864, 42012, 5904,  63820, 18468, 41108,
                53228, 60368, 33480, 5096,  21824, 42576, 15496, 56052, 18456,
                39000, 13768, 63712, 40560, 32796, 46628, 6868,  59592, 55248,
                24976, 49044, 15484, 31352, 7652,  65088, 33452, 11116, 49332,
                3016,  40128, 16176, 10544, 29008, 34484, 7828,  41376, 11008,
                36028, 60060, 51388, 12,    34164, 7804,  29572, 52040, 33244,
                49048, 10552, 23040, 48152, 57796, 9408,  25604, 36364, 64964,
                19504, 46148, 3804,  35636, 18392, 44060, 31720, 10696, 46332,
                25752, 14492, 8032,  52896, 46980, 28200, 63572, 32600, 37684,
                43868, 3388,  8236,  40888, 23724, 48,    45804, 9288,  27568,
                42568, 57696, 47280, 20340, 7368,  27980, 23964, 14048, 35888,
                25148, 16308, 28128, 42588, 49900, 36184, 24268, 51168, 11152,
                65352, 23348, 29924, 5612,  15748, 32096, 2724,  39332, 49148,
                59352, 26048,
            },
            {
                63488, 2660,  22752, 38892, 46324, 25388, 32140, 44620, 35024,
                54164, 2852,  38984, 45424, 11072, 52572, 34732, 3264,  14992,
                23364, 1256,  43624, 30544, 57436, 9788,  35012, 62468, 24280,
                48804, 31048, 7408,  52520, 8,     23852, 19436, 42504, 34236,
                2364,  63344, 43648, 57080, 23792, 292,   17100, 44520, 26004,
                53688, 63988, 32892, 1584,  50444, 63272, 14380, 22304, 55148,
                4800,  16532, 42348, 26320, 59248, 21872, 65144, 16012, 5804,
                61216, 27776, 888,   40036, 34120, 50472, 17436, 5504,  31332,
                40872, 23780, 54680, 38748, 8136,  51724, 64432, 16600, 36412,
                548,   41680, 33180, 23248, 38816, 19064, 42748, 10372, 56600,
                17404, 65080, 46636, 57320, 34224, 14256, 55672, 36096, 60416,
                15128, 23400, 10648, 35032, 53744, 49872, 38856, 46756, 2804,
                53272, 38544, 8276,  30952, 20776, 6472,  40408, 27468, 17692,
                37120, 53600, 44800, 57340, 25520, 42828, 19672, 64920, 23388,
                9700,  37192,
            },
            {
                13900, 33904, 60444, 30456, 4204,  55216, 15704, 11176, 22892,
                47128, 31036, 14340, 25856, 60016, 20672, 30664, 38708, 47536,
                55352, 36340, 61836, 17124, 26452, 53148, 624,   46240, 5588,
                28132, 54800, 42892, 15912, 57752, 29232, 65304, 49676, 14216,
                9708,  28876, 35488, 19248, 50572, 28008, 38172, 57516, 14516,
                6528,  22484, 58876, 19036, 45576, 5224,  39140, 26956, 48896,
                64600, 32376, 9764,  38324, 18084, 43728, 46964, 1836,  39308,
                19904, 43496, 13464, 64428, 20428, 29144, 59480, 46912, 52340,
                15376, 9652,  62972, 13668, 56532, 27408, 23100, 40204, 59024,
                20628, 55456, 65292, 4812,  15624, 53972, 3832,  25728, 21868,
                35876, 28600, 12372, 20480, 62036, 29444, 39840, 6000,  50288,
                32876, 1284,  55600, 40620, 4064,  30964, 11404, 63828, 19296,
                32744, 58924, 12020, 63208, 53760, 15280, 56132, 46968, 2440,
                59948, 13960, 20828, 9632,  50836, 61080, 12168, 52936, 17140,
                31544, 47656,
            },
            {
                20152, 43484, 51228, 10680, 19108, 62440, 37012, 51408, 59112,
                17668, 5248,  65404, 36172, 60,    48800, 24788, 57840, 9332,
                29252, 20192, 11984, 6932,  49236, 37720, 20452, 32976, 12412,
                59020, 2872,  20000, 35584, 45788, 12740, 4836,  36820, 56376,
                23112, 47056, 3736,  12640, 41696, 54520, 9060,  61808, 30392,
                35436, 47728, 12052, 41212, 24708, 60288, 31400, 53308, 1888,
                19912, 45868, 57384, 3444,  54784, 10852, 35792, 24176, 50544,
                56772, 31900, 46172, 54424, 3240,  42788, 12428, 37992, 1832,
                28112, 43532, 33176, 20860, 46808, 1368,  12524, 50140, 5676,
                27976, 48368, 10908, 44576, 59884, 34928, 49816, 61928, 45388,
                6296,  51524, 38440, 2320,  48456, 10160, 18164, 24980, 63496,
                19500, 44020, 26292, 63072, 17044, 22004, 42968, 55892, 7588,
                50376, 22404, 48052, 44092, 384,   34112, 60860, 9372,  43480,
                32448, 4532,  39456, 34552, 444,   27732, 37924, 6260,  42200,
                1536,  54020,
            },
            {
                25280, 7756,  56016, 27040, 48876, 42176, 7016,  28680, 39580,
                9228,  49360, 41580, 55308, 16856, 7604,  42856, 13072, 64292,
                4088,  51540, 39952, 45784, 65276, 14532, 58168, 41708, 50680,
                22624, 38124, 61144, 25504, 8400,  51840, 40028, 26832, 17276,
                60904, 52144, 38524, 62700, 6236,  32828, 47380, 19820, 42268,
                3456,  56104, 28364, 36832, 52372, 9496,  16956, 35504, 12860,
                40512, 28584, 22676, 50292, 31268, 63132, 14240, 28968, 60140,
                9724,  17740, 6264,  35776, 24548, 8256,  63184, 21972, 54228,
                61180, 48556, 3288,  58684, 30488, 41984, 60572, 34104, 9300,
                31556, 17160, 38080, 24700, 29172, 8472,  13644, 30248, 608,
                59096, 15260, 54816, 26064, 43552, 58788, 53152, 2812,  46704,
                12480, 37560, 8720,  48796, 57972, 33744, 716,   27236, 15536,
                41032, 29204, 5828,  18352, 26368, 38780, 22716, 29432, 19112,
                52496, 24944, 63924, 48224, 56236, 15008, 46608, 34076, 57000,
                29056, 62096,
            },
            {
                15940, 40296, 828,   13340, 34856, 21488, 2136,  54656, 24880,
                61556, 20264, 28020, 10304, 32048, 61140, 53088, 19184, 44928,
                32904, 26136, 56260, 23548, 32008, 3236,  28832, 8812,  63640,
                16928, 44796, 10776, 48532, 33868, 61932, 20716, 1272,  45092,
                7172,  30196, 20960, 16224, 58744, 23016, 2232,  13276, 25060,
                51384, 17588, 7712,  65356, 804,   21692, 57152, 44968, 62116,
                8156,  59524, 15028, 5640,  25672, 40744, 7512,  53440, 3300,
                37996, 26232, 61504, 51708, 16216, 56652, 32900, 26600, 18152,
                7860,  35188, 24648, 16504, 5328,  36852, 18956, 54004, 45764,
                62912, 56980, 1864,  51796, 20148, 57424, 40608, 48172, 34124,
                19736, 41916, 31680, 7776,  22408, 36584, 14868, 41512, 31080,
                58288, 52000, 29348, 14392, 6040,  45984, 61964, 52152, 36920,
                59992, 3248,  35028, 57324, 51476, 12624, 62744, 5616,  48840,
                58736, 11032, 42424, 17184, 30560, 8448,  24188, 60616, 20844,
                11200, 45112,
            },
            {
                32120, 57824, 37628, 65120, 45220, 58708, 32988, 15088, 43988,
                4420,  34284, 51896, 45572, 22936, 39652, 27132, 5524,  37124,
                15632, 59860, 1708,  10436, 18940, 44296, 54188, 25376, 35432,
                4312,  31548, 53852, 2044,  15032, 30108, 54592, 59364, 33212,
                41336, 10460, 55328, 43380, 27216, 36564, 52544, 64352, 37944,
                58508, 46140, 32076, 14772, 48992, 42036, 29512, 3908,  24352,
                37756, 51684, 33740, 47500, 61496, 19824, 48928, 32872, 22244,
                42276, 48112, 12144, 29536, 39728, 49548, 45536, 4348,  41796,
                57296, 11580, 40280, 53636, 49252, 62548, 25440, 14376, 4496,
                22308, 42376, 13200, 35700, 63920, 2972,  17312, 55112, 23996,
                64296, 9872,  49376, 61208, 3888,  33204, 64912, 27000, 7620,
                22628, 3856,  61204, 36332, 24956, 39612, 18824, 9096,  24116,
                13476, 45580, 64740, 41836, 7948,  31572, 54328, 40736, 14784,
                36488, 28040, 6432,  21992, 62572, 51432, 41116, 3124,  48824,
                36344, 4188,
            },
            {
                18736, 50056, 28532, 23704, 16976, 8668,  50244, 63788, 11820,
                37580, 56904, 2204,  13712, 58060, 35292, 1500,  62424, 49984,
                21772, 41816, 47964, 35972, 60872, 38852, 5228,  47576, 13308,
                56504, 26744, 39544, 64120, 23476, 42612, 6304,  12280, 50068,
                24480, 64812, 34068, 772,   49028, 9508,  30848, 44140, 5116,
                10332, 21132, 40156, 26480, 59556, 34500, 11076, 47960, 55696,
                18008, 664,   43172, 12060, 36916, 2556,  44172, 56388, 14980,
                63764, 4784,  57572, 20976, 1260,  10728, 36620, 14564, 30612,
                51276, 21372, 64884, 28320, 7220,  10800, 43940, 29616, 38676,
                50936, 26732, 7736,  46820, 31244, 43732, 27032, 11700, 6688,
                37744, 27768, 17708, 56808, 13052, 45796, 19924, 48560, 55540,
                39344, 17356, 45072, 20584, 54940, 11824, 30496, 48528, 56556,
                32636, 19724, 10352, 24392, 16228, 46872, 3152,  20956, 33308,
                1012,  55412, 45976, 38540, 2316,  32972, 18356, 13636, 27184,
                64280, 52580,
            },
            {
                59800, 5776,  10464, 47224, 3816,  30940, 40784, 23092, 29392,
                48104, 18476, 26364, 64384, 6912,  16212, 47124, 30096, 11136,
                53872, 6160,  13912, 29884, 51136, 15860, 63036, 21392, 41228,
                60688, 19456, 7740,  50440, 18172, 37776, 46788, 28124, 19644,
                15732, 4620,  46564, 13568, 61700, 18632, 55000, 15588, 28244,
                34108, 62612, 2856,  54696, 5872,  19508, 63340, 15308, 32632,
                26760, 64128, 21480, 57164, 29608, 17332, 59892, 11096, 27744,
                36188, 18148, 31576, 44496, 65312, 23352, 58504, 62152, 47132,
                432,   38112, 13892, 44936, 33788, 22996, 55764, 2888,  58372,
                34628, 19220, 61272, 54664, 22036, 5180,  39092, 59676, 47456,
                52676, 1552,  44448, 38864, 29000, 52100, 10032, 468,   34200,
                62692, 10580, 32224, 1804,  50252, 65192, 5360,  43284, 2544,
                60988, 52976, 28696, 49944, 36652, 59192, 25888, 45012, 64092,
                51028, 23804, 60220, 13112, 53276, 44440, 59040, 54924, 39336,
                9056,  22728,
            },
            {
                14080, 34132, 41672, 62276, 51816, 56232, 19624, 604,   53556,
                60472, 9032,  32940, 40248, 49560, 20888, 42704, 56488, 24824,
                34244, 63256, 26888, 56948, 7972,  24048, 32804, 9712,  52440,
                656,   46216, 29360, 11764, 55568, 4072,  63076, 35452, 57628,
                39856, 52952, 25836, 37916, 22208, 40828, 3184,  60248, 23636,
                49544, 42868, 13008, 51520, 24192, 38928, 43612, 4924,  53660,
                9980,  40600, 6724,  53112, 25224, 8976,  38940, 51096, 312,
                46956, 55040, 8532,  34740, 52556, 6120,  28464, 19520, 8768,
                25700, 60308, 4868,  18568, 59104, 51368, 40792, 16808, 64624,
                11576, 140,   40460, 15804, 10272, 50400, 62660, 32692, 14428,
                21244, 35016, 62928, 23456, 5824,  40920, 60168, 24540, 14476,
                42644, 53620, 26208, 59688, 40844, 34320, 16500, 27832, 38376,
                22220, 6536,  40076, 892,   62960, 18796, 6940,  56512, 11408,
                16812, 8232,  35052, 19088, 28636, 10396, 24940, 6508,  30984,
                46944, 42948,
            },
            {
                25568, 54636, 20440, 36576, 26096, 12500, 38636, 45036, 5980,
                15756, 42352, 22412, 54572, 28428, 12212, 60432, 8104,  17480,
                2796,  38744, 19728, 44816, 1148,  42640, 58236, 28408, 37352,
                15240, 34564, 42980, 59596, 32996, 26088, 10028, 52016, 1456,
                8452,  60420, 29328, 56324, 6768,  51036, 35316, 46364, 7760,
                56528, 16864, 29828, 36016, 8632,  57336, 29116, 22540, 49320,
                35860, 45908, 30824, 14776, 47192, 64948, 32352, 21016, 24356,
                40380, 61432, 14104, 25584, 42004, 16440, 54432, 34152, 43516,
                55704, 32036, 48068, 36656, 1524,  30756, 8740,  21348, 47324,
                31884, 49488, 28676, 57992, 36836, 25136, 18516, 45256, 29548,
                4568,  54672, 11012, 15504, 32396, 55804, 18264, 37452, 49808,
                29668, 6100,  47652, 14060, 8772,  23128, 57640, 51836, 12324,
                47496, 15600, 55276, 33352, 12984, 42632, 30976, 27720, 39200,
                43436, 31688, 48060, 65244, 4228,  36792, 49168, 63204, 16464,
                35200, 3348,
            },
            {
                49776, 30240, 1112,  15196, 7192,  60176, 33280, 63276, 24504,
                35704, 46952, 2740,  62544, 5396,  37316, 31320, 52092, 40696,
                46396, 59144, 10256, 49200, 34812, 53200, 18132, 3848,  48348,
                24656, 64708, 21768, 2772,  48632, 14740, 22744, 44768, 31412,
                21444, 43456, 17304, 10932, 32708, 63876, 26072, 11900, 19996,
                38204, 100,   59940, 44932, 18276, 47444, 13232, 61648, 3104,
                58444, 19312, 62260, 2216,  35120, 5352,  43492, 54428, 7556,
                30516, 4292,  20120, 49992, 38240, 3724,  48772, 12240, 39704,
                15968, 22204, 53240, 26540, 63348, 13076, 45624, 37552, 5940,
                53996, 23208, 44332, 3972,  33436, 53312, 1000,  8564,  58376,
                41492, 26496, 50048, 59244, 47520, 2504,  27476, 64404, 4240,
                21700, 58028, 18576, 38104, 29080, 46600, 40,    19048, 64016,
                35332, 43860, 61480, 23600, 8972,  48912, 53208, 3648,  21740,
                58652, 1776,  25424, 40804, 57124, 21684, 43204, 144,   20104,
                56500, 60528,
            },
            {
                11308, 64616, 46520, 57896, 43176, 29100, 17844, 9796,  50000,
                58396, 30616, 11208, 51128, 18048, 45300, 1364,  22584, 64952,
                27888, 14900, 31872, 22356, 63980, 13496, 40228, 60192, 11172,
                54876, 6852,  51504, 17472, 40952, 57048, 61444, 38720, 12884,
                64316, 34436, 49520, 2640,  45120, 14832, 30124, 42160, 52396,
                62924, 32440, 10712, 26612, 64804, 1960,  33228, 41180, 16660,
                28216, 11852, 38624, 22736, 50240, 58572, 18504, 13564, 63304,
                51828, 45500, 59296, 11216, 56564, 29928, 62464, 24344, 2148,
                64460, 6604,  10320, 41956, 17116, 56536, 24632, 61128, 27540,
                13924, 59376, 9012,  64256, 12960, 42128, 56688, 23292, 38280,
                65384, 17612, 7728,  36528, 21104, 43676, 8728,  53340, 45608,
                12108, 35116, 63716, 3400,  55068, 61096, 42300, 31860, 8020,
                26448, 3564,  30024, 19828, 58052, 38136, 14984, 34904, 61616,
                49988, 13560, 53640, 7568,  15436, 30360, 11948, 52904, 38820,
                28616, 7808,
            },
            {
                36072, 18128, 39776, 23712, 52996, 48276, 2324,  55120, 27556,
                13792, 20356, 39248, 57068, 34016, 24456, 55292, 12924, 35512,
                5512,  51280, 55116, 3044,  25980, 7048,  29784, 21052, 44580,
                31592, 39112, 27332, 9360,  35768, 29944, 6356,  18400, 47668,
                3684,  25216, 40160, 59784, 20472, 58096, 1716,  55232, 5280,
                25200, 48248, 21628, 51996, 38564, 7320,  50488, 56028, 24748,
                7724,  51484, 43088, 56708, 26464, 10532, 40776, 28196, 34288,
                16740, 36872, 27364, 1560,  33020, 21312, 8260,  44888, 35672,
                57632, 46116, 29740, 49800, 5092,  34524, 52032, 3216,  42816,
                35132, 17464, 39464, 20632, 47792, 27816, 16484, 49396, 12096,
                32220, 46176, 3432,  30012, 62328, 12896, 34572, 16684, 30872,
                41004, 50956, 25112, 44436, 21256, 11604, 36348, 24232, 54452,
                59420, 11288, 51324, 46440, 6036,  64880, 24740, 45156, 10052,
                19412, 29416, 37312, 45332, 60632, 51092, 33552, 61904, 24216,
                14344, 44884,
            },
            {
                22000, 5644,  32540, 3324,  11624, 21260, 36876, 40632, 4604,
                43560, 65208, 7584,  26968, 14692, 41412, 61512, 48888, 20328,
                9116,  41968, 17372, 37516, 44136, 55900, 35868, 50380, 62020,
                5008,  14028, 58528, 62452, 46828, 212,   53856, 27548, 51268,
                56656, 15508, 5696,  53172, 24208, 38396, 47640, 17908, 36380,
                14512, 41440, 3824,  58752, 15364, 30936, 20208, 37104, 44528,
                60448, 33960, 4680,  15792, 31604, 1244,  45832, 60648, 3060,
                9352,  22848, 42860, 64692, 46664, 17216, 59500, 51764, 26848,
                19308, 13332, 37620, 61768, 21080, 40468, 11068, 19132, 48892,
                55524, 1288,  26024, 52288, 31156, 6196,  63008, 35348, 2564,
                20784, 53032, 42524, 24468, 51628, 39812, 56900, 22664, 60500,
                816,   9388,  15160, 32588, 7084,  52768, 16088, 4324,  48972,
                37584, 17400, 41000, 34448, 28208, 1440,  18096, 54684, 5356,
                40400, 63300, 23160, 2712,  18236, 26860, 5316,  47384, 2360,
                40940, 53732,
            },
            {
                48836, 57224, 27300, 60724, 34480, 64000, 15472, 59028, 23236,
                33840, 53212, 268,   46044, 59120, 9560,  4464,  38596, 32372,
                57684, 62652, 28796, 47548, 59576, 9532,  15980, 876,   24136,
                18336, 34460, 43692, 20212, 12604, 24524, 42400, 37200, 8588,
                30648, 36276, 63376, 28732, 9648,  33672, 7280,  65048, 29024,
                59652, 8428,  34884, 55032, 43364, 23820, 62508, 10416, 612,
                29852, 54040, 21160, 47556, 62896, 36400, 20080, 55320, 48616,
                39536, 58072, 13288, 6368,  53932, 38724, 14148, 41344, 5456,
                33220, 54764, 23920, 936,   58728, 31652, 28328, 65152, 7656,
                32244, 61812, 45380, 4688,  60392, 38740, 10400, 54376, 25344,
                61164, 14316, 56572, 9892,  18684, 1672,  48588, 6772,  26396,
                54100, 37396, 57328, 62444, 48592, 39396, 30328, 63460, 43320,
                21736, 2584,  53596, 14092, 56400, 42172, 32064, 48472, 27164,
                57816, 32564, 8956,  55192, 35088, 41944, 10164, 58120, 17028,
                31892, 62964,
            },
            {
                9232,  13800, 51720, 42460, 7888,  45632, 30308, 50824, 9072,
                47480, 18232, 37616, 22268, 50540, 30252, 17164, 25808, 45768,
                692,   23288, 11784, 5140,  19920, 32720, 65136, 42044, 47036,
                57412, 52988, 2304,  30412, 55404, 64904, 15852, 21180, 60636,
                11692, 44488, 18656, 41784, 13336, 51804, 43912, 21528, 11368,
                50620, 45488, 18976, 28296, 12152, 5404,  53496, 46204, 14704,
                18588, 65424, 8748,  39832, 13096, 52088, 24120, 6712,  30860,
                25956, 53076, 18680, 35128, 28972, 23700, 188,   31024, 61456,
                9776,  42512, 47760, 16264, 8804,  53100, 47016, 15312, 24100,
                41624, 11644, 36512, 14556, 23660, 18952, 43512, 46708, 29492,
                40240, 5288,  33852, 63776, 37680, 28836, 61580, 32328, 46684,
                19788, 43576, 28316, 17916, 1868,  26736, 58488, 12976, 8372,
                32956, 62308, 25316, 9428,  20788, 62728, 11432, 36804, 14708,
                660,   43184, 46948, 12720, 49504, 64604, 22916, 36864, 54904,
                25916, 3920,
            },
            {
                29768, 37008, 17296, 24564, 55664, 19588, 1036,  26020, 61892,
                12716, 28472, 60372, 32724, 3332,  42972, 53424, 64184, 13720,
                50060, 39808, 34648, 52148, 25240, 54056, 12816, 28572, 37248,
                8932,  26432, 40624, 49532, 7380,  33532, 4096,  57616, 46624,
                2184,  23328, 55256, 48848, 460,   61724, 26332, 56484, 39360,
                32192, 23104, 912,   64108, 49116, 40476, 26788, 58648, 35592,
                49784, 41760, 25456, 28948, 4128,  58884, 42924, 10868, 15644,
                61112, 44232, 2252,  49904, 62868, 10896, 48144, 55732, 17664,
                50740, 3528,  63404, 27072, 35068, 44068, 2264,  37492, 56256,
                20468, 28896, 54496, 50896, 58360, 33936, 1168,  57488, 7956,
                17200, 49032, 26904, 45800, 4492,  15728, 42392, 10652, 14408,
                64632, 5840,  34860, 11056, 42208, 22340, 46032, 18780, 55992,
                27948, 47712, 58240, 38432, 45596, 50916, 6836,  60356, 22084,
                51624, 17340, 25228, 59012, 20708, 30072, 7268,  45640, 11988,
                19876, 43628,
            },
            {
                47736, 65368, 40488, 3712,  49316, 13584, 39128, 56776, 5720,
                42192, 54544, 15680, 10264, 62272, 19896, 35912, 7356,  29248,
                56728, 16348, 61212, 43296, 1848,  38968, 48884, 21840, 3940,
                61564, 15800, 11396, 22768, 38152, 44256, 50364, 25932, 39772,
                32592, 62332, 6492,  35232, 20364, 31240, 16340, 6016,  2920,
                61364, 15608, 55968, 37648, 33032, 17456, 2604,  31716, 21856,
                3556,  12184, 57288, 35172, 49328, 17896, 32820, 64552, 37764,
                4444,  21788, 32420, 40840, 7232,  58984, 44040, 25832, 36548,
                21620, 29540, 38632, 57736, 19636, 13036, 60052, 5368,  50212,
                62904, 3084,  17948, 8396,  47396, 27148, 12836, 21096, 52244,
                36388, 60012, 11668, 20004, 54860, 23948, 58304, 52024, 39044,
                2536,  23832, 49664, 56448, 61352, 5256,  50812, 35724, 600,
                40116, 15184, 5500,  30876, 2168,  23900, 29516, 39792, 55832,
                34388, 63756, 5976,  37804, 3644,  15444, 40068, 62532, 51780,
                34968, 59348,
            },
            {
                2000,  22436, 10856, 28372, 62576, 32056, 44408, 35812, 20764,
                31316, 2520,  39920, 46784, 26436, 55340, 11660, 23732, 44316,
                3516,  21116, 8460,  30552, 63260, 18108, 6892,  58888, 31360,
                51264, 33952, 56328, 63592, 18596, 28712, 9412,  13444, 18028,
                52564, 27136, 15392, 58760, 40536, 54128, 47248, 36112, 42508,
                52744, 25692, 47040, 6988,  60496, 10012, 51664, 63452, 38884,
                55652, 44480, 6012,  62072, 22352, 276,   54448, 27972, 45944,
                50704, 12712, 57380, 26716, 16316, 20492, 33560, 3012,  65400,
                14604, 45528, 11336, 6124,  54172, 41328, 22480, 26588, 33344,
                10340, 44456, 31396, 40316, 4208,  65008, 42540, 32464, 62068,
                23444, 2012,  30212, 40884, 49636, 35752, 7680,  20792, 33616,
                59580, 45000, 30088, 14128, 37728, 32356, 9636,  65472, 23972,
                44112, 52532, 19716, 36036, 59280, 12840, 44248, 18552, 3108,
                10152, 30736, 41608, 27876, 50808, 56812, 32672, 556,   27672,
                6308,  15720,
            },
            {
                32316, 53580, 57596, 35428, 6064,  52180, 9296,  16816, 64768,
                49160, 23032, 58084, 34256, 5432,  38176, 48964, 60008, 32000,
                53040, 37592, 46888, 27044, 14248, 55408, 45152, 10744, 41236,
                19092, 44640, 5988,  47272, 2616,  59264, 53880, 61448, 35692,
                7672,  47724, 10948, 29472, 4344,  23708, 8960,  63880, 19648,
                10348, 30200, 13224, 43772, 20736, 24688, 42716, 13868, 7968,
                24276, 30380, 16604, 46372, 38976, 14020, 41244, 8052,  19532,
                35476, 63308, 9192,  46900, 37240, 54708, 12776, 39940, 8140,
                56444, 52020, 33880, 24312, 49364, 31124, 63832, 16620, 46496,
                39096, 59044, 25000, 60908, 35928, 16536, 50612, 5760,  39072,
                14680, 43924, 56124, 8824,  65148, 492,   30676, 47428, 25984,
                16868, 54704, 8144,  20036, 52856, 26016, 16380, 54756, 29592,
                11940, 7908,  63244, 49092, 16696, 53044, 65052, 26144, 49652,
                45852, 14052, 53224, 60888, 11328, 43348, 18204, 48608, 24004,
                55812, 38872,
            },
            {
                7964,  17964, 42692, 46256, 20116, 24140, 58868, 27844, 45776,
                11956, 52772, 8084,  18840, 63604, 15380, 1408,  41304, 17600,
                6248,  57900, 11128, 51372, 33684, 22712, 36156, 26188, 62664,
                508,   23460, 27952, 36292, 15192, 32116, 41348, 22972, 752,
                43152, 65200, 37324, 51416, 45400, 59832, 14000, 27784, 50100,
                34316, 58612, 3752,  36360, 54652, 29628, 58296, 47592, 19168,
                50688, 60144, 34100, 9608,  26996, 51952, 57608, 24444, 60312,
                1940,  30192, 23152, 52888, 4968,  62476, 28748, 49820, 22908,
                42048, 4764,  17412, 62044, 464,   9860,  36960, 7100,  53656,
                13264, 1748,  19764, 49412, 9536,  23120, 55156, 28724, 10356,
                47436, 58976, 33724, 16832, 26412, 13432, 54288, 62720, 3744,
                11724, 39784, 64040, 1220,  41772, 45728, 4120,  38956, 48148,
                60360, 22560, 32472, 27180, 41100, 4964,  33508, 8352,  36524,
                62268, 23744, 19616, 2404,  34916, 22392, 7764,  59788, 12956,
                42064, 61900,
            },
            {
                50952, 25792, 12484, 56,    61004, 14732, 40192, 5272,  1276,
                36736, 25812, 43068, 29728, 51584, 44788, 21980, 27616, 65396,
                34956, 24536, 42000, 59956, 5548,  2340,  50500, 56908, 13820,
                38572, 54108, 60480, 11856, 51772, 20064, 5444,  46420, 30176,
                56108, 20504, 24552, 1892,  18480, 32836, 38236, 56760, 1184,
                22704, 40064, 65156, 48916, 15328, 1252,  34872, 4856,  27868,
                37340, 1660,  54296, 20796, 63196, 31416, 5384,  48536, 16512,
                43036, 55748, 14500, 41616, 31732, 808,   45988, 18760, 60924,
                31040, 26304, 58152, 39364, 43620, 27708, 48000, 57940, 21816,
                28652, 34704, 55868, 14620, 30260, 45480, 92,    19552, 63952,
                25320, 3208,  20912, 53056, 39480, 44432, 18880, 37384, 42928,
                51116, 28004, 35420, 23576, 58792, 11508, 62304, 20412, 33964,
                3052,  42288, 55368, 1304,  10640, 57548, 21248, 42988, 16220,
                57960, 6600,  38964, 48204, 26528, 65004, 54052, 36840, 45844,
                20260, 29192,
            },
            {
                47296, 34332, 64364, 38288, 28952, 55012, 49964, 33728, 55888,
                62792, 14320, 59516, 3476,  11044, 32516, 56684, 9184,  13316,
                49312, 52,    19420, 15896, 39184, 64204, 29276, 8784,  17452,
                47808, 30684, 7896,  42500, 58036, 37948, 26912, 63472, 12468,
                16112, 33460, 58164, 42120, 12580, 62536, 6548,  43332, 16476,
                46144, 5784,  18420, 27068, 9320,  62236, 41020, 53828, 11740,
                64704, 42492, 15024, 47956, 2744,  12852, 37472, 33828, 9948,
                27572, 38140, 6552,  50272, 17392, 24704, 58872, 10452, 35080,
                1872,  47268, 12064, 21148, 55136, 13948, 19076, 4480,  41456,
                64764, 51788, 6416,  41044, 62832, 37484, 57920, 41516, 31876,
                51768, 37284, 6828,  28992, 48452, 5088,  60800, 9516,  22776,
                57092, 6240,  47020, 17628, 31308, 50152, 27124, 7360,  57508,
                17280, 13300, 46656, 38556, 24652, 48100, 30080, 54200, 176,
                28460, 32844, 55172, 9456,  44572, 30000, 16160, 4560,  31828,
                10296, 2612,
            },
            {
                15092, 21912, 53136, 7296,  32492, 10804, 43520, 21900, 17856,
                30760, 39320, 20160, 47568, 24056, 61472, 36996, 53840, 45472,
                29604, 62484, 54464, 31840, 44752, 48356, 20316, 41876, 34548,
                4412,  65248, 21576, 25512, 1412,  48636, 9816,  54824, 39564,
                50284, 5740,  9244,  53108, 26796, 48656, 21136, 53512, 29792,
                61228, 55668, 32460, 51512, 44992, 21548, 31152, 16884, 45916,
                22264, 8560,  29576, 39432, 58516, 44504, 22732, 64932, 53648,
                45888, 59372, 21004, 64288, 35996, 43776, 7188,  39180, 53944,
                15592, 62716, 36520, 6680,  33288, 50656, 60892, 31744, 45272,
                10944, 23620, 47344, 27008, 3948,  12308, 52792, 7560,  17072,
                13128, 42720, 62296, 11124, 57936, 24424, 34352, 29648, 16168,
                33160, 13716, 61780, 53752, 2836,  37060, 14572, 43596, 51704,
                36568, 28368, 59856, 18860, 64020, 14400, 37300, 61756, 12072,
                51316, 42316, 17252, 59048, 13692, 1324,  40912, 49256, 25552,
                63364, 57404,
            },
            {
                5648,  43964, 59360, 17380, 48164, 3768,  25408, 59232, 8700,
                50800, 6392,  54120, 34780, 41188, 6980,  18144, 2632,  40020,
                21540, 7700,  36500, 25844, 12000, 6960,  24292, 59564, 53396,
                44392, 13056, 50608, 33364, 16580, 61872, 34748, 18668, 3716,
                25292, 45484, 61056, 38704, 30904, 3740,  35372, 10976, 25428,
                8072,  36976, 12156, 23924, 38840, 56604, 7076,  59404, 26036,
                33500, 51864, 56172, 25116, 6896,  16984, 50828, 3428,  19216,
                13320, 1268,  32444, 11340, 27312, 55484, 13764, 50892, 28288,
                20348, 44108, 49536, 29316, 64356, 2208,  25012, 36324, 16276,
                836,   38404, 17984, 57388, 33608, 21640, 26164, 35036, 61256,
                49752, 22324, 55272, 35472, 15552, 2416,  52220, 46748, 58948,
                800,   44376, 20916, 9864,  40544, 56872, 22108, 30512, 64348,
                23756, 5152,  8520,  50320, 34160, 3368,  7460,  45548, 20048,
                25144, 4852,  63128, 21804, 37376, 51640, 60224, 18748, 55008,
                35544, 39956,
            },
            {
                29816, 9904,  27088, 41240, 20748, 63608, 36924, 47136, 13172,
                41792, 26948, 2368,  64376, 15216, 50304, 31028, 25392, 60424,
                14664, 47092, 58388, 3640,  52480, 61684, 14848, 31104, 2748,
                27368, 19068, 37332, 56860, 43928, 6968,  22864, 41920, 60304,
                29020, 35940, 17396, 22568, 14364, 56308, 64652, 40268, 49936,
                15200, 42548, 1852,  63228, 4756,  14124, 36024, 49624, 408,
                62000, 5308,  18724, 35960, 60548, 41676, 26772, 30656, 61024,
                40288, 24620, 57344, 47236, 3868,  61636, 22276, 32920, 5576,
                56824, 3588,  9964,  23188, 15880, 42464, 8944,  52196, 57072,
                29052, 62012, 8048,  43920, 50424, 16060, 59508, 46932, 2968,
                27608, 4980,  30796, 46100, 19948, 41752, 64372, 7824,  39632,
                25760, 50012, 34888, 28476, 63504, 4932,  48860, 10488, 352,
                45436, 54536, 31212, 42700, 21952, 53080, 27740, 59464, 40732,
                49220, 35684, 30944, 46088, 6052,  24036, 33412, 8220,  12536,
                22884, 51948,
            },
            {
                62224, 37048, 1924,  56840, 51516, 14948, 29976, 1700,  61972,
                33996, 57852, 22724, 11436, 28376, 57444, 43268, 5080,  51560,
                33632, 10392, 42592, 28556, 18636, 37760, 49916, 40448, 10156,
                55312, 61344, 4960,  10900, 28784, 52752, 31464, 46740, 13984,
                53568, 920,   63644, 47968, 7456,  44176, 18292, 420,   31780,
                54140, 20448, 58832, 47904, 29644, 52884, 19760, 11000, 41880,
                38168, 14312, 46248, 11260, 32264, 53264, 9180,  48292, 6176,
                34764, 44144, 52008, 8444,  37904, 18620, 48500, 42652, 64980,
                25048, 40768, 59092, 53528, 38648, 46824, 59604, 12496, 20524,
                48464, 54332, 31420, 11212, 63908, 1600,  38696, 9940,  44284,
                18836, 39896, 63280, 8684,  50980, 27284, 12936, 21752, 54732,
                17440, 59944, 5848,  43044, 24372, 16664, 33516, 53448, 19044,
                39176, 61992, 15664, 11332, 57148, 39796, 17092, 33076, 9364,
                14896, 1964,  53156, 11596, 56920, 28872, 64368, 47116, 43396,
                1008,  16980,
            },
            {
                45560, 49284, 24252, 12116, 34908, 6004,  40272, 53336, 19528,
                44072, 16456, 48484, 38412, 45412, 9048,  20276, 63948, 38184,
                17288, 56244, 22180, 64752, 32924, 1084,  57248, 21464, 35144,
                46492, 23976, 39416, 48340, 14908, 64088, 1948,  10344, 57644,
                20476, 32532, 11312, 40760, 51868, 28536, 58404, 23424, 46680,
                61776, 27168, 33992, 16732, 41136, 24868, 64116, 44844, 27524,
                21224, 57768, 50108, 23384, 2004,  63024, 15268, 37260, 22284,
                54984, 16728, 29148, 14236, 41408, 30320, 1124,  11616, 36648,
                17560, 31540, 12804, 34144, 19136, 27344, 3900,  34636, 40668,
                24664, 5932,  37044, 19564, 41820, 29508, 23596, 56180, 31484,
                52712, 12272, 24220, 424,   59888, 38064, 31440, 3312,  36032,
                11236, 30324, 52840, 13396, 47824, 38216, 60728, 42072, 13004,
                26316, 35184, 48948, 4200,  25092, 63372, 1448,  51672, 23180,
                65380, 55836, 26380, 19752, 40252, 15636, 3880,  38048, 26580,
                60656, 33000,
            },
            {
                14056, 19600, 64584, 31288, 44876, 61236, 23528, 27880, 10180,
                4656,  31460, 54632, 60968, 680,   36092, 53532, 12592, 27424,
                1508,  49572, 5924,  40652, 13680, 45196, 26252, 6084,  64236,
                15936, 264,   32068, 59084, 21092, 25600, 40052, 49500, 36848,
                6704,  43756, 58848, 25840, 3072,  33884, 10516, 37164, 4896,
                12928, 7648,  44536, 3500,  57172, 8412,  32980, 2728,  55084,
                30916, 3956,  65348, 39716, 28640, 20076, 45344, 58332, 11752,
                62100, 4540,  20852, 63992, 58828, 24404, 53836, 60784, 8108,
                51776, 47140, 200,   63052, 6332,  55944, 30924, 61368, 44856,
                2664,  15060, 58900, 25860, 53804, 5600,  48792, 14748, 65324,
                36316, 58004, 43580, 33468, 16400, 55980, 45628, 61520, 48240,
                41576, 65280, 2120,  19936, 57708, 8364,  2620,  29040, 56220,
                7168,  21192, 58968, 29400, 44524, 36012, 13068, 47392, 29224,
                41948, 37032, 8096,  44276, 48820, 58616, 10368, 51204, 20944,
                56060, 4720,
            },
            {
                52540, 39444, 3080,  54416, 8208,  17032, 48576, 56144, 38108,
                65260, 21424, 7244,  25784, 15948, 32812, 23144, 47700, 59344,
                31328, 44872, 24996, 53940, 8924,  60696, 48080, 11600, 29916,
                51744, 43616, 8680,  54368, 36108, 5704,  60820, 16996, 27664,
                23464, 50780, 15596, 37892, 19348, 53768, 63068, 15832, 50644,
                39696, 55756, 22764, 36788, 11672, 51352, 18020, 60804, 47868,
                16144, 9216,  34556, 43652, 5540,  56284, 33192, 28,    25616,
                39392, 47852, 33696, 2420,  50020, 35520, 16008, 44556, 28120,
                56472, 21332, 26300, 44360, 50112, 14164, 22048, 8576,  17524,
                65000, 51420, 33028, 46484, 13472, 61864, 34576, 21072, 2200,
                7344,  17644, 28596, 49404, 10588, 22952, 5668,  18592, 26532,
                8852,  23344, 37220, 45360, 26844, 32184, 51080, 22692, 64916,
                47156, 2276,  40808, 9264,  18744, 54908, 6776,  20772, 57756,
                4600,  17496, 60324, 32060, 2272,  35228, 18512, 30972, 41312,
                7696,  28808,
            },
            {
                34760, 9688,  43208, 27656, 22080, 59304, 336,   13708, 45520,
                51364, 35404, 12440, 42108, 50692, 63096, 3760,  41360, 8184,
                19568, 62164, 34664, 16232, 29364, 36944, 17680, 56176, 40908,
                20388, 62376, 27992, 17824, 42100, 12024, 45556, 33172, 54660,
                65460, 3996,  30672, 61596, 8764,  42104, 25056, 45444, 29064,
                18972, 65212, 30788, 48496, 59424, 28512, 39760, 5996,  23644,
                37364, 53400, 25580, 17636, 52216, 13012, 41220, 49840, 30376,
                9028,  53508, 26972, 43796, 10472, 5780,  21784, 32680, 2792,
                14452, 39192, 60436, 9696,  36812, 41280, 53356, 47476, 26428,
                38164, 10220, 22228, 504,   39528, 8856,  43248, 51480, 26276,
                40956, 47216, 61688, 4692,  39244, 53160, 34676, 13372, 58452,
                51344, 33260, 15188, 53672, 63088, 12044, 17900, 44320, 37160,
                15892, 33300, 52784, 61592, 49648, 31764, 39304, 61176, 45004,
                34020, 12612, 50932, 22312, 63900, 25564, 53740, 61820, 12864,
                47424, 58400,
            },
            {
                21380, 60556, 15400, 50528, 37224, 41692, 33832, 30448, 24860,
                3472,  59916, 28772, 47000, 19352, 57652, 29860, 14804, 55048,
                36784, 11192, 4180,  57732, 50468, 2036,  23556, 33152, 3624,
                37460, 13196, 46768, 2176,  51088, 30132, 56992, 2668,  9328,
                39316, 46648, 12284, 55932, 48468, 21552, 1484,  59680, 34964,
                9472,  52696, 2180,  14976, 20580, 45760, 34372, 13908, 43264,
                58644, 11892, 61524, 46480, 7664,  60088, 23224, 15944, 64720,
                42752, 18532, 13164, 57944, 38980, 55076, 46816, 62368, 41208,
                49456, 4876,  30428, 16556, 23840, 64024, 1416,  35284, 57560,
                29636, 42384, 54920, 60340, 27580, 19032, 57364, 30384, 10924,
                54340, 22040, 13672, 56568, 26060, 42236, 62936, 29620, 43852,
                908,   56764, 7228,  40748, 4580,  35580, 58740, 3780,  9656,
                54712, 27244, 12448, 23896, 884,   15100, 27460, 10364, 24884,
                53500, 2948,  28152, 39820, 14136, 6452,  43228, 228,   36492,
                16700, 25232,
            },
            {
                45096, 55440, 32104, 6444,  64028, 11224, 20244, 57928, 8896,
                18432, 40452, 55616, 1816,  10284, 38836, 6468,  26124, 46476,
                51212, 23320, 40048, 44288, 20592, 63416, 42408, 52300, 59008,
                7152,  25772, 55196, 34420, 64452, 24144, 19964, 14396, 42808,
                22136, 17744, 35168, 28188, 6204,  32780, 13888, 56696, 4552,
                43600, 24240, 38796, 6808,  62608, 26044, 572,   64468, 50580,
                29128, 1788,  32368, 21504, 38312, 27924, 35848, 3420,  55168,
                6072,  36896, 60672, 22520, 30228, 17736, 26684, 12328, 57716,
                19424, 35004, 52492, 57128, 5956,  32480, 19772, 12728, 6948,
                50296, 3688,  16592, 48320, 35608, 4952,  45948, 16192, 62876,
                33620, 37540, 1096,  31204, 18968, 8320,  3572,  49528, 21676,
                38800, 18068, 30204, 48568, 20864, 24964, 41716, 49912, 31228,
                62812, 19540, 46068, 37456, 43028, 64736, 55952, 5484,  19000,
                37700, 62668, 47936, 8992,  55680, 46212, 29268, 22816, 49940,
                65184, 5212,
            },
            {
                37672, 1236,  19024, 25764, 47620, 3840,  53392, 43340, 62644,
                49120, 14004, 32676, 23664, 53408, 44696, 34156, 60748, 17756,
                244,   65340, 27860, 13816, 32320, 6676,  10876, 27472, 15040,
                48852, 60928, 21760, 10672, 5136,  37668, 48776, 62696, 52836,
                26512, 60848, 136,   44120, 51968, 63956, 40200, 26456, 49176,
                16472, 61048, 32488, 42296, 52060, 10128, 54972, 21876, 8212,
                18440, 41764, 49040, 4824,  63708, 10760, 51252, 19776, 45928,
                24576, 32016, 988,   51440, 7856,  65240, 3628,  37184, 8636,
                24668, 63556, 11464, 43764, 28164, 49368, 59820, 45136, 24328,
                62932, 20820, 12320, 31200, 64536, 24384, 38492, 2684,  58544,
                6692,  43740, 50448, 64800, 45504, 58800, 37076, 16156, 9732,
                63836, 27092, 60232, 10796, 55028, 61988, 28868, 13880, 23392,
                39788, 5392,  60280, 8040,  17820, 33812, 51144, 40900, 46592,
                14608, 31168, 59132, 20100, 36280, 17052, 58124, 9784,  39884,
                31456, 11884,
            },
            {
                52068, 62988, 40416, 13600, 56420, 28720, 35824, 16136, 38308,
                27288, 5160,  61492, 36640, 16392, 64332, 21916, 11744, 43024,
                31072, 56372, 8712,  47008, 61260, 37932, 54540, 45792, 19472,
                35248, 43256, 31016, 16312, 58368, 41464, 8152,  29388, 33920,
                5944,  48136, 57400, 23720, 15428, 36636, 10224, 31068, 20228,
                54692, 11608, 27104, 57672, 18604, 35840, 40568, 31756, 57544,
                36468, 26728, 54144, 16320, 56464, 44600, 30148, 59068, 9520,
                61788, 48832, 16092, 44984, 40948, 33380, 47944, 51924, 29560,
                42728, 896,   47256, 21560, 38628, 3944,  54884, 15576, 37360,
                33388, 40780, 57860, 43736, 9392,  52212, 13728, 48976, 28764,
                18116, 25328, 9992,  14880, 22512, 52072, 32336, 25356, 54640,
                47288, 4264,  44988, 36688, 16124, 36,    7980,  46508, 57988,
                2028,  34948, 52156, 29876, 48620, 3580,  22152, 29244, 57304,
                552,   23140, 41664, 4844,  27232, 51404, 33048, 61068, 3492,
                54504, 27492,
            },
            {
                22468, 9224,  34600, 44372, 61640, 23164, 7468,  47108, 1032,
                21820, 56560, 42476, 8592,  28256, 4076,  48600, 58576, 5464,
                36048, 19916, 53092, 2824,  24760, 16408, 30432, 620,   64888,
                9492,  4212,  39272, 53192, 46064, 25912, 18112, 1932,  59444,
                13548, 37844, 9856,  41392, 19264, 55300, 2384,  45924, 63004,
                7500,  37780, 1348,  47624, 13556, 3720,  49132, 15768, 46296,
                3128,  62480, 12784, 34308, 25052, 764,   41372, 14308, 34428,
                39600, 28212, 11088, 56672, 24892, 13756, 20572, 59428, 16244,
                55332, 32644, 14812, 61324, 8488,  18364, 42032, 28816, 9564,
                53516, 6272,  27160, 1692,  22548, 55988, 32468, 41472, 21352,
                52804, 60708, 35784, 40180, 28340, 2068,  12460, 60660, 40968,
                34000, 13956, 22636, 31524, 42800, 50532, 33660, 53584, 17084,
                42912, 21660, 56976, 10608, 25084, 59620, 13236, 9204,  63976,
                35192, 50804, 11712, 44660, 63412, 1712,  12700, 20728, 42292,
                17768, 47144,
            },
            {
                59308, 29136, 49516, 2228,  17204, 32496, 51784, 12336, 64864,
                30880, 51060, 19580, 46004, 52248, 31964, 14324, 38460, 26756,
                49928, 15180, 39556, 33512, 49224, 59236, 22128, 40424, 51396,
                24640, 56804, 28312, 1076,  12656, 61744, 36164, 51256, 44952,
                21268, 31588, 64796, 27388, 6624,  48400, 59032, 23940, 42088,
                34276, 51608, 22632, 44312, 60120, 29396, 25468, 61340, 10888,
                23552, 44304, 7940,  39876, 19272, 48132, 6544,  23676, 53724,
                2376,  21068, 63536, 36152, 5364,  61000, 1640,  44388, 23196,
                6884,  40032, 26284, 53244, 34896, 24724, 65452, 340,   47420,
                60184, 17956, 51644, 37980, 60960, 18628, 7088,  63628, 11592,
                45044, 3220,  55688, 5624,  48540, 57204, 43120, 6952,  20032,
                1332,  52956, 58940, 8840,  65116, 19236, 25448, 37500, 11048,
                64284, 27316, 14972, 45336, 40220, 36196, 53780, 43896, 18268,
                26052, 7528,  54944, 16292, 38732, 24572, 48336, 35528, 56712,
                7208,  14096,
            },
            {
                43040, 5768,  20180, 54396, 10524, 38996, 57316, 25436, 43820,
                34744, 6388,  11564, 24972, 60256, 960,   41592, 54948, 9808,
                63140, 22840, 60196, 11620, 43336, 4484,  56276, 13312, 33052,
                44804, 17552, 63264, 50316, 32144, 22836, 55224, 10244, 39452,
                56332, 16924, 3056,  52912, 34568, 29812, 12264, 17180, 3656,
                28448, 14288, 64852, 31392, 6660,  20536, 55864, 5164,  51580,
                38776, 30424, 58044, 52240, 60236, 32560, 64620, 17884, 57692,
                43468, 7112,  52812, 18240, 29760, 54136, 38896, 34596, 10684,
                64048, 48516, 2524,  58108, 11772, 51040, 31856, 56756, 13492,
                23316, 30232, 45324, 15012, 34580, 46668, 26268, 712,   36608,
                29840, 16608, 32556, 62604, 20548, 34992, 16900, 64500, 29452,
                37976, 48728, 26204, 5168,  39816, 56160, 2840,  60620, 30316,
                7108,  50096, 32776, 6008,  63424, 16896, 1976,  31060, 39036,
                47856, 60332, 33312, 28564, 57916, 7924,  52992, 29732, 64260,
                25652, 33648,
            },
            {
                51172, 62512, 36800, 26628, 60412, 42140, 4936,  18992, 59668,
                15492, 40556, 62456, 37896, 17068, 57184, 23380, 18752, 30460,
                1796,  45732, 5920,  29256, 18856, 35344, 8432,  27836, 61880,
                5812,  11036, 35648, 20044, 7444,  43004, 15352, 28252, 5472,
                25120, 49788, 43104, 60068, 21748, 44512, 62340, 38348, 53812,
                58380, 19364, 40684, 10000, 53612, 37148, 41476, 33120, 18716,
                65236, 14764, 21184, 2492,  27604, 10972, 50844, 36560, 26180,
                12576, 49588, 32428, 46560, 42056, 9440,  26028, 50716, 56964,
                19484, 36716, 30588, 17432, 45468, 40800, 7276,  20144, 43376,
                35852, 3276,  64340, 5108,  11292, 50492, 39996, 58692, 54096,
                48268, 23908, 42284, 12900, 26864, 9348,  45956, 50712, 24084,
                10940, 55788, 15476, 46376, 29124, 22164, 13280, 44236, 48096,
                18908, 38536, 628,   55496, 20948, 27968, 50340, 61304, 4672,
                13996, 21612, 2732,  46312, 19332, 41224, 15164, 4404,  45212,
                38660, 652,
            },
            {
                16104, 30920, 12012, 45228, 3488,  22088, 29752, 49428, 9140,
                53572, 2696,  29424, 47832, 33224, 8180,  44472, 50904, 34616,
                57700, 41192, 26076, 55124, 64524, 52420, 46320, 20704, 38072,
                49192, 23808, 40856, 47044, 57880, 3360,  64496, 33628, 47356,
                60932, 12140, 7784,  37052, 14744, 880,   50016, 26100, 8608,
                47172, 4760,  35772, 50052, 16628, 62580, 12616, 45396, 1116,
                27076, 49420, 5724,  37420, 46352, 15580, 42684, 4004,  30528,
                62592, 37812, 23044, 2716,  14328, 64516, 17520, 4472,  29220,
                13252, 43212, 5436,  62808, 22556, 3608,  37152, 26524, 62364,
                10504, 48932, 39608, 24924, 55476, 29520, 8780,  22240, 14244,
                4424,  59724, 7628,  51428, 38828, 54872, 31448, 2992,  59208,
                42188, 18932, 33216, 60000, 10116, 51880, 35356, 4624,  58500,
                23296, 53468, 61652, 42368, 12492, 46572, 8656,  23784, 56288,
                42092, 52352, 37112, 65284, 10520, 32268, 60948, 23064, 11112,
                19860, 54116,
            },
            {
                40904, 23912, 57620, 48388, 33112, 64148, 13772, 36948, 45952,
                26824, 22672, 56200, 5044,  13536, 65132, 27808, 4712,  12504,
                47948, 16888, 9156,  37724, 14424, 1292,  41520, 59532, 16480,
                2236,  53772, 60200, 14208, 26192, 38328, 52960, 18272, 540,
                41012, 20256, 31184, 25780, 57468, 10568, 33236, 21012, 43056,
                30720, 61464, 24624, 284,   27528, 47840, 23268, 8996,  57308,
                35496, 42220, 56080, 31444, 63252, 22792, 8004,  56248, 45132,
                16372, 9752,  59272, 27508, 56004, 48944, 32792, 45304, 61860,
                24180, 54404, 49444, 27756, 14444, 59100, 47704, 55060, 15976,
                52108, 32436, 57740, 20744, 16336, 61568, 33612, 43636, 19344,
                65408, 35300, 46272, 18260, 232,   63332, 21328, 14720, 35484,
                7688,  62796, 3756,  37196, 43112, 17004, 63668, 26812, 40688,
                13968, 9556,  35656, 24832, 31472, 57924, 37972, 34304, 19104,
                10836, 30580, 6344,  24332, 54668, 1180,  47604, 36248, 59452,
                48764, 8328,
            },
            {
                63732, 2412,  18948, 8056,  15648, 52748, 56072, 1808,  32656,
                63428, 17688, 43432, 50200, 21108, 36596, 53140, 61292, 39196,
                21592, 63440, 32204, 49640, 22960, 30644, 25528, 11876, 55776,
                31600, 28444, 4288,  34116, 9000,  49552, 30180, 23524, 57252,
                35680, 63596, 51448, 46504, 39276, 54204, 64668, 5752,  15212,
                57088, 11140, 44840, 55184, 34204, 6024,  59528, 30776, 52168,
                13532, 24260, 10236, 18200, 53288, 40528, 34544, 61420, 20800,
                844,   54280, 39660, 19448, 6668,  36868, 22188, 288,   40444,
                7624,  34056, 10188, 39288, 51988, 33404, 8980,  29780, 1452,
                19400, 6784,  28012, 41960, 1200,  46128, 5448,  51148, 38600,
                27696, 10476, 30420, 57576, 24488, 44124, 6460,  40372, 53708,
                45460, 27824, 21756, 50384, 696,   30732, 54380, 8312,  33588,
                46852, 29044, 17516, 4460,  52120, 2672,  15520, 64448, 49340,
                27264, 61536, 44168, 14624, 50408, 26936, 42820, 16760, 29208,
                55908, 26308,
            },
            {
                37064, 44152, 51548, 34988, 40320, 26132, 6320,  20384, 39516,
                12136, 58660, 35092, 10200, 41284, 24768, 2160,  15692, 29800,
                6756,  52208, 3304,  42760, 60380, 5480,  47504, 34660, 7632,
                44132, 65328, 18036, 45876, 21564, 61288, 11284, 44032, 6760,
                14484, 28692, 2472,  16508, 4592,  18964, 29112, 36320, 52332,
                38992, 17772, 21600, 64304, 14132, 43096, 19020, 38188, 63672,
                4016,  48264, 61172, 7032,  26312, 1772,  14068, 28964, 51716,
                24968, 47544, 33836, 63032, 43752, 12004, 60296, 52668, 15464,
                57540, 20456, 61100, 1108,  18944, 64488, 24436, 42656, 38656,
                60684, 45124, 13176, 63292, 36892, 53908, 11920, 25380, 56716,
                1876,  41400, 53412, 14036, 34280, 60504, 29072, 49468, 25020,
                11944, 31908, 55264, 14156, 60868, 24184, 48624, 20444, 1720,
                56580, 64540, 49620, 44028, 60020, 20416, 41252, 6824,  45384,
                104,   17212, 55740, 33664, 38468, 20564, 62212, 7036,  3484,
                33376, 13608,
            },
            {
                58192, 5052,  27932, 61944, 22444, 46640, 58392, 42780, 51288,
                7636,  30056, 108,   54420, 60736, 18120, 47080, 43092, 59156,
                25508, 35828, 19740, 12204, 56360, 17336, 51892, 62916, 40224,
                22644, 12584, 52576, 39368, 55916, 1628,  16852, 37092, 48372,
                55180, 9884,  43404, 62028, 24152, 41612, 47416, 60536, 1480,
                26804, 50420, 4100,  31920, 39552, 53404, 2140,  28352, 21440,
                16040, 33012, 44272, 37856, 58476, 49700, 46616, 9356,  36312,
                41628, 13184, 6148,  29712, 51884, 25708, 8868,  30492, 47292,
                37072, 25396, 45996, 31312, 44048, 5024,  11648, 57276, 21944,
                34716, 53492, 9104,  49672, 22384, 30280, 18328, 62996, 32760,
                16148, 49324, 22020, 3576,  38208, 9432,  16648, 56640, 4536,
                64012, 17796, 47112, 41080, 7012,  38364, 11680, 61764, 42240,
                15660, 5836,  22292, 36660, 11684, 28432, 54212, 25168, 32520,
                59336, 40324, 22844, 3968,  8688,  57140, 12856, 52920, 39960,
                46196, 21524,
            },
            {
                10688, 31956, 54148, 12952, 952,   9960,  31004, 16188, 61312,
                24164, 48604, 15424, 26592, 31816, 8468,  57024, 33448, 10560,
                54880, 45708, 61664, 28484, 33268, 38496, 9400,  19952, 180,
                50104, 36452, 26900, 5468,  31772, 41952, 63896, 27376, 59848,
                32728, 21532, 52528, 34588, 58768, 7416,  13440, 22332, 9616,
                33972, 42488, 59692, 7952,  25480, 58132, 11832, 50732, 46444,
                40004, 54896, 28704, 12568, 20284, 32476, 17080, 60048, 4252,
                65448, 18412, 56856, 3048,  15752, 40820, 34960, 18864, 65060,
                3196,  12112, 56296, 15868, 54008, 35728, 50568, 14964, 48188,
                5428,  25624, 16804, 33156, 3040,  59908, 40832, 8404,  44940,
                6168,  59124, 62504, 27196, 47844, 52464, 20252, 32748, 39412,
                1376,  36668, 9384,  26104, 58184, 18868, 45168, 35468, 25004,
                31172, 39156, 54780, 8376,  33296, 46460, 62408, 14108, 9824,
                36428, 52568, 12228, 64072, 48968, 29588, 35480, 24660, 64876,
                17828, 49460,
            },
            {
                61104, 42184, 16952, 38760, 50040, 65444, 35988, 27516, 2904,
                37344, 44596, 64476, 40000, 51452, 3936,  38052, 22612, 14384,
                592,   39836, 8148,  48988, 2152,  45188, 26488, 57996, 29832,
                15568, 61612, 9984,  59064, 24228, 13832, 50264, 3704,  18920,
                25044, 4976,  38196, 11768, 27724, 49932, 32164, 55928, 45348,
                63560, 13028, 29504, 48216, 16708, 44996, 36064, 8820,  60984,
                6380,  580,   23036, 64648, 3384,  56412, 43368, 23220, 53028,
                27588, 32272, 49372, 24028, 44980, 59552, 55260, 5560,  27996,
                42796, 51164, 7180,  29152, 20964, 40636, 26800, 62300, 2072,
                30948, 64712, 39160, 57396, 46552, 27036, 14188, 52600, 24156,
                35968, 19228, 11572, 43240, 5808,  64928, 12764, 45892, 58532,
                23504, 51692, 61940, 33200, 5004,  28488, 53720, 4008,  51124,
                58620, 12684, 26884, 61528, 18556, 1356,  38944, 21792, 51012,
                4972,  19324, 28060, 46056, 41332, 1840,  18888, 43892, 9576,
                324,   29996,
            },
            {
                25516, 7372,  20624, 59332, 24888, 44264, 18664, 52636, 13368,
                56340, 21636, 5200,  19212, 12888, 62336, 48784, 27772, 64276,
                51084, 18528, 24580, 14892, 63892, 21892, 12908, 52940, 46620,
                33416, 4112,  43180, 47772, 19976, 53428, 34512, 8952,  45744,
                41564, 64820, 47748, 17996, 640,   54036, 39992, 15816, 2820,
                24244, 53128, 37832, 20700, 2928,  62228, 22608, 31436, 26560,
                18460, 58860, 35744, 50604, 40604, 6720,  29888, 38360, 11548,
                45980, 7996,  39132, 62252, 20496, 1768,  48040, 13296, 22432,
                58840, 33136, 38256, 63700, 2704,  59496, 8572,  18428, 44700,
                55620, 41832, 19968, 10764, 5020,  36532, 56068, 560,   48748,
                29412, 39756, 54900, 31812, 24300, 35416, 41536, 7992,  28224,
                15140, 42708, 20648, 55976, 49860, 14724, 65476, 9596,  17572,
                43780, 2192,  48672, 41936, 51872, 5604,  56196, 29292, 47452,
                57712, 31676, 60284, 24024, 15916, 61844, 32040, 58772, 51652,
                38012, 55720,
            },
            {
                45144, 36068, 47716, 3396,  33520, 5536,  55392, 8024,  46400,
                33040, 9664,  58172, 36192, 43564, 30244, 20372, 6132,  44924,
                34844, 30700, 58436, 54548, 41168, 36244, 59952, 3144,  41600,
                23340, 56316, 17788, 37632, 6752,  29068, 40044, 57504, 12644,
                54440, 15252, 29672, 61632, 44624, 20132, 25696, 61984, 36484,
                18672, 4696,  57564, 10620, 33860, 55364, 40384, 13864, 53832,
                42388, 47996, 15304, 10616, 25548, 54064, 14448, 63060, 24,
                21420, 58188, 14228, 34240, 10556, 37480, 31584, 53060, 39988,
                9248,  17936, 49128, 13000, 23680, 47560, 33252, 38032, 28516,
                13516, 7352,  52228, 24720, 61328, 42952, 20664, 33568, 64744,
                9084,  46864, 2864,  17076, 58212, 1612,  21844, 61532, 49292,
                5748,  30852, 12376, 3036,  36368, 41568, 30652, 23412, 34440,
                60652, 21084, 30060, 15156, 24348, 64992, 35252, 16908, 42876,
                2900,  39484, 7028,  10092, 36300, 50456, 5460,  27324, 14888,
                22456, 4428,
            },
            {
                11280, 62024, 14368, 56876, 29200, 11908, 39376, 23624, 62724,
                41356, 28728, 48108, 25036, 1068,  52708, 15856, 56140, 38664,
                3612,  11568, 43720, 4572,  27360, 7008,  31536, 18896, 8940,
                64588, 28036, 51588, 11400, 63492, 59684, 472,   22288, 30928,
                36128, 2332,  8248,  56436, 33440, 10276, 5580,  58000, 30520,
                46432, 41148, 49772, 64640, 28092, 6632,  51436, 1824,  64056,
                33488, 7496,  29464, 45736, 61936, 33844, 19316, 49124, 35596,
                42660, 54556, 4140,  50720, 27456, 17500, 61036, 25256, 45604,
                62288, 544,   27004, 57888, 41884, 4636,  55100, 10960, 60912,
                23184, 35948, 47516, 15788, 30816, 49976, 11816, 17192, 58472,
                22984, 14536, 61948, 37768, 50356, 29988, 55356, 18332, 37512,
                53544, 64196, 45408, 25164, 47884, 19632, 132,   56544, 46316,
                5236,  37644, 55336, 10508, 32512, 44908, 8240,  12268, 25796,
                62956, 15084, 53320, 44052, 55604, 21396, 12344, 46996, 41732,
                63296, 33164,
            },
            {
                49616, 27016, 52948, 22328, 42680, 63576, 49692, 31276, 19908,
                2104,  14876, 60408, 54364, 11296, 33864, 61248, 9176,  23584,
                59768, 53296, 21112, 34012, 17060, 49476, 44560, 55104, 39168,
                14352, 35640, 2040,  31972, 43844, 24608, 16540, 47252, 61240,
                51104, 26704, 39740, 23272, 48780, 37572, 42712, 50964, 11992,
                8016,  26384, 14960, 22896, 43872, 17416, 46300, 20216, 10428,
                24040, 38784, 56792, 21128, 4788,  44012, 9280,  26916, 59548,
                17000, 24896, 30172, 64264, 43076, 56112, 8068,  4684,  15540,
                35748, 21196, 44308, 30968, 52624, 15316, 20184, 45880, 50968,
                196,   57856, 27716, 3708,  63688, 7552,  40212, 28708, 43748,
                4752,  53644, 27500, 44552, 6992,  14064, 47208, 10636, 24044,
                848,   34692, 17592, 58336, 10700, 61740, 38292, 50404, 12292,
                26344, 63172, 7000,  40880, 59368, 20432, 50084, 58216, 37524,
                22296, 49232, 33776, 26564, 2448,  38972, 57432, 34364, 8064,
                54024, 16764,
            },
            {
                40504, 2212,  8356,  37624, 18076, 428,   15356, 58012, 6424,
                52064, 35540, 18216, 7148,  39012, 46168, 26660, 41604, 48524,
                13752, 28528, 64664, 9436,  52348, 61960, 11808, 25536, 60772,
                5928,  47120, 21492, 54508, 14044, 49604, 36936, 9768,  5552,
                20016, 43936, 16848, 60292, 3412,  14472, 28136, 21964, 62760,
                35160, 53916, 32612, 84,    38904, 61124, 35320, 30208, 49440,
                59980, 3916,  52172, 17236, 58632, 39648, 2516,  51528, 31624,
                6184,  40120, 9740,  46368, 1080,  21968, 39440, 48680, 29096,
                56868, 51676, 10772, 6972,  36672, 25524, 62856, 29972, 6212,
                39644, 17932, 43440, 53360, 34228, 22144, 55596, 2356,  50960,
                36212, 32552, 19800, 9968,  24712, 60880, 40208, 31896, 44344,
                59468, 8756,  40348, 29468, 52428, 7200,  27700, 42916, 16388,
                32772, 51392, 18764, 47432, 564,   27352, 4808,  31488, 54060,
                1224,  10788, 18704, 65512, 30724, 17136, 61508, 24128, 1232,
                29176, 20188,
            },
            {
                64840, 31888, 44516, 60400, 34340, 46764, 25620, 37036, 44688,
                61676, 24064, 43132, 30832, 65220, 21812, 3160,  17292, 31912,
                36480, 972,   46924, 38860, 23360, 1812,  37088, 20768, 30792,
                50944, 58680, 26544, 40256, 4020,  56788, 28388, 42016, 64464,
                33008, 58032, 11484, 52864, 30884, 65376, 54596, 1380,  17240,
                47688, 5296,  60352, 56624, 12932, 8848,  24836, 55808, 15056,
                43672, 28192, 12940, 36544, 30888, 24432, 64868, 45896, 13044,
                53180, 60588, 20248, 14616, 36060, 52284, 12172, 63516, 33824,
                3664,  40364, 64676, 17564, 60160, 1656,  41308, 34796, 13904,
                65332, 32012, 10192, 59164, 14296, 37920, 45544, 60992, 25220,
                11376, 63480, 41688, 57820, 52660, 34940, 5344,  63388, 15996,
                27052, 49012, 21336, 3616,  34008, 14332, 22124, 64076, 2296,
                58384, 23244, 11588, 35612, 52844, 39108, 63772, 17780, 44428,
                28856, 41632, 58552, 46012, 5192,  51680, 9840,  44340, 48844,
                37208, 57032,
            },
            {
                24684, 15564, 50848, 10644, 56432, 5300,  53716, 9928,  28048,
                12572, 49416, 3984,  56208, 13596, 50312, 58636, 6348,  63108,
                51412, 19252, 42020, 15208, 57660, 28304, 48064, 4276,  15772,
                41804, 10484, 17980, 61052, 8348,  33424, 18872, 52592, 13412,
                24784, 4084,  46528, 35760, 7328,  19072, 45140, 39656, 58880,
                24520, 43276, 19176, 28932, 48012, 52704, 2476,  37640, 6060,
                62492, 22812, 704,   48724, 54180, 10824, 16096, 21920, 37852,
                3452,  33304, 48116, 57584, 26608, 31176, 59172, 24256, 19160,
                13980, 46992, 23252, 32240, 48048, 55820, 9092,  22600, 53976,
                48212, 25264, 20640, 47076, 1516,  26572, 9628,  19156, 15340,
                39588, 6224,  46076, 120,   17188, 28864, 21436, 2884,  54352,
                36392, 12880, 56096, 42496, 59740, 46180, 54740, 35820, 9172,
                39512, 44848, 28576, 60612, 14688, 23960, 9344,  48392, 13732,
                61416, 35296, 8008,  23256, 13628, 40892, 27632, 19140, 59828,
                12992, 4000,
            },
            {
                47388, 6572,  28496, 19664, 23448, 30548, 39980, 64520, 20752,
                32688, 40500, 9020,  27484, 19356, 37096, 43768, 28080, 54988,
                8272,  24984, 60796, 31408, 7104,  43372, 63620, 53808, 34464,
                45416, 736,   29896, 36308, 46688, 63116, 22580, 1352,  38488,
                55648, 29340, 41128, 21624, 49852, 25592, 34456, 9148,  31364,
                13828, 37132, 10336, 40508, 21832, 32044, 65224, 42024, 19408,
                33956, 46700, 40196, 59408, 6616,  34656, 42144, 56612, 28116,
                62420, 24132, 41424, 7212,  17724, 2816,  44780, 9504,  42608,
                55188, 27320, 2508,  38372, 12788, 28632, 45016, 16636, 59760,
                2880,  37296, 6840,  41488, 30220, 63016, 49212, 31944, 57360,
                54156, 22200, 30592, 13456, 38272, 47784, 57012, 41828, 50248,
                6708,  31140, 64860, 16840, 24912, 5292,  30292, 18612, 48260,
                53736, 3508,  31916, 6108,  55644, 43260, 33320, 21496, 56932,
                3812,  25332, 50536, 56040, 36676, 62568, 33332, 54976, 7128,
                42560, 34804,
            },
            {
                61308, 55348, 38260, 62628, 13776, 48220, 3212,  17364, 57228,
                1228,  52984, 59640, 46804, 33536, 2280,  11120, 22652, 34172,
                45312, 12036, 4040,  54620, 35240, 12772, 18572, 8340,  56960,
                22400, 65428, 52188, 24236, 12676, 42744, 6748,  59004, 48324,
                15672, 8876,  63652, 532,   61108, 12780, 56364, 4388,  52628,
                63800, 3088,  50840, 61920, 6324,  17012, 45648, 27080, 11540,
                57568, 8908,  29528, 18728, 25952, 61868, 1472,  47628, 7864,
                18296, 11428, 51132, 64788, 38420, 54320, 49736, 5100,  37392,
                60644, 7432,  53084, 58992, 19260, 50820, 4944,  26548, 42456,
                11416, 50260, 61952, 55796, 16988, 52456, 5568,  35380, 3180,
                47488, 26700, 59752, 51916, 65432, 8132,  25852, 11232, 18804,
                23480, 46596, 1912,  38428, 50788, 11532, 41088, 60944, 25488,
                13940, 21204, 64572, 17108, 46892, 1972,  62460, 36844, 6856,
                40300, 15676, 31984, 19668, 124,   47788, 4436,  15432, 31160,
                52048, 21924,
            },
            {
                17776, 43644, 2016,  34032, 41840, 59200, 51340, 35788, 46052,
                14260, 37772, 22092, 16304, 62400, 52692, 57612, 40740, 15556,
                59644, 38252, 27248, 47284, 22032, 51064, 40704, 32232, 26908,
                13356, 38132, 5284,  55584, 16524, 51100, 30988, 27172, 44756,
                34408, 50700, 17944, 37840, 27984, 43412, 16596, 48280, 28668,
                20312, 46000, 26128, 55204, 35936, 13592, 58716, 3772,  54772,
                50728, 15500, 64164, 43828, 13268, 50220, 20268, 30300, 53888,
                35756, 45160, 596,   28684, 12920, 33924, 20756, 62156, 29580,
                32992, 16080, 44356, 10112, 35164, 63932, 39496, 57176, 33400,
                19100, 28232, 34880, 23332, 12432, 39468, 20352, 43472, 64084,
                8948,  17804, 34084, 4068,  20636, 43800, 32156, 59088, 39240,
                61264, 53388, 9580,  28828, 20396, 62836, 1296,  33360, 6576,
                57008, 42216, 50032, 38392, 25876, 30144, 11516, 50912, 27856,
                53676, 46276, 64104, 10132, 43324, 25724, 21428, 38844, 64132,
                26568, 9340,
            },
            {
                30100, 12824, 52664, 5632,  26784, 9624,  21688, 7284,  29356,
                24840, 63996, 5388,  42604, 25660, 7464,  30032, 48548, 1384,
                20652, 50516, 64944, 16844, 152,   61560, 24556, 3020,  59600,
                47896, 19304, 40972, 32888, 2288,  60208, 37256, 10576, 20608,
                4776,  59716, 23856, 53624, 10512, 58600, 40292, 22756, 35444,
                57104, 15932, 32668, 1532,  23168, 49608, 29376, 39688, 21208,
                25460, 37276, 2496,  52560, 32964, 38212, 10084, 60032, 40080,
                15136, 57028, 32136, 22424, 59712, 41756, 15248, 25184, 10980,
                47740, 21156, 40788, 24496, 30856, 1188,  21712, 8344,  52104,
                60920, 15152, 1004,  45180, 8280,  57964, 24816, 28852, 13964,
                37532, 42312, 50176, 11476, 36772, 54716, 15360, 1608,  27792,
                33712, 13824, 43416, 36176, 56392, 45452, 15952, 52292, 37608,
                27788, 10820, 34212, 8044,  59384, 20288, 41052, 17352, 60468,
                22280, 2572,  38156, 28792, 60056, 52764, 57932, 10668, 49276,
                1040,  40628,
            },
            {
                58656, 46856, 24360, 65344, 45148, 15780, 32808, 61064, 41256,
                54892, 10280, 31792, 50724, 12744, 38768, 17880, 63796, 6208,
                32620, 42320, 9572,  30452, 37588, 56012, 10140, 44420, 35412,
                6964,  61852, 28396, 45900, 8964,  25208, 18212, 53920, 62380,
                41340, 13228, 30576, 45964, 6792,  32736, 2388,  62524, 6532,
                11656, 41380, 8440,  60072, 42996, 10088, 63632, 33428, 7844,
                48024, 31804, 6352,  22508, 58316, 4456,  45592, 23540, 2912,
                26324, 63220, 5216,  49060, 8696,  46632, 3120,  56968, 50940,
                776,   65180, 56184, 5572,  49436, 61244, 14612, 45496, 30364,
                4640,  38692, 53864, 64612, 31704, 47796, 2348,  60896, 53472,
                22788, 1048,  62472, 29160, 23916, 5852,  48708, 62984, 45264,
                4744,  17648, 58344, 25632, 7404,  31988, 23672, 59436, 47528,
                17928, 62188, 868,   55268, 13632, 53064, 3176,  44684, 8676,
                34024, 12444, 55676, 17960, 7272,  14224, 34700, 45060, 18500,
                60676, 36208,
            },
            {
                15372, 7472,  31960, 19872, 57180, 39228, 54044, 47768, 2128,
                18228, 44728, 35392, 376,   56592, 46264, 23024, 35732, 54272,
                13156, 24460, 57648, 5424,  45960, 14252, 28936, 53372, 17048,
                50564, 11276, 56428, 21556, 63968, 49248, 40104, 3228,  32932,
                26440, 56424, 1844,  36040, 49140, 19796, 25948, 51400, 38000,
                44948, 64580, 27964, 48360, 37176, 19512, 52252, 400,   44748,
                17972, 59780, 54252, 42304, 16588, 27448, 65300, 13856, 51500,
                43488, 19808, 37400, 16680, 55880, 27660, 63092, 39300, 18064,
                36120, 26960, 13376, 45812, 17728, 37428, 54568, 25312, 10860,
                48504, 42908, 19720, 26000, 36980, 16424, 50768, 10420, 45612,
                31232, 56300, 16048, 44652, 58156, 39944, 19892, 35624, 9124,
                52548, 22100, 63812, 2676,  49352, 12944, 42596, 9444,  4320,
                21888, 44904, 31116, 24000, 48228, 35324, 65164, 31684, 56704,
                47940, 24676, 42332, 49576, 31580, 40520, 3748,  29600, 23876,
                4948,  54176,
            },
            {
                21456, 49096, 37416, 240,   11968, 28468, 4512,  24052, 13424,
                62244, 27048, 59072, 20436, 29236, 62004, 9144,  44240, 59160,
                28420, 52516, 18256, 34496, 49780, 19988, 64156, 39312, 22196,
                30808, 3792,  15404, 43708, 35112, 12848, 57812, 46304, 15232,
                8172,  43604, 21800, 63484, 15712, 60040, 54300, 13812, 30092,
                3864,  22160, 17132, 54216, 4772,  25040, 15148, 55948, 61572,
                13448, 39204, 28560, 11496, 49052, 35832, 54584, 31236, 6312,
                34208, 10848, 52728, 40616, 23640, 35264, 6172,  30896, 43696,
                54072, 4244,  34148, 59544, 28760, 7392,  42008, 34528, 63360,
                22748, 56988, 7292,  11872, 59792, 4888,  40672, 34296, 18848,
                5628,  38812, 26280, 8716,  51696, 13152, 30640, 55736, 25128,
                41420, 47584, 30216, 39880, 54516, 19384, 65316, 35576, 29764,
                50748, 57556, 39844, 16564, 42848, 5820,  21732, 26768, 14816,
                4632,  36052, 62632, 1428,  22904, 65056, 51552, 56680, 12340,
                42696, 27736,
            },
            {
                9920,  62060, 55752, 42900, 50268, 63448, 19128, 37144, 51544,
                8036,  39548, 48008, 15076, 52328, 3340,  26272, 16004, 4304,
                38424, 45252, 1860,  59756, 41500, 25768, 6516,  1736,  58076,
                42636, 36684, 53020, 26752, 304,   29868, 6500,  23740, 65172,
                49704, 52796, 38920, 11304, 28156, 40720, 8912,  46844, 18560,
                57772, 49684, 34220, 12844, 31292, 46796, 41048, 27576, 35604,
                9368,  23716, 63768, 2008,  20560, 8300,  40988, 18100, 47912,
                57764, 28436, 60764, 1336,  12732, 45716, 20460, 9848,  14420,
                61468, 22684, 40100, 11084, 52552, 20812, 3896,  58596, 17448,
                2924,  28776, 33036, 52444, 44204, 21584, 27624, 54688, 65032,
                12452, 49204, 59396, 3496,  33016, 64424, 436,   16784, 59664,
                11972, 37080, 6336,  15228, 34140, 27704, 32,    53444, 38772,
                14940, 11380, 3408,  63520, 28752, 10096, 50548, 39472, 61360,
                19204, 52320, 28096, 15828, 45860, 9220,  19768, 37612, 48180,
                63564, 33580,
            },
            {
                41120, 3252,  17016, 25788, 34788, 9196,  46284, 30336, 57512,
                33700, 22668, 4796,  11144, 41748, 37596, 33072, 50576, 21316,
                11080, 61956, 23212, 8936,  31196, 12348, 60604, 33184, 48032,
                24932, 63320, 7744,  60524, 17544, 51912, 38712, 19900, 36236,
                28656, 17784, 3876,  57480, 34396, 44184, 204,   32072, 24452,
                61224, 39620, 1196,  56524, 62888, 7440,  58916, 21652, 4548,
                43840, 51356, 30728, 46120, 56576, 60792, 24696, 20,    62740,
                21808, 7832,  44364, 31348, 64892, 54788, 51488, 59076, 24852,
                47100, 8100,  50696, 31144, 63872, 26152, 47916, 13524, 37956,
                51244, 40860, 14600, 63692, 260,   30800, 58068, 7704,  24588,
                43116, 20968, 35100, 17568, 46920, 23108, 43080, 49512, 28648,
                3800,  20304, 51296, 60776, 10328, 57156, 46780, 24368, 6872,
                62564, 26556, 52780, 36432, 19652, 59840, 55056, 1028,  43316,
                10792, 57452, 5860,  39068, 60464, 34088, 27028, 1996,  16552,
                6476,  52472,
            },
            {
                13992, 29168, 22064, 60884, 5948,  15344, 59880, 42148, 2576,
                16800, 64656, 54192, 30692, 60996, 18800, 55144, 65288, 43124,
                30256, 48432, 15336, 36296, 51180, 54484, 43540, 16544, 9136,
                13384, 19928, 40060, 47096, 32576, 56168, 43408, 59196, 1420,
                10384, 32400, 61500, 24920, 5964,  21512, 64956, 55316, 36232,
                6740,  10360, 26964, 42764, 19052, 37752, 11348, 49804, 32848,
                55480, 18660, 7040,  37520, 14464, 33264, 44208, 12476, 39428,
                35964, 15744, 50228, 25104, 18816, 3316,  33912, 38500, 29180,
                1900,  42068, 19748, 15688, 396,   44280, 56020, 31796, 8308,
                60476, 46524, 24512, 35956, 49304, 17868, 13528, 46424, 37228,
                1340,  62220, 52928, 41036, 28052, 10956, 61120, 7448,  33420,
                54612, 63268, 25940, 45328, 22572, 40960, 17088, 59728, 43308,
                20084, 32360, 46972, 7748,  44604, 31452, 13136, 37100, 23944,
                46760, 30236, 20856, 50144, 12076, 53236, 43664, 57720, 31132,
                24744, 46152,
            },
            {
                35896, 54012, 38848, 44820, 31376, 51876, 20936, 26056, 12388,
                49828, 44004, 35980, 25252, 45820, 7064,  13032, 744,   35180,
                7572,  56348, 40040, 63120, 5204,  21472, 27668, 38464, 50192,
                57200, 29992, 2708,  23136, 11916, 8604,  25936, 14272, 47820,
                54804, 41368, 45808, 13616, 51900, 47764, 17384, 11952, 42168,
                52756, 15440, 45836, 22796, 51564, 3660,  29436, 63964, 16296,
                1312,  62092, 41248, 26120, 52672, 4232,  29688, 50664, 54776,
                27024, 4596,  56848, 39164, 10212, 42736, 15808, 5544,  49336,
                63552, 32544, 54412, 58956, 35780, 10316, 39860, 22236, 27680,
                1820,  19124, 9600,  56156, 6372,  39116, 61428, 51572, 32132,
                15468, 29756, 9736,  4780,  55380, 36604, 19040, 39188, 46244,
                12988, 42364, 1760,  36084, 7944,  31928, 3932,  12424, 34720,
                49388, 1704,  58584, 15480, 25308, 4524,  49208, 17256, 33856,
                64420, 8092,  41800, 32840, 24992, 4736,  14540, 62284, 39616,
                18984, 58520,
            },
            {
                64976, 4872,  48520, 12192, 57376, 1328,  39868, 54832, 6592,
                29092, 19864, 9376,  2308,  57976, 22828, 48756, 27156, 58804,
                19396, 25132, 2832,  28608, 17912, 46516, 720,   65188, 34540,
                6056,  44492, 54388, 36372, 50036, 64228, 4920,  34356, 60852,
                22992, 7144,  19700, 37288, 56136, 30016, 39020, 2936,  27544,
                20588, 64064, 30764, 59864, 34972, 14656, 46348, 40584, 24104,
                36144, 47804, 10412, 21596, 58100, 17460, 64380, 9716,  19856,
                46268, 60096, 13304, 32884, 48808, 26924, 61772, 21852, 56924,
                17580, 11756, 37576, 24628, 5376,  49956, 17684, 65456, 53176,
                43744, 34772, 62200, 22112, 42228, 26828, 10528, 22604, 5532,
                56724, 47600, 25260, 60264, 14204, 50792, 2248,  57348, 23828,
                16032, 31052, 58460, 18388, 48880, 64388, 51752, 29312, 55492,
                9484,  40396, 22596, 53848, 38588, 61716, 56780, 28168, 3296,
                53024, 15036, 59796, 520,   56212, 37232, 21860, 8252,  49592,
                10996, 856,
            },
            {
                32028, 27112, 8388,  19388, 24692, 36160, 64248, 33116, 45972,
                61816, 56312, 41076, 15716, 51668, 38516, 32344, 16616, 41648,
                52652, 12692, 45552, 33472, 55712, 59148, 11268, 23496, 52452,
                18016, 26212, 62428, 15300, 41204, 20952, 29308, 39088, 16688,
                51508, 31084, 63680, 1696,  26504, 9292,  62232, 50872, 58592,
                34708, 48412, 2100,  8664,  54344, 25828, 58116, 8120,  52604,
                12708, 27852, 60132, 2892,  34892, 47068, 23688, 6216,  33968,
                42124, 1792,  22888, 63912, 6860,  55592, 46392, 36268, 8704,
                27800, 44616, 2652,  46904, 29104, 60260, 33544, 6140,  15052,
                58280, 11936, 29228, 47676, 3100,  54564, 34260, 44824, 64060,
                40132, 18524, 34632, 43460, 21536, 31800, 26376, 65124, 9252,
                52868, 38504, 5732,  27912, 54928, 14544, 37600, 23436, 61696,
                17752, 28100, 65016, 12080, 33100, 20224, 9592,  45240, 21716,
                38908, 48376, 27120, 18892, 62872, 47148, 29680, 54836, 34432,
                44088, 21140,
            },
            {
                13588, 56452, 40644, 61908, 50168, 4156,  14864, 9640,  22920,
                38220, 4300,  31976, 26652, 63704, 10324, 44404, 61392, 4576,
                37272, 64324, 50616, 9704,  41560, 36740, 14840, 31852, 40992,
                3356,  59700, 10104, 31508, 1024,  58204, 53048, 44736, 3464,
                27520, 11812, 57240, 43536, 16236, 33640, 23068, 13840, 44380,
                6088,  17860, 38552, 12964, 41444, 18648, 2644,  32160, 20516,
                56844, 44476, 31148, 49644, 13404, 42672, 38004, 56384, 61576,
                16748, 28672, 37336, 52832, 19620, 30660, 688,   13612, 40412,
                51972, 23204, 55800, 62584, 20740, 12540, 42572, 25360, 49016,
                37816, 4384,  51812, 32696, 14456, 60004, 19880, 1752,  28332,
                12040, 53776, 2976,  7988,  62712, 48244, 41196, 5144,  35408,
                20656, 49836, 60828, 11252, 43428, 20896, 1928,  41700, 5072,
                46020, 36768, 5968,  43632, 368,   51984, 40964, 62780, 6996,
                58088, 12332, 35464, 44464, 10268, 16172, 41132, 2636,  26644,
                59656, 51600,
            },
            {
                23648, 46448, 33716, 16460, 29864, 42436, 47316, 28228, 52776,
                17608, 13520, 59108, 47540, 20804, 1540,  28740, 55576, 8620,
                23588, 30620, 16056, 21216, 5900,  26620, 49004, 62136, 7796,
                47260, 35548, 22024, 43556, 48720, 18396, 12664, 63040, 8416,
                47360, 39936, 35008, 4956,  49696, 60456, 46260, 3808,  31280,
                25492, 57044, 62800, 28016, 45116, 61668, 50508, 38100, 65492,
                5756,  16940, 39488, 7668,  62704, 27276, 928,   30716, 12056,
                51616, 48184, 9468,  44036, 15296, 41480, 24820, 58780, 33248,
                64988, 6832,  14924, 39224, 8480,  51636, 57416, 944,   30332,
                41184, 20932, 16916, 64548, 8188,  38476, 50676, 16184, 58896,
                23796, 49760, 38200, 57656, 29164, 11300, 16936, 55716, 44128,
                29912, 536,   46548, 24076, 32984, 8644,  47616, 58816, 26248,
                13324, 51224, 57220, 30004, 47812, 26360, 13284, 34672, 30352,
                24532, 2312,  54512, 31008, 6340,  51944, 23848, 64168, 5788,
                16912, 38040,
            },
            {
                61132, 1780,  6440,  52892, 11024, 57908, 19932, 61332, 280,
                34712, 50636, 42256, 6924,  36476, 53728, 14280, 19712, 34516,
                47596, 992,   57144, 43864, 61076, 2260,  53892, 19268, 29344,
                56380, 13652, 51240, 27820, 7484,  33944, 25100, 36704, 21708,
                58732, 17904, 24596, 53588, 21340, 7504,  36888, 55504, 40540,
                9792,  22120, 50156, 33760, 6496,  22872, 11220, 28900, 42724,
                348,   25008, 51632, 21152, 55660, 16132, 53008, 20732, 40716,
                25188, 3544,  57264, 34688, 4740,  60720, 10628, 50460, 20204,
                3520,  43016, 34828, 30688, 18072, 45660, 36220, 22976, 10496,
                61980, 56000, 27140, 44212, 24212, 46464, 30052, 36496, 42956,
                6672,  31384, 15168, 20200, 45320, 1596,  33324, 61484, 12472,
                25896, 63648, 14768, 40548, 55972, 63152, 29652, 16768, 53896,
                33888, 21680, 8556,  18564, 59380, 15844, 3676,  55036, 50428,
                16776, 41540, 65508, 21384, 38332, 58972, 32756, 13024, 48020,
                42784, 9068,
            },
            {
                54476, 28384, 44636, 63936, 22740, 37212, 7072,  55824, 44164,
                23956, 65064, 10988, 30368, 24308, 62784, 40096, 49484, 59936,
                11860, 38924, 27356, 51832, 32764, 23132, 37940, 10228, 44832,
                24908, 39356, 4804,  64908, 57136, 40124, 2392,  54264, 31608,
                50784, 344,   65336, 13084, 41896, 29032, 18708, 64188, 14968,
                51928, 43392, 684,   16516, 55860, 36744, 59276, 15288, 48440,
                54736, 35908, 60760, 9996,  32880, 5064,  43960, 65100, 7508,
                59484, 31848, 21400, 63156, 27764, 53536, 37136, 46936, 16932,
                28880, 60916, 48312, 53632, 3904,  27452, 64212, 16248, 53432,
                46164, 7196,  34400, 1240,  58432, 11664, 4336,  63200, 9924,
                55744, 46848, 64808, 25740, 53104, 40232, 23736, 50476, 19612,
                54336, 7596,  37000, 18852, 2800,  35668, 50452, 10732, 39504,
                1396,  63432, 42380, 35388, 23752, 64328, 37488, 44772, 19524,
                61160, 9004,  46336, 26236, 3820,  48908, 19300, 55844, 36396,
                30596, 20196,
            },
            {
                12380, 35208, 18452, 39420, 2480,  32504, 26288, 12704, 39900,
                28888, 3136,  55432, 17924, 45476, 3796,  32664, 5984,  25928,
                43160, 63164, 18436, 7576,  13020, 46696, 64592, 16180, 58356,
                76,    20240, 53504, 16964, 11344, 45884, 62132, 14716, 42648,
                10168, 28260, 45696, 32584, 59616, 10492, 47708, 1636,  26692,
                35864, 59964, 12220, 31620, 46380, 3776,  26112, 8512,  33624,
                19680, 12632, 28180, 46544, 38548, 24528, 48716, 34924, 14836,
                46136, 38936, 17584, 11488, 45600, 22964, 2600,  31012, 57100,
                9476,  25176, 12828, 22348, 60084, 7364,  39916, 33108, 3156,
                19452, 37380, 13552, 41524, 54356, 21124, 52360, 26176, 18304,
                34952, 80,    12652, 36116, 5736,  58568, 8988,  37936, 3352,
                42152, 31868, 52028, 58640, 27372, 6648,  43976, 24508, 59972,
                30784, 15096, 49020, 3116,  53004, 40612, 10388, 27432, 5056,
                33128, 36708, 14944, 53416, 11976, 42432, 8484,  27652, 72,
                63056, 50624,
            },
            {
                40772, 59052, 25908, 14436, 47672, 60768, 51936, 16672, 49656,
                7936,  20668, 48300, 38532, 12912, 52232, 57776, 16376, 21776,
                54528, 3560,  35492, 59000, 40188, 28768, 6300,  34252, 41668,
                30540, 60544, 37056, 32752, 22500, 29716, 49408, 19556, 4848,
                60716, 35900, 15960, 6152,  38580, 56800, 22860, 33548, 53904,
                6904,  19320, 39056, 64776, 20908, 49796, 40588, 62824, 45192,
                58668, 6136,  41776, 63684, 2164,  57912, 18900, 10656, 27644,
                1528,  55760, 49892, 5896,  40340, 16068, 64408, 7124,  44200,
                39372, 54700, 224,   36972, 42136, 56228, 11748, 48676, 26472,
                57872, 51152, 30504, 61600, 17556, 33368, 39976, 48084, 59624,
                21996, 41852, 28592, 61220, 18820, 48984, 14920, 27840, 56896,
                47376, 17144, 10572, 45292, 22176, 13380, 61800, 19708, 4820,
                46308, 55640, 27116, 12256, 32024, 7264,  56216, 21956, 59080,
                49556, 728,   57284, 29456, 62616, 34724, 60364, 45524, 15784,
                24716, 4376,
            },
            {
                32100, 7920,  49740, 55628, 10136, 41620, 5516,  35096, 63288,
                31296, 58208, 34348, 62292, 27176, 9208,  42416, 29280, 46556,
                9880,  31668, 15132, 49424, 20488, 55624, 2916,  52732, 49868,
                12276, 7336,  48160, 43360, 3436,  8244,  26260, 57472, 38756,
                23284, 55612, 49768, 26328, 3168,  51156, 16644, 45020, 62432,
                29684, 48564, 25140, 9052,  54284, 30188, 17812, 1580,  23488,
                31388, 50860, 17484, 22588, 13760, 50392, 31224, 61348, 53352,
                36080, 23836, 60564, 29584, 54468, 33668, 48920, 13108, 21500,
                33144, 62688, 18416, 50080, 31512, 15640, 21264, 62636, 42840,
                9188,  24416, 5040,  48460, 8384,  28240, 2096,  14284, 5816,
                32176, 54124, 50556, 10292, 43608, 30996, 64252, 34380, 21908,
                59960, 5324,  29384, 65480, 49668, 32604, 54840, 41556, 36256,
                9284,  22880, 37792, 62520, 45048, 17508, 29568, 47264, 41848,
                11412, 24764, 20836, 39824, 17676, 2408,  22464, 52800, 38144,
                58224, 46908,
            },
            {
                17348, 65252, 3392,  23404, 30708, 19276, 57524, 22096, 44880,
                10584, 15488, 42920, 1920,  22272, 60300, 35976, 1204,  65388,
                38436, 52416, 25160, 44608, 10664, 62216, 26092, 17792, 22856,
                63868, 27188, 15256, 56416, 61424, 52852, 35136, 11896, 30912,
                47104, 7708,  42224, 20092, 61680, 30712, 40984, 8204,  13696,
                2460,  41824, 59060, 4164,  43704, 13460, 57464, 52648, 11556,
                38680, 3504,  56736, 29676, 36600, 8228,  40352, 4176,  43300,
                19732, 8504,  13580, 43012, 568,   25072, 36760, 59696, 52084,
                27532, 5596,  45748, 9832,  25804, 52200, 1588,  29980, 35460,
                14184, 60268, 38444, 22248, 65268, 43048, 55520, 37084, 62168,
                45700, 16140, 3292,  24544, 38764, 4728,  46072, 792,   11964,
                40660, 35740, 24380, 3668,  38808, 16364, 296,   27648, 52112,
                16872, 58564, 6268,  19744, 51068, 60336, 2464,  14276, 63100,
                31520, 52616, 44540, 5908,  50224, 31836, 13660, 41436, 6292,
                10916, 21568,
            },
            {
                42940, 28140, 37704, 45512, 62008, 1064,  27436, 38076, 4080,
                54268, 25988, 6032,  51456, 31700, 17324, 49732, 13220, 56912,
                19004, 6580,  61388, 872,   30484, 37004, 42880, 8892,  46292,
                39804, 35400, 1724,  24284, 40576, 18384, 44852, 64644, 1144,
                17328, 62752, 13192, 53096, 35212, 11792, 24960, 58228, 37468,
                22316, 56036, 16332, 35140, 27796, 38068, 5664,  34232, 27220,
                64136, 43448, 9692,  53928, 44892, 63064, 25300, 16020, 57244,
                48072, 32248, 38092, 62596, 20300, 58140, 9308,  17816, 2020,
                40676, 14668, 57604, 35588, 65168, 44720, 6316,  59036, 47336,
                18044, 53756, 412,   45816, 15656, 11364, 51056, 23776, 8076,
                27328, 40404, 57208, 21040, 60092, 54588, 17968, 26196, 53380,
                62056, 14480, 55884, 43280, 9452,  58408, 47532, 11880, 64268,
                31304, 49896, 42936, 812,   34048, 24376, 39340, 35500, 7692,
                18380, 37836, 64608, 9900,  59240, 47176, 25156, 64160, 29824,
                34384, 53876,
            },
            {
                60508, 12148, 51332, 15320, 34344, 43388, 53120, 13648, 47200,
                60608, 40244, 19372, 46192, 55728, 7316,  40976, 23932, 33484,
                28084, 48328, 40356, 22344, 57968, 14372, 53984, 32404, 4132,
                59224, 19380, 51108, 10044, 30296, 13436, 4616,  51020, 41268,
                28076, 33564, 22516, 5320,  44592, 632,   64836, 19572, 47752,
                52308, 32052, 11448, 63696, 51740, 22060, 47328, 59316, 20096,
                48760, 14496, 24936, 33004, 480,   20604, 52312, 29060, 11960,
                64532, 3652,  26596, 51232, 6620,  46280, 28996, 42600, 64008,
                24092, 48044, 29284, 20320, 6956,  38608, 24060, 54796, 4044,
                28000, 41788, 32148, 26160, 56956, 34756, 29968, 58688, 20164,
                12356, 64484, 35040, 7436,  29404, 13944, 36056, 50388, 8692,
                31564, 48512, 18928, 52164, 26012, 36912, 21228, 34336, 3860,
                24732, 39668, 13016, 28500, 56740, 10808, 53488, 48816, 26764,
                55936, 3192,  15412, 27396, 35196, 54532, 4652,  18996, 57216,
                49196, 1140,
            },
            {
                36632, 6464,  20520, 54868, 7260,  11420, 64232, 18288, 30268,
                23564, 35556, 64828, 11372, 37496, 26460, 62708, 45652, 4024,
                11256, 55360, 16828, 34808, 5744,  47024, 19956, 62908, 11760,
                29076, 55852, 45080, 63144, 38284, 59128, 25656, 21284, 54768,
                10016, 58996, 39764, 56884, 49000, 29012, 53696, 34136, 4028,
                27384, 5936,  45232, 19152, 88,    60744, 9512,  15956, 40480,
                6640,  35924, 61168, 18124, 58468, 47184, 6816,  34608, 39448,
                22680, 44628, 17504, 56508, 14584, 32704, 49728, 12016, 55420,
                34272, 4032,  61660, 12404, 55960, 17428, 33292, 40496, 12244,
                21044, 63884, 10432, 49812, 19404, 5312,  2648,  41364, 47680,
                31432, 940,   52476, 48344, 10596, 42620, 63656, 20760, 44324,
                6164,  23264, 1492,  63192, 30404, 6764,  61088, 44812, 56064,
                8292,  59748, 47340, 22068, 65204, 44228, 16624, 4104,  61548,
                22524, 46084, 30740, 42904, 21036, 12392, 37204, 43476, 8880,
                15984, 26080,
            },
            {
                46724, 32484, 58540, 40304, 25412, 48864, 32252, 3364,  50832,
                9200,  584,   28892, 57892, 14916, 2436,  53384, 20024, 58484,
                36720, 43232, 8524,  63636, 51792, 27552, 2552,  38716, 50024,
                16692, 6820,  33492, 21704, 6488,  48704, 32260, 15448, 35536,
                48028, 2488,  25372, 14548, 37252, 17672, 9272,  42428, 15224,
                62088, 39436, 24388, 49844, 41404, 31500, 25968, 45724, 1880,
                53684, 28660, 50088, 4748,  37436, 10912, 42688, 59772, 1956,
                49288, 54244, 10736, 35384, 40464, 22784, 4992,  37692, 19780,
                8616,  51576, 39152, 43288, 31092, 49568, 8828,  62880, 51284,
                45296, 36004, 6888,  60844, 38028, 44396, 62864, 15908, 53912,
                38416, 18768, 43824, 25172, 33140, 56620, 2528,  38160, 28248,
                57748, 39852, 33892, 12520, 41396, 17616, 50484, 14268, 29016,
                19340, 35944, 15700, 4792,  31724, 8392,  20688, 42272, 33284,
                12092, 51000, 60136, 7960,  56252, 2108,  52184, 27752, 62108,
                40072, 22532,
            },
            {
                63844, 4508,  17096, 29736, 61148, 22388, 36460, 41872, 59448,
                54960, 17104, 49244, 22648, 43272, 34404, 9548,  30388, 15724,
                23396, 51076, 31380, 25324, 12200, 41264, 55508, 23532, 34980,
                43128, 26372, 14200, 54372, 388,   42076, 56516, 62264, 7900,
                44064, 19444, 64100, 32128, 6876,  60212, 23308, 56272, 46344,
                10408, 58464, 33720, 7892,  14232, 55384, 37264, 64844, 32596,
                56940, 21172, 12316, 41172, 23324, 55132, 26952, 14524, 19416,
                30932, 7792,  28264, 63548, 1476,  59568, 53344, 62356, 27128,
                45636, 15584, 22944, 1120,  26872, 60876, 14904, 23088, 29564,
                1676,  16948, 54076, 24288, 14072, 28508, 50344, 22780, 10056,
                59808, 5172,  14760, 62748, 22052, 5656,  16752, 60780, 11272,
                47492, 15696, 59512, 45620, 53964, 2592,  24196, 38948, 53116,
                1212,  63020, 41124, 54500, 50772, 36516, 59476, 28940, 54240,
                38356, 360,   18644, 40580, 24304, 61184, 46472, 33256, 3532,
                13144, 55896,
            },
            {
                10260, 49948, 42740, 256,   9988,  56048, 5620,  14652, 26724,
                39232, 44532, 6716,  31372, 52036, 61284, 39636, 47976, 64456,
                316,   60252, 5268,  44760, 18244, 59628, 30168, 8660,  58320,
                3512,  64660, 47152, 36764, 28928, 18912, 11732, 4236,  37860,
                29808, 52972, 11168, 50188, 45352, 27804, 39924, 2548,  30500,
                21300, 17320, 53484, 62948, 29188, 3448,  18624, 11836, 24200,
                8456,  44196, 62064, 30408, 16272, 65500, 32392, 50736, 61692,
                37292, 57420, 42552, 20508, 45848, 25644, 16824, 31076, 2768,
                57268, 32624, 59984, 53452, 5636,  45104, 37548, 4296,  56888,
                33928, 59268, 40024, 47888, 9312,  57312, 33336, 6584,  26664,
                36124, 46436, 28644, 55544, 39736, 49356, 31192, 52536, 24600,
                35560, 5124,  20408, 26500, 9932,  64960, 32152, 43452, 10784,
                48436, 33056, 23304, 11904, 26184, 1468,  48156, 13780, 5028,
                63724, 25944, 34652, 48296, 13872, 31248, 17168, 21608, 50752,
                44348, 29288,
            },
            {
                34820, 23408, 53668, 37504, 46840, 19120, 44896, 62392, 20928,
                11196, 33976, 63444, 13272, 3232,  17732, 25420, 7004,  42164,
                13148, 38508, 20660, 48904, 34248, 1496,  15416, 45656, 20880,
                51848, 39972, 10380, 23200, 61644, 52324, 45940, 27348, 23612,
                59904, 16028, 40420, 3828,  20420, 63224, 12832, 51688, 35892,
                48744, 1132,  37872, 44080, 22460, 47744, 58676, 41964, 50464,
                16640, 38128, 3284,  47372, 52364, 1092,  9024,  45308, 4912,
                25248, 15736, 3568,  32284, 51964, 8128,  13212, 44160, 40148,
                10404, 48960, 18248, 35992, 11932, 20644, 55072, 48668, 41736,
                13124, 26656, 5440,  31468, 21324, 724,   40824, 64872, 55096,
                20344, 51216, 1616,  11980, 35364, 8920,  42084, 19548, 220,
                56100, 62764, 49432, 29848, 37984, 56676, 5916,  21320, 60836,
                26976, 7540,  58092, 45432, 18320, 62732, 39892, 24484, 17368,
                44568, 10300, 57460, 5700,  65420, 38728, 9852,  58244, 7060,
                37028, 18884,
            },
            {
                60980, 15064, 26508, 65096, 13040, 33540, 28280, 49712, 1624,
                56816, 24472, 46832, 37636, 59016, 21648, 50132, 28976, 55700,
                33644, 26476, 57280, 10064, 65196, 53520, 37464, 61976, 12876,
                31356, 18060, 56076, 7492,  32916, 16120, 39520, 65272, 49864,
                1404,  34328, 57780, 25892, 54436, 33756, 5340,  58048, 24792,
                65504, 13420, 27676, 5260,  10248, 52136, 34532, 6104,  62736,
                26932, 33876, 57520, 7580,  25776, 35204, 39560, 21984, 12680,
                47660, 53016, 60384, 11204, 37876, 48740, 58624, 35448, 22368,
                65012, 6428,  25572, 41200, 62684, 33676, 28344, 17716, 9416,
                65108, 19792, 51072, 43544, 61152, 35644, 17800, 44680, 13180,
                31864, 61856, 42404, 17888, 60044, 25880, 65264, 13936, 44548,
                32188, 8584,  41744, 3344,  18168, 46916, 13916, 36224, 55080,
                16164, 42360, 3604,  37556, 29480, 55924, 9164,  32952, 58784,
                49756, 30508, 20360, 53680, 27596, 1172,  42096, 24916, 63148,
                54112, 1568,
            },
            {
                47880, 7640,  31088, 2776,  51712, 59312, 8568,  38368, 31208,
                52588, 18088, 4564,  27940, 53816, 10216, 36088, 2240,  46224,
                16656, 52268, 4060,  28800, 40140, 23280, 6192,  26892, 48712,
                980,   25432, 60740, 49600, 43344, 2156,  20920, 9260,  13884,
                42624, 21936, 46720, 9828,  38224, 16432, 43908, 19080, 8872,
                43060, 32712, 55956, 61376, 39640, 15244, 30572, 21028, 636,
                45584, 53260, 13688, 19464, 42772, 59560, 17024, 56088, 63728,
                33692, 40328, 23052, 17852, 64172, 27728, 112,   18696, 51468,
                29596, 14488, 56024, 47160, 8496,  51336, 380,   60272, 24604,
                46604, 37412, 2860,  15272, 54348, 10828, 49188, 25304, 2736,
                39076, 8464,  23984, 30476, 47900, 2964,  54184, 28380, 50280,
                38616, 22076, 14736, 51648, 33652, 58852, 25972, 2188,  50332,
                31100, 20404, 64724, 51756, 14580, 5380,  43356, 20992, 52612,
                6908,  41292, 37068, 15988, 45008, 51536, 34300, 47364, 14648,
                27272, 41160,
            },
            {
                58724, 44584, 56320, 20684, 40868, 24428, 17532, 3788,  64344,
                42484, 9464,  60824, 40680, 15456, 43948, 57192, 63176, 19980,
                8836,  61656, 43984, 13928, 47512, 17624, 32864, 43384, 54828,
                38804, 34840, 4704,  13620, 26168, 59328, 35688, 56796, 31656,
                53924, 6564,  62772, 29532, 332,   61604, 48232, 28440, 38952,
                6252,  49764, 23544, 18444, 46008, 25744, 54324, 60024, 40264,
                11060, 29836, 22488, 63124, 49344, 10604, 29260, 43980, 26632,
                1764,  8548,  29700, 44656, 5728,  33496, 41548, 55428, 4556,
                45368, 37476, 2284,  30524, 23336, 15924, 39708, 43856, 32200,
                6916,  53304, 27960, 63112, 23572, 29772, 58756, 6484,  52808,
                16744, 57040, 45088, 4708,  58180, 21728, 37024, 6812,  11640,
                61188, 25640, 54980, 63824, 10880, 23008, 44692, 63080, 9268,
                39948, 47972, 11456, 34624, 22832, 47304, 60592, 36132, 2484,
                28324, 12796, 61040, 3256,  22948, 11504, 56820, 4328,  20484,
                32312, 12032,
            },
            {
                22536, 37712, 16420, 35360, 5372,  43816, 55828, 47308, 14560,
                35736, 22148, 29940, 48940, 7252,  24032, 32412, 12464, 40852,
                24584, 37328, 31824, 21520, 51260, 3244,  59804, 11248, 8192,
                64336, 22132, 45772, 40336, 30436, 53220, 5556,  46928, 28024,
                37304, 18548, 12948, 50416, 32784, 22452, 53436, 11392, 60460,
                54748, 15628, 3272,  35568, 57724, 2244,  8224,  36748, 18184,
                48868, 58604, 3732,  37320, 32580, 2424,  53564, 6480,  19188,
                36504, 58908, 49100, 54088, 14360, 23992, 62160, 12216, 26220,
                60964, 21060, 43244, 58248, 63792, 4488,  55416, 11040, 21668,
                57832, 34492, 12088, 41152, 7660,  37828, 46488, 32816, 63848,
                27896, 34728, 49960, 14140, 40440, 33688, 16444, 46728, 58496,
                18652, 2444,  43956, 29372, 6848,  40924, 52968, 28840, 18536,
                4816,  59116, 27688, 116,   54624, 31256, 10936, 15796, 64320,
                23768, 45824, 55208, 33524, 63400, 30276, 17808, 60428, 39508,
                52860, 2956,
            },
            {
                29524, 9588,  63380, 50044, 12416, 61516, 29484, 10232, 26244,
                51208, 57884, 1264,  34604, 65416, 18772, 3912,  51044, 29544,
                54968, 1572,  58220, 7916,  62352, 36284, 56616, 29196, 19784,
                14756, 52380, 57600, 17640, 9956,  63784, 23888, 15532, 3092,
                58696, 45032, 25068, 56092, 41868, 14596, 4584,  36420, 31404,
                20540, 41388, 64112, 28860, 12232, 42964, 50092, 14088, 65104,
                24492, 6744,  41940, 55200, 24804, 14292, 38736, 60808, 50864,
                11736, 42440, 21032, 3328,  57680, 38352, 46124, 7564,  32124,
                53968, 9316,  17604, 12488, 33328, 36928, 47980, 29380, 62324,
                2268,  16892, 48652, 57064, 18340, 3468,  20544, 13796, 41572,
                22104, 11092, 62844, 20128, 52908, 9836,  63528, 41544, 31428,
                5844,  35076, 48120, 37736, 17064, 1160,  34752, 13060, 55996,
                38180, 24172, 44008, 62076, 17720, 41496, 25824, 56368, 39568,
                48200, 17988, 8776,  26116, 49088, 6652,  36856, 44268, 8540,
                64760, 49264,
            },
            {
                17952, 54316, 3872,  27200, 32908, 21796, 52872, 36964, 18988,
                5520,  44424, 13248, 54084, 26616, 46532, 36808, 60492, 6568,
                47892, 15824, 42204, 26676, 12648, 45236, 23844, 41072, 50372,
                31932, 1952,  27872, 6556,  48384, 37912, 19460, 50984, 41068,
                10692, 64544, 2292,  39048, 7976,  58292, 26988, 45268, 1632,
                25544, 46960, 9160,  51424, 32424, 61924, 22212, 28068, 34268,
                44836, 31252, 9944,  17832, 46712, 64684, 21644, 45516, 30636,
                15388, 62804, 32832, 27480, 9876,  50708, 20272, 16256, 48056,
                35236, 40284, 49748, 27428, 52488, 19008, 25364, 14152, 50928,
                38484, 45376, 24736, 31728, 52408, 43424, 60148, 50504, 5496,
                55212, 404,   37448, 7176,  25688, 29320, 2076,  23312, 51188,
                27152, 57036, 13328, 21120, 60128, 49848, 61912, 22056, 43196,
                32020, 15220, 52740, 8744,  37172, 3260,  49980, 7412,  32608,
                768,   59732, 38336, 43328, 14872, 54472, 22192, 27900, 13468,
                35000, 24808,
            },
            {
                42888, 38900, 47228, 58272, 7388,  39828, 440,   59632, 41508,
                31660, 61784, 20876, 39236, 10308, 58284, 42080, 14264, 22028,
                34504, 64728, 19428, 53748, 33820, 484,   16560, 6048,  46536,
                62552, 37236, 44116, 60568, 33900, 12352, 43236, 59920, 33088,
                26624, 20392, 35312, 16860, 46884, 62016, 19516, 50868, 63012,
                12596, 58704, 37508, 4160,  19592, 39540, 55512, 5412,  1344,
                56984, 15548, 60700, 50552, 28276, 8304,  34192, 4216,  25664,
                55872, 820,   39268, 17408, 65296, 35676, 30036, 57076, 1056,
                23172, 64560, 6244,  1908,  44292, 60164, 7216,  41636, 4908,
                20712, 59588, 8640,  672,   64412, 28456, 9168,  35816, 26156,
                47412, 30064, 43932, 57368, 49152, 60940, 44764, 55152, 14592,
                38988, 64940, 9524,  53868, 31516, 26424, 7880,  47012, 2908,
                64052, 5960,  48700, 21232, 58512, 28624, 62976, 19288, 53552,
                12236, 29844, 21008, 3764,  58128, 41028, 968,   51888, 46288,
                5488,  59584,
            },
            {
                1512,  32112, 21188, 13504, 44252, 62788, 15544, 48380, 8012,
                55396, 25152, 3372,  49972, 30820, 17228, 192,   53456, 27780,
                10756, 38876, 4260,  46660, 29692, 63916, 39348, 55448, 25736,
                3852,  12892, 23368, 20168, 55128, 4,     29720, 4468,  53052,
                7304,  48980, 54912, 29900, 23592, 32236, 10024, 39424, 16568,
                34488, 22320, 53072, 26520, 15176, 46412, 10612, 48420, 37796,
                52276, 20884, 39904, 35532, 216,   58148, 52124, 41596, 10120,
                47928, 22760, 52716, 45672, 6020,  41892, 4192,  61280, 43148,
                13352, 28716, 54724, 38380, 22428, 10124, 55840, 30828, 64824,
                35340, 27564, 15044, 39908, 34368, 22616, 16280, 56644, 39624,
                18584, 60668, 14856, 32944, 17864, 4340,  36100, 10600, 19836,
                780,   43164, 24560, 3672,  45588, 39292, 11524, 57456, 17444,
                36352, 26712, 12368, 33984, 46060, 13932, 42844, 23004, 35456,
                44672, 51748, 64968, 34668, 11020, 31936, 63048, 18808, 56664,
                30844, 15236,
            },
            {
                52388, 10108, 55284, 25960, 18492, 35272, 28568, 23988, 11728,
                34452, 47048, 14660, 63188, 6944,  23616, 33124, 62620, 44444,
                49260, 57836, 24204, 8332,  50256, 21572, 10072, 60080, 18600,
                34536, 53300, 41352, 8860,  50124, 25368, 62680, 14172, 22224,
                37116, 61156, 11804, 5184,  51592, 860,   55372, 43304, 5592,
                56496, 2656,  44096, 63284, 6644,  30344, 61028, 17548, 25236,
                63660, 12224, 4676,  23764, 44100, 13244, 20176, 16452, 59472,
                36288, 7420,  60388, 13656, 24408, 54628, 11520, 26720, 51192,
                18580, 59288, 31832, 15836, 63084, 34092, 46500, 17260, 52720,
                10748, 43008, 56116, 49832, 12400, 46232, 62384, 1904,  11500,
                52620, 4648,  23812, 42308, 11928, 39888, 22396, 58988, 47188,
                33808, 29496, 50276, 58548, 15460, 19684, 34188, 53280, 29960,
                50436, 59400, 40564, 55064, 4392,  31560, 9540,  57500, 4860,
                15560, 24928, 7684,  50004, 17196, 24068, 48304, 9088,  39984,
                23228, 62372,
            },
            {
                28064, 36472, 64556, 2500,  49540, 56588, 4280,  52436, 64924,
                17620, 38096, 28804, 42976, 56832, 51808, 39684, 8708,  18308,
                2608,  31164, 14728, 56268, 35880, 43200, 13848, 28348, 48288,
                7812,  58764, 30812, 64792, 16448, 45356, 39500, 56596, 47352,
                28236, 15900, 39800, 44588, 63864, 13500, 36904, 27848, 47992,
                30068, 18012, 33044, 40492, 58160, 23176, 34684, 43100, 7516,
                29336, 32928, 47220, 61540, 56224, 31128, 39064, 63780, 27256,
                32084, 43312, 19608, 29796, 34476, 48660, 21164, 39016, 33196,
                7376,  45184, 10884, 47816, 4172,  40388, 26808, 1044,  23028,
                47664, 4012,  61828, 20292, 5608,  53652, 31336, 24856, 44920,
                33948, 63336, 8380,  55488, 27280, 63944, 30900, 6780,  53364,
                62768, 17748, 6228,  36228, 41728, 64272, 476,   24420, 42116,
                9804,  22656, 1544,  18720, 65440, 24616, 48544, 37740, 61624,
                54456, 41584, 28756, 45540, 60188, 37716, 27512, 3884,  35632,
                11844, 45420,
            },
            {
                6612,  16576, 40376, 46040, 9044,  30992, 42028, 20616, 44716,
                1400,  59324, 9488,  19596, 35620, 46184, 13188, 59184, 25716,
                36984, 63456, 41452, 18712, 61752, 6396,  32324, 52004, 37800,
                44336, 14952, 27060, 2468,  36508, 32300, 5864,  18960, 9420,
                43156, 3632,  57980, 25500, 34080, 20532, 60276, 23260, 7852,
                65140, 50560, 13160, 9404,  49400, 1504,  53800, 13408, 56044,
                40864, 50948, 9680,  18344, 26340, 6664,  49380, 2172,  53948,
                3988,  12104, 51140, 63420, 1308,  58480, 15872, 62500, 2932,
                56292, 36252, 25472, 20696, 51348, 57784, 12800, 37408, 58864,
                32716, 16360, 30164, 25608, 36692, 41216, 7752,  59432, 20136,
                28960, 48016, 36580, 50368, 1020,  46092, 51728, 16064, 26356,
                40308, 12564, 23056, 55016, 4628,  28492, 60756, 46252, 6736,
                15964, 61672, 44612, 28400, 38888, 51236, 16396, 2144,  27884,
                10892, 19544, 1088,  55792, 5048,  12756, 43464, 54152, 59180,
                20376, 49680,
            },
            {
                25076, 60956, 33060, 22700, 59996, 12364, 37168, 57948, 5712,
                32700, 48208, 54028, 26672, 5076,  2300,  21288, 29500, 55140,
                5888,  50988, 11472, 27416, 3172,  54048, 24336, 1300,  63508,
                22016, 4996,  56480, 47988, 10768, 22824, 59204, 50600, 35060,
                65464, 31904, 18316, 8732,  49792, 41908, 16260, 52848, 11716,
                35800, 25024, 57060, 20984, 27488, 37424, 45712, 19848, 3620,
                58912, 22496, 2024,  42872, 52468, 36828, 11016, 21880, 45316,
                57760, 24872, 38452, 8968,  41428, 27956, 46208, 10020, 23472,
                41056, 49856, 252,   61372, 29804, 5856,  19116, 44016, 62656,
                7772,  39524, 54812, 2336,  58252, 18104, 49136, 13392, 39220,
                3076,  16108, 21932, 60108, 13452, 19368, 34816, 3140,  43760,
                8492,  61072, 47448, 32032, 10780, 51736, 13744, 21048, 56128,
                37180, 32740, 53664, 11536, 6384,  60100, 20900, 33168, 47060,
                39712, 63524, 36272, 31760, 21960, 51532, 15760, 64672, 32800,
                68,    41992,
            },
            {
                51660, 14016, 3536,  53480, 17872, 27464, 50856, 15120, 25864,
                40056, 22852, 13480, 64240, 41660, 31996, 61396, 47684, 15616,
                43724, 22372, 33504, 48408, 39212, 44912, 58352, 17528, 10968,
                40812, 33792, 18916, 52356, 61476, 42372, 13344, 26140, 956,
                53328, 23652, 46340, 56084, 29440, 6180,  2032,  57876, 39744,
                45456, 208,   42616, 62120, 16300, 4984,  31740, 63180, 26224,
                16128, 38268, 64568, 33752, 15276, 62144, 29304, 40936, 18056,
                34936, 14852, 47872, 56732, 18788, 4416,  36992, 53476, 30680,
                64400, 17224, 13736, 42832, 33464, 54552, 24500, 49112, 27892,
                11468, 21828, 50412, 45200, 10076, 27628, 65176, 34784, 52236,
                57256, 10176, 41432, 31664, 7224,  38792, 58144, 23904, 55532,
                30284, 37656, 20020, 59056, 44024, 25872, 39752, 31220, 49080,
                3440,  25484, 47580, 14996, 35664, 55452, 43804, 7872,  58580,
                23692, 14508, 49216, 9008,  61896, 40108, 25720, 7984,  29420,
                18092, 58008,
            },
            {
                35352, 30356, 6328,  43524, 39252, 676,   63736, 8268,  30224,
                60812, 52316, 6796,  16920, 57356, 49744, 11188, 39024, 35216,
                732,   53316, 60976, 7544,  20552, 14348, 34876, 29656, 50336,
                60404, 24800, 45780, 38512, 28920, 4356,  54752, 30608, 38912,
                14884, 60972, 2688,  12532, 38520, 63216, 47168, 26668, 31752,
                19628, 5772,  52512, 29552, 39476, 59816, 50756, 9568,  35660,
                48580, 28788, 12456, 54392, 5016,  23872, 47572, 59720, 7332,
                65084, 30908, 5244,  22900, 33348, 51444, 14168, 60396, 19944,
                8116,  28552, 52916, 38316, 9844,  65128, 15452, 2080,  53248,
                42336, 34852, 63476, 14520, 32968, 42516, 328,   23232, 6448,
                43848, 63744, 25204, 54820, 44732, 28588, 65364, 11384, 48628,
                15016, 904,   50908, 7488,  16756, 34932, 2064,  58176, 18464,
                64528, 8408,  40432, 62648, 824,   30156, 25684, 13132, 52820,
                5240,  29872, 56748, 44220, 18472, 34848, 2700,  48488, 45056,
                38448, 9672,
            },
            {
                22252, 46236, 57120, 62112, 24112, 34196, 47032, 55572, 18520,
                10904, 45276, 34072, 37748, 28044, 23868, 7524,  18844, 64700,
                10048, 25496, 16788, 30624, 55816, 65216, 8552,  47052, 5692,
                54888, 13116, 668,   8752,  17120, 21344, 63408, 47464, 10480,
                20280, 42280, 35572, 52096, 21724, 33340, 17976, 8844,  61356,
                55772, 15116, 33980, 7600,  47084, 12084, 22932, 43352, 55460,
                6880,  41856, 20568, 45224, 58264, 31980, 900,   13388, 50796,
                26820, 42756, 53376, 60112, 11360, 44956, 25416, 41912, 3892,
                35224, 46772, 59188, 22640, 6732,  45992, 36444, 31712, 6512,
                57580, 18676, 4664,  24160, 60156, 53144, 16324, 47468, 31020,
                27192, 18176, 36076, 1856,  15284, 47860, 21276, 5120,  35872,
                62640, 27204, 41140, 24016, 63720, 54304, 9424,  45100, 12588,
                29036, 51024, 20028, 23416, 42776, 17704, 48444, 64436, 34392,
                41724, 17264, 2996,  26688, 11560, 58892, 53164, 23084, 12980,
                55376, 63960,
            },
            {
                7092,  19688, 36884, 11264, 49888, 4988,  21168, 41460, 36260,
                49508, 4668,  20848, 61792, 1072,  44632, 51856, 56344, 28356,
                41260, 57792, 45752, 36664, 2532,  25900, 42896, 21408, 37368,
                27392, 32088, 64508, 58112, 35796, 44792, 33092, 7548,  58456,
                50128, 28884, 5660,  25820, 59504, 14432, 43808, 50972, 22924,
                37964, 48124, 63540, 24168, 18756, 54400, 2432,  14932, 30600,
                60576, 1464,  50116, 25556, 9016,  39100, 19676, 55344, 37128,
                10148, 3096,  16520, 39388, 28424, 64036, 7020,  32228, 49076,
                55984, 11840, 2988,  27084, 17992, 56836, 40640, 21240, 61276,
                13572, 38684, 29240, 48596, 7536,  37244, 21020, 56240, 62176,
                12736, 49472, 5264,  51956, 61588, 9528,  32416, 42260, 53460,
                18376, 56700, 33384, 4108,  46868, 37568, 22232, 61016, 42244,
                34520, 4612,  56476, 31788, 58232, 9684,  37652, 3928,  20172,
                45968, 60064, 38116, 50776, 31300, 6232,  41860, 62396, 16992,
                1936,  26984,
            },
            {
                48692, 14388, 54488, 28032, 16468, 31948, 59844, 13836, 1800,
                63352, 25532, 42248, 12300, 54168, 14956, 33896, 21508, 3584,
                32180, 13704, 5276,  51604, 40724, 12312, 53012, 62180, 3068,
                15740, 48448, 42196, 23760, 50236, 14504, 1884,  40664, 24612,
                16488, 45040, 64244, 9952,  40836, 54728, 2896,  30184, 12260,
                3964,  27684, 10252, 36952, 58672, 32508, 40164, 65496, 24904,
                18408, 11440, 35176, 62840, 16084, 51824, 43832, 33636, 23372,
                46860, 61848, 20892, 35508, 416,   54848, 21632, 58932, 16404,
                24728, 39584, 43592, 60864, 49720, 29212, 10992, 47836, 25884,
                51800, 44412, 1136,  55636, 31820, 12128, 40596, 4400,  9212,
                38652, 33096, 59744, 23500, 39748, 17540, 57428, 25384, 2764,
                45920, 9256,  12868, 60240, 29140, 14808, 5452,  27228, 52392,
                16208, 38672, 11324, 45440, 2944,  50184, 61336, 28412, 11708,
                55044, 8264,  22592, 65256, 46652, 20012, 27928, 36624, 32616,
                51248, 43168,
            },
            {
                30044, 59416, 1176,  41808, 65468, 7848,  45024, 23684, 28824,
                54792, 32288, 59136, 47612, 30288, 39864, 5796,  59928, 46988,
                50216, 63212, 24072, 19984, 59988, 29084, 18004, 33188, 57300,
                40112, 19992, 10564, 3952,  56148, 26776, 53256, 61888, 36552,
                4768,  55964, 31596, 19336, 48936, 7068,  36020, 64972, 40136,
                53216, 57588, 43020, 1360,  45392, 27304, 6364,  47484, 37308,
                52264, 57636, 31096, 40728, 4432,  27012, 64216, 6420,  15204,
                56540, 30464, 50324, 43420, 9152,  47408, 13512, 37960, 1756,
                62900, 19328, 31844, 14684, 35020, 1392,  63368, 4360,  16716,
                34168, 9448,  64772, 19376, 26940, 46792, 61092, 25404, 45428,
                53628, 20156, 42852, 8000,  27904, 34768, 51112, 13664, 38916,
                30628, 21376, 43000, 52224, 19164, 49184, 32824, 57112, 64,
                24396, 61728, 53524, 26604, 14640, 33800, 22184, 52140, 39872,
                32308, 26032, 14040, 1916,  54408, 15068, 57496, 4544,  10676,
                60488, 39404,
            },
            {
                23440, 33604, 9812,  51760, 25028, 35336, 58040, 52412, 38592,
                9916,  16024, 3980,  19488, 8652,  64004, 26484, 42812, 11796,
                18196, 38340, 8168,  34592, 48036, 488,   9912,  45508, 23492,
                6928,  52432, 61384, 30748, 38640, 18640, 11720, 29952, 21836,
                51980, 13540, 1152,  38120, 57704, 28292, 24592, 46456, 19840,
                16000, 31628, 21780, 13860, 50992, 16876, 59636, 21056, 2940,
                44044, 8028,  22376, 47908, 56396, 12048, 29612, 59976, 1604,
                40236, 25732, 12396, 57668, 18540, 34216, 52584, 29964, 44400,
                51052, 8796,  55024, 6156,  52272, 23864, 42180, 55424, 30536,
                60328, 22688, 40916, 50308, 15684, 57868, 1668,  35276, 17128,
                28688, 2428,  14568, 55228, 46584, 156,   62448, 7044,  59648,
                48952, 64564, 34776, 1424,  39576, 10812, 64032, 40516, 20324,
                48272, 7932,  35716, 19056, 59540, 41768, 7140,  16356, 372,
                62624, 48284, 36464, 42992, 34004, 8800,  39848, 45564, 25628,
                19060, 3596,
            },
            {
                63616, 17152, 38056, 47312, 19560, 2624,  12360, 18160, 6376,
                46408, 40816, 51304, 24400, 37156, 55588, 16820, 35564, 924,
                29632, 54460, 44056, 15348, 56008, 64152, 38752, 50760, 13876,
                59868, 28144, 34560, 46160, 6280,  63804, 48396, 8544,  43548,
                60540, 33908, 47720, 22808, 11528, 16328, 59876, 524,   8420,
                60936, 48572, 4716,  64176, 35308, 8916,  53900, 12660, 33408,
                28056, 14632, 61584, 160,   18280, 37928, 21764, 49628, 45532,
                19196, 7312,  64748, 4440,  27404, 62320, 23632, 5000,  11148,
                27760, 36204, 21516, 61796, 45372, 12304, 19172, 37784, 7676,
                46356, 13100, 36680, 5128,  33372, 10312, 43084, 22256, 63468,
                47932, 58340, 30960, 64732, 11352, 19480, 43900, 22988, 28308,
                16368, 4900,  26008, 58276, 23376, 54196, 7052,  45680, 13804,
                30780, 43572, 65072, 5652,  30020, 47404, 55436, 24440, 44460,
                57744, 18224, 5036,  60900, 29352, 23884, 63396, 49240, 55940,
                13428, 52280,
            },
            {
                44808, 57016, 4884,  28944, 60560, 43668, 50120, 30568, 56188,
                22172, 64472, 33444, 57984, 2112,  48924, 22528, 52460, 62556,
                25212, 58712, 4184,  31692, 26444, 22008, 5792,  30272, 35804,
                43836, 1652,  17512, 12820, 22980, 41924, 33272, 57532, 2980,
                17176, 25452, 42212, 62140, 52688, 44544, 32388, 51312, 42132,
                34832, 24020, 39060, 55020, 25444, 30304, 41296, 49156, 63588,
                38612, 54916, 25296, 45616, 32524, 53824, 42664, 9748,  34680,
                54224, 37824, 31952, 46360, 41484, 15112, 39352, 57332, 47636,
                65412, 41640, 2580,  26336, 39172, 33084, 58560, 50064, 27064,
                53844, 3424,  56692, 24752, 62540, 51844, 30152, 54844, 13168,
                6804,  40392, 36264, 24796, 49920, 37788, 31552, 53820, 10444,
                36988, 55560, 8736,  44776, 15144, 30136, 36572, 26148, 59612,
                10376, 50764, 23124, 1444,  38276, 12196, 63568, 35920, 10456,
                29776, 21364, 53036, 11388, 50208, 16880, 576,   21236, 30752,
                35244, 7884,
            },
            {
                40524, 12008, 21696, 54220, 14676, 39676, 26404, 62588, 35836,
                948,   27420, 11452, 44676, 29556, 13896, 9728,  45340, 6788,
                40260, 21252, 10440, 46776, 41928, 12748, 59216, 54808, 25096,
                8888,  65112, 49708, 55236, 59356, 0,     27988, 20072, 39040,
                50360, 9380,  28520, 6540,  3416,  35396, 21356, 14116, 56460,
                29108, 11652, 7160,  44368, 19224, 61044, 3836,  23216, 17696,
                5332,  50876, 10592, 35844, 7056,  63464, 2752,  16780, 24568,
                58944, 13976, 52116, 21948, 2220,  55468, 8088,  20220, 33584,
                17300, 13064, 59228, 48812, 16736, 8648,  64636, 148,   14936,
                20724, 42528, 29132, 45208, 7732,  20040, 39664, 4272,  27212,
                50852, 10208, 21176, 4928,  16204, 41684, 3380,  58056, 46752,
                19096, 41040, 50596, 32296, 60580, 2788,  17836, 51960, 4700,
                33576, 17040, 41416, 57688, 52376, 20232, 26856, 2848,  50584,
                40764, 7820,  38088, 26228, 44192, 58960, 37688, 6188,  42676,
                61840, 26172,
            },
            {
                50072, 31188, 36440, 64360, 8424,  33240, 3924,  10708, 42804,
                14912, 59824, 18508, 6200,  39296, 61884, 34096, 56632, 31264,
                14628, 51032, 36168, 60840, 53184, 3728,  19192, 48640, 15972,
                41272, 21448, 39080, 26108, 36356, 15292, 45756, 53604, 12904,
                64896, 36456, 54852, 19232, 39928, 63888, 9800,  26876, 4144,
                63248, 50076, 16288, 58308, 1100,  37864, 11512, 46104, 57004,
                31636, 41864, 20236, 60216, 15108, 26296, 51220, 30864, 62488,
                3692,  28780, 9396,  61060, 35932, 49880, 30532, 60320, 616,
                52788, 24324, 31368, 54312, 4048,  29908, 23096, 36044, 47440,
                32568, 63808, 17208, 59140, 12760, 48316, 36328, 16652, 59932,
                33132, 45052, 56628, 61192, 52012, 26708, 63612, 14552, 34324,
                976,   62600, 13204, 20692, 38788, 48684, 56408, 21988, 63356,
                37708, 55632, 28540, 14212, 9036,  33104, 46028, 60624, 15080,
                32172, 64416, 56264, 3308,  13640, 31476, 53700, 10500, 47864,
                15332, 2780,
            },
            {
                60788, 18616, 46588, 168,   23468, 49056, 59456, 20872, 53348,
                47608, 37372, 49924, 54760, 21144, 42252, 26816, 3224,  18924,
                48076, 65520, 1644,  17036, 28904, 38044, 33768, 45260, 2540,
                56972, 31968, 10840, 4284,  52596, 8124,  61760, 24088, 3960,
                31480, 44180, 14820, 59708, 30264, 47588, 54000, 38344, 46108,
                18812, 40344, 32304, 26584, 47400, 52176, 34044, 27240, 8316,
                62128, 2132,  48552, 29272, 56608, 44784, 38228, 12636, 42112,
                20832, 48092, 43812, 17764, 24952, 12512, 45076, 26408, 42864,
                38604, 5680,  45936, 10864, 40928, 57448, 44664, 11676, 55304,
                6068,  39284, 2380,  34944, 23300, 55724, 456,   65320, 42052,
                23952, 13920, 1216,  29788, 34644, 9744,  6404,  21404, 30160,
                52508, 24008, 28956, 4256,  65488, 11492, 27560, 41024, 1944,
                45068, 7236,  25284, 49036, 59340, 43136, 17488, 4588,  54952,
                23148, 42544, 17376, 46984, 34620, 22420, 65020, 18372, 28620,
                54560, 22804,
            },
            {
                6560,  10144, 52924, 27828, 42520, 55444, 17424, 25596, 5220,
                31748, 8324,  23968, 63580, 4368,  16196, 52052, 60452, 38456,
                11424, 25976, 33432, 7904,  22552, 63324, 10624, 60440, 7204,
                28028, 61316, 44828, 18732, 48480, 30372, 41008, 34920, 56996,
                6936,  22360, 49172, 756,   24900, 17056, 6984,  58952, 23428,
                2196,  53168, 62192, 10004, 22216, 14300, 65036, 43192, 15992,
                36644, 24296, 13104, 39964, 4644,  22336, 8176,  55656, 35268,
                58024, 6436,  39700, 32652, 54384, 5884,  63972, 15652, 9780,
                56552, 19016, 35424, 61020, 20620, 27296, 51860, 18188, 59536,
                25560, 49824, 10952, 52760, 27444, 44156, 31528, 9040,  19280,
                53420, 36932, 47800, 18700, 43032, 54648, 48612, 39672, 44964,
                58980, 7644,  47732, 42472, 33732, 8336,  46156, 15612, 31640,
                12808, 20680, 62496, 35552, 5304,  22720, 39532, 28744, 47924,
                12436, 964,   27748, 61200, 7616,  50672, 4056,  41552, 33592,
                57476, 38472,
            },
            {
                44148, 34864, 58824, 16416, 38304, 12628, 35044, 45176, 65028,
                40944, 28108, 12560, 34900, 46188, 31000, 8476,  44960, 22868,
                57952, 41300, 54256, 44352, 50180, 40360, 25712, 52484, 20108,
                36968, 14008, 53956, 34312, 64200, 21604, 10548, 17308, 46576,
                27400, 63028, 11584, 33556, 56852, 42792, 12696, 34264, 29696,
                14800, 35616, 5868,  42532, 29296, 57392, 4380,  20556, 55388,
                30444, 52736, 58776, 18404, 33708, 64756, 46940, 17476, 512,
                26944, 50660, 14788, 62124, 1372,  38084, 21076, 34416, 49300,
                28524, 62920, 14576, 50480, 7240,  38192, 2692,  41988, 8436,
                30112, 21664, 60712, 41084, 5672,  15516, 57664, 49884, 26492,
                3204,  58524, 7788,  64312, 23156, 12752, 60228, 25704, 16584,
                11164, 37880, 55400, 18000, 57844, 24756, 36008, 51612, 61232,
                54276, 47692, 39124, 648,   30400, 64044, 53952, 8672,  34472,
                58796, 36956, 51908, 20112, 39680, 58268, 25032, 45840, 12516,
                1060,  26992,
            },
            {
                63928, 13416, 32092, 4136,  62980, 5992,  30116, 56668, 2060,
                15528, 58556, 52700, 320,   59392, 25288, 12996, 36240, 2092,
                29892, 4904,  19972, 57084, 13508, 300,   16428, 31056, 42344,
                49072, 23772, 996,   6276,  27020, 43252, 1828,  60584, 38300,
                51976, 19696, 41384, 37268, 5716,  51820, 21064, 65372, 49524,
                44300, 60036, 19812, 54580, 37280, 7868,  50148, 39596, 832,
                47028, 9336,  43944, 3112,  50348, 11156, 25108, 53196, 32196,
                63232, 10872, 23420, 57108, 28088, 46580, 52684, 59152, 3784,
                22968, 44244, 1684,  25244, 33784, 64308, 15840, 32216, 62196,
                37544, 14192, 46736, 33064, 63240, 20996, 38584, 12100, 61712,
                32528, 40756, 16052, 27636, 38320, 1664,  32408, 4660,  62052,
                34824, 2352,  26980, 13988, 50172, 392,   59488, 6400,  23656,
                29048, 10184, 18448, 56768, 15820, 11828, 50136, 19756, 61968,
                24848, 5800,  45028, 11028, 30008, 15020, 36724, 8580,  62560,
                51732, 19960,
            },
            {
                39780, 24772, 49660, 21304, 45864, 51492, 10068, 19144, 38960,
                48508, 21672, 43656, 17876, 37988, 49068, 62820, 53884, 16836,
                61620, 46416, 9668,  27812, 32280, 58920, 47240, 4372,  55876,
                11628, 62280, 39592, 57372, 14824, 51464, 55912, 24948, 13700,
                3004,  55244, 8816,  59168, 27920, 45872, 3636,  38668, 7912,
                11100, 25352, 96,    48252, 16960, 33356, 60640, 25924, 12608,
                63760, 22072, 27712, 61708, 37188, 29856, 41904, 5336,  39144,
                19804, 45464, 36408, 8528,  18936, 41652, 11632, 31324, 7816,
                40552, 53764, 30396, 58536, 47196, 12120, 54744, 24272, 45720,
                1104,  53616, 18608, 9620,  1968,  48168, 29880, 6588,  46112,
                22116, 52680, 11108, 44416, 50588, 57196, 20380, 42580, 53592,
                49584, 22416, 63816, 44000, 29948, 21272, 39400, 17492, 43224,
                3188,  34468, 41844, 51196, 44976, 26748, 37696, 2720,  43568,
                16352, 56744, 32648, 64068, 1744,  55516, 49228, 21476, 31044,
                16676, 47648,
            },
            {
                9112,  55160, 1732,  57492, 26084, 41064, 61544, 23608, 53936,
                7080,  33572, 10172, 29660, 56468, 6608,  19692, 42556, 32896,
                50668, 14528, 39084, 63624, 6628,  36816, 23012, 64492, 8724,
                34976, 29476, 17244, 46732, 31708, 36556, 7592,  49064, 33080,
                29368, 44916, 16268, 22564, 31736, 61296, 18360, 26836, 57848,
                52296, 31648, 39492, 64224, 11864, 23356, 44452, 31180, 17344,
                35368, 41504, 6656,  14752, 20464, 56164, 61008, 15708, 49024,
                58420, 2400,  30120, 53792, 48732, 3132,  25648, 17316, 64696,
                36000, 13256, 9100,  18032, 42884, 4516,  49908, 19816, 57904,
                5492,  42412, 28908, 57188, 36404, 25336, 53860, 42448, 17892,
                35696, 4316,  30128, 59812, 6028,  35084, 14112, 28912, 8500,
                18484, 31272, 40184, 4448,  9760,  62848, 54676, 11636, 49180,
                64848, 58260, 24864, 7876,  61320, 33680, 6256,  47232, 31120,
                13240, 40472, 22504, 47552, 18192, 26516, 40700, 3164,  58664,
                35916, 4476,
            },
            {
                42452, 30632, 18208, 36780, 7720,  33316, 14672, 28984, 36424,
                59912, 25964, 63852, 3976,  41156, 23568, 10816, 28392, 5892,
                25088, 35524, 22168, 48956, 18284, 52824, 41060, 27612, 19360,
                43996, 50976, 6844,  22380, 63312, 11104, 20060, 41712, 65044,
                5508,  35808, 62436, 50684, 1320,  14180, 54492, 33276, 41720,
                16152, 21656, 45492, 27916, 58324, 3148,  53540, 5348,  48664,
                54720, 58428, 32380, 52076, 47472, 1156,  9064,  23628, 28452,
                12688, 42732, 65308, 15396, 32788, 61380, 37444, 44868, 56032,
                20400, 48308, 61720, 38880, 21896, 29120, 36588, 9720,  26868,
                35072, 65024, 23880, 50648, 16016, 59736, 3548,  64480, 14160,
                56120, 48364, 63392, 24644, 17284, 47088, 55324, 64780, 39052,
                46036, 12132, 57056, 51380, 35220, 45288, 32108, 26640, 37356,
                21580, 5420,  31808, 13784, 20528, 55580, 23512, 65436, 52368,
                27336, 8832,  54260, 4832,  34736, 60704, 13080, 43972, 53000,
                28172, 61488,
            },
            {
                22260, 46384, 65160, 12068, 43712, 59256, 644,   47300, 5012,
                12916, 50520, 45248, 15104, 52160, 34584, 47524, 64984, 56928,
                784,   60204, 54996, 3552,  43684, 10724, 33704, 14304, 60116,
                3600,  25476, 58416, 40484, 2456,  28176, 59852, 15620, 23000,
                47776, 12188, 25800, 39028, 42468, 10424, 48004, 23580, 2372,
                62040, 5068,  55848, 8596,  36200, 15264, 40796, 62256, 20008,
                10740, 2760,  23928, 39264, 11888, 35708, 44648, 33396, 54644,
                7308,  38264, 25268, 4500,  22364, 57624, 14120, 50628, 1016,
                27620, 6504,  32660, 51700, 1316,  56336, 63544, 41092, 15512,
                48584, 12408, 7608,  39104, 45164, 11180, 33940, 23020, 27692,
                9236,  39328, 760,   20780, 37868, 9500,  26392, 1992,  23536,
                6112,  60692, 16344, 25400, 19852, 7024,  14700, 1784,  55920,
                15928, 46228, 52828, 38560, 2116,  41112, 10288, 17700, 58448,
                248,   38060, 62184, 43220, 9868,  50328, 31568, 7136,  24080,
                10920, 14860,
            },
            {
                33848, 6288,  52336, 28548, 17008, 50816, 20924, 64392, 42628,
                18368, 39256, 31496, 20980, 61196, 1728,  37348, 14012, 44972,
                20368, 41532, 12668, 30348, 26352, 61608, 1648,  55288, 46452,
                37564, 52880, 12600, 34036, 49452, 44864, 54032, 38408, 272,
                58440, 53560, 18488, 6408,  56720, 35156, 64444, 7300,  37216,
                49448, 13544, 34064, 19084, 51656, 47164, 26420, 8060,  37664,
                28184, 45664, 16076, 65092, 26240, 57592, 18300, 63328, 50432,
                21212, 59572, 34376, 52128, 43588, 6636,  10080, 29708, 23080,
                41316, 57484, 16200, 25636, 45404, 13748, 7184,  52424, 32856,
                59520, 22300, 54092, 31344, 5104,  20140, 40592, 49084, 57788,
                32340, 43688, 12812, 53796, 31632, 58812, 44188, 50164, 33476,
                52944, 36696, 1192,  41900, 59872, 53332, 47276, 62032, 41320,
                29820, 9324,  63044, 27096, 58844, 48996, 29332, 34508, 42444,
                21388, 51160, 16612, 29004, 23604, 57528, 19868, 64688, 39136,
                56456, 49996,
            },
            {
                60532, 38020, 2628,  23276, 55784, 38568, 9480,  30668, 56352,
                24148, 3008,  57552, 8296,  27100, 54608, 17920, 30704, 7324,
                52448, 33804, 8624,  47204, 57804, 17268, 50232, 22476, 8160,
                31284, 15812, 20824, 61440, 9120,  18140, 4892,  32536, 27380,
                43072, 9552,  30556, 46220, 21588, 27496, 17156, 44564, 29920,
                54248, 24844, 42932, 63500, 1280,  30836, 21720, 56304, 33456,
                51308, 60856, 41888, 4092,  53068, 29408, 5112,  40008, 3024,
                16492, 46016, 10632, 18740, 62756, 40176, 35148, 55240, 46048,
                4396,  62816, 10720, 34800, 60244, 23668, 30584, 19284, 4608,
                43680, 236,   17840, 62944, 26256, 52256, 60552, 1388,  7424,
                19440, 51596, 25996, 62148, 41104, 3524,  16528, 12552, 62892,
                20596, 27412, 48352, 32432, 10660, 28612, 23116, 35856, 4268,
                50028, 19496, 34912, 44500, 16712, 4828,  54600, 14176, 45932,
                5476,  32212, 12124, 36776, 46200, 1980,  15384, 35516, 47320,
                364,   19392,
            },
            {
                26068, 9772,  58364, 32332, 48236, 5952,  25676, 34792, 52876,
                11052, 61804, 47004, 34060, 12208, 43064, 59260, 50008, 26264,
                38576, 16412, 63740, 23728, 36628, 6144,  39280, 28072, 41932,
                65068, 984,   55608, 42836, 30312, 23860, 57132, 13292, 51556,
                20868, 63532, 34176, 60628, 4332,  52900, 58748, 11996, 19888,
                59968, 9584,  28736, 39216, 58936, 11688, 43640, 63136, 13724,
                356,   18876, 30868, 8628,  20840, 47984, 13740, 43508, 25848,
                30804, 56204, 164,   28628, 47952, 25920, 15864, 60124, 32208,
                18904, 37900, 53188, 42584, 3268,  49164, 39572, 62080, 46388,
                28204, 38016, 56828, 41972, 47244, 14828, 36336, 29932, 45912,
                63104, 35256, 15764, 6456,  47876, 22696, 36376, 29724, 42232,
                9108,  55500, 13576, 65232, 4724,  38868, 18080, 59212, 12384,
                25340, 56648, 796,   11912, 23240, 62348, 37016, 7836,  63584,
                24448, 59292, 55684, 4036,  62828, 39968, 51932, 26636, 5060,
                30340, 41656,
            },
            {
                54016, 44724, 16116, 40692, 12872, 61876, 45904, 15904, 1436,
                37888, 28300, 17248, 51460, 39768, 24264, 4864,  10196, 61628,
                21848, 2560,  53960, 42944, 13684, 32168, 62412, 11160, 18792,
                48900, 36140, 26792, 5684,  38688, 64124, 46612, 36304, 7480,
                39732, 15888, 2256,  49144, 14696, 40708, 32500, 496,   39692,
                47212, 3700,  16572, 22912, 7164,  50196, 17656, 6284,  40172,
                23520, 57260, 49496, 36728, 59388, 32948, 62240, 9820,  54080,
                35764, 64144, 39008, 13348, 51200, 5196,  20720, 52500, 8792,
                13092, 47644, 28924, 20332, 8196,  57352, 16944, 11064, 54576,
                24096, 8788,  13360, 33772, 2596,  9304,  55472, 17652, 12288,
                23816, 2808,  55708, 28604, 11228, 60316, 51360, 56492, 4124,
                18220, 38244, 44900, 21468, 50592, 57232, 44224, 8164,  33024,
                64512, 43108, 39384, 52132, 32456, 46836, 20296, 26696, 50640,
                39408, 18688, 48492, 27504, 21024, 8360,  32292, 55768, 17848,
                63160, 13216,
            },
            {
                35652, 20904, 63512, 27640, 3320,  19844, 41780, 59320, 49876,
                21904, 43968, 7384,  3540,  64628, 20336, 48140, 33208, 41144,
                56192, 29324, 48644, 5084,  19940, 56056, 44484, 52304, 4528,
                58020, 22956, 45684, 11868, 50920, 15324, 1556,  59608, 25464,
                54992, 45448, 28404, 36940, 24224, 8808,  50524, 26200, 63984,
                34984, 52656, 43884, 55464, 36448, 32852, 25680, 54236, 47564,
                34836, 10832, 43792, 14780, 24776, 6116,  37808, 22660, 48752,
                14988, 7476,  22012, 61252, 31532, 36756, 2396,  64856, 43292,
                26536, 452,   63752, 14636, 31940, 37108, 26740, 3844,  34996,
                64576, 48088, 20600, 61116, 28980, 24364, 65360, 39032, 50996,
                42564, 60028, 37660, 44708, 33656, 18832, 7716,  25540, 46692,
                61460, 30604, 7156,  24828, 34412, 44,    14412, 48256, 52980,
                21544, 16252, 7256,  28336, 59412, 3280,  40896, 12556, 30892,
                1548,  10100, 34184, 41528, 53980, 14084, 43052, 10984, 58628,
                45644, 7768,
            },
            {
                31616, 1208,  49280, 56404, 36860, 53788, 29028, 5132,  13604,
                32384, 62968, 55004, 36216, 30140, 15776, 56944, 916,   14416,
                45064, 11240, 35124, 26920, 60172, 172,   24636, 29448, 14404,
                33872, 8536,  61012, 54160, 19604, 28700, 34444, 10752, 49564,
                18040, 5584,  58200, 11696, 63168, 43788, 57408, 15604, 22448,
                6412,  31424, 61916, 13844, 46812, 65528, 3064,  29428, 60512,
                4352,  27268, 64300, 2052,  55108, 45688, 18024, 57800, 1564,
                27944, 52384, 42328, 45216, 9128,  58608, 41164, 23452, 49296,
                33392, 58064, 22044, 55052, 45280, 61552, 52056, 42824, 14468,
                30744, 51016, 5564,  37140, 53232, 44328, 21200, 33232, 4452,
                26852, 9460,  20816, 49612, 308,   64996, 39604, 14440, 32868,
                1488,  58904, 53204, 16284, 62312, 42040, 29904, 25868, 37440,
                2784,  31260, 61824, 48428, 10008, 17940, 53368, 56892, 60816,
                44508, 16036, 65076, 6864,  24780, 60516, 49348, 36612, 21940,
                25272, 51272,
            },
            {
                59776, 6076,  24012, 11004, 32860, 8284,  64080, 24320, 39724,
                53324, 19200, 25748, 13120, 60104, 46096, 27340, 36824, 62676,
                18776, 6596,  65524, 40428, 50160, 9976,  38024, 47180, 63748,
                40016, 17172, 31644, 4152,  41176, 62344, 21976, 43716, 30480,
                65532, 23432, 41588, 52400, 30696, 18828, 3000,  38232, 48648,
                11356, 19492, 27292, 1688,  10204, 20500, 41704, 15000, 38008,
                21484, 50924, 17576, 39932, 28664, 51292, 11612, 40876, 31696,
                60152, 19208, 3376,  24812, 16904, 54232, 29156, 14144, 6976,
                39120, 51320, 6092,  40932, 2844,  10060, 23060, 19660, 59672,
                1984,  40368, 16792, 57808, 11480, 3460,  48788, 7780,  58412,
                16240, 53252, 31112, 13484, 57212, 27440, 53692, 43504, 21088,
                49008, 12296, 40752, 28284, 9972,  55176, 19476, 61136, 5876,
                45668, 54956, 13140, 36616, 25576, 43660, 34292, 6080,  22820,
                36536, 52504, 30040, 46896, 788,   18312, 28988, 5708,  2224,
                40092, 16496,
            },
            {
                46876, 38248, 43500, 15048, 51176, 45244, 17388, 34696, 47332,
                8760,  2828,  42060, 48768, 9756,  40456, 5688,  50496, 24464,
                30956, 52260, 22772, 15408, 32548, 17760, 54616, 21280, 3404,
                56440, 43584, 25180, 48476, 36416, 7396,  55552, 2876,  38832,
                13808, 33764, 748,   20972, 6700,  35260, 54936, 28148, 59764,
                42340, 56356, 50652, 40312, 24516, 58080, 52404, 45568, 11848,
                54864, 32348, 8904,  59924, 34040, 4520,  63600, 24248, 8508,
                46512, 38148, 33988, 57052, 11776, 46644, 35104, 19292, 62104,
                11184, 16548, 36148, 25276, 29936, 49580, 33596, 7288,  45676,
                55856, 22576, 26416, 43216, 32276, 15124, 61996, 40088, 28848,
                46132, 63904, 3592,  41996, 23508, 5972,  36136, 9660,  63000,
                24316, 35056, 5148,  46568, 37604, 3200,  51008, 11800, 58312,
                40332, 24108, 19180, 57160, 2088,  64784, 14644, 27908, 48736,
                3680,  20088, 11788, 58424, 38480, 34612, 63940, 44216, 52644,
                62388, 28160,
            },
            {
                57116, 19028, 65516, 26576, 58808, 21616, 184,   56784, 11924,
                61268, 30840, 58856, 22708, 1680,  32768, 21292, 54068, 8288,
                59176, 37732, 46468, 2124,  57292, 42668, 7428,  35280, 12180,
                27308, 51300, 708,   13952, 58104, 27108, 16384, 46880, 61400,
                8884,  50888, 56520, 45084, 60680, 12968, 46404, 8368,  32912,
                3992,  15172, 36700, 60348, 30772, 34640, 7800,  18980, 62248,
                5408,  42960, 48176, 22940, 13048, 20068, 44104, 35440, 55280,
                13260, 65228, 5912,  49968, 26928, 60600, 1128,  52632, 43876,
                31032, 59704, 46328, 19012, 12508, 56936, 62860, 39004, 28416,
                11132, 35884, 8216,  60180, 47072, 19148, 25192, 55564, 10964,
                35712, 18496, 38388, 61032, 47828, 16720, 30468, 58196, 4220,
                52344, 17020, 56880, 20676, 64596, 23696, 31768, 35328, 16968,
                28220, 8864,  50352, 32684, 39628, 20988, 46676, 9080,  40084,
                61748, 26216, 55548, 42396, 22928, 10536, 54524, 15524, 32080,
                9132,  12620,
            },
            {
                2960,  34180, 7452,  40152, 4224,  30472, 41980, 51904, 27536,
                37968, 16972, 51048, 35108, 55904, 64192, 17272, 43992, 13488,
                41740, 4168,  11704, 61732, 28268, 23788, 62884, 49272, 30440,
                60664, 9240,  22440, 64396, 45128, 10240, 32628, 52932, 19832,
                25904, 37432, 28812, 16100, 40144, 25588, 51852, 21308, 64832,
                23980, 45704, 9276,  5208,  16632, 49716, 528,   28844, 36592,
                25016, 58700, 1164,  38644, 53576, 61084, 29640, 2984,  49268,
                17420, 22544, 30076, 40572, 8080,  21360, 38528, 5252,  23892,
                27592, 1896,  53464, 65392, 42356, 15848, 852,   53292, 17596,
                47848, 63856, 51496, 448,   29264, 37536, 51940, 2344,  22620,
                50300, 6920,  25816, 32572, 11552, 51328, 20052, 45828, 38296,
                27864, 42324, 31216, 49596, 14076, 43904, 7928,  47508, 63860,
                53784, 700,   44704, 62516, 6524,  54132, 30768, 58136, 51120,
                17360, 33068, 5416,  14712, 50204, 7340,  26400, 20424, 48416,
                36180, 22492,
            },
            {
                53132, 29624, 55164, 49108, 13208, 36436, 62528, 7096,  20124,
                44860, 4732,  14100, 26468, 45692, 10928, 29088, 61412, 33932,
                25992, 55088, 20612, 36104, 51568, 14032, 2868,  45204, 18784,
                37884, 53988, 33736, 18252, 39260, 50396, 23800, 5032,  42156,
                59220, 3480,  48192, 9908,  32256, 62360, 2452,  17232, 39720,
                53284, 28544, 62940, 21000, 54604, 43952, 64180, 22156, 56660,
                13676, 30980, 16772, 26896, 46428, 6900,  15876, 41276, 26040,
                58928, 516,   44384, 62204, 15420, 56152, 31992, 64140, 49632,
                57096, 9544,  34052, 4196,  48404, 27224, 36752, 24124, 5176,
                31340, 14196, 20056, 41644, 6360,  64716, 13624, 34128, 42536,
                57676, 14796, 44740, 54308, 2756,  40980, 64440, 1052,  14792,
                10704, 61704, 7532,  2056,  26880, 57440, 39324, 3696,  21452,
                42264, 14844, 29460, 11316, 23516, 16668, 35600, 12792, 128,
                43880, 37676, 64220, 30564, 46080, 59896, 39208, 56752, 4880,
                60752, 43188,
            },
            {
                47632, 16724, 10448, 23748, 60076, 46024, 15920, 25196, 33748,
                64936, 54856, 41288, 7520,  19244, 39148, 4916,  48872, 500,
                16044, 49688, 30028, 43888, 8900,  33796, 40168, 58016, 5156,
                15004, 41976, 6220,  29704, 60376, 1740,  35904, 56916, 30104,
                12724, 18156, 64208, 23072, 6044,  49724, 35984, 44084, 12028,
                840,   35064, 48828, 41324, 26680, 11320, 38932, 46976, 9076,
                40848, 49492, 64816, 10540, 35152, 56952, 50884, 9896,  37820,
                53704, 33600, 12164, 51620, 35628, 45856, 10716, 17536, 41468,
                14220, 20308, 39460, 22800, 31880, 7840,  59440, 44644, 62172,
                40712, 56636, 34424, 23824, 54360, 9612,  45108, 60952, 5832,
                29984, 63384, 21416, 8928,  57020, 28360, 23076, 33780, 55092,
                48128, 19536, 58836, 36796, 53608, 17712, 62208, 12544, 25612,
                34028, 59460, 36900, 49104, 60344, 42984, 4940,  63436, 25064,
                21460, 10036, 52640, 3336,  17148, 33816, 1248,  29744, 13852,
                40436, 25348,
            },
            {
                588,   38036, 62992, 32936, 19256, 3032,  53712, 9936,  48332,
                1620,  23192, 29664, 62048, 52964, 58328, 35952, 21884, 56548,
                39912, 64680, 5880,  58964, 17660, 55528, 26840, 20940, 65040,
                25080, 47460, 57168, 12248, 43528, 21220, 14928, 63664, 8112,
                45740, 33916, 54292, 42480, 57956, 14336, 29488, 55296, 58720,
                31612, 18348, 13364, 57728, 2680,  31776, 15368, 52724, 33528,
                2512,  54444, 20512, 43140, 4052,  23936, 32004, 62776, 20808,
                46716, 28272, 4504,  19656, 24680, 2588,  28580, 47780, 7132,
                37040, 61408, 44744, 55780, 11244, 52192, 19432, 13088, 49384,
                2568,  9888,  27208, 59252, 48696, 16704, 31492, 26388, 19576,
                932,   39380, 35048, 48688, 15884, 36908, 59300, 7120,  40216,
                25668, 32640, 45172, 23048, 9644,  34360, 29216, 48856, 52080,
                6684,  55252, 18136, 4116,  26648, 52816, 38824, 29184, 55220,
                47920, 59284, 27936, 41016, 24340, 62952, 43700, 51376, 18692,
                64096, 8600,
            },
        },
        {
            {
                45080, 33012, 51224, 35724, 29100, 59044, 9064,  34140, 22164,
                43912, 51600, 58024, 37920, 21144, 56300, 480,   17980, 48044,
                14832, 55368, 62548, 37116, 16292, 41976, 56532, 65352, 7076,
                20148, 29892, 9176,  56256, 18776, 7400,  47192, 25148, 18188,
                44804, 29028, 2340,  12920, 17700, 5188,  30648, 45068, 12696,
                28764, 62120, 15720, 23688, 37508, 33596, 48004, 9552,  43388,
                28628, 49344, 13520, 53048, 17612, 62272, 20672, 38932, 25180,
                14352, 1340,  55448, 20488, 24580, 62772, 32668, 50548, 17436,
                26396, 13920, 22544, 31636, 7420,  63860, 34940, 3548,  9116,
                47104, 33488, 13632, 8324,  45788, 4156,  64984, 35112, 18160,
                57000, 21836, 9632,  15536, 59748, 55080, 27188, 2712,  14492,
                53384, 8228,  28004, 5572,  50136, 18756, 25772, 7992,  15164,
                40056, 64596, 10512, 31932, 51216, 19736, 45844, 15540, 11496,
                33916, 5640,  53188, 31720, 10700, 39532, 1064,  26312, 12764,
                38940, 29672,
            },
            {
                54328, 61044, 18852, 1752,  46988, 20644, 13000, 27224, 1164,
                61844, 11724, 7904,  27740, 13580, 30028, 64836, 52516, 36088,
                4780,  28728, 9124,  19216, 6188,  50044, 33400, 12416, 36368,
                53144, 48320, 41072, 14580, 31376, 64372, 41896, 1024,  61360,
                50864, 20572, 36220, 43312, 58132, 48256, 23336, 2008,  19092,
                38244, 56676, 1032,  27292, 61212, 4292,  14224, 56696, 17432,
                61032, 22748, 2212,  37144, 26596, 47432, 11988, 43496, 30212,
                49160, 64808, 44444, 9484,  47132, 13604, 28432, 44112, 54992,
                37836, 60716, 50980, 56960, 39296, 20160, 42228, 59656, 37912,
                54252, 772,   26996, 50832, 20832, 29412, 23492, 40620, 7544,
                49532, 28188, 36812, 124,   23268, 38696, 5204,  45512, 30232,
                59380, 43028, 47900, 22304, 55972, 2100,  33032, 61824, 45580,
                52832, 23028, 42924, 14036, 55748, 28596, 352,   61576, 51796,
                38452, 16748, 57616, 27656, 46632, 23140, 55352, 17740, 63464,
                20924, 5192,
            },
            {
                23620, 14052, 9416,  25572, 64544, 38464, 52632, 49064, 39976,
                31148, 24216, 47096, 63164, 50716, 43256, 10668, 23280, 32116,
                57836, 42668, 48536, 24584, 58552, 29340, 21488, 47220, 4628,
                26708, 1800,  24128, 59392, 3540,  21828, 52928, 12432, 34688,
                7912,  59524, 24112, 52904, 9984,  38952, 60724, 34792, 51816,
                8600,  46276, 32096, 49660, 18268, 53136, 41832, 25528, 35676,
                6832,  55000, 45116, 63432, 4076,  32248, 54916, 60232, 5496,
                18892, 34300, 27360, 4224,  39000, 58976, 7280,  19828, 3404,
                9348,  42432, 1252,  27948, 16520, 48076, 30628, 26304, 11760,
                19032, 43924, 61120, 36364, 57588, 10732, 51940, 59784, 32696,
                12512, 63680, 46500, 30792, 52912, 11256, 34844, 56516, 21284,
                24972, 11564, 37684, 15296, 59700, 39204, 12960, 49176, 3840,
                30196, 5968,  59520, 47092, 6868,  36864, 42332, 4996,  26284,
                22868, 44200, 9076,  36632, 6412,  62176, 42192, 35212, 46208,
                8388,  58048,
            },
            {
                27912, 37312, 48788, 42912, 32264, 15884, 3940,  60096, 17712,
                6336,  55648, 34744, 1828,  17052, 38900, 3916,  46120, 13064,
                61832, 2136,  38624, 11720, 63836, 420,   40344, 15556, 57420,
                43304, 62008, 11544, 34120, 49868, 37100, 16448, 56892, 30276,
                39228, 15220, 4920,  31988, 27424, 14552, 7100,  55276, 25968,
                63056, 21496, 13360, 39336, 10496, 63848, 31332, 284,   51248,
                40584, 15712, 29552, 10924, 42096, 22184, 8148,  36528, 52460,
                23448, 11124, 53500, 16656, 56268, 22336, 35384, 61192, 30220,
                47216, 20900, 35216, 11268, 62324, 5900,  1888,  50416, 65368,
                32072, 23604, 15624, 5716,  31520, 43536, 1040,  16208, 26672,
                3764,  54020, 17532, 6644,  62032, 42884, 16272, 65448, 7160,
                50640, 1000,  63472, 31092, 9092,  43548, 24432, 36208, 21016,
                57540, 37784, 18108, 25420, 63524, 22096, 50300, 32824, 54648,
                63928, 2344,  19140, 60300, 50684, 14244, 29376, 4592,  50168,
                32000, 40752,
            },
            {
                544,   62572, 56456, 5028,  54000, 8356,  24516, 35288, 57196,
                46096, 15000, 41824, 20512, 57684, 25768, 60940, 53740, 18512,
                27112, 21596, 51096, 31340, 43892, 52688, 9800,  60732, 31868,
                17952, 38224, 54632, 45948, 5444,  27180, 43728, 24544, 3144,
                63276, 49184, 55872, 41860, 65372, 20260, 43796, 30080, 16412,
                41128, 4860,  58852, 44156, 6920,  22804, 46708, 19984, 58948,
                9204,  32788, 48124, 19516, 59112, 50056, 15328, 28252, 2752,
                46180, 62416, 37016, 31032, 49720, 2260,  40816, 52544, 15672,
                63648, 25620, 58632, 52200, 43452, 23884, 54928, 14860, 44984,
                6816,  56804, 49488, 41464, 63912, 19376, 48052, 38504, 56492,
                44624, 24056, 41136, 33804, 50692, 20368, 26268, 48212, 40356,
                35624, 18452, 52820, 26724, 46752, 17856, 54172, 63328, 10772,
                15832, 51712, 1364,  33864, 40600, 12392, 17020, 8760,  46312,
                13336, 40292, 48604, 24864, 33208, 2848,  21356, 59308, 11432,
                16484, 52532,
            },
            {
                19812, 12340, 30176, 21816, 40108, 61616, 29496, 43540, 21368,
                10224, 27956, 53876, 31560, 5552,  36476, 8668,  33780, 40872,
                5976,  55244, 14500, 35268, 17596, 26208, 37540, 23764, 49700,
                7920,  29012, 20852, 15372, 63468, 9084,  60140, 47560, 10584,
                21544, 45484, 18368, 324,   35304, 57820, 4104,  50224, 11116,
                47596, 33136, 24732, 54060, 29160, 57184, 37020, 12612, 27836,
                65360, 23272, 56788, 5700,  34984, 25456, 64492, 39748, 57828,
                13716, 41408, 7740,  60380, 25564, 12248, 45824, 10056, 33464,
                4904,  39844, 13684, 7664,  32828, 37560, 58968, 21132, 34752,
                39236, 10444, 28972, 2788,  25584, 13144, 61472, 8124,  35608,
                20016, 10024, 60016, 13788, 28580, 3356,  9216,  32236, 13532,
                58592, 44848, 4468,  34092, 61152, 6340,  80,    31908, 26524,
                47568, 43424, 28876, 10004, 53724, 61168, 30376, 57564, 20168,
                28220, 56120, 31112, 10840, 53540, 43576, 56248, 38520, 26868,
                65068, 44284,
            },
            {
                7652,  47452, 34564, 17080, 46152, 11352, 50776, 628,   65128,
                37616, 3136,  60680, 12304, 45676, 50012, 22136, 29124, 48992,
                64528, 44888, 8040,  60092, 5248,  56320, 3208,  45776, 12876,
                64756, 984,   41704, 51284, 30692, 39368, 18964, 32940, 52656,
                28116, 37008, 8196,  25536, 12156, 46528, 22980, 61612, 37140,
                1232,  64460, 18800, 35668, 2052,  16264, 50160, 5920,  39468,
                44528, 2440,  37428, 51748, 13012, 1224,  45052, 18468, 32776,
                49372, 21232, 448,   43940, 19192, 65028, 27896, 57316, 23512,
                50104, 55676, 18848, 29524, 47696, 17088, 9088,  28332, 3384,
                60652, 53304, 17904, 37436, 54696, 33740, 22592, 52796, 29504,
                64760, 47656, 1268,  39348, 58100, 46176, 61140, 53820, 2432,
                28760, 20928, 56228, 12484, 23036, 40952, 50620, 59024, 39036,
                7548,  65188, 56304, 20076, 48020, 5528,  43556, 37608, 1396,
                35248, 5988,  15864, 64744, 36896, 18076, 8500,  47628, 23052,
                3600,  36016,
            },
            {
                24448, 51924, 58732, 2072,  27056, 57528, 19268, 13828, 32688,
                48200, 51824, 23108, 39536, 63888, 15920, 58916, 11284, 632,
                17004, 36900, 29804, 22888, 47412, 62936, 33364, 20280, 53180,
                35788, 25848, 56576, 10920, 23556, 2236,  55056, 6296,  41504,
                13272, 60472, 30972, 54016, 50756, 39168, 32384, 17524, 27512,
                56352, 14304, 51116, 8772,  60888, 41616, 32040, 59884, 53348,
                14496, 18540, 26864, 62956, 41740, 30984, 54392, 10104, 4756,
                24704, 29600, 54896, 15012, 33112, 38120, 5712,  17344, 43004,
                668,   36156, 45376, 64452, 2860,  53624, 40464, 63416, 45936,
                13800, 22392, 42996, 58724, 8964,  46164, 2088,  41316, 5380,
                14584, 31968, 25552, 54580, 22144, 35728, 17848, 38752, 24280,
                62964, 42244, 7880,  38236, 48500, 29456, 14656, 19304, 34368,
                3080,  22696, 13980, 36084, 2500,  27032, 23396, 59956, 48956,
                62724, 42700, 21876, 45860, 76,    28336, 60924, 15068, 32948,
                54184, 60436,
            },
            {
                28304, 14456, 39044, 64376, 6516,  35976, 41540, 54836, 25824,
                5628,  42756, 18508, 7452,  27304, 2816,  54424, 38988, 43436,
                25288, 58224, 52044, 10824, 41076, 28016, 15144, 6972,  42760,
                58700, 4384,  32792, 45168, 59140, 35388, 48684, 15840, 64700,
                1624,  56580, 43208, 15696, 63652, 3244,  9652,  53316, 7248,
                42800, 30724, 38572, 46024, 26884, 19620, 10748, 25128, 3424,
                34380, 55944, 46388, 7808,  16544, 22020, 61248, 36788, 52112,
                63500, 35220, 58216, 47124, 8988,  53704, 49232, 30788, 62808,
                12664, 27584, 22288, 6164,  57772, 25352, 12140, 50148, 32500,
                26280, 48292, 4736,  30280, 15780, 60556, 27472, 49412, 18304,
                57824, 37696, 51200, 4724,  11832, 43776, 6752,  50484, 10800,
                15320, 31600, 52020, 17256, 64800, 9904,  56520, 25152, 45296,
                53924, 41516, 31544, 61632, 39560, 51780, 16112, 8204,  12924,
                25980, 52348, 9656,  58680, 24392, 41400, 50540, 5424,  40372,
                10656, 42364,
            },
            {
                3064,  32460, 9840,  43952, 23832, 49388, 30328, 8896,  62280,
                15652, 35520, 30532, 56416, 44212, 34816, 19180, 31504, 61712,
                13616, 4348,  33088, 18972, 2280,  50572, 55120, 38748, 29720,
                16496, 48400, 19384, 13964, 7388,  61956, 21120, 27356, 37748,
                24040, 33676, 5864,  22276, 29040, 19872, 45104, 60180, 24340,
                49076, 20932, 11972, 3120,  52484, 63804, 44256, 48464, 21908,
                62108, 30516, 11232, 38656, 58832, 48828, 27696, 7032,  42448,
                16240, 11384, 3944,  40444, 23816, 59596, 2140,  21352, 39172,
                58900, 51812, 10612, 41684, 31820, 20668, 36416, 84,    19012,
                6544,  62356, 34912, 51468, 20904, 39316, 11488, 55724, 23828,
                43864, 9264,  61796, 16856, 29800, 63564, 26348, 57012, 34308,
                47660, 60000, 1456,  26428, 35256, 43048, 3744,  60528, 12864,
                8984,  50048, 17424, 57780, 11452, 45916, 33708, 55028, 40868,
                18336, 32492, 3948,  49272, 34016, 12624, 63624, 30260, 20896,
                57208, 17796,
            },
            {
                61380, 50336, 54440, 20108, 15392, 60844, 3340,  46564, 22464,
                58660, 53168, 10284, 61284, 13432, 23936, 47236, 6332,  50468,
                21268, 56448, 39996, 64188, 45252, 12240, 22320, 62140, 9596,
                24532, 61104, 40084, 53440, 29296, 41848, 11272, 44408, 51596,
                18696, 46820, 10892, 49008, 36516, 58476, 13476, 34196, 39776,
                4476,  57476, 61384, 23332, 33640, 15064, 5012,  36412, 8496,
                40392, 49564, 464,   24652, 33268, 3896,  12364, 46848, 20680,
                59780, 26760, 50948, 20140, 28860, 13780, 34996, 46256, 8288,
                16324, 33264, 48060, 61348, 14756, 44540, 52052, 60076, 41524,
                54528, 38468, 12508, 1744,  65124, 44564, 6780,  30844, 63424,
                3460,  33860, 21000, 41472, 49792, 32808, 580,   19668, 40772,
                4428,  22232, 44952, 55520, 5476,  21092, 51628, 37332, 30656,
                62728, 27392, 4304,  24296, 7284,  29656, 63100, 22088, 4684,
                59248, 29084, 39440, 55808, 19448, 7376,  53148, 36224, 1168,
                48924, 45092,
            },
            {
                22400, 6040,  29940, 36672, 56504, 11476, 33484, 18380, 40044,
                4244,  26716, 45596, 868,   38360, 51720, 65396, 9996,  27536,
                35096, 46496, 8664,  28932, 24900, 59420, 34764, 132,   44228,
                52376, 6072,  36296, 1796,  24804, 57084, 32104, 3660,  8364,
                58688, 40152, 62988, 2632,  55016, 41328, 26704, 568,   64832,
                15984, 35948, 7680,  54284, 39892, 29928, 58328, 54856, 28348,
                15936, 19832, 60900, 55088, 43644, 18416, 57292, 30720, 39456,
                1012,  44964, 36936, 64384, 6144,  43456, 61428, 26192, 54176,
                3796,  24800, 38100, 1652,  55196, 7360,  27564, 10428, 23872,
                16024, 57720, 28544, 47360, 24884, 34024, 17068, 37024, 51828,
                13472, 28040, 47196, 56404, 7892,  14292, 45904, 54008, 64512,
                9196,  30024, 37820, 13452, 62188, 32716, 47332, 16180, 1220,
                20588, 47804, 35088, 44244, 53164, 18672, 516,   48276, 36460,
                45332, 10808, 61980, 16228, 27488, 44104, 22956, 58596, 15940,
                26720, 37668,
            },
            {
                13036, 63556, 47244, 404,   27576, 40928, 52288, 64044, 37076,
                12644, 49980, 21220, 33184, 16396, 29396, 3568,  55280, 42312,
                17332, 1236,  53068, 15528, 38016, 4568,  18256, 48860, 31168,
                12740, 20404, 47036, 65276, 15316, 50028, 17864, 63456, 54264,
                29352, 14648, 25016, 31956, 17416, 8048,  52512, 47956, 19196,
                30152, 43296, 27792, 46844, 18296, 872,   42492, 12708, 64880,
                44972, 52536, 6612,  13796, 36064, 50544, 65160, 23424, 54544,
                33380, 9160,  17756, 55980, 32364, 52224, 10528, 18864, 40964,
                57088, 65300, 19884, 29368, 17440, 34724, 63956, 47100, 31408,
                6624,  43740, 18500, 9036,  53644, 57212, 2776,  47848, 20480,
                40112, 59132, 2044,  24896, 38512, 60700, 23032, 36924, 16592,
                27164, 58460, 49036, 18184, 24672, 8412,  27876, 42156, 54864,
                39396, 58736, 14772, 64080, 37960, 42188, 56808, 26512, 14476,
                53908, 24000, 1620,  35924, 47824, 3284,  39008, 11372, 62060,
                33632, 8508,
            },
            {
                40404, 25460, 17356, 43108, 59628, 7416,  24384, 1776,  28420,
                55492, 41788, 7020,  62752, 56992, 40812, 22244, 13080, 37360,
                58868, 23824, 62200, 32516, 50100, 57388, 42064, 26404, 63176,
                56252, 34136, 27832, 10108, 38980, 5540,  46308, 36656, 22740,
                1260,  34976, 50200, 44436, 61528, 11880, 22920, 32984, 6416,
                55572, 50552, 13752, 63544, 9900,  51644, 21244, 26140, 5708,
                34232, 23000, 29760, 41564, 26388, 2832,  10028, 15348, 5240,
                49812, 60896, 12532, 47728, 3168,  16076, 37308, 49920, 31300,
                6788,  45100, 9668,  49628, 40368, 57632, 2376,  21004, 37680,
                50356, 62864, 33160, 22332, 41732, 13932, 60356, 26436, 10216,
                54180, 5844,  35448, 16376, 52576, 11192, 6048,  31028, 43564,
                11984, 52416, 6548,  40544, 53712, 59232, 11860, 64280, 23672,
                7092,  10708, 32200, 2652,  21588, 12684, 8872,  32772, 65408,
                6320,  42748, 50436, 12952, 64000, 56496, 29700, 46304, 5104,
                51844, 55288,
            },
            {
                58828, 4228,  34476, 53808, 14012, 20876, 48480, 45264, 15880,
                31756, 59984, 18008, 47748, 26052, 8436,  45840, 60584, 31024,
                48816, 11592, 6760,  40764, 20812, 6444,  10984, 36688, 15852,
                2644,  41084, 59280, 22356, 54792, 33544, 26584, 13092, 42664,
                9608,  60712, 20084, 4212,  28548, 37660, 57804, 45480, 61948,
                10552, 21712, 2672,  25096, 37764, 59324, 32328, 49352, 39124,
                2216,  57200, 47816, 8576,  62404, 32276, 45292, 37736, 28396,
                42292, 25368, 21692, 29888, 41784, 24552, 63436, 1876,  27816,
                14020, 34932, 23084, 53360, 12324, 25540, 43264, 14468, 55904,
                3616,  11792, 59160, 812,   38136, 30264, 7304,  42896, 32712,
                62544, 44784, 19364, 65296, 42612, 26892, 58044, 50740, 2892,
                61716, 20748, 31652, 328,   44584, 36196, 3412,  33660, 17632,
                45716, 51076, 25188, 55744, 28716, 50736, 59612, 40232, 20604,
                29788, 17824, 59988, 31564, 21176, 7668,  16956, 24848, 42304,
                19672, 30864,
            },
            {
                44932, 21684, 50060, 9108,  38716, 61640, 32820, 57496, 9920,
                5576,  23820, 35576, 11500, 2608,  53984, 34160, 18812, 5480,
                26608, 44000, 56092, 14088, 29896, 65000, 44856, 54444, 23152,
                8316,  50156, 17132, 43724, 724,   62304, 19928, 57848, 52496,
                47992, 38660, 56688, 7212,  51732, 42592, 14824, 1756,  26128,
                38788, 41644, 60288, 34192, 5212,  43988, 15452, 62552, 10852,
                18716, 61488, 36928, 16744, 21420, 58208, 51576, 18592, 56244,
                62652, 2032,  36072, 58876, 53092, 8116,  56600, 21992, 47228,
                59548, 42288, 62776, 5220,  30808, 61144, 8068,  51832, 29252,
                24436, 35860, 48724, 27600, 51952, 64088, 17836, 49472, 23636,
                14732, 29172, 8544,  31496, 48704, 1300,  34264, 19272, 46644,
                39032, 34904, 25648, 63120, 15268, 21784, 48760, 52256, 57572,
                28388, 37052, 61052, 6432,  44860, 34732, 17016, 1984,  47524,
                52440, 38316, 9200,  25944, 41928, 53044, 33300, 49640, 65136,
                1408,  15292,
            },
            {
                37908, 11584, 64400, 26484, 30248, 3068,  17812, 36608, 51060,
                64920, 44508, 53104, 39024, 29948, 63692, 15216, 50968, 1936,
                64340, 20000, 36216, 46936, 25404, 51872, 3164,  28624, 60832,
                46432, 32440, 4988,  52084, 29808, 11392, 40848, 6572,  30848,
                16648, 27052, 13736, 23880, 65348, 31076, 21028, 36032, 53800,
                17396, 31388, 48912, 56756, 19432, 28556, 7312,  55364, 46488,
                25680, 42472, 12120, 54408, 980,   39728, 24796, 11808, 7180,
                46504, 14596, 39956, 4672,  18968, 45192, 33752, 39152, 10900,
                52156, 17128, 744,   37728, 47720, 20236, 33440, 39224, 64996,
                45516, 19636, 15948, 7968,  45152, 21432, 4376,  56156, 36312,
                196,   51272, 57116, 22108, 37420, 14912, 63528, 24220, 8020,
                13840, 54924, 48136, 10620, 57160, 30104, 39828, 5372,  13636,
                940,   42428, 19332, 15224, 48556, 4664,  62716, 25780, 11328,
                57984, 3520,  34684, 47108, 440,   58692, 13676, 37148, 10096,
                28416, 48040,
            },
            {
                60332, 33368, 1884,  56700, 44384, 51980, 12172, 42344, 25732,
                19780, 152,   14232, 57736, 20316, 43292, 24316, 37688, 41812,
                53572, 33340, 9148,  676,   58752, 17252, 34508, 13288, 38796,
                19240, 26272, 64176, 14164, 36484, 48540, 60036, 24528, 3968,
                63628, 33148, 46892, 40624, 10996, 3320,  59444, 46660, 8296,
                64368, 4752,  14768, 45208, 11204, 52748, 23380, 36100, 30932,
                4636,  52232, 32952, 28028, 49084, 5876,  64048, 30452, 34640,
                20684, 50628, 31880, 64932, 11640, 28412, 15168, 62240, 4892,
                20720, 32784, 27344, 55068, 15256, 58712, 2924,  17644, 10460,
                5648,  41272, 61772, 57440, 12652, 33892, 40508, 60868, 10988,
                46200, 39400, 61312, 12500, 4732,  55684, 41008, 52932, 29276,
                59708, 3532,  17556, 41688, 7164,  26488, 19028, 62612, 31912,
                22512, 65020, 8948,  39624, 30484, 23320, 54460, 36532, 44040,
                22084, 15508, 53964, 62352, 18880, 39904, 4824,  61048, 23148,
                56848, 7208,
            },
            {
                25092, 19052, 40804, 15808, 23232, 6648,  62868, 28924, 4388,
                40300, 31160, 49656, 27172, 6484,  48344, 10052, 58056, 7188,
                28980, 15700, 50208, 61600, 37968, 21940, 42860, 5580,  53000,
                57448, 9732,  40036, 23956, 55596, 17792, 8540,  45624, 38028,
                54344, 28,    19400, 55228, 34520, 49480, 18404, 28000, 12596,
                51144, 23124, 29392, 1944,  38252, 64180, 41444, 260,   58244,
                21140, 14752, 7596,  60624, 44876, 35868, 16040, 43404, 53792,
                59816, 9804,  26412, 54760, 48564, 37056, 1052,  50116, 25324,
                41232, 57768, 46184, 9452,  23508, 44056, 49772, 27764, 56712,
                53036, 31500, 26232, 37096, 2060,  54152, 24840, 15520, 28436,
                20272, 3488,  25840, 43320, 47644, 27532, 17680, 10388, 33072,
                42900, 22984, 65120, 36888, 52900, 45424, 59632, 9392,  43692,
                47984, 35136, 56288, 52404, 12072, 58764, 41348, 7696,  31348,
                64396, 27640, 44944, 12216, 24232, 29364, 51108, 20340, 41476,
                32060, 52456,
            },
            {
                43376, 6060,  54652, 47624, 35492, 58004, 21136, 46036, 54876,
                59212, 11064, 62560, 34668, 3264,  61056, 30900, 12964, 21728,
                62584, 24524, 40476, 12116, 30944, 7832,  49612, 63820, 30164,
                976,   34252, 44552, 59544, 2544,  28840, 34020, 51736, 21376,
                12388, 43848, 8860,  61972, 25660, 6172,  43308, 56020, 33036,
                37484, 42164, 54348, 59004, 26204, 48312, 13396, 17528, 49956,
                44124, 64748, 40128, 23656, 19252, 10300, 56864, 4576,  23012,
                96,    41756, 17268, 6108,  23368, 42328, 58296, 31252, 54316,
                7368,  12936, 64728, 35740, 5840,  40512, 63172, 35040, 13668,
                47160, 3504,  21984, 50008, 43248, 29784, 8684,  48340, 52752,
                64348, 34580, 17140, 53920, 7168,  35632, 61800, 1816,  45932,
                50936, 5564,  28164, 32480, 1372,  12784, 34600, 24888, 54300,
                15772, 6140,  26804, 21288, 33768, 172,   18680, 14116, 51420,
                4248,  39252, 32844, 5744,  57432, 8936,  35024, 46628, 2452,
                16896, 12660,
            },
            {
                49432, 62028, 27852, 31440, 10000, 39260, 14744, 33792, 8408,
                16768, 37504, 22112, 45400, 18216, 39992, 55624, 35172, 44776,
                47852, 3328,  55532, 19036, 45312, 57860, 25240, 15972, 47144,
                22560, 13004, 50800, 6096,  20948, 42952, 64828, 15088, 27372,
                60820, 35664, 29284, 50924, 16296, 39144, 63512, 22052, 620,
                61188, 6292,  17912, 35052, 20848, 7960,  33100, 61392, 28880,
                9516,  35028, 54980, 1704,  31384, 59504, 27068, 47724, 39040,
                29060, 62796, 45968, 33620, 61156, 20188, 8652,  16872, 45416,
                37984, 29336, 18576, 50564, 26448, 12008, 20880, 240,   24060,
                38456, 59180, 9788,  14984, 64676, 18984, 58544, 37656, 6556,
                41996, 31276, 9708,  60144, 23136, 30544, 56924, 21216, 38140,
                15724, 58176, 19760, 47436, 56728, 16812, 51020, 2584,  38592,
                29564, 58468, 3672,  45352, 63708, 49648, 27900, 61356, 46744,
                10132, 20072, 55436, 49116, 42684, 63112, 14696, 25748, 55736,
                63784, 36596,
            },
            {
                248,   21780, 13848, 3680,  64124, 48876, 1284,  61116, 23912,
                47776, 52240, 6904,  54664, 14724, 25900, 51188, 1548,  17296,
                59464, 32932, 5816,  27084, 52864, 2820,  39584, 10268, 36732,
                55976, 62928, 17244, 31264, 48108, 11560, 39028, 3060,  55916,
                48432, 5076,  22380, 57840, 1920,  30592, 11456, 47944, 15404,
                25312, 49852, 10448, 63144, 45536, 4128,  56108, 22300, 38928,
                3192,  26108, 16528, 46244, 50312, 37476, 13592, 52368, 18840,
                11580, 36452, 56192, 3312,  13032, 52184, 27664, 63948, 3524,
                22916, 60752, 1504,  53476, 32192, 60396, 55252, 30564, 43592,
                62468, 17920, 33624, 54940, 4804,  32452, 45372, 1056,  22420,
                56640, 13408, 50856, 38720, 45032, 14236, 49716, 4532,  26092,
                62700, 12092, 40448, 9168,  23812, 62320, 41884, 21116, 61020,
                11928, 50084, 41620, 17012, 10312, 36932, 43364, 23744, 33572,
                59484, 26160, 1036,  17204, 37112, 31064, 53248, 4144,  44180,
                10376, 29208,
            },
            {
                53056, 39944, 45028, 55900, 19504, 25476, 53084, 30228, 43060,
                3112,  26904, 32664, 41244, 63452, 8584,  21240, 38384, 27848,
                11128, 42260, 36720, 63208, 14440, 32520, 60320, 20456, 28496,
                42200, 7884,  26372, 36056, 54748, 61492, 25020, 7780,  32164,
                18032, 38480, 42688, 13480, 46260, 36336, 53676, 7816,  40668,
                57728, 44128, 31040, 14120, 28284, 40788, 51804, 11692, 46944,
                58928, 51228, 12164, 63212, 7372,  22004, 2616,  65252, 32612,
                6792,  50212, 15596, 25924, 39692, 47376, 35768, 11320, 33004,
                49100, 42532, 14844, 38800, 4148,  47420, 9260,  16416, 50360,
                7868,  27872, 46156, 40020, 23064, 12212, 26808, 61624, 17568,
                47688, 27808, 5432,  20048, 648,   63020, 10880, 42024, 55124,
                34812, 6588,  53656, 30392, 37952, 4836,  27464, 46004, 7836,
                19372, 35660, 24712, 60368, 30112, 7128,  57044, 2624,  16232,
                52672, 42040, 63812, 23376, 12856, 6916,  19564, 59760, 38332,
                32796, 18168,
            },
            {
                25000, 61232, 33840, 7728,  37036, 41948, 11280, 17716, 36288,
                65056, 13576, 58508, 776,   29840, 47344, 57640, 4768,  65312,
                53912, 20844, 8612,  48192, 23164, 43528, 51280, 5324,  53744,
                2292,  46008, 58820, 40828, 1244,  15992, 51052, 45148, 58708,
                10704, 25776, 52524, 61068, 20176, 24676, 59472, 32592, 27688,
                19492, 35804, 3128,  53396, 58528, 16964, 24904, 35512, 6428,
                31480, 19852, 41520, 34176, 29484, 42988, 55856, 40212, 24688,
                59620, 43580, 21708, 31356, 57900, 1788,  43792, 59808, 55264,
                19824, 8504,  56996, 24108, 44184, 19220, 35872, 41108, 4872,
                56964, 36648, 1328,  63048, 52064, 49024, 41588, 34072, 9396,
                36816, 63772, 43076, 58432, 25284, 36236, 32036, 17968, 28648,
                48116, 21688, 44424, 64120, 50288, 14372, 33820, 55912, 31308,
                64696, 53596, 1332,  47416, 13516, 54512, 20728, 40216, 5448,
                35820, 11576, 29120, 45816, 58836, 50452, 27368, 47140, 22564,
                7860,  57932,
            },
            {
                4968,  11664, 16432, 50796, 27616, 60272, 4784,  57652, 49392,
                9344,  19860, 44756, 23544, 11520, 34408, 43480, 14132, 31840,
                49680, 15656, 58452, 29936, 184,   11768, 34896, 18016, 61904,
                31768, 13896, 19488, 9744,  22996, 30084, 35392, 20528, 41044,
                64016, 2836,  33588, 6092,  9612,  41304, 17412, 2472,  64896,
                54768, 9184,  22864, 48100, 37824, 1736,  65504, 43880, 15360,
                62220, 56412, 23780, 3880,  60976, 17844, 9032,  14980, 46656,
                3980,  53052, 10080, 64244, 7444,  18372, 24356, 14384, 5116,
                26812, 35208, 62880, 29736, 12604, 58260, 65184, 22648, 52612,
                13384, 25084, 20004, 10508, 29268, 6496,  59088, 14836, 55128,
                24160, 3236,  33128, 54192, 15836, 51900, 46088, 8264,  59384,
                3528,  15080, 25640, 492,   18828, 60172, 10916, 49092, 4368,
                15312, 39896, 21972, 32980, 38668, 26380, 50904, 65228, 30856,
                48508, 57580, 8104,  38916, 33256, 1672,  40996, 15876, 65412,
                50956, 42240,
            },
            {
                48356, 31000, 57428, 22732, 46080, 14448, 32324, 21968, 38968,
                28508, 53900, 35560, 50576, 60040, 18632, 53280, 23216, 40768,
                6708,  26212, 45008, 38868, 54568, 64616, 26676, 40680, 47368,
                24304, 38484, 65180, 51772, 46912, 58016, 4896,  13148, 28528,
                49500, 16284, 55768, 29628, 62980, 48288, 51776, 5704,  45736,
                13536, 39540, 61436, 7068,  32916, 20024, 9412,  54476, 27228,
                576,   38108, 9908,  45232, 52592, 26024, 48528, 30868, 35640,
                19892, 28352, 41252, 34768, 48616, 53620, 38376, 30428, 50944,
                39832, 46700, 17260, 6240,  49836, 26148, 1840,  33508, 29448,
                61080, 48360, 43764, 57696, 16900, 38336, 20884, 2412,  45728,
                50568, 18912, 11956, 28848, 40352, 6228,  65356, 23208, 37740,
                50996, 31872, 58288, 35776, 7504,  43080, 22620, 37272, 25904,
                43996, 9296,  57364, 5616,  62952, 18100, 9712,  44644, 13192,
                22760, 18804, 3712,  54808, 20652, 62124, 10616, 31272, 3492,
                13072, 35524,
            },
            {
                63084, 2348,  39176, 6512,  64484, 696,   55204, 43544, 6844,
                62624, 4400,  15304, 39564, 3632,  7660,  28244, 62128, 1344,
                36152, 60648, 18360, 6216,  21892, 49376, 15960, 7440,  10692,
                55796, 4500,  34332, 27264, 6948,  42632, 62688, 54872, 436,
                23540, 46596, 36740, 44044, 26840, 14820, 34612, 38124, 24076,
                29952, 51048, 16512, 26784, 46084, 59820, 30876, 41320, 49492,
                18464, 47280, 28984, 14560, 35956, 58072, 1240,  63700, 55004,
                12732, 60560, 692,   16104, 27036, 61684, 9748,  21020, 63656,
                388,   11516, 53224, 32296, 37548, 54668, 10588, 46320, 18340,
                8468,  39596, 31888, 4272,  35012, 53524, 65132, 26328, 30860,
                40048, 62444, 8156,  48628, 21792, 2016,  56136, 13308, 43116,
                61376, 9680,  40920, 47672, 28384, 57172, 52992, 2812,  63148,
                18732, 51740, 27820, 49552, 43704, 2196,  34488, 58808, 27460,
                38204, 61732, 51368, 14944, 25892, 43984, 56828, 37068, 53816,
                28812, 20808,
            },
            {
                26496, 43400, 53268, 18564, 29424, 49804, 35168, 16488, 24852,
                46492, 31800, 56888, 26356, 64432, 47812, 37644, 12308, 46532,
                51912, 10484, 33460, 56628, 3904,  37192, 30892, 58152, 50408,
                20636, 44108, 16176, 54052, 12108, 37572, 18480, 32740, 39888,
                59216, 7940,  20856, 12316, 4832,  56944, 21312, 58788, 10728,
                62676, 208,   41852, 57148, 11176, 51476, 23436, 4728,  13140,
                33328, 53616, 59480, 6124,  20860, 11340, 39496, 22616, 7024,
                44332, 38144, 56644, 22788, 45732, 5724,  42648, 33576, 57816,
                44472, 23480, 60212, 3784,  21324, 43140, 15760, 59144, 36308,
                3512,  54260, 15072, 61680, 23964, 46832, 8916,  42744, 13648,
                4800,  55764, 38052, 59608, 44336, 31116, 35032, 26544, 19552,
                2504,  17136, 54888, 12516, 20616, 39568, 16152, 29792, 46444,
                34404, 59096, 12080, 31240, 15188, 24328, 47968, 6936,  53780,
                1392,  42348, 30256, 46888, 35376, 5128,  18140, 23564, 60736,
                45620, 17216,
            },
            {
                55324, 10140, 36448, 24364, 12568, 40524, 9188,  61292, 51008,
                12152, 19128, 42528, 10084, 21512, 31044, 16236, 56104, 20064,
                29576, 24244, 63972, 41156, 12908, 45672, 61300, 1472,  35612,
                28660, 60728, 2172,  32048, 22044, 49180, 25644, 9700,  14932,
                53332, 30308, 65376, 39272, 52620, 1360,  31200, 44380, 18284,
                48524, 33648, 20864, 36264, 3584,  15708, 39284, 56112, 63992,
                7724,  22340, 40588, 62804, 50376, 43552, 33084, 27396, 50744,
                18228, 30420, 10648, 51896, 31936, 13764, 54640, 3508,  17776,
                28344, 14512, 40960, 48228, 63924, 7408,  39132, 27100, 50848,
                64140, 21112, 28104, 49916, 12476, 840,   32732, 51168, 59952,
                22484, 34144, 26084, 14572, 18128, 61964, 10760, 46920, 54048,
                29408, 34644, 23800, 64488, 1728,  32992, 61448, 6504,  13392,
                40892, 320,   21064, 37164, 60704, 55468, 39676, 20596, 16332,
                33176, 9480,  24180, 11964, 64500, 52336, 8740,  49268, 720,
                39616, 6800,
            },
            {
                14544, 47384, 58312, 32888, 62328, 45316, 20688, 27096, 2824,
                59172, 34156, 1044,  50000, 44364, 53844, 2720,  41296, 59848,
                8180,  44140, 15248, 52908, 27668, 22972, 17832, 42420, 24908,
                12536, 40376, 57320, 46228, 63580, 3452,  59728, 51360, 45060,
                35068, 3216,  17972, 49140, 25068, 61976, 40700, 6736,  27776,
                55784, 6160,  25480, 64520, 44824, 27824, 61040, 19224, 35544,
                44116, 26604, 2276,  31568, 17460, 4404,  54504, 15552, 62036,
                2328,  47024, 64848, 4168,  36280, 59400, 25576, 49872, 37500,
                7704,  55932, 35564, 25464, 18448, 30120, 55720, 528,   23640,
                11764, 40824, 5916,  44308, 37084, 19876, 56672, 17236, 28740,
                10440, 46616, 340,   53648, 6928,  40856, 50660, 4776,  38764,
                62900, 49300, 5260,  44836, 51624, 10220, 25416, 49364, 56380,
                23328, 8260,  62556, 42692, 3448,  10464, 28988, 59756, 45284,
                63608, 49784, 57036, 4020,  39472, 32344, 28408, 42148, 11816,
                33196, 59340,
            },
            {
                30064, 22000, 1484,  7828,  52396, 4204,  56748, 31080, 38508,
                53436, 22628, 63440, 36604, 13836, 58772, 33944, 26120, 4852,
                35148, 50428, 956,   32144, 7308,  48308, 54804, 9432,  64448,
                52120, 7040,  18344, 36772, 8644,  30008, 41816, 20584, 5468,
                26548, 60324, 42776, 8552,  33020, 16684, 46804, 9888,  60196,
                14520, 42864, 54204, 12292, 50252, 7952,  30412, 1076,  48152,
                11132, 15932, 55472, 37376, 24024, 47620, 58372, 8796,  40948,
                34440, 21124, 26332, 42388, 16860, 20292, 40696, 11228, 64552,
                46572, 31744, 1600,  9540,  51556, 12768, 61184, 45012, 34416,
                47668, 31424, 60304, 54956, 25652, 63840, 40968, 5568,  37692,
                63124, 19996, 49632, 36248, 29624, 24444, 56856, 16136, 21744,
                8616,  42220, 26124, 15352, 36204, 58796, 43760, 18492, 37492,
                53868, 26848, 47640, 52608, 18884, 35900, 51064, 4200,  12640,
                26200, 37188, 17564, 21900, 59304, 14652, 54944, 20392, 63552,
                25236, 51260,
            },
            {
                38612, 64740, 41912, 28296, 17560, 36848, 13968, 48156, 7224,
                41892, 15776, 27972, 5728,  24632, 8732,  18092, 49004, 63404,
                22056, 38840, 18816, 57504, 62384, 36300, 2940,  32876, 15460,
                30304, 48636, 23444, 27016, 13876, 56188, 16736, 37944, 62904,
                12044, 22660, 56464, 37404, 13540, 53896, 22268, 36108, 51356,
                38604, 30804, 1820,  17608, 40276, 34660, 53864, 24556, 38444,
                52268, 61500, 46248, 8164,  65168, 29452, 12272, 36492, 24664,
                53552, 13304, 57520, 6940,  47872, 61920, 1808,  30192, 22524,
                53520, 19624, 62260, 58388, 43520, 32728, 5404,  20124, 15308,
                58200, 2992,  18428, 10532, 7140,  30116, 14032, 45808, 54120,
                3368,  42932, 58420, 12320, 64752, 44716, 1848,  33224, 58620,
                13040, 31244, 60340, 55884, 7584,  21412, 30996, 4812,  63300,
                33228, 15548, 30404, 6180,  25100, 64940, 32208, 22480, 41584,
                7556,  54660, 72,    42772, 46076, 26008, 5932,  36488, 46984,
                16632, 4064,
            },
            {
                34656, 19244, 12836, 49996, 60408, 43968, 25400, 65488, 18760,
                10636, 46960, 56796, 39088, 51392, 62212, 30036, 42928, 10600,
                13724, 54040, 45176, 25328, 11512, 40104, 21076, 58480, 44224,
                37980, 292,   61328, 52916, 44960, 34384, 1400,  47920, 54044,
                31760, 46460, 780,   28064, 58704, 4320,  64072, 26072, 3376,
                19704, 62832, 23712, 47808, 60264, 21196, 14276, 58032, 5224,
                32240, 20744, 28240, 13624, 41528, 116,   20448, 45308, 61004,
                3752,  49704, 29020, 38000, 9688,  55480, 34056, 44668, 4648,
                13300, 42044, 27548, 15952, 37276, 23144, 49952, 41180, 8572,
                52888, 24188, 38864, 51332, 35356, 48712, 58800, 23476, 33548,
                26552, 15680, 31320, 22764, 9320,  18568, 37520, 27716, 52340,
                45780, 432,   19260, 39080, 29116, 48532, 41216, 12488, 1948,
                44892, 10292, 58496, 40092, 13496, 43768, 16944, 56916, 49120,
                34864, 28492, 60592, 31128, 8444,  51408, 61768, 2104,  56436,
                43816, 9420,
            },
            {
                53152, 46108, 55860, 23720, 10396, 33168, 160,   54128, 35372,
                30140, 60064, 3104,  32624, 19676, 46032, 372,   55996, 37012,
                28032, 59516, 3388,  30760, 16248, 51264, 28204, 47012, 4440,
                20040, 55892, 11080, 40156, 5668,  65016, 24472, 28712, 9072,
                40548, 16080, 51612, 20428, 44628, 30664, 48856, 11992, 33720,
                45740, 8624,  56832, 32436, 4604,  10416, 45300, 63492, 17936,
                42768, 2648,  57268, 34052, 51788, 59912, 49012, 30940, 7484,
                17240, 43608, 62692, 32184, 23112, 52160, 27120, 16408, 60384,
                50780, 38648, 6636,  47908, 2592,  54776, 62680, 26792, 36144,
                65452, 29096, 43156, 15584, 61784, 21364, 1308,  11424, 51928,
                8332,  56296, 40640, 3936,  52740, 60548, 48920, 5612,  63684,
                23572, 35176, 51344, 11312, 2852,  65076, 16480, 52088, 60120,
                22192, 49620, 35588, 1264,  55052, 47120, 8852,  2572,  62984,
                14680, 19652, 48240, 12148, 38076, 19084, 34116, 14056, 29068,
                23204, 60952,
            },
            {
                26532, 2868,  31788, 5284,  40316, 57796, 21192, 6388,  50884,
                23804, 43280, 8304,  53208, 12256, 40744, 15560, 23488, 5836,
                47044, 33316, 8944,  65484, 42752, 5492,  60660, 10264, 25704,
                63796, 34928, 17104, 31964, 21392, 50424, 12748, 58092, 19584,
                6284,  34560, 64568, 11248, 39416, 7688,  18136, 41576, 58008,
                52136, 16204, 26732, 43040, 37336, 51044, 29092, 7404,  26064,
                49424, 10204, 39360, 23252, 5772,  18144, 26168, 55848, 39808,
                33760, 10992, 19740, 900,   14748, 40228, 6264,  57392, 35504,
                24812, 10232, 56840, 21404, 29608, 11188, 17456, 4596,  45680,
                12912, 960,   57276, 32640, 4932,  27524, 44176, 39936, 63596,
                19316, 36876, 61556, 47328, 34680, 25568, 14140, 39652, 10256,
                17060, 41424, 62248, 46652, 22932, 54340, 34776, 25320, 38676,
                28784, 17348, 63828, 20544, 27804, 61228, 37756, 30660, 24648,
                39720, 53600, 5800,  65112, 24400, 58252, 41580, 10244, 50308,
                39312, 6696,
            },
            {
                57920, 14924, 37648, 63892, 48744, 15252, 28440, 60840, 39932,
                13260, 17176, 63220, 21980, 27476, 35276, 60880, 50476, 63940,
                17520, 40336, 20552, 49168, 23684, 55140, 34340, 14616, 53884,
                41972, 6396,  47828, 28132, 57016, 2548,  43900, 36664, 60992,
                49056, 54960, 24700, 2680,  33216, 61344, 54372, 23288, 424,
                29008, 39492, 12776, 2728,  65008, 22852, 56028, 41220, 36140,
                61016, 53612, 15260, 62672, 44432, 37712, 3484,  14712, 63308,
                23760, 54144, 58860, 45696, 50124, 65344, 11696, 19324, 47552,
                176,   31536, 64416, 34756, 43916, 59996, 39968, 31220, 56088,
                21584, 49144, 9696,  19424, 47164, 55020, 36104, 16968, 29732,
                49204, 1824,  13488, 28592, 6536,  43088, 20788, 57140, 29664,
                55232, 4488,  32672, 15176, 27928, 5364,  44488, 9120,  57236,
                6652,  41668, 53296, 9672,  33764, 5428,  21768, 52380, 10848,
                45256, 1836,  33536, 44068, 15628, 3360,  47472, 21212, 64136,
                32632, 18332,
            },
            {
                49420, 44620, 29016, 20216, 8424,  43020, 36128, 46812, 2368,
                27048, 55988, 37456, 48080, 2524,  54832, 7492,  32132, 26668,
                1708,  52792, 58236, 12428, 37380, 740,   18928, 39524, 29416,
                22236, 51656, 14168, 60884, 8220,  41332, 32636, 15156, 23176,
                3636,  29072, 42336, 59124, 46732, 27204, 14664, 37352, 63076,
                6932,  59976, 49248, 53672, 19144, 15028, 33344, 592,   12452,
                22148, 46676, 27320, 8360,  31748, 58108, 11804, 42636, 51400,
                2604,  27904, 38908, 5132,  25756, 36184, 43236, 29872, 61444,
                40924, 17308, 52664, 13652, 49428, 8012,  24276, 51584, 14884,
                34200, 61224, 26316, 41648, 64336, 13500, 8080,  60308, 5512,
                25140, 45456, 22204, 54356, 17404, 59284, 31960, 2688,  44532,
                48572, 26016, 8348,  56400, 37928, 59408, 19660, 32448, 14064,
                47492, 3016,  24408, 45500, 15032, 50324, 42948, 17804, 58160,
                61412, 27504, 21476, 56656, 51976, 26796, 30624, 53824, 1376,
                42732, 11180,
            },
            {
                22772, 61496, 1048,  51744, 24988, 54516, 18236, 11408, 64092,
                33628, 44676, 4788,  30796, 58636, 42564, 11368, 21040, 44516,
                14076, 34988, 7156,  27692, 45884, 31368, 63280, 48708, 8320,
                58984, 3432,  44652, 37876, 19328, 25800, 63000, 52856, 10356,
                45388, 37856, 13348, 17180, 21844, 4884,  50656, 9600,  44700,
                20524, 34956, 24504, 31236, 6232,  44452, 60404, 50316, 17192,
                31108, 5400,  35332, 55096, 20304, 50228, 28380, 35888, 21188,
                8868,  48048, 34696, 17468, 56160, 21532, 2560,  54712, 7640,
                22776, 45564, 4140,  26384, 19964, 1324,  36520, 63356, 3816,
                43408, 6840,  37528, 52804, 1932,  24576, 32068, 51172, 55832,
                41040, 33412, 64264, 11000, 38640, 51028, 8560,  65476, 12668,
                36884, 21396, 60672, 43416, 12168, 49556, 704,   64028, 51352,
                36724, 62088, 31972, 56060, 39580, 60216, 25976, 3436,  32588,
                14280, 37416, 10032, 40884, 7512,  36228, 62224, 5388,  16568,
                59576, 35556,
            },
            {
                8024,  40908, 32524, 12980, 62940, 4740,  30252, 57256, 7676,
                20420, 49908, 10364, 14864, 24288, 17816, 37236, 48692, 60192,
                55576, 23188, 42436, 62256, 16420, 56792, 5776,  43688, 12228,
                36536, 31548, 23920, 54600, 11468, 46932, 4600,  30132, 56360,
                18960, 63484, 7804,  48576, 56884, 35720, 42072, 29996, 55852,
                17212, 46592, 10752, 57556, 36968, 27768, 9556,  38792, 54540,
                43064, 63780, 2264,  48416, 16436, 64944, 6404,  45972, 59268,
                15688, 64160, 30784, 12636, 60292, 9580,  32860, 48608, 27720,
                12332, 37844, 62436, 54644, 42152, 58124, 46412, 27860, 18436,
                54280, 22992, 11892, 29328, 17428, 39328, 44800, 20740, 10524,
                15848, 4352,  57464, 30676, 180,   46264, 35532, 23468, 53492,
                18396, 1420,  51764, 29244, 17744, 24620, 41172, 29668, 21928,
                16800, 11652, 28084, 19524, 308,   12420, 35508, 54532, 47312,
                6368,  50844, 63704, 31684, 18528, 13208, 45004, 38732, 25196,
                28688, 55356,
            },
            {
                4512,  18648, 57064, 46160, 34708, 39056, 48980, 23548, 41636,
                52800, 25864, 59688, 39424, 64812, 52180, 28456, 4708,  9172,
                38712, 30432, 3048,  50496, 10120, 25344, 21556, 52360, 26908,
                61872, 17840, 892,   64772, 33924, 50992, 16836, 39960, 380,
                26628, 34112, 52636, 31456, 1452,  63944, 12480, 23948, 33436,
                5196,  64156, 41376, 1444,  48296, 62872, 24248, 3392,  58664,
                19988, 25380, 38296, 10684, 41572, 24440, 1692,  32764, 53252,
                25264, 40704, 6596,  51696, 44744, 39836, 63244, 15456, 52116,
                58812, 33732, 9536,  29356, 16216, 32976, 12584, 8748,  48544,
                39552, 60048, 33180, 57608, 48836, 62452, 3096,  58504, 27868,
                37440, 50052, 42544, 19924, 25988, 61200, 15976, 28128, 41952,
                62476, 33252, 39764, 6876,  63188, 35440, 10488, 46292, 55512,
                4888,  43908, 58548, 7984,  47768, 65472, 29316, 9256,  41468,
                19556, 24820, 45980, 504,   60204, 22836, 56348, 10424, 51088,
                46368, 14204,
            },
            {
                38148, 52764, 26752, 9832,  16676, 1972,  58904, 14060, 31620,
                1184,  16560, 34700, 6732,  46392, 732,   56560, 33692, 62620,
                16832, 47788, 19864, 59168, 36396, 41308, 2176,  33528, 15108,
                56056, 49400, 40692, 29064, 5948,  59416, 22500, 36132, 48848,
                58512, 41792, 11404, 24460, 39612, 19020, 53324, 60244, 2184,
                51412, 26788, 13916, 21052, 55212, 17496, 12268, 47116, 29288,
                8000,  45796, 13672, 57056, 30584, 60788, 38996, 18376, 11292,
                43584, 348,   55340, 23128, 27400, 3704,  18832, 24708, 42256,
                1580,  20476, 50480, 5740,  39304, 64972, 21884, 56204, 31008,
                276,   15532, 45268, 4264,  10092, 22040, 34392, 14144, 47072,
                63068, 23680, 7456,  53244, 13428, 40480, 5956,  49744, 4060,
                10604, 56148, 14484, 45668, 2700,  57560, 53140, 7712,  26256,
                60604, 40272, 34324, 52424, 24036, 38116, 16668, 21948, 61708,
                56932, 29904, 15816, 53548, 42536, 49336, 33496, 6640,  19968,
                65316, 31444,
            },
            {
                48732, 62292, 21424, 43252, 60520, 28264, 19540, 45144, 36736,
                55320, 62004, 43708, 21552, 31772, 13380, 19296, 43904, 24476,
                12232, 54292, 6200,  32720, 13424, 53936, 63864, 47500, 38828,
                7084,  10044, 45280, 20780, 13844, 42576, 9636,  62192, 6912,
                15024, 20804, 3820,  61700, 45616, 27556, 7028,  47256, 16124,
                39100, 58428, 35460, 30136, 7144,  42716, 32660, 52768, 35780,
                16120, 60252, 33584, 52940, 21380, 8960,  47040, 56232, 28572,
                62196, 20320, 35704, 14644, 47888, 37916, 56940, 6220,  35536,
                46376, 14008, 59980, 24020, 45096, 2772,  52092, 37460, 25248,
                62516, 20620, 51700, 27388, 36792, 42644, 53132, 7768,  30284,
                1208,  18112, 35852, 58944, 47684, 34076, 22124, 58292, 30472,
                47064, 25760, 21736, 50876, 31580, 19972, 15104, 38872, 32596,
                1896,  49992, 20628, 14156, 3952,  57620, 44988, 51472, 1956,
                38664, 5300,  35408, 10928, 26556, 3124,  29136, 58792, 41260,
                800,   24332,
            },
            {
                11824, 2912,  36388, 6372,  50020, 53388, 8980,  65156, 5508,
                12084, 27672, 50472, 9336,  57832, 26400, 49080, 39868, 2636,
                28872, 65088, 40484, 26872, 45404, 17628, 29492, 4420,  20400,
                60028, 25512, 35428, 53116, 57100, 27568, 45940, 31896, 25136,
                51504, 43972, 29684, 54848, 15436, 9916,  34592, 42940, 29168,
                22508, 9436,  45392, 61852, 51884, 38280, 22048, 65508, 4916,
                41764, 49448, 488,   28012, 4448,  50860, 14968, 36560, 7324,
                49688, 31764, 58624, 9228,  62656, 29836, 11896, 53124, 64736,
                28292, 40280, 55844, 31680, 11532, 60872, 16984, 6348,  44048,
                10844, 40724, 7392,  65144, 56660, 16672, 25792, 61592, 40472,
                55240, 44064, 10408, 27636, 2180,  62708, 8920,  54468, 37324,
                16804, 60360, 8032,  37768, 27176, 59880, 43336, 23020, 64856,
                17548, 10344, 29768, 62388, 42512, 31364, 6796,  26944, 33428,
                12948, 48736, 57992, 19320, 64456, 37792, 16932, 13444, 35848,
                54816, 44268,
            },
            {
                28616, 58240, 15052, 31120, 22840, 40052, 33988, 25244, 48324,
                38960, 18308, 2872,  40912, 63764, 36620, 5396,  59588, 52984,
                35292, 46324, 21208, 1368,  61136, 8840,  23904, 42876, 54564,
                31140, 16608, 63572, 4640,  38264, 2800,  18488, 54972, 12200,
                65500, 34796, 6064,  59364, 38056, 50176, 21080, 64688, 55508,
                6044,  49104, 18624, 3204,  25856, 14796, 1096,  57676, 11024,
                26696, 22668, 62852, 17728, 44312, 34284, 63504, 22832, 2724,
                39504, 17120, 45448, 3180,  51120, 21644, 43736, 32256, 9016,
                22608, 4380,  18044, 48376, 26136, 35472, 49692, 28484, 59188,
                32840, 47540, 23736, 13464, 31516, 5596,  49236, 19512, 12396,
                32924, 50396, 64632, 16576, 31552, 43068, 18916, 12284, 44792,
                820,   33876, 41868, 63868, 4024,  11216, 48776, 5232,  52604,
                37264, 46480, 25192, 54860, 36112, 11136, 64004, 15492, 55564,
                42832, 62780, 22408, 40504, 8056,  46876, 51484, 61620, 22116,
                8816,  17640,
            },
            {
                34068, 47008, 55616, 42056, 62608, 13020, 2468,  56624, 21764,
                59904, 30608, 54140, 24052, 15768, 51724, 10712, 22452, 15276,
                7948,  57096, 10960, 51860, 38088, 57612, 48948, 12716, 36324,
                1748,  46720, 11772, 33028, 24172, 61132, 48224, 40284, 1536,
                37612, 17800, 47480, 23068, 1004,  32148, 58068, 4052,  13436,
                36728, 60580, 32268, 11524, 56168, 49764, 28952, 44740, 19472,
                55112, 37180, 7616,  39608, 57712, 11636, 42136, 26832, 60280,
                53556, 13028, 25472, 40932, 33784, 17652, 816,   59068, 16252,
                49896, 61352, 37788, 952,   8372,  41664, 20088, 13084, 54828,
                2132,  18120, 35316, 54064, 1088,  45896, 38492, 58880, 3468,
                21676, 25360, 4496,  39520, 56744, 23724, 52028, 28608, 64900,
                24396, 52964, 13024, 47364, 18224, 55168, 34672, 30400, 13276,
                57188, 42004, 5892,  924,   18156, 49304, 20892, 47076, 24344,
                9300,  28700, 1304,  32496, 54704, 23980, 3932,  43092, 32028,
                49864, 63488,
            },
            {
                5652,  25132, 10072, 18668, 4612,  46512, 30040, 16288, 43360,
                9884,  35244, 47152, 6400,  33308, 44544, 29680, 61860, 32556,
                42796, 18164, 25616, 31616, 14892, 34276, 6420,  64780, 26764,
                58564, 50328, 42132, 19648, 52008, 14628, 7916,  28884, 21976,
                57232, 9052,  26692, 52504, 11924, 42248, 25836, 17932, 40348,
                53444, 23732, 44252, 40780, 63660, 34432, 8700,  39136, 61572,
                32416, 13352, 47208, 53736, 24500, 31188, 5924,  18584, 47444,
                10100, 29432, 65264, 56316, 7784,  61236, 26504, 46972, 41336,
                34352, 12300, 53940, 44376, 64108, 57260, 32136, 4540,  38548,
                63388, 50016, 28008, 41404, 60908, 22476, 29148, 9424,  52580,
                36432, 57700, 45660, 12736, 49252, 5604,  35164, 3224,  40140,
                20044, 6460,  57424, 29500, 23372, 39948, 400,   63044, 27480,
                7888,  21812, 61256, 51292, 27728, 57076, 40712, 2640,  35796,
                53016, 17660, 46020, 14024, 61148, 29568, 11708, 57244, 26684,
                2228,  40396,
            },
            {
                21160, 60876, 38560, 51320, 27000, 54272, 36680, 61560, 51920,
                604,   14080, 62564, 20472, 55896, 3288,  18824, 47272, 8,
                38636, 63748, 48448, 3728,  44340, 21620, 52896, 18252, 40180,
                9564,  22884, 6244,  29780, 57752, 44324, 34348, 62824, 50272,
                42000, 32580, 60640, 15956, 63444, 46344, 8452,  51692, 30732,
                0,     27484, 15604, 5056,  20152, 23428, 47060, 16596, 5356,
                51524, 2516,  28588, 15844, 64820, 1964,  51916, 58204, 32652,
                43124, 37752, 5352,  23276, 48144, 36356, 13216, 54920, 6408,
                24892, 30704, 19352, 27560, 15132, 22768, 52744, 45540, 24872,
                15400, 8140,  58212, 11036, 51092, 15632, 34224, 43196, 62268,
                15096, 30060, 8160,  20532, 37664, 62056, 15580, 46452, 59776,
                32056, 49216, 16036, 36628, 8888,  45604, 59372, 20536, 16392,
                48392, 35228, 31048, 15044, 38284, 8208,  32572, 60328, 5660,
                58456, 38348, 50768, 4948,  43964, 34876, 20136, 7328,  37396,
                15736, 52556,
            },
            {
                44004, 13324, 220,   32288, 64548, 8792,  20444, 6196,  25696,
                41264, 57372, 28300, 38068, 12048, 41752, 57908, 26856, 50804,
                23584, 5784,  54576, 28288, 62504, 41392, 708,   30688, 45664,
                15148, 62180, 53652, 38808, 4,     10740, 25972, 16996, 3920,
                13776, 45056, 2880,  39852, 28868, 19784, 36456, 60220, 48104,
                10544, 61936, 55144, 35800, 59012, 2056,  54200, 30968, 25076,
                58276, 43800, 35060, 9280,  21048, 45288, 36068, 13908, 22120,
                728,   55108, 19700, 15424, 52204, 30076, 38904, 20492, 2420,
                58096, 63028, 6616,  51380, 36652, 2888,  9428,  61264, 30108,
                42380, 21904, 37284, 3976,  19292, 63984, 6744,  26636, 47508,
                2360,  41300, 60656, 54160, 25940, 30620, 10384, 55148, 7760,
                25668, 43500, 2076,  62600, 54012, 25908, 12004, 43440, 51792,
                40320, 58588, 10968, 45528, 63428, 23048, 12880, 44592, 29508,
                20220, 11220, 26340, 56524, 16388, 39700, 53836, 64344, 46624,
                59196, 30236,
            },
            {
                55136, 35908, 48620, 22404, 43680, 14896, 39592, 49844, 32960,
                17964, 45548, 8172,  49348, 24692, 64988, 35188, 9212,  13620,
                59300, 36344, 11656, 17164, 58648, 8580,  25252, 55948, 33896,
                2964,  35940, 27244, 17552, 64952, 49220, 36504, 59204, 53376,
                30368, 20656, 55840, 34704, 6560,  54436, 2460,  14556, 33664,
                21492, 45748, 7684,  30188, 48512, 13936, 42340, 64604, 11420,
                40468, 18024, 60504, 49988, 55968, 38948, 27236, 62112, 8648,
                50456, 63132, 34584, 44080, 3908,  9572,  64200, 45348, 50632,
                35844, 43448, 10156, 40456, 47604, 59412, 34468, 18352, 48996,
                5236,  55456, 33120, 25520, 45180, 39636, 55824, 12984, 18900,
                23840, 51256, 33848, 17284, 1684,  42600, 50240, 18616, 38372,
                12180, 58356, 21660, 33716, 50464, 3644,  32232, 37000, 6500,
                1952,  25764, 19248, 5152,  53300, 48168, 26648, 16348, 62860,
                41628, 33592, 65268, 22532, 8832,  48808, 856,   25056, 12380,
                19136, 9060,
            },
            {
                63736, 28184, 17328, 58864, 3516,  55964, 28684, 59512, 11492,
                64240, 22488, 32084, 1740,  52208, 16988, 4180,  53944, 40592,
                30540, 20752, 46016, 33500, 38768, 13928, 51240, 61308, 20464,
                48660, 56752, 12592, 43756, 31740, 21564, 6976,  46136, 23960,
                5608,  64564, 10688, 48764, 22780, 59192, 43696, 25032, 65220,
                41820, 17156, 38616, 52024, 10276, 26196, 36904, 7004,  20756,
                48832, 4556,  23160, 30508, 12460, 6692,  17512, 48588, 40428,
                25304, 28928, 11628, 59000, 24540, 56608, 17928, 28172, 11872,
                23388, 16476, 56128, 21612, 29708, 13760, 26444, 39780, 11812,
                64256, 44136, 14152, 59712, 48652, 30504, 24,    53156, 38060,
                65512, 10764, 28056, 7008,  46056, 63800, 22060, 34060, 61580,
                52072, 28144, 5140,  41692, 14192, 19288, 56572, 62012, 23132,
                53832, 64936, 33372, 42540, 2464,  35116, 6952,  54428, 50236,
                20,    47516, 4056,  37060, 58908, 27996, 32404, 42468, 51068,
                34184, 2756,
            },
            {
                24520, 6464,  41880, 11088, 33960, 24412, 45944, 5176,  36292,
                3364,  54432, 15180, 60812, 39672, 29360, 21944, 44864, 16064,
                61588, 1656,  52844, 6776,  22504, 46996, 28756, 11008, 5732,
                39796, 23836, 8008,  59552, 3316,  40984, 55500, 12264, 38856,
                33736, 51136, 26012, 17144, 38012, 12680, 50340, 29528, 5440,
                56132, 1644,  24008, 63264, 19176, 44568, 60128, 32692, 57020,
                27880, 38176, 62884, 44,    42184, 59680, 33712, 3500,  53660,
                16016, 4504,  46764, 39380, 31448, 1068,  42128, 33360, 62048,
                3556,  48780, 32600, 148,   65516, 6056,  51808, 57376, 1336,
                20256, 52320, 28724, 3116,  10200, 22676, 58936, 32504, 43620,
                4584,  49088, 58412, 39812, 55688, 13056, 5288,  29184, 232,
                45224, 17100, 37604, 65216, 30372, 46068, 9624,  27908, 15636,
                44692, 12700, 29596, 56724, 21760, 61908, 40080, 19436, 9828,
                25088, 15084, 30148, 52648, 18932, 13236, 62744, 5736,  22180,
                39460, 45600,
            },
            {
                15440, 57624, 50680, 38500, 61644, 52304, 13628, 19708, 40440,
                48196, 26000, 43212, 34524, 6032,  47608, 62524, 33332, 7228,
                48560, 27620, 42604, 65232, 55484, 2304,  36844, 43368, 16344,
                64228, 29876, 46704, 51676, 25504, 15820, 60668, 28768, 18504,
                43044, 57328, 1688,  41412, 63156, 32044, 7936,  19072, 36060,
                11752, 50040, 27984, 34228, 6488,  53568, 16044, 1196,  51312,
                13808, 8308,  34504, 46556, 24832, 54852, 20816, 43936, 30884,
                60512, 36760, 18748, 49452, 61012, 14816, 53012, 7844,  55396,
                38180, 27152, 60632, 42720, 17664, 37292, 46028, 22964, 30836,
                35736, 7928,  39112, 61540, 18152, 36352, 15368, 8492,  27416,
                20696, 35380, 15756, 22928, 31296, 36660, 59376, 41916, 24484,
                9880,  56100, 48172, 7276,  24236, 59832, 38756, 49408, 34964,
                8188,  39480, 50076, 16848, 10520, 27960, 58020, 31460, 37904,
                56004, 61756, 43732, 7200,  40220, 45856, 55592, 17032, 29400,
                61220, 53264,
            },
            {
                37028, 20920, 30100, 2084,  18412, 8052,  31904, 55464, 63180,
                29900, 8900,  59156, 11936, 19184, 54976, 10248, 25508, 56900,
                37384, 18004, 10860, 24940, 31360, 19108, 49644, 57740, 32724,
                52980, 3888,  19008, 37260, 33420, 9836,  50088, 908,   62392,
                8708,  14728, 47700, 28504, 4216,  45912, 57480, 53276, 40340,
                60020, 46524, 14176, 56772, 3716,  40000, 29676, 23552, 45688,
                41780, 55812, 17292, 51864, 14872, 29128, 8904,  65048, 12184,
                23024, 55716, 6828,  9940,  21696, 35680, 25992, 47636, 20184,
                14404, 5420,  52068, 11672, 25468, 54680, 8928,  41808, 62052,
                16032, 47532, 23504, 53956, 42168, 49520, 63540, 45656, 56652,
                12408, 61276, 944,   47820, 9528,  17732, 52972, 48972, 14908,
                63408, 35160, 20284, 12436, 52756, 916,   17672, 5124,  55476,
                21068, 59264, 536,   37256, 46768, 51672, 14296, 6520,  45436,
                21480, 34320, 11048, 60176, 24264, 2128,  35708, 10184, 58488,
                3860,  12012,
            },
            {
                48700, 8908,  44916, 65172, 26660, 47952, 42856, 23472, 808,
                16664, 52724, 21700, 50184, 27272, 37080, 828,   41776, 13268,
                50516, 3972,  59240, 40076, 15004, 61512, 4796,  25828, 21548,
                9440,  42120, 14184, 54508, 63732, 45088, 20508, 35828, 53680,
                31152, 23316, 59916, 35284, 10172, 21936, 15900, 26820, 3004,
                22384, 30988, 43024, 61268, 20592, 47856, 64024, 35580, 10548,
                61452, 22176, 31472, 63768, 3152,  38552, 48176, 35832, 996,
                45328, 40632, 27380, 51608, 64572, 44924, 4940,  40168, 59456,
                30916, 45792, 39600, 20736, 58156, 33388, 14464, 4108,  50384,
                58012, 26772, 4700,  13044, 29812, 1784,  25496, 5888,  52040,
                40944, 29820, 54188, 43852, 64316, 26900, 2628,  20792, 38876,
                31980, 3788,  26828, 44248, 57812, 33832, 42308, 63644, 31196,
                46372, 26352, 62412, 7516,  23440, 42212, 3700,  59536, 49440,
                1768,  16992, 27680, 50036, 32816, 20312, 52344, 47800, 43204,
                26932, 32928,
            },
            {
                1008,  24932, 54232, 14400, 35864, 4620,  60480, 38160, 12572,
                45452, 35476, 5296,  40040, 63960, 14692, 59696, 23016, 31884,
                63008, 21032, 34080, 52264, 8968,  45324, 35312, 12844, 62480,
                38232, 59876, 27148, 2220,  22940, 6384,  40424, 26180, 46736,
                4436,  38580, 19208, 54700, 51276, 61808, 43476, 33220, 64236,
                6724,  17636, 9544,  36644, 25752, 12552, 7524,  58308, 18956,
                2428,  26652, 6212,  43648, 11568, 59056, 18068, 26300, 52588,
                14736, 58136, 33612, 1968,  30160, 12940, 17484, 63064, 10492,
                24080, 34852, 8236,  62420, 1860,  49548, 27744, 19300, 34372,
                11040, 43712, 64992, 33080, 56260, 20440, 38956, 33888, 16788,
                22372, 4084,  38020, 24984, 7116,  33656, 57764, 45992, 6632,
                54368, 50172, 60968, 39924, 29080, 21896, 10940, 25280, 13924,
                3960,  18432, 52624, 29220, 33984, 63776, 18288, 25964, 35992,
                64796, 41176, 54728, 5228,  14452, 64292, 25388, 7000,  38356,
                19600, 63604,
            },
            {
                39820, 59200, 31528, 41612, 21932, 57896, 10352, 50072, 27496,
                62232, 57204, 31832, 2508,  44460, 30324, 46712, 51960, 5252,
                43784, 28936, 47260, 312,   57400, 27704, 41372, 51376, 1212,
                45984, 31208, 49172, 56684, 12052, 31692, 57952, 16764, 13104,
                63364, 43700, 7044,  12496, 25728, 316,   37884, 13248, 47924,
                51544, 39256, 54984, 612,   32628, 53020, 41428, 28604, 50604,
                39412, 54332, 36276, 49904, 20612, 41200, 55868, 7412,  31628,
                62812, 19476, 11324, 42828, 23532, 56972, 36944, 50004, 768,
                52892, 56468, 16164, 47348, 30072, 43352, 63844, 39952, 54988,
                532,   21456, 37348, 45412, 9100,  61092, 50732, 11428, 58524,
                46792, 62896, 19060, 13768, 51932, 40840, 11044, 61464, 29796,
                13548, 18548, 9456,  2828,  15448, 51648, 47352, 60776, 53748,
                41132, 35744, 43600, 15128, 11172, 54352, 39668, 30128, 12972,
                8932,  23420, 47020, 36948, 57916, 40644, 30548, 12956, 56284,
                16008, 51040,
            },
            {
                6224,  18208, 11264, 2620,  51616, 29140, 17544, 33920, 20212,
                7604,  42216, 24368, 17892, 57980, 8820,  20092, 35944, 16784,
                8460,  55032, 12560, 37900, 23224, 16504, 7112,  55236, 18736,
                24320, 5272,  16616, 36000, 43460, 47976, 61532, 2556,  51372,
                28896, 56332, 32780, 48568, 41052, 30172, 19756, 8224,  58140,
                23940, 28912, 63400, 44752, 49328, 22576, 16508, 3856,  33108,
                13388, 47204, 9292,  61112, 28476, 33008, 4288,  50556, 22208,
                38440, 3592,  48716, 59972, 53284, 4100,  32372, 27064, 43572,
                19132, 28612, 4492,  36984, 22284, 11416, 5936,  24132, 60708,
                30332, 17276, 52704, 6156,  15236, 27364, 43112, 2664,  24360,
                31588, 9820,  35120, 59528, 48352, 28692, 16864, 23184, 37268,
                42596, 25820, 56928, 35604, 64588, 6376,  37552, 1440,  32896,
                9852,  58256, 5848,  49224, 57240, 1540,  21824, 44980, 52428,
                57500, 31316, 2748,  18484, 9364,  44492, 392,   62000, 34572,
                23496, 45724,
            },
            {
                28792, 36548, 55792, 62380, 46420, 6848,  39988, 55632, 3576,
                53392, 14528, 48812, 11336, 52552, 25936, 3268,  56044, 64716,
                39148, 25600, 18544, 62400, 32140, 49136, 64304, 29536, 34452,
                58228, 10452, 65496, 20384, 27892, 8384,  38212, 24376, 34828,
                10320, 22028, 17036, 64636, 5904,  60848, 55440, 44948, 34424,
                2532,  11056, 19952, 14588, 6004,  60996, 55312, 43836, 65392,
                23756, 56616, 16824, 844,   24152, 14016, 64208, 43168, 10144,
                46064, 25580, 35468, 16012, 39384, 20660, 47388, 7528,  58492,
                12384, 64668, 41836, 51444, 59932, 17924, 53528, 36160, 9312,
                46508, 41092, 25316, 59616, 48440, 18780, 35644, 64084, 53856,
                7240,  42360, 56076, 1512,  21204, 5180,  44936, 56372, 684,
                63140, 46284, 22296, 49068, 30244, 19748, 44088, 16536, 27492,
                22424, 65480, 19912, 24996, 31556, 46904, 8004,  60920, 4344,
                16116, 43056, 62908, 51196, 28956, 22212, 53988, 48768, 4644,
                9780,  60284,
            },
            {
                13492, 44316, 20516, 32644, 24464, 14676, 64300, 47404, 22844,
                36556, 29460, 65492, 38300, 33880, 61128, 40896, 28448, 11776,
                50112, 3872,  58604, 42452, 5592,  10556, 21636, 38760, 13812,
                43812, 51012, 40224, 53728, 252,   14148, 54892, 19612, 42356,
                59092, 3140,  46172, 36376, 23352, 14868, 49588, 27260, 16796,
                60488, 42588, 31512, 57784, 38344, 26980, 10672, 36400, 7708,
                19788, 29832, 45112, 34620, 58440, 47520, 36992, 16980, 30092,
                60800, 55384, 5880,  29280, 8724,  61548, 14336, 40664, 22528,
                36348, 32004, 9276,  25164, 3200,  44808, 32244, 48476, 15608,
                3892,  57688, 12104, 31676, 1416,  55388, 29584, 13528, 38556,
                49596, 26916, 15464, 30436, 39856, 65176, 35416, 8876,  52308,
                33248, 5384,  11796, 41016, 53464, 8788,  59896, 56484, 49672,
                39752, 46144, 2448,  38228, 60136, 17200, 35076, 27168, 38912,
                20716, 34204, 25332, 12224, 59744, 35980, 15020, 26988, 42572,
                32168, 53236,
            },
            {
                3020,  26432, 49856, 8744,  37408, 43016, 144,   31224, 60932,
                9408,  45560, 1352,  21520, 5664,  43596, 14104, 47860, 33512,
                21996, 45468, 29688, 35264, 52848, 44976, 60260, 1940,  47948,
                26424, 7700,  31852, 23040, 60412, 49360, 29644, 64192, 5052,
                50152, 39664, 26600, 53064, 1764,  32512, 10308, 40308, 4360,
                36496, 53996, 24816, 47680, 1680,  18388, 30872, 46748, 58600,
                2248,  41984, 62892, 12376, 51968, 8096,  26116, 57348, 1112,
                13244, 51004, 18740, 65072, 45024, 25740, 55628, 51104, 2864,
                60156, 47864, 15388, 55044, 39292, 13512, 57252, 27196, 64556,
                22200, 50772, 35056, 62424, 39128, 22948, 47136, 4828,  17704,
                21472, 45496, 62020, 11304, 54276, 24164, 49880, 13016, 20252,
                28156, 16732, 60352, 1612,  26576, 34164, 23852, 12192, 30684,
                6988,  14256, 55208, 28052, 42324, 13128, 50600, 63312, 10572,
                48892, 56612, 5752,  39920, 45868, 6580,  19888, 58572, 37988,
                17572, 64224,
            },
            {
                40612, 57664, 5096,  16756, 59120, 53176, 11552, 26308, 16268,
                41544, 58576, 27208, 54752, 50024, 17452, 23952, 736,   59868,
                7672,  54288, 15804, 2692,  13404, 26928, 18172, 54788, 4960,
                59424, 15512, 37532, 4096,  41324, 33604, 17280, 44940, 12128,
                31488, 15660, 8812,  59796, 43420, 57248, 21156, 52252, 64976,
                46384, 13164, 7148,  33700, 61836, 40576, 51560, 15056, 24924,
                53356, 37580, 5524,  22064, 39376, 3088,  53604, 20336, 34760,
                39680, 23652, 41844, 32912, 12132, 964,   35004, 29916, 16884,
                44328, 5884,  27608, 62888, 34780, 20968, 7964,  1724,  42704,
                38044, 28752, 5164,  19732, 8284,  42068, 10736, 57488, 60572,
                33156, 6492,  51480, 36956, 3676,  16312, 31784, 42372, 58540,
                39408, 48184, 55064, 37044, 14620, 62996, 42232, 2160,  52272,
                18644, 61368, 33172, 10128, 3852,  53752, 23660, 256,   44060,
                29372, 14564, 53060, 1508,  32292, 65340, 50584, 10784, 1924,
                47708, 21880,
            },
            {
                52128, 33304, 38684, 62972, 28376, 20104, 35308, 57380, 51132,
                4928,  19164, 13448, 31796, 10076, 62072, 56632, 30524, 38192,
                19388, 64116, 40608, 23608, 61896, 37248, 32568, 41796, 22368,
                30364, 63108, 18704, 56776, 46348, 10240, 6664,  25228, 36972,
                55740, 62604, 19916, 29752, 12728, 38380, 26096, 6468,  18476,
                29544, 22456, 50348, 9992,  55708, 21772, 3324,  62640, 33476,
                9684,  27252, 49616, 32052, 17984, 42680, 28828, 45684, 62376,
                9304,  47612, 3296,  58128, 52616, 38728, 19576, 63332, 10676,
                24092, 53024, 19228, 264,   49608, 30456, 61804, 52732, 18176,
                9988,  54500, 14220, 44872, 49876, 65292, 25956, 36552, 52836,
                760,   25116, 41420, 19484, 60236, 46852, 26100, 2028,  63696,
                7352,  23676, 30908, 45164, 19096, 5796,  47304, 37940, 58784,
                35340, 43568, 22784, 47868, 64128, 37252, 19112, 32348, 58856,
                7568,  37536, 61476, 23776, 17184, 28280, 41028, 54784, 24780,
                30820, 8300,
            },
            {
                15964, 12260, 23100, 46340, 2668,  49060, 7644,  44292, 22352,
                33348, 37848, 62960, 46464, 40024, 34888, 6260,  52824, 44012,
                11276, 27316, 50888, 9268,  46756, 57960, 7588,  51716, 11600,
                36200, 44604, 52312, 12788, 26088, 61332, 50672, 58684, 21516,
                748,   47276, 35300, 51000, 3996,  62096, 48068, 33884, 58064,
                784,   39308, 63616, 16612, 42844, 28960, 12868, 44096, 47960,
                18744, 60972, 14852, 55224, 64704, 10328, 59312, 14300, 5584,
                31732, 54840, 16716, 28124, 22072, 45996, 4924,  48888, 56852,
                32760, 37304, 42264, 58336, 10180, 46268, 40756, 25052, 33868,
                47600, 60448, 23908, 56536, 33040, 16912, 3984,  30480, 44272,
                14916, 63932, 29300, 9012,  33704, 57344, 6328,  53692, 14988,
                35876, 10392, 62440, 3776,  50752, 56024, 21104, 28644, 8640,
                25232, 4456,  51180, 15668, 30156, 6204,  45908, 55008, 16580,
                26392, 41940, 20288, 47296, 56164, 8588,  34820, 14084, 62364,
                44560, 59888,
            },
            {
                640,   43384, 56200, 10048, 31640, 39332, 14268, 63716, 1584,
                55664, 49504, 7132,  24792, 2976,  20200, 26712, 15160, 48852,
                2036,  34180, 55780, 31428, 20180, 428,   16096, 64968, 25072,
                48908, 8728,  1084,  29164, 20052, 35232, 2908,  42672, 53756,
                28324, 41000, 6880,  23848, 16540, 54684, 8596,  41728, 14776,
                44832, 55024, 26252, 35280, 4764,  59360, 37860, 22988, 6000,
                56812, 40184, 272,   24536, 46552, 33976, 22912, 38400, 50616,
                25724, 64064, 10828, 37108, 6892,  62076, 14072, 26880, 40088,
                8084,  65364, 13284, 29144, 23416, 15916, 4616,  58980, 12884,
                3396,  30208, 40248, 360,   27516, 51848, 12704, 22016, 39900,
                7876,  48756, 55836, 13076, 43656, 21756, 38816, 28680, 44812,
                51316, 21184, 42092, 27796, 12368, 33076, 64324, 15868, 54124,
                11900, 62360, 39244, 20424, 58500, 25516, 40144, 11668, 51336,
                64476, 2780,  33564, 12628, 43240, 4984,  49032, 20952, 3792,
                36828, 27756,
            },
            {
                34212, 65080, 26104, 18196, 53980, 60468, 24680, 29696, 40940,
                11852, 28236, 16444, 53688, 59064, 42504, 63856, 36748, 22872,
                61088, 17708, 6104,  39004, 49464, 28708, 44368, 39740, 3620,
                20432, 58376, 33292, 55100, 39788, 64660, 14636, 30896, 18000,
                9592,  13820, 65192, 44732, 37320, 31436, 20308, 24760, 62968,
                10412, 30812, 2900,  48928, 19536, 52480, 8536,  64948, 30320,
                35672, 12032, 43524, 29476, 7476,  3692,  56384, 17336, 41748,
                1532,  20916, 44072, 49624, 58884, 30768, 42556, 54364, 2320,
                21776, 47476, 3964,  52364, 35320, 56376, 37880, 20408, 50916,
                43664, 63448, 16280, 7572,  35756, 61440, 46060, 54624, 59020,
                18124, 34412, 23284, 2268,  50872, 64708, 11072, 18724, 33396,
                57536, 168,   16936, 59148, 39576, 24308, 6896,  44672, 40684,
                32024, 46588, 2296,  55516, 8768,  34364, 62512, 4676,  22324,
                36092, 9724,  49708, 59676, 30672, 62920, 26452, 39276, 57576,
                18392, 49964,
            },
            {
                20692, 47212, 4232,  36592, 42624, 5588,  51452, 17392, 47372,
                59856, 21344, 44772, 10500, 30292, 50840, 12580, 8340,  57288,
                41164, 45828, 13944, 62428, 24608, 12028, 56036, 33912, 60756,
                27296, 15196, 42740, 5416,  48128, 23116, 7636,  38600, 49040,
                60616, 33444, 56396, 27312, 1216,  59540, 46448, 3332,  51384,
                36640, 21448, 60536, 14200, 45632, 32080, 25712, 54480, 1908,
                50608, 21440, 59152, 52548, 37400, 48408, 12556, 61216, 28308,
                36120, 59828, 14788, 33836, 24072, 9676,  17992, 34808, 60808,
                16440, 31736, 59460, 18656, 44724, 6308,  64296, 32112, 26912,
                8992,  37004, 22432, 57904, 42384, 19944, 9492,  37672, 1780,
                27888, 46404, 60960, 37196, 17056, 30816, 48316, 3628,  62100,
                24752, 47904, 31928, 53768, 9272,  35760, 49684, 836,   60060,
                18356, 26336, 36172, 13264, 49576, 42552, 14996, 48456, 31100,
                44648, 57176, 18020, 25144, 556,   16260, 52152, 45812, 11168,
                6784,  54268,
            },
            {
                39928, 8420,  30012, 57928, 13008, 21148, 34084, 9180,  37132,
                3148,  32408, 64960, 36240, 356,   18364, 46740, 32856, 3688,
                25392, 29920, 58584, 2856,  53320, 36600, 18060, 9220,  46440,
                51936, 37924, 63304, 17724, 10864, 44420, 52076, 57600, 24824,
                2144,  21952, 52248, 18952, 10536, 50372, 13184, 28640, 40408,
                17220, 53432, 7720,  38620, 58400, 11356, 41024, 17624, 14348,
                45444, 27612, 9340,  16720, 62488, 20068, 31284, 52132, 8400,
                46868, 53968, 5412,  40596, 384,   53184, 47896, 25260, 11684,
                43928, 27288, 39096, 8456,  25484, 11868, 48584, 1280,  54312,
                18784, 45872, 52668, 12188, 48664, 3536,  25060, 31380, 63640,
                11156, 53256, 4820,  42508, 25872, 7464,  55680, 41284, 13656,
                8532,  37568, 5360,  61240, 45552, 18844, 57568, 29660, 22220,
                51364, 6456,  65084, 29380, 22540, 1560,  27384, 18976, 60516,
                3756,  28568, 40252, 53032, 37628, 9128,  32996, 22704, 61788,
                31624, 13660,
            },
            {
                24196, 51668, 15812, 62340, 48460, 2192,  64148, 53560, 25996,
                56096, 5672,  41536, 23384, 57644, 39344, 28428, 54248, 19604,
                50440, 9716,  34804, 21504, 42252, 64020, 4268,  23056, 30588,
                12848, 2380,  24452, 31716, 59892, 28080, 3748,  34008, 12352,
                43380, 46716, 4868,  38532, 42224, 63632, 34936, 57144, 5068,
                61544, 33472, 43284, 28228, 23648, 5688,  47740, 61288, 33016,
                38836, 63740, 3348,  34528, 40864, 26188, 44468, 2336,  24176,
                11908, 18276, 28968, 57216, 20352, 62764, 37368, 4196,  56328,
                51460, 896,   49016, 62344, 54616, 42424, 29320, 40072, 14412,
                62068, 5280,  32380, 28488, 64436, 34248, 56048, 50196, 15784,
                41036, 20708, 32608, 14488, 52148, 59964, 22516, 35132, 27300,
                52716, 43488, 15332, 22376, 26924, 3240,  41556, 14444, 9860,
                38220, 57008, 16916, 44536, 52296, 61524, 56556, 38276, 11096,
                54112, 14524, 7184,  22012, 64916, 42956, 55644, 2168,  28516,
                42560, 59672,
            },
            {
                55888, 1356,  35036, 41060, 23392, 27980, 44032, 39020, 15212,
                19628, 49152, 13136, 52276, 7620,  14992, 62332, 5952,  43636,
                65328, 38268, 16844, 47448, 7348,  32016, 49244, 59336, 40656,
                6900,  56976, 49512, 36004, 54388, 15872, 20360, 62336, 37356,
                29260, 63252, 15112, 31924, 25440, 7192,  16056, 44396, 23080,
                26688, 12472, 16,    63380, 50532, 36444, 3156,  56508, 19904,
                7048,  53196, 23696, 57592, 13860, 5100,  55700, 65384, 35368,
                42652, 61676, 32120, 50512, 13680, 27448, 7812,  41384, 64052,
                22904, 35988, 14320, 33644, 21296, 16300, 60432, 23008, 34884,
                56836, 25676, 1664,  40492, 18440, 14508, 5908,  44152, 22860,
                7088,  57968, 27092, 62644, 10400, 38496, 1844,  46548, 58464,
                20020, 63260, 30316, 51072, 12688, 64720, 54612, 34420, 61388,
                47980, 3580,  33320, 40268, 10560, 7288,  32252, 46976, 23740,
                42036, 35044, 58932, 46664, 12068, 19464, 35824, 14960, 48272,
                4688,  37048,
            },
            {
                44236, 27348, 19148, 9824,  55164, 31352, 11716, 6888,  61484,
                45900, 29428, 60388, 34304, 26376, 48140, 21740, 35636, 11864,
                55444, 1092,  28100, 52216, 11396, 26508, 15484, 54104, 19468,
                62016, 44028, 21172, 8248,  12,    41592, 47528, 9248,  53684,
                17868, 7564,  54604, 58360, 48204, 21524, 30396, 54456, 9644,
                49816, 46948, 18620, 55656, 15476, 21668, 29636, 42980, 26476,
                12656, 46816, 29744, 43084, 49840, 10272, 33232, 16316, 49340,
                21320, 3544,  38304, 10020, 43200, 46356, 33068, 15588, 29992,
                19336, 9380,  57636, 5624,  30736, 3696,  52172, 7776,  10964,
                42620, 49776, 60084, 10336, 47032, 59048, 38324, 28916, 61996,
                35884, 45648, 336,   49148, 19744, 44500, 16604, 31644, 4508,
                11028, 40264, 1156,  36268, 47156, 31824, 5896,  23668, 42892,
                28260, 12944, 24744, 55156, 20208, 36676, 16156, 5292,  64260,
                992,   50536, 26060, 31572, 5632,  49976, 58248, 25636, 62836,
                17588, 10468,
            },
            {
                64732, 49200, 60524, 6152,  46484, 16920, 50588, 57356, 33188,
                24300, 9736,  1544,  18808, 44524, 59476, 2588,  40880, 30696,
                14720, 23324, 61628, 39708, 56764, 43884, 700,   37388, 28652,
                33816, 14708, 26756, 39428, 63760, 30840, 59072, 26040, 4524,
                40732, 22924, 35700, 12088, 1588,  51904, 39688, 62160, 3188,
                37092, 58644, 40760, 32544, 9140,  53404, 64268, 10188, 50216,
                60044, 36320, 1700,  18240, 60944, 22580, 29180, 39912, 59060,
                7592,  55528, 24604, 64212, 2040,  22344, 60576, 49748, 4900,
                38740, 53088, 46576, 41240, 64860, 44880, 36940, 47548, 28368,
                20712, 15912, 36428, 23924, 53448, 21056, 2004,  52124, 12244,
                17408, 55620, 39248, 5520,  33936, 28904, 54524, 64536, 49948,
                24876, 55556, 17916, 60200, 10040, 39436, 19808, 52328, 16400,
                8064,  63520, 45988, 52,    59016, 26132, 43804, 51768, 29196,
                20560, 16772, 62484, 43508, 3132,  29000, 38892, 8044,  52408,
                33748, 21316,
            },
            {
                2996,  14112, 32736, 37772, 58668, 460,   36328, 21872, 4088,
                42852, 63532, 39816, 55496, 31096, 10624, 52788, 24948, 58120,
                48964, 45212, 4908,  32956, 18836, 63160, 24260, 48072, 10168,
                3824,  52996, 46332, 11464, 51528, 23244, 12780, 44512, 32864,
                50380, 60784, 27520, 43272, 64612, 5868,  18104, 34236, 14436,
                20996, 29024, 6184,  24548, 44764, 1616,  39444, 34128, 16372,
                4548,  21920, 55428, 38528, 7344,  52420, 46196, 588,   13204,
                27004, 47016, 15432, 34100, 58392, 17896, 54136, 12024, 44504,
                59500, 23868, 27992, 11376, 19000, 25028, 59028, 60,    55012,
                64204, 6360,  44480, 29944, 8120,  33236, 42084, 25684, 48424,
                9352,  30048, 23484, 63832, 57272, 12528, 22196, 6984,  15036,
                34568, 44852, 28452, 4692,  57192, 25656, 44616, 1492,  58424,
                37136, 53364, 18048, 29868, 49228, 62944, 13160, 39432, 55584,
                33948, 8240,  38064, 13696, 54236, 61364, 22428, 13052, 41144,
                46280, 29880,
            },
            {
                24012, 53632, 42824, 25672, 20324, 64412, 41248, 27184, 53200,
                14480, 35516, 50728, 15888, 5208,  37496, 64288, 17188, 7996,
                34464, 20008, 59804, 8924,  13884, 35792, 5336,  58768, 41760,
                64404, 18300, 56264, 34880, 6248,  18876, 38040, 55820, 15644,
                904,   47028, 9332,  19616, 32584, 46288, 28036, 59644, 42936,
                51192, 65336, 11400, 35904, 60792, 48404, 19056, 24980, 57368,
                41964, 31808, 64628, 14780, 34096, 25736, 62316, 18692, 36588,
                53380, 30440, 41168, 51948, 5684,  29348, 36684, 26020, 1476,
                32444, 16516, 61520, 3028,  55636, 33928, 13504, 17508, 31164,
                39092, 51888, 3024,  62252, 55336, 13900, 65380, 4964,  59344,
                34968, 53764, 14328, 42140, 8628,  36704, 47988, 42696, 38652,
                60892, 8016,  52828, 22652, 48984, 14876, 62916, 30476, 49544,
                33048, 21648, 41352, 6352,  35400, 9236,  22744, 2208,  60364,
                11460, 48916, 57112, 24636, 18192, 47648, 32428, 204,   55424,
                5780,  58116,
            },
            {
                40132, 7656,  11144, 50920, 4936,  29864, 12892, 48452, 18524,
                58448, 7096,  27732, 23212, 43192, 20452, 46436, 28532, 51428,
                2232,  42520, 26264, 53828, 29776, 46456, 21276, 51156, 13328,
                31828, 23892, 1480,  29620, 58300, 49108, 2904,  65428, 21652,
                30488, 57472, 14248, 37224, 24256, 55400, 12548, 8184,  24868,
                1276,  45920, 17096, 56480, 30552, 14092, 5824,  62396, 45744,
                8448,  27644, 48520, 11092, 44768, 3848,  56668, 9964,  42840,
                63316, 4136,  19608, 11648, 38812, 47732, 8620,  65240, 41944,
                50724, 7196,  37128, 48676, 39884, 8276,  51056, 42104, 23500,
                11732, 34064, 19544, 26700, 38072, 46132, 18036, 31256, 21656,
                43504, 3176,  19160, 50392, 25108, 1288,  61696, 26984, 3648,
                20328, 30632, 11948, 41488, 32880, 37932, 7744,  12456, 26416,
                3380,  10788, 60764, 15208, 54296, 32064, 47084, 17872, 26644,
                44912, 30312, 2576,  40736, 34892, 9784,  44548, 63808, 19656,
                36808, 16048,
            },
            {
                47504, 31396, 63240, 16460, 45532, 54208, 9068,  61060, 2064,
                30924, 46224, 11976, 57164, 61668, 33900, 4132,  13744, 39448,
                63104, 11472, 37576, 49436, 1572,  40192, 55548, 27020, 7052,
                38536, 44360, 60336, 16148, 41020, 24952, 8780,  34188, 52012,
                39712, 6856,  61892, 53308, 41160, 2188,  49476, 57092, 39300,
                32336, 53916, 4392,  42032, 21804, 51592, 28072, 37432, 12400,
                52596, 884,   20520, 54116, 41208, 29856, 21360, 50080, 32316,
                23004, 8824,  59732, 44392, 24336, 57680, 14660, 19412, 55952,
                22024, 12756, 53952, 30448, 21408, 26844, 62632, 5320,  59852,
                46472, 57516, 15048, 49292, 10480, 552,   56956, 40388, 7976,
                61596, 47464, 28068, 33052, 59452, 52352, 18264, 32152, 54068,
                58088, 46860, 64912, 16600, 3032,  60664, 53516, 18924, 45756,
                65236, 38188, 44016, 27632, 58080, 4092,  40568, 52728, 35920,
                6676,  65420, 20772, 52884, 6136,  58728, 15116, 25172, 50592,
                27572, 61940,
            },
            {
                1404,  21960, 55604, 35224, 38944, 23092, 33060, 44196, 37480,
                21096, 65036, 39160, 140,   48696, 9224,  55268, 60224, 21528,
                31416, 56152, 15364, 23600, 64712, 17400, 10504, 33956, 61460,
                20396, 49884, 9740,  36584, 47184, 62276, 28356, 42816, 11540,
                17960, 26572, 45476, 4460,  16972, 63248, 22092, 35808, 15420,
                61652, 20300, 26824, 38260, 8592,  58824, 2928,  55332, 33680,
                23628, 39756, 62876, 17044, 6272,  58896, 37300, 15664, 1720,
                55936, 35048, 49256, 28472, 164,   52712, 33000, 5040,  29772,
                35684, 44928, 63816, 1200,  15376, 56620, 44132, 36048, 2388,
                28076, 7972,  41204, 60768, 23768, 34652, 50652, 27220, 15544,
                36784, 56080, 11748, 6764,  39452, 14760, 45720, 9488,  13320,
                24312, 236,   36252, 51288, 27936, 21072, 42180, 35352, 56664,
                22976, 52000, 4816,  48672, 24488, 20372, 63880, 14540, 58532,
                23408, 42408, 16052, 61100, 27044, 49396, 30752, 42444, 8200,
                11688, 34632,
            },
            {
                59404, 18708, 26736, 2896,  60424, 14288, 4424,  55940, 25112,
                9976,  52388, 32676, 25912, 17748, 29924, 36880, 25348, 43944,
                7336,  47284, 3832,  58280, 31136, 6380,  44696, 57456, 14900,
                2932,  54100, 27700, 4340,  13048, 19844, 55076, 5080,  59208,
                48784, 22716, 35016, 28964, 10720, 31288, 44240, 5376,  28672,
                9444,  48680, 13220, 63620, 47112, 31984, 44024, 19848, 49212,
                15788, 57692, 31268, 35628, 24860, 47840, 12444, 65044, 45692,
                25852, 16776, 61536, 13176, 21256, 62532, 40796, 46640, 60532,
                9932,  26224, 18324, 40972, 47940, 10776, 31844, 18536, 50404,
                21796, 54380, 30700, 4324,  43980, 20096, 63460, 12160, 52772,
                24136, 1496,  64784, 44656, 21864, 29248, 63724, 34692, 50096,
                38924, 18860, 43560, 9956,  58924, 6472,  47564, 848,   14688,
                8880,  30912, 18244, 12204, 36780, 42904, 10332, 28888, 3596,
                48164, 12504, 32472, 37472, 1176,  21912, 39052, 3100,  57312,
                52920, 43896,
            },
            {
                5456,  49848, 42196, 8676,  47396, 28464, 63600, 17124, 50192,
                42488, 15244, 5368,  54324, 41552, 12336, 49972, 1520,  53112,
                18272, 27748, 35336, 20276, 41096, 51880, 37040, 24372, 48180,
                30636, 41364, 64792, 22604, 52488, 32396, 45340, 15732, 36916,
                63664, 1760,  57132, 50964, 38880, 58920, 47712, 19104, 52508,
                42500, 57660, 35080, 520,   23364, 16196, 10540, 65024, 6728,
                42784, 3956,  9496,  51036, 60988, 2492,  27944, 40288, 52176,
                5548,  37744, 41744, 31420, 7756,  36012, 16928, 24600, 1872,
                50064, 57716, 4660,  34436, 59440, 6076,  24588, 65148, 13996,
                39240, 61888, 16924, 36636, 53412, 7268,  29640, 3740,  45556,
                32076, 41116, 16948, 35592, 58536, 2956,  5808,  41388, 57128,
                7252,  62660, 26580, 33808, 54720, 24028, 31848, 62044, 28540,
                40748, 59104, 34040, 62536, 55316, 688,   50788, 33556, 38692,
                56184, 7896,  51688, 46220, 10820, 54304, 64580, 18280, 32560,
                22876, 14604,
            },
            {
                39548, 30964, 12372, 57888, 52168, 20028, 40688, 34532, 7296,
                29292, 62312, 20244, 46900, 59824, 22968, 63996, 16004, 32868,
                59396, 42236, 50596, 8636,  61176, 12896, 28560, 5136,  62792,
                18148, 11556, 34596, 57800, 7576,  39528, 572,   24212, 29572,
                9080,  41864, 13824, 20496, 7364,  25796, 12968, 64872, 3552,
                23944, 30460, 6820,  51212, 60344, 40564, 36076, 25124, 30268,
                61336, 27352, 45964, 13572, 21568, 43372, 8528,  19792, 30832,
                11412, 54076, 3780,  57168, 45888, 50836, 11164, 54448, 38152,
                14108, 43104, 22756, 51756, 27436, 37340, 53804, 45644, 9576,
                33568, 968,   46684, 25408, 13252, 58284, 48016, 38576, 59316,
                19724, 7384,  54552, 24772, 48232, 53420, 20548, 26144, 15908,
                30300, 46364, 4952,  14612, 49028, 11780, 39076, 17480, 53932,
                49756, 2256,  45428, 7716,  27076, 16624, 60024, 45260, 19588,
                25524, 63296, 28736, 19124, 59568, 35180, 13612, 6924,  47052,
                61396, 28180,
            },
            {
                55256, 63372, 24228, 37160, 32228, 1296,  10936, 58960, 46148,
                3196,  56344, 38388, 33952, 2488,  8076,  45124, 39604, 5276,
                10832, 63568, 14180, 25884, 46272, 288,   53860, 21848, 42812,
                37796, 1316,  46236, 25604, 14948, 50492, 61776, 56444, 19580,
                53720, 31576, 46536, 62592, 53416, 2768,  32872, 36832, 40520,
                60636, 15100, 45160, 18896, 27772, 55552, 2272,  52960, 12544,
                37720, 19204, 55776, 39184, 32100, 54216, 58568, 34456, 62172,
                22624, 48220, 18260, 25080, 63508, 2540,  27784, 20388, 64640,
                29112, 32388, 8440,  62456, 12996, 19816, 41532, 3732,  56868,
                23264, 49924, 10948, 64508, 41900, 33284, 17488, 22936, 10340,
                61760, 49968, 30340, 13196, 9692,  37956, 43604, 60984, 11444,
                52572, 22216, 56340, 41120, 19956, 64424, 44904, 4716,  25556,
                12928, 20196, 23856, 38920, 47932, 31672, 22640, 8608,  53732,
                1596,  16336, 41828, 4184,  24512, 44320, 29308, 40672, 51520,
                36464, 9512,
            },
            {
                17196, 46424, 3760,  15740, 44496, 26288, 54452, 22856, 36960,
                13912, 24324, 10644, 28044, 16880, 57976, 30388, 24844, 56948,
                20972, 37972, 3184,  55040, 32300, 17752, 35616, 59556, 8804,
                50912, 55360, 20608, 60372, 30096, 43188, 35192, 11828, 48188,
                39356, 3992,  17248, 34460, 43160, 22588, 49832, 56388, 17768,
                10836, 53852, 38992, 33828, 8996,  48000, 21508, 44992, 59488,
                50808, 48,    23860, 64428, 5004,  17540, 49240, 14592, 6740,
                56084, 43036, 29512, 13688, 38824, 32708, 58892, 41872, 5792,
                47592, 17064, 55760, 39364, 332,   30612, 60400, 17092, 28200,
                43136, 59252, 30068, 20984, 4632,  27408, 54404, 1192,  35364,
                42616, 26616, 4296,  63092, 33728, 17736, 888,   49124, 32548,
                35964, 2796,  60600, 29516, 1432,  34516, 8280,  56012, 36612,
                42916, 60540, 52852, 14408, 64868, 5308,  40460, 13584, 61604,
                30408, 37776, 57788, 34088, 8660,  48600, 62768, 3884,  25220,
                20296, 988,
            },
            {
                33456, 50852, 21832, 60124, 6680,  65320, 48088, 17976, 31088,
                61208, 50816, 44160, 64764, 53328, 36180, 13172, 51852, 47468,
                34240, 28832, 44780, 22600, 39972, 65404, 47536, 14324, 31700,
                27240, 16356, 6036,  38700, 9796,  3408,  18080, 27428, 5532,
                65092, 25012, 10856, 28400, 60936, 14920, 8952,  27444, 1576,
                46916, 26276, 4312,  57280, 13460, 63012, 5084,  32832, 15380,
                7820,  34356, 48472, 10660, 28600, 37156, 1792,  41360, 26992,
                36480, 1108,  60032, 8556,  21964, 44820, 15716, 9372,  35272,
                60980, 2760,  44208, 21720, 49740, 46212, 8776,  51876, 36272,
                6324,  14888, 38632, 55880, 9008,  44900, 63016, 49264, 13664,
                56904, 16360, 45420, 39500, 57844, 28216, 55504, 23784, 65444,
                9136,  44816, 17152, 38208, 50488, 59684, 22468, 30728, 62616,
                10164, 28844, 3608,  35916, 19356, 57404, 51660, 35204, 43676,
                47772, 6964,  50676, 14344, 55192, 21572, 16636, 56172, 12276,
                58628, 42440,
            },
            {
                29256, 7612,  53584, 39404, 34312, 13156, 41736, 8632,  53120,
                852,   40124, 6448,  19720, 48336, 4188,  42880, 18348, 876,
                8256,  61820, 16472, 58652, 6700,  10280, 25364, 2564,  57744,
                45184, 64184, 33056, 48844, 54168, 63476, 22900, 44664, 57384,
                36028, 51152, 58520, 40,    37364, 54964, 41256, 31060, 62064,
                35584, 19908, 64480, 23196, 42984, 29540, 18064, 40204, 26960,
                57220, 42088, 16712, 45076, 59592, 52448, 21888, 46836, 63200,
                10908, 51564, 33904, 47512, 53948, 4808,  56512, 49280, 23716,
                53040, 26320, 11004, 34216, 63352, 14536, 24944, 32552, 64096,
                19480, 47000, 2092,  51388, 35716, 15336, 39984, 6476,  24032,
                32656, 2528,  52332, 22364, 7500,  46680, 14196, 5060,  19156,
                40532, 51508, 25340, 13412, 9896,  26956, 46396, 15764, 2656,
                47496, 33408, 55712, 42108, 11212, 29332, 3928,  26080, 10260,
                23432, 19836, 65052, 26852, 42920, 344,   32220, 37828, 27604,
                45836, 64504,
            },
            {
                38132, 11364, 26520, 19024, 57104, 29884, 4276,  59720, 21388,
                35008, 16172, 26172, 31712, 9388,  22596, 63236, 27412, 54800,
                41140, 48960, 12136, 52220, 36512, 30144, 50260, 40720, 21348,
                4840,  12520, 23748, 42008, 28468, 13708, 47572, 32224, 8736,
                15300, 20956, 43832, 48900, 18992, 4976,  45876, 21608, 58076,
                7628,  49312, 31812, 52292, 2744,  36824, 60420, 49944, 10976,
                22188, 62208, 30716, 6944,  24912, 12932, 32564, 57340, 18820,
                24268, 15968, 40500, 65440, 18652, 26632, 37316, 30496, 12648,
                18772, 38608, 59724, 28824, 5636,  54936, 37868, 1668,  12208,
                54156, 26776, 61724, 31392, 24468, 57360, 20468, 29440, 53128,
                37780, 64980, 19348, 11508, 60392, 31508, 42320, 37452, 58268,
                28696, 5928,  63536, 55104, 42628, 5144,  53260, 37464, 21088,
                51536, 17576, 7216,  25024, 49380, 45592, 63348, 17940, 55376,
                59080, 2108,  31280, 11244, 39288, 60312, 49800, 9868,  53232,
                5504,  14700,
            },
            {
                48840, 61672, 43820, 1124,  46856, 24144, 50220, 38708, 27760,
                45240, 62148, 55984, 41800, 59224, 38476, 32936, 11388, 60088,
                23728, 31776, 20332, 1568,  43244, 56784, 19040, 62668, 34624,
                52680, 37620, 56224, 1488,  19508, 36472, 58748, 2596,  53160,
                41448, 29548, 6812,  32456, 24572, 64220, 11380, 52568, 16168,
                43660, 12060, 40364, 16704, 9640,  54692, 24624, 44596, 5856,
                53080, 37448, 2204,  54844, 39860, 63964, 9056,  4572,  30348,
                45488, 7340,  28108, 3652,  12076, 42972, 59348, 652,   64076,
                46328, 3868,  50188, 16244, 42116, 20112, 58036, 48752, 40244,
                22552, 42476, 10472, 17696, 4120,  47880, 11616, 59084, 43588,
                9548,  27976, 41632, 34916, 50448, 4036,  26248, 54360, 47580,
                11960, 32172, 21732, 35488, 17996, 31248, 58104, 12424, 64576,
                39684, 27024, 58484, 61316, 21672, 56,    39268, 33772, 14260,
                41548, 36284, 45892, 53796, 18052, 6748,  22612, 62348, 34480,
                18792, 23560,
            },
            {
                3560,  30492, 54592, 15412, 63268, 9936,  17372, 55540, 13596,
                5268,  11060, 50128, 1928,  14460, 54004, 46188, 3428,  15640,
                35952, 5912,  64100, 28276, 24124, 14956, 3864,  11608, 44100,
                28112, 17340, 9164,  59772, 50180, 6772,  39716, 17228, 23576,
                63096, 12220, 54608, 60248, 14312, 39640, 34336, 3292,  25256,
                37064, 716,   27544, 61320, 46416, 34244, 14432, 1020,  64524,
                28460, 18868, 13712, 49284, 20356, 34908, 43324, 50420, 37544,
                61280, 53108, 58344, 32204, 22656, 50748, 7988,  34036, 21536,
                41696, 55924, 31176, 9360,  24148, 45132, 4396,  27508, 62580,
                7872,  34388, 59448, 45472, 37168, 64408, 32332, 228,   17388,
                61296, 5500,  48380, 24564, 16108, 63392, 10116, 21464, 16548,
                62144, 3036,  45652, 59040, 49000, 796,   41228, 24508, 8132,
                44356, 1812,  12852, 36976, 15568, 31132, 52784, 6552,  27596,
                49696, 4680,  61924, 24096, 35092, 47428, 30204, 43808, 1640,
                41452, 56680,
            },
            {
                35184, 9112,  21100, 33096, 42400, 36920, 2660,  31592, 64472,
                47780, 33600, 18612, 24628, 28856, 7432,  20232, 50984, 39840,
                57876, 45784, 53204, 37636, 48668, 60828, 33352, 54900, 7764,
                47836, 61508, 30616, 43788, 25720, 11536, 61664, 28788, 49580,
                34944, 1892,  38080, 46612, 8404,  27828, 59368, 49888, 30088,
                62720, 56196, 51032, 22292, 6908,  58756, 20264, 31728, 39352,
                47760, 34012, 60004, 44220, 26612, 2916,  16876, 55536, 13200,
                300,   20416, 35784, 48740, 39156, 61468, 16940, 53456, 28176,
                14764, 7056,  36008, 52164, 65260, 34772, 13312, 31632, 17316,
                50820, 2732,  14380, 53428, 27864, 7536,  40380, 51236, 22664,
                45588, 14216, 57656, 2148,  55612, 36508, 44164, 364,   34360,
                50068, 40188, 8720,  15016, 27920, 63168, 19976, 34712, 55200,
                29132, 48084, 32836, 53564, 42736, 8912,  57508, 46840, 62784,
                21540, 9752,  29088, 14936, 2536,  56908, 11904, 15860, 51540,
                25920, 59668,
            },
            {
                13560, 40296, 51164, 4848,  27456, 60916, 53004, 25212, 40976,
                22152, 58408, 37804, 44728, 63728, 35412, 60628, 29860, 25808,
                9856,  18552, 13704, 4372,  9020,  45108, 39372, 26592, 23060,
                2312,  40312, 13872, 35140, 21036, 55084, 33272, 46140, 4792,
                57792, 26420, 17720, 51604, 22252, 43032, 55588, 19688, 6592,
                45576, 13364, 18660, 38416, 29216, 41672, 50648, 57060, 12124,
                8828,  23916, 5216,  56952, 10716, 61992, 29104, 22720, 41192,
                25788, 44388, 9316,  14124, 2152,  25184, 45272, 10744, 37832,
                63024, 25432, 57452, 18520, 880,   47460, 60948, 9924,  43828,
                56016, 29980, 39216, 23340, 19452, 49732, 13356, 25932, 35896,
                53592, 33392, 29724, 39744, 19684, 27780, 52212, 59356, 30180,
                24992, 53892, 37212, 23168, 6508,  45048, 11108, 50248, 4316,
                16088, 61988, 22700, 5172,  65436, 19776, 24668, 12312, 37584,
                17304, 54968, 44280, 50508, 59936, 40776, 27284, 64788, 20700,
                7036,  46780,
            },
            {
                18116, 63852, 56072, 22908, 47484, 12020, 19048, 44168, 8168,
                368,   29972, 9604,  51432, 4980,  17008, 48236, 43228, 608,
                54932, 33996, 41968, 58940, 20992, 30500, 16524, 64356, 51908,
                19632, 57136, 4408,  65212, 51652, 456,   41608, 14952, 10136,
                20632, 44192, 64776, 5464,  31664, 936,   16780, 10360, 40580,
                23536, 31952, 4704,  65108, 11288, 2984,  25688, 16452, 43632,
                62080, 51892, 17376, 30936, 38328, 51112, 46728, 6584,  64656,
                31892, 57532, 17876, 63676, 55160, 30044, 5536,  57988, 42568,
                1904,  48436, 12468, 39800, 29520, 21916, 53240, 37372, 24828,
                19900, 6568,  63272, 57628, 1980,  43300, 62372, 56596, 3344,
                20144, 8336,  64040, 11704, 47068, 7624,  13772, 41212, 5680,
                19064, 13116, 61416, 56568, 33540, 52720, 38976, 60152, 31660,
                19280, 56988, 10592, 40360, 49788, 29264, 44608, 4016,  33152,
                58352, 39696, 5964,  31940, 19572, 37120, 8484,  53716, 32968,
                38540, 28852,
            },
            {
                49524, 656,   36232, 14840, 7152,  57436, 33448, 49940, 15828,
                60052, 53212, 13952, 56472, 40676, 23792, 12692, 8144,  64816,
                22140, 31020, 62184, 25436, 50560, 55560, 136,   12720, 36420,
                32368, 49460, 24208, 45544, 8516,  17684, 27628, 62736, 38404,
                55380, 30380, 13088, 36020, 61324, 47176, 37872, 63988, 48720,
                58320, 36040, 53400, 43976, 48820, 59860, 36752, 54820, 3804,
                27712, 35656, 41596, 948,   21704, 14208, 59320, 36096, 10456,
                49276, 3012,  27580, 41560, 47320, 36540, 20872, 51140, 18040,
                31464, 22440, 44796, 59764, 6016,  15152, 41568, 4972,  59052,
                49316, 35856, 46568, 9620,  33376, 16556, 30468, 10632, 38488,
                59836, 42868, 23968, 50896, 60568, 32752, 23120, 64804, 56008,
                47340, 1676,  43628, 29704, 17272, 2716,  26112, 13888, 46168,
                37184, 43344, 26360, 35396, 2332,  16808, 59664, 51580, 26516,
                928,   22828, 13484, 63592, 25444, 4164,  48800, 44720, 2384,
                60748, 10724,
            },
            {
                24716, 31792, 44640, 59812, 29488, 38596, 4176,  63204, 28268,
                39940, 35108, 20364, 26528, 2496,  32180, 57972, 38460, 51836,
                16000, 46608, 2476,  11360, 35840, 6208,  41660, 46668, 59352,
                10060, 15676, 28748, 38308, 31704, 60068, 23400, 52780, 7064,
                47556, 24048, 41088, 9144,  53292, 24420, 13784, 33616, 27088,
                2288,  9132,  15648, 22820, 33324, 18220, 8112,  31328, 22008,
                47324, 7580,  65332, 53888, 45236, 32908, 3684,  19408, 39392,
                15468, 54384, 33964, 22520, 8176,  12492, 62740, 33288, 4564,
                61028, 54824, 9664,  34000, 51328, 26748, 55732, 32848, 1636,
                15896, 28140, 12888, 40904, 24720, 54228, 46980, 5160,  28340,
                48280, 15384, 1172,  37176, 6052,  17656, 45364, 38256, 9384,
                35552, 27012, 51268, 10404, 41656, 58404, 22568, 8604,  63920,
                512,   7472,  52056, 13344, 56424, 31752, 38972, 9848,  64164,
                48496, 41804, 52976, 34788, 11012, 55296, 17324, 23360, 14188,
                56336, 42960,
            },
            {
                52660, 19800, 8756,  25700, 41640, 52080, 21228, 10932, 24140,
                47148, 3604,  65288, 43468, 48940, 61828, 18516, 27788, 34848,
                5304,  40304, 49416, 29048, 17352, 62976, 22800, 27104, 6716,
                43392, 61952, 5516,  56604, 12992, 42724, 49260, 3420,  35196,
                16564, 59648, 1556,  19936, 58232, 29304, 3708,  54588, 20824,
                61740, 41224, 56876, 28136, 200,   63196, 45572, 13804, 39512,
                60508, 19840, 12724, 25448, 9000,  56692, 27952, 52476, 24104,
                62696, 45140, 5032,  51856, 59136, 16328, 40832, 26216, 46648,
                14252, 28364, 3212,  38112, 62500, 19532, 11968, 64420, 45628,
                22104, 60484, 51684, 4300,  65032, 35424, 21292, 61456, 52400,
                18316, 32340, 55488, 26656, 42100, 54520, 2424,  28160, 14800,
                21868, 62216, 4280,  20500, 65196, 35444, 47832, 53992, 28248,
                20776, 59276, 47456, 23788, 63136, 6424,  21852, 14704, 35696,
                18872, 29968, 7448,  46040, 58972, 42276, 31180, 63256, 27844,
                34992, 5112,
            },
            {
                65464, 12712, 54756, 62228, 16688, 1312,  45772, 36624, 56648,
                7532,  17264, 30596, 10812, 36424, 6872,  45304, 55152, 10664,
                24936, 56432, 20128, 58396, 38396, 52412, 31344, 56292, 18640,
                39200, 53532, 21716, 46992, 2284,  36080, 19200, 29816, 12236,
                63752, 32768, 51348, 45492, 16060, 42272, 50268, 7648,  44572,
                17616, 31072, 46784, 12252, 38588, 50352, 24696, 57156, 52776,
                2012,  33468, 48772, 58612, 40060, 16468, 42976, 60824, 7424,
                30384, 11700, 18604, 38004, 29200, 560,   56032, 7480,  35732,
                53228, 41508, 17108, 24184, 43948, 8352,  47964, 29604, 39180,
                7436,  54628, 31540, 18556, 44868, 596,   14420, 39648, 25040,
                6704,  45084, 58992, 13188, 20760, 63040, 48968, 31412, 52144,
                57048, 40240, 32156, 49484, 15408, 5804,  31052, 42788, 16700,
                39656, 33424, 29976, 17784, 45380, 41048, 54032, 46724, 4452,
                55568, 60696, 16492, 27452, 21008, 524,   38248, 9328,  50828,
                21236, 40740,
            },
            {
                2708,  37288, 46696, 6128,  34616, 57996, 13904, 31232, 53628,
                42172, 61024, 50108, 23192, 54024, 14828, 21328, 1256,  42160,
                60256, 14128, 8212,  43856, 3900,  13556, 8784,  48624, 35084,
                1160,  25500, 33416, 17084, 63876, 9568,  59228, 54484, 44020,
                25844, 5852,  38784, 28060, 11112, 35072, 59900, 38392, 12840,
                25384, 51124, 6084,  60596, 20100, 42368, 4444,  28584, 10516,
                43428, 17072, 29632, 35936, 4336,  22848, 47188, 680,   35236,
                41924, 56780, 25928, 48944, 65164, 43860, 23700, 50528, 10956,
                21200, 64684, 57604, 31372, 50284, 64,    36408, 16752, 25268,
                43132, 10796, 37516, 26968, 58188, 9156,  42608, 56968, 11644,
                36768, 64768, 9048,  30416, 35484, 4668,  11736, 60208, 17368,
                6864,  45960, 12804, 25492, 55524, 38368, 1648,  12000, 50520,
                61728, 4236,  55704, 10768, 1148,  34044, 61868, 28632, 23932,
                43000, 10568, 2980,  36580, 51800, 12828, 60148, 47264, 6600,
                58220, 16220,
            },
            {
                48976, 32620, 22036, 28892, 50368, 23896, 64308, 19428, 4580,
                27128, 13368, 34484, 1916,  59288, 40260, 29464, 64112, 32356,
                50764, 36692, 27676, 64320, 33520, 45768, 21108, 60444, 14784,
                64888, 10944, 49760, 40516, 52452, 26816, 22348, 39644, 8100,
                48396, 21224, 57032, 2844,  65468, 22312, 4844,  31708, 55960,
                64332, 1528,  36260, 54244, 8844,  34500, 16696, 59564, 37564,
                23456, 63720, 6288,  51220, 61864, 11120, 31804, 50344, 13700,
                21340, 60276, 2580,  9812,  34980, 19380, 32108, 61876, 38524,
                1632,  45408, 6608,  11596, 55308, 21724, 61404, 58168, 50704,
                2372,  63088, 15120, 48328, 22536, 51436, 29468, 32904, 49020,
                3624,  22548, 40540, 47544, 53352, 25216, 38832, 43052, 23240,
                832,   36712, 58744, 7924,  44300, 21600, 63116, 57948, 24612,
                9244,  15008, 37488, 52696, 26664, 19592, 7852,  12632, 51148,
                32700, 38852, 65152, 47744, 56896, 24788, 33404, 19004, 44440,
                30336, 26004,
            },
            {
                39736, 60816, 17888, 11236, 40116, 2968,  43144, 48896, 9968,
                39060, 57704, 18936, 46072, 25888, 9252,  52432, 47240, 17300,
                5000,  21856, 48444, 764,   24292, 54560, 40800, 5200,  29964,
                44204, 57872, 28520, 3924,  13792, 45636, 32320, 636,   57884,
                17444, 35816, 13280, 54088, 46952, 26228, 18320, 47892, 9500,
                21168, 40164, 15076, 27440, 23076, 63872, 51500, 32008, 7356,
                46092, 55116, 27116, 14356, 38680, 56356, 18888, 64592, 27432,
                53344, 39012, 46940, 15228, 54472, 3656,  12900, 47400, 17500,
                30240, 25628, 35104, 15344, 39908, 27684, 5984,  13120, 33932,
                20624, 56280, 5436,  34268, 61164, 6564,  17028, 63004, 19856,
                54092, 27552, 15200, 68,    18752, 57544, 8416,  33608, 48148,
                27136, 62844, 18996, 33972, 29420, 51636, 17224, 27592, 34832,
                48824, 41908, 21808, 65012, 43672, 49728, 38436, 57108, 2512,
                58912, 19420, 22752, 30512, 4744,  15684, 40556, 3092,  62712,
                10888, 55180,
            },
            {
                5560,  468,   44148, 53708, 59492, 8380,  27988, 33356, 61636,
                22328, 51664, 5788,  31596, 62540, 37732, 3720,  24592, 12328,
                61692, 39768, 30744, 15480, 59296, 10780, 26836, 51456, 37228,
                22952, 7932,  18908, 59924, 37852, 5812,  62648, 14880, 51588,
                29228, 61900, 43260, 32196, 8520,  40400, 61648, 52988, 28552,
                43780, 32900, 48884, 58184, 44760, 13212, 1716,  47692, 19500,
                12288, 40820, 1228,  20764, 45396, 25272, 2944,  37652, 8800,
                17148, 6268,  31476, 22448, 42352, 59692, 27968, 56428, 5168,
                52520, 60296, 48728, 63152, 43516, 32352, 53760, 46428, 40716,
                30224, 24116, 45432, 41676, 12608, 38776, 26032, 2120,  43288,
                34824, 58192, 50712, 62168, 44660, 29240, 14640, 64364, 55172,
                11016, 40980, 53452, 2876,  61204, 10064, 41276, 6688,  45220,
                2568,  56936, 29076, 5820,  13232, 60864, 31204, 17360, 26156,
                45584, 14000, 50400, 8752,  44052, 61584, 28776, 53504, 22692,
                35620, 14632,
            },
            {
                51784, 24224, 62932, 31292, 15092, 36820, 52540, 16904, 45120,
                88,    35772, 41600, 11208, 16200, 56980, 20032, 35540, 44576,
                54772, 9772,  52016, 43012, 34948, 18676, 63368, 2444,  16724,
                46884, 54240, 41960, 30800, 21260, 55392, 35324, 25048, 41184,
                10176, 3612,  23844, 50696, 16212, 296,   36952, 14100, 3076,
                60956, 17232, 7632,  4032,  30852, 41196, 55804, 26640, 60928,
                34868, 58028, 48484, 32508, 54072, 7692,  43484, 58556, 49112,
                33852, 44188, 63360, 26324, 51304, 36440, 8272,  39620, 22816,
                41888, 8864,  19312, 2080,  23580, 9972,  18444, 3352,  65424,
                8428,  16380, 50940, 1348,  30992, 54548, 58988, 46252, 13988,
                4912,  9648,  31688, 37296, 5872,  22388, 51572, 2396,  20348,
                30976, 15996, 48804, 23632, 13852, 46788, 36188, 55220, 19276,
                32260, 59940, 16660, 34512, 24284, 4152,  47300, 9816,  36052,
                62828, 5860,  41432, 34032, 55756, 11528, 37676, 48548, 7788,
                43008, 57724,
            },
            {
                19896, 38564, 7016,  26408, 46600, 21452, 4240,  56324, 25308,
                14364, 63712, 27324, 55300, 48872, 43276, 30560, 6996,  58776,
                26220, 2000,  19768, 56736, 6316,  49444, 38984, 32176, 57284,
                12540, 34316, 1412,  63412, 47884, 11316, 49912, 18456, 59008,
                46128, 38156, 19680, 63344, 28772, 56760, 45608, 22824, 35152,
                57300, 25044, 52100, 61884, 20248, 9240,  38240, 15892, 5024,
                22460, 29828, 9912,  16456, 63884, 35404, 28976, 21604, 13224,
                61180, 1604,  55928, 11160, 5016,  19980, 64388, 15524, 34148,
                58416, 13240, 28224, 37760, 56124, 59584, 49768, 26456, 36316,
                53076, 58060, 27332, 62732, 18796, 22132, 10420, 52104, 40172,
                63936, 24200, 17076, 42496, 12112, 59292, 39476, 45464, 35912,
                7540,  43844, 4520,  38736, 57596, 25832, 376,   64352, 12772,
                23984, 7608,  46520, 51448, 40100, 55176, 21384, 42392, 53848,
                92,    28256, 52880, 17040, 25412, 20800, 1248,  64600, 17592,
                32312, 28232,
            },
            {
                9876,  49892, 34784, 12464, 54672, 64884, 40488, 10804, 31104,
                59388, 46956, 20268, 7856,  23412, 2240,  63896, 50132, 13372,
                33164, 37800, 65104, 27500, 13728, 46112, 8696,  23972, 61752,
                4856,  52108, 26260, 15600, 8192,  43744, 28208, 2364,  7428,
                30644, 55772, 12036, 5620,  42012, 33200, 10192, 50256, 6012,
                41480, 11332, 37840, 46124, 34748, 23900, 65256, 49516, 42416,
                52444, 62704, 5940,  39824, 51496, 11952, 4652,  55036, 40256,
                24044, 29480, 18204, 40808, 49528, 31192, 46048, 53424, 476,
                44344, 31432, 51296, 45880, 5460,  34536, 13940, 44036, 20036,
                11020, 39068, 4956,  35100, 47796, 7752,  37244, 32284, 28096,
                20568, 47232, 56276, 26492, 49368, 32304, 4112,  25452, 61076,
                56844, 28020, 64104, 32684, 50960, 20460, 30920, 39876, 49664,
                53580, 38048, 11224, 63688, 1712,  27308, 59736, 14516, 32616,
                23772, 39072, 60620, 7060,  58368, 45700, 50988, 26780, 13112,
                59572, 47088,
            },
            {
                63952, 16652, 42176, 58740, 2316,  18856, 34004, 50500, 7244,
                37888, 3800,  53512, 33968, 38964, 14928, 27724, 40888, 18028,
                47716, 22896, 41876, 4328,  30464, 60464, 20988, 43868, 28392,
                40860, 19644, 36908, 56740, 23356, 33280, 64252, 53664, 21024,
                61272, 34124, 47916, 52628, 24808, 59652, 20172, 64464, 27420,
                54320, 19456, 29556, 14028, 2804,  53508, 28576, 11604, 452,
                36912, 18328, 44400, 26952, 23256, 60440, 18532, 32128, 47912,
                9532,  52560, 37412, 59468, 13608, 3008,  24156, 27280, 10304,
                62596, 21280, 3640,  64468, 17168, 40852, 29988, 62244, 548,
                48372, 31992, 13720, 43184, 56420, 24960, 61916, 16092, 6532,
                60056, 36212, 2020,  7840,  65280, 19496, 54376, 16692, 13060,
                49724, 22272, 10816, 17472, 7956,  60772, 5008,  16128, 28668,
                61748, 20960, 43232, 30740, 18688, 36892, 7316,  49356, 64584,
                18232, 47588, 12588, 36836, 30032, 4588,  34296, 41124, 6176,
                36796, 2456,
            },
            {
                54196, 30652, 22396, 6068,  44968, 28800, 24068, 58316, 43412,
                17988, 29152, 12040, 44428, 57776, 60780, 9944,  52652, 4696,
                61072, 8568,  15924, 50276, 53488, 37104, 212,   54420, 14568,
                48796, 10124, 60852, 45248, 4008,  40420, 16384, 36800, 43120,
                13984, 26472, 2124,  17604, 37600, 13440, 3476,  31336, 15692,
                47248, 1144,  63128, 49860, 59236, 17024, 44600, 31876, 58804,
                25156, 13096, 55920, 2480,  49496, 38024, 46116, 104,   65416,
                15824, 5484,  44736, 21468, 34272, 62024, 55652, 41280, 18084,
                49196, 36716, 25592, 53976, 11344, 22712, 7072,  54908, 24736,
                58780, 21748, 64056, 17476, 3276,  40452, 49188, 508,   44748,
                53496, 12976, 50932, 29960, 38196, 10288, 46928, 34428, 41484,
                1016,  37524, 53784, 45852, 35500, 41700, 56000, 44404, 9468,
                1976,  26184, 5644,  15264, 56880, 45704, 11844, 29388, 3300,
                57756, 9404,  21924, 42456, 62464, 14848, 53368, 18944, 56212,
                45276, 25004,
            },
            {
                39572, 13376, 62152, 51704, 37640, 9464,  48332, 13864, 61736,
                22680, 51308, 65272, 1152,  30928, 19088, 36744, 24240, 55752,
                29748, 44688, 57412, 34208, 24428, 11084, 17692, 33516, 64672,
                6540,  31648, 50976, 28804, 12860, 59844, 6300,  24928, 51964,
                9504,  39880, 65096, 45200, 8292,  54488, 49096, 43444, 38544,
                58272, 9628,  32488, 22240, 38860, 6256,  56208, 8476,  20436,
                46604, 33636, 61008, 30708, 8328,  14576, 58304, 42548, 25120,
                36164, 56860, 26896, 8512,  48096, 16628, 37704, 6968,  32476,
                56564, 14532, 8128,  42524, 33116, 60760, 50700, 36700, 16184,
                41456, 6024,  45928, 26860, 51708, 12676, 30216, 57460, 19236,
                33552, 41056, 23220, 15616, 44484, 59792, 28316, 6860,  63672,
                19284, 30712, 59740, 3160,  26948, 11920, 22632, 33132, 47172,
                36304, 59220, 48748, 33844, 62264, 52692, 20600, 40992, 35128,
                43992, 52280, 32160, 1832,  49332, 23624, 10432, 60184, 31156,
                21500, 8136,
            },
            {
                49324, 27540, 1132,  32800, 56768, 16828, 41724, 1388,  5696,
                34836, 40120, 15800, 25716, 49824, 5148,  46476, 32892, 564,
                39016, 12016, 20164, 2736,  63288, 40432, 58816, 46516, 25300,
                39196, 22160, 1060,  17772, 54620, 48024, 31068, 58084, 472,
                49164, 22228, 29716, 58996, 35420, 27160, 62664, 23292, 7080,
                34972, 25176, 44260, 4308,  42076, 26976, 36136, 48988, 63228,
                39108, 4656,  52584, 41440, 21416, 34328, 27624, 10952, 20376,
                49584, 31864, 60564, 1120,  52812, 29272, 11876, 47308, 61084,
                1880,  29044, 59908, 46760, 19232, 27916, 2352,  45156, 9096,
                33788, 54416, 29692, 60692, 8692,  34540, 22684, 64892, 10652,
                26296, 3444,  62204, 57124, 21012, 1564,  51956, 23752, 55408,
                9476,  43348, 24764, 15232, 52236, 48632, 65388, 17944, 57444,
                14340, 52096, 40792, 22444, 9284,  1100,  25544, 56308, 16224,
                6168,  27248, 63908, 19368, 55876, 28312, 35812, 396,   42396,
                15496, 61400,
            },
            {
                34608, 19172, 46924, 11240, 25292, 64008, 30772, 53588, 26364,
                56584, 47268, 8464,  21460, 54732, 42016, 13992, 64444, 21800,
                50928, 61912, 27328, 48880, 43752, 7792,  29444, 3400,  12196,
                57408, 43464, 61968, 34448, 38820, 20576, 10756, 44840, 18664,
                33024, 56920, 15288, 4760,  20564, 40876, 756,   18096, 12616,
                61764, 52300, 56488, 11740, 64496, 19116, 13544, 23704, 2024,
                16084, 28360, 10608, 17676, 64068, 54716, 6028,  51728, 61960,
                3480,  13256, 39628, 23596, 42412, 64864, 4332,  22796, 43872,
                19868, 39724, 52472, 3988,  37708, 57708, 13340, 64928, 26152,
                49984, 11996, 1804,  37812, 19728, 54028, 46880, 38848, 7300,
                43748, 54596, 31124, 6132,  35648, 42464, 14672, 31948, 38888,
                48120, 57308, 6008,  62528, 29956, 37392, 2436,  6980,  25008,
                31304, 11104, 4160,  28900, 44412, 38844, 31524, 50092, 61660,
                23312, 38364, 13524, 45128, 8268,  39208, 48248, 64964, 26164,
                52284, 4044,
            },
            {
                57080, 43624, 60460, 40208, 20868, 7520,  49636, 37344, 19152,
                11296, 62472, 32532, 60188, 38036, 10680, 57264, 26728, 8088,
                16888, 35688, 5832,  32212, 14212, 19076, 55800, 49600, 35960,
                16620, 52700, 8972,  24656, 3172,  64844, 26740, 36116, 62820,
                5472,  42584, 53788, 11548, 31856, 46824, 57296, 51208, 30424,
                39848, 20412, 16424, 29004, 50624, 59752, 32944, 55132, 43216,
                50504, 57868, 24192, 48252, 44296, 2200,  36756, 40636, 29848,
                45016, 19212, 55188, 10068, 33452, 18720, 57852, 26964, 54096,
                12744, 34280, 24352, 16464, 10112, 48304, 20980, 30904, 40016,
                18400, 62760, 23996, 43640, 58384, 5392,  16304, 28536, 51204,
                35240, 16852, 47408, 11440, 49936, 25812, 62568, 3564,  17580,
                12100, 33872, 21680, 40916, 9192,  19560, 53668, 42944, 39320,
                63232, 19992, 60168, 54780, 17312, 65060, 13976, 4068,  10564,
                46672, 54676, 2764,  33432, 59164, 17000, 5092,  12792, 45952,
                37592, 10148,
            },
            {
                22892, 6236,  15272, 52196, 3056,  34544, 59864, 14792, 45848,
                29312, 2972,  43684, 6276,  28196, 17948, 34588, 48412, 43512,
                54108, 40176, 57956, 23296, 52816, 38032, 26236, 61792, 20908,
                5332,  28088, 46620, 14668, 50432, 41396, 7560,  53008, 13552,
                28564, 38128, 24084, 50644, 60964, 16072, 9588,  25744, 45360,
                5972,  1900,  48032, 36980, 7824,  3272,  46216, 10372, 30272,
                6112,  35124, 61704, 6720,  31260, 15500, 22808, 57040, 16868,
                7748,  35260, 61420, 46872, 14684, 51184, 39188, 35764, 8672,
                64036, 49960, 6080,  62840, 32188, 54996, 43472, 6528,  56040,
                4256,  35528, 47316, 14316, 31604, 41288, 2988,  61656, 13416,
                59624, 24660, 20156, 64152, 39704, 8836,  53972, 45924, 58472,
                28676, 51568, 112,   47200, 34960, 57072, 12872, 28024, 49676,
                860,   45320, 36616, 24724, 6620,  48008, 57332, 33980, 42080,
                29052, 18312, 61000, 25336, 41380, 51972, 21300, 32536, 58052,
                17780, 29188,
            },
            {
                54132, 31696, 36856, 26876, 56144, 42968, 22572, 4772,  54496,
                39484, 23588, 14396, 49320, 53336, 1500,  59272, 4472,  31056,
                12800, 2048,  46408, 10596, 65456, 1180,  45136, 9660,  31484,
                42208, 63916, 36988, 56392, 30528, 58656, 16908, 22812, 47080,
                60492, 8848,  2740,  43960, 34552, 4192,  38292, 64692, 33580,
                54884, 60496, 41708, 31996, 24168, 53340, 38320, 17172, 64172,
                21860, 41068, 12820, 38432, 26028, 60116, 48932, 11588, 64440,
                24596, 52468, 27156, 3084,  30640, 7176,  44996, 304,   16976,
                30124, 42268, 58436, 27256, 40836, 1436,  23876, 61780, 15340,
                51488, 21560, 59496, 9472,  50760, 25612, 55452, 22172, 45820,
                804,   38424, 4712,  55672, 27840, 18600, 34628, 22584, 5676,
                37716, 64652, 26044, 16256, 60428, 44788, 23072, 61840, 34108,
                8956,  14972, 51252, 31012, 11980, 37624, 19568, 22728, 52968,
                7800,  36360, 49736, 5960,  29908, 9760,  55432, 24644, 3252,
                40416, 62296,
            },
            {
                47704, 600,   65460, 18628, 12832, 46636, 28192, 10016, 64644,
                33992, 58760, 20012, 36024, 64132, 24088, 40784, 21308, 62628,
                25664, 19308, 59968, 28620, 41936, 16428, 34444, 54584, 13640,
                59076, 1552,  22688, 11840, 4624,  44172, 34152, 1464,  40236,
                55600, 17668, 26624, 63576, 21624, 28920, 53288, 19068, 13152,
                22224, 27080, 10208, 15124, 62432, 20724, 42480, 27144, 57152,
                1080,  53880, 18948, 45508, 55992, 8564,  33856, 644,   46300,
                38780, 5328,  41920, 57512, 22156, 62576, 24976, 59872, 48260,
                21128, 3588,  11756, 46100, 18572, 53096, 34800, 11436, 38288,
                26768, 41988, 108,   28808, 63584, 17648, 37280, 33064, 9980,
                29740, 57808, 42964, 32308, 13168, 1272,  41772, 60856, 14504,
                44264, 19772, 54744, 11152, 7220,  31836, 3256,  16892, 55072,
                21752, 27192, 58672, 3280,  42792, 61196, 27752, 408,   62992,
                15796, 58112, 22080, 13108, 63420, 44708, 34872, 60552, 50296,
                7772,  14172,
            },
            {
                25396, 10580, 41992, 50032, 7944,  61564, 32484, 18088, 41512,
                156,   51404, 8688,  46828, 30604, 12064, 45172, 15428, 51176,
                37700, 55272, 33652, 7468,  21336, 51512, 4748,  23808, 47784,
                38568, 17364, 33336, 49156, 20224, 27652, 61244, 51100, 11680,
                31144, 35652, 47652, 13972, 56588, 49128, 7292,  43096, 2704,
                36468, 49932, 56116, 44580, 280,   58380, 12524, 35480, 8472,
                47676, 33104, 27964, 2808,  50388, 20840, 42656, 28512, 54464,
                31944, 20540, 15928, 49404, 37152, 10868, 52936, 13868, 32704,
                56408, 38172, 23044, 35568, 14308, 8108,  29032, 49208, 60080,
                32424, 7732,  53608, 34496, 12296, 45196, 6960,  49052, 64624,
                15416, 51868, 7864,  46352, 59704, 52384, 29932, 48420, 10088,
                32432, 2948,  41496, 28320, 49132, 38896, 52036, 6524,  41652,
                46688, 64392, 39140, 18056, 54220, 8716,  46192, 50668, 39804,
                31084, 43840, 2068,  47380, 37892, 15204, 920,   28480, 19712,
                36392, 43956,
            },
            {
                33240, 21248, 56912, 29756, 39048, 1868,  52944, 48364, 24956,
                13296, 56240, 26936, 16840, 39120, 7124,  56500, 32988, 9584,
                5452,  44008, 14284, 48348, 36996, 57028, 29648, 61424, 8060,
                25916, 44456, 53100, 7232,  63548, 38104, 15192, 25548, 3844,
                21180, 59128, 40936, 36,    10904, 39488, 25224, 58144, 46968,
                61516, 17620, 6768,  34604, 28796, 48644, 5944,  51752, 24748,
                15396, 60316, 10252, 65244, 14428, 36868, 62040, 17764, 12752,
                59428, 9720,  63224, 1424,  33140, 18472, 40148, 28148, 43152,
                9768,  51992, 62308, 48036, 57324, 64284, 20580, 44896, 3072,
                17884, 46584, 57004, 20640, 39872, 2520,  58000, 19360, 41500,
                26564, 36044, 21640, 16640, 24424, 37088, 7264,  55344, 25488,
                62856, 51016, 35144, 61504, 18764, 24388, 58608, 29224, 35928,
                11252, 5344,  32648, 49384, 23568, 34860, 14716, 5156,  25104,
                11728, 53544, 33668, 27140, 54880, 18712, 41720, 48612, 11912,
                63668, 53220,
            },
            {
                16132, 61124, 5828,  35360, 24064, 58164, 15748, 37556, 62948,
                30952, 5692,  42804, 62236, 3000,  52948, 27336, 63588, 47576,
                29436, 61372, 3768,  24616, 63744, 11944, 40008, 19392, 50664,
                3572,  57760, 29156, 41012, 55460, 9808,  46240, 54036, 43220,
                65208, 7396,  51532, 19948, 30168, 62288, 16368, 32604, 9356,
                30004, 40536, 25356, 64060, 19016, 38724, 31172, 63340, 44828,
                37240, 52640, 42112, 23236, 31468, 4080,  52060, 5768,  23592,
                35020, 50784, 43396, 28656, 47056, 55608, 4720,  65248, 2484,
                19396, 26560, 7320,  752,   30748, 39324, 5244,  56236, 24768,
                10348, 65204, 14804, 25160, 49752, 62104, 30572, 24016, 54164,
                1852,  11624, 62520, 49668, 3416,  65432, 20504, 15612, 39588,
                4536,  22708, 12620, 56216, 2112,  43920, 13892, 63060, 19940,
                56636, 25588, 14048, 2392,  57936, 29384, 63792, 37328, 59944,
                20820, 9004,  65304, 4484,  22960, 61856, 6684,  57120, 23516,
                30360, 2404,
            },
            {
                40560, 51128, 46044, 13880, 20116, 44352, 9324,  4516,  21372,
                45804, 50244, 33688, 23180, 59840, 19820, 35748, 660,   16500,
                20944, 39772, 53372, 17808, 31532, 244,   46296, 15240, 35156,
                64924, 10436, 14472, 24272, 500,   19168, 31724, 6252,  36764,
                28732, 16552, 33204, 54868, 45832, 36148, 3228,  22636, 54080,
                14332, 2156,  51300, 10576, 54308, 14040, 4040,  22316, 17908,
                3044,  29588, 6604,  47288, 57732, 40528, 25880, 56548, 48056,
                40064, 3828,  25376, 13228, 60676, 8368,  24100, 49820, 36436,
                59260, 45020, 33504, 17384, 24380, 43172, 11848, 36036, 52260,
                38672, 30668, 43316, 4172,  36564, 8892,  14068, 43720, 4944,
                38340, 56368, 29344, 34168, 41064, 10792, 44588, 58040, 28820,
                47392, 17320, 45188, 7600,  37512, 32468, 9792,  47736, 416,
                51640, 44304, 61096, 41104, 20940, 10980, 44556, 17788, 55060,
                42460, 48552, 16708, 40660, 50900, 32032, 9616,  45864, 38772,
                5036,  58840,
            },
            {
                25860, 9844,  28212, 3472,  54948, 61928, 26292, 34376, 59532,
                11196, 18496, 38092, 9208,  14228, 44348, 49560, 24496, 42376,
                58324, 10324, 27124, 49508, 43128, 9520,  59580, 27528, 42060,
                31780, 20676, 37220, 60476, 34048, 48012, 62684, 23308, 13564,
                57484, 44476, 9872,  23464, 6148,  58848, 42516, 50364, 65040,
                37200, 45524, 59036, 23300, 41980, 46380, 60644, 56176, 39964,
                11484, 58560, 20380, 35252, 16352, 11148, 44844, 30016, 8432,
                64724, 17288, 57224, 37964, 20964, 44920, 16364, 30824, 11940,
                53700, 14740, 40956, 60376, 54708, 50232, 15572, 27932, 60912,
                1104,  22360, 50868, 60240, 27376, 17492, 52600, 33796, 61252,
                47972, 18072, 44908, 5992,  25708, 51620, 33092, 540,   36576,
                52868, 59920, 30756, 26500, 64956, 53216, 22068, 40332, 33940,
                16028, 28500, 36804, 7204,  48028, 52952, 1428,  32412, 27028,
                3456,  35600, 29612, 59432, 12360, 36124, 26480, 52564, 14416,
                34260, 18420,
            },
            {
                55728, 37072, 64144, 49048, 40068, 30556, 50792, 42708, 54536,
                28816, 1116,  57068, 52464, 30000, 40012, 11744, 7508,  55692,
                30948, 36668, 2684,  60684, 34676, 56140, 23348, 51840, 2308,
                55348, 45072, 49712, 4012,  42712, 16728, 56552, 38744, 49796,
                4220,  25960, 48488, 63904, 15284, 27060, 12144, 19264, 5348,
                28424, 20828, 33296, 6344,  27924, 34952, 8884,  26176, 32756,
                49540, 62636, 27072, 54688, 912,   63080, 19520, 37808, 14160,
                21988, 53872, 31400, 6436,  51820, 34740, 41292, 56816, 5316,
                22412, 29404, 48952, 10160, 3308,  21616, 62924, 33724, 18768,
                40900, 13400, 7104,  32812, 56272, 40196, 46316, 21304, 12440,
                27736, 8808,  59768, 14856, 55216, 18728, 60796, 12808, 21592,
                6088,  10008, 41236, 14548, 49780, 17880, 5656,  60104, 24480,
                8392,  54764, 18988, 65100, 31016, 24784, 57024, 8232,  50320,
                14264, 57648, 7136,  20192, 45228, 1660,  60228, 20768, 64876,
                48864, 44464,
            },
            {
                22492, 1628,  32280, 18904, 11788, 6356,  17048, 2676,  13644,
                40708, 65224, 24776, 47048, 3724,  58676, 64380, 34028, 51396,
                13508, 63284, 22672, 15856, 7172,  19796, 37996, 5748,  12404,
                17536, 62788, 6992,  26972, 52684, 29852, 11200, 1912,  20556,
                61944, 34636, 40648, 1320,  38272, 52192, 33684, 47440, 40096,
                60416, 11784, 16644, 56984, 1128,  50292, 15704, 53312, 1772,
                43824, 14004, 38164, 8704,  50140, 33244, 58888, 3220,  46560,
                34172, 100,   42316, 11348, 63976, 1960,  26464, 62448, 46800,
                39104, 1732,  64540, 25952, 37232, 46052, 8252,  4560,  47996,
                58716, 54084, 44884, 19752, 11260, 1608,  64360, 6672,  57304,
                32124, 23452, 50460, 38584, 2784,  30960, 48064, 27232, 43532,
                63968, 33756, 57668, 23460, 1188,  55828, 42284, 29056, 49308,
                38412, 59108, 3836,  11916, 35200, 15172, 41904, 38428, 60720,
                23612, 46964, 39116, 25424, 56052, 15944, 42808, 4000,  29532,
                11076, 6576,
            },
            {
                51464, 15792, 43224, 59636, 24968, 37896, 63612, 22496, 48284,
                36192, 7736,  15728, 33124, 20836, 26920, 16960, 2116,  19460,
                45040, 5048,  47764, 40552, 52956, 29192, 63900, 46796, 25596,
                39084, 30780, 22556, 13956, 59604, 8244,  25428, 41680, 52048,
                31036, 12096, 18408, 60456, 29324, 56452, 8976,  24252, 616,
                54736, 43164, 47928, 62136, 38184, 19596, 64168, 41436, 21272,
                6120,  24416, 46540, 18596, 41604, 28168, 23304, 52760, 61812,
                26244, 49604, 58616, 28996, 47424, 19256, 14096, 33276, 9460,
                59788, 20132, 34900, 13180, 56716, 28864, 52708, 42580, 24836,
                9948,  35344, 26468, 62748, 48092, 29712, 24560, 37204, 42204,
                268,   63320, 34492, 43268, 22256, 62800, 7460,  40324, 54556,
                17688, 48300, 4028,  36696, 45712, 32020, 10884, 62116, 3040,
                13292, 45216, 23156, 51080, 46360, 5544,  63072, 21484, 10876,
                33908, 120,   64312, 10316, 48936, 31584, 38216, 54912, 24568,
                41368, 61432,
            },
            {
                27216, 35572, 56820, 8488,  46692, 53272, 33384, 57924, 27132,
                19960, 51552, 42888, 62164, 6100,  37588, 54336, 48388, 28992,
                38688, 25296, 55260, 32216, 10912, 43932, 14360, 33696, 60804,
                53484, 9892,  57228, 47180, 40028, 34556, 63324, 45356, 16188,
                58364, 7488,  53840, 21956, 5088,  43876, 17420, 63216, 30828,
                7552,  35328, 25816, 9728,  30600, 45204, 11932, 29472, 36340,
                60160, 54056, 30680, 63756, 4876,  12916, 43332, 6956,  10640,
                40616, 18788, 15504, 23796, 37468, 55580, 6628,  52492, 24640,
                16792, 44092, 50908, 6480,  40628, 17828, 32540, 14600, 64648,
                30184, 2244,  15184, 38420, 4608,  54636, 16820, 50812, 14272,
                53928, 19440, 4528,  13100, 57052, 10368, 35496, 15324, 2416,
                24756, 29652, 12176, 20240, 63032, 27040, 15472, 35712, 20976,
                52924, 32420, 26940, 39792, 58444, 28664, 17160, 52140, 30020,
                44232, 18700, 52500, 29108, 4880,  62496, 17956, 8480,  33480,
                57892, 13468,
            },
            {
                47756, 4992,  20912, 28948, 496,   14812, 43432, 9960,  5312,
                61172, 30200, 11888, 55544, 45504, 10012, 41932, 22416, 58196,
                9104,  60132, 17860, 864,   26540, 57748, 3812,  49900, 19340,
                1140,  44084, 36500, 5020,  17900, 21060, 54724, 6392,  37208,
                27588, 47128, 42908, 35892, 49568, 13316, 59660, 36852, 50720,
                14368, 22128, 52228, 3248,  55640, 23692, 4364,  56872, 48656,
                16816, 11032, 2324,  57416, 36244, 51548, 17116, 56824, 30288,
                36332, 5064,  60904, 8524,  44372, 61720, 30296, 42280, 36840,
                55248, 4260,  30880, 58348, 23248, 62156, 664,   55304, 20784,
                39544, 57672, 52356, 22880, 42848, 60108, 34736, 7980,  45760,
                26076, 39232, 48264, 29580, 52032, 26620, 46000, 59116, 50264,
                38096, 61340, 55372, 40004, 51340, 7908,  47664, 57468, 43716,
                17464, 63480, 9232,  624,   20060, 54656, 3052,  47292, 6668,
                61932, 14904, 41856, 35348, 22436, 13068, 51024, 46232, 824,
                19528, 39220,
            },
            {
                9928,  64680, 52436, 41460, 62368, 19548, 30980, 55660, 45976,
                38516, 1384,  34920, 23228, 28704, 60348, 14376, 31816, 5556,
                35432, 12816, 42028, 65200, 48648, 37032, 22264, 8656,  41340,
                28272, 15980, 32376, 65140, 51244, 788,   30520, 13692, 23172,
                2252,  61408, 10036, 25276, 31916, 1864,  26744, 46012, 19804,
                58172, 39632, 64620, 17584, 41152, 34256, 61304, 8036,  26068,
                44520, 32576, 40136, 21436, 25632, 47936, 32744, 64904, 20936,
                45336, 54952, 51324, 33044, 2600,  21428, 10872, 49456, 484,
                28120, 63560, 15136, 48160, 10380, 45520, 37424, 11800, 49592,
                5488,  46580, 18200, 8376,  31216, 11836, 20648, 28444, 58640,
                61744, 9288,  16952, 64196, 37724, 972,   19100, 32092, 21820,
                5756,  9448,  44076, 16572, 4480,  24880, 38804, 1468,  29912,
                6312,  41344, 59948, 36380, 13756, 43340, 34344, 23988, 37976,
                56440, 25868, 2792,  60452, 54492, 43612, 27108, 36176, 63636,
                53640, 30956,
            },
            {
                45000, 23708, 33776, 11676, 48468, 36384, 4072,  23976, 12988,
                18248, 59256, 48580, 4864,  18608, 49828, 188,   52736, 63496,
                44636, 50972, 20704, 34288, 6884,  16588, 62460, 31004, 55412,
                59928, 24204, 49072, 11620, 27404, 42728, 58720, 46468, 50580,
                32748, 56068, 20056, 15576, 65528, 55416, 40988, 10896, 3664,
                28836, 5980,  32528, 12600, 46776, 51680, 15040, 38884, 19444,
                65532, 6656,  52872, 15444, 60612, 8216,  792,   38704, 13948,
                3260,  27800, 12344, 25200, 39280, 47752, 17516, 59328, 22644,
                12280, 39464, 33824, 20344, 27276, 4464,  58964, 24404, 43148,
                33524, 27660, 59600, 36256, 50120, 65520, 2164,  41112, 5044,
                31492, 21580, 44632, 6280,  23644, 55184, 42872, 12056, 64560,
                52808, 34104, 27812, 60012, 32392, 53468, 65308, 22248, 49652,
                11884, 55292, 22736, 48268, 30568, 64664, 9704,  59176, 12796,
                32272, 48368, 9376,  39388, 19764, 6712,  58956, 11632, 23260,
                16100, 3464,
            },
            {
                37124, 56312, 17380, 2884,  25948, 60112, 51232, 40200, 63824,
                52876, 26368, 15488, 41148, 64328, 35836, 24916, 39420, 16680,
                27212, 2696,  30576, 53460, 24492, 45800, 39340, 51632, 12760,
                6116,  45708, 2952,  60688, 22724, 35752, 10228, 64032, 17760,
                38408, 5720,  41712, 51440, 37444, 8152,  22792, 34292, 54572,
                42640, 48428, 59436, 26588, 1988,  21332, 29036, 54740, 192,
                35452, 49808, 27648, 42780, 11712, 45612, 28940, 53904, 23528,
                48688, 62132, 42144, 57552, 15060, 65524, 35436, 6364,  56364,
                45384, 51492, 7900,  54400, 41624, 35000, 52528, 29728, 15516,
                9152,  63336, 2840,  13832, 44448, 25608, 56708, 48592, 35880,
                12672, 51416, 57396, 32680, 14388, 59960, 8396,  48112, 25876,
                41100, 14624, 224,   46400, 19664, 13124, 8764,  34840, 60588,
                26056, 33800, 16068, 51984, 7012,  25804, 40412, 50144, 20664,
                4208,  53636, 28720, 15592, 33212, 47488, 2612,  30052, 42300,
                49192, 61064,
            },
            {
                6440,  13740, 29824, 39732, 54224, 9040,  16160, 28780, 7120,
                32804, 44712, 8940,  54412, 30444, 11572, 57524, 7848,  21632,
                47584, 9756,  57964, 14608, 60740, 11052, 1448,  26048, 34400,
                21044, 36872, 53960, 39660, 14964, 56220, 3772,  40572, 25784,
                12812, 62508, 28328, 3232,  47252, 58872, 18384, 50304, 63384,
                16532, 23344, 8856,  36372, 61880, 42992, 10236, 47616, 59992,
                13588, 22260, 58696, 3336,  34648, 62492, 17820, 41312, 58264,
                9400,  34548, 19640, 6808,  29844, 53172, 26240, 40844, 32360,
                18292, 24964, 59716, 2552,  65064, 17600, 7332,  61608, 50636,
                19932, 40652, 23992, 55420, 6020,  39064, 15280, 18580, 53772,
                24348, 42052, 3912,  36568, 49928, 39980, 30580, 3440,  35996,
                18588, 57856, 23088, 50164, 36404, 55908, 42404, 18180, 45640,
                4416,  39212, 2400,  44956, 19080, 55696, 1204,  16740, 45344,
                63396, 23104, 42764, 57336, 64852, 24856, 37992, 55668, 8312,
                20484, 26460,
            },
            {
                59560, 50444, 43668, 58148, 21788, 34576, 46864, 42836, 19880,
                1992,  36524, 58516, 21444, 3496,  47168, 43328, 32464, 55048,
                61816, 41356, 37948, 4284,  28372, 55956, 18980, 42296, 64248,
                57492, 9368,  18424, 31324, 7236,  48132, 20204, 29984, 8344,
                53576, 45568, 34728, 23888, 14136, 29236, 5340,  31312, 12348,
                444,   38312, 52644, 18636, 56732, 6192,  33672, 23616, 41840,
                30352, 8712,  38472, 55284, 20080, 24920, 52188, 5996,  31212,
                16276, 38084, 1028,  50952, 44276, 9776,  2300,  13856, 62912,
                3668,  37936, 11480, 44120, 31404, 13960, 22472, 46204, 1516,
                57180, 34664, 52840, 30356, 21400, 47336, 33260, 9524,  63788,
                1696,  28944, 60608, 10972, 26800, 21084, 16320, 61848, 54796,
                5072,  43176, 63516, 10696, 29592, 2356,  27008, 57944, 14424,
                53192, 64012, 28636, 58332, 11300, 37172, 61160, 27340, 34132,
                6852,  36860, 11856, 672,   50824, 17852, 13332, 62408, 51944,
                34720, 40604,
            },
            {
                31656, 10476, 19044, 5256,  64216, 932,   12448, 60744, 56476,
                23404, 50332, 14044, 39516, 27468, 62848, 19404, 2224,  13600,
                34856, 23096, 17448, 48868, 43616, 35596, 50612, 5408,  15564,
                29176, 43888, 51516, 61480, 26536, 34472, 51988, 43492, 57880,
                32,    19692, 60072, 10296, 52860, 44416, 61552, 39916, 46808,
                57352, 27940, 44684, 14392, 30776, 49572, 63184, 17208, 4412,
                53480, 46336, 16192, 31900, 48384, 4544,  37216, 12412, 64232,
                47224, 55816, 24120, 63192, 33492, 22100, 60100, 46772, 53776,
                30344, 50204, 21252, 26568, 48244, 56544, 39556, 36544, 25984,
                12356, 44144, 16584, 10212, 62816, 412,   27856, 57864, 38448,
                45460, 16020, 19696, 46908, 65004, 1292,  52004, 44288, 28524,
                13456, 31860, 38200, 7272,  60860, 39864, 48504, 6452,  31452,
                20732, 9028,  42676, 32852, 22076, 47844, 30888, 14240, 52372,
                57940, 18460, 31184, 45044, 8680,  40436, 32400, 4124,  45956,
                14808, 216,
            },
            {
                53408, 46544, 28048, 38168, 49296, 32088, 25624, 37816, 48208,
                11100, 65400, 31612, 6660,  53472, 15752, 36964, 50412, 29212,
                5760,  52244, 64516, 31228, 8092,  21264, 61568, 31976, 47876,
                2096,  25208, 13420, 41416, 1380,  63292, 16308, 23200, 36776,
                49288, 31608, 16144, 42124, 56540, 1524,  35968, 19920, 25204,
                7256,  32920, 65116, 3304,  40032, 25692, 12904, 56180, 35984,
                26440, 60500, 1136,  63980, 10628, 41956, 59508, 27200, 21840,
                3876,  43100, 28152, 11820, 48492, 38644, 18920, 27884, 42820,
                9044,  16140, 62300, 35692, 584,   9864,  60836, 5120,  31920,
                64532, 7108,  48516, 36708, 59032, 42484, 50708, 6836,  21164,
                34220, 52324, 55864, 8072,  41080, 34716, 57584, 23864, 9508,
                50688, 46896, 25532, 20888, 52676, 16340, 22944, 63036, 37632,
                50892, 24728, 13568, 62092, 5600,  53696, 3736,  43652, 9764,
                39192, 49536, 61688, 27268, 55092, 22280, 60164, 28908, 19344,
                25168, 65284,
            },
            {
                22308, 7260,  61984, 24004, 14940, 53536, 18212, 6756,  29616,
                4116,  42020, 25372, 60268, 45064, 10152, 23928, 61260, 58340,
                44680, 11504, 1072,  25560, 59100, 13732, 39508, 10196, 54256,
                37596, 58580, 5108,  22168, 45440, 11140, 58952, 4552,  13816,
                64908, 26036, 6304,  38352, 21628, 26888, 9308,  15356, 59244,
                48792, 21664, 11068, 55328, 19716, 45752, 1592,  38936, 51160,
                11448, 19416, 44216, 23664, 29232, 50880, 15744, 45764, 32972,
                51760, 18132, 7380,  59332, 15620, 5184,  56704, 36168, 23520,
                5764,  58180, 40496, 54904, 19188, 29204, 49468, 43072, 18012,
                51996, 28744, 3108,  24740, 18940, 32012, 11612, 25896, 61036,
                13132, 30056, 4356,  25036, 31668, 12824, 6824,  37864, 20228,
                60544, 1460,  56720, 12576, 45036, 4004,  34396, 11184, 44612,
                712,   56528, 48640, 38628, 17112, 41188, 24456, 65324, 21152,
                1996,  25064, 13748, 3808,  35932, 48216, 6804,  43180, 9952,
                56064, 36572,
            },
            {
                16404, 41768, 11660, 56800, 4252,  41032, 45244, 63052, 54812,
                15988, 35456, 18560, 2960,  48904, 33192, 41268, 7796,  26344,
                39760, 20796, 33744, 46104, 53028, 2920,  27708, 23524, 44704,
                16760, 62756, 33812, 50524, 38008, 28092, 54212, 32884, 40160,
                9560,  55404, 47632, 62284, 33144, 51084, 64272, 41004, 53812,
                4432,  35296, 42660, 51424, 8028,  59640, 29560, 21576, 64608,
                5264,  33312, 39784, 57548, 7496,  35064, 1856,  56592, 9024,
                61108, 40328, 35972, 30764, 52392, 41716, 128,   11308, 64840,
                47792, 33560, 4048,  45368, 12564, 53072, 24684, 14976, 57912,
                22032, 40384, 62084, 55788, 39164, 15140, 54148, 47004, 2408,
                43356, 39264, 62588, 50280, 43772, 18684, 48596, 63376, 32964,
                40728, 17504, 35548, 64824, 30536, 41492, 54396, 58844, 28404,
                19120, 34924, 3372,  26680, 60008, 8548,  35464, 56460, 29764,
                47356, 59800, 42048, 20120, 64276, 15904, 52316, 33532, 59184,
                49044, 2936,
            },
        },
        {
            {
                35256, 40800, 58396, 29468, 37804, 8040,  27564, 60964, 11036,
                20976, 52400, 13700, 8264,  24580, 240,   47160, 15912, 25476,
                56456, 13476, 53540, 668,   42708, 14688, 50944, 25068, 59304,
                9560,  56560, 20620, 65244, 46604, 2196,  17976, 39384, 45252,
                53012, 144,   43688, 21252, 33140, 24528, 40956, 35264, 9820,
                22712, 52436, 7108,  19676, 55592, 2684,  13692, 21952, 62360,
                50760, 11360, 44824, 40136, 33388, 51444, 58632, 22072, 9340,
                2300,  16836, 46292, 21800, 11652, 43792, 23360, 54932, 60236,
                896,   27116, 14956, 58864, 52948, 38064, 33284, 60608, 42304,
                31336, 63696, 3504,  33992, 58320, 7492,  18212, 59912, 1940,
                50256, 21072, 14428, 46948, 10244, 17964, 34836, 5992,  28188,
                56904, 9076,  48924, 29188, 428,   22224, 45748, 40176, 12864,
                49580, 61684, 7228,  32320, 59720, 4376,  29860, 8624,  3072,
                47764, 58688, 1748,  36592, 41908, 13916, 34960, 63944, 42732,
                19184, 62084,
            },
            {
                49804, 3060,  11720, 21144, 62160, 32808, 43656, 1832,  40704,
                34500, 30516, 38804, 44540, 18396, 34200, 12140, 59200, 36636,
                49396, 6568,  32436, 61232, 37508, 7860,  62248, 32332, 16896,
                49640, 36056, 12672, 32864, 24992, 51984, 60476, 30704, 13272,
                63400, 35000, 16384, 47580, 7100,  49840, 14212, 54084, 63408,
                47432, 58784, 41204, 10900, 30400, 59984, 26224, 33072, 41744,
                17220, 29756, 3924,  63640, 7404,  26252, 11876, 35468, 55180,
                28956, 52464, 61020, 36668, 55776, 15524, 31556, 39004, 18156,
                34712, 47712, 22732, 8056,  44756, 16560, 21108, 6932,  48404,
                23392, 16728, 45240, 13644, 30356, 42152, 53052, 46440, 30844,
                61772, 41076, 52908, 32752, 39304, 63084, 45212, 14084, 50332,
                16644, 3464,  38028, 62120, 14436, 51664, 26088, 33648, 53884,
                1740,  24124, 43492, 13796, 35688, 46976, 57092, 40556, 54396,
                12264, 32532, 51572, 10076, 26156, 59652, 20956, 732,   11972,
                28468, 55340,
            },
            {
                25580, 32260, 45244, 17348, 53276, 5236,  49384, 54116, 15352,
                64976, 5336,  58772, 27864, 54708, 40848, 51612, 31112, 3204,
                19424, 23692, 45580, 18292, 26472, 47320, 20344, 1860,  41276,
                5496,  45712, 53740, 4152,  42896, 10232, 36916, 49280, 8036,
                19840, 25216, 38424, 57424, 10760, 62556, 3024,  44848, 28268,
                38372, 2156,  34620, 46012, 15920, 49992, 38964, 9608,  744,
                58080, 36044, 14512, 59468, 31424, 43644, 16332, 4444,  59928,
                44348, 13136, 26388, 8332,  33452, 65312, 5932,  45432, 10576,
                63820, 4456,  56072, 35976, 25056, 49972, 65100, 13288, 54964,
                38604, 9428,  52036, 24668, 61616, 10696, 3744,  36520, 11916,
                25184, 5492,  28308, 3596,  58060, 24392, 1964,  55444, 42064,
                64632, 33936, 24884, 54572, 41348, 9736,  61276, 6836,  58428,
                16580, 39140, 28464, 62940, 20152, 10460, 16148, 26768, 61804,
                18416, 43040, 22028, 65384, 16744, 45584, 30352, 57376, 39160,
                5292,  16200,
            },
            {
                8176,  65148, 56936, 39100, 9996,  24320, 35888, 19512, 26104,
                9120,  47576, 22948, 2028,  63808, 6712,  21784, 9664,  43876,
                63436, 40124, 52168, 10768, 30228, 56784, 34764, 54400, 64268,
                22688, 26756, 61452, 18832, 57576, 14960, 22840, 3452,  55824,
                42300, 61032, 5840,  52456, 30940, 36816, 25948, 16528, 6192,
                20500, 13248, 25708, 64856, 22592, 5256,  53832, 63172, 45164,
                20808, 53096, 48820, 23960, 19884, 53920, 48304, 39788, 23456,
                32440, 18196, 3276,  42520, 48940, 2000,  25420, 57444, 21660,
                50300, 32024, 41240, 12232, 59804, 29488, 3912,  34720, 26408,
                12,    62684, 35444, 4864,  47704, 38256, 22416, 16452, 58552,
                44064, 19440, 65172, 48328, 12456, 30428, 20404, 36680, 26552,
                10544, 21004, 46624, 5420,  19344, 36156, 48576, 21220, 30612,
                46588, 9588,  56400, 48964, 2772,  38216, 50272, 23492, 35040,
                5228,  29020, 7420,  56264, 38124, 6168,  50724, 24280, 47912,
                60112, 43556,
            },
            {
                36848, 22924, 4672,  27220, 47740, 63604, 12820, 60000, 31788,
                56540, 42580, 17340, 36308, 14288, 47996, 61344, 28440, 57744,
                33728, 14480, 5136,  62636, 2552,  43316, 13012, 6900,  38200,
                14188, 31356, 8780,  39592, 34524, 63984, 27112, 46788, 32584,
                15320, 29068, 1992,  23088, 43236, 19356, 59576, 55468, 32996,
                61576, 51544, 56460, 7612,  42524, 36596, 18980, 28640, 12580,
                25504, 40792, 5440,  10356, 37460, 1120,  28104, 64980, 6888,
                49740, 38224, 62692, 53792, 19696, 39984, 29356, 52628, 7596,
                14024, 28216, 19052, 2308,  43152, 9512,  39628, 45600, 19456,
                59068, 28512, 43012, 18812, 55312, 27272, 64144, 49944, 32924,
                54960, 9252,  15584, 35744, 43072, 7744,  61744, 47772, 4952,
                51708, 30256, 57628, 13028, 63872, 27908, 2200,  42648, 12368,
                64120, 34468, 5644,  22524, 53676, 30656, 64760, 1032,  45456,
                53000, 39740, 48224, 12808, 33592, 54148, 3352,  14820, 34232,
                20124, 53144,
            },
            {
                13108, 50180, 42376, 14884, 30780, 3692,  41248, 45664, 2316,
                37992, 49952, 10640, 53220, 32820, 25288, 38568, 16824, 964,
                53452, 25856, 38000, 49136, 17116, 59472, 24492, 28892, 47980,
                52956, 43428, 2740,  50656, 29232, 6136,  53092, 40016, 58860,
                10964, 50844, 45716, 65452, 12948, 8004,  40168, 47808, 11096,
                43664, 29184, 17436, 48296, 31284, 1768,  59580, 47500, 34812,
                3428,  64520, 32224, 44452, 61148, 56748, 13940, 34144, 20896,
                10672, 56468, 29804, 23736, 9392,  59284, 16640, 35456, 42256,
                61604, 37676, 46524, 53736, 62952, 17684, 56868, 51576, 7516,
                47900, 15168, 57404, 32424, 8260,  14072, 41728, 636,   7248,
                28900, 38688, 50840, 21752, 56536, 52512, 16800, 40372, 13616,
                60804, 1204,  39548, 44568, 32204, 16852, 56236, 37440, 24832,
                51340, 18700, 27056, 41272, 12628, 43356, 7952,  55868, 20716,
                14720, 60228, 24612, 62576, 19464, 27928, 41760, 64188, 10360,
                31096, 1560,
            },
            {
                28812, 18600, 61852, 34692, 52284, 58436, 17532, 23480, 7444,
                28060, 20800, 61092, 29588, 44996, 4128,  55888, 12256, 46328,
                21040, 8524,  56260, 31772, 22088, 35904, 51380, 9780,  19448,
                60192, 16716, 56636, 21396, 46072, 12772, 17896, 700,   21112,
                35740, 26108, 39192, 16812, 33836, 53504, 29608, 904,   23292,
                36692, 3548,  39792, 10072, 62032, 24640, 14736, 8648,  57348,
                51116, 16444, 55320, 27072, 18548, 7980,  41804, 51800, 45616,
                60488, 4692,  15152, 36048, 44940, 12612, 63000, 48124, 408,
                24572, 57168, 5632,  22468, 27080, 32300, 13972, 21984, 30192,
                40420, 11244, 23036, 2344,  51132, 59716, 19660, 34924, 23920,
                46992, 57952, 3996,  25740, 33476, 4,     27800, 32240, 54928,
                18608, 35500, 22624, 7796,  53312, 49868, 4768,  60080, 8064,
                44436, 0,     61264, 36396, 57756, 17796, 33548, 28508, 10776,
                36896, 4432,  31408, 280,   43976, 8848,  22736, 56940, 38640,
                46912, 60912,
            },
            {
                40696, 9552,  56132, 324,   20712, 9080,  36924, 55784, 51608,
                63328, 13576, 3116,  56912, 8432,  19864, 50976, 41224, 30916,
                36088, 64304, 42108, 11772, 46692, 3480,  40232, 63348, 33012,
                188,   36076, 10944, 25540, 59852, 37424, 44152, 63204, 54288,
                7156,  60152, 3960,  55944, 48788, 21152, 5888,  64076, 50548,
                14796, 59840, 21496, 52704, 34184, 55792, 43252, 38352, 30900,
                22600, 7032,  39348, 12924, 47092, 30316, 24500, 2464,  16504,
                26968, 41296, 47692, 1484,  51756, 26584, 4912,  20900, 33152,
                10828, 51192, 15652, 34536, 48620, 1248,  37576, 64104, 3084,
                60268, 53564, 36400, 46716, 39292, 29424, 44828, 54320, 62796,
                17844, 13676, 60632, 40708, 11304, 63748, 46132, 9032,  24780,
                43508, 48672, 63196, 11328, 25472, 35200, 20360, 13816, 39704,
                54000, 32408, 14660, 46016, 3804,  25188, 47480, 61364, 40892,
                58464, 46252, 17512, 55264, 51772, 36548, 49144, 17916, 6716,
                25800, 55044,
            },
            {
                4176,  32108, 46032, 25348, 43940, 28908, 48572, 32860, 15744,
                40480, 35116, 24384, 42276, 37456, 65180, 23676, 59172, 6400,
                50004, 17984, 1968,  28400, 61436, 15532, 55828, 5744,  45836,
                27704, 41800, 64832, 48692, 3860,  31220, 8888,  24660, 29808,
                48396, 18756, 31340, 9896,  25036, 59016, 43988, 34944, 26876,
                56796, 6852,  45480, 27500, 12496, 19920, 50248, 2632,  18164,
                45688, 62280, 35712, 2152,  50328, 63908, 58480, 36336, 53576,
                32032, 19988, 64484, 33920, 57960, 38708, 30816, 55460, 44620,
                18412, 59832, 40068, 8320,  62088, 11572, 53064, 44624, 33896,
                25976, 5648,  17224, 64948, 25140, 4976,  9300,  12296, 37568,
                2668,  43676, 30872, 6988,  49888, 18960, 57248, 37940, 6092,
                59640, 15632, 29000, 3916,  58572, 46860, 30292, 65028, 22552,
                27952, 59028, 9424,  21420, 63932, 52352, 5896,  16052, 23900,
                50672, 9024,  34648, 26568, 12268, 61208, 29788, 2160,  44916,
                14148, 21564,
            },
            {
                48352, 63964, 12208, 37900, 59672, 5504,  65012, 11076, 1464,
                46940, 58732, 6544,  49244, 15864, 10996, 34112, 2496,  26924,
                14784, 60512, 23272, 52888, 7916,  44160, 26464, 21652, 12648,
                55096, 23172, 6572,  33752, 19000, 56180, 50792, 15728, 40688,
                12248, 43172, 63592, 36944, 1564,  40912, 16000, 11620, 18972,
                38952, 32264, 63032, 88,    41404, 5848,  65252, 28032, 59072,
                10008, 26124, 53280, 20224, 33188, 10960, 21968, 5092,  39224,
                8552,  55308, 11900, 6680,  22472, 16036, 8916,  41184, 64676,
                3032,  25664, 30444, 45952, 55604, 41528, 24620, 18764, 9220,
                49796, 42264, 31088, 12836, 57688, 21484, 61096, 32208, 52268,
                26660, 20608, 54092, 35932, 23988, 42396, 13168, 21548, 51200,
                1812,  33596, 55420, 42364, 38736, 16308, 9656,  43360, 3128,
                47388, 17264, 50320, 30068, 39432, 11752, 35804, 54556, 1328,
                29900, 21276, 64904, 39156, 6236,  15820, 58068, 32884, 63308,
                52216, 36476,
            },
            {
                16968, 7208,  23176, 54256, 14524, 18944, 42188, 26052, 22428,
                30660, 18308, 53460, 26648, 31520, 60136, 18784, 47920, 54680,
                44000, 30424, 39032, 48240, 34976, 18360, 32016, 38148, 59156,
                47860, 15340, 40064, 52836, 13236, 42452, 61664, 34388, 2212,
                57636, 22644, 46248, 14236, 51272, 28584, 62380, 46672, 54820,
                8832,  24216, 49140, 17240, 36940, 53640, 23536, 33504, 48716,
                40212, 14120, 43632, 57528, 6064,  41108, 45300, 13592, 62276,
                49320, 25100, 43752, 28920, 49996, 62476, 53328, 13452, 28024,
                49476, 37140, 12872, 4044,  20664, 28520, 5948,  58856, 15256,
                61816, 20960, 55032, 836,   34668, 49108, 16656, 41464, 47940,
                7872,  64804, 46216, 15124, 61492, 2340,  34420, 30236, 65364,
                40540, 26560, 12736, 21164, 62140, 712,   57340, 52552, 33340,
                6684,  38080, 56488, 2096,  43080, 26680, 19764, 62960, 44388,
                13500, 57356, 41852, 2364,  47192, 24740, 42612, 20076, 10472,
                27424, 59036,
            },
            {
                34228, 29404, 40508, 49260, 2412,  35380, 51872, 60596, 54944,
                44008, 9644,  63076, 92,    44896, 52332, 40384, 7648,  36560,
                10136, 57692, 13124, 4944,  56612, 65412, 10428, 52152, 4012,
                30440, 8664,  62268, 28860, 1292,  26256, 22012, 6300,  37604,
                52032, 27696, 4500,  33432, 19724, 56952, 4940,  31260, 2104,
                51940, 43412, 14092, 57296, 29720, 46484, 11012, 15936, 5196,
                55992, 564,   30012, 23888, 61888, 16096, 27572, 56416, 30852,
                17652, 84,    61228, 40236, 18596, 45460, 2056,  35848, 21696,
                6504,  57324, 52668, 61060, 16820, 35488, 50756, 39164, 29712,
                46256, 36760, 8020,  45168, 40112, 27992, 56492, 1976,  22784,
                29940, 11028, 38920, 5128,  28124, 58248, 46972, 54512, 10736,
                18340, 47664, 7548,  50424, 31732, 24400, 36480, 18692, 26020,
                63316, 13420, 24132, 34080, 60160, 48204, 8532,  32064, 38104,
                5580,  49668, 32508, 19160, 56280, 51400, 35036, 7920,  38924,
                43456, 1008,
            },
            {
                51312, 57536, 19856, 62568, 27872, 31612, 8172,  37556, 4024,
                13160, 39512, 34632, 22056, 13652, 4932,  28684, 24228, 61516,
                21172, 648,   33516, 25628, 20448, 40536, 1728,  60740, 17528,
                43476, 50184, 19972, 37788, 59668, 46320, 10968, 48660, 65064,
                16616, 9128,  61488, 54108, 39332, 10816, 23876, 37928, 20784,
                64796, 27916, 3312,  60032, 21348, 7048,  61544, 44276, 37520,
                63024, 17740, 35180, 9200,  48904, 37284, 51768, 3944,  22988,
                46800, 35948, 14172, 33244, 4796,  24268, 31416, 48160, 60588,
                16404, 43276, 23380, 32220, 45248, 65500, 476,   57104, 11948,
                3732,  27128, 63448, 18840, 52408, 4532,  13964, 62892, 35492,
                58344, 17336, 55488, 33108, 51564, 8516,  16328, 25384, 3104,
                44168, 58960, 37376, 54252, 5704,  45884, 11132, 48960, 41020,
                54984, 9072,  45148, 18500, 5108,  14812, 51680, 58064, 27652,
                17144, 24388, 10424, 63264, 28608, 13704, 3524,  61836, 54768,
                23564, 13552,
            },
            {
                45832, 9920,  4936,  44140, 11408, 57196, 15424, 48260, 19928,
                61644, 27436, 57136, 48872, 37596, 55480, 63960, 15192, 46436,
                41012, 50996, 62408, 43776, 14692, 50372, 45752, 27592, 36536,
                24560, 34196, 3188,  54372, 15980, 32960, 57108, 29660, 20184,
                42828, 31852, 24768, 48188, 7460,  44644, 60864, 48828, 33984,
                12152, 41168, 35724, 9420,  32652, 40376, 54844, 25176, 31120,
                21140, 52028, 45948, 59760, 3100,  19404, 32748, 65504, 42388,
                9660,  59612, 51556, 56908, 10784, 54064, 58556, 8144,  39956,
                33440, 11044, 1772,  38732, 9148,  13944, 22928, 47672, 33088,
                53788, 23748, 14488, 58408, 9940,  33252, 25380, 44588, 6128,
                50768, 41212, 676,   23340, 19248, 43832, 39812, 62036, 35556,
                22976, 29948, 15892, 64316, 19824, 28404, 60448, 14948, 3700,
                30544, 59448, 51004, 28788, 65292, 40596, 21816, 1804,  46124,
                62180, 54668, 43996, 6972,  37300, 21976, 46264, 30264, 49684,
                18472, 64736,
            },
            {
                31056, 36040, 24856, 53344, 38796, 21824, 45768, 25128, 32772,
                50860, 7616,  16700, 3020,  42928, 20080, 10588, 34288, 29920,
                6152,  17944, 27980, 54912, 7584,  31036, 22580, 12816, 5692,
                57844, 64552, 10372, 23416, 44144, 7800,  39000, 4372,  53716,
                12004, 59292, 244,   35964, 17712, 27372, 14808, 58256, 6256,
                18020, 55388, 23588, 47848, 51752, 17820, 1404,  13020, 49376,
                4144,  27700, 11716, 39944, 25236, 54340, 7696,  12608, 38808,
                28800, 3048,  26316, 21376, 38272, 42660, 15188, 26980, 19296,
                50812, 63068, 29152, 56192, 51864, 26704, 42656, 19504, 7608,
                60916, 39752, 43396, 30168, 21892, 47276, 54704, 38528, 20120,
                12576, 27308, 59712, 48116, 63500, 11832, 31508, 6220,  49168,
                53160, 4384,  10000, 34484, 43156, 1644,  39580, 53288, 23052,
                35496, 20504, 780,   36984, 11192, 54224, 25612, 35464, 12300,
                39408, 2856,  34000, 16024, 52676, 60520, 40632, 12024, 6416,
                26676, 2872,
            },
            {
                41536, 14440, 61548, 17392, 720,   64492, 6052,  59660, 2124,
                40552, 29880, 64792, 24636, 32348, 58824, 49680, 2188,  54036,
                59752, 11744, 38108, 3832,  35936, 58252, 63028, 39672, 52936,
                46592, 14528, 41496, 28120, 51344, 63560, 18376, 25524, 35044,
                46004, 39848, 21088, 56360, 63008, 41912, 1760,  52532, 30224,
                44020, 62024, 4556,  14976, 64264, 26832, 36368, 60320, 42008,
                7924,  64728, 55680, 31716, 14756, 43472, 60368, 21732, 48592,
                55036, 18992, 44508, 7268,  61432, 29688, 1104,  65016, 44680,
                4112,  24744, 46760, 18088, 6384,  60048, 30800, 36992, 50452,
                17352, 1836,  49204, 5892,  37608, 65204, 16236, 8680,  61016,
                31924, 45704, 34980, 3776,  26332, 36904, 56660, 20764, 14224,
                59744, 40268, 27036, 48172, 13144, 57732, 32744, 62112, 6160,
                47124, 42588, 15300, 56744, 44272, 30828, 7376,  60432, 50732,
                19880, 30164, 59680, 48896, 26236, 368,   17540, 57216, 34348,
                38244, 56068,
            },
            {
                7904,  58212, 46944, 33164, 26540, 42240, 35584, 55524, 18120,
                44600, 54516, 12420, 46988, 8660,  27320, 17160, 36424, 43084,
                21624, 48496, 64196, 24304, 47156, 10248, 17256, 464,   32272,
                20928, 6960,  55800, 31760, 936,   13148, 49176, 61084, 2664,
                15164, 51184, 5344,  29916, 12544, 33024, 22384, 37336, 10032,
                26092, 49768, 38156, 30700, 43224, 5924,  20140, 56900, 33788,
                15852, 44460, 22568, 2332,  35860, 47072, 27280, 884,   34408,
                15112, 64020, 32048, 50716, 17156, 47744, 34872, 12704, 52544,
                37468, 57860, 14616, 41068, 34704, 2628,  54688, 10620, 64340,
                27760, 34824, 12800, 59736, 51532, 3520,  28460, 42568, 1624,
                53480, 21780, 9480,  15064, 53976, 7508,  50428, 29016, 284,
                45228, 18824, 62524, 55204, 21632, 25160, 8616,  17568, 50136,
                11640, 27100, 60076, 23972, 4148,  18888, 47304, 15576, 42556,
                5020,  23316, 11524, 42056, 8436,  32456, 44448, 24164, 62548,
                47348, 20532,
            },
            {
                28784, 22888, 4004,  50388, 12276, 52476, 29256, 14328, 9680,
                23076, 34368, 4272,  52860, 41468, 61204, 5380,  57056, 25308,
                8788,  31856, 15608, 53088, 19852, 42044, 28888, 56100, 49568,
                25992, 37968, 18080, 60400, 36092, 21612, 40860, 55172, 29160,
                22968, 64236, 8944,  49460, 45364, 54840, 5032,  47296, 15700,
                59628, 20296, 616,   11588, 58560, 46172, 53424, 9700,  24036,
                38312, 50568, 18404, 58748, 9948,  63376, 17692, 51228, 58304,
                5788,  41128, 11636, 36804, 3552,  23484, 55260, 20452, 6724,
                31128, 9604,  22184, 49844, 62580, 20656, 44524, 15512, 4764,
                45676, 56288, 24980, 31800, 10496, 20736, 35736, 56344, 48584,
                16980, 64400, 38844, 43284, 60436, 17920, 41692, 64868, 24708,
                32720, 37528, 7388,  3508,  50904, 36608, 44980, 29484, 38456,
                64528, 32296, 7944,  49292, 39076, 63368, 33204, 56376, 27248,
                64776, 54152, 37924, 57672, 21000, 64320, 53264, 4308,  10040,
                15560, 51456,
            },
            {
                43812, 37216, 60244, 8880,  39624, 20160, 7000,  48640, 62460,
                38500, 59084, 20204, 15084, 37912, 22504, 45464, 13520, 51884,
                39520, 1088,  44556, 33828, 3044,  60888, 35276, 8340,  42940,
                11920, 63272, 45100, 5568,  47528, 26792, 10104, 6600,  44584,
                57604, 33924, 27028, 38452, 16704, 24960, 65308, 56856, 40488,
                6860,  52800, 34832, 22312, 39676, 28560, 14248, 31848, 1936,
                62256, 29004, 5064,  48220, 40320, 30500, 6752,  37724, 23200,
                29924, 46184, 25104, 57460, 62944, 9916,  40400, 59132, 26040,
                42884, 61188, 1572,  28664, 12160, 38496, 25212, 58588, 40764,
                22148, 52128, 18828, 39976, 43936, 61848, 24116, 12896, 30760,
                25860, 5096,  51240, 28580, 22792, 33552, 4512,  46704, 9500,
                57624, 12592, 42564, 30716, 16180, 63720, 2312,  56968, 20216,
                4732,  54808, 17044, 34864, 10800, 52304, 22364, 1128,  9728,
                35004, 47596, 15060, 3468,  29140, 36544, 13728, 39868, 31180,
                54872, 1584,
            },
            {
                65368, 13068, 17880, 27896, 55960, 63544, 43436, 32052, 25932,
                444,   47716, 27836, 63392, 33064, 1324,  30484, 65112, 19068,
                28276, 61440, 55288, 12652, 26804, 50608, 15184, 23544, 59204,
                2648,  53356, 29732, 15952, 52224, 58768, 32524, 37408, 17260,
                11796, 41456, 19624, 61752, 3080,  10568, 35768, 19220, 27740,
                32624, 44888, 63132, 16340, 50784, 3408,  63796, 47676, 42460,
                52924, 12528, 34528, 54896, 24212, 13536, 53560, 42596, 10352,
                62284, 1540,  52696, 14708, 28372, 44856, 32236, 49348, 15792,
                35920, 54300, 45628, 33712, 53208, 7964,  47984, 29124, 33304,
                9212,  16,    63744, 7300,  15156, 46656, 2432,  59012, 41268,
                36780, 11372, 57572, 1116,  47924, 12316, 38332, 15872, 51416,
                21360, 53972, 60932, 25988, 46872, 10844, 41360, 14304, 53048,
                24432, 40656, 45976, 61328, 2676,  28168, 13952, 45668, 40748,
                18184, 6200,  25520, 44992, 51392, 18504, 48560, 27008, 60772,
                19016, 33804,
            },
            {
                5272,  49360, 31428, 45376, 23892, 2864,  16376, 53472, 57656,
                11892, 41880, 5672,  50412, 10892, 54756, 48568, 7472,  35360,
                42560, 5100,  22808, 58680, 38864, 6156,  64896, 47324, 37332,
                19444, 34212, 9240,  24420, 2128,  42432, 19980, 62252, 48648,
                752,   51604, 7052,  53588, 43060, 31536, 50252, 1504,  13532,
                60664, 9648,  24360, 56052, 7512,  25652, 36084, 17596, 21388,
                6604,  26136, 60624, 19584, 1720,  45864, 57400, 26712, 16464,
                48980, 19728, 35084, 7828,  38684, 18520, 5544,  264,   64296,
                11424, 19100, 4816,  23772, 64040, 17620, 3840,  61964, 13756,
                57124, 37296, 47656, 29288, 55068, 33128, 52740, 18720, 6784,
                62216, 45936, 19904, 31624, 62780, 55988, 27044, 59960, 34652,
                29228, 1680,  17968, 35300, 52688, 22508, 59056, 27888, 34520,
                8,     13072, 29692, 19916, 42332, 58292, 37164, 62716, 30884,
                53108, 60212, 32788, 63104, 10712, 1756,  58876, 6008,  45760,
                23584, 41080,
            },
            {
                25996, 53936, 968,   38160, 58724, 34700, 5484,  22288, 36712,
                18868, 30060, 56428, 16780, 25060, 40448, 20788, 59208, 11436,
                16692, 49248, 30388, 9228,  45732, 21364, 31448, 4220,  27832,
                13632, 44132, 56736, 39336, 65440, 13296, 4052,  54240, 25344,
                31108, 60484, 23700, 14040, 28312, 58848, 45868, 22008, 54388,
                42408, 4224,  48440, 31652, 41364, 60420, 11084, 54608, 59008,
                39436, 45076, 9108,  37268, 65108, 32900, 21024, 3120,  60240,
                31712, 39780, 56104, 22152, 50392, 60976, 53944, 24464, 46568,
                29328, 39060, 56812, 14132, 41832, 50796, 35620, 54228, 19556,
                42816, 25528, 16928, 58508, 4520,  26984, 38380, 9632,  49648,
                23140, 34136, 14716, 42368, 8296,  18604, 3156,  40576, 6556,
                44384, 49000, 8816,  40260, 3956,  32444, 5872,  48536, 44036,
                60604, 50340, 55640, 5852,  24824, 49552, 17244, 3820,  23512,
                48800, 12624, 20560, 39556, 55572, 23128, 42252, 35616, 12500,
                8836,  58012,
            },
            {
                16020, 21272, 63064, 9844,  13848, 51420, 41148, 46980, 8908,
                61160, 44372, 34560, 8068,  60148, 36472, 2592,  44240, 26572,
                53316, 63792, 36972, 14452, 52232, 56820, 17628, 40116, 55040,
                50376, 61884, 21664, 31588, 49064, 35796, 28896, 45540, 15080,
                39748, 35088, 47012, 56112, 36988, 17304, 5556,  63724, 38448,
                29892, 18100, 37068, 12760, 19984, 45568, 29080, 300,   32612,
                14636, 51476, 29656, 16572, 41696, 7532,  50384, 35792, 44728,
                5384,  12560, 64912, 3280,  30796, 12188, 41848, 34684, 21384,
                51628, 8568,  60500, 26456, 31688, 1044,  22436, 45372, 5824,
                31388, 50080, 10344, 35160, 22896, 13436, 64716, 43092, 29044,
                55720, 2544,  53336, 24416, 36372, 49304, 52808, 23236, 62096,
                13584, 24944, 64292, 55972, 14904, 61792, 38376, 18144, 7748,
                21540, 37360, 9764,  63572, 31632, 11968, 54612, 7932,  57284,
                28500, 576,   43836, 8000,  34672, 29572, 16312, 53624, 63996,
                50888, 36824,
            },
            {
                46488, 33356, 42104, 48428, 19108, 30452, 25584, 65072, 15224,
                49896, 2824,  23112, 52348, 46068, 28936, 14916, 56780, 33308,
                3788,  41056, 19776, 332,   33604, 42796, 11516, 60532, 8920,
                24868, 672,   6664,  16988, 10560, 55440, 22392, 8400,  64012,
                4600,  11064, 20864, 2560,  8964,  41196, 25116, 11352, 51252,
                26268, 58496, 1296,  64612, 52864, 5244,  38792, 48360, 23976,
                64092, 4352,  56724, 49444, 25412, 11820, 62136, 15040, 28300,
                54072, 24168, 42528, 46500, 26352, 58612, 6696,  14940, 62520,
                2584,  43212, 36532, 47652, 10680, 58932, 39796, 27544, 12136,
                59772, 2788,  62972, 41156, 45528, 51536, 728,   21104, 16092,
                60276, 38996, 5860,  65524, 28912, 44820, 10444, 32172, 19252,
                54576, 37916, 31244, 20668, 43620, 27244, 10100, 52084, 65520,
                33008, 26800, 16140, 39724, 45340, 34440, 21488, 43528, 39104,
                35684, 15972, 58180, 26272, 50564, 62312, 4804,  32080, 20528,
                28376, 6652,
            },
            {
                11048, 61124, 4576,  27508, 59964, 7152,  56512, 1140,  33196,
                27296, 39928, 13432, 64380, 19304, 4916,  62792, 21948, 47164,
                8688,  24760, 59060, 28692, 63080, 23872, 3348,  47908, 34928,
                30020, 41740, 46808, 59780, 26640, 43496, 58356, 18704, 52640,
                29536, 50336, 43732, 63364, 32500, 60296, 47856, 34564, 15272,
                7328,  44492, 49864, 22696, 34108, 14896, 58140, 18752, 10200,
                35692, 43136, 21836, 1020,  58504, 30836, 53140, 38208, 8496,
                58988, 18260, 34036, 9352,  16612, 36968, 48700, 27860, 55764,
                31992, 17472, 5732,  20748, 55136, 16488, 8140,  64436, 47168,
                38144, 21232, 53188, 6700,  18280, 60728, 36104, 48128, 32400,
                11060, 26744, 46516, 13000, 58280, 704,   16100, 56960, 42236,
                3412,  47736, 11452, 580,   49752, 57980, 23556, 30476, 13992,
                3728,  47872, 59324, 1336,  52928, 4880,  61700, 25944, 10584,
                51900, 65264, 46684, 3404,  18908, 13440, 47964, 44232, 176,
                39524, 57088,
            },
            {
                30624, 14496, 22528, 36528, 54488, 43296, 12172, 38612, 20196,
                53376, 58372, 6072,  31844, 42640, 11844, 50872, 39040, 30104,
                13336, 55012, 48868, 44932, 7628,  37984, 27360, 53508, 14596,
                18668, 63476, 36444, 51704, 2456,  32244, 5124,  36332, 41556,
                23164, 59360, 16292, 26864, 51972, 19684, 152,   56676, 21180,
                61416, 31264, 10664, 27924, 43184, 61856, 26072, 51016, 6984,
                55564, 27812, 12984, 39276, 46616, 5796,  23304, 19656, 48100,
                40820, 504,   50100, 62904, 55200, 2224,  20164, 39680, 9100,
                45840, 24908, 65336, 50560, 29752, 43864, 34172, 24424, 55644,
                14600, 28768, 33484, 11748, 25292, 30620, 8580,  56592, 4000,
                41724, 51064, 17576, 21792, 34320, 39864, 25408, 63452, 35232,
                7716,  27712, 60260, 36428, 16844, 5572,  46240, 40104, 54416,
                43096, 56836, 23812, 19668, 28256, 14704, 47484, 58704, 18464,
                5916,  30076, 21860, 40276, 36540, 59452, 9980,  23728, 61632,
                17112, 49968,
            },
            {
                45052, 40692, 52564, 2448,  16832, 31892, 23520, 46272, 62828,
                10144, 17004, 48152, 36788, 26452, 54276, 33916, 1684,  60928,
                18552, 35676, 4960,  16908, 12636, 50552, 61648, 21832, 43908,
                56876, 8200,  12880, 20552, 40048, 15600, 49468, 61108, 13204,
                1224,  33636, 39056, 6068,  13556, 30260, 37432, 42984, 4628,
                53852, 40076, 16712, 56996, 8396,  3720,  40992, 30684, 37816,
                61428, 19736, 33148, 63396, 17368, 34984, 43880, 2760,  64212,
                25684, 13424, 29172, 21936, 43924, 31344, 61500, 52004, 13780,
                57852, 35140, 11644, 39264, 2992,  61408, 4784,  50964, 18148,
                828,   43352, 57272, 48532, 39148, 53868, 15276, 44736, 23936,
                63380, 36828, 54676, 8860,  61412, 51736, 5712,  29584, 45444,
                14656, 52340, 23896, 41512, 55424, 34308, 62988, 1480,  19208,
                11004, 34776, 7144,  41796, 64596, 37560, 31136, 3000,  41620,
                33812, 12464, 55160, 8656,  52160, 27192, 41628, 33768, 54208,
                26752, 3108,
            },
            {
                59420, 24792, 64420, 47460, 9380,  61996, 50780, 4256,  35572,
                29980, 43580, 23868, 320,   57832, 9364,  16536, 24284, 49724,
                43564, 65496, 31904, 39884, 57824, 19352, 31532, 1424,  6316,
                39540, 33092, 28576, 54796, 64600, 24736, 8032,  46336, 26340,
                55580, 9312,  48612, 57892, 44876, 55080, 10432, 64740, 24836,
                33780, 47220, 2116,  36380, 53416, 21216, 12408, 47260, 16196,
                2376,  44116, 49020, 8544,  26420, 59704, 54664, 10676, 32388,
                56520, 36260, 52792, 7088,  38624, 11212, 25544, 4580,  41736,
                22024, 760,   53800, 27124, 14460, 48380, 21768, 37052, 31512,
                9296,  62384, 22776, 5388,  64180, 1696,  27716, 59688, 19036,
                6420,  29444, 2764,  31480, 42736, 19900, 12344, 49616, 21960,
                58092, 18252, 64864, 28940, 8948,  12756, 21716, 26200, 37136,
                60872, 29280, 51588, 46652, 12352, 8228,  55780, 21064, 49416,
                62092, 45056, 24800, 63220, 31384, 2068,  56884, 14104, 7372,
                37312, 19144,
            },
            {
                32724, 5944,  12752, 34436, 20840, 39508, 28344, 15004, 55536,
                7804,  59924, 51848, 20684, 62028, 40968, 46220, 56408, 5576,
                10940, 25964, 2048,  22400, 54008, 9404,  36496, 46996, 58920,
                50172, 23028, 4612,  45788, 10880, 35068, 53500, 30592, 17624,
                37700, 65176, 20652, 24628, 3272,  17536, 49060, 28360, 8252,
                13868, 19384, 63384, 24184, 45072, 32000, 65348, 51924, 24644,
                54444, 11480, 57616, 3784,  51216, 13832, 40452, 28612, 16732,
                45700, 4644,  61104, 15368, 48156, 58300, 17608, 33444, 63516,
                29104, 47312, 59968, 19360, 32932, 42156, 11216, 58728, 52656,
                40368, 46432, 15940, 36392, 19792, 42920, 12176, 34280, 39828,
                49880, 13864, 57796, 24312, 47928, 4104,  56172, 37800, 9320,
                40096, 5248,  2272,  47268, 32528, 52580, 59152, 44304, 49720,
                15020, 4260,  17612, 32644, 60392, 25032, 52428, 16160, 27332,
                1364,  37848, 14968, 5600,  48216, 16948, 21304, 46372, 65068,
                51144, 10276,
            },
            {
                56144, 38484, 28944, 57452, 44040, 1800,  58776, 19284, 41232,
                25496, 3652,  38052, 14316, 30992, 6620,  27748, 36116, 21284,
                41284, 50972, 59880, 46088, 28076, 42664, 64504, 16484, 11868,
                26220, 62876, 17924, 41592, 59264, 528,   21572, 62472, 43088,
                4200,  12236, 29040, 41748, 35100, 62300, 22720, 39108, 52452,
                58036, 30016, 49564, 11412, 59608, 17780, 788,   35164, 6508,
                29376, 39844, 22492, 31060, 36580, 20044, 5376,  61628, 50476,
                21596, 9768,  41964, 24556, 34792, 1344,  44488, 49908, 6144,
                37316, 16416, 43748, 8352,  63036, 56056, 28976, 6292,  25148,
                60384, 3372,  27096, 50264, 31864, 58568, 52328, 47524, 9528,
                22252, 60684, 45636, 38904, 15324, 64580, 27064, 33420, 61152,
                30928, 51000, 43320, 20792, 15868, 39516, 3316,  30948, 8472,
                63860, 53752, 39900, 22632, 328,   35580, 44284, 40056, 9544,
                59800, 51496, 19524, 43516, 34860, 61048, 39116, 3708,  29348,
                42440, 22128,
            },
            {
                48012, 952,   17856, 49756, 25764, 6816,  52776, 11988, 47200,
                33568, 64808, 10456, 44688, 49344, 18860, 64080, 13660, 58812,
                29976, 15888, 34276, 7140,  14060, 3640,  23952, 29868, 34784,
                55152, 2548,  36892, 48776, 29984, 16576, 39268, 5784,  50704,
                32412, 53152, 47028, 59180, 7632,  31596, 12664, 1492,  44676,
                35388, 4016,  41576, 6904,  38128, 28096, 42096, 57804, 20336,
                46296, 60876, 15508, 42548, 64952, 47216, 24524, 33628, 1472,
                37880, 58936, 30616, 19048, 65056, 54076, 28324, 21124, 55924,
                10440, 52600, 3544,  24584, 35364, 1436,  17020, 45220, 13580,
                19176, 34936, 10836, 56332, 7760,  24288, 2900,  17328, 65076,
                27188, 33496, 288,   10504, 35728, 52368, 17456, 1304,  46880,
                13548, 26096, 36128, 61968, 54748, 7192,  47896, 18052, 23324,
                42508, 27584, 10596, 57564, 49208, 6336,  13684, 57040, 32896,
                23068, 29360, 56044, 11120, 27656, 53492, 24532, 12060, 35296,
                15484, 62232,
            },
            {
                8328,  53308, 41860, 14680, 60880, 32288, 36932, 62936, 22376,
                50464, 17120, 27940, 57228, 34820, 3224,  51540, 39080, 776,
                53408, 9236,  63616, 38188, 57048, 49184, 53016, 40896, 8428,
                44236, 14856, 52568, 7496,  25640, 57580, 45264, 14140, 24012,
                60440, 19060, 2612,  15396, 54600, 50508, 40960, 60300, 16936,
                21872, 62152, 26188, 15544, 50612, 55508, 9824,  14336, 62860,
                37348, 9152,  26920, 32,    52716, 7900,  57268, 45000, 13512,
                53776, 26564, 48928, 3604,  12320, 39972, 7384,  14420, 59272,
                26172, 31968, 39596, 49760, 21160, 53520, 37612, 65000, 49456,
                30956, 54504, 39464, 62836, 14484, 41568, 37428, 30396, 4656,
                53488, 42444, 19716, 56880, 28928, 7180,  44424, 23808, 54936,
                19516, 6788,  58740, 11320, 24100, 28240, 62820, 34224, 58340,
                1788,  36404, 20464, 45400, 64220, 28904, 19204, 62788, 6912,
                45856, 3948,  36764, 64288, 468,   8828,  49424, 58912, 55476,
                4652,  26960,
            },
            {
                19644, 35548, 64072, 10652, 23000, 45680, 9224,  29380, 4636,
                42976, 1164,  53632, 21344, 40676, 8776,  23856, 31568, 46452,
                18676, 43128, 24920, 20832, 31980, 17644, 62492, 316,   19872,
                60672, 32076, 21760, 61844, 12044, 33800, 54216, 9060,  28072,
                35544, 40132, 10516, 27060, 37464, 19636, 5416,  25712, 54892,
                7940,  46576, 53548, 32540, 2848,  22680, 47788, 25736, 32712,
                3632,  50040, 55948, 34628, 17400, 11704, 29316, 41440, 18372,
                63624, 6996,  43848, 56788, 32136, 23420, 47144, 35916, 41356,
                18936, 64432, 12572, 57500, 46048, 9536,  26904, 41312, 3992,
                21916, 44180, 600,   29276, 47140, 21328, 60352, 45204, 56220,
                7928,  16212, 51060, 62752, 22132, 40472, 59536, 10300, 63012,
                41588, 52644, 33588, 184,   45036, 37372, 13280, 51164, 10088,
                46552, 54920, 14396, 31364, 3016,  41936, 25816, 53400, 38132,
                17500, 13228, 50432, 42400, 20680, 33236, 41092, 17432, 44268,
                31808, 46020,
            },
            {
                39364, 24368, 30952, 4160,  54428, 40560, 18528, 57128, 14280,
                37728, 31640, 59548, 12976, 26616, 60412, 55296, 11560, 62824,
                27168, 3824,  55464, 47864, 6248,  10812, 36612, 27484, 57716,
                47444, 38644, 28156, 3112,  43768, 19332, 64348, 48364, 1368,
                56752, 45888, 62760, 23056, 44124, 64508, 29876, 48488, 33532,
                11632, 38524, 20000, 60044, 36004, 64240, 44032, 5332,  52940,
                18904, 13360, 44544, 59896, 21288, 38300, 62488, 4496,  23048,
                35248, 10724, 16784, 37196, 61656, 52064, 9840,  62708, 44,
                44912, 5596,  29148, 15624, 4728,  61708, 31764, 11976, 52324,
                8132,  61352, 24864, 18124, 5636,  53136, 11256, 25616, 35340,
                13092, 38100, 31696, 5488,  48196, 14008, 35124, 30128, 2660,
                37860, 15384, 29204, 49420, 18564, 56336, 4736,  40248, 26508,
                32392, 5548,  61724, 38692, 16620, 50848, 34636, 10788, 47756,
                60764, 30736, 58220, 23800, 15248, 60364, 28924, 2844,  22952,
                10832, 61240,
            },
            {
                13448, 6328,  51376, 57976, 34504, 128,   49088, 26296, 51960,
                61956, 8424,  45392, 5536,  48052, 17488, 43784, 37368, 5240,
                49924, 35372, 13156, 61456, 39892, 45060, 22692, 51868, 4988,
                13232, 9244,  50304, 56224, 40604, 6080,  30408, 37272, 21828,
                16888, 7168,  51964, 33616, 416,   9612,  56404, 14668, 43076,
                63776, 864,   29728, 12900, 8348,  16848, 29216, 38536, 57176,
                41680, 23652, 31232, 6344,  48468, 25812, 50660, 55336, 31168,
                46888, 52652, 27720, 21508, 1896,  15140, 29844, 55084, 33856,
                22768, 48492, 60388, 34592, 42544, 18724, 23432, 58600, 34132,
                16176, 55864, 36720, 49892, 64244, 34076, 2476,  18940, 48936,
                63512, 23532, 45880, 26444, 3088,  53968, 18928, 50456, 25756,
                46260, 22096, 65304, 8936,  42652, 60172, 22412, 16064, 64556,
                19616, 50296, 24488, 8208,  59916, 56116, 5080,  22728, 1552,
                40776, 26384, 7936,  34340, 47272, 5480,  51692, 63804, 37108,
                49104, 54644,
            },
            {
                59280, 16924, 43348, 27108, 20576, 15464, 65320, 6512,  34040,
                20244, 24256, 39544, 29516, 64540, 1776,  33364, 15200, 20964,
                57984, 41580, 30724, 1420,  28632, 58796, 14464, 33112, 41764,
                25400, 18084, 63144, 35716, 14640, 24872, 52080, 11156, 61404,
                42808, 31236, 13220, 58752, 47456, 17824, 36220, 3868,  22276,
                27316, 57240, 44724, 52112, 40440, 58448, 20988, 10716, 61520,
                1360,  27820, 63456, 39824, 2752,  33368, 9596,  14628, 1180,
                39444, 58152, 5048,  42448, 50156, 45536, 25440, 6464,  17072,
                51268, 10108, 38324, 25300, 55144, 50984, 1212,  39144, 46828,
                28552, 42232, 13184, 9328,  30784, 40224, 57200, 43616, 28808,
                1168,  58628, 41460, 11928, 61624, 32604, 43100, 8080,  58384,
                12448, 56024, 3636,  36436, 25888, 31440, 52244, 1356,  44072,
                57060, 11960, 41632, 28616, 20724, 12788, 44128, 31916, 63504,
                19620, 54472, 2640,  44500, 56516, 12360, 39232, 20364, 7808,
                26476, 1112,
            },
            {
                29440, 46956, 36628, 11824, 48032, 58956, 38220, 43588, 10984,
                47340, 56208, 14504, 36152, 51008, 22484, 54052, 28492, 61156,
                9936,  23712, 16532, 53724, 19140, 50472, 2944,  65256, 55612,
                45820, 30860, 1220,  22380, 47668, 59588, 3500,  39652, 26216,
                49464, 3844,  20584, 28544, 41376, 24956, 52548, 59600, 40152,
                49816, 7056,  17696, 24804, 48084, 1956,  33704, 51160, 15924,
                35772, 47452, 12068, 53324, 16396, 58268, 42132, 60884, 19500,
                24952, 64712, 13480, 34164, 60068, 19212, 38768, 58176, 42224,
                27640, 56768, 20480, 3208,  8700,  45360, 14332, 62728, 5688,
                20544, 2636,  59348, 45656, 26376, 15096, 22000, 61004, 9484,
                17148, 52020, 7556,  20644, 56992, 37624, 15992, 64060, 4572,
                40256, 32312, 17128, 53808, 11996, 6976,  47612, 34748, 28128,
                37112, 2608,  46820, 54132, 36216, 48416, 25252, 57232, 15732,
                49664, 11792, 37552, 64816, 18696, 27240, 31648, 53844, 15088,
                34044, 40872,
            },
            {
                64560, 22336, 2396,  61784, 8324,  28728, 22168, 30908, 2032,
                52828, 17768, 4120,  57700, 10464, 41176, 7624,  45988, 38588,
                52180, 6876,  44392, 63564, 37748, 24860, 9568,  35928, 20568,
                7280,  60312, 54248, 33448, 9816,  44208, 18740, 53756, 14004,
                34240, 65088, 55280, 38304, 4616,  61720, 12476, 30536, 19240,
                10412, 37084, 32396, 63004, 5680,  55176, 26928, 45860, 7072,
                22212, 55456, 8612,  20380, 45020, 28760, 22616, 36312, 44516,
                48860, 7852,  29208, 54948, 9372,  3584,  32360, 11768, 65272,
                4088,  13740, 32888, 40772, 64128, 29852, 35748, 26116, 56944,
                49120, 33224, 23260, 53876, 6704,  51104, 3900,  36256, 54456,
                39204, 30524, 34508, 49180, 27792, 544,   24008, 47588, 19604,
                27468, 50776, 45316, 60652, 39372, 20612, 63168, 14220, 8900,
                59592, 17664, 33160, 6088,  65416, 512,   39272, 8976,  35092,
                43180, 29512, 22280, 50764, 8892,  41900, 1596,  62244, 44088,
                57320, 19020,
            },
            {
                9452,  50016, 32132, 53764, 41808, 3688,  55184, 13792, 63236,
                40652, 27728, 61368, 32432, 19308, 26032, 62764, 13708, 56,
                31504, 48308, 34116, 11740, 5316,  42540, 47364, 16124, 28396,
                39456, 12452, 42136, 16568, 64512, 27856, 32232, 57448, 7700,
                45032, 23664, 10564, 16408, 48804, 7880,  34396, 45416, 1620,
                55684, 60528, 14744, 42600, 22828, 36808, 13924, 64544, 30448,
                42784, 60848, 32920, 37592, 65428, 7188,  52116, 3092,  11464,
                32476, 17672, 37792, 23524, 41088, 62496, 48996, 21972, 35660,
                46332, 53124, 60816, 16480, 48956, 21680, 10912, 52824, 17588,
                8852,  65432, 38556, 18008, 61796, 42080, 31724, 47732, 13732,
                22860, 4364,  64820, 14652, 44172, 10020, 55000, 30912, 35828,
                62320, 9760,  23072, 840,   28876, 56916, 42164, 24616, 53448,
                49216, 22708, 61832, 14888, 30520, 18792, 51136, 60428, 27408,
                6308,  59112, 3568,  32484, 15768, 59520, 47636, 24096, 11136,
                50920, 5312,
            },
            {
                44656, 15436, 39428, 24308, 18716, 33888, 45180, 50244, 25172,
                35720, 7840,  46276, 42684, 2372,  49732, 35320, 55268, 18508,
                25284, 59144, 20720, 54684, 27764, 60736, 32004, 56892, 52504,
                2360,  48812, 26348, 51224, 5180,  38680, 60,    21944, 60040,
                36648, 2428,  52764, 31876, 57300, 43672, 23040, 64052, 51580,
                26044, 47048, 3836,  29476, 11104, 58908, 19408, 39904, 2568,
                49776, 17300, 26260, 556,   49300, 13672, 56500, 27172, 61908,
                50744, 57120, 436,   47288, 14384, 27288, 54176, 16592, 7824,
                29740, 43144, 1900,  25744, 58952, 6228,  44604, 38072, 31068,
                41844, 12684, 28064, 20,    35128, 11300, 19752, 63644, 27268,
                45380, 55712, 40640, 19056, 60744, 51824, 38896, 13340, 57096,
                6540,  43712, 15052, 33692, 49940, 17852, 2896,  31884, 38628,
                4400,  43064, 26548, 40512, 10740, 44892, 23220, 4156,  13752,
                46536, 53468, 40532, 24948, 55196, 35408, 21068, 4172,  38196,
                31004, 25480,
            },
            {
                55912, 60116, 4772,  12852, 64164, 57632, 16364, 5352,  60216,
                20968, 12388, 54448, 23100, 15348, 58780, 29008, 43056, 4080,
                64908, 40208, 14928, 2180,  49672, 21920, 13260, 4392,  62664,
                23252, 36912, 8584,  20400, 56140, 13016, 46676, 50620, 29520,
                17980, 42176, 27000, 62352, 20316, 13884, 28136, 5320,  16664,
                38892, 21588, 35440, 54324, 49036, 44464, 8452,  52184, 24536,
                13172, 5132,  59140, 41416, 30924, 19128, 34940, 40156, 15656,
                5324,  41868, 63496, 20424, 52316, 31324, 2064,  39840, 61880,
                23792, 19196, 37240, 12052, 31948, 54100, 844,   23716, 60456,
                4168,  55380, 44972, 48372, 58008, 24600, 52784, 8048,  1508,
                59336, 11080, 25624, 6632,  33020, 3512,  21776, 42932, 2176,
                25108, 52516, 37448, 64684, 5844,  46364, 10916, 61080, 19396,
                12700, 56420, 7500,  52104, 58440, 33676, 54268, 63736, 37124,
                21280, 17732, 63072, 12444, 44360, 6792,  29416, 52832, 13948,
                61112, 35236,
            },
            {
                20328, 28116, 52256, 46560, 36264, 27344, 10196, 39488, 32580,
                500,   48908, 29908, 64708, 38852, 5396,  11624, 21964, 47508,
                35956, 10220, 29700, 57472, 36468, 8284,  40644, 44580, 18244,
                33808, 58676, 45484, 30200, 62176, 34760, 41612, 9464,  14684,
                63508, 47224, 6992,  39756, 680,   36204, 54308, 41048, 32148,
                12120, 8108,  64900, 18632, 524,   27816, 62592, 31880, 57496,
                38504, 53556, 46064, 23004, 9984,  62572, 4108,  46780, 21476,
                30472, 25912, 35176, 11324, 5828,  44380, 57968, 10632, 47792,
                55808, 5232,  52024, 47308, 39952, 18644, 57280, 13980, 49712,
                35344, 16492, 21796, 7580,  30084, 15364, 44296, 38276, 33644,
                17552, 50208, 36860, 46928, 29200, 57928, 48480, 16960, 61140,
                30048, 18852, 59024, 12180, 25660, 54596, 35872, 44536, 27164,
                64444, 34868, 47256, 20892, 2712,  28180, 16136, 41776, 31488,
                50232, 9412,  33968, 140,   60668, 49364, 40036, 65372, 18228,
                42608, 652,
            },
            {
                11532, 38360, 31456, 8692,  1308,  49356, 20068, 53192, 44368,
                57996, 17312, 36384, 9272,  51352, 45288, 33472, 60840, 16548,
                52088, 6084,  43300, 46228, 16624, 64192, 25820, 50152, 29644,
                6056,  14568, 54028, 1984,  17036, 22960, 60572, 26000, 4212,
                51980, 32684, 12512, 24700, 50932, 59532, 9856,  48288, 61168,
                56888, 43888, 50240, 33220, 40920, 16276, 5364,  35972, 10436,
                20436, 28592, 34532, 15468, 55544, 42860, 24748, 52612, 59644,
                9164,  54692, 45848, 60256, 24508, 33976, 18776, 36736, 26396,
                14732, 33528, 64232, 27600, 7772,  61972, 42168, 29504, 8316,
                63488, 26820, 51716, 62156, 40880, 4680,  60760, 56032, 28564,
                42992, 2948,  21196, 62164, 12812, 8756,  26460, 34716, 53996,
                40364, 8216,  47700, 31300, 39716, 21532, 58072, 7812,  51076,
                1216,  31028, 15804, 38180, 61176, 6692,  48512, 11404, 1724,
                25900, 57412, 38676, 27252, 19820, 15400, 9864,  26140, 55100,
                7960,  48632,
            },
            {
                58296, 16920, 63344, 21708, 42284, 61064, 30560, 6672,  23540,
                63464, 41552, 3772,  26292, 19808, 56800, 24928, 7488,  30792,
                55568, 26488, 62304, 23612, 32956, 53240, 404,   59432, 11308,
                39964, 64848, 25020, 37892, 48596, 5616,  53428, 31312, 19484,
                37572, 58308, 55156, 17152, 44428, 30212, 19564, 3988,  22704,
                27068, 2416,  14112, 24652, 59728, 55304, 22104, 47776, 43432,
                60340, 7004,  63952, 2260,  50592, 37028, 7864,  32284, 13212,
                38164, 18572, 2244,  15488, 40424, 64840, 7356,  51316, 60580,
                532,   44260, 10256, 22196, 15672, 34664, 24764, 46816, 19784,
                39588, 2220,  32732, 10856, 36656, 50616, 20672, 12540, 23400,
                64476, 52276, 15208, 54824, 42052, 37752, 65192, 4240,  11332,
                45552, 22092, 1568,  51916, 16204, 3304,  29320, 14272, 41324,
                23060, 54632, 9956,  25488, 43912, 35460, 19560, 55076, 60096,
                44924, 7320,  52220, 47132, 30644, 58076, 37036, 3340,  46164,
                33400, 23620,
            },
            {
                51068, 2928,  44204, 54924, 25192, 14760, 57208, 37544, 10804,
                28548, 14200, 55968, 31784, 61948, 1384,  40852, 49988, 13076,
                37972, 1108,  19420, 11836, 5116,  39036, 20320, 35376, 55016,
                21764, 42756, 9496,  32704, 43928, 11800, 57236, 39880, 44928,
                1316,  22344, 9180,  34916, 3172,  64664, 14576, 46180, 37532,
                53168, 31144, 62188, 7360,  38364, 11592, 51096, 26544, 3240,
                17752, 40868, 48540, 11936, 26688, 17428, 57556, 944,   64516,
                43328, 51284, 27960, 56544, 49100, 29772, 13460, 22668, 41420,
                31212, 57252, 38384, 54508, 49612, 2732,  5340,  52748, 11676,
                58764, 44456, 54780, 19136, 25700, 1236,  32280, 48768, 6132,
                9688,  30764, 34568, 5532,  23616, 18188, 51012, 31792, 15232,
                62756, 28212, 35220, 56424, 42628, 63876, 49488, 33956, 60464,
                17356, 39500, 49116, 56572, 13504, 64916, 24136, 39888, 29304,
                17092, 22620, 13956, 63928, 5884,  42060, 51404, 21480, 28980,
                62956, 40628,
            },
            {
                35924, 27020, 7024,  33200, 11292, 40304, 2264,  47136, 18916,
                50092, 5816,  44836, 37920, 11368, 17744, 35224, 63428, 21116,
                58576, 41928, 51492, 47180, 60968, 30252, 9104,  16008, 45728,
                28084, 4696,  52444, 19876, 63160, 28348, 15764, 7788,  65476,
                48972, 27828, 62048, 42632, 50020, 26276, 56176, 33912, 6480,
                17076, 42768, 20132, 46768, 29028, 1704,  33760, 64576, 13656,
                55688, 30580, 22228, 33216, 61496, 44684, 29372, 20616, 48184,
                23740, 5668,  36320, 8864,  21036, 2796,  54280, 46148, 4504,
                17084, 20628, 6372,  29244, 59100, 43840, 65060, 31460, 36140,
                22864, 6184,  14540, 42140, 64084, 57924, 16184, 44944, 36072,
                40904, 58400, 26016, 45872, 59956, 1868,  44432, 56580, 24244,
                49328, 5404,  60356, 10204, 19896, 6104,  24724, 10520, 45784,
                3724,  63352, 6500,  29724, 756,   51280, 8704,  33036, 3392,
                62172, 36324, 43372, 2556,  24040, 34252, 12092, 59564, 16632,
                5304,  14028,
            },
            {
                10228, 61584, 48064, 18636, 51368, 64584, 34888, 54348, 24704,
                33052, 59380, 52904, 22740, 47936, 54304, 5144,  45956, 28844,
                9388,  33784, 15008, 27476, 56484, 41188, 48944, 63568, 2600,
                57488, 49324, 13304, 59560, 584,   35788, 51056, 24252, 34220,
                11380, 15116, 30808, 5612,  38468, 21560, 11784, 40736, 63040,
                57748, 9744,  35524, 52616, 58452, 41940, 19092, 44748, 37080,
                52140, 8696,  58088, 39176, 3616,  53680, 10068, 40468, 34092,
                14900, 62104, 31832, 59352, 44660, 38336, 62452, 26828, 35596,
                50628, 63652, 46968, 13540, 36880, 18268, 26908, 56048, 16324,
                48836, 61528, 29196, 47056, 8256,  38716, 27848, 53464, 62920,
                18920, 1024,  49604, 11168, 29888, 39700, 20984, 7820,  36412,
                41924, 17992, 38880, 26740, 32856, 44696, 59312, 37584, 51652,
                26916, 19288, 33348, 42776, 21912, 45652, 57328, 14764, 47620,
                53224, 9876,  31908, 56448, 17892, 54204, 47440, 1100,  32180,
                44712, 56364,
            },
            {
                30136, 22892, 37420, 59320, 4060,  28108, 16984, 8244,  62660,
                42832, 3308,  16320, 7868,  27336, 42360, 14252, 24092, 2984,
                48616, 65492, 6884,  35996, 3528,  22204, 13120, 26004, 33404,
                37324, 17708, 30920, 25920, 39312, 46776, 4412,  18076, 41572,
                58584, 53616, 46936, 18240, 60416, 8488,  52012, 28224, 256,
                24120, 49692, 12904, 3864,  22756, 15628, 60676, 5500,  23444,
                27840, 28,    46700, 19680, 14124, 24436, 49508, 60732, 4640,
                53420, 11288, 41920, 25268, 16432, 11664, 32380, 7652,  58816,
                10884, 25492, 40572, 1352,  23576, 11160, 41028, 8672,  892,
                38972, 24972, 34356, 2988,  51932, 21956, 10956, 3612,  24964,
                13468, 55608, 37484, 16872, 53200, 63800, 13880, 28736, 59288,
                832,   51620, 13316, 46764, 53748, 14996, 132,   21092, 31468,
                12396, 58652, 36376, 61640, 16628, 27852, 37076, 5208,  41596,
                26620, 20548, 59088, 39220, 28260, 8804,  64412, 39912, 25608,
                53116, 19860,
            },
            {
                42228, 236,   15392, 45796, 31500, 21592, 44752, 13404, 39024,
                29640, 20428, 36096, 64784, 30804, 60288, 57264, 33044, 39496,
                16856, 53388, 24396, 45120, 18336, 58528, 52172, 43132, 7288,
                23288, 61268, 41624, 8336,  56556, 21528, 54484, 61444, 26988,
                2408,  21292, 35652, 56928, 1948,  32560, 44668, 15444, 47100,
                30564, 38872, 65404, 27448, 32784, 56292, 9556,  31008, 48208,
                62696, 16636, 34964, 43116, 65224, 7020,  36292, 17000, 27284,
                46096, 20212, 55976, 340,   47888, 52044, 19376, 55368, 42856,
                15644, 30856, 53652, 60168, 33624, 51128, 62588, 46044, 21016,
                58260, 50968, 12380, 18544, 56772, 31720, 61280, 47944, 33852,
                43708, 22200, 61916, 8148,  33344, 3384,  46932, 54864, 11092,
                33580, 64640, 23024, 7712,  61704, 28680, 40504, 56616, 7336,
                53872, 47328, 9188,  3344,  53284, 11232, 59828, 18856, 63556,
                49676, 12888, 6364,  45620, 50072, 14836, 22368, 36364, 60536,
                7732,  49504,
            },
            {
                65008, 57244, 8876,  53444, 41016, 6588,  58416, 50516, 1280,
                60372, 10624, 49576, 40312, 424,   19256, 10024, 50864, 61676,
                20592, 30384, 55860, 11124, 38184, 32020, 1196,  60100, 47556,
                11000, 52872, 3492,  45068, 14672, 32428, 9812,  30196, 44056,
                7332,  39132, 10316, 25916, 42120, 23624, 55672, 61464, 20440,
                7664,  54548, 17412, 45188, 6952,  49188, 40436, 53720, 12616,
                38784, 58844, 10808, 51560, 28408, 55916, 31664, 58928, 2092,
                38292, 30728, 7604,  62900, 34884, 28792, 4140,  39344, 23184,
                2596,  48680, 8520,  19416, 44472, 6232,  30092, 54540, 14232,
                32364, 4468,  64620, 45408, 36852, 14324, 41772, 7172,  58692,
                30232, 4820,  40284, 27356, 49056, 23388, 37064, 18400, 25512,
                44300, 30160, 57024, 2996,  36280, 48408, 17800, 65300, 43680,
                23344, 15460, 41352, 25680, 49440, 32616, 40296, 23844, 30756,
                412,   35096, 61560, 24576, 2024,  31156, 44100, 4184,  18132,
                13192, 34696,
            },
            {
                24060, 38960, 27260, 12284, 62392, 24208, 36356, 55112, 26536,
                47408, 23448, 56184, 13040, 52448, 25452, 37248, 6576,  43652,
                12524, 1396,  40796, 61164, 49848, 8100,  28568, 20188, 39248,
                16116, 65084, 35032, 19372, 50036, 63632, 37652, 13276, 52468,
                48588, 63936, 14804, 50936, 62332, 12412, 37364, 3896,  34752,
                58924, 42040, 2620,  36756, 61760, 25072, 35228, 1616,  21044,
                26084, 5160,  32184, 22596, 4316,  40752, 12072, 44312, 22996,
                64156, 49012, 17700, 40732, 22564, 60024, 13620, 65328, 45532,
                34428, 61592, 37616, 28184, 55904, 16772, 3008,  35940, 26128,
                43440, 9384,  40072, 27364, 23656, 692,   53728, 20600, 16476,
                51396, 46040, 12432, 19400, 57072, 42912, 5856,  63052, 49620,
                3852,  15548, 20032, 42464, 24444, 8960,  12920, 35108, 3136,
                29292, 38328, 57004, 20708, 64044, 4752,  46368, 7768,  55532,
                44352, 15944, 38036, 54908, 11396, 62456, 51684, 57948, 28452,
                46832, 3296,
            },
            {
                16120, 50736, 20024, 34120, 49080, 3600,  18744, 31960, 8928,
                16108, 34316, 43780, 6108,  32228, 45564, 15648, 56080, 27456,
                35608, 47468, 5760,  26492, 16604, 63976, 13984, 54016, 32880,
                5476,  29456, 25548, 58288, 1524,  23984, 5540,  60028, 17364,
                23152, 33208, 28948, 46288, 6012,  31080, 17480, 49412, 10524,
                26196, 13608, 22068, 51712, 10752, 19648, 15144, 64464, 45824,
                41532, 55272, 49044, 60248, 15676, 47280, 19964, 8440,  54852,
                13252, 26360, 52752, 10508, 5956,  37004, 49860, 26080, 9524,
                57640, 21268, 4840,  13920, 64276, 40020, 23180, 49316, 61756,
                18152, 52728, 59440, 6644,  49964, 63320, 29096, 38824, 9712,
                36192, 65508, 53912, 164,   60852, 14852, 30376, 40044, 9036,
                34736, 54464, 38252, 50288, 58944, 32304, 52884, 25980, 49976,
                6272,  60204, 31928, 924,   14052, 35892, 17464, 58640, 10492,
                21808, 51432, 27384, 18652, 41704, 33948, 20828, 8388,  41000,
                55452, 31404,
            },
            {
                45124, 63152, 1872,  56320, 29936, 14932, 43512, 65512, 38488,
                4232,  61596, 27744, 58788, 21756, 63868, 49132, 2780,  22904,
                63176, 58264, 32944, 22044, 42952, 35332, 46444, 23784, 42088,
                56168, 50652, 12372, 38548, 42500, 28648, 47368, 34364, 41024,
                4164,  59376, 624,   19712, 40520, 53816, 22752, 43448, 52968,
                63684, 32212, 46688, 29240, 55472, 42788, 4676,  57836, 29824,
                17564, 8240,  36112, 2700,  25748, 62712, 37668, 50688, 34472,
                4876,  43052, 33132, 57428, 46792, 16132, 54032, 1852,  29748,
                17592, 50680, 43340, 24820, 46740, 11576, 59020, 7544,  38392,
                12696, 30532, 21656, 34856, 11776, 17688, 47236, 60360, 25500,
                2756,  28444, 22724, 32324, 35784, 10348, 52252, 58624, 21668,
                61864, 45348, 4892,  11732, 63900, 1340,  21300, 45048, 62416,
                19380, 11608, 48724, 44704, 55276, 29136, 50448, 26240, 38772,
                65516, 32416, 2888,  48344, 57436, 4624,  15240, 37404, 25112,
                60824, 5880,
            },
            {
                10312, 35764, 42644, 8180,  60692, 47116, 10920, 51764, 22236,
                41472, 53612, 18364, 2112,  39460, 9460,  28780, 41700, 18068,
                52416, 14532, 9720,  51460, 2808,  55496, 4788,  10236, 61076,
                628,   21640, 45756, 8188,  16652, 53888, 57212, 20476, 9408,
                26748, 54424, 37500, 57468, 9704,  64624, 1888,  28532, 39084,
                4480,  18748, 57380, 948,   38076, 26656, 49960, 33536, 52360,
                11432, 62056, 20268, 42552, 53812, 30640, 560,   58536, 28528,
                15836, 59876, 1372,  24460, 30308, 43892, 20536, 33660, 41636,
                12252, 55220, 30708, 1416,  52508, 32824, 20040, 28688, 384,
                47032, 56672, 3448,  44252, 55600, 31392, 4292,  43604, 56004,
                15212, 41548, 8484,  48056, 44316, 25632, 17772, 1580,  28088,
                12996, 24888, 31656, 18532, 28352, 41656, 56524, 38876, 15956,
                54124, 35640, 26892, 8404,  23136, 62192, 42288, 1676,  53780,
                14276, 7096,  59476, 9828,  23244, 30032, 64592, 47572, 11684,
                51256, 21900,
            },
            {
                54784, 28004, 17640, 22820, 37664, 25716, 520,   58132, 28752,
                13324, 7908,  46812, 36136, 14320, 54188, 33952, 57736, 7536,
                31104, 44968, 39620, 19116, 59176, 30296, 38032, 26880, 48948,
                18616, 35808, 59196, 63240, 33096, 11416, 2356,  44812, 64772,
                49624, 13052, 44340, 16388, 25688, 35272, 58456, 15032, 48248,
                9012,  34332, 60868, 15808, 8304,  63180, 13380, 22464, 5964,
                38348, 28368, 45500, 13844, 33880, 10468, 17912, 24068, 40360,
                47800, 20996, 37548, 64564, 12680, 4356,  58700, 62224, 47604,
                6140,  63612, 39420, 35568, 8952,  56852, 42908, 51484, 65248,
                24428, 40660, 14860, 61960, 37796, 8744,  23880, 12848, 33068,
                49704, 58892, 18392, 62676, 4888,  56096, 37380, 47392, 41932,
                51052, 57868, 7644,  47148, 35984, 14844, 6844,  10112, 30268,
                1912,  42680, 64764, 17136, 38580, 12192, 6304,  20300, 34448,
                43920, 28652, 39820, 45520, 35196, 53664, 43160, 1272,  19112,
                33560, 39692,
            },
            {
                14264, 58868, 46224, 5052,  53532, 40588, 33268, 19364, 48740,
                63332, 31816, 24844, 51472, 62004, 20108, 3656,  47208, 12280,
                37580, 192,   25264, 63576, 48256, 12688, 17296, 64524, 40412,
                13748, 29996, 6268,  26324, 48644, 23204, 39216, 14908, 31148,
                35952, 6772,  21464, 51452, 32308, 45556, 7620,  20304, 61904,
                25220, 44864, 40996, 23468, 47880, 30432, 44496, 1952,  59136,
                48484, 24380, 56564, 6856,  51028, 63836, 46236, 55748, 7136,
                62212, 10016, 50504, 17860, 54888, 39872, 8796,  23956, 36408,
                27368, 15964, 22396, 59424, 18420, 26364, 4552,  36352, 16380,
                33384, 8448,  49404, 25824, 19236, 53412, 40380, 64336, 20888,
                5620,  38092, 52856, 29448, 13888, 21084, 60656, 31964, 4084,
                16460, 39072, 62808, 55628, 23096, 50484, 59584, 34128, 47284,
                23828, 57220, 5508,  32700, 53196, 47684, 30812, 60904, 16696,
                49028, 24072, 61468, 13408, 17288, 5920,  25784, 56340, 29364,
                61976, 2804,
            },
            {
                24692, 51840, 32248, 12428, 63832, 15840, 55596, 6020,  36416,
                3332,  42748, 56608, 11316, 30204, 41060, 26424, 65184, 23592,
                60064, 53712, 28884, 6460,  34264, 23328, 44700, 7848,  32636,
                56760, 52100, 43844, 3396,  18296, 61484, 52240, 5628,  24348,
                60936, 42752, 29548, 62536, 2656,  12508, 50436, 36652, 30876,
                54956, 12128, 5268,  53128, 35864, 55980, 17444, 40908, 34724,
                14964, 65096, 1320,  40240, 21928, 27228, 3892,  43524, 13696,
                35396, 29592, 3668,  42624, 32800, 27784, 51644, 18660, 196,
                53272, 10056, 48784, 3268,  61568, 14036, 46232, 10872, 54916,
                20740, 59568, 29260, 1988,  34180, 47640, 58148, 868,   45208,
                27052, 11312, 34216, 1924,  39468, 50584, 8024,  11556, 54160,
                34312, 20272, 360,   10616, 43200, 3696,  26448, 61912, 20088,
                51676, 13348, 40280, 21616, 2588,  58404, 25244, 37392, 55588,
                10820, 5260,  53180, 21008, 50148, 63020, 37828, 12988, 49352,
                7968,  44012,
            },
            {
                64648, 4076,  36672, 49212, 29336, 21720, 9580,  45548, 26580,
                59128, 17100, 21856, 688,   45368, 5832,  15800, 50748, 35368,
                18708, 46396, 15500, 41648, 54640, 2460,  57704, 50220, 3984,
                24364, 10096, 37232, 55396, 40932, 27920, 35416, 58656, 17816,
                54976, 10284, 4332,  53496, 38576, 23796, 56240, 42020, 448,
                17164, 49904, 28596, 20060, 2924,  11008, 60088, 25672, 54356,
                9400,  18956, 30184, 35424, 60592, 16172, 38232, 32216, 23080,
                52092, 57028, 25516, 46528, 6612,  60444, 14588, 38192, 44692,
                58284, 32548, 43020, 28832, 40584, 50528, 31276, 62432, 41968,
                5520,  44740, 52604, 63980, 6824,  14192, 28644, 36716, 16956,
                54980, 61672, 22544, 46960, 64396, 42944, 27808, 23744, 63472,
                44468, 26732, 30368, 52972, 38012, 32448, 16860, 43964, 4524,
                36660, 28740, 49284, 60544, 43380, 15332, 8884,  46340, 3180,
                63904, 32592, 37012, 960,   31044, 9156,  41488, 22348, 58032,
                35428, 18300,
            },
            {
                40668, 20992, 8644,  60332, 1692,  42000, 50444, 62740, 13872,
                39172, 9028,  34004, 50416, 60472, 38116, 55168, 9796,  43016,
                4068,  8460,  62288, 21308, 11072, 36388, 27556, 20016, 62640,
                46472, 16164, 21932, 65216, 13268, 8784,  45452, 168,   48200,
                27264, 40760, 15716, 46600, 18864, 27772, 59748, 6276,  22332,
                63784, 38268, 58744, 32760, 64788, 38940, 21204, 6940,  31620,
                47232, 42248, 52980, 12224, 49172, 8992,  54660, 59508, 1712,
                19436, 38956, 15264, 63672, 22356, 11376, 49232, 30144, 64996,
                20940, 13200, 55716, 6648,  23192, 37488, 19532, 1152,  28144,
                23648, 13240, 38592, 17908, 42936, 22156, 10192, 51568, 31952,
                7888,  42028, 15904, 25808, 6596,  17604, 35648, 58100, 15044,
                5224,  49644, 61292, 13428, 21372, 65132, 53948, 12244, 57896,
                8356,  63208, 18368, 10528, 27548, 34084, 52052, 22444, 41096,
                18976, 27092, 42880, 58760, 46728, 55528, 27824, 5040,  16044,
                46424, 30324,
            },
            {
                54532, 27348, 43612, 16788, 25092, 56716, 34572, 30740, 23132,
                54236, 43992, 64652, 27416, 18312, 32152, 22556, 59480, 29128,
                33104, 57224, 37976, 50892, 31420, 60380, 43000, 14152, 38400,
                28772, 59696, 33884, 2120,  30552, 51152, 21156, 32988, 12292,
                37204, 22760, 58052, 34372, 65444, 9548,  14752, 48020, 34208,
                10176, 43544, 14872, 7948,  48564, 27632, 45404, 51700, 63256,
                2784,  22660, 58380, 5836,  44176, 20348, 28972, 47628, 11708,
                62784, 7992,  49636, 976,   35908, 41308, 55964, 2884,  5720,
                25368, 35292, 46840, 16708, 64136, 52692, 8528,  58412, 47768,
                35520, 56756, 9776,  30748, 48848, 58880, 62992, 24568, 46512,
                57516, 2720,  50856, 58964, 31544, 54588, 916,   46080, 37656,
                9432,  40988, 18116, 46712, 7680,  28504, 1828,  40008, 25436,
                46348, 31604, 1200,  37632, 56060, 5972,  62700, 29972, 13048,
                57344, 49336, 15308, 24548, 11992, 18640, 34020, 51724, 59980,
                68,    10748,
            },
            {
                61476, 13544, 57660, 47024, 38020, 11580, 6880,  18568, 1192,
                48516, 4660,  36996, 11808, 7216,  47076, 13716, 1428,  48412,
                19976, 12356, 26848, 1764,  47396, 17748, 9116,  52632, 456,
                41876, 6452,  48472, 54012, 43188, 15856, 60540, 56440, 7348,
                63648, 49700, 1408,  30768, 5464,  44248, 39448, 29764, 53008,
                26400, 55892, 2216,  24440, 41544, 16188, 3748,  13604, 36784,
                28388, 15948, 38024, 25372, 32740, 64536, 4748,  24732, 41872,
                33848, 44792, 30284, 58096, 18760, 26500, 33468, 17028, 52540,
                42124, 9144,  58940, 1916,  31860, 12272, 25848, 40060, 15828,
                50828, 4648,  61320, 25532, 34576, 5652,  40756, 3704,  19600,
                38856, 28160, 36168, 9624,  13224, 48992, 20768, 29392, 52364,
                22856, 56036, 2280,  35540, 58712, 42476, 48580, 19800, 34660,
                55492, 15564, 24024, 45560, 50408, 20104, 39572, 296,   45136,
                34768, 8376,  3800,  65116, 40164, 2500,  62316, 43344, 25600,
                38212, 48392,
            },
            {
                6736,  34708, 2480,  31768, 20260, 53796, 61636, 40180, 58276,
                28476, 15612, 24520, 57160, 52388, 41244, 64028, 25448, 39600,
                53296, 63536, 45112, 23996, 56588, 4744,  61828, 25204, 32044,
                55840, 12096, 18628, 23780, 38560, 4284,  25636, 40228, 29296,
                43900, 19848, 13856, 55128, 24904, 50928, 60896, 18220, 4300,
                46104, 19612, 36236, 61824, 52852, 57204, 33748, 60808, 43196,
                10364, 55232, 62412, 508,   51220, 40308, 15100, 36644, 53668,
                17012, 26948, 52280, 10552, 43484, 61928, 7896,  45696, 60128,
                38860, 18996, 27672, 49272, 36184, 45140, 54776, 3436,  63668,
                32368, 19836, 45792, 24,    55756, 15880, 29696, 54440, 10876,
                14564, 65164, 21516, 44976, 40580, 61312, 4780,  63972, 12108,
                33016, 60344, 25656, 31252, 52164, 23464, 9508,  14296, 61200,
                5156,  52724, 41824, 64100, 12764, 26036, 59160, 17280, 53784,
                23408, 60636, 51512, 29400, 35884, 54648, 8824,  20340, 14204,
                32676, 22796,
            },
            {
                18712, 39996, 51208, 65188, 4808,  26888, 44572, 13216, 51412,
                32928, 60756, 45928, 19688, 29408, 3292,  33608, 56164, 6720,
                16808, 35280, 5804,  15108, 39852, 29736, 35132, 49660, 21392,
                64200, 36976, 27156, 57768, 10148, 62420, 47812, 18096, 2624,
                52272, 9324,  61780, 41300, 35840, 21012, 2688,  57552, 13688,
                62744, 32112, 50348, 12020, 30052, 18780, 8732,  23404, 50500,
                19580, 31820, 46356, 12928, 18656, 56828, 8708,  61304, 2492,
                58580, 6048,  21224, 37840, 4368,  48556, 13840, 22640, 29176,
                11692, 62656, 53392, 4812,  21492, 60576, 17884, 29760, 10556,
                22936, 41792, 13600, 37808, 51196, 21184, 44900, 35204, 49856,
                60012, 33000, 5772,  56084, 18092, 33756, 24540, 43532, 16660,
                40100, 7124,  45268, 15284, 4304,  37320, 63420, 50648, 29632,
                39064, 21448, 8072,  32828, 3324,  36212, 9476,  48448, 31660,
                5016,  38316, 44108, 16552, 22108, 45852, 31528, 49572, 4720,
                64068, 52484,
            },
            {
                44732, 29388, 23488, 15572, 48732, 9176,  35756, 21428, 4132,
                42904, 10340, 5716,  38672, 62292, 14592, 44628, 21332, 11140,
                50352, 30744, 59076, 43424, 53928, 19788, 12828, 44616, 3228,
                16424, 45984, 7424,  50916, 35188, 31728, 13056, 54624, 36176,
                23572, 33684, 27204, 47336, 7764,  12196, 31744, 37160, 23904,
                8932,  42348, 6584,  22608, 772,   37888, 47188, 26404, 5460,
                59048, 39804, 7392,  27312, 34552, 44984, 29300, 47992, 23436,
                31912, 13024, 65236, 54952, 24852, 30932, 56348, 37304, 50924,
                604,   33724, 24108, 43400, 14116, 41260, 6780,  34772, 46804,
                59148, 53592, 28412, 64956, 7036,  12100, 61532, 26440, 1084,
                42456, 23732, 52320, 2320,  27636, 37564, 9004,  57368, 3040,
                51308, 20580, 64604, 10976, 55188, 17888, 32952, 1068,  25804,
                11700, 59436, 48272, 18236, 29284, 56832, 42424, 64644, 12332,
                27440, 20208, 9904,  58156, 5748,  13416, 59368, 28192, 41616,
                56472, 9772,
            },
            {
                3328,  60996, 11456, 42340, 59268, 29064, 52988, 62844, 25168,
                17008, 35284, 55908, 22852, 48916, 9232,  36768, 60184, 27912,
                42216, 620,   22532, 10452, 65360, 7240,  37784, 57588, 9692,
                53348, 30116, 60180, 1000,  43932, 22064, 5044,  64932, 46268,
                15448, 59816, 3808,  17384, 52636, 63712, 43248, 48764, 54332,
                39300, 27136, 56644, 45196, 60104, 40844, 63816, 53708, 34956,
                2324,  48840, 21904, 54604, 60480, 3192,  20796, 11016, 42576,
                50364, 35504, 45776, 15932, 40768, 2384,  62976, 19960, 6476,
                46448, 16128, 55820, 8812,  65408, 28228, 51688, 56980, 38916,
                2080,  8248,  17212, 33280, 24200, 40428, 56444, 18216, 8604,
                48104, 30464, 11856, 46188, 59224, 14380, 53992, 48036, 26372,
                36208, 30332, 49236, 38984, 27844, 59228, 46608, 40984, 56624,
                45104, 4280,  36980, 61564, 53304, 15176, 23848, 2040,  40204,
                55324, 62328, 32912, 47796, 26664, 53056, 39168, 1920,  17704,
                35532, 26320,
            },
            {
                53672, 36928, 56696, 888,   33816, 18256, 39296, 2240,  46884,
                30304, 64472, 51084, 96,    26512, 31636, 53824, 17988, 46460,
                62864, 38368, 52336, 33732, 27464, 48756, 23672, 62540, 40160,
                33824, 23044, 41112, 14700, 19012, 58460, 39012, 26844, 10648,
                42352, 55920, 37996, 29560, 58420, 22684, 272,   28164, 15408,
                4700,  65092, 17784, 13516, 32620, 4492,  15428, 11428, 29308,
                17492, 65280, 10124, 42320, 15540, 37768, 52876, 63924, 17616,
                39608, 252,   28220, 8412,  59256, 34336, 10764, 43764, 26868,
                58360, 39988, 30216, 48284, 37096, 1276,  24588, 20144, 15028,
                26308, 49192, 60272, 43720, 47916, 3168,  31332, 37308, 53320,
                15360, 39744, 19488, 63852, 7232,  40884, 30984, 19772, 62240,
                13376, 58004, 1660,  23228, 43240, 5640,  21120, 8720,  16208,
                35016, 23188, 13668, 26776, 44220, 6796,  46952, 34640, 18444,
                50128, 14292, 41668, 940,   36348, 64456, 23332, 11584, 61812,
                46584, 14448,
            },
            {
                31240, 48336, 20704, 26048, 46196, 7060,  14552, 57944, 54344,
                7428,  40120, 12844, 18560, 42672, 58020, 6016,  3064,  24892,
                8368,  14476, 19280, 3796,  45828, 15640, 1056,  28336, 18040,
                3964,  11840, 49016, 64016, 28988, 53072, 8124,  50132, 30528,
                820,   20916, 5940,  45256, 13744, 40144, 9444,  61008, 33720,
                20776, 47564, 35400, 53440, 27796, 50124, 21604, 42688, 56252,
                45296, 36740, 31328, 23836, 49872, 6456,  25908, 30832, 4860,
                59684, 54408, 20096, 51656, 23120, 49196, 17676, 53828, 35832,
                63732, 3536,  12656, 18436, 32764, 58792, 11736, 42780, 63444,
                30540, 36120, 4040,  19952, 57676, 10608, 64000, 21924, 6024,
                62068, 55664, 25136, 34900, 50460, 22496, 824,   45764, 6360,
                42268, 18264, 9184,  54284, 32336, 12660, 61252, 52664, 30772,
                65288, 49148, 55348, 632,   38344, 21060, 51936, 58620, 8044,
                29496, 4056,  24236, 51336, 16884, 8184,  44212, 33572, 50712,
                21236, 7452,
            },
            {
                63440, 16740, 9000,  41152, 64132, 50164, 23280, 36504, 11268,
                20276, 27388, 45232, 34024, 61600, 15528, 39732, 51148, 34204,
                56776, 30364, 61396, 40444, 58376, 54500, 32164, 51236, 43232,
                61172, 55668, 25956, 34744, 2436,  45496, 24496, 17176, 61340,
                33192, 48008, 64168, 25356, 35120, 49512, 18328, 55560, 44264,
                51296, 1520,  10132, 24152, 58716, 7988,  37180, 62528, 24344,
                6116,  13996, 57988, 1780,  62376, 33652, 44280, 57304, 14416,
                24784, 10208, 62468, 32648, 5436,  42392, 29652, 14364, 8596,
                23680, 20752, 52820, 61716, 7076,  45320, 50292, 5152,  55412,
                9492,  52576, 12860, 38664, 29272, 16264, 50604, 44652, 27180,
                33136, 1640,  9668,  43740, 16356, 60568, 33376, 11440, 52492,
                28660, 34896, 45160, 62644, 15968, 36304, 48088, 27516, 2916,
                19476, 42316, 9868,  29776, 63692, 11500, 31804, 25924, 62912,
                37252, 46116, 57568, 61400, 30676, 19888, 55104, 5188,  29616,
                58352, 40088,
            },
            {
                24828, 4348,  32816, 55216, 12552, 30112, 60660, 43868, 32976,
                49008, 60008, 3424,  52440, 10536, 22244, 29164, 64944, 12204,
                47584, 22940, 44520, 11356, 25536, 6896,  36796, 13044, 46892,
                20468, 8736,  38508, 6176,  57100, 13412, 41972, 36316, 54900,
                14432, 51860, 11512, 7520,  57308, 31076, 5060,  24924, 12112,
                30120, 59936, 38968, 43168, 3444,  46544, 17872, 32068, 160,
                49596, 27692, 53268, 40336, 19292, 12784, 8480,  48704, 38760,
                35012, 46848, 43260, 12796, 38176, 64176, 1064,  57292, 45084,
                32472, 49528, 42092, 26524, 38440, 21888, 16588, 34096, 39940,
                45896, 22576, 61728, 25928, 54480, 920,   35168, 40964, 12804,
                57372, 46392, 38280, 29092, 3940,  54740, 25852, 65396, 39260,
                23504, 59796, 4528,  26724, 51584, 812,   24372, 39480, 57600,
                6532,  33084, 51072, 17812, 57008, 40624, 2868,  15696, 43024,
                10888, 21608, 6676,  39712, 11924, 48312, 60404, 25880, 37692,
                388,   43756,
            },
            {
                13244, 60208, 51760, 37344, 2520,  19152, 5552,  25896, 1500,
                16280, 55768, 23688, 37152, 6608,  48280, 43660, 19940, 37276,
                2380,  53628, 5560,  34876, 17344, 64388, 22080, 59500, 4848,
                30696, 52248, 16752, 47832, 32156, 21552, 59400, 6812,  2980,
                23428, 38396, 28816, 41780, 20236, 52356, 36820, 64404, 40836,
                7688,  22408, 16344, 63940, 29576, 55632, 10896, 52120, 60620,
                38544, 20932, 9616,  46456, 29716, 59812, 54836, 21728, 1512,
                18380, 29472, 3248,  26436, 55884, 19120, 25152, 51248, 39048,
                5516,  59216, 1688,  10644, 56124, 29960, 62424, 25596, 708,
                18584, 31900, 6380,  41980, 47064, 8904,  24404, 60072, 4124,
                20780, 16944, 51388, 58684, 12972, 48548, 36488, 17380, 2708,
                56464, 14536, 48816, 20008, 40740, 63956, 10296, 44356, 14788,
                60704, 37648, 25096, 4724,  47632, 34260, 60720, 19932, 48676,
                54760, 33940, 15016, 27404, 35060, 2252,  40944, 15588, 11020,
                19104, 53176,
            },
            {
                35612, 45940, 21804, 27304, 44744, 56872, 40832, 51856, 65488,
                37896, 8724,  29992, 63584, 32420, 54856, 1028,  26132, 59552,
                16676, 41788, 27624, 55956, 49380, 39092, 9808,  41956, 26780,
                35760, 63060, 23860, 41252, 61688, 10168, 29500, 46724, 63252,
                18224, 44404, 60408, 1380,  15908, 61840, 2892,  14468, 47036,
                57188, 36020, 52900, 12604, 33640, 20496, 40184, 25768, 15268,
                7244,  34156, 64116, 16448, 3672,  36572, 27140, 41652, 64988,
                50116, 60524, 53292, 15596, 47868, 8084,  35244, 61376, 12404,
                17404, 28080, 36132, 15304, 47376, 3356,  52376, 13332, 57416,
                64440, 49828, 15504, 58564, 33480, 63116, 18424, 52952, 47840,
                31072, 64376, 23508, 6164,  41320, 20220, 8052,  46868, 43552,
                10116, 32812, 37736, 7476,  30596, 56720, 17656, 34088, 53644,
                22240, 11872, 62704, 43548, 13860, 22876, 52916, 5448,  28272,
                220,   64064, 51744, 43916, 56896, 23756, 53580, 46784, 65268,
                57116, 28496,
            },
            {
                49436, 5444,  17460, 9724,  63128, 14992, 31380, 10864, 21444,
                46636, 42492, 14268, 19276, 40188, 58336, 14868, 8196,  50836,
                31572, 10328, 63296, 20516, 30972, 1496,  52744, 16012, 57180,
                2072,  44840, 13896, 4276,  53372, 18844, 50468, 39360, 33900,
                26900, 9248,  54020, 32804, 47512, 23640, 43716, 27964, 32544,
                19272, 2336,  27088, 48864, 6428,  61460, 2680,  45740, 58232,
                41916, 48356, 55236, 25364, 43852, 51508, 5168,  16028, 32504,
                9788,  6732,  22328, 36624, 58996, 41768, 3592,  22016, 31100,
                46696, 65024, 54048, 20228, 60036, 43124, 8984,  37852, 28304,
                43536, 10904, 36512, 4560,  21256, 12460, 39640, 28536, 7312,
                37524, 10272, 34404, 61272, 27160, 31932, 57908, 62816, 29904,
                21812, 51976, 61940, 12472, 47244, 22956, 4116,  50356, 7736,
                45972, 28952, 1188,  54968, 27532, 8012,  45332, 38096, 58444,
                41256, 23160, 18000, 3532,  62228, 9652,  31176, 6640,  34008,
                22612, 3184,
            },
            {
                42148, 30360, 60988, 38936, 48132, 22812, 36028, 54192, 6728,
                27496, 59764, 50524, 4836,  11388, 27948, 46008, 35404, 62200,
                39324, 48368, 3644,  14044, 46312, 60832, 23704, 33212, 48420,
                19496, 10408, 56352, 27004, 36872, 548,   24984, 12708, 57784,
                3928,  49536, 21512, 12288, 58756, 38908, 8360,  50048, 54080,
                5428,  62340, 44956, 41340, 24632, 50808, 35508, 28480, 18988,
                11932, 30028, 1460,  21992, 33464, 62856, 11596, 45648, 24352,
                56380, 40172, 44764, 31432, 10948, 27988, 54364, 49340, 9360,
                43292, 23992, 7528,  40192, 31736, 23116, 34612, 48096, 20004,
                2956,  24148, 56272, 29436, 51660, 44420, 2468,  57068, 15376,
                42692, 55008, 2052,  45024, 14924, 53236, 52,    24720, 15668,
                4216,  55700, 25760, 2192,  42196, 60056, 28036, 38720, 19672,
                31748, 59364, 39808, 18684, 36108, 64688, 31116, 16932, 12732,
                33060, 7776,  30488, 49496, 13908, 37340, 18480, 44872, 51304,
                39376, 10012,
            },
            {
                57724, 14056, 52308, 33564, 928,   6332,  59212, 49072, 17948,
                33600, 440,   24900, 44768, 61896, 22816, 52496, 17840, 4708,
                21128, 24596, 58644, 37396, 8380,  27904, 44196, 6320,  63856,
                38248, 29460, 49428, 33100, 64660, 44548, 8088,  55212, 42504,
                16564, 65448, 29352, 35844, 4740,  26504, 17452, 11392, 59648,
                22992, 37932, 10240, 18072, 59108, 14376, 9172,  64468, 53692,
                4928,  37588, 61100, 14216, 8912,  57880, 19468, 37476, 52680,
                28204, 13264, 61932, 1632,  20444, 64496, 34012, 16516, 57664,
                48,    35056, 51616, 12588, 4548,  63848, 17324, 6148,  60900,
                53164, 40784, 46852, 17228, 61504, 25312, 32496, 63716, 22020,
                50800, 25968, 19228, 48824, 39236, 10636, 35576, 42416, 50380,
                39776, 34352, 45424, 16232, 53848, 35852, 13900, 64876, 56384,
                4872,  49076, 14984, 51488, 5792,  57932, 10064, 24656, 49884,
                60224, 45800, 55316, 26164, 40288, 21896, 60948, 1148,  26484,
                16680, 62948,
            },
            {
                20192, 24564, 7668,  56012, 25864, 43692, 29024, 13636, 62804,
                40352, 57112, 36464, 54312, 33156, 2840,  42116, 30276, 11724,
                55788, 43256, 33576, 52260, 18496, 40252, 12132, 53804, 14544,
                22984, 59408, 5104,  40744, 21176, 15580, 60648, 31740, 22548,
                37292, 7296,  46120, 41332, 56088, 51636, 64024, 34800, 42024,
                29816, 14776, 54732, 32144, 816,   38660, 22256, 44148, 32688,
                24224, 56840, 42604, 50700, 39728, 30416, 47516, 348,   60220,
                6032,  34828, 16876, 51804, 46344, 5028,  12916, 39320, 26796,
                62132, 19096, 29624, 58172, 46388, 26168, 50980, 56708, 30680,
                13928, 33688, 8372,  212,   37352, 10044, 49160, 5300,  35104,
                12036, 59888, 30188, 4268,  64860, 22572, 56620, 5264,  61000,
                19024, 7832,  64408, 20944, 31584, 9448,  364,   44076, 23476,
                10848, 42220, 25328, 32904, 21340, 47948, 42512, 54128, 1668,
                20604, 36124, 11264, 64824, 4404,  47748, 57508, 29636, 12748,
                54584, 32660,
            },
            {
                1820,  37688, 44816, 18320, 64812, 39768, 20492, 2792,  45028,
                9008,  12340, 20356, 16256, 7196,  38428, 57648, 64680, 47492,
                27472, 7484,  15876, 2276,  65196, 56120, 35072, 344,   31472,
                42972, 8468,  17388, 54996, 10924, 51648, 28196, 47264, 2076,
                52912, 59460, 14260, 19156, 352,   31680, 21380, 6396,  45916,
                1744,  49532, 65220, 27452, 47544, 57012, 52412, 3664,  16428,
                49024, 7640,  20012, 27400, 4312,  54564, 23708, 14832, 42812,
                20816, 49032, 25724, 38884, 56656, 24300, 43004, 52756, 6708,
                47448, 14556, 42292, 2284,  37088, 15452, 41752, 10732, 38700,
                21876, 65332, 27012, 58888, 41856, 14728, 54652, 18808, 38816,
                46176, 8112,  56000, 33392, 17448, 51920, 28712, 13824, 31700,
                48120, 11536, 28556, 57392, 39576, 51516, 26528, 48028, 16736,
                36772, 63284, 1848,  60700, 12364, 38832, 4420,  27208, 63432,
                39932, 5412,  16596, 43384, 33184, 8752,  52976, 35588, 43572,
                7236,  47128,
            },
            {
                62100, 50496, 29840, 12876, 9456,  51180, 33040, 56312, 24328,
                30404, 52524, 64364, 27324, 48140, 9976,  21584, 14408, 592,
                37000, 61348, 49696, 30604, 25668, 21248, 47752, 58476, 26556,
                50676, 61612, 45612, 25936, 1608,  34680, 39244, 6312,  19552,
                27176, 33456, 10684, 25296, 60980, 47852, 15120, 39632, 57256,
                25224, 20488, 8280,  36244, 5472,  18788, 30300, 41120, 62020,
                13084, 34844, 46000, 65424, 17360, 34444, 7956,  62464, 29368,
                57064, 2812,  63660, 8392,  30148, 18388, 59328, 31576, 21336,
                37844, 55556, 25120, 60716, 9132,  54272, 28700, 1060,  45436,
                5372,  50112, 18580, 53568, 30576, 23372, 45184, 27680, 61356,
                1132,  23752, 40888, 13036, 44104, 37680, 8684,  45744, 25320,
                58364, 37092, 3540,  43624, 5988,  18448, 62372, 33240, 53940,
                28676, 8224,  52624, 45384, 29884, 56092, 17972, 34384, 14404,
                29412, 52760, 24292, 58236, 28016, 19992, 15336, 3712,  64452,
                22764, 39920,
            },
            {
                27152, 4252,  59572, 36448, 54328, 4760,  15280, 60548, 37400,
                49516, 3472,  41816, 31492, 60164, 51384, 25840, 33876, 44396,
                52812, 19812, 12724, 41392, 45504, 10260, 4476,  16392, 39492,
                19912, 3716,  36516, 30864, 48316, 63676, 13100, 56328, 62144,
                41992, 50512, 38152, 54828, 42824, 8772,  28968, 62768, 11964,
                53544, 33172, 60124, 43828, 13292, 63088, 26100, 9620,  37256,
                58820, 25196, 1208,  11400, 59484, 43504, 50108, 36832, 10660,
                40856, 32916, 44480, 12400, 37212, 984,   49608, 9892,  63548,
                2836,  11284, 32736, 48636, 22260, 34304, 19148, 62600, 24796,
                57864, 36344, 12104, 47592, 6928,  64032, 3256,  57172, 31348,
                16600, 53004, 62908, 27352, 6000,  59492, 20540, 63912, 2036,
                52996, 22124, 14644, 49092, 24412, 59632, 11944, 3212,  39948,
                58160, 20180, 34688, 16436, 3012,  23760, 59404, 47300, 50988,
                9268,  60556, 46284, 1016,  38408, 49792, 42068, 25240, 58668,
                17932, 11228,
            },
            {
                34992, 15748, 21312, 46316, 23908, 42372, 27144, 47172, 18284,
                6112,  22456, 14604, 45588, 1876,  18796, 40952, 55516, 8288,
                31188, 23684, 57192, 6224,  59620, 37472, 51788, 62484, 33892,
                11420, 55520, 14076, 22404, 58112, 18064, 43684, 24204, 9468,
                15896, 2744,  64152, 5348,  22972, 33972, 50868, 3792,  18288,
                37756, 4828,  16216, 40500, 22652, 46404, 50772, 2008,  21240,
                54724, 29132, 52096, 38712, 31208, 21520, 3004,  25388, 52584,
                18924, 15220, 55056, 22536, 46904, 61696, 15756, 35680, 28416,
                45216, 52192, 17544, 64828, 6820,  43904, 59340, 48016, 32040,
                16352, 40928, 1892,  21524, 34268, 39528, 13388, 9308,  42996,
                50060, 36328, 20852, 46920, 1556,  49808, 35052, 40272, 16916,
                42712, 33144, 62852, 55384, 29956, 35512, 46476, 22160, 6520,
                14160, 49716, 26204, 41264, 65032, 37412, 10952, 6800,  21864,
                41600, 32128, 19044, 13188, 63056, 56324, 10604, 37024, 31400,
                48508, 53980,
            },
            {
                61192, 41732, 56984, 7012,  31484, 63772, 684,   10480, 34544,
                62364, 55332, 38988, 57792, 11852, 36420, 63588, 4324,  16816,
                62396, 39504, 2852,  33708, 17728, 28516, 23940, 7344,  46504,
                29420, 64936, 42380, 52372, 8856,  2912,  29100, 36752, 49484,
                31896, 20972, 28176, 46908, 16952, 36936, 59556, 26392, 44324,
                47972, 30568, 51448, 57484, 10720, 34644, 55648, 31756, 44936,
                17476, 41784, 5800,  48228, 15848, 55576, 12964, 45812, 64112,
                4028,  27444, 60084, 5984,  51904, 26160, 41960, 4264,  57760,
                23364, 40388, 5072,  30088, 14096, 39604, 3380,  12504, 8576,
                55064, 28984, 63016, 43772, 59236, 51260, 19588, 25256, 33944,
                5004,  14372, 9932,  57696, 31988, 24816, 11196, 55548, 6968,
                27504, 10028, 38976, 1544,  8640,  15816, 41372, 54848, 31256,
                44412, 61900, 10184, 5904,  46548, 53024, 31616, 44320, 62688,
                2304,  36960, 54716, 7408,  23320, 30288, 5176,  52300, 460,
                44416, 8104,
            },
            {
                24104, 2920,  33540, 11896, 50168, 17052, 38288, 59052, 43304,
                29012, 8508,  26312, 33424, 6748,  23948, 29212, 46024, 49824,
                11056, 26692, 54412, 48832, 64216, 43628, 13508, 53904, 40780,
                1072,  25336, 6432,  38552, 33256, 60516, 46796, 5084,  55164,
                59244, 44796, 12436, 57420, 53384, 1080,  13568, 21472, 56396,
                9716,  62508, 24144, 268,   28200, 6776,  15604, 61296, 4588,
                12336, 63780, 22900, 9256,  61820, 27780, 33844, 57856, 7684,
                39284, 49788, 36100, 31092, 19956, 10852, 32844, 53456, 19312,
                13008, 59992, 36300, 50960, 57332, 27084, 53184, 23156, 35412,
                50696, 4396,  25048, 10216, 15660, 27932, 54460, 47828, 58672,
                64352, 29544, 40052, 54200, 17788, 62324, 15012, 30460, 47968,
                61256, 19088, 52156, 45488, 20868, 65344, 50636, 27292, 59092,
                1184,  36884, 55148, 21996, 28364, 13772, 19720, 55636, 25304,
                16072, 27560, 50276, 43468, 35112, 46844, 17560, 63680, 21244,
                13892, 28796,
            },
            {
                51672, 47600, 19244, 61944, 27884, 55208, 22980, 13764, 52960,
                20028, 50852, 16748, 48320, 61512, 53744, 13372, 58188, 22048,
                35812, 42800, 15312, 9592,  21416, 2248,  32568, 60924, 17180,
                21724, 57312, 50216, 15884, 20172, 25868, 40332, 14308, 22628,
                34496, 7416,  39760, 30820, 9912,  43568, 65040, 40108, 29432,
                2528,  35824, 19744, 47020, 63844, 39340, 42876, 25004, 48456,
                38060, 32768, 54156, 35632, 44636, 808,   41408, 20064, 30776,
                23768, 9848,  43404, 1784,  56504, 39568, 65400, 7304,  48180,
                43244, 26264, 2232,  21408, 46532, 18208, 37772, 63920, 42668,
                20252, 45596, 56584, 37244, 31560, 2404,  6288,  37884, 21884,
                392,   45604, 23144, 7292,  43584, 38480, 51792, 4544,  44216,
                23820, 57740, 13080, 32092, 25872, 37444, 6828,  11128, 18848,
                24248, 15480, 32796, 48792, 59864, 148,   35628, 4884,  39836,
                57820, 11648, 60940, 3368,  15132, 59240, 26812, 41004, 33316,
                55844, 39380,
            },
            {
                65340, 9708,  44228, 37008, 2288,  41236, 47892, 5756,  32120,
                3264,  64924, 41288, 472,   20420, 43112, 38704, 3176,  32292,
                6472,  59940, 46836, 29784, 37128, 51264, 58128, 10400, 36032,
                48060, 31940, 11184, 44440, 62872, 54180, 10388, 51024, 63528,
                2292,  17504, 60812, 25340, 20472, 49544, 32104, 7316,  53084,
                15556, 42244, 12016, 53880, 31292, 18304, 58636, 9084,  20136,
                57288, 2508,  26716, 18204, 58836, 6980,  51424, 16456, 48148,
                61392, 53952, 17196, 62800, 14000, 45512, 23864, 17056, 29768,
                55360, 9280,  33656, 61384, 7184,  10512, 31268, 336,   14772,
                59976, 7400,  17940, 49452, 60820, 46060, 62648, 41948, 11880,
                16868, 51552, 61324, 33508, 2748,  27880, 21788, 59444, 33840,
                276,   36640, 5900,  48932, 56700, 3036,  60552, 34176, 47824,
                42852, 64208, 3580,  39452, 17660, 42584, 63608, 51364, 30320,
                7844,  45344, 33056, 20372, 39356, 8872,  49264, 11616, 60908,
                6356,  16964,
            },
            {
                31224, 26120, 14560, 52384, 21096, 9208,  34344, 56804, 25484,
                45944, 12780, 36664, 30652, 51912, 8168,  26180, 17956, 65472,
                52780, 20036, 180,   55804, 25084, 5216,  18892, 27944, 43176,
                2972,  61580, 27460, 4604,  35260, 100,   31152, 18872, 26952,
                42212, 48136, 52212, 36224, 5512,  57920, 17668, 23980, 48664,
                60560, 27380, 58016, 37936, 5204,  13528, 52500, 35780, 29508,
                50588, 15128, 46732, 11188, 39564, 24728, 65020, 37224, 3440,
                12168, 34380, 21544, 28620, 48852, 3952,  35288, 60348, 572,
                37480, 63824, 16544, 44328, 40244, 54940, 48112, 58228, 26336,
                29964, 33964, 39856, 13300, 23264, 9056,  18556, 32572, 28668,
                55352, 36888, 26284, 10572, 49156, 65208, 13384, 8844,  41180,
                16372, 63700, 28588, 40620, 17232, 43388, 13812, 53552, 39008,
                8548,  51728, 29684, 12064, 56188, 26380, 9900,  22296, 47356,
                18476, 65232, 24140, 51816, 29568, 57084, 1648,  23596, 45492,
                37384, 2392,
            },
            {
                58332, 35048, 55740, 6616,  60020, 29968, 64036, 16004, 39200,
                58368, 9600,  23840, 59544, 15036, 34032, 56712, 45516, 28708,
                39252, 13640, 33928, 41136, 63312, 45428, 38476, 49900, 7636,
                14936, 53528, 37732, 58828, 23312, 48656, 43288, 57560, 36564,
                9304,  29116, 13936, 1532,  63248, 41884, 11760, 38584, 34456,
                4484,  44832, 8292,  25208, 49408, 22292, 45660, 1260,  64332,
                6260,  41988, 62060, 53148, 20856, 32516, 13836, 55248, 26684,
                46360, 42048, 6492,  59044, 37876, 9516,  26760, 51504, 41208,
                12116, 22420, 49388, 28656, 13768, 24484, 35136, 5192,  51044,
                11540, 65136, 2004,  52424, 27184, 35636, 56496, 50884, 6636,
                43364, 3876,  14520, 57884, 18836, 35856, 45808, 56640, 26668,
                50556, 54384, 21848, 9752,  61988, 23924, 30904, 20588, 27756,
                4684,  57872, 21052, 45844, 6892,  49928, 33996, 14980, 54316,
                36620, 876,   41400, 5608,  62608, 36484, 18736, 53964, 28112,
                50752, 20572,
            },
            {
                43736, 4436,  39936, 24044, 49224, 12216, 42968, 1012,  20812,
                50188, 28732, 43948, 4776,  47352, 61920, 11552, 2012,  50344,
                9196,  23460, 61244, 7364,  15860, 11712, 22432, 54772, 64752,
                24688, 19708, 46380, 12640, 17188, 7552,  65468, 13588, 4856,
                55028, 62072, 23108, 45016, 33620, 26496, 55132, 2860,  62812,
                21940, 18024, 32520, 64756, 3376,  34188, 59700, 26912, 39908,
                23276, 32188, 8344,  28868, 3244,  49368, 43816, 9124,  30132,
                57164, 452,   24680, 52068, 31672, 55504, 20692, 15048, 46896,
                31888, 53360, 5996,  58832, 1628,  63148, 20920, 17048, 40916,
                46648, 22604, 55092, 44004, 4984,  40976, 14828, 24880, 64744,
                20200, 60200, 47088, 30976, 41424, 53044, 2132,  31704, 20100,
                11628, 44408, 3460,  34892, 51908, 47104, 660,   56308, 62544,
                16828, 35696, 25064, 63136, 31288, 38284, 61668, 3320,  43280,
                12584, 27200, 56156, 11040, 47152, 13820, 43208, 32672, 8060,
                60108, 12008,
            },
            {
                48460, 17284, 63460, 32464, 45292, 18540, 26992, 53436, 35668,
                6920,  62928, 18276, 55256, 21672, 37708, 24716, 41864, 16792,
                57336, 47988, 27104, 52056, 31476, 58904, 3252,  29248, 33792,
                40496, 9160,  30720, 41676, 56356, 28696, 38992, 24788, 45724,
                32564, 19216, 40024, 54088, 16060, 8668,  46192, 29084, 13760,
                57184, 51956, 39772, 15292, 54812, 42312, 9640,  16972, 12600,
                55552, 19260, 45476, 58120, 35356, 62776, 4904,  22516, 60232,
                15824, 39352, 63540, 12968, 17828, 43860, 64372, 7040,  57408,
                3152,  25324, 19472, 33380, 38564, 51776, 43700, 9472,  56016,
                36900, 7016,  19040, 29452, 63120, 58804, 1228,  48332, 9748,
                34548, 39392, 22308, 8016,  24696, 5252,  17024, 39308, 62128,
                6924,  37492, 29556, 14500, 58548, 7672,  33668, 11864, 49628,
                44720, 9944,  41192, 14068, 1656,  19132, 24048, 57584, 29252,
                59996, 48712, 17468, 32268, 22388, 26708, 3828,  64568, 15708,
                40712, 23032,
            },
            {
                52648, 27604, 10580, 1528,  58000, 37740, 4296,  61360, 46580,
                14356, 33460, 40216, 1604,  30176, 7324,  53512, 32140, 63424,
                36164, 4092,  43460, 19348, 35552, 47228, 41944, 17272, 59988,
                1884,  52312, 62344, 5356,  50640, 33700, 20048, 53120, 2816,
                59868, 11816, 7176,  49052, 30788, 59184, 19936, 50952, 42724,
                6284,  47568, 10656, 28448, 61136, 19768, 30712, 53252, 48232,
                60616, 36460, 36,    14648, 25772, 17724, 41072, 52712, 32980,
                19780, 50400, 34804, 8204,  47472, 1548,  39980, 34584, 28340,
                61608, 42428, 46128, 55292, 11108, 30504, 3576,  62124, 31780,
                25788, 60688, 15388, 33300, 11280, 21412, 38004, 30556, 44988,
                13524, 2644,  50668, 54860, 62964, 28704, 58516, 49936, 23440,
                46212, 55996, 64872, 24896, 39184, 19568, 42084, 25832, 37856,
                2776,  31944, 54788, 59724, 47404, 53028, 9264,  39656, 6440,
                21080, 34416, 8540,  52396, 61260, 39616, 55500, 48168, 35448,
                64,    30480,
            },
            {
                7464,  37192, 54884, 14876, 22448, 51740, 8996,  31296, 24016,
                54636, 10772, 25648, 51112, 64748, 44552, 14492, 20128, 5928,
                29676, 12832, 59624, 1440,  10060, 25404, 56732, 5976,  13736,
                48400, 36284, 21168, 25460, 1264,  60956, 9436,  15788, 41608,
                50224, 26228, 37188, 3880,  22316, 64852, 1136,  37060, 24996,
                30936, 35876, 484,   23348, 46492, 35472, 2604,  44080, 5468,
                27660, 10268, 50064, 39152, 54904, 11384, 47760, 36708, 6740,
                10936, 44844, 28148, 21352, 58204, 30392, 23696, 10688, 17580,
                37660, 8620,  14584, 64224, 18176, 26956, 48988, 22120, 13648,
                764,   51668, 39016, 49584, 45440, 54336, 16804, 52136, 61732,
                27868, 57456, 32596, 16104, 11780, 43636, 35336, 9888,  14892,
                33180, 1664,  17836, 48976, 4632,  45468, 63228, 15520, 58976,
                21580, 51108, 17720, 5768,  27936, 42988, 32776, 16400, 50912,
                45108, 62996, 38168, 2236,  44096, 6280,  20508, 10320, 25424,
                56552, 62500,
            },
            {
                45912, 20292, 42716, 34556, 28572, 65104, 40492, 16760, 59372,
                5200,  42532, 57680, 16300, 35432, 9696,  49200, 55696, 38632,
                46520, 22172, 53660, 40292, 64416, 49780, 21452, 37872, 44364,
                27120, 10484, 55240, 15228, 44784, 37032, 47644, 57972, 21844,
                29928, 63552, 43896, 56148, 34740, 14580, 41040, 11164, 60176,
                18816, 63164, 53132, 58488, 13680, 7856,  38412, 62584, 24448,
                33832, 41688, 64628, 22824, 31204, 60696, 1588,  27148, 64768,
                55940, 2388,  61056, 41432, 4540,  52480, 62504, 48452, 54432,
                21676, 50876, 4388,  36080, 41172, 7212,  57644, 45524, 34788,
                58660, 42772, 9800,  2692,  26432, 5452,  35144, 7504,  24056,
                18588, 42160, 6204,  37496, 46352, 20700, 912,   53932, 27620,
                61216, 41304, 12220, 32060, 55060, 10592, 30156, 52868, 6668,
                28872, 64280, 39400, 22964, 35420, 12148, 55620, 64428, 26056,
                4568,  14392, 24480, 57396, 28840, 16668, 31564, 51992, 44576,
                18348, 12980,
            },
            {
                59104, 3284,  61196, 49728, 6436,  47068, 12632, 44244, 20396,
                36868, 29324, 47956, 3676,  21148, 27040, 59388, 656,   25280,
                62296, 17192, 33116, 28316, 15416, 7220,  31040, 54264, 63216,
                18012, 32908, 42204, 64228, 30960, 12048, 27396, 5700,  34236,
                724,   14080, 18112, 9756,  51748, 28236, 46628, 53700, 7524,
                15092, 43808, 5288,  40716, 26736, 50832, 55676, 21296, 14952,
                57548, 18468, 4516,  52200, 7784,  20084, 44664, 15592, 23296,
                38420, 14100, 48952, 24912, 37040, 16288, 12776, 43416, 2108,
                58948, 32124, 28824, 59932, 23636, 53036, 16192, 38436, 5764,
                19432, 28828, 63664, 23208, 56948, 60468, 42636, 11888, 55416,
                116,   63764, 48624, 25692, 59080, 30868, 64272, 39696, 48004,
                6004,  21456, 52288, 26596, 60496, 22304, 1244,  36508, 48244,
                13472, 9216,  46108, 736,   62196, 49268, 20760, 2184,  31192,
                41428, 54164, 19640, 11548, 49800, 59900, 36908, 63636, 6756,
                39124, 33004,
            },
            {
                23452, 26728, 9676,  17600, 56848, 2328,  25156, 33264, 49916,
                224,   63968, 12012, 39028, 61212, 42892, 33360, 12376, 41052,
                8444,  57076, 4800,  45644, 37356, 60600, 12156, 956,   23824,
                51288, 3476,  58104, 6832,  22748, 49948, 40804, 62220, 54140,
                38840, 46112, 58468, 25124, 39636, 6484,  57544, 22772, 33960,
                49812, 28012, 21504, 32692, 17332, 65284, 11176, 29708, 49312,
                1972,  45932, 13096, 28040, 42676, 34432, 58484, 50720, 29264,
                53572, 32196, 18772, 9732,  56576, 32640, 27340, 6296,  39660,
                25576, 45132, 10092, 47696, 1400,  32936, 11728, 64992, 25000,
                53688, 47008, 17088, 32668, 37120, 19172, 28292, 46400, 31824,
                39560, 21424, 10416, 52536, 3904,  14368, 8748,  24876, 18732,
                36188, 59248, 43884, 8212,  38788, 33512, 16892, 55872, 24624,
                43216, 34192, 57780, 26212, 15984, 7720,  38008, 46620, 10792,
                59496, 36060, 47496, 33028, 41036, 980,   13912, 21868, 28456,
                4204,  50044,
            },
            {
                53380, 41524, 31936, 38836, 21636, 29856, 54232, 60836, 7572,
                52880, 23580, 18900, 54988, 30840, 5624,  52392, 18160, 48524,
                35480, 51140, 11564, 23396, 52932, 19124, 41452, 46412, 35304,
                8800,  28744, 38236, 18912, 52804, 2736,  16244, 24608, 19492,
                8136,  50544, 32756, 3216,  62748, 19452, 31872, 2060,  64284,
                39196, 55364, 9988,  48600, 1816,  44852, 34732, 6564,  39212,
                60156, 31776, 55120, 37780, 63156, 5076,  12324, 39648, 8504,
                3812,  43268, 62884, 5528,  45900, 60328, 20772, 35384, 64960,
                14108, 17172, 56300, 20176, 62012, 43464, 50024, 30340, 2832,
                40524, 11444, 7284,  49068, 13724, 3400,  65316, 50308, 15288,
                59604, 36272, 28996, 17424, 34160, 55228, 42956, 50804, 3432,
                55752, 29312, 15976, 2352,  50896, 46900, 64700, 40568, 3496,
                61288, 18488, 30880, 53404, 41664, 56600, 24084, 61040, 18032,
                27708, 7892,  4100,  64984, 26184, 9284,  55984, 48544, 42536,
                57684, 15812,
            },
            {
                1052,  64548, 13492, 45088, 62840, 35452, 10380, 39720, 14020,
                28208, 45672, 34816, 8164,  47196, 15076, 23008, 64460, 30140,
                2820,  26636, 42948, 59220, 3388,  33744, 27524, 56628, 16048,
                61736, 48668, 13456, 45324, 59820, 35672, 55876, 43744, 11604,
                65120, 28916, 21368, 12744, 43120, 49492, 16088, 45064, 25444,
                12956, 3976,  61936, 36864, 57148, 24000, 19076, 53608, 43008,
                9880,  25588, 19592, 48760, 16032, 24684, 47436, 56204, 21032,
                59192, 35560, 26612, 51720, 38744, 120,   49744, 55140, 7996,
                51500, 34140, 3968,  38828, 13180, 27528, 7836,  56816, 21432,
                35476, 55812, 61788, 41996, 53112, 38900, 9192,  25164, 4180,
                53840, 7780,  45632, 61300, 40612, 21840, 62308, 32256, 12936,
                46152, 10140, 34292, 62236, 19908, 12716, 7380,  28052, 21076,
                50820, 6388,  11764, 36840, 3680,  29832, 13564, 34056, 50576,
                44512, 57712, 22844, 15776, 53620, 45004, 33716, 25016, 61952,
                11348, 35516,
            },
            {
                28932, 47380, 7540,  51944, 4320,  16252, 48340, 19924, 57720,
                41840, 16512, 62368, 25232, 57848, 1904,  37172, 43788, 58500,
                20848, 14360, 63360, 30672, 9096,  48984, 65260, 5056,  21200,
                40040, 54972, 25040, 33288, 9908,  28244, 7396,  31684, 48072,
                36588, 4996,  53596, 59824, 34328, 27552, 10500, 58992, 36288,
                20308, 46324, 30328, 15960, 41412, 4792,  61012, 27540, 15568,
                64172, 36016, 3364,  8268,  59784, 33372, 516,   31052, 17520,
                45544, 11912, 22036, 15024, 30080, 10868, 18128, 44308, 28332,
                22512, 59384, 42100, 54144, 23240, 36552, 17904, 61088, 45780,
                14848, 31464, 25560, 872,   29628, 22220, 56508, 34656, 19704,
                43036, 13392, 24180, 1932,  49252, 6760,  26856, 16880, 38652,
                65528, 22884, 42004, 56704, 25052, 31272, 44760, 57524, 15068,
                38140, 47204, 60120, 21736, 51292, 65532, 43140, 5912,  260,
                38868, 12940, 51440, 29648, 37288, 20148, 3620,  17496, 8028,
                38260, 20844,
            },
            {
                59872, 18172, 24552, 57812, 27432, 37684, 1376,  61868, 31668,
                6036,  2728,  50728, 10988, 39992, 53432, 27588, 7200,  10156,
                54696, 40544, 47648, 18028, 38944, 14012, 25248, 44164, 11472,
                31352, 6124,  376,   64636, 20596, 39736, 61332, 1432,  57504,
                26068, 15296, 45156, 37812, 136,   55848, 5708,  51324, 41812,
                8008,  58048, 52620, 26176, 11952, 50160, 31280, 46204, 664,
                22584, 51040, 56532, 45092, 21908, 52404, 41984, 27328, 61052,
                50204, 1840,  64260, 42480, 56232, 24188, 62064, 40684, 12348,
                1488,  46708, 30512, 6096,  63948, 48092, 292,   41480, 10004,
                51332, 5120,  20240, 58884, 44952, 16016, 62668, 40876, 30100,
                49656, 64392, 32872, 58184, 36876, 11448, 45080, 57612, 596,
                51780, 27752, 6516,  48024, 4488,  54220, 35800, 10392, 62560,
                32492, 2296,  24988, 44804, 9504,  18820, 27300, 55224, 21568,
                63468, 32176, 42412, 1844,  61588, 57352, 40000, 52124, 46496,
                30996, 54656,
            },
            {
                5724,  39896, 11172, 33176, 42592, 55448, 22716, 44044, 25836,
                54792, 37416, 29664, 44256, 18516, 32656, 61144, 46428, 16864,
                36616, 32372, 208,   22452, 57916, 54120, 1908,  51172, 36776,
                58208, 46984, 17548, 42272, 49540, 14944, 51356, 22664, 18036,
                41292, 61044, 9564,  23852, 17316, 40092, 22136, 62896, 28424,
                18192, 33680, 932,   21748, 64088, 8560,  38356, 12944, 57480,
                33168, 40648, 11544, 30000, 38608, 13312, 65352, 9636,  5656,
                37536, 31828, 53960, 7564,  34600, 48044, 5776,  31600, 36748,
                63288, 18512, 9740,  26520, 15684, 33324, 52848, 28864, 24052,
                39052, 64248, 48384, 36340, 11940, 6488,  47504, 1672,  10932,
                55732, 5588,  27612, 15412, 19480, 54352, 30344, 5696,  35484,
                20264, 60196, 14508, 37636, 17764, 41104, 156,   23284, 52708,
                42704, 28764, 56248, 15688, 35064, 39552, 58852, 47416, 15916,
                25940, 8592,  18884, 46964, 11812, 7412,  27608, 13364, 63324,
                2932,  44084,
            },
            {
                34376, 51092, 62404, 20248, 14144, 7092,  50120, 9964,  34604,
                13208, 63580, 22076, 58808, 5148,  13572, 21260, 50260, 24976,
                4536,  62516, 51796, 7456,  35700, 29340, 41952, 19328, 60724,
                23568, 28420, 53040, 36148, 4900,  26836, 44640, 12492, 33544,
                52432, 3560,  30964, 50360, 64532, 46280, 31540, 14196, 2704,
                60992, 49308, 39860, 44444, 55048, 34904, 59276, 20352, 48720,
                7116,  17096, 27132, 62732, 5000,  18452, 35664, 54752, 16240,
                23668, 40788, 18948, 27616, 60004, 2968,  16756, 57016, 25144,
                49004, 53228, 38340, 60324, 44336, 11568, 57388, 4328,  59616,
                33872, 17016, 8764,  27420, 52560, 33032, 60504, 26632, 16904,
                35752, 21700, 47060, 52060, 8312,  39612, 60712, 24808, 49548,
                40824, 9288,  53156, 29872, 57840, 63212, 26852, 49588, 7876,
                19316, 12308, 63620, 49256, 1536,  7044,  32344, 11208, 52964,
                37056, 60784, 54992, 34920, 24260, 49096, 32252, 58648, 22180,
                26076, 15252,
            },
            {
                56680, 28068, 356,   48236, 59168, 29596, 65420, 18316, 52528,
                46136, 17068, 7676,  48920, 35328, 64656, 1048,  41672, 29800,
                56256, 13032, 27212, 45772, 11116, 63760, 15760, 32836, 9316,
                6560,  14180, 63244, 11088, 32168, 57876, 62612, 37264, 6872,
                46668, 20676, 56604, 27076, 7104,  11828, 54360, 35172, 43824,
                24088, 10992, 15372, 6376,  28636, 17320, 2484,  41716, 25432,
                61660, 53332, 43952, 1448,  58604, 47080, 25752, 43204, 49276,
                58272, 10548, 47108, 13664, 37988, 21356, 52420, 45116, 7432,
                14676, 33868, 20836, 2020,  50788, 22300, 37904, 19064, 13344,
                44156, 1980,  56316, 42328, 19008, 38668, 23628, 50824, 44376,
                62016, 38532, 848,   42144, 63740, 22588, 47384, 12536, 32160,
                17584, 43980, 2828,  34052, 21692, 13112, 46208, 16304, 60800,
                35348, 4908,  40348, 30848, 54700, 61388, 23528, 41828, 2716,
                45176, 5780,  28524, 15160, 64256, 43520, 4832,  36180, 41644,
                50200, 9672,
            },
            {
                38084, 22908, 44908, 4924,  36144, 40700, 24712, 4208,  39188,
                1700,  56824, 42016, 24332, 28540, 39096, 54380, 11336, 59952,
                44780, 19804, 41116, 59260, 24672, 48068, 4688,  56392, 49772,
                40324, 44292, 21100, 55408, 24176, 2880,  9368,  54292, 28716,
                63828, 38948, 14016, 43368, 36432, 19572, 48292, 4596,  56988,
                38136, 53256, 31984, 59792, 47476, 23552, 52472, 30508, 4592,
                36732, 14156, 23196, 34360, 50480, 20728, 7064,  32848, 2576,
                29564, 62532, 3780,  51600, 43592, 32512, 65140, 11600, 29600,
                60920, 4564,  42804, 55928, 31000, 7752,  65376, 26008, 47240,
                53604, 29464, 23224, 63336, 3028,  58136, 14228, 4020,  30944,
                9052,  58980, 12884, 29552, 33820, 4956,  16296, 3220,  62336,
                55512, 26248, 64008, 11032, 50532, 6868,  32728, 38756, 27776,
                58348, 48040, 22800, 17952, 26788, 13328, 50088, 19188, 29780,
                59428, 21852, 40728, 2976,  51592, 20632, 16608, 54476, 1808,
                19264, 64884,
            },
            {
                7204,  17032, 55004, 32096, 10628, 15692, 47248, 58540, 31196,
                27032, 33292, 10128, 51888, 3236,  46736, 16268, 23300, 34460,
                6352,  38044, 16500, 33488, 2532,  21320, 38512, 26356, 62208,
                30280, 34796, 1256,  46156, 38600, 18984, 42964, 15988, 24544,
                1124,  10368, 33080, 61380, 1864,  59232, 25272, 29524, 16764,
                8232,  65144, 26588, 3872,  37328, 13400, 63600, 43488, 10780,
                55356, 46720, 8384,  30380, 39960, 11148, 57316, 63840, 14088,
                21404, 35776, 25228, 56008, 9168,  26784, 908,   41640, 22488,
                39068, 58040, 10288, 27276, 16724, 48884, 41896, 31976, 5684,
                60944, 40148, 12384, 49124, 31812, 9928,  54628, 41508, 20520,
                53524, 25564, 18460, 56436, 44596, 26964, 58316, 37948, 51468,
                7576,  22672, 46508, 39208, 59188, 42616, 3556,  55300, 20688,
                2512,  42184, 52944, 8564,  38404, 46036, 35820, 62880, 8868,
                34256, 47784, 11068, 57912, 37824, 8652,  30124, 60252, 11672,
                47624, 31084,
            },
            {
                43312, 12868, 62916, 25376, 52188, 60960, 21620, 8588,  54496,
                19632, 63044, 15000, 60460, 19164, 58192, 31708, 8716,  52772,
                3068,  65124, 50228, 9068,  55660, 52892, 43796, 12416, 18108,
                5284,  59040, 16540, 50492, 60892, 31228, 48264, 34512, 59124,
                49708, 55392, 17108, 22880, 52228, 40636, 9952,  62108, 50312,
                22360, 45924, 18728, 41328, 55832, 8116,  33796, 49472, 19540,
                27788, 64920, 17928, 60308, 54068, 15740, 28488, 37776, 52072,
                46100, 41448, 6076,  16152, 61556, 19508, 35008, 55072, 49820,
                17876, 24676, 46084, 62932, 35992, 3884,  14388, 51348, 10304,
                20948, 34948, 16368, 6948,  45356, 36696, 27216, 65156, 7068,
                45876, 35028, 3740,  49048, 9520,  53204, 20636, 42744, 28288,
                34808, 15148, 800,   28748, 18484, 24756, 65168, 10224, 44772,
                14988, 31956, 11656, 64572, 5368,  58240, 1092,  15260, 52148,
                24592, 17556, 54060, 32460, 25884, 62868, 45960, 39256, 27376,
                34852, 58512,
            },
            {
                4472,  39424, 49688, 19168, 42324, 1944,  34064, 44648, 12712,
                49400, 40264, 36196, 43760, 6956,  26416, 37512, 62040, 48696,
                27892, 22540, 31016, 60972, 14472, 29112, 35316, 1824,  46376,
                52076, 25260, 10532, 28776, 7352,  13152, 53364, 4824,  19948,
                40484, 44868, 27512, 5808,  46028, 31372, 14712, 36952, 42208,
                796,   34540, 12000, 50692, 21704, 60336, 16468, 112,   58472,
                32576, 38888, 2208,  42572, 4668,  22804, 44200, 748,   8988,
                18060, 58608, 30648, 50056, 39328, 47532, 13904, 6936,  63340,
                3336,  32792, 13116, 53732, 40356, 23376, 61540, 57052, 36856,
                536,   55656, 25796, 51876, 59768, 22144, 17276, 48324, 32376,
                15496, 39240, 63304, 23020, 14284, 35960, 64888, 1960,  11528,
                48736, 61284, 57036, 36492, 54044, 14400, 30312, 51968, 36964,
                62388, 24916, 55540, 34464, 43048, 20460, 28092, 56480, 39800,
                3980,  64964, 42848, 14164, 1600,  50052, 22836, 5276,  14880,
                51784, 24264,
            },
            {
                46644, 29056, 9016,  57596, 27668, 6840,  38296, 64308, 25508,
                5212,  80,    22216, 30496, 12440, 55836, 45304, 14168, 41500,
                18332, 12080, 46144, 42280, 7080,  19388, 57540, 64368, 22560,
                37720, 56152, 42436, 65436, 22116, 40864, 26764, 63260, 11612,
                30456, 7792,  65044, 35080, 57668, 3736,  20912, 54728, 6804,
                58324, 28296, 62564, 2768,  30172, 45352, 26932, 40416, 23764,
                5952,  13132, 51520, 26028, 35392, 61480, 48880, 56452, 26608,
                33124, 63404, 11956, 23472, 2144,  59464, 31172, 45388, 37176,
                28436, 52000, 7568,  20256, 1792,  29792, 44716, 18664, 28056,
                46632, 61980, 42900, 30420, 2488,  40816, 12088, 57132, 1880,
                50900, 10980, 59708, 41388, 32072, 6120,  46300, 18048, 31360,
                41164, 20860, 9044,  43856, 5604,  47952, 34728, 22920, 7164,
                49632, 612,   29060, 16412, 51048, 23964, 32384, 44612, 12312,
                49240, 30572, 7120,  36584, 19528, 43804, 10908, 56064, 61776,
                716,   20376,
            },
            {
                64048, 33412, 2564,  36556, 14768, 48000, 56548, 16992, 30024,
                53248, 57788, 47096, 62440, 50940, 4664,  20876, 900,   35644,
                54804, 58224, 2140,  33988, 25844, 49164, 40828, 10080, 31436,
                6424,  14048, 33320, 3856,  36700, 57520, 232,   45992, 23412,
                37864, 15072, 51232, 19080, 12056, 49296, 63292, 25780, 32556,
                47536, 14312, 38572, 54436, 6264,  36276, 10384, 63200, 53600,
                43956, 56568, 21188, 47844, 10540, 31048, 19268, 40564, 13256,
                53656, 3588,  43984, 36200, 28044, 53244, 21880, 9968,  16900,
                57708, 42404, 47424, 34848, 60060, 55088, 6528,  12040, 39416,
                23776, 8420,  13852, 19320, 63532, 6172,  29144, 35600, 62168,
                20904, 26516, 29896, 172,   50104, 57904, 25592, 53760, 59316,
                4048,  50708, 26940, 32192, 12260, 61768, 2940,  17408, 59844,
                39536, 19796, 46744, 60584, 4380,  10612, 63520, 7440,  37260,
                26696, 20696, 61424, 55616, 28428, 59512, 33776, 40972, 17900,
                37452, 54260,
            },
            {
                14032, 59064, 22340, 53068, 62000, 31444, 23308, 9136,  41444,
                35252, 14608, 27392, 9348,  33908, 39288, 25024, 64204, 29944,
                6536,  24452, 38928, 52008, 63388, 4336,  15832, 27568, 47836,
                61132, 20484, 45272, 54492, 18440, 49832, 15712, 35528, 59972,
                54520, 3052,  42728, 24376, 38636, 28848, 43944, 17060, 10972,
                52380, 20052, 24604, 42888, 17440, 57956, 47884, 14864, 34624,
                8500,  28604, 37504, 58084, 15180, 7592,  64672, 4980,  36728,
                20952, 47120, 8276,  57276, 18272, 4620,  42868, 62148, 26300,
                72,    14632, 64940, 9804,  25892, 15780, 33632, 49288, 64608,
                4064,  53352, 34100, 49592, 37964, 56108, 46092, 24504, 42076,
                4868,  54720, 44484, 16508, 19892, 38656, 8600,  13396, 23148,
                37168, 15996, 64480, 52672, 19200, 56528, 41504, 46160, 28392,
                12912, 57772, 8456,  36632, 41124, 54736, 47552, 17064, 59884,
                53476, 76,    46076, 9336,  51120, 16336, 6524,  24932, 48808,
                32008, 10120,
            },
            {
                26412, 40084, 45640, 18432, 11344, 43444, 1268,  51892, 20512,
                60292, 6244,  18624, 42496, 54136, 15796, 59332, 49836, 10420,
                43596, 61552, 17248, 9204,  21648, 36704, 59860, 54168, 1156,
                39368, 51088, 26480, 8636,  30968, 43264, 10180, 28484, 6180,
                47876, 32028, 61308, 55896, 588,   8492,  59732, 40220, 2440,
                64056, 44632, 8712,  61184, 32100, 22500, 51880, 2084,  20556,
                46756, 17308, 63728, 2936,  44884, 52992, 34124, 24156, 51124,
                60508, 29528, 15724, 40408, 64836, 13140, 50316, 33764, 55692,
                40616, 23124, 30436, 38228, 50140, 43148, 22140, 57512, 30272,
                17524, 41384, 59096, 10708, 23064, 15204, 53232, 9112,  17848,
                58388, 37116, 7972,  52488, 62680, 28356, 43648, 34048, 62272,
                45420, 7272,  40328, 23916, 1036,  37944, 25696, 9832,  53956,
                32940, 43272, 25876, 14516, 30336, 19340, 34756, 3200,  23168,
                13628, 41708, 34300, 24076, 38820, 2256,  45592, 65388, 28804,
                7368,  43540,
            },
            {
                17208, 6196,  50632, 3760,  26024, 39020, 63188, 48744, 12240,
                45144, 38120, 65240, 23232, 3160,  29048, 45804, 19084, 33272,
                53340, 14340, 47816, 32056, 56632, 12720, 43972, 23928, 33664,
                11100, 16584, 63300, 2952,  58872, 24472, 62348, 51852, 21132,
                17080, 9344,  27196, 13612, 47344, 33772, 53212, 23012, 36160,
                29532, 5088,  34996, 50008, 12212, 4036,  26428, 38776, 30548,
                61236, 5308,  33120, 25180, 41132, 28836, 58800, 16496, 42916,
                1752,  25728, 55024, 32716, 47904, 24936, 37952, 7660,  19844,
                48500, 5664,  59300, 18492, 4244,  53100, 1040,  10188, 36268,
                47724, 26624, 2228,  32012, 44528, 27644, 1160,  50420, 31160,
                13064, 47688, 33396, 23944, 11860, 3848,  56228, 48604, 1732,
                29828, 11260, 58168, 35240, 49500, 30584, 14800, 63788, 5968,
                20804, 3752,  65036, 49784, 1388,  61220, 27576, 43608, 50684,
                31524, 58312, 18016, 11756, 62720, 31064, 53080, 12840, 21576,
                51244, 56664,
            },
            {
                63484, 35604, 30628, 61116, 54560, 33980, 4896,  27732, 32968,
                56216, 2348,  31164, 47428, 57384, 12892, 7160,  40664, 2472,
                22480, 37388, 27488, 216,   45816, 29604, 18456, 7132,  41520,
                57900, 29796, 37044, 46748, 13776, 40340, 2204,  33736, 38472,
                57808, 41436, 22272, 36724, 62984, 20284, 14748, 6548,  58520,
                48348, 56076, 16228, 28100, 41160, 65380, 54816, 43704, 58736,
                10212, 51328, 12932, 49448, 18352, 420,   11908, 46540, 8676,
                38780, 62628, 11224, 5868,  20756, 1300,  30220, 59776, 11884,
                52684, 32468, 44904, 13176, 63232, 28820, 40316, 60628, 14816,
                6688,  62620, 19996, 56740, 4852,  64692, 34968, 60140, 38932,
                22188, 64148, 2420,  59596, 36296, 41604, 16168, 20280, 26592,
                54744, 17936, 47016, 5296,  21212, 58984, 43668, 51276, 35988,
                48464, 55936, 38520, 22352, 52596, 12028, 57080, 38320, 8840,
                64668, 6188,  27648, 48436, 57364, 20368, 42180, 4460,  60452,
                38432, 1332,
            },
            {
                23500, 12124, 42296, 8476,  20072, 13560, 58596, 16220, 22040,
                8512,  25552, 50600, 10336, 34412, 37696, 61028, 26468, 56284,
                63888, 5592,  50948, 60264, 40476, 3908,  52460, 48520, 62672,
                20056, 5184,  23356, 53636, 32628, 20324, 55708, 44960, 12532,
                64312, 1708,  53648, 45284, 5036,  30240, 49920, 26672, 41564,
                12556, 21688, 496,   59892, 36744, 18576, 7436,  14176, 24064,
                35564, 42260, 55192, 22192, 38444, 62832, 54296, 21556, 31752,
                49520, 19460, 35732, 52520, 44920, 61536, 54568, 15752, 39684,
                2616,  26808, 56764, 8768,  35156, 24848, 19760, 45708, 54452,
                24032, 42792, 37628, 48608, 16684, 41008, 20412, 11180, 5280,
                42960, 27480, 15432, 45968, 30072, 51524, 65128, 7816,  38800,
                61120, 33500, 41976, 13712, 63140, 9020,  3360,  18248, 24028,
                29332, 10728, 16776, 34028, 44948, 5752,  24772, 15620, 20524,
                46752, 40464, 54096, 2876,  36800, 8608,  26644, 35024, 15056,
                32608, 47804,
            },
            {
                53900, 28232, 49228, 57816, 24476, 37148, 47044, 41144, 63880,
                53536, 43576, 17184, 61876, 21208, 52144, 16284, 48476, 30980,
                12568, 42980, 18180, 24296, 11484, 64968, 35268, 26840, 13720,
                55724, 44184, 50068, 10376, 64844, 7252,  17632, 26232, 5408,
                29612, 50644, 32832, 18140, 11688, 56432, 37820, 64972, 17832,
                32200, 45236, 25468, 51204, 10332, 46556, 33408, 48876, 996,
                19596, 27968, 3764,  59308, 30608, 7084,  36036, 26328, 56860,
                14456, 3164,  59000, 41336, 27224, 9260,  34616, 23548, 46660,
                64184, 36836, 21744, 41584, 47332, 58164, 5728,  31396, 12076,
                33672, 51812, 8956,  28960, 12644, 54056, 46304, 25028, 56844,
                49556, 18968, 55856, 10152, 5012,  21712, 32352, 13104, 50032,
                22176, 3936,  53060, 25044, 28852, 54468, 33332, 40456, 62052,
                248,   46480, 58708, 7740,  31216, 40720, 63412, 54620, 29836,
                4196,  35212, 14412, 22648, 44776, 16648, 54800, 58592, 45980,
                9140,  18964,
            },
            {
                3288,  39280, 15492, 1576,  44880, 52296, 7112,  29224, 852,
                35536, 13196, 39816, 5024,  27984, 540,   42192, 9332,  20392,
                35352, 54392, 8128,  33428, 57152, 16040, 22424, 38204, 9276,
                31868, 768,   16112, 27520, 42472, 37640, 49600, 59836, 46924,
                15704, 23720, 8308,  40080, 60052, 3232,  43560, 9416,  2172,
                54592, 62116, 39396, 6060,  57620, 22084, 29076, 61712, 56388,
                39668, 64424, 45412, 9396,  15136, 47464, 41888, 4248,  65464,
                44092, 23724, 30692, 12824, 17376, 50236, 3888,  43392, 29032,
                18232, 6864,  14548, 54672, 1736,  16472, 51100, 38616, 64500,
                18648, 308,   59656, 22916, 61808, 32948, 7600,  29744, 62968,
                644,   32708, 52732, 37956, 25568, 57464, 46596, 488,   58532,
                43960, 10160, 31496, 48192, 17124, 37832, 45736, 11492, 56976,
                14920, 42036, 26012, 61448, 18524, 48048, 2088,  36228, 11252,
                59344, 25604, 62076, 32036, 50280, 63356, 1076,  40344, 24408,
                29864, 61892,
            },
            {
                57140, 21192, 34492, 65160, 31692, 17996, 11112, 61336, 50000,
                18932, 59788, 30668, 55244, 45260, 64252, 23816, 58840, 3484,
                46408, 62724, 28600, 39404, 49084, 5812,  53768, 3148,  46200,
                59756, 40396, 35184, 60776, 4032,  22060, 30688, 10048, 35944,
                41892, 56964, 62652, 25960, 47976, 21468, 33328, 24752, 51480,
                35900, 11240, 19756, 31368, 42696, 3456,  16080, 51548, 11296,
                5068,  16720, 25572, 33228, 52292, 61764, 19072, 10744, 33932,
                53300, 7480,  47292, 63892, 38056, 22164, 56028, 62356, 10448,
                49040, 60016, 51820, 29680, 33260, 61748, 9628,  27492, 4440,
                56128, 45172, 15516, 36252, 5456,  51176, 39876, 17204, 36676,
                13596, 41220, 6368,  44800, 63480, 16976, 40608, 34880, 27724,
                15636, 37200, 61740, 1176,  57260, 6408,  22208, 27256, 52736,
                31608, 36604, 4716,  53896, 12676, 27972, 22112, 44192, 49128,
                17680, 51988, 9532,  42336, 6580,  28672, 19740, 11668, 52196,
                5936,  43308,
            },
            {
                46876, 10404, 59392, 26872, 5864,  55116, 22944, 39120, 26304,
                4608,  48300, 23256, 8160,  14244, 36524, 31972, 50012, 38812,
                25792, 14724, 1652,  45040, 19816, 31248, 42720, 62008, 28624,
                21056, 52796, 24316, 56692, 13128, 51632, 62888, 380,   53908,
                20408, 4756,  34840, 13876, 29168, 55124, 16056, 60844, 46612,
                7340,  28880, 15356, 64448, 54244, 35148, 24840, 38112, 44024,
                30208, 48164, 37104, 57652, 1232,  23604, 29036, 50212, 39128,
                16796, 27804, 57360, 312,   32964, 6252,  40592, 14256, 32084,
                860,   25984, 39484, 11220, 23608, 43724, 20908, 48684, 40900,
                24456, 30412, 50268, 42308, 26824, 1716,  55432, 10292, 47932,
                23884, 57828, 20564, 29236, 12516, 3816,  54712, 8940,  63048,
                24240, 55372, 20156, 40724, 13088, 65484, 49984, 2368,  19392,
                8236,  64328, 20924, 50196, 38764, 57608, 8536,  60828, 31676,
                41368, 492,   20872, 57992, 13320, 38540, 33740, 48796, 64720,
                36956, 13788,
            },
            {
                25012, 4360,  50992, 42072, 13284, 35980, 44052, 56932, 32328,
                11660, 42516, 34248, 58244, 19224, 53772, 6216,  11980, 17868,
                56792, 51732, 22868, 61024, 10756, 58196, 25132, 14208, 48912,
                17420, 11200, 6412,  47212, 33232, 44028, 16224, 39708, 28152,
                11848, 49480, 43192, 1516,  51948, 6040,  39476, 12328, 20732,
                41836, 59116, 48728, 1636,  45508, 13004, 63192, 6592,  20824,
                54420, 60680, 12468, 20312, 40004, 44344, 13484, 59524, 1996,
                21400, 42864, 10488, 51408, 25332, 60316, 18896, 52700, 36360,
                57964, 19828, 45472, 64880, 3660,  37016, 59504, 13356, 35076,
                63092, 7148,  10796, 57532, 18384, 65480, 21980, 43228, 60796,
                28028, 8192,  61924, 35704, 48900, 23268, 30752, 19028, 47780,
                4964,  51080, 7884,  34596, 25556, 30348, 42484, 60424, 34284,
                48528, 24272, 44592, 1412,  16068, 33520, 51428, 3628,  14780,
                26884, 56244, 35868, 47176, 23804, 54196, 60280, 4072,  22476,
                17372, 32868,
            },
            {
                53172, 30372, 38240, 19536, 48552, 62616, 3132,  8408,  20936,
                52788, 64800, 2696,  26652, 47004, 40528, 28992, 62264, 43376,
                4704,  30892, 37236, 6744,  50664, 35152, 1288,  40012, 7468,
                36500, 64344, 30180, 2400,  19324, 25464, 7724,  58044, 23336,
                65460, 31304, 59664, 19428, 36684, 44788, 63988, 30600, 56956,
                4136,  25776, 38308, 22912, 9048,  27688, 41200, 57800, 32840,
                8760,  24340, 3124,  49932, 6348,  56040, 34932, 7728,  53988,
                31796, 62444, 35708, 15552, 42032, 46140, 28284, 8416,  23660,
                42468, 5980,  34588, 16912, 50540, 7656,  55052, 17760, 3196,
                52816, 21316, 33248, 38848, 45996, 12908, 31184, 34516, 2960,
                52984, 14972, 50572, 1592,  43108, 59516, 51832, 37716, 42356,
                13988, 29052, 44288, 59908, 47488, 8968,  16348, 38596, 13800,
                58116, 40392, 10928, 29480, 63124, 25392, 42740, 20232, 65296,
                38492, 5876,  29580, 63708, 2580,  16248, 42820, 9884,  28172,
                40936, 56304,
            },
            {
                8156,  15844, 63916, 1284,  24336, 29672, 16768, 47400, 60752,
                14624, 37600, 16996, 51624, 10172, 1392,  59412, 21436, 34272,
                9540,  65276, 41484, 13308, 28328, 17040, 63812, 54544, 32276,
                57476, 43480, 50284, 37908, 62044, 55108, 35312, 45308, 3260,
                47372, 15216, 7588,  55952, 23600, 9836,  26772, 104,   34912,
                50580, 17132, 33276, 53216, 60612, 49560, 17808, 880,   51808,
                15196, 36024, 42872, 64892, 31376, 26152, 16144, 63756, 45448,
                24812, 48780, 3516,  20288, 56276, 11804, 2148,  48376, 63100,
                54868, 12480, 47720, 28008, 57440, 31996, 25396, 46772, 29344,
                43068, 14792, 60144, 2652,  24160, 49432, 59528, 7028,  19520,
                45280, 38016, 32404, 18408, 26604, 6100,  11476, 61072, 2164,
                32600, 63688, 17756, 3076,  21136, 52208, 55584, 3756,  28140,
                5960,  17736, 54880, 35880, 47040, 6848,  12084, 35216, 54112,
                9972,  50536, 18004, 11460, 45336, 25456, 31320, 57728, 47420,
                552,   61316,
            },
            {
                36452, 22100, 45572, 9784,  59856, 53516, 34608, 40032, 27996,
                45396, 5820,  30004, 62736, 35912, 24356, 50740, 15784, 45920,
                26996, 53892, 18428, 56296, 47520, 23368, 44504, 10164, 19544,
                3564,  22268, 26576, 15236, 41492, 10700, 29156, 52176, 18044,
                38648, 34152, 27232, 41032, 61680, 17516, 47252, 53020, 14136,
                43500, 61524, 11696, 5660,  20388, 30636, 36792, 44964, 26700,
                62552, 46864, 28432, 18356, 52588, 10084, 48212, 37704, 18952,
                12424, 5908,  59740, 39180, 30008, 65048, 34060, 38388, 16076,
                30664, 2908,  61224, 22232, 9872,  41140, 1096,  38696, 62184,
                9040,  48388, 28020, 53820, 5164,  37280, 16360, 54404, 41084,
                24516, 63884, 10324, 56688, 39440, 15536, 34244, 20884, 26060,
                54104, 39088, 11204, 56808, 37100, 32316, 24004, 64004, 45964,
                51596, 31920, 60984, 4344,  22636, 56212, 60188, 30096, 47548,
                22848, 43420, 59488, 34148, 53076, 37156, 5400,  51640, 19608,
                12596, 44564,
            },
            {
                5220,  50144, 55400, 33076, 41280, 5360,  12520, 58144, 396,
                23968, 55880, 41476, 20624, 44476, 12484, 56160, 38464, 3140,
                49372, 22324, 304,   32480, 60092, 3920,  37620, 52108, 27768,
                60768, 12564, 54024, 6768,  58916, 1144,  21628, 13464, 60856,
                6324,  54184, 11024, 50520, 3056,  33352, 58432, 38068, 22676,
                8300,  29428, 46572, 39924, 57000, 13932, 4096,  59120, 11144,
                21740, 55624, 5172,  13828, 38980, 58328, 22780, 2424,  30244,
                40812, 54764, 26720, 8820,  52920, 23212, 6808,  51020, 20744,
                25644, 40028, 52016, 14696, 44532, 63416, 20116, 12740, 54212,
                23216, 36000, 17396, 64096, 44016, 30108, 9960,  26944, 58024,
                12620, 4228,  29912, 46244, 53396, 65212, 44112, 57776, 9088,
                49764, 23104, 45892, 28000, 6628,  43032, 11788, 19192, 35436,
                9576,  25904, 42112, 14240, 52344, 18880, 40128, 1252,  16272,
                4416,  26112, 39968, 7708,  21264, 61472, 14424, 39044, 63656,
                34296, 26148,
            },
            {
                17648, 39688, 28048, 13968, 19180, 26208, 51464, 21772, 10280,
                50176, 33416, 8076,  59904, 4428,  28252, 32588, 7560,  60708,
                13976, 36568, 44136, 11496, 40300, 8092,  31020, 14620, 48080,
                41228, 34104, 46052, 18536, 32452, 47608, 63268, 42760, 30732,
                25008, 58544, 21460, 45720, 30248, 12952, 6656,  19868, 65152,
                55796, 4448,  24968, 63492, 34476, 43028, 24112, 50368, 39764,
                7260,  33556, 41712, 40,    60860, 34168, 6916,  43728, 61372,
                50324, 15316, 32780, 47052, 17636, 42764, 13960, 61420, 44188,
                9376,  59164, 36576, 4192,  29540, 56200, 49912, 33612, 5584,
                59004, 1796,  41344, 12144, 20456, 51896, 62512, 204,   48108,
                36008, 51360, 21028, 7756,  23932, 856,   28384, 47316, 4340,
                15772, 34780, 108,   59456, 14612, 61572, 48444, 2044,  41044,
                62400, 20820, 432,   45684, 37220, 8924,  26972, 44560, 57764,
                62596, 31412, 13808, 55344, 988,   27900, 49748, 8628,  23084,
                30152, 58616,
            },
            {
                11236, 61984, 3624,  57492, 48856, 43968, 65080, 37048, 30896,
                61248, 18200, 13368, 53696, 48076, 16556, 64588, 41100, 19732,
                30064, 57592, 63752, 26064, 51188, 20880, 63280, 24776, 57144,
                640,   9292,  65052, 39316, 51836, 24276, 36172, 4508,  48892,
                40200, 1476,  36440, 16260, 62924, 43332, 49652, 26244, 40940,
                32664, 51320, 18768, 1452,  54172, 8808,  16688, 64616, 29268,
                19232, 61620, 51372, 24196, 45312, 19692, 28628, 57020, 9792,
                35324, 21820, 62204, 992,   37208, 58696, 4424,  31628, 55376,
                568,   46824, 33296, 18688, 24648, 6468,  37132, 15928, 26628,
                45200, 31644, 56596, 25416, 7224,  33864, 40432, 22460, 31552,
                17416, 59920, 41684, 61800, 14064, 37144, 18168, 31548, 40672,
                55728, 64356, 19748, 51300, 39644, 25732, 53584, 29952, 56652,
                15720, 49524, 58968, 29192, 64384, 32992, 50396, 12548, 34580,
                51952, 19576, 48688, 64704, 33312, 46384, 18136, 43452, 55704,
                2168,  52572,
            },
            {
                42172, 31836, 23912, 8740,  34952, 2016,  17252, 7256,  42128,
                46564, 27016, 39916, 35592, 22656, 804,   53032, 25080, 47360,
                52048, 9064,  4972,  16672, 34988, 55328, 45224, 5328,  17792,
                35968, 29848, 22788, 3300,  27736, 7976,  16840, 56920, 10264,
                19412, 64696, 51528, 7704,  24324, 35308, 54524, 3932,  10692,
                15476, 36812, 44708, 13060, 27596, 48176, 32696, 55020, 2100,
                37436, 47540, 16156, 31012, 11488, 53864, 48708, 14348, 25620,
                3608,  55932, 44048, 12200, 28964, 25076, 49392, 36012, 18104,
                27412, 12792, 53860, 65356, 48748, 11052, 43044, 61872, 52264,
                19548, 10476, 38264, 49980, 60396, 46468, 13444, 55252, 6044,
                44860, 27236, 2904,  32972, 48888, 9992,  52556, 62260, 12228,
                25088, 43164, 29384, 8572,  33584, 17572, 5140,  22284, 37712,
                6944,  33492, 12032, 23560, 4920,  16520, 60792, 24468, 6964,
                37960, 9860,  3416,  42012, 24220, 11364, 59396, 4188,  35624,
                15664, 47708,
            },
            {
                36844, 20432, 45908, 63112, 38912, 29396, 59356, 55760, 14740,
                3144,  63096, 5432,  58108, 29704, 43800, 37020, 11276, 3972,
                34480, 23832, 42200, 29120, 47728, 1856,  12488, 38752, 52768,
                62080, 50096, 15440, 58392, 54552, 12768, 44808, 51036, 33524,
                26976, 14184, 42384, 29088, 57752, 972,   17804, 60492, 28132,
                48504, 62480, 22032, 58664, 39136, 6212,  20980, 45096, 14300,
                10036, 56372, 26816, 7912,  63524, 36052, 1612,  41516, 65004,
                18592, 38516, 5736,  51696, 63768, 20416, 56972, 10840, 41756,
                62632, 22052, 7692,  40548, 31316, 58216, 22744, 372,   29108,
                47820, 62772, 3768,  15680, 28720, 2540,  18344, 38088, 64360,
                9356,  15244, 38724, 56020, 25200, 43600, 58896, 22004, 7276,
                37644, 4844,  48652, 12728, 58424, 45276, 63596, 10600, 47184,
                26860, 52844, 39412, 43696, 55852, 47680, 2452,  41216, 21644,
                56136, 46056, 29492, 16084, 37228, 53368, 30588, 40600, 25360,
                65228, 7128,
            },
            {
                59296, 124,   50440, 15380, 53924, 21536, 11248, 25704, 52040,
                34016, 23632, 11504, 50624, 19336, 7984,  57044, 61652, 18056,
                45012, 59808, 13784, 56684, 19944, 60780, 32552, 27628, 43104,
                21020, 11468, 45624, 41396, 31032, 63896, 38460, 21228, 61180,
                2800,  46420, 54876, 11152, 21048, 39832, 46856, 31592, 42844,
                56196, 2524,  9572,  30888, 51928, 63184, 36232, 25972, 59416,
                42700, 32488, 3420,  58124, 39664, 17168, 22320, 32680, 52132,
                27976, 47412, 31580, 16524, 40460, 2672,  45328, 6908,  29932,
                48276, 38048, 15456, 45044, 3488,  17508, 54368, 39532, 13624,
                7008,  41660, 34676, 23016, 53484, 42620, 57940, 25720, 49736,
                35192, 59032, 51996, 19628, 5008,  30036, 2268,  34424, 50192,
                17484, 60644, 54580, 23248, 36688, 1348,  31124, 42420, 59848,
                19004, 2536,  57888, 8272,  20332, 27876, 54040, 31140, 63704,
                14444, 26288, 60640, 57432, 6624,  20168, 62436, 9684,  50908,
                21684, 28856,
            },
            {
                17864, 56368, 10308, 27024, 4236,  45128, 32356, 48252, 19652,
                38620, 54536, 45008, 15344, 63992, 32984, 26896, 14352, 39784,
                30468, 1456,  52592, 37836, 7308,  41380, 15900, 8896,  58720,
                6208,  26532, 4288,  33696, 19700, 1240,  25316, 6660,  30056,
                58900, 37072, 5112,  32632, 61692, 52608, 8632,  13488, 20112,
                6448,  34400, 25676, 41560, 17960, 400,   12164, 49876, 4948,
                23496, 52660, 20092, 43872, 50028, 6404,  60376, 45608, 11272,
                7264,  59676, 22872, 9488,  54376, 35208, 14344, 24080, 52840,
                58492, 2136,  25428, 61036, 28248, 35896, 8220,  32892, 59944,
                24512, 55652, 17716, 65200, 8096,  32088, 10860, 20384, 29988,
                1172,  23092, 11520, 46308, 64724, 16440, 40196, 56412, 27684,
                45576, 32116, 3096,  40980, 19968, 55900, 15472, 50404, 34392,
                13496, 65324, 31024, 15104, 35020, 38416, 10252, 17776, 43408,
                200,   34972, 50076, 22700, 44400, 1444,  47560, 14572, 33904,
                44120, 12392,
            },
            {
                38740, 30988, 42836, 34452, 60304, 40516, 6240,  64108, 740,
                9440,  59636, 31308, 2516,  41540, 48628, 5676,  54832, 50816,
                22372, 62624, 10668, 25828, 49332, 64780, 23424, 53836, 34488,
                48268, 37764, 55284, 60284, 10052, 48424, 43324, 53856, 15328,
                41720, 18620, 23788, 48752, 15736, 26936, 36248, 64732, 24172,
                38728, 53704, 60788, 14824, 46168, 57156, 28756, 61128, 40840,
                15420, 65392, 34908, 12992, 29180, 25276, 54616, 4008,  37184,
                56476, 13804, 42840, 48772, 26280, 60564, 32340, 64488, 17140,
                5524,  33048, 55744, 49152, 12304, 51436, 64124, 20640, 44224,
                50488, 29820, 11904, 45692, 36456, 51212, 61992, 4992,  47660,
                54528, 41820, 31840, 6764,  36068, 51156, 13164, 20660, 63372,
                10396, 14912, 25952, 64928, 47616, 27956, 8364,  24664, 3648,
                40808, 22832, 50956, 46640, 62448, 5392,  58524, 49112, 52896,
                6340,  40140, 9092,  12692, 32852, 39388, 27580, 57936, 5424,
                61068, 54004,
            },
            {
                48432, 6552,  64300, 24232, 19032, 12856, 51032, 36240, 29072,
                42488, 17236, 26368, 36064, 21600, 10704, 37980, 20020, 3572,
                33436, 46680, 17292, 35816, 4712,  31132, 45904, 2964,  18672,
                63224, 14128, 23644, 51168, 16420, 35836, 62428, 11508, 34068,
                50084, 8972,  64160, 39112, 2572,  44672, 5564,  50596, 58028,
                44060, 11984, 48844, 4408,  23352, 34072, 8792,  18876, 31200,
                37760, 1928,  48144, 8980,  61860, 38040, 14664, 30632, 19368,
                24940, 33860, 62848, 228,   18680, 38172, 8556,  47112, 40948,
                21324, 36920, 9852,  19300, 41064, 24192, 46416, 15404, 2724,
                36948, 6028,  59540, 26344, 784,   15172, 39472, 44332, 13352,
                63628, 17200, 27536, 60748, 42924, 26192, 8152,  47960, 1004,
                42344, 53104, 35656, 6392,  11680, 52720, 37868, 62980, 56864,
                45152, 9924,  27428, 792,   41316, 25352, 12960, 33336, 20648,
                28320, 61508, 53984, 18612, 63276, 55484, 23516, 17104, 37744,
                2504,  26144,
            },
            {
                22264, 14840, 51844, 1468,  46664, 55816, 16316, 23384, 57032,
                46916, 4452,  62604, 49852, 56268, 60952, 29220, 43220, 65456,
                8728,  40924, 28264, 58972, 21440, 55736, 11340, 39228, 28724,
                43820, 480,   30492, 40680, 7448,  28280, 56728, 22248, 4584,
                26600, 58056, 29668, 12184, 55428, 60132, 18804, 30040, 696,
                16940, 32536, 27676, 63864, 37516, 43336, 53260, 47084, 6444,
                56648, 27048, 22520, 55404, 18324, 608,   47000, 41904, 64140,
                49956, 39584, 6496,  30044, 58200, 50880, 3684,  28380, 12668,
                51828, 43640, 63108, 30824, 5740,  1312,  56924, 27664, 10824,
                53916, 19664, 40404, 49220, 21500, 55772, 28472, 24020, 34324,
                8464,  37672, 56668, 2444,  22440, 53684, 59692, 31516, 38748,
                22932, 59948, 18800, 44068, 32876, 59252, 17268, 4968,  29812,
                19832, 36600, 55436, 17216, 52236, 21988, 44608, 64324, 37540,
                15616, 24128, 45192, 29620, 4464,  48968, 8120,  52204, 31452,
                46464, 9584,
            },
        },
        {
            {
                24212, 57704, 52224, 29988, 1336,  38416, 11452, 31912, 40884,
                24204, 30260, 43868, 18712, 1312,  31292, 50048, 28736, 2144,
                57208, 11140, 29668, 45008, 25112, 32296, 5744,  18308, 26468,
                1248,  55688, 13152, 19436, 52976, 57228, 10936, 38660, 29164,
                16572, 7848,  41920, 63460, 35512, 24788, 48720, 6520,  37744,
                59712, 44652, 11200, 61612, 27064, 7088,  60592, 50680, 38624,
                19004, 22868, 14232, 54440, 16940, 4604,  29928, 63760, 23012,
                39576, 18432, 65028, 26964, 48284, 23860, 16860, 12124, 41496,
                18232, 48164, 38908, 59212, 3504,  16640, 48876, 23564, 44896,
                17952, 5084,  24940, 62368, 8968,  32228, 1948,  64516, 15420,
                50580, 28252, 13068, 38236, 47960, 31992, 60624, 2572,  39420,
                62780, 32264, 4904,  23400, 64924, 10360, 2564,  40824, 55268,
                36712, 19508, 26144, 676,   17988, 31552, 27808, 43524, 37048,
                10992, 58144, 2464,  33508, 47940, 24136, 58616, 17032, 49388,
                55360, 9244,
            },
            {
                33888, 39808, 21152, 61748, 48144, 26356, 6952,  47068, 57088,
                15756, 62848, 39012, 10616, 61384, 21076, 37100, 65364, 18404,
                35140, 22120, 38856, 52632, 12944, 56584, 22536, 42572, 52108,
                36276, 44864, 4516,  40140, 30280, 15496, 34404, 50108, 2620,
                54388, 26412, 51228, 21344, 17820, 1788,  57776, 42324, 21596,
                28872, 2024,  20184, 51924, 36800, 1476,  21728, 13320, 8984,
                46776, 62764, 40288, 44912, 36472, 49536, 9348,  34332, 6632,
                56236, 560,   34812, 14356, 4856,  40048, 35688, 62636, 30716,
                508,   65320, 11144, 27172, 51348, 33892, 29604, 8424,  2284,
                41512, 30320, 14032, 50828, 40960, 60084, 12032, 39244, 54208,
                18868, 43852, 56116, 62412, 10428, 6852,  22792, 57240, 29884,
                7612,  47780, 18032, 41604, 45908, 53300, 34932, 28344, 21996,
                3864,  61752, 39508, 11216, 54144, 48120, 60516, 2072,  18988,
                64016, 30292, 6492,  21460, 52016, 8816,  3288,  31732, 40612,
                13664, 64476,
            },
            {
                18700, 46152, 5352,  9032,  41844, 53060, 60772, 18328, 28008,
                51548, 8272,  48084, 34608, 52840, 15172, 41676, 9104,  52028,
                47124, 60616, 3380,  17504, 35564, 46376, 63552, 8616,  30508,
                14656, 61736, 24904, 47864, 63372, 8,     22072, 45292, 64064,
                40572, 33452, 4236,  56676, 38796, 46864, 27688, 10288, 16636,
                49740, 32156, 63540, 16140, 41804, 48176, 31724, 43796, 28536,
                53296, 33820, 7664,  26552, 57860, 21564, 53872, 43624, 15504,
                51716, 46768, 21908, 43196, 59776, 52404, 9228,  45620, 56636,
                24224, 34748, 45204, 14600, 42420, 63136, 36944, 57132, 52588,
                64156, 35392, 7320,  54704, 19292, 28412, 48772, 23008, 35476,
                9564,  4192,  20816, 33972, 50264, 36680, 54400, 42132, 15476,
                24716, 36204, 56644, 13496, 26464, 5544,  17404, 63228, 43944,
                50604, 14108, 33356, 45024, 23892, 14720, 6264,  34328, 26108,
                49392, 15724, 38904, 43136, 18088, 56324, 37908, 45440, 6776,
                53232, 640,
            },
            {
                28616, 12000, 56832, 32912, 15388, 22280, 34804, 43660, 56,
                37284, 21644, 3068,  25544, 45096, 5768,  58844, 26836, 12868,
                30988, 6360,  41024, 62324, 9532,  28384, 2172,  38420, 54268,
                20584, 49760, 32812, 7308,  37460, 27308, 58224, 8256,  14348,
                19648, 11520, 60892, 29972, 7420,  14408, 34160, 61296, 53280,
                39340, 8376,  46356, 25056, 56276, 10252, 64612, 58120, 16660,
                740,   59700, 11872, 31916, 2328,  13760, 38668, 27240, 62460,
                31288, 10272, 38008, 55040, 3136,  32124, 20976, 27404, 6560,
                38080, 19744, 4276,  55564, 21296, 1264,  10692, 19448, 26560,
                47484, 21964, 58340, 38040, 3144,  45644, 6212,  58820, 30080,
                63472, 46556, 24300, 58496, 1636,  27136, 19920, 12080, 51624,
                60296, 4,     49764, 8864,  33408, 57792, 47636, 11788, 31236,
                8532,  27164, 58040, 5016,  63280, 38072, 41732, 57320, 45312,
                8132,  53604, 60944, 27720, 13416, 65004, 29136, 22060, 60340,
                26020, 43100,
            },
            {
                62116, 37068, 50976, 25276, 64204, 3608,  10436, 58580, 13424,
                32268, 54180, 60016, 12336, 55748, 32736, 22996, 556,   44372,
                56836, 15300, 25980, 54680, 20752, 48500, 59484, 17024, 41564,
                3472,  10192, 59056, 17620, 12372, 51968, 42008, 31816, 25240,
                55504, 36752, 47672, 23744, 50628, 63736, 44152, 4876,  23324,
                57072, 13636, 35580, 3048,  29632, 19696, 39400, 4712,  23152,
                36948, 42884, 19944, 51076, 64960, 47852, 4976,  58700, 19704,
                2748,  25152, 61680, 28884, 18892, 12964, 49504, 58980, 15648,
                51568, 60548, 48560, 31608, 25312, 58864, 45740, 39224, 12624,
                3652,  15936, 44236, 11136, 25924, 33960, 55940, 14480, 1316,
                51748, 40604, 14004, 30444, 16896, 45424, 65236, 4288,  33788,
                44552, 20936, 29028, 40004, 61652, 22800, 37708, 960,   59460,
                53444, 40456, 48696, 21456, 29940, 9168,  51480, 20580, 12784,
                32160, 22668, 3780,  47000, 34132, 1024,  50368, 10872, 35404,
                15216, 50016,
            },
            {
                22712, 2592,  17320, 44064, 29684, 39916, 55368, 25988, 46700,
                64640, 16708, 41484, 29824, 19304, 39556, 48892, 62060, 37600,
                19936, 34340, 50328, 43056, 32592, 5000,  13660, 33812, 56884,
                22984, 29292, 42932, 55068, 46360, 20472, 4004,  60472, 49316,
                5764,  43860, 888,   16912, 40744, 10964, 19840, 31052, 220,
                26652, 42876, 59392, 6312,  53632, 14672, 34304, 52248, 26780,
                47252, 55312, 6352,  24408, 41168, 17496, 33092, 11512, 45512,
                40660, 49720, 15976, 7808,  46184, 64392, 39520, 1824,  43268,
                11856, 28228, 9296,  40076, 6096,  16484, 50128, 32720, 54224,
                62160, 28940, 32328, 50464, 65388, 16916, 42444, 21388, 37176,
                26824, 6548,  60652, 52764, 37768, 9176,  49196, 40112, 27620,
                9908,  63012, 55076, 3676,  19368, 51736, 14784, 42480, 25052,
                20040, 16564, 2132,  36052, 59800, 17412, 25456, 65328, 360,
                40104, 59216, 36464, 10396, 24596, 54772, 39480, 19664, 56988,
                4996,  32164,
            },
            {
                46652, 59628, 7736,  53464, 13144, 49088, 19760, 5572,  38820,
                23188, 4076,  49928, 7952,  63528, 3328,  15988, 28760, 52996,
                8836,  64116, 1724,  11508, 23644, 37980, 50920, 26580, 45768,
                62812, 36480, 5832,  25524, 64668, 33936, 10064, 38244, 16256,
                28496, 65256, 32696, 52836, 27292, 58996, 55536, 37376, 47580,
                65056, 18156, 50528, 22408, 62904, 44628, 49264, 8280,  12684,
                63716, 30108, 15416, 60848, 28152, 56268, 37248, 23620, 53200,
                6040,  35932, 58408, 33264, 53648, 25436, 36100, 22504, 55200,
                33332, 17864, 62244, 52620, 35444, 64800, 27692, 5128,  23624,
                42280, 8512,  59256, 608,   23176, 53168, 7364,  61140, 48252,
                18296, 33260, 11324, 42316, 5324,  57412, 31800, 23344, 58508,
                17628, 37108, 14040, 43528, 32108, 7164,  28164, 64808, 34532,
                6256,  56624, 45920, 12204, 54424, 3352,  33488, 47420, 55552,
                27352, 18448, 51400, 63332, 43700, 7000,  16388, 47408, 63212,
                41592, 9764,
            },
            {
                20252, 39200, 34252, 58136, 1228,  36012, 62784, 31224, 52052,
                9956,  35780, 27200, 46000, 33772, 58172, 11204, 42412, 5404,
                24584, 46492, 30148, 55724, 60228, 17896, 65224, 8180,  724,
                12748, 51732, 15704, 39664, 2252,  29732, 53920, 45836, 22952,
                57604, 12980, 8960,  21876, 3120,  34472, 5872,  15892, 52040,
                11524, 33172, 38516, 10052, 31020, 25280, 1604,  60564, 18656,
                40116, 2728,  35676, 10116, 46372, 116,   8188,  60032, 27788,
                64644, 12652, 22348, 932,   41600, 10892, 5008,  30228, 63416,
                46784, 2824,  42564, 22028, 13696, 44308, 9936,  56888, 17972,
                49024, 37124, 19952, 39912, 47308, 12744, 30788, 39280, 9808,
                57052, 44536, 22200, 64024, 25912, 19324, 14448, 1972,  53468,
                6604,  48264, 24936, 52880, 11332, 57440, 46476, 49824, 10440,
                38640, 61048, 32152, 27660, 42824, 49548, 37612, 15352, 10060,
                42568, 4788,  14860, 30004, 20888, 58840, 32556, 27684, 3272,
                24932, 54468,
            },
            {
                64796, 11240, 27868, 18528, 24000, 42916, 8476,  15588, 44508,
                61232, 18456, 57312, 13828, 24104, 51412, 21248, 35600, 60404,
                18048, 39924, 14616, 6800,  44272, 28248, 40488, 53644, 19960,
                31996, 59436, 48244, 21736, 57768, 13464, 18772, 61532, 1656,
                35056, 41708, 54752, 38348, 62504, 45232, 25104, 41008, 21708,
                7604,  27828, 3388,  41376, 16540, 58524, 37244, 42772, 32392,
                52988, 22484, 49912, 43256, 31300, 21096, 49488, 14516, 42860,
                18560, 30456, 56492, 44860, 17184, 61320, 48320, 14984, 8788,
                25844, 37136, 7068,  55876, 30112, 240,   38056, 60480, 33952,
                3180,  13788, 55696, 27260, 5552,  35692, 63196, 24764, 3908,
                53616, 29404, 344,   49840, 35636, 55084, 47168, 63384, 34488,
                41700, 30344, 1532,  64100, 39004, 23380, 17224, 2844,  29660,
                21952, 50656, 14552, 7180,  62952, 23092, 5516,  30884, 61692,
                45964, 33932, 57336, 49484, 1708,  40672, 9440,  52268, 13404,
                37400, 30248,
            },
            {
                15864, 44496, 50480, 4896,  47688, 60248, 27420, 54828, 1940,
                29520, 42012, 53376, 748,   38184, 43728, 4392,  54972, 31928,
                48416, 57176, 36608, 21280, 49696, 2400,  10836, 35060, 24608,
                44640, 4100,  28232, 9084,  35684, 49788, 40768, 7352,  26708,
                51284, 20052, 46764, 29248, 17876, 49744, 13228, 60120, 53896,
                45856, 61556, 56848, 48596, 54352, 6856,  20900, 28036, 9656,
                56460, 5508,  59036, 13024, 63456, 54572, 38388, 33592, 2176,
                51672, 47676, 4308,  38028, 27516, 52036, 20732, 57500, 39668,
                53396, 19480, 59608, 24656, 48108, 17232, 51920, 21676, 46348,
                30596, 63940, 51180, 10308, 59848, 45316, 19364, 50708, 15604,
                35028, 59508, 13352, 17384, 40348, 3692,  28860, 44052, 12392,
                20564, 60980, 15828, 33768, 45020, 4916,  35696, 61824, 54776,
                43724, 496,   25652, 40628, 19580, 10884, 58728, 51740, 21116,
                24804, 7860,  38148, 12244, 26448, 62632, 35588, 44744, 60720,
                49048, 6536,
            },
            {
                484,   56364, 62064, 32680, 14316, 40580, 21424, 34508, 49432,
                12148, 20868, 6908,  64976, 30904, 16964, 62616, 27580, 12904,
                948,   9668,  26432, 61592, 32932, 16092, 47280, 56360, 61244,
                37836, 18160, 42100, 53480, 62472, 24292, 44736, 30924, 11716,
                63004, 3892,  15040, 6768,  57876, 9756,  32040, 2644,  29396,
                35568, 18876, 23536, 11976, 33800, 46052, 14096, 62708, 44440,
                16704, 38280, 25396, 19088, 3868,  26496, 16648, 61200, 10592,
                39712, 25552, 9364,  60144, 32580, 6728,  35164, 43420, 768,
                31856, 49848, 12272, 34196, 61584, 41244, 26796, 11604, 7184,
                43288, 25148, 34724, 17476, 41916, 29140, 1464,  57740, 43076,
                26452, 7804,  45684, 32020, 62508, 10540, 22384, 37956, 8360,
                51256, 26976, 55852, 9328,  19924, 58624, 26112, 40864, 12328,
                18664, 63624, 36552, 52616, 56400, 44636, 28680, 39588, 1280,
                53764, 64624, 18804, 47200, 53236, 22620, 16984, 2480,  19136,
                23196, 42060,
            },
            {
                31400, 25448, 20520, 36876, 53676, 10804, 5952,  63884, 24684,
                58908, 36700, 47196, 26200, 10544, 50120, 7924,  45524, 23652,
                52084, 64372, 42140, 4320,  53040, 58852, 22572, 29412, 6472,
                14564, 64472, 11616, 32276, 436,   6072,  15640, 57076, 38448,
                48232, 32308, 55736, 36248, 22804, 42976, 39076, 63404, 6168,
                14820, 44004, 968,   64444, 29612, 3844,  51176, 24576, 868,
                47916, 29804, 34184, 52636, 41360, 7404,  45804, 57064, 22476,
                31744, 63028, 20188, 54692, 13248, 23596, 63892, 11684, 28240,
                62040, 15804, 4888,  45396, 8928,  3060,  64300, 54556, 16020,
                57648, 1036,  22356, 4732,  54720, 13652, 33480, 22560, 10460,
                39812, 61436, 51860, 24364, 6144,  58216, 52912, 16652, 59512,
                45596, 3376,  40384, 48924, 29352, 51588, 14968, 47492, 5360,
                31040, 58240, 9360,  33028, 15824, 2752,  35376, 17696, 13564,
                41792, 28124, 3596,  31512, 42464, 5432,  55136, 29100, 57640,
                34456, 53120,
            },
            {
                12712, 39888, 8948,  2108,  29472, 58236, 44944, 17112, 3548,
                40188, 31704, 15244, 59940, 41164, 56348, 20284, 36932, 40680,
                16232, 34216, 19376, 30412, 13044, 38452, 8496,  41656, 2856,
                48572, 55432, 26196, 45660, 20152, 59204, 34516, 52644, 21932,
                8224,  25412, 44220, 64584, 628,   51752, 26124, 20216, 55400,
                50560, 40176, 26000, 37356, 17924, 59324, 39716, 35524, 57664,
                12084, 65304, 8648,  59828, 48704, 36240, 29268, 3336,  52584,
                43948, 5568,  15924, 49200, 40888, 2604,  50960, 18728, 47344,
                22680, 40980, 58164, 37740, 20360, 32096, 49248, 36520, 29020,
                40084, 62484, 48184, 38604, 61548, 46600, 6968,  64932, 49148,
                18628, 2048,  36688, 20532, 47608, 42292, 33604, 25944, 5216,
                31956, 23912, 36400, 13344, 996,   60640, 8260,  33244, 53384,
                24488, 45256, 3636,  48916, 23812, 64256, 46984, 61212, 8664,
                48788, 57968, 36548, 9944,  59544, 13716, 39040, 46788, 10792,
                7696,  61328,
            },
            {
                17760, 47920, 43996, 64500, 51136, 25756, 38036, 48360, 56608,
                9132,  51392, 2092,  22920, 34188, 3776,  28556, 61104, 2884,
                58668, 47060, 7244,  44056, 25300, 62816, 46080, 19520, 52340,
                30944, 21948, 7444,  37028, 50876, 41160, 27956, 2652,  17704,
                59652, 40424, 12588, 18788, 28624, 59196, 16060, 46932, 33540,
                10484, 58392, 7224,  53080, 47844, 22048, 8072,  31080, 53716,
                20012, 42196, 2452,  15252, 23052, 11224, 64148, 18920, 13840,
                34596, 58452, 26852, 36380, 45328, 29560, 56004, 37196, 4112,
                9536,  53868, 25776, 29328, 56572, 13300, 23800, 5196,  19396,
                52324, 14104, 8348,  31396, 20176, 25388, 52384, 37472, 27892,
                56300, 31008, 15056, 55448, 28096, 12796, 824,   49100, 65152,
                54448, 18044, 62488, 57140, 22260, 42684, 38116, 64956, 20500,
                39852, 16612, 28456, 12872, 38232, 6448,  31360, 26128, 55568,
                33060, 23016, 15280, 62984, 25140, 50884, 32628, 65444, 26064,
                37696, 50068,
            },
            {
                28588, 58804, 23824, 15248, 3888,  19092, 32356, 12916, 27568,
                19792, 62264, 44888, 54364, 18488, 48904, 13956, 32416, 53852,
                10752, 22352, 50988, 56716, 2012,  33208, 11368, 26820, 58016,
                34556, 40360, 60460, 16428, 63564, 13236, 9748,  46572, 64040,
                30328, 3556,  53804, 33684, 48432, 6464,  11916, 37644, 1936,
                22528, 28936, 61208, 13920, 43180, 10932, 63240, 16160, 4296,
                23648, 46200, 27836, 39360, 56304, 32868, 50432, 25480, 41668,
                8924,  47220, 1164,  64908, 10380, 16456, 25032, 58620, 44388,
                33792, 65168, 1684,  17584, 51532, 39624, 46032, 59984, 10096,
                44456, 25868, 35924, 59024, 11072, 41372, 16768, 2804,  12932,
                45188, 5580,  64076, 9340,  39088, 61952, 19344, 36940, 14088,
                41508, 7228,  10372, 44876, 30756, 16432, 26632, 2984,  11044,
                50028, 62424, 55004, 42144, 59716, 50620, 20540, 11880, 43672,
                5644,  19360, 40304, 45092, 196,   20496, 6436,  15664, 43716,
                21272, 4048,
            },
            {
                33608, 5976,  55168, 35752, 41184, 60788, 7600,  54104, 43192,
                36156, 29692, 6224,  12228, 37756, 63708, 46048, 6424,  25464,
                38716, 29252, 14192, 60884, 17984, 49092, 37188, 64172, 15716,
                1072,  10496, 46992, 5048,  29848, 23756, 55676, 42864, 35768,
                14100, 49908, 23328, 10136, 61888, 41808, 30888, 57272, 65488,
                44980, 17044, 32664, 4772,  35020, 27920, 55848, 40964, 50060,
                33376, 62148, 52100, 18008, 5800,  45164, 476,   38268, 62156,
                55640, 30464, 22976, 53608, 4812,  32976, 61348, 7156,  14512,
                30180, 48332, 11312, 43744, 7176,  63276, 2640,  33912, 55728,
                30484, 1920,  50088, 53972, 3568,  29296, 57144, 34540, 60428,
                23108, 41620, 48792, 32828, 3364,  44680, 53252, 30020, 22928,
                46888, 28504, 34684, 52192, 5968,  48456, 56040, 36020, 58560,
                6208,  32520, 21980, 8404,  18056, 53732, 652,   36720, 60408,
                51224, 2724,  54200, 29740, 48076, 56816, 36220, 59156, 1720,
                54476, 62408,
            },
            {
                39000, 9772,  20024, 30216, 11672, 50040, 22692, 216,   65164,
                14652, 24072, 59428, 41536, 26568, 9376,  57148, 16980, 42848,
                65492, 192,   35124, 40192, 23884, 5252,  53180, 7124,  42028,
                49888, 24908, 19232, 53256, 33184, 1660,  49032, 20528, 26292,
                5932,  58432, 45336, 36048, 2428,  21464, 53112, 25232, 8040,
                49500, 41212, 56292, 20200, 51668, 36,    25176, 45304, 6336,
                13800, 36356, 9804,  60552, 30720, 21320, 58744, 15672, 6772,
                20640, 12720, 39684, 17948, 49532, 42996, 21268, 40172, 52428,
                18420, 38700, 22024, 59480, 35412, 15952, 27332, 21448, 41856,
                17228, 65252, 22660, 15316, 43236, 63616, 47460, 9116,  39628,
                19468, 54152, 26516, 16516, 22032, 57496, 7004,  10868, 58800,
                1752,  61068, 20756, 39208, 63868, 12156, 23200, 44300, 28100,
                14220, 46620, 1808,  35816, 29900, 45084, 24372, 41012, 28488,
                14456, 62164, 34512, 8492,  26672, 11828, 41992, 23592, 31284,
                46364, 13548,
            },
            {
                42816, 51820, 47500, 63300, 44448, 57036, 33484, 39736, 47008,
                4412,  49460, 33224, 52088, 900,   31128, 22624, 36016, 51684,
                20032, 48440, 9264,  55372, 43200, 30860, 13740, 22472, 29160,
                61316, 36328, 58968, 43520, 39220, 57796, 7760,  37724, 61032,
                32076, 16560, 55316, 27056, 38960, 14628, 43932, 18480, 35396,
                12992, 3304,  24760, 63956, 46724, 36820, 60424, 18648, 29544,
                57764, 1276,  25044, 48180, 13108, 42364, 53412, 28360, 36312,
                51640, 43604, 60068, 27572, 35456, 12332, 54584, 32,    26880,
                62128, 4544,  50640, 25400, 31276, 53352, 47820, 12756, 61088,
                7576,  46672, 32820, 38048, 6456,  19724, 24272, 31864, 51324,
                1880,  10772, 35976, 61000, 50552, 25548, 34000, 39540, 49876,
                16228, 53912, 26392, 4264,  18532, 32364, 248,   52928, 19228,
                39288, 60604, 25824, 56672, 65496, 9900,  15472, 57068, 7548,
                46924, 23712, 38880, 16628, 64136, 50312, 18408, 7296,  52452,
                27644, 17156,
            },
            {
                57156, 24624, 1608,  26704, 4676,  17064, 9276,  25580, 18940,
                57948, 10220, 16452, 20796, 55336, 43592, 61948, 4000,  12072,
                58480, 31652, 16188, 27316, 3164,  62136, 57492, 44928, 18148,
                54520, 12732, 3196,  26584, 14344, 62872, 17956, 11704, 51140,
                40,    42544, 20060, 8428,  63040, 50472, 5756,  61028, 28480,
                54296, 38336, 9964,  30384, 15596, 6904,  12188, 39132, 64780,
                21912, 43304, 54428, 38504, 3820,  64580, 34236, 10408, 48524,
                5680,  32376, 1852,  56244, 7816,  63304, 30740, 46616, 9996,
                35916, 56616, 13824, 41384, 864,   9684,  57904, 36440, 3860,
                39308, 54716, 11924, 27264, 49224, 55760, 13348, 4592,  62988,
                28716, 57920, 46192, 5928,  13264, 42176, 17692, 64060, 4540,
                31796, 43212, 13160, 46072, 59600, 50276, 40692, 62956, 7872,
                34256, 49544, 16356, 40160, 4660,  47984, 33464, 63036, 19976,
                31148, 11048, 55900, 43952, 3228,  32080, 61064, 39944, 10584,
                63724, 3356,
            },
            {
                31920, 37780, 59252, 14492, 35308, 29204, 61468, 54496, 36632,
                30448, 44400, 63948, 38800, 7560,  14132, 49268, 28804, 40800,
                5328,  45744, 63052, 51996, 19252, 47472, 38616, 9572,  33632,
                5628,  47932, 31676, 8944,  21492, 44584, 29344, 54504, 24264,
                39784, 65064, 12664, 47720, 30308, 56112, 33232, 720,   45972,
                21984, 62196, 50216, 43820, 57868, 23488, 53988, 31572, 9272,
                51380, 15572, 32708, 19172, 27424, 7856,  23696, 17788, 57536,
                26092, 64048, 16068, 46156, 24832, 38444, 19288, 59672, 23556,
                43548, 28920, 7512,  64272, 19352, 44592, 23476, 50948, 29912,
                25220, 20112, 59224, 324,   35252, 61704, 38852, 45308, 17796,
                42628, 14796, 30840, 38316, 76,    55272, 28180, 45264, 23664,
                37352, 8384,  56992, 35416, 24672, 9152,  15628, 29948, 57080,
                3604,  10388, 23340, 51844, 13120, 27116, 38456, 2460,  50576,
                42076, 4268,  59620, 21260, 37040, 14204, 54764, 25836, 48488,
                35820, 21832,
            },
            {
                12280, 8392,  18976, 40820, 52944, 48372, 43120, 13208, 7076,
                50912, 2152,  27220, 34824, 60736, 24692, 18808, 33704, 56256,
                25712, 22368, 37892, 6624,  12004, 35288, 25500, 592,   65032,
                39704, 23116, 62200, 52400, 35708, 48688, 2740,  33860, 46216,
                7400,  28140, 35284, 4312,  23720, 17236, 10852, 40376, 58940,
                15064, 7232,  34344, 1592,  19312, 40576, 48756, 2236,  45720,
                26812, 7240,  62528, 56812, 46904, 51240, 61272, 39644, 45100,
                3756,  37916, 22332, 11032, 50136, 5272,  14188, 51476, 2912,
                17008, 49416, 33584, 54732, 37824, 60664, 4736,  17600, 43260,
                62256, 9444,  50588, 44492, 16472, 8212,  25668, 33440, 54808,
                23268, 7488,  65500, 53068, 21528, 61508, 8736,  12200, 58176,
                19632, 62980, 28956, 3432,  53188, 37736, 43696, 26176, 20864,
                44984, 62020, 31388, 43008, 55764, 21284, 59468, 17216, 54084,
                26232, 33804, 48820, 6276,  28696, 46020, 1092,  20116, 5212,
                44484, 60444,
            },
            {
                49208, 54348, 45472, 64512, 6116,  22416, 1268,  20364, 63096,
                40988, 23144, 12464, 48160, 3708,  45272, 54232, 1320,  64184,
                8884,  14236, 42576, 32840, 59684, 21440, 55928, 14632, 50928,
                27272, 45672, 15852, 57400, 41076, 5476,  64032, 15084, 56468,
                22168, 58644, 51700, 43768, 59904, 38020, 26736, 49000, 19592,
                31536, 25540, 53392, 13420, 28628, 59948, 33692, 17052, 56464,
                36732, 41316, 2628,  11640, 35204, 15264, 808,   30088, 12884,
                20000, 51940, 60572, 29392, 34120, 44188, 56996, 31852, 40708,
                62684, 11396, 21196, 46692, 14868, 28388, 32960, 54096, 13436,
                2264,  32064, 39820, 28612, 21340, 51856, 60096, 11248, 1184,
                49560, 34640, 25892, 18324, 48208, 35712, 31408, 47160, 52044,
                1528,  41328, 49348, 22284, 14120, 65140, 1956,  48644, 12452,
                53860, 37508, 18316, 1140,  7956,  30264, 44576, 6384,  36748,
                13752, 64872, 17980, 53084, 24096, 58816, 39024, 33204, 56352,
                16016, 28208,
            },
            {
                372,   33756, 24176, 30404, 11176, 38524, 59488, 27608, 33080,
                56280, 17756, 52992, 58112, 31636, 9904,  39748, 16116, 36664,
                47372, 53220, 61292, 28112, 2304,  48940, 41016, 30024, 59184,
                19900, 7948,  1516,  28348, 11156, 19608, 25676, 38432, 9584,
                18152, 3324,  31620, 14400, 53788, 7588,  63916, 43216, 3468,
                57236, 37016, 47616, 42724, 62672, 4600,  25680, 11548, 63424,
                20504, 50208, 23900, 30780, 44568, 21588, 41940, 59556, 54576,
                33168, 7052,  41740, 55388, 3564,  65404, 21472, 8784,  27080,
                37024, 57460, 1740,  25860, 6416,  40832, 10276, 63928, 35664,
                48308, 56176, 14572, 64696, 3016,  41692, 30492, 46812, 20604,
                58344, 40236, 10316, 43736, 2800,  15764, 39972, 5436,  26756,
                34500, 16920, 10464, 33304, 55412, 19308, 31036, 59832, 34924,
                5996,  27300, 57644, 63920, 35972, 49220, 11128, 61832, 23180,
                732,   44884, 30296, 11908, 62552, 8652,  50940, 12632, 65172,
                7788,  41704,
            },
            {
                20788, 61444, 3968,  16792, 56892, 34548, 49492, 15236, 46312,
                4284,  36896, 8016,  42152, 20692, 28016, 59616, 50508, 21092,
                31024, 3580,  18440, 45016, 8480,  53756, 17084, 4848,  11036,
                42516, 37172, 54932, 34312, 61112, 50056, 31904, 52908, 43408,
                61996, 48496, 40536, 19660, 1240,  22540, 34576, 12548, 51232,
                6092,  64804, 10468, 21680, 7844,  37840, 51564, 43140, 3792,
                29484, 14300, 58188, 61696, 52656, 5400,  36576, 9204,  46448,
                27176, 48420, 13972, 18340, 25660, 39732, 16420, 47504, 53204,
                4388,  44468, 30972, 52000, 58932, 49188, 21580, 45280, 26592,
                19484, 6948,  23244, 53600, 34296, 6000,  56528, 13852, 37584,
                5376,  62704, 51288, 29272, 60048, 56336, 22460, 64648, 13368,
                53840, 59744, 44276, 5920,  46872, 40068, 7552,  23692, 51160,
                16000, 41048, 46884, 13908, 24708, 52508, 41300, 28012, 56632,
                48100, 38556, 7084,  41276, 35120, 16680, 42804, 22740, 30784,
                38132, 52684,
            },
            {
                57444, 14024, 39444, 50672, 44140, 24956, 7676,  53792, 10608,
                29688, 61476, 25664, 15544, 64708, 35864, 12760, 6580,  41956,
                27028, 11192, 57556, 39196, 24012, 64140, 36404, 46772, 32408,
                63000, 51552, 17324, 44108, 22716, 13632, 59136, 828,   29124,
                36008, 11308, 26352, 62924, 46264, 30192, 55872, 16724, 28828,
                24280, 41080, 17776, 30564, 55280, 15296, 22728, 59304, 34956,
                46124, 6552,  39188, 9892,  18596, 28308, 65084, 24848, 17176,
                1760,  63496, 36268, 9352,  52544, 612,   35620, 58260, 13148,
                24304, 18016, 64728, 12604, 35280, 17196, 536,   56924, 5260,
                42156, 60912, 37364, 11160, 44868, 17572, 24552, 61840, 27996,
                17060, 32092, 3884,  13736, 24620, 8068,  48852, 43144, 20224,
                37704, 30236, 24028, 61716, 27724, 58128, 11076, 42752, 63388,
                2432,  22444, 9616,  32952, 5144,  19752, 15356, 34552, 9188,
                19068, 60708, 26004, 50032, 2964,  54992, 27476, 4576,  47936,
                17908, 25904,
            },
            {
                35604, 46648, 8760,  27896, 62560, 2904,  19256, 65452, 39192,
                21900, 44520, 50228, 1628,  47432, 4936,  52600, 24228, 63188,
                55672, 48556, 35576, 15256, 31344, 12216, 26620, 56876, 21844,
                2916,  24652, 29844, 4344,  8276,  46604, 40256, 6324,  23128,
                16124, 57552, 5620,  37072, 50488, 9752,  61256, 39092, 45496,
                54008, 316,   34828, 58448, 44540, 48464, 31884, 9380,  18248,
                55732, 26216, 48200, 33428, 1988,  55128, 49924, 12316, 58044,
                40556, 22172, 56520, 31064, 60812, 43072, 28440, 5672,  61488,
                33372, 39204, 7792,  42500, 23220, 61964, 31232, 38680, 51052,
                15972, 30284, 47564, 27160, 62940, 50164, 40392, 8400,  53148,
                42832, 22236, 46684, 54324, 38976, 34068, 29920, 680,   58012,
                9124,  4720,  50512, 15220, 916,   35772, 18004, 47964, 32344,
                56160, 28832, 61332, 53656, 38768, 58292, 64564, 3648,  31168,
                54436, 14464, 22020, 58004, 32052, 20516, 45248, 61764, 58420,
                11008, 5884,
            },
            {
                63848, 31212, 55104, 21160, 12648, 37576, 32004, 42716, 412,
                59088, 13744, 33736, 56732, 40336, 30700, 17916, 44488, 33156,
                308,   19676, 6260,  62432, 52464, 1176,  43740, 7280,  49176,
                13996, 59360, 38628, 56288, 65448, 35152, 18936, 55296, 49632,
                44764, 32688, 53516, 24236, 13864, 41896, 2348,  21008, 8320,
                60164, 14112, 49796, 26788, 6460,  2392,  63748, 40676, 53240,
                512,   64404, 16584, 42660, 22932, 38204, 43892, 6356,  35040,
                29712, 4716,  44992, 19796, 49568, 11740, 22580, 45572, 50400,
                20084, 55716, 47996, 28148, 4624,  52528, 9144,  24900, 12344,
                34192, 59148, 8772,  4200,  20036, 31908, 2324,  34896, 48336,
                11096, 57188, 36128, 18932, 63208, 11800, 51708, 16380, 26136,
                46528, 62728, 41988, 21032, 39184, 52420, 25116, 14016, 4908,
                37348, 19112, 42372, 48672, 112,   26764, 46036, 42216, 50468,
                36532, 1632,  46760, 10188, 64192, 13308, 37056, 908,   33456,
                51296, 43560,
            },
            {
                1776,  17188, 41472, 5232,  51908, 57756, 47176, 16220, 27084,
                51780, 6912,  19420, 11184, 22832, 61776, 55108, 9136,  14500,
                40000, 58708, 23260, 46460, 40564, 20704, 59784, 17732, 33732,
                41340, 10180, 45584, 15480, 26288, 31072, 10940, 62260, 27700,
                8724,  64356, 2908,  17920, 59300, 33556, 27232, 48564, 36604,
                31960, 19528, 62500, 11840, 39160, 20392, 28244, 13648, 24008,
                37396, 30832, 11812, 60488, 57316, 14000, 31156, 19248, 48056,
                61720, 53448, 13064, 38816, 6860,  33948, 64316, 15552, 30548,
                3656,  10776, 60088, 16012, 37404, 46256, 58140, 43064, 65360,
                3500,  22092, 52080, 38512, 57516, 12924, 54944, 22960, 15956,
                64364, 836,   26828, 7204,  44796, 3236,  60992, 40560, 55052,
                35844, 12600, 32924, 56372, 9672,  64220, 30316, 54404, 60060,
                45896, 7412,  12232, 16600, 30856, 21928, 7628,  12860, 24276,
                62968, 28432, 43348, 39372, 6656,  52292, 24720, 15792, 40208,
                28704, 23112,
            },
            {
                13452, 49520, 25288, 61128, 35100, 28924, 8120,  23508, 55784,
                36148, 30268, 63824, 38136, 27540, 2448,  46704, 37160, 25516,
                51416, 29528, 10224, 33816, 4476,  28448, 50344, 37416, 64456,
                27132, 52752, 276,   21144, 49096, 53708, 2420,  37360, 14852,
                21292, 42104, 38608, 29468, 45664, 6648,  65188, 15948, 57288,
                3296,  44212, 23428, 53472, 35748, 60372, 45612, 50180, 6804,
                43980, 20916, 49444, 7424,  27376, 3312,  63156, 51928, 9660,
                23964, 16032, 28004, 2616,  58456, 25392, 52308, 8892,  41852,
                36056, 25880, 53624, 1504,  32272, 20436, 14296, 29188, 18748,
                54536, 45848, 25808, 16644, 43556, 28196, 60508, 45484, 5092,
                30824, 40632, 59236, 14708, 50052, 27936, 19904, 31772, 4256,
                23080, 18388, 2764,  28572, 48448, 6488,  43552, 1996,  20772,
                26484, 52004, 34876, 56692, 62340, 36840, 53956, 18640, 58900,
                4920,  17076, 33720, 56536, 18464, 30100, 48256, 55604, 8020,
                19568, 59152,
            },
            {
                37492, 33268, 9648,  45216, 2256,  14444, 60392, 49344, 10856,
                41112, 3712,  47868, 54036, 43252, 15872, 21256, 59992, 4052,
                65312, 44204, 17588, 54608, 61624, 13608, 9160,  55932, 23064,
                5452,  31528, 62352, 36264, 7056,  40704, 23704, 47552, 56552,
                4368,  52068, 12532, 22724, 55992, 51428, 11736, 24896, 52724,
                28756, 9608,  40856, 4168,  16112, 55892, 10516, 33764, 57700,
                62652, 3584,  52924, 40420, 34624, 46560, 21508, 37288, 100,
                42880, 33512, 47324, 62464, 41332, 17288, 37560, 1236,  57220,
                63044, 44244, 22308, 40496, 64004, 55544, 5804,  49684, 1328,
                36184, 40984, 11556, 63680, 208,   35912, 7812,  39016, 25740,
                50492, 20168, 33160, 42452, 23464, 37452, 57388, 9576,  48204,
                64056, 42872, 51436, 59636, 16592, 22748, 38084, 33344, 10568,
                40156, 64676, 23492, 3216,  43916, 9880,  47804, 32668, 39568,
                52372, 11852, 49168, 26388, 2584,  59936, 42232, 11700, 64844,
                45936, 52488,
            },
            {
                6060,  62856, 56148, 17964, 22396, 39512, 43456, 4396,  20600,
                62080, 17780, 24780, 8896,  59220, 6132,  34948, 49056, 31468,
                13048, 7144,  38404, 48676, 25028, 42968, 30768, 1696,  16544,
                47648, 13084, 42612, 18164, 57396, 12492, 60996, 43360, 30356,
                34100, 60600, 48152, 540,   35364, 19204, 43152, 5372,  37960,
                46868, 63348, 33144, 49396, 24988, 30368, 1188,  18980, 27076,
                15240, 36192, 25076, 18356, 10416, 54636, 15304, 59808, 26540,
                57916, 54968, 7712,  21496, 30820, 54396, 46316, 21104, 28580,
                12912, 17848, 51072, 7656,  12096, 35348, 26624, 39544, 62132,
                30900, 7716,  55988, 32464, 49552, 21620, 52568, 18180, 62088,
                56044, 12536, 8740,  54016, 65104, 5820,  45552, 13576, 52940,
                38004, 7136,  25376, 35432, 45992, 13088, 57568, 61480, 47288,
                15456, 29980, 5984,  50072, 28216, 14824, 60632, 1856,  29348,
                43988, 22864, 8324,  62644, 37996, 22148, 5028,  35272, 26532,
                2976,  29936,
            },
            {
                21376, 40528, 26748, 48040, 31100, 64000, 54280, 26260, 34400,
                57656, 45136, 14252, 32532, 51712, 29024, 10928, 56296, 42120,
                20268, 27208, 57828, 2188,  15904, 35640, 59040, 46040, 39296,
                54256, 59952, 24196, 50756, 33024, 28444, 16856, 5824,  19496,
                9516,  26256, 16632, 63588, 8516,  39788, 30152, 61540, 21436,
                13092, 17828, 58000, 7644,  42892, 65108, 37772, 51800, 42024,
                8792,  46956, 56100, 32144, 64700, 42184, 29800, 6664,  39832,
                18416, 11284, 36596, 51300, 13904, 4868,  10572, 60220, 49140,
                34584, 3280,  31264, 47080, 24296, 43464, 59100, 10292, 23028,
                15392, 48504, 19564, 4752,  58384, 14328, 30168, 10832, 42856,
                1676,  36828, 47888, 29356, 2512,  17856, 34664, 25648, 29664,
                15636, 60816, 10876, 31060, 296,   52728, 27548, 18632, 7916,
                55012, 43060, 59124, 18544, 39052, 24392, 56072, 20848, 5840,
                64348, 54820, 35744, 14592, 45648, 51488, 32300, 16804, 57624,
                39228, 12312,
            },
            {
                60036, 15452, 1168,  52240, 11792, 5560,  36832, 15920, 30732,
                2692,  50140, 37252, 728,   40876, 62620, 24444, 17088, 1196,
                62024, 52204, 32540, 22108, 64084, 6024,  52436, 21664, 10764,
                34680, 28964, 3140,  9336,  45192, 1372,  53016, 63772, 36428,
                46172, 57584, 41268, 31812, 23820, 58992, 49988, 3108,  34316,
                54236, 1484,  27696, 21120, 13500, 46540, 5812,  22736, 58604,
                30000, 12804, 60356, 1008,  5940,  23764, 49068, 12940, 46104,
                31720, 65264, 1540,  43912, 27532, 63700, 33000, 24712, 7096,
                39640, 55356, 60752, 16200, 52564, 2200,  18132, 46136, 51884,
                57540, 28024, 42688, 23588, 39936, 46716, 63260, 5668,  33664,
                23728, 59604, 15376, 22388, 60888, 50996, 41784, 58348, 1292,
                21176, 54916, 44804, 19412, 65380, 41000, 4132,  50304, 36372,
                22464, 920,   32944, 11540, 63288, 7064,  41904, 34972, 46960,
                16088, 27448, 1344,  58104, 20180, 10676, 61804, 43708, 23808,
                48356, 53944,
            },
            {
                45040, 8412,  42972, 33884, 57716, 19916, 9068,  44348, 53000,
                12488, 27928, 65048, 22492, 18952, 45828, 36408, 54100, 39580,
                47380, 9492,  36784, 45852, 11892, 40908, 18188, 27256, 63328,
                7172,  19656, 37992, 64900, 58280, 21784, 39364, 49540, 24184,
                2064,  13848, 54336, 5076,  45276, 10944, 14904, 56980, 25532,
                43908, 39028, 51252, 61136, 34936, 53780, 16656, 61608, 3244,
                38988, 50476, 21988, 44124, 16976, 51840, 35776, 62372, 3204,
                22372, 49792, 25188, 57476, 18984, 37832, 42116, 56452, 19640,
                45500, 26712, 9636,  37684, 29576, 63488, 32672, 6824,  34884,
                4176,  13124, 64620, 33988, 8684,  26252, 37520, 54868, 49304,
                27728, 45696, 6600,  39344, 32060, 12196, 8284,  63600, 46508,
                32968, 40516, 5960,  56704, 37144, 9864,  24872, 31780, 63016,
                13544, 49320, 37480, 45072, 53476, 31048, 51440, 13180, 9768,
                38840, 50880, 31832, 41688, 5736,  29448, 55016, 124,   9028,
                18796, 31692,
            },
            {
                35848, 28408, 65220, 23364, 38552, 49648, 62612, 23872, 60216,
                7556,  42368, 54696, 9920,  58464, 4536,  13536, 7524,  29760,
                22856, 14596, 4280,  56068, 29892, 49664, 33216, 2712,  44164,
                56780, 49296, 41788, 12176, 26524, 34860, 14740, 10156, 59736,
                29156, 38304, 19740, 50608, 26956, 37088, 18472, 47656, 8148,
                63932, 10708, 30872, 4456,  40408, 9452,  28328, 33032, 45228,
                19276, 7188,  35116, 28560, 38620, 57404, 9604,  27304, 54212,
                41796, 14956, 34928, 8692,  47756, 3460,  16312, 50712, 292,
                14576, 65128, 22700, 4040,  56768, 12864, 41628, 21772, 60724,
                38648, 44824, 1144,  53636, 60092, 16992, 2044,  20568, 13332,
                61676, 17940, 53508, 43616, 56164, 20288, 36764, 23876, 4552,
                50632, 13016, 28372, 23552, 48976, 15404, 54192, 42308, 6400,
                28784, 56120, 25920, 20712, 3100,  16840, 26380, 61968, 57308,
                23732, 18476, 61016, 47708, 25096, 40100, 15516, 49748, 37272,
                63232, 3576,
            },
            {
                50172, 55648, 17640, 5080,  14168, 29792, 388,   47072, 39452,
                20784, 35024, 16384, 47900, 31760, 27512, 50412, 64240, 34232,
                59364, 43208, 18644, 61124, 25528, 8364,  59656, 53940, 13376,
                24880, 31976, 16280, 3976,  54956, 47568, 6496,  31712, 43164,
                52148, 7288,  33668, 60056, 2568,  63080, 29180, 40912, 32368,
                22924, 15796, 47180, 19536, 57416, 23676, 49656, 12360, 55488,
                26096, 62936, 53116, 13724, 61344, 2164,  20448, 32996, 17448,
                58304, 4836,  29304, 53136, 60644, 12524, 30164, 62604, 35300,
                52936, 32204, 41028, 49864, 19964, 47732, 27528, 55680, 16424,
                25360, 50352, 18784, 29984, 11744, 51360, 44668, 31900, 41220,
                3156,  35492, 10532, 25984, 1600,  48964, 28876, 16036, 54176,
                38296, 17844, 63272, 3416,  34016, 61568, 20432, 45772, 59448,
                17632, 9728,  65024, 5796,  58440, 48312, 36908, 1012,  29964,
                43228, 3736,  8620,  12960, 52236, 63568, 33380, 27204, 58596,
                13688, 25072,
            },
            {
                21064, 11208, 46920, 61184, 42200, 25908, 55212, 17420, 31880,
                5280,  26228, 61052, 3300,  38712, 56756, 42668, 20160, 1924,
                48776, 26996, 52660, 38952, 72,    45104, 15592, 36452, 39952,
                5208,  61168, 51832, 29700, 43468, 19296, 63128, 56952, 16444,
                22228, 65316, 12644, 42092, 21804, 55064, 6544,  53064, 68,
                59976, 36344, 55656, 26656, 2280,  43312, 64268, 36172, 404,
                41500, 10144, 47088, 5200,  24332, 45348, 40796, 48384, 7080,
                37432, 61528, 45368, 20488, 39176, 23608, 43628, 8124,  21920,
                11168, 44228, 5720,  58956, 34728, 8808,  616,   52648, 10712,
                31508, 61800, 7684,  36864, 42220, 22780, 58232, 65324, 8316,
                56560, 51276, 30844, 64260, 5256,  57972, 44660, 61860, 9540,
                25800, 59260, 46988, 43400, 8152,  29608, 12128, 1564,  34436,
                38384, 47188, 40944, 14724, 33920, 42748, 22212, 7984,  49472,
                64884, 34136, 55420, 37320, 21836, 2492,  6516,  19504, 46284,
                41576, 6628,
            },
            {
                59812, 39932, 2032,  31332, 52572, 10048, 35516, 13304, 58052,
                49128, 52700, 11608, 44248, 23544, 8712,  15848, 25684, 11412,
                55524, 5064,  32032, 12584, 23056, 64852, 20132, 29332, 48088,
                22272, 10020, 36072, 59336, 23504, 356,   40316, 26180, 4448,
                48540, 27592, 46568, 9472,  16164, 34780, 49104, 13328, 20676,
                45532, 4980,  50692, 13912, 59560, 31768, 6660,  21488, 15880,
                58472, 30984, 18228, 33836, 54644, 29764, 11340, 64144, 51384,
                13484, 24428, 10376, 1784,  32524, 49404, 58600, 27268, 38596,
                60384, 17900, 28732, 13880, 25164, 39144, 62456, 43844, 36272,
                3160,  21036, 47240, 56052, 27432, 5904,  15368, 28552, 37784,
                24800, 21168, 14116, 38864, 18424, 33212, 12928, 40616, 34908,
                884,   30692, 11400, 21416, 57840, 51804, 39892, 55864, 23960,
                4724,  52460, 21688, 28472, 61012, 10268, 56700, 39772, 14340,
                19820, 45732, 27276, 16700, 60128, 43512, 35540, 11496, 56056,
                30524, 52864,
            },
            {
                33088, 26296, 36844, 20204, 58552, 45460, 6704,  63668, 42632,
                2248,  37792, 30604, 18780, 63644, 35248, 52984, 60396, 37592,
                41036, 17020, 62420, 35952, 51004, 41448, 10492, 58116, 1408,
                63108, 18724, 45568, 7060,  13488, 50076, 33004, 10848, 36332,
                55380, 1060,  38884, 31216, 57884, 43692, 25620, 38220, 65000,
                28508, 41548, 33840, 8812,  38412, 18348, 45948, 53380, 39112,
                48584, 23040, 65196, 42504, 15444, 59824, 3912,  26344, 18944,
                31424, 42592, 55120, 64632, 17160, 6304,  53680, 3916,  46860,
                55540, 1524,  50752, 63936, 45432, 18696, 30028, 15080, 49408,
                58500, 40928, 14384, 63692, 2668,  35168, 50152, 10816, 48272,
                5448,  46148, 59172, 42484, 50460, 22948, 7012,  47828, 19552,
                52252, 56008, 41476, 35920, 2600,  16928, 48508, 27468, 14228,
                62404, 31220, 392,   54472, 45632, 18520, 31888, 52132, 25316,
                5268,  58856, 10664, 31228, 47116, 53592, 24252, 64968, 38464,
                1032,  16196,
            },
            {
                62908, 7936,  50616, 15360, 3940,  22968, 40052, 19336, 28280,
                24328, 61960, 14432, 55276, 572,   46900, 6108,  32432, 45588,
                8340,  21796, 46948, 6788,  56396, 3904,  33740, 52948, 26920,
                42552, 32800, 56128, 27752, 38124, 54020, 61976, 20748, 42300,
                60756, 18436, 52496, 23640, 62092, 2136,  8632,  18068, 58184,
                11236, 24248, 17104, 63292, 52476, 29008, 10860, 60596, 27096,
                4020,  8484,  37436, 1560,  47972, 21400, 35400, 39472, 56668,
                2556,  47404, 34800, 26068, 40836, 11988, 36364, 19604, 14388,
                24912, 31476, 36628, 7372,  54040, 3644,  56856, 23388, 5108,
                26992, 33560, 10068, 24368, 45448, 18080, 54332, 40332, 19208,
                62724, 33780, 88,    26808, 9180,  61084, 55152, 27680, 65080,
                5704,  23956, 14928, 60852, 25956, 32548, 64528, 7324,  43300,
                18964, 49252, 35936, 12556, 6692,  26660, 2204,  61572, 36292,
                53916, 41908, 532,   39048, 7568,  13992, 29636, 49300, 9268,
                22544, 44176,
            },
            {
                12892, 54876, 41468, 64224, 29200, 34088, 55776, 50840, 10564,
                45988, 7408,  21800, 49712, 40504, 28484, 20800, 13796, 65144,
                30068, 58488, 27428, 14712, 30704, 24828, 45404, 17204, 7360,
                50416, 14540, 2624,  64820, 17996, 8796,  46180, 14912, 29276,
                7748,  33828, 13932, 5100,  48024, 29540, 55756, 33520, 50008,
                6316,  54328, 43840, 976,   47488, 24600, 2836,  34632, 14352,
                55964, 51016, 28060, 12848, 57832, 52592, 6300,  44416, 63104,
                22568, 7904,  15020, 50800, 56408, 29104, 63508, 33108, 41392,
                62076, 10232, 42440, 22852, 48668, 12112, 33868, 41660, 64840,
                19024, 54964, 50808, 39240, 59092, 31828, 61364, 4184,  30572,
                55652, 11784, 17004, 53772, 30428, 36420, 15492, 2120,  32452,
                44024, 49328, 8456,  38896, 47204, 5060,  20984, 37412, 57244,
                10160, 59956, 24348, 38692, 63800, 50388, 43132, 16320, 11696,
                29488, 22876, 48960, 62900, 20824, 57008, 2960,  17596, 60808,
                28272, 47712,
            },
            {
                19280, 5124,  24636, 11336, 46524, 60636, 1424,  15004, 32196,
                54548, 36300, 58936, 33316, 10448, 59816, 51960, 3020,  24500,
                53824, 4316,  50424, 42540, 18968, 63340, 38848, 12376, 60780,
                37488, 24560, 41136, 48640, 31240, 25736, 2016,  51308, 56320,
                45184, 25196, 64168, 41420, 20588, 15212, 39860, 46748, 3152,
                31200, 37464, 60936, 21408, 33116, 57728, 40784, 63720, 43264,
                20096, 32928, 61848, 40480, 24808, 9708,  32608, 16336, 11576,
                53620, 29896, 61288, 580,   21768, 45124, 8904,  49044, 2360,
                51756, 20512, 58660, 15984, 27864, 38024, 60212, 8624,  45144,
                13100, 6364,  29144, 460,   21544, 8452,  13360, 25856, 42356,
                23000, 37260, 44192, 64020, 4984,  46816, 41772, 21452, 37880,
                12104, 17944, 63432, 29088, 54360, 11172, 44684, 52916, 33924,
                28260, 5244,  42044, 15284, 57748, 33536, 21600, 47452, 7440,
                64128, 33748, 17488, 51608, 26520, 33124, 36888, 42124, 52360,
                3800,  35048,
            },
            {
                56880, 31096, 49060, 35784, 17992, 8092,  26364, 38264, 65356,
                2996,  17724, 43168, 4160,  25916, 16816, 38916, 34412, 43336,
                18620, 11804, 36728, 60204, 9212,  48848, 2124,  57616, 30072,
                19736, 54848, 11124, 5292,  43640, 60180, 34808, 23164, 4348,
                38012, 11676, 49868, 54432, 35460, 59476, 11768, 23068, 62580,
                19428, 13440, 27804, 7608,  11912, 16052, 49600, 6796,  23480,
                10012, 46416, 4524,  18744, 45332, 29408, 60540, 48972, 26848,
                35876, 41912, 18136, 38480, 5012,  60160, 24148, 17376, 56740,
                28416, 35156, 4460,  55044, 43924, 804,   21132, 52200, 31016,
                37168, 47144, 62996, 16440, 43484, 37060, 52180, 46964, 2084,
                58264, 7196,  51540, 20560, 24752, 11276, 57760, 60468, 51664,
                25708, 56972, 34204, 288,   19192, 60156, 13856, 25040, 2244,
                16872, 46196, 55348, 20304, 9172,  28988, 3684,  59052, 39828,
                55660, 2784,  12444, 42960, 5156,  61856, 45944, 12184, 25408,
                58964, 39284,
            },
            {
                22240, 61432, 232,   53664, 43320, 58192, 20836, 47836, 41864,
                23316, 29260, 48484, 13568, 55744, 45300, 63180, 9324,  48536,
                57328, 40032, 756,   32744, 21636, 53552, 34820, 23140, 47448,
                3752,  62736, 35760, 21712, 57368, 12768, 39272, 63792, 16500,
                58400, 31544, 1348,  9412,  28192, 5540,  44544, 26628, 52468,
                41176, 59128, 45792, 51772, 38708, 54836, 19072, 30976, 37304,
                52780, 58196, 14864, 36496, 55056, 104,   20508, 38612, 3116,
                58892, 50324, 9784,  46912, 32248, 14028, 37332, 43680, 5708,
                13072, 40012, 65464, 9484,  32480, 62268, 26040, 16696, 2680,
                58768, 23524, 34772, 53368, 57480, 27656, 64928, 34384, 16260,
                48896, 28764, 13780, 40136, 33044, 49780, 3552,  28896, 8004,
                46224, 4376,  40752, 50156, 23688, 41644, 31588, 62544, 39236,
                51444, 65216, 31340, 1596,  48444, 53536, 36816, 24480, 18872,
                45800, 27148, 37496, 58564, 15692, 9048,  54092, 20232, 31964,
                7624,  14704,
            },
            {
                43812, 10260, 28048, 39612, 13984, 31680, 51704, 11732, 6152,
                59912, 52732, 8840,  62072, 31540, 22516, 1416,  27888, 15432,
                31252, 22988, 64012, 14164, 44624, 5300,  27944, 15632, 43824,
                8940,  31684, 51560, 16376, 29016, 49644, 19644, 6760,  43296,
                27252, 21180, 60580, 46512, 18200, 38120, 64576, 780,   16292,
                9252,  33404, 22836, 5180,  64792, 25728, 3448,  44720, 62320,
                1152,  25976, 31452, 63896, 7668,  50908, 42788, 62600, 13892,
                5304,  21012, 25324, 64856, 53720, 27816, 51132, 58952, 31204,
                47764, 25380, 18776, 46384, 50600, 14044, 40756, 56640, 48612,
                19992, 42460, 9024,  12352, 3464,  19440, 6112,  22216, 10324,
                62220, 35632, 59580, 2468,  56136, 17616, 35452, 14556, 20120,
                62692, 30828, 15772, 10184, 36084, 56524, 3668,  48748, 9128,
                21944, 12816, 35704, 26544, 40232, 11816, 62792, 14436, 31248,
                9504,  52972, 21748, 47588, 30672, 23904, 39688, 840,   63656,
                46876, 51376,
            },
            {
                36432, 17716, 64736, 7128,  23836, 4080,  62740, 27736, 34348,
                15344, 39124, 20324, 35904, 5656,  50748, 37484, 54544, 61396,
                7072,  46176, 51912, 25828, 56328, 40792, 11188, 64492, 54240,
                40092, 26420, 59572, 45888, 492,   9300,  54800, 32856, 47360,
                52392, 13552, 41064, 34008, 23908, 57040, 50892, 30592, 35944,
                55592, 3536,  49240, 30040, 42520, 35740, 59796, 13256, 28800,
                17544, 41820, 48520, 22772, 12576, 34056, 17392, 24308, 45636,
                31088, 56724, 34504, 42984, 11472, 19776, 1100,  8368,  21872,
                60856, 53192, 2116,  29832, 23124, 5780,  34988, 10920, 28568,
                4424,  61852, 26492, 32616, 39380, 49728, 44184, 56380, 40900,
                25612, 19144, 46116, 9316,  26720, 43764, 61928, 39460, 53836,
                42952, 22860, 58484, 52112, 27556, 7116,  45068, 18364, 29852,
                59168, 43856, 7168,  61492, 17556, 44708, 4940,  49940, 42636,
                60568, 5528,  34496, 65484, 2516,  50392, 57900, 34784, 16676,
                28708, 4292,
            },
            {
                24852, 56316, 47340, 34032, 55236, 44948, 37528, 18816, 56824,
                45504, 456,   25244, 57488, 42696, 18312, 12828, 41072, 21020,
                3720,  35732, 29052, 8308,  18024, 60836, 50096, 36880, 1300,
                18856, 13640, 6408,  37788, 23616, 41344, 61360, 25136, 2140,
                36856, 8012,  63144, 3672,  48796, 10968, 7024,  21524, 61132,
                43772, 18752, 58056, 15196, 10548, 20644, 47476, 8084,  50584,
                56948, 10824, 5004,  59752, 39872, 28236, 56240, 8240,  36168,
                52844, 16096, 7340,  2944,  57824, 40204, 62896, 34360, 42204,
                10880, 36320, 16372, 38920, 59568, 52676, 45364, 64552, 37848,
                53880, 15272, 51116, 46352, 59264, 14952, 29244, 32316, 816,
                54132, 5496,  31056, 64764, 53228, 22252, 6816,  31836, 10844,
                1964,  37132, 6216,  46944, 61248, 21136, 14756, 53588, 34692,
                636,   24100, 50196, 54708, 21112, 58152, 34372, 28044, 1672,
                56688, 15120, 41288, 10588, 18216, 43632, 27072, 11364, 54860,
                41440, 60360,
            },
            {
                13060, 2376,  41196, 20988, 15808, 60792, 1756,  9932,  50112,
                31076, 63120, 11320, 46940, 28900, 64508, 8600,  26332, 58364,
                49640, 11644, 59740, 38380, 2276,  30688, 21312, 32940, 25204,
                57392, 48344, 33500, 65288, 52960, 30324, 15208, 11084, 57104,
                18736, 29236, 54000, 16368, 31404, 42912, 14800, 39676, 48012,
                12036, 27444, 37680, 61796, 336,   33600, 55920, 39984, 22188,
                32412, 37852, 45116, 16076, 53904, 2076,  65376, 47652, 11352,
                61596, 39424, 29384, 49680, 17292, 24744, 47396, 15100, 27068,
                56036, 6556,  63636, 43324, 8748,  20248, 2444,  31792, 18548,
                7884,  41364, 21080, 1396,  23988, 7272,  63480, 12020, 50420,
                38168, 15372, 41892, 12608, 36936, 600,   45900, 58208, 49584,
                27060, 65480, 18012, 12460, 33200, 38872, 64096, 26368, 57224,
                41748, 16128, 37876, 3616,  32132, 8704,  23212, 64160, 38756,
                20332, 25168, 51656, 29048, 38172, 60240, 5868,  48576, 22596,
                8064,  33052,
            },
            {
                49564, 27652, 52560, 10652, 29868, 49084, 25780, 42436, 22056,
                4332,  40952, 55008, 16212, 2532,  33676, 52904, 45004, 31728,
                16968, 42988, 20104, 54172, 47304, 14536, 5652,  45624, 8552,
                61660, 41516, 20672, 11972, 4372,  44632, 35672, 48936, 64180,
                39600, 23020, 44564, 58008, 25716, 62444, 51916, 28840, 1876,
                23700, 7720,  53128, 45476, 24460, 52140, 16448, 26988, 2000,
                64480, 19596, 51628, 25564, 30940, 6504,  20876, 41056, 26636,
                1512,  22760, 44500, 63672, 36580, 31716, 55228, 5892,  50728,
                20076, 32644, 48004, 13192, 27624, 55428, 24520, 12256, 48092,
                60184, 34416, 29768, 62336, 54724, 36116, 18040, 45464, 60688,
                23796, 57960, 48132, 19932, 51172, 30092, 16824, 24580, 13580,
                34276, 44772, 55284, 29820, 1052,  43176, 9424,  5184,  47660,
                11388, 63392, 28656, 46328, 13600, 41188, 52172, 16716, 10900,
                48304, 32500, 61464, 7536,  55628, 21352, 14780, 36640, 64448,
                45128, 19384,
            },
            {
                37544, 63408, 5504,  59624, 38996, 7304,  32916, 64916, 53048,
                14224, 27472, 35488, 22776, 60440, 48428, 4556,  23240, 36984,
                260,   65392, 24840, 33548, 41636, 63236, 56904, 52196, 16480,
                29572, 2848,  55252, 27480, 17516, 59356, 21480, 3452,  32008,
                5908,  50704, 11992, 396,   37012, 4816,  19300, 34492, 55132,
                65416, 32584, 41092, 4468,  29300, 63604, 6320,  43184, 58976,
                35032, 4212,  9496,  62344, 36256, 43992, 50356, 32948, 18896,
                59748, 51776, 13280, 9224,  21356, 4144,  12364, 38272, 44848,
                61552, 24144, 1148,  57888, 34660, 38332, 62796, 43084, 26952,
                55952, 5188,  13372, 38704, 10112, 42676, 26936, 3748,  20720,
                8832,  33900, 3084,  25264, 59372, 10104, 63400, 41616, 54344,
                19788, 4492,  40148, 25020, 60232, 49608, 23192, 36792, 31184,
                20316, 52024, 8112,  59928, 25596, 55800, 47264, 6484,  35908,
                54948, 43984, 172,   13112, 45876, 33528, 3704,  52892, 30432,
                1252,  57384,
            },
            {
                15668, 32232, 24344, 44252, 19712, 54568, 12420, 17396, 37104,
                60124, 6724,  51644, 9460,  38888, 19988, 14364, 62660, 56596,
                7732,  51536, 13756, 4304,  10092, 27464, 22628, 37976, 43452,
                11104, 36228, 50804, 46320, 39756, 8208,  55668, 25692, 42496,
                15812, 60436, 34388, 20544, 41208, 53548, 9408,  60176, 44288,
                15784, 20744, 57600, 13032, 18460, 38092, 49576, 11456, 14880,
                53428, 28540, 47164, 13680, 23216, 57284, 8888,  14472, 54736,
                5240,  35200, 27980, 59032, 46324, 52092, 57620, 29120, 3332,
                16836, 41096, 30804, 51332, 17868, 4068,  49956, 15888, 144,
                22488, 45060, 52512, 19488, 47952, 32792, 57340, 52432, 37440,
                29060, 43328, 62944, 6680,  40024, 35112, 47516, 3740,  8048,
                61880, 51852, 10024, 15820, 56772, 18800, 53304, 13512, 60784,
                40444, 2368,  34176, 18144, 36756, 940,   30380, 61836, 27504,
                4084,  18512, 22972, 49852, 26152, 62744, 41840, 17372, 25108,
                40508, 9872,
            },
            {
                21824, 46832, 3088,  13708, 35328, 58476, 45840, 4932,  29644,
                47576, 18960, 44256, 32084, 57692, 42344, 29536, 10972, 40448,
                27364, 45560, 30388, 57820, 48844, 18288, 34440, 584,   59880,
                24784, 63696, 6900,  22840, 33112, 62820, 13364, 46892, 53728,
                27984, 8996,  45600, 63576, 28460, 47112, 13240, 25128, 6140,
                36672, 50956, 9676,  47292, 34732, 59432, 22696, 31580, 45716,
                24220, 39416, 58024, 41588, 3524,  18084, 61072, 38320, 29696,
                46624, 41952, 16552, 432,   40296, 26084, 18224, 64208, 35648,
                10636, 59296, 7852,  45680, 22292, 10504, 40700, 32768, 58884,
                36476, 8988,  64332, 25252, 1872,  15708, 6584,  65372, 13336,
                48856, 17168, 53436, 27576, 14688, 56232, 21856, 28880, 38076,
                32372, 22312, 48224, 36144, 2932,  32216, 45372, 3984,  27456,
                54880, 24312, 49400, 42792, 65092, 11504, 19728, 43376, 13960,
                57504, 40168, 64688, 37192, 29788, 9628,  57780, 46996, 12388,
                60192, 54156,
            },
            {
                49884, 39804, 60728, 52116, 27228, 776,   23304, 40200, 57324,
                25068, 11832, 64188, 832,   24664, 5948,  54488, 34036, 49456,
                16184, 21560, 36508, 62228, 39660, 7152,  54616, 13176, 49440,
                19560, 31936, 15288, 57160, 2320,  29116, 36620, 19060, 1388,
                38368, 57020, 24616, 7044,  16876, 58948, 31596, 39464, 49012,
                2704,  29828, 26224, 61644, 1360,  41880, 8560,  56424, 2476,
                61228, 17308, 6128,  32284, 53100, 48288, 25764, 2300,  64232,
                10976, 24168, 56212, 32572, 61308, 8680,  42940, 22892, 48840,
                53532, 26668, 14484, 65060, 54980, 29620, 60876, 6872,  51792,
                28268, 17056, 40164, 30636, 60448, 49904, 22756, 40776, 31436,
                1556,  56656, 10808, 45360, 32128, 2100,  18492, 50548, 59988,
                14276, 26792, 42296, 63192, 28660, 8612,  41368, 64724, 17304,
                9724,  58328, 14184, 4584,  22768, 57092, 38684, 51044, 24376,
                31376, 7988,  53208, 15800, 4364,  19732, 50932, 2772,  35240,
                28908, 6148,
            },
            {
                33824, 8548,  30184, 17736, 41728, 63808, 50668, 9420,  34408,
                3508,  54196, 37776, 48860, 16812, 60988, 44872, 19272, 1836,
                61404, 53496, 11820, 2384,  24396, 44656, 64328, 28596, 40552,
                4892,  52768, 43896, 10132, 49064, 41696, 51864, 10728, 64888,
                21336, 48680, 13504, 33148, 50844, 1572,  21936, 55824, 18260,
                63796, 42992, 54920, 21696, 15424, 52020, 28336, 18840, 36824,
                49004, 10508, 21632, 64616, 28984, 11928, 35504, 43088, 21212,
                51192, 6592,  48980, 19532, 37224, 4792,  14336, 33516, 2316,
                44280, 20572, 39632, 36092, 5524,  25308, 47284, 20144, 11772,
                46108, 57812, 3092,  44144, 55204, 35808, 11000, 46296, 25504,
                61808, 19708, 36316, 23088, 51784, 64964, 41280, 9716,  45180,
                80,    57636, 6032,  12676, 52776, 59080, 21420, 34392, 47440,
                38948, 31988, 45052, 28824, 53364, 33292, 8592,  2796,  46024,
                59416, 34336, 11652, 44616, 60272, 32116, 38520, 22404, 62392,
                44224, 18428,
            },
            {
                65336, 25036, 55416, 11256, 6368,  31320, 20048, 14972, 61600,
                42704, 20740, 30652, 13524, 27596, 51452, 9876,  38308, 26148,
                5440,  32336, 42680, 18948, 52120, 15144, 31592, 9184,  46444,
                59028, 25848, 38216, 60380, 17144, 24124, 4436,  57856, 30724,
                34816, 42052, 3256,  55032, 37608, 43112, 11120, 60804, 34268,
                8100,  13844, 38772, 5320,  45796, 33188, 65044, 4952,  43940,
                26428, 34244, 51264, 38540, 256,   44820, 55868, 15508, 33284,
                58684, 39084, 13412, 63908, 28520, 51344, 55112, 60416, 29756,
                6808,  57116, 32176, 744,   43412, 52380, 14872, 37888, 63828,
                23972, 33908, 14268, 20996, 8408,  28724, 18500, 58592, 5116,
                39252, 7880,  42256, 58984, 5880,  12820, 35828, 24400, 55600,
                17468, 34140, 46564, 19388, 37856, 25484, 14792, 5888,  51652,
                408,   19940, 62772, 6712,  16172, 47064, 26184, 63344, 17036,
                21172, 4804,  48168, 24060, 41460, 55548, 7864,  14520, 26300,
                56940, 340,
            },
            {
                37796, 14280, 45376, 61908, 48880, 38156, 55984, 46848, 27440,
                51556, 8184,  58796, 41272, 2792,  35380, 22184, 64400, 55256,
                46392, 9196,  59872, 27848, 34652, 4180,  57268, 22440, 17480,
                34128, 12572, 1208,  20716, 35336, 62052, 27216, 45200, 16120,
                6452,  61252, 18184, 29308, 64420, 25560, 5596,  45816, 28172,
                23348, 52868, 30968, 57800, 10008, 24456, 40224, 12696, 54508,
                62300, 14744, 7692,  56804, 24660, 19864, 4872,  60676, 8660,
                27036, 3192,  31336, 44532, 10100, 21592, 46204, 39116, 11392,
                16588, 50812, 61672, 11896, 18196, 57672, 32496, 2352,  42340,
                5428,  48452, 53992, 37764, 4464,  63660, 42392, 14468, 52664,
                34084, 50380, 29564, 15836, 47224, 27212, 60928, 4548,  30096,
                38964, 64484, 23440, 50596, 1668,  43832, 56428, 29512, 61896,
                12260, 26416, 36412, 50192, 40312, 59852, 13140, 35812, 41768,
                52412, 28844, 62068, 18608, 1156,  27820, 48864, 53328, 41128,
                10212, 51484,
            },
            {
                42868, 4064,  22204, 35872, 2936,  25348, 12168, 33308, 1704,
                16948, 36180, 23168, 63100, 46736, 57192, 6468,  29372, 15124,
                40684, 23404, 56188, 48136, 38968, 60984, 50436, 36904, 3076,
                55832, 64984, 30584, 47248, 53976, 6696,  13728, 39068, 49768,
                23352, 52808, 10176, 46532, 20428, 14716, 52456, 40388, 16512,
                47788, 52,    35800, 62960, 17048, 49692, 58704, 20660, 30436,
                1448,  41648, 46552, 16832, 62720, 50148, 30056, 37216, 52932,
                18572, 47532, 54844, 23920, 1896,  58308, 17680, 25640, 64524,
                36980, 23880, 45904, 28548, 22096, 62804, 9256,  26884, 55444,
                30776, 9868,  62584, 26280, 50900, 32192, 56312, 312,   27452,
                20852, 10204, 63900, 1128,  33624, 19408, 53244, 48808, 43656,
                11444, 53744, 7964,  30696, 62556, 11068, 48512, 35884, 22812,
                58740, 42508, 55740, 9984,  22040, 3660,  30200, 54252, 568,
                10284, 38600, 56776, 36044, 13216, 64296, 33964, 4972,  16944,
                31548, 20376,
            },
            {
                27100, 58628, 52624, 29424, 18468, 43564, 59596, 7108,  65116,
                45488, 55556, 4416,  10668, 31932, 18904, 43688, 12248, 33524,
                50660, 17660, 848,   13592, 7284,  20536, 11132, 27196, 43284,
                8056,  49340, 40936, 27792, 9480,  43644, 32880, 55772, 300,
                59500, 26564, 39584, 7532,  56064, 35736, 30372, 3412,  63412,
                56748, 12408, 19808, 43584, 27044, 3052,  35324, 7256,  52288,
                23472, 59720, 32536, 27400, 10016, 40992, 13560, 46028, 22980,
                42348, 61728, 15944, 36096, 41084, 33064, 6480,  49352, 1368,
                53668, 8336,  41336, 3516,  48492, 34856, 40132, 51012, 19176,
                58836, 22828, 16504, 40428, 12736, 46608, 23948, 37340, 60352,
                47748, 54988, 25016, 44392, 56920, 39904, 8136,  14816, 21128,
                2872,  27780, 16008, 41216, 33228, 4680,  18280, 40272, 8076,
                46252, 17624, 4480,  33020, 63820, 18716, 44604, 58468, 22824,
                49368, 15008, 6184,  25404, 47256, 9356,  21088, 59696, 36588,
                62976, 48392,
            },
            {
                7376,  33476, 16180, 9036,  56984, 50248, 22592, 40932, 30232,
                19800, 39396, 25584, 50520, 15840, 53828, 39008, 58988, 3480,
                62000, 37212, 30752, 64216, 25788, 41192, 47020, 62892, 53092,
                23580, 19244, 15028, 4008,  58512, 22576, 63504, 19872, 11876,
                36740, 31640, 44268, 62440, 1076,  49276, 59892, 10368, 37140,
                25928, 50860, 40892, 7592,  54648, 47544, 14584, 38936, 45420,
                10812, 19056, 37720, 2540,  54372, 34580, 6700,  65268, 928,
                11720, 29428, 7688,  50440, 63008, 12668, 28976, 42244, 34592,
                19636, 31364, 13936, 59188, 54524, 15896, 6784,  45296, 13428,
                604,   36068, 44396, 53004, 2676,  20064, 9304,  16252, 43536,
                6228,  13080, 18128, 36916, 3832,  23392, 63476, 32784, 58636,
                36540, 61196, 45112, 55804, 21264, 60008, 52416, 27604, 2160,
                31260, 14048, 53140, 24424, 47928, 38340, 11480, 26980, 33940,
                65308, 31348, 44284, 51896, 58492, 30620, 42764, 46164, 23844,
                1892,  12512,
            },
            {
                61100, 40816, 47548, 64152, 38860, 692,   14900, 53724, 9644,
                49364, 13128, 34080, 60932, 28324, 120,   23572, 48016, 26728,
                7744,  21668, 45700, 53808, 33392, 16284, 2128,  30220, 13516,
                39336, 33352, 62416, 51992, 35928, 16340, 48812, 29400, 5944,
                46936, 15748, 21316, 13384, 33784, 24268, 17772, 44548, 21412,
                32816, 4116,  61868, 29368, 22244, 60040, 31844, 63972, 26536,
                57124, 49732, 6540,  61036, 47444, 20476, 58268, 25964, 32748,
                55632, 40064, 4036,  56580, 19416, 45284, 60064, 15140, 55468,
                60956, 47092, 25784, 37672, 5148,  29724, 24508, 65280, 38164,
                47824, 61060, 5896,  29240, 62016, 34912, 50224, 65052, 29076,
                33152, 40452, 62452, 30376, 50236, 12276, 28740, 41928, 51676,
                24604, 47632, 6444,  12716, 26012, 37588, 15108, 55440, 63024,
                49776, 35472, 60288, 28420, 1800,  56556, 51500, 5164,  17336,
                8140,  40108, 20356, 1912,  16788, 37988, 4152,  14160, 56720,
                29000, 54276,
            },
            {
                19220, 24388, 4960,  12812, 31568, 25888, 35316, 60492, 27284,
                3012,  57432, 42352, 5140,  45656, 36612, 65020, 9928,  52800,
                42260, 58060, 11348, 4472,  43108, 59116, 55520, 35660, 6344,
                59724, 696,   45864, 11632, 24836, 42596, 2288,  40028, 54884,
                65412, 52152, 4096,  57712, 28528, 42720, 6236,  53848, 58752,
                8700,  46480, 15680, 34788, 11460, 1212,  42760, 17804, 4056,
                34672, 13644, 29264, 42924, 24088, 15312, 38944, 51320, 17560,
                48780, 21740, 34792, 26688, 9284,  24324, 51512, 5512,  22652,
                10040, 2928,  63876, 52280, 20696, 43096, 55808, 10364, 31524,
                26576, 20464, 33384, 11544, 56548, 25468, 39968, 3936,  57676,
                22128, 1980,  53008, 9392,  46396, 60412, 6008,  17580, 1648,
                10240, 19264, 34616, 63968, 700,   44180, 9568,  24024, 42228,
                19316, 10780, 39456, 43804, 8776,  15168, 36304, 42644, 59464,
                46880, 55364, 28264, 63244, 11004, 54080, 27124, 50376, 8564,
                39216, 44560,
            },
            {
                2488,  37044, 53420, 20612, 46260, 62292, 6124,  44384, 37948,
                16488, 63976, 21364, 9092,  56152, 14052, 20384, 31748, 16004,
                35012, 19104, 28620, 51088, 24044, 8744,  18832, 22144, 50556,
                26036, 55072, 29516, 38400, 56200, 8104,  61408, 26016, 34380,
                9040,  23976, 41200, 50404, 11012, 38300, 64680, 14172, 27852,
                39324, 19000, 52668, 57844, 37944, 50296, 24856, 9260,  53660,
                40664, 21788, 64468, 52596, 10536, 31176, 3268,  44360, 8880,
                60324, 14060, 63872, 47800, 38784, 176,   32172, 37448, 44340,
                27648, 40548, 17764, 11808, 32620, 61744, 1492,  17180, 51816,
                57848, 8524,  54384, 42472, 17096, 47604, 7888,  14156, 51972,
                45808, 26440, 59348, 16400, 21548, 34480, 55396, 44716, 38228,
                57896, 31500, 54124, 39976, 50496, 30140, 47884, 33636, 4136,
                57720, 22720, 6240,  64380, 20128, 32168, 62364, 21576, 25440,
                12568, 4408,  35384, 23236, 41724, 60528, 19832, 33680, 64836,
                15940, 31012,
            },
            {
                49156, 58756, 28184, 42000, 55828, 10612, 18096, 22116, 52556,
                32872, 46640, 30480, 24824, 51472, 40788, 5956,  60208, 46212,
                1768,  39528, 63376, 14892, 37812, 48196, 65348, 40212, 45044,
                9788,  15676, 20652, 4988,  32240, 18176, 50720, 13252, 45064,
                17280, 30452, 36124, 60072, 18504, 47212, 2948,  31136, 49612,
                1376,  63844, 25644, 4664,  44748, 19836, 61604, 30084, 58676,
                45824, 1640,  16996, 33532, 5036,  55392, 63148, 28128, 35868,
                1728,  43124, 30476, 17012, 53948, 58516, 12132, 65184, 16556,
                57544, 34900, 49772, 45028, 8032,  36644, 48652, 22636, 41116,
                3316,  15224, 38528, 22992, 1744,  59440, 31672, 37620, 18496,
                11356, 35548, 42604, 4824,  39368, 25292, 13596, 64132, 26768,
                49020, 14776, 5364,  23044, 17120, 59456, 6988,  65408, 13572,
                29460, 47244, 51080, 26032, 53816, 45968, 28916, 904,   38804,
                51096, 61504, 14696, 49216, 32012, 7192,  46588, 1108,  22180,
                52300, 10736,
            },
            {
                62512, 14376, 8344,  33652, 3496,  51168, 29996, 58072, 1284,
                12192, 7300,  38740, 17592, 62688, 34528, 29324, 54228, 24976,
                49676, 6924,  55408, 31392, 452,   27380, 12752, 3456,  32456,
                61632, 42252, 49992, 64568, 46500, 57892, 28672, 21700, 3528,
                60832, 56220, 1488,  7632,  26308, 53288, 22080, 35668, 56564,
                23336, 41596, 9688,  31656, 13976, 54908, 6764,  36500, 12616,
                51124, 27312, 37844, 48884, 60140, 40724, 12240, 18720, 53432,
                24512, 57332, 11040, 4488,  42128, 28468, 19876, 48600, 3928,
                53088, 30440, 1960,  23924, 58556, 27412, 12776, 59792, 34324,
                46120, 64360, 50456, 28812, 45160, 63056, 21396, 27600, 55844,
                61876, 7380,  31028, 57108, 49596, 328,   52480, 30660, 8568,
                21236, 41444, 62712, 28532, 11536, 36736, 20724, 51976, 41068,
                36212, 236,   16248, 33796, 11984, 3948,  57912, 10208, 54684,
                18836, 30588, 43392, 3168,  57688, 17784, 36336, 40868, 57168,
                25628, 35420,
            },
            {
                18376, 45692, 23584, 65096, 16244, 36768, 40472, 48140, 25608,
                61392, 54864, 49136, 1952,  44844, 11232, 3220,  42896, 12844,
                61904, 22732, 10400, 46840, 60388, 34868, 51796, 57364, 17648,
                27904, 6792,  34920, 23816, 1568,  10528, 36660, 54048, 39300,
                48736, 12208, 21044, 45564, 32656, 62332, 9592,  16732, 44128,
                12416, 33688, 48800, 62520, 28364, 40432, 47856, 16396, 23532,
                5460,  62204, 9096,  19852, 25696, 46012, 22684, 48412, 37896,
                7092,  32600, 46340, 22896, 62188, 7268,  33728, 39980, 25460,
                8732,  14476, 63084, 19716, 51432, 40228, 4704,  29616, 18636,
                7028,  25084, 12400, 35480, 4740,  10704, 51144, 43776, 860,
                48248, 24080, 19524, 64880, 10392, 43884, 18252, 36848, 3920,
                60828, 44952, 2980,  52260, 57048, 46168, 1884,  26972, 17676,
                61708, 56592, 42492, 59920, 38176, 18284, 49464, 34460, 44524,
                6392,  26340, 37064, 9468,  53072, 25160, 13284, 61472, 5676,
                43020, 3852,
            },
            {
                54592, 39560, 380,   49944, 27224, 59384, 13624, 8436,  43472,
                35228, 20992, 14392, 58320, 26984, 22328, 59308, 18708, 38192,
                28224, 34104, 41584, 20328, 16972, 43044, 5636,  23424, 38104,
                53492, 12152, 58928, 39776, 15052, 43800, 63296, 6328,  32804,
                42404, 26804, 64384, 37932, 14020, 42112, 57528, 4796,  52580,
                60336, 6828,  18000, 56356, 22100, 504,   34116, 65344, 43344,
                32508, 39920, 57772, 13836, 272,   34776, 5612,  59352, 15184,
                64952, 50864, 19392, 35448, 52256, 13948, 45540, 55912, 21584,
                60004, 43292, 38580, 6252,  56516, 15780, 47136, 62928, 54168,
                42928, 31428, 59208, 55936, 19456, 41020, 33096, 6280,  16192,
                39696, 52688, 13396, 37668, 27588, 32624, 59564, 23724, 54540,
                12292, 31980, 35276, 24960, 8872,  42728, 32836, 54788, 24140,
                7648,  10660, 30680, 5356,  23748, 63368, 41304, 14896, 22904,
                64660, 58608, 47456, 21324, 63768, 29904, 50144, 10088, 28340,
                48236, 32324,
            },
            {
                11764, 29492, 57564, 20292, 44940, 5936,  24020, 63648, 18652,
                28728, 5348,  41972, 31504, 53336, 36424, 47832, 7564,  56284,
                15532, 2268,  52652, 58412, 26204, 8544,  62800, 30116, 45704,
                1020,  47728, 18972, 26860, 52012, 31352, 19588, 24352, 15728,
                9500,  52608, 17792, 50308, 6156,  24484, 29220, 39884, 20380,
                27772, 46368, 38564, 4248,  50796, 11624, 26572, 56840, 2760,
                20844, 54380, 28816, 44008, 52396, 30792, 56388, 9740,  29012,
                41572, 26248, 2180,  60612, 38240, 27104, 1296,  62808, 11552,
                35788, 47412, 26424, 31944, 10296, 35408, 24540, 2308,  37940,
                9948,  21444, 136,   47696, 15328, 53692, 25624, 57952, 63356,
                29288, 34240, 2548,  47316, 54832, 15292, 6220,  46388, 38836,
                50568, 16080, 58876, 19472, 39044, 15116, 59836, 12140, 49372,
                39388, 45212, 20296, 52552, 47044, 27500, 7912,  31108, 50444,
                2068,  11836, 40184, 15644, 148,   44200, 38584, 53924, 16620,
                20904, 59844,
            },
            {
                24876, 6868,  42328, 10328, 35796, 54140, 2664,  33008, 51788,
                56976, 37920, 65396, 9156,  16332, 4324,  63484, 32180, 50856,
                44644, 64636, 4956,  37036, 13136, 49036, 55100, 20524, 14532,
                63784, 33104, 56080, 7752,  60236, 2876,  48468, 55308, 61900,
                204,   29908, 34976, 56028, 2672,  59632, 48996, 36516, 760,
                65008, 24216, 15088, 35292, 43784, 60656, 18380, 51868, 10028,
                46160, 15500, 6576,  36448, 64416, 17124, 39376, 21432, 45076,
                4820,  12612, 55324, 16352, 9372,  49312, 18588, 31584, 5484,
                51164, 17364, 464,   53832, 64312, 44776, 20700, 50092, 13872,
                61456, 51648, 39100, 27760, 64092, 8500,  36652, 12436, 21004,
                43040, 8900,  61284, 22584, 4088,  40728, 62176, 9812,  19828,
                26912, 1716,  48212, 6820,  63492, 28772, 47524, 5468,  62880,
                34560, 28068, 15036, 56076, 1472,  12948, 61220, 37512, 56476,
                17500, 33312, 28084, 55496, 34744, 5296,  22672, 32824, 7652,
                64560, 38248,
            },
            {
                47612, 52968, 33580, 17284, 60744, 29932, 40044, 47100, 16040,
                11268, 788,   50084, 23300, 45868, 40464, 12480, 26116, 21048,
                10084, 30036, 24076, 46400, 32236, 2524,  40072, 35108, 9580,
                42700, 4188,  21884, 36920, 41556, 13456, 28396, 44728, 38160,
                58776, 46796, 7516,  22324, 43780, 31984, 16264, 12680, 51356,
                32700, 9792,  53344, 59144, 29624, 7764,  41060, 37392, 30868,
                25228, 49412, 60948, 10864, 23868, 49756, 2544,  61260, 53876,
                34600, 48032, 39836, 29656, 58972, 43372, 24192, 54300, 41456,
                28288, 58764, 22820, 13172, 40844, 28000, 7836,  56180, 33760,
                26284, 17244, 44196, 5648,  31876, 22816, 45556, 49828, 4924,
                54044, 26172, 45036, 58520, 18916, 51100, 25132, 33976, 56908,
                65156, 29744, 43732, 55888, 22632, 2656,  31696, 18524, 22160,
                2196,  50924, 64940, 36616, 32900, 43972, 21060, 4568,  25488,
                42448, 52744, 7048,  60964, 49280, 19064, 62448, 56024, 45156,
                1308,  14916,
            },
            {
                58076, 4156,  63556, 22940, 50676, 14292, 8976,  62308, 22000,
                44572, 25972, 34048, 58716, 29588, 18884, 55352, 320,   35304,
                60344, 40644, 18352, 56096, 61204, 27628, 16752, 51596, 59896,
                24920, 29084, 50036, 16864, 65208, 9888,  35256, 4672,  21220,
                11936, 25344, 40920, 63852, 11024, 19812, 61984, 7252,  44704,
                57696, 41356, 2440,  20272, 24944, 47024, 14692, 63512, 5224,
                58296, 33880, 19052, 4108,  27668, 42648, 13056, 31872, 17816,
                24064, 62656, 20924, 6568,  33324, 3640,  64876, 36780, 15468,
                8504,  34320, 61864, 49192, 3880,  16952, 60300, 30520, 46580,
                4768,  35960, 57736, 13020, 60872, 40640, 2732,  59492, 18412,
                38644, 14772, 31848, 11756, 36208, 29432, 44112, 13808, 5500,
                40568, 9080,  36492, 13356, 51328, 41968, 37500, 53892, 43396,
                57576, 6936,  24724, 9992,  17768, 58204, 53260, 48188, 10108,
                59756, 20092, 45924, 24412, 10696, 41924, 13720, 26612, 51208,
                35312, 30800,
            },
            {
                19160, 12528, 27960, 46344, 1028,  56808, 36572, 27748, 3484,
                54804, 60916, 13764, 6780,  48552, 37112, 61732, 43516, 49288,
                5396,  14608, 52368, 6756,  10800, 42936, 23120, 6028,  57032,
                38744, 11080, 54580, 44320, 32388, 23284, 57752, 50696, 17212,
                33120, 53672, 15096, 28136, 37712, 46584, 54856, 27092, 22604,
                17464, 30496, 48624, 12520, 62380, 33492, 1680,  23036, 53292,
                12936, 41960, 38636, 54664, 46908, 59980, 8312,  37312, 50716,
                880,   10524, 56496, 46248, 52680, 13276, 21240, 46676, 2228,
                56944, 44368, 19996, 38144, 32460, 52516, 42072, 984,   19600,
                65384, 9404,  22156, 52860, 47108, 16844, 26904, 30540, 35320,
                64628, 1380,  55976, 49628, 7636,  63904, 1088,  52884, 47268,
                21904, 59536, 17712, 33164, 25340, 10424, 60524, 15752, 26640,
                12772, 38372, 47424, 41816, 5052,  28332, 39348, 15760, 34112,
                29784, 2812,  14012, 37380, 31532, 57260, 2404,  39620, 10312,
                23528, 43280,
            },
            {
                51900, 40096, 36956, 7940,  42088, 25120, 19100, 49556, 41452,
                31456, 38408, 20164, 52692, 2756,  10556, 22192, 15992, 28380,
                58220, 38548, 45196, 20880, 34420, 50624, 64760, 1452,  31556,
                46288, 14936, 61020, 620,   5876,  53032, 26304, 42408, 63204,
                2424,  48220, 60608, 4844,  51464, 1220,  34208, 39988, 3796,
                63560, 35608, 54244, 5344,  38476, 56000, 49976, 19500, 44172,
                28888, 688,   63132, 21868, 30488, 15876, 55840, 26056, 64236,
                44132, 28256, 37652, 17424, 25476, 40356, 10152, 60764, 50832,
                30668, 12300, 5792,  26156, 9384,  58132, 23604, 11860, 38792,
                54456, 42588, 29728, 1288,  34292, 7800,  56480, 51340, 10624,
                47840, 24260, 41936, 17672, 39864, 22376, 55576, 16508, 31560,
                26328, 49580, 84,    54632, 64044, 3572,  48816, 7932,  33544,
                62388, 29772, 19780, 61352, 49736, 23172, 63072, 800,   44740,
                65292, 39960, 51528, 61844, 4900,  47628, 28980, 18360, 59380,
                5416,  62124,
            },
            {
                1820,  21500, 55796, 16168, 32396, 64244, 11440, 59004, 5624,
                17164, 8488,  43720, 27932, 63068, 42164, 32360, 52008, 8676,
                25328, 32808, 1260,  62112, 28932, 14144, 47208, 18912, 36360,
                7212,  21016, 26832, 39680, 18444, 47156, 14256, 7432,  39316,
                30252, 9432,  35892, 18556, 58036, 24884, 10164, 59420, 47956,
                14080, 8644,  25864, 44588, 16884, 28204, 7160,  36564, 59676,
                9520,  17644, 49028, 7472,  34996, 3308,  40904, 20628, 5912,
                34012, 15060, 4032,  62012, 31164, 58396, 34992, 27612, 18168,
                24472, 39572, 54420, 64720, 47784, 14508, 36060, 62100, 48700,
                15308, 24448, 49936, 38064, 20264, 63888, 13964, 42980, 21864,
                6708,  29524, 60680, 4352,  27756, 11164, 34288, 38500, 4512,
                63092, 11752, 44900, 40020, 20920, 28072, 57352, 19512, 45168,
                51984, 488,   54872, 14528, 31520, 8764,  35880, 12688, 54668,
                22428, 8296,  27384, 16828, 21532, 53372, 64308, 36392, 46696,
                15624, 29360,
            },
            {
                33700, 9988,  59820, 47376, 52824, 4612,  45148, 34620, 51964,
                24036, 64532, 56820, 34848, 14732, 24676, 55884, 2096,  46128,
                65180, 12224, 23308, 57164, 37368, 7980,  40840, 24476, 53340,
                63032, 49604, 34272, 56308, 30460, 62004, 36804, 58848, 20004,
                54924, 23160, 45220, 41664, 31120, 14752, 20832, 43104, 29004,
                37636, 57000, 21692, 60544, 41412, 10828, 64912, 24740, 47624,
                32724, 55620, 26208, 13672, 52424, 45688, 58808, 11620, 48628,
                53164, 57988, 41204, 50572, 7908,  44,    49384, 42224, 5288,
                63048, 46016, 35084, 2968,  18568, 29748, 45388, 6612,  27744,
                32896, 3392,  58780, 11648, 54656, 25792, 39516, 2052,  62032,
                16216, 45728, 33620, 51824, 57724, 46856, 61664, 42388, 19652,
                56260, 35996, 7784,  30472, 15132, 46628, 35656, 41120, 4564,
                23548, 11016, 34736, 44080, 3124,  60136, 51876, 26876, 17832,
                47684, 32704, 58672, 42268, 35072, 9640,  25000, 6564,  41404,
                56760, 49292,
            },
            {
                65420, 43788, 24384, 3028,  29880, 20664, 38732, 15160, 29132,
                46232, 12116, 3744,  47704, 39648, 6084,  60172, 36868, 19120,
                40860, 53400, 16528, 48096, 3896,  54940, 59540, 10168, 28676,
                43156, 12068, 3320,  45508, 8656,  22264, 11592, 1232,  49916,
                26816, 65036, 12788, 6972,  53784, 48656, 64176, 5860,  51872,
                19236, 48,    50516, 33016, 3560,  52792, 30936, 14884, 40588,
                2688,  61380, 37868, 43504, 65260, 23256, 32332, 17016, 38844,
                29944, 22220, 12028, 19116, 44664, 23516, 14812, 56228, 9544,
                52872, 15684, 21840, 57484, 42800, 51956, 1868,  21300, 56576,
                8720,  63440, 18020, 45996, 5924,  31708, 48728, 34952, 53052,
                58108, 37972, 8916,  20340, 14332, 24548, 2192,  9164,  28276,
                14728, 51576, 24048, 60864, 53076, 5608,  12060, 30932, 64436,
                16908, 59280, 40340, 25896, 56432, 20372, 38832, 42640, 5632,
                56864, 3276,  11960, 49572, 956,   44836, 14424, 32256, 20424,
                12016, 25704,
            },
            {
                7348,  17432, 35644, 13616, 62248, 26316, 8636,  61024, 60,
                54320, 33412, 26900, 21244, 51104, 17460, 11956, 29108, 49860,
                4216,  35004, 30364, 43128, 19892, 26384, 33036, 15964, 2004,
                58084, 17836, 24160, 64496, 41800, 51848, 28636, 34588, 43272,
                15616, 3724,  33340, 59760, 27360, 2080,  36280, 32252, 11360,
                62676, 40284, 27584, 13268, 46752, 18824, 56540, 4520,  21148,
                51188, 11712, 29888, 19460, 2184,  9664,  27832, 61280, 1172,
                8608,  63532, 26680, 55484, 36952, 64412, 32772, 20828, 37804,
                1548,  32224, 11228, 27344, 7972,  33776, 59888, 40976, 52920,
                44120, 35552, 29128, 40436, 61216, 22360, 9548,  18508, 27524,
                12504, 23936, 364,   65332, 44028, 30956, 54068, 48912, 59924,
                44168, 32764, 2472,  18320, 38196, 58872, 25008, 55184, 50212,
                38000, 28436, 48296, 7016,  16132, 46420, 10488, 30312, 62696,
                24688, 38044, 20468, 63268, 29568, 55768, 60560, 50372, 54204,
                2892,  38788,
            },
            {
                31324, 53540, 48580, 40972, 57180, 50316, 43332, 56020, 22456,
                40352, 18564, 62316, 9612,  58356, 31444, 44512, 63168, 21940,
                9776,  59692, 6888,  63676, 11464, 51836, 45480, 62276, 38780,
                30772, 51364, 37308, 14648, 4508,  59960, 17152, 47664, 6720,
                56512, 38356, 50972, 21744, 40180, 17068, 45548, 54376, 25508,
                15912, 44840, 7572,  61816, 23436, 37092, 43380, 59292, 34764,
                45884, 24172, 6684,  58032, 40280, 49620, 36340, 54128, 46300,
                41868, 33628, 48052, 2632,  29584, 11108, 47416, 59592, 28512,
                44012, 61268, 49080, 39856, 62588, 24316, 16728, 11476, 26404,
                19432, 13508, 51120, 4120,  15204, 43864, 59528, 3540,  41396,
                30736, 49816, 55528, 40308, 35852, 6372,  17220, 37116, 22084,
                5548,  40544, 65040, 49212, 9832,  43500, 2924,  14624, 20972,
                9044,  2336,  53928, 22036, 64868, 33400, 3836,  49964, 14284,
                34888, 45984, 53184, 15512, 40344, 22964, 4260,  36968, 27916,
                61636, 45932,
            },
            {
                58536, 1520,  20140, 10720, 6064,  31648, 17728, 13000, 36900,
                7456,  50012, 42840, 1432,  36636, 54552, 3248,  26048, 42584,
                14808, 56032, 27484, 39180, 596,   35532, 22500, 4784,  48532,
                20668, 6916,  55512, 27320, 32888, 39504, 54700, 24492, 62912,
                30120, 19156, 46644, 11020, 61944, 7776,  57436, 22688, 2972,
                59232, 30160, 53556, 34428, 1620,  50188, 9972,  26684, 6016,
                64072, 16672, 52816, 34200, 14380, 56204, 5112,  18760, 25172,
                14604, 4756,  59404, 16960, 53124, 6500,  40060, 3532,  52064,
                17356, 23224, 55344, 3960,  13684, 37204, 49844, 4628,  38376,
                61576, 1664,  23568, 56168, 37152, 26188, 50736, 54840, 63992,
                5532,  45108, 18240, 10740, 27340, 51212, 62384, 11380, 26244,
                57300, 13164, 29176, 21556, 34444, 26776, 48104, 63124, 35224,
                46008, 61324, 32016, 12808, 36972, 58812, 23776, 54812, 19132,
                59644, 9112,  26744, 6048,  34108, 10912, 18244, 43588, 8860,
                16324, 22552,
            },
            {
                42068, 28992, 63596, 38312, 23864, 35000, 2292,  65076, 47860,
                30848, 60308, 15564, 28836, 23768, 46808, 8044,  35584, 52388,
                31972, 46456, 23660, 50336, 17808, 60320, 13916, 54564, 9720,
                42284, 34448, 59332, 46896, 10256, 20896, 2112,  13156, 36388,
                9488,  59200, 416,   25772, 35212, 29860, 13196, 41832, 49616,
                37904, 10244, 19860, 57172, 16464, 63444, 29776, 14240, 54056,
                38728, 31208, 44380, 60704, 26948, 22140, 43092, 10684, 62524,
                50664, 21164, 38152, 43900, 24680, 62224, 19340, 26324, 12320,
                35980, 6812,  30008, 47096, 20352, 54444, 63860, 31496, 45748,
                57936, 33612, 47668, 65112, 7740,  32596, 12180, 20592, 36064,
                15260, 33468, 61640, 22880, 2808,  56436, 41504, 32488, 47736,
                1244,  45576, 54792, 6976,  61924, 16760, 31460, 41520, 6160,
                24544, 17360, 42524, 51236, 1392,  41400, 28788, 7672,  44096,
                500,   31112, 41996, 57348, 47348, 64948, 25992, 56844, 48692,
                34720, 12956,
            },
            {
                5044,  25492, 55244, 15092, 59884, 46432, 53224, 28020, 24704,
                3900,  11568, 52348, 38940, 64712, 13988, 19856, 61300, 17116,
                1812,  37624, 12824, 58048, 44504, 31416, 41236, 28404, 65472,
                25604, 13212, 156,   18276, 50636, 63952, 31004, 45668, 52784,
                22320, 42208, 14764, 55160, 44100, 51680, 20100, 65016, 33288,
                4496,  47352, 42852, 24984, 39172, 6992,  42264, 20368, 48280,
                3788,  10580, 20728, 972,   8164,  47924, 31280, 58360, 35188,
                28640, 54824, 9840,  31808, 14008, 35220, 50764, 45520, 56860,
                65468, 42692, 10280, 58944, 33248, 892,   27968, 9320,  22004,
                6396,  42172, 11064, 27764, 16888, 46380, 39780, 1136,  47740,
                25356, 7640,  58336, 30012, 46292, 15396, 20908, 7824,  63652,
                18756, 35500, 15428, 39436, 51292, 520,   57652, 11884, 52604,
                29476, 56784, 10072, 27032, 19720, 48528, 15612, 63620, 38720,
                52060, 12656, 62008, 17348, 2148,  51092, 38748, 31644, 224,
                63200, 52316,
            },
            {
                46680, 8472,  33320, 49960, 4240,  42180, 9912,  19576, 58544,
                45236, 34676, 21756, 56620, 4760,  32832, 48480, 40036, 55508,
                6188,  64536, 21052, 9008,  2828,  24796, 7264,  37596, 17440,
                52328, 45172, 61428, 23248, 43244, 7596,  57028, 40668, 4992,
                27784, 49704, 32984, 63752, 5592,  2408,  37276, 8956,  17548,
                28284, 62932, 12840, 2720,  49284, 32384, 55452, 60420, 35968,
                25364, 62476, 50540, 41088, 36836, 64588, 16028, 1844,  39448,
                6620,  45344, 64920, 856,   57452, 42016, 8416,  30552, 15916,
                1224,  38876, 26060, 17740, 44956, 41156, 15340, 48732, 55476,
                17936, 30188, 53108, 20872, 55180, 5056,  58504, 29376, 62868,
                53856, 42548, 13288, 50132, 34960, 4652,  59396, 38096, 28476,
                53212, 24568, 59668, 10748, 44812, 23132, 37936, 20124, 60100,
                3924,  44832, 39032, 62876, 5684,  57928, 11448, 35804, 25688,
                20336, 48888, 23612, 37084, 30060, 20820, 13776, 6876,  24092,
                39900, 18928,
            },
            {
                57816, 37120, 44404, 18108, 22256, 30076, 63140, 14760, 40760,
                54392, 6944,  17568, 44344, 9844,  28064, 59108, 11288, 25248,
                29580, 49476, 42240, 33448, 54896, 62788, 47328, 57184, 3132,
                32436, 6244,  39080, 29968, 36668, 26024, 15268, 33904, 18036,
                61152, 7876,  38892, 17416, 21968, 47128, 26528, 59640, 56048,
                23520, 51204, 36236, 58244, 27296, 21888, 11300, 528,   17604,
                46452, 56932, 14992, 29596, 12448, 53908, 23984, 51520, 20008,
                13040, 25656, 17720, 48608, 21760, 4012,  60668, 23740, 34224,
                53096, 20760, 49972, 62288, 7868,  52952, 24756, 35988, 59944,
                13036, 39468, 184,   36284, 60924, 43016, 23832, 17968, 9960,
                37660, 21348, 1652,  39128, 54532, 25568, 44072, 12596, 49636,
                3980,  42336, 30400, 5284,  32956, 55684, 28500, 7780,  48968,
                35984, 22296, 14684, 54220, 34656, 45436, 30816, 53740, 5192,
                59972, 33180, 3284,  55196, 8800,  45592, 58436, 53248, 60700,
                10756, 30176,
            },
            {
                14304, 64368, 952,   11428, 52440, 38824, 7544,  36224, 668,
                31624, 49524, 26548, 62700, 41480, 51424, 716,   21380, 44932,
                35104, 59788, 18392, 27012, 51048, 15176, 19880, 11088, 50020,
                21684, 57980, 15908, 54248, 3436,  11688, 58912, 47184, 2580,
                53952, 24528, 11760, 58424, 31600, 52964, 40656, 14396, 43712,
                6332,  32024, 15620, 8144,  45752, 65460, 52276, 40592, 28928,
                8468,  33240, 22744, 59408, 43340, 4964,  33656, 46100, 61512,
                42096, 57996, 34568, 29364, 53696, 37752, 11868, 48156, 4668,
                59932, 13448, 31384, 3344,  37052, 12076, 64752, 2816,  43116,
                26700, 62852, 50732, 9148,  31420, 13900, 49376, 34756, 6056,
                51492, 27280, 60532, 18900, 10248, 64824, 31188, 17268, 58028,
                9448,  21232, 61648, 49360, 18536, 63580, 13204, 41108, 16532,
                65012, 988,   32304, 8804,  25268, 21536, 2372,  17748, 43012,
                9976,  46548, 15932, 41104, 62628, 27712, 42212, 16688, 35720,
                47908, 21520,
            },
            {
                51304, 27328, 32148, 55712, 60896, 25948, 48376, 56684, 23288,
                61376, 12428, 37860, 2852,  22916, 16224, 36960, 53584, 7616,
                15048, 3784,  11516, 39260, 5588,  36436, 30016, 43224, 35064,
                63732, 27632, 8780,  48064, 63088, 51388, 21720, 29340, 65204,
                36772, 42744, 48380, 28808, 6884,  61872, 10888, 34468, 548,
                53968, 21084, 61496, 41720, 18272, 4780,  34228, 13820, 43808,
                54480, 3208,  39136, 6636,  19184, 27052, 56788, 3148,  30136,
                9052,  50176, 6164,  40512, 15484, 27004, 63856, 19080, 43676,
                27880, 56248, 46404, 22788, 57248, 29284, 19772, 32728, 5696,
                47332, 22884, 15688, 45176, 25372, 2700,  62096, 41148, 56872,
                15556, 45000, 32756, 48424, 6980,  22008, 51604, 92,    33616,
                39840, 47056, 13692, 36484, 26120, 1792,  52244, 46972, 24208,
                29952, 51316, 58732, 47716, 41324, 60908, 50612, 39156, 65456,
                29336, 57864, 26460, 51456, 11932, 5556,  32640, 3732,  25672,
                43048, 2880,
            },
            {
                7428,  40396, 23600, 42836, 13952, 3184,  33708, 16736, 43792,
                29072, 19804, 55292, 58640, 34260, 46436, 64336, 30912, 41040,
                61740, 46720, 52536, 57660, 22336, 45392, 60044, 1584,  24532,
                40956, 12888, 44676, 19968, 32576, 39592, 6204,  44444, 9800,
                19612, 14428, 1460,  56092, 44732, 16408, 24240, 50220, 28584,
                46664, 38536, 11180, 55580, 29548, 37816, 24516, 58612, 20020,
                64104, 26608, 51420, 45356, 63220, 49324, 11600, 17312, 37568,
                23496, 63516, 2576,  55948, 9436,  46276, 32340, 51744, 39896,
                9208,  35008, 5564,  40460, 16204, 44240, 50936, 58588, 10480,
                54784, 34452, 4128,  57804, 37900, 52828, 11148, 21812, 30416,
                64432, 4504,  12700, 55636, 29440, 41876, 36584, 62284, 24152,
                53568, 28076, 2896,  55124, 43612, 9588,  34524, 60648, 6052,
                11220, 38180, 19464, 4776,  16272, 11848, 56404, 7392,  23272,
                13748, 36524, 4572,  19016, 35132, 22224, 64664, 50396, 9780,
                56108, 62108,
            },
            {
                34712, 19168, 59132, 5136,  46544, 20308, 62680, 10820, 52176,
                5752,  47084, 8440,  14368, 28524, 6136,  11940, 18600, 56420,
                23376, 28032, 32028, 304,   64980, 13772, 9232,  54024, 17316,
                4656,  55616, 35992, 1968,  25004, 17108, 55088, 35440, 26840,
                51000, 60112, 33364, 21156, 36376, 3588,  39720, 64692, 19128,
                59868, 3828,  25968, 49076, 1272,  62044, 50820, 6036,  47816,
                12008, 36676, 16308, 9980,  30808, 35700, 41260, 59732, 54012,
                14588, 32660, 20312, 43600, 24864, 58444, 348,   7040,  16988,
                24732, 64488, 11668, 52320, 61304, 632,   24116, 38656, 18140,
                30576, 41544, 20552, 64896, 28312, 18880, 33280, 47148, 672,
                24992, 36260, 40016, 61580, 16288, 47364, 6076,  14420, 44364,
                19240, 7316,  65296, 16568, 38812, 58156, 21056, 31244, 45120,
                57112, 26692, 43416, 63912, 29092, 37344, 26008, 33832, 48712,
                1848,  54484, 45240, 60168, 39656, 56488, 46792, 18372, 38508,
                30876, 15464,
            },
            {
                44996, 49260, 10404, 37572, 29916, 50740, 26868, 41408, 2344,
                64944, 39264, 32036, 52628, 42136, 60000, 25952, 49996, 2736,
                9652,  37556, 16392, 42768, 25744, 33984, 50288, 27976, 46428,
                59008, 30896, 50824, 62028, 42380, 59472, 13224, 448,   57512,
                39880, 5040,  25616, 63776, 51812, 9560,  55304, 31092, 7236,
                13292, 35680, 44328, 22420, 33368, 10120, 16892, 39760, 31668,
                1832,  60628, 21892, 55816, 188,   24496, 7460,  21192, 44916,
                27356, 47872, 38352, 60732, 12692, 35840, 21392, 62360, 55780,
                37388, 48324, 20172, 26968, 33644, 8876,  49112, 14260, 63464,
                7416,  51468, 12304, 47904, 6376,  44076, 59856, 8556,  55092,
                49820, 19200, 9416,  23848, 1804,  57744, 27024, 54712, 10600,
                58772, 35368, 30812, 50776, 24536, 4644,  14524, 40328, 17960,
                2208,  52812, 13540, 33360, 54308, 108,   45940, 18812, 61276,
                41428, 21608, 31432, 8220,  25184, 14360, 29112, 2496,  59804,
                22944, 53572,
            },
            {
                28668, 1700,  65352, 17272, 54292, 8160,  36684, 57120, 21896,
                34700, 17508, 24420, 48764, 20228, 1084,  54640, 39148, 33196,
                63324, 49108, 20300, 55596, 7260,  39956, 61400, 20080, 37828,
                11212, 22468, 15744, 6836,  10228, 29708, 49008, 22888, 31604,
                16156, 47052, 11664, 42304, 18092, 27492, 47760, 22104, 42400,
                57360, 17444, 63380, 52188, 14180, 57940, 46140, 28028, 23252,
                52576, 41944, 33856, 46928, 58888, 14272, 51904, 65248, 5456,
                1200,  10796, 52304, 17248, 30560, 49932, 41976, 28132, 14504,
                2648,  43204, 30648, 4252,  55608, 42608, 36200, 28684, 45468,
                56412, 26312, 36108, 2940,  55644, 14128, 39292, 17256, 27552,
                42424, 59140, 53104, 44808, 34536, 20028, 39168, 32320, 3252,
                47968, 41964, 21704, 11328, 46064, 63336, 28352, 54284, 62104,
                35508, 23072, 8304,  49528, 21276, 62496, 9760,  30424, 52492,
                15660, 10784, 63984, 49204, 544,   53276, 11028, 44136, 35388,
                5368,  12456,
            },
            {
                21140, 57196, 32844, 42020, 24052, 59660, 15164, 31172, 12172,
                45760, 55788, 4164,  61312, 9860,  36120, 45224, 22044, 14056,
                43364, 4580,  60256, 11776, 47592, 15788, 2364,  32348, 6308,
                64344, 40324, 47260, 27336, 53264, 37552, 64464, 45652, 8820,
                61684, 54076, 37292, 7148,  32140, 59020, 14612, 1048,  33928,
                48636, 8980,  30048, 6292,  41264, 20576, 54620, 8228,  62752,
                15128, 4640,  27140, 18540, 38532, 43664, 28156, 31968, 39848,
                57216, 34940, 63152, 6512,  3424,  54936, 8596,  47192, 32780,
                52704, 60020, 17512, 62948, 12348, 20940, 59268, 5576,  1780,
                22664, 61812, 17136, 40880, 31256, 24736, 63548, 35372, 5316,
                13076, 31740, 3372,  28952, 64036, 11656, 51112, 62668, 24696,
                15336, 61172, 1132,  56964, 33744, 8352,  49120, 3620,  10656,
                47392, 41860, 60260, 4440,  38980, 14828, 43032, 59048, 6288,
                38560, 28464, 43460, 17840, 33916, 61788, 40772, 51084, 26488,
                63360, 39744,
            },
            {
                8908,  51636, 26140, 13324, 5848,  44460, 64,    48632, 60972,
                7660,  27884, 40828, 13340, 30544, 63608, 16608, 28856, 7580,
                58528, 26220, 35148, 30300, 53748, 24040, 44780, 56900, 51460,
                25632, 1116,  34904, 57464, 20624, 2788,  17524, 41240, 4420,
                20280, 28648, 23296, 60844, 2832,  45516, 38428, 62624, 53512,
                23828, 39328, 26908, 59192, 36252, 2168,  32484, 43752, 37228,
                57256, 49668, 9344,  53736, 3512,  61700, 8936,  16936, 50332,
                24928, 19452, 28400, 45352, 23432, 39428, 18740, 58880, 23944,
                10952, 6752,  50168, 39064, 46504, 25936, 16072, 53312, 39876,
                33432, 9308,  48908, 58664, 21224, 7208,  51036, 46756, 22496,
                62396, 38292, 15524, 48112, 8000,  41652, 17240, 5584,  37540,
                29480, 52896, 39392, 27008, 18332, 42488, 37336, 21484, 32420,
                25940, 16476, 30640, 57520, 27564, 48128, 24844, 3188,  35556,
                23384, 55436, 3716,  57924, 37536, 21924, 15700, 7832,  19008,
                55972, 46332,
            },
            {
                36456, 61192, 3112,  47808, 34836, 63804, 19032, 38808, 24792,
                52772, 16304, 36708, 57344, 23024, 51248, 3520,  56184, 47016,
                52520, 17612, 41380, 3044,  63964, 37020, 28748, 9000,  14496,
                42756, 18516, 60968, 13096, 44412, 32776, 25216, 54928, 35264,
                50896, 44032, 13520, 48944, 55532, 10456, 25060, 19380, 5856,
                12412, 55968, 3240,  47004, 17192, 63788, 26164, 5160,  11636,
                19476, 29840, 64292, 13316, 33012, 20776, 36788, 46740, 12396,
                59908, 42272, 14440, 56272, 32260, 62120, 13768, 1692,  36600,
                41296, 28864, 21992, 34704, 2504,  31788, 65120, 49660, 19556,
                44556, 13888, 27716, 420,   43620, 54460, 11376, 29228, 1536,
                56012, 51760, 20856, 25444, 59584, 55248, 22908, 45832, 50024,
                8856,  20156, 13220, 6296,  53888, 60900, 14920, 57976, 50868,
                65124, 1428,  36716, 19404, 11292, 53796, 32504, 17912, 50184,
                63184, 13388, 47752, 26088, 6404,  30392, 54768, 47536, 32972,
                852,   14844,
            },
            {
                23452, 18268, 30240, 39412, 55492, 22340, 51032, 28712, 3600,
                43436, 64288, 1400,  47792, 42904, 6748,  34368, 39728, 11488,
                31892, 23356, 9368,  50848, 21332, 5412,  18060, 62532, 33256,
                54892, 49040, 30132, 8432,  50744, 5664,  58092, 14548, 10904,
                63224, 1204,  33996, 18992, 40240, 29752, 35948, 51504, 44160,
                32112, 64572, 21188, 42908, 12704, 50320, 22708, 60496, 48008,
                34464, 45080, 39484, 25064, 48544, 56588, 5204,  23276, 53196,
                2708,  8028,  48724, 36152, 10356, 51948, 26616, 45976, 64600,
                53996, 15608, 58180, 9312,  55192, 41632, 7276,  11420, 30032,
                60920, 37692, 52264, 64596, 32980, 15440, 37316, 60080, 45032,
                33588, 9828,  43496, 4232,  36076, 31896, 1324,  27908, 56472,
                35128, 44596, 64540, 47776, 30616, 2560,  25180, 44452, 5692,
                13200, 53488, 43316, 46488, 6880,  64212, 40804, 56448, 9012,
                44156, 31860, 20388, 10340, 41812, 65136, 3404,  24504, 58568,
                29148, 42900,
            },
            {
                4748,  49468, 57784, 9620,  16348, 6668,  32612, 10924, 59084,
                34180, 21368, 11728, 32220, 18292, 25996, 59340, 20688, 65340,
                552,   37616, 61500, 43476, 13188, 58160, 39356, 46304, 22564,
                11532, 4356,  38568, 23208, 40732, 62576, 28212, 46592, 38984,
                30864, 26076, 57580, 5464,  65272, 15928, 3632,  58824, 27840,
                16548, 49688, 9680,  33864, 54184, 30304, 39020, 15720, 52748,
                2380,  22052, 7112,  60952, 820,   28852, 44580, 63352, 34060,
                30208, 38492, 64768, 704,   21820, 42556, 4832,  34092, 19688,
                6004,  44260, 25512, 61992, 14216, 47896, 24056, 35896, 54600,
                4124,  24440, 7892,  19140, 40276, 4300,  25884, 19816, 7504,
                16868, 27244, 65528, 12880, 53424, 40220, 14924, 63688, 11980,
                4340,  58228, 32752, 22344, 41320, 9856,  35856, 18848, 29508,
                39948, 9200,  61524, 23788, 34564, 812,   13868, 21732, 27872,
                2056,  39500, 53712, 60012, 50364, 13588, 44436, 38392, 11660,
                52352, 63924,
            },
            {
                33956, 12288, 44292, 27640, 62376, 42532, 53324, 46744, 17904,
                8216,  50124, 27388, 54888, 61792, 9524,  48604, 14416, 27536,
                44816, 54412, 15540, 27016, 48648, 31380, 52760, 664,   60316,
                36176, 27676, 65476, 52820, 16328, 34396, 21628, 2272,  59388,
                17700, 8292,  52072, 42808, 21960, 53688, 47700, 8824,  40736,
                1500,  37408, 24820, 59664, 6996,  576,   57056, 9108,  41532,
                26960, 59316, 17340, 42288, 52356, 14248, 38956, 9712,  16296,
                54852, 20044, 26052, 15568, 57956, 29500, 50292, 60500, 11968,
                30960, 49160, 96,    37504, 29196, 20460, 59588, 1944,  17344,
                57684, 46968, 12640, 29680, 56604, 47572, 62296, 50408, 41612,
                57380, 38924, 49336, 29600, 18576, 7728,  60376, 21656, 42888,
                17872, 25840, 400,   15520, 59072, 51724, 62304, 48368, 57128,
                33852, 21000, 28088, 56172, 16604, 51720, 37808, 58652, 47388,
                61892, 16848, 5132,  33504, 27708, 17576, 35096, 61372, 20400,
                8268,  40124,
            },
            {
                59648, 53520, 20108, 37004, 1256,  24284, 13804, 56392, 37232,
                62592, 40380, 4728,  45412, 38088, 2988,  41764, 51524, 35764,
                5900,  59520, 19768, 2416,  34644, 8372,  24432, 16776, 41672,
                7768,  19036, 45528, 1420,  12048, 48348, 9216,  53504, 43448,
                23420, 48228, 35536, 11612, 26604, 32244, 13604, 62828, 20220,
                55172, 61148, 14148, 41140, 27508, 45492, 18484, 32936, 65508,
                13444, 55460, 31840, 10960, 35216, 19332, 57908, 26944, 49900,
                3728,  60748, 46484, 40532, 53020, 8716,  16712, 38664, 24360,
                56084, 40128, 18660, 52640, 5032,  43920, 51408, 39604, 27188,
                42624, 34096, 61560, 51268, 23280, 9696,  34796, 13656, 30964,
                4912,  21808, 212,   58584, 44312, 50784, 33716, 47848, 30212,
                54316, 40720, 50544, 37220, 27396, 12368, 6732,  23360, 1888,
                52484, 14876, 4880,  48988, 31448, 44476, 25748, 7328,  30744,
                11996, 36648, 23952, 45756, 8448,  56968, 23060, 1928,  48768,
                27040, 16460,
            },
            {
                3024,  24888, 7336,  47384, 58784, 31116, 40748, 6080,  22076,
                30044, 15032, 57984, 23184, 12952, 29556, 17332, 57592, 23668,
                9088,  30644, 47556, 40948, 64460, 56060, 12580, 29780, 49436,
                58688, 55332, 24924, 31940, 56852, 60556, 18580, 36164, 4204,
                63744, 15076, 39768, 61760, 140,   44972, 38224, 23940, 34544,
                45912, 29960, 4944,  48744, 20804, 62972, 36704, 23560, 46272,
                5616,  37476, 50992, 46688, 24112, 61768, 6440,  41248, 22124,
                43872, 12100, 31924, 5236,  35080, 59524, 21068, 47236, 3816,
                61520, 9624,  34840, 64284, 11708, 33192, 15412, 63164, 10348,
                21764, 5640,  38100, 1612,  17652, 43888, 2840,  54560, 24232,
                63172, 46668, 11196, 36724, 23096, 26436, 2588,  10304, 37864,
                5732,  63176, 9064,  20256, 55812, 45788, 33076, 16300, 42456,
                63448, 37676, 59000, 41312, 3408,  10648, 61040, 18100, 42528,
                54676, 49356, 63500, 444,   40716, 51632, 31304, 6284,  55916,
                45456, 30612,
            },
            {
                37964, 64320, 32528, 15488, 50428, 4384,  65212, 27020, 45860,
                564,   52364, 19180, 35464, 64556, 53704, 32632, 1544,  63364,
                39548, 12476, 52500, 22520, 4808,  37296, 44316, 62280, 3080,
                32964, 14288, 43052, 6348,  38032, 29184, 42064, 25876, 31440,
                55260, 5964,  28368, 19844, 54780, 60188, 6864,  49984, 4028,
                10908, 18064, 52332, 35260, 8236,  53456, 12064, 2888,  50004,
                28392, 20912, 1444,  8464,  30124, 54112, 2508,  32852, 63812,
                36196, 8244,  56600, 23980, 13432, 43540, 1436,  32992, 27992,
                14660, 45428, 22764, 56928, 25796, 47172, 7620,  31368, 53844,
                49672, 14836, 45616, 31868, 63816, 27844, 59452, 37912, 8576,
                52504, 34076, 14804, 61756, 6020,  56660, 16404, 52612, 61340,
                24016, 14136, 34572, 43976, 3232,  64772, 38828, 29068, 47012,
                7468,  26504, 12092, 19672, 64756, 22656, 52716, 35544, 5380,
                20840, 26676, 14944, 58748, 19756, 11316, 64164, 36036, 41716,
                13008, 51068,
            },
            {
                10032, 55376, 43532, 11468, 21612, 35888, 17436, 10788, 34004,
                60820, 42360, 8628,  47912, 25760, 6844,  43828, 20208, 46596,
                34124, 56648, 17380, 28188, 10264, 50360, 19216, 26336, 39652,
                21204, 9060,  50964, 62832, 22316, 2868,  13812, 51312, 45980,
                11252, 58788, 33136, 43352, 9924,  17368, 30500, 57964, 27184,
                64812, 43424, 23204, 58332, 15600, 42828, 60696, 30892, 56368,
                16880, 40600, 58316, 63980, 15576, 43440, 48716, 12976, 17888,
                52048, 29040, 48404, 19260, 62468, 27048, 49724, 65436, 54416,
                41848, 6120,  30360, 16772, 2608,  38764, 60776, 23500, 796,
                29064, 58792, 20596, 55224, 11056, 41172, 15156, 20260, 44784,
                28904, 17852, 42332, 53596, 30684, 41232, 35520, 44856, 19584,
                28820, 47336, 59916, 30728, 6388,  24772, 18604, 60584, 10332,
                22276, 51024, 35172, 45952, 29316, 38576, 47980, 1588,  57372,
                33072, 9732,  43608, 30340, 37656, 47468, 15960, 25224, 59876,
                19012, 22400,
            },
            {
                264,   40008, 28300, 61356, 52524, 41932, 57852, 47600, 53884,
                24156, 12484, 31032, 3212,  39940, 60200, 11092, 50256, 14848,
                26236, 3992,  43028, 60484, 32072, 58100, 14980, 5788,  47356,
                54164, 36528, 27812, 16932, 46828, 54356, 64732, 7840,  37700,
                17260, 22452, 48684, 3400,  36592, 51516, 22016, 41492, 15332,
                54260, 32472, 1080,  39608, 28104, 4528,  25296, 38572, 9236,
                62716, 44296, 25804, 35724, 20088, 38288, 27704, 57304, 24588,
                252,   59688, 37688, 3932,  53484, 39996, 11244, 7584,  18028,
                36776, 51764, 48548, 59764, 42616, 55000, 19212, 36088, 44612,
                65276, 40260, 8200,  25212, 52184, 33300, 5988,  49716, 57836,
                2212,  25048, 48828, 9140,  19888, 63988, 13244, 4864,  57880,
                1840,  51592, 11344, 41308, 53776, 14312, 50000, 656,   54588,
                31616, 57596, 2224,  16148, 55096, 8204,  14076, 28056, 63836,
                39408, 50688, 3032,  55216, 5472,  28168, 53984, 9056,  3972,
                33420, 62568,
            },
            {
                48952, 17552, 5760,  24032, 2396,  29504, 8444,  20404, 3804,
                38468, 50500, 62140, 55500, 15380, 22288, 37516, 29096, 54744,
                7820,  61936, 35900, 1736,  46228, 24188, 52736, 34688, 64784,
                12056, 60244, 228,   40196, 10948, 34028, 20480, 24860, 61180,
                1580,  40764, 53028, 64280, 25352, 12988, 46800, 1860,  37952,
                8380,  48396, 13104, 63452, 34164, 51156, 19284, 47640, 14140,
                33724, 6088,  12308, 52376, 3104,  10384, 60672, 7464,  45628,
                40372, 15104, 10336, 44600, 31308, 16496, 35860, 25284, 57152,
                21468, 872,   12852, 33992, 9248,  28080, 13612, 48872, 6012,
                17492, 12672, 37000, 4072,  47680, 61080, 22844, 35616, 12404,
                64996, 5152,  59012, 37628, 1044,  27776, 49832, 23512, 32676,
                38484, 17400, 26740, 21976, 46268, 34264, 58428, 40492, 36488,
                13676, 43444, 62776, 5980,  33496, 60076, 23840, 44920, 11260,
                17128, 60512, 24592, 34740, 62732, 22136, 39764, 43748, 57296,
                46536, 26240,
            },
            {
                53652, 34892, 58532, 45320, 37524, 14308, 63064, 31700, 56208,
                15980, 28176, 19784, 44756, 33572, 51980, 5120,  58648, 41252,
                23968, 49016, 19984, 13584, 38676, 8192,  41568, 2220,  28592,
                18212, 23784, 45048, 31272, 59096, 49752, 4104,  44356, 30128,
                55896, 27324, 14636, 8096,  31752, 56104, 61076, 34968, 19748,
                62056, 26132, 21100, 55664, 10472, 44608, 59552, 164,   54624,
                21780, 50300, 31484, 57012, 47272, 22508, 34220, 53272, 31664,
                20948, 64396, 27948, 56140, 22112, 6268,  59160, 45956, 29628,
                61616, 40264, 26844, 63832, 21852, 52548, 3612,  57612, 32796,
                26168, 50872, 56792, 30420, 18172, 468,   27416, 54148, 42844,
                30332, 39792, 32988, 22412, 45384, 60196, 10604, 54464, 43080,
                65244, 8528,  55424, 2768,  63076, 7680,  27436, 20412, 4532,
                48276, 25600, 20952, 40320, 49272, 18204, 41836, 31756, 53524,
                21828, 6940,  45960, 13472, 17880, 49164, 1440,  12620, 29816,
                37468, 14568,
            },
            {
                8520,  31192, 11920, 19620, 54688, 49708, 25256, 44408, 41032,
                6380,  36556, 1480,  9396,  26784, 62840, 18336, 180,   32292,
                10688, 44672, 30256, 63612, 54312, 21540, 58916, 31044, 43904,
                7708,  56764, 52128, 5836,  26456, 15548, 41540, 58168, 12792,
                34980, 46336, 20236, 38544, 44480, 4428,  28292, 11584, 52788,
                42428, 3700,  46084, 30532, 6232,  16268, 23264, 36104, 42188,
                27964, 61780, 17752, 5104,  39928, 65148, 42708, 16744, 4616,
                50984, 35208, 47312, 2864,  62860, 41780, 51008, 2040,  14404,
                8992,  54028, 18768, 5024,  46660, 37384, 41608, 62768, 9848,
                43636, 60464, 22304, 42080, 64052, 45676, 38776, 16364, 6716,
                19548, 51108, 13940, 55340, 16924, 34648, 40088, 6964,  15148,
                29496, 48316, 35340, 39332, 15816, 31180, 11492, 44852, 64440,
                16748, 30468, 9852,  52672, 27088, 200,   56664, 4688,  48568,
                36796, 27488, 41552, 59104, 8172,  52888, 32604, 65440, 18684,
                5308,  61176,
            },
            {
                21228, 40540, 48044, 64744, 7508,  33568, 784,   11692, 59076,
                22608, 64988, 49332, 57376, 41424, 12560, 48020, 36040, 64652,
                15736, 57204, 6432,  50536, 27348, 11572, 16492, 61460, 49980,
                35572, 13784, 38424, 21552, 62492, 8508,  36976, 18668, 51128,
                9824,  5700,  62664, 57456, 17172, 23916, 48956, 6920,  32044,
                58148, 17536, 36924, 60348, 40896, 53320, 31948, 64704, 8752,
                3628,  37424, 45536, 24812, 29416, 13532, 1332,  27000, 55956,
                12544, 8168,  25472, 18860, 33132, 13052, 23856, 38128, 31952,
                48072, 35352, 43928, 58088, 31356, 16236, 23932, 29592, 20420,
                35728, 2500,  7332,  14940, 34476, 11748, 8868,  57276, 62272,
                47036, 26500, 9744,  63428, 6100,  48436, 25536, 19348, 61620,
                352,   23004, 57548, 19692, 60824, 48832, 52540, 24120, 55472,
                7292,  58540, 38200, 61240, 12540, 36460, 65232, 9428,  14960,
                62216, 1056,  55856, 30676, 38140, 26264, 44960, 55612, 23804,
                49836, 42820,
            },
            {
                59120, 25336, 4608,  27924, 15448, 39824, 61108, 18076, 51828,
                34052, 45736, 14908, 31628, 23636, 6984,  44232, 27408, 51580,
                22556, 37096, 18624, 1000,  33648, 47880, 37240, 3476,  20192,
                25732, 63840, 1908,  48124, 33596, 55288, 28652, 752,   61724,
                22676, 30796, 49256, 1404,  36024, 40712, 65072, 21624, 15072,
                50284, 29044, 9220,  24200, 48868, 2456,  12284, 26644, 49592,
                57628, 15228, 10520, 54976, 59276, 19492, 48516, 62048, 37280,
                44088, 60976, 40916, 54264, 48588, 9528,  60660, 53160, 17252,
                64452, 3004,  24336, 12380, 8396,  55624, 384,   51620, 14264,
                47048, 54088, 27876, 50100, 55908, 25092, 48948, 31516, 22268,
                1764,  53332, 43876, 29464, 2332,  58460, 31000, 53024, 46056,
                33552, 13028, 44116, 10172, 5072,  42312, 35964, 1120,  33444,
                41488, 50232, 2776,  32056, 46280, 24700, 20452, 29648, 44040,
                33220, 18732, 50592, 11424, 20932, 3488,  15324, 35016, 10552,
                28780, 1096,
            },
            {
                16580, 51272, 35484, 43836, 56612, 21304, 47140, 26864, 6956,
                29452, 10596, 3988,  53580, 38932, 60504, 20736, 55924, 8540,
                3264,  40248, 60052, 43220, 24556, 56956, 6068,  45780, 54492,
                9820,  42948, 30196, 17708, 12136, 44020, 24164, 47620, 39120,
                53932, 42148, 26276, 13296, 53284, 9968,  58760, 45580, 38652,
                624,   54728, 13492, 64068, 19912, 38696, 55980, 17932, 43684,
                20884, 39432, 33848, 50772, 7920,  36132, 30580, 9704,  23680,
                17484, 30980, 912,   14932, 36188, 28968, 43816, 5600,  21108,
                41884, 28428, 50836, 62564, 39304, 48740, 61096, 6176,  38360,
                64672, 10724, 32208, 39752, 20708, 5276,  59840, 36744, 41464,
                12780, 35068, 17656, 38724, 23656, 42468, 11112, 37748, 4228,
                59244, 50484, 25012, 54004, 29212, 22448, 17472, 60152, 13184,
                28160, 19400, 22872, 15152, 6892,  54500, 40636, 50268, 58416,
                23780, 39476, 5176,  63960, 47988, 60400, 41744, 7200,  62328,
                38688, 54628,
            },
            {
                12264, 61688, 2356,  10672, 52472, 31268, 4828,  36860, 55480,
                43188, 19868, 58632, 26376, 16800, 1900,  34424, 14084, 30836,
                45928, 53268, 28492, 14768, 9476,  65432, 39740, 27940, 32740,
                15860, 56332, 39312, 60276, 52980, 5068,  65200, 14748, 32424,
                3396,  16756, 59272, 34044, 20648, 29720, 3176,  33436, 25832,
                61656, 41684, 35184, 4400,  28316, 45056, 33576, 6640,  61056,
                30336, 992,   63740, 22640, 3996,  44752, 53628, 40404, 58380,
                5916,  50240, 57472, 21916, 63996, 3812,  56728, 26556, 11680,
                58656, 7396,  15136, 33336, 20960, 26212, 34300, 44044, 18672,
                24572, 58080, 16820, 1004,  63436, 44208, 18264, 3420,  28296,
                55156, 7944,  61156, 57024, 50724, 14788, 64504, 21880, 16668,
                27520, 39992, 6840,  37076, 64656, 3368,  56544, 38912, 9508,
                47120, 54116, 64108, 43384, 57708, 33752, 16808, 3664,  12120,
                8060,  54136, 26916, 16412, 32200, 23148, 56752, 52220, 19896,
                46712, 32904,
            },
            {
                44724, 30224, 19124, 38460, 23792, 58724, 13408, 63640, 16208,
                39496, 61972, 46836, 36232, 50760, 62964, 42580, 49516, 59240,
                25272, 11724, 62312, 35356, 51200, 18124, 22480, 12920, 62236,
                50280, 7008,  24628, 3072,  21252, 35192, 9596,  57944, 20320,
                45268, 11100, 64112, 6616,  47644, 43956, 55456, 18792, 12160,
                6412,  22068, 47512, 57932, 52076, 15012, 59064, 10344, 24968,
                47296, 54052, 12876, 42160, 26272, 62640, 15400, 2992,  20444,
                46632, 26760, 38496, 7756,  45784, 18704, 39908, 49948, 33872,
                37372, 54748, 46060, 1916,  42736, 17080, 11384, 57252, 31132,
                4024,  41284, 46244, 51448, 36136, 29796, 14124, 51808, 62648,
                45820, 25368, 32404, 20344, 3772,  36000, 7476,  56216, 43576,
                9700,  62884, 18368, 32512, 46112, 14664, 49784, 30272, 62596,
                24964, 5088,  37208, 10716, 48048, 1732,  63284, 27972, 45152,
                60684, 34248, 42108, 46092, 8672,  36384, 2412,  27368, 14460,
                4224,  24380,
            },
            {
                7436,  57408, 48148, 64124, 8128,  42248, 2660,  49892, 24644,
                8604,  588,   22532, 12252, 6172,  28944, 10452, 18052, 5248,
                33100, 20276, 6588,  47436, 2612,  31488, 53452, 42004, 440,
                20808, 36468, 45340, 29172, 49424, 42036, 27180, 38108, 7220,
                55560, 28744, 37032, 51216, 25088, 15768, 8588,  63468, 49448,
                53388, 29924, 16780, 8388,  25452, 1364,  36368, 50452, 41224,
                5264,  16360, 28576, 48984, 18452, 33056, 11560, 50904, 34964,
                64892, 12832, 32908, 52160, 25156, 31612, 14324, 16,    61988,
                23468, 18828, 29864, 52448, 59964, 6736,  54272, 48116, 27636,
                62184, 8116,  22648, 12044, 26100, 6248,  58720, 38672, 21672,
                16056, 160,   49152, 11844, 46636, 28776, 51936, 33968, 47724,
                30880, 2484,  51572, 57428, 10988, 26400, 43492, 19084, 2520,
                41948, 34376, 17408, 29992, 25932, 39248, 20136, 51692, 37324,
                21648, 13896, 1356,  58256, 51368, 12440, 39964, 31084, 49452,
                64776, 37148,
            },
            {
                22064, 40848, 14176, 26752, 33048, 17528, 46144, 28664, 35052,
                54160, 30556, 52228, 33040, 44988, 56340, 24108, 37264, 64848,
                41872, 57808, 38992, 23752, 43488, 60368, 5228,  35036, 46980,
                58924, 9280,  64264, 13260, 57280, 16416, 52124, 60840, 24768,
                49804, 41352, 22232, 368,   60940, 40996, 37580, 27732, 34636,
                43480, 60476, 39552, 32648, 42656, 62480, 19040, 22752, 65512,
                32876, 37128, 59996, 6896,  55572, 38284, 59016, 24464, 28892,
                42732, 2156,  17040, 61484, 10956, 59444, 53864, 47152, 8668,
                44432, 4588,  65240, 10632, 27236, 38188, 23076, 1192,  13668,
                37180, 53348, 33672, 61188, 54596, 47744, 32120, 10640, 6676,
                43308, 65532, 36812, 59516, 40924, 17300, 25700, 924,   60576,
                24068, 13732, 42032, 21372, 5740,  35076, 55208, 8196,  51892,
                58096, 13860, 61092, 53316, 7976,  59900, 14452, 31312, 4852,
                48300, 64936, 29836, 19628, 24892, 63060, 17456, 59320, 42376,
                10140, 55148,
            },
            {
                29152, 5828,  53440, 152,   50700, 37664, 61264, 21124, 11408,
                58252, 41432, 15276, 63880, 19424, 40216, 3036,  47520, 52336,
                1688,  16344, 54512, 8932,  29232, 15000, 56828, 11432, 26480,
                30748, 54108, 18864, 39672, 6180,  31784, 1180,  12508, 34356,
                4328,  17884, 57788, 13816, 31328, 5816,  56800, 22588, 1304,
                14320, 4620,  20792, 11152, 55116, 46780, 30764, 3872,  12052,
                56456, 20056, 44968, 10044, 30396, 332,   47232, 6192,  53900,
                8952,  56652, 39708, 44264, 5488,  36692, 22300, 29224, 16740,
                56444, 32552, 40608, 14544, 47300, 34852, 62748, 41776, 50852,
                20632, 44760, 15580, 1796,  19328, 41524, 23736, 57084, 50448,
                27144, 30408, 8912,  23448, 5716,  54984, 63756, 12212, 19764,
                53528, 37564, 59708, 28608, 49132, 61588, 23760, 37428, 31492,
                21516, 45452, 1160,  49072, 23456, 42512, 54952, 10124, 57448,
                25556, 7344,  38276, 55516, 44036, 4840,  47028, 21660, 844,
                34144, 16044,
            },
            {
                46088, 63112, 35652, 20456, 58020, 9464,  54900, 6200,  44528,
                18400, 3624,  48328, 8232,  26928, 59532, 14212, 30348, 21568,
                27544, 12468, 49652, 64028, 37728, 19884, 48900, 23460, 40412,
                15968, 2088,  34064, 25432, 61916, 48068, 19848, 45640, 63536,
                30104, 9884,  46096, 35624, 52408, 19624, 47992, 10512, 59376,
                46220, 26872, 64592, 49812, 6532,  13928, 38588, 52756, 43580,
                27108, 1828,  51880, 23684, 63396, 40968, 21504, 16152, 36928,
                62356, 19824, 23372, 28052, 49144, 2952,  42084, 63684, 12384,
                38324, 24868, 50260, 21184, 3768,  57668, 8920,  18344, 32788,
                5520,  59704, 29676, 38060, 9120,  64228, 3688,  34212, 17812,
                40468, 56224, 52696, 14644, 44768, 31068, 38212, 42780, 7960,
                34668, 46308, 4360,  16956, 40056, 480,   12636, 16468, 63780,
                6272,  27496, 38396, 18764, 33128, 4244,  46412, 34716, 18208,
                41292, 53132, 16048, 10644, 34916, 28356, 8252,  53544, 25816,
                50200, 60588,
            },
            {
                3544,  25208, 11372, 43964, 29856, 14948, 25900, 32104, 65400,
                36512, 25100, 60536, 38252, 4696,  53460, 35136, 9292,  57136,
                44800, 36216, 32100, 25424, 944,   44472, 62760, 6688,  52832,
                61388, 44216, 51184, 11052, 43068, 22704, 54304, 39104, 15200,
                42664, 53768, 61536, 26360, 3440,  65368, 42476, 32860, 52096,
                18220, 37820, 56680, 35468, 23540, 28452, 58200, 17520, 8300,
                61772, 40252, 15024, 34752, 49856, 11948, 60692, 45408, 31564,
                13380, 47540, 34628, 53216, 14888, 58064, 19684, 34872, 51888,
                2436,  61628, 7796,  54904, 30916, 15856, 45288, 28692, 64832,
                10700, 25320, 56484, 50080, 44084, 28304, 14976, 46472, 61412,
                12108, 4884,  19572, 33808, 61956, 3360,  21652, 49232, 15712,
                56384, 25588, 10420, 65300, 32588, 44692, 57608, 47948, 41124,
                50916, 10996, 56736, 65132, 12764, 28604, 61368, 22900, 284,
                62240, 44792, 32088, 2216,  60268, 39140, 56916, 32428, 12740,
                37968, 18996,
            },
            {
                42236, 32188, 48920, 59728, 38928, 4484,  48028, 40144, 1040,
                51544, 12900, 29876, 21024, 44048, 49180, 17388, 63320, 40852,
                6928,  61564, 4148,  17452, 55752, 33252, 9896,  28688, 36308,
                4700,  21572, 57468, 3952,  36696, 29280, 9100,  5340,  59180,
                23408, 2020,  20780, 39492, 12268, 29216, 15996, 24728, 40040,
                8688,  30204, 3428,  16064, 42056, 28,    60868, 34612, 24632,
                47812, 29872, 54996, 3340,  19028, 27800, 56440, 3696,  25496,
                51612, 1064,  6960,  64324, 9552,  32312, 25872, 6596,  44060,
                28108, 19152, 41900, 36324, 60364, 25636, 52876, 2636,  55048,
                48364, 40740, 3384,  22784, 12424, 58572, 36504, 53176, 1644,
                22796, 38472, 47496, 26732, 50788, 10620, 58404, 28644, 62436,
                1616,  30568, 50272, 20240, 53700, 26664, 9016,  29320, 3876,
                34172, 23996, 44016, 7388,  51660, 40364, 15868, 49508, 36352,
                12496, 27152, 51060, 23444, 48664, 19948, 15232, 64424, 44880,
                6608,  57680,
            },
            {
                52056, 14668, 1748,  18116, 23672, 52720, 62180, 16904, 22088,
                57292, 45812, 33640, 55588, 10432, 23888, 31688, 24,    51028,
                18924, 23084, 47228, 52208, 13712, 39276, 58828, 18112, 47560,
                13116, 31820, 27120, 48752, 17000, 64408, 57060, 33084, 27392,
                36568, 47876, 7992,  33328, 49880, 55820, 6832,  62572, 2060,
                54528, 22208, 48388, 61932, 52532, 10004, 44908, 50384, 5784,
                20764, 12728, 64200, 37532, 43936, 6652,  33276, 39232, 18072,
                42396, 59412, 30608, 38736, 17608, 54672, 46852, 60280, 10760,
                57096, 48616, 13480, 876,   43960, 11272, 39440, 34352, 21860,
                14152, 35792, 17860, 61828, 32564, 5808,  19928, 25812, 42560,
                31104, 63312, 57560, 772,   41736, 17976, 35160, 6476,  23396,
                44904, 37884, 58284, 7100,  14412, 38632, 22424, 52740, 18956,
                62172, 14996, 30708, 36444, 21384, 2536,  55384, 8696,  30516,
                58920, 19188, 6196,  63728, 12088, 42416, 2956,  51260, 27664,
                21328, 9940,
            },
            {
                29036, 36760, 56156, 64608, 8264,  34284, 11864, 43404, 28796,
                9600,  6572,  16108, 64248, 2860,  41756, 60108, 12972, 26160,
                38260, 11060, 58352, 41828, 27112, 5332,  50780, 22936, 42276,
                65176, 8828,  59504, 40584, 13528, 12,    49872, 18820, 11780,
                52032, 64080, 17132, 44700, 59860, 19320, 36624, 47428, 14068,
                60264, 44352, 11964, 33272, 25420, 19980, 30948, 15180, 37984,
                57508, 32448, 8964,  25332, 59228, 15384, 48460, 65228, 8176,
                55880, 11564, 26892, 45724, 22512, 40696, 660,   36996, 16316,
                23576, 34148, 29552, 63632, 23156, 49052, 19356, 7452,  59112,
                43648, 30712, 7900,  51944, 47104, 39564, 55240, 10236, 49808,
                7492,  16276, 36028, 12984, 24404, 54796, 46612, 39596, 52856,
                11528, 17276, 42432, 34156, 62920, 1984,  60104, 35940, 45604,
                58680, 764,   53936, 47692, 58276, 26856, 43760, 64340, 38752,
                3764,  46240, 40624, 55188, 36288, 30288, 24416, 5748,  35236,
                40268, 62916,
            },
            {
                24340, 5408,  20964, 41348, 45608, 26348, 2552,  54064, 36140,
                61668, 40524, 50644, 19680, 37200, 28424, 53760, 34768, 45380,
                55328, 29976, 33896, 8088,  21200, 63628, 30928, 1340,  55144,
                16240, 37604, 53612, 20072, 45872, 35088, 24668, 44092, 40476,
                4636,  30656, 55264, 24980, 10320, 4140,  43568, 30992, 26696,
                34880, 18676, 41228, 7484,  56960, 39524, 64972, 2296,  53576,
                42740, 4444,  46496, 51584, 22176, 54216, 10980, 29652, 23320,
                36348, 20416, 50116, 4800,  62152, 13004, 51244, 30664, 64716,
                5168,  53564, 8852,  17532, 51932, 4404,  61980, 27952, 46916,
                244,   63944, 55996, 26888, 15192, 2596,  65068, 33472, 60224,
                45208, 28720, 52272, 8768,  32212, 61452, 4208,  14700, 31660,
                64304, 21360, 4632,  25200, 47796, 43756, 16784, 6044,  12144,
                39816, 26192, 9736,  17140, 5312,  33660, 18852, 24648, 14488,
                54188, 22164, 33696, 16720, 8536,  61544, 48036, 59368, 55904,
                16664, 47132,
            },
            {
                60740, 54452, 32444, 12660, 51056, 30276, 60452, 19196, 48932,
                23416, 1576,  26940, 58692, 47596, 14964, 8108,  20968, 4744,
                65284, 1864,  15832, 60252, 44892, 35824, 11796, 46076, 26044,
                34040, 2684,  24180, 5688,  31124, 52708, 62836, 8080,  56344,
                22364, 14580, 736,   41560, 28700, 53172, 22600, 56504, 9288,
                49952, 4684,  63664, 28792, 3760,  46068, 12324, 27156, 23312,
                62548, 17092, 28632, 39800, 644,   35360, 41712, 2820,  53056,
                44420, 63116, 14736, 35296, 56124, 8572,  25260, 19956, 47944,
                38112, 45088, 58960, 40688, 36004, 31196, 55708, 15348, 38256,
                24132, 11904, 37164, 20812, 44648, 29956, 23928, 12896, 18584,
                3096,  21972, 39404, 64604, 44068, 20684, 27768, 59780, 8332,
                48836, 56252, 28960, 51688, 10196, 31576, 55060, 28092, 49800,
                32636, 20768, 42712, 63520, 37924, 60800, 10732, 52168, 48472,
                7448,  29080, 58324, 1352,  26472, 43968, 18384, 11484, 376,
                31412, 7996,
            },
            {
                19044, 38592, 48400, 4172,  16136, 57732, 38436, 5220,  13884,
                31764, 54740, 35232, 11404, 5392,  42776, 62192, 48824, 17100,
                40152, 50252, 24356, 52796, 3064,  19096, 56500, 40300, 7248,
                60304, 50944, 43036, 61940, 10768, 28116, 3172,  16100, 37268,
                60768, 46440, 34696, 62208, 37716, 15408, 64520, 1104,  39152,
                58288, 24288, 52852, 37328, 17208, 50888, 59496, 36032, 49124,
                10576, 34024, 60132, 13132, 19148, 62824, 26104, 59864, 17028,
                6420,  32380, 1508,  41580, 28868, 59068, 33876, 43004, 2716,
                14208, 26600, 20860, 1772,  12800, 25236, 42944, 9692,  52444,
                18612, 49244, 3956,  41152, 9076,  58272, 47772, 36988, 41416,
                54612, 58124, 5172,  48760, 16852, 2028,  51020, 36912, 24948,
                40780, 0,     35612, 61912, 13556, 20944, 41100, 24128, 64740,
                2780,  57232, 51064, 29056, 13468, 46044, 31160, 896,   35176,
                42672, 63252, 10916, 49116, 52712, 39212, 32712, 22432, 53820,
                41980, 13700,
            },
            {
                44464, 1552,  25572, 63524, 35756, 21776, 10200, 42048, 64904,
                46728, 17296, 44324, 22152, 30632, 56708, 25024, 32560, 58860,
                27560, 10444, 42600, 37444, 28600, 61424, 15528, 49700, 21308,
                29192, 12708, 18256, 56144, 39352, 21476, 58300, 48928, 32068,
                26896, 51404, 12220, 20492, 6744,  51040, 33396, 12724, 20892,
                16144, 31736, 46952, 13628, 22524, 32864, 8288,  18692, 41132,
                3444,  21640, 48268, 55024, 5388,  45544, 32716, 13704, 47584,
                39384, 51496, 24084, 18616, 48784, 4220,  16084, 61420, 55404,
                7772,  62992, 32568, 56912, 47276, 65424, 5424,  33388, 61236,
                29640, 57572, 34168, 63256, 53144, 17264, 980,   51768, 27192,
                9796,  25192, 14200, 29812, 34520, 56696, 42620, 12340, 19076,
                46188, 16104, 7496,  39060, 57872, 4092,  60860, 8328,  46236,
                15732, 34944, 7140,  23484, 3680,  54816, 40808, 20636, 56936,
                15844, 25428, 37632, 20980, 13944, 6428,  64992, 46424, 36884,
                26932, 51152,
            },
            {
                33980, 11416, 58832, 28912, 7368,  44976, 55792, 24776, 28516,
                8036,  59328, 3348,  63592, 51728, 38364, 1468,  11944, 36244,
                5864,  21848, 57424, 13392, 47400, 5020,  32732, 9952,  64008,
                38344, 48260, 33140, 7704,  47040, 35560, 12996, 42320, 19048,
                9632,  5444,  43024, 58576, 48340, 26080, 40620, 46576, 60880,
                42964, 7968,  2008,  61416, 56192, 42796, 1068,  54032, 30852,
                56868, 43880, 8024,  37184, 29168, 50792, 8708,  21208, 56712,
                27912, 10896, 61160, 54072, 45132, 12024, 39616, 52232, 22612,
                29736, 49588, 42536, 10500, 17072, 50648, 20656, 39636, 1624,
                45416, 7312,  25724, 13832, 31480, 22548, 61120, 7132,  34864,
                63248, 45708, 37420, 60796, 10744, 23368, 6340,  58548, 32516,
                63264, 54368, 22956, 44828, 17744, 34020, 52156, 37236, 19372,
                11588, 53416, 39036, 48660, 17928, 59680, 8728,  27900, 61440,
                4948,  45444, 32048, 60292, 56376, 29420, 2528,  15436, 58212,
                5536,  63020,
            },
            {
                55700, 23228, 41004, 53044, 18396, 50044, 32492, 1112,  52136,
                37300, 12564, 41180, 26476, 9512,  15884, 20148, 47076, 53404,
                44376, 61884, 31824, 8356,  20396, 53964, 43668, 23896, 516,
                55300, 4272,  26396, 15740, 63864, 936,   25124, 53812, 3964,
                64352, 55720, 24004, 16436, 31472, 9916,  3592,  55320, 28972,
                35628, 53800, 26588, 40080, 10412, 28376, 64120, 24640, 13924,
                61920, 26408, 15696, 64816, 23100, 58312, 42920, 37760, 4560,
                64544, 31152, 7724,  36872, 21752, 26772, 31632, 9816,  36112,
                18120, 39072, 4764,  24452, 37156, 29312, 59768, 14244, 55036,
                22380, 16796, 43240, 50532, 4968,  39320, 43848, 29436, 14988,
                49968, 18908, 648,   53036, 40368, 46708, 29032, 52164, 4044,
                26924, 10056, 50524, 30760, 26444, 48060, 1384,  29444, 43544,
                62864, 30920, 58068, 26596, 44000, 35596, 22808, 47480, 12516,
                39164, 19020, 1932,  8756,  41640, 35268, 50504, 24972, 10216,
                20556, 30244,
            },
            {
                16624, 8584,  46844, 14092, 3292,  39056, 60116, 15320, 20608,
                57356, 32920, 18688, 49512, 34708, 55020, 40648, 64428, 28948,
                18304, 712,   49236, 35196, 64788, 39108, 27372, 58896, 35424,
                19540, 44304, 59968, 52296, 41436, 30624, 59284, 45400, 37800,
                28996, 33176, 39692, 1496,  53360, 63704, 18592, 23772, 5844,
                11580, 65192, 17540, 34484, 48240, 19972, 45776, 37256, 5096,
                49896, 39256, 1412,  32000, 11116, 17668, 2232,  53752, 25592,
                18192, 43388, 15016, 59612, 3128,  56796, 65100, 47216, 20,
                60716, 13168, 54060, 62212, 2744,  44332, 7520,  27128, 47976,
                36544, 62144, 10560, 59548, 36080, 55220, 11500, 20408, 56896,
                6104,  42192, 31776, 21716, 7928,  17800, 62516, 38328, 14856,
                35244, 42812, 2312,  65160, 11824, 55836, 15044, 59164, 25576,
                5604,  21512, 268,   10080, 14676, 65520, 2696,  53356, 34072,
                50348, 64088, 54756, 47032, 23032, 17664, 62084, 44104, 39844,
                48292, 3856,
            },
            {
                43356, 65516, 36808, 30908, 61784, 26268, 11596, 43572, 48620,
                4708,  24244, 62536, 44936, 6644,  60760, 3056,  24524, 7828,
                14680, 39536, 55704, 25852, 16520, 2388,  11280, 51280, 46824,
                13968, 29704, 9556,  22616, 5660,  20212, 10840, 16692, 50816,
                21616, 13620, 60432, 46004, 27856, 43596, 36964, 59060, 49380,
                44696, 20680, 51396, 3824,  58868, 6672,  15776, 52344, 10000,
                34308, 18888, 46464, 53152, 41052, 35428, 61820, 12592, 33296,
                50244, 684,   47464, 33944, 40872, 17428, 6528,  43232, 25080,
                50980, 28144, 45244, 21816, 34760, 56532, 52212, 11296, 32476,
                3848,  40596, 28544, 2240,  17892, 26724, 64276, 46468, 33236,
                24468, 59312, 12552, 65524, 56132, 25752, 1816,  49308, 21404,
                60456, 56628, 19268, 40812, 4860,  38068, 22132, 9020,  50340,
                40400, 46732, 61712, 32400, 51372, 40292, 29808, 17328, 6740,
                23992, 30352, 14840, 27248, 52312, 12012, 7216,  28848, 54660,
                61164, 35436,
            },
            {
                50564, 524,   19164, 56508, 21956, 5852,  35332, 63544, 30144,
                39868, 54120, 428,   14196, 29672, 21604, 32352, 43432, 58376,
                51696, 30600, 22392, 5492,  59424, 42384, 31464, 17564, 7020,
                40652, 63308, 32692, 37872, 49480, 56416, 34832, 2104,  61336,
                7104,  48080, 35180, 4692,  20956, 12040, 32652, 15188, 472,
                38868, 31372, 24748, 13460, 54652, 32136, 39532, 61224, 29532,
                23332, 60148, 57100, 4060,  27672, 48592, 24256, 44688, 58712,
                39700, 22248, 55584, 28320, 9192,  49420, 21028, 34844, 15460,
                55692, 5728,  32760, 9004,  15584, 41256, 18552, 23852, 63316,
                50204, 19544, 57016, 45844, 23292, 49496, 8460,  1456,  53408,
                38900, 3492,  27824, 48216, 33424, 43704, 11628, 31144, 45252,
                6508,  12836, 33348, 24564, 46820, 60328, 31140, 54516, 34604,
                12628, 17200, 55080, 37080, 19908, 7700,  56088, 44620, 59176,
                42168, 10476, 36992, 4432,  39488, 58368, 34152, 2900,  18680,
                13232, 25304,
            },
            {
                53640, 27988, 12164, 48992, 40500, 45916, 54408, 17688, 7480,
                10680, 46804, 27860, 59772, 38052, 51336, 12356, 47892, 35832,
                10076, 62888, 45880, 12500, 48192, 52956, 23048, 62348, 56264,
                25520, 1904,  54288, 12432, 44964, 25768, 65428, 40484, 27616,
                42784, 10300, 18104, 54760, 62756, 7032,  51220, 56568, 28200,
                62540, 7528,  60284, 37008, 43276, 26320, 21288, 168,   55464,
                44428, 7500,  13272, 21040, 63584, 6524,  15560, 8972,  29256,
                5776,  63420, 11952, 24612, 62168, 53500, 30504, 58696, 10352,
                40416, 19444, 60028, 49184, 64548, 30628, 964,   58736, 43428,
                15112, 7896,  30996, 53308, 34364, 14620, 41624, 30064, 62428,
                16684, 45012, 51616, 15068, 5420,  36560, 59248, 17148, 55140,
                39724, 29456, 52284, 62400, 16900, 6932,  42652, 18844, 64196,
                4596,  28120, 43172, 3008,  25384, 61044, 11888, 26716, 1124,
                20244, 62252, 57044, 44924, 21072, 64684, 47320, 23628, 38488,
                8248,  32184,
            },
            {
                21428, 6848,  59044, 32884, 9400,  3000,  28752, 23708, 58388,
                33564, 21216, 16576, 41824, 9240,  56016, 17824, 4452,  27168,
                19700, 2920,  41388, 36416, 27796, 8848,  34432, 3840,  37648,
                20484, 50684, 16524, 59400, 3040,  18236, 31804, 14072, 55176,
                22912, 58248, 50064, 25720, 38208, 41144, 23104, 46408, 9836,
                42040, 16764, 47508, 2340,  9388,  63764, 49428, 12296, 40940,
                16596, 26800, 37732, 43160, 31316, 51352, 55860, 37548, 20548,
                52804, 16536, 36296, 45324, 1712,  13792, 38760, 3260,  63712,
                46520, 36396, 1976,  25820, 11900, 37456, 46976, 5336,  35144,
                26072, 39268, 65088, 12236, 5712,  60620, 37640, 19668, 11264,
                23136, 35956, 9072,  20620, 40244, 53156, 23716, 8640,  64388,
                792,   22196, 48708, 9456,  35836, 27460, 52900, 1016,  38972,
                48408, 22644, 59576, 13876, 50160, 46208, 38440, 32684, 48476,
                35496, 50952, 8052,  32288, 128,   15900, 30512, 51148, 56744,
                60312, 41984,
            },
            {
                62608, 38016, 44336, 24916, 64252, 50104, 14640, 43000, 51196,
                36892, 64860, 49228, 4196,  34280, 23632, 63160, 39096, 53560,
                60024, 33068, 16176, 55028, 20196, 60712, 14832, 44148, 29388,
                47368, 8420,  41752, 28564, 36160, 48352, 8580,  51988, 4380,
                36852, 32468, 708,   29716, 15536, 59288, 3200,  19108, 35092,
                25960, 50968, 22436, 57200, 30156, 18300, 45260, 4336,  32892,
                65504, 52144, 48172, 10128, 34984, 19224, 1216,  46132, 60520,
                32440, 3808,  41680, 51508, 19616, 33712, 44424, 26508, 22012,
                29208, 14372, 57264, 43368, 52848, 20016, 28040, 55164, 10036,
                51600, 20928, 132,   43652, 27740, 56196, 51952, 4500,  47528,
                57588, 60960, 30952, 55232, 63572, 3944,  28768, 50768, 35528,
                26648, 43508, 15364, 3224,  57212, 44788, 13012, 24952, 32544,
                57524, 10148, 30172, 35044, 5480,  21792, 16024, 63216, 4648,
                13980, 28220, 17636, 25416, 53980, 41760, 5724,  10984, 44516,
                2036,  15656,
            },
            {
                47768, 4016,  13312, 17352, 35592, 55960, 20440, 61516, 2260,
                5992,  12908, 54604, 26512, 57992, 45712, 7540,  14064, 42956,
                24320, 49172, 7384,  64376, 280,   39796, 49920, 58372, 11436,
                64864, 33844, 23412, 61116, 5972,  43248, 24436, 62232, 20068,
                46656, 60332, 12968, 45140, 9332,  53012, 31296, 64828, 13400,
                54912, 4928,  33416, 40440, 14560, 53960, 35344, 24816, 57420,
                5772,  30536, 1992,  61144, 24996, 58904, 41888, 12856, 26372,
                48804, 10628, 57632, 30052, 8008,  60904, 55944, 5384,  50652,
                54340, 8156,  32280, 22848, 7036,  62036, 40120, 13476, 61008,
                32848, 57532, 48000, 36656, 16616, 21724, 44712, 26028, 33460,
                7120,  41528, 424,   25144, 17684, 43148, 14036, 46516, 19516,
                11304, 53684, 59344, 37928, 30528, 20348, 49624, 61004, 15652,
                7356,  45764, 52216, 64748, 41044, 57004, 8844,  52104, 23232,
                54960, 40520, 45892, 61448, 12472, 36536, 62844, 27288, 20616,
                35716, 29380,
            },
        },
        {
            {
                28368, 33668, 5536,  53436, 12956, 64844, 28200, 19732, 47888,
                7772,  2352,  22904, 50288, 28020, 480,   62484, 30384, 11016,
                51276, 4836,  44468, 39252, 30120, 23208, 45400, 33080, 20452,
                3784,  27360, 22704, 7228,  57020, 19376, 43472, 49868, 38792,
                21620, 48132, 62700, 34312, 7544,  29296, 64864, 40420, 53204,
                12744, 2328,  43864, 62736, 24412, 55964, 64136, 22624, 48344,
                40928, 13736, 21660, 32472, 17188, 28824, 22736, 13516, 39892,
                3008,  43444, 29164, 62336, 444,   33644, 23096, 48884, 12088,
                33304, 15984, 40368, 8400,  30500, 58016, 45128, 12284, 42876,
                25332, 688,   20468, 45168, 30268, 1780,  49352, 6284,  43708,
                62536, 51036, 4852,  53512, 45020, 19988, 1732,  32096, 23768,
                15776, 35592, 29080, 12876, 60588, 32608, 51536, 23532, 38548,
                1524,  43672, 27884, 9672,  20088, 30548, 25876, 39356, 58076,
                51672, 7436,  12880, 23792, 31508, 49464, 55996, 39060, 64324,
                32512, 2592,
            },
            {
                48264, 63416, 17088, 26444, 21968, 39904, 10100, 59436, 52008,
                39064, 30688, 54868, 43908, 15376, 20432, 42144, 56016, 17184,
                65376, 32684, 58800, 12004, 2476,  51496, 16820, 56448, 60928,
                11932, 53748, 17316, 47800, 33976, 2172,  63784, 30632, 13312,
                27780, 56308, 24776, 10604, 39228, 21332, 1520,  16812, 27752,
                20044, 59368, 31428, 14484, 3300,  33388, 10004, 15464, 53476,
                3536,  60908, 38300, 2100,  50548, 41548, 63260, 52956, 19820,
                59408, 35492, 7248,  11280, 55968, 44168, 41044, 8816,  38104,
                55340, 22068, 53248, 63032, 14232, 49088, 6704,  21624, 62144,
                8832,  54204, 32692, 14788, 60896, 19080, 26016, 59120, 23304,
                13852, 35044, 25388, 12628, 38352, 27864, 49164, 7148,  63084,
                42336, 4308,  46040, 18144, 39824, 8976,  3696,  46280, 53612,
                19156, 60736, 49944, 37296, 55244, 60040, 2060,  10444, 32024,
                15232, 47144, 41040, 65372, 10640, 19760, 5324,  26524, 45744,
                20684, 14876,
            },
            {
                22968, 36276, 55256, 1760,  60440, 34452, 44836, 852,   15076,
                35864, 19124, 61648, 9080,  33156, 48324, 5984,  34696, 46268,
                1432,  37608, 27472, 18836, 62284, 35100, 6348,  25316, 47452,
                37408, 41676, 29588, 61904, 14368, 40132, 23560, 4612,  59964,
                36512, 980,   16196, 51252, 59744, 55468, 35916, 49368, 61972,
                9096,  46996, 54572, 39184, 28192, 47524, 19884, 43480, 59164,
                29684, 18268, 45040, 57300, 12488, 35944, 9184,  31432, 4772,
                26536, 46616, 22156, 49412, 18620, 60772, 27892, 3024,  65480,
                26228, 6144,  48252, 3232,  37200, 24428, 52860, 34408, 39072,
                28400, 48280, 37012, 56664, 7508,  39436, 33444, 46472, 3160,
                40272, 58080, 7524,  55636, 65000, 16828, 58676, 51764, 11812,
                20792, 53632, 25208, 65360, 49904, 21436, 59284, 35360, 29448,
                8016,  14040, 24596, 4648,  46580, 17116, 50324, 23468, 61352,
                36676, 26832, 56720, 3848,  52512, 43464, 61484, 34352, 9720,
                54368, 60072,
            },
            {
                6312,  11348, 30712, 42324, 50664, 7100,  30064, 24524, 63304,
                11392, 27172, 3564,  40080, 59760, 53120, 26116, 13500, 21184,
                53772, 24264, 49992, 7852,  54736, 39864, 43500, 9708,  31156,
                1680,  58900, 5348,  10228, 50656, 54928, 32532, 47000, 17940,
                52492, 41848, 32372, 44828, 3728,  25576, 43580, 6692,  32720,
                41796, 25216, 5408,  18276, 52352, 61916, 7160,  37736, 26664,
                34052, 5152,  52416, 23684, 27148, 5592,  55520, 44096, 16440,
                51172, 57252, 14756, 40108, 31664, 5816,  16224, 50652, 20344,
                58572, 42552, 32272, 27952, 19636, 41584, 64068, 2536,  13964,
                60256, 17788, 23640, 3628,  42820, 51372, 11824, 55108, 28752,
                17816, 48220, 21888, 31608, 42248, 4072,  24728, 37508, 30896,
                57108, 9472,  34232, 2200,  30528, 43128, 16700, 12632, 62372,
                40788, 56044, 31728, 65088, 12144, 34176, 40740, 53856, 5380,
                19184, 1508,  33972, 22280, 38028, 29312, 17648, 13524, 936,
                42080, 38124,
            },
            {
                51860, 46272, 57748, 14388, 20064, 47416, 16956, 53348, 41428,
                49396, 57140, 46660, 24200, 17828, 1940,  63496, 38676, 59092,
                9732,  42344, 15220, 46780, 29028, 21996, 14076, 64608, 52248,
                19440, 23704, 45716, 36280, 26672, 20832, 7584,  43804, 11068,
                65196, 7040,  20308, 61524, 14504, 30008, 11444, 58224, 22696,
                15452, 36284, 60412, 10824, 41412, 23472, 56952, 1564,  11992,
                49668, 65476, 9824,  42088, 61796, 47760, 21376, 60296, 38472,
                10360, 29824, 1928,  64432, 24892, 37044, 54556, 45108, 34500,
                10128, 17308, 12648, 45704, 59492, 9380,  16548, 30248, 51280,
                6468,  45952, 11004, 63396, 27684, 16032, 21192, 64788, 8968,
                36820, 61192, 952,   45560, 9504,  34744, 14312, 46436, 196,
                43732, 39388, 15424, 58248, 6328,  37248, 55480, 26500, 5484,
                48256, 21824, 36104, 736,   43364, 20964, 28804, 14100, 48188,
                43656, 62708, 49956, 15952, 7692,  59836, 47560, 56880, 50292,
                27784, 18400,
            },
            {
                3632,  25248, 39444, 8736,  64148, 36080, 56008, 4292,  21388,
                32828, 5892,  14408, 31468, 36464, 11188, 29604, 44620, 6784,
                31520, 62880, 3948,  34272, 60644, 16,    57212, 26356, 38764,
                15500, 55260, 32100, 64232, 516,   60516, 15852, 57840, 34748,
                24268, 29552, 49272, 9364,  37816, 53080, 18884, 47996, 4188,
                64492, 51216, 29540, 812,   34852, 13644, 31312, 44948, 54844,
                21244, 15760, 36736, 31188, 18728, 14220, 33736, 2692,  24456,
                62984, 42540, 34440, 52304, 9404,  59188, 12464, 29104, 1384,
                39616, 51168, 62096, 54536, 508,   33512, 56192, 43400, 22124,
                57412, 39860, 31160, 49052, 34960, 59688, 1256,  44112, 32216,
                50156, 15028, 27696, 52700, 19508, 62192, 54816, 28848, 18572,
                60088, 26760, 50668, 46984, 23048, 10732, 51696, 33024, 2288,
                44388, 17472, 51184, 26852, 58428, 7628,  56280, 63840, 9536,
                38932, 30116, 12512, 54480, 25256, 40724, 4880,  35840, 23800,
                31736, 62796,
            },
            {
                58752, 33780, 21444, 29216, 2668,  25696, 12048, 61928, 37748,
                9376,  43048, 65136, 51432, 58100, 41832, 49784, 22984, 56364,
                18192, 36652, 51772, 20524, 10480, 44992, 32336, 5212,  47924,
                7948,  42304, 11668, 18312, 40956, 48596, 38320, 27604, 54364,
                1916,  40748, 58448, 22788, 46700, 63864, 34080, 39664, 55064,
                12424, 44100, 20376, 57768, 46692, 50600, 17664, 63252, 39400,
                24712, 47312, 6676,  57668, 188,   40304, 54012, 28596, 49496,
                7068,  13328, 20540, 46016, 3924,  41828, 22348, 47696, 63644,
                24336, 30060, 5072,  22500, 38692, 25684, 47300, 11252, 35952,
                26452, 1804,  20068, 53844, 8744,  25048, 38444, 52836, 5556,
                24576, 41700, 11144, 58852, 39048, 23384, 48992, 8068,  64428,
                5244,  21304, 12580, 61548, 28456, 44716, 63856, 19936, 58008,
                38456, 63068, 10148, 15404, 47252, 38008, 3468,  32668, 25744,
                21404, 6592,  57724, 45244, 32984, 10832, 20380, 64584, 44652,
                12148, 8244,
            },
            {
                48328, 16556, 44020, 60220, 52320, 40716, 31068, 45396, 15748,
                28328, 54496, 22400, 4380,  8344,  20160, 15756, 3036,  27628,
                47752, 11836, 25148, 55472, 40800, 17092, 50456, 35960, 61852,
                28900, 59792, 51148, 6296,  23232, 30308, 12764, 4004,  50252,
                19112, 13388, 35888, 4888,  27392, 260,   8280,  24824, 31080,
                2364,  26988, 38032, 7748,  62288, 26260, 3176,  9324,  29428,
                4080,  60664, 43224, 27936, 50496, 63796, 11628, 45444, 17968,
                37392, 55416, 61276, 27596, 17692, 32552, 56080, 15288, 7572,
                53484, 44264, 36360, 14472, 49472, 7216,  19008, 60456, 5220,
                65288, 44672, 13496, 58524, 4524,  47116, 29768, 13004, 57688,
                20316, 63000, 34396, 5972,  30568, 2776,  12708, 41112, 36012,
                52672, 31400, 38448, 1260,  18096, 40200, 8132,  25204, 13300,
                28736, 6568,  54748, 30728, 60812, 24032, 18060, 44888, 51596,
                59892, 16892, 36400, 192,   62084, 51212, 28336, 2436,  15440,
                53328, 41324,
            },
            {
                692,   55756, 12928, 5572,  47072, 17856, 57436, 80,    49896,
                60368, 18440, 39168, 26872, 45432, 61292, 34432, 52576, 64624,
                39816, 1296,  60124, 30504, 4652,  63328, 22932, 11996, 19912,
                3344,  25084, 33928, 57716, 44884, 53608, 8724,  61048, 43008,
                31464, 63244, 55160, 17516, 42420, 56840, 14884, 45320, 61508,
                18056, 49424, 55656, 15772, 22476, 33820, 43044, 58740, 35612,
                13808, 53064, 33212, 20692, 8284,  25424, 35416, 3968,  58160,
                31952, 23748, 1024,  48692, 39492, 62804, 4936,  35508, 19708,
                60464, 9776,  18408, 58128, 64348, 42360, 29424, 50404, 15932,
                32072, 52236, 37488, 17644, 41956, 22204, 63812, 16868, 40044,
                2220,  54936, 47176, 17708, 51404, 43964, 56500, 25972, 16676,
                47552, 9088,  56284, 34112, 54088, 4756,  48592, 35744, 53556,
                42204, 23356, 34260, 4196,  40660, 52360, 10560, 35208, 2412,
                40176, 48368, 27644, 22944, 42224, 17348, 55800, 39204, 59320,
                22688, 36308,
            },
            {
                26892, 65116, 31648, 37328, 10072, 22236, 34516, 7640,  24552,
                35696, 3236,  48004, 13016, 30448, 55324, 38220, 5900,  13968,
                32872, 44004, 16208, 48644, 38140, 27740, 56612, 43304, 53424,
                46316, 13936, 38092, 2556,  16788, 63704, 36504, 21484, 25656,
                6192,  45856, 9772,  51396, 32296, 60592, 20808, 50516, 35512,
                9468,  42592, 32164, 4768,  40012, 11620, 54144, 19784, 48616,
                40952, 17900, 11148, 56304, 46460, 15684, 59676, 21548, 51636,
                9780,  44212, 15912, 53732, 11152, 24556, 50272, 42792, 27636,
                39188, 32028, 2480,  26700, 33084, 12292, 55008, 1184,  40228,
                23080, 7548,  28488, 61532, 33108, 55344, 10240, 35488, 45844,
                31220, 8676,  25736, 13932, 36684, 59968, 20144, 33140, 1620,
                59236, 23760, 43180, 63484, 14140, 30600, 58796, 16408, 308,
                50488, 59920, 45296, 20140, 13384, 64776, 27984, 55732, 14936,
                63536, 11240, 53212, 4336,  31452, 8436,  47064, 33652, 6600,
                29940, 19104,
            },
            {
                9388,  39936, 23456, 50116, 61716, 27876, 53372, 64276, 43548,
                10652, 57736, 32564, 63728, 1152,  23192, 10836, 26004, 46544,
                58484, 22332, 6372,  54132, 9496,  14588, 2096,  31168, 8332,
                39944, 65224, 21212, 49880, 27432, 32612, 14640, 47896, 39836,
                59160, 15992, 28868, 23504, 37108, 11816, 40456, 5356,  53660,
                23844, 13576, 58904, 64824, 45960, 50984, 28412, 6932,  63764,
                1208,  26624, 62064, 37268, 3456,  39752, 30224, 42916, 6080,
                38980, 60076, 29184, 35896, 7372,  30908, 58908, 200,   12404,
                48980, 56324, 45364, 52016, 4520,  23552, 36672, 20460, 62292,
                56816, 46744, 11136, 25376, 0,     48868, 6748,  26668, 51688,
                61304, 21448, 42512, 65292, 28524, 4668,  10152, 63316, 44764,
                29004, 12132, 3688,  19596, 25888, 45936, 22592, 65168, 11512,
                32040, 17684, 9256,  36644, 49336, 900,   21532, 46988, 5444,
                30572, 20280, 58088, 45600, 14064, 63652, 24944, 11800, 48924,
                62324, 52420,
            },
            {
                46084, 3904,  54876, 15556, 1636,  41856, 13340, 19512, 30056,
                51848, 15252, 40824, 53128, 17460, 43140, 50284, 56716, 19496,
                8444,  29148, 36152, 64284, 24780, 47080, 61136, 35012, 17800,
                58336, 29832, 10396, 55740, 7180,  43328, 872,   56064, 11236,
                33448, 52652, 1360,  43852, 4320,  48608, 26108, 29964, 63944,
                3032,  37456, 28860, 18800, 388,   24492, 15256, 56964, 32076,
                22168, 44736, 6016,  49564, 23820, 64752, 54640, 12268, 27156,
                63132, 19172, 4600,  56420, 47068, 14804, 20984, 53284, 65036,
                6464,  22044, 14932, 62132, 39632, 48000, 8084,  44356, 13120,
                33784, 3872,  42892, 50848, 15140, 40704, 59768, 19044, 3200,
                13324, 38544, 49644, 496,   53656, 22964, 47980, 38788, 14800,
                52280, 37092, 49044, 40764, 51712, 9476,  36808, 41156, 27572,
                47704, 3644,  62908, 56752, 29828, 41972, 59012, 38096, 24304,
                43200, 34140, 7080,  38404, 60024, 35376, 20796, 54500, 2964,
                42108, 13900,
            },
            {
                58112, 20104, 28988, 44604, 36044, 58296, 6128,  48360, 37424,
                21880, 5516,  25076, 9020,  36824, 60636, 4624,  31528, 41532,
                61388, 51992, 17768, 42976, 33616, 20168, 41568, 50672, 25836,
                660,   48380, 35972, 41588, 18944, 51692, 24860, 62944, 4676,
                20664, 38416, 65068, 57048, 18988, 61660, 8060,  16344, 57388,
                20388, 47380, 52272, 6512,  35496, 61128, 38876, 10600, 46728,
                36036, 55392, 29208, 14320, 32848, 18184, 752,   48276, 34428,
                14164, 50068, 41732, 22644, 63520, 38400, 44996, 34096, 17068,
                25420, 37676, 30240, 10024, 18064, 55848, 59992, 30880, 26876,
                53796, 18468, 63748, 57888, 20956, 36936, 31632, 44012, 54020,
                29280, 58344, 7200,  33588, 17052, 41268, 31176, 55036, 5388,
                21780, 62704, 7012,  32368, 60712, 2232,  56780, 6484,  20844,
                54912, 38532, 22792, 26296, 7120,  15948, 32496, 9036,  53516,
                62256, 16300, 26508, 50884, 18552, 1244,  44276, 27556, 37944,
                17600, 32680,
            },
            {
                6856,  34928, 62756, 8424,  24956, 17632, 32436, 55548, 2404,
                59488, 46104, 62332, 28792, 47472, 20940, 34728, 15412, 328,
                38460, 12500, 5096,  50060, 2724,  59140, 10720, 5776,  54584,
                15544, 22832, 4412,  62504, 12472, 58968, 30156, 35588, 17240,
                49764, 27420, 8904,  14244, 31620, 54172, 45116, 33892, 39248,
                43264, 10892, 26192, 55428, 44384, 31048, 20944, 2508,  52060,
                16256, 8728,  59356, 42460, 52964, 9908,  44552, 20552, 57228,
                24788, 1824,  32820, 11952, 26772, 3216,  8900,  29252, 41364,
                47652, 57672, 1540,  43160, 34528, 25568, 15648, 2532,  49504,
                9120,  39144, 35212, 29720, 2756,  11904, 64504, 23464, 15896,
                35992, 10744, 24504, 44712, 61116, 12076, 7728,  58640, 26220,
                34668, 16936, 57076, 28204, 13632, 18524, 33620, 44972, 62172,
                14708, 43116, 51592, 12104, 45944, 61332, 19228, 50092, 13008,
                1492,  56936, 47348, 10316, 29748, 40296, 56536, 5520,  59716,
                23256, 50840,
            },
            {
                26316, 42524, 12716, 47864, 52392, 61264, 39964, 11736, 25936,
                41988, 17136, 33136, 1800,  57200, 12220, 27532, 64936, 47992,
                22712, 26704, 57700, 31256, 13656, 23340, 28984, 64012, 36864,
                45132, 59572, 27804, 33236, 47032, 21572, 8464,  45276, 41100,
                55232, 24056, 47360, 41820, 36052, 22260, 3788,  51876, 13360,
                1376,  31940, 63356, 14852, 9284,  49752, 58464, 41912, 65160,
                24804, 39708, 20204, 2908,  60892, 28052, 36084, 61788, 8072,
                38560, 45548, 53244, 58688, 17812, 51316, 60584, 55368, 4260,
                11384, 61540, 50980, 21104, 64272, 5060,  52592, 41852, 62692,
                22796, 14332, 5628,  24736, 55828, 46508, 8136,  49740, 5020,
                62140, 47336, 19588, 55668, 27580, 37388, 50412, 19128, 2868,
                41952, 45536, 10608, 24308, 39348, 54072, 50260, 29536, 10276,
                1352,  34720, 30892, 4816,  53724, 35124, 3212,  44216, 28568,
                36364, 41252, 22748, 4220,  53008, 65512, 13196, 49712, 30648,
                10512, 63788,
            },
            {
                39216, 56636, 3068,  31192, 21856, 4344,  45328, 29132, 65516,
                8208,  54596, 13696, 38628, 51488, 6456,  43432, 52900, 9072,
                55196, 44460, 62900, 37532, 46588, 53340, 40300, 18952, 32144,
                11420, 40836, 51104, 6188,  15264, 39300, 53628, 1888,  11716,
                61984, 5948,  30332, 2656,  58504, 10460, 28152, 62680, 24544,
                59428, 48996, 22084, 40268, 18140, 6632,  26584, 12804, 29872,
                4048,  34036, 50208, 12640, 38276, 22436, 4972,  51584, 30544,
                16928, 65432, 9816,  36688, 31288, 43244, 20532, 14020, 23672,
                36144, 31744, 7760,  27548, 46200, 12120, 37612, 20132, 32464,
                45284, 59652, 48464, 52740, 17336, 41948, 28128, 38852, 56456,
                32500, 40664, 2256,  51448, 5144,  15300, 64476, 46604, 30344,
                61760, 54508, 1444,  48100, 63532, 3988,  22292, 16204, 59576,
                25512, 56380, 18792, 64124, 40056, 22076, 58056, 25104, 64696,
                6008,  19608, 60604, 33036, 15332, 24676, 7808,  34580, 16504,
                46636, 296,
            },
            {
                53644, 15800, 19132, 59020, 38528, 9624,  15136, 50480, 19828,
                35624, 49144, 22980, 45208, 18616, 59132, 23808, 36220, 16692,
                29900, 3820,  19860, 7284,  16508, 57168, 1772,  49284, 6944,
                61368, 20124, 2620,  56932, 25312, 64780, 34204, 28384, 57588,
                15660, 37372, 19664, 63708, 51012, 17160, 38200, 46972, 19192,
                8532,  36232, 4848,  57892, 33572, 61640, 36880, 47928, 19004,
                56496, 62348, 45768, 26152, 57364, 47772, 15212, 41176, 55824,
                2804,  28352, 21924, 6628,  48028, 764,   39312, 64288, 50160,
                45080, 19140, 14528, 54600, 39792, 58960, 29412, 7332,  56928,
                1104,  27272, 41016, 10112, 32696, 58804, 21748, 728,   18164,
                26184, 13984, 63584, 30684, 21972, 42972, 33964, 23744, 8872,
                13440, 36324, 20444, 31492, 43448, 35616, 7312,  41716, 47344,
                37940, 49652, 8192,  14172, 27456, 48392, 10192, 38928, 14840,
                50700, 30624, 54344, 44496, 37400, 48180, 20356, 42040, 61348,
                37084, 21424,
            },
            {
                44512, 33060, 49996, 27376, 63292, 34464, 24108, 57676, 412,
                30636, 4924,  62652, 27056, 9836,  31776, 40680, 2088,  62392,
                33808, 11672, 41944, 24868, 34752, 9724,  26632, 43756, 14568,
                24768, 54992, 37716, 29912, 43716, 7108,  18752, 51336, 22744,
                32180, 44416, 48976, 12940, 25624, 43212, 6564,  56088, 29772,
                41396, 54580, 27484, 43624, 52328, 23016, 3028,  53996, 44088,
                6072,  15788, 8384,  31044, 1320,  64424, 33688, 11312, 24244,
                49296, 43616, 60796, 15832, 57448, 24516, 33760, 5644,  28340,
                56888, 3520,  60396, 33196, 580,   23288, 50508, 16228, 35500,
                11732, 19304, 65072, 37208, 3764,  13896, 50660, 60652, 10412,
                52992, 43876, 8692,  35456, 59840, 11092, 908,   57472, 49416,
                39448, 26968, 59732, 15600, 9868,  57564, 27328, 64616, 31324,
                3484,  20756, 44472, 60492, 384,   31832, 17512, 55628, 33600,
                46400, 8680,  12336, 600,   61872, 28084, 58788, 3684,  54840,
                25560, 8912,
            },
            {
                64940, 4984,  11072, 42284, 1884,  46872, 53976, 41464, 60504,
                38268, 12508, 42640, 55496, 2904,  61416, 13820, 46976, 21320,
                50720, 59640, 54240, 48460, 65444, 39528, 59252, 30576, 52244,
                35008, 46788, 17172, 10628, 49960, 60984, 13428, 47212, 3496,
                60572, 9484,  54612, 34656, 180,   61176, 32832, 14260, 3320,
                65276, 16512, 11844, 992,   30228, 14024, 39828, 10120, 32544,
                24096, 41360, 36768, 54808, 17732, 42908, 6924,  20048, 59996,
                35088, 12776, 39996, 52056, 27568, 11216, 54688, 17508, 9164,
                38100, 42484, 25132, 48124, 17780, 9452,  42236, 63400, 46448,
                52076, 30444, 6724,  23772, 61340, 44768, 35160, 30184, 46832,
                37688, 22896, 57260, 17292, 46064, 38684, 53272, 29068, 16528,
                4144,  52952, 6532,  49928, 24000, 53492, 18644, 13236, 51976,
                10828, 24460, 57484, 36264, 51532, 41336, 6668,  61920, 2420,
                23816, 58236, 26804, 18156, 40424, 6360,  11360, 32224, 51008,
                13948, 29556,
            },
            {
                47988, 36320, 23776, 52696, 13920, 18308, 6576,  27124, 9200,
                16780, 52052, 32648, 20252, 36596, 49560, 28876, 57612, 5864,
                39176, 27240, 724,   31444, 13456, 18364, 4324,  21180, 62316,
                28,    8128,  63688, 23068, 40712, 1356,  35664, 27000, 38696,
                42096, 5192,  21540, 28724, 40172, 52972, 22872, 49884, 44960,
                34164, 21124, 50912, 46308, 62728, 17492, 57124, 64220, 28176,
                51124, 60304, 21832, 11112, 51988, 24740, 28908, 46032, 53972,
                5784,  18784, 32260, 3324,  63468, 44812, 41304, 49100, 62576,
                22008, 52212, 11520, 65300, 35984, 54024, 26796, 2816,  21712,
                38424, 56164, 43588, 16904, 53360, 26712, 7352,  20016, 64764,
                2728,  6492,  28268, 50128, 3908,  26064, 19468, 61480, 43248,
                22816, 65440, 41128, 33848, 36,    44792, 37056, 4564,  39880,
                61664, 33872, 16744, 28500, 11704, 21652, 44864, 52600, 19292,
                37244, 42680, 63164, 35340, 52004, 56368, 23512, 44000, 18844,
                2572,  58324,
            },
            {
                7344,  19872, 61120, 29108, 57176, 32540, 36852, 21204, 48436,
                44956, 23868, 64668, 7360,  16000, 53832, 24984, 10616, 18396,
                45640, 15204, 22852, 6540,  45024, 55376, 50392, 38084, 12372,
                41408, 27268, 33460, 58616, 31076, 20980, 53868, 59472, 11000,
                24852, 65008, 16116, 56768, 7848,  18856, 11308, 58284, 25496,
                9460,  59948, 37036, 23888, 8100,  48744, 34688, 20536, 456,
                13296, 46356, 5148,  63192, 34968, 48440, 58596, 37872, 64,
                64180, 25652, 47700, 36992, 7956,  19940, 29980, 13616, 34724,
                1740,  16268, 28828, 4512,  44976, 59348, 13860, 33916, 61036,
                4716,  14416, 48848, 32108, 1288,  56900, 11740, 42112, 16012,
                54176, 33420, 40148, 63216, 13728, 55380, 7472,  33284, 11568,
                47024, 30608, 18152, 12532, 62448, 28968, 59432, 22056, 46208,
                54188, 1696,  43272, 5676,  55176, 64344, 26288, 9260,  29916,
                13408, 49800, 3532,  21820, 14512, 30128, 47480, 38636, 62540,
                34488, 41088,
            },
            {
                55692, 15372, 38740, 3252,  44260, 64508, 12016, 55652, 61316,
                29436, 2680,  57316, 38832, 44244, 4136,  34628, 42292, 31920,
                64384, 36904, 52092, 61404, 34348, 28288, 9004,  24136, 46128,
                57720, 54052, 14176, 3920,  48032, 15700, 7840,  44584, 17688,
                52340, 30972, 45708, 36516, 62588, 47632, 31612, 37804, 1500,
                40668, 53300, 5256,  32228, 41752, 26532, 4440,  45136, 38152,
                58408, 18272, 29800, 40028, 2144,  13660, 8920,  16424, 31204,
                41052, 10172, 53256, 14720, 23352, 57940, 2716,  59984, 25988,
                46640, 57156, 32136, 40772, 20092, 7044,  30304, 49848, 10668,
                24856, 28308, 62820, 9524,  39992, 22140, 49216, 36792, 58212,
                24620, 48284, 10792, 20884, 31980, 42172, 47964, 36988, 1596,
                58624, 8264,  37800, 55624, 47948, 15956, 32624, 7992,  26480,
                14216, 30336, 63040, 37860, 47428, 14520, 34148, 39496, 60764,
                56124, 6164,  32864, 45732, 8496,  64816, 1684,  16016, 9532,
                22528, 27072,
            },
            {
                45624, 32128, 51492, 9804,  22344, 49696, 1040,  39384, 6032,
                34808, 15088, 11264, 26708, 48044, 21776, 62868, 55760, 1552,
                8140,  58668, 11132, 20108, 42500, 2452,  63616, 32404, 16864,
                5540,  20448, 43952, 51260, 36700, 64400, 29056, 56864, 33564,
                1628,  49544, 12180, 3416,  26696, 43108, 4932,  16908, 63876,
                27648, 14380, 18960, 56660, 60776, 12504, 52572, 55528, 8756,
                25324, 33880, 50088, 56832, 27176, 21300, 61400, 44296, 23000,
                50048, 57112, 29316, 61836, 46076, 32928, 39240, 51304, 6272,
                37524, 61696, 8512,  53044, 23780, 38704, 56800, 18216, 43260,
                52784, 39324, 19684, 34756, 45476, 63836, 28060, 5704,  31364,
                13560, 44568, 60852, 292,   57568, 24324, 15848, 63580, 51428,
                27264, 20600, 43860, 25128, 2928,  10576, 42492, 51248, 57040,
                19200, 49492, 9416,  23324, 18460, 4104,  50616, 1544,  22176,
                47188, 27296, 17604, 54376, 41480, 25904, 36216, 57860, 49072,
                54036, 12264,
            },
            {
                64240, 4708,  59444, 26396, 41004, 15980, 30492, 24920, 18968,
                52796, 41392, 50248, 31316, 60100, 8600,  13344, 51364, 24180,
                48496, 29492, 40440, 25724, 57404, 15072, 37076, 48632, 60084,
                29612, 39000, 8896,  25748, 11684, 41808, 23644, 5500,  38184,
                20504, 60236, 40468, 23300, 14992, 59028, 51436, 21788, 55412,
                48108, 31064, 45376, 2308,  38948, 22352, 16648, 30456, 43268,
                62572, 3052,  15704, 7732,  42676, 52400, 33336, 54956, 6240,
                13040, 20404, 2236,  42084, 5216,  55372, 18132, 12032, 22708,
                42752, 19328, 13980, 49180, 63724, 3736,  46672, 35568, 116,
                59556, 7652,  57656, 3800,  12904, 17504, 52288, 2248,  61968,
                19216, 7684,  27108, 38120, 51820, 5616,  30352, 21860, 13080,
                56452, 34372, 4824,  61216, 52772, 21348, 64804, 38708, 5924,
                34796, 59832, 40168, 27584, 56772, 31132, 59088, 41984, 15708,
                10500, 64024, 37564, 58508, 12288, 50452, 20488, 5932,  28612,
                42788, 740,
            },
            {
                24388, 17020, 48064, 35684, 6740,  58388, 54332, 47256, 63152,
                8996,  58144, 20712, 500,   17400, 40988, 28316, 37960, 19668,
                16416, 35352, 3168,  46552, 7708,  52640, 22268, 11380, 1284,
                53076, 34908, 63200, 56512, 18824, 760,   61016, 47236, 14240,
                27244, 55020, 7456,  35092, 53540, 29560, 9344,  33556, 42404,
                6640,  10748, 63020, 50560, 28124, 6172,  65044, 36128, 11228,
                51744, 22972, 47264, 37520, 64724, 18652, 3364,  36952, 27852,
                63276, 38968, 34632, 16736, 25824, 9592,  48232, 63544, 30916,
                53564, 784,   34188, 27992, 10260, 15752, 25932, 62644, 12536,
                29744, 23012, 46796, 54148, 26140, 59916, 34020, 43788, 38680,
                50628, 54940, 33792, 17596, 45812, 9840,  59304, 41312, 3196,
                45568, 16632, 49952, 39976, 28092, 36060, 17328, 1392,  24928,
                44288, 12556, 2432,  52552, 44612, 7896,  35652, 24772, 53764,
                44208, 30324, 1020,  23732, 6720,  31628, 44808, 60740, 33432,
                18528, 37368,
            },
            {
                30432, 41744, 11116, 19404, 63680, 32960, 13096, 43100, 2280,
                28240, 45412, 36664, 64188, 55120, 33508, 46732, 2824,  62268,
                43064, 56808, 13648, 50812, 18948, 60600, 27236, 44772, 40904,
                24600, 14784, 3556,  45516, 31768, 54724, 34448, 51768, 9864,
                62532, 43456, 17572, 47148, 64468, 648,   38428, 13476, 61704,
                24664, 36540, 20872, 15200, 34340, 42732, 49068, 1664,  19408,
                40332, 59208, 12672, 25892, 31540, 10516, 49332, 24276, 59476,
                47408, 8324,  44664, 58772, 52180, 28784, 35528, 43968, 7536,
                26280, 56416, 45996, 58888, 42316, 31556, 51928, 20804, 40648,
                49432, 15312, 37068, 31252, 41788, 7064,  23764, 11096, 21356,
                29580, 3360,  42364, 12728, 64044, 35740, 48852, 28004, 37908,
                60684, 9616,  31200, 13768, 7264,  46536, 58600, 30968, 48544,
                62308, 21836, 32788, 16340, 65392, 11616, 20028, 61700, 4804,
                33516, 19380, 52024, 40248, 61408, 16500, 55056, 3112,  13484,
                52724, 61924,
            },
            {
                6196,  57572, 53892, 29036, 45204, 3880,  22032, 37512, 16724,
                33768, 23488, 4988,  13904, 25052, 6344,  59596, 11808, 54192,
                26648, 4992,  64212, 30288, 38780, 34064, 5744,  65052, 18356,
                57224, 49008, 22144, 28284, 40416, 7660,  15856, 21508, 39768,
                30380, 2140,  32600, 10936, 19696, 25600, 45868, 50216, 18540,
                3148,  58532, 43936, 55044, 9028,  59664, 24392, 56552, 28740,
                32708, 5736,  54304, 44484, 4404,  57476, 15368, 40520, 888,
                17912, 31844, 22672, 13416, 65264, 1312,  21452, 15228, 60496,
                37040, 17156, 4780,  21816, 37724, 2068,  55132, 8712,  33648,
                5204,  64900, 18908, 1720,  50584, 14752, 55972, 47976, 65468,
                15540, 60048, 24176, 55900, 20776, 1668,  53700, 18044, 6744,
                24744, 54560, 19888, 63052, 55912, 23576, 11592, 53260, 15276,
                8564,  55664, 41332, 4584,  36948, 49704, 28788, 47824, 39032,
                8780,  60044, 14012, 48508, 34956, 9956,  26216, 39044, 23056,
                8856,  49468,
            },
            {
                14700, 21664, 1528,  38044, 52112, 9432,  27332, 61812, 50400,
                11776, 60392, 47736, 39360, 52892, 43220, 20800, 31376, 7972,
                49172, 33204, 21728, 10372, 43824, 1852,  54760, 29420, 9348,
                32528, 6252,  61764, 12564, 59376, 49480, 26784, 44492, 4424,
                57496, 24328, 50436, 56340, 40892, 60524, 5304,  57028, 28660,
                40292, 52664, 26600, 1272,  30800, 18088, 13228, 45888, 7924,
                61504, 16492, 34964, 20472, 61188, 29228, 45984, 33980, 12312,
                51528, 55612, 5392,  49300, 36784, 40948, 56996, 3976,  49748,
                11124, 40692, 64876, 13132, 48888, 61784, 17656, 45152, 60200,
                26964, 42728, 54968, 9680,  62224, 28800, 35732, 1140,  40620,
                8556,  45392, 36544, 4996,  31560, 25700, 14184, 33344, 65060,
                42720, 47464, 35348, 4396,  41520, 33704, 3424,  42984, 38020,
                28936, 25508, 46960, 19028, 58692, 23736, 60,    13240, 57308,
                26716, 45612, 3580,  28396, 21144, 43540, 62772, 47692, 58276,
                35244, 45008,
            },
            {
                40252, 33172, 61452, 24612, 15524, 47940, 56084, 40140, 7072,
                53604, 30844, 19204, 9896,  28780, 49768, 14976, 37172, 23620,
                40544, 15688, 52848, 58364, 24444, 16880, 49732, 13520, 37276,
                46624, 42060, 52532, 35556, 17652, 2268,  65260, 53280, 36412,
                13760, 63460, 6636,  37128, 15296, 30828, 21560, 35448, 8468,
                16172, 32936, 12444, 47340, 63976, 40480, 52312, 37980, 21840,
                48836, 42424, 324,   50236, 38324, 7440,  52544, 63896, 21512,
                41916, 61528, 29708, 24816, 10468, 18764, 31472, 45388, 23044,
                28244, 51380, 32840, 25220, 7008,  28964, 35812, 24104, 2672,
                51112, 11924, 32340, 24672, 39372, 46172, 17636, 58252, 26960,
                32652, 52348, 11256, 49380, 62548, 40596, 44556, 57808, 10968,
                22212, 372,   28520, 51756, 17928, 26856, 64168, 20728, 60212,
                832,   50756, 63196, 6756,  30564, 54744, 41780, 63904, 17560,
                36340, 22576, 55808, 65124, 6376,  53312, 420,   29948, 19580,
                4184,  27428,
            },
            {
                55304, 11692, 44232, 6888,  59696, 34468, 18252, 2840,  25120,
                43764, 57704, 3832,  62212, 34368, 1268,  65328, 56908, 44120,
                61460, 224,   36456, 7268,  47604, 40244, 62008, 21492, 58456,
                26104, 472,   20428, 9016,  38632, 24756, 31340, 11456, 18976,
                47944, 28156, 22384, 45288, 2520,  52800, 11948, 42128, 65496,
                48956, 59332, 4728,  22572, 35540, 6496,  26272, 2820,  63180,
                13908, 27196, 56352, 23832, 11720, 17376, 25240, 3888,  28360,
                9644,  37892, 2740,  46220, 59980, 53548, 6828,  62444, 34552,
                55816, 8256,  18880, 53840, 43832, 58240, 10812, 14536, 57144,
                37880, 21604, 47840, 57508, 5548,  21136, 10684, 53224, 6412,
                22096, 18456, 59080, 28404, 16624, 8428,  51652, 3076,  30000,
                50364, 39560, 12676, 59352, 9180,  44984, 49596, 6436,  35636,
                17484, 12924, 33320, 38856, 10364, 15608, 46080, 32428, 6988,
                50864, 11224, 42156, 31336, 18296, 37292, 12228, 41664, 15804,
                50916, 63368,
            },
            {
                2372,  17448, 50108, 28392, 41648, 12460, 31244, 64204, 20816,
                35808, 15328, 40752, 23028, 55440, 17820, 27096, 5560,  10664,
                18972, 46020, 26940, 63404, 31100, 11496, 34844, 3768,  51416,
                16388, 63256, 30196, 55536, 48140, 57792, 41372, 7304,  60164,
                33364, 42268, 9900,  61232, 34088, 48384, 27036, 54892, 932,
                23852, 38572, 18916, 50960, 42436, 56868, 11080, 32220, 53812,
                36024, 9448,  30980, 65364, 41132, 54812, 43896, 36312, 58004,
                48700, 19212, 13716, 33224, 42276, 16132, 26840, 39464, 14248,
                2976,  47036, 62784, 636,   39148, 20928, 47956, 41300, 30480,
                6604,  63284, 16356, 3016,  43576, 33672, 63716, 38196, 48984,
                61908, 42764, 556,   38380, 46804, 22908, 60384, 36780, 19260,
                56516, 15796, 62180, 37256, 23104, 56884, 14496, 30460, 54244,
                45520, 58204, 21616, 43648, 52520, 26024, 3548,  21036, 40100,
                54112, 2076,  34356, 14808, 57732, 50024, 24548, 32920, 59516,
                10224, 23920,
            },
            {
                30984, 36016, 65424, 20676, 128,   57204, 45448, 50804, 10532,
                29088, 49028, 8300,  45932, 38216, 12196, 47200, 51292, 34732,
                29652, 54076, 13568, 20616, 2576,  56240, 27844, 44044, 7676,
                39960, 12200, 44560, 23200, 14660, 3380,  45816, 21956, 55028,
                132,   52012, 17024, 55620, 19372, 7952,  62196, 14844, 45588,
                29976, 10092, 62748, 28180, 15360, 60420, 20828, 44520, 17552,
                59424, 47908, 4952,  19252, 33592, 2444,  60976, 8096,  15512,
                53460, 26720, 63116, 56292, 22180, 476,   48484, 10156, 58760,
                20760, 42212, 30784, 15196, 33788, 4436,  64420, 52660, 18776,
                45652, 35364, 27944, 60500, 51868, 29680, 13068, 24960, 3560,
                30516, 35372, 14128, 56960, 5632,  33984, 12972, 26740, 7260,
                44204, 31888, 5248,  46920, 1600,  33040, 39896, 10988, 24316,
                41012, 9056,  27620, 1924,  62000, 34904, 59672, 56400, 29456,
                24952, 61952, 47352, 26548, 8924,  44820, 63740, 4964,  54224,
                45784, 38484,
            },
            {
                57640, 47680, 9032,  53964, 38772, 23424, 5116,  37240, 55352,
                1704,  62996, 26204, 60060, 2788,  31264, 41660, 21932, 58152,
                3704,  60104, 38412, 48912, 41920, 17740, 60232, 23692, 31992,
                54428, 35336, 59856, 5828,  33484, 63940, 28560, 36876, 15488,
                25864, 5352,  38224, 29300, 43692, 24208, 37852, 5472,  20340,
                34700, 53744, 43532, 7596,  33228, 604,   48564, 28924, 4200,
                39540, 22404, 43668, 58384, 14336, 47092, 29640, 21152, 32576,
                44800, 5476,  39268, 7940,  51560, 35716, 64448, 30272, 52504,
                37284, 26136, 9188,  60284, 50212, 27276, 9632,  24716, 1252,
                55720, 8732,  13684, 40852, 18932, 8188,  47440, 56608, 16056,
                45200, 9876,  24480, 52980, 19984, 63232, 40088, 53936, 49232,
                64020, 25136, 20696, 54984, 27840, 16952, 62556, 2792,  51444,
                65020, 5084,  55684, 49248, 13464, 18200, 47600, 8636,  12392,
                44320, 16428, 5844,  60360, 38912, 1660,  21368, 35860, 27700,
                7532,  20368,
            },
            {
                14144, 4724,  25764, 33496, 16420, 61396, 27144, 14308, 42280,
                17728, 52324, 32868, 13792, 19700, 52872, 62668, 8168,  39716,
                16988, 24532, 8592,  32636, 52508, 5056,  46088, 10464, 64392,
                18588, 1912,  49000, 26732, 52896, 19176, 8216,  50028, 62280,
                40488, 47172, 64836, 12604, 3296,  59392, 31796, 41024, 57576,
                47812, 2564,  14052, 55572, 24076, 37644, 52828, 64408, 12984,
                25488, 7872,  51656, 36856, 27080, 10164, 50640, 40020, 64480,
                1276,  24048, 47740, 30928, 11696, 18872, 44052, 23428, 4744,
                16980, 56836, 44824, 22512, 54796, 37772, 16816, 59468, 33328,
                39376, 49356, 25188, 54708, 252,   36888, 59224, 41492, 20248,
                54848, 64832, 49820, 27860, 43888, 3660,  30496, 17544, 1004,
                10596, 35620, 40780, 8376,  52276, 42768, 58376, 21208, 34824,
                29364, 19248, 37192, 31448, 23124, 40876, 4552,  37936, 63144,
                19992, 36632, 50376, 22756, 30076, 56680, 13108, 48572, 16604,
                61468, 42928,
            },
            {
                29460, 60676, 43800, 11884, 51964, 46516, 8520,  58356, 30476,
                22420, 6516,  43924, 36552, 58412, 4968,  25688, 14468, 33192,
                49984, 44776, 12100, 65004, 22224, 36796, 14264, 49852, 38668,
                28640, 21800, 41296, 10772, 37972, 43572, 13208, 58124, 30368,
                10960, 23236, 32644, 56892, 48804, 16360, 51520, 11060, 64140,
                17128, 26292, 39784, 61768, 45680, 18580, 8420,  41476, 34736,
                55508, 62720, 16400, 820,   54140, 24168, 58948, 18160, 12436,
                55512, 35964, 15064, 61376, 58168, 27856, 6452,  40556, 60548,
                49520, 1568,  35192, 13028, 5368,  46620, 30112, 43460, 12476,
                62872, 20084, 31012, 44424, 64500, 22916, 27368, 5124,  33948,
                2348,  38652, 7292,  32616, 11724, 57464, 47496, 23544, 42596,
                58700, 14696, 48376, 61688, 12856, 36428, 25800, 7316,  48060,
                13084, 60784, 46248, 9964,  63880, 51720, 27260, 32736, 53108,
                560,   31172, 54908, 11020, 43088, 18828, 52668, 40708, 31996,
                2748,  51632,
            },
            {
                39856, 18648, 56312, 1996,  30764, 40112, 19460, 64756, 35224,
                47844, 56856, 10820, 23856, 48808, 29480, 45368, 54420, 1812,
                61364, 27728, 56744, 1144,  29932, 54952, 26196, 6608,  57008,
                4092,  52108, 15080, 61152, 56848, 1048,  21536, 34972, 4096,
                18304, 54056, 6816,  20748, 26592, 35656, 1612,  28388, 22616,
                7328,  49744, 30512, 4956,  11712, 57844, 31516, 15904, 2036,
                45172, 28140, 32240, 41888, 61776, 5888,  33956, 3412,  43056,
                28588, 51096, 20180, 41432, 2224,  46560, 55336, 13772, 33372,
                10964, 28952, 64976, 41544, 19944, 62204, 2596,  51556, 22892,
                5660,  57876, 3812,  10996, 32976, 15568, 51392, 11860, 29444,
                60188, 23212, 18204, 41836, 60940, 15780, 8916,  37596, 55224,
                32316, 28108, 2160,  18840, 30840, 3436,  50136, 44476, 16200,
                39712, 380,   25264, 42872, 6200,  15676, 57580, 45212, 23728,
                14432, 41368, 64636, 3844,  34576, 59700, 5436,  25180, 64292,
                11316, 22844,
            },
            {
                7996,  49132, 36432, 62624, 22200, 6664,  50384, 3256,  12740,
                39008, 528,   63472, 40728, 16716, 9224,  64720, 37732, 18504,
                42300, 7036,  35520, 19688, 40460, 61932, 16856, 34492, 43076,
                62492, 31720, 7960,  45324, 23976, 32324, 53704, 48320, 41860,
                59480, 45664, 37332, 42496, 8928,  62972, 45792, 55360, 42772,
                33536, 59324, 20580, 36348, 51736, 27348, 22208, 60960, 50760,
                20000, 11008, 48404, 21552, 13776, 45528, 38620, 48968, 22824,
                57556, 6768,  9756,  33596, 25096, 16832, 36452, 62424, 20400,
                45660, 24148, 52380, 7868,  32492, 56096, 15532, 36900, 28256,
                46236, 16712, 53056, 37620, 46748, 6332,  62124, 43380, 48604,
                53140, 14000, 46584, 1536,  36300, 50764, 27252, 65404, 5760,
                21544, 51684, 59932, 43760, 23668, 64100, 10592, 56740, 31904,
                58920, 52096, 17608, 54896, 35932, 20864, 3152,  10356, 60948,
                47972, 7756,  26924, 16708, 46768, 28544, 9508,  37444, 46052,
                53900, 34708,
            },
            {
                27160, 3668,  24680, 14880, 54700, 34316, 43680, 24932, 53804,
                27912, 20152, 31676, 51588, 4208,  34300, 21196, 11556, 31600,
                23160, 52136, 48096, 13176, 44188, 3276,  47140, 10060, 23476,
                18124, 47796, 35956, 28056, 16836, 6004,  62016, 25340, 9884,
                27624, 15380, 840,   60596, 52292, 13348, 19516, 5928,  38112,
                12448, 53216, 15672, 65164, 2848,  42200, 47132, 6096,  36712,
                58148, 4856,  39364, 56520, 8296,  29856, 63296, 10768, 15820,
                60620, 37904, 45164, 65220, 53980, 49016, 8616,  30832, 2588,
                59196, 38272, 17536, 48624, 26788, 40084, 10896, 59024, 7980,
                34012, 40476, 26224, 60688, 20496, 56260, 39420, 17392, 35408,
                8552,  26380, 63588, 54692, 20512, 31128, 3056,  18980, 45076,
                12156, 39412, 7552,  34532, 55408, 38836, 19964, 27948, 5016,
                22232, 8864,  33248, 28380, 59220, 49720, 39084, 29520, 18392,
                34932, 57396, 38236, 21000, 62048, 49844, 14748, 56288, 356,
                17340, 59976,
            },
            {
                12920, 64924, 42520, 47216, 10220, 28508, 61308, 16544, 8980,
                60576, 45692, 14488, 26164, 55880, 43016, 49648, 56980, 59888,
                4116,  15732, 63188, 25472, 8456,  58220, 50948, 30920, 59552,
                12840, 54516, 2780,  65492, 39612, 46688, 14156, 36756, 3180,
                64048, 49612, 33952, 29476, 23956, 39740, 31052, 57896, 24752,
                47100, 788,   44172, 25368, 10132, 33800, 55272, 13764, 24844,
                30028, 64908, 17424, 34864, 51996, 24748, 18900, 53648, 32344,
                26176, 48,    30136, 12936, 22160, 3680,  39416, 50908, 25832,
                54392, 5576,  13952, 60660, 436,   23572, 47820, 20420, 54532,
                65408, 12240, 49860, 1752,  29664, 9668,  25116, 4332,  21360,
                57400, 39756, 33288, 5224,  24160, 59272, 40508, 53228, 33832,
                61552, 25776, 17124, 46260, 4448,  15056, 52372, 42016, 61444,
                37576, 48724, 63220, 2548,  11912, 24436, 65336, 43612, 54648,
                2492,  51324, 11792, 43384, 2064,  31864, 23600, 41964, 20704,
                30392, 44528,
            },
            {
                38304, 52268, 31816, 18188, 1512,  56708, 37336, 48864, 33160,
                41248, 58036, 7004,  37160, 62216, 1368,  27996, 7392,  46644,
                40912, 29072, 53884, 38576, 33708, 21280, 27280, 39296, 32,
                42008, 26428, 20700, 57296, 11184, 52028, 19720, 56256, 31292,
                40192, 20312, 11108, 55156, 16740, 2484,  50468, 62120, 17824,
                8992,  61180, 31848, 38892, 49252, 17232, 62364, 39900, 9156,
                43780, 49836, 2880,  27164, 60376, 1808,  36944, 42620, 4456,
                47388, 51888, 17720, 56616, 43348, 28664, 63360, 15624, 41712,
                10040, 43772, 29816, 35548, 54028, 63572, 4776,  42516, 31140,
                2732,  23988, 43636, 15000, 35852, 53640, 32092, 64228, 45872,
                700,   29256, 12368, 49680, 44432, 13640, 9304,  47376, 15520,
                2440,  57636, 50352, 28776, 9796,  32332, 47316, 1920,  12452,
                25896, 44076, 14796, 40888, 46888, 7800,  33700, 14044, 6444,
                22584, 30940, 25144, 58496, 53532, 6356,  36092, 62936, 50936,
                5680,  57928,
            },
            {
                9312,  21704, 6012,  59340, 41348, 21032, 13280, 5296,  22864,
                2208,  18604, 52752, 11856, 23136, 17496, 39504, 13540, 24864,
                35072, 10292, 828,   17568, 61032, 5312,  14516, 53032, 63996,
                8620,  37364, 49156, 4980,  33732, 28564, 42656, 7936,  50692,
                23508, 57920, 5424,  46936, 65084, 44124, 34168, 4548,  37104,
                29432, 55084, 22448, 5344,  57956, 29152, 21084, 112,   32944,
                54156, 22720, 12188, 41608, 46304, 15116, 57188, 7964,  64176,
                20976, 40164, 61848, 5416,  35480, 11180, 19628, 58044, 33560,
                22820, 57036, 19024, 46008, 11552, 33292, 27764, 17104, 50828,
                37180, 19296, 61944, 7696,  47500, 59584, 41356, 11492, 50100,
                16296, 59928, 42184, 7600,  62660, 37656, 57100, 22692, 30284,
                42064, 10888, 36636, 64664, 56188, 22520, 62600, 35560, 18120,
                54316, 6068,  20200, 30696, 53396, 17324, 56756, 27300, 40492,
                45976, 63300, 8336,  14292, 39684, 19256, 59200, 12080, 26124,
                33548, 16108,
            },
            {
                55608, 28836, 34880, 50444, 25980, 62832, 45416, 30464, 63948,
                49932, 35096, 29120, 43796, 47676, 32664, 60964, 51808, 19108,
                64096, 55728, 50008, 31088, 45740, 56244, 35728, 19116, 46376,
                23156, 32140, 13956, 44152, 59808, 24476, 572,   62904, 17148,
                12684, 43168, 36380, 26576, 8440,  21872, 11796, 27112, 49152,
                14280, 41484, 19012, 52020, 12988, 7632,  46116, 50920, 59116,
                14944, 37672, 63140, 6996,  31724, 22328, 49456, 28468, 35392,
                10028, 14152, 32724, 24040, 48364, 53144, 6304,  46284, 1624,
                37460, 64644, 3524,  51200, 21812, 39124, 7176,  45012, 59628,
                10180, 55904, 28956, 51840, 22316, 17852, 3488,  28064, 23632,
                36840, 53320, 19980, 25956, 17380, 34364, 27880, 5672,  63384,
                49096, 24404, 18724, 40,    43028, 26628, 6612,  58544, 29372,
                49888, 57600, 36020, 23564, 61164, 37604, 1748,  49384, 59952,
                19448, 36444, 48560, 33148, 27688, 45120, 8800,  47232, 3480,
                40548, 48336,
            },
            {
                1012,  60848, 11820, 38640, 15860, 8224,  52932, 39696, 10496,
                55892, 24784, 3840,  65400, 9396,  54564, 4700,  29988, 45044,
                3004,  21892, 42368, 12796, 24256, 40768, 10448, 29880, 3980,
                57500, 51236, 62688, 18352, 9560,  54292, 37900, 46004, 34216,
                61020, 2092,  52148, 31712, 58976, 41812, 53412, 57004, 45156,
                64156, 2332,  35188, 60536, 43292, 37376, 64472, 26284, 18732,
                5132,  28688, 48112, 19900, 54972, 39192, 12732, 59484, 24484,
                55556, 42996, 2320,  58704, 38516, 26516, 60692, 30292, 12384,
                48588, 26312, 8220,  41160, 15728, 61680, 56972, 13092, 25444,
                4364,  40976, 33368, 492,   38468, 13352, 44784, 55264, 61992,
                9104,  32716, 47088, 4268,  55204, 50932, 904,   20196, 54260,
                38920, 7240,  32776, 53496, 13036, 38176, 15944, 41440, 9420,
                33348, 684,   64008, 10784, 4832,  45176, 21456, 32012, 10268,
                4504,  16220, 55700, 620,   65048, 51924, 16564, 29864, 54388,
                63848, 24248,
            },
            {
                18432, 43024, 46860, 4572,  56092, 32728, 632,   27116, 17244,
                42716, 14268, 58368, 38600, 21168, 15152, 35752, 59032, 8316,
                38172, 27228, 58608, 7416,  64928, 1952,  49792, 62076, 42844,
                16404, 27592, 1548,  40516, 30680, 21416, 14672, 5176,  29596,
                25540, 48828, 18692, 14108, 38616, 2920,  16092, 20256, 7152,
                24368, 10980, 47628, 26724, 3348,  23116, 31592, 10796, 42056,
                45492, 57096, 34264, 10116, 61500, 652,   44656, 5480,  17012,
                46736, 29172, 51420, 18632, 9296,  14692, 42352, 21372, 54660,
                17192, 32604, 59900, 29272, 53432, 1336,  35276, 30748, 48736,
                62648, 16044, 46596, 26744, 64792, 57804, 34564, 6728,  40144,
                14464, 2336,  64984, 29820, 39336, 11896, 60472, 45948, 34892,
                13492, 58348, 44928, 21340, 48308, 60612, 51152, 20720, 45884,
                24348, 14096, 47656, 42168, 51552, 25792, 58656, 13508, 52908,
                62328, 29576, 41168, 21268, 5412,  23824, 34672, 43312, 20412,
                13232, 36884,
            },
            {
                53240, 26992, 64660, 23188, 19388, 48456, 36356, 60312, 21808,
                46760, 7476,  31820, 50952, 156,   42036, 23964, 49120, 12060,
                16584, 33888, 48628, 19904, 37184, 27720, 54284, 22172, 12176,
                34780, 7648,  55856, 36384, 48080, 64544, 51716, 58156, 39520,
                10452, 56640, 7208,  61556, 24068, 29732, 62620, 36156, 32376,
                39812, 57480, 16944, 33260, 54604, 14688, 56272, 1736,  36112,
                62160, 23712, 3664,  50996, 17660, 25620, 52300, 33044, 38072,
                61248, 7224,  22288, 64520, 45084, 1028,  34068, 62616, 39484,
                4800,  50332, 44228, 10760, 24696, 47836, 18512, 22848, 53024,
                36316, 21064, 6976,  53400, 10320, 24400, 18984, 30868, 51752,
                21176, 44192, 58020, 23204, 15552, 42940, 31456, 8852,  17640,
                27284, 2936,  61384, 29248, 8960,  4692,  31536, 2560,  62852,
                55096, 39288, 19208, 28772, 16520, 8404,  34760, 43640, 38840,
                24356, 50572, 44580, 12576, 59764, 38144, 57532, 1864,  61568,
                7568,  31892,
            },
            {
                3944,  14596, 9140,  31368, 58528, 42332, 13756, 54360, 4872,
                62164, 37136, 26376, 17796, 45820, 62608, 27816, 55944, 40236,
                62992, 53780, 4844,  14848, 46808, 33000, 17396, 5440,  38776,
                60672, 45260, 23788, 12996, 2948,  26656, 8328,  17996, 47544,
                22460, 41496, 33408, 54380, 44256, 48120, 10160, 50868, 484,
                52676, 28884, 61940, 8888,  40376, 48612, 61092, 17628, 51884,
                8032,  14060, 27616, 43368, 36564, 30204, 65284, 20548, 3072,
                49816, 12320, 31284, 36572, 56728, 28208, 52152, 7540,  24488,
                57456, 14284, 20876, 36844, 6064,  64440, 40232, 8792,  3648,
                43520, 12348, 56940, 31680, 42468, 49924, 2016,  46464, 59712,
                27092, 35280, 10492, 48680, 6168,  56428, 25140, 50084, 64280,
                42568, 51668, 16384, 40224, 35148, 25024, 53760, 36928, 11300,
                27120, 7164,  59656, 32392, 55788, 64828, 968,   28144, 20192,
                3528,  8052,  34344, 26884, 47528, 9564,  14716, 28028, 49580,
                39604, 46252,
            },
            {
                57052, 35264, 52040, 39452, 2612,  24968, 9788,  29632, 34128,
                49624, 12308, 53956, 59528, 10556, 31208, 6300,  19452, 2472,
                23148, 43724, 30088, 61560, 57044, 8408,  43820, 58988, 31440,
                48768, 19584, 52000, 61420, 41344, 32836, 44308, 35700, 4068,
                63364, 28496, 884,   20904, 12560, 4556,  26304, 59448, 43092,
                13020, 19800, 5504,  44400, 20892, 4276,  38168, 29264, 22024,
                32996, 54288, 39744, 60264, 11760, 6872,  55424, 40672, 14780,
                26144, 54064, 41180, 4028,  16096, 48116, 19552, 11364, 47112,
                34992, 216,   61084, 54740, 31544, 45556, 56504, 15236, 60160,
                29200, 63628, 39232, 17108, 4532,  36048, 63228, 11656, 16468,
                5092,  41424, 53420, 19180, 37096, 62352, 3492,  21112, 36244,
                6504,  23480, 10992, 55220, 65144, 18476, 44132, 59380, 16160,
                42800, 52208, 3012,  37496, 12712, 40588, 47832, 54264, 60860,
                15528, 56724, 64084, 17760, 53828, 31360, 41980, 55560, 22164,
                10884, 60356,
            },
            {
                29320, 21276, 44676, 60924, 17532, 50632, 64252, 44144, 19776,
                1848,  40916, 22256, 3984,  35300, 51704, 13940, 46752, 32808,
                52160, 10680, 25712, 592,   39536, 12492, 23484, 52620, 26328,
                4480,  9844,  29784, 17080, 6292,  54864, 20596, 60116, 13144,
                53288, 15892, 46068, 36272, 64380, 40180, 31784, 16684, 22536,
                65500, 37232, 49592, 30924, 53196, 25944, 11588, 46592, 41540,
                64396, 2812,  20300, 47968, 16248, 44332, 23444, 9664,  58744,
                45348, 34624, 63128, 10416, 25020, 60224, 37984, 31232, 65464,
                43496, 28472, 41036, 17712, 12420, 2072,  26900, 33052, 50696,
                24212, 2468,  48580, 26236, 61008, 21896, 28808, 38244, 55140,
                23996, 62892, 232,   28356, 32632, 45480, 12388, 40008, 29032,
                56068, 46300, 32672, 1844,  47648, 12748, 6824,  28684, 22600,
                34476, 63552, 20948, 49112, 5076,  23740, 18564, 10212, 36132,
                45828, 30948, 39516, 1656,  6844,  62396, 18864, 3624,  36188,
                25404, 16844,
            },
            {
                42612, 780,   7824,  26640, 12164, 36912, 6520,  56600, 27492,
                16080, 63632, 29852, 43564, 24948, 39224, 57516, 63884, 36468,
                7076,  59496, 41652, 21220, 50416, 35484, 62744, 2116,  15216,
                42136, 65200, 34424, 58536, 25412, 49796, 10696, 31152, 24452,
                37996, 50464, 9300,  58420, 18052, 49892, 56392, 8196,  46500,
                27444, 3656,  56556, 14032, 63744, 34684, 59972, 6264,  15824,
                49684, 9696,  31404, 56592, 1584,  62924, 33392, 48964, 28748,
                924,   17276, 21188, 50708, 44036, 5876,  55580, 2720,  16756,
                22048, 9528,  50992, 25632, 63024, 37720, 20336, 42240, 10484,
                35016, 18904, 45068, 8932,  14112, 54400, 43436, 7704,  47720,
                34076, 13432, 50556, 60140, 9168,  16804, 53712, 59396, 1036,
                14352, 62988, 38136, 27956, 21608, 41316, 57632, 50000, 836,
                46180, 9984,  30976, 26384, 44756, 62104, 33064, 6036,  51980,
                25440, 11960, 20992, 43704, 24100, 50264, 33740, 45760, 64808,
                5920,  50784,
            },
            {
                13140, 63108, 49064, 54540, 30436, 47184, 22920, 39256, 51956,
                9116,  46352, 55076, 6868,  60468, 16652, 1068,  22108, 28332,
                17888, 48048, 15432, 31084, 55464, 18116, 28712, 46176, 56652,
                21672, 37236, 300,   43524, 14540, 38924, 64060, 1372,  56732,
                42848, 3444,  26068, 30796, 5268,  23416, 34828, 2240,  53568,
                41816, 33864, 9872,  24568, 17992, 43376, 1480,  56920, 27064,
                37100, 59192, 23912, 35660, 26792, 51576, 4316,  19420, 36908,
                61840, 8232,  56472, 29244, 33584, 13752, 41500, 26652, 52732,
                58880, 3716,  33112, 47620, 7644,  52464, 59300, 46900, 6560,
                55792, 52624, 37588, 58932, 33252, 50988, 19400, 1476,  58444,
                30612, 17952, 39316, 22284, 42140, 26844, 48156, 34104, 19832,
                51080, 8292,  17356, 59040, 4148,  53088, 31144, 37788, 61324,
                14276, 56532, 17280, 51328, 58604, 14740, 28992, 57184, 40932,
                3140,  48452, 60216, 53012, 36740, 10724, 26684, 13844, 53112,
                39980, 32804,
            },
            {
                23872, 37344, 18964, 33912, 3280,  62244, 14764, 4252,  58844,
                31688, 35024, 13436, 20372, 49304, 9600,  45148, 41376, 11852,
                56736, 37812, 65056, 4000,  44404, 5868,  10508, 39180, 32480,
                50004, 12108, 53908, 22728, 47828, 5172,  28344, 45496, 17372,
                33932, 21720, 62528, 53912, 43516, 14300, 60748, 38656, 11644,
                19352, 59168, 47712, 39972, 7060,  50196, 22924, 53688, 19032,
                44624, 13304, 5364,  42744, 18032, 39108, 13884, 54384, 42120,
                23968, 47644, 38504, 2380,  61056, 22884, 49228, 35704, 12344,
                45864, 38204, 55452, 14604, 23516, 30032, 2972,  13748, 64548,
                22776, 15484, 348,   30004, 5428,  39692, 12516, 27304, 65472,
                9656,  45608, 5732,  56760, 3208,  64524, 6880,  24440, 44576,
                30872, 41908, 25796, 49220, 35552, 15120, 8764,  19488, 26132,
                4540,  36484, 41468, 2000,  7888,  38148, 43420, 21752, 62980,
                14360, 34768, 8348,  29600, 15884, 61080, 540,   57936, 20476,
                9408,  59244,
            },
            {
                46564, 55948, 15428, 41288, 57620, 20588, 35828, 43236, 24760,
                18284, 1240,  61496, 38240, 25960, 33532, 55364, 31004, 50580,
                2980,  26264, 9356,  34016, 60512, 24724, 52192, 63804, 7124,
                18868, 61336, 28948, 8624,  57116, 35848, 19816, 51300, 8148,
                59648, 12820, 47584, 39152, 10332, 29012, 48152, 25880, 62912,
                15456, 29488, 720,   61728, 36656, 28940, 12036, 33404, 7928,
                30244, 62128, 52368, 46816, 10208, 57420, 30900, 60176, 11700,
                5000,  32288, 15308, 53380, 9660,  18820, 64016, 7528,  30616,
                62400, 19956, 5384,  61360, 44360, 40784, 18328, 36420, 31804,
                27900, 60424, 42832, 24668, 63760, 46924, 56132, 22508, 41552,
                49708, 25732, 35080, 53028, 31212, 15168, 37548, 11120, 62384,
                4360,  57180, 12000, 60980, 45144, 23880, 64236, 40064, 48092,
                53500, 23100, 65388, 33936, 20164, 54436, 11268, 56,    49964,
                18388, 27496, 57692, 4076,  41564, 47580, 38864, 31000, 44040,
                27632, 2408,
            },
            {
                31348, 5624,  10272, 25692, 45700, 8720,  51048, 11600, 64516,
                48208, 40632, 52536, 28668, 3600,  64744, 19120, 5860,  61124,
                23060, 42776, 53692, 13720, 48504, 20396, 36256, 14900, 27356,
                3540,  39968, 45140, 16808, 32256, 62752, 11868, 55308, 40524,
                27536, 5956,  35400, 19444, 168,   56000, 20860, 6680,  51092,
                32508, 45352, 21580, 51776, 14656, 57916, 41188, 65028, 48704,
                38452, 1232,  21216, 29000, 63928, 7376,  22388, 45736, 27656,
                51192, 65024, 43192, 25548, 46724, 39620, 57060, 848,   24520,
                42220, 16148, 28972, 35424, 11196, 58440, 54300, 49524, 4236,
                40336, 11536, 50372, 20116, 9808,  16396, 31196, 36268, 2896,
                15364, 61712, 20500, 12636, 43944, 50820, 58612, 17984, 54652,
                34776, 21316, 39024, 268,   6524,  33400, 55188, 11648, 6588,
                29956, 45100, 12836, 28016, 48212, 60272, 25856, 32032, 36968,
                55544, 45304, 23408, 65108, 19188, 54108, 22928, 5240,  17048,
                63548, 51896,
            },
            {
                19968, 42988, 64704, 29532, 1300,  60808, 32396, 27084, 55320,
                6964,  22408, 9968,  57776, 46784, 12896, 51724, 39104, 15100,
                34996, 46468, 18424, 28916, 40288, 1712,  59156, 43124, 47208,
                23404, 57820, 51828, 2568,  25544, 42684, 4172,  23992, 31480,
                65272, 16480, 57560, 51608, 32148, 63576, 44636, 35920, 3544,
                57764, 8268,  55088, 25820, 4560,  19924, 46044, 2856,  16476,
                24132, 55100, 40432, 15192, 34136, 49256, 628,   36000, 18600,
                8752,  56956, 20324, 30052, 4052,  34484, 14732, 44600, 51880,
                9920,  57664, 48960, 22464, 564,   26776, 8492,  21524, 62344,
                45532, 6928,  57368, 35052, 53820, 3956,  44200, 57988, 52032,
                7276,  38552, 58984, 912,   27772, 22780, 40212, 2228,  28636,
                47460, 15712, 53364, 30588, 18412, 51044, 27652, 43476, 57800,
                18076, 60888, 8688,  52396, 4008,  16784, 46188, 64172, 9132,
                40400, 6176,  50484, 32676, 12692, 8716,  34592, 56376, 48816,
                36404, 12072,
            },
            {
                57424, 37828, 49976, 53848, 22548, 39844, 16488, 3760,  37412,
                30320, 14828, 42836, 17432, 35724, 23292, 43684, 8648,  27464,
                58232, 236,   63100, 7668,  57148, 33152, 11540, 54396, 30964,
                9576,  34180, 13404, 37168, 58940, 48584, 14948, 38024, 49716,
                1604,  45256, 41688, 24848, 14896, 8608,  40152, 17196, 23388,
                42748, 38364, 11432, 34920, 63264, 31700, 9604,  26552, 60700,
                35420, 11484, 58816, 4224,  53072, 25236, 41672, 62844, 14636,
                40340, 2028,  37080, 11988, 59592, 54616, 27400, 21248, 32916,
                37132, 3824,  53148, 39508, 65132, 46656, 34004, 15724, 37888,
                25604, 18000, 47688, 28648, 38952, 61888, 25356, 11396, 18656,
                29060, 33012, 47276, 10248, 63436, 35772, 49140, 9076,  42856,
                25344, 7368,  61732, 41404, 22480, 62952, 3172,  14832, 35788,
                948,   32660, 39172, 21960, 41976, 35108, 6580,  22648, 13220,
                30044, 16932, 2660,  37808, 44500, 62740, 25900, 14440, 41228,
                7880,  23452,
            },
            {
                33240, 15988, 4680,  12780, 35596, 47756, 58192, 20148, 61820,
                45264, 54208, 33208, 5228,  59804, 1876,  30440, 62436, 20544,
                48900, 12352, 31884, 22296, 50280, 25908, 17132, 6260,  62088,
                49736, 19456, 64860, 7188,  29716, 20768, 61244, 9352,  53740,
                22072, 11220, 29524, 4064,  58884, 49084, 27576, 54268, 12952,
                64916, 28104, 18596, 44056, 49316, 39244, 52344, 56464, 43176,
                6688,  50064, 27344, 19880, 44328, 11040, 58096, 31836, 53756,
                48432, 23152, 61684, 50340, 17736, 5916,  48176, 60952, 12552,
                63640, 25680, 7904,  17120, 31552, 12332, 50972, 55884, 1948,
                59240, 32536, 13780, 1116,  21984, 8044,  33656, 48516, 64300,
                22196, 55564, 42808, 17412, 54460, 4868,  19932, 32328, 65248,
                56784, 13308, 36064, 48648, 10244, 46360, 38052, 60000, 21200,
                50256, 25608, 56396, 62896, 14104, 28796, 59016, 51228, 43544,
                57444, 61744, 53808, 27032, 20584, 1516,  51056, 59824, 3692,
                29016, 61724,
            },
            {
                92,    46772, 27872, 18920, 56224, 6476,  43792, 13260, 50576,
                568,   24384, 64064, 49196, 26568, 55640, 36980, 52792, 4128,
                40820, 55128, 37976, 44504, 4384,  64596, 41216, 21732, 37396,
                576,   41800, 24640, 44564, 54280, 1992,  34048, 43920, 18488,
                36204, 61856, 54620, 38492, 20628, 34400, 2736,  60324, 31532,
                47136, 1112,  58996, 7128,  16380, 3384,  21724, 13272, 31096,
                18092, 45780, 32732, 64768, 37504, 17028, 6820,  21412, 4300,
                28492, 34380, 44464, 7560,  31780, 42308, 38064, 1972,  18996,
                47156, 30356, 43712, 60400, 20364, 41416, 5292,  28944, 43284,
                10056, 52756, 64856, 41196, 51000, 58732, 14860, 42192, 1796,
                39672, 13652, 4168,  24836, 30904, 61224, 14668, 53048, 23184,
                45088, 1456,  29176, 59112, 16760, 7804,  24808, 31300, 42816,
                7132,  46476, 11248, 2652,  48540, 54296, 38524, 1220,  19632,
                24996, 34988, 9792,  48316, 58520, 39888, 30280, 46092, 18376,
                53268, 44148,
            },
            {
                25396, 59108, 40320, 62428, 31596, 10124, 25636, 34420, 28608,
                8684,  39272, 11848, 19876, 41516, 15492, 10736, 44932, 17528,
                29064, 24644, 9480,  15588, 35104, 55736, 46156, 29572, 53456,
                15016, 27980, 56112, 10420, 17032, 39668, 27452, 57628, 5040,
                25816, 46684, 6248,  13356, 64248, 46024, 10548, 51400, 6076,
                21288, 34044, 53000, 25008, 61312, 28604, 36344, 62860, 212,
                23396, 60052, 9244,  1960,  54432, 29688, 46296, 38936, 60800,
                9960,  55888, 13804, 26332, 64580, 22148, 53096, 9236,  56216,
                40760, 13868, 54836, 2188,  36368, 57980, 24616, 63352, 18624,
                22808, 35908, 5980,  26816, 44524, 19548, 30312, 54008, 6156,
                26072, 57832, 51256, 36972, 44448, 7252,  41116, 10880, 37640,
                4644,  19428, 51600, 26084, 34236, 56544, 51912, 12408, 64692,
                53392, 16252, 36492, 31020, 19100, 8848,  26248, 32912, 47192,
                4192,  41124, 14116, 31624, 7016,  16592, 11288, 34160, 22036,
                38904, 9972,
            },
            {
                50184, 6940,  52448, 23244, 2104,  49328, 64884, 40572, 58876,
                17716, 56480, 46060, 31912, 7500,  61644, 22320, 33184, 65504,
                6836,  59212, 51484, 61280, 19892, 12844, 3084,  10144, 59788,
                47932, 4256,  61592, 32036, 47076, 52176, 13216, 62712, 31236,
                15576, 51508, 32892, 23540, 42672, 17488, 26232, 37820, 15888,
                41656, 56828, 13512, 40016, 46112, 10488, 54328, 41900, 48340,
                38420, 52048, 14584, 42432, 22812, 61536, 12784, 51816, 25384,
                17436, 47400, 52,    39916, 57504, 15268, 29648, 25252, 34812,
                5748,  23220, 49916, 27640, 10348, 52412, 14644, 45356, 39016,
                49664, 30620, 55540, 16848, 3312,  36624, 10084, 60020, 34588,
                45896, 29724, 9748,  59944, 21516, 47792, 27612, 58868, 49428,
                33456, 62072, 39760, 6044,  44292, 1860,  40896, 19240, 3972,
                28264, 39932, 58216, 24060, 44732, 64972, 15612, 60228, 11528,
                53020, 63076, 22364, 44032, 54528, 24416, 61028, 5448,  55764,
                65344, 14084,
            },
            {
                20924, 32964, 11476, 45016, 37884, 15176, 21128, 3356,  52500,
                22956, 36252, 2760,  28088, 53736, 50432, 976,   46964, 13268,
                35940, 42888, 1348,  30412, 47368, 26728, 39196, 32824, 23884,
                18612, 34884, 38596, 21024, 5420,  23960, 8652,  48468, 1264,
                39820, 59876, 8972,  57236, 876,   30428, 56128, 61616, 24364,
                48788, 9176,  30524, 20184, 50176, 33488, 6792,  15716, 26360,
                4596,  56152, 29100, 36196, 5452,  48664, 33664, 2316,  42624,
                36604, 63332, 20272, 33580, 10756, 3864,  50836, 45604, 62380,
                58644, 16608, 32792, 64200, 4372,  47456, 33696, 7716,  2884,
                12992, 60884, 9064,  46232, 63224, 52332, 23896, 47636, 17268,
                62628, 20176, 176,   16152, 33680, 54120, 2312,  18004, 30096,
                15172, 55384, 11744, 21296, 64108, 14228, 29328, 58992, 34948,
                9856,  20920, 62252, 6500,  51660, 41284, 4864,  55940, 36804,
                17948, 28232, 49576, 448,   64372, 36916, 42380, 47484, 1776,
                28076, 36480,
            },
            {
                43000, 63500, 17248, 28764, 59780, 55148, 42712, 32520, 6120,
                47320, 63448, 14348, 59464, 37696, 24812, 40024, 57592, 26436,
                49312, 18464, 23260, 40352, 5340,  58084, 52072, 63692, 44728,
                7512,  51244, 11980, 57868, 63488, 42528, 36868, 55680, 21848,
                43424, 28372, 19056, 48348, 34900, 53176, 7944,  44720, 2084,
                35644, 62940, 4712,  59660, 1440,  23936, 65184, 57648, 20328,
                33840, 10568, 18756, 63516, 26020, 16188, 55436, 20612, 59336,
                7412,  30488, 53184, 49344, 43184, 60720, 36816, 19764, 11840,
                1076,  38328, 44684, 21428, 40496, 17680, 30180, 61980, 57104,
                21012, 40608, 25184, 33296, 13592, 39408, 28428, 7084,  12540,
                37792, 41512, 48860, 64540, 12160, 39260, 25492, 62884, 6340,
                42708, 24564, 46416, 31696, 53200, 37768, 49840, 23588, 45688,
                55448, 48792, 304,   32548, 12916, 35168, 20820, 29928, 45344,
                8228,  38748, 5708,  33804, 18696, 27460, 13468, 51920, 31972,
                16332, 58032,
            },
            {
                2528,  6024,  53776, 35564, 8012,  26344, 12052, 61572, 16552,
                29608, 10432, 42452, 19360, 5728,  9976,  16764, 30664, 3512,
                55992, 10588, 53988, 64260, 34248, 16320, 21044, 1700,  13988,
                55596, 41420, 30192, 1452,  26564, 14556, 32764, 18288, 64640,
                6812,  12024, 37308, 61964, 14676, 40356, 22116, 12668, 29440,
                18760, 52648, 26268, 38116, 16916, 43512, 12084, 30252, 45280,
                61228, 40500, 49864, 43700, 58648, 8760,  40072, 27964, 11344,
                44804, 14744, 5028,  24156, 27812, 17204, 7624,  32044, 42152,
                28616, 52636, 7424,  12484, 60352, 55104, 23008, 37380, 27520,
                47180, 53800, 5492,  57984, 20564, 756,   60996, 55040, 22684,
                4036,  56032, 31512, 24116, 5564,  57024, 45972, 35600, 9820,
                50768, 59176, 424,   8460,  26484, 17208, 3396,  61580, 5872,
                14984, 25740, 43428, 17592, 56860, 27192, 50172, 1572,  24192,
                60768, 14412, 57740, 51460, 10296, 56692, 7364,  23280, 40124,
                8884,  48236,
            },
            {
                30820, 38728, 22456, 46712, 704,   19568, 50268, 45360, 35712,
                25056, 51052, 55460, 32968, 48656, 63868, 43808, 60732, 20880,
                38860, 32504, 14456, 6968,  50360, 9704,  42980, 28760, 36460,
                25976, 60916, 16656, 49760, 45552, 53864, 3264,  29292, 51072,
                46152, 24376, 52460, 3472,  26920, 5820,  65252, 50588, 58728,
                42628, 10868, 45764, 32236, 55584, 51156, 39572, 3572,  53180,
                7672,  24280, 1092,  12600, 31032, 4016,  47016, 65228, 32816,
                22960, 54804, 38588, 62552, 2524,  56852, 47028, 54476, 65324,
                24120, 48876, 57492, 35968, 26688, 100,   50120, 10928, 16036,
                1744,  35200, 17832, 43504, 49188, 31248, 42092, 50608, 33896,
                45300, 27132, 8524,  51384, 43120, 19724, 14008, 53188, 22504,
                38344, 18404, 34100, 62464, 56944, 47872, 10920, 32780, 42392,
                38312, 30108, 53520, 60580, 39156, 10012, 63212, 42756, 53468,
                31876, 47332, 21612, 40980, 30596, 46292, 60428, 19808, 63012,
                54488, 25504,
            },
            {
                60136, 49824, 13816, 56220, 62580, 30676, 39600, 4024,  57288,
                7916,  38988, 68,    26996, 22012, 35536, 13012, 51900, 5460,
                46164, 62260, 28236, 44324, 25196, 38180, 56644, 62508, 48708,
                5372,  22136, 9228,  34304, 19868, 39564, 61208, 10392, 35068,
                16180, 58576, 31668, 41768, 20040, 46992, 32468, 15744, 38664,
                5376,  22368, 64152, 14136, 7900,  19836, 35220, 27136, 47784,
                14812, 37176, 57072, 20856, 53416, 35288, 18472, 50956, 880,
                57384, 48312, 18008, 34952, 12380, 40000, 21640, 14304, 3772,
                9728,  18544, 31164, 15284, 46108, 6204,  41640, 64116, 44516,
                51792, 29656, 9852,  65096, 26936, 4940,  16600, 9648,  63768,
                14188, 59000, 17704, 34612, 62052, 28532, 1100,  31328, 64736,
                4056,  28280, 40812, 13676, 43356, 36096, 22040, 58092, 18668,
                64036, 11632, 7788,  22184, 2864,  47984, 16064, 6624,  18992,
                11832, 63636, 26432, 2264,  16168, 34804, 3108,  37616, 45032,
                12168, 18340,
            },
            {
                43568, 10044, 27508, 41328, 33812, 9700,  53068, 14200, 21628,
                60172, 18040, 62916, 44680, 3636,  54340, 28632, 8628,  34308,
                24236, 2580,  48552, 19040, 60004, 31392, 4392,  17864, 11788,
                32580, 54444, 43608, 65488, 6684,  12652, 57728, 25528, 5612,
                38868, 72,    63612, 10752, 54044, 57604, 36660, 8788,  26636,
                54504, 34224, 48216, 3372,  29240, 62176, 58288, 10068, 18180,
                64256, 32596, 27836, 44668, 62824, 38896, 24320, 13880, 41236,
                6316,  29260, 9248,  43752, 59296, 29712, 50564, 34616, 27140,
                60192, 40188, 4760,  62636, 53836, 34132, 19952, 31704, 7556,
                59752, 22120, 55228, 36872, 12364, 59500, 38608, 20284, 25872,
                1308,  52816, 39988, 3508,  10660, 37988, 48920, 57784, 43872,
                11892, 48196, 54128, 20320, 2644,  30300, 7612,  50368, 27660,
                1404,  51348, 36496, 45756, 25292, 30816, 35980, 58808, 40312,
                34460, 4608,  44312, 59384, 53896, 24656, 49940, 6424,  27972,
                33464, 4408,
            },
            {
                64620, 21052, 57816, 2968,  17004, 23164, 65232, 28132, 42188,
                32168, 47412, 11372, 15496, 58172, 39276, 18832, 42104, 59288,
                16140, 56764, 36120, 12212, 964,   53296, 23072, 46412, 40444,
                59924, 404,   38212, 27408, 51892, 22740, 47128, 42912, 56056,
                49804, 22308, 44344, 16852, 29700, 4376,  23708, 45160, 61288,
                1880,  17312, 56704, 41244, 24964, 45072, 796,   22952, 54732,
                42372, 3132,  51708, 16472, 6056,  10136, 57932, 27232, 61112,
                36536, 20916, 64360, 25408, 52120, 7196,  640,   45252, 37536,
                55616, 47404, 21160, 43208, 24496, 12256, 58828, 38464, 25676,
                14444, 40388, 3268,  46424, 24224, 52228, 32304, 44316, 48332,
                36224, 29892, 21772, 44444, 59644, 15516, 21092, 7384,  25952,
                16572, 61268, 6268,  24472, 51580, 65256, 45508, 40160, 14088,
                55860, 33636, 19224, 62296, 57264, 12644, 51604, 21744, 27320,
                55812, 9272,  36772, 20112, 12244, 64436, 42164, 14892, 56408,
                51144, 36408,
            },
            {
                15616, 31856, 5944,  51732, 45460, 35976, 48240, 1408,  54548,
                6408,  24468, 37432, 29968, 49364, 6308,  65064, 25840, 50300,
                11048, 39800, 21876, 63412, 41804, 15128, 35180, 7856,  52044,
                24604, 18576, 48056, 14824, 31572, 36848, 1988,  17288, 32380,
                13604, 27744, 8180,  35688, 60240, 51196, 12232, 18892, 49856,
                39928, 31028, 12020, 60244, 36556, 15976, 49692, 38756, 29836,
                7092,  59248, 11664, 23616, 49124, 30508, 2752,  45456, 16860,
                53004, 46876, 2192,  15304, 33116, 19052, 61900, 23580, 16308,
                11900, 32876, 1892,  28904, 8544,  50512, 16728, 55704, 4620,
                48676, 61800, 33752, 16324, 7908,  56020, 2344,  61432, 6276,
                57372, 11624, 65192, 50016, 25564, 54888, 33712, 41448, 52260,
                36584, 32308, 56712, 38644, 11296, 16284, 25924, 4704,  61284,
                23368, 42920, 6208,  15208, 39804, 3672,  44480, 65348, 512,
                49128, 15388, 52404, 28676, 39220, 8112,  31692, 22264, 61912,
                312,   23984,
            },
            {
                54904, 47748, 37864, 61668, 25520, 7828,  58912, 19268, 12468,
                34924, 51412, 60920, 8540,  23176, 32884, 13544, 816,   44964,
                30808, 54216, 7112,  28928, 49628, 56236, 27340, 64032, 13612,
                30408, 57304, 10256, 62488, 4944,  58832, 28476, 64144, 20620,
                53152, 61860, 47532, 40256, 25464, 1412,  42244, 63952, 28732,
                6956,  23380, 51836, 20024, 4656,  53788, 11500, 63348, 34212,
                20224, 47280, 39512, 35076, 60868, 41932, 54252, 33552, 7588,
                12124, 31640, 41844, 58064, 48772, 11376, 53716, 41276, 63984,
                5696,  52084, 57824, 36528, 64920, 46720, 2340,  30040, 43880,
                11440, 28112, 51088, 20672, 42824, 29092, 13168, 18068, 23284,
                42312, 16192, 7604,  31896, 4916,  46664, 2496,  63104, 9640,
                21940, 1756,  28628, 46600, 59292, 34584, 54592, 37312, 9400,
                47196, 30552, 49476, 27960, 54372, 33424, 8528,  17616, 30176,
                41668, 23312, 62876, 46392, 1396,  57660, 48072, 18708, 9612,
                46120, 40884,
            },
            {
                7504,  28892, 19368, 12136, 42320, 15400, 30216, 39136, 44508,
                56788, 3088,  19676, 43204, 55872, 46704, 53104, 36352, 19492,
                62356, 4040,  33716, 44128, 9744,  19364, 39208, 2832,  44740,
                36388, 3912,  41968, 20744, 44936, 53596, 9540,  41120, 6392,
                37324, 2428,  57344, 13088, 32052, 20708, 55184, 33968, 14792,
                58068, 47052, 35004, 9000,  42736, 31964, 25664, 5808,  44304,
                14452, 26088, 56184, 240,   18208, 13368, 21936, 64028, 39112,
                26128, 55924, 22800, 4164,  39396, 35768, 26904, 8500,  30640,
                49360, 18300, 25984, 14564, 40532, 20436, 35064, 22900, 63456,
                18548, 37568, 468,   57972, 62816, 39608, 49832, 35084, 53440,
                28272, 38724, 55492, 19480, 37252, 12860, 29736, 18700, 58360,
                44660, 13824, 41204, 8696,  19136, 624,   31124, 21132, 57908,
                17260, 2180,  64320, 10800, 21100, 60856, 24840, 47724, 11356,
                57152, 32348, 6548,  17988, 34156, 25108, 52680, 37824, 29728,
                59344, 12908,
            },
            {
                52376, 60036, 1128,  33176, 55600, 3308,  52224, 63120, 26036,
                16540, 28756, 63712, 40136, 2052,  16884, 60372, 28516, 9232,
                23688, 47468, 17216, 59540, 25500, 5976,  32388, 57992, 21400,
                50192, 26540, 54720, 33868, 24240, 15864, 48776, 34568, 11968,
                45908, 24272, 18608, 7280,  62500, 48528, 5608,  37952, 10252,
                43884, 316,   65080, 27692, 61488, 46388, 56984, 17964, 60348,
                50604, 8956,  64944, 31828, 45772, 28008, 50420, 4688,  19616,
                48448, 62640, 9736,  28692, 60752, 20492, 56212, 2708,  43412,
                22488, 38624, 61172, 6960,  54048, 10704, 60128, 51936, 41000,
                8108,  54412, 24704, 10808, 33032, 5100,  25556, 63916, 9340,
                46572, 1460,  61060, 43408, 23360, 62036, 51308, 39264, 26952,
                48040, 64456, 53672, 23392, 62148, 49508, 44488, 12192, 51728,
                41104, 25580, 36628, 52104, 5188,  42576, 38368, 59268, 35820,
                3940,  50924, 13556, 40920, 60152, 5032,  11728, 44236, 2744,
                34792, 26148,
            },
            {
                39012, 43812, 23092, 64340, 49136, 37124, 10236, 22532, 5812,
                49672, 9556,  32488, 12736, 27012, 21760, 5160,  42584, 38476,
                58228, 52124, 12772, 37152, 64980, 53100, 47764, 11652, 61492,
                16896, 8356,  60308, 12548, 39628, 744,   26276, 61252, 56628,
                29996, 51312, 33756, 43488, 52928, 28348, 16616, 59504, 25940,
                21048, 54780, 17100, 38536, 13960, 2244,  21928, 40060, 3740,
                29336, 37216, 21520, 5968,  55004, 9280,  36708, 58380, 42948,
                1052,  34332, 17932, 45988, 6440,  14428, 47512, 32760, 59532,
                13320, 46368, 776,   33072, 44700, 27672, 5940,  32212, 14964,
                46340, 59228, 30732, 44352, 15560, 47716, 19728, 2872,  58392,
                14524, 33396, 26468, 49376, 9464,  56812, 15652, 6696,  35452,
                3804,  17588, 33824, 5656,  37760, 14928, 27708, 63336, 3964,
                33908, 58952, 13864, 44980, 31656, 16104, 1784,  53116, 20172,
                26352, 44988, 61772, 22092, 54544, 27680, 16124, 65104, 55248,
                21272, 17212,
            },
            {
                5108,  10764, 46792, 16456, 27292, 20680, 60380, 31908, 41824,
                58028, 46456, 37260, 54136, 58712, 35036, 50928, 64092, 15112,
                2892,  27664, 41096, 148,   22804, 15668, 42504, 28872, 1224,
                41272, 31960, 46404, 6140,  64848, 50940, 18168, 42968, 21852,
                3304,  59680, 15656, 38500, 10088, 22948, 46320, 3464,  40744,
                50504, 32268, 6620,  24292, 52780, 30420, 49408, 35252, 54636,
                11944, 42904, 52240, 15964, 39832, 62404, 24464, 11828, 30084,
                15244, 54624, 37592, 52612, 42652, 64784, 24588, 17368, 53060,
                9948,  29392, 56412, 23720, 50036, 17076, 57528, 1332,  25532,
                35576, 4452,  21768, 50716, 7404,  57220, 36592, 41192, 30376,
                21432, 52692, 5788,  17352, 34788, 204,   41636, 24220, 55688,
                11596, 30700, 50704, 26044, 43308, 56484, 7048,  39460, 23132,
                46800, 8412,  19344, 61584, 23840, 56196, 28812, 13256, 64268,
                8056,  38580, 29400, 9904,  35604, 49756, 42196, 32132, 6904,
                48600, 62888,
            },
            {
                35748, 58312, 31184, 7884,  40580, 4900,  46000, 13912, 35796,
                360,   23700, 15336, 4464,  44760, 8048,  30220, 11208, 45728,
                33440, 20644, 56684, 31216, 45956, 5048,  34664, 55240, 24536,
                37584, 52256, 19500, 29388, 36744, 55712, 32448, 8172,  14180,
                39472, 6052,  26188, 65316, 856,   57064, 35284, 63272, 29812,
                12216, 60144, 47568, 41784, 10856, 59076, 7712,  15680, 63240,
                23696, 1428,  59308, 26472, 48820, 2252,  32560, 44912, 51188,
                8120,  59988, 25584, 12588, 31092, 84,    37020, 40412, 4240,
                63600, 35544, 19340, 62024, 12596, 37116, 43232, 64528, 48248,
                11584, 61200, 17556, 38564, 65448, 27792, 11984, 51440, 44696,
                62388, 11076, 40240, 58632, 28212, 45824, 31240, 63372, 49208,
                21040, 60956, 1176,  58480, 10916, 20268, 31924, 54492, 16524,
                30016, 55388, 96,    39576, 49280, 9916,  46676, 34392, 43096,
                17284, 58260, 1084,  47392, 19168, 3516,  59052, 24228, 13848,
                40636, 28420,
            },
            {
                24796, 14972, 51864, 55124, 34060, 57512, 50476, 18016, 62312,
                52564, 20004, 64740, 50144, 25212, 41060, 19680, 56360, 24592,
                49512, 7688,  62948, 10672, 51204, 18788, 62100, 8836,  14356,
                63608, 4488,  58568, 15164, 22856, 2900,  46908, 28304, 63624,
                49776, 45216, 54712, 30752, 20236, 42124, 13680, 52560, 8252,
                18684, 36752, 3040,  56036, 33924, 20128, 44748, 27388, 47488,
                32312, 18768, 45052, 34600, 12800, 20220, 55672, 17252, 65244,
                22620, 40940, 4496,  20752, 49032, 10264, 57948, 51024, 21908,
                26976, 42840, 8004,  39920, 4528,  30580, 22596, 19092, 39648,
                28652, 52820, 42944, 33604, 1872,  54704, 22940, 16444, 4248,
                35580, 25040, 47672, 64896, 12208, 53344, 19424, 8124,  43996,
                36640, 40408, 15408, 46696, 35792, 61604, 2924,  48548, 11708,
                64968, 42924, 27212, 35032, 6472,  63452, 21668, 2944,  50228,
                23948, 52452, 32232, 15048, 63048, 38040, 8536,  20208, 52036,
                56896, 1364,
            },
            {
                62092, 42376, 2784,  19744, 12432, 24560, 2132,  29668, 8480,
                27180, 40076, 30740, 10844, 59568, 16048, 61628, 5400,  36668,
                54096, 17724, 43596, 35756, 58492, 26424, 38976, 49688, 21948,
                44140, 27008, 48904, 9136,  43552, 59636, 11572, 53816, 34284,
                19244, 23436, 11036, 35988, 47912, 5764,  32744, 24828, 45060,
                61636, 22764, 28416, 16240, 64484, 1728,  38056, 60448, 5004,
                39636, 56372, 8008,  29848, 61040, 41996, 6788,  35524, 3596,
                28116, 46828, 33632, 62228, 54880, 27476, 34220, 14620, 6832,
                48400, 55316, 15736, 51800, 46608, 55804, 9740,  51356, 7028,
                56824, 3352,  13472, 24832, 9940,  46012, 32160, 60820, 8176,
                55896, 19288, 2056,  37516, 22724, 4304,  57624, 14396, 27756,
                4684,  54996, 23804, 29360, 8104,  41388, 51812, 25804, 37272,
                5064,  21224, 52496, 13572, 57744, 17920, 37896, 30708, 61024,
                12068, 6660,  56344, 41616, 25572, 53860, 29568, 44396, 33100,
                11340, 47056,
            },
            {
                21912, 29368, 37440, 60080, 44436, 65384, 38612, 43296, 54696,
                12828, 48076, 3500,  34120, 38192, 972,   47492, 31868, 12948,
                40448, 1832,  29504, 23432, 3404,  12872, 30716, 59608, 2984,
                33744, 11976, 56580, 31320, 39432, 25288, 17364, 37868, 6920,
                42408, 2148,  61588, 14868, 58896, 50800, 17700, 2004,  54872,
                40092, 5468,  49912, 43836, 25616, 51388, 13668, 22524, 10168,
                53208, 15068, 64604, 3124,  50852, 22840, 47284, 59512, 38672,
                10656, 56904, 15972, 44180, 7432,  18568, 62968, 45696, 30484,
                59848, 2540,  33340, 25772, 63756, 1896,  34608, 59736, 15876,
                32116, 20568, 62632, 49556, 58724, 18772, 42160, 38292, 26836,
                49440, 29960, 43660, 15812, 32408, 50472, 39424, 34200, 59872,
                10140, 18028, 65452, 52944, 13104, 22652, 18264, 44780, 33568,
                59908, 9040,  47732, 32088, 24888, 41684, 53984, 4912,  26476,
                40128, 34916, 20896, 45796, 2712,  13024, 48796, 64224, 3828,
                38388, 18012,
            },
            {
                8308,  50648, 5836,  48304, 27676, 9944,  20952, 35132, 58636,
                6212,  61392, 44196, 22432, 54920, 28252, 51944, 21784, 65508,
                26560, 59852, 14956, 54728, 48168, 41032, 7032,  45472, 17480,
                53924, 41444, 20784, 868,   62964, 51464, 3916,  60804, 26488,
                57380, 51780, 29192, 39052, 8028,  27308, 64076, 37544, 29384,
                14552, 57856, 11352, 32192, 8568,  40696, 57196, 35140, 30836,
                49724, 25000, 36032, 11168, 38992, 15460, 26052, 30912, 52524,
                19672, 49812, 1764,  30036, 39772, 23656, 3452,  41596, 11688,
                18108, 37540, 21060, 10632, 28912, 14344, 24044, 44940, 37756,
                26604, 46756, 40316, 35404, 29116, 5800,  53052, 1088,  13976,
                64056, 10096, 53752, 6480,  61520, 45000, 24628, 1616,  46912,
                51128, 31604, 43112, 2168,  34548, 58072, 62716, 1632,  14680,
                56624, 38960, 17096, 61184, 1532,  11404, 46028, 59152, 15564,
                48728, 65268, 28432, 10368, 60780, 36548, 17304, 6584,  27020,
                59724, 54776,
            },
            {
                31636, 63648, 13332, 17452, 32940, 3816,  51424, 15900, 23916,
                32280, 19336, 14340, 57540, 7664,  17936, 42020, 9784,  44860,
                4660,  50620, 37348, 8936,  63752, 20656, 57012, 36160, 28216,
                65180, 7468,  36576, 16376, 47324, 33852, 21312, 44704, 13992,
                9608,  32980, 17300, 46540, 21844, 43104, 56328, 10280, 20332,
                46276, 34572, 62864, 20788, 54104, 4340,  18172, 63700, 532,
                41232, 20260, 46716, 28556, 53504, 57164, 8644,  428,   13628,
                62928, 25044, 36436, 12816, 60988, 52708, 35516, 56172, 25484,
                65380, 44372, 53576, 58580, 38800, 42544, 49404, 6400,  63280,
                10976, 55012, 144,   8432,  15548, 61884, 21380, 47044, 57836,
                33016, 40732, 20968, 58864, 27312, 9060,  17296, 63956, 21328,
                38012, 25768, 6716,  48948, 39068, 27224, 7680,  31136, 50344,
                24064, 29040, 3836,  43164, 51628, 28080, 35412, 19796, 8844,
                43600, 400,   17668, 50568, 31732, 23460, 58472, 42828, 34944,
                14704, 45224,
            },
            {
                41208, 36192, 25760, 54100, 40324, 56604, 62136, 46652, 920,
                41708, 53160, 36892, 25784, 46568, 62760, 3620,  34876, 57788,
                19532, 31332, 46444, 17404, 33528, 25480, 52252, 14836, 1488,
                23272, 50536, 60872, 28600, 13064, 5664,  55768, 30704, 49340,
                41008, 64628, 5308,  53948, 272,   13100, 34028, 49004, 4696,
                52468, 26300, 892,   37784, 45684, 29764, 48272, 26596, 43916,
                7172,  57944, 61808, 5712,  18484, 42932, 64072, 34032, 45184,
                40068, 6552,  58780, 22128, 46420, 9336,  15504, 48084, 1108,
                31740, 8504,  48720, 5488,  17456, 61160, 31036, 21408, 3724,
                51076, 17980, 23420, 44224, 56336, 31568, 11604, 35904, 23928,
                17176, 3116,  36812, 48172, 13292, 42228, 53036, 29692, 11772,
                56160, 14816, 62468, 20228, 54184, 16496, 42348, 55164, 10540,
                19536, 46628, 63912, 36212, 22228, 7592,  62956, 31460, 55708,
                23320, 38432, 57328, 6088,  39908, 54976, 9308,  20648, 53016,
                24216, 88,
            },
            {
                19928, 57408, 3104,  22556, 45372, 7448,  28996, 11908, 26416,
                48932, 64568, 2584,  10624, 32592, 38848, 13548, 54276, 25064,
                39556, 11560, 60972, 804,   44080, 5232,  10944, 60340, 48036,
                32640, 10076, 43976, 53304, 40616, 24692, 63604, 36172, 1484,
                19712, 23980, 37304, 59388, 26076, 62004, 40856, 23608, 59084,
                8204,  41628, 17056, 59956, 24020, 14736, 60944, 10428, 54900,
                17036, 12704, 33472, 24004, 2424,  36724, 49268, 21868, 17144,
                55332, 31988, 42604, 50428, 4592,  32700, 27704, 60520, 21632,
                40640, 14004, 23860, 35432, 608,   52220, 12128, 56176, 41696,
                33356, 28512, 64956, 37144, 50388, 25592, 41580, 51872, 5008,
                45188, 62432, 24976, 30852, 916,   35632, 57284, 5600,  41240,
                3584,  35296, 44752, 9692,  32888, 3860,  46184, 64684, 35240,
                40832, 6020,  53624, 12304, 16176, 57392, 49160, 3592,  13664,
                52988, 33416, 61792, 26032, 14632, 47004, 1908,  29860, 49584,
                61976, 11192,
            },
            {
                28744, 51640, 9456,  60640, 14628, 36836, 19648, 39656, 59868,
                8596,  16872, 29500, 51700, 59260, 23196, 49148, 29876, 16060,
                6100,  52436, 22640, 28168, 55952, 39128, 30256, 42704, 18080,
                38496, 26240, 3792,  19612, 58268, 7876,  11416, 16840, 53668,
                28484, 47768, 11244, 44072, 30200, 17892, 3292,  31916, 15180,
                65156, 30168, 50244, 12092, 56508, 6616,  34280, 21988, 37316,
                30208, 51136, 39584, 45860, 58924, 29676, 11564, 27204, 60184,
                4020,  10200, 28172, 16392, 64352, 19996, 39160, 54168, 6892,
                50712, 59328, 28036, 55648, 46484, 19736, 27448, 36500, 15436,
                59104, 47476, 13200, 5200,  19000, 2272,  63508, 8580,  29052,
                54232, 11284, 56576, 51064, 19060, 63036, 23336, 33048, 48640,
                60208, 51908, 23628, 29496, 61212, 12760, 21684, 25352, 464,
                14236, 58652, 26736, 32712, 44952, 40612, 25176, 37600, 46556,
                28888, 11012, 3244,  19908, 44060, 64412, 33860, 13460, 39332,
                5036,  47540,
            },
            {
                64676, 33376, 42068, 48752, 31116, 63096, 4640,  55312, 31752,
                35836, 21392, 41152, 45048, 5780,  19624, 644,   64132, 46852,
                58840, 42100, 35120, 64712, 14424, 50500, 21076, 63308, 6112,
                54644, 59216, 15320, 35136, 30340, 46036, 50880, 42476, 61816,
                4112,  57968, 15352, 34292, 7492,  52728, 47708, 55576, 38884,
                44848, 19916, 35776, 4084,  43316, 39476, 52296, 2044,  47508,
                62544, 4460,  20848, 9764,  54404, 15644, 6752,  52172, 37856,
                47108, 20688, 53316, 35800, 56700, 1648,  45232, 11160, 36292,
                18244, 43740, 33520, 10300, 64464, 40208, 7976,  62360, 24684,
                1468,  8868,  42852, 57884, 33996, 48068, 14068, 59204, 37840,
                20136, 42572, 7576,  15156, 38392, 46124, 8548,  12932, 20008,
                27820, 17084, 2284,  56440, 40512, 50220, 59148, 37912, 48696,
                30848, 51016, 20732, 8892,  60268, 1192,  18516, 5856,  63564,
                21500, 42260, 49484, 36208, 52196, 6948,  26928, 56264, 59264,
                36476, 16372,
            },
            {
                6972,  13224, 26588, 1556,  17756, 24876, 42400, 49636, 14120,
                3748,  54152, 61220, 12584, 27480, 56268, 41524, 33876, 8768,
                27336, 2324,  18720, 8116,  47120, 3460,  36376, 26808, 12884,
                46372, 22604, 49200, 64564, 340,   38132, 21688, 26012, 33384,
                9152,  38796, 50296, 20484, 63476, 37060, 10780, 21992, 27716,
                2196,  10308, 53528, 26000, 31424, 64488, 18748, 28184, 58812,
                14856, 43528, 26120, 65460, 32420, 40600, 61672, 43280, 1304,
                25516, 62684, 12688, 7744,  41488, 24904, 14324, 63320, 29944,
                4828,  61752, 2604,  16136, 22996, 32020, 4044,  44456, 51904,
                38440, 30536, 22064, 53536, 27200, 39532, 22888, 31268, 16364,
                49676, 26680, 33624, 61300, 3700,  28440, 52444, 43536, 64892,
                39580, 8152,  47516, 36612, 18676, 4792,  28300, 8080,  16888,
                42048, 62220, 3336,  38820, 54668, 29896, 51224, 33940, 56248,
                16288, 7336,  59096, 30348, 16972, 22632, 41436, 9716,  19088,
                25272, 44108,
            },
            {
                22004, 38372, 53652, 58560, 34596, 52556, 9628,  22492, 58708,
                46440, 25012, 7480,  34268, 37928, 16748, 60728, 11412, 21596,
                37744, 49700, 55032, 32524, 24140, 59560, 9520,  56488, 34008,
                2632,  41028, 8288,  27364, 13396, 56796, 4736,  59720, 14328,
                45668, 23140, 55956, 5584,  25448, 45240, 14288, 60528, 5896,
                49308, 57536, 61720, 21700, 47160, 9124,  13116, 42072, 24372,
                8364,  35204, 52704, 668,   47268, 19524, 23636, 33764, 14372,
                57684, 31372, 44536, 49500, 29472, 59740, 34196, 46904, 21964,
                52144, 25296, 38000, 54196, 42384, 58684, 49828, 11544, 18196,
                55112, 60876, 16236, 10948, 62472, 7020,  44944, 55832, 1120,
                65308, 4632,  47240, 22628, 40756, 59604, 16260, 25364, 368,
                54684, 32124, 14392, 63560, 10408, 43816, 33256, 53480, 56912,
                23412, 10904, 34816, 47548, 14684, 22428, 61236, 9988,  44272,
                27528, 39080, 54160, 12340, 2628,  62060, 53368, 45632, 1136,
                31968, 55172,
            },
            {
                59828, 46944, 19716, 5264,  12272, 45580, 64840, 396,   28460,
                38348, 18072, 64208, 47592, 2136,  50528, 30264, 45628, 53580,
                15596, 62776, 12456, 44724, 40220, 17540, 52200, 43340, 30992,
                19472, 62040, 36924, 53968, 31932, 47372, 18740, 40536, 29324,
                1060,  65456, 31016, 41084, 1936,  58244, 29180, 34980, 42252,
                18416, 33276, 38284, 15448, 104,   55928, 49448, 32772, 3440,
                55208, 39120, 18048, 12324, 28540, 5300,  55504, 9012,  50872,
                18292, 37024, 2516,  22664, 17140, 4264,  55300, 8776,  40464,
                57924, 12608, 48480, 6916,  29232, 13840, 35236, 21528, 28136,
                6040,  33692, 40844, 46408, 3652,  18368, 52308, 10032, 35232,
                41264, 13372, 58040, 31436, 10620, 55432, 35184, 6364,  37480,
                58476, 45836, 24084, 52188, 26448, 60156, 20764, 13584, 2156,
                45800, 28728, 17880, 64868, 26048, 6952,  41164, 13480, 48536,
                1956,  64996, 23440, 32740, 47308, 37436, 28012, 15108, 63312,
                50624, 4204,
            },
            {
                35368, 9204,  62408, 41144, 28068, 37452, 15996, 32956, 43940,
                11084, 51540, 31008, 14724, 23652, 55180, 6536,  25336, 3048,
                43196, 31060, 26388, 4484,  61828, 29212, 552,   14988, 49876,
                57796, 11436, 24528, 17152, 43440, 62676, 10380, 54932, 51624,
                36072, 17040, 11128, 54124, 18816, 49772, 8656,  63136, 52264,
                25160, 12440, 7388,  45408, 29584, 60832, 37028, 20624, 63908,
                46324, 29904, 57356, 49288, 60364, 38508, 45380, 64164, 29140,
                41212, 6896,  54356, 64772, 39680, 51516, 19652, 27324, 108,
                16620, 31588, 45112, 20604, 62568, 1216,  56276, 45712, 63920,
                48684, 1976,  25016, 51020, 31944, 37492, 28164, 61632, 25228,
                20740, 52976, 37384, 18332, 1644,  48424, 20996, 50788, 30260,
                11140, 20440, 5796,  35060, 1292,  47936, 40104, 64160, 36176,
                51824, 58432, 4388,  43748, 50204, 37052, 58548, 31616, 24700,
                35884, 19300, 9492,  43148, 60436, 5360,  20264, 34296, 39956,
                11292, 28840,
            },
            {
                43332, 15476, 30692, 22912, 50644, 57976, 7184,  54884, 20560,
                59620, 2808,  41864, 56948, 8984,  39524, 18804, 63924, 36008,
                59232, 7204,  19768, 57348, 10564, 37212, 21656, 65340, 25808,
                4764,  45720, 51952, 2300,  7088,  23348, 34940, 6132,  21020,
                60248, 48716, 27048, 43652, 32972, 37632, 23376, 15960, 1248,
                30364, 48372, 63828, 40992, 24880, 17224, 5432,  50976, 11176,
                15592, 22324, 6700,  34108, 24432, 16128, 1900,  21364, 11872,
                60148, 24648, 47780, 9952,  32796, 12524, 43932, 60532, 49540,
                35856, 65112, 9196,  26308, 51100, 17476, 39456, 24092, 7912,
                37188, 56872, 13124, 64732, 21324, 58628, 13712, 43372, 6048,
                48104, 28932, 7460,  45488, 63376, 27488, 13708, 62012, 44548,
                17180, 61012, 41576, 29792, 57616, 16640, 7324,  31056, 25416,
                6148,  39236, 21592, 32400, 12172, 656,   19812, 53192, 61896,
                4672,  51612, 56656, 14548, 25460, 50140, 8276,  58512, 46740,
                24112, 17884,
            },
            {
                65420, 708,   56120, 47892, 3248,  18236, 25628, 40120, 48812,
                5880,  34716, 26824, 21140, 61436, 44300, 33352, 48352, 13740,
                23004, 39848, 51352, 46648, 34412, 53680, 48052, 41280, 8708,
                38944, 34640, 29908, 60716, 37968, 50040, 28220, 13248, 44184,
                4232,  39028, 7104,  61832, 13452, 4908,  56252, 44692, 39404,
                59008, 55000, 22468, 3128,  52936, 34540, 43992, 26456, 40264,
                60096, 1688,  44268, 63344, 9516,  52996, 36372, 49572, 43984,
                4124,  35304, 15664, 28224, 57760, 37356, 5404,  30768, 23568,
                53352, 4012,  41624, 33576, 59600, 5648,  30160, 10572, 43020,
                15764, 19920, 29616, 44016, 8704,  2360,  49264, 32952, 17112,
                59580, 11636, 55072, 24512, 42880, 34056, 8840,  40344, 24624,
                4476,  49260, 53572, 13276, 37280, 23172, 55168, 11212, 45036,
                15940, 62420, 9192,  53880, 57340, 29468, 46504, 6368,  16144,
                39544, 45892, 29544, 244,   40628, 54628, 30812, 13424, 2260,
                56792, 52644,
            },
            {
                26028, 39340, 33244, 8184,  60556, 35876, 11408, 62188, 30124,
                13688, 63524, 53220, 36732, 12224, 28816, 160,   52432, 9800,
                56076, 32844, 16436, 2008,  23836, 13620, 6288,  28040, 18556,
                59184, 13056, 21116, 56148, 15036, 40984, 63656, 57696, 25192,
                33544, 15356, 22564, 51468, 58716, 26060, 47664, 20908, 11336,
                5588,  17612, 36600, 10048, 59452, 13812, 56404, 8144,  31412,
                52284, 37300, 27216, 13504, 42024, 30092, 59100, 25668, 32364,
                56668, 19348, 61864, 45584, 1148,  21164, 63732, 14980, 39548,
                11328, 19332, 57016, 14648, 36776, 46764, 54788, 61692, 32476,
                52424, 60628, 4400,  35040, 54676, 40380, 23908, 55976, 38228,
                584,   64416, 31024, 15160, 5196,  58672, 52484, 2616,  56388,
                33724, 27868, 9268,  65368, 3732,  42900, 50200, 61464, 19792,
                33836, 48824, 27288, 41928, 16948, 63008, 23112, 42456, 27832,
                11452, 33608, 63156, 21680, 35380, 17164, 64184, 42340, 21260,
                36616, 6544,
            },
            {
                10524, 46140, 13376, 20868, 27540, 53448, 42580, 1380,  23020,
                46148, 16644, 7244,  49788, 4664,  58280, 17236, 26504, 41896,
                5184,  28624, 60008, 43848, 63504, 30604, 55696, 61148, 44896,
                52920, 1576,  47880, 26908, 8908,  19320, 32188, 2876,  47852,
                54756, 64316, 30672, 9992,  40736, 3144,  31748, 64992, 35316,
                27504, 43468, 31956, 47224, 20156, 29024, 1044,  62768, 23500,
                17960, 58048, 48512, 20296, 55988, 2996,  17840, 8092,  14256,
                39056, 50732, 7580,  41644, 26080, 55400, 48260, 7816,  46224,
                61608, 29304, 44632, 24172, 2400,  21396, 13060, 49436, 432,
                25996, 39868, 47660, 10900, 27372, 16004, 63044, 4732,  21584,
                45980, 34896, 40552, 51376, 21476, 38248, 17804, 30736, 63676,
                14768, 39096, 21504, 45564, 17896, 33144, 28188, 140,   40560,
                56540, 3780,  13264, 37924, 2696,  35708, 8876,  55524, 60328,
                50336, 18744, 7024,  57852, 11064, 48292, 4160,  27184, 59904,
                31396, 49040,
            },
            {
                19196, 55236, 61564, 37796, 44412, 15344, 32264, 51504, 57488,
                38688, 25380, 43256, 19572, 32196, 45748, 62112, 36964, 21172,
                64888, 49184, 12040, 8448,  38376, 20232, 3368,  33412, 16028,
                24164, 31708, 64112, 42180, 4892,  46228, 53464, 11164, 17672,
                41692, 208,   46048, 19436, 36168, 15604, 50688, 7920,  53488,
                61108, 14448, 51788, 4284,  64556, 41868, 49080, 35444, 45512,
                3392,  10104, 32252, 5128,  35216, 46264, 40624, 63060, 54464,
                2184,  29984, 23264, 13420, 51968, 31936, 17696, 35428, 26820,
                1416,  55092, 7420,  50124, 62656, 40968, 27920, 17784, 36180,
                6448,  22976, 58012, 18224, 51360, 45192, 30864, 12620, 52720,
                27068, 18228, 9712,  2988,  62460, 47520, 10712, 23064, 43736,
                48160, 1592,  59404, 25964, 51680, 60452, 8160,  14820, 46496,
                29920, 22152, 59692, 52316, 24988, 49604, 32000, 14092, 37416,
                1588,  25992, 43452, 52588, 37948, 23524, 45028, 9328,  51296,
                14148, 40944,
            },
            {
                23592, 2368,  29868, 49536, 3896,  63248, 9212,  18672, 4752,
                33856, 10440, 60708, 54632, 39652, 24052, 14384, 53876, 2684,
                44648, 18336, 35248, 27060, 58256, 42480, 50684, 10476, 40308,
                5720,  37140, 11304, 50968, 35152, 61132, 22336, 29276, 37556,
                8560,  58104, 27552, 53236, 63004, 43980, 28820, 18664, 40436,
                536,   23024, 58024, 38296, 25716, 16352, 10788, 21028, 55080,
                14664, 43072, 65016, 51616, 23624, 61088, 11524, 21232, 27188,
                47916, 65204, 34820, 58164, 3720,  10216, 41068, 59124, 50860,
                21736, 33332, 38660, 11680, 31484, 58540, 8340,  44840, 65436,
                55820, 13972, 32080, 63780, 1328,  36748, 59312, 8392,  41892,
                48896, 57132, 24184, 44340, 32988, 27888, 55844, 36416, 7308,
                19432, 54828, 35332, 5168,  12280, 38744, 24352, 54552, 36100,
                10352, 63792, 43336, 7516,  19576, 45220, 65520, 4120,  21980,
                47060, 59172, 15384, 28920, 3020,  32412, 61052, 18584, 34712,
                4748,  63736,
            },
            {
                58944, 35344, 16924, 7212,  25156, 58304, 22412, 47152, 28644,
                65236, 48556, 1072,  29592, 9172,  3952,  50524, 11032, 31180,
                38584, 55284, 22668, 1212,  52688, 14036, 25284, 47164, 63148,
                55052, 20352, 58200, 16976, 25372, 1056,  39688, 59036, 51268,
                24584, 35028, 13704, 5772,  23520, 2292,  60068, 12568, 25260,
                49104, 9288,  30104, 45616, 6432,  32856, 52832, 37932, 61472,
                30236, 25152, 39592, 15840, 7296,  28716, 50132, 34092, 44816,
                5604,  10804, 43036, 16596, 37916, 62724, 24580, 5620,  13128,
                42668, 16068, 64592, 18716, 25812, 3604,  54068, 33640, 20668,
                41604, 46520, 7988,  38592, 21088, 5688,  25004, 34116, 62376,
                14592, 6708,  61240, 53916, 16704, 13180, 288,   49908, 60936,
                28580, 10232, 41940, 31040, 56316, 47640, 20724, 62560, 5336,
                50844, 25828, 16412, 34240, 58116, 11748, 28276, 17440, 53920,
                33124, 40260, 9024,  64652, 49488, 12852, 55980, 39748, 53552,
                29408, 44284,
            },
            {
                8304,  51452, 42412, 54408, 12912, 39764, 35628, 55784, 41880,
                13212, 21072, 36288, 16084, 56444, 63964, 34836, 27924, 58824,
                7464,  15096, 61576, 46196, 31500, 7096,  17764, 34936, 332,
                28572, 44368, 6848,  30668, 54352, 12868, 44904, 15260, 3852,
                20096, 61600, 43240, 49204, 32948, 54960, 37340, 46956, 57276,
                34912, 63556, 18248, 12792, 56108, 60260, 3100,  27396, 7620,
                50020, 4444,  59844, 19484, 37680, 53992, 164,   15392, 38308,
                19016, 60196, 22020, 54220, 28148, 49012, 19236, 56200, 30072,
                60300, 2956,  47776, 52364, 43728, 37668, 15416, 50748, 10688,
                3044,  29048, 53156, 26320, 42588, 57232, 50356, 18812, 28720,
                1840,  36228, 30556, 38812, 5280,  42076, 64932, 24800, 40504,
                14072, 50964, 22516, 64312, 16024, 2176,  43276, 32444, 18212,
                39568, 3156,  48008, 30724, 1200,  53124, 39020, 43132, 61936,
                5052,  24196, 55356, 19960, 42296, 26752, 7116,  22216, 524,
                16040, 26180,
            },
            {
                11464, 21568, 32688, 64560, 45592, 27440, 1768,  16304, 7488,
                53408, 26768, 59772, 42472, 22760, 46776, 40656, 16660, 47848,
                24896, 50276, 4492,  40568, 63888, 37472, 56060, 59684, 22768,
                49600, 14508, 38264, 47384, 64948, 32896, 8020,  62664, 48012,
                31344, 55632, 11024, 18104, 39948, 8948,  20736, 6228,  31900,
                3616,  42664, 21708, 51040, 40720, 23944, 47744, 18780, 41384,
                12944, 34504, 55660, 45484, 10584, 31716, 59548, 24688, 57092,
                52488, 32016, 1868,  46344, 8320,  33076, 984,   45272, 35924,
                53252, 27152, 9860,  34644, 5912,  22424, 61044, 30396, 25164,
                62512, 17200, 59508, 11780, 48204, 15420, 9888,  54924, 39984,
                22360, 47616, 10932, 20528, 59708, 46212, 18848, 34444, 4508,
                32068, 58776, 45004, 6900,  37224, 29528, 10912, 52912, 58964,
                27968, 13536, 55772, 41340, 60904, 22676, 5992,  49108, 10612,
                30400, 13580, 36860, 2040,  34604, 16568, 63444, 45596, 58320,
                37048, 47856,
            },
            {
                56208, 38872, 1228,  18500, 30776, 10016, 48940, 62612, 32200,
                38984, 3428,  50188, 12152, 32056, 5724,  19856, 940,   60624,
                42560, 34084, 29348, 10324, 20360, 26764, 43948, 12300, 32424,
                8988,  61880, 26544, 2384,  23228, 19896, 42356, 35832, 26748,
                6876,  38720, 1944,  29136, 57964, 14204, 62516, 27908, 44688,
                15468, 54256, 37148, 1716,  8604,  35228, 14208, 62416, 57280,
                44392, 22304, 2456,  26420, 62792, 48144, 5044,  42636, 8860,
                13316, 40048, 25304, 62168, 15004, 41720, 64404, 10872, 22860,
                6796,  40280, 19780, 57952, 13876, 56052, 46240, 1096,  39320,
                49036, 36332, 4576,  33480, 62200, 30372, 45656, 3884,  65212,
                43300, 58332, 52940, 26324, 50396, 29752, 8572,  57520, 54060,
                17468, 1132,  26868, 19220, 49212, 57772, 25320, 7700,  45924,
                35172, 64796, 20852, 9332,  37016, 15148, 26616, 34704, 18876,
                58328, 44376, 47556, 61544, 52948, 30144, 10340, 50440, 31840,
                5768,  62800,
            },
            {
                28504, 14616, 60836, 5320,  52408, 59256, 20516, 43892, 23812,
                57416, 18928, 45572, 7892,  52656, 61748, 26944, 54468, 9372,
                21472, 13044, 57548, 53524, 16112, 6116,  51408, 2852,  41536,
                54080, 18532, 43188, 58980, 10636, 49388, 57160, 17360, 52184,
                22784, 46160, 65188, 50408, 25092, 41884, 48224, 53132, 23400,
                11100, 61440, 27436, 31356, 65412, 45056, 29332, 440,   32276,
                9588,  52764, 29672, 14056, 40096, 17904, 21740, 35460, 64304,
                27808, 49588, 6136,  36440, 51760, 20900, 54916, 38568, 16824,
                61196, 46480, 32176, 63568, 28452, 40924, 8828,  18896, 55192,
                13588, 44092, 23496, 19528, 320,   38280, 24284, 34860, 12360,
                16912, 32460, 7784,  14888, 3608,  37660, 21920, 11920, 47356,
                38384, 62604, 34692, 53584, 13160, 63184, 41620, 16968, 612,
                23276, 43172, 5792,  50148, 29224, 46632, 54752, 63492, 336,
                51548, 26364, 8372,  22544, 5136,  43584, 38816, 24720, 13552,
                19280, 41872,
            },
            {
                50724, 22452, 46856, 34856, 24124, 37704, 13996, 51220, 6184,
                11428, 34660, 64336, 29340, 38512, 13888, 35868, 44452, 30760,
                65352, 3432,  36140, 23828, 60840, 48184, 29084, 64820, 20972,
                36508, 3776,  51032, 34072, 39116, 5140,  29644, 1436,  11472,
                60632, 14576, 32656, 19748, 5180,  34228, 716,   17576, 39328,
                58872, 5752,  48832, 16960, 56688, 20056, 53636, 23248, 38604,
                16576, 58784, 47536, 36200, 7864,  56168, 51856, 30296, 2504,
                45724, 18384, 58588, 12012, 29884, 4272,  25780, 31672, 49948,
                12720, 24980, 152,   49568, 4500,  24144, 35572, 64532, 32516,
                6336,  58340, 50816, 41508, 54000, 8588,  59136, 20012, 51804,
                27800, 824,   63512, 41184, 33436, 61140, 51940, 27404, 42616,
                7408,  23536, 10000, 40604, 3328,  21796, 33104, 48396, 61676,
                11916, 52540, 32156, 18360, 62020, 12276, 7720,  40396, 21120,
                38060, 15696, 32748, 59280, 18444, 56232, 1496,  60792, 35584,
                54576, 2912,
            },
            {
                33492, 10288, 40652, 57552, 8076,  42688, 2552,  30388, 60404,
                26340, 41472, 548,   16264, 24660, 58488, 3080,  49972, 17548,
                39036, 51972, 45644, 41380, 1168,  33660, 38396, 14420, 25168,
                57376, 31120, 12616, 16628, 53560, 24508, 63664, 40996, 55444,
                36828, 44024, 8452,  53472, 60012, 11928, 63824, 29776, 8312,
                33308, 45852, 25072, 12592, 38256, 4356,  10924, 50308, 63392,
                6556,  41048, 19620, 64708, 1032,  43628, 11580, 60560, 16052,
                54016, 33960, 22876, 43856, 60488, 47448, 8672,  57680, 2860,
                42440, 55296, 37088, 15784, 44248, 53276, 11676, 48148, 21756,
                27524, 10052, 29696, 15908, 63968, 26556, 49608, 5824,  39368,
                55484, 45912, 24632, 56804, 18020, 44116, 1388,  63808, 15872,
                30560, 50492, 60332, 46204, 30876, 6236,  38360, 55288, 28680,
                36704, 58516, 25612, 1828,  56012, 42964, 24340, 31808, 57216,
                3892,  64872, 49920, 41064, 11400, 28160, 47260, 21352, 8024,
                45916, 25844,
            },
            {
                6852,  63324, 29760, 16772, 54248, 27128, 64988, 18024, 36932,
                47596, 53292, 21308, 55744, 48668, 42288, 10108, 23088, 56564,
                6508,  15348, 26212, 8664,  18496, 56348, 10692, 44856, 4784,
                47328, 62932, 27600, 6984,  60112, 45928, 14168, 33628, 27736,
                3612,  17776, 25848, 38356, 45544, 22244, 36296, 50164, 56468,
                19788, 41560, 2164,  51664, 60864, 43388, 26572, 35640, 45804,
                28364, 3272,  24872, 51272, 33772, 22828, 26848, 37832, 5684,
                41572, 9812,  55676, 488,   15828, 39852, 18924, 35156, 63340,
                28656, 21008, 10284, 62028, 18420, 30932, 59180, 1980,  41220,
                52884, 61476, 34652, 5208,  46364, 12964, 31228, 43604, 61620,
                10708, 35996, 20840, 48660, 9652,  13252, 24128, 35760, 5652,
                55328, 20424, 13872, 27544, 57128, 51648, 14904, 9208,  20348,
                3568,  15572, 46380, 38956, 34524, 4812,  16984, 52472, 44136,
                13048, 28700, 6160,  24088, 36448, 14908, 50832, 31112, 65296,
                16452, 58468,
            },
            {
                53172, 19752, 48840, 220,   12808, 45880, 33684, 56492, 4244,
                14920, 8812,  62440, 32900, 5528,  19148, 63420, 34124, 27768,
                47048, 32584, 58500, 63480, 30472, 22100, 52088, 61428, 17676,
                8772,  40540, 22376, 44104, 36692, 19540, 9144,  47884, 21236,
                50796, 58264, 31276, 1672,  56048, 15880, 42956, 25644, 3188,
                14652, 64264, 35440, 22380, 31812, 7444,  58928, 18348, 13836,
                54440, 60168, 11156, 30532, 15032, 58136, 46488, 50112, 20632,
                61756, 25452, 38408, 31788, 65528, 27732, 52480, 44852, 14608,
                5884,  51852, 33164, 56548, 6840,  38916, 26612, 15184, 45648,
                17972, 3400,  39480, 21488, 56156, 36760, 17808, 23040, 3388,
                15816, 30164, 6180,  38128, 54040, 59360, 31644, 48020, 57848,
                39092, 44420, 2764,  36248, 18148, 65320, 25436, 43620, 59884,
                51180, 41092, 10328, 64452, 20640, 49552, 59756, 27424, 9424,
                47920, 35020, 55280, 45420, 62840, 57996, 4884,  39284, 11104,
                42488, 36488,
            },
            {
                13744, 44176, 25032, 59936, 39088, 21284, 9544,  49616, 22988,
                44068, 28584, 37728, 11964, 46288, 29756, 39644, 13488, 59960,
                20,    20932, 11548, 48428, 43508, 6324,  39924, 26896, 35324,
                54800, 50232, 2688,  55936, 32112, 24,    52548, 61072, 5524,
                42028, 64000, 12892, 48856, 28708, 9636,  5692,  59520, 47012,
                52736, 27932, 9128,  55568, 15104, 40964, 48988, 1580,  33324,
                22052, 37264, 48300, 41724, 62108, 8964,  3756,  32800, 13636,
                29376, 46868, 7348,  19772, 49320, 5088,  11088, 23796, 60204,
                39344, 47588, 25672, 41924, 22444, 48952, 63124, 8200,  36620,
                57360, 25300, 44428, 11044, 60316, 1420,  53452, 47868, 33476,
                58196, 50348, 62476, 2032,  29128, 19804, 40484, 3808,  17344,
                10952, 24992, 62368, 8640,  42012, 1080,  46848, 34000, 5156,
                30424, 22612, 57352, 7156,  29628, 13924, 37476, 1472,  22440,
                61448, 16560, 20120, 792,   9292,  33272, 19036, 52440, 23224,
                27724, 1708,
            },
            {
                30888, 38164, 4416,  33088, 51140, 5960,  62056, 40156, 31524,
                59820, 1676,  51160, 25360, 60552, 2204,  50808, 7736,  52804,
                42696, 37992, 54456, 3060,  34620, 16184, 59400, 1504,  31576,
                20216, 14296, 29220, 11172, 65416, 25860, 38900, 30068, 16316,
                23756, 7780,  39624, 20032, 61356, 52204, 40036, 31504, 21096,
                11480, 38436, 61516, 46532, 29624, 20480, 64748, 9684,  57080,
                43252, 7340,  16920, 4896,  53508, 38908, 18232, 56356, 64040,
                1792,  51068, 59060, 13112, 41876, 33828, 56696, 1196,  30860,
                17916, 9276,  2212,  58696, 13364, 3576,  34172, 51344, 29844,
                12656, 65152, 49444, 32152, 28072, 42208, 8484,  64700, 26984,
                40860, 12660, 42992, 25920, 46100, 8368,  52356, 64952, 28048,
                34024, 52776, 48488, 32204, 54236, 22868, 10984, 39100, 62848,
                17408, 48800, 33500, 26096, 53336, 45196, 62520, 32432, 42564,
                57812, 38648, 30024, 53616, 41736, 26580, 44544, 2992,  56144,
                48136, 61372,
            },
            {
                21916, 56620, 63832, 17792, 29008, 54772, 25928, 16232, 11508,
                55920, 19656, 41200, 16792, 55292, 22132, 43844, 35764, 18036,
                24608, 61892, 14864, 28424, 23180, 55752, 49076, 12544, 46512,
                63440, 38252, 58680, 46096, 17872, 41764, 12664, 57780, 36108,
                45384, 53932, 32860, 3752,  35676, 24344, 17384, 63528, 34832,
                6672,  43688, 17908, 1236,  5532,  53664, 38036, 24360, 29356,
                50532, 61320, 25852, 35308, 21556, 28696, 44572, 24332, 36568,
                42804, 17060, 35468, 53824, 24912, 61844, 20836, 37712, 54512,
                43492, 64552, 35472, 29620, 53928, 44028, 19096, 23784, 55516,
                896,   20556, 7380,  16432, 51284, 24028, 14316, 35504, 19084,
                6764,  21976, 56180, 17044, 35256, 60824, 14400, 22660, 43324,
                7000,  58660, 20912, 15792, 28464, 60680, 49780, 13788, 27208,
                53620, 808,   12624, 41704, 3224,  18240, 10848, 54272, 6912,
                25720, 3504,  11880, 49400, 22300, 59668, 13164, 63972, 34192,
                17624, 6244,
            },
            {
                52080, 11784, 8000,  42448, 14028, 45096, 2600,  36260, 46864,
                4840,  34764, 63720, 6732,  33312, 14048, 26888, 64680, 10188,
                31304, 4312,  50312, 40972, 65040, 9436,  37572, 25200, 4628,
                42960, 6648,  23972, 34800, 3332,  54116, 49324, 4428,  10036,
                62320, 27256, 15240, 47504, 56916, 12116, 45124, 248,   55456,
                26460, 58764, 32756, 50728, 25640, 34520, 11768, 47084, 4616,
                19144, 12768, 55588, 46332, 65012, 2464,  12680, 52568, 5964,
                10504, 22700, 28292, 3900,  9148,  46256, 15584, 48936, 26172,
                12328, 50772, 20576, 15636, 38188, 6104,  59372, 10204, 46844,
                42328, 33612, 38260, 61876, 4876,  57872, 39872, 55420, 2888,
                51796, 31104, 63660, 10436, 5080,  50596, 31984, 256,   47432,
                37448, 12960, 2048,  40576, 5936,  37156, 19076, 58756, 7656,
                44008, 35804, 63852, 46896, 58308, 36148, 21804, 39940, 15292,
                50888, 45752, 63464, 32352, 16164, 5716,  37288, 29236, 8740,
                41020, 45336,
            },
            {
                26200, 35464, 48356, 23308, 34336, 57272, 20464, 64460, 52628,
                23268, 27516, 48620, 10388, 45776, 39200, 58592, 5012,  56136,
                45636, 35176, 19564, 7484,  44592, 32704, 20772, 61344, 54004,
                28376, 16688, 51524, 8940,  62152, 21496, 25088, 33944, 18912,
                50460, 1164,  21952, 65100, 40868, 4860,  28096, 51120, 15324,
                48908, 23252, 13136, 39588, 56116, 60544, 15868, 41792, 63204,
                32924, 39788, 76,    31352, 9828,  49460, 58300, 39840, 30996,
                60704, 55268, 44364, 63432, 39596, 29804, 6380,  59880, 3260,
                33004, 7428,  60844, 46144, 25172, 63980, 31872, 39884, 15720,
                60180, 26864, 54764, 45840, 19692, 29508, 47228, 11752, 61256,
                44920, 36896, 48244, 24232, 38732, 42396, 26660, 56004, 18956,
                62564, 29840, 44608, 64376, 56204, 33132, 3136,  24252, 31380,
                56436, 21004, 16280, 24424, 8820,  28000, 47804, 65524, 30628,
                19560, 35532, 8236,  56976, 39724, 46980, 53376, 24072, 50424,
                14760, 59316,
            },
            {
                4212,  18936, 53592, 996,   61456, 9252,  40840, 29788, 7288,
                43216, 14460, 58272, 30468, 53384, 2796,  20520, 48492, 29096,
                13000, 53684, 60900, 25968, 13828, 748,   51164, 17744, 34184,
                10472, 56648, 40348, 30824, 47684, 14532, 59912, 44540, 29308,
                55348, 37624, 43004, 8248,  30988, 19064, 60292, 37780, 9488,
                41748, 3712,  64216, 19520, 7168,  43988, 22272, 3120,  51472,
                26956, 59056, 16076, 42936, 19592, 34560, 26860, 15188, 47008,
                18660, 33188, 732,   14600, 51564, 19272, 35692, 42428, 22716,
                57120, 28436, 41076, 408,   11052, 49176, 27988, 3316,  52632,
                22636, 4352,  13624, 10400, 36056, 8,     33028, 22992, 27788,
                9084,  15472, 1280,  54716, 20076, 58856, 11804, 34868, 9068,
                53952, 25456, 50540, 22112, 10728, 45940, 52876, 41600, 48200,
                10424, 39212, 4472,  60932, 31852, 51916, 6220,  172,   59412,
                12700, 54948, 24296, 28120, 2704,  10816, 60608, 18636, 452,
                32120, 38808,
            },
            {
                28832, 64648, 39676, 31108, 46680, 26948, 13032, 50080, 17464,
                60416, 38844, 680,   18704, 24420, 62456, 37032, 15740, 41148,
                23492, 2388,  37876, 58132, 46884, 30048, 59044, 39164, 2940,
                48476, 64496, 20572, 5024,  37220, 27564, 2080,  40452, 7140,
                14132, 60696, 11260, 25948, 48764, 54416, 33280, 21464, 62116,
                30080, 36076, 45900, 10080, 27776, 49092, 30780, 57428, 9392,
                37468, 5952,  48784, 24708, 55016, 60568, 5120,  22248, 63176,
                7984,  49236, 37692, 25224, 57240, 10972, 64880, 52760, 13528,
                47444, 17580, 36164, 52528, 21644, 56332, 17956, 36680, 8472,
                63540, 30856, 50856, 43080, 63112, 58972, 16008, 53792, 42272,
                19544, 59704, 33728, 28768, 44644, 3936,  16668, 61272, 40052,
                4720,  15144, 33884, 7136,  17620, 26756, 61424, 15248, 6124,
                59812, 28548, 50104, 40816, 14272, 56104, 42780, 25916, 37228,
                44628, 4536,  41308, 17944, 62452, 21420, 35056, 42444, 55984,
                62236, 10008,
            },
            {
                50032, 13336, 55716, 16272, 5236,  58792, 37752, 1968,  32932,
                54608, 9368,  35900, 50520, 43012, 11368, 32060, 6908,  51340,
                64104, 9620,  33096, 17072, 52768, 5508,  42608, 23448, 14916,
                44532, 26252, 12724, 58836, 43676, 55796, 10312, 64612, 46968,
                23548, 32104, 52456, 17016, 58664, 6528,  13284, 43904, 1964,
                17500, 57032, 51572, 32300, 61740, 944,   17844, 35396, 13856,
                53888, 20288, 62836, 12028, 7396,  38048, 51288, 2676,  41632,
                27796, 12528, 59776, 21292, 47364, 31756, 1904,  27100, 38964,
                4180,  63236, 8952,  59544, 33316, 13944, 42032, 58400, 48524,
                34556, 18448, 39776, 25124, 21264, 8784,  48844, 5496,  64536,
                38996, 50636, 6388,  13156, 63988, 51948, 30804, 49024, 23676,
                46668, 60016, 42796, 38804, 51620, 30140, 392,   38080, 22608,
                17752, 33844, 53904, 2216,  19756, 34976, 10184, 17272, 49872,
                21564, 64004, 52788, 33056, 43828, 49244, 29972, 5164,  25400,
                46384, 21576,
            },
            {
                42884, 3096,  25100, 44156, 22080, 53040, 19316, 63092, 25328,
                46168, 21228, 65428, 27016, 4176,  57256, 45312, 60480, 26608,
                18808, 43744, 49048, 22496, 11388, 27352, 64052, 8612,  31580,
                52904, 36336, 544,   32992, 18640, 51748, 15968, 35112, 20188,
                58060, 5272,  35784, 44744, 2392,  39304, 24900, 47608, 53544,
                27076, 4960,  22588, 15024, 37580, 52844, 40908, 65356, 23120,
                44880, 31792, 40644, 28248, 45968, 17564, 30592, 44252, 56560,
                36068, 52880, 4636,  40880, 7056,  43912, 16484, 34436, 55852,
                20392, 43360, 30660, 24152, 5904,  45440, 2304,  25348, 12252,
                44616, 1316,  57432, 6280,  52960, 38288, 26440, 34416, 2460,
                30656, 24792, 57468, 40676, 22240, 8164,  37164, 1724,  19356,
                28196, 11972, 2512,  56672, 63288, 13192, 34740, 54768, 65148,
                44832, 25276, 12096, 63684, 45524, 23036, 62340, 57756, 32568,
                7768,  28980, 11488, 1000,  14968, 58736, 8380,  13448, 52812,
                16664, 35928,
            },
            {
                32904, 7792,  59612, 36720, 9912,  33816, 48288, 42740, 11660,
                3708,  31260, 41292, 13672, 54336, 17332, 22104, 35476, 376,
                54484, 30520, 4740,  40808, 57088, 36956, 19284, 46432, 60288,
                6000,  22000, 62696, 48944, 8064,  24396, 39380, 30756, 49224,
                672,   41776, 63960, 28260, 22252, 61736, 35196, 8668,  64904,
                39912, 12316, 60028, 42536, 25112, 5396,  11576, 47576, 26512,
                1608,  58188, 4420,  36040, 52428, 64572, 13444, 24380, 9880,
                20020, 32292, 16992, 62584, 29168, 50904, 9548,  61804, 24764,
                11460, 48872, 15444, 51028, 38068, 55060, 65304, 30360, 21336,
                60724, 27760, 14924, 46244, 31416, 17416, 44240, 56776, 14436,
                47296, 10196, 17848, 35164, 46528, 26520, 55604, 43228, 65176,
                53404, 36184, 31296, 20292, 23596, 48472, 9568,  42264, 4032,
                48880, 8040,  36996, 29888, 5568,  48444, 27560, 1816,  40472,
                15632, 46548, 57336, 38880, 51740, 26372, 37404, 64964, 40328,
                2020,  57544,
            },
            {
                62812, 19048, 51512, 29516, 61624, 14480, 6772,  29044, 55748,
                16516, 58052, 49020, 5908,  34252, 29284, 8796,  47564, 12812,
                38712, 59940, 15628, 62264, 1324,  33904, 55500, 12520, 40700,
                28536, 55068, 15340, 41260, 29452, 61004, 5740,  53728, 12064,
                26880, 18492, 51208, 13596, 9832,  55592, 49808, 15808, 31308,
                20592, 34384, 46812, 7812,  55048, 20212, 58892, 29924, 8240,
                50448, 15836, 61948, 10552, 21648, 676,   33988, 59064, 46836,
                63816, 2108,  26496, 55488, 37196, 23052, 57912, 45316, 6224,
                54164, 39736, 1156,  61100, 27468, 19152, 10376, 40528, 51676,
                6712,  55252, 35680, 64728, 3240,  59796, 10536, 51544, 20960,
                62044, 43320, 54180, 988,   62480, 14728, 10776, 32784, 5996,
                16072, 50180, 8824,  43960, 4468,  59364, 26412, 19384, 31800,
                14500, 58184, 55212, 41728, 15768, 52864, 38160, 12888, 53940,
                60756, 25280, 35672, 22552, 5512,  45904, 18520, 31072, 48672,
                9688,  27500,
            },
            {
                45464, 12236, 39876, 47248, 136,   56924, 23664, 38736, 50944,
                35648, 7752,  23168, 62156, 39500, 51476, 63696, 41528, 58424,
                24204, 7968,  27028, 45268, 50792, 7272,  24652, 48636, 1932,
                18112, 38156, 10020, 45832, 57452, 2376,  21068, 63408, 45064,
                57056, 7820,  38768, 31548, 45920, 19164, 28960, 5804,  43040,
                56632, 616,   49968, 29156, 63268, 32852, 43956, 38520, 56568,
                34840, 19312, 43344, 25428, 56028, 48228, 40848, 5596,  15508,
                38340, 50168, 42896, 10648, 14224, 184,   18436, 35892, 28312,
                32456, 64332, 22396, 8272,  35328, 47396, 4288,  32880, 16216,
                37848, 9580,  49276, 23032, 12968, 28576, 41352, 24300, 7356,
                36976, 27712, 5332,  30148, 20384, 50892, 58748, 22656, 40284,
                57460, 25392, 61512, 47244, 40592, 16460, 37072, 51844, 61064,
                27916, 20780, 864,   23952, 60092, 9316,  30936, 21252, 43632,
                3868,  50740, 7192,  16996, 59784, 54324, 3744,  23724, 60388,
                21156, 53960,
            },
            {
                31652, 23372, 5068,  16636, 27040, 41756, 63860, 2932,  20652,
                61204, 27896, 43776, 18280, 2152,  25476, 15008, 3676,  19412,
                33200, 52924, 36088, 12204, 21108, 29512, 16328, 62920, 32752,
                59068, 50680, 23332, 34508, 13380, 26056, 42648, 32908, 15480,
                36580, 23900, 61104, 4588,  58292, 41400, 1692,  52100, 23296,
                60880, 10644, 18452, 36940, 13916, 2488,  16940, 4904,  22192,
                12900, 53708, 39040, 32572, 8036,  27940, 18736, 54664, 30100,
                8476,  21468, 58564, 34144, 48420, 53232, 41072, 59728, 4088,
                13412, 46452, 18136, 42548, 57584, 14080, 53784, 44160, 62524,
                24540, 42232, 18220, 33128, 54524, 37664, 62828, 596,   32248,
                58180, 16244, 49620, 60432, 41992, 37740, 2640,  28620, 45424,
                12752, 712,   18680, 28024, 33020, 6992,  56876, 2012,  44064,
                10860, 39440, 50744, 32768, 18320, 44876, 2916,  65032, 34320,
                19072, 30188, 62592, 42556, 27848, 32628, 11888, 44708, 35356,
                14560, 6092,
            },
            {
                37700, 56572, 65216, 35436, 54308, 18852, 31488, 45576, 9768,
                14776, 47624, 37844, 11940, 59588, 45992, 32048, 56588, 50152,
                5544,  46916, 17772, 63936, 40384, 54228, 37420, 43288, 9264,
                26336, 3796,  65484, 6488,  54092, 37484, 51232, 9100,  3588,
                29344, 47168, 53084, 16612, 34324, 25756, 63892, 36240, 14196,
                45104, 27380, 54448, 41140, 24080, 51960, 59896, 48732, 62780,
                28896, 46820, 2448,  59564, 14960, 61612, 35936, 51332, 25596,
                61168, 45468, 5328,  24820, 65532, 30884, 8584,  21440, 50304,
                37636, 25468, 52868, 2648,  29780, 23904, 60064, 20508, 520,
                29288, 57652, 2624,  44408, 6084,  50052, 19232, 46312, 13148,
                52824, 39728, 23344, 12416, 33220, 7616,  17420, 48756, 62808,
                35116, 39076, 54520, 10908, 63772, 49240, 12976, 24448, 35272,
                64388, 46428, 5988,  61988, 35780, 56748, 49392, 24884, 55200,
                10544, 39488, 13608, 49348, 9240,  63992, 39732, 56968, 664,
                42176, 50876,
            },
            {
                26676, 2828,  43592, 10528, 49632, 7236,  12788, 52748, 58916,
                33120, 860,   54852, 29936, 52384, 9596,  36520, 22016, 28320,
                42052, 60656, 9848,  31224, 2500,  58396, 4544,  13784, 52744,
                19948, 42000, 47728, 31388, 19600, 60648, 16800, 48532, 59128,
                40776, 11468, 2128,  21900, 49452, 12008, 6884,  57192, 32484,
                38760, 3856,  64576, 6572,  46336, 9752,  35608, 26168, 41772,
                10384, 6652,  23876, 51004, 44596, 4152,  11876, 42692, 1160,
                33300, 12756, 40116, 16348, 3476,  46492, 26408, 62240, 16580,
                55960, 9892,  59276, 33524, 49528, 11204, 36960, 6396,  48268,
                52888, 12356, 63932, 27104, 21944, 9048,  56596, 34772, 26208,
                65332, 3000,  9108,  45876, 56320, 64128, 25268, 52852, 8576,
                20888, 30452, 50780, 4156,  22188, 38316, 30132, 53164, 17524,
                3932,  29660, 15092, 26392, 7300,  11764, 40184, 14940, 5288,
                46824, 58404, 22060, 33900, 2768,  20276, 51320, 16536, 29484,
                63172, 18380,
            },
            {
                58848, 13732, 21764, 33008, 60460, 24732, 36920, 40364, 26692,
                21884, 64600, 6152,  24188, 19848, 40900, 6596,  65140, 13188,
                1008,  23680, 55908, 44348, 25884, 49420, 22416, 46056, 35000,
                57068, 29548, 16088, 10580, 44844, 416,   28408, 23128, 54672,
                19396, 65076, 32284, 43392, 61656, 39796, 20408, 47816, 8700,
                17168, 50544, 21716, 33540, 28552, 56992, 19476, 280,   31860,
                55136, 64192, 17520, 29796, 37528, 20812, 64812, 23216, 57208,
                17876, 52584, 29076, 55396, 20052, 38108, 12248, 42812, 2356,
                30316, 44640, 6644,  39428, 16752, 64444, 41592, 26528, 32244,
                15536, 39004, 34496, 47204, 60828, 15928, 29704, 4432,  43144,
                18084, 48296, 35736, 26912, 19640, 3640,  43084, 31880, 15316,
                5456,  60968, 42760, 57136, 14364, 45340, 60540, 8804,  42416,
                23076, 58452, 53676, 48192, 43060, 22580, 29268, 61068, 36984,
                26492, 352,   43972, 56300, 37560, 25704, 46524, 7052,  24500,
                10304, 48128,
            },
            {
                7932,  40276, 53264, 29196, 1984,  44892, 56676, 5580,  17444,
                50612, 42148, 35268, 57960, 48916, 60816, 17064, 44916, 53944,
                38208, 16020, 34328, 6864,  18940, 11272, 64672, 28032, 7520,
                61868, 2112,  38488, 54964, 63592, 40392, 34404, 12412, 6780,
                35844, 26348, 56228, 9160,  27828, 15920, 53872, 29112, 62068,
                24940, 55876, 12260, 42952, 15916, 62184, 39552, 13184, 45428,
                20936, 34480, 40196, 57900, 8660,  49640, 31824, 46952, 9936,
                37008, 63596, 47952, 7860,  60788, 32004, 57332, 48688, 34636,
                22936, 62976, 13204, 27024, 21240, 3228,  55152, 9384,  44796,
                58956, 20080, 4216,  10840, 40936, 52516, 37964, 62080, 10456,
                22276, 55144, 30652, 60108, 38824, 13532, 54832, 36532, 59048,
                46708, 24288, 17220, 34256, 26256, 1188,  19824, 55916, 47876,
                33992, 12496, 37120, 19464, 64088, 1856,  51264, 17836, 53428,
                31760, 64368, 16464, 6352,  52808, 14376, 31684, 61000, 38784,
                54820, 34388,
            },
            {
                45292, 16720, 62304, 47904, 19416, 14996, 63668, 30416, 3092,
                46576, 9220,  16276, 13072, 31408, 1820,  26464, 33380, 48388,
                30020, 62960, 50056, 40004, 59624, 36788, 31928, 16796, 40804,
                23932, 12848, 49060, 25660, 21256, 4928,  52840, 43152, 62496,
                45672, 15396, 5668,  51456, 37464, 284,   45404, 5252,  42532,
                31384, 1204,  59420, 47292, 3184,  52476, 7608,  49660, 60564,
                15280, 52232, 1400,  12572, 25788, 55476, 16100, 2700,  41136,
                26160, 3996,  22392, 34984, 41904, 928,   15620, 6384,  54212,
                18796, 40872, 51240, 60344, 45848, 35292, 50592, 18860, 62788,
                7232,  49980, 23528, 55776, 31660, 1652,  24008, 49548, 58552,
                40204, 14952, 1124,  51500, 6800,  47808, 22356, 10700, 27220,
                40796, 3448,  9928,  48412, 65396, 41456, 36392, 5276,  27416,
                63080, 588,   41224, 8260,  31976, 57712, 34888, 6760,  42388,
                9360,  23604, 47860, 29124, 61960, 10852, 58176, 43136, 4280,
                20060, 30772,
            },
            {
                960,   24312, 5852,  35872, 9500,  41452, 22752, 34244, 54588,
                60992, 38480, 27976, 45248, 4568,  39292, 56040, 10740, 7400,
                20244, 3220,  12056, 27316, 53332, 12,    47436, 55552, 4808,
                51116, 35912, 58436, 7844,  32208, 14572, 59440, 24936, 30584,
                1344,  39132, 47572, 23648, 60276, 34512, 57524, 22772, 13400,
                39704, 35388, 19604, 26100, 37684, 30952, 22028, 25524, 36424,
                5112,  27668, 47612, 60912, 43868, 5756,  35128, 28528, 60056,
                50328, 15044, 44336, 11448, 27248, 50676, 24012, 64196, 28100,
                8396,  36524, 364,   31476, 5832,  57312, 29416, 14160, 25232,
                37064, 42132, 28444, 59860, 17868, 45620, 14016, 6028,  27824,
                33264, 8088,  61596, 42116, 16768, 29396, 57828, 120,   50320,
                63428, 37956, 54348, 29144, 7256,  52388, 16336, 31584, 10716,
                21696, 51784, 16680, 55404, 25536, 45504, 13928, 21384, 48760,
                59144, 12824, 39280, 19324, 41680, 34544, 1448,  22560, 52164,
                13076, 63820,
            },
            {
                59416, 37652, 56524, 27168, 51832, 59072, 48652, 7796,  12296,
                20456, 24908, 56820, 63800, 51176, 22680, 62300, 43556, 24972,
                60120, 55116, 45788, 22220, 15020, 42784, 9572,  60408, 20636,
                44280, 30644, 17976, 62248, 46216, 37000, 48416, 8388,  17388,
                52064, 58352, 20072, 13172, 30956, 10336, 18372, 64800, 49656,
                53308, 9444,  63016, 5700,  54860, 11424, 64120, 43696, 57752,
                9584,  41840, 22372, 32320, 19308, 38972, 63056, 52968, 20036,
                7144,  31272, 62596, 57904, 53588, 19392, 38556, 43396, 11756,
                58460, 48016, 15052, 53696, 23752, 11864, 40360, 1172,  31948,
                53092, 3420,  13244, 35144, 7764,  64760, 36328, 43664, 19704,
                52916, 46840, 25432, 21344, 34536, 64912, 43768, 32356, 18428,
                13288, 21460, 59524, 32812, 12140, 23364, 62032, 48972, 58720,
                38752, 46192, 29160, 60476, 11276, 3284,  39780, 62272, 27412,
                1640,  33360, 55840, 4100,  50316, 27088, 17320, 47272, 36124,
                27588, 49532,
            },
            {
                18508, 11324, 43928, 32556, 13564, 1424,  37920, 28480, 43644,
                52128, 276,   32588, 7496,  19276, 36716, 13692, 52712, 17748,
                41256, 36588, 32416, 4368,  63424, 29952, 25028, 34276, 14612,
                53720, 10344, 1064,  42464, 27384, 2960,  19976, 55836, 33796,
                11056, 28296, 42700, 63620, 4516,  48164, 38016, 26972, 2296,
                16532, 44588, 28672, 48892, 41460, 17256, 34040, 1836,  18592,
                30276, 56100, 65280, 3192,  50736, 14192, 23892, 10404, 43052,
                56296, 37764, 18316, 3340,  9008,  32452, 60484, 4788,  30152,
                45228, 20824, 26244, 65208, 43068, 19660, 62208, 48740, 58108,
                16900, 63844, 47304, 54320, 21692, 50824, 29564, 57608, 3376,
                63380, 12428, 37004, 56424, 9996,  4348,  24572, 53600, 7836,
                35320, 44164, 1560,  17860, 45676, 37708, 2276,  43156, 14580,
                6460,  33428, 4228,  20660, 36472, 50076, 30212, 54980, 17000,
                37112, 45236, 63168, 8416,  23864, 54472, 65120, 9932,  57320,
                6060,  41556,
            },
            {
                30404, 53168, 4132,  21016, 62764, 25648, 55644, 18176, 65312,
                15124, 35816, 48024, 11200, 42864, 29380, 6320,  34456, 768,
                27904, 9320,  50896, 57880, 19504, 39308, 50224, 3288,  64328,
                27044, 39660, 22836, 56460, 51368, 12184, 64852, 41504, 22880,
                61144, 36396, 2636,  52716, 25588, 55780, 44048, 7724,  33748,
                60744, 22340, 12696, 57248, 24024, 4060,  59536, 47104, 53388,
                38088, 11612, 16588, 36028, 8352,  58316, 47288, 44,    33452,
                13052, 48520, 25244, 40428, 45964, 13796, 49192, 35668, 16776,
                52616, 2772,  33776, 9760,  37552, 6736,  45308, 34648, 5260,
                22088, 43900, 9512,  26800, 4,     40404, 10940, 25036, 16448,
                39392, 31148, 2120,  44924, 51084, 14872, 40216, 60760, 47668,
                28324, 57244, 51132, 25752, 60668, 54032, 19972, 27004, 56528,
                23940, 64960, 42044, 53356, 15272, 63672, 23316, 4976,  10676,
                52168, 20100, 30012, 15084, 36696, 44084, 4948,  39952, 32440,
                15040, 23328,
            },
            {
                8360,  64688, 46880, 16368, 40688, 6256,  46136, 32064, 3928,
                40828, 59748, 23108, 54200, 15844, 58768, 49988, 45500, 56476,
                64716, 15936, 23924, 6216,  44220, 12396, 57000, 7776,  37360,
                45808, 59460, 5464,  34680, 16696, 30328, 38232, 4580,  50240,
                6808,  46348, 16876, 33216, 40584, 14912, 21148, 51568, 30172,
                54988, 38828, 6776,  32084, 36560, 50752, 14712, 28228, 7320,
                23716, 50072, 46072, 26780, 54784, 30720, 40864, 27748, 59632,
                21864, 5848,  52684, 28880, 64588, 21676, 1340,  57084, 23612,
                63064, 39720, 58936, 49516, 17228, 54680, 27928, 24636, 11232,
                39808, 30540, 37776, 18712, 61956, 32360, 46328, 54568, 7452,
                50044, 59004, 18480, 29020, 62412, 20240, 36764, 11516, 2844,
                22732, 15640, 5552,  41080, 9044,  34152, 4296,  30960, 50900,
                9440,  17924, 47900, 27512, 8808,  43784, 33232, 47096, 41056,
                58416, 25708, 2952,  48408, 60444, 11936, 28852, 19740, 62232,
                44788, 55216,
            },
            {
                34288, 38888, 28044, 50380, 34848, 60252, 11608, 21600, 50012,
                9092,  26812, 2664,  39700, 63160, 25868, 3876,  21904, 31772,
                12112, 47424, 38004, 61996, 33268, 17008, 47020, 21828, 31420,
                18344, 12480, 48748, 62856, 8516,  54656, 24260, 44440, 13724,
                26980, 56432, 9412,  22568, 61824, 6108,  59456, 11504, 41740,
                3408,  47788, 19264, 65128, 43560, 8944,  21588, 39468, 63088,
                33092, 772,   61096, 40032, 21056, 4108,  63776, 15692, 44900,
                36236, 61652, 2124,  16212, 34472, 54824, 26916, 10864, 42256,
                7636,  13700, 28592, 4920,  32008, 60320, 14624, 50552, 61328,
                56024, 2544,  51644, 57292, 15580, 4604,  60280, 20608, 34784,
                42632, 23556, 9216,  48500, 6404,  26400, 55868, 30744, 42660,
                49728, 38336, 64632, 29992, 14252, 47220, 62732, 39640, 12864,
                36116, 59816, 2608,  39140, 58556, 124,   18888, 61076, 6936,
                13832, 32184, 42644, 56384, 21792, 33468, 52608, 264,   25552,
                48780, 3552,
            },
            {
                21636, 14124, 800,   57440, 8632,  24016, 53136, 37352, 62276,
                29636, 55532, 46948, 19852, 31280, 9552,  38700, 61156, 18628,
                2800,  42508, 28976, 52856, 956,   26040, 54452, 61296, 1788,
                52596, 24924, 32384, 41320, 20716, 46928, 228,   59800, 32620,
                19644, 64296, 37500, 49116, 696,   31496, 36728, 18128, 63208,
                25728, 15012, 58820, 344,   27052, 54084, 60508, 45436, 12980,
                19160, 42868, 5640,  15132, 34360, 11320, 51932, 19068, 6656,
                50096, 30944, 41936, 58620, 8508,  39352, 44908, 31764, 51060,
                36648, 19844, 55932, 47124, 22540, 1464,  41760, 8212,  19864,
                46132, 13800, 23660, 34208, 48088, 43032, 12832, 28856, 2836,
                64656, 14212, 37648, 54944, 33720, 43416, 3992,  63900, 18688,
                58208, 33180, 10956, 52580, 23144, 58120, 18560, 6980,  55024,
                44872, 25308, 31564, 13392, 22312, 55864, 28704, 37836, 24408,
                49292, 63872, 9980,  38716, 6232,  16672, 46940, 58584, 36800,
                12400, 60616,
            },
            {
                52336, 25060, 61780, 19020, 30788, 42724, 15924, 4820,  44380,
                13152, 17428, 34376, 5636,  52984, 44968, 14544, 49168, 35384,
                54856, 60336, 8156,  19840, 36004, 10520, 40372, 28448, 42600,
                9112,  57644, 15060, 3164,  28844, 61412, 35720, 16312, 52604,
                40960, 29740, 12376, 54312, 26112, 57708, 46612, 23584, 8748,
                45452, 29464, 51984, 35260, 12044, 17584, 30792, 2416,  57596,
                51480, 29204, 53852, 24916, 56844, 43720, 37320, 26156, 56072,
                9676,  23848, 12612, 47420, 20304, 5316,  63388, 18256, 4140,
                61708, 25080, 43484, 11028, 52156, 38540, 64308, 33168, 29188,
                36304, 5840,  65172, 27608, 7832,  22484, 38940, 52216, 56584,
                26620, 47040, 504,   61380, 16120, 23084, 52116, 13600, 8488,
                26828, 2024,  20416, 43840, 36608, 460,   42216, 28864, 49056,
                20988, 5104,  63072, 52140, 34676, 48928, 11532, 16156, 54424,
                35048, 1016,  18260, 53324, 26932, 62672, 42004, 9052,  30232,
                18100, 40564,
            },
            {
                43352, 10176, 45540, 37628, 48712, 3064,  56140, 33504, 25068,
                58372, 41612, 64512, 11956, 36832, 58000, 24036, 7220,  40792,
                26644, 22472, 14772, 48576, 57268, 64080, 5284,  14404, 49936,
                35880, 65240, 44868, 38332, 52132, 11640, 26008, 8880,  58140,
                6428,  3204,  45180, 18536, 42860, 13892, 50776, 4796,  33920,
                56988, 38076, 21480, 7740,  49372, 41960, 37204, 6416,  25912,
                35312, 9428,  64364, 46892, 7564,  60132, 1180,  47960, 33068,
                65092, 17264, 53768, 37428, 26092, 52068, 14476, 57864, 29352,
                46396, 844,   60032, 34872, 18324, 26368, 3960,  55276, 16732,
                49900, 58860, 40684, 11332, 53444, 63028, 35948, 16964, 9924,
                32172, 19556, 41108, 29808, 10064, 59864, 36136, 46932, 40040,
                54792, 61260, 48568, 6804,  55724, 26404, 15224, 64244, 10876,
                33676, 57324, 16292, 41172, 6420,  45092, 64356, 4328,  43404,
                21080, 29404, 59616, 45332, 35824, 2396,  14492, 23240, 51108,
                6860,  57172,
            },
        },
        {
            {
                46336, 60916, 11484, 48156, 57304, 39148, 60024, 32232, 23676,
                48056, 8508,  13104, 25012, 34516, 2388,  15984, 62896, 25308,
                48680, 640,   43316, 27608, 54180, 32096, 44612, 10404, 1436,
                49948, 32760, 54440, 43392, 10124, 51488, 15704, 41308, 23908,
                54284, 7472,  48516, 32556, 25992, 10364, 29332, 15852, 1256,
                23616, 28572, 34132, 14696, 56388, 6676,  45024, 59852, 35928,
                4696,  8976,  62544, 43508, 36804, 28812, 15732, 4956,  40636,
                49688, 20608, 15364, 3956,  28488, 34064, 19356, 26364, 48648,
                62908, 45672, 676,   61276, 33680, 9244,  20764, 48300, 24792,
                18800, 54036, 27580, 38796, 62892, 47556, 19152, 28048, 64264,
                16260, 35564, 6764,  46848, 3360,  61996, 22068, 18076, 3720,
                26684, 38032, 64916, 33572, 43892, 6528,  55344, 13168, 8916,
                65176, 28144, 38136, 6088,  51444, 16560, 38800, 13004, 30396,
                2508,  61896, 56608, 47988, 32260, 21952, 16900, 42664, 100,
                39040, 20208,
            },
            {
                28992, 32756, 15728, 41788, 7504,  21424, 27352, 10668, 53352,
                35848, 28648, 4172,  54004, 58336, 22436, 51892, 45260, 4700,
                20148, 57324, 39636, 23740, 5600,  64700, 18036, 56640, 41268,
                62316, 6944,  12524, 28188, 468,   35492, 46496, 20008, 2332,
                36924, 17812, 59876, 14252, 20772, 54880, 50980, 33452, 44532,
                10832, 50076, 39272, 2512,  22248, 49080, 29648, 16976, 42588,
                53052, 48032, 25812, 18072, 916,   52116, 34204, 59324, 11748,
                36420, 64332, 31800, 39992, 61804, 50724, 1252,  38460, 57956,
                9072,  24924, 13632, 42556, 49156, 15820, 36680, 55176, 12056,
                8088,  41852, 49408, 3372,  6880,  31760, 60208, 8792,  39744,
                50520, 25708, 59584, 54172, 14144, 34184, 57388, 11992, 60248,
                46152, 30936, 96,    57416, 22864, 28204, 62188, 46280, 39364,
                1756,  17864, 53592, 61388, 30948, 4440,  47004, 64768, 6996,
                23172, 39380, 13888, 4636,  41592, 53160, 7560,  46836, 54824,
                13912, 58288,
            },
            {
                49596, 5060,  63824, 18888, 34060, 62004, 50260, 6092,  40472,
                16988, 61008, 43992, 48716, 7116,  41412, 32440, 9300,  35896,
                53660, 31348, 16360, 60116, 49348, 2696,  38200, 52696, 22820,
                30500, 19656, 45412, 64136, 21848, 55432, 58804, 12708, 65120,
                49868, 30604, 42216, 46276, 35344, 63176, 3468,  19488, 61380,
                55904, 7180,  63968, 43248, 27100, 61924, 12816, 412,   64272,
                21712, 14404, 31096, 54780, 10420, 65528, 21804, 48088, 29720,
                8856,  53956, 44484, 17828, 23548, 13296, 55968, 44220, 16768,
                30512, 53768, 19676, 56996, 28228, 64796, 4024,  59296, 45256,
                34208, 64176, 21324, 55984, 17040, 25172, 43816, 53556, 22524,
                2704,  31476, 40736, 18912, 29760, 37880, 44100, 51516, 8244,
                23508, 49272, 19340, 10700, 41140, 51120, 15576, 21072, 26356,
                58968, 33884, 10428, 21856, 41464, 57696, 25584, 33980, 20324,
                54340, 46084, 26960, 19312, 59784, 37664, 29684, 61268, 25856,
                34944, 9908,
            },
            {
                22428, 39444, 25132, 53884, 44600, 3692,  14676, 45972, 58772,
                732,   20800, 31056, 36868, 14116, 18964, 60264, 28320, 65244,
                13180, 6312,  46604, 10048, 20524, 34280, 27140, 8180,  15596,
                48584, 4108,  39252, 50524, 30872, 4740,  24612, 39484, 27708,
                9372,  55964, 3000,  24568, 6744,  38096, 48884, 41404, 13320,
                31612, 25576, 16288, 54348, 10488, 37724, 52032, 33292, 25196,
                40056, 3268,  60512, 45940, 41416, 27136, 3020,  43712, 14916,
                25316, 204,   59084, 5064,  47716, 30060, 7604,  22052, 35140,
                4568,  50632, 37420, 3008,  11124, 23204, 40096, 30808, 26244,
                1212,  14188, 29264, 36628, 45928, 58008, 716,   14352, 37148,
                57080, 11688, 44912, 65260, 868,   6360,  26008, 16536, 40968,
                34940, 55512, 13500, 60964, 36884, 4068,  32628, 49076, 5768,
                43000, 56084, 47488, 7184,  36128, 18340, 11064, 43676, 58896,
                15416, 35780, 9464,  50304, 1904,  24740, 11112, 15892, 50996,
                3572,  43844,
            },
            {
                57444, 51752, 788,   10896, 37132, 30204, 23272, 54644, 33024,
                25604, 51456, 10204, 63992, 27044, 55832, 1000,  47224, 22100,
                42792, 26248, 37104, 62768, 41480, 12800, 58596, 43524, 61032,
                36500, 26344, 57832, 14196, 17432, 42404, 8096,  32488, 45200,
                15804, 21900, 62568, 11792, 58080, 16308, 27496, 23308, 4868,
                37196, 45864, 20456, 35084, 4464,  46660, 18884, 7828,  58284,
                49368, 36688, 6792,  33536, 13216, 19196, 38828, 55304, 62432,
                50440, 19892, 38024, 11164, 34868, 52880, 40240, 58884, 65204,
                11832, 41300, 62400, 31756, 46988, 53416, 7732,  16604, 50816,
                38532, 60376, 52660, 8392,  12228, 33688, 40688, 29812, 63524,
                18088, 52568, 8108,  23544, 49576, 55396, 62304, 32380, 2016,
                63676, 5564,  29276, 44396, 25472, 53716, 9728,  63332, 36528,
                12744, 24140, 29832, 14504, 50864, 62376, 1600,  48616, 28680,
                4856,  63932, 42620, 56964, 33476, 63372, 5632,  41112, 31928,
                62936, 18136,
            },
            {
                30680, 14172, 59312, 27888, 48656, 65492, 17832, 38396, 11812,
                62672, 44932, 2880,  39708, 23724, 49776, 11880, 38040, 17032,
                58756, 50336, 1812,  29396, 55948, 24356, 51028, 31404, 872,
                11840, 53048, 8944,  33612, 60240, 47228, 53772, 61948, 1272,
                51276, 34228, 39792, 52772, 30108, 43744, 9600,  59656, 51952,
                62976, 712,   50412, 58892, 30012, 23252, 56700, 42296, 28028,
                11332, 17288, 55892, 23564, 51404, 58556, 8420,  31048, 5684,
                33836, 42056, 27896, 57164, 63784, 25256, 15740, 1968,  46632,
                27660, 23900, 7468,  17748, 35520, 43672, 62008, 21816, 58116,
                5372,  19284, 42524, 23524, 62268, 51000, 5888,  21120, 46816,
                3748,  35040, 28080, 15132, 39344, 20012, 11012, 45856, 21744,
                52580, 39176, 17600, 47900, 2476,  20240, 56760, 29420, 17132,
                51948, 560,   63908, 44884, 26780, 54588, 31788, 38424, 8836,
                52284, 24152, 30692, 12224, 17660, 44632, 49004, 20288, 56036,
                37932, 8044,
            },
            {
                45640, 34832, 42048, 20860, 7144,  55852, 1908,  42420, 7636,
                19624, 35636, 16068, 57548, 5416,  43576, 34292, 4384,  55044,
                8460,  32792, 18608, 52896, 4516,  16244, 6728,  46200, 18752,
                65488, 41532, 24164, 2456,  37540, 26880, 11504, 18952, 36516,
                58504, 4504,  26484, 19240, 46780, 1536,  55224, 34880, 18232,
                28264, 41860, 13732, 8564,  65256, 39724, 2812,  14716, 61684,
                45088, 20756, 63996, 29580, 4156,  47548, 36228, 16784, 22548,
                60420, 12892, 48664, 6916,  20572, 45304, 9828,  33244, 19268,
                55076, 49092, 14804, 60156, 26596, 108,   13184, 48216, 34996,
                10880, 46260, 31804, 2532,  17772, 27148, 59500, 10296, 49808,
                25360, 42044, 57824, 61064, 47252, 35800, 29180, 58624, 14276,
                27188, 8556,  57856, 33732, 61840, 14228, 39844, 45268, 4372,
                60552, 40976, 19640, 37960, 3680,  22828, 13360, 60664, 17100,
                40320, 756,   20928, 54080, 38840, 27600, 58688, 23732, 1148,
                12484, 26428,
            },
            {
                64468, 3416,  17064, 60828, 32152, 13324, 46808, 26864, 57156,
                30616, 53664, 47040, 28356, 60928, 17980, 52988, 29860, 40956,
                62480, 23364, 11620, 44668, 35428, 63808, 39900, 22016, 34828,
                28792, 47672, 56784, 20768, 63312, 5788,  49496, 23100, 29008,
                42872, 12820, 48508, 7572,  32896, 64992, 21396, 40672, 11744,
                7292,  56320, 24104, 33492, 16840, 48988, 53504, 35552, 30908,
                1096,  50348, 39480, 10208, 42268, 25248, 63128, 52912, 45392,
                2892,  55940, 17260, 32256, 3548,  42280, 51860, 61036, 29732,
                38496, 2804,  42656, 51372, 56132, 19940, 32408, 40800, 24804,
                56380, 27716, 65476, 48996, 37332, 55728, 43116, 15552, 32856,
                61864, 6456,  12516, 31792, 2448,  8768,  54384, 3620,  42568,
                36160, 49996, 11708, 22680, 41948, 6872,  27120, 34884, 22128,
                48320, 32304, 10072, 58712, 34144, 6060,  42924, 20180, 57208,
                32900, 49404, 61288, 46668, 3212,  8348,  14032, 36052, 42888,
                53848, 48136,
            },
            {
                21612, 56840, 9656,  50784, 39980, 23752, 34988, 61736, 49572,
                4760,  22772, 12864, 8972,  32292, 21124, 64444, 14048, 25484,
                3092,  47808, 38756, 60168, 26732, 48732, 9900,  54764, 59076,
                3200,  16844, 12916, 44868, 30480, 41288, 14984, 55812, 6720,
                64360, 17460, 54752, 60792, 37624, 14560, 5344,  49504, 31456,
                47152, 61148, 38388, 44716, 27544, 5548,  21572, 10056, 24736,
                54548, 6208,  34040, 16024, 60112, 1496,  13684, 28060, 10572,
                40024, 24780, 37292, 54236, 62368, 27080, 35948, 13000, 6636,
                57620, 21908, 10696, 30300, 37516, 5796,  63684, 8668,  53676,
                1760,  15348, 39864, 7008,  13240, 34320, 3292,  22816, 54948,
                38664, 19460, 52036, 22540, 43312, 17760, 50768, 24264, 60232,
                19148, 64184, 780,   30160, 55088, 50460, 65016, 11172, 57660,
                25988, 7388,  55460, 16756, 49980, 65496, 53408, 27868, 45684,
                11368, 25400, 7016,  15916, 34504, 65108, 50740, 30144, 61652,
                18868, 5592,
            },
            {
                28600, 37348, 25072, 44880, 2664,  52864, 18784, 9976,  15420,
                40460, 64168, 37552, 50700, 41524, 6656,  45616, 10224, 51176,
                34980, 57456, 15072, 44,    19700, 31896, 13948, 25052, 5624,
                38564, 50564, 8072,  35572, 52192, 776,   59128, 38684, 46400,
                31712, 24884, 40984, 10524, 23116, 29048, 58680, 25268, 54464,
                16052, 20916, 2292,  52700, 11556, 57284, 46356, 63140, 37868,
                43480, 59320, 27208, 46656, 53996, 19880, 34488, 43452, 49276,
                65008, 30516, 7820,  46156, 14744, 23064, 580,   48568, 17244,
                44888, 33784, 64232, 4248,  24280, 45576, 50268, 29124, 18476,
                44268, 60028, 21248, 52976, 25416, 46492, 63404, 30372, 8484,
                45692, 68,    36664, 56544, 26420, 64784, 33608, 39808, 5052,
                31288, 43748, 25380, 46380, 15756, 36872, 2052,  18728, 52740,
                14552, 39116, 28660, 45976, 11652, 24840, 37032, 7844,  3324,
                63508, 42072, 37464, 28704, 55112, 21652, 45324, 3784,  10288,
                33180, 40356,
            },
            {
                49180, 54756, 12992, 29768, 63744, 5496,  58792, 43848, 33560,
                128,   26228, 59184, 4052,  56528, 35860, 27012, 58816, 17344,
                42748, 21548, 29496, 55092, 7708,  41584, 60804, 51912, 43880,
                32584, 62504, 27332, 55424, 17972, 24544, 33296, 20200, 9764,
                3504,  50344, 35176, 48,    44908, 51496, 38856, 2628,  43424,
                36056, 6260,  30708, 63560, 18640, 41152, 31888, 15388, 4208,
                18992, 13112, 22724, 7400,  38392, 29992, 56984, 6072,  17988,
                924,   20984, 59964, 11084, 38908, 52916, 58708, 40864, 62784,
                25340, 52564, 14012, 39812, 59340, 16188, 11500, 58400, 37008,
                4336,  33772, 29880, 9820,  57952, 19508, 40508, 12380, 51244,
                28184, 63848, 15676, 48340, 5772,  13516, 46332, 10472, 15988,
                56468, 7944,  38072, 59388, 20816, 9240,  32092, 46908, 42800,
                3088,  63384, 21468, 42024, 1724,  31124, 48112, 15496, 35228,
                54708, 18612, 51376, 59924, 10844, 40752, 17304, 26068, 52712,
                58380, 15120,
            },
            {
                400,   62212, 7596,  35652, 16176, 38848, 27440, 22076, 54732,
                46236, 19968, 30444, 16448, 24044, 48560, 900,   39072, 31340,
                5088,  9380,  65076, 44200, 50272, 36720, 2516,  29808, 19124,
                10828, 22384, 40656, 3608,  64260, 11308, 43784, 61784, 52828,
                14376, 59608, 20704, 56872, 16732, 61892, 8576,  19408, 64132,
                10992, 58276, 50168, 25888, 36812, 268,   60696, 28160, 51484,
                57520, 32344, 64276, 49904, 11188, 61516, 23076, 51624, 36892,
                58068, 33724, 50508, 28092, 18856, 5660,  30836, 20876, 8260,
                28772, 1504,  47608, 19468, 54448, 34704, 41636, 22168, 47712,
                13756, 51192, 63768, 43440, 5468,  49772, 2148,  60416, 18148,
                42612, 24244, 10352, 30724, 58456, 37492, 28248, 61972, 22320,
                52924, 48912, 13408, 4256,  51620, 41320, 60988, 24324, 29884,
                59072, 36080, 5424,  52008, 62100, 19056, 40436, 60524, 23112,
                29872, 13492, 4872,  23904, 1216,  32028, 60764, 38272, 7196,
                23572, 43284,
            },
            {
                18376, 32004, 46700, 20568, 57756, 48224, 31812, 7064,  13420,
                60740, 9388,  53180, 43584, 11924, 63396, 20120, 54572, 60808,
                47036, 52628, 25124, 33716, 11420, 21316, 63164, 16252, 57048,
                48244, 59908, 14828, 45764, 29912, 48992, 7300,  26416, 37484,
                30468, 43008, 27680, 4860,  31816, 47740, 12972, 34156, 28392,
                46628, 22676, 42016, 8688,  13600, 49144, 23148, 7876,  39552,
                47364, 1684,  36336, 17356, 44476, 3440,  41024, 9092,  26928,
                13192, 40540, 4124,  43260, 64000, 47076, 34116, 55352, 11944,
                43560, 36612, 61324, 9296,  27596, 6796,  1120,  25968, 62140,
                7688,  39568, 23860, 16740, 37732, 32460, 26848, 35912, 55456,
                6924,  33924, 53348, 40184, 3488,  19920, 54720, 1004,  40820,
                35064, 18364, 29132, 63408, 34524, 27488, 56252, 6532,  12672,
                49176, 17624, 32716, 24084, 13644, 54336, 9604,  50408, 272,
                58112, 46396, 34084, 43456, 49020, 56656, 13612, 46748, 64248,
                35900, 11212,
            },
            {
                59980, 52188, 24100, 41348, 1428,  11364, 51828, 64496, 41496,
                35580, 49908, 39256, 2352,  34604, 29260, 14988, 8136,  23776,
                13044, 38052, 18180, 3672,  58308, 27832, 46608, 7456,  37824,
                552,   26072, 36252, 6100,  20728, 34904, 54420, 16888, 2272,
                48328, 62704, 12156, 53028, 36968, 25868, 41640, 56400, 52104,
                14944, 3516,  33736, 55836, 59768, 44284, 34808, 54896, 11508,
                25204, 42896, 5872,  28308, 53364, 33120, 15300, 63368, 45552,
                53616, 23744, 16216, 56728, 9224,  25488, 2524,  38188, 49976,
                16072, 56664, 22620, 31128, 43084, 64580, 52340, 32228, 54872,
                18928, 28232, 444,   56580, 11672, 62372, 22596, 14896, 46932,
                3808,  59620, 21048, 44228, 62848, 9108,  32588, 47792, 25752,
                60388, 2548,  45444, 23696, 10776, 17128, 40,    38872, 20564,
                53688, 9144,  46452, 58352, 37984, 5952,  26524, 44112, 20244,
                38368, 8064,  64560, 27364, 19544, 8776,  28932, 2244,  20672,
                50548, 27524,
            },
            {
                38992, 4072,  9744,  63348, 55668, 25608, 17948, 3024,  28820,
                24240, 5332,  18936, 55848, 60292, 45872, 51476, 42416, 36148,
                1640,  28636, 61412, 40716, 14848, 54196, 23828, 42976, 33204,
                53484, 9776,  56188, 50692, 60436, 12996, 39872, 57752, 23080,
                6364,  18976, 40328, 8236,  64628, 21100, 1208,  5852,  24224,
                39572, 62564, 17504, 29592, 20996, 5704,  16276, 64936, 19720,
                30792, 61884, 56648, 21040, 59588, 25044, 48448, 19612, 31164,
                2608,  61376, 48548, 30156, 36200, 14312, 59684, 18316, 65368,
                4964,  33096, 3136,  51220, 17400, 46720, 14168, 36756, 10128,
                45100, 59648, 48248, 30704, 53636, 45348, 8376,  58224, 39404,
                30124, 12560, 49720, 16932, 25156, 51756, 14764, 43064, 6984,
                12344, 31504, 57940, 42368, 53532, 48792, 62160, 44724, 33632,
                64744, 27768, 1108,  43168, 29728, 56176, 34628, 62700, 31740,
                11300, 53804, 16564, 59444, 36780, 52372, 42236, 34200, 55312,
                4888,  44496,
            },
            {
                56796, 34756, 15712, 29456, 37048, 43764, 33728, 46896, 56876,
                15748, 62852, 32352, 26808, 10356, 22272, 3972,  32492, 63720,
                49536, 56568, 44904, 5940,  49072, 31584, 3080,  13032, 64736,
                19672, 41264, 31200, 23868, 43052, 1044,  27556, 65164, 45452,
                35964, 56220, 24332, 45068, 14780, 57984, 50220, 61316, 31092,
                9612,  53760, 44976, 1556,  38832, 52584, 27156, 42304, 4148,
                50452, 14640, 9316,  39048, 12512, 848,   36676, 7320,  58864,
                11452, 38324, 6788,  21772, 51164, 41900, 53156, 23044, 45676,
                26104, 40720, 59236, 12232, 38880, 24308, 60000, 20908, 3764,
                42064, 12832, 35296, 4772,  20192, 40792, 1484,  50968, 19168,
                25980, 65240, 36188, 1656,  29364, 38936, 57000, 21400, 63588,
                52520, 37888, 8428,  20484, 5188,  37068, 13868, 23176, 6988,
                40224, 15560, 60568, 11460, 21296, 17108, 3892,  14816, 47944,
                24520, 40912, 5428,  22496, 3296,  15052, 63192, 24728, 16848,
                31044, 13332,
            },
            {
                25772, 21328, 45664, 50196, 7284,  13976, 60496, 21588, 9088,
                37912, 45328, 12964, 49292, 7036,  39748, 57976, 18552, 26696,
                15440, 21768, 9732,  34780, 19592, 60088, 39348, 51508, 28476,
                47412, 15944, 5016,  63092, 17908, 33532, 8616,  51044, 15260,
                10856, 32568, 49924, 2032,  34624, 29756, 17884, 43236, 37376,
                20052, 48968, 25592, 12416, 63344, 46436, 10140, 32816, 37832,
                55636, 23876, 34396, 46740, 51232, 65532, 42164, 55360, 27664,
                43728, 33508, 17648, 63156, 152,   11280, 32124, 6424,  29508,
                9608,  48396, 20592, 55144, 6076,  2192,  29740, 49412, 56956,
                26176, 65112, 22868, 50488, 15592, 27852, 63376, 33528, 10444,
                54292, 41728, 7980,  46100, 61328, 11424, 5336,  35968, 17696,
                28016, 49716, 15720, 65264, 32936, 26836, 55844, 31008, 51576,
                56852, 25220, 36340, 50636, 47348, 64380, 42312, 52200, 2000,
                57588, 29324, 50760, 32776, 47136, 39804, 10976, 58912, 49168,
                8452,  62884,
            },
            {
                40432, 2380,  61400, 54520, 19320, 3612,  40172, 50652, 54072,
                728,   59436, 21260, 42156, 65524, 35288, 53372, 11376, 43912,
                6576,  54476, 30360, 64464, 25640, 10972, 56232, 7864,  22640,
                1788,  57592, 37576, 11260, 47772, 53724, 20168, 30644, 41816,
                4436,  60612, 28004, 53940, 39508, 10348, 46784, 7408,  13504,
                55516, 4656,  35720, 57732, 31596, 22812, 2472,  60984, 47732,
                17964, 1420,  43688, 28852, 6116,  16852, 32332, 21684, 14152,
                52444, 24528, 46864, 55096, 27860, 45192, 38704, 60500, 54176,
                13780, 62696, 35224, 28428, 44852, 63432, 33824, 40236, 7460,
                16692, 32112, 38772, 9408,  60832, 44224, 23748, 37532, 3472,
                47656, 22532, 15380, 56096, 33712, 50340, 23848, 59052, 46672,
                3004,  24672, 56592, 40548, 1844,  47768, 17752, 9956,  2764,
                45544, 19480, 5104,  32252, 7760,  23836, 38984, 27696, 60852,
                19220, 44896, 12984, 62228, 56224, 26704, 6020,  43536, 36900,
                19696, 52756,
            },
            {
                6876,  28408, 11240, 33064, 23712, 64988, 31160, 12108, 25428,
                34464, 30148, 4828,  55212, 17024, 24548, 30640, 2760,  62296,
                36532, 39956, 47980, 1240,  43376, 36908, 17528, 45080, 35184,
                61020, 24984, 43884, 28720, 58928, 4160,  38940, 61740, 25728,
                57076, 21448, 17276, 5256,  63620, 23276, 26988, 59596, 33872,
                65508, 28840, 42700, 7756,  15636, 50020, 56800, 13556, 25740,
                8336,  63948, 52664, 19448, 58216, 26040, 9948,  49988, 2268,
                61852, 4900,  40276, 8264,  57908, 15996, 19680, 2900,  34140,
                42844, 17796, 7676,  37884, 15048, 50804, 19000, 11580, 53680,
                46792, 1852,  59044, 54640, 5756,  30952, 13352, 57296, 17800,
                60328, 32084, 4884,  27368, 19756, 43804, 512,   31328, 41384,
                9840,  34716, 45052, 12572, 22744, 59252, 42668, 61472, 35172,
                12200, 63144, 41304, 52968, 59004, 10412, 35768, 13460, 6756,
                34068, 9460,  37812, 1340,  18648, 30608, 54156, 22932, 908,
                32656, 46184,
            },
            {
                17556, 57192, 43180, 38184, 48892, 27728, 57528, 6036,  43420,
                18480, 48420, 38656, 27884, 1544,  13628, 47008, 50356, 20444,
                59508, 12868, 23388, 16108, 50948, 4040,  62864, 29548, 13864,
                49748, 8812,  52408, 14904, 35332, 22988, 46024, 12520, 1352,
                47696, 37012, 43644, 13464, 48620, 56032, 2932,  52040, 21828,
                368,   17148, 24004, 61828, 37320, 19800, 40892, 29668, 35796,
                59144, 40152, 31808, 4444,  37980, 62312, 46064, 39964, 57616,
                36112, 20160, 30752, 12860, 35560, 25020, 63644, 49728, 22228,
                57760, 1316,  51684, 24576, 60532, 8832,  27128, 58204, 36392,
                21860, 28832, 42676, 18792, 35892, 47372, 52528, 7684,  43304,
                28748, 51116, 63716, 40464, 8620,  53844, 16500, 64568, 13544,
                54656, 60064, 5784,  30132, 53696, 7748,  38236, 20440, 29784,
                49124, 15220, 26932, 2088,  29540, 17684, 56452, 46508, 21844,
                54264, 64892, 25628, 43344, 8308,  35528, 15540, 47408, 65504,
                12728, 59480,
            },
            {
                49724, 35348, 14384, 380,   8524,  16088, 45964, 35876, 60776,
                14976, 63836, 10252, 51272, 44044, 61676, 37384, 8596,  27400,
                32644, 4940,  53212, 28252, 57892, 33044, 21840, 53856, 5452,
                39212, 32640, 20540, 2172,  63548, 7916,  55028, 16616, 34416,
                52232, 9360,  61504, 31000, 19944, 35400, 41280, 15844, 45228,
                38312, 48080, 53020, 11360, 45492, 3240,  54028, 6392,  44820,
                21992, 11292, 15360, 55552, 22948, 12944, 33812, 7592,  29072,
                15572, 49108, 64876, 52768, 42468, 4220,  47128, 10112, 40176,
                26828, 31428, 46240, 56060, 21204, 40860, 276,   44152, 64008,
                4536,  14612, 48808, 11648, 24664, 1156,  64724, 21640, 39424,
                10984, 2672,  35760, 57500, 12548, 25892, 38000, 47636, 28892,
                22216, 18780, 39076, 62032, 16704, 28020, 51304, 4788,  24552,
                58192, 37272, 55648, 20656, 44620, 62436, 4268,  50240, 30200,
                39768, 15888, 48536, 22200, 57648, 61536, 51200, 4000,  38512,
                29344, 24616,
            },
            {
                3224,  22084, 63284, 53444, 59808, 40596, 20784, 1876,  52716,
                23644, 7344,  57004, 19648, 31852, 58576, 22728, 42380, 56140,
                17580, 46136, 62612, 40924, 7256,  12144, 47204, 41708, 18732,
                65360, 27224, 57020, 41920, 49064, 31432, 27624, 42716, 64460,
                23716, 28708, 6480,  38696, 59040, 11768, 7108,  32308, 61556,
                8892,  30852, 58388, 5544,  33384, 28064, 63480, 17312, 49236,
                2028,  51008, 27712, 43432, 48344, 372,   54424, 18808, 24068,
                44068, 9468,  1244,  22568, 59848, 33084, 29028, 14524, 52676,
                5972,  64956, 11852, 4168,  33240, 48928, 13280, 31208, 24060,
                51320, 61172, 33072, 56884, 41128, 27532, 15824, 34588, 55060,
                25384, 14796, 45752, 22260, 49332, 61748, 34168, 7672,  3868,
                52060, 32848, 49608, 972,   36096, 47068, 13716, 63868, 43664,
                536,   8640,  46860, 11512, 39620, 33524, 24892, 42564, 488,
                58356, 5736,  52952, 31772, 13584, 40524, 27344, 20224, 55364,
                9104,  41540,
            },
            {
                52228, 31304, 44608, 18756, 26124, 32532, 56312, 11284, 29700,
                41452, 46300, 26116, 36072, 6404,  16144, 4308,  12268, 52224,
                232,   35516, 10284, 19520, 37804, 60176, 26620, 792,   55576,
                10180, 46516, 4632,  17448, 37372, 10784, 58200, 5728,  19424,
                40452, 14880, 53564, 680,   45660, 24180, 50912, 27912, 54432,
                13108, 25800, 40168, 21168, 50904, 59664, 23624, 12536, 32116,
                56264, 61844, 34680, 8876,  59536, 30504, 42112, 64100, 51992,
                59264, 38140, 26984, 45924, 17716, 7628,  56164, 37264, 61108,
                16668, 35132, 19436, 43220, 28540, 62644, 53420, 17944, 6584,
                39772, 9804,  20380, 3652,  62244, 50840, 45368, 5508,  58836,
                48404, 62592, 18244, 6388,  30844, 2060,  19256, 55620, 43140,
                62684, 14792, 9200,  44516, 25596, 57152, 9740,  31856, 53248,
                17896, 34188, 23428, 65500, 52324, 7360,  14200, 61336, 19432,
                11760, 27792, 36496, 2564,  45524, 6280,  10944, 34124, 44216,
                16296, 62392,
            },
            {
                12104, 27460, 5300,  10240, 47284, 4244,  50416, 37760, 61636,
                3768,  33424, 12940, 54000, 40724, 49696, 29632, 64844, 39012,
                25500, 57412, 30548, 44700, 23884, 49588, 15424, 31228, 36656,
                23164, 13120, 60724, 53668, 25272, 21416, 45408, 50068, 2624,
                32624, 56776, 48044, 26296, 64912, 17380, 57740, 43484, 3760,
                19512, 64344, 2180,  43896, 14264, 36552, 8932,  42640, 37928,
                25244, 18040, 3536,  20992, 37036, 16716, 26168, 6264,  2928,
                13816, 34220, 55420, 11736, 39760, 51124, 20944, 888,   43856,
                24312, 48532, 39088, 58808, 15768, 5096,  37744, 56716, 34248,
                26384, 45264, 53836, 37584, 8192,  31656, 12408, 20268, 29240,
                72,    32784, 38748, 53200, 44372, 58316, 39780, 24516, 11956,
                27640, 40928, 23232, 64908, 19472, 3728,  41756, 21728, 39172,
                61312, 50528, 30716, 4056,  16620, 28500, 55308, 37636, 32348,
                47176, 63236, 17464, 60352, 50320, 24556, 63964, 58448, 48012,
                5760,  37044,
            },
            {
                20104, 55912, 39016, 34520, 64348, 23512, 13916, 27336, 17776,
                49376, 21460, 59200, 764,   62596, 24248, 45696, 33900, 20236,
                48076, 14520, 5720,  64404, 2140,  56396, 8924,  62240, 51768,
                44116, 34276, 30092, 40340, 4,     62416, 14460, 36564, 60444,
                11532, 43968, 20976, 34160, 10212, 36920, 5392,  21976, 39548,
                35328, 55676, 49336, 17516, 26584, 56628, 692,   47872, 65188,
                7392,  41048, 46036, 53108, 63292, 11028, 50152, 39496, 47480,
                31148, 21352, 61908, 3364,  30044, 63180, 26064, 12724, 57544,
                30356, 8080,  54392, 1520,  25804, 46056, 22012, 11252, 48020,
                59356, 15172, 29488, 23356, 17548, 59572, 40112, 53576, 37016,
                9112,  42964, 23576, 11480, 28372, 15708, 8884,  60880, 48748,
                35840, 2488,  55100, 30248, 50308, 33820, 59552, 15764, 26992,
                6516,  12788, 37908, 59344, 42944, 48272, 21980, 9620,  51652,
                23628, 43240, 8144,  21136, 38276, 14760, 30352, 500,   22560,
                32132, 60060,
            },
            {
                1588,  45860, 61484, 16424, 51888, 42260, 58048, 6840,  44344,
                65012, 8752,  38812, 28180, 18660, 9912,  58064, 2336,  8328,
                59872, 41988, 54052, 27236, 17736, 33956, 42604, 21064, 3220,
                59272, 6188,  15940, 50672, 9056,  33140, 54728, 26020, 29672,
                8040,  63184, 16452, 3856,  42068, 29772, 48744, 62756, 15040,
                45748, 10408, 31796, 7272,  63024, 29836, 52148, 20644, 15468,
                30564, 60012, 13188, 28208, 5504,  56652, 22240, 35128, 57796,
                18256, 7276,  41520, 49000, 16164, 35856, 47436, 41032, 3640,
                51616, 21872, 10760, 61956, 35844, 52392, 7724,  64840, 41016,
                5604,  744,   63104, 42940, 47820, 2056,  25048, 63892, 16680,
                49244, 61048, 57124, 5196,  65376, 36856, 51560, 21236, 45124,
                5436,  58856, 16892, 38288, 6960,  11628, 54032, 46116, 2908,
                48416, 56960, 19932, 25636, 1068,  35052, 57568, 5588,  40620,
                2156,  54636, 33392, 28824, 56792, 42692, 53924, 18176, 40128,
                26280, 49148,
            },
            {
                15008, 41696, 22284, 7776,  28956, 2716,  20276, 35352, 54912,
                30528, 15772, 52680, 48280, 43660, 13984, 37336, 51840, 16724,
                23536, 33012, 11916, 50688, 37268, 47052, 12848, 29060, 39392,
                18508, 48220, 28116, 64504, 43188, 19144, 3560,  41272, 52156,
                23228, 37284, 49672, 54672, 60628, 13268, 52308, 33000, 988,
                28760, 22852, 59488, 37800, 41608, 11664, 45000, 34840, 4324,
                55208, 48868, 23332, 39216, 32852, 44260, 15224, 8756,  27540,
                60536, 44856, 24820, 53320, 9980,  5380,  59428, 18460, 34236,
                64296, 45464, 32360, 13936, 42228, 29568, 16968, 24652, 30944,
                19796, 50792, 33300, 12920, 55684, 35088, 10292, 44568, 4476,
                27268, 13412, 19872, 31724, 47840, 26076, 736,   33208, 18032,
                30788, 52748, 12768, 47604, 60816, 28480, 36684, 23072, 64208,
                29608, 41064, 9496,  45296, 62608, 12476, 18164, 64552, 26636,
                15652, 59592, 12828, 46512, 2904,  9476,  35292, 51096, 12680,
                56912, 8780,
            },
            {
                29612, 54372, 32676, 12616, 37952, 60596, 48596, 10380, 40048,
                1980,  23888, 34856, 3528,  32012, 56352, 26212, 42752, 29192,
                63472, 3132,  40032, 21520, 61704, 3940,  53792, 64124, 8288,
                24956, 56632, 11544, 37780, 22788, 60008, 47856, 12684, 17960,
                58636, 1584,  31540, 27288, 6716,  24316, 18664, 8984,  54212,
                61296, 47648, 4744,  52788, 19128, 2948,  23948, 59032, 27180,
                10308, 36132, 932,   18468, 52252, 2772,  61688, 41248, 54496,
                1604,  12204, 37020, 65100, 32860, 23348, 54516, 28592, 7164,
                15700, 26924, 58220, 19852, 50284, 2580,  60324, 43940, 54188,
                36460, 57564, 27196, 6828,  21184, 50148, 30136, 58032, 21960,
                51596, 36028, 41108, 3236,  55108, 42636, 14036, 57352, 7576,
                63912, 26520, 21936, 41360, 32,    18684, 44384, 8840,  14176,
                34848, 55016, 15924, 32764, 53208, 29056, 39168, 46016, 30904,
                49788, 35940, 5212,  63708, 19172, 26788, 61512, 6668,  44720,
                64588, 35956,
            },
            {
                50860, 4008,  58668, 46944, 25040, 53904, 31556, 14540, 25964,
                59940, 46644, 11204, 63220, 22328, 7100,  60456, 4528,  35100,
                54848, 46304, 9416,  31132, 7132,  25572, 16112, 32700, 50072,
                44452, 35468, 1524,  53340, 5580,  27508, 56904, 34564, 4904,
                46388, 42840, 13908, 39984, 58060, 46884, 37416, 43580, 26120,
                40256, 16368, 12712, 25632, 33944, 55584, 62352, 39880, 17700,
                51264, 43448, 64372, 57748, 11756, 29828, 48204, 24052, 32548,
                16720, 51636, 28864, 19344, 300,   43760, 38304, 11268, 50212,
                42060, 36400, 5412,  39276, 9068,  56424, 34448, 11996, 3524,
                15792, 9692,  46840, 38124, 65168, 3960,  14652, 40660, 33156,
                7904,  56372, 46160, 17296, 9584,  21972, 61304, 39584, 49460,
                35480, 43796, 9060,  33128, 63248, 25672, 57744, 50448, 1704,
                60972, 21456, 5012,  50644, 8048,  22404, 3776,  54076, 10076,
                20684, 42092, 24768, 52280, 39692, 48676, 22108, 32424, 3484,
                17208, 23856,
            },
            {
                38472, 7148,  19316, 63916, 224,   17492, 44572, 62972, 6152,
                42384, 57084, 17848, 54552, 41060, 50592, 18984, 12048, 48844,
                20552, 14632, 58228, 51704, 44836, 59776, 41204, 22652, 2824,
                58696, 14824, 20548, 46500, 32284, 16168, 9756,  38688, 65280,
                25144, 19644, 61728, 9872,  21556, 2464,  64764, 56500, 3616,
                20612, 35728, 57864, 43296, 49604, 7964,  14308, 46900, 5476,
                31296, 21196, 8100,  25436, 45952, 37248, 20116, 4676,  64044,
                38804, 6608,  46964, 55920, 14492, 60876, 46484, 20988, 62424,
                55624, 1384,  23672, 63320, 46408, 26488, 21304, 48876, 62748,
                40012, 23760, 59812, 17804, 42272, 26404, 52868, 18416, 62920,
                2576,  29248, 23984, 63448, 33932, 52112, 29136, 11416, 24040,
                3724,  15208, 56748, 50984, 5076,  15504, 39140, 31488, 42548,
                26868, 37688, 47264, 25264, 41680, 61580, 35568, 14232, 60504,
                28,    57036, 16348, 29536, 11352, 57968, 14620, 54864, 41804,
                28328, 60736,
            },
            {
                11176, 43156, 27312, 35404, 40996, 8488,  22756, 50940, 33744,
                20504, 29128, 37896, 4724,  14448, 30348, 45352, 39132, 61600,
                1476,  24880, 37092, 17592, 28240, 12312, 55996, 36364, 10448,
                29876, 61672, 26184, 40280, 63520, 51148, 44512, 29984, 54412,
                7308,  33432, 50580, 53640, 35792, 29976, 15612, 11344, 31864,
                51328, 8300,  63260, 456,   28604, 32016, 22048, 36880, 54168,
                61132, 13648, 33332, 40516, 62344, 14724, 53472, 9896,  49580,
                43288, 22520, 58848, 35440, 26888, 7936,  31524, 3872,  25392,
                13404, 48356, 52860, 17520, 31576, 14576, 38036, 5520,  28332,
                45788, 53152, 1136,  30816, 8700,  61364, 35708, 44936, 10928,
                59024, 39052, 12220, 49328, 6256,  37708, 1964,  45504, 54660,
                20100, 60052, 37460, 46432, 29824, 55292, 20836, 6708,  59424,
                17608, 10564, 64996, 2264,  18804, 57812, 44336, 25688, 32984,
                47304, 37840, 8276,  45216, 34128, 1200,  37136, 46956, 9288,
                20480, 48192,
            },
            {
                33356, 15832, 56360, 51388, 12280, 29636, 57332, 37140, 1364,
                12696, 49844, 8956,  26456, 35952, 65520, 23368, 8268,  28056,
                32508, 43936, 64632, 6056,  34712, 344,   48452, 19548, 52220,
                42736, 5148,  54792, 12160, 7648,  24256, 1808,  20900, 14980,
                48428, 27872, 356,   17200, 44972, 24920, 49972, 41760, 62052,
                28128, 45188, 23476, 39452, 17568, 64964, 44064, 11456, 2132,
                27812, 48688, 56076, 3060,  6436,  28744, 57432, 35264, 26568,
                13428, 31224, 2768,  10812, 41472, 52196, 16812, 57292, 40248,
                33360, 29064, 10560, 43620, 60744, 160,   51480, 58164, 32364,
                20300, 12564, 34688, 56368, 48436, 13328, 1744,  23120, 47956,
                26736, 53752, 15860, 43532, 60280, 26336, 17936, 64748, 32200,
                40684, 28096, 2304,  12420, 23756, 9424,  47428, 53164, 13048,
                45172, 51900, 33696, 55740, 30388, 11584, 16612, 51928, 6268,
                64092, 22760, 53840, 60020, 6128,  65420, 24032, 30432, 62688,
                1976,  58272,
            },
            {
                23472, 45336, 3344,  20604, 59864, 47388, 4952,  16552, 53440,
                64020, 43216, 60588, 47248, 53736, 528,   58212, 16596, 53032,
                57372, 11032, 41352, 53972, 21996, 60980, 38952, 7356,  63972,
                16624, 34392, 48600, 37252, 18344, 58492, 41180, 62196, 36092,
                57212, 12052, 38336, 63888, 4392,  55428, 7136,  34764, 18556,
                5364,  14464, 54100, 10664, 56636, 4340,  50328, 58588, 24260,
                42288, 16920, 36216, 51536, 23520, 42880, 17412, 1172,  61044,
                55200, 17952, 63612, 48916, 20724, 62040, 36784, 47664, 8632,
                19100, 65232, 35752, 6672,  22776, 54584, 41872, 8820,  16608,
                64400, 6368,  43356, 25476, 21420, 39160, 55236, 32300, 5444,
                20056, 36424, 1220,  31244, 21056, 56988, 14516, 48412, 6564,
                9704,  52936, 17692, 63832, 42256, 36248, 62044, 32868, 4316,
                29096, 22848, 40772, 8512,  37144, 48984, 3160,  39340, 29176,
                18388, 13272, 31932, 26564, 20076, 43340, 16904, 51552, 39888,
                14088, 52920,
            },
            {
                30656, 64944, 9644,  39360, 32248, 24380, 61544, 41340, 27032,
                31640, 23024, 3056,  18504, 32320, 10436, 41844, 35360, 5832,
                50008, 19324, 2912,  30268, 46616, 14688, 25832, 31848, 45564,
                27964, 22860, 1024,  61056, 29220, 33380, 47568, 10612, 4612,
                43488, 22660, 58924, 41460, 32076, 14024, 59896, 22192, 46712,
                58648, 37296, 32652, 47572, 20472, 35476, 15196, 30172, 38884,
                8992,  63388, 20328, 11540, 58768, 32600, 47564, 21964, 40584,
                7444,  45248, 38264, 25128, 6296,  29544, 3084,  23200, 59188,
                44984, 2520,  55928, 39896, 27932, 11712, 35012, 24660, 44948,
                50404, 37512, 60944, 4184,  51236, 16328, 29380, 59644, 42684,
                50472, 64968, 9216,  55180, 39948, 4672,  29948, 42120, 36184,
                61620, 24940, 44592, 31684, 58140, 1264,  18356, 25016, 38408,
                63020, 824,   15200, 60488, 26764, 20372, 62572, 54832, 9708,
                42728, 50376, 2184,  40420, 48848, 55764, 11984, 6952,  26392,
                35880, 5100,
            },
            {
                41316, 48624, 26060, 53336, 1828,  14112, 45156, 10416, 19792,
                6852,  58440, 15144, 39588, 25120, 49592, 21024, 46420, 25948,
                38232, 14224, 24192, 62404, 8704,  50744, 56836, 4328,  12980,
                58084, 9280,  41972, 51400, 13560, 3788,  53572, 26648, 17892,
                52608, 29492, 8628,  19404, 26464, 48456, 9984,  39280, 52820,
                3508,  24752, 61648, 1912,  41652, 26476, 60600, 6760,  55672,
                47472, 812,   30764, 45820, 38516, 8684,  64388, 51076, 11900,
                28192, 53080, 33448, 14784, 56724, 44144, 12128, 53756, 15460,
                26544, 50620, 13992, 20448, 48908, 63108, 18564, 59716, 2340,
                27552, 14600, 29956, 57844, 10600, 45280, 63648, 7996,  12500,
                17252, 33440, 45116, 23584, 47676, 11080, 52420, 58664, 22072,
                13508, 49952, 35008, 5356,  21600, 51324, 14284, 41596, 57356,
                48932, 19352, 53812, 45596, 5780,  42276, 33308, 23096, 46812,
                58868, 35232, 62120, 10920, 3844,  28916, 34576, 61608, 44756,
                56196, 17816,
            },
            {
                8004,  13136, 34844, 62292, 18680, 50104, 34104, 56308, 38724,
                48544, 35276, 45700, 55296, 63040, 13148, 59724, 2328,  63988,
                31472, 44920, 58376, 39604, 34044, 18120, 42160, 21220, 53084,
                38416, 63056, 19364, 25492, 45028, 57364, 22088, 39352, 32596,
                63304, 46776, 5348,  51844, 36840, 62576, 936,   27724, 31088,
                11732, 17284, 50932, 29292, 9256,  52984, 45308, 18536, 33864,
                21928, 52516, 14424, 61204, 26316, 3576,  15568, 30408, 36860,
                4532,  20040, 60632, 1292,  50064, 35336, 64224, 31784, 42284,
                37788, 5700,  60424, 33060, 44492, 3164,  30672, 38944, 53068,
                21500, 8168,  41528, 33676, 19092, 2808,  36540, 24588, 40896,
                57408, 28148, 3716,  14140, 62620, 34640, 19608, 352,   27372,
                4036,  55900, 15964, 8540,  47660, 27952, 54988, 10160, 6448,
                34928, 12036, 32088, 24424, 51224, 58304, 13952, 928,   27092,
                5352,  21132, 15556, 53228, 37640, 58628, 22220, 47860, 748,
                20628, 59884,
            },
            {
                28444, 22368, 58320, 5996,  37816, 28044, 7872,  64676, 2348,
                52652, 11348, 28512, 3828,  7640,  29688, 34348, 16224, 9700,
                54488, 4016,  51984, 12072, 27548, 1160,  65292, 36164, 48132,
                2536,  32660, 55288, 6284,  35920, 16336, 8204,  59692, 1492,
                13748, 35144, 16548, 56620, 23444, 44736, 15784, 42192, 65460,
                57460, 44380, 38980, 14100, 62800, 22984, 4764,  12656, 64596,
                26784, 43144, 35644, 6240,  54084, 19396, 44832, 56116, 59616,
                24452, 47328, 10000, 41740, 17644, 27240, 8216,  20196, 624,
                62452, 30040, 24600, 9236,  57556, 15952, 6504,  46764, 11052,
                55868, 48164, 62412, 23540, 53356, 48872, 27628, 54360, 576,
                21868, 51956, 60732, 38068, 25784, 7668,  44168, 63732, 38452,
                46624, 29752, 60380, 39756, 64940, 36768, 31144, 45440, 61452,
                27172, 43268, 64300, 3420,  9944,  36372, 17680, 65096, 38440,
                56280, 30284, 45512, 25512, 18104, 42832, 8692,  14924, 32204,
                38360, 50496,
            },
            {
                3532,  46320, 42240, 15536, 55488, 43904, 21660, 30464, 17468,
                25404, 62256, 20792, 43916, 37404, 51392, 42616, 56692, 22572,
                28412, 36000, 20492, 6452,  59392, 45136, 30104, 10656, 24624,
                17340, 29016, 11464, 49744, 40756, 64036, 31176, 43956, 50296,
                23820, 42460, 61188, 2496,  11612, 33216, 51128, 19856, 8132,
                35032, 21464, 6520,  56204, 48464, 36952, 31084, 41116, 50124,
                3824,  10084, 56768, 22960, 40384, 49568, 34172, 92,    42356,
                14132, 65456, 32044, 54788, 23660, 58744, 39820, 52644, 48240,
                11516, 54772, 17876, 36476, 50156, 42076, 26160, 64680, 36120,
                31720, 1080,  16824, 38636, 6568,  43088, 14156, 61320, 34440,
                47332, 6780,  30884, 18212, 49960, 56248, 31768, 16872, 51688,
                11296, 42408, 20316, 24596, 12296, 2708,  16992, 23144, 3736,
                20804, 56292, 16304, 39308, 28456, 48284, 31172, 52100, 11388,
                41664, 8568,  63100, 33484, 5152,  51296, 64420, 24400, 54112,
                10712, 63168,
            },
            {
                36320, 30832, 52540, 436,   25188, 10772, 59064, 51108, 5708,
                41800, 33168, 14484, 59028, 24048, 18248, 5408,  47084, 39224,
                61716, 17248, 42520, 48296, 23824, 54532, 16008, 7660,  61208,
                40408, 45468, 59156, 22768, 0,     27464, 12308, 19744, 56088,
                10064, 27264, 38212, 49220, 29936, 59540, 4032,  55232, 25192,
                47984, 2620,  26776, 33444, 19176, 296,   59372, 55152, 16160,
                38432, 60220, 29944, 16940, 63880, 11044, 27468, 22252, 8804,
                51492, 29272, 38616, 5448,  12876, 46196, 4428,  16148, 33628,
                22348, 45868, 40748, 3800,  53312, 22160, 58932, 13680, 19456,
                43372, 58292, 27084, 12020, 65236, 30340, 20232, 8868,  39596,
                16456, 11720, 42632, 58908, 2360,  41092, 13076, 23260, 58172,
                7432,  33412, 1116,  53000, 57236, 42960, 59772, 48576, 52096,
                37700, 8296,  46880, 59820, 20112, 55340, 7236,  44444, 24868,
                19072, 49456, 1596,  59688, 12256, 30584, 40316, 2924,  27056,
                43972, 16984,
            },
            {
                55320, 12112, 19840, 33548, 63504, 39592, 47056, 13532, 36512,
                56556, 46560, 856,   53928, 10088, 64976, 26548, 12436, 8,
                49632, 8368,  31860, 63088, 13124, 36996, 49800, 33544, 56376,
                52048, 5124,  15012, 34360, 60844, 46068, 54400, 5036,  34036,
                64584, 6220,  18404, 53796, 21644, 7288,  40332, 36744, 13468,
                61240, 53220, 42648, 63724, 10640, 44604, 24960, 7744,  28396,
                20404, 48268, 1444,  44060, 32276, 5296,  60960, 39428, 57476,
                18436, 2848,  45148, 21076, 63052, 35924, 29044, 60956, 25912,
                57784, 6048,  63896, 13208, 32024, 816,   29228, 7004,  52208,
                24236, 4360,  46096, 34244, 55388, 3772,  58464, 50324, 24220,
                64212, 55136, 35620, 21144, 10216, 28552, 48392, 4972,  35312,
                26320, 61812, 48976, 18252, 32264, 26196, 7296,  40164, 13348,
                33100, 29660, 244,   23992, 41928, 13400, 2280,  60784, 35340,
                14436, 54300, 37916, 22004, 46960, 56936, 19516, 49736, 33888,
                58016, 5656,
            },
            {
                48060, 26444, 61100, 49652, 6688,  17184, 32188, 1860,  60728,
                18832, 8824,  30192, 48632, 40512, 33904, 52620, 30812, 58732,
                14772, 53596, 25944, 1872,  40044, 5584,  21808, 42908, 3068,
                19824, 26260, 38764, 8880,  52656, 17196, 37736, 25064, 48092,
                40972, 30860, 46372, 13392, 43196, 63444, 28288, 17672, 45292,
                31440, 9456,  16212, 29736, 50704, 14380, 34240, 46736, 61548,
                12068, 34924, 24816, 51036, 13860, 53460, 46360, 15432, 35108,
                48064, 25848, 53272, 32912, 9440,  55572, 50484, 2500,  43068,
                9688,  35436, 28292, 19868, 60144, 38860, 48376, 34792, 40368,
                9924,  61348, 49692, 21384, 15796, 41148, 45500, 32504, 1928,
                28732, 44660, 4116,  25348, 62540, 54068, 37756, 65308, 45040,
                54436, 14148, 37408, 5128,  46292, 10452, 35368, 63132, 18352,
                54840, 61836, 11216, 52816, 33916, 63792, 27356, 21792, 57180,
                5960,  28880, 44132, 26288, 15904, 36488, 7068,  62028, 13388,
                38760, 23292,
            },
            {
                2780,  41012, 8848,  37504, 22440, 53912, 28536, 43244, 23288,
                26852, 39024, 63076, 21956, 15956, 3208,  45036, 19692, 36592,
                23256, 44276, 34336, 57168, 18584, 59868, 27736, 64120, 31380,
                11804, 62076, 49040, 29900, 20748, 42484, 3340,  58420, 14920,
                21428, 804,   60508, 34908, 25684, 10220, 48444, 57800, 1372,
                22896, 39932, 60192, 3148,  57300, 37656, 21884, 52760, 2428,
                40216, 54916, 7248,  58452, 37848, 20500, 2228,  30588, 63672,
                7024,  12324, 61396, 16492, 40484, 24444, 19224, 13712, 38708,
                52292, 15660, 49340, 43708, 55020, 10964, 16512, 62668, 56456,
                18332, 30972, 37620, 8316,  51816, 25760, 11256, 18156, 37456,
                59856, 14016, 51568, 47144, 33460, 15840, 1560,  21452, 17708,
                9916,  28948, 22712, 64040, 52304, 15696, 56520, 28532, 2988,
                25304, 42796, 49212, 15912, 4552,  37368, 46448, 50420, 40008,
                32516, 3048,  64648, 10096, 55160, 176,   29268, 45380, 9120,
                18392, 65060,
            },
            {
                28896, 14580, 59204, 45208, 3260,  57200, 9308,  65332, 49440,
                12148, 55380, 4944,  36016, 28200, 56476, 8036,  62300, 41208,
                3624,  64604, 10980, 47164, 29868, 51268, 9856,  15148, 47484,
                36640, 44032, 56888, 1972,  65156, 32724, 10960, 28960, 62236,
                52288, 9148,  57280, 16952, 54696, 5252,  33388, 12276, 64564,
                52136, 35660, 19584, 24388, 41428, 5108,  65128, 17656, 31320,
                43400, 62808, 18328, 28944, 9536,  42464, 26232, 54748, 22996,
                43108, 36636, 57840, 1380,  44164, 5916,  47884, 64324, 30084,
                21672, 61384, 4688,  25448, 8076,  22764, 42628, 26380, 1956,
                45012, 13980, 27908, 59904, 472,   35584, 63160, 53496, 6068,
                22024, 9232,  39300, 18924, 7060,  43152, 30440, 51104, 40776,
                3040,  60004, 43364, 39260, 30592, 1592,  22264, 50132, 45112,
                5640,  21112, 38608, 58412, 31392, 18760, 7564,  10736, 16468,
                61656, 20532, 41004, 51776, 33992, 59176, 41656, 21544, 53296,
                32464, 43212,
            },
            {
                56016, 35000, 18628, 31212, 25388, 48120, 15244, 40104, 33828,
                6556,  52216, 44368, 13452, 59844, 50004, 25080, 11536, 54652,
                28940, 6028,  21356, 16080, 41616, 4692,  38196, 55372, 652,
                22444, 16804, 7372,  24688, 13624, 46468, 50244, 7084,  35388,
                38972, 44348, 24352, 3580,  37940, 50764, 42180, 18932, 27016,
                4756,  49196, 7652,  55328, 27936, 12736, 48980, 9180,  26760,
                5484,  14720, 23392, 46872, 64924, 34252, 59912, 11568, 4180,
                52072, 20044, 27584, 49888, 30888, 58632, 34656, 7816,  56624,
                452,   32444, 46168, 37200, 65392, 34052, 52464, 5928,  32636,
                50728, 64512, 4980,  39972, 57328, 22796, 29640, 46752, 42020,
                56296, 31956, 27116, 54524, 58512, 11676, 60748, 25836, 56300,
                32564, 48184, 7332,  12464, 20164, 61164, 36928, 40780, 11892,
                65252, 34732, 8708,  25808, 47576, 55824, 43032, 59020, 25260,
                52780, 46164, 7804,  18208, 24560, 14216, 49384, 26644, 60200,
                6288,  50224,
            },
            {
                10908, 1076,  53120, 63800, 11848, 36008, 20172, 4140,  60056,
                21624, 17292, 31824, 2084,  41376, 18348, 37764, 32392, 15604,
                47864, 51784, 36124, 56124, 61176, 22920, 44704, 26400, 59364,
                51708, 34008, 41160, 53604, 36272, 59080, 22124, 55412, 26348,
                18472, 11884, 31948, 47512, 62708, 29560, 22388, 56428, 38612,
                43984, 14572, 30472, 46344, 59604, 33224, 44004, 54368, 60432,
                36196, 51644, 32608, 480,   50388, 6324,  17008, 48788, 40412,
                32356, 8772,  14680, 38976, 21796, 11752, 26504, 45656, 17212,
                41784, 54192, 12764, 18600, 3152,  57160, 14292, 47316, 21148,
                10724, 24092, 43076, 19476, 48016, 9508,  14592, 3184,  60616,
                17036, 48820, 64660, 192,   34912, 23224, 46284, 8464,  14512,
                35992, 18440, 57932, 26920, 45372, 53976, 8492,  31752, 57096,
                15188, 53088, 60016, 1856,  12912, 22512, 29168, 920,   35688,
                30436, 12452, 37120, 48312, 4288,  63860, 10456, 37648, 2160,
                16404, 24376,
            },
            {
                39532, 21488, 44040, 6884,  41536, 58812, 51868, 46008, 29644,
                37920, 56900, 47684, 24216, 64328, 6856,  46080, 1184,  60848,
                26460, 39716, 8256,  31652, 2788,  12348, 34584, 19164, 6548,
                29552, 63516, 10440, 27568, 18200, 4844,  40052, 15464, 1456,
                49188, 64848, 40880, 19860, 8572,  14316, 180,   60824, 9664,
                32580, 62772, 53116, 10480, 18192, 700,   23608, 38556, 20712,
                11196, 45496, 56712, 39516, 12932, 22600, 37564, 57376, 28588,
                62500, 46416, 55864, 64920, 4680,  60400, 53708, 36932, 23936,
                9860,  27360, 59468, 50436, 30828, 40288, 28400, 61424, 38164,
                53516, 35028, 55628, 16300, 31544, 52876, 38820, 26796, 34724,
                12292, 23852, 4932,  41764, 14908, 49660, 38524, 4364,  63556,
                53652, 23912, 600,   49780, 33668, 4192,  17676, 24252, 47048,
                27776, 19208, 30336, 41220, 64080, 51716, 39164, 62524, 15068,
                5284,  55544, 60172, 27992, 32404, 39968, 19916, 56444, 47508,
                35832, 63080,
            },
            {
                58328, 30496, 50964, 27324, 16648, 32528, 336,   26012, 13568,
                8584,  62208, 27648, 10728, 34456, 52364, 22092, 57088, 42860,
                19444, 12704, 62984, 24976, 46212, 53144, 64864, 49472, 42360,
                13416, 3464,  48388, 60284, 43864, 31448, 63188, 45276, 30224,
                57996, 5648,  51800, 27760, 55656, 36824, 45236, 52424, 17240,
                24808, 2012,  21832, 39528, 35716, 63500, 14952, 4852,  29952,
                58500, 3436,  26396, 19588, 61868, 30776, 44812, 2972,  19112,
                13312, 584,   24908, 34076, 18284, 42488, 1892,  14480, 61588,
                49012, 39296, 6352,  21968, 44160, 10172, 968,   19036, 58196,
                7584,  28984, 2444,  61520, 6460,  44300, 64188, 20416, 51340,
                8020,  45120, 37316, 28972, 19392, 61696, 32020, 20840, 27972,
                44464, 10876, 41036, 64856, 14960, 58820, 42512, 61944, 6428,
                2640,  50660, 44544, 35904, 17136, 4784,  10244, 49128, 44864,
                23652, 42308, 20960, 1696,  54316, 44952, 6620,  31004, 12772,
                27840, 45424,
            },
            {
                8128,  14868, 4396,  62192, 22556, 9512,  53816, 64240, 40952,
                49600, 3248,  43036, 19900, 55480, 14260, 29356, 9676,  35148,
                4388,  57980, 49620, 17324, 37668, 28544, 8904,  16248, 32136,
                57312, 20136, 38328, 23216, 2276,  9340,  52076, 12636, 37496,
                23128, 34136, 16012, 2732,  60080, 31668, 25820, 6328,  34824,
                47288, 57596, 42324, 6544,  26468, 55760, 45084, 50504, 40832,
                64312, 16484, 8764,  42932, 52180, 55264, 9720,  25692, 59636,
                36088, 43556, 52972, 8252,  48672, 27836, 51472, 20776, 32884,
                2996,  63780, 35188, 15268, 55548, 63064, 25696, 45812, 16020,
                41564, 13308, 48896, 26048, 36916, 11800, 1100,  57936, 41044,
                31036, 62116, 53324, 57816, 9988,  52128, 1944,  56732, 39736,
                5956,  51724, 29656, 37156, 22472, 10032, 28740, 52168, 34400,
                40016, 10884, 23704, 7380,  55164, 32952, 26844, 18876, 57244,
                33664, 65336, 10996, 49824, 15372, 62264, 23456, 52176, 61360,
                3952,  18988,
            },
            {
                54876, 37820, 33720, 47928, 56048, 36508, 44188, 18448, 23324,
                33376, 16292, 36648, 59412, 1648,  40132, 48924, 63596, 23968,
                52928, 30100, 428,   43292, 20832, 55248, 1776,  39800, 61968,
                25412, 44828, 54608, 33344, 15980, 57028, 26592, 20716, 7376,
                61332, 10816, 46244, 42836, 21676, 11668, 49656, 40400, 65316,
                12600, 29196, 15688, 62144, 47776, 31272, 8600,  25452, 18980,
                33684, 48440, 37072, 28768, 5628,  34852, 14788, 48148, 51600,
                6052,  22528, 40252, 15428, 31196, 38112, 6612,  45132, 56948,
                29856, 18100, 46804, 24536, 5028,  33496, 51064, 36492, 4264,
                32036, 63696, 44860, 21284, 59008, 33340, 24132, 17880, 47260,
                3756,  15508, 21532, 6172,  43512, 26532, 13588, 45892, 34548,
                16808, 61916, 19912, 3424,  56328, 48824, 37924, 13204, 20720,
                63112, 57664, 14440, 61152, 46324, 21812, 59780, 37964, 2776,
                8124,  17456, 39472, 25872, 35740, 9500,  42912, 17096, 34312,
                41392, 49152,
            },
            {
                64812, 25456, 12320, 40796, 3388,  29300, 11640, 5400,  60716,
                54576, 7348,  51280, 24996, 44764, 31912, 5224,  16856, 38076,
                45480, 14800, 33516, 6932,  59624, 11564, 23796, 47452, 4592,
                36728, 8188,  12188, 62580, 49540, 41672, 35120, 47632, 56040,
                39840, 53368, 25000, 63316, 38180, 18128, 58424, 3012,  20512,
                55004, 37720, 51336, 19960, 2560,  59096, 13256, 53528, 1144,
                55416, 12116, 23972, 58208, 1768,  64576, 21700, 39464, 32448,
                63632, 29500, 55472, 61072, 10744, 57792, 62788, 12988, 40980,
                7988,  11520, 52204, 58852, 41912, 12796, 20588, 54760, 59600,
                23412, 51440, 9116,  16944, 40496, 49792, 53988, 28460, 10344,
                55772, 25920, 33568, 48432, 36312, 17704, 64016, 8816,  24656,
                58252, 12976, 32944, 45708, 7496,  24932, 44356, 1228,  53896,
                26432, 32288, 48096, 28340, 596,   42552, 13096, 51100, 29328,
                47720, 53012, 31308, 58296, 5056,  29572, 57360, 1936,  26192,
                10608, 22420,
            },
            {
                572,   44776, 17436, 60976, 20140, 59056, 50756, 38588, 31052,
                45900, 12432, 28644, 64664, 10316, 19016, 60092, 27408, 7520,
                61136, 21708, 40120, 64116, 51204, 35260, 29428, 56408, 18068,
                50832, 30664, 156,   28304, 19264, 5820,  64704, 3412,  17360,
                28472, 996,   32864, 4996,  47936, 8056,  52960, 30324, 24148,
                9028,  4236,  44648, 11372, 34340, 22576, 37344, 42608, 29468,
                6560,  61992, 44536, 50016, 18512, 41712, 27444, 7172,  17396,
                11828, 2460,  20260, 46680, 3492,  18904, 25084, 35748, 22336,
                55184, 27228, 37056, 1356,  30080, 48944, 7696,  27656, 11040,
                39180, 320,   30188, 56184, 5024,  14280, 7700,  61368, 35512,
                39304, 65404, 12896, 852,   60300, 40092, 30896, 49084, 53936,
                2616,  41960, 50300, 60660, 16096, 31628, 64192, 18188, 41876,
                9044,  4424,  39104, 19452, 37040, 54108, 9136,  63584, 25076,
                41236, 13616, 416,   62916, 46948, 21372, 50556, 38888, 60268,
                52892, 31372,
            },
            {
                6120,  57448, 51288, 8604,  26512, 46996, 14820, 24288, 1232,
                20544, 58340, 39420, 4668,  35544, 55000, 50880, 42100, 12456,
                54444, 48380, 10060, 26340, 15516, 45784, 41888, 60968, 13792,
                22332, 53916, 46620, 59560, 39136, 24684, 14124, 31552, 42560,
                50916, 19820, 14360, 55284, 27232, 34408, 15312, 41552, 45724,
                61808, 32916, 26956, 60908, 40308, 51944, 16828, 63636, 46404,
                21308, 39196, 15272, 31312, 10392, 60780, 47272, 54116, 58248,
                44136, 49420, 37432, 26612, 34556, 43112, 54092, 116,   49324,
                64368, 44012, 19836, 61120, 39436, 17204, 65020, 42916, 47908,
                18696, 61264, 36232, 26668, 64536, 45948, 30824, 43020, 2064,
                19372, 44316, 51612, 29308, 23244, 55616, 4048,  20204, 37364,
                29020, 22136, 9748,  26728, 36208, 51252, 5440,  55708, 35496,
                49684, 16520, 56192, 65000, 5912,  30988, 16712, 35244, 3840,
                20460, 55748, 24020, 43876, 17820, 37028, 13284, 6784,  45108,
                15044, 36136,
            },
            {
                20692, 29144, 38504, 33048, 1880,  41828, 35208, 55724, 62720,
                42500, 16792, 48800, 22736, 14864, 46456, 24436, 2676,  36404,
                29448, 4276,  57184, 32208, 2308,  19712, 5136,  9048,  33920,
                65216, 42448, 15500, 35540, 10004, 52320, 45244, 60236, 11320,
                57872, 36792, 61952, 9680,  43468, 64228, 608,   59212, 35784,
                13904, 49136, 17764, 56616, 7364,  25288, 3888,  10304, 59728,
                26876, 34748, 2864,  52904, 23284, 36152, 13364, 820,   33776,
                23928, 59824, 14128, 5132,  65288, 50712, 16100, 29032, 6016,
                14832, 32520, 3696,  9800,  23680, 55660, 31632, 2752,  34304,
                14628, 53132, 44036, 20144, 2916,  38008, 22028, 15992, 49448,
                24232, 7244,  58280, 10568, 15864, 42576, 7848,  61244, 12272,
                47424, 64708, 38852, 248,   58952, 21212, 11548, 29724, 22832,
                60048, 44840, 25324, 11700, 50208, 23124, 47024, 57736, 43360,
                61440, 37524, 7780,  33584, 10648, 54860, 64492, 28672, 19160,
                55720, 42232,
            },
            {
                48652, 13900, 62056, 22892, 53136, 65284, 10704, 6272,  27520,
                33276, 9196,  53672, 30328, 140,   57608, 32692, 20332, 65436,
                43800, 18276, 23844, 52688, 37936, 59208, 50040, 27564, 39504,
                2888,  25568, 6336,  56824, 21656, 1624,  27632, 34596, 7740,
                22064, 48880, 40424, 30052, 22904, 50684, 19936, 26264, 5500,
                53992, 22152, 1308,  43200, 30524, 48504, 55876, 32512, 19200,
                51004, 8636,  57676, 40652, 5144,  56420, 29792, 19956, 42204,
                7772,  28676, 53404, 41020, 21876, 8864,  32932, 46048, 58704,
                39776, 25716, 53656, 49900, 13676, 45568, 5368,  21380, 57092,
                25508, 6416,  10468, 33008, 50656, 55356, 60096, 11740, 53584,
                62584, 31676, 37876, 47368, 35060, 52508, 26912, 44624, 33740,
                17440, 6384,  56112, 42992, 14220, 46224, 40348, 62512, 2884,
                13480, 31564, 1468,  42948, 33908, 62068, 27476, 5316,  11096,
                31904, 15664, 51680, 60516, 26616, 4240,  32420, 47888, 24500,
                8928,  63544,
            },
            {
                26860, 10484, 45240, 5228,  16356, 30376, 19096, 44520, 52448,
                3600,  37080, 60288, 44092, 62600, 11056, 41076, 8212,  58580,
                14192, 34344, 46536, 63488, 12792, 30384, 22804, 54352, 11288,
                45356, 19788, 32616, 48476, 40580, 62184, 54676, 15868, 46992,
                25552, 1840,  6024,  17028, 57496, 12192, 38744, 47352, 10092,
                62860, 28968, 38412, 10952, 64900, 14912, 36480, 41512, 1616,
                13920, 47616, 65348, 16928, 26376, 46256, 63692, 38060, 52108,
                10932, 63096, 17720, 31700, 56356, 12584, 61416, 37324, 18580,
                10988, 47104, 62220, 35608, 28492, 60520, 37232, 50720, 63336,
                40728, 46912, 58920, 13832, 24296, 8580,  27960, 34196, 40668,
                4216,  27200, 17872, 2392,  21432, 64396, 13724, 58552, 1676,
                51084, 24944, 30820, 19020, 52576, 33272, 8140,  26940, 47240,
                37808, 53292, 20096, 58436, 7524,  14004, 39284, 18672, 53308,
                48336, 28596, 2200,  39652, 49164, 22080, 41632, 1440,  60684,
                33940, 3228,
            },
            {
                40616, 55032, 35764, 58840, 47704, 39988, 57188, 22316, 49256,
                13472, 24856, 18216, 7120,  27220, 38344, 16436, 48028, 25864,
                51240, 796,   9420,  40212, 6308,  44208, 16392, 61724, 36300,
                57992, 51580, 63240, 13016, 30048, 18144, 4540,  38536, 63616,
                31284, 59544, 52836, 45476, 33700, 7268,  54784, 31512, 40940,
                15760, 50516, 34148, 59452, 20336, 5328,  45776, 23248, 58364,
                29916, 37328, 21524, 32880, 43176, 9384,  3300,  16104, 25788,
                48772, 35536, 1188,  47280, 6700,  26964, 42124, 24000, 4092,
                55468, 21688, 6604,  16416, 42176, 8456,  26128, 11384, 16684,
                1396,  29452, 36912, 62728, 41436, 47972, 636,   19052, 45688,
                9928,  56392, 60836, 50492, 39656, 5116,  32104, 23532, 40612,
                54992, 35696, 10516, 62136, 4840,  23832, 58056, 54244, 18500,
                9640,  64004, 28504, 40552, 52172, 21916, 56104, 45472, 3392,
                23468, 65028, 19596, 44548, 14768, 59512, 52684, 12620, 38544,
                16708, 52088,
            },
            {
                5844,  31344, 19484, 2964,  25036, 12812, 7680,  32432, 63212,
                40600, 55972, 47700, 34380, 21276, 52264, 56348, 30228, 5900,
                37600, 60408, 28388, 20464, 55604, 33324, 1124,  47892, 7384,
                15016, 27904, 2436,  8888,  44420, 24228, 49864, 10644, 43164,
                18944, 35464, 13380, 26896, 62308, 2092,  23980, 18452, 58000,
                2592,  44416, 8092,  24540, 54388, 28548, 51432, 11276, 62048,
                44340, 53332, 5964,  12196, 54960, 49636, 22544, 60412, 57380,
                4604,  43736, 23184, 39008, 58988, 51924, 1820,  49252, 63680,
                30628, 43412, 33752, 660,   57220, 19748, 53388, 43920, 32620,
                22836, 56344, 19360, 5068,  16344, 31316, 58160, 64872, 36796,
                22412, 14736, 42428, 8304,  28448, 54248, 18812, 48308, 8796,
                15324, 59992, 44996, 27652, 48160, 37052, 15744, 3712,  43444,
                34088, 48640, 15320, 4728,  36116, 10164, 30208, 61220, 35524,
                41780, 12460, 58180, 9268,  34772, 6012,  18620, 29460, 45636,
                58004, 22204,
            },
            {
                61764, 15276, 43588, 28484, 50136, 60556, 38048, 564,   16036,
                29524, 10192, 1992,  64152, 43328, 13212, 2840,  61616, 21780,
                42320, 53908, 15668, 49584, 62216, 26132, 41432, 19064, 24468,
                31716, 42904, 38296, 60364, 53476, 36224, 58608, 28844, 7224,
                55664, 3688,  41372, 21020, 37784, 50056, 43840, 64692, 36276,
                27492, 52764, 17996, 47432, 40556, 196,   16472, 34308, 7656,
                25092, 2648,  28620, 62792, 18544, 35364, 30484, 40824, 12692,
                32684, 19116, 62064, 13940, 29968, 16760, 20624, 34804, 9480,
                14296, 27260, 51740, 62996, 39188, 30296, 3408,  59976, 48816,
                39628, 9356,  51156, 27132, 54324, 43612, 7012,  12824, 49088,
                53252, 32796, 24760, 46564, 34436, 11148, 62084, 43652, 30024,
                3552,  21628, 38928, 1360,  12648, 65408, 41600, 30244, 61460,
                23280, 2176,  26088, 45128, 62604, 49400, 104,   16272, 26352,
                7328,  54564, 37352, 30560, 24076, 55816, 63084, 36076, 9892,
                25540, 50236,
            },
            {
                12080, 37208, 56080, 63924, 9920,  34708, 53500, 26632, 58944,
                44816, 20696, 51052, 31384, 5692,  24328, 35376, 45400, 17900,
                32416, 11104, 23560, 36064, 3352,  12004, 57712, 50976, 63700,
                4020,  55808, 21784, 16316, 32908, 432,   20824, 14268, 51396,
                24584, 65388, 49060, 8800,  58612, 14756, 28908, 8404,  12836,
                22672, 4780,  63172, 13228, 30720, 56676, 64112, 38728, 19376,
                41244, 15520, 47132, 57900, 39120, 384,   52368, 6212,  45648,
                27732, 55384, 50736, 7960,  36712, 44108, 54056, 59640, 40284,
                47736, 58232, 18168, 11632, 24416, 46984, 15092, 35704, 6724,
                13528, 63852, 44804, 2992,  34072, 21664, 38956, 25408, 29628,
                5540,  2120,  63296, 58392, 17104, 408,   56848, 37652, 25776,
                64052, 49288, 17912, 57136, 32000, 51380, 19812, 6928,  49752,
                12284, 54816, 59736, 31940, 18092, 24196, 38896, 58824, 32316,
                49940, 21092, 45320, 1612,  50900, 27276, 47172, 4684,  42144,
                200,   32996,
            },
            {
                47596, 8016,  24028, 1516,  21140, 42224, 18324, 47044, 4480,
                35680, 56916, 38632, 16584, 60872, 54208, 48828, 8908,  64548,
                2040,  56788, 44796, 7240,  47460, 30068, 38132, 8444,  35268,
                45772, 10684, 49316, 26672, 6372,  47020, 61856, 38932, 33368,
                45996, 11572, 30552, 17224, 53872, 32844, 3848,  47784, 55188,
                42152, 59580, 33364, 37680, 6860,  21172, 46820, 26976, 55056,
                60936, 50800, 33928, 23068, 7976,  26692, 15152, 64384, 21516,
                9760,  39816, 2440,  25224, 64164, 11200, 5320,  31780, 24844,
                2960,  7728,  37536, 44580, 5304,  55988, 65276, 21180, 27880,
                55064, 31608, 24124, 60332, 10800, 50052, 62180, 17560, 55440,
                43964, 20284, 38256, 13196, 41336, 22972, 50668, 6592,  12716,
                33268, 53588, 6084,  42452, 22708, 9172,  26308, 35692, 57584,
                38648, 20580, 41500, 6040,  11772, 53704, 43352, 8652,  13448,
                63776, 4188,  17424, 61356, 39500, 11912, 16184, 21552, 54228,
                65088, 19028,
            },
            {
                59268, 41284, 52812, 45884, 30772, 14108, 6552,  64336, 23160,
                12632, 7892,  26216, 42476, 10900, 28564, 38080, 14856, 25668,
                50648, 28996, 39952, 63288, 17152, 54120, 21360, 14208, 28212,
                59960, 18172, 40456, 64472, 12752, 55400, 42364, 17652, 3108,
                60152, 21592, 36652, 1132,  42544, 24764, 61180, 20384, 38988,
                29600, 10692, 49648, 25612, 58460, 43300, 9592,  2356,  12676,
                31016, 3944,  10528, 45384, 60036, 42936, 56020, 48572, 36376,
                61216, 17176, 47140, 33552, 56240, 28136, 46704, 15648, 19768,
                64520, 54932, 32472, 22112, 51020, 34108, 10200, 42348, 50432,
                172,   17116, 35980, 40960, 14884, 57196, 1268,  9264,  34384,
                61284, 50772, 6972,  31072, 52992, 27740, 35472, 20376, 46596,
                58964, 10116, 28300, 35412, 60668, 54680, 45584, 464,   14968,
                28032, 8148,  34900, 46472, 64792, 28900, 19540, 56560, 47064,
                40272, 29216, 52632, 32928, 8172,  43080, 58568, 30876, 49140,
                38912, 26748,
            },
            {
                29576, 4796,  17272, 33856, 61528, 55500, 49852, 38948, 29172,
                52388, 59956, 47580, 18996, 4164,  58520, 20408, 41732, 33580,
                59092, 4640,  13424, 34000, 25544, 58776, 44244, 5280,  52612,
                23268, 1652,  34152, 52028, 30184, 23040, 9556,  27048, 52980,
                6144,  40448, 27676, 62964, 46556, 5740,  35448, 51524, 16076,
                740,   45520, 19412, 3264,  15308, 32688, 53520, 36168, 48740,
                22212, 56856, 37424, 17308, 31844, 11616, 19752, 33164, 3380,
                29252, 52616, 13064, 41996, 21348, 672,   61144, 39476, 50248,
                42824, 26740, 13652, 61668, 28764, 2644,  17640, 25108, 38104,
                61604, 47216, 7528,  20256, 45388, 30856, 27428, 46936, 40804,
                15192, 26700, 10384, 47380, 64668, 3732,  42580, 60540, 16664,
                39664, 24604, 44508, 14588, 2856,  17372, 39784, 63656, 52744,
                47476, 58640, 17044, 51384, 1412,  37572, 4580,  34264, 22628,
                6512,  25440, 57552, 14348, 22980, 63580, 2556,  34728, 10848,
                6488,  14684,
            },
            {
                35884, 56820, 11528, 39412, 3948,  25164, 10636, 33284, 43692,
                15488, 1716,  32080, 64960, 35056, 51584, 496,   55404, 7832,
                46504, 18588, 22144, 51760, 9488,  184,   40588, 65192, 32236,
                38732, 57820, 8184,  43684, 4376,  37480, 63048, 31492, 44848,
                58028, 15104, 49668, 55268, 19040, 13084, 57400, 9084,  64180,
                24136, 55924, 61112, 41200, 50988, 62932, 23812, 17724, 61772,
                42344, 27320, 6000,  65052, 53508, 2600,  25240, 41276, 8548,
                57040, 24276, 4356,  59952, 17928, 37004, 53008, 7464,  11936,
                35300, 1104,  47528, 8740,  40268, 59360, 46132, 53784, 5944,
                12340, 29112, 56364, 51764, 4800,  37612, 63392, 22700, 4280,
                59532, 36724, 23648, 57072, 19232, 14508, 9284,  54060, 1300,
                31352, 4880,  52640, 64480, 48708, 30064, 11592, 33004, 22608,
                3980,  30804, 24796, 13736, 60484, 25960, 48492, 15452, 62124,
                44232, 35816, 772,   48972, 37288, 45832, 18000, 56180, 24492,
                61260, 45144,
            },
            {
                50112, 63640, 22720, 48040, 28236, 58540, 19136, 2896,  62408,
                21108, 40644, 54928, 23872, 13752, 43824, 30700, 23296, 11704,
                63840, 36692, 43564, 61196, 30684, 35756, 19684, 46684, 11472,
                16204, 48100, 25616, 13552, 56812, 19636, 49760, 1848,  12040,
                24972, 33760, 7336,  10248, 31032, 38620, 26840, 44140, 31460,
                34512, 7424,  36548, 28336, 11976, 5244,  29972, 45736, 7956,
                14488, 51788, 20272, 39928, 28988, 46268, 61552, 50588, 14240,
                44768, 38084, 31252, 49820, 9936,  44628, 23592, 29708, 58476,
                21788, 52184, 16820, 56808, 20388, 36708, 13924, 32040, 57708,
                43520, 22296, 64320, 25940, 13080, 54160, 18456, 49876, 11624,
                31872, 53744, 1012,  39700, 33520, 44656, 29208, 62560, 21948,
                50352, 58108, 18776, 37380, 7808,  25556, 59288, 19464, 41692,
                10100, 62492, 43988, 39672, 20856, 55368, 31240, 52456, 9332,
                58960, 12216, 20016, 54920, 26624, 4832,  29080, 40036, 52548,
                20808, 1868,
            },
            {
                25928, 18428, 8672,  52384, 15204, 44392, 36808, 53384, 47912,
                6896,  27800, 37444, 9080,  46824, 6132,  60672, 38604, 50024,
                27784, 1664,  54232, 5984,  48936, 15328, 56576, 27252, 3124,
                62156, 29676, 20932, 61092, 46376, 15932, 34800, 54592, 38700,
                17404, 64432, 42052, 23376, 60428, 50924, 2236,  21376, 48940,
                13656, 53184, 17512, 22740, 47560, 39824, 55704, 1764,  38088,
                32956, 59248, 708,   48952, 13340, 7212,  36848, 17740, 58736,
                22056, 63204, 6276,  55300, 27452, 65344, 14616, 33896, 3908,
                62280, 38128, 31388, 43272, 4400,  63300, 27024, 1816,  19080,
                34536, 9124,  39244, 1500,  42812, 33468, 7764,  58020, 43548,
                20412, 46004, 16332, 60396, 5260,  49296, 24320, 38280, 34644,
                13672, 42036, 10716, 22856, 33800, 56236, 44728, 4916,  49616,
                36012, 54964, 2420,  33756, 7260,  11024, 41648, 2800,  18396,
                39112, 30544, 65512, 42136, 10176, 60448, 50776, 12952, 8532,
                32536, 42200,
            },
            {
                4492,  30572, 40836, 34404, 396,   65516, 8200,  24128, 31568,
                57916, 11872, 50884, 17320, 62980, 26916, 19428, 15096, 57248,
                32296, 16432, 24680, 41476, 12076, 60160, 23784, 6440,  50164,
                42300, 54996, 35852, 604,   40148, 26656, 8592,  60476, 21340,
                47096, 51728, 136,   36936, 45712, 15976, 62836, 40260, 4896,
                59916, 43160, 1508,  65340, 9140,  58692, 19904, 26200, 64084,
                10840, 44028, 24336, 34636, 62736, 22648, 54940, 27848, 2240,
                34468, 11244, 16508, 41124, 35396, 2832,  49304, 56288, 18668,
                46108, 5892,  24496, 53552, 11432, 49068, 22536, 52160, 60320,
                47692, 15332, 30740, 49184, 59988, 16420, 24700, 35944, 2076,
                29556, 63796, 8544,  27248, 52488, 11608, 56584, 7324,  46444,
                3384,  27796, 47832, 61860, 40344, 1064,  52248, 16116, 61392,
                27176, 13056, 18576, 50608, 58240, 45180, 63668, 27604, 24064,
                50992, 47268, 3140,  33652, 16388, 21736, 35272, 47012, 64600,
                15960, 58432,
            },
            {
                12000, 47100, 54980, 60312, 20744, 29940, 41136, 13784, 17888,
                61096, 42956, 944,   57100, 33160, 41156, 53900, 3272,  44740,
                8724,  39608, 62664, 19984, 29024, 44460, 37828, 53036, 33624,
                18700, 9960,  6820,  51160, 31680, 65048, 4064,  42392, 29716,
                5516,  27020, 57336, 20132, 4556,  29464, 11524, 54852, 18424,
                26220, 30196, 38404, 52124, 32500, 14956, 35624, 49944, 53788,
                16996, 29388, 57344, 9392,  16264, 43276, 32120, 5556,  40084,
                47728, 53380, 25780, 46428, 20496, 8232,  39328, 25252, 42372,
                9404,  28312, 64452, 15368, 34876, 29892, 7316,  41752, 37448,
                3812,  62628, 54684, 20060, 9932,  28628, 65424, 40428, 55980,
                13844, 50972, 41868, 37088, 21240, 31708, 15580, 65200, 19576,
                55272, 60076, 30932, 5764,  14060, 20820, 28848, 37992, 6712,
                23800, 45956, 65160, 29352, 22592, 16428, 33052, 37260, 56136,
                5420,  14660, 28108, 52732, 7208,  44184, 56928, 508,   27340,
                38044, 53148,
            },
            {
                61660, 36556, 6196,  12840, 25460, 50188, 55828, 46112, 5072,
                34836, 25376, 30232, 20972, 4708,  48888, 10836, 35156, 22508,
                48152, 58660, 3884,  34744, 55712, 2252,  9012,  64220, 14076,
                39680, 24572, 58072, 43596, 12096, 23092, 48684, 15024, 58528,
                10660, 32400, 13608, 53276, 34332, 58644, 24512, 36396, 46524,
                8648,  56448, 12468, 45800, 25424, 3076,  44500, 6292,  22132,
                41088, 3448,  46592, 52524, 38640, 60108, 11860, 51588, 64832,
                18860, 30580, 220,   57276, 62148, 12352, 54404, 30304, 60336,
                13132, 49680, 40488, 1488,  59792, 44088, 55692, 17048, 12124,
                27956, 23636, 5828,  34920, 45880, 52056, 4080,  47948, 6200,
                23816, 34268, 18300, 4368,  61628, 42984, 120,   39916, 26032,
                9844,  36408, 16924, 54020, 42776, 63812, 47116, 11160, 31692,
                54040, 41328, 9016,  38364, 4992,  48352, 1260,  12968, 60224,
                44824, 22196, 58368, 38260, 61816, 24848, 30964, 5992,  19008,
                43416, 23152,
            },
            {
                1552,  27780, 17004, 44844, 37956, 2552,  10824, 63200, 38864,
                49564, 54124, 14656, 45284, 38448, 24176, 61012, 29680, 65320,
                13812, 26480, 52496, 10672, 46968, 17388, 31968, 21820, 28520,
                45792, 62832, 16544, 28176, 54364, 19108, 34212, 52852, 37616,
                44664, 63380, 40392, 48276, 9100,  42980, 6252,  51428, 31820,
                63484, 22040, 4100,  60996, 18948, 40528, 62676, 28368, 13012,
                60768, 31892, 19388, 6140,  26756, 1224,  20740, 44468, 24724,
                8736,  60892, 42720, 15004, 33144, 23056, 47468, 1168,  16864,
                35888, 57484, 22780, 18892, 47324, 9816,  25464, 65068, 33252,
                51528, 44640, 40532, 58784, 12880, 37836, 21320, 15492, 27576,
                58956, 10708, 47196, 54620, 25096, 58360, 48920, 33808, 52752,
                43960, 23580, 2296,  50464, 26236, 8500,  35096, 56720, 60340,
                17664, 304,   34652, 56944, 26324, 61224, 53432, 19908, 8716,
                34176, 41296, 10892, 18312, 48764, 13396, 40700, 62812, 50552,
                8104,  33456,
            },
            {
                56764, 20220, 62876, 51368, 59332, 33228, 18824, 27892, 22324,
                3328,  9484,  64688, 59416, 7440,  51640, 16836, 1784,  37632,
                6948,  42708, 31028, 22096, 38876, 60772, 49464, 58560, 1248,
                5524,  48264, 34616, 2692,  38152, 59460, 9624,  1472,  25180,
                18184, 22664, 3684,  16496, 26084, 65064, 21152, 14320, 648,
                41840, 16280, 37228, 28884, 51040, 9888,  56244, 33352, 48484,
                8412,  55756, 36468, 63872, 49560, 30420, 57924, 35908, 13836,
                49984, 38220, 4824,  28324, 51804, 36992, 6476,  64064, 52336,
                32376, 7480,  54504, 37548, 31832, 4512,  20676, 39200, 1328,
                57652, 8860,  17596, 26100, 376,   30996, 60940, 53632, 44404,
                38300, 63028, 2316,  29148, 12928, 8872,  17392, 29624, 4176,
                13852, 63496, 38240, 32808, 59116, 18520, 23048, 39688, 4752,
                49204, 21584, 52256, 15456, 11728, 42652, 39264, 30016, 50160,
                26496, 62648, 984,   32168, 54544, 3932,  21012, 35596, 55172,
                14844, 46044,
            },
            {
                49112, 40404, 9576,  30276, 23300, 7920,  43544, 57064, 15644,
                31908, 41324, 19280, 35372, 12688, 28068, 56864, 46076, 54200,
                19368, 49896, 15828, 63952, 4548,  25844, 12956, 41548, 36484,
                56384, 12528, 20812, 61292, 6228,  45404, 29236, 62588, 47356,
                7712,  54416, 36156, 60468, 30536, 55560, 38340, 47760, 59668,
                34484, 54184, 48736, 5908,  59112, 23432, 16032, 124,   38268,
                24872, 43044, 14368, 23552, 10632, 41240, 16912, 55244, 3100,
                32888, 21748, 59228, 10372, 44892, 18532, 40296, 27144, 21244,
                45252, 3284,  26152, 12244, 62108, 58104, 50676, 45528, 14624,
                30264, 21648, 48832, 62476, 55452, 42868, 9348,  33620, 18820,
                6648,  31260, 51544, 40232, 45160, 36668, 22612, 61900, 46072,
                57768, 20536, 7924,  44992, 12604, 48752, 1732,  27816, 13692,
                43868, 30112, 63324, 46584, 32544, 7484,  23436, 64524, 16736,
                5880,  55524, 46904, 36828, 27940, 9572,  44940, 59708, 25620,
                11088, 31156,
            },
            {
                22888, 3540,  15480, 42572, 4448,  54288, 48556, 35936, 61940,
                51572, 46648, 24620, 1464,  43184, 32144, 21364, 40844, 11716,
                33020, 58300, 940,   35664, 54868, 43812, 6920,  18844, 53712,
                30928, 26284, 52004, 40136, 24428, 50576, 16372, 40812, 33028,
                58184, 28256, 50264, 1792,  45360, 12184, 4808,  18260, 27068,
                10312, 24368, 43724, 13896, 30576, 35788, 45992, 53072, 64852,
                17904, 51540, 2096,  59440, 34032, 47628, 7716,  62752, 28684,
                46636, 17236, 54704, 24988, 62816, 3180,  58176, 13932, 42528,
                10680, 63120, 48488, 41228, 15624, 28124, 34964, 6660,  53700,
                61088, 37128, 4876,  34192, 15800, 23416, 3120,  50088, 12404,
                55596, 22696, 14692, 19816, 64284, 56324, 5620,  50108, 10820,
                27456, 41184, 55348, 29528, 5268,  64928, 41956, 53288, 62088,
                36504, 9784,  24460, 2940,  18896, 58268, 47492, 1948,  35236,
                43720, 21176, 15264, 24056, 64772, 49840, 16056, 29504, 2748,
                38892, 64128,
            },
            {
                36344, 57896, 52672, 35024, 64884, 13964, 25796, 992,   11824,
                6752,  29036, 58564, 54556, 48212, 63900, 8964,  3920,  61500,
                27572, 24088, 45572, 9992,  28624, 51168, 33416, 23700, 9736,
                64808, 44288, 8280,  15236, 32172, 11108, 56072, 21084, 5240,
                13028, 43024, 9884,  19568, 34112, 24292, 41468, 52940, 31020,
                62516, 7488,  19844, 64244, 2368,  41944, 20660, 11496, 27420,
                4976,  30784, 45556, 20576, 53980, 3632,  26136, 38776, 11228,
                52492, 39936, 6748,  34480, 48628, 31376, 8624,  50176, 35020,
                59652, 17508, 30652, 53168, 252,   43128, 23736, 10504, 18768,
                41916, 27272, 11896, 51848, 45652, 28464, 63804, 40156, 25680,
                61280, 42788, 35484, 7800,  26812, 1924,  31412, 39096, 18744,
                35204, 840,   51364, 16480, 24668, 56332, 33400, 15788, 20124,
                6216,  55888, 40200, 59804, 37124, 27988, 52832, 13800, 56672,
                38548, 8896,  52140, 2944,  39488, 58416, 6540,  42172, 53060,
                18152, 7080,
            },
            {
                12760, 28928, 24900, 47200, 18604, 39080, 30892, 60104, 40664,
                21336, 37596, 14500, 4664,  18024, 25356, 35640, 15296, 50956,
                37176, 6204,  62948, 40932, 20620, 15304, 62204, 47420, 38348,
                17084, 4348,  59108, 42104, 63420, 864,   35976, 27076, 49500,
                64720, 15876, 39524, 57104, 62356, 7860,  49364, 58604, 2128,
                46028, 39624, 55496, 32968, 50140, 58012, 7040,  39904, 56952,
                35152, 61540, 38920, 12668, 29232, 43632, 18772, 59752, 22588,
                952,   64696, 13300, 42212, 20780, 16000, 55896, 23768, 28516,
                5532,  38488, 23304, 8112,  20264, 49232, 63748, 56068, 32388,
                47336, 2024,  64804, 38584, 7112,  19828, 57576, 36440, 5276,
                48372, 684,   59168, 46460, 54008, 16540, 42852, 60132, 53244,
                14536, 63008, 32452, 21632, 46928, 37304, 8284,  30624, 50824,
                45816, 26944, 13020, 50012, 5732,  44248, 10168, 22176, 30732,
                59732, 27004, 61592, 12924, 30508, 19024, 34740, 23312, 61928,
                45580, 50928,
            },
            {
                41400, 60700, 260,   10852, 58996, 22148, 8968,  45732, 17576,
                53400, 63228, 33640, 10624, 39208, 51876, 57704, 42444, 22244,
                55800, 18224, 48840, 12472, 56756, 4816,  58760, 340,   29340,
                50500, 22360, 34776, 27380, 19632, 53232, 43740, 59316, 3500,
                37396, 31220, 22300, 52120, 27300, 14372, 36788, 21344, 16580,
                35632, 12504, 27328, 5120,  17408, 25228, 62224, 47652, 14972,
                22748, 9452,  50060, 6580,  64572, 57012, 15020, 50444, 35116,
                30940, 45456, 27192, 58544, 2168,  36888, 44400, 64436, 1016,
                54428, 45960, 57560, 34592, 59948, 13668, 36896, 4792,  25652,
                58584, 14288, 22924, 54776, 31192, 10652, 47440, 14096, 32272,
                17060, 28780, 10868, 23932, 32840, 48776, 9168,  23400, 6300,
                25824, 43792, 9668,  61232, 40376, 3188,  11904, 60900, 23640,
                1336,  34420, 64816, 21088, 16180, 33816, 62924, 40696, 4468,
                18740, 45608, 33648, 41644, 54952, 47744, 11340, 56548, 1512,
                32220, 20872,
            },
            {
                54492, 16576, 33036, 44084, 5812,  50732, 34096, 55556, 27104,
                2468,  48960, 23372, 45220, 62092, 27968, 228,   30728, 8332,
                44432, 2744,  29616, 38652, 32056, 25104, 35356, 41812, 13808,
                55788, 45628, 2756,  57492, 48364, 14184, 9156,  24160, 18112,
                46020, 54296, 6176,  348,   47496, 31868, 64544, 4224,  56228,
                23668, 51988, 61160, 47208, 37644, 9352,  29484, 32708, 1320,
                54612, 42008, 17068, 24952, 37064, 32384, 8664,  41144, 4508,
                56272, 19188, 9396,  50536, 29776, 53176, 11576, 39856, 19060,
                32552, 14704, 10948, 40788, 29156, 2540,  46148, 16776, 40300,
                8696,  50424, 44328, 17384, 59720, 42440, 1416,  54332, 21888,
                51808, 65148, 41196, 56536, 37988, 13304, 63712, 29704, 36524,
                57068, 48588, 4344,  13564, 58036, 27348, 48520, 42828, 57252,
                17524, 52956, 42196, 29800, 2216,  54900, 25284, 51416, 47964,
                12012, 24,    50708, 7556,  22308, 4076,  27376, 37768, 14556,
                26096, 9004,
            },
            {
                4720,  38768, 63512, 26516, 56432, 41560, 3644,  64144, 13248,
                42672, 8472,  31332, 55120, 6420,  20468, 12440, 47804, 63740,
                34256, 14388, 60440, 51344, 8612,  45920, 52512, 21504, 6600,
                63060, 31972, 11636, 37716, 5748,  30128, 39632, 62176, 34020,
                11820, 25528, 61428, 43380, 17852, 10756, 40336, 44696, 29412,
                9660,  41660, 540,   21692, 54724, 13336, 42968, 51864, 19380,
                59832, 28384, 46296, 52784, 520,   21944, 48636, 63340, 24704,
                14256, 37568, 62020, 23196, 4704,  60576, 26056, 7448,  48188,
                22356, 51812, 3564,  65180, 24964, 55024, 31324, 52728, 61708,
                20488, 35252, 28784, 6412,  33996, 24784, 63196, 27504, 44352,
                6940,  34752, 18908, 5536,  2668,  21540, 51720, 45536, 1700,
                19948, 39648, 30180, 34572, 18044, 53960, 20980, 35676, 4804,
                39268, 11316, 7908,  47796, 57716, 38712, 7620,  14836, 32604,
                65116, 37792, 24828, 16384, 63572, 44712, 60296, 50392, 42680,
                65428, 47620,
            },
            {
                29584, 22268, 49392, 12580, 19524, 29424, 15896, 23600, 36976,
                52500, 19660, 59396, 15812, 36328, 43500, 58844, 16972, 39720,
                25768, 53624, 22936, 18968, 1940,  64656, 16440, 10676, 27512,
                39564, 18568, 25152, 51736, 65144, 22180, 46892, 2484,  51140,
                57472, 8224,  29360, 38120, 58800, 51408, 24732, 7228,  59740,
                48052, 31732, 15544, 34860, 44956, 65448, 3428,  23612, 36368,
                4616,  13160, 33868, 62152, 5512,  55700, 12212, 28980, 43472,
                53328, 6628,  46916, 34180, 17756, 41936, 14056, 36040, 58324,
                61744, 27280, 43028, 49732, 17840, 7096,  43944, 11192, 27620,
                664,   38792, 62484, 3588,  52648, 15292, 36816, 9272,  60720,
                39292, 12336, 49028, 62252, 27720, 42028, 58992, 17472, 11048,
                54220, 65432, 22928, 50532, 44264, 36,    64068, 14964, 29304,
                62348, 25176, 32788, 61128, 23004, 18012, 45104, 28040, 58092,
                20520, 53984, 29440, 57240, 35112, 10036, 30864, 17728, 6064,
                34328, 57392,
            },
            {
                44456, 1884,  36236, 53644, 7984,  38248, 58752, 48524, 32672,
                5208,  28800, 40352, 1804,  25068, 50748, 32940, 3544,  56208,
                10688, 6588,  46696, 42412, 27948, 37024, 59496, 43928, 53776,
                48696, 59944, 8220,  43124, 16860, 33220, 54628, 27928, 15316,
                41084, 21032, 35216, 14136, 3592,  20364, 34452, 53860, 13856,
                19336, 62956, 57228, 7880,  27984, 18384, 38960, 56440, 49052,
                63552, 8920,  40468, 19704, 27164, 44528, 38808, 18264, 1636,
                59660, 31944, 40188, 10788, 56004, 49704, 31108, 45428, 2312,
                16728, 34956, 5680,  37508, 21508, 60316, 34120, 23088, 49380,
                57780, 13252, 55128, 47688, 21720, 41408, 49912, 19420, 4120,
                57600, 25744, 31232, 53064, 15116, 34532, 7600,  31744, 37852,
                26716, 15616, 5896,  60384, 8372,  25896, 32412, 10136, 45740,
                54568, 18816, 44048, 13768, 4088,  35256, 62260, 1572,  41896,
                9684,  5236,  43516, 13436, 39368, 20688, 1072,  54268, 24392,
                12316, 18972,
            },
            {
                10336, 58904, 15472, 32164, 62464, 46328, 1152,  10340, 61756,
                44192, 12532, 65356, 47072, 9596,  61068, 28920, 21484, 49208,
                35828, 30448, 62460, 55592, 12180, 33148, 23780, 4648,  30460,
                1712,  14740, 35280, 56012, 524,   12360, 59432, 20036, 5220,
                44644, 49452, 64364, 54480, 46252, 28272, 61584, 1448,  37740,
                26328, 4004,  39396, 24184, 50944, 59548, 11224, 6968,  26680,
                43628, 31104, 58188, 49476, 15160, 60528, 33124, 9212,  47748,
                20892, 26368, 15356, 65452, 132,   20516, 62520, 24404, 10040,
                55220, 30676, 13156, 56924, 9412,  40004, 15088, 64148, 5988,
                42252, 18828, 25148, 30428, 7952,  12016, 56160, 31884, 47120,
                16196, 22304, 43592, 9848,  46488, 61116, 24012, 49344, 57888,
                3976,  46228, 40808, 12732, 36988, 52400, 41216, 58720, 6632,
                49804, 1092,  36776, 52000, 28904, 49248, 11780, 53004, 23344,
                36256, 60692, 49352, 3036,  52380, 59328, 48072, 40868, 62508,
                37356, 51884,
            },
            {
                61272, 41008, 23396, 4908,  43204, 25700, 21256, 54664, 17544,
                26108, 56668, 22000, 37892, 53100, 14248, 5464,  41724, 64756,
                18124, 1048,  39092, 15396, 3604,  49860, 56860, 19212, 38244,
                62692, 22552, 47376, 26580, 40476, 45344, 8952,  36568, 62760,
                30308, 1040,  10888, 24008, 7512,  41796, 16644, 49116, 32480,
                43120, 51820, 12328, 46364, 1752,  33912, 30456, 47160, 14328,
                17924, 22792, 2404,  37308, 6484,  23964, 51968, 62888, 35864,
                57272, 4144,  52536, 43852, 27772, 38064, 6032,  50960, 40364,
                19884, 44584, 62712, 47404, 26792, 52080, 2816,  46600, 36972,
                32340, 9492,  45484, 64636, 34936, 59504, 2208,  28360, 38108,
                53800, 164,   58740, 5172,  19808, 39204, 1276,  12936, 43464,
                62200, 33320, 53224, 29408, 19492, 47360, 23192, 17180, 38380,
                27560, 21752, 59276, 9588,  56492, 39336, 19732, 31436, 47168,
                15628, 26148, 18920, 32948, 27900, 23168, 7968,  14356, 32436,
                3072,  27036,
            },
            {
                6908,  47520, 28632, 51248, 11940, 57540, 30920, 40604, 36144,
                2828,  50480, 6572,  30520, 19104, 34720, 57172, 24532, 12264,
                44800, 52268, 26276, 22020, 61372, 40948, 7420,  46140, 10052,
                32872, 57904, 5856,  18940, 63760, 30780, 23424, 53264, 39448,
                25280, 58592, 18228, 32240, 38576, 55804, 11868, 65056, 21232,
                6148,  55332, 29820, 63752, 16028, 42168, 21480, 62080, 35700,
                55856, 51824, 11140, 45560, 54472, 29928, 3348,  16700, 42340,
                11856, 30636, 23492, 48760, 8988,  33692, 13068, 57572, 28752,
                4312,  53548, 23360, 460,   32048, 58096, 20228, 29204, 55884,
                15632, 51500, 2740,  39184, 16816, 44148, 23960, 63428, 13728,
                42516, 33104, 64740, 36616, 29564, 55532, 51936, 18016, 28776,
                21128, 9552,  24348, 57348, 1864,  62940, 4716,  55680, 13316,
                65072, 31988, 41628, 16004, 24712, 4968,  64488, 7972,  55716,
                40560, 63004, 6892,  45916, 10940, 64204, 43832, 29152, 56752,
                45840, 21156,
            },
            {
                54016, 33480, 17532, 63464, 35308, 3984,  14444, 52052, 8660,
                45416, 33976, 16092, 42820, 62024, 656,   46052, 38160, 7396,
                59384, 32280, 9504,  53876, 34892, 29480, 16208, 26452, 51332,
                42388, 12780, 28912, 37560, 52436, 3904,  48768, 17480, 7000,
                13496, 34568, 51664, 47824, 61000, 4252,  25184, 45204, 9432,
                59308, 17228, 22616, 36960, 9008,  58148, 52556, 5200,  312,
                39828, 64392, 26292, 33600, 58672, 13344, 40160, 22372, 50716,
                7508,  61644, 36772, 18220, 59528, 53892, 46128, 16532, 64500,
                37184, 11692, 41792, 17940, 36348, 13592, 42784, 4948,  25580,
                62012, 22392, 58156, 27436, 53544, 6004,  19784, 50844, 9940,
                6692,  26872, 16696, 50400, 11724, 25588, 63816, 6232,  36316,
                56200, 3044,  49268, 16640, 34364, 10732, 30412, 43408, 34948,
                3192,  48400, 7216,  61732, 46576, 33832, 42928, 27684, 12568,
                2320,  34872, 53260, 58040, 38420, 17280, 36024, 5048,  50952,
                16628, 39060,
            },
            {
                12904, 880,   56108, 9096,  39612, 19888, 47296, 64256, 23924,
                60648, 27636, 57972, 11132, 49284, 26820, 54240, 16444, 28524,
                50172, 20064, 42208, 5176,  47756, 11684, 58532, 64984, 20648,
                54820, 2416,  49892, 60356, 10492, 14876, 33876, 61448, 46092,
                56064, 41556, 2836,  21716, 14776, 30600, 36584, 52604, 28436,
                34288, 48004, 41516, 4748,  25996, 49120, 18852, 28364, 44308,
                31592, 7152,  16868, 20428, 42740, 8712,  64932, 27316, 32824,
                45340, 54272, 14160, 2400,  41080, 21472, 25216, 1280,  32680,
                48144, 26240, 7032,  50808, 63772, 8760,  60656, 49008, 11492,
                41232, 35612, 7616,  13372, 49056, 33256, 40480, 60560, 35868,
                56404, 48128, 23484, 41356, 2660,  45224, 32744, 40704, 48256,
                14428, 42644, 64732, 38568, 44928, 60228, 50828, 24580, 19048,
                52908, 11680, 26572, 20868, 53968, 424,   17484, 58616, 48668,
                20852, 30292, 14812, 24304, 544,   54888, 20432, 61624, 25396,
                9568,  64788,
            },
            {
                36580, 26304, 43872, 22408, 49280, 59224, 28156, 60,    18096,
                12784, 53948, 4432,  22880, 39912, 8120,  21004, 62740, 3064,
                36304, 13788, 55784, 63904, 18196, 44312, 284,   36240, 6244,
                39324, 23568, 16884, 43952, 25504, 57656, 42724, 2284,  27184,
                10368, 64076, 28584, 44488, 8784,  57640, 17824, 56,    40072,
                13200, 3304,  60796, 31536, 55732, 11864, 38476, 59748, 24208,
                10424, 49812, 46384, 60592, 1740,  36032, 47184, 55976, 628,
                19780, 38752, 28568, 63440, 5432,  30956, 43096, 61236, 8588,
                20708, 60204, 55564, 30004, 45196, 22312, 34232, 53648, 18652,
                724,   47544, 30036, 20368, 59160, 1140,  25900, 15620, 29996,
                44788, 20188, 61876, 8388,  58052, 15412, 21724, 10012, 59920,
                24928, 31728, 7412,  27612, 21000, 14700, 40124, 7896,  62384,
                45044, 57044, 39940, 30316, 13060, 36600, 51744, 23480, 38520,
                60452, 44180, 9128,  41856, 50192, 31464, 12556, 47448, 2068,
                42188, 30620,
            },
            {
                19216, 50476, 61564, 15168, 31216, 5864,  37968, 44024, 32324,
                41440, 36356, 47000, 30848, 64456, 13996, 33192, 43136, 47500,
                60460, 24268, 1644,  27976, 38116, 22396, 31900, 48316, 14324,
                30008, 62900, 33076, 7020,  36204, 20848, 29472, 54596, 18692,
                23032, 37240, 5396,  53932, 39288, 63152, 49416, 23020, 56612,
                63980, 19928, 51672, 15084, 44708, 34600, 7580,  15736, 54380,
                62844, 14092, 37604, 29040, 24772, 53452, 18464, 14728, 6524,
                59696, 24636, 10804, 47108, 35316, 55776, 12132, 50288, 39388,
                15284, 34768, 3288,  40192, 16220, 2256,  27416, 38904, 31616,
                65384, 54796, 43820, 38028, 63216, 11304, 46980, 54452, 5724,
                2100,  12808, 34284, 51660, 38144, 30020, 62444, 53040, 956,
                19088, 54768, 12304, 58244, 5004,  53728, 504,   28872, 36820,
                16156, 32924, 2612,  59836, 43640, 63492, 6184,  32328, 10604,
                4260,  51316, 65296, 28668, 5312,  58796, 39868, 33768, 60072,
                22516, 57256,
            },
            {
                4228,  8364,  40624, 2568,  24692, 55192, 11016, 53092, 62624,
                6824,  20392, 59672, 1576,  51360, 37400, 56868, 10500, 30272,
                6936,  40676, 34092, 51504, 7768,  56504, 25232, 53096, 61104,
                9716,  45644, 56804, 48552, 688,   65416, 8996,  38644, 51180,
                32468, 15680, 60084, 19600, 25972, 12032, 31828, 43148, 8000,
                26892, 45932, 37904, 22912, 1128,  65004, 48236, 20964, 33040,
                41256, 6320,  56532, 4420,  50560, 11324, 31080, 61988, 34788,
                49512, 43252, 56932, 16152, 51348, 7368,  19292, 29200, 57268,
                44412, 52296, 12612, 24524, 56896, 47848, 59564, 5876,  14340,
                23840, 9188,  16516, 4988,  22824, 42040, 31876, 18320, 64416,
                39492, 58940, 28276, 3924,  18488, 46520, 5092,  34432, 27152,
                39332, 46708, 36700, 50180, 23528, 33336, 47752, 59520, 10512,
                22400, 6508,  50696, 19260, 8912,  27088, 15672, 46232, 56564,
                18616, 25736, 36292, 16592, 22116, 45064, 8248,  26752, 14752,
                52484, 45600,
            },
            {
                28100, 59612, 32476, 46768, 65228, 34812, 15968, 21580, 26268,
                50092, 10268, 16324, 44576, 25444, 18624, 4012,  53360, 22492,
                17616, 57684, 44552, 15408, 62104, 10476, 42184, 17376, 5248,
                40884, 19688, 26676, 11952, 52012, 41260, 14104, 47464, 6096,
                57420, 43192, 48796, 35048, 2384,  46572, 5112,  35744, 16340,
                54356, 10540, 30288, 6816,  56844, 28712, 40080, 3928,  51460,
                27040, 18304, 34296, 21680, 63568, 38444, 44472, 23996, 40388,
                9616,  2920,  32052, 21776, 26604, 64216, 36624, 2632,  23104,
                5376,  27824, 64800, 8164,  37112, 20252, 10792, 52044, 46012,
                57428, 28560, 50256, 34540, 56588, 52328, 8496,  37192, 24300,
                49484, 21268, 53620, 43608, 24836, 56336, 13484, 50544, 43172,
                16048, 63920, 2504,  29692, 62472, 17364, 42316, 55872, 26016,
                46664, 63148, 37628, 24484, 47612, 55216, 39848, 29280, 62468,
                33708, 1528,  47976, 11220, 53428, 62488, 18712, 49872, 5612,
                38204, 11396,
            },
            {
                34992, 54808, 13620, 19668, 51144, 7128,  58900, 45988, 1988,
                39644, 55528, 28856, 35200, 5980,  60840, 40180, 27536, 49640,
                62960, 11836, 31112, 21028, 48812, 3320,  59304, 35576, 28580,
                51436, 3648,  37996, 16128, 22872, 60140, 31276, 24504, 62332,
                1608,  13072, 9204,  30212, 51880, 58152, 21408, 61904, 49968,
                1672,  33892, 62556, 42704, 18008, 24980, 60888, 12544, 44988,
                836,   58828, 47668, 41924, 15928, 7840,  3636,  58136, 20732,
                29320, 52452, 60564, 5040,  41604, 45364, 14644, 48480, 62132,
                33200, 18836, 41572, 49920, 30748, 62868, 42624, 17844, 33576,
                40292, 2004,  60480, 25724, 14068, 3820,  28404, 57452, 44236,
                14584, 7092,  31748, 11036, 65312, 36644, 8744,  23000, 57692,
                6664,  10916, 20356, 44304, 8732,  38320, 12776, 20028, 2188,
                52588, 41484, 14244, 58348, 34580, 1800,  22156, 5216,  12660,
                40828, 54716, 59400, 38580, 30220, 2544,  35712, 56024, 30992,
                64376, 17668,
            },
            {
                49132, 22952, 39068, 9952,  26856, 41620, 29696, 36764, 13764,
                31600, 61872, 23404, 57912, 12424, 47124, 32396, 14948, 2036,
                37672, 46368, 4588,  26904, 39312, 33428, 12588, 46760, 22140,
                60032, 31580, 63876, 55068, 35072, 3312,  44540, 17604, 34552,
                27764, 39460, 64760, 24096, 17168, 40740, 13700, 29004, 38468,
                24648, 57764, 13040, 47800, 53188, 9220,  35916, 55376, 30976,
                23028, 61980, 10024, 29816, 55860, 26720, 52948, 48168, 13444,
                64484, 17416, 37280, 12356, 33672, 59140, 8788,  52872, 38692,
                10720, 59700, 45936, 14604, 1180,  35076, 25756, 4104,  61964,
                21104, 11784, 36956, 45796, 19244, 48720, 61432, 11928, 1392,
                35416, 62744, 40564, 17020, 360,   48852, 41312, 29400, 61532,
                32632, 53376, 35504, 26500, 55252, 5676,  31100, 65412, 35080,
                28348, 4560,  31528, 11008, 17916, 64880, 49356, 52808, 43828,
                20752, 7792,  23772, 14272, 43072, 25208, 10264, 41192, 21512,
                328,   43104,
            },
            {
                6768,  62716, 1836,  44964, 57852, 3596,  17992, 63116, 52792,
                4404,  47788, 7932,  42108, 52360, 21388, 65300, 43768, 55536,
                24748, 9020,  52020, 64712, 54804, 18548, 25860, 56168, 860,
                14900, 43528, 8228,  47156, 28164, 7088,  58100, 11072, 50252,
                55956, 21160, 53748, 45184, 6912,  60896, 53076, 9636,  45508,
                18412, 40900, 21252, 2480,  32576, 15280, 46412, 19764, 7796,
                38592, 52440, 13772, 2204,  45844, 33172, 18048, 36388, 1192,
                42756, 25840, 46724, 54280, 280,   19616, 24412, 30840, 17164,
                1832,  25028, 53468, 5968,  22276, 55504, 50796, 8440,  30096,
                53216, 43368, 6732,  64440, 30668, 40020, 22172, 33248, 51780,
                25952, 47400, 54484, 27644, 60212, 33176, 20528, 3176,  17348,
                47392, 24508, 60608, 49016, 15444, 58600, 51032, 45332, 9672,
                17156, 60948, 54024, 44676, 7048,  29796, 36876, 15344, 26712,
                60304, 31424, 46040, 5168,  64104, 57440, 48176, 15920, 60464,
                53512, 25532,
            },
            {
                57384, 16320, 30260, 36244, 53732, 23792, 48084, 11404, 22500,
                43604, 19032, 15076, 37900, 1164,  30216, 10144, 6736,  34372,
                20128, 60348, 35836, 13660, 1916,  45152, 7532,  63328, 36748,
                49320, 11436, 24788, 18864, 40444, 52944, 21576, 37948, 46208,
                4572,  16120, 32140, 1036,  37412, 26600, 33396, 3396,  65484,
                5608,  51632, 60372, 27404, 39144, 64252, 4608,  58488, 43396,
                16660, 34960, 25140, 40108, 65480, 21312, 59788, 9132,  30880,
                50872, 5848,  22656, 62904, 40060, 28416, 57604, 43388, 47192,
                56212, 36360, 29532, 39944, 58700, 16764, 44376, 38552, 13872,
                57884, 24116, 17056, 55240, 9544,  2724,  43048, 16060, 59972,
                19044, 4624,  9532,  21988, 44784, 14928, 56688, 52312, 38372,
                5868,  13356, 41824, 760,   39468, 21764, 3512,  24440, 37360,
                48592, 21604, 39248, 25432, 56596, 41748, 61248, 9260,  20,
                48836, 16780, 35432, 51256, 19236, 33304, 7264,  28688, 45852,
                13100, 32196,
            },
            {
                40816, 46868, 20956, 60920, 14564, 33132, 7692,  38596, 59936,
                34696, 25976, 63756, 33092, 55156, 17836, 50812, 26832, 58332,
                41380, 16876, 28836, 43060, 23656, 50752, 30596, 39832, 19772,
                54148, 33636, 58480, 2072,  62112, 14672, 30612, 64316, 25956,
                8608,  41884, 61932, 48704, 11100, 19652, 44080, 22632, 55632,
                31336, 11328, 36448, 6472,  49848, 23460, 29620, 50888, 26448,
                63208, 4028,  57724, 50316, 5816,  11844, 43836, 54892, 39660,
                15692, 58548, 10276, 31408, 49160, 14040, 3988,  65364, 6904,
                12884, 20084, 62340, 9376,  32976, 11932, 64352, 27412, 47340,
                800,   32740, 49708, 26968, 34896, 52840, 62616, 7500,  45760,
                31120, 38720, 58684, 35808, 50120, 6224,  11380, 26768, 63468,
                44872, 31184, 19204, 10108, 33108, 28652, 61880, 42080, 13968,
                58088, 632,   6180,  50876, 19560, 12428, 23500, 33964, 54968,
                39064, 63044, 10628, 28012, 2020,  52796, 23188, 39356, 3668,
                36452, 9324,
            },
            {
                51676, 27256, 5664,  11004, 43004, 64508, 50856, 28964, 388,
                53600, 9580,  49240, 5860,  23880, 61680, 39152, 45908, 12064,
                12,    48700, 5792,  57436, 33152, 61256, 16140, 9852,  27216,
                4864,  42264, 29908, 50868, 35960, 9972,  42920, 16,    19288,
                59524, 35824, 13796, 28120, 57032, 51016, 59456, 36084, 15136,
                48728, 25004, 16964, 43908, 57516, 13476, 41768, 1540,  11764,
                32804, 47536, 19976, 15336, 37220, 28608, 23632, 4544,  63664,
                27516, 35392, 44792, 18280, 7492,  37084, 52092, 21904, 34984,
                41492, 50604, 27028, 45420, 49032, 24476, 3480,  36140, 19972,
                62828, 41344, 5044,  58780, 12948, 23448, 37244, 28132, 11236,
                55608, 24072, 13704, 2604,  30164, 61960, 42244, 35068, 21280,
                2656,  58724, 51112, 64516, 54252, 46180, 18060, 7188,  53344,
                31292, 64268, 28000, 35616, 47112, 2700,  58144, 45212, 28828,
                20152, 4200,  56704, 41964, 36940, 59000, 11444, 62272, 49836,
                20108, 63944,
            },
            {
                1404,  33948, 56460, 48860, 2492,  25564, 19728, 44916, 16652,
                57672, 41100, 28104, 58888, 42996, 14344, 3232,  22288, 53424,
                31356, 63456, 38528, 21412, 11488, 3996,  41456, 59036, 46568,
                23016, 65040, 16896, 6112,  22344, 48104, 57260, 34048, 51092,
                29512, 54276, 23140, 3816,  40856, 16380, 7704,  1772,  27996,
                41612, 63352, 54812, 33540, 9040,  19192, 61228, 37168, 21732,
                55840, 8948,  44128, 31396, 54328, 61632, 48424, 33644, 19848,
                12332, 52636, 2588,  61712, 55196, 42456, 26552, 11152, 31508,
                59476, 15908, 5528,  516,   18268, 41848, 59840, 7516,  51136,
                15884, 10808, 38092, 20476, 44076, 48200, 18004, 57580, 588,
                49312, 42492, 64644, 52724, 39676, 24800, 15776, 48608, 9184,
                56056, 27820, 22992, 6536,  36100, 12176, 57108, 26608, 34300,
                10592, 43460, 15972, 9032,  62168, 32148, 16672, 6644,  51468,
                13288, 25420, 47724, 21492, 14420, 44960, 17500, 31588, 56156,
                24756, 15100,
            },
            {
                38208, 59712, 23604, 17124, 31204, 40312, 55768, 5776,  12856,
                31452, 21096, 2212,  11876, 35136, 48124, 29188, 36604, 59568,
                15036, 25344, 8352,  54540, 47016, 19328, 52472, 35648, 1580,
                55744, 13220, 38964, 44440, 60924, 25664, 5020,  16352, 11560,
                47212, 6492,  43932, 63124, 31952, 24876, 38996, 61004, 46856,
                9968,  5232,  20968, 392,   46540, 53720, 31136, 6868,  52344,
                39908, 27828, 60752, 188,   8060,  41504, 16960, 1728,  45780,
                56780, 41104, 20940, 24676, 33232, 15548, 62796, 47708, 3432,
                24016, 55116, 64048, 37872, 52348, 56660, 31420, 22484, 54580,
                28640, 45540, 61476, 31060, 1720,  65248, 6236,  41052, 34668,
                20920, 16488, 32964, 7868,  19416, 57488, 1008,  53456, 32720,
                40208, 14608, 37580, 43668, 16688, 1824,  49436, 39860, 20004,
                47584, 23108, 56284, 38168, 53464, 26660, 40744, 64200, 36296,
                42988, 60624, 32244, 6804,  65080, 26724, 696,   41424, 8084,
                29596, 44772,
            },
            {
                18716, 12248, 42000, 52668, 36716, 9304,  60688, 35124, 51532,
                64236, 38788, 46220, 18336, 51976, 8408,  63976, 5272,  43348,
                18900, 51216, 44320, 34528, 28440, 64408, 25712, 14412, 31180,
                49552, 8156,  28376, 52624, 11272, 31636, 54836, 39764, 63660,
                22120, 38384, 18288, 10080, 52276, 45448, 12140, 19308, 33116,
                52704, 37676, 29924, 62128, 38868, 26816, 15600, 44876, 64828,
                17628, 12628, 36260, 22476, 26140, 52212, 11392, 59088, 25624,
                36632, 6408,  29604, 50280, 9796,  1568,  57468, 19156, 39408,
                9164,  43716, 33968, 21008, 29108, 14432, 10396, 39924, 34508,
                2984,  56512, 24916, 8644,  53316, 15112, 29336, 25032, 52164,
                60856, 3916,  27384, 47868, 11128, 43700, 29436, 64428, 18308,
                47092, 4524,  50032, 61340, 25720, 31976, 63252, 5204,  59124,
                2408,  51604, 30072, 3896,  13636, 21924, 49612, 10748, 18516,
                3280,  53024, 9696,  39920, 50568, 54456, 34700, 47952, 60544,
                4460,  54800,
            },
            {
                8900,  65352, 28496, 6808,  62552, 14472, 22352, 26980, 47456,
                3964,  24860, 56276, 61436, 32128, 25880, 20452, 56972, 10580,
                32712, 3476,  60604, 13456, 980,   39192, 6156,  44748, 62232,
                20600, 34388, 57960, 18020, 1304,  36472, 19832, 45020, 27704,
                2124,  56708, 34352, 59196, 616,   28888, 63936, 56144, 23084,
                14712, 58484, 49100, 12236, 22900, 59928, 2684,  33764, 24472,
                4912,  49856, 58168, 46692, 64340, 32540, 38740, 50572, 8940,
                62928, 14452, 46476, 60676, 38824, 44240, 28876, 35384, 54012,
                49664, 27920, 12384, 6964,  60860, 44564, 4292,  64896, 47932,
                13236, 18080, 36264, 50000, 39996, 33212, 59764, 46288, 13092,
                9560,  44524, 54700, 37520, 60244, 22976, 36044, 7548,  26044,
                12088, 59244, 30140, 9548,  19620, 52848, 44256, 23804, 36444,
                12412, 62944, 18724, 42436, 59300, 46060, 844,   30960, 57464,
                22876, 34888, 16200, 29844, 4096,  23896, 10596, 20396, 14052,
                36048, 50232,
            },
            {
                32372, 47080, 3308,  20320, 44204, 49548, 1296,  42768, 17632,
                10120, 36944, 14940, 6400,  41776, 54648, 16064, 39704, 47088,
                27424, 37660, 22968, 56008, 17636, 50396, 57700, 23864, 10536,
                37860, 3968,  46976, 24744, 63984, 42328, 58948, 8424,  13540,
                51668, 30424, 15232, 48604, 21036, 36760, 6376,  49916, 2412,
                44332, 26312, 4352,  18028, 42732, 51012, 10196, 48024, 58984,
                42424, 29804, 2952,  18676, 13548, 5716,  20844, 30120, 43404,
                18204, 32836, 3740,  56484, 17564, 23132, 6848,  13536, 61200,
                2136,  17588, 58728, 47640, 36572, 25088, 50360, 19580, 26824,
                43428, 59448, 5636,  21984, 11552, 55580, 18788, 2344,  38600,
                63960, 29980, 21528, 5140,  14080, 51832, 3752,  45176, 60992,
                20948, 41864, 704,   56416, 34968, 13488, 7544,  29520, 16236,
                45836, 39236, 8196,  25168, 33840, 6316,  37392, 61792, 26000,
                46844, 55520, 44292, 61352, 18592, 57536, 38624, 63764, 27808,
                42712, 21896,
            },
            {
                26180, 39536, 58472, 34080, 54924, 29664, 57512, 32648, 60188,
                53740, 29224, 44436, 22480, 3144,  49396, 29896, 752,   52476,
                65024, 6332,  42140, 47852, 29368, 8748,  32456, 41772, 16312,
                54044, 61084, 40536, 13848, 50200, 29212, 3376,  47828, 24168,
                61848, 41488, 4820,  26004, 54712, 42772, 17268, 31560, 40396,
                9292,  35628, 65032, 32812, 7140,  56980, 36004, 20652, 14888,
                8380,  37972, 55588, 34632, 41188, 54500, 61408, 2372,  23556,
                48008, 54132, 27744, 11980, 36464, 52964, 63856, 45668, 25364,
                32780, 41976, 22944, 54408, 1312,  16408, 33056, 8960,  52592,
                80,    30800, 63592, 46952, 27528, 43788, 7536,  22808, 50204,
                34972, 17172, 56920, 42132, 33616, 63036, 17012, 39232, 54668,
                33404, 51916, 23440, 38508, 65104, 47836, 41028, 60344, 54320,
                26900, 32768, 57880, 50216, 65304, 17808, 53888, 12448, 39696,
                7632,  1368,  27480, 11808, 49224, 32960, 45828, 5744,  51960,
                56736, 1032,
            },
            {
                13152, 51464, 15872, 24456, 11184, 4472,  39032, 12400, 20884,
                7664,  48260, 63608, 33564, 11588, 59376, 35772, 13432, 21288,
                58128, 16744, 11092, 62036, 36544, 21880, 63788, 46640, 1960,
                29748, 26576, 6812,  22036, 9792,  34100, 16084, 54312, 32496,
                37108, 10956, 45804, 65208, 8436,  13264, 59484, 24344, 62440,
                55464, 20344, 46192, 53280, 28424, 13720, 40964, 30452, 63652,
                25676, 51072, 21920, 9864,  45808, 27292, 15848, 58256, 36984,
                4924,  65196, 21436, 42764, 476,   49104, 31268, 20312, 4732,
                51908, 38156, 10020, 30076, 63000, 41176, 56260, 61488, 38192,
                23684, 41292, 14708, 57216, 3456,  35452, 62636, 31416, 58260,
                26036, 3052,  8480,  48408, 24720, 30756, 49992, 9808,  28280,
                15124, 6356,  46032, 17220, 27500, 3256,  21824, 10464, 50512,
                264,   6044,  21216, 14892, 10388, 44104, 28172, 20912, 50896,
                15176, 64540, 37188, 22464, 41952, 2796,  15376, 30900, 9628,
                17540, 61420,
            },
            {
                43600, 7552,  62764, 37544, 18880, 45876, 25732, 64888, 41736,
                36036, 948,   57116, 19004, 39128, 25336, 62776, 43696, 8516,
                34460, 26204, 31840, 2688,  52600, 13244, 4960,  55124, 19532,
                50780, 33408, 43860, 57396, 52692, 38668, 63012, 20664, 6740,
                60120, 18108, 22884, 39240, 28576, 34560, 47256, 4204,  11440,
                29244, 15352, 2008,  39400, 24640, 61832, 720,   54064, 45704,
                4152,  57480, 16568, 62820, 1052,  49372, 33264, 12508, 52068,
                40904, 9428,  35016, 58468, 26440, 8240,  40068, 11076, 57868,
                15524, 65036, 6168,  45848, 13292, 21188, 28224, 6432,  12240,
                34648, 54216, 9364,  20436, 38816, 16632, 48660, 10588, 41880,
                14408, 52460, 39100, 59516, 11696, 76,    20640, 58344, 2736,
                36800, 62660, 11468, 54884, 5576,  32192, 57264, 17976, 34820,
                62528, 41904, 47320, 30924, 55408, 35408, 2048,  60260, 42116,
                33088, 19536, 58496, 51420, 8400,  62276, 25548, 59348, 40372,
                23596, 34412,
            },
            {
                55444, 27844, 2572,  48172, 53952, 59680, 6136,  15656, 49832,
                24080, 13892, 27124, 52016, 8176,  46716, 4520,  23384, 50664,
                40632, 46272, 55492, 20024, 43496, 27392, 40140, 35092, 59892,
                11020, 14852, 65464, 404,   18528, 46480, 4412,  26708, 43772,
                49764, 960,   53172, 57772, 3196,  51060, 19228, 38016, 53016,
                43092, 49676, 60100, 8728,  17016, 49192, 6464,  18736, 33472,
                12364, 28380, 43228, 31644, 23808, 39124, 8320,  44296, 26208,
                16748, 31012, 50144, 13960, 61984, 16980, 55392, 34016, 43016,
                27396, 47968, 19084, 35588, 52236, 3172,  44732, 48856, 58832,
                17368, 45824, 29160, 51424, 60760, 25704, 53480, 1084,  20088,
                45968, 63364, 28152, 18360, 45312, 55132, 40572, 25764, 44272,
                49044, 31264, 24696, 42688, 51056, 61168, 39000, 44636, 23588,
                13876, 56908, 25936, 3852,  40144, 23240, 47960, 5572,  29404,
                9768,  47188, 4776,  30116, 34952, 18160, 53448, 36696, 46552,
                5192,  49036,
            },
            {
                10768, 21300, 41224, 31444, 9780,  35320, 29092, 52480, 56508,
                4300,  31280, 61692, 42660, 15588, 55316, 32176, 17856, 54224,
                1452,  14548, 60368, 5488,  49432, 57948, 16016, 25160, 8068,
                38480, 48196, 24112, 36832, 28508, 12488, 41068, 58764, 9652,
                29652, 36060, 12392, 32268, 15880, 44196, 61496, 26588, 33196,
                5472,  22224, 36736, 31764, 55752, 43012, 22752, 37712, 60652,
                40440, 48512, 6956,  58676, 53536, 18796, 64060, 55012, 6104,
                61028, 20156, 2636,  44924, 23052, 29416, 46732, 3016,  21864,
                60644, 1548,  31572, 57232, 26412, 39560, 15476, 31300, 22280,
                4984,  64620, 2220,  33136, 6684,  13260, 40380, 29864, 55780,
                6380,  36020, 22164, 32064, 5804,  65444, 34272, 13008, 60868,
                17920, 56052, 8356,  20212, 35736, 15228, 1796,  29888, 8432,
                49468, 37488, 18560, 63736, 9000,  59148, 16636, 52504, 63072,
                24852, 54828, 39456, 13840, 56120, 43996, 316,   12396, 20248,
                64824, 30256,
            },
            {
                36212, 58372, 17488, 64088, 22956, 64,    44016, 19556, 33436,
                40116, 46352, 10016, 36980, 1920,  28936, 59704, 36660, 11156,
                63728, 30476, 24480, 37692, 10552, 32212, 1376,  62360, 45048,
                20936, 53832, 5616,  31920, 61788, 54692, 22416, 33316, 15184,
                56256, 24824, 47552, 63400, 21852, 7856,  55796, 52,    14180,
                64072, 57008, 27000, 11972, 46876, 64304, 27592, 9724,  51940,
                2860,  20888, 15056, 37212, 11060, 3676,  29744, 22232, 35604,
                46940, 56828, 38836, 53524, 4596,  37436, 64172, 49880, 14072,
                36432, 54344, 12060, 23212, 60180, 9152,  62824, 35972, 55644,
                42480, 26560, 37472, 47812, 23380, 44504, 64028, 34424, 24204,
                60256, 12804, 2116,  51712, 38020, 15936, 47880, 7720,  22488,
                4060,  39416, 64196, 28860, 12540, 48228, 25536, 64672, 54856,
                4320,  33420, 11964, 51932, 44448, 27112, 37728, 13376, 34544,
                43624, 17352, 1632,  26688, 64032, 22060, 29104, 49936, 57052,
                3404,  15180,
            },
            {
                53240, 45300, 6496,  50852, 14456, 38844, 61760, 11476, 7884,
                60044, 17536, 21912, 50612, 65472, 20020, 5404,  48232, 26556,
                39556, 6864,  17584, 45488, 64868, 21668, 41688, 51656, 30240,
                3116,  59352, 42220, 16556, 8808,  48500, 1888,  50892, 64528,
                39892, 19664, 6304,  41968, 27672, 11144, 40876, 23664, 47916,
                17984, 40492, 6776,  19980, 1408,  34376, 14120, 58076, 31076,
                24888, 65468, 33880, 51212, 25924, 47300, 59240, 41836, 1564,
                13224, 28268, 10188, 32992, 59380, 7404,  19012, 9772,  24992,
                40848, 7056,  43756, 49784, 4456,  18420, 52932, 784,   10968,
                50100, 19252, 14336, 59632, 10256, 56680, 4044,  17732, 9312,
                48528, 40920, 44364, 61976, 10520, 27060, 53684, 29764, 43100,
                51520, 33748, 828,   46204, 59016, 6640,  52428, 40504, 19716,
                46312, 28612, 58408, 1060,  31736, 20560, 3356,  57144, 7044,
                21476, 59796, 50276, 45768, 10328, 38428, 7280,  33500, 42352,
                26540, 39432,
            },
            {
                24024, 2224,  32820, 42952, 26080, 57060, 47032, 28072, 23728,
                49264, 3156,  58516, 25828, 33936, 45232, 13696, 41932, 56644,
                21016, 50332, 53540, 34316, 7836,  28168, 55228, 12608, 17788,
                34612, 26948, 11788, 56488, 43980, 25112, 37468, 18688, 7784,
                45056, 2936,  60680, 50456, 36324, 59256, 30368, 52432, 35036,
                9368,  44588, 60756, 54304, 38784, 50680, 4660,  41908, 16832,
                54256, 43776, 592,   60252, 40692, 14332, 32828, 17300, 50640,
                24628, 64728, 43552, 16132, 26028, 41992, 31360, 56772, 51872,
                28816, 62680, 16228, 37772, 33288, 47416, 41368, 29516, 24448,
                62072, 8116,  40196, 52824, 31364, 20760, 49712, 38216, 27204,
                54096, 30416, 17452, 24564, 57424, 2852,  19896, 36384, 59220,
                11208, 15340, 26372, 54104, 37164, 22644, 32108, 15900, 10556,
                61468, 22448, 39796, 15028, 50372, 62548, 40708, 49308, 30556,
                11336, 36220, 5936,  32156, 60708, 15948, 52900, 62880, 17612,
                8936,  61796,
            },
            {
                12652, 59888, 28808, 10548, 52800, 5324,  16956, 36380, 64424,
                42148, 29788, 38500, 12152, 7340,  53304, 23504, 61824, 8656,
                32736, 2844,  42472, 59464, 15032, 48116, 37096, 4272,  46532,
                63576, 39640, 51188, 20068, 35160, 4652,  57728, 30868, 12900,
                28220, 55168, 33512, 16800, 1620,  45592, 19496, 62804, 3780,
                28196, 51512, 30796, 15404, 25328, 62164, 21932, 46652, 7232,
                36332, 19248, 9524,  28976, 21620, 4848,  53044, 8844,  61820,
                36428, 6900,  48804, 21272, 51564, 12252, 61212, 44780, 112,
                34224, 20292, 58312, 10008, 27856, 64476, 21332, 13804, 45588,
                33952, 57508, 28076, 1348,  36280, 15436, 43280, 62320, 448,
                14596, 59180, 7912,  33804, 49228, 42508, 62988, 5360,  46756,
                23916, 61888, 41684, 18368, 9964,  44252, 3216,  57404, 35192,
                43336, 7220,  53560, 25332, 35684, 9320,  18116, 46176, 23620,
                65152, 53808, 41820, 19804, 25060, 48384, 4452,  23488, 46308,
                31400, 50468,
            },
            {
                35380, 47600, 19120, 63360, 34216, 21568, 2868,  55148, 13708,
                6500,  52416, 16136, 56172, 40888, 30344, 612,   35804, 15640,
                28696, 63356, 12368, 25908, 216,   20072, 61596, 24632, 57176,
                9444,  23208, 1112,  29848, 64160, 15000, 46124, 62324, 52560,
                38464, 20796, 10752, 25792, 54260, 14392, 6832,  38540, 57808,
                12496, 22784, 36308, 2712,  43476, 10860, 32772, 28724, 63068,
                12300, 57024, 49544, 45716, 63224, 37592, 57860, 43948, 30736,
                19332, 39836, 904,   56304, 35104, 3664,  38400, 22604, 14544,
                6108,  53568, 46548, 23944, 1736,  55960, 7192,  38492, 54616,
                3332,  16908, 44052, 22432, 65172, 5080,  29296, 51448, 22908,
                35508, 46588, 3556,  39156, 21800, 12664, 32336, 17188, 56472,
                7176,  31152, 49516, 4768,  63308, 29384, 60472, 49360, 26800,
                1952,  17780, 64412, 47444, 5000,  59628, 28140, 56100, 2364,
                39320, 27212, 14468, 3452,  56940, 34916, 40500, 13144, 59368,
                892,   20952,
            },
            {
                56028, 15816, 38316, 8012,  41676, 61040, 50116, 31916, 44760,
                20048, 34816, 2164,  46972, 63704, 18272, 49796, 58656, 43888,
                47940, 18872, 38284, 56540, 31480, 51048, 41040, 6624,  32988,
                14664, 44480, 59968, 7644,  49556, 27168, 10432, 41744, 23692,
                5180,  59068, 47292, 65132, 40264, 32216, 49816, 25292, 17072,
                46248, 64820, 6080,  49300, 59676, 17332, 55952, 52468, 3900,
                39684, 26652, 32072, 5824,  13024, 18408, 25100, 2108,  12084,
                54144, 60128, 14212, 29432, 62912, 47236, 17444, 27304, 48956,
                65328, 39372, 30688, 13172, 43504, 59860, 17328, 51088, 31116,
                6444,  49024, 59996, 12372, 47524, 55944, 9064,  40628, 12008,
                55052, 20420, 61568, 52404, 29376, 55336, 568,   40592, 27484,
                38224, 20348, 57848, 35004, 14496, 40244, 20636, 12592, 38228,
                55908, 30088, 33704, 13052, 21636, 41980, 12024, 33260, 16232,
                8520,  51628, 58976, 30280, 43732, 9208,  54740, 28556, 36752,
                43924, 7204,
            },
            {
                40936, 4304,  58044, 24936, 46120, 28316, 11232, 39852, 24108,
                62428, 57688, 27616, 22692, 9516,  37452, 26256, 11408, 3912,
                54376, 23956, 5596,  45376, 35240, 10148, 16504, 28736, 53104,
                48360, 18632, 37276, 54604, 40520, 21440, 33328, 288,   16268,
                35656, 43324, 29712, 3832,  22684, 9540,  56604, 42856, 2376,
                33016, 54944, 20680, 40576, 27308, 8324,  37976, 20340, 48784,
                15384, 61060, 22424, 55140, 42400, 34024, 49828, 28084, 47384,
                23008, 33112, 45316, 24364, 5808,  10236, 53820, 59420, 36672,
                9036,  18596, 4644,  50380, 32876, 36456, 25276, 11068, 63864,
                20296, 35500, 24608, 41544, 32972, 18084, 25932, 60196, 32592,
                5652,  43752, 26664, 9564,  15156, 64616, 45140, 50312, 9824,
                53284, 12960, 1932,  44900, 25984, 54624, 7824,  46172, 24120,
                51208, 9832,  44692, 56880, 38012, 168,   53236, 44388, 62060,
                35304, 19068, 45620, 11988, 61700, 21696, 2720,  18916, 51300,
                65396, 27096,
            },
            {
                52996, 31548, 49480, 1780,  14204, 18400, 59120, 912,   8680,
                48468, 11908, 43040, 32368, 60904, 5384,  55260, 20896, 34444,
                40204, 60952, 9052,  52376, 21940, 58916, 43208, 63884, 38352,
                4496,  26172, 31520, 13596, 3444,  61640, 47924, 54904, 60788,
                50096, 7940,  13664, 52272, 36852, 62364, 20216, 29348, 60548,
                37684, 10044, 14532, 31484, 56992, 45888, 364,   34164, 25136,
                44752, 1900,  35932, 9160,  52352, 3656,  64556, 39004, 56524,
                9712,  3992,  52132, 17968, 41000, 32224, 43308, 2232,  29840,
                24384, 41808, 61252, 54088, 20828, 3028,  47308, 42892, 27388,
                39668, 55540, 9812,  3876,  52572, 37312, 6652,  45720, 16124,
                64096, 37844, 17956, 48108, 35284, 6976,  25516, 22104, 60272,
                32728, 47312, 65212, 22840, 51772, 33184, 556,   62732, 18764,
                5672,  61612, 16040, 27448, 50080, 60864, 30712, 25024, 5712,
                22184, 49048, 832,   38004, 26492, 49768, 32980, 58444, 15164,
                9520,  22916,
            },
            {
                11960, 19500, 64024, 36180, 54908, 31040, 51896, 36948, 26188,
                54048, 4132,  18704, 51152, 14732, 41252, 45944, 60040, 31236,
                14000, 28088, 17256, 65044, 3104,  26936, 13140, 1660,  20864,
                60580, 56216, 10740, 51648, 35456, 18236, 25468, 9400,  31140,
                19628, 27284, 57288, 17428, 45460, 548,   12844, 48288, 7852,
                51132, 24396, 44408, 3400,  61300, 23036, 13928, 64952, 10360,
                57644, 40712, 17744, 28788, 59136, 14364, 20584, 7612,  16676,
                42596, 62328, 37428, 57776, 26640, 64108, 21060, 12852, 51356,
                57532, 45744, 10796, 26884, 15064, 62496, 8476,  58384, 14236,
                292,   61124, 16256, 30380, 62856, 20788, 56832, 49216, 24036,
                2152,  30768, 58024, 4128,  41720, 59336, 18444, 36284, 3336,
                15856, 28868, 5924,  38628, 9648,  16600, 57016, 36068, 42760,
                31688, 40324, 3520,  23516, 8160,  19604, 14996, 47920, 40916,
                55072, 64280, 31836, 7200,  17052, 63416, 39740, 5552,  42696,
                47996, 34324,
            },
            {
                61576, 28692, 42088, 9876,  23156, 6348,  44020, 65136, 16192,
                30000, 39020, 58880, 35212, 1708,  28528, 17160, 7568,  51608,
                976,   49428, 41568, 30912, 37388, 46788, 55436, 49756, 34796,
                8360,  41168, 23316, 65272, 42972, 5492,  57628, 44672, 39728,
                3276,  63412, 33656, 40908, 25644, 55204, 35196, 41448, 16364,
                27688, 63460, 53864, 18656, 35672, 50128, 41984, 53580, 31064,
                6248,  51228, 63688, 47904, 23892, 31776, 45896, 61016, 35460,
                30032, 21404, 12480, 7168,  49172, 15528, 54984, 35556, 5288,
                16752, 33796, 768,   38252, 55820, 40764, 29964, 22668, 52736,
                32428, 45516, 50384, 26332, 43780, 1196,  13576, 29088, 40100,
                53868, 10924, 50364, 22460, 32100, 53692, 11776, 40000, 62288,
                43636, 57148, 19296, 42012, 59984, 29988, 48036, 27008, 13524,
                53124, 59048, 35532, 46832, 64980, 36964, 3936,  57944, 10496,
                28756, 13604, 42876, 23336, 53764, 46676, 12700, 30488, 25648,
                56572, 2288,
            },
            {
                45060, 5668,  15484, 47532, 60308, 40064, 20508, 12644, 33844,
                46264, 22008, 7948,  47624, 64652, 53780, 23136, 38100, 63528,
                25352, 44688, 11136, 57204, 19348, 7028,  24432, 31368, 17232,
                45032, 28952, 808,   47180, 15724, 29588, 37588, 13088, 22704,
                51972, 46920, 10912, 4564,  59932, 30968, 5976,  23180, 57836,
                1460,  33588, 39580, 7156,  11656, 29116, 4936,  19184, 38356,
                26972, 21368, 11920, 4600,  37944, 41940, 144,   26224, 53612,
                2792,  50848, 44652, 34620, 668,   39520, 8340,  61776, 48068,
                22292, 63940, 29444, 49488, 17768, 4084,  34472, 48644, 5904,
                19276, 36412, 7788,  11596, 39540, 59292, 35164, 9248,  61156,
                19564, 43232, 14044, 62968, 28420, 1532,  47504, 27072, 51292,
                8560,  34492, 25008, 50044, 12168, 4112,  21740, 64356, 2680,
                10156, 20616, 28796, 54736, 11400, 43332, 52144, 33504, 18052,
                2104,  37060, 50820, 59132, 9880,  35248, 1236,  20880, 60620,
                17552, 39108,
            },
            {
                22468, 50936, 56740, 26388, 32752, 2980,  48964, 57804, 4812,
                54972, 62536, 13384, 25496, 31532, 10152, 42884, 12596, 55612,
                19924, 36172, 5084,  50908, 14872, 62656, 40220, 11000, 52544,
                62096, 14416, 57340, 33236, 21068, 60216, 49624, 1432,  61936,
                15608, 36608, 24284, 20092, 49096, 15212, 51836, 62780, 43656,
                20352, 13036, 46424, 59060, 26052, 62632, 47644, 56340, 14992,
                61508, 46580, 34760, 16464, 56128, 51452, 10872, 58236, 13760,
                23688, 65124, 18484, 28008, 60884, 23408, 30744, 44360, 26164,
                41072, 13368, 7752,  59756, 21596, 45612, 64948, 12288, 57128,
                41164, 23464, 55048, 64592, 17784, 22636, 51172, 44556, 5156,
                33860, 25876, 36596, 7608,  45004, 15684, 56268, 6344,  21496,
                13880, 54956, 1028,  63232, 36904, 54152, 45272, 39228, 24776,
                44124, 50584, 17120, 1284,  31960, 26360, 21560, 63032, 39616,
                25368, 60712, 19528, 3708,  28284, 57112, 44056, 51904, 37256,
                6800,  54600,
            },
            {
                13164, 34932, 148,   38572, 17420, 63436, 28260, 10132, 24716,
                41212, 492,   44324, 19964, 57316, 35988, 4892,  46924, 32312,
                2584,  61524, 22964, 33788, 29076, 43704, 58716, 4620,  22340,
                38676, 5920,  50292, 26024, 7304,  10376, 53944, 26628, 32008,
                7588,  54936, 42396, 64448, 8720,  38680, 28352, 10780, 37152,
                49200, 5008,  30152, 51792, 16460, 37340, 3204,  33348, 8528,
                41364, 1056,  30168, 60164, 8208,  27752, 19440, 36536, 48756,
                40040, 32032, 10280, 55992, 46688, 52844, 19076, 11248, 3796,
                56044, 36176, 53192, 43256, 25524, 9632,  37556, 27472, 16588,
                62172, 1748,  28804, 46392, 32832, 3168,  26804, 15252, 55040,
                65400, 212,   51980, 57140, 38780, 20184, 34012, 64780, 30980,
                37776, 46000, 17092, 27916, 32068, 18960, 6396,  15392, 55936,
                30696, 7476,  62532, 41396, 14008, 59800, 5752,  45168, 8504,
                55476, 46348, 34356, 15808, 41096, 24832, 14084, 64608, 9276,
                27748, 31996,
            },
            {
                63820, 19760, 58536, 8536,  52152, 13776, 42804, 36864, 51312,
                17712, 29120, 37704, 52260, 2728,  49388, 18432, 58972, 27944,
                15584, 47764, 58132, 8552,  52856, 876,   35592, 48292, 27108,
                55008, 19504, 36520, 63424, 44280, 34864, 41444, 19400, 45680,
                58620, 28716, 1984,  32664, 53436, 44980, 3660,  18240, 31880,
                55568, 65380, 22688, 42128, 9344,  20736, 44560, 23788, 51412,
                57876, 25312, 50036, 43264, 22624, 63136, 32748, 5292,  45092,
                7928,  59152, 4584,  42004, 13440, 3032,  38176, 49884, 32696,
                59232, 19740, 2080,  31672, 14528, 51024, 54744, 3460,  43648,
                31992, 49508, 14396, 5480,  52776, 57504, 38436, 48304, 31168,
                22044, 17080, 47396, 10320, 23940, 60136, 42532, 2968,  48948,
                24156, 61480, 4920,  43056, 9328,  49520, 58936, 33660, 61752,
                36732, 48252, 23236, 56464, 38172, 47816, 16788, 30176, 51308,
                12576, 5160,  30532, 49260, 61720, 6160,  33032, 47148, 16572,
                49528, 43132,
            },
            {
                10508, 46800, 25236, 30320, 44968, 22380, 55484, 7428,  31924,
                59164, 11120, 61052, 15512, 27160, 62640, 23988, 40088, 7104,
                53392, 41388, 37476, 12888, 26272, 20700, 16656, 64624, 31696,
                9336,  42600, 3128,  12624, 55652, 16772, 5644,  64716, 38056,
                11356, 18140, 47992, 13820, 26424, 21756, 56968, 60912, 25320,
                8384,  14860, 35168, 2260,  57624, 53920, 30648, 63272, 11448,
                18748, 36704, 14304, 3700,  39044, 46828, 54528, 14840, 60820,
                29312, 17264, 37216, 22208, 33592, 27064, 55280, 64836, 17076,
                9192,  28024, 47344, 63256, 5308,  33780, 61076, 20596, 8272,
                24896, 58872, 37160, 21080, 41892, 10324, 60812, 6696,  12028,
                41668, 59408, 27700, 32668, 4196,  12720, 52888, 17792, 9752,
                40544, 12388, 51700, 57636, 23496, 39876, 12872, 27296, 332,
                11168, 18636, 4212,  33848, 9024,  53140, 1896,  36268, 23060,
                41624, 65184, 21444, 10576, 54508, 36740, 22256, 2876,  56000,
                23976, 4928,
            },
            {
                36560, 53628, 41172, 3880,  61404, 34684, 2248,  62872, 20304,
                46888, 5836,  34260, 41832, 8676,  44944, 13128, 34004, 65440,
                10532, 30236, 18492, 63536, 46440, 56684, 39884, 3804,  13884,
                46104, 61572, 29068, 22032, 47896, 30492, 24812, 52240, 3836,
                22452, 34496, 56436, 6192,  62336, 41120, 34368, 896,   46772,
                52596, 40852, 27864, 61080, 48540, 13580, 39960, 1628,  28452,
                45396, 7052,  65224, 53112, 11264, 17868, 1688,  42376, 21208,
                25904, 53256, 49444, 57612, 62396, 15836, 6680,  41704, 24360,
                37300, 51260, 39576, 23012, 41580, 17144, 29000, 48048, 39220,
                53608, 11312, 45016, 63564, 16936, 30392, 25200, 19992, 35872,
                45984, 2872,  39376, 62284, 50628, 29392, 36416, 26092, 55276,
                59216, 29904, 20904, 35424, 2324,  65092, 46612, 20360, 51592,
                54536, 43616, 28664, 64684, 25916, 20424, 58124, 61800, 27756,
                15128, 56976, 38716, 324,   45076, 18132, 42084, 59576, 30400,
                39712, 61140,
            },
            {
                28216, 16044, 59744, 12100, 18572, 49424, 27244, 15400, 39544,
                24188, 50028, 54396, 21608, 30632, 55880, 3860,  51692, 20668,
                25300, 5456,  43572, 2424,  32732, 7624,  50616, 24172, 59172,
                33988, 17688, 51964, 40076, 58120, 420,   60068, 14164, 40784,
                50048, 60572, 43808, 37236, 16400, 9448,  48900, 13276, 37796,
                17620, 4628,  45532, 18840, 24948, 6596,  34500, 16880, 59900,
                55508, 31468, 20308, 34028, 58876, 24464, 30056, 50228, 63628,
                3744,  12376, 43976, 1176,  9252,  47876, 30252, 45604, 236,
                13988, 62016, 7252,  11968, 56516, 45432, 644,   57720, 14020,
                34976, 2300,  27788, 7888,  34476, 50428, 1020,  62992, 52316,
                56496, 8032,  14476, 19724, 43384, 6844,  63828, 46144, 484,
                33596, 5948,  45096, 15780, 53300, 30568, 5264,  42292, 24372,
                38916, 58748, 15532, 46728, 12552, 42584, 31604, 10400, 44684,
                3096,  32904, 50184, 25056, 28700, 63600, 8008,  11660, 51548,
                19304, 1332,
            },
            {
                50788, 22236, 6772,  33068, 38900, 53852, 10232, 45548, 57828,
                1324,  13232, 64532, 3112,  38736, 60640, 17216, 36836, 46316,
                57524, 49700, 60184, 22188, 54204, 27876, 42816, 10584, 37608,
                53880, 2196,  25852, 8024,  37100, 10904, 33080, 46852, 26772,
                8468,  29288, 1088,  24420, 51796, 30404, 59828, 21052, 28728,
                63452, 54976, 10616, 31980, 37528, 64308, 43492, 49524, 23328,
                4296,  40760, 47780, 27052, 8028,  37856, 55640, 33488, 9868,
                39732, 35180, 30660, 24212, 19272, 36352, 59404, 21292, 53488,
                57928, 31024, 19132, 26996, 52332, 36288, 10332, 24200, 63928,
                19384, 51068, 60492, 23264, 55324, 46796, 39752, 15716, 28924,
                22580, 33556, 25816, 48332, 57988, 16376, 22504, 39788, 13956,
                19180, 48712, 63540, 26252, 10864, 37696, 60276, 14748, 8292,
                31624, 5568,  35444, 1424,  52412, 38660, 6164,  48724, 19140,
                53824, 8152,  17112, 59100, 13640, 52720, 32180, 26436, 45624,
                14028, 33984,
            },
            {
                9472,  56552, 47844, 44172, 24408, 65268, 4284,  36436, 29316,
                42504, 32892, 26112, 18524, 48692, 11180, 28616, 42432, 964,
                32060, 16396, 38456, 12120, 35812, 15564, 62952, 19708, 29780,
                6340,  48368, 15108, 60800, 19572, 44680, 55256, 17088, 63532,
                20280, 54512, 12756, 65084, 19300, 4484,  54308, 44616, 7160,
                33372, 23764, 50624, 88,    55932, 12092, 20632, 53396, 9904,
                37180, 15248, 52244, 440,   62724, 41716, 13688, 5688,  57788,
                45912, 16240, 54560, 64292, 40840, 51080, 11428, 26536, 4408,
                44120, 35220, 48864, 2784,  65372, 20924, 31256, 43436, 4232,
                40680, 31660, 15216, 42780, 4736,  12908, 58860, 3704,  44072,
                10260, 64640, 53128, 35732, 2432,  31188, 9528,  51748, 61664,
                28468, 38560, 8416,  42540, 57224, 18540, 49628, 27804, 55448,
                62652, 21836, 49928, 61464, 24272, 17496, 56152, 64140, 25656,
                36844, 60636, 42808, 34784, 3864,  48324, 39084, 2144,  57132,
                64860, 42900,
            },
            {
                38148, 61920, 29012, 620,   14300, 31248, 21192, 55688, 16948,
                61768, 8344,  45164, 35600, 53056, 6888,  24144, 59104, 13744,
                9176,  26744, 52552, 4712,  61184, 48140, 84,    57320, 44808,
                22564, 65220, 41700, 31964, 50596, 23708, 6616,  36108, 2596,
                39512, 31620, 47276, 42592, 33604, 38968, 25212, 11796, 41420,
                57056, 15080, 39440, 59816, 28112, 46744, 3628,  30312, 61308,
                26144, 58264, 11604, 44744, 21892, 17140, 48000, 27980, 23420,
                19988, 60148, 2528,  27692, 5164,  14668, 61444, 33792, 40028,
                10068, 16476, 60932, 41132, 6124,  15140, 59260, 53828, 28432,
                47516, 6980,  57120, 37864, 19864, 32560, 26528, 36576, 49964,
                18372, 40768, 5388,  12492, 55036, 41588, 59880, 34692, 4488,
                55792, 23340, 51184, 1344,  32572, 22364, 4332,  34608, 47232,
                12208, 18292, 44428, 8448,  30332, 34660, 3584,  40416, 13116,
                29184, 21264, 47244, 10764, 23352, 57920, 18708, 35548, 21704,
                6008,  24904,
            },
            {
                3568,  16412, 20000, 52804, 40992, 58788, 7072,  48472, 11600,
                51196, 22732, 59280, 14808, 40944, 63844, 33188, 47592, 54128,
                62448, 40648, 45288, 30760, 24644, 9228,  32524, 38924, 12740,
                55080, 34676, 27588, 11948, 1692,  62420, 28656, 43224, 52396,
                58980, 15752, 5032,  8980,  49932, 58404, 1996,  62388, 48348,
                3252,  19876, 44000, 7736,  22376, 16528, 41332, 33852, 7312,
                42864, 19552, 29960, 35420, 60124, 51556, 3316,  64776, 37172,
                51920, 8052,  48904, 32800, 45072, 55736, 7540,  47588, 19952,
                56892, 29164, 24488, 46340, 32484, 38484, 49644, 8828,  17476,
                62380, 11888, 25520, 48780, 65140, 9076,  53492, 62000, 6992,
                31516, 58652, 45632, 23320, 28296, 47540, 20032, 25600, 44008,
                11644, 17000, 35324, 62456, 13740, 54136, 44176, 64056, 884,
                41056, 37440, 26508, 53412, 58508, 14636, 22456, 45436, 51284,
                1204,  5840,  63276, 30984, 40988, 7352,  62248, 15060, 54140,
                47220, 31704,
            },
            {
                12640, 49956, 34664, 63476, 9836,  45904, 25480, 33972, 40360,
                28244, 5696,  54844, 208,   27432, 21164, 4136,  17860, 35668,
                22716, 2976,  19776, 56148, 17192, 42032, 51732, 26300, 18380,
                3368,  8852,  52356, 45980, 57368, 39056, 13828, 21392, 10460,
                25560, 35044, 61912, 22800, 28036, 13572, 17572, 31496, 36104,
                26408, 64972, 30540, 35724, 48564, 63016, 57308, 13972, 50600,
                64752, 2396,  54468, 5460,  25296, 32184, 10184, 43680, 30916,
                14400, 41508, 12044, 21564, 38308, 17316, 29920, 23720, 64612,
                2112,  50368, 7992,  12164, 55696, 22940, 1388,  26472, 37500,
                34428, 21536, 52708, 256,   29372, 16524, 45756, 21200, 24708,
                13824, 37752, 532,   16916, 63956, 7452,  14932, 1480,  50084,
                31500, 58524, 46464, 6704,  29140, 39036, 10104, 24968, 16172,
                29852, 60392, 3244,  10936, 41576, 48496, 7436,  61560, 32612,
                58396, 38292, 52532, 14568, 55084, 25884, 44212, 29476, 10272,
                40568, 59760,
            },
            {
                55600, 43540, 22844, 5340,  37076, 18496, 52024, 62840, 3496,
                20080, 37748, 31648, 47060, 10300, 50540, 56600, 44096, 7900,
                29084, 50836, 64012, 7124,  37204, 59284, 4416,  63244, 47028,
                58572, 40304, 21224, 4836,  18056, 31068, 48208, 56412, 3792,
                44424, 50920, 18956, 56092, 37116, 45728, 53204, 21616, 8312,
                51696, 12748, 4380,  52884, 9996,  25748, 1680,  38332, 23452,
                46528, 16284, 39976, 49492, 13176, 39028, 56480, 18720, 1288,
                61456, 34736, 57668, 25116, 63268, 2820,  52084, 36588, 42248,
                15256, 54164, 35776, 59556, 18064, 44156, 63604, 13520, 56744,
                46088, 5560,  43568, 59192, 35488, 40872, 2452,  57632, 42848,
                55104, 29284, 61024, 51856, 32704, 39316, 56316, 36192, 63264,
                5932,  21116, 40640, 25372, 60360, 48580, 19996, 58428, 51352,
                6836,  55916, 33280, 20176, 63624, 35820, 27924, 18848, 10228,
                24592, 16796, 44596, 20400, 1668,  37000, 49532, 4500,  51852,
                17932, 27668,
            },
            {
                36620, 2044,  26156, 54460, 29932, 13512, 1400,  43900, 15288,
                56696, 64904, 42744, 18296, 61492, 39384, 30028, 12136, 38376,
                59472, 14936, 33960, 11412, 47680, 21760, 14064, 29256, 35984,
                24340, 15240, 60584, 37224, 25660, 63280, 7076,  33464, 61344,
                29100, 12172, 40228, 308,   64288, 5800,  42332, 60404, 38672,
                57964, 46188, 18644, 60704, 40732, 32160, 19736, 54688, 11116,
                28052, 32920, 57680, 21044, 7416,  61780, 46544, 23920, 54632,
                26952, 6468,  46216, 53196, 10028, 43320, 59492, 12260, 5884,
                31936, 21228, 39600, 26908, 5184,  52300, 30364, 40608, 2956,
                61192, 31984, 15448, 10544, 23220, 51996, 12444, 34056, 4576,
                48460, 10620, 20556, 42336, 24912, 9788,  48612, 29052, 42096,
                53964, 14648, 9436,  52064, 17336, 2652,  35996, 31664, 42496,
                22584, 48180, 13944, 45008, 24864, 240,   56816, 53268, 42536,
                49740, 34672, 8396,  28344, 65324, 11816, 59012, 33312, 23952,
                64156, 7812,
            },
        },
        {
            {
                42144, 7472,  55232, 50844, 33144, 2460,  36384, 18208, 30572,
                39044, 10704, 41440, 3592,  57508, 13116, 54752, 16964, 39400,
                46540, 14984, 34624, 25896, 17004, 42768, 37876, 3840,  24316,
                7656,  40928, 57128, 29100, 9320,  34984, 60716, 27764, 2100,
                15224, 60224, 31156, 27072, 45020, 32716, 22000, 11024, 7024,
                19516, 38868, 15580, 49792, 41952, 34364, 10672, 51716, 38784,
                8336,  14272, 42420, 59368, 18748, 54948, 23892, 59996, 49400,
                1416,  29488, 8176,  41076, 19464, 45044, 9932,  16384, 40120,
                61824, 768,   52312, 14016, 19868, 9692,  58952, 34476, 21672,
                48364, 12948, 59528, 54412, 46140, 37144, 20720, 13292, 6620,
                64960, 16652, 2628,  20400, 50316, 14216, 61020, 56412, 11228,
                5452,  19588, 13696, 55696, 10524, 21260, 62256, 48396, 27104,
                50692, 34116, 43376, 31120, 56636, 36388, 6492,  40656, 47372,
                13472, 30992, 10012, 36712, 18732, 46916, 59352, 30232, 21396,
                45508, 1100,
            },
            {
                24224, 64812, 46608, 4900,  25792, 41048, 57144, 47028, 65336,
                26876, 15092, 50440, 17844, 36640, 43788, 29156, 488,   26244,
                58372, 8280,  44564, 54540, 20960, 57824, 29276, 63900, 34024,
                47516, 11936, 20608, 44356, 64268, 16352, 54212, 21584, 46260,
                36992, 9964,  5140,  62720, 53304, 2488,  35724, 50428, 45916,
                28020, 32192, 64836, 22644, 56088, 3144,  25128, 20776, 44620,
                35756, 62260, 30404, 344,   10204, 37644, 7112,  15912, 25952,
                54536, 21480, 58896, 15392, 62680, 30592, 52556, 24732, 49764,
                5064,  21692, 35632, 56132, 39328, 30364, 45456, 25804, 8024,
                44316, 29960, 3152,  18652, 15192, 31684, 61148, 26128, 50752,
                22404, 41544, 55156, 36484, 8100,  34076, 42652, 17144, 51008,
                62484, 35844, 27396, 48000, 38228, 32452, 16404, 892,   36608,
                55048, 15196, 2612,  8468,  48068, 24828, 1532,  28868, 61100,
                54252, 24444, 57540, 63632, 27868, 3408,  35208, 10276, 49628,
                14356, 33276,
            },
            {
                19816, 10400, 31100, 21224, 61936, 15828, 7788,  23012, 4496,
                43536, 54984, 61920, 25784, 5848,  64256, 9148,  47668, 51436,
                36928, 29580, 64872, 5836,  32660, 10948, 51592, 44844, 18120,
                27688, 61472, 51276, 4992,  25848, 588,   42892, 12564, 57328,
                33292, 51700, 41284, 21004, 12908, 58580, 26072, 61720, 14212,
                54672, 924,   10296, 44480, 17592, 31228, 63436, 13088, 53828,
                27432, 5556,  49528, 39780, 64636, 52180, 28848, 46380, 39264,
                11216, 43956, 36132, 48560, 256,   56748, 33756, 7496,  64548,
                29908, 42492, 46384, 6712,  60916, 2576,  51156, 55560, 14844,
                39436, 56908, 35856, 49948, 41664, 10616, 53500, 3248,  46860,
                33676, 10236, 61776, 44692, 25688, 53892, 2240,  23148, 29136,
                45900, 372,   40428, 64220, 4088,  26000, 59632, 41004, 9740,
                19668, 44848, 60616, 39016, 13096, 59532, 51992, 11132, 21164,
                3772,  43116, 6420,  48300, 52296, 12976, 44200, 63104, 39000,
                3948,  53908,
            },
            {
                48544, 58564, 38772, 13520, 44388, 53792, 29308, 49140, 12832,
                32560, 20768, 1460,  34052, 22180, 39908, 59736, 20112, 11852,
                23140, 2600,  18640, 38496, 47460, 1576,  15268, 22484, 56032,
                2660,  35948, 14628, 58540, 33484, 40136, 49544, 30060, 7360,
                64144, 25312, 16240, 47752, 37676, 43168, 18364, 3720,  40496,
                23708, 48460, 36352, 58808, 6792,  39488, 47680, 59564, 2188,
                19028, 55756, 24564, 16656, 21664, 34628, 13456, 61076, 32972,
                17744, 65032, 6176,  25284, 12220, 21172, 42228, 37552, 11724,
                18476, 57936, 15404, 27472, 23220, 12456, 36780, 18924, 264,
                61300, 20968, 5392,  25236, 64344, 7324,  28552, 57316, 15456,
                38740, 444,   29696, 13112, 18452, 63228, 48352, 37312, 8948,
                58984, 15680, 23840, 8408,  18760, 54420, 50944, 6868,  24388,
                62696, 29176, 49588, 22728, 18284, 30432, 44296, 64800, 37840,
                33416, 15200, 39712, 20352, 32772, 56880, 22760, 7940,  25948,
                60488, 36156,
            },
            {
                16260, 1704,  27528, 52264, 3396,  37300, 18964, 35072, 60944,
                58200, 9464,  45844, 52872, 48712, 14652, 31024, 35344, 55752,
                61772, 42968, 49776, 59228, 24008, 41516, 62004, 7032,  39192,
                10228, 42344, 23292, 46612, 18372, 9848,  61888, 23644, 18832,
                43840, 4096,  29312, 55812, 6344,  9416,  31016, 56328, 34620,
                6056,  62596, 19892, 52052, 28300, 14880, 35480, 9116,  41792,
                33360, 46532, 11072, 58032, 43760, 2684,  48480, 5540,  57368,
                2288,  31088, 51056, 39964, 28896, 46880, 4280,  60004, 23740,
                48476, 32364, 9172,  63984, 47832, 41616, 28872, 63304, 32280,
                43204, 11348, 52632, 33768, 45500, 21836, 36744, 42812, 19108,
                59908, 52484, 23016, 57856, 40032, 4372,  11668, 31648, 20616,
                42060, 52996, 56964, 31160, 44320, 35336, 13860, 46264, 56528,
                33368, 12116, 5824,  53728, 34536, 41112, 7228,  26460, 17412,
                49980, 62252, 9244,  26772, 1136,  16012, 40764, 50512, 18496,
                28908, 43692,
            },
            {
                7100,  41704, 61352, 33016, 9284,  64064, 56296, 464,   24808,
                40716, 17096, 38012, 27780, 7896,  56932, 2096,  45380, 6208,
                15636, 31964, 8844,  13384, 27904, 53576, 34712, 30560, 48088,
                65172, 53888, 28644, 6668,  55320, 38236, 51912, 2724,  35892,
                48776, 12208, 59824, 34776, 23320, 64572, 49728, 45476, 16224,
                28920, 42876, 12280, 25300, 1660,  55508, 50268, 23700, 29604,
                15272, 60872, 36900, 6624,  31404, 62592, 24872, 20156, 41928,
                53196, 23084, 14340, 58572, 63028, 17852, 55132, 14732, 51584,
                1712,  39936, 54860, 34864, 3440,  16372, 53452, 6236,  22920,
                49524, 27572, 16136, 58048, 1700,  13692, 60620, 31996, 8244,
                25452, 48236, 5196,  32636, 46676, 27792, 56792, 51908, 65072,
                3756,  33988, 11812, 49268, 61432, 1616,  29512, 21132, 37936,
                2308,  42332, 27032, 63516, 3268,  57844, 14232, 54688, 240,
                44812, 29864, 53080, 58920, 37136, 65448, 30072, 55880, 4880,
                12100, 57288,
            },
            {
                24856, 19112, 49876, 22064, 46040, 26188, 11704, 43132, 51420,
                6108,  30504, 54432, 12496, 63556, 41904, 18036, 27152, 52604,
                40696, 25252, 55120, 36148, 4808,  17256, 59792, 12360, 25800,
                19228, 964,   32632, 60556, 12916, 31240, 27004, 14968, 58352,
                54004, 39340, 1340,  19056, 52792, 27684, 13212, 21324, 60424,
                9032,  57532, 32768, 40304, 63968, 44116, 20152, 4956,  62460,
                52024, 3284,  26948, 50168, 40540, 9264,  56052, 27992, 37472,
                7752,  45876, 35628, 9580,  3168,  32536, 44324, 27240, 35996,
                61136, 5976,  25084, 19628, 50696, 60076, 10292, 45312, 57620,
                37208, 8308,  62692, 30596, 39060, 49024, 53028, 3916,  45084,
                12396, 35636, 64408, 15576, 7952,  19924, 43032, 14472, 25460,
                38864, 6916,  27848, 17120, 22684, 39504, 10480, 64940, 17380,
                51756, 59260, 16124, 45448, 10156, 21360, 48528, 36212, 24048,
                57284, 4608,  22388, 46492, 11624, 43492, 2872,  34444, 46060,
                62308, 32148,
            },
            {
                9976,  35792, 56552, 14860, 5364,  30792, 38616, 15596, 21352,
                47604, 62144, 2476,  20300, 33324, 23836, 50616, 10508, 64540,
                21180, 3484,  45756, 62672, 50256, 21580, 45088, 3884,  52244,
                37476, 16048, 49988, 44112, 21540, 5384,  63488, 41864, 8928,
                21092, 31712, 62104, 46036, 42156, 4868,  39604, 528,   37004,
                50980, 3312,  47560, 16764, 7832,  34064, 11544, 56864, 38940,
                43532, 22052, 12704, 54368, 18932, 14632, 45028, 50528, 11584,
                63820, 19284, 26924, 54060, 49744, 38268, 6896,  10424, 19900,
                29620, 46056, 12168, 43644, 30576, 38620, 26412, 34120, 17964,
                3232,  40924, 47208, 19992, 6720,  24312, 17596, 27584, 61684,
                55504, 20312, 42072, 51140, 37864, 62148, 34368, 996,   49964,
                18268, 47296, 59364, 42556, 5280,  52176, 57548, 25844, 47920,
                7632,  31288, 23312, 36636, 51096, 32420, 61868, 28976, 10752,
                40160, 13600, 34072, 18224, 7200,  24616, 53960, 20296, 14596,
                38580, 52004,
            },
            {
                44724, 64988, 3652,  40412, 60068, 48800, 55064, 63292, 33800,
                10016, 25564, 43828, 59356, 37148, 4084,  60612, 29868, 37752,
                33852, 58684, 18304, 11188, 31296, 39632, 8568,  56692, 42164,
                23024, 62660, 9512,  39840, 57292, 47500, 17556, 33704, 45328,
                28316, 6632,  24752, 15424, 10552, 33080, 62476, 54864, 31256,
                26256, 18952, 54336, 23324, 60796, 29956, 47236, 26280, 17176,
                8508,  34596, 46116, 64452, 29748, 36420, 808,   32776, 16744,
                59680, 30216, 1396,  41600, 20952, 24636, 57784, 65496, 41836,
                52532, 16628, 62856, 56424, 7576,  22032, 1300,  64828, 13724,
                55244, 25024, 12004, 54188, 43596, 65500, 9488,  34152, 40140,
                1264,  30480, 9804,  25752, 3000,  53952, 23128, 10596, 61244,
                30752, 54812, 35796, 14168, 62556, 31720, 43920, 4368,  34684,
                40812, 12680, 56004, 1608,  39696, 16972, 5128,  43016, 19288,
                64448, 51324, 60324, 38544, 49020, 61344, 27856, 8836,  49452,
                22328, 16,
            },
            {
                26092, 13272, 28604, 33944, 17648, 23604, 7756,  2172,  28716,
                56952, 39916, 14360, 49860, 7492,  46680, 15396, 44380, 20,
                13732, 48264, 28748, 42672, 1724,  64388, 26912, 14840, 33520,
                58584, 6204,  27588, 35280, 3468,  25504, 53640, 200,   61160,
                12744, 48180, 55964, 37604, 51084, 58384, 17764, 23924, 46352,
                10972, 65492, 38436, 13328, 4180,  52704, 37240, 196,   63432,
                31456, 57536, 2152,  24700, 5600,  57980, 61764, 23908, 4204,
                39144, 47700, 57492, 15768, 61840, 12292, 30820, 47264, 564,
                22908, 33352, 2932,  36292, 14848, 58592, 47148, 42116, 50104,
                31532, 61744, 35076, 764,   29088, 37384, 56724, 50332, 15304,
                22832, 52268, 59432, 17756, 47900, 29612, 57500, 44936, 40964,
                8340,  21104, 3344,  24684, 8736,  37212, 19124, 14904, 54436,
                61276, 20104, 47332, 64236, 25608, 59700, 8668,  55884, 47092,
                2616,  26316, 5948,  30964, 56544, 16348, 35808, 41644, 63180,
                30696, 55464,
            },
            {
                37252, 48404, 53652, 43316, 10692, 52012, 45052, 19664, 36504,
                52568, 4540,  18724, 30904, 55912, 26644, 20824, 57216, 8380,
                53456, 39296, 5992,  55816, 23496, 51552, 36252, 48424, 2256,
                29876, 46308, 14244, 19512, 64688, 11392, 38032, 22952, 51304,
                36184, 18308, 64924, 1864,  26816, 7868,  48024, 14180, 40900,
                6656,  44368, 34304, 27524, 41892, 58604, 21952, 14276, 53864,
                49620, 19636, 41968, 15960, 38984, 47956, 20644, 52016, 43000,
                55144, 7064,  10648, 32988, 44656, 5504,  53520, 15052, 36692,
                9144,  54328, 48220, 27804, 40520, 52112, 11140, 19436, 28392,
                6316,  21296, 51712, 16728, 58704, 13128, 20860, 4336,  46392,
                63320, 36464, 6484,  43704, 33408, 12316, 5060,  16284, 36436,
                26704, 63748, 45520, 50680, 55704, 27360, 48876, 288,   24104,
                29880, 9704,  5608,  28564, 13540, 44640, 31140, 23436, 37380,
                32804, 15552, 42596, 21468, 812,   44960, 12440, 3340,  58112,
                5688,  16244,
            },
            {
                60444, 6568,  21052, 1252,  63740, 37620, 27084, 58980, 12536,
                46472, 62968, 23980, 35324, 11184, 52440, 39864, 32252, 62400,
                24632, 20196, 61532, 16112, 32728, 12060, 18668, 61092, 21036,
                10404, 51892, 40952, 55996, 50500, 31924, 43508, 15756, 57184,
                5872,  41536, 32676, 22220, 43980, 30208, 35760, 2776,  57248,
                21400, 59336, 1428,  51212, 9528,  17788, 45100, 6972,  40100,
                27900, 10512, 61360, 52880, 33524, 12136, 8596,  28544, 13560,
                34568, 25368, 21848, 37376, 49052, 27056, 18816, 40200, 59124,
                25528, 61260, 20192, 4828,  63828, 24052, 33008, 4432,  56588,
                39036, 9776,  45792, 41196, 5336,  47760, 26348, 42604, 32136,
                11036, 27800, 14372, 55432, 64612, 39728, 19744, 59948, 52900,
                1960,  13020, 33332, 17264, 39768, 11444, 64412, 57912, 45748,
                38220, 52204, 42728, 35136, 54092, 18360, 744,   52396, 12164,
                59344, 50008, 54712, 10084, 63808, 29420, 53248, 23256, 47192,
                39364, 19532,
            },
            {
                32932, 9368,  57456, 24620, 31128, 14688, 4276,  41808, 32412,
                16792, 8520,  57956, 42700, 1708,  65464, 5484,  17276, 43024,
                27964, 49636, 9864,  37260, 59028, 46568, 4284,  43420, 54808,
                37828, 62220, 23176, 1776,  28540, 8360,  59648, 4016,  29964,
                26408, 10184, 53108, 13524, 59676, 20048, 63468, 49812, 28136,
                52816, 31796, 14888, 62000, 48272, 30920, 34924, 60092, 24100,
                3444,  36812, 47168, 6588,  26492, 55732, 41248, 65204, 46704,
                18312, 60080, 52592, 64232, 2824,  56572, 34220, 3940,  50736,
                44164, 12580, 31748, 42712, 8888,  16800, 37532, 62096, 43852,
                15436, 60828, 33124, 23588, 63608, 31060, 8188,  60584, 54568,
                2388,  38664, 49500, 21240, 708,   28388, 47052, 24060, 49240,
                30128, 56928, 42724, 60492, 2564,  30692, 21256, 33880, 7068,
                13284, 62136, 22124, 3412,  58092, 48380, 41364, 63576, 20932,
                28320, 7444,  39720, 25016, 47972, 37548, 18084, 34048, 27172,
                11312, 42852,
            },
            {
                29148, 62424, 45256, 39992, 49920, 55572, 60996, 21440, 51092,
                552,   39148, 28548, 49036, 14056, 22632, 36740, 50572, 11592,
                3320,  34728, 44916, 1076,  25080, 52848, 8152,  31044, 26220,
                16824, 5104,  32392, 44536, 17356, 48132, 21248, 40084, 61904,
                49804, 45160, 3292,  38468, 54668, 5024,  11744, 39156, 17336,
                9880,  37444, 24408, 19888, 4896,  25972, 54872, 11988, 48820,
                64784, 14992, 30320, 22464, 59788, 17636, 2996,  31832, 6160,
                38080, 176,   29468, 16952, 8744,  42860, 62336, 21484, 29672,
                6768,  17404, 35764, 55456, 46156, 26240, 53396, 336,   22292,
                47848, 27136, 2208,  55012, 18212, 36364, 52416, 15924, 24084,
                19260, 57924, 42268, 25244, 9564,  60964, 34868, 6808,  11048,
                38512, 20388, 7664,  23504, 46404, 5732,  52776, 40984, 17972,
                26928, 49308, 32680, 16096, 37940, 24156, 9540,  34384, 4404,
                45468, 17368, 62192, 33152, 13780, 4256,  59980, 7692,  64348,
                54500, 50840,
            },
            {
                5212,  15744, 35652, 12068, 18508, 7232,  34736, 10304, 64032,
                25852, 54552, 20376, 61064, 33288, 45232, 56300, 29940, 59848,
                15788, 64184, 54996, 30088, 41448, 21544, 38956, 65272, 13460,
                48988, 59456, 35676, 12716, 63024, 36752, 54384, 11740, 34528,
                19564, 24152, 62864, 16488, 46976, 32332, 42636, 23480, 60580,
                5528,  47256, 43276, 56664, 39552, 63552, 1128,  43648, 20716,
                32964, 56824, 42384, 712,   44736, 36356, 49212, 21028, 57772,
                50872, 44444, 11896, 40756, 31764, 23796, 14104, 10328, 37928,
                64748, 48716, 58464, 2648,  13980, 59780, 49376, 29608, 35700,
                8056,  57756, 13584, 38444, 10624, 48792, 3996,  40680, 34096,
                45920, 61576, 4780,  32436, 51488, 44424, 15124, 54884, 41856,
                65328, 15872, 51628, 28688, 35848, 62296, 15500, 44048, 60356,
                1412,  56408, 11044, 65412, 7240,  27552, 60116, 14944, 49348,
                56944, 36444, 2432,  20004, 57420, 41164, 49736, 44124, 15028,
                988,   22680,
            },
            {
                48140, 26124, 52908, 2180,  28628, 46408, 23356, 48628, 30248,
                43880, 6232,  36180, 10128, 4296,  25716, 18888, 8088,  46584,
                39188, 20856, 6848,  13888, 60264, 17232, 56648, 33996, 308,
                41252, 9008,  25400, 57768, 6308,  27120, 880,   46284, 14832,
                6992,  58380, 35580, 29028, 8808,  25692, 51804, 360,   55092,
                29104, 64288, 33096, 8304,  13360, 36216, 16688, 29256, 7956,
                53636, 18180, 9636,  51360, 13632, 62848, 27812, 11084, 24364,
                15264, 62072, 26568, 55992, 47564, 51676, 59556, 45284, 54020,
                1172,  26880, 22544, 30508, 39556, 20556, 6932,  12252, 65184,
                18892, 50964, 42280, 29376, 59904, 21880, 27564, 64936, 12724,
                8484,  29356, 17520, 13268, 37636, 56836, 19008, 3880,  32104,
                25968, 544,   59360, 48432, 12692, 55872, 25152, 9584,  37008,
                30260, 45016, 19472, 41104, 46632, 31668, 53524, 40172, 23008,
                29976, 11088, 43460, 51356, 28064, 9436,  21272, 25480, 32176,
                58600, 36680,
            },
            {
                41772, 56872, 20132, 65304, 58640, 42524, 3800,  37216, 13500,
                18124, 58784, 46368, 15936, 51568, 63544, 41956, 2392,  53212,
                24336, 32600, 51188, 26972, 47276, 11016, 6008,  28268, 52368,
                22144, 45732, 53336, 20052, 50044, 42272, 22912, 65128, 31000,
                51516, 41732, 1424,  18880, 56468, 64768, 14948, 34428, 45216,
                12452, 18676, 1976,  53788, 22892, 51164, 46396, 59296, 40788,
                4860,  37984, 61820, 25096, 33688, 7188,  39928, 55280, 42584,
                33420, 3824,  36124, 19452, 6516,  1756,  35360, 25196, 18680,
                40872, 11716, 52876, 8516,  63456, 33908, 44628, 40832, 54372,
                32180, 24452, 3744,  46024, 7016,  53668, 43708, 156,   56688,
                50080, 36044, 53488, 64204, 2164,  23532, 29640, 62364, 46828,
                53276, 34340, 10072, 38152, 18744, 4252,  32896, 51064, 22848,
                5856,  54012, 25448, 2712,  50900, 12612, 5736,  18444, 1232,
                61072, 54216, 24412, 5960,  65208, 36068, 55596, 3136,  39732,
                19148, 10788,
            },
            {
                3688,  31452, 7904,  39224, 33000, 15440, 54940, 60432, 52528,
                2480,  40296, 24360, 30676, 57480, 38252, 28756, 13412, 35168,
                58468, 10540, 43104, 2184,  63016, 35804, 50240, 44176, 60596,
                18020, 30832, 2340,  39472, 15328, 33228, 56268, 9768,  39056,
                27296, 54196, 12512, 43992, 48912, 37400, 7344,  20124, 40340,
                58440, 26696, 49048, 41372, 60976, 27364, 10116, 33984, 24704,
                49832, 21752, 28804, 46356, 58148, 16720, 51972, 1548,  64076,
                8572,  48832, 22300, 60904, 38904, 16528, 30104, 7924,  57332,
                32460, 46336, 37056, 16708, 50400, 4124,  23800, 15676, 1820,
                60760, 37024, 16836, 63328, 33664, 14008, 19552, 31656, 38156,
                22240, 5660,  26236, 42836, 47992, 8708,  40460, 11880, 21264,
                6128,  44780, 22480, 42424, 63108, 28004, 45616, 58948, 14460,
                63352, 39184, 33496, 61040, 21008, 58260, 35160, 63928, 44420,
                37596, 14116, 32516, 47468, 16692, 31132, 12736, 61536, 46968,
                52040, 63592,
            },
            {
                44612, 13852, 23628, 50508, 10976, 25776, 20568, 8800,  27492,
                32784, 64860, 12284, 48092, 1096,  9124,  21516, 61632, 48700,
                5292,  18184, 55976, 38668, 30768, 19912, 24520, 3532,  14380,
                37708, 55512, 64372, 11360, 29152, 60000, 3044,  17864, 48268,
                4804,  21368, 59704, 31724, 3968,  22500, 61112, 30328, 50436,
                9120,  37028, 15908, 31304, 5840,  19024, 2652,  62940, 14296,
                57380, 1908,  12024, 41128, 4380,  31112, 23192, 37792, 18488,
                30264, 58904, 44740, 11300, 53368, 64444, 42136, 49984, 62604,
                15036, 5220,  61564, 42488, 28052, 55916, 59420, 46668, 26968,
                9084,  48924, 11632, 55616, 39564, 25440, 50888, 61788, 15776,
                47200, 10492, 58180, 19696, 33208, 59088, 50668, 35692, 60372,
                17472, 57640, 30704, 2892,  54320, 8624,  40436, 3300,  18936,
                47780, 12128, 8240,  16328, 29164, 42508, 23756, 48160, 27376,
                8236,  20896, 40384, 57852, 872,   44900, 53504, 27220, 8264,
                16264, 28440,
            },
            {
                58872, 53548, 35088, 47288, 1540,  61932, 40728, 45408, 49648,
                22256, 7252,  55740, 18520, 34240, 44000, 54236, 16668, 31036,
                45032, 65020, 22808, 14268, 7616,  54708, 40916, 62384, 9448,
                26784, 4648,  35164, 48120, 7348,  44772, 52148, 24760, 34032,
                62240, 15416, 36700, 26440, 10868, 41100, 53180, 2804,  24748,
                63200, 4964,  21992, 57036, 45144, 38628, 55168, 47796, 30740,
                43796, 36100, 53016, 64980, 20268, 48184, 9936,  45416, 53968,
                25924, 14396, 5416,  32900, 27604, 23424, 12684, 2992,  21644,
                28620, 54476, 24584, 740,   19200, 10160, 37908, 30940, 20404,
                52196, 43148, 22740, 28808, 1688,  58708, 9392,  4560,  41652,
                28144, 63348, 39160, 14364, 5100,  25264, 15976, 1140,  27820,
                39084, 13536, 50488, 25540, 16028, 48736, 21728, 34452, 56012,
                28484, 43236, 52808, 36488, 56512, 2012,  9760,  15360, 55804,
                3900,  51772, 61800, 10336, 19572, 39068, 23088, 4772,  37912,
                33832, 22024,
            },
            {
                40360, 6116,  62840, 16732, 29536, 56676, 5792,  35260, 16200,
                60968, 36596, 42432, 27712, 62740, 23728, 5908,  40424, 26008,
                496,   36956, 28524, 50608, 59444, 45584, 16596, 32984, 49384,
                58220, 43772, 23684, 17148, 62064, 20848, 37236, 12212, 57436,
                45176, 8400,  50412, 63940, 46508, 57124, 13572, 17100, 47352,
                33628, 43056, 51308, 13652, 64832, 34672, 11916, 20900, 6776,
                17644, 59760, 8552,  15168, 34860, 55852, 61212, 12984, 2928,
                36480, 40912, 57196, 51012, 8716,  43940, 56172, 34172, 39492,
                48512, 9572,  35552, 58780, 32936, 48868, 13224, 64336, 4444,
                34828, 57716, 5936,  18616, 48332, 36192, 44236, 23380, 54880,
                33876, 1404,  51572, 30180, 55032, 45788, 63652, 42612, 53804,
                47784, 7476,  64252, 37096, 59468, 31524, 11672, 65024, 37888,
                160,   23332, 62356, 5088,  45632, 39352, 50516, 31616, 59776,
                34564, 45780, 29048, 25508, 49184, 63048, 15188, 43328, 60300,
                48940, 380,
            },
            {
                12188, 26804, 19720, 43068, 37556, 51548, 13276, 30932, 120,
                53980, 10964, 4712,  52380, 14140, 49624, 58088, 11992, 60368,
                52660, 9604,  42400, 4188,  34204, 11720, 960,   29340, 38788,
                20220, 12568, 31708, 54424, 40272, 27576, 5032,  42736, 29044,
                128,   39596, 23904, 17712, 6508,  34580, 27784, 39080, 55528,
                11480, 60808, 3280,  28952, 8428,  24024, 42496, 27636, 52424,
                39180, 23692, 32748, 27144, 304,   42260, 24796, 29244, 59372,
                49220, 21552, 17308, 604,   36932, 19324, 47308, 5628,  60184,
                17956, 44372, 13712, 52124, 22376, 43520, 7072,  53784, 40196,
                14856, 61328, 32324, 41228, 64752, 13296, 30444, 17724, 59796,
                12264, 20872, 43812, 8532,  37332, 21960, 11112, 31184, 4856,
                23516, 33928, 10864, 20392, 1064,  42212, 52132, 26044, 7432,
                50536, 32076, 17456, 26828, 13248, 64420, 18740, 25104, 41428,
                12120, 17548, 6596,  36592, 2748,  33608, 55920, 29820, 10020,
                18600, 54972,
            },
            {
                30876, 45996, 59636, 3564,  9780,  22896, 64552, 18944, 43552,
                25212, 47040, 20808, 39240, 31576, 2920,  35476, 46780, 19860,
                33088, 16080, 62632, 20992, 26324, 48464, 64264, 22836, 52972,
                5520,  60984, 51116, 1196,  9888,  58688, 14932, 63336, 52832,
                19584, 32108, 54952, 3368,  30220, 60012, 43944, 704,   19748,
                31608, 23652, 37772, 18468, 53920, 49340, 784,   56712, 63972,
                4508,  49512, 45188, 61860, 51184, 38452, 7640,  19496, 33888,
                4768,  65364, 46220, 30436, 58308, 63316, 14064, 31168, 25904,
                1648,  65080, 29752, 40380, 2964,  62500, 17680, 24848, 29228,
                404,   46276, 26292, 8316,  54684, 3616,  52680, 46956, 7012,
                25680, 49536, 57364, 18192, 61228, 2556,  34732, 56828, 18980,
                61044, 44136, 27556, 55416, 46372, 5924,  17776, 57816, 14820,
                44980, 10632, 58800, 48012, 21548, 34180, 7220,  54340, 556,
                22108, 64816, 43036, 52956, 59036, 12412, 7052,  50960, 25352,
                64976, 36304,
            },
            {
                51644, 8020,  24168, 33424, 55276, 27996, 47728, 59080, 38292,
                57180, 29096, 63876, 16832, 59620, 9820,  26940, 38760, 6948,
                24264, 47620, 54288, 39588, 57960, 18428, 43244, 56204, 8540,
                41712, 16052, 45796, 25704, 35560, 46816, 32640, 22276, 9088,
                47988, 59040, 12448, 42200, 51596, 21416, 9592,  65276, 52432,
                7688,  57828, 46376, 40624, 59896, 15072, 32212, 36440, 9924,
                16368, 29972, 12720, 21464, 6120,  16924, 63788, 52496, 43336,
                9532,  26592, 53724, 11800, 23896, 40008, 7540,  54900, 36368,
                50640, 20832, 55760, 6048,  27088, 46204, 36376, 57256, 49344,
                19864, 11508, 51368, 22360, 16520, 34404, 27048, 38116, 62664,
                40944, 35608, 3428,  28764, 47652, 52468, 41572, 26600, 13884,
                50836, 2196,  40536, 14580, 62028, 34820, 29172, 39380, 61388,
                20816, 41328, 54744, 36060, 3620,  51964, 28912, 60992, 38300,
                48376, 56684, 31360, 15540, 23632, 46748, 20236, 41072, 3160,
                44060, 15600,
            },
            {
                57024, 39544, 63720, 49824, 15144, 40408, 7108,  3316,  12320,
                33172, 8104,  3812,  41964, 23068, 44636, 51816, 15308, 57192,
                61364, 2072,  30456, 6100,  10104, 36052, 3400,  14156, 33632,
                27648, 36912, 3932,  65368, 18900, 50348, 56276, 3176,  41220,
                16208, 34952, 26300, 61856, 37580, 14400, 25612, 46000, 36296,
                27500, 15996, 2000,  33840, 5864,  26036, 62120, 19596, 46712,
                40688, 59312, 2580,  55024, 35408, 47864, 31200, 13740, 56464,
                38944, 15656, 33160, 42368, 2676,  28592, 48752, 16508, 42576,
                11980, 8324,  38036, 60304, 15804, 11676, 31536, 8824,  42388,
                63692, 35156, 38576, 59016, 44832, 61444, 20580, 664,   10516,
                15708, 31368, 64160, 13100, 24644, 16292, 6384,  65232, 37440,
                8488,  58400, 30844, 53020, 22272, 9308,  49208, 23744, 5496,
                33620, 1920,  29564, 8688,  62040, 43740, 11320, 46096, 14776,
                27188, 9064,  3512,  40264, 28024, 63704, 37256, 31828, 58392,
                22368, 5696,
            },
            {
                28536, 11536, 18108, 912,   44100, 20688, 34872, 51312, 24116,
                15496, 53288, 49040, 35024, 55664, 772,   64664, 29884, 41480,
                12480, 36748, 44540, 14924, 52224, 28700, 59784, 24744, 47480,
                62020, 54964, 21940, 30724, 12268, 7224,  24592, 37032, 28512,
                64704, 1696,  46332, 7560,  18704, 48624, 58196, 5304,  41240,
                12572, 50432, 63892, 21792, 48308, 11772, 43976, 3848,  22828,
                51944, 33572, 25340, 43048, 58224, 10736, 23780, 45828, 1044,
                22192, 62436, 6976,  59668, 51976, 20176, 62316, 23048, 58560,
                31816, 46744, 24188, 33920, 49768, 53992, 59452, 23196, 5076,
                16004, 55184, 2140,  29116, 5288,  12196, 50276, 42772, 57752,
                23096, 53964, 43324, 9396,  39428, 33052, 55152, 44924, 21824,
                48288, 16768, 25020, 38052, 3888,  43764, 57040, 12740, 53832,
                46784, 64048, 15128, 24912, 40596, 17284, 23696, 32836, 4564,
                35840, 62100, 20576, 50708, 10532, 728,   17064, 53308, 13028,
                47268, 34764,
            },
            {
                20480, 48204, 37036, 58660, 30528, 10848, 63252, 45200, 60108,
                40936, 20016, 61944, 26344, 12892, 18708, 33824, 21596, 4320,
                50596, 26560, 22704, 64916, 46448, 20416, 38520, 50724, 11220,
                18340, 6876,  44596, 58336, 38608, 43092, 61548, 13528, 54544,
                50744, 23184, 11156, 56228, 31820, 2344,  33924, 22348, 61192,
                31096, 55228, 42988, 9484,  29236, 52768, 38872, 57984, 28368,
                13940, 62560, 9072,  18056, 39032, 27484, 5004,  60200, 35092,
                28284, 49480, 18448, 35720, 45884, 10352, 38584, 6464,  3488,
                52164, 63860, 17800, 672,   41028, 20572, 3228,  39004, 51640,
                32576, 25408, 10056, 48684, 40780, 32208, 55632, 28452, 6556,
                48408, 19408, 4820,  51280, 58764, 20724, 152,   11616, 27680,
                35068, 5428,  62684, 11780, 18792, 64504, 31800, 2980,  37064,
                27132, 18864, 51168, 37660, 58072, 1360,  53644, 59156, 49924,
                18012, 54692, 45136, 33092, 56236, 42516, 61028, 8148,  26004,
                63160, 1812,
            },
            {
                41832, 61304, 25836, 5432,  52812, 55888, 26632, 17152, 29952,
                1056,  36704, 10316, 30716, 46740, 7336,  54496, 48452, 9364,
                56348, 18148, 34044, 57476, 180,   32172, 7780,  63660, 2224,
                40876, 32792, 14444, 52388, 576,   29120, 18000, 45096, 4356,
                19972, 39164, 43988, 28904, 63664, 39756, 54804, 10476, 17768,
                3540,  25072, 19312, 36092, 59332, 15684, 31912, 8108,  48932,
                1144,  36988, 54164, 4260,  50300, 63192, 19820, 40848, 51616,
                11364, 44360, 4416,  24992, 14664, 29948, 53956, 34636, 27308,
                13936, 43696, 29184, 10444, 61628, 44912, 29828, 13604, 46648,
                60396, 43592, 21388, 63096, 15104, 23872, 18104, 65120, 36952,
                33372, 61004, 26364, 36020, 46228, 62468, 30976, 49756, 59608,
                42000, 56048, 32696, 51460, 47440, 39664, 25624, 15876, 60644,
                43440, 6796,  56304, 12232, 48252, 29928, 20820, 7148,  41844,
                25512, 11808, 38600, 6252,  24472, 29504, 48996, 35508, 40388,
                30708, 52304,
            },
            {
                16056, 7836,  33036, 13872, 22584, 38516, 2940,  8608,  48232,
                56672, 43808, 5108,  59064, 40444, 62452, 37196, 27924, 43064,
                63128, 39216, 6476,  11356, 41272, 16980, 54192, 43616, 29476,
                23772, 49496, 60240, 25980, 10428, 56904, 49260, 34392, 9588,
                31424, 60728, 13976, 4764,  50260, 15700, 26732, 51936, 44160,
                48020, 39024, 7036,  62912, 220,   23452, 56140, 18396, 65444,
                41608, 20632, 46420, 29648, 14756, 32832, 55540, 8272,  16776,
                30864, 58728, 39644, 56592, 64788, 48,    42004, 61400, 19428,
                39692, 57564, 6744,  55116, 25644, 35284, 8156,  65440, 27124,
                17932, 6660,  57568, 35916, 52588, 7820,  39404, 2668,  10980,
                45400, 13784, 1464,  29332, 7564,  14484, 17904, 38792, 3680,
                9684,  19716, 44800, 1620,  28000, 7808,  53152, 21344, 50068,
                10548, 32316, 22068, 3960,  34236, 65040, 10320, 36588, 60836,
                30516, 2936,  64272, 16412, 58700, 13744, 21588, 5020,  18848,
                10748, 56152,
            },
            {
                23848, 44720, 51208, 65384, 47088, 42692, 34168, 62180, 21932,
                13144, 25748, 51852, 22532, 14808, 2200,  24308, 17008, 13324,
                2912,  30772, 53036, 48856, 61648, 27044, 21996, 12956, 57528,
                37420, 4532,  19760, 35100, 41580, 21708, 5996,  62628, 26172,
                57600, 52536, 35772, 24712, 20664, 42068, 7320,  36600, 64580,
                13608, 57584, 32380, 49816, 40352, 44648, 4984,  34936, 25492,
                11376, 32416, 61080, 23376, 43824, 1348,  47408, 26028, 64196,
                2528,  53084, 20488, 8384,  32704, 46940, 16400, 24132, 49808,
                1948,  36204, 21956, 48240, 15280, 52288, 19444, 56608, 1436,
                37736, 11908, 30500, 3892,  44680, 59716, 47860, 25984, 52964,
                17124, 41324, 54928, 22140, 57396, 42560, 25700, 54096, 64000,
                23912, 29944, 15256, 60504, 36284, 13344, 59308, 41496, 428,
                28520, 63084, 40236, 45192, 26336, 16220, 43220, 47428, 13828,
                52548, 21356, 34840, 46588, 51868, 1716,  43884, 54080, 59988,
                46288, 3716,
            },
            {
                38720, 57320, 27700, 10024, 4512,  19492, 15120, 50656, 39452,
                32300, 64456, 17612, 34248, 56244, 45184, 51440, 32620, 60440,
                45932, 20752, 24924, 14676, 36508, 3560,  47216, 34148, 51532,
                16304, 61808, 8588,  47396, 64468, 31596, 15348, 39736, 46552,
                1208,  17484, 7852,  47548, 62324, 476,   59232, 30368, 23536,
                1900,  28480, 16928, 10412, 26792, 13264, 54232, 47344, 58668,
                16072, 51352, 6256,  9988,  58512, 35348, 12288, 37992, 42620,
                23724, 13988, 36648, 26848, 50604, 12032, 55904, 7448,  59148,
                11452, 46080, 30948, 63236, 38396, 4952,  42668, 33164, 47508,
                54132, 41192, 50636, 18972, 27316, 13252, 32820, 21432, 57992,
                30548, 62276, 48760, 37348, 10588, 50968, 5664,  33576, 12868,
                46644, 52800, 40088, 6392,  22672, 55424, 33348, 18040, 46444,
                35824, 53656, 13680, 59020, 50820, 5716,  56968, 23828, 116,
                40004, 56084, 27012, 7568,  39704, 32060, 63564, 37520, 25276,
                13280, 34080,
            },
            {
                20804, 216,   16976, 36736, 31604, 60824, 57764, 27600, 6552,
                54452, 2464,  47044, 7744,  28956, 38612, 10224, 5372,  54044,
                36076, 58620, 8620,  44252, 30028, 55204, 59492, 9648,  1048,
                26588, 44932, 54524, 13308, 2280,  51688, 55604, 23824, 12712,
                42588, 30148, 38704, 55440, 32672, 13076, 45976, 18896, 9272,
                35404, 54408, 45668, 60548, 51880, 20264, 30184, 38148, 1856,
                43332, 28096, 55140, 40416, 17704, 52480, 21232, 59832, 6488,
                31836, 48116, 60640, 4000,  40664, 22188, 29524, 43296, 33432,
                26668, 53340, 18332, 2664,  12416, 58680, 24432, 10384, 21496,
                15060, 64020, 24232, 34504, 54944, 532,   63480, 43272, 5272,
                8664,  24328, 4248,  18776, 31692, 63040, 45376, 20628, 36964,
                888,   58364, 26152, 65064, 43588, 48728, 9940,  4980,  61144,
                24020, 7484,  19672, 2124,  38356, 31844, 18688, 28680, 62588,
                9136,  48520, 18824, 60364, 11056, 23284, 16988, 8768,  28496,
                49956, 64016,
            },
            {
                30896, 60208, 40604, 54016, 48468, 24960, 1176,  45080, 10960,
                19952, 36944, 42080, 12248, 58484, 20252, 65468, 41884, 18624,
                27304, 1152,  50796, 64228, 18692, 6212,  38896, 20332, 65088,
                41380, 30360, 22268, 39064, 27916, 18596, 36888, 4220,  49828,
                63920, 21628, 58908, 10456, 27796, 53260, 37960, 56604, 49472,
                62584, 20620, 41556, 2956,  33340, 6804,  62360, 9348,  22564,
                63400, 3924,  37388, 25068, 64512, 3076,  29144, 49656, 55324,
                18532, 44028, 10120, 54400, 17652, 61660, 37152, 4792,  63916,
                15516, 8396,  60132, 41632, 27336, 31856, 49652, 62208, 36568,
                29200, 2860,  8212,  60568, 42056, 10440, 36512, 15076, 50992,
                39300, 34548, 12996, 58316, 40104, 2704,  27440, 60168, 16740,
                49580, 8328,  34908, 18628, 2900,  30760, 26860, 39112, 56668,
                15612, 30952, 48672, 24932, 61468, 8560,  52140, 35600, 44756,
                15260, 32508, 4136,  42824, 29360, 56484, 45812, 2592,  58436,
                41680, 7088,
            },
            {
                47712, 11736, 43792, 21556, 7488,  12880, 35296, 41068, 61340,
                29796, 48960, 62540, 26756, 49944, 316,   31316, 23888, 47704,
                11996, 40824, 32840, 23224, 11152, 43156, 25120, 32196, 46604,
                11708, 4996,  34488, 59856, 7400,  43856, 61316, 9748,  28364,
                34808, 6080,  15816, 43860, 3796,  23036, 16456, 5884,  42916,
                26268, 4936,  14520, 23900, 36656, 56456, 42176, 14280, 50876,
                33712, 19612, 49084, 13688, 32092, 45720, 9016,  15112, 34760,
                956,   63716, 28208, 33780, 45108, 2284,  51824, 19856, 47868,
                40036, 23300, 34464, 45288, 51768, 17132, 436,   44428, 6356,
                58300, 49068, 45464, 16936, 22548, 50128, 29456, 19316, 56504,
                27008, 64068, 46912, 53168, 22956, 15508, 9344,  55668, 41564,
                22156, 31728, 11580, 50548, 14468, 61616, 20684, 52324, 12076,
                42472, 64756, 36996, 44220, 54840, 12608, 41108, 4732,  58096,
                22356, 37836, 65472, 50780, 14568, 36788, 49156, 33492, 19940,
                52884, 15452,
            },
            {
                55648, 26040, 2508,  63312, 29012, 56840, 51340, 17452, 55956,
                24192, 14684, 4104,  22120, 35548, 16236, 55684, 7196,  61972,
                15548, 56292, 4644,  36852, 53512, 61368, 14404, 50180, 58396,
                17700, 52064, 56248, 15532, 47000, 25268, 33148, 57928, 19648,
                53748, 47304, 26228, 51152, 65348, 41156, 34008, 61284, 12184,
                31408, 39360, 53468, 64820, 47936, 17580, 26232, 46428, 29684,
                57004, 11108, 44600, 6944,  56100, 22712, 41824, 61204, 26452,
                40252, 12540, 22768, 7332,  57268, 14540, 26248, 10640, 57796,
                30468, 55420, 6424,  20840, 64516, 9340,  39512, 53172, 18204,
                25908, 33004, 13436, 38492, 57384, 6832,  46092, 61188, 3740,
                44528, 17468, 860,   28780, 43364, 35444, 47896, 30064, 5348,
                62340, 44336, 55040, 39464, 57592, 36552, 45368, 1312,  33856,
                50232, 4640,  10364, 28456, 16324, 21592, 63708, 26524, 49412,
                11684, 54240, 25472, 948,   21436, 6400,  62432, 12432, 24372,
                4152,  36360,
            },
            {
                8660,  19144, 35020, 49896, 15528, 38732, 23020, 8728,  2624,
                34000, 52100, 9460,  45556, 59800, 39680, 51656, 44496, 34612,
                28624, 49328, 59204, 17220, 47064, 1508,  29004, 3712,  36316,
                8296,  24548, 40328, 368,   63816, 12000, 51284, 2500,  40976,
                13380, 59844, 1336,  36544, 19272, 8228,  29220, 52560, 2428,
                46888, 58740, 8612,  29008, 11732, 636,   61376, 5132,  39868,
                16752, 60192, 35888, 27280, 61908, 38460, 51720, 4492,  19784,
                58748, 49320, 53060, 38060, 30688, 64932, 46544, 34888, 1524,
                13576, 49504, 3212,  36776, 14412, 56264, 23788, 30568, 12028,
                55332, 40440, 62812, 28172, 2056,  32328, 24776, 37952, 9336,
                22232, 32612, 11276, 59624, 6156,  65312, 51512, 12752, 37892,
                24656, 2128,  28032, 20088, 5228,  9080,  23752, 63176, 27752,
                18196, 22616, 57688, 984,   34800, 46732, 32220, 2352,  18004,
                30700, 7612,  45624, 34132, 59048, 52348, 27128, 40676, 44512,
                60632, 29692,
            },
            {
                65004, 46144, 57868, 31956, 3576,  61900, 46792, 31272, 43936,
                64880, 38260, 54728, 30156, 5644,  13408, 25884, 10500, 1824,
                38700, 20408, 8536,  31344, 25932, 39776, 55856, 21884, 42740,
                63324, 28152, 48640, 31732, 21408, 37788, 16852, 30204, 45436,
                23344, 39012, 31320, 11564, 45776, 57880, 49092, 20732, 24840,
                16100, 34960, 19020, 50356, 38660, 32124, 21192, 52080, 8232,
                24516, 2740,  53412, 20516, 124,   17024, 11516, 31204, 46848,
                36244, 3048,  16380, 42880, 20544, 5580,  39764, 22400, 62688,
                42720, 25048, 61012, 28800, 47100, 33508, 4908,  61132, 42964,
                20956, 9060,  4756,  51260, 18096, 65140, 54824, 14080, 52272,
                41668, 55768, 48952, 38408, 20792, 25516, 1488,  19000, 58284,
                52944, 15896, 63836, 46856, 32248, 53460, 16340, 41708, 7316,
                53996, 45988, 40484, 60700, 51636, 6628,  56356, 43196, 59816,
                39324, 62016, 16392, 41756, 10004, 18292, 31428, 1528,  54580,
                17228, 38816,
            },
            {
                51380, 13968, 23984, 6036,  42580, 18728, 11828, 58312, 6448,
                19712, 27520, 16660, 42804, 62160, 33316, 19164, 64212, 54140,
                22904, 60684, 45072, 12852, 51856, 7572,  62380, 33168, 12036,
                54100, 19600, 4236,  57472, 8784,  42788, 54888, 5480,  64496,
                8480,  17608, 61580, 54388, 24304, 3036,  13568, 37784, 55780,
                63996, 44592, 6260,  22980, 43340, 55520, 59136, 35380, 45116,
                63992, 30828, 42020, 9972,  50056, 33960, 65296, 23472, 54892,
                8848,  29492, 24676, 60876, 11012, 49712, 54308, 28100, 8128,
                18432, 53716, 38840, 7260,  58868, 19548, 50220, 37624, 1796,
                47672, 59460, 24588, 35688, 41356, 11660, 47804, 34856, 2496,
                27948, 63216, 7460,  13552, 30892, 56800, 40192, 33320, 45696,
                7372,  35400, 10568, 41144, 25708, 59380, 48348, 29844, 58568,
                36160, 14328, 31412, 25156, 19180, 38412, 14780, 10564, 24076,
                52732, 4448,  28348, 56760, 47648, 38560, 64316, 8444,  48440,
                22724, 5036,
            },
            {
                26888, 40348, 10112, 54292, 36968, 27896, 53384, 25488, 39836,
                48136, 11460, 1148,  56644, 21308, 47940, 4392,  42296, 30604,
                13912, 3120,  40960, 64992, 35524, 19832, 15900, 46168, 5460,
                38308, 14512, 45208, 34920, 60436, 23920, 49096, 27160, 56632,
                33660, 51904, 5944,  28428, 35036, 62880, 43472, 31812, 9920,
                920,   27664, 54376, 62236, 3356,  15504, 10448, 28260, 19168,
                13080, 37896, 47504, 58956, 28840, 44264, 57392, 5444,  42148,
                13924, 63076, 45952, 56068, 508,   32480, 15448, 58720, 36008,
                45664, 32276, 16204, 10764, 43948, 26528, 13172, 64188, 28376,
                34260, 16248, 54284, 45704, 21900, 7932,  29688, 20440, 59284,
                15472, 24504, 35224, 53324, 46480, 16808, 10052, 61332, 28412,
                21676, 49668, 60272, 4048,  13636, 37392, 2272,  11380, 21340,
                3908,  62764, 8252,  11824, 49272, 64480, 29516, 34156, 48412,
                20288, 36340, 12332, 22132, 3748,  25112, 14488, 35696, 57208,
                11400, 33564,
            },
            {
                840,   44760, 59628, 21336, 63944, 49692, 348,   14584, 34892,
                59512, 50948, 24268, 36416, 8204,  27960, 58616, 50764, 37328,
                46864, 56752, 28072, 6004,  24056, 43676, 58976, 26468, 49908,
                60844, 29424, 10256, 51472, 17656, 1444,  14152, 36208, 11196,
                20996, 43672, 48428, 15288, 40792, 18480, 5096,  58532, 48196,
                40092, 33056, 12856, 36960, 25872, 47540, 41208, 1544,  49596,
                54784, 4456,  16064, 22424, 6784,  14288, 25864, 38008, 18436,
                51600, 33896, 6692,  27344, 37088, 19232, 41988, 3944,  11928,
                51456, 824,   65268, 22640, 40524, 2828,  55908, 17672, 7168,
                52640, 10956, 30296, 24,    60400, 56720, 43752, 5296,  50524,
                39860, 45152, 19420, 3852,  62284, 42100, 50808, 3188,  14240,
                43008, 30916, 17812, 52152, 22456, 44624, 64216, 33212, 50072,
                39232, 26796, 56736, 42504, 3308,  22324, 53708, 760,   8080,
                44544, 63548, 50856, 33264, 60776, 54272, 44716, 20492, 28652,
                42448, 61000,
            },
            {
                19372, 30092, 35376, 16376, 7244,  32500, 44608, 61540, 22040,
                5344,  31848, 63912, 40244, 53100, 12444, 35080, 16184, 8896,
                25240, 18228, 33548, 48840, 12,    54724, 9948,  31472, 1828,
                22976, 41780, 65288, 25428, 31380, 40640, 62796, 46128, 4108,
                39748, 25876, 8,     57064, 9412,  22992, 52308, 26112, 15132,
                21632, 51872, 17892, 59728, 8300,  31308, 65192, 23560, 57612,
                33412, 26312, 60792, 35312, 40752, 53944, 48536, 1380,  30788,
                59440, 21852, 40420, 12640, 51224, 60100, 46920, 24844, 62092,
                21076, 29548, 56948, 47856, 53316, 35148, 30660, 45420, 22352,
                39432, 43448, 63724, 14160, 38204, 25060, 16512, 64124, 31752,
                10436, 644,   59960, 28992, 23368, 7212,  35884, 26424, 54776,
                64400, 512,   39176, 56256, 29128, 8580,  19240, 54768, 15204,
                46656, 52784, 17820, 35396, 45452, 27416, 60912, 41160, 58128,
                26352, 15692, 1872,  42900, 29484, 7364,  50352, 2412,  62780,
                15420, 52896,
            },
            {
                49028, 56308, 3288,  47224, 24464, 10580, 40948, 17360, 55724,
                9844,  43680, 14348, 18412, 46244, 1956,  61200, 22340, 3804,
                63264, 52136, 11404, 58164, 37720, 14220, 40316, 52436, 36128,
                16676, 55264, 3640,  37428, 6112,  53824, 19728, 28964, 57932,
                60960, 13104, 32044, 64896, 45000, 37464, 60296, 30424, 7132,
                63012, 42112, 4420,  49336, 20604, 53672, 38524, 17160, 6096,
                42548, 11060, 51424, 2800,  63932, 9196,  20972, 61708, 11212,
                43604, 2836,  17036, 64352, 23260, 8616,  30132, 55268, 40072,
                5120,  37304, 13764, 25808, 5708,  62976, 10196, 49548, 58480,
                25724, 5400,  19592, 50012, 32968, 3172,  53072, 36324, 26572,
                57168, 48620, 37292, 12172, 32844, 56040, 20700, 47792, 37724,
                11656, 24508, 48784, 5724,  34680, 61380, 26476, 42320, 6700,
                23820, 92,    32152, 59524, 5740,  16256, 36768, 12980, 19004,
                31780, 54576, 9724,  37728, 18528, 13192, 39796, 32008, 24204,
                37628, 7480,
            },
            {
                25888, 14332, 62184, 39288, 52036, 57408, 29508, 2504,  47392,
                38000, 28572, 3636,  58212, 30564, 25632, 44180, 39376, 55860,
                29632, 41796, 6852,  22004, 63684, 28584, 19280, 59812, 7276,
                48472, 20468, 46300, 13168, 59472, 43736, 7944,  50312, 34436,
                17924, 53400, 21700, 50216, 29324, 3148,  12012, 47012, 35420,
                55360, 10528, 28608, 45060, 34604, 3768,  12276, 46152, 29780,
                63080, 19508, 44988, 29248, 17068, 32232, 36548, 45864, 28076,
                55340, 49680, 35252, 53972, 44464, 1952,  14256, 33904, 17408,
                50392, 44944, 8424,  33488, 16592, 20624, 38192, 1108,  14912,
                61460, 35532, 54852, 27324, 9240,  48056, 41800, 13000, 7192,
                22100, 17508, 52328, 43568, 64840, 15232, 4312,  59872, 18272,
                8116,  33244, 58184, 15812, 46044, 39888, 1752,  57544, 30628,
                37740, 64716, 13256, 21724, 55368, 9816,  46548, 51200, 4684,
                39256, 47520, 59288, 24120, 64052, 46132, 55940, 4636,  58744,
                46452, 10740,
            },
            {
                40736, 32756, 5916,  28188, 12944, 19204, 65396, 34224, 25892,
                53592, 62392, 23112, 48852, 7580,  54844, 32872, 18912, 49304,
                13508, 34668, 45444, 16736, 32120, 46900, 4584,  25576, 62992,
                9712,  34100, 57660, 27628, 22396, 33196, 15492, 24652, 2108,
                9788,  42476, 36300, 7592,  16908, 55056, 20324, 43108, 1808,
                24252, 38420, 56984, 14184, 62108, 26916, 58152, 50188, 36472,
                22620, 7972,  38968, 56360, 23948, 50360, 58524, 4112,  15556,
                24560, 9004,  31488, 5052,  28304, 38676, 48948, 63960, 10416,
                27248, 61424, 58244, 41300, 51248, 60336, 24576, 42092, 32264,
                8964,  46684, 3656,  40784, 62616, 22872, 18920, 55068, 45992,
                62960, 40508, 4916,  24784, 8752,  46296, 40324, 31176, 51812,
                44880, 61704, 27692, 20888, 10216, 53272, 13440, 18320, 60460,
                10880, 44076, 51348, 40692, 29800, 62788, 32976, 24496, 64908,
                28568, 21108, 5820,  33796, 51524, 27244, 9328,  17332, 35016,
                20076, 64720,
            },
            {
                51540, 43848, 21148, 58716, 45388, 37180, 4672,  49604, 7912,
                19824, 11292, 33872, 41148, 15772, 64996, 10756, 5508,  62164,
                1000,  26100, 59900, 53932, 2260,  50792, 41292, 13244, 44216,
                29920, 39500, 524,   50984, 10556, 64368, 48508, 55876, 39292,
                63512, 46736, 4660,  27392, 61676, 40284, 32160, 63672, 16104,
                48540, 19380, 32344, 252,   23240, 42956, 8908,  15724, 1368,
                52780, 59508, 14436, 47904, 560,   12516, 41848, 19788, 64864,
                39220, 57896, 42328, 19388, 61152, 56288, 21608, 6104,  43484,
                23228, 1732,  19216, 30408, 4464,  45672, 12084, 57388, 53216,
                28924, 21812, 16060, 57556, 11700, 30236, 60496, 1472,  28252,
                33992, 14588, 30656, 58404, 35528, 53508, 28088, 1248,  23268,
                13556, 41500, 2620,  38164, 65340, 23972, 50504, 33868, 47476,
                20472, 8076,  26976, 2584,  48984, 18568, 7524,  43284, 2848,
                55460, 11768, 44096, 15064, 724,   21936, 61524, 43464, 53984,
                30632, 1160,
            },
            {
                55824, 16872, 35728, 2300,  9316,  54204, 22788, 42040, 14032,
                44584, 59200, 50892, 460,   21156, 36624, 47108, 24300, 42376,
                36976, 20260, 8724,  39332, 24208, 10724, 57188, 33224, 54304,
                23484, 15220, 61884, 18080, 41996, 37804, 3504,  27872, 12588,
                30308, 22792, 59552, 14236, 48164, 24596, 8876,  51364, 28016,
                59324, 6696,  64260, 50940, 39996, 54512, 19916, 60900, 32296,
                41468, 25548, 5284,  34784, 61848, 27020, 8000,  51980, 33624,
                5776,  13420, 48260, 26164, 10904, 15868, 36272, 31188, 52332,
                56000, 35184, 48328, 9956,  54716, 27740, 35780, 3432,  18664,
                44020, 65196, 51000, 33656, 38328, 6404,  44272, 35832, 51232,
                10176, 61104, 2788,  49444, 21404, 17000, 11200, 63872, 57012,
                34956, 6584,  54332, 48488, 31908, 3792,  43248, 5928,  28464,
                55060, 35504, 58544, 14696, 38744, 23064, 52988, 58248, 17020,
                35448, 49856, 62044, 40724, 57148, 36892, 29368, 12404, 49852,
                6672,  23100,
            },
            {
                11412, 62804, 48888, 24884, 60740, 32016, 16188, 61768, 56232,
                30812, 6324,  38640, 26920, 57624, 52864, 29352, 14620, 54180,
                57972, 50176, 30116, 61912, 15672, 36564, 64200, 20796, 2944,
                49872, 5972,  44928, 55612, 30796, 6544,  20736, 44504, 57512,
                51748, 18408, 38488, 54036, 780,   34616, 58136, 4208,  12924,
                41116, 34912, 10044, 16844, 30668, 5188,  35392, 27704, 47260,
                11628, 55484, 30556, 18852, 44384, 38044, 56656, 29936, 45560,
                22812, 36112, 62600, 44,    51500, 45708, 59584, 3252,  40880,
                12336, 15980, 38748, 64808, 21280, 15044, 61756, 50324, 6964,
                37608, 12952, 864,   24920, 53680, 17428, 58304, 4552,  24352,
                20592, 42544, 55576, 27320, 6708,  60676, 38232, 42844, 19768,
                50088, 30256, 15484, 18868, 26064, 56520, 37116, 15236, 63612,
                1272,  24524, 45980, 62140, 5260,  34492, 12488, 41064, 30004,
                8912,  26076, 18288, 31268, 8276,  48100, 4024,  40040, 15736,
                26680, 38828,
            },
            {
                31556, 7860,  29060, 13812, 41268, 51292, 27332, 1612,  35704,
                24696, 17116, 62980, 12768, 42744, 8984,  3708,  32476, 17908,
                6536,  12064, 46800, 4040,  43888, 27768, 6960,  17364, 37780,
                58880, 27236, 35104, 8868,  24864, 53660, 60672, 16868, 34556,
                1580,  8192,  32732, 43424, 11076, 19640, 45728, 38048, 22612,
                53768, 44752, 26372, 47624, 59092, 12888, 44432, 64792, 3420,
                21124, 37652, 63752, 9312,  53280, 22160, 16588, 2116,  11440,
                58924, 53772, 16896, 30244, 40256, 7872,  24876, 19792, 28080,
                62748, 6824,  24628, 43804, 296,   32656, 40808, 22860, 26484,
                56388, 31376, 41484, 8096,  46088, 27252, 14052, 39568, 64956,
                47748, 18052, 36332, 12484, 45344, 32004, 48420, 5552,  26272,
                9552,  62984, 46316, 58936, 7916,  11956, 61588, 22284, 40648,
                52172, 17312, 42036, 30340, 54588, 26516, 61372, 596,   48484,
                37808, 58724, 2532,  53256, 24552, 65476, 20292, 55080, 60564,
                45320, 58084,
            },
            {
                3704,  46628, 53604, 37572, 18644, 6592,  47032, 10800, 40132,
                52984, 4400,  46048, 33112, 19304, 49388, 63464, 44820, 60416,
                27292, 41056, 33476, 22604, 55644, 48744, 31612, 52656, 41820,
                12140, 46936, 20008, 63760, 14036, 36516, 47948, 10916, 40748,
                62172, 25092, 49848, 28596, 64644, 56208, 15608, 30176, 62228,
                5376,  20212, 56560, 2596,  21872, 38596, 52976, 17440, 7888,
                51392, 43240, 15640, 49448, 4576,  33192, 62772, 49088, 39760,
                27876, 3388,  42948, 21696, 34164, 65480, 13704, 48656, 58140,
                45236, 33400, 53616, 29500, 49280, 58848, 8856,  47072, 63188,
                16440, 48916, 60072, 19776, 63568, 49940, 32876, 56320, 7732,
                31312, 232,   52636, 39924, 23944, 56344, 1852,  14988, 54912,
                37452, 22968, 312,   39800, 35096, 44664, 29196, 47932, 9168,
                32824, 12576, 4004,  20080, 10388, 50248, 44260, 21420, 14548,
                64176, 22576, 45224, 13352, 34768, 42308, 10308, 32856, 1604,
                36108, 19140,
            },
            {
                42012, 22044, 57340, 276,   63528, 34704, 58020, 20424, 64364,
                48496, 21964, 28940, 59604, 1840,  25292, 39392, 21328, 36116,
                1372,  51984, 65504, 18988, 13484, 584,   60112, 9660,  24956,
                61520, 1520,  32236, 40452, 51952, 2832,  21892, 26656, 5224,
                46104, 15344, 58944, 21384, 6652,  40608, 26056, 2160,  50108,
                33220, 14812, 37480, 29388, 61780, 9360,  25420, 57372, 34108,
                23876, 59880, 1748,  26620, 58100, 41628, 13824, 24392, 60708,
                20020, 9424,  47076, 57068, 4480,  53008, 37812, 31696, 1476,
                9664,  17244, 60892, 5560,  13288, 37164, 18140, 2328,  34716,
                11020, 4408,  23592, 35588, 10376, 1836,  22056, 43476, 11892,
                26596, 59748, 15632, 64008, 9260,  18472, 61852, 34324, 29328,
                59400, 43188, 12788, 51776, 20456, 54656, 2236,  18280, 59768,
                38552, 57728, 47036, 65156, 36860, 7772,  17516, 55736, 33100,
                4184,  51988, 39480, 5828,  56200, 16860, 27892, 52292, 23576,
                13432, 64208,
            },
            {
                34004, 15648, 26996, 44520, 12368, 23976, 42888, 30136, 8312,
                13644, 37460, 55820, 10536, 35432, 53408, 13196, 7456,  56776,
                16504, 24488, 8960,  58452, 35384, 39988, 45228, 21332, 29808,
                50832, 15732, 57572, 5544,  29016, 59100, 42748, 65508, 30624,
                55020, 35200, 12204, 3764,  52544, 35968, 47328, 59752, 10928,
                41700, 65356, 7536,  49696, 42552, 33012, 696,   48280, 40364,
                13332, 29680, 35980, 20188, 10760, 30348, 46500, 6188,  35556,
                52696, 32496, 63392, 14508, 27184, 10600, 18500, 54988, 42108,
                22492, 51300, 39388, 20432, 26932, 56144, 52200, 30292, 45140,
                53752, 28444, 43632, 55580, 40184, 29676, 61060, 16644, 37832,
                50328, 44412, 34104, 5868,  29900, 42256, 51464, 21688, 46052,
                3500,  17560, 32568, 27368, 60924, 6380,  32260, 50732, 25320,
                6956,  27776, 52824, 22696, 31984, 60196, 39856, 28516, 47360,
                10432, 26564, 19136, 29668, 59724, 46564, 6564,  62448, 48500,
                8648,  51412,
            },
            {
                4744,  59924, 9352,  31940, 49608, 5368,  54764, 16116, 33892,
                60816, 3040,  41008, 50292, 16140, 61140, 28400, 46256, 31336,
                48908, 38096, 43060, 5264,  26332, 53928, 16624, 64424, 3752,
                35960, 43684, 23832, 48968, 11792, 17740, 33280, 50672, 8452,
                19060, 39168, 48004, 42816, 31484, 17872, 8164,  23432, 55476,
                18492, 46176, 24236, 12528, 52276, 19308, 14740, 28116, 61116,
                5412,  44748, 55900, 48016, 38948, 64764, 608,   55428, 43468,
                17616, 6904,  38364, 23996, 50404, 44460, 61572, 5116,  25696,
                35212, 57060, 2764,  46820, 10676, 64224, 41384, 21176, 14012,
                38860, 58968, 18048, 6608,  13796, 46360, 52596, 4784,  55172,
                23288, 3524,  20780, 58172, 48168, 25616, 37692, 7668,  11764,
                53848, 49124, 64904, 8916,  23804, 16008, 42608, 64244, 13836,
                44784, 34880, 884,   15324, 42976, 3080,  24216, 6012,  63064,
                35828, 43252, 61240, 49292, 136,   37448, 21372, 43212, 30392,
                38952, 25476,
            },
            {
                47664, 40052, 52384, 19576, 38244, 60256, 27676, 1244,  51004,
                44808, 18328, 26812, 6604,  23168, 42908, 4616,  63856, 10324,
                54504, 13876, 61488, 30780, 50552, 11572, 33268, 7412,  39280,
                56352, 8720,  18860, 62244, 38188, 45936, 1400,  13956, 57860,
                23372, 2492,  63600, 24788, 60724, 54488, 13948, 39140, 32064,
                28228, 1276,  35748, 59860, 5256,  63408, 45648, 54064, 21500,
                10008, 63540, 17752, 7040,  53088, 21796, 16032, 25944, 10248,
                50016, 28768, 58768, 41648, 1032,  35604, 29528, 47312, 11940,
                64492, 14500, 30172, 43124, 33640, 24320, 3236,  7648,  62048,
                25296, 656,   50848, 62488, 33788, 25920, 19396, 35332, 9652,
                28760, 62716, 39108, 11128, 53664, 2812,  13960, 63208, 41360,
                25144, 5068,  36004, 40476, 57044, 48372, 37268, 4332,  21732,
                56436, 18940, 62816, 49152, 11600, 57276, 45856, 53564, 13208,
                57888, 16152, 7892,  32760, 12308, 25868, 15096, 58012, 2156,
                17340, 55672,
            },
            {
                11932, 28932, 65216, 3024,  13756, 45576, 21756, 36796, 57272,
                24184, 31824, 65136, 47024, 57576, 32800, 18620, 40332, 22528,
                2120,  27788, 20380, 46824, 3016,  23308, 57452, 48228, 26892,
                13808, 53436, 34396, 27928, 6728,  56848, 25524, 36828, 41624,
                29296, 51932, 15848, 10212, 28380, 224,   44280, 61592, 4668,
                52716, 57988, 16016, 26952, 40212, 30420, 37264, 3632,  35304,
                51172, 31860, 41124, 12928, 28128, 34176, 60036, 37904, 31716,
                62212, 13124, 4292,  20336, 56120, 16160, 7948,  59104, 19688,
                38432, 49564, 8064,  61844, 50812, 15696, 36268, 46476, 56576,
                32384, 10168, 36916, 22776, 47980, 3116,  64248, 42088, 59184,
                46924, 14208, 32352, 45808, 17292, 34976, 57484, 28864, 19544,
                59964, 30380, 14200, 45204, 1268,  29552, 11052, 53692, 40884,
                30748, 9832,  38284, 25188, 29980, 35440, 16704, 20920, 31208,
                1432,  38456, 23448, 55296, 62964, 40908, 54056, 10688, 33616,
                60420, 20692,
            },
            {
                44228, 6416,  23336, 34968, 53812, 62828, 8092,  42032, 12144,
                5564,  52712, 9184,  13844, 37120, 52,    52460, 59276, 34516,
                44168, 57808, 36072, 7968,  63616, 37816, 18456, 42564, 61964,
                20552, 46592, 204,   41204, 51576, 21640, 62888, 10560, 44824,
                59828, 7392,  34088, 38360, 50000, 21840, 34780, 48548, 9868,
                20948, 42304, 47616, 8600,  21948, 57072, 11096, 17272, 58344,
                26500, 1408,  23212, 59164, 3112,  50472, 45684, 4892,  56400,
                22948, 44884, 48400, 33364, 64024, 23120, 39824, 52516, 33184,
                592,   27696, 22820, 17980, 4816,  60032, 54316, 28256, 18460,
                49072, 42172, 15628, 29984, 8544,  56660, 12812, 31416, 6520,
                18060, 52320, 944,   26960, 65524, 22320, 43832, 49632, 692,
                38796, 55524, 21820, 52420, 18764, 59328, 34084, 26608, 62168,
                2960,  46232, 55028, 5784,  51840, 59476, 8644,  39548, 50448,
                25620, 52888, 44876, 20148, 3548,  35008, 50244, 28372, 45296,
                4092,  36816,
            },
            {
                31852, 15948, 57152, 41432, 26060, 17420, 31164, 48436, 19484,
                62248, 38912, 43488, 28744, 20672, 49172, 11728, 26388, 15644,
                6280,  51452, 17060, 40956, 54208, 13260, 28656, 2408,  5964,
                31960, 25304, 64892, 9772,  15624, 32256, 53896, 17344, 4656,
                21068, 47048, 54876, 19320, 57596, 5440,  63308, 26308, 14496,
                37796, 64080, 33552, 54848, 1980,  44268, 24664, 48764, 42192,
                14300, 46576, 55100, 36868, 43608, 9160,  19764, 14020, 40572,
                1832,  18524, 37016, 10780, 51220, 26872, 13316, 44036, 5888,
                55236, 45848, 58116, 32112, 38928, 9696,  42664, 22176, 12436,
                3896,  65344, 53380, 60448, 44240, 38172, 21868, 49288, 40204,
                24280, 57324, 42928, 8392,  55388, 4452,  31568, 9764,  15972,
                46808, 33532, 10664, 7000,  63956, 23188, 13612, 7652,  49004,
                16892, 60172, 13304, 22016, 41660, 1972,  27424, 61828, 42864,
                4592,  65112, 9904,  30272, 47688, 17180, 5404,  64596, 22964,
                52492, 62368,
            },
            {
                9020,  50396, 47140, 680,   10488, 51128, 4800,  58280, 27988,
                34948, 1592,  17044, 54608, 63344, 30684, 8052,  38564, 47204,
                64844, 30476, 11164, 25356, 32468, 44552, 60660, 51784, 36760,
                58520, 11944, 55784, 44224, 28948, 3784,  47452, 35616, 27212,
                64696, 31736, 1868,  40544, 12616, 30536, 41584, 17816, 51400,
                29584, 3272,  12160, 18608, 50560, 32464, 65028, 6144,  61392,
                8632,  33252, 19052, 64404, 30464, 25192, 61892, 52356, 27984,
                65076, 54152, 29760, 6760,  42456, 2908,  31520, 62928, 21144,
                15716, 40988, 11424, 52668, 26116, 48076, 1292,  34544, 57628,
                39132, 6268,  20108, 27216, 1768,  17056, 54508, 27980, 2792,
                62320, 34588, 20496, 37932, 13372, 40276, 51556, 61480, 24540,
                5780,  62704, 26780, 43020, 35276, 46952, 40016, 52188, 20172,
                36424, 42840, 27940, 33784, 64088, 19400, 48224, 12240, 33480,
                17772, 35644, 14384, 59912, 42244, 24728, 9468,  39200, 18916,
                13708, 26448,
            },
            {
                58912, 37516, 18604, 29592, 59712, 33600, 40228, 14764, 53948,
                22304, 47284, 59600, 24936, 4216,  41748, 60600, 53840, 23688,
                2816,  20056, 61748, 48572, 580,   21768, 9452,  15880, 48900,
                40392, 17532, 22588, 34928, 49892, 60804, 12964, 40800, 56164,
                9304,  14524, 43912, 25832, 59972, 9028,  46696, 56036, 7204,
                59212, 24756, 45240, 60280, 28460, 14612, 38216, 20316, 29440,
                39792, 50772, 4836,  11240, 15764, 39172, 5900,  46844, 35540,
                8120,  24404, 58992, 15952, 61108, 53712, 47524, 9356,  25396,
                60452, 34316, 2640,  64564, 20532, 14096, 30452, 63172, 45120,
                24028, 31072, 35836, 12096, 51316, 58752, 33696, 9896,  15336,
                50700, 11636, 29532, 48592, 60592, 17944, 27656, 35180, 53320,
                41612, 57020, 17664, 50320, 2472,  28476, 4264,  58772, 25108,
                32272, 88,    8872,  50648, 15524, 37416, 55212, 6340,  22676,
                46716, 57116, 27592, 1588,  51124, 55728, 31952, 58144, 48964,
                1068,  42444,
            },
            {
                4796,  21736, 63732, 12864, 44340, 23624, 64708, 2976,  44888,
                10096, 31440, 12848, 7292,  34408, 45944, 17856, 13392, 33356,
                56540, 36756, 42416, 5512,  59376, 55480, 35236, 24148, 29740,
                8004,  53284, 2216,  38688, 7216,  19960, 24880, 776,   50664,
                37988, 23580, 61636, 52676, 16996, 35900, 23076, 1028,  32724,
                39972, 53720, 36428, 5572,  41604, 9536,  52336, 32,    55924,
                22380, 60340, 27724, 48340, 53608, 58056, 22260, 32048, 17172,
                12080, 43892, 49416, 38844, 21920, 35412, 17604, 56216, 37668,
                50460, 29124, 7596,  44256, 36096, 55692, 50132, 7424,  16216,
                52520, 59660, 46320, 41044, 7628,  24692, 43312, 64592, 37368,
                45392, 6296,  58788, 1640,  23964, 45252, 6740,  11952, 20548,
                2112,  29812, 12872, 37432, 61052, 16232, 55712, 43780, 10892,
                65416, 54260, 45316, 58064, 3700,  24396, 44040, 29448, 60572,
                51956, 7800,  39956, 21460, 36832, 12520, 6560,  43908, 35424,
                29384, 54664,
            },
            {
                40240, 32408, 6680,  55356, 36256, 20060, 8760,  37880, 26032,
                61832, 49900, 41012, 57416, 51692, 22128, 1504,  28272, 44836,
                9508,  50868, 14996, 29560, 39076, 18808, 46460, 63796, 3216,
                61412, 44996, 26640, 57108, 63952, 43076, 33188, 59304, 18024,
                30076, 47632, 5204,  33272, 2948,  49356, 65220, 43516, 19536,
                10064, 15340, 21152, 62580, 23868, 57644, 34336, 45528, 12420,
                36196, 16604, 44088, 1156,  34272, 42284, 2968,  61124, 51444,
                57160, 184,   32564, 5236,  12672, 28676, 1352,  41840, 4704,
                13592, 47128, 23144, 16748, 60832, 3384,  40560, 19692, 26944,
                9980,  28,    14748, 63332, 20904, 47876, 5492,  18844, 30416,
                56124, 25552, 17028, 33116, 53704, 36652, 62952, 48976, 58192,
                38508, 45968, 23404, 53852, 9372,  22048, 31252, 6044,  37820,
                14504, 23328, 18856, 39336, 31068, 9952,  63060, 14600, 648,
                31936, 11712, 45076, 64040, 18112, 61440, 27068, 15916, 60220,
                10932, 24176,
            },
            {
                14672, 51864, 46756, 26868, 2292,  52620, 48676, 56580, 16684,
                6244,  36064, 19100, 27036, 15040, 37344, 61996, 58008, 39880,
                4928,  26836, 22496, 53444, 12088, 6572,  26988, 13820, 42188,
                32944, 20868, 13228, 31080, 16280, 10420, 54532, 45440, 7984,
                63396, 11948, 20092, 56808, 28616, 38804, 14076, 26628, 61128,
                50096, 46528, 31540, 2092,  48880, 17252, 26660, 63112, 4128,
                52952, 24904, 8288,  63248, 20040, 13564, 27112, 10060, 37684,
                21096, 41172, 26428, 63628, 46484, 58328, 23572, 65372, 31244,
                19412, 53472, 58892, 39784, 25176, 12008, 29304, 61988, 43176,
                33724, 49116, 55372, 28812, 32552, 38908, 60348, 1180,  52756,
                22112, 40432, 61816, 43256, 10356, 3208,  22504, 15368, 32000,
                8556,  63580, 4776,  33120, 47960, 39532, 63164, 18244, 51496,
                47740, 29392, 6664,  61672, 48604, 34696, 51344, 41312, 19904,
                37204, 54616, 25168, 3992,  33740, 47160, 52564, 2044,  20420,
                45572, 61976,
            },
            {
                34700, 17536, 9708,  62272, 41456, 15544, 28824, 32780, 43112,
                21624, 54136, 3644,  65052, 9208,  47656, 30168, 11548, 19384,
                63100, 46728, 60124, 33732, 43436, 49884, 57200, 38500, 52664,
                10040, 59052, 46724, 51476, 5744,  37288, 28332, 21972, 2608,
                35124, 40176, 51564, 42600, 7084,  58448, 10840, 54072, 35228,
                3604,  28240, 56128, 12748, 43580, 7264,  40292, 19700, 31032,
                47912, 41200, 59264, 29660, 38416, 56612, 50224, 44352, 29300,
                62516, 14028, 53876, 19352, 9052,  40268, 51796, 44976, 11092,
                61356, 35668, 472,   9620,  32692, 47456, 54048, 5424,  58188,
                38472, 23264, 18512, 4052,  10572, 57344, 26728, 13452, 34848,
                9152,  4020,  50120, 13772, 30532, 56584, 40620, 28028, 44052,
                19032, 50788, 26148, 14772, 59032, 1084,  44984, 25572, 33836,
                2436,  41188, 55564, 12764, 26192, 17552, 5164,  56444, 23612,
                47400, 16580, 58812, 29704, 8880,  40972, 23180, 31420, 38016,
                49276, 3760,
            },
            {
                57444, 43748, 22624, 31224, 58176, 5192,  11612, 63404, 1060,
                59140, 29596, 45364, 32580, 40060, 55288, 6064,  24500, 52720,
                35264, 1860,  17400, 8432,  65332, 1240,  31048, 22364, 4224,
                17948, 36152, 972,   40868, 24088, 62668, 48896, 13588, 53268,
                57864, 27256, 22864, 15100, 46560, 19044, 31688, 5832,  22216,
                40760, 8376,  64304, 37764, 32740, 59576, 50212, 10144, 56768,
                14792, 5844,  33308, 11416, 45912, 24160, 4612,  17696, 7440,
                47772, 4012,  34472, 50124, 30200, 6240,  15588, 34200, 26088,
                7056,  42196, 27864, 56940, 51580, 17392, 36400, 21720, 13676,
                2820,  62644, 44688, 35660, 53076, 16476, 49740, 42540, 46692,
                63736, 28012, 37824, 21084, 47684, 65044, 5860,  52476, 59684,
                480,   34572, 55788, 42536, 20100, 28960, 12428, 54564, 58420,
                10208, 60520, 35536, 21312, 1224,  45636, 59176, 28420, 7604,
                63476, 2444,  41912, 49568, 14692, 57376, 5016,  54484, 65244,
                8256,  25880,
            },
            {
                53332, 140,   37408, 49960, 19632, 35788, 45948, 24220, 38352,
                13916, 51384, 10968, 23760, 376,   16428, 50228, 43984, 31704,
                14448, 41724, 54908, 25656, 20216, 36856, 15836, 61196, 47952,
                25828, 64624, 29280, 55292, 19500, 3588,  41828, 31664, 16500,
                44332, 4460,  9112,  60852, 36348, 24720, 63656, 44152, 51136,
                58928, 18388, 25008, 52088, 15616, 27852, 3328,  23660, 38064,
                64568, 21648, 54796, 18356, 2104,  52028, 64968, 31512, 36164,
                55620, 24860, 16904, 60024, 37012, 55224, 20936, 1964,  57632,
                49724, 22008, 14252, 44416, 3876,  64972, 8180,  46184, 31976,
                51100, 26068, 7792,  40632, 65104, 23028, 7208,  31236, 19956,
                55216, 15964, 58256, 7428,  26084, 18548, 35468, 10900, 24556,
                13984, 40096, 10048, 64528, 5252,  36800, 49424, 7856,  22468,
                17324, 44012, 27408, 52724, 64780, 37504, 11924, 40000, 33260,
                53032, 10888, 35516, 21040, 62124, 36940, 11396, 18300, 42316,
                13580, 30068,
            },
            {
                40556, 64072, 12500, 7388,  26340, 53404, 60980, 17544, 49128,
                7300,  41368, 18560, 57076, 60668, 34188, 21112, 64136, 3196,
                57720, 10612, 38712, 30144, 58648, 45676, 10816, 55088, 33544,
                43152, 8356,  49360, 11756, 34112, 58656, 9216,  61164, 24344,
                38160, 64888, 32960, 53480, 2756,  49008, 39416, 516,   15760,
                30784, 11464, 45056, 1188,  21276, 62036, 54148, 33828, 44468,
                796,   28068, 47404, 36820, 58644, 15108, 40712, 21784, 60532,
                9600,  45488, 1328,  42960, 11472, 27040, 63056, 39344, 46764,
                17300, 63688, 31076, 38088, 20240, 24640, 40836, 28236, 55772,
                17016, 59596, 12224, 30108, 45824, 2052,  37568, 60764, 11532,
                41528, 272,   34228, 51920, 44708, 1924,  54980, 46832, 38004,
                61728, 48576, 30612, 22716, 52412, 16040, 41344, 62536, 31192,
                39028, 3240,  14560, 49224, 8892,  30924, 21912, 50296, 14312,
                25412, 44704, 28224, 6336,  51828, 24832, 44860, 27476, 33864,
                59224, 19968,
            },
            {
                24332, 46996, 16636, 59672, 39920, 1848,  9500,  33844, 55364,
                27200, 63784, 36220, 47232, 26664, 42484, 8564,  38124, 28148,
                46992, 22660, 51180, 3780,  49060, 6936,  28532, 40552, 5080,
                14248, 23280, 57616, 17140, 39276, 27272, 45784, 50896, 356,
                19224, 48552, 11488, 28736, 17448, 55948, 12400, 27380, 62084,
                37184, 48568, 55316, 41336, 6332,  36528, 46620, 17928, 11840,
                60896, 51052, 8940,  42952, 25628, 34352, 6388,  48648, 13004,
                38920, 28356, 64440, 21888, 48844, 5148,  32228, 52572, 12592,
                36628, 2572,  54296, 60352, 11644, 50028, 57944, 832,   10132,
                42632, 34292, 48616, 20660, 14716, 56372, 26288, 51696, 5812,
                24228, 29716, 62676, 12044, 39912, 31592, 15476, 21320, 29428,
                4472,  17912, 7380,  44672, 32828, 57948, 26708, 4100,  47060,
                53392, 63524, 32940, 19236, 41736, 2716,  61224, 43872, 4516,
                19488, 55016, 59840, 17128, 32584, 3612,  63220, 47824, 2220,
                50556, 6028,
            },
            {
                10240, 28820, 33528, 55628, 44456, 30048, 20836, 42460, 3856,
                31460, 21896, 2560,  14636, 5456,  52344, 12708, 18128, 60328,
                7164,  15800, 63680, 33300, 13444, 24456, 62944, 20668, 52452,
                60148, 37316, 44864, 4340,  53164, 21748, 13040, 35680, 7824,
                30540, 56516, 40860, 45336, 6812,  34356, 20764, 52184, 7980,
                23352, 4036,  34036, 29036, 58004, 14048, 8028,  30520, 41760,
                24940, 16148, 31896, 3224,  62648, 10908, 57052, 26908, 53732,
                18368, 33384, 57516, 14476, 40580, 56488, 18188, 7848,  28884,
                24740, 9892,  45892, 5168,  27076, 34592, 15660, 62216, 37232,
                23460, 6428,  54480, 4440,  61312, 33216, 18424, 44344, 35044,
                54068, 48200, 17576, 22900, 60988, 56816, 8220,  64036, 42688,
                52048, 26992, 55180, 60920, 1600,  10988, 35372, 18984, 13200,
                24124, 7296,  46008, 57848, 25004, 53808, 16680, 56568, 36524,
                64576, 236,   39092, 48516, 12544, 56192, 38692, 8812,  16036,
                55344, 36684,
            },
            {
                62948, 49516, 5396,  23060, 14576, 51112, 65392, 12804, 46252,
                59244, 10124, 53912, 39408, 62496, 29628, 56416, 48824, 24928,
                36236, 44328, 53492, 19084, 42412, 37716, 50824, 16968, 112,
                31216, 27468, 7408,  62756, 32040, 2484,  64128, 54828, 26404,
                59544, 14324, 21524, 25272, 63260, 59108, 42124, 32084, 46196,
                56788, 19256, 65060, 9824,  49732, 22780, 63504, 52364, 56992,
                5596,  39496, 59516, 53144, 19828, 30024, 45360, 536,   42324,
                4968,  51764, 8060,  29860, 2732,  23252, 44548, 62268, 41984,
                59692, 33540, 51288, 18248, 43180, 6968,  47444, 30400, 52372,
                19012, 64116, 38756, 28600, 50720, 39984, 8904,  13700, 64280,
                3452,  38400, 9432,  43556, 3952,  26496, 49012, 34140, 2332,
                58360, 11816, 37296, 20364, 24996, 51036, 42944, 55072, 59540,
                37076, 28180, 11232, 38632, 5612,  29372, 34360, 10192, 27100,
                46708, 30984, 9384,  22244, 43360, 28980, 19196, 23408, 60308,
                30492, 44304,
            },
            {
                1256,  18172, 38572, 61508, 3360,  36408, 24968, 57056, 37980,
                16396, 50384, 25232, 32432, 44744, 22556, 1792,  40820, 31372,
                55252, 916,   29024, 8980,  60752, 2696,  45244, 35056, 56432,
                10260, 48112, 18872, 50632, 14908, 43308, 40064, 17748, 46840,
                38556, 4348,  51236, 1304,  37284, 10188, 16316, 4944,  60864,
                13616, 39832, 26140, 16532, 44140, 38388, 27060, 3864,  48524,
                20744, 35992, 12988, 23512, 49204, 16568, 37540, 64132, 21268,
                59152, 24248, 47908, 37704, 60652, 34040, 50196, 14120, 4144,
                55380, 22332, 39948, 29520, 64384, 55108, 20880, 12960, 44476,
                2320,  25820, 46720, 11244, 22316, 856,   58120, 47436, 28084,
                20784, 59888, 32604, 51144, 36616, 19416, 12968, 39672, 24040,
                16672, 33132, 47172, 40588, 14864, 65256, 6304,  29344, 248,
                49928, 20452, 60476, 51428, 13900, 63240, 48052, 23044, 6840,
                41088, 15884, 61740, 34876, 58628, 1448,  52840, 35416, 41080,
                12296, 21568,
            },
            {
                57744, 42132, 53024, 11332, 28104, 48284, 7908,  18796, 5200,
                28500, 41716, 60244, 7028,  17048, 35388, 10812, 65160, 5692,
                20244, 12476, 40108, 23528, 57164, 31876, 12124, 26024, 64984,
                38680, 42044, 22592, 34932, 59132, 28876, 24036, 5584,  11068,
                33396, 62204, 43072, 30944, 53544, 47056, 24536, 29216, 35876,
                43400, 1500,  52904, 31392, 60408, 2356,  19208, 34744, 10460,
                29056, 65432, 46012, 4852,  40516, 56168, 9036,  32292, 15184,
                36012, 11528, 43724, 16724, 53208, 9828,  26612, 37168, 19996,
                46640, 15464, 940,   12348, 36584, 23916, 3608,  40112, 58988,
                9520,  56148, 33568, 15376, 43712, 63364, 31332, 23748, 41860,
                16156, 56056, 6736,  14724, 28696, 53296, 60056, 44440, 6076,
                50588, 63496, 8772,  3964,  30824, 48216, 21452, 39124, 16600,
                44376, 4548,  34740, 42076, 17900, 980,   43164, 57700, 13320,
                50520, 53628, 24896, 5620,  49888, 14920, 64944, 46524, 3972,
                50928, 27180,
            },
            {
                35188, 6996,  31676, 20504, 59012, 41436, 32032, 52408, 63132,
                33940, 1088,  13068, 20464, 47432, 58040, 51396, 15136, 45628,
                59060, 33596, 62388, 47692, 16544, 5268,  54836, 21856, 6524,
                14704, 61024, 1624,  11408, 56044, 8216,  48780, 60744, 52968,
                20560, 27544, 8372,  18516, 13356, 57788, 3064,  49868, 54792,
                8748,  22076, 47888, 6164,  12556, 54596, 41424, 61700, 15080,
                43528, 54456, 8184,  33452, 27580, 2700,  51608, 25388, 47120,
                2176,  62824, 27536, 5488,  20536, 65240, 2036,  57968, 31348,
                7096,  63184, 52916, 58388, 8652,  49844, 61752, 32144, 27884,
                36288, 17860, 60484, 3860,  53200, 36772, 6860,  11836, 52612,
                2004,  45396, 24604, 65528, 46908, 668,   9808,  30224, 56196,
                19880, 28112, 43572, 58884, 54680, 34424, 10140, 61180, 56680,
                32304, 64340, 9104,  22420, 31196, 52844, 37100, 19540, 60772,
                28792, 2416,  20028, 41900, 31740, 10464, 26184, 7356,  32928,
                61584, 15432,
            },
            {
                46928, 25224, 64584, 44676, 16756, 412,   56108, 11872, 21488,
                44852, 49372, 55248, 63848, 38892, 3336,  27836, 23928, 37224,
                50368, 26132, 3508,  52628, 37068, 28264, 46960, 42856, 49552,
                30044, 52060, 25652, 45196, 38068, 20032, 31584, 1052,  44400,
                15176, 56960, 48416, 64324, 34916, 39980, 21080, 63604, 17564,
                27140, 62416, 33708, 57712, 36620, 24276, 46964, 30936, 58320,
                284,   25764, 18216, 58760, 63340, 43348, 13120, 61048, 39884,
                30212, 50164, 54960, 40968, 31772, 46240, 12696, 41396, 49428,
                27644, 38320, 32492, 42500, 26156, 17388, 45612, 14788, 51708,
                6320,  48788, 41376, 29912, 25136, 19356, 48336, 56900, 38852,
                29460, 35512, 11204, 41032, 56996, 22152, 38476, 62700, 13640,
                36104, 1072,  23396, 12532, 18160, 25956, 45800, 6952,  23596,
                13628, 26712, 54460, 46216, 59144, 25444, 8412,  32400, 4132,
                39628, 35984, 63908, 57424, 45700, 17716, 39892, 55936, 19988,
                9676,  54116,
            },
            {
                2772,  13548, 50452, 9288,  35880, 24292, 46208, 39580, 26748,
                8640,  36468, 23360, 30020, 9252,  33440, 54220, 42680, 6908,
                17460, 9960,  43268, 13832, 20964, 60388, 8348,  34380, 18656,
                3460,  57240, 33392, 4988,  53856, 13012, 65460, 35300, 25204,
                40568, 36840, 2248,  23600, 28436, 6348,  42780, 32528, 11512,
                38980, 45620, 14344, 19964, 50912, 9236,  17528, 6756,  50480,
                22084, 37612, 49404, 10772, 21520, 35520, 17432, 6512,  57916,
                19152, 9224,  22664, 14604, 57176, 35716, 24736, 17320, 61712,
                9996,  23464, 18820, 47976, 5248,  34648, 56812, 424,   22736,
                65316, 12364, 21188, 55492, 8700,  61492, 33764, 17328, 4824,
                63140, 50420, 18588, 31744, 5536,  16812, 34440, 25648, 45904,
                54268, 48756, 38764, 61960, 50756, 2228,  37104, 53064, 42300,
                48580, 1816,  38900, 15792, 3472,  12300, 65388, 44516, 54652,
                17052, 47576, 7776,  12828, 27640, 36644, 60044, 23852, 43864,
                29916, 38880,
            },
            {
                22512, 60800, 28668, 5616,  53516, 60276, 14260, 4524,  61408,
                57300, 15572, 4228,  43636, 18064, 61456, 12860, 21056, 63232,
                56440, 29700, 35196, 64700, 676,   40816, 57800, 13132, 63356,
                24068, 40368, 15832, 62884, 27956, 41920, 17136, 49972, 58492,
                6648,  12324, 54404, 45172, 10728, 60316, 15356, 52428, 168,
                58756, 4620,  29712, 42392, 1916,  64464, 27816, 40080, 12356,
                32888, 53560, 41512, 3692,  29772, 48008, 54516, 23388, 44604,
                34724, 4324,  38212, 60268, 208,   8072,  51612, 54584, 3868,
                44404, 56116, 1780,  13148, 59664, 29576, 10284, 41872, 37648,
                26508, 45972, 1736,  39204, 51024, 13496, 45012, 27064, 59240,
                21376, 8084,  54748, 26752, 61880, 43096, 51028, 3184,  7884,
                16268, 32168, 5568,  29268, 41764, 15156, 63840, 28280, 11080,
                18336, 35944, 62576, 50716, 28996, 41224, 48884, 21788, 26692,
                10348, 33404, 23304, 55160, 1804,  51492, 6184,  47820, 504,
                63696, 17164,
            },
            {
                43172, 56740, 40168, 32912, 21048, 38368, 29212, 47852, 18592,
                31084, 40320, 52912, 25484, 50032, 46084, 56,    31660, 39612,
                4032,  46248, 22536, 49840, 31500, 17584, 26384, 51032, 37524,
                9856,  46768, 20696, 7708,  47608, 23540, 3332,  9624,  29084,
                21184, 61232, 18008, 31628, 49540, 56376, 25960, 44616, 36784,
                23392, 49316, 55808, 25712, 59640, 33204, 44668, 55452, 63036,
                5468,  16444, 61512, 13964, 56988, 39272, 836,   28352, 11888,
                53148, 64428, 25672, 47324, 43288, 28660, 21380, 39048, 30868,
                14980, 35328, 64856, 51924, 39268, 20436, 62736, 52820, 18552,
                56324, 34160, 62332, 16312, 32156, 5680,  37924, 332,   42292,
                14112, 46188, 39384, 1672,  48704, 12800, 58652, 20740, 40044,
                64884, 57504, 21876, 9248,  58528, 32812, 4920,  20524, 60048,
                30252, 56212, 5436,  20852, 34232, 56764, 6220,  14420, 61796,
                52056, 59504, 43224, 30928, 20924, 62872, 16020, 33948, 12584,
                49700, 35496,
            },
            {
                4120,  11272, 18380, 49312, 2604,  64164, 9928,  33652, 51216,
                1420,  65012, 11524, 59248, 7160,  36048, 58160, 26760, 52144,
                14728, 59644, 7764,  55384, 11296, 45540, 5748,  32388, 1904,
                55136, 59348, 30728, 35972, 51148, 60104, 38820, 55084, 45548,
                34056, 51328, 41640, 5320,  35908, 2588,  19896, 8448,  30584,
                65092, 18028, 7352,  10912, 38424, 15116, 4396,  23172, 36460,
                26856, 47280, 31300, 44732, 24988, 9012,  62808, 33328, 48636,
                41136, 16920, 10592, 32864, 18504, 63136, 11224, 60060, 6716,
                48652, 20912, 27080, 43344, 7404,  24768, 45212, 4272,  30972,
                7516,  10680, 42660, 28492, 58552, 21808, 63980, 53796, 30956,
                24384, 34496, 58024, 10108, 23164, 33304, 53940, 30052, 10576,
                26416, 43496, 52284, 18800, 45428, 24544, 55176, 40020, 50408,
                7816,  43452, 24944, 10856, 45544, 17940, 37996, 30376, 448,
                40288, 4468,  15048, 49200, 39104, 9912,  29112, 41812, 58340,
                26456, 54660,
            },
            {
                31904, 47228, 25732, 57920, 43664, 16308, 55104, 23104, 7552,
                43120, 20384, 34844, 29068, 14564, 41316, 19724, 10264, 48644,
                24468, 37456, 19120, 27828, 38988, 61724, 54156, 21448, 42404,
                28336, 14108, 44408, 844,   11468, 19344, 32524, 14188, 63624,
                468,   26544, 14700, 64692, 22772, 39740, 62116, 47596, 54028,
                12840, 41412, 35492, 53120, 20828, 47528, 51668, 18660, 10136,
                59160, 1728,  21072, 6088,  36676, 50852, 20248, 14616, 59076,
                2420,  30764, 51228, 55892, 3480,  49560, 34520, 45716, 25596,
                57356, 38184, 3200,  31436, 15888, 55552, 35896, 13492, 49672,
                60120, 23736, 53432, 3404,  49196, 25816, 46876, 11560, 6884,
                49788, 62188, 16464, 28984, 37972, 6300,  17480, 60204, 47532,
                34812, 13908, 1944,  37424, 49440, 11876, 35576, 520,   46520,
                14224, 33176, 59392, 53936, 1568,  63444, 22984, 58132, 34708,
                46412, 19476, 25536, 7008,  57032, 44776, 53140, 22816, 5672,
                20180, 8692,
            },
            {
                15240, 37680, 62832, 6988,  12424, 27596, 41244, 59480, 37536,
                26356, 54184, 47096, 3012,  23040, 63668, 55516, 4688,  33848,
                42480, 1516,  63536, 44132, 3008,  24600, 15016, 47348, 64472,
                18752, 4652,  52540, 62456, 26432, 56820, 43384, 5172,  24272,
                37860, 47720, 8172,  29896, 52128, 11692, 28636, 15808, 34068,
                6052,  22252, 45348, 63008, 31544, 688,   61068, 29284, 43044,
                50052, 39660, 54772, 63800, 17184, 57976, 42640, 5388,  26420,
                21704, 61500, 7120,  37200, 23412, 15172, 40548, 1220,  18068,
                12552, 50932, 60384, 10696, 63500, 48072, 1968,  27728, 16880,
                38728, 46616, 19644, 13176, 40592, 8524,  18016, 35672, 56184,
                19680, 2544,  43960, 53204, 64608, 46348, 42152, 104,   23524,
                5124,  55848, 30860, 63620, 7076,  60628, 16912, 27192, 65108,
                22568, 38684, 16716, 28140, 40904, 51728, 8992,  49324, 12236,
                28384, 64588, 37508, 60508, 32396, 18164, 3372,  65280, 38536,
                51928, 61608,
            },
            {
                41548, 848,   23428, 31136, 52216, 35484, 2040,  49032, 13684,
                62532, 9596,  16712, 39560, 50416, 30276, 44556, 17240, 61864,
                53584, 12328, 32556, 16496, 52248, 34192, 7136,  36040, 11100,
                39524, 24288, 33964, 40732, 15592, 7116,  29764, 52352, 20256,
                10936, 53988, 57832, 43544, 18416, 58612, 4692,  42184, 60732,
                26200, 56868, 3364,  28124, 13624, 40776, 55868, 7128,  33932,
                12772, 24448, 8472,  33032, 27624, 11432, 47144, 35436, 54856,
                39676, 45852, 13184, 42820, 27480, 58712, 52920, 29836, 63700,
                33804, 7812,  44920, 19220, 33468, 22804, 41388, 61652, 57084,
                32816, 6352,  64628, 36632, 57488, 33072, 61016, 43432, 27736,
                39744, 13060, 32372, 4192,  20448, 13996, 27708, 36312, 50576,
                62552, 40644, 16084, 27612, 22288, 42264, 31920, 51916, 9968,
                4172,  61664, 48456, 6872,  13516, 31988, 25348, 43352, 16772,
                54800, 8460,  50676, 2452,  12092, 36200, 27176, 14320, 33200,
                45680, 28108,
            },
            {
                58796, 48688, 54956, 19432, 45600, 63584, 21944, 17848, 5340,
                45300, 31776, 57000, 60512, 6168,  12244, 37856, 27352, 8132,
                20988, 29736, 57440, 49484, 9616,  60096, 29852, 57172, 49112,
                60748, 8404,  56224, 21284, 49284, 36500, 60404, 46884, 41288,
                62312, 16360, 32288, 2032,  36804, 46160, 23864, 49436, 1216,
                37852, 50996, 17040, 58516, 8636,  25040, 36844, 16664, 65236,
                57312, 19336, 46324, 52688, 41052, 4328,  30884, 64920, 8656,
                17720, 29224, 56916, 19368, 64600, 9496,  5040,  21636, 55208,
                41684, 23316, 28424, 54124, 39648, 5324,  51192, 11120, 21044,
                43640, 700,   29636, 24440, 50804, 2424,  15148, 5648,  64392,
                22596, 51132, 59464, 26196, 48532, 8796,  61032, 54676, 11972,
                18960, 45124, 8820,  53736, 47136, 3020,  56616, 19504, 44312,
                30352, 52940, 35032, 21064, 57708, 36984, 3056,  61248, 5592,
                21772, 41084, 30552, 23656, 46776, 54380, 59448, 48928, 10708,
                2540,  17676,
            },
            {
                34308, 9852,  13736, 36936, 4484,  9140,  33028, 58456, 29188,
                36580, 1228,  20844, 25940, 34456, 52708, 908,   47636, 58228,
                36176, 45768, 6408,  22916, 40804, 19440, 43548, 4876,  26276,
                16620, 31356, 46600, 2752,  63280, 10368, 17708, 1584,  27404,
                34836, 4360,  21980, 25988, 62768, 9640,  32796, 56176, 20528,
                30336, 10544, 46988, 34608, 43504, 50340, 21804, 45692, 3244,
                27908, 38540, 14952, 540,   61348, 21908, 16164, 51996, 24080,
                48860, 684,   34816, 5932,  47968, 32376, 44804, 36612, 14824,
                4060,  48304, 59172, 268,   13904, 58356, 29824, 7288,  25900,
                52924, 15296, 55044, 10392, 45272, 21492, 30680, 53048, 47240,
                10152, 36668, 7044,  41296, 55980, 33860, 39540, 21800, 7272,
                32472, 25160, 59268, 34296, 13044, 39020, 6192,  36660, 58460,
                25308, 12152, 2304,  43620, 64632, 47008, 19268, 53104, 35084,
                58776, 45588, 14712, 63244, 20120, 42140, 7500,  24608, 40248,
                56852, 22432,
            },
            {
                64500, 44184, 28796, 60880, 40152, 26584, 54448, 50380, 42252,
                23444, 53496, 48276, 14092, 43088, 19568, 65352, 24092, 15480,
                40144, 2048,  26048, 64648, 37040, 12596, 55004, 22184, 936,
                53228, 37920, 12876, 28788, 43744, 23720, 33504, 57952, 13404,
                55444, 44868, 50644, 39396, 13792, 54280, 17424, 7004,  61732,
                14352, 39876, 64376, 19328, 5012,  61640, 11796, 53292, 32036,
                6312,  49456, 59256, 35868, 25172, 56380, 43836, 2856,  37632,
                11760, 62080, 52504, 25216, 38800, 12808, 50776, 26320, 61696,
                10344, 37560, 16700, 25208, 35976, 46148, 18156, 62904, 37128,
                47676, 33916, 61968, 18376, 41524, 60188, 37900, 25376, 624,
                57876, 29232, 18720, 62504, 24000, 16176, 2744,  29260, 49164,
                56980, 38144, 968,   19908, 50192, 61296, 24096, 14884, 47828,
                41216, 17784, 55008, 27168, 9376,  15408, 30268, 39828, 26552,
                1168,  32924, 10288, 52120, 28900, 244,   16420, 62752, 30472,
                50308, 5576,
            },
            {
                25856, 7384,  51372, 16864, 57648, 46796, 11640, 15252, 7080,
                63964, 10520, 40232, 61924, 7748,  55256, 31340, 10220, 50884,
                61396, 13220, 54620, 47884, 3580,  28928, 62636, 46136, 33108,
                41476, 65176, 19580, 5356,  54120, 39636, 51068, 6704,  22436,
                37616, 8956,  58900, 5956,  47840, 27652, 41092, 35340, 44156,
                53160, 2244,  24064, 32268, 54920, 27052, 1680,  39616, 62732,
                23244, 43584, 11004, 30100, 7672,  49780, 12824, 32952, 58432,
                42120, 31352, 16088, 59384, 21252, 1492,  56136, 17936, 42848,
                31672, 52444, 65188, 43480, 9656,  49800, 32080, 55844, 12548,
                2904,  39536, 8016,  26216, 4156,  55836, 12200, 17500, 42684,
                33240, 14900, 45004, 1692,  11652, 46016, 52936, 63428, 42932,
                14768, 5684,  64772, 30036, 43260, 9716,  28432, 64004, 1192,
                8112,  60128, 31768, 38808, 23248, 51108, 6576,  11976, 63572,
                56336, 18260, 37112, 5144,  56784, 38856, 34584, 45260, 19732,
                12920, 37492,
            },
            {
                53648, 20208, 32424, 3676,  24032, 132,   20612, 39008, 59568,
                27504, 18316, 32624, 4056,  28856, 38532, 46064, 3032,  33752,
                27880, 20204, 43084, 16108, 33956, 7960,  17796, 51620, 14148,
                6780,  57304, 49772, 25928, 59404, 14936, 30836, 64712, 48556,
                29072, 17156, 63368, 20140, 31364, 320,   65284, 12180, 25140,
                28988, 59952, 8136,  42276, 15400, 47916, 29816, 18464, 13800,
                55448, 34268, 20372, 64484, 45836, 38296, 18712, 63144, 26488,
                20680, 4160,  45248, 9900,  41400, 63032, 29724, 7724,  58060,
                1984,  20916, 28892, 6412,  60428, 21572, 4504,  41696, 23664,
                17112, 58536, 50144, 31792, 46660, 34988, 7416,  62916, 49248,
                21244, 54628, 38304, 51732, 31496, 59168, 36088, 4424,  18256,
                27232, 47924, 52672, 23116, 16876, 55568, 33284, 52076, 35596,
                22116, 49784, 44896, 5112,  56524, 61516, 42356, 49160, 23960,
                8068,  47732, 42872, 61560, 25184, 49796, 9524,  59128, 4072,
                55864, 43184,
            },
            {
                14720, 61916, 47272, 41420, 35952, 65168, 30912, 44644, 34752,
                2568,  45744, 51484, 58104, 15852, 22840, 59936, 18232, 42064,
                56492, 8792,  31040, 60868, 50372, 39724, 58236, 26956, 36140,
                23520, 9700,  30300, 36908, 164,   8900,  18928, 45404, 2448,
                41176, 11588, 33684, 43208, 56480, 22884, 51660, 46072, 4556,
                48732, 21204, 57252, 37732, 10372, 60536, 35956, 50728, 41636,
                8756,  4116,  52212, 16552, 1928,  28340, 53580, 5160,  8860,
                55640, 50492, 27920, 35936, 53348, 14680, 46572, 22752, 35108,
                49192, 12600, 40048, 55472, 34280, 15248, 27616, 64060, 51940,
                45048, 28860, 20984, 64776, 14552, 51560, 28288, 39896, 58588,
                9156,  4912,  65532, 27288, 8036,  19876, 25432, 40396, 9860,
                58828, 34972, 12156, 40720, 6748,  46180, 3956,  18584, 40400,
                29544, 62352, 13364, 16884, 33344, 1552,  20328, 35852, 14192,
                52836, 28212, 21020, 12132, 32116, 14972, 22608, 52508, 28120,
                33428, 1288,
            },
            {
                39212, 29556, 10768, 55404, 13620, 48864, 53040, 5624,  22936,
                56188, 13340, 37140, 25332, 43756, 11388, 49492, 6016,  63560,
                36720, 4352,  53012, 24512, 2468,  21780, 10468, 42936, 1876,
                60784, 44448, 16468, 47112, 62224, 42168, 35364, 25088, 54868,
                61448, 26436, 52992, 3736,  15428, 37372, 9232,  19244, 38976,
                16132, 34312, 51272, 3672,  44472, 21656, 6440,  56556, 24680,
                59740, 47068, 26376, 40612, 57904, 22764, 42704, 35240, 48040,
                13596, 38092, 65100, 2692,  24356, 33500, 5308,  39640, 60624,
                25740, 44956, 18648, 3092,  50580, 45652, 38324, 804,   9492,
                35740, 5352,  11384, 41168, 1016,  19460, 23808, 3424,  30996,
                13728, 47556, 22656, 34792, 43200, 49584, 13388, 54936, 32808,
                21288, 45040, 1936,  62396, 37132, 59320, 26720, 12524, 57760,
                9612,  3552,  25760, 52772, 37776, 27968, 58636, 45532, 31384,
                3520,  40480, 59864, 2296,  44192, 64652, 6196,  40892, 17920,
                63004, 23712,
            },
            {
                49676, 58856, 5808,  26360, 19036, 8840,  28312, 16436, 62900,
                49300, 20136, 8528,  64320, 600,   34508, 54520, 26808, 13008,
                22372, 46944, 38716, 11680, 45156, 55408, 30188, 64168, 48588,
                19984, 32860, 56008, 11964, 21564, 52236, 57608, 12820, 5332,
                20980, 35864, 7992,  49952, 29464, 61088, 58156, 32052, 54248,
                63764, 11304, 30544, 17828, 26296, 64964, 32992, 36,    16272,
                37280, 31248, 13084, 62076, 33556, 10164, 15292, 60768, 30124,
                23640, 17496, 7248,  58820, 19468, 49880, 55412, 10744, 16196,
                53092, 8292,  63452, 30484, 23456, 10952, 57668, 18948, 30988,
                61288, 53364, 24812, 37084, 56160, 61568, 44684, 53000, 36476,
                26052, 41348, 60376, 3836,  16480, 62112, 416,   46272, 64360,
                7396,  53556, 30776, 25280, 15012, 49144, 21168, 42372, 54224,
                47488, 36136, 20652, 64284, 48212, 10796, 5952,  17624, 65404,
                55676, 33720, 16944, 49132, 55000, 26984, 35664, 47984, 11556,
                45608, 8012,
            },
            {
                17088, 21760, 34884, 44908, 61528, 57748, 42568, 38104, 11180,
                32072, 40992, 54176, 30332, 46984, 20484, 40012, 31476, 51332,
                17032, 58696, 28732, 63044, 18904, 35356, 6140,  15752, 38264,
                53624, 3980,  24800, 39952, 6292,  27668, 32204, 16816, 49136,
                39412, 58376, 44120, 24324, 12040, 41672, 2396,  26520, 7368,
                42896, 900,   46672, 59008, 53312, 12784, 49000, 28588, 45496,
                63596, 21140, 2632,  48660, 6880,  55304, 50660, 640,   39372,
                45924, 56640, 41992, 30840, 12904, 43800, 27436, 64460, 3788,
                32216, 37188, 56508, 14196, 42528, 62176, 53816, 26016, 47496,
                43028, 15568, 59500, 48292, 8248,  32588, 10808, 16300, 63508,
                55776, 18348, 10496, 52644, 30152, 56912, 36664, 22996, 28828,
                16076, 38996, 18976, 56472, 9324,  34416, 65224, 72,    32024,
                15796, 59084, 43844, 7512,  40336, 23108, 53868, 41972, 25344,
                11448, 7048,  23400, 36696, 9048,  19392, 816,   60016, 31052,
                56896, 36280,
            },
            {
                64760, 1760,  52072, 39808, 2844,  32908, 4604,  24532, 60636,
                1452,  27024, 5756,  16992, 59652, 9280,  61940, 3380,  44392,
                7712,  33312, 292,   14424, 49392, 41932, 59428, 25584, 12660,
                28752, 8332,  63288, 34348, 48188, 1772,  64380, 44588, 9420,
                30512, 820,   18076, 64616, 46836, 20708, 35248, 49520, 13840,
                56696, 28308, 40024, 23132, 7996,  38312, 42204, 19840, 7548,
                11192, 54088, 39508, 29192, 18100, 44696, 25936, 19556, 63880,
                4488,  11160, 34480, 47716, 62008, 40,    38168, 22060, 42336,
                19620, 47132, 26840, 6636,  35904, 2336,  32616, 7468,  13108,
                22020, 33884, 2076,  17792, 29336, 42808, 21816, 47116, 2636,
                7236,  33744, 45956, 38832, 20976, 11804, 6452,  42224, 50924,
                3436,  60812, 48128, 42984, 4728,  29432, 51044, 7696,  23648,
                38960, 27744, 1844,  31636, 13064, 60736, 29588, 388,   50684,
                39096, 62156, 44816, 58412, 29404, 39816, 53484, 15244, 25000,
                3724,  42464,
            },
            {
                54348, 14060, 30040, 23556, 15604, 50712, 19872, 54464, 44080,
                47388, 58676, 35912, 51624, 23988, 37576, 14876, 55584, 24804,
                64948, 40708, 56856, 26724, 8492,  31968, 3180,  51736, 46596,
                57696, 41392, 50612, 14528, 58576, 19480, 37052, 54160, 23156,
                60908, 14368, 54760, 33376, 4240,  53372, 16424, 61836, 23440,
                36864, 20116, 62512, 14376, 34552, 2524,  61420, 57908, 51800,
                35216, 43040, 57212, 23992, 59884, 37664, 12392, 31992, 53424,
                28404, 21860, 51884, 25028, 8440,  17092, 29988, 58232, 50376,
                12372, 1164,  59820, 52108, 20944, 17188, 49016, 40740, 65400,
                3668,  57228, 39576, 26396, 51252, 4568,  60288, 38248, 27944,
                50760, 23564, 59252, 1988,  26444, 48564, 32284, 59992, 14128,
                26736, 35744, 11576, 22296, 61760, 40124, 17804, 44992, 60456,
                11596, 53620, 19132, 51016, 56884, 16336, 36336, 46456, 19976,
                30636, 15412, 52000, 3124,  13204, 63852, 43524, 21828, 51120,
                9444,  27532,
            },
            {
                32544, 48148, 10312, 56536, 64104, 36808, 12192, 29784, 8344,
                15388, 21988, 12376, 42648, 2552,  28396, 49104, 19360, 35708,
                11136, 48064, 21472, 37364, 54492, 62032, 23420, 19656, 33436,
                660,   17416, 22552, 29380, 43656, 11284, 26096, 3660,  41504,
                50920, 27824, 38084, 10092, 25964, 39228, 6200,  30732, 45888,
                9812,  51960, 5768,  49332, 55988, 30644, 15840, 23716, 3264,
                27268, 15160, 5136,  9576,  34252, 2376,  62200, 41552, 6528,
                58548, 15712, 2148,  37272, 57244, 53844, 46172, 6068,  35220,
                62520, 23856, 39516, 44364, 29348, 60696, 37744, 23676, 51544,
                30008, 46120, 11748, 63588, 54548, 14284, 34320, 56832, 12388,
                42520, 31092, 14976, 54924, 43300, 64096, 17108, 53740, 39852,
                8740,  55496, 32884, 1344,  52032, 14044, 27284, 55748, 35144,
                5588,  63676, 45568, 24948, 33816, 3108,  63424, 9608,  59220,
                5072,  35352, 21136, 25788, 47412, 32320, 4624,  34804, 61544,
                39120, 19852,
            },
            {
                5892,  60332, 40896, 7256,  45640, 27116, 48360, 62820, 33792,
                39444, 53232, 65260, 32788, 45552, 57092, 5904,  41788, 30228,
                2888,  17512, 52252, 4864,  43776, 16560, 10668, 36980, 65424,
                44652, 10272, 61172, 38568, 6892,  51740, 62524, 15892, 33692,
                6060,  45352, 21652, 58852, 48124, 62996, 43564, 54600, 1556,
                59940, 33024, 17444, 43952, 26864, 9400,  46556, 39976, 32448,
                49468, 19276, 65428, 52552, 46512, 16344, 50040, 23124, 36380,
                48596, 44212, 62708, 40932, 19980, 33592, 14644, 26180, 9688,
                53328, 31504, 15332, 8584,  56924, 11288, 4888,  15068, 55468,
                8996,  19072, 36344, 6092,  20564, 40628, 24780, 80,    65000,
                20036, 8676,  61476, 4924,  35028, 9560,  24436, 1092,  20656,
                44904, 63384, 24612, 46580, 57348, 36576, 9916,  20428, 48108,
                30488, 15020, 40840, 9180,  48980, 21712, 42868, 27456, 13788,
                48664, 56620, 42028, 10028, 54392, 18384, 7844,  48808, 12728,
                56448, 45908,
            },
            {
                36728, 25824, 18116, 34288, 21412, 400,   6152,  55680, 18088,
                2368,  26368, 6752,  19080, 10080, 62640, 13188, 52860, 61208,
                45128, 59388, 25124, 12508, 29840, 47340, 58888, 40576, 5796,
                27108, 54528, 35308, 2440,  56624, 31652, 46932, 20460, 58000,
                12112, 63744, 3028,  31528, 13944, 8504,  18540, 28060, 12732,
                41260, 25384, 3832,  63824, 37948, 21560, 53192, 63224, 12468,
                58932, 44568, 31144, 38652, 21532, 27936, 55896, 8200,  11696,
                18884, 32532, 5788,  28968, 10844, 49992, 4148,  60236, 41676,
                17824, 48392, 2672,  34720, 46268, 26160, 63088, 42624, 34940,
                27808, 60180, 43660, 49904, 30460, 8780,  48036, 17476, 53440,
                36084, 49244, 39608, 28416, 19160, 46108, 38028, 58964, 29932,
                50236, 5028,  18572, 15444, 30744, 6612,  64332, 2212,  42428,
                23052, 3624,  37108, 60020, 5920,  52764, 37976, 55192, 23764,
                32548, 63072, 992,   28184, 59720, 36948, 62728, 25364, 29904,
                16536, 1212,
            },
            {
                50092, 62440, 12604, 53068, 43192, 60084, 37968, 23704, 42240,
                50060, 57588, 30888, 48256, 39668, 23072, 34420, 26788, 21116,
                9000,  32744, 39520, 64120, 35272, 1320,  26252, 53220, 14960,
                31116, 47420, 19192, 25044, 13416, 40224, 8852,  28740, 36492,
                48892, 40636, 18804, 52068, 35752, 24492, 57812, 37600, 21680,
                48848, 55284, 35820, 11316, 58632, 732,   17952, 5472,  36032,
                8048,  25532, 1652,  12056, 60860, 4268,  43416, 30344, 64604,
                25568, 52512, 14100, 59292, 23844, 65320, 44892, 21424, 38428,
                28776, 63888, 55272, 22784, 18784, 50304, 31588, 364,   20892,
                48084, 4476,  15860, 23812, 56104, 62492, 44276, 32348, 26144,
                5632,  45092, 22856, 53176, 13164, 56596, 49120, 6224,  34264,
                12816, 42016, 28468, 59180, 44056, 38936, 50116, 26080, 58296,
                51432, 61812, 18696, 26504, 31124, 11832, 17288, 3988,  44660,
                7452,  17880, 39620, 45688, 14796, 22344, 41140, 3260,  43916,
                58996, 22924,
            },
            {
                41944, 29252, 2896,  57308, 31432, 16572, 13488, 29204, 61504,
                11268, 36024, 14516, 59852, 484,   51504, 16940, 38376, 1764,
                57296, 15512, 50484, 7644,  56112, 20044, 49976, 8712,  22248,
                60576, 3416,  50364, 63768, 45164, 22224, 60260, 1200,  55796,
                24172, 7156,  27276, 55436, 64,    42084, 50336, 5044,  65360,
                31232, 15212, 19736, 47124, 30012, 50600, 42828, 27772, 54612,
                22516, 41180, 56312, 47600, 33812, 14408, 39820, 59116, 720,
                45840, 38644, 55300, 3296,  42360, 35776, 30316, 788,   57080,
                7104,  12632, 42408, 4200,  59536, 40068, 7624,  58672, 52748,
                12408, 64488, 33136, 1496,  37248, 13448, 3976,  60052, 11368,
                57892, 16000, 1800,  63792, 31888, 3820,  27204, 15652, 65308,
                52580, 37244, 1992,  53688, 11172, 21928, 32644, 13024, 16932,
                34540, 10300, 54976, 46852, 41980, 57028, 65008, 29480, 50496,
                36080, 12104, 53124, 31152, 4940,  49912, 56016, 11104, 52576,
                32764, 8352,
            },
            {
                54644, 20320, 46280, 9736,  24476, 48692, 52404, 8032,  46068,
                3528,  20184, 44288, 24460, 29092, 56180, 4932,  64676, 47928,
                42512, 27632, 3728,  23004, 41784, 13804, 33040, 44284, 56876,
                38404, 11504, 41492, 16180, 32848, 5008,  52316, 17868, 43004,
                14816, 33140, 61220, 44576, 16192, 60212, 33464, 10992, 44968,
                2028,  61280, 7620,  41776, 24296, 13920, 32752, 59756, 46004,
                15004, 62624, 18236, 6856,  26472, 54264, 20128, 50860, 16516,
                34448, 9472,  20864, 48048, 17216, 8008,  54352, 15904, 46504,
                24964, 36904, 50908, 27428, 13400, 36328, 16356, 24528, 45064,
                38636, 26576, 41404, 59024, 52220, 28672, 22168, 38200, 41576,
                29924, 51724, 34344, 10100, 41940, 60284, 21216, 44108, 24044,
                9296,  20096, 61928, 25064, 5804,  60704, 46688, 54364, 7588,
                40216, 29400, 440,   13992, 33580, 21128, 1784,  39900, 60544,
                20636, 57692, 24568, 64560, 9040,  34276, 27340, 17304, 38108,
                65380, 13864,
            },
            {
                4788,  38884, 63864, 35736, 5328,  40504, 63388, 33336, 22452,
                39252, 53588, 64084, 7684,  41472, 12656, 45480, 31220, 9984,
                54344, 35988, 18756, 46440, 59976, 28728, 62152, 2168,  17600,
                34432, 28276, 7308,  54932, 37000, 25916, 47484, 30968, 65164,
                9732,  50152, 38344, 11568, 22440, 6788,  28852, 17728, 53352,
                25780, 39468, 51416, 57112, 5980,  64728, 38480, 10656, 3808,
                49752, 29856, 37484, 51948, 64056, 9908,  36556, 24380, 5876,
                56548, 27620, 63204, 31756, 25772, 51404, 11264, 63300, 33980,
                20168, 60472, 9840,  32652, 65048, 48296, 54468, 30112, 9480,
                17916, 55624, 7176,  20348, 9884,  17012, 46772, 54816, 7976,
                19660, 62408, 24900, 48248, 17884, 36228, 53880, 39772, 6984,
                56708, 33064, 47644, 40512, 35428, 17760, 1284,  27912, 63196,
                44712, 23932, 49056, 62800, 7964,  25220, 46164, 15312, 10076,
                27228, 47580, 2020,  38272, 20068, 58428, 46292, 172,   21428,
                47756, 26400,
            },
            {
                17732, 32020, 51388, 13152, 27420, 58268, 2136,  17280, 56080,
                27756, 5652,  31560, 50956, 18168, 37176, 60516, 20444, 24240,
                13056, 63120, 30816, 51888, 10792, 37488, 5940,  48724, 24200,
                64672, 52092, 21120, 58840, 10088, 61552, 12636, 39248, 2760,
                53924, 19944, 4580,  30324, 64192, 47244, 40456, 56888, 21200,
                35612, 9784,  28220, 33952, 22028, 2644,  53760, 26136, 20284,
                34676, 9188,  228,   21832, 43876, 31464, 2720,  48632, 62068,
                41000, 12648, 43732, 1912,  38040, 58108, 40672, 28008, 48804,
                5676,  53884, 17436, 45276, 1116,  21660, 5532,  61668, 35452,
                1932,  50280, 31172, 47724, 43100, 34896, 64396, 868,   49708,
                13660, 36924, 43996, 6532,  29436, 500,   11280, 61464, 30524,
                45860, 16576, 13180, 51468, 30000, 56064, 42992, 21088, 35872,
                4544,  57220, 19756, 37748, 51760, 59484, 35488, 53612, 6368,
                42924, 33236, 16696, 51704, 43576, 13160, 62060, 30712, 6084,
                34656, 60932,
            },
            {
                44768, 1480,  59764, 22708, 43932, 19456, 36876, 11752, 47660,
                60552, 15300, 35060, 10268, 48028, 26260, 6216,  34756, 50800,
                6828,  40180, 58168, 144,   16548, 25456, 53416, 40404, 13424,
                43144, 4364,  45960, 30140, 396,   42348, 20584, 56892, 28048,
                35048, 45740, 25404, 57724, 51408, 36432, 13464, 2736,  62860,
                48312, 14464, 60528, 18132, 44308, 47800, 16252, 61268, 40376,
                56972, 45008, 60252, 15780, 40488, 58076, 13468, 45112, 29452,
                22196, 6920,  50148, 60156, 19296, 4384,  22444, 14068, 43456,
                2268,  39600, 29728, 56260, 25728, 43668, 12148, 41212, 57820,
                22940, 62896, 14388, 25324, 60948, 6288,  23956, 32592, 27548,
                60480, 3072,  21508, 55984, 59208, 49576, 14660, 25520, 51020,
                2728,  63020, 23152, 3912,  8696,  64416, 14572, 49836, 11900,
                52616, 16068, 31600, 10996, 43380, 4872,  18044, 31008, 63844,
                55708, 4176,  61596, 29732, 6732,  25032, 40300, 50468, 56652,
                41560, 10896,
            },
            {
                49408, 28556, 15844, 55240, 8284,  49968, 30056, 41452, 25424,
                800,   43500, 57224, 22880, 61872, 54696, 1080,  58500, 44452,
                15140, 26616, 5180,  44084, 33968, 63416, 19812, 31580, 9516,
                58052, 26716, 14392, 49432, 33972, 16916, 51256, 6364,  23364,
                15856, 62232, 40920, 1656,  18264, 8976,  24348, 43972, 31572,
                6172,  42396, 432,   52948, 11608, 36404, 31508, 7340,  51536,
                13216, 24012, 32852, 50444, 27976, 4700,  19184, 60088, 34124,
                17080, 53916, 36056, 14896, 46192, 33444, 61952, 9100,  52740,
                24480, 62652, 14416, 7728,  37584, 51320, 33296, 15664, 28176,
                45820, 37768, 3684,  53820, 39312, 12492, 57604, 44788, 15704,
                40312, 53264, 11848, 26548, 38592, 45280, 19796, 34852, 5984,
                38112, 28508, 41868, 59436, 26392, 38604, 32920, 6548,  60360,
                25592, 39440, 2656,  58264, 22960, 28504, 49216, 38724, 21504,
                13768, 40660, 22932, 11008, 54780, 36168, 18876, 8776,  15032,
                53156, 24724,
            },
            {
                37156, 6844,  41020, 34600, 62480, 4664,  54104, 64668, 8968,
                51648, 19708, 38348, 3276,  13752, 32132, 39528, 17260, 29412,
                62132, 48192, 22208, 38392, 57008, 7532,  45324, 55348, 2784,
                36732, 18780, 62292, 38924, 22692, 64276, 44948, 36572, 59196,
                48324, 8436,  13236, 33380, 55036, 28164, 49556, 59772, 17200,
                54636, 23468, 38256, 29788, 57780, 62876, 22944, 43560, 2264,
                28944, 63492, 6000,  10924, 55700, 24708, 37588, 53132, 10036,
                932,   64436, 30384, 24580, 7760,  54992, 29108, 37396, 57736,
                31928, 21208, 49760, 58444, 18772, 62328, 2520,  53096, 19920,
                7508,  11064, 33560, 19104, 28816, 2708,  51480, 20304, 9388,
                47384, 33736, 65084, 16540, 7804,  32360, 63272, 56340, 43216,
                17684, 57664, 11688, 48708, 19156, 53568, 22560, 41332, 45964,
                29656, 65420, 47764, 34196, 14428, 63092, 1120,  11260, 26168,
                47188, 34576, 50252, 59004, 45504, 2952,  63904, 29000, 22756,
                2516,  59216,
            },
            {
                64724, 21236, 11860, 46200, 31640, 24836, 14656, 21456, 34128,
                45772, 30496, 63448, 28092, 42616, 46700, 21364, 52928, 7700,
                36392, 55536, 18556, 12380, 49864, 29568, 14964, 22408, 61156,
                50952, 32708, 6940,  55932, 3052,  9076,  29540, 12020, 3904,
                30880, 56024, 43652, 21668, 63532, 39308, 4676,  34692, 10644,
                27400, 64640, 49916, 19848, 4536,  9056,  26980, 55488, 19096,
                38824, 48080, 17384, 35860, 46340, 65248, 7828,  49720, 39872,
                26884, 42996, 57236, 11324, 49256, 41508, 212,   16460, 45872,
                11436, 35472, 4288,  41540, 31004, 9644,  24824, 60160, 48600,
                56404, 43292, 65376, 48144, 59520, 41876, 36452, 30908, 55328,
                22868, 5912,  29756, 42756, 52524, 23232, 2016,  10408, 52932,
                21864, 32712, 612,   45332, 30616, 4588,  10684, 58472, 928,
                18404, 9292,  54396, 7140,  40856, 55972, 45024, 52280, 61096,
                7920,  17876, 632,   26688, 15784, 32648, 48504, 57464, 38932,
                33180, 18484,
            },
            {
                30304, 52168, 58068, 452,   18296, 48368, 38876, 3664,  57964,
                12836, 6644,  16584, 53600, 9440,  4632,  64556, 25228, 11552,
                41720, 3256,  32608, 65148, 24688, 4760,  39460, 47592, 27464,
                41688, 10652, 24764, 43512, 53136, 26512, 60320, 40220, 51788,
                19652, 37172, 26304, 5752,  46112, 15468, 53184, 20520, 40344,
                46804, 7988,  33760, 15372, 45896, 40844, 49640, 14752, 34560,
                59112, 3536,  53904, 41308, 20640, 1468,  31680, 14868, 22928,
                47416, 18736, 5408,  38276, 21060, 63976, 26556, 51244, 19524,
                6924,  64796, 47572, 27148, 13544, 46788, 35800, 4720,  39088,
                31388, 26108, 15152, 5156,  23340, 17196, 7936,  63812, 1184,
                58804, 38140, 50140, 3496,  59932, 13512, 40468, 27832, 46488,
                7060,  65056, 36144, 55096, 16228, 62760, 34832, 49368, 14940,
                37356, 23784, 43928, 20476, 26936, 17296, 31808, 6072,  36276,
                57804, 30080, 65200, 37884, 41908, 21348, 11868, 6360,  43720,
                47080, 9728,
            },
            {
                5084,  42716, 26700, 37640, 61496, 9428,  43320, 60412, 26768,
                54968, 40148, 50020, 24368, 36520, 57680, 14744, 34144, 50272,
                59560, 27660, 8920,  52376, 42656, 58368, 11328, 35116, 1596,
                16276, 57048, 48172, 13336, 35812, 17984, 47316, 15024, 24420,
                63772, 572,   49936, 59572, 11604, 32224, 1132,  61992, 57840,
                13748, 2984,  56428, 25380, 61120, 32128, 1236,  64928, 10660,
                44524, 22572, 26532, 12676, 59612, 28408, 43624, 57684, 62428,
                3628,  33248, 51752, 59072, 15668, 34796, 4752,  61252, 42532,
                24972, 38992, 2008,  55220, 22460, 63640, 42784, 16612, 12304,
                21612, 148,   52020, 35500, 55600, 45596, 13048, 26744, 43404,
                17780, 11308, 25468, 20164, 35624, 48104, 30652, 58252, 15520,
                24984, 50184, 13848, 8420,  39420, 24484, 44024, 27448, 56332,
                32404, 51664, 61272, 3156,  35640, 59548, 12972, 24140, 41976,
                20360, 44532, 9752,  53052, 5244,  60712, 54032, 27264, 62444,
                14072, 55556,
            },
            {
                16796, 49076, 22580, 13668, 53872, 29064, 51104, 16888, 36240,
                22744, 2884,  32596, 60720, 19116, 48668, 30196, 44016, 1892,
                19560, 46980, 14640, 35964, 828,   17840, 61600, 53684, 20748,
                64744, 30388, 3984,  21476, 63296, 32184, 2192,  54572, 33584,
                10604, 41256, 17204, 29444, 51968, 38136, 25604, 44632, 30032,
                22852, 36836, 42732, 52488, 11884, 21968, 38020, 52828, 29396,
                6684,  32572, 62508, 8044,  51268, 34652, 17992, 10620, 36396,
                55376, 13016, 28204, 44484, 9268,  47812, 30448, 12776, 55764,
                32488, 59836, 17688, 52084, 34012, 7304,  57448, 28632, 49688,
                62544, 59068, 40500, 10360, 31760, 62196, 49228, 39572, 53764,
                34460, 61980, 45304, 56600, 9504,  64044, 18400, 4076,  37868,
                61324, 41588, 29076, 21012, 60608, 52228, 2144,  19948, 63868,
                4428,  12312, 28880, 49080, 9672,  53776, 39684, 50704, 2312,
                15384, 54720, 25180, 13856, 49532, 30664, 17376, 36532, 852,
                24892, 35320,
            },
            {
                39368, 3324,  63636, 34060, 6272,  20276, 1896,  31804, 10484,
                47996, 62608, 44204, 11352, 1384,  40464, 6248,  62564, 23348,
                54228, 39656, 60788, 21764, 29832, 49100, 25392, 32312, 6020,
                45384, 37048, 59396, 39960, 50828, 6228,  42752, 57884, 7636,
                46624, 61056, 35132, 6928,  22448, 56804, 8924,  16484, 4972,
                50136, 63412, 17976, 29160, 6800,  58324, 16640, 42832, 20500,
                57580, 49364, 37712, 16144, 45356, 2924,  53744, 25664, 6864,
                41692, 21844, 63148, 2876,  24816, 54108, 40156, 22336, 8628,
                48992, 14800, 29572, 10232, 45148, 1020,  19780, 53236, 36792,
                8500,  44032, 28192, 19684, 5880,  25076, 2316,  21024, 30164,
                4028,  14648, 28324, 39848, 100,   24624, 44208, 54740, 32484,
                11168, 1636,  56732, 44856, 5852,  31944, 12760, 41916, 8680,
                47184, 38588, 16408, 42340, 64620, 22104, 4832,  28296, 63484,
                47512, 32140, 3140,  35128, 22476, 45736, 9132,  41340, 20308,
                51264, 60648,
            },
            {
                28216, 45472, 10920, 57732, 41276, 46812, 56220, 64952, 42572,
                7328,  20508, 15216, 29624, 56060, 26676, 52400, 17076, 37360,
                10636, 31212, 5772,  45484, 56252, 38176, 9204,  43896, 14164,
                51808, 8684,  17208, 25560, 11456, 28248, 19772, 37848, 22700,
                27568, 14132, 55532, 42792, 18992, 64660, 48244, 41320, 35368,
                54604, 10824, 1884,  39708, 48944, 35656, 3936,  25768, 46948,
                13664, 392,   55800, 29804, 23208, 39968, 61216, 48772, 30440,
                46388, 17084, 37656, 31788, 60500, 18240, 1204,  58824, 36172,
                5720,  43768, 21716, 38100, 60820, 26604, 40888, 32200, 3828,
                24284, 17352, 47472, 57336, 37944, 51240, 15956, 58044, 9680,
                47356, 51072, 5816,  54172, 16320, 33700, 51336, 6148,  22228,
                47180, 18672, 34300, 25668, 49464, 17104, 55652, 35460, 22888,
                58940, 52796, 25256, 192,   30396, 46536, 18072, 33776, 56284,
                11484, 38464, 62344, 57096, 39844, 4044,  63936, 56020, 47696,
                31468, 7864,
            },
            {
                19016, 52736, 31900, 16044, 25660, 37276, 23216, 14336, 28036,
                53112, 38024, 58664, 34632, 46364, 22312, 12664, 33104, 7740,
                57776, 24916, 63924, 16948, 12644, 3928,  63124, 40356, 55828,
                23636, 29052, 47156, 84,    54112, 61716, 45816, 16120, 65300,
                31492, 4300,  48972, 25288, 1788,  33648, 12896, 61320, 26480,
                20368, 32688, 59056, 45180, 23500, 62024, 53780, 9556,  63776,
                35120, 18536, 41464, 10340, 64180, 5640,  20232, 12228, 58496,
                1668,  52752, 7704,  50004, 42940, 35292, 14172, 45512, 51588,
                25840, 63780, 3516,  56384, 12884, 48208, 15728, 64848, 11520,
                55112, 34208, 1632,  64108, 12072, 32948, 44436, 61256, 35584,
                23776, 64868, 31972, 21228, 60392, 41948, 12792, 62852, 27488,
                58972, 52624, 4600,  63644, 10148, 38132, 59876, 45984, 30192,
                5712,  19128, 34500, 57792, 13428, 37528, 60780, 9044,  43408,
                21536, 27356, 7376,  18840, 50532, 28580, 14668, 23732, 2688,
                11788, 57132,
            },
            {
                23032, 41936, 2360,  60956, 8368,  50628, 4140,  34388, 57560,
                188,   25588, 49616, 5000,  8944,  65252, 41592, 60464, 47368,
                50564, 41016, 2232,  34372, 53464, 22720, 27672, 18768, 2068,
                34640, 62792, 42248, 56976, 30900, 35288, 8804,  50036, 1440,
                53188, 39100, 10884, 59944, 46032, 30356, 3572,  51844, 6496,
                46868, 14784, 27760, 8320,  19340, 12932, 31148, 40128, 27860,
                5896,  60136, 25012, 48096, 52684, 14532, 32880, 38696, 23488,
                64532, 27016, 14760, 57468, 10828, 23616, 65488, 28560, 19616,
                54440, 40372, 30808, 50424, 35592, 23860, 7928,  57900, 45372,
                38240, 50456, 14836, 23200, 41896, 54556, 27460, 6600,  39260,
                18716, 42884, 11648, 37308, 48344, 29080, 8832,  39128, 15864,
                43304, 36448, 14136, 40584, 29292, 21304, 2536,  26284, 14316,
                40796, 62836, 10396, 44508, 54168, 6468,  50204, 25036, 1628,
                52648, 16296, 59688, 43700, 11040, 33460, 59000, 38316, 43964,
                34788, 64912,
            },
            {
                5636,  37500, 49236, 27888, 18276, 63116, 12272, 46100, 18908,
                41132, 10804, 62924, 17492, 37160, 53700, 19800, 716,   27260,
                14228, 18968, 28972, 42760, 47964, 58424, 31884, 50264, 60228,
                11904, 19808, 6764,  15088, 22236, 41040, 13232, 25056, 43368,
                58408, 20788, 36028, 15228, 53860, 40260, 17620, 38196, 23384,
                56028, 43392, 65484, 51508, 34412, 56500, 1684,  44492, 50916,
                21684, 54052, 31548, 3304,  36260, 27312, 56072, 44188, 9164,
                34772, 41060, 20648, 29792, 6264,  48796, 4196,  32956, 11664,
                2276,  16492, 8828,  19176, 62304, 4844,  42232, 30096, 20772,
                27000, 6640,  59696, 30608, 8704,  3556,  20588, 13396, 55308,
                1008,  58736, 26536, 7720,  2796,  57428, 19604, 49644, 2324,
                30856, 8388,  23584, 60928, 47612, 53676, 7184,  64824, 50024,
                55148, 2868,  22416, 27300, 40076, 20676, 15000, 36120, 58332,
                40768, 31284, 47292, 328,   25996, 52500, 5988,  17988, 54560,
                26964, 15352,
            },
            {
                59804, 12992, 56460, 34900, 44104, 54416, 39804, 29364, 60176,
                51836, 31916, 23608, 43372, 28448, 3732,  31516, 44840, 36188,
                6460,  56772, 52096, 10836, 7504,  37072, 14872, 4748,  45928,
                38076, 52760, 32512, 49180, 3096,  59316, 52208, 5420,  33716,
                17348, 7020,  61792, 27224, 21924, 9096,  57404, 28936, 60588,
                10180, 96,    36872, 5232,  25328, 47380, 17808, 60848, 14852,
                37512, 8972,  43264, 16900, 46312, 59416, 896,   50540, 17268,
                4848,  54256, 45292, 62712, 38752, 56156, 41960, 60664, 37020,
                47836, 59300, 34216, 45592, 28196, 53836, 17236, 49820, 2060,
                61680, 40704, 19212, 53128, 45492, 37320, 63156, 50156, 29496,
                46652, 15620, 52788, 63256, 44828, 23548, 35924, 54024, 25976,
                64656, 56920, 50688, 352,   15748, 32668, 42912, 11620, 18136,
                36880, 30852, 48032, 61736, 4064,  32336, 64028, 28648, 48152,
                5476,  13300, 35012, 65068, 21212, 41852, 62056, 30736, 9548,
                50076, 45804,
            },
            {
                31180, 25052, 9796,  21196, 1388,  31564, 5152,  22080, 8140,
                15720, 4008,  47584, 12796, 58028, 50288, 15560, 59616, 23668,
                64292, 38372, 21392, 61436, 25812, 17832, 65408, 42024, 9568,
                26740, 57836, 24108, 64300, 36672, 19064, 27388, 62372, 47376,
                29708, 41728, 50784, 4628,  44044, 62956, 49176, 13656, 33512,
                19608, 40828, 30312, 15920, 62376, 38768, 10504, 24136, 33256,
                4520,  57652, 62528, 12048, 22748, 7680,  19740, 29240, 61692,
                47844, 32012, 11920, 340,   25744, 16416, 20928, 8476,  24672,
                55548, 42468, 22732, 11960, 324,   58556, 36884, 33076, 13648,
                55832, 10720, 33820, 25796, 48384, 16092, 57088, 24376, 32700,
                8936,  40492, 34484, 18176, 31028, 14292, 61416, 5464,  46400,
                20812, 12340, 33936, 45132, 19364, 57704, 28132, 35064, 24004,
                45536, 8576,  16212, 51680, 11820, 55196, 45264, 9632,  17504,
                61612, 24180, 54200, 8464,  37436, 12936, 47788, 3596,  21976,
                39592, 752,
            },
            {
                53116, 43280, 61604, 47880, 15084, 58960, 25464, 49296, 63728,
                38380, 56532, 35316, 61428, 21444, 39052, 7420,  54788, 11496,
                30580, 3004,  44232, 32980, 548,   46972, 54472, 22264, 30720,
                14124, 1040,  43820, 8548,  46464, 11252, 39476, 54756, 21620,
                9404,  56744, 13072, 31292, 34512, 1296,  24660, 6124,  45268,
                50476, 54324, 21904, 58348, 42380, 7436,  29416, 55312, 49108,
                41152, 19528, 28156, 51684, 39424, 65264, 42104, 35712, 13816,
                24424, 58416, 19068, 36688, 53344, 31264, 46304, 52160, 15008,
                29408, 6396,  63472, 52600, 39196, 24908, 9456,  63988, 46872,
                23792, 44144, 5216,  65228, 408,   35004, 10608, 42592, 2972,
                62012, 21916, 49188, 6180,  56272, 39716, 10784, 32684, 16984,
                41656, 37760, 6688,  26648, 39700, 9544,  62264, 51792, 1316,
                61008, 58276, 39040, 34092, 19088, 41888, 23272, 3088,  56364,
                38888, 20200, 44004, 29600, 56844, 16648, 25200, 35648, 57260,
                63276, 19448,
            },
            {
                34400, 16524, 38812, 6580,  52744, 36036, 42644, 11856, 32996,
                44560, 19348, 26176, 1036,  9836,  33772, 25720, 48156, 42052,
                17568, 49664, 14000, 55744, 40600, 29180, 6456,  35620, 59892,
                48448, 39624, 17192, 29320, 55544, 4212,  32100, 14828, 736,
                37324, 24196, 63804, 47768, 18812, 55124, 42352, 37060, 64240,
                3872,  12472, 26900, 48680, 2768,  52652, 20884, 64732, 616,
                13156, 45644, 35244, 2404,  25496, 31868, 53596, 10032, 56796,
                2364,  39688, 8160,  49508, 61404, 12940, 3128,  64328, 38384,
                1484,  19844, 32368, 14440, 48812, 43716, 20540, 6136,  28832,
                38836, 17968, 54276, 14144, 29652, 22204, 59408, 17572, 38056,
                54128, 13092, 25260, 1560,  51160, 27452, 42972, 58832, 1324,
                52856, 29616, 55840, 60140, 3544,  46664, 21776, 5316,  41532,
                14264, 28528, 6284,  25336, 620,   59044, 29888, 50880, 34244,
                26824, 10776, 4884,  49660, 2132,  60952, 44940, 51780, 11248,
                28328, 7544,
            },
            {
                55720, 3476,  27516, 22648, 65512, 8732,  17224, 55164, 1880,
                28244, 6504,  54008, 41744, 52192, 46224, 62420, 20224, 5516,
                35456, 62784, 8224,  24112, 60540, 11124, 50084, 18612, 3392,
                53420, 63168, 21300, 34996, 61036, 25636, 43080, 64900, 51204,
                45764, 16848, 3456,  39304, 10332, 58792, 14892, 27444, 17524,
                32096, 61336, 8144,  35204, 14536, 32356, 44952, 36716, 26264,
                59588, 30648, 7144,  55952, 48720, 15364, 4244,  46892, 26844,
                43728, 64172, 22392, 28708, 5048,  41024, 34524, 26820, 44068,
                50652, 60936, 36536, 27096, 3844,  60380, 15988, 57352, 51876,
                2988,  60008, 36496, 50464, 41264, 7660,  49264, 45220, 5524,
                28772, 58124, 43816, 36456, 65520, 8196,  22072, 47212, 25164,
                63284, 9792,  15536, 22876, 51080, 13032, 37412, 55392, 32444,
                19932, 49592, 43140, 53244, 65516, 37468, 8124,  13776, 62920,
                46212, 52960, 59656, 32916, 19520, 39236, 6432,  31644, 15460,
                41304, 48608,
            },
            {
                12384, 62908, 46344, 33128, 40996, 29768, 50112, 20728, 61644,
                47152, 14592, 64852, 30640, 17248, 13376, 2512,  29032, 57672,
                51528, 26540, 45880, 20012, 37672, 15688, 57104, 42776, 23672,
                32628, 11476, 5700,  51060, 13532, 48816, 18252, 7608,  28720,
                34212, 60312, 26672, 53044, 21220, 30160, 7716,  47424, 52728,
                22800, 38624, 56096, 46020, 19092, 59968, 11776, 5604,  16956,
                52104, 21984, 63052, 18152, 11336, 60524, 22096, 38120, 17736,
                52448, 33672, 14624, 45760, 56076, 17996, 58836, 22092, 11540,
                56728, 7784,  46496, 10380, 54632, 30804, 35268, 41620, 12700,
                32240, 8952,  27164, 20064, 62300, 55188, 31552, 25912, 64296,
                19376, 10000, 47708, 30428, 20024, 12652, 52260, 34964, 4572,
                19172, 48612, 44128, 35256, 65116, 30280, 17212, 26020, 63756,
                10068, 57016, 22172, 12628, 47248, 16960, 44700, 21576, 40564,
                1004,  16288, 36896, 12668, 27844, 54916, 23056, 64508, 1740,
                58488, 23968,
            },
            {
                30872, 20228, 51832, 456,   13476, 57100, 5300,  36824, 24340,
                40208, 8456,  49396, 37220, 24212, 55716, 40700, 32892, 10628,
                38972, 3348,  30372, 53388, 4716,  33680, 26636, 61556, 8416,
                46328, 28160, 42048, 57524, 1376,  38180, 22600, 58080, 40864,
                11912, 55608, 6900,  43900, 35940, 62412, 40684, 2840,  59744,
                10700, 43136, 876,   24852, 62936, 40940, 28200, 54312, 39116,
                9276,  47588, 41568, 37080, 44148, 33612, 57740, 7280,  30800,
                3492,  11148, 60216, 36308, 24976, 9744,  47744, 5760,  30672,
                16828, 24260, 41816, 18700, 65124, 22428, 1260,  25544, 48696,
                63420, 22668, 47176, 2024,  11256, 37800, 3648,  13760, 50988,
                33084, 40668, 15928, 60344, 3776,  38848, 61804, 15164, 57460,
                31624, 38504, 6372,  27372, 1124,  42288, 53360, 6888,  47552,
                38128, 1888,  31104, 35784, 4388,  26224, 32520, 55336, 6540,
                31276, 24868, 63884, 41740, 48444, 9476,  43388, 34532, 18956,
                45656, 37228,
            },
            {
                4316,  42708, 9992,  59496, 18836, 26204, 44172, 60040, 10820,
                33900, 57520, 22148, 4164,  59580, 7284,  44972, 64520, 22636,
                16024, 60152, 12844, 43428, 64312, 47816, 1308,  39320, 14556,
                55656, 37044, 16856, 24400, 33156, 45412, 10452, 52892, 2380,
                19704, 31840, 15320, 49684, 420,   24416, 50864, 19928, 34944,
                28784, 16172, 49600, 30848, 9628,  3448,  48828, 23620, 57496,
                34028, 14452, 4708,  24820, 952,   27972, 49704, 63268, 41352,
                55588, 20072, 51048, 1996,  43052, 32720, 52340, 62932, 40308,
                35112, 49716, 58240, 4812,  39284, 51196, 46028, 7876,  55792,
                16608, 39584, 58876, 33748, 44792, 16820, 23568, 60972, 35768,
                904,   53240, 6816,  23936, 55200, 32504, 44964, 28472, 41096,
                11492, 54144, 60884, 20160, 49420, 10852, 59120, 34256, 15276,
                44072, 59592, 17888, 54040, 62288, 10712, 59956, 48836, 18636,
                58208, 51448, 7556,  22212, 3352,  61876, 14004, 50816, 26684,
                8932,  60656,
            },
            {
                50160, 27092, 37964, 54360, 34376, 47640, 2548,  15984, 51088,
                28704, 18580, 45432, 11968, 34332, 27720, 19076, 52980, 260,
                48044, 55592, 34980, 23296, 9800,  19252, 51848, 31480, 21016,
                64684, 3132,  52700, 59708, 7312,  62088, 27952, 35732, 63380,
                44872, 22988, 38780, 64740, 11000, 32736, 13928, 45340, 57360,
                4976,  65144, 54832, 21292, 37872, 58216, 18220, 43396, 1356,
                64356, 29020, 51040, 61176, 54640, 19804, 9220,  13892, 23880,
                47776, 27508, 40116, 7900,  65212, 15380, 19752, 628,   55260,
                13140, 2916,  28612, 33044, 14804, 11340, 37092, 19676, 29776,
                43412, 5240,  14024, 53572, 28232, 57412, 42128, 47568, 9228,
                21512, 58864, 27716, 42208, 49476, 17836, 1024,  7880,  50620,
                23816, 3204,  14084, 32868, 57156, 39812, 22412, 3100,  24572,
                28888, 8496,  40532, 23552, 50284, 42236, 1572,  38340, 28640,
                11344, 42800, 35192, 57204, 17632, 31056, 37756, 57552, 5668,
                54084, 16388,
            },
            {
                22308, 62548, 15180, 7268,  23276, 63896, 31064, 42008, 6912,
                62776, 652,   54244, 42220, 61956, 50172, 14304, 36604, 25640,
                42364, 18392, 7124,  39940, 57264, 29316, 12464, 44500, 6500,
                26120, 47448, 10872, 30284, 20756, 49612, 16432, 5764,  25360,
                48768, 4528,  57120, 28576, 18440, 54592, 61308, 8168,  25372,
                39356, 12560, 34020, 6980,  51860, 14708, 35572, 31324, 7840,
                20272, 45168, 11032, 32340, 15940, 39348, 45308, 35464, 5500,
                61620, 16168, 31872, 22164, 54732, 29272, 38484, 44572, 27208,
                61784, 21316, 44008, 62744, 53552, 26624, 60888, 57656, 3220,
                52240, 35920, 24772, 8364,  63068, 19624, 5448,  30016, 56368,
                38448, 45524, 13952, 34748, 64100, 11176, 26208, 59984, 20512,
                64524, 43232, 47164, 25316, 8040,  18092, 45460, 61264, 52456,
                48872, 63376, 13972, 5704,  28056, 34824, 15740, 22652, 64680,
                2880,  46760, 14736, 27384, 53632, 45832, 76,    21000, 29744,
                40980, 34904,
            },
            {
                1456,  31980, 44488, 49232, 39788, 11208, 56420, 21600, 53356,
                38656, 32440, 15584, 25248, 2864,  39244, 9192,  58272, 5176,
                28844, 62052, 32456, 49748, 2088,  62972, 36412, 54076, 58860,
                40616, 18324, 35140, 43612, 14176, 38736, 54704, 41488, 12504,
                60560, 35000, 8788,  46468, 41804, 4168,  37496, 29872, 53476,
                42452, 18828, 47004, 27732, 44248, 63832, 11236, 53528, 60248,
                25860, 56092, 40772, 22520, 6724,  65016, 51900, 29992, 56780,
                4,     37352, 46424, 58624, 4740,  49488, 10984, 23476, 7464,
                51296, 37688, 9408,  47628, 6132,  17640, 42180, 33636, 9756,
                64736, 20876, 50344, 31700, 568,   39136, 51672, 11984, 25500,
                17072, 62128, 2780,  8592,  22012, 38072, 53448, 46416, 35564,
                29968, 16364, 55076, 37916, 63440, 29264, 13312, 36848, 4960,
                19836, 33536, 55928, 46432, 19292, 57636, 8764,  44300, 50768,
                33604, 20280, 62892, 4232,  40028, 24648, 60064, 10252, 48936,
                65096, 12688,
            },
            {
                58508, 55128, 4948,  28692, 17628, 3584,  35816, 27196, 13348,
                47084, 22972, 58816, 48740, 29848, 21100, 63460, 31400, 46904,
                11028, 52116, 13932, 24952, 46124, 16760, 22508, 8884,  33048,
                68,    61184, 50568, 5056,  64140, 2204,  32244, 58204, 18544,
                30600, 52300, 14480, 25992, 59280, 22088, 48356, 16556, 1112,
                22844, 61948, 3084,  59272, 24980, 5208,  22384, 47364, 38440,
                16332, 3376,  36000, 58608, 47220, 3068,  24888, 17660, 42904,
                12756, 53056, 9176,  25736, 13896, 34016, 59868, 46636, 32056,
                15208, 59488, 19188, 24668, 31448, 0,     49148, 22824, 28360,
                13052, 45604, 41120, 60468, 48492, 15284, 34328, 65152, 43600,
                4412,  31784, 52008, 47464, 29472, 56936, 4696,  13480, 40280,
                1940,  9668,  34184, 5548,  50904, 280,   56452, 31396, 41520,
                11420, 25756, 37544, 2064,  30756, 52232, 61656, 25684, 13368,
                54700, 30084, 9300,  36560, 50584, 12460, 32664, 42676, 17960,
                25116, 46436,
            },
            {
                36920, 19884, 11784, 61624, 51652, 59920, 45580, 19248, 64876,
                9256,  4736,  35928, 10940, 55968, 6480,  41460, 16632, 54300,
                38288, 20760, 3696,  59340, 28168, 5276,  41236, 49996, 24792,
                15488, 29208, 21604, 56956, 26464, 47016, 22524, 9212,  44092,
                976,   21528, 40472, 2808,  50624, 34660, 12052, 64092, 44728,
                31620, 50592, 10732, 38280, 16780, 33232, 41768, 2372,  29720,
                49932, 9068,  53004, 13672, 28040, 34664, 10832, 59928, 21616,
                33456, 63152, 19264, 44196, 40448, 62404, 3192,  17668, 56316,
                35568, 1720,  43012, 57280, 39652, 63948, 54648, 15824, 38648,
                59424, 2400,  17540, 10200, 26764, 22540, 55688, 7768,  20940,
                54428, 36320, 19424, 60232, 15056, 43540, 32088, 19040, 59236,
                52392, 26896, 61484, 20412, 42628, 23508, 47336, 16472, 62472,
                58504, 7520,  43160, 64536, 12344, 39904, 3464,  36724, 6616,
                60292, 41280, 23952, 56240, 16448, 6676,  62464, 52804, 4436,
                39208, 8208,
            },
            {
                50972, 26328, 42796, 33808, 23884, 8816,  41184, 1644,  50048,
                29288, 43128, 61292, 18996, 44764, 50976, 34136, 23416, 1392,
                44348, 65436, 33472, 42920, 37592, 54736, 61896, 12912, 46516,
                55960, 38224, 7528,  41924, 12260, 36372, 51632, 28664, 62844,
                37696, 54444, 61236, 17316, 32028, 6376,  56280, 27028, 9128,
                35544, 14928, 55944, 28724, 48956, 57140, 61384, 13136, 19332,
                62612, 42696, 31444, 20084, 60856, 43904, 54820, 38332, 48060,
                6820,  27840, 50200, 1536,  36232, 20908, 52852, 25436, 41416,
                5380,  29892, 52156, 10584, 13880, 34688, 7796,  43924, 4904,
                53252, 35052, 30120, 56628, 37404, 3600,  46752, 41036, 29168,
                49380, 10280, 26652, 40164, 108,   24128, 62724, 7152,  48748,
                23160, 44796, 56716, 12900, 32164, 10244, 54000, 27560, 2456,
                21744, 50064, 54356, 17372, 24144, 47736, 20596, 31632, 49572,
                18200, 748,   47252, 59188, 44064, 22688, 35684, 27608, 56756,
                14956, 31020,
            },
            {
                3164,  64112, 15992, 38548, 55660, 31280, 14432, 58596, 33516,
                55500, 25580, 39752, 1012,  14916, 26800, 59916, 12620, 57136,
                27496, 8512,  15820, 56392, 10860, 18344, 30620, 2348,  34468,
                65036, 10472, 52608, 32904, 18032, 60028, 4596,  15564, 48320,
                7180,  26776, 46076, 10172, 62988, 42980, 20344, 39456, 60164,
                53800, 6040,  40852, 20704, 384,   9380,  51376, 35172, 26380,
                56396, 4308,  24164, 49064, 1104,  16784, 8260,  26212, 2680,
                41232, 58288, 30980, 16564, 56564, 7676,  29040, 10716, 64804,
                48316, 22472, 62348, 27512, 47020, 20600, 29644, 61688, 26012,
                46896, 23204, 7092,  64148, 43688, 12256, 32308, 61452, 1664,
                14492, 63000, 45068, 6836,  54904, 51176, 35152, 28044, 11844,
                37192, 4304,  17396, 49460, 38916, 64156, 7172,  36224, 45036,
                32428, 14088, 28836, 35040, 6024,  59096, 55396, 14608, 43628,
                28292, 11864, 33912, 4724,  29140, 49252, 19300, 1364,  47536,
                60756, 22036,
            },
            {
                53536, 9720,  47320, 300,   6772,  48176, 20396, 37956, 5728,
                16788, 12352, 53540, 31932, 64544, 37844, 4068,  48388, 18684,
                36264, 50544, 24016, 47196, 6472,  22200, 52044, 44292, 20340,
                5184,  27412, 43868, 62620, 1280,  25132, 40744, 56496, 20356,
                33068, 13036, 36248, 23000, 29132, 52472, 13716, 4344,  47492,
                17692, 23940, 45752, 63712, 30960, 43784, 23544, 6328,  40188,
                45868, 10876, 36708, 63360, 39932, 29664, 50740, 64432, 14308,
                53900, 23492, 11116, 64152, 47544, 32540, 43228, 50748, 14456,
                38672, 7584,  17168, 36972, 2384,  56700, 50936, 11144, 18352,
                58036, 14204, 50100, 20000, 52584, 16616, 58844, 24428, 18632,
                39316, 57232, 23092, 33020, 17488, 9944,  41880, 57996, 15932,
                65180, 31108, 41596, 1512,  59192, 25416, 18788, 52360, 40056,
                9108,  47104, 60940, 492,   41752, 27540, 8988,  62568, 38528,
                53428, 65132, 21032, 39072, 13720, 8864,  64308, 41408, 11500,
                33668, 44396,
            },
            {
                39448, 29996, 58916, 22628, 63212, 28344, 53224, 24716, 44580,
                62656, 22136, 47944, 7600,  19936, 52256, 9872,  42436, 30288,
                61168, 3104,  40652, 31260, 63372, 36016, 26904, 58692, 39484,
                16840, 49044, 22796, 13868, 55352, 45940, 31012, 10944, 65456,
                42764, 3920,  58476, 55400, 60,    48584, 37124, 25100, 30240,
                2080,  33976, 57872, 12780, 37340, 18564, 54896, 15316, 65452,
                32188, 17588, 53376, 6032,  12624, 57432, 20800, 31892, 36764,
                18420, 45660, 4080,  38708, 13504, 23136, 60840, 2252,  20144,
                33388, 54624, 59732, 43444, 5968,  24244, 41444, 37700, 31948,
                1332,  42440, 38964, 33644, 5656,  28684, 48920, 9024,  35176,
                52692, 4500,  12216, 38208, 64012, 46236, 21160, 1744,  47808,
                53976, 8604,  22280, 28712, 34992, 14544, 56860, 3568,  23080,
                58016, 17896, 38336, 52868, 11372, 45516, 33060, 23236, 2852,
                7736,  26104, 50388, 60680, 53696, 31864, 55636, 37564, 25600,
                6436,  18144,
            },
            {
                27328, 13240, 35932, 17464, 43356, 12108, 60144, 9092,  51520,
                2084,  34644, 59412, 28488, 45724, 23680, 33448, 62868, 6276,
                21740, 53300, 12176, 59808, 756,   49352, 9200,  14040, 53756,
                60604, 31880, 3060,  37336, 50208, 19404, 6444,  38776, 52464,
                24256, 49168, 18576, 39944, 8672,  16452, 64476, 56704, 42216,
                61084, 51896, 8268,  26852, 4680,  60688, 47300, 28916, 1564,
                57940, 22488, 47892, 26580, 33728, 45424, 3816,  42312, 9876,
                61984, 34284, 51744, 26832, 55052, 4840,  36540, 57676, 27932,
                45564, 11428, 25676, 32068, 48904, 65292, 16128, 4376,  53532,
                63520, 27348, 9332,  24072, 62572, 792,   40528, 56476, 44244,
                27748, 31328, 47872, 58732, 27156, 5092,  30588, 39152, 25148,
                33588, 43968, 60692, 51604, 5800,  45712, 42160, 30412, 12016,
                65324, 5312,  25556, 20712, 63648, 15968, 51076, 19048, 58292,
                35232, 45104, 16804, 1676,  42096, 23768, 5360,  15944, 51820,
                62280, 57400,
            },
        },
        {
            {
                10896, 56144, 27156, 4400,  47012, 28684, 11800, 41656, 54872,
                30388, 63420, 39496, 33392, 2736,  35944, 15052, 52984, 6132,
                23068, 15596, 55676, 20652, 61952, 53380, 25268, 46188, 20768,
                42844, 52660, 13036, 40400, 31648, 62500, 14828, 49692, 30720,
                46012, 18772, 62268, 29216, 59912, 40020, 6364,  21536, 61592,
                34512, 12152, 46216, 36300, 16484, 62676, 11368, 924,   55212,
                65180, 35708, 3856,  27160, 38268, 60344, 32516, 56612, 12924,
                8808,  30312, 48544, 14588, 22196, 12136, 4000,  27516, 22812,
                6796,  43712, 61156, 2336,  26628, 7600,  19032, 61916, 14192,
                38148, 63632, 16668, 31840, 41628, 63032, 46832, 38088, 54636,
                32012, 9364,  36472, 20228, 58108, 53944, 28632, 13200, 51656,
                25232, 16164, 48748, 9952,  60216, 50792, 5896,  38176, 58128,
                16076, 34472, 48968, 38804, 4868,  43144, 20788, 11128, 25068,
                39812, 27460, 5232,  59844, 29864, 21084, 7756,  50636, 5436,
                28780, 42364,
            },
            {
                36568, 51492, 40328, 23564, 18096, 58348, 32320, 24616, 59492,
                8800,  51332, 47620, 23812, 55496, 49340, 65468, 31748, 39328,
                58264, 33720, 51264, 29812, 36344, 40012, 6564,  12428, 35624,
                8996,  59012, 24576, 28668, 2168,  21704, 37360, 27208, 3892,
                40564, 12576, 51624, 34868, 14552, 56484, 47404, 28464, 54648,
                41196, 26220, 4608,  56000, 2652,  47600, 38540, 32652, 17640,
                9772,  30444, 50900, 12696, 47828, 16400, 6288,  22152, 46944,
                38480, 4084,  24464, 61492, 45600, 53444, 38616, 58700, 63196,
                36676, 28600, 18116, 38852, 49148, 11620, 42564, 55692, 28264,
                9252,  1924,  45748, 5584,  50344, 14728, 8020,  60360, 6056,
                16412, 47336, 62820, 12316, 40964, 16512, 46764, 22748, 56204,
                3964,  58144, 28380, 39652, 22476, 43940, 26552, 64684, 8952,
                25368, 46160, 12348, 31228, 26156, 55116, 47408, 61304, 50856,
                16020, 9424,  43336, 48520, 3476,  64780, 38096, 44996, 34796,
                1228,  64156,
            },
            {
                16432, 31948, 1764,  60916, 53388, 37824, 49584, 3496,  19384,
                36032, 1732,  17396, 59184, 6772,  10164, 21836, 45476, 27572,
                8948,  63692, 2520,  44552, 5128,  15004, 64172, 27304, 55576,
                900,   16852, 47340, 6328,  57300, 43996, 53632, 17416, 59488,
                22440, 54976, 8232,  44240, 2936,  24536, 18472, 9820,  1232,
                50128, 19436, 61080, 43400, 27772, 20580, 51544, 23616, 60696,
                45180, 39764, 20596, 62680, 2440,  54696, 36880, 43972, 26252,
                65432, 17428, 57180, 36328, 1448,  19740, 9184,  44452, 16624,
                244,   57008, 52340, 23752, 58080, 32804, 36892, 3616,  22564,
                51048, 58924, 25752, 56664, 11084, 27836, 34028, 18916, 44236,
                57688, 22364, 52856, 4100,  31764, 64560, 356,   33696, 9512,
                37380, 45564, 20352, 7100,  55252, 3632,  13092, 52388, 20740,
                54364, 2808,  58952, 62488, 9892,  17400, 36456, 6616,  31964,
                58680, 37088, 54112, 12768, 34316, 26524, 54924, 18468, 14108,
                58476, 25568,
            },
            {
                8480,  48680, 13344, 43840, 5388,  9396,  15176, 44908, 64980,
                27992, 42032, 13500, 31384, 26424, 41256, 18048, 3256,  13744,
                49064, 19488, 11856, 24316, 48176, 32412, 50728, 41828, 30996,
                62588, 37452, 51400, 33668, 63752, 11636, 8096,  34592, 65016,
                164,   32596, 26620, 19904, 38828, 52924, 63880, 36252, 58564,
                30632, 14700, 33416, 10708, 58836, 6952,  13924, 41680, 4744,
                26128, 15316, 58928, 33064, 42380, 28216, 10812, 58664, 280,
                50824, 32156, 11136, 43100, 29196, 64160, 32264, 25052, 49616,
                12272, 33948, 8008,  13904, 45408, 5140,  60380, 15936, 47520,
                40284, 34352, 18448, 43056, 37496, 59224, 51880, 40164, 26572,
                1604,  34980, 28880, 42828, 24228, 8420,  39036, 49780, 17344,
                62456, 30960, 13644, 63964, 34188, 29440, 48056, 40820, 32216,
                36420, 43784, 19052, 39996, 50500, 476,   28512, 42016, 22576,
                2460,  19860, 62332, 23960, 16328, 57552, 9852,  31056, 52212,
                39188, 46080,
            },
            {
                56896, 22928, 63392, 28204, 34168, 21748, 56512, 39048, 11960,
                52732, 21464, 45316, 62520, 37576, 50648, 61292, 56704, 36752,
                53528, 42732, 38660, 60748, 17412, 58860, 21904, 10000, 18592,
                45020, 23140, 13584, 41616, 19748, 26132, 46468, 50664, 13448,
                42528, 47800, 58800, 62628, 31736, 4688,  45500, 12988, 42676,
                6744,  39604, 23788, 49136, 37416, 30508, 64112, 34800, 56876,
                52640, 1352,  46492, 7588,  24008, 18780, 52040, 34600, 13984,
                20776, 40388, 54088, 7348,  15996, 48312, 40076, 4984,  55304,
                41692, 21652, 47364, 63856, 29452, 19672, 52932, 26088, 31380,
                11884, 65020, 7176,  30156, 21372, 20,    23520, 13084, 64664,
                9992,  50000, 15016, 61776, 45920, 56752, 20084, 59668, 26336,
                53072, 2492,  41652, 51376, 18084, 58576, 212,   16868, 62224,
                5176,  14116, 29252, 22268, 33844, 57408, 63248, 13912, 55868,
                49448, 29260, 5700,  46268, 40688, 68,    43140, 22024, 60844,
                3100,  20108,
            },
            {
                42040, 6004,  37016, 18624, 51224, 61920, 30768, 26148, 6468,
                33704, 57924, 4708,  53948, 372,   12524, 33576, 29304, 23144,
                5760,  31368, 26212, 55180, 8448,  3112,  37728, 53984, 4616,
                60036, 7728,  29912, 1548,  53040, 38568, 5688,  29564, 23924,
                36744, 16244, 5556,  11428, 49796, 16968, 33768, 25736, 21096,
                52160, 64596, 55552, 96,    16700, 53812, 43992, 8656,  19472,
                29376, 36788, 11240, 55888, 39112, 63404, 4928,  30244, 48720,
                61344, 5516,  27280, 59216, 23472, 51752, 13400, 59732, 18668,
                30372, 60976, 37280, 1780,  40600, 10320, 44204, 62708, 948,
                20816, 52284, 15036, 55348, 48644, 61928, 5800,  47020, 31508,
                55236, 38380, 5248,  18628, 11548, 35616, 29868, 4468,  43372,
                11732, 22608, 36392, 8732,  46304, 24296, 37356, 10668, 23168,
                57072, 49572, 8348,  53412, 4736,  45832, 24936, 7932,  44768,
                39944, 10948, 35568, 51896, 31636, 63352, 49644, 6196,  28020,
                12336, 33752,
            },
            {
                30068, 54128, 59116, 10764, 40932, 2848,  47284, 17264, 60420,
                49968, 9964,  29944, 16804, 24428, 43760, 8208,  47044, 58776,
                16296, 62280, 688,   35012, 46136, 29312, 43268, 14340, 26668,
                34396, 46952, 55940, 60772, 15392, 32064, 61800, 18908, 55976,
                9348,  52720, 41084, 21796, 28148, 57476, 8032,  61236, 47264,
                3732,  27660, 9288,  46044, 22328, 26432, 2912,  12228, 47652,
                60056, 22672, 50316, 30832, 14264, 43236, 59828, 9148,  24912,
                44832, 36856, 19228, 46220, 34092, 3580,  37436, 27652, 8376,
                53128, 3944,  26356, 16204, 55504, 24040, 34576, 7956,  56540,
                42204, 36108, 4476,  44840, 10364, 33548, 53988, 16812, 43388,
                20712, 60128, 25312, 48016, 54092, 6556,  40656, 14932, 47312,
                33404, 56396, 61012, 26960, 4732,  42884, 60136, 52008, 30628,
                46544, 26320, 64812, 42164, 16440, 11424, 38132, 19104, 32392,
                65496, 17308, 60184, 21180, 8252,  14604, 25304, 36296, 47356,
                65148, 50888,
            },
            {
                3960,  26104, 14776, 46152, 24236, 8108,  54820, 42596, 1092,
                23704, 40264, 47988, 35992, 63428, 56024, 20720, 2860,  39028,
                10820, 44896, 21328, 13536, 51600, 65064, 23672, 57972, 50188,
                39912, 11504, 20900, 25264, 49132, 4016,  58152, 44256, 2624,
                62940, 25984, 31048, 55476, 43748, 936,   39184, 53972, 15188,
                37104, 18528, 35048, 61728, 41060, 59396, 50624, 33268, 40088,
                15732, 5292,  64836, 18008, 2084,  47212, 21620, 35476, 16664,
                55364, 1624,  13056, 62316, 9940,  56936, 21016, 63052, 33524,
                45256, 11452, 49420, 32384, 59120, 50992, 14572, 38928, 27788,
                49292, 24740, 60796, 18016, 28272, 41208, 25048, 36980, 7748,
                29156, 3128,  41820, 33448, 27796, 60752, 50496, 23828, 64212,
                1404,  19196, 48992, 15584, 32440, 54768, 13720, 6568,  41432,
                18244, 1324,  34448, 59724, 31108, 48044, 58648, 53784, 1184,
                26280, 4760,  54824, 27884, 38868, 58272, 18232, 55708, 40336,
                9064,  17356,
            },
            {
                45296, 38932, 612,   32752, 64668, 35588, 13032, 20812, 37372,
                64032, 14432, 19400, 5684,  27740, 40604, 14084, 52356, 64420,
                28024, 50132, 57608, 39716, 18064, 11080, 33440, 1032,  19500,
                62996, 3428,  36896, 43740, 9548,  40448, 22632, 12120, 34224,
                39304, 47428, 4516,  14164, 36176, 65460, 23536, 10636, 30028,
                59108, 44384, 12436, 31844, 5464,  14800, 20248, 55092, 62524,
                27768, 44560, 38560, 33892, 54256, 27440, 57088, 4116,  41700,
                64376, 28624, 52768, 40740, 30648, 43624, 1196,  47636, 14936,
                23964, 42252, 65352, 20640, 6188,  46508, 3248,  19048, 59824,
                9896,  2712,  31672, 38440, 63792, 14052, 2184,  57832, 62952,
                11340, 51240, 56968, 13580, 1208,  21412, 9736,  54940, 37876,
                7880,  29520, 40456, 10776, 65492, 20404, 27700, 35300, 63100,
                9828,  38220, 12972, 23984, 3060,  21012, 28124, 35468, 50876,
                14400, 46732, 41408, 12268, 1736,  45680, 33120, 2840,  13616,
                23980, 57312,
            },
            {
                20192, 62756, 48964, 53672, 16720, 29632, 51028, 57420, 28100,
                32528, 44676, 52836, 58940, 9488,  49336, 33032, 25516, 18684,
                36152, 7796,  32092, 4056,  27196, 54368, 6692,  48428, 30376,
                15928, 52192, 27784, 64640, 17740, 35292, 53496, 28424, 50320,
                16844, 20544, 60876, 8932,  46352, 17940, 32224, 48324, 41972,
                2304,  51724, 21428, 55816, 47348, 38488, 30032, 7196,  23956,
                588,   10080, 58372, 24728, 7972,  12632, 37736, 49888, 10180,
                32536, 23108, 7624,  50032, 16160, 25588, 54448, 39020, 6580,
                57504, 17928, 38304, 9520,  28672, 36052, 63232, 31020, 53788,
                16548, 46740, 57964, 21200, 5280,  46144, 52460, 20024, 48528,
                34496, 23412, 17720, 64396, 37164, 43684, 16572, 31984, 26728,
                58844, 45444, 53356, 3220,  37968, 50460, 1716,  47908, 22044,
                56476, 44756, 51760, 55460, 39776, 63888, 6112,  10328, 61408,
                23360, 33428, 58872, 50184, 22892, 61552, 52904, 29236, 43724,
                60612, 35696,
            },
            {
                7984,  12064, 27980, 21728, 5004,  59468, 44388, 3692,  11212,
                7776,  55912, 1728,  30956, 22468, 61596, 5144,  45684, 55232,
                1480,  43344, 23500, 62728, 46976, 41340, 35892, 61720, 42956,
                8268,  57280, 33088, 6016,  55268, 46504, 14492, 6892,  64012,
                636,   32692, 57360, 24212, 52352, 2956,  59976, 20752, 6608,
                26060, 63292, 8256,  28292, 1508,  65140, 11092, 43172, 35144,
                53460, 47008, 19976, 41536, 51616, 62300, 30684, 19008, 59620,
                14252, 46928, 38324, 20632, 4636,  64040, 11272, 28868, 60556,
                34840, 51176, 828,   54152, 43640, 22160, 12496, 40920, 23632,
                43200, 34328, 12236, 50628, 55820, 9256,  27148, 31352, 15544,
                44616, 39316, 7140,  30212, 46888, 52632, 61560, 3788,  42264,
                13024, 22292, 16892, 57484, 24124, 43428, 60632, 16456, 32160,
                4172,  29132, 19504, 7664,  15164, 49960, 42672, 17736, 38468,
                44140, 8144,  18860, 30304, 36680, 6680,  10888, 20672, 50072,
                4964,  31896,
            },
            {
                51172, 55944, 42988, 36796, 9900,  39748, 24976, 18604, 61880,
                47060, 25816, 38600, 12976, 43436, 36764, 17032, 11588, 29884,
                59652, 14580, 52716, 9712,  58884, 20644, 14136, 25956, 22600,
                38308, 46096, 13308, 24144, 1332,  30896, 60252, 25452, 40976,
                54544, 11408, 37716, 42592, 27144, 34828, 12724, 50556, 57840,
                16768, 34564, 40736, 49700, 17660, 24760, 57932, 50040, 16876,
                60456, 13096, 32016, 3312,  16420, 45528, 940,   25104, 43912,
                54068, 2700,  61112, 56572, 35760, 32344, 18488, 49032, 21860,
                3600,  30920, 26092, 14328, 60688, 57400, 5476,  48912, 364,
                64696, 7484,  28572, 40048, 24160, 35676, 42348, 59036, 5964,
                484,   61256, 54184, 10596, 24320, 5680,  34584, 20324, 49684,
                63680, 36220, 6352,  33680, 30040, 8412,  12036, 39116, 58588,
                49256, 24784, 62180, 36428, 46400, 25160, 31716, 57336, 2276,
                28892, 54120, 3292,  63740, 15616, 42708, 25200, 64428, 38692,
                15724, 25704,
            },
            {
                40420, 17992, 1876,  61404, 47964, 14876, 33592, 50084, 40936,
                35112, 15904, 20692, 65260, 51548, 2444,  57116, 40744, 49068,
                20916, 38696, 34208, 16940, 30208, 1872,  55876, 10348, 59564,
                2360,  18500, 50044, 61984, 37768, 19896, 49436, 9300,  44972,
                22132, 28908, 48832, 7024,  15608, 63548, 40184, 28772, 37340,
                54668, 13544, 23120, 61104, 36184, 45008, 32664, 3028,  21504,
                37524, 29080, 49084, 64068, 36168, 22276, 58608, 39872, 7072,
                34948, 17896, 27132, 10056, 45328, 53400, 40128, 8000,  44520,
                13104, 62672, 41324, 46776, 17516, 33216, 52072, 26936, 37140,
                15108, 54948, 19236, 61940, 1640,  16832, 64976, 11776, 37700,
                25472, 49976, 19628, 41620, 58996, 28044, 14708, 56696, 10368,
                460,   25660, 47140, 51216, 63284, 19172, 46512, 26676, 6812,
                14620, 42720, 11060, 32892, 708,   61136, 12144, 52240, 21624,
                62692, 47840, 11608, 40260, 56232, 48592, 34240, 54324, 788,
                44912, 58428,
            },
            {
                6180,  33904, 23308, 30476, 56568, 7376,  63956, 0,     22420,
                52572, 60100, 3900,  48120, 28232, 34044, 23064, 61052, 8416,
                3936,  64880, 25320, 45308, 6268,  37244, 49300, 44064, 32200,
                53336, 29028, 41576, 10068, 57564, 42920, 3104,  35580, 18004,
                59192, 3464,  62304, 19380, 55392, 4132,  44440, 9760,  1084,
                46476, 30764, 4344,  10532, 53832, 6748,  14148, 40480, 63440,
                8472,  4464,  56552, 26416, 6076,  53536, 11316, 28244, 64968,
                50800, 31392, 42468, 23544, 72,    14396, 59560, 25120, 52316,
                37424, 55932, 4836,  23432, 7832,  39592, 11168, 20488, 60996,
                30744, 48068, 4552,  44060, 32632, 49180, 22060, 45376, 29188,
                56408, 32480, 13240, 35752, 1688,  48904, 39440, 44664, 29748,
                53548, 41076, 60260, 14120, 4204,  36580, 55652, 52204, 33896,
                64944, 2760,  53308, 58832, 18476, 27840, 4628,  40788, 36200,
                16108, 25388, 33812, 20344, 27644, 4060,  8936,  18784, 30416,
                12692, 63068,
            },
            {
                47256, 10740, 52548, 44700, 19852, 26836, 54540, 29720, 12472,
                5416,  31720, 42356, 10376, 18304, 7304,  53088, 26512, 32280,
                47080, 12396, 56544, 50984, 40856, 63536, 27464, 16740, 5300,
                65412, 21484, 7052,  33564, 26436, 15564, 64468, 29888, 52704,
                38688, 14076, 33356, 46072, 30560, 22464, 53196, 24972, 19148,
                64572, 41780, 59496, 47844, 20532, 29220, 56336, 25664, 50912,
                45676, 23560, 42796, 14548, 40532, 32764, 48284, 15848, 21324,
                4300,  12916, 63104, 58048, 48328, 29184, 5336,  64940, 16308,
                27952, 10268, 48524, 35840, 64456, 29320, 3444,  56808, 45028,
                8748,  25188, 38160, 58440, 14176, 7864,  54700, 2664,  18672,
                62800, 8528,  46580, 22416, 65280, 17420, 7556,  62092, 21972,
                32932, 17680, 8676,  28480, 21480, 41792, 780,   23240, 17668,
                40396, 29576, 21920, 38492, 43980, 55224, 48584, 14160, 8632,
                45788, 59208, 6244,  51472, 60284, 13752, 45872, 57200, 36412,
                22156, 27692,
            },
            {
                16960, 38420, 65512, 13412, 2504,  42076, 37640, 17588, 44300,
                57944, 26548, 54192, 39340, 62764, 44624, 13828, 37668, 16580,
                42412, 29036, 276,   18832, 9280,  22952, 52560, 35052, 11684,
                39344, 56112, 14008, 51804, 46608, 22488, 54252, 12528, 6048,
                26208, 50272, 9588,  57860, 36988, 11996, 60852, 48336, 33304,
                7200,  26956, 16516, 38000, 764,   42916, 61788, 11332, 31476,
                18572, 59664, 10356, 51528, 19828, 57828, 8560,  35764, 55372,
                46528, 38664, 7948,  16956, 37080, 21424, 43072, 34964, 1544,
                20876, 58480, 31860, 19444, 53724, 49824, 42384, 17000, 34612,
                52784, 12104, 21472, 51440, 27492, 60488, 34724, 39684, 52400,
                15760, 42628, 3764,  55556, 31164, 52044, 25348, 36956, 13376,
                3260,  56076, 38392, 44980, 58288, 31440, 60980, 10640, 45144,
                57148, 8064,  50388, 15736, 5920,  23276, 34756, 64700, 30480,
                53468, 232,   37444, 43552, 31908, 23844, 39736, 62376, 48900,
                2984,  53648,
            },
            {
                31832, 4072,  25496, 35376, 58812, 51160, 9960,  61064, 47796,
                35852, 8400,  14860, 22088, 29840, 58916, 1104,  49532, 62172,
                53892, 21416, 36048, 59296, 32700, 14352, 3916,  59888, 48188,
                24460, 44648, 36700, 124,   59364, 4424,  40608, 34012, 47568,
                63072, 41140, 23756, 1012,  42980, 17548, 5400,  39432, 58056,
                11292, 50520, 56924, 32140, 23484, 51852, 17096, 35656, 2008,
                55096, 38536, 34272, 644,   29472, 62808, 3576,  43168, 61640,
                29832, 25296, 54532, 33116, 51148, 11512, 57124, 31012, 50192,
                39248, 43916, 14564, 2752,  25772, 12312, 59580, 22768, 5216,
                40240, 64256, 1076,  31864, 41840, 10900, 24856, 47432, 6492,
                23796, 59800, 26944, 38816, 11108, 43248, 4332,  58132, 45876,
                49348, 64720, 23892, 11696, 53084, 16192, 6040,  49484, 27580,
                35492, 13008, 46700, 63264, 31544, 10220, 57076, 19376, 26616,
                41896, 22660, 17540, 10116, 65136, 2600,  16488, 26200, 7268,
                41264, 60464,
            },
            {
                43496, 57256, 49196, 8004,  22504, 16112, 33192, 6124,  24204,
                19756, 64472, 50280, 3136,  55728, 34856, 20204, 5840,  24292,
                10472, 6792,  63000, 47924, 25728, 55068, 41936, 20276, 30368,
                2720,  17292, 62712, 31424, 18988, 28336, 8920,  57100, 17472,
                2632,  20456, 54072, 28356, 65084, 50776, 26396, 14744, 36000,
                20872, 44076, 3664,  13208, 64220, 8120,  39804, 48072, 6384,
                27780, 15144, 65252, 47276, 25272, 45096, 17016, 24080, 11848,
                1004,  18836, 41864, 3272,  62308, 6720,  46968, 18260, 9444,
                54564, 60528, 7108,  62888, 45848, 37068, 1420,  32684, 61536,
                26440, 45904, 16496, 56944, 19084, 3276,  64092, 30548, 12536,
                37084, 33312, 51288, 19872, 15228, 60680, 34236, 9744,  18884,
                27548, 6828,  35064, 1652,  48092, 25360, 38020, 63700, 20320,
                4336,  60084, 25684, 1096,  36848, 51404, 43128, 3328,  7368,
                61664, 57724, 49444, 28196, 54980, 35304, 52272, 11844, 33280,
                20516, 9628,
            },
            {
                13864, 19332, 40200, 29924, 46260, 63724, 27404, 41132, 53644,
                1040,  32452, 45800, 37412, 25376, 12208, 41316, 46280, 56020,
                33580, 39324, 44100, 12764, 2420,  38364, 46432, 62064, 9108,
                57248, 27376, 49844, 10736, 42736, 61528, 36948, 24676, 45188,
                30660, 60088, 15116, 35560, 8608,  31644, 55828, 1680,  63124,
                29704, 24504, 54344, 34528, 45108, 26876, 57128, 20212, 61296,
                43812, 21880, 53192, 7444,  12984, 37272, 56012, 32468, 40016,
                52060, 48204, 60636, 13836, 27436, 23648, 40548, 63140, 4068,
                26284, 22832, 34392, 40712, 16360, 55536, 28824, 48288, 14464,
                54180, 36080, 6136,  49116, 37644, 44888, 53136, 20984, 58384,
                49636, 2508,  63748, 6972,  29980, 47200, 21720, 54360, 41124,
                31236, 52140, 43488, 20536, 62484, 33568, 13484, 43176, 54528,
                32388, 39964, 53112, 19768, 28532, 61084, 16672, 47544, 39456,
                11712, 31428, 14644, 46448, 6804,  19512, 59028, 44296, 49916,
                56044, 28936,
            },
            {
                2280,  53032, 62624, 4772,  12296, 54984, 3324,  50052, 13948,
                38996, 11236, 60472, 17424, 5164,  52396, 31248, 59556, 27332,
                14912, 51480, 29836, 17596, 58236, 28168, 5588,  15644, 33760,
                52068, 40704, 6780,  23280, 55104, 48412, 13916, 51312, 7060,
                11204, 38320, 44156, 5036,  47468, 19780, 41168, 45448, 52304,
                6228,  40592, 9632,  59832, 15992, 3020,  30620, 12500, 50576,
                33272, 3188,  40660, 31016, 60656, 20648, 50360, 6220,  59960,
                21872, 9408,  35976, 31732, 44652, 55852, 15404, 28960, 37744,
                13180, 47996, 30112, 51468, 9672,  21240, 43532, 6788,  19600,
                9872,  30072, 24072, 63012, 8968,  28088, 15300, 5344,  43196,
                17788, 10280, 45084, 40948, 56872, 184,   26500, 64016, 2844,
                13868, 59680, 16828, 40136, 8788,  57820, 29560, 2048,  9600,
                23904, 16596, 44404, 8524,  38796, 12280, 24540, 33368, 52216,
                20792, 35784, 2172,  38768, 23092, 41812, 29844, 724,   37888,
                23700, 61448,
            },
            {
                36172, 25060, 33540, 17012, 38084, 43676, 19060, 30184, 62196,
                56684, 21808, 28576, 42568, 48340, 63132, 9480,  18168, 1740,
                64816, 5068,  23180, 53596, 8696,  49620, 36408, 22636, 44164,
                12612, 64524, 36044, 15828, 33164, 5116,  20924, 64832, 32488,
                57984, 52820, 22708, 61764, 13156, 58852, 23852, 10564, 17960,
                60816, 49364, 19668, 37592, 47508, 22352, 63316, 36804, 8804,
                24300, 59060, 17716, 49140, 9920,  42472, 2424,  28596, 14896,
                56772, 26168, 5136,  53756, 20296, 1244,  34068, 58120, 43520,
                53248, 468,   18776, 57896, 4800,  25072, 65404, 57044, 40584,
                51092, 59304, 42932, 13672, 54932, 35160, 60792, 39800, 32644,
                55488, 28692, 24988, 13300, 52612, 36740, 16528, 49852, 38280,
                24400, 10344, 56424, 28048, 4144,  45488, 50708, 18800, 58708,
                47680, 64760, 3124,  55484, 48780, 58548, 1784,  64248, 27856,
                55196, 5624,  63384, 50612, 60660, 9464,  14212, 62744, 5664,
                16340, 45580,
            },
            {
                50836, 10992, 48004, 56460, 23476, 59852, 9544,  36096, 24944,
                44496, 4316,  8240,  57748, 14408, 22624, 36360, 40008, 45388,
                20748, 35548, 41236, 60292, 32524, 18948, 63900, 55752, 844,
                26080, 19724, 3204,  46004, 58256, 39856, 1920,  27736, 43192,
                18748, 700,   26224, 50156, 33880, 2728,  38192, 28316, 35008,
                14140, 31724, 248,   27876, 55804, 6532,  41556, 53668, 46576,
                14256, 55620, 5304,  35744, 27000, 54304, 64484, 33864, 46236,
                38016, 43272, 17532, 65320, 47532, 9984,  50968, 7572,  21000,
                64568, 10388, 36504, 61148, 32784, 49632, 37844, 11984, 33716,
                180,   21772, 4584,  32336, 20368, 2836,  25480, 48344, 22428,
                1172,  59196, 34956, 20552, 62828, 7840,  32372, 44500, 5312,
                61732, 46876, 32180, 22284, 53876, 14780, 24748, 41004, 36340,
                12424, 27964, 34692, 15008, 31104, 21936, 45336, 7688,  15504,
                41648, 44668, 17644, 25776, 33156, 56368, 47664, 26636, 53544,
                32376, 8112,
            },
            {
                58356, 42396, 380,   28132, 6908,  32732, 47596, 2068,  16448,
                49092, 31364, 52020, 34552, 26324, 2480,  54608, 28720, 7816,
                57584, 48532, 11736, 1192,  44416, 25396, 10508, 40220, 31024,
                47932, 53552, 59760, 29656, 25344, 11704, 53856, 46972, 15576,
                35176, 41300, 10060, 30200, 16660, 55084, 64144, 48376, 3848,
                54076, 43284, 65368, 12872, 51104, 32672, 18188, 2384,  26384,
                39692, 29600, 63708, 45868, 23188, 15768, 7212,  19656, 11584,
                62288, 2124,  29928, 39908, 12672, 37192, 61372, 25192, 32292,
                16636, 45888, 26704, 42340, 14616, 2052,  29860, 17908, 62420,
                26640, 47728, 38764, 61836, 45280, 51040, 12100, 8080,  64728,
                14600, 46892, 51024, 3680,  40124, 23588, 11404, 58004, 20340,
                35372, 51004, 868,   38608, 65152, 34524, 7364,  60964, 52232,
                6256,  42640, 20668, 60764, 4860,  41172, 36320, 57388, 10584,
                23780, 59464, 30348, 12276, 3300,  21252, 35536, 18456, 40556,
                65000, 21376,
            },
            {
                38904, 15060, 20028, 64176, 52524, 40444, 21396, 63320, 53960,
                12676, 38144, 18328, 65296, 40968, 47308, 61624, 13028, 50768,
                24380, 30796, 16212, 62328, 37976, 5888,  50908, 14452, 60932,
                6996,  38532, 9380,  42028, 18028, 63464, 37400, 22168, 60736,
                5432,  54656, 62816, 46032, 7312,  42992, 20388, 8768,  58456,
                25692, 21732, 39252, 8552,  23968, 44728, 59204, 11224, 62652,
                19356, 34032, 404,   11988, 58180, 40308, 36668, 48388, 53300,
                23876, 8540,  51676, 58848, 22860, 28176, 42716, 4348,  49948,
                39728, 56456, 6340,  22208, 54672, 46848, 8584,  53568, 44056,
                15216, 56404, 11028, 17604, 28560, 58464, 35968, 53804, 42200,
                30992, 37908, 8960,  16984, 59708, 48136, 54148, 26988, 15056,
                29240, 8212,  18332, 43708, 11656, 49128, 29876, 21548, 176,
                32148, 56192, 49864, 9656,  26684, 51560, 18296, 29556, 49220,
                37864, 768,   53180, 46008, 39984, 51668, 7604,  43672, 2740,
                12020, 29896,
            },
            {
                6144,  61352, 31492, 44744, 10652, 14216, 57848, 5396,  26920,
                41496, 60384, 224,   11156, 20608, 6832,  31944, 17524, 59948,
                43080, 3568,  56208, 27844, 46844, 22092, 58416, 27468, 17448,
                35216, 21756, 13540, 51704, 48224, 6080,  30808, 8740,  49848,
                12908, 28608, 20948, 36104, 51800, 24404, 32352, 40316, 15944,
                45836, 5060,  56824, 35108, 61688, 15268, 28660, 36148, 48264,
                7828,  52684, 43944, 20912, 49676, 30804, 1684,  59400, 27552,
                32500, 45196, 14668, 34484, 6524,  54520, 18692, 13900, 60368,
                2176,  29588, 12420, 63532, 38628, 19992, 59348, 24120, 6252,
                36732, 30300, 52608, 7720,  41500, 1416,  16312, 27476, 19636,
                5156,  61476, 25844, 43860, 33684, 30124, 1772,  42448, 63800,
                39848, 55904, 60476, 26400, 5024,  55396, 16276, 39148, 63156,
                25620, 17380, 38064, 45912, 63980, 12968, 54548, 6312,  60828,
                20240, 33912, 8820,  57864, 16472, 63704, 28768, 59316, 49480,
                56264, 25528,
            },
            {
                52208, 35776, 3756,  49452, 25632, 37540, 29464, 44228, 33364,
                7952,  23808, 57184, 30336, 44572, 53216, 37988, 4544,  34824,
                10096, 39268, 19720, 33336, 13000, 53316, 2212,  41504, 45588,
                54972, 32284, 61968, 23948, 560,   34656, 56116, 43828, 25652,
                58112, 39524, 4308,  14584, 56860, 1540,  12508, 62576, 36512,
                51248, 30664, 18280, 48932, 1528,  40892, 5676,  57628, 22264,
                41996, 15048, 59984, 6064,  25936, 55052, 9652,  42128, 16788,
                3640,  56988, 19980, 49356, 1496,  64064, 31252, 46984, 35868,
                23664, 52052, 33908, 48888, 4320,  27892, 34568, 41180, 50220,
                2476,  22852, 64868, 34140, 49176, 23936, 38896, 63112, 46324,
                11256, 52268, 20960, 55956, 12452, 6764,  19308, 51780, 10132,
                4592,  46344, 14284, 36212, 20848, 59100, 47376, 9864,  44044,
                13552, 59952, 3504,  23528, 35288, 1320,  32540, 24908, 43580,
                14392, 65372, 27560, 22804, 36508, 4052,  13600, 23448, 33968,
                16964, 46728,
            },
            {
                10372, 22696, 55404, 16708, 59712, 1536,  18148, 51296, 56052,
                20272, 46788, 35196, 50264, 25416, 14736, 56892, 21988, 27060,
                64436, 54488, 49956, 7920,  63324, 35668, 30672, 10036, 65028,
                3036,  5764,  43508, 57276, 27344, 15172, 61620, 18960, 2876,
                33260, 47644, 65108, 41676, 26696, 60000, 48712, 28896, 2940,
                9764,  60412, 12264, 26848, 53868, 20416, 31176, 50808, 2020,
                55492, 27956, 32228, 37536, 17684, 61400, 13488, 22316, 64272,
                35864, 40840, 62024, 25948, 43952, 38944, 10848, 57612, 7460,
                43504, 19012, 9096,  57340, 16316, 44632, 10924, 63816, 13796,
                57732, 45688, 19492, 4892,  59360, 13452, 55428, 3880,  33520,
                57708, 29604, 624,   36832, 65292, 46808, 60740, 35156, 25184,
                32324, 22224, 53100, 30340, 41472, 2616,  33184, 27612, 54496,
                48508, 8072,  30644, 53340, 15588, 40616, 62720, 47724, 3608,
                39332, 51016, 5708,  41904, 48484, 31788, 54444, 8912,  37900,
                1564,  42888,
            },
            {
                58868, 27748, 39928, 8124,  34268, 42332, 64692, 9448,  13772,
                3108,  63048, 16240, 4904,  9176,  63604, 41272, 48852, 45348,
                772,   14660, 23488, 43768, 4228,  17976, 48424, 20856, 24884,
                50372, 29020, 17084, 8908,  37124, 41380, 11152, 52112, 38068,
                23400, 17132, 10440, 30984, 19396, 6824,  43928, 17300, 53376,
                21116, 33708, 43604, 7248,  37516, 64528, 45700, 13480, 25236,
                38516, 8816,  65212, 51536, 4268,  43484, 33508, 52360, 46604,
                5648,  12256, 29580, 8940,  16188, 51496, 22412, 27480, 15628,
                54004, 61508, 40536, 25280, 31632, 60188, 744,   18376, 27032,
                32608, 9336,  37892, 26564, 43832, 31776, 21024, 8700,  43360,
                16908, 40084, 49604, 15044, 27752, 22980, 38752, 16716, 57596,
                50284, 1360,  61768, 10684, 45752, 63612, 18980, 5736,  22792,
                34332, 41712, 19884, 57912, 44776, 7272,  21592, 56104, 11764,
                30868, 18484, 55144, 10340, 61704, 19712, 44400, 60312, 27024,
                63168, 19584,
            },
            {
                33048, 13680, 63716, 48124, 20840, 54332, 24612, 36620, 49748,
                39508, 27356, 42784, 54432, 36560, 29224, 2884,  11752, 18548,
                30640, 36396, 59368, 26248, 40332, 60376, 54644, 37820, 14988,
                57948, 39484, 47176, 63972, 50844, 21520, 29804, 46016, 7472,
                54144, 60364, 16,    51032, 55300, 34220, 38476, 23580, 57560,
                41108, 63556, 47812, 24028, 58840, 16856, 10052, 33888, 62032,
                18640, 46232, 11768, 23456, 47476, 28972, 38772, 7820,  25672,
                55776, 21436, 48440, 37324, 56256, 4272,  33860, 62760, 620,
                37672, 28848, 13320, 2964,  47436, 36516, 51436, 54484, 40004,
                48316, 61600, 16028, 54736, 10728, 62296, 47128, 51508, 25412,
                60280, 22348, 6060,  43060, 54776, 2644,  8388,  45232, 13124,
                41980, 18728, 37764, 7424,  24888, 13884, 51924, 37952, 57492,
                1636,  64496, 26764, 11016, 50088, 28708, 17528, 36992, 26588,
                60448, 45164, 35128, 15296, 25460, 64,    39696, 47480, 14840,
                53584, 4992,
            },
            {
                50924, 45672, 2392,  30240, 12056, 5180,  45772, 30932, 22236,
                57928, 32548, 11348, 60836, 19880, 23556, 58308, 33144, 61452,
                51196, 6044,  46644, 10688, 29392, 13516, 12,    44828, 7652,
                34476, 11420, 19944, 32860, 5440,  55716, 2056,  63312, 26888,
                13144, 42816, 36524, 22556, 45780, 9528,  64168, 13220, 5812,
                27696, 760,   14596, 30356, 3408,  42624, 52816, 6092,  40256,
                58724, 3072,  35332, 15668, 57456, 1532,  63128, 18756, 31680,
                50764, 60,    60892, 32380, 18844, 59540, 42080, 48800, 20100,
                45828, 5360,  50676, 65204, 20612, 8304,  23524, 30400, 5780,
                21752, 3164,  42188, 50620, 1168,  18088, 28436, 37100, 2116,
                12996, 35416, 62776, 10228, 32868, 50988, 59200, 30616, 3820,
                64424, 28236, 48676, 55188, 34652, 58188, 28776, 40996, 12340,
                17856, 51088, 36376, 5132,  62160, 33168, 59340, 9272,  52384,
                2120,  22568, 4896,  57352, 50492, 29508, 11940, 6668,  21868,
                30108, 41768,
            },
            {
                24280, 17732, 37840, 58020, 26360, 60924, 16012, 676,   62044,
                6704,  17952, 46208, 1556,  40052, 50504, 43948, 7896,  54024,
                21700, 39516, 16688, 52276, 56952, 22592, 32512, 61520, 26832,
                51776, 59460, 1308,  26180, 13784, 43024, 35148, 16356, 58956,
                32036, 49508, 27944, 62284, 14892, 2344,  30180, 49936, 46460,
                37092, 19016, 51764, 56064, 35652, 21604, 27176, 48448, 29948,
                20892, 50404, 42524, 26976, 53132, 21260, 10996, 59044, 15520,
                43188, 39668, 14152, 7092,  45352, 25244, 13100, 9696,  30692,
                55376, 24128, 34832, 41924, 56072, 15236, 43440, 62956, 12704,
                35640, 57464, 24312, 30064, 34740, 40248, 5540,  64952, 54196,
                31052, 48012, 56420, 24496, 16320, 41056, 19816, 25608, 53740,
                33792, 5692,  23116, 42856, 17332, 556,   49408, 8984,  61632,
                31800, 46436, 14624, 24176, 39108, 436,   44012, 14996, 40812,
                48300, 64392, 32588, 43256, 37384, 63344, 53012, 33692, 57632,
                37008, 9648,
            },
            {
                60048, 55120, 6536,  43316, 50616, 33384, 52848, 41772, 10620,
                48664, 24796, 53000, 14080, 31548, 5420,  16404, 37548, 12928,
                28080, 2088,  65500, 34988, 8164,  42540, 49856, 18440, 5048,
                41280, 22932, 45068, 52892, 60724, 48304, 23660, 39084, 3992,
                20576, 8944,  4996,  18108, 39808, 59412, 24960, 54168, 32492,
                10516, 59656, 39884, 7768,  11920, 62544, 57428, 100,   13992,
                54684, 7508,  31520, 62372, 5912,  40732, 34040, 45708, 3196,
                27672, 65120, 22880, 54356, 29004, 3008,  52596, 36324, 64128,
                7040,  58712, 17832, 10832, 28192, 38472, 3536,  58208, 46836,
                17636, 52652, 8076,  13356, 60392, 56624, 23264, 15672, 41348,
                9044,  19564, 3144,  28736, 46140, 63560, 37144, 11460, 47880,
                39460, 15440, 61468, 11692, 31272, 65400, 21616, 44524, 25256,
                55128, 2924,  42808, 56328, 19580, 48940, 54300, 23496, 6860,
                29420, 19864, 12948, 8188,  17700, 23300, 2916,  16260, 49076,
                65072, 1260,
            },
            {
                28492, 34404, 21136, 14732, 9004,  37200, 19280, 28516, 56316,
                34176, 38124, 64260, 27920, 59500, 55412, 25708, 63780, 47072,
                42088, 56292, 31616, 19888, 4364,  62840, 37260, 11944, 30580,
                64580, 16008, 37056, 28948, 7080,  18060, 9552,  56244, 51244,
                41428, 46408, 57468, 33452, 52176, 43392, 20164, 3752,  16256,
                62864, 22444, 26240, 49264, 45820, 32532, 17388, 38048, 44364,
                24560, 59840, 17792, 37184, 13624, 49016, 54800, 24640, 36872,
                56964, 9500,  34688, 47984, 38548, 61936, 17368, 41164, 22080,
                14784, 44260, 32404, 1044,  62408, 49296, 25064, 32800, 9780,
                28724, 64636, 37284, 48820, 19820, 45808, 11688, 49404, 27096,
                59484, 43772, 37560, 52532, 13296, 6900,  52,    60868, 21740,
                8752,  58568, 51640, 4808,  46040, 37404, 3932,  15192, 35436,
                7144,  29068, 60152, 8664,  31124, 12244, 27424, 62568, 35608,
                58368, 38656, 53888, 27940, 46744, 59268, 40952, 26364, 43696,
                19412, 13224,
            },
            {
                52104, 40764, 62736, 48372, 24000, 2776,  65504, 13172, 21572,
                43744, 2960,  8280,  20688, 45404, 10296, 34716, 19604, 3840,
                23912, 9788,  48960, 26072, 45156, 15636, 24684, 57364, 48048,
                54412, 3672,  10436, 57152, 39816, 31324, 65312, 25976, 34128,
                14664, 63516, 24416, 11536, 28916, 7404,  36144, 56384, 41184,
                44836, 2016,  30876, 15384, 3860,  41736, 23696, 9420,  64180,
                33776, 10912, 46900, 1056,  23004, 30196, 8332,  62096, 12588,
                17916, 51304, 4884,  20172, 11680, 56732, 32856, 1440,  50884,
                27408, 38784, 52740, 46308, 21668, 7336,  53976, 19116, 40452,
                44780, 56,    26244, 42572, 3416,  32232, 6940,  53052, 34580,
                17244, 5784,  62072, 21928, 57012, 34720, 29968, 55040, 44620,
                27708, 18220, 35660, 24452, 56548, 27452, 53164, 59300, 40304,
                19028, 51908, 22148, 37676, 64860, 41956, 3004,  18520, 46036,
                10604, 1672,  49944, 61392, 4040,  35296, 10408, 56016, 7452,
                31856, 45936,
            },
            {
                25008, 10808, 4940,  31452, 58620, 54744, 39832, 46572, 5044,
                60320, 16584, 50140, 36384, 41512, 416,   48692, 30228, 57836,
                61972, 36228, 13656, 55112, 39980, 51928, 1968,  34596, 8012,
                27248, 19664, 33744, 61780, 21280, 1580,  45624, 12028, 3016,
                29528, 53228, 38264, 1272,  49000, 64916, 12940, 27576, 9040,
                33940, 52420, 55672, 65476, 20052, 58668, 53732, 49688, 2608,
                28408, 39972, 56532, 52308, 64828, 43616, 19456, 4532,  47064,
                31196, 42036, 59992, 27116, 43824, 8180,  24036, 47372, 60880,
                4560,  9080,  60096, 12784, 30436, 37036, 61076, 14028, 51448,
                5916,  21948, 53776, 16408, 62348, 28256, 37916, 63852, 692,
                24808, 47456, 32276, 9856,  42108, 18384, 49304, 14376, 40756,
                2816,  32396, 49560, 41892, 9384,  19536, 12240, 30852, 46628,
                62716, 10816, 48564, 16416, 6068,  34628, 57304, 50756, 31740,
                15824, 24792, 41688, 21036, 14848, 31028, 51384, 22296, 38988,
                61544, 3656,
            },
            {
                57092, 17488, 38360, 44740, 26928, 17052, 7712,  30848, 51996,
                26136, 32168, 58268, 12820, 23592, 61272, 52688, 14512, 7692,
                44484, 18036, 932,   60356, 28956, 11088, 59260, 21496, 43064,
                38800, 45992, 50788, 14364, 41976, 49496, 54320, 37476, 59928,
                18616, 6292,  21644, 45072, 16540, 58060, 22956, 47124, 61908,
                18724, 6576,  36400, 9996,  39176, 27732, 5852,  36016, 15244,
                45604, 19772, 4396,  26152, 15776, 38044, 59276, 28032, 52432,
                38980, 23452, 856,   16064, 50148, 63460, 29852, 13572, 55108,
                35516, 19932, 25640, 55840, 16864, 43240, 2148,  34940, 58964,
                31220, 61376, 12360, 34152, 57852, 9688,  21492, 44476, 55616,
                12260, 39824, 51232, 1944,  26540, 65040, 23228, 36772, 7532,
                56928, 63076, 13136, 1064,  60572, 39228, 49004, 5936,  24208,
                2224,  33916, 27044, 45036, 53588, 23088, 13980, 4512,  39320,
                54836, 63988, 33616, 5600,  56688, 45856, 64308, 792,   15388,
                35880, 47976,
            },
            {
                20600, 64196, 53396, 536,   12512, 50180, 35508, 23348, 63300,
                10256, 42548, 54176, 29356, 5944,  18324, 38500, 26112, 40852,
                32724, 53424, 22664, 37712, 7480,  32340, 46724, 16932, 63728,
                12592, 492,   55456, 24964, 4816,  28324, 16628, 22752, 43432,
                48064, 61368, 35024, 55228, 40552, 32144, 5608,  37956, 50536,
                13940, 29048, 43220, 24148, 46988, 13040, 31560, 62932, 22336,
                51452, 61048, 31276, 35488, 8860,  50216, 2284,  33632, 14456,
                7736,  63944, 54600, 33436, 37020, 5468,  39616, 18368, 42996,
                31312, 49672, 41372, 3348,  64928, 47832, 11124, 23384, 26900,
                49056, 38340, 7168,  46612, 24528, 51156, 41224, 14656, 30724,
                19964, 58276, 28384, 15472, 46360, 59168, 4064,  52484, 26116,
                45940, 20656, 29724, 53816, 22732, 33412, 63872, 17224, 57292,
                42876, 13732, 58352, 896,   29752, 61676, 43384, 26720, 59972,
                19404, 9152,  44424, 27416, 37832, 12356, 19120, 29540, 54952,
                26840, 8148,
            },
            {
                42756, 29388, 34788, 22436, 61716, 41396, 59372, 1600,  15252,
                38300, 19824, 2696,  47576, 64992, 33688, 57368, 10944, 50796,
                5240,  28280, 47188, 64660, 19760, 41968, 4156,  52840, 23972,
                29348, 58172, 31988, 9624,  63092, 36068, 6864,  56908, 9244,
                31804, 13520, 26504, 10292, 3224,  19476, 53572, 25544, 300,
                60372, 21248, 57968, 51208, 1224,  60068, 18312, 41916, 55968,
                11388, 6936,  43264, 12960, 58088, 40944, 25004, 55608, 61260,
                20996, 44852, 29364, 12324, 58772, 21212, 46180, 3460,  10396,
                63948, 15488, 6632,  33396, 22192, 28928, 54460, 41028, 4848,
                15456, 20088, 55960, 40188, 1620,  18024, 4880,  61324, 48772,
                7648,  63528, 35600, 54732, 6152,  33244, 12004, 17456, 61332,
                9668,  38596, 15832, 47416, 7236,  44196, 3184,  27636, 37804,
                53384, 18708, 64060, 40180, 9904,  19948, 36192, 7320,  47820,
                30188, 312,   49344, 59356, 23624, 6628,  41100, 49772, 44112,
                60276, 12800,
            },
            {
                51168, 6404,  14752, 47660, 9048,  28060, 19388, 44048, 48848,
                57048, 7352,  35072, 25084, 15400, 44916, 1880,  21712, 63552,
                16632, 58148, 3440,  12760, 51120, 25656, 62184, 35712, 7096,
                48840, 15880, 37564, 43716, 19224, 59136, 33316, 51728, 25308,
                552,   42136, 50092, 62984, 29676, 59472, 41932, 10836, 33532,
                45172, 39740, 4148,  16368, 34304, 53408, 8452,  25552, 3064,
                37488, 27848, 47896, 63028, 18056, 21812, 45480, 11280, 36840,
                3244,  17432, 48292, 9164,  52676, 26024, 61612, 56576, 28456,
                22940, 45152, 57868, 39000, 51000, 8872,  17884, 57224, 63544,
                44280, 10076, 28452, 65332, 31348, 54372, 36608, 25800, 33832,
                2584,  23620, 10400, 18808, 43404, 39992, 30216, 48716, 34156,
                42760, 3424,  64724, 25860, 36292, 58716, 13896, 49924, 9504,
                31660, 6960,  36488, 24696, 50004, 46116, 56276, 12148, 22876,
                53008, 36520, 11304, 17276, 54560, 62848, 34452, 9768,  3500,
                22760, 32992,
            },
            {
                2072,  60788, 37352, 54860, 32212, 3952,  56220, 11380, 33124,
                28368, 61912, 51516, 39296, 9172,  55528, 30540, 48232, 35720,
                43096, 24708, 39628, 33656, 15764, 56888, 30232, 11780, 40112,
                59864, 3156,  22220, 52896, 48148, 11928, 40364, 15412, 64776,
                38728, 58344, 20256, 7544,  36628, 15064, 46420, 64412, 17268,
                55468, 31136, 11876, 64052, 28276, 37992, 44760, 48792, 33552,
                59212, 16224, 23296, 53332, 340,   32476, 5836,  49216, 27184,
                42376, 58012, 24132, 40696, 2220,  15804, 34284, 50296, 37732,
                228,   52808, 26784, 12620, 62696, 1148,  36776, 46712, 32736,
                2612,  36100, 49880, 22764, 13188, 45824, 59004, 10936, 42248,
                52828, 45572, 37772, 50572, 62936, 20988, 55920, 1220,  58448,
                23732, 51412, 32008, 56296, 10232, 18252, 30492, 41608, 55788,
                21100, 45420, 52152, 4340,  15600, 31116, 2784,  65376, 41000,
                15432, 61996, 42820, 31708, 2768,  47288, 25924, 52812, 16796,
                56608, 40276,
            },
            {
                19460, 44596, 25336, 17256, 63820, 39120, 51272, 24868, 4684,
                45796, 22120, 12532, 58768, 18952, 41544, 26812, 13756, 6496,
                60568, 10456, 55772, 45048, 8228,  48512, 2064,  19140, 54632,
                44684, 27192, 64512, 6348,  30100, 1948,  21068, 45440, 4392,
                29952, 17120, 53004, 44160, 24188, 49380, 5028,  28012, 22500,
                8488,  50012, 25760, 47592, 20380, 4756,  61456, 13700, 19424,
                54440, 40176, 4804,  35168, 28648, 57244, 64520, 15872, 53748,
                30600, 6224,  35272, 65392, 31372, 44088, 7636,  12744, 18964,
                61016, 9236,  36008, 20524, 44092, 31072, 24552, 13408, 21092,
                52912, 60664, 16776, 5188,  42656, 7792,  27880, 21044, 16880,
                56864, 29128, 14228, 3208,  25016, 8484,  27744, 15084, 37508,
                19336, 12812, 5340,  40848, 21296, 52472, 61904, 24068, 2448,
                60024, 26724, 12644, 33504, 54748, 60628, 18208, 27360, 33648,
                4740,  25252, 20132, 51336, 39500, 14000, 20956, 30980, 37532,
                65264, 27852,
            },
            {
                48724, 57788, 10568, 5740,  42840, 21912, 13800, 36468, 64644,
                16780, 41872, 568,   31204, 49712, 3592,  62560, 53532, 20060,
                32024, 188,   29396, 21532, 61124, 36356, 42964, 22756, 32576,
                8368,  17784, 34928, 13824, 42148, 61348, 27396, 49832, 55696,
                23164, 10872, 33376, 2544,  57272, 12184, 34640, 52500, 2060,
                37552, 61140, 5776,  42532, 55080, 9916,  23676, 29712, 1560,
                65096, 9752,  45864, 50812, 12216, 41940, 38316, 20144, 9388,
                62132, 50484, 13112, 19368, 56092, 48784, 23148, 42168, 54500,
                32416, 47300, 15200, 60192, 5552,  53624, 40392, 58404, 8248,
                30044, 26052, 39588, 57736, 33584, 61796, 50528, 1212,  64552,
                39416, 6484,  60624, 32568, 57676, 46132, 53140, 5592,  62668,
                44612, 28812, 61036, 49212, 240,   45664, 37396, 6024,  16264,
                48008, 39656, 63020, 20412, 42172, 8640,  37936, 51164, 44568,
                58740, 55356, 7244,  27972, 9452,  56920, 60824, 5576,  46244,
                11616, 7432,
            },
            {
                23028, 31848, 52128, 28764, 46688, 1808,  58628, 30332, 53344,
                9492,  55136, 26260, 61224, 35912, 23048, 8316,  38812, 45648,
                50652, 37236, 53900, 17944, 5596,  27604, 57956, 14324, 47324,
                61864, 50772, 39400, 56760, 24044, 53992, 9812,  33004, 6104,
                36780, 47852, 63864, 27076, 39536, 20868, 62452, 41156, 58364,
                44216, 18956, 35460, 14984, 32564, 39504, 57652, 49640, 42968,
                32068, 26108, 20736, 61388, 7656,  24200, 47536, 2572,  34076,
                22544, 45636, 656,   37204, 27536, 4004,  62532, 29824, 4920,
                25116, 41112, 2188,  29148, 49112, 16916, 61924, 3648,  47804,
                42832, 11832, 384,   52144, 19840, 14972, 37152, 47268, 31556,
                12304, 22484, 48656, 16652, 35404, 11480, 41120, 31360, 48220,
                9412,  53928, 35428, 16644, 27380, 32756, 12904, 65128, 28796,
                35040, 10124, 616,   56784, 29044, 47252, 22560, 1340,  10316,
                13840, 37388, 46520, 64588, 35628, 44628, 1748,  40796, 54620,
                15712, 35836,
            },
            {
                160,   39988, 14496, 62892, 35060, 18560, 48408, 7084,  39404,
                20156, 33756, 47204, 5652,  16300, 44324, 56536, 28268, 17080,
                12180, 23684, 63436, 40572, 49816, 11144, 64964, 38104, 1124,
                25536, 10984, 4656,  31760, 47232, 16680, 37872, 62980, 14064,
                59252, 41920, 15468, 7152,  53840, 31656, 16612, 9036,  30292,
                13636, 24600, 56840, 51916, 508,   63452, 17020, 35996, 6508,
                52456, 15068, 38060, 55540, 17620, 31284, 60148, 14240, 56500,
                39640, 26292, 58340, 16288, 52944, 9836,  38788, 58432, 16836,
                64000, 52148, 21792, 56180, 33924, 10656, 27204, 38028, 18896,
                64404, 55500, 23748, 46568, 30688, 10128, 24680, 55308, 4768,
                44504, 26808, 54100, 42748, 352,   65520, 21924, 17328, 25548,
                2724,  39412, 22816, 7184,  60132, 55424, 8856,  43536, 50476,
                57796, 18124, 44544, 24104, 6232,  14384, 53852, 63596, 32108,
                49476, 21864, 2324,  16500, 32428, 23460, 18444, 29228, 24704,
                51344, 61848,
            },
            {
                18760, 59380, 4124,  54276, 24248, 11132, 61328, 27276, 44200,
                2908,  63064, 13264, 38528, 52752, 11052, 33252, 65328, 4908,
                59180, 47380, 2516,  14724, 34084, 24432, 4120,  51664, 29592,
                55452, 43776, 19544, 58824, 120,   7912,  44412, 20140, 26012,
                1408,  51920, 19596, 44732, 59784, 596,   47400, 25980, 50956,
                4388,  65220, 10724, 29092, 22184, 45416, 26916, 11564, 21388,
                58732, 3228,  44480, 34532, 1436,  49908, 43800, 28036, 51840,
                7524,  11592, 41716, 61228, 46532, 21932, 13872, 44932, 35360,
                8224,  11744, 37788, 42800, 7048,  45560, 23008, 51460, 32556,
                14852, 35232, 7332,  37848, 62852, 2996,  41240, 58896, 17564,
                34548, 63200, 9496,  20120, 29352, 37924, 52088, 58744, 34560,
                56392, 64324, 14480, 42312, 47788, 24668, 19288, 39352, 22356,
                4676,  31688, 51904, 37012, 59816, 34108, 40412, 26368, 17124,
                41912, 30020, 60504, 52332, 11520, 58816, 49996, 63832, 9052,
                33096, 42680,
            },
            {
                26632, 49460, 36976, 8516,  31168, 41096, 51872, 14868, 23176,
                49628, 56796, 29100, 24244, 58912, 21192, 1100,  41464, 25180,
                35028, 9204,  43420, 28372, 56560, 45516, 17560, 41800, 20952,
                36088, 13372, 61020, 23020, 35612, 28996, 50400, 57592, 39864,
                31444, 10144, 28352, 35680, 23832, 11516, 37740, 56272, 20304,
                45260, 32048, 40880, 48468, 8496,  3736,  53352, 41724, 62492,
                47028, 29104, 12652, 63484, 26516, 11008, 5672,  35924, 18228,
                64136, 32268, 20440, 6320,  29160, 33936, 51268, 2920,  19988,
                48672, 28056, 59612, 17840, 65268, 13816, 57960, 6520,  1884,
                28484, 49716, 60008, 17136, 43664, 53728, 28964, 21236, 7564,
                50680, 39008, 3540,  59440, 49840, 5120,  13832, 6976,  43040,
                11032, 19928, 31960, 51712, 3868,  36268, 58780, 1724,  53760,
                13652, 26296, 64084, 11308, 49288, 4200,  19616, 61240, 7780,
                57332, 5096,  25504, 38572, 6156,  42044, 14544, 36952, 4480,
                56832, 12488,
            },
            {
                6968,  43808, 21264, 16116, 45460, 57636, 732,   37584, 32308,
                10468, 18160, 42296, 7856,  46204, 30224, 48980, 14096, 51940,
                55712, 30784, 20092, 62604, 7148,  52992, 31628, 9264,  63240,
                5324,  49516, 27756, 40588, 52804, 64824, 11796, 5072,  22872,
                47512, 62384, 55756, 3888,  50540, 64508, 42208, 6836,  60432,
                36540, 2488,  17828, 61960, 37580, 57176, 30820, 15660, 33340,
                8692,  23864, 56836, 41244, 20072, 53200, 62240, 23184, 42452,
                3268,  54508, 49512, 36364, 1568,  64784, 25592, 55044, 39376,
                57644, 32964, 2536,  25612, 50008, 30392, 40116, 54968, 62924,
                44936, 10840, 22180, 4128,  26820, 13272, 47992, 35796, 61044,
                14172, 23112, 32256, 12044, 45456, 24508, 60928, 28208, 46872,
                50472, 916,   27072, 62012, 12096, 30264, 16004, 46868, 33516,
                61168, 42696, 7536,  16824, 29988, 56364, 45816, 11900, 51948,
                35260, 44356, 19684, 47180, 54728, 27668, 964,   45884, 21004,
                47752, 30048,
            },
            {
                62248, 33628, 55560, 63768, 26328, 6028,  20808, 65200, 54772,
                4356,  60756, 34920, 2044,  63808, 18580, 36060, 60452, 7496,
                17688, 39088, 50952, 12392, 35824, 22204, 59796, 15128, 57512,
                32988, 46128, 8308,  2832,  15020, 18740, 42584, 34296, 54220,
                16592, 7344,  41072, 32648, 14036, 18388, 28748, 33840, 15428,
                23576, 55400, 51568, 13060, 25140, 19144, 61164, 5288,  50848,
                39636, 18204, 48492, 4112,  31912, 38652, 57664, 13152, 48088,
                29672, 15148, 24480, 44128, 56200, 17632, 11352, 30612, 6596,
                23380, 14764, 46384, 5860,  36276, 20480, 10028, 34048, 24652,
                19340, 41468, 57220, 51836, 33724, 65168, 9344,  696,   45092,
                30288, 57620, 41964, 54832, 35704, 18380, 40372, 32816, 16180,
                23100, 38256, 55176, 44320, 8292,  40664, 64200, 27968, 9948,
                38040, 20232, 47760, 40040, 22460, 1800,  27720, 38184, 23356,
                368,   64364, 13760, 8728,  61752, 22344, 34228, 58096, 39420,
                17168, 52780,
            },
            {
                23636, 11336, 2032,  38644, 48808, 12876, 35224, 28524, 45984,
                40224, 26488, 51580, 15528, 54296, 39924, 4240,  27592, 42544,
                22704, 2896,  57824, 26408, 576,   40044, 47744, 3588,  26000,
                38636, 17764, 54808, 61944, 32172, 47784, 26308, 1908,  61268,
                37960, 12248, 46288, 21384, 57524, 48480, 5008,  53096, 10960,
                47184, 7944,  27764, 34456, 44168, 47888, 10828, 36496, 26064,
                716,   64360, 35188, 51680, 7560,  16200, 45240, 132,   34036,
                61040, 39292, 10464, 60112, 7852,  40872, 47856, 61500, 43160,
                50880, 63152, 40484, 54688, 61056, 1052,  43896, 15548, 48560,
                3148,  36020, 30168, 7884,  39168, 20216, 55740, 40580, 25992,
                52860, 2528,  16272, 27268, 62528, 3844,  56060, 9228,  63096,
                58192, 6200,  34132, 17364, 21828, 54000, 50144, 5352,  23888,
                56096, 2992,  58616, 32796, 53232, 63088, 42984, 15716, 48916,
                32852, 27164, 56440, 36240, 30456, 16072, 51856, 9868,  25700,
                59684, 3232,
            },
            {
                41516, 51072, 29112, 19112, 58988, 42104, 52776, 15932, 7372,
                19640, 11600, 45000, 31872, 9020,  25420, 56324, 11320, 46780,
                64480, 33484, 45300, 16424, 60984, 44084, 29316, 11804, 51576,
                21528, 42780, 11252, 24384, 36936, 57908, 9404,  50828, 30176,
                19284, 27724, 63656, 25240, 2352,  39096, 61692, 26392, 42852,
                63672, 39448, 59148, 1024,  6620,  40360, 22680, 55544, 60064,
                43608, 14884, 10552, 22508, 60820, 29400, 25220, 9132,  20760,
                56792, 4208,  19100, 52220, 28108, 22280, 36924, 15676, 320,
                9980,  19180, 28580, 12464, 22720, 51812, 58644, 28184, 53712,
                61176, 12668, 47084, 59436, 15248, 24368, 49472, 17748, 11652,
                64056, 20624, 6176,  47780, 10168, 21680, 48972, 1900,  42048,
                29764, 13212, 46388, 61060, 25824, 1336,  34632, 18872, 45640,
                51628, 15540, 26768, 5884,  13304, 35532, 8628,  59636, 54416,
                10024, 17820, 41260, 2620,  49564, 43588, 63432, 5220,  31968,
                14124, 35876,
            },
            {
                8356,  46908, 15220, 32696, 4412,  9756,  24408, 47960, 62020,
                56584, 22724, 59740, 37208, 50124, 20888, 61504, 31308, 52852,
                14804, 5308,  49784, 37856, 6740,  18752, 55380, 35032, 65300,
                1932,  56728, 30236, 51112, 5372,  21224, 44856, 14044, 41436,
                56344, 6192,  52600, 36076, 9684,  45076, 31200, 20444, 1712,
                16988, 30668, 21468, 14904, 63176, 52936, 29708, 3132,  20000,
                31152, 53512, 27908, 45960, 41564, 55072, 36844, 65100, 51444,
                26692, 45924, 35480, 31876, 13568, 3584,  57376, 33460, 26520,
                58888, 35256, 7528,  45088, 31192, 17252, 37596, 4524,  8832,
                39528, 17912, 26068, 1248,  43816, 5032,  61844, 31564, 7028,
                38504, 34072, 43480, 52320, 39356, 30656, 35080, 53692, 25144,
                20336, 52496, 4264,  39540, 56724, 14180, 43464, 62056, 12468,
                31036, 36500, 65308, 43976, 50392, 18676, 30752, 5052,  21568,
                46464, 60728, 52412, 23776, 6776,  20464, 12352, 40348, 54104,
                45424, 64888,
            },
            {
                20832, 56596, 61648, 22944, 54248, 63408, 30256, 1464,  33640,
                38428, 2716,  28740, 4956,  13780, 44016, 412,   17324, 37504,
                9180,  29484, 24468, 63424, 32244, 9720,  23324, 49080, 16036,
                27540, 7416,  45696, 14436, 63508, 39264, 3200,  59572, 23192,
                33196, 344,   42972, 59876, 17512, 51768, 12456, 58652, 55000,
                35916, 50748, 46184, 56268, 32976, 17384, 49104, 13236, 46364,
                7232,  38172, 59448, 1796,  13808, 5844,  49036, 17204, 40424,
                12720, 7120,  62112, 49088, 42724, 64336, 53736, 45656, 20932,
                52876, 38708, 48588, 64704, 5268,  42368, 25768, 63416, 32184,
                22648, 50080, 63912, 33148, 54016, 28312, 35920, 45468, 57808,
                50136, 28932, 59804, 24648, 13164, 64332, 15900, 44920, 11476,
                37288, 65088, 32560, 10020, 48788, 30716, 37656, 6656,  57648,
                41660, 516,   21184, 10748, 61128, 24140, 56744, 40784, 36908,
                3176,  28912, 11872, 38756, 58572, 33852, 47528, 28820, 1892,
                18256, 27500,
            },
            {
                5728,  34500, 888,   39900, 45344, 36380, 18416, 43668, 13396,
                53664, 48868, 16928, 64864, 57396, 34424, 48196, 23096, 41220,
                60032, 54428, 19648, 11916, 52096, 59124, 42236, 5864,  61308,
                37160, 41248, 60080, 34384, 18216, 28828, 53464, 35444, 65024,
                10744, 47240, 14816, 29828, 23516, 34308, 6664,  38520, 24996,
                8972,  3628,  11808, 24268, 37920, 8684,  26496, 65436, 41012,
                56352, 23856, 33732, 19004, 62688, 21852, 31828, 2948,  43888,
                30140, 22700, 54964, 956,   25076, 18532, 6436,  11888, 30460,
                4036,  16376, 24492, 56820, 34112, 10700, 48104, 14144, 44984,
                56716, 5928,  41884, 11228, 20908, 58864, 13548, 1812,  22736,
                14872, 8512,  18272, 852,   57156, 5100,  27488, 59904, 6816,
                49728, 17772, 27568, 22692, 2856,  59156, 20592, 28300, 9312,
                24800, 54420, 47612, 29176, 39052, 1972,  48624, 26348, 63812,
                15332, 32496, 45220, 65236, 26468, 16900, 55612, 60872, 23880,
                50244, 38484,
            },
            {
                44032, 52528, 25464, 11468, 14612, 7224,  51432, 59220, 25868,
                9352,  31256, 42768, 24304, 40208, 7824,  27688, 63288, 4496,
                34024, 43452, 2144,  47092, 39280, 27824, 1128,  31532, 46200,
                19316, 10572, 22584, 660,   49656, 10160, 25024, 7208,  16524,
                50060, 20864, 55316, 40148, 2872,  63260, 48652, 14716, 43312,
                64280, 29060, 41388, 61604, 4620,  44028, 58036, 31852, 11148,
                16944, 3904,  47916, 9748,  52452, 35484, 56376, 10732, 60348,
                52628, 37172, 16676, 10032, 40096, 29644, 37836, 50424, 62476,
                43728, 59624, 13416, 2004,  21516, 53916, 59788, 408,   19708,
                35524, 29628, 16048, 52328, 38244, 8192,  48380, 41064, 27092,
                53292, 63472, 46020, 37072, 32332, 47500, 19300, 39744, 55684,
                140,   43652, 60544, 51932, 41268, 16756, 46168, 64304, 50972,
                34412, 17696, 59756, 7860,  33496, 16744, 44656, 12060, 51324,
                7904,  56212, 19540, 1088,  9604,  42912, 4444,  36736, 9092,
                57576, 12640,
            },
            {
                17220, 60536, 31008, 47808, 57616, 28144, 22004, 4824,  40684,
                63108, 20436, 5824,  52488, 11220, 18912, 55660, 50544, 12504,
                16292, 26556, 56800, 35688, 15312, 21404, 54788, 13492, 25224,
                57768, 48280, 32368, 54568, 40384, 59092, 46696, 42440, 30320,
                38388, 26760, 61644, 8588,  45948, 27608, 53184, 21960, 31936,
                47464, 18144, 53056, 34216, 19744, 51428, 836,   22012, 35940,
                62000, 50944, 26736, 38992, 43092, 24512, 46812, 28172, 20408,
                4988,  63308, 33960, 57996, 47696, 60864, 15000, 24032, 9212,
                40428, 32712, 27904, 50784, 41020, 18068, 29024, 38400, 51064,
                9968,  62432, 2164,  46368, 24024, 64896, 19208, 55892, 5636,
                34712, 40032, 11724, 22108, 54840, 42232, 8868,  23928, 30912,
                34504, 14608, 7640,  36208, 12224, 54316, 6464,  23156, 14748,
                40236, 4748,  43204, 13736, 53016, 58320, 5524,  20392, 34544,
                42228, 23272, 37268, 50068, 31120, 53036, 22320, 15256, 32760,
                63256, 29512,
            },
            {
                3372,  9580,  41384, 20080, 65092, 37692, 32660, 46224, 16128,
                55024, 35812, 47096, 60532, 32824, 38384, 2308,  30856, 44884,
                59000, 21916, 8436,  62572, 4792,  44764, 64328, 35412, 51756,
                2928,  15516, 64108, 25944, 4784,  13464, 19916, 56228, 5668,
                52408, 3852,  34812, 13140, 57252, 18596, 5064,  59936, 648,
                55908, 13404, 6476,  58556, 28140, 15120, 39888, 54472, 6164,
                42608, 29692, 58748, 15448, 7324,  63924, 864,   16268, 39476,
                48476, 14104, 26272, 44608, 21072, 3808,  55836, 35392, 1060,
                19080, 54652, 7016,  45756, 36248, 62632, 7976,  24136, 55064,
                40676, 26744, 57488, 34572, 4648,  28804, 32708, 43444, 16228,
                61628, 4096,  29848, 51360, 2824,  60812, 13768, 48688, 63364,
                21508, 57660, 47004, 24992, 62868, 33604, 29916, 1240,  48116,
                56348, 31336, 63840, 26568, 22712, 36720, 62928, 29732, 53956,
                61656, 4564,  47152, 58100, 13640, 62788, 40832, 46800, 284,
                39632, 48352,
            },
            {
                21900, 35724, 55856, 6260,  2388,  53104, 10868, 60240, 196,
                29120, 12548, 3280,  27224, 15292, 62504, 46716, 20520, 7004,
                36916, 51320, 28900, 48452, 32584, 18156, 9592,  40748, 6992,
                28556, 37708, 8440,  44696, 29740, 36348, 63244, 32832, 12292,
                58484, 17464, 43328, 22452, 31232, 37276, 41440, 10544, 35240,
                26672, 39544, 23640, 45652, 10616, 48552, 61092, 25324, 46852,
                13428, 20928, 2396,  54916, 30988, 12892, 59284, 33556, 57260,
                7964,  42416, 1848,  31472, 8424,  52080, 26732, 46024, 63884,
                49124, 22260, 61072, 11904, 3828,  31160, 15792, 46896, 3216,
                13696, 21300, 43884, 17776, 49504, 12564, 59932, 9616,  24596,
                48708, 20056, 58508, 25972, 16692, 35252, 28296, 52700, 4820,
                10720, 39208, 29360, 19256, 4408,  44352, 39584, 61188, 36264,
                11392, 20992, 2340,  46536, 50240, 10184, 41136, 136,   25216,
                16640, 10664, 28344, 18236, 35324, 5720,  27868, 51648, 18840,
                25288, 54512,
            },
            {
                64008, 27412, 50120, 15752, 24308, 42772, 18276, 49744, 39424,
                22856, 58000, 50672, 44104, 23408, 9328,  53160, 25648, 65408,
                42144, 672,   11660, 40216, 24844, 58400, 50224, 22360, 61412,
                56616, 42792, 21336, 59648, 16704, 51612, 23032, 980,   45292,
                24360, 63760, 48168, 1860,  51132, 65348, 15396, 44956, 20432,
                62216, 49800, 2576,  64768, 36760, 31040, 3708,  17880, 33468,
                8756,  63600, 35136, 49108, 18828, 37760, 45032, 21600, 50704,
                29536, 23420, 54212, 65324, 13360, 39672, 33060, 17160, 10916,
                29972, 38260, 15280, 26008, 52492, 57680, 42052, 65364, 34280,
                59188, 31416, 7104,  61696, 55288, 39620, 168,   51860, 37180,
                31296, 44576, 9136,  41444, 65524, 6652,  44444, 37256, 18020,
                43340, 54752, 2548,  50524, 58516, 10064, 25668, 18820, 51696,
                27776, 59244, 37912, 17964, 7112,  32100, 57080, 14248, 48936,
                38180, 59720, 43620, 1836,  24392, 55192, 11196, 61548, 7848,
                43528, 13468,
            },
            {
                4604,  39160, 45136, 31608, 61680, 34732, 26892, 6560,  64232,
                32000, 16972, 41292, 6712,  59164, 34952, 40780, 13108, 16848,
                32916, 56164, 61284, 14808, 53540, 3068,  29776, 12856, 33092,
                18348, 48884, 12068, 53792, 2456,  39224, 9460,  49568, 40576,
                29268, 36024, 10224, 55036, 25316, 6208,  28808, 58232, 52392,
                8132,  32432, 17008, 42868, 21312, 7888,  57540, 38940, 52120,
                56188, 43964, 25524, 40612, 5204,  53608, 26648, 3524,  11324,
                60884, 17092, 37316, 49412, 19960, 59552, 4936,  57760, 42500,
                6116,  56416, 44072, 33920, 48032, 21996, 11024, 27716, 5328,
                49144, 52656, 10752, 25732, 35844, 15552, 46060, 21596, 57544,
                13564, 1956,  50592, 33444, 12556, 21152, 57016, 1572,  61884,
                26652, 32680, 64872, 22972, 16136, 48576, 55160, 13988, 7960,
                45056, 5252,  53896, 42284, 60648, 27984, 19296, 45192, 33360,
                6980,  50868, 30728, 64240, 38972, 45628, 21080, 30864, 37132,
                57884, 33180,
            },
            {
                10588, 19200, 58584, 12448, 8236,  47448, 57192, 14012, 44800,
                36660, 9084,  53744, 21088, 30364, 1468,  56480, 28404, 4960,
                47156, 23760, 19752, 36084, 6588,  43292, 38620, 46748, 108,
                25000, 5104,  34664, 30824, 46856, 62084, 27888, 18544, 60716,
                14640, 6768,  20292, 32740, 39480, 12128, 48396, 23244, 3356,
                38396, 12628, 55516, 27180, 51036, 14540, 62872, 28544, 22472,
                1236,  16172, 11276, 61108, 22884, 9332,  64792, 41492, 35664,
                55604, 45596, 3192,  10432, 41852, 28716, 22912, 50288, 36072,
                24744, 1552,  64356, 8556,  18360, 2196,  37948, 45216, 19940,
                16460, 38624, 42444, 22368, 4092,  30468, 64188, 27228, 6808,
                53580, 62236, 18744, 38412, 54080, 46992, 31464, 24172, 49980,
                15524, 6800,  13020, 41944, 37420, 32948, 5952,  40988, 64516,
                30976, 23992, 15728, 34460, 11924, 52032, 2952,  65012, 22536,
                56132, 12932, 20268, 53328, 8724,  15632, 59344, 48500, 2292,
                16536, 52564,
            },
            {
                60948, 24892, 704,   53420, 21952, 38452, 2000,  20472, 54660,
                25108, 2792,  62812, 38188, 48616, 18556, 61900, 50200, 37660,
                52824, 8844,  30680, 45724, 64632, 27008, 16904, 62252, 51388,
                54760, 41192, 65248, 7868,  24264, 13712, 55668, 33572, 3160,
                52544, 57060, 42560, 59660, 16948, 61808, 33836, 42952, 18044,
                30012, 47236, 59880, 34960, 192,   40692, 45184, 11624, 48892,
                37688, 31468, 57436, 45520, 34104, 50308, 30056, 15112, 19736,
                5960,  32612, 24688, 62076, 34784, 336,   15648, 63180, 12552,
                53176, 20504, 28332, 39936, 55272, 61804, 25364, 53392, 60620,
                29192, 996,   63188, 48108, 53860, 8916,  40884, 17728, 34164,
                42120, 29256, 23764, 5408,  27384, 59688, 8176,  11524, 40828,
                34444, 46276, 53060, 30096, 452,   62440, 27088, 21672, 35396,
                58212, 49576, 1412,  63348, 25436, 47452, 38888, 5460,  41560,
                27064, 36188, 3400,  41880, 26448, 33780, 4504,  41116, 23396,
                28444, 47056,
            },
            {
                42944, 30412, 35644, 41624, 63736, 28568, 51620, 30968, 11756,
                58636, 46404, 28004, 14184, 10548, 45200, 4024,  22232, 11484,
                43116, 60104, 2540,  55432, 11208, 20976, 58920, 9188,  35120,
                15232, 27364, 58240, 20008, 36716, 42192, 5932,  44264, 21560,
                37472, 47392, 26908, 420,   45768, 4916,  54236, 9612,  57344,
                63668, 24592, 5168,  10172, 53680, 24012, 32880, 4296,  19168,
                64460, 7512,  28040, 4104,  17484, 58496, 500,   47656, 61668,
                52076, 27620, 13976, 57136, 44124, 53796, 47396, 8712,  32040,
                45632, 60060, 14200, 49764, 30384, 6296,  33292, 12912, 8380,
                36128, 56100, 13892, 32676, 19428, 60552, 49932, 2524,  59352,
                10780, 47720, 15560, 63648, 488,   42648, 18516, 51936, 60780,
                20548, 56756, 9596,  60324, 18212, 45524, 56852, 3336,  43512,
                9240,  19240, 39940, 55992, 8136,  21172, 30576, 59476, 16936,
                9368,  61244, 46920, 17704, 62264, 50016, 12824, 56488, 65396,
                35088, 7020,
            },
            {
                14460, 56960, 49500, 17404, 5108,  14968, 60092, 42588, 4284,
                34848, 18012, 51080, 32204, 60676, 25284, 36444, 31664, 64076,
                17988, 26536, 34660, 15956, 50820, 37440, 31952, 5448,  22996,
                45284, 11376, 2968,  50732, 16884, 53204, 58972, 11432, 64788,
                24656, 8340,  13196, 30908, 51464, 22076, 27292, 36748, 1608,
                40940, 14928, 44948, 20384, 62156, 16156, 55824, 59536, 25764,
                42436, 54404, 47516, 21064, 38776, 12124, 43108, 24192, 38080,
                9012,  40868, 48636, 18356, 7696,  30760, 25392, 38052, 18804,
                41036, 3056,  35344, 10632, 43736, 17208, 58228, 50552, 41796,
                24016, 46176, 6844,  27448, 43968, 11912, 37428, 25928, 45716,
                21216, 56472, 39796, 50256, 34008, 14472, 36432, 25788, 4312,
                38368, 2248,  28112, 49284, 24628, 11760, 50740, 16616, 53704,
                12844, 28784, 46340, 32328, 15092, 36860, 54188, 12596, 48364,
                52960, 32640, 23052, 57132, 7316,  29616, 37940, 18924, 9584,
                51192, 21648,
            },
            {
                62592, 3304,  10152, 26600, 46444, 33636, 9676,  49400, 38836,
                22588, 65164, 7632,  592,   42176, 55312, 15620, 6944,  40204,
                57068, 5492,  48604, 41640, 23908, 1504,  43864, 56740, 40168,
                63008, 29636, 38736, 46196, 32436, 1048,  27232, 31268, 16392,
                49952, 34912, 61252, 18936, 38092, 64244, 14380, 50204, 20004,
                32252, 52292, 28392, 39068, 49096, 29932, 7088,  36136, 51504,
                10352, 33352, 14204, 61948, 52580, 27108, 55732, 31808, 4484,
                59272, 21848, 64620, 4048,  39032, 21112, 61212, 57580, 5236,
                51140, 26480, 54764, 23204, 63448, 236,   39200, 21456, 3596,
                64752, 18396, 34736, 57920, 1936,  22796, 55344, 15040, 4556,
                36492, 7068,  30948, 9844,  58688, 45908, 55260, 64452, 30624,
                44304, 15924, 22100, 35972, 40144, 4752,  34160, 30260, 61760,
                37800, 23136, 59632, 3548,  44176, 62836, 736,   24604, 43212,
                28940, 2316,  37612, 14476, 43596, 428,   53904, 44796, 27256,
                1816,  39844,
            },
            {
                33200, 19676, 53828, 37332, 57880, 24112, 19292, 61708, 26476,
                15352, 43908, 54240, 39220, 20104, 49280, 59104, 27352, 51884,
                13664, 21524, 30136, 61976, 7732,  53716, 28228, 18072, 49332,
                4164,  20844, 55584, 8812,  62564, 22244, 48436, 39568, 3912,
                57816, 41540, 2820,  55076, 10312, 41888, 7056,  45176, 60720,
                11544, 56400, 7716,  35808, 1916,  42836, 12860, 46296, 17612,
                1792,  23612, 41176, 3236,  35828, 7204,  63236, 18620, 11604,
                46272, 29504, 34004, 12516, 56032, 50328, 2256,  11772, 29232,
                64972, 16084, 7180,  46560, 37076, 27084, 47912, 14768, 31300,
                54348, 10192, 49276, 61608, 40356, 30708, 51380, 65056, 32772,
                60264, 53296, 25260, 19812, 3920,  28712, 22404, 6360,  10672,
                48400, 61652, 57892, 7384,  55032, 64676, 21144, 42012, 976,
                48540, 6716,  52336, 11176, 26904, 49740, 18288, 34344, 56856,
                6376,  11852, 64740, 51892, 25352, 33024, 21352, 63148, 15796,
                55780, 46680,
            },
            {
                11948, 29012, 43692, 64748, 7392,  40468, 908,   53188, 5696,
                57384, 10796, 27524, 33928, 23840, 9028,  2404,  44368, 35152,
                46640, 400,   38860, 58008, 14568, 46172, 63892, 9532,  13380,
                31536, 60588, 15856, 25968, 41368, 13256, 59856, 7240,  52224,
                20680, 27948, 45536, 23744, 32768, 58720, 29296, 25112, 35204,
                3572,  23172, 65156, 17752, 58244, 22272, 63396, 27472, 39676,
                60460, 55436, 29908, 49244, 16452, 45976, 22512, 35000, 54036,
                15788, 1852,  52228, 43764, 27016, 35756, 15304, 46380, 33608,
                43012, 21888, 58704, 31884, 18972, 56640, 9024,  60248, 43460,
                27988, 38588, 14416, 25276, 5016,  17116, 8636,  42908, 28052,
                12160, 17248, 44288, 62972, 41568, 12492, 49696, 17320, 54108,
                33660, 41844, 12804, 31376, 45080, 15360, 8924,  58032, 26048,
                17996, 33736, 65424, 20968, 41148, 58672, 8928,  39852, 61888,
                20376, 46488, 40904, 18104, 59076, 10512, 47876, 5804,  31080,
                36792, 24088,
            },
            {
                59112, 51900, 4492,  21992, 13456, 48828, 29152, 45168, 36672,
                31396, 48024, 3528,  59584, 13496, 64384, 30512, 19532, 10304,
                62944, 55240, 25520, 11312, 33012, 20540, 35672, 25136, 52864,
                37932, 47708, 34468, 2140,  53764, 35980, 18576, 44688, 33288,
                9576,  13956, 63252, 5656,  16436, 48796, 1036,  55548, 16920,
                47624, 42008, 50436, 26604, 10260, 53256, 32052, 5536,  48392,
                20148, 8276,  65428, 25576, 10856, 59416, 2372,  41952, 49464,
                37232, 58164, 23236, 17552, 8536,  63660, 40888, 24048, 55448,
                9120,  38832, 49324, 4632,  12252, 52476, 34116, 24620, 5264,
                51084, 1152,  20896, 45584, 56524, 35692, 47712, 20288, 3532,
                49228, 37684, 1856,  51584, 32624, 57000, 37312, 40528, 24216,
                3720,  19480, 26164, 1756,  50236, 28680, 38512, 47112, 13340,
                55532, 43008, 28388, 15976, 35884, 4696,  30104, 22988, 15240,
                51144, 26796, 30792, 8040,  35564, 54828, 38984, 13076, 42668,
                61512, 8100,
            },
            {
                48444, 40956, 16372, 31980, 55796, 34864, 62988, 17568, 12600,
                21804, 62136, 18424, 52312, 45988, 36308, 40960, 53620, 23492,
                32288, 7276,  18420, 51512, 40912, 5984,  60308, 42388, 2892,
                58796, 7492,  23372, 43836, 64576, 10628, 30080, 25128, 62016,
                54244, 36704, 47900, 40024, 53492, 21028, 12828, 38564, 62660,
                30712, 5456,  13284, 33816, 44864, 3316,  38348, 15264, 57032,
                34000, 13424, 37500, 44020, 51796, 32776, 28328, 8476,  64132,
                25688, 6444,  40132, 60520, 32420, 48164, 5772,  18248, 51972,
                1400,  14032, 60972, 28688, 44672, 2772,  39644, 62412, 17968,
                35908, 58124, 63340, 32972, 11628, 53144, 62428, 24332, 41024,
                55680, 61820, 22052, 14812, 26608, 8284,  804,   59820, 29200,
                63652, 51960, 36404, 62616, 18320, 53524, 22776, 5872,  63024,
                36616, 9284,  3824,  57212, 51368, 13068, 47164, 55336, 1868,
                36544, 59576, 4432,  50168, 23388, 3152,  60640, 26752, 52788,
                3996,  17904,
            },
            {
                26276, 1392,  37004, 60140, 26992, 10044, 2756,  42100, 54960,
                8036,  38724, 42612, 29444, 6012,  17180, 57028, 4452,  15080,
                49768, 43468, 36216, 64408, 27312, 1708,  50268, 15460, 28856,
                19800, 12388, 51284, 28160, 5704,  47488, 57320, 220,   39240,
                17128, 29608, 2132,  25916, 34644, 64732, 42900, 52244, 9524,
                19660, 59064, 39624, 55764, 20716, 60940, 50724, 24344, 43216,
                464,   53372, 21896, 5816,  18856, 39548, 55568, 13216, 19868,
                31280, 45400, 11140, 54452, 1108,  21556, 58068, 31212, 62292,
                26860, 34808, 19984, 41592, 64556, 16380, 22040, 54792, 12788,
                47100, 8688,  29172, 42352, 6148,  27140, 308,   14304, 33492,
                9660,  29408, 6072,  35496, 47580, 65284, 20584, 44516, 13472,
                9232,  46860, 6552,  43048, 10504, 33536, 60804, 2232,  30888,
                50064, 23800, 45964, 32044, 61232, 25156, 42404, 64440, 32960,
                10336, 43456, 13920, 63836, 45552, 16092, 29892, 19252, 49416,
                22844, 34120,
            },
            {
                45492, 54912, 8716,  19996, 46756, 51772, 23056, 58880, 26028,
                49432, 34536, 840,   24752, 11728, 48384, 26464, 61340, 38424,
                28348, 59456, 3488,  16184, 47036, 56972, 22480, 32152, 54616,
                45880, 40104, 61672, 16952, 38580, 21320, 14532, 50696, 5968,
                22520, 55884, 12172, 60124, 8888,  31100, 6248,  27976, 23772,
                36684, 46212, 668,   28844, 16324, 8312,  30252, 11260, 63676,
                27220, 47160, 30772, 57440, 3484,  62772, 24156, 43516, 328,
                61248, 15132, 50440, 36552, 28472, 13820, 43900, 10196, 37228,
                45104, 57112, 7280,  51300, 25456, 33008, 48760, 6324,  30872,
                40488, 22772, 15032, 52100, 18524, 39152, 57064, 46484, 59896,
                18316, 45132, 52376, 58980, 42288, 16888, 30828, 50416, 35280,
                56660, 22036, 39124, 58380, 14648, 25400, 40984, 44924, 19528,
                14956, 54724, 39768, 200,   17628, 8320,  37600, 5424,  18428,
                54292, 24488, 39144, 20568, 56128, 34388, 41356, 1112,  57992,
                10644, 64448,
            },
            {
                30204, 12752, 62068, 43112, 5404,  39276, 14132, 32348, 4280,
                63616, 15740, 54204, 58028, 62540, 33300, 8564,  45060, 1068,
                12572, 21688, 54400, 30836, 10716, 37776, 44208, 6784,  65452,
                9988,  33388, 56148, 3404,  31496, 59328, 42056, 34684, 46148,
                62992, 40980, 48988, 19496, 44220, 15852, 47224, 2596,  60336,
                14856, 53172, 25044, 62320, 47884, 42244, 36900, 58312, 18436,
                7220,  40668, 61540, 12140, 35732, 15984, 47292, 52508, 34400,
                56556, 27236, 3684,  18904, 62508, 39004, 25124, 49608, 5224,
                16696, 23016, 11456, 37796, 1028,  59384, 10100, 43680, 26560,
                56124, 60512, 4080,  45140, 64840, 31264, 21640, 36604, 7192,
                25856, 39704, 13160, 23944, 4188,  10920, 54392, 25780, 2416,
                16040, 32484, 27484, 388,   54572, 49100, 7876,  35172, 59704,
                27328, 10880, 64120, 20800, 49040, 28224, 52256, 22112, 45436,
                29372, 49240, 664,   32136, 11436, 6548,  62512, 44712, 36484,
                15180, 39408,
            },
            {
                20724, 50312, 25092, 15840, 29516, 65344, 35960, 19260, 45380,
                28536, 10156, 20360, 43872, 37224, 22188, 50960, 18816, 55720,
                34744, 41532, 48612, 7928,  24832, 60956, 13120, 18540, 36436,
                1428,  21764, 25880, 48996, 8864,  53124, 24720, 11648, 27864,
                8088,  31916, 4376,  58016, 24524, 35684, 56388, 50580, 41008,
                31924, 7520,  11956, 34892, 5792,  23424, 53964, 2732,  34624,
                50108, 15444, 23860, 44752, 50560, 28888, 6736,  10212, 38464,
                21944, 41776, 47608, 33256, 52428, 56308, 3120,  65448, 29284,
                52756, 60396, 47772, 30524, 53996, 14692, 36584, 63628, 19372,
                2564,  49592, 34292, 24396, 8260,  12608, 50212, 3876,  53696,
                64116, 31696, 1472,  56088, 33176, 38924, 59528, 7356,  41728,
                61180, 52868, 18696, 62080, 31096, 20964, 12164, 56628, 4580,
                51636, 37196, 6392,  44396, 34372, 59608, 11596, 62192, 14688,
                57108, 8504,  60012, 27520, 47524, 53244, 24412, 9100,  27388,
                55636, 6272,
            },
            {
                2468,  41552, 35132, 58316, 208,   53920, 48744, 7380,  56636,
                40296, 51632, 31420, 5580,  14112, 2648,  29780, 39860, 65172,
                25164, 5384,  62748, 20068, 33804, 40320, 53508, 27152, 48732,
                57988, 43332, 14068, 37136, 64152, 19204, 2888,  60600, 51556,
                20424, 15328, 37556, 52712, 600,   62200, 21204, 10864, 17872,
                63788, 44276, 56764, 49988, 19000, 64296, 13812, 45112, 21340,
                56056, 37964, 9124,  1276,  58544, 20196, 32300, 64948, 17152,
                4796,  58828, 9440,  23804, 6612,  12072, 42476, 20620, 14756,
                33380, 40440, 4140,  18340, 45364, 23820, 5080,  51716, 32472,
                11528, 16444, 37628, 54140, 41744, 58528, 28200, 17240, 42928,
                11072, 47144, 20676, 62784, 49188, 18608, 45744, 22904, 36596,
                48752, 9860,  43632, 5956,  45860, 38136, 64500, 29612, 17028,
                42864, 22628, 30420, 57380, 15452, 2368,  38976, 31340, 3804,
                35368, 40916, 16772, 63372, 37300, 14440, 19956, 51180, 31692,
                47108, 60908,
            },
            {
                33072, 52772, 7980,  47368, 22072, 10932, 25432, 61204, 13268,
                1700,  23932, 48128, 64088, 41048, 53284, 58892, 15836, 10576,
                31592, 52372, 17036, 45360, 58300, 112,   5544,  62404, 30924,
                16468, 52188, 5160,  29144, 45384, 33972, 16104, 38200, 56140,
                44604, 65232, 27068, 42424, 13064, 28840, 39172, 33600, 5192,
                27188, 21760, 38448, 30152, 1832,  40092, 26952, 31712, 4764,
                62740, 28016, 33036, 53940, 39436, 25820, 42580, 55184, 48660,
                30296, 13660, 63732, 44868, 16992, 30132, 35236, 47116, 104,
                55760, 9140,  27600, 64204, 56748, 41696, 28976, 58632, 39756,
                46752, 61700, 28588, 20124, 1612,  47296, 62324, 34604, 23604,
                58052, 15380, 37884, 26576, 5644,  28920, 14024, 64236, 3776,
                29736, 12956, 24588, 34792, 15372, 3308,  50928, 23652, 47052,
                1376,  62356, 12648, 52620, 25560, 41984, 55832, 48096, 26412,
                20224, 51952, 2864,  22688, 43892, 5124,  58876, 40056, 4020,
                17236, 23332,
            },
            {
                10300, 63592, 18600, 38680, 31772, 44360, 17144, 42132, 32820,
                38140, 59596, 16504, 27300, 9316,  46328, 23768, 6708,  49060,
                43644, 2268,  37064, 13684, 29568, 51484, 22896, 42328, 9016,
                24756, 39552, 59748, 11268, 54604, 6096,  43044, 30588, 1744,
                10680, 34520, 6592,  18716, 49544, 8364,  44876, 53068, 58936,
                48600, 3604,  15096, 60444, 9976,  52556, 59264, 47504, 11672,
                41672, 17932, 60116, 14412, 5020,  62140, 12476, 2156,  24448,
                35748, 53720, 39164, 26804, 57460, 50548, 59940, 38232, 24804,
                61744, 44108, 21360, 35340, 12232, 8048,  17064, 21612, 396,
                25600, 8852,  57036, 5604,  32996, 14796, 9508,  39952, 2788,
                29796, 49724, 8592,  43628, 53260, 34972, 9644,  50752, 19972,
                57772, 40064, 63644, 48200, 59644, 27800, 41336, 9636,  55420,
                32896, 39600, 8384,  47332, 19072, 5876,  22840, 9680,  65196,
                44968, 57936, 33052, 10276, 55280, 28420, 34780, 64716, 12540,
                43408, 57520,
            },
            {
                29096, 14296, 26484, 3716,  56040, 63164, 2588,  28116, 52948,
                20824, 4728,  35472, 54904, 19688, 32656, 36372, 62004, 27808,
                20664, 60352, 56288, 26404, 9924,  47684, 14948, 35220, 46396,
                3024,  63588, 20016, 47248, 22924, 26664, 62780, 49316, 21660,
                25584, 47892, 60180, 54260, 32240, 63540, 2980,  24364, 13444,
                42256, 35800, 54352, 25332, 43648, 33988, 16588, 7836,  24636,
                51980, 2436,  46684, 22288, 48924, 35104, 18788, 45496, 60832,
                6316,  19796, 824,   33976, 4076,  22252, 7644,  10824, 17920,
                49756, 13628, 39076, 2272,  50744, 61428, 33472, 49008, 55020,
                14060, 44312, 40908, 63412, 52764, 26580, 21444, 45432, 51864,
                6100,  35456, 60232, 17876, 172,   61636, 40472, 55744, 25920,
                33424, 1072,  53404, 21984, 8612,  18852, 58024, 35648, 13676,
                20260, 59968, 28008, 36256, 63368, 33236, 51068, 14388, 36480,
                6584,  12880, 25676, 50516, 41392, 17744, 2104,  48156, 26004,
                54280, 35832,
            },
            {
                49012, 45040, 59072, 51876, 12432, 34944, 50364, 8532,  57672,
                47444, 11632, 44284, 60896, 572,   13288, 56984, 4256,  39256,
                12328, 33900, 7440,  41600, 64756, 38284, 20696, 59392, 54884,
                12684, 31928, 35636, 564,   38964, 58488, 13388, 7668,  40292,
                57764, 13888, 4176,  23224, 37216, 15656, 55984, 19776, 30004,
                8992,  65060, 19304, 47040, 6088,  22140, 38056, 55736, 64900,
                35572, 30120, 42904, 9912,  27052, 53168, 7504,  39732, 28412,
                51540, 42896, 62040, 46652, 14048, 40876, 64140, 54396, 32272,
                27120, 5904,  53628, 31132, 25488, 46264, 4500,  37572, 65288,
                30840, 35384, 22676, 17088, 48572, 36904, 60920, 55520, 18932,
                65184, 25384, 54388, 13148, 32132, 23364, 44348, 15184, 7752,
                47088, 17104, 5512,  37480, 32208, 51832, 1988,  25328, 43796,
                49680, 3748,  16132, 57776, 848,   43276, 28984, 60692, 40496,
                18700, 30008, 60168, 46768, 7476,  61748, 30900, 21584, 8352,
                38880, 776,
            },
            {
                5948,  21876, 40928, 6872,  30052, 19592, 39820, 23508, 15212,
                64892, 25912, 7156,  49720, 29264, 42508, 50816, 17584, 46064,
                54716, 22828, 50292, 18536, 4404,  52956, 32544, 7216,  18164,
                28432, 50864, 56464, 16860, 9400,  50356, 32688, 18900, 52672,
                36112, 30164, 45356, 41044, 10460, 28072, 47068, 61300, 38584,
                51020, 116,   32456, 12092, 61892, 29244, 1204,  49160, 13420,
                20372, 6984,  56496, 37376, 63744, 31332, 14944, 56332, 9556,
                22376, 16096, 11288, 30936, 52692, 18848, 28696, 43224, 1804,
                58788, 45732, 63216, 18632, 55656, 15024, 43208, 10360, 19192,
                6640,  50464, 3320,  11708, 30192, 7608,  604,   13440, 32424,
                10704, 42324, 21884, 39396, 57904, 48516, 3052,  30584, 38156,
                60052, 43148, 28968, 57004, 14300, 45568, 61532, 6120,  65388,
                30800, 53220, 24660, 45592, 11992, 54028, 17212, 3332,  55444,
                49488, 304,   38668, 21104, 14196, 36940, 51788, 58304, 15592,
                62340, 19136,
            },
            {
                31996, 64608, 36528, 15640, 24860, 43564, 62396, 4180,  37616,
                30816, 34136, 17372, 39388, 21580, 64316, 25132, 8672,  30396,
                63208, 1252,  44024, 31140, 58696, 24108, 1676,  62664, 39780,
                44224, 4724,  25808, 42536, 64996, 29072, 4872,  43732, 62088,
                800,   16792, 64184, 20492, 58976, 1520,  42636, 6472,  34144,
                16600, 55748, 27496, 39556, 50304, 57800, 18736, 44680, 40520,
                26376, 61196, 16168, 50276, 272,   20048, 59628, 47904, 32780,
                37972, 57172, 49372, 25492, 58436, 37248, 4968,  48356, 34492,
                15812, 23200, 36576, 8272,  40432, 22616, 60416, 27664, 52664,
                24852, 59616, 55632, 45304, 40156, 57804, 23868, 43600, 38344,
                28476, 2296,  46816, 5184,  28128, 11300, 19176, 64956, 51792,
                21488, 10040, 62904, 23996, 40340, 11492, 28304, 38604, 17496,
                10428, 40816, 7296,  21176, 31412, 38892, 8300,  27048, 44192,
                23208, 33728, 62768, 4488,  57084, 25036, 10928, 44892, 33820,
                42184, 50980,
            },
            {
                27504, 54116, 9808,  60576, 49156, 56844, 10876, 46704, 55204,
                1200,  43416, 52644, 59024, 10484, 37408, 2604,  53348, 35268,
                15156, 25832, 10696, 36600, 14092, 45996, 27444, 49392, 10004,
                22644, 60468, 14276, 48144, 21244, 37292, 54944, 23824, 10768,
                27584, 55276, 8216,  48764, 34760, 52264, 25796, 12544, 21908,
                44748, 59868, 23724, 4972,  15480, 35248, 10412, 31060, 5732,
                52728, 32840, 11540, 23872, 44508, 5088,  41144, 24980, 1752,
                65272, 6876,  35068, 2412,  9032,  23600, 61192, 12700, 20728,
                52364, 41612, 11328, 29544, 752,   57268, 32360, 2868,  46548,
                12656, 38648, 33224, 20476, 64504, 27400, 51096, 17508, 49072,
                56592, 62856, 15708, 51128, 61964, 37148, 54784, 6204,  27012,
                13332, 35604, 48496, 1596,  19652, 55056, 50228, 34488, 22684,
                48464, 56680, 37032, 63960, 49308, 59776, 19808, 35284, 64228,
                11020, 52760, 16304, 45776, 31900, 40628, 1452,  28260, 5000,
                23596, 12792,
            },
            {
                3456,  17756, 46620, 1020,  33108, 5528,  17520, 28836, 21348,
                13560, 62060, 23976, 3620,  15492, 32088, 44940, 19612, 57744,
                41128, 47640, 60704, 56036, 7940,  40680, 16684, 34016, 57452,
                36688, 52616, 7992,  33664, 1896,  58044, 15196, 46312, 34248,
                50656, 38952, 24880, 3740,  30592, 17892, 62700, 54848, 48648,
                3044,  10072, 41400, 62884, 45892, 53240, 23080, 63220, 58540,
                2560,  46392, 39580, 57532, 29548, 35780, 52468, 12852, 44944,
                17460, 27368, 41960, 62876, 45484, 16760, 55988, 39488, 7420,
                65112, 3772,  49868, 62232, 47136, 13248, 36180, 42152, 62916,
                16428, 29052, 1432,  9540,  15680, 4044,  34676, 60020, 5012,
                8956,  35872, 20040, 32636, 7628,  24608, 45620, 34232, 41604,
                58116, 4236,  53520, 44372, 32928, 7844,  58492, 15076, 4664,
                60684, 27308, 1952,  14016, 25512, 4304,  46676, 57196, 14740,
                6052,  42240, 28644, 8744,  53640, 18292, 65160, 54796, 47424,
                60332, 57144,
            },
            {
                34992, 43304, 22596, 38912, 27036, 53080, 35984, 60204, 49896,
                40252, 8988,  46332, 28944, 55880, 48580, 61660, 12460, 5980,
                28756, 3564,  20616, 50712, 29524, 63636, 53684, 12288, 20128,
                2656,  30552, 41452, 61436, 11968, 27112, 40752, 3116,  18568,
                60440, 13192, 42892, 58084, 14420, 36812, 8764,  39216, 31976,
                28676, 51956, 36332, 20428, 8468,  3644,  27828, 42308, 16784,
                36632, 21452, 6636,  64672, 17900, 10448, 62260, 21736, 30516,
                48556, 55296, 20316, 13960, 31940, 51200, 26980, 44308, 30504,
                24732, 55388, 33480, 26680, 17076, 51888, 20560, 6904,  54680,
                22104, 49536, 61024, 42776, 47732, 53968, 11372, 30740, 22824,
                45840, 25624, 53872, 43036, 12920, 59964, 16056, 748,   50368,
                18352, 25356, 30520, 15784, 64036, 26332, 2708,  46084, 41668,
                31612, 18720, 44268, 33800, 51684, 10140, 40992, 22408, 31584,
                48208, 25628, 58760, 35856, 23024, 50036, 13072, 37464, 20200,
                8680,  39572,
            },
            {
                29276, 11640, 62256, 57516, 19680, 13184, 42420, 7552,  25248,
                33324, 19108, 57976, 36040, 6956,  26532, 39760, 22740, 51152,
                65208, 38072, 17108, 33372, 684,   22372, 5564,  42556, 47228,
                64104, 15420, 24184, 45616, 19700, 52964, 63060, 8604,  48076,
                30884, 5744,  21364, 51348, 45012, 60340, 23816, 5348,  64616,
                13168, 17352, 58932, 26016, 33276, 55900, 37984, 12520, 50916,
                25604, 55244, 13748, 48260, 26828, 54064, 7388,  37724, 59944,
                4160,  9884,  39040, 53656, 4672,  36092, 512,   10852, 59320,
                37096, 14360, 19420, 42940, 5260,  58948, 30596, 9732,  39196,
                34064, 5380,  56432, 25292, 36532, 21156, 39464, 63920, 14880,
                41040, 59096, 1776,  28788, 39156, 48856, 21148, 31652, 62548,
                8880,  38848, 60304, 11248, 41252, 36640, 20236, 62360, 53476,
                12108, 8200,  55088, 62596, 17048, 29084, 54536, 1280,  61396,
                38120, 2640,  19856, 63476, 3676,  43232, 6648,  32980, 26236,
                15908, 52696,
            },
            {
                48152, 5992,  50684, 3088,  31184, 65440, 45896, 2328,  54780,
                63480, 4912,  12708, 42060, 52288, 17868, 28,    34244, 43844,
                8904,  24572, 54812, 11576, 45288, 36196, 60256, 25756, 8736,
                28340, 38292, 55916, 6236,  49872, 36132, 29308, 22308, 56600,
                37808, 65316, 33620, 10536, 27628, 532,   47496, 42212, 20116,
                56304, 44232, 1592,  49836, 14292, 65256, 47244, 524,   60616,
                9068,  30160, 34680, 42216, 1288,  32448, 43184, 15224, 51276,
                34636, 24004, 61004, 28728, 47868, 21788, 63116, 18120, 49260,
                2828,  46664, 8596,  61288, 38116, 24356, 48768, 64340, 26344,
                45576, 11580, 18180, 31516, 62584, 7764,  44600, 972,   56068,
                33764, 11572, 18480, 64688, 4192,  57236, 10088, 44144, 28164,
                54896, 46248, 5752,  23104, 48704, 52132, 9416,  29008, 23896,
                35336, 48160, 21636, 39892, 6488,  36728, 44808, 12728, 18588,
                50396, 55148, 11840, 46228, 15272, 56868, 29808, 61208, 41860,
                152,   63784,
            },
            {
                19124, 32876, 25720, 15288, 40504, 9192,  22808, 30220, 16396,
                38212, 51256, 31460, 21276, 64192, 10384, 59996, 54516, 14156,
                31588, 46600, 61828, 40120, 27200, 49808, 18408, 55172, 32908,
                50644, 58804, 32,    17288, 32552, 10940, 4568,  43636, 16032,
                25876, 2804,  54040, 40800, 19364, 56788, 16024, 53576, 25468,
                33884, 38404, 6928,  30452, 41748, 21968, 17800, 31620, 43932,
                19524, 49600, 61576, 6368,  59068, 20772, 47328, 25940, 2300,
                57500, 16872, 43788, 12408, 8084,  56956, 41684, 33044, 54200,
                22580, 40232, 29728, 56172, 1996,  12364, 43560, 15720, 860,
                60296, 52512, 41448, 2684,  13880, 28212, 51732, 19272, 29292,
                49588, 7500,  52648, 46596, 34872, 23404, 52016, 37608, 2452,
                14272, 20500, 35200, 56312, 880,   16820, 57668, 39308, 3976,
                42924, 59568, 144,   30440, 57444, 23328, 64584, 27252, 34876,
                7400,  41580, 28992, 33544, 39384, 24456, 48252, 10204, 55600,
                45340, 23728,
            },
            {
                38168, 58904, 44580, 54892, 36460, 48896, 52796, 59676, 10660,
                44180, 27316, 48272, 3376,  24984, 44584, 37704, 27960, 20176,
                57940, 4776,  15844, 7616,  58220, 2568,  14536, 39260, 4088,
                13080, 21440, 44052, 26492, 64652, 54868, 39232, 59988, 51992,
                12024, 45944, 29552, 7456,  62968, 35700, 31580, 9608,  3432,
                61840, 11644, 46300, 60860, 9456,  52084, 6432,  35804, 56248,
                4612,  40060, 15948, 23792, 38524, 55512, 11812, 64464, 29432,
                40280, 6408,  32944, 63992, 37896, 25404, 15344, 5504,  28188,
                12580, 64400, 50940, 34764, 21432, 53268, 32296, 57020, 19876,
                29336, 35180, 23124, 59140, 48488, 55472, 24532, 5660,  61384,
                36808, 21820, 26304, 14628, 30424, 6000,  17296, 63328, 24776,
                41764, 51184, 65076, 31796, 27124, 44904, 33020, 13432, 64656,
                16100, 26896, 50920, 11000, 14832, 47740, 3388,  52184, 59892,
                15988, 22668, 62416, 9160,  59240, 1928,  17360, 21500, 34588,
                4220,  13708,
            },
            {
                53324, 6852,  10540, 21676, 61068, 5208,  26460, 34364, 20312,
                1476,  61584, 14788, 58592, 33952, 16236, 7412,  49668, 3012,
                41068, 35988, 52576, 21544, 31092, 43028, 64604, 23940, 46068,
                62036, 35320, 9708,  40500, 47604, 14168, 24584, 1304,  34916,
                20744, 61144, 17312, 49928, 23572, 12636, 39792, 48872, 27860,
                51044, 22516, 18584, 54628, 28584, 39648, 58820, 26656, 23040,
                63776, 11400, 28704, 52116, 3240,  17980, 35896, 8172,  45024,
                19908, 48948, 52436, 1296,  18928, 46524, 51424, 61712, 35888,
                44636, 6416,  16656, 10500, 27680, 46936, 4212,  40080, 8660,
                50480, 14504, 5848,  38008, 9372,  33408, 42096, 16196, 47216,
                2800,  43156, 62472, 38748, 59512, 42392, 55640, 11384, 32744,
                58444, 3472,  17948, 12384, 7760,  62104, 21744, 47360, 7172,
                55588, 19696, 37484, 61824, 42504, 33136, 19576, 9848,  43280,
                31260, 47192, 4680,  50852, 27100, 53364, 37604, 64344, 49820,
                31000, 60524,
            },
            {
                42460, 17664, 35244, 29500, 2012,  18460, 13952, 47440, 41848,
                55648, 23288, 8704,  40000, 46708, 56700, 30528, 61484, 24020,
                12320, 63868, 25852, 48776, 35076, 9536,  51100, 29088, 7308,
                17848, 53688, 29940, 61032, 6848,  19924, 31292, 49312, 9112,
                41836, 55508, 37328, 2236,  44340, 57740, 4600,  64268, 43016,
                14704, 57952, 36876, 24940, 1144,  16068, 12308, 48860, 2592,
                53600, 32940, 46792, 43576, 63136, 31540, 50232, 23368, 60652,
                13588, 55808, 23012, 27792, 59172, 30904, 9908,  2128,  56656,
                19508, 25996, 59812, 42336, 63204, 17724, 35792, 61684, 25476,
                42812, 65276, 54720, 46316, 17860, 63584, 11740, 38960, 56996,
                31316, 12888, 55292, 9476,  440,   20452, 45852, 36232, 7036,
                47344, 29204, 39664, 43572, 53808, 36028, 2244,  51644, 29872,
                34276, 45236, 2944,  24948, 53772, 5228,  39284, 58200, 24476,
                540,   56004, 36696, 20448, 13764, 44464, 5712,  12300, 40772,
                8372,  27728,
            },
            {
                1316,  63356, 48020, 56160, 41080, 63968, 37980, 58396, 6924,
                31820, 36888, 52520, 28428, 12080, 1156,  53368, 18192, 43076,
                47548, 29800, 796,   17708, 60548, 12748, 20076, 55724, 33488,
                41508, 49224, 3392,  23036, 36912, 58252, 44692, 63688, 27932,
                5820,  32164, 14320, 26144, 53264, 30128, 21368, 17816, 35140,
                2348,  31044, 6212,  64004, 42848, 32592, 61364, 45272, 37812,
                14976, 20684, 6880,  25448, 9800,  14500, 58408, 80,    38204,
                31112, 10240, 35632, 3940,  40524, 13244, 43568, 23736, 38948,
                48952, 31812, 256,   37636, 5532,  54456, 23320, 12868, 48276,
                2796,  31868, 21228, 26316, 76,    30092, 59764, 20804, 8568,
                24840, 50468, 18300, 34020, 28248, 49664, 25696, 13972, 60672,
                21588, 9620,  49860, 24076, 59432, 19056, 41948, 25564, 58624,
                12084, 49552, 8620,  31180, 17100, 63524, 28440, 49992, 35308,
                12008, 65456, 17592, 41232, 32504, 62048, 29300, 57528, 18876,
                24916, 51228,
            },
            {
                22068, 32060, 11952, 24700, 45148, 8840,  28136, 50964, 12000,
                17216, 62912, 3768,  19692, 64808, 22836, 38296, 33264, 9168,
                54876, 6512,  57168, 40036, 44992, 3800,  37296, 62644, 1180,
                25196, 11964, 56916, 15376, 51604, 2080,  11356, 16508, 56564,
                22800, 47016, 62556, 9888,  38876, 6644,  59956, 46048, 9200,
                52724, 40540, 46960, 11232, 50172, 54032, 19408, 29368, 9308,
                41784, 57164, 60480, 36448, 54464, 40728, 27272, 47648, 5716,
                42432, 62948, 16544, 48140, 54376, 65444, 34540, 17316, 52988,
                7808,  57856, 45740, 14516, 50164, 30352, 7580,  57684, 38312,
                16980, 10492, 58684, 35772, 51488, 44184, 5244,  53212, 35540,
                64124, 45428, 4028,  40792, 54312, 64884, 4788,  31188, 51652,
                38416, 16332, 57316, 5548,  30756, 14448, 9268,  38872, 4900,
                22300, 60784, 40768, 56448, 12664, 46052, 21940, 7128,  15424,
                45016, 26036, 52900, 7680,  47748, 23440, 816,   36124, 54468,
                46416, 14508,
            },
            {
                39060, 58324, 5480,  16124, 51988, 33240, 22640, 268,   43984,
                25228, 48908, 41416, 34844, 43856, 50584, 5508,  13852, 62704,
                20920, 36968, 14696, 32020, 23304, 52748, 27264, 47272, 15884,
                59420, 38332, 45324, 28416, 32844, 42292, 25556, 39372, 51280,
                36352, 920,   58560, 18880, 49236, 15208, 33188, 26788, 55644,
                23252, 60516, 16840, 21188, 34752, 7724,  3420,  24084, 64988,
                34616, 756,   49384, 18268, 4168,  21400, 33596, 53448, 18892,
                24924, 57792, 45124, 8720,  20136, 26372, 6544,  60944, 29448,
                11444, 21696, 34356, 24836, 59292, 19788, 44408, 33284, 52260,
                28068, 45332, 6528,  39680, 12816, 23196, 48308, 27028, 1500,
                14908, 29596, 60904, 23312, 11896, 16556, 44080, 56428, 2428,
                63640, 27456, 34768, 292,   61416, 48080, 52480, 63380, 16564,
                43988, 27428, 18864, 35164, 51408, 2636,  37868, 55100, 61424,
                40100, 29996, 2376,  56708, 10800, 59768, 16000, 43020, 6516,
                33128, 60992,
            },
            {
                28604, 43880, 54156, 36756, 3084,  59744, 19344, 54664, 60760,
                30024, 5332,  56236, 10288, 15916, 26924, 58248, 40652, 24896,
                45512, 27820, 50832, 65488, 8828,  58736, 6676,  42744, 30964,
                9220,  21108, 64920, 7660,  18496, 54008, 61852, 5272,  19644,
                13004, 44036, 28752, 34368, 41588, 63504, 51252, 88,    37680,
                13204, 28816, 4324,  56672, 26232, 38640, 58296, 47764, 17340,
                52536, 26700, 30676, 12936, 61832, 43032, 7812,  64960, 14844,
                50780, 28876, 4640,  38780, 31640, 50336, 3140,  46284, 41404,
                15536, 63896, 54052, 3784,  39080, 11284, 64216, 912,   22332,
                60848, 53800, 19184, 62164, 56136, 15876, 33624, 41708, 58532,
                38456, 20328, 6396,  47572, 35900, 8444,  39444, 19520, 23584,
                42320, 11044, 54704, 44472, 37392, 21272, 28284, 32872, 55008,
                36768, 712,   64904, 6756,  24256, 60176, 31932, 10520, 19188,
                4352,  50376, 21288, 38460, 34904, 26716, 49556, 21576, 64984,
                11056, 3368,
            },
            {
                49048, 8652,  19836, 64372, 30280, 13436, 47048, 34288, 39104,
                14712, 21160, 32728, 59508, 2288,  46636, 30324, 52368, 2932,
                59532, 11396, 1912,  42124, 17060, 34672, 13176, 22228, 50688,
                54172, 34348, 4572,  48700, 59860, 9880,  35016, 47320, 31728,
                65116, 8776,  54576, 22172, 4852,  24520, 10792, 43656, 19448,
                65484, 49540, 32176, 44008, 63280, 14188, 42220, 31552, 5868,
                11792, 45116, 22784, 38108, 47208, 56224, 29956, 11164, 37320,
                2180,  34480, 22020, 62188, 14244, 56252, 22888, 58656, 37120,
                1460,  27420, 42960, 9104,  48256, 28500, 16152, 41216, 5724,
                13788, 34436, 42280, 2076,  29140, 8900,  63044, 18652, 11184,
                51700, 43132, 57188, 32116, 53064, 59736, 29384, 49168, 33784,
                5880,  47872, 18824, 24828, 12584, 6840,  45644, 3552,  10772,
                50456, 14672, 47980, 29744, 44376, 15864, 42304, 27340, 47352,
                33512, 57876, 12688, 64028, 45160, 2904,  55152, 30148, 37816,
                52172, 17176,
            },
            {
                35264, 23712, 40716, 26456, 49904, 42428, 9956,  3896,  65188,
                8296,  52916, 45556, 24516, 62464, 19392, 36624, 8492,  17676,
                34340, 54336, 20112, 48984, 25532, 55220, 39840, 63444, 3696,
                17648, 44148, 26644, 40244, 23444, 14528, 29212, 2744,  57756,
                25172, 38576, 3380,  48432, 13328, 56528, 29460, 58764, 5988,
                41160, 8576,  36316, 1256,  51736, 10244, 20512, 54708, 62612,
                39784, 58660, 8668,  51552, 1660,  16724, 24152, 45728, 60200,
                41644, 54956, 11700, 48812, 42084, 35736, 10420, 18732, 32464,
                52024, 47412, 17492, 60288, 32080, 55812, 51364, 36928, 26880,
                49492, 9432,  31156, 24632, 50056, 38112, 3080,  54308, 31668,
                7784,  25032, 13688, 3520,  18092, 25444, 812,   12848, 58580,
                15484, 62176, 31676, 40516, 65336, 56848, 17392, 60040, 23920,
                31244, 58424, 39596, 9816,  20572, 57096, 1628,  52940, 63332,
                8324,  24544, 41664, 15684, 31572, 19328, 7924,  13392, 41760,
                25884, 57348,
            },
            {
                884,   62100, 12332, 6448,  57700, 17672, 37240, 25892, 49268,
                28592, 40708, 11932, 37720, 6856,  55016, 12864, 48728, 64148,
                43660, 29108, 38552, 4828,  31512, 46088, 252,   28884, 37024,
                61488, 13968, 56152, 892,   37128, 43876, 50632, 21816, 41312,
                15368, 51688, 18284, 61160, 45372, 36536, 17156, 32572, 48028,
                25848, 54224, 16088, 23552, 46472, 30016, 2552,  25644, 35192,
                15868, 3168,  32580, 64288, 27712, 35044, 5196,  52568, 19848,
                7576,  26268, 59088, 17768, 444,   28360, 52880, 7488,  62964,
                12836, 23392, 5284,  35820, 21008, 2676,  24240, 62600, 46996,
                57588, 16728, 64852, 45736, 20244, 60492, 43704, 22744, 47120,
                65416, 36424, 49596, 61316, 41720, 45928, 64320, 51824, 37620,
                27928, 52908, 8572,  1668,  50208, 26544, 33876, 39064, 43084,
                20180, 4652,  26100, 54160, 62724, 35544, 12964, 22312, 36592,
                17040, 44488, 28924, 4944,  51328, 61860, 47104, 58072, 3864,
                20264, 44704,
            },
            {
                15104, 53592, 46120, 33000, 1656,  62792, 23316, 55624, 16016,
                58504, 992,   17780, 50112, 27936, 42112, 21724, 31624, 3924,
                23988, 14772, 62620, 56588, 11160, 60164, 19352, 49580, 24692,
                10608, 32400, 52156, 19912, 58260, 63228, 7136,  55140, 10956,
                35276, 58944, 30408, 26228, 9248,  40324, 53288, 3908,  62444,
                21020, 11972, 57980, 61956, 18660, 37512, 60540, 50332, 43396,
                22240, 53980, 19160, 41584, 13724, 57728, 39492, 63496, 31816,
                15276, 44960, 30448, 6420,  64532, 45900, 24372, 43548, 38592,
                29816, 56108, 40352, 64936, 14224, 42300, 8016,  12168, 19936,
                1664,  36236, 54588, 3980,  12568, 35228, 26816, 5076,  15012,
                29380, 1292,  21408, 28240, 10784, 34620, 6988,  20604, 43308,
                3172,  45368, 22532, 36416, 14792, 53768, 9060,  2252,  63996,
                51356, 12716, 45712, 32956, 3652,  49660, 40776, 6172,  58728,
                50932, 1300,  59924, 54424, 10136, 37368, 22948, 33476, 63520,
                9428,  31388,
            },
            {
                59248, 28448, 18888, 39428, 51220, 29880, 43720, 7032,  46164,
                35208, 22396, 54436, 64044, 33980, 1768,  61312, 57204, 39876,
                10148, 47076, 7568,  22432, 36116, 15780, 6308,  43052, 56904,
                7256,  47132, 28620, 5856,  12204, 31596, 17576, 27544, 48100,
                424,   42872, 6388,  33748, 64540, 1344,  23128, 14404, 44588,
                35100, 39712, 28520, 5768,  41528, 55864, 8784,  13316, 4232,
                29328, 46720, 60144, 24904, 49528, 9292,  22000, 47944, 1268,
                36572, 50600, 22620, 39288, 32968, 57040, 15896, 60328, 2040,
                19952, 50176, 10252, 25988, 46424, 52800, 59480, 33612, 44120,
                29964, 23216, 39968, 7592,  58784, 52296, 17024, 56644, 40072,
                59040, 44212, 54208, 38508, 16120, 56468, 24052, 32260, 9944,
                17476, 57872, 60856, 29920, 42520, 19468, 48244, 29412, 55256,
                16216, 37336, 60840, 8396,  18128, 24872, 30428, 46336, 26800,
                11508, 39136, 21140, 34728, 26044, 43288, 16568, 28220, 53280,
                38496, 50444,
            },
            {
                4528,  36288, 8428,  56652, 14520, 20944, 10892, 32104, 61784,
                4216,  42684, 30880, 9692,  15152, 44660, 25724, 16384, 50736,
                35116, 59388, 52744, 30276, 44436, 51820, 64856, 33232, 38824,
                22032, 60224, 41352, 35964, 24876, 46452, 39720, 60120, 20508,
                63828, 23680, 14020, 50448, 20252, 47776, 59288, 27704, 7772,
                51416, 2192,  48548, 31744, 14676, 24824, 33872, 47936, 57496,
                38224, 10416, 6084,  36272, 2356,  44116, 17452, 28616, 12376,
                56300, 61480, 9640,  53752, 20400, 4812,  11748, 26940, 47676,
                7000,  33308, 58600, 4008,  30748, 18264, 27556, 39024, 4948,
                61200, 51672, 13936, 28252, 41908, 32188, 9784,  48920, 34416,
                12188, 18992, 8616,  63304, 4288,  50352, 40720, 61816, 54060,
                26096, 34896, 12116, 47168, 3928,  61872, 23480, 11344, 34748,
                6336,  22780, 28656, 42604, 52592, 14484, 65224, 54996, 19076,
                32864, 63712, 48876, 7428,  13932, 60744, 148,   6504,  45980,
                12076, 21832,
            },
            {
                65384, 43492, 24096, 60776, 46904, 3560,  54228, 39956, 13596,
                19248, 60208, 24920, 47952, 4536,  52064, 36920, 6948,  28664,
                19624, 1380,  26140, 40196, 3624,  20852, 27804, 13292, 1976,
                17844, 51056, 14680, 64592, 1632,  53676, 4704,  33332, 9144,
                45212, 52920, 39368, 57308, 10904, 43104, 31144, 38208, 55964,
                19064, 64312, 10208, 53652, 43920, 1008,  65032, 18392, 26596,
                61772, 15340, 51204, 30500, 62976, 54580, 33212, 59716, 40972,
                24820, 4292,  42660, 13964, 48060, 37468, 51524, 34380, 41460,
                62152, 16816, 44420, 37432, 55028, 1120,  49172, 15512, 56120,
                9560,  46372, 18636, 63512, 47460, 21168, 628,   62636, 24116,
                5616,  51292, 32748, 23268, 46864, 29664, 1588,  14312, 47972,
                39336, 5376,  21208, 55952, 7704,  38012, 57264, 41476, 46252,
                62608, 49376, 2136,  59160, 38956, 34148, 392,   9260,  43352,
                4644,  15772, 29640, 45804, 56452, 40312, 49792, 55440, 18664,
                41364, 26852,
            },
            {
                49232, 1864,  11788, 38216, 27348, 34200, 64264, 49456, 26196,
                52504, 7436,  38700, 56828, 21048, 60484, 11172, 55156, 46376,
                64680, 42860, 16552, 12176, 57888, 48180, 9072,  54056, 62480,
                45392, 26160, 9392,  56412, 42764, 16348, 49912, 13384, 37188,
                29480, 16800, 4184,  27056, 36160, 17544, 5428,  61568, 11552,
                33856, 23060, 26772, 59920, 36652, 21460, 52348, 5148,  40808,
                33028, 21768, 45052, 18564, 11448, 25748, 3712,  51740, 8204,
                19416, 33996, 28180, 63572, 1584,  29960, 58284, 21784, 9740,
                53276, 25168, 13312, 21512, 62752, 11048, 35424, 63936, 21840,
                26120, 33204, 37264, 4372,  25056, 55688, 38632, 30060, 45412,
                60172, 27392, 41520, 57624, 12832, 35512, 19156, 57284, 8140,
                30780, 64820, 51588, 17072, 33432, 27916, 14072, 1136,  26380,
                18000, 32444, 54756, 20420, 10884, 48404, 23376, 60596, 36476,
                52048, 57404, 23656, 2160,  19276, 31304, 24388, 35504, 62144,
                7700,  33848,
            },
            {
                54096, 17712, 30812, 51984, 6916,  16144, 23044, 456,   36828,
                29436, 46624, 12404, 34820, 16896, 31992, 40672, 22864, 13740,
                4864,  33560, 54408, 61812, 32124, 38100, 24288, 35020, 40844,
                29668, 3796,  34196, 19560, 30464, 22392, 57568, 25484, 61740,
                6964,  55284, 32792, 63056, 2408,  53780, 25040, 49548, 15808,
                46428, 42400, 4012,  17172, 47556, 11892, 30536, 45812, 9224,
                54048, 40,    56768, 39300, 58968, 42488, 15532, 37028, 63212,
                45612, 49708, 16464, 55860, 25204, 45204, 17888, 65104, 5040,
                39180, 29428, 50872, 6460,  41212, 24736, 45044, 7188,  53436,
                42272, 1372,  60272, 11488, 50720, 14636, 7300,  17924, 53312,
                2680,  36852, 16520, 6456,  44432, 62552, 52444, 27532, 22868,
                45228, 496,   40836, 24848, 43960, 63932, 50096, 21316, 58224,
                9340,  44272, 14960, 27168, 5296,  56284, 45064, 28376, 20836,
                12756, 41412, 62344, 37252, 10908, 64368, 4720,  13252, 29760,
                57656, 15408,
            },
            {
                45128, 58992, 21476, 62580, 41204, 56240, 44928, 59144, 10120,
                17624, 55264, 2208,  65048, 26912, 43780, 36,    62424, 30036,
                38672, 24644, 49192, 7464,  18412, 632,   59792, 15652, 5632,
                58076, 48864, 38436, 52464, 6868,  62796, 39092, 2260,  48296,
                41832, 21304, 47260, 13044, 58756, 44724, 21120, 39608, 288,
                29992, 58412, 35440, 8168,  55352, 39392, 61132, 24224, 13908,
                63848, 27996, 7872,  23532, 34428, 5452,  48472, 29976, 22968,
                528,   59032, 38968, 11104, 35328, 8152,  42464, 14652, 32004,
                56372, 44,    60960, 32812, 48320, 57556, 19152, 32056, 13324,
                50020, 17440, 56780, 30472, 44328, 35124, 64800, 43000, 22212,
                13512, 47616, 64164, 21052, 26032, 3092,  10648, 38024, 59592,
                15960, 36156, 13048, 55048, 9972,  3364,  31172, 53304, 35592,
                40476, 4260,  64252, 37752, 31216, 62128, 16352, 39724, 2988,
                31888, 8104,  48132, 26964, 53952, 44456, 39312, 21708, 47632,
                2496,  25012,
            },
            {
                38792, 6008,  35036, 47420, 9792,  19548, 5444,  31604, 42492,
                62052, 22548, 39836, 50716, 6752,  57536, 47704, 9156,  50408,
                58640, 20708, 10612, 45452, 28700, 41876, 51392, 46552, 20152,
                10692, 23340, 60708, 12796, 46108, 26972, 10580, 34836, 18764,
                59524, 9876,  24444, 40624, 29788, 7468,  33400, 64072, 54928,
                6344,  51964, 20044, 63684, 27832, 1844,  16560, 34900, 50076,
                42196, 37108, 47848, 13532, 52636, 65144, 19132, 9928,  54328,
                13280, 31792, 5484,  22176, 52888, 60604, 3468,  49328, 23628,
                45956, 11556, 19572, 36884, 14836, 2432,  28364, 40228, 3812,
                62208, 27336, 39128, 5908,  20188, 58752, 26948, 10444, 39880,
                56508, 31524, 9360,  54164, 39472, 49028, 32248, 42752, 4668,
                50512, 62168, 28832, 47560, 59836, 19044, 39272, 7064,  61356,
                12772, 47668, 51844, 24092, 42804, 12052, 7124,  50860, 59692,
                54844, 18040, 34204, 3512,  15612, 50488, 8160,  59848, 52440,
                42068, 10676,
            },
            {
                64284, 26688, 960,   13648, 28980, 37116, 25792, 48684, 14232,
                34056, 4428,  45660, 30732, 13944, 24420, 19432, 34996, 15912,
                53472, 2108,  36588, 63976, 55384, 22552, 12372, 63500, 32888,
                54888, 43004, 17192, 2628,  32096, 54992, 43688, 14888, 51784,
                28096, 680,   36820, 50380, 15436, 52424, 10176, 27136, 13524,
                37220, 24900, 10952, 49044, 31880, 43124, 6428,  59056, 20396,
                3636,  17808, 60404, 31484, 1648,  27240, 44136, 57412, 38228,
                25932, 42268, 62536, 46736, 18508, 39896, 28528, 54856, 36012,
                62400, 26756, 43540, 59376, 9324,  54288, 65380, 46884, 22976,
                35184, 9356,  47716, 23668, 54476, 2152,  32128, 48696, 4328,
                60560, 28000, 376,   34376, 59048, 14004, 22028, 55332, 18492,
                9088,  32924, 20980, 6304,  35092, 24056, 46100, 16248, 25540,
                29496, 20904, 33932, 2028,  53824, 19268, 34884, 22820, 46356,
                25896, 43448, 60808, 22496, 58064, 28864, 33220, 16748, 36244,
                19920, 31320,
            },
            {
                14820, 48460, 42664, 54692, 59900, 51008, 64848, 1888,  53432,
                58420, 27624, 18504, 8848,  52736, 38236, 59516, 28076, 5488,
                40596, 44344, 26172, 14556, 3836,  31500, 6240,  26472, 37308,
                1424,  28308, 64796, 35528, 50300, 20796, 4244,  63664, 31756,
                56196, 44532, 65360, 5828,  19732, 61220, 42484, 46956, 17972,
                60236, 44848, 40416, 14332, 21844, 57392, 46156, 26996, 52832,
                30328, 10788, 55628, 45312, 21608, 40108, 6760,  33056, 51012,
                17044, 8392,  55416, 27128, 1840,  32716, 12220, 20528, 8600,
                16604, 3744,  52884, 22528, 30496, 17652, 37460, 7788,  59640,
                55208, 15704, 42576, 63564, 13116, 38036, 16924, 52668, 24772,
                36312, 19212, 50904, 43700, 17272, 7012,  29792, 64048, 46056,
                25804, 57720, 1992,  53024, 42496, 12484, 63336, 49520, 156,
                56216, 60108, 16752, 9664,  58156, 48040, 64648, 968,   30532,
                10272, 14236, 5808,  36712, 41856, 12016, 1016,  26056, 61564,
                4360,  55792,
            },
            {
                23284, 58280, 18336, 32604, 22424, 8056,  16480, 39056, 20972,
                11360, 41824, 63580, 56184, 33208, 2972,  43068, 11036, 64476,
                31128, 18240, 60212, 34252, 47688, 39244, 57228, 43792, 8824,
                52004, 14288, 40460, 24864, 9516,  60388, 41032, 23540, 6684,
                12088, 17504, 33080, 58040, 23784, 35316, 1456,  57120, 22652,
                32508, 2580,  62460, 5500,  53884, 36824, 8792,  12840, 39192,
                62204, 24816, 5356,  35448, 16576, 58524, 12400, 61276, 2812,
                21124, 44640, 35096, 14336, 52028, 63756, 47824, 58104, 42116,
                50604, 33296, 39932, 6276,  45276, 49876, 25740, 12132, 43820,
                20064, 29652, 260,   33420, 50508, 8404,  44860, 61724, 6372,
                14824, 46496, 11120, 23464, 61432, 53156, 35400, 1264,  11820,
                36944, 40648, 48976, 16732, 27260, 57448, 8520,  32076, 36644,
                42024, 6216,  45692, 39204, 29756, 26584, 37156, 15572, 55784,
                38844, 63268, 48504, 52952, 19360, 65080, 46660, 54804, 43752,
                8980,  50100,
            },
            {
                33984, 6932,  11068, 40212, 3792,  44332, 30284, 46840, 25148,
                6184,  36548, 47940, 22900, 16620, 61472, 49368, 20884, 46292,
                54592, 12444, 7360,  51136, 21128, 61440, 15476, 18552, 48456,
                21776, 59008, 45264, 6452,  56320, 16220, 29624, 46828, 38276,
                49760, 26452, 40272, 3760,  47736, 28988, 38704, 6888,  50532,
                9216,  28028, 51396, 33964, 25784, 18200, 65464, 48216, 2380,
                34124, 42692, 50700, 64096, 8456,  49388, 23468, 28548, 46932,
                64600, 30308, 59332, 4440,  23072, 37040, 6688,  25508, 1356,
                64544, 27924, 55896, 13508, 63144, 34184, 1348,  52056, 32356,
                6036,  58332, 53224, 26256, 19264, 59280, 29476, 21380, 42064,
                55012, 65240, 29280, 40632, 3948,  26592, 47920, 41900, 60408,
                22604, 14368, 65472, 30952, 38272, 2976,  44528, 54988, 19232,
                11296, 50996, 22400, 61736, 13476, 7908,  42456, 4716,  51316,
                20860, 32364, 2364,  24952, 30116, 6540,  38556, 22164, 13052,
                28636, 37928,
            },
            {
                44736, 60968, 27640, 52844, 63224, 12412, 35520, 61792, 56492,
                51572, 31456, 1164,  12624, 39520, 6440,  25872, 35956, 784,
                23504, 38032, 57508, 29288, 10012, 952,   53480, 29900, 62336,
                34464, 3612,  31076, 19320, 48112, 36868, 1964,  53924, 19552,
                61120, 9116,  52588, 14348, 54624, 11064, 16976, 64628, 41548,
                56048, 19764, 15688, 58856, 640,   41188, 31064, 23212, 16280,
                54552, 20880, 14524, 29000, 37652, 54268, 42372, 34508, 15204,
                6280,  39100, 11464, 49184, 41088, 17148, 44812, 31084, 15088,
                37696, 10264, 19244, 24556, 4588,  38820, 60592, 21656, 15028,
                48036, 37664, 11076, 41284, 46456, 2764,  34852, 12040, 38432,
                1512,  32848, 7740,  56880, 37544, 12952, 20036, 8432,  51660,
                28612, 3816,  54596, 10452, 20636, 61524, 23256, 14592, 28652,
                64912, 25964, 33152, 4368,  53044, 57024, 24348, 61380, 45208,
                27900, 8336,  57844, 44428, 10860, 56516, 34324, 51420, 18132,
                62684, 1828,
            },
            {
                15940, 20564, 47032, 37212, 24252, 55216, 19024, 2464,  9936,
                15968, 60608, 27172, 44816, 54340, 30272, 57208, 14088, 51532,
                62844, 5172,  42632, 25152, 45972, 36056, 41632, 25616, 7008,
                11668, 55360, 39132, 63120, 26792, 10760, 58168, 13576, 34192,
                3180,  43556, 21692, 31436, 62860, 44248, 25572, 30892, 13352,
                34880, 46648, 38444, 43848, 10332, 49736, 60888, 4624,  44804,
                59780, 9572,  448,   46092, 25636, 4980,  18656, 1824,  56340,
                52584, 24968, 19128, 57056, 28852, 9376,  60644, 55164, 22124,
                46124, 59544, 41420, 48332, 58204, 16712, 28320, 54676, 40404,
                61856, 23232, 4924,  64488, 14940, 23952, 56976, 63192, 26712,
                49272, 20348, 52124, 16140, 45792, 63488, 32032, 58692, 17436,
                34100, 44880, 7584,  47304, 34936, 50596, 5568,  40192, 53416,
                2216,  38240, 43120, 18136, 47316, 31240, 20032, 10980, 16736,
                59548, 41484, 36036, 17836, 49052, 15308, 59884, 3668,  31600,
                41424, 53616,
            },
            {
                56676, 30788, 6500,  548,   15100, 49200, 28104, 40568, 33672,
                43180, 19716, 49920, 8328,  65528, 18080, 42160, 9436,  28760,
                33456, 15580, 49024, 18812, 64300, 13276, 4380,  58520, 37784,
                46796, 23720, 13704, 216,   51052, 32460, 43320, 25440, 63876,
                28952, 56720, 36724, 7396,  18712, 1984,  59308, 48236, 5628,
                24196, 3284,  63620, 29468, 22456, 13928, 55248, 36336, 26420,
                29904, 40544, 57216, 32920, 60228, 11568, 63036, 48420, 36692,
                9728,  43756, 32984, 62380, 316,   53020, 33676, 12480, 49424,
                3884,  29332, 8024,  52976, 31404, 11188, 43260, 8544,  2672,
                27244, 34648, 18152, 49352, 31068, 53612, 7540,  43804, 17616,
                10200, 58596, 24956, 34984, 1940,  22848, 49652, 5364,  39560,
                56668, 25088, 62468, 41704, 17864, 26708, 59808, 33740, 48212,
                8836,  58452, 14220, 63160, 652,   36064, 40376, 49428, 34172,
                3972,  13364, 23260, 63904, 204,   40644, 25864, 8116,  47948,
                23644, 10804,
            },
            {
                26192, 65340, 51596, 42072, 32936, 60500, 7900,  46412, 23512,
                58136, 5152,  37944, 24564, 34264, 2112,  48184, 59916, 19588,
                40508, 58900, 2312,  31432, 55664, 22964, 32952, 49984, 16996,
                65532, 28508, 53028, 41288, 16764, 61320, 22216, 7164,  39788,
                16176, 46592, 24472, 58512, 40804, 50384, 35928, 21164, 54380,
                61516, 11864, 51912, 7228,  57160, 33100, 19348, 7996,  52448,
                12416, 49120, 17600, 22572, 51720, 40152, 30736, 20828, 27324,
                60900, 16052, 3700,  46668, 26216, 20756, 43152, 5832,  36304,
                61932, 15952, 34704, 480,   20588, 46824, 35788, 64932, 50948,
                12776, 44900, 59520, 9128,  36784, 40288, 3984,  51456, 31904,
                47388, 5112,  41456, 62244, 9484,  54232, 27872, 43544, 14916,
                296,   21284, 29716, 13336, 57232, 1704,  10976, 44068, 17112,
                21540, 30088, 45532, 23848, 10104, 52012, 6480,  65508, 25212,
                55004, 46924, 52680, 28800, 33068, 53820, 20208, 43368, 63720,
                36864, 4932,
            },
            {
                39868, 34968, 18512, 11816, 57052, 20936, 3296,  64612, 13860,
                52344, 29272, 62392, 14632, 52968, 22084, 36800, 3872,  45244,
                26428, 53484, 11192, 37304, 7672,  51848, 44392, 20308, 9468,
                43296, 4832,  20704, 33788, 8780,  45504, 4136,  53360, 49248,
                10556, 1516,  51692, 13016, 32904, 27080, 9568,  15336, 43088,
                37344, 32120, 17468, 26288, 47700, 3728,  43356, 38760, 62368,
                5572,  34888, 63844, 3832,  8044,  14980, 45224, 57712, 6896,
                40900, 23548, 51608, 35936, 14208, 39364, 65176, 18224, 26624,
                56444, 23336, 44460, 62900, 26388, 57480, 3724,  24220, 19556,
                56816, 30000, 876,   54640, 19900, 27896, 13608, 22388, 36284,
                61008, 28732, 13128, 44512, 30608, 18452, 37748, 11416, 64536,
                52324, 48596, 36996, 5892,  53076, 38712, 31032, 24424, 62448,
                55972, 3412,  37176, 54936, 27212, 60584, 15072, 29324, 1484,
                38716, 19804, 6140,  9776,  38076, 61172, 11676, 28156, 57572,
                14280, 49900,
            },
            {
                45464, 8244,  59128, 24580, 44536, 38908, 30700, 53932, 36708,
                10496, 348,   47172, 41636, 11100, 31752, 61616, 55592, 12808,
                7252,  21632, 47024, 62228, 16080, 27676, 61096, 1364,  30940,
                54900, 35952, 59772, 48756, 56520, 29344, 37792, 18612, 60156,
                34924, 20780, 65044, 5320,  45248, 61152, 4032,  57356, 28540,
                1160,  45472, 59228, 40636, 35596, 64708, 24764, 16608, 46480,
                21232, 25904, 43852, 37528, 55368, 25100, 34212, 820,   12992,
                54584, 31148, 63824, 5496,  56580, 8156,  30404, 51060, 1720,
                40640, 53564, 12196, 38612, 50588, 14260, 41756, 32072, 47836,
                38356, 15692, 41092, 24436, 62656, 46320, 60160, 55936, 15888,
                520,   38900, 21132, 50452, 56156, 4108,  60700, 24664, 34908,
                31344, 10016, 59588, 19220, 46028, 64736, 15140, 51868, 6168,
                34696, 12192, 49812, 18868, 32600, 42700, 21292, 44784, 58216,
                31824, 62312, 43584, 50324, 22144, 3344,  45932, 17188, 33796,
                1176,  21196,
            },
            {
                54556, 16232, 2228,  28552, 50660, 9564,  17004, 26040, 42824,
                22008, 32312, 17804, 59132, 4660,  43924, 16044, 24352, 38720,
                64744, 34804, 29124, 42480, 3556,  38864, 12032, 57372, 40380,
                15324, 25888, 11412, 2704,  24568, 64548, 12200, 31972, 26748,
                55480, 42936, 30144, 38352, 17500, 22728, 51352, 39140, 18768,
                53120, 8964,  21664, 14316, 1960,  50640, 11500, 58612, 31208,
                1116,  56356, 13620, 29416, 47220, 18172, 65304, 50248, 38408,
                44720, 18688, 11364, 48844, 22448, 45100, 59444, 10528, 47384,
                32304, 19632, 8892,  30636, 5200,  18140, 61360, 10480, 7328,
                63568, 5560,  51340, 11836, 32912, 2320,  8184,  42552, 26340,
                52996, 64624, 7708,  33700, 15496, 40268, 47968, 7408,  44772,
                17200, 3384,  41276, 27432, 33104, 21964, 8288,  42224, 28120,
                46784, 60076, 40560, 7800,  2484,  56900, 11496, 48816, 8028,
                17444, 12680, 26508, 57104, 14992, 30696, 55700, 6920,  51808,
                60424, 29688,
            },
            {
                33016, 62212, 48172, 35728, 63804, 4248,  55928, 47768, 5996,
                57752, 50564, 38856, 26184, 54272, 28504, 8508,  51236, 30928,
                49204, 128,   17812, 56980, 24788, 48368, 33992, 22340, 50692,
                5924,  62732, 44644, 19164, 40028, 15604, 47860, 872,   40924,
                8220,  14576, 25080, 53456, 11040, 47552, 30944, 64208, 12884,
                25500, 49252, 62804, 31004, 54712, 20364, 28904, 8644,  53864,
                41740, 60988, 10108, 51212, 2668,  59020, 9704,  28488, 21800,
                60016, 3360,  27684, 41808, 33772, 16364, 25300, 37044, 14356,
                57696, 4472,  64292, 46256, 22540, 55408, 34932, 52200, 28400,
                22048, 34440, 56084, 43324, 17232, 50348, 38544, 34052, 10964,
                18792, 46104, 24484, 59256, 1384,  27284, 20700, 57716, 53880,
                22920, 62272, 55596, 12616, 988,   49396, 56992, 36004, 4196,
                19568, 25580, 14444, 63388, 51108, 38328, 34260, 24440, 53912,
                36836, 41868, 4576,  34420, 64764, 48632, 40368, 35848, 23436,
                42652, 10284,
            },
            {
                25372, 40824, 6128,  22256, 13232, 31892, 20356, 61496, 34432,
                15260, 63984, 7988,  20284, 36224, 63276, 46348, 18680, 6020,
                41152, 14308, 60268, 9796,  52540, 19092, 63952, 7968,  45764,
                29660, 37648, 52092, 32704, 54216, 5756,  59052, 22056, 62028,
                45844, 3452,  57424, 62648, 35556, 92,    41988, 7260,  34408,
                58392, 4780,  36972, 44152, 6624,  60400, 39920, 35212, 48668,
                15980, 33644, 19892, 40512, 23568, 36260, 42976, 15664, 33172,
                8128,  51260, 37168, 62148, 1492,  52236, 6160,  63492, 21628,
                43376, 28092, 35948, 59604, 40896, 26776, 928,   44380, 58536,
                17852, 46804, 25960, 4276,  29404, 63928, 23152, 47956, 57916,
                4540,  31088, 41360, 11720, 52280, 62496, 36556, 13612, 4712,
                29168, 38920, 50420, 24812, 43252, 60244, 18176, 11560, 63916,
                53936, 31920, 44468, 22960, 29208, 5092,  15800, 64100, 2200,
                28640, 60428, 46772, 18372, 1444,  25340, 9208,  19096, 63400,
                3780,  13692,
            },
            {
                46760, 19324, 58460, 37624, 52196, 43956, 39688, 1692,  12012,
                27812, 42156, 2332,  48620, 13604, 3660,  34360, 56884, 60912,
                23708, 54864, 36440, 31448, 43904, 4420,  14056, 26984, 60508,
                17408, 584,   21780, 8500,  27216, 43592, 35904, 51308, 17304,
                34180, 50048, 19784, 6600,  27912, 16252, 54920, 20628, 46564,
                29032, 41524, 23344, 17196, 12112, 26612, 45548, 5276,  22380,
                3264,  27040, 64348, 7264,  31408, 53556, 4976,  61876, 46940,
                56380, 24336, 13876, 57780, 20012, 29772, 54852, 40172, 3352,
                50104, 16532, 52416, 12732, 7612,  48608, 15364, 37456, 11980,
                2852,  39828, 60580, 9724,  57324, 20184, 12604, 6284,  28396,
                62824, 49752, 35620, 17056, 44964, 30268, 9832,  42788, 33248,
                46948, 15160, 8708,  35084, 4952,  30484, 38152, 23740, 48248,
                39512, 84,    9804,  59452, 18052, 41480, 58468, 45268, 20784,
                51976, 10624, 22992, 54524, 38372, 12900, 59152, 44788, 27588,
                49624, 56932,
            },
            {
                264,   30572, 53836, 10424, 27104, 15820, 59084, 24180, 46616,
                53560, 30972, 56620, 24324, 59408, 40160, 22116, 11012, 27760,
                2688,  45704, 20280, 7044,  28152, 37904, 55524, 42000, 35408,
                11784, 57704, 46880, 63624, 13556, 55980, 29820, 9320,  25680,
                11908, 30568, 39016, 44132, 49152, 59176, 24276, 61896, 10592,
                15556, 1788,  53488, 57288, 33132, 50260, 64020, 13844, 58092,
                51520, 38376, 46556, 56028, 14720, 48824, 20160, 26872, 1328,
                40344, 17796, 31504, 6700,  47692, 43612, 12712, 26884, 33348,
                62008, 24164, 2204,  32520, 20484, 57604, 30360, 23692, 62960,
                53848, 31576, 14684, 36656, 52180, 32028, 44244, 54284, 37348,
                14760, 21332, 2556,  56632, 22492, 5748,  49964, 18920, 63456,
                2512,  58292, 21056, 64080, 45760, 14040, 52164, 3508,  29572,
                15920, 56504, 33868, 46912, 53048, 26176, 8760,  31528, 13668,
                39948, 6332,  32616, 62896, 43476, 29584, 53500, 5056,  15500,
                37860, 34312,
            },
            {
                64352, 17068, 44952, 3032,  63040, 7284,  48736, 36140, 9000,
                21040, 37568, 16476, 10248, 44844, 29768, 53208, 15816, 43364,
                32408, 50840, 11880, 64492, 47000, 58984, 23248, 3340,  48740,
                53452, 25020, 41104, 34336, 3436,  18648, 39348, 2036,  63860,
                52972, 59504, 22936, 13856, 9008,  33344, 3252,  38744, 51076,
                35716, 65480, 47564, 9056,  38916, 332,   18704, 24672, 42620,
                30556, 11200, 17608, 720,   41376, 63184, 12284, 34772, 54480,
                10560, 44492, 64772, 38936, 10084, 35312, 60952, 18184, 8052,
                46112, 10712, 42740, 54612, 39452, 65356, 4508,  50760, 42180,
                7936,  20764, 49020, 24260, 24,    40068, 16208, 60732, 3396,
                42704, 53504, 9472,  26968, 65192, 37756, 58908, 25208, 53116,
                27648, 36984, 51708, 17824, 26868, 55704, 61100, 8360,  44000,
                62364, 21448, 6672,  12944, 36964, 1488,  48628, 55060, 35584,
                59324, 50252, 26740, 15756, 7684,  20660, 50668, 31768, 60668,
                22016, 7916,
            },
            {
                50804, 12740, 25028, 34816, 41788, 29936, 18364, 55572, 64876,
                4888,  43500, 62120, 33160, 51124, 1080,  65216, 37448, 4448,
                62108, 39212, 25596, 53516, 904,   16388, 32900, 9700,  19440,
                31288, 5392,  15508, 60004, 49940, 23884, 58552, 46672, 42260,
                6400,  36388, 1396,  64844, 41308, 53236, 30076, 44556, 6108,
                18984, 31684, 26300, 21032, 61756, 29056, 36648, 55844, 8876,
                62116, 35388, 59728, 25828, 29680, 8624,  38128, 22916, 61184,
                29136, 49732, 2532,  26264, 55564, 23132, 48,    51564, 56912,
                37996, 29180, 60300, 17580, 26076, 45004, 9840,  18996, 33956,
                27004, 61444, 43524, 5472,  65008, 46588, 8648,  26864, 23688,
                33140, 61580, 39012, 48348, 32112, 1644,  15700, 40700, 7892,
                12212, 44188, 31328, 5976,  10380, 39916, 33328, 19068, 25900,
                36280, 51372, 42408, 65228, 30544, 60800, 19728, 24712, 3704,
                17480, 42600, 360,   47540, 56360, 36452, 3048,  40860, 11244,
                55320, 42516,
            },
            {
                20168, 38684, 59672, 55948, 21856, 50608, 740,   32628, 26528,
                13460, 52300, 3076,  18796, 26084, 8772,  20468, 47196, 56280,
                18308, 8196,  14560, 34080, 21716, 40464, 51592, 61116, 43244,
                65244, 38676, 28468, 10216, 44292, 31224, 12380, 20940, 15696,
                28064, 47628, 18944, 55800, 26444, 17656, 11612, 21824, 60712,
                55328, 13088, 42140, 5640,  44976, 15744, 53272, 3492,  47484,
                20496, 4548,  52928, 22096, 44004, 58160, 51748, 46240, 5900,
                14268, 35576, 21420, 58472, 15964, 48568, 29488, 41320, 21256,
                14488, 49440, 6380,  34860, 1132,  14128, 36664, 55768, 47792,
                2264,  12560, 57548, 17556, 28872, 34516, 19792, 58604, 50892,
                11440, 18344, 6728,  43412, 13716, 55124, 45968, 34700, 22200,
                56604, 324,   41488, 59236, 47756, 22324, 1312,  49612, 54816,
                10844, 3212,  28084, 23428, 15464, 39752, 11112, 44448, 64416,
                29016, 22128, 61868, 33824, 11860, 24328, 65004, 17936, 46500,
                25744, 29340,
            },
            {
                5256,  47864, 9196,  4436,  14924, 61264, 10968, 40620, 45540,
                58808, 29076, 39532, 49208, 59872, 41804, 34608, 13132, 24500,
                30380, 49164, 57812, 42728, 63004, 6412,  29456, 13632, 24100,
                1696,  55872, 21308, 52248, 6696,  61464, 37000, 54084, 33808,
                57900, 10972, 32196, 44872, 5796,  37112, 57692, 48416, 27320,
                39380, 2748,  58500, 50024, 11216, 60044, 41052, 33828, 13260,
                28460, 39576, 45720, 6732,  34088, 15624, 1904,  31956, 18100,
                59232, 53108, 42092, 8408,  33652, 44336, 5788,  64432, 35860,
                3516,  62640, 23076, 47492, 58336, 53152, 29116, 61992, 16148,
                41016, 30776, 37588, 53440, 11004, 56260, 48228, 2092,  36960,
                45396, 29040, 58176, 24340, 20096, 10324, 29424, 60768, 18196,
                48836, 63080, 24284, 16344, 35348, 64908, 28496, 41344, 14344,
                60072, 17956, 48084, 57296, 4224,  51116, 55996, 7132,  38336,
                12440, 53700, 8464,  39236, 59016, 28628, 48928, 7160,  33464,
                1524,  61988,
            },
            {
                35432, 52380, 31784, 27596, 46964, 37780, 54020, 22984, 15892,
                5940,  35352, 23220, 12156, 6660,  55580, 27656, 52036, 5612,
                60496, 36464, 1980,  27564, 11116, 47584, 36120, 57640, 49788,
                8060,  46540, 35932, 17572, 40760, 3000,  26352, 8068,  49776,
                4700,  62832, 24724, 50412, 14964, 62220, 608,   33588, 8092,
                16336, 52792, 24376, 34668, 30344, 22788, 7292,  25908, 50568,
                64564, 16284, 55200, 11976, 50152, 19832, 65068, 42512, 25380,
                38740, 3544,  28288, 19464, 61280, 13528, 53844, 10308, 17376,
                26660, 52708, 31356, 11532, 39072, 21076, 5084,  24232, 7340,
                59696, 21976, 50428, 25176, 6572,  41816, 13848, 30248, 63764,
                15748, 4416,  52624, 35356, 49704, 63608, 3480,  51816, 5316,
                33040, 13996, 29064, 54124, 4456,  12780, 57140, 6264,  46192,
                30704, 39044, 34708, 9296,  42948, 32316, 26856, 34556, 47672,
                58140, 31704, 46000, 17164, 2400,  42276, 13728, 37632, 53252,
                58376, 15168,
            },
            {
                41304, 22524, 65516, 44172, 19020, 34300, 7596,  63272, 48360,
                20020, 56712, 64276, 46076, 37052, 15568, 62920, 44316, 10524,
                40864, 22612, 45608, 19620, 55464, 16808, 2660,  26312, 18644,
                33320, 11828, 62908, 25408, 56648, 47280, 64712, 17984, 41332,
                22384, 38248, 2880,  40408, 29784, 52252, 23836, 41752, 63084,
                46584, 36612, 19608, 64444, 1188,  47928, 17336, 56948, 43300,
                2096,  23352, 30916, 61588, 37364, 4384,  28564, 54384, 7744,
                47588, 12736, 63696, 50976, 4692,  39612, 25096, 32620, 58676,
                45508, 42020, 7448,  18400, 63376, 43424, 49972, 32220, 45952,
                10188, 35056, 1388,  45252, 63296, 21344, 38884, 6300,  54492,
                22248, 40300, 62276, 580,   27288, 42004, 36816, 23716, 39660,
                44824, 8976,  38288, 50116, 20556, 44096, 37060, 24716, 62124,
                20732, 1760,  53148, 63468, 21552, 17284, 62352, 944,   15124,
                20220, 3988,  26020, 52168, 21268, 55132, 62516, 23416, 19516,
                44252, 10600,
            },
            {
                56940, 16912, 1284,  12344, 59336, 3688,  29684, 25716, 42880,
                2100,  9932,  31836, 17692, 544,   30860, 21684, 3448,  32788,
                16492, 53428, 4520,  61460, 31480, 52000, 39360, 45120, 60564,
                41596, 54136, 30316, 432,   14372, 32884, 11180, 29620, 44708,
                55340, 12896, 60436, 19968, 8796,  45784, 13348, 20332, 29164,
                10756, 4252,  43348, 13792, 54880, 38252, 62388, 31780, 10488,
                37048, 59312, 8344,  25712, 41228, 57240, 21892, 10988, 61028,
                32808, 23608, 44820, 36636, 30084, 57472, 21564, 49892, 37492,
                984,   13804, 56008, 34096, 27508, 3096,  13368, 37852, 64380,
                52516, 19088, 58328, 15416, 27616, 32828, 59600, 49360, 25952,
                10236, 32668, 46916, 17348, 12660, 55924, 7548,  15320, 54740,
                26824, 58360, 3288,  61216, 31568, 8548,  52604, 16648, 33500,
                9824,  58792, 25836, 5620,  46516, 11716, 49524, 23900, 52872,
                41928, 60936, 35420, 10112, 44540, 31400, 4596,  8884,  50340,
                29984, 26204,
            },
            {
                5972,  38164, 53908, 24768, 49828, 39700, 57432, 13012, 33228,
                50160, 54264, 41296, 24780, 52404, 48000, 38732, 56812, 49884,
                64388, 29248, 13228, 38004, 23916, 7076,  63940, 10008, 22304,
                6032,  15972, 44592, 50936, 23000, 39564, 59428, 51828, 1140,
                16060, 27632, 33084, 53604, 58196, 35380, 4876,  55168, 48804,
                59404, 32236, 57416, 26844, 9304,  45320, 21220, 4152,  52136,
                19312, 46632, 33712, 504,   48880, 14424, 45544, 35364, 50028,
                15860, 56080, 728,   10476, 17140, 46820, 2692,  8460,  62436,
                29532, 24168, 59980, 46980, 9716,  51944, 58960, 17184, 26124,
                42316, 29492, 4840,  48536, 8896,  43868, 17280, 1000,  35452,
                44548, 56804, 7116,  30564, 21956, 59752, 32192, 48052, 65052,
                21060, 11868, 46440, 17760, 41200, 27296, 63576, 3956,  50196,
                39960, 44716, 13592, 36564, 30432, 40140, 59700, 8580,  36932,
                29696, 6724,  49804, 63360, 14920, 27020, 36164, 59424, 39904,
                2500,  63172,
            },
            {
                48956, 34060, 30604, 43136, 9076,  21356, 52552, 17536, 61572,
                37520, 27372, 4856,  58184, 61980, 8264,  12368, 25812, 19036,
                35500, 7804,  59080, 48640, 43380, 34256, 14428, 29648, 56176,
                38196, 26932, 4460,  60316, 35552, 7288,  19844, 24928, 35004,
                63908, 48192, 6884,  42712, 17228, 25840, 65132, 37880, 2024,
                24932, 18076, 39976, 50432, 5412,  34320, 14900, 59908, 28860,
                40492, 13776, 53660, 18112, 63772, 30172, 5212,  39772, 2472,
                27528, 20300, 60028, 40724, 26188, 64804, 42344, 15284, 19452,
                54504, 39468, 4844,  20820, 41328, 35768, 23160, 472,   6820,
                56776, 11824, 39708, 61000, 36204, 55848, 12048, 51744, 65420,
                19484, 14100, 61088, 38808, 49320, 43936, 10092, 18940, 1576,
                30488, 36368, 53320, 24060, 808,   56692, 47148, 11936, 23084,
                55324, 28792, 18464, 51500, 56736, 3040,  27816, 44564, 18432,
                56436, 12980, 22908, 38840, 832,   57328, 47472, 17260, 32836,
                13504, 45668,
            },
            {
                18976, 7516,  60196, 14952, 64024, 35740, 4,     45916, 6356,
                21980, 15348, 11472, 44792, 19704, 28744, 42616, 60220, 1368,
                46656, 41572, 26780, 10404, 8,     18404, 58388, 49468, 2900,
                32720, 64924, 19040, 48240, 10392, 54908, 43164, 5368,  57600,
                10048, 37828, 23292, 1616,  49648, 11264, 31488, 44040, 15356,
                51968, 7676,  63796, 22136, 29700, 54012, 42360, 25428, 48268,
                6912,  22716, 57784, 43228, 9276,  24272, 52108, 18968, 62880,
                42644, 53708, 34776, 6604,  51476, 31700, 56168, 34976, 47816,
                11096, 50896, 32084, 15136, 65124, 54044, 30652, 44988, 48944,
                33528, 21392, 53092, 24548, 2780,  20460, 29856, 24064, 3968,
                41992, 27512, 51188, 2240,  25424, 5520,  35464, 61420, 40436,
                43472, 6424,  63016, 33112, 14468, 35816, 19216, 30844, 42688,
                1216,  61556, 7620,  22656, 65036, 14864, 33944, 53636, 1820,
                64224, 46692, 33076, 18032, 52980, 41732, 11664, 24624, 61336,
                54132, 22064,
            },
        },
        {
            {
                18364, 41508, 62756, 27968, 32016, 53288, 7352,  23768, 50948,
                2316,  47584, 38544, 14616, 7500,  40460, 23516, 50604, 38212,
                64056, 44200, 55316, 61020, 17584, 33960, 52204, 696,   26580,
                55540, 22784, 39612, 51672, 56284, 14764, 37500, 64184, 40308,
                16336, 31000, 23760, 45172, 32908, 56088, 22184, 60140, 9684,
                32412, 1504,  12676, 25572, 20612, 51432, 16776, 2692,  57672,
                34296, 7044,  59572, 39040, 46612, 61420, 19968, 39888, 52,
                29456, 25984, 6668,  47684, 33712, 53192, 17880, 888,   59688,
                52000, 25824, 3888,  16012, 21620, 28848, 3168,  43088, 52208,
                996,   19480, 50568, 7704,  55760, 21420, 52068, 17368, 12224,
                760,   27720, 35696, 51092, 45716, 12268, 31684, 37688, 50140,
                8668,  2944,  19996, 31312, 5704,  54996, 63684, 51328, 42176,
                5476,  27156, 38820, 1092,  7940,  37116, 24748, 2268,  21228,
                6456,  30868, 40476, 59744, 16768, 48844, 29332, 6232,  21272,
                39368, 30600,
            },
            {
                52920, 22548, 36992, 3928,  60556, 39732, 11368, 30252, 36788,
                5948,  21512, 28480, 55220, 58516, 18540, 11688, 33104, 3536,
                28444, 8172,  22824, 2788,  48264, 11412, 41348, 58820, 37628,
                45944, 64124, 12640, 6792,  24184, 32820, 49976, 20792, 26360,
                47288, 61072, 53560, 5408,  39020, 27808, 50512, 6308,  26204,
                47548, 38588, 50208, 57452, 33740, 10636, 45492, 23184, 40344,
                26640, 48804, 53852, 14120, 5536,  28452, 11520, 58188, 49000,
                9492,  62980, 57536, 11544, 21600, 63984, 9132,  35660, 23112,
                11264, 63020, 45500, 32264, 53740, 40504, 59456, 25896, 37540,
                56940, 29728, 35708, 60188, 13060, 37972, 26404, 6340,  39992,
                32040, 61468, 4720,  59236, 25004, 2280,  65412, 17864, 45084,
                24084, 63556, 47404, 36352, 41028, 15308, 1724,  24988, 9104,
                47256, 32360, 11856, 18068, 45052, 29108, 54564, 47820, 57032,
                36436, 63036, 11320, 4348,  37536, 22576, 2064,  51932, 25688,
                63576, 10380,
            },
            {
                60000, 7980,  49668, 13228, 56428, 21056, 46748, 64500, 54300,
                41860, 60884, 34028, 24848, 720,   43604, 47680, 60588, 56064,
                41600, 52084, 14224, 39352, 30648, 65384, 22504, 14480, 7468,
                18664, 32028, 28292, 58032, 43436, 61688, 7960,  44740, 3300,
                10164, 35000, 13348, 20068, 63256, 2192,  42868, 14060, 54484,
                20324, 65052, 30048, 6044,  41584, 61604, 28404, 55764, 4856,
                64600, 9764,  30308, 24436, 36972, 51760, 33584, 25140, 14984,
                38060, 51980, 16668, 30804, 38704, 26592, 43908, 50440, 29712,
                41764, 18940, 37608, 57664, 5064,  47708, 8208,  33088, 11336,
                47168, 5132,  42332, 23524, 32628, 1588,  44668, 64584, 49252,
                55584, 20872, 43308, 16880, 29368, 41552, 56564, 35212, 13912,
                29624, 53928, 12372, 26500, 60120, 49460, 28916, 37764, 57908,
                21184, 63292, 54072, 49336, 59424, 13952, 4576,  32500, 41816,
                16996, 49484, 27360, 53700, 32228, 44544, 57048, 40920, 12908,
                45728, 32852,
            },
            {
                15592, 42596, 25772, 34436, 17280, 43180, 520,   25912, 14464,
                18284, 9240,  49352, 5428,  51828, 36140, 26552, 9396,  21024,
                17064, 34712, 25668, 45776, 54496, 4984,  28384, 49500, 42788,
                53488, 2748,  47320, 36572, 3984,  18044, 30132, 59068, 55520,
                22616, 41532, 7528,  57996, 48460, 17080, 31216, 62220, 34796,
                3236,  16596, 44600, 23540, 14604, 1244,  48340, 31800, 51944,
                15728, 42840, 18508, 62604, 57384, 2712,  43672, 65488, 4304,
                22736, 34816, 42872, 1544,  55232, 4580,  15344, 61532, 6164,
                54276, 1876,  27404, 9956,  24136, 14568, 51416, 18796, 62572,
                22088, 15508, 63520, 9912,  47992, 58232, 19520, 29472, 10048,
                24080, 13796, 38256, 8004,  49748, 10432, 20808, 5568,  52352,
                1064,  38516, 57044, 4264,  9316,  33416, 18832, 45096, 13800,
                35292, 4168,  41376, 24180, 34168, 40544, 19408, 64268, 8344,
                23460, 1280,  58884, 20008, 14248, 65320, 9720,  18040, 35364,
                54644, 2540,
            },
            {
                28520, 47028, 64988, 5504,  52100, 29112, 8620,  35532, 56744,
                31680, 45228, 12884, 64148, 30560, 15144, 62232, 2660,  31712,
                46780, 57980, 784,   60240, 19032, 9268,  35608, 57132, 25336,
                10528, 61952, 15052, 21508, 53860, 11424, 38736, 14044, 27936,
                50824, 64792, 29544, 44048, 25796, 36472, 8112,  52096, 24028,
                40348, 10012, 55132, 35256, 58796, 19104, 37636, 12420, 21300,
                36348, 648,   33288, 46056, 7952,  21248, 17428, 40496, 46904,
                59564, 6220,  19448, 49412, 60296, 46124, 20116, 32464, 39300,
                13600, 59028, 49124, 43204, 64864, 36576, 29980, 44980, 1972,
                53360, 39012, 28004, 54632, 3876,  40632, 15000, 53468, 36208,
                2652,  46436, 63276, 54352, 33684, 60604, 46776, 27348, 59492,
                43372, 21664, 17512, 46024, 23332, 65068, 55300, 7064,  50500,
                60220, 16684, 28636, 2188,  9496,  61452, 26980, 12128, 52404,
                30104, 46608, 42496, 34668, 7052,  26536, 49684, 29772, 59344,
                5204,  20780,
            },
            {
                57876, 39228, 10608, 23120, 59128, 38564, 61516, 48940, 3520,
                23248, 59808, 37436, 19524, 42400, 22744, 39356, 54144, 13468,
                63452, 6852,  37948, 12028, 32368, 44276, 62592, 1432,  20328,
                38568, 33776, 50964, 26192, 64592, 42356, 47936, 34212, 1216,
                18776, 37592, 15312, 3604,  55380, 21428, 12196, 45288, 57160,
                28856, 61476, 7080,  47156, 26920, 52644, 62720, 7752,  44788,
                60892, 56396, 22980, 39540, 11140, 50328, 29848, 55896, 12584,
                31808, 53576, 28096, 13252, 24404, 36092, 10852, 56608, 47648,
                22252, 34912, 16908, 31616, 20508, 56176, 4320,  60380, 25416,
                34072, 7176,  49908, 17916, 25000, 60600, 34040, 7452,  42648,
                57656, 26828, 31016, 19444, 560,   24248, 15276, 39736, 32280,
                10136, 63008, 34640, 51528, 41920, 12764, 38908, 936,   22684,
                31204, 10780, 44456, 57072, 21520, 50408, 36116, 44168, 58316,
                38148, 15272, 10080, 62068, 52512, 38484, 36,    43244, 23876,
                37804, 51032,
            },
            {
                33860, 18324, 1632,  31816, 43992, 14988, 19772, 11516, 52496,
                40264, 26712, 1900,  53216, 8012,  57164, 4964,  49100, 43764,
                24048, 29580, 50428, 21728, 53388, 39808, 14816, 47128, 29896,
                60152, 6020,  40756, 536,   31456, 6892,  23996, 60268, 54252,
                5336,  46532, 32588, 10392, 49840, 60448, 38876, 1440,  18736,
                48604, 13804, 21096, 42436, 3892,  10272, 40072, 24908, 29816,
                6028,  47740, 13656, 52684, 27364, 60444, 36588, 908,   26084,
                8972,  45148, 62616, 40996, 7828,  29384, 64980, 316,   26020,
                8384,  63220, 3216,  52704, 7008,  12316, 39652, 16664, 42576,
                58476, 13404, 44064, 36896, 31068, 11872, 50852, 21568, 62528,
                17100, 5800,  51588, 12840, 43876, 36340, 63916, 7532,  50400,
                25412, 14656, 6224,  28712, 3492,  31748, 58692, 26544, 42240,
                52632, 37232, 64404, 47516, 6820,  55160, 16472, 2636,  6388,
                20928, 55868, 25624, 3868,  47408, 21808, 60972, 16064, 8868,
                63952, 12440,
            },
            {
                6700,  56280, 49240, 53708, 26680, 4800,  46128, 33172, 63836,
                6052,  16552, 47728, 58792, 29244, 34368, 17744, 26884, 36020,
                10124, 41416, 16448, 64192, 27064, 4020,  24396, 51532, 12236,
                55564, 17404, 45732, 58332, 19412, 49724, 9640,  16616, 40024,
                26004, 62576, 57376, 24060, 41212, 33764, 27656, 63376, 5160,
                32508, 37380, 53220, 30204, 64328, 33156, 16100, 50912, 58552,
                18152, 26516, 3120,  63156, 32312, 5100,  15524, 48272, 63944,
                20988, 35744, 2140,  16092, 58196, 52468, 42324, 17580, 51252,
                44808, 29204, 41156, 60164, 23176, 48528, 28216, 50776, 9148,
                30664, 20044, 55468, 5944,  65104, 45596, 96,    28652, 47764,
                11044, 40936, 37580, 58556, 4524,  48080, 55392, 19836, 2988,
                58072, 36824, 47752, 60920, 53580, 20364, 15956, 48292, 62356,
                8120,  18780, 24808, 14336, 31792, 39220, 25144, 29304, 60888,
                49828, 32740, 40876, 18504, 30484, 12156, 55172, 32708, 48820,
                45280, 25880,
            },
            {
                61064, 40388, 21608, 13140, 62852, 37216, 57708, 22232, 42692,
                28440, 35324, 12132, 21912, 44732, 10628, 50972, 59908, 1384,
                20128, 56788, 4672,  47720, 8192,  58916, 18280, 33376, 42168,
                8488,  23340, 28572, 13452, 35092, 55932, 62212, 29796, 44472,
                11960, 20564, 51556, 2236,  18124, 7108,  15164, 51204, 43200,
                25220, 59840, 252,   12004, 23688, 55524, 46588, 1880,  42108,
                34096, 38220, 55244, 16952, 41596, 44780, 23708, 54236, 39044,
                17664, 57100, 50720, 32792, 38532, 21276, 5892,  32048, 37132,
                55012, 15020, 10932, 38000, 45372, 34340, 62724, 480,   23788,
                63800, 47336, 3180,  22796, 57176, 16644, 38316, 24464, 55128,
                32056, 20552, 64888, 25340, 29216, 17260, 10924, 30880, 45444,
                41020, 54440, 18460, 24420, 8784,  44772, 36396, 11184, 29656,
                4572,  54832, 34432, 144,   42820, 11432, 63744, 45468, 35452,
                12600, 920,   64444, 44560, 58124, 35952, 5616,  40740, 20032,
                2164,  30576,
            },
            {
                43588, 16196, 35620, 8204,  48148, 1164,  30520, 17328, 7544,
                56368, 50584, 62548, 41016, 3456,  64364, 14172, 32296, 45656,
                61948, 52660, 30796, 34512, 43296, 37376, 54456, 2536,  64704,
                36920, 49340, 62908, 3956,  43324, 38252, 21328, 5936,  52708,
                36068, 7812,  43112, 30624, 64400, 46376, 58492, 21204, 8540,
                55756, 17004, 39796, 49980, 18620, 36204, 7364,  21700, 13004,
                65144, 9856,  49440, 7232,  20420, 58284, 10592, 33428, 6804,
                42440, 27488, 11236, 23912, 4364,  47508, 13032, 60472, 24044,
                2604,  20348, 57768, 26184, 3808,  14272, 19268, 54288, 36768,
                12052, 40352, 34656, 27280, 10140, 41496, 53004, 4884,  61588,
                35024, 1512,  49208, 8136,  41740, 52932, 34256, 61628, 21848,
                28060, 11684, 512,   32836, 39752, 63620, 2292,  56720, 21860,
                45652, 40128, 60488, 49876, 57624, 19712, 52724, 8040,  17516,
                23328, 53500, 27960, 14588, 8276,  24560, 51420, 27788, 62884,
                14680, 52300,
            },
            {
                24160, 4544,  28612, 59000, 41316, 24472, 10348, 54220, 46544,
                14340, 280,   31400, 18584, 25680, 38584, 55832, 23268, 7492,
                37560, 12484, 25264, 15220, 61120, 10696, 29044, 46344, 20688,
                5152,  31932, 53228, 10484, 25480, 15748, 2940,  46760, 32892,
                59736, 14892, 26716, 38116, 12980, 53492, 36256, 29720, 11632,
                34844, 46244, 6600,  62172, 43328, 57700, 28996, 60560, 51624,
                31084, 44404, 24088, 36240, 29932, 50936, 2016,  61348, 14140,
                49180, 3632,  60200, 44572, 64132, 53732, 28052, 43316, 9992,
                62508, 46796, 32488, 50316, 8292,  56040, 42036, 31820, 6352,
                52008, 59776, 15824, 50124, 62380, 31512, 19876, 14068, 9052,
                45068, 16136, 55856, 13220, 22756, 59380, 1872,  38932, 14368,
                6952,  64916, 42960, 59112, 13856, 50032, 25936, 33984, 51804,
                12792, 16976, 27240, 6184,  22768, 30596, 3924,  48384, 41588,
                60032, 36916, 4888,  49020, 38912, 60564, 17408, 3612,  56100,
                37296, 9588,
            },
            {
                46712, 64168, 54040, 32384, 18444, 51516, 65480, 39600, 25976,
                59448, 37724, 9312,  54408, 48424, 5724,  29916, 41928, 18900,
                48932, 3900,  40660, 20912, 4,     49688, 23180, 14200, 57828,
                26772, 16504, 39192, 47504, 59968, 54884, 28140, 65484, 23468,
                0,     55060, 47772, 60676, 5020,  23736, 988,   41852, 49264,
                65248, 22204, 31656, 26880, 2824,  14304, 39312, 25436, 4824,
                19068, 492,   57148, 14048, 63760, 38460, 25964, 45376, 29240,
                21588, 52956, 31052, 19004, 9076,  35028, 16324, 56772, 39176,
                6908,  35728, 17476, 65296, 29536, 22172, 61512, 46652, 17636,
                25244, 29392, 1248,  44764, 7020,  36176, 47540, 58624, 42088,
                26708, 59916, 30428, 36032, 46432, 5500,  26508, 50632, 56236,
                46940, 35548, 52268, 22432, 29272, 5524,  17980, 61568, 38528,
                3376,  65344, 32484, 46200, 15244, 38020, 62556, 33228, 27020,
                10000, 46012, 21316, 56432, 33392, 11452, 45892, 42236, 21884,
                33168, 58268,
            },
            {
                38804, 20432, 11448, 2404,  44860, 13756, 35128, 3080,  32616,
                19396, 23224, 45320, 60640, 34812, 16524, 51892, 9896,  57776,
                27880, 64944, 53704, 44640, 57024, 32220, 62440, 39788, 8900,
                44612, 61484, 860,   21864, 33508, 8160,  41288, 11788, 50332,
                17316, 40536, 20296, 9608,  33332, 44876, 18888, 57064, 26320,
                3160,  15112, 54320, 10876, 34128, 48752, 8804,  45480, 54816,
                40984, 61384, 27972, 46912, 4972,  11896, 19124, 54568, 35656,
                62988, 40212, 12800, 37648, 26116, 1368,  50116, 20996, 30152,
                49068, 24992, 736,   40436, 52360, 11812, 38612, 2480,  10088,
                43384, 56212, 20652, 12912, 54304, 25032, 1852,  29708, 51264,
                22044, 3008,  39832, 52044, 19420, 64208, 43648, 9920,  18616,
                25196, 31268, 16632, 10148, 48000, 55044, 43716, 8212,  30344,
                23676, 42432, 8948,  53348, 59064, 11084, 24648, 55568, 2856,
                15484, 58456, 31120, 18224, 1400,  64108, 29388, 7748,  49596,
                12752, 464,
            },
            {
                16760, 50600, 42120, 27172, 61096, 22068, 6604,  49628, 61640,
                42272, 4764,  28240, 13292, 2580,  22160, 43964, 62164, 1140,
                32784, 15968, 35484, 7784,  26428, 17644, 5760,  51840, 33848,
                55824, 12208, 28956, 50816, 18308, 45784, 37000, 57780, 30892,
                6196,  35228, 57232, 29124, 50020, 62536, 15696, 39148, 5972,
                60868, 37032, 44620, 58964, 52288, 20496, 62684, 32144, 15088,
                35444, 10332, 22080, 33628, 51768, 43552, 59368, 8496,  16248,
                392,   7484,  46604, 58940, 54972, 41708, 62864, 3748,  13548,
                59116, 54224, 43712, 15496, 5956,  34720, 26612, 59256, 49640,
                64616, 37624, 32668, 60548, 39744, 16772, 62972, 11396, 38028,
                17748, 63688, 6524,  11096, 33252, 15064, 29904, 36488, 62512,
                2736,  5928,  56776, 36704, 1784,  39424, 21220, 14276, 59660,
                49728, 56068, 19840, 35168, 1888,  40380, 44000, 19008, 49756,
                64992, 39844, 7040,  43560, 50608, 23492, 54688, 36328, 59672,
                26332, 29984,
            },
            {
                55124, 33772, 5864,  57356, 37708, 53160, 28424, 17008, 11268,
                47308, 57348, 52132, 40236, 64784, 56444, 26156, 36620, 12744,
                46308, 23388, 59400, 11232, 47616, 37932, 42912, 1948,  19392,
                23856, 36680, 42028, 6704,  63728, 14208, 2076,  19976, 26456,
                43856, 63208, 13712, 3276,  25200, 53884, 10716, 30908, 51464,
                19660, 28696, 7824,  24356, 16704, 37584, 1736,  23276, 58720,
                49328, 6916,  53548, 16600, 40456, 3036,  31644, 24104, 48220,
                57088, 33208, 22676, 15012, 6068,  32244, 24148, 45712, 34292,
                19216, 11292, 28416, 60908, 46356, 56576, 20024, 31160, 14372,
                22872, 4028,  8884,  48640, 28272, 22400, 56748, 34328, 7556,
                54840, 46908, 24772, 42676, 57744, 284,   55208, 23084, 48884,
                41392, 60404, 44880, 26736, 63832, 32388, 57644, 47148, 35856,
                672,   12068, 26352, 48576, 29328, 61664, 6936,  13236, 34736,
                28972, 22672, 52740, 12812, 26844, 40700, 15736, 5056,  19264,
                44588, 62480,
            },
            {
                9032,  47976, 24720, 15304, 9400,  46096, 40988, 55268, 30808,
                7876,  15848, 33264, 19816, 10676, 31368, 6808,  50100, 20512,
                39724, 51708, 5416,  30324, 63456, 13624, 54984, 29372, 65092,
                49036, 4080,  59716, 54480, 31228, 24152, 52188, 61044, 47268,
                9100,  53080, 22404, 38196, 42372, 6868,  34580, 59536, 43232,
                12964, 47892, 40768, 63192, 5260,  42564, 54024, 11840, 29096,
                18520, 37700, 63260, 26236, 56200, 20984, 65032, 36720, 42300,
                26988, 52304, 63904, 28888, 44360, 17976, 10212, 52964, 40076,
                64284, 7820,  37128, 23616, 3140,  16920, 42976, 7524,  54528,
                35080, 41780, 18516, 52656, 5424,  42800, 3296,  49480, 44268,
                31832, 14240, 28988, 61108, 20860, 47528, 38772, 7244,  12444,
                32924, 21156, 14192, 50416, 18676, 11372, 25444, 6408,  28380,
                17556, 39904, 62780, 4744,  16148, 51304, 21152, 54708, 47192,
                388,   9500,  36228, 62728, 3948,  33696, 60996, 10624, 52548,
                40312, 3524,
            },
            {
                21976, 36964, 64408, 31308, 19896, 2744,  63524, 23412, 36624,
                62544, 25664, 892,   37344, 48720, 41648, 15456, 59784, 29508,
                3552,  56536, 18656, 41520, 50856, 21296, 25108, 9952,  45404,
                14132, 17300, 26288, 9776,  35020, 43288, 5372,  38668, 15640,
                32516, 1468,  49032, 17752, 64880, 46844, 20528, 2124,  23604,
                56680, 33200, 940,   50232, 31284, 27148, 46444, 64800, 4116,
                42056, 45336, 1328,  30764, 8992,  14584, 50204, 5308,  12496,
                19348, 9924,  3860,  48964, 38748, 61256, 57332, 26924, 4872,
                22528, 47940, 58160, 32948, 50112, 63716, 39140, 47592, 1552,
                27716, 62008, 45780, 12204, 31568, 64264, 15392, 26336, 20684,
                58452, 4708,  36880, 52972, 8932,  26824, 16848, 51636, 58996,
                28116, 53900, 8340,  40432, 61392, 3320,  53060, 42580, 64224,
                51440, 32824, 44212, 22748, 36532, 57316, 31728, 26284, 38548,
                61908, 55920, 45600, 19536, 57228, 48636, 21772, 46108, 31424,
                25016, 14612,
            },
            {
                60216, 43080, 1748,  51700, 58992, 34316, 13088, 5676,  19020,
                50128, 44728, 59640, 55072, 22976, 4360,  45256, 53804, 9576,
                62936, 33844, 26968, 864,   36732, 6572,  59888, 35904, 57288,
                32164, 52816, 39552, 47660, 20476, 58472, 29360, 11244, 55640,
                24844, 41332, 57844, 30468, 11904, 26412, 55140, 37660, 8308,
                64112, 17736, 10772, 21828, 56152, 14092, 8104,  34280, 51736,
                24956, 57940, 13108, 48436, 60656, 39284, 27800, 55592, 45848,
                62040, 40732, 34576, 20968, 12188, 828,   35920, 15892, 51004,
                30496, 1448,  41992, 13764, 9504,  29016, 24408, 11952, 57556,
                51008, 15480, 24864, 58868, 36524, 54036, 9364,  40960, 62184,
                11736, 50256, 18348, 2456,  33908, 41548, 63356, 4516,  44260,
                17892, 37968, 808,   30472, 23100, 34248, 16116, 37388, 20788,
                10108, 6720,  58488, 13568, 46140, 9248,  41692, 4248,  11772,
                17192, 24920, 32336, 15032, 28224, 8356,  37856, 972,   65140,
                35768, 55940,
            },
            {
                29400, 11568, 17776, 39848, 47452, 27432, 43696, 58184, 39396,
                3764,  13852, 29036, 8596,  63692, 27324, 35056, 17772, 24884,
                42636, 14216, 45932, 61024, 16416, 31180, 48680, 2720,  41012,
                7560,  22592, 1636,  63288, 13312, 3412,  48652, 64684, 35716,
                18996, 61836, 7368,  36640, 4024,  60272, 15972, 29152, 45544,
                51096, 27576, 39820, 61320, 36288, 18936, 60348, 39080, 20664,
                15604, 5792,  33360, 22712, 43068, 2672,  35076, 17248, 30656,
                1868,  57752, 25272, 51116, 55224, 31224, 47560, 8740,  43620,
                62284, 18056, 53120, 21132, 55448, 5068,  61488, 36216, 20544,
                33176, 8108,  39296, 544,   19488, 46788, 28824, 35436, 1136,
                23476, 39560, 65336, 45024, 56648, 13444, 30852, 24364, 35320,
                10672, 65024, 46556, 57252, 43392, 48952, 55788, 5328,  46728,
                60708, 30404, 52792, 25792, 1556,  64280, 18548, 60388, 43848,
                50780, 6636,  41196, 2240,  51776, 42836, 58780, 17464, 12700,
                49576, 5284,
            },
            {
                46456, 53408, 26368, 6528,  22260, 9848,  50920, 31788, 24780,
                54184, 34608, 42888, 18160, 50988, 12852, 61280, 1524,  37904,
                49272, 7068,  22628, 52492, 56072, 43632, 12868, 19964, 27460,
                62120, 44228, 36764, 28564, 55464, 16980, 40516, 22460, 6300,
                45700, 28092, 14676, 44352, 52160, 32744, 49168, 40508, 13264,
                3640,  34828, 53200, 6084,  43208, 25784, 48900, 152,   30160,
                54500, 61456, 36816, 52416, 19168, 57428, 10428, 53252, 23108,
                37868, 14288, 43540, 4904,  65468, 18968, 23588, 59876, 37820,
                12592, 34204, 26136, 38312, 45260, 32044, 51812, 17348, 44576,
                4096,  65212, 30280, 56380, 23216, 5748,  51352, 17712, 55888,
                47416, 32612, 25868, 10456, 21460, 48960, 60704, 1408,  55484,
                49836, 22212, 6588,  26564, 13652, 9908,  59868, 27752, 24288,
                14236, 42040, 19368, 38040, 49324, 33660, 28432, 53072, 22772,
                30612, 58336, 36860, 63460, 11352, 23980, 30332, 53272, 26804,
                41964, 19852,
            },
            {
                57880, 32844, 62992, 14296, 61228, 55352, 140,   16444, 65172,
                10920, 48072, 22020, 2392,  39132, 32104, 46932, 20940, 54728,
                30196, 65472, 11784, 39112, 9004,  24112, 64212, 34308, 54524,
                10500, 18612, 59232, 6732,  45272, 33896, 26852, 53104, 796,
                50308, 10576, 54908, 23920, 19972, 9196,  324,   62600, 21040,
                58776, 24212, 15172, 47304, 2648,  12624, 58324, 9744,  45540,
                40604, 26840, 11104, 6596,  46596, 25476, 64304, 6124,  48344,
                60932, 8332,  32348, 16508, 27492, 41412, 6236,  54508, 28020,
                3244,  57156, 6780,  64884, 15756, 292,   41644, 9108,  60264,
                26296, 48324, 41448, 11088, 44252, 61668, 12976, 59416, 30336,
                7884,  15684, 51924, 5216,  28568, 37900, 7852,  19756, 41088,
                31892, 15336, 61088, 36372, 20076, 4032,  32088, 39784, 1372,
                35516, 56620, 3916,  11248, 54956, 15512, 7864,  35916, 1128,
                13480, 46092, 20748, 54420, 34408, 48052, 6264,  62168, 3240,
                34644, 8728,
            },
            {
                23716, 1204,  38324, 44968, 35376, 29020, 37984, 20724, 41460,
                6816,  60740, 30724, 58648, 55812, 5964,  41272, 10320, 58524,
                4848,  35748, 19116, 32444, 2212,  28660, 50680, 3944,  46184,
                14900, 49716, 31840, 23764, 51112, 8592,  58056, 14344, 31316,
                60652, 38920, 33852, 64232, 41004, 57056, 25428, 35600, 43768,
                7448,  55308, 32004, 65340, 28984, 51544, 33436, 22812, 64088,
                4684,  49536, 17168, 62648, 30028, 14832, 32596, 40448, 19620,
                28128, 52144, 46176, 58272, 10188, 34772, 45472, 14720, 20244,
                47092, 42364, 10564, 49144, 59140, 27540, 22880, 55744, 38068,
                13692, 21596, 53604, 16724, 33532, 27044, 37896, 41856, 3840,
                43840, 61396, 36304, 58096, 42988, 53980, 14704, 45984, 58896,
                27820, 3312,  51680, 41872, 54784, 63560, 44648, 50732, 18028,
                64900, 48144, 28768, 61152, 40720, 21388, 58848, 48368, 39760,
                65476, 9212,  26188, 4016,  18408, 14292, 39976, 21164, 45440,
                60024, 16016,
            },
            {
                41320, 49888, 10756, 52520, 18748, 5080,  48728, 57028, 45964,
                26928, 15472, 37320, 9344,  25780, 14688, 52832, 28000, 16532,
                44552, 24512, 56872, 47636, 59740, 42072, 17540, 37312, 58000,
                25296, 40248, 620,   12628, 41984, 19316, 62420, 37236, 43252,
                21192, 17284, 2364,  5672,  47564, 16712, 12040, 30060, 49696,
                17812, 38456, 9964,  20468, 40204, 17184, 55884, 37448, 13636,
                19628, 32736, 55460, 42260, 38480, 1316,  50984, 44060, 12464,
                36052, 56,    22180, 38344, 62320, 50176, 1892,  31940, 63572,
                52164, 24128, 31140, 19832, 35404, 12456, 47240, 30084, 2448,
                50848, 35192, 4920,  63228, 2036,  50336, 8556,  24756, 19376,
                54652, 22168, 12656, 2976,  17948, 64196, 34440, 25720, 11828,
                38948, 47656, 8860,  33420, 16460, 28904, 22136, 12212, 53920,
                8656,  22524, 32684, 6260,  24904, 44160, 3484,  26740, 55192,
                16440, 50216, 29800, 59632, 44188, 56660, 31872, 10076, 37136,
                54876, 29736,
            },
            {
                63700, 21332, 3940,  31364, 25388, 59364, 8876,  13396, 33792,
                3028,  53436, 49496, 19048, 43780, 64544, 23252, 33648, 62324,
                376,   51584, 40688, 15260, 6076,  53996, 22124, 9328,  30692,
                6364,  60700, 35272, 64940, 53908, 2832,  25392, 11256, 6956,
                46736, 58288, 30244, 51128, 26892, 37164, 60940, 54172, 5420,
                63284, 27036, 45996, 59804, 6628,  44548, 3248,  27692, 42684,
                59652, 1884,  23916, 8864,  57732, 21064, 60292, 4196,  55880,
                63028, 17084, 54240, 4312,  13676, 25040, 56768, 39828, 9096,
                36536, 16528, 60360, 2880,  40152, 53244, 7036,  64020, 18700,
                59024, 9868,  28804, 46516, 19928, 57840, 15084, 64808, 34728,
                48552, 29180, 40548, 47096, 32192, 22820, 1920,  52852, 5624,
                63696, 19296, 24004, 60288, 288,   38660, 6872,  58764, 26812,
                42516, 15908, 45908, 52156, 13156, 62888, 32248, 10492, 19960,
                34024, 42424, 6412,  37484, 51884, 32,    64664, 25172, 50980,
                13212, 6728,
            },
            {
                34084, 43880, 55632, 15832, 64556, 42632, 22116, 29688, 63236,
                39956, 22864, 61972, 1000,  35284, 47700, 3724,  38492, 49608,
                13020, 29744, 8352,  26540, 63492, 33616, 13240, 62264, 43012,
                51360, 20960, 16608, 28828, 47544, 38796, 33216, 49520, 28476,
                53644, 23664, 12692, 62792, 7860,  45000, 21456, 2984,  41952,
                33692, 14116, 1172,  52276, 35528, 24672, 62968, 7988,  53368,
                48012, 35636, 29120, 45844, 12932, 27644, 35276, 23784, 8752,
                31532, 25532, 42336, 48444, 30116, 18436, 60696, 22028, 4404,
                28844, 55148, 6252,  44928, 25360, 57676, 15556, 43664, 32580,
                40932, 24012, 55272, 36952, 40492, 31536, 44632, 52996, 1540,
                10416, 62624, 7028,  26280, 59824, 9692,  48776, 42568, 16972,
                55136, 30016, 44284, 13056, 53460, 49588, 46208, 34532, 3796,
                37928, 61976, 684,   35668, 56224, 18000, 47340, 38652, 60780,
                2876,  23804, 62012, 12356, 22064, 28044, 41796, 16624, 2440,
                47768, 27096,
            },
            {
                17640, 58592, 12620, 47588, 39332, 1940,  53988, 50024, 17092,
                5224,  44948, 31824, 13084, 29312, 59676, 8164,  18564, 43100,
                21888, 61136, 37240, 20212, 45576, 1412,  38864, 48076, 27008,
                3568,  55296, 45920, 8788,  22792, 5516,  56036, 15236, 64436,
                3564,  35112, 39580, 42792, 14968, 32936, 51960, 10360, 24416,
                48276, 58468, 22036, 30356, 11340, 49556, 14804, 31352, 21688,
                10824, 15984, 51616, 65200, 6360,  53956, 47600, 15620, 49600,
                39032, 59080, 10752, 34016, 7224,  37444, 43816, 12104, 48060,
                41604, 13556, 50320, 62452, 33736, 9708,  37480, 21748, 5960,
                49448, 14364, 60984, 6740,  11476, 22648, 5392,  28176, 38632,
                17208, 33568, 56412, 14484, 51220, 36004, 28412, 62060, 37808,
                33084, 2264,  57548, 36596, 26000, 9756,  18800, 62764, 14072,
                30088, 57204, 20524, 39972, 8548,  28232, 5304,  51820, 13680,
                57404, 45772, 31484, 17716, 48800, 9056,  35572, 58440, 20360,
                61660, 39500,
            },
            {
                8892,  24760, 36492, 6496,  27028, 32260, 11792, 37052, 60016,
                26128, 10388, 57596, 42284, 21548, 51852, 11580, 56252, 31220,
                5764,  53292, 58064, 11172, 30588, 52284, 23356, 57284, 18684,
                12116, 32732, 37456, 59588, 13512, 61896, 20036, 40540, 44436,
                18084, 9580,  55928, 20140, 904,   58808, 28952, 56452, 18376,
                39876, 8188,  54788, 42152, 61556, 19344, 57004, 46316, 40300,
                61344, 3836,  38136, 20392, 33476, 18112, 40784, 63648, 28872,
                5884,  44616, 19736, 64876, 55248, 50560, 820,   53624, 25516,
                64420, 32304, 19040, 22612, 980,   29428, 51940, 61652, 27296,
                1836,  44836, 17792, 26572, 62788, 47824, 56120, 13208, 59656,
                51752, 45048, 20692, 460,   41200, 18992, 6832,  13744, 22296,
                10640, 48212, 7316,  20568, 65432, 42096, 32160, 23500, 52720,
                47668, 10632, 25460, 50536, 31324, 64788, 23324, 43388, 36468,
                27548, 8228,  54916, 39476, 63188, 4660,  54084, 46172, 33112,
                5580,  53512,
            },
            {
                30680, 704,   61460, 51332, 18580, 56924, 7680,  46116, 20272,
                34092, 51080, 7344,  54924, 16132, 39624, 26664, 63940, 35520,
                46616, 2276,  16732, 49104, 42528, 4796,  15628, 35776, 7788,
                64104, 41112, 1796,  25828, 43488, 51212, 31340, 8,     59092,
                27004, 51784, 31092, 61172, 25588, 46928, 38776, 13188, 64740,
                27416, 36268, 16060, 3648,  26232, 38176, 500,   34148, 6152,
                55668, 26524, 42512, 58296, 652,   44972, 10256, 2840,  21280,
                53140, 14596, 2112,  27536, 23132, 15520, 29276, 34940, 17672,
                57264, 7496,  38448, 43132, 48780, 17076, 11980, 46416, 56712,
                34648, 53944, 30660, 51228, 104,   35988, 19172, 42588, 25604,
                3356,  23452, 37468, 64636, 30212, 54268, 58948, 45928, 52324,
                26948, 61548, 40244, 14852, 29416, 5736,  57896, 2148,  43644,
                7016,  18340, 60436, 2980,  41972, 16052, 54532, 968,   20992,
                49404, 15636, 2008,  33900, 25728, 43448, 14672, 28964, 23272,
                12032, 44720,
            },
            {
                49408, 41208, 21928, 33904, 44016, 62588, 23836, 41040, 456,
                14848, 64836, 28264, 36520, 4700,  45436, 1580,  23068, 14388,
                41264, 27956, 24132, 34468, 64932, 55952, 28228, 61048, 45212,
                30112, 52988, 16228, 57460, 29376, 10644, 35928, 23772, 7928,
                37528, 45612, 4316,  49268, 17000, 7236,  35380, 2092,  49820,
                4600,  45864, 59556, 32720, 48608, 9416,  64844, 28600, 23520,
                17824, 49612, 11984, 30864, 24456, 61520, 37028, 56860, 32696,
                60988, 35724, 46968, 58016, 41436, 9060,  62776, 45988, 3580,
                10692, 27988, 59508, 4344,  55780, 65436, 39932, 4948,  19148,
                9304,  38768, 12664, 41820, 58760, 32680, 9456,  50104, 30592,
                61880, 8376,  49056, 11924, 43292, 4420,  24576, 39440, 3192,
                18360, 55816, 34200, 50768, 46428, 54020, 16764, 35384, 39508,
                27840, 55280, 33480, 45392, 12144, 58360, 35148, 10172, 30836,
                63600, 40860, 58572, 21376, 52364, 10668, 60168, 3284,  56824,
                37544, 64052,
            },
            {
                19272, 10680, 55176, 13720, 2680,  16068, 28396, 58536, 53776,
                47732, 39224, 23376, 18928, 62500, 33160, 48928, 53712, 59372,
                10064, 62540, 38972, 6248,  19580, 9548,  40328, 440,   20816,
                49680, 23796, 6476,  48232, 18864, 4260,  55116, 47388, 62816,
                14088, 21632, 11616, 34460, 63816, 22760, 43756, 60128, 31328,
                23148, 53240, 12012, 20844, 43428, 17160, 51076, 13064, 45332,
                36528, 63584, 8312,  54724, 15148, 50900, 27128, 13192, 42124,
                24620, 51196, 11512, 38684, 5188,  33012, 56376, 20444, 39252,
                51412, 35968, 46992, 13288, 26144, 35196, 23264, 31772, 60008,
                25232, 64164, 21252, 7780,  24664, 16328, 64540, 5012,  40136,
                14920, 33220, 54872, 27120, 16364, 34412, 9948,  63736, 31028,
                43188, 5364,  24108, 11384, 1076,  22376, 61340, 12488, 51232,
                64520, 14548, 37520, 22688, 5860,  47520, 25560, 61124, 52980,
                18672, 11996, 47104, 6316,  30168, 36428, 19548, 40452, 47980,
                15840, 26172,
            },
            {
                5348,  58952, 29476, 47184, 38872, 52320, 9852,  35544, 4996,
                30828, 11168, 3152,  49992, 58504, 6452,  28980, 17828, 7256,
                20760, 50516, 55376, 13204, 46332, 25372, 33024, 54044, 10984,
                37184, 13864, 60660, 33440, 38112, 65256, 42472, 16924, 32848,
                53056, 57568, 42024, 29288, 54836, 9524,  52292, 19932, 15408,
                42412, 7592,  29484, 62216, 55556, 4844,  40840, 54180, 59820,
                1640,  32856, 21124, 46884, 39940, 5808,  19684, 48492, 4088,
                8284,  18240, 30416, 21672, 61188, 49784, 13136, 43300, 24120,
                62084, 15668, 21528, 31088, 8852,  53324, 1620,  42464, 14740,
                50640, 44216, 4228,  56416, 45860, 37952, 54208, 21668, 47140,
                57804, 19452, 5612,  61288, 46684, 57388, 21160, 49856, 37048,
                15476, 59600, 28356, 63484, 36476, 41544, 31008, 25748, 9040,
                21080, 4928,  49564, 62332, 29260, 17384, 39104, 7292,  43860,
                4064,  37224, 24776, 61644, 16588, 49932, 65260, 8468,  32080,
                1744,  51968,
            },
            {
                43368, 37100, 24168, 4204,  65160, 31572, 48732, 21484, 62976,
                18244, 44692, 55664, 41696, 25252, 12880, 43136, 38248, 34944,
                44864, 32032, 3684,  29396, 60508, 50940, 15056, 58976, 42764,
                4008,  27504, 54540, 44784, 8536,  22164, 12396, 28336, 2352,
                25724, 6500,  38380, 1508,  14772, 26748, 40092, 5444,  56312,
                63144, 34536, 39464, 2344,  24340, 35424, 30988, 21572, 7668,
                16020, 43960, 27824, 3528,  59136, 34708, 64700, 31592, 55188,
                44368, 63748, 1516,  53448, 16072, 26696, 2564,  31464, 7188,
                54080, 136,   41932, 61080, 49720, 19540, 45744, 57984, 6880,
                36828, 28884, 33540, 49152, 13440, 29524, 2808,  26460, 11572,
                35416, 44680, 24196, 38368, 2964,  52168, 28836, 12952, 660,
                47532, 52748, 8032,  44708, 19112, 56344, 3560,  48360, 58220,
                44792, 31520, 232,   41228, 56996, 51664, 33188, 21856, 48784,
                28556, 32700, 53892, 41868, 496,   45252, 26676, 55588, 22132,
                59884, 34312,
            },
            {
                63468, 17452, 8700,  56652, 19496, 25612, 6612,  42004, 13472,
                27124, 60104, 34352, 15548, 31864, 9048,  56384, 65364, 132,
                25760, 58724, 16432, 42224, 36136, 2628,  22708, 47172, 30436,
                63652, 19868, 40192, 948,   25332, 57944, 48648, 39240, 59996,
                50372, 17652, 61980, 47948, 58944, 44988, 33236, 11672, 47364,
                25692, 18296, 51660, 14572, 47456, 57696, 10612, 27092, 39128,
                49072, 56476, 37504, 11744, 52784, 23156, 9736,  16256, 38184,
                28236, 57092, 34548, 42592, 46548, 37304, 18908, 65048, 48328,
                27452, 33464, 56960, 5432,  36968, 15844, 63116, 27676, 11440,
                55416, 17144, 1024,  59176, 19872, 61252, 43836, 51808, 63200,
                716,   53048, 10264, 31516, 18448, 40464, 7932,  59160, 33148,
                23012, 17632, 39120, 31784, 14204, 50424, 7480,  37924, 17520,
                34544, 53680, 26448, 19664, 8684,  13276, 1672,  59816, 15432,
                64620, 9532,  57120, 13776, 23060, 34176, 5032,  11932, 42544,
                28436, 13848,
            },
            {
                46192, 50360, 32408, 41152, 12200, 45308, 60756, 55408, 37980,
                51152, 8232,  1392,  21844, 61844, 46292, 19292, 51876, 14580,
                48280, 10708, 21904, 54600, 8512,  64244, 38520, 5708,  16964,
                50188, 9404,  31604, 61876, 15816, 52316, 3652,  34444, 10152,
                44148, 31128, 20460, 35756, 23816, 3380,  65496, 29000, 36912,
                188,   9408,  60180, 28188, 6116,  63788, 18228, 52624, 61920,
                4652,  24544, 60376, 18060, 29752, 45016, 41384, 804,   50548,
                22176, 14184, 9392,  24744, 5852,  59056, 54848, 9296,  40004,
                12608, 17876, 45224, 11048, 29308, 40892, 3920,  33888, 20620,
                47384, 62200, 23528, 40340, 10200, 35884, 7180,  31928, 17836,
                39704, 28212, 58392, 65308, 14504, 45036, 25324, 54660, 41768,
                64484, 10540, 57740, 4296,  60976, 27340, 21640, 63280, 42508,
                11204, 60720, 15388, 36404, 64288, 45852, 27528, 54852, 40364,
                3056,  20300, 46256, 7136,  39248, 58852, 51548, 18304, 38432,
                54060, 6480,
            },
            {
                25756, 128,   61900, 53736, 35980, 1768,  17028, 32904, 3996,
                23696, 46668, 40296, 53508, 36976, 28048, 3016,  23576, 30292,
                39576, 62336, 33924, 45828, 18592, 28300, 12192, 54952, 25776,
                34952, 57772, 13028, 46368, 35824, 29852, 19596, 63776, 14932,
                24228, 56052, 7288,  12516, 51160, 17172, 53588, 21240, 58480,
                50004, 40728, 45204, 20536, 37600, 43980, 2012,  41564, 34372,
                12848, 31432, 8776,  50084, 2488,  62048, 56084, 26408, 60848,
                6540,  47444, 39772, 62948, 12292, 29444, 34064, 44296, 21980,
                60304, 51136, 25316, 64480, 22516, 56272, 50956, 24656, 39264,
                53496, 9016,  43224, 27176, 50860, 15876, 55848, 23800, 12804,
                43128, 7868,  22092, 49796, 34820, 1604,  61996, 19284, 3844,
                27816, 45876, 35220, 24828, 53916, 46828, 12704, 29976, 2232,
                24208, 6864,  47856, 55720, 22584, 42476, 34456, 10552, 24580,
                36312, 50468, 26964, 31860, 63396, 15376, 29404, 47360, 62820,
                9932,  20972,
            },
            {
                39676, 29912, 14996, 22016, 27508, 58248, 50240, 10996, 28672,
                64680, 18712, 57520, 11648, 4548,  50220, 41220, 60596, 7848,
                55660, 5024,  26376, 1228,  52536, 40836, 32884, 43828, 62156,
                1860,  41916, 23236, 51256, 5484,  43184, 54340, 7772,  40160,
                47016, 616,   63000, 42156, 32084, 9008,  39116, 5128,  13260,
                23944, 55820, 31348, 11808, 54232, 32504, 15136, 25548, 47692,
                19752, 65352, 43016, 38856, 14592, 20764, 32872, 12164, 36568,
                19096, 58008, 32636, 49652, 20648, 848,   51988, 15224, 3420,
                36248, 6112,  39024, 1428,  48104, 13772, 7776,  60688, 2096,
                30300, 14544, 5104,  32392, 64924, 3936,  38692, 49312, 59764,
                34164, 47212, 4896,  55708, 11824, 29344, 47944, 36344, 15296,
                51316, 6840,  20500, 42728, 1476,  33548, 39852, 52744, 59196,
                45412, 28728, 38964, 2916,  30984, 4716,  49808, 17220, 57924,
                44204, 61480, 17848, 53024, 10472, 43116, 2004,  23976, 4336,
                35348, 58020,
            },
            {
                53168, 10620, 44492, 48588, 7804,  39584, 20572, 43824, 53868,
                36440, 14624, 31212, 26588, 62920, 16984, 32716, 12572, 35832,
                20164, 42896, 49300, 13640, 61312, 22364, 58568, 9160,  20588,
                48364, 7092,  18380, 56584, 27688, 11636, 21852, 58372, 32876,
                26688, 53432, 37424, 27928, 57036, 46648, 60012, 43520, 34052,
                16284, 4256,  64536, 7624,  23284, 61156, 50760, 8568,  56092,
                356,   22884, 53720, 35216, 25660, 48268, 4664,  52516, 45528,
                30012, 2912,  25904, 16556, 55768, 41284, 24176, 63672, 27844,
                57848, 31380, 53696, 19320, 34744, 30716, 42628, 17412, 46112,
                63308, 35592, 52012, 57920, 21396, 46660, 29156, 1696,  20188,
                62660, 15720, 26548, 38104, 42220, 21044, 53152, 9256,  39720,
                31044, 62476, 55656, 11388, 16652, 65232, 8600,  18628, 36024,
                14528, 51072, 62880, 18096, 12536, 52820, 61884, 7212,  30188,
                13368, 5756,  1108,  35492, 21744, 56600, 37036, 60948, 31480,
                49476, 16824,
            },
            {
                5592,  59800, 33080, 3608,  64016, 14032, 31112, 61796, 8856,
                444,   42560, 48660, 7076,  20964, 44304, 59012, 47436, 53176,
                16172, 64440, 37564, 31164, 7340,  47060, 4044,  15760, 52196,
                37828, 29364, 64028, 39124, 1460,  62116, 36604, 48556, 4192,
                18164, 10436, 21524, 14668, 2772,  24996, 18480, 30228, 62524,
                47908, 27444, 35488, 17340, 47228, 3196,  29208, 36124, 59516,
                45220, 15656, 5772,  58580, 7880,  63064, 40256, 17416, 8844,
                64984, 53788, 43684, 4736,  62124, 37472, 7328,  45948, 49768,
                10808, 16040, 43400, 62136, 9968,  59524, 54768, 36692, 11296,
                20148, 25520, 44624, 17240, 11928, 41836, 25024, 57192, 10340,
                30128, 53784, 2600,  63740, 17016, 6272,  60536, 23824, 57216,
                13100, 48440, 26616, 36940, 49996, 23656, 44432, 5184,  56464,
                21444, 9652,  33276, 58588, 40964, 26732, 20856, 47108, 39336,
                22924, 54628, 42288, 28928, 47796, 6656,  18728, 44340, 13984,
                27188, 42012,
            },
            {
                22680, 37140, 17968, 55936, 24308, 35448, 5564,  48096, 25504,
                21768, 59928, 33664, 52312, 38084, 10176, 5176,  25276, 1932,
                28616, 9432,  23480, 55984, 16844, 26908, 36400, 31620, 57116,
                25156, 10900, 33444, 15096, 46080, 24932, 16400, 30364, 41160,
                64128, 50828, 44356, 60616, 49232, 6120,  36608, 10764, 51508,
                2284,  41960, 52500, 38884, 57108, 42296, 20920, 11212, 39716,
                27352, 33076, 51400, 29560, 12576, 44068, 28308, 59340, 24020,
                38980, 13976, 35780, 10040, 47912, 30108, 12844, 33316, 19760,
                40208, 23420, 6836,  47056, 26108, 4504,  21760, 27600, 49764,
                7048,  55332, 2592,  37776, 8124,  62064, 34928, 6060,  51656,
                40220, 44504, 23056, 32996, 58300, 30628, 43724, 3208,  34048,
                18236, 180,   41064, 6040,  57508, 28820, 60056, 32068, 48596,
                27500, 596,   43568, 24540, 8256,  55276, 36188, 1740,  65324,
                33284, 48992, 58708, 12112, 64340, 26104, 52220, 9612,  55452,
                2520,  65072,
            },
            {
                12768, 50796, 28748, 41180, 52004, 44940, 58832, 17460, 40572,
                56504, 12248, 3164,  64220, 24252, 55204, 39868, 31488, 57864,
                45328, 51748, 3644,  60352, 39964, 51208, 65264, 12708, 44196,
                2872,  60492, 52940, 41524, 8880,  50192, 59280, 6944,  12836,
                56404, 1984,  33472, 29480, 40600, 64424, 22056, 56320, 25916,
                8672,  21032, 60856, 12264, 25404, 5720,  64048, 53856, 17688,
                4200,  63324, 40804, 19276, 55572, 22440, 3628,  33816, 49948,
                228,   21016, 27896, 58432, 22904, 18048, 59356, 52528, 1340,
                61168, 56460, 28720, 37400, 14776, 52256, 40980, 332,   58212,
                31600, 40392, 64064, 28260, 48572, 54324, 15192, 45604, 18724,
                36108, 13572, 8748,  51100, 46988, 11284, 25580, 50076, 64660,
                45312, 22304, 60820, 33144, 13900, 20136, 2056,  38788, 11472,
                64172, 54032, 36796, 48860, 15812, 32092, 44696, 10228, 14876,
                27292, 18640, 8436,  37720, 16088, 33032, 40292, 59560, 20740,
                34100, 46636,
            },
            {
                25684, 60880, 844,   8664,  20100, 11832, 2000,  30024, 36960,
                51240, 27812, 45136, 15724, 30092, 18624, 62188, 14176, 21100,
                35504, 41396, 12332, 34144, 20004, 45160, 384,   22948, 39572,
                17856, 47604, 21472, 3952,  31528, 19236, 53724, 43772, 28268,
                23880, 37004, 19532, 8020,  12712, 53108, 15664, 44608, 38304,
                59124, 15188, 29828, 824,   33380, 46064, 14300, 31208, 49740,
                24632, 10068, 46740, 1344,  49244, 37392, 15588, 56656, 10952,
                42192, 61824, 45408, 32880, 54068, 5660,  42812, 26644, 8912,
                34392, 48504, 3588,  64352, 18824, 45396, 33232, 62952, 16396,
                46960, 13956, 22316, 33784, 19140, 608,   31404, 58644, 27260,
                3692,  60636, 56328, 19920, 772,   40708, 15580, 37404, 8440,
                52928, 29948, 10232, 46396, 54264, 42388, 51912, 15912, 24232,
                40828, 19156, 4132,  60464, 6376,  63512, 19516, 56980, 51476,
                60132, 2752,  41280, 24100, 50788, 6024,  724,   29568, 48788,
                38552, 7836,
            },
            {
                44100, 31544, 15632, 57784, 33944, 27268, 53416, 65428, 14752,
                4152,  19812, 35184, 8476,  48084, 744,   42832, 49780, 7104,
                63564, 18016, 26076, 48760, 5952,  10396, 28860, 59292, 55108,
                5732,  35540, 28080, 55804, 64768, 37920, 12,    34748, 61724,
                47760, 16004, 54348, 59228, 26256, 34600, 448,   31732, 4852,
                46408, 34976, 54592, 49380, 18544, 61684, 36636, 7360,  44240,
                60048, 34776, 14808, 61084, 31524, 8532,  63604, 46020, 30272,
                52680, 16740, 7160,  49544, 1776,  39272, 14836, 65392, 45064,
                21584, 12500, 54156, 32000, 8180,  57392, 11592, 38168, 24056,
                8800,  59476, 3576,  52620, 60664, 24448, 40792, 11484, 65128,
                48304, 24072, 38892, 27660, 34980, 63248, 56704, 19504, 27232,
                4076,  38984, 16756, 24628, 3780,  63476, 9300,  34908, 61052,
                47064, 30236, 13728, 22896, 52116, 28796, 41840, 25184, 5220,
                34692, 45708, 30460, 61856, 55876, 46120, 22492, 63488, 11780,
                17440, 54900,
            },
            {
                20416, 63264, 37568, 45888, 49464, 60572, 42292, 23136, 9384,
                47372, 61600, 41140, 58280, 53112, 36936, 27212, 10536, 32316,
                53800, 1264,  58752, 30732, 54336, 61904, 14908, 33856, 49644,
                9592,  62568, 14348, 44672, 11508, 26356, 22084, 14712, 9436,
                4892,  39824, 44820, 3184,  48448, 42280, 62612, 50628, 19484,
                11428, 65124, 23300, 42952, 9740,  51688, 27760, 22852, 56900,
                3316,  38700, 21692, 26720, 42612, 53664, 24796, 19428, 4860,
                25920, 37964, 12780, 20092, 60320, 25008, 36276, 55356, 30296,
                17228, 38868, 42348, 24824, 49972, 21232, 3064,  43944, 29732,
                50904, 35312, 41940, 12288, 44920, 9012,  50744, 21284, 7060,
                32216, 16436, 43004, 7548,  12972, 53280, 5596,  32496, 59920,
                43320, 56024, 62444, 49092, 37612, 30964, 26936, 45236, 3416,
                7764,  57372, 33976, 43972, 38416, 1120,  12336, 48308, 39864,
                21356, 54392, 13524, 4488,  19636, 35940, 14412, 42932, 26764,
                58744, 2956,
            },
            {
                41712, 4760,  12428, 22448, 6328,  17188, 3260,  32672, 38468,
                54752, 30952, 25068, 5000,  22108, 13344, 56244, 60668, 23672,
                38696, 46280, 15288, 43176, 22096, 37816, 42000, 25420, 19564,
                30756, 42696, 23960, 6520,  49192, 59580, 40904, 52448, 57060,
                32196, 65224, 20704, 30528, 10480, 17472, 22828, 27764, 57244,
                40316, 6896,  26972, 4120,  38780, 58104, 1716,  41472, 11640,
                18836, 51996, 56140, 6144,  12140, 35400, 1952,  40924, 57720,
                34992, 64588, 54928, 41252, 31344, 47392, 11180, 3480,  50136,
                6928,  58168, 932,   14424, 36316, 60628, 26996, 53824, 18404,
                5256,  55688, 26432, 17120, 30076, 38436, 56568, 35388, 44092,
                55088, 1644,  62296, 49444, 29888, 44756, 23008, 48484, 11036,
                14324, 20868, 34208, 6912,  13016, 18528, 58356, 21836, 54636,
                17448, 50624, 26496, 10424, 58740, 17788, 62368, 31472, 8704,
                16612, 63416, 27768, 43652, 10268, 57868, 52836, 31104, 6768,
                50432, 33536,
            },
            {
                53992, 48024, 29996, 56484, 39836, 25860, 11056, 44736, 58656,
                6632,  16316, 11628, 63408, 32748, 45364, 2996,  16520, 48544,
                5316,  28544, 9236,  64564, 2776,  51340, 7616,  46496, 1808,
                53456, 36776, 57824, 17324, 33040, 3104,  46160, 17932, 27872,
                49912, 12404, 25248, 56008, 60244, 38940, 8156,  53896, 14164,
                30912, 48848, 17060, 60176, 32308, 20592, 15448, 64308, 48668,
                33296, 28764, 43692, 65088, 17808, 50724, 59472, 14556, 48392,
                9888,  22500, 3100,  28624, 7700,  63196, 18696, 23740, 35124,
                61328, 27256, 45632, 63428, 30700, 6276,  47876, 13416, 65108,
                39004, 21048, 62104, 48756, 4352,  63668, 2300,  17740, 29052,
                12548, 52108, 36696, 20944, 57580, 17212, 2888,  41424, 36292,
                50832, 984,   28472, 59224, 41684, 51404, 1324,  40080, 12328,
                36932, 64776, 5048,  46512, 53936, 35864, 22352, 50728, 56624,
                36840, 168,   50268, 39160, 33748, 24980, 1824,  61224, 37272,
                10408, 24328,
            },
            {
                17944, 27392, 9064,  61356, 35200, 52340, 62520, 48412, 21116,
                27640, 51596, 43264, 37748, 50048, 19492, 29684, 34888, 41124,
                20596, 57052, 36924, 25128, 32380, 11768, 17608, 56964, 64120,
                12772, 4648,  51104, 39548, 28628, 63592, 8076,  36380, 22808,
                1684,  38188, 6576,  35040, 51376, 3752,  47272, 36320, 1572,
                62148, 45060, 35820, 55616, 12732, 44596, 29576, 53668, 25036,
                8264,  14144, 396,   47620, 30840, 39100, 21320, 28372, 6788,
                32492, 44084, 51000, 58292, 15280, 43548, 52120, 56688, 41356,
                9204,  22388, 52952, 18552, 10364, 56184, 40716, 33728, 1240,
                46136, 11136, 6984,  33424, 53540, 14540, 26888, 47608, 61428,
                39672, 25228, 4868,  33364, 9840,  26468, 60840, 30772, 65416,
                24964, 54756, 47020, 9900,  23544, 64292, 33668, 48036, 29628,
                25088, 41956, 20716, 15688, 30476, 3040,  42600, 5912,  26092,
                46164, 11656, 22776, 60480, 7436,  48824, 18396, 40060, 46664,
                15080, 64644,
            },
            {
                2168,  40652, 50544, 14620, 1700,  19360, 29612, 13604, 36132,
                256,   18652, 60284, 3960,  9440,  57328, 65156, 7648,  53476,
                61592, 13000, 50524, 19076, 55040, 60644, 35412, 29632, 40044,
                27112, 22444, 61368, 10508, 20604, 54860, 13784, 43688, 58012,
                62180, 46768, 42172, 18880, 13736, 29012, 64492, 43504, 24936,
                20488, 9984,  2644,  22920, 50472, 7588,  35332, 4788,  40168,
                61436, 36584, 58384, 23116, 10260, 4460,  63112, 44936, 53364,
                62160, 16544, 26272, 10712, 36580, 32624, 516,   27884, 13336,
                47968, 33140, 4276,  38476, 43576, 23200, 16840, 59668, 24868,
                31032, 57560, 43524, 23304, 37192, 20124, 57936, 6244,  21952,
                9180,  58508, 14564, 64228, 46744, 37356, 53660, 8484,  16360,
                5140,  39856, 17784, 31508, 44408, 15468, 5376,  56736, 8984,
                60368, 2144,  33260, 61832, 9280,  27408, 59312, 14360, 65044,
                32900, 18316, 53568, 30712, 15868, 63972, 28576, 4384,  21480,
                56572, 35604,
            },
            {
                45284, 58600, 21188, 32076, 44056, 55400, 4552,  41504, 64576,
                56448, 31704, 46816, 23632, 27932, 39688, 14496, 43508, 25452,
                1692,  33336, 44388, 4408,  40956, 47284, 21592, 6416,  44776,
                15624, 48176, 31336, 1444,  46576, 35172, 25556, 4144,  30616,
                10092, 15944, 54544, 61236, 24024, 32620, 10828, 17252, 58628,
                33700, 52348, 61116, 39712, 26484, 62708, 47276, 57900, 16796,
                45316, 20340, 52776, 41368, 34304, 54892, 25552, 12380, 36184,
                19300, 1360,  39328, 46536, 22156, 61772, 6420,  38336, 20268,
                64272, 15952, 59032, 49776, 28244, 5072,  52760, 8716,  37464,
                50448, 14196, 28208, 61212, 9880,  45324, 41268, 33948, 50344,
                31376, 45824, 41908, 19340, 51552, 300,   13332, 43728, 21656,
                57340, 35472, 62560, 2548,  54008, 37104, 27412, 19864, 52692,
                14228, 44524, 49540, 55964, 39628, 47688, 52604, 20224, 38536,
                3532,  58244, 40832, 2260,  44664, 55084, 11252, 42700, 52260,
                30516, 7464,
            },
            {
                25972, 37080, 5452,  64180, 24520, 38360, 9996,  49504, 22724,
                8168,  40564, 12152, 35072, 54656, 1112,  22372, 51984, 11076,
                30096, 17096, 63132, 8400,  27848, 14380, 484,   52628, 59620,
                9080,  34004, 55372, 41800, 16352, 60772, 50288, 40568, 53444,
                21212, 33400, 27108, 1036,  45348, 57484, 4632,  40944, 49420,
                6980,  29420, 16144, 43048, 11344, 19184, 900,   23396, 10904,
                31292, 2844,  27252, 7416,  61104, 15896, 49248, 3512,  42204,
                30424, 56216, 60896, 4908,  54168, 17536, 49012, 57968, 44464,
                30676, 1992,  36016, 12344, 62360, 32400, 47116, 20676, 63216,
                3756,  18744, 40148, 172,   51568, 30156, 12756, 64448, 1536,
                16812, 54704, 3620,  29448, 23568, 40052, 59396, 33916, 28180,
                46240, 10908, 26364, 49964, 21920, 8280,  61608, 45992, 31856,
                38512, 22620, 6920,  24792, 18008, 12312, 34800, 7596,  44012,
                29256, 48416, 25204, 9168,  36908, 20464, 34380, 58968, 24032,
                13196, 62004,
            },
            {
                9844,  17164, 52596, 12280, 46832, 27944, 59912, 16048, 33780,
                26048, 52436, 15420, 62960, 48888, 17896, 61056, 36668, 46724,
                59724, 39144, 48988, 22472, 58436, 31876, 63752, 24860, 36220,
                20012, 3856,  65360, 23608, 7164,  29760, 11588, 18596, 64612,
                5872,  49212, 36780, 8576,  50592, 39244, 22208, 54332, 27052,
                13936, 37872, 55304, 5296,  31944, 52048, 39076, 33956, 55976,
                51024, 62912, 13308, 46704, 29292, 19824, 38152, 58424, 9136,
                24276, 50064, 33828, 13128, 41980, 30192, 24444, 11624, 8240,
                54668, 41736, 21304, 25236, 56892, 2460,  15212, 42264, 27596,
                55004, 34060, 47440, 56668, 17616, 4832,  54000, 24260, 38404,
                59360, 26064, 35896, 7604,  56468, 17312, 49040, 6488,  63392,
                51816, 14736, 4272,  42344, 59300, 12680, 40752, 120,   17244,
                58416, 11108, 36076, 63588, 856,   31300, 60328, 23744, 55728,
                10812, 62192, 15160, 51504, 60116, 26560, 5192,  48028, 420,
                33108, 49788,
            },
            {
                56060, 41292, 34156, 816,   57000, 18176, 6560,  53248, 46060,
                2380,  58084, 5200,  30180, 44592, 7240,  32660, 27368, 5900,
                20232, 3736,  34492, 53612, 11316, 37848, 50148, 17304, 40668,
                57684, 49416, 13500, 37556, 52388, 58972, 556,   38644, 24788,
                44096, 58736, 12820, 63436, 19888, 15100, 35260, 62516, 64,
                47780, 64236, 22324, 45748, 59304, 27856, 65236, 8080,  42760,
                25492, 37360, 18496, 57748, 43596, 1212,  32172, 64696, 45520,
                15564, 6968,  20400, 27144, 64976, 2088,  52768, 34696, 62408,
                26512, 6664,  51916, 45116, 9288,  39568, 34896, 58984, 11272,
                6744,  22900, 64748, 9028,  25888, 36044, 46692, 19424, 10496,
                44120, 13532, 49368, 63664, 11228, 32364, 25640, 1996,  19464,
                37852, 31288, 55644, 18476, 33268, 28368, 49228, 23624, 65112,
                4212,  51392, 29300, 54028, 42396, 45936, 4680,  50068, 16928,
                36144, 5820,  21984, 32512, 46364, 13456, 40484, 65444, 16224,
                38596, 20060,
            },
            {
                28112, 47644, 61408, 30340, 21908, 42640, 37020, 30748, 61204,
                43492, 18468, 38092, 21532, 10740, 41440, 55872, 13872, 50480,
                57612, 42104, 15128, 25696, 45764, 6992,  43340, 2732,  10944,
                30276, 25820, 43456, 19060, 27544, 45292, 33840, 48456, 14156,
                31672, 2396,  41168, 29756, 56112, 5684,  31048, 11676, 42520,
                19080, 8652,  35176, 3672,  17668, 13372, 47916, 21676, 15216,
                4056,  49028, 5816,  34916, 10796, 50596, 22332, 12832, 27780,
                53044, 38784, 58152, 48684, 9244,  40144, 46032, 15060, 19400,
                38048, 48524, 17072, 65292, 29992, 19336, 49432, 24564, 53236,
                44916, 38140, 13616, 31624, 43192, 61500, 57588, 29076, 6888,
                62072, 30900, 20440, 42744, 38880, 60412, 45452, 54432, 41324,
                7948,  24124, 61940, 47316, 39052, 5768,  56288, 34784, 42880,
                26224, 46948, 14972, 19508, 57364, 26628, 13844, 64160, 40612,
                27612, 54700, 43108, 2676,  56372, 7760,  19100, 28960, 53484,
                44024, 6484,
            },
            {
                3440,  23812, 15372, 7732,  51140, 63312, 3144,  13248, 21060,
                10168, 28068, 49804, 59728, 25104, 64316, 3156,  38272, 23832,
                9804,  28900, 64964, 2272,  56512, 20776, 28376, 62176, 54064,
                46152, 6128,  61012, 2960,  10156, 54612, 21968, 8056,  61380,
                51788, 18132, 23316, 47580, 25864, 44424, 60732, 52716, 24040,
                32236, 59988, 50716, 25600, 57068, 37144, 2228,  54248, 29716,
                58604, 40268, 64116, 24352, 53672, 40808, 55672, 4724,  35428,
                60172, 60,    44108, 16900, 36036, 21956, 59200, 5412,  29088,
                56784, 764,   32988, 13384, 55576, 6192,  61648, 976,   30944,
                17480, 59752, 2728,  50656, 21392, 14868, 2104,  40276, 52140,
                34924, 1124,  53472, 5300,  15804, 22200, 9596,  29220, 13832,
                59072, 44652, 2388,  9732,  15552, 62804, 20236, 11444, 54292,
                8048,  40028, 33880, 2632,  9980,  37336, 22188, 32120, 1812,
                47428, 18932, 62984, 38440, 30080, 60872, 51044, 35692, 10572,
                57788, 63232,
            },
            {
                13008, 37364, 55068, 45200, 11092, 34524, 24376, 55472, 39068,
                65312, 32788, 812,   54212, 35616, 15864, 47960, 18868, 62424,
                44156, 52036, 31628, 18268, 35852, 59708, 13924, 33576, 22196,
                16104, 35316, 51456, 32100, 63920, 41388, 16648, 56912, 26304,
                37936, 10548, 55020, 3544,  36376, 9488,  18020, 3092,  46676,
                40064, 12860, 29060, 44088, 10280, 41512, 33504, 60904, 44748,
                11364, 20696, 31184, 16056, 8220,  26904, 62564, 17456, 47460,
                31452, 22988, 11936, 29700, 55696, 3136,  32352, 51308, 43600,
                12008, 60592, 24336, 37064, 41528, 22384, 43928, 14728, 36120,
                48320, 28516, 40864, 54328, 7296,  33564, 48924, 22944, 17336,
                45616, 24892, 58364, 28136, 47640, 3436,  36544, 65028, 49956,
                34384, 21408, 53020, 29960, 25440, 51732, 1816,  31156, 16480,
                59960, 21624, 63128, 49824, 41408, 61616, 52540, 7972,  58668,
                34236, 12636, 25852, 9768,  16096, 23312, 42856, 2468,  25732,
                21580, 32112,
            },
            {
                50384, 59452, 18840, 40096, 26860, 58544, 15860, 47380, 4464,
                51868, 41812, 14084, 45640, 8464,  30632, 52912, 28024, 148,
                35368, 6692,  12740, 40888, 48008, 9224,  51380, 39156, 4036,
                56156, 8796,  40116, 23144, 12588, 35932, 2204,  29280, 4900,
                46220, 34360, 63932, 15796, 57916, 51088, 38672, 27908, 58528,
                6724,  55184, 1356,  62956, 20580, 53208, 6056,  18104, 26788,
                35984, 944,   56560, 47124, 33448, 2716,  38396, 20908, 8940,
                41988, 63960, 5212,  50396, 61336, 25120, 10236, 64040, 21196,
                39748, 46644, 8608,  53000, 2884,  27732, 51292, 10120, 63712,
                5716,  20492, 10760, 63100, 27152, 38744, 11944, 65424, 55476,
                9188,  14104, 37744, 62640, 33256, 51888, 56896, 17052, 26244,
                5608,  11860, 37260, 42060, 57704, 35496, 44256, 48356, 38080,
                5196,  29940, 12508, 24392, 6448,  29008, 16368, 44076, 20916,
                50996, 4924,  53620, 45836, 34860, 48920, 5696,  62140, 14916,
                46868, 41656,
            },
            {
                9520,  29128, 4940,  32964, 1896,  49656, 42208, 29540, 8952,
                26216, 17572, 23700, 5648,  58040, 20752, 40200, 11708, 57080,
                46764, 21404, 61352, 55512, 23636, 960,   43036, 25344, 64752,
                47784, 28992, 58140, 17956, 47048, 59512, 50364, 43580, 62416,
                19608, 52756, 6468,  42128, 31952, 21120, 65500, 14252, 34792,
                21756, 17020, 37956, 47424, 14748, 30620, 23984, 50356, 8916,
                62204, 51976, 41700, 12540, 59664, 43796, 49452, 57436, 14448,
                51604, 25848, 33952, 19072, 38300, 47052, 15932, 36280, 27440,
                3468,  18172, 29620, 63420, 16332, 57760, 33304, 38580, 54820,
                25528, 57296, 34704, 15608, 44684, 58928, 4952,  28360, 42420,
                32340, 50540, 6772,  18892, 11888, 23928, 40216, 1612,  31964,
                48136, 55320, 63872, 18660, 7380,  13176, 23000, 64376, 26724,
                51496, 55596, 45580, 58936, 35796, 47556, 56440, 11324, 38848,
                29924, 64552, 40816, 1452,  59440, 20280, 31504, 55600, 37516,
                52736, 916,
            },
            {
                63928, 24912, 46592, 61860, 21492, 7384,  63676, 19468, 35536,
                60344, 55808, 37892, 62856, 33588, 43396, 4300,  60432, 33056,
                16220, 26384, 38636, 4916,  29424, 62944, 17700, 31984, 11800,
                20132, 42488, 196,   26120, 6688,  31064, 20924, 9556,  39664,
                13392, 30492, 22992, 27140, 12072, 1304,  48252, 8304,  53904,
                43160, 60608, 24552, 32956, 3972,  57528, 64416, 37532, 45872,
                16696, 23048, 4568,  28252, 18788, 23956, 6672,  30412, 37196,
                1960,  55024, 45628, 7672,  13120, 54176, 42312, 7096,  56196,
                48956, 58920, 34528, 42624, 11712, 47344, 7116,  17972, 45768,
                3128,  42808, 49532, 524,   24036, 51648, 19944, 36452, 2932,
                61620, 21940, 40908, 55712, 43808, 8252,  46492, 20388, 62020,
                42968, 16188, 424,   27912, 50060, 60260, 3400,  40848, 9676,
                18312, 840,   32188, 15248, 3904,  19436, 26392, 532,   59856,
                23456, 14508, 18288, 28388, 54804, 12924, 39984, 8976,  27248,
                17936, 34480,
            },
            {
                13464, 42660, 51632, 16380, 56364, 37208, 12788, 52764, 44300,
                1396,  49148, 11120, 47044, 26636, 13552, 50376, 24532, 7896,
                42360, 53744, 49660, 10532, 34224, 45660, 54424, 6752,  58988,
                36548, 14436, 53532, 61740, 38308, 52092, 15232, 25080, 56048,
                964,   59320, 45416, 49744, 61000, 33980, 44208, 25448, 30260,
                4608,  49164, 9876,  52024, 40252, 8412,  42972, 1616,  13356,
                31024, 55436, 38732, 64472, 51200, 35688, 61528, 11836, 22700,
                62824, 16384, 39792, 58672, 28428, 924,   20144, 61636, 31888,
                14000, 23296, 6100,  50492, 25884, 21172, 59496, 31660, 23044,
                13896, 29588, 62028, 18168, 32136, 8640,  46752, 57012, 16484,
                26276, 47956, 328,   30020, 64388, 27168, 59284, 13684, 36028,
                8816,  24684, 40032, 31640, 45176, 20516, 33624, 15120, 58024,
                36084, 43592, 21112, 39928, 65400, 52992, 42132, 34836, 48896,
                6940,  44400, 36800, 50092, 6424,  24612, 43952, 64892, 4748,
                57640, 20852,
            },
            {
                54520, 5796,  36000, 10992, 30936, 40868, 23660, 5752,  32532,
                15364, 21412, 29740, 3228,  18276, 61448, 55096, 37384, 19916,
                63068, 1528,  14148, 58728, 20880, 15348, 38192, 50464, 27616,
                3432,  49112, 33372, 10556, 45244, 3792,  65180, 33036, 49024,
                36496, 17128, 8812,  39088, 5472,  56884, 18816, 39680, 62436,
                12528, 36516, 28032, 63504, 17592, 22076, 27532, 48968, 60064,
                34216, 7084,  48156, 9724,  15048, 540,   45756, 53572, 42724,
                27804, 10028, 31764, 21732, 65076, 34852, 49816, 25652, 45248,
                9892,  54696, 37524, 200,   61612, 40580, 4324,  52272, 64568,
                39860, 7396,  36944, 53752, 60080, 40040, 13588, 30304, 52864,
                11020, 59052, 35372, 15200, 4156,  38320, 52892, 4936,  29564,
                58036, 51480, 60760, 4428,  10972, 56148, 52372, 29640, 47220,
                24644, 61300, 10936, 50508, 8260,  30348, 13528, 61748, 9696,
                31676, 55680, 62236, 10520, 33320, 60356, 15964, 49708, 30056,
                45816, 39028,
            },
            {
                31448, 23208, 59720, 1060,  26532, 48712, 54960, 61360, 27560,
                57608, 40372, 64604, 52544, 35964, 9276,  30452, 3636,  47672,
                28408, 32240, 44444, 37096, 26072, 2472,  61248, 9628,  23212,
                44628, 63332, 21452, 29164, 17548, 57272, 41832, 7072,  27888,
                43096, 63080, 20616, 54664, 15016, 23724, 50664, 2896,  16488,
                57888, 20380, 656,   44912, 55348, 35752, 11408, 53124, 18944,
                25180, 44328, 20976, 29724, 58088, 25676, 32300, 17600, 4776,
                59780, 48208, 2968,  43636, 52240, 5940,  39436, 17152, 2560,
                41164, 64100, 19332, 30896, 45156, 13540, 35712, 27300, 10792,
                56172, 47040, 21720, 11704, 4104,  25048, 62964, 43476, 5360,
                39036, 18648, 44956, 53872, 22380, 33044, 17820, 48772, 44184,
                21348, 12612, 33964, 47896, 23556, 37576, 8244,  62896, 6080,
                2924,  54012, 27580, 33688, 56944, 22436, 45808, 18092, 24960,
                51612, 3328,  20824, 26556, 47012, 40,    35892, 22368, 11528,
                2496,  61560,
            },
            {
                8444,  49200, 19280, 44888, 65096, 9020,  17704, 2436,  46628,
                10252, 34300, 7460,  42384, 23228, 45560, 15888, 39980, 11400,
                56868, 17656, 8316,  65380, 52676, 48100, 41148, 31240, 56532,
                17048, 40164, 7796,  54492, 24636, 35468, 12928, 22428, 54260,
                11348, 3452,  31612, 35800, 28132, 63812, 10244, 32272, 46392,
                52460, 41616, 31584, 14232, 5996,  61444, 30284, 4236,  39596,
                56324, 2544,  62492, 36812, 41516, 54716, 8148,  39056, 50872,
                34428, 19384, 57276, 14812, 24008, 10724, 29928, 59048, 51832,
                33280, 26756, 15716, 52968, 8996,  56676, 17264, 48864, 1116,
                33800, 16500, 26760, 42008, 48744, 35340, 1364,  20700, 33672,
                63956, 7572,  25368, 12300, 50080, 62292, 10004, 24432, 56128,
                2420,  38572, 18432, 65244, 15692, 43104, 26900, 18604, 41784,
                13996, 38352, 48792, 16688, 2484,  37912, 5520,  58228, 41048,
                35908, 16340, 42604, 57112, 39460, 19044, 52380, 56480, 41724,
                26176, 16672,
            },
            {
                52904, 40736, 28552, 57928, 14824, 34760, 42984, 37756, 20176,
                51124, 25284, 13760, 55852, 580,   59016, 49468, 64240, 24224,
                51644, 41420, 22072, 5292,  30032, 12432, 19224, 4480,  35048,
                13076, 60060, 1256,  37792, 49752, 2624,  60912, 19092, 46792,
                58688, 24852, 52224, 47744, 244,   42956, 37812, 56204, 26152,
                7488,  22832, 59180, 50884, 25400, 47496, 42448, 15460, 65376,
                10288, 50164, 13492, 18036, 4100,  47868, 21812, 64744, 13296,
                24492, 41244, 7984,  36876, 55544, 46800, 63320, 13428, 22216,
                7024,  48120, 59596, 39472, 23104, 28684, 62904, 42188, 20288,
                61176, 51676, 5548,  57972, 29432, 54760, 15920, 51396, 27728,
                48904, 55608, 31260, 60420, 42944, 1688,  40692, 35632, 63724,
                7728,  27736, 53428, 5496,  56920, 1016,  50556, 32060, 55240,
                22520, 64200, 7088,  44380, 25568, 63124, 53792, 28852, 1832,
                64692, 12652, 30004, 4988,  63616, 14008, 31924, 7800,  63184,
                47816, 33496,
            },
            {
                60384, 5060,  11700, 38144, 3804,  22532, 52508, 28968, 62940,
                4436,  44964, 60312, 18632, 28072, 32984, 20384, 6904,  34516,
                2424,  61820, 46248, 35524, 55528, 24736, 43800, 63908, 51172,
                26700, 47248, 30564, 11128, 63964, 43936, 31476, 39280, 5640,
                34464, 14352, 40260, 8196,  58120, 13388, 19656, 5120,  35160,
                64948, 11060, 38132, 3212,  33864, 19892, 58704, 23692, 36500,
                32648, 27700, 42908, 59624, 34152, 11208, 28744, 55956, 1928,
                30352, 62716, 49624, 27420, 32728, 18576, 4628,  37788, 44520,
                57084, 11492, 1976,  34968, 5920,  51388, 3600,  37188, 24832,
                9228,  44348, 32284, 65020, 18532, 9632,  45428, 61564, 22912,
                10524, 2568,  35784, 20456, 6400,  27984, 14076, 31596, 16660,
                47068, 41760, 30972, 45572, 25540, 35004, 60460, 11796, 46072,
                35300, 29836, 19592, 59040, 13200, 47800, 34188, 11392, 21684,
                45508, 54312, 48548, 23560, 9568,  44656, 28248, 3508,  37716,
                20372, 13224,
            },
            {
                36156, 25280, 54720, 47000, 31760, 56700, 7168,  12436, 58276,
                16388, 30856, 36408, 39416, 5932,  43760, 12688, 55396, 38288,
                14800, 27448, 10860, 58092, 16320, 416,   37512, 53832, 7192,
                21988, 42468, 58380, 19784, 14952, 27132, 53284, 9536,  50756,
                29248, 64832, 18252, 22336, 61776, 30548, 53332, 45512, 15404,
                29520, 48312, 17676, 62252, 40696, 9680,  7000,  51764, 164,
                46976, 21868, 6088,  51896, 24724, 61676, 44580, 16156, 37496,
                46212, 5228,  12064, 59568, 348,   41444, 54192, 25224, 34484,
                19984, 29320, 43284, 65208, 47216, 12992, 31256, 45664, 59212,
                14500, 38992, 2312,  12616, 23644, 36760, 6988,  39432, 14308,
                42084, 46864, 15820, 38852, 56916, 45904, 54108, 59748, 49916,
                11224, 22040, 61780, 13576, 9428,  40676, 21468, 3832,  58612,
                8936,  51028, 1040,  41380, 31552, 8116,  17132, 51060, 38600,
                26080, 7832,  33164, 37180, 59036, 49512, 54864, 24268, 58340,
                44072, 776,
            },
            {
                64488, 16280, 42492, 20256, 63348, 25892, 48452, 33340, 40184,
                23896, 49552, 2332,  54368, 61888, 51472, 25100, 29952, 60484,
                48936, 19404, 31884, 39840, 49880, 60828, 28792, 10368, 32968,
                16468, 4084,  55080, 34660, 46312, 5004,  23164, 16852, 60124,
                1492,  43496, 46272, 4468,  26576, 49896, 40796, 24164, 57408,
                1080,  43584, 54136, 27668, 13408, 55692, 44844, 28756, 18216,
                58132, 54556, 15132, 30708, 1160,  40020, 19924, 6616,  52576,
                58584, 21448, 16944, 44420, 22720, 61272, 9700,  50636, 2860,
                61868, 54596, 15168, 25716, 21384, 58116, 18920, 7512,  52732,
                29144, 21400, 56548, 47300, 41404, 59848, 30904, 55916, 4400,
                59104, 29068, 65268, 50820, 9552,  22996, 18852, 4456,  26324,
                472,   58240, 36064, 19696, 49560, 63384, 16788, 28448, 39316,
                15752, 26040, 62308, 36192, 53300, 21020, 43516, 61964, 5984,
                57636, 18440, 61180, 1276,  20608, 15772, 34892, 10988, 17500,
                50700, 29004,
            },
            {
                21892, 7372,  49864, 2796,  10312, 17444, 44332, 264,   53772,
                8720,  64956, 11776, 22420, 16108, 9540,  46916, 1156,  42948,
                9000,  53352, 4696,  23840, 7908,  20480, 14316, 45168, 61624,
                39640, 49492, 24524, 8408,  38960, 62752, 55864, 41744, 36728,
                25456, 11004, 56136, 37732, 34228, 9156,  2972,  11908, 60396,
                20308, 36484, 6584,  21900, 49184, 31196, 60212, 38108, 12408,
                40560, 8696,  62288, 37672, 48520, 57756, 9272,  31668, 34672,
                26052, 40320, 53296, 29880, 35836, 15008, 46224, 28164, 17492,
                40416, 8016,  32396, 37428, 9772,  40288, 55260, 35248, 3084,
                63892, 49316, 34232, 26632, 52264, 108,   19776, 25872, 33580,
                53136, 18248, 24696, 1072,  30440, 62428, 37452, 43544, 33460,
                51860, 39412, 6472,  54676, 2024,  31768, 44952, 52464, 6200,
                48168, 55516, 10688, 23444, 4540,  57956, 27836, 372,   30992,
                40704, 14632, 51920, 27100, 39308, 3976,  42304, 62584, 6288,
                39892, 56316,
            },
            {
                45752, 35008, 30976, 58176, 40900, 35624, 59932, 14872, 28304,
                19992, 45968, 33832, 27384, 41668, 35772, 63992, 17684, 22144,
                34172, 58684, 44800, 64668, 34900, 41468, 48336, 56296, 25996,
                1584,  65148, 12988, 29232, 51288, 592,   32916, 13216, 48724,
                20320, 32052, 61196, 14560, 54124, 17696, 64312, 31264, 38712,
                50784, 9444,  32528, 64096, 1660,  16160, 24680, 4932,  63848,
                33544, 26316, 45124, 18716, 12936, 53844, 23620, 63444, 50436,
                10928, 1596,  65316, 8616,  56628, 5552,  32132, 63764, 12476,
                47836, 51972, 60144, 508,   50144, 62248, 16080, 26312, 42248,
                10820, 18108, 5164,  15572, 8896,  63540, 46008, 50172, 11696,
                37548, 6556,  44536, 13376, 34688, 48396, 7564,  12176, 64736,
                23952, 17180, 29792, 42816, 26264, 57564, 37016, 23668, 64652,
                33356, 42620, 17708, 46580, 39588, 14472, 49224, 34864, 55536,
                23124, 46984, 11488, 43812, 65440, 52876, 30400, 48064, 25740,
                32460, 9928,
            },
            {
                51836, 60112, 12660, 27080, 54200, 22052, 6460,  62484, 38284,
                42784, 57096, 3368,  48592, 59152, 30524, 4288,  56336, 27088,
                40596, 2756,  13168, 29648, 54968, 2180,  23088, 5544,  36736,
                18264, 32124, 42160, 59084, 17604, 21796, 7516,  28716, 57800,
                3848,  51536, 7208,  28392, 48088, 21268, 43092, 46672, 27184,
                16300, 55144, 25496, 45680, 39924, 35232, 53312, 20624, 43732,
                2444,  51184, 22804, 3704,  35464, 28160, 41116, 2768,  14516,
                43748, 19616, 38160, 47664, 25308, 51592, 39708, 21708, 36060,
                4968,  24312, 18756, 45344, 28496, 5576,  32764, 48032, 23188,
                60944, 38024, 44104, 58408, 28924, 35736, 16832, 3324,  62496,
                21652, 41000, 61440, 52064, 58652, 16560, 41364, 28656, 55860,
                45760, 9072,  60748, 48152, 15116, 8248,  11956, 3204,  20120,
                13164, 30148, 2172,  60136, 26504, 65064, 11660, 19372, 8924,
                63440, 4376,  35976, 31696, 8792,  23004, 13072, 19572, 60680,
                2892,  15076,
            },
            {
                23900, 4048,  19440, 38688, 1912,  51176, 30144, 48980, 24324,
                5508,  17760, 52612, 14128, 7584,  20632, 38376, 12376, 51696,
                61492, 16252, 25396, 50992, 17596, 60276, 11820, 30036, 59384,
                53068, 21432, 10056, 45092, 36412, 61540, 47120, 43060, 64452,
                16344, 38604, 23280, 40640, 1968,  59192, 35064, 7424,  3732,
                62316, 42136, 12724, 57220, 6032,  61372, 10052, 48004, 56796,
                16424, 29892, 58816, 65012, 6536,  46508, 16676, 60916, 48248,
                28532, 59008, 32972, 4040,  13124, 18668, 59120, 1312,  53888,
                42524, 30500, 13876, 56132, 22392, 43332, 12696, 53924, 1568,
                29820, 50456, 21880, 53564, 12320, 39804, 23432, 43164, 57420,
                28500, 9800,  32212, 3816,  26528, 21896, 53760, 2244,  20112,
                13932, 35792, 3308,  22624, 34420, 62396, 54004, 41776, 59616,
                37836, 56640, 49900, 8520,  32440, 36856, 54512, 46180, 42484,
                28732, 50740, 60084, 17964, 55980, 5628,  58148, 34472, 45028,
                53536, 37168,
            },
            {
                62504, 43028, 47724, 65188, 14536, 44824, 9424,  34636, 12856,
                58696, 31276, 36892, 24880, 62632, 54476, 43168, 47804, 5692,
                32012, 36984, 47112, 9472,  38860, 43752, 33804, 50252, 14944,
                46084, 6176,  57136, 2512,  26220, 53632, 11680, 24500, 35044,
                9688,  54868, 45380, 63360, 12468, 25512, 55988, 14220, 52856,
                23072, 34076, 2348,  18300, 50520, 28540, 14264, 31948, 36900,
                7968,  42044, 11940, 49344, 32520, 56264, 20520, 36260, 5896,
                54856, 23160, 15976, 61924, 41924, 29064, 45144, 10744, 26776,
                57492, 62456, 38792, 3740,  35456, 63828, 8872,  39400, 57628,
                14376, 8036,  32920, 4388,  60316, 48236, 31144, 7916,  14552,
                48692, 54988, 19432, 46872, 39544, 6900,  61284, 32456, 49128,
                63076, 40800, 57188, 52056, 19328, 46216, 29956, 24800, 47704,
                5244,  27436, 19056, 43984, 22152, 6172,  2740,  25212, 15092,
                38400, 21236, 1788,  25804, 45804, 49736, 28688, 304,   41492,
                18212, 28064,
            },
            {
                5924,  29872, 10412, 33096, 25632, 59328, 18876, 55104, 1148,
                64368, 40912, 10460, 45040, 1388,  33152, 15732, 23536, 10044,
                57268, 20800, 63072, 6228,  27388, 19940, 64036, 8472,  40992,
                27524, 38420, 49388, 34108, 15828, 5236,  39488, 19560, 3032,
                49236, 30668, 18856, 33556, 6048,  50008, 30264, 37668, 19788,
                48624, 59488, 30072, 38180, 21436, 43064, 24368, 1028,  63048,
                52860, 21168, 39292, 24820, 9184,  42876, 25956, 11540, 31388,
                39632, 9544,  50764, 35740, 55168, 7620,  64412, 34116, 16348,
                6280,  46716, 9352,  53076, 17724, 58876, 25988, 20368, 45132,
                36164, 65508, 25044, 41864, 18952, 1336,  64176, 53088, 34376,
                25092, 624,   37024, 14964, 57320, 11576, 43804, 36644, 6332,
                24732, 30980, 11176, 28088, 5016,  38812, 220,   16916, 33708,
                10284, 53320, 63808, 15464, 51704, 41176, 58888, 31332, 62364,
                7252,  57472, 13352, 32712, 40552, 15576, 37796, 63792, 11916,
                8336,  56908,
            },
            {
                39764, 50228, 16892, 55960, 6784,  41256, 37176, 26976, 46876,
                21800, 28604, 50736, 56508, 18812, 27636, 49988, 64996, 29876,
                41976, 312,   52664, 32952, 58236, 3592,  53876, 24704, 676,
                62280, 11152, 23416, 64360, 29532, 50684, 60468, 55908, 27664,
                61808, 13920, 1052,  57124, 42456, 17200, 8732,  63996, 40532,
                11148, 6708,  46016, 65004, 8368,  52248, 58156, 46104, 17884,
                27316, 4768,  55548, 15316, 60620, 468,   51448, 57884, 64528,
                3108,  46236, 26876, 644,   21052, 48288, 23892, 38216, 50812,
                20336, 33020, 23572, 29136, 48516, 1004,  31896, 52080, 16896,
                3720,  54828, 47204, 10160, 56612, 26816, 37284, 20672, 5324,
                59792, 44140, 64624, 29512, 23292, 50016, 27224, 18260, 53092,
                868,   44816, 16568, 48644, 65184, 13596, 43364, 57424, 62132,
                22424, 40120, 1192,  35288, 60876, 12452, 48584, 18188, 52600,
                34344, 47464, 42804, 54416, 9784,  61140, 20276, 25304, 55100,
                46836, 22456,
            },
            {
                58712, 1032,  36088, 52472, 21136, 3688,  63352, 16288, 52112,
                7740,  15004, 4356,  34920, 59468, 8236,  36772, 3768,  45104,
                18012, 24812, 39372, 14792, 45456, 12108, 36392, 48256, 16268,
                31908, 53452, 18708, 44336, 13052, 41792, 31780, 8944,  37060,
                43472, 22496, 51720, 26440, 36480, 60632, 23872, 44852, 68,
                28036, 56144, 24660, 13180, 32752, 4124,  40140, 10656, 34540,
                59592, 44128, 31108, 35876, 19200, 45668, 38292, 14136, 22128,
                42144, 18156, 53172, 12100, 32184, 57184, 14440, 2048,  60516,
                41628, 12512, 64936, 43820, 13704, 37340, 42116, 6960,  61576,
                28280, 21036, 39204, 30652, 15948, 51348, 12720, 45812, 39604,
                17392, 10568, 51668, 6156,  34884, 62796, 8572,  14328, 59576,
                38728, 55412, 7708,  58464, 35460, 26388, 52476, 7280,  49604,
                14152, 45568, 30920, 24676, 9348,  28332, 37800, 22972, 752,
                10856, 27220, 64812, 3012,  23440, 35144, 4280,  50888, 31012,
                14708, 33788,
            },
            {
                19796, 12124, 24316, 61216, 45188, 30884, 11100, 43220, 33008,
                38616, 61712, 43532, 23368, 12084, 40656, 52584, 14040, 60552,
                55428, 11132, 48128, 61244, 22752, 30876, 42532, 57544, 21008,
                59980, 35704, 2928,  7392,  59408, 20888, 276,   17232, 47024,
                5092,  58428, 39740, 10016, 47880, 4688,  54608, 32868, 51156,
                16008, 35840, 41540, 54112, 16792, 61704, 29072, 22640, 49076,
                13508, 2952,  51928, 63340, 6136,  28560, 33572, 7736,  49216,
                30136, 59936, 36904, 63404, 40880, 4804,  44480, 29900, 53224,
                25908, 3268,  49548, 5804,  56420, 21820, 59952, 11300, 47972,
                34988, 13320, 1956,  63044, 43500, 33604, 8428,  61968, 28680,
                56168, 31960, 41900, 19052, 2308,  55264, 40124, 46412, 28808,
                22300, 33004, 19500, 41096, 22940, 10036, 32268, 20204, 28296,
                36888, 5916,  57976, 47924, 19988, 55256, 4772,  44900, 56760,
                39960, 16576, 19848, 30568, 46296, 57688, 7652,  43564, 38828,
                3472,  64332,
            },
            {
                43240, 48872, 40376, 27876, 8588,  48240, 56724, 24016, 2664,
                54824, 19604, 48380, 31460, 63300, 46276, 21068, 26604, 32572,
                7768,  35848, 29100, 2304,  56192, 17372, 7248,  28196, 4332,
                39816, 46460, 28692, 56752, 37488, 48768, 54428, 23936, 64780,
                11992, 32524, 20196, 63040, 15140, 29528, 19356, 12304, 58100,
                21992, 63136, 3304,  47844, 20240, 37084, 55972, 6748,  64532,
                38956, 26100, 9808,  47632, 23532, 54576, 61792, 17388, 56116,
                4432,  24460, 14756, 8144,  27900, 18524, 62384, 10384, 35956,
                16872, 59544, 38996, 34036, 27356, 16296, 50712, 30328, 23596,
                43912, 58540, 52868, 6092,  23092, 59324, 3096,  49528, 21964,
                4520,  14052, 26088, 58788, 48476, 30924, 20832, 3476,  62036,
                51236, 12672, 63432, 2340,  53684, 46808, 59252, 1800,  42228,
                56180, 17992, 63120, 3496,  42824, 33068, 64460, 13992, 29808,
                61308, 51728, 5584,  37996, 53196, 15196, 28320, 62580, 10660,
                52908, 26808,
            },
            {
                7144,  59628, 3256,  14416, 35156, 18980, 62056, 13840, 29080,
                59164, 10248, 25788, 628,   16816, 6204,  54736, 2176,  40016,
                50928, 64136, 19612, 6440,  51368, 37680, 65456, 50000, 13688,
                9376,  52136, 15028, 24752, 33124, 9828,  27396, 40444, 34764,
                50152, 28832, 7332,  53128, 34500, 43888, 2040,  39896, 47176,
                5788,  31272, 9812,  27216, 50868, 668,   43608, 15540, 32416,
                20404, 57872, 17148, 41400, 12776, 2216,  40408, 26180, 10704,
                43624, 34284, 57496, 45140, 51620, 38472, 54844, 22328, 47736,
                7936,  55488, 19708, 10240, 62868, 2084,  40324, 4232,  64396,
                8604,  17204, 27200, 35628, 47492, 18128, 26252, 41452, 53716,
                36336, 63316, 45108, 37620, 11156, 16312, 43452, 34272, 9864,
                5440,  36848, 44384, 29968, 6240,  17344, 37588, 64152, 11028,
                34032, 25840, 13184, 38816, 52452, 16176, 8060,  50156, 35612,
                24424, 8744,  48160, 60340, 11652, 41132, 18512, 48688, 23852,
                36364, 16376,
            },
            {
                29296, 54552, 32756, 63148, 53096, 112,   39048, 50200, 36160,
                4980,  42076, 52200, 37508, 57616, 29604, 35512, 59864, 22892,
                15676, 44476, 25536, 41300, 33328, 10464, 24280, 43740, 34288,
                61944, 22048, 41888, 64000, 4444,  45020, 61040, 14428, 2616,
                56308, 16968, 45736, 3668,  23364, 56844, 65284, 25056, 36600,
                60232, 18644, 44468, 61144, 34088, 12556, 25300, 53440, 46564,
                1844,  50652, 36456, 29636, 59004, 20048, 52780, 47920, 37348,
                63884, 1056,  21464, 48856, 5976,  24984, 33608, 2668,  43076,
                28144, 31804, 44856, 24468, 52668, 46400, 35960, 19256, 55120,
                32476, 49676, 41704, 10976, 55312, 38888, 13400, 32644, 9648,
                47088, 1564,  22600, 7808,  50960, 65328, 24300, 57712, 48048,
                26820, 16184, 23908, 49176, 61192, 25564, 13828, 45688, 22552,
                54372, 50528, 8820,  29616, 22780, 59288, 26672, 41808, 2116,
                20820, 43892, 32176, 26112, 1176,  22544, 33632, 59984, 2320,
                56492, 45800,
            },
            {
                9872,  37992, 17804, 22716, 42828, 26416, 6368,  21660, 46000,
                17352, 33528, 65288, 14244, 49584, 44116, 18420, 11460, 48708,
                5136,  58312, 12604, 54436, 62376, 46964, 852,   19108, 53748,
                26792, 2516,  49064, 18144, 12400, 52408, 20088, 6972,  42768,
                62736, 25464, 38076, 60568, 41720, 10848, 50120, 7976,  28492,
                14096, 52104, 38676, 23256, 7324,  57592, 41360, 9616,  60992,
                28200, 5620,  43248, 64092, 7656,  34824, 31040, 5468,  13700,
                51108, 18328, 31560, 11556, 60832, 16000, 58076, 12828, 64204,
                51188, 92,    14332, 57396, 7348,  28700, 59612, 13904, 25940,
                38264, 1132,  20896, 62208, 30068, 2252,  65040, 57576, 20184,
                16076, 59648, 33812, 54888, 28352, 88,    38968, 13724, 52884,
                41500, 59704, 56016, 10732, 34276, 40360, 51344, 7684,  31132,
                3020,  43404, 61840, 46540, 344,   37156, 19028, 62772, 12036,
                55420, 58400, 14408, 36384, 63896, 54692, 44976, 5876,  13632,
                32232, 21352,
            },
            {
                65452, 4412,  50908, 11304, 46784, 58620, 32208, 53608, 12308,
                60512, 27584, 22220, 6776,  9660,  24688, 62636, 41844, 27708,
                31852, 38032, 3024,  29832, 20784, 15380, 60052, 31320, 57312,
                37988, 7640,  30824, 55840, 39320, 29040, 58500, 36560, 31440,
                21780, 9716,  49004, 13752, 30996, 20848, 16784, 33432, 54384,
                42940, 1352,  56696, 15740, 46380, 62924, 30456, 17900, 35116,
                22396, 56932, 11312, 24484, 46696, 15792, 44136, 62344, 23848,
                28780, 54712, 42536, 36360, 27264, 3896,  41876, 20952, 37152,
                18148, 60224, 39908, 35208, 22468, 43260, 10476, 51684, 4976,
                46936, 57016, 15152, 6976,  51492, 22408, 42652, 6324,  51036,
                30552, 25256, 40584, 13068, 19716, 59336, 30220, 4536,  18740,
                31720, 1772,  38680, 20556, 4944,  54748, 28192, 19416, 60936,
                39404, 15516, 20072, 33408, 55000, 6856,  49276, 30636, 45940,
                5340,  38764, 18116, 4072,  50692, 9380,  28776, 38640, 51744,
                41624, 58216,
            },
            {
                47432, 24716, 40712, 28708, 7904,  15700, 64464, 40012, 8456,
                55604, 2080,  47488, 40556, 56424, 33244, 1560,  53264, 8724,
                61596, 17292, 50296, 45548, 7892,  27400, 39532, 5600,  12220,
                17024, 45820, 59584, 35480, 1104,  23748, 47776, 16408, 51276,
                54800, 800,   27940, 55968, 35868, 2684,  46656, 59168, 4876,
                22344, 11480, 32860, 28912, 3000,  20812, 37676, 4244,  52412,
                40356, 15024, 32940, 53184, 204,   55844, 21736, 9620,  59528,
                39616, 2608,  8296,  65448, 46972, 53424, 30872, 48976, 7152,
                26196, 11196, 47252, 4508,  65196, 16940, 33136, 41084, 62596,
                22956, 29500, 59700, 45012, 33484, 16888, 36712, 27588, 44448,
                3500,  62680, 48020, 5688,  42992, 35304, 47144, 55580, 22728,
                63612, 8708,  46524, 15072, 64912, 43268, 588,   57504, 48404,
                36040, 5384,  27748, 58308, 11072, 40648, 14520, 53392, 34184,
                27792, 61912, 49060, 30240, 41772, 20560, 15836, 62768, 26160,
                18732, 788,
            },
            {
                35204, 14020, 54048, 57512, 36628, 1756,  20020, 29596, 24140,
                43832, 36928, 19000, 59844, 29268, 16276, 38924, 21108, 46956,
                55724, 34564, 22636, 40780, 56304, 35684, 52252, 48300, 42180,
                25096, 63544, 10516, 21256, 44496, 8516,  65204, 3776,  11536,
                45264, 39660, 18916, 7056,  63976, 52420, 39008, 26956, 62404,
                37212, 48388, 64632, 40616, 50184, 54164, 13328, 44320, 65300,
                8328,  49332, 19552, 61872, 27864, 39948, 35816, 3504,  47420,
                33720, 14960, 57660, 22452, 17308, 39172, 9672,  62432, 56756,
                33936, 54316, 21216, 30316, 49924, 1308,  55364, 19980, 7872,
                36420, 11716, 40112, 25208, 9788,  48332, 60684, 11988, 56500,
                35088, 10128, 17924, 52072, 63936, 25736, 10964, 7284,  44584,
                36096, 27484, 52444, 32816, 23424, 30608, 12076, 16584, 25020,
                9824,  63580, 49704, 23288, 44412, 65276, 25164, 2500,  21740,
                9704,  42752, 23380, 12388, 56096, 34404, 59096, 7552,  48532,
                11564, 30860,
            },
            {
                44220, 6568,  20332, 32992, 60728, 43256, 51520, 47812, 3964,
                62608, 31248, 12796, 52504, 5540,  45692, 64296, 4452,  13940,
                25328, 84,    11332, 59444, 4380,  13916, 64712, 22888, 1424,
                33992, 50660, 28168, 15492, 55036, 32288, 41232, 26660, 34804,
                61892, 29964, 58980, 44080, 24536, 10100, 14356, 44832, 18988,
                8348,  25968, 17176, 6012,  24284, 9220,  59020, 27520, 23360,
                1668,  45384, 38100, 4512,  12940, 50496, 17376, 56888, 27076,
                19956, 44516, 50352, 25928, 34336, 728,   14080, 28676, 45192,
                5456,  41344, 61008, 15204, 38348, 27272, 58576, 45636, 31356,
                50880, 17920, 54380, 4472,  63516, 53008, 1456,  24076, 15440,
                39452, 54344, 22224, 31444, 2432,  15536, 39880, 60956, 51016,
                16856, 12552, 61632, 3516,  49860, 37308, 60028, 44996, 33744,
                53400, 41804, 13608, 1732,  31416, 17564, 35588, 61036, 48472,
                16752, 54092, 6948,  64868, 44,    45448, 24204, 3336,  37040,
                55404, 59956,
            },
            {
                27456, 63296, 49292, 3372,  16988, 25948, 12968, 34560, 10180,
                16492, 49456, 25800, 35164, 10648, 23128, 51284, 36148, 30844,
                43792, 63704, 51640, 18956, 25924, 31968, 9144,  19808, 58412,
                54104, 6432,  40624, 3188,  61424, 51020, 13412, 57180, 18064,
                23336, 4636,  15068, 50532, 32344, 21360, 55624, 336,   30556,
                57852, 53012, 43380, 60184, 36180, 31744, 47512, 16036, 34008,
                60500, 30376, 25636, 58328, 42556, 30948, 64828, 11192, 52796,
                4704,  63168, 12180, 6336,  55560, 61148, 42252, 23472, 17620,
                2408,  25132, 51560, 7964,  43660, 11496, 24172, 3908,  14312,
                64076, 296,   43736, 28484, 38332, 19232, 31124, 46464, 59148,
                7272,  26896, 45592, 37740, 58068, 48508, 20200, 31992, 24428,
                780,   43416, 56332, 40844, 17996, 5968,  55384, 21752, 2820,
                29692, 19128, 56948, 36872, 52400, 7900,  56256, 40100, 4604,
                58892, 37008, 26016, 32832, 18332, 39200, 53180, 29972, 43032,
                22192, 16236,
            },
            {
                39768, 10512, 23888, 45960, 8848,  55796, 63496, 38756, 53940,
                58180, 40856, 740,   61932, 55388, 42016, 27424, 18232, 58608,
                9360,  37772, 28668, 42392, 47260, 60712, 38524, 43336, 29188,
                12504, 18452, 36988, 48628, 24856, 19588, 38280, 6680,  47224,
                52396, 9508,  36948, 41068, 3288,  62832, 35052, 41612, 49760,
                13256, 34104, 1804,  11848, 55180, 18560, 2452,  42376, 56044,
                11040, 52124, 16800, 8832,  22416, 6380,  44812, 24604, 38384,
                32072, 41576, 29516, 37220, 46068, 20396, 49572, 53156, 36300,
                64580, 13324, 32568, 19556, 53968, 61720, 35344, 49084, 40644,
                26476, 56276, 34356, 22060, 13792, 8360,  41592, 33824, 20628,
                50576, 552,   61736, 12448, 8528,  27868, 53268, 4836,  57412,
                38900, 29116, 21312, 10420, 25960, 47188, 9260,  39780, 62392,
                7220,  46852, 25836, 5124,  45556, 21292, 27344, 10788, 43940,
                29584, 13560, 51408, 46840, 58344, 10868, 14820, 63628, 8712,
                50244, 4560,
            },
            {
                53376, 57308, 37736, 31056, 42148, 21092, 28008, 5352,  18884,
                23392, 7840,  44992, 20408, 15356, 3072,  60520, 6760,  48180,
                53972, 15428, 6512,  33820, 1936,  16640, 50944, 3624,  46420,
                63172, 31836, 56552, 43920, 11008, 30320, 192,   60776, 42672,
                31904, 64260, 56976, 26308, 47932, 17508, 27652, 6348,  61264,
                23864, 39164, 21104, 45676, 26372, 64044, 38156, 49568, 6660,
                20936, 36432, 47312, 63388, 54808, 34652, 49156, 896,   15680,
                61232, 9036,  18004, 59604, 2992,  32276, 11240, 7304,  29908,
                47400, 58860, 37824, 45884, 1856,  29676, 17356, 8968,  60044,
                20956, 6812,  46148, 58756, 50380, 61816, 55368, 4656,  64772,
                42892, 30200, 17484, 55508, 34180, 41896, 62032, 13432, 34832,
                49376, 6884,  60252, 33656, 52800, 64140, 28108, 15452, 51180,
                35432, 58880, 16216, 64060, 39208, 13096, 47848, 32432, 19652,
                62996, 2208,  41372, 21176, 4416,  27772, 47988, 35296, 19352,
                60964, 33596,
            },
            {
                1236,  18464, 13740, 51964, 61464, 408,   50180, 33344, 46388,
                30480, 64724, 37596, 28488, 33796, 49960, 38656, 12544, 33132,
                20892, 23972, 58080, 63096, 11844, 54604, 23308, 34996, 14884,
                25488, 8364,  22140, 4612,  63880, 35652, 54620, 22580, 15444,
                28124, 2368,  20344, 12956, 8024,  59680, 52308, 10892, 46320,
                16836, 5052,  63012, 51168, 29844, 8132,  22652, 13148, 62152,
                28620, 41608, 1224,  13644, 39276, 18972, 28864, 57448, 21504,
                47536, 55016, 24092, 51628, 27380, 15500, 63024, 39684, 56292,
                21072, 9752,  27592, 6216,  63924, 22288, 42444, 52984, 31700,
                38496, 16304, 10720, 30216, 3052,  25064, 16744, 27992, 10324,
                14256, 36884, 24584, 44744, 3980,  22632, 18072, 46424, 9836,
                65132, 18872, 45032, 13768, 36508, 1916,  43512, 31972, 12228,
                22484, 41276, 10452, 29032, 53584, 60156, 876,   57416, 50056,
                23780, 35032, 9044,  55904, 31588, 60020, 41752, 1908,  25764,
                12864, 29468,
            },
            {
                47676, 65060, 26796, 6312,  35568, 12392, 40488, 59896, 14404,
                3680,  51248, 11584, 57304, 46484, 8692,  24728, 65216, 42864,
                1152,  45972, 39936, 49140, 19692, 30488, 57260, 5668,  61376,
                41536, 49280, 59272, 16948, 27604, 46504, 8560,  40424, 50028,
                11112, 45956, 34508, 54284, 43852, 39232, 21972, 36864, 29488,
                57200, 32980, 41756, 14948, 3760,  57816, 40672, 32376, 53524,
                4740,  59076, 33292, 26452, 51488, 3708,  60100, 8424,  35936,
                42996, 2224,  13104, 40036, 44848, 58256, 4052,  26008, 16708,
                1020,  41072, 50572, 55552, 14924, 36980, 57680, 5396,  12748,
                47480, 51956, 65504, 37200, 42720, 47900, 35140, 39260, 53768,
                48616, 60308, 7944,  51484, 64248, 30812, 2136,  38200, 27164,
                23584, 54772, 31096, 5744,  56832, 23968, 19632, 61060, 55828,
                160,   49472, 33016, 3660,  23924, 34788, 17036, 38452, 6268,
                15180, 44840, 61164, 17364, 38228, 7128,  22276, 52212, 57524,
                44932, 40684,
            },
            {
                8416,  34400, 22668, 58716, 46772, 17816, 24488, 8396,  56852,
                43420, 25952, 17288, 1864,  59812, 31428, 19512, 51740, 29028,
                56780, 5560,  26396, 8460,  36740, 44124, 27552, 10748, 38072,
                52648, 1376,  33884, 39344, 53328, 13820, 58748, 32760, 19064,
                56240, 38052, 62272, 23776, 30736, 4816,  14648, 65492, 1416,
                54244, 26784, 10300, 48352, 35224, 52688, 17780, 45340, 25352,
                15156, 19724, 44868, 56208, 10832, 31608, 41144, 14400, 53036,
                27712, 33368, 65036, 7428,  21944, 35860, 48880, 33048, 44236,
                62460, 34840, 23684, 11692, 32116, 48716, 2904,  28120, 62932,
                23244, 1752,  26932, 19804, 6404,  12260, 57724, 2328,  22308,
                32292, 5240,  19820, 39700, 14992, 48116, 58200, 52552, 43044,
                15396, 1188,  40664, 50840, 16936, 39420, 48132, 4956,  37640,
                26260, 45164, 62644, 19948, 43376, 51132, 9464,  26300, 64960,
                54936, 28880, 52532, 25188, 12524, 49632, 64648, 15780, 37204,
                5008,  21000,
            },
            {
                50704, 15400, 43904, 2852,  53880, 28528, 64216, 37108, 21536,
                52888, 34948, 41788, 22664, 54052, 14016, 40000, 3348,  16536,
                35676, 61656, 31916, 14488, 53260, 3220,  59520, 47984, 21200,
                16244, 28724, 12044, 44428, 20448, 24692, 2132,  64872, 4732,
                25656, 7432,  15988, 700,   51300, 58192, 33768, 48580, 19680,
                44248, 7408,  61496, 20220, 23992, 60036, 664,   9832,  37864,
                48164, 62744, 7100,  23512, 17140, 64572, 46572, 25152, 62244,
                4588,  19136, 49636, 30704, 55752, 10356, 19240, 54548, 12920,
                52424, 4532,  18792, 43156, 60576, 25712, 19908, 44712, 35016,
                56792, 41632, 33100, 54152, 60624, 23828, 51692, 18076, 63032,
                45704, 26836, 55784, 35308, 11376, 25592, 6320,  32600, 8224,
                59480, 47500, 28284, 63548, 11200, 58376, 7924,  29172, 51992,
                17088, 8808,  56604, 14628, 6644,  61504, 30956, 46528, 41948,
                12048, 4752,  40180, 712,   33996, 43680, 27512, 9656,  31756,
                61956, 54992,
            },
            {
                28812, 57140, 38988, 31384, 10736, 41716, 48908, 2476,  31072,
                10204, 63368, 6620,  27980, 43956, 4728,  62276, 47368, 54672,
                11724, 18720, 50672, 22692, 64372, 41172, 18192, 7336,  32828,
                65460, 55704, 4216,  62228, 7124,  51356, 37632, 29336, 43776,
                53204, 59880, 41192, 47296, 18352, 26692, 9560,  40396, 25060,
                13144, 51880, 38716, 31188, 43556, 12412, 27996, 65120, 54516,
                2724,  30256, 35336, 50564, 43432, 37412, 6508,  20900, 11816,
                39428, 58904, 43900, 14928, 208,   38724, 63796, 5320,  24668,
                30752, 46980, 57104, 38364, 7172,  52636, 10304, 39484, 14180,
                5904,  17928, 8928,  45832, 14856, 31004, 44052, 8072,  41024,
                13268, 1592,  58912, 29176, 44244, 54624, 20180, 62488, 36664,
                12060, 21872, 35100, 4140,  25744, 32200, 44132, 13172, 65464,
                21680, 34348, 29812, 40280, 54412, 36708, 21380, 2072,  18416,
                33092, 22148, 47348, 63092, 19780, 54128, 3360,  46804, 18384,
                24572, 11548,
            },
            {
                4068,  61160, 7312,  20056, 62224, 15184, 6168,  55776, 19228,
                47864, 15040, 38648, 61016, 49304, 35972, 21556, 26848, 7356,
                38428, 59172, 42460, 1648,  34068, 12348, 25312, 51216, 45184,
                23868, 37148, 47280, 31280, 57792, 41880, 10328, 48348, 14168,
                33932, 20932, 29644, 11948, 63140, 43236, 55196, 3880,  62096,
                35560, 17560, 3044,  56580, 5512,  37244, 50280, 33680, 21840,
                42032, 12728, 58484, 9644,  3148,  28340, 57368, 52028, 48488,
                34932, 8988,  25932, 62044, 23552, 47828, 28632, 42480, 8760,
                61260, 15704, 27372, 2372,  64320, 16580, 46680, 59692, 54360,
                30380, 49016, 58512, 38392, 432,   63888, 34592, 26208, 59696,
                37248, 49396, 17068, 8756,  63532, 708,   40636, 16420, 29572,
                50168, 55912, 18196, 45360, 60504, 20168, 35720, 55052, 41652,
                3864,  58812, 47084, 1232,  25644, 11080, 49288, 59500, 55992,
                37656, 51336, 9564,  57540, 30604, 14396, 38944, 56840, 34572,
                48944, 42452,
            },
            {
                17136, 23400, 46252, 35440, 25856, 51600, 33000, 44416, 26568,
                59216, 640,   32540, 18704, 12240, 8960,  56632, 33612, 45268,
                23704, 29380, 9820,  47712, 55948, 30052, 62388, 39528, 436,
                13928, 9448,  19084, 26380, 15584, 34716, 22740, 18024, 62900,
                8888,  49772, 3116,  37044, 32256, 5836,  22572, 50420, 30044,
                58864, 46268, 27624, 63104, 15528, 46812, 18764, 7472,  56820,
                16308, 26916, 39388, 61708, 22320, 33928, 15596, 1044,  29860,
                17396, 54100, 3820,  32544, 52244, 17112, 58680, 34136, 21140,
                39920, 50844, 11160, 35732, 22272, 28816, 33324, 1068,  21648,
                26060, 62340, 11664, 20948, 28028, 50460, 4484,  11280, 20108,
                30360, 52880, 42708, 21716, 32808, 50772, 23756, 46336, 5604,
                61316, 2800,  38708, 9388,  53480, 48568, 1284,  16120, 24708,
                49732, 38088, 12736, 18636, 64256, 44372, 15660, 29284, 7600,
                62128, 26952, 16828, 35788, 6384,  60736, 22856, 7956,  62892,
                268,   38004,
            },
            {
                33120, 49984, 12568, 55996, 952,   39656, 22512, 63004, 12000,
                40304, 53676, 45788, 24836, 51856, 30208, 64468, 15332, 360,
                49848, 63508, 5208,  20768, 37692, 16124, 4668,  58028, 27976,
                52872, 60424, 40576, 49700, 2864,  55032, 59388, 832,   27568,
                38840, 57344, 24768, 52948, 59188, 14036, 37840, 16932, 10940,
                72,    22928, 40048, 9152,  53656, 24944, 61220, 29664, 44568,
                4820,  49052, 53308, 18412, 46144, 54640, 63868, 40592, 44884,
                22844, 57572, 37368, 41672, 11216, 2696,  45504, 13824, 54196,
                928,   31828, 58560, 45460, 55292, 42216, 50348, 9208,  37432,
                43988, 3992,  34244, 42548, 53144, 16956, 36464, 47884, 56596,
                5980,  24592, 3388,  60280, 13948, 37880, 10604, 53780, 26668,
                42504, 33312, 24480, 14420, 41104, 6072,  28524, 62016, 10144,
                31548, 7156,  60812, 27952, 51500, 35356, 3744,  24272, 43140,
                13672, 2704,  45216, 24476, 42184, 50072, 29200, 45128, 12892,
                26684, 54460,
            },
            {
                8772,  64432, 29608, 42780, 60040, 17952, 8288,  36460, 3872,
                29804, 21604, 7628,  58108, 2812,  38260, 42688, 19860, 53232,
                40368, 13944, 57632, 27160, 53864, 46404, 11032, 35396, 20640,
                43536, 33644, 5156,  62712, 12760, 44028, 30816, 51652, 45980,
                4528,  42328, 10404, 19208, 44908, 28468, 63780, 47716, 42308,
                54932, 31576, 50924, 20548, 34664, 41688, 1380,  11464, 38356,
                63660, 24200, 272,   31564, 13612, 5148,  25992, 10872, 59892,
                6844,  13496, 46624, 20160, 61032, 26464, 36308, 7264,  65372,
                43640, 25468, 19768, 8200,  13884, 4148,  60800, 18600, 64952,
                51948, 15768, 24292, 56708, 8380,  60332, 22936, 44796, 65056,
                15708, 40088, 35408, 47100, 27744, 56664, 4492,  64716, 19668,
                13080, 57480, 49356, 30540, 64356, 22464, 37884, 56408, 45564,
                19648, 53848, 42404, 22848, 5172,  57488, 31988, 54076, 39884,
                48704, 31236, 63804, 53164, 11752, 1420,  17612, 40404, 51864,
                20576, 58824,
            },
            {
                41032, 3124,  19576, 5588,  53016, 13964, 46920, 58320, 49672,
                15960, 65512, 41996, 35136, 17488, 60072, 6188,  25356, 10768,
                31076, 36168, 17436, 44008, 7720,  23140, 60784, 49384, 16780,
                8184,  56988, 29556, 21344, 38624, 25176, 7744,  36512, 60540,
                22292, 15264, 64640, 35132, 1084,  8272,  56716, 19800, 33676,
                7184,  14844, 64848, 4960,  13040, 56388, 31996, 51048, 58260,
                19844, 34972, 9164,  43120, 38056, 49904, 20028, 36356, 31740,
                50864, 27376, 64628, 5268,  30408, 50300, 56972, 22968, 17960,
                37552, 4644,  49284, 63152, 39092, 24000, 31168, 11920, 27476,
                6860,  46856, 39556, 2400,  31540, 13424, 40812, 1708,  32576,
                10276, 54148, 62372, 7260,  18080, 43932, 30740, 34872, 39988,
                8564,  45608, 404,   17680, 58252, 34584, 12560, 51056, 26608,
                2412,  33868, 15292, 48092, 39516, 8764,  17424, 60692, 10560,
                19640, 56880, 5740,  37440, 20840, 59504, 33724, 65516, 5288,
                31172, 14796,
            },
            {
                45476, 34752, 26944, 48740, 37352, 31956, 28104, 24372, 54648,
                33560, 10344, 26960, 50488, 13788, 47748, 28896, 54880, 61404,
                46328, 2492,  62300, 33352, 39636, 1828,  30392, 41224, 55152,
                2640,  24508, 47996, 10216, 54132, 65008, 16232, 19764, 11468,
                33248, 56124, 26780, 48664, 52088, 23564, 40620, 4176,  26240,
                59532, 45880, 36808, 43484, 27192, 48420, 22480, 8092,  14724,
                28100, 47072, 60752, 55740, 29212, 58640, 8128,  62696, 42584,
                2464,  18180, 34504, 55492, 14980, 40232, 10096, 47552, 29440,
                52376, 59796, 15880, 27964, 34476, 53812, 48464, 41640, 56032,
                32536, 58960, 19912, 63500, 48808, 25508, 55432, 18948, 28740,
                49132, 20980, 26132, 12416, 58208, 49892, 1924,  15208, 51268,
                22604, 62732, 27332, 52148, 7444,  43528, 3272,  16700, 40412,
                64720, 58520, 11144, 29660, 63108, 21496, 45796, 36792, 320,
                23168, 34396, 14452, 27628, 46632, 55480, 25240, 10184, 36956,
                48040, 24068,
            },
            {
                53464, 11288, 62812, 22984, 9476,  61580, 1520,  43628, 5844,
                19176, 46156, 1180,  55620, 23384, 3824,  43784, 34632, 8140,
                21564, 50588, 24528, 11868, 52804, 64496, 14660, 25768, 32468,
                63372, 37784, 17648, 45004, 34220, 5448,  41308, 48224, 28840,
                53344, 5312,  39184, 17420, 31688, 12996, 36100, 61756, 50284,
                11732, 23792, 18488, 57736, 3408,  61524, 17588, 36332, 54388,
                41336, 5632,  12472, 16864, 22608, 1848,  46720, 15228, 23548,
                53828, 39380, 9420,  43744, 21696, 3772,  33600, 62304, 2596,
                12520, 32332, 41080, 6132,  57532, 156,   16392, 21264, 2760,
                36172, 13980, 9604,  28592, 44364, 5784,  62076, 37704, 7920,
                59240, 34260, 224,   41560, 36688, 20376, 24888, 60796, 55684,
                6064,  32564, 37316, 10956, 21076, 47324, 61716, 32692, 8624,
                23484, 44540, 37228, 1628,  52936, 13244, 50392, 27000, 65080,
                52152, 43864, 60300, 8660,  39916, 3464,  15568, 43444, 57196,
                1676,  60768,
            },
            {
                17552, 38276, 56636, 15788, 40628, 55440, 20720, 12904, 39996,
                58836, 36868, 62088, 31100, 40420, 63852, 18808, 12808, 38008,
                58660, 4792,  29592, 56588, 18760, 48112, 6104,  44892, 10664,
                50388, 14124, 60648, 688,   27024, 51144, 58676, 1780,  61696,
                43352, 9192,  46584, 3444,  59964, 54160, 44452, 15996, 29824,
                39136, 1332,  53600, 30444, 10220, 39592, 46168, 64300, 2248,
                24616, 50412, 32560, 65272, 40272, 52772, 34332, 28276, 56992,
                11964, 47524, 60816, 25292, 53028, 59044, 16464, 42916, 25084,
                56348, 8964,  44828, 22588, 46892, 10652, 37664, 60108, 43872,
                25348, 50444, 54472, 38224, 17320, 34676, 12096, 50952, 43216,
                14268, 45696, 52060, 30248, 63568, 9336,  47624, 38464, 28596,
                43356, 14012, 59732, 41732, 55424, 25260, 29224, 57128, 19952,
                48676, 6148,  55504, 17768, 25376, 34808, 3616,  41748, 15800,
                29764, 4864,  49096, 18208, 32556, 50312, 29920, 52588, 19884,
                27904, 7792,
            },
            {
                49692, 29988, 2156,  44636, 6592,  50800, 34616, 63448, 30312,
                51540, 15352, 8008,  20428, 10980, 33224, 57224, 52176, 27072,
                16604, 41108, 45180, 8632,  35880, 22248, 38560, 59268, 19828,
                4208,  42552, 30172, 57248, 20080, 13584, 31716, 23036, 37280,
                14460, 24972, 63268, 20424, 27684, 9796,  2200,  21500, 55532,
                8300,  62312, 34448, 49188, 15648, 25052, 33212, 6932,  29600,
                58932, 44292, 20600, 4640,  25808, 11116, 43868, 18976, 4184,
                37092, 29184, 428,   32328, 7140,  46132, 28156, 38576, 20352,
                48656, 36388, 63756, 18120, 26656, 61800, 52752, 29324, 8780,
                63876, 4592,  23016, 59976, 584,   53316, 21916, 27060, 2908,
                64080, 23204, 6564,  16112, 55324, 4780,  33128, 11644, 17840,
                3340,  48672, 19204, 1728,  34112, 14904, 4128,  52332, 36072,
                13860, 27608, 31648, 61984, 42776, 59900, 7520,  32452, 54740,
                9972,  38016, 25012, 56012, 64032, 22536, 6372,  61768, 13044,
                33876, 42656,
            },
            {
                4476,  21148, 59464, 32688, 25708, 17988, 46500, 10112, 23348,
                3548,  25980, 44232, 53640, 49320, 5644,  24764, 692,   48432,
                65100, 32772, 13888, 60076, 27472, 236,   52192, 28940, 34568,
                54536, 22932, 36152, 52336, 9252,  38844, 45400, 7276,  18400,
                51524, 30848, 57648, 34620, 42316, 49436, 65368, 33028, 47076,
                41828, 27464, 20316, 6016,  42848, 59432, 52428, 12948, 18804,
                38044, 10104, 53912, 36564, 48316, 61744, 7204,  59352, 49712,
                64428, 20804, 14476, 49364, 36424, 12184, 64820, 1260,  53960,
                14652, 5368,  30536, 51320, 13488, 33060, 4252,  18848, 45856,
                15612, 31020, 42268, 11500, 32656, 46028, 58564, 40500, 31632,
                18336, 53976, 38760, 27672, 42736, 22412, 58632, 44872, 65420,
                54204, 24400, 29944, 63824, 50292, 38608, 10440, 41432, 63400,
                564,   50792, 40056, 9232,  47164, 22764, 56764, 19120, 46488,
                21832, 62100, 13516, 2764,  41480, 11552, 36232, 46004, 39620,
                23732, 64324,
            },
            {
                14696, 36284, 47808, 13092, 64608, 28760, 2504,  57008, 36080,
                48188, 59948, 29056, 38488, 61208, 15992, 45532, 39728, 7580,
                21180, 55236, 3584,  50264, 43124, 63244, 16208, 9124,  47572,
                64732, 12244, 6716,  41052, 62760, 24568, 49204, 64348, 56556,
                2572,  40976, 11876, 6876,  15300, 37832, 24332, 5344,  17720,
                57952, 13908, 51800, 63732, 36852, 548,   21636, 41092, 48996,
                62844, 15916, 872,   57764, 30008, 14112, 38740, 31848, 24296,
                8448,  40744, 56400, 61936, 18772, 23844, 50968, 8504,  34056,
                61112, 22024, 58388, 1624,  38832, 42844, 23820, 49488, 34612,
                57336, 39340, 48480, 19748, 65332, 8612,  15872, 5040,  56164,
                35828, 9904,  48536, 61268, 12876, 51364, 992,   26248, 35760,
                8980,  40336, 58204, 5776,  44704, 22568, 59992, 18320, 45196,
                24496, 58172, 20668, 15928, 4160,  29048, 11892, 39268, 59260,
                1300,  43344, 35104, 47292, 28288, 53840, 17296, 912,   58052,
                9320,  54912,
            },
            {
                51724, 24652, 41036, 9112,  52232, 38556, 42796, 14784, 53836,
                7320,  18556, 12460, 1664,  22008, 35680, 28172, 58956, 30760,
                10884, 36552, 17868, 24360, 31492, 12088, 56228, 39944, 2868,
                26600, 18500, 46552, 28648, 16456, 4596,  33488, 11164, 27648,
                36200, 46204, 22596, 50612, 54976, 29696, 59244, 11188, 36008,
                25424, 2336,  44724, 11596, 28584, 55228, 9324,  31468, 56260,
                23612, 27832, 33732, 42904, 18052, 22256, 1652,  55056, 45584,
                16904, 33760, 44440, 4496,  30064, 42244, 57360, 17116, 45516,
                40428, 28364, 43884, 10024, 55092, 64928, 7300,  59220, 11900,
                2184,  26444, 6544,  52180, 28040, 37408, 24320, 47468, 12644,
                59924, 25524, 2120,  33872, 19568, 30696, 39448, 15368, 50644,
                20796, 32168, 12232, 16692, 26856, 7992,  54120, 30832, 5252,
                34680, 11052, 54812, 37712, 64672, 52440, 35900, 49360, 14976,
                26752, 31232, 57652, 20260, 8216,  59832, 25616, 31412, 48868,
                19160, 28644,
            },
            {
                6140,  61812, 28,    55924, 20264, 5728,  59144, 32064, 21440,
                40108, 64896, 34196, 43272, 56836, 9572,  63624, 14004, 42748,
                53276, 46996, 61412, 39608, 6848,  20292, 33808, 23504, 43996,
                57516, 32632, 61388, 1720,  53544, 58448, 43304, 20452, 54404,
                16736, 60372, 3884,  62400, 18784, 748,   43468, 48400, 62784,
                53132, 39480, 32548, 23024, 47852, 17860, 61732, 45080, 3396,
                6108,  46288, 52344, 8920,  64144, 51040, 41664, 27304, 12492,
                52040, 2428,  26140, 53692, 10196, 37944, 5880,  31652, 25608,
                11608, 3076,  50012, 35508, 16240, 20248, 27756, 40788, 53948,
                22004, 62836, 17628, 56528, 1764,  44224, 61852, 51792, 29196,
                42052, 20836, 45496, 5712,  56904, 63164, 46700, 7632,  60524,
                3700,  46284, 52592, 37012, 56520, 33552, 47472, 13364, 27784,
                61332, 48056, 6652,  32204, 44392, 1904,  24556, 6096,  63528,
                9292,  52672, 4308,  16200, 38628, 50748, 42212, 62928, 4092,
                34684, 44164,
            },
            {
                12252, 31136, 45648, 34264, 15744, 24256, 62656, 11532, 27336,
                45952, 4372,  52356, 30584, 47696, 18032, 2552,  50044, 23496,
                5528,  26520, 1252,  57600, 45420, 53820, 48748, 62468, 5388,
                14468, 37888, 50224, 21996, 35808, 13340, 29864, 16,    40104,
                8480,  25384, 32480, 44308, 26652, 39348, 13780, 22236, 7676,
                28868, 4620,  15924, 59264, 7112,  34768, 26036, 39256, 15268,
                36128, 60336, 12888, 25584, 3968,  35280, 61076, 5848,  59088,
                36648, 63768, 22012, 13868, 60392, 48916, 21004, 59420, 52384,
                64188, 19036, 56540, 24948, 31796, 48500, 3788,  36804, 14744,
                32020, 46480, 36264, 30396, 14160, 21368, 33524, 3088,  16680,
                7660,  64688, 52616, 37256, 17040, 10292, 24344, 34044, 56028,
                41936, 27924, 63052, 20,    20084, 40760, 3540,  65116, 38340,
                16476, 42256, 23192, 27328, 12900, 60952, 17216, 33516, 45300,
                40608, 23020, 48468, 64824, 33388, 2528,  21932, 10448, 15504,
                56672, 39236,
            },
            {
                22732, 17800, 59872, 27680, 48832, 36820, 44984, 1404,  51012,
                58396, 16512, 23040, 7688,  26068, 54572, 38664, 32976, 19456,
                62876, 34848, 15672, 29228, 10316, 37112, 2020,  17496, 30508,
                51904, 10580, 7216,  42080, 26012, 60792, 47356, 51716, 34520,
                57440, 49048, 12984, 36700, 9308,  56984, 51260, 33920, 18424,
                55200, 45916, 64676, 50324, 41580, 2576,  53636, 11012, 65016,
                29884, 19024, 56968, 38124, 48876, 29504, 15616, 20052, 47352,
                9092,  30744, 39364, 46600, 27572, 792,   34628, 14280, 4224,
                47136, 37604, 6996,  13280, 62704, 45076, 9332,  60900, 51608,
                42936, 7608,  11000, 59904, 49792, 41484, 9468,  55892, 39384,
                48852, 31252, 12056, 27276, 40872, 53552, 2296,  29316, 18768,
                10916, 23404, 14684, 48736, 60960, 9976,  24412, 50248, 21084,
                1296,  52608, 59460, 18844, 50580, 40172, 56488, 29352, 54116,
                18960, 37172, 11852, 28936, 44344, 13892, 55340, 37088, 46688,
                26212, 64004,
            },
            {
                8096,  49940, 40528, 10132, 3696,  53372, 29780, 18752, 9216,
                32804, 37844, 61992, 13232, 41456, 60452, 4912,  45924, 56104,
                12016, 42100, 50452, 21876, 64840, 14028, 27012, 58264, 41340,
                24940, 20156, 45644, 56728, 17660, 9944,  5144,  22916, 14760,
                64904, 5828,  21012, 53520, 63720, 30776, 2708,  61544, 41136,
                10588, 36504, 20744, 30532, 13660, 57692, 19792, 50876, 23232,
                48260, 41260, 6736,  21824, 44396, 10208, 56004, 33180, 23964,
                42668, 17332, 6296,  62672, 18608, 43084, 55612, 39776, 29160,
                22960, 32944, 60088, 41944, 400,   54400, 29408, 23260, 18536,
                1144,  24952, 54588, 38952, 4108,  64008, 26940, 19388, 35596,
                23580, 58784, 412,   44508, 22348, 60092, 48700, 43172, 64344,
                38244, 4556,  34700, 43480, 30124, 17272, 57836, 44112, 32652,
                56436, 9088,  34880, 3172,  46568, 8152,  21788, 11024, 24,
                62464, 5480,  55836, 59276, 7132,  23808, 61304, 30572, 20228,
                5088,  52712,
            },
            {
                36684, 1712,  57168, 21724, 64860, 13708, 39512, 56220, 42320,
                25168, 48564, 604,   53148, 35872, 21144, 15240, 28580, 8768,
                37768, 59828, 31396, 7644,  52556, 43720, 33308, 6396,  60980,
                54308, 34268, 65532, 3784,  31920, 39392, 62916, 44020, 27104,
                31080, 41728, 46560, 17276, 4340,  23720, 15532, 47208, 26480,
                58048, 368,   24916, 8584,  37976, 27620, 43196, 33520, 8268,
                2196,  54188, 32584, 16212, 62628, 3280,  39900, 52432, 48,
                57892, 49928, 54684, 11524, 31556, 51512, 7636,  63380, 10504,
                44264, 16372, 52700, 27204, 21372, 39060, 12136, 57476, 33456,
                64548, 48612, 28588, 17156, 22868, 45120, 12424, 52296, 61416,
                5992,  14828, 55112, 33512, 8052,  13968, 36048, 5572,  16032,
                54228, 50272, 59060, 7476,  26472, 53404, 35812, 5824,  11804,
                28788, 39536, 14716, 63472, 30372, 37072, 58700, 42900, 49428,
                24700, 32324, 17012, 41128, 26584, 48108, 51580, 632,   43152,
                59184, 33204,
            },
            {
                29236, 14524, 25816, 43572, 32428, 7228,  60804, 22356, 5264,
                64156, 15124, 29148, 43656, 10468, 31752, 65528, 51312, 24804,
                3392,  17056, 47236, 4328,  24220, 39168, 18984, 46304, 12364,
                504,   38836, 15320, 27556, 49580, 53084, 36676, 18904, 55772,
                10816, 1292,  60844, 28076, 49936, 43144, 38208, 6800,  32356,
                13152, 43844, 51908, 61296, 47612, 5908,  63204, 16728, 58900,
                44644, 11864, 27480, 58304, 50552, 25072, 12276, 65524, 28536,
                13960, 34596, 24928, 2688,  36724, 22268, 15936, 26292, 49852,
                57988, 3404,  8648,  47788, 17844, 34960, 49884, 5700,  44180,
                40008, 13112, 58828, 35152, 6464,  30800, 57324, 1500,  43424,
                28312, 47376, 19220, 63344, 50836, 30224, 20660, 57352, 25364,
                44944, 31436, 21308, 13300, 38896, 1548,  63860, 15324, 46900,
                60196, 24188, 43612, 54616, 25628, 5044,  15784, 34552, 27860,
                13808, 51952, 46440, 3460,  35664, 18356, 8836,  39496, 12916,
                17224, 47576,
            },
            {
                41636, 63084, 54504, 18184, 51164, 35476, 26596, 45488, 12080,
                34020, 57856, 20152, 6712,  59484, 48796, 2108,  40400, 58044,
                44004, 20312, 54780, 36212, 61536, 57020, 9484,  50668, 23080,
                28892, 48840, 7996,  59860, 22660, 2152,  13620, 7196,  48124,
                35648, 24096, 39456, 33064, 11740, 55676, 59940, 21224, 53592,
                63608, 19252, 34588, 16140, 56232, 22312, 1048,  29436, 37120,
                24740, 62024, 39108, 6008,  34936, 18492, 46324, 37124, 20904,
                5460,  61928, 45056, 40748, 59684, 56684, 45868, 1600,  19732,
                30960, 36252, 40852, 65520, 55652, 4164,  62352, 15764, 26728,
                9024,  20864, 3132,  51564, 62268, 47952, 37652, 16404, 32840,
                10836, 41100, 36612, 25216, 4424,  40188, 61472, 11668, 28512,
                8680,  2028,  62848, 55360, 46264, 23096, 41680, 31736, 51756,
                19164, 6532,  48984, 11220, 20484, 50932, 65220, 45672, 7456,
                60148, 36652, 21416, 10912, 62828, 54216, 32436, 65408, 27696,
                53596, 9584,
            },
            {
                21128, 2900,  37752, 11416, 4712,  47904, 15884, 54744, 2508,
                51280, 38800, 46052, 54468, 27032, 17504, 22508, 13012, 34324,
                29840, 63772, 10804, 27244, 14768, 2356,  30688, 63968, 37264,
                17044, 56524, 42500, 11612, 45008, 29748, 58352, 21612, 62260,
                51452, 15284, 58772, 5180,  19544, 35360, 9140,  29652, 48892,
                5248,  39800, 2948,  31392, 44700, 12360, 40776, 49308, 52828,
                14576, 19704, 47860, 1268,  42860, 30140, 55156, 8176,  42048,
                48912, 30368, 19188, 9340,  27916, 13284, 33072, 38408, 60204,
                53932, 22560, 14100, 29552, 10844, 24548, 31360, 46076, 52844,
                36632, 56160, 32180, 42616, 14788, 9128,  20412, 25648, 49844,
                65240, 2700,  57552, 16884, 45232, 54792, 1012,  46848, 52236,
                36756, 40972, 18924, 33272, 10308, 50892, 18220, 3252,  27056,
                62748, 36716, 732,   33116, 60724, 38916, 3344,  19324, 54776,
                40240, 1680,  56340, 30232, 42712, 15108, 21764, 45720, 4284,
                24388, 60256,
            },
            {
                6796,  31912, 45792, 61760, 29680, 20772, 59768, 37056, 30792,
                18456, 8580,  23948, 3596,  32724, 38036, 55716, 46372, 7120,
                53040, 212,   40156, 32864, 49044, 42232, 21364, 53556, 3728,
                32800, 61988, 25320, 18588, 35240, 54944, 40824, 33384, 2828,
                42740, 8324,  54016, 44312, 64760, 26200, 2060,  45524, 15808,
                23904, 57256, 27564, 9860,  53336, 25704, 60160, 34240, 3988,
                8956,  32148, 54764, 22240, 63900, 14880, 59836, 3060,  23628,
                58368, 15644, 53952, 4188,  64852, 49508, 5076,  42664, 12256,
                6000,  46260, 61004, 1320,  43276, 38232, 59108, 21924, 1480,
                60672, 17908, 46640, 23652, 27632, 40224, 60496, 55636, 7004,
                22908, 52652, 29788, 8904,  12896, 35268, 32096, 17432, 64816,
                23508, 14532, 48620, 60744, 4112,  28752, 59248, 55792, 8840,
                40928, 22104, 57236, 16452, 42380, 27208, 13036, 31036, 23712,
                9728,  61752, 26912, 48696, 6436,  58844, 2044,  38120, 56544,
                34724, 49516,
            },
            {
                16428, 57716, 23032, 52484, 40948, 1184,  9988,  43024, 63640,
                27828, 41352, 61068, 12480, 63224, 42608, 9412,  60096, 26704,
                16180, 23344, 45624, 59436, 18100, 7724,  25944, 44904, 13664,
                40940, 6284,  47332, 1208,  64252, 9372,  16260, 46732, 25500,
                29024, 18572, 31176, 22644, 13520, 50976, 40772, 61788, 11420,
                36556, 42920, 50476, 65388, 37372, 17912, 7540,  20756, 64380,
                45484, 57456, 28548, 7148,  51572, 10116, 26832, 32912, 51900,
                10596, 39188, 35644, 47196, 23860, 20356, 53188, 28876, 63840,
                18136, 34160, 26124, 52016, 19700, 49116, 6684,  13592, 41204,
                29768, 7200,  11068, 63912, 100,   52488, 4784,  34496, 45588,
                13648, 38936, 20732, 62872, 50196, 26624, 22032, 7032,  42704,
                58664, 30504, 5840,  24896, 43280, 37760, 12160, 33940, 45208,
                13716, 53396, 28920, 46448, 7268,  52228, 57728, 48204, 34124,
                16548, 44768, 19688, 12784, 34876, 25112, 50276, 29348, 18896,
                10880, 43708,
            },
            {
                27284, 36104, 8544,  13836, 26344, 56828, 50132, 23172, 5680,
                14320, 52824, 48196, 35352, 16356, 51460, 1532,  19856, 35888,
                62256, 50620, 5272,  12564, 52480, 35580, 56056, 60584, 9760,
                50096, 20712, 30728, 38388, 52020, 27920, 5096,  59332, 12120,
                63980, 38424, 49220, 308,   57860, 37332, 17236, 53032, 32156,
                59348, 8100,  21544, 14392, 3556,  46520, 55328, 27236, 38172,
                11308, 41824, 17532, 34080, 40228, 47328, 37492, 16748, 44460,
                63016, 26420, 1704,  31580, 41488, 16152, 36012, 9792,  24608,
                50916, 40472, 8452,  57808, 35552, 16592, 54948, 33196, 64804,
                24624, 49968, 38720, 54256, 33404, 12872, 29132, 42020, 17468,
                58444, 31900, 47872, 5856,  42092, 56496, 61132, 38620, 10728,
                2784,  51084, 56644, 35328, 16516, 65396, 20436, 48140, 25268,
                4840,  32372, 64512, 10192, 24452, 35912, 2220,  63056, 41904,
                51436, 4448,  37908, 63596, 52808, 41044, 8500,  61680, 13816,
                40068, 63988,
            },
            {
                260,   47756, 19476, 38508, 65176, 31724, 17360, 34488, 46468,
                58164, 352,   21260, 29264, 6516,  25076, 30968, 48428, 41216,
                10352, 31636, 38296, 20708, 65000, 1168,  31500, 16564, 28152,
                36112, 54448, 58840, 14636, 23428, 49296, 19936, 37396, 52840,
                3828,  56692, 10600, 45912, 33412, 7400,  28256, 4292,  25116,
                19012, 1220,  47652, 30384, 57948, 32812, 42416, 15412, 50212,
                23076, 2656,  62692, 13360, 24264, 61572, 572,   56080, 21028,
                5780,  13536, 60408, 55216, 7412,  62412, 57964, 47568, 248,
                56392, 15416, 30768, 2816,  61552, 28328, 9368,  47032, 3428,
                43924, 15176, 57832, 21540, 45088, 19088, 48836, 62676, 10072,
                27116, 1100,  60192, 37300, 18344, 4004,  14108, 53516, 47596,
                19528, 27516, 12684, 45368, 7856,  53764, 364,   39564, 61364,
                17568, 50036, 1760,  38444, 59100, 18372, 21816, 11560, 6764,
                28456, 57144, 32424, 488,   22788, 16992, 47160, 55028, 33300,
                4808,  51872,
            },
            {
                30928, 60612, 54096, 3004,  44960, 7308,  12368, 55252, 37940,
                25752, 9780,  39524, 55048, 58732, 45464, 64024, 13628, 57076,
                3292,  44484, 58136, 28948, 24384, 47412, 39304, 5492,  23028,
                62620, 2920,  11328, 43976, 7012,  61828, 32128, 44532, 23052,
                34856, 16720, 41476, 24516, 60836, 20208, 48284, 63856, 44512,
                55496, 35120, 62552, 40952, 23408, 5988,  63304, 176,   30000,
                59316, 52392, 36592, 45072, 4812,  19472, 49920, 30644, 41056,
                34904, 48600, 18860, 44924, 12148, 26984, 3800,  31976, 44376,
                36832, 22264, 48068, 12352, 44488, 24064, 39064, 20656, 56816,
                35012, 27084, 5532,  37076, 2620,  58804, 24144, 36224, 3852,
                53528, 43788, 23728, 11064, 28800, 33716, 44760, 23884, 31116,
                35992, 63424, 39968, 21644, 31880, 49400, 28012, 6004,  56592,
                35996, 21476, 42572, 14896, 54732, 30268, 45724, 39444, 53964,
                24968, 14284, 46360, 59548, 10960, 31408, 3232,  26488, 20284,
                57380, 23592,
            },
            {
                15856, 11880, 41572, 33892, 24348, 49108, 61960, 3332,  19676,
                60528, 33052, 43968, 17852, 11436, 36748, 4368,  21804, 27948,
                18200, 54140, 7508,  15544, 43008, 10968, 51372, 57500, 41968,
                45896, 18272, 25832, 33968, 55444, 41236, 2584,  13544, 48044,
                8028,  62740, 27852, 4564,  53616, 14432, 41912, 9884,  38164,
                13160, 50896, 26648, 10296, 18472, 52696, 12944, 35420, 47264,
                18828, 6696,  25900, 54452, 57960, 28420, 12324, 8372,  64756,
                24900, 52572, 29192, 37420, 22360, 51576, 39912, 19880, 11504,
                59204, 4624,  63252, 41240, 53412, 5112,  62688, 50304, 11756,
                18204, 61240, 9668,  30640, 63464, 14864, 7420,  55748, 40468,
                20064, 34000, 15036, 52172, 63844, 49260, 240,   57932, 8552,
                15904, 54560, 1472,  60416, 10544, 58616, 23352, 15384, 43916,
                30100, 11016, 60004, 27068, 49196, 5436,  60532, 32960, 17524,
                64708, 8824,  20040, 39016, 54056, 44176, 64336, 37416, 42680,
                7664,  45388,
            },
            {
                51064, 28932, 6256,  59308, 18088, 28504, 39756, 43440, 30184,
                15104, 51052, 64796, 2376,  23648, 32140, 53728, 39324, 49664,
                34984, 25380, 63272, 37288, 3572,  60968, 20096, 13972, 32932,
                8392,  52848, 39648, 64856, 28704, 10020, 17728, 60824, 29936,
                54964, 19720, 50404, 11968, 36656, 30432, 58872, 23152, 3488,
                29672, 16808, 6580,  54796, 37684, 44564, 25028, 61100, 9452,
                40524, 31936, 15776, 9940,  33184, 42368, 60544, 45900, 15436,
                4616,  58128, 9116,  1200,  56864, 33620, 14840, 60924, 29452,
                52900, 17400, 27228, 33492, 19248, 14444, 32108, 528,   29668,
                53796, 40520, 47448, 52216, 42716, 26348, 32420, 50676, 12216,
                64560, 45740, 56352, 8044,  40084, 20504, 26340, 38236, 62196,
                42064, 5280,  25844, 47232, 18140, 41568, 33652, 51272, 7576,
                62472, 53100, 3264,  34452, 8908,  43360, 13460, 956,   47888,
                36244, 42756, 50688, 29140, 25408, 6676,  14492, 49952, 10616,
                61184, 34624,
            },
            {
                38328, 63180, 22000, 55628, 14384, 52640, 9916,  22564, 54272,
                4676,  8064,  27040, 48016, 40916, 61092, 6924,  16204, 59772,
                636,   12272, 48200, 30820, 55344, 34140, 26864, 1576,  63784,
                29356, 48376, 15600, 124,   20584, 51324, 37960, 57468, 26148,
                1792,  39696, 32472, 44144, 65152, 1088,  51148, 33988, 56956,
                47180, 61028, 43052, 33752, 59428, 3176,  29092, 51468, 21776,
                56356, 44036, 65228, 48296, 21324, 2068,  35804, 53628, 22228,
                32704, 40196, 16860, 43072, 64476, 49120, 5464,  42200, 23488,
                7696,  38824, 50752, 9600,  56516, 37160, 46188, 59540, 42964,
                24240, 4396,  16572, 22284, 11356, 38592, 46232, 17796, 29776,
                5168,  26032, 2360,  30788, 16412, 59712, 6208,  12632, 50088,
                22540, 33468, 52328, 29340, 37324, 2556,  63864, 12668, 38504,
                24600, 46048, 19304, 40884, 22816, 64012, 26044, 55732, 29460,
                22556, 5832,  2416,  62144, 17764, 35500, 59208, 22120, 28084,
                17940, 2836,
            },
            {
                25484, 9512,  43148, 3712,  37292, 46196, 884,   63060, 35764,
                46924, 38412, 20680, 57432, 13116, 50804, 9528,  44676, 30388,
                42408, 52524, 22112, 9176,  45424, 17256, 49372, 38268, 54940,
                21516, 6180,  36784, 59552, 46088, 24244, 43412, 5400,  34416,
                46476, 14936, 60248, 6356,  21960, 16912, 26868, 8088,  19364,
                39520, 22696, 756,   14732, 20644, 46472, 7388,  38904, 14608,
                1484,  28184, 4500,  38372, 51796, 17832, 26028, 39084, 3224,
                59608, 47832, 11728, 31060, 25288, 18688, 10084, 47036, 63820,
                34868, 45100, 2936,  24440, 65168, 6608,  26800, 21288, 8688,
                36752, 64504, 33836, 57292, 6304,  61916, 1008,  56876, 22476,
                38012, 49648, 61700, 35944, 47244, 42772, 52896, 29464, 18568,
                45684, 11360, 64728, 14664, 6548,  55944, 44716, 20192, 29104,
                4756,  16024, 56268, 31192, 51428, 17124, 37780, 50368, 10444,
                60864, 57240, 33636, 12716, 47476, 56456, 1096,  32604, 40512,
                54488, 47396,
            },
            {
                57444, 19144, 32888, 50040, 26424, 31496, 58348, 16272, 24876,
                12296, 55736, 31152, 16620, 35244, 28608, 20104, 24816, 65192,
                5888,  19016, 39812, 61848, 5276,  24052, 58800, 10060, 44192,
                13048, 25136, 56740, 31776, 4440,  14456, 62000, 11456, 21552,
                52916, 9068,  25160, 56616, 47840, 41076, 55212, 44804, 63160,
                11604, 53064, 27704, 64972, 31812, 56856, 17576, 62800, 34132,
                48948, 58908, 24192, 11972, 61276, 7912,  56300, 13668, 50696,
                27724, 19180, 62112, 52812, 3664,  35844, 55500, 26768, 836,
                54296, 15712, 60456, 30464, 42340, 48216, 16868, 55648, 13380,
                50648, 1988,  28016, 49160, 19728, 53648, 34604, 9284,  60440,
                43056, 13880, 19288, 10376, 23240, 3068,  32592, 56472, 36744,
                3448,  57796, 43212, 20876, 49524, 25544, 9172,  52788, 58496,
                48560, 35584, 63336, 12020, 1820,  58768, 4392,  20472, 45152,
                14912, 39360, 53256, 23448, 41120, 9636,  45112, 51444, 5028,
                65084, 13024,
            },
            {
                680,   45552, 60068, 13580, 64068, 7536,  20304, 33912, 51296,
                41008, 61764, 1608,  43020, 6504,  62532, 46860, 56076, 36448,
                14888, 33452, 57208, 28640, 14260, 36056, 41248, 3812,  30580,
                60852, 50160, 40764, 9516,  53420, 35556, 48512, 30684, 41184,
                64384, 18368, 37696, 29548, 10948, 35392, 2792,  15044, 31148,
                5332,  36196, 48764, 8524,  40440, 50484, 11404, 25812, 53808,
                6344,  19196, 41660, 35236, 28944, 45304, 31692, 64072, 6492,
                44032, 36996, 23196, 7716,  45976, 59756, 39288, 13272, 32404,
                20016, 10784, 49088, 21576, 12580, 1496,  39504, 62052, 32768,
                44660, 59132, 14600, 39216, 44044, 23988, 28784, 16028, 51844,
                6160,  32024, 53756, 57584, 27468, 65164, 14640, 9480,  61584,
                25192, 7816,  39000, 30512, 60324, 35672, 16876, 32448, 41060,
                2256,  26436, 8420,  39072, 47008, 28400, 34780, 42112, 6624,
                31420, 18964, 27288, 4172,  63364, 30672, 19900, 24976, 16128,
                36416, 30448,
            },
            {
                52052, 39468, 5664,  22840, 41328, 11276, 56020, 44688, 6036,
                27740, 9460,  22704, 59412, 52976, 3424,  38752, 11764, 2288,
                54376, 26024, 47004, 1464,  51120, 63176, 20540, 52924, 15980,
                35084, 2588,  17984, 28220, 63708, 19624, 25876, 7376,  56952,
                768,   49172, 43000, 3932,  59636, 20000, 50236, 38540, 60228,
                25620, 57904, 43896, 13476, 24380, 4180,  36836, 42928, 29856,
                46040, 13732, 63412, 54364, 476,   16264, 40480, 21704, 10800,
                34012, 880,   56732, 40896, 28664, 16084, 22100, 49620, 58060,
                28772, 40680, 36444, 54896, 59156, 34424, 52456, 28508, 5356,
                19192, 24840, 9120,  31868, 4828,  12816, 64968, 36540, 47628,
                25148, 40332, 568,   44604, 38204, 49080, 21208, 40968, 47080,
                28460, 16540, 53652, 2032,  12172, 46824, 5380,  62808, 22280,
                14056, 60476, 43312, 23436, 54396, 15328, 61672, 24588, 52368,
                65252, 46348, 8404,  49800, 37460, 14024, 54920, 61728, 7888,
                42852, 21488,
            },
            {
                26228, 15560, 35036, 53688, 28348, 48408, 37476, 2780,  65356,
                18484, 49616, 33348, 45536, 26056, 18256, 32640, 22836, 49868,
                43948, 8068,  59392, 11620, 31980, 43704, 7440,  27136, 46896,
                65136, 22964, 45432, 12384, 42644, 2052,  59644, 16496, 36616,
                27308, 12648, 32252, 52128, 23932, 63636, 28056, 8676,  46340,
                17032, 2384,  19644, 29412, 52580, 63480, 21336, 2128,  57992,
                9712,  22800, 32796, 5488,  49812, 58224, 25124, 46756, 61128,
                54584, 15252, 49424, 12092, 33240, 62664, 2100,  8084,  43408,
                61400, 6292,  3676,  26268, 17732, 7756,  23600, 11260, 47484,
                55064, 41848, 63088, 51780, 57820, 46384, 41304, 2324,  58596,
                11600, 21340, 63212, 16636, 8508,  4880,  33592, 55284, 1196,
                51224, 35180, 62668, 23680, 40284, 57300, 27048, 44272, 10336,
                55164, 33192, 51712, 19904, 5812,  32008, 9264,  56808, 12960,
                216,   35564, 59944, 17108, 43700, 1840,  34388, 27796, 48632,
                59788, 10864,
            },
            {
                56848, 46708, 62348, 1980,  17624, 60808, 24156, 14700, 30164,
                54904, 36844, 12024, 15256, 39644, 7712,  56936, 63632, 29492,
                17380, 40724, 21088, 37860, 24872, 17872, 56248, 33396, 10896,
                38808, 6648,  52184, 57172, 32780, 50052, 39096, 54680, 46044,
                22488, 62448, 55900, 16628, 5556,  13696, 33704, 54464, 21784,
                41884, 49832, 35068, 60428, 6444,  45296, 32676, 15424, 61508,
                39872, 51936, 27180, 43464, 20636, 9356,  35700, 2804,  18428,
                30288, 26620, 64524, 20736, 5084,  44500, 51424, 35060, 23464,
                16804, 46880, 63328, 32608, 45044, 41676, 64392, 37876, 2524,
                30040, 35948, 17268, 184,   27312, 19308, 8320,  32896, 17692,
                27892, 55456, 34956, 29784, 52280, 60928, 17888, 24116, 12824,
                44324, 19672, 9748,  32152, 49872, 18680, 3384,  37728, 29868,
                49348, 17544, 612,   36672, 63680, 44752, 49592, 37644, 21712,
                41188, 25472, 54444, 29496, 22656, 57464, 11380, 40176, 5404,
                53380, 32320,
            },
            {
                19600, 8492,  29704, 42980, 6964,  32552, 52032, 43348, 21244,
                47200, 380,   57620, 62840, 29168, 51772, 42276, 13988, 4692,
                61200, 34320, 53212, 64596, 4060,  48816, 60400, 2160,  54088,
                19744, 58856, 25672, 37252, 8644,  21616, 4992,  13420, 31372,
                8828,  3112,  35096, 46888, 39212, 43616, 57668, 76,    12340,
                64516, 31664, 9936,  56108, 39180, 18388, 55076, 49008, 35640,
                7404,  17904, 56812, 38128, 14644, 60236, 48540, 53304, 38500,
                42428, 8636,  46300, 37276, 57280, 18292, 26328, 55288, 11976,
                38096, 53356, 10032, 51068, 13748, 58004, 20216, 49944, 15360,
                60788, 21936, 11884, 48192, 38064, 22860, 54356, 50260, 61292,
                44896, 3912,  49392, 13484, 25576, 42924, 36272, 58420, 30784,
                64456, 5108,  56000, 42884, 14860, 60764, 53116, 20728, 65304,
                6756,  40384, 28148, 57400, 13504, 26096, 18392, 3200,  30420,
                48228, 15488, 10488, 5636,  50808, 64508, 46820, 17756, 23372,
                37268, 3048,
            },
            {
                64764, 38928, 50108, 21560, 58532, 10400, 39408, 4220,  59760,
                8736,  41296, 24304, 5116,  47964, 21424, 10224, 36660, 48496,
                24216, 80,    9816,  15652, 28908, 39196, 13436, 23640, 42140,
                29252, 14212, 1272,  30544, 16168, 63240, 24712, 43460, 60860,
                51244, 41620, 20532, 26168, 7844,  30120, 48828, 24784, 37572,
                52076, 6824,  22952, 14780, 27016, 600,   11748, 23752, 28316,
                3364,  47152, 11124, 64920, 4136,  30932, 27428, 12284, 5232,
                59404, 23320, 1964,  52564, 31244, 9624,  41556, 65280, 31708,
                2612,  28204, 19152, 24236, 452,   30720, 5656,  26872, 56360,
                44316, 6552,  53340, 58924, 33640, 63536, 14692, 30176, 6828,
                37616, 9960,  41428, 58144, 1656,  46944, 10584, 6640,  39668,
                48172, 22340, 37328, 116,   29084, 7612,  34192, 12596, 46036,
                24596, 59376, 9664,  48184, 4268,  41892, 59852, 53504, 7568,
                58112, 63552, 43040, 32928, 38240, 20172, 3352,  31296, 61324,
                44172, 14064,
            },
            {
                34120, 24640, 12168, 55336, 36296, 26400, 63644, 13564, 34732,
                26992, 31844, 52728, 19312, 35188, 1944,  59972, 27196, 45276,
                57912, 31304, 50340, 42352, 57152, 46100, 8628,  36324, 50504,
                62860, 44280, 48972, 60580, 41464, 46952, 53384, 34364, 18820,
                28464, 11912, 58460, 65348, 52568, 19260, 62108, 4240,  16164,
                28324, 40588, 61864, 43668, 47928, 30940, 59296, 42540, 52960,
                62652, 32036, 25700, 51192, 34756, 42196, 22876, 63656, 16192,
                50616, 35252, 13448, 61784, 24504, 14956, 48812, 6428,  20828,
                56928, 43228, 60364, 39952, 36368, 54580, 46452, 34556, 9084,
                39692, 32224, 25596, 4012,  40820, 10372, 2848,  42732, 24924,
                19740, 64276, 23220, 32664, 18912, 62700, 28344, 52944, 16044,
                26744, 11720, 51824, 62240, 25432, 48372, 39376, 56804, 2532,
                35576, 14512, 43676, 30888, 22244, 33500, 10700, 39492, 27496,
                34964, 19132, 1436,  55800, 26492, 13132, 42068, 53984, 9200,
                27320, 51384,
            },
            {
                57604, 5868,  45620, 16292, 1488,  48244, 19212, 44288, 57040,
                16656, 62092, 10840, 44556, 64908, 30916, 54280, 15852, 6212,
                19584, 12532, 35924, 26164, 5652,  20884, 62080, 31200, 16820,
                5036,  10008, 34252, 19460, 3656,  27776, 10776, 340,   56800,
                6392,  49136, 37068, 1348,  15340, 10888, 41312, 35264, 45840,
                58404, 18692, 53816, 4584,  36236, 64084, 8388,  37916, 19380,
                13304, 44152, 21628, 1288,  17104, 55008, 7692,  45240, 32776,
                55700, 19244, 44040, 40132, 3716,  58548, 28976, 38976, 46352,
                14188, 33972, 8000,  49256, 63948, 11760, 17528, 61692, 23064,
                13812, 65404, 45356, 16716, 28736, 47132, 60716, 34740, 57212,
                46620, 53052, 14940, 7504,  39152, 54980, 21792, 4000,  60208,
                42164, 33756, 57812, 16960, 44528, 10684, 21948, 30780, 17196,
                61432, 52560, 20252, 56188, 64656, 50708, 15940, 61804, 45352,
                13316, 51848, 23940, 46228, 8432,  62328, 35108, 58636, 15900,
                47792, 576,
            },
        },
        {
            {
                54560, 61564, 50068, 16880, 21040, 65292, 31528, 40648, 22276,
                5644,  50484, 64284, 1292,  57064, 32320, 63004, 25852, 46452,
                16268, 8776,  24928, 45028, 61012, 38716, 52412, 24252, 57452,
                41888, 27132, 20288, 61428, 53604, 47364, 27612, 23524, 34028,
                46328, 1888,  27024, 63876, 3536,  51448, 29324, 37056, 22268,
                26784, 8552,  63724, 21228, 28484, 904,   32916, 20224, 6648,
                15692, 62412, 46704, 35744, 43200, 49356, 7500,  55416, 44520,
                4888,  50988, 2644,  59968, 16284, 37552, 2056,  57724, 18032,
                40148, 15088, 1772,  45960, 61864, 11404, 16068, 1128,  40628,
                5924,  31516, 46256, 56468, 63304, 35864, 6452,  53024, 23660,
                59424, 49296, 26448, 57432, 45920, 19716, 61424, 54384, 26420,
                30868, 36960, 10124, 58868, 45908, 12256, 32000, 7688,  46976,
                42424, 19288, 38416, 12076, 28820, 6364,  22704, 52652, 18852,
                34768, 50132, 2264,  18248, 56428, 26512, 4728,  32952, 57032,
                3388,  18076,
            },
            {
                47520, 10912, 29068, 41744, 45412, 52536, 24816, 3104,  58508,
                54748, 13104, 39068, 47948, 43216, 14496, 51988, 7212,  20008,
                53928, 37672, 50820, 340,   18024, 58552, 28272, 13388, 8096,
                64824, 49548, 536,   43980, 36976, 16560, 42144, 10300, 38964,
                17992, 61200, 31488, 40984, 20144, 34340, 58108, 13760, 152,
                42368, 33372, 38656, 45876, 52936, 40536, 54796, 26116, 59624,
                29348, 33664, 2980,  20644, 63868, 25444, 14832, 59552, 36352,
                29688, 22368, 26352, 8644,  45620, 53920, 24120, 42612, 10352,
                53188, 29112, 25744, 49952, 41096, 29792, 53336, 44592, 22384,
                65200, 8924,  15624, 41308, 22052, 27212, 57684, 39392, 9084,
                19504, 4312,  12932, 52488, 1032,  29072, 35076, 39784, 15632,
                7240,  62680, 41572, 27396, 1748,  53276, 49800, 24052, 3620,
                29852, 52388, 8476,  65492, 54876, 16364, 37100, 62884, 9668,
                45388, 25644, 11808, 29888, 44152, 49368, 23036, 40380, 50968,
                30732, 24196,
            },
            {
                37872, 948,   22632, 60224, 4500,  8168,  34152, 48552, 16200,
                36164, 26196, 18504, 28996, 9064,  22456, 37096, 11572, 30980,
                41288, 61492, 13796, 33248, 22780, 9536,  36276, 44628, 31060,
                15648, 33380, 23684, 58556, 12936, 4932,  63024, 57540, 52232,
                13280, 43276, 5352,  11824, 54248, 25328, 7096,  65504, 47512,
                55912, 12088, 4848,  16360, 24164, 13440, 47000, 3548,  38408,
                44120, 23916, 13692, 56668, 30972, 4384,  51636, 19860, 10432,
                47040, 54196, 61804, 42100, 30856, 63332, 13328, 33444, 60888,
                44968, 6368,  57296, 9428,  18508, 35488, 4220,  57400, 37480,
                50516, 33524, 54552, 940,   12184, 49500, 3024,  61164, 29004,
                44476, 55004, 31032, 38792, 63172, 24424, 5576,  49864, 57656,
                20792, 52416, 17180, 33680, 22080, 64492, 18092, 37244, 58580,
                15208, 35172, 49156, 21468, 43448, 3540,  47928, 30224, 852,
                56884, 38296, 53820, 63376, 33776, 3044,  16796, 61908, 7032,
                13048, 64084,
            },
            {
                58084, 15448, 31936, 35904, 13208, 56928, 19248, 43496, 11004,
                63160, 41928, 6476,  59972, 34900, 62012, 48616, 56492, 44184,
                2500,  27648, 6504,  48072, 40692, 64052, 50340, 5024,  21484,
                46644, 53040, 6960,  35228, 26240, 50856, 21512, 1024,  29292,
                24492, 59128, 48804, 37732, 60720, 45032, 18012, 39868, 30964,
                19572, 60132, 50904, 61972, 31520, 9272,  64692, 18184, 49812,
                10956, 60840, 51320, 41620, 8960,  38492, 61256, 27184, 41192,
                6332,  15888, 35124, 11988, 408,   19296, 27652, 51224, 3204,
                20316, 38348, 64944, 32692, 22040, 61068, 14204, 26872, 19348,
                11204, 25780, 60196, 36848, 43892, 19920, 32976, 47676, 11752,
                35268, 65496, 22260, 15484, 43716, 9860,  59468, 13140, 32136,
                192,   45040, 60668, 14000, 39092, 43084, 28604, 10540, 44404,
                63288, 25960, 4488,  31328, 60368, 24996, 51572, 13224, 42288,
                27876, 15360, 6600,  20916, 9368,  58996, 36092, 46356, 20568,
                27040, 43484,
            },
            {
                6092,  51640, 46616, 54052, 26564, 38804, 61940, 27628, 30908,
                364,   23192, 50868, 54432, 3096,  16548, 26416, 4696,  65508,
                23188, 18548, 58180, 54840, 30428, 15372, 1560,  56488, 60676,
                38228, 10960, 62812, 18084, 56244, 39556, 32044, 48508, 36672,
                6964,  19864, 33276, 15956, 1416,  28568, 10516, 51968, 3972,
                24940, 35560, 27892, 2348,  43552, 57824, 34960, 22592, 56236,
                7280,  36852, 1596,  27804, 17588, 47912, 33140, 872,   56068,
                64580, 38836, 21084, 49488, 58832, 39440, 46872, 8320,  36076,
                24848, 14676, 47764, 624,   55580, 43604, 8064,  48808, 61996,
                45792, 6268,  17428, 29508, 53520, 64108, 6704,  24248, 16960,
                40912, 2524,  7364,  50184, 32776, 18724, 46632, 41160, 23112,
                35932, 28724, 48112, 8584,  5592,  57636, 2508,  55208, 20888,
                6940,  54164, 40976, 17752, 11192, 34356, 39644, 20084, 64272,
                23108, 59560, 46936, 40216, 52180, 14472, 28376, 55068, 10372,
                53072, 34308,
            },
            {
                40872, 20800, 8588,  64440, 2708,  15688, 49664, 4828,  52908,
                57948, 46364, 32804, 12780, 44532, 20808, 40172, 33584, 15168,
                49744, 35580, 45300, 11272, 20464, 26396, 34536, 41420, 18588,
                28008, 2116,  42684, 30276, 46348, 8728,  12176, 16936, 61672,
                45460, 53896, 9256,  50880, 23428, 56136, 36812, 62408, 15136,
                44064, 9040,  49016, 17776, 38164, 5928,  28860, 41492, 15400,
                32032, 46188, 21212, 65500, 55276, 11748, 22564, 44708, 13436,
                31232, 24720, 4108,  53176, 32704, 5936,  16508, 57028, 61744,
                30440, 54172, 42128, 12328, 28028, 51492, 38480, 30968, 1940,
                35036, 40868, 51432, 23796, 8528,  14860, 38124, 56620, 51800,
                26688, 45836, 60040, 36392, 56220, 2184,  27768, 53524, 64892,
                4432,  11488, 54948, 25688, 50692, 31588, 15892, 47472, 39616,
                33452, 13424, 61656, 45692, 1260,  58644, 7892,  53564, 4624,
                32840, 50320, 2432,  30900, 25132, 43036, 724,   38392, 60904,
                4452,  17392,
            },
            {
                62472, 25168, 11712, 30504, 42192, 23804, 45292, 9648,  37420,
                20992, 16872, 8416,  38028, 30080, 59292, 55632, 10444, 28864,
                59816, 8636,  38924, 3440,  62380, 47468, 52572, 9008,  24304,
                54324, 48776, 14732, 22784, 3164,  65272, 43816, 55624, 3916,
                26544, 40340, 30084, 64488, 42660, 5880,  32792, 48140, 22776,
                55028, 40400, 59272, 21776, 54004, 12460, 51920, 344,   62508,
                26560, 53216, 14112, 34524, 42868, 3664,  53004, 36112, 19028,
                48780, 58328, 9612,  44412, 26040, 64364, 22624, 43068, 11292,
                48988, 5748,  18132, 36380, 24520, 4736,  16956, 58972, 23456,
                13448, 63484, 2880,  46932, 58792, 31256, 42872, 428,   62496,
                9208,  30236, 20352, 11992, 25352, 61360, 38196, 8240,  17376,
                43564, 58636, 37516, 19952, 63728, 35280, 22772, 61304, 27624,
                3252,  48612, 23712, 36972, 55396, 27036, 16040, 46636, 36228,
                10700, 17712, 37312, 12804, 64044, 56276, 19156, 32340, 23604,
                48964, 28692,
            },
            {
                55536, 44812, 37644, 50324, 56396, 20252, 60404, 32560, 14156,
                64996, 40940, 25740, 62332, 1424,  47544, 6724,  52092, 42376,
                820,   53436, 31876, 23592, 42508, 6048,  13304, 63656, 31680,
                5320,  57820, 36544, 60048, 51668, 19532, 27936, 34012, 22452,
                14412, 56764, 2388,  18408, 47108, 13184, 20196, 57852, 1168,
                30364, 6828,  14152, 33352, 63380, 25228, 44296, 19776, 48024,
                4172,  40288, 8148,  57616, 30008, 24948, 62732, 8700,  60376,
                2600,  41784, 29440, 17720, 12736, 37148, 55316, 28576, 1468,
                34264, 21932, 56784, 60332, 46556, 64140, 10660, 53268, 42724,
                28584, 55448, 20640, 39216, 11640, 26252, 50052, 18840, 34412,
                13652, 57676, 52800, 42708, 5900,  48308, 14168, 21612, 51164,
                31476, 23816, 15012, 42208, 1148,  45272, 12396, 8156,  51352,
                18716, 29956, 8604,  50376, 21200, 32160, 42696, 60164, 29052,
                55848, 44636, 60796, 21848, 5000,  48380, 8048,  45448, 12180,
                39692, 1620,
            },
            {
                14348, 58920, 5316,  17760, 772,   35048, 6764,  54508, 29132,
                48816, 3648,  56696, 51644, 18736, 24272, 14280, 37492, 22100,
                17616, 26176, 61140, 15964, 56612, 29408, 37668, 44940, 16840,
                40332, 11920, 32960, 26004, 6496,  37708, 41576, 50580, 8120,
                63048, 36856, 10864, 25524, 34612, 61300, 27788, 9364,  37540,
                64724, 45732, 27364, 1864,  47200, 9372,  31592, 56732, 35628,
                59260, 24036, 18560, 48980, 5604,  15908, 39996, 50188, 27256,
                15180, 34684, 54940, 61512, 50844, 2480,  41056, 15276, 52720,
                63892, 39360, 9212,  2992,  31820, 40556, 34344, 21344, 47528,
                9044,  16528, 32304, 61212, 5632,  36328, 55356, 4824,  22364,
                47760, 38756, 3796,  17204, 29412, 55236, 33224, 62916, 1788,
                47256, 6256,  62148, 9980,  29640, 53456, 58352, 41244, 36256,
                56628, 63164, 43916, 14928, 5444,  64896, 12420, 3408,  19372,
                25436, 7660,  51460, 40928, 27752, 34588, 59328, 16252, 65412,
                51404, 31468,
            },
            {
                8280,  22512, 33416, 27156, 63416, 39760, 11948, 24976, 43360,
                6376,  34772, 11504, 28664, 43116, 35120, 60968, 31540, 45868,
                64224, 48532, 12524, 34268, 50420, 19556, 58400, 3236,  51348,
                61460, 20508, 46900, 9732,  53576, 15424, 1164,  60268, 47448,
                31340, 43088, 52328, 59700, 39700, 3556,  53344, 41844, 17076,
                11496, 51852, 19216, 36196, 60916, 40132, 17004, 6712,  13276,
                29272, 10392, 63552, 45208, 37752, 58824, 20624, 32092, 45536,
                7316,  22824, 46684, 4884,  21024, 33748, 8888,  46500, 19792,
                25904, 44304, 29496, 50696, 13828, 19232, 6388,  57112, 96,
                36968, 50528, 3880,  45192, 23008, 15584, 64476, 28712, 44132,
                60724, 32372, 24004, 64680, 40480, 45236, 10236, 36916, 28388,
                56872, 39668, 34236, 49636, 18848, 25940, 4584,  17016, 24644,
                496,   11012, 34148, 57644, 39452, 51944, 23568, 48344, 40076,
                63452, 104,   33172, 15856, 10480, 53628, 24480, 42064, 3824,
                20068, 36024,
            },
            {
                42840, 61472, 48348, 53032, 14896, 46240, 58392, 50756, 18252,
                60828, 22568, 46660, 15732, 63644, 9432,  4592,  54340, 11320,
                3832,  39924, 7172,  44576, 1348,  10248, 26600, 35716, 23232,
                28776, 24,    42592, 64072, 30664, 57052, 25028, 12044, 18956,
                4952,  21784, 15224, 48888, 7728,  23732, 45164, 31088, 49752,
                25372, 58360, 42632, 55592, 5112,  22976, 49584, 64880, 38728,
                43524, 54788, 32612, 804,   26724, 52060, 12204, 1744,  56972,
                36888, 64936, 10364, 39648, 27560, 56664, 59948, 31364, 6692,
                12504, 58712, 16520, 23652, 54096, 61548, 44872, 30040, 24436,
                65140, 58320, 26744, 41608, 53568, 48324, 7596,  40632, 11288,
                1476,  15472, 49648, 8780,  20828, 2872,  24760, 59264, 18540,
                12680, 26316, 54184, 3012,  60540, 37860, 32564, 65220, 46312,
                53228, 42188, 27264, 19752, 2096,  30124, 36564, 8724,  53444,
                14512, 29908, 57764, 47304, 62788, 2200,  37076, 29804, 57568,
                26676, 53872,
            },
            {
                16992, 3248,  10736, 41376, 30912, 8428,  21896, 36604, 1232,
                39520, 55844, 31904, 2328,  38772, 49188, 26960, 57832, 20656,
                29768, 36096, 59268, 54968, 21860, 40600, 65096, 14304, 48460,
                8572,  55936, 38316, 17756, 22220, 45276, 33516, 40128, 54120,
                28428, 57668, 1460,  32468, 17592, 55204, 13544, 62600, 5660,
                34844, 22088, 3152,  15504, 29784, 10836, 52924, 27148, 20324,
                2884,  50548, 21888, 13804, 35188, 8992,  63092, 24428, 41468,
                51220, 19204, 30820, 49308, 16060, 42264, 24100, 50404, 61812,
                35380, 48148, 788,   42176, 37404, 27100, 12012, 52188, 15412,
                40024, 10728, 18644, 33764, 12708, 30184, 20152, 57836, 52120,
                37448, 27500, 56472, 34972, 59956, 51684, 15312, 48592, 41944,
                8116,  45648, 21156, 15744, 42516, 47436, 7684,  12944, 30388,
                21824, 15792, 60592, 7484,  47440, 61904, 17532, 58608, 34324,
                43656, 20128, 38112, 23468, 44020, 18808, 13648, 48896, 5960,
                11436, 46864,
            },
            {
                30320, 38548, 25312, 20012, 56828, 2928,  28212, 61956, 15480,
                27048, 8248,  52084, 19944, 58740, 23240, 40592, 16320, 43768,
                50264, 24408, 17836, 28172, 32632, 52520, 46228, 4548,  59676,
                34116, 52336, 13264, 5944,  48792, 2940,  61440, 8464,  35656,
                64876, 45864, 38700, 27180, 63848, 35856, 21052, 304,   39180,
                60308, 8684,  32204, 49000, 37656, 59992, 44464, 34460, 6424,
                61344, 16688, 41072, 59804, 53856, 42928, 47328, 16324, 28620,
                5456,  13656, 58776, 100,   63384, 11772, 2900,  17748, 38684,
                5228,  54820, 32012, 63456, 8304,  3700,  59520, 35396, 4416,
                31556, 49132, 6784,  62432, 1000,  60244, 35384, 25456, 17504,
                63032, 6844,  44652, 12848, 30800, 39120, 5116,  54488, 33568,
                388,   64612, 31184, 57280, 10788, 22952, 51912, 56352, 40056,
                5568,  49716, 37812, 54720, 24780, 44456, 11736, 2736,  26588,
                50256, 4424,  9556,  54736, 6656,  31776, 52368, 61340, 39560,
                23136, 63648,
            },
            {
                54932, 59388, 6932,  50208, 65132, 33816, 44580, 53492, 48156,
                12308, 64496, 44340, 35788, 13204, 6884,  32432, 108,   52940,
                8948,  62840, 2492,  13156, 61772, 7816,  16740, 30516, 42804,
                18816, 26288, 62040, 36176, 58928, 27704, 14132, 43272, 16952,
                23888, 7160,  11816, 50996, 43468, 9684,  57448, 48176, 28968,
                14452, 54356, 45480, 63208, 18216, 24376, 648,   14408, 55684,
                30932, 46180, 25496, 4212,  29224, 18972, 33440, 3496,  61652,
                53280, 33984, 38528, 26268, 44904, 36000, 53824, 29280, 43580,
                22020, 26736, 10800, 18900, 49656, 22208, 40836, 46764, 20328,
                56184, 43252, 22812, 54652, 45812, 38960, 9716,  2920,  46568,
                31880, 22148, 54280, 1900,  25588, 64000, 19932, 29444, 61148,
                23152, 50476, 36368, 4128,  28588, 62836, 34636, 19084, 2420,
                62188, 33576, 10140, 29256, 14396, 40424, 32096, 55824, 22324,
                60468, 13392, 64204, 27608, 40516, 59080, 21544, 9004,  44320,
                33828, 320,
            },
            {
                12080, 35200, 45464, 13860, 37676, 17544, 10888, 5732,  30824,
                40944, 23896, 4484,  28468, 54848, 60232, 45712, 65212, 34744,
                14340, 39196, 42920, 47952, 37092, 24840, 57404, 38428, 11124,
                22712, 2640,  30168, 41912, 10464, 51096, 20684, 53108, 3564,
                48504, 33160, 58980, 19780, 4616,  30492, 41120, 24504, 52552,
                19268, 41476, 26552, 12240, 7292,  40232, 58176, 48120, 22144,
                36136, 8124,  11820, 37924, 65168, 7288,  56096, 39416, 20912,
                44172, 8560,  22936, 55432, 6812,  20572, 47832, 9132,  65416,
                15048, 56992, 46140, 34104, 57980, 28592, 16084, 63220, 8832,
                26428, 13560, 36732, 27828, 15884, 21556, 50672, 56768, 13872,
                41032, 49020, 18804, 36044, 43056, 8964,  46120, 11756, 37848,
                16816, 43260, 13368, 53148, 40276, 5940,  14916, 44764, 27660,
                47180, 23688, 58208, 1656,  52648, 20480, 64652, 6748,  46388,
                41404, 36416, 32436, 48372, 16340, 1184,  35576, 26936, 15244,
                51720, 18648,
            },
            {
                42628, 21416, 26748, 4868,  55084, 23408, 42568, 58684, 19128,
                34472, 56900, 16620, 50136, 41704, 18340, 11220, 28012, 20276,
                56084, 30852, 58540, 4772,  51188, 20476, 992,   44680, 53888,
                63816, 49960, 46344, 16396, 23836, 32580, 37888, 62480, 30068,
                56436, 40844, 15080, 25764, 54304, 61924, 16696, 6792,  10724,
                65388, 36088, 1616,  57068, 52028, 33472, 28332, 10244, 42220,
                62728, 52108, 57956, 47856, 23080, 14576, 49816, 25944, 11164,
                58264, 48212, 17336, 62532, 31976, 14208, 60604, 33804, 4092,
                51068, 40180, 2152,  13944, 43288, 52832, 1028,  32312, 50316,
                58640, 2560,  63700, 51596, 4560,  42036, 32788, 64800, 27224,
                3980,  10212, 62468, 58000, 15796, 50740, 56728, 26900, 5564,
                58276, 8444,  25604, 19972, 59052, 49080, 24628, 37700, 57352,
                12132, 17084, 43060, 63140, 5232,  34112, 49884, 10452, 28844,
                18432, 2312,  51976, 24524, 11788, 56416, 46584, 64764, 4060,
                60020, 28680,
            },
            {
                8656,  62880, 52716, 31480, 47732, 696,   62656, 26180, 51724,
                2392,  61396, 10312, 31728, 1776,  25020, 37416, 48144, 3076,
                23904, 7324,  18656, 27248, 12048, 34244, 61040, 27656, 14968,
                5552,  39612, 8376,  54520, 876,   57804, 5664,  46232, 12640,
                21632, 644,   63480, 44956, 37592, 2052,  34920, 44488, 32052,
                47316, 5256,  22932, 30916, 43692, 19760, 64556, 4800,  17352,
                26412, 1236,  19656, 30332, 43452, 2144,  60496, 31496, 36496,
                1396,  29588, 41840, 3904,  52000, 39268, 23336, 57768, 27668,
                19260, 36488, 24980, 64068, 6616,  37768, 24396, 12564, 38812,
                45128, 17500, 33508, 10440, 59940, 29532, 7528,  44452, 23024,
                53172, 39260, 28448, 4916,  23700, 32536, 1412,  40376, 53480,
                48268, 34660, 62260, 46208, 1308,  31912, 9828,  64588, 7208,
                50252, 31224, 21204, 36424, 46700, 16296, 25276, 37948, 54676,
                14604, 58008, 5808,  62176, 42240, 19388, 38324, 31152, 23880,
                49228, 39096,
            },
            {
                47404, 1872,  15116, 58888, 39736, 29084, 13312, 36428, 8104,
                45156, 22136, 38648, 47280, 63884, 53504, 7996,  61560, 51304,
                43644, 35908, 46880, 63316, 41584, 55752, 9652,  48336, 36240,
                31896, 56976, 20028, 35092, 65372, 42472, 18308, 9192,  27016,
                36804, 50204, 8616,  29312, 11456, 22544, 50684, 59288, 27404,
                55652, 17492, 61948, 49480, 15156, 3136,  37172, 54320, 50284,
                32512, 39792, 15420, 55248, 35012, 9868,  52280, 45376, 16600,
                64484, 13116, 50392, 27240, 10152, 45820, 1196,  42124, 11852,
                47008, 60000, 8840,  30384, 48468, 18372, 55436, 62256, 5648,
                29776, 23500, 40428, 48280, 19856, 55172, 12560, 18240, 36280,
                60516, 14712, 33952, 47004, 41652, 61688, 12672, 21952, 30876,
                18580, 3608,  29096, 38008, 16356, 54684, 21324, 41832, 34840,
                40,    55528, 39948, 9444,  27140, 60168, 43792, 980,   61836,
                30892, 44468, 20896, 34476, 29128, 3560,  8344,  55188, 13456,
                5744,  57812,
            },
            {
                32416, 24456, 35988, 18828, 9768,  56192, 20708, 49468, 40404,
                15204, 55180, 27072, 6500,  20840, 13896, 33232, 40240, 16148,
                59488, 10384, 53740, 15188, 23100, 30564, 3436,  17804, 22320,
                59904, 10876, 25424, 29056, 13924, 48648, 24604, 40244, 54048,
                60140, 34092, 18796, 53312, 47644, 60864, 14068, 20040, 2824,
                39832, 12884, 37856, 8380,  26044, 59136, 46532, 23264, 12840,
                59868, 45052, 5020,  63796, 27492, 41016, 18772, 24236, 4864,
                40356, 21712, 57104, 36980, 19000, 55168, 30548, 17096, 53204,
                5528,  32648, 54564, 21352, 61080, 10576, 33896, 42296, 20484,
                53036, 7916,  56816, 720,   25752, 38468, 47152, 5324,  57240,
                1172,  49620, 20724, 7872,  53892, 17896, 36320, 59092, 44560,
                65512, 10132, 52140, 12336, 43872, 60760, 27884, 51804, 13956,
                25784, 61740, 4468,  52216, 13360, 57048, 7612,  19620, 51368,
                24160, 39112, 9948,  47492, 53496, 59536, 49768, 42980, 21104,
                36556, 17164,
            },
            {
                11072, 50924, 41968, 64508, 6444,  45704, 33604, 3332,  63764,
                29816, 11424, 59612, 35700, 43332, 58076, 29652, 22116, 4924,
                26432, 32120, 576,   39604, 6320,  45576, 64960, 53484, 42020,
                1524,  51688, 43868, 60744, 3820,  52248, 31176, 63236, 2592,
                15640, 43620, 24232, 3984,  39004, 31288, 7416,  42784, 53760,
                24632, 59928, 45340, 53112, 34552, 11300, 40420, 29216, 6312,
                35708, 9400,  49004, 21992, 13212, 57808, 7492,  61380, 54904,
                47224, 34648, 7008,  60008, 24908, 64168, 7696,  38144, 62244,
                23668, 44292, 15728, 41104, 2364,  28120, 47276, 3752,  59708,
                14388, 45892, 65516, 35184, 14756, 62120, 51756, 33244, 25300,
                42412, 30264, 11144, 63916, 26304, 2952,  50144, 7340,  15004,
                25052, 41756, 56704, 23340, 33876, 2780,  5628,  18000, 59568,
                44620, 19784, 29716, 48432, 32796, 22588, 40960, 35424, 12620,
                48576, 6584,  65056, 15176, 24964, 17832, 11560, 33208, 26484,
                63952, 44736,
            },
            {
                28960, 60520, 3404,  27176, 22612, 52040, 60364, 17052, 24268,
                53196, 32956, 424,   49900, 17268, 3352,  46408, 56588, 12248,
                64608, 49124, 21140, 57500, 35252, 50140, 25648, 37584, 29800,
                13408, 47624, 38352, 7156,  17372, 35960, 11400, 20360, 46724,
                28848, 5884,  58224, 65004, 16776, 25660, 56876, 35952, 10164,
                33076, 6056,  28912, 21312, 56,    63276, 17064, 55988, 20436,
                62304, 24920, 31096, 53260, 2668,  37960, 48696, 33156, 26096,
                10528, 30200, 16160, 2292,  42704, 33600, 14600, 49048, 28732,
                3344,  51480, 12172, 58300, 36252, 52208, 16436, 24708, 37188,
                32036, 26828, 11128, 43140, 30712, 3708,  21092, 9700,  63512,
                16256, 52636, 37280, 44712, 56496, 31920, 39428, 28148, 47600,
                512,   32720, 19544, 6908,  63668, 48680, 40464, 30472, 47664,
                10428, 37900, 15768, 65288, 2496,  53924, 28456, 63284, 3420,
                33316, 55352, 28164, 432,   35748, 39656, 62744, 1768,  52228,
                56524, 4544,
            },
            {
                18300, 54428, 12636, 44280, 37352, 30672, 10948, 42504, 5448,
                46596, 18780, 41272, 63260, 25900, 9820,  51624, 35372, 42700,
                38096, 16728, 29032, 61320, 12760, 19332, 7764,  16004, 4512,
                62504, 19184, 32852, 22960, 55832, 44860, 58520, 7496,  38184,
                55308, 13628, 41296, 33200, 46308, 1088,  62204, 49844, 16168,
                47712, 64544, 17996, 42436, 57888, 31392, 49728, 3476,  42848,
                52080, 38888, 16428, 59352, 43292, 29388, 20552, 14888, 392,
                43640, 63292, 48952, 52896, 12220, 46424, 21972, 59120, 35512,
                19436, 39356, 27008, 6252,  22804, 43100, 64436, 9016,  57512,
                48536, 5148,  21980, 50772, 17528, 56484, 40396, 45736, 28264,
                6000,  58248, 23416, 4264,  14368, 20340, 10692, 62888, 55132,
                36824, 60220, 50736, 38540, 15880, 24896, 57904, 35536, 7776,
                54480, 23852, 42132, 6848,  45168, 18120, 10096, 58268, 43596,
                17312, 21840, 41316, 60692, 45352, 7312,  29592, 46488, 14948,
                8876,  40616,
            },
            {
                48832, 21760, 34300, 57700, 15588, 1252,  55800, 26700, 38160,
                59056, 8056,  22948, 13064, 38788, 60776, 28196, 18440, 2716,
                7868,  46092, 53056, 4132,  43312, 32664, 60636, 54492, 45240,
                57592, 27316, 9824,  63444, 39884, 1700,  26472, 33780, 60992,
                49028, 22112, 8412,  51504, 11236, 21132, 29044, 40540, 23140,
                1884,  52256, 12096, 36084, 5048,  24148, 45548, 10360, 33496,
                13992, 784,   47048, 7140,  34624, 11184, 64872, 53616, 39040,
                56600, 18576, 23488, 38448, 28032, 5376,  54392, 508,   10708,
                64940, 56204, 46824, 32872, 60088, 13012, 30616, 216,   40144,
                18564, 61520, 54784, 37544, 7740,  60704, 24072, 36512, 13468,
                47936, 18872, 40744, 34068, 51628, 60780, 42024, 22808, 17368,
                5996,  13500, 26796, 46152, 9488,  53464, 12816, 21716, 63944,
                3692,  33332, 56924, 26368, 36016, 50116, 38720, 14684, 25728,
                52004, 57580, 11228, 31832, 50768, 23044, 57464, 19196, 36904,
                24024, 31984,
            },
            {
                184,   38912, 8360,  63400, 50092, 20000, 47896, 65348, 13920,
                34988, 50888, 55984, 31388, 47844, 6668,  54660, 23860, 62028,
                55944, 34004, 24348, 9800,  27392, 51900, 39144, 11160, 24032,
                34732, 49460, 5504,  52808, 29332, 15252, 50564, 23476, 17640,
                476,   31228, 26952, 37224, 54620, 43464, 13380, 4704,  56148,
                31148, 38252, 26856, 46752, 54752, 14816, 37972, 26532, 65144,
                57520, 28552, 19336, 61784, 23144, 46156, 5312,  50604, 27940,
                8084,  35220, 3756,  61660, 32548, 57652, 40764, 26408, 44096,
                31252, 8184,  17688, 1672,  49560, 19996, 54920, 44848, 51316,
                13824, 33716, 2048,  29092, 44644, 12068, 2652,  53052, 59372,
                31672, 132,   65128, 9512,  28472, 46048, 2228,  34688, 52312,
                44432, 30240, 3272,  57220, 31864, 43044, 748,   28424, 39788,
                14824, 51840, 60700, 12580, 20520, 62484, 668,   31652, 46796,
                8300,  37424, 3856,  19576, 14100, 53980, 2656,  42256, 59188,
                51196, 62144,
            },
            {
                14192, 47096, 29456, 4068,  25208, 40360, 9116,  31696, 21248,
                1760,  28080, 43920, 4040,  20376, 34284, 14796, 44984, 31296,
                20852, 13624, 40124, 62640, 47900, 2436,  21068, 30656, 41248,
                20,    16836, 42664, 21136, 46512, 12004, 64712, 42156, 9856,
                44600, 63128, 57224, 18768, 6064,  63924, 35436, 59016, 19412,
                44344, 60384, 8980,  62876, 20268, 7192,  60124, 50800, 21792,
                8268,  41212, 54616, 37548, 26868, 57108, 17740, 41724, 22480,
                13020, 58680, 44976, 15428, 10008, 20420, 48316, 17284, 13728,
                50172, 24244, 41764, 61868, 37884, 6640,  25724, 35148, 22420,
                27988, 9936,  41920, 64212, 25364, 49340, 34080, 20256, 8452,
                42996, 55104, 25704, 48900, 16896, 6952,  57124, 25292, 8836,
                64276, 39972, 22416, 61168, 20016, 36528, 62044, 50816, 46560,
                18360, 30636, 1712,  47812, 8932,  28824, 59252, 22996, 5072,
                64040, 27784, 48864, 43384, 63184, 9484,  27468, 33808, 11916,
                5696,  26852,
            },
            {
                55332, 58736, 17828, 43092, 53836, 33468, 5160,  45312, 52532,
                62348, 11212, 16544, 58296, 64684, 42136, 888,   38400, 10772,
                50328, 1784,  57612, 17960, 35804, 14952, 58960, 7080,  64756,
                55252, 12924, 61224, 37044, 56836, 32316, 4056,  54900, 36296,
                52348, 15196, 39956, 2344,  48192, 30660, 26308, 50916, 7268,
                14252, 24776, 2700,  49484, 29204, 34672, 41816, 17160, 2572,
                31908, 47824, 10780, 4032,  14436, 32212, 1676,  36384, 62596,
                31684, 49896, 25832, 40188, 51716, 6816,  60440, 36892, 63052,
                5132,  53728, 34072, 14840, 28888, 57020, 11216, 63244, 4008,
                53384, 58368, 47796, 19036, 14644, 56004, 29500, 63156, 15812,
                38532, 21680, 12584, 36192, 59540, 38796, 31372, 13884, 48240,
                18820, 54956, 11420, 49148, 5424,  16860, 8580,  25652, 59224,
                6140,  43832, 22724, 39516, 33752, 52580, 42736, 55220, 40164,
                34076, 15628, 58604, 24300, 35364, 39248, 17292, 46912, 64820,
                20664, 44324,
            },
            {
                36236, 32732, 22532, 11168, 61216, 14616, 59576, 17468, 24572,
                39300, 48540, 36208, 23508, 9920,  52724, 29416, 59820, 47212,
                63468, 27820, 42796, 5392,  54916, 25344, 43976, 47288, 18316,
                36128, 29920, 23648, 2956,  8352,  18640, 39584, 28492, 5828,
                25576, 11556, 33720, 59304, 24000, 16228, 9908,  46544, 39608,
                32868, 52964, 17548, 40268, 12888, 57136, 4784,  44876, 53940,
                36292, 24112, 63504, 51472, 43780, 59944, 47956, 10568, 55284,
                6124,  19400, 1248,  65400, 30380, 22912, 33912, 1896,  28452,
                39224, 58412, 21628, 9344,  47352, 42588, 18052, 39552, 46028,
                16632, 36272, 23252, 6408,  38968, 4372,  41344, 46248, 2140,
                51956, 61384, 30252, 3488,  53400, 19960, 62664, 43064, 27384,
                37288, 1996,  33944, 41488, 29016, 45116, 55600, 32768, 20920,
                11112, 36324, 64696, 56240, 16376, 3932,  13068, 18412, 9388,
                21028, 53152, 1140,  29692, 5328,  60240, 49956, 3592,  30392,
                40068, 9816,
            },
            {
                2564,  63136, 6544,  50892, 38256, 26776, 35924, 56736, 3688,
                30136, 55524, 6972,  40264, 26456, 49404, 18156, 7428,  25092,
                16272, 37032, 31636, 22152, 63948, 33488, 9688,  52988, 27480,
                5156,  51272, 44752, 59508, 26580, 49540, 62064, 21884, 58376,
                48544, 20292, 45952, 8256,  41812, 53704, 61260, 840,   22184,
                65340, 35608, 55956, 43840, 31964, 23608, 63628, 27328, 12228,
                61044, 15356, 29248, 18748, 7840,  38380, 24584, 28772, 16172,
                42764, 46732, 37636, 54252, 12832, 46304, 43204, 52528, 19536,
                11360, 45380, 3028,  64260, 53048, 4764,  32200, 59824, 8356,
                30064, 1584,  62104, 32080, 52600, 59484, 10076, 22556, 26696,
                33784, 6016,  47332, 44232, 24292, 10564, 744,   55564, 5840,
                59760, 15452, 24696, 52344, 65124, 13660, 38204, 2160,  42048,
                54360, 50532, 25480, 7448,  45872, 27260, 49412, 36612, 62068,
                47744, 41872, 11856, 45852, 54964, 21548, 13404, 56208, 23400,
                52872, 16744,
            },
            {
                25620, 41864, 29152, 45728, 732,   20872, 49440, 10348, 42052,
                13576, 19120, 60748, 2384,  32576, 12796, 56624, 35040, 3400,
                54372, 8936,  52064, 12516, 49284, 944,   38520, 14024, 21660,
                62560, 40200, 10796, 33336, 53328, 13196, 43388, 16048, 35096,
                2868,  55380, 29040, 62676, 21224, 36752, 31988, 12100, 58168,
                27376, 4300,  10508, 21436, 524,   51692, 9692,  19376, 39624,
                49828, 956,   41552, 34856, 55704, 21208, 64616, 4760,  52796,
                60672, 22236, 8868,  28256, 3892,  58316, 15976, 8012,  56500,
                25516, 30648, 17020, 36108, 23516, 27172, 49272, 21152, 13056,
                55540, 50352, 44044, 11888, 27144, 35440, 17260, 50780, 57876,
                11684, 19152, 55952, 15124, 39732, 49684, 34944, 22204, 50716,
                32220, 46284, 9120,  58616, 3960,  23020, 49944, 62592, 15672,
                28100, 3324,  13912, 19468, 62768, 31892, 59532, 24108, 5956,
                26876, 31536, 65488, 18612, 8484,  32348, 43188, 36920, 7944,
                61368, 47444,
            },
            {
                51476, 18904, 55076, 13696, 64592, 31240, 7456,  44480, 63832,
                33936, 27344, 45616, 53808, 42712, 62928, 22372, 39464, 43928,
                19804, 58424, 45420, 6464,  28688, 41760, 59572, 31300, 55816,
                2224,  47908, 15840, 20392, 37864, 1044,  46816, 7592,  31116,
                64200, 38580, 13432, 5984,  50180, 3712,  44800, 19172, 41040,
                48856, 16276, 45684, 62512, 57984, 37908, 47232, 33612, 56248,
                7068,  21696, 46324, 59060, 3052,  13348, 49552, 40452, 34716,
                11908, 32028, 57292, 17584, 35768, 25256, 61532, 32184, 41428,
                48436, 61020, 51608, 40760, 12224, 58660, 1112,  37628, 64568,
                24088, 40584, 7488,  20732, 47564, 65008, 3348,  31200, 43268,
                63420, 35620, 27504, 8192,  64644, 28880, 17308, 40576, 12452,
                63752, 20072, 39116, 27816, 18344, 35388, 10224, 30448, 59772,
                48332, 34496, 57576, 41276, 37560, 936,   10932, 43400, 56920,
                2828,  14836, 37200, 51180, 40456, 26108, 63488, 900,   27908,
                33668, 12772,
            },
            {
                57936, 37184, 9240,  23384, 40608, 57732, 18016, 52692, 22924,
                1276,  50680, 9096,  21268, 16140, 5620,  46712, 28556, 65520,
                13520, 26376, 38864, 61796, 16008, 19192, 24080, 7420,  43180,
                34692, 25672, 6336,  57076, 65524, 28788, 23268, 59916, 11008,
                50520, 17764, 43988, 33304, 26848, 15340, 56776, 29488, 52472,
                6068,  37472, 30528, 25464, 7804,  14876, 28540, 3860,  60044,
                25560, 31120, 52496, 10380, 26760, 45124, 30628, 18632, 140,
                25984, 51016, 41776, 63772, 49300, 39404, 10756, 22428, 4652,
                14276, 312,   33388, 7580,  45104, 15968, 54188, 43336, 33284,
                27956, 17464, 60656, 38136, 15192, 54944, 25196, 39188, 7276,
                48448, 1084,  41740, 58096, 32968, 4976,  61016, 44840, 7988,
                29828, 53672, 972,   43220, 47776, 56912, 6680,  40656, 19160,
                23636, 9112,  45152, 29180, 53632, 21728, 51532, 16972, 33940,
                53468, 60444, 23556, 6396,  58088, 15576, 48016, 19608, 59172,
                44164, 4844,
            },
            {
                30736, 1636,  60752, 33888, 47584, 2972,  28220, 38500, 14912,
                58164, 35840, 62172, 30764, 37504, 58524, 11036, 112,   50196,
                32208, 5204,  21800, 35160, 55972, 50444, 45752, 64584, 12192,
                17916, 60788, 49916, 31612, 9764,  41312, 54672, 36832, 26620,
                41676, 224,   24444, 60340, 54220, 38276, 64092, 8472,  24336,
                13832, 59420, 54208, 18800, 49804, 40988, 64956, 17484, 43760,
                37112, 14004, 64132, 17272, 33836, 54580, 8420,  57900, 63256,
                37372, 6956,  14504, 2128,  20512, 6276,  54568, 50584, 64412,
                37760, 28084, 20428, 55548, 63600, 29712, 19720, 10180, 3284,
                47852, 5184,  53252, 29812, 324,   9220,  45832, 21380, 13240,
                16680, 54472, 23560, 20400, 12876, 51248, 25476, 55916, 2844,
                23884, 36584, 61412, 13008, 32104, 52980, 25948, 45964, 36,
                38092, 63964, 4292,  12360, 61160, 6484,  46812, 28632, 39064,
                20320, 10536, 48500, 29164, 34352, 4504,  11304, 53780, 39480,
                16184, 22424,
            },
            {
                40972, 49388, 27532, 15788, 53916, 11836, 61844, 32484, 6224,
                47392, 24748, 12508, 3600,  48860, 25624, 34360, 53136, 17400,
                41424, 60736, 48116, 1516,  10080, 32296, 3264,  37212, 51864,
                29228, 39160, 44540, 22340, 3208,  17264, 48512, 5288,  19828,
                52660, 57752, 9320,  46728, 18892, 11344, 1332,  34952, 47308,
                62052, 2032,  42668, 35408, 3292,  32572, 11492, 23348, 54688,
                9100,  48828, 4700,  39992, 60644, 23284, 42408, 15248, 47508,
                21676, 44684, 55888, 30496, 47124, 27368, 42896, 34396, 18568,
                45504, 59756, 10944, 24412, 38800, 6288,  35676, 51380, 62004,
                14584, 58596, 35044, 23016, 42328, 57308, 32636, 61860, 51748,
                29932, 60768, 37688, 46836, 3312,  42540, 35464, 14700, 47968,
                57912, 17676, 50216, 5804,  16300, 21568, 63240, 11804, 58236,
                15104, 51284, 32948, 17852, 25128, 35348, 14016, 64460, 8032,
                45532, 41180, 284,   55184, 43612, 61248, 24176, 31048, 50540,
                8768,  65048,
            },
            {
                12324, 44824, 20452, 4980,  37456, 25060, 46088, 20784, 54540,
                40092, 17788, 43896, 56120, 40884, 19224, 61196, 8368,  24144,
                56820, 11592, 29420, 43576, 63132, 14248, 27124, 20812, 58196,
                8976,  1340,  14320, 52676, 35352, 58428, 12616, 32700, 62800,
                14540, 30128, 35728, 4308,  40640, 23464, 50560, 43248, 20576,
                33080, 27096, 11936, 22300, 61284, 55672, 45740, 51084, 1368,
                34400, 41916, 20540, 29392, 1960,  36344, 5672,  51788, 28944,
                10520, 61312, 24528, 35336, 11584, 62516, 15020, 2608,  25868,
                5460,  52884, 41696, 49160, 1816,  60488, 46680, 22284, 39940,
                26000, 44548, 10820, 50960, 63688, 18324, 36260, 4364,  40864,
                26184, 6236,  10208, 53380, 30556, 59460, 9412,  21412, 39316,
                32996, 10732, 26808, 39912, 58988, 4052,  42392, 35896, 30992,
                54760, 26896, 42828, 56528, 49052, 40416, 58876, 3928,  26188,
                31836, 62864, 13244, 21648, 17516, 38432, 46136, 2204,  35968,
                26276, 55452,
            },
            {
                34444, 7604,  63248, 57316, 51060, 8540,  42004, 3940,  59472,
                10476, 29556, 64972, 7400,  51652, 13256, 30512, 44192, 37988,
                2832,  35912, 16372, 25108, 52768, 40000, 54712, 5656,  46744,
                24352, 63732, 55876, 27348, 42548, 61788, 25412, 39544, 3448,
                44500, 21996, 64796, 48836, 31624, 61008, 28308, 58140, 17068,
                40004, 7072,  52200, 44484, 16756, 5400,  26500, 30500, 19056,
                63020, 57176, 25048, 53936, 49452, 12232, 62672, 19552, 39152,
                33456, 4716,  17944, 51324, 38784, 58968, 31524, 55608, 9976,
                57992, 35108, 14804, 31852, 17772, 27116, 12896, 30752, 55044,
                7556,  19444, 31688, 3996,  13876, 28020, 49012, 11536, 55320,
                19600, 44724, 34368, 15868, 62628, 18556, 27836, 52584, 65264,
                43736, 6888,  62684, 45512, 34124, 28572, 51032, 8264,  20048,
                47272, 6836,  21960, 9996,  1264,  30536, 19112, 53784, 50148,
                16672, 56268, 35484, 51776, 27592, 9248,  64076, 14624, 57572,
                5364,  18328,
            },
            {
                53132, 24364, 31600, 42924, 17872, 34820, 64372, 14196, 26360,
                50004, 564,   33860, 22468, 27996, 2176,  47632, 64172, 20796,
                54348, 49688, 59256, 19588, 7968,  34680, 60288, 16944, 42016,
                32832, 36708, 18984, 10532, 5752,  20772, 46192, 8284,  49980,
                55576, 27556, 11792, 16716, 6968,  55400, 14172, 4708,  9832,
                63200, 47836, 57376, 29464, 37328, 9676,  58532, 39400, 14648,
                6260,  46292, 10100, 15808, 44080, 32524, 56596, 26084, 1440,
                54016, 58624, 42536, 8696,  656,   19516, 23088, 48636, 39548,
                29188, 21064, 7864,  62700, 43700, 52616, 57932, 2772,  16640,
                65472, 37440, 56948, 46956, 39892, 7692,  24016, 58356, 1372,
                31568, 64904, 50332, 24852, 176,   38340, 45060, 6020,  2240,
                30180, 19908, 53992, 23424, 1156,  55908, 14300, 24564, 65084,
                2212,  39844, 60528, 37036, 63640, 44000, 23200, 11324, 37836,
                2232,  24308, 7568,  59896, 3196,  33356, 49196, 20620, 42484,
                29208, 47228,
            },
            {
                3504,  39336, 13716, 660,   28704, 22576, 55708, 30836, 36620,
                19360, 53100, 38868, 60424, 16848, 35612, 57212, 6548,  14424,
                27728, 4780,  45668, 31072, 3744,  48756, 22560, 30076, 11472,
                50824, 4320,  47548, 30620, 51580, 34036, 56560, 16380, 31128,
                37904, 2260,  42832, 51564, 36540, 20940, 45304, 38192, 54496,
                30696, 23744, 14440, 60,    64428, 48572, 21096, 43120, 60452,
                32680, 27568, 37868, 61452, 7560,  22000, 40876, 14228, 45404,
                48992, 15552, 26692, 64152, 46012, 53360, 7092,  43516, 16808,
                64984, 45796, 50728, 3640,  23244, 34696, 8944,  41664, 33736,
                49600, 1020,  26944, 21260, 54092, 62844, 43416, 16764, 46748,
                38088, 14148, 4900,  41228, 56964, 48232, 13568, 56012, 25676,
                15652, 49492, 36336, 12036, 18224, 60484, 37984, 48820, 43948,
                16720, 29624, 12836, 50784, 16112, 5272,  47960, 34320, 58072,
                42416, 29336, 47324, 39352, 44512, 56520, 23632, 11524, 60052,
                37892, 62696,
            },
            {
                21580, 50640, 61400, 46352, 59300, 11100, 2584,  48596, 44868,
                62904, 8576,  13948, 46456, 54152, 10768, 39816, 23788, 32784,
                42272, 10256, 63340, 39204, 13188, 43144, 64316, 260,   57704,
                15664, 61988, 40048, 59144, 14940, 984,   64324, 23436, 10928,
                61596, 18664, 58912, 24796, 62456, 1504,  32924, 26088, 50940,
                2996,  36156, 19472, 41924, 25260, 33256, 53168, 2676,  11544,
                22288, 52116, 520,   18428, 58700, 29676, 3120,  65300, 9992,
                31284, 36700, 21148, 29808, 13344, 33528, 37232, 60568, 12364,
                1304,  25232, 56036, 40032, 11296, 60172, 19980, 46460, 24260,
                11980, 43112, 61028, 15860, 33268, 2728,  29540, 35324, 61088,
                9180,  22708, 59176, 29160, 20648, 8208,  31716, 35320, 60936,
                40372, 58512, 8328,  47396, 41504, 31492, 5976,  10908, 33164,
                57788, 53220, 34896, 24404, 28340, 54836, 8536,  61676, 20560,
                14492, 65152, 10560, 19832, 16116, 6592,  30412, 51484, 892,
                16844, 9704,
            },
            {
                54844, 27680, 16384, 7876,  37576, 52592, 40300, 16760, 5736,
                24156, 41892, 31972, 3864,  25500, 43320, 61912, 18496, 52380,
                60236, 34976, 21496, 56180, 52052, 24652, 9236,  37568, 44944,
                27968, 22392, 8216,  24860, 45400, 37496, 28436, 42068, 50980,
                6344,  47128, 33652, 8904,  41264, 12696, 48416, 65260, 18212,
                11108, 44744, 60512, 54960, 12648, 7508,  17348, 35880, 55836,
                47516, 64664, 41544, 34752, 50724, 47028, 37144, 55256, 24136,
                6856,  60948, 3828,  41256, 55456, 24744, 4288,  27912, 52360,
                32828, 36644, 14232, 29868, 48096, 37816, 4644,  54200, 62556,
                28748, 52484, 5500,  10144, 38636, 50456, 12756, 20504, 51948,
                27300, 54592, 44276, 11716, 36664, 64116, 23544, 51160, 10880,
                21788, 32640, 3368,  64296, 27448, 53552, 21272, 62436, 26384,
                4440,  19496, 43148, 2484,  58572, 40152, 32672, 26640, 45640,
                5720,  49676, 32352, 53948, 26508, 63476, 41176, 34868, 46004,
                25472, 33112,
            },
            {
                12116, 44312, 35112, 49396, 25188, 32112, 21004, 61308, 12656,
                29076, 55928, 59656, 20880, 50448, 5896,  29912, 792,   48012,
                7636,  25804, 16020, 1652,  28784, 17848, 33928, 54896, 20116,
                52696, 2272,  35128, 54300, 11688, 19080, 3784,  54656, 35244,
                21448, 27220, 14716, 52476, 29632, 57760, 23156, 7712,  39680,
                59004, 33628, 6480,  49564, 39060, 62692, 27944, 44380, 5652,
                30052, 14124, 24808, 5176,  11720, 26732, 15932, 51924, 19364,
                44216, 56832, 50468, 10228, 47504, 62416, 18628, 49516, 9480,
                20412, 58844, 6620,  18520, 63632, 27540, 14780, 32040, 7116,
                17604, 35632, 22908, 59548, 45484, 25708, 56632, 5296,  41624,
                1920,  17356, 33572, 49400, 2776,  53996, 16144, 45720, 1072,
                43192, 54768, 14572, 24324, 44396, 16224, 1688,  50600, 40980,
                46756, 8008,  61632, 11000, 47088, 18220, 13320, 672,   56552,
                40568, 23000, 2516,  60808, 37444, 4084,  14180, 53512, 7644,
                56932, 40792,
            },
            {
                5908,  58932, 3100,  19300, 63028, 4776,  43404, 57388, 34484,
                49764, 2064,  17440, 37180, 63528, 34168, 15440, 55516, 37384,
                12788, 43864, 50472, 38216, 58896, 47192, 60972, 6040,  13724,
                41708, 48056, 30144, 62616, 42676, 60056, 47788, 14096, 58492,
                40712, 480,   55920, 38108, 19524, 3768,  43888, 15904, 28892,
                52880, 21804, 26984, 3596,  31428, 22600, 50396, 58808, 39808,
                19416, 8608,  57940, 45308, 20736, 63084, 4408,  34088, 40212,
                12296, 33016, 17444, 38660, 1756,  15064, 58528, 42384, 38556,
                62740, 44964, 53960, 42040, 172,   51204, 57044, 21732, 39636,
                58256, 48580, 41464, 30632, 18788, 64740, 8456,  47156, 31332,
                63352, 39228, 6520,  61828, 19096, 41868, 5920,  28796, 63312,
                18596, 37004, 29668, 50088, 6800,  39076, 59464, 35240, 12344,
                22876, 54500, 30048, 36600, 21644, 64536, 52316, 37528, 30148,
                16484, 51916, 35596, 12520, 43820, 48488, 22328, 61692, 28284,
                19772, 65396,
            },
            {
                30248, 22896, 42112, 56168, 10068, 14508, 53352, 25936, 9128,
                38288, 45384, 11588, 27744, 8140,  57628, 46128, 20544, 27972,
                65052, 57444, 5616,  31572, 11196, 3500,  40236, 25856, 32188,
                65316, 10840, 17100, 6612,  21120, 32344, 5388,  25848, 8800,
                31348, 62296, 44316, 5556,  63844, 49908, 35868, 55616, 768,
                47172, 14136, 63460, 17844, 46440, 9104,  15232, 996,   25776,
                62488, 35496, 51544, 32272, 55776, 42820, 9080,  59148, 29108,
                248,   25308, 64464, 28444, 22656, 35452, 30476, 5988,  24092,
                2236,  31212, 22828, 10148, 25716, 33676, 43028, 2792,  45764,
                9396,  13792, 4232,  55372, 380,   14812, 37580, 21768, 58468,
                13492, 23716, 45612, 26208, 31016, 59044, 38776, 24548, 47996,
                9904,  4956,  57200, 61272, 33032, 10192, 55428, 28404, 17408,
                63960, 38816, 15120, 50232, 4252,  43812, 24656, 6660,  62608,
                9588,  58892, 25552, 18684, 57036, 29580, 10032, 17276, 39012,
                1916,  48340,
            },
            {
                52436, 15320, 33556, 27296, 39436, 47064, 30268, 1212,  18756,
                63972, 22296, 54992, 48036, 39984, 23584, 11188, 41712, 3956,
                32588, 22520, 18396, 45980, 63788, 23084, 15560, 49924, 57372,
                3128,  36828, 56032, 50372, 27564, 52788, 38932, 64852, 18436,
                49464, 11656, 16932, 23988, 32448, 10204, 26572, 61488, 31748,
                42544, 9880,  37016, 40840, 55520, 60652, 34160, 53652, 11620,
                48564, 40816, 15456, 1664,  27928, 38724, 23092, 48884, 61960,
                45808, 52932, 5844,  57968, 44988, 53804, 11796, 46720, 57096,
                16616, 13220, 50452, 35884, 60028, 17384, 12236, 64352, 25004,
                34548, 62756, 26912, 50900, 33392, 42880, 28004, 53620, 35052,
                10112, 50652, 56908, 15032, 8764,  51444, 12692, 34500, 59936,
                53088, 40260, 22376, 12596, 19680, 48244, 24028, 3740,  45228,
                32268, 1116,  60388, 26928, 33148, 9952,  55820, 48912, 20348,
                44804, 31452, 47484, 8036,  200,   33884, 59100, 50796, 44908,
                13420, 36656,
            },
            {
                8512,  62128, 372,   50416, 64456, 21956, 36340, 58280, 51216,
                41380, 31164, 5004,  14892, 32180, 3040,  60188, 53264, 49608,
                9264,  40908, 54212, 27208, 36116, 52776, 44356, 8628,  29060,
                21652, 45828, 24952, 40304, 712,   15756, 34480, 45672, 23360,
                54080, 37068, 28832, 46820, 59884, 41792, 13548, 18836, 6284,
                24168, 57328, 51388, 2156,  28700, 43548, 20900, 38396, 31584,
                3376,  22760, 60884, 7460,  18228, 52300, 13504, 3524,  16440,
                20968, 35116, 13688, 40388, 19292, 8092,  63864, 26372, 34056,
                48496, 40484, 65256, 3844,  46268, 56300, 6108,  49104, 29560,
                52644, 19560, 38936, 22256, 10368, 61668, 48668, 2860,  18188,
                42000, 29832, 692,   37852, 47208, 21892, 56164, 2080,  15748,
                31956, 26300, 46016, 244,   42564, 65528, 37308, 8232,  57828,
                51728, 20076, 7108,  41396, 58772, 17228, 35520, 28648, 14416,
                39036, 5076,  63900, 41460, 55196, 38120, 23840, 6096,  32452,
                60292, 25236,
            },
            {
                46896, 18668, 44016, 12260, 7300,  17456, 4168,  44624, 13572,
                7196,  26716, 58724, 50104, 64528, 44552, 17696, 25596, 35064,
                15256, 59280, 532,   13776, 7264,  19692, 33632, 61992, 39264,
                18096, 59444, 13340, 8996,  63396, 57000, 11032, 3360,  41988,
                6756,  57600, 1832,  52744, 7976,  21780, 53772, 38056, 49172,
                64444, 34084, 19984, 7608,  24788, 13200, 4216,  65468, 17552,
                57196, 43172, 29396, 46652, 64028, 34876, 58144, 31804, 41804,
                55900, 9596,  48660, 31324, 2968,  51184, 37264, 20564, 4444,
                54316, 27524, 7756,  20168, 30432, 40016, 22280, 37140, 15980,
                1640,  58940, 7024,  44960, 57460, 16812, 5684,  24936, 59860,
                7296,  65324, 19672, 32904, 61432, 4088,  27740, 44572, 20232,
                7648,  62832, 51244, 34644, 16708, 52992, 30520, 14032, 40336,
                25076, 11468, 48404, 53448, 22928, 3140,  61208, 42596, 1324,
                54644, 22752, 12952, 28076, 20124, 15096, 64552, 41900, 3036,
                21080, 56376,
            },
            {
                34100, 28896, 55404, 37980, 31868, 54256, 60324, 23720, 33728,
                62016, 17012, 36212, 220,   20444, 37832, 29372, 6916,  63152,
                38984, 47148, 29748, 62852, 42728, 56708, 1828,  12356, 53396,
                6128,  34888, 49180, 31724, 43936, 22528, 48352, 29604, 59688,
                33484, 20848, 14656, 40160, 35328, 3412,  58440, 45544, 2464,
                11608, 29944, 15708, 62376, 36356, 58220, 46800, 50808, 26644,
                9332,  52480, 12828, 37248, 24732, 10512, 44760, 26704, 7000,
                38004, 23980, 62708, 26956, 59048, 42236, 14772, 60396, 43808,
                10320, 58184, 38032, 52296, 14104, 26388, 54408, 61116, 10860,
                42140, 47252, 30344, 12724, 35584, 52784, 31932, 38988, 46404,
                55272, 26476, 44056, 52732, 11728, 41332, 35712, 64624, 49760,
                37780, 14128, 4400,  28828, 59380, 5532,  21044, 47076, 62476,
                34828, 43444, 29524, 14640, 37824, 46036, 11840, 25632, 50100,
                32656, 60500, 35412, 49184, 52556, 9260,  46672, 26904, 53116,
                11348, 40040,
            },
            {
                3672,  6384,  22640, 61060, 26036, 48256, 40652, 29048, 10600,
                47376, 53696, 42932, 25156, 9720,  52020, 13080, 55896, 2356,
                19596, 23540, 10740, 49724, 38328, 24576, 30684, 46740, 26880,
                43232, 2788,  61188, 28504, 19312, 37388, 53724, 17316, 12988,
                27088, 61840, 48608, 25460, 65088, 30572, 16316, 28016, 22716,
                55112, 41240, 45044, 48812, 53432, 10304, 30232, 41500, 5492,
                34532, 20616, 59908, 6440,  49044, 856,   54028, 18964, 65160,
                46676, 1488,  52404, 17880, 11248, 33152, 23856, 456,   30152,
                18288, 23172, 33116, 1328,  62132, 8340,  43984, 4768,  33368,
                56132, 17496, 63988, 23964, 1296,  41616, 63388, 13768, 21540,
                11096, 36744, 16264, 6188,  23504, 54460, 17728, 9756,  30104,
                58104, 42096, 23968, 55996, 10812, 39696, 27104, 1836,  54668,
                17200, 5676,  63696, 56188, 8080,  30988, 64700, 19072, 57072,
                9136,  17380, 4164,  43712, 1988,  31940, 57784, 18472, 30072,
                62996, 16128,
            },
            {
                58188, 42888, 51824, 14260, 8848,  2084,  15824, 52012, 5404,
                39200, 21704, 12604, 56824, 33228, 62240, 41860, 48632, 31856,
                44660, 53912, 34172, 17220, 60092, 4628,  51548, 16636, 64188,
                22672, 54772, 15388, 7724,  57928, 2104,  6360,  62760, 39344,
                51904, 9124,  43944, 18276, 12032, 51008, 43024, 9232,  35168,
                60272, 5192,  26312, 1288,  32848, 18264, 23096, 14972, 62232,
                45688, 39292, 16328, 56076, 30352, 21700, 40012, 12188, 58908,
                29368, 14956, 36464, 43732, 5280,  47924, 55856, 39332, 50608,
                62944, 46436, 15300, 41512, 47576, 31336, 18924, 49756, 27980,
                21808, 3968,  37304, 50360, 27416, 19052, 8164,  29320, 51744,
                33408, 1840,  62868, 48788, 31404, 59600, 5068,  25564, 46928,
                1540,  54108, 18752, 36400, 44508, 61588, 49976, 32100, 10272,
                58952, 23320, 604,   34216, 20860, 50812, 40700, 6008,  36592,
                46524, 53284, 26344, 62776, 22316, 36780, 13616, 7304,  44524,
                37320, 48940,
            },
            {
                26480, 64864, 35564, 20364, 46080, 57324, 37024, 19800, 65108,
                2552,  30720, 61376, 6540,  46760, 4200,  22412, 16464, 26496,
                7624,  58612, 3468,  27588, 9300,  21476, 40776, 58308, 10120,
                32556, 38040, 41564, 25864, 51252, 35296, 42964, 24452, 46336,
                264,   36372, 5572,  54804, 33308, 4680,  62196, 47860, 18464,
                51512, 13596, 21640, 39220, 59200, 6104,  37936, 56788, 124,
                54260, 24276, 4004,  33512, 43492, 61392, 5692,  50492, 33760,
                22352, 4224,  57084, 60952, 21616, 28320, 64728, 7368,  12904,
                35652, 4908,  59796, 55408, 24420, 11508, 64784, 35852, 13836,
                52976, 40352, 60664, 10020, 46076, 54732, 58560, 43164, 4192,
                47356, 57632, 40572, 27844, 14116, 38752, 43860, 62072, 21928,
                12820, 32896, 8508,  48200, 15036, 22832, 7204,  19064, 36864,
                45896, 27584, 39588, 47804, 16044, 59432, 2424,  29148, 23656,
                14596, 39664, 30940, 10548, 56140, 40696, 61820, 50952, 812,
                22836, 9216,
            },
            {
                18932, 11696, 1312,  41080, 33144, 62748, 27676, 44284, 55692,
                34404, 50524, 27188, 18364, 39920, 28476, 36668, 57184, 65360,
                12216, 41300, 36988, 62328, 47916, 31928, 14220, 35872, 308,
                49444, 20220, 11432, 47020, 64924, 14064, 32284, 10492, 19140,
                31040, 59096, 22860, 28992, 57312, 37748, 24684, 620,   40176,
                31784, 63348, 55796, 9708,  43800, 64236, 26996, 48748, 31744,
                11860, 28792, 64732, 51584, 13532, 27076, 37588, 16712, 42476,
                8308,  49544, 31028, 40044, 9244,  35248, 16120, 45260, 25520,
                53708, 9608,  28624, 17488, 3908,  51664, 39460, 1576,  59116,
                24884, 7148,  31432, 15284, 34248, 2816,  23276, 16156, 60872,
                25244, 18764, 8908,  21076, 51428, 400,   34176, 16088, 52576,
                40772, 60180, 26888, 64780, 2672,  34096, 57916, 52844, 41408,
                13708, 51612, 60836, 9944,  26216, 53680, 12608, 44516, 61484,
                57892, 6820,  20496, 47416, 16012, 4964,  24892, 33872, 59924,
                54176, 32308,
            },
            {
                48000, 56548, 28432, 50928, 23440, 11056, 6072,  13336, 24476,
                16244, 8460,  43588, 54996, 13916, 60392, 9912,  796,   33092,
                51088, 15016, 22852, 53060, 43048, 6552,  55032, 45716, 25492,
                6412,  61920, 56344, 3512,  17900, 28216, 59456, 55216, 49072,
                63372, 15508, 40668, 45904, 10024, 20876, 15076, 59236, 26780,
                6892,  36288, 16692, 29736, 52156, 12588, 20720, 42616, 8000,
                19188, 41388, 46548, 9452,  18812, 58012, 3288,  63124, 24812,
                55488, 45344, 18676, 13092, 50828, 54464, 2888,  19456, 31424,
                40888, 21360, 44536, 37564, 33436, 57504, 21100, 45108, 9580,
                47816, 42364, 57252, 20156, 65028, 39876, 49268, 31816, 37640,
                12928, 54612, 35860, 45424, 58704, 11368, 56252, 29220, 7520,
                37376, 3520,  50840, 30584, 42816, 11776, 26012, 63564, 5124,
                30324, 3304,  18832, 43168, 32464, 37240, 21444, 49212, 33852,
                764,   42456, 65484, 35720, 28440, 49640, 43788, 19668, 28088,
                14620, 42336,
            },
            {
                4928,  38212, 16412, 61332, 4012,  53560, 31712, 58984, 40432,
                47660, 59984, 37432, 2088,  21596, 48928, 53224, 43256, 20700,
                46992, 29940, 1708,  18728, 11356, 64776, 28288, 59660, 17320,
                52440, 29356, 34520, 23236, 45056, 40020, 7936,  21424, 4904,
                26488, 12272, 52196, 2452,  64100, 48484, 42160, 53776, 11028,
                45472, 49660, 3300,  24932, 47032, 4688,  34488, 61644, 51236,
                36940, 57380, 2536,  23308, 35236, 47596, 30884, 52288, 11068,
                35972, 27616, 63928, 740,   25976, 62160, 38368, 48220, 57744,
                61444, 1132,  50400, 63148, 7808,  27032, 15408, 30188, 61832,
                17112, 28200, 736,   50896, 12060, 26660, 6212,  9808,  63056,
                42468, 5396,  30920, 64964, 6708,  26256, 48360, 19344, 63856,
                45508, 23392, 16460, 20456, 56232, 38940, 46788, 15928, 21588,
                56852, 35800, 65092, 6996,  55468, 4152,  62872, 8664,  17700,
                26940, 51276, 13124, 55012, 2616,  58812, 8804,  12056, 39908,
                2804,  63268,
            },
            {
                22244, 31000, 7920,  47144, 42620, 19092, 36652, 49808, 1536,
                20740, 11568, 25572, 64452, 34616, 29752, 17288, 25288, 6400,
                38444, 61524, 56568, 34632, 24716, 38176, 3260,  20864, 40136,
                43836, 13868, 38764, 9544,  50080, 960,   52972, 33120, 37664,
                43652, 56056, 34392, 17636, 27420, 6324,  30680, 34964, 18916,
                56516, 21964, 41224, 60428, 37348, 17252, 54436, 1952,  14432,
                25220, 60728, 16100, 39468, 54764, 7512,  44224, 21060, 1852,
                15580, 59284, 7076,  33824, 43160, 22644, 29288, 11396, 6716,
                34292, 14476, 24132, 11932, 48472, 41208, 54536, 5368,  34440,
                38072, 53864, 22720, 44128, 35664, 56060, 45500, 21320, 51328,
                27976, 2380,  49824, 17300, 22476, 38268, 42440, 4028,  32172,
                11084, 59160, 35444, 5824,  52364, 516,   29104, 8736,  49364,
                44528, 25284, 12976, 22940, 46620, 15152, 41364, 31112, 56052,
                38740, 9988,  23148, 32884, 18180, 38128, 63652, 46892, 55124,
                35284, 52272,
            },
            {
                44852, 12704, 59764, 34180, 26052, 14224, 63548, 7564,  28500,
                54276, 32660, 5948,  51172, 45936, 4016,  40860, 63036, 54352,
                10436, 26772, 5136,  49360, 45096, 15556, 51292, 33672, 8556,
                63896, 4620,  57952, 31376, 60432, 25448, 62368, 16448, 47904,
                1812,  8704,  21868, 39424, 59640, 50656, 13412, 61944, 2216,
                28660, 65208, 13900, 8752,  31304, 58120, 22344, 28964, 45496,
                33324, 5688,  49128, 28044, 63808, 12880, 26232, 60372, 39080,
                49032, 41340, 20244, 47300, 14744, 58504, 4476,  52700, 17556,
                43636, 55748, 30300, 59336, 19392, 2704,  64128, 23420, 49056,
                11280, 4712,  62828, 14728, 30012, 3920,  60032, 17684, 33908,
                56952, 23976, 44364, 13476, 34928, 62152, 51736, 14868, 24904,
                55484, 41252, 48528, 13704, 63112, 24228, 58852, 34436, 40108,
                1956,  53000, 31844, 39124, 58336, 28176, 52100, 24380, 3180,
                47720, 60632, 44288, 6180,  53124, 24552, 31420, 21356, 6752,
                16916, 25428,
            },
            {
                57816, 40788, 20140, 51696, 44,    56584, 39328, 22848, 46236,
                62320, 41820, 18672, 57140, 12740, 8824,  58760, 14272, 34132,
                44504, 17692, 59604, 40968, 7940,  30288, 56968, 12424, 48276,
                27012, 21708, 53684, 15772, 19808, 35784, 12808, 41716, 28580,
                65404, 58160, 31244, 45268, 3392,  36964, 23824, 44272, 7932,
                39368, 33412, 53364, 1176,  43040, 49324, 10984, 40832, 64448,
                9036,  53212, 20868, 32068, 808,   41568, 17660, 55772, 32744,
                5496,  29780, 53508, 56508, 9760,  39780, 32364, 20824, 65240,
                26712, 40052, 4044,  36388, 46196, 31968, 13604, 43076, 59892,
                19892, 32244, 58372, 8496,  38588, 24820, 52428, 41064, 14076,
                7948,  38808, 59736, 53192, 1480,  28868, 8712,  59888, 46972,
                1120,  28144, 9012,  31172, 44352, 37696, 19408, 11464, 62552,
                17540, 59800, 9316,  49984, 48,    18444, 11176, 64392, 34280,
                14460, 19936, 29168, 62660, 40880, 14776, 384,   56916, 28744,
                62124, 9812,
            },
            {
                50572, 5264,  29304, 65364, 10652, 44400, 30328, 17088, 10036,
                35508, 14668, 27252, 43904, 22788, 31052, 37488, 20184, 3456,
                50436, 32056, 13376, 21764, 53832, 63008, 2220,  23920, 61100,
                36576, 1732,  44564, 40460, 6568,  46372, 57024, 9696,  23844,
                18584, 50060, 13972, 25592, 54036, 9748,  20448, 58044, 52264,
                16212, 25736, 45676, 19432, 62316, 24624, 5144,  55292, 18208,
                36480, 13296, 43500, 59752, 10088, 48384, 35032, 3836,  22508,
                64848, 12340, 24596, 36552, 2120,  61024, 49912, 45972, 37048,
                8788,  51452, 16752, 10108, 53356, 26068, 39192, 7620,  52056,
                27372, 45664, 41152, 50028, 18940, 64240, 10752, 52,    48364,
                63636, 19476, 32604, 10644, 40736, 56152, 21056, 33660, 37632,
                18400, 53460, 22128, 60416, 16832, 7136,  51124, 54632, 30808,
                5848,  27280, 43556, 20836, 61540, 36588, 45680, 7372,  42680,
                58652, 36840, 1720,  49344, 9564,  45148, 51308, 36692, 42388,
                48368, 33020,
            },
            {
                2092,  37360, 23624, 16540, 36060, 53752, 3020,  58452, 51376,
                4744,  60264, 556,   38848, 63860, 52624, 47880, 27800, 55864,
                23828, 64376, 360,   36800, 27340, 18976, 46628, 42116, 16644,
                32084, 50660, 11648, 28108, 64472, 2596,  30600, 52376, 5680,
                36452, 40308, 4388,  60412, 16552, 63888, 41436, 31640, 47588,
                3952,  59864, 10416, 36048, 28268, 15488, 38412, 31828, 50620,
                26132, 3048,  56028, 37512, 23908, 52408, 62192, 14072, 46392,
                51488, 8336,  17892, 45088, 28024, 16028, 23996, 12300, 932,
                60036, 22988, 47740, 63736, 21492, 352,   57688, 17908, 36472,
                3116,  15912, 24192, 1752,  55164, 32504, 27760, 37108, 23052,
                45196, 29648, 5044,  24516, 48956, 16588, 4536,  43876, 11768,
                64408, 5208,  39536, 49836, 33296, 3460,  26728, 41292, 21900,
                47980, 38132, 15644, 33700, 5224,  55364, 30156, 21912, 50864,
                4752,  25568, 56540, 17176, 33864, 27080, 64328, 12568, 4352,
                19328, 15112,
            },
            {
                58716, 54484, 45772, 6628,  48772, 25432, 20528, 41600, 31704,
                24316, 48712, 55368, 33204, 6852,  16136, 2060,  61808, 42260,
                7384,  39104, 48412, 57800, 10628, 34228, 38920, 6304,  54084,
                9292,  59720, 23212, 55716, 33688, 49508, 21088, 43740, 61464,
                55232, 32224, 11408, 47360, 29572, 35208, 208,   27068, 12200,
                37256, 22604, 49872, 56000, 6984,  46980, 58672, 376,   61468,
                22024, 47556, 29936, 15236, 19008, 6672,  30524, 27380, 42892,
                38076, 59132, 32024, 62252, 52500, 7020,  42552, 30132, 53756,
                33312, 4860,  29200, 41268, 34804, 61292, 44704, 29520, 10048,
                65428, 53688, 35136, 12664, 46996, 42644, 16108, 58048, 5892,
                54888, 15656, 61576, 57640, 45884, 27452, 62568, 50568, 29972,
                57436, 25916, 43016, 12288, 56812, 65532, 45888, 13816, 2504,
                55884, 63820, 11644, 53424, 24056, 40856, 13592, 60284, 16700,
                32088, 53572, 12136, 39128, 59124, 20212, 7748,  23404, 60204,
                39168, 25996,
            },
            {
                8196,  41324, 62280, 11896, 32288, 60588, 5520,  13292, 64560,
                36632, 17432, 11060, 20488, 46316, 26160, 10060, 34652, 18292,
                11680, 29820, 16332, 44144, 5012,  52016, 14380, 58444, 28684,
                20172, 37876, 47016, 17820, 8028,  13960, 38900, 26860, 15492,
                848,   22768, 42584, 53016, 19884, 6832,  49644, 57088, 18744,
                64396, 42072, 4856,  32620, 63492, 21404, 12784, 44240, 8100,
                41280, 58028, 4492,  65080, 45816, 39848, 56844, 9548,  19896,
                144,   26608, 4588,  39484, 20524, 34540, 64060, 58092, 18792,
                14532, 43940, 56452, 12092, 7132,  15404, 49972, 23288, 56304,
                47716, 31104, 5708,  62372, 21440, 7672,  60764, 49856, 33800,
                11760, 42908, 36808, 3148,  34504, 12572, 39384, 23376, 7656,
                15736, 35824, 1792,  19708, 29144, 8612,  35056, 59544, 25148,
                32444, 7852,  45016, 28940, 49088, 63300, 2984,  26756, 39500,
                8396,  44072, 22844, 46904, 2696,  31216, 54312, 34668, 46380,
                52812, 30408,
            },
            {
                21328, 34220, 18528, 28128, 56840, 42984, 37736, 47868, 27872,
                7904,  53440, 42212, 29376, 56712, 60740, 40444, 50956, 45024,
                59416, 52892, 25812, 62048, 22816, 30944, 65328, 25088, 44048,
                4316,  63412, 168,   41884, 24792, 60380, 57516, 10216, 34728,
                48160, 63328, 8144,  26328, 38104, 62264, 14924, 43804, 8480,
                54224, 29596, 13700, 17948, 40344, 53316, 29988, 35264, 16664,
                27528, 11540, 34420, 51036, 25540, 2316,  60184, 36268, 53988,
                15848, 49940, 57156, 13536, 47988, 2908,  10524, 25836, 39728,
                48892, 62968, 25068, 20180, 54532, 32332, 4324,  38036, 13332,
                20020, 40348, 26080, 57536, 38364, 30420, 3732,  25240, 18864,
                52672, 26992, 9284,  21920, 52184, 19044, 560,   55140, 33096,
                44828, 51752, 61416, 47784, 24020, 38736, 17340, 50076, 42780,
                19512, 36244, 58100, 1224,  17864, 9916,  47072, 34948, 57648,
                49588, 65164, 28520, 6456,  61648, 48984, 13908, 42916, 1204,
                10968, 63440,
            },
            {
                44460, 14056, 1400,  50536, 22168, 9436,  16132, 55500, 1216,
                45588, 22964, 59396, 2936,  36912, 13664, 23528, 30744, 5408,
                21524, 36132, 3216,  8968,  41236, 47168, 1268,  11708, 49632,
                33072, 15172, 29884, 52836, 36204, 45220, 2960,  50628, 20332,
                29740, 56676, 17144, 45172, 2528,  55156, 23756, 39172, 33656,
                1556,  25932, 51712, 44608, 3124,  9592,  25056, 48948, 63756,
                54288, 38948, 20532, 9020,  43080, 13452, 32568, 22132, 47084,
                64320, 33636, 8784,  44220, 23412, 56048, 37176, 51104, 5600,
                9028,  35736, 1492,  38668, 46360, 26492, 59712, 42636, 1352,
                61476, 8884,  44420, 17656, 51848, 10500, 45552, 40800, 63800,
                1064,  39280, 48796, 65304, 31132, 59828, 42120, 10084, 63192,
                20588, 5756,  30888, 14844, 55544, 52544, 396,   10892, 62220,
                5060,  14792, 51240, 21968, 38504, 32324, 54816, 20492, 11624,
                760,   19236, 15132, 35060, 41636, 25024, 18172, 57340, 28116,
                16792, 49144,
            },
            {
                5612,  61032, 53232, 39712, 4340,  62960, 34468, 19976, 61004,
                39476, 14184, 33648, 6208,  48124, 18760, 53876, 868,   64632,
                14688, 55780, 49236, 33340, 57256, 17056, 21020, 36308, 54924,
                40120, 57440, 10496, 62180, 5152,  27672, 17412, 64232, 6804,
                41024, 13120, 35836, 59448, 32516, 12632, 30176, 60984, 20976,
                48476, 62612, 35460, 22992, 60104, 37820, 57412, 1976,  19272,
                5120,  45516, 31676, 62056, 55304, 17940, 49572, 3424,  11312,
                25100, 41964, 18268, 28368, 62716, 31308, 16748, 43376, 21592,
                28716, 57908, 17248, 51468, 61984, 9792,  19116, 53084, 28760,
                33920, 50872, 3188,  14372, 28184, 55496, 35468, 14036, 22336,
                31924, 56072, 17476, 6696,  45332, 14736, 28656, 48656, 25144,
                38264, 58496, 11240, 41136, 6780,  44196, 21464, 31800, 28324,
                54132, 40560, 64932, 26916, 43488, 7056,  61700, 24740, 41992,
                31092, 37784, 55008, 58384, 10632, 4568,  38688, 64672, 6876,
                37164, 25224,
            },
            {
                56592, 31360, 36432, 24764, 45948, 30284, 49352, 26392, 10884,
                31136, 50260, 64024, 25640, 43436, 62116, 8864,  33492, 46856,
                40028, 28236, 19428, 12684, 37920, 52664, 61436, 27512, 7052,
                17956, 23948, 48676, 20712, 13324, 32456, 55016, 37652, 46780,
                24288, 51568, 5640,  21488, 48916, 9424,  52048, 4136,  45824,
                16032, 11088, 56656, 6448,  47060, 15264, 32856, 43020, 51648,
                13788, 59680, 23956, 952,   28696, 37128, 63064, 41092, 30304,
                6100,  60464, 37776, 4876,  53340, 12156, 568,   60928, 54404,
                45956, 32772, 11604, 41660, 30056, 5436,  36504, 16092, 47400,
                24012, 58648, 36984, 64308, 23012, 1520,  61664, 47604, 8292,
                59216, 44156, 12124, 24872, 37532, 57780, 5188,  53416, 17332,
                3056,  46396, 27756, 63956, 33724, 60356, 37152, 57336, 47840,
                23868, 9108,  31384, 12404, 59228, 16304, 45644, 4020,  51792,
                63144, 7336,  43752, 21736, 50588, 29404, 53692, 20804, 32168,
                51796, 40624,
            },
            {
                22360, 11564, 16900, 7800,  59064, 13612, 2412,  54008, 43284,
                4404,  21560, 16536, 52856, 11260, 28136, 38604, 57284, 25356,
                6532,  43704, 63272, 2040,  29244, 5284,  42756, 9776,  59324,
                45396, 5816,  35088, 43392, 51424, 40588, 22880, 11132, 30792,
                61884, 3244,  28204, 39472, 65188, 42056, 18456, 27684, 58600,
                7632,  39592, 31248, 19652, 27716, 65336, 10848, 22012, 29600,
                37368, 7844,  48664, 15704, 52780, 5032,  26516, 58288, 45064,
                54696, 14020, 51136, 20948, 46540, 40544, 26436, 34984, 13744,
                3964,  24440, 63704, 2472,  22692, 49348, 55964, 64760, 6508,
                11120, 20780, 30768, 47068, 42372, 33136, 17988, 53296, 27436,
                4936,  35820, 29568, 61544, 1704,  21288, 34760, 40476, 12628,
                56384, 36020, 22540, 2172,  18884, 25828, 13148, 3628,  16648,
                44780, 1660,  35548, 49512, 3060,  53292, 36064, 13216, 27964,
                17188, 47204, 25888, 2512,  33024, 62164, 12444, 45252, 9228,
                59724, 3232,
            },
            {
                28924, 63880, 42900, 55644, 20716, 41412, 65112, 17936, 36032,
                58136, 8220,  41068, 35516, 1804,  20248, 49876, 12992, 17708,
                51536, 10128, 22584, 58620, 46024, 15280, 32932, 22188, 51656,
                31036, 64676, 28384, 2208,  56888, 7784,  59716, 1012,  15416,
                43560, 58128, 54412, 16180, 25124, 912,   34780, 54812, 37220,
                24616, 42944, 53944, 2136,  49784, 40716, 5836,  55508, 61568,
                26460, 41936, 56644, 32928, 40524, 12412, 20384, 8744,  16584,
                23256, 35456, 1948,  58848, 33068, 8052,  64344, 48556, 19464,
                59320, 37252, 53612, 15100, 44188, 34008, 12752, 25956, 45136,
                41140, 54908, 4368,  9844,  57288, 6632,  12500, 38820, 20596,
                49612, 15468, 54292, 40904, 51672, 47848, 8940,  27020, 64736,
                31796, 50220, 15852, 53304, 47116, 7504,  50732, 63320, 39000,
                20676, 61132, 55268, 25164, 19916, 29880, 39528, 22464, 60400,
                33976, 10104, 59496, 14608, 40140, 47480, 836,   24152, 35028,
                15184, 46624,
            },
            {
                50272, 240,   32864, 48560, 27848, 37840, 6492,  23864, 47044,
                51556, 27196, 61536, 55720, 46580, 30164, 59068, 3720,  36304,
                61704, 31196, 38884, 53532, 25104, 50012, 62892, 39320, 13740,
                3296,  37344, 15348, 60920, 25656, 17800, 44784, 49436, 35392,
                19460, 8620,  33056, 11476, 47368, 59776, 45092, 12856, 5016,
                64244, 17780, 13620, 61108, 35084, 24880, 17192, 46464, 34272,
                1604,  18280, 10376, 21740, 60680, 46688, 51336, 37996, 32724,
                49096, 63716, 29024, 10856, 24468, 17524, 56448, 29952, 6232,
                39916, 9152,  47496, 27720, 61136, 20368, 39724, 196,   32392,
                61764, 14920, 38260, 18740, 26124, 50300, 60328, 30368, 42148,
                62952, 2376,  23196, 10356, 18284, 32240, 60596, 44076, 19632,
                6460,  42084, 9088,  39388, 59240, 30460, 42740, 10044, 33052,
                51952, 13752, 40964, 5340,  46384, 63992, 8592,  56984, 48800,
                5084,  54344, 36740, 19700, 53144, 27232, 16980, 60716, 41720,
                54136, 19352,
            },
            {
                38360, 60868, 10264, 4880,  53044, 11660, 33428, 60296, 15140,
                544,   31768, 12400, 22856, 5040,  64920, 42060, 23460, 45492,
                55144, 15464, 4436,  34816, 8708,  19104, 920,   55052, 26140,
                48312, 19564, 53840, 9468,  39136, 33796, 28916, 64844, 53476,
                26120, 41824, 49888, 37120, 20132, 7424,  30644, 22124, 48644,
                33004, 51312, 28948, 44568, 8128,  58692, 52888, 4184,  13024,
                50912, 63176, 45456, 35888, 6380,  29564, 65244, 592,   56328,
                4684,  19384, 42292, 38320, 50164, 44028, 3480,  22664, 42812,
                51288, 31628, 18420, 6608,  56896, 4580,  52352, 58748, 17624,
                28052, 48400, 52560, 63168, 34932, 44112, 23704, 3636,  10012,
                57560, 32984, 45760, 64400, 25928, 4380,  14080, 55036, 16,
                23972, 61640, 56860, 28240, 13952, 35616, 22676, 56144, 26720,
                6340,  29120, 57396, 17108, 33740, 11864, 43132, 420,   19076,
                41444, 29764, 45160, 4064,  64196, 8224,  37740, 56272, 30508,
                5432,  12744,
            },
            {
                44596, 22732, 35776, 25600, 18552, 45604, 55960, 29620, 9632,
                38376, 45248, 18128, 34276, 39432, 14980, 8596,  33064, 27732,
                464,   21032, 42492, 65120, 12868, 29860, 44604, 35368, 10920,
                57864, 41216, 45856, 23224, 50576, 12016, 3924,  21864, 5800,
                13472, 61048, 1972,  28564, 62932, 55724, 15608, 39852, 60480,
                9888,  332,   21480, 38436, 11952, 31668, 20468, 37680, 43600,
                23248, 30740, 58544, 2912,  54528, 24188, 15332, 43660, 25512,
                12872, 46988, 7012,  54204, 61144, 13308, 36532, 53064, 15516,
                25732, 65352, 12000, 35556, 42168, 30376, 10488, 22900, 36676,
                8132,  2944,  21852, 29844, 864,   14528, 55240, 37040, 46852,
                26584, 17568, 7360,  36052, 42956, 58668, 38856, 49868, 30612,
                46160, 12268, 34372, 20892, 1336,  64948, 4564,  18304, 45912,
                62668, 37764, 9736,  48180, 60012, 27084, 54928, 32144, 14884,
                24864, 61076, 11964, 22956, 31380, 50384, 43520, 3716,  21656,
                65332, 26652,
            },
            {
                57604, 51092, 15368, 58472, 62856, 1848,  40408, 49628, 19420,
                64176, 52452, 58588, 7252,  50296, 54240, 19020, 48272, 11412,
                60152, 50752, 26112, 47892, 56172, 40392, 60832, 7412,  21576,
                63676, 5580,  32048, 632,   62824, 54988, 46692, 40292, 31544,
                37828, 56904, 23304, 6088,  51012, 42672, 25504, 3604,  57060,
                26976, 46708, 62820, 54756, 3680,  48040, 26948, 64048, 55968,
                6928,  15944, 26224, 39032, 19144, 9728,  34240, 40220, 59192,
                31160, 62444, 27356, 16516, 1404,  30688, 58420, 9876,  61980,
                880,   44328, 55420, 24184, 50120, 16444, 64032, 46416, 51344,
                42432, 60880, 11900, 45696, 40508, 8564,  64504, 19748, 51048,
                13268, 39768, 52728, 56228, 11844, 29100, 22396, 16480, 36768,
                52688, 26200, 5544,  49248, 44832, 53900, 38344, 48768, 14784,
                776,   23644, 42512, 20996, 2488,  37196, 23448, 49832, 63544,
                38236, 7028,  48064, 57896, 35156, 14216, 25440, 11140, 48168,
                34544, 8172,
            },
            {
                30956, 2636,  39044, 43308, 31888, 13800, 22156, 4668,  26672,
                42464, 24060, 2680,  29236, 43592, 26380, 62572, 36896, 56392,
                39796, 7432,  17648, 31980, 5876,  23600, 15876, 47500, 28672,
                34052, 17148, 13252, 27456, 36220, 19924, 16064, 58448, 10328,
                47876, 18088, 44896, 14740, 32228, 10988, 34884, 53272, 19168,
                36772, 41932, 15916, 24360, 59696, 41400, 14908, 9168,  33432,
                40612, 49312, 11976, 52432, 44748, 62308, 48840, 6560,  21264,
                52524, 36124, 9336,  23124, 34316, 48228, 20396, 27832, 39016,
                33520, 5712,  20628, 60492, 38012, 1908,  26788, 34184, 13564,
                56296, 25304, 32676, 59024, 53324, 16784, 28344, 34200, 6192,
                61856, 30604, 1016,  20908, 47384, 3032,  62336, 7256,  10028,
                63708, 18492, 39820, 60544, 16204, 25140, 11208, 29672, 60208,
                35276, 50556, 31268, 65380, 52132, 15972, 7828,  44928, 3160,
                52632, 28508, 17404, 41088, 2012,  62400, 18980, 52112, 59360,
                40248, 17520,
            },
            {
                53256, 6644,  21160, 9200,  28056, 52304, 36748, 60988, 56792,
                8024,  14200, 37216, 60612, 11892, 21916, 2244,  6372,  29428,
                13736, 44692, 35752, 61880, 2768,  51456, 36948, 59000, 3776,
                52968, 49692, 56740, 44244, 59912, 7392,  42600, 25872, 63508,
                3380,  29628, 54872, 64600, 38744, 1052,  61752, 43992, 8016,
                13040, 31264, 5348,  34712, 18636, 51256, 2168,  60600, 28636,
                18140, 120,   64520, 31576, 4304,  27692, 55784, 11632, 17460,
                1740,  44920, 39420, 55732, 64816, 41384, 6732,  46008, 17564,
                49448, 53588, 29476, 13864, 8072,  44856, 57260, 3948,  20120,
                5980,  39084, 17868, 4840,  23204, 49036, 58232, 1992,  22272,
                41796, 58904, 24784, 15228, 34584, 52032, 40672, 32520, 43952,
                57276, 1904,  30024, 55656, 8716,  33180, 43304, 57660, 20272,
                8400,  54692, 12920, 5140,  28548, 40532, 58240, 20728, 13084,
                34000, 56556, 10176, 21748, 54232, 44376, 36836, 28676, 988,
                13996, 46144,
            },
            {
                27548, 61852, 56116, 49068, 64360, 6524,  47120, 16912, 34640,
                31236, 54864, 47756, 17136, 33732, 57356, 46020, 51884, 16788,
                64348, 22792, 53320, 10896, 27896, 19424, 43244, 12592, 24832,
                41532, 37468, 9160,  24180, 30652, 2800,  51872, 33644, 22516,
                50876, 35568, 7332,  26272, 21292, 47696, 16532, 23792, 48972,
                65116, 52204, 57884, 10188, 29512, 37292, 45328, 21624, 53880,
                46536, 57868, 34992, 42608, 14264, 22120, 37072, 61236, 28972,
                50388, 58592, 13772, 19024, 4268,  25392, 12428, 56284, 60068,
                3088,  11020, 40604, 47568, 32500, 22492, 62580, 41392, 30844,
                47196, 65068, 51000, 36224, 10420, 31208, 38692, 45180, 54856,
                10832, 36640, 49432, 63464, 6028,  26988, 58324, 19284, 23536,
                48236, 14632, 42480, 36312, 21976, 2632,  52332, 5772,  40324,
                26464, 44716, 18404, 59104, 46428, 10936, 35212, 62576, 25696,
                41852, 4960,  65024, 32116, 27112, 5540,  9460,  56480, 32752,
                63188, 23692,
            },
            {
                10900, 41084, 34676, 18424, 12208, 24332, 41416, 1364,  11612,
                45392, 21392, 5468,  63760, 39304, 9500,  31084, 41556, 25296,
                34156, 1456,  58364, 38568, 46664, 56444, 32940, 8660,  61572,
                20516, 2072,  64968, 18448, 47708, 39504, 14188, 55744, 9076,
                15828, 44052, 40688, 12368, 52072, 58720, 30456, 5088,  28208,
                20100, 2468,  43328, 48328, 63692, 6136,  25404, 11440, 36144,
                5704,  24560, 20056, 8228,  60024, 46448, 2612,  41692, 33712,
                23520, 5096,  31812, 42948, 49200, 52756, 37204, 31276, 21472,
                35724, 27136, 63428, 58020, 18160, 52708, 10252, 15396, 55080,
                8544,  27324, 13988, 43540, 63040, 3800,  25720, 14904, 48080,
                18468, 28188, 8668,  43584, 17796, 46060, 12844, 54680, 5212,
                35008, 27152, 11364, 62076, 49948, 28280, 64256, 16804, 34044,
                60996, 2796,  38616, 32740, 24804, 56332, 1228,  30772, 51228,
                18112, 45916, 37340, 49108, 15460, 60820, 46828, 20980, 42308,
                4464,  37084,
            },
            {
                47476, 15536, 1104,  44424, 32412, 54516, 58948, 29176, 63360,
                40008, 51400, 28064, 348,   24496, 53676, 19636, 5236,  59668,
                49152, 9032,  30336, 21276, 14012, 64192, 252,   54264, 30544,
                46224, 15084, 33400, 54128, 11256, 62648, 27736, 37520, 76,
                61896, 19568, 59704, 2604,  33916, 8912,  41232, 54308, 60576,
                34192, 39628, 26028, 22104, 14088, 52684, 57696, 42856, 61936,
                14560, 51576, 29484, 38488, 54476, 25768, 16080, 53012, 8972,
                47296, 62912, 26884, 10596, 61000, 676,   15948, 63920, 7380,
                43632, 14592, 24084, 6248,  716,   38896, 28296, 36420, 49120,
                21300, 34836, 468,   54192, 19208, 41148, 60624, 7220,  33108,
                65440, 3704,  56688, 31436, 37460, 61316, 780,   30576, 39272,
                65104, 51732, 3812,  46252, 18704, 38524, 12312, 45788, 23128,
                49232, 14336, 63216, 7344,  20388, 48456, 16416, 43648, 8872,
                61372, 23396, 12512, 180,   52464, 39340, 24296, 13032, 50160,
                17984, 55344,
            },
            {
                31260, 22764, 60348, 26332, 38308, 4480,  19988, 50308, 23432,
                9276,  18536, 59344, 42248, 49316, 14764, 62236, 37712, 12676,
                43964, 18476, 52192, 41076, 6744,  26280, 16772, 39828, 49332,
                27236, 58244, 6720,  42968, 21220, 4572,  58040, 45296, 31320,
                48752, 23900, 29344, 56108, 45776, 18604, 37008, 13528, 45144,
                10584, 16312, 55740, 7668,  38516, 33184, 1048,  17296, 30948,
                39944, 2352,  63560, 11284, 48728, 6292,  30748, 64900, 19812,
                38220, 15324, 54076, 21036, 33900, 29212, 44980, 23564, 39236,
                52172, 55676, 45756, 34708, 49652, 61456, 24536, 44632, 2988,
                57004, 61760, 23760, 46840, 28496, 56880, 11636, 21340, 52680,
                42624, 23800, 14040, 51372, 21636, 9896,  25584, 49792, 15760,
                8288,  20924, 59312, 24828, 7532,  56532, 31848, 368,   53528,
                9932,  29724, 42180, 51928, 36484, 4392,  54144, 37992, 27780,
                3572,  55860, 29900, 59140, 18720, 33608, 7040,  64516, 29644,
                58924, 8276,
            },
            {
                52772, 3900,  48720, 56408, 8740,  46492, 14724, 35680, 3516,
                56996, 32508, 13136, 7124,  34796, 28876, 46588, 1844,  55828,
                27748, 36572, 4344,  61036, 45216, 50592, 35980, 22308, 5092,
                11652, 38148, 52068, 29020, 35640, 50072, 24692, 18116, 11928,
                53640, 39256, 6572,  14444, 27360, 64340, 4072,  25252, 49852,
                472,   59296, 30828, 47100, 61600, 19696, 27708, 50108, 9392,
                47184, 56224, 17968, 43296, 21128, 35732, 57016, 12532, 44228,
                2008,  57472, 35604, 7732,  40708, 59180, 11576, 48924, 2684,
                18256, 9712,  30260, 20668, 42396, 12900, 6920,  59856, 18004,
                12456, 39688, 32388, 9172,  16056, 34128, 49772, 37796, 29576,
                600,   59440, 39740, 4740,  55596, 42228, 35492, 58816, 44176,
                54056, 29000, 40992, 34204, 15160, 43708, 61408, 26024, 58708,
                35588, 21432, 57204, 26836, 12612, 64748, 22888, 59932, 14392,
                47628, 35140, 40596, 9752,  26152, 43012, 53668, 36232, 1452,
                25636, 38944,
            },
            {
                63612, 42488, 13160, 17672, 65032, 30292, 42032, 62224, 26536,
                38600, 53852, 44732, 65216, 22140, 58052, 10316, 32888, 23928,
                63912, 14936, 54984, 23628, 33880, 9780,  57624, 62000, 43848,
                19180, 63436, 23812, 1612,  61268, 14564, 7720,  41984, 65344,
                3992,  33300, 60844, 41896, 51076, 22524, 55212, 32712, 62520,
                21332, 36688, 51520, 3728,  11116, 41440, 54584, 65464, 22504,
                34596, 26524, 4112,  33132, 59620, 824,   40080, 51140, 25096,
                28644, 5784,  50336, 46216, 16828, 4988,  56456, 27952, 62312,
                33044, 57856, 3888,  65172, 16656, 54444, 33196, 51808, 29276,
                43008, 5852,  50776, 60280, 2192,  44756, 5252,  62464, 16476,
                46576, 35176, 26664, 47792, 16348, 63740, 2864,  22408, 11672,
                32876, 17664, 1644,  55300, 50276, 4528,  20356, 40748, 15936,
                5052,  47752, 1624,  17152, 45212, 9164,  33544, 41168, 19580,
                6980,  51860, 20928, 45176, 62688, 4508,  16624, 47976, 11460,
                45140, 20208,
            },
            {
                6152,  34364, 28068, 36876, 22176, 51592, 11224, 6168,  48092,
                16096, 1880,  25152, 37124, 17792, 3624,  51660, 42164, 19876,
                7284,  48304, 31740, 1192,  17840, 28416, 12972, 3696,  32140,
                55024, 8504,  16824, 40676, 46072, 32356, 56288, 27204, 36728,
                21532, 9796,  17360, 47432, 1108,  11024, 38332, 15764, 8176,
                42556, 26648, 17812, 44796, 24648, 35812, 4448,  15308, 7036,
                44544, 61052, 13396, 50700, 28364, 23372, 17072, 8272,  33280,
                62140, 42284, 22356, 64548, 25692, 52420, 20176, 36624, 13808,
                41124, 25840, 50920, 37880, 5636,  27308, 21692, 9520,  35964,
                64480, 25896, 19940, 53768, 40448, 22684, 26804, 54740, 8524,
                19904, 12020, 61900, 7184,  33348, 19488, 28400, 46832, 5036,
                60932, 47888, 37896, 63576, 11148, 29924, 36956, 8436,  51152,
                44148, 62536, 31720, 39980, 52952, 28808, 49708, 844,   58132,
                32248, 64416, 15212, 2256,  55088, 31824, 22332, 57244, 60924,
                33040, 15200,
            },
            {
                23892, 50128, 61276, 7544,  164,   54332, 34048, 18912, 43324,
                60772, 30804, 10976, 47248, 54828, 40036, 15676, 61232, 45592,
                38244, 11388, 58312, 43224, 65444, 52912, 47388, 40896, 24724,
                50288, 35072, 48084, 59244, 10296, 20472, 51412, 2252,  47820,
                58204, 52320, 25892, 35256, 30776, 58344, 44092, 47964, 29116,
                57120, 5336,  64300, 14296, 56128, 59780, 29964, 48520, 39208,
                53580, 19016, 37600, 9508,  63296, 44772, 54708, 59028, 48480,
                18620, 13888, 30700, 9352,  38476, 32684, 2164,  43396, 7856,
                46984, 21552, 11692, 59684, 48044, 41044, 57304, 45524, 1060,
                48764, 14900, 30716, 10556, 36160, 63840, 13052, 32076, 42960,
                57756, 23116, 53332, 30140, 10940, 50932, 55048, 36792, 41528,
                24556, 13844, 7744,  26556, 45256, 22500, 53812, 65204, 27464,
                18960, 11552, 24212, 59196, 5472,  61496, 13416, 25360, 42940,
                10408, 23620, 38452, 28260, 48932, 12716, 39652, 27424, 8952,
                41548, 55808,
            },
            {
                2556,  11780, 45572, 57988, 40312, 25264, 59664, 28936, 50508,
                21828, 52748, 5260,  61916, 27824, 8332,  31076, 25420, 2712,
                53388, 29544, 21508, 39684, 8088,  36760, 20692, 15352, 59920,
                816,   27812, 13232, 30592, 4944,  38760, 63580, 16248, 30112,
                13192, 43176, 62720, 7676,  20612, 63776, 4836,  18988, 53104,
                12164, 39244, 30468, 34016, 1464,  21520, 43672, 11580, 25380,
                31604, 5452,  57508, 47524, 14960, 4376,  36660, 11628, 3240,
                39008, 60580, 488,   55100, 12304, 48248, 63336, 24096, 54780,
                60756, 116,   35660, 29616, 14752, 2588,  62896, 17196, 23820,
                56388, 38232, 4076,  58632, 46964, 17448, 51028, 3384,  39048,
                49256, 1684,  36868, 44492, 40824, 60160, 14448, 8152,  64640,
                31064, 52508, 57920, 18348, 33708, 59952, 1208,  13260, 33188,
                38544, 55064, 7912,  34764, 21936, 37932, 18380, 54416, 29608,
                48524, 56684, 6084,  61172, 35780, 7200,  52168, 19508, 4228,
                47188, 30004,
            },
            {
                52400, 38484, 18376, 31872, 14996, 44584, 3676,  13300, 8792,
                35844, 41156, 14872, 34020, 20036, 49840, 56572, 13004, 34916,
                63364, 4872,  15780, 50972, 26540, 2856,  56280, 30488, 6428,
                37940, 64772, 21948, 43480, 53556, 25064, 34388, 44640, 5912,
                23876, 38372, 3444,  54064, 41100, 12776, 25040, 35540, 1680,
                61516, 23164, 45860, 52668, 9656,  50200, 18232, 62448, 56604,
                484,   21284, 40704, 24240, 32476, 42152, 20460, 30360, 45928,
                24568, 51812, 33972, 44036, 59412, 18480, 29424, 15044, 50000,
                31344, 17912, 45076, 53592, 24636, 34164, 10664, 39564, 32072,
                7004,  62112, 43852, 21876, 28092, 7600,  56360, 25012, 60532,
                28668, 14696, 64304, 5548,  21192, 2180,  26920, 18080, 49668,
                12,    21144, 42496, 3576,  49536, 15636, 41604, 46920, 57272,
                5780,  50224, 43364, 15108, 48208, 3200,  44912, 63472, 4276,
                16904, 33692, 45988, 18996, 24824, 59500, 43424, 65248, 37428,
                16400, 62652,
            },
            {
                21336, 56780, 26972, 9296,  49376, 20004, 63196, 38404, 56808,
                916,   64388, 24612, 58260, 42604, 6228,  37556, 59976, 17964,
                49176, 24204, 44788, 60792, 11996, 34428, 62816, 42648, 46276,
                18572, 51144, 7444,  56636, 17620, 1080,  12152, 54860, 62208,
                19396, 32060, 46132, 15988, 28536, 48976, 56536, 32132, 41980,
                49680, 19928, 6564,  60816, 41020, 28124, 36936, 7540,  33768,
                50944, 64524, 29352, 7408,  60420, 53420, 26612, 65228, 6664,
                56340, 20752, 15376, 27312, 6200,  40500, 4148,  34912, 10672,
                39512, 5524,  64124, 8312,  20044, 59040, 46784, 28248, 52276,
                19356, 11384, 49992, 33500, 8,     42252, 35552, 19624, 10288,
                34336, 17904, 47424, 25320, 57544, 35152, 45840, 39824, 56940,
                10624, 36180, 28360, 62544, 38980, 9744,  31044, 25084, 3072,
                21116, 29528, 64120, 26320, 56512, 31056, 9600,  35524, 40328,
                12764, 52568, 1316,  41780, 10156, 14800, 29516, 448,   49788,
                25980, 35480,
            },
            {
                6624,  43136, 1548,  60248, 35292, 54808, 30976, 23280, 47348,
                17876, 28232, 45488, 2848,  12408, 23444, 46480, 212,   28336,
                41672, 7836,  32488, 55360, 19012, 48704, 23552, 9024,  12440,
                58284, 25760, 40156, 35404, 61184, 49616, 28192, 41012, 9092,
                50760, 56980, 11352, 60304, 22380, 37844, 8928,  59608, 14768,
                27304, 10456, 37436, 15524, 24216, 3228,  54868, 14028, 42560,
                16424, 45468, 12280, 48904, 17952, 1144,  50192, 13512, 35304,
                41588, 8772,  48872, 37260, 22548, 53028, 45440, 62020, 55680,
                22688, 27796, 57092, 37284, 42576, 50720, 3584,  13964, 55552,
                41348, 26064, 15776, 54116, 65236, 13812, 48376, 62584, 4940,
                52608, 41648, 7736,  54160, 13248, 29536, 62956, 23040, 32992,
                15344, 44664, 54800, 5508,  23772, 52260, 58992, 17472, 62924,
                36648, 13524, 40468, 128,   19744, 51020, 59880, 22668, 27276,
                58732, 20904, 63212, 31316, 50636, 56160, 34424, 23292, 58216,
                9616,  13316,
            },
            {
                33272, 47656, 63768, 22212, 4336,  11180, 41800, 6760,  53160,
                32880, 9644,  51300, 38576, 55324, 63852, 31996, 20744, 54504,
                10636, 59212, 38652, 1572,  29976, 5740,  39156, 54100, 28920,
                33364, 2460,  15532, 10704, 31616, 46792, 22484, 58860, 36900,
                2876,  26404, 34860, 596,   51816, 5820,  18136, 45324, 3544,
                64512, 54104, 47056, 58480, 32280, 63744, 47692, 22840, 60084,
                26592, 2784,  37400, 55696, 34136, 10136, 38832, 57740, 17024,
                28816, 62588, 3484,  57588, 64420, 9584,  26624, 19840, 1528,
                43052, 51772, 16240, 12196, 31440, 6308,  22596, 64708, 34704,
                1508,  60684, 37316, 29984, 9672,  23848, 39276, 30348, 44612,
                22648, 59032, 32424, 38596, 49796, 9660,  3308,  53156, 5220,
                60712, 25176, 47464, 13180, 19628, 35308, 6992,  42688, 48996,
                53636, 8624,  59108, 46300, 11316, 38704, 15960, 6936,  49408,
                43528, 8648,  26072, 39148, 4788,  46648, 17972, 6060,  45100,
                40936, 54440,
            },
            {
                27864, 16856, 39408, 52088, 28988, 46696, 16196, 26764, 59020,
                13764, 61204, 19644, 30092, 15864, 9000,  43696, 50948, 14480,
                36004, 47292, 22096, 13540, 63524, 43608, 16496, 61800, 20972,
                48264, 52948, 62624, 45232, 3652,  19724, 6988,  14328, 30840,
                17172, 48420, 40660, 64928, 43572, 29732, 55392, 23664, 40184,
                30724, 34848, 68,    18332, 44012, 11376, 5300,  39676, 30704,
                9492,  58484, 20052, 24972, 63684, 43428, 22680, 46888, 2280,
                53584, 32492, 44672, 12996, 30640, 16864, 49504, 37964, 13748,
                59428, 33596, 4176,  48740, 26228, 60156, 38284, 44124, 9348,
                47268, 21364, 6488,  44436, 58292, 51844, 16780, 2332,  55412,
                12128, 27336, 540,   19912, 60076, 16876, 42572, 48172, 20260,
                38748, 7888,  32360, 64888, 56848, 29660, 45780, 11548, 27044,
                32736, 16572, 24076, 33964, 52864, 28152, 65476, 32300, 1980,
                54776, 36992, 15800, 57468, 22608, 12024, 53076, 62352, 30928,
                20112, 59960,
            },
            {
                4612,  10172, 24664, 13632, 36456, 56432, 65480, 1964,  39132,
                43796, 35936, 3808,  48440, 26356, 35016, 60548, 5196,  24672,
                65012, 3644,  27444, 52512, 34864, 50020, 26192, 460,   36436,
                5216,  41940, 23168, 27948, 57188, 39212, 64564, 53308, 43924,
                61352, 23364, 8180,  19612, 14244, 33996, 61096, 11076, 50712,
                20660, 6316,  25820, 56672, 29088, 35992, 51560, 18624, 53828,
                35312, 48076, 41700, 4180,  14268, 30228, 6216,  61928, 25816,
                10712, 19032, 23944, 36072, 42312, 2724,  56804, 32216, 47408,
                8548,  24844, 40512, 62620, 19340, 53008, 15000, 27712, 18036,
                31204, 57332, 49568, 3396,  19492, 27632, 7168,  60016, 36712,
                47140, 15520, 64992, 43880, 24544, 31024, 36448, 12728, 57416,
                28980, 51356, 17232, 908,   40680, 4024,  61124, 21564, 38872,
                1628,  62024, 44372, 6300,  57772, 3936,  42080, 47136, 18696,
                62092, 29636, 3432,  44444, 64368, 35500, 27352, 39596, 14588,
                2016,  50676,
            },
            {
                42420, 32384, 58828, 45356, 7180,  18652, 33404, 50832, 21304,
                5792,  24388, 52620, 62860, 41260, 1592,  18544, 39572, 30196,
                55840, 16888, 41200, 57360, 19964, 8856,  60112, 14176, 44648,
                59308, 18200, 8108,  36776, 12464, 50044, 33592, 26908, 4812,
                11744, 55224, 31884, 58148, 25528, 4048,  46484, 36820, 16188,
                62036, 42316, 52444, 14240, 8212,  21756, 60688, 1096,  13428,
                63324, 7104,  31708, 53184, 49928, 59084, 19616, 33460, 37660,
                50788, 59844, 47592, 7584,  52224, 21072, 61264, 6352,  28168,
                65460, 18244, 45012, 1928,  35420, 8004,  56420, 2428,  51888,
                61892, 12544, 25400, 40320, 34432, 63016, 42344, 32192, 20424,
                8796,  51464, 34724, 4332,  55612, 6676,  63660, 25988, 1608,
                61776, 10816, 43536, 36616, 26364, 53716, 15548, 50936, 5172,
                55572, 29364, 18608, 37296, 21772, 14612, 25016, 35892, 12624,
                23480, 9924,  53372, 32728, 19824, 1036,  8468,  48424, 24580,
                35996, 64704,
            },
            {
                21988, 48868, 54032, 412,   62184, 42720, 25324, 8860,  47940,
                30904, 56308, 12264, 7908,  22076, 58064, 53732, 9968,  49740,
                33504, 7376,  45800, 11308, 3000,  32236, 39752, 54824, 29728,
                11040, 32496, 63976, 54972, 16556, 24464, 72,    41524, 19244,
                46640, 37356, 2440,  50244, 39324, 53392, 7928,  28328, 2304,
                57408, 11784, 37596, 48736, 65276, 41432, 46104, 24956, 28464,
                42912, 22572, 17044, 38312, 27484, 8896,  45596, 55160, 4120,
                40528, 15028, 700,   56156, 26144, 39772, 11884, 44180, 22172,
                36872, 53548, 10852, 58016, 29196, 47620, 39880, 33088, 24064,
                41836, 36100, 16488, 54452, 10648, 46212, 13464, 49732, 25336,
                62404, 41164, 28488, 11516, 46288, 52008, 18776, 40996, 45724,
                34384, 23332, 56364, 20788, 47736, 9144,  33252, 24640, 63672,
                41772, 12372, 50616, 60344, 47972, 9504,  56112, 60896, 51416,
                41128, 46528, 59356, 13484, 49904, 42304, 60260, 54664, 11264,
                56476, 17856,
            },
            {
                38584, 8112,  15616, 30424, 20944, 38080, 11868, 59748, 15432,
                63540, 39896, 18148, 44948, 31456, 47104, 28140, 13176, 44104,
                21108, 60708, 37228, 25116, 64264, 47672, 21880, 6700,  49116,
                24208, 51268, 40384, 6076,  47420, 61128, 9836,  58628, 52240,
                28984, 63616, 16468, 44448, 12384, 21112, 62908, 48848, 24712,
                32892, 45264, 22884, 30396, 2764,  16568, 32764, 57236, 49288,
                3944,  55388, 10916, 61684, 228,   41196, 16580, 12472, 28608,
                22092, 64532, 34564, 29708, 62796, 16232, 33792, 54284, 968,
                15068, 49140, 31736, 23776, 13668, 20884, 64164, 10572, 45848,
                4116,  7752,  63824, 30812, 1380,  22968, 56264, 4920,  38780,
                1800,  17884, 56744, 23068, 38224, 14316, 33008, 8828,  54604,
                15740, 49624, 6176,  63392, 13672, 59872, 43908, 18948, 35592,
                47284, 8044,  25964, 2276,  32820, 43432, 28516, 588,   7396,
                31192, 16884, 26340, 6244,  37136, 28956, 16076, 32124, 3580,
                45004, 29156,
            },
            {
                12668, 63280, 35332, 26520, 3612,  51436, 55472, 28740, 45944,
                756,   26820, 34144, 60216, 15092, 2896,  36412, 24040, 62784,
                572,   14976, 29380, 50596, 17452, 42876, 28048, 58116, 15996,
                35544, 1876,  20608, 28884, 43512, 31280, 36104, 14828, 22032,
                6012,  34528, 9740,  27636, 59204, 31484, 17512, 40088, 54268,
                9056,  18680, 59164, 6728,  54724, 38632, 10716, 7552,  36608,
                19788, 44892, 33960, 57692, 24008, 35684, 62992, 49580, 58752,
                43368, 8760,  18688, 46044, 10056, 4348,  50032, 60476, 30060,
                57548, 5100,  38876, 62060, 43000, 5596,  50480, 17168, 57012,
                22068, 52760, 27108, 47428, 59808, 36316, 17120, 29452, 52852,
                58500, 31788, 7352,  49260, 61176, 2108,  58032, 22064, 29788,
                3764,  39308, 27332, 31660, 2028,  38732, 28976, 6688,  58036,
                14864, 31500, 40252, 64648, 16336, 53976, 38680, 19480, 34812,
                64468, 3792,  39312, 52176, 62276, 23616, 46472, 20820, 40472,
                61624, 5384,
            },
            {
                50512, 46172, 56988, 40916, 60856, 44416, 17420, 5164,  36396,
                53932, 10268, 51080, 4396,  38552, 64384, 48920, 56124, 6432,
                41956, 52828, 59012, 4912,  35260, 12860, 1152,  38048, 63500,
                46280, 56576, 13060, 52960, 3212,  18392, 56424, 45608, 38624,
                60460, 49576, 24044, 54624, 1092,  35692, 5716,  43420, 13488,
                64136, 964,   35316, 27428, 62108, 20304, 50688, 59784, 14544,
                64424, 29772, 51108, 13852, 47240, 20764, 3004,  32148, 24532,
                5244,  54156, 39176, 51208, 23548, 43280, 37788, 18908, 25172,
                41904, 9640,  17928, 52164, 1444,  36796, 26008, 30308, 60908,
                37808, 43352, 13732, 19176, 6328,  42844, 64868, 9280,  21672,
                44924, 14692, 42656, 35360, 20296, 27696, 43380, 47640, 65312,
                11944, 59632, 42172, 18196, 55328, 50152, 11596, 53428, 416,
                23344, 51828, 19992, 56760, 4356,  22892, 11092, 49712, 58080,
                44008, 22084, 56348, 18868, 10580, 2820,  57164, 7436,  52324,
                34260, 25000,
            },
            {
                1564,  22616, 18952, 10696, 33328, 7924,  23312, 31756, 41876,
                19280, 22660, 57728, 43104, 25556, 20236, 8880,  16924, 31552,
                26212, 38612, 9784,  22636, 55460, 61528, 51936, 19732, 10184,
                5292,  26844, 61888, 34568, 23388, 64792, 7708,  26172, 1648,
                12712, 42836, 18860, 39904, 65192, 46184, 22552, 57672, 29504,
                38016, 51868, 47668, 14808, 43960, 24200, 30952, 41184, 26568,
                1320,  39396, 8600,  4948,  27168, 42380, 52284, 11016, 37208,
                47864, 14048, 31004, 3356,  59168, 27932, 13236, 7464,  47652,
                63780, 34784, 27460, 46772, 59524, 32900, 54396, 14748, 8688,
                268,   33772, 58836, 50848, 39812, 25684, 48628, 33840, 12332,
                61728, 25032, 3280,  64180, 10640, 53608, 5952,  37612, 17032,
                35216, 51540, 7896,  46112, 24384, 34824, 21376, 64336, 46260,
                37648, 59848, 10064, 29308, 35224, 45560, 62428, 25788, 15164,
                8756,  27900, 47612, 32912, 42860, 35300, 65308, 13284, 27764,
                17224, 59316,
            },
            {
                31100, 42772, 6280,  53376, 14932, 49424, 62764, 56716, 13288,
                64840, 48060, 7472,  30088, 12688, 54608, 40804, 34736, 58488,
                46052, 18992, 48464, 32808, 44300, 7956,  31008, 24752, 41788,
                32260, 44336, 16308, 38860, 48640, 11616, 40720, 50368, 32800,
                55792, 29928, 4420,  8720,  51508, 15304, 10904, 50380, 4748,
                25548, 20584, 41580, 11136, 53644, 5776,  3092,  45968, 52824,
                17572, 23472, 60192, 32824, 55072, 65384, 7576,  18460, 56652,
                26740, 62288, 21252, 55440, 16920, 64916, 32716, 52596, 21608,
                2268,  54980, 13940, 22620, 7188,  11880, 45204, 40796, 49076,
                20536, 28780, 11044, 3724,  32064, 15008, 55640, 612,   52076,
                36520, 47920, 29948, 51056, 16472, 32460, 23984, 60812, 336,
                25924, 20956, 57080, 14652, 62388, 3192,  41304, 16352, 27644,
                33564, 3772,  42904, 48444, 13976, 53140, 5584,  32292, 40368,
                54228, 12892, 1764,  59728, 15572, 25120, 49204, 38640, 43744,
                9624,  55192,
            },
            {
                13904, 38188, 64980, 29480, 25080, 36924, 1392,  45416, 27860,
                3316,  37928, 16280, 33552, 61228, 50432, 4692,  23160, 11512,
                1984,  65184, 14164, 3804,  27060, 39492, 15620, 48672, 54628,
                60556, 21720, 8676,  59112, 4240,  28300, 53884, 19956, 62500,
                15836, 46960, 60976, 33104, 24988, 28544, 61780, 34456, 17028,
                60108, 31504, 8040,  57128, 33992, 18356, 63620, 35572, 11732,
                56504, 42788, 48684, 19520, 15564, 38572, 29992, 60148, 44616,
                1256,  34296, 10748, 36536, 41220, 436,   45348, 57860, 11392,
                39444, 44700, 31020, 57232, 19728, 65072, 24128, 5268,  62096,
                55228, 44496, 64312, 23228, 57680, 8324,  20380, 28112, 40208,
                4532,  18532, 8984,  38904, 59348, 41484, 13444, 49392, 54368,
                44068, 32972, 4160,  37396, 10196, 31156, 44884, 8296,  56400,
                12416, 18236, 63100, 7144,  26636, 41360, 18100, 1188,  48880,
                63784, 30204, 38044, 51072, 4892,  29448, 53796, 64,    21428,
                48108, 35704,
            },
            {
                26832, 50096, 2288,  46508, 58800, 40520, 20964, 9960,  34332,
                51392, 24984, 59036, 46032, 1632,  28040, 43828, 62492, 52148,
                29172, 54892, 35532, 60504, 20672, 56608, 63996, 2924,  12644,
                37192, 1284,  50664, 30880, 42732, 14552, 35624, 2692,  23596,
                9956,  37392, 21348, 57144, 44256, 38592, 1860,  48288, 42532,
                55056, 3840,  63076, 38976, 29064, 48744, 58436, 21984, 6768,
                31608, 62604, 10232, 36300, 1692,  45784, 12732, 22228, 41004,
                16492, 52564, 46376, 7792,  49472, 25952, 6024,  35688, 29328,
                17304, 61556, 4196,  41592, 49748, 37324, 29552, 17824, 35144,
                25532, 15600, 38628, 52244, 35772, 45984, 62540, 43440, 60064,
                23272, 54088, 57712, 26264, 1512,  46272, 8068,  19528, 30356,
                10792, 63968, 47944, 28160, 59404, 52956, 25860, 61072, 39376,
                51112, 24048, 55136, 38060, 21280, 60312, 57228, 37060, 23912,
                19816, 7308,  44668, 22448, 63264, 9940,  18388, 32440, 60116,
                4204,  62360,
            },
            {
                8568,  20160, 56324, 11876, 16768, 4804,  53816, 60136, 18168,
                41480, 5760,  53368, 21256, 10388, 36364, 18176, 13780, 38356,
                7900,  42460, 24836, 50976, 11152, 45580, 6604,  34908, 29252,
                18320, 45748, 25628, 55504, 19252, 63104, 46572, 58576, 43912,
                27516, 52268, 608,   13552, 7084,  53860, 20148, 9408,  23260,
                12388, 46368, 16104, 24960, 256,   13584, 40412, 27052, 15984,
                46908, 2628,  28708, 52136, 58056, 25384, 50056, 4140,  32616,
                6184,  64228, 24416, 29760, 56960, 20092, 14660, 63088, 23780,
                51332, 8364,  25544, 33812, 10200, 1244,  56320, 51528, 3064,
                9604,  48252, 6144,  18272, 2284,  30216, 11520, 16292, 7236,
                32532, 12960, 44368, 21384, 35920, 63596, 27904, 56640, 39748,
                5608,  16000, 41736, 22232, 17736, 49380, 5372,  20500, 1280,
                47336, 29684, 32480, 2308,  50412, 8488,  29140, 11444, 46412,
                60800, 34600, 17128, 57476, 41036, 36120, 45992, 55584, 24484,
                41204, 16576,
            },
            {
                52868, 33536, 23872, 43508, 35100, 26468, 30760, 47460, 12316,
                62300, 31352, 14376, 39240, 63940, 24344, 56856, 49064, 32368,
                22036, 16016, 2744,  37952, 30588, 17008, 41516, 23612, 58664,
                53404, 65452, 11380, 34376, 6580,  24312, 9268,  31396, 5724,
                40096, 64104, 49008, 35828, 17124, 59184, 31764, 64572, 39936,
                28096, 35432, 50704, 21000, 60648, 44836, 9052,  51516, 55568,
                38208, 24588, 41508, 21244, 7100,  33848, 61824, 54236, 27576,
                57264, 38240, 18708, 12576, 61252, 38824, 54112, 42500, 2664,
                48004, 37692, 53904, 16164, 62848, 46444, 13980, 39540, 60336,
                42428, 32760, 27724, 59980, 41472, 54012, 25396, 50648, 37160,
                47244, 65176, 5476,  31444, 15272, 52236, 4100,  33988, 61480,
                25008, 51856, 55620, 1736,  38852, 12800, 32668, 36440, 15444,
                65420, 10680, 45132, 16684, 62324, 43128, 33924, 15784, 51784,
                4636,  55340, 10996, 25664, 5764,  14580, 61592, 7760,  12660,
                30244, 45360,
            },
            {
                64096, 39580, 3532,  59648, 51760, 63908, 6944,  37616, 23784,
                32,    57168, 43624, 7952,  29680, 46604, 5972,  680,   58784,
                45112, 62008, 48020, 6196,  59332, 52920, 27272, 49320, 8516,
                38608, 3552,  15844, 40636, 57792, 51780, 37620, 17932, 54936,
                12448, 19164, 30484, 24500, 41732, 47132, 26656, 5304,  52096,
                57976, 2340,  42760, 32596, 53240, 5360,  36476, 64804, 33176,
                4656,  59492, 14520, 64208, 11200, 39532, 17036, 47080, 10280,
                14456, 42776, 50908, 2024,  33464, 47164, 10000, 31448, 58460,
                20844, 12720, 59964, 28900, 43240, 19320, 26156, 31356, 21408,
                12028, 55688, 49892, 13712, 22008, 63432, 34232, 1544,  58568,
                27292, 18064, 39672, 55588, 10612, 48624, 17628, 22404, 45476,
                13028, 35756, 30848, 9048,  62528, 44332, 53868, 58228, 42640,
                22460, 6772,  59220, 35848, 13144, 25160, 54572, 2540,  39380,
                27508, 42332, 31560, 48068, 53164, 28180, 20816, 49988, 38000,
                57008, 5200,
            },
            {
                48296, 11244, 31400, 19108, 8956,  14556, 44768, 56100, 28836,
                48968, 20416, 35192, 51024, 17060, 54272, 41560, 26632, 19004,
                9864,  28292, 55880, 33532, 19712, 28,    63224, 14052, 21188,
                32176, 43532, 27792, 49100, 492,   29232, 14008, 47488, 60940,
                34628, 3508,  57964, 9636,  62228, 2444,  12912, 37104, 19220,
                15240, 7704,  63572, 10684, 17324, 30212, 23740, 19768, 12040,
                49736, 18484, 44952, 53964, 30560, 43784, 292,   23696, 36264,
                60508, 30924, 7224,  44676, 23048, 4244,  17560, 26768, 6512,
                34380, 45660, 2132,  22872, 4792,  52712, 58172, 7164,  63344,
                45316, 24224, 3336,  37684, 9060,  46776, 5856,  20756, 42404,
                9972,  51232, 2688,  24068, 60620, 42356, 37088, 8448,  58956,
                2456,  19316, 46948, 57484, 23380, 6904,  28840, 25212, 3884,
                51996, 39640, 26684, 49168, 20632, 5512,  45768, 58156, 22060,
                64540, 14324, 652,   62344, 38840, 3868,  33548, 43072, 1428,
                27120, 22248,
            },
            {
                15072, 60892, 27840, 41972, 49524, 22744, 2372,  40828, 15924,
                9340,  65456, 26220, 2852,  60960, 12284, 34592, 62804, 51632,
                36172, 40724, 13352, 23356, 43668, 10484, 36028, 46400, 5832,
                51972, 62268, 23028, 60252, 20136, 44408, 64432, 2336,  22292,
                41456, 26812, 45068, 54380, 21664, 33448, 49920, 43628, 61280,
                30480, 46944, 25972, 55636, 40920, 58388, 47800, 2004,  43300,
                29848, 8632,  34492, 2976,  26248, 57584, 19676, 63720, 5488,
                52292, 21168, 26440, 58380, 52928, 62384, 36736, 49532, 65296,
                15496, 40932, 55788, 31780, 38440, 9464,  34924, 47992, 636,
                36360, 18896, 61872, 57736, 28720, 16928, 39776, 56412, 14628,
                30444, 61756, 34700, 45744, 27004, 548,   54544, 29704, 40548,
                50836, 65100, 27164, 15216, 34620, 48492, 18500, 10160, 61748,
                34140, 17860, 56956, 504,   40780, 63836, 29824, 17388, 32988,
                9224,  49520, 18880, 22796, 44740, 57972, 16192, 52024, 62780,
                18512, 35976,
            },
            {
                7848,  54976, 528,   37408, 57716, 32908, 61848, 19796, 53020,
                32428, 45520, 5516,  40728, 23064, 31700, 8408,  15128, 21584,
                3184,  64808, 7176,  50748, 60628, 39860, 55096, 28872, 57944,
                17680, 10292, 4460,  36056, 11872, 39720, 25608, 32540, 52820,
                6032,  50228, 14720, 6736,  39164, 16392, 57428, 10040, 23832,
                53624, 38892, 21372, 34064, 3268,  14524, 62396, 38508, 26324,
                57320, 61348, 21844, 51132, 37572, 13228, 48724, 29012, 40168,
                46084, 9552,  35472, 14788, 39364, 28640, 12436, 42992, 52460,
                24756, 10460, 61636, 17716, 50304, 64404, 13000, 28380, 16260,
                54180, 30668, 6392,  42244, 33128, 53408, 25036, 64160, 48960,
                22820, 7516,  57268, 19604, 12380, 32592, 63116, 14880, 25508,
                10656, 5480,  38020, 43152, 88,    63496, 40756, 55532, 14568,
                44864, 29268, 12072, 32196, 52816, 9788,  36696, 50552, 6416,
                44348, 37720, 59072, 34940, 12496, 8372,  24320, 30708, 10744,
                58916, 44844,
            },
            {
                25536, 51212, 17436, 46868, 12484, 26056, 36408, 4456,  60300,
                11924, 37728, 55932, 18524, 58676, 49592, 38512, 44440, 55348,
                30832, 47036, 18068, 26668, 31408, 15716, 3364,  24800, 33588,
                41728, 47608, 55992, 30864, 53984, 6872,  57348, 9524,  16652,
                60072, 37268, 31900, 65320, 25884, 52612, 29340, 4608,  35180,
                1356,  12548, 49696, 6296,  46040, 27688, 51940, 7524,  54400,
                15872, 40072, 46600, 5668,  62340, 8384,  53452, 34032, 3912,
                17668, 55604, 64720, 1160,  19276, 8424,  56088, 21752, 84,
                30340, 36168, 6796,  46612, 25452, 42012, 20988, 57100, 43856,
                39836, 10776, 51004, 14704, 48052, 976,   10544, 35832, 3220,
                44820, 37976, 15992, 41000, 52944, 6880,  46552, 20312, 57960,
                44260, 32128, 54456, 21216, 12292, 51524, 30728, 37476, 2756,
                49964, 7820,  62792, 46844, 23484, 58872, 13556, 26504, 61732,
                55108, 24676, 29872, 2892,  54020, 65196, 47872, 37452, 5420,
                41132, 32844,
            },
            {
                63120, 20936, 30580, 4896,  64788, 7628,  50236, 29468, 43096,
                24448, 51700, 28396, 14144, 46716, 1272,  27408, 5180,  59788,
                24968, 11172, 38420, 53792, 4972,  45568, 20960, 63828, 12528,
                1076,  38248, 26336, 15336, 45408, 18888, 35024, 48588, 42864,
                29036, 19448, 10592, 47052, 204,   41960, 20264, 63792, 45368,
                59836, 17808, 56580, 65156, 37276, 19136, 11052, 22496, 32276,
                580,   12480, 25340, 17816, 31944, 42448, 15952, 25044, 59088,
                12252, 32328, 41680, 50280, 46476, 59876, 33292, 5380,  47728,
                57608, 19452, 53300, 14500, 1436,  33288, 4996,  51964, 24856,
                3896,  65368, 26616, 22432, 59504, 19404, 43756, 31664, 12824,
                54704, 28104, 4360,  64744, 49848, 23132, 38916, 3416,  35068,
                48568, 17104, 61092, 4104,  58340, 26032, 6264,  22756, 60536,
                19868, 25756, 38388, 4096,  19040, 43544, 1892,  40192, 20548,
                4208,  10808, 51680, 16984, 41048, 26800, 20204, 56404, 14664,
                53244, 2448,
            },
            {
                12144, 38908, 43228, 55728, 22072, 45452, 15060, 55296, 17888,
                2000,  7356,  63408, 35644, 9928,  53600, 64144, 17116, 41808,
                50024, 832,   58768, 34788, 61932, 9540,  48716, 36880, 51064,
                59516, 19592, 64688, 6004,  50356, 61792, 22868, 13384, 63424,
                4552,  56368, 23572, 58964, 35672, 8260,  48300, 14516, 26816,
                40504, 31644, 9328,  24356, 29876, 42300, 60980, 34872, 45584,
                64628, 50176, 36580, 55512, 58536, 22224, 1300,  63080, 43900,
                49216, 27640, 22864, 6436,  30796, 24608, 40852, 17212, 38168,
                62808, 26840, 43688, 60876, 29796, 58744, 37916, 8500,  60360,
                19648, 34776, 45624, 37412, 7788,  55424, 27064, 61240, 51892,
                20064, 60060, 33756, 10588, 29492, 16736, 61192, 55940, 13164,
                7272,  24256, 28904, 39840, 33968, 44792, 16176, 47380, 35196,
                52392, 43032, 55336, 16152, 34800, 28352, 54216, 47532, 15724,
                34576, 42444, 60640, 46808, 5796,  33048, 860,   44108, 28804,
                23316, 47456,
            },
            {
                35340, 9384,  60964, 16220, 34060, 40488, 10276, 59588, 38536,
                48452, 22216, 33012, 42748, 20300, 30608, 23808, 36764, 13400,
                33476, 20768, 44136, 14212, 22984, 41496, 57056, 16948, 7016,
                29744, 23496, 43264, 33344, 3080,  28228, 41028, 828,   26260,
                38472, 51100, 44032, 13128, 18020, 61064, 32652, 54880, 5932,
                21816, 51880, 44004, 14288, 1100,  57596, 48008, 4036,  18416,
                28392, 42824, 9852,  3660,  27916, 47932, 39020, 10052, 35876,
                19948, 2840,  61968, 15920, 54296, 11480, 64156, 51832, 13892,
                9416,  2932,  39856, 21504, 11156, 49428, 17140, 46332, 13644,
                32020, 55868, 2476,  12488, 62988, 40316, 4540,  16660, 38560,
                8316,  25272, 48196, 43124, 2188,  36908, 26348, 42528, 30784,
                64112, 53096, 9804,  49476, 19256, 64620, 8852,  56380, 12652,
                1040,  30192, 9448,  63984, 49528, 60004, 7452,  31752, 64836,
                56212, 27920, 14044, 22048, 38200, 61608, 51052, 9288,  63936,
                18920, 58004,
            },
            {
                27400, 50544, 2836,  29028, 24916, 53180, 1180,  31656, 26452,
                61964, 12956, 56700, 2904,  61152, 45020, 6120,  52044, 8916,
                62452, 27984, 6576,  50964, 29896, 1580,  26060, 32552, 44588,
                54648, 9312,  13640, 38996, 52860, 10720, 58728, 46420, 54376,
                8732,  33956, 2720,  30676, 53664, 39632, 24220, 11232, 58756,
                37484, 3156,  61724, 33820, 53720, 8748,  16072, 38880, 59400,
                7228,  53236, 20984, 63520, 34832, 12748, 51992, 30220, 55020,
                7572,  57748, 38696, 34512, 43764, 2020,  28276, 45200, 23216,
                34572, 31508, 51280, 7388,  55700, 34956, 22916, 63588, 27572,
                49084, 42092, 17744, 52604, 29192, 33932, 50248, 23728, 46504,
                156,   56256, 14548, 21316, 58764, 54424, 9628,  51408, 704,
                21688, 36684, 43728, 14488, 1912,  53500, 28316, 41520, 32584,
                62032, 23676, 14364, 40664, 5108,  21296, 12392, 38100, 23180,
                300,   49240, 7216,  31312, 57132, 18384, 13272, 36080, 39976,
                31564, 4272,
            },
            {
                41828, 13932, 57160, 48032, 6112,  63228, 36012, 20192, 46064,
                8808,  40644, 50804, 26136, 15544, 39488, 58252, 18944, 46768,
                56892, 40224, 54576, 17768, 65036, 38280, 11532, 61388, 4156,
                35928, 49060, 57528, 61056, 21452, 16344, 36784, 31916, 18692,
                14848, 61604, 21196, 27268, 5416,  49700, 1600,  43676, 30096,
                47748, 16612, 27520, 49420, 39764, 22700, 26892, 51176, 13072,
                23768, 30984, 39952, 15612, 45428, 6740,  60584, 16968, 23924,
                46164, 13480, 51264, 9252,  58944, 20688, 36500, 4256,  55556,
                60436, 47220, 16368, 65000, 25796, 44972, 5964,  40064, 328,
                10240, 58584, 6204,  24772, 47684, 15144, 9380,  64636, 30000,
                41284, 35696, 62972, 31368, 45700, 6132,  19228, 33744, 15660,
                46736, 59368, 5356,  56648, 31620, 38304, 20832, 5056,  58788,
                18048, 37792, 47700, 57664, 33360, 25668, 52144, 45540, 9532,
                18488, 39804, 63108, 44308, 10424, 25484, 48132, 59248, 6116,
                15896, 52396,
            },
            {
                24372, 20856, 37336, 44088, 18876, 12556, 49968, 58464, 4660,
                29840, 16996, 36516, 5276,  54852, 11700, 27160, 1724,  31472,
                24340, 4080,  12168, 32836, 46116, 55760, 20060, 52372, 14820,
                27552, 18936, 1500,  25748, 47260, 30044, 7112,  63932, 24280,
                49328, 41612, 55980, 46320, 64716, 36628, 15500, 62084, 19548,
                8060,  55756, 11484, 20032, 4496,  64220, 31948, 55892, 35400,
                44880, 61696, 2620,  55200, 25824, 20372, 41688, 4664,  65280,
                32980, 42580, 28628, 17596, 25376, 48428, 61708, 8252,  18844,
                39100, 12160, 28736, 41880, 1696,  14680, 52212, 30280, 54388,
                21944, 35916, 30860, 62156, 20108, 1868,  42808, 53848, 12476,
                18700, 51588, 4296,  10972, 39964, 24992, 61616, 48692, 28252,
                40552, 11452, 26968, 23452, 62168, 46884, 11940, 25680, 44556,
                49932, 6636,  27596, 2760,  44392, 16704, 61324, 29612, 58408,
                53092, 24432, 35428, 3276,  54328, 30020, 2036,  21812, 55176,
                46496, 61876,
            },
            {
                1780,  65376, 7860,  30896, 59832, 41352, 23576, 14964, 42884,
                52996, 65136, 19880, 46656, 32472, 49224, 35104, 64604, 43412,
                16024, 37380, 48760, 22200, 8160,  3068,  42652, 23936, 39888,
                62976, 45636, 32688, 5560,  42108, 55852, 50168, 3848,  44248,
                11600, 36036, 6868,  16988, 9892,  22400, 28132, 52900, 34304,
                42044, 63568, 36468, 46244, 57840, 43236, 10844, 1668,  19740,
                6160,  49164, 11724, 33320, 58212, 50408, 29384, 37156, 53544,
                276,   21420, 62920, 5008,  53764, 40284, 14292, 29892, 49456,
                25192, 6528,  58404, 20604, 33028, 62212, 37020, 19124, 44236,
                61220, 14400, 39620, 45280, 56044, 37064, 60448, 22240, 34656,
                7232,  59732, 28000, 16512, 53488, 36068, 12980, 57300, 3816,
                63000, 18192, 34904, 41640, 16420, 51820, 3632,  35944, 54060,
                9900,  65284, 20284, 55060, 10824, 36596, 1552,  40440, 4640,
                33240, 11800, 59900, 16452, 50632, 40732, 34608, 43196, 26752,
                8392,  33420,
            },
            {
                44888, 17156, 54700, 34852, 2532,  9528,  28560, 56216, 33424,
                288,   24704, 57524, 9984,  22920, 60804, 7128,  21308, 51116,
                9772,  62980, 58696, 26980, 36404, 60524, 29300, 49372, 7468,
                56692, 11972, 38084, 65252, 10168, 14356, 20240, 38928, 57492,
                1240,  25880, 31124, 58864, 40580, 55040, 45008, 3788,  13108,
                25808, 356,   23104, 31108, 7404,  17732, 37944, 47560, 60176,
                41144, 27228, 17632, 38172, 9360,  1944,  47024, 13936, 26596,
                10804, 49252, 37772, 31596, 45628, 1200,  33384, 57392, 42232,
                52448, 2396,  35668, 54600, 48700, 9196,  59384, 12320, 3568,
                26148, 49776, 8520,  3976,  11740, 27432, 31760, 4992,  57208,
                25580, 42452, 47236, 21924, 65424, 1384,  43344, 23032, 38272,
                7244,  54524, 49604, 1008,  8812,  57996, 22444, 61328, 30016,
                16216, 34560, 42104, 31168, 48224, 63356, 19368, 50292, 14672,
                43568, 26964, 47312, 20652, 8020,  64912, 14420, 61104, 18732,
                11704, 38292,
            },
            {
                27880, 49024, 13036, 25612, 52456, 64012, 37804, 6516,  49280,
                11956, 38956, 44084, 2360,  28800, 41540, 14256, 38712, 55668,
                29720, 148,   41752, 14384, 50764, 17092, 10980, 34452, 2076,
                30872, 22488, 17508, 51764, 35004, 23736, 60456, 28060, 33236,
                52904, 63012, 19968, 50624, 13856, 33084, 7716,  58272, 48936,
                39576, 52308, 59276, 14428, 28224, 53208, 62424, 24944, 14760,
                30116, 54636, 64812, 43824, 24328, 62460, 18516, 59652, 40492,
                57456, 16592, 7768,  56016, 12552, 23680, 64908, 16724, 10344,
                21904, 63584, 44268, 17208, 4576,  23060, 27320, 40812, 57216,
                16800, 33844, 65044, 23492, 52128, 17364, 49040, 44688, 14944,
                50504, 37704, 2568,  33260, 49720, 29296, 9184,  32252, 51500,
                20592, 30816, 45284, 64952, 28524, 33100, 43348, 13356, 39800,
                728,   24508, 60256, 13756, 23004, 7152,  28528, 56316, 22192,
                64288, 6080,  55924, 38456, 28908, 23640, 4604,  52764, 30788,
                50124, 60080,
            },
            {
                5248,  40956, 58780, 22044, 45372, 15720, 47540, 21164, 26708,
                61620, 34968, 15392, 62712, 51984, 18260, 58348, 3616,  25844,
                47452, 20336, 34108, 5868,  56936, 44208, 63812, 19664, 54124,
                43372, 48396, 59672, 27056, 800,   53660, 42936, 8404,  17704,
                13076, 45336, 4676,  37536, 2248,  23992, 63680, 30316, 21048,
                16868, 6044,  33264, 42324, 50016, 2548,  21604, 34416, 8640,
                51360, 444,   21940, 5344,  52504, 31732, 35796, 6896,  22740,
                33624, 44060, 27476, 60784, 19540, 36184, 6348,  46924, 37604,
                27580, 32004, 13588, 39932, 29916, 45900, 51148, 32644, 7348,
                47552, 53516, 29316, 42268, 58516, 39088, 684,   64004, 10308,
                19304, 62272, 8652,  13984, 58124, 17920, 55260, 47340, 15380,
                60696, 10448, 24868, 14236, 39108, 19132, 48620, 7260,  56356,
                46056, 50312, 4724,  52792, 38972, 58820, 34348, 46220, 8816,
                37132, 31080, 924,   42348, 12540, 57720, 45184, 36148, 4,
                56756, 21536,
            },
            {
                32708, 10340, 1360,  30028, 39456, 4284,  32016, 59556, 17600,
                53656, 7812,  30568, 48184, 4328,  35808, 45880, 32920, 11968,
                17216, 61448, 54140, 24472, 31460, 3492,  37500, 25772, 58804,
                14988, 4632,  8680,  40436, 46168, 30532, 4144,  64248, 48100,
                22736, 41052, 28420, 61712, 48408, 17480, 42520, 10784, 37716,
                61500, 46516, 9724,  65224, 37332, 11984, 56544, 45924, 39756,
                58936, 13044, 36492, 48088, 15528, 10472, 55560, 45244, 3472,
                64356, 52740, 2416,  41536, 50084, 29240, 52916, 3340,  60852,
                55768, 7824,  50708, 61612, 10688, 56008, 1808,  63516, 21176,
                38336, 1136,  19640, 13088, 5728,  33212, 21460, 26604, 40684,
                30996, 55376, 23932, 45224, 40808, 5484,  25920, 63532, 3132,
                36560, 42972, 5700,  55476, 51200, 2408,  26400, 63604, 21184,
                28612, 37028, 9188,  18152, 43680, 3452,  12244, 26296, 17328,
                53908, 48048, 19100, 62948, 32812, 49416, 18060, 10828, 25416,
                42028, 15364,
            },
            {
                53288, 36008, 61720, 50212, 19200, 56464, 10504, 41976, 3168,
                45036, 23296, 40496, 20432, 26864, 54044, 8388,  60352, 43720,
                52104, 36704, 9324,  46200, 12940, 21856, 52840, 9176,  40900,
                28764, 36248, 62364, 13152, 19736, 58192, 15696, 38140, 35132,
                5888,  59512, 10260, 53788, 34328, 26792, 56660, 51296, 4756,
                27472, 55480, 22056, 18368, 26016, 5812,  30936, 17000, 4248,
                26128, 32420, 42768, 57344, 29136, 41320, 26292, 50064, 19872,
                30464, 12140, 24688, 15512, 9440,  58564, 43004, 26100, 18712,
                45072, 296,   21008, 25388, 37912, 18616, 14468, 42716, 24728,
                10332, 60572, 45844, 35740, 62080, 47112, 54556, 7064,  51932,
                36284, 3588,  28312, 61084, 36816, 21180, 34408, 11848, 40204,
                27808, 58656, 22108, 34516, 60120, 11064, 36508, 53128, 16892,
                11960, 58024, 64332, 25332, 30404, 49384, 62216, 41452, 2520,
                60824, 24392, 10116, 51708, 26444, 6840,  38180, 61952, 47772,
                7480,  64036,
            },
            {
                46124, 23960, 43340, 7780,  13968, 62752, 36756, 28652, 50792,
                13848, 58840, 55764, 10668, 64268, 13496, 24172, 30312, 2112,
                40272, 4816,  28408, 65180, 39704, 48852, 61120, 640,   46468,
                19088, 51040, 24540, 56092, 33904, 50008, 10620, 26284, 55812,
                51156, 31808, 15296, 21236, 7328,  46296, 884,   35504, 15260,
                44472, 30692, 1408,  53800, 48260, 59584, 41108, 63904, 49704,
                20440, 61364, 7980,  17980, 63072, 1376,  13608, 60564, 8920,
                39412, 48128, 34580, 63096, 38064, 21572, 32748, 14404, 10872,
                34996, 40768, 58456, 47536, 5788,  30780, 59476, 35448, 54588,
                28812, 15604, 56616, 25616, 9308,  16608, 29996, 13600, 59812,
                46204, 17048, 12108, 51168, 316,   53648, 59392, 46100, 17280,
                52984, 1344,  46592, 18008, 29696, 41448, 4852,  44696, 31692,
                2624,  41008, 33620, 54448, 15316, 20600, 37524, 52152, 29460,
                13508, 44140, 34740, 15820, 60276, 2808,  22536, 55288, 33788,
                19756, 29124,
            },
            {
                12348, 17244, 55092, 27604, 33980, 52876, 24400, 7048,  64976,
                34556, 1532,  32336, 5428,  38152, 42400, 48732, 19688, 63232,
                22972, 15568, 57180, 18928, 6824,  26576, 16388, 33480, 12064,
                64656, 31648, 1824,  39296, 6164,  22304, 43460, 60608, 19148,
                136,   24700, 43972, 57532, 40280, 12700, 19888, 64280, 23672,
                58332, 12908, 39660, 34224, 43968, 14140, 23076, 35600, 10552,
                54336, 45080, 3112,  23748, 34212, 53712, 22028, 36944, 57040,
                17080, 4648,  54420, 45804, 752,   7060,  55444, 47704, 64056,
                51552, 28480, 15268, 33220, 65432, 52252, 8892,  44808, 4000,
                50036, 40100, 32008, 3176,  50984, 37080, 57420, 41996, 22164,
                5536,  64660, 32156, 19684, 43108, 9964,  29904, 6776,  23588,
                32608, 13132, 50348, 7972,  62984, 14636, 57480, 24284, 62088,
                47580, 22160, 6588,  45084, 452,   56868, 10400, 6948,  21820,
                57496, 39448, 3852,  56200, 40848, 44816, 30452, 12948, 39968,
                50500, 4428,
            },
            {
                57248, 39024, 3372,  21232, 48028, 500,   44428, 20308, 39372,
                25792, 18072, 46608, 51192, 28768, 16456, 57880, 6684,  55244,
                34892, 49596, 44116, 32232, 51384, 35792, 43212, 58396, 23328,
                6976,  54912, 17040, 45288, 28348, 63308, 2732,  30160, 12852,
                46952, 38644, 65436, 3224,  29960, 60408, 53536, 32376, 40924,
                7984,  50156, 60944, 4556,  9148,  28372, 55976, 1932,  29836,
                15292, 37508, 27536, 50612, 39744, 45932, 6220,  32164, 43476,
                27128, 61180, 22388, 29980, 17976, 59628, 40112, 23940, 4516,
                20228, 8348,  54244, 3668,  22944, 41112, 27216, 20704, 12916,
                64252, 6556,  22440, 44384, 63368, 19380, 1432,  25216, 10868,
                34808, 48292, 39900, 57192, 25348, 15940, 49244, 64860, 41948,
                57148, 61816, 38068, 26104, 44024, 32964, 19852, 49860, 9560,
                35764, 14332, 51616, 27700, 62772, 32256, 47092, 35232, 65064,
                48876, 31632, 7884,  20680, 27888, 9456,  64832, 16604, 2300,
                26244, 60900,
            },
            {
                9356,  31140, 64752, 41372, 59852, 11832, 16124, 56724, 48708,
                11336, 42852, 59592, 21744, 61336, 584,   36932, 31532, 10016,
                26824, 13168, 1068,  59232, 10764, 4472,  55280, 29432, 47412,
                37168, 42008, 10992, 59616, 48204, 9140,  52848, 40740, 34664,
                54744, 9576,  17424, 50344, 37000, 25072, 9068,  48652, 3464,
                18824, 36140, 24768, 16820, 63060, 20636, 51420, 7440,  42076,
                48600, 58880, 12084, 65356, 9156,  16036, 62100, 49336, 2296,
                51908, 13880, 41172, 10004, 49780, 27440, 12492, 36444, 31012,
                60200, 43208, 36996, 12276, 45996, 16964, 2100,  57368, 47344,
                34188, 18028, 52640, 12052, 28072, 39348, 49192, 61288, 44160,
                54884, 26932, 2124,  7832,  62548, 33124, 38424, 2320,  19584,
                9572,  29260, 4280,  21668, 54952, 188,   39496, 6156,  29360,
                55648, 59376, 19068, 38828, 12768, 23724, 42692, 18040, 1484,
                25908, 16408, 59740, 50460, 53968, 35376, 47176, 58060, 52736,
                43456, 36376,
            },
            {
                22652, 49556, 14992, 5968,  37116, 29284, 32944, 60956, 4524,
                30540, 54068, 14344, 8200,  25200, 45320, 12432, 51960, 46804,
                41952, 61768, 38052, 24876, 21400, 63204, 14312, 20024, 3876,
                52220, 27028, 21240, 33060, 14308, 36952, 18108, 23324, 62292,
                6536,  28156, 33168, 22264, 5916,  44252, 16564, 27868, 62644,
                46340, 56312, 29436, 53200, 32936, 38708, 44776, 59996, 24912,
                32624, 19240, 5104,  55264, 31272, 20760, 28596, 11528, 24924,
                35636, 7536,  57364, 33192, 65060, 44212, 53472, 2576,  48584,
                16236, 57172, 24888, 62632, 50112, 35164, 61356, 31412, 38844,
                24592, 60912, 42088, 56104, 7992,  14984, 33376, 5064,  30436,
                17784, 14360, 50852, 21528, 46520, 12964, 55384, 27388, 51704,
                35416, 45364, 16940, 48688, 11340, 65392, 52540, 16288, 43472,
                25268, 1936,  42352, 8236,  49672, 4260,  60472, 28616, 55120,
                11764, 46148, 36788, 23764, 13784, 552,   19308, 24488, 6788,
                32108, 18164,
            },
            {
                1124,  53744, 26020, 44996, 55904, 23300, 51128, 8432,  37464,
                63448, 2324,  36188, 40892, 33396, 54792, 18784, 64768, 22180,
                3320,  18056, 52492, 30400, 47648, 41748, 38676, 8820,  61716,
                15540, 57556, 2368,  64080, 25204, 50428, 4984,  58976, 15052,
                49112, 42196, 56460, 62940, 13840, 52340, 59480, 37956, 12008,
                21872, 2400,  42752, 11080, 48104, 280,   13172, 18044, 64064,
                2660,  40256, 23184, 36332, 47264, 616,   42204, 58476, 18352,
                63624, 45556, 19704, 37744, 5768,  15596, 21016, 62136, 10220,
                27776, 39508, 1716,  19660, 29184, 10324, 54832, 15040, 5128,
                9404,  29756, 232,   35476, 46940, 24104, 58688, 52752, 9204,
                63556, 38664, 59208, 29080, 36152, 5080,  43748, 22728, 6356,
                14352, 56176, 40228, 59636, 35000, 27856, 23176, 36680, 61404,
                11252, 34024, 64668, 30172, 57848, 37012, 15056, 52356, 39252,
                6148,  61976, 42952, 4720,  63180, 32780, 42360, 38384, 11276,
                63712, 46860,
            },
            {
                39716, 59340, 34448, 10604, 17924, 2740,  41848, 19440, 46176,
                24124, 28304, 20080, 48784, 62440, 3780,  29700, 39600, 6240,
                33892, 56260, 8188,  15220, 2752,  53540, 25992, 31580, 49820,
                34748, 45656, 39140, 7664,  42524, 56196, 30596, 44656, 26424,
                2148,  20280, 10924, 46432, 1448,  34040, 41656, 5332,  31188,
                51896, 15476, 65040, 6652,  22904, 57116, 27496, 35900, 9840,
                50884, 57564, 44200, 13100, 60732, 50992, 34508, 54728, 38992,
                3528,  29472, 24116, 55128, 1588,  26076, 41856, 34120, 56024,
                45432, 5624,  32860, 52704, 7476,  44168, 40364, 21600, 51740,
                48152, 59156, 16732, 21516, 65232, 3108,  41356, 20404, 36460,
                23352, 3872,  42744, 10396, 52384, 61584, 17256, 58884, 48388,
                64020, 25184, 1820,  31220, 7652,  14084, 45632, 3172,  54548,
                47780, 21500, 16504, 53080, 25800, 20200, 9072,  44040, 21620,
                33948, 30120, 18144, 10608, 47808, 28928, 59452, 51396, 15752,
                55148, 28756,
            },
            {
                13364, 20556, 6860,  61580, 47724, 64292, 27192, 54468, 15024,
                9848,  52160, 57424, 5992,  11448, 43356, 15384, 26048, 59768,
                49220, 28460, 45048, 64216, 33560, 59644, 12212, 43932, 236,
                24140, 10616, 29264, 53748, 19500, 3144,  34256, 11328, 38952,
                53348, 35984, 60228, 30296, 26548, 55660, 19212, 23952, 57384,
                45000, 33580, 26628, 36548, 41596, 61508, 31180, 54716, 46848,
                15712, 28932, 6404,  26092, 17184, 4820,  22580, 9420,  14852,
                48824, 52588, 12376, 61296, 47372, 31144, 51600, 7248,  23072,
                18296, 64016, 14060, 46968, 59988, 25408, 1056,  63252, 26924,
                43316, 36860, 11372, 55000, 31960, 51044, 29008, 12600, 46668,
                56040, 49496, 31548, 18600, 25112, 1220,  40948, 29664, 37300,
                32544, 10760, 52036, 18856, 42320, 49996, 60316, 19484, 8492,
                31512, 40752, 5412,  44932, 1004,  48164, 32696, 64148, 2948,
                57776, 50268, 25488, 56720, 37724, 21456, 8532,  2964,  26680,
                44388, 4732,
            },
            {
                33216, 51260, 42280, 29656, 38464, 12812, 35272, 4808,  60552,
                40116, 44360, 16404, 31772, 23752, 50644, 56864, 35648, 9472,
                13680, 41060, 19844, 37236, 22980, 6272,  18104, 55804, 65448,
                20408, 60144, 16908, 47884, 13112, 62392, 21796, 47160, 64184,
                17580, 7960,  23220, 39988, 15900, 47680, 10404, 61420, 708,
                39524, 8692,  59692, 50048, 16500, 19900, 3684,  7616,  38460,
                21388, 33684, 62936, 53596, 41188, 30756, 64596, 27600, 43684,
                32628, 6472,  41644, 17576, 9680,  38768, 57796, 12968, 49264,
                36200, 41336, 55152, 22196, 37800, 17612, 34720, 12112, 56944,
                19060, 3748,  40296, 25700, 44936, 6052,  17344, 62704, 664,
                26204, 7548,  13676, 65020, 56772, 33640, 8844,  12468, 20188,
                4672,  43956, 22436, 63044, 55464, 4124,  30372, 39928, 25876,
                50440, 62564, 13096, 35288, 60860, 56372, 12104, 27544, 46876,
                14536, 8204,  40620, 3760,  53700, 14120, 49136, 62724, 35760,
                22016, 61156,
            },
            {
                48188, 16628, 24660, 53412, 928,   22348, 58304, 49208, 30100,
                33832, 160,   65016, 38024, 59528, 19264, 1472,  46096, 53248,
                24460, 61552, 896,   10964, 50668, 46264, 29400, 35816, 40060,
                7044,  52656, 32152, 36524, 58432, 40820, 28532, 6172,  56800,
                31792, 50488, 44992, 3016,  64856, 7320,  36348, 49292, 28952,
                13684, 20696, 54596, 4520,  12120, 45564, 52424, 24600, 43156,
                59744, 1568,  47984, 10336, 37756, 56292, 45976, 2196,  58900,
                21368, 62636, 35956, 27412, 59792, 20088, 3328,  62964, 29856,
                80,    26168, 10952, 31068, 4596,  57872, 50464, 29584, 6924,
                32380, 63980, 49092, 8136,  14284, 58068, 38564, 43996, 33000,
                60100, 40104, 35528, 46108, 20932, 44264, 50240, 54640, 61112,
                47012, 58112, 39072, 15700, 26528, 36928, 11904, 57552, 15592,
                2544,  55736, 28412, 23708, 7700,  38672, 17416, 41668, 23580,
                60660, 19820, 35020, 65268, 27488, 43884, 31860, 18452, 41328,
                55948, 8900,
            },
            {
                37364, 3988,  58796, 9872,  32756, 16052, 43664, 7680,  25248,
                11828, 21832, 47688, 27092, 7880,  41768, 30032, 63748, 5440,
                17536, 32400, 57620, 27284, 39708, 58416, 3256,  13636, 48844,
                27852, 42800, 5588,  1496,  25968, 9376,  51620, 16208, 404,
                25368, 12536, 59076, 33540, 54148, 21172, 42792, 53068, 17652,
                64008, 46564, 25468, 31952, 39028, 63592, 34464, 56564, 14708,
                11812, 51836, 18968, 24264, 7088,  15328, 20164, 11912, 37272,
                53844, 15832, 688,   50744, 44900, 32816, 24512, 43504, 16524,
                52516, 60616, 7796,  48604, 64884, 14484, 41684, 46068, 53736,
                23512, 15816, 60320, 28512, 54000, 35356, 19836, 9884,  50364,
                21684, 16596, 54040, 2748,  5860,  27288, 38496, 16432, 25712,
                628,   28852, 8672,  33616, 6468,  48136, 23532, 64576, 34208,
                45436, 37968, 18336, 43184, 51604, 30552, 4796,  54072, 36040,
                440,   51364, 45120, 11500, 17396, 1728,  58152, 6596,  25180,
                12352, 30624,
            },
            {
                52628, 44728, 19848, 63068, 46916, 55312, 39052, 19324, 62900,
                56336, 52468, 4236,  13720, 54512, 34312, 11048, 21908, 38156,
                43616, 48356, 52548, 7588,  15680, 63404, 25280, 54024, 9476,
                22252, 15288, 63664, 45444, 22800, 55116, 34936, 43724, 38116,
                61680, 42224, 19048, 5028,  27772, 12864, 31992, 24668, 6420,
                35344, 40952, 1968,  58856, 22748, 8440,  27244, 2580,  30208,
                65076, 40084, 32396, 44916, 60560, 34756, 52124, 48320, 25912,
                8244,  30416, 23368, 56440, 5168,  14464, 54364, 9304,  38300,
                45600, 20952, 33856, 42988, 27924, 2916,  20344, 9496,  36568,
                2404,  43844, 34288, 21012, 1388,  61736, 25136, 4968,  55628,
                28036, 11332, 48392, 30036, 59008, 14160, 63608, 6864,  41908,
                34980, 51440, 13372, 54168, 61504, 44100, 1420,  51340, 9664,
                22004, 5864,  59364, 10072, 63804, 20748, 48944, 62284, 10468,
                28356, 55712, 31292, 24368, 60096, 34252, 39056, 51676, 47216,
                0,     64828,
            },
            {
                27664, 13820, 40196, 26164, 5872,  28600, 50860, 1924,  34880,
                16852, 42340, 36720, 60848, 45528, 16976, 49880, 55872, 27960,
                12792, 3736,  21396, 42184, 35080, 20104, 31840, 43772, 61244,
                33696, 56064, 50424, 39232, 18204, 59840, 13580, 49276, 20580,
                29548, 9568,  36724, 48908, 57488, 39184, 62248, 2812,  59596,
                11416, 30256, 48548, 14856, 43408, 51120, 17644, 47616, 37052,
                22472, 4832,  27000, 57708, 92,    28728, 62200, 3656,  42216,
                64088, 39284, 47320, 11104, 41116, 65408, 35204, 27200, 59408,
                4188,  12148, 55664, 17236, 51980, 38620, 24900, 62736, 56080,
                12984, 51496, 39288, 10676, 47572, 51876, 41632, 31464, 64380,
                1796,  42976, 62356, 22696, 36716, 51768, 32404, 23208, 57152,
                18312, 65148, 21124, 39960, 30108, 16676, 20096, 40564, 27620,
                53952, 32600, 48284, 15436, 34604, 2044,  26236, 14092, 43776,
                39872, 15804, 5240,  47768, 8076,  53120, 20776, 28752, 15668,
                59580, 21724,
            },
            {
                7964,  33868, 2648,  56752, 36884, 10412, 14108, 60128, 45940,
                27992, 9620,  30960, 20216, 24620, 5904,  39864, 2068,  59152,
                34792, 64500, 29968, 55492, 11668, 5464,  47828, 1856,  17608,
                37608, 4412,  11676, 29864, 7772,  32408, 3008,  64988, 6900,
                56020, 52804, 15148, 23056, 45652, 8988,  16668, 44204, 49936,
                53956, 18592, 56796, 6036,  35948, 61628, 11268, 57924, 42276,
                50496, 9516,  53972, 17240, 11428, 39568, 22628, 32264, 18124,
                13460, 57844, 34196, 19764, 28856, 50040, 7120,  21836, 48516,
                31416, 63536, 36636, 23120, 6808,  58548, 33036, 48216, 28244,
                18660, 59564, 24736, 4600,  30272, 17724, 13016, 45708, 15684,
                37624, 33704, 7640,  17132, 41368, 10284, 3428,  47636, 11664,
                44720, 3084,  49304, 24680, 5308,  56680, 35940, 60212, 13928,
                62420, 272,   25316, 56748, 40784, 46000, 58200, 32528, 6912,
                59436, 22312, 62524, 36844, 41628, 13516, 63872, 10092, 45188,
                36216, 42200,
            },
        },
        {
            {
                6948,  53532, 3388,  33600, 9052,  56812, 42972, 61312, 9504,
                53264, 348,   41332, 36176, 21496, 9900,  63088, 19180, 36900,
                54888, 22516, 43148, 60364, 27424, 38028, 56308, 64416, 40736,
                27172, 11980, 1244,  37652, 9780,  55488, 13628, 35772, 42260,
                16416, 61196, 3420,  25688, 20360, 11528, 62704, 37728, 49072,
                58464, 2060,  39012, 63460, 32860, 57432, 24916, 34912, 9376,
                56704, 6300,  15584, 40668, 32044, 12516, 63544, 19800, 15820,
                57360, 44788, 176,   15032, 8684,  24568, 63344, 1760,  37824,
                42784, 33464, 17388, 47540, 26976, 61872, 49656, 16768, 52680,
                19976, 7108,  11644, 18252, 25588, 4792,  13960, 50508, 61884,
                31868, 56232, 2892,  20220, 52020, 15464, 23776, 344,   34408,
                59384, 41636, 25028, 44368, 5364,  33708, 57108, 14752, 48420,
                63896, 40288, 46252, 13904, 7064,  52600, 33780, 57260, 60660,
                24776, 15020, 40936, 11704, 31148, 51992, 18220, 12988, 8560,
                59836, 16492,
            },
            {
                36100, 48504, 58456, 14272, 17860, 36412, 2164,  13176, 39896,
                29064, 16812, 49948, 45896, 58532, 3752,  24464, 49004, 60848,
                10476, 2976,  30564, 49308, 5964,  46656, 24216, 9488,  3360,
                52636, 60540, 46816, 64840, 28264, 19328, 62312, 46956, 220,
                28156, 39568, 32696, 7804,  55776, 46492, 784,   15668, 32568,
                10648, 45596, 27168, 6896,  20964, 46220, 51300, 508,   19292,
                48160, 35916, 55056, 45920, 58204, 24476, 51812, 4728,  34608,
                25444, 10964, 38560, 32648, 58400, 43592, 16360, 28320, 58820,
                13696, 61424, 424,   55912, 10852, 20636, 7464,  57552, 33276,
                65196, 37360, 49140, 59588, 40284, 64324, 44540, 27468, 4036,
                41548, 8880,  65508, 43732, 57824, 39936, 61176, 43048, 27140,
                14228, 37848, 2224,  52608, 18844, 50140, 42556, 25216, 31284,
                6132,  21908, 33288, 56776, 19552, 37200, 9968,  1524,  45416,
                32272, 8612,  49568, 640,   38748, 60384, 45904, 34548, 25100,
                51372, 21560,
            },
            {
                11328, 29224, 41944, 23696, 65484, 49528, 26760, 20672, 34492,
                57580, 23260, 11156, 27608, 14800, 34580, 52920, 28900, 13252,
                40760, 25384, 58036, 13796, 17808, 41660, 33840, 14792, 45272,
                36476, 20796, 16844, 5096,  40664, 23524, 6072,  59180, 11212,
                21936, 65376, 13272, 51464, 30344, 39032, 22384, 56780, 41692,
                20052, 62224, 35028, 54044, 14816, 10904, 59760, 39444, 62096,
                13096, 28068, 22828, 10204, 29808, 2068,  37016, 59832, 42276,
                55364, 47056, 62412, 22304, 51200, 4772,  36400, 46696, 21664,
                5988,  24824, 30304, 36252, 59400, 41796, 31160, 12840, 45764,
                24196, 15108, 28324, 376,   32996, 8348,  16588, 34592, 22180,
                37924, 18136, 49476, 29112, 4804,  32464, 9252,  18716, 50180,
                5940,  55732, 61408, 30936, 12644, 38660, 1036,  10764, 60352,
                17460, 54964, 1960,  43280, 62348, 25712, 49092, 28584, 42028,
                22144, 54716, 35696, 62084, 15864, 22916, 4308,  56564, 40772,
                2208,  62932,
            },
            {
                55868, 19348, 1052,  31728, 44384, 5376,  54968, 46388, 4184,
                63940, 43624, 7132,  54480, 62456, 39408, 44212, 4964,  32644,
                47532, 51820, 35980, 64096, 54332, 2304,  61364, 29476, 59028,
                6864,  30948, 49912, 34108, 54560, 44208, 30688, 51884, 34320,
                54352, 47776, 19192, 42516, 4468,  63824, 26840, 6980,  52140,
                29480, 17052, 2924,  43480, 37516, 29588, 4380,  23732, 31440,
                44236, 52912, 64424, 5532,  49996, 43828, 20892, 13832, 28624,
                2660,  18484, 6996,  27584, 12656, 56432, 31108, 10632, 54892,
                51688, 40352, 45892, 15496, 4880,  52272, 25996, 39160, 2012,
                6088,  54800, 44004, 51644, 20080, 55596, 48144, 59120, 51880,
                13544, 54644, 24260, 11564, 36760, 21900, 53280, 64496, 29716,
                44868, 16668, 35340, 21244, 57744, 63428, 28520, 52108, 44760,
                37872, 49736, 12500, 29600, 4672,  15260, 59512, 18656, 12124,
                63596, 4976,  19340, 26548, 47976, 9924,  28672, 64692, 15228,
                31188, 44732,
            },
            {
                26796, 8876,  60516, 52216, 38428, 11672, 62080, 30920, 15004,
                51508, 37592, 32556, 20864, 2628,  18224, 8932,  22808, 63548,
                17328, 6460,  20624, 9632,  31628, 22472, 50444, 18736, 53888,
                25456, 43076, 12864, 58056, 1692,  14696, 38432, 17448, 26180,
                8708,  2048,  35596, 59412, 16904, 48488, 12776, 36500, 61084,
                4840,  47868, 56472, 25280, 65316, 49240, 17768, 54728, 7728,
                16444, 3024,  33928, 39900, 17496, 60924, 32608, 48480, 9896,
                53232, 31836, 59476, 49360, 41152, 19296, 64828, 2772,  34096,
                18512, 63504, 8024,  49980, 23136, 63084, 17520, 48060, 59228,
                34348, 62076, 10460, 23068, 38100, 12348, 29648, 1468,  5912,
                63448, 31204, 45548, 1772,  58892, 46972, 13268, 38744, 2856,
                23532, 48468, 8700,  26744, 45872, 7160,  16100, 20512, 34208,
                8624,  23860, 64924, 35792, 51536, 40520, 54144, 6556,  38556,
                47132, 30156, 43812, 57792, 33188, 53548, 43312, 19656, 50088,
                6000,  38828,
            },
            {
                16388, 49608, 35552, 21872, 7484,  17096, 24308, 41164, 8792,
                18904, 1504,  47876, 60016, 30268, 50052, 57516, 37208, 54652,
                156,   44568, 27276, 39876, 45664, 7144,  37452, 11524, 528,
                39712, 63256, 8544,  27052, 21648, 48772, 63664, 3848,  45724,
                41360, 57308, 29208, 24788, 9224,  53360, 33516, 43232, 24256,
                10236, 38680, 22004, 12468, 7468,  33344, 44836, 58176, 36136,
                41356, 59604, 20192, 11944, 27232, 54924, 7652,  65372, 40460,
                23132, 37336, 15556, 44228, 1028,  23752, 38856, 48792, 26472,
                43748, 11676, 28536, 38144, 33412, 2876,  55432, 9840,  20992,
                29616, 16512, 40644, 3276,  63768, 46264, 24520, 36920, 42676,
                26648, 7288,  40124, 62620, 17132, 26776, 6976,  34032, 56712,
                11584, 62520, 41988, 3836,  54264, 32740, 40688, 61764, 55796,
                4908,  28000, 46604, 19844, 9352,  31792, 22572, 34832, 57136,
                16232, 52388, 3300,  13400, 7368,  39276, 1336,  35840, 11716,
                24000, 61956,
            },
            {
                4584,  46456, 13656, 54660, 28292, 57980, 48732, 36072, 59796,
                28484, 56480, 25244, 40228, 12036, 45564, 26344, 14412, 41776,
                29700, 59608, 52968, 12800, 57692, 62832, 27892, 43700, 57060,
                23024, 16260, 52364, 37080, 60644, 9748,  29016, 56512, 12548,
                22652, 63172, 14588, 38268, 45556, 21488, 1508,  58076, 15360,
                50488, 31272, 60380, 41808, 51792, 1260,  21136, 26860, 11196,
                50732, 24416, 46920, 52304, 37656, 536,   15756, 25752, 44964,
                4516,  62960, 11160, 34996, 54216, 29652, 8852,  57776, 15392,
                4332,  55384, 58876, 19972, 13460, 43876, 27412, 36360, 42360,
                53028, 48880, 26784, 57732, 31564, 6860,  60788, 17288, 56676,
                47444, 14516, 20348, 34440, 55612, 42208, 49732, 60000, 19752,
                31524, 51684, 37528, 14468, 23268, 48940, 2676,  25552, 13692,
                58972, 41536, 15944, 60784, 3596,  48108, 63284, 96,    27076,
                10188, 24084, 37344, 64240, 21756, 59040, 25716, 61168, 46056,
                53044, 34036,
            },
            {
                42272, 30072, 64100, 3812,  43392, 33016, 792,   53416, 22164,
                11344, 42628, 7620,  15960, 65136, 5616,  33804, 60672, 10756,
                22232, 35040, 16364, 3032,  24496, 47956, 15572, 34448, 49284,
                4852,  33128, 46932, 2980,  31652, 43352, 18612, 36308, 49964,
                32224, 6912,  52120, 2868,  61868, 56040, 30768, 18996, 64504,
                45212, 2560,  34648, 16176, 28408, 61744, 38740, 14484, 64132,
                1892,  29752, 8056,  63288, 43208, 22152, 57532, 34640, 51140,
                19608, 56516, 26848, 6248,  61344, 17228, 45752, 62604, 35740,
                22212, 31920, 1560,  46564, 51256, 60216, 5972,  64312, 13936,
                7704,  4200,  35616, 18848, 14384, 41604, 52508, 21556, 11000,
                32328, 61444, 50204, 9520,  29404, 3432,  15012, 25172, 40380,
                1412,  27328, 18284, 59004, 65348, 11072, 44112, 36224, 30548,
                50212, 864,   38536, 53568, 25604, 42548, 14264, 45452, 59792,
                50080, 32560, 55552, 41704, 16672, 50628, 31620, 14456, 8204,
                21156, 57716,
            },
            {
                18368, 10956, 25056, 39784, 19764, 60744, 14056, 5796,  45320,
                63188, 33556, 53780, 36652, 23556, 52060, 2328,  19412, 47728,
                6680,  64652, 49668, 38688, 32444, 5488,  21048, 8868,  65516,
                29628, 19136, 59404, 40884, 14244, 54752, 24876, 5656,  61272,
                40312, 20152, 43788, 34432, 26900, 11876, 6204,  39684, 28012,
                8196,  54992, 20028, 57500, 9620,  46184, 53968, 5548,  32988,
                45260, 56888, 14924, 32516, 4884,  28916, 47416, 12236, 2488,
                30820, 39184, 46584, 22160, 50200, 33104, 12564, 5812,  42380,
                53324, 39604, 65524, 24564, 10400, 40028, 19080, 31428, 47084,
                22496, 61580, 54740, 44820, 50056, 9868,  28464, 2496,  35836,
                54232, 23828, 260,   38160, 52908, 22788, 63016, 43960, 10092,
                53892, 47212, 6140,  34472, 29068, 19880, 52588, 7720,  17848,
                45280, 10428, 21980, 31256, 57636, 11124, 29240, 20480, 39656,
                7624,  18820, 1600,  29528, 10444, 46952, 3416,  55124, 39644,
                27852, 288,
            },
            {
                60232, 37136, 56556, 51280, 9476,  46692, 26660, 39252, 30640,
                17028, 48532, 3040,  20420, 56940, 43268, 27696, 39176, 56072,
                31060, 43512, 9064,  19648, 55544, 42480, 58668, 52228, 13408,
                38384, 55896, 11152, 23832, 6632,  64676, 47364, 1612,  27772,
                16076, 55140, 10148, 49560, 17024, 41956, 47852, 52036, 36368,
                13812, 25964, 48924, 3896,  36492, 23804, 18520, 40908, 49752,
                22308, 37276, 19220, 54472, 10312, 60104, 39980, 62276, 17412,
                53836, 8756,  14240, 59068, 3700,  40464, 24960, 51316, 19232,
                27464, 14212, 6932,  35412, 29856, 53940, 184,   25296, 57248,
                38772, 11484, 24804, 900,   33712, 58952, 39596, 62900, 44632,
                5336,  17844, 41932, 57660, 12016, 32920, 47744, 4564,  35220,
                63932, 13728, 56176, 43216, 48,    39640, 57992, 62488, 23764,
                55076, 64144, 34720, 5716,  18012, 37332, 51960, 4832,  34484,
                62304, 43436, 48164, 61308, 26324, 36528, 18968, 43864, 65144,
                32656, 50380,
            },
            {
                7452,  22984, 31432, 1972,  16304, 35848, 64984, 20628, 52152,
                4568,  26156, 59568, 13276, 31576, 8384,  61472, 50852, 15512,
                1444,  25380, 53828, 28572, 61684, 872,   36488, 25428, 46028,
                2452,  26904, 43820, 51044, 20368, 33732, 39300, 53476, 11900,
                58604, 30468, 4748,  65064, 22088, 32072, 60988, 23416, 236,
                62120, 40200, 43636, 64012, 31100, 12612, 60196, 29312, 8656,
                62824, 3744,  41820, 25976, 49868, 35972, 20456, 6952,  43568,
                27640, 64300, 35408, 42644, 19960, 29252, 56748, 1164,  60360,
                9804,  44524, 49432, 18192, 57820, 15412, 43356, 50092, 33564,
                16868, 42620, 29800, 65132, 21020, 5672,  25632, 15852, 49680,
                30028, 59556, 26224, 46052, 6572,  18924, 58440, 16408, 29784,
                21268, 38764, 24072, 9608,  50780, 15420, 31704, 4744,  37756,
                28284, 14772, 43200, 60128, 48948, 2312,  65292, 56152, 25000,
                11384, 15460, 22468, 4900,  53464, 58228, 6500,  24184, 11592,
                16144, 45432,
            },
            {
                41156, 14260, 48104, 61908, 28144, 53988, 7112,  11988, 58628,
                41448, 10196, 44456, 38592, 49444, 17312, 35300, 11572, 22768,
                62892, 37900, 13244, 46324, 17216, 10968, 30560, 41176, 18040,
                61240, 6180,  35024, 62656, 29160, 9084,  16764, 44380, 22464,
                35892, 46732, 24736, 37352, 56388, 3164,  9184,  16336, 53840,
                30212, 10176, 22432, 5924,  52844, 47248, 624,   55152, 16252,
                27492, 12828, 61052, 44672, 16536, 1188,  24548, 56300, 33136,
                48392, 22692, 256,   52836, 10768, 63292, 33996, 47628, 37648,
                31088, 55528, 62220, 2420,  23392, 37180, 63656, 4828,  9468,
                60120, 3120,  52252, 37224, 17536, 47424, 56932, 12404, 38348,
                8416,  34892, 13848, 65220, 36556, 27816, 39696, 51100, 60484,
                7284,  45356, 32476, 61704, 26568, 46800, 21924, 12336, 48196,
                1876,  52052, 7992,  26700, 45772, 23628, 15096, 41728, 30972,
                50944, 57828, 38040, 33068, 13800, 40892, 30512, 60692, 35936,
                2900,  53832,
            },
            {
                63528, 34112, 5772,  43716, 22272, 41804, 48984, 32296, 23508,
                35572, 62400, 28908, 1696,  64124, 25108, 4148,  42128, 46888,
                32752, 58216, 3484,  35352, 23960, 51392, 63676, 7216,  22336,
                53356, 48036, 15732, 140,   55472, 49288, 59564, 31056, 7560,
                62452, 676,   41260, 51628, 13016, 44628, 58756, 34328, 46496,
                19372, 59448, 33764, 17224, 38488, 25120, 42392, 33300, 58712,
                39460, 48800, 33820, 7292,  58272, 31044, 52240, 13740, 38208,
                5392,  16256, 57316, 26564, 45624, 7432,  13928, 22764, 16740,
                3908,  25460, 41032, 33316, 8508,  45984, 12592, 28344, 54868,
                21640, 48332, 13872, 8172,  55204, 41916, 31884, 22604, 63780,
                52008, 20592, 3152,  49428, 22136, 54208, 908,   10780, 26068,
                48976, 2308,  16660, 57284, 3688,  35648, 54384, 42532, 61140,
                33536, 20520, 40024, 11760, 54748, 33096, 19132, 9012,  46760,
                2056,  28548, 64464, 45204, 9544,  51552, 21332, 46488, 56384,
                26416, 20056,
            },
            {
                28660, 10420, 55780, 18420, 59192, 3284,  14984, 57216, 136,
                18836, 50600, 15152, 22356, 45960, 52784, 30116, 59428, 7072,
                19456, 52132, 40720, 6200,  59784, 48432, 14460, 33972, 57456,
                10676, 31532, 42148, 37436, 12344, 25764, 3148,  37884, 52632,
                14548, 27376, 17732, 6068,  20596, 28788, 38968, 24820, 41504,
                4544,  50332, 55568, 13940, 61316, 7828,  20408, 11028, 51484,
                23328, 1672,  20020, 53412, 11920, 40512, 65000, 46076, 9716,
                61508, 51088, 31948, 39020, 18332, 36236, 54464, 43368, 64460,
                51624, 11652, 20392, 48300, 60628, 52504, 19448, 41388, 35672,
                26004, 44352, 31488, 61464, 27216, 10952, 1304,  45392, 4680,
                27632, 42928, 55212, 9672,  31712, 44764, 14684, 42568, 55452,
                36992, 19808, 52460, 40128, 11280, 64264, 18476, 5700,  24872,
                55968, 16436, 62784, 30208, 36984, 6840,  59696, 38912, 53764,
                20904, 6192,  18104, 25504, 516,   63116, 17124, 4124,  8764,
                49788, 38552,
            },
            {
                1392,  52220, 24760, 37492, 11380, 34700, 29584, 39756, 45412,
                54260, 6256,  31984, 55940, 7760,  12476, 36932, 54828, 27408,
                10172, 15948, 29892, 21352, 43088, 26888, 3792,  45220, 39168,
                28084, 19420, 60116, 23228, 46152, 18768, 64376, 42520, 20772,
                48028, 55368, 59140, 35012, 62816, 49212, 15144, 7336,  64756,
                12260, 27688, 1968,  44676, 28864, 48368, 65236, 36720, 4604,
                45456, 30236, 63684, 36980, 27396, 22092, 3316,  18864, 29524,
                25544, 41852, 12600, 4276,  61984, 49864, 2072,  27620, 8648,
                39136, 58172, 29512, 14784, 4076,  24652, 31588, 59072, 6656,
                63060, 18308, 39912, 5296,  50208, 35356, 60676, 53576, 18372,
                33268, 61172, 39284, 17404, 58676, 5756,  62408, 23384, 32736,
                64844, 13000, 30084, 22928, 44484, 28228, 8664,  38776, 30720,
                45112, 4300,  50044, 22024, 888,   63748, 49744, 25392, 13564,
                35952, 42580, 55008, 49280, 34972, 38816, 27988, 42616, 32316,
                13432, 59364,
            },
            {
                44872, 31132, 47448, 4548,  62700, 51632, 25276, 61260, 9640,
                27068, 37620, 42604, 58996, 18124, 40392, 21040, 456,   65296,
                44204, 49228, 55952, 63012, 8828,  31988, 17640, 55108, 1288,
                64500, 7748,  52040, 4132,  57944, 34724, 28736, 5484,  11436,
                33044, 8164,  43264, 30584, 10812, 54128, 1116,  56708, 31528,
                47696, 37304, 23168, 58188, 34496, 2848,  52628, 26628, 14812,
                56936, 6364,  16988, 47492, 9088,  43044, 50316, 33520, 57812,
                1564,  21188, 59540, 47376, 23624, 30032, 57852, 19500, 33176,
                46176, 5348,  35976, 54252, 44060, 38604, 1788,  15896, 50688,
                11752, 672,   58000, 23864, 19672, 14428, 38928, 25008, 10152,
                47624, 14964, 1636,  25304, 47204, 29044, 35784, 18096, 4444,
                8812,  47116, 59236, 6516,  33596, 48628, 58540, 51544, 13512,
                59860, 9552,  41892, 57236, 13992, 43648, 28676, 3904,  56864,
                62180, 31844, 11576, 60332, 22628, 14736, 53664, 48032, 62240,
                23312, 17740,
            },
            {
                7576,  64364, 15188, 40528, 19828, 44220, 7508,  17364, 48024,
                13504, 63976, 2276,  24944, 34288, 62020, 48096, 14932, 33504,
                22568, 37212, 2016,  13100, 35988, 52340, 59104, 23672, 36676,
                49520, 13596, 26296, 40216, 9484,  15200, 51588, 56640, 24604,
                63404, 39112, 22596, 2524,  45364, 25344, 36552, 21472, 17500,
                42908, 53556, 8100,  19652, 41056, 11844, 18132, 43224, 32408,
                62236, 40196, 25152, 55440, 59196, 5116,  62284, 14720, 36172,
                54476, 44600, 8336,  35000, 15532, 6748,  41252, 12784, 61008,
                23500, 17232, 62672, 26872, 10848, 65088, 57012, 47284, 29232,
                37256, 53884, 33360, 46976, 64720, 30096, 43360, 58940, 6992,
                56172, 29928, 36996, 64372, 41288, 11780, 51192, 56652, 39096,
                27368, 53728, 41376, 1744,  62576, 14472, 21544, 620,   35868,
                19540, 27844, 34100, 24580, 47692, 17452, 34668, 10504, 22100,
                45516, 16420, 7448,  40824, 2796,  58708, 10724, 5840,  36148,
                56740, 41760,
            },
            {
                34916, 21748, 57688, 26788, 32780, 54948, 1988,  22880, 57976,
                34776, 20648, 49624, 10700, 44824, 3680,  26124, 51648, 41512,
                5624,  60944, 25560, 46528, 19860, 40068, 11340, 5680,  42824,
                21128, 33396, 44084, 62460, 30744, 47660, 39508, 916,   44856,
                16972, 50708, 13912, 60864, 18620, 39948, 63436, 50556, 9952,
                59768, 26372, 14904, 63232, 30916, 55088, 60100, 7364,  21760,
                49264, 10268, 34960, 13916, 31544, 38464, 23916, 46576, 10788,
                39616, 17988, 52188, 26812, 65300, 55828, 36712, 48352, 53124,
                1004,  42376, 49276, 7672,  34460, 17960, 23160, 42572, 8320,
                20856, 45140, 15364, 9432,  4340,  51948, 2624,  20948, 34360,
                50284, 12560, 22816, 52584, 7864,  20928, 2564,  60408, 45180,
                15488, 20756, 35096, 25532, 17540, 53136, 42072, 26316, 47272,
                65112, 54080, 2984,  38608, 8052,  58648, 53116, 39972, 48576,
                1084,  29788, 52360, 26080, 33540, 44340, 30332, 19820, 25780,
                2340,  11964,
            },
            {
                54196, 48384, 568,   9128,  13208, 49748, 64724, 39088, 31016,
                4288,  53192, 28884, 16488, 55360, 30680, 8920,  56792, 11596,
                31460, 17576, 53384, 29124, 4440,  64112, 50360, 30272, 61564,
                16244, 53748, 3112,  55672, 17868, 6700,  21800, 61460, 35184,
                29612, 6160,  26572, 53480, 48548, 4120,  33148, 28124, 5704,
                34740, 1404,  38992, 49924, 4172,  24396, 46472, 38520, 28812,
                54272, 3556,  44988, 21172, 1804,  52716, 17372, 7240,  27912,
                63820, 3728,  30808, 11784, 43336, 2988,  21692, 9580,  26312,
                31772, 13868, 56336, 21308, 52136, 4808,  32208, 12796, 55836,
                63536, 24772, 41160, 27940, 55744, 36896, 12000, 46412, 27420,
                63484, 42036, 4624,  56856, 16212, 34708, 48860, 24356, 30840,
                3532,  63512, 49836, 11620, 37892, 31000, 56332, 6956,  40012,
                11648, 16784, 51132, 62040, 20464, 30980, 5136,  26620, 64992,
                36796, 57804, 20164, 63424, 47740, 17508, 55188, 65392, 39356,
                51696, 28452,
            },
            {
                18440, 38152, 30180, 60868, 42192, 35640, 28392, 10928, 45768,
                41648, 62140, 7124,  40004, 60324, 37956, 19140, 62828, 45956,
                38896, 58764, 8188,  43432, 56884, 15552, 26196, 35128, 8712,
                47012, 29004, 11684, 23404, 37512, 59024, 27340, 13236, 54012,
                3256,  57940, 37292, 32128, 9200,  13144, 58744, 15992, 45032,
                61892, 22076, 57344, 43888, 10108, 36168, 15952, 404,   12768,
                61048, 18376, 51320, 64716, 29152, 42212, 60460, 56196, 32596,
                48180, 22812, 60796, 38424, 49984, 32964, 17796, 62416, 40224,
                58316, 3756,  37176, 29032, 41464, 61552, 48928, 26632, 35232,
                2316,  5808,  60560, 49380, 18024, 22972, 62712, 40268, 16060,
                12,    19528, 45476, 31340, 62068, 28252, 42728, 13704, 10100,
                55676, 43328, 7332,  58116, 46512, 3228,  10232, 61108, 23496,
                32572, 5844,  29260, 45000, 12740, 42436, 60220, 14520, 18316,
                8428,  43576, 12732, 5292,  37816, 9192,  3552,  49724, 14616,
                45620, 60288,
            },
            {
                23516, 4992,  44496, 52868, 17104, 21324, 5580,  56892, 15060,
                18672, 24448, 47260, 12960, 22732, 50464, 4820,  28516, 24172,
                1316,  13752, 49468, 22424, 33092, 748,   45656, 20292, 56284,
                1992,  38840, 65080, 48840, 33812, 10604, 43460, 49488, 19244,
                41296, 46196, 65404, 20576, 42524, 55216, 23836, 41036, 52428,
                11296, 48064, 32396, 18400, 26896, 64436, 51788, 57448, 33080,
                41084, 26844, 36636, 8616,  48040, 11808, 25876, 20260, 804,
                41436, 57152, 16020, 5996,  24880, 54372, 46680, 29244, 6448,
                44592, 19032, 64052, 45980, 11476, 252,   38888, 59932, 15984,
                51640, 30940, 38140, 13292, 33884, 57388, 30740, 5884,  53788,
                60252, 36428, 25948, 10612, 39156, 1512,  51952, 61236, 38312,
                33004, 18184, 23140, 29180, 62192, 20632, 34944, 44156, 49792,
                58508, 37572, 22544, 55408, 8,     33452, 50136, 23644, 55532,
                28564, 34064, 50720, 24080, 56352, 27920, 42024, 21224, 33304,
                6296,  9876,
            },
            {
                57292, 15548, 63744, 24664, 3092,  59520, 47804, 26152, 51896,
                912,   36608, 54328, 31584, 1768,  35252, 15244, 42284, 52300,
                20328, 35456, 61820, 40816, 10288, 37888, 60688, 13384, 41192,
                59788, 25952, 19680, 5148,  52940, 1472,  63100, 31360, 7880,
                23688, 11608, 16080, 6616,  28984, 47584, 32,    19508, 36788,
                29384, 4920,  13528, 53256, 6416,  30256, 20544, 47788, 23756,
                6728,  43844, 14728, 57612, 33680, 5576,  37740, 51004, 14224,
                34796, 10024, 45756, 19980, 59268, 384,   11012, 15344, 52284,
                34508, 24320, 8284,  15452, 25820, 53504, 18580, 7244,  46556,
                43380, 19568, 54344, 7912,  1040,  44064, 9904,  49076, 23844,
                32588, 8228,  58008, 47944, 54648, 23440, 19196, 5644,  47508,
                26732, 52688, 1104,  40472, 15440, 51144, 27312, 13264, 18404,
                2112,  46720, 14876, 63992, 26176, 9660,  37712, 3656,  45928,
                41128, 1860,  62560, 15536, 31580, 61044, 11804, 36420, 25132,
                63076, 31008,
            },
            {
                39840, 34104, 11096, 50324, 36776, 31692, 8704,  38648, 63196,
                33280, 58220, 9848,  43416, 63600, 57116, 47896, 6916,  64176,
                55412, 30308, 5152,  26712, 18360, 53140, 48900, 23528, 6208,
                51332, 32888, 14664, 45004, 40504, 25376, 16228, 55416, 38824,
                60308, 33352, 51400, 25416, 59700, 34416, 63652, 8452,  56488,
                24516, 63020, 37668, 40960, 60176, 44776, 2948,  39400, 10696,
                62920, 55392, 1368,  22340, 61380, 16576, 44864, 55144, 61948,
                5088,  27524, 52652, 30240, 35384, 41860, 63464, 38676, 22044,
                60652, 49800, 55160, 31572, 59060, 36028, 22588, 29968, 57368,
                10656, 23460, 64340, 47440, 26240, 61396, 19260, 38220, 13888,
                43620, 51244, 17572, 4924,  14276, 63412, 36836, 8992,  58776,
                14660, 65140, 35996, 45228, 8304,  55272, 4008,  64180, 29960,
                54388, 8836,  40612, 35052, 48828, 17392, 61896, 52668, 12092,
                58496, 21608, 47456, 39432, 2768,  44516, 51892, 58904, 768,
                42956, 50880,
            },
            {
                47316, 2020,  27712, 41420, 13948, 55516, 19332, 44052, 12332,
                21904, 6172,  28200, 16924, 20708, 26684, 11300, 31860, 37524,
                9812,  16204, 47100, 57304, 64764, 31048, 3572,  28664, 35800,
                17680, 9180,  55856, 30164, 58644, 21124, 36460, 4100,  28096,
                47464, 1068,  56772, 44356, 5156,  39244, 14524, 31232, 46116,
                2908,  16616, 55284, 21632, 9508,  34652, 14376, 58848, 19184,
                49112, 31612, 25396, 53112, 40272, 28184, 2732,  30924, 21368,
                48892, 37984, 64760, 12980, 7836,  56836, 26704, 32252, 5080,
                12932, 1484,  39248, 47388, 5176,  43052, 64640, 13668, 40248,
                4352,  28352, 35064, 41396, 14808, 52088, 29148, 58960, 2236,
                27288, 65504, 22056, 41444, 33964, 27708, 44960, 30352, 21196,
                42372, 4652,  12012, 57672, 32028, 23780, 39672, 35912, 42812,
                25060, 60720, 19996, 3920,  57068, 43560, 30448, 20616, 6852,
                32356, 26808, 9300,  54096, 19120, 23256, 7660,  17196, 55492,
                13476, 19476,
            },
            {
                54040, 21588, 62132, 7320,  45936, 65268, 29080, 4280,  48856,
                41836, 60748, 50932, 46060, 3248,  40584, 60884, 18236, 45016,
                25500, 50804, 2332,  42344, 13036, 7772,  44216, 62684, 54444,
                46940, 41908, 63712, 6868,  12636, 46256, 50656, 10076, 64384,
                14576, 21780, 36196, 10712, 17720, 53652, 22744, 50828, 61016,
                42652, 26984, 50036, 808,   47344, 25436, 54528, 36780, 28276,
                4028,  16236, 36200, 10044, 47044, 19148, 64128, 8488,  42860,
                24364, 16848, 1824,  43988, 23208, 49688, 18744, 47192, 56024,
                42772, 28936, 17712, 10856, 20676, 50888, 2864,  33400, 48264,
                61712, 16724, 56376, 3180,  21848, 36496, 4780,  46592, 55340,
                11424, 35700, 3192,  46748, 59484, 11092, 53408, 560,   56424,
                33684, 50612, 25648, 19228, 48688, 60404, 16716, 6628,  52236,
                11996, 32404, 50420, 28932, 6316,  13340, 24888, 40140, 64664,
                16428, 37112, 60476, 13320, 35196, 64528, 46848, 29096, 38440,
                26724, 5788,
            },
            {
                36240, 30004, 56624, 17864, 23180, 40,    34524, 53312, 16296,
                25536, 31288, 13616, 37664, 56048, 33436, 52800, 23156, 5776,
                58452, 38880, 20808, 33592, 24016, 37288, 15740, 21296, 11180,
                44,    27104, 22772, 37676, 2356,  61796, 32972, 18624, 43340,
                52984, 40808, 30604, 61700, 48836, 27848, 3664,  12068, 19172,
                34928, 10276, 32668, 65288, 29712, 17884, 62208, 7376,  51072,
                45404, 64916, 42404, 6264,  59320, 13440, 50460, 35496, 11692,
                58492, 54188, 33336, 60032, 39744, 14608, 3068,  9496,  36600,
                65180, 25116, 58152, 34736, 62200, 27532, 9276,  54860, 25424,
                37400, 8736,  50020, 31872, 10920, 63732, 42304, 16028, 20668,
                40080, 30480, 54340, 25256, 6764,  18000, 49372, 40508, 24268,
                16984, 62592, 38696, 29432, 1576,  10776, 45880, 21656, 58944,
                1352,  16012, 44700, 23332, 61304, 35528, 46856, 1584,  54596,
                49040, 43696, 5724,  51248, 25828, 40812, 4448,  33652, 49320,
                9772,  60972,
            },
            {
                16008, 3476,  43604, 37788, 51580, 26772, 60520, 10560, 37284,
                55316, 2132,  64420, 23520, 5188,  9296,  14776, 49252, 300,
                28888, 61668, 10740, 52828, 46320, 60148, 56240, 32308, 39416,
                57996, 50388, 33916, 16728, 53660, 28532, 23760, 57896, 5036,
                26624, 7988,  19560, 2872,  42552, 64328, 37448, 57256, 40152,
                5896,  45360, 13924, 56136, 4552,  43652, 11772, 41460, 22460,
                32956, 12900, 51984, 29084, 21832, 32636, 55572, 25736, 39560,
                6540,  46964, 20344, 4752,  51652, 29464, 61136, 53272, 21044,
                15480, 45040, 6620,  53852, 41228, 15008, 45544, 17928, 59328,
                668,   20916, 45176, 59772, 53912, 25052, 6816,  33660, 61068,
                49628, 8928,  62356, 15276, 32172, 38472, 64288, 4284,  13152,
                47020, 9708,  6456,  53484, 42032, 62936, 33392, 28356, 48056,
                37116, 65424, 55048, 9796,  39116, 51748, 57768, 10868, 33604,
                19556, 29352, 364,   56248, 30960, 10328, 15140, 57080, 63420,
                22580, 44656,
            },
            {
                12008, 59152, 33056, 8988,  13008, 46968, 41028, 21372, 58660,
                8020,  19572, 40864, 47848, 28772, 58832, 36808, 65224, 41788,
                34948, 54784, 15432, 6216,  28056, 2704,  50072, 25852, 5660,
                19064, 13336, 60236, 42584, 48696, 7860,  40044, 13556, 35432,
                49376, 59872, 54536, 12964, 24632, 33448, 8576,  48152, 24236,
                59316, 52640, 20732, 39004, 23876, 33960, 52484, 2040,  26792,
                60760, 19896, 144,   57088, 37960, 4688,  44468, 1132,  60844,
                14804, 31276, 27016, 10860, 36012, 45660, 24740, 31688, 40624,
                2352,  49516, 30636, 22324, 1172,  37936, 31908, 22756, 52256,
                42724, 29676, 13788, 39016, 18208, 48340, 30204, 56504, 23724,
                476,   18592, 42456, 51380, 57680, 21492, 27060, 35568, 61188,
                30968, 44104, 58140, 22412, 36288, 14644, 54916, 8572,  41212,
                5328,  26588, 34044, 18796, 3296,  15288, 22032, 27544, 8104,
                63048, 14400, 38732, 21856, 47820, 59764, 43272, 19968, 1232,
                31824, 51988,
            },
            {
                41508, 49172, 25268, 61636, 56084, 17012, 5932,  32240, 49880,
                45328, 34860, 11520, 52516, 17396, 43916, 20844, 26600, 12172,
                18680, 44400, 30912, 64028, 40468, 17992, 9456,  42940, 65512,
                45828, 29336, 3304,  10020, 20472, 56880, 44900, 1740,  63168,
                22264, 32084, 37752, 46088, 58724, 15636, 20824, 30668, 996,
                16124, 28580, 61348, 8120,  48624, 58404, 15352, 37408, 56544,
                8972,  39908, 46524, 24660, 62752, 17164, 48816, 29840, 18948,
                53004, 42464, 62544, 55844, 17020, 63792, 6968,  12556, 59376,
                34212, 9100,  61548, 12272, 47548, 56484, 63636, 6704,  11244,
                34372, 65148, 5728,  27324, 1556,  44116, 9860,  13600, 38092,
                45740, 28652, 36152, 12384, 2096,  44412, 9272,  54876, 20124,
                51848, 2620,  27384, 18068, 49128, 3912,  25324, 17632, 57836,
                20740, 13280, 42996, 49292, 62480, 31708, 45128, 60152, 41216,
                50724, 24992, 45352, 61752, 17416, 3488,  53656, 25032, 39540,
                7956,  27364,
            },
            {
                18456, 6292,  20972, 1432,  39588, 28508, 63916, 2412,  14704,
                29440, 61624, 26208, 924,   62796, 31404, 6824,  55808, 47516,
                8260,  24432, 1688,  49332, 22280, 34316, 54268, 14364, 36264,
                23536, 52024, 38640, 63472, 25164, 36232, 31052, 18020, 47224,
                9892,  16000, 436,   29056, 5324,  52792, 44304, 55876, 63264,
                42180, 36068, 3384,  31840, 18588, 63880, 6092,  30804, 47588,
                17512, 55020, 34004, 7492,  13944, 41724, 9592,  63872, 23064,
                36604, 3376,  8312,  39236, 21768, 1496,  42672, 48192, 18348,
                55112, 27064, 51444, 36076, 19116, 4360,  28816, 49456, 39724,
                16460, 56004, 50660, 23172, 62056, 35516, 51940, 59916, 5472,
                63952, 53820, 4488,  24180, 48536, 60504, 30092, 6052,  42248,
                15540, 37812, 34140, 59904, 12104, 64004, 39104, 50892, 30624,
                61516, 52820, 412,   29424, 55704, 6792,  36864, 17444, 4724,
                35348, 58024, 11184, 6600,  34556, 28316, 36820, 12568, 46664,
                55960, 62852,
            },
            {
                37328, 54008, 31152, 35812, 45068, 50648, 24280, 54772, 42152,
                22676, 6372,  55480, 36140, 13760, 50404, 39384, 4224,  32652,
                51348, 60448, 35788, 58068, 12376, 46820, 61808, 30704, 1320,
                58736, 7424,  32744, 54872, 14948, 6404,  61128, 51800, 27208,
                55984, 42328, 65204, 50560, 39996, 11080, 34260, 6696,  13140,
                25824, 51424, 46420, 12308, 40660, 27356, 43932, 21092, 62440,
                12552, 2992,  29776, 50576, 59532, 27440, 34344, 52012, 12300,
                47644, 58348, 25440, 49536, 33432, 28196, 57700, 36912, 22444,
                4988,  44024, 16368, 24588, 42004, 60156, 14208, 26088, 58572,
                3016,  46200, 32448, 8524,  55192, 16704, 21344, 41532, 26492,
                14940, 20148, 57072, 33272, 39648, 17384, 13736, 47288, 58612,
                25288, 10704, 54568, 45572, 7540,  32284, 43536, 1924,  11200,
                45464, 37876, 23992, 9356,  40712, 25972, 12152, 48808, 53936,
                2264,  20784, 31632, 52896, 41744, 8844,  65332, 50300, 16612,
                34220, 4072,
            },
            {
                13344, 47268, 64576, 15028, 10608, 19360, 34156, 8532,  38024,
                59276, 18088, 46784, 42592, 9452,  24336, 58016, 17000, 63032,
                40700, 14164, 19708, 38800, 7512,  26508, 4612,  20340, 40780,
                48412, 11604, 18244, 43032, 27924, 46568, 12160, 3584,  38484,
                34376, 7052,  25232, 18084, 22656, 62044, 27120, 49032, 39340,
                20084, 9576,  57900, 22864, 54392, 600,   10536, 50992, 25308,
                41800, 36672, 22256, 44620, 19432, 54484, 1996,  40388, 6252,
                32616, 16320, 44508, 11256, 54844, 14580, 51744, 9596,  64608,
                29860, 39212, 62512, 7352,  52808, 33484, 44916, 9060,  36280,
                23940, 19252, 12412, 42256, 38292, 28964, 2720,  33024, 49908,
                8356,  30656, 43564, 10496, 64580, 53328, 23248, 32852, 3968,
                39800, 65072, 52,    23620, 19216, 28500, 55932, 21616, 34976,
                6580,  16776, 63300, 46724, 19316, 57528, 64388, 23076, 28600,
                42852, 13908, 63668, 26332, 48756, 23020, 58640, 2604,  28956,
                21712, 57972,
            },
            {
                26276, 8636,  22960, 52308, 59008, 3044,  62228, 48080, 12588,
                51576, 4312,  32412, 21532, 60908, 29264, 45008, 1380,  22492,
                27656, 53456, 3956,  29672, 55692, 44092, 51184, 57336, 16272,
                27456, 62100, 22648, 50188, 1460,  57728, 40744, 23852, 19640,
                59036, 13684, 47808, 55168, 36628, 2348,  59576, 17588, 54136,
                1808,  64748, 33180, 16572, 49388, 37948, 57280, 34684, 4504,
                58652, 53308, 64000, 11056, 4876,  37692, 25128, 56532, 20936,
                28396, 65396, 372,   20288, 60888, 41300, 2996,  25856, 46400,
                13708, 480,   56104, 11140, 30404, 1908,  21088, 54696, 40984,
                63280, 28524, 60396, 48424, 4512,  64476, 11500, 57488, 46448,
                37368, 59472, 51036, 6340,  26940, 828,   36696, 62188, 50112,
                20612, 29320, 48308, 41772, 52076, 61284, 15176, 48708, 58352,
                27752, 53904, 32708, 13984, 51432, 2600,  38788, 6344,  33244,
                55928, 46924, 37228, 820,   14992, 19176, 43540, 31912, 10360,
                45116, 39828,
            },
            {
                60108, 43640, 29780, 5564,  40580, 26388, 43404, 16356, 30748,
                26836, 64060, 40148, 15056, 53268, 7256,  34688, 37828, 11624,
                46368, 8948,  42292, 63432, 16860, 33172, 23408, 9112,  35180,
                44904, 54148, 4980,  36724, 64904, 33408, 15868, 53336, 62384,
                30444, 44504, 3380,  32632, 9248,  43388, 14068, 31228, 37092,
                45288, 28412, 42976, 4644,  61936, 29372, 19012, 46044, 14764,
                8344,  27748, 16064, 32804, 48404, 62296, 14320, 46024, 60048,
                41612, 51188, 37880, 30628, 6740,  35732, 17504, 32440, 60256,
                50048, 34340, 19728, 47032, 38284, 64868, 48580, 17324, 5460,
                51076, 172,   34888, 15424, 22116, 44432, 53100, 18628, 24472,
                1252,  16832, 22368, 38624, 46708, 19380, 42680, 8484,  12420,
                55816, 16164, 5880,  33444, 13052, 4180,  37124, 9124,  24644,
                41920, 60056, 3560,  22556, 35608, 30172, 44612, 15504, 61448,
                9220,  18732, 5432,  60536, 56604, 39392, 6136,  54256, 61736,
                51228, 308,
            },
            {
                33308, 49632, 18004, 36448, 56584, 31992, 21452, 58020, 396,
                36380, 10760, 56080, 3100,  47948, 18424, 64932, 49556, 57096,
                32148, 59868, 24988, 47708, 11696, 332,   38016, 64524, 13368,
                2644,  31316, 39852, 10544, 20364, 29272, 8308,  43204, 4364,
                10924, 51096, 27300, 63516, 20488, 57064, 47176, 4492,  22036,
                60588, 7472,  24692, 35824, 13196, 7776,  24152, 65192, 32116,
                39664, 49304, 832,   43452, 23348, 7384,  30992, 17932, 9000,
                4244,  13240, 54124, 24248, 47216, 63480, 43884, 21240, 4508,
                40952, 24076, 59220, 27768, 14896, 23396, 57752, 12076, 31568,
                43504, 10316, 56908, 25640, 59232, 30876, 8036,  39488, 61368,
                34756, 11956, 62976, 29220, 14112, 60076, 56648, 30772, 26544,
                44616, 35720, 61728, 57784, 26252, 40444, 46464, 64156, 19488,
                4972,  12252, 49344, 43532, 8240,  59464, 53120, 21164, 48436,
                25088, 40368, 30356, 50920, 33668, 27256, 12684, 35876, 24232,
                15232, 20000,
            },
            {
                7408,  12576, 62640, 3516,  46688, 11412, 7544,  53036, 45332,
                19124, 49596, 23448, 28256, 38496, 12504, 25364, 5572,  20356,
                15656, 2556,  35500, 21636, 52424, 60740, 41556, 28648, 47164,
                58328, 24532, 17176, 60052, 47912, 56244, 25776, 49684, 35604,
                22612, 38784, 16580, 52764, 40964, 29396, 25016, 50808, 10864,
                56076, 14716, 53096, 47340, 59132, 41964, 55084, 1936,  52548,
                21644, 61032, 18832, 55116, 40184, 58584, 51636, 36856, 26592,
                48884, 34728, 18652, 57424, 9712,  27548, 52336, 11456, 55496,
                16508, 8924,  53352, 5632,  43836, 3108,  35320, 26828, 61296,
                38568, 21000, 52768, 6368,  36468, 49992, 14408, 28132, 4588,
                48740, 41372, 55576, 3312,  51732, 34284, 5536,  17040, 52852,
                1592,  38908, 10028, 21996, 53772, 18216, 992,   31492, 51816,
                36096, 29580, 39336, 65172, 17944, 26824, 11404, 1640,  36204,
                57576, 12456, 44152, 22372, 8996,  46084, 63388, 48364, 4324,
                37980, 55908,
            },
            {
                28092, 41340, 24728, 53524, 15368, 38460, 65476, 24044, 40844,
                61340, 6612,  58420, 44476, 33224, 59988, 53956, 30088, 43284,
                51452, 38644, 56140, 7324,  31028, 18724, 5076,  53552, 20504,
                7976,  51420, 42448, 5984,  14048, 44660, 248,   63620, 18140,
                57480, 7676,  60836, 1140,  12680, 6552,  35344, 63044, 41272,
                32604, 19424, 39420, 2532,  21484, 33528, 16872, 44396, 11776,
                29544, 36484, 9916,  26464, 34632, 12088, 1420,  63316, 43308,
                22124, 61488, 2172,  40208, 15468, 3696,  58824, 38540, 30508,
                45192, 36744, 63944, 31768, 51384, 39708, 53848, 7580,  47512,
                14248, 29200, 45876, 18396, 41204, 2168,  63508, 43676, 20180,
                53400, 25728, 31936, 44792, 9428,  24672, 49048, 40876, 63496,
                23680, 47420, 31012, 6960,  49592, 28684, 11744, 57144, 44668,
                15744, 54544, 25464, 664,   55652, 34028, 41936, 63276, 28304,
                52248, 3184,  64508, 17076, 54780, 1832,  20768, 16448, 30904,
                42880, 64860,
            },
            {
                2568,  35016, 59136, 21108, 28756, 49932, 34088, 3852,  13748,
                29760, 34788, 15688, 2116,  21032, 8896,  40620, 544,   62516,
                10464, 28076, 64248, 14012, 45012, 49204, 26000, 14996, 36532,
                61816, 33808, 27176, 55228, 35164, 22148, 38344, 11736, 31848,
                42044, 46432, 24504, 33020, 48980, 19160, 57916, 16500, 148,
                27004, 49224, 64260, 30596, 9676,  50784, 38216, 25660, 6668,
                46684, 58144, 3352,  64544, 45708, 21148, 29400, 15568, 54884,
                31828, 10912, 45504, 29696, 50676, 33892, 23380, 7188,  62884,
                25696, 1604,  13248, 21988, 17820, 10168, 62504, 19280, 24288,
                56216, 1732,  65280, 32548, 12540, 55200, 23452, 33896, 58936,
                10516, 6504,  15632, 19756, 65500, 37736, 21280, 11732, 4160,
                58396, 19276, 13784, 64788, 43508, 59756, 35032, 23340, 7608,
                60552, 20584, 9624,  47076, 14312, 50620, 4776,  18936, 46236,
                15104, 32824, 7032,  37964, 29076, 41564, 57988, 51440, 26552,
                10164, 52804,
            },
            {
                14104, 47968, 8564,  44028, 1100,  55616, 17776, 27164, 48388,
                9400,  54164, 42296, 64244, 51520, 46840, 17184, 35776, 23080,
                46180, 18180, 4336,  40212, 34272, 59460, 9664,  56056, 43912,
                1240,  12288, 18920, 63948, 3072,  30536, 61024, 53712, 5028,
                28112, 13976, 54972, 37496, 65056, 45264, 38972, 9008,  52472,
                43476, 5648,  12216, 56616, 27780, 60960, 3844,  63440, 56268,
                40404, 17608, 51360, 14652, 6384,  50012, 57592, 4648,  39572,
                7168,  58720, 25080, 64824, 20252, 42980, 47904, 14432, 19464,
                49696, 57888, 42084, 46476, 59908, 29012, 33424, 41676, 4220,
                50668, 36888, 9616,  59656, 26968, 47352, 8492,  16624, 29916,
                45996, 61788, 39348, 57512, 1112,  28164, 59332, 45720, 35332,
                27528, 51532, 40552, 33744, 3424,  16140, 38500, 48524, 2192,
                40804, 32900, 62420, 37468, 22084, 57932, 31268, 38692, 8528,
                23232, 58836, 48068, 25340, 61596, 10772, 34528, 6872,  60400,
                45616, 21912,
            },
            {
                57696, 39316, 18956, 31264, 60612, 10264, 41952, 62336, 57228,
                38192, 19616, 25784, 11800, 31448, 27308, 55100, 13744, 57648,
                32612, 50476, 25564, 54156, 22360, 2180,  29724, 39520, 22920,
                31980, 46544, 49804, 9556,  41132, 51328, 15776, 20072, 48252,
                59148, 2684,  21200, 10128, 27436, 3588,  23444, 30008, 62260,
                34760, 24240, 17832, 37416, 44908, 15312, 20568, 30892, 13376,
                33832, 23220, 28224, 42916, 32300, 37968, 24160, 46596, 19088,
                52068, 14008, 37036, 5636,  55332, 1456,  60800, 36040, 53184,
                4164,  28400, 35088, 7080,  50064, 556,   15324, 45468, 58556,
                31416, 17480, 44108, 22456, 3404,  51528, 38044, 56620, 388,
                36376, 22172, 50500, 33492, 42712, 14304, 54276, 31796, 8448,
                15932, 56416, 5436,  24892, 54680, 20664, 9528,  63604, 26320,
                13716, 50812, 4104,  28680, 45124, 6828,  25260, 61000, 54928,
                42432, 35620, 19324, 52848, 44604, 14832, 23364, 39956, 936,
                17676, 32800,
            },
            {
                50548, 4052,  64208, 35612, 14392, 46204, 21628, 5168,  32376,
                1728,  45628, 60388, 5628,  39332, 3884,  61856, 7584,  41316,
                3156,  60368, 8980,  42912, 12208, 62792, 17492, 50856, 65360,
                4572,  57564, 37404, 28940, 23600, 45680, 6752,  26008, 39516,
                34884, 50924, 44128, 59888, 15376, 50100, 55720, 11832, 20812,
                46000, 60512, 53580, 1388,  48764, 7736,  42232, 53200, 49552,
                416,   62468, 8796,  54088, 60772, 17212, 10436, 64056, 35480,
                27260, 48324, 42368, 17528, 32144, 12268, 27880, 9196,  41100,
                31976, 16808, 10796, 24372, 56272, 36572, 64572, 22600, 6648,
                26380, 12968, 54460, 40648, 34856, 19028, 62016, 41972, 13488,
                52656, 4784,  26144, 11240, 7068,  48204, 18608, 2284,  61688,
                44068, 37012, 11292, 61096, 45812, 28044, 41928, 52544, 30732,
                56448, 43612, 17256, 59048, 11968, 40920, 16324, 48568, 10884,
                100,   29644, 12792, 6164,  2408,  31960, 48692, 54404, 36304,
                63144, 24972,
            },
            {
                29596, 11260, 54496, 26048, 6636,  52860, 37320, 25160, 51612,
                16084, 22840, 50604, 36008, 57832, 18384, 24156, 49100, 29140,
                20320, 36296, 16096, 30828, 57052, 37248, 46756, 7176,  13444,
                26456, 21248, 16516, 5748,  54064, 59516, 11136, 32236, 64628,
                9216,  17524, 30492, 5900,  35880, 42364, 32436, 58844, 2324,
                15068, 8460,  40940, 26084, 33920, 58424, 23456, 36616, 10324,
                19320, 47308, 35852, 4188,  25760, 41312, 2936,  30064, 53796,
                108,   22832, 9464,  62064, 51240, 39700, 46356, 24704, 55096,
                59804, 47436, 63312, 39308, 19864, 27444, 12132, 52540, 40048,
                61156, 49008, 4532,  63756, 14980, 30420, 6288,  24028, 27760,
                64068, 18060, 46828, 56100, 30148, 62876, 38812, 25408, 50260,
                20432, 29884, 48360, 320,   32504, 14096, 58480, 3028,  18664,
                36792, 7312,  24144, 34456, 53512, 63500, 3544,  33728, 20952,
                65076, 50292, 59776, 40604, 56508, 63776, 19952, 27668, 12832,
                42220, 7444,
            },
            {
                16688, 43552, 22896, 49236, 40360, 29728, 58252, 12388, 64668,
                41240, 7228,  29980, 14072, 47780, 42656, 34052, 53180, 11460,
                45420, 64956, 47932, 52352, 5260,  20452, 27660, 34824, 42752,
                59840, 52192, 39864, 62996, 14024, 35688, 42640, 55524, 1328,
                23308, 41004, 61988, 25136, 53636, 18932, 7504,  39632, 48244,
                36684, 31624, 57376, 19804, 12424, 64876, 4860,  27476, 60428,
                43740, 30024, 57328, 21080, 13420, 50796, 59204, 44284, 12716,
                60088, 34336, 56264, 28796, 3616,  21704, 64296, 18412, 6520,
                13732, 22408, 2100,  44544, 54100, 4040,  47884, 9168,  33984,
                19688, 37684, 28384, 8788,  57772, 46276, 50172, 10820, 44712,
                32828, 40264, 59388, 15812, 35524, 23036, 52492, 9800,  41248,
                6104,  64200, 23428, 17952, 51288, 37708, 6800,  47228, 22640,
                11432, 64892, 48796, 884,   20128, 29756, 51040, 26376, 37356,
                44744, 16864, 23568, 27268, 36872, 16276, 8268,  59636, 3452,
                47356, 56692,
            },
            {
                60456, 37412, 368,   62680, 15860, 2820,  19760, 8860,  47096,
                34368, 56404, 62668, 10084, 21564, 2244,  63532, 15780, 38168,
                1268,  23468, 7456,  25980, 39760, 1684,  61216, 55376, 9264,
                30908, 608,   44984, 32992, 2740,  27008, 18776, 46928, 15516,
                52328, 57204, 12392, 46372, 796,   63576, 14084, 25936, 52084,
                22452, 63892, 3816,  50304, 29872, 46080, 17204, 40500, 55832,
                6036,  15840, 39216, 65356, 46216, 34056, 7416,  21512, 40056,
                16604, 6320,  44664, 37976, 14676, 57628, 35044, 660,   43680,
                37836, 51048, 33472, 59128, 14488, 31212, 43060, 58232, 15728,
                956,   45312, 23632, 51912, 33008, 1448,  21004, 60028, 54620,
                3104,  9104,  21668, 1736,  44008, 5208,  13404, 59612, 33108,
                54628, 12756, 42992, 57404, 8720,  62808, 25356, 34092, 60480,
                40428, 54060, 27372, 45552, 38392, 8340,  43100, 13180, 57220,
                6988,  31756, 9648,  46984, 4696,  51104, 41456, 30396, 52244,
                21520, 34016,
            },
            {
                5120,  46712, 20404, 32840, 55404, 43800, 60780, 31648, 26676,
                940,   17892, 43704, 27560, 54396, 31808, 8668,  26488, 59676,
                55820, 33672, 13652, 58260, 44560, 17064, 50308, 23400, 15356,
                48124, 19772, 10976, 57124, 48904, 21708, 62500, 5520,  37172,
                28176, 3740,  33488, 38372, 21824, 29520, 44260, 60224, 5384,
                28656, 16820, 42756, 9368,  37268, 53864, 1148,  33200, 12024,
                25360, 52592, 1656,  10156, 27888, 18312, 56120, 26740, 32060,
                63112, 49776, 19700, 25004, 48572, 7536,  52832, 30880, 61920,
                25488, 9604,  29204, 5476,  18160, 38300, 63124, 24696, 29412,
                55264, 62396, 11372, 17840, 42632, 25924, 39268, 35112, 15308,
                29984, 49396, 37480, 53332, 64908, 28372, 46504, 36828, 16876,
                27180, 1952,  34872, 39496, 28612, 20140, 55032, 44408, 15824,
                4848,  31964, 9776,  15256, 62172, 55684, 18212, 60572, 1752,
                52412, 39628, 61976, 55724, 33760, 22072, 45344, 11064, 38064,
                14852, 26908,
            },
            {
                41220, 53708, 13200, 9760,  27416, 36632, 50228, 14000, 39536,
                52560, 23816, 37548, 4452,  59960, 39692, 51620, 43872, 5688,
                18812, 41732, 28628, 62496, 35280, 11276, 32348, 41144, 4252,
                36784, 63680, 24828, 29344, 38564, 9332,  50632, 30588, 59704,
                44452, 20060, 49160, 8064,  55956, 50596, 10004, 34296, 54616,
                11640, 46360, 55712, 24980, 14824, 61848, 22328, 50884, 63328,
                20268, 48044, 31888, 42648, 53880, 38236, 2552,  47536, 9756,
                52552, 1700,  30324, 61436, 10480, 40400, 27056, 15876, 49116,
                56852, 19692, 64900, 41896, 52676, 8128,  21388, 3212,  50952,
                41044, 6512,  35472, 59544, 55680, 13184, 64740, 4684,  19440,
                42156, 61568, 26364, 11168, 32056, 19660, 57420, 3128,  22228,
                48780, 60328, 45736, 15212, 3832,  49316, 12280, 1012,  29132,
                51056, 20484, 57484, 35636, 3932,  23908, 32536, 47092, 22380,
                28632, 14788, 20184, 2500,  13120, 26244, 64656, 756,   55456,
                62164, 9068,
            },
            {
                30752, 65272, 24616, 57928, 47556, 6804,  22616, 3900,  55628,
                61664, 11516, 49416, 15416, 46104, 19532, 13056, 22872, 30728,
                49352, 53688, 9668,  21160, 48712, 5960,  64636, 28868, 59176,
                52520, 42100, 17004, 54412, 6336,  58576, 41068, 13216, 24676,
                10120, 63924, 15120, 60492, 32068, 17168, 3292,  40592, 19460,
                37800, 1476,  59456, 34172, 41264, 7396,  28836, 44768, 38032,
                8208,  35512, 58336, 22996, 5056,  62836, 15472, 57872, 35684,
                23092, 41672, 13524, 33664, 58804, 45164, 21068, 11464, 41324,
                2756,  36396, 12952, 48440, 26448, 34180, 56736, 45808, 12896,
                32104, 20656, 49220, 2448,  29708, 7824,  47024, 51524, 56340,
                23704, 6400,  57912, 17320, 39804, 50412, 8424,  42684, 62904,
                30996, 10300, 24860, 53248, 65388, 32200, 42120, 59116, 37220,
                62568, 47860, 25868, 42624, 52464, 11480, 41256, 4812,  35504,
                63816, 42844, 49564, 36320, 58808, 53528, 17756, 32188, 23792,
                49152, 19208,
            },
            {
                1828,  50912, 34536, 3288,  41740, 18232, 64212, 34804, 44812,
                20588, 6332,  31372, 65432, 34848, 724,   58520, 37004, 64224,
                3444,  16412, 39060, 164,   56400, 25184, 53604, 13328, 21016,
                7984,  34404, 3244,  13620, 44296, 33252, 17748, 452,   52016,
                35752, 40640, 4616,  27224, 42336, 23592, 65116, 47568, 25272,
                62216, 30524, 21480, 4664,  48216, 18564, 57644, 2776,  14444,
                27228, 60752, 17068, 12444, 30496, 49012, 25096, 43424, 12128,
                4712,  64076, 54564, 18072, 5444,  2204,  55296, 60856, 32552,
                6120,  45336, 54908, 22708, 420,   61584, 10332, 36740, 17592,
                65036, 27048, 44280, 39556, 22800, 16836, 37088, 27200, 10068,
                45072, 13968, 33888, 47612, 60,    24412, 14620, 52672, 38516,
                4480,  56008, 19188, 6576,  36656, 21976, 9044,  17620, 23660,
                7208,  13164, 1872,  18552, 58924, 30080, 49940, 16712, 54436,
                10808, 25024, 5608,  45832, 29820, 7204,  38836, 43584, 6244,
                35964, 45784,
            },
            {
                27776, 17280, 38504, 14624, 59952, 30076, 51936, 8676,  15752,
                28724, 41668, 57000, 24720, 8244,  29052, 50116, 10472, 45108,
                26992, 60600, 46700, 31320, 42900, 17824, 38212, 2816,  44936,
                27296, 57640, 47488, 61480, 22592, 26752, 65256, 46532, 56580,
                31684, 23116, 53752, 46868, 11708, 36960, 57748, 14748, 6272,
                50472, 43628, 13112, 52952, 60912, 31816, 10940, 49784, 41588,
                54424, 5864,  44072, 51488, 36384, 8840,  61056, 19100, 29144,
                46048, 36948, 25808, 47372, 38804, 28420, 36016, 51584, 24188,
                17296, 27944, 58592, 39412, 15976, 43724, 28604, 48852, 5144,
                54864, 9056,  14648, 61092, 53984, 33372, 62612, 972,   31600,
                38916, 64092, 4372,  43412, 55132, 62320, 34896, 27404, 20572,
                13076, 33924, 50160, 43768, 58380, 26936, 46852, 54688, 40096,
                34444, 44840, 64252, 38224, 7556,  26428, 63008, 39044, 8180,
                57848, 33100, 18756, 61144, 40716, 10424, 50524, 15340, 60060,
                56920, 12396,
            },
            {
                61164, 7716,  44912, 53304, 23712, 11304, 37972, 25320, 58128,
                48500, 2544,  36212, 52320, 21364, 55556, 41408, 17208, 33980,
                7012,  13372, 36052, 23684, 7644,  59524, 47280, 32792, 50696,
                10488, 18964, 30952, 38112, 53084, 4068,  36660, 20476, 7964,
                16284, 2572,  58920, 19168, 6928,  52776, 36,    28336, 33264,
                56592, 17660, 8380,  26800, 39036, 22792, 35724, 65016, 24012,
                19000, 33160, 280,   64088, 21180, 40496, 2104,  33236, 53260,
                59380, 15900, 8300,  50948, 22052, 65428, 14768, 8628,  47832,
                63268, 34800, 10840, 7308,  31644, 51860, 19944, 60264, 38988,
                23284, 34880, 51668, 3928,  10708, 43028, 49760, 21192, 59244,
                17912, 52904, 28168, 21876, 10792, 30440, 5888,  58084, 45188,
                63888, 40452, 16220, 29748, 1344,  14092, 50608, 4012,  60820,
                27800, 55860, 31036, 21992, 46552, 14192, 408,   21260, 44248,
                27952, 50956, 1156,  14500, 21716, 25264, 62848, 28300, 20540,
                3992,  40280,
            },
            {
                54692, 21100, 63552, 32488, 6044,  47892, 42876, 580,   32776,
                61376, 18268, 10036, 43804, 14396, 5500,  61672, 25248, 54020,
                20640, 57276, 49944, 63724, 11188, 29408, 14988, 61648, 23316,
                40384, 64152, 944,   15768, 9680,  48560, 12032, 41344, 28136,
                62800, 45104, 38204, 29876, 61560, 34584, 44224, 21320, 10888,
                41072, 36540, 64348, 46572, 696,   16088, 55888, 3820,  29072,
                47496, 39204, 56632, 25580, 14236, 46232, 55304, 6884,  39172,
                21272, 472,   32484, 57524, 11852, 44020, 30736, 19824, 42892,
                3332,  53240, 20536, 46672, 64356, 4204,  13592, 25704, 1528,
                57856, 46008, 30540, 19988, 25884, 57664, 6260,  12536, 46148,
                24908, 8476,  49616, 37420, 59712, 41472, 17120, 48616, 9568,
                2036,  25848, 60624, 8388,  38656, 63200, 33456, 10664, 20744,
                15760, 3140,  9728,  51268, 33988, 54984, 60832, 31804, 48296,
                12936, 65420, 34808, 53920, 57324, 47772, 2232,  35020, 52160,
                31480, 24292,
            },
            {
                42952, 10736, 28876, 2708,  18912, 55852, 62732, 20984, 52796,
                12876, 39624, 27512, 63260, 47048, 38060, 32216, 2188,  48120,
                39484, 28728, 2752,  18760, 52644, 37192, 1436,  56168, 4940,
                35760, 52048, 24952, 42268, 59188, 30216, 61116, 54832, 50232,
                33876, 12884, 9020,  51164, 24996, 16596, 49336, 63236, 54584,
                2612,  23944, 58124, 29484, 42460, 51916, 8692,  44436, 13300,
                61652, 7636,  11192, 49712, 29692, 59020, 17152, 27188, 49232,
                11308, 62316, 27996, 40132, 6100,  60096, 1424,  56296, 38260,
                13700, 30112, 59896, 24408, 36340, 40596, 53776, 32816, 42808,
                16464, 7640,  62052, 48380, 37864, 15444, 28852, 40968, 35436,
                55564, 1652,  32664, 14220, 19796, 2640,  52080, 36580, 23300,
                31812, 52928, 47080, 19892, 55916, 24112, 45152, 30328, 52816,
                41736, 48540, 59292, 17156, 43104, 5316,  37008, 18504, 3400,
                23720, 38548, 7568,  42508, 31208, 5300,  37680, 44488, 8864,
                14144, 47864,
            },
            {
                28,    36680, 50340, 58256, 40608, 35056, 15292, 29332, 5024,
                45668, 22532, 58772, 3608,  30140, 19040, 57600, 9416,  15088,
                62736, 42704, 6284,  33524, 45584, 21844, 41856, 27096, 17356,
                12116, 44576, 56872, 6020,  34844, 17904, 22796, 5408,  1668,
                25492, 58132, 21896, 41544, 4888,  13364, 38984, 8048,  30832,
                48140, 15180, 6468,  20144, 11512, 62896, 33512, 37688, 21356,
                52440, 31740, 17972, 42408, 3320,  34984, 9852,  64708, 23936,
                42540, 52128, 45692, 18572, 24108, 48716, 33580, 26036, 62484,
                6496,  41092, 49500, 1888,  17736, 57476, 9268,  47928, 63036,
                28748, 12228, 40872, 232,   32764, 64604, 52032, 3260,  60952,
                16440, 42868, 63492, 56720, 45836, 25768, 65164, 12752, 42288,
                57188, 5468,  35564, 11420, 41884, 6304,  17336, 37052, 64940,
                7016,  26688, 38948, 23952, 11848, 28576, 52000, 41712, 59584,
                55536, 16780, 46836, 26996, 19516, 12720, 63812, 17916, 55784,
                59620, 33932,
            },
            {
                62176, 15796, 25528, 46328, 12660, 27000, 9308,  49972, 36372,
                55644, 7548,  34388, 49580, 11376, 24036, 51972, 44696, 34716,
                22624, 11860, 55164, 25872, 58428, 8632,  65012, 49324, 54200,
                7564,  29852, 20720, 49496, 27132, 45800, 39364, 14536, 43976,
                36548, 17376, 47104, 63800, 55764, 32748, 60044, 18804, 26672,
                43780, 35264, 53380, 32336, 49720, 24688, 17724, 60160, 4936,
                58148, 26916, 37196, 63356, 22484, 51900, 44364, 37540, 4816,
                14604, 34532, 3216,  55768, 36916, 16288, 53668, 10484, 22016,
                50972, 15832, 27100, 44780, 11936, 29592, 22620, 5760,  19336,
                36908, 54452, 21968, 57140, 18148, 9832,  23480, 47500, 20076,
                29988, 11580, 27112, 5128,  39780, 33816, 7304,  29284, 18492,
                61392, 15304, 22204, 50800, 28436, 59816, 54024, 24,    22724,
                12996, 32864, 61400, 1452,  56292, 64472, 7656,  25568, 10908,
                30252, 4520,  62008, 9560,  52568, 58796, 23964, 40008, 27116,
                5596,  19588,
            },
            {
                39476, 52564, 8016,  21808, 65084, 4048,  59424, 43668, 17556,
                64280, 25692, 15044, 54380, 42452, 604,   64920, 27380, 4416,
                59312, 30436, 51028, 16548, 39196, 13428, 31520, 19744, 34332,
                39524, 62328, 14052, 2492,  63368, 8032,  57596, 32184, 64672,
                54108, 6744,  31380, 2852,  27252, 44940, 1192,  23028, 57408,
                12276, 62376, 3708,  59552, 39580, 1880,  28208, 47292, 41432,
                15660, 1176,  45704, 54500, 6156,  13588, 31820, 56944, 19940,
                60732, 30200, 9176,  63908, 12812, 41440, 4464,  45908, 36644,
                58136, 32152, 8660,  55700, 61852, 34080, 51000, 59172, 45024,
                2808,  25212, 49892, 44080, 5820,  39668, 54016, 33608, 7460,
                38544, 49196, 52760, 23204, 9788,  60532, 54000, 49424, 524,
                45440, 39140, 30888, 62888, 2648,  34588, 14308, 46872, 58112,
                43384, 51712, 18824, 36220, 44728, 21516, 15348, 33476, 50296,
                45160, 35632, 22860, 40792, 49328, 33376, 1300,  46212, 11236,
                49704, 30000,
            },
            {
                3600,  57296, 32684, 42788, 37536, 53468, 31736, 22956, 38760,
                1428,  31064, 40544, 20492, 59440, 36024, 16648, 39676, 49524,
                18940, 37128, 1060,  47004, 61528, 3864,  44756, 5768,  60260,
                47244, 23812, 32912, 53612, 37024, 11712, 52028, 19600, 27724,
                10580, 49916, 40036, 20460, 11660, 35580, 52288, 40540, 6608,
                50844, 38052, 22448, 16280, 45972, 7896,  54116, 11912, 23040,
                50736, 34228, 10564, 24956, 19248, 61776, 28140, 860,   46992,
                39752, 53164, 22292, 26736, 49872, 31184, 61612, 27956, 18052,
                180,   64988, 39448, 19732, 4316,  42332, 15160, 38716, 11068,
                65380, 34500, 14288, 30292, 61456, 27284, 44956, 14036, 63192,
                57704, 3628,  36120, 17936, 47236, 14916, 21072, 37560, 27084,
                10000, 55588, 4668,  43964, 18712, 49312, 24936, 39500, 8496,
                28668, 3636,  47276, 9284,  30708, 49168, 39788, 62644, 840,
                57356, 19768, 53808, 14936, 6112,  28904, 16300, 53952, 35656,
                64520, 23176,
            },
            {
                45064, 12004, 61252, 18604, 1092,  16036, 47144, 7028,  12048,
                57840, 51208, 9372,  47672, 6056,  31660, 12868, 54944, 7700,
                43548, 63360, 10240, 23856, 34624, 28952, 56256, 25468, 15680,
                312,   9936,  58752, 41500, 16652, 25076, 47608, 628,   42316,
                23144, 61720, 15112, 55000, 58956, 47872, 15764, 65412, 31944,
                18016, 27676, 9612,  42164, 30052, 57608, 36364, 64848, 30824,
                7356,  56428, 60500, 38576, 47800, 41276, 8784,  50752, 25448,
                17348, 6356,  43168, 58224, 1128,  20436, 8516,  56164, 43712,
                12284, 47656, 23244, 35680, 53252, 24800, 1184,  27488, 31640,
                51752, 8200,  58748, 3540,  36160, 11232, 56380, 1160,  25904,
                21680, 43824, 61412, 31800, 55948, 3160,  43092, 32512, 63208,
                51324, 24592, 13768, 58372, 27400, 10336, 63844, 31788, 19780,
                54540, 15284, 63624, 24460, 13452, 53860, 5816,  18080, 43080,
                13136, 28108, 3060,  64688, 37164, 43972, 61828, 21396, 8076,
                42172, 14756,
            },
            {
                27932, 48520, 6392,  25176, 50592, 28784, 63212, 54744, 26604,
                45396, 19096, 62492, 28360, 22924, 61536, 45480, 26072, 21536,
                32896, 14872, 57148, 41496, 20352, 52960, 11132, 37148, 42384,
                51180, 28720, 19436, 45036, 4060,  30696, 61244, 35628, 6544,
                46284, 29940, 37928, 8008,  24812, 4056,  29164, 9932,  44292,
                3076,  47988, 54656, 63796, 12948, 18508, 25804, 3948,  44148,
                39660, 17408, 28608, 2240,  32600, 15956, 59360, 35400, 11872,
                56344, 32244, 14180, 35536, 46172, 38704, 52116, 24344, 34840,
                30192, 54432, 5920,  14120, 48972, 63688, 45844, 17956, 55920,
                20908, 42196, 16804, 47300, 22752, 18992, 50700, 32048, 41108,
                16676, 8808,  12700, 28552, 40856, 25508, 58840, 12180, 6920,
                19684, 34980, 47720, 37724, 7272,  41624, 52624, 4232,  36416,
                59264, 41516, 33640, 56768, 1976,  59824, 27736, 36876, 24064,
                61072, 38904, 31668, 47620, 24896, 9944,  50512, 4044,  58384,
                32400, 55320,
            },
            {
                38084, 20600, 35316, 56520, 39940, 10228, 33784, 20204, 41112,
                35276, 3980,  14136, 38088, 2396,  50564, 10292, 58004, 3780,
                51928, 28492, 6892,  49364, 2260,  17316, 48292, 64072, 21776,
                57668, 35228, 6904,  65184, 50760, 21132, 54588, 14040, 59628,
                18304, 2320,  51872, 34264, 41752, 61020, 21376, 37372, 56548,
                60168, 34748, 24328, 5948,  33064, 49436, 52576, 20848, 58948,
                13896, 48876, 21600, 64196, 52144, 5312,  22944, 44896, 65176,
                2456,  49060, 61012, 10112, 17752, 64392, 5256,  15400, 60444,
                3468,  41364, 26708, 58288, 10200, 29936, 7260,  61088, 48376,
                4844,  38952, 26972, 52780, 64120, 38076, 7848,  60280, 47636,
                35208, 55244, 50516, 960,   64408, 5696,  48736, 16948, 54912,
                40492, 1904,  64980, 31556, 21720, 57268, 16700, 45100, 11992,
                22252, 4996,  26932, 38848, 43608, 20208, 32164, 48208, 9836,
                51108, 7400,  55156, 12232, 59272, 18324, 31164, 39076, 25932,
                17456, 928,
            },
            {
                8768,  63592, 30284, 13348, 46020, 4676,  60996, 14600, 5952,
                59288, 52696, 32468, 44184, 56364, 41020, 18500, 33824, 64188,
                46520, 38600, 24948, 61964, 35960, 59156, 7532,  30264, 2940,
                13360, 54168, 39720, 11388, 27028, 38000, 9048,  40800, 33036,
                57112, 26640, 64484, 12664, 17728, 45500, 54288, 26020, 12428,
                19712, 41076, 1780,  60964, 44552, 38924, 152,   10284, 33416,
                54304, 4432,  36456, 12084, 42596, 55184, 30132, 18916, 38036,
                27796, 40652, 24148, 54576, 28804, 33060, 43008, 10992, 49652,
                20884, 63152, 17560, 38388, 33144, 21732, 41180, 12196, 34176,
                24220, 62688, 10252, 30844, 2296,  43040, 15456, 28188, 4024,
                63672, 23372, 45684, 18908, 34196, 21792, 36344, 44240, 30012,
                23236, 59508, 16112, 11224, 50252, 772,   29156, 24300, 62292,
                50772, 47888, 17672, 7148,  51448, 11352, 65188, 2636,  57712,
                17048, 45388, 20760, 35216, 392,   43096, 57020, 13584, 45824,
                60968, 51396,
            },
            {
                40708, 53616, 2508,  58880, 22276, 51904, 43004, 24332, 47844,
                29364, 10384, 65152, 24840, 15924, 6836,  27868, 37460, 13468,
                204,   16976, 54120, 11952, 31880, 43276, 26060, 40112, 46112,
                17900, 24376, 48888, 31952, 15576, 62652, 3268,  23868, 49180,
                10096, 43744, 22060, 48356, 6492,  32976, 380,   49244, 7496,
                30848, 15156, 51388, 21316, 28760, 15784, 63056, 42496, 28032,
                61040, 24272, 45932, 8584,  26116, 14352, 58280, 3784,  9912,
                53740, 20284, 5524,  47704, 2220,  21960, 59212, 25888, 36860,
                46652, 7856,  51304, 44168, 1664,  59420, 52708, 36988, 15404,
                788,   56788, 44880, 13712, 58060, 24792, 54724, 20704, 11820,
                30412, 6436,  38632, 14128, 53068, 56724, 10160, 62624, 3356,
                52752, 45860, 26780, 43196, 55176, 34540, 60124, 39040, 3204,
                32132, 9696,  63092, 36292, 22936, 45792, 15644, 40616, 33864,
                26576, 4356,  41568, 63392, 23184, 27960, 52956, 7088,  22240,
                34396, 11568,
            },
            {
                18472, 24192, 43848, 16108, 36956, 27980, 1292,  56752, 37860,
                16980, 22040, 688,   48720, 35404, 60648, 53432, 43172, 23056,
                59548, 30184, 44796, 21064, 5224,  15436, 55864, 9492,  33340,
                60608, 4856,  58484, 1032,  46944, 56064, 43376, 29292, 53500,
                16472, 36940, 4384,  58536, 29444, 62204, 14284, 38852, 64108,
                53424, 45252, 36976, 11288, 6652,  55252, 23872, 48100, 7572,
                18076, 40740, 31536, 62636, 49884, 39352, 33428, 47480, 62868,
                15528, 31368, 44172, 63308, 13188, 39372, 51564, 56784, 1400,
                32280, 12480, 28424, 55760, 15936, 5244,  26012, 45224, 54788,
                28824, 19272, 50084, 35592, 5692,  32980, 48864, 39884, 58548,
                51784, 43332, 26768, 59560, 31124, 7232,  39688, 27344, 14368,
                33656, 8724,  4428,  38128, 19156, 6084,  47036, 14760, 54180,
                43756, 28088, 13500, 58992, 54956, 30040, 5124,  52660, 22224,
                62012, 30380, 51852, 15236, 8540,  47924, 36568, 2268,  65492,
                49440, 27460,
            },
            {
                60468, 47160, 32528, 7668,  64852, 18896, 31616, 9080,  63028,
                50392, 41812, 55548, 12608, 30608, 20196, 9784,  4528,  48240,
                55752, 7868,  39860, 65488, 51676, 46912, 22820, 62256, 52072,
                37576, 27652, 42744, 21728, 34412, 18260, 5496,  60828, 20696,
                896,   31676, 55240, 40120, 24120, 51436, 20532, 43256, 28072,
                4628,  59184, 25772, 57048, 46884, 31136, 37748, 13388, 35076,
                50640, 2716,  57300, 16292, 540,   20960, 7136,  42988, 23784,
                51692, 36752, 8292,  55944, 34160, 16952, 6476,  29624, 19236,
                40776, 60804, 23436, 35372, 64700, 19520, 50276, 9548,  60320,
                32268, 7688,  39404, 22120, 60940, 9992,  44404, 316,   36036,
                17700, 61844, 2368,  11020, 46676, 24052, 17544, 51460, 47940,
                20684, 57652, 61632, 30360, 12924, 63708, 26304, 8832,  18556,
                57984, 21036, 40796, 33112, 276,   25880, 60200, 35768, 12908,
                47184, 10416, 37792, 56328, 32332, 61452, 40328, 16968, 30408,
                42748, 4260,
            },
            {
                36356, 6012,  55800, 11408, 39680, 53488, 46864, 13260, 33716,
                5212,  26892, 36884, 58656, 3412,  44688, 63104, 25900, 32320,
                19472, 11116, 34656, 2952,  27504, 36560, 780,   12632, 19676,
                6624,  14860, 64164, 10716, 52276, 26496, 36092, 12488, 39492,
                63900, 46508, 11612, 15964, 8144,  35992, 57460, 10388, 34268,
                23148, 17108, 8956,  33772, 62072, 17644, 3448,  57964, 65312,
                20776, 11324, 53148, 28920, 35968, 59108, 54212, 27516, 12408,
                1096,  60244, 18696, 26016, 42144, 48668, 9884,  65384, 44428,
                15672, 54604, 2656,  10528, 47392, 31112, 39888, 22512, 3876,
                43180, 64320, 15940, 53800, 28388, 18548, 65028, 14656, 25420,
                8536,  32676, 20804, 50148, 41368, 65496, 484,   60668, 36640,
                11544, 40828, 24060, 48960, 53128, 22012, 41380, 32984, 37588,
                2028,  52064, 5620,  49420, 17244, 43260, 8776,  19664, 57024,
                42840, 1372,  18860, 25708, 3712,  12704, 20304, 55120, 10080,
                58080, 14480,
            },
            {
                20764, 29508, 61372, 49156, 26032, 2736,  59208, 21652, 44548,
                54224, 19408, 8192,  46728, 22908, 38944, 51136, 14424, 41052,
                61004, 50364, 24116, 57232, 18356, 59780, 41864, 29360, 49612,
                44272, 55388, 30616, 39324, 7712,  57436, 44564, 50104, 8248,
                24648, 51964, 27592, 60248, 45376, 3524,  17788, 47716, 1520,
                62536, 49716, 39532, 2248,  42944, 51832, 22672, 6396,  45044,
                26644, 38104, 43396, 23412, 48256, 10296, 17876, 61544, 34664,
                39928, 46644, 29280, 4208,  61780, 22696, 36348, 53448, 25300,
                4892,  49000, 37308, 42776, 27032, 6936,  62728, 13308, 36000,
                48584, 24848, 1920,  46876, 41664, 4556,  51032, 29956, 56124,
                47264, 53648, 37820, 15676, 34780, 28476, 5868,  43124, 30860,
                55716, 1864,  16404, 34964, 43852, 2476,  56468, 48284, 61904,
                29680, 24624, 64780, 11024, 46548, 37916, 63568, 50220, 6280,
                28620, 54764, 64592, 49588, 43920, 59080, 26956, 45540, 33768,
                24868, 53088,
            },
            {
                212,   41868, 17432, 23128, 34628, 15076, 41540, 29572, 3592,
                61680, 14088, 32012, 63920, 16560, 28116, 57264, 1624,  36300,
                6212,  28848, 42848, 13644, 47936, 5668,  32788, 63804, 24480,
                35144, 2928,  48224, 17220, 24284, 63072, 14528, 29996, 59284,
                33848, 18668, 5780,  42264, 32352, 65156, 25368, 53224, 41236,
                13844, 55004, 29192, 20332, 12696, 27316, 39976, 54248, 30248,
                15136, 60412, 7764,  4064,  64020, 45520, 31716, 2932,  49700,
                21696, 57884, 10844, 14572, 51272, 30988, 352,   12596, 59600,
                33132, 21432, 62148, 14020, 56688, 33952, 53056, 17264, 57568,
                27900, 10144, 59092, 12748, 36716, 23748, 57736, 38832, 11600,
                3988,  22736, 63272, 3188,  57808, 52124, 19312, 23192, 7632,
                45640, 27040, 62924, 6568,  10132, 28556, 17180, 12108, 7180,
                45076, 14944, 35364, 56204, 27896, 22068, 13552, 32112, 24368,
                39280, 15564, 33948, 22364, 6184,  35708, 51500, 7684,  3196,
                48220, 39000,
            },
            {
                62872, 51292, 9408,  45208, 5584,  63788, 50996, 10208, 38324,
                23824, 48916, 39964, 52476, 6480,  34256, 11416, 18704, 54808,
                46460, 15888, 62428, 8432,  38332, 54608, 10876, 16708, 56956,
                8804,  21084, 59728, 42000, 1776,  32760, 5048,  22244, 41596,
                2700,  56668, 38056, 21916, 49928, 12452, 28832, 37236, 21140,
                31552, 44968, 4932,  64612, 48664, 58856, 9684,  34600, 760,
                49300, 55756, 33220, 19868, 13996, 38892, 24500, 41968, 15168,
                6856,  54468, 32856, 38960, 58760, 43236, 19924, 46996, 39528,
                27540, 9212,  51556, 18740, 4400,  23948, 44528, 132,   41572,
                20116, 50976, 34148, 30460, 62596, 8176,  20500, 33776, 45484,
                59660, 42104, 27496, 13192, 9324,  45244, 33388, 54076, 14708,
                60068, 18816, 51252, 39092, 57968, 64536, 36624, 52380, 22636,
                60092, 39452, 19716, 7796,  51776, 2280,  58432, 41888, 3504,
                60928, 8400,  46516, 11764, 40656, 29340, 14828, 64192, 22104,
                59496, 12340,
            },
            {
                33256, 27428, 54840, 37908, 57880, 27072, 19076, 55840, 34956,
                59492, 27588, 10948, 2964,  43896, 59936, 49184, 30672, 64744,
                22288, 2424,  53292, 31348, 26112, 21852, 45308, 4236,  40116,
                52524, 12400, 27448, 36748, 53700, 49548, 46092, 16732, 54524,
                10348, 47996, 14316, 61932, 856,   56016, 8820,  59348, 5872,
                61200, 11032, 24836, 37476, 32712, 15928, 43996, 19416, 63640,
                10984, 24004, 41644, 51376, 29356, 56212, 5020,  53012, 65104,
                26408, 43924, 19104, 2372,  24932, 8256,  63808, 15408, 57520,
                6004,  46012, 30484, 41232, 59972, 49512, 11128, 30020, 61228,
                38304, 5072,  54160, 16856, 43692, 49108, 1272,  15772, 26104,
                6924,  17800, 31516, 48844, 39108, 25068, 62820, 10468, 37580,
                48476, 34128, 12364, 31412, 20380, 46248, 25548, 3916,  32180,
                49248, 968,   42760, 30760, 61324, 34040, 16312, 48492, 53736,
                19428, 30348, 52788, 62432, 596,   55540, 18788, 38136, 43084,
                30600, 16916,
            },
            {
                46588, 7092,  14108, 20988, 1360,  32088, 48232, 7692,  16424,
                336,   42820, 18568, 55972, 25792, 20800, 38452, 9036,  25336,
                44464, 34380, 39716, 19576, 50448, 58828, 36316, 28496, 61288,
                32176, 46408, 65212, 7192,  19916, 11556, 61468, 38752, 28248,
                64336, 35448, 26272, 30572, 18288, 39084, 43728, 16156, 50832,
                35488, 18764, 57560, 52260, 1376,  7152,  61328, 25572, 47220,
                37260, 16912, 2360,  61864, 35520, 8888,  59664, 17596, 30576,
                9740,  36668, 62708, 47816, 55740, 33992, 28820, 3176,  52732,
                34852, 16992, 64380, 1796,  26140, 35816, 21236, 55068, 7768,
                14564, 45588, 22760, 3336,  26680, 59944, 55276, 39832, 63936,
                51716, 36260, 54852, 60240, 5268,  16592, 57272, 1364,  28752,
                41392, 6348,  24388, 54632, 192,   14844, 41008, 55872, 18452,
                10180, 58180, 25252, 12744, 45460, 5560,  26664, 36324, 11396,
                44532, 26204, 37296, 16796, 24620, 59884, 47200, 10512, 54356,
                25540, 4768,
            },
            {
                40480, 65108, 35716, 59880, 43904, 12380, 40672, 23324, 45408,
                52996, 65436, 31436, 36192, 61924, 15040, 952,   42204, 52392,
                4592,  59844, 11636, 6784,  63148, 2008,  13900, 49068, 18640,
                648,   25072, 15268, 55584, 29368, 35200, 23640, 116,   51356,
                20376, 3716,  44832, 7872,  54112, 33700, 24008, 47240, 1956,
                27160, 42096, 8168,  45856, 23200, 29568, 53396, 41172, 5204,
                31388, 58564, 45236, 26360, 12532, 48508, 21956, 37776, 46336,
                51616, 5304,  23164, 12820, 16884, 50616, 37612, 21572, 41592,
                24324, 11472, 54952, 38628, 8512,  15652, 46440, 65244, 33088,
                27020, 57028, 63216, 41048, 9820,  32140, 13092, 46824, 29844,
                10352, 21860, 680,   44336, 19992, 29948, 42340, 50168, 20728,
                64896, 3528,  59444, 42544, 50024, 8472,  34620, 62392, 44096,
                28640, 36004, 53716, 63304, 18264, 39796, 55280, 22836, 65324,
                6940,  56984, 3008,  50768, 42528, 32204, 5936,  34712, 1644,
                50132, 57132,
            },
            {
                19356, 23820, 49368, 29792, 52100, 3860,  63372, 56868, 29316,
                5516,  21796, 13116, 8068,  50244, 45184, 33012, 57044, 13624,
                17764, 27576, 55900, 46752, 16116, 33208, 43644, 6076,  23060,
                51504, 43116, 39124, 3172,  44472, 58468, 8276,  42564, 32452,
                12808, 58816, 40484, 60824, 49340, 4116,  64080, 12888, 30816,
                53588, 62616, 14612, 39584, 18204, 56816, 35176, 13132, 21804,
                50672, 8520,  54420, 19368, 4524,  41140, 28016, 168,   13572,
                34168, 60012, 28340, 40276, 44736, 4108,  62116, 9532,  48156,
                60716, 44244, 19876, 32420, 58304, 52436, 40632, 5492,  18784,
                50008, 11756, 35304, 18444, 52536, 37384, 19776, 24228, 5372,
                43136, 14636, 62344, 33572, 8696,  53444, 35860, 13480, 26108,
                55636, 16308, 46768, 29548, 21920, 60728, 27184, 13104, 5360,
                51264, 15692, 3052,  21812, 48148, 10072, 59056, 1628,  31332,
                15124, 40972, 33632, 9384,  20188, 13508, 27984, 63108, 21384,
                36576, 11560,
            },
            {
                43372, 2152,  9828,  17484, 25412, 38708, 15476, 34024, 10532,
                37608, 47072, 40880, 54336, 28896, 5944,  22664, 63852, 29992,
                36772, 49636, 21624, 41872, 29492, 23968, 53868, 63840, 37244,
                58072, 10184, 33168, 62280, 18240, 50328, 26804, 15716, 56800,
                47000, 25240, 16932, 22548, 11208, 28348, 19696, 41016, 58616,
                21596, 3348,  34144, 50040, 60392, 10048, 48048, 2744,  62628,
                28048, 15116, 39180, 64540, 32904, 52316, 56904, 63380, 43176,
                20228, 55992, 1680,  52980, 10672, 30932, 26356, 56228, 14344,
                708,   28968, 5196,  48832, 13436, 27908, 2536,  24068, 37564,
                43428, 1224,  30296, 47156, 6324,  64948, 2212,  50184, 58980,
                55976, 38476, 27144, 47760, 23768, 63860, 3048,  45716, 7248,
                39064, 33196, 9188,  36804, 17628, 4580,  53216, 38380, 23356,
                65048, 40928, 31504, 7520,  37272, 28004, 43296, 51176, 20312,
                47180, 59624, 23376, 63692, 48460, 57892, 43880, 52204, 16328,
                60900, 29300,
            },
            {
                58292, 54072, 34560, 61148, 5804,  54572, 20336, 59240, 26076,
                51604, 60684, 3232,  24900, 16760, 58636, 47572, 10584, 40996,
                7616,  60956, 612,   35156, 9380,  57452, 39612, 12060, 7500,
                30424, 20560, 52948, 5064,  13068, 37932, 63716, 5396,  52604,
                9288,  37732, 4692,  53204, 35940, 45952, 56396, 9108,  37868,
                48528, 25196, 11536, 30632, 5004,  27124, 38184, 17276, 55484,
                42612, 35884, 1044,  23700, 44608, 16348, 10668, 24484, 6548,
                48132, 15816, 32456, 64772, 20872, 58300, 18036, 39764, 33040,
                53628, 36952, 63476, 22200, 42696, 61812, 31444, 54636, 59344,
                9160,  61676, 24856, 15128, 54996, 27876, 41348, 34552, 16400,
                31424, 2792,  52648, 11796, 40556, 16016, 57920, 32052, 61860,
                22968, 50788, 12524, 63572, 57536, 45348, 31196, 48656, 19544,
                8780,  45796, 57340, 50400, 60556, 33216, 14116, 4872,  35644,
                53156, 28244, 12544, 4264,  38860, 25208, 2672,  8728,  39868,
                45976, 6644,
            },
            {
                26856, 15296, 47320, 41488, 30884, 45232, 8216,  48744, 1220,
                14172, 32848, 20200, 63524, 35416, 39120, 2252,  19400, 55232,
                24808, 45472, 14132, 64800, 51172, 2760,  18428, 27700, 46188,
                16352, 59812, 26280, 47712, 22632, 31596, 45728, 20156, 34936,
                29636, 65100, 43556, 31672, 15048, 62880, 440,   32540, 16568,
                6684,  65368, 56112, 43304, 21008, 63248, 45132, 24312, 32508,
                10880, 59368, 48348, 29932, 7708,  60072, 36064, 50372, 30152,
                39620, 26064, 8680,  37032, 42216, 49448, 5712,  46244, 8072,
                24940, 16556, 50968, 9956,  35392, 6780,  17116, 48212, 21168,
                14016, 51112, 40064, 57860, 21932, 44344, 12044, 60792, 8908,
                22912, 45568, 18868, 59500, 4408,  51148, 28172, 11176, 18296,
                42832, 54280, 27572, 2148,  40244, 24744, 10720, 1020,  58332,
                34200, 27320, 12448, 24360, 328,   17308, 63960, 25700, 39148,
                8368,  17896, 45088, 54172, 35584, 17084, 30364, 55708, 22848,
                13028, 32544,
            },
            {
                50744, 38196, 3168,  22348, 12788, 62212, 35944, 28992, 42560,
                17908, 57428, 8760,  43240, 11932, 52528, 27332, 33748, 50840,
                4740,  30780, 38712, 20024, 26200, 47676, 62160, 35652, 50520,
                41780, 1204,  34752, 40536, 56500, 10692, 2468,  41560, 60724,
                17464, 1236,  50224, 57764, 7004,  25452, 42768, 51828, 27520,
                45424, 19152, 39068, 52384, 15788, 720,   57908, 7596,  51728,
                20416, 5904,  53436, 13636, 21276, 40376, 2364,  19204, 62168,
                11740, 59480, 51052, 4916,  24088, 13108, 35036, 63972, 20428,
                59692, 40764, 2336,  46620, 25512, 56360, 11488, 44972, 33856,
                27600, 5344,  36536, 2884,  32584, 7592,  51956, 26024, 47964,
                37140, 64352, 7436,  30532, 35380, 21336, 38720, 48116, 552,
                60348, 5528,  20388, 49036, 33872, 15600, 54824, 42076, 61540,
                17032, 3960,  38264, 20276, 44268, 53388, 46792, 11656, 56628,
                62840, 932,   31856, 60488, 6380,  50928, 65004, 34136, 47984,
                704,   62720,
            },
            {
                19912, 10752, 64564, 55460, 18044, 50988, 4376,  22992, 64808,
                38420, 46132, 29900, 49188, 5352,  60172, 14672, 43952, 61716,
                17044, 58308, 10832, 54364, 32936, 6984,  14532, 22404, 4756,
                54244, 64884, 14300, 7056,  60452, 28764, 53584, 24816, 48268,
                12264, 21928, 26988, 10596, 39848, 20780, 55040, 12616, 61492,
                34900, 2084,  28544, 8916,  35668, 31636, 40752, 13412, 64952,
                46536, 25920, 34820, 62308, 56460, 45676, 32196, 54848, 4632,
                42804, 34604, 17784, 46096, 61880, 28036, 1792,  51944, 29560,
                11628, 56028, 31748, 61184, 19376, 38868, 64404, 444,   41024,
                53996, 63700, 19536, 48804, 61416, 17648, 39144, 160,   20172,
                56988, 12976, 41528, 25168, 54940, 44308, 8236,  56596, 24700,
                36436, 30868, 13920, 44160, 7968,  62472, 21216, 36464, 29576,
                47008, 51656, 55824, 62372, 30128, 5892,  40516, 22488, 29276,
                48964, 42836, 24440, 20912, 14328, 41656, 10620, 19068, 59052,
                36972, 24656,
            },
            {
                4956,  43112, 28636, 33548, 25664, 40108, 9872,  53152, 12528,
                55328, 2588,  21468, 25584, 56496, 31780, 23104, 7940,  37316,
                21552, 48280, 42240, 3576,  60040, 44388, 40348, 56760, 31304,
                9816,  24100, 44512, 18120, 50900, 37072, 15132, 7996,  58388,
                39072, 55444, 34276, 61112, 17716, 47336, 36596, 5420,  22948,
                49816, 14076, 60268, 46960, 54988, 22720, 50000, 27388, 37040,
                2092,  15828, 41768, 3824,  27812, 9436,  25316, 47452, 14184,
                22932, 57548, 736,   30652, 54796, 15508, 57844, 43420, 38404,
                3936,  44952, 14864, 27680, 5452,  51480, 29796, 22868, 8096,
                16040, 30964, 10372, 45576, 13560, 29660, 55560, 62952, 33792,
                28704, 4732,  50356, 61220, 14956, 3396,  64040, 34048, 16248,
                51968, 39592, 65440, 57016, 28480, 50692, 3328,  13256, 25284,
                6188,  10508, 41384, 15208, 35104, 9364,  59432, 18560, 3132,
                33788, 10872, 37216, 57320, 46844, 28924, 25724, 4420,  44120,
                15524, 53316,
            },
            {
                57720, 49192, 8000,  59488, 104,   44424, 58516, 32416, 27204,
                6228,  34864, 62768, 15620, 39836, 464,   46796, 65032, 12512,
                1812,  27092, 35336, 16048, 23432, 29296, 11400, 61356, 19352,
                38240, 28280, 49016, 32640, 1660,  21360, 62036, 33284, 4904,
                28980, 44012, 2840,  49452, 30280, 3804,  59352, 31548, 9472,
                40088, 56948, 24832, 4212,  12096, 61620, 6032,  17976, 43948,
                60416, 31864, 49904, 18792, 37640, 51664, 16744, 65416, 38244,
                28432, 52500, 10380, 40136, 21508, 37428, 9096,  23280, 17236,
                50128, 34636, 22564, 53640, 42896, 13396, 35660, 59900, 49164,
                57208, 43468, 25644, 58544, 35728, 23664, 42168, 6048,  15484,
                53472, 44740, 17856, 33052, 10260, 49056, 20136, 27784, 12168,
                46348, 9328,  22248, 4636,  17428, 38792, 59416, 43456, 54048,
                32952, 64596, 22584, 2828,  49256, 26056, 54440, 37596, 51068,
                15580, 61572, 52496, 7820,  2228,  61968, 49772, 54532, 8464,
                31292, 40868,
            },
            {
                35068, 22064, 16880, 46948, 36892, 14908, 20556, 47592, 16480,
                50108, 42064, 11120, 51780, 35956, 18280, 54512, 29308, 40904,
                57352, 52148, 63828, 9152,  46804, 51404, 228,   34476, 47360,
                3000,  59016, 12184, 62988, 39920, 26308, 46860, 42396, 19388,
                52400, 15596, 9092,  63628, 23736, 42320, 15700, 52488, 64444,
                44136, 17080, 32492, 37568, 19812, 45400, 29828, 57192, 9392,
                21964, 55136, 7420,  63324, 11928, 57800, 1536,  34012, 7800,
                48944, 19704, 63736, 45144, 6820,  48636, 33356, 61604, 26820,
                6108,  65336, 9516,  58364, 2124,  47460, 17532, 26480, 3492,
                37772, 21456, 4708,  52200, 1948,  9440,  50636, 46332, 11088,
                38672, 23276, 2156,  37600, 29416, 58476, 40084, 43448, 5732,
                59912, 55292, 26172, 35460, 47404, 32004, 23584, 8848,  48968,
                19200, 37484, 28504, 60372, 43856, 32472, 13356, 64960, 44316,
                5456,  27136, 19392, 40320, 31140, 35808, 21464, 38620, 63876,
                27348, 12432,
            },
            {
                2292,  29824, 54296, 11444, 61384, 30400, 3892,  63720, 38048,
                23564, 59304, 31020, 7440,  60852, 43152, 9584,  50544, 34120,
                23924, 6844,  30612, 55352, 38012, 18188, 26596, 52972, 15336,
                41984, 55500, 20528, 8824,  53720, 4196,  56664, 12676, 64448,
                35844, 25592, 45864, 37636, 13172, 56612, 26720, 1568,  21012,
                29092, 6472,  49148, 63520, 41616, 51824, 2460,  34924, 39740,
                12880, 28988, 45532, 23880, 40860, 30620, 22020, 44192, 56236,
                3612,  32392, 13852, 26052, 58932, 2592,  53048, 12824, 56224,
                46340, 40396, 19108, 37660, 32732, 62232, 8748,  41876, 53320,
                11792, 33328, 61840, 40588, 27904, 64588, 20944, 31336, 60508,
                26608, 57392, 65128, 47408, 53816, 22536, 1072,  62088, 30504,
                19084, 2272,  41748, 53440, 11496, 268,   63468, 16024, 58096,
                4320,  46304, 12192, 17600, 56536, 892,   20424, 8220,  30228,
                23240, 58964, 45968, 13464, 55248, 16552, 11112, 2960,  46260,
                18544, 60140,
            },
            {
                44992, 62540, 39732, 5568,  23808, 53536, 41652, 8272,  56356,
                976,   18884, 45612, 27832, 3568,  21204, 26484, 15192, 3084,
                18660, 45080, 13808, 20900, 4156,  62352, 43288, 8084,  63912,
                25388, 5676,  30232, 35468, 43868, 16892, 31076, 9500,  23012,
                272,   59992, 54756, 20176, 6964,  50436, 34400, 46772, 37064,
                12328, 53644, 200,   26960, 10220, 15320, 24424, 64220, 53564,
                48660, 592,   58588, 34488, 4912,  47148, 61644, 13060, 39736,
                24444, 60472, 42388, 35920, 18272, 28972, 39904, 20240, 31500,
                88,    51600, 29436, 12100, 24124, 55520, 20212, 30464, 64944,
                15428, 46832, 7808,  18856, 14332, 56700, 36704, 16896, 2812,
                41400, 8596,  19384, 12904, 6376,  26468, 16680, 51672, 36848,
                49508, 33420, 14680, 61928, 20580, 51408, 41416, 30144, 26504,
                39512, 55104, 7876,  50748, 36508, 24716, 41308, 47684, 56900,
                39100, 34568, 216,   64428, 24668, 43108, 60640, 32576, 23848,
                50904, 7264,
            },
            {
                25576, 36228, 19580, 50004, 27684, 45840, 34920, 12992, 26560,
                33636, 54004, 14232, 64452, 47840, 55512, 38336, 58448, 63180,
                49120, 36268, 60984, 41352, 33368, 57412, 12984, 31484, 22028,
                36964, 50368, 46288, 13780, 23976, 59876, 37716, 47596, 50664,
                40432, 30044, 4596,  32796, 41336, 62124, 10304, 18596, 57504,
                61120, 41064, 22184, 56464, 36020, 59528, 32016, 4864,  16800,
                26436, 37348, 19740, 15240, 54324, 8856,  18380, 28020, 52264,
                16960, 9072,  54360, 4084,  50820, 10572, 64204, 44256, 7552,
                24508, 15148, 63068, 48684, 6312,  45048, 39188, 1124,  49728,
                24252, 29100, 54812, 34908, 49472, 43316, 5540,  47784, 54708,
                34384, 51336, 30716, 42428, 35256, 45700, 57032, 13288, 7372,
                22952, 45380, 57372, 29120, 6712,  36432, 45696, 13024, 2692,
                34236, 63832, 23288, 31240, 5280,  61996, 52296, 16392, 3872,
                12316, 49676, 9600,  28596, 51704, 6832,  48092, 58044, 13952,
                41520, 55804,
            },
            {
                33124, 8732,  13576, 64344, 1324,  17284, 58088, 22008, 49480,
                44040, 9920,  40408, 23368, 34308, 12112, 4736,  31068, 41684,
                11144, 27916, 1056,  50280, 24780, 6276,  47824, 54664, 40324,
                3496,  17200, 56576, 65124, 988,   52056, 27480, 6128,  14592,
                57440, 18100, 11452, 52232, 22728, 28776, 4080,  39428, 24436,
                7280,  33508, 46064, 18460, 6688,  43472, 47552, 21220, 41424,
                61804, 10396, 44300, 64680, 25616, 50876, 37204, 58324, 340,
                34572, 46452, 29804, 62444, 23608, 47432, 15836, 34300, 60336,
                36936, 56680, 41576, 2904,  27972, 60896, 13828, 34616, 58212,
                5308,  43860, 59616, 488,   22980, 30056, 12360, 25144, 62940,
                21460, 14544, 1340,  58664, 63244, 9964,  32480, 39008, 64168,
                27472, 10556, 3684,  39228, 25040, 56220, 18228, 60980, 53008,
                20232, 44636, 14440, 42608, 58888, 11504, 27740, 32892, 63336,
                21144, 54052, 17440, 41284, 35284, 3564,  19852, 27884, 36588,
                728,   16148,
            },
            {
                49064, 58884, 52748, 42412, 32388, 10520, 39224, 61292, 4140,
                63240, 30664, 2160,  51212, 17160, 60496, 44772, 24540, 7488,
                54104, 22476, 58908, 9792,  17008, 37500, 28768, 19620, 9972,
                61180, 32932, 27788, 7816,  40820, 19548, 34000, 63444, 44124,
                26100, 35072, 46424, 65320, 14900, 43900, 59396, 48596, 15996,
                29816, 50756, 13972, 62864, 28780, 53924, 13148, 8468,  51060,
                33184, 3944,  49460, 29964, 2076,  41904, 32008, 6596,  64292,
                40956, 12248, 21112, 7084,  38004, 57312, 1500,  27240, 53452,
                10088, 18632, 33076, 22440, 54192, 17260, 50940, 9688,  21288,
                37672, 16920, 11272, 39264, 63580, 8580,  58284, 38080, 3808,
                28892, 45028, 37156, 24140, 48872, 20956, 3056,  54704, 18092,
                43000, 60524, 50264, 16468, 64888, 9396,  32660, 5824,  49356,
                10280, 25740, 1588,  48496, 18344, 39608, 8984,  45748, 36968,
                26368, 43796, 31040, 61264, 22416, 56452, 39152, 9208,  52880,
                65468, 22284,
            },
            {
                40260, 29536, 3308,  21416, 47756, 54896, 6592,  28800, 19604,
                15712, 37300, 58860, 6756,  28368, 36700, 53288, 19072, 47648,
                15808, 38508, 30428, 45732, 52720, 64880, 1724,  58408, 45268,
                51768, 12296, 43068, 22552, 49540, 10932, 54312, 2540,  21208,
                8672,  60584, 1676,  38148, 6232,  54976, 32372, 1900,  53144,
                64696, 9940,  3252,  39776, 23120, 2032,  37832, 56672, 27836,
                59216, 23360, 39932, 12492, 60592, 21676, 10916, 48184, 19644,
                26832, 50428, 59800, 44016, 31972, 13044, 41696, 21548, 48248,
                3648,  44716, 59536, 7788,  36440, 42720, 25020, 31296, 45816,
                62968, 26928, 52168, 20036, 32096, 53516, 45436, 18600, 50016,
                10052, 61576, 53108, 16788, 4788,  29896, 41816, 24724, 48448,
                644,   31144, 21264, 35292, 42160, 47544, 22436, 28220, 40784,
                37028, 62524, 57168, 28616, 34704, 22080, 55344, 1200,  59964,
                7024,  57176, 2680,  14632, 8132,  44692, 62552, 12484, 30516,
                44036, 5044,
            },
            {
                11052, 26260, 37988, 15380, 59948, 24884, 35704, 51740, 41764,
                48276, 25348, 55072, 20792, 42932, 10060, 120,   65464, 33500,
                56392, 6148,  62756, 13532, 32820, 41996, 23652, 14884, 35508,
                25140, 2176,  57684, 37060, 29924, 62144, 15892, 39456, 31720,
                55356, 41824, 49860, 27840, 19452, 25596, 11748, 21840, 36768,
                43688, 26424, 56200, 34828, 49796, 60064, 31300, 18172, 45276,
                14888, 7528,  55464, 36256, 17092, 57184, 43908, 24296, 14496,
                55624, 36736, 2416,  17188, 26128, 54624, 5092,  62108, 30432,
                38528, 13640, 50288, 29740, 11836, 63560, 1868,  53052, 56756,
                8112,  2480,  41784, 47468, 4968,  26400, 15064, 40980, 33120,
                56304, 6080,  27264, 40104, 11788, 56188, 62156, 15036, 34564,
                53540, 12928, 58560, 4660,  52280, 14356, 1008,  54240, 60164,
                15748, 31900, 8420,  53208, 4472,  65240, 49532, 24452, 14196,
                19092, 39296, 47128, 50536, 25812, 32460, 17268, 47476, 24576,
                60024, 18840,
            },
            {
                53560, 45852, 63376, 7920,  30684, 44460, 1984,  13884, 56952,
                8060,  11540, 45888, 32948, 62024, 50028, 14148, 26964, 40724,
                20968, 43772, 3856,  26216, 20044, 8440,  49640, 5140,  30388,
                63728, 19060, 47524, 14560, 4168,  44720, 25292, 59144, 48588,
                13160, 23296, 16564, 33584, 58120, 47108, 62804, 41640, 5588,
                17684, 31776, 20692, 44892, 16196, 11172, 25124, 65460, 824,
                35288, 20248, 51412, 26028, 45944, 3324,  34084, 52164, 62860,
                6152,  32808, 10404, 46988, 65200, 19492, 35452, 50860, 16104,
                25312, 64660, 20852, 55336, 47252, 6024,  40192, 19264, 13664,
                35548, 29904, 61732, 12508, 34464, 60112, 1916,  63984, 23252,
                13012, 43496, 20300, 60080, 51456, 36124, 45820, 5736,  9540,
                38340, 26412, 45496, 8360,  29256, 62908, 34480, 44416, 19240,
                4984,  23692, 46272, 41688, 16532, 11896, 31420, 40768, 51760,
                29380, 34076, 11448, 64832, 37424, 55024, 5664,  51492, 2052,
                34768, 57156,
            },
            {
                6532,  17148, 33740, 56312, 11960, 18952, 64856, 23124, 33228,
                60636, 29668, 2832,  16136, 39368, 23188, 31604, 58264, 3064,
                11940, 51168, 35296, 57620, 53872, 38108, 61280, 43656, 54236,
                40076, 7516,  33624, 60916, 53624, 7140,  18852, 36188, 3888,
                29324, 5928,  63224, 10392, 51996, 7604,  34992, 14692, 54084,
                48020, 58356, 7924,  62424, 4088,  40944, 47764, 6848,  52772,
                42792, 62388, 31084, 4944,  63760, 9140,  29328, 38412, 1416,
                42692, 28644, 58376, 52376, 39328, 8288,  43408, 10972, 58048,
                6420,  40948, 588,   26268, 16396, 33496, 60304, 28100, 48704,
                43516, 21700, 55128, 24316, 17420, 50680, 36832, 7360,  29420,
                48228, 35588, 4,     32020, 8080,  18416, 27872, 58100, 22332,
                64468, 41096, 19020, 55788, 24040, 39468, 11008, 26292, 56280,
                13084, 51660, 38200, 20552, 59856, 35908, 44928, 7944,  61876,
                3972,  58780, 23648, 19832, 576,   28740, 42352, 21592, 38572,
                14420, 41976,
            },
            {
                49848, 22852, 240,   48456, 41412, 53960, 37796, 47152, 5228,
                40092, 20064, 64184, 53024, 5852,  47256, 8148,  52468, 37456,
                61708, 28828, 47068, 17380, 564,   12156, 28180, 21380, 16544,
                10944, 50824, 26636, 21524, 40704, 32092, 51128, 10216, 45536,
                61300, 53228, 37280, 44180, 1168,  40072, 29184, 59644, 25148,
                776,   38328, 13324, 52176, 27336, 55620, 32812, 21300, 39024,
                28488, 10364, 48136, 14348, 40564, 53980, 19164, 60948, 16200,
                49028, 21476, 13204, 23980, 3408,  29128, 60936, 22196, 32156,
                46580, 35248, 9752,  43156, 58240, 37944, 23112, 3644,  65528,
                16496, 5876,  49620, 39544, 9116,  57120, 43072, 19468, 53824,
                58928, 15616, 65328, 45092, 25676, 49896, 2140,  32876, 16372,
                49348, 3868,  60704, 32384, 16964, 48016, 59580, 3136,  36108,
                49088, 64104, 0,     25908, 54552, 3124,  27192, 21176, 17036,
                42280, 53608, 9164,  45492, 57464, 15604, 60904, 10816, 64064,
                27152, 31308,
            },
            {
                62660, 37232, 60548, 25756, 6016,  28440, 9960,  16240, 26512,
                51368, 44200, 13048, 36516, 27808, 59924, 17836, 42828, 24456,
                15648, 7392,  22680, 41124, 63988, 48328, 34152, 58732, 2652,
                37556, 59852, 44164, 56,    12368, 64048, 27596, 58200, 20880,
                40356, 25048, 18364, 30764, 13672, 23052, 18928, 44724, 10748,
                61740, 22128, 29704, 42500, 19364, 36156, 9136,  61512, 13820,
                56540, 2520,  18300, 58672, 33324, 22804, 12056, 46352, 25928,
                8500,  37584, 62268, 45340, 33904, 15000, 48472, 1480,  56052,
                18408, 52988, 61500, 30872, 50712, 12668, 8620,  52268, 32924,
                10688, 37056, 59708, 3236,  27644, 46168, 31400, 11356, 25916,
                4404,  38512, 22576, 53196, 12816, 39564, 63096, 43064, 54880,
                29768, 11816, 36480, 1596,  50776, 6768,  21392, 31156, 42184,
                7932,  28844, 33620, 10244, 15024, 48812, 63136, 56412, 37100,
                48076, 13756, 26876, 40296, 35712, 7832,  48820, 33940, 54092,
                4128,  9444,
            },
            {
                43544, 29496, 13456, 19936, 52156, 34596, 62528, 55312, 1356,
                58688, 33912, 24400, 55448, 1616,  20840, 34968, 11004, 54776,
                45444, 32960, 59280, 9744,  30956, 24796, 6328,  45688, 55728,
                24024, 31456, 17612, 48372, 54804, 37488, 16168, 1532,  33480,
                14404, 3020,  47564, 57788, 54284, 63588, 49176, 5180,  32192,
                50916, 34744, 46616, 3344,  63928, 15272, 49920, 5012,  24920,
                45712, 22296, 37020, 50684, 26980, 44144, 5556,  59100, 34304,
                56436, 51204, 17580, 6808,  57384, 52856, 25892, 41244, 37696,
                27968, 14080, 23588, 5740,  19908, 45384, 55980, 41040, 26096,
                57616, 44876, 30556, 14044, 64488, 21440, 948,   62648, 51092,
                42224, 9524,  29972, 3720,  57200, 21076, 10040, 37084, 6224,
                46416, 25748, 52944, 42884, 13608, 65444, 45084, 57472, 15328,
                19788, 61124, 45284, 58416, 42468, 30788, 6268,  11268, 33072,
                1108,  30136, 60204, 5172,  52712, 30800, 23996, 17772, 46032,
                58412, 20888,
            },
            {
                52812, 2972,  57796, 40252, 46192, 3772,  21944, 42588, 30520,
                18576, 6588,  10012, 46624, 41136, 63128, 49484, 30100, 2080,
                64728, 5104,  49808, 37184, 14416, 53076, 19036, 40636, 9388,
                14004, 65400, 4764,  29516, 9028,  23016, 46908, 42664, 55668,
                50480, 65052, 7236,  35116, 26476, 3676,  36732, 57224, 41320,
                15912, 6888,  54600, 11828, 24224, 56928, 43520, 29116, 53236,
                60296, 31468, 42236, 8184,  632,   64812, 39192, 52736, 3080,
                29764, 41116, 208,   31260, 37048, 20280, 63856, 4700,  11812,
                59920, 2784,  48556, 64396, 36248, 1932,  30108, 17708, 48000,
                224,   23668, 19784, 53792, 34252, 40888, 15908, 55664, 32720,
                18728, 47124, 61992, 35324, 17292, 48084, 60756, 24208, 14688,
                19836, 63408, 7844,  22540, 33752, 26920, 37856, 10308, 52740,
                24092, 39436, 5252,  18064, 52004, 22712, 39704, 45868, 24264,
                64568, 18960, 50248, 22300, 63416, 12712, 43760, 2688,  25984,
                39548, 15916,
            },
            {
                47188, 35140, 8552,  16664, 31364, 12440, 59572, 48952, 13988,
                39600, 50164, 61160, 16888, 31352, 4500,  14360, 57868, 39792,
                27036, 17128, 20832, 56572, 44264, 4096,  63184, 27220, 50196,
                36352, 52580, 43020, 56828, 33976, 62676, 5400,  26228, 8600,
                29604, 38736, 21620, 11104, 42924, 17072, 9320,  28160, 20940,
                64712, 26256, 59168, 39808, 31956, 92,    37996, 17016, 34544,
                3796,  12028, 62196, 16224, 56808, 30456, 19748, 15072, 24168,
                11228, 20644, 64492, 44140, 12312, 8900,  45992, 17744, 51012,
                31464, 44480, 39388, 26616, 15224, 59012, 22156, 62600, 13484,
                38956, 61276, 7972,  50496, 4560,  47652, 24912, 37160, 6736,
                13720, 26944, 54368, 43464, 6008,  32108, 28212, 616,   50836,
                56408, 31328, 39952, 60208, 54712, 18808, 5464,  29376, 47828,
                2548,  54856, 13304, 28024, 36128, 1940,  61232, 15968, 54348,
                8160,  42736, 34792, 16540, 1580,  38276, 59340, 51596, 7184,
                32500, 11892,
            },
            {
                61660, 24560, 49584, 64548, 55904, 38252, 24928, 7892,  35532,
                65260, 28512, 22528, 37504, 56960, 25896, 44572, 23004, 9312,
                35360, 52876, 41620, 2436,  29040, 35004, 59744, 16316, 32248,
                1836,  20712, 25836, 13168, 18488, 39164, 51496, 60228, 12652,
                17888, 45148, 60632, 32076, 51516, 59308, 46716, 53732, 12640,
                1964,  44288, 18352, 8772,  49104, 20976, 59632, 10600, 47952,
                19928, 38864, 52196, 25188, 45916, 10408, 36208, 49672, 43016,
                60008, 47960, 27280, 53916, 24244, 60860, 29456, 35092, 55260,
                8028,  21256, 56832, 10432, 41844, 53172, 7744,  35396, 5640,
                55268, 32168, 16644, 36612, 12220, 58488, 9340,  61204, 49096,
                40160, 58728, 1264,  23492, 14204, 52572, 41328, 58716, 34072,
                44324, 2824,  10640, 15792, 46040, 880,   58512, 41292, 62964,
                35376, 32064, 46224, 64912, 8952,  56192, 19956, 28960, 48920,
                37764, 57624, 11060, 46308, 55812, 28148, 19484, 35932, 65232,
                55208, 28308,
            },
            {
                18532, 5416,  42324, 21292, 27820, 712,   44888, 19444, 53756,
                4716,  45512, 984,   51680, 12020, 6876,  53812, 19396, 60616,
                46428, 12728, 62556, 23912, 10804, 48824, 21788, 7784,  46764,
                61724, 40000, 6732,  58988, 45296, 2184,  21184, 31784, 35856,
                57160, 24380, 6176,  15204, 40188, 520,   23896, 30852, 39028,
                50060, 35464, 29608, 53000, 63160, 26764, 45372, 7296,  65008,
                56000, 27580, 5684,  33696, 21580, 54140, 6672,  61692, 32264,
                8108,  35896, 5016,  16044, 39208, 49764, 5828,  42312, 24904,
                63252, 16528, 34068, 3880,  49824, 31608, 46660, 27828, 43808,
                51196, 25180, 42056, 63556, 27568, 43928, 30288, 22888, 2880,
                20088, 33936, 10900, 38780, 62740, 45776, 8372,  12244, 38272,
                17880, 27452, 49572, 36648, 30244, 51924, 13128, 22168, 16608,
                8088,  57924, 21056, 25104, 49852, 41480, 33560, 12620, 5544,
                25992, 3116,  31376, 23656, 41104, 9228,  49216, 14464, 22132,
                41912, 1000,
            },
            {
                38876, 54516, 33116, 11492, 6660,  52420, 63024, 32772, 11360,
                26420, 56276, 15704, 34244, 42140, 64412, 32928, 38256, 428,
                51276, 30712, 7008,  57740, 38588, 54612, 13228, 42764, 57380,
                29268, 11720, 49412, 36044, 28560, 53428, 15172, 48728, 41792,
                1756,  47752, 54508, 28712, 63764, 19628, 34812, 62184, 5040,
                56696, 23344, 14628, 2800,  36764, 13316, 32880, 41088, 23576,
                1764,  43348, 49600, 13020, 63364, 41280, 28236, 3460,  18152,
                53284, 13660, 58248, 33156, 2512,  56440, 21984, 14436, 1120,
                38120, 47724, 29048, 60764, 18648, 24128, 12164, 63660, 20272,
                2712,  10548, 47380, 21744, 1516,  55988, 17624, 53492, 45368,
                64864, 51216, 29036, 56860, 18176, 26264, 21660, 65068, 4248,
                23388, 61592, 56916, 6428,  24032, 64116, 33968, 49208, 26300,
                43776, 11680, 744,   37616, 15560, 3548,  59064, 52864, 44520,
                62360, 15332, 59752, 53104, 6236,  63052, 30976, 3640,  46788,
                10568, 59996,
            },
            {
                29732, 14820, 45760, 58244, 36800, 48112, 15448, 40560, 59892,
                43212, 20932, 9448,  59160, 29172, 17344, 48428, 27612, 5976,
                16056, 25328, 33836, 18684, 45560, 1180,  25744, 37472, 5264,
                23488, 53724, 16640, 64276, 9736,  24544, 61628, 7044,  11332,
                64400, 20244, 4268,  37144, 49296, 13220, 7740,  45216, 17792,
                10192, 60620, 47348, 42108, 6388,  58340, 18748, 54376, 30316,
                15856, 36056, 57588, 18388, 1196,  38180, 47208, 56528, 26396,
                44748, 22428, 40988, 62980, 19004, 47060, 31764, 60036, 51772,
                43672, 12672, 6708,  53856, 39080, 752,   59296, 37520, 16152,
                56996, 30160, 59980, 7076,  39304, 33204, 13516, 36144, 5220,
                25472, 15696, 7852,  47920, 36524, 5448,  55508, 30452, 48320,
                53132, 13580, 40692, 20160, 44056, 9628,  39240, 2896,  61060,
                51700, 29236, 54056, 61900, 47312, 31032, 9588,  23008, 35676,
                40548, 20524, 48088, 34204, 17968, 25864, 43444, 57936, 33852,
                24304, 50896,
            },
            {
                20752, 2144,  62584, 25352, 18256, 30336, 8944,  23072, 2504,
                29848, 49888, 38124, 46468, 2804,  22000, 10344, 58620, 44076,
                61608, 40292, 49132, 65040, 14960, 29540, 51152, 63632, 18772,
                34436, 656,   41980, 32000, 3500,  43572, 55696, 29880, 39320,
                26924, 33736, 43228, 9996,  25620, 52416, 41452, 54668, 26692,
                33232, 40060, 20444, 55504, 25092, 48484, 4144,  50540, 11880,
                61616, 46916, 9144,  31344, 59864, 23604, 16004, 11348, 65168,
                37628, 848,   50964, 28732, 11688, 7756,  37392, 26656, 10056,
                57348, 20868, 25732, 65228, 44704, 9460,  32968, 54308, 5008,
                40476, 35048, 14668, 52172, 19504, 47980, 63140, 8716,  42020,
                31396, 59828, 43580, 2388,  32564, 50256, 40524, 16032, 35100,
                45136, 8608,  33468, 3952,  59984, 27856, 56108, 19256, 5592,
                35888, 40240, 17816, 7120,  43248, 26864, 64148, 17360, 50644,
                980,   28060, 11548, 4092,  38820, 60840, 11972, 52372, 16996,
                7276,  36584,
            },
            {
                43528, 9844,  50124, 40016, 3800,  60920, 44188, 56992, 36444,
                17548, 61768, 7200,  24964, 62788, 52184, 40896, 13892, 34576,
                53520, 20396, 4456,  8976,  42308, 59164, 32884, 9316,  55736,
                45900, 60880, 26524, 19840, 47600, 37364, 22048, 17172, 59044,
                51064, 14968, 61524, 56644, 32380, 2256,  21740, 58984, 13772,
                1136,  65304, 8212,  31476, 16380, 62572, 34952, 27928, 39748,
                2836,  21060, 25800, 42080, 51428, 7812,  35060, 50408, 20372,
                31540, 9700,  46120, 24096, 53092, 41900, 64684, 17240, 35388,
                4192,  31220, 50396, 34692, 14380, 22900, 42088, 27080, 48512,
                22260, 64928, 26540, 43488, 58872, 3440,  27248, 50268, 57760,
                11728, 39200, 22964, 63540, 13232, 20548, 61132, 10456, 1348,
                26444, 63752, 29864, 51260, 16936, 47028, 12212, 32304, 45636,
                14176, 23552, 59372, 34188, 20828, 13040, 39132, 55424, 7628,
                32700, 63204, 56684, 45256, 50236, 21868, 16,    27624, 39844,
                64232, 57036,
            },
            {
                26728, 53976, 31932, 22176, 13312, 54676, 26556, 5856,  51608,
                11976, 32212, 54820, 14256, 35308, 6440,  56600, 23728, 3512,
                28976, 11440, 56092, 23336, 36312, 3672,  21284, 15980, 38356,
                12416, 49936, 7900,  56980, 13860, 51856, 5200,  34772, 244,
                45608, 7960,  23088, 18028, 46004, 62724, 37784, 29888, 48400,
                36088, 51976, 28272, 43632, 11316, 38456, 21864, 8644,  59076,
                52724, 33320, 64140, 14372, 54932, 29772, 2596,  42856, 58680,
                5752,  55748, 16824, 34988, 59224, 5504,  21312, 45196, 54068,
                62364, 41428, 16628, 2728,  47528, 57468, 17696, 51888, 8392,
                12760, 45912, 816,   9652,  31096, 23100, 38444, 16216, 21104,
                1076,  55148, 17368, 53168, 28052, 4484,  57724, 42960, 53992,
                18200, 22188, 58768, 42300, 20,    36184, 53300, 21576, 65092,
                56744, 9536,  47792, 3828,  51352, 58028, 4656,  29504, 47576,
                13932, 41840, 24748, 8152,  35764, 14732, 32496, 55348, 46124,
                4412,  13520,
            },
            {
                35148, 16692, 7060,  64044, 35832, 49124, 33312, 20004, 64456,
                45804, 196,   40436, 19668, 44556, 31244, 49648, 18480, 63740,
                37532, 44808, 31656, 60568, 52728, 27628, 48672, 44088, 2668,
                30120, 22420, 40020, 33588, 59592, 10340, 62332, 41120, 25112,
                12584, 53372, 30488, 40316, 5236,  11924, 16828, 6584,  44032,
                23892, 18636, 3748,  60136, 46608, 500,   55044, 43192, 17852,
                45448, 6604,  37904, 4016,  19344, 45632, 63040, 24768, 13472,
                40168, 27756, 61520, 2956,  14268, 30104, 50192, 400,   13492,
                28380, 8740,  58692, 38072, 6424,  62748, 30676, 2044,  60464,
                32312, 55028, 18448, 36756, 53620, 12436, 61320, 45940, 33756,
                28528, 44944, 35820, 8316,  46156, 38728, 23612, 29468, 36728,
                47668, 4796,  38308, 14124, 24104, 62844, 6560,  26952, 41608,
                1408,  28472, 31680, 63348, 24596, 36244, 45884, 19112, 60528,
                22520, 53932, 17100, 30036, 64804, 58864, 42936, 9480,  18876,
                30500, 48168,
            },
            {
                3220,  57864, 41148, 46164, 852,   10592, 42424, 15084, 38976,
                23304, 28260, 49200, 57496, 26528, 1852,  38796, 9120,  26092,
                47880, 1048,  17252, 39376, 12916, 6444,  56804, 62244, 25224,
                53340, 65208, 4396,  16180, 27720, 45240, 31092, 18980, 48608,
                58108, 36392, 3280,  60680, 54292, 28080, 50652, 63884, 10452,
                54936, 58164, 14880, 49952, 33612, 26580, 64512, 13816, 24612,
                30048, 12472, 48896, 58052, 27156, 10356, 38532, 48464, 33704,
                52432, 19224, 43164, 48680, 38596, 57128, 25084, 40176, 33576,
                48260, 23596, 19872, 53016, 27860, 11220, 37132, 20748, 43252,
                39048, 24164, 62812, 49464, 42048, 4948,  55884, 7104,  51864,
                63964, 4152,  24684, 62136, 33212, 51908, 14712, 7388,  62780,
                12356, 56420, 9304,  49976, 55236, 30724, 10628, 38164, 50456,
                17468, 53972, 44000, 11468, 16184, 296,   42356, 10828, 34352,
                5332,  37464, 1572,  51720, 19920, 5184,  25228, 37380, 62252,
                23320, 60192,
            },
            {
                20516, 51756, 28708, 24680, 17924, 59652, 29632, 4824,  53684,
                8408,  59336, 16636, 10008, 65044, 13080, 60188, 42732, 55468,
                14476, 61532, 50868, 25720, 46780, 19564, 35008, 40236, 9768,
                18168, 43364, 36544, 50740, 1228,  23972, 55016, 7220,  64872,
                27636, 42816, 20604, 47836, 24608, 34932, 39772, 20468, 33028,
                5320,  40924, 37252, 22376, 7732,  18888, 50936, 5000,  36520,
                61388, 53908, 22396, 41580, 32428, 60700, 16752, 21948, 8436,
                1816,  64024, 11148, 22236, 32832, 9424,  63340, 19304, 7252,
                55256, 61208, 43132, 32624, 46128, 24584, 55648, 50424, 15100,
                6900,  29028, 4112,  15884, 34008, 25044, 29920, 18676, 10376,
                40916, 14912, 49044, 11364, 19284, 920,   60020, 49768, 19792,
                32040, 41476, 27432, 33944, 17812, 44652, 48564, 15164, 61440,
                5428,  36472, 20108, 39652, 56060, 27936, 52936, 65364, 26164,
                49084, 58436, 44828, 10684, 40680, 47768, 54448, 2200,  51084,
                11884, 39232,
            },
            {
                43664, 9724,  14032, 62112, 38316, 56036, 50416, 25484, 61980,
                35084, 43024, 4476,  37396, 30124, 53364, 21152, 32532, 5032,
                35576, 22268, 7344,  10104, 64620, 30644, 64,    14508, 59716,
                32292, 7096,  56372, 20040, 63156, 42700, 13156, 38172, 2664,
                16332, 10616, 33240, 7116,  14252, 57196, 1896,  59720, 26716,
                47400, 30340, 12140, 63296, 42244, 56588, 31916, 40304, 47856,
                2120,  9076,  35132, 15220, 68,    50960, 5368,  55604, 59640,
                30896, 37000, 26288, 54456, 4136,  16264, 52452, 46036, 36840,
                3536,  15628, 10136, 1216,  64332, 13856, 4424,  33404, 61972,
                56524, 46740, 11100, 58344, 20820, 64552, 43992, 37632, 60344,
                22400, 31700, 58596, 42488, 55936, 26500, 44132, 34868, 3520,
                24640, 46280, 63996, 2024,  21252, 60576, 3976,  25672, 33460,
                22780, 59112, 8136,  48188, 61944, 32680, 21972, 8232,  40344,
                13424, 31280, 61772, 27236, 34020, 12860, 29008, 16792, 44920,
                6508,  33248,
            },
            {
                64736, 1716,  48544, 31180, 7724,  21212, 2584,  46808, 12200,
                30856, 21604, 52408, 47328, 23920, 6124,  46136, 17056, 49832,
                29212, 40600, 58552, 33868, 43036, 53632, 22988, 49144, 44844,
                26668, 12736, 47616, 29452, 9412,  34468, 52884, 22508, 32228,
                56160, 50532, 61588, 44444, 18540, 49492, 8964,  43400, 15500,
                61912, 1088,  25064, 53844, 4032,  28404, 10588, 15396, 58368,
                27664, 19856, 65456, 55964, 24392, 43012, 35872, 27964, 46600,
                14836, 43984, 6772,  47016, 61036, 41680, 27484, 12204, 58976,
                30660, 25956, 51472, 38940, 18528, 42040, 48652, 8912,  26212,
                18780, 35560, 52620, 39892, 2748,  48176, 13536, 80,    50032,
                26880, 5440,  37264, 2724,  29976, 16120, 39820, 11048, 57252,
                53176, 16452, 7040,  58104, 36048, 8892,  39472, 56288, 43120,
                12836, 51568, 26044, 2404,  14584, 5832,  46380, 17804, 56260,
                2380,  21400, 16456, 6972,  56568, 22140, 63564, 35804, 58684,
                24428, 55640,
            },
            {
                18496, 36512, 22824, 54220, 44816, 33164, 16504, 37240, 56608,
                18156, 2196,  63176, 14160, 33908, 41492, 58444, 11108, 64216,
                2844,  52396, 18392, 27732, 3436,  16128, 57364, 36164, 5424,
                52704, 21724, 61420, 40836, 3632,  49504, 17668, 60580, 46640,
                40180, 25600, 800,   37388, 64584, 30792, 22740, 36408, 52924,
                19612, 50484, 44376, 17192, 35744, 46268, 62340, 23084, 52356,
                44584, 38408, 31116, 46068, 7884,  61916, 19016, 11632, 40848,
                23900, 51364, 17584, 57952, 29684, 35948, 1384,  22892, 42444,
                63836, 47412, 35212, 57040, 22700, 59436, 28860, 38448, 44648,
                636,   22312, 60976, 31732, 26452, 8872,  55380, 35172, 57544,
                16520, 46892, 53784, 20688, 61784, 8444,  47900, 64480, 21684,
                29188, 38932, 13540, 43616, 23788, 53544, 28856, 19048, 1748,
                64644, 45644, 29736, 35204, 52324, 38652, 59256, 29944, 50716,
                35312, 42948, 53040, 37896, 3264,  48908, 41484, 8816,  31472,
                14280, 27976,
            },
            {
                52892, 42400, 60808, 5800,  12604, 40440, 65480, 9004,  43832,
                49408, 27108, 39256, 57008, 18988, 684,   27804, 37700, 24492,
                44500, 56348, 12580, 46404, 62632, 39052, 11036, 28696, 64084,
                1928,  33544, 38320, 15736, 25332, 58192, 28296, 4960,  8556,
                14492, 21120, 53896, 28716, 12972, 41716, 3144,  56368, 11320,
                7164,  38352, 32728, 9156,  59940, 20700, 812,   34424, 5480,
                17616, 11840, 3788,  52616, 13648, 29656, 48988, 3372,  64616,
                54736, 700,   34292, 12460, 20400, 8252,  50468, 56716, 17564,
                5248,  20508, 12780, 7300,  31384, 3004,  16432, 54032, 65532,
                13088, 41524, 5764,  45652, 14972, 52096, 28944, 19720, 42660,
                9344,  65276, 12724, 34124, 50272, 23464, 31876, 5052,  36944,
                532,   60292, 51548, 31560, 47580, 12224, 62448, 50144, 32032,
                9880,  40232, 17112, 57632, 23676, 43660, 11552, 4004,  62580,
                24600, 8744,  64160, 46208, 25516, 60284, 19044, 504,   50492,
                46744, 4536,
            },
            {
                11368, 33828, 16188, 26532, 57400, 50872, 28468, 23744, 5916,
                60992, 32672, 10744, 7428,  50120, 61792, 53680, 8396,  14296,
                32256, 21436, 6360,  35828, 24968, 7196,  20724, 50572, 41552,
                18436, 58968, 6720,  54400, 45324, 12148, 36284, 41924, 63704,
                30756, 57676, 5552,  9704,  51156, 58900, 24732, 46300, 31760,
                27764, 65120, 56820, 14504, 49260, 39344, 54152, 26284, 42536,
                60664, 49692, 25400, 40416, 21836, 57876, 38008, 33048, 9016,
                21304, 38724, 61832, 25684, 40488, 65352, 45172, 32784, 9764,
                39312, 28592, 54900, 44852, 62972, 50348, 11312, 34660, 24200,
                51080, 30016, 17488, 57756, 38156, 23272, 62772, 5956,  32136,
                24572, 38872, 28456, 6788,  41628, 55308, 13776, 44708, 17980,
                48936, 9908,  26328, 19592, 3012,  41224, 16524, 7100,  37440,
                54592, 21528, 3508,  63000, 8592,  19288, 27128, 37312, 14292,
                48444, 18944, 28928, 12052, 32520, 15248, 53596, 37604, 21816,
                62608, 39480,
            },
            {
                59200, 3036,  30224, 47688, 432,   18800, 59088, 36032, 14724,
                41012, 54300, 25012, 44580, 29912, 35864, 20836, 47796, 61100,
                38884, 1024,  59808, 48600, 54960, 43484, 61028, 31724, 14156,
                47168, 24176, 10660, 30796, 64976, 2576,  19212, 51920, 23560,
                45056, 35152, 39504, 46900, 20112, 34428, 5084,  17352, 60356,
                42472, 21328, 2516,  24404, 29832, 6692,  12772, 64432, 30384,
                8904,  56796, 35796, 63644, 1144,  16476, 53392, 26336, 59000,
                13980, 30528, 5060,  48316, 53496, 3704,  15092, 24764, 53876,
                48700, 62028, 84,    41060, 26384, 37108, 20924, 5276,  58792,
                8296,  47484, 62272, 10644, 33692, 3364,  40204, 46500, 60004,
                2128,  51980, 45052, 17304, 3200,  59928, 27352, 52456, 62476,
                33292, 42228, 56324, 65264, 5860,  34676, 57492, 24528, 46296,
                27552, 13724, 49268, 42176, 31192, 53404, 60212, 45020, 31896,
                55428, 1540,  41184, 57904, 4388,  44328, 10064, 28540, 57092,
                7048,  25636,
            },
            {
                20496, 51804, 64228, 38768, 22112, 10272, 44492, 3732,  51840,
                20920, 1440,  17272, 64704, 12912, 3224,  43492, 16344, 26520,
                51236, 42092, 29108, 13864, 32916, 17636, 1296,  26340, 8364,
                56444, 36812, 51708, 42968, 22096, 48128, 32844, 55660, 1332,
                11084, 17136, 62128, 26348, 14796, 63608, 40340, 54416, 13676,
                48004, 6408,  52348, 40976, 45488, 58700, 18692, 37152, 47172,
                14556, 20256, 6352,  31852, 44252, 10368, 46776, 5848,  41632,
                50352, 44804, 56840, 10320, 18292, 31172, 37952, 58812, 2764,
                34764, 14740, 23616, 17828, 8940,  47632, 56096, 42600, 31892,
                39380, 26696, 1492,  20620, 55688, 49604, 18008, 12352, 54408,
                15196, 21820, 62288, 57004, 35440, 19948, 39464, 11392, 24864,
                6716,  22480, 15016, 37808, 27716, 49576, 60708, 20092, 324,
                63848, 56456, 34184, 5600,  15372, 47304, 652,   22392, 6196,
                38900, 60932, 23048, 35224, 51284, 20816, 65248, 34452, 42964,
                16684, 45428,
            },
            {
                36620, 8352,  14140, 42800, 34612, 55092, 31928, 63400, 29388,
                47040, 58528, 37912, 48676, 22608, 55172, 33332, 4704,  56764,
                7588,  18584, 64036, 23548, 10016, 51796, 58064, 45788, 39636,
                63120, 2288,  28128, 16908, 5708,  38808, 61756, 14152, 27792,
                50216, 59672, 4216,  32232, 53212, 8328,  28428, 37168, 1704,
                26248, 34232, 17664, 63320, 10988, 33348, 51724, 2004,  23984,
                40932, 53672, 28240, 48748, 60316, 24556, 35272, 20012, 63452,
                16756, 27556, 22560, 35244, 43140, 62916, 28028, 21228, 11700,
                43736, 30544, 58472, 53072, 33552, 13296, 60736, 2000,  18900,
                14780, 54760, 36336, 44276, 27704, 64768, 8568,  30260, 36592,
                43144, 32908, 10448, 25912, 47324, 7916,  30944, 2344,  57556,
                36104, 47196, 1548,  54228, 44536, 8404,  32424, 14340, 42984,
                30552, 10892, 38936, 25608, 58208, 36132, 10624, 64272, 17872,
                51592, 10224, 15880, 47664, 6640,  26236, 54904, 1608,  12188,
                32124, 49956,
            },
            {
                28700, 56316, 24636, 61080, 6880,  49644, 16772, 25192, 8116,
                11888, 34060, 5132,  26748, 59748, 9864,  40052, 62760, 30416,
                11948, 35236, 53676, 3764,  40900, 36332, 5232,  22344, 33844,
                12240, 20440, 55396, 34836, 60144, 9360,  24756, 41196, 7000,
                37508, 20716, 42668, 48760, 72,    44312, 22660, 50320, 10680,
                62588, 55196, 37844, 27392, 3464,  21940, 43056, 56044, 7480,
                33880, 59504, 3096,  17964, 38492, 13836, 55036, 30864, 2088,
                36880, 7888,  61428, 1016,  13548, 50588, 5412,  46560, 61076,
                51572, 7600,  37852, 3840,  64796, 28232, 23888, 45304, 51476,
                64008, 23108, 48932, 6168,  14324, 41708, 24844, 59300, 19524,
                6664,  51120, 716,   38468, 14512, 53032, 64816, 44100, 50076,
                13284, 61404, 31176, 17780, 23224, 11724, 39816, 53344, 48640,
                4952,  22704, 52224, 61748, 19816, 28880, 40840, 54732, 33648,
                26516, 43220, 29640, 62944, 38228, 17300, 40852, 48784, 24048,
                58276, 4620,
            },
            {
                10980, 18320, 46896, 2216,  27824, 12648, 37444, 60544, 40300,
                54768, 43980, 15080, 30784, 42116, 51876, 17760, 24340, 46384,
                49780, 21672, 44680, 59124, 27304, 47680, 65452, 15592, 29720,
                51016, 4576,  46016, 13824, 49404, 29488, 18468, 57948, 46316,
                30276, 54836, 24384, 12320, 18516, 56132, 61496, 33084, 20308,
                44932, 14856, 8156,  60432, 49820, 14200, 28588, 62000, 16632,
                50584, 11616, 22940, 43128, 8604,  62948, 4720,  49136, 58184,
                12064, 52900, 47296, 39360, 24132, 54720, 8800,  40156, 16840,
                26816, 22320, 41700, 49664, 20096, 6724,  40456, 30196, 9636,
                4344,  33440, 11428, 60812, 31408, 52992, 2472,  47428, 56552,
                27272, 63648, 48724, 30060, 60564, 22976, 40992, 16928, 28120,
                20896, 9348,  40832, 50792, 63352, 56824, 2444,  26392, 36692,
                59956, 17920, 45528, 2064,  7928,  50624, 23772, 13876, 48236,
                7752,  56492, 3760,  12892, 58852, 31832, 8968,  61760, 15052,
                38616, 63080,
            },
            {
                43816, 33296, 39552, 52332, 20996, 58012, 45672, 904,   18752,
                22504, 63956, 50728, 8332,  20100, 76,    36112, 13804, 58156,
                2484,  38364, 8752,  14848, 31024, 19632, 10800, 54920, 43708,
                61960, 40676, 25940, 56972, 356,   43752, 52964, 3668,  63868,
                15904, 8960,  35484, 65340, 38480, 29744, 15724, 3624,  58092,
                40756, 30812, 23212, 46484, 35136, 58168, 9808,  32036, 25680,
                45096, 65024, 35664, 56848, 51232, 28948, 21764, 39924, 25612,
                43596, 32360, 19268, 29556, 59392, 33152, 20216, 35780, 57100,
                1276,  63228, 11044, 15608, 46072, 35692, 53928, 16944, 59260,
                38020, 43440, 57240, 17332, 39056, 21568, 33956, 5512,  38232,
                11868, 23352, 41600, 18336, 3996,  10896, 34224, 6432,  54684,
                38188, 58268, 25968, 3660,  35160, 29868, 19480, 62048, 15280,
                8652,  33616, 41304, 13332, 55632, 32580, 43944, 2440,  59740,
                20788, 36852, 46344, 24212, 52480, 2580,  28312, 45580, 20008,
                2944,  53592,
            },
            {
                22208, 6064,  64032, 15988, 31512, 41940, 5836,  52692, 33000,
                28216, 3472,  36816, 56148, 60776, 45248, 28688, 65060, 6944,
                32620, 26584, 56088, 62248, 496,   52684, 37780, 6676,  24712,
                18056, 9240,  37104, 31312, 63004, 22684, 36060, 12688, 32592,
                50580, 2108,  57288, 27088, 6116,  42016, 9720,  48516, 25816,
                6220,  52208, 836,   42252, 19636, 4460,  39944, 47732, 360,
                38664, 5792,  15384, 26612, 1552,  33568, 46240, 9292,  17688,
                65096, 3652,  57212, 15252, 4896,  45292, 64600, 12520, 29428,
                47520, 34216, 55592, 30836, 57816, 740,   62404, 12692, 47368,
                25860, 20032, 28996, 264,   50176, 9984,  63332, 44440, 16940,
                52444, 35368, 7696,  58524, 55180, 45780, 25656, 62436, 112,
                32340, 44884, 7156,  16208, 47736, 41992, 6028,  46228, 55060,
                23956, 50152, 29620, 65408, 37044, 17472, 62264, 5508,  38756,
                31200, 64836, 17996, 34248, 41848, 21892, 56924, 35624, 51308,
                25960, 30592,
            },
            {
                48552, 13392, 56032, 36708, 9948,  62152, 24348, 14388, 57164,
                48612, 11016, 17248, 25404, 33140, 4348,  53600, 39292, 20448,
                43600, 50528, 17604, 34520, 23424, 42112, 60696, 32836, 2788,
                48200, 59736, 21232, 6908,  16340, 10524, 39728, 26404, 19964,
                44624, 40728, 21952, 51764, 46668, 60132, 34312, 54132, 37068,
                18116, 64784, 12496, 57180, 25236, 53020, 63864, 18248, 55288,
                21420, 30168, 48008, 41200, 19128, 59688, 54548, 14336, 35120,
                51024, 22316, 41168, 10836, 49272, 26440, 2136,  43344, 53508,
                6464,  18984, 25220, 43588, 9260,  27024, 21448, 32100, 3620,
                52932, 7840,  62692, 45964, 24020, 54580, 13880, 28460, 59724,
                1884,  46392, 15316, 32024, 20652, 36904, 51020, 14064, 48452,
                18880, 64256, 52512, 12920, 59680, 22352, 10212, 38368, 28104,
                1648,  59084, 20236, 4608,  25476, 10032, 46812, 28008, 15300,
                53080, 9172,  304,   49756, 10540, 61224, 5388,  13004, 42188,
                8012,  60272,
            },
            {
                40372, 188,   26352, 49844, 3580,  29024, 47232, 35268, 7340,
                42740, 61256, 39128, 46636, 13032, 23032, 9976,  48304, 15872,
                60084, 11508, 40448, 4836,  46108, 13124, 49660, 27212, 57656,
                12072, 34116, 53948, 42060, 50764, 46648, 60440, 55080, 5108,
                61696, 29136, 10960, 14868, 3392,  19736, 23708, 13352, 59408,
                28872, 47140, 33192, 38396, 15664, 29664, 7328,  35600, 13496,
                58580, 9988,  61476, 53348, 12372, 6240,  42440, 62104, 128,
                29104, 6536,  54176, 36080, 61824, 17476, 38612, 23196, 15492,
                61504, 37544, 3428,  52344, 60604, 39272, 49968, 42796, 36276,
                56656, 15216, 40912, 35492, 5272,  58360, 42136, 31904, 20264,
                25480, 64964, 53804, 27948, 43320, 2968,  59684, 30220, 41268,
                10500, 22876, 28268, 37340, 55400, 32716, 64648, 50864, 43524,
                35328, 11768, 47384, 53220, 41960, 58740, 21444, 49988, 57604,
                24976, 42872, 29020, 55996, 15588, 38400, 47504, 18720, 64968,
                33532, 16720,
            },
            {
                58800, 35124, 44924, 22904, 58460, 17704, 38700, 65284, 20168,
                23572, 31004, 1248,  50344, 62912, 42012, 57444, 26868, 31324,
                1396,  24752, 52368, 63616, 29448, 7780,  21024, 15848, 39288,
                45120, 28152, 4496,  64136, 24524, 1720,  33644, 8324,  48012,
                16584, 34876, 58504, 37744, 62564, 31968, 44232, 1208,  40412,
                8688,  21784, 5216,  61940, 10636, 48992, 44848, 23904, 42708,
                50376, 33384, 2392,  24276, 37324, 31248, 20980, 26168, 38296,
                48344, 59848, 24352, 31496, 7792,  51844, 56636, 32432, 9032,
                50336, 41000, 12292, 22856, 14892, 7020,  17424, 64172, 10576,
                23476, 48648, 32324, 12324, 26912, 18524, 36844, 7224,  49876,
                10552, 39396, 5240,  12708, 61800, 9232,  17088, 24056, 4540,
                57332, 34504, 43188, 1312,  18196, 26536, 4020,  13840, 17400,
                62928, 7380,  38096, 15920, 30928, 844,   34816, 40100, 6812,
                12176, 45924, 20120, 63164, 32692, 26136, 53244, 29532, 1632,
                54980, 10784,
            },
            {
                28448, 19008, 53160, 7952,  42864, 12256, 54816, 1800,  10440,
                52112, 58624, 14920, 28912, 7476,  18276, 34392, 5356,  62548,
                54504, 37096, 9256,  19144, 58632, 35900, 55064, 64632, 964,
                52916, 19584, 14216, 38068, 30476, 18536, 58160, 28332, 23472,
                53696, 572,   42904, 24984, 48868, 7412,  52104, 64016, 16736,
                50440, 54428, 44392, 26948, 56068, 1784,  31940, 60184, 4228,
                26884, 40032, 17652, 45168, 64532, 49052, 10412, 56964, 43784,
                16656, 12040, 45948, 19512, 42504, 28376, 3340,  46980, 59596,
                26184, 30376, 65520, 48416, 35192, 31236, 55324, 2464,  29176,
                18972, 59356, 1912,  65216, 52092, 47064, 2780,  62060, 56320,
                44196, 34784, 22220, 48272, 40172, 33796, 55420, 46936, 37644,
                51808, 6096,  61360, 49828, 8936,  45988, 53296, 21428, 31216,
                55656, 23000, 27500, 56912, 44684, 13448, 64236, 18340, 32080,
                61656, 35984, 6144,  39824, 3088,  22784, 7664,  44332, 37760,
                24136, 46480,
            },
            {
                5340,  64368, 14952, 30580, 60872, 33724, 50312, 27648, 45200,
                40420, 5288,  36084, 55772, 21736, 37992, 51344, 46160, 13964,
                21404, 44348, 32524, 48172, 2300,  25668, 43940, 31744, 10116,
                23932, 61556, 48632, 57084, 11216, 44996, 14596, 41828, 38176,
                64820, 9336,  30700, 18048, 56184, 28064, 10728, 35476, 25496,
                31252, 2632,  36272, 19984, 41188, 16852, 52180, 11284, 19724,
                63132, 7904,  58032, 29288, 4436,  16092, 52612, 3208,  32944,
                8224,  64304, 37704, 1212,  13612, 63396, 21408, 11588, 36180,
                4392,  20132, 55012, 1308,  44176, 58784, 25036, 46364, 40144,
                51468, 7212,  43764, 21612, 30300, 9572,  39960, 23420, 13224,
                29460, 448,   58040, 26132, 51340, 19300, 1284,  27604, 62664,
                15624, 20680, 12624, 30372, 58696, 40748, 34904, 60300, 284,
                39576, 49400, 4328,  61212, 8884,  24512, 51416, 3964,  54792,
                22644, 48592, 53768, 13680, 59668, 49960, 57956, 14744, 61952,
                20660, 50848,
            },
            {
                32724, 38116, 2384,  40256, 20876, 5164,  24488, 15264, 31996,
                62004, 18976, 25788, 43720, 64752, 11664, 1944,  59468, 39988,
                6760,  27992, 57076, 12144, 39872, 51540, 18032, 5980,  41720,
                36116, 29552, 7980,  3368,  34356, 52532, 62508, 6040,  21028,
                12940, 45156, 52700, 4176,  14060, 39768, 22192, 58196, 42124,
                12120, 57104, 60768, 9024,  34240, 65448, 22716, 38684, 54320,
                35928, 47120, 12804, 21884, 55792, 35080, 40572, 23740, 59032,
                20316, 27864, 54672, 48644, 57960, 33800, 39880, 53744, 17140,
                42576, 46100, 8004,  28416, 18616, 38248, 5192,  13604, 62032,
                33688, 25832, 38360, 14452, 55848, 60436, 16376, 33260, 53460,
                18872, 62696, 15800, 8124,  30984, 64516, 44752, 11164, 41756,
                32260, 25628, 47640, 38524, 22500, 5284,  11856, 44588, 26188,
                9888,  43292, 33676, 20016, 36688, 47604, 29168, 42688, 10140,
                27148, 2400,  17060, 28744, 42420, 10256, 35424, 31592, 3924,
                41880, 9244,
            },
            {
                25556, 55372, 48396, 11264, 56476, 46632, 63456, 37804, 7648,
                54492, 12572, 46904, 4304,  52840, 32220, 24468, 29216, 17948,
                49900, 64316, 15640, 23580, 5112,  59732, 13568, 47332, 57584,
                51224, 16132, 44320, 40696, 26160, 19888, 49080, 2888,  32768,
                50552, 24420, 61064, 34132, 47112, 62956, 1820,  45744, 5100,
                18708, 48312, 14568, 24536, 50240, 6524,  46540, 28792, 1080,
                15720, 31356, 51868, 43500, 9280,  27196, 61600, 13212, 44976,
                50504, 35420, 5744,  25520, 16268, 44800, 5404,  24552, 62324,
                57172, 13416, 34280, 61268, 10248, 49812, 21880, 53704, 9656,
                16620, 56968, 47812, 4256,  35756, 25200, 42780, 49024, 5652,
                38376, 46144, 35904, 54556, 4800,  14168, 23264, 58500, 7348,
                53900, 1840,  65308, 56012, 16160, 29000, 63384, 19308, 57508,
                52212, 16956, 12956, 53060, 1844,  62744, 16068, 56728, 38668,
                60876, 44784, 33720, 64732, 21500, 46376, 18108, 27012, 54184,
                13064, 59648,
            },
            {
                44448, 21752, 62776, 27244, 35444, 18144, 3272,  58312, 21412,
                48996, 34680, 30312, 9732,  40576, 15684, 56180, 44596, 9132,
                34192, 292,   42476, 36564, 62300, 30472, 34732, 26852, 20048,
                4000,  59252, 22388, 65252, 55300, 9512,  36928, 56896, 28840,
                59452, 40336, 19056, 7172,  37076, 20564, 53376, 33032, 61336,
                28444, 37624, 43840, 30568, 2912,  58236, 13792, 43300, 56660,
                25652, 5068,  62368, 36868, 584,   47748, 19052, 6776,  29688,
                2528,  41468, 11040, 61352, 30900, 8588,  49384, 28204, 548,
                31696, 22524, 51560, 40220, 15544, 63904, 27564, 36388, 44948,
                460,   29500, 63772, 19932, 8280,  54204, 1544,  64560, 26756,
                9980,  50704, 20736, 42132, 60312, 38920, 34644, 50096, 19904,
                36924, 44420, 9856,  33900, 3480,  47088, 50568, 37432, 5784,
                31924, 65020, 28808, 58608, 39856, 22216, 5604,  35188, 13956,
                19596, 50908, 8376,  37720, 5160,  56560, 492,   63488, 47916,
                36216, 17568,
            },
            {
                1488,  6452,  15804, 42008, 8264,  52556, 43892, 29088, 41208,
                1280,  16696, 56876, 61248, 20412, 48408, 3692,  37552, 60712,
                53276, 25944, 55892, 46312, 21052, 8412,  53760, 63968, 11076,
                38996, 33380, 12764, 512,   31128, 16484, 23800, 43160, 8040,
                15844, 1152,  27292, 11336, 56976, 29952, 15972, 9040,  50156,
                23512, 7316,  63220, 52756, 40332, 18464, 32688, 9204,  63612,
                49544, 40424, 18688, 23988, 57708, 31104, 51220, 38980, 55220,
                62856, 17552, 22748, 52404, 37940, 20384, 64936, 52044, 10492,
                38580, 47908, 5968,  25524, 55880, 42416, 3456,  31452, 60656,
                23840, 42068, 11904, 51160, 32344, 45648, 12464, 21888, 31508,
                58788, 3240,  28692, 11668, 25156, 47700, 3940,  29812, 14840,
                60892, 28364, 18432, 51932, 42636, 25432, 8504,  14976, 23928,
                41584, 2696,  46628, 25204, 10996, 45316, 32364, 49392, 25624,
                876,   58296, 30776, 12944, 26040, 51836, 39916, 22668, 6796,
                29248, 52312,
            },
            {
                34164, 49840, 30188, 59976, 23884, 32628, 14100, 10652, 26120,
                62532, 50984, 23636, 2608,  28192, 35240, 63584, 22688, 13380,
                19884, 31168, 10732, 3724,  16900, 49708, 40364, 1708,  23796,
                45592, 28328, 53188, 48776, 38436, 46292, 60816, 12304, 34696,
                47972, 55580, 43936, 63784, 48752, 41080, 25988, 12856, 39220,
                55476, 124,   21096, 11532, 26192, 61484, 37768, 23716, 20104,
                34516, 7612,  59324, 14700, 42052, 10824, 65160, 15388, 24784,
                46704, 32868, 57572, 43324, 1980,  14188, 34672, 41404, 16052,
                58320, 18892, 60376, 1856,  32872, 20068, 7908,  47992, 18112,
                55052, 6528,  39424, 58392, 27672, 37920, 17692, 57056, 40732,
                14640, 44360, 63696, 33364, 17940, 57396, 9420,  52824, 40628,
                2860,  56844, 23540, 12872, 60596, 31120, 53692, 62380, 34324,
                48912, 21340, 7936,  35556, 55608, 18164, 59248, 8044,  62992,
                39440, 22844, 54488, 41832, 61432, 15520, 32368, 44980, 10936,
                61192, 40684,
            },
            {
                57244, 9928,  37188, 54700, 692,   48072, 64284, 55600, 35924,
                45508, 8456,  38280, 43184, 58896, 11916, 7404,  51116, 41948,
                4868,  48288, 38964, 58704, 24924, 43792, 29228, 15184, 55436,
                61104, 8548,  18828, 62764, 5720,  27360, 2432,  52744, 64360,
                21424, 38288, 31392, 23216, 2376,  5992,  58912, 45060, 64996,
                34364, 16600, 47136, 35736, 56020, 3776,  48956, 54036, 2616,
                45524, 11824, 29348, 53368, 4292,  33628, 21688, 1712,  36404,
                9236,  13072, 4600,  26652, 60276, 46140, 56208, 23504, 3768,
                44644, 29908, 36664, 12136, 45848, 52976, 58868, 37840, 13764,
                50384, 34512, 21772, 15708, 2428,  62984, 48768, 7036,  34940,
                52292, 23152, 7680,  53964, 1620,  42716, 21584, 64268, 26148,
                45604, 6060,  35428, 38844, 732,   20224, 39992, 4640,  11984,
                56128, 61332, 15612, 51296, 42348, 4272,  27536, 53420, 43684,
                11204, 47052, 17436, 2920,  48336, 9404,  36824, 19900, 56144,
                14540, 23292,
            },
            {
                64556, 19024, 12844, 44860, 20296, 6308,  38584, 16192, 4368,
                19848, 32704, 13688, 52448, 17656, 46612, 26232, 32940, 57780,
                28040, 65344, 14448, 52664, 32756, 61888, 7128,  35748, 48052,
                4760,  42492, 35108, 14552, 21828, 58568, 40164, 18328, 30176,
                4928,  9644,  14028, 53944, 17144, 33860, 51736, 19404, 4240,
                30320, 42200, 59968, 8640,  29304, 44372, 12848, 16172, 31080,
                60424, 50980, 37376, 25840, 47472, 56156, 44048, 27744, 60420,
                53064, 40532, 49740, 18644, 29392, 6412,  11864, 32276, 62092,
                26432, 7268,  54520, 64792, 14928, 29060, 23484, 10124, 63064,
                26220, 3604,  46880, 61152, 9564,  23544, 42484, 29564, 764,
                60180, 16816, 39260, 48604, 61640, 31072, 37120, 11908, 17340,
                32120, 62464, 48788, 54640, 16384, 46444, 58916, 27692, 43956,
                18700, 30068, 38416, 468,   31224, 64308, 12628, 21116, 30392,
                36504, 6488,  27728, 34420, 58600, 24204, 63980, 1848,  49020,
                27508, 3984,
            },
            {
                46436, 25844, 39984, 61836, 28288, 51312, 22884, 30648, 47560,
                59096, 25372, 64972, 5612,  30692, 55224, 40308, 1064,  16072,
                45600, 7948,  35260, 19496, 868,   12080, 51008, 21504, 17516,
                31752, 57416, 24708, 51512, 44640, 32288, 7524,  50452, 24852,
                57540, 45300, 49856, 36424, 60340, 42920, 28568, 8480,  24628,
                53572, 13632, 23044, 50432, 17984, 5380,  40788, 64624, 27044,
                42512, 21540, 1464,  62716, 18128, 8140,  38636, 48848, 6484,
                30368, 21348, 64440, 35544, 54028, 39812, 48620, 19624, 52888,
                9824,  50068, 22108, 39968, 4296,  35168, 43968, 1256,  40568,
                29836, 56732, 43244, 31664, 52596, 36328, 55924, 20324, 11248,
                46396, 25796, 35540, 12852, 27616, 6564,  46736, 56116, 51124,
                8092,  20860, 11252, 29196, 64776, 9148,  22776, 36452, 2916,
                52872, 9692,  26460, 59820, 23096, 47396, 40040, 16748, 3736,
                50816, 65472, 56252, 20608, 44044, 5908,  29472, 54316, 38132,
                42888, 32160,
            },
        },
        {
            {
                5216,  22184, 58012, 50416, 41856, 62372, 5852,  48788, 19452,
                64004, 32664, 6408,  29484, 48312, 62156, 25484, 17756, 5712,
                39176, 48588, 33872, 25384, 2752,  53580, 38760, 30676, 26176,
                45732, 18836, 42480, 65052, 39140, 15856, 60064, 23504, 38104,
                63392, 57708, 31700, 53532, 35368, 16312, 43728, 52308, 64252,
                27504, 8436,  31988, 24868, 3852,  54948, 63896, 16208, 8232,
                19824, 30292, 4772,  16060, 22628, 52516, 25816, 60744, 4156,
                17752, 31068, 44556, 57400, 15028, 53424, 43204, 48504, 63856,
                45852, 5148,  56140, 40356, 25088, 14388, 3328,  56524, 12216,
                18860, 58472, 51976, 9144,  24468, 40092, 2068,  46672, 22396,
                49036, 19072, 54320, 44008, 7624,  36652, 4960,  41396, 29956,
                45936, 17104, 62920, 27880, 53688, 852,   60748, 13492, 53168,
                48208, 8464,  55272, 62440, 18300, 40192, 23964, 15860, 8024,
                48192, 35648, 4544,  32956, 13472, 2896,  52972, 16644, 30100,
                25528, 51340,
            },
            {
                33008, 46980, 2320,  10900, 16748, 33204, 52948, 24212, 40448,
                13204, 51208, 57860, 23020, 8704,  3220,  32816, 60416, 43860,
                56440, 1044,  59028, 42656, 64616, 8252,  21496, 13140, 62268,
                55720, 5348,  23092, 52872, 46276, 20796, 6636,  44264, 50804,
                12596, 19092, 6012,  23916, 46716, 6728,  59868, 2192,  20108,
                46032, 15940, 43164, 62216, 21104, 46724, 26372, 2328,  36860,
                57696, 25180, 34900, 49240, 55976, 24,    45040, 39800, 48720,
                35376, 63580, 25164, 11660, 62264, 1644,  24296, 11016, 34016,
                14208, 37724, 20540, 2404,  33364, 53944, 21652, 35424, 30516,
                46796, 40668, 16152, 35120, 45468, 13088, 54168, 27216, 35492,
                6300,  62852, 13804, 33184, 47996, 65380, 11720, 55592, 584,
                49716, 23540, 35720, 11944, 21324, 42632, 25836, 5388,  22516,
                58276, 32264, 24628, 45240, 336,   48688, 63568, 53516, 28348,
                18732, 57840, 44340, 21176, 39460, 26500, 35276, 63212, 57192,
                10148, 14652,
            },
            {
                18668, 42712, 30568, 54308, 21072, 27616, 9112,  43300, 30228,
                204,   37208, 45520, 18564, 35500, 54920, 46712, 10020, 23660,
                31016, 16772, 51568, 22376, 29620, 17840, 51216, 41756, 932,
                37164, 60268, 30276, 3056,  11392, 28608, 48968, 32680, 1504,
                27152, 36092, 40624, 65304, 13500, 26884, 38548, 32600, 11972,
                56436, 36900, 224,   52020, 38728, 13112, 31228, 53220, 42188,
                11924, 64264, 46216, 8992,  18808, 31136, 12796, 21236, 57684,
                15308, 2096,  51812, 41412, 21056, 32336, 39916, 17312, 57656,
                51376, 8240,  59764, 46912, 10312, 62504, 45204, 6588,  60812,
                26632, 280,   63148, 28600, 50720, 19456, 3968,  61524, 15512,
                52672, 38036, 28244, 3636,  22876, 17328, 52008, 34424, 26600,
                8388,  43212, 56076, 4376,  50952, 33644, 63464, 45800, 36584,
                16404, 39776, 11880, 52468, 7412,  36080, 21144, 4820,  59488,
                40888, 24484, 8824,  55428, 60212, 48660, 11552, 23248, 45404,
                37492, 54644,
            },
            {
                27920, 60764, 7136,  38960, 63548, 45492, 59496, 15280, 55240,
                61920, 10596, 26724, 14860, 65356, 41900, 20592, 13712, 37900,
                63452, 6944,  12980, 40012, 46828, 4556,  58400, 32652, 24664,
                10524, 48236, 16260, 34832, 55080, 62720, 37304, 14840, 61340,
                52748, 56564, 10072, 49352, 21220, 57964, 50916, 42016, 23468,
                63160, 49480, 29928, 10572, 58888, 4888,  18132, 61180, 22044,
                49752, 1588,  14444, 43036, 59524, 37060, 64768, 8028,  28036,
                43332, 6392,  36404, 28592, 7288,  49456, 60948, 4580,  27936,
                42368, 24736, 30364, 52564, 17396, 28064, 38812, 51352, 15020,
                54872, 23224, 11400, 57240, 7004,  31216, 38952, 43788, 58800,
                24488, 376,   42660, 60248, 56748, 30788, 45080, 21064, 38616,
                63664, 14640, 30996, 61088, 38196, 15488, 7864,  29292, 50380,
                1704,  64992, 19880, 34192, 29060, 57100, 10480, 46496, 31920,
                1436,  51960, 29668, 15732, 5680,  31036, 42184, 19448, 6644,
                3044,  64852,
            },
            {
                12136, 23016, 48828, 14028, 36176, 1896,  6440,  34600, 22124,
                46940, 4948,  58736, 39528, 1944,  30072, 52384, 3964,  47828,
                54128, 27760, 36288, 61940, 9808,  34728, 15112, 44472, 54332,
                19540, 40584, 26660, 59248, 42908, 4792,  18616, 9492,  42096,
                22284, 4208,  33380, 44120, 1116,  30076, 9504,  17288, 4324,
                26144, 7256,  19532, 35228, 27588, 48800, 44260, 33752, 7108,
                28224, 38336, 32660, 23412, 27252, 3076,  51092, 33256, 54536,
                23880, 60168, 18532, 55772, 45904, 10108, 53092, 37296, 19376,
                692,   64552, 13108, 36236, 43452, 1272,  19700, 9628,  42148,
                32844, 48912, 5124,  42772, 36112, 65068, 21856, 8088,  10960,
                30576, 50800, 20072, 9584,  39704, 6756,  14080, 3052,  58172,
                52884, 19352, 2048,  25212, 48280, 20008, 56680, 10676, 59528,
                26364, 6216,  42964, 60240, 15484, 50868, 25624, 38268, 14240,
                61792, 11208, 36672, 45916, 64476, 848,   53356, 59404, 49916,
                33772, 40928,
            },
            {
                52132, 412,   57632, 26556, 18384, 56332, 51088, 39196, 17428,
                28668, 53072, 32400, 49848, 24272, 7788,  59512, 26012, 33332,
                18148, 42420, 3672,  55964, 24144, 48776, 60060, 28356, 6696,
                64084, 51904, 8120,  13672, 21980, 51244, 25468, 55860, 29788,
                47268, 16400, 26112, 60636, 18408, 36704, 55140, 60912, 45264,
                39768, 53596, 13792, 56904, 22596, 65228, 39488, 9944,  56160,
                17236, 52228, 62312, 54908, 6860,  40424, 17644, 45988, 10320,
                38192, 48380, 12700, 65000, 33840, 25348, 14588, 30816, 45352,
                56252, 49044, 22052, 6140,  61432, 55684, 24824, 64848, 3824,
                20904, 37800, 61100, 17848, 24920, 13948, 56240, 47852, 34292,
                17900, 46148, 55128, 35272, 25760, 49420, 61240, 24368, 10172,
                32728, 41348, 46320, 11116, 54260, 4912,  39976, 43764, 18164,
                31808, 47252, 53376, 23456, 2284,  41904, 64340, 18200, 43776,
                55788, 21996, 49972, 18392, 25352, 38696, 14688, 28740, 9440,
                25196, 16624,
            },
            {
                36400, 43236, 32352, 9780,  47248, 23740, 30904, 11108, 63792,
                8132,  42416, 19388, 12132, 61848, 44196, 16268, 38856, 57252,
                9024,  21700, 50132, 31868, 18588, 1384,  38532, 12388, 22612,
                2460,  36412, 32280, 47340, 316,   39368, 33420, 64592, 7368,
                38232, 58448, 12416, 39896, 52600, 5892,  47700, 14648, 34388,
                31012, 61796, 47004, 42356, 6148,  1208,  15044, 25000, 60280,
                46928, 5092,  19848, 10964, 48260, 57948, 14016, 61872, 4044,
                20564, 31304, 2852,  40544, 5636,  21484, 58204, 63272, 7476,
                34844, 10768, 40712, 32108, 15412, 47720, 29860, 35028, 58028,
                44996, 13304, 29360, 53832, 45808, 1012,  51768, 27668, 41136,
                62692, 6144,  12548, 64080, 16128, 53496, 42876, 35732, 47800,
                27948, 6964,  59256, 36196, 30304, 64312, 22764, 35048, 13932,
                55176, 38064, 9352,  12936, 36960, 31104, 4396,  8236,  34136,
                27204, 2876,  41620, 32840, 8172,  57396, 35088, 21008, 62000,
                46784, 58428,
            },
            {
                8308,  21268, 53456, 64292, 41408, 3756,  60200, 43916, 49260,
                25648, 3140,  36808, 54692, 21736, 34916, 49016, 11364, 64512,
                284,   44564, 61084, 11732, 27300, 65124, 42820, 50464, 30820,
                45320, 55660, 17628, 62972, 29208, 58312, 12624, 45532, 1796,
                20496, 50176, 31552, 8396,  63816, 24560, 28392, 2212,  21660,
                11216, 3608,  17592, 37412, 32428, 54476, 50252, 29876, 35948,
                2676,  41532, 30988, 44644, 34980, 21608, 30164, 26152, 42428,
                52200, 58100, 27196, 16924, 51504, 43424, 1884,  39352, 23576,
                16696, 27344, 60084, 53324, 4064,  38272, 12140, 50924, 17164,
                26156, 2244,  49944, 9424,  39784, 32548, 5532,  15944, 57832,
                2480,  23052, 39084, 28680, 1392,  31624, 5632,  18684, 64760,
                15152, 51624, 22192, 17176, 44952, 27116, 120,   50920, 61164,
                5352,  21676, 28768, 62056, 49708, 58088, 20848, 53256, 48572,
                62952, 6232,  58592, 53624, 12760, 48084, 44212, 2464,  40208,
                30908, 4372,
            },
            {
                62724, 29204, 16068, 5360,  35224, 28040, 20168, 14568, 34040,
                56928, 60908, 15464, 46100, 1104,  27960, 5372,  30924, 51668,
                24688, 35316, 15656, 54520, 37108, 7992,  20680, 57320, 15576,
                61364, 11236, 24472, 43768, 5968,  49432, 17204, 23696, 54092,
                34896, 62472, 3696,  44800, 20044, 35536, 42756, 56584, 64428,
                51444, 24412, 58636, 8492,  26828, 59808, 18524, 44940, 12612,
                23100, 64440, 14960, 25756, 60888, 1072,  50388, 36456, 7684,
                63040, 14396, 45140, 35628, 61220, 29108, 48112, 12348, 54868,
                51052, 2788,  45704, 19140, 23700, 59140, 8124,  40848, 5464,
                54544, 63076, 34668, 22556, 59176, 63512, 20236, 37712, 25424,
                48772, 32088, 44868, 58196, 21528, 47108, 12116, 40828, 56044,
                824,   39016, 62244, 3948,  13344, 58016, 9788,  46236, 25192,
                41676, 2708,  56216, 44016, 16876, 26080, 45344, 11472, 23708,
                39284, 16572, 29012, 20344, 61504, 5340,  23780, 52724, 11824,
                18476, 50520,
            },
            {
                13528, 44568, 38780, 58668, 11808, 51460, 55076, 1560,  6220,
                22836, 29520, 39804, 8868,  63956, 58156, 40364, 20368, 14096,
                46460, 28884, 41012, 5868,  47188, 52696, 33148, 3384,  40880,
                26792, 4812,  38468, 20820, 53760, 9592,  36556, 28148, 42836,
                10408, 15708, 27332, 57228, 13136, 48936, 16620, 9748,  40700,
                36232, 29316, 48584, 43432, 11740, 21420, 40144, 7388,  51708,
                57776, 37996, 8944,  53284, 6312,  39492, 12076, 55628, 19000,
                32740, 368,   22592, 10860, 55364, 8560,  18492, 36660, 42544,
                30060, 62060, 9300,  36792, 43172, 27996, 63844, 21764, 46852,
                30608, 38660, 7428,  16220, 28316, 12408, 44200, 54484, 7616,
                60548, 14608, 52292, 9188,  34884, 61980, 54388, 22980, 29800,
                9616,  25408, 49200, 31696, 41564, 52276, 34092, 16828, 30420,
                63620, 48288, 19276, 33136, 7296,  828,   35964, 60736, 14476,
                31636, 46884, 9668,  36752, 42716, 30632, 16040, 65404, 56696,
                37500, 24908,
            },
            {
                54812, 3400,  49460, 23356, 17440, 45844, 37560, 65152, 41748,
                48184, 11412, 52188, 32064, 25280, 17648, 43396, 53668, 60800,
                7840,  56428, 2332,  22448, 59048, 17112, 25304, 10204, 35656,
                52328, 47992, 58792, 34304, 15100, 40792, 61352, 2820,  59444,
                47820, 22504, 41416, 51604, 1000,  30492, 59836, 6352,  46112,
                18452, 468,   14856, 52632, 61996, 34908, 2168,  63156, 32276,
                17568, 27700, 49304, 56984, 18508, 43568, 65396, 24184, 47244,
                41192, 57024, 49412, 38844, 4160,  33112, 26676, 62916, 6504,
                21256, 33720, 13556, 55520, 49392, 1596,  14360, 33880, 58432,
                11436, 19360, 43124, 56116, 47316, 1752,  50632, 29812, 10616,
                36000, 19432, 4060,  41308, 16984, 26444, 2796,  37968, 59952,
                34308, 43908, 19084, 55200, 7760,  37840, 20532, 59888, 6748,
                13620, 35868, 10804, 39536, 65260, 54348, 28352, 41140, 56944,
                2052,  51908, 64308, 25556, 112,   50168, 34524, 27400, 6972,
                45948, 32692,
            },
            {
                60540, 10416, 27232, 33248, 62368, 6996,  31244, 26108, 16016,
                35876, 19228, 56000, 3796,  49532, 36648, 10164, 2736,  33476,
                38136, 19048, 63596, 30856, 43692, 12816, 60596, 45836, 63756,
                18664, 8612,  30008, 1688,  65340, 25144, 51500, 19212, 30468,
                55376, 4872,  33260, 37856, 61832, 20896, 52888, 32672, 26468,
                54716, 62968, 38764, 23460, 4848,  45720, 28812, 55332, 10992,
                42536, 1528,  34680, 22268, 31960, 3000,  28572, 16100, 9936,
                5468,  29740, 25616, 64600, 20424, 52804, 58904, 14916, 628,
                57464, 47372, 25656, 6712,  31116, 17980, 52060, 44392, 24668,
                480,   60564, 52552, 25784, 5976,  40760, 34360, 22028, 62600,
                42960, 56104, 28228, 64664, 49328, 7440,  45160, 50548, 16444,
                4340,  61464, 11820, 27992, 65088, 24344, 3260,  54068, 40660,
                50216, 27148, 58912, 23884, 15432, 48756, 10276, 19792, 5192,
                34392, 21972, 40708, 13248, 59876, 54656, 19820, 10152, 41720,
                1296,  21644,
            },
            {
                36104, 42520, 56652, 816,   40200, 20840, 10024, 53428, 58572,
                7968,  61720, 42768, 22500, 13412, 56804, 62444, 23360, 27100,
                50384, 11468, 48264, 26124, 34816, 4384,  38576, 29272, 996,
                22964, 42508, 54576, 12580, 44676, 32500, 5424,  8888,  39180,
                13236, 64320, 17728, 7716,  24772, 11712, 38940, 3912,  22320,
                12532, 33960, 7488,  31124, 57012, 16432, 49540, 25096, 20652,
                47196, 6704,  63576, 44796, 12880, 51936, 59352, 34068, 54032,
                37460, 59916, 17168, 13400, 46012, 40416, 24020, 49652, 43508,
                38384, 17276, 64832, 41184, 60432, 35556, 56900, 8856,  28896,
                49152, 35340, 13972, 31416, 65484, 18692, 57624, 15196, 3172,
                47448, 24156, 12732, 37388, 20328, 58940, 32644, 13268, 21344,
                52868, 46600, 36736, 1428,  15336, 48576, 44480, 10388, 32316,
                22160, 1412,  53208, 45812, 4144,  32008, 42708, 62512, 49832,
                26856, 54976, 7820,  18432, 45028, 4016,  39136, 58436, 47672,
                63084, 15880,
            },
            {
                7880,  19548, 47228, 14936, 50572, 60768, 43072, 23536, 46352,
                1164,  27492, 33896, 6136,  47204, 29940, 16084, 45020, 5736,
                58000, 15108, 40160, 8420,  51140, 55412, 21552, 48680, 56876,
                32656, 16336, 60296, 27392, 50096, 21756, 57512, 46888, 35068,
                26052, 53076, 46060, 29368, 56712, 43548, 47716, 65436, 57636,
                41364, 49644, 60364, 19672, 42032, 13064, 37156, 3404,  61404,
                39240, 58136, 15724, 26216, 38448, 48036, 7916,  41864, 21444,
                50580, 1764,  43636, 7984,  34704, 2352,  11180, 32156, 27680,
                9564,  53224, 22800, 3356,  11256, 20504, 39724, 4712,  16660,
                62332, 41580, 3620,  23080, 37396, 8664,  53212, 27108, 39548,
                6420,  33244, 54036, 60,    30296, 10736, 42436, 57220, 28312,
                8476,  23956, 57584, 42476, 33392, 58836, 28672, 62580, 18296,
                56424, 43224, 8608,  29924, 60532, 21504, 36600, 7516,  16916,
                45856, 59348, 37952, 32912, 48976, 29588, 14772, 23388, 31140,
                52072, 26560,
            },
            {
                53896, 30284, 64704, 5012,  28576, 35052, 3256,  13764, 30448,
                38312, 51576, 16832, 64380, 39648, 508,   35496, 51720, 41344,
                31584, 21732, 61012, 668,   18448, 64720, 15420, 6500,  11728,
                50864, 39968, 3664,  37012, 17492, 10628, 41520, 62016, 15352,
                192,   42244, 10772, 49952, 2652,  34616, 14524, 8228,  17524,
                29080, 3148,  10384, 47024, 26804, 63996, 8272,  54224, 33208,
                12084, 29252, 53044, 9364,  19688, 61856, 4552,  25172, 13144,
                63764, 31804, 27236, 55256, 61728, 47708, 17912, 63992, 56048,
                4320,  34736, 29892, 51272, 46692, 26636, 63636, 54412, 37456,
                21584, 10012, 51416, 45196, 59572, 12788, 48616, 31900, 59200,
                18020, 63304, 50292, 44228, 61068, 25140, 5208,  36228, 64244,
                40412, 31308, 6040,  18084, 22236, 12376, 39640, 4780,  36756,
                14320, 64184, 38324, 17712, 12504, 52396, 57404, 25040, 13404,
                30692, 2452,  11076, 63492, 24864, 6520,  61080, 35444, 12176,
                3624,  40228,
            },
            {
                44256, 12608, 23860, 37600, 53164, 18612, 56964, 49256, 62820,
                19968, 11940, 45212, 57600, 26192, 53588, 20200, 9624,  65412,
                3228,  46744, 36744, 53604, 32928, 41300, 27720, 43540, 35908,
                63104, 24704, 7884,  47468, 63704, 4460,  29584, 23476, 55884,
                20020, 59784, 36784, 22720, 62564, 19596, 27200, 51556, 36392,
                45108, 25016, 53864, 35456, 944,   50608, 30460, 44572, 16972,
                22428, 48980, 35324, 20,    55516, 30536, 36340, 56664, 45776,
                18388, 10456, 48988, 37084, 22824, 29920, 6928,  51192, 20976,
                45408, 12736, 59244, 15912, 43116, 1952,  30900, 13432, 47696,
                32520, 57116, 17948, 28652, 5060,  42452, 24604, 2268,  45612,
                10876, 22412, 8112,  15216, 38540, 48132, 17460, 51124, 1148,
                14812, 47684, 62188, 54880, 50264, 2232,  53028, 25588, 47408,
                6648,  24248, 33988, 49008, 27512, 516,   44592, 39340, 64488,
                52904, 42888, 21632, 15840, 51848, 42252, 55268, 8572,  49224,
                18196, 57988,
            },
            {
                1988,  34024, 58880, 7596,  45148, 11420, 27040, 5788,  40784,
                54752, 8928,  32172, 21932, 7692,  12684, 59872, 27940, 17180,
                54648, 24760, 29036, 13240, 45596, 9580,  23572, 58988, 2340,
                14300, 29880, 55704, 20804, 34116, 54196, 38096, 50512, 7448,
                32760, 27944, 4416,  16144, 40244, 58224, 32028, 728,   55920,
                62340, 15312, 21388, 58860, 40288, 18496, 23928, 59560, 6080,
                41584, 4152,  60680, 46208, 23120, 43000, 16332, 28200, 3588,
                39296, 58104, 5912,  19800, 14076, 41788, 57748, 37452, 15288,
                40720, 62612, 7532,  36828, 55908, 23492, 50284, 6028,  43960,
                25704, 2380,  64112, 34732, 54992, 20568, 61544, 36148, 52240,
                29440, 41608, 35356, 27412, 2984,  56312, 23188, 33584, 54244,
                19932, 26536, 37592, 9776,  44968, 35396, 61552, 19836, 31340,
                51616, 59904, 3276,  41640, 62388, 32688, 6240,  19108, 34928,
                4244,  28388, 56568, 36424, 1564,  27376, 20352, 44852, 64436,
                38108, 21984,
            },
            {
                61712, 47796, 15444, 39780, 21084, 60080, 32808, 16096, 35636,
                24640, 59272, 3124,  37264, 49500, 42020, 33068, 47792, 38348,
                7180,  10840, 58316, 19976, 5188,  61496, 31148, 47772, 53392,
                19564, 45868, 59760, 42944, 12036, 25840, 2300,  44832, 12876,
                64876, 43140, 51064, 54500, 6808,  45460, 10504, 23756, 42280,
                5336,  39192, 32988, 6604,  13988, 56296, 10744, 38076, 51548,
                64344, 27192, 17520, 39408, 12704, 64896, 9812,  52700, 62116,
                32852, 24720, 43148, 60276, 52924, 3848,  25452, 33272, 2660,
                23588, 48812, 26860, 19544, 32968, 9760,  58484, 18832, 61520,
                8304,  40516, 14908, 49528, 10584, 39404, 17004, 7568,  14056,
                56544, 60584, 19344, 52900, 65256, 12304, 45700, 8840,  42996,
                59752, 12664, 3596,  29764, 23792, 15736, 8136,  57572, 43652,
                11152, 16388, 54356, 21140, 10656, 55936, 50892, 23088, 11536,
                58732, 48816, 9004,  46108, 62616, 39416, 4484,  15136, 32864,
                28500, 6464,
            },
            {
                51096, 25892, 31376, 55084, 392,   48148, 42600, 64984, 1800,
                46648, 29164, 43808, 63508, 15436, 56132, 1924,  22880, 43980,
                62424, 34000, 41052, 50024, 37604, 55928, 16736, 7612,  38692,
                33620, 9984,  604,   28216, 51000, 15900, 61236, 18396, 48256,
                35612, 9844,  24652, 31160, 13176, 37228, 64124, 49084, 20096,
                12104, 29492, 51820, 64784, 43364, 28120, 47760, 1940,  33812,
                13788, 55980, 31460, 7216,  50556, 33448, 1932,  47480, 21912,
                14744, 50948, 420,   34464, 11416, 63140, 45004, 9376,  60780,
                29704, 53464, 492,   39668, 65196, 14548, 41372, 36088, 29268,
                53908, 22272, 46008, 26608, 332,   30512, 63684, 44188, 26244,
                1368,  46972, 4784,  32292, 40332, 20644, 29124, 62572, 5588,
                31932, 49592, 41920, 63656, 56308, 32860, 40980, 26908, 984,
                38664, 29300, 45412, 7044,  26044, 14940, 46848, 61716, 41964,
                16524, 32232, 24132, 18760, 12764, 30776, 58496, 52536, 24064,
                55624, 11144,
            },
            {
                41288, 17972, 9508,  63164, 29212, 24800, 10136, 50744, 22420,
                13880, 52792, 18820, 5396,  24876, 30120, 61428, 4736,  14492,
                51492, 17800, 1672,  63524, 26800, 3552,  51864, 21744, 26464,
                62420, 41404, 17864, 37364, 5796,  58144, 39960, 30688, 21656,
                3416,  57768, 17604, 47376, 60108, 28136, 3884,  53204, 34220,
                60840, 46380, 8900,  23380, 3812,  32040, 62476, 20980, 25636,
                8956,  42784, 20232, 59008, 24708, 54952, 18824, 36636, 41100,
                7872,  56052, 30080, 16892, 39148, 28384, 49264, 55440, 18656,
                43824, 11736, 58044, 47088, 4964,  52144, 24656, 1172,  48640,
                11980, 38152, 59800, 33528, 57704, 53060, 22000, 50192, 33928,
                38364, 24008, 11696, 49628, 7064,  57760, 37000, 15792, 24508,
                39108, 58240, 21456, 17200, 6432,  47296, 13216, 65448, 49996,
                19036, 58380, 34380, 64364, 40376, 36512, 30028, 2764,  26948,
                38464, 5944,  52680, 59936, 34240, 50044, 9836,  41984, 840,
                46084, 36440,
            },
            {
                57676, 2668,  43968, 52076, 6076,  36040, 18360, 57724, 7276,
                38404, 61140, 9768,  35364, 40548, 52440, 19504, 37076, 55356,
                25740, 31400, 48108, 22584, 14136, 34868, 40552, 11516, 58212,
                4700,  49760, 55052, 65020, 22924, 33252, 9204,  55608, 27164,
                52124, 38628, 62828, 1056,  21904, 41556, 17968, 14228, 25856,
                1996,  56856, 17128, 49188, 37188, 54840, 15556, 45244, 53112,
                36772, 49812, 1068,  44916, 37948, 5884,  29332, 60884, 12108,
                26776, 45136, 64676, 48228, 23764, 7264,  21216, 14520, 38804,
                34312, 7060,  16756, 31368, 21480, 44428, 60632, 33972, 16320,
                63232, 4240,  19372, 7208,  13356, 42004, 4352,  9892,  58280,
                15768, 61604, 54364, 17732, 26516, 44104, 624,   52408, 47036,
                10660, 1756,  28512, 50996, 36768, 3296,  53988, 22196, 35564,
                4708,  13728, 23684, 48420, 1248,  18088, 59388, 8692,  54152,
                45280, 65216, 308,   41496, 6908,  22164, 26780, 17416, 62396,
                14044, 20696,
            },
            {
                65452, 33084, 23436, 38560, 14468, 60944, 40524, 30472, 54608,
                33292, 26624, 48364, 58456, 12196, 46500, 7824,  28088, 11176,
                60544, 6248,  42324, 9796,  29756, 44488, 64644, 47100, 32164,
                15316, 24288, 30312, 12300, 47916, 44068, 1168,  63276, 41668,
                6600,  14704, 33396, 43940, 8984,  56384, 30396, 59376, 44924,
                40368, 36064, 27744, 61672, 19760, 10212, 41124, 6536,  58724,
                16796, 63444, 11908, 27632, 61808, 14596, 49192, 43348, 58200,
                4644,  35644, 19940, 2684,  59192, 52416, 31764, 1416,  64564,
                50988, 25812, 61284, 37332, 28052, 12368, 7940,  55292, 27260,
                44844, 30864, 52336, 25036, 47908, 36888, 27884, 18696, 46184,
                6744,  30200, 42344, 35652, 63776, 9548,  30780, 61064, 18596,
                35348, 55056, 14156, 43884, 61444, 25296, 30720, 10156, 46080,
                60488, 53124, 8852,  31464, 57104, 43096, 22432, 49880, 13588,
                20640, 35752, 26116, 15752, 55968, 37372, 47964, 59324, 34852,
                29460, 7808,
            },
            {
                27120, 47148, 12236, 59104, 20456, 49216, 3960,  45780, 16576,
                3008,  42796, 21792, 68,    16988, 32488, 64916, 49860, 39516,
                45636, 20728, 57876, 53448, 60100, 260,   19044, 6668,  54024,
                42884, 2712,  35776, 7548,  19660, 53040, 15276, 24336, 11792,
                46388, 20028, 55016, 25604, 50792, 35480, 5648,  48436, 9960,
                22344, 6968,  13320, 42812, 1152,  26360, 60464, 30616, 2936,
                22588, 29472, 34516, 8392,  52620, 21624, 3532,  32000, 23160,
                16460, 53820, 9100,  40752, 12560, 43260, 36716, 57264, 4768,
                22652, 42360, 56064, 3072,  53004, 48952, 39008, 18720, 2860,
                56996, 9532,  41452, 65120, 16556, 60128, 53868, 31708, 64392,
                39940, 21500, 2416,  47612, 14556, 55584, 22200, 40180, 4944,
                26016, 64868, 33016, 8336,  19704, 39372, 57428, 17392, 42472,
                27268, 39856, 20188, 51204, 12600, 5724,  63772, 34452, 30800,
                58036, 10972, 51104, 29816, 62780, 44128, 2456,  6188,  54220,
                39712, 49244,
            },
            {
                56196, 16352, 5048,  54140, 34100, 27792, 10692, 25476, 62308,
                12780, 51396, 63928, 28864, 54452, 43556, 21368, 1572,  33484,
                16292, 3720,  35288, 38436, 16940, 49576, 24984, 28632, 37992,
                20560, 62048, 46056, 57340, 39504, 28852, 36416, 49572, 31872,
                60012, 37128, 29256, 4348,  12676, 64640, 38828, 19604, 33152,
                63604, 49908, 54512, 32596, 51628, 46868, 12640, 35108, 39908,
                48856, 57108, 42128, 47412, 17296, 40064, 55568, 10196, 63316,
                38592, 47272, 29304, 33172, 62592, 26140, 17908, 47516, 28940,
                13160, 46532, 19756, 9560,  15448, 64180, 32096, 43220, 23656,
                36540, 49836, 21036, 35016, 3248,  11168, 23244, 896,   50696,
                13484, 26956, 56880, 5948,  25128, 33608, 50568, 12456, 57912,
                48080, 41804, 22844, 53396, 12080, 49064, 236,   63176, 7496,
                33048, 2260,  62784, 36708, 28164, 47388, 25220, 16784, 6780,
                42464, 3896,  46776, 19296, 8212,  13476, 20964, 31828, 23844,
                11616, 3644,
            },
            {
                35748, 41044, 30920, 45024, 1048,  64544, 41768, 56828, 37204,
                19728, 8408,  39332, 34932, 4664,  57616, 14064, 24380, 61700,
                55644, 28456, 12556, 23896, 7632,  33132, 56636, 13000, 59844,
                9220,  51284, 26716, 13968, 63672, 3776,  58780, 18016, 53800,
                2388,  9732,  62068, 46896, 16728, 23452, 53704, 348,   14768,
                57492, 29636, 4280,  21288, 38328, 65344, 23920, 55632, 19516,
                7800,  14232, 4824,  24312, 64952, 35824, 26872, 45644, 732,
                51632, 25124, 57692, 15208, 54844, 5568,  10232, 60576, 40496,
                7780,  32444, 62356, 35632, 41548, 22016, 6472,  59380, 11860,
                62436, 14800, 5580,  29356, 47032, 38988, 42988, 56040, 34924,
                9148,  44408, 62232, 52452, 19316, 38168, 3164,  44656, 28916,
                16448, 7048,  2516,  59896, 44920, 27852, 33940, 51988, 14252,
                23972, 54984, 44296, 15704, 4124,  56372, 41144, 37748, 54584,
                63052, 23028, 38916, 60196, 33504, 50268, 40852, 64088, 45448,
                51524, 19120,
            },
            {
                58692, 8808,  62540, 24552, 17808, 51968, 7036,  22084, 48164,
                31716, 59408, 24096, 46296, 10700, 27020, 40392, 52684, 9476,
                47500, 43340, 64356, 52348, 45952, 62316, 40048, 3648,  48676,
                34748, 17372, 1388,  32976, 23592, 10436, 42752, 5824,  27560,
                44236, 22672, 39872, 57652, 32104, 43012, 27648, 61096, 41272,
                25252, 45772, 16776, 56316, 9152,  5256,  17364, 44072, 50824,
                26640, 59708, 33140, 54108, 1808,  12452, 60116, 18956, 34268,
                13044, 6884,  21316, 3240,  45112, 49604, 23400, 35112, 54284,
                16516, 50408, 2144,  24516, 54104, 29976, 128,   51760, 26728,
                45940, 32544, 54464, 57960, 25592, 62184, 7948,  15980, 20256,
                59320, 37212, 30936, 11548, 41592, 59228, 8168,  63092, 20888,
                37244, 50788, 30152, 35764, 15728, 5312,  21680, 37672, 46700,
                58652, 18728, 10264, 31044, 60136, 21128, 11596, 1616,  49040,
                27840, 14924, 53288, 1292,  24896, 55144, 28240, 9456,  15476,
                61132, 25972,
            },
            {
                44288, 21532, 50992, 11752, 37400, 29972, 13952, 34520, 2012,
                54028, 5356,  15172, 50468, 18952, 63012, 31060, 3200,  35708,
                19404, 6480,  31612, 1836,  21156, 10508, 15644, 43620, 30744,
                22404, 56360, 41276, 45076, 54748, 48628, 21432, 34948, 65456,
                14316, 51468, 19076, 1584,  49828, 10872, 7192,  51040, 37140,
                3252,  11252, 35584, 42124, 28292, 58640, 31664, 248,   10988,
                63744, 37556, 44348, 20212, 30508, 50872, 6280,  41516, 28848,
                61020, 42892, 65460, 36332, 39400, 19576, 30972, 572,   64216,
                26712, 38304, 59068, 45216, 12836, 58140, 47172, 37716, 16740,
                7356,  40156, 1536,  18404, 12772, 51688, 32716, 28736, 48620,
                5452,  23520, 17144, 1792,  46628, 27432, 14984, 32512, 53676,
                10552, 61824, 24568, 57172, 41072, 63920, 54572, 11052, 29696,
                4624,  49872, 25736, 38444, 46028, 52784, 33368, 60992, 8976,
                18676, 32184, 43716, 11948, 35904, 17052, 58324, 3040,  37532,
                33344, 208,
            },
            {
                29420, 38900, 5384,  48224, 60260, 42980, 55600, 61680, 16992,
                40256, 44704, 30132, 60672, 37640, 7012,  48488, 42512, 58696,
                26272, 14408, 39264, 58320, 27768, 36580, 55460, 25728, 64820,
                5252,  11784, 61568, 7228,  30124, 15824, 61208, 39376, 8092,
                56368, 33860, 26380, 6404,  36124, 59768, 21600, 30956, 18488,
                47620, 59412, 23496, 62868, 14352, 48736, 36676, 53096, 41160,
                29712, 3188,  16592, 9480,  46668, 38512, 22544, 56776, 47936,
                16864, 31784, 50308, 10896, 27972, 59732, 52084, 13664, 44088,
                21376, 11064, 5144,  28508, 18472, 34500, 9388,  20476, 61104,
                53008, 22904, 63976, 43948, 36180, 21924, 2312,  45804, 64068,
                41032, 50472, 55284, 34204, 65168, 49340, 23820, 56444, 476,
                39716, 45888, 13288, 19364, 8864,  47932, 26136, 17860, 61752,
                41712, 34820, 64964, 844,   7732,  17580, 29276, 24124, 45016,
                36804, 58984, 7080,  64604, 46180, 4652,  48144, 22956, 42568,
                54072, 13888,
            },
            {
                64652, 55532, 15756, 27444, 1748,  23348, 9012,  45964, 28212,
                25120, 10076, 56416, 968,   23196, 55180, 16656, 12004, 22048,
                51120, 61448, 48636, 17252, 42080, 51452, 2564,  18948, 46544,
                52796, 38524, 28000, 19300, 37008, 52168, 832,   25020, 30732,
                47856, 11768, 42376, 63800, 15688, 45388, 55896, 13308, 64928,
                8380,  52400, 32460, 1768,  44808, 19964, 25392, 61984, 15340,
                21852, 48000, 55192, 61652, 26048, 58728, 15228, 2880,  53052,
                8844,  23036, 1720,  56024, 16000, 7480,  41672, 3996,  57224,
                48128, 31840, 52380, 40924, 65268, 4388,  50124, 42308, 28248,
                35292, 10728, 30552, 48840, 6688,  60628, 56600, 11684, 26260,
                3860,  14364, 29148, 9568,  19868, 6268,  36128, 43268, 17584,
                26356, 4908,  34084, 52676, 1240,  31572, 39116, 2904,  44820,
                8428,  20460, 13716, 56676, 50984, 42372, 62632, 13028, 55344,
                4112,  50552, 25696, 29796, 20320, 38552, 62364, 31000, 18780,
                49816, 7560,
            },
            {
                46204, 19308, 35408, 58392, 32564, 19828, 38452, 3680,  50784,
                64400, 21044, 35992, 13464, 43664, 33460, 28084, 65112, 37944,
                432,   33856, 9724,  24480, 6072,  32044, 60472, 8836,  35416,
                14804, 24244, 48024, 59224, 4328,  12328, 57752, 45324, 17072,
                4404,  60572, 53716, 29340, 39188, 24216, 33672, 4996,  43128,
                28516, 20656, 40128, 6448,  54116, 11984, 7460,  33712, 4120,
                58412, 25004, 34672, 6720,  42160, 32588, 10796, 36436, 26988,
                40196, 58620, 35300, 25828, 46596, 63064, 33748, 23892, 36928,
                17868, 7988,  60816, 15356, 22540, 54724, 25176, 13224, 5820,
                45524, 55512, 14992, 25968, 41572, 17160, 39272, 34536, 19488,
                61260, 38296, 57952, 45252, 40224, 59976, 12808, 51980, 30332,
                58664, 48524, 64708, 42740, 23516, 56572, 60428, 14740, 51328,
                55648, 32648, 40132, 27724, 22712, 35216, 2628,  48384, 39772,
                21336, 14416, 42224, 56300, 8924,  52816, 13396, 5744,  57140,
                10832, 25080,
            },
            {
                31704, 2732,  52164, 10996, 46952, 63132, 52844, 12432, 33328,
                6628,  41588, 49484, 62296, 3952,  52832, 8848,  46312, 5592,
                55940, 28832, 44584, 54360, 62856, 13648, 49220, 29392, 40748,
                384,   63320, 9372,  43404, 34244, 26644, 40492, 50240, 20432,
                37520, 23508, 2964,  19536, 8488,  48568, 1032,  53872, 38412,
                57416, 14644, 49712, 26888, 60388, 38212, 56952, 42748, 50360,
                10040, 38308, 13520, 52064, 596,   63644, 19692, 46232, 62012,
                4040,  18536, 44372, 12752, 53748, 19196, 28732, 9744,  62100,
                54600, 25644, 35100, 44344, 1728,  38748, 32848, 57660, 63516,
                1160,  19772, 59856, 3504,  52376, 9396,  29776, 54212, 44356,
                7784,  51636, 25168, 1156,  31796, 22632, 3980,  63356, 9924,
                21360, 7688,  16304, 28156, 37068, 10780, 20800, 35260, 28664,
                24308, 3572,  60224, 45268, 10592, 15796, 57356, 6684,  27416,
                34256, 60892, 436,   16712, 34688, 23948, 44220, 27220, 61368,
                36956, 41328,
            },
            {
                60304, 23156, 43184, 7052,  40472, 15080, 29692, 59660, 18864,
                57468, 15664, 31640, 26184, 19312, 39452, 59500, 24076, 18108,
                40832, 13196, 20204, 3800,  34968, 22872, 43868, 57992, 20492,
                55576, 32440, 16872, 54380, 20920, 62300, 14600, 8640,  64296,
                32436, 51748, 46372, 13924, 58996, 62888, 27180, 16980, 23040,
                34512, 2836,  63248, 17652, 31052, 47092, 22952, 28704, 18984,
                45340, 64804, 30324, 17816, 23532, 49720, 28624, 54272, 13272,
                33028, 49300, 64160, 5776,  38184, 1204,  49092, 43624, 14376,
                40100, 6460,  47524, 11996, 29140, 61536, 48608, 16316, 30916,
                39900, 50592, 37544, 33444, 64936, 23180, 48028, 660,   15480,
                22180, 35788, 11380, 17736, 55068, 50204, 27328, 46300, 38456,
                41648, 32572, 55152, 3792,  50812, 44208, 6768,  47280, 64372,
                11828, 17348, 49096, 5644,  63300, 38088, 31468, 19056, 65424,
                9536,  53272, 31596, 47984, 63348, 40312, 2532,  50940, 33700,
                15888, 5056,
            },
            {
                49052, 13252, 63904, 26668, 56820, 4492,  24372, 44048, 37512,
                46772, 2624,  54884, 8096,  47980, 29728, 14808, 35488, 51912,
                31288, 63540, 57868, 46844, 38936, 17048, 10868, 4312,  25912,
                45164, 6376,  49804, 38236, 2084,  31024, 55984, 5116,  27836,
                41524, 10712, 57168, 34764, 30628, 40632, 51344, 9280,  61256,
                47304, 11760, 43748, 36224, 9368,  1196,  15572, 62624, 6088,
                53712, 2400,  41060, 56816, 44736, 8460,  38872, 5584,  42968,
                24340, 9216,  29264, 41360, 21940, 31856, 59336, 51756, 4568,
                21092, 30752, 59588, 51612, 18992, 9696,  3340,  21604, 43712,
                24188, 9988,  27976, 6532,  43352, 13192, 57232, 26940, 63080,
                32632, 58832, 46512, 64188, 42352, 13884, 34580, 19156, 2060,
                52940, 60828, 22848, 14004, 62816, 17768, 57944, 1680,  40768,
                53084, 37088, 30268, 19952, 53552, 24848, 46196, 50896, 41688,
                23688, 45156, 11836, 27796, 6476,  58040, 19324, 9608,  46536,
                21428, 56112,
            },
            {
                38672, 17876, 30572, 36876, 20788, 49552, 34436, 320,   10272,
                27372, 23280, 60592, 12200, 44400, 64076, 4936,  57580, 10112,
                43228, 2188,  25440, 8596,  53196, 28060, 65512, 51028, 36548,
                61280, 13604, 29004, 22832, 11164, 47224, 43852, 36360, 59552,
                16124, 488,   25800, 44020, 5964,  21212, 12712, 44964, 31420,
                6244,  53228, 24948, 20336, 55456, 50980, 40468, 12296, 32072,
                35952, 26276, 21348, 11672, 60032, 35024, 15968, 56148, 21392,
                59704, 51200, 15068, 55700, 61344, 10724, 16904, 34412, 26880,
                64200, 484,   42176, 23624, 56688, 37160, 46136, 53088, 7188,
                57124, 15524, 46824, 58816, 17772, 31548, 37916, 5844,  49664,
                41612, 2576,  28220, 5456,  37316, 8660,  62288, 55864, 15272,
                25992, 11688, 45272, 34716, 39992, 30212, 25668, 33080, 21708,
                7620,  43528, 57080, 52,    41316, 8620,  13280, 58804, 2236,
                17268, 37408, 59468, 21840, 36344, 13984, 54772, 30668, 65156,
                1064,  28484,
            },
            {
                8348,  45088, 2172,  55028, 8820,  62092, 17432, 53960, 65516,
                50356, 40856, 34044, 37056, 21004, 788,   49892, 22188, 27308,
                47784, 16568, 37280, 32876, 59648, 1216,  40172, 31928, 8248,
                17964, 41744, 57648, 64620, 52040, 24900, 18240, 53504, 22156,
                48896, 62180, 38124, 17152, 55500, 3288,  36696, 58252, 19440,
                27888, 38796, 59056, 4264,  65136, 33904, 25628, 58304, 47824,
                60608, 16116, 50196, 5328,  31884, 24860, 1496,  65132, 30748,
                37876, 2288,  34700, 26324, 46988, 3728,  24024, 57792, 45860,
                37832, 55168, 16228, 33312, 5832,  64724, 26200, 33768, 12832,
                63124, 35504, 54268, 21328, 2832,  61976, 45676, 19720, 10716,
                56256, 14140, 20604, 53264, 24292, 29904, 4688,  48216, 31276,
                37100, 6356,  49104, 1276,  59684, 8832,  49940, 54924, 15504,
                61664, 26340, 14196, 33852, 59964, 28436, 35980, 4892,  30440,
                56008, 7708,  51288, 1724,  43932, 48880, 25788, 37760, 42492,
                11812, 53384,
            },
            {
                24972, 59940, 51528, 32516, 12380, 45828, 39592, 29240, 14132,
                19716, 6400,  52988, 16236, 56604, 41876, 33000, 38700, 7128,
                61932, 52548, 12180, 22260, 48844, 15024, 19936, 55060, 23744,
                48424, 1488,  34336, 4576,  39616, 8040,  33552, 2376,  12884,
                30272, 7444,  52456, 23128, 65240, 48428, 25496, 41176, 63360,
                72,    50112, 13868, 42132, 28992, 7116,  44492, 19908, 3972,
                9604,  42940, 37612, 63000, 46644, 52712, 40780, 48188, 11496,
                17696, 45144, 7436,  19768, 52192, 36776, 42576, 6856,  12356,
                19480, 9076,  50344, 13560, 28380, 40532, 17572, 59548, 42156,
                29984, 264,   25620, 51144, 41256, 8996,  34872, 53736, 25456,
                30896, 39692, 47444, 61660, 16948, 44748, 58444, 20988, 43056,
                57132, 65308, 29184, 20436, 24252, 12652, 42180, 4952,  35588,
                45680, 10200, 50828, 22340, 48020, 16652, 63700, 21276, 47592,
                40060, 25248, 64172, 18308, 32752, 60000, 4032,  16492, 23336,
                58416, 34860,
            },
            {
                40204, 5296,  15544, 22908, 38040, 26204, 5792,  58828, 42992,
                32676, 3352,  62104, 28876, 10448, 25956, 60980, 13580, 55000,
                19556, 28928, 4972,  63908, 42056, 6916,  46092, 58624, 11240,
                28160, 53336, 20132, 45624, 15868, 59112, 62800, 27316, 42732,
                58092, 33900, 45748, 9884,  29372, 33300, 15012, 6988,  54764,
                17620, 32964, 22696, 47732, 10904, 17120, 38316, 51972, 24112,
                29908, 54192, 1912,  27828, 14040, 9248,  18792, 26808, 4648,
                53844, 58388, 63564, 39932, 13548, 62384, 28432, 53828, 48780,
                30012, 61484, 35704, 45228, 1976,  48448, 10664, 3772,  20744,
                49068, 5880,  38876, 12184, 29640, 22852, 59924, 4308,  16792,
                65220, 7332,  34644, 10236, 50624, 38420, 12460, 76,    23912,
                9996,  17556, 52012, 38648, 56512, 46316, 63932, 18840, 28336,
                59032, 2444,  63120, 39048, 6528,  55124, 43100, 52156, 10144,
                14684, 34740, 42744, 12160, 29176, 9048,  52528, 62884, 6208,
                49588, 18912,
            },
            {
                64064, 48180, 42680, 61348, 57164, 1492,  47816, 21968, 9420,
                56236, 48708, 22536, 45588, 6828,  52224, 17536, 2948,  42724,
                34576, 45692, 39124, 55780, 30340, 24744, 34160, 3108,  43520,
                63720, 37288, 12656, 25860, 30532, 49524, 10932, 37820, 47344,
                4076,  20396, 14072, 40152, 1460,  57640, 51540, 11104, 35512,
                45980, 8792,  61200, 36924, 54416, 62064, 760,   57196, 12868,
                64472, 34284, 19612, 57316, 22656, 59400, 35808, 61736, 42316,
                33488, 28520, 23604, 9908,  31776, 964,   15624, 21688, 39656,
                2740,  25264, 58268, 21204, 54424, 62236, 23836, 51944, 37904,
                56420, 18440, 45560, 64608, 55380, 14824, 48044, 38656, 51872,
                44280, 23220, 59668, 2812,  32348, 26512, 60796, 53080, 40460,
                32736, 3480,  44044, 7660,  15200, 31200, 3224,  37384, 52736,
                23464, 32456, 18436, 27600, 12364, 1876,  24436, 33196, 61756,
                27848, 58108, 4476,  53816, 46556, 38440, 21556, 34060, 44956,
                29748, 13824,
            },
            {
                33128, 27276, 10336, 19132, 29944, 53244, 16520, 63008, 36480,
                25676, 39120, 13080, 31788, 58628, 37548, 47184, 30620, 23856,
                57740, 10288, 696,   18208, 50000, 13024, 62168, 38564, 16368,
                31496, 5808,  60860, 55744, 41504, 5416,  19740, 56780, 16484,
                64048, 26376, 54912, 61388, 18008, 44504, 20884, 42168, 24052,
                29432, 57256, 2492,  26824, 20956, 30392, 45840, 33388, 41284,
                16808, 46332, 7484,  39300, 43480, 31372, 6100,  50236, 22176,
                12976, 1856,  48496, 43900, 56536, 50004, 59584, 33984, 65428,
                43708, 5240,  17352, 38460, 8204,  29388, 32684, 44824, 13256,
                27244, 59168, 32228, 7144,  43672, 1780,  27424, 32580, 12476,
                592,   28656, 53612, 19252, 42100, 5708,  15956, 35476, 49544,
                62804, 27508, 59900, 35096, 53656, 26068, 58508, 11348, 40648,
                5812,  49288, 42760, 56280, 46816, 30540, 38388, 7416,  17880,
                532,   49056, 20220, 24048, 62212, 15396, 2724,  41224, 10688,
                57000, 80,
            },
            {
                21888, 4020,  50956, 36032, 7720,  41652, 34172, 12028, 51356,
                1920,  17952, 63488, 43156, 1340,  20076, 65100, 8800,  50564,
                62640, 15132, 26296, 59788, 35960, 8724,  52876, 26672, 20608,
                50900, 9552,  47680, 22296, 12,    35848, 53700, 24200, 8556,
                40972, 35160, 48748, 5288,  37176, 27484, 59720, 3084,  63872,
                52428, 15400, 40008, 5556,  51276, 14632, 8052,  22436, 27188,
                4732,  60192, 50640, 11928, 53252, 16,    46064, 15428, 56100,
                39456, 52592, 17260, 35988, 5484,  24872, 18556, 7920,  11440,
                55508, 31396, 51148, 11976, 43180, 58868, 16172, 6692,  62740,
                35184, 9320,  15960, 24500, 36036, 19096, 57344, 63384, 21060,
                49168, 36588, 14460, 57568, 46328, 64540, 29572, 8268,  19464,
                14092, 47320, 10924, 20024, 992,   49608, 22216, 44384, 16636,
                57044, 14068, 8540,  35792, 64628, 20060, 59712, 44948, 54640,
                41528, 37032, 30992, 6388,  56732, 27380, 50648, 61224, 25460,
                53188, 37596,
            },
            {
                61836, 56324, 44364, 14188, 64944, 24164, 3652,  44784, 30564,
                59996, 8164,  27604, 50664, 35180, 24924, 12668, 40320, 4528,
                20704, 32944, 53560, 43816, 22056, 2348,  46916, 56432, 3868,
                59640, 39576, 33492, 14552, 65172, 43680, 28492, 32612, 51420,
                2112,  12024, 30372, 23272, 9236,  49868, 31504, 6044,  37756,
                12124, 19764, 49440, 43076, 64276, 35412, 59268, 55100, 49204,
                10820, 36516, 29308, 24696, 17336, 64588, 26092, 8812,  30056,
                62376, 6552,  21296, 64052, 30464, 41172, 46044, 37268, 27036,
                47640, 22772, 63424, 34792, 25540, 1344,  40268, 55232, 22572,
                3568,  52908, 46956, 60716, 50844, 40252, 5104,  9856,  42868,
                55952, 7900,  40112, 25724, 11336, 21748, 54688, 44152, 58216,
                4916,  23500, 37628, 41968, 64480, 32908, 9108,  62664, 34556,
                29384, 61276, 24992, 3744,  52512, 14968, 10516, 50492, 23340,
                13584, 64968, 10916, 45072, 40076, 12756, 32060, 19620, 7520,
                17244, 46832,
            },
            {
                39652, 6936,  31208, 20828, 58560, 49136, 28204, 55368, 20444,
                40816, 46696, 54768, 15360, 5164,  57176, 52896, 28012, 36764,
                47900, 41240, 5932,  28816, 64972, 17028, 40652, 29844, 13960,
                24168, 45224, 27456, 17672, 50372, 7604,  12952, 62292, 45104,
                22068, 60444, 42592, 53360, 64612, 13516, 16900, 56012, 45124,
                33936, 60676, 25012, 31648, 11220, 18744, 39360, 1852,  43780,
                63668, 21292, 2596,  56772, 33212, 37964, 59040, 19988, 36476,
                44360, 11304, 26488, 54440, 13800, 3120,  57852, 52744, 15164,
                172,   40724, 6328,  14392, 52824, 47028, 19292, 36608, 50084,
                30264, 41912, 20708, 2804,  11900, 30740, 46036, 25404, 33952,
                17720, 62128, 30768, 3876,  51792, 37248, 1892,  25136, 39840,
                30500, 51032, 55480, 16328, 28984, 46504, 4588,  19152, 51724,
                464,   46120, 21412, 39328, 32576, 26628, 4832,  35056, 29404,
                3420,  58872, 51900, 18592, 35200, 1668,  48120, 59116, 43092,
                28936, 12052,
            },
            {
                24528, 16356, 52544, 1180,  37040, 10604, 17716, 62480, 14412,
                4748,  33832, 22256, 10520, 42000, 31484, 44764, 16944, 60756,
                1604,  56960, 13644, 49004, 37392, 11140, 33412, 61724, 42984,
                8160,  53732, 2800,  57136, 38244, 59604, 19016, 3268,  38836,
                15184, 55484, 18548, 440,   32996, 46132, 39860, 22640, 27084,
                868,   47544, 7892,  57880, 4336,  46488, 25552, 13600, 30880,
                17996, 53684, 42644, 47488, 13736, 3472,  41824, 49584, 4360,
                58188, 32856, 46836, 38584, 60524, 9792,  22368, 32708, 61640,
                19776, 56868, 45096, 28260, 59972, 4292,  65016, 10048, 26004,
                12696, 63696, 37676, 26960, 54372, 62832, 15716, 59208, 52080,
                4572,  22688, 47692, 58556, 17012, 32284, 48484, 12956, 61924,
                34128, 2524,  60712, 6032,  12848, 54496, 40260, 26868, 38156,
                12044, 54564, 42292, 17056, 58124, 48876, 62052, 40024, 55840,
                21464, 33456, 25848, 8624,  63096, 54164, 23264, 36932, 4804,
                64548, 34216,
            },
            {
                48344, 60408, 27568, 40560, 46540, 33108, 6372,  38484, 52636,
                24808, 58236, 37828, 64036, 18888, 59692, 7380,  22928, 10752,
                30952, 24792, 18964, 8008,  58148, 25512, 50340, 5096,  18628,
                36844, 63628, 11520, 30760, 23808, 46660, 34228, 25632, 57836,
                29464, 46812, 5984,  38092, 24816, 51472, 7364,  62096, 10404,
                54248, 17116, 38376, 22476, 53720, 29068, 62644, 50456, 6940,
                33992, 39132, 9124,  62272, 23116, 52688, 10792, 28272, 17792,
                23960, 51180, 656,   16700, 28404, 49684, 43552, 6852,  29408,
                48972, 35436, 9232,  17232, 33452, 21880, 31324, 44108, 56936,
                500,   17888, 58288, 8296,  34620, 21472, 6412,  38568, 29216,
                12920, 41784, 34984, 10120, 43412, 63280, 6880,  56244, 17804,
                10244, 45000, 21052, 25532, 35688, 61564, 22968, 48372, 59108,
                6872,  30912, 63204, 9360,  1516,  43600, 18844, 8048,  47000,
                16136, 43144, 48356, 4260,  29352, 42272, 15592, 9860,  55260,
                50888, 2164,
            },
            {
                43544, 8672,  54664, 13128, 63504, 21812, 57936, 43888, 9612,
                31008, 2980,  47540, 28488, 64,    48884, 36072, 54860, 64208,
                46364, 52372, 35012, 62732, 44616, 328,   55108, 22936, 57736,
                32324, 48376, 21080, 42320, 4720,  9752,  54064, 49856, 6796,
                10464, 35696, 26952, 62932, 11448, 56756, 19500, 35532, 42044,
                29600, 65388, 33020, 12800, 41660, 2848,  36428, 56988, 23440,
                60940, 14956, 27584, 5248,  31176, 44888, 60620, 34248, 55348,
                14432, 64824, 8256,  41896, 20196, 34940, 63004, 12588, 40104,
                3760,  24904, 62648, 38816, 55784, 49548, 6836,  15564, 39324,
                47896, 32320, 44544, 14324, 49212, 42584, 24072, 47256, 2512,
                54972, 64792, 26348, 1092,  20776, 28320, 40476, 23008, 52432,
                27932, 38520, 48824, 58356, 43436, 9672,  2716,  14892, 33280,
                18336, 25984, 50212, 36884, 13312, 29072, 24332, 57372, 11852,
                936,   37292, 60720, 13740, 56932, 20464, 46652, 26688, 31668,
                14236, 22468,
            },
            {
                37940, 30860, 18444, 5152,  25324, 51036, 2396,  27128, 49820,
                61904, 12428, 16668, 53540, 40440, 25876, 13700, 3488,  38996,
                15748, 42604, 2912,  29536, 20864, 38912, 15652, 46444, 28132,
                13040, 1652,  52472, 35524, 63048, 16508, 30240, 40608, 64728,
                20140, 48604, 59196, 15776, 40844, 30052, 2428,  58760, 14776,
                50160, 6256,  44636, 51432, 60468, 15816, 20312, 10364, 44804,
                512,   48060, 57668, 35928, 16164, 20596, 7580,  39472, 2588,
                43576, 36908, 30928, 59744, 54052, 2240,  25824, 55252, 18076,
                58944, 52356, 13440, 1568,  42140, 11432, 27336, 52220, 62024,
                23648, 5204,  53492, 28880, 1784,  56596, 11232, 61188, 35680,
                20112, 16080, 49784, 59804, 53152, 13840, 35760, 46548, 364,
                64772, 14212, 7544,  32196, 17692, 29608, 52580, 65392, 41544,
                55416, 3136,  22856, 47112, 53936, 64884, 38264, 31848, 52648,
                63556, 28448, 23056, 51252, 32508, 38852, 616,   59988, 41368,
                19004, 57060,
            },
            {
                11132, 61560, 35744, 45396, 56172, 32080, 15988, 36168, 41356,
                19164, 44696, 34796, 60480, 8952,  20260, 51436, 33516, 27688,
                21416, 9200,  60272, 12904, 53940, 32020, 10220, 64092, 6052,
                41456, 59672, 25356, 14172, 56080, 44548, 676,   23012, 13760,
                43648, 31908, 3024,  50652, 21332, 44912, 5272,  47648, 25400,
                21752, 1956,  18980, 26932, 8076,  47364, 31732, 40776, 55392,
                26088, 19444, 51884, 40876, 65208, 48724, 53860, 25804, 61732,
                21772, 10612, 47968, 24524, 6384,  15040, 46752, 37524, 9928,
                44000, 31768, 20756, 47728, 23984, 59620, 37152, 19616, 34276,
                8788,  60052, 20472, 40716, 65488, 18172, 31492, 50796, 7928,
                44976, 32372, 5836,  37848, 45508, 8576,  57384, 5080,  31780,
                19992, 41872, 54228, 3712,  49840, 37464, 20872, 45548, 8328,
                36276, 61244, 15424, 33764, 6236,  21264, 2692,  44244, 17724,
                6864,  40964, 19116, 9932,  5408,  65352, 49756, 7956,  35140,
                63068, 3820,
            },
            {
                25544, 49936, 356,   20524, 42040, 8152,  11864, 63192, 22120,
                6904,  56916, 23952, 4956,  32032, 43344, 56228, 61744, 6156,
                50144, 56720, 36936, 25880, 48368, 3956,  43524, 35304, 51060,
                17148, 38188, 8316,  47180, 27908, 19856, 58596, 36524, 52360,
                5024,  56136, 24080, 8804,  34208, 53016, 61828, 37552, 32204,
                55960, 62224, 34836, 54560, 37780, 24580, 64656, 5516,  35032,
                12844, 30176, 4148,  11000, 24712, 1040,  12916, 29980, 46568,
                5560,  57408, 18608, 51388, 35916, 29552, 64416, 21964, 50348,
                27772, 6064,  36188, 63900, 29508, 5508,  54668, 2336,  43384,
                12912, 26400, 50496, 35344, 4496,  25244, 39036, 14912, 27340,
                57612, 40564, 12088, 23752, 30236, 18796, 62284, 25868, 50660,
                59204, 36324, 23772, 63256, 11356, 56708, 1252,  24880, 12908,
                28412, 43792, 57716, 10644, 40696, 59628, 14876, 50560, 25312,
                34316, 58004, 54324, 44716, 36004, 17024, 24388, 12824, 52508,
                28620, 45820,
            },
            {
                40328, 53328, 13796, 65144, 28344, 59220, 47652, 54132, 948,
                30036, 51076, 38956, 14988, 65520, 47084, 11008, 17872, 40272,
                45632, 31568, 5196,  17384, 40572, 61500, 19384, 24444, 57324,
                29784, 20664, 61184, 32932, 5804,  39508, 11480, 61556, 26784,
                17412, 39028, 60368, 42936, 13524, 27532, 17632, 11640, 7664,
                43644, 13976, 40704, 11032, 57784, 1676,  49376, 17748, 52464,
                62228, 38396, 59948, 43376, 32344, 56408, 37680, 17532, 51844,
                33548, 27664, 39876, 12512, 44812, 56640, 4788,  33092, 736,
                61748, 16012, 53904, 8892,  18056, 45920, 14612, 64648, 30652,
                47300, 38400, 16224, 10756, 58512, 46640, 54080, 100,   63692,
                22644, 52264, 2076,  60788, 54404, 34076, 15984, 43872, 12264,
                8964,  29260, 16672, 44712, 26996, 40616, 60160, 32012, 48196,
                5324,  50960, 19208, 30824, 26532, 46220, 35712, 9016,  61316,
                47548, 13260, 1908,  26528, 59148, 30288, 42896, 55948, 21704,
                5860,  16120,
            },
            {
                30144, 22548, 7472,  33404, 38128, 17832, 23672, 39684, 34008,
                60164, 10292, 48248, 27832, 3380,  22332, 30280, 892,   25084,
                14060, 64752, 22664, 51296, 57644, 27988, 7860,  54240, 1004,
                11244, 49232, 42972, 2656,  54756, 50516, 31404, 7828,  45248,
                34640, 11080, 29448, 2000,  65024, 55288, 23164, 39680, 59236,
                28180, 48556, 3580,  30704, 21588, 14488, 28556, 42916, 8604,
                22252, 46244, 16684, 7712,  20068, 45484, 63552, 9276,  41816,
                14660, 62728, 4236,  60400, 8756,  17140, 41740, 59340, 13276,
                46368, 39260, 42528, 57260, 32584, 50540, 39984, 22524, 58120,
                3428,  54368, 62248, 31984, 7772,  42212, 19896, 36552, 9968,
                43700, 18064, 28892, 48324, 7280,  41492, 2636,  37736, 55776,
                47508, 1684,  60988, 6296,  34352, 14560, 18764, 53848, 63584,
                22088, 38688, 48,    62704, 52272, 4444,  55400, 28960, 20748,
                5696,  31212, 38992, 50260, 11780, 47040, 2720,  33324, 38556,
                48388, 58864,
            },
            {
                35464, 60876, 43668, 55616, 2688,  52232, 4920,  45832, 13772,
                26508, 17464, 55388, 35392, 42052, 57480, 37696, 51776, 59172,
                34772, 9660,  43660, 1364,  11600, 33980, 15008, 39228, 45132,
                65432, 34756, 15692, 23276, 63448, 18124, 24828, 48204, 1448,
                63592, 54264, 19484, 46908, 36644, 6456,  49120, 240,   52216,
                18576, 24028, 63972, 52740, 45052, 61356, 39100, 58644, 32992,
                1300,  27912, 50452, 34564, 58936, 27076, 4884,  22568, 55024,
                2056,  48580, 20908, 31544, 52800, 24068, 37872, 20084, 53300,
                26384, 22464, 3520,  12144, 25580, 1176,  35448, 9956,  27956,
                18116, 6360,  43976, 21876, 28548, 13876, 60360, 49472, 30348,
                4524,  61740, 38736, 14564, 26472, 65192, 49676, 23232, 19128,
                32748, 39676, 52624, 20700, 46876, 51048, 4628,  9340,  35040,
                16232, 56752, 8648,  42008, 22992, 17424, 11612, 39432, 43132,
                64712, 16764, 23620, 62556, 21136, 8276,  60436, 18688, 64072,
                10192, 1648,
            },
            {
                19280, 11588, 46936, 15496, 25888, 10052, 31108, 63828, 20388,
                43456, 2840,  62752, 7932,  19012, 12748, 63416, 7424,  48032,
                19840, 54012, 29884, 37588, 63284, 46976, 50432, 31152, 22064,
                4896,  26696, 53372, 8932,  36308, 12728, 41828, 57188, 15304,
                22356, 49744, 40276, 25464, 15996, 31740, 44300, 63188, 34036,
                9328,  37232, 5492,  16824, 34164, 9652,  4660,  25072, 53948,
                12252, 64424, 55900, 3304,  14356, 49560, 35800, 30560, 59364,
                38536, 25292, 35072, 43800, 1380,  28304, 49876, 10548, 6656,
                30876, 58708, 34664, 62568, 44328, 20508, 61372, 52716, 41916,
                49496, 24740, 36096, 56188, 52000, 2900,  33684, 23668, 56692,
                12820, 34560, 46440, 57864, 21468, 11204, 30884, 6200,  62948,
                14260, 57208, 25360, 10180, 30580, 58660, 38000, 27844, 42500,
                46452, 25488, 33120, 13284, 48944, 60964, 32624, 56924, 872,
                51796, 36520, 55092, 3632,  41188, 28724, 51828, 13744, 26448,
                42196, 54288,
            },
            {
                50656, 4468,  28016, 39924, 61908, 49508, 35840, 56192, 6320,
                51652, 37572, 22976, 45992, 53796, 25908, 33164, 16256, 28116,
                40892, 3112,  59944, 16604, 25516, 20724, 3368,  62036, 56264,
                13540, 40404, 58516, 30116, 45740, 60020, 4332,  38240, 28460,
                32776, 3976,  8044,  56480, 59700, 10536, 21452, 28976, 14268,
                46732, 57184, 42552, 26232, 50428, 56184, 19832, 47884, 16184,
                36396, 41424, 21096, 25600, 39184, 10668, 53012, 18708, 44540,
                7212,  16452, 65060, 54624, 13544, 61860, 45128, 33996, 64684,
                48064, 18356, 8116,  51832, 29044, 16288, 55796, 7284,  13680,
                33408, 63152, 748,   11620, 47348, 64464, 17036, 6900,  40056,
                52808, 20400, 8532,  3324,  36688, 44232, 59296, 53408, 27640,
                42264, 3372,  36712, 64948, 43120, 808,   23848, 62324, 12048,
                2760,  53236, 64736, 37184, 28524, 2584,  45424, 23992, 14372,
                27524, 9600,  45012, 33052, 15952, 56892, 37376, 45356, 6892,
                31316, 23692,
            },
            {
                36604, 64516, 32824, 21540, 58404, 18848, 12632, 41088, 28172,
                58256, 32720, 11572, 30000, 1556,  50272, 44292, 4080,  56484,
                12072, 23204, 45200, 7024,  55820, 42560, 9896,  27764, 36812,
                7148,  44496, 19112, 8,     49968, 27296, 20440, 51648, 10316,
                61392, 42848, 13816, 34404, 50692, 4724,  41728, 54000, 3212,
                20088, 60852, 31560, 11892, 1096,  37688, 29992, 62660, 44052,
                4180,  30832, 9008,  47628, 61612, 43028, 536,   62960, 12240,
                28584, 51108, 9948,  40480, 19040, 5500,  56884, 16088, 2696,
                38716, 55536, 14452, 36852, 48560, 4228,  38768, 30968, 45764,
                20948, 58876, 15948, 40612, 25768, 37448, 44760, 28044, 48068,
                62716, 25944, 32604, 54776, 50688, 15660, 528,   34824, 46484,
                8456,  22208, 49356, 17560, 13052, 55204, 19864, 48964, 57516,
                30444, 19520, 5596,  44060, 20852, 54044, 8004,  35972, 58604,
                47888, 20176, 59792, 6524,  49076, 24812, 244,   34196, 61816,
                16928, 57924,
            },
            {
                14176, 9196,  48600, 2088,  7040,  44720, 24584, 444,   16664,
                9160,  49020, 61492, 15000, 40124, 60624, 20632, 38588, 62488,
                46880, 35380, 51016, 32408, 13300, 38668, 52916, 18368, 48904,
                60316, 23988, 52344, 33580, 16464, 6984,  64956, 34888, 44752,
                18552, 53240, 23600, 64352, 27044, 16968, 37004, 62164, 25200,
                39608, 6516,  54612, 22328, 64384, 41724, 13720, 7100,  22892,
                51916, 59556, 18584, 54504, 29328, 16108, 33760, 24572, 48056,
                58260, 32376, 3308,  23032, 36892, 30712, 25380, 41464, 22008,
                27928, 43192, 216,   24596, 63616, 10856, 58296, 26120, 2028,
                37816, 9496,  50672, 30836, 19732, 4164,  59044, 10300, 15156,
                912,   42524, 17216, 63952, 28568, 40348, 24840, 18328, 11680,
                51952, 31980, 58908, 6840,  28368, 40396, 33192, 7852,  15668,
                39824, 50180, 10228, 58712, 14972, 40456, 63836, 17988, 31576,
                3988,  40184, 13056, 29548, 64492, 10396, 19588, 53260, 48004,
                2972,  44272,
            },
            {
                55816, 25028, 42048, 53752, 37732, 30812, 54684, 47240, 64988,
                21232, 42764, 25208, 55184, 35220, 6932,  10852, 24680, 31320,
                8144,  18176, 64396, 27052, 4176,  61328, 33888, 1140,  14848,
                32248, 10400, 63632, 38052, 12204, 41204, 54580, 24304, 2448,
                58968, 30204, 39112, 744,   44456, 48048, 58128, 11716, 30624,
                51304, 15548, 44680, 35236, 18204, 49112, 26736, 57388, 33640,
                11584, 38416, 45688, 2556,  7328,  21828, 57128, 37796, 4012,
                20720, 42332, 60804, 46096, 55452, 48900, 8968,  59436, 52056,
                11968, 61288, 46284, 32704, 21616, 42164, 17704, 47504, 64892,
                54556, 27860, 43532, 6792,  62088, 53444, 22512, 32368, 56088,
                37656, 5940,  45712, 22820, 9452,  4620,  61264, 55668, 38320,
                62256, 15296, 44452, 34676, 53436, 61688, 3528,  45456, 36668,
                23472, 60724, 27420, 34760, 3748,  51464, 25388, 11068, 50220,
                62840, 23216, 35148, 52604, 43496, 38604, 58612, 23076, 12212,
                28236, 39168,
            },
            {
                5368,  29996, 18232, 63432, 15368, 9716,  60420, 35020, 29400,
                38964, 6016,  2640,  17828, 47492, 28756, 57772, 51300, 15148,
                54636, 2296,  40212, 21028, 57620, 44436, 23172, 46800, 55372,
                28752, 43136, 3612,  21724, 59520, 46760, 31540, 9060,  15048,
                47892, 6308,  12032, 20216, 31488, 8292,  22940, 1620,  34368,
                47164, 9708,  28308, 58376, 2928,  8752,  53296, 45296, 168,
                61480, 24672, 14216, 32464, 65476, 51412, 40812, 9052,  54056,
                15036, 35676, 26896, 12412, 620,   64100, 14820, 34708, 5936,
                31448, 19236, 9648,  57628, 6212,  53480, 34468, 13116, 23292,
                4776,  18248, 33744, 56956, 13296, 35328, 42172, 18288, 51312,
                28972, 60728, 12284, 57700, 35896, 49132, 43628, 20876, 29664,
                2252,  26584, 5404,  23260, 47644, 10092, 26180, 52140, 63960,
                1024,  13032, 47496, 54988, 22532, 30796, 46588, 37704, 6324,
                42136, 54280, 1868,  26188, 17000, 4072,  31064, 41792, 8000,
                62992, 20280,
            },
            {
                60124, 52640, 11844, 34188, 27068, 51184, 22240, 4052,  13940,
                50772, 57288, 33636, 63044, 22228, 43688, 176,   36820, 42204,
                60328, 29528, 48240, 9980,  16132, 30104, 5428,  64912, 19928,
                39948, 6184,  53812, 49388, 26620, 1632,  17788, 37484, 57248,
                26284, 35172, 62748, 51992, 60044, 40232, 55888, 19068, 53056,
                65312, 4088,  24404, 40436, 61804, 30728, 21148, 39304, 17368,
                28636, 49748, 55932, 36920, 43820, 26692, 12988, 30984, 63888,
                47156, 5044,  51920, 18192, 29312, 39440, 20308, 43592, 47352,
                54188, 37568, 26076, 50420, 39604, 29192, 59968, 8184,  51440,
                40940, 60452, 46616, 23712, 2792,  49028, 8100,  65284, 2420,
                24280, 49656, 19920, 30588, 53160, 14308, 33056, 6608,  48220,
                57368, 41168, 64228, 18528, 37700, 13852, 21260, 31260, 17508,
                43492, 33040, 5972,  41808, 16584, 61144, 388,   56672, 28416,
                14748, 18724, 60604, 8592,  46960, 61648, 13536, 54760, 35864,
                49688, 32380,
            },
            {
                45984, 3060,  43232, 57856, 1348,  39728, 44056, 18520, 59156,
                10980, 45084, 26992, 8300,  51836, 12280, 65224, 20004, 5280,
                25932, 13036, 36204, 53912, 41772, 51644, 8628,  37708, 11524,
                57796, 24612, 34856, 14296, 30300, 56272, 43328, 62320, 50136,
                21252, 41036, 45848, 15964, 3376,  36020, 13624, 27428, 42076,
                16564, 37180, 50184, 12564, 46360, 15176, 36192, 63832, 5540,
                42468, 8320,  20684, 4548,  17688, 59912, 1188,  44856, 19216,
                25112, 57380, 9540,  61952, 33560, 53268, 4084,  24012, 61852,
                1624,  16588, 65504, 3676,  15076, 20276, 2496,  44064, 36328,
                30112, 14976, 9900,  52720, 27064, 38744, 29932, 45476, 11476,
                34776, 39428, 7492,  41680, 1088,  25680, 64996, 10096, 17108,
                35336, 12692, 50016, 340,   60236, 42064, 54596, 58856, 9056,
                28264, 56808, 19648, 65332, 8496,  39524, 12804, 20296, 34108,
                58132, 45100, 31428, 37260, 50324, 21068, 27140, 44840, 1184,
                24540, 15016,
            },
            {
                8516,  36056, 23448, 19860, 47416, 6336,  62812, 25832, 37348,
                31272, 54136, 14616, 40824, 37852, 31940, 16372, 48888, 33892,
                45400, 21872, 63088, 836,   24836, 59736, 34540, 26700, 50500,
                15864, 60844, 45664, 8544,  63128, 20556, 5716,  11044, 32556,
                2816,  53744, 8732,  24796, 29116, 49740, 63172, 45232, 6824,
                59820, 21776, 56552, 33176, 6572,  54468, 25576, 11160, 51236,
                58068, 35064, 63208, 48684, 10248, 23004, 50080, 35144, 7020,
                29720, 41108, 36796, 22700, 44744, 7420,  56760, 28128, 13180,
                40528, 29760, 55892, 33948, 45512, 49448, 62712, 26408, 56284,
                21568, 3880,  37428, 63964, 17624, 58072, 14456, 21208, 59212,
                53632, 15528, 62456, 47232, 18180, 59696, 38784, 22352, 45556,
                53568, 24396, 31656, 56004, 29284, 6652,  35128, 3856,  46768,
                38800, 51744, 24780, 36248, 49660, 27016, 53472, 43920, 9128,
                49100, 5180,  22760, 11692, 56328, 33556, 5656,  65444, 18484,
                57544, 40452,
            },
            {
                63820, 27864, 50436, 61584, 13724, 32332, 55640, 8796,  48836,
                972,   23544, 19780, 61380, 3152,  53388, 24016, 59612, 9520,
                56788, 50312, 6848,  32636, 19472, 12768, 47764, 3116,  44036,
                32144, 1244,  19192, 40992, 36240, 47712, 24452, 39888, 28396,
                13956, 58052, 18660, 33828, 55308, 10452, 4420,  23868, 32744,
                11100, 29564, 2224,  19428, 43048, 59928, 3736,  47312, 22496,
                31092, 12740, 26964, 41252, 31820, 54628, 38968, 58440, 16200,
                62528, 50612, 2204,  13660, 48956, 17092, 60076, 36632, 50708,
                8500,  22636, 42696, 12208, 24916, 9264,  32188, 17080, 11608,
                48596, 61616, 44928, 31832, 160,   41352, 52176, 4904,  26224,
                43604, 3500,  32896, 27500, 55276, 4212,  50508, 28728, 58420,
                3236,  39496, 8512,  20528, 46172, 16204, 50188, 24192, 62908,
                15212, 2020,  11136, 45152, 3388,  32076, 59344, 23640, 64412,
                26572, 39748, 62736, 716,   41468, 15804, 52776, 38528, 29708,
                11276, 51800,
            },
            {
                16240, 4740,  38792, 9864,  29056, 52708, 35528, 16072, 41932,
                63736, 5904,  47776, 58064, 29700, 6956,  44536, 27856, 1928,
                38100, 16024, 43020, 55408, 38788, 62192, 17100, 57004, 22092,
                63480, 28268, 51024, 12516, 3932,  54652, 16028, 52172, 60348,
                44404, 5960,  64872, 38756, 43316, 20768, 59144, 37516, 53680,
                49156, 40004, 62976, 52436, 28024, 17476, 34064, 41016, 55036,
                1520,  16428, 46412, 2996,  60936, 14532, 27872, 3656,  46664,
                11832, 20416, 56084, 30848, 39892, 25596, 10460, 32780, 19680,
                63328, 48332, 5344,  60708, 52652, 36068, 57308, 1076,  39444,
                54472, 7724,  19708, 25064, 12292, 48416, 36052, 63024, 31520,
                9040,  56476, 21948, 10648, 36368, 13436, 43060, 33908, 5840,
                15540, 61148, 43784, 11500, 63412, 27228, 39580, 12172, 33284,
                21692, 58184, 29716, 61972, 14796, 18868, 6560,  37196, 2488,
                13564, 52284, 18036, 28964, 59508, 25148, 10056, 46680, 3460,
                34400, 21988,
            },
            {
                47660, 31500, 56176, 17956, 41600, 24232, 3204,  21476, 59816,
                28284, 33780, 43560, 17388, 35872, 11452, 55488, 41380, 19144,
                64132, 30504, 11200, 26848, 45552, 4300,  29912, 40788, 10624,
                7344,  38544, 55876, 59832, 26436, 31660, 64204, 352,   34504,
                22580, 48672, 26684, 1424,  14636, 47768, 30544, 17512, 288,
                15300, 45788, 7924,  23940, 38008, 9888,  65008, 29232, 19644,
                37488, 61840, 52608, 24460, 35828, 7816,  21160, 52248, 33320,
                24676, 43608, 6180,  65128, 52532, 920,   46256, 42028, 3708,
                57592, 31752, 17156, 38360, 21400, 6316,  42068, 63640, 23140,
                27692, 34416, 42596, 59084, 55464, 6116,  22988, 18456, 47064,
                37728, 16788, 64360, 45332, 52280, 24784, 8148,  18632, 63768,
                49344, 26388, 36952, 32492, 51584, 2160,  59632, 44440, 7172,
                53600, 48160, 42092, 34552, 55552, 52052, 40400, 47476, 17084,
                32980, 55472, 35604, 48100, 44276, 6920,  61196, 20136, 55800,
                41956, 59264,
            },
            {
                25368, 140,   60352, 45312, 6660,  65176, 46736, 11248, 50724,
                39220, 13328, 9000,  25780, 50544, 21780, 62044, 14180, 33564,
                51680, 3452,  22788, 58752, 8448,  52668, 23876, 35924, 54208,
                48308, 15052, 33648, 21648, 42920, 10304, 45912, 7904,  19492,
                37320, 11196, 31332, 52496, 61776, 7196,  40984, 56864, 64316,
                26128, 35208, 13212, 58468, 47860, 900,   50320, 13780, 7848,
                58568, 42872, 5616,  18788, 56896, 44332, 64028, 40644, 9416,
                59368, 37480, 27488, 16112, 35132, 21352, 61176, 53808, 14628,
                27168, 44860, 404,   55116, 28716, 15232, 46248, 30188, 50448,
                14032, 2968,  47604, 15788, 28424, 33416, 10580, 60856, 1352,
                28804, 50040, 40236, 2368,  30092, 61424, 56888, 40136, 30844,
                21868, 1472,  55312, 17824, 6124,  23136, 56152, 18988, 37124,
                27548, 16844, 4940,  8764,  21284, 28104, 11648, 61204, 29688,
                43064, 9640,  3264,  20588, 12680, 31252, 36484, 49960, 26720,
                13416, 7648,
            },
            {
                43952, 36336, 21224, 12480, 54548, 37440, 31048, 57608, 18636,
                4868,  56632, 53216, 64680, 672,   46128, 38704, 4900,  24608,
                46900, 60684, 36940, 49012, 31936, 14420, 65384, 228,   18364,
                43256, 25268, 2132,  5916,  49360, 17548, 38288, 29724, 58720,
                51320, 41940, 17136, 56548, 23632, 35428, 27808, 9544,  43476,
                21508, 60504, 4304,  32068, 15884, 43736, 56580, 23044, 45864,
                32432, 27732, 10376, 39484, 48468, 12436, 272,   30320, 18268,
                54408, 4216,  47724, 8616,  57448, 12220, 29280, 6896,  39004,
                23580, 10016, 34624, 64240, 49268, 11268, 59080, 19512, 5576,
                53200, 60976, 36832, 8736,  64256, 50912, 38840, 42824, 54204,
                12492, 24428, 59020, 6228,  14980, 20960, 47948, 11300, 54100,
                45464, 13768, 59284, 47060, 34876, 42240, 9436,  30764, 63796,
                424,   60340, 24424, 39080, 64568, 44768, 876,   50060, 22100,
                57664, 63308, 24576, 50728, 57112, 40580, 16500, 1280,  62276,
                32404, 52524,
            },
            {
                64448, 9240,  50224, 33432, 27136, 15608, 1080,  43696, 26368,
                36464, 22932, 31800, 40932, 15332, 30708, 58368, 9336,  53980,
                28968, 12488, 18052, 6112,  20860, 41760, 49900, 27368, 60176,
                31284, 57360, 63868, 35736, 52944, 60792, 23704, 55072, 14184,
                2304,  63344, 8644,  44516, 4356,  13384, 50780, 19288, 5448,
                51808, 29296, 41624, 54676, 20624, 26704, 34584, 62084, 3360,
                51588, 15244, 64500, 33436, 29016, 22916, 60520, 35352, 49648,
                14784, 62008, 32128, 42792, 24236, 50012, 37144, 17608, 62796,
                47440, 52236, 19224, 40596, 24320, 3544,  33912, 38636, 9852,
                32160, 17820, 41020, 21892, 1984,  45576, 25584, 14920, 57548,
                7096,  35400, 18600, 33060, 44112, 38204, 220,   34696, 6764,
                27564, 40988, 9800,  25052, 64908, 15144, 52952, 49148, 13348,
                41068, 46104, 51280, 31880, 13456, 58424, 25256, 8244,  36172,
                5276,  15124, 38292, 27812, 4452,  64828, 22748, 45620, 5624,
                39468, 18272,
            },
            {
                14620, 29560, 40684, 3900,  62108, 22756, 51592, 8772,  63404,
                48348, 2272,  44988, 19948, 5496,  26648, 49336, 17320, 35460,
                1480,  44168, 39760, 54916, 61760, 34596, 10940, 5512,  45944,
                12864, 8896,  19748, 40620, 27576, 12424, 3496,  41228, 46740,
                28144, 24692, 33632, 20164, 38116, 59384, 46820, 32560, 55832,
                12056, 37648, 49436, 9308,  63608, 38920, 6096,  11664, 40676,
                25188, 20536, 56068, 2520,  51012, 16532, 54956, 25752, 45092,
                2592,  22736, 39448, 18740, 63812, 5228,  44588, 56212, 32628,
                2276,  58076, 13596, 7008,  61308, 43040, 26520, 55320, 65200,
                44180, 24788, 57824, 30416, 53892, 19008, 5364,  32016, 20580,
                47404, 42012, 61892, 51292, 54828, 26100, 65400, 17196, 60036,
                37380, 52036, 19160, 4024,  29580, 38808, 2824,  25940, 34356,
                22012, 10600, 56668, 19200, 4108,  34996, 41716, 54040, 18580,
                46684, 31388, 53020, 44816, 9084,  33652, 53880, 29348, 11212,
                55504, 23680,
            },
            {
                1916,  57972, 53928, 18944, 46924, 59096, 39152, 13688, 33576,
                16436, 58704, 11568, 61128, 54568, 37620, 62896, 43216, 20984,
                56852, 64636, 27032, 3768,  45236, 16156, 56392, 39060, 21512,
                36452, 51100, 47068, 15640, 7324,  33104, 65044, 19916, 35472,
                5888,  60916, 48948, 53708, 29136, 65212, 1316,  39288, 23208,
                62808, 2104,  18068, 24932, 3764,  52128, 30380, 17596, 54936,
                35320, 47484, 8344,  43208, 36724, 4744,  10748, 41084, 7452,
                29120, 52184, 12660, 54616, 1408,  31056, 26020, 11184, 21548,
                41628, 27696, 36816, 31096, 53292, 16800, 47072, 21244, 612,
                49764, 11560, 4196,  48244, 13016, 60040, 36408, 51788, 64844,
                26392, 10176, 3156,  22864, 12112, 7888,  29444, 49932, 23240,
                3100,  31632, 62040, 48088, 56556, 21312, 45288, 61420, 5756,
                58608, 29916, 7252,  37684, 49276, 15808, 29504, 63916, 12572,
                39736, 60536, 976,   18856, 58960, 14088, 48464, 19676, 60384,
                34632, 49160,
            },
            {
                45436, 37172, 24764, 11120, 42804, 6120,  28844, 54672, 21108,
                41948, 27476, 50836, 34892, 8012,  23564, 12168, 31744, 6724,
                50200, 14496, 32888, 22388, 9044,  29836, 24960, 52136, 63260,
                3036,  29100, 24176, 59576, 54276, 44040, 48744, 9484,  57820,
                16048, 39956, 11708, 3592,  15256, 25432, 10368, 16692, 44184,
                27208, 34440, 57424, 46636, 12872, 59308, 43324, 48768, 60760,
                640,   13428, 58928, 26416, 63144, 46416, 57560, 20172, 64960,
                37236, 60208, 9192,  34532, 58796, 48460, 15820, 51816, 4220,
                59636, 9032,  50364, 21960, 1640,  57276, 7976,  36252, 15168,
                28904, 61788, 39236, 34864, 26876, 43516, 9828,  452,   40052,
                29576, 58248, 16276, 31412, 46676, 57120, 41868, 14144, 44884,
                55692, 12828, 43284, 36012, 7392,  11704, 54420, 18344, 37028,
                15584, 52848, 43580, 26788, 61636, 55136, 22624, 3128,  51560,
                27124, 23200, 11228, 35816, 41696, 25772, 7112,  37972, 43052,
                26984, 5076,
            },
            {
                20772, 62872, 7396,  31364, 16912, 35432, 2604,  49596, 60372,
                7572,  3984,  39864, 29804, 14836, 47208, 2468,  60056, 40740,
                25076, 10440, 37584, 52588, 48232, 60068, 2072,  32784, 17436,
                41972, 55764, 5224,  31712, 38068, 588,   25436, 30044, 52048,
                22732, 55556, 31220, 45668, 42112, 35944, 52120, 58084, 48400,
                7796,  14516, 42720, 31360, 36624, 20152, 27892, 6804,  22668,
                38256, 28888, 41320, 17844, 22112, 30700, 13808, 32792, 48408,
                16716, 44116, 26920, 20672, 41120, 7876,  38372, 63216, 35256,
                44080, 17360, 64856, 45608, 39672, 27952, 12336, 59828, 42248,
                22960, 54740, 8068,  16036, 56348, 62984, 23364, 17612, 55540,
                5992,  36800, 49396, 63680, 39068, 4672,  18916, 33200, 63016,
                8960,  21564, 26604, 16732, 58920, 32384, 27756, 43016, 50476,
                2156,  65244, 23816, 12016, 1328,  45116, 9868,  34088, 42928,
                7404,  48252, 54696, 64060, 29676, 51348, 1824,  63288, 9576,
                16324, 59252,
            },
            {
                12324, 33540, 51784, 47912, 56460, 64764, 25284, 12012, 31516,
                45880, 24036, 57816, 18188, 65524, 38752, 51856, 27968, 16888,
                45996, 58540, 544,   63340, 18372, 41500, 13120, 58164, 8504,
                44908, 14480, 62604, 11148, 21816, 17600, 61896, 13504, 43296,
                37044, 1236,  19088, 61488, 56768, 6592,  21032, 31724, 4104,
                60580, 53972, 22060, 64168, 1052,  55768, 15404, 9740,  65464,
                31992, 54008, 10528, 3732,  52520, 6196,  39756, 53724, 792,
                24324, 4816,  50936, 14312, 46520, 29188, 23524, 55120, 12404,
                30484, 25664, 4980,  14852, 33604, 62240, 48564, 31380, 51924,
                5040,  33400, 46268, 20620, 3080,  41064, 49108, 33884, 45384,
                11428, 24352, 42844, 1284,  21384, 35608, 59452, 25692, 1816,
                39624, 53032, 4844,  49680, 41132, 764,   62412, 8632,  22308,
                31600, 40464, 17668, 48116, 32468, 38864, 19628, 62496, 56840,
                16996, 31972, 3604,  20916, 15620, 57432, 45500, 23404, 31892,
                52692, 39752,
            },
            {
                44424, 15452, 26540, 860,   21836, 38720, 44560, 19272, 37356,
                63572, 10608, 52628, 124,   43996, 21192, 56464, 4540,  33956,
                54252, 20244, 42548, 27644, 34912, 6924,  38032, 47136, 27876,
                23192, 35008, 39356, 51964, 45876, 56472, 34544, 4380,  59644,
                8032,  26900, 50756, 9444,  23488, 28720, 63824, 40744, 18928,
                36312, 29152, 10708, 6004,  48492, 40772, 33356, 51692, 46340,
                19064, 44476, 62524, 49476, 36136, 61452, 27352, 9840,  59472,
                35600, 56224, 31680, 64148, 2892,  60376, 18712, 92,    49772,
                20552, 56848, 37328, 52768, 10124, 24088, 19608, 2208,  38684,
                17928, 64056, 25948, 52760, 31112, 7184,  28032, 14340, 62432,
                50820, 19020, 54680, 13928, 28092, 53308, 10512, 51364, 47124,
                29852, 57680, 34812, 64700, 24004, 14928, 38208, 48316, 12776,
                56824, 4676,  35640, 60028, 55996, 6844,  50876, 25504, 13364,
                37064, 60984, 44336, 39096, 5100,  34344, 11800, 18504, 56072,
                3916,  28980,
            },
            {
                6068,  57348, 41384, 60784, 9228,  14220, 50604, 6452,  55636,
                16580, 34320, 47520, 27012, 32472, 9940,  36100, 13612, 64020,
                8412,  30640, 5004,  15252, 55680, 24316, 50968, 19640, 64300,
                904,   48752, 18800, 3456,  26832, 8652,  49668, 40304, 20792,
                47876, 64692, 34340, 38932, 14116, 49140, 88,    12856, 46124,
                24820, 50048, 39312, 18304, 26548, 61668, 23812, 37240, 5376,
                14244, 2264,  24936, 33600, 15096, 43416, 23048, 45684, 19284,
                12224, 42256, 7204,  36732, 53352, 10472, 33964, 45784, 40188,
                6888,  61868, 47220, 2924,  41888, 55156, 7076,  44700, 56628,
                13732, 9384,  43796, 59188, 12552, 37416, 57284, 21944, 2324,
                30696, 60356, 34252, 6812,  61580, 44460, 16364, 5764,  36560,
                18424, 13652, 9880,  28692, 46168, 20104, 53156, 34324, 25320,
                63184, 45616, 9348,  28280, 21804, 15328, 29952, 41244, 96,
                49864, 28276, 9500,  47632, 25272, 62508, 41268, 50368, 36620,
                47044, 65140,
            },
            {
                10680, 37052, 19356, 30528, 53628, 33820, 62500, 29112, 1628,
                42776, 22520, 13856, 54944, 6192,  60508, 24884, 48544, 39244,
                23176, 51196, 61820, 47536, 10620, 60928, 4256,  31916, 54076,
                10764, 56976, 60344, 30772, 42328, 63864, 15832, 28480, 32136,
                12440, 44388, 16608, 4836,  53920, 43248, 33220, 55188, 8816,
                62672, 2548,  52812, 57996, 45628, 12344, 2828,  59396, 55280,
                27496, 39936, 56496, 20740, 8084,  58384, 3332,  51168, 63352,
                29652, 47740, 21788, 25236, 17124, 43368, 27736, 58408, 15188,
                32552, 10956, 27440, 18224, 30736, 63472, 35956, 26912, 60824,
                29628, 35084, 49616, 776,   18680, 65104, 51764, 43732, 5236,
                39564, 26496, 9432,  48076, 39988, 30932, 24600, 64044, 42084,
                22648, 59664, 43804, 3924,  55728, 7696,  60448, 2504,  29156,
                18816, 53916, 14048, 51448, 3752,  64280, 47160, 59088, 5524,
                23392, 18640, 58060, 14024, 53116, 31028, 6952,  27784, 396,
                21100, 24556,
            },
            {
                53148, 49504, 3600,  45924, 23804, 4488,  40904, 25884, 59060,
                51228, 8128,  62280, 40488, 19664, 50148, 42588, 1212,  18044,
                44932, 12332, 37168, 32448, 20880, 44004, 28760, 14204, 40868,
                34048, 26024, 6428,  13552, 36568, 23068, 1432,  52204, 55752,
                6104,  24588, 58632, 30436, 62392, 17984, 25796, 58896, 37308,
                16344, 31888, 13628, 34548, 7744,  30476, 43400, 20348, 49724,
                31188, 60588, 11272, 48016, 29512, 38984, 17504, 32356, 37792,
                1864,  15408, 60920, 56908, 39436, 50108, 4604,  64432, 24256,
                54780, 42924, 21760, 59064, 49452, 15616, 11888, 22472, 50748,
                3548,  21184, 39520, 24592, 42072, 32836, 10036, 16380, 36220,
                58672, 46348, 17452, 22212, 57392, 3184,  12616, 55216, 48648,
                408,   32880, 52004, 16848, 39500, 31312, 11884, 44508, 41056,
                6468,  36872, 24392, 43832, 40316, 33616, 10224, 35888, 54508,
                43472, 65408, 34592, 2092,  22116, 56400, 17248, 61416, 13908,
                58232, 34480,
            },
            {
                16488, 27008, 63936, 12888, 57788, 18072, 48532, 10000, 20048,
                30784, 36856, 2988,  28792, 57148, 15744, 31084, 62148, 28192,
                56648, 2776,  25548, 53996, 39920, 1716,  58500, 46508, 7464,
                17132, 44216, 21280, 53616, 47356, 58812, 11508, 37528, 63032,
                18932, 40604, 51536, 2036,  36200, 10500, 47836, 22312, 5160,
                44380, 20692, 42448, 64776, 22752, 54552, 15872, 9664,  35592,
                6740,  16976, 44164, 164,   64508, 53964, 12900, 5212,  57596,
                26552, 51996, 9312,  3584,  30216, 13168, 21300, 8988,  38048,
                1692,  51736, 35384, 6288,  39172, 688,   53640, 41436, 8480,
                46992, 57532, 15636, 61476, 4876,  26980, 47616, 55604, 23904,
                12840, 52044, 784,   64524, 36008, 50316, 19804, 38644, 27580,
                8080,  61540, 25748, 36296, 63292, 23104, 49976, 58284, 16528,
                48796, 60960, 31128, 928,   56992, 26404, 17240, 21304, 12672,
                30892, 7536,  51732, 42060, 37912, 10540, 44132, 48284, 39252,
                7912,  43292,
            },
            {
                1804,  32700, 39200, 8332,  42688, 35852, 60640, 14400, 39584,
                46996, 53548, 24636, 45756, 34768, 4116,  11360, 53460, 8016,
                33500, 59656, 16044, 7068,  65184, 17796, 35936, 22380, 61888,
                52836, 38572, 64880, 3804,  33308, 17784, 44600, 25776, 3292,
                34140, 9572,  46420, 21120, 27396, 41776, 3064,  52480, 29412,
                61072, 56180, 26916, 768,   37888, 49048, 62220, 25684, 41260,
                63892, 23324, 52780, 36468, 26172, 41852, 23852, 48984, 44576,
                20712, 40952, 33428, 45364, 63072, 35780, 52576, 47144, 30976,
                17356, 46192, 13336, 65508, 25204, 45380, 32768, 61684, 17484,
                36836, 31684, 11288, 54852, 50712, 8188,  20420, 29948, 63264,
                7316,  42220, 32524, 28588, 10280, 43484, 6540,  34004, 15268,
                56528, 11796, 20716, 47212, 2080,  14428, 26760, 35844, 3864,
                22144, 54964, 11408, 20228, 62516, 7844,  52864, 45304, 60704,
                39204, 25980, 16504, 28944, 63436, 5304,  33468, 24928, 54236,
                30424, 62260,
            },
            {
                46604, 55476, 20360, 52540, 25524, 344,   31512, 55436, 5692,
                63136, 17096, 12708, 8872,  64864, 38300, 22288, 46968, 40344,
                20124, 43360, 49556, 30032, 10084, 52032, 26292, 56744, 12040,
                1332,  29456, 50064, 10004, 27520, 55008, 7764,  41472, 49116,
                58152, 28952, 13616, 61212, 55468, 15416, 65368, 38888, 12688,
                46932, 7160,  11048, 51512, 18128, 4680,  32668, 46472, 1664,
                51068, 13392, 32972, 4608,  18644, 9324,  61032, 34780, 7248,
                65272, 11368, 54304, 23308, 16760, 6948,  59232, 26344, 56396,
                62132, 4472,  28956, 55852, 9588,  20268, 28648, 56220, 5380,
                26196, 63528, 1968,  44848, 34508, 38332, 60600, 1760,  45360,
                35104, 20000, 56336, 14712, 53064, 25160, 62404, 58364, 45316,
                30308, 40352, 50276, 5996,  43088, 54296, 9428,  61812, 33076,
                8108,  39292, 45760, 15684, 49372, 37824, 29288, 1660,  48860,
                4428,  56684, 11424, 46280, 49928, 20160, 59980, 3232,  18316,
                11464, 22780,
            },
            {
                14436, 5320,  29468, 61268, 15992, 50288, 45172, 22728, 27708,
                2436,  33480, 58492, 41388, 18784, 50852, 59392, 25300, 13980,
                63940, 3808,  37976, 22900, 46840, 41540, 4596,  31392, 48336,
                36984, 23652, 14732, 40996, 59688, 20448, 31076, 62076, 15976,
                21920, 53992, 38284, 6492,  48808, 31196, 8340,  34168, 18320,
                23872, 40796, 35264, 29792, 59960, 40084, 12540, 58528, 28988,
                19188, 38496, 57152, 60096, 47428, 31688, 55328, 14536, 28428,
                17680, 37132, 448,   48552, 28532, 42300, 2772,  19260, 10652,
                40900, 22840, 33440, 42496, 16600, 48920, 3692,  38392, 12792,
                52116, 42268, 22704, 28616, 17188, 12248, 52752, 40540, 15468,
                26336, 59652, 3840,  39160, 46804, 17256, 1484,  22132, 51268,
                4392,  18112, 65500, 33736, 28924, 57536, 20120, 41508, 51384,
                28776, 65036, 25808, 35212, 4924,  59280, 42404, 24532, 14864,
                33376, 61960, 22828, 680,   36280, 14276, 28008, 51660, 37120,
                41340, 58648,
            },
            {
                50832, 36364, 44096, 11004, 34408, 6716,  65252, 11988, 36596,
                43772, 49444, 21928, 54896, 27348, 6152,  32120, 2116,  35892,
                51552, 27256, 55808, 12508, 60836, 34232, 15672, 8740,  42664,
                18652, 62428, 6672,  46344, 35452, 324,   51128, 12744, 4408,
                45496, 32904, 1112,  44092, 24768, 19892, 57908, 50976, 60072,
                4268,  54340, 63520, 14764, 45220, 24844, 21516, 6168,  54856,
                43616, 8216,  27684, 11840, 44692, 21908, 5748,  39212, 50576,
                59180, 31384, 56520, 14128, 61376, 37804, 32764, 44632, 14816,
                36264, 50232, 7652,  60872, 52852, 35060, 63036, 23528, 46004,
                18828, 59432, 7540,  48528, 64444, 24988, 31616, 5472,  54728,
                10696, 47940, 23236, 8060,  60932, 32092, 27984, 41780, 9688,
                35856, 25056, 53648, 10788, 16064, 37836, 46076, 200,   18144,
                13900, 56724, 2976,  53104, 31956, 18324, 9296,  64576, 36628,
                53984, 18892, 43264, 32216, 58396, 54808, 9596,  44828, 64976,
                6912,  26484,
            },
            {
                19080, 62824, 2704,  56792, 21396, 28560, 39280, 52412, 19256,
                61052, 7740,  30452, 564,   47284, 16140, 44520, 56340, 10476,
                17960, 6580,  33036, 19124, 268,   53664, 63688, 21440, 51516,
                58588, 33096, 54156, 25568, 16804, 63612, 39276, 23888, 36144,
                59884, 26708, 18412, 63948, 11340, 36992, 43008, 552,   26256,
                32304, 48612, 20248, 1904,  56020, 9068,  50732, 36760, 15740,
                61272, 22620, 49384, 1120,  16548, 52928, 62448, 2200,  23596,
                43596, 4692,  20032, 25716, 8484,  50776, 21832, 64796, 53488,
                600,   58564, 19808, 26316, 1872,  14288, 41024, 57216, 9784,
                33916, 30600, 14828, 40148, 2944,  56508, 43288, 18976, 49732,
                28764, 65188, 36300, 43240, 12312, 54180, 37652, 63944, 14152,
                60004, 46552, 1200,  22740, 60660, 7176,  31628, 63228, 24492,
                48328, 42208, 10380, 22576, 44728, 13004, 55696, 21116, 5728,
                46984, 9872,  51932, 25688, 7456,  40140, 21848, 31236, 1028,
                48452, 33512,
            },
            {
                8688,  38880, 24544, 47564, 41796, 59216, 16952, 9180,  57520,
                26008, 15088, 39052, 63600, 12148, 37272, 60968, 29144, 42580,
                62400, 47744, 58340, 44028, 25092, 38340, 28744, 45708, 3704,
                27176, 10944, 2672,  43152, 56836, 9140,  28580, 47056, 56056,
                10908, 7868,  40428, 50128, 56624, 5548,  28660, 13264, 44992,
                16376, 9556,  38112, 27612, 42672, 33804, 64000, 47676, 30520,
                4048,  35000, 63396, 40592, 29648, 35576, 42676, 27000, 10864,
                47636, 63880, 39644, 52968, 45932, 11912, 57040, 5488,  27712,
                46436, 30672, 39092, 11512, 47392, 31556, 7232,  27704, 49636,
                136,   55132, 21164, 51008, 35596, 9468,  22484, 58992, 37844,
                400,   17472, 30868, 51316, 21380, 6172,  48716, 3160,  20412,
                55420, 31528, 38972, 42624, 48668, 26504, 52460, 11492, 36700,
                6276,  30792, 58932, 38472, 61408, 28112, 50524, 39928, 26420,
                30248, 41420, 2768,  64328, 17740, 47436, 61900, 15772, 24140,
                12924, 56200,
            },
            {
                45192, 16476, 31292, 13444, 4880,  49184, 1508,  33072, 46308,
                3564,  42556, 50932, 33680, 24432, 53368, 8224,  20636, 23716,
                39396, 15064, 29808, 9156,  50424, 5412,  13684, 59728, 35756,
                17064, 39832, 50616, 13104, 22148, 32312, 5900,  42116, 19332,
                51712, 61436, 30184, 15236, 34384, 22096, 61780, 39056, 55432,
                62924, 22684, 52660, 5676,  58852, 17896, 2748,  25952, 11156,
                42288, 51508, 13936, 24976, 56412, 7640,  19148, 58504, 33348,
                16340, 7832,  34496, 29104, 1888,  18120, 31432, 40120, 23736,
                9356,  17016, 64040, 55672, 43584, 60144, 22076, 17760, 64596,
                44144, 38004, 61988, 5308,  45956, 60864, 13532, 33708, 7776,
                45568, 55336, 14332, 2648,  25960, 56700, 16704, 33816, 44032,
                27048, 8180,  17220, 56612, 13656, 3068,  21248, 58360, 44304,
                54216, 16160, 19984, 1476,  47432, 7104,  33288, 2044,  57892,
                16880, 60396, 13632, 38144, 28996, 3724,  34212, 42652, 52664,
                60952, 28824,
            },
            {
                1936,  52016, 64556, 54184, 26812, 36948, 63376, 23416, 29488,
                55228, 21112, 10132, 57144, 18416, 3688,  48712, 34572, 1144,
                54148, 4984,  36824, 64468, 20780, 54928, 40568, 7944,  22996,
                56352, 30316, 65532, 37276, 48276, 59304, 52764, 14716, 33796,
                2008,  23552, 44524, 4516,  52964, 46088, 17700, 49728, 7384,
                34744, 41700, 12500, 46892, 31692, 14104, 53348, 39544, 21076,
                57068, 18276, 32220, 5140,  47988, 64816, 13156, 37880, 51084,
                55656, 22796, 13376, 58180, 63196, 36748, 43892, 13896, 61864,
                49788, 34656, 3348,  24496, 15828, 37424, 51620, 35312, 6056,
                13784, 24172, 10948, 26864, 18160, 29524, 52024, 25720, 63380,
                21020, 40240, 60048, 34652, 62996, 39320, 29900, 58980, 11564,
                52252, 4172,  64368, 29616, 36132, 62464, 40216, 32828, 4368,
                27716, 64260, 34428, 51804, 25024, 15648, 42948, 63748, 11376,
                48512, 35360, 21308, 54328, 49796, 12020, 56872, 6344,  35940,
                20764, 40508,
            },
            {
                59312, 22888, 6576,  34608, 10468, 19972, 43684, 51240, 12228,
                35516, 62844, 5460,  28628, 45068, 40808, 65528, 12784, 50768,
                60528, 26768, 11724, 46748, 34032, 16596, 31184, 62928, 48516,
                1136,  44444, 20304, 4828,  26524, 18136, 3048,  64220, 27388,
                47788, 37924, 12008, 58688, 26944, 1576,  10412, 30556, 24956,
                2356,  56784, 29096, 65080, 23832, 37340, 61508, 6816,  46048,
                59920, 1736,  54448, 39040, 10372, 21636, 44124, 4560,  30068,
                1452,  44972, 41076, 49536, 6548,  24756, 54112, 2700,  58264,
                20940, 42424, 52560, 6436,  29024, 10160, 2472,  58716, 42024,
                30088, 53508, 58032, 33228, 39744, 2128,  47568, 4344,  42236,
                11624, 27408, 5432,  47052, 10360, 44652, 7244,  22984, 40672,
                47656, 35252, 21844, 49924, 9516,  45640, 18756, 14580, 49672,
                23256, 9224,  41312, 12256, 62204, 36680, 19168, 52444, 23304,
                4232,  44892, 7736,  32004, 24268, 59984, 19528, 27280, 46872,
                4272,  11636,
            },
            {
                37112, 49704, 18264, 41536, 60244, 55676, 14964, 7092,  40432,
                17576, 47048, 38056, 14148, 59184, 31296, 25896, 16468, 44136,
                31952, 18968, 41936, 23484, 1468,  57848, 42956, 10776, 25660,
                53404, 14884, 8636,  61396, 34788, 45032, 40028, 9680,  57092,
                20856, 62620, 16840, 32360, 40080, 64900, 54804, 36572, 59932,
                20468, 15852, 50816, 9288,  36,    19884, 49736, 28596, 33572,
                9472,  27384, 23132, 45348, 60820, 32984, 52704, 25764, 59992,
                18464, 62112, 27540, 20208, 11020, 33024, 48052, 28452, 7588,
                37772, 11936, 31844, 48304, 62892, 54400, 46292, 25608, 19752,
                47924, 1592,  16180, 44620, 64920, 55396, 15060, 35932, 57456,
                53048, 31912, 50528, 23612, 19392, 53440, 14720, 63676, 300,
                18748, 61124, 12544, 24980, 5260,  54940, 26924, 864,   61312,
                38012, 46608, 55756, 4932,  29020, 57064, 8580,  30648, 38680,
                55248, 26732, 62552, 42200, 780,   45256, 38708, 9172,  63532,
                55708, 32272,
            },
            {
                43904, 14200, 28196, 46756, 2148,  24260, 30944, 61772, 712,
                57872, 26616, 53120, 2308,  22360, 9272,  55620, 5800,  37636,
                58244, 3888,  49252, 61028, 52612, 27788, 4512,  18624, 37020,
                32884, 40936, 57272, 29476, 11920, 55012, 23084, 30684, 36268,
                6340,  42636, 50020, 8684,  23980, 19412, 42504, 13692, 48012,
                44192, 4136,  33664, 40556, 45064, 57688, 11856, 43024, 16092,
                64584, 36996, 50300, 16852, 28552, 548,   15344, 40088, 48268,
                10140, 36216, 5184,  54788, 38820, 60332, 16008, 19420, 45752,
                55300, 26492, 60868, 18080, 21596, 39156, 12340, 32340, 61548,
                9072,  36272, 50504, 7376,  20500, 10484, 31144, 23776, 18904,
                8384,  16496, 37920, 58584, 1712,  28504, 36496, 46016, 25700,
                32308, 53636, 38084, 43740, 59780, 33920, 51608, 41832, 57088,
                7120,  18348, 31604, 22424, 49176, 380,   45672, 60220, 14124,
                33792, 17708, 57588, 10556, 15628, 53316, 33296, 14348, 50280,
                17456, 25008,
            },
            {
                62140, 8548,  57844, 32732, 5752,  38160, 52488, 48092, 34120,
                23124, 8972,  32168, 50244, 62124, 39316, 47596, 20384, 28464,
                10256, 55004, 15764, 8352,  37788, 13332, 46116, 50304, 60392,
                5608,  22220, 47016, 51564, 1552,  16308, 60484, 50484, 13568,
                54048, 132,   29040, 56500, 47116, 3180,  33188, 5952,  27728,
                62484, 38044, 59036, 26056, 53932, 32504, 4364,  24504, 56028,
                2952,  41568, 12396, 5740,  57180, 35520, 54832, 7300,  23408,
                32148, 51752, 15712, 43640, 23288, 180,   51336, 36024, 64904,
                4204,  14904, 40916, 1880,  44624, 34372, 5848,  16884, 55340,
                40884, 22324, 62608, 25428, 42684, 37508, 59680, 49172, 61656,
                44224, 3576,  62696, 13372, 42432, 33064, 60648, 50404, 10116,
                57744, 6960,  15380, 1948,  30388, 16408, 22152, 11444, 29544,
                35484, 13468, 60424, 43308, 15248, 39248, 25996, 20816, 3208,
                49520, 6260,  39700, 20572, 30172, 65320, 5000,  23064, 41232,
                29832, 40,
            },
            {
                53672, 21424, 39308, 50400, 63884, 16688, 27480, 11396, 19468,
                41552, 64140, 15920, 43756, 18672, 33968, 44,    52160, 64032,
                22860, 33836, 40280, 25508, 31480, 64940, 21404, 55096, 28808,
                12628, 64136, 17764, 24912, 38356, 42484, 4192,  46656, 26060,
                18400, 34148, 60612, 14728, 37368, 62860, 50752, 22884, 53556,
                10800, 18244, 22168, 13508, 6708,  17448, 63100, 35812, 46428,
                21460, 53692, 31876, 62776, 20324, 42692, 61412, 12620, 45540,
                65376, 2392,  56704, 29324, 63500, 8696,  41148, 30716, 10488,
                22292, 33800, 59120, 9104,  56740, 27636, 65328, 51520, 3412,
                28820, 14100, 32036, 4188,  52340, 28048, 560,   6632,  39816,
                29052, 34492, 48104, 25288, 54256, 6760,  21524, 3192,  17660,
                40388, 27896, 46780, 55912, 39904, 62788, 5980,  46020, 65296,
                25132, 53620, 3628,  51080, 33536, 63400, 10564, 51704, 42900,
                64116, 28364, 46240, 51868, 36944, 25564, 47952, 54736, 60848,
                7088,  46152,
            },
            {
                11260, 26168, 3468,  13100, 20356, 43812, 7792,  60132, 45768,
                55212, 3280,  36980, 6484,  29428, 11772, 60256, 14896, 41764,
                46448, 1772,  62200, 19340, 44864, 2528,  34568, 9268,  39552,
                292,   43928, 35716, 7292,  62584, 32396, 20584, 8712,  65092,
                41196, 45816, 22300, 6776,  26744, 11932, 17480, 57764, 40292,
                7804,  31248, 51580, 48156, 60252, 30148, 39708, 51956, 14192,
                7748,  26040, 47400, 37632, 9316,  24888, 49164, 18236, 27820,
                38368, 21172, 11876, 34636, 47008, 14368, 58328, 25340, 43992,
                53728, 47276, 29672, 50596, 23664, 13756, 42120, 20648, 37956,
                48992, 56616, 45900, 58924, 12272, 17380, 46384, 54824, 13812,
                20992, 56620, 9184,  18104, 39556, 11748, 56128, 43388, 31072,
                65048, 51984, 20224, 23968, 8716,  49404, 36420, 19396, 2536,
                48272, 40044, 9464,  27468, 18900, 5652,  55360, 31816, 36576,
                12852, 23932, 1788,  61632, 8196,  42780, 3168,  12520, 20156,
                37884, 34124,
            },
            {
                64404, 42408, 56036, 29888, 58952, 34956, 53784, 4616,  28900,
                13608, 49600, 25032, 58192, 53332, 45472, 23584, 36536, 6700,
                30256, 12092, 53344, 48728, 6132,  58520, 41692, 17032, 25364,
                57500, 31408, 54084, 10684, 47844, 58700, 28688, 37536, 57564,
                2908,  11040, 51928, 39696, 55528, 44756, 30160, 4584,  34476,
                45580, 64120, 1036,  36304, 41952, 10352, 20128, 1420,  61004,
                29596, 58772, 18000, 3716,  51600, 30180, 2872,  34020, 5684,
                57756, 41908, 49792, 25904, 4056,  53180, 18168, 6284,  62020,
                1220,  37984, 16264, 5064,  36544, 46564, 312,   57968, 25988,
                6596,  10044, 18880, 34420, 39392, 63116, 22408, 33724, 26312,
                65416, 2612,  44732, 30328, 63784, 49180, 27184, 14384, 37256,
                5084,  11544, 35164, 60668, 13708, 26580, 56844, 42668, 32900,
                14736, 21520, 62072, 37072, 58544, 46624, 23000, 16396, 58892,
                8784,  54444, 34960, 13820, 17340, 59356, 33612, 28912, 58160,
                15908, 51404,
            },
            {
                31352, 17212, 36788, 48536, 9820,  25068, 41004, 22232, 38928,
                31564, 61336, 9920,  20264, 40628, 4464,  56540, 26396, 51160,
                18184, 59756, 35768, 27608, 15032, 23252, 51044, 62760, 47328,
                14248, 50028, 20316, 27320, 15320, 5156,  52424, 12536, 33352,
                24448, 30604, 63268, 19560, 35192, 952,   59448, 48412, 24728,
                13092, 55588, 28708, 5688,  23312, 56232, 27464, 49884, 33624,
                42620, 11292, 39620, 56248, 14692, 64156, 40520, 59480, 53100,
                16296, 8524,  62468, 18876, 31536, 39512, 35616, 49236, 20736,
                13020, 26240, 64008, 19996, 54836, 61320, 30408, 11488, 35136,
                60656, 42832, 24264, 1812,  54060, 30480, 8220,  50584, 41376,
                11352, 36212, 52820, 22896, 644,   33704, 61292, 45392, 22020,
                59476, 48480, 3556,  44672, 31976, 1396,  53232, 10636, 59292,
                29000, 54592, 44724, 1512,  12604, 41704, 29656, 2248,  40840,
                48520, 20040, 29864, 56796, 50092, 22036, 45660, 9636,  40872,
                24408, 2216,
            },
            {
                47512, 22972, 7156,  62756, 632,   46200, 15264, 65084, 2440,
                17664, 35468, 47200, 28004, 64376, 16964, 32820, 9512,  63312,
                43612, 21808, 4028,  56576, 38624, 11372, 30084, 6556,  33688,
                4656,  59880, 2616,  38884, 55824, 23796, 43564, 17884, 48212,
                54900, 15460, 5052,  49348, 9712,  42612, 21572, 15600, 60956,
                39012, 19632, 43428, 15936, 47260, 65420, 8156,  44984, 16816,
                5120,  63560, 23732, 32796, 44268, 21048, 11124, 47096, 23900,
                29496, 36284, 908,   46156, 60644, 10828, 56968, 28544, 42304,
                59956, 52096, 40600, 10704, 33156, 7164,  18048, 50964, 44772,
                15784, 29780, 64788, 49368, 5640,  14872, 44076, 60808, 4520,
                17040, 48548, 58372, 15084, 38344, 5664,  19104, 8668,  53528,
                24944, 29452, 41296, 18540, 64128, 38640, 16640, 23396, 5332,
                49620, 7512,  17500, 24624, 34992, 53068, 64888, 7220,  26328,
                61024, 44664, 3836,  40536, 26072, 1356,  36356, 63296, 49248,
                55688, 5604,
            },
            {
                60112, 13388, 52656, 27900, 19032, 55264, 32812, 50680, 57896,
                44688, 7308,  52644, 916,   13448, 38028, 48320, 2784,  39384,
                14424, 47736, 8288,  31968, 43104, 61600, 54492, 19172, 37404,
                22552, 42144, 29604, 45372, 63760, 35124, 2016,  61304, 7636,
                38620, 44148, 59016, 25328, 32528, 62380, 28108, 53000, 8948,
                2344,  26564, 62152, 53876, 32116, 12940, 38432, 57916, 22304,
                36244, 52388, 276,   49296, 7352,  26592, 37224, 1744,  13836,
                43924, 51112, 21740, 55088, 5480,  24084, 15588, 2884,  33932,
                8508,  31476, 3660,  48820, 43488, 24356, 39540, 63460, 22080,
                2808,  53176, 40420, 20808, 36840, 57940, 25316, 19476, 55316,
                32288, 28028, 7336,  42532, 26064, 51572, 57328, 40680, 2432,
                34180, 15896, 58552, 7272,  51136, 28472, 46336, 61248, 34720,
                39216, 64288, 32192, 57712, 47904, 9992,  21168, 50140, 33160,
                17932, 37960, 10428, 64504, 32260, 53780, 6264,  18924, 14540,
                27248, 43880,
            },
            {
                20380, 39792, 33340, 44156, 38072, 12192, 6000,  27004, 10760,
                21340, 55916, 24040, 42860, 30224, 58740, 22224, 54516, 27824,
                34080, 58272, 24700, 51456, 17332, 1700,  26132, 45592, 9176,
                65276, 52860, 16956, 7756,  11540, 19696, 41324, 51004, 27740,
                21672, 1532,  36256, 17656, 53976, 12932, 4092,  40948, 31508,
                36076, 50328, 11072, 3540,  35196, 19136, 2004,  26460, 54172,
                13220, 46632, 28176, 18996, 58020, 53536, 61000, 32052, 55844,
                64532, 10260, 33264, 27536, 41220, 44788, 65096, 47424, 55564,
                23420, 17172, 57300, 27516, 59440, 13996, 53840, 4480,  47472,
                33676, 12896, 8424,  27660, 46524, 10784, 34904, 1060,  38892,
                46224, 60204, 20832, 63028, 10356, 46704, 31356, 12948, 64452,
                49956, 10324, 54888, 36152, 21952, 12128, 4412,  42232, 19668,
                25920, 13424, 43244, 4440,  28444, 15800, 38500, 55748, 13676,
                5112,  52960, 24100, 15916, 47128, 12320, 61168, 39348, 31736,
                8916,  35784,
            },
            {
                54528, 10268, 4168,  57280, 60732, 23444, 48960, 62176, 37116,
                40908, 14676, 33496, 61968, 10984, 51308, 6092,  18620, 61624,
                11652, 496,   40800, 64716, 10444, 36164, 49568, 58464, 40108,
                12472, 24776, 32616, 60312, 49228, 26268, 31532, 14120, 56588,
                10632, 63840, 29076, 47076, 7236,  37896, 50032, 18156, 57052,
                46620, 21916, 59300, 41216, 24196, 49060, 43304, 61120, 6396,
                31580, 59864, 39024, 10104, 34976, 16616, 4536,  41112, 19876,
                6624,  39032, 58972, 17316, 12724, 36444, 7672,  19592, 51256,
                38060, 12316, 46040, 21544, 1288,  36720, 29336, 9632,  25744,
                38424, 59568, 55712, 17444, 61880, 52180, 29568, 64176, 9380,
                24092, 13124, 2960,  35568, 54632, 16032, 22744, 36740, 27552,
                19600, 43724, 212,   26148, 47752, 58048, 31456, 52476, 940,
                55612, 8776,  51020, 20100, 61628, 148,   43656, 30108, 62536,
                46068, 35296, 60188, 7628,  43160, 28860, 21696, 51772, 57780,
                988,   65160,
            },
            {
                29680, 50336, 25976, 16076, 2560,  30412, 41928, 17852, 1308,
                29772, 60140, 3788,  18212, 46176, 35420, 25420, 41848, 45648,
                30336, 52936, 19924, 46424, 29540, 22280, 5544,  15612, 28300,
                3284,  46944, 57048, 652,   37980, 53856, 4668,  62328, 34296,
                45036, 40500, 14584, 57540, 22336, 64836, 44640, 24440, 61596,
                5300,  16384, 29960, 7608,  53144, 63388, 29248, 10532, 40732,
                20548, 15812, 3928,  63876, 42952, 47556, 23372, 28700, 48624,
                15500, 25416, 2184,  49888, 53788, 61964, 31120, 26568, 740,
                42880, 62576, 34972, 7508,  64980, 50156, 16348, 57008, 61528,
                18716, 30724, 520,   41596, 4296,  22248, 43392, 16212, 49196,
                54016, 41048, 31588, 44552, 28528, 1108,  58964, 4508,  47236,
                61008, 38868, 32112, 62944, 15092, 8372,  37644, 63724, 16104,
                47104, 29964, 36528, 40588, 53452, 24300, 59092, 8860,  22804,
                2064,  28252, 49912, 20408, 55868, 37540, 4284,  25396, 41992,
                46688, 17856,
            },
            {
                5392,  41200, 62940, 20952, 45960, 52316, 35004, 9064,  54532,
                47960, 25964, 50228, 39344, 8260,  65316, 2032,  14756, 56940,
                7432,  36488, 15372, 4728,  60404, 56716, 43840, 34332, 61044,
                51700, 20144, 36044, 14696, 43752, 22264, 8936,  47264, 24416,
                18572, 6368,  51224, 2644,  33524, 11012, 29380, 416,   13776,
                33628, 43760, 55812, 38260, 14380, 4704,  17208, 51380, 35572,
                45744, 55064, 25472, 50636, 30156, 12944, 8728,  62936, 52088,
                34460, 61108, 45504, 29032, 4248,  21040, 40340, 14292, 60556,
                5780,  28840, 52260, 40016, 19100, 31772, 44376, 40968, 5220,
                51488, 45440, 24104, 49624, 33224, 13316, 56404, 6348,  36432,
                19852, 4864,  57904, 50088, 18500, 63648, 41884, 51176, 8192,
                23976, 13696, 5476,  52148, 20520, 45008, 23908, 27592, 10496,
                58824, 22480, 63112, 6620,  12120, 33788, 48908, 36904, 18736,
                42400, 57504, 13172, 33336, 568,   59276, 49580, 16248, 11756,
                34396, 24108,
            },
            {
                14668, 48072, 32124, 36968, 7704,  13872, 64740, 22488, 43464,
                12420, 5784,  56380, 21824, 28400, 53524, 32388, 38176, 49980,
                23636, 63468, 27364, 39660, 32892, 13200, 53280, 18884, 7556,
                31100, 42460, 10216, 27448, 64408, 17640, 59460, 39632, 1020,
                55496, 30980, 61016, 25928, 42788, 55836, 19636, 36988, 52728,
                9832,  64324, 27224, 19816, 46392, 33360, 57200, 22816, 924,
                62120, 7644,  34176, 21884, 1360,  59240, 37324, 43572, 2956,
                11532, 22400, 42036, 9132,  58096, 34952, 10348, 49492, 56204,
                32940, 15928, 10028, 25308, 54660, 12064, 2280,  22812, 35244,
                13912, 10332, 37472, 64632, 7812,  26968, 39572, 61092, 30852,
                26288, 62548, 14788, 9088,  25060, 37664, 11868, 31792, 16836,
                56660, 29220, 59596, 41484, 33592, 56492, 2552,  40728, 34840,
                44100, 4004,  14948, 26852, 45572, 17192, 3844,  55048, 11324,
                65492, 6868,  24996, 41212, 18152, 10032, 31180, 63752, 6544,
                52568, 60440,
            },
            {
                56208, 1992,  11404, 54784, 59776, 28020, 4532,  57020, 32300,
                19496, 62836, 35660, 42616, 11700, 16908, 58352, 20376, 9684,
                44172, 3364,  55384, 47804, 8980,  24804, 880,   38024, 48740,
                62628, 23568, 54700, 6292,  50668, 33544, 29820, 11764, 50052,
                36848, 21228, 46228, 16628, 39000, 7960,  49320, 60016, 25412,
                40336, 48444, 1972,  11284, 60616, 25708, 37660, 48200, 12716,
                28076, 42088, 57436, 14724, 39628, 52880, 19436, 26236, 57728,
                30804, 54352, 37036, 18032, 47380, 24148, 53276, 43844, 18852,
                23228, 47584, 58488, 4632,  46260, 61916, 28072, 48764, 63372,
                55324, 21012, 28748, 57888, 16680, 44684, 50860, 1404,  11460,
                46864, 51892, 34156, 39764, 6108,  46272, 21436, 53108, 34988,
                44628, 2108,  48784, 11092, 18216, 6060,  50904, 61620, 17744,
                53792, 32424, 49332, 56656, 60288, 30960, 40324, 26252, 51392,
                31436, 38488, 53888, 48500, 62196, 45564, 36460, 26892, 43068,
                21356, 37812,
            },
            {
                45048, 27404, 23112, 43188, 18960, 39808, 50808, 25372, 45716,
                38824, 15284, 30524, 184,   60380, 48700, 4992,  27056, 61928,
                13084, 33924, 21576, 17264, 51260, 42816, 64536, 27096, 11332,
                16300, 2744,  58600, 38508, 45736, 3512,  57364, 15388, 27024,
                65076, 8440,  12484, 53512, 3668,  63368, 31172, 45208, 17404,
                6020,  21628, 36028, 51740, 41656, 3524,  8748,  65004, 30940,
                53380, 19180, 10252, 47920, 64580, 32296, 7372,  16812, 46720,
                5896,  14404, 64812, 1708,  32224, 62876, 5036,  28288, 2580,
                36972, 64152, 41336, 20688, 33776, 37864, 8208,  17812, 32480,
                6676,  42336, 1900,  53136, 34484, 20272, 59416, 23828, 17920,
                42384, 2608,  22528, 55112, 61884, 30024, 58228, 4840,  64336,
                39664, 22204, 26588, 36684, 65496, 30592, 47532, 13048, 7016,
                21000, 39688, 1376,  35880, 19744, 8072,  63456, 14440, 46000,
                20996, 2916,  16052, 29168, 5136,  22564, 54232, 2152,  57160,
                29988, 8520,
            },
            {
                18292, 35044, 64236, 49408, 648,   33784, 16272, 10808, 2484,
                58848, 8568,  51324, 44904, 23864, 33588, 40248, 46920, 31080,
                53852, 42152, 60996, 2228,  58616, 30496, 20116, 56164, 45336,
                40408, 33216, 29092, 13512, 18812, 25044, 41164, 53024, 4600,
                43720, 33316, 59288, 28476, 35684, 23024, 14680, 1636,  54432,
                28676, 57084, 62136, 13904, 30020, 55160, 21320, 43632, 15680,
                38596, 5016,  26576, 44776, 4036,  23768, 55904, 34964, 61768,
                39380, 20484, 50600, 26664, 40960, 13352, 16868, 38948, 60104,
                8036,  12592, 30384, 104,   57332, 14760, 59892, 52448, 26308,
                46464, 59316, 14888, 47668, 9916,  4224,  37808, 31756, 55732,
                65108, 27800, 10640, 16648, 48916, 1524,  27088, 15056, 9904,
                18940, 55448, 8064,  52584, 14832, 23296, 38476, 26428, 58524,
                29556, 64744, 24732, 11096, 43032, 50056, 23184, 796,   59600,
                35580, 44252, 61228, 11744, 34072, 40068, 14572, 19652, 48340,
                13228, 62560,
            },
            {
                52288, 3424,  15392, 9848,  30040, 58332, 47176, 63408, 36372,
                53576, 20660, 26756, 63860, 6164,  13944, 52392, 1848,  18312,
                7224,  38516, 28612, 10968, 36612, 5988,  14336, 35076, 4764,
                53412, 21240, 47292, 60832, 8564,  35724, 63324, 46560, 22676,
                17344, 39844, 5400,  47396, 19220, 41632, 58460, 11528, 34224,
                43468, 9036,  32484, 24364, 45656, 17496, 59504, 5908,  32936,
                58080, 50736, 61876, 36116, 29500, 11592, 41736, 51220, 232,
                28184, 44484, 10740, 59716, 7312,  48864, 57292, 45444, 33692,
                53544, 49124, 26764, 44320, 50884, 24204, 40664, 3016,  12360,
                36616, 23432, 39852, 30192, 25900, 63652, 48704, 13096, 8104,
                35080, 45728, 60024, 32236, 41980, 19796, 36448, 43112, 49776,
                31232, 62352, 45908, 3992,  42564, 60664, 812,   44936, 9392,
                51596, 14264, 46432, 55716, 4424,  29048, 34272, 57072, 10240,
                27324, 7524,  55724, 24536, 51164, 64748, 8920,  59492, 32712,
                4808,  40688,
            },
            {
                28564, 57884, 39420, 53884, 24832, 7124,  41604, 21900, 28232,
                5660,  48140, 41128, 17280, 55988, 37300, 21488, 64800, 25232,
                57804, 15472, 50628, 24348, 46052, 41152, 61764, 49128, 9344,
                25444, 64212, 28,    42700, 51672, 20900, 2136,  10588, 30584,
                62004, 49844, 24456, 56356, 9488,  51780, 26820, 48924, 65364,
                39164, 19176, 5168,  49920, 32,    40000, 27360, 49088, 11388,
                24032, 1776,  21088, 16440, 59908, 49416, 14952, 22004, 9244,
                57212, 33116, 23840, 55172, 36140, 30432, 21936, 9976,  25500,
                19872, 15440, 55848, 6772,  18280, 9644,  29436, 43512, 64924,
                5108,  19028, 61468, 7032,  57096, 43108, 15836, 52772, 40636,
                3832,  20512, 38280, 6664,  52304, 12152, 56016, 60496, 24512,
                460,   13668, 34096, 29064, 54904, 19380, 34648, 56388, 31824,
                41236, 5440,  18332, 37420, 61696, 52364, 16612, 47576, 41800,
                19052, 49992, 37720, 18012, 3312,  42808, 26352, 45928, 37092,
                50676, 24376,
            },
            {
                7668,  19960, 45276, 35796, 61232, 19408, 52404, 14224, 31860,
                61456, 12644, 34936, 9620,  29008, 58808, 42704, 11504, 34712,
                44352, 3904,  63056, 32420, 17936, 53140, 22040, 28536, 59536,
                17488, 37252, 11776, 56764, 26772, 31964, 55872, 14624, 37692,
                54456, 472,   13208, 32180, 62676, 2600,  37576, 6832,  23560,
                15632, 52504, 57980, 36728, 63252, 9676,  52912, 35156, 63660,
                46376, 40484, 54144, 8432,  43744, 2620,  31620, 62880, 36880,
                47368, 16280, 3436,  42800, 18936, 772,   51188, 65360, 41636,
                2124,  61036, 32208, 39636, 62904, 35268, 53652, 20612, 48300,
                55492, 33508, 50972, 11816, 36292, 22108, 116,   29128, 24940,
                58756, 54160, 14508, 26212, 63848, 3444,  32952, 46468, 6976,
                38732, 58176, 41208, 21580, 11556, 48652, 4988,  16632, 22440,
                62764, 49640, 33384, 27452, 21536, 12812, 39064, 5700,  30400,
                64012, 13340, 32100, 46476, 58116, 29936, 20936, 188,   16720,
                11316, 60492,
            },
            {
                47588, 13640, 31996, 4184,  12100, 43704, 3300,  38428, 55404,
                1016,  44412, 23320, 53836, 2408,  31904, 7896,  49324, 27748,
                53304, 22364, 47452, 7972,  56320, 956,   12156, 33472, 3476,
                41892, 30636, 49464, 15532, 5068,  39388, 48136, 59796, 6784,
                27460, 18896, 43276, 36208, 16420, 44740, 20932, 31592, 60300,
                3408,  44608, 12308, 20300, 30872, 14664, 22604, 4288,  18228,
                29744, 13292, 33728, 27160, 38180, 57420, 25336, 53804, 19244,
                4860,  64332, 52268, 12972, 62144, 27156, 38276, 14500, 5772,
                47216, 36656, 22104, 11960, 46572, 1844,  58884, 31760, 15780,
                27292, 1324,  42440, 17468, 54704, 32244, 46528, 62588, 17588,
                49512, 10128, 30064, 47928, 23328, 40020, 16020, 21124, 52920,
                27656, 17400, 50760, 8760,  25864, 63544, 37192, 52980, 11916,
                25732, 1608,  60652, 8536,  42348, 3004,  62792, 24952, 53192,
                21956, 1612,  60460, 6508,  11224, 35284, 52108, 62988, 56032,
                41988, 30756,
            },
            {
                65116, 22444, 55560, 49292, 27780, 63732, 34364, 50164, 25228,
                18568, 39480, 60088, 50036, 20092, 45872, 16176, 60500, 524,
                37608, 10344, 14484, 40384, 27132, 65440, 38380, 44580, 51880,
                57336, 7000,  23760, 34724, 62864, 18284, 42312, 23924, 34184,
                45432, 61116, 7752,  57812, 28932, 53420, 11896, 47152, 54980,
                28080, 33520, 25572, 48176, 60476, 43196, 54960, 38132, 44500,
                7200,  56448, 48456, 65472, 18040, 5284,  45600, 12468, 40944,
                30208, 26036, 38740, 34280, 45976, 8708,  54124, 31864, 58220,
                28872, 52028, 3616,  57476, 27804, 23344, 13496, 5732,  38976,
                10284, 62688, 47336, 25520, 5268,  59624, 9772,  38140, 6176,
                33268, 42396, 2220,  59516, 44792, 8584,  29196, 63020, 11192,
                35804, 61592, 2844,  46212, 59160, 30196, 40764, 7908,  57928,
                44512, 38832, 15160, 53660, 56812, 45536, 32536, 9460,  36564,
                56452, 43848, 40032, 23608, 54376, 15192, 39588, 8324,  23352,
                34528, 2940,
            },
            {
                43440, 6128,  37080, 59564, 17292, 23428, 8140,  57036, 10928,
                65204, 30140, 7320,  13636, 38016, 63496, 24616, 40576, 19424,
                55424, 29868, 60008, 35176, 20428, 5504,  48352, 24128, 14284,
                19524, 61040, 45884, 53584, 9876,  29224, 3096,  12000, 53320,
                21408, 50840, 38908, 25260, 4852,  64248, 39952, 18140, 8552,
                41996, 63716, 2316,  39732, 7500,  28372, 1696,  58784, 25224,
                61056, 20452, 556,   10848, 23424, 51888, 35092, 7268,  60552,
                49080, 10080, 58452, 21796, 5264,  60692, 17044, 23384, 43936,
                10952, 63240, 16188, 42540, 49700, 36964, 63708, 51484, 45300,
                57452, 21200, 30828, 40284, 14448, 50480, 20012, 44648, 56644,
                22660, 64624, 36492, 18100, 54540, 34236, 57508, 48760, 4400,
                44324, 23548, 55664, 33168, 15240, 1192,  18752, 47688, 28784,
                34132, 19944, 48168, 30232, 23480, 17676, 59532, 47944, 15140,
                4276,  28800, 17068, 49280, 27104, 61460, 5028,  48172, 27816,
                18544, 52996,
            },
            {
                15876, 26160, 10296, 40864, 504,   47080, 31020, 15844, 45584,
                4640,  48732, 33732, 26320, 55208, 5668,  51852, 12188, 32756,
                64224, 4592,  43896, 51544, 57484, 16196, 31004, 55236, 9704,
                28056, 39072, 1228,  21492, 43676, 58820, 49308, 65264, 16004,
                1820,  31728, 10936, 15104, 48632, 23212, 804,   35388, 59100,
                15128, 49768, 10836, 56108, 17324, 33848, 50068, 10672, 15520,
                35728, 52212, 42608, 32568, 40116, 58676, 28540, 16708, 22492,
                43988, 1008,  15204, 50716, 29848, 41104, 48404, 35624, 7304,
                20628, 26028, 34448, 4968,  19416, 8356,  41428, 17184, 24136,
                35036, 7996,  53128, 2780,  65480, 28324, 35152, 1964,  26440,
                15384, 12060, 51640, 5704,  21640, 13008, 1444,  41880, 19508,
                31444, 14008, 7400,  38148, 43200, 52068, 24284, 64840, 13748,
                3196,  59676, 6820,  10884, 35976, 252,   27776, 41460, 20392,
                51408, 33844, 65236, 8904,  37284, 19568, 32532, 44668, 12968,
                58684, 38676,
            },
            {
                49852, 61704, 29896, 20668, 53184, 58348, 37580, 42104, 28376,
                21976, 57932, 17088, 43444, 3032,  35548, 28684, 44944, 7348,
                48656, 16724, 23268, 2040,  10976, 42260, 36352, 60456, 2500,
                33756, 64572, 12352, 31164, 37564, 7700,  25672, 35968, 40804,
                28780, 62492, 55044, 42192, 33144, 60216, 51360, 30128, 25104,
                5316,  21016, 36384, 23944, 45308, 64668, 20016, 41732, 31472,
                46592, 4096,  28124, 62032, 14472, 47140, 2256,  64016, 37048,
                54096, 32640, 19552, 55972, 25560, 12276, 1304,  63980, 56532,
                40040, 46304, 55040, 30220, 61472, 56488, 32832, 800,   28604,
                61192, 13232, 37504, 18028, 56092, 42904, 11656, 61136, 53924,
                41096, 47308, 27596, 31948, 39364, 61172, 24964, 37096, 53500,
                65072, 49896, 26480, 20372, 62408, 11060, 35836, 4612,  56168,
                41820, 24852, 52196, 64284, 40264, 50376, 13072, 63220, 7584,
                59012, 11456, 45652, 856,   57604, 41560, 3104,  51332, 63912,
                9404,  1540,
            },
            {
                32392, 54800, 44448, 13832, 34028, 5572,  12232, 62956, 2756,
                53484, 40360, 9168,  62360, 22276, 59484, 15456, 57016, 21196,
                38980, 26748, 34104, 47332, 29132, 63244, 7592,  21592, 50460,
                41512, 47132, 17304, 55104, 50908, 14528, 20184, 56144, 6024,
                47560, 18460, 3908,  46264, 8400,  19888, 13164, 43876, 53608,
                38772, 57496, 31424, 52092, 13380, 5856,  27112, 53468, 62912,
                8020,  22948, 17420, 54428, 9020,  20972, 31268, 50172, 8528,
                26796, 59544, 39476, 3684,  44528, 33864, 51824, 19304, 13964,
                2856,  50588, 9756,  14512, 44316, 25492, 11644, 47840, 54120,
                5200,  43912, 48932, 23676, 32800, 6496,  21784, 49824, 30404,
                8452,  57352, 608,   63108, 44368, 50488, 15560, 28296, 5928,
                10060, 40036, 58764, 4008,  54020, 31944, 45604, 21612, 50440,
                37104, 31240, 16252, 44660, 20192, 5172,  54732, 31168, 37936,
                26300, 22692, 53520, 30096, 14168, 25100, 55244, 20912, 29180,
                35668, 24056,
            },
            {
                19240, 7980,  3640,  64660, 50208, 22868, 26544, 48292, 19320,
                35740, 12892, 50644, 30356, 46728, 10492, 41644, 1336,  50248,
                58916, 9292,  62028, 53172, 13864, 25612, 45968, 17916, 53968,
                25108, 5528,  58344, 27144, 4200,  61216, 33004, 43984, 10008,
                58300, 37136, 22316, 27240, 56376, 38020, 3336,  63476, 10208,
                46612, 16416, 60896, 540,   40372, 58292, 36976, 2664,  14108,
                39076, 57244, 48852, 37468, 4636,  44160, 56292, 12384, 42340,
                4448,  13488, 47780, 64808, 9816,  58948, 23724, 28360, 37436,
                31748, 62252, 23096, 38612, 3176,  52556, 21372, 64688, 39208,
                19368, 59580, 26936, 10068, 62348, 47120, 16552, 38860, 3464,
                19248, 34880, 23784, 17060, 10432, 7152,  56592, 46948, 60308,
                18004, 34808, 47360, 29244, 15848, 8200,  60568, 27572, 17272,
                8876,  61936, 2024,  26972, 57808, 34472, 24224, 45328, 3944,
                48892, 42364, 18512, 35372, 62656, 43956, 7600,  16360, 39996,
                56980, 46456,
            },
            {
                41812, 59812, 36108, 27964, 16860, 42840, 60904, 8416,  32212,
                59124, 25344, 304,   38228, 18700, 53768, 31836, 24328, 35232,
                13148, 43356, 5600,  19724, 40164, 58548, 4716,  38220, 30376,
                14844, 9092,  35672, 22348, 41040, 46584, 580,   28208, 23728,
                51728, 12964, 34052, 64732, 15892, 50444, 28868, 22768, 33596,
                4500,  26032, 8472,  28772, 48848, 21820, 9952,  47664, 24724,
                29516, 60324, 11328, 33944, 26680, 63060, 39144, 24384, 18776,
                61512, 35540, 22032, 30808, 17228, 38172, 6048,  53740, 45516,
                58336, 6564,  18420, 48868, 59072, 35996, 5920,  30136, 14880,
                34348, 3272,  52100, 40692, 456,   36320, 58656, 25480, 64420,
                45724, 40308, 59164, 53248, 30048, 35984, 20576, 2176,  31032,
                23148, 12992, 156,   42216, 24548, 48996, 39424, 5612,  44284,
                53416, 12244, 47608, 38600, 14272, 61572, 9736,  17776, 57296,
                14932, 2180,  59592, 5876,  48040, 10844, 32152, 60156, 49468,
                5520,  11956,
            },
            {
                15180, 23060, 47956, 39868, 9700,  55924, 1264,  38492, 54344,
                15604, 44024, 63984, 7132,  27092, 60412, 4436,  65056, 17992,
                29532, 55352, 37432, 31524, 256,   49316, 34456, 11872, 57076,
                63728, 44596, 49024, 62532, 10420, 18920, 54176, 64192, 16168,
                39268, 60120, 1312,  41708, 6640,  45060, 59540, 17284, 41092,
                55572, 62744, 43004, 35204, 17944, 61644, 32924, 55304, 42276,
                19024, 1548,  44896, 22172, 51248, 16932, 1320,  33668, 52756,
                46288, 28340, 51684, 7504,  42444, 49380, 62708, 1840,  16424,
                10912, 26656, 42828, 32960, 27924, 16544, 41080, 46492, 8744,
                57524, 22456, 31440, 15516, 54796, 28496, 12860, 52840, 32256,
                9912,  13860, 5444,  47832, 26208, 43536, 63968, 40444, 54892,
                45120, 51656, 57376, 64272, 19232, 55224, 13324, 34012, 58308,
                19812, 24000, 32328, 54384, 7168,  29872, 41684, 52788, 35884,
                64484, 32476, 39720, 21768, 26844, 52352, 37752, 22560, 428,
                27436, 63336,
            },
            {
                31264, 54292, 1828,  20148, 51532, 33032, 13592, 45376, 28948,
                5620,  21024, 52296, 34288, 47848, 13360, 37016, 45528, 52112,
                3012,  60092, 22608, 15536, 64528, 54604, 23556, 43448, 3484,
                20488, 32592, 1656,  29376, 38608, 13460, 35280, 49632, 4760,
                30344, 8908,  48360, 25156, 32024, 11576, 36184, 960,   48396,
                13576, 20332, 51132, 11952, 54088, 44176, 3816,  16148, 65040,
                50392, 31812, 5672,  55020, 13844, 59332, 30260, 6512,  56832,
                10100, 15580, 152,   57672, 25712, 13188, 20480, 30368, 41840,
                35248, 51264, 64096, 296,   9412,  60924, 55580, 24644, 50536,
                42648, 11904, 64268, 45544, 20676, 8280,  44012, 5072,  22808,
                50880, 62680, 21180, 38248, 3448,  14780, 11172, 33660, 25820,
                5020,  38080, 9692,  32364, 36912, 3216,  61576, 30016, 756,
                41332, 65288, 4316,  43504, 22072, 49424, 980,   20600, 28188,
                12096, 50764, 8828,  55944, 16936, 3516,  64108, 13752, 44876,
                34588, 51896,
            },
            {
                38580, 26084, 7240,  58976, 29480, 63780, 18604, 23376, 49696,
                60512, 11308, 41292, 56560, 16392, 22792, 40836, 6160,  11788,
                26096, 48152, 8444,  45188, 27288, 10340, 17224, 61300, 28168,
                40300, 52976, 16752, 55652, 24228, 59464, 7952,  42228, 20516,
                45284, 55740, 18768, 53036, 61360, 21152, 57444, 52104, 27528,
                31224, 37668, 2364,  24360, 6612,  30252, 26432, 36260, 7680,
                12496, 39596, 61944, 35772, 8264,  41444, 48432, 65180, 20844,
                40504, 44464, 63428, 36388, 54312, 33012, 46164, 60148, 23996,
                56096, 14084, 20944, 53956, 44604, 13132, 20284, 1740,  61676,
                28256, 37360, 6800,  49400, 34848, 61912, 57028, 37476, 15924,
                42392, 28796, 34200, 54480, 61060, 49488, 57920, 7856,  16540,
                60700, 21188, 14944, 27172, 7140,  46792, 22460, 51056, 25916,
                36508, 49808, 10188, 60264, 16744, 37024, 62684, 46516, 6752,
                24120, 44416, 29624, 61608, 36316, 46808, 29028, 40640, 54968,
                18648, 9252,
            },
            {
                47012, 56920, 13076, 44836, 35664, 4684,  41664, 57576, 3392,
                37148, 30596, 24748, 1980,  8364,  50100, 57828, 30656, 62304,
                20824, 39532, 33876, 51840, 41752, 6332,  32868, 47460, 51212,
                8376,  12928, 60752, 43044, 4432,  51156, 27060, 32176, 62700,
                24476, 36348, 13572, 29228, 4100,  39964, 8676,  24160, 65324,
                5828,  46324, 56344, 39788, 63624, 14344, 57732, 46856, 23108,
                59076, 27544, 19980, 47420, 23788, 28420, 11264, 37200, 25936,
                4128,  31344, 23300, 18428, 3088,  10560, 38848, 8056,  3560,
                47580, 40096, 5244,  24892, 37908, 49780, 31652, 35168, 4800,
                17528, 55828, 25852, 2292,  17892, 24400, 30004, 3320,  48664,
                55880, 1100,  8780,  17836, 31204, 24632, 20056, 46708, 52932,
                29364, 40976, 50072, 62344, 53476, 42856, 16920, 9120,  55792,
                13992, 18340, 27672, 34300, 55596, 25448, 11028, 58480, 38896,
                55444, 15348, 4796,  41844, 12392, 20052, 57440, 6732,  24660,
                59724, 2680,
            },
            {
                20752, 33124, 61956, 16560, 24648, 11024, 47624, 27080, 9332,
                65372, 17636, 46400, 32804, 62964, 27624, 18972, 34920, 43080,
                54816, 14504, 1960,  62668, 19336, 58840, 38776, 1124,  22024,
                36592, 26476, 6204,  34628, 21728, 46356, 15760, 3316,  11668,
                58576, 2196,  42912, 63988, 46788, 15552, 33656, 43500, 18376,
                10816, 60180, 16456, 33240, 45452, 20784, 51860, 38632, 196,
                53312, 43168, 14868, 2360,  57976, 44084, 18560, 50944, 14544,
                54848, 60320, 38480, 48928, 28696, 58788, 50684, 65028, 19204,
                29644, 10184, 33716, 59360, 28640, 7260,  65292, 39980, 53400,
                14112, 44420, 58776, 39796, 52324, 11084, 41488, 60688, 19264,
                26000, 58956, 46072, 12648, 39224, 2412,  42108, 36120, 65348,
                1268,  44532, 4140,  23616, 34488, 11848, 39828, 62772, 44900,
                6252,  59004, 40380, 2100,  45420, 14700, 30644, 3508,  33464,
                19184, 59824, 49144, 22920, 53364, 1732,  33808, 10568, 50620,
                15568, 42852,
            },
            {
                28328, 3872,  41280, 50008, 37928, 60228, 55032, 15720, 33740,
                52892, 42456, 12996, 58748, 38724, 53340, 10424, 576,   49612,
                7576,  28712, 56948, 11992, 30612, 24856, 53772, 15072, 64932,
                57236, 17904, 48476, 30360, 63808, 37892, 57528, 53764, 40168,
                48644, 17564, 33088, 7564,  26752, 50856, 56504, 2728,  36640,
                52572, 22508, 28152, 9208,  59260, 3536,  11036, 32412, 18468,
                9448,  30840, 63168, 51664, 32696, 5292,  62336, 1084,  34444,
                45480, 8768,  12568, 5128,  43972, 21712, 15004, 26740, 36664,
                57156, 52244, 62768, 14900, 45972, 18060, 11344, 23644, 47808,
                30548, 21560, 9764,  32416, 63364, 14792, 46396, 32916, 6876,
                36048, 40512, 22416, 51400, 63440, 56860, 6488,  26836, 13480,
                32240, 11320, 59372, 18260, 30456, 58024, 1832,  28468, 33180,
                21664, 47880, 31088, 53132, 7676,  64388, 50104, 42932, 52984,
                26904, 4,     35404, 31328, 65164, 26424, 43856, 61048, 30660,
                36868, 64024,
            },
            {
                11296, 52992, 22708, 8720,  0,     31336, 20616, 40072, 1860,
                22244, 48872, 6568,  20404, 3780,  44308, 15120, 25152, 59428,
                17308, 37988, 23748, 47288, 36380, 4068,  43836, 10824, 29752,
                45428, 41264, 54720, 10308, 360,   12720, 19060, 34376, 27752,
                9400,  52152, 22724, 59608, 38216, 19904, 12268, 61152, 48124,
                32056, 40956, 1132,  49904, 35832, 25276, 41616, 64348, 48296,
                56260, 24564, 6364,  37496, 16856, 55736, 39848, 22944, 59052,
                27472, 52500, 17376, 61588, 55524, 34692, 41008, 752,   44708,
                16896, 4456,  21620, 41440, 1224,  54488, 57900, 42380, 6584,
                62520, 36376, 4252,  48440, 28332, 1232,  22140, 53572, 10064,
                13892, 62160, 4752,  27652, 34684, 10720, 50256, 17616, 55544,
                22136, 48096, 37768, 54792, 5872,  49284, 19656, 38120, 52492,
                3144,  63484, 24208, 12576, 20252, 39412, 23072, 17008, 8588,
                47572, 62636, 10732, 40296, 7468,  16056, 47812, 38904, 19624,
                5008,  48508,
            },
            {
                35544, 56276, 18096, 64756, 28140, 52312, 7084,  45044, 57056,
                29344, 62452, 26596, 55220, 35508, 29632, 64780, 40656, 46576,
                33100, 63236, 51072, 6008,  16536, 61156, 49516, 34144, 6992,
                23936, 3092,  61440, 20924, 51940, 43864, 25048, 6736,  45896,
                61708, 37336, 13184, 54996, 84,    41944, 24684, 29320, 6980,
                55196, 13152, 63332, 43320, 18908, 53900, 13916, 28100, 4856,
                34944, 40860, 11964, 46188, 26104, 9524,  29732, 49032, 7408,
                19788, 32772, 41400, 24180, 30488, 7072,  52856, 11384, 60776,
                31300, 48940, 26164, 36060, 50704, 32724, 20036, 2868,  27620,
                51476, 12984, 60172, 19348, 43460, 56516, 36916, 65232, 49072,
                44140, 31644, 54336, 15696, 43280, 20968, 30244, 45176, 61692,
                40176, 8656,  26220, 15224, 43252, 64860, 24116, 12444, 57312,
                16284, 9720,  43408, 36472, 59456, 29236, 5088,  60900, 36864,
                13848, 21896, 44872, 18516, 58476, 51368, 23368, 2508,  54588,
                14012, 25376,
            },
            {
                6616,  39464, 31896, 46480, 42640, 14464, 61400, 34660, 11628,
                4504,  37740, 16480, 9096,  47972, 22616, 52368, 6272,  20240,
                11280, 2932,  42728, 21132, 39944, 55296, 26412, 19956, 58536,
                38944, 13408, 33304, 27072, 39600, 59740, 31608, 65248, 1500,
                19784, 29824, 5176,  48008, 31672, 9528,  64456, 45180, 16216,
                20892, 26652, 4756,  56468, 30664, 7528,  61160, 44880, 16192,
                21716, 60696, 49692, 19584, 63852, 53432, 12960, 36500, 43380,
                3828,  64144, 47756, 1544,  14020, 63200, 19580, 47464, 23628,
                8368,  42864, 12584, 58744, 8884,  24152, 63712, 38164, 16472,
                45184, 24716, 54200, 39044, 8912,  26248, 5864,  17544, 29172,
                24220, 19684, 56,    47976, 8284,  59424, 37352, 2140,  5132,
                34056, 51876, 62596, 372,   36084, 27916, 7728,  47600, 41860,
                35332, 26840, 50396, 56304, 720,   47192, 54708, 32608, 41432,
                57552, 30168, 54396, 3784,  37344, 12464, 29084, 9256,  62416,
                44780, 59136,
            },
            {
                51232, 15324, 2992,  57800, 10920, 23512, 48472, 17924, 25116,
                50700, 59132, 41448, 32540, 61252, 12528, 2372,  37216, 30948,
                53696, 27312, 58008, 9728,  32268, 13068, 2572,  62900, 46252,
                17076, 53592, 5232,  47528, 14996, 8680,  50152, 16244, 54524,
                42976, 24240, 60336, 44432, 15364, 57304, 35692, 3892,  39820,
                59772, 46860, 37744, 15508, 23332, 48792, 1464,  37220, 51428,
                57556, 2540,  29200, 33824, 5032,  41576, 21864, 61112, 15972,
                56728, 11128, 35912, 51152, 39256, 27556, 56972, 33276, 37784,
                55148, 64496, 3028,  39836, 29768, 14304, 48592, 34612, 56456,
                8404,  31040, 2424,  34112, 15596, 59748, 51372, 41000, 3244,
                57956, 61296, 38352, 33044, 64460, 23720, 52596, 14000, 58056,
                24752, 19572, 31156, 46160, 10544, 53952, 60232, 30680, 4564,
                61384, 19400, 5956,  32048, 15292, 18804, 26264, 11632, 2120,
                24464, 5760,  50332, 27028, 63420, 32872, 56800, 40736, 34264,
                21448, 29840,
            },
            {
                10168, 61516, 20544, 26612, 36780, 5436,  54864, 38712, 64560,
                30964, 14036, 684,   44960, 18480, 57488, 26456, 48832, 61800,
                14328, 45056, 35440, 65280, 52332, 29416, 41476, 8600,  35920,
                50120, 30428, 64232, 36692, 56608, 2632,  22452, 35700, 39336,
                9964,  50928, 34604, 18092, 27284, 52956, 22128, 50412, 32452,
                10392, 52732, 34152, 9028,  58900, 40220, 32788, 11188, 25652,
                42412, 8700,  14708, 38652, 59328, 884,   45796, 31852, 51480,
                24968, 29160, 20868, 58844, 9116,  45416, 2568,  15904, 5628,
                28408, 20288, 51676, 17408, 61844, 4696,  43372, 11056, 60516,
                21272, 41116, 64672, 50076, 23168, 45892, 32084, 13060, 35620,
                9080,  50296, 12260, 26280, 5924,  17976, 40820, 28440, 48692,
                16412, 6680,  41480, 58680, 21720, 17684, 39560, 14280, 22680,
                54436, 45696, 39884, 65064, 9304,  42628, 62080, 51172, 45168,
                63924, 34800, 14752, 43084, 20080, 1128,  45792, 52320, 17552,
                3396,  42284,
            },
            {
                24060, 33696, 49000, 41248, 63224, 32496, 1440,  43336, 7552,
                20180, 47384, 23800, 54300, 6788,  38924, 43176, 16676, 55804,
                7964,  24520, 17540, 144,   22912, 47872, 15932, 56156, 25216,
                1256,  10880, 19712, 24276, 41960, 29660, 46764, 58580, 26928,
                14052, 3020,  64104, 7768,  40896, 2384,  13012, 62544, 25536,
                18380, 820,   65032, 44204, 27264, 19900, 62980, 54392, 18252,
                31348, 65468, 54932, 24420, 50740, 17940, 27304, 9164,  2920,
                38408, 46408, 5552,  16960, 32252, 22716, 42388, 61992, 49272,
                13420, 34828, 44980, 32140, 55856, 26452, 52616, 888,   28836,
                47552, 6224,  18220, 57032, 10892, 724,   62652, 21364, 55548,
                46404, 16780, 41836, 56912, 45488, 54744, 11088, 35240, 43828,
                64696, 54460, 12524, 49948, 34784, 3492,  51948, 44240, 33236,
                1260,  11604, 28788, 23824, 52828, 34432, 37776, 7340,  20364,
                16820, 39740, 9804,  59616, 48804, 25184, 15492, 5816,  27212,
                65012, 47168,
            },
            {
                53756, 708,   55956, 8360,  18704, 51716, 15220, 28096, 58168,
                10088, 52256, 35900, 27980, 64304, 31676, 9824,  22392, 34328,
                3920,  41156, 49988, 37860, 6444,  59860, 43420, 32948, 21668,
                39912, 59152, 44396, 6036,  61784, 18352, 11804, 4976,  62208,
                32920, 55164, 21800, 37764, 59024, 30888, 46144, 6084,  42572,
                57984, 29592, 22384, 5564,  51696, 14256, 3132,  46024, 6380,
                36016, 47704, 20732, 44312, 7028,  57268, 35820, 64520, 53776,
                14160, 61532, 40920, 54620, 65336, 12288, 52208, 25924, 39128,
                58532, 108,   23316, 7936,  47020, 19460, 39232, 15700, 33568,
                53644, 13368, 36532, 27272, 42516, 29684, 38200, 52420, 25640,
                5420,  28908, 34804, 20836, 3432,  31192, 62172, 1600,  9136,
                26332, 37868, 2476,  29396, 24044, 64196, 7056,  26228, 49428,
                63180, 17020, 56736, 46964, 2544,  21684, 14164, 29088, 54004,
                46904, 31132, 56120, 6416,  36160, 11112, 61324, 38252, 57720,
                12636, 36504,
            },
            {
                28920, 17300, 13244, 45292, 29856, 59840, 22372, 46140, 34260,
                39568, 60880, 3936,  12068, 20760, 50352, 1400,  58768, 45824,
                63804, 30352, 54712, 61332, 27904, 19268, 11676, 3740,  63588,
                51424, 28068, 54876, 13920, 49364, 34752, 52300, 41028, 19912,
                45260, 48392, 25844, 11312, 49564, 19328, 56060, 37312, 14672,
                49208, 39104, 12372, 47864, 35860, 57412, 28856, 23144, 60772,
                13452, 1372,  27868, 10576, 32620, 13828, 47868, 20064, 42488,
                23284, 33868, 700,   26816, 47324, 6304,  35308, 18256, 9408,
                53824, 41924, 60972, 11580, 36156, 5768,  59000, 22784, 63536,
                25240, 44468, 61076, 4928,  48696, 15260, 7656,  18024, 43272,
                60284, 1456,  63740, 50848, 14656, 39020, 47456, 22600, 51496,
                33232, 20340, 59852, 44612, 15116, 56288, 37444, 10328, 59128,
                20928, 35188, 38504, 6424,  60152, 49984, 57460, 3700,  60740,
                25792, 704,   22776, 41976, 28828, 54820, 21992, 32200, 49800,
                19844, 7836,
            },
            {
                39880, 63788, 35552, 25332, 3940,  37616, 12164, 56124, 2888,
                25872, 16768, 43944, 55992, 40756, 14592, 36896, 52484, 27676,
                12400, 20292, 9284,  14604, 45368, 35116, 57472, 48540, 7360,
                15376, 35560, 9212,  31924, 664,   26304, 56364, 8940,  29968,
                4000,  15676, 42296, 636,   61948, 28644, 9260,  23152, 33976,
                4132,  54316, 60292, 17780, 40912, 9656,  33424, 43588, 50532,
                39612, 53564, 59848, 41392, 62848, 37932, 5720,  29828, 10888,
                56268, 50116, 8468,  19736, 37624, 29424, 59420, 3440,  21860,
                31280, 27356, 16512, 49964, 65148, 29612, 42736, 49312, 8312,
                38224, 2864,  31720, 20812, 55760, 64164, 33372, 58112, 11484,
                31452, 39812, 9972,  24620, 58208, 27628, 18872, 60560, 13704,
                57204, 5536,  39972, 8880,  47748, 32132, 18772, 41180, 29736,
                3344,  43312, 14576, 26976, 31256, 10812, 43964, 33532, 39088,
                12448, 65300, 51116, 17516, 62460, 46580, 8176,  41304, 1580,
                44248, 60184,
            },
        },
        {
            {
                60984, 9512,  58828, 20648, 49556, 44676, 2276,  39772, 12304,
                22228, 4764,  63704, 38728, 53128, 47596, 40348, 33236, 56428,
                4824,  62752, 37200, 41588, 456,   45196, 9328,  33800, 60004,
                16680, 37664, 13328, 34124, 49944, 40944, 6080,  32236, 58632,
                53596, 64904, 25964, 49404, 5740,  17704, 29216, 42356, 26032,
                15972, 8,     48628, 10184, 34900, 38876, 44656, 8892,  27956,
                41500, 6580,  54960, 33220, 63448, 26596, 11020, 17440, 6536,
                62500, 31640, 51032, 5852,  23704, 42532, 15120, 56460, 18808,
                39844, 35444, 16832, 43800, 30948, 21620, 35172, 39388, 17224,
                28288, 58452, 14444, 48264, 1276,  29592, 4912,  55708, 39560,
                50124, 62868, 56652, 22476, 33084, 13368, 5536,  42128, 34332,
                12008, 17956, 65420, 23824, 48896, 59868, 2316,  9672,  46296,
                64232, 21012, 43312, 58968, 8264,  21956, 61316, 34592, 52264,
                45076, 22408, 56424, 39780, 11696, 25672, 57296, 20936, 45208,
                50520, 24788,
            },
            {
                1944,  43808, 12888, 32476, 64864, 5620,  24096, 62692, 56272,
                34224, 45960, 27648, 16,    12892, 22460, 64532, 2804,  26908,
                45752, 22272, 53432, 25628, 50256, 61072, 21088, 53608, 3048,
                24876, 45876, 57844, 21036, 55060, 25892, 9756,  47692, 13840,
                3368,  44668, 31104, 59644, 1992,  56800, 34612, 47488, 9408,
                59108, 36752, 41792, 19516, 56252, 1876,  22528, 54124, 36168,
                65356, 12748, 24404, 3240,  19432, 41064, 50192, 35224, 56440,
                40124, 27192, 44392, 59496, 54752, 2036,  37660, 50032, 31464,
                53496, 60428, 248,   51720, 40624, 57244, 1964,  24688, 55536,
                46500, 2688,  42160, 34352, 60220, 16204, 43068, 24948, 18948,
                33688, 26144, 4652,  41360, 61296, 50616, 37888, 63800, 20172,
                26424, 52388, 4752,  44944, 32448, 13912, 37552, 24940, 50340,
                30572, 212,   35384, 27408, 45852, 1928,  29656, 39452, 8940,
                27060, 36636, 62504, 19856, 53468, 32432, 3144,  16020, 7664,
                36344, 31084,
            },
            {
                39752, 52908, 26468, 7104,  41188, 53672, 35736, 47492, 15796,
                7984,  41556, 60420, 18192, 36792, 30596, 8908,  49500, 15368,
                38616, 60564, 13724, 10012, 18160, 4464,  36604, 40940, 11776,
                62060, 31160, 8212,  3968,  43308, 17748, 61516, 52080, 23512,
                39784, 8688,  36712, 13380, 45656, 40144, 21044, 64148, 4228,
                33208, 55388, 13372, 64696, 25644, 46804, 60624, 15420, 4280,
                20996, 47344, 43216, 57320, 52124, 14628, 2412,  30136, 21216,
                12832, 980,   9748,  16720, 32888, 20328, 46364, 7736,  11444,
                22680, 5084,  27304, 19948, 7556,  14944, 62152, 43268, 32872,
                9652,  19828, 62992, 23616, 11512, 51028, 35644, 63976, 13524,
                7720,  53244, 46160, 11536, 2716,  17868, 23964, 156,   48044,
                59460, 36468, 41300, 10024, 20912, 53780, 28104, 58324, 17936,
                13216, 60244, 52724, 15376, 62952, 55068, 13608, 18404, 63736,
                4984,  16220, 46868, 29064, 5720,  60724, 48976, 40856, 27684,
                62736, 19012,
            },
            {
                48460, 21896, 57888, 17132, 29748, 11572, 19652, 3432,  31700,
                52484, 20616, 49600, 6308,  44548, 61208, 54868, 20136, 42672,
                30104, 2072,  34644, 48152, 63996, 31784, 57072, 27128, 48892,
                18980, 52520, 40096, 63488, 29096, 36284, 1428,  33444, 19860,
                29420, 56368, 18428, 24072, 53132, 27404, 11372, 51040, 17200,
                45236, 22212, 7340,  50368, 4836,  31076, 11016, 40380, 49812,
                58572, 30024, 35328, 7828,  28404, 37416, 46040, 64968, 42612,
                52528, 57764, 37184, 48672, 40580, 26732, 62988, 29172, 57912,
                44168, 48316, 32932, 65212, 37228, 49084, 29336, 4920,  13436,
                50064, 37376, 30652, 53768, 8384,  27088, 57620, 2516,  48372,
                42580, 29108, 20216, 36220, 31188, 54244, 27868, 56488, 8344,
                15048, 30192, 2920,  55984, 62360, 34928, 5980,  44348, 39508,
                7888,  47988, 23900, 4028,  37816, 49208, 25072, 42748, 57548,
                33336, 49980, 1468,  42108, 14304, 37656, 22836, 58592, 12000,
                55944, 6108,
            },
            {
                33344, 63904, 2700,  36892, 50920, 61756, 56992, 43664, 25224,
                37968, 10916, 29860, 57780, 14848, 34884, 25376, 11472, 59168,
                51076, 7540,  57948, 28236, 43656, 15168, 23120, 7584,  44592,
                1848,  35292, 22600, 15340, 47392, 12340, 59312, 45516, 64084,
                5440,  49060, 62512, 33832, 7508,  60448, 2936,  38192, 25348,
                29964, 53484, 39116, 27532, 43600, 18232, 52016, 33748, 26372,
                412,   17040, 11500, 62972, 22552, 59256, 9256,  25220, 4360,
                33408, 19072, 24460, 64336, 7000,  13384, 51532, 3416,  17584,
                36384, 13732, 54760, 23620, 10976, 45248, 18908, 52396, 59824,
                22516, 57068, 6324,  17456, 38644, 46224, 5816,  31440, 22736,
                55064, 59924, 15692, 65060, 48700, 43956, 14168, 39528, 33468,
                45440, 61540, 24812, 12780, 47512, 16904, 1164,  63688, 21344,
                56508, 32352, 41760, 19156, 11160, 31756, 6436,  51884, 23072,
                11912, 30460, 55464, 25276, 64464, 9348,  46324, 34300, 1420,
                44824, 15104,
            },
            {
                28020, 11048, 42116, 45976, 22820, 8644,  28112, 13972, 59052,
                63060, 1628,  54316, 23248, 39688, 2544,  47028, 5120,  33484,
                17536, 24088, 40600, 20752, 6092,  38776, 52804, 12980, 65300,
                28708, 58392, 10412, 50684, 56756, 6604,  24984, 38072, 15436,
                42740, 11948, 660,   41372, 47860, 15296, 32180, 55720, 61552,
                12268, 1332,  59692, 15940, 35952, 62212, 23088, 7980,  63884,
                44860, 55296, 39536, 48360, 1700,  18384, 54452, 49088, 16036,
                61568, 45056, 30760, 3792,  55256, 35632, 43048, 24884, 61732,
                40088, 8756,  58764, 42264, 1544,  60748, 25952, 34660, 39600,
                3024,  27528, 43388, 49188, 65416, 20888, 41480, 16468, 37976,
                10560, 1020,  39884, 6600,  21588, 9384,  58976, 3844,  52168,
                22128, 17504, 50548, 39056, 26936, 42664, 51544, 30012, 11748,
                36100, 5504,  64712, 28336, 44772, 61084, 35876, 752,   46264,
                40904, 59348, 21140, 6932,  52636, 17560, 29876, 20080, 54272,
                24048, 38164,
            },
            {
                51516, 4576,  56084, 14704, 32892, 416,   40492, 47936, 5356,
                17660, 34096, 46152, 27180, 8528,  52628, 62000, 28384, 37692,
                64832, 53956, 11056, 55868, 47152, 836,   59712, 33732, 41476,
                16596, 5916,  38448, 26272, 32584, 18940, 54560, 9116,  51052,
                26864, 54904, 22712, 57560, 28784, 20376, 44380, 8572,  42036,
                19424, 48664, 31720, 9676,  57304, 2116,  54680, 13048, 37844,
                19924, 6264,  24592, 30944, 43696, 33928, 12692, 29204, 39320,
                7248,  11080, 51324, 14980, 22672, 60576, 10044, 31568, 53064,
                21260, 2828,  28752, 16744, 31352, 53708, 5956,  15620, 10428,
                47436, 61156, 14404, 56,    33400, 12760, 59292, 52196, 63148,
                34564, 27492, 51272, 57880, 25392, 35460, 63392, 29536, 11464,
                41848, 1596,  57632, 6804,  33536, 9036,  23268, 55408, 46268,
                26024, 16280, 53724, 50352, 8188,  57132, 20904, 16656, 65120,
                8796,  15112, 36072, 44044, 33504, 49748, 3928,  62256, 41804,
                8364,  59512,
            },
            {
                30956, 20444, 61976, 25728, 49216, 55092, 36388, 21268, 31132,
                51064, 41292, 12136, 64384, 19288, 43092, 15932, 21780, 48820,
                1280,  44992, 15604, 35608, 63212, 29400, 18568, 49712, 24420,
                54104, 46236, 61524, 588,   41552, 44788, 30420, 21996, 2324,
                35520, 31488, 17244, 38696, 5104,  65412, 50468, 23624, 34064,
                4884,  64248, 45040, 21512, 40628, 47892, 28508, 42636, 31876,
                49328, 60532, 14500, 53028, 62168, 5712,  41436, 56712, 21976,
                35964, 59676, 26624, 42012, 46584, 19468, 744,   48196, 15116,
                34196, 64068, 47336, 50948, 35980, 20860, 41296, 64260, 54900,
                31732, 19124, 36356, 25368, 55420, 28772, 3664,  24020, 7312,
                19472, 44572, 13308, 32620, 42976, 4992,  16224, 46612, 26092,
                62420, 36340, 31208, 19984, 63296, 59236, 14036, 40708, 3072,
                60792, 38660, 1656,  22616, 14660, 40128, 26284, 55140, 28640,
                38308, 4628,  61644, 27076, 11164, 59144, 38852, 13736, 26680,
                48808, 17076,
            },
            {
                45316, 35556, 39592, 5824,  18524, 60320, 10752, 64816, 7648,
                56052, 22492, 4672,  58464, 36596, 32132, 56296, 12704, 40152,
                9052,  27244, 32344, 3904,  25016, 11988, 37628, 8840,  2832,
                30868, 20132, 34536, 14568, 64256, 4328,  58432, 39712, 63340,
                47124, 60708, 9596,  52360, 13416, 36352, 1736,  58436, 16548,
                53060, 36452, 25560, 6816,  14612, 34384, 5124,  16952, 58404,
                2852,  27816, 35436, 10216, 20432, 50336, 25984, 64128, 252,
                47780, 54584, 3028,  34076, 58080, 29636, 38600, 56392, 5128,
                41596, 26628, 6892,  12812, 57472, 9280,  46636, 28116, 22828,
                42328, 7132,  52564, 62180, 9368,  44196, 49984, 31052, 46820,
                56808, 61580, 3056,  18180, 55608, 50688, 38176, 20308, 53192,
                5412,  48280, 10800, 44260, 52840, 4232,  35572, 18420, 49692,
                9860,  33640, 58128, 42896, 31288, 3584,  48652, 10136, 33032,
                50380, 53952, 18340, 47716, 172,   23396, 53684, 6172,  36488,
                65152, 964,
            },
            {
                57452, 13064, 8992,  53900, 30152, 42956, 16112, 27064, 45360,
                38380, 14556, 29616, 48472, 496,   50712, 6008,  30260, 58132,
                52164, 20404, 61452, 42492, 50940, 58372, 21944, 45372, 60928,
                56276, 11564, 51924, 23464, 49016, 10028, 16868, 52924, 12868,
                19952, 3788,  44936, 58924, 26548, 47252, 30600, 10632, 39596,
                28400, 12928, 56540, 51348, 59900, 24412, 63556, 53252, 22152,
                8776,  46476, 40928, 57492, 3868,  37572, 16628, 9104,  32000,
                19524, 13348, 40204, 17400, 8508,  50052, 12652, 61116, 23888,
                45184, 54384, 19544, 39108, 24540, 61716, 4312,  36920, 1424,
                58636, 11992, 45648, 21352, 40220, 17712, 37128, 11440, 15108,
                25900, 38676, 49348, 9884,  30124, 23564, 808,   60484, 33212,
                14528, 56384, 23744, 28164, 15856, 47840, 30812, 25308, 63124,
                20608, 29024, 47264, 11792, 63564, 35480, 58948, 45144, 20004,
                2296,  24372, 40408, 56724, 32044, 42384, 15756, 46100, 32460,
                10876, 21812,
            },
            {
                42332, 28468, 63608, 46412, 23324, 3656,  34668, 52544, 1988,
                24952, 62624, 44484, 17496, 25976, 10364, 41856, 63928, 24428,
                4804,  36996, 47984, 17668, 7028,  53892, 32644, 40312, 15596,
                27300, 42952, 7788,  57252, 29112, 36784, 25536, 33676, 6840,
                28952, 40228, 24324, 33192, 6444,  19360, 55244, 43608, 63248,
                20976, 436,   41216, 29948, 18768, 10176, 44568, 36328, 12396,
                51096, 64628, 17484, 23284, 29596, 46944, 53368, 42568, 60316,
                28448, 44664, 62208, 24816, 5464,  65380, 27420, 36260, 17952,
                10920, 30696, 62524, 516,   43780, 32976, 14216, 51316, 17616,
                49064, 29624, 34100, 5248,  58344, 2120,  63920, 54816, 42192,
                5768,  34056, 21824, 64652, 41112, 58360, 12012, 44624, 8320,
                40544, 18624, 65220, 37820, 444,   42024, 56988, 5664,  53520,
                44360, 7200,  17356, 24848, 51892, 18700, 5016,  13752, 42280,
                63144, 30672, 7804,  12940, 64300, 20944, 29028, 61056, 56056,
                24836, 52320,
            },
            {
                33592, 16728, 2736,  50524, 38320, 13464, 57732, 48076, 19632,
                59344, 8724,  32964, 53460, 61004, 22140, 35544, 18276, 46072,
                14412, 59472, 10508, 30708, 39104, 14184, 3484,  62592, 6004,
                35820, 48092, 2576,  39360, 20696, 46028, 59480, 50172, 43484,
                55040, 62540, 15252, 51204, 42420, 60080, 14828, 24796, 7128,
                49668, 46232, 62284, 4704,  38296, 48860, 1636,  30296, 39808,
                25876, 33128, 1156,  43824, 61372, 12452, 33648, 24184, 5092,
                51112, 10116, 37600, 52796, 32456, 20592, 44304, 1952,  51528,
                7928,  59068, 35240, 48816, 17252, 29016, 55756, 60236, 26248,
                39260, 64684, 15760, 24248, 50980, 27612, 32480, 20168, 60740,
                532,   53176, 27900, 7136,  14984, 47752, 36088, 27176, 54488,
                30520, 2752,  50624, 9272,  59448, 21308, 11084, 39148, 13552,
                36788, 60224, 55964, 308,   38436, 30000, 61232, 25540, 36936,
                57716, 16268, 46748, 35308, 3380,  51300, 8468,  18528, 2192,
                40332, 5344,
            },
            {
                47476, 58972, 26240, 20180, 32424, 61216, 6364,  40216, 30852,
                11824, 36940, 3604,  39788, 13568, 46896, 55316, 2212,  33416,
                28656, 54744, 2564,  65124, 21040, 46632, 28136, 50484, 23944,
                18864, 64660, 32176, 16328, 61708, 6164,  1264,  14476, 23272,
                9404,  36732, 316,   21476, 10220, 34992, 2468,  37624, 53584,
                32256, 9768,  16420, 33572, 54764, 27032, 60912, 56376, 19464,
                4904,  53640, 13996, 36056, 8136,  55364, 2088,  15264, 57300,
                34988, 22376, 1224,  56464, 48244, 15328, 41096, 55572, 33088,
                46736, 14440, 21924, 5444,  52784, 10076, 40476, 6220,  20580,
                9496,  3400,  56656, 37492, 47320, 10264, 6628,  23596, 36448,
                48624, 17024, 44472, 55020, 32380, 5564,  63540, 16856, 22008,
                61040, 43680, 25436, 34220, 45632, 29168, 64112, 51184, 32308,
                2868,  27016, 41328, 49196, 9024,  22196, 54032, 6596,  50708,
                10552, 22096, 53196, 26832, 38924, 59832, 43080, 35656, 50420,
                14068, 62028,
            },
            {
                8024,  37116, 11436, 55428, 44476, 9248,  28184, 17216, 54920,
                43176, 49988, 21172, 57860, 6684,  26820, 62956, 8676,  51600,
                39580, 22504, 43988, 26056, 35440, 57056, 9592,  55132, 41984,
                12164, 58424, 25648, 52620, 12552, 42044, 31068, 65180, 18788,
                57284, 47996, 30560, 54300, 64580, 28188, 48824, 61628, 18188,
                26648, 59320, 23412, 43328, 12128, 21696, 7352,  15564, 42864,
                62984, 46124, 57952, 27432, 48488, 20816, 65108, 40500, 45280,
                18348, 63360, 14096, 29836, 9444,  59632, 4508,  23196, 63768,
                28312, 39716, 57288, 26364, 65008, 46080, 23176, 34940, 47804,
                44048, 52812, 31152, 12728, 42376, 60016, 52532, 45868, 13024,
                29692, 59152, 11360, 39380, 19912, 24932, 51088, 3824,  46788,
                38224, 6276,  13648, 54196, 16608, 4712,  24700, 18120, 48340,
                22636, 16060, 34312, 65452, 14020, 43320, 46316, 33996, 27948,
                1084,  40872, 61852, 5408,  15140, 24024, 11216, 46596, 27024,
                30780, 19816,
            },
            {
                23492, 43280, 356,   64784, 15228, 51436, 22420, 62428, 864,
                24272, 65388, 28664, 16396, 44152, 31240, 37688, 12580, 43188,
                16512, 7280,  50360, 12132, 60680, 1708,  16928, 31116, 37524,
                352,   44496, 9192,  35024, 49564, 55788, 27148, 39632, 34844,
                3668,  25924, 43196, 17284, 38400, 4416,  22380, 11728, 44808,
                5456,  35832, 51048, 2520,  64012, 45764, 35248, 50716, 10772,
                24268, 31252, 6448,  17920, 38992, 51668, 24980, 30068, 7884,
                49872, 26976, 41748, 45696, 35220, 25548, 38864, 11916, 19240,
                6564,  43160, 2684,  36660, 12548, 30528, 1284,  63012, 13260,
                27564, 61140, 22324, 912,   19132, 34360, 16052, 39124, 64296,
                8008,  22588, 35392, 61624, 1760,  56628, 41736, 34740, 10636,
                58124, 20380, 31932, 62788, 37264, 56064, 41256, 8304,  58076,
                61680, 10064, 54396, 4452,  31628, 58600, 17396, 12176, 64368,
                47916, 31328, 18208, 44272, 55372, 32924, 57996, 6500,  64480,
                38736, 55760,
            },
            {
                49304, 53428, 34528, 29524, 41272, 4596,  36360, 45152, 33888,
                14456, 7488,  35244, 52464, 1380,  48920, 19404, 59120, 25104,
                62036, 56112, 32488, 18684, 40824, 45108, 23084, 63784, 48772,
                20884, 54252, 29304, 3852,  22412, 17332, 8576,  45684, 52684,
                11996, 62192, 7252,  13324, 50592, 56848, 41892, 29548, 58136,
                38892, 13980, 55488, 28960, 39476, 17632, 31828, 58568, 180,
                37668, 21020, 61828, 42100, 10556, 32140, 4676,  59816, 12104,
                38556, 5296,  58952, 2796,  17728, 63156, 50808, 30992, 53864,
                60756, 17056, 50036, 55012, 20100, 42408, 58244, 50768, 18616,
                33100, 7364,  40116, 54544, 63324, 29416, 5108,  25524, 54344,
                42988, 3580,  50840, 26792, 45272, 13060, 29856, 62532, 15568,
                28560, 52596, 49416, 2124,  11724, 47408, 31020, 972,   35276,
                42992, 45956, 28780, 19384, 24256, 39336, 3084,  54884, 20356,
                37956, 56896, 8932,  49660, 29212, 800,   21152, 52444, 16412,
                4012,  12296,
            },
            {
                31948, 17276, 7060,  20644, 48940, 25120, 58848, 53680, 10620,
                48144, 57120, 41384, 60240, 11220, 23456, 53912, 5520,  29952,
                160,   36696, 47764, 4492,  28444, 52920, 7468,  34092, 15076,
                5524,  61492, 38764, 46824, 63468, 37080, 2424,  59856, 21264,
                32844, 40496, 58780, 46244, 24568, 32428, 8472,  16128, 48292,
                824,   62380, 19696, 9084,  48576, 4204,  53944, 26584, 41356,
                49448, 13176, 52648, 3016,  56304, 15768, 35744, 46716, 54020,
                20544, 32960, 51836, 22648, 55124, 7644,  43708, 272,   47572,
                34496, 9828,  24052, 32064, 7776,  16056, 38524, 3984,  56024,
                37176, 16308, 49340, 26104, 8848,  44072, 57972, 1460,  19604,
                32928, 57152, 10060, 17912, 48932, 7504,  22816, 53568, 888,
                40036, 7712,  42512, 22708, 27576, 60464, 19964, 52240, 26108,
                13388, 5476,  37864, 52436, 48592, 60996, 8072,  41932, 25140,
                14468, 4056,  23172, 35684, 12440, 62644, 41820, 34832, 24904,
                44764, 59272,
            },
            {
                40572, 26852, 62936, 56868, 37996, 12696, 18084, 2696,  29924,
                20384, 25596, 4568,  18688, 32696, 64288, 41756, 34852, 47004,
                21392, 13864, 64412, 9208,  58160, 37136, 12848, 59252, 27572,
                40952, 25080, 18316, 13200, 31984, 57124, 24892, 48640, 15712,
                4980,  27840, 18472, 35748, 1340,  20048, 63560, 34700, 54320,
                21960, 31636, 42364, 25268, 60824, 14340, 22812, 65088, 5872,
                29680, 59556, 34356, 26780, 44704, 62352, 19140, 432,   28228,
                63880, 15472, 10608, 47644, 28724, 13036, 37156, 21000, 26900,
                14152, 38624, 58724, 44240, 62304, 48740, 28260, 10292, 24616,
                43228, 4876,  58876, 46456, 14520, 21640, 50376, 36228, 47672,
                14140, 40516, 30548, 65376, 37496, 33296, 58560, 18740, 47824,
                25708, 64824, 16800, 34556, 45112, 9244,  15144, 62544, 40020,
                21852, 57196, 63088, 1260,  11600, 35620, 29176, 51376, 32716,
                62128, 45528, 59880, 54108, 39416, 18604, 47884, 8104,  28804,
                61484, 1704,
            },
            {
                21936, 13776, 45448, 3384,  9060,  61512, 32592, 42716, 63932,
                52048, 36820, 46676, 28080, 38908, 2972,  15444, 9536,  58060,
                51780, 26180, 42536, 31280, 20452, 24692, 46448, 2724,  54964,
                50004, 10148, 57904, 52104, 6636,  42708, 10644, 30328, 64276,
                43792, 51276, 55800, 10424, 61192, 52076, 44364, 4636,  25756,
                10908, 46484, 6236,  34192, 56904, 37256, 10988, 44192, 19740,
                9236,  16592, 47308, 22872, 7384,  38144, 50404, 23656, 43400,
                56496, 36484, 25248, 60024, 40548, 33568, 61560, 56664, 5604,
                64568, 51496, 1408,  26572, 5204,  34304, 21348, 61344, 53564,
                30340, 65288, 11616, 35364, 31844, 39640, 61304, 11172, 27876,
                63000, 24288, 6204,  53152, 15164, 41160, 4512,  44352, 30752,
                37024, 12716, 53772, 58964, 4116,  38080, 55328, 32920, 7084,
                50632, 17804, 33552, 27104, 15792, 21504, 44320, 58000, 1880,
                10872, 28176, 16368, 6044,  26088, 3208,  57380, 13528, 50460,
                36304, 9888,
            },
            {
                57916, 52504, 35808, 29020, 54620, 47168, 26416, 6632,  39424,
                15952, 8984,  62172, 13412, 49928, 56680, 26872, 44972, 18072,
                39256, 3140,  54268, 15716, 50496, 39492, 63244, 17864, 43404,
                31556, 35648, 1068,  44832, 20932, 14988, 39100, 54828, 1584,
                36436, 22912, 6128,  30660, 39924, 27344, 13476, 59748, 40788,
                50640, 64748, 17160, 52956, 3020,  30176, 47636, 52244, 33124,
                57064, 39552, 2184,  63652, 11424, 30456, 58812, 13584, 8820,
                39800, 2420,  6348,  44404, 19440, 3336,  16704, 45944, 30392,
                41900, 22064, 18492, 46724, 12288, 56280, 40668, 14688, 45484,
                656,   22848, 18872, 53860, 2216,  6476,  55332, 18008, 7732,
                51752, 44712, 20528, 184,   60144, 25460, 11860, 62112, 9180,
                56608, 2784,  20844, 29520, 48600, 23408, 43232, 2644,  47588,
                30256, 10520, 42792, 47012, 55660, 64716, 5912,  18000, 48208,
                36944, 41204, 52232, 33116, 43940, 65348, 31220, 22928, 54936,
                17000, 43044,
            },
            {
                6480,  48464, 23880, 18880, 39908, 14748, 21464, 49772, 58048,
                23820, 1144,  55696, 42836, 7092,  22328, 62764, 31168, 6176,
                59872, 11520, 33968, 61584, 1032,  6540,  29508, 11336, 21904,
                7372,  65072, 24120, 28816, 62344, 33984, 49860, 26428, 19416,
                13404, 59696, 46972, 15900, 49168, 2888,  37356, 18280, 7864,
                29068, 36988, 13096, 44440, 20972, 40112, 16044, 1740,  61012,
                24880, 43660, 55052, 19512, 51696, 41836, 3672,  34016, 47944,
                18116, 51384, 31696, 64768, 53492, 9400,  50140, 24824, 11656,
                55144, 8436,  35560, 29944, 63304, 49912, 3504,  31952, 8708,
                51460, 38688, 27792, 42340, 63528, 24924, 30072, 43168, 33972,
                3332,  57756, 36620, 47428, 28496, 55532, 21484, 49672, 35084,
                23740, 50740, 40772, 7232,  63416, 17580, 27256, 12920, 64552,
                24716, 58696, 3112,  38468, 8452,  25300, 40240, 13020, 23796,
                63384, 19764, 8660,  49160, 20916, 10300, 36976, 45844, 5144,
                63584, 32828,
            },
            {
                29968, 2188,  11328, 65096, 43632, 572,   60640, 35360, 11104,
                45768, 33792, 30320, 17080, 35672, 52156, 12228, 37372, 49204,
                23600, 28484, 44500, 19376, 48344, 56668, 35380, 52688, 60360,
                38492, 16316, 48272, 40324, 59176, 2860,  6520,  45356, 61620,
                9148,  41672, 33872, 25372, 64576, 21752, 55276, 32248, 57868,
                23280, 1348,  60536, 26984, 58500, 8208,  63264, 28064, 36160,
                4948,  13516, 31184, 36832, 27748, 15364, 54296, 25784, 61360,
                29084, 57356, 22184, 14128, 27144, 38188, 58912, 32780, 4104,
                39244, 60676, 52492, 15552, 38180, 24776, 19736, 58920, 35840,
                16712, 47864, 57864, 9320,  49948, 16144, 46920, 59452, 22188,
                39160, 15820, 11364, 32328, 7424,  43416, 38720, 1488,  17052,
                45284, 60844, 14708, 31824, 10656, 51592, 57008, 37060, 19900,
                52696, 35372, 14636, 61648, 29696, 50756, 32336, 53440, 4840,
                56228, 30736, 1272,  55228, 60896, 15452, 41020, 120,   27080,
                19724, 38920,
            },
            {
                61932, 16120, 58564, 34116, 8020,  30668, 51804, 4344,  28224,
                19716, 53072, 60188, 25096, 47080, 2176,  19760, 42204, 61396,
                4192,  55808, 36656, 7924,  25564, 41312, 14900, 23828, 45456,
                3964,  55820, 12608, 8612,  18544, 52904, 23136, 56560, 37840,
                28660, 51812, 116,   57272, 11276, 7460,  42164, 45712, 14668,
                53264, 48184, 33200, 5200,  50516, 23528, 43036, 11384, 48388,
                21704, 50216, 8824,  60216, 45836, 5836,  65360, 20900, 10868,
                44840, 6996,  35984, 48288, 732,   43016, 20424, 62724, 14488,
                48480, 23660, 2896,  43628, 6720,  10596, 42692, 53148, 26384,
                61616, 6180,  33544, 13728, 21444, 37312, 4488,  12284, 52952,
                62480, 25928, 50268, 60976, 18220, 52236, 14264, 63076, 32896,
                27332, 5048,  37364, 58156, 45724, 34380, 472,   41576, 8716,
                45020, 4616,  22688, 54208, 19040, 260,   45224, 60192, 35136,
                26932, 46548, 13492, 38620, 24080, 28712, 52576, 59232, 49264,
                12656, 53488,
            },
            {
                24476, 41772, 50868, 27228, 47912, 56924, 17696, 38132, 64224,
                14192, 41636, 3532,  10268, 65192, 29628, 54704, 8672,  33300,
                16924, 46956, 13152, 64956, 31400, 9920,  58704, 2196,  33584,
                19476, 30800, 51520, 26632, 36576, 31480, 11240, 16616, 32668,
                5012,  21276, 44416, 18028, 35940, 52608, 28004, 3596,  63852,
                19420, 39636, 10532, 17576, 36672, 53964, 32324, 18564, 56044,
                39192, 64272, 17488, 636,   23540, 34776, 49200, 37940, 1472,
                41108, 16516, 62432, 11568, 55596, 29768, 7440,  35352, 44200,
                27892, 18104, 32360, 59492, 55484, 28988, 64152, 2164,  12216,
                22004, 43880, 3032,  62404, 31124, 57112, 41100, 27660, 8432,
                31504, 2028,  41624, 23068, 35868, 3428,  29772, 57856, 8408,
                54748, 19108, 43360, 25452, 21320, 5920,  62184, 15636, 31200,
                60392, 27724, 40784, 48004, 11468, 37480, 15404, 21064, 9944,
                42800, 16916, 58796, 34128, 3764,  47132, 8052,  34784, 22308,
                6896,  44328,
            },
            {
                9040,  19024, 4728,  37648, 21972, 11828, 25296, 44396, 55084,
                5832,  49624, 37160, 21564, 40284, 58296, 26096, 14380, 51564,
                22472, 40488, 3364,  52088, 21400, 43896, 50844, 27920, 63160,
                47292, 41696, 5032,  63860, 44448, 57656, 41220, 47220, 64896,
                53620, 14060, 62400, 31416, 48108, 59944, 24152, 12080, 35104,
                6860,  43500, 61460, 29788, 46672, 13744, 932,   59388, 7176,
                25720, 33752, 53576, 40868, 57684, 9436,  17844, 31340, 59684,
                53316, 26352, 33024, 40168, 23776, 51336, 16788, 56752, 1620,
                53560, 65264, 12604, 47120, 20864, 33952, 17100, 46256, 37180,
                29512, 54136, 39496, 25520, 51828, 396,   17836, 64304, 48420,
                19788, 45468, 56944, 9764,  64916, 46392, 25200, 41452, 11612,
                47624, 64640, 2012,  12436, 53856, 28832, 46996, 24344, 50364,
                55748, 17368, 7100,  33500, 57516, 62864, 28268, 49996, 2848,
                64596, 22564, 6516,  41540, 62696, 11928, 18392, 64452, 29688,
                57236, 36148,
            },
            {
                60456, 32112, 55664, 14608, 1916,  63080, 40816, 32652, 9304,
                22932, 31492, 56744, 16080, 6468,  34048, 46008, 1076,  63808,
                30036, 57596, 26544, 59752, 37548, 17164, 5760,  39444, 13284,
                9128,  24684, 59488, 21188, 552,   14492, 7428,  25004, 2992,
                39112, 26744, 9528,  40528, 6256,  15724, 38768, 50500, 30640,
                57500, 25860, 21820, 55300, 4372,  65252, 41340, 29088, 44872,
                14996, 3936,  46976, 27164, 12900, 43968, 55704, 24832, 13804,
                46520, 4080,  57800, 19180, 5276,  63644, 46504, 11000, 21604,
                40632, 8988,  36724, 26308, 40,    51416, 41052, 7412,  49540,
                57308, 15432, 19216, 47364, 10436, 44916, 35692, 54580, 14684,
                38324, 5328,  28132, 13868, 54144, 5692,  20368, 51068, 34716,
                22768, 30020, 39220, 50092, 60292, 18140, 39672, 10224, 3588,
                38116, 12380, 64984, 42540, 3900,  23612, 7472,  39616, 57216,
                36280, 48232, 53276, 31268, 25712, 56136, 43652, 39412, 15784,
                3632,  47000,
            },
            {
                51832, 28108, 40092, 46436, 58800, 52500, 16272, 2900,  60476,
                47684, 27312, 62792, 43860, 11720, 50668, 18760, 37992, 43336,
                10948, 7140,  18968, 33520, 812,   54932, 61328, 22832, 49732,
                56548, 37236, 15960, 34108, 48428, 27836, 51248, 59008, 18248,
                34896, 49276, 55612, 22952, 29296, 1976,  62876, 20856, 46528,
                1008,  51568, 15068, 8500,  38692, 24792, 20532, 34924, 52480,
                61412, 10592, 30876, 20112, 63476, 38948, 3272,  50644, 8080,
                21980, 36780, 49920, 9524,  42308, 32028, 37740, 25532, 61288,
                29516, 50176, 4432,  58412, 39212, 14024, 62068, 23696, 32100,
                3876,  64976, 7832,  33884, 60652, 21920, 6988,  24376, 32708,
                61472, 52024, 34956, 17740, 31112, 39872, 60124, 15956, 6608,
                53348, 4084,  16560, 35204, 8048,  31988, 56584, 63532, 21468,
                34560, 49008, 25528, 19620, 30488, 51944, 44156, 32540, 12512,
                18632, 28760, 9288,  14972, 51240, 21296, 1948,  32496, 54328,
                26420, 13116,
            },
            {
                668,   62584, 10384, 23000, 34916, 7396,  29408, 45548, 36268,
                12624, 18292, 28,    53500, 28488, 59628, 5088,  24156, 55000,
                34972, 49760, 44784, 14300, 47460, 11688, 30724, 35408, 18516,
                1480,  29132, 51904, 10856, 39940, 61948, 30912, 21892, 44948,
                12360, 61144, 5184,  46212, 58184, 43040, 54044, 9120,  17004,
                41528, 36400, 63008, 33604, 49636, 57760, 11144, 2844,  48748,
                23328, 37412, 58596, 51188, 6728,  33472, 29044, 61048, 42564,
                64036, 29880, 15668, 60076, 28044, 2044,  14316, 52416, 5960,
                33596, 45568, 16968, 23076, 55048, 9852,  27668, 59208, 19836,
                11560, 42176, 27476, 55672, 13536, 29440, 58228, 43284, 11192,
                1360,  21456, 59036, 44372, 48992, 576,   36948, 27984, 45552,
                61332, 41980, 57104, 26436, 44604, 1456,  13900, 43008, 52848,
                29544, 844,   59604, 46312, 9780,  56188, 16208, 61608, 25488,
                54600, 796,   44684, 61832, 5156,  36664, 48804, 10084, 61252,
                41652, 21196,
            },
            {
                16652, 44600, 5680,  30920, 18520, 49360, 24664, 56128, 20548,
                51756, 41780, 7528,  35884, 21984, 46932, 31940, 62452, 15828,
                2988,  60860, 38932, 28852, 63892, 24780, 41800, 7880,  46048,
                65316, 42732, 6884,  54688, 19120, 4276,  37336, 8976,  54220,
                988,   31996, 20192, 16508, 34464, 12964, 25580, 32736, 60972,
                27860, 11548, 3264,  45832, 17336, 28208, 43556, 62444, 18184,
                7752,  42620, 16424, 1208,  23004, 48384, 14924, 19272, 34996,
                192,   12064, 45180, 54364, 20772, 48436, 57504, 43432, 19092,
                59124, 13620, 63824, 42456, 31604, 47992, 5096,  53104, 34636,
                38592, 46084, 50784, 1892,  37460, 40712, 3548,  51208, 27216,
                47040, 39240, 7632,  11932, 23232, 63600, 9584,  56444, 13204,
                32436, 20720, 10096, 62056, 23336, 49380, 36288, 26840, 6708,
                16168, 39420, 53792, 13668, 36864, 1872,  21716, 47188, 5492,
                40764, 60160, 33692, 39092, 17404, 28264, 57968, 24336, 6400,
                50024, 35208,
            },
            {
                38276, 25512, 53304, 59736, 42608, 65284, 10892, 39292, 5340,
                63372, 33384, 24852, 61016, 39604, 13928, 9232,  41684, 27884,
                20196, 53292, 22644, 8996,  51504, 4128,  58496, 53936, 14768,
                26380, 60312, 32036, 23664, 46240, 56968, 13600, 43292, 64116,
                25456, 41764, 38048, 51820, 65408, 7392,  37500, 48148, 4388,
                53048, 58904, 23348, 30936, 53760, 6980,  14524, 54716, 39912,
                29356, 56328, 34184, 44640, 60452, 54556, 10696, 26800, 56248,
                52032, 39228, 24060, 3704,  33836, 64868, 8324,  35852, 24200,
                1248,  38412, 28192, 7480,  18676, 36048, 43700, 15080, 936,
                62436, 24180, 17108, 20924, 62896, 48596, 16500, 64632, 19048,
                56672, 30324, 62768, 26276, 54500, 33424, 19148, 40640, 24840,
                1600,  50800, 37752, 14800, 54472, 4692,  20088, 60560, 45856,
                56856, 8844,  23008, 31588, 62560, 41260, 28036, 35108, 10712,
                50076, 20460, 23748, 11344, 46704, 64180, 13712, 44244, 18744,
                30164, 64688,
            },
            {
                57184, 48120, 13408, 3148,  36608, 14964, 1236,  28364, 57900,
                15600, 48512, 10704, 55380, 17280, 51164, 1324,  56832, 48544,
                11756, 32288, 5400,  42640, 16568, 37132, 20348, 33292, 3216,
                21568, 38548, 12460, 1924,  63660, 26556, 35168, 17272, 49528,
                29632, 59848, 10132, 3880,  28068, 22024, 55996, 18236, 39952,
                14032, 44464, 20056, 39044, 64396, 52,    36804, 32468, 25204,
                3524,  63876, 13420, 26296, 8348,  36616, 41508, 46384, 6032,
                31528, 17492, 62316, 41212, 26264, 11764, 17112, 29996, 47388,
                53808, 10684, 50596, 57016, 2232,  63024, 21580, 49896, 30424,
                57600, 6780,  54200, 31536, 5216,  25304, 33216, 8088,  36204,
                13944, 3748,  41680, 16540, 2604,  43492, 51784, 7344,  47960,
                64080, 28868, 5940,  46540, 30540, 65352, 41308, 11808, 33072,
                18976, 63888, 44016, 6472,  17820, 48800, 58176, 14648, 65428,
                29976, 3320,  57396, 52860, 31392, 1328,  40132, 34284, 53448,
                2504,  11408,
            },
            {
                7144,  28976, 40620, 20952, 54784, 32884, 51172, 43928, 22960,
                31172, 2428,  45012, 28996, 4208,  65068, 34432, 25056, 37720,
                45188, 62824, 56008, 26076, 60100, 29972, 47976, 11284, 50900,
                57440, 45416, 17084, 50180, 41088, 7696,  52664, 2824,  22456,
                6680,  14932, 47140, 57536, 43992, 50296, 452,   63112, 24496,
                32048, 5884,  50576, 9736,  26268, 47504, 58100, 21492, 51368,
                11356, 47128, 20612, 52384, 31088, 18268, 3856,  65148, 21644,
                58604, 9268,  48216, 6952,  51456, 37860, 54880, 5372,  41516,
                59904, 20880, 32764, 46408, 24764, 11856, 54768, 39120, 25868,
                10240, 14052, 36000, 45088, 59636, 12432, 42056, 54736, 45792,
                22796, 52788, 49396, 36764, 57712, 27736, 14512, 59364, 35536,
                16640, 43224, 58280, 18728, 34388, 9392,  52512, 24116, 2916,
                49484, 27916, 34948, 51480, 26068, 4048,  53932, 7700,  44988,
                37848, 15844, 42968, 8064,  26156, 55416, 21832, 8740,  59616,
                46068, 22632,
            },
            {
                33308, 17932, 63404, 9096,  25232, 46276, 61872, 18944, 8176,
                53544, 38184, 58552, 19368, 43108, 30784, 21404, 59372, 7436,
                18056, 2092,  34876, 13628, 45688, 360,   64340, 40932, 28424,
                36144, 8924,  55188, 33920, 28720, 20868, 61544, 31768, 39896,
                56080, 36024, 24576, 17968, 33168, 11788, 30308, 42432, 9012,
                56936, 35628, 61500, 16004, 41032, 19176, 8000,  44184, 61184,
                5644,  41128, 37772, 2100,  61968, 56792, 25008, 38652, 50220,
                13700, 36364, 29328, 19660, 59056, 44412, 22560, 62220, 13432,
                26560, 3068,  15948, 37544, 60572, 28880, 7836,  17312, 47156,
                42984, 60692, 28548, 39836, 8964,  51672, 20256, 28200, 4936,
                62228, 9788,  31920, 20184, 6156,  46712, 30892, 22256, 2932,
                54836, 11332, 22908, 40472, 204,   27248, 47200, 38272, 58776,
                14460, 40936, 1172,  60444, 12084, 38316, 32612, 19656, 24500,
                51404, 59464, 34984, 19208, 47632, 4528,  62116, 15488, 26972,
                37268, 51700,
            },
            {
                3784,  43520, 49868, 58380, 30360, 5240,  11952, 40804, 34952,
                64440, 12744, 26280, 7184,  36740, 47868, 11024, 52680, 15008,
                29424, 50556, 40192, 19940, 54216, 9200,  23940, 18252, 5880,
                62264, 25152, 3892,  59576, 14172, 44204, 10404, 58384, 45524,
                12500, 52316, 62492, 2312,  39736, 60852, 15548, 53364, 20992,
                45500, 27232, 2064,  55204, 33372, 60384, 29796, 13740, 35592,
                17444, 55840, 28348, 48960, 15396, 44228, 11800, 33132, 1108,
                26532, 44940, 55496, 2728,  15148, 31884, 608,   49508, 34024,
                39976, 64448, 44644, 6440,  51356, 41808, 33908, 65384, 5532,
                52424, 22076, 2260,  49464, 15928, 57204, 1132,  60284, 39064,
                34280, 25688, 59084, 13028, 65116, 38216, 10340, 61020, 39480,
                26404, 32820, 51440, 62780, 13872, 59500, 17204, 6280,  30232,
                53472, 9900,  20324, 55424, 46144, 22704, 62856, 41488, 136,
                27328, 5180,  12560, 63100, 41124, 36852, 50440, 31836, 42208,
                12756, 61736,
            },
            {
                24888, 15032, 34860, 592,   38872, 17148, 60048, 27272, 3564,
                45668, 21636, 50872, 61964, 14728, 54632, 5220,  39084, 44108,
                64200, 23308, 58376, 6616,  31872, 38240, 51916, 59048, 14296,
                43772, 31232, 19308, 39248, 49000, 168,   36816, 25684, 19204,
                4792,  27552, 42560, 21356, 7940,  47748, 25872, 37188, 3424,
                64008, 13016, 48636, 23056, 11488, 53024, 3004,  49880, 24012,
                31936, 64704, 8852,  22860, 34920, 4756,  51612, 60308, 16756,
                41768, 61816, 23016, 34488, 64076, 10276, 28300, 57468, 17812,
                9220,  52584, 30432, 22876, 13540, 19844, 55632, 1588,  37064,
                32152, 18556, 63576, 26328, 34812, 23092, 30656, 43916, 14544,
                17940, 47724, 380,   43688, 23264, 50104, 4720,  52716, 19336,
                45908, 8332,  4216,  48412, 35896, 55156, 43744, 21748, 64408,
                37008, 24928, 43244, 15836, 29280, 8700,  48424, 13468, 60736,
                54348, 32900, 49312, 23304, 29460, 10544, 18052, 58860, 6116,
                20236, 54448,
            },
            {
                38484, 10304, 55512, 21648, 47564, 52756, 23720, 56192, 49104,
                16256, 32544, 684,   40464, 23424, 27744, 60788, 19000, 32532,
                3476,  10604, 47280, 27428, 63104, 12160, 42892, 34088, 2600,
                48676, 53728, 11596, 63412, 24108, 54924, 16360, 50456, 64880,
                34080, 9520,  31060, 49848, 55732, 35112, 5024,  58772, 51168,
                17168, 29540, 38372, 42648, 5360,  45840, 26876, 39972, 54468,
                996,   45436, 13184, 58216, 42312, 19144, 29448, 46480, 8836,
                53408, 5696,  11876, 50908, 39400, 42544, 20620, 45640, 4040,
                25028, 56400, 36012, 4296,  59116, 48236, 10088, 23788, 58420,
                41176, 11760, 55956, 4172,  47352, 38012, 64872, 10444, 50408,
                56236, 7036,  39864, 54000, 26816, 17260, 33528, 41812, 13244,
                64488, 57524, 29888, 19812, 24964, 2672,  32528, 12020, 48176,
                3536,  56952, 32004, 63536, 35232, 2692,  57324, 17448, 36592,
                9308,  45948, 16980, 58256, 1124,  54516, 25180, 45664, 2132,
                48380, 30264,
            },
            {
                64284, 45380, 27812, 62572, 13452, 7840,  36036, 30764, 10312,
                42676, 59820, 55284, 9352,  46560, 34140, 2032,  49440, 25516,
                55828, 35256, 42576, 17192, 1836,  49432, 20408, 25936, 56852,
                37340, 21788, 7196,  46348, 33108, 5528,  30384, 41136, 3136,
                46880, 54092, 16136, 63836, 13656, 28728, 19460, 10528, 40688,
                32616, 7872,  59688, 18916, 65240, 36272, 16416, 61992, 10368,
                19576, 37016, 26168, 53376, 7044,  63524, 24672, 56124, 21016,
                31748, 37528, 18368, 47548, 24828, 6568,  53260, 60544, 37304,
                48812, 12184, 18796, 62800, 38640, 31580, 27048, 45156, 14736,
                29412, 51556, 8244,  42644, 12876, 18840, 53324, 3212,  27276,
                22428, 63504, 30244, 35412, 8488,  62700, 56764, 28168, 1160,
                36320, 16288, 43392, 38848, 61664, 9156,  51984, 40372, 27440,
                17892, 8032,  50008, 5072,  40248, 52828, 28012, 44040, 30828,
                21532, 63868, 39760, 7204,  43944, 14076, 65164, 34752, 39656,
                57544, 7956,
            },
            {
                51352, 2456,  18932, 32284, 40428, 44224, 2076,  61656, 20340,
                5432,  28652, 36224, 18512, 64028, 12872, 58144, 41412, 8300,
                61764, 21116, 13932, 55096, 36912, 61180, 30556, 7800,  16100,
                64960, 29004, 41964, 17220, 57664, 9916,  61920, 22360, 13424,
                60168, 37556, 23956, 720,   38968, 45952, 61364, 23484, 44336,
                56600, 49736, 604,   25716, 55528, 31128, 22108, 7112,  42752,
                59020, 49192, 4144,  31524, 39196, 10972, 35900, 2308,  40392,
                64592, 27708, 57044, 3196,  61428, 33712, 16408, 13288, 30808,
                41444, 7576,  28696, 45488, 468,   16276, 53876, 62064, 3092,
                21060, 46172, 33272, 62720, 28940, 59240, 6808,  32632, 36952,
                42400, 11304, 19220, 47024, 14992, 2772,  44984, 21156, 48844,
                24300, 54456, 5336,  12488, 28392, 46120, 15852, 62536, 34728,
                59648, 44744, 22540, 14016, 59276, 19324, 24124, 6420,  50660,
                1968,  12108, 26448, 31972, 36412, 51264, 21368, 9144,  27380,
                16028, 23060,
            },
            {
                13072, 36676, 59520, 5924,  25880, 57812, 51148, 14484, 39740,
                53764, 24740, 50044, 3724,  30720, 44528, 22440, 16312, 29960,
                37812, 5004,  51856, 29144, 23632, 5784,  44724, 52960, 10756,
                47224, 968,   35336, 51580, 26684, 38536, 45044, 52228, 28340,
                7296,  43488, 19752, 58544, 6872,  31724, 52916, 2340,  27640,
                14088, 21592, 35312, 47136, 9276,  13236, 50984, 56760, 33068,
                28676, 14504, 62292, 21440, 47212, 59180, 15992, 49364, 13068,
                4324,  45228, 14844, 30624, 9464,  55740, 26944, 1520,  65184,
                22020, 58608, 54672, 24176, 50260, 42348, 6968,  34460, 40068,
                60364, 5292,  24364, 15468, 39544, 21760, 44564, 49136, 16008,
                58692, 52340, 4524,  60604, 55628, 37452, 11872, 30816, 61388,
                9804,  32136, 50880, 59384, 22484, 55524, 6492,  20556, 1100,
                53988, 11064, 38208, 26992, 45780, 9800,  64332, 37924, 55900,
                42268, 59756, 52896, 19628, 57080, 3592,  30448, 49116, 60668,
                43732, 33944,
            },
            {
                4952,  42500, 48964, 54332, 9948,  17424, 33588, 23044, 63756,
                46396, 11744, 15608, 56700, 38996, 6304,  51140, 54376, 11740,
                45244, 48432, 65480, 9784,  40136, 58648, 18548, 33176, 38836,
                60132, 24668, 5068,  62820, 12896, 20276, 1792,  34880, 17916,
                56380, 32752, 26748, 48188, 12124, 41640, 16240, 64820, 36600,
                5256,  60648, 53156, 16172, 41548, 4540,  38088, 46568, 2368,
                23708, 40656, 51864, 17524, 500,   26476, 43052, 53008, 33228,
                23920, 60056, 52136, 21516, 43296, 38596, 46648, 50656, 34296,
                44344, 2840,  14344, 33064, 9552,  64600, 21968, 12668, 48880,
                17604, 53512, 36756, 50852, 220,   56572, 9588,  62052, 24724,
                1304,  33932, 28248, 21396, 40828, 25408, 51256, 58752, 7040,
                40652, 19096, 47092, 1440,  41324, 30400, 37300, 49576, 25044,
                42220, 30128, 61504, 51732, 620,   33420, 47340, 13172, 16356,
                34256, 23548, 4260,  45564, 15556, 62616, 41700, 11940, 464,
                19396, 56340,
            },
            {
                25100, 29380, 15516, 22356, 65476, 29076, 48300, 4908,  37276,
                8068,  32252, 42228, 62204, 20120, 25768, 36472, 312,   60068,
                18416, 25912, 2416,  34068, 14976, 47648, 1448,  26464, 54896,
                20740, 14908, 44468, 31512, 55080, 7580,  48084, 58884, 40448,
                10864, 2864,  62200, 51428, 35388, 57156, 8544,  19100, 46828,
                40028, 10928, 28644, 32720, 58708, 62332, 27548, 18672, 64096,
                11168, 6784,  44084, 34120, 55812, 8092,  30196, 61564, 18580,
                9908,  41200, 35448, 364,   63288, 17624, 23560, 6880,  11116,
                19312, 51992, 39504, 59664, 18132, 36348, 25664, 56596, 30880,
                9136,  27856, 58944, 11280, 47904, 26472, 30860, 18172, 39016,
                12784, 45092, 50116, 9292,  31928, 5500,  18488, 43736, 15440,
                37708, 63140, 26008, 34212, 17268, 64964, 13196, 57708, 33096,
                15056, 4420,  20036, 35888, 16828, 58096, 30368, 4996,  61760,
                28756, 48848, 10320, 38940, 27844, 7744,  24164, 37400, 54412,
                31336, 63668,
            },
            {
                39916, 52428, 2884,  45864, 38112, 42120, 112,   56940, 26356,
                19136, 60112, 1364,  27820, 47448, 10100, 64492, 33496, 23160,
                40808, 31708, 58108, 43612, 21784, 62280, 50952, 12880, 42260,
                6964,  58004, 49684, 40052, 22948, 60516, 29872, 14836, 24956,
                53748, 45576, 15332, 4872,  21424, 25260, 29376, 50020, 55432,
                24032, 62808, 44896, 19508, 1812,  24332, 43728, 15360, 53464,
                36564, 60300, 29260, 12672, 65396, 37972, 22132, 4848,  57432,
                47828, 6396,  26212, 49252, 12788, 57832, 29220, 54432, 61792,
                37004, 25920, 29780, 4880,  46960, 52752, 1816,  44792, 38528,
                63740, 2540,  43300, 20148, 34520, 4484,  41888, 53224, 6040,
                57048, 65336, 15580, 42808, 62260, 53888, 35264, 548,   28476,
                56316, 3696,  11608, 53524, 8252,  43464, 2996,  45424, 9068,
                62980, 48508, 55448, 7300,  42904, 25440, 53220, 21872, 40612,
                7936,  18732, 64752, 32564, 53648, 47288, 58856, 17620, 50540,
                44892, 7512,
            },
            {
                20372, 61448, 35428, 12516, 7916,  20024, 61036, 13852, 43844,
                49824, 54636, 35552, 7284,  52604, 14584, 43508, 56532, 4468,
                50132, 13344, 7052,  55192, 27632, 35988, 4188,  31224, 64184,
                37056, 28592, 2288,  10832, 16484, 36324, 43424, 6088,  65256,
                21808, 37112, 30212, 40004, 63480, 44236, 1732,  34272, 13000,
                31304, 3624,  8308,  50212, 36928, 52376, 7012,  34524, 47612,
                20468, 55172, 4068,  24092, 45804, 51144, 14536, 34808, 28060,
                38684, 54988, 19944, 32172, 8552,  36108, 4564,  40996, 15880,
                47932, 8772,  63408, 42084, 11672, 28024, 60832, 5864,  19528,
                14572, 23476, 31776, 54352, 60028, 13952, 63260, 35936, 47300,
                20336, 26172, 36844, 2300,  22692, 13848, 48500, 64676, 23852,
                52052, 44552, 21488, 36812, 60956, 27484, 21948, 52740, 18204,
                28196, 40288, 23808, 65172, 10256, 38416, 2280,  49648, 55400,
                44864, 35944, 57416, 944,   12480, 20984, 34020, 5572,  27044,
                14716, 10672,
            },
            {
                57052, 32304, 24320, 59680, 55116, 31644, 50944, 34420, 10472,
                29588, 16464, 22080, 40344, 58352, 31472, 18080, 28280, 9100,
                61788, 37152, 19896, 52180, 10548, 17348, 45992, 23432, 8856,
                52516, 18804, 34492, 63592, 54112, 27036, 50860, 768,   33756,
                49320, 8792,  58332, 17992, 10172, 52460, 59408, 6216,  43132,
                17648, 58116, 39156, 22348, 13572, 56636, 10764, 31008, 776,
                26188, 39428, 50016, 18152, 9620,  41400, 1904,  63224, 45332,
                11288, 15916, 64936, 42780, 52944, 60612, 45612, 20808, 32300,
                57588, 3716,  22556, 54972, 15776, 21164, 40992, 33784, 50392,
                55888, 46652, 40180, 7772,  17300, 45324, 22396, 28612, 8728,
                32732, 4292,  55184, 30084, 58480, 45200, 9940,  20020, 33620,
                7692,  30052, 49140, 14880, 56880, 32188, 5764,  38936, 59876,
                34396, 1540,  13304, 31956, 45504, 59532, 19400, 11924, 26892,
                3992,  14396, 50968, 25280, 43468, 60868, 40700, 2744,  62160,
                36524, 48164,
            },
            {
                1556,  16252, 6068,  47328, 27700, 16724, 23488, 5668,  38888,
                64584, 4108,  45672, 12260, 24680, 1900,  38584, 48224, 53744,
                15840, 42396, 29840, 64060, 5288,  40972, 60352, 56796, 15272,
                44852, 25332, 47704, 4352,  41732, 8684,  18692, 61260, 12968,
                40848, 27796, 2524,  46668, 32420, 14064, 38248, 22660, 64020,
                51344, 26764, 46948, 33404, 65052, 28140, 40292, 61420, 57820,
                8896,  14788, 33356, 59484, 30964, 56884, 20848, 52660, 24744,
                3552,  58192, 29600, 2328,  24240, 14220, 27160, 1580,  51232,
                12628, 43276, 31048, 34840, 49424, 58252, 9452,  29660, 62372,
                11544, 36028, 3744,  64924, 25396, 50056, 1632,  11812, 58864,
                51084, 43756, 17732, 40024, 7080,  26552, 38196, 57476, 41996,
                12904, 59840, 5132,  39984, 18988, 46640, 50552, 24636, 11120,
                54648, 47456, 21112, 51312, 28420, 15628, 35356, 63812, 32240,
                60400, 39484, 22164, 30856, 6764,  16552, 51748, 29180, 55924,
                22940, 42028,
            },
            {
                64760, 38464, 51608, 43340, 2488,  40740, 62312, 53948, 48056,
                24976, 56264, 32952, 61988, 50108, 59984, 6748,  21244, 34368,
                25336, 1044,  47304, 23112, 38024, 49620, 28800, 33600, 104,
                39176, 61528, 12572, 31960, 22156, 57140, 45904, 31004, 55600,
                20200, 52996, 61692, 25656, 54976, 19936, 48704, 28440, 35780,
                10728, 340,   55044, 16492, 4828,  45760, 18396, 49696, 22288,
                43980, 64348, 2572,  47144, 26760, 36588, 7592,  17072, 42472,
                33008, 50280, 37192, 47148, 18408, 39928, 56564, 34756, 64540,
                25412, 18772, 61112, 36,    37948, 6828,  45192, 24416, 1180,
                16948, 27268, 52392, 10648, 30032, 37532, 54928, 41304, 61744,
                14336, 23440, 48248, 11244, 60820, 52552, 31508, 1936,  17232,
                47276, 35324, 63484, 25736, 10484, 948,   63984, 36032, 16532,
                6788,  43028, 61104, 3544,  57092, 5420,  48276, 23676, 43248,
                8624,  17772, 47968, 62968, 55216, 37592, 9568,  45396, 18212,
                6956,  30404,
            },
            {
                54132, 26636, 19684, 11624, 57720, 36548, 9324,  20732, 3036,
                14108, 42436, 8600,  19624, 36132, 44008, 11096, 55884, 63168,
                44876, 11884, 33060, 58408, 14436, 3080,  19668, 11492, 53832,
                21300, 6332,  58940, 50312, 38424, 14132, 25064, 36860, 3840,
                43684, 16176, 5756,  36300, 42112, 1184,  8260,  60784, 15424,
                41876, 21252, 60060, 9228,  42276, 24720, 3232,  12740, 37280,
                29684, 51940, 41024, 6152,  13120, 62576, 48712, 29236, 59784,
                13456, 5592,  22464, 10496, 61704, 6868,  49608, 9776,  38772,
                5776,  47108, 52356, 14592, 26304, 65344, 18432, 54128, 38800,
                59732, 47784, 21072, 43112, 58204, 4900,  33516, 19332, 25844,
                38724, 408,   63636, 34244, 20812, 15492, 5752,  62844, 54380,
                27524, 22524, 3164,  55320, 42060, 29452, 14120, 44116, 56500,
                30916, 26444, 37440, 17520, 41608, 9184,  38828, 54036, 1120,
                58348, 28608, 11132, 2476,  33244, 26400, 13592, 63960, 35176,
                49884, 12188,
            },
            {
                44960, 8736,  33840, 62964, 30240, 15244, 46112, 27308, 59376,
                37472, 28764, 52204, 2536,  15720, 29704, 26232, 40400, 3224,
                18912, 50344, 61280, 8392,  26712, 56072, 43000, 64736, 48156,
                30884, 42296, 27584, 17700, 1368,  64504, 6724,  59096, 10968,
                23240, 48904, 30436, 12844, 64520, 23984, 56168, 44956, 3860,
                53820, 32548, 37332, 29148, 61824, 52980, 35304, 55452, 6644,
                16824, 25312, 19808, 55968, 22936, 39300, 216,   54356, 35788,
                20008, 62732, 41224, 52600, 33352, 28984, 43948, 22072, 16812,
                54892, 29060, 40336, 10144, 56836, 32444, 48116, 13340, 42552,
                31308, 7192,  34780, 61488, 18016, 13880, 44800, 7548,  31600,
                52372, 6192,  56068, 27324, 45820, 50612, 24360, 44420, 39432,
                9124,  50388, 15388, 32756, 52108, 58484, 22720, 4044,  62020,
                20300, 9664,  53020, 33580, 25036, 62460, 30116, 14056, 21284,
                34204, 52212, 40980, 44804, 59612, 19988, 48504, 236,   24912,
                58188, 21208,
            },
            {
                15456, 60696, 920,   48556, 22344, 6556,  52560, 41544, 32412,
                17756, 62836, 46452, 23164, 55128, 64472, 47720, 14720, 31564,
                57168, 6000,  39324, 21412, 51956, 35096, 7004,  24972, 36776,
                15744, 9572,  55404, 34320, 44004, 52760, 28160, 47656, 33564,
                63320, 38840, 57772, 9160,  50824, 33448, 39464, 30076, 18764,
                25132, 50128, 6248,  13836, 49024, 19708, 31596, 63344, 46796,
                59976, 10352, 34440, 61336, 45168, 16300, 11004, 25384, 44596,
                8952,  55724, 26700, 16064, 58576, 940,   13640, 62528, 31792,
                59352, 2448,  23400, 44148, 20520, 5284,  35804, 3280,  22640,
                56432, 15236, 2560,  39860, 24264, 49020, 57240, 64512, 47180,
                16572, 29844, 41948, 8808,  37804, 3012,  33204, 12388, 19104,
                36140, 59324, 45132, 20152, 37764, 7860,  47640, 33960, 39612,
                49656, 128,   64680, 12640, 44368, 55392, 18952, 50836, 45376,
                4964,  65432, 14652, 23792, 4224,  38988, 53320, 42288, 32024,
                5472,  36908,
            },
            {
                29324, 51220, 42336, 25272, 55992, 34684, 64948, 10992, 1688,
                56976, 12428, 6148,  39568, 32568, 9016,  4748,  52872, 22676,
                35784, 27784, 16588, 44128, 30616, 46848, 17476, 59412, 2228,
                51688, 62520, 4248,  24340, 11176, 20292, 40108, 15560, 51840,
                268,   26920, 21028, 45540, 18040, 4236,  14420, 48168, 63052,
                12060, 58072, 45620, 23224, 39232, 2204,  10808, 26460, 41592,
                904,   38352, 53092, 3488,  30332, 50652, 33512, 64524, 5148,
                47740, 31408, 2872,  38332, 45544, 24784, 51768, 36960, 7616,
                45808, 12376, 36504, 62804, 50472, 27740, 60596, 52780, 10448,
                64312, 26332, 50464, 54480, 9076,  28360, 1228,  21952, 10836,
                36312, 60184, 19784, 13296, 58152, 65056, 28604, 55872, 61028,
                304,   30680, 6016,  11772, 65308, 27996, 18060, 12392, 53840,
                28900, 15480, 46400, 22776, 6852,  2024,  35120, 11380, 57128,
                25812, 36956, 18572, 54944, 30444, 8580,  61224, 17008, 11248,
                63164, 40444,
            },
            {
                57352, 7636,  18012, 12736, 39328, 4092,  19972, 49940, 24348,
                47676, 35028, 26860, 59568, 42940, 20512, 37516, 60744, 43784,
                10108, 65280, 53588, 12732, 748,   62924, 10296, 40672, 29256,
                22424, 45748, 38152, 49108, 56828, 60712, 30484, 9048,  19084,
                42248, 7236,  53884, 35908, 61100, 27772, 57340, 9448,  35476,
                40520, 1404,  64204, 17428, 54804, 44096, 57884, 15476, 21544,
                30776, 48736, 14084, 27260, 7988,  42368, 58712, 18680, 40328,
                22892, 14356, 50084, 64132, 6392,  18628, 56964, 41392, 26644,
                20044, 52900, 33156, 3772,  17068, 8712,  39472, 30148, 44636,
                19592, 37608, 32160, 12632, 63132, 34152, 42008, 38408, 55472,
                25256, 2612,  48840, 53508, 22896, 17308, 42852, 7260,  48396,
                26044, 53280, 41524, 23912, 49700, 1568,  60988, 25216, 6144,
                42680, 58736, 31608, 37880, 50148, 60344, 28308, 40820, 7604,
                32020, 49708, 9892,  47424, 58720, 35300, 22496, 28572, 50892,
                46344, 3816,
            },
            {
                22760, 35624, 54436, 31736, 45600, 58804, 16092, 29884, 43564,
                8076,  21332, 51104, 884,   17156, 49584, 12752, 25700, 1336,
                49152, 41344, 4156,  58284, 37560, 23756, 54156, 33796, 13312,
                57700, 18736, 32092, 14860, 5684,  36116, 2332,  46468, 55540,
                59980, 31804, 15156, 3176,  43024, 22532, 52160, 5260,  20060,
                27236, 31368, 8732,  34980, 28316, 5552,  33464, 50764, 8248,
                59040, 43568, 19028, 63924, 55224, 21724, 4112,  28680, 52704,
                36372, 60660, 10668, 21120, 35092, 30388, 11212, 4200,  49384,
                64848, 15816, 57460, 41600, 46576, 55680, 13984, 24440, 49392,
                632,   58312, 46660, 4760,  44020, 18780, 53332, 4256,  14700,
                62648, 44164, 32040, 5820,  35508, 46912, 10640, 40320, 21768,
                13496, 63016, 17028, 57412, 34436, 40340, 44532, 55344, 35864,
                21740, 3628,  17808, 56180, 26100, 14508, 47812, 64024, 22264,
                59380, 692,   39368, 27152, 5380,  15088, 44584, 2592,  55668,
                19328, 26568,
            },
            {
                60328, 48332, 2608,  63092, 9296,  26316, 36392, 61160, 54540,
                38124, 63308, 14448, 56288, 29124, 63792, 34008, 58120, 17720,
                30300, 20668, 33508, 26164, 48536, 19260, 5516,  42848, 49764,
                8272,  3104,  65136, 27788, 41276, 22548, 63208, 25940, 34400,
                12644, 23652, 65528, 49776, 11508, 32784, 37876, 62008, 43724,
                55476, 51588, 42504, 13040, 61024, 47688, 23588, 65532, 36716,
                54420, 4592,  24384, 35576, 46780, 38508, 15084, 11648, 56308,
                1004,  43788, 28028, 41864, 54004, 48200, 61284, 23496, 34072,
                584,   29384, 10612, 25136, 31444, 21572, 63596, 5984,  41456,
                16632, 8448,  28992, 23104, 15544, 58956, 26132, 30756, 50912,
                21136, 9132,  40076, 26724, 60460, 1288,  30492, 52612, 34676,
                4532,  37912, 46440, 9412,  3796,  29792, 14936, 8492,  59960,
                49144, 11196, 63656, 41404, 8944,  20304, 36420, 4456,  16760,
                44104, 13504, 62388, 21048, 42000, 51928, 64772, 37232, 13448,
                33708, 9644,
            },
            {
                43828, 13844, 28172, 20568, 41072, 52000, 188,   13136, 5636,
                18272, 41952, 31896, 10192, 45520, 4004,  40604, 7876,  46060,
                55864, 13428, 60496, 8628,  44660, 14824, 56824, 26824, 61352,
                38916, 24548, 44092, 12204, 54340, 50864, 16440, 43316, 4892,
                38520, 47084, 28824, 40156, 58580, 860,   16908, 46416, 24436,
                15508, 3804,  22060, 57040, 18724, 38068, 11676, 2640,  17180,
                26888, 12852, 60836, 10156, 1268,  31764, 60416, 48984, 33804,
                17884, 24252, 57944, 3304,  16216, 7728,  37744, 13232, 54728,
                44268, 38804, 6132,  60272, 52100, 2508,  37040, 33232, 54596,
                61812, 35260, 52580, 61008, 37884, 48660, 7288,  12156, 46372,
                35032, 57992, 17908, 50356, 14920, 55308, 24856, 63964, 15944,
                59132, 27360, 31968, 51004, 22192, 62076, 52296, 19296, 32056,
                39308, 24504, 51888, 34240, 2408,  53268, 43152, 31316, 54372,
                24936, 51176, 33812, 55912, 11128, 31456, 24556, 7704,  48952,
                62628, 39820,
            },
            {
                17208, 53628, 37488, 6640,  56552, 23184, 48444, 33316, 27960,
                58528, 2820,  24760, 36992, 54240, 21764, 27756, 52708, 23864,
                36584, 5308,  39900, 51652, 29716, 64400, 32416, 1412,  20672,
                52672, 35344, 48100, 59392, 7520,  32636, 10152, 57624, 20508,
                51492, 9204,  18360, 6028,  25240, 54120, 30280, 12596, 7016,
                59268, 36244, 48416, 26512, 48,    53112, 42900, 29916, 46616,
                39856, 33644, 49628, 41420, 17764, 51908, 25428, 44688, 6488,
                65292, 9332,  51196, 32212, 63440, 27168, 46356, 59044, 17360,
                22000, 62132, 48308, 18600, 12768, 44736, 58476, 19224, 10932,
                25568, 43324, 20796, 2928,  10432, 32700, 65400, 40596, 56480,
                28684, 1840,  64280, 11552, 36896, 43856, 5076,  19616, 49552,
                43384, 1752,  55604, 18400, 12504, 37464, 26492, 47192, 1064,
                13940, 28216, 5388,  46020, 30088, 63020, 11820, 57824, 38612,
                8100,  29428, 3736,  46024, 18484, 1512,  58268, 41116, 21408,
                29480, 672,
            },
            {
                24328, 31452, 65328, 46988, 15868, 10736, 62216, 39660, 44620,
                22044, 52472, 47400, 6228,  62092, 12168, 15964, 61060, 2056,
                49932, 63040, 16716, 22384, 2704,  11324, 37344, 46904, 15644,
                10568, 29988, 17264, 488,   21420, 37672, 47412, 29348, 62424,
                1356,  55936, 36196, 61768, 47888, 20332, 40964, 63516, 49892,
                32664, 10416, 64548, 40308, 7684,  32320, 62608, 20260, 51332,
                58468, 7096,  21148, 56140, 28412, 63332, 8564,  22100, 39936,
                29584, 46792, 37392, 12720, 40616, 20064, 1624,  42496, 31712,
                9008,  51424, 26912, 34820, 40704, 8232,  27604, 50560, 1652,
                30412, 13704, 49132, 40252, 55312, 24076, 80,    20252, 16380,
                5304,  24632, 42468, 31548, 21684, 61940, 9460,  32988, 38816,
                12092, 7868,  24484, 45028, 57984, 41868, 4196,  63832, 56332,
                42844, 60764, 16892, 56984, 22868, 15572, 27120, 1464,  19008,
                48192, 63816, 15848, 36556, 61400, 27592, 47228, 15984, 52216,
                5736,  59060,
            },
            {
                49704, 10020, 3292,  34640, 43528, 30464, 19076, 51008, 8536,
                15212, 64424, 34568, 17768, 43060, 30252, 38560, 44452, 10000,
                31880, 27116, 42572, 34308, 59524, 53236, 23976, 40796, 55652,
                4856,  63352, 56900, 40384, 27020, 64136, 3220,  14540, 40724,
                23316, 32016, 44776, 15428, 10316, 34364, 56624, 2048,  26040,
                38820, 17292, 29372, 45052, 14672, 58052, 25188, 9420,  15160,
                3360,  64124, 30872, 44256, 2968,  13556, 35688, 58356, 2356,
                14760, 55176, 19080, 5264,  56784, 23872, 52036, 28700, 4852,
                36480, 14632, 2000,  56524, 23480, 65216, 15864, 38712, 46280,
                62828, 56996, 5452,  17832, 28376, 46088, 35904, 43240, 61588,
                51372, 38056, 47104, 7304,  53692, 26940, 47896, 57920, 29140,
                61312, 52120, 35600, 64776, 6168,  32984, 10792, 23376, 35020,
                7844,  48732, 36692, 9904,  40012, 50448, 45288, 61124, 35004,
                10964, 21868, 42868, 7168,  53288, 38972, 32492, 10600, 56212,
                34316, 45572,
            },
            {
                39028, 22148, 60136, 26980, 57612, 5232,  25208, 1884,  56104,
                31212, 11580, 26736, 59400, 448,   50536, 58232, 34720, 21228,
                14676, 47160, 57584, 7552,  18956, 49040, 6344,  62396, 27624,
                43304, 33992, 23500, 12712, 44908, 50480, 19004, 34652, 52928,
                11892, 60504, 27392, 4184,  52692, 28568, 7544,  43920, 21860,
                53656, 3260,  56004, 20652, 49480, 35280, 4772,  54640, 44952,
                37728, 25660, 9836,  16108, 39072, 54168, 48456, 18596, 42912,
                62392, 34604, 26128, 49512, 61900, 35400, 11012, 60920, 55952,
                46992, 63192, 43588, 30476, 3848,  47444, 32760, 54072, 21728,
                7764,  36264, 32148, 63728, 12332, 58460, 53444, 8828,  30996,
                13140, 55860, 19168, 59260, 3316,  40256, 13964, 484,   22920,
                17972, 41092, 14812, 28152, 20836, 49924, 16644, 45652, 30040,
                18928, 54424, 21460, 62368, 3612,  33700, 6948,  24040, 51968,
                39652, 56324, 59596, 25620, 13112, 22180, 5212,  60876, 19444,
                26204, 12856,
            },
            {
                7332,  51792, 18552, 14116, 37736, 52880, 63312, 35348, 46188,
                38752, 6692,  49248, 39980, 23668, 8616,  18904, 5656,  63684,
                55492, 3576,  11652, 25092, 43976, 35920, 12944, 31468, 17612,
                8832,  48540, 3924,  53540, 31624, 6260,  59740, 25320, 43532,
                7628,  49272, 20948, 42428, 64468, 37676, 18124, 60892, 14256,
                46732, 62144, 6336,  24220, 11604, 61468, 41104, 29208, 21908,
                33112, 52992, 61068, 46860, 23672, 6288,  31284, 26836, 51024,
                10128, 22656, 76,    41912, 8400,  15584, 48320, 39440, 18888,
                25236, 11476, 17212, 52736, 20664, 59964, 11712, 4816,  42256,
                15132, 24508, 44340, 51036, 22088, 4572,  15304, 26456, 48716,
                22980, 33880, 10200, 29332, 16124, 52008, 34264, 46036, 56028,
                4800,  48696, 31412, 3312,  54576, 38780, 61432, 52668, 12948,
                41036, 2040,  26592, 31260, 13520, 59172, 41992, 16960, 30508,
                4780,  28232, 72,    33396, 48080, 64528, 44720, 1820,  36428,
                42736, 62812,
            },
            {
                32384, 55288, 40884, 816,   49004, 11852, 42072, 16320, 23420,
                61276, 19908, 56732, 14576, 33456, 54996, 45892, 26312, 52028,
                39804, 29484, 37520, 64616, 54140, 64,    21836, 45592, 58444,
                51740, 20232, 36916, 61612, 16564, 10456, 38668, 55784, 4436,
                17228, 36904, 57676, 344,   23848, 11660, 51072, 31228, 35564,
                9224,  41628, 34112, 52348, 38032, 2108,  16348, 47872, 57748,
                12816, 564,   18460, 34584, 57556, 62932, 11664, 37632, 4356,
                60128, 30700, 53396, 45588, 27944, 32452, 2404,  22296, 7108,
                33868, 41428, 58836, 38220, 6888,  35892, 25608, 56320, 29360,
                51876, 58832, 9844,  2060,  38140, 42104, 34208, 59656, 6504,
                63236, 2348,  45096, 61188, 36496, 20564, 65236, 25556, 8520,
                37696, 63028, 11112, 59336, 42764, 8900,  232,   27200, 58808,
                6060,  65460, 44436, 53068, 47332, 19840, 55764, 9732,  64812,
                46328, 14496, 50164, 18824, 8784,  40676, 28704, 54832, 50276,
                15176, 3776,
            },
            {
                27848, 44480, 24128, 64432, 28908, 21288, 32520, 9380,  53356,
                3956,  29728, 42832, 3000,  65496, 29120, 11148, 36128, 1592,
                23024, 17316, 50736, 32464, 14892, 28492, 60980, 38580, 3124,
                26176, 14416, 40896, 28912, 57568, 21876, 48028, 28320, 64988,
                30688, 54612, 14196, 33172, 47216, 39748, 59456, 2492,  49212,
                27040, 18560, 30140, 59332, 44248, 27500, 19504, 65032, 6404,
                39208, 43236, 49836, 28896, 3888,  42212, 20496, 56184, 47312,
                16364, 38672, 12320, 63712, 18328, 59700, 52884, 44408, 64804,
                54852, 420,   27452, 9864,  50504, 44976, 18260, 61272, 712,
                37836, 19280, 27696, 48036, 61340, 29744, 19980, 46444, 39904,
                18156, 53920, 41692, 24704, 49768, 5688,  11188, 43812, 29620,
                53752, 21844, 45368, 26148, 17848, 23688, 32084, 37076, 47920,
                22304, 33276, 16012, 7348,  38364, 25352, 1292,  36456, 54008,
                20892, 41580, 35960, 53056, 57848, 16132, 24228, 11840, 30820,
                21556, 58288,
            },
            {
                17352, 11268, 34232, 8112,  46812, 60268, 5468,  58276, 27368,
                48328, 36580, 10588, 51484, 21316, 41164, 16340, 59932, 48828,
                42320, 57924, 9240,  4164,  40844, 56640, 49792, 10204, 34268,
                65492, 55324, 1956,  8288,  42932, 34052, 956,   13104, 45160,
                23524, 40660, 9388,  62416, 26528, 16536, 5036,  22740, 57208,
                12356, 64708, 772,   15028, 8668,  56388, 50692, 31944, 10744,
                26432, 59812, 22260, 8556,  13920, 52380, 32996, 25024, 8012,
                43676, 57892, 21080, 6272,  36104, 24756, 12912, 37408, 29488,
                15100, 48648, 32536, 23132, 64072, 14376, 40644, 31544, 48792,
                12700, 64164, 33736, 16556, 54984, 8524,  52488, 1240,  12540,
                37100, 27872, 8216,  13544, 57604, 31760, 40900, 58892, 19116,
                13684, 33288, 1888,  36240, 49452, 64196, 55552, 15220, 10052,
                39816, 51228, 58092, 11392, 29312, 60592, 49332, 32216, 12680,
                26496, 7276,  63108, 30716, 3376,  37832, 60528, 6672,  47540,
                64004, 38036,
            },
            {
                46308, 51192, 4300,  55916, 36648, 16876, 44176, 39684, 13500,
                63064, 17328, 45220, 25576, 60840, 4860,  54052, 31388, 13316,
                6588,  20764, 60656, 47800, 24356, 19820, 5448,  16528, 42788,
                23148, 32052, 46096, 50112, 24676, 63032, 52248, 19408, 35724,
                3636,  48764, 20144, 6368,  51972, 45172, 30056, 53548, 42812,
                36980, 46004, 21900, 48644, 33324, 23800, 2984,  36768, 46108,
                55560, 15512, 35880, 64228, 39840, 45776, 17432, 1216,  65080,
                28456, 33948, 3096,  49784, 41044, 56204, 3952,  50316, 9316,
                21052, 61848, 42860, 4916,  55896, 28048, 3352,  9032,  22608,
                44700, 6376,  40460, 3828,  23388, 13664, 64728, 25944, 57784,
                50788, 32508, 64120, 21912, 908,   47360, 16432, 4212,  62196,
                48296, 7308,  58368, 52632, 13160, 4384,  44680, 20524, 62240,
                4020,  24316, 36060, 18720, 43536, 4244,  22124, 62296, 43868,
                2712,  59224, 23296, 11036, 44208, 20412, 33608, 52004, 41244,
                9284,  1516,
            },
            {
                57088, 62244, 20448, 13616, 25500, 53848, 1256,  51060, 22664,
                31924, 2152,  57540, 33864, 8228,  38128, 46720, 24112, 63328,
                33744, 44980, 27628, 35756, 12344, 62712, 30604, 46968, 53940,
                6528,  12056, 62004, 18412, 13992, 5660,  40064, 56704, 10068,
                60704, 53756, 28144, 58980, 36044, 10480, 63172, 19316, 7948,
                25340, 5716,  55460, 39776, 61600, 42600, 13768, 62888, 20204,
                4400,  51452, 30984, 2004,  54776, 27580, 60916, 36164, 51776,
                13460, 23468, 54440, 46320, 10272, 28580, 17656, 58584, 40200,
                2904,  53216, 18436, 12404, 46620, 33716, 53644, 62832, 35424,
                52308, 56492, 25292, 59992, 42796, 35824, 31476, 44580, 21220,
                5236,  15876, 43576, 55584, 38104, 26000, 53188, 35212, 39188,
                23752, 42460, 16796, 27732, 40576, 34552, 7600,  29364, 42016,
                56576, 27712, 840,   63944, 54792, 40084, 8652,  16472, 51796,
                37540, 47416, 15400, 27672, 48876, 56176, 4812,  26672, 18300,
                35288, 23580,
            },
            {
                15000, 39572, 31892, 42816, 61148, 30204, 19584, 34836, 8004,
                55036, 40376, 20660, 12992, 49224, 28212, 1188,  18856, 9912,
                39020, 53352, 15356, 1072,  43840, 51996, 37944, 26884, 59800,
                20872, 39460, 28352, 36668, 58984, 31428, 26952, 46816, 16376,
                32628, 42584, 1416,  39068, 22480, 14588, 41120, 33360, 59248,
                15800, 51512, 31092, 3872,  11272, 17872, 53340, 29824, 8116,
                40836, 24572, 47604, 18992, 10584, 23212, 5100,  49028, 9972,
                40364, 18576, 61120, 31072, 15284, 64108, 34936, 22984, 31840,
                47068, 26440, 35708, 39096, 59104, 16732, 24868, 42624, 19864,
                15260, 29080, 10796, 47208, 51292, 17596, 6100,  49052, 10724,
                61572, 34768, 3540,  29956, 18832, 60848, 10164, 28284, 2500,
                51152, 30740, 63284, 9964,  21652, 60468, 48692, 53716, 17568,
                12776, 49960, 46116, 31436, 14452, 48452, 34480, 28668, 57520,
                19484, 31800, 55008, 984,   39060, 65188, 13788, 45700, 61772,
                54048, 29668,
            },
            {
                6812,  26600, 48912, 9600,  3500,  38928, 11540, 64972, 47544,
                26988, 5744,  51948, 64048, 16148, 59000, 43740, 55516, 50152,
                25584, 5596,  65048, 31584, 56240, 8692,  4096,  14104, 35164,
                680,   57264, 52012, 9576,  43436, 3252,  20600, 63848, 7216,
                24516, 13012, 18044, 31164, 56336, 4520,  49840, 2268,  28564,
                38900, 63512, 19692, 27204, 58416, 37700, 25780, 44816, 60084,
                33696, 12764, 62516, 43560, 38000, 59356, 15624, 42668, 29708,
                56844, 2272,  37216, 25616, 7780,  41824, 48392, 5488,  61088,
                11072, 56692, 7360,  22252, 2380,  50100, 11184, 6036,  59584,
                1776,  38300, 32740, 20436, 376,   62860, 28008, 37396, 41332,
                23928, 53080, 46984, 11804, 50428, 6940,  44444, 56916, 20456,
                11960, 54484, 37204, 1232,  56768, 31672, 24988, 2568,  37240,
                33184, 60780, 9504,  20596, 6408,  59188, 23604, 11588, 4624,
                40540, 8124,  62456, 35076, 17760, 9088,  31632, 22576, 2352,
                12052, 43512,
            },
            {
                34168, 384,   64160, 22624, 50636, 58616, 45212, 24352, 15040,
                59640, 43192, 30228, 36380, 23220, 10820, 35152, 30772, 4072,
                58320, 17880, 48916, 22240, 39708, 19256, 25228, 63276, 41956,
                48956, 16660, 6936,  22628, 48008, 55644, 35128, 50452, 38392,
                44488, 61752, 51016, 46380, 65524, 25856, 44056, 54800, 10048,
                46524, 12972, 35320, 44424, 49740, 7048,  240,   48548, 21676,
                57000, 3348,  52524, 6912,  29000, 56224, 32856, 19876, 46664,
                62232, 6384,  52656, 44628, 59476, 568,   20220, 54116, 43716,
                16116, 30248, 65516, 45432, 28688, 61684, 36876, 30804, 49228,
                45604, 65036, 8152,  58756, 39664, 12264, 53388, 57248, 2444,
                19068, 59416, 26500, 40164, 63128, 23436, 33896, 15268, 64932,
                46180, 6300,  18800, 43620, 47356, 15344, 11644, 64980, 44964,
                5844,  22992, 38504, 42876, 26660, 52420, 37020, 64940, 46404,
                25468, 13352, 43288, 50580, 24148, 52824, 42652, 57580, 37032,
                50288, 59920,
            },
            {
                47176, 17436, 53412, 36416, 15764, 28600, 4808,  56684, 37324,
                10348, 18312, 24,    46376, 6756,  40768, 62756, 21224, 14156,
                42064, 36404, 28692, 11052, 61256, 47432, 33056, 53392, 10332,
                29892, 45140, 33340, 60156, 25956, 11348, 14928, 196,   29396,
                57668, 4160,  34772, 7416,  11376, 16996, 35416, 20820, 61804,
                23812, 1312,  53976, 8948,  22732, 63856, 34680, 15200, 9896,
                38912, 17636, 25552, 35200, 13672, 50248, 372,   8336,  26060,
                34548, 22104, 17248, 11904, 32220, 51444, 27692, 13192, 38108,
                23992, 1640,  41016, 52192, 8868,  20476, 41752, 54992, 26344,
                13052, 17996, 54288, 27356, 22956, 34372, 16084, 7516,  30768,
                45240, 14228, 8444,  36212, 17344, 4496,  41884, 49172, 29316,
                24752, 34704, 61720, 27008, 4956,  38952, 51724, 20248, 27372,
                58512, 15788, 54324, 63356, 3372,  17680, 41440, 580,   30972,
                54232, 60472, 20956, 6188,  29308, 59796, 3460,  16844, 28180,
                8480,  20228,
            },
            {
                56616, 40412, 12172, 43804, 7072,  33140, 18716, 41252, 3392,
                32516, 61696, 54204, 25732, 57032, 50544, 2880,  27296, 61136,
                46464, 7716,  54872, 2392,  42776, 14856, 58168, 3204,  18608,
                23712, 64792, 2580,  38060, 18096, 62440, 41648, 53272, 22268,
                10488, 27004, 19928, 54160, 29912, 39812, 58644, 6944,  31796,
                50304, 42216, 60052, 16988, 32008, 56536, 41504, 54496, 27676,
                65520, 31752, 46352, 61348, 20840, 40456, 63796, 53712, 38844,
                14080, 49908, 64788, 39332, 23372, 56512, 35928, 8504,  63240,
                32640, 49284, 18876, 13580, 33248, 57192, 15748, 4516,  22204,
                34888, 41008, 50060, 5728,  43480, 46940, 61808, 25968, 50832,
                65512, 33452, 1728,  58088, 53420, 31312, 59308, 4,     9056,
                40556, 13988, 50372, 58044, 22536, 33776, 55656, 8356,  40780,
                50648, 1680,  29556, 46544, 33076, 10112, 56720, 21388, 49904,
                16912, 34032, 2388,  46928, 39852, 10760, 33668, 48356, 40840,
                63252, 24208,
            },
            {
                4136,  29804, 58888, 25848, 62668, 55384, 47924, 51800, 27544,
                22372, 49688, 38588, 12684, 19884, 33148, 17060, 53312, 38228,
                12044, 32704, 60152, 23996, 35296, 5460,  28052, 39032, 50916,
                56416, 12584, 41184, 54236, 31296, 4832,  45644, 32948, 64556,
                15824, 48448, 41320, 60088, 23360, 676,   48204, 14008, 37872,
                18452, 28292, 5672,  39920, 13392, 29184, 19644, 4960,  50884,
                43100, 1432,  55736, 9456,  4320,  44768, 24204, 11320, 59724,
                5152,  43184, 29104, 9080,  46624, 4644,  18256, 58768, 44880,
                4252,  55076, 59988, 25948, 38732, 32,    47556, 62656, 9716,
                58456, 2236,  30348, 14864, 56660, 2912,  20960, 40920, 11432,
                38156, 22584, 49504, 28076, 44740, 13076, 21600, 36560, 55980,
                60760, 2960,  31036, 9740,  17104, 42480, 480,   62136, 32192,
                10812, 36336, 19436, 13208, 60356, 48688, 28812, 14348, 44120,
                8956,  38312, 57808, 26904, 64016, 14960, 21720, 55396, 5640,
                13860, 32808,
            },
            {
                45828, 9428,  19916, 1808,  35432, 21672, 8460,  12524, 63716,
                45392, 15656, 7432,  29224, 65196, 44064, 9356,  47744, 24520,
                1532,  20352, 51772, 16664, 49992, 64460, 21124, 44252, 7816,
                35676, 27460, 15496, 8540,  48744, 27828, 20164, 7992,  37904,
                56132, 2816,  36424, 13696, 45744, 62704, 27288, 52640, 57488,
                3396,  64876, 34516, 48024, 58792, 2160,  46564, 24656, 12536,
                36120, 16304, 22996, 49476, 28332, 36256, 17708, 31396, 47296,
                27284, 57424, 1376,  33956, 60276, 14560, 50228, 40056, 21584,
                28276, 9928,  35588, 5392,  63588, 44068, 28296, 32076, 51200,
                37788, 45008, 24400, 63948, 36868, 9488,  32504, 54700, 17816,
                6244,  56292, 41232, 19776, 9796,  64252, 25696, 51392, 16436,
                19960, 44516, 37984, 53612, 64236, 49220, 28732, 14308, 23508,
                47772, 57692, 25264, 41920, 7164,  23340, 35252, 59620, 5704,
                62940, 30160, 12348, 18884, 53924, 36528, 45024, 60568, 26200,
                51572, 38388,
            },
            {
                15996, 65372, 50972, 47060, 39576, 14388, 30788, 59128, 1496,
                34416, 55928, 42820, 2400,  35848, 58532, 4928,  31248, 56644,
                64172, 44672, 39832, 30344, 8312,  46156, 12144, 31576, 60856,
                1080,  46876, 61972, 22368, 58916, 52260, 13364, 60580, 24468,
                43596, 30904, 52744, 9468,  33312, 4440,  42508, 9072,  24564,
                45128, 11900, 20572, 25360, 9480,  52324, 37952, 63048, 8368,
                59340, 30552, 41076, 60508, 15280, 51816, 56528, 2588,  21128,
                36964, 16088, 53880, 20128, 42188, 30452, 26160, 1028,  52876,
                16456, 47484, 42920, 22852, 17888, 53300, 11752, 23876, 19232,
                6648,  60376, 11028, 19768, 52044, 27936, 60900, 48612, 624,
                29604, 60548, 15216, 4132,  47536, 38496, 6664,  42372, 32316,
                48364, 23768, 7292,  13128, 25824, 5436,  37288, 59844, 18036,
                43352, 3972,  62796, 53928, 37776, 3620,  52072, 40148, 25960,
                53052, 22292, 42688, 50120, 7476,  280,   29908, 9640,  18996,
                2248,  54740,
            },
            {
                35180, 23368, 31980, 6360,  61672, 54212, 44144, 25244, 40224,
                20744, 9996,  60884, 23760, 48756, 14048, 21680, 41852, 16040,
                28368, 5624,  13300, 59404, 25480, 37448, 56872, 16448, 24772,
                53896, 19196, 43260, 36680, 3692,  39932, 34132, 1644,  50160,
                18200, 6320,  63500, 22028, 16752, 56412, 20116, 35000, 15416,
                30584, 55112, 50732, 42872, 61668, 32648, 17172, 44284, 55280,
                20416, 46892, 5988,  11964, 32792, 64700, 7892,  41280, 62384,
                50744, 9868,  24768, 61780, 49032, 7824,  64268, 34648, 12416,
                62176, 31384, 57508, 13092, 49488, 7976,  34412, 56740, 41248,
                15884, 53732, 33740, 48032, 3988,  42180, 14148, 23520, 45064,
                35188, 52096, 24900, 33012, 56972, 28808, 53676, 11316, 4784,
                63552, 29092, 58624, 46840, 34868, 20712, 45328, 54460, 6960,
                51788, 33820, 9376,  30352, 17152, 64560, 11908, 19568, 1528,
                46056, 35472, 3920,  57060, 25448, 47756, 41448, 33284, 63700,
                43252, 27444,
            },
            {
                58440, 11292, 42316, 17672, 28016, 3708,  36924, 16612, 5792,
                52748, 31196, 17364, 39564, 53992, 27488, 62892, 37368, 49436,
                10336, 35492, 54664, 19020, 68,    52304, 5000,  40424, 49352,
                33856, 6096,  30372, 11124, 25284, 16700, 55616, 46764, 29200,
                11980, 58340, 26720, 47820, 39088, 29740, 51540, 63620, 40776,
                60388, 7492,  524,   36516, 14328, 22352, 6120,  28252, 33924,
                2548,  26300, 53828, 38028, 992,   43372, 25836, 13892, 34136,
                3756,  45336, 39824, 12592, 2760,  37320, 22748, 56748, 41560,
                5944,  37900, 2008,  26916, 40276, 59776, 3168,  46252, 64192,
                1548,  29496, 39624, 26220, 17088, 58236, 7020,  38576, 64188,
                12004, 8240,  43164, 62412, 1152,  18480, 61408, 35100, 15092,
                54732, 952,   39988, 17512, 61520, 2168,  31272, 12036, 39152,
                27968, 21796, 14012, 45912, 50136, 27348, 43752, 32864, 55712,
                16076, 10692, 65232, 31716, 13576, 17240, 58056, 23156, 12664,
                49412, 7172,
            },
            {
                39404, 596,   52936, 60032, 24056, 10420, 50572, 64484, 47184,
                58428, 26080, 45596, 4660,  11252, 33428, 7760,  976,   52364,
                23028, 61880, 42168, 47796, 33376, 62872, 28840, 10252, 21508,
                63632, 13916, 57180, 50856, 65312, 44212, 6844,  62148, 21632,
                41168, 35056, 1016,  43380, 61168, 12408, 6428,  1796,  46556,
                21988, 38568, 17840, 26740, 54012, 3288,  58400, 40968, 13596,
                49376, 62080, 18304, 58220, 23536, 19804, 48180, 55252, 28796,
                59032, 17924, 32372, 27140, 55544, 46132, 18592, 9532,  50492,
                24536, 19780, 54028, 64744, 30008, 20788, 37260, 25916, 12708,
                49568, 22508, 8284,  63268, 44752, 34540, 30580, 55824, 18308,
                27400, 48996, 20828, 37248, 13612, 45968, 23052, 49524, 27028,
                43760, 20312, 32580, 10768, 52132, 41464, 57376, 24308, 65364,
                2952,  56116, 40924, 61700, 148,   22340, 58588, 7756,  48052,
                60616, 28128, 40716, 20440, 38788, 62324, 51236, 5428,  37096,
                56048, 20792,
            },
            {
                61440, 47876, 13816, 34696, 45296, 56300, 19672, 28876, 12412,
                35548, 560,   63388, 51656, 20124, 56196, 44696, 18224, 57680,
                32200, 3184,  26996, 8860,  22120, 13636, 43872, 59304, 3076,
                42236, 38348, 47240, 2252,  28324, 19888, 32060, 9752,  37796,
                54248, 15312, 51824, 8120,  23724, 36200, 48928, 26064, 10732,
                33136, 56908, 48608, 63180, 30108, 45292, 50412, 64840, 23960,
                39236, 10072, 34816, 45068, 51108, 30844, 10840, 38784, 5652,
                22208, 53208, 65204, 6676,  15908, 59760, 33392, 43964, 28556,
                60880, 46552, 8720,  14876, 44716, 4864,  55436, 16880, 31668,
                43156, 59296, 55588, 13564, 1060,  50664, 10504, 22092, 2208,
                53592, 40184, 5876,  55152, 30864, 8884,  40588, 3244,  59652,
                37072, 8420,  56580, 49368, 27468, 14948, 7952,  47700, 16496,
                35836, 49240, 18848, 6412,  34960, 39268, 14260, 29532, 37244,
                23936, 6464,  49516, 54100, 8744,  34500, 1604,  28892, 46768,
                15704, 31144,
            },
            {
                25988, 18812, 29764, 7796,  38904, 1684,  32032, 42616, 8096,
                21756, 41468, 14820, 37164, 29276, 61712, 39004, 26228, 11848,
                40272, 46688, 16296, 58208, 50996, 35860, 17988, 54796, 31856,
                26136, 16284, 8864,  22752, 53852, 36432, 14200, 58880, 25632,
                3644,  64636, 20240, 32264, 55624, 16104, 58840, 19112, 53344,
                28828, 4640,  12496, 40260, 7852,  35036, 10944, 19676, 6776,
                53124, 29832, 15448, 7640,  4176,  60632, 16816, 63568, 44000,
                9312,  36192, 48476, 42344, 30028, 51412, 4272,  14000, 2396,
                32312, 16852, 36216, 50700, 33616, 11136, 52068, 61292, 9020,
                35612, 5352,  18664, 37616, 24808, 61264, 41172, 47580, 59564,
                32292, 16164, 63628, 24452, 50960, 58668, 33264, 16668, 52252,
                12568, 24168, 64740, 3088,  22332, 36884, 62484, 29904, 43912,
                9992,  60732, 31648, 26216, 57176, 9656,  50780, 63952, 2240,
                42476, 18364, 59164, 3556,  26612, 45080, 21304, 60404, 9976,
                42020, 4088,
            },
            {
                54780, 43452, 63200, 21700, 57956, 48552, 15464, 61592, 51964,
                57436, 32852, 47584, 9424,  24084, 2284,  15196, 48240, 60096,
                20728, 4736,  63680, 30408, 39172, 2360,  24448, 48128, 7244,
                52988, 61128, 34908, 58536, 40536, 46168, 388,   50512, 42272,
                47368, 28528, 11556, 45308, 4976,  28032, 42292, 38200, 65200,
                44312, 16232, 59864, 20588, 24612, 57512, 15808, 31900, 47044,
                152,   42888, 64088, 27376, 37208, 54568, 33304, 1756,  49900,
                24596, 14180, 872,   20700, 10232, 38532, 22336, 63744, 53720,
                40692, 59016, 22880, 1244,  62920, 24408, 41012, 47116, 21248,
                27720, 51636, 45900, 29992, 54088, 19732, 6292,  28396, 13748,
                44460, 36096, 3040,  11504, 28244, 43420, 21376, 7076,  62816,
                45736, 30216, 42424, 34348, 46388, 5224,  52712, 19932, 892,
                54076, 22964, 12368, 42596, 52676, 16976, 44924, 20632, 54888,
                12544, 32724, 45880, 36460, 11668, 56632, 16772, 52864, 24276,
                64800, 35856,
            },
            {
                51260, 2956,  10624, 50348, 26692, 5324,  35776, 23804, 3768,
                27508, 18504, 6296,  58620, 42944, 50928, 35532, 5976,  29672,
                54912, 34856, 12620, 45036, 6584,  55988, 65020, 11296, 37504,
                44836, 19128, 29264, 5936,  11888, 18024, 63232, 23208, 5316,
                17372, 33720, 57816, 39992, 50724, 61944, 12988, 3680,  8532,
                23144, 35816, 47064, 52336, 1804,  43272, 37920, 54952, 26020,
                59704, 21452, 55848, 47792, 13360, 41660, 21176, 26868, 57740,
                30588, 40000, 56220, 60628, 25624, 54548, 47248, 34976, 26208,
                11524, 5724,  56456, 43128, 28408, 18756, 6916,  38160, 476,
                64908, 14776, 41416, 3448,  11300, 32816, 36536, 64496, 4148,
                25156, 56816, 47016, 18776, 60636, 1476,  48604, 39012, 26604,
                19192, 6452,  54180, 13792, 18388, 58516, 40304, 25676, 34004,
                38896, 5008,  46964, 59420, 2532,  23716, 33480, 4336,  26804,
                60932, 51852, 15460, 22944, 62716, 31256, 43140, 39184, 33180,
                7448,  13056,
            },
            {
                23116, 38340, 33636, 16764, 41048, 65140, 12192, 46288, 39276,
                55272, 49596, 64316, 30976, 12996, 54152, 22136, 65304, 8936,
                42660, 24132, 52432, 19228, 26484, 41460, 15532, 30100, 22400,
                828,   13256, 64264, 48492, 26664, 56076, 33988, 30180, 54520,
                60924, 37352, 9140,  25040, 1616,  21336, 34408, 31108, 50320,
                55480, 2776,  30728, 10476, 63780, 27896, 5060,  61536, 12364,
                36500, 17412, 32440, 2384,  24192, 8424,  52588, 46784, 11716,
                62300, 18064, 33964, 45420, 5368,  16884, 61576, 8276,  19304,
                49236, 30496, 37536, 13604, 48400, 60396, 31064, 58140, 12212,
                33540, 56268, 23096, 62772, 47972, 57652, 16384, 43064, 50436,
                9680,  20536, 52368, 32788, 37580, 14288, 55876, 10124, 35156,
                58180, 2020,  39448, 60948, 29244, 10260, 48564, 12772, 56928,
                64220, 28256, 15536, 36476, 30544, 63204, 40244, 49048, 36004,
                8620,  29868, 492,   40264, 48352, 5848,  13888, 2096,  49632,
                57256, 18020,
            },
            {
                59200, 46128, 61000, 6976,  30940, 53624, 20316, 59672, 9540,
                16228, 2112,  25192, 40044, 19340, 3328,  45720, 32376, 17340,
                49752, 600,   37684, 61416, 9628,  33780, 45936, 59708, 51584,
                56820, 42388, 32552, 52300, 2664,  43496, 9632,  39608, 7148,
                13624, 22144, 52276, 63624, 17032, 48012, 57260, 43580, 15324,
                61204, 25908, 41716, 17464, 33664, 46016, 22544, 49952, 41208,
                4368,  9648,  51284, 38260, 62960, 58520, 15832, 35080, 6656,
                43204, 3044,  50204, 28588, 12984, 31744, 41908, 228,   44756,
                52932, 65500, 21004, 8640,  54848, 3284,  15688, 52508, 26580,
                48312, 4740,  17640, 38328, 26048, 8156,  52496, 22512, 39408,
                30356, 62600, 7272,  41828, 4788,  65448, 23924, 53084, 31532,
                47732, 15740, 50232, 23216, 4396,  32832, 62048, 6752,  17528,
                44232, 49720, 8548,  19796, 55904, 6820,  11400, 15224, 57400,
                42172, 19564, 64644, 53528, 9472,  28356, 58852, 20280, 26336,
                44264, 30364,
            },
            {
                1392,  27764, 13400, 55556, 24644, 108,   43376, 28464, 33196,
                37588, 44428, 11180, 34748, 61244, 27768, 56020, 41040, 11428,
                58488, 28736, 14360, 57096, 48884, 1844,  20096, 5580,  25588,
                35396, 4124,  21232, 38404, 24588, 14796, 59424, 19852, 45608,
                49300, 3344,  42656, 29808, 6856,  36624, 26524, 9692,  20068,
                39692, 6160,  49316, 59244, 54656, 14144, 8268,  18588, 30516,
                45476, 57212, 26704, 42448, 19236, 30016, 3528,  39180, 20320,
                54096, 23532, 8920,  64404, 38212, 58008, 21864, 36152, 28220,
                16072, 3800,  41472, 25544, 34796, 45636, 39500, 22700, 43904,
                36568, 9848,  30792, 59744, 1672,  45176, 12932, 27504, 440,
                58164, 34804, 15520, 26564, 45508, 30096, 17684, 40976, 6700,
                21688, 63908, 36736, 27096, 45268, 54428, 41960, 24244, 31432,
                2144,  22016, 61228, 41872, 51012, 25724, 45580, 61840, 22724,
                5300,  46708, 25168, 34036, 18264, 37792, 54660, 63096, 4548,
                37224, 9556,
            },
            {
                53108, 20748, 42404, 48040, 35124, 17896, 51664, 7240,  63612,
                22800, 53384, 59428, 47196, 7912,  15588, 36808, 4504,  25404,
                61928, 47008, 7088,  22592, 30664, 53380, 63072, 38656, 10468,
                49100, 15172, 59076, 8352,  62688, 47496, 35584, 28472, 65144,
                25420, 32880, 55716, 14392, 46432, 60716, 53968, 1148,  64104,
                29240, 12612, 36252, 21180, 1036,  39280, 51860, 65464, 34672,
                23640, 14792, 61632, 5540,  11896, 50072, 44520, 65064, 27912,
                59148, 36872, 16264, 53032, 2136,  48784, 10356, 55772, 60264,
                12256, 32980, 61892, 51676, 18216, 11088, 63708, 7572,  19580,
                61676, 54464, 50896, 41788, 20480, 35160, 54624, 61032, 18448,
                46592, 11784, 55360, 22248, 58684, 50252, 12400, 0,     59600,
                43540, 12088, 8812,  56860, 1252,  19648, 35640, 14896, 51548,
                55260, 37712, 32728, 13680, 616,   29392, 17976, 32348, 51468,
                38704, 13156, 59288, 3192,  49956, 44868, 23592, 32768, 15736,
                47524, 64388,
            },
            {
                40040, 16188, 4348,  62776, 10956, 58672, 39728, 14472, 49164,
                4428,  20040, 29720, 644,   50384, 23504, 63256, 52288, 20516,
                39144, 32776, 18128, 41840, 36640, 12276, 27536, 16964, 41192,
                61776, 28792, 46148, 18532, 31264, 5416,  52948, 1852,  16684,
                41264, 10576, 59804, 19532, 39436, 11584, 23036, 33476, 45232,
                51744, 57828, 24580, 9196,  61924, 29464, 25388, 3516,  58716,
                53604, 1384,  47528, 35652, 55212, 32840, 24804, 9604,  48220,
                13332, 31540, 41632, 26708, 19824, 43668, 24968, 4552,  46248,
                38956, 23188, 47272, 1040,  58020, 31688, 28056, 56556, 2768,
                13480, 24696, 6316,  15372, 28532, 65208, 4340,  32168, 42488,
                5556,  37896, 48524, 2552,  8880,  36172, 62476, 26052, 46832,
                28872, 33892, 51980, 16808, 39756, 64996, 8148,  46740, 60196,
                4716,  25124, 10716, 47464, 65084, 38356, 55024, 9500,  1996,
                27824, 55796, 30932, 16184, 62288, 11796, 7768,  41664, 52060,
                12420, 34160,
            },
            {
                25636, 56364, 31888, 37824, 21816, 3436,  45260, 25112, 31244,
                55940, 12792, 41944, 17452, 57332, 43636, 31236, 13956, 8388,
                44612, 2940,  55072, 64436, 4288,  44508, 57728, 50904, 32072,
                6792,  23700, 54264, 11416, 40648, 57164, 22040, 12464, 58196,
                50224, 36156, 132,   27412, 32232, 3916,  50096, 18048, 37800,
                4556,  16692, 42772, 48260, 33016, 45116, 16792, 42136, 11152,
                20428, 39200, 28932, 16452, 22216, 60252, 404,   17964, 56344,
                5616,  45884, 3896,  60440, 33848, 14276, 63300, 30276, 18748,
                53812, 6740,  27516, 14940, 38168, 5056,  42768, 49244, 35184,
                40792, 46360, 32940, 57232, 49492, 10008, 39036, 14732, 51396,
                24464, 64100, 28452, 16972, 43616, 31348, 20472, 56360, 38444,
                18636, 2964,  60372, 30592, 22416, 49756, 26784, 11496, 29436,
                18336, 41240, 58868, 53228, 20392, 5812,  42700, 34824, 59916,
                20968, 43816, 7068,  40664, 36276, 27680, 1116,  60720, 29320,
                22500, 6508,
            },
            {
                61916, 44060, 8256,  50080, 29272, 53452, 33936, 60664, 9924,
                37104, 63036, 26588, 33684, 38516, 10572, 3652,  47516, 35716,
                57592, 26968, 10776, 48212, 15136, 24456, 8040,  21500, 2436,
                56244, 43460, 34708, 520,   50992, 25668, 43900, 38368, 30224,
                6112,  23332, 51988, 43972, 64364, 56676, 41584, 26084, 8280,
                60280, 31656, 54276, 3836,  13440, 57012, 6712,  36440, 48616,
                32272, 8220,  64292, 43864, 6524,  37852, 53000, 40892, 34732,
                21596, 61876, 11312, 55456, 49888, 6612,  40160, 51136, 34468,
                9416,  57464, 43712, 50292, 60936, 20676, 24028, 53580, 16780,
                29492, 63432, 21776, 2584,  43932, 24944, 19560, 59280, 8968,
                20724, 33556, 54444, 60180, 39700, 52548, 4008,  14912, 7904,
                53908, 24492, 48068, 42440, 13444, 6084,  58036, 38096, 43836,
                63672, 33788, 7452,  28084, 15060, 23896, 46836, 62996, 16768,
                49548, 10904, 52720, 23288, 57752, 20104, 54084, 45812, 18496,
                58292, 48620,
            },
            {
                756,   18972, 12616, 24712, 63988, 15772, 6124,  19600, 47832,
                2740,  44560, 52652, 5544,  21732, 60020, 54336, 29008, 63968,
                16192, 22972, 52188, 34180, 29164, 60964, 35752, 47348, 63696,
                14208, 37932, 20272, 64624, 15780, 33188, 9152,  63176, 19184,
                46092, 61868, 8960,  16388, 21656, 6576,  14176, 62348, 46680,
                11972, 22168, 27780, 40352, 19488, 62944, 52868, 27280, 59940,
                13828, 51644, 57444, 25828, 10844, 46852, 14368, 62908, 29644,
                51092, 25172, 38360, 18100, 29344, 22856, 58304, 1388,  15700,
                64328, 21616, 30648, 11768, 34632, 8204,  65012, 11308, 140,
                59156, 8816,  38664, 12588, 61464, 35956, 47420, 53632, 29340,
                44780, 1748,  13484, 6736,  23772, 11260, 34104, 49036, 41776,
                63676, 10896, 37012, 4220,  62588, 34476, 52820, 928,   20896,
                14564, 3508,  45312, 36076, 57312, 31404, 2624,  13044, 26112,
                37612, 33332, 64928, 4120,  47472, 14100, 32364, 38280, 5172,
                10816, 36492,
            },
            {
                27240, 54856, 59888, 39364, 1788,  42948, 56612, 27992, 40532,
                16676, 23200, 8656,  61936, 14852, 41000, 25652, 19552, 50088,
                2052,  40876, 60204, 19712, 336,   39372, 54304, 18476, 28040,
                9772,  49876, 4680,  29576, 59948, 47736, 2908,  54564, 34532,
                14692, 39628, 29072, 54148, 47940, 37108, 30236, 52792, 34736,
                55852, 832,   64756, 51000, 35332, 10288, 21664, 296,   43444,
                23364, 4380,  40584, 18924, 50244, 31772, 27052, 3160,  8476,
                15920, 43332, 880,   47372, 8800,  36632, 12152, 45584, 25212,
                41568, 37120, 2656,  17792, 52844, 45072, 39880, 32692, 47384,
                19364, 26856, 51056, 54844, 17608, 30500, 868,   5804,  40316,
                63116, 36292, 50424, 19060, 46656, 64860, 26408, 58744, 22176,
                31572, 16524, 57628, 28000, 44828, 15732, 23544, 31120, 48088,
                56408, 51624, 22596, 9728,  49072, 39744, 52644, 58112, 7608,
                45824, 144,   17852, 29724, 42524, 8788,  63004, 24604, 50184,
                41432, 31324,
            },
            {
                51708, 45860, 33412, 21032, 48468, 35872, 12180, 51328, 65504,
                35192, 57688, 48852, 30732, 46220, 1296,  34084, 12016, 7388,
                37644, 31000, 9108,  43852, 50396, 6208,  11976, 41712, 33432,
                45136, 25252, 57484, 40300, 12308, 21528, 27988, 50680, 4424,
                25048, 57704, 2376,  33656, 11700, 59588, 1692,  23452, 19300,
                42972, 38052, 15640, 7652,  25680, 47324, 30840, 38488, 17236,
                63508, 29528, 34892, 1780,  61200, 56472, 20692, 45464, 53836,
                36040, 63748, 31992, 59508, 52940, 42484, 62568, 28096, 54668,
                4988,  61220, 47852, 26768, 59852, 4664,  25400, 13948, 56876,
                36248, 43368, 3996,  33624, 7932,  41688, 56152, 25840, 16920,
                10540, 56864, 26776, 41988, 30576, 54192, 1868,  37732, 6548,
                46032, 728,   52448, 19744, 9112,  55620, 39668, 61724, 10324,
                26576, 37168, 60260, 17544, 62364, 5116,  19748, 28372, 41916,
                22036, 56092, 60768, 25752, 51432, 35616, 55568, 2264,  13228,
                65100, 16480,
            },
            {
                3188,  9580,  14968, 5280,  58824, 26028, 7024,  22220, 31976,
                916,   13756, 27616, 36848, 18032, 51244, 56260, 43020, 62044,
                46292, 58148, 14164, 25832, 65508, 31620, 22844, 59316, 1560,
                62224, 17176, 53232, 7900,  45492, 37124, 17380, 41496, 60408,
                11092, 49372, 43256, 18836, 25852, 40504, 45556, 9684,  28672,
                5560,  49280, 32600, 56736, 44316, 3568,  61404, 55976, 50672,
                11404, 45924, 55264, 15240, 36932, 6020,  42352, 11832, 58068,
                23236, 5348,  13560, 26520, 20584, 16196, 5800,  32356, 18896,
                50588, 13512, 33000, 42184, 55792, 16160, 30200, 51408, 6196,
                63084, 23832, 14832, 47948, 64344, 23080, 13320, 60644, 48832,
                32484, 21932, 62024, 4376,  9432,  16352, 44400, 13356, 29048,
                62040, 35528, 40852, 25324, 32588, 49420, 3200,  18244, 42196,
                6992,  32156, 1648,  42804, 25592, 11936, 33852, 64216, 50272,
                31080, 14432, 39584, 11576, 6232,  21628, 16864, 30312, 43892,
                56980, 22276,
            },
            {
                38476, 60828, 42720, 30744, 53984, 17128, 46012, 61132, 9544,
                43428, 52968, 4604,  59300, 9856,  64764, 23384, 29652, 16776,
                21692, 4304,  54408, 36464, 17784, 56396, 46064, 15308, 51936,
                5160,  30980, 35828, 23392, 1168,  55640, 63864, 6908,  31356,
                22364, 36720, 62840, 5648,  55692, 64672, 15924, 50760, 58264,
                62740, 12924, 23048, 59996, 18332, 39720, 14780, 5188,  33252,
                7456,  21008, 25496, 48768, 9712,  24188, 64900, 33004, 38740,
                18984, 49664, 55844, 39848, 2440,  35268, 56604, 46588, 10360,
                39344, 23780, 7624,  20492, 1220,  38004, 62016, 21548, 40912,
                10120, 53816, 29196, 58316, 19904, 45320, 34608, 51976, 38040,
                6832,  14424, 47660, 38832, 35060, 61248, 20704, 56920, 51128,
                18696, 11692, 47668, 64500, 6660,  60044, 28576, 13212, 54416,
                65268, 46936, 14124, 29928, 55104, 45048, 38232, 15896, 4608,
                9264,  53736, 34764, 48528, 58872, 45264, 38792, 60340, 26340,
                7064,  35052,
            },
            {
                28716, 49808, 23152, 64516, 10720, 40860, 28860, 37568, 19372,
                55972, 24860, 40360, 20736, 44904, 6652,  38636, 3268,  53100,
                34944, 27704, 47908, 10824, 3340,  40212, 8428,  27212, 37780,
                20908, 43748, 13800, 58816, 49148, 32804, 25748, 14372, 46700,
                56592, 17036, 8768,  31016, 21432, 35704, 3676,  32400, 39392,
                17532, 46808, 2336,  29352, 10852, 53916, 23844, 27980, 37292,
                58740, 41936, 62308, 30968, 39724, 52404, 16600, 27888, 1508,
                46512, 30288, 7972,  44112, 65340, 51308, 22580, 60620, 292,
                29472, 64588, 52292, 59220, 11068, 48908, 34576, 3572,  46164,
                17960, 32204, 1052,  37476, 5992,  27864, 9624,  2756,  18920,
                43516, 28844, 288,   56036, 51844, 24748, 32656, 41492, 8568,
                23572, 54772, 4448,  14804, 21840, 37436, 46184, 34044, 20604,
                24312, 38884, 58212, 20108, 8060,  51388, 1024,  59624, 47520,
                24896, 61888, 1784,  18612, 27972, 3808,  52144, 10228, 48104,
                19456, 53704,
            },
            {
                12836, 4392,  18196, 36972, 332,   51716, 57456, 6212,  49184,
                15320, 34596, 62620, 12472, 32848, 26324, 14680, 48516, 60940,
                8164,  41604, 62760, 23836, 30188, 49844, 63364, 33916, 57836,
                48376, 10524, 64852, 28620, 19264, 9840,  43412, 53532, 39348,
                1612,  26880, 52764, 45016, 48968, 12576, 53536, 24236, 7376,
                27132, 36708, 52912, 42244, 34828, 48856, 65016, 43952, 19344,
                49724, 1092,  13508, 4100,  18828, 59004, 5052,  50444, 61740,
                10248, 59912, 24736, 33704, 11528, 27436, 14312, 37652, 43548,
                55220, 15632, 35700, 45164, 28888, 23444, 54756, 13280, 26964,
                60288, 49128, 42156, 12140, 61508, 40432, 57496, 31156, 65156,
                53616, 59360, 23124, 18112, 12208, 5860,  48680, 3228,  38648,
                60672, 27496, 34588, 44012, 53240, 9816,  57636, 124,   51808,
                4724,  10804, 48588, 33364, 63548, 22824, 27588, 18164, 35796,
                40984, 20920, 43884, 32624, 63972, 14784, 23732, 34172, 652,
                41084, 63336,
            },
            {
                45400, 57788, 26388, 47808, 34164, 14596, 24480, 3560,  63912,
                30064, 46628, 2480,  49728, 54024, 58224, 42588, 32012, 11640,
                19920, 56812, 15696, 44928, 53800, 18820, 1912,  12820, 24424,
                6624,  54820, 41348, 4140,  38868, 61424, 3172,  21108, 12424,
                33940, 59716, 41068, 15012, 1012,  28864, 41904, 61860, 44856,
                56892, 4700,  63460, 20188, 8016,  1552,  32196, 12532, 8912,
                53652, 27220, 34620, 54916, 47100, 42840, 35516, 13884, 22284,
                41156, 15812, 53688, 18656, 3480,  49080, 31344, 6668,  20076,
                48000, 26244, 6056,  40436, 18372, 8588,  43180, 64856, 38304,
                7996,  56620, 22976, 53168, 17320, 50828, 22116, 46200, 25076,
                11032, 40196, 33976, 45628, 63840, 29564, 58364, 15676, 45352,
                31460, 1352,  57976, 40140, 30132, 16672, 25804, 40636, 62884,
                44276, 28380, 16940, 5700,  36828, 42284, 57276, 10536, 53144,
                29468, 13108, 57448, 8496,  37760, 54172, 42140, 61480, 30692,
                16152, 8680,
            },
            {
                32904, 39620, 54720, 7960,  62552, 21240, 44388, 33164, 39456,
                10984, 22652, 8108,  28628, 37148, 18320, 208,   51660, 25328,
                39216, 1744,  33120, 5576,  37092, 22032, 43456, 60944, 39384,
                17540, 32332, 26016, 51156, 16000, 34904, 56948, 28904, 47592,
                65444, 4560,  22804, 37756, 57228, 60232, 18708, 10212, 21340,
                14868, 31652, 12076, 25820, 60540, 55336, 22224, 46884, 59540,
                16736, 38964, 64044, 21448, 8760,  25816, 29776, 55700, 45120,
                36572, 2660,  47404, 38244, 57408, 62320, 40956, 9548,  58896,
                33900, 2172,  12804, 62880, 51596, 57224, 31660, 1492,  19892,
                34236, 15348, 5404,  26452, 35468, 1920,  14244, 7380,  36728,
                16520, 4584,  51116, 8360,  42592, 36772, 26640, 10460, 65440,
                17856, 48924, 12636, 20032, 63572, 5780,  50156, 31204, 14532,
                35340, 54188, 59808, 50608, 13548, 3440,  31704, 45444, 6512,
                64884, 2948,  47056, 25640, 49676, 5044,  12028, 20284, 55892,
                49232, 24960,
            },
            {
                21056, 3464,  15672, 30144, 41704, 11592, 59972, 53136, 18464,
                58652, 42784, 55548, 15652, 61368, 6136,  22012, 59368, 44540,
                64648, 28980, 49120, 13532, 60228, 55948, 9212,  28776, 51680,
                45204, 92,    21880, 59468, 7784,  44920, 24008, 50776, 17552,
                9336,  30472, 49864, 10912, 25792, 32680, 6532,  48584, 35064,
                52220, 41180, 49804, 45704, 37768, 14624, 40752, 4744,  30504,
                24292, 5968,  45060, 32244, 60796, 12312, 96,    64356, 7256,
                31692, 26392, 63496, 6184,  21380, 44732, 24528, 52624, 16840,
                61952, 22808, 49580, 32744, 4168,  15072, 25764, 47032, 52592,
                29040, 44160, 62472, 46684, 31916, 63732, 42828, 49460, 56108,
                27620, 61392, 31500, 13924, 21364, 1576,  54260, 50796, 22764,
                37564, 7656,  51952, 24620, 2432,  45616, 60304, 22436, 8780,
                19492, 1864,  25692, 40208, 20504, 54592, 61176, 14764, 23516,
                50432, 39352, 16888, 31320, 21536, 58656, 28520, 44616, 6544,
                36064, 60120,
            },
            {
                11204, 44300, 59112, 51360, 1828,  27924, 47048, 7320,  26036,
                1112,  51472, 35216, 25128, 45660, 40720, 30796, 9360,  35604,
                17120, 7268,  52408, 40960, 26320, 31832, 3908,  35712, 14352,
                56912, 63420, 37036, 47376, 12100, 31424, 62652, 6140,  38092,
                42632, 54840, 19640, 63436, 46532, 3120,  54492, 38796, 65104,
                428,   23556, 58612, 3960,  17664, 27800, 50792, 35236, 62084,
                42412, 51508, 14808, 3512,  37704, 48020, 52852, 20484, 16820,
                57960, 51340, 12840, 29580, 35008, 15152, 4036,  28748, 37044,
                42088, 55856, 27664, 39024, 59100, 21708, 41388, 6900,  59536,
                10960, 39512, 3300,  20264, 54860, 11412, 29432, 60104, 20160,
                1104,  47452, 55200, 25604, 62672, 47096, 18816, 32916, 4968,
                43692, 27084, 59608, 32796, 55816, 35636, 12236, 42744, 57360,
                38100, 62248, 46304, 30480, 9252,  47964, 26360, 37980, 19188,
                33440, 55880, 10280, 62948, 1128,  35568, 15392, 65436, 39296,
                1768,  52400,
            },
            {
                63444, 24296, 37308, 18784, 35048, 56156, 16624, 31908, 37724,
                13252, 65160, 19320, 3408,  11456, 63376, 49408, 14232, 55120,
                24212, 57876, 21024, 10244, 45412, 16748, 65092, 49496, 20760,
                5568,  10168, 29648, 53012, 19164, 41484, 1800,  14640, 59196,
                26124, 34624, 6928,  15680, 35948, 43124, 13144, 25020, 29100,
                8748,  19016, 30564, 9984,  33548, 64168, 56588, 688,   19992,
                11480, 58032, 28100, 55520, 18288, 41720, 24524, 34580, 39868,
                9824,  22604, 42452, 54864, 8916,  49716, 56728, 11680, 46752,
                3064,  7968,  19044, 45988, 10344, 36840, 49916, 63272, 17676,
                24728, 57344, 50604, 8560,  37424, 24036, 5252,  39136, 32908,
                9876,  41532, 17408, 38344, 3976,  59228, 40728, 13088, 56164,
                62852, 15188, 41740, 10056, 39252, 18144, 27608, 6704,  48256,
                32876, 4600,  12448, 23260, 64320, 34472, 460,   44172, 57908,
                3948,  27384, 43408, 53372, 40756, 46492, 51364, 10016, 26236,
                17600, 29896,
            },
            {
                14428, 5112,  49616, 9164,  64692, 23472, 4264,  61652, 49800,
                41056, 28820, 47620, 32676, 56708, 37828, 27568, 5020,  46952,
                2464,  38856, 34340, 61980, 716,   23680, 53256, 41228, 27172,
                34448, 43012, 15500, 4708,  56356, 27056, 36136, 51932, 20988,
                45920, 348,   60812, 52832, 29676, 21800, 61444, 16804, 46856,
                55744, 60256, 39676, 53424, 43448, 7120,  13164, 26116, 47768,
                7812,  35924, 22924, 49296, 9560,  30272, 62676, 5856,  49612,
                45496, 60000, 1696,  17460, 61164, 39548, 32576, 64544, 21772,
                53400, 31180, 14028, 63844, 648,   29268, 33268, 13240, 2156,
                35916, 31140, 14604, 42392, 58992, 18356, 52148, 15192, 45344,
                64564, 23356, 51728, 7188,  34848, 29984, 9000,  24284, 35504,
                628,   30440, 47816, 22052, 3728,  49572, 64892, 55032, 23916,
                15892, 51304, 43504, 55304, 6904,  17420, 51736, 11968, 62556,
                8132,  36824, 13968, 24280, 18380, 7568,  32936, 22568, 54512,
                46908, 41676,
            },
            {
                56960, 32560, 27364, 46428, 40396, 13676, 43984, 54788, 9720,
                21096, 4896,  60368, 8380,  22904, 16936, 53844, 20756, 42300,
                63896, 30628, 13188, 47980, 55648, 37584, 7820,  11844, 46336,
                61784, 24392, 59836, 38716, 22392, 64476, 48408, 10700, 32144,
                56016, 12916, 23576, 39960, 8696,  58024, 50012, 5080,  42152,
                34724, 2460,  15004, 48632, 24648, 21204, 45888, 37988, 54588,
                31616, 65324, 40120, 1824,  60412, 15300, 56200, 2512,  13908,
                28124, 33040, 37596, 24864, 46140, 19800, 424,   26260, 16260,
                35464, 60436, 43348, 24652, 52152, 56120, 19968, 44512, 54652,
                48788, 20824, 65112, 28516, 400,   48072, 26412, 57136, 2732,
                35792, 28632, 12688, 57364, 44036, 16096, 49852, 60588, 45680,
                52328, 19548, 6076,  62108, 53004, 14408, 31372, 820,   40484,
                28636, 58548, 18964, 36628, 41656, 59192, 28972, 39964, 21848,
                30900, 50028, 61640, 4696,  59024, 29812, 56788, 2792,  62120,
                34616, 7688,
            },
            {
                38564, 21212, 59828, 2856,  54056, 20016, 29456, 35728, 24544,
                57840, 15128, 36968, 44076, 50964, 852,   60904, 33768, 11044,
                25996, 51224, 18844, 5600,  28388, 32660, 19872, 57988, 1764,
                17508, 48336, 32116, 2780,  44324, 8316,  16620, 60552, 5508,
                38500, 28524, 49336, 44492, 18792, 2180,  37636, 31812, 10380,
                22056, 26928, 62748, 35764, 3616,  59140, 29700, 61212, 17796,
                14480, 4844,  45460, 20636, 33632, 42916, 26716, 36396, 53504,
                19172, 65132, 4776,  50956, 30380, 7368,  43072, 58396, 50696,
                9660,  39768, 6312,  48348, 16404, 5028,  61960, 9188,  27652,
                38984, 4472,  45512, 10740, 33524, 62096, 40552, 12240, 21552,
                49124, 6340,  61896, 19680, 26788, 53996, 2676,  20964, 27776,
                11532, 38148, 57776, 33628, 26540, 43764, 37140, 20488, 11232,
                63380, 8596,  1720,  31192, 14004, 24444, 5496,  48560, 15860,
                54724, 44796, 19880, 34800, 48720, 12676, 38288, 43472, 19636,
                11708, 50264,
            },
            {
                224,   43200, 16984, 11484, 33672, 62660, 7676,  700,   48160,
                42324, 30748, 53040, 25704, 12908, 40268, 29368, 45348, 16016,
                59572, 8028,  41380, 60816, 44216, 14320, 63764, 39704, 30612,
                54064, 7484,  51080, 12244, 54876, 34060, 29560, 24908, 42996,
                18352, 64844, 3496,  33348, 62508, 12492, 27556, 54616, 64376,
                51268, 45340, 11956, 19496, 55248, 40060, 10464, 2244,  43076,
                50144, 27512, 59396, 12476, 52092, 7228,  46460, 21964, 59160,
                8396,  42076, 15504, 11040, 53696, 62928, 14072, 38136, 4364,
                28240, 56544, 20552, 34200, 26772, 38016, 41832, 23180, 58908,
                7224,  52256, 24260, 55836, 16648, 36532, 7404,  30960, 60480,
                55028, 42040, 31816, 46760, 10580, 63820, 38708, 42520, 7464,
                64720, 16636, 44556, 23584, 10680, 7400,  61356, 46368, 53600,
                33760, 44608, 21612, 61728, 46176, 53248, 33980, 65248, 2652,
                26012, 11008, 764,   40452, 26504, 63292, 16428, 5732,  28460,
                60608, 24920,
            },
            {
                53552, 64040, 29192, 50532, 24800, 39312, 52112, 60212, 18092,
                11420, 64000, 2496,  19088, 35272, 63184, 6876,  50200, 3468,
                37068, 57020, 23128, 2128,  52312, 25508, 48728, 4180,  22788,
                13936, 37212, 27952, 18136, 62236, 40420, 1316,  53480, 47072,
                58472, 9484,  15096, 51712, 24140, 56420, 47500, 20072, 14616,
                6732,  38420, 32208, 5828,  49828, 15612, 33036, 52452, 23276,
                35068, 9044,  54308, 24472, 38120, 18076, 64056, 11224, 40100,
                30824, 47440, 57116, 27012, 35144, 22316, 48124, 33160, 23784,
                46300, 12372, 65488, 2292,  59444, 11264, 30896, 47928, 17124,
                34444, 12824, 60864, 41336, 4032,  22068, 53660, 44132, 18620,
                3752,  15016, 39048, 88,    22780, 34428, 14160, 32120, 55592,
                48404, 29920, 1972,  54968, 40888, 58732, 17900, 3404,  24532,
                49968, 38292, 26620, 55960, 4536,  10660, 19588, 42924, 37620,
                57076, 33240, 59968, 52064, 9340,  45404, 23608, 53140, 47760,
                31780, 14952,
            },
            {
                45692, 35404, 5972,  58628, 3912,  46204, 15752, 32096, 27224,
                38440, 6380,  55340, 44968, 57964, 9872,  24016, 54628, 19756,
                27636, 47380, 35088, 16488, 38628, 9492,  33824, 55508, 44844,
                63120, 42200, 58328, 24172, 45796, 21200, 14696, 35316, 4016,
                22792, 36760, 42240, 30428, 6024,  35196, 40388, 856,   43776,
                24732, 61076, 57292, 28148, 41796, 64732, 25888, 7336,  62464,
                19412, 39080, 30644, 504,   61320, 32224, 49680, 3932,  24608,
                51896, 900,   20244, 38428, 59884, 3660,  9364,  55500, 61548,
                17304, 41976, 30092, 53784, 44308, 50728, 14916, 2720,  64352,
                43876, 19448, 31552, 49744, 25884, 63776, 47576, 10392, 27644,
                34572, 24600, 59072, 53180, 49520, 5840,  57532, 25316, 18860,
                4308,  36084, 13808, 50040, 21068, 35016, 30536, 14816, 57280,
                6012,  12272, 16708, 49012, 39376, 28304, 60148, 8372,  23352,
                47204, 14112, 29284, 21576, 6552,  55752, 35720, 1444,  40684,
                57428, 7116,
            },
            {
                18640, 22916, 41144, 13224, 36512, 20708, 9300,  44180, 56520,
                50400, 21272, 32992, 27908, 15804, 48440, 31820, 38480, 42908,
                13784, 10308, 30060, 62468, 21524, 58844, 6356,  18376, 26848,
                10616, 724,   33612, 5040,  9688,  57172, 49856, 63640, 12292,
                27716, 54400, 19728, 57660, 45772, 9812,  16740, 59956, 29232,
                52728, 9564,  17724, 46800, 44,    21076, 37172, 45708, 56040,
                4060,  58676, 48268, 43720, 15528, 6432,  27352, 57316, 13688,
                34404, 63456, 7008,  45300, 17048, 41236, 29864, 19596, 1452,
                52556, 36052, 8664,  23428, 19036, 6760,  57420, 36520, 55100,
                28924, 1056,  37680, 8044,  14188, 29980, 1940,  38336, 51500,
                65484, 45800, 8876,  29760, 17860, 60960, 44692, 10180, 51868,
                40368, 59440, 26808, 62900, 5900,  47316, 52272, 27472, 41784,
                60872, 32228, 3100,  63980, 34664, 13708, 50932, 31364, 17140,
                3884,  39532, 64140, 17944, 42380, 31028, 13168, 65000, 21324,
                37360, 10688,
            },
            {
                61324, 31216, 56000, 49112, 26252, 53416, 64912, 1896,  23636,
                13760, 46756, 9064,  41140, 4632,  22048, 60036, 1344,  56372,
                64416, 50600, 5904,  54284, 45964, 12956, 41316, 49536, 36180,
                60092, 20464, 47532, 53096, 28968, 37384, 6924,  31560, 41572,
                48096, 7620,  61456, 2556,  26196, 63724, 22320, 49644, 33020,
                36408, 4500,  23300, 54280, 13832, 60008, 11140, 29404, 16848,
                41368, 13960, 10188, 22280, 56096, 35376, 45256, 38680, 17592,
                43476, 29012, 23416, 55160, 12280, 50564, 64308, 44536, 27184,
                6064,  49544, 15408, 39304, 63220, 33492, 24992, 40988, 21608,
                9820,  52976, 45216, 59660, 54508, 42096, 17904, 57388, 13772,
                5136,  20716, 36316, 12508, 41132, 27396, 37404, 1772,  61856,
                22620, 45872, 8200,  17256, 39132, 612,   65168, 8512,  37420,
                19056, 53116, 43264, 24136, 20364, 84,    57640, 44588, 61908,
                54228, 49588, 5928,  34504, 57608, 50188, 27728, 17064, 4940,
                51576, 27000,
            },
            {
                48140, 8980,  2788,  17564, 60524, 30220, 40176, 34788, 5228,
                62632, 36232, 59204, 51524, 65424, 12324, 35664, 17692, 7808,
                24560, 33488, 41004, 300,   26368, 31496, 65276, 3236,  29820,
                51880, 14272, 39876, 64664, 16932, 43768, 25760, 20000, 60324,
                16604, 33860, 39488, 49972, 14040, 37856, 54224, 3456,  12828,
                42856, 64176, 39468, 30208, 34964, 43212, 51692, 1980,  49156,
                33724, 26152, 64796, 29476, 51848, 19572, 62744, 2620,  53572,
                59528, 10160, 48780, 32572, 1932,  25060, 34248, 14240, 38748,
                58336, 32080, 60348, 244,   27808, 46536, 12148, 3720,  49256,
                26508, 63136, 15976, 21168, 34188, 5888,  61796, 23568, 32512,
                42964, 54552, 62708, 47608, 3640,  53700, 21496, 15728, 31032,
                34144, 12072, 54184, 29500, 33080, 23892, 43820, 13276, 22236,
                47568, 28836, 7964,  58664, 46864, 40564, 25600, 6772,  35228,
                12328, 27336, 22968, 45848, 10284, 2372,  60952, 47076, 32604,
                58788, 41972,
            },
            {
                15384, 64428, 33532, 44376, 6768,  11060, 15256, 57564, 43356,
                29292, 17012, 20,    25196, 30988, 46136, 40296, 53044, 29568,
                44728, 21100, 57572, 18660, 37048, 53184, 15484, 22572, 56648,
                7444,  25344, 31728, 3052,  12808, 59284, 54604, 200,   51960,
                5140,  24196, 11732, 30112, 19032, 44456, 28088, 7680,  59184,
                20420, 15684, 50720, 3060,  58172, 8292,  24388, 63544, 20768,
                53972, 5196,  37084, 2148,  40864, 8376,  12196, 31176, 22160,
                5584,  40468, 15352, 62340, 37808, 57936, 6744,  54080, 10400,
                24368, 20212, 45032, 55564, 9260,  51420, 62032, 18068, 57928,
                32772, 5176,  40440, 10996, 27880, 46696, 36796, 50300, 8192,
                27188, 1308,  16696, 23868, 33808, 8328,  64808, 46692, 50568,
                4732,  43116, 20092, 57896, 50704, 10328, 59720, 54684, 34260,
                1536,  62848, 11636, 36684, 15888, 51920, 10208, 29736, 21144,
                42232, 1088,  59080, 38544, 15208, 25356, 36748, 43648, 12048,
                23988, 1664,
            },
            {
                28740, 20576, 54712, 37000, 22404, 47552, 51044, 25416, 19996,
                53776, 11628, 38812, 55576, 18712, 3132,  26656, 61384, 15288,
                48988, 3412,  12528, 60520, 7708,  43592, 10036, 47268, 39040,
                17716, 44384, 61044, 50236, 23320, 34336, 9344,  38540, 29252,
                45384, 56840, 65028, 53016, 1200,  32684, 61748, 40748, 25000,
                45728, 55920, 10452, 26896, 47788, 18456, 32268, 38756, 11836,
                44140, 59548, 46772, 14884, 24304, 49268, 58028, 26192, 47172,
                35976, 51400, 27068, 8172,  18648, 46900, 21712, 42304, 62088,
                48228, 3444,  12960, 41616, 22200, 37380, 30336, 43172, 14292,
                36092, 47480, 24100, 51252, 56144, 740,   12112, 19792, 60688,
                40356, 48760, 30632, 60040, 55636, 42756, 29156, 11236, 57328,
                25744, 63788, 35912, 2484,  46192, 16068, 39948, 26696, 5588,
                49936, 17556, 55136, 30524, 3740,  64620, 55444, 37892, 47848,
                63068, 17652, 52856, 32280, 62580, 20876, 54676, 8144,  18960,
                53308, 38472,
            },
            {
                45896, 50620, 12116, 40800, 27928, 62268, 2616,  32972, 8224,
                60700, 48572, 22612, 42712, 8704,  49816, 57036, 5332,  10408,
                37512, 63396, 34508, 28480, 49964, 24216, 61800, 34252, 2080,
                58388, 10924, 36296, 6104,  41708, 18088, 47468, 63424, 14620,
                35732, 21480, 41860, 8296,  47396, 56348, 11396, 17388, 49288,
                30924, 6456,  34392, 21888, 60992, 40592, 5384,  56448, 28156,
                7180,  18176, 30836, 55580, 61688, 32944, 42724, 16544, 65404,
                100,   20344, 60492, 43144, 30172, 52184, 2636,  28072, 16956,
                35844, 29936, 64828, 34376, 16048, 4656,  59952, 7748,  53284,
                1396,  19276, 58820, 7612,  31868, 64244, 25504, 44652, 15232,
                56312, 11208, 37468, 5484,  13896, 19428, 38268, 640,   17572,
                39828, 7152,  14364, 27804, 62188, 4404,  19868, 31376, 57528,
                44748, 38944, 25904, 42444, 33728, 23228, 18344, 2596,  14092,
                56484, 8456,  28596, 4648,  51216, 40416, 3388,  57368, 34688,
                62684, 6864,
            },
            {
                59772, 25088, 760,   58900, 5512,  55356, 17980, 39140, 46332,
                35740, 5612,  31904, 62336, 13632, 34324, 21916, 43644, 31664,
                25364, 54824, 46420, 16032, 39732, 5708,  19212, 54504, 26488,
                30292, 46776, 21236, 53904, 28500, 57552, 2680,  25184, 50936,
                10376, 3600,  27424, 16156, 36652, 23692, 4316,  35500, 57672,
                696,   38432, 63400, 44544, 12724, 52536, 15936, 62124, 35972,
                22988, 51280, 39792, 10748, 20852, 3324,  5964,  38204, 55352,
                13824, 33844, 56256, 4152,  12600, 64572, 33320, 8888,  56452,
                39888, 5952,  52540, 25424, 48864, 55004, 20360, 27416, 39644,
                65260, 29388, 44100, 37928, 17324, 41364, 53456, 30048, 3128,
                33680, 21928, 52084, 45452, 26796, 49796, 59064, 34424, 52772,
                22448, 48048, 54536, 41928, 23404, 36880, 52808, 64156, 13976,
                9808,  21624, 61556, 5996,  13032, 49472, 44080, 60172, 32068,
                24584, 35596, 44504, 13132, 46744, 9932,  30284, 26616, 47856,
                14236, 31484,
            },
            {
                9880,  35072, 19452, 15064, 32388, 42548, 10784, 64612, 14268,
                27340, 2344,  51684, 58260, 28092, 38252, 64208, 17044, 52352,
                40880, 8412,  22388, 1372,  56476, 32688, 51180, 13100, 42464,
                64380, 15292, 1192,  61864, 33104, 12456, 43888, 20224, 61092,
                40232, 32184, 57804, 62164, 43604, 51288, 20940, 64372, 27456,
                13488, 54040, 17020, 29444, 1676,  26004, 45996, 3608,  48888,
                14136, 64360, 1504,  26728, 36208, 46260, 52456, 28936, 10652,
                45732, 25108, 49344, 39524, 23292, 37272, 44884, 14644, 49592,
                59896, 19064, 10708, 58680, 1960,  31860, 45676, 10140, 23252,
                48580, 12040, 3820,  62072, 13716, 22244, 4588,  58688, 38456,
                62448, 7660,  17744, 64324, 2748,  23312, 7524,  62680, 44088,
                30416, 9956,  59860, 32128, 11920, 49056, 7672,  42696, 24624,
                35668, 3356,  52176, 46000, 58748, 9004,  27264, 39356, 6484,
                50208, 41148, 19500, 64456, 24000, 17092, 60336, 42928, 21668,
                4688,  41284,
            },
            {
                55932, 44220, 63992, 52172, 47244, 22784, 29612, 52616, 57648,
                24044, 45004, 16580, 20388, 6620,  47052, 960,   11864, 58700,
                4284,  19536, 60248, 42980, 29848, 63664, 3812,  38064, 21992,
                7212,  49832, 39520, 26120, 8416,  51620, 37444, 30888, 7328,
                46104, 53824, 19284, 12232, 6048,  30608, 40104, 9176,  46284,
                42048, 23448, 50416, 7896,  37432, 55780, 20296, 29932, 41352,
                9028,  33028, 43552, 54060, 58508, 17776, 63228, 23244, 42032,
                6572,  31360, 17096, 9700,  58556, 18284, 53796, 26224, 1176,
                22840, 45940, 28948, 38264, 42880, 13692, 63280, 35524, 56284,
                16476, 32968, 52224, 26676, 49444, 35996, 45980, 10564, 50748,
                24628, 43220, 28944, 57576, 32396, 41896, 12484, 27760, 15336,
                4544,  37916, 18752, 2068,  57384, 16944, 29756, 320,   46888,
                59432, 28768, 15024, 19612, 36080, 30120, 53212, 21540, 63828,
                11340, 54476, 2220,  57480, 37640, 33380, 49640, 176,   65332,
                52412, 17392,
            },
            {
                28120, 3732,  38860, 26348, 6696,  37252, 1500,  20140, 7316,
                40696, 63348, 36644, 10040, 41816, 55208, 30468, 23968, 50000,
                33280, 27656, 48520, 10512, 14400, 25464, 45304, 9708,  60116,
                32296, 56148, 19348, 45428, 16324, 65224, 4408,  55676, 17184,
                24064, 2320,  34484, 48368, 25808, 59504, 15124, 52440, 2764,
                19688, 58308, 32472, 60556, 42960, 9968,  35012, 59212, 53556,
                25116, 60512, 16336, 6688,  30316, 13292, 8608,  34456, 19700,
                57768, 63720, 52984, 1400,  47616, 29640, 4888,  63044, 42052,
                32748, 13272, 62140, 7532,  23628, 17736, 50976, 2976,  6424,
                60684, 41880, 20176, 9440,  57148, 6252,  28544, 19380, 16024,
                47712, 528,   35580, 9704,  39076, 53516, 47088, 20800, 56160,
                48980, 65228, 26292, 44356, 34744, 61836, 40908, 55180, 18440,
                50308, 32500, 65456, 40744, 55908, 4620,  17016, 664,   46844,
                15248, 30676, 26752, 5364,  52208, 7560,  14756, 39128, 11632,
                36176, 22900,
            },
            {
                60424, 8584,  49308, 12976, 57336, 61496, 43796, 49604, 34344,
                11984, 30004, 54016, 49388, 25796, 60808, 18652, 44888, 36744,
                13812, 64888, 38460, 53476, 35456, 58040, 18500, 48796, 28328,
                36612, 4580,  11460, 59780, 35040, 22696, 40832, 27316, 59136,
                49560, 13876, 63752, 38700, 804,   55240, 33816, 28848, 61172,
                35992, 10900, 4024,  48252, 21660, 65044, 17288, 13008, 544,
                46980, 20640, 37220, 49044, 24160, 39968, 50848, 60176, 47236,
                3648,  36540, 27388, 42628, 61436, 35132, 11736, 20012, 55776,
                36544, 3832,  52140, 47952, 34216, 58104, 39944, 29300, 44812,
                25472, 37428, 60,    59728, 34028, 42528, 54360, 65004, 32556,
                59892, 12884, 54528, 20208, 15904, 5268,  60064, 36124, 1320,
                32836, 53404, 6972,  21384, 51628, 9608,  27112, 22468, 12384,
                38744, 8516,  2016,  23648, 11620, 60968, 43524, 37316, 57872,
                22884, 34912, 59936, 42516, 21372, 29160, 55196, 25176, 58084,
                30512, 46044,
            },
            {
                15616, 33876, 21416, 30712, 17468, 10080, 25740, 56060, 15648,
                60432, 556,   22888, 4820,  14724, 34872, 2980,  62664, 7856,
                56776, 1712,  17948, 5932,  23204, 40948, 368,   54404, 15020,
                24432, 52768, 42132, 29116, 48684, 508,   14284, 44292, 10884,
                36308, 28552, 42824, 20784, 10092, 44708, 18240, 6984,  48900,
                25084, 39516, 15872, 27100, 5632,  30848, 50752, 39008, 6416,
                28648, 56780, 11388, 3760,  62252, 43960, 736,   26396, 15316,
                39396, 11108, 21744, 14516, 7536,  24512, 39796, 50628, 8484,
                27252, 16872, 56772, 20680, 792,   26668, 11816, 54392, 21560,
                14464, 53164, 46516, 29852, 23140, 14752, 1984,  37576, 7156,
                27320, 39956, 45252, 61476, 50240, 25476, 29732, 10516, 42604,
                23644, 12220, 39340, 46580, 14552, 4612,  37328, 63692, 3684,
                56716, 44524, 27464, 48136, 34176, 50476, 25896, 31612, 9476,
                40680, 48776, 17688, 10388, 47560, 61108, 44932, 19292, 5796,
                50584, 1524,
            },
            {
                55016, 42936, 62916, 52964, 40524, 36020, 3276,  31516, 46604,
                19052, 43104, 57268, 32392, 39764, 52892, 10860, 28272, 41520,
                21940, 44024, 31740, 46228, 60908, 8592,  33368, 62104, 43344,
                3008,  64032, 21428, 6496,  58016, 53360, 32524, 61376, 5168,
                52268, 7908,  60164, 31592, 50676, 23816, 65320, 41612, 12864,
                56216, 63056, 46052, 53980, 40916, 57372, 23952, 45364, 33256,
                63188, 40560, 52040, 31852, 19152, 54908, 33152, 5608,  56088,
                29800, 49820, 57220, 33052, 51872, 44632, 60208, 31136, 46872,
                64668, 43148, 38824, 30636, 63772, 8804,  48936, 61844, 32340,
                7848,  64604, 11452, 17780, 48752, 61096, 25788, 51900, 20980,
                56804, 4240,  23108, 31044, 1860,  63604, 38236, 18852, 57616,
                62328, 16584, 28916, 56504, 60200, 31292, 48324, 52836, 33828,
                16332, 60500, 20780, 54644, 6284,  18668, 13264, 62488, 54068,
                6796,  1608,  63464, 36704, 13084, 2452,  40512, 34288, 9616,
                62376, 37456,
            },
            {
                12248, 24916, 2256,  5868,  45784, 64780, 20832, 51632, 6328,
                27832, 37144, 9172,  65176, 45984, 21084, 59552, 48304, 16576,
                54608, 25972, 50528, 12952, 28928, 51648, 21104, 38376, 12068,
                47708, 17516, 34000, 38512, 9516,  25572, 19976, 47776, 38976,
                22444, 17876, 54980, 4480,  14664, 57348, 35680, 3688,  21804,
                31148, 256,   18892, 34512, 13856, 2084,  8872,  60800, 18468,
                14712, 23012, 8168,  45928, 27688, 12556, 38172, 22300, 62548,
                17788, 45148, 2924,  65128, 19356, 5244,  16372, 276,   13796,
                23908, 5772,  10936, 45560, 15576, 36008, 42148, 18324, 3700,
                35284, 40812, 56208, 4444,  39224, 8036,  45084, 11228, 41620,
                49652, 15204, 8972,  34692, 42124, 13124, 48132, 7668,  45000,
                3980,  50988, 34148, 784,   24660, 42760, 19720, 7356,  25032,
                30044, 41060, 10236, 36836, 63368, 41940, 3248,  45124, 20268,
                28956, 50876, 24996, 56568, 31912, 27560, 64212, 16180, 48496,
                26924, 19848,
            },
            {
                40080, 47628, 58300, 28884, 14904, 23972, 59028, 11704, 39680,
                61636, 48060, 13472, 26756, 17480, 6372,  31100, 37784, 4332,
                34544, 9792,  63472, 2876,  56932, 16236, 25864, 5548,  31056,
                59436, 26960, 51476, 13268, 46216, 62904, 15968, 2648,  29188,
                64608, 34012, 25444, 47284, 39696, 27208, 8648,  53076, 45104,
                37604, 50804, 9724,  29056, 63804, 49092, 36236, 26828, 52800,
                43364, 2708,  35496, 65244, 16212, 59792, 48172, 10196, 51912,
                40732, 7944,  24068, 36188, 28856, 55088, 38084, 58064, 21328,
                53328, 34792, 61080, 25164, 55108, 5312,  58504, 23860, 46196,
                51536, 26948, 20776, 31680, 62912, 35660, 57796, 18108, 33328,
                28744, 64064, 46424, 58784, 17800, 55504, 22520, 52468, 30952,
                25932, 41080, 9092,  64052, 17376, 11100, 39288, 58012, 13180,
                46208, 51560, 1048,  14956, 32104, 28416, 52332, 24144, 58996,
                33460, 14204, 43996, 7920,  18540, 51604, 5164,  23100, 42080,
                57200, 31420,
            },
            {
                52284, 10004, 18228, 33652, 49788, 8604,  42728, 34932, 54940,
                3712,  22432, 51020, 1832,  55684, 43640, 52020, 24488, 14324,
                57980, 39272, 19772, 42416, 36700, 47508, 65024, 44820, 54948,
                8236,  41196, 1484,  57100, 29828, 42684, 35116, 54312, 12352,
                41288, 45624, 1140,  11516, 62156, 17644, 32712, 59788, 14212,
                26480, 6716,  58932, 23344, 44052, 20656, 55368, 11448, 4796,
                58640, 49428, 25776, 55832, 1564,  42068, 6592,  35812, 25484,
                31808, 13644, 59668, 42224, 10828, 47836, 26184, 32912, 41424,
                48484, 29136, 2472,  51380, 19956, 39168, 28432, 12468, 60332,
                15660, 2224,  44296, 50048, 13720, 27600, 22312, 5908,  60772,
                3180,  36688, 25148, 50812, 5788,  28204, 3308,  35452, 14252,
                61268, 20396, 49292, 36376, 45480, 55376, 2304,  61984, 35368,
                5396,  64536, 23168, 58448, 47652, 8632,  39000, 16400, 10628,
                37484, 55168, 40032, 22084, 46472, 58928, 37748, 53868, 13652,
                7264,  2944,
            },
            {
                35420, 21792, 61996, 38880, 56012, 788,   30304, 18004, 26072,
                15180, 33660, 62728, 29552, 35768, 12032, 63916, 324,   49468,
                61660, 29712, 4868,  23460, 32800, 10664, 1136,  19608, 35148,
                14548, 22684, 63452, 18536, 4684,  23728, 8404,  49096, 57724,
                6072,  16392, 58936, 37088, 50284, 43136, 1856,  48064, 20084,
                62276, 41744, 54956, 38604, 3944,  16344, 32124, 42004, 37868,
                29036, 19388, 9508,  39164, 21736, 30376, 53740, 18936, 63940,
                1000,  54532, 49260, 17296, 2104,  61532, 6824,  12336, 62976,
                9216,  17828, 59216, 13148, 32608, 48016, 1436,  53788, 30908,
                38256, 63760, 24708, 9396,  54292, 924,   47164, 53200, 43440,
                12936, 54696, 19540, 10780, 39588, 32956, 65040, 58540, 38632,
                10952, 54388, 5896,  22756, 30168, 51296, 26812, 21856, 48868,
                17984, 37964, 27092, 44136, 20052, 55268, 61064, 264,   46340,
                64920, 5892,  2812,  60776, 34760, 11200, 848,   29816, 60600,
                45408, 64952,
            },
            {
                28344, 43208, 6388,  25612, 12120, 44848, 63616, 52056, 7420,
                59328, 44996, 19392, 41728, 8184,  22772, 39892, 18868, 27108,
                7496,  44332, 53036, 56032, 14840, 60012, 39540, 50328, 58760,
                28512, 48532, 37284, 32408, 55164, 39204, 60072, 21132, 26516,
                31788, 52732, 20500, 28504, 7220,  22716, 30156, 55236, 10440,
                34968, 17144, 30704, 12224, 47844, 61820, 7124,  23764, 64420,
                13004, 44760, 61300, 33092, 50924, 14384, 58272, 44188, 5208,
                46508, 27932, 37936, 21828, 34600, 51704, 43848, 19520, 54812,
                4668,  35772, 40048, 44288, 7500,  65368, 22452, 43096, 9988,
                6836,  57028, 17500, 33560, 59580, 40568, 30396, 16460, 23932,
                38760, 31096, 284,   62100, 43084, 15708, 46600, 19252, 1096,
                44032, 28624, 15664, 59544, 3940,  13588, 33260, 43704, 9636,
                29504, 56232, 3472,  11944, 34656, 5064,  25800, 31276, 49976,
                18296, 27072, 30568, 15540, 49400, 25772, 43032, 20288, 33912,
                24232, 16504,
            },
            {
                50488, 3520,  59516, 53436, 16292, 36184, 21436, 40608, 48568,
                10676, 38552, 2800,  56956, 47696, 60296, 31848, 56404, 46608,
                33772, 12252, 17752, 48872, 26688, 7740,  30268, 23856, 4052,
                10980, 44280, 6848,  50820, 11880, 45532, 896,   14772, 37388,
                43544, 9744,  64092, 34328, 56688, 12648, 63492, 39052, 25160,
                6352,  46644, 1204,  53204, 27136, 35484, 57024, 51132, 708,
                31332, 54524, 16992, 3452,  46728, 10880, 27596, 23380, 34292,
                15980, 58728, 8340,  63580, 30684, 13764, 23840, 38572, 27852,
                45816, 50888, 23192, 26844, 56468, 16896, 36552, 50168, 19480,
                34712, 47900, 41968, 5376,  20400, 37196, 10396, 65272, 50196,
                8464,  45788, 57404, 22668, 52128, 26608, 8196,  50068, 24004,
                57144, 33576, 48112, 37508, 41668, 64836, 7792,  53872, 59928,
                14740, 42556, 52888, 61904, 16900, 50816, 41376, 13220, 36112,
                23032, 52572, 41644, 62784, 8440,  57084, 17136, 51212, 39556,
                8752,  56100,
            },
            {
                36800, 14224, 32276, 47728, 29572, 3156,  58200, 5192,  27964,
                31964, 23904, 52776, 13248, 26140, 16444, 4064,  9696,  41512,
                22112, 59264, 36900, 2200,  61884, 43924, 52116, 17472, 42360,
                54176, 61236, 21192, 26376, 16860, 65472, 28964, 51616, 62272,
                3492,  24104, 47956, 4412,  41028, 52280, 16140, 4768,  43908,
                51552, 65076, 22172, 59560, 19268, 9952,  45388, 15524, 21312,
                48708, 40508, 5808,  24380, 64144, 37348, 7588,  61240, 40172,
                50772, 11684, 43004, 4972,  55412, 47112, 60728, 3420,  31676,
                64240, 15044, 876,   61596, 10492, 30184, 4000,  55292, 62612,
                27156, 12660, 29664, 61912, 49324, 14744, 56172, 2364,  34276,
                61196, 27196, 14600, 35512, 6224,  56436, 12444, 30532, 41408,
                4932,  63648, 8764,  25380, 56352, 17196, 20684, 38980, 1572,
                24844, 32404, 7408,  39648, 30080, 57792, 21256, 62496, 7160,
                58248, 44912, 12300, 20624, 32596, 45916, 6052,  63520, 12520,
                1908,  46488,
            },
            {
                21280, 10372, 40072, 19244, 62596, 24640, 46272, 14248, 55804,
                64992, 17116, 60888, 6560,  34452, 51124, 37908, 54256, 14888,
                65468, 5628,  29288, 40280, 20688, 34040, 12796, 64444, 36332,
                32108, 15412, 2836,  58492, 35760, 41564, 8928,  33388, 18584,
                55468, 38960, 14280, 19160, 26652, 46496, 35696, 20628, 57160,
                27752, 33048, 14656, 37028, 42884, 2808,  28616, 33764, 60804,
                8128,  26956, 57644, 34864, 52972, 41924, 18444, 55348, 2628,
                21092, 32824, 25012, 17824, 27292, 40404, 8636,  16688, 57108,
                11156, 20804, 34156, 53172, 41536, 46572, 14044, 24872, 40236,
                392,   52816, 22488, 3256,  43832, 25144, 28540, 41724, 17588,
                21160, 4460,  40736, 48836, 32088, 20560, 62604, 36068, 53296,
                17548, 12860, 21472, 45276, 536,   31012, 50508, 46916, 35932,
                63216, 19352, 49068, 23064, 9832,  44432, 2140,  47600, 33224,
                10032, 1300,  38384, 53960, 3864,  36444, 27448, 54692, 31520,
                60140, 26536,
            },
            {
                57004, 63936, 51320, 7724,  42092, 11076, 33436, 38020, 20540,
                1212,  35968, 41396, 29152, 45716, 63152, 21184, 27976, 1196,
                48284, 24396, 52476, 10500, 47036, 5320,  57392, 1660,  25980,
                8904,  39996, 47256, 31040, 53336, 4924,  24552, 48724, 12316,
                45740, 29128, 57940, 61380, 704,   9612,  31024, 60380, 2632,
                11868, 40760, 8160,  49780, 25052, 63872, 40016, 55148, 12024,
                36984, 43936, 20156, 13336, 29248, 164,   48948, 13780, 28788,
                64656, 57188, 45932, 53160, 540,   58960, 21292, 36508, 52568,
                42144, 49356, 7056,  38008, 18424, 60108, 33144, 6200,  59012,
                15912, 45472, 56696, 35800, 8084,  51640, 59768, 6460,  47880,
                53668, 58524, 11352, 64508, 2892,  39316, 44648, 1724,  60584,
                27540, 40188, 51464, 61052, 35044, 11256, 27124, 4944,  57852,
                12564, 44900, 2668,  36368, 65296, 15036, 28428, 37960, 55688,
                17188, 64392, 28536, 23980, 59088, 14580, 41268, 22728, 48656,
                18508, 42704,
            },
            {
                4076,  29228, 23736, 512,   55232, 60752, 52200, 7324,  48944,
                43624, 9668,  50332, 19832, 3296,  11368, 43120, 59908, 36016,
                30928, 43672, 57744, 15988, 63316, 23256, 38808, 49076, 19664,
                55768, 62640, 23020, 10848, 44576, 19556, 56516, 61152, 1716,
                35488, 7032,  51120, 32812, 43088, 54572, 23552, 48972, 37716,
                18900, 45756, 57736, 4476,  52344, 17628, 5748,  47664, 23948,
                1668,  62408, 49532, 9168,  58660, 22232, 32368, 43572, 37052,
                9372,  3780,  39284, 35540, 13080, 49180, 33044, 45100, 2528,
                26288, 29900, 58240, 23884, 3152,  27520, 49456, 44124, 20928,
                36856, 31300, 10788, 63900, 18148, 32868, 12800, 36888, 23684,
                31380, 44028, 28584, 18704, 52700, 25432, 16248, 9936,  22744,
                49176, 32164, 3116,  15276, 54808, 43056, 62564, 22864, 16200,
                29052, 55728, 51488, 25992, 56888, 5480,  53664, 19704, 25288,
                42252, 31684, 50324, 47260, 18168, 62012, 328,   9960,  38284,
                5676,  34228,
            },
            {
                16836, 47020, 38608, 35140, 14872, 27480, 17928, 30620, 59592,
                26256, 54708, 15184, 32260, 57932, 25068, 7564,  17384, 51448,
                12096, 19248, 7208,  32860, 27904, 55056, 14332, 29752, 43396,
                5948,  51100, 28436, 12,    63956, 15052, 38044, 27940, 21360,
                41844, 16784, 10888, 21884, 38076, 63196, 13660, 6800,  53088,
                62636, 31448, 23040, 29032, 61308, 13820, 31864, 58988, 18644,
                51760, 30832, 16244, 38452, 45536, 5272,  60488, 53636, 24224,
                47680, 15380, 20028, 61604, 29608, 9980,  64944, 24820, 6268,
                62356, 13376, 47232, 9760,  63428, 53804, 12200, 8140,  64724,
                51160, 5424,  25492, 40008, 54368, 21520, 45972, 61956, 10104,
                780,   15592, 37296, 8056,  46924, 59092, 29784, 55440, 46076,
                6920,  57696, 38396, 28920, 7596,  19200, 40480, 53120, 33904,
                6240,  39264, 10940, 17416, 32672, 41152, 48668, 11780, 3108,
                59764, 13904, 4268,  8180,  34628, 43584, 51764, 29940, 65392,
                13396, 54164,
            },
        },
};
// clang-format on

}  // namespace pbrt
